# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.9.1
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x10s\xe6\
/\
*!\x0a  Highlight.j\
s v11.10.0 (git:\
 366a8bd012)\x0a  (\
c) 2006-2024 Jos\
h Goebel <hello@\
joshgoebel.com> \
and other contri\
butors\x0a  License\
: BSD-3-Clause\x0a \
*/\x0avar hljs=func\
tion(){\x22use stri\
ct\x22;function e(t\
){\x0areturn t inst\
anceof Map?t.cle\
ar=t.delete=t.se\
t=()=>{\x0athrow Er\
ror(\x22map is read\
-only\x22)}:t insta\
nceof Set&&(t.ad\
d=t.clear=t.dele\
te=()=>{\x0athrow E\
rror(\x22set is rea\
d-only\x22)\x0a}),Obje\
ct.freeze(t),Obj\
ect.getOwnProper\
tyNames(t).forEa\
ch((n=>{\x0aconst i\
=t[n],s=typeof i\
;\x22object\x22!==s&&\x22\
function\x22!==s||O\
bject.isFrozen(i\
)||e(i)\x0a})),t}cl\
ass t{constructo\
r(e){\x0avoid 0===e\
.data&&(e.data={\
}),this.data=e.d\
ata,this.isMatch\
Ignored=!1}\x0aigno\
reMatch(){this.i\
sMatchIgnored=!0\
}}function n(e){\
\x0areturn e.replac\
e(/&/g,\x22&amp;\x22).\
replace(/</g,\x22&l\
t;\x22).replace(/>/\
g,\x22&gt;\x22).replac\
e(/\x22/g,\x22&quot;\x22)\
.replace(/'/g,\x22&\
#x27;\x22)\x0a}functio\
n i(e,...t){cons\
t n=Object.creat\
e(null);for(cons\
t t in e)n[t]=e[\
t]\x0a;return t.for\
Each((e=>{for(co\
nst t in e)n[t]=\
e[t]})),n}const \
s=e=>!!e.scope\x0a;\
class o{construc\
tor(e,t){\x0athis.b\
uffer=\x22\x22,this.cl\
assPrefix=t.clas\
sPrefix,e.walk(t\
his)}addText(e){\
\x0athis.buffer+=n(\
e)}openNode(e){i\
f(!s(e))return;c\
onst t=((e,{pref\
ix:t})=>{\x0aif(e.s\
tartsWith(\x22langu\
age:\x22))return e.\
replace(\x22languag\
e:\x22,\x22language-\x22)\
\x0a;if(e.includes(\
\x22.\x22)){const n=e.\
split(\x22.\x22)\x0a;retu\
rn[`${t}${n.shif\
t()}`,...n.map((\
(e,t)=>`${e}${\x22_\
\x22.repeat(t+1)}`)\
)].join(\x22 \x22)\x0a}re\
turn`${t}${e}`})\
(e.scope,{prefix\
:this.classPrefi\
x});this.span(t)\
}\x0acloseNode(e){s\
(e)&&(this.buffe\
r+=\x22</span>\x22)}va\
lue(){return thi\
s.buffer}span(e)\
{\x0athis.buffer+=`\
<span class=\x22${e\
}\x22>`}}const r=(e\
={})=>{const t={\
children:[]}\x0a;re\
turn Object.assi\
gn(t,e),t};class\
 a{constructor()\
{\x0athis.rootNode=\
r(),this.stack=[\
this.rootNode]}g\
et top(){\x0areturn\
 this.stack[this\
.stack.length-1]\
}get root(){retu\
rn this.rootNode\
}add(e){\x0athis.to\
p.children.push(\
e)}openNode(e){c\
onst t=r({scope:\
e})\x0a;this.add(t)\
,this.stack.push\
(t)}closeNode(){\
\x0aif(this.stack.l\
ength>1)return t\
his.stack.pop()}\
closeAllNodes(){\
\x0afor(;this.close\
Node(););}toJSON\
(){return JSON.s\
tringify(this.ro\
otNode,null,4)}\x0a\
walk(e){return t\
his.constructor.\
_walk(e,this.roo\
tNode)}static _w\
alk(e,t){\x0areturn\
\x22string\x22==typeof\
 t?e.addText(t):\
t.children&&(e.o\
penNode(t),\x0at.ch\
ildren.forEach((\
t=>this._walk(e,\
t))),e.closeNode\
(t)),e}static _c\
ollapse(e){\x0a\x22str\
ing\x22!=typeof e&&\
e.children&&(e.c\
hildren.every((e\
=>\x22string\x22==type\
of e))?e.childre\
n=[e.children.jo\
in(\x22\x22)]:e.childr\
en.forEach((e=>{\
\x0aa._collapse(e)}\
)))}}class c ext\
ends a{construct\
or(e){super(),th\
is.options=e}\x0aad\
dText(e){\x22\x22!==e&\
&this.add(e)}sta\
rtScope(e){this.\
openNode(e)}endS\
cope(){\x0athis.clo\
seNode()}__addSu\
blanguage(e,t){c\
onst n=e.root\x0a;t\
&&(n.scope=\x22lang\
uage:\x22+t),this.a\
dd(n)}toHTML(){\x0a\
return new o(thi\
s,this.options).\
value()}finalize\
(){\x0areturn this.\
closeAllNodes(),\
!0}}function l(e\
){\x0areturn e?\x22str\
ing\x22==typeof e?e\
:e.source:null}f\
unction g(e){ret\
urn h(\x22(?=\x22,e,\x22)\
\x22)}\x0afunction u(e\
){return h(\x22(?:\x22\
,e,\x22)*\x22)}functio\
n d(e){return h(\
\x22(?:\x22,e,\x22)?\x22)}\x0af\
unction h(...e){\
return e.map((e=\
>l(e))).join(\x22\x22)\
}function f(...e\
){const t=(e=>{\x0a\
const t=e[e.leng\
th-1]\x0a;return\x22ob\
ject\x22==typeof t&\
&t.constructor==\
=Object?(e.splic\
e(e.length-1,1),\
t):{}\x0a})(e);retu\
rn\x22(\x22+(t.capture\
?\x22\x22:\x22?:\x22)+e.map(\
(e=>l(e))).join(\
\x22|\x22)+\x22)\x22}\x0afuncti\
on p(e){return R\
egExp(e.toString\
()+\x22|\x22).exec(\x22\x22)\
.length-1}\x0aconst\
 b=/\x5c[(?:[^\x5c\x5c\x5c]]\
|\x5c\x5c.)*\x5c]|\x5c(\x5c??|\x5c\
\x5c([1-9][0-9]*)|\x5c\
\x5c./\x0a;function m(\
e,{joinWith:t}){\
let n=0;return e\
.map((e=>{n+=1;c\
onst t=n\x0a;let i=\
l(e),s=\x22\x22;for(;i\
.length>0;){cons\
t e=b.exec(i);if\
(!e){s+=i;break}\
\x0as+=i.substring(\
0,e.index),\x0ai=i.\
substring(e.inde\
x+e[0].length),\x22\
\x5c\x5c\x22===e[0][0]&&e\
[1]?s+=\x22\x5c\x5c\x22+(Num\
ber(e[1])+t):(s+\
=e[0],\x0a\x22(\x22===e[0\
]&&n++)}return s\
})).map((e=>`(${\
e})`)).join(t)}\x0a\
const E=\x22[a-zA-Z\
]\x5c\x5cw*\x22,x=\x22[a-zA-\
Z_]\x5c\x5cw*\x22,w=\x22\x5c\x5cb\x5c\
\x5cd+(\x5c\x5c.\x5c\x5cd+)?\x22,y\
=\x22(-?)(\x5c\x5cb0[xX][\
a-fA-F0-9]+|(\x5c\x5cb\
\x5c\x5cd+(\x5c\x5c.\x5c\x5cd*)?|\x5c\
\x5c.\x5c\x5cd+)([eE][-+]\
?\x5c\x5cd+)?)\x22,_=\x22\x5c\x5cb\
(0b[01]+)\x22,O={\x0ab\
egin:\x22\x5c\x5c\x5c\x5c[\x5c\x5cs\x5c\x5c\
S]\x22,relevance:0}\
,v={scope:\x22strin\
g\x22,begin:\x22'\x22,end\
:\x22'\x22,\x0aillegal:\x22\x5c\
\x5cn\x22,contains:[O]\
},k={scope:\x22stri\
ng\x22,begin:'\x22',en\
d:'\x22',illegal:\x22\x5c\
\x5cn\x22,\x0acontains:[O\
]},N=(e,t,n={})=\
>{const s=i({sco\
pe:\x22comment\x22,beg\
in:e,end:t,\x0acont\
ains:[]},n);s.co\
ntains.push({sco\
pe:\x22doctag\x22,\x0abeg\
in:\x22[ ]*(?=(TODO\
|FIXME|NOTE|BUG|\
OPTIMIZE|HACK|XX\
X):)\x22,\x0aend:/(TOD\
O|FIXME|NOTE|BUG\
|OPTIMIZE|HACK|X\
XX):/,excludeBeg\
in:!0,relevance:\
0})\x0a;const o=f(\x22\
I\x22,\x22a\x22,\x22is\x22,\x22so\x22\
,\x22us\x22,\x22to\x22,\x22at\x22,\
\x22if\x22,\x22in\x22,\x22it\x22,\x22\
on\x22,/[A-Za-z]+['\
](d|ve|re|ll|t|s\
|n)/,/[A-Za-z]+[\
-][a-z]+/,/[A-Za\
-z][a-z]{2,}/)\x0a;\
return s.contain\
s.push({begin:h(\
/[ ]+/,\x22(\x22,o,/[.\
]?[:]?([.][ ]|[ \
])/,\x22){3}\x22)}),s\x0a\
},S=N(\x22//\x22,\x22$\x22),\
M=N(\x22/\x5c\x5c*\x22,\x22\x5c\x5c*/\
\x22),R=N(\x22#\x22,\x22$\x22);\
var j=Object.fre\
eze({\x0a__proto__:\
null,APOS_STRING\
_MODE:v,BACKSLAS\
H_ESCAPE:O,BINAR\
Y_NUMBER_MODE:{\x0a\
scope:\x22number\x22,b\
egin:_,relevance\
:0},BINARY_NUMBE\
R_RE:_,COMMENT:N\
,\x0aC_BLOCK_COMMEN\
T_MODE:M,C_LINE_\
COMMENT_MODE:S,C\
_NUMBER_MODE:{sc\
ope:\x22number\x22,\x0abe\
gin:y,relevance:\
0},C_NUMBER_RE:y\
,END_SAME_AS_BEG\
IN:e=>Object.ass\
ign(e,{\x0a\x22on:begi\
n\x22:(e,t)=>{t.dat\
a._beginMatch=e[\
1]},\x22on:end\x22:(e,\
t)=>{\x0at.data._be\
ginMatch!==e[1]&\
&t.ignoreMatch()\
}}),HASH_COMMENT\
_MODE:R,IDENT_RE\
:E,\x0aMATCH_NOTHIN\
G_RE:/\x5cb\x5cB/,METH\
OD_GUARD:{begin:\
\x22\x5c\x5c.\x5c\x5cs*\x22+x,rele\
vance:0},\x0aNUMBER\
_MODE:{scope:\x22nu\
mber\x22,begin:w,re\
levance:0},NUMBE\
R_RE:w,\x0aPHRASAL_\
WORDS_MODE:{\x0abeg\
in:/\x5cb(a|an|the|\
are|I'm|isn't|do\
n't|doesn't|won'\
t|but|just|shoul\
d|pretty|simply|\
enough|gonna|goi\
ng|wtf|so|such|w\
ill|you|your|the\
y|like|more)\x5cb/\x0a\
},QUOTE_STRING_M\
ODE:k,REGEXP_MOD\
E:{scope:\x22regexp\
\x22,begin:/\x5c/(?=[^\
/\x5cn]*\x5c/)/,\x0aend:/\
\x5c/[gimuy]*/,cont\
ains:[O,{begin:/\
\x5c[/,end:/\x5c]/,rel\
evance:0,contain\
s:[O]}]},\x0aRE_STA\
RTERS_RE:\x22!|!=|!\
==|%|%=|&|&&|&=|\
\x5c\x5c*|\x5c\x5c*=|\x5c\x5c+|\x5c\x5c+\
=|,|-|-=|/=|/|:|\
;|<<|<<=|<=|<|==\
=|==|=|>>>=|>>=|\
>=|>>>|>>|>|\x5c\x5c?|\
\x5c\x5c[|\x5c\x5c{|\x5c\x5c(|\x5c\x5c^|\
\x5c\x5c^=|\x5c\x5c||\x5c\x5c|=|\x5c\x5c\
|\x5c\x5c||~\x22,\x0aSHEBANG\
:(e={})=>{const \
t=/^#![ ]*\x5c//\x0a;r\
eturn e.binary&&\
(e.begin=h(t,/.*\
\x5cb/,e.binary,/\x5cb\
.*/)),i({scope:\x22\
meta\x22,begin:t,\x0ae\
nd:/$/,relevance\
:0,\x22on:begin\x22:(e\
,t)=>{0!==e.inde\
x&&t.ignoreMatch\
()}},e)},\x0aTITLE_\
MODE:{scope:\x22tit\
le\x22,begin:E,rele\
vance:0},UNDERSC\
ORE_IDENT_RE:x,\x0a\
UNDERSCORE_TITLE\
_MODE:{scope:\x22ti\
tle\x22,begin:x,rel\
evance:0}});func\
tion A(e,t){\x0a\x22.\x22\
===e.input[e.ind\
ex-1]&&t.ignoreM\
atch()}function \
I(e,t){\x0avoid 0!=\
=e.className&&(e\
.scope=e.classNa\
me,delete e.clas\
sName)}function \
T(e,t){\x0at&&e.beg\
inKeywords&&(e.b\
egin=\x22\x5c\x5cb(\x22+e.be\
ginKeywords.spli\
t(\x22 \x22).join(\x22|\x22)\
+\x22)(?!\x5c\x5c.)(?=\x5c\x5cb\
|\x5c\x5cs)\x22,\x0ae.__befo\
reBegin=A,e.keyw\
ords=e.keywords|\
|e.beginKeywords\
,delete e.beginK\
eywords,\x0avoid 0=\
==e.relevance&&(\
e.relevance=0))}\
function L(e,t){\
\x0aArray.isArray(e\
.illegal)&&(e.il\
legal=f(...e.ill\
egal))}function \
B(e,t){\x0aif(e.mat\
ch){\x0aif(e.begin|\
|e.end)throw Err\
or(\x22begin & end \
are not supporte\
d with match\x22)\x0a;\
e.begin=e.match,\
delete e.match}}\
function P(e,t){\
\x0avoid 0===e.rele\
vance&&(e.releva\
nce=1)}const D=(\
e,t)=>{if(!e.bef\
oreMatch)return\x0a\
;if(e.starts)thr\
ow Error(\x22before\
Match cannot be \
used with starts\
\x22)\x0a;const n=Obje\
ct.assign({},e);\
Object.keys(e).f\
orEach((t=>{dele\
te e[t]\x0a})),e.ke\
ywords=n.keyword\
s,e.begin=h(n.be\
foreMatch,g(n.be\
gin)),e.starts={\
\x0arelevance:0,con\
tains:[Object.as\
sign(n,{endsPare\
nt:!0})]\x0a},e.rel\
evance=0,delete \
n.beforeMatch\x0a},\
H=[\x22of\x22,\x22and\x22,\x22f\
or\x22,\x22in\x22,\x22not\x22,\x22\
or\x22,\x22if\x22,\x22then\x22,\
\x22parent\x22,\x22list\x22,\
\x22value\x22],C=\x22keyw\
ord\x22\x0a;function $\
(e,t,n=C){const \
i=Object.create(\
null)\x0a;return\x22st\
ring\x22==typeof e?\
s(n,e.split(\x22 \x22)\
):Array.isArray(\
e)?s(n,e):Object\
.keys(e).forEach\
((n=>{\x0aObject.as\
sign(i,$(e[n],t,\
n))})),i;functio\
n s(e,n){\x0at&&(n=\
n.map((e=>e.toLo\
werCase()))),n.f\
orEach((t=>{cons\
t n=t.split(\x22|\x22)\
\x0a;i[n[0]]=[e,U(n\
[0],n[1])]}))}}f\
unction U(e,t){\x0a\
return t?Number(\
t):(e=>H.include\
s(e.toLowerCase(\
)))(e)?0:1}const\
 z={},W=e=>{\x0acon\
sole.error(e)},X\
=(e,...t)=>{cons\
ole.log(\x22WARN: \x22\
+e,...t)},G=(e,t\
)=>{\x0az[`${e}/${t\
}`]||(console.lo\
g(`Deprecated as\
 of ${e}. ${t}`)\
,z[`${e}/${t}`]=\
!0)\x0a},K=Error();\
function F(e,t,{\
key:n}){let i=0;\
const s=e[n],o={\
},r={}\x0a;for(let \
e=1;e<=t.length;\
e++)r[e+i]=s[e],\
o[e+i]=!0,i+=p(t\
[e-1])\x0a;e[n]=r,e\
[n]._emit=o,e[n]\
._multi=!0}funct\
ion Z(e){(e=>{\x0ae\
.scope&&\x22object\x22\
==typeof e.scope\
&&null!==e.scope\
&&(e.beginScope=\
e.scope,\x0adelete \
e.scope)})(e),\x22s\
tring\x22==typeof e\
.beginScope&&(e.\
beginScope={\x0a_wr\
ap:e.beginScope}\
),\x22string\x22==type\
of e.endScope&&(\
e.endScope={_wra\
p:e.endScope\x0a}),\
(e=>{if(Array.is\
Array(e.begin)){\
\x0aif(e.skip||e.ex\
cludeBegin||e.re\
turnBegin)throw \
W(\x22skip, exclude\
Begin, returnBeg\
in not compatibl\
e with beginScop\
e: {}\x22),\x0aK\x0a;if(\x22\
object\x22!=typeof \
e.beginScope||nu\
ll===e.beginScop\
e)throw W(\x22begin\
Scope must be ob\
ject\x22),\x0aK;F(e,e.\
begin,{key:\x22begi\
nScope\x22}),e.begi\
n=m(e.begin,{joi\
nWith:\x22\x22})}})(e)\
,(e=>{\x0aif(Array.\
isArray(e.end)){\
\x0aif(e.skip||e.ex\
cludeEnd||e.retu\
rnEnd)throw W(\x22s\
kip, excludeEnd,\
 returnEnd not c\
ompatible with e\
ndScope: {}\x22),\x0aK\
\x0a;if(\x22object\x22!=t\
ypeof e.endScope\
||null===e.endSc\
ope)throw W(\x22end\
Scope must be ob\
ject\x22),\x0aK;F(e,e.\
end,{key:\x22endSco\
pe\x22}),e.end=m(e.\
end,{joinWith:\x22\x22\
})}})(e)}functio\
n V(e){\x0afunction\
 t(t,n){\x0areturn \
RegExp(l(t),\x22m\x22+\
(e.case_insensit\
ive?\x22i\x22:\x22\x22)+(e.u\
nicodeRegex?\x22u\x22:\
\x22\x22)+(n?\x22g\x22:\x22\x22))\x0a\
}class n{constru\
ctor(){\x0athis.mat\
chIndexes={},thi\
s.regexes=[],thi\
s.matchAt=1,this\
.position=0}\x0aadd\
Rule(e,t){\x0at.pos\
ition=this.posit\
ion++,this.match\
Indexes[this.mat\
chAt]=t,this.reg\
exes.push([t,e])\
,\x0athis.matchAt+=\
p(e)+1}compile()\
{0===this.regexe\
s.length&&(this.\
exec=()=>null)\x0a;\
const e=this.reg\
exes.map((e=>e[1\
]));this.matcher\
Re=t(m(e,{joinWi\
th:\x22|\x22\x0a}),!0),th\
is.lastIndex=0}e\
xec(e){this.matc\
herRe.lastIndex=\
this.lastIndex\x0a;\
const t=this.mat\
cherRe.exec(e);i\
f(!t)return null\
\x0a;const n=t.find\
Index(((e,t)=>t>\
0&&void 0!==e)),\
i=this.matchInde\
xes[n]\x0a;return t\
.splice(0,n),Obj\
ect.assign(t,i)}\
}class s{constru\
ctor(){\x0athis.rul\
es=[],this.multi\
Regexes=[],\x0athis\
.count=0,this.la\
stIndex=0,this.r\
egexIndex=0}getM\
atcher(e){\x0aif(th\
is.multiRegexes[\
e])return this.m\
ultiRegexes[e];c\
onst t=new n\x0a;re\
turn this.rules.\
slice(e).forEach\
((([e,n])=>t.add\
Rule(e,n))),\x0at.c\
ompile(),this.mu\
ltiRegexes[e]=t,\
t}resumingScanAt\
SamePosition(){\x0a\
return 0!==this.\
regexIndex}consi\
derAll(){this.re\
gexIndex=0}addRu\
le(e,t){\x0athis.ru\
les.push([e,t]),\
\x22begin\x22===t.type\
&&this.count++}e\
xec(e){\x0aconst t=\
this.getMatcher(\
this.regexIndex)\
;t.lastIndex=thi\
s.lastIndex\x0a;let\
 n=t.exec(e)\x0a;if\
(this.resumingSc\
anAtSamePosition\
())if(n&&n.index\
===this.lastInde\
x);else{\x0aconst t\
=this.getMatcher\
(0);t.lastIndex=\
this.lastIndex+1\
,n=t.exec(e)}\x0are\
turn n&&(this.re\
gexIndex+=n.posi\
tion+1,\x0athis.reg\
exIndex===this.c\
ount&&this.consi\
derAll()),n}}\x0aif\
(e.compilerExten\
sions||(e.compil\
erExtensions=[])\
,\x0ae.contains&&e.\
contains.include\
s(\x22self\x22))throw \
Error(\x22ERR: cont\
ains `self` is n\
ot supported at \
the top-level of\
 a language.  Se\
e documentation.\
\x22)\x0a;return e.cla\
ssNameAliases=i(\
e.classNameAlias\
es||{}),function\
 n(o,r){const a=\
o\x0a;if(o.isCompil\
ed)return a\x0a;[I,\
B,Z,D].forEach((\
e=>e(o,r))),e.co\
mpilerExtensions\
.forEach((e=>e(o\
,r))),\x0ao.__befor\
eBegin=null,[T,L\
,P].forEach((e=>\
e(o,r))),o.isCom\
piled=!0;let c=n\
ull\x0a;return\x22obje\
ct\x22==typeof o.ke\
ywords&&o.keywor\
ds.$pattern&&(o.\
keywords=Object.\
assign({},o.keyw\
ords),\x0ac=o.keywo\
rds.$pattern,\x0ade\
lete o.keywords.\
$pattern),c=c||/\
\x5cw+/,o.keywords&\
&(o.keywords=$(o\
.keywords,e.case\
_insensitive)),\x0a\
a.keywordPattern\
Re=t(c,!0),\x0ar&&(\
o.begin||(o.begi\
n=/\x5cB|\x5cb/),a.beg\
inRe=t(a.begin),\
o.end||o.endsWit\
hParent||(o.end=\
/\x5cB|\x5cb/),\x0ao.end&\
&(a.endRe=t(a.en\
d)),\x0aa.terminato\
rEnd=l(a.end)||\x22\
\x22,o.endsWithPare\
nt&&r.terminator\
End&&(a.terminat\
orEnd+=(o.end?\x22|\
\x22:\x22\x22)+r.terminat\
orEnd)),\x0ao.illeg\
al&&(a.illegalRe\
=t(o.illegal)),\x0a\
o.contains||(o.c\
ontains=[]),o.co\
ntains=[].concat\
(...o.contains.m\
ap((e=>(e=>(e.va\
riants&&!e.cache\
dVariants&&(e.ca\
chedVariants=e.v\
ariants.map((t=>\
i(e,{\x0avariants:n\
ull},t)))),e.cac\
hedVariants?e.ca\
chedVariants:q(e\
)?i(e,{\x0astarts:e\
.starts?i(e.star\
ts):null\x0a}):Obje\
ct.isFrozen(e)?i\
(e):e))(\x22self\x22==\
=e?o:e)))),o.con\
tains.forEach((e\
=>{n(e,a)\x0a})),o.\
starts&&n(o.star\
ts,r),a.matcher=\
(e=>{const t=new\
 s\x0a;return e.con\
tains.forEach((e\
=>t.addRule(e.be\
gin,{rule:e,type\
:\x22begin\x22\x0a}))),e.\
terminatorEnd&&t\
.addRule(e.termi\
natorEnd,{type:\x22\
end\x22\x0a}),e.illega\
l&&t.addRule(e.i\
llegal,{type:\x22il\
legal\x22}),t})(a),\
a}(e)}function q\
(e){\x0areturn!!e&&\
(e.endsWithParen\
t||q(e.starts))}\
class J extends \
Error{\x0aconstruct\
or(e,t){super(e)\
,this.name=\x22HTML\
InjectionError\x22,\
this.html=t}}\x0aco\
nst Y=n,Q=i,ee=S\
ymbol(\x22nomatch\x22)\
,te=n=>{\x0aconst i\
=Object.create(n\
ull),s=Object.cr\
eate(null),o=[];\
let r=!0\x0a;const \
a=\x22Could not fin\
d the language '\
{}', did you for\
get to load/incl\
ude a language m\
odule?\x22,l={\x0adisa\
bleAutodetect:!0\
,name:\x22Plain tex\
t\x22,contains:[]};\
let p={\x0aignoreUn\
escapedHTML:!1,t\
hrowUnescapedHTM\
L:!1,noHighlight\
Re:/^(no-?highli\
ght)$/i,\x0alanguag\
eDetectRe:/\x5cblan\
g(?:uage)?-([\x5cw-\
]+)\x5cb/i,classPre\
fix:\x22hljs-\x22,\x0acss\
Selector:\x22pre co\
de\x22,languages:nu\
ll,__emitter:c};\
function b(e){\x0ar\
eturn p.noHighli\
ghtRe.test(e)}fu\
nction m(e,t,n){\
let i=\x22\x22,s=\x22\x22\x0a;\x22\
object\x22==typeof \
t?(i=e,\x0an=t.igno\
reIllegals,s=t.l\
anguage):(G(\x2210.\
7.0\x22,\x22highlight(\
lang, code, ...a\
rgs) has been de\
precated.\x22),\x0aG(\x22\
10.7.0\x22,\x22Please \
use highlight(co\
de, options) ins\
tead.\x5cnhttps://g\
ithub.com/highli\
ghtjs/highlight.\
js/issues/2277\x22)\
,\x0as=e,i=t),void \
0===n&&(n=!0);co\
nst o={code:i,la\
nguage:s};N(\x22bef\
ore:highlight\x22,o\
)\x0a;const r=o.res\
ult?o.result:E(o\
.language,o.code\
,n)\x0a;return r.co\
de=o.code,N(\x22aft\
er:highlight\x22,r)\
,r}function E(e,\
n,s,o){\x0aconst c=\
Object.create(nu\
ll);function l()\
{if(!N.keywords)\
return void M.ad\
dText(R)\x0a;let e=\
0;N.keywordPatte\
rnRe.lastIndex=0\
;let t=N.keyword\
PatternRe.exec(R\
),n=\x22\x22\x0a;for(;t;)\
{n+=R.substring(\
e,t.index)\x0a;cons\
t s=_.case_insen\
sitive?t[0].toLo\
werCase():t[0],o\
=(i=s,N.keywords\
[i]);if(o){\x0acons\
t[e,i]=o\x0a;if(M.a\
ddText(n),n=\x22\x22,c\
[s]=(c[s]||0)+1,\
c[s]<=7&&(j+=i),\
e.startsWith(\x22_\x22\
))n+=t[0];else{\x0a\
const n=_.classN\
ameAliases[e]||e\
;u(t[0],n)}}else\
 n+=t[0]\x0a;e=N.ke\
ywordPatternRe.l\
astIndex,t=N.key\
wordPatternRe.ex\
ec(R)}var i\x0a;n+=\
R.substring(e),M\
.addText(n)}func\
tion g(){null!=N\
.subLanguage?(()\
=>{\x0aif(\x22\x22===R)re\
turn;let e=null;\
if(\x22string\x22==typ\
eof N.subLanguag\
e){\x0aif(!i[N.subL\
anguage])return \
void M.addText(R\
)\x0a;e=E(N.subLang\
uage,R,!0,S[N.su\
bLanguage]),S[N.\
subLanguage]=e._\
top\x0a}else e=x(R,\
N.subLanguage.le\
ngth?N.subLangua\
ge:null)\x0a;N.rele\
vance>0&&(j+=e.r\
elevance),M.__ad\
dSublanguage(e._\
emitter,e.langua\
ge)\x0a})():l(),R=\x22\
\x22}function u(e,t\
){\x0a\x22\x22!==e&&(M.st\
artScope(t),M.ad\
dText(e),M.endSc\
ope())}function \
d(e,t){let n=1\x0a;\
const i=t.length\
-1;for(;n<=i;){i\
f(!e._emit[n]){n\
++;continue}\x0acon\
st i=_.className\
Aliases[e[n]]||e\
[n],s=t[n];i?u(s\
,i):(R=s,l(),R=\x22\
\x22),n++}}\x0afunctio\
n h(e,t){\x0areturn\
 e.scope&&\x22strin\
g\x22==typeof e.sco\
pe&&M.openNode(_\
.classNameAliase\
s[e.scope]||e.sc\
ope),\x0ae.beginSco\
pe&&(e.beginScop\
e._wrap?(u(R,_.c\
lassNameAliases[\
e.beginScope._wr\
ap]||e.beginScop\
e._wrap),\x0aR=\x22\x22):\
e.beginScope._mu\
lti&&(d(e.beginS\
cope,t),R=\x22\x22)),N\
=Object.create(e\
,{parent:{\x0avalue\
:N}}),N}function\
 f(e,n,i){let s=\
((e,t)=>{const n\
=e&&e.exec(t)\x0a;r\
eturn n&&0===n.i\
ndex})(e.endRe,i\
);if(s){if(e[\x22on\
:end\x22]){const i=\
new t(e)\x0a;e[\x22on:\
end\x22](n,i),i.isM\
atchIgnored&&(s=\
!1)}if(s){\x0afor(;\
e.endsParent&&e.\
parent;)e=e.pare\
nt;return e}}\x0aif\
(e.endsWithParen\
t)return f(e.par\
ent,n,i)}functio\
n b(e){\x0areturn 0\
===N.matcher.reg\
exIndex?(R+=e[0]\
,1):(T=!0,0)}fun\
ction m(e){\x0acons\
t t=e[0],i=n.sub\
string(e.index),\
s=f(N,e,i);if(!s\
)return ee;const\
 o=N\x0a;N.endScope\
&&N.endScope._wr\
ap?(g(),\x0au(t,N.e\
ndScope._wrap)):\
N.endScope&&N.en\
dScope._multi?(g\
(),\x0ad(N.endScope\
,e)):o.skip?R+=t\
:(o.returnEnd||o\
.excludeEnd||(R+\
=t),\x0ag(),o.exclu\
deEnd&&(R=t));do\
{\x0aN.scope&&M.clo\
seNode(),N.skip|\
|N.subLanguage||\
(j+=N.relevance)\
,N=N.parent\x0a}whi\
le(N!==s.parent)\
;return s.starts\
&&h(s.starts,e),\
o.returnEnd?0:t.\
length}\x0alet w={}\
;function y(i,o)\
{const a=o&&o[0]\
;if(R+=i,null==a\
)return g(),0\x0a;i\
f(\x22begin\x22===w.ty\
pe&&\x22end\x22===o.ty\
pe&&w.index===o.\
index&&\x22\x22===a){\x0a\
if(R+=n.slice(o.\
index,o.index+1)\
,!r){const t=Err\
or(`0 width matc\
h regex (${e})`)\
\x0a;throw t.langua\
geName=e,t.badRu\
le=w.rule,t}retu\
rn 1}\x0aif(w=o,\x22be\
gin\x22===o.type)re\
turn(e=>{\x0aconst \
n=e[0],i=e.rule,\
s=new t(i),o=[i.\
__beforeBegin,i[\
\x22on:begin\x22]]\x0a;fo\
r(const t of o)i\
f(t&&(t(e,s),s.i\
sMatchIgnored))r\
eturn b(n)\x0a;retu\
rn i.skip?R+=n:(\
i.excludeBegin&&\
(R+=n),\x0ag(),i.re\
turnBegin||i.exc\
ludeBegin||(R=n)\
),h(i,e),i.retur\
nBegin?0:n.lengt\
h})(o)\x0a;if(\x22ille\
gal\x22===o.type&&!\
s){\x0aconst e=Erro\
r('Illegal lexem\
e \x22'+a+'\x22 for mo\
de \x22'+(N.scope||\
\x22<unnamed>\x22)+'\x22'\
)\x0a;throw e.mode=\
N,e}if(\x22end\x22===o\
.type){const e=m\
(o);if(e!==ee)re\
turn e}\x0aif(\x22ille\
gal\x22===o.type&&\x22\
\x22===a)return 1\x0a;\
if(I>1e5&&I>3*o.\
index)throw Erro\
r(\x22potential inf\
inite loop, way \
more iterations \
than matches\x22)\x0a;\
return R+=a,a.le\
ngth}const _=O(e\
)\x0a;if(!_)throw W\
(a.replace(\x22{}\x22,\
e)),Error('Unkno\
wn language: \x22'+\
e+'\x22')\x0a;const v=\
V(_);let k=\x22\x22,N=\
o||v;const S={},\
M=new p.__emitte\
r(p);(()=>{const\
 e=[]\x0a;for(let t\
=N;t!==_;t=t.par\
ent)t.scope&&e.u\
nshift(t.scope)\x0a\
;e.forEach((e=>M\
.openNode(e)))})\
();let R=\x22\x22,j=0,\
A=0,I=0,T=!1;try\
{\x0aif(_.__emitTok\
ens)_.__emitToke\
ns(n,M);else{for\
(N.matcher.consi\
derAll();;){\x0aI++\
,T?T=!1:N.matche\
r.considerAll(),\
N.matcher.lastIn\
dex=A\x0a;const e=N\
.matcher.exec(n)\
;if(!e)break;con\
st t=y(n.substri\
ng(A,e.index),e)\
\x0a;A=e.index+t}y(\
n.substring(A))}\
return M.finaliz\
e(),k=M.toHTML()\
,{language:e,\x0ava\
lue:k,relevance:\
j,illegal:!1,_em\
itter:M,_top:N}}\
catch(t){\x0aif(t.m\
essage&&t.messag\
e.includes(\x22Ille\
gal\x22))return{lan\
guage:e,value:Y(\
n),\x0aillegal:!0,r\
elevance:0,_ille\
galBy:{message:t\
.message,index:A\
,\x0acontext:n.slic\
e(A-100,A+100),m\
ode:t.mode,resul\
tSoFar:k},_emitt\
er:M};if(r)retur\
n{\x0alanguage:e,va\
lue:Y(n),illegal\
:!1,relevance:0,\
errorRaised:t,_e\
mitter:M,_top:N}\
\x0a;throw t}}funct\
ion x(e,t){t=t||\
p.languages||Obj\
ect.keys(i);cons\
t n=(e=>{\x0aconst \
t={value:Y(e),il\
legal:!1,relevan\
ce:0,_top:l,_emi\
tter:new p.__emi\
tter(p)}\x0a;return\
 t._emitter.addT\
ext(e),t})(e),s=\
t.filter(O).filt\
er(k).map((t=>E(\
t,e,!1)))\x0a;s.uns\
hift(n);const o=\
s.sort(((e,t)=>{\
\x0aif(e.relevance!\
==t.relevance)re\
turn t.relevance\
-e.relevance\x0a;if\
(e.language&&t.l\
anguage){if(O(e.\
language).supers\
etOf===t.languag\
e)return 1\x0a;if(O\
(t.language).sup\
ersetOf===e.lang\
uage)return-1}re\
turn 0})),[r,a]=\
o,c=r\x0a;return c.\
secondBest=a,c}f\
unction w(e){let\
 t=null;const n=\
(e=>{\x0alet t=e.cl\
assName+\x22 \x22;t+=e\
.parentNode?e.pa\
rentNode.classNa\
me:\x22\x22\x0a;const n=p\
.languageDetectR\
e.exec(t);if(n){\
const t=O(n[1])\x0a\
;return t||(X(a.\
replace(\x22{}\x22,n[1\
])),\x0aX(\x22Falling \
back to no-highl\
ight mode for th\
is block.\x22,e)),t\
?n[1]:\x22no-highli\
ght\x22}\x0areturn t.s\
plit(/\x5cs+/).find\
((e=>b(e)||O(e))\
)})(e);if(b(n))r\
eturn\x0a;if(N(\x22bef\
ore:highlightEle\
ment\x22,{el:e,lang\
uage:n\x0a}),e.data\
set.highlighted)\
return void cons\
ole.log(\x22Element\
 previously high\
lighted. To high\
light again, fir\
st unset `datase\
t.highlighted`.\x22\
,e)\x0a;if(e.childr\
en.length>0&&(p.\
ignoreUnescapedH\
TML||(console.wa\
rn(\x22One of your \
code blocks incl\
udes unescaped H\
TML. This is a p\
otentially serio\
us security risk\
.\x22),\x0aconsole.war\
n(\x22https://githu\
b.com/highlightj\
s/highlight.js/w\
iki/security\x22),\x0a\
console.warn(\x22Th\
e element with u\
nescaped HTML:\x22)\
,\x0aconsole.warn(e\
)),p.throwUnesca\
pedHTML))throw n\
ew J(\x22One of you\
r code blocks in\
cludes unescaped\
 HTML.\x22,e.innerH\
TML)\x0a;t=e;const \
i=t.textContent,\
o=n?m(i,{languag\
e:n,ignoreIllega\
ls:!0}):x(i)\x0a;e.\
innerHTML=o.valu\
e,e.dataset.high\
lighted=\x22yes\x22,((\
e,t,n)=>{const i\
=t&&s[t]||n\x0a;e.c\
lassList.add(\x22hl\
js\x22),e.classList\
.add(\x22language-\x22\
+i)\x0a})(e,n,o.lan\
guage),e.result=\
{language:o.lang\
uage,re:o.releva\
nce,\x0arelevance:o\
.relevance},o.se\
condBest&&(e.sec\
ondBest={\x0alangua\
ge:o.secondBest.\
language,relevan\
ce:o.secondBest.\
relevance\x0a}),N(\x22\
after:highlightE\
lement\x22,{el:e,re\
sult:o,text:i})}\
let y=!1;functio\
n _(){\x0a\x22loading\x22\
!==document.read\
yState?document.\
querySelectorAll\
(p.cssSelector).\
forEach(w):y=!0\x0a\
}function O(e){r\
eturn e=(e||\x22\x22).\
toLowerCase(),i[\
e]||i[s[e]]}\x0afun\
ction v(e,{langu\
ageName:t}){\x22str\
ing\x22==typeof e&&\
(e=[e]),e.forEac\
h((e=>{\x0as[e.toLo\
werCase()]=t}))}\
function k(e){co\
nst t=O(e)\x0a;retu\
rn t&&!t.disable\
Autodetect}funct\
ion N(e,t){const\
 n=e;o.forEach((\
e=>{\x0ae[n]&&e[n](\
t)}))}\x0a\x22undefine\
d\x22!=typeof windo\
w&&window.addEve\
ntListener&&wind\
ow.addEventListe\
ner(\x22DOMContentL\
oaded\x22,(()=>{\x0ay&\
&_()}),!1),Objec\
t.assign(n,{high\
light:m,highligh\
tAuto:x,highligh\
tAll:_,\x0ahighligh\
tElement:w,\x0ahigh\
lightBlock:e=>(G\
(\x2210.7.0\x22,\x22highl\
ightBlock will b\
e removed entire\
ly in v12.0\x22),\x0aG\
(\x2210.7.0\x22,\x22Pleas\
e use highlightE\
lement now.\x22),w(\
e)),configure:e=\
>{p=Q(p,e)},\x0aini\
tHighlighting:()\
=>{\x0a_(),G(\x2210.6.\
0\x22,\x22initHighligh\
ting() deprecate\
d.  Use highligh\
tAll() now.\x22)},\x0a\
initHighlighting\
OnLoad:()=>{\x0a_()\
,G(\x2210.6.0\x22,\x22ini\
tHighlightingOnL\
oad() deprecated\
.  Use highlight\
All() now.\x22)\x0a},r\
egisterLanguage:\
(e,t)=>{let s=nu\
ll;try{s=t(n)}ca\
tch(t){\x0aif(W(\x22La\
nguage definitio\
n for '{}' could\
 not be register\
ed.\x22.replace(\x22{}\
\x22,e)),\x0a!r)throw \
t;W(t),s=l}\x0as.na\
me||(s.name=e),i\
[e]=s,s.rawDefin\
ition=t.bind(nul\
l,n),s.aliases&&\
v(s.aliases,{\x0ala\
nguageName:e})},\
unregisterLangua\
ge:e=>{delete i[\
e]\x0a;for(const t \
of Object.keys(s\
))s[t]===e&&dele\
te s[t]},\x0alistLa\
nguages:()=>Obje\
ct.keys(i),getLa\
nguage:O,registe\
rAliases:v,\x0aauto\
Detection:k,inhe\
rit:Q,addPlugin:\
e=>{(e=>{\x0ae[\x22bef\
ore:highlightBlo\
ck\x22]&&!e[\x22before\
:highlightElemen\
t\x22]&&(e[\x22before:\
highlightElement\
\x22]=t=>{\x0ae[\x22befor\
e:highlightBlock\
\x22](Object.assign\
({block:t.el},t)\
)\x0a}),e[\x22after:hi\
ghlightBlock\x22]&&\
!e[\x22after:highli\
ghtElement\x22]&&(e\
[\x22after:highligh\
tElement\x22]=t=>{\x0a\
e[\x22after:highlig\
htBlock\x22](Object\
.assign({block:t\
.el},t))})})(e),\
o.push(e)},\x0aremo\
vePlugin:e=>{con\
st t=o.indexOf(e\
);-1!==t&&o.spli\
ce(t,1)}}),n.deb\
ugMode=()=>{\x0ar=!\
1},n.safeMode=()\
=>{r=!0},n.versi\
onString=\x2211.10.\
0\x22,n.regex={conc\
at:h,\x0alookahead:\
g,either:f,optio\
nal:d,anyNumberO\
fTimes:u}\x0a;for(c\
onst t in j)\x22obj\
ect\x22==typeof j[t\
]&&e(j[t]);retur\
n Object.assign(\
n,j),n\x0a},ne=te({\
});return ne.new\
Instance=()=>te(\
{}),ne}()\x0a;\x22obje\
ct\x22==typeof expo\
rts&&\x22undefined\x22\
!=typeof module&\
&(module.exports\
=hljs);/*! `1c` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var s=(()=>{\x22\
use strict\x22;retu\
rn s=>{\x0aconst x=\
\x22[A-Za-z\x5cu0410-\x5c\
u042f\x5cu0430-\x5cu04\
4f\x5cu0451\x5cu0401_]\
[A-Za-z\x5cu0410-\x5cu\
042f\x5cu0430-\x5cu044\
f\x5cu0451\x5cu0401_0-\
9]+\x22,n=\x22\x5cu0434\x5cu\
0430\x5cu043b\x5cu0435\
\x5cu0435 \x5cu0432\x5cu0\
43e\x5cu0437\x5cu0432\x5c\
u0440\x5cu0430\x5cu044\
2 \x5cu0432\x5cu044b\x5cu\
0437\x5cu0432\x5cu0430\
\x5cu0442\x5cu044c\x5cu04\
38\x5cu0441\x5cu043a\x5cu\
043b\x5cu044e\x5cu0447\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435 \x5cu0432\x5c\
u044b\x5cu043f\x5cu043\
e\x5cu043b\x5cu043d\x5cu0\
438\x5cu0442\x5cu044c \
\x5cu0434\x5cu043b\x5cu04\
4f \x5cu0435\x5cu0441\x5c\
u043b\x5cu0438 \x5cu04\
38 \x5cu0438\x5cu0437 \
\x5cu0438\x5cu043b\x5cu04\
38 \x5cu0438\x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
5 \x5cu0438\x5cu043d\x5cu\
0430\x5cu0447\x5cu0435\
\x5cu0435\x5cu0441\x5cu04\
3b\x5cu0438 \x5cu0438\x5c\
u0441\x5cu043a\x5cu043\
b\x5cu044e\x5cu0447\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435 \x5cu043a\x5cu04\
30\x5cu0436\x5cu0434\x5cu\
043e\x5cu0433\x5cu043e\
 \x5cu043a\x5cu043e\x5cu0\
43d\x5cu0435\x5cu0446\x5c\
u0435\x5cu0441\x5cu043\
b\x5cu0438 \x5cu043a\x5cu\
043e\x5cu043d\x5cu0435\
\x5cu0446\x5cu043f\x5cu04\
3e\x5cu043f\x5cu044b\x5cu\
0442\x5cu043a\x5cu0438\
 \x5cu043a\x5cu043e\x5cu0\
43d\x5cu0435\x5cu0446\x5c\
u0446\x5cu0438\x5cu043\
a\x5cu043b\x5cu0430 \x5cu\
043d\x5cu0435 \x5cu043\
d\x5cu043e\x5cu0432\x5cu0\
44b\x5cu0439 \x5cu043f\
\x5cu0435\x5cu0440\x5cu04\
35\x5cu0439\x5cu0442\x5cu\
0438 \x5cu043f\x5cu043\
5\x5cu0440\x5cu0435\x5cu0\
43c \x5cu043f\x5cu043e\
 \x5cu043f\x5cu043e\x5cu0\
43a\x5cu0430 \x5cu043f\
\x5cu043e\x5cu043f\x5cu04\
4b\x5cu0442\x5cu043a\x5cu\
0430 \x5cu043f\x5cu044\
0\x5cu0435\x5cu0440\x5cu0\
432\x5cu0430\x5cu0442\x5c\
u044c \x5cu043f\x5cu04\
40\x5cu043e\x5cu0434\x5cu\
043e\x5cu043b\x5cu0436\
\x5cu0438\x5cu0442\x5cu04\
4c \x5cu0442\x5cu043e\x5c\
u0433\x5cu0434\x5cu043\
0 \x5cu0446\x5cu0438\x5cu\
043a\x5cu043b \x5cu044\
d\x5cu043a\x5cu0441\x5cu0\
43f\x5cu043e\x5cu0440\x5c\
u0442 \x22,e=\x22null \
\x5cu0438\x5cu0441\x5cu04\
42\x5cu0438\x5cu043d\x5cu\
0430 \x5cu043b\x5cu043\
e\x5cu0436\x5cu044c \x5cu\
043d\x5cu0435\x5cu043e\
\x5cu043f\x5cu0440\x5cu04\
35\x5cu0434\x5cu0435\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu043e\x22,o=s.inhe\
rit(s.NUMBER_MOD\
E),t={\x0aclassName\
:\x22string\x22,begin:\
'\x22|\x5c\x5c|',end:'\x22|$\
',contains:[{beg\
in:'\x22\x22'}]},a={\x0ab\
egin:\x22'\x22,end:\x22'\x22\
,excludeBegin:!0\
,excludeEnd:!0,c\
ontains:[{classN\
ame:\x22number\x22,\x0abe\
gin:\x22\x5c\x5cd{4}([\x5c\x5c.\
\x5c\x5c\x5c\x5c/:-]?\x5c\x5cd{2})\
{0,5}\x22}]},m=s.in\
herit(s.C_LINE_C\
OMMENT_MODE)\x0a;re\
turn{name:\x221C:En\
terprise\x22,case_i\
nsensitive:!0,ke\
ywords:{$pattern\
:x,keyword:n,\x0abu\
ilt_in:\x22\x5cu0440\x5cu\
0430\x5cu0437\x5cu0434\
\x5cu0435\x5cu043b\x5cu04\
38\x5cu0442\x5cu0435\x5cu\
043b\x5cu044c\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
0446 \x5cu0440\x5cu043\
0\x5cu0437\x5cu0434\x5cu0\
435\x5cu043b\x5cu0438\x5c\
u0442\x5cu0435\x5cu043\
b\x5cu044c\x5cu0441\x5cu0\
442\x5cu0440\x5cu043e\x5c\
u043a \x5cu0441\x5cu04\
38\x5cu043c\x5cu0432\x5cu\
043e\x5cu043b\x5cu0442\
\x5cu0430\x5cu0431\x5cu04\
43\x5cu043b\x5cu044f\x5cu\
0446\x5cu0438\x5cu0438\
 ansitooem oemto\
ansi \x5cu0432\x5cu043\
2\x5cu0435\x5cu0441\x5cu0\
442\x5cu0438\x5cu0432\x5c\
u0438\x5cu0434\x5cu044\
1\x5cu0443\x5cu0431\x5cu0\
43a\x5cu043e\x5cu043d\x5c\
u0442\x5cu043e \x5cu04\
32\x5cu0432\x5cu0435\x5cu\
0441\x5cu0442\x5cu0438\
\x5cu043f\x5cu0435\x5cu04\
40\x5cu0435\x5cu0447\x5cu\
0438\x5cu0441\x5cu043b\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435 \x5cu0432\x5c\
u0432\x5cu0435\x5cu044\
1\x5cu0442\x5cu0438\x5cu0\
43f\x5cu0435\x5cu0440\x5c\
u0438\x5cu043e\x5cu043\
4 \x5cu0432\x5cu0432\x5cu\
0435\x5cu0441\x5cu0442\
\x5cu0438\x5cu043f\x5cu04\
3b\x5cu0430\x5cu043d\x5cu\
0441\x5cu0447\x5cu0435\
\x5cu0442\x5cu043e\x5cu04\
32 \x5cu0432\x5cu044b\x5c\
u0431\x5cu0440\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0439\x5cu043f\x5c\
u043b\x5cu0430\x5cu043\
d\x5cu0441\x5cu0447\x5cu0\
435\x5cu0442\x5cu043e\x5c\
u0432 \x5cu0434\x5cu04\
30\x5cu0442\x5cu0430\x5cu\
0433\x5cu043e\x5cu0434\
 \x5cu0434\x5cu0430\x5cu0\
442\x5cu0430\x5cu043c\x5c\
u0435\x5cu0441\x5cu044\
f\x5cu0446 \x5cu0434\x5cu\
0430\x5cu0442\x5cu0430\
\x5cu0447\x5cu0438\x5cu04\
41\x5cu043b\x5cu043e \x5c\
u0437\x5cu0430\x5cu043\
3\x5cu043e\x5cu043b\x5cu0\
43e\x5cu0432\x5cu043e\x5c\
u043a\x5cu0441\x5cu043\
8\x5cu0441\x5cu0442\x5cu0\
435\x5cu043c\x5cu044b \
\x5cu0437\x5cu043d\x5cu04\
30\x5cu0447\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0432\x5cu0441\x5cu04\
42\x5cu0440\x5cu043e\x5cu\
043a\x5cu0443 \x5cu043\
7\x5cu043d\x5cu0430\x5cu0\
447\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu043\
8\x5cu0437\x5cu0441\x5cu0\
442\x5cu0440\x5cu043e\x5c\
u043a\x5cu0438 \x5cu04\
3a\x5cu0430\x5cu0442\x5cu\
0430\x5cu043b\x5cu043e\
\x5cu0433\x5cu0438\x5cu04\
31 \x5cu043a\x5cu0430\x5c\
u0442\x5cu0430\x5cu043\
b\x5cu043e\x5cu0433\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u044c\x5cu0437\x5cu043\
e\x5cu0432\x5cu0430\x5cu0\
442\x5cu0435\x5cu043b\x5c\
u044f \x5cu043a\x5cu04\
3e\x5cu0434\x5cu0441\x5cu\
0438\x5cu043c\x5cu0432\
 \x5cu043a\x5cu043e\x5cu0\
43d\x5cu0433\x5cu043e\x5c\
u0434\x5cu0430 \x5cu04\
3a\x5cu043e\x5cu043d\x5cu\
0435\x5cu0446\x5cu043f\
\x5cu0435\x5cu0440\x5cu04\
38\x5cu043e\x5cu0434\x5cu\
0430\x5cu0431\x5cu0438\
 \x5cu043a\x5cu043e\x5cu0\
43d\x5cu0435\x5cu0446\x5c\
u0440\x5cu0430\x5cu044\
1\x5cu0441\x5cu0447\x5cu0\
438\x5cu0442\x5cu0430\x5c\
u043d\x5cu043d\x5cu043\
e\x5cu0433\x5cu043e\x5cu0\
43f\x5cu0435\x5cu0440\x5c\
u0438\x5cu043e\x5cu043\
4\x5cu0430\x5cu0431\x5cu0\
438 \x5cu043a\x5cu043e\
\x5cu043d\x5cu0435\x5cu04\
46\x5cu0441\x5cu0442\x5cu\
0430\x5cu043d\x5cu0434\
\x5cu0430\x5cu0440\x5cu04\
42\x5cu043d\x5cu043e\x5cu\
0433\x5cu043e\x5cu0438\
\x5cu043d\x5cu0442\x5cu04\
35\x5cu0440\x5cu0432\x5cu\
0430\x5cu043b\x5cu0430\
 \x5cu043a\x5cu043e\x5cu0\
43d\x5cu043a\x5cu0432\x5c\
u0430\x5cu0440\x5cu044\
2\x5cu0430\x5cu043b\x5cu0\
430 \x5cu043a\x5cu043e\
\x5cu043d\x5cu043c\x5cu04\
35\x5cu0441\x5cu044f\x5cu\
0446\x5cu0430 \x5cu043\
a\x5cu043e\x5cu043d\x5cu0\
43d\x5cu0435\x5cu0434\x5c\
u0435\x5cu043b\x5cu043\
8 \x5cu043b\x5cu043e\x5cu\
0433 \x5cu043b\x5cu043\
e\x5cu043310 \x5cu043c\
\x5cu0430\x5cu043a\x5cu04\
41\x5cu0438\x5cu043c\x5cu\
0430\x5cu043b\x5cu044c\
\x5cu043d\x5cu043e\x5cu04\
35\x5cu043a\x5cu043e\x5cu\
043b\x5cu0438\x5cu0447\
\x5cu0435\x5cu0441\x5cu04\
42\x5cu0432\x5cu043e\x5cu\
0441\x5cu0443\x5cu0431\
\x5cu043a\x5cu043e\x5cu04\
3d\x5cu0442\x5cu043e \x5c\
u043d\x5cu0430\x5cu043\
7\x5cu0432\x5cu0430\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0438\x5cu043d\x5cu044\
2\x5cu0435\x5cu0440\x5cu0\
444\x5cu0435\x5cu0439\x5c\
u0441\x5cu0430 \x5cu04\
3d\x5cu0430\x5cu0437\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
3d\x5cu0430\x5cu0431\x5cu\
043e\x5cu0440\x5cu0430\
\x5cu043f\x5cu0440\x5cu04\
30\x5cu0432 \x5cu043d\x5c\
u0430\x5cu0437\x5cu043\
d\x5cu0430\x5cu0447\x5cu0\
438\x5cu0442\x5cu044c\x5c\
u0432\x5cu0438\x5cu043\
4 \x5cu043d\x5cu0430\x5cu\
0437\x5cu043d\x5cu0430\
\x5cu0447\x5cu0438\x5cu04\
42\x5cu044c\x5cu0441\x5cu\
0447\x5cu0435\x5cu0442\
 \x5cu043d\x5cu0430\x5cu0\
439\x5cu0442\x5cu0438\x5c\
u0441\x5cu0441\x5cu044\
b\x5cu043b\x5cu043a\x5cu0\
438 \x5cu043d\x5cu0430\
\x5cu0447\x5cu0430\x5cu04\
3b\x5cu043e\x5cu043f\x5cu\
0435\x5cu0440\x5cu0438\
\x5cu043e\x5cu0434\x5cu04\
30\x5cu0431\x5cu0438 \x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0430\x5cu043b\x5cu0\
43e\x5cu0441\x5cu0442\x5c\
u0430\x5cu043d\x5cu043\
4\x5cu0430\x5cu0440\x5cu0\
442\x5cu043d\x5cu043e\x5c\
u0433\x5cu043e\x5cu043\
8\x5cu043d\x5cu0442\x5cu0\
435\x5cu0440\x5cu0432\x5c\
u0430\x5cu043b\x5cu043\
0 \x5cu043d\x5cu0430\x5cu\
0447\x5cu0433\x5cu043e\
\x5cu0434\x5cu0430 \x5cu0\
43d\x5cu0430\x5cu0447\x5c\
u043a\x5cu0432\x5cu043\
0\x5cu0440\x5cu0442\x5cu0\
430\x5cu043b\x5cu0430 \
\x5cu043d\x5cu0430\x5cu04\
47\x5cu043c\x5cu0435\x5cu\
0441\x5cu044f\x5cu0446\
\x5cu0430 \x5cu043d\x5cu0\
430\x5cu0447\x5cu043d\x5c\
u0435\x5cu0434\x5cu043\
5\x5cu043b\x5cu0438 \x5cu\
043d\x5cu043e\x5cu043c\
\x5cu0435\x5cu0440\x5cu04\
34\x5cu043d\x5cu044f\x5cu\
0433\x5cu043e\x5cu0434\
\x5cu0430 \x5cu043d\x5cu0\
43e\x5cu043c\x5cu0435\x5c\
u0440\x5cu0434\x5cu043\
d\x5cu044f\x5cu043d\x5cu0\
435\x5cu0434\x5cu0435\x5c\
u043b\x5cu0438 \x5cu04\
3d\x5cu043e\x5cu043c\x5cu\
0435\x5cu0440\x5cu043d\
\x5cu0435\x5cu0434\x5cu04\
35\x5cu043b\x5cu0438\x5cu\
0433\x5cu043e\x5cu0434\
\x5cu0430 \x5cu043e\x5cu0\
431\x5cu0440\x5cu0430\x5c\
u0431\x5cu043e\x5cu044\
2\x5cu043a\x5cu0430\x5cu0\
43e\x5cu0436\x5cu0438\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu0438\x5cu044f \x5cu\
043e\x5cu0441\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
3d\x5cu043e\x5cu0439\x5cu\
0436\x5cu0443\x5cu0440\
\x5cu043d\x5cu0430\x5cu04\
3b\x5cu0440\x5cu0430\x5cu\
0441\x5cu0447\x5cu0435\
\x5cu0442\x5cu043e\x5cu04\
32 \x5cu043e\x5cu0441\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu043d\x5cu043e\x5cu0\
439\x5cu043f\x5cu043b\x5c\
u0430\x5cu043d\x5cu044\
1\x5cu0447\x5cu0435\x5cu0\
442\x5cu043e\x5cu0432 \
\x5cu043e\x5cu0441\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
043d\x5cu043e\x5cu0439\
\x5cu044f\x5cu0437\x5cu04\
4b\x5cu043a \x5cu043e\x5c\
u0447\x5cu0438\x5cu044\
1\x5cu0442\x5cu0438\x5cu0\
442\x5cu044c\x5cu043e\x5c\
u043a\x5cu043d\x5cu043\
e\x5cu0441\x5cu043e\x5cu0\
43e\x5cu0431\x5cu0449\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0439 \x5cu043f\x5cu\
0435\x5cu0440\x5cu0438\
\x5cu043e\x5cu0434\x5cu04\
41\x5cu0442\x5cu0440 \x5c\
u043f\x5cu043e\x5cu043\
b\x5cu0443\x5cu0447\x5cu0\
438\x5cu0442\x5cu044c\x5c\
u0432\x5cu0440\x5cu043\
5\x5cu043c\x5cu044f\x5cu0\
442\x5cu0430 \x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
43\x5cu0447\x5cu0438\x5cu\
0442\x5cu044c\x5cu0434\
\x5cu0430\x5cu0442\x5cu04\
43\x5cu0442\x5cu0430 \x5c\
u043f\x5cu043e\x5cu043\
b\x5cu0443\x5cu0447\x5cu0\
438\x5cu0442\x5cu044c\x5c\
u0434\x5cu043e\x5cu043\
a\x5cu0443\x5cu043c\x5cu0\
435\x5cu043d\x5cu0442\x5c\
u0442\x5cu0430 \x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
0443\x5cu0447\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
37\x5cu043d\x5cu0430\x5cu\
0447\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
3e\x5cu0442\x5cu0431\x5cu\
043e\x5cu0440\x5cu0430\
 \x5cu043f\x5cu043e\x5cu0\
43b\x5cu0443\x5cu0447\x5c\
u0438\x5cu0442\x5cu044\
c\x5cu043f\x5cu043e\x5cu0\
437\x5cu0438\x5cu0446\x5c\
u0438\x5cu044e\x5cu044\
2\x5cu0430 \x5cu043f\x5cu\
043e\x5cu043b\x5cu0443\
\x5cu0447\x5cu0438\x5cu04\
42\x5cu044c\x5cu043f\x5cu\
0443\x5cu0441\x5cu0442\
\x5cu043e\x5cu0435\x5cu04\
37\x5cu043d\x5cu0430\x5cu\
0447\x5cu0435\x5cu043d\
\x5cu0438\x5cu0435 \x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u0443\x5cu0447\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
442\x5cu0430 \x5cu043f\
\x5cu0440\x5cu0435\x5cu04\
44\x5cu0438\x5cu043a\x5cu\
0441\x5cu0430\x5cu0432\
\x5cu0442\x5cu043e\x5cu04\
3d\x5cu0443\x5cu043c\x5cu\
0435\x5cu0440\x5cu0430\
\x5cu0446\x5cu0438\x5cu04\
38 \x5cu043f\x5cu0440\x5c\
u043e\x5cu043f\x5cu043\
8\x5cu0441\x5cu044c \x5cu\
043f\x5cu0443\x5cu0441\
\x5cu0442\x5cu043e\x5cu04\
35\x5cu0437\x5cu043d\x5cu\
0430\x5cu0447\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35 \x5cu0440\x5cu0430\x5c\
u0437\x5cu043c \x5cu04\
40\x5cu0430\x5cu0437\x5cu\
043e\x5cu0431\x5cu0440\
\x5cu0430\x5cu0442\x5cu04\
4c\x5cu043f\x5cu043e\x5cu\
0437\x5cu0438\x5cu0446\
\x5cu0438\x5cu044e\x5cu04\
34\x5cu043e\x5cu043a\x5cu\
0443\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30 \x5cu0440\x5cu0430\x5c\
u0441\x5cu0441\x5cu044\
7\x5cu0438\x5cu0442\x5cu0\
430\x5cu0442\x5cu044c\x5c\
u0440\x5cu0435\x5cu043\
3\x5cu0438\x5cu0441\x5cu0\
442\x5cu0440\x5cu044b\x5c\
u043d\x5cu0430 \x5cu04\
40\x5cu0430\x5cu0441\x5cu\
0441\x5cu0447\x5cu0438\
\x5cu0442\x5cu0430\x5cu04\
42\x5cu044c\x5cu0440\x5cu\
0435\x5cu0433\x5cu0438\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu044b\x5cu043f\x5cu\
043e \x5cu0441\x5cu043\
8\x5cu043c\x5cu0432 \x5cu\
0441\x5cu043e\x5cu0437\
\x5cu0434\x5cu0430\x5cu04\
42\x5cu044c\x5cu043e\x5cu\
0431\x5cu044a\x5cu0435\
\x5cu043a\x5cu0442 \x5cu0\
441\x5cu0442\x5cu0430\x5c\
u0442\x5cu0443\x5cu044\
1\x5cu0432\x5cu043e\x5cu0\
437\x5cu0432\x5cu0440\x5c\
u0430\x5cu0442\x5cu043\
0 \x5cu0441\x5cu0442\x5cu\
0440\x5cu043a\x5cu043e\
\x5cu043b\x5cu0438\x5cu04\
47\x5cu0435\x5cu0441\x5cu\
0442\x5cu0432\x5cu043e\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu043e\x5cu043a \x5c\
u0441\x5cu0444\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
438\x5cu0440\x5cu043e\x5c\
u0432\x5cu0430\x5cu044\
2\x5cu044c\x5cu043f\x5cu0\
43e\x5cu0437\x5cu0438\x5c\
u0446\x5cu0438\x5cu044\
e\x5cu0434\x5cu043e\x5cu0\
43a\x5cu0443\x5cu043c\x5c\
u0435\x5cu043d\x5cu044\
2\x5cu0430 \x5cu0441\x5cu\
0447\x5cu0435\x5cu0442\
\x5cu043f\x5cu043e\x5cu04\
3a\x5cu043e\x5cu0434\x5cu\
0443 \x5cu0442\x5cu043\
5\x5cu043a\x5cu0443\x5cu0\
449\x5cu0435\x5cu0435\x5c\
u0432\x5cu0440\x5cu043\
5\x5cu043c\x5cu044f \x5cu\
0442\x5cu0438\x5cu043f\
\x5cu0437\x5cu043d\x5cu04\
30\x5cu0447\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
 \x5cu0442\x5cu0438\x5cu0\
43f\x5cu0437\x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f\x5cu0441\x5cu0442\x5c\
u0440 \x5cu0443\x5cu04\
41\x5cu0442\x5cu0430\x5cu\
043d\x5cu043e\x5cu0432\
\x5cu0438\x5cu0442\x5cu04\
4c\x5cu0442\x5cu0430\x5cu\
043d\x5cu0430 \x5cu044\
3\x5cu0441\x5cu0442\x5cu0\
430\x5cu043d\x5cu043e\x5c\
u0432\x5cu0438\x5cu044\
2\x5cu044c\x5cu0442\x5cu0\
430\x5cu043f\x5cu043e \
\x5cu0444\x5cu0438\x5cu04\
3a\x5cu0441\x5cu0448\x5cu\
0430\x5cu0431\x5cu043b\
\x5cu043e\x5cu043d \x5cu0\
448\x5cu0430\x5cu0431\x5c\
u043b\x5cu043e\x5cu043\
d acos asin atan\
 base64\x5cu0437\x5cu0\
43d\x5cu0430\x5cu0447\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435 base64\x5cu\
0441\x5cu0442\x5cu0440\
\x5cu043e\x5cu043a\x5cu04\
30 cos exp log l\
og10 pow sin sqr\
t tan xml\x5cu0437\x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435 xml\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u043e\x5cu043a\x5cu043\
0 xml\x5cu0442\x5cu043\
8\x5cu043f xml\x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
437\x5cu043d\x5cu0447 \
\x5cu0430\x5cu043a\x5cu04\
42\x5cu0438\x5cu0432\x5cu\
043d\x5cu043e\x5cu0435\
\x5cu043e\x5cu043a\x5cu04\
3d\x5cu043e \x5cu0431\x5c\
u0435\x5cu0437\x5cu043\
e\x5cu043f\x5cu0430\x5cu0\
441\x5cu043d\x5cu044b\x5c\
u0439\x5cu0440\x5cu043\
5\x5cu0436\x5cu0438\x5cu0\
43c \x5cu0431\x5cu0435\
\x5cu0437\x5cu043e\x5cu04\
3f\x5cu0430\x5cu0441\x5cu\
043d\x5cu044b\x5cu0439\
\x5cu0440\x5cu0435\x5cu04\
36\x5cu0438\x5cu043c\x5cu\
0440\x5cu0430\x5cu0437\
\x5cu0434\x5cu0435\x5cu04\
3b\x5cu0435\x5cu043d\x5cu\
0438\x5cu044f\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0445 \x5c\
u0431\x5cu0443\x5cu043\
b\x5cu0435\x5cu0432\x5cu0\
43e \x5cu0432\x5cu0432\
\x5cu0435\x5cu0441\x5cu04\
42\x5cu0438\x5cu0434\x5cu\
0430\x5cu0442\x5cu0443\
 \x5cu0432\x5cu0432\x5cu0\
435\x5cu0441\x5cu0442\x5c\
u0438\x5cu0437\x5cu043\
d\x5cu0430\x5cu0447\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435 \x5cu0432\x5cu04\
32\x5cu0435\x5cu0441\x5cu\
0442\x5cu0438\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
3e\x5cu043a\x5cu0443 \x5c\
u0432\x5cu0432\x5cu043\
5\x5cu0441\x5cu0442\x5cu0\
438\x5cu0447\x5cu0438\x5c\
u0441\x5cu043b\x5cu043\
e \x5cu0432\x5cu043e\x5cu\
0437\x5cu043c\x5cu043e\
\x5cu0436\x5cu043d\x5cu04\
3e\x5cu0441\x5cu0442\x5cu\
044c\x5cu0447\x5cu0442\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044fxml \x5cu04\
32\x5cu043e\x5cu043f\x5cu\
0440\x5cu043e\x5cu0441\
 \x5cu0432\x5cu043e\x5cu0\
441\x5cu0441\x5cu0442\x5c\
u0430\x5cu043d\x5cu043\
e\x5cu0432\x5cu0438\x5cu0\
442\x5cu044c\x5cu0437\x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435 \x5cu0432\
\x5cu0440\x5cu0435\x5cu04\
33 \x5cu0432\x5cu044b\x5c\
u0433\x5cu0440\x5cu044\
3\x5cu0437\x5cu0438\x5cu0\
442\x5cu044c\x5cu0436\x5c\
u0443\x5cu0440\x5cu043\
d\x5cu0430\x5cu043b\x5cu0\
440\x5cu0435\x5cu0433\x5c\
u0438\x5cu0441\x5cu044\
2\x5cu0440\x5cu0430\x5cu0\
446\x5cu0438\x5cu0438 \
\x5cu0432\x5cu044b\x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
043d\x5cu0438\x5cu0442\
\x5cu044c\x5cu043e\x5cu04\
31\x5cu0440\x5cu0430\x5cu\
0431\x5cu043e\x5cu0442\
\x5cu043a\x5cu0443\x5cu04\
3e\x5cu043f\x5cu043e\x5cu\
0432\x5cu0435\x5cu0449\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f \x5cu0432\x5c\
u044b\x5cu043f\x5cu043\
e\x5cu043b\x5cu043d\x5cu0\
438\x5cu0442\x5cu044c\x5c\
u043f\x5cu0440\x5cu043\
e\x5cu0432\x5cu0435\x5cu0\
440\x5cu043a\x5cu0443\x5c\
u043f\x5cu0440\x5cu043\
0\x5cu0432\x5cu0434\x5cu0\
43e\x5cu0441\x5cu0442\x5c\
u0443\x5cu043f\x5cu043\
0 \x5cu0432\x5cu044b\x5cu\
0447\x5cu0438\x5cu0441\
\x5cu043b\x5cu0438\x5cu04\
42\x5cu044c \x5cu0433\x5c\
u043e\x5cu0434 \x5cu04\
34\x5cu0430\x5cu043d\x5cu\
043d\x5cu044b\x5cu0435\
\x5cu0444\x5cu043e\x5cu04\
40\x5cu043c\x5cu044b\x5cu\
0432\x5cu0437\x5cu043d\
\x5cu0430\x5cu0447\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435 \x5cu0434\x5cu043\
0\x5cu0442\x5cu0430 \x5cu\
0434\x5cu0435\x5cu043d\
\x5cu044c \x5cu0434\x5cu0\
435\x5cu043d\x5cu044c\x5c\
u0433\x5cu043e\x5cu043\
4\x5cu0430 \x5cu0434\x5cu\
0435\x5cu043d\x5cu044c\
\x5cu043d\x5cu0435\x5cu04\
34\x5cu0435\x5cu043b\x5cu\
0438 \x5cu0434\x5cu043\
e\x5cu0431\x5cu0430\x5cu0\
432\x5cu0438\x5cu0442\x5c\
u044c\x5cu043c\x5cu043\
5\x5cu0441\x5cu044f\x5cu0\
446 \x5cu0437\x5cu0430\
\x5cu0431\x5cu043b\x5cu04\
3e\x5cu043a\x5cu0438\x5cu\
0440\x5cu043e\x5cu0432\
\x5cu0430\x5cu0442\x5cu04\
4c\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0435\x5cu0434\x5cu04\
3b\x5cu044f\x5cu0440\x5cu\
0435\x5cu0434\x5cu0430\
\x5cu043a\x5cu0442\x5cu04\
38\x5cu0440\x5cu043e\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu0438\x5cu044f \x5cu0\
437\x5cu0430\x5cu0431\x5c\
u043b\x5cu043e\x5cu043\
a\x5cu0438\x5cu0440\x5cu0\
43e\x5cu0432\x5cu0430\x5c\
u0442\x5cu044c\x5cu044\
0\x5cu0430\x5cu0431\x5cu0\
43e\x5cu0442\x5cu0443\x5c\
u043f\x5cu043e\x5cu043\
b\x5cu044c\x5cu0437\x5cu0\
43e\x5cu0432\x5cu0430\x5c\
u0442\x5cu0435\x5cu043\
b\x5cu044f \x5cu0437\x5cu\
0430\x5cu0432\x5cu0435\
\x5cu0440\x5cu0448\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
0440\x5cu0430\x5cu0431\
\x5cu043e\x5cu0442\x5cu04\
43\x5cu0441\x5cu0438\x5cu\
0441\x5cu0442\x5cu0435\
\x5cu043c\x5cu044b \x5cu0\
437\x5cu0430\x5cu0433\x5c\
u0440\x5cu0443\x5cu043\
7\x5cu0438\x5cu0442\x5cu0\
44c\x5cu0432\x5cu043d\x5c\
u0435\x5cu0448\x5cu043\
d\x5cu044e\x5cu044e\x5cu0\
43a\x5cu043e\x5cu043c\x5c\
u043f\x5cu043e\x5cu043\
d\x5cu0435\x5cu043d\x5cu0\
442\x5cu0443 \x5cu0437\
\x5cu0430\x5cu043a\x5cu04\
40\x5cu044b\x5cu0442\x5cu\
044c\x5cu0441\x5cu043f\
\x5cu0440\x5cu0430\x5cu04\
32\x5cu043a\x5cu0443 \x5c\
u0437\x5cu0430\x5cu043\
f\x5cu0438\x5cu0441\x5cu0\
430\x5cu0442\x5cu044cj\
son \x5cu0437\x5cu0430\
\x5cu043f\x5cu0438\x5cu04\
41\x5cu0430\x5cu0442\x5cu\
044cxml \x5cu0437\x5cu\
0430\x5cu043f\x5cu0438\
\x5cu0441\x5cu0430\x5cu04\
42\x5cu044c\x5cu0434\x5cu\
0430\x5cu0442\x5cu0443\
json \x5cu0437\x5cu043\
0\x5cu043f\x5cu0438\x5cu0\
441\x5cu044c\x5cu0436\x5c\
u0443\x5cu0440\x5cu043\
d\x5cu0430\x5cu043b\x5cu0\
430\x5cu0440\x5cu0435\x5c\
u0433\x5cu0438\x5cu044\
1\x5cu0442\x5cu0440\x5cu0\
430\x5cu0446\x5cu0438\x5c\
u0438 \x5cu0437\x5cu04\
30\x5cu043f\x5cu043e\x5cu\
043b\x5cu043d\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
37\x5cu043d\x5cu0430\x5cu\
0447\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
41\x5cu0432\x5cu043e\x5cu\
0439\x5cu0441\x5cu0442\
\x5cu0432 \x5cu0437\x5cu0\
430\x5cu043f\x5cu0440\x5c\
u043e\x5cu0441\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
440\x5cu0430\x5cu0437\x5c\
u0440\x5cu0435\x5cu044\
8\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu043f\x5c\
u043e\x5cu043b\x5cu044\
c\x5cu0437\x5cu043e\x5cu0\
432\x5cu0430\x5cu0442\x5c\
u0435\x5cu043b\x5cu044\
f \x5cu0437\x5cu0430\x5cu\
043f\x5cu0443\x5cu0441\
\x5cu0442\x5cu0438\x5cu04\
42\x5cu044c\x5cu043f\x5cu\
0440\x5cu0438\x5cu043b\
\x5cu043e\x5cu0436\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435 \x5cu0437\x5cu043\
0\x5cu043f\x5cu0443\x5cu0\
441\x5cu0442\x5cu0438\x5c\
u0442\x5cu044c\x5cu044\
1\x5cu0438\x5cu0441\x5cu0\
442\x5cu0435\x5cu043c\x5c\
u0443 \x5cu0437\x5cu04\
30\x5cu0444\x5cu0438\x5cu\
043a\x5cu0441\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0430\x5cu0442\x5cu\
044c\x5cu0442\x5cu0440\
\x5cu0430\x5cu043d\x5cu04\
37\x5cu0430\x5cu043a\x5cu\
0446\x5cu0438\x5cu044e\
 \x5cu0437\x5cu043d\x5cu0\
430\x5cu0447\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu0432\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0435\x5cu044\
4\x5cu043e\x5cu0440\x5cu0\
43c\x5cu044b \x5cu0437\
\x5cu043d\x5cu0430\x5cu04\
47\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu0432\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu043e\x5cu043a\x5cu\
0443\x5cu0432\x5cu043d\
\x5cu0443\x5cu0442\x5cu04\
40 \x5cu0437\x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu0432\x5cu0444\x5c\
u0430\x5cu0439\x5cu043\
b \x5cu0437\x5cu043d\x5cu\
0430\x5cu0447\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu0437\x5cu0430\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu043d\x5cu0435\x5cu04\
3d\x5cu043e \x5cu0437\x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu0438\x5c\
u0437\x5cu0441\x5cu044\
2\x5cu0440\x5cu043e\x5cu0\
43a\x5cu0438\x5cu0432\x5c\
u043d\x5cu0443\x5cu044\
2\x5cu0440 \x5cu0437\x5cu\
043d\x5cu0430\x5cu0447\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0438\x5cu\
0437\x5cu0444\x5cu0430\
\x5cu0439\x5cu043b\x5cu04\
30 \x5cu0438\x5cu0437x\
ml\x5cu0442\x5cu0438\x5cu\
043f\x5cu0430 \x5cu043\
8\x5cu043c\x5cu043f\x5cu0\
43e\x5cu0440\x5cu0442\x5c\
u043c\x5cu043e\x5cu043\
4\x5cu0435\x5cu043b\x5cu0\
438xdto \x5cu0438\x5cu\
043c\x5cu044f\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu044c\x5cu044e\x5cu\
0442\x5cu0435\x5cu0440\
\x5cu0430 \x5cu0438\x5cu0\
43c\x5cu044f\x5cu043f\x5c\
u043e\x5cu043b\x5cu044\
c\x5cu0437\x5cu043e\x5cu0\
432\x5cu0430\x5cu0442\x5c\
u0435\x5cu043b\x5cu044\
f \x5cu0438\x5cu043d\x5cu\
0438\x5cu0446\x5cu0438\
\x5cu0430\x5cu043b\x5cu04\
38\x5cu0437\x5cu0438\x5cu\
0440\x5cu043e\x5cu0432\
\x5cu0430\x5cu0442\x5cu04\
4c\x5cu043f\x5cu0440\x5cu\
0435\x5cu0434\x5cu043e\
\x5cu043f\x5cu0440\x5cu04\
35\x5cu0434\x5cu0435\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
35\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0435 \x5cu0438\x5cu0\
43d\x5cu0444\x5cu043e\x5c\
u0440\x5cu043c\x5cu043\
0\x5cu0446\x5cu0438\x5cu0\
44f\x5cu043e\x5cu0431\x5c\
u043e\x5cu0448\x5cu043\
8\x5cu0431\x5cu043a\x5cu0\
435 \x5cu043a\x5cu0430\
\x5cu0442\x5cu0430\x5cu04\
3b\x5cu043e\x5cu0433\x5cu\
0431\x5cu0438\x5cu0431\
\x5cu043b\x5cu0438\x5cu04\
3e\x5cu0442\x5cu0435\x5cu\
043a\x5cu0438\x5cu043c\
\x5cu043e\x5cu0431\x5cu04\
38\x5cu043b\x5cu044c\x5cu\
043d\x5cu043e\x5cu0433\
\x5cu043e\x5cu0443\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
043e\x5cu0439\x5cu0441\
\x5cu0442\x5cu0432\x5cu04\
30 \x5cu043a\x5cu0430\x5c\
u0442\x5cu0430\x5cu043\
b\x5cu043e\x5cu0433\x5cu0\
432\x5cu0440\x5cu0435\x5c\
u043c\x5cu0435\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445\x5cu0444\x5cu0430\x5c\
u0439\x5cu043b\x5cu043\
e\x5cu0432 \x5cu043a\x5cu\
0430\x5cu0442\x5cu0430\
\x5cu043b\x5cu043e\x5cu04\
33\x5cu0434\x5cu043e\x5cu\
043a\x5cu0443\x5cu043c\
\x5cu0435\x5cu043d\x5cu04\
42\x5cu043e\x5cu0432 \x5c\
u043a\x5cu0430\x5cu044\
2\x5cu0430\x5cu043b\x5cu0\
43e\x5cu0433\x5cu043f\x5c\
u0440\x5cu043e\x5cu043\
3\x5cu0440\x5cu0430\x5cu0\
43c\x5cu043c\x5cu044b \
\x5cu043a\x5cu043e\x5cu04\
34\x5cu0438\x5cu0440\x5cu\
043e\x5cu0432\x5cu0430\
\x5cu0442\x5cu044c\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
043e\x5cu043a\x5cu0443\
 \x5cu043a\x5cu043e\x5cu0\
434\x5cu043b\x5cu043e\x5c\
u043a\x5cu0430\x5cu043\
b\x5cu0438\x5cu0437\x5cu0\
430\x5cu0446\x5cu0438\x5c\
u0438\x5cu0438\x5cu043\
d\x5cu0444\x5cu043e\x5cu0\
440\x5cu043c\x5cu0430\x5c\
u0446\x5cu0438\x5cu043\
e\x5cu043d\x5cu043d\x5cu0\
43e\x5cu0439\x5cu0431\x5c\
u0430\x5cu0437\x5cu044\
b \x5cu043a\x5cu043e\x5cu\
0434\x5cu0441\x5cu0438\
\x5cu043c\x5cu0432\x5cu04\
3e\x5cu043b\x5cu0430 \x5c\
u043a\x5cu043e\x5cu043\
c\x5cu0430\x5cu043d\x5cu0\
434\x5cu0430\x5cu0441\x5c\
u0438\x5cu0441\x5cu044\
2\x5cu0435\x5cu043c\x5cu0\
44b \x5cu043a\x5cu043e\
\x5cu043d\x5cu0435\x5cu04\
46\x5cu0433\x5cu043e\x5cu\
0434\x5cu0430 \x5cu043\
a\x5cu043e\x5cu043d\x5cu0\
435\x5cu0446\x5cu0434\x5c\
u043d\x5cu044f \x5cu04\
3a\x5cu043e\x5cu043d\x5cu\
0435\x5cu0446\x5cu043a\
\x5cu0432\x5cu0430\x5cu04\
40\x5cu0442\x5cu0430\x5cu\
043b\x5cu0430 \x5cu043\
a\x5cu043e\x5cu043d\x5cu0\
435\x5cu0446\x5cu043c\x5c\
u0435\x5cu0441\x5cu044\
f\x5cu0446\x5cu0430 \x5cu\
043a\x5cu043e\x5cu043d\
\x5cu0435\x5cu0446\x5cu04\
3c\x5cu0438\x5cu043d\x5cu\
0443\x5cu0442\x5cu044b\
 \x5cu043a\x5cu043e\x5cu0\
43d\x5cu0435\x5cu0446\x5c\
u043d\x5cu0435\x5cu043\
4\x5cu0435\x5cu043b\x5cu0\
438 \x5cu043a\x5cu043e\
\x5cu043d\x5cu0435\x5cu04\
46\x5cu0447\x5cu0430\x5cu\
0441\x5cu0430 \x5cu043\
a\x5cu043e\x5cu043d\x5cu0\
444\x5cu0438\x5cu0433\x5c\
u0443\x5cu0440\x5cu043\
0\x5cu0446\x5cu0438\x5cu0\
44f\x5cu0431\x5cu0430\x5c\
u0437\x5cu044b\x5cu043\
4\x5cu0430\x5cu043d\x5cu0\
43d\x5cu044b\x5cu0445\x5c\
u0438\x5cu0437\x5cu043\
c\x5cu0435\x5cu043d\x5cu0\
435\x5cu043d\x5cu0430\x5c\
u0434\x5cu0438\x5cu043\
d\x5cu0430\x5cu043c\x5cu0\
438\x5cu0447\x5cu0435\x5c\
u0441\x5cu043a\x5cu043\
8 \x5cu043a\x5cu043e\x5cu\
043d\x5cu0444\x5cu0438\
\x5cu0433\x5cu0443\x5cu04\
40\x5cu0430\x5cu0446\x5cu\
0438\x5cu044f\x5cu0438\
\x5cu0437\x5cu043c\x5cu04\
35\x5cu043d\x5cu0435\x5cu\
043d\x5cu0430 \x5cu043\
a\x5cu043e\x5cu043f\x5cu0\
438\x5cu0440\x5cu043e\x5c\
u0432\x5cu0430\x5cu044\
2\x5cu044c\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0435\x5cu044\
4\x5cu043e\x5cu0440\x5cu0\
43c\x5cu044b \x5cu043a\
\x5cu043e\x5cu043f\x5cu04\
38\x5cu0440\x5cu043e\x5cu\
0432\x5cu0430\x5cu0442\
\x5cu044c\x5cu0444\x5cu04\
30\x5cu0439\x5cu043b \x5c\
u043a\x5cu0440\x5cu043\
0\x5cu0442\x5cu043a\x5cu0\
43e\x5cu0435\x5cu043f\x5c\
u0440\x5cu0435\x5cu043\
4\x5cu0441\x5cu0442\x5cu0\
430\x5cu0432\x5cu043b\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435\x5cu043e\x5cu0\
448\x5cu0438\x5cu0431\x5c\
u043a\x5cu0438 \x5cu04\
3b\x5cu0435\x5cu0432 \x5c\
u043c\x5cu0430\x5cu043\
a\x5cu0441 \x5cu043c\x5cu\
0435\x5cu0441\x5cu0442\
\x5cu043d\x5cu043e\x5cu04\
35\x5cu0432\x5cu0440\x5cu\
0435\x5cu043c\x5cu044f\
 \x5cu043c\x5cu0435\x5cu0\
441\x5cu044f\x5cu0446 \
\x5cu043c\x5cu0438\x5cu04\
3d \x5cu043c\x5cu0438\x5c\
u043d\x5cu0443\x5cu044\
2\x5cu0430 \x5cu043c\x5cu\
043e\x5cu043d\x5cu043e\
\x5cu043f\x5cu043e\x5cu04\
3b\x5cu044c\x5cu043d\x5cu\
044b\x5cu0439\x5cu0440\
\x5cu0435\x5cu0436\x5cu04\
38\x5cu043c \x5cu043d\x5c\
u0430\x5cu0439\x5cu044\
2\x5cu0438 \x5cu043d\x5cu\
0430\x5cu0439\x5cu0442\
\x5cu0438\x5cu043d\x5cu04\
35\x5cu0434\x5cu043e\x5cu\
043f\x5cu0443\x5cu0441\
\x5cu0442\x5cu0438\x5cu04\
3c\x5cu044b\x5cu0435\x5cu\
0441\x5cu0438\x5cu043c\
\x5cu0432\x5cu043e\x5cu04\
3b\x5cu044bxml \x5cu04\
3d\x5cu0430\x5cu0439\x5cu\
0442\x5cu0438\x5cu043e\
\x5cu043a\x5cu043d\x5cu04\
3e\x5cu043f\x5cu043e\x5cu\
043d\x5cu0430\x5cu0432\
\x5cu0438\x5cu0433\x5cu04\
30\x5cu0446\x5cu0438\x5cu\
043e\x5cu043d\x5cu043d\
\x5cu043e\x5cu0439\x5cu04\
41\x5cu0441\x5cu044b\x5cu\
043b\x5cu043a\x5cu0435\
 \x5cu043d\x5cu0430\x5cu0\
439\x5cu0442\x5cu0438\x5c\
u043f\x5cu043e\x5cu043\
c\x5cu0435\x5cu0447\x5cu0\
435\x5cu043d\x5cu043d\x5c\
u044b\x5cu0435\x5cu043\
d\x5cu0430\x5cu0443\x5cu0\
434\x5cu0430\x5cu043b\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435 \x5cu043d\x5cu\
0430\x5cu0439\x5cu0442\
\x5cu0438\x5cu043f\x5cu04\
3e\x5cu0441\x5cu0441\x5cu\
044b\x5cu043b\x5cu043a\
\x5cu0430\x5cu043c \x5cu0\
43d\x5cu0430\x5cu0439\x5c\
u0442\x5cu0438\x5cu044\
4\x5cu0430\x5cu0439\x5cu0\
43b\x5cu044b \x5cu043d\
\x5cu0430\x5cu0447\x5cu04\
30\x5cu043b\x5cu043e\x5cu\
0433\x5cu043e\x5cu0434\
\x5cu0430 \x5cu043d\x5cu0\
430\x5cu0447\x5cu0430\x5c\
u043b\x5cu043e\x5cu043\
4\x5cu043d\x5cu044f \x5cu\
043d\x5cu0430\x5cu0447\
\x5cu0430\x5cu043b\x5cu04\
3e\x5cu043a\x5cu0432\x5cu\
0430\x5cu0440\x5cu0442\
\x5cu0430\x5cu043b\x5cu04\
30 \x5cu043d\x5cu0430\x5c\
u0447\x5cu0430\x5cu043\
b\x5cu043e\x5cu043c\x5cu0\
435\x5cu0441\x5cu044f\x5c\
u0446\x5cu0430 \x5cu04\
3d\x5cu0430\x5cu0447\x5cu\
0430\x5cu043b\x5cu043e\
\x5cu043c\x5cu0438\x5cu04\
3d\x5cu0443\x5cu0442\x5cu\
044b \x5cu043d\x5cu043\
0\x5cu0447\x5cu0430\x5cu0\
43b\x5cu043e\x5cu043d\x5c\
u0435\x5cu0434\x5cu043\
5\x5cu043b\x5cu0438 \x5cu\
043d\x5cu0430\x5cu0447\
\x5cu0430\x5cu043b\x5cu04\
3e\x5cu0447\x5cu0430\x5cu\
0441\x5cu0430 \x5cu043\
d\x5cu0430\x5cu0447\x5cu0\
430\x5cu0442\x5cu044c\x5c\
u0437\x5cu0430\x5cu043\
f\x5cu0440\x5cu043e\x5cu0\
441\x5cu0440\x5cu0430\x5c\
u0437\x5cu0440\x5cu043\
5\x5cu0448\x5cu0435\x5cu0\
43d\x5cu0438\x5cu044f\x5c\
u043f\x5cu043e\x5cu043\
b\x5cu044c\x5cu0437\x5cu0\
43e\x5cu0432\x5cu0430\x5c\
u0442\x5cu0435\x5cu043\
b\x5cu044f \x5cu043d\x5cu\
0430\x5cu0447\x5cu0430\
\x5cu0442\x5cu044c\x5cu04\
37\x5cu0430\x5cu043f\x5cu\
0443\x5cu0441\x5cu043a\
\x5cu043f\x5cu0440\x5cu04\
38\x5cu043b\x5cu043e\x5cu\
0436\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f \x5cu0\
43d\x5cu0430\x5cu0447\x5c\
u0430\x5cu0442\x5cu044\
c\x5cu043a\x5cu043e\x5cu0\
43f\x5cu0438\x5cu0440\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
435\x5cu0444\x5cu0430\x5c\
u0439\x5cu043b\x5cu043\
0 \x5cu043d\x5cu0430\x5cu\
0447\x5cu0430\x5cu0442\
\x5cu044c\x5cu043f\x5cu04\
35\x5cu0440\x5cu0435\x5cu\
043c\x5cu0435\x5cu0449\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0444\x5cu\
0430\x5cu0439\x5cu043b\
\x5cu0430 \x5cu043d\x5cu0\
430\x5cu0447\x5cu0430\x5c\
u0442\x5cu044c\x5cu043\
f\x5cu043e\x5cu0434\x5cu0\
43a\x5cu043b\x5cu044e\x5c\
u0447\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
432\x5cu043d\x5cu0435\x5c\
u0448\x5cu043d\x5cu043\
5\x5cu0439\x5cu043a\x5cu0\
43e\x5cu043c\x5cu043f\x5c\
u043e\x5cu043d\x5cu043\
5\x5cu043d\x5cu0442\x5cu0\
44b \x5cu043d\x5cu0430\
\x5cu0447\x5cu0430\x5cu04\
42\x5cu044c\x5cu043f\x5cu\
043e\x5cu0434\x5cu043a\
\x5cu043b\x5cu044e\x5cu04\
47\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu0440\
\x5cu0430\x5cu0441\x5cu04\
48\x5cu0438\x5cu0440\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu0440\x5cu04\
30\x5cu0431\x5cu043e\x5cu\
0442\x5cu044b\x5cu0441\
\x5cu043a\x5cu0440\x5cu04\
38\x5cu043f\x5cu0442\x5cu\
043e\x5cu0433\x5cu0440\
\x5cu0430\x5cu0444\x5cu04\
38\x5cu0435\x5cu0439 \x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0430\x5cu0442\x5cu0\
44c\x5cu043f\x5cu043e\x5c\
u0434\x5cu043a\x5cu043\
b\x5cu044e\x5cu0447\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435\x5cu0440\x5cu043\
0\x5cu0441\x5cu0448\x5cu0\
438\x5cu0440\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f\x5cu0440\x5cu0430\x5cu0\
431\x5cu043e\x5cu0442\x5c\
u044b\x5cu0441\x5cu044\
4\x5cu0430\x5cu0439\x5cu0\
43b\x5cu0430\x5cu043c\x5c\
u0438 \x5cu043d\x5cu04\
30\x5cu0447\x5cu0430\x5cu\
0442\x5cu044c\x5cu043f\
\x5cu043e\x5cu0438\x5cu04\
41\x5cu043a\x5cu0444\x5cu\
0430\x5cu0439\x5cu043b\
\x5cu043e\x5cu0432 \x5cu0\
43d\x5cu0430\x5cu0447\x5c\
u0430\x5cu0442\x5cu044\
c\x5cu043f\x5cu043e\x5cu0\
43b\x5cu0443\x5cu0447\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435\x5cu043a\x5cu0\
430\x5cu0442\x5cu0430\x5c\
u043b\x5cu043e\x5cu043\
3\x5cu0430\x5cu0432\x5cu0\
440\x5cu0435\x5cu043c\x5c\
u0435\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445\x5cu0\
444\x5cu0430\x5cu0439\x5c\
u043b\x5cu043e\x5cu043\
2 \x5cu043d\x5cu0430\x5cu\
0447\x5cu0430\x5cu0442\
\x5cu044c\x5cu043f\x5cu04\
3e\x5cu043b\x5cu0443\x5cu\
0447\x5cu0435\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
3a\x5cu0430\x5cu0442\x5cu\
0430\x5cu043b\x5cu043e\
\x5cu0433\x5cu0430\x5cu04\
34\x5cu043e\x5cu043a\x5cu\
0443\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
3e\x5cu0432 \x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
0\x5cu0442\x5cu044c\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u0443\x5cu0447\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu0440\x5cu0430\x5c\
u0431\x5cu043e\x5cu044\
7\x5cu0435\x5cu0433\x5cu0\
43e\x5cu043a\x5cu0430\x5c\
u0442\x5cu0430\x5cu043\
b\x5cu043e\x5cu0433\x5cu0\
430\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445\x5cu043f\x5cu0\
43e\x5cu043b\x5cu044c\x5c\
u0437\x5cu043e\x5cu043\
2\x5cu0430\x5cu0442\x5cu0\
435\x5cu043b\x5cu044f \
\x5cu043d\x5cu0430\x5cu04\
47\x5cu0430\x5cu0442\x5cu\
044c\x5cu043f\x5cu043e\
\x5cu043b\x5cu0443\x5cu04\
47\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu0444\
\x5cu0430\x5cu0439\x5cu04\
3b\x5cu043e\x5cu0432 \x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0430\x5cu0442\x5cu0\
44c\x5cu043f\x5cu043e\x5c\
u043c\x5cu0435\x5cu044\
9\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu0444\x5c\
u0430\x5cu0439\x5cu043\
b\x5cu0430 \x5cu043d\x5cu\
0430\x5cu0447\x5cu0430\
\x5cu0442\x5cu044c\x5cu04\
3f\x5cu043e\x5cu043c\x5cu\
0435\x5cu0449\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu0444\x5cu0430\x5cu\
0439\x5cu043b\x5cu043e\
\x5cu0432 \x5cu043d\x5cu0\
430\x5cu0447\x5cu0430\x5c\
u0442\x5cu044c\x5cu044\
1\x5cu043e\x5cu0437\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u0438\x5cu0435\x5cu043\
4\x5cu0432\x5cu043e\x5cu0\
438\x5cu0447\x5cu043d\x5c\
u044b\x5cu0445\x5cu043\
4\x5cu0430\x5cu043d\x5cu0\
43d\x5cu044b\x5cu0445\x5c\
u0438\x5cu0437\x5cu044\
4\x5cu0430\x5cu0439\x5cu0\
43b\x5cu0430 \x5cu043d\
\x5cu0430\x5cu0447\x5cu04\
30\x5cu0442\x5cu044c\x5cu\
0441\x5cu043e\x5cu0437\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
043a\x5cu0430\x5cu0442\
\x5cu0430\x5cu043b\x5cu04\
3e\x5cu0433\x5cu0430 \x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0430\x5cu0442\x5cu0\
44c\x5cu0442\x5cu0440\x5c\
u0430\x5cu043d\x5cu043\
7\x5cu0430\x5cu043a\x5cu0\
446\x5cu0438\x5cu044e \
\x5cu043d\x5cu0430\x5cu04\
47\x5cu0430\x5cu0442\x5cu\
044c\x5cu0443\x5cu0434\
\x5cu0430\x5cu043b\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu0444\x5cu0430\
\x5cu0439\x5cu043b\x5cu04\
3e\x5cu0432 \x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
0\x5cu0442\x5cu044c\x5cu0\
443\x5cu0441\x5cu0442\x5c\
u0430\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
443\x5cu0432\x5cu043d\x5c\
u0435\x5cu0448\x5cu043\
d\x5cu0435\x5cu0439\x5cu0\
43a\x5cu043e\x5cu043c\x5c\
u043f\x5cu043e\x5cu043\
d\x5cu0435\x5cu043d\x5cu0\
442\x5cu044b \x5cu043d\
\x5cu0430\x5cu0447\x5cu04\
30\x5cu0442\x5cu044c\x5cu\
0443\x5cu0441\x5cu0442\
\x5cu0430\x5cu043d\x5cu04\
3e\x5cu0432\x5cu043a\x5cu\
0443\x5cu0440\x5cu0430\
\x5cu0441\x5cu0448\x5cu04\
38\x5cu0440\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu0440\x5cu0430\x5cu04\
31\x5cu043e\x5cu0442\x5cu\
044b\x5cu0441\x5cu043a\
\x5cu0440\x5cu0438\x5cu04\
3f\x5cu0442\x5cu043e\x5cu\
0433\x5cu0440\x5cu0430\
\x5cu0444\x5cu0438\x5cu04\
35\x5cu0439 \x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
0\x5cu0442\x5cu044c\x5cu0\
443\x5cu0441\x5cu0442\x5c\
u0430\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
443\x5cu0440\x5cu0430\x5c\
u0441\x5cu0448\x5cu043\
8\x5cu0440\x5cu0435\x5cu0\
43d\x5cu0438\x5cu044f\x5c\
u0440\x5cu0430\x5cu043\
1\x5cu043e\x5cu0442\x5cu0\
44b\x5cu0441\x5cu0444\x5c\
u0430\x5cu0439\x5cu043\
b\x5cu0430\x5cu043c\x5cu0\
438 \x5cu043d\x5cu0435\
\x5cu0434\x5cu0435\x5cu04\
3b\x5cu044f\x5cu0433\x5cu\
043e\x5cu0434\x5cu0430\
 \x5cu043d\x5cu0435\x5cu0\
43e\x5cu0431\x5cu0445\x5c\
u043e\x5cu0434\x5cu043\
8\x5cu043c\x5cu043e\x5cu0\
441\x5cu0442\x5cu044c\x5c\
u0437\x5cu0430\x5cu043\
2\x5cu0435\x5cu0440\x5cu0\
448\x5cu0435\x5cu043d\x5c\
u0438\x5cu044f\x5cu044\
1\x5cu043e\x5cu0435\x5cu0\
434\x5cu0438\x5cu043d\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f \x5cu043d\x5cu\
043e\x5cu043c\x5cu0435\
\x5cu0440\x5cu0441\x5cu04\
35\x5cu0430\x5cu043d\x5cu\
0441\x5cu0430\x5cu0438\
\x5cu043d\x5cu0444\x5cu04\
3e\x5cu0440\x5cu043c\x5cu\
0430\x5cu0446\x5cu0438\
\x5cu043e\x5cu043d\x5cu04\
3d\x5cu043e\x5cu0439\x5cu\
0431\x5cu0430\x5cu0437\
\x5cu044b \x5cu043d\x5cu0\
43e\x5cu043c\x5cu0435\x5c\
u0440\x5cu0441\x5cu043\
e\x5cu0435\x5cu0434\x5cu0\
438\x5cu043d\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f\x5cu0438\x5cu043d\x5cu0\
444\x5cu043e\x5cu0440\x5c\
u043c\x5cu0430\x5cu044\
6\x5cu0438\x5cu043e\x5cu0\
43d\x5cu043d\x5cu043e\x5c\
u0439\x5cu0431\x5cu043\
0\x5cu0437\x5cu044b \x5cu\
043d\x5cu0440\x5cu0435\
\x5cu0433 \x5cu043d\x5cu0\
441\x5cu0442\x5cu0440 \
\x5cu043e\x5cu0431\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu0438\x5cu043d\x5cu04\
42\x5cu0435\x5cu0440\x5cu\
0444\x5cu0435\x5cu0439\
\x5cu0441 \x5cu043e\x5cu0\
431\x5cu043d\x5cu043e\x5c\
u0432\x5cu0438\x5cu044\
2\x5cu044c\x5cu043d\x5cu0\
443\x5cu043c\x5cu0435\x5c\
u0440\x5cu0430\x5cu044\
6\x5cu0438\x5cu044e\x5cu0\
43e\x5cu0431\x5cu044a\x5c\
u0435\x5cu043a\x5cu044\
2\x5cu043e\x5cu0432 \x5cu\
043e\x5cu0431\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
043f\x5cu043e\x5cu0432\
\x5cu0442\x5cu043e\x5cu04\
40\x5cu043d\x5cu043e\x5cu\
0438\x5cu0441\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
4c\x5cu0437\x5cu0443\x5cu\
0435\x5cu043c\x5cu044b\
\x5cu0435\x5cu0437\x5cu04\
3d\x5cu0430\x5cu0447\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f \x5cu043e\x5cu0\
431\x5cu0440\x5cu0430\x5c\
u0431\x5cu043e\x5cu044\
2\x5cu043a\x5cu0430\x5cu0\
43f\x5cu0440\x5cu0435\x5c\
u0440\x5cu044b\x5cu043\
2\x5cu0430\x5cu043d\x5cu0\
438\x5cu044f\x5cu043f\x5c\
u043e\x5cu043b\x5cu044\
c\x5cu0437\x5cu043e\x5cu0\
432\x5cu0430\x5cu0442\x5c\
u0435\x5cu043b\x5cu044\
f \x5cu043e\x5cu0431\x5cu\
044a\x5cu0435\x5cu0434\
\x5cu0438\x5cu043d\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
0444\x5cu0430\x5cu0439\
\x5cu043b\x5cu044b \x5cu0\
43e\x5cu043a\x5cu0440 \
\x5cu043e\x5cu043f\x5cu04\
38\x5cu0441\x5cu0430\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu043e\x5cu0448\x5cu04\
38\x5cu0431\x5cu043a\x5cu\
0438 \x5cu043e\x5cu043\
f\x5cu043e\x5cu0432\x5cu0\
435\x5cu0441\x5cu0442\x5c\
u0438\x5cu0442\x5cu044\
c \x5cu043e\x5cu043f\x5cu\
043e\x5cu0432\x5cu0435\
\x5cu0441\x5cu0442\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
043e\x5cu0431\x5cu0438\
\x5cu0437\x5cu043c\x5cu04\
35\x5cu043d\x5cu0435\x5cu\
043d\x5cu0438\x5cu0438\
 \x5cu043e\x5cu0442\x5cu0\
43a\x5cu043b\x5cu044e\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu044c\x5cu043e\x5cu0\
431\x5cu0440\x5cu0430\x5c\
u0431\x5cu043e\x5cu044\
2\x5cu0447\x5cu0438\x5cu0\
43a\x5cu0437\x5cu0430\x5c\
u043f\x5cu0440\x5cu043\
e\x5cu0441\x5cu0430\x5cu0\
43d\x5cu0430\x5cu0441\x5c\
u0442\x5cu0440\x5cu043\
e\x5cu0435\x5cu043a\x5cu0\
43a\x5cu043b\x5cu0438\x5c\
u0435\x5cu043d\x5cu044\
2\x5cu0430\x5cu043b\x5cu0\
438\x5cu0446\x5cu0435\x5c\
u043d\x5cu0437\x5cu043\
8\x5cu0440\x5cu043e\x5cu0\
432\x5cu0430\x5cu043d\x5c\
u0438\x5cu044f \x5cu04\
3e\x5cu0442\x5cu043a\x5cu\
043b\x5cu044e\x5cu0447\
\x5cu0438\x5cu0442\x5cu04\
4c\x5cu043e\x5cu0431\x5cu\
0440\x5cu0430\x5cu0431\
\x5cu043e\x5cu0442\x5cu04\
47\x5cu0438\x5cu043a\x5cu\
043e\x5cu0436\x5cu0438\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu0438\x5cu044f \x5c\
u043e\x5cu0442\x5cu043\
a\x5cu043b\x5cu044e\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu043e\x5cu043\
1\x5cu0440\x5cu0430\x5cu0\
431\x5cu043e\x5cu0442\x5c\
u0447\x5cu0438\x5cu043\
a\x5cu043e\x5cu043f\x5cu0\
43e\x5cu0432\x5cu0435\x5c\
u0449\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f \x5cu\
043e\x5cu0442\x5cu043a\
\x5cu0440\x5cu044b\x5cu04\
42\x5cu044c\x5cu0437\x5cu\
043d\x5cu0430\x5cu0447\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435 \x5cu043e\x5c\
u0442\x5cu043a\x5cu044\
0\x5cu044b\x5cu0442\x5cu0\
44c\x5cu0438\x5cu043d\x5c\
u0434\x5cu0435\x5cu043\
a\x5cu0441\x5cu0441\x5cu0\
43f\x5cu0440\x5cu0430\x5c\
u0432\x5cu043a\x5cu043\
8 \x5cu043e\x5cu0442\x5cu\
043a\x5cu0440\x5cu044b\
\x5cu0442\x5cu044c\x5cu04\
41\x5cu043e\x5cu0434\x5cu\
0435\x5cu0440\x5cu0436\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu0435\x5cu0441\x5cu\
043f\x5cu0440\x5cu0430\
\x5cu0432\x5cu043a\x5cu04\
38 \x5cu043e\x5cu0442\x5c\
u043a\x5cu0440\x5cu044\
b\x5cu0442\x5cu044c\x5cu0\
441\x5cu043f\x5cu0440\x5c\
u0430\x5cu0432\x5cu043\
a\x5cu0443 \x5cu043e\x5cu\
0442\x5cu043a\x5cu0440\
\x5cu044b\x5cu0442\x5cu04\
4c\x5cu0444\x5cu043e\x5cu\
0440\x5cu043c\x5cu0443\
 \x5cu043e\x5cu0442\x5cu0\
43a\x5cu0440\x5cu044b\x5c\
u0442\x5cu044c\x5cu044\
4\x5cu043e\x5cu0440\x5cu0\
43c\x5cu0443\x5cu043c\x5c\
u043e\x5cu0434\x5cu043\
0\x5cu043b\x5cu044c\x5cu0\
43d\x5cu043e \x5cu043e\
\x5cu0442\x5cu043c\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0442\x5cu044c\x5cu0442\
\x5cu0440\x5cu0430\x5cu04\
3d\x5cu0437\x5cu0430\x5cu\
043a\x5cu0446\x5cu0438\
\x5cu044e \x5cu043e\x5cu0\
447\x5cu0438\x5cu0441\x5c\
u0442\x5cu0438\x5cu044\
2\x5cu044c\x5cu0436\x5cu0\
443\x5cu0440\x5cu043d\x5c\
u0430\x5cu043b\x5cu044\
0\x5cu0435\x5cu0433\x5cu0\
438\x5cu0441\x5cu0442\x5c\
u0440\x5cu0430\x5cu044\
6\x5cu0438\x5cu0438 \x5cu\
043e\x5cu0447\x5cu0438\
\x5cu0441\x5cu0442\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
043d\x5cu0430\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
3e\x5cu0439\x5cu043a\x5cu\
0438\x5cu043f\x5cu043e\
\x5cu043b\x5cu044c\x5cu04\
37\x5cu043e\x5cu0432\x5cu\
0430\x5cu0442\x5cu0435\
\x5cu043b\x5cu044f \x5cu0\
43e\x5cu0447\x5cu0438\x5c\
u0441\x5cu0442\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
441\x5cu043e\x5cu043e\x5c\
u0431\x5cu0449\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f \x5cu043f\x5cu0430\
\x5cu0440\x5cu0430\x5cu04\
3c\x5cu0435\x5cu0442\x5cu\
0440\x5cu044b\x5cu0434\
\x5cu043e\x5cu0441\x5cu04\
42\x5cu0443\x5cu043f\x5cu\
0430 \x5cu043f\x5cu043\
5\x5cu0440\x5cu0435\x5cu0\
439\x5cu0442\x5cu0438\x5c\
u043f\x5cu043e\x5cu043\
d\x5cu0430\x5cu0432\x5cu0\
438\x5cu0433\x5cu0430\x5c\
u0446\x5cu0438\x5cu043\
e\x5cu043d\x5cu043d\x5cu0\
43e\x5cu0439\x5cu0441\x5c\
u0441\x5cu044b\x5cu043\
b\x5cu043a\x5cu0435 \x5cu\
043f\x5cu0435\x5cu0440\
\x5cu0435\x5cu043c\x5cu04\
35\x5cu0441\x5cu0442\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu0444\x5cu0430\x5cu04\
39\x5cu043b \x5cu043f\x5c\
u043e\x5cu0434\x5cu043\
a\x5cu043b\x5cu044e\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu0432\x5cu043\
d\x5cu0435\x5cu0448\x5cu0\
43d\x5cu044e\x5cu044e\x5c\
u043a\x5cu043e\x5cu043\
c\x5cu043f\x5cu043e\x5cu0\
43d\x5cu0435\x5cu043d\x5c\
u0442\x5cu0443 \x5cu04\
3f\x5cu043e\x5cu0434\x5cu\
043a\x5cu043b\x5cu044e\
\x5cu0447\x5cu0438\x5cu04\
42\x5cu044c\x5cu043e\x5cu\
0431\x5cu0440\x5cu0430\
\x5cu0431\x5cu043e\x5cu04\
42\x5cu0447\x5cu0438\x5cu\
043a\x5cu0437\x5cu0430\
\x5cu043f\x5cu0440\x5cu04\
3e\x5cu0441\x5cu0430\x5cu\
043d\x5cu0430\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
3e\x5cu0435\x5cu043a\x5cu\
043a\x5cu043b\x5cu0438\
\x5cu0435\x5cu043d\x5cu04\
42\x5cu0430\x5cu043b\x5cu\
0438\x5cu0446\x5cu0435\
\x5cu043d\x5cu0437\x5cu04\
38\x5cu0440\x5cu043e\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu0438\x5cu044f \x5cu0\
43f\x5cu043e\x5cu0434\x5c\
u043a\x5cu043b\x5cu044\
e\x5cu0447\x5cu0438\x5cu0\
442\x5cu044c\x5cu043e\x5c\
u0431\x5cu0440\x5cu043\
0\x5cu0431\x5cu043e\x5cu0\
442\x5cu0447\x5cu0438\x5c\
u043a\x5cu043e\x5cu043\
6\x5cu0438\x5cu0434\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u044f \x5cu043f\x5cu04\
3e\x5cu0434\x5cu043a\x5cu\
043b\x5cu044e\x5cu0447\
\x5cu0438\x5cu0442\x5cu04\
4c\x5cu043e\x5cu0431\x5cu\
0440\x5cu0430\x5cu0431\
\x5cu043e\x5cu0442\x5cu04\
47\x5cu0438\x5cu043a\x5cu\
043e\x5cu043f\x5cu043e\
\x5cu0432\x5cu0435\x5cu04\
49\x5cu0435\x5cu043d\x5cu\
0438\x5cu044f \x5cu043\
f\x5cu043e\x5cu0434\x5cu0\
43a\x5cu043b\x5cu044e\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu044c\x5cu0440\x5cu0\
430\x5cu0441\x5cu0448\x5c\
u0438\x5cu0440\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu0440\x5cu0430\x5c\
u0431\x5cu043e\x5cu044\
2\x5cu044b\x5cu0441\x5cu0\
43a\x5cu0440\x5cu0438\x5c\
u043f\x5cu0442\x5cu043\
e\x5cu0433\x5cu0440\x5cu0\
430\x5cu0444\x5cu0438\x5c\
u0435\x5cu0439 \x5cu04\
3f\x5cu043e\x5cu0434\x5cu\
043a\x5cu043b\x5cu044e\
\x5cu0447\x5cu0438\x5cu04\
42\x5cu044c\x5cu0440\x5cu\
0430\x5cu0441\x5cu0448\
\x5cu0438\x5cu0440\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu0440\x5cu0430\
\x5cu0431\x5cu043e\x5cu04\
42\x5cu044b\x5cu0441\x5cu\
0444\x5cu0430\x5cu0439\
\x5cu043b\x5cu0430\x5cu04\
3c\x5cu0438 \x5cu043f\x5c\
u043e\x5cu0434\x5cu044\
0\x5cu043e\x5cu0431\x5cu0\
43d\x5cu043e\x5cu0435\x5c\
u043f\x5cu0440\x5cu043\
5\x5cu0434\x5cu0441\x5cu0\
442\x5cu0430\x5cu0432\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
43e\x5cu0448\x5cu0438\x5c\
u0431\x5cu043a\x5cu043\
8 \x5cu043f\x5cu043e\x5cu\
043a\x5cu0430\x5cu0437\
\x5cu0430\x5cu0442\x5cu04\
4c\x5cu0432\x5cu0432\x5cu\
043e\x5cu0434\x5cu0434\
\x5cu0430\x5cu0442\x5cu04\
4b \x5cu043f\x5cu043e\x5c\
u043a\x5cu0430\x5cu043\
7\x5cu0430\x5cu0442\x5cu0\
44c\x5cu0432\x5cu0432\x5c\
u043e\x5cu0434\x5cu043\
7\x5cu043d\x5cu0430\x5cu0\
447\x5cu0435\x5cu043d\x5c\
u0438\x5cu044f \x5cu04\
3f\x5cu043e\x5cu043a\x5cu\
0430\x5cu0437\x5cu0430\
\x5cu0442\x5cu044c\x5cu04\
32\x5cu0432\x5cu043e\x5cu\
0434\x5cu0441\x5cu0442\
\x5cu0440\x5cu043e\x5cu04\
3a\x5cu0438 \x5cu043f\x5c\
u043e\x5cu043a\x5cu043\
0\x5cu0437\x5cu0430\x5cu0\
442\x5cu044c\x5cu0432\x5c\
u0432\x5cu043e\x5cu043\
4\x5cu0447\x5cu0438\x5cu0\
441\x5cu043b\x5cu0430 \
\x5cu043f\x5cu043e\x5cu04\
3a\x5cu0430\x5cu0437\x5cu\
0430\x5cu0442\x5cu044c\
\x5cu0432\x5cu043e\x5cu04\
3f\x5cu0440\x5cu043e\x5cu\
0441 \x5cu043f\x5cu043\
e\x5cu043a\x5cu0430\x5cu0\
437\x5cu0430\x5cu0442\x5c\
u044c\x5cu0437\x5cu043\
d\x5cu0430\x5cu0447\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435 \x5cu043f\x5cu04\
3e\x5cu043a\x5cu0430\x5cu\
0437\x5cu0430\x5cu0442\
\x5cu044c\x5cu0438\x5cu04\
3d\x5cu0444\x5cu043e\x5cu\
0440\x5cu043c\x5cu0430\
\x5cu0446\x5cu0438\x5cu04\
4e\x5cu043e\x5cu0431\x5cu\
043e\x5cu0448\x5cu0438\
\x5cu0431\x5cu043a\x5cu04\
35 \x5cu043f\x5cu043e\x5c\
u043a\x5cu0430\x5cu043\
7\x5cu0430\x5cu0442\x5cu0\
44c\x5cu043d\x5cu0430\x5c\
u043a\x5cu0430\x5cu044\
0\x5cu0442\x5cu0435 \x5cu\
043f\x5cu043e\x5cu043a\
\x5cu0430\x5cu0437\x5cu04\
30\x5cu0442\x5cu044c\x5cu\
043e\x5cu043f\x5cu043e\
\x5cu0432\x5cu0435\x5cu04\
49\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
4c\x5cu0437\x5cu043e\x5cu\
0432\x5cu0430\x5cu0442\
\x5cu0435\x5cu043b\x5cu04\
4f \x5cu043f\x5cu043e\x5c\
u043a\x5cu0430\x5cu043\
7\x5cu0430\x5cu0442\x5cu0\
44c\x5cu043f\x5cu0440\x5c\
u0435\x5cu0434\x5cu044\
3\x5cu043f\x5cu0440\x5cu0\
435\x5cu0436\x5cu0434\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435 \x5cu043f\x5cu\
043e\x5cu043b\x5cu043d\
\x5cu043e\x5cu0435\x5cu04\
38\x5cu043c\x5cu044f\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu044c\x5cu0437\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
0442\x5cu0435\x5cu043b\
\x5cu044f \x5cu043f\x5cu0\
43e\x5cu043b\x5cu0443\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu044ccom\x5cu043e\
\x5cu0431\x5cu044a\x5cu04\
35\x5cu043a\x5cu0442 \x5c\
u043f\x5cu043e\x5cu043\
b\x5cu0443\x5cu0447\x5cu0\
438\x5cu0442\x5cu044cx\
ml\x5cu0442\x5cu0438\x5cu\
043f \x5cu043f\x5cu043\
e\x5cu043b\x5cu0443\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu0430\x5cu043\
4\x5cu0440\x5cu0435\x5cu0\
441\x5cu043f\x5cu043e\x5c\
u043c\x5cu0435\x5cu044\
1\x5cu0442\x5cu043e\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u043e\x5cu0436\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44e \x5cu043f\x5cu043e\
\x5cu043b\x5cu0443\x5cu04\
47\x5cu0438\x5cu0442\x5cu\
044c\x5cu0431\x5cu043b\
\x5cu043e\x5cu043a\x5cu04\
38\x5cu0440\x5cu043e\x5cu\
0432\x5cu043a\x5cu0443\
\x5cu0441\x5cu0435\x5cu04\
30\x5cu043d\x5cu0441\x5cu\
043e\x5cu0432 \x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
443\x5cu0447\x5cu0438\x5c\
u0442\x5cu044c\x5cu043\
2\x5cu0440\x5cu0435\x5cu0\
43c\x5cu044f\x5cu0437\x5c\
u0430\x5cu0432\x5cu043\
5\x5cu0440\x5cu0448\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044f\x5cu0441\x5cu043\
f\x5cu044f\x5cu0449\x5cu0\
435\x5cu0433\x5cu043e\x5c\
u0441\x5cu0435\x5cu043\
0\x5cu043d\x5cu0441\x5cu0\
430 \x5cu043f\x5cu043e\
\x5cu043b\x5cu0443\x5cu04\
47\x5cu0438\x5cu0442\x5cu\
044c\x5cu0432\x5cu0440\
\x5cu0435\x5cu043c\x5cu04\
4f\x5cu0437\x5cu0430\x5cu\
0441\x5cu044b\x5cu043f\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu044f\x5cu043f\x5cu\
0430\x5cu0441\x5cu0441\
\x5cu0438\x5cu0432\x5cu04\
3d\x5cu043e\x5cu0433\x5cu\
043e\x5cu0441\x5cu0435\
\x5cu0430\x5cu043d\x5cu04\
41\x5cu0430 \x5cu043f\x5c\
u043e\x5cu043b\x5cu044\
3\x5cu0447\x5cu0438\x5cu0\
442\x5cu044c\x5cu0432\x5c\
u0440\x5cu0435\x5cu043\
c\x5cu044f\x5cu043e\x5cu0\
436\x5cu0438\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu044f\x5cu0431\x5cu0\
43b\x5cu043e\x5cu043a\x5c\
u0438\x5cu0440\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
438\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 \x5cu043f\x5cu\
043e\x5cu043b\x5cu0443\
\x5cu0447\x5cu0438\x5cu04\
42\x5cu044c\x5cu0434\x5cu\
0430\x5cu043d\x5cu043d\
\x5cu044b\x5cu0435\x5cu04\
32\x5cu044b\x5cu0431\x5cu\
043e\x5cu0440\x5cu0430\
 \x5cu043f\x5cu043e\x5cu0\
43b\x5cu0443\x5cu0447\x5c\
u0438\x5cu0442\x5cu044\
c\x5cu0434\x5cu043e\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u043d\x5cu0438\x5cu044\
2\x5cu0435\x5cu043b\x5cu0\
44c\x5cu043d\x5cu044b\x5c\
u0439\x5cu043f\x5cu043\
0\x5cu0440\x5cu0430\x5cu0\
43c\x5cu0435\x5cu0442\x5c\
u0440\x5cu043a\x5cu043\
b\x5cu0438\x5cu0435\x5cu0\
43d\x5cu0442\x5cu0430\x5c\
u043b\x5cu0438\x5cu044\
6\x5cu0435\x5cu043d\x5cu0\
437\x5cu0438\x5cu0440\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
44f \x5cu043f\x5cu043e\
\x5cu043b\x5cu0443\x5cu04\
47\x5cu0438\x5cu0442\x5cu\
044c\x5cu0434\x5cu043e\
\x5cu043f\x5cu0443\x5cu04\
41\x5cu0442\x5cu0438\x5cu\
043c\x5cu044b\x5cu0435\
\x5cu043a\x5cu043e\x5cu04\
34\x5cu044b\x5cu043b\x5cu\
043e\x5cu043a\x5cu0430\
\x5cu043b\x5cu0438\x5cu04\
37\x5cu0430\x5cu0446\x5cu\
0438\x5cu0438 \x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
443\x5cu0447\x5cu0438\x5c\
u0442\x5cu044c\x5cu043\
4\x5cu043e\x5cu043f\x5cu0\
443\x5cu0441\x5cu0442\x5c\
u0438\x5cu043c\x5cu044\
b\x5cu0435\x5cu0447\x5cu0\
430\x5cu0441\x5cu043e\x5c\
u0432\x5cu044b\x5cu043\
5\x5cu043f\x5cu043e\x5cu0\
44f\x5cu0441\x5cu0430 \
\x5cu043f\x5cu043e\x5cu04\
3b\x5cu0443\x5cu0447\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu0437\x5cu0430\x5cu04\
33\x5cu043e\x5cu043b\x5cu\
043e\x5cu0432\x5cu043e\
\x5cu043a\x5cu043a\x5cu04\
3b\x5cu0438\x5cu0435\x5cu\
043d\x5cu0442\x5cu0441\
\x5cu043a\x5cu043e\x5cu04\
33\x5cu043e\x5cu043f\x5cu\
0440\x5cu0438\x5cu043b\
\x5cu043e\x5cu0436\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f \x5cu043f\x5cu043\
e\x5cu043b\x5cu0443\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu0437\x5cu043\
0\x5cu0433\x5cu043e\x5cu0\
43b\x5cu043e\x5cu0432\x5c\
u043e\x5cu043a\x5cu044\
1\x5cu0438\x5cu0441\x5cu0\
442\x5cu0435\x5cu043c\x5c\
u044b \x5cu043f\x5cu04\
3e\x5cu043b\x5cu0443\x5cu\
0447\x5cu0438\x5cu0442\
\x5cu044c\x5cu0437\x5cu04\
3d\x5cu0430\x5cu0447\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu043e\x5cu04\
42\x5cu0431\x5cu043e\x5cu\
0440\x5cu0430\x5cu0436\
\x5cu0443\x5cu0440\x5cu04\
3d\x5cu0430\x5cu043b\x5cu\
0430\x5cu0440\x5cu0435\
\x5cu0433\x5cu0438\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
0430\x5cu0446\x5cu0438\
\x5cu0438 \x5cu043f\x5cu0\
43e\x5cu043b\x5cu0443\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu044c\x5cu0438\x5cu0\
434\x5cu0435\x5cu043d\x5c\
u0442\x5cu0438\x5cu044\
4\x5cu0438\x5cu043a\x5cu0\
430\x5cu0442\x5cu043e\x5c\
u0440\x5cu043a\x5cu043\
e\x5cu043d\x5cu0444\x5cu0\
438\x5cu0433\x5cu0443\x5c\
u0440\x5cu0430\x5cu044\
6\x5cu0438\x5cu0438 \x5cu\
043f\x5cu043e\x5cu043b\
\x5cu0443\x5cu0447\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
0438\x5cu0437\x5cu0432\
\x5cu0440\x5cu0435\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
043d\x5cu043e\x5cu0433\
\x5cu043e\x5cu0445\x5cu04\
40\x5cu0430\x5cu043d\x5cu\
0438\x5cu043b\x5cu0438\
\x5cu0449\x5cu0430 \x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u0443\x5cu0447\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
438\x5cu043c\x5cu044f\x5c\
u0432\x5cu0440\x5cu043\
5\x5cu043c\x5cu0435\x5cu0\
43d\x5cu043d\x5cu043e\x5c\
u0433\x5cu043e\x5cu044\
4\x5cu0430\x5cu0439\x5cu0\
43b\x5cu0430 \x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
43\x5cu0447\x5cu0438\x5cu\
0442\x5cu044c\x5cu0438\
\x5cu043c\x5cu044f\x5cu04\
3a\x5cu043b\x5cu0438\x5cu\
0435\x5cu043d\x5cu0442\
\x5cu0430\x5cu043b\x5cu04\
38\x5cu0446\x5cu0435\x5cu\
043d\x5cu0437\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0430\x5cu043d\x5cu\
0438\x5cu044f \x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
443\x5cu0447\x5cu0438\x5c\
u0442\x5cu044c\x5cu043\
8\x5cu043d\x5cu0444\x5cu0\
43e\x5cu0440\x5cu043c\x5c\
u0430\x5cu0446\x5cu043\
8\x5cu044e\x5cu044d\x5cu0\
43a\x5cu0440\x5cu0430\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu043a\x5cu043b\x5cu0\
438\x5cu0435\x5cu043d\x5c\
u0442\x5cu0430 \x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
0443\x5cu0447\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
38\x5cu0441\x5cu043f\x5cu\
043e\x5cu043b\x5cu044c\
\x5cu0437\x5cu043e\x5cu04\
32\x5cu0430\x5cu043d\x5cu\
0438\x5cu0435\x5cu0436\
\x5cu0443\x5cu0440\x5cu04\
3d\x5cu0430\x5cu043b\x5cu\
0430\x5cu0440\x5cu0435\
\x5cu0433\x5cu0438\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
0430\x5cu0446\x5cu0438\
\x5cu0438 \x5cu043f\x5cu0\
43e\x5cu043b\x5cu0443\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu044c\x5cu0438\x5cu0\
441\x5cu043f\x5cu043e\x5c\
u043b\x5cu044c\x5cu043\
7\x5cu043e\x5cu0432\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u0435\x5cu0441\x5cu043\
e\x5cu0431\x5cu044b\x5cu0\
442\x5cu0438\x5cu044f\x5c\
u0436\x5cu0443\x5cu044\
0\x5cu043d\x5cu0430\x5cu0\
43b\x5cu0430\x5cu0440\x5c\
u0435\x5cu0433\x5cu043\
8\x5cu0441\x5cu0442\x5cu0\
440\x5cu0430\x5cu0446\x5c\
u0438\x5cu0438 \x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
0443\x5cu0447\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
3a\x5cu0440\x5cu0430\x5cu\
0442\x5cu043a\x5cu0438\
\x5cu0439\x5cu0437\x5cu04\
30\x5cu0433\x5cu043e\x5cu\
043b\x5cu043e\x5cu0432\
\x5cu043e\x5cu043a\x5cu04\
3f\x5cu0440\x5cu0438\x5cu\
043b\x5cu043e\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f \x5cu043f\x5c\
u043e\x5cu043b\x5cu044\
3\x5cu0447\x5cu0438\x5cu0\
442\x5cu044c\x5cu043c\x5c\
u0430\x5cu043a\x5cu043\
5\x5cu0442\x5cu043e\x5cu0\
444\x5cu043e\x5cu0440\x5c\
u043c\x5cu043b\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f \x5cu043f\x5cu043e\
\x5cu043b\x5cu0443\x5cu04\
47\x5cu0438\x5cu0442\x5cu\
044c\x5cu043c\x5cu0430\
\x5cu0441\x5cu043a\x5cu04\
43\x5cu0432\x5cu0441\x5cu\
0435\x5cu0444\x5cu0430\
\x5cu0439\x5cu043b\x5cu04\
4b \x5cu043f\x5cu043e\x5c\
u043b\x5cu0443\x5cu044\
7\x5cu0438\x5cu0442\x5cu0\
44c\x5cu043c\x5cu0430\x5c\
u0441\x5cu043a\x5cu044\
3\x5cu0432\x5cu0441\x5cu0\
435\x5cu0444\x5cu0430\x5c\
u0439\x5cu043b\x5cu044\
b\x5cu043a\x5cu043b\x5cu0\
438\x5cu0435\x5cu043d\x5c\
u0442\x5cu0430 \x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
0443\x5cu0447\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
3c\x5cu0430\x5cu0441\x5cu\
043a\x5cu0443\x5cu0432\
\x5cu0441\x5cu0435\x5cu04\
44\x5cu0430\x5cu0439\x5cu\
043b\x5cu044b\x5cu0441\
\x5cu0435\x5cu0440\x5cu04\
32\x5cu0435\x5cu0440\x5cu\
0430 \x5cu043f\x5cu043\
e\x5cu043b\x5cu0443\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu043c\x5cu043\
5\x5cu0441\x5cu0442\x5cu0\
43e\x5cu043f\x5cu043e\x5c\
u043b\x5cu043e\x5cu043\
6\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu043f\x5c\
u043e\x5cu0430\x5cu043\
4\x5cu0440\x5cu0435\x5cu0\
441\x5cu0443 \x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
43\x5cu0447\x5cu0438\x5cu\
0442\x5cu044c\x5cu043c\
\x5cu0438\x5cu043d\x5cu04\
38\x5cu043c\x5cu0430\x5cu\
043b\x5cu044c\x5cu043d\
\x5cu0443\x5cu044e\x5cu04\
34\x5cu043b\x5cu0438\x5cu\
043d\x5cu0443\x5cu043f\
\x5cu0430\x5cu0440\x5cu04\
3e\x5cu043b\x5cu0435\x5cu\
0439\x5cu043f\x5cu043e\
\x5cu043b\x5cu044c\x5cu04\
37\x5cu043e\x5cu0432\x5cu\
0430\x5cu0442\x5cu0435\
\x5cu043b\x5cu0435\x5cu04\
39 \x5cu043f\x5cu043e\x5c\
u043b\x5cu0443\x5cu044\
7\x5cu0438\x5cu0442\x5cu0\
44c\x5cu043d\x5cu0430\x5c\
u0432\x5cu0438\x5cu043\
3\x5cu0430\x5cu0446\x5cu0\
438\x5cu043e\x5cu043d\x5c\
u043d\x5cu0443\x5cu044\
e\x5cu0441\x5cu0441\x5cu0\
44b\x5cu043b\x5cu043a\x5c\
u0443 \x5cu043f\x5cu04\
3e\x5cu043b\x5cu0443\x5cu\
0447\x5cu0438\x5cu0442\
\x5cu044c\x5cu043d\x5cu04\
30\x5cu0432\x5cu0438\x5cu\
0433\x5cu0430\x5cu0446\
\x5cu0438\x5cu043e\x5cu04\
3d\x5cu043d\x5cu0443\x5cu\
044e\x5cu0441\x5cu0441\
\x5cu044b\x5cu043b\x5cu04\
3a\x5cu0443\x5cu0438\x5cu\
043d\x5cu0444\x5cu043e\
\x5cu0440\x5cu043c\x5cu04\
30\x5cu0446\x5cu0438\x5cu\
043e\x5cu043d\x5cu043d\
\x5cu043e\x5cu0439\x5cu04\
31\x5cu0430\x5cu0437\x5cu\
044b \x5cu043f\x5cu043\
e\x5cu043b\x5cu0443\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu043e\x5cu043\
1\x5cu043d\x5cu043e\x5cu0\
432\x5cu043b\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu043a\x5cu043e\x5cu0\
43d\x5cu0444\x5cu0438\x5c\
u0433\x5cu0443\x5cu044\
0\x5cu0430\x5cu0446\x5cu0\
438\x5cu0438\x5cu0431\x5c\
u0430\x5cu0437\x5cu044\
b\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu043f\x5cu04\
3e\x5cu043b\x5cu0443\x5cu\
0447\x5cu0438\x5cu0442\
\x5cu044c\x5cu043e\x5cu04\
31\x5cu043d\x5cu043e\x5cu\
0432\x5cu043b\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu043f\x5cu0440\x5cu\
0435\x5cu0434\x5cu043e\
\x5cu043f\x5cu0440\x5cu04\
35\x5cu0434\x5cu0435\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
45\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445\x5cu0438\x5cu04\
3d\x5cu0444\x5cu043e\x5cu\
0440\x5cu043c\x5cu0430\
\x5cu0446\x5cu0438\x5cu04\
3e\x5cu043d\x5cu043d\x5cu\
043e\x5cu0439\x5cu0431\
\x5cu0430\x5cu0437\x5cu04\
4b \x5cu043f\x5cu043e\x5c\
u043b\x5cu0443\x5cu044\
7\x5cu0438\x5cu0442\x5cu0\
44c\x5cu043e\x5cu0431\x5c\
u0449\x5cu0438\x5cu043\
9\x5cu043c\x5cu0430\x5cu0\
43a\x5cu0435\x5cu0442 \
\x5cu043f\x5cu043e\x5cu04\
3b\x5cu0443\x5cu0447\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu043e\x5cu0431\x5cu04\
49\x5cu0443\x5cu044e\x5cu\
0444\x5cu043e\x5cu0440\
\x5cu043c\x5cu0443 \x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u0443\x5cu0447\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
43e\x5cu043a\x5cu043d\x5c\
u0430 \x5cu043f\x5cu04\
3e\x5cu043b\x5cu0443\x5cu\
0447\x5cu0438\x5cu0442\
\x5cu044c\x5cu043e\x5cu04\
3f\x5cu0435\x5cu0440\x5cu\
0430\x5cu0442\x5cu0438\
\x5cu0432\x5cu043d\x5cu04\
43\x5cu044e\x5cu043e\x5cu\
0442\x5cu043c\x5cu0435\
\x5cu0442\x5cu043a\x5cu04\
43\x5cu0432\x5cu0440\x5cu\
0435\x5cu043c\x5cu0435\
\x5cu043d\x5cu0438 \x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u0443\x5cu0447\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
43e\x5cu0442\x5cu043a\x5c\
u043b\x5cu044e\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu0431\x5c\
u0435\x5cu0437\x5cu043\
e\x5cu043f\x5cu0430\x5cu0\
441\x5cu043d\x5cu043e\x5c\
u0433\x5cu043e\x5cu044\
0\x5cu0435\x5cu0436\x5cu0\
438\x5cu043c\x5cu0430 \
\x5cu043f\x5cu043e\x5cu04\
3b\x5cu0443\x5cu0447\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu043f\x5cu0430\x5cu04\
40\x5cu0430\x5cu043c\x5cu\
0435\x5cu0442\x5cu0440\
\x5cu044b\x5cu0444\x5cu04\
43\x5cu043d\x5cu043a\x5cu\
0446\x5cu0438\x5cu043e\
\x5cu043d\x5cu0430\x5cu04\
3b\x5cu044c\x5cu043d\x5cu\
044b\x5cu0445\x5cu043e\
\x5cu043f\x5cu0446\x5cu04\
38\x5cu0439\x5cu0438\x5cu\
043d\x5cu0442\x5cu0435\
\x5cu0440\x5cu0444\x5cu04\
35\x5cu0439\x5cu0441\x5cu\
0430 \x5cu043f\x5cu043\
e\x5cu043b\x5cu0443\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu043f\x5cu043\
e\x5cu043b\x5cu043d\x5cu0\
43e\x5cu0435\x5cu0438\x5c\
u043c\x5cu044f\x5cu043\
f\x5cu0440\x5cu0435\x5cu0\
434\x5cu043e\x5cu043f\x5c\
u0440\x5cu0435\x5cu043\
4\x5cu0435\x5cu043b\x5cu0\
435\x5cu043d\x5cu043d\x5c\
u043e\x5cu0433\x5cu043\
e\x5cu0437\x5cu043d\x5cu0\
430\x5cu0447\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f \x5cu043f\x5cu043e\x5cu\
043b\x5cu0443\x5cu0447\
\x5cu0438\x5cu0442\x5cu04\
4c\x5cu043f\x5cu0440\x5cu\
0435\x5cu0434\x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
32\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu043d\x5cu0430\x5cu04\
32\x5cu0438\x5cu0433\x5cu\
0430\x5cu0446\x5cu0438\
\x5cu043e\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0445\x5cu\
0441\x5cu0441\x5cu044b\
\x5cu043b\x5cu043e\x5cu04\
3a \x5cu043f\x5cu043e\x5c\
u043b\x5cu0443\x5cu044\
7\x5cu0438\x5cu0442\x5cu0\
44c\x5cu043f\x5cu0440\x5c\
u043e\x5cu0432\x5cu043\
5\x5cu0440\x5cu043a\x5cu0\
443\x5cu0441\x5cu043b\x5c\
u043e\x5cu0436\x5cu043\
d\x5cu043e\x5cu0441\x5cu0\
442\x5cu0438\x5cu043f\x5c\
u0430\x5cu0440\x5cu043\
e\x5cu043b\x5cu0435\x5cu0\
439\x5cu043f\x5cu043e\x5c\
u043b\x5cu044c\x5cu043\
7\x5cu043e\x5cu0432\x5cu0\
430\x5cu0442\x5cu0435\x5c\
u043b\x5cu0435\x5cu043\
9 \x5cu043f\x5cu043e\x5cu\
043b\x5cu0443\x5cu0447\
\x5cu0438\x5cu0442\x5cu04\
4c\x5cu0440\x5cu0430\x5cu\
0437\x5cu0434\x5cu0435\
\x5cu043b\x5cu0438\x5cu04\
42\x5cu0435\x5cu043b\x5cu\
044c\x5cu043f\x5cu0443\
\x5cu0442\x5cu0438 \x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u0443\x5cu0447\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
440\x5cu0430\x5cu0437\x5c\
u0434\x5cu0435\x5cu043\
b\x5cu0438\x5cu0442\x5cu0\
435\x5cu043b\x5cu044c\x5c\
u043f\x5cu0443\x5cu044\
2\x5cu0438\x5cu043a\x5cu0\
43b\x5cu0438\x5cu0435\x5c\
u043d\x5cu0442\x5cu043\
0 \x5cu043f\x5cu043e\x5cu\
043b\x5cu0443\x5cu0447\
\x5cu0438\x5cu0442\x5cu04\
4c\x5cu0440\x5cu0430\x5cu\
0437\x5cu0434\x5cu0435\
\x5cu043b\x5cu0438\x5cu04\
42\x5cu0435\x5cu043b\x5cu\
044c\x5cu043f\x5cu0443\
\x5cu0442\x5cu0438\x5cu04\
41\x5cu0435\x5cu0440\x5cu\
0432\x5cu0435\x5cu0440\
\x5cu0430 \x5cu043f\x5cu0\
43e\x5cu043b\x5cu0443\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu044c\x5cu0441\x5cu0\
435\x5cu0430\x5cu043d\x5c\
u0441\x5cu044b\x5cu043\
8\x5cu043d\x5cu0444\x5cu0\
43e\x5cu0440\x5cu043c\x5c\
u0430\x5cu0446\x5cu043\
8\x5cu043e\x5cu043d\x5cu0\
43d\x5cu043e\x5cu0439\x5c\
u0431\x5cu0430\x5cu043\
7\x5cu044b \x5cu043f\x5cu\
043e\x5cu043b\x5cu0443\
\x5cu0447\x5cu0438\x5cu04\
42\x5cu044c\x5cu0441\x5cu\
043a\x5cu043e\x5cu0440\
\x5cu043e\x5cu0441\x5cu04\
42\x5cu044c\x5cu043a\x5cu\
043b\x5cu0438\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
41\x5cu043a\x5cu043e\x5cu\
0433\x5cu043e\x5cu0441\
\x5cu043e\x5cu0435\x5cu04\
34\x5cu0438\x5cu043d\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f \x5cu043f\x5cu0\
43e\x5cu043b\x5cu0443\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu044c\x5cu0441\x5cu0\
43e\x5cu0435\x5cu0434\x5c\
u0438\x5cu043d\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f\x5cu0438\x5cu043d\x5c\
u0444\x5cu043e\x5cu044\
0\x5cu043c\x5cu0430\x5cu0\
446\x5cu0438\x5cu043e\x5c\
u043d\x5cu043d\x5cu043\
e\x5cu0439\x5cu0431\x5cu0\
430\x5cu0437\x5cu044b \
\x5cu043f\x5cu043e\x5cu04\
3b\x5cu0443\x5cu0447\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu0441\x5cu043e\x5cu04\
3e\x5cu0431\x5cu0449\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu043f\x5cu04\
3e\x5cu043b\x5cu044c\x5cu\
0437\x5cu043e\x5cu0432\
\x5cu0430\x5cu0442\x5cu04\
35\x5cu043b\x5cu044e \x5c\
u043f\x5cu043e\x5cu043\
b\x5cu0443\x5cu0447\x5cu0\
438\x5cu0442\x5cu044c\x5c\
u0441\x5cu043e\x5cu043\
e\x5cu0442\x5cu0432\x5cu0\
435\x5cu0442\x5cu0441\x5c\
u0442\x5cu0432\x5cu043\
8\x5cu0435\x5cu043e\x5cu0\
431\x5cu044a\x5cu0435\x5c\
u043a\x5cu0442\x5cu043\
0\x5cu0438\x5cu0444\x5cu0\
43e\x5cu0440\x5cu043c\x5c\
u044b \x5cu043f\x5cu04\
3e\x5cu043b\x5cu0443\x5cu\
0447\x5cu0438\x5cu0442\
\x5cu044c\x5cu0441\x5cu04\
3e\x5cu0441\x5cu0442\x5cu\
0430\x5cu0432\x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
3d\x5cu0434\x5cu0430\x5cu\
0440\x5cu0442\x5cu043d\
\x5cu043e\x5cu0433\x5cu04\
3e\x5cu0438\x5cu043d\x5cu\
0442\x5cu0435\x5cu0440\
\x5cu0444\x5cu0435\x5cu04\
39\x5cu0441\x5cu0430od\
ata \x5cu043f\x5cu043e\
\x5cu043b\x5cu0443\x5cu04\
47\x5cu0438\x5cu0442\x5cu\
044c\x5cu0441\x5cu0442\
\x5cu0440\x5cu0443\x5cu04\
3a\x5cu0442\x5cu0443\x5cu\
0440\x5cu0443\x5cu0445\
\x5cu0440\x5cu0430\x5cu04\
3d\x5cu0435\x5cu043d\x5cu\
0438\x5cu044f\x5cu0431\
\x5cu0430\x5cu0437\x5cu04\
4b\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445 \x5cu043f\x5cu0\
43e\x5cu043b\x5cu0443\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu044c\x5cu0442\x5cu0\
435\x5cu043a\x5cu0443\x5c\
u0449\x5cu0438\x5cu043\
9\x5cu0441\x5cu0435\x5cu0\
430\x5cu043d\x5cu0441\x5c\
u0438\x5cu043d\x5cu044\
4\x5cu043e\x5cu0440\x5cu0\
43c\x5cu0430\x5cu0446\x5c\
u0438\x5cu043e\x5cu043\
d\x5cu043d\x5cu043e\x5cu0\
439\x5cu0431\x5cu0430\x5c\
u0437\x5cu044b \x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
0443\x5cu0447\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
44\x5cu0430\x5cu0439\x5cu\
043b \x5cu043f\x5cu043\
e\x5cu043b\x5cu0443\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu0444\x5cu043\
0\x5cu0439\x5cu043b\x5cu0\
44b \x5cu043f\x5cu043e\
\x5cu043b\x5cu0443\x5cu04\
47\x5cu0438\x5cu0442\x5cu\
044c\x5cu0444\x5cu043e\
\x5cu0440\x5cu043c\x5cu04\
43 \x5cu043f\x5cu043e\x5c\
u043b\x5cu0443\x5cu044\
7\x5cu0438\x5cu0442\x5cu0\
44c\x5cu0444\x5cu0443\x5c\
u043d\x5cu043a\x5cu044\
6\x5cu0438\x5cu043e\x5cu0\
43d\x5cu0430\x5cu043b\x5c\
u044c\x5cu043d\x5cu044\
3\x5cu044e\x5cu043e\x5cu0\
43f\x5cu0446\x5cu0438\x5c\
u044e \x5cu043f\x5cu04\
3e\x5cu043b\x5cu0443\x5cu\
0447\x5cu0438\x5cu0442\
\x5cu044c\x5cu0444\x5cu04\
43\x5cu043d\x5cu043a\x5cu\
0446\x5cu0438\x5cu043e\
\x5cu043d\x5cu0430\x5cu04\
3b\x5cu044c\x5cu043d\x5cu\
0443\x5cu044e\x5cu043e\
\x5cu043f\x5cu0446\x5cu04\
38\x5cu044e\x5cu0438\x5cu\
043d\x5cu0442\x5cu0435\
\x5cu0440\x5cu0444\x5cu04\
35\x5cu0439\x5cu0441\x5cu\
0430 \x5cu043f\x5cu043\
e\x5cu043b\x5cu0443\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u044c\x5cu0447\x5cu043\
0\x5cu0441\x5cu043e\x5cu0\
432\x5cu043e\x5cu0439\x5c\
u043f\x5cu043e\x5cu044\
f\x5cu0441\x5cu0438\x5cu0\
43d\x5cu0444\x5cu043e\x5c\
u0440\x5cu043c\x5cu043\
0\x5cu0446\x5cu0438\x5cu0\
43e\x5cu043d\x5cu043d\x5c\
u043e\x5cu0439\x5cu043\
1\x5cu0430\x5cu0437\x5cu0\
44b \x5cu043f\x5cu043e\
\x5cu043b\x5cu044c\x5cu04\
37\x5cu043e\x5cu0432\x5cu\
0430\x5cu0442\x5cu0435\
\x5cu043b\x5cu0438\x5cu04\
3e\x5cu0441 \x5cu043f\x5c\
u043e\x5cu043c\x5cu043\
5\x5cu0441\x5cu0442\x5cu0\
438\x5cu0442\x5cu044c\x5c\
u0432\x5cu043e\x5cu043\
2\x5cu0440\x5cu0435\x5cu0\
43c\x5cu0435\x5cu043d\x5c\
u043d\x5cu043e\x5cu043\
5\x5cu0445\x5cu0440\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u043b\x5cu0438\x5cu044\
9\x5cu0435 \x5cu043f\x5cu\
043e\x5cu043c\x5cu0435\
\x5cu0441\x5cu0442\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
0444\x5cu0430\x5cu0439\
\x5cu043b \x5cu043f\x5cu0\
43e\x5cu043c\x5cu0435\x5c\
u0441\x5cu0442\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
444\x5cu0430\x5cu0439\x5c\
u043b\x5cu044b \x5cu04\
3f\x5cu0440\x5cu0430\x5cu\
0432 \x5cu043f\x5cu044\
0\x5cu0430\x5cu0432\x5cu0\
43e\x5cu0434\x5cu043e\x5c\
u0441\x5cu0442\x5cu044\
3\x5cu043f\x5cu0430 \x5cu\
043f\x5cu0440\x5cu0435\
\x5cu0434\x5cu043e\x5cu04\
3f\x5cu0440\x5cu0435\x5cu\
0434\x5cu0435\x5cu043b\
\x5cu0435\x5cu043d\x5cu04\
3d\x5cu043e\x5cu0435\x5cu\
0437\x5cu043d\x5cu0430\
\x5cu0447\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435 \x5c\
u043f\x5cu0440\x5cu043\
5\x5cu0434\x5cu0441\x5cu0\
442\x5cu0430\x5cu0432\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
43a\x5cu043e\x5cu0434\x5c\
u0430\x5cu043b\x5cu043\
e\x5cu043a\x5cu0430\x5cu0\
43b\x5cu0438\x5cu0437\x5c\
u0430\x5cu0446\x5cu043\
8\x5cu0438 \x5cu043f\x5cu\
0440\x5cu0435\x5cu0434\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu0432\x5cu043b\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu0435\x5cu043f\x5cu04\
35\x5cu0440\x5cu0438\x5cu\
043e\x5cu0434\x5cu0430\
 \x5cu043f\x5cu0440\x5cu0\
435\x5cu0434\x5cu0441\x5c\
u0442\x5cu0430\x5cu043\
2\x5cu043b\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u043f\x5cu0440\x5cu043\
0\x5cu0432\x5cu0430 \x5cu\
043f\x5cu0440\x5cu0435\
\x5cu0434\x5cu0441\x5cu04\
42\x5cu0430\x5cu0432\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
3f\x5cu0440\x5cu0438\x5cu\
043b\x5cu043e\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f \x5cu043f\x5c\
u0440\x5cu0435\x5cu043\
4\x5cu0441\x5cu0442\x5cu0\
430\x5cu0432\x5cu043b\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435\x5cu0441\x5cu0\
43e\x5cu0431\x5cu044b\x5c\
u0442\x5cu0438\x5cu044\
f\x5cu0436\x5cu0443\x5cu0\
440\x5cu043d\x5cu0430\x5c\
u043b\x5cu0430\x5cu044\
0\x5cu0435\x5cu0433\x5cu0\
438\x5cu0441\x5cu0442\x5c\
u0440\x5cu0430\x5cu044\
6\x5cu0438\x5cu0438 \x5cu\
043f\x5cu0440\x5cu0435\
\x5cu0434\x5cu0441\x5cu04\
42\x5cu0430\x5cu0432\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
47\x5cu0430\x5cu0441\x5cu\
043e\x5cu0432\x5cu043e\
\x5cu0433\x5cu043e\x5cu04\
3f\x5cu043e\x5cu044f\x5cu\
0441\x5cu0430 \x5cu043\
f\x5cu0440\x5cu0435\x5cu0\
434\x5cu0443\x5cu043f\x5c\
u0440\x5cu0435\x5cu043\
6\x5cu0434\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435 \
\x5cu043f\x5cu0440\x5cu04\
35\x5cu043a\x5cu0440\x5cu\
0430\x5cu0442\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
40\x5cu0430\x5cu0431\x5cu\
043e\x5cu0442\x5cu0443\
\x5cu0441\x5cu0438\x5cu04\
41\x5cu0442\x5cu0435\x5cu\
043c\x5cu044b \x5cu043\
f\x5cu0440\x5cu0438\x5cu0\
432\x5cu0438\x5cu043b\x5c\
u0435\x5cu0433\x5cu043\
8\x5cu0440\x5cu043e\x5cu0\
432\x5cu0430\x5cu043d\x5c\
u043d\x5cu044b\x5cu043\
9\x5cu0440\x5cu0435\x5cu0\
436\x5cu0438\x5cu043c \
\x5cu043f\x5cu0440\x5cu04\
3e\x5cu0434\x5cu043e\x5cu\
043b\x5cu0436\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
32\x5cu044b\x5cu0437\x5cu\
043e\x5cu0432 \x5cu043\
f\x5cu0440\x5cu043e\x5cu0\
447\x5cu0438\x5cu0442\x5c\
u0430\x5cu0442\x5cu044\
cjson \x5cu043f\x5cu04\
40\x5cu043e\x5cu0447\x5cu\
0438\x5cu0442\x5cu0430\
\x5cu0442\x5cu044cxml \
\x5cu043f\x5cu0440\x5cu04\
3e\x5cu0447\x5cu0438\x5cu\
0442\x5cu0430\x5cu0442\
\x5cu044c\x5cu0434\x5cu04\
30\x5cu0442\x5cu0443js\
on \x5cu043f\x5cu0443\x5c\
u0441\x5cu0442\x5cu043\
0\x5cu044f\x5cu0441\x5cu0\
442\x5cu0440\x5cu043e\x5c\
u043a\x5cu0430 \x5cu04\
40\x5cu0430\x5cu0431\x5cu\
043e\x5cu0447\x5cu0438\
\x5cu0439\x5cu043a\x5cu04\
30\x5cu0442\x5cu0430\x5cu\
043b\x5cu043e\x5cu0433\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu043d\x5cu044b\x5cu\
0445\x5cu043f\x5cu043e\
\x5cu043b\x5cu044c\x5cu04\
37\x5cu043e\x5cu0432\x5cu\
0430\x5cu0442\x5cu0435\
\x5cu043b\x5cu044f \x5cu0\
440\x5cu0430\x5cu0437\x5c\
u0431\x5cu043b\x5cu043\
e\x5cu043a\x5cu0438\x5cu0\
440\x5cu043e\x5cu0432\x5c\
u0430\x5cu0442\x5cu044\
c\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0435\x5cu0434\x5cu043\
b\x5cu044f\x5cu0440\x5cu0\
435\x5cu0434\x5cu0430\x5c\
u043a\x5cu0442\x5cu043\
8\x5cu0440\x5cu043e\x5cu0\
432\x5cu0430\x5cu043d\x5c\
u0438\x5cu044f \x5cu04\
40\x5cu0430\x5cu0437\x5cu\
0434\x5cu0435\x5cu043b\
\x5cu0438\x5cu0442\x5cu04\
4c\x5cu0444\x5cu0430\x5cu\
0439\x5cu043b \x5cu044\
0\x5cu0430\x5cu0437\x5cu0\
43e\x5cu0440\x5cu0432\x5c\
u0430\x5cu0442\x5cu044\
c\x5cu0441\x5cu043e\x5cu0\
435\x5cu0434\x5cu0438\x5c\
u043d\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
441\x5cu0432\x5cu043d\x5c\
u0435\x5cu0448\x5cu043\
d\x5cu0438\x5cu043c\x5cu0\
438\x5cu0441\x5cu0442\x5c\
u043e\x5cu0447\x5cu043\
d\x5cu0438\x5cu043a\x5cu0\
43e\x5cu043c\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445 \x5cu\
0440\x5cu0430\x5cu0441\
\x5cu043a\x5cu043e\x5cu04\
34\x5cu0438\x5cu0440\x5cu\
043e\x5cu0432\x5cu0430\
\x5cu0442\x5cu044c\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
043e\x5cu043a\x5cu0443\
 \x5cu0440\x5cu043e\x5cu0\
43b\x5cu044c\x5cu0434\x5c\
u043e\x5cu0441\x5cu044\
2\x5cu0443\x5cu043f\x5cu0\
43d\x5cu0430 \x5cu0441\
\x5cu0435\x5cu043a\x5cu04\
43\x5cu043d\x5cu0434\x5cu\
0430 \x5cu0441\x5cu043\
8\x5cu0433\x5cu043d\x5cu0\
430\x5cu043b \x5cu0441\
\x5cu0438\x5cu043c\x5cu04\
32\x5cu043e\x5cu043b \x5c\
u0441\x5cu043a\x5cu043\
e\x5cu043f\x5cu0438\x5cu0\
440\x5cu043e\x5cu0432\x5c\
u0430\x5cu0442\x5cu044\
c\x5cu0436\x5cu0443\x5cu0\
440\x5cu043d\x5cu0430\x5c\
u043b\x5cu0440\x5cu043\
5\x5cu0433\x5cu0438\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0430\x5cu0446\x5cu043\
8\x5cu0438 \x5cu0441\x5cu\
043c\x5cu0435\x5cu0449\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu043b\x5cu\
0435\x5cu0442\x5cu043d\
\x5cu0435\x5cu0433\x5cu04\
3e\x5cu0432\x5cu0440\x5cu\
0435\x5cu043c\x5cu0435\
\x5cu043d\x5cu0438 \x5cu0\
441\x5cu043c\x5cu0435\x5c\
u0449\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
441\x5cu0442\x5cu0430\x5c\
u043d\x5cu0434\x5cu043\
0\x5cu0440\x5cu0442\x5cu0\
43d\x5cu043e\x5cu0433\x5c\
u043e\x5cu0432\x5cu044\
0\x5cu0435\x5cu043c\x5cu0\
435\x5cu043d\x5cu0438 \
\x5cu0441\x5cu043e\x5cu04\
35\x5cu0434\x5cu0438\x5cu\
043d\x5cu0438\x5cu0442\
\x5cu044c\x5cu0431\x5cu04\
43\x5cu0444\x5cu0435\x5cu\
0440\x5cu044b\x5cu0434\
\x5cu0432\x5cu043e\x5cu04\
38\x5cu0447\x5cu043d\x5cu\
044b\x5cu0445\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0445 \x5c\
u0441\x5cu043e\x5cu043\
7\x5cu0434\x5cu0430\x5cu0\
442\x5cu044c\x5cu043a\x5c\
u0430\x5cu0442\x5cu043\
0\x5cu043b\x5cu043e\x5cu0\
433 \x5cu0441\x5cu043e\
\x5cu0437\x5cu0434\x5cu04\
30\x5cu0442\x5cu044c\x5cu\
0444\x5cu0430\x5cu0431\
\x5cu0440\x5cu0438\x5cu04\
3a\x5cu0443xdto \x5cu0\
441\x5cu043e\x5cu043a\x5c\
u0440\x5cu043b \x5cu04\
41\x5cu043e\x5cu043a\x5cu\
0440\x5cu043b\x5cu043f\
 \x5cu0441\x5cu043e\x5cu0\
43a\x5cu0440\x5cu043f \
\x5cu0441\x5cu043e\x5cu04\
3e\x5cu0431\x5cu0449\x5cu\
0438\x5cu0442\x5cu044c\
 \x5cu0441\x5cu043e\x5cu0\
441\x5cu0442\x5cu043e\x5c\
u044f\x5cu043d\x5cu043\
8\x5cu0435 \x5cu0441\x5cu\
043e\x5cu0445\x5cu0440\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
0437\x5cu043d\x5cu0430\
\x5cu0447\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435 \x5c\
u0441\x5cu043e\x5cu044\
5\x5cu0440\x5cu0430\x5cu0\
43d\x5cu0438\x5cu0442\x5c\
u044c\x5cu043d\x5cu043\
0\x5cu0441\x5cu0442\x5cu0\
440\x5cu043e\x5cu0439\x5c\
u043a\x5cu0438\x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
44c\x5cu0437\x5cu043e\x5c\
u0432\x5cu0430\x5cu044\
2\x5cu0435\x5cu043b\x5cu0\
44f \x5cu0441\x5cu0440\
\x5cu0435\x5cu0434 \x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0434\x5cu043b\x5cu043\
8\x5cu043d\x5cu0430 \x5cu\
0441\x5cu0442\x5cu0440\
\x5cu0437\x5cu0430\x5cu04\
3a\x5cu0430\x5cu043d\x5cu\
0447\x5cu0438\x5cu0432\
\x5cu0430\x5cu0435\x5cu04\
42\x5cu0441\x5cu044f\x5cu\
043d\x5cu0430 \x5cu044\
1\x5cu0442\x5cu0440\x5cu0\
437\x5cu0430\x5cu043c\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0442\x5cu044c \x5cu\
0441\x5cu0442\x5cu0440\
\x5cu043d\x5cu0430\x5cu04\
39\x5cu0442\x5cu0438 \x5c\
u0441\x5cu0442\x5cu044\
0\x5cu043d\x5cu0430\x5cu0\
447\x5cu0438\x5cu043d\x5c\
u0430\x5cu0435\x5cu044\
2\x5cu0441\x5cu044f\x5cu0\
441 \x5cu0441\x5cu0442\
\x5cu0440\x5cu043e\x5cu04\
3a\x5cu0430 \x5cu0441\x5c\
u0442\x5cu0440\x5cu043\
e\x5cu043a\x5cu0430\x5cu0\
441\x5cu043e\x5cu0435\x5c\
u0434\x5cu0438\x5cu043\
d\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu0438\x5c\
u043d\x5cu0444\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
430\x5cu0446\x5cu0438\x5c\
u043e\x5cu043d\x5cu043\
d\x5cu043e\x5cu0439\x5cu0\
431\x5cu0430\x5cu0437\x5c\
u044b \x5cu0441\x5cu04\
42\x5cu0440\x5cu043f\x5cu\
043e\x5cu043b\x5cu0443\
\x5cu0447\x5cu0438\x5cu04\
42\x5cu044c\x5cu0441\x5cu\
0442\x5cu0440\x5cu043e\
\x5cu043a\x5cu0443 \x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0440\x5cu0430\x5cu043\
7\x5cu0434\x5cu0435\x5cu0\
43b\x5cu0438\x5cu0442\x5c\
u044c \x5cu0441\x5cu04\
42\x5cu0440\x5cu0441\x5cu\
043e\x5cu0435\x5cu0434\
\x5cu0438\x5cu043d\x5cu04\
38\x5cu0442\x5cu044c \x5c\
u0441\x5cu0442\x5cu044\
0\x5cu0441\x5cu0440\x5cu0\
430\x5cu0432\x5cu043d\x5c\
u0438\x5cu0442\x5cu044\
c \x5cu0441\x5cu0442\x5cu\
0440\x5cu0447\x5cu0438\
\x5cu0441\x5cu043b\x5cu04\
3e\x5cu0432\x5cu0445\x5cu\
043e\x5cu0436\x5cu0434\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0439 \x5cu0441\x5c\
u0442\x5cu0440\x5cu044\
7\x5cu0438\x5cu0441\x5cu0\
43b\x5cu043e\x5cu0441\x5c\
u0442\x5cu0440\x5cu043\
e\x5cu043a \x5cu0441\x5cu\
0442\x5cu0440\x5cu0448\
\x5cu0430\x5cu0431\x5cu04\
3b\x5cu043e\x5cu043d \x5c\
u0442\x5cu0435\x5cu043\
a\x5cu0443\x5cu0449\x5cu0\
430\x5cu044f\x5cu0434\x5c\
u0430\x5cu0442\x5cu043\
0 \x5cu0442\x5cu0435\x5cu\
043a\x5cu0443\x5cu0449\
\x5cu0430\x5cu044f\x5cu04\
34\x5cu0430\x5cu0442\x5cu\
0430\x5cu0441\x5cu0435\
\x5cu0430\x5cu043d\x5cu04\
41\x5cu0430 \x5cu0442\x5c\
u0435\x5cu043a\x5cu044\
3\x5cu0449\x5cu0430\x5cu0\
44f\x5cu0443\x5cu043d\x5c\
u0438\x5cu0432\x5cu043\
5\x5cu0440\x5cu0441\x5cu0\
430\x5cu043b\x5cu044c\x5c\
u043d\x5cu0430\x5cu044\
f\x5cu0434\x5cu0430\x5cu0\
442\x5cu0430 \x5cu0442\
\x5cu0435\x5cu043a\x5cu04\
43\x5cu0449\x5cu0430\x5cu\
044f\x5cu0443\x5cu043d\
\x5cu0438\x5cu0432\x5cu04\
35\x5cu0440\x5cu0441\x5cu\
0430\x5cu043b\x5cu044c\
\x5cu043d\x5cu0430\x5cu04\
4f\x5cu0434\x5cu0430\x5cu\
0442\x5cu0430\x5cu0432\
\x5cu043c\x5cu0438\x5cu04\
3b\x5cu043b\x5cu0438\x5cu\
0441\x5cu0435\x5cu043a\
\x5cu0443\x5cu043d\x5cu04\
34\x5cu0430\x5cu0445 \x5c\
u0442\x5cu0435\x5cu043\
a\x5cu0443\x5cu0449\x5cu0\
438\x5cu0439\x5cu0432\x5c\
u0430\x5cu0440\x5cu043\
8\x5cu0430\x5cu043d\x5cu0\
442\x5cu0438\x5cu043d\x5c\
u0442\x5cu0435\x5cu044\
0\x5cu0444\x5cu0435\x5cu0\
439\x5cu0441\x5cu0430\x5c\
u043a\x5cu043b\x5cu043\
8\x5cu0435\x5cu043d\x5cu0\
442\x5cu0441\x5cu043a\x5c\
u043e\x5cu0433\x5cu043\
e\x5cu043f\x5cu0440\x5cu0\
438\x5cu043b\x5cu043e\x5c\
u0436\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f \x5cu\
0442\x5cu0435\x5cu043a\
\x5cu0443\x5cu0449\x5cu04\
38\x5cu0439\x5cu0432\x5cu\
0430\x5cu0440\x5cu0438\
\x5cu0430\x5cu043d\x5cu04\
42\x5cu043e\x5cu0441\x5cu\
043d\x5cu043e\x5cu0432\
\x5cu043d\x5cu043e\x5cu04\
33\x5cu043e\x5cu0448\x5cu\
0440\x5cu0438\x5cu0444\
\x5cu0442\x5cu0430\x5cu04\
3a\x5cu043b\x5cu0438\x5cu\
0435\x5cu043d\x5cu0442\
\x5cu0441\x5cu043a\x5cu04\
3e\x5cu0433\x5cu043e\x5cu\
043f\x5cu0440\x5cu0438\
\x5cu043b\x5cu043e\x5cu04\
36\x5cu0435\x5cu043d\x5cu\
0438\x5cu044f \x5cu044\
2\x5cu0435\x5cu043a\x5cu0\
443\x5cu0449\x5cu0438\x5c\
u0439\x5cu043a\x5cu043\
e\x5cu0434\x5cu043b\x5cu0\
43e\x5cu043a\x5cu0430\x5c\
u043b\x5cu0438\x5cu043\
7\x5cu0430\x5cu0446\x5cu0\
438\x5cu0438 \x5cu0442\
\x5cu0435\x5cu043a\x5cu04\
43\x5cu0449\x5cu0438\x5cu\
0439\x5cu0440\x5cu0435\
\x5cu0436\x5cu0438\x5cu04\
3c\x5cu0437\x5cu0430\x5cu\
043f\x5cu0443\x5cu0441\
\x5cu043a\x5cu0430 \x5cu0\
442\x5cu0435\x5cu043a\x5c\
u0443\x5cu0449\x5cu043\
8\x5cu0439\x5cu044f\x5cu0\
437\x5cu044b\x5cu043a \
\x5cu0442\x5cu0435\x5cu04\
3a\x5cu0443\x5cu0449\x5cu\
0438\x5cu0439\x5cu044f\
\x5cu0437\x5cu044b\x5cu04\
3a\x5cu0441\x5cu0438\x5cu\
0441\x5cu0442\x5cu0435\
\x5cu043c\x5cu044b \x5cu0\
442\x5cu0438\x5cu043f \
\x5cu0442\x5cu0438\x5cu04\
3f\x5cu0437\x5cu043d\x5cu\
0447 \x5cu0442\x5cu044\
0\x5cu0430\x5cu043d\x5cu0\
437\x5cu0430\x5cu043a\x5c\
u0446\x5cu0438\x5cu044\
f\x5cu0430\x5cu043a\x5cu0\
442\x5cu0438\x5cu0432\x5c\
u043d\x5cu0430 \x5cu04\
42\x5cu0440\x5cu0435\x5cu\
0433 \x5cu0443\x5cu043\
4\x5cu0430\x5cu043b\x5cu0\
438\x5cu0442\x5cu044c\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
435\x5cu0438\x5cu043d\x5c\
u0444\x5cu043e\x5cu044\
0\x5cu043c\x5cu0430\x5cu0\
446\x5cu0438\x5cu043e\x5c\
u043d\x5cu043d\x5cu043\
e\x5cu0439\x5cu0431\x5cu0\
430\x5cu0437\x5cu044b \
\x5cu0443\x5cu0434\x5cu04\
30\x5cu043b\x5cu0438\x5cu\
0442\x5cu044c\x5cu0438\
\x5cu0437\x5cu0432\x5cu04\
40\x5cu0435\x5cu043c\x5cu\
0435\x5cu043d\x5cu043d\
\x5cu043e\x5cu0433\x5cu04\
3e\x5cu0445\x5cu0440\x5cu\
0430\x5cu043d\x5cu0438\
\x5cu043b\x5cu0438\x5cu04\
49\x5cu0430 \x5cu0443\x5c\
u0434\x5cu0430\x5cu043\
b\x5cu0438\x5cu0442\x5cu0\
44c\x5cu043e\x5cu0431\x5c\
u044a\x5cu0435\x5cu043\
a\x5cu0442\x5cu044b \x5cu\
0443\x5cu0434\x5cu0430\
\x5cu043b\x5cu0438\x5cu04\
42\x5cu044c\x5cu0444\x5cu\
0430\x5cu0439\x5cu043b\
\x5cu044b \x5cu0443\x5cu0\
43d\x5cu0438\x5cu0432\x5c\
u0435\x5cu0440\x5cu044\
1\x5cu0430\x5cu043b\x5cu0\
44c\x5cu043d\x5cu043e\x5c\
u0435\x5cu0432\x5cu044\
0\x5cu0435\x5cu043c\x5cu0\
44f \x5cu0443\x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu0431\x5cu0435\x5cu04\
37\x5cu043e\x5cu043f\x5cu\
0430\x5cu0441\x5cu043d\
\x5cu044b\x5cu0439\x5cu04\
40\x5cu0435\x5cu0436\x5cu\
0438\x5cu043c \x5cu044\
3\x5cu0441\x5cu0442\x5cu0\
430\x5cu043d\x5cu043e\x5c\
u0432\x5cu0438\x5cu044\
2\x5cu044c\x5cu0431\x5cu0\
435\x5cu0437\x5cu043e\x5c\
u043f\x5cu0430\x5cu044\
1\x5cu043d\x5cu044b\x5cu0\
439\x5cu0440\x5cu0435\x5c\
u0436\x5cu0438\x5cu043\
c\x5cu0440\x5cu0430\x5cu0\
437\x5cu0434\x5cu0435\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u043d\x5cu044b\x5cu044\
5 \x5cu0443\x5cu0441\x5cu\
0442\x5cu0430\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
0431\x5cu043b\x5cu043e\
\x5cu043a\x5cu0438\x5cu04\
40\x5cu043e\x5cu0432\x5cu\
043a\x5cu0443\x5cu0441\
\x5cu0435\x5cu0430\x5cu04\
3d\x5cu0441\x5cu043e\x5cu\
0432 \x5cu0443\x5cu044\
1\x5cu0442\x5cu0430\x5cu0\
43d\x5cu043e\x5cu0432\x5c\
u0438\x5cu0442\x5cu044\
c\x5cu0432\x5cu043d\x5cu0\
435\x5cu0448\x5cu043d\x5c\
u044e\x5cu044e\x5cu043\
a\x5cu043e\x5cu043c\x5cu0\
43f\x5cu043e\x5cu043d\x5c\
u0435\x5cu043d\x5cu044\
2\x5cu0443 \x5cu0443\x5cu\
0441\x5cu0442\x5cu0430\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu0438\x5cu0442\x5cu\
044c\x5cu0432\x5cu0440\
\x5cu0435\x5cu043c\x5cu04\
4f\x5cu0437\x5cu0430\x5cu\
0432\x5cu0435\x5cu0440\
\x5cu0448\x5cu0435\x5cu04\
3d\x5cu0438\x5cu044f\x5cu\
0441\x5cu043f\x5cu044f\
\x5cu0449\x5cu0435\x5cu04\
33\x5cu043e\x5cu0441\x5cu\
0435\x5cu0430\x5cu043d\
\x5cu0441\x5cu0430 \x5cu0\
443\x5cu0441\x5cu0442\x5c\
u0430\x5cu043d\x5cu043\
e\x5cu0432\x5cu0438\x5cu0\
442\x5cu044c\x5cu0432\x5c\
u0440\x5cu0435\x5cu043\
c\x5cu044f\x5cu0437\x5cu0\
430\x5cu0441\x5cu044b\x5c\
u043f\x5cu0430\x5cu043\
d\x5cu0438\x5cu044f\x5cu0\
43f\x5cu0430\x5cu0441\x5c\
u0441\x5cu0438\x5cu043\
2\x5cu043d\x5cu043e\x5cu0\
433\x5cu043e\x5cu0441\x5c\
u0435\x5cu0430\x5cu043\
d\x5cu0441\x5cu0430 \x5cu\
0443\x5cu0441\x5cu0442\
\x5cu0430\x5cu043d\x5cu04\
3e\x5cu0432\x5cu0438\x5cu\
0442\x5cu044c\x5cu0432\
\x5cu0440\x5cu0435\x5cu04\
3c\x5cu044f\x5cu043e\x5cu\
0436\x5cu0438\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu044f\x5cu0431\x5cu\
043b\x5cu043e\x5cu043a\
\x5cu0438\x5cu0440\x5cu04\
3e\x5cu0432\x5cu043a\x5cu\
0438\x5cu0434\x5cu0430\
\x5cu043d\x5cu043d\x5cu04\
4b\x5cu0445 \x5cu0443\x5c\
u0441\x5cu0442\x5cu043\
0\x5cu043d\x5cu043e\x5cu0\
432\x5cu0438\x5cu0442\x5c\
u044c\x5cu0437\x5cu043\
0\x5cu0433\x5cu043e\x5cu0\
43b\x5cu043e\x5cu0432\x5c\
u043e\x5cu043a\x5cu043\
a\x5cu043b\x5cu0438\x5cu0\
435\x5cu043d\x5cu0442\x5c\
u0441\x5cu043a\x5cu043\
e\x5cu0433\x5cu043e\x5cu0\
43f\x5cu0440\x5cu0438\x5c\
u043b\x5cu043e\x5cu043\
6\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f \x5cu0443\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu043d\x5cu043e\x5cu\
0432\x5cu0438\x5cu0442\
\x5cu044c\x5cu0437\x5cu04\
30\x5cu0433\x5cu043e\x5cu\
043b\x5cu043e\x5cu0432\
\x5cu043e\x5cu043a\x5cu04\
41\x5cu0438\x5cu0441\x5cu\
0442\x5cu0435\x5cu043c\
\x5cu044b \x5cu0443\x5cu0\
441\x5cu0442\x5cu0430\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu0438\x5cu0442\x5cu0\
44c\x5cu0438\x5cu0441\x5c\
u043f\x5cu043e\x5cu043\
b\x5cu044c\x5cu0437\x5cu0\
43e\x5cu0432\x5cu0430\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu0436\x5cu0443\x5cu0\
440\x5cu043d\x5cu0430\x5c\
u043b\x5cu0430\x5cu044\
0\x5cu0435\x5cu0433\x5cu0\
438\x5cu0441\x5cu0442\x5c\
u0440\x5cu0430\x5cu044\
6\x5cu0438\x5cu0438 \x5cu\
0443\x5cu0441\x5cu0442\
\x5cu0430\x5cu043d\x5cu04\
3e\x5cu0432\x5cu0438\x5cu\
0442\x5cu044c\x5cu0438\
\x5cu0441\x5cu043f\x5cu04\
3e\x5cu043b\x5cu044c\x5cu\
0437\x5cu043e\x5cu0432\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu0435\x5cu0441\x5cu\
043e\x5cu0431\x5cu044b\
\x5cu0442\x5cu0438\x5cu04\
4f\x5cu0436\x5cu0443\x5cu\
0440\x5cu043d\x5cu0430\
\x5cu043b\x5cu0430\x5cu04\
40\x5cu0435\x5cu0433\x5cu\
0438\x5cu0441\x5cu0442\
\x5cu0440\x5cu0430\x5cu04\
46\x5cu0438\x5cu0438 \x5c\
u0443\x5cu0441\x5cu044\
2\x5cu0430\x5cu043d\x5cu0\
43e\x5cu0432\x5cu0438\x5c\
u0442\x5cu044c\x5cu043\
a\x5cu0440\x5cu0430\x5cu0\
442\x5cu043a\x5cu0438\x5c\
u0439\x5cu0437\x5cu043\
0\x5cu0433\x5cu043e\x5cu0\
43b\x5cu043e\x5cu0432\x5c\
u043e\x5cu043a\x5cu043\
f\x5cu0440\x5cu0438\x5cu0\
43b\x5cu043e\x5cu0436\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f \x5cu0443\x5cu\
0441\x5cu0442\x5cu0430\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu0438\x5cu0442\x5cu\
044c\x5cu043c\x5cu0438\
\x5cu043d\x5cu0438\x5cu04\
3c\x5cu0430\x5cu043b\x5cu\
044c\x5cu043d\x5cu0443\
\x5cu044e\x5cu0434\x5cu04\
3b\x5cu0438\x5cu043d\x5cu\
0443\x5cu043f\x5cu0430\
\x5cu0440\x5cu043e\x5cu04\
3b\x5cu0435\x5cu0439\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu044c\x5cu0437\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
0442\x5cu0435\x5cu043b\
\x5cu0435\x5cu0439 \x5cu0\
443\x5cu0441\x5cu0442\x5c\
u0430\x5cu043d\x5cu043\
e\x5cu0432\x5cu0438\x5cu0\
442\x5cu044c\x5cu043c\x5c\
u043e\x5cu043d\x5cu043\
e\x5cu043f\x5cu043e\x5cu0\
43b\x5cu044c\x5cu043d\x5c\
u044b\x5cu0439\x5cu044\
0\x5cu0435\x5cu0436\x5cu0\
438\x5cu043c \x5cu0443\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu043d\x5cu043e\x5cu\
0432\x5cu0438\x5cu0442\
\x5cu044c\x5cu043d\x5cu04\
30\x5cu0441\x5cu0442\x5cu\
0440\x5cu043e\x5cu0439\
\x5cu043a\x5cu0438\x5cu04\
3a\x5cu043b\x5cu0438\x5cu\
0435\x5cu043d\x5cu0442\
\x5cu0430\x5cu043b\x5cu04\
38\x5cu0446\x5cu0435\x5cu\
043d\x5cu0437\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0430\x5cu043d\x5cu\
0438\x5cu044f \x5cu044\
3\x5cu0441\x5cu0442\x5cu0\
430\x5cu043d\x5cu043e\x5c\
u0432\x5cu0438\x5cu044\
2\x5cu044c\x5cu043e\x5cu0\
431\x5cu043d\x5cu043e\x5c\
u0432\x5cu043b\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu043f\x5cu0440\x5c\
u0435\x5cu0434\x5cu043\
e\x5cu043f\x5cu0440\x5cu0\
435\x5cu0434\x5cu0435\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445\x5cu0438\x5cu0\
43d\x5cu0444\x5cu043e\x5c\
u0440\x5cu043c\x5cu043\
0\x5cu0446\x5cu0438\x5cu0\
43e\x5cu043d\x5cu043d\x5c\
u043e\x5cu0439\x5cu043\
1\x5cu0430\x5cu0437\x5cu0\
44b \x5cu0443\x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu043e\x5cu0442\x5cu04\
3a\x5cu043b\x5cu044e\x5cu\
0447\x5cu0435\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
31\x5cu0435\x5cu0437\x5cu\
043e\x5cu043f\x5cu0430\
\x5cu0441\x5cu043d\x5cu04\
3e\x5cu0433\x5cu043e\x5cu\
0440\x5cu0435\x5cu0436\
\x5cu0438\x5cu043c\x5cu04\
30 \x5cu0443\x5cu0441\x5c\
u0442\x5cu0430\x5cu043\
d\x5cu043e\x5cu0432\x5cu0\
438\x5cu0442\x5cu044c\x5c\
u043f\x5cu0430\x5cu044\
0\x5cu0430\x5cu043c\x5cu0\
435\x5cu0442\x5cu0440\x5c\
u044b\x5cu0444\x5cu044\
3\x5cu043d\x5cu043a\x5cu0\
446\x5cu0438\x5cu043e\x5c\
u043d\x5cu0430\x5cu043\
b\x5cu044c\x5cu043d\x5cu0\
44b\x5cu0445\x5cu043e\x5c\
u043f\x5cu0446\x5cu043\
8\x5cu0439\x5cu0438\x5cu0\
43d\x5cu0442\x5cu0435\x5c\
u0440\x5cu0444\x5cu043\
5\x5cu0439\x5cu0441\x5cu0\
430 \x5cu0443\x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu043f\x5cu0440\x5cu04\
38\x5cu0432\x5cu0438\x5cu\
043b\x5cu0435\x5cu0433\
\x5cu0438\x5cu0440\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0439\x5cu0440\x5cu04\
35\x5cu0436\x5cu0438\x5cu\
043c \x5cu0443\x5cu044\
1\x5cu0442\x5cu0430\x5cu0\
43d\x5cu043e\x5cu0432\x5c\
u0438\x5cu0442\x5cu044\
c\x5cu043f\x5cu0440\x5cu0\
43e\x5cu0432\x5cu0435\x5c\
u0440\x5cu043a\x5cu044\
3\x5cu0441\x5cu043b\x5cu0\
43e\x5cu0436\x5cu043d\x5c\
u043e\x5cu0441\x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
430\x5cu0440\x5cu043e\x5c\
u043b\x5cu0435\x5cu043\
9\x5cu043f\x5cu043e\x5cu0\
43b\x5cu044c\x5cu0437\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu0442\x5cu0435\x5cu0\
43b\x5cu0435\x5cu0439 \
\x5cu0443\x5cu0441\x5cu04\
42\x5cu0430\x5cu043d\x5cu\
043e\x5cu0432\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
40\x5cu0430\x5cu0441\x5cu\
0448\x5cu0438\x5cu0440\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0440\x5cu\
0430\x5cu0431\x5cu043e\
\x5cu0442\x5cu044b\x5cu04\
41\x5cu043a\x5cu0440\x5cu\
0438\x5cu043f\x5cu0442\
\x5cu043e\x5cu0433\x5cu04\
40\x5cu0430\x5cu0444\x5cu\
0438\x5cu0435\x5cu0439\
 \x5cu0443\x5cu0441\x5cu0\
442\x5cu0430\x5cu043d\x5c\
u043e\x5cu0432\x5cu043\
8\x5cu0442\x5cu044c\x5cu0\
440\x5cu0430\x5cu0441\x5c\
u0448\x5cu0438\x5cu044\
0\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu0440\x5c\
u0430\x5cu0431\x5cu043\
e\x5cu0442\x5cu044b\x5cu0\
441\x5cu0444\x5cu0430\x5c\
u0439\x5cu043b\x5cu043\
0\x5cu043c\x5cu0438 \x5cu\
0443\x5cu0441\x5cu0442\
\x5cu0430\x5cu043d\x5cu04\
3e\x5cu0432\x5cu0438\x5cu\
0442\x5cu044c\x5cu0441\
\x5cu043e\x5cu0435\x5cu04\
34\x5cu0438\x5cu043d\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu0435\x5cu0441\x5cu04\
32\x5cu043d\x5cu0435\x5cu\
0448\x5cu043d\x5cu0438\
\x5cu043c\x5cu0438\x5cu04\
41\x5cu0442\x5cu043e\x5cu\
0447\x5cu043d\x5cu0438\
\x5cu043a\x5cu043e\x5cu04\
3c\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445 \x5cu0443\x5cu0\
441\x5cu0442\x5cu0430\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu0438\x5cu0442\x5cu0\
44c\x5cu0441\x5cu043e\x5c\
u043e\x5cu0442\x5cu043\
2\x5cu0435\x5cu0442\x5cu0\
441\x5cu0442\x5cu0432\x5c\
u0438\x5cu0435\x5cu043\
e\x5cu0431\x5cu044a\x5cu0\
435\x5cu043a\x5cu0442\x5c\
u0430\x5cu0438\x5cu044\
4\x5cu043e\x5cu0440\x5cu0\
43c\x5cu044b \x5cu0443\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu043d\x5cu043e\x5cu\
0432\x5cu0438\x5cu0442\
\x5cu044c\x5cu0441\x5cu04\
3e\x5cu0441\x5cu0442\x5cu\
0430\x5cu0432\x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
3d\x5cu0434\x5cu0430\x5cu\
0440\x5cu0442\x5cu043d\
\x5cu043e\x5cu0433\x5cu04\
3e\x5cu0438\x5cu043d\x5cu\
0442\x5cu0435\x5cu0440\
\x5cu0444\x5cu0435\x5cu04\
39\x5cu0441\x5cu0430od\
ata \x5cu0443\x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu0447\x5cu0430\x5cu04\
41\x5cu043e\x5cu0432\x5cu\
043e\x5cu0439\x5cu043f\
\x5cu043e\x5cu044f\x5cu04\
41\x5cu0438\x5cu043d\x5cu\
0444\x5cu043e\x5cu0440\
\x5cu043c\x5cu0430\x5cu04\
46\x5cu0438\x5cu043e\x5cu\
043d\x5cu043d\x5cu043e\
\x5cu0439\x5cu0431\x5cu04\
30\x5cu0437\x5cu044b \x5c\
u0443\x5cu0441\x5cu044\
2\x5cu0430\x5cu043d\x5cu0\
43e\x5cu0432\x5cu0438\x5c\
u0442\x5cu044c\x5cu044\
7\x5cu0430\x5cu0441\x5cu0\
43e\x5cu0432\x5cu043e\x5c\
u0439\x5cu043f\x5cu043\
e\x5cu044f\x5cu0441\x5cu0\
441\x5cu0435\x5cu0430\x5c\
u043d\x5cu0441\x5cu043\
0 \x5cu0444\x5cu043e\x5cu\
0440\x5cu043c\x5cu0430\
\x5cu0442 \x5cu0446\x5cu0\
435\x5cu043b \x5cu0447\
\x5cu0430\x5cu0441 \x5cu0\
447\x5cu0430\x5cu0441\x5c\
u043e\x5cu0432\x5cu043\
e\x5cu0439\x5cu043f\x5cu0\
43e\x5cu044f\x5cu0441 \
\x5cu0447\x5cu0430\x5cu04\
41\x5cu043e\x5cu0432\x5cu\
043e\x5cu0439\x5cu043f\
\x5cu043e\x5cu044f\x5cu04\
41\x5cu0441\x5cu0435\x5cu\
0430\x5cu043d\x5cu0441\
\x5cu0430 \x5cu0447\x5cu0\
438\x5cu0441\x5cu043b\x5c\
u043e \x5cu0447\x5cu04\
38\x5cu0441\x5cu043b\x5cu\
043e\x5cu043f\x5cu0440\
\x5cu043e\x5cu043f\x5cu04\
38\x5cu0441\x5cu044c\x5cu\
044e \x5cu044d\x5cu044\
2\x5cu043e\x5cu0430\x5cu0\
434\x5cu0440\x5cu0435\x5c\
u0441\x5cu0432\x5cu044\
0\x5cu0435\x5cu043c\x5cu0\
435\x5cu043d\x5cu043d\x5c\
u043e\x5cu0433\x5cu043\
e\x5cu0445\x5cu0440\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u043b\x5cu0438\x5cu044\
9\x5cu0430 ws\x5cu0441\
\x5cu0441\x5cu044b\x5cu04\
3b\x5cu043a\x5cu0438 \x5c\
u0431\x5cu0438\x5cu043\
1\x5cu043b\x5cu0438\x5cu0\
43e\x5cu0442\x5cu0435\x5c\
u043a\x5cu0430\x5cu043\
a\x5cu0430\x5cu0440\x5cu0\
442\x5cu0438\x5cu043d\x5c\
u043e\x5cu043a \x5cu04\
31\x5cu0438\x5cu0431\x5cu\
043b\x5cu0438\x5cu043e\
\x5cu0442\x5cu0435\x5cu04\
3a\x5cu0430\x5cu043c\x5cu\
0430\x5cu043a\x5cu0435\
\x5cu0442\x5cu043e\x5cu04\
32\x5cu043e\x5cu0444\x5cu\
043e\x5cu0440\x5cu043c\
\x5cu043b\x5cu0435\x5cu04\
3d\x5cu0438\x5cu044f\x5cu\
043a\x5cu043e\x5cu043c\
\x5cu043f\x5cu043e\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
043a\x5cu0438\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0445 \x5c\
u0431\x5cu0438\x5cu043\
1\x5cu043b\x5cu0438\x5cu0\
43e\x5cu0442\x5cu0435\x5c\
u043a\x5cu0430\x5cu044\
1\x5cu0442\x5cu0438\x5cu0\
43b\x5cu0435\x5cu0439 \
\x5cu0431\x5cu0438\x5cu04\
37\x5cu043d\x5cu0435\x5cu\
0441\x5cu043f\x5cu0440\
\x5cu043e\x5cu0446\x5cu04\
35\x5cu0441\x5cu0441\x5cu\
044b \x5cu0432\x5cu043\
d\x5cu0435\x5cu0448\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0438\x5cu0441\x5cu044\
2\x5cu043e\x5cu0447\x5cu0\
43d\x5cu0438\x5cu043a\x5c\
u0438\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu0432\
\x5cu043d\x5cu0435\x5cu04\
48\x5cu043d\x5cu0438\x5cu\
0435\x5cu043e\x5cu0431\
\x5cu0440\x5cu0430\x5cu04\
31\x5cu043e\x5cu0442\x5cu\
043a\x5cu0438 \x5cu043\
2\x5cu043d\x5cu0435\x5cu0\
448\x5cu043d\x5cu0438\x5c\
u0435\x5cu043e\x5cu044\
2\x5cu0447\x5cu0435\x5cu0\
442\x5cu044b \x5cu0432\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu043e\x5cu0435\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0435\x5cu043f\x5cu04\
3e\x5cu043a\x5cu0443\x5cu\
043f\x5cu043a\x5cu0438\
 \x5cu0433\x5cu043b\x5cu0\
430\x5cu0432\x5cu043d\x5c\
u044b\x5cu0439\x5cu043\
8\x5cu043d\x5cu0442\x5cu0\
435\x5cu0440\x5cu0444\x5c\
u0435\x5cu0439\x5cu044\
1 \x5cu0433\x5cu043b\x5cu\
0430\x5cu0432\x5cu043d\
\x5cu044b\x5cu0439\x5cu04\
41\x5cu0442\x5cu0438\x5cu\
043b\x5cu044c \x5cu043\
4\x5cu043e\x5cu043a\x5cu0\
443\x5cu043c\x5cu0435\x5c\
u043d\x5cu0442\x5cu044\
b \x5cu0434\x5cu043e\x5cu\
0441\x5cu0442\x5cu0430\
\x5cu0432\x5cu043b\x5cu04\
4f\x5cu0435\x5cu043c\x5cu\
044b\x5cu0435\x5cu0443\
\x5cu0432\x5cu0435\x5cu04\
34\x5cu043e\x5cu043c\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f \x5cu0\
436\x5cu0443\x5cu0440\x5c\
u043d\x5cu0430\x5cu043\
b\x5cu044b\x5cu0434\x5cu0\
43e\x5cu043a\x5cu0443\x5c\
u043c\x5cu0435\x5cu043\
d\x5cu0442\x5cu043e\x5cu0\
432 \x5cu0437\x5cu0430\
\x5cu0434\x5cu0430\x5cu04\
47\x5cu0438 \x5cu0438\x5c\
u043d\x5cu0444\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
430\x5cu0446\x5cu0438\x5c\
u044f\x5cu043e\x5cu043\
1\x5cu0438\x5cu043d\x5cu0\
442\x5cu0435\x5cu0440\x5c\
u043d\x5cu0435\x5cu044\
2\x5cu0441\x5cu043e\x5cu0\
435\x5cu0434\x5cu0438\x5c\
u043d\x5cu0435\x5cu043\
d\x5cu0438\x5cu0438 \x5cu\
0438\x5cu0441\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
4c\x5cu0437\x5cu043e\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
40\x5cu0430\x5cu0431\x5cu\
043e\x5cu0447\x5cu0435\
\x5cu0439\x5cu0434\x5cu04\
30\x5cu0442\x5cu044b \x5c\
u0438\x5cu0441\x5cu044\
2\x5cu043e\x5cu0440\x5cu0\
438\x5cu044f\x5cu0440\x5c\
u0430\x5cu0431\x5cu043\
e\x5cu0442\x5cu044b\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u044c\x5cu0437\x5cu043\
e\x5cu0432\x5cu0430\x5cu0\
442\x5cu0435\x5cu043b\x5c\
u044f \x5cu043a\x5cu04\
3e\x5cu043d\x5cu0441\x5cu\
0442\x5cu0430\x5cu043d\
\x5cu0442\x5cu044b \x5cu0\
43a\x5cu0440\x5cu0438\x5c\
u0442\x5cu0435\x5cu044\
0\x5cu0438\x5cu0438\x5cu0\
43e\x5cu0442\x5cu0431\x5c\
u043e\x5cu0440\x5cu043\
0 \x5cu043c\x5cu0435\x5cu\
0442\x5cu0430\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0435 \x5c\
u043e\x5cu0431\x5cu044\
0\x5cu0430\x5cu0431\x5cu0\
43e\x5cu0442\x5cu043a\x5c\
u0438 \x5cu043e\x5cu04\
42\x5cu043e\x5cu0431\x5cu\
0440\x5cu0430\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0440\x5cu\
0435\x5cu043a\x5cu043b\
\x5cu0430\x5cu043c\x5cu04\
4b \x5cu043e\x5cu0442\x5c\
u043f\x5cu0440\x5cu043\
0\x5cu0432\x5cu043a\x5cu0\
430\x5cu0434\x5cu043e\x5c\
u0441\x5cu0442\x5cu043\
0\x5cu0432\x5cu043b\x5cu0\
44f\x5cu0435\x5cu043c\x5c\
u044b\x5cu0445\x5cu044\
3\x5cu0432\x5cu0435\x5cu0\
434\x5cu043e\x5cu043c\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu0439 \x5cu\
043e\x5cu0442\x5cu0447\
\x5cu0435\x5cu0442\x5cu04\
4b \x5cu043f\x5cu0430\x5c\
u043d\x5cu0435\x5cu043\
b\x5cu044c\x5cu0437\x5cu0\
430\x5cu0434\x5cu0430\x5c\
u0447\x5cu043e\x5cu044\
1 \x5cu043f\x5cu0430\x5cu\
0440\x5cu0430\x5cu043c\
\x5cu0435\x5cu0442\x5cu04\
40\x5cu0437\x5cu0430\x5cu\
043f\x5cu0443\x5cu0441\
\x5cu043a\x5cu0430 \x5cu0\
43f\x5cu0430\x5cu0440\x5c\
u0430\x5cu043c\x5cu043\
5\x5cu0442\x5cu0440\x5cu0\
44b\x5cu0441\x5cu0435\x5c\
u0430\x5cu043d\x5cu044\
1\x5cu0430 \x5cu043f\x5cu\
0435\x5cu0440\x5cu0435\
\x5cu0447\x5cu0438\x5cu04\
41\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
 \x5cu043f\x5cu043b\x5cu0\
430\x5cu043d\x5cu044b\x5c\
u0432\x5cu0438\x5cu043\
4\x5cu043e\x5cu0432\x5cu0\
440\x5cu0430\x5cu0441\x5c\
u0447\x5cu0435\x5cu044\
2\x5cu0430 \x5cu043f\x5cu\
043b\x5cu0430\x5cu043d\
\x5cu044b\x5cu0432\x5cu04\
38\x5cu0434\x5cu043e\x5cu\
0432\x5cu0445\x5cu0430\
\x5cu0440\x5cu0430\x5cu04\
3a\x5cu0442\x5cu0435\x5cu\
0440\x5cu0438\x5cu0441\
\x5cu0442\x5cu0438\x5cu04\
3a \x5cu043f\x5cu043b\x5c\
u0430\x5cu043d\x5cu044\
b\x5cu043e\x5cu0431\x5cu0\
43c\x5cu0435\x5cu043d\x5c\
u0430 \x5cu043f\x5cu04\
3b\x5cu0430\x5cu043d\x5cu\
044b\x5cu0441\x5cu0447\
\x5cu0435\x5cu0442\x5cu04\
3e\x5cu0432 \x5cu043f\x5c\
u043e\x5cu043b\x5cu043\
d\x5cu043e\x5cu0442\x5cu0\
435\x5cu043a\x5cu0441\x5c\
u0442\x5cu043e\x5cu043\
2\x5cu044b\x5cu0439\x5cu0\
43f\x5cu043e\x5cu0438\x5c\
u0441\x5cu043a \x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
044c\x5cu0437\x5cu043e\
\x5cu0432\x5cu0430\x5cu04\
42\x5cu0435\x5cu043b\x5cu\
0438\x5cu0438\x5cu043d\
\x5cu0444\x5cu043e\x5cu04\
40\x5cu043c\x5cu0430\x5cu\
0446\x5cu0438\x5cu043e\
\x5cu043d\x5cu043d\x5cu04\
3e\x5cu0439\x5cu0431\x5cu\
0430\x5cu0437\x5cu044b\
 \x5cu043f\x5cu043e\x5cu0\
441\x5cu043b\x5cu0435\x5c\
u0434\x5cu043e\x5cu043\
2\x5cu0430\x5cu0442\x5cu0\
435\x5cu043b\x5cu044c\x5c\
u043d\x5cu043e\x5cu044\
1\x5cu0442\x5cu0438 \x5cu\
043f\x5cu0440\x5cu043e\
\x5cu0432\x5cu0435\x5cu04\
40\x5cu043a\x5cu0430\x5cu\
0432\x5cu0441\x5cu0442\
\x5cu0440\x5cu043e\x5cu04\
35\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445\x5cu043f\
\x5cu043e\x5cu043a\x5cu04\
43\x5cu043f\x5cu043e\x5cu\
043a \x5cu0440\x5cu043\
0\x5cu0431\x5cu043e\x5cu0\
447\x5cu0430\x5cu044f\x5c\
u0434\x5cu0430\x5cu044\
2\x5cu0430 \x5cu0440\x5cu\
0430\x5cu0441\x5cu0448\
\x5cu0438\x5cu0440\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f\x5cu043a\x5cu043e\
\x5cu043d\x5cu0444\x5cu04\
38\x5cu0433\x5cu0443\x5cu\
0440\x5cu0430\x5cu0446\
\x5cu0438\x5cu0438 \x5cu0\
440\x5cu0435\x5cu0433\x5c\
u0438\x5cu0441\x5cu044\
2\x5cu0440\x5cu044b\x5cu0\
431\x5cu0443\x5cu0445\x5c\
u0433\x5cu0430\x5cu043\
b\x5cu0442\x5cu0435\x5cu0\
440\x5cu0438\x5cu0438 \
\x5cu0440\x5cu0435\x5cu04\
33\x5cu0438\x5cu0441\x5cu\
0442\x5cu0440\x5cu044b\
\x5cu043d\x5cu0430\x5cu04\
3a\x5cu043e\x5cu043f\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f \x5cu0\
440\x5cu0435\x5cu0433\x5c\
u0438\x5cu0441\x5cu044\
2\x5cu0440\x5cu044b\x5cu0\
440\x5cu0430\x5cu0441\x5c\
u0447\x5cu0435\x5cu044\
2\x5cu0430 \x5cu0440\x5cu\
0435\x5cu0433\x5cu0438\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu044b\x5cu0441\x5cu\
0432\x5cu0435\x5cu0434\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0439 \x5cu0440\x5c\
u0435\x5cu0433\x5cu043\
b\x5cu0430\x5cu043c\x5cu0\
435\x5cu043d\x5cu0442\x5c\
u043d\x5cu044b\x5cu043\
5\x5cu0437\x5cu0430\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u0438\x5cu044f \x5cu04\
41\x5cu0435\x5cu0440\x5cu\
0438\x5cu0430\x5cu043b\
\x5cu0438\x5cu0437\x5cu04\
30\x5cu0442\x5cu043e\x5cu\
0440xdto \x5cu0441\x5c\
u043f\x5cu0440\x5cu043\
0\x5cu0432\x5cu043e\x5cu0\
447\x5cu043d\x5cu0438\x5c\
u043a\x5cu0438 \x5cu04\
41\x5cu0440\x5cu0435\x5cu\
0434\x5cu0441\x5cu0442\
\x5cu0432\x5cu0430\x5cu04\
33\x5cu0435\x5cu043e\x5cu\
043f\x5cu043e\x5cu0437\
\x5cu0438\x5cu0446\x5cu04\
38\x5cu043e\x5cu043d\x5cu\
0438\x5cu0440\x5cu043e\
\x5cu0432\x5cu0430\x5cu04\
3d\x5cu0438\x5cu044f \x5c\
u0441\x5cu0440\x5cu043\
5\x5cu0434\x5cu0441\x5cu0\
442\x5cu0432\x5cu0430\x5c\
u043a\x5cu0440\x5cu043\
8\x5cu043f\x5cu0442\x5cu0\
43e\x5cu0433\x5cu0440\x5c\
u0430\x5cu0444\x5cu043\
8\x5cu0438 \x5cu0441\x5cu\
0440\x5cu0435\x5cu0434\
\x5cu0441\x5cu0442\x5cu04\
32\x5cu0430\x5cu043c\x5cu\
0443\x5cu043b\x5cu044c\
\x5cu0442\x5cu0438\x5cu04\
3c\x5cu0435\x5cu0434\x5cu\
0438\x5cu0430 \x5cu044\
1\x5cu0440\x5cu0435\x5cu0\
434\x5cu0441\x5cu0442\x5c\
u0432\x5cu0430\x5cu043\
e\x5cu0442\x5cu043e\x5cu0\
431\x5cu0440\x5cu0430\x5c\
u0436\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f\x5cu0\
440\x5cu0435\x5cu043a\x5c\
u043b\x5cu0430\x5cu043\
c\x5cu044b \x5cu0441\x5cu\
0440\x5cu0435\x5cu0434\
\x5cu0441\x5cu0442\x5cu04\
32\x5cu0430\x5cu043f\x5cu\
043e\x5cu0447\x5cu0442\
\x5cu044b \x5cu0441\x5cu0\
440\x5cu0435\x5cu0434\x5c\
u0441\x5cu0442\x5cu043\
2\x5cu0430\x5cu0442\x5cu0\
435\x5cu043b\x5cu0435\x5c\
u0444\x5cu043e\x5cu043\
d\x5cu0438\x5cu0438 \x5cu\
0444\x5cu0430\x5cu0431\
\x5cu0440\x5cu0438\x5cu04\
3a\x5cu0430xdto \x5cu0\
444\x5cu0430\x5cu0439\x5c\
u043b\x5cu043e\x5cu043\
2\x5cu044b\x5cu0435\x5cu0\
43f\x5cu043e\x5cu0442\x5c\
u043e\x5cu043a\x5cu043\
8 \x5cu0444\x5cu043e\x5cu\
043d\x5cu043e\x5cu0432\
\x5cu044b\x5cu0435\x5cu04\
37\x5cu0430\x5cu0434\x5cu\
0430\x5cu043d\x5cu0438\
\x5cu044f \x5cu0445\x5cu0\
440\x5cu0430\x5cu043d\x5c\
u0438\x5cu043b\x5cu043\
8\x5cu0449\x5cu0430\x5cu0\
43d\x5cu0430\x5cu0441\x5c\
u0442\x5cu0440\x5cu043\
e\x5cu0435\x5cu043a \x5cu\
0445\x5cu0440\x5cu0430\
\x5cu043d\x5cu0438\x5cu04\
3b\x5cu0438\x5cu0449\x5cu\
0435\x5cu0432\x5cu0430\
\x5cu0440\x5cu0438\x5cu04\
30\x5cu043d\x5cu0442\x5cu\
043e\x5cu0432\x5cu043e\
\x5cu0442\x5cu0447\x5cu04\
35\x5cu0442\x5cu043e\x5cu\
0432 \x5cu0445\x5cu044\
0\x5cu0430\x5cu043d\x5cu0\
438\x5cu043b\x5cu0438\x5c\
u0449\x5cu0435\x5cu043\
d\x5cu0430\x5cu0441\x5cu0\
442\x5cu0440\x5cu043e\x5c\
u0435\x5cu043a\x5cu043\
4\x5cu0430\x5cu043d\x5cu0\
43d\x5cu044b\x5cu0445\x5c\
u0444\x5cu043e\x5cu044\
0\x5cu043c \x5cu0445\x5cu\
0440\x5cu0430\x5cu043d\
\x5cu0438\x5cu043b\x5cu04\
38\x5cu0449\x5cu0435\x5cu\
043e\x5cu0431\x5cu0449\
\x5cu0438\x5cu0445\x5cu04\
3d\x5cu0430\x5cu0441\x5cu\
0442\x5cu0440\x5cu043e\
\x5cu0435\x5cu043a \x5cu0\
445\x5cu0440\x5cu0430\x5c\
u043d\x5cu0438\x5cu043\
b\x5cu0438\x5cu0449\x5cu0\
435\x5cu043f\x5cu043e\x5c\
u043b\x5cu044c\x5cu043\
7\x5cu043e\x5cu0432\x5cu0\
430\x5cu0442\x5cu0435\x5c\
u043b\x5cu044c\x5cu044\
1\x5cu043a\x5cu0438\x5cu0\
445\x5cu043d\x5cu0430\x5c\
u0441\x5cu0442\x5cu044\
0\x5cu043e\x5cu0435\x5cu0\
43a\x5cu0434\x5cu0438\x5c\
u043d\x5cu0430\x5cu043\
c\x5cu0438\x5cu0447\x5cu0\
435\x5cu0441\x5cu043a\x5c\
u0438\x5cu0445\x5cu044\
1\x5cu043f\x5cu0438\x5cu0\
441\x5cu043a\x5cu043e\x5c\
u0432 \x5cu0445\x5cu04\
40\x5cu0430\x5cu043d\x5cu\
0438\x5cu043b\x5cu0438\
\x5cu0449\x5cu0435\x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
044c\x5cu0437\x5cu043e\
\x5cu0432\x5cu0430\x5cu04\
42\x5cu0435\x5cu043b\x5cu\
044c\x5cu0441\x5cu043a\
\x5cu0438\x5cu0445\x5cu04\
3d\x5cu0430\x5cu0441\x5cu\
0442\x5cu0440\x5cu043e\
\x5cu0435\x5cu043a\x5cu04\
3e\x5cu0442\x5cu0447\x5cu\
0435\x5cu0442\x5cu043e\
\x5cu0432 \x5cu0445\x5cu0\
440\x5cu0430\x5cu043d\x5c\
u0438\x5cu043b\x5cu043\
8\x5cu0449\x5cu0435\x5cu0\
441\x5cu0438\x5cu0441\x5c\
u0442\x5cu0435\x5cu043\
c\x5cu043d\x5cu044b\x5cu0\
445\x5cu043d\x5cu0430\x5c\
u0441\x5cu0442\x5cu044\
0\x5cu043e\x5cu0435\x5cu0\
43a \x22,\x0aclass:\x22we\
b\x5cu0446\x5cu0432\x5cu0\
435\x5cu0442\x5cu0430 \
windows\x5cu0446\x5cu0\
432\x5cu0435\x5cu0442\x5c\
u0430 windows\x5cu0\
448\x5cu0440\x5cu0438\x5c\
u0444\x5cu0442\x5cu044\
b \x5cu0431\x5cu0438\x5cu\
0431\x5cu043b\x5cu0438\
\x5cu043e\x5cu0442\x5cu04\
35\x5cu043a\x5cu0430\x5cu\
043a\x5cu0430\x5cu0440\
\x5cu0442\x5cu0438\x5cu04\
3d\x5cu043e\x5cu043a \x5c\
u0440\x5cu0430\x5cu043\
c\x5cu043a\x5cu0438\x5cu0\
441\x5cu0442\x5cu0438\x5c\
u043b\x5cu044f \x5cu04\
41\x5cu0438\x5cu043c\x5cu\
0432\x5cu043e\x5cu043b\
\x5cu044b \x5cu0446\x5cu0\
432\x5cu0435\x5cu0442\x5c\
u0430\x5cu0441\x5cu044\
2\x5cu0438\x5cu043b\x5cu0\
44f \x5cu0448\x5cu0440\
\x5cu0438\x5cu0444\x5cu04\
42\x5cu044b\x5cu0441\x5cu\
0442\x5cu0438\x5cu043b\
\x5cu044f \x5cu0430\x5cu0\
432\x5cu0442\x5cu043e\x5c\
u043c\x5cu0430\x5cu044\
2\x5cu0438\x5cu0447\x5cu0\
435\x5cu0441\x5cu043a\x5c\
u043e\x5cu0435\x5cu044\
1\x5cu043e\x5cu0445\x5cu0\
440\x5cu0430\x5cu043d\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0445\x5cu044\
4\x5cu043e\x5cu0440\x5cu0\
43c\x5cu044b\x5cu0432\x5c\
u043d\x5cu0430\x5cu044\
1\x5cu0442\x5cu0440\x5cu0\
43e\x5cu0439\x5cu043a\x5c\
u0430\x5cu0445 \x5cu04\
30\x5cu0432\x5cu0442\x5cu\
043e\x5cu043d\x5cu0443\
\x5cu043c\x5cu0435\x5cu04\
40\x5cu0430\x5cu0446\x5cu\
0438\x5cu044f\x5cu0432\
\x5cu0444\x5cu043e\x5cu04\
40\x5cu043c\x5cu0435 \x5c\
u0430\x5cu0432\x5cu044\
2\x5cu043e\x5cu0440\x5cu0\
430\x5cu0437\x5cu0434\x5c\
u0432\x5cu0438\x5cu043\
6\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu0441\x5c\
u0435\x5cu0440\x5cu043\
8\x5cu0439 \x5cu0430\x5cu\
043d\x5cu0438\x5cu043c\
\x5cu0430\x5cu0446\x5cu04\
38\x5cu044f\x5cu0434\x5cu\
0438\x5cu0430\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
3c\x5cu043c\x5cu044b \x5c\
u0432\x5cu0430\x5cu044\
0\x5cu0438\x5cu0430\x5cu0\
43d\x5cu0442\x5cu0432\x5c\
u044b\x5cu0440\x5cu043\
0\x5cu0432\x5cu043d\x5cu0\
438\x5cu0432\x5cu0430\x5c\
u043d\x5cu0438\x5cu044\
f\x5cu044d\x5cu043b\x5cu0\
435\x5cu043c\x5cu0435\x5c\
u043d\x5cu0442\x5cu043\
e\x5cu0432\x5cu0438\x5cu0\
437\x5cu0430\x5cu0433\x5c\
u043e\x5cu043b\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
43e\x5cu0432 \x5cu0432\
\x5cu0430\x5cu0440\x5cu04\
38\x5cu0430\x5cu043d\x5cu\
0442\x5cu0443\x5cu043f\
\x5cu0440\x5cu0430\x5cu04\
32\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu0432\x5cu044b\x5cu04\
41\x5cu043e\x5cu0442\x5cu\
043e\x5cu0439\x5cu0442\
\x5cu0430\x5cu0431\x5cu04\
3b\x5cu0438\x5cu0446\x5cu\
044b \x5cu0432\x5cu043\
5\x5cu0440\x5cu0442\x5cu0\
438\x5cu043a\x5cu0430\x5c\
u043b\x5cu044c\x5cu043\
d\x5cu0430\x5cu044f\x5cu0\
43f\x5cu0440\x5cu043e\x5c\
u043a\x5cu0440\x5cu044\
3\x5cu0442\x5cu043a\x5cu0\
430\x5cu0444\x5cu043e\x5c\
u0440\x5cu043c\x5cu044\
b \x5cu0432\x5cu0435\x5cu\
0440\x5cu0442\x5cu0438\
\x5cu043a\x5cu0430\x5cu04\
3b\x5cu044c\x5cu043d\x5cu\
043e\x5cu0435\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
3e\x5cu0436\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
 \x5cu0432\x5cu0435\x5cu0\
440\x5cu0442\x5cu0438\x5c\
u043a\x5cu0430\x5cu043\
b\x5cu044c\x5cu043d\x5cu0\
43e\x5cu0435\x5cu043f\x5c\
u043e\x5cu043b\x5cu043\
e\x5cu0436\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u044d\x5cu043b\x5cu043\
5\x5cu043c\x5cu0435\x5cu0\
43d\x5cu0442\x5cu0430 \
\x5cu0432\x5cu0438\x5cu04\
34\x5cu0433\x5cu0440\x5cu\
0443\x5cu043f\x5cu043f\
\x5cu044b\x5cu0444\x5cu04\
3e\x5cu0440\x5cu043c\x5cu\
044b \x5cu0432\x5cu043\
8\x5cu0434\x5cu0434\x5cu0\
435\x5cu043a\x5cu043e\x5c\
u0440\x5cu0430\x5cu044\
6\x5cu0438\x5cu0438\x5cu0\
444\x5cu043e\x5cu0440\x5c\
u043c\x5cu044b \x5cu04\
32\x5cu0438\x5cu0434\x5cu\
0434\x5cu043e\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
3d\x5cu0435\x5cu043d\x5cu\
0438\x5cu044f\x5cu044d\
\x5cu043b\x5cu0435\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
0442\x5cu0430\x5cu0444\
\x5cu043e\x5cu0440\x5cu04\
3c\x5cu044b \x5cu0432\x5c\
u0438\x5cu0434\x5cu043\
8\x5cu0437\x5cu043c\x5cu0\
435\x5cu043d\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu0432\x5cu04\
38\x5cu0434\x5cu043a\x5cu\
043d\x5cu043e\x5cu043f\
\x5cu043a\x5cu0438\x5cu04\
44\x5cu043e\x5cu0440\x5cu\
043c\x5cu044b \x5cu043\
2\x5cu0438\x5cu0434\x5cu0\
43f\x5cu0435\x5cu0440\x5c\
u0435\x5cu043a\x5cu043\
b\x5cu044e\x5cu0447\x5cu0\
430\x5cu0442\x5cu0435\x5c\
u043b\x5cu044f \x5cu04\
32\x5cu0438\x5cu0434\x5cu\
043f\x5cu043e\x5cu0434\
\x5cu043f\x5cu0438\x5cu04\
41\x5cu0435\x5cu0439\x5cu\
043a\x5cu0434\x5cu0438\
\x5cu0430\x5cu0433\x5cu04\
40\x5cu0430\x5cu043c\x5cu\
043c\x5cu0435 \x5cu043\
2\x5cu0438\x5cu0434\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u044f\x5cu0444\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
44b \x5cu0432\x5cu0438\
\x5cu0434\x5cu0444\x5cu04\
3b\x5cu0430\x5cu0436\x5cu\
043a\x5cu0430 \x5cu043\
2\x5cu043b\x5cu0438\x5cu0\
44f\x5cu043d\x5cu0438\x5c\
u0435\x5cu0440\x5cu043\
0\x5cu0437\x5cu043c\x5cu0\
435\x5cu0440\x5cu0430\x5c\
u043d\x5cu0430\x5cu043\
f\x5cu0443\x5cu0437\x5cu0\
44b\x5cu0440\x5cu0435\x5c\
u043a\x5cu0434\x5cu043\
8\x5cu0430\x5cu0433\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u043c\x5cu044b \x5cu04\
33\x5cu043e\x5cu0440\x5cu\
0438\x5cu0437\x5cu043e\
\x5cu043d\x5cu0442\x5cu04\
30\x5cu043b\x5cu044c\x5cu\
043d\x5cu043e\x5cu0435\
\x5cu043f\x5cu043e\x5cu04\
3b\x5cu043e\x5cu0436\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu0435 \x5cu0433\x5cu0\
43e\x5cu0440\x5cu0438\x5c\
u0437\x5cu043e\x5cu043\
d\x5cu0442\x5cu0430\x5cu0\
43b\x5cu044c\x5cu043d\x5c\
u043e\x5cu0435\x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
43e\x5cu0436\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu044d\x5cu043b\x5cu0\
435\x5cu043c\x5cu0435\x5c\
u043d\x5cu0442\x5cu043\
0 \x5cu0433\x5cu0440\x5cu\
0443\x5cu043f\x5cu043f\
\x5cu0438\x5cu0440\x5cu04\
3e\x5cu0432\x5cu043a\x5cu\
0430\x5cu043a\x5cu043e\
\x5cu043b\x5cu043e\x5cu04\
3d\x5cu043e\x5cu043a \x5c\
u0433\x5cu0440\x5cu044\
3\x5cu043f\x5cu043f\x5cu0\
438\x5cu0440\x5cu043e\x5c\
u0432\x5cu043a\x5cu043\
0\x5cu043f\x5cu043e\x5cu0\
434\x5cu0447\x5cu0438\x5c\
u043d\x5cu0435\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445\x5cu044d\x5cu043b\x5c\
u0435\x5cu043c\x5cu043\
5\x5cu043d\x5cu0442\x5cu0\
43e\x5cu0432\x5cu0444\x5c\
u043e\x5cu0440\x5cu043\
c\x5cu044b \x5cu0433\x5cu\
0440\x5cu0443\x5cu043f\
\x5cu043f\x5cu044b\x5cu04\
38\x5cu044d\x5cu043b\x5cu\
0435\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
4b \x5cu0434\x5cu0435\x5c\
u0439\x5cu0441\x5cu044\
2\x5cu0432\x5cu0438\x5cu0\
435\x5cu043f\x5cu0435\x5c\
u0440\x5cu0435\x5cu044\
2\x5cu0430\x5cu0441\x5cu0\
43a\x5cu0438\x5cu0432\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu044f \x5cu0434\x5cu\
043e\x5cu043f\x5cu043e\
\x5cu043b\x5cu043d\x5cu04\
38\x5cu0442\x5cu0435\x5cu\
043b\x5cu044c\x5cu043d\
\x5cu044b\x5cu0439\x5cu04\
40\x5cu0435\x5cu0436\x5cu\
0438\x5cu043c\x5cu043e\
\x5cu0442\x5cu043e\x5cu04\
31\x5cu0440\x5cu0430\x5cu\
0436\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f \x5cu0\
434\x5cu043e\x5cu043f\x5c\
u0443\x5cu0441\x5cu044\
2\x5cu0438\x5cu043c\x5cu0\
44b\x5cu0435\x5cu0434\x5c\
u0435\x5cu0439\x5cu044\
1\x5cu0442\x5cu0432\x5cu0\
438\x5cu044f\x5cu043f\x5c\
u0435\x5cu0440\x5cu043\
5\x5cu0442\x5cu0430\x5cu0\
441\x5cu043a\x5cu0438\x5c\
u0432\x5cu0430\x5cu043\
d\x5cu0438\x5cu044f \x5cu\
0438\x5cu043d\x5cu0442\
\x5cu0435\x5cu0440\x5cu04\
32\x5cu0430\x5cu043b\x5cu\
043c\x5cu0435\x5cu0436\
\x5cu0434\x5cu0443\x5cu04\
4d\x5cu043b\x5cu0435\x5cu\
043c\x5cu0435\x5cu043d\
\x5cu0442\x5cu0430\x5cu04\
3c\x5cu0438\x5cu0444\x5cu\
043e\x5cu0440\x5cu043c\
\x5cu044b \x5cu0438\x5cu0\
441\x5cu043f\x5cu043e\x5c\
u043b\x5cu044c\x5cu043\
7\x5cu043e\x5cu0432\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u0435\x5cu0432\x5cu044\
b\x5cu0432\x5cu043e\x5cu0\
434\x5cu0430 \x5cu0438\
\x5cu0441\x5cu043f\x5cu04\
3e\x5cu043b\x5cu044c\x5cu\
0437\x5cu043e\x5cu0432\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu0435\x5cu043f\x5cu\
043e\x5cu043b\x5cu043e\
\x5cu0441\x5cu044b\x5cu04\
3f\x5cu0440\x5cu043e\x5cu\
043a\x5cu0440\x5cu0443\
\x5cu0442\x5cu043a\x5cu04\
38 \x5cu0438\x5cu0441\x5c\
u043f\x5cu043e\x5cu043\
b\x5cu044c\x5cu0437\x5cu0\
443\x5cu0435\x5cu043c\x5c\
u043e\x5cu0435\x5cu043\
7\x5cu043d\x5cu0430\x5cu0\
447\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu044\
2\x5cu043e\x5cu0447\x5cu0\
43a\x5cu0438\x5cu0431\x5c\
u0438\x5cu0440\x5cu043\
6\x5cu0435\x5cu0432\x5cu0\
43e\x5cu0439\x5cu0434\x5c\
u0438\x5cu0430\x5cu043\
3\x5cu0440\x5cu0430\x5cu0\
43c\x5cu043c\x5cu044b \
\x5cu0438\x5cu0441\x5cu04\
42\x5cu043e\x5cu0440\x5cu\
0438\x5cu044f\x5cu0432\
\x5cu044b\x5cu0431\x5cu04\
3e\x5cu0440\x5cu0430\x5cu\
043f\x5cu0440\x5cu0438\
\x5cu0432\x5cu0432\x5cu04\
3e\x5cu0434\x5cu0435 \x5c\
u0438\x5cu0441\x5cu044\
2\x5cu043e\x5cu0447\x5cu0\
43d\x5cu0438\x5cu043a\x5c\
u0437\x5cu043d\x5cu043\
0\x5cu0447\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0439\x5c\
u043e\x5cu0441\x5cu043\
8\x5cu0442\x5cu043e\x5cu0\
447\x5cu0435\x5cu043a\x5c\
u0434\x5cu0438\x5cu043\
0\x5cu0433\x5cu0440\x5cu0\
430\x5cu043c\x5cu043c\x5c\
u044b \x5cu0438\x5cu04\
41\x5cu0442\x5cu043e\x5cu\
0447\x5cu043d\x5cu0438\
\x5cu043a\x5cu0437\x5cu04\
3d\x5cu0430\x5cu0447\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu0440\x5cu04\
30\x5cu0437\x5cu043c\x5cu\
0435\x5cu0440\x5cu0430\
\x5cu043f\x5cu0443\x5cu04\
37\x5cu044b\x5cu0440\x5cu\
044c\x5cu043a\x5cu0430\
\x5cu0434\x5cu0438\x5cu04\
30\x5cu0433\x5cu0440\x5cu\
0430\x5cu043c\x5cu043c\
\x5cu044b \x5cu043a\x5cu0\
430\x5cu0442\x5cu0435\x5c\
u0433\x5cu043e\x5cu044\
0\x5cu0438\x5cu044f\x5cu0\
433\x5cu0440\x5cu0443\x5c\
u043f\x5cu043f\x5cu044\
b\x5cu043a\x5cu043e\x5cu0\
43c\x5cu0430\x5cu043d\x5c\
u0434 \x5cu043c\x5cu04\
30\x5cu043a\x5cu0441\x5cu\
0438\x5cu043c\x5cu0443\
\x5cu043c\x5cu0441\x5cu04\
35\x5cu0440\x5cu0438\x5cu\
0439 \x5cu043d\x5cu043\
0\x5cu0447\x5cu0430\x5cu0\
43b\x5cu044c\x5cu043d\x5c\
u043e\x5cu0435\x5cu043\
e\x5cu0442\x5cu043e\x5cu0\
431\x5cu0440\x5cu0430\x5c\
u0436\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
434\x5cu0435\x5cu0440\x5c\
u0435\x5cu0432\x5cu043\
0 \x5cu043d\x5cu0430\x5cu\
0447\x5cu0430\x5cu043b\
\x5cu044c\x5cu043d\x5cu04\
3e\x5cu0435\x5cu043e\x5cu\
0442\x5cu043e\x5cu0431\
\x5cu0440\x5cu0430\x5cu04\
36\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu0441\
\x5cu043f\x5cu0438\x5cu04\
41\x5cu043a\x5cu0430 \x5c\
u043e\x5cu0431\x5cu043\
d\x5cu043e\x5cu0432\x5cu0\
43b\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu044\
2\x5cu0435\x5cu043a\x5cu0\
441\x5cu0442\x5cu0430\x5c\
u0440\x5cu0435\x5cu043\
4\x5cu0430\x5cu043a\x5cu0\
442\x5cu0438\x5cu0440\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
44f \x5cu043e\x5cu0440\
\x5cu0438\x5cu0435\x5cu04\
3d\x5cu0442\x5cu0430\x5cu\
0446\x5cu0438\x5cu044f\
\x5cu0434\x5cu0435\x5cu04\
3d\x5cu0434\x5cu0440\x5cu\
043e\x5cu0433\x5cu0440\
\x5cu0430\x5cu043c\x5cu04\
3c\x5cu044b \x5cu043e\x5c\
u0440\x5cu0438\x5cu043\
5\x5cu043d\x5cu0442\x5cu0\
430\x5cu0446\x5cu0438\x5c\
u044f\x5cu0434\x5cu043\
8\x5cu0430\x5cu0433\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u043c\x5cu044b \x5cu04\
3e\x5cu0440\x5cu0438\x5cu\
0435\x5cu043d\x5cu0442\
\x5cu0430\x5cu0446\x5cu04\
38\x5cu044f\x5cu043c\x5cu\
0435\x5cu0442\x5cu043e\
\x5cu043a\x5cu0434\x5cu04\
38\x5cu0430\x5cu0433\x5cu\
0440\x5cu0430\x5cu043c\
\x5cu043c\x5cu044b \x5cu0\
43e\x5cu0440\x5cu0438\x5c\
u0435\x5cu043d\x5cu044\
2\x5cu0430\x5cu0446\x5cu0\
438\x5cu044f\x5cu043c\x5c\
u0435\x5cu0442\x5cu043\
e\x5cu043a\x5cu0441\x5cu0\
432\x5cu043e\x5cu0434\x5c\
u043d\x5cu043e\x5cu043\
9\x5cu0434\x5cu0438\x5cu0\
430\x5cu0433\x5cu0440\x5c\
u0430\x5cu043c\x5cu043\
c\x5cu044b \x5cu043e\x5cu\
0440\x5cu0438\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30\x5cu0446\x5cu0438\x5cu\
044f\x5cu044d\x5cu043b\
\x5cu0435\x5cu043c\x5cu04\
35\x5cu043d\x5cu0442\x5cu\
0430\x5cu0444\x5cu043e\
\x5cu0440\x5cu043c\x5cu04\
4b \x5cu043e\x5cu0442\x5c\
u043e\x5cu0431\x5cu044\
0\x5cu0430\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435\x5cu0432\x5cu043\
4\x5cu0438\x5cu0430\x5cu0\
433\x5cu0440\x5cu0430\x5c\
u043c\x5cu043c\x5cu043\
5 \x5cu043e\x5cu0442\x5cu\
043e\x5cu0431\x5cu0440\
\x5cu0430\x5cu0436\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu0432\x5cu043b\
\x5cu0435\x5cu0433\x5cu04\
35\x5cu043d\x5cu0434\x5cu\
0435\x5cu0434\x5cu0438\
\x5cu0430\x5cu0433\x5cu04\
40\x5cu0430\x5cu043c\x5cu\
043c\x5cu044b \x5cu043\
e\x5cu0442\x5cu043e\x5cu0\
431\x5cu0440\x5cu0430\x5c\
u0436\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
433\x5cu0440\x5cu0443\x5c\
u043f\x5cu043f\x5cu044\
b\x5cu043a\x5cu043d\x5cu0\
43e\x5cu043f\x5cu043e\x5c\
u043a \x5cu043e\x5cu04\
42\x5cu043e\x5cu0431\x5cu\
0440\x5cu0430\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0437\x5cu\
0430\x5cu0433\x5cu043e\
\x5cu043b\x5cu043e\x5cu04\
32\x5cu043a\x5cu0430\x5cu\
0448\x5cu043a\x5cu0430\
\x5cu043b\x5cu044b\x5cu04\
34\x5cu0438\x5cu0430\x5cu\
0433\x5cu0440\x5cu0430\
\x5cu043c\x5cu043c\x5cu04\
4b \x5cu043e\x5cu0442\x5c\
u043e\x5cu0431\x5cu044\
0\x5cu0430\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435\x5cu0437\x5cu043\
d\x5cu0430\x5cu0447\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0439\x5cu0441\x5cu043\
2\x5cu043e\x5cu0434\x5cu0\
43d\x5cu043e\x5cu0439\x5c\
u0434\x5cu0438\x5cu043\
0\x5cu0433\x5cu0440\x5cu0\
430\x5cu043c\x5cu043c\x5c\
u044b \x5cu043e\x5cu04\
42\x5cu043e\x5cu0431\x5cu\
0440\x5cu0430\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0437\x5cu\
043d\x5cu0430\x5cu0447\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f\x5cu0438\x5cu\
0437\x5cu043c\x5cu0435\
\x5cu0440\x5cu0438\x5cu04\
42\x5cu0435\x5cu043b\x5cu\
044c\x5cu043d\x5cu043e\
\x5cu0439\x5cu0434\x5cu04\
38\x5cu0430\x5cu0433\x5cu\
0440\x5cu0430\x5cu043c\
\x5cu043c\x5cu044b \x5cu0\
43e\x5cu0442\x5cu043e\x5c\
u0431\x5cu0440\x5cu043\
0\x5cu0436\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0438\x5cu043d\x5cu044\
2\x5cu0435\x5cu0440\x5cu0\
432\x5cu0430\x5cu043b\x5c\
u0430\x5cu0434\x5cu043\
8\x5cu0430\x5cu0433\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u043c\x5cu044b\x5cu043\
3\x5cu0430\x5cu043d\x5cu0\
442\x5cu0430 \x5cu043e\
\x5cu0442\x5cu043e\x5cu04\
31\x5cu0440\x5cu0430\x5cu\
0436\x5cu0435\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
3a\x5cu043d\x5cu043e\x5cu\
043f\x5cu043a\x5cu0438\
 \x5cu043e\x5cu0442\x5cu0\
43e\x5cu0431\x5cu0440\x5c\
u0430\x5cu0436\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu043a\x5cu043d\x5c\
u043e\x5cu043f\x5cu043\
a\x5cu0438\x5cu0432\x5cu0\
44b\x5cu0431\x5cu043e\x5c\
u0440\x5cu0430 \x5cu04\
3e\x5cu0442\x5cu043e\x5cu\
0431\x5cu0440\x5cu0430\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
043e\x5cu0431\x5cu0441\
\x5cu0443\x5cu0436\x5cu04\
34\x5cu0435\x5cu043d\x5cu\
0438\x5cu0439\x5cu0444\
\x5cu043e\x5cu0440\x5cu04\
3c\x5cu044b \x5cu043e\x5c\
u0442\x5cu043e\x5cu043\
1\x5cu0440\x5cu0430\x5cu0\
436\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu043\
e\x5cu0431\x5cu044b\x5cu0\
447\x5cu043d\x5cu043e\x5c\
u0439\x5cu0433\x5cu044\
0\x5cu0443\x5cu043f\x5cu0\
43f\x5cu044b \x5cu043e\
\x5cu0442\x5cu043e\x5cu04\
31\x5cu0440\x5cu0430\x5cu\
0436\x5cu0435\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
3e\x5cu0442\x5cu0440\x5cu\
0438\x5cu0446\x5cu0430\
\x5cu0442\x5cu0435\x5cu04\
3b\x5cu044c\x5cu043d\x5cu\
044b\x5cu0445\x5cu0437\
\x5cu043d\x5cu0430\x5cu04\
47\x5cu0435\x5cu043d\x5cu\
0438\x5cu0439\x5cu043f\
\x5cu0443\x5cu0437\x5cu04\
4b\x5cu0440\x5cu044c\x5cu\
043a\x5cu043e\x5cu0432\
\x5cu043e\x5cu0439\x5cu04\
34\x5cu0438\x5cu0430\x5cu\
0433\x5cu0440\x5cu0430\
\x5cu043c\x5cu043c\x5cu04\
4b \x5cu043e\x5cu0442\x5c\
u043e\x5cu0431\x5cu044\
0\x5cu0430\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435\x5cu043f\x5cu043\
0\x5cu043d\x5cu0435\x5cu0\
43b\x5cu0438\x5cu043f\x5c\
u043e\x5cu0438\x5cu044\
1\x5cu043a\x5cu0430 \x5cu\
043e\x5cu0442\x5cu043e\
\x5cu0431\x5cu0440\x5cu04\
30\x5cu0436\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu043f\x5cu043e\x5cu04\
34\x5cu0441\x5cu043a\x5cu\
0430\x5cu0437\x5cu043a\
\x5cu0438 \x5cu043e\x5cu0\
442\x5cu043e\x5cu0431\x5c\
u0440\x5cu0430\x5cu043\
6\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu043f\x5c\
u0440\x5cu0435\x5cu043\
4\x5cu0443\x5cu043f\x5cu0\
440\x5cu0435\x5cu0436\x5c\
u0434\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f\x5cu0\
43f\x5cu0440\x5cu0438\x5c\
u0440\x5cu0435\x5cu043\
4\x5cu0430\x5cu043a\x5cu0\
442\x5cu0438\x5cu0440\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
438 \x5cu043e\x5cu0442\
\x5cu043e\x5cu0431\x5cu04\
40\x5cu0430\x5cu0436\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu0435\x5cu0440\x5cu04\
30\x5cu0437\x5cu043c\x5cu\
0435\x5cu0442\x5cu043a\
\x5cu0438\x5cu043f\x5cu04\
3e\x5cu043b\x5cu043e\x5cu\
0441\x5cu044b\x5cu0440\
\x5cu0435\x5cu0433\x5cu04\
43\x5cu043b\x5cu0438\x5cu\
0440\x5cu043e\x5cu0432\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu044f \x5cu043e\x5c\
u0442\x5cu043e\x5cu043\
1\x5cu0440\x5cu0430\x5cu0\
436\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu044\
1\x5cu0442\x5cu0440\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u0446\x5cu0444\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
44b \x5cu043e\x5cu0442\
\x5cu043e\x5cu0431\x5cu04\
40\x5cu0430\x5cu0436\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu0435\x5cu0442\x5cu04\
30\x5cu0431\x5cu043b\x5cu\
0438\x5cu0446\x5cu044b\
 \x5cu043e\x5cu0442\x5cu0\
43e\x5cu0431\x5cu0440\x5c\
u0430\x5cu0436\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu0442\x5cu0435\x5c\
u043a\x5cu0441\x5cu044\
2\x5cu0430\x5cu0437\x5cu0\
43d\x5cu0430\x5cu0447\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f\x5cu0434\x5cu0\
438\x5cu0430\x5cu0433\x5c\
u0440\x5cu0430\x5cu043\
c\x5cu043c\x5cu044b\x5cu0\
433\x5cu0430\x5cu043d\x5c\
u0442\x5cu0430 \x5cu04\
3e\x5cu0442\x5cu043e\x5cu\
0431\x5cu0440\x5cu0430\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
0443\x5cu043f\x5cu0440\
\x5cu0430\x5cu0432\x5cu04\
3b\x5cu0435\x5cu043d\x5cu\
0438\x5cu044f\x5cu043e\
\x5cu0431\x5cu044b\x5cu04\
47\x5cu043d\x5cu043e\x5cu\
0439\x5cu0433\x5cu0440\
\x5cu0443\x5cu043f\x5cu04\
3f\x5cu044b \x5cu043e\x5c\
u0442\x5cu043e\x5cu043\
1\x5cu0440\x5cu0430\x5cu0\
436\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu044\
4\x5cu0438\x5cu0433\x5cu0\
443\x5cu0440\x5cu044b\x5c\
u043a\x5cu043d\x5cu043\
e\x5cu043f\x5cu043a\x5cu0\
438 \x5cu043f\x5cu0430\
\x5cu043b\x5cu0438\x5cu04\
42\x5cu0440\x5cu0430\x5cu\
0446\x5cu0432\x5cu0435\
\x5cu0442\x5cu043e\x5cu04\
32\x5cu0434\x5cu0438\x5cu\
0430\x5cu0433\x5cu0440\
\x5cu0430\x5cu043c\x5cu04\
3c\x5cu044b \x5cu043f\x5c\
u043e\x5cu0432\x5cu043\
5\x5cu0434\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u043e\x5cu0431\x5cu044\
b\x5cu0447\x5cu043d\x5cu0\
43e\x5cu0439\x5cu0433\x5c\
u0440\x5cu0443\x5cu043\
f\x5cu043f\x5cu044b \x5cu\
043f\x5cu043e\x5cu0434\
\x5cu0434\x5cu0435\x5cu04\
40\x5cu0436\x5cu043a\x5cu\
0430\x5cu043c\x5cu0430\
\x5cu0441\x5cu0448\x5cu04\
42\x5cu0430\x5cu0431\x5cu\
0430\x5cu0434\x5cu0435\
\x5cu043d\x5cu0434\x5cu04\
40\x5cu043e\x5cu0433\x5cu\
0440\x5cu0430\x5cu043c\
\x5cu043c\x5cu044b \x5cu0\
43f\x5cu043e\x5cu0434\x5c\
u0434\x5cu0435\x5cu044\
0\x5cu0436\x5cu043a\x5cu0\
430\x5cu043c\x5cu0430\x5c\
u0441\x5cu0448\x5cu044\
2\x5cu0430\x5cu0431\x5cu0\
430\x5cu0434\x5cu0438\x5c\
u0430\x5cu0433\x5cu044\
0\x5cu0430\x5cu043c\x5cu0\
43c\x5cu044b\x5cu0433\x5c\
u0430\x5cu043d\x5cu044\
2\x5cu0430 \x5cu043f\x5cu\
043e\x5cu0434\x5cu0434\
\x5cu0435\x5cu0440\x5cu04\
36\x5cu043a\x5cu0430\x5cu\
043c\x5cu0430\x5cu0441\
\x5cu0448\x5cu0442\x5cu04\
30\x5cu0431\x5cu0430\x5cu\
0441\x5cu0432\x5cu043e\
\x5cu0434\x5cu043d\x5cu04\
3e\x5cu0439\x5cu0434\x5cu\
0438\x5cu0430\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
3c\x5cu043c\x5cu044b \x5c\
u043f\x5cu043e\x5cu043\
8\x5cu0441\x5cu043a\x5cu0\
432\x5cu0442\x5cu0430\x5c\
u0431\x5cu043b\x5cu043\
8\x5cu0446\x5cu0435\x5cu0\
43f\x5cu0440\x5cu0438\x5c\
u0432\x5cu0432\x5cu043\
e\x5cu0434\x5cu0435 \x5cu\
043f\x5cu043e\x5cu043b\
\x5cu043e\x5cu0436\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu0437\x5cu0430\
\x5cu0433\x5cu043e\x5cu04\
3b\x5cu043e\x5cu0432\x5cu\
043a\x5cu0430\x5cu044d\
\x5cu043b\x5cu0435\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
0442\x5cu0430\x5cu0444\
\x5cu043e\x5cu0440\x5cu04\
3c\x5cu044b \x5cu043f\x5c\
u043e\x5cu043b\x5cu043\
e\x5cu0436\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u043a\x5cu0430\x5cu044\
0\x5cu0442\x5cu0438\x5cu0\
43d\x5cu043a\x5cu0438\x5c\
u043a\x5cu043d\x5cu043\
e\x5cu043f\x5cu043a\x5cu0\
438\x5cu0444\x5cu043e\x5c\
u0440\x5cu043c\x5cu044\
b \x5cu043f\x5cu043e\x5cu\
043b\x5cu043e\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu043a\x5cu\
0430\x5cu0440\x5cu0442\
\x5cu0438\x5cu043d\x5cu04\
3a\x5cu0438\x5cu044d\x5cu\
043b\x5cu0435\x5cu043c\
\x5cu0435\x5cu043d\x5cu04\
42\x5cu0430\x5cu0433\x5cu\
0440\x5cu0430\x5cu0444\
\x5cu0438\x5cu0447\x5cu04\
35\x5cu0441\x5cu043a\x5cu\
043e\x5cu0439\x5cu0441\
\x5cu0445\x5cu0435\x5cu04\
3c\x5cu044b \x5cu043f\x5c\
u043e\x5cu043b\x5cu043\
e\x5cu0436\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u043a\x5cu043e\x5cu043\
c\x5cu0430\x5cu043d\x5cu0\
434\x5cu043d\x5cu043e\x5c\
u0439\x5cu043f\x5cu043\
0\x5cu043d\x5cu0435\x5cu0\
43b\x5cu0438\x5cu0444\x5c\
u043e\x5cu0440\x5cu043\
c\x5cu044b \x5cu043f\x5cu\
043e\x5cu043b\x5cu043e\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
043a\x5cu043e\x5cu043c\
\x5cu0430\x5cu043d\x5cu04\
34\x5cu043d\x5cu043e\x5cu\
0439\x5cu043f\x5cu0430\
\x5cu043d\x5cu0435\x5cu04\
3b\x5cu0438\x5cu044d\x5cu\
043b\x5cu0435\x5cu043c\
\x5cu0435\x5cu043d\x5cu04\
42\x5cu0430\x5cu0444\x5cu\
043e\x5cu0440\x5cu043c\
\x5cu044b \x5cu043f\x5cu0\
43e\x5cu043b\x5cu043e\x5c\
u0436\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
43e\x5cu043f\x5cu043e\x5c\
u0440\x5cu043d\x5cu043\
e\x5cu0439\x5cu0442\x5cu0\
43e\x5cu0447\x5cu043a\x5c\
u0438\x5cu043e\x5cu044\
2\x5cu0440\x5cu0438\x5cu0\
441\x5cu043e\x5cu0432\x5c\
u043a\x5cu0438 \x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
043e\x5cu0436\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu043f\x5cu043e\x5cu\
0434\x5cu043f\x5cu0438\
\x5cu0441\x5cu0435\x5cu04\
39\x5cu043a\x5cu0434\x5cu\
0438\x5cu0430\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
3c\x5cu043c\x5cu0435 \x5c\
u043f\x5cu043e\x5cu043\
b\x5cu043e\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435\x5cu043f\x5cu043\
e\x5cu0434\x5cu043f\x5cu0\
438\x5cu0441\x5cu0435\x5c\
u0439\x5cu0448\x5cu043\
a\x5cu0430\x5cu043b\x5cu0\
44b\x5cu0437\x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
439\x5cu0438\x5cu0437\x5c\
u043c\x5cu0435\x5cu044\
0\x5cu0438\x5cu0442\x5cu0\
435\x5cu043b\x5cu044c\x5c\
u043d\x5cu043e\x5cu043\
9\x5cu0434\x5cu0438\x5cu0\
430\x5cu0433\x5cu0440\x5c\
u0430\x5cu043c\x5cu043\
c\x5cu044b \x5cu043f\x5cu\
043e\x5cu043b\x5cu043e\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
0441\x5cu043e\x5cu0441\
\x5cu0442\x5cu043e\x5cu04\
4f\x5cu043d\x5cu0438\x5cu\
044f\x5cu043f\x5cu0440\
\x5cu043e\x5cu0441\x5cu04\
3c\x5cu043e\x5cu0442\x5cu\
0440\x5cu0430 \x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
43e\x5cu0436\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu0441\x5cu0442\x5cu0\
440\x5cu043e\x5cu043a\x5c\
u0438\x5cu043f\x5cu043\
e\x5cu0438\x5cu0441\x5cu0\
43a\x5cu0430 \x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
3e\x5cu0436\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0442\x5cu0435\x5cu04\
3a\x5cu0441\x5cu0442\x5cu\
0430\x5cu0441\x5cu043e\
\x5cu0435\x5cu0434\x5cu04\
38\x5cu043d\x5cu0438\x5cu\
0442\x5cu0435\x5cu043b\
\x5cu044c\x5cu043d\x5cu04\
3e\x5cu0439\x5cu043b\x5cu\
0438\x5cu043d\x5cu0438\
\x5cu0438 \x5cu043f\x5cu0\
43e\x5cu043b\x5cu043e\x5c\
u0436\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
443\x5cu043f\x5cu0440\x5c\
u0430\x5cu0432\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu043f\x5c\
u043e\x5cu0438\x5cu044\
1\x5cu043a\x5cu043e\x5cu0\
43c \x5cu043f\x5cu043e\
\x5cu043b\x5cu043e\x5cu04\
36\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu0448\
\x5cu043a\x5cu0430\x5cu04\
3b\x5cu044b\x5cu0432\x5cu\
0440\x5cu0435\x5cu043c\
\x5cu0435\x5cu043d\x5cu04\
38 \x5cu043f\x5cu043e\x5c\
u0440\x5cu044f\x5cu043\
4\x5cu043e\x5cu043a\x5cu0\
43e\x5cu0442\x5cu043e\x5c\
u0431\x5cu0440\x5cu043\
0\x5cu0436\x5cu0435\x5cu0\
43d\x5cu0438\x5cu044f\x5c\
u0442\x5cu043e\x5cu044\
7\x5cu0435\x5cu043a\x5cu0\
433\x5cu043e\x5cu0440\x5c\
u0438\x5cu0437\x5cu043\
e\x5cu043d\x5cu0442\x5cu0\
430\x5cu043b\x5cu044c\x5c\
u043d\x5cu043e\x5cu043\
9\x5cu0433\x5cu0438\x5cu0\
441\x5cu0442\x5cu043e\x5c\
u0433\x5cu0440\x5cu043\
0\x5cu043c\x5cu043c\x5cu0\
44b \x5cu043f\x5cu043e\
\x5cu0440\x5cu044f\x5cu04\
34\x5cu043e\x5cu043a\x5cu\
0441\x5cu0435\x5cu0440\
\x5cu0438\x5cu0439\x5cu04\
32\x5cu043b\x5cu0435\x5cu\
0433\x5cu0435\x5cu043d\
\x5cu0434\x5cu0435\x5cu04\
34\x5cu0438\x5cu0430\x5cu\
0433\x5cu0440\x5cu0430\
\x5cu043c\x5cu043c\x5cu04\
4b \x5cu0440\x5cu0430\x5c\
u0437\x5cu043c\x5cu043\
5\x5cu0440\x5cu043a\x5cu0\
430\x5cu0440\x5cu0442\x5c\
u0438\x5cu043d\x5cu043\
a\x5cu0438 \x5cu0440\x5cu\
0430\x5cu0441\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
3e\x5cu0436\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0437\x5cu0430\x5cu04\
33\x5cu043e\x5cu043b\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0430\x5cu0448\x5cu04\
3a\x5cu0430\x5cu043b\x5cu\
044b\x5cu0434\x5cu0438\
\x5cu0430\x5cu0433\x5cu04\
40\x5cu0430\x5cu043c\x5cu\
043c\x5cu044b \x5cu044\
0\x5cu0430\x5cu0441\x5cu0\
442\x5cu044f\x5cu0433\x5c\
u0438\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
435\x5cu043f\x5cu043e\x5c\
u0432\x5cu0435\x5cu044\
0\x5cu0442\x5cu0438\x5cu0\
43a\x5cu0430\x5cu043b\x5c\
u0438\x5cu0434\x5cu043\
8\x5cu0430\x5cu0433\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u043c\x5cu044b\x5cu043\
3\x5cu0430\x5cu043d\x5cu0\
442\x5cu0430 \x5cu0440\
\x5cu0435\x5cu0436\x5cu04\
38\x5cu043c\x5cu0430\x5cu\
0432\x5cu0442\x5cu043e\
\x5cu043e\x5cu0442\x5cu04\
3e\x5cu0431\x5cu0440\x5cu\
0430\x5cu0436\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
4f\x5cu0441\x5cu043e\x5cu\
0441\x5cu0442\x5cu043e\
\x5cu044f\x5cu043d\x5cu04\
38\x5cu044f \x5cu0440\x5c\
u0435\x5cu0436\x5cu043\
8\x5cu043c\x5cu0432\x5cu0\
432\x5cu043e\x5cu0434\x5c\
u0430\x5cu0441\x5cu044\
2\x5cu0440\x5cu043e\x5cu0\
43a\x5cu0442\x5cu0430\x5c\
u0431\x5cu043b\x5cu043\
8\x5cu0446\x5cu044b \x5cu\
0440\x5cu0435\x5cu0436\
\x5cu0438\x5cu043c\x5cu04\
32\x5cu044b\x5cu0431\x5cu\
043e\x5cu0440\x5cu0430\
\x5cu043d\x5cu0435\x5cu04\
37\x5cu0430\x5cu043f\x5cu\
043e\x5cu043b\x5cu043d\
\x5cu0435\x5cu043d\x5cu04\
3d\x5cu043e\x5cu0433\x5cu\
043e \x5cu0440\x5cu043\
5\x5cu0436\x5cu0438\x5cu0\
43c\x5cu0432\x5cu044b\x5c\
u0434\x5cu0435\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu0434\x5c\
u0430\x5cu0442\x5cu044\
b \x5cu0440\x5cu0435\x5cu\
0436\x5cu0438\x5cu043c\
\x5cu0432\x5cu044b\x5cu04\
34\x5cu0435\x5cu043b\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu0441\x5cu04\
42\x5cu0440\x5cu043e\x5cu\
043a\x5cu0438\x5cu0442\
\x5cu0430\x5cu0431\x5cu04\
3b\x5cu0438\x5cu0446\x5cu\
044b \x5cu0440\x5cu043\
5\x5cu0436\x5cu0438\x5cu0\
43c\x5cu0432\x5cu044b\x5c\
u0434\x5cu0435\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu0442\x5c\
u0430\x5cu0431\x5cu043\
b\x5cu0438\x5cu0446\x5cu0\
44b \x5cu0440\x5cu0435\
\x5cu0436\x5cu0438\x5cu04\
3c\x5cu0438\x5cu0437\x5cu\
043c\x5cu0435\x5cu043d\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f\x5cu0440\x5cu\
0430\x5cu0437\x5cu043c\
\x5cu0435\x5cu0440\x5cu04\
30 \x5cu0440\x5cu0435\x5c\
u0436\x5cu0438\x5cu043\
c\x5cu0438\x5cu0437\x5cu0\
43c\x5cu0435\x5cu043d\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f\x5cu0441\x5cu0\
432\x5cu044f\x5cu0437\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu043e\x5cu0433\x5cu0\
43e\x5cu0437\x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f \x5cu0440\x5cu0435\
\x5cu0436\x5cu0438\x5cu04\
3c\x5cu0438\x5cu0441\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu044c\x5cu0437\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu0434\x5cu0438\x5cu04\
30\x5cu043b\x5cu043e\x5cu\
0433\x5cu0430\x5cu043f\
\x5cu0435\x5cu0447\x5cu04\
30\x5cu0442\x5cu0438 \x5c\
u0440\x5cu0435\x5cu043\
6\x5cu0438\x5cu043c\x5cu0\
438\x5cu0441\x5cu043f\x5c\
u043e\x5cu043b\x5cu044\
c\x5cu0437\x5cu043e\x5cu0\
432\x5cu0430\x5cu043d\x5c\
u0438\x5cu044f\x5cu043\
f\x5cu0430\x5cu0440\x5cu0\
430\x5cu043c\x5cu0435\x5c\
u0442\x5cu0440\x5cu043\
0\x5cu043a\x5cu043e\x5cu0\
43c\x5cu0430\x5cu043d\x5c\
u0434\x5cu044b \x5cu04\
40\x5cu0435\x5cu0436\x5cu\
0438\x5cu043c\x5cu043c\
\x5cu0430\x5cu0441\x5cu04\
48\x5cu0442\x5cu0430\x5cu\
0431\x5cu0438\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
044f\x5cu043f\x5cu0440\
\x5cu043e\x5cu0441\x5cu04\
3c\x5cu043e\x5cu0442\x5cu\
0440\x5cu0430 \x5cu044\
0\x5cu0435\x5cu0436\x5cu0\
438\x5cu043c\x5cu043e\x5c\
u0441\x5cu043d\x5cu043\
e\x5cu0432\x5cu043d\x5cu0\
43e\x5cu0433\x5cu043e\x5c\
u043e\x5cu043a\x5cu043\
d\x5cu0430\x5cu043a\x5cu0\
43b\x5cu0438\x5cu0435\x5c\
u043d\x5cu0442\x5cu044\
1\x5cu043a\x5cu043e\x5cu0\
433\x5cu043e\x5cu043f\x5c\
u0440\x5cu0438\x5cu043\
b\x5cu043e\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044f \x5cu0440\x5cu04\
35\x5cu0436\x5cu0438\x5cu\
043c\x5cu043e\x5cu0442\
\x5cu043a\x5cu0440\x5cu04\
4b\x5cu0442\x5cu0438\x5cu\
044f\x5cu043e\x5cu043a\
\x5cu043d\x5cu0430\x5cu04\
44\x5cu043e\x5cu0440\x5cu\
043c\x5cu044b \x5cu044\
0\x5cu0435\x5cu0436\x5cu0\
438\x5cu043c\x5cu043e\x5c\
u0442\x5cu043e\x5cu043\
1\x5cu0440\x5cu0430\x5cu0\
436\x5cu0435\x5cu043d\x5c\
u0438\x5cu044f\x5cu043\
2\x5cu044b\x5cu0434\x5cu0\
435\x5cu043b\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f \x5cu0440\x5cu0435\x5cu\
0436\x5cu0438\x5cu043c\
\x5cu043e\x5cu0442\x5cu04\
3e\x5cu0431\x5cu0440\x5cu\
0430\x5cu0436\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
4f\x5cu0433\x5cu0435\x5cu\
043e\x5cu0433\x5cu0440\
\x5cu0430\x5cu0444\x5cu04\
38\x5cu0447\x5cu0435\x5cu\
0441\x5cu043a\x5cu043e\
\x5cu0439\x5cu0441\x5cu04\
45\x5cu0435\x5cu043c\x5cu\
044b \x5cu0440\x5cu043\
5\x5cu0436\x5cu0438\x5cu0\
43c\x5cu043e\x5cu0442\x5c\
u043e\x5cu0431\x5cu044\
0\x5cu0430\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044f\x5cu0437\x5cu043\
d\x5cu0430\x5cu0447\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0439\x5cu0441\x5cu043\
5\x5cu0440\x5cu0438\x5cu0\
438 \x5cu0440\x5cu0435\
\x5cu0436\x5cu0438\x5cu04\
3c\x5cu043e\x5cu0442\x5cu\
0440\x5cu0438\x5cu0441\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu0438\x5cu0441\x5cu\
0435\x5cu0442\x5cu043a\
\x5cu0438\x5cu0433\x5cu04\
40\x5cu0430\x5cu0444\x5cu\
0438\x5cu0447\x5cu0435\
\x5cu0441\x5cu043a\x5cu04\
3e\x5cu0439\x5cu0441\x5cu\
0445\x5cu0435\x5cu043c\
\x5cu044b \x5cu0440\x5cu0\
435\x5cu0436\x5cu0438\x5c\
u043c\x5cu043f\x5cu043\
e\x5cu043b\x5cu0443\x5cu0\
43f\x5cu0440\x5cu043e\x5c\
u0437\x5cu0440\x5cu043\
0\x5cu0447\x5cu043d\x5cu0\
43e\x5cu0441\x5cu0442\x5c\
u0438\x5cu0434\x5cu043\
8\x5cu0430\x5cu0433\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u043c\x5cu044b \x5cu04\
40\x5cu0435\x5cu0436\x5cu\
0438\x5cu043c\x5cu043f\
\x5cu0440\x5cu043e\x5cu04\
31\x5cu0435\x5cu043b\x5cu\
043e\x5cu0432\x5cu0434\
\x5cu0438\x5cu0430\x5cu04\
33\x5cu0440\x5cu0430\x5cu\
043c\x5cu043c\x5cu044b\
 \x5cu0440\x5cu0435\x5cu0\
436\x5cu0438\x5cu043c\x5c\
u0440\x5cu0430\x5cu043\
7\x5cu043c\x5cu0435\x5cu0\
449\x5cu0435\x5cu043d\x5c\
u0438\x5cu044f\x5cu043\
d\x5cu0430\x5cu0441\x5cu0\
442\x5cu0440\x5cu0430\x5c\
u043d\x5cu0438\x5cu044\
6\x5cu0435 \x5cu0440\x5cu\
0435\x5cu0436\x5cu0438\
\x5cu043c\x5cu0440\x5cu04\
35\x5cu0434\x5cu0430\x5cu\
043a\x5cu0442\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0430\x5cu043d\x5cu\
0438\x5cu044f\x5cu043a\
\x5cu043e\x5cu043b\x5cu04\
3e\x5cu043d\x5cu043a\x5cu\
0438 \x5cu0440\x5cu043\
5\x5cu0436\x5cu0438\x5cu0\
43c\x5cu0441\x5cu0433\x5c\
u043b\x5cu0430\x5cu043\
6\x5cu0438\x5cu0432\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u044f\x5cu0434\x5cu043\
8\x5cu0430\x5cu0433\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u043c\x5cu044b \x5cu04\
40\x5cu0435\x5cu0436\x5cu\
0438\x5cu043c\x5cu0441\
\x5cu0433\x5cu043b\x5cu04\
30\x5cu0436\x5cu0438\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
38\x5cu043d\x5cu0434\x5cu\
0438\x5cu043a\x5cu0430\
\x5cu0442\x5cu043e\x5cu04\
40\x5cu0430 \x5cu0440\x5c\
u0435\x5cu0436\x5cu043\
8\x5cu043c\x5cu0441\x5cu0\
43f\x5cu0438\x5cu0441\x5c\
u043a\x5cu0430\x5cu043\
7\x5cu0430\x5cu0434\x5cu0\
430\x5cu0447 \x5cu0441\
\x5cu043a\x5cu0432\x5cu04\
3e\x5cu0437\x5cu043d\x5cu\
043e\x5cu0435\x5cu0432\
\x5cu044b\x5cu0440\x5cu04\
30\x5cu0432\x5cu043d\x5cu\
0438\x5cu0432\x5cu0430\
\x5cu043d\x5cu0438\x5cu04\
35 \x5cu0441\x5cu043e\x5c\
u0445\x5cu0440\x5cu043\
0\x5cu043d\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445\x5cu0444\x5cu043e\x5c\
u0440\x5cu043c\x5cu044\
b\x5cu0432\x5cu043d\x5cu0\
430\x5cu0441\x5cu0442\x5c\
u0440\x5cu043e\x5cu043\
9\x5cu043a\x5cu0430\x5cu0\
445 \x5cu0441\x5cu043f\
\x5cu043e\x5cu0441\x5cu04\
3e\x5cu0431\x5cu0437\x5cu\
0430\x5cu043f\x5cu043e\
\x5cu043b\x5cu043d\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f\x5cu0442\x5cu0435\
\x5cu043a\x5cu0441\x5cu04\
42\x5cu0430\x5cu0437\x5cu\
0430\x5cu0433\x5cu043e\
\x5cu043b\x5cu043e\x5cu04\
32\x5cu043a\x5cu0430\x5cu\
0448\x5cu043a\x5cu0430\
\x5cu043b\x5cu044b\x5cu04\
34\x5cu0438\x5cu0430\x5cu\
0433\x5cu0440\x5cu0430\
\x5cu043c\x5cu043c\x5cu04\
4b \x5cu0441\x5cu043f\x5c\
u043e\x5cu0441\x5cu043\
e\x5cu0431\x5cu043e\x5cu0\
43f\x5cu0440\x5cu0435\x5c\
u0434\x5cu0435\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu043e\x5c\
u0433\x5cu0440\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
447\x5cu0438\x5cu0432\x5c\
u0430\x5cu044e\x5cu044\
9\x5cu0435\x5cu0433\x5cu0\
43e\x5cu0437\x5cu043d\x5c\
u0430\x5cu0447\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f\x5cu0434\x5cu0438\x5c\
u0430\x5cu0433\x5cu044\
0\x5cu0430\x5cu043c\x5cu0\
43c\x5cu044b \x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
3d\x5cu0434\x5cu0430\x5cu\
0440\x5cu0442\x5cu043d\
\x5cu0430\x5cu044f\x5cu04\
33\x5cu0440\x5cu0443\x5cu\
043f\x5cu043f\x5cu0430\
\x5cu043a\x5cu043e\x5cu04\
3c\x5cu0430\x5cu043d\x5cu\
0434 \x5cu0441\x5cu044\
2\x5cu0430\x5cu043d\x5cu0\
434\x5cu0430\x5cu0440\x5c\
u0442\x5cu043d\x5cu043\
e\x5cu0435\x5cu043e\x5cu0\
444\x5cu043e\x5cu0440\x5c\
u043c\x5cu043b\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435 \x5cu0441\x5cu0442\
\x5cu0430\x5cu0442\x5cu04\
43\x5cu0441\x5cu043e\x5cu\
043f\x5cu043e\x5cu0432\
\x5cu0435\x5cu0449\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f\x5cu043f\x5cu043e\
\x5cu043b\x5cu044c\x5cu04\
37\x5cu043e\x5cu0432\x5cu\
0430\x5cu0442\x5cu0435\
\x5cu043b\x5cu044f \x5cu0\
441\x5cu0442\x5cu0438\x5c\
u043b\x5cu044c\x5cu044\
1\x5cu0442\x5cu0440\x5cu0\
435\x5cu043b\x5cu043a\x5c\
u0438 \x5cu0442\x5cu04\
38\x5cu043f\x5cu0430\x5cu\
043f\x5cu043f\x5cu0440\
\x5cu043e\x5cu043a\x5cu04\
41\x5cu0438\x5cu043c\x5cu\
0430\x5cu0446\x5cu0438\
\x5cu0438\x5cu043b\x5cu04\
38\x5cu043d\x5cu0438\x5cu\
0438\x5cu0442\x5cu0440\
\x5cu0435\x5cu043d\x5cu04\
34\x5cu0430\x5cu0434\x5cu\
0438\x5cu0430\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
3c\x5cu043c\x5cu044b \x5c\
u0442\x5cu0438\x5cu043\
f\x5cu0434\x5cu0438\x5cu0\
430\x5cu0433\x5cu0440\x5c\
u0430\x5cu043c\x5cu043\
c\x5cu044b \x5cu0442\x5cu\
0438\x5cu043f\x5cu0435\
\x5cu0434\x5cu0438\x5cu04\
3d\x5cu0438\x5cu0446\x5cu\
044b\x5cu0448\x5cu043a\
\x5cu0430\x5cu043b\x5cu04\
4b\x5cu0432\x5cu0440\x5cu\
0435\x5cu043c\x5cu0435\
\x5cu043d\x5cu0438 \x5cu0\
442\x5cu0438\x5cu043f\x5c\
u0438\x5cu043c\x5cu043\
f\x5cu043e\x5cu0440\x5cu0\
442\x5cu0430\x5cu0441\x5c\
u0435\x5cu0440\x5cu043\
8\x5cu0439\x5cu0441\x5cu0\
43b\x5cu043e\x5cu044f\x5c\
u0433\x5cu0435\x5cu043\
e\x5cu0433\x5cu0440\x5cu0\
430\x5cu0444\x5cu0438\x5c\
u0447\x5cu0435\x5cu044\
1\x5cu043a\x5cu043e\x5cu0\
439\x5cu0441\x5cu0445\x5c\
u0435\x5cu043c\x5cu044\
b \x5cu0442\x5cu0438\x5cu\
043f\x5cu043b\x5cu0438\
\x5cu043d\x5cu0438\x5cu04\
38\x5cu0433\x5cu0435\x5cu\
043e\x5cu0433\x5cu0440\
\x5cu0430\x5cu0444\x5cu04\
38\x5cu0447\x5cu0435\x5cu\
0441\x5cu043a\x5cu043e\
\x5cu0439\x5cu0441\x5cu04\
45\x5cu0435\x5cu043c\x5cu\
044b \x5cu0442\x5cu043\
8\x5cu043f\x5cu043b\x5cu0\
438\x5cu043d\x5cu0438\x5c\
u0438\x5cu0434\x5cu043\
8\x5cu0430\x5cu0433\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u043c\x5cu044b \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
043c\x5cu0430\x5cu0440\
\x5cu043a\x5cu0435\x5cu04\
40\x5cu0430\x5cu0433\x5cu\
0435\x5cu043e\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
44\x5cu0438\x5cu0447\x5cu\
0435\x5cu0441\x5cu043a\
\x5cu043e\x5cu0439\x5cu04\
41\x5cu0445\x5cu0435\x5cu\
043c\x5cu044b \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
43c\x5cu0430\x5cu0440\x5c\
u043a\x5cu0435\x5cu044\
0\x5cu0430\x5cu0434\x5cu0\
438\x5cu0430\x5cu0433\x5c\
u0440\x5cu0430\x5cu043\
c\x5cu043c\x5cu044b \x5cu\
0442\x5cu0438\x5cu043f\
\x5cu043e\x5cu0431\x5cu04\
3b\x5cu0430\x5cu0441\x5cu\
0442\x5cu0438\x5cu043e\
\x5cu0444\x5cu043e\x5cu04\
40\x5cu043c\x5cu043b\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f \x5cu0442\x5cu0\
438\x5cu043f\x5cu043e\x5c\
u0440\x5cu0433\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
437\x5cu0430\x5cu0446\x5c\
u0438\x5cu0438\x5cu043\
8\x5cu0441\x5cu0442\x5cu0\
43e\x5cu0447\x5cu043d\x5c\
u0438\x5cu043a\x5cu043\
0\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445\x5cu0433\x5cu043\
5\x5cu043e\x5cu0433\x5cu0\
440\x5cu0430\x5cu0444\x5c\
u0438\x5cu0447\x5cu043\
5\x5cu0441\x5cu043a\x5cu0\
43e\x5cu0439\x5cu0441\x5c\
u0445\x5cu0435\x5cu043\
c\x5cu044b \x5cu0442\x5cu\
0438\x5cu043f\x5cu043e\
\x5cu0442\x5cu043e\x5cu04\
31\x5cu0440\x5cu0430\x5cu\
0436\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
41\x5cu0435\x5cu0440\x5cu\
0438\x5cu0438\x5cu0441\
\x5cu043b\x5cu043e\x5cu04\
4f\x5cu0433\x5cu0435\x5cu\
043e\x5cu0433\x5cu0440\
\x5cu0430\x5cu0444\x5cu04\
38\x5cu0447\x5cu0435\x5cu\
0441\x5cu043a\x5cu043e\
\x5cu0439\x5cu0441\x5cu04\
45\x5cu0435\x5cu043c\x5cu\
044b \x5cu0442\x5cu043\
8\x5cu043f\x5cu043e\x5cu0\
442\x5cu043e\x5cu0431\x5c\
u0440\x5cu0430\x5cu043\
6\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu0442\x5c\
u043e\x5cu0447\x5cu043\
5\x5cu0447\x5cu043d\x5cu0\
43e\x5cu0433\x5cu043e\x5c\
u043e\x5cu0431\x5cu044\
a\x5cu0435\x5cu043a\x5cu0\
442\x5cu0430\x5cu0433\x5c\
u0435\x5cu043e\x5cu043\
3\x5cu0440\x5cu0430\x5cu0\
444\x5cu0438\x5cu0447\x5c\
u0435\x5cu0441\x5cu043\
a\x5cu043e\x5cu0439\x5cu0\
441\x5cu0445\x5cu0435\x5c\
u043c\x5cu044b \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
043e\x5cu0442\x5cu043e\
\x5cu0431\x5cu0440\x5cu04\
30\x5cu0436\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu0448\x5cu043a\x5cu04\
30\x5cu043b\x5cu044b\x5cu\
044d\x5cu043b\x5cu0435\
\x5cu043c\x5cu0435\x5cu04\
3d\x5cu0442\x5cu0430\x5cu\
043b\x5cu0435\x5cu0433\
\x5cu0435\x5cu043d\x5cu04\
34\x5cu044b\x5cu0433\x5cu\
0435\x5cu043e\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
44\x5cu0438\x5cu0447\x5cu\
0435\x5cu0441\x5cu043a\
\x5cu043e\x5cu0439\x5cu04\
41\x5cu0445\x5cu0435\x5cu\
043c\x5cu044b \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
43f\x5cu043e\x5cu0438\x5c\
u0441\x5cu043a\x5cu043\
0\x5cu043e\x5cu0431\x5cu0\
44a\x5cu0435\x5cu043a\x5c\
u0442\x5cu043e\x5cu043\
2\x5cu0433\x5cu0435\x5cu0\
43e\x5cu0433\x5cu0440\x5c\
u0430\x5cu0444\x5cu043\
8\x5cu0447\x5cu0435\x5cu0\
441\x5cu043a\x5cu043e\x5c\
u0439\x5cu0441\x5cu044\
5\x5cu0435\x5cu043c\x5cu0\
44b \x5cu0442\x5cu0438\
\x5cu043f\x5cu043f\x5cu04\
40\x5cu043e\x5cu0435\x5cu\
043a\x5cu0446\x5cu0438\
\x5cu0438\x5cu0433\x5cu04\
35\x5cu043e\x5cu0433\x5cu\
0440\x5cu0430\x5cu0444\
\x5cu0438\x5cu0447\x5cu04\
35\x5cu0441\x5cu043a\x5cu\
043e\x5cu0439\x5cu0441\
\x5cu0445\x5cu0435\x5cu04\
3c\x5cu044b \x5cu0442\x5c\
u0438\x5cu043f\x5cu044\
0\x5cu0430\x5cu0437\x5cu0\
43c\x5cu0435\x5cu0449\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f\x5cu0438\x5cu0\
437\x5cu043c\x5cu0435\x5c\
u0440\x5cu0435\x5cu043\
d\x5cu0438\x5cu0439 \x5cu\
0442\x5cu0438\x5cu043f\
\x5cu0440\x5cu0430\x5cu04\
37\x5cu043c\x5cu0435\x5cu\
0449\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
40\x5cu0435\x5cu043a\x5cu\
0432\x5cu0438\x5cu0437\
\x5cu0438\x5cu0442\x5cu04\
3e\x5cu0432\x5cu0438\x5cu\
0437\x5cu043c\x5cu0435\
\x5cu0440\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0439 \x5c\
u0442\x5cu0438\x5cu043\
f\x5cu0440\x5cu0430\x5cu0\
43c\x5cu043a\x5cu0438\x5c\
u044d\x5cu043b\x5cu043\
5\x5cu043c\x5cu0435\x5cu0\
43d\x5cu0442\x5cu0430\x5c\
u0443\x5cu043f\x5cu044\
0\x5cu0430\x5cu0432\x5cu0\
43b\x5cu0435\x5cu043d\x5c\
u0438\x5cu044f \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0441\x5cu0432\x5cu043e\
\x5cu0434\x5cu043d\x5cu04\
3e\x5cu0439\x5cu0434\x5cu\
0438\x5cu0430\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
3c\x5cu043c\x5cu044b \x5c\
u0442\x5cu0438\x5cu043\
f\x5cu0441\x5cu0432\x5cu0\
44f\x5cu0437\x5cu0438\x5c\
u0434\x5cu0438\x5cu043\
0\x5cu0433\x5cu0440\x5cu0\
430\x5cu043c\x5cu043c\x5c\
u044b\x5cu0433\x5cu043\
0\x5cu043d\x5cu0442\x5cu0\
430 \x5cu0442\x5cu0438\
\x5cu043f\x5cu0441\x5cu04\
3e\x5cu0435\x5cu0434\x5cu\
0438\x5cu043d\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
4f\x5cu0437\x5cu043d\x5cu\
0430\x5cu0447\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
39\x5cu043f\x5cu043e\x5cu\
0441\x5cu0435\x5cu0440\
\x5cu0438\x5cu044f\x5cu04\
3c\x5cu0434\x5cu0438\x5cu\
0430\x5cu0433\x5cu0440\
\x5cu0430\x5cu043c\x5cu04\
3c\x5cu044b \x5cu0442\x5c\
u0438\x5cu043f\x5cu044\
1\x5cu043e\x5cu0435\x5cu0\
434\x5cu0438\x5cu043d\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f\x5cu0442\x5cu0\
43e\x5cu0447\x5cu0435\x5c\
u043a\x5cu0434\x5cu043\
8\x5cu0430\x5cu0433\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u043c\x5cu044b \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0441\x5cu043e\x5cu0435\
\x5cu0434\x5cu0438\x5cu04\
3d\x5cu0438\x5cu0442\x5cu\
0435\x5cu043b\x5cu044c\
\x5cu043d\x5cu043e\x5cu04\
39\x5cu043b\x5cu0438\x5cu\
043d\x5cu0438\x5cu0438\
 \x5cu0442\x5cu0438\x5cu0\
43f\x5cu0441\x5cu0442\x5c\
u043e\x5cu0440\x5cu043\
e\x5cu043d\x5cu044b\x5cu0\
44d\x5cu043b\x5cu0435\x5c\
u043c\x5cu0435\x5cu043\
d\x5cu0442\x5cu0430\x5cu0\
433\x5cu0440\x5cu0430\x5c\
u0444\x5cu0438\x5cu044\
7\x5cu0435\x5cu0441\x5cu0\
43a\x5cu043e\x5cu0439\x5c\
u0441\x5cu0445\x5cu043\
5\x5cu043c\x5cu044b \x5cu\
0442\x5cu0438\x5cu043f\
\x5cu0444\x5cu043e\x5cu04\
40\x5cu043c\x5cu044b\x5cu\
043e\x5cu0442\x5cu0447\
\x5cu0435\x5cu0442\x5cu04\
30 \x5cu0442\x5cu0438\x5c\
u043f\x5cu0448\x5cu043\
a\x5cu0430\x5cu043b\x5cu0\
44b\x5cu0440\x5cu0430\x5c\
u0434\x5cu0430\x5cu044\
0\x5cu043d\x5cu043e\x5cu0\
439\x5cu0434\x5cu0438\x5c\
u0430\x5cu0433\x5cu044\
0\x5cu0430\x5cu043c\x5cu0\
43c\x5cu044b \x5cu0444\
\x5cu0430\x5cu043a\x5cu04\
42\x5cu043e\x5cu0440\x5cu\
043b\x5cu0438\x5cu043d\
\x5cu0438\x5cu0438\x5cu04\
42\x5cu0440\x5cu0435\x5cu\
043d\x5cu0434\x5cu0430\
\x5cu0434\x5cu0438\x5cu04\
30\x5cu0433\x5cu0440\x5cu\
0430\x5cu043c\x5cu043c\
\x5cu044b \x5cu0444\x5cu0\
438\x5cu0433\x5cu0443\x5c\
u0440\x5cu0430\x5cu043\
a\x5cu043d\x5cu043e\x5cu0\
43f\x5cu043a\x5cu0438 \
\x5cu0444\x5cu0438\x5cu04\
33\x5cu0443\x5cu0440\x5cu\
044b\x5cu0433\x5cu0440\
\x5cu0430\x5cu0444\x5cu04\
38\x5cu0447\x5cu0435\x5cu\
0441\x5cu043a\x5cu043e\
\x5cu0439\x5cu0441\x5cu04\
45\x5cu0435\x5cu043c\x5cu\
044b \x5cu0444\x5cu043\
8\x5cu043a\x5cu0441\x5cu0\
430\x5cu0446\x5cu0438\x5c\
u044f\x5cu0432\x5cu044\
2\x5cu0430\x5cu0431\x5cu0\
43b\x5cu0438\x5cu0446\x5c\
u0435 \x5cu0444\x5cu04\
3e\x5cu0440\x5cu043c\x5cu\
0430\x5cu0442\x5cu0434\
\x5cu043d\x5cu044f\x5cu04\
48\x5cu043a\x5cu0430\x5cu\
043b\x5cu044b\x5cu0432\
\x5cu0440\x5cu0435\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
0438 \x5cu0444\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
430\x5cu0442\x5cu043a\x5c\
u0430\x5cu0440\x5cu044\
2\x5cu0438\x5cu043d\x5cu0\
43a\x5cu0438 \x5cu0448\
\x5cu0438\x5cu0440\x5cu04\
38\x5cu043d\x5cu0430\x5cu\
043f\x5cu043e\x5cu0434\
\x5cu0447\x5cu0438\x5cu04\
3d\x5cu0435\x5cu043d\x5cu\
043d\x5cu044b\x5cu0445\
\x5cu044d\x5cu043b\x5cu04\
35\x5cu043c\x5cu0435\x5cu\
043d\x5cu0442\x5cu043e\
\x5cu0432\x5cu0444\x5cu04\
3e\x5cu0440\x5cu043c\x5cu\
044b \x5cu0432\x5cu043\
8\x5cu0434\x5cu0434\x5cu0\
432\x5cu0438\x5cu0436\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f\x5cu0431\x5cu0\
443\x5cu0445\x5cu0433\x5c\
u0430\x5cu043b\x5cu044\
2\x5cu0435\x5cu0440\x5cu0\
438\x5cu0438 \x5cu0432\
\x5cu0438\x5cu0434\x5cu04\
34\x5cu0432\x5cu0438\x5cu\
0436\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
3d\x5cu0430\x5cu043a\x5cu\
043e\x5cu043f\x5cu043b\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f \x5cu0432\x5c\
u0438\x5cu0434\x5cu043\
f\x5cu0435\x5cu0440\x5cu0\
438\x5cu043e\x5cu0434\x5c\
u0430\x5cu0440\x5cu043\
5\x5cu0433\x5cu0438\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0430\x5cu0440\x5cu043\
0\x5cu0441\x5cu0447\x5cu0\
435\x5cu0442\x5cu0430 \
\x5cu0432\x5cu0438\x5cu04\
34\x5cu0441\x5cu0447\x5cu\
0435\x5cu0442\x5cu0430\
 \x5cu0432\x5cu0438\x5cu0\
434\x5cu0442\x5cu043e\x5c\
u0447\x5cu043a\x5cu043\
8\x5cu043c\x5cu0430\x5cu0\
440\x5cu0448\x5cu0440\x5c\
u0443\x5cu0442\x5cu043\
0\x5cu0431\x5cu0438\x5cu0\
437\x5cu043d\x5cu0435\x5c\
u0441\x5cu043f\x5cu044\
0\x5cu043e\x5cu0446\x5cu0\
435\x5cu0441\x5cu0441\x5c\
u0430 \x5cu0438\x5cu04\
41\x5cu043f\x5cu043e\x5cu\
043b\x5cu044c\x5cu0437\
\x5cu043e\x5cu0432\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
0435\x5cu0430\x5cu0433\
\x5cu0440\x5cu0435\x5cu04\
33\x5cu0430\x5cu0442\x5cu\
0430\x5cu0440\x5cu0435\
\x5cu0433\x5cu0438\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
0430\x5cu043d\x5cu0430\
\x5cu043a\x5cu043e\x5cu04\
3f\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
 \x5cu0438\x5cu0441\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u044c\x5cu0437\x5cu043\
e\x5cu0432\x5cu0430\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0433\x5cu0440\x5cu044\
3\x5cu043f\x5cu043f\x5cu0\
438\x5cu044d\x5cu043b\x5c\
u0435\x5cu043c\x5cu043\
5\x5cu043d\x5cu0442\x5cu0\
43e\x5cu0432 \x5cu0438\
\x5cu0441\x5cu043f\x5cu04\
3e\x5cu043b\x5cu044c\x5cu\
0437\x5cu043e\x5cu0432\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu0435\x5cu0440\x5cu\
0435\x5cu0436\x5cu0438\
\x5cu043c\x5cu0430\x5cu04\
3f\x5cu0440\x5cu043e\x5cu\
0432\x5cu0435\x5cu0434\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f \x5cu0438\x5c\
u0441\x5cu043f\x5cu043\
e\x5cu043b\x5cu044c\x5cu0\
437\x5cu043e\x5cu0432\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu0435\x5cu0441\x5cu0\
440\x5cu0435\x5cu0437\x5c\
u0430 \x5cu043f\x5cu04\
35\x5cu0440\x5cu0438\x5cu\
043e\x5cu0434\x5cu0438\
\x5cu0447\x5cu043d\x5cu04\
3e\x5cu0441\x5cu0442\x5cu\
044c\x5cu0430\x5cu0433\
\x5cu0440\x5cu0435\x5cu04\
33\x5cu0430\x5cu0442\x5cu\
0430\x5cu0440\x5cu0435\
\x5cu0433\x5cu0438\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
0430\x5cu043d\x5cu0430\
\x5cu043a\x5cu043e\x5cu04\
3f\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
 \x5cu0440\x5cu0435\x5cu0\
436\x5cu0438\x5cu043c\x5c\
u0430\x5cu0432\x5cu044\
2\x5cu043e\x5cu0432\x5cu0\
440\x5cu0435\x5cu043c\x5c\
u044f \x5cu0440\x5cu04\
35\x5cu0436\x5cu0438\x5cu\
043c\x5cu0437\x5cu0430\
\x5cu043f\x5cu0438\x5cu04\
41\x5cu0438\x5cu0434\x5cu\
043e\x5cu043a\x5cu0443\
\x5cu043c\x5cu0435\x5cu04\
3d\x5cu0442\x5cu0430 \x5c\
u0440\x5cu0435\x5cu043\
6\x5cu0438\x5cu043c\x5cu0\
43f\x5cu0440\x5cu043e\x5c\
u0432\x5cu0435\x5cu043\
4\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu0434\x5c\
u043e\x5cu043a\x5cu044\
3\x5cu043c\x5cu0435\x5cu0\
43d\x5cu0442\x5cu0430 \
\x5cu0430\x5cu0432\x5cu04\
42\x5cu043e\x5cu0440\x5cu\
0435\x5cu0433\x5cu0438\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu0430\x5cu0446\x5cu\
0438\x5cu044f\x5cu0438\
\x5cu0437\x5cu043c\x5cu04\
35\x5cu043d\x5cu0435\x5cu\
043d\x5cu0438\x5cu0439\
 \x5cu0434\x5cu043e\x5cu0\
43f\x5cu0443\x5cu0441\x5c\
u0442\x5cu0438\x5cu043\
c\x5cu044b\x5cu0439\x5cu0\
43d\x5cu043e\x5cu043c\x5c\
u0435\x5cu0440\x5cu044\
1\x5cu043e\x5cu043e\x5cu0\
431\x5cu0449\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f \x5cu043e\x5cu0442\x5cu\
043f\x5cu0440\x5cu0430\
\x5cu0432\x5cu043a\x5cu04\
30\x5cu044d\x5cu043b\x5cu\
0435\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445 \x5cu043f\x5cu0\
43e\x5cu043b\x5cu0443\x5c\
u0447\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
44d\x5cu043b\x5cu0435\x5c\
u043c\x5cu0435\x5cu043\
d\x5cu0442\x5cu0430\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u043d\x5cu044b\x5cu044\
5 \x5cu0438\x5cu0441\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu044c\x5cu0437\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0440\x5cu0430\x5cu04\
41\x5cu0448\x5cu0438\x5cu\
0444\x5cu0440\x5cu043e\
\x5cu0432\x5cu043a\x5cu04\
38\x5cu0442\x5cu0430\x5cu\
0431\x5cu043b\x5cu0438\
\x5cu0447\x5cu043d\x5cu04\
3e\x5cu0433\x5cu043e\x5cu\
0434\x5cu043e\x5cu043a\
\x5cu0443\x5cu043c\x5cu04\
35\x5cu043d\x5cu0442\x5cu\
0430 \x5cu043e\x5cu044\
0\x5cu0438\x5cu0435\x5cu0\
43d\x5cu0442\x5cu0430\x5c\
u0446\x5cu0438\x5cu044\
f\x5cu0441\x5cu0442\x5cu0\
440\x5cu0430\x5cu043d\x5c\
u0438\x5cu0446\x5cu044\
b \x5cu043f\x5cu043e\x5cu\
043b\x5cu043e\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0438\x5cu\
0442\x5cu043e\x5cu0433\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu043e\x5cu043b\x5cu\
043e\x5cu043d\x5cu043e\
\x5cu043a\x5cu0441\x5cu04\
32\x5cu043e\x5cu0434\x5cu\
043d\x5cu043e\x5cu0439\
\x5cu0442\x5cu0430\x5cu04\
31\x5cu043b\x5cu0438\x5cu\
0446\x5cu044b \x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
43e\x5cu0436\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu0438\x5cu0442\x5cu0\
43e\x5cu0433\x5cu043e\x5c\
u0432\x5cu0441\x5cu044\
2\x5cu0440\x5cu043e\x5cu0\
43a\x5cu0441\x5cu0432\x5c\
u043e\x5cu0434\x5cu043\
d\x5cu043e\x5cu0439\x5cu0\
442\x5cu0430\x5cu0431\x5c\
u043b\x5cu0438\x5cu044\
6\x5cu044b \x5cu043f\x5cu\
043e\x5cu043b\x5cu043e\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
0442\x5cu0435\x5cu043a\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu043e\x5cu0442\x5cu\
043d\x5cu043e\x5cu0441\
\x5cu0438\x5cu0442\x5cu04\
35\x5cu043b\x5cu044c\x5cu\
043d\x5cu043e\x5cu043a\
\x5cu0430\x5cu0440\x5cu04\
42\x5cu0438\x5cu043d\x5cu\
043a\x5cu0438 \x5cu044\
0\x5cu0430\x5cu0441\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u043e\x5cu0436\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu0437\x5cu0430\x5c\
u0433\x5cu043e\x5cu043\
b\x5cu043e\x5cu0432\x5cu0\
43a\x5cu0430\x5cu0433\x5c\
u0440\x5cu0443\x5cu043\
f\x5cu043f\x5cu0438\x5cu0\
440\x5cu043e\x5cu0432\x5c\
u043a\x5cu0438\x5cu044\
2\x5cu0430\x5cu0431\x5cu0\
43b\x5cu0438\x5cu0447\x5c\
u043d\x5cu043e\x5cu043\
3\x5cu043e\x5cu0434\x5cu0\
43e\x5cu043a\x5cu0443\x5c\
u043c\x5cu0435\x5cu043\
d\x5cu0442\x5cu0430 \x5cu\
0441\x5cu043f\x5cu043e\
\x5cu0441\x5cu043e\x5cu04\
31\x5cu0447\x5cu0442\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu0437\x5cu04\
3d\x5cu0430\x5cu0447\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu0439\x5cu0442\x5cu04\
30\x5cu0431\x5cu043b\x5cu\
0438\x5cu0447\x5cu043d\
\x5cu043e\x5cu0433\x5cu04\
3e\x5cu0434\x5cu043e\x5cu\
043a\x5cu0443\x5cu043c\
\x5cu0435\x5cu043d\x5cu04\
42\x5cu0430 \x5cu0442\x5c\
u0438\x5cu043f\x5cu043\
4\x5cu0432\x5cu0443\x5cu0\
441\x5cu0442\x5cu043e\x5c\
u0440\x5cu043e\x5cu043\
d\x5cu043d\x5cu0435\x5cu0\
439\x5cu043f\x5cu0435\x5c\
u0447\x5cu0430\x5cu044\
2\x5cu0438 \x5cu0442\x5cu\
0438\x5cu043f\x5cu0437\
\x5cu0430\x5cu043f\x5cu04\
3e\x5cu043b\x5cu043d\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu043e\x5cu04\
31\x5cu043b\x5cu0430\x5cu\
0441\x5cu0442\x5cu0438\
\x5cu0442\x5cu0430\x5cu04\
31\x5cu043b\x5cu0438\x5cu\
0447\x5cu043d\x5cu043e\
\x5cu0433\x5cu043e\x5cu04\
34\x5cu043e\x5cu043a\x5cu\
0443\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30 \x5cu0442\x5cu0438\x5c\
u043f\x5cu043a\x5cu044\
3\x5cu0440\x5cu0441\x5cu0\
43e\x5cu0440\x5cu043e\x5c\
u0432\x5cu0442\x5cu043\
0\x5cu0431\x5cu043b\x5cu0\
438\x5cu0447\x5cu043d\x5c\
u043e\x5cu0433\x5cu043\
e\x5cu0434\x5cu043e\x5cu0\
43a\x5cu0443\x5cu043c\x5c\
u0435\x5cu043d\x5cu044\
2\x5cu0430 \x5cu0442\x5cu\
0438\x5cu043f\x5cu043b\
\x5cu0438\x5cu043d\x5cu04\
38\x5cu0438\x5cu0440\x5cu\
0438\x5cu0441\x5cu0443\
\x5cu043d\x5cu043a\x5cu04\
30\x5cu0442\x5cu0430\x5cu\
0431\x5cu043b\x5cu0438\
\x5cu0447\x5cu043d\x5cu04\
3e\x5cu0433\x5cu043e\x5cu\
0434\x5cu043e\x5cu043a\
\x5cu0443\x5cu043c\x5cu04\
35\x5cu043d\x5cu0442\x5cu\
0430 \x5cu0442\x5cu043\
8\x5cu043f\x5cu043b\x5cu0\
438\x5cu043d\x5cu0438\x5c\
u0438\x5cu044f\x5cu044\
7\x5cu0435\x5cu0439\x5cu0\
43a\x5cu0438\x5cu0442\x5c\
u0430\x5cu0431\x5cu043\
b\x5cu0438\x5cu0447\x5cu0\
43d\x5cu043e\x5cu0433\x5c\
u043e\x5cu0434\x5cu043\
e\x5cu043a\x5cu0443\x5cu0\
43c\x5cu0435\x5cu043d\x5c\
u0442\x5cu0430 \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
043d\x5cu0430\x5cu043f\
\x5cu0440\x5cu0430\x5cu04\
32\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu043f\x5cu0435\x5cu04\
40\x5cu0435\x5cu0445\x5cu\
043e\x5cu0434\x5cu0430\
\x5cu0442\x5cu0430\x5cu04\
31\x5cu043b\x5cu0438\x5cu\
0447\x5cu043d\x5cu043e\
\x5cu0433\x5cu043e\x5cu04\
34\x5cu043e\x5cu043a\x5cu\
0443\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30 \x5cu0442\x5cu0438\x5c\
u043f\x5cu043e\x5cu044\
2\x5cu043e\x5cu0431\x5cu0\
440\x5cu0430\x5cu0436\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f\x5cu0432\x5cu0\
44b\x5cu0434\x5cu0435\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f\x5cu0\
442\x5cu0430\x5cu0431\x5c\
u043b\x5cu0438\x5cu044\
7\x5cu043d\x5cu043e\x5cu0\
433\x5cu043e\x5cu0434\x5c\
u043e\x5cu043a\x5cu044\
3\x5cu043c\x5cu0435\x5cu0\
43d\x5cu0442\x5cu0430 \
\x5cu0442\x5cu0438\x5cu04\
3f\x5cu043e\x5cu0442\x5cu\
043e\x5cu0431\x5cu0440\
\x5cu0430\x5cu0436\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f\x5cu043b\x5cu0438\
\x5cu043d\x5cu0438\x5cu04\
39\x5cu0441\x5cu0432\x5cu\
043e\x5cu0434\x5cu043d\
\x5cu043e\x5cu0439\x5cu04\
42\x5cu0430\x5cu0431\x5cu\
043b\x5cu0438\x5cu0446\
\x5cu044b \x5cu0442\x5cu0\
438\x5cu043f\x5cu0440\x5c\
u0430\x5cu0437\x5cu043\
c\x5cu0435\x5cu0449\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044f\x5cu0442\x5cu043\
5\x5cu043a\x5cu0441\x5cu0\
442\x5cu0430\x5cu0442\x5c\
u0430\x5cu0431\x5cu043\
b\x5cu0438\x5cu0447\x5cu0\
43d\x5cu043e\x5cu0433\x5c\
u043e\x5cu0434\x5cu043\
e\x5cu043a\x5cu0443\x5cu0\
43c\x5cu0435\x5cu043d\x5c\
u0442\x5cu0430 \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0440\x5cu0438\x5cu0441\
\x5cu0443\x5cu043d\x5cu04\
3a\x5cu0430\x5cu0442\x5cu\
0430\x5cu0431\x5cu043b\
\x5cu0438\x5cu0447\x5cu04\
3d\x5cu043e\x5cu0433\x5cu\
043e\x5cu0434\x5cu043e\
\x5cu043a\x5cu0443\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
0442\x5cu0430 \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
441\x5cu043c\x5cu0435\x5c\
u0449\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f\x5cu0\
442\x5cu0430\x5cu0431\x5c\
u043b\x5cu0438\x5cu044\
7\x5cu043d\x5cu043e\x5cu0\
433\x5cu043e\x5cu0434\x5c\
u043e\x5cu043a\x5cu044\
3\x5cu043c\x5cu0435\x5cu0\
43d\x5cu0442\x5cu0430 \
\x5cu0442\x5cu0438\x5cu04\
3f\x5cu0443\x5cu0437\x5cu\
043e\x5cu0440\x5cu0430\
\x5cu0442\x5cu0430\x5cu04\
31\x5cu043b\x5cu0438\x5cu\
0447\x5cu043d\x5cu043e\
\x5cu0433\x5cu043e\x5cu04\
34\x5cu043e\x5cu043a\x5cu\
0443\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30 \x5cu0442\x5cu0438\x5c\
u043f\x5cu0444\x5cu043\
0\x5cu0439\x5cu043b\x5cu0\
430\x5cu0442\x5cu0430\x5c\
u0431\x5cu043b\x5cu043\
8\x5cu0447\x5cu043d\x5cu0\
43e\x5cu0433\x5cu043e\x5c\
u0434\x5cu043e\x5cu043\
a\x5cu0443\x5cu043c\x5cu0\
435\x5cu043d\x5cu0442\x5c\
u0430 \x5cu0442\x5cu04\
3e\x5cu0447\x5cu043d\x5cu\
043e\x5cu0441\x5cu0442\
\x5cu044c\x5cu043f\x5cu04\
35\x5cu0447\x5cu0430\x5cu\
0442\x5cu0438 \x5cu044\
7\x5cu0435\x5cu0440\x5cu0\
435\x5cu0434\x5cu043e\x5c\
u0432\x5cu0430\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
440\x5cu0430\x5cu0441\x5c\
u043f\x5cu043e\x5cu043\
b\x5cu043e\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044f\x5cu0441\x5cu044\
2\x5cu0440\x5cu0430\x5cu0\
43d\x5cu0438\x5cu0446 \
\x5cu043e\x5cu0442\x5cu04\
3e\x5cu0431\x5cu0440\x5cu\
0430\x5cu0436\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu0432\x5cu0440\x5cu\
0435\x5cu043c\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
4d\x5cu043b\x5cu0435\x5cu\
043c\x5cu0435\x5cu043d\
\x5cu0442\x5cu043e\x5cu04\
32\x5cu043f\x5cu043b\x5cu\
0430\x5cu043d\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0449\x5cu0438\x5cu\
043a\x5cu0430 \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
444\x5cu0430\x5cu0439\x5c\
u043b\x5cu0430\x5cu044\
4\x5cu043e\x5cu0440\x5cu0\
43c\x5cu0430\x5cu0442\x5c\
u0438\x5cu0440\x5cu043\
e\x5cu0432\x5cu0430\x5cu0\
43d\x5cu043d\x5cu043e\x5c\
u0433\x5cu043e\x5cu043\
4\x5cu043e\x5cu043a\x5cu0\
443\x5cu043c\x5cu0435\x5c\
u043d\x5cu0442\x5cu043\
0 \x5cu043e\x5cu0431\x5cu\
0445\x5cu043e\x5cu0434\
\x5cu0440\x5cu0435\x5cu04\
37\x5cu0443\x5cu043b\x5cu\
044c\x5cu0442\x5cu0430\
\x5cu0442\x5cu0430\x5cu04\
37\x5cu0430\x5cu043f\x5cu\
0440\x5cu043e\x5cu0441\
\x5cu0430 \x5cu0442\x5cu0\
438\x5cu043f\x5cu0437\x5c\
u0430\x5cu043f\x5cu043\
8\x5cu0441\x5cu0438\x5cu0\
437\x5cu0430\x5cu043f\x5c\
u0440\x5cu043e\x5cu044\
1\x5cu0430 \x5cu0432\x5cu\
0438\x5cu0434\x5cu0437\
\x5cu0430\x5cu043f\x5cu04\
3e\x5cu043b\x5cu043d\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu0440\x5cu04\
30\x5cu0441\x5cu0448\x5cu\
0438\x5cu0444\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu0438\x5cu043f\x5cu\
043e\x5cu0441\x5cu0442\
\x5cu0440\x5cu043e\x5cu04\
38\x5cu0442\x5cu0435\x5cu\
043b\x5cu044f\x5cu043e\
\x5cu0442\x5cu0447\x5cu04\
35\x5cu0442\x5cu0430 \x5c\
u0442\x5cu0438\x5cu043\
f\x5cu0434\x5cu043e\x5cu0\
431\x5cu0430\x5cu0432\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f\x5cu0\
43f\x5cu0440\x5cu0435\x5c\
u0434\x5cu0441\x5cu044\
2\x5cu0430\x5cu0432\x5cu0\
43b\x5cu0435\x5cu043d\x5c\
u0438\x5cu0439 \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0438\x5cu0437\x5cu043c\
\x5cu0435\x5cu0440\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f\x5cu043f\x5cu043e\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu043e\x5cu0438\x5cu\
0442\x5cu0435\x5cu043b\
\x5cu044f\x5cu043e\x5cu04\
42\x5cu0447\x5cu0435\x5cu\
0442\x5cu0430 \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
440\x5cu0430\x5cu0437\x5c\
u043c\x5cu0435\x5cu044\
9\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu0438\x5c\
u0442\x5cu043e\x5cu043\
3\x5cu043e\x5cu0432 \x5cu\
0434\x5cu043e\x5cu0441\
\x5cu0442\x5cu0443\x5cu04\
3f\x5cu043a\x5cu0444\x5cu\
0430\x5cu0439\x5cu043b\
\x5cu0443 \x5cu0440\x5cu0\
435\x5cu0436\x5cu0438\x5c\
u043c\x5cu0434\x5cu043\
8\x5cu0430\x5cu043b\x5cu0\
43e\x5cu0433\x5cu0430\x5c\
u0432\x5cu044b\x5cu043\
1\x5cu043e\x5cu0440\x5cu0\
430\x5cu0444\x5cu0430\x5c\
u0439\x5cu043b\x5cu043\
0 \x5cu0440\x5cu0435\x5cu\
0436\x5cu0438\x5cu043c\
\x5cu043e\x5cu0442\x5cu04\
3a\x5cu0440\x5cu044b\x5cu\
0442\x5cu0438\x5cu044f\
\x5cu0444\x5cu0430\x5cu04\
39\x5cu043b\x5cu0430 \x5c\
u0442\x5cu0438\x5cu043\
f\x5cu0438\x5cu0437\x5cu0\
43c\x5cu0435\x5cu0440\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f\x5cu043f\x5cu0\
43e\x5cu0441\x5cu0442\x5c\
u0440\x5cu043e\x5cu043\
8\x5cu0442\x5cu0435\x5cu0\
43b\x5cu044f\x5cu0437\x5c\
u0430\x5cu043f\x5cu044\
0\x5cu043e\x5cu0441\x5cu0\
430 \x5cu0432\x5cu0438\
\x5cu0434\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445\x5cu0430\
\x5cu043d\x5cu0430\x5cu04\
3b\x5cu0438\x5cu0437\x5cu\
0430 \x5cu043c\x5cu043\
5\x5cu0442\x5cu043e\x5cu0\
434\x5cu043a\x5cu043b\x5c\
u0430\x5cu0441\x5cu044\
2\x5cu0435\x5cu0440\x5cu0\
438\x5cu0437\x5cu0430\x5c\
u0446\x5cu0438\x5cu043\
8 \x5cu0442\x5cu0438\x5cu\
043f\x5cu0435\x5cu0434\
\x5cu0438\x5cu043d\x5cu04\
38\x5cu0446\x5cu044b\x5cu\
0438\x5cu043d\x5cu0442\
\x5cu0435\x5cu0440\x5cu04\
32\x5cu0430\x5cu043b\x5cu\
0430\x5cu0432\x5cu0440\
\x5cu0435\x5cu043c\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0430\x5cu043d\x5cu0430\
\x5cu043b\x5cu0438\x5cu04\
37\x5cu0430\x5cu0434\x5cu\
0430\x5cu043d\x5cu043d\
\x5cu044b\x5cu0445 \x5cu0\
442\x5cu0438\x5cu043f\x5c\
u0437\x5cu0430\x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
43d\x5cu0435\x5cu043d\x5c\
u0438\x5cu044f\x5cu044\
2\x5cu0430\x5cu0431\x5cu0\
43b\x5cu0438\x5cu0446\x5c\
u044b\x5cu0440\x5cu043\
5\x5cu0437\x5cu0443\x5cu0\
43b\x5cu044c\x5cu0442\x5c\
u0430\x5cu0442\x5cu043\
0\x5cu0430\x5cu043d\x5cu0\
430\x5cu043b\x5cu0438\x5c\
u0437\x5cu0430\x5cu043\
4\x5cu0430\x5cu043d\x5cu0\
43d\x5cu044b\x5cu0445 \
\x5cu0442\x5cu0438\x5cu04\
3f\x5cu0438\x5cu0441\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu044c\x5cu0437\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu0447\x5cu0438\x5cu04\
41\x5cu043b\x5cu043e\x5cu\
0432\x5cu044b\x5cu0445\
\x5cu0437\x5cu043d\x5cu04\
30\x5cu0447\x5cu0435\x5cu\
043d\x5cu0438\x5cu0439\
\x5cu0430\x5cu043d\x5cu04\
30\x5cu043b\x5cu0438\x5cu\
0437\x5cu0430\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0445 \x5c\
u0442\x5cu0438\x5cu043\
f\x5cu0438\x5cu0441\x5cu0\
442\x5cu043e\x5cu0447\x5c\
u043d\x5cu0438\x5cu043\
a\x5cu0430\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0445\x5cu043\
f\x5cu043e\x5cu0438\x5cu0\
441\x5cu043a\x5cu0430\x5c\
u0430\x5cu0441\x5cu044\
1\x5cu043e\x5cu0446\x5cu0\
438\x5cu0430\x5cu0446\x5c\
u0438\x5cu0439 \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
043a\x5cu043e\x5cu043b\
\x5cu043e\x5cu043d\x5cu04\
3a\x5cu0438\x5cu0430\x5cu\
043d\x5cu0430\x5cu043b\
\x5cu0438\x5cu0437\x5cu04\
30\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445\x5cu0434\x5cu04\
35\x5cu0440\x5cu0435\x5cu\
0432\x5cu043e\x5cu0440\
\x5cu0435\x5cu0448\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0439 \x5cu0442\x5cu043\
8\x5cu043f\x5cu043a\x5cu0\
43e\x5cu043b\x5cu043e\x5c\
u043d\x5cu043a\x5cu043\
8\x5cu0430\x5cu043d\x5cu0\
430\x5cu043b\x5cu0438\x5c\
u0437\x5cu0430\x5cu043\
4\x5cu0430\x5cu043d\x5cu0\
43d\x5cu044b\x5cu0445\x5c\
u043a\x5cu043b\x5cu043\
0\x5cu0441\x5cu0442\x5cu0\
435\x5cu0440\x5cu0438\x5c\
u0437\x5cu0430\x5cu044\
6\x5cu0438\x5cu044f \x5cu\
0442\x5cu0438\x5cu043f\
\x5cu043a\x5cu043e\x5cu04\
3b\x5cu043e\x5cu043d\x5cu\
043a\x5cu0438\x5cu0430\
\x5cu043d\x5cu0430\x5cu04\
3b\x5cu0438\x5cu0437\x5cu\
0430\x5cu0434\x5cu0430\
\x5cu043d\x5cu043d\x5cu04\
4b\x5cu0445\x5cu043e\x5cu\
0431\x5cu0449\x5cu0430\
\x5cu044f\x5cu0441\x5cu04\
42\x5cu0430\x5cu0442\x5cu\
0438\x5cu0441\x5cu0442\
\x5cu0438\x5cu043a\x5cu04\
30 \x5cu0442\x5cu0438\x5c\
u043f\x5cu043a\x5cu043\
e\x5cu043b\x5cu043e\x5cu0\
43d\x5cu043a\x5cu0438\x5c\
u0430\x5cu043d\x5cu043\
0\x5cu043b\x5cu0438\x5cu0\
437\x5cu0430\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445\x5cu0\
43f\x5cu043e\x5cu0438\x5c\
u0441\x5cu043a\x5cu043\
0\x5cu0441\x5cu0441\x5cu0\
43e\x5cu0446\x5cu0438\x5c\
u0430\x5cu0446\x5cu043\
8\x5cu0439 \x5cu0442\x5cu\
0438\x5cu043f\x5cu043a\
\x5cu043e\x5cu043b\x5cu04\
3e\x5cu043d\x5cu043a\x5cu\
0438\x5cu0430\x5cu043d\
\x5cu0430\x5cu043b\x5cu04\
38\x5cu0437\x5cu0430\x5cu\
0434\x5cu0430\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
45\x5cu043f\x5cu043e\x5cu\
0438\x5cu0441\x5cu043a\
\x5cu043f\x5cu043e\x5cu04\
41\x5cu043b\x5cu0435\x5cu\
0434\x5cu043e\x5cu0432\
\x5cu0430\x5cu0442\x5cu04\
35\x5cu043b\x5cu044c\x5cu\
043d\x5cu043e\x5cu0441\
\x5cu0442\x5cu0435\x5cu04\
39 \x5cu0442\x5cu0438\x5c\
u043f\x5cu043a\x5cu043\
e\x5cu043b\x5cu043e\x5cu0\
43d\x5cu043a\x5cu0438\x5c\
u043c\x5cu043e\x5cu043\
4\x5cu0435\x5cu043b\x5cu0\
438\x5cu043f\x5cu0440\x5c\
u043e\x5cu0433\x5cu043\
d\x5cu043e\x5cu0437\x5cu0\
430 \x5cu0442\x5cu0438\
\x5cu043f\x5cu043c\x5cu04\
35\x5cu0440\x5cu044b\x5cu\
0440\x5cu0430\x5cu0441\
\x5cu0441\x5cu0442\x5cu04\
3e\x5cu044f\x5cu043d\x5cu\
0438\x5cu044f\x5cu0430\
\x5cu043d\x5cu0430\x5cu04\
3b\x5cu0438\x5cu0437\x5cu\
0430\x5cu0434\x5cu0430\
\x5cu043d\x5cu043d\x5cu04\
4b\x5cu0445 \x5cu0442\x5c\
u0438\x5cu043f\x5cu043\
e\x5cu0442\x5cu0441\x5cu0\
435\x5cu0447\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f\x5cu043f\x5cu0440\x5cu0\
430\x5cu0432\x5cu0438\x5c\
u043b\x5cu0430\x5cu044\
1\x5cu0441\x5cu043e\x5cu0\
446\x5cu0438\x5cu0430\x5c\
u0446\x5cu0438\x5cu043\
8 \x5cu0442\x5cu0438\x5cu\
043f\x5cu043f\x5cu043e\
\x5cu043b\x5cu044f\x5cu04\
30\x5cu043d\x5cu0430\x5cu\
043b\x5cu0438\x5cu0437\
\x5cu0430\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
441\x5cu0442\x5cu0430\x5c\
u043d\x5cu0434\x5cu043\
0\x5cu0440\x5cu0442\x5cu0\
438\x5cu0437\x5cu0430\x5c\
u0446\x5cu0438\x5cu043\
8\x5cu0430\x5cu043d\x5cu0\
430\x5cu043b\x5cu0438\x5c\
u0437\x5cu0430\x5cu043\
4\x5cu0430\x5cu043d\x5cu0\
43d\x5cu044b\x5cu0445 \
\x5cu0442\x5cu0438\x5cu04\
3f\x5cu0443\x5cu043f\x5cu\
043e\x5cu0440\x5cu044f\
\x5cu0434\x5cu043e\x5cu04\
47\x5cu0438\x5cu0432\x5cu\
0430\x5cu043d\x5cu0438\
\x5cu044f\x5cu043f\x5cu04\
40\x5cu0430\x5cu0432\x5cu\
0438\x5cu043b\x5cu0430\
\x5cu0441\x5cu0441\x5cu04\
3e\x5cu0446\x5cu0438\x5cu\
0430\x5cu0446\x5cu0438\
\x5cu0438\x5cu0430\x5cu04\
3d\x5cu0430\x5cu043b\x5cu\
0438\x5cu0437\x5cu0430\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu043d\x5cu044b\x5cu\
0445 \x5cu0442\x5cu043\
8\x5cu043f\x5cu0443\x5cu0\
43f\x5cu043e\x5cu0440\x5c\
u044f\x5cu0434\x5cu043\
e\x5cu0447\x5cu0438\x5cu0\
432\x5cu0430\x5cu043d\x5c\
u0438\x5cu044f\x5cu044\
8\x5cu0430\x5cu0431\x5cu0\
43b\x5cu043e\x5cu043d\x5c\
u043e\x5cu0432\x5cu043\
f\x5cu043e\x5cu0441\x5cu0\
43b\x5cu0435\x5cu0434\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu0442\x5cu0435\x5cu0\
43b\x5cu044c\x5cu043d\x5c\
u043e\x5cu0441\x5cu044\
2\x5cu0435\x5cu0439\x5cu0\
430\x5cu043d\x5cu0430\x5c\
u043b\x5cu0438\x5cu043\
7\x5cu0430\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0445 \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0443\x5cu043f\x5cu0440\
\x5cu043e\x5cu0449\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f\x5cu0434\x5cu0435\
\x5cu0440\x5cu0435\x5cu04\
32\x5cu0430\x5cu0440\x5cu\
0435\x5cu0448\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
39 ws\x5cu043d\x5cu043\
0\x5cu043f\x5cu0440\x5cu0\
430\x5cu0432\x5cu043b\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435\x5cu043f\x5cu0\
430\x5cu0440\x5cu0430\x5c\
u043c\x5cu0435\x5cu044\
2\x5cu0440\x5cu0430 \x5cu\
0432\x5cu0430\x5cu0440\
\x5cu0438\x5cu0430\x5cu04\
3d\x5cu0442xpathxs \
\x5cu0432\x5cu0430\x5cu04\
40\x5cu0438\x5cu0430\x5cu\
043d\x5cu0442\x5cu0437\
\x5cu0430\x5cu043f\x5cu04\
38\x5cu0441\x5cu0438\x5cu\
0434\x5cu0430\x5cu0442\
\x5cu044bjson \x5cu043\
2\x5cu0430\x5cu0440\x5cu0\
438\x5cu0430\x5cu043d\x5c\
u0442\x5cu043f\x5cu044\
0\x5cu043e\x5cu0441\x5cu0\
442\x5cu043e\x5cu0433\x5c\
u043e\x5cu0442\x5cu043\
8\x5cu043f\x5cu0430xs \
\x5cu0432\x5cu0438\x5cu04\
34\x5cu0433\x5cu0440\x5cu\
0443\x5cu043f\x5cu043f\
\x5cu044b\x5cu043c\x5cu04\
3e\x5cu0434\x5cu0435\x5cu\
043b\x5cu0438xs \x5cu0\
432\x5cu0438\x5cu0434\x5c\
u0444\x5cu0430\x5cu044\
1\x5cu0435\x5cu0442\x5cu0\
430xdto \x5cu0434\x5cu\
0435\x5cu0439\x5cu0441\
\x5cu0442\x5cu0432\x5cu04\
38\x5cu0435\x5cu043f\x5cu\
043e\x5cu0441\x5cu0442\
\x5cu0440\x5cu043e\x5cu04\
38\x5cu0442\x5cu0435\x5cu\
043b\x5cu044fdom \x5cu\
0437\x5cu0430\x5cu0432\
\x5cu0435\x5cu0440\x5cu04\
48\x5cu0435\x5cu043d\x5cu\
043d\x5cu043e\x5cu0441\
\x5cu0442\x5cu044c\x5cu04\
3f\x5cu0440\x5cu043e\x5cu\
0441\x5cu0442\x5cu043e\
\x5cu0433\x5cu043e\x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0430xs \x5cu0437\x5cu0\
430\x5cu0432\x5cu0435\x5c\
u0440\x5cu0448\x5cu043\
5\x5cu043d\x5cu043d\x5cu0\
43e\x5cu0441\x5cu0442\x5c\
u044c\x5cu0441\x5cu043\
e\x5cu0441\x5cu0442\x5cu0\
430\x5cu0432\x5cu043d\x5c\
u043e\x5cu0433\x5cu043\
e\x5cu0442\x5cu0438\x5cu0\
43f\x5cu0430xs \x5cu04\
37\x5cu0430\x5cu0432\x5cu\
0435\x5cu0440\x5cu0448\
\x5cu0435\x5cu043d\x5cu04\
3d\x5cu043e\x5cu0441\x5cu\
0442\x5cu044c\x5cu0441\
\x5cu0445\x5cu0435\x5cu04\
3c\x5cu044bxs \x5cu043\
7\x5cu0430\x5cu043f\x5cu0\
440\x5cu0435\x5cu0449\x5c\
u0435\x5cu043d\x5cu043\
d\x5cu044b\x5cu0435\x5cu0\
43f\x5cu043e\x5cu0434\x5c\
u0441\x5cu0442\x5cu043\
0\x5cu043d\x5cu043e\x5cu0\
432\x5cu043a\x5cu0438x\
s \x5cu0438\x5cu0441\x5cu\
043a\x5cu043b\x5cu044e\
\x5cu0447\x5cu0435\x5cu04\
3d\x5cu0438\x5cu044f\x5cu\
0433\x5cu0440\x5cu0443\
\x5cu043f\x5cu043f\x5cu04\
3f\x5cu043e\x5cu0434\x5cu\
0441\x5cu0442\x5cu0430\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu043a\x5cu0438xs\
 \x5cu043a\x5cu0430\x5cu0\
442\x5cu0435\x5cu0433\x5c\
u043e\x5cu0440\x5cu043\
8\x5cu044f\x5cu0438\x5cu0\
441\x5cu043f\x5cu043e\x5c\
u043b\x5cu044c\x5cu043\
7\x5cu043e\x5cu0432\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u044f\x5cu0430\x5cu044\
2\x5cu0440\x5cu0438\x5cu0\
431\x5cu0443\x5cu0442\x5c\
u0430xs \x5cu043a\x5cu\
0430\x5cu0442\x5cu0435\
\x5cu0433\x5cu043e\x5cu04\
40\x5cu0438\x5cu044f\x5cu\
043e\x5cu0433\x5cu0440\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu0447\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu0438\x5cu0434\x5cu04\
35\x5cu043d\x5cu0442\x5cu\
0438\x5cu0447\x5cu043d\
\x5cu043e\x5cu0441\x5cu04\
42\x5cu0438xs \x5cu043\
a\x5cu0430\x5cu0442\x5cu0\
435\x5cu0433\x5cu043e\x5c\
u0440\x5cu0438\x5cu044\
f\x5cu043e\x5cu0433\x5cu0\
440\x5cu0430\x5cu043d\x5c\
u0438\x5cu0447\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f\x5cu043f\x5cu0440\x5c\
u043e\x5cu0441\x5cu044\
2\x5cu0440\x5cu0430\x5cu0\
43d\x5cu0441\x5cu0442\x5c\
u0432\x5cu0438\x5cu043\
c\x5cu0435\x5cu043dxs \
\x5cu043c\x5cu0435\x5cu04\
42\x5cu043e\x5cu0434\x5cu\
043d\x5cu0430\x5cu0441\
\x5cu043b\x5cu0435\x5cu04\
34\x5cu043e\x5cu0432\x5cu\
0430\x5cu043d\x5cu0438\
\x5cu044fxs \x5cu043c\x5c\
u043e\x5cu0434\x5cu043\
5\x5cu043b\x5cu044c\x5cu0\
441\x5cu043e\x5cu0434\x5c\
u0435\x5cu0440\x5cu043\
6\x5cu0438\x5cu043c\x5cu0\
43e\x5cu0433\x5cu043ex\
s \x5cu043d\x5cu0430\x5cu\
0437\x5cu043d\x5cu0430\
\x5cu0447\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
0442\x5cu0438\x5cu043f\
\x5cu0430xml \x5cu043d\
\x5cu0435\x5cu0434\x5cu04\
3e\x5cu043f\x5cu0443\x5cu\
0441\x5cu0442\x5cu0438\
\x5cu043c\x5cu044b\x5cu04\
35\x5cu043f\x5cu043e\x5cu\
0434\x5cu0441\x5cu0442\
\x5cu0430\x5cu043d\x5cu04\
3e\x5cu0432\x5cu043a\x5cu\
0438xs \x5cu043e\x5cu0\
431\x5cu0440\x5cu0430\x5c\
u0431\x5cu043e\x5cu044\
2\x5cu043a\x5cu0430\x5cu0\
43f\x5cu0440\x5cu043e\x5c\
u0431\x5cu0435\x5cu043\
b\x5cu044c\x5cu043d\x5cu0\
44b\x5cu0445\x5cu0441\x5c\
u0438\x5cu043c\x5cu043\
2\x5cu043e\x5cu043b\x5cu0\
43e\x5cu0432xs \x5cu04\
3e\x5cu0431\x5cu0440\x5cu\
0430\x5cu0431\x5cu043e\
\x5cu0442\x5cu043a\x5cu04\
30\x5cu0441\x5cu043e\x5cu\
0434\x5cu0435\x5cu0440\
\x5cu0436\x5cu0438\x5cu04\
3c\x5cu043e\x5cu0433\x5cu\
043exs \x5cu043e\x5cu0\
433\x5cu0440\x5cu0430\x5c\
u043d\x5cu0438\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu0437\x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu044fxs \x5cu04\
3f\x5cu0430\x5cu0440\x5cu\
0430\x5cu043c\x5cu0435\
\x5cu0442\x5cu0440\x5cu04\
4b\x5cu043e\x5cu0442\x5cu\
0431\x5cu043e\x5cu0440\
\x5cu0430\x5cu0443\x5cu04\
37\x5cu043b\x5cu043e\x5cu\
0432dom \x5cu043f\x5cu\
0435\x5cu0440\x5cu0435\
\x5cu043d\x5cu043e\x5cu04\
41\x5cu0441\x5cu0442\x5cu\
0440\x5cu043e\x5cu043a\
json \x5cu043f\x5cu043\
e\x5cu0437\x5cu0438\x5cu0\
446\x5cu0438\x5cu044f\x5c\
u0432\x5cu0434\x5cu043\
e\x5cu043a\x5cu0443\x5cu0\
43c\x5cu0435\x5cu043d\x5c\
u0442\x5cu0435dom \x5c\
u043f\x5cu0440\x5cu043\
e\x5cu0431\x5cu0435\x5cu0\
43b\x5cu044c\x5cu043d\x5c\
u044b\x5cu0435\x5cu044\
1\x5cu0438\x5cu043c\x5cu0\
432\x5cu043e\x5cu043b\x5c\
u044bxml \x5cu0442\x5c\
u0438\x5cu043f\x5cu043\
0\x5cu0442\x5cu0440\x5cu0\
438\x5cu0431\x5cu0443\x5c\
u0442\x5cu0430xml \x5c\
u0442\x5cu0438\x5cu043\
f\x5cu0437\x5cu043d\x5cu0\
430\x5cu0447\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
fjson \x5cu0442\x5cu04\
38\x5cu043f\x5cu043a\x5cu\
0430\x5cu043d\x5cu043e\
\x5cu043d\x5cu0438\x5cu04\
47\x5cu0435\x5cu0441\x5cu\
043a\x5cu043e\x5cu0433\
\x5cu043exml \x5cu0442\
\x5cu0438\x5cu043f\x5cu04\
3a\x5cu043e\x5cu043c\x5cu\
043f\x5cu043e\x5cu043d\
\x5cu0435\x5cu043d\x5cu04\
42\x5cu044bxs \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
43f\x5cu0440\x5cu043e\x5c\
u0432\x5cu0435\x5cu044\
0\x5cu043a\x5cu0438xml\
 \x5cu0442\x5cu0438\x5cu0\
43f\x5cu0440\x5cu0435\x5c\
u0437\x5cu0443\x5cu043\
b\x5cu044c\x5cu0442\x5cu0\
430\x5cu0442\x5cu0430d\
omxpath \x5cu0442\x5cu\
0438\x5cu043f\x5cu0443\
\x5cu0437\x5cu043b\x5cu04\
30dom \x5cu0442\x5cu04\
38\x5cu043f\x5cu0443\x5cu\
0437\x5cu043b\x5cu0430\
xml \x5cu0444\x5cu043e\
\x5cu0440\x5cu043c\x5cu04\
30xml \x5cu0444\x5cu04\
3e\x5cu0440\x5cu043c\x5cu\
0430\x5cu043f\x5cu0440\
\x5cu0435\x5cu0434\x5cu04\
41\x5cu0442\x5cu0430\x5cu\
0432\x5cu043b\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
4fxs \x5cu0444\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
430\x5cu0442\x5cu0434\x5c\
u0430\x5cu0442\x5cu044\
bjson \x5cu044d\x5cu04\
3a\x5cu0440\x5cu0430\x5cu\
043d\x5cu0438\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
0435\x5cu0441\x5cu0438\
\x5cu043c\x5cu0432\x5cu04\
3e\x5cu043b\x5cu043e\x5cu\
0432json \x5cu0432\x5c\
u0438\x5cu0434\x5cu044\
1\x5cu0440\x5cu0430\x5cu0\
432\x5cu043d\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f\x5cu043a\x5cu043e\x5cu0\
43c\x5cu043f\x5cu043e\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu043a\x5cu0438\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u043d\x5cu044b\x5cu044\
5 \x5cu0434\x5cu0435\x5cu\
0439\x5cu0441\x5cu0442\
\x5cu0432\x5cu0438\x5cu04\
35\x5cu043e\x5cu0431\x5cu\
0440\x5cu0430\x5cu0431\
\x5cu043e\x5cu0442\x5cu04\
3a\x5cu0438\x5cu0440\x5cu\
0430\x5cu0441\x5cu0448\
\x5cu0438\x5cu0444\x5cu04\
40\x5cu043e\x5cu0432\x5cu\
043a\x5cu0438\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu043e\x5cu043d\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0438\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu043\
d\x5cu0430\x5cu043f\x5cu0\
440\x5cu0430\x5cu0432\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
441\x5cu043e\x5cu0440\x5c\
u0442\x5cu0438\x5cu044\
0\x5cu043e\x5cu0432\x5cu0\
43a\x5cu0438\x5cu043a\x5c\
u043e\x5cu043c\x5cu043\
f\x5cu043e\x5cu043d\x5cu0\
43e\x5cu0432\x5cu043a\x5c\
u0438\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu0440\
\x5cu0430\x5cu0441\x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
043e\x5cu0436\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu0432\x5cu043b\x5cu\
043e\x5cu0436\x5cu0435\
\x5cu043d\x5cu043d\x5cu04\
4b\x5cu0445\x5cu044d\x5cu\
043b\x5cu0435\x5cu043c\
\x5cu0435\x5cu043d\x5cu04\
42\x5cu043e\x5cu0432\x5cu\
0440\x5cu0435\x5cu0437\
\x5cu0443\x5cu043b\x5cu04\
4c\x5cu0442\x5cu0430\x5cu\
0442\x5cu0430\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu043e\x5cu043d\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0438\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu044\
0\x5cu0430\x5cu0441\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u043e\x5cu0436\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu0438\x5cu0442\x5c\
u043e\x5cu0433\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
43e\x5cu043c\x5cu043f\x5c\
u043e\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
438\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 \x5cu0440\x5cu\
0430\x5cu0441\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
3e\x5cu0436\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0433\x5cu0440\x5cu04\
43\x5cu043f\x5cu043f\x5cu\
0438\x5cu0440\x5cu043e\
\x5cu0432\x5cu043a\x5cu04\
38\x5cu043a\x5cu043e\x5cu\
043c\x5cu043f\x5cu043e\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu043a\x5cu0438\x5cu\
0434\x5cu0430\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
45 \x5cu0440\x5cu0430\x5c\
u0441\x5cu043f\x5cu043\
e\x5cu043b\x5cu043e\x5cu0\
436\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
435\x5cu0439\x5cu0433\x5c\
u0440\x5cu0443\x5cu043\
f\x5cu043f\x5cu0438\x5cu0\
440\x5cu043e\x5cu0432\x5c\
u043a\x5cu0438\x5cu043\
a\x5cu043e\x5cu043c\x5cu0\
43f\x5cu043e\x5cu043d\x5c\
u043e\x5cu0432\x5cu043\
a\x5cu0438\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0445 \x5cu04\
40\x5cu0430\x5cu0441\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu043e\x5cu0436\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu043f\x5cu043e\
\x5cu043b\x5cu044f\x5cu04\
3a\x5cu043e\x5cu043c\x5cu\
043f\x5cu043e\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu0438\x5cu0434\x5cu\
0430\x5cu043d\x5cu043d\
\x5cu044b\x5cu0445 \x5cu0\
440\x5cu0430\x5cu0441\x5c\
u043f\x5cu043e\x5cu043\
b\x5cu043e\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435\x5cu0440\x5cu043\
5\x5cu043a\x5cu0432\x5cu0\
438\x5cu0437\x5cu0438\x5c\
u0442\x5cu043e\x5cu043\
2\x5cu043a\x5cu043e\x5cu0\
43c\x5cu043f\x5cu043e\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu043a\x5cu0438\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u043d\x5cu044b\x5cu044\
5 \x5cu0440\x5cu0430\x5cu\
0441\x5cu043f\x5cu043e\
\x5cu043b\x5cu043e\x5cu04\
36\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu0440\
\x5cu0435\x5cu0441\x5cu04\
43\x5cu0440\x5cu0441\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu043e\x5cu043d\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0438\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
431\x5cu0443\x5cu0445\x5c\
u0433\x5cu0430\x5cu043\
b\x5cu0442\x5cu0435\x5cu0\
440\x5cu0441\x5cu043a\x5c\
u043e\x5cu0433\x5cu043\
e\x5cu043e\x5cu0441\x5cu0\
442\x5cu0430\x5cu0442\x5c\
u043a\x5cu0430\x5cu043\
a\x5cu043e\x5cu043c\x5cu0\
43f\x5cu043e\x5cu043d\x5c\
u043e\x5cu0432\x5cu043\
a\x5cu0438\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0445 \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0432\x5cu044b\x5cu0432\
\x5cu043e\x5cu0434\x5cu04\
30\x5cu0442\x5cu0435\x5cu\
043a\x5cu0441\x5cu0442\
\x5cu0430\x5cu043a\x5cu04\
3e\x5cu043c\x5cu043f\x5cu\
043e\x5cu043d\x5cu043e\
\x5cu0432\x5cu043a\x5cu04\
38\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445 \x5cu0442\x5cu0\
438\x5cu043f\x5cu0433\x5c\
u0440\x5cu0443\x5cu043\
f\x5cu043f\x5cu0438\x5cu0\
440\x5cu043e\x5cu0432\x5c\
u043a\x5cu0438\x5cu043\
a\x5cu043e\x5cu043c\x5cu0\
43f\x5cu043e\x5cu043d\x5c\
u043e\x5cu0432\x5cu043\
a\x5cu0438\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0445 \x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0433\x5cu0440\x5cu0443\
\x5cu043f\x5cu043f\x5cu04\
4b\x5cu044d\x5cu043b\x5cu\
0435\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
3e\x5cu0432\x5cu043e\x5cu\
0442\x5cu0431\x5cu043e\
\x5cu0440\x5cu0430\x5cu04\
3a\x5cu043e\x5cu043c\x5cu\
043f\x5cu043e\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu0438\x5cu0434\x5cu\
0430\x5cu043d\x5cu043d\
\x5cu044b\x5cu0445 \x5cu0\
442\x5cu0438\x5cu043f\x5c\
u0434\x5cu043e\x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
43d\x5cu0435\x5cu043d\x5c\
u0438\x5cu044f\x5cu043\
f\x5cu0435\x5cu0440\x5cu0\
438\x5cu043e\x5cu0434\x5c\
u0430\x5cu043a\x5cu043\
e\x5cu043c\x5cu043f\x5cu0\
43e\x5cu043d\x5cu043e\x5c\
u0432\x5cu043a\x5cu043\
8\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu0442\x5cu04\
38\x5cu043f\x5cu0437\x5cu\
0430\x5cu0433\x5cu043e\
\x5cu043b\x5cu043e\x5cu04\
32\x5cu043a\x5cu0430\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu0435\x5cu0439\x5cu04\
3a\x5cu043e\x5cu043c\x5cu\
043f\x5cu043e\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu0438\x5cu0434\x5cu\
0430\x5cu043d\x5cu043d\
\x5cu044b\x5cu0445 \x5cu0\
442\x5cu0438\x5cu043f\x5c\
u043c\x5cu0430\x5cu043\
a\x5cu0435\x5cu0442\x5cu0\
430\x5cu0433\x5cu0440\x5c\
u0443\x5cu043f\x5cu043\
f\x5cu0438\x5cu0440\x5cu0\
43e\x5cu0432\x5cu043a\x5c\
u0438\x5cu043a\x5cu043\
e\x5cu043c\x5cu043f\x5cu0\
43e\x5cu043d\x5cu043e\x5c\
u0432\x5cu043a\x5cu043\
8\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu0442\x5cu04\
38\x5cu043f\x5cu043c\x5cu\
0430\x5cu043a\x5cu0435\
\x5cu0442\x5cu0430\x5cu04\
3e\x5cu0431\x5cu043b\x5cu\
0430\x5cu0441\x5cu0442\
\x5cu0438\x5cu043a\x5cu04\
3e\x5cu043c\x5cu043f\x5cu\
043e\x5cu043d\x5cu043e\
\x5cu0432\x5cu043a\x5cu04\
38\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445 \x5cu0442\x5cu0\
438\x5cu043f\x5cu043e\x5c\
u0441\x5cu0442\x5cu043\
0\x5cu0442\x5cu043a\x5cu0\
430\x5cu043a\x5cu043e\x5c\
u043c\x5cu043f\x5cu043\
e\x5cu043d\x5cu043e\x5cu0\
432\x5cu043a\x5cu0438\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445 \x5cu0442\x5cu0438\
\x5cu043f\x5cu043f\x5cu04\
35\x5cu0440\x5cu0438\x5cu\
043e\x5cu0434\x5cu0430\
\x5cu043a\x5cu043e\x5cu04\
3c\x5cu043f\x5cu043e\x5cu\
043d\x5cu043e\x5cu0432\
\x5cu043a\x5cu0438\x5cu04\
34\x5cu0430\x5cu043d\x5cu\
043d\x5cu044b\x5cu0445\
 \x5cu0442\x5cu0438\x5cu0\
43f\x5cu0440\x5cu0430\x5c\
u0437\x5cu043c\x5cu043\
5\x5cu0449\x5cu0435\x5cu0\
43d\x5cu0438\x5cu044f\x5c\
u0442\x5cu0435\x5cu043\
a\x5cu0441\x5cu0442\x5cu0\
430\x5cu043a\x5cu043e\x5c\
u043c\x5cu043f\x5cu043\
e\x5cu043d\x5cu043e\x5cu0\
432\x5cu043a\x5cu0438\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445 \x5cu0442\x5cu0438\
\x5cu043f\x5cu0441\x5cu04\
32\x5cu044f\x5cu0437\x5cu\
0438\x5cu043d\x5cu0430\
\x5cu0431\x5cu043e\x5cu04\
40\x5cu043e\x5cu0432\x5cu\
0434\x5cu0430\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
45\x5cu043a\x5cu043e\x5cu\
043c\x5cu043f\x5cu043e\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu043a\x5cu0438\x5cu\
0434\x5cu0430\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
45 \x5cu0442\x5cu0438\x5c\
u043f\x5cu044d\x5cu043\
b\x5cu0435\x5cu043c\x5cu0\
435\x5cu043d\x5cu0442\x5c\
u0430\x5cu0440\x5cu043\
5\x5cu0437\x5cu0443\x5cu0\
43b\x5cu044c\x5cu0442\x5c\
u0430\x5cu0442\x5cu043\
0\x5cu043a\x5cu043e\x5cu0\
43c\x5cu043f\x5cu043e\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu043a\x5cu0438\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u043d\x5cu044b\x5cu044\
5 \x5cu0440\x5cu0430\x5cu\
0441\x5cu043f\x5cu043e\
\x5cu043b\x5cu043e\x5cu04\
36\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu043b\
\x5cu0435\x5cu0433\x5cu04\
35\x5cu043d\x5cu0434\x5cu\
044b\x5cu0434\x5cu0438\
\x5cu0430\x5cu0433\x5cu04\
40\x5cu0430\x5cu043c\x5cu\
043c\x5cu044b\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu043e\x5cu043d\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0438\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
43f\x5cu0440\x5cu0438\x5c\
u043c\x5cu0435\x5cu043\
d\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu043e\x5c\
u0442\x5cu0431\x5cu043\
e\x5cu0440\x5cu0430\x5cu0\
43a\x5cu043e\x5cu043c\x5c\
u043f\x5cu043e\x5cu043\
d\x5cu043e\x5cu0432\x5cu0\
43a\x5cu0438\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445 \x5cu\
0440\x5cu0435\x5cu0436\
\x5cu0438\x5cu043c\x5cu04\
3e\x5cu0442\x5cu043e\x5cu\
0431\x5cu0440\x5cu0430\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu044f\x5cu\
044d\x5cu043b\x5cu0435\
\x5cu043c\x5cu0435\x5cu04\
3d\x5cu0442\x5cu0430\x5cu\
043d\x5cu0430\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
3e\x5cu0439\x5cu043a\x5cu\
0438\x5cu043a\x5cu043e\
\x5cu043c\x5cu043f\x5cu04\
3e\x5cu043d\x5cu043e\x5cu\
0432\x5cu043a\x5cu0438\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu043d\x5cu044b\x5cu\
0445 \x5cu0440\x5cu043\
5\x5cu0436\x5cu0438\x5cu0\
43c\x5cu043e\x5cu0442\x5c\
u043e\x5cu0431\x5cu044\
0\x5cu0430\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044f\x5cu043d\x5cu043\
0\x5cu0441\x5cu0442\x5cu0\
440\x5cu043e\x5cu0435\x5c\
u043a\x5cu043a\x5cu043\
e\x5cu043c\x5cu043f\x5cu0\
43e\x5cu043d\x5cu043e\x5c\
u0432\x5cu043a\x5cu043\
8\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu0441\x5cu04\
3e\x5cu0441\x5cu0442\x5cu\
043e\x5cu044f\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
4d\x5cu043b\x5cu0435\x5cu\
043c\x5cu0435\x5cu043d\
\x5cu0442\x5cu0430\x5cu04\
3d\x5cu0430\x5cu0441\x5cu\
0442\x5cu0440\x5cu043e\
\x5cu0439\x5cu043a\x5cu04\
38\x5cu043a\x5cu043e\x5cu\
043c\x5cu043f\x5cu043e\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu043a\x5cu0438\x5cu\
0434\x5cu0430\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
45 \x5cu0441\x5cu043f\x5c\
u043e\x5cu0441\x5cu043\
e\x5cu0431\x5cu0432\x5cu0\
43e\x5cu0441\x5cu0441\x5c\
u0442\x5cu0430\x5cu043\
d\x5cu043e\x5cu0432\x5cu0\
43b\x5cu0435\x5cu043d\x5c\
u0438\x5cu044f\x5cu043\
d\x5cu0430\x5cu0441\x5cu0\
442\x5cu0440\x5cu043e\x5c\
u0435\x5cu043a\x5cu043\
a\x5cu043e\x5cu043c\x5cu0\
43f\x5cu043e\x5cu043d\x5c\
u043e\x5cu0432\x5cu043\
a\x5cu0438\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0445 \x5cu04\
40\x5cu0435\x5cu0436\x5cu\
0438\x5cu043c\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu043e\x5cu043d\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0438\x5cu0440\x5cu04\
35\x5cu0437\x5cu0443\x5cu\
043b\x5cu044c\x5cu0442\
\x5cu0430\x5cu0442\x5cu04\
30 \x5cu0438\x5cu0441\x5c\
u043f\x5cu043e\x5cu043\
b\x5cu044c\x5cu0437\x5cu0\
43e\x5cu0432\x5cu0430\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu043f\x5cu0430\x5cu0\
440\x5cu0430\x5cu043c\x5c\
u0435\x5cu0442\x5cu044\
0\x5cu0430\x5cu043a\x5cu0\
43e\x5cu043c\x5cu043f\x5c\
u043e\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
438\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 \x5cu0430\x5cu\
0432\x5cu0442\x5cu043e\
\x5cu043f\x5cu043e\x5cu04\
37\x5cu0438\x5cu0446\x5cu\
0438\x5cu044f\x5cu0440\
\x5cu0435\x5cu0441\x5cu04\
43\x5cu0440\x5cu0441\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu043e\x5cu043d\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0438\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu043\
2\x5cu0430\x5cu0440\x5cu0\
438\x5cu0430\x5cu043d\x5c\
u0442\x5cu0438\x5cu044\
1\x5cu043f\x5cu043e\x5cu0\
43b\x5cu044c\x5cu0437\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
44f\x5cu0433\x5cu0440\x5c\
u0443\x5cu043f\x5cu043\
f\x5cu0438\x5cu0440\x5cu0\
43e\x5cu0432\x5cu043a\x5c\
u0438\x5cu043a\x5cu043\
e\x5cu043c\x5cu043f\x5cu0\
43e\x5cu043d\x5cu043e\x5c\
u0432\x5cu043a\x5cu043\
8\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu0440\x5cu04\
30\x5cu0441\x5cu043f\x5cu\
043e\x5cu043b\x5cu043e\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
0440\x5cu0435\x5cu0441\
\x5cu0443\x5cu0440\x5cu04\
41\x5cu043e\x5cu0432\x5cu\
0432\x5cu0434\x5cu0438\
\x5cu0430\x5cu0433\x5cu04\
40\x5cu0430\x5cu043c\x5cu\
043c\x5cu0435\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu043e\x5cu043d\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0438\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu044\
4\x5cu0438\x5cu043a\x5cu0\
441\x5cu0430\x5cu0446\x5c\
u0438\x5cu044f\x5cu043\
a\x5cu043e\x5cu043c\x5cu0\
43f\x5cu043e\x5cu043d\x5c\
u043e\x5cu0432\x5cu043\
a\x5cu0438\x5cu0434\x5cu0\
430\x5cu043d\x5cu043d\x5c\
u044b\x5cu0445 \x5cu04\
38\x5cu0441\x5cu043f\x5cu\
043e\x5cu043b\x5cu044c\
\x5cu0437\x5cu043e\x5cu04\
32\x5cu0430\x5cu043d\x5cu\
0438\x5cu0435\x5cu0443\
\x5cu0441\x5cu043b\x5cu04\
3e\x5cu0432\x5cu043d\x5cu\
043e\x5cu0433\x5cu043e\
\x5cu043e\x5cu0444\x5cu04\
3e\x5cu0440\x5cu043c\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
3a\x5cu043e\x5cu043c\x5cu\
043f\x5cu043e\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu0438\x5cu0434\x5cu\
0430\x5cu043d\x5cu043d\
\x5cu044b\x5cu0445 \x5cu0\
432\x5cu0430\x5cu0436\x5c\
u043d\x5cu043e\x5cu044\
1\x5cu0442\x5cu044c\x5cu0\
438\x5cu043d\x5cu0442\x5c\
u0435\x5cu0440\x5cu043\
d\x5cu0435\x5cu0442\x5cu0\
43f\x5cu043e\x5cu0447\x5c\
u0442\x5cu043e\x5cu043\
2\x5cu043e\x5cu0433\x5cu0\
43e\x5cu0441\x5cu043e\x5c\
u043e\x5cu0431\x5cu044\
9\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f \x5cu043e\
\x5cu0431\x5cu0440\x5cu04\
30\x5cu0431\x5cu043e\x5cu\
0442\x5cu043a\x5cu0430\
\x5cu0442\x5cu0435\x5cu04\
3a\x5cu0441\x5cu0442\x5cu\
0430\x5cu0438\x5cu043d\
\x5cu0442\x5cu0435\x5cu04\
40\x5cu043d\x5cu0435\x5cu\
0442\x5cu043f\x5cu043e\
\x5cu0447\x5cu0442\x5cu04\
3e\x5cu0432\x5cu043e\x5cu\
0433\x5cu043e\x5cu0441\
\x5cu043e\x5cu043e\x5cu04\
31\x5cu0449\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
 \x5cu0441\x5cu043f\x5cu0\
43e\x5cu0441\x5cu043e\x5c\
u0431\x5cu043a\x5cu043\
e\x5cu0434\x5cu0438\x5cu0\
440\x5cu043e\x5cu0432\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu044f\x5cu0438\x5cu0\
43d\x5cu0442\x5cu0435\x5c\
u0440\x5cu043d\x5cu043\
5\x5cu0442\x5cu043f\x5cu0\
43e\x5cu0447\x5cu0442\x5c\
u043e\x5cu0432\x5cu043\
e\x5cu0433\x5cu043e\x5cu0\
432\x5cu043b\x5cu043e\x5c\
u0436\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f \x5cu\
0441\x5cu043f\x5cu043e\
\x5cu0441\x5cu043e\x5cu04\
31\x5cu043a\x5cu043e\x5cu\
0434\x5cu0438\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
044f\x5cu043d\x5cu0435\
ascii\x5cu0441\x5cu043\
8\x5cu043c\x5cu0432\x5cu0\
43e\x5cu043b\x5cu043e\x5c\
u0432\x5cu0438\x5cu043\
d\x5cu0442\x5cu0435\x5cu0\
440\x5cu043d\x5cu0435\x5c\
u0442\x5cu043f\x5cu043\
e\x5cu0447\x5cu0442\x5cu0\
43e\x5cu0432\x5cu043e\x5c\
u0433\x5cu043e\x5cu044\
1\x5cu043e\x5cu043e\x5cu0\
431\x5cu0449\x5cu0435\x5c\
u043d\x5cu0438\x5cu044\
f \x5cu0442\x5cu0438\x5cu\
043f\x5cu0442\x5cu0435\
\x5cu043a\x5cu0441\x5cu04\
42\x5cu0430\x5cu043f\x5cu\
043e\x5cu0447\x5cu0442\
\x5cu043e\x5cu0432\x5cu04\
3e\x5cu0433\x5cu043e\x5cu\
0441\x5cu043e\x5cu043e\
\x5cu0431\x5cu0449\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f \x5cu043f\x5cu044\
0\x5cu043e\x5cu0442\x5cu0\
43e\x5cu043a\x5cu043e\x5c\
u043b\x5cu0438\x5cu043\
d\x5cu0442\x5cu0435\x5cu0\
440\x5cu043d\x5cu0435\x5c\
u0442\x5cu043f\x5cu043\
e\x5cu0447\x5cu0442\x5cu0\
44b \x5cu0441\x5cu0442\
\x5cu0430\x5cu0442\x5cu04\
43\x5cu0441\x5cu0440\x5cu\
0430\x5cu0437\x5cu0431\
\x5cu043e\x5cu0440\x5cu04\
30\x5cu043f\x5cu043e\x5cu\
0447\x5cu0442\x5cu043e\
\x5cu0432\x5cu043e\x5cu04\
33\x5cu043e\x5cu0441\x5cu\
043e\x5cu043e\x5cu0431\
\x5cu0449\x5cu0435\x5cu04\
3d\x5cu0438\x5cu044f \x5c\
u0440\x5cu0435\x5cu043\
6\x5cu0438\x5cu043c\x5cu0\
442\x5cu0440\x5cu0430\x5c\
u043d\x5cu0437\x5cu043\
0\x5cu043a\x5cu0446\x5cu0\
438\x5cu0438\x5cu0437\x5c\
u0430\x5cu043f\x5cu043\
8\x5cu0441\x5cu0438\x5cu0\
436\x5cu0443\x5cu0440\x5c\
u043d\x5cu0430\x5cu043\
b\x5cu0430\x5cu0440\x5cu0\
435\x5cu0433\x5cu0438\x5c\
u0441\x5cu0442\x5cu044\
0\x5cu0430\x5cu0446\x5cu0\
438\x5cu0438 \x5cu0441\
\x5cu0442\x5cu0430\x5cu04\
42\x5cu0443\x5cu0441\x5cu\
0442\x5cu0440\x5cu0430\
\x5cu043d\x5cu0437\x5cu04\
30\x5cu043a\x5cu0446\x5cu\
0438\x5cu0438\x5cu0437\
\x5cu0430\x5cu043f\x5cu04\
38\x5cu0441\x5cu0438\x5cu\
0436\x5cu0443\x5cu0440\
\x5cu043d\x5cu0430\x5cu04\
3b\x5cu0430\x5cu0440\x5cu\
0435\x5cu0433\x5cu0438\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu0430\x5cu0446\x5cu\
0438\x5cu0438 \x5cu044\
3\x5cu0440\x5cu043e\x5cu0\
432\x5cu0435\x5cu043d\x5c\
u044c\x5cu0436\x5cu044\
3\x5cu0440\x5cu043d\x5cu0\
430\x5cu043b\x5cu0430\x5c\
u0440\x5cu0435\x5cu043\
3\x5cu0438\x5cu0441\x5cu0\
442\x5cu0440\x5cu0430\x5c\
u0446\x5cu0438\x5cu043\
8 \x5cu0440\x5cu0430\x5cu\
0441\x5cu043f\x5cu043e\
\x5cu043b\x5cu043e\x5cu04\
36\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu0445\
\x5cu0440\x5cu0430\x5cu04\
3d\x5cu0438\x5cu043b\x5cu\
0438\x5cu0449\x5cu0430\
\x5cu0441\x5cu0435\x5cu04\
40\x5cu0442\x5cu0438\x5cu\
0444\x5cu0438\x5cu043a\
\x5cu0430\x5cu0442\x5cu04\
3e\x5cu0432\x5cu043a\x5cu\
0440\x5cu0438\x5cu043f\
\x5cu0442\x5cu043e\x5cu04\
33\x5cu0440\x5cu0430\x5cu\
0444\x5cu0438\x5cu0438\
 \x5cu0440\x5cu0435\x5cu0\
436\x5cu0438\x5cu043c\x5c\
u0432\x5cu043a\x5cu043\
b\x5cu044e\x5cu0447\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044f\x5cu0441\x5cu043\
5\x5cu0440\x5cu0442\x5cu0\
438\x5cu0444\x5cu0438\x5c\
u043a\x5cu0430\x5cu044\
2\x5cu043e\x5cu0432\x5cu0\
43a\x5cu0440\x5cu0438\x5c\
u043f\x5cu0442\x5cu043\
e\x5cu0433\x5cu0440\x5cu0\
430\x5cu0444\x5cu0438\x5c\
u0438 \x5cu0440\x5cu04\
35\x5cu0436\x5cu0438\x5cu\
043c\x5cu043f\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
35\x5cu0440\x5cu043a\x5cu\
0438\x5cu0441\x5cu0435\
\x5cu0440\x5cu0442\x5cu04\
38\x5cu0444\x5cu0438\x5cu\
043a\x5cu0430\x5cu0442\
\x5cu0430\x5cu043a\x5cu04\
40\x5cu0438\x5cu043f\x5cu\
0442\x5cu043e\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
44\x5cu0438\x5cu0438 \x5c\
u0442\x5cu0438\x5cu043\
f\x5cu0445\x5cu0440\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u043b\x5cu0438\x5cu044\
9\x5cu0430\x5cu0441\x5cu0\
435\x5cu0440\x5cu0442\x5c\
u0438\x5cu0444\x5cu043\
8\x5cu043a\x5cu0430\x5cu0\
442\x5cu043e\x5cu0432\x5c\
u043a\x5cu0440\x5cu043\
8\x5cu043f\x5cu0442\x5cu0\
43e\x5cu0433\x5cu0440\x5c\
u0430\x5cu0444\x5cu043\
8\x5cu0438 \x5cu043a\x5cu\
043e\x5cu0434\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu043a\x5cu0430\x5cu\
0438\x5cu043c\x5cu0435\
\x5cu043d\x5cu0444\x5cu04\
30\x5cu0439\x5cu043b\x5cu\
043e\x5cu0432\x5cu0432\
zip\x5cu0444\x5cu0430\x5c\
u0439\x5cu043b\x5cu043\
5 \x5cu043c\x5cu0435\x5cu\
0442\x5cu043e\x5cu0434\
\x5cu0441\x5cu0436\x5cu04\
30\x5cu0442\x5cu0438\x5cu\
044fzip \x5cu043c\x5cu\
0435\x5cu0442\x5cu043e\
\x5cu0434\x5cu0448\x5cu04\
38\x5cu0444\x5cu0440\x5cu\
043e\x5cu0432\x5cu0430\
\x5cu043d\x5cu0438\x5cu04\
4fzip \x5cu0440\x5cu04\
35\x5cu0436\x5cu0438\x5cu\
043c\x5cu0432\x5cu043e\
\x5cu0441\x5cu0441\x5cu04\
42\x5cu0430\x5cu043d\x5cu\
043e\x5cu0432\x5cu043b\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f\x5cu043f\x5cu\
0443\x5cu0442\x5cu0435\
\x5cu0439\x5cu0444\x5cu04\
30\x5cu0439\x5cu043b\x5cu\
043e\x5cu0432zip \x5cu\
0440\x5cu0435\x5cu0436\
\x5cu0438\x5cu043c\x5cu04\
3e\x5cu0431\x5cu0440\x5cu\
0430\x5cu0431\x5cu043e\
\x5cu0442\x5cu043a\x5cu04\
38\x5cu043f\x5cu043e\x5cu\
0434\x5cu043a\x5cu0430\
\x5cu0442\x5cu0430\x5cu04\
3b\x5cu043e\x5cu0433\x5cu\
043e\x5cu0432zip \x5cu\
0440\x5cu0435\x5cu0436\
\x5cu0438\x5cu043c\x5cu04\
41\x5cu043e\x5cu0445\x5cu\
0440\x5cu0430\x5cu043d\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f\x5cu043f\x5cu\
0443\x5cu0442\x5cu0435\
\x5cu0439zip \x5cu0443\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0435\x5cu043d\x5cu\
044c\x5cu0441\x5cu0436\
\x5cu0430\x5cu0442\x5cu04\
38\x5cu044fzip \x5cu04\
37\x5cu0432\x5cu0443\x5cu\
043a\x5cu043e\x5cu0432\
\x5cu043e\x5cu0435\x5cu04\
3e\x5cu043f\x5cu043e\x5cu\
0432\x5cu0435\x5cu0449\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435 \x5cu043d\x5c\
u0430\x5cu043f\x5cu044\
0\x5cu0430\x5cu0432\x5cu0\
43b\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu043\
f\x5cu0435\x5cu0440\x5cu0\
435\x5cu0445\x5cu043e\x5c\
u0434\x5cu0430\x5cu043\
a\x5cu0441\x5cu0442\x5cu0\
440\x5cu043e\x5cu043a\x5c\
u0435 \x5cu043f\x5cu04\
3e\x5cu0437\x5cu0438\x5cu\
0446\x5cu0438\x5cu044f\
\x5cu0432\x5cu043f\x5cu04\
3e\x5cu0442\x5cu043e\x5cu\
043a\x5cu0435 \x5cu043\
f\x5cu043e\x5cu0440\x5cu0\
44f\x5cu0434\x5cu043e\x5c\
u043a\x5cu0431\x5cu043\
0\x5cu0439\x5cu0442\x5cu0\
43e\x5cu0432 \x5cu0440\
\x5cu0435\x5cu0436\x5cu04\
38\x5cu043c\x5cu0431\x5cu\
043b\x5cu043e\x5cu043a\
\x5cu0438\x5cu0440\x5cu04\
3e\x5cu0432\x5cu043a\x5cu\
0438\x5cu0434\x5cu0430\
\x5cu043d\x5cu043d\x5cu04\
4b\x5cu0445 \x5cu0440\x5c\
u0435\x5cu0436\x5cu043\
8\x5cu043c\x5cu0443\x5cu0\
43f\x5cu0440\x5cu0430\x5c\
u0432\x5cu043b\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f\x5cu0431\x5cu043b\x5c\
u043e\x5cu043a\x5cu043\
8\x5cu0440\x5cu043e\x5cu0\
432\x5cu043a\x5cu043e\x5c\
u0439\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu0441\
\x5cu0435\x5cu0440\x5cu04\
32\x5cu0438\x5cu0441\x5cu\
0432\x5cu0441\x5cu0442\
\x5cu0440\x5cu043e\x5cu04\
35\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445\x5cu043f\
\x5cu043e\x5cu043a\x5cu04\
43\x5cu043f\x5cu043e\x5cu\
043a \x5cu0441\x5cu043\
e\x5cu0441\x5cu0442\x5cu0\
43e\x5cu044f\x5cu043d\x5c\
u0438\x5cu0435\x5cu044\
4\x5cu043e\x5cu043d\x5cu0\
43e\x5cu0432\x5cu043e\x5c\
u0433\x5cu043e\x5cu043\
7\x5cu0430\x5cu0434\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u044f \x5cu0442\x5cu04\
38\x5cu043f\x5cu043f\x5cu\
043e\x5cu0434\x5cu043f\
\x5cu0438\x5cu0441\x5cu04\
47\x5cu0438\x5cu043a\x5cu\
0430\x5cu0434\x5cu043e\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu0432\x5cu043b\x5cu\
044f\x5cu0435\x5cu043c\
\x5cu044b\x5cu0445\x5cu04\
43\x5cu0432\x5cu0435\x5cu\
0434\x5cu043e\x5cu043c\
\x5cu043b\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0439 \x5c\
u0443\x5cu0440\x5cu043\
e\x5cu0432\x5cu0435\x5cu0\
43d\x5cu044c\x5cu0438\x5c\
u0441\x5cu043f\x5cu043\
e\x5cu043b\x5cu044c\x5cu0\
437\x5cu043e\x5cu0432\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu044f\x5cu0437\x5cu0\
430\x5cu0449\x5cu0438\x5c\
u0449\x5cu0435\x5cu043\
d\x5cu043d\x5cu043e\x5cu0\
433\x5cu043e\x5cu0441\x5c\
u043e\x5cu0435\x5cu043\
4\x5cu0438\x5cu043d\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044fftp \x5cu043d\x5c\
u0430\x5cu043f\x5cu044\
0\x5cu0430\x5cu0432\x5cu0\
43b\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435\x5cu043\
f\x5cu043e\x5cu0440\x5cu0\
44f\x5cu0434\x5cu043a\x5c\
u0430\x5cu0441\x5cu044\
5\x5cu0435\x5cu043c\x5cu0\
44b\x5cu0437\x5cu0430\x5c\
u043f\x5cu0440\x5cu043\
e\x5cu0441\x5cu0430 \x5cu\
0442\x5cu0438\x5cu043f\
\x5cu0434\x5cu043e\x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
043d\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
3f\x5cu0435\x5cu0440\x5cu\
0438\x5cu043e\x5cu0434\
\x5cu0430\x5cu043c\x5cu04\
38\x5cu0441\x5cu0445\x5cu\
0435\x5cu043c\x5cu044b\
\x5cu0437\x5cu0430\x5cu04\
3f\x5cu0440\x5cu043e\x5cu\
0441\x5cu0430 \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
43a\x5cu043e\x5cu043d\x5c\
u0442\x5cu0440\x5cu043\
e\x5cu043b\x5cu044c\x5cu0\
43d\x5cu043e\x5cu0439\x5c\
u0442\x5cu043e\x5cu044\
7\x5cu043a\x5cu0438\x5cu0\
441\x5cu0445\x5cu0435\x5c\
u043c\x5cu044b\x5cu043\
7\x5cu0430\x5cu043f\x5cu0\
440\x5cu043e\x5cu0441\x5c\
u0430 \x5cu0442\x5cu04\
38\x5cu043f\x5cu043e\x5cu\
0431\x5cu044a\x5cu0435\
\x5cu0434\x5cu0438\x5cu04\
3d\x5cu0435\x5cu043d\x5cu\
0438\x5cu044f\x5cu0441\
\x5cu0445\x5cu0435\x5cu04\
3c\x5cu044b\x5cu0437\x5cu\
0430\x5cu043f\x5cu0440\
\x5cu043e\x5cu0441\x5cu04\
30 \x5cu0442\x5cu0438\x5c\
u043f\x5cu043f\x5cu043\
0\x5cu0440\x5cu0430\x5cu0\
43c\x5cu0435\x5cu0442\x5c\
u0440\x5cu0430\x5cu043\
4\x5cu043e\x5cu0441\x5cu0\
442\x5cu0443\x5cu043f\x5c\
u043d\x5cu043e\x5cu043\
9\x5cu0442\x5cu0430\x5cu0\
431\x5cu043b\x5cu0438\x5c\
u0446\x5cu044b\x5cu044\
1\x5cu0445\x5cu0435\x5cu0\
43c\x5cu044b\x5cu0437\x5c\
u0430\x5cu043f\x5cu044\
0\x5cu043e\x5cu0441\x5cu0\
430 \x5cu0442\x5cu0438\
\x5cu043f\x5cu0441\x5cu04\
3e\x5cu0435\x5cu0434\x5cu\
0438\x5cu043d\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
4f\x5cu0441\x5cu0445\x5cu\
0435\x5cu043c\x5cu044b\
\x5cu0437\x5cu0430\x5cu04\
3f\x5cu0440\x5cu043e\x5cu\
0441\x5cu0430 http\x5c\
u043c\x5cu0435\x5cu044\
2\x5cu043e\x5cu0434 \x5cu\
0430\x5cu0432\x5cu0442\
\x5cu043e\x5cu0438\x5cu04\
41\x5cu043f\x5cu043e\x5cu\
043b\x5cu044c\x5cu0437\
\x5cu043e\x5cu0432\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
0435\x5cu043e\x5cu0431\
\x5cu0449\x5cu0435\x5cu04\
33\x5cu043e\x5cu0440\x5cu\
0435\x5cu043a\x5cu0432\
\x5cu0438\x5cu0437\x5cu04\
38\x5cu0442\x5cu0430 \x5c\
u0430\x5cu0432\x5cu044\
2\x5cu043e\x5cu043f\x5cu0\
440\x5cu0435\x5cu0444\x5c\
u0438\x5cu043a\x5cu044\
1\x5cu043d\x5cu043e\x5cu0\
43c\x5cu0435\x5cu0440\x5c\
u0430\x5cu0437\x5cu043\
0\x5cu0434\x5cu0430\x5cu0\
447\x5cu0438 \x5cu0432\
\x5cu0430\x5cu0440\x5cu04\
38\x5cu0430\x5cu043d\x5cu\
0442\x5cu0432\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
3e\x5cu0435\x5cu043d\x5cu\
043d\x5cu043e\x5cu0433\
\x5cu043e\x5cu044f\x5cu04\
37\x5cu044b\x5cu043a\x5cu\
0430 \x5cu0432\x5cu043\
8\x5cu0434\x5cu0438\x5cu0\
435\x5cu0440\x5cu0430\x5c\
u0440\x5cu0445\x5cu043\
8\x5cu0438 \x5cu0432\x5cu\
0438\x5cu0434\x5cu0440\
\x5cu0435\x5cu0433\x5cu04\
38\x5cu0441\x5cu0442\x5cu\
0440\x5cu0430\x5cu043d\
\x5cu0430\x5cu043a\x5cu04\
3e\x5cu043f\x5cu043b\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f \x5cu0432\x5cu0\
438\x5cu0434\x5cu0442\x5c\
u0430\x5cu0431\x5cu043\
b\x5cu0438\x5cu0446\x5cu0\
44b\x5cu0432\x5cu043d\x5c\
u0435\x5cu0448\x5cu043\
d\x5cu0435\x5cu0433\x5cu0\
43e\x5cu0438\x5cu0441\x5c\
u0442\x5cu043e\x5cu044\
7\x5cu043d\x5cu0438\x5cu0\
43a\x5cu0430\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445 \x5cu\
0437\x5cu0430\x5cu043f\
\x5cu0438\x5cu0441\x5cu04\
4c\x5cu0434\x5cu0432\x5cu\
0438\x5cu0436\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
39\x5cu043f\x5cu0440\x5cu\
0438\x5cu043f\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
35\x5cu0434\x5cu0435\x5cu\
043d\x5cu0438\x5cu0438\
 \x5cu0437\x5cu0430\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u043d\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
43f\x5cu043e\x5cu0441\x5c\
u043b\x5cu0435\x5cu043\
4\x5cu043e\x5cu0432\x5cu0\
430\x5cu0442\x5cu0435\x5c\
u043b\x5cu044c\x5cu043\
d\x5cu043e\x5cu0441\x5cu0\
442\x5cu0435\x5cu0439 \
\x5cu0438\x5cu043d\x5cu04\
34\x5cu0435\x5cu043a\x5cu\
0441\x5cu0438\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
0435 \x5cu0438\x5cu044\
1\x5cu043f\x5cu043e\x5cu0\
43b\x5cu044c\x5cu0437\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
435\x5cu0431\x5cu0430\x5c\
u0437\x5cu044b\x5cu043\
f\x5cu043b\x5cu0430\x5cu0\
43d\x5cu0430\x5cu0432\x5c\
u0438\x5cu0434\x5cu043\
e\x5cu0432\x5cu0440\x5cu0\
430\x5cu0441\x5cu0447\x5c\
u0435\x5cu0442\x5cu043\
0 \x5cu0438\x5cu0441\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu044c\x5cu0437\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0431\x5cu044b\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
043e\x5cu0433\x5cu043e\
\x5cu0432\x5cu044b\x5cu04\
31\x5cu043e\x5cu0440\x5cu\
0430 \x5cu0438\x5cu044\
1\x5cu043f\x5cu043e\x5cu0\
43b\x5cu044c\x5cu0437\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
435\x5cu043e\x5cu0431\x5c\
u0449\x5cu0435\x5cu043\
3\x5cu043e\x5cu0440\x5cu0\
435\x5cu043a\x5cu0432\x5c\
u0438\x5cu0437\x5cu043\
8\x5cu0442\x5cu0430 \x5cu\
0438\x5cu0441\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
4c\x5cu0437\x5cu043e\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
3f\x5cu043e\x5cu0434\x5cu\
0447\x5cu0438\x5cu043d\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f \x5cu0438\x5c\
u0441\x5cu043f\x5cu043\
e\x5cu043b\x5cu044c\x5cu0\
437\x5cu043e\x5cu0432\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu0435\x5cu043f\x5cu0\
43e\x5cu043b\x5cu043d\x5c\
u043e\x5cu0442\x5cu043\
5\x5cu043a\x5cu0441\x5cu0\
442\x5cu043e\x5cu0432\x5c\
u043e\x5cu0433\x5cu043\
e\x5cu043f\x5cu043e\x5cu0\
438\x5cu0441\x5cu043a\x5c\
u0430 \x5cu0438\x5cu04\
41\x5cu043f\x5cu043e\x5cu\
043b\x5cu044c\x5cu0437\
\x5cu043e\x5cu0432\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
0435\x5cu0440\x5cu0430\
\x5cu0437\x5cu0434\x5cu04\
35\x5cu043b\x5cu044f\x5cu\
0435\x5cu043c\x5cu044b\
\x5cu0445\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445\x5cu043e\
\x5cu0431\x5cu0449\x5cu04\
35\x5cu0433\x5cu043e\x5cu\
0440\x5cu0435\x5cu043a\
\x5cu0432\x5cu0438\x5cu04\
37\x5cu0438\x5cu0442\x5cu\
0430 \x5cu0438\x5cu044\
1\x5cu043f\x5cu043e\x5cu0\
43b\x5cu044c\x5cu0437\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
435\x5cu0440\x5cu0435\x5c\
u043a\x5cu0432\x5cu043\
8\x5cu0437\x5cu0438\x5cu0\
442\x5cu0430 \x5cu043d\
\x5cu0430\x5cu0437\x5cu04\
3d\x5cu0430\x5cu0447\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu0435\x5cu0438\x5cu04\
41\x5cu043f\x5cu043e\x5cu\
043b\x5cu044c\x5cu0437\
\x5cu043e\x5cu0432\x5cu04\
30\x5cu043d\x5cu0438\x5cu\
044f\x5cu043f\x5cu0440\
\x5cu0438\x5cu043b\x5cu04\
3e\x5cu0436\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
 \x5cu043d\x5cu0430\x5cu0\
437\x5cu043d\x5cu0430\x5c\
u0447\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
440\x5cu0430\x5cu0441\x5c\
u0448\x5cu0438\x5cu044\
0\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu043a\x5c\
u043e\x5cu043d\x5cu044\
4\x5cu0438\x5cu0433\x5cu0\
443\x5cu0440\x5cu0430\x5c\
u0446\x5cu0438\x5cu043\
8 \x5cu043d\x5cu0430\x5cu\
043f\x5cu0440\x5cu0430\
\x5cu0432\x5cu043b\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu043f\x5cu0435\
\x5cu0440\x5cu0435\x5cu04\
34\x5cu0430\x5cu0447\x5cu\
0438 \x5cu043e\x5cu043\
1\x5cu043d\x5cu043e\x5cu0\
432\x5cu043b\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu043f\x5cu0440\x5cu0\
435\x5cu0434\x5cu043e\x5c\
u043f\x5cu0440\x5cu043\
5\x5cu0434\x5cu0435\x5cu0\
43b\x5cu0435\x5cu043d\x5c\
u043d\x5cu044b\x5cu044\
5\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu043e\x5cu04\
3f\x5cu0435\x5cu0440\x5cu\
0430\x5cu0442\x5cu0438\
\x5cu0432\x5cu043d\x5cu04\
3e\x5cu0435\x5cu043f\x5cu\
0440\x5cu043e\x5cu0432\
\x5cu0435\x5cu0434\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435 \x5cu043e\x5cu044\
1\x5cu043d\x5cu043e\x5cu0\
432\x5cu043d\x5cu043e\x5c\
u0435\x5cu043f\x5cu044\
0\x5cu0435\x5cu0434\x5cu0\
441\x5cu0442\x5cu0430\x5c\
u0432\x5cu043b\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu0432\x5cu0438\x5c\
u0434\x5cu0430\x5cu044\
0\x5cu0430\x5cu0441\x5cu0\
447\x5cu0435\x5cu0442\x5c\
u0430 \x5cu043e\x5cu04\
41\x5cu043d\x5cu043e\x5cu\
0432\x5cu043d\x5cu043e\
\x5cu0435\x5cu043f\x5cu04\
40\x5cu0435\x5cu0434\x5cu\
0441\x5cu0442\x5cu0430\
\x5cu0432\x5cu043b\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu0432\x5cu0438\
\x5cu0434\x5cu0430\x5cu04\
45\x5cu0430\x5cu0440\x5cu\
0430\x5cu043a\x5cu0442\
\x5cu0435\x5cu0440\x5cu04\
38\x5cu0441\x5cu0442\x5cu\
0438\x5cu043a\x5cu0438\
 \x5cu043e\x5cu0441\x5cu0\
43d\x5cu043e\x5cu0432\x5c\
u043d\x5cu043e\x5cu043\
5\x5cu043f\x5cu0440\x5cu0\
435\x5cu0434\x5cu0441\x5c\
u0442\x5cu0430\x5cu043\
2\x5cu043b\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0437\x5cu0430\x5cu043\
4\x5cu0430\x5cu0447\x5cu0\
438 \x5cu043e\x5cu0441\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu043d\x5cu043e\x5cu\
0435\x5cu043f\x5cu0440\
\x5cu0435\x5cu0434\x5cu04\
41\x5cu0442\x5cu0430\x5cu\
0432\x5cu043b\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu043f\x5cu043b\x5cu\
0430\x5cu043d\x5cu0430\
\x5cu043e\x5cu0431\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
0430 \x5cu043e\x5cu044\
1\x5cu043d\x5cu043e\x5cu0\
432\x5cu043d\x5cu043e\x5c\
u0435\x5cu043f\x5cu044\
0\x5cu0435\x5cu0434\x5cu0\
441\x5cu0442\x5cu0430\x5c\
u0432\x5cu043b\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu0441\x5cu043f\x5c\
u0440\x5cu0430\x5cu043\
2\x5cu043e\x5cu0447\x5cu0\
43d\x5cu0438\x5cu043a\x5c\
u0430 \x5cu043e\x5cu04\
41\x5cu043d\x5cu043e\x5cu\
0432\x5cu043d\x5cu043e\
\x5cu0435\x5cu043f\x5cu04\
40\x5cu0435\x5cu0434\x5cu\
0441\x5cu0442\x5cu0430\
\x5cu0432\x5cu043b\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu0441\x5cu0447\
\x5cu0435\x5cu0442\x5cu04\
30 \x5cu043f\x5cu0435\x5c\
u0440\x5cu0435\x5cu043\
c\x5cu0435\x5cu0449\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435\x5cu0433\x5cu044\
0\x5cu0430\x5cu043d\x5cu0\
438\x5cu0446\x5cu044b\x5c\
u043f\x5cu0440\x5cu043\
8\x5cu043f\x5cu0440\x5cu0\
43e\x5cu0432\x5cu0435\x5c\
u0434\x5cu0435\x5cu043\
d\x5cu0438\x5cu0438 \x5cu\
043f\x5cu0435\x5cu0440\
\x5cu0438\x5cu043e\x5cu04\
34\x5cu0438\x5cu0447\x5cu\
043d\x5cu043e\x5cu0441\
\x5cu0442\x5cu044c\x5cu04\
3d\x5cu043e\x5cu043c\x5cu\
0435\x5cu0440\x5cu0430\
\x5cu0431\x5cu0438\x5cu04\
37\x5cu043d\x5cu0435\x5cu\
0441\x5cu043f\x5cu0440\
\x5cu043e\x5cu0446\x5cu04\
35\x5cu0441\x5cu0441\x5cu\
0430 \x5cu043f\x5cu043\
5\x5cu0440\x5cu0438\x5cu0\
43e\x5cu0434\x5cu0438\x5c\
u0447\x5cu043d\x5cu043\
e\x5cu0441\x5cu0442\x5cu0\
44c\x5cu043d\x5cu043e\x5c\
u043c\x5cu0435\x5cu044\
0\x5cu0430\x5cu0434\x5cu0\
43e\x5cu043a\x5cu0443\x5c\
u043c\x5cu0435\x5cu043\
d\x5cu0442\x5cu0430 \x5cu\
043f\x5cu0435\x5cu0440\
\x5cu0438\x5cu043e\x5cu04\
34\x5cu0438\x5cu0447\x5cu\
043d\x5cu043e\x5cu0441\
\x5cu0442\x5cu044c\x5cu04\
40\x5cu0435\x5cu0433\x5cu\
0438\x5cu0441\x5cu0442\
\x5cu0440\x5cu0430\x5cu04\
40\x5cu0430\x5cu0441\x5cu\
0447\x5cu0435\x5cu0442\
\x5cu0430 \x5cu043f\x5cu0\
435\x5cu0440\x5cu0438\x5c\
u043e\x5cu0434\x5cu043\
8\x5cu0447\x5cu043d\x5cu0\
43e\x5cu0441\x5cu0442\x5c\
u044c\x5cu0440\x5cu043\
5\x5cu0433\x5cu0438\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0430\x5cu0441\x5cu043\
2\x5cu0435\x5cu0434\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0439 \x5cu043f\x5cu04\
3e\x5cu0432\x5cu0442\x5cu\
043e\x5cu0440\x5cu043d\
\x5cu043e\x5cu0435\x5cu04\
38\x5cu0441\x5cu043f\x5cu\
043e\x5cu043b\x5cu044c\
\x5cu0437\x5cu043e\x5cu04\
32\x5cu0430\x5cu043d\x5cu\
0438\x5cu0435\x5cu0432\
\x5cu043e\x5cu0437\x5cu04\
32\x5cu0440\x5cu0430\x5cu\
0449\x5cu0430\x5cu0435\
\x5cu043c\x5cu044b\x5cu04\
45\x5cu0437\x5cu043d\x5cu\
0430\x5cu0447\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
39 \x5cu043f\x5cu043e\x5c\
u043b\x5cu043d\x5cu043\
e\x5cu0442\x5cu0435\x5cu0\
43a\x5cu0441\x5cu0442\x5c\
u043e\x5cu0432\x5cu044\
b\x5cu0439\x5cu043f\x5cu0\
43e\x5cu0438\x5cu0441\x5c\
u043a\x5cu043f\x5cu044\
0\x5cu0438\x5cu0432\x5cu0\
432\x5cu043e\x5cu0434\x5c\
u0435\x5cu043f\x5cu043\
e\x5cu0441\x5cu0442\x5cu0\
440\x5cu043e\x5cu043a\x5c\
u0435 \x5cu043f\x5cu04\
40\x5cu0438\x5cu043d\x5cu\
0430\x5cu0434\x5cu043b\
\x5cu0435\x5cu0436\x5cu04\
3d\x5cu043e\x5cu0441\x5cu\
0442\x5cu044c\x5cu043e\
\x5cu0431\x5cu044a\x5cu04\
35\x5cu043a\x5cu0442\x5cu\
0430 \x5cu043f\x5cu044\
0\x5cu043e\x5cu0432\x5cu0\
435\x5cu0434\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5 \x5cu0440\x5cu0430\x5cu\
0437\x5cu0434\x5cu0435\
\x5cu043b\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
0430\x5cu0443\x5cu0442\
\x5cu0435\x5cu043d\x5cu04\
42\x5cu0438\x5cu0444\x5cu\
0438\x5cu043a\x5cu0430\
\x5cu0446\x5cu0438\x5cu04\
38\x5cu043e\x5cu0431\x5cu\
0449\x5cu0435\x5cu0433\
\x5cu043e\x5cu0440\x5cu04\
35\x5cu043a\x5cu0432\x5cu\
0438\x5cu0437\x5cu0438\
\x5cu0442\x5cu0430 \x5cu0\
440\x5cu0430\x5cu0437\x5c\
u0434\x5cu0435\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445\x5cu0\
43e\x5cu0431\x5cu0449\x5c\
u0435\x5cu0433\x5cu043\
e\x5cu0440\x5cu0435\x5cu0\
43a\x5cu0432\x5cu0438\x5c\
u0437\x5cu0438\x5cu044\
2\x5cu0430 \x5cu0440\x5cu\
0430\x5cu0437\x5cu0434\
\x5cu0435\x5cu043b\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu0440\x5cu0430\
\x5cu0441\x5cu0448\x5cu04\
38\x5cu0440\x5cu0435\x5cu\
043d\x5cu0438\x5cu0439\
\x5cu043a\x5cu043e\x5cu04\
3d\x5cu0444\x5cu0438\x5cu\
0433\x5cu0443\x5cu0440\
\x5cu0430\x5cu0446\x5cu04\
38\x5cu0438\x5cu043e\x5cu\
0431\x5cu0449\x5cu0435\
\x5cu0433\x5cu043e\x5cu04\
40\x5cu0435\x5cu043a\x5cu\
0432\x5cu0438\x5cu0437\
\x5cu0438\x5cu0442\x5cu04\
30 \x5cu0440\x5cu0435\x5c\
u0436\x5cu0438\x5cu043\
c\x5cu0430\x5cu0432\x5cu0\
442\x5cu043e\x5cu043d\x5c\
u0443\x5cu043c\x5cu043\
5\x5cu0440\x5cu0430\x5cu0\
446\x5cu0438\x5cu0438\x5c\
u043e\x5cu0431\x5cu044\
a\x5cu0435\x5cu043a\x5cu0\
442\x5cu043e\x5cu0432 \
\x5cu0440\x5cu0435\x5cu04\
36\x5cu0438\x5cu043c\x5cu\
0437\x5cu0430\x5cu043f\
\x5cu0438\x5cu0441\x5cu04\
38\x5cu0440\x5cu0435\x5cu\
0433\x5cu0438\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
30 \x5cu0440\x5cu0435\x5c\
u0436\x5cu0438\x5cu043\
c\x5cu0438\x5cu0441\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u044c\x5cu0437\x5cu043\
e\x5cu0432\x5cu0430\x5cu0\
43d\x5cu0438\x5cu044f\x5c\
u043c\x5cu043e\x5cu043\
4\x5cu0430\x5cu043b\x5cu0\
44c\x5cu043d\x5cu043e\x5c\
u0441\x5cu0442\x5cu043\
8 \x5cu0440\x5cu0435\x5cu\
0436\x5cu0438\x5cu043c\
\x5cu0438\x5cu0441\x5cu04\
3f\x5cu043e\x5cu043b\x5cu\
044c\x5cu0437\x5cu043e\
\x5cu0432\x5cu0430\x5cu04\
3d\x5cu0438\x5cu044f\x5cu\
0441\x5cu0438\x5cu043d\
\x5cu0445\x5cu0440\x5cu04\
3e\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445\x5cu0432\
\x5cu044b\x5cu0437\x5cu04\
3e\x5cu0432\x5cu043e\x5cu\
0432\x5cu0440\x5cu0430\
\x5cu0441\x5cu0448\x5cu04\
38\x5cu0440\x5cu0435\x5cu\
043d\x5cu0438\x5cu0439\
\x5cu043f\x5cu043b\x5cu04\
30\x5cu0442\x5cu0444\x5cu\
043e\x5cu0440\x5cu043c\
\x5cu044b\x5cu0438\x5cu04\
32\x5cu043d\x5cu0435\x5cu\
0448\x5cu043d\x5cu0438\
\x5cu0445\x5cu043a\x5cu04\
3e\x5cu043c\x5cu043f\x5cu\
043e\x5cu043d\x5cu0435\
\x5cu043d\x5cu0442 \x5cu0\
440\x5cu0435\x5cu0436\x5c\
u0438\x5cu043c\x5cu043\
f\x5cu043e\x5cu0432\x5cu0\
442\x5cu043e\x5cu0440\x5c\
u043d\x5cu043e\x5cu043\
3\x5cu043e\x5cu0438\x5cu0\
441\x5cu043f\x5cu043e\x5c\
u043b\x5cu044c\x5cu043\
7\x5cu043e\x5cu0432\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u044f\x5cu0441\x5cu043\
5\x5cu0430\x5cu043d\x5cu0\
441\x5cu043e\x5cu0432 \
\x5cu0440\x5cu0435\x5cu04\
36\x5cu0438\x5cu043c\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu0443\x5cu0447\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f\x5cu0434\x5cu0430\
\x5cu043d\x5cu043d\x5cu04\
4b\x5cu0445\x5cu0432\x5cu\
044b\x5cu0431\x5cu043e\
\x5cu0440\x5cu0430\x5cu04\
3f\x5cu0440\x5cu0438\x5cu\
0432\x5cu0432\x5cu043e\
\x5cu0434\x5cu0435\x5cu04\
3f\x5cu043e\x5cu0441\x5cu\
0442\x5cu0440\x5cu043e\
\x5cu043a\x5cu0435 \x5cu0\
440\x5cu0435\x5cu0436\x5c\
u0438\x5cu043c\x5cu044\
1\x5cu043e\x5cu0432\x5cu0\
43c\x5cu0435\x5cu0441\x5c\
u0442\x5cu0438\x5cu043\
c\x5cu043e\x5cu0441\x5cu0\
442\x5cu0438 \x5cu0440\
\x5cu0435\x5cu0436\x5cu04\
38\x5cu043c\x5cu0441\x5cu\
043e\x5cu0432\x5cu043c\
\x5cu0435\x5cu0441\x5cu04\
42\x5cu0438\x5cu043c\x5cu\
043e\x5cu0441\x5cu0442\
\x5cu0438\x5cu0438\x5cu04\
3d\x5cu0442\x5cu0435\x5cu\
0440\x5cu0444\x5cu0435\
\x5cu0439\x5cu0441\x5cu04\
30 \x5cu0440\x5cu0435\x5c\
u0436\x5cu0438\x5cu043\
c\x5cu0443\x5cu043f\x5cu0\
440\x5cu0430\x5cu0432\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu044f\x5cu0\
431\x5cu043b\x5cu043e\x5c\
u043a\x5cu0438\x5cu044\
0\x5cu043e\x5cu0432\x5cu0\
43a\x5cu043e\x5cu0439\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445\x5cu043f\x5cu043e\x5c\
u0443\x5cu043c\x5cu043\
e\x5cu043b\x5cu0447\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u044e \x5cu0441\x5cu04\
35\x5cu0440\x5cu0438\x5cu\
0438\x5cu043a\x5cu043e\
\x5cu0434\x5cu043e\x5cu04\
32\x5cu043f\x5cu043b\x5cu\
0430\x5cu043d\x5cu0430\
\x5cu0432\x5cu0438\x5cu04\
34\x5cu043e\x5cu0432\x5cu\
0445\x5cu0430\x5cu0440\
\x5cu0430\x5cu043a\x5cu04\
42\x5cu0435\x5cu0440\x5cu\
0438\x5cu0441\x5cu0442\
\x5cu0438\x5cu043a \x5cu0\
441\x5cu0435\x5cu0440\x5c\
u0438\x5cu0438\x5cu043\
a\x5cu043e\x5cu0434\x5cu0\
43e\x5cu0432\x5cu043f\x5c\
u043b\x5cu0430\x5cu043\
d\x5cu0430\x5cu0441\x5cu0\
447\x5cu0435\x5cu0442\x5c\
u043e\x5cu0432 \x5cu04\
41\x5cu0435\x5cu0440\x5cu\
0438\x5cu0438\x5cu043a\
\x5cu043e\x5cu0434\x5cu04\
3e\x5cu0432\x5cu0441\x5cu\
043f\x5cu0440\x5cu0430\
\x5cu0432\x5cu043e\x5cu04\
47\x5cu043d\x5cu0438\x5cu\
043a\x5cu0430 \x5cu044\
1\x5cu043e\x5cu0437\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u0438\x5cu0435\x5cu043\
f\x5cu0440\x5cu0438\x5cu0\
432\x5cu0432\x5cu043e\x5c\
u0434\x5cu0435 \x5cu04\
41\x5cu043f\x5cu043e\x5cu\
0441\x5cu043e\x5cu0431\
\x5cu0432\x5cu044b\x5cu04\
31\x5cu043e\x5cu0440\x5cu\
0430 \x5cu0441\x5cu043\
f\x5cu043e\x5cu0441\x5cu0\
43e\x5cu0431\x5cu043f\x5c\
u043e\x5cu0438\x5cu044\
1\x5cu043a\x5cu0430\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u043e\x5cu043a\x5cu043\
8\x5cu043f\x5cu0440\x5cu0\
438\x5cu0432\x5cu0432\x5c\
u043e\x5cu0434\x5cu043\
5\x5cu043f\x5cu043e\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u043e\x5cu043a\x5cu043\
5 \x5cu0441\x5cu043f\x5cu\
043e\x5cu0441\x5cu043e\
\x5cu0431\x5cu0440\x5cu04\
35\x5cu0434\x5cu0430\x5cu\
043a\x5cu0442\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0430\x5cu043d\x5cu\
0438\x5cu044f \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u043d\x5cu044b\x5cu044\
5\x5cu0442\x5cu0430\x5cu0\
431\x5cu043b\x5cu0438\x5c\
u0446\x5cu044b\x5cu043\
2\x5cu043d\x5cu0435\x5cu0\
448\x5cu043d\x5cu0435\x5c\
u0433\x5cu043e\x5cu043\
8\x5cu0441\x5cu0442\x5cu0\
43e\x5cu0447\x5cu043d\x5c\
u0438\x5cu043a\x5cu043\
0\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu0442\x5cu04\
38\x5cu043f\x5cu043a\x5cu\
043e\x5cu0434\x5cu0430\
\x5cu043f\x5cu043b\x5cu04\
30\x5cu043d\x5cu0430\x5cu\
0432\x5cu0438\x5cu0434\
\x5cu043e\x5cu0432\x5cu04\
40\x5cu0430\x5cu0441\x5cu\
0447\x5cu0435\x5cu0442\
\x5cu0430 \x5cu0442\x5cu0\
438\x5cu043f\x5cu043a\x5c\
u043e\x5cu0434\x5cu043\
0\x5cu0441\x5cu043f\x5cu0\
440\x5cu0430\x5cu0432\x5c\
u043e\x5cu0447\x5cu043\
d\x5cu0438\x5cu043a\x5cu0\
430 \x5cu0442\x5cu0438\
\x5cu043f\x5cu043c\x5cu04\
30\x5cu043a\x5cu0435\x5cu\
0442\x5cu0430 \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
43d\x5cu043e\x5cu043c\x5c\
u0435\x5cu0440\x5cu043\
0\x5cu0431\x5cu0438\x5cu0\
437\x5cu043d\x5cu0435\x5c\
u0441\x5cu043f\x5cu044\
0\x5cu043e\x5cu0446\x5cu0\
435\x5cu0441\x5cu0441\x5c\
u0430 \x5cu0442\x5cu04\
38\x5cu043f\x5cu043d\x5cu\
043e\x5cu043c\x5cu0435\
\x5cu0440\x5cu0430\x5cu04\
34\x5cu043e\x5cu043a\x5cu\
0443\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30 \x5cu0442\x5cu0438\x5c\
u043f\x5cu043d\x5cu043\
e\x5cu043c\x5cu0435\x5cu0\
440\x5cu0430\x5cu0437\x5c\
u0430\x5cu0434\x5cu043\
0\x5cu0447\x5cu0438 \x5cu\
0442\x5cu0438\x5cu043f\
\x5cu0444\x5cu043e\x5cu04\
40\x5cu043c\x5cu044b \x5c\
u0443\x5cu0434\x5cu043\
0\x5cu043b\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0434\x5cu0432\x5cu043\
8\x5cu0436\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0439 \
\x5cu0432\x5cu0430\x5cu04\
36\x5cu043d\x5cu043e\x5cu\
0441\x5cu0442\x5cu044c\
\x5cu043f\x5cu0440\x5cu04\
3e\x5cu0431\x5cu043b\x5cu\
0435\x5cu043c\x5cu044b\
\x5cu043f\x5cu0440\x5cu04\
38\x5cu043c\x5cu0435\x5cu\
043d\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
40\x5cu0430\x5cu0441\x5cu\
0448\x5cu0438\x5cu0440\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f\x5cu043a\x5cu\
043e\x5cu043d\x5cu0444\
\x5cu0438\x5cu0433\x5cu04\
43\x5cu0440\x5cu0430\x5cu\
0446\x5cu0438\x5cu0438\
 \x5cu0432\x5cu0430\x5cu0\
440\x5cu0438\x5cu0430\x5c\
u043d\x5cu0442\x5cu043\
8\x5cu043d\x5cu0442\x5cu0\
435\x5cu0440\x5cu0444\x5c\
u0435\x5cu0439\x5cu044\
1\x5cu0430\x5cu043a\x5cu0\
43b\x5cu0438\x5cu0435\x5c\
u043d\x5cu0442\x5cu044\
1\x5cu043a\x5cu043e\x5cu0\
433\x5cu043e\x5cu043f\x5c\
u0440\x5cu0438\x5cu043\
b\x5cu043e\x5cu0436\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u044f \x5cu0432\x5cu04\
30\x5cu0440\x5cu0438\x5cu\
0430\x5cu043d\x5cu0442\
\x5cu043c\x5cu0430\x5cu04\
41\x5cu0448\x5cu0442\x5cu\
0430\x5cu0431\x5cu0430\
\x5cu0444\x5cu043e\x5cu04\
40\x5cu043c\x5cu043a\x5cu\
043b\x5cu0438\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
41\x5cu043a\x5cu043e\x5cu\
0433\x5cu043e\x5cu043f\
\x5cu0440\x5cu0438\x5cu04\
3b\x5cu043e\x5cu0436\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f \x5cu0432\x5cu0\
430\x5cu0440\x5cu0438\x5c\
u0430\x5cu043d\x5cu044\
2\x5cu043e\x5cu0441\x5cu0\
43d\x5cu043e\x5cu0432\x5c\
u043d\x5cu043e\x5cu043\
3\x5cu043e\x5cu0448\x5cu0\
440\x5cu0438\x5cu0444\x5c\
u0442\x5cu0430\x5cu043\
a\x5cu043b\x5cu0438\x5cu0\
435\x5cu043d\x5cu0442\x5c\
u0441\x5cu043a\x5cu043\
e\x5cu0433\x5cu043e\x5cu0\
43f\x5cu0440\x5cu0438\x5c\
u043b\x5cu043e\x5cu043\
6\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f \x5cu0432\
\x5cu0430\x5cu0440\x5cu04\
38\x5cu0430\x5cu043d\x5cu\
0442\x5cu0441\x5cu0442\
\x5cu0430\x5cu043d\x5cu04\
34\x5cu0430\x5cu0440\x5cu\
0442\x5cu043d\x5cu043e\
\x5cu0433\x5cu043e\x5cu04\
3f\x5cu0435\x5cu0440\x5cu\
0438\x5cu043e\x5cu0434\
\x5cu0430 \x5cu0432\x5cu0\
430\x5cu0440\x5cu0438\x5c\
u0430\x5cu043d\x5cu044\
2\x5cu0441\x5cu0442\x5cu0\
430\x5cu043d\x5cu0434\x5c\
u0430\x5cu0440\x5cu044\
2\x5cu043d\x5cu043e\x5cu0\
439\x5cu0434\x5cu0430\x5c\
u0442\x5cu044b\x5cu043\
d\x5cu0430\x5cu0447\x5cu0\
430\x5cu043b\x5cu0430 \
\x5cu0432\x5cu0438\x5cu04\
34\x5cu0433\x5cu0440\x5cu\
0430\x5cu043d\x5cu0438\
\x5cu0446\x5cu044b \x5cu0\
432\x5cu0438\x5cu0434\x5c\
u043a\x5cu0430\x5cu044\
0\x5cu0442\x5cu0438\x5cu0\
43d\x5cu043a\x5cu0438 \
\x5cu0432\x5cu0438\x5cu04\
34\x5cu043e\x5cu0442\x5cu\
043e\x5cu0431\x5cu0440\
\x5cu0430\x5cu0436\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f\x5cu043f\x5cu043e\
\x5cu043b\x5cu043d\x5cu04\
3e\x5cu0442\x5cu0435\x5cu\
043a\x5cu0441\x5cu0442\
\x5cu043e\x5cu0432\x5cu04\
3e\x5cu0433\x5cu043e\x5cu\
043f\x5cu043e\x5cu0438\
\x5cu0441\x5cu043a\x5cu04\
30 \x5cu0432\x5cu0438\x5c\
u0434\x5cu0440\x5cu043\
0\x5cu043c\x5cu043a\x5cu0\
438 \x5cu0432\x5cu0438\
\x5cu0434\x5cu0441\x5cu04\
40\x5cu0430\x5cu0432\x5cu\
043d\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f \x5cu0\
432\x5cu0438\x5cu0434\x5c\
u0446\x5cu0432\x5cu043\
5\x5cu0442\x5cu0430 \x5cu\
0432\x5cu0438\x5cu0434\
\x5cu0447\x5cu0438\x5cu04\
41\x5cu043b\x5cu043e\x5cu\
0432\x5cu043e\x5cu0433\
\x5cu043e\x5cu0437\x5cu04\
3d\x5cu0430\x5cu0447\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f \x5cu0432\x5cu0\
438\x5cu0434\x5cu0448\x5c\
u0440\x5cu0438\x5cu044\
4\x5cu0442\x5cu0430 \x5cu\
0434\x5cu043e\x5cu043f\
\x5cu0443\x5cu0441\x5cu04\
42\x5cu0438\x5cu043c\x5cu\
0430\x5cu044f\x5cu0434\
\x5cu043b\x5cu0438\x5cu04\
3d\x5cu0430 \x5cu0434\x5c\
u043e\x5cu043f\x5cu044\
3\x5cu0441\x5cu0442\x5cu0\
438\x5cu043c\x5cu044b\x5c\
u0439\x5cu0437\x5cu043\
d\x5cu0430\x5cu043a \x5cu\
0438\x5cu0441\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
4c\x5cu0437\x5cu043e\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu0438\x5cu0435byte\
ordermark \x5cu0438\
\x5cu0441\x5cu043f\x5cu04\
3e\x5cu043b\x5cu044c\x5cu\
0437\x5cu043e\x5cu0432\
\x5cu0430\x5cu043d\x5cu04\
38\x5cu0435\x5cu043c\x5cu\
0435\x5cu0442\x5cu0430\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu043d\x5cu044b\x5cu\
0445\x5cu043f\x5cu043e\
\x5cu043b\x5cu043d\x5cu04\
3e\x5cu0442\x5cu0435\x5cu\
043a\x5cu0441\x5cu0442\
\x5cu043e\x5cu0432\x5cu04\
3e\x5cu0433\x5cu043e\x5cu\
043f\x5cu043e\x5cu0438\
\x5cu0441\x5cu043a\x5cu04\
30 \x5cu0438\x5cu0441\x5c\
u0442\x5cu043e\x5cu044\
7\x5cu043d\x5cu0438\x5cu0\
43a\x5cu0440\x5cu0430\x5c\
u0441\x5cu0448\x5cu043\
8\x5cu0440\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0439\x5c\
u043a\x5cu043e\x5cu043\
d\x5cu0444\x5cu0438\x5cu0\
433\x5cu0443\x5cu0440\x5c\
u0430\x5cu0446\x5cu043\
8\x5cu0438 \x5cu043a\x5cu\
043b\x5cu0430\x5cu0432\
\x5cu0438\x5cu0448\x5cu04\
30 \x5cu043a\x5cu043e\x5c\
u0434\x5cu0432\x5cu043\
e\x5cu0437\x5cu0432\x5cu0\
440\x5cu0430\x5cu0442\x5c\
u0430\x5cu0434\x5cu043\
8\x5cu0430\x5cu043b\x5cu0\
43e\x5cu0433\x5cu0430 \
\x5cu043a\x5cu043e\x5cu04\
34\x5cu0438\x5cu0440\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0430xbase \x5cu04\
3a\x5cu043e\x5cu0434\x5cu\
0438\x5cu0440\x5cu043e\
\x5cu0432\x5cu043a\x5cu04\
30\x5cu0442\x5cu0435\x5cu\
043a\x5cu0441\x5cu0442\
\x5cu0430 \x5cu043d\x5cu0\
430\x5cu043f\x5cu0440\x5c\
u0430\x5cu0432\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu043f\x5c\
u043e\x5cu0438\x5cu044\
1\x5cu043a\x5cu0430 \x5cu\
043d\x5cu0430\x5cu043f\
\x5cu0440\x5cu0430\x5cu04\
32\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0441\x5cu043e\x5cu04\
40\x5cu0442\x5cu0438\x5cu\
0440\x5cu043e\x5cu0432\
\x5cu043a\x5cu0438 \x5cu0\
43e\x5cu0431\x5cu043d\x5c\
u043e\x5cu0432\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu043f\x5c\
u0440\x5cu0435\x5cu043\
4\x5cu043e\x5cu043f\x5cu0\
440\x5cu0435\x5cu0434\x5c\
u0435\x5cu043b\x5cu043\
5\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445 \x5cu\
043e\x5cu0431\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
3b\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu043f\
\x5cu0440\x5cu0438\x5cu04\
38\x5cu0437\x5cu043c\x5cu\
0435\x5cu043d\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
38\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445 \x5cu043e\x5cu0\
442\x5cu043e\x5cu0431\x5c\
u0440\x5cu0430\x5cu043\
6\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu043f\x5c\
u0430\x5cu043d\x5cu043\
5\x5cu043b\x5cu0438\x5cu0\
440\x5cu0430\x5cu0437\x5c\
u0434\x5cu0435\x5cu043\
b\x5cu043e\x5cu0432 \x5cu\
043f\x5cu0440\x5cu043e\
\x5cu0432\x5cu0435\x5cu04\
40\x5cu043a\x5cu0430\x5cu\
0437\x5cu0430\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
3d\x5cu0435\x5cu043d\x5cu\
0438\x5cu044f \x5cu044\
0\x5cu0435\x5cu0436\x5cu0\
438\x5cu043c\x5cu0434\x5c\
u0438\x5cu0430\x5cu043\
b\x5cu043e\x5cu0433\x5cu0\
430\x5cu0432\x5cu043e\x5c\
u043f\x5cu0440\x5cu043\
e\x5cu0441 \x5cu0440\x5cu\
0435\x5cu0436\x5cu0438\
\x5cu043c\x5cu0437\x5cu04\
30\x5cu043f\x5cu0443\x5cu\
0441\x5cu043a\x5cu0430\
\x5cu043a\x5cu043b\x5cu04\
38\x5cu0435\x5cu043d\x5cu\
0442\x5cu0441\x5cu043a\
\x5cu043e\x5cu0433\x5cu04\
3e\x5cu043f\x5cu0440\x5cu\
0438\x5cu043b\x5cu043e\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu044f \x5c\
u0440\x5cu0435\x5cu043\
6\x5cu0438\x5cu043c\x5cu0\
43e\x5cu043a\x5cu0440\x5c\
u0443\x5cu0433\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f \x5cu0440\
\x5cu0435\x5cu0436\x5cu04\
38\x5cu043c\x5cu043e\x5cu\
0442\x5cu043a\x5cu0440\
\x5cu044b\x5cu0442\x5cu04\
38\x5cu044f\x5cu0444\x5cu\
043e\x5cu0440\x5cu043c\
\x5cu043f\x5cu0440\x5cu04\
38\x5cu043b\x5cu043e\x5cu\
0436\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f \x5cu0\
440\x5cu0435\x5cu0436\x5c\
u0438\x5cu043c\x5cu043\
f\x5cu043e\x5cu043b\x5cu0\
43d\x5cu043e\x5cu0442\x5c\
u0435\x5cu043a\x5cu044\
1\x5cu0442\x5cu043e\x5cu0\
432\x5cu043e\x5cu0433\x5c\
u043e\x5cu043f\x5cu043\
e\x5cu0438\x5cu0441\x5cu0\
43a\x5cu0430 \x5cu0441\
\x5cu043a\x5cu043e\x5cu04\
40\x5cu043e\x5cu0441\x5cu\
0442\x5cu044c\x5cu043a\
\x5cu043b\x5cu0438\x5cu04\
35\x5cu043d\x5cu0442\x5cu\
0441\x5cu043a\x5cu043e\
\x5cu0433\x5cu043e\x5cu04\
41\x5cu043e\x5cu0435\x5cu\
0434\x5cu0438\x5cu043d\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f \x5cu0441\x5c\
u043e\x5cu0441\x5cu044\
2\x5cu043e\x5cu044f\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0432\x5cu043d\x5cu043\
5\x5cu0448\x5cu043d\x5cu0\
435\x5cu0433\x5cu043e\x5c\
u0438\x5cu0441\x5cu044\
2\x5cu043e\x5cu0447\x5cu0\
43d\x5cu0438\x5cu043a\x5c\
u0430\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu0441\
\x5cu043e\x5cu0441\x5cu04\
42\x5cu043e\x5cu044f\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu043e\x5cu0431\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
043b\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
3a\x5cu043e\x5cu043d\x5cu\
0444\x5cu0438\x5cu0433\
\x5cu0443\x5cu0440\x5cu04\
30\x5cu0446\x5cu0438\x5cu\
0438\x5cu0431\x5cu0430\
\x5cu0437\x5cu044b\x5cu04\
34\x5cu0430\x5cu043d\x5cu\
043d\x5cu044b\x5cu0445\
 \x5cu0441\x5cu043f\x5cu0\
43e\x5cu0441\x5cu043e\x5c\
u0431\x5cu0432\x5cu044\
b\x5cu0431\x5cu043e\x5cu0\
440\x5cu0430\x5cu0441\x5c\
u0435\x5cu0440\x5cu044\
2\x5cu0438\x5cu0444\x5cu0\
438\x5cu043a\x5cu0430\x5c\
u0442\x5cu0430windo\
ws \x5cu0441\x5cu043f\x5c\
u043e\x5cu0441\x5cu043\
e\x5cu0431\x5cu043a\x5cu0\
43e\x5cu0434\x5cu0438\x5c\
u0440\x5cu043e\x5cu043\
2\x5cu0430\x5cu043d\x5cu0\
438\x5cu044f\x5cu0441\x5c\
u0442\x5cu0440\x5cu043\
e\x5cu043a\x5cu0438 \x5cu\
0441\x5cu0442\x5cu0430\
\x5cu0442\x5cu0443\x5cu04\
41\x5cu0441\x5cu043e\x5cu\
043e\x5cu0431\x5cu0449\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f \x5cu0442\x5c\
u0438\x5cu043f\x5cu043\
2\x5cu043d\x5cu0435\x5cu0\
448\x5cu043d\x5cu0435\x5c\
u0439\x5cu043a\x5cu043\
e\x5cu043c\x5cu043f\x5cu0\
43e\x5cu043d\x5cu0435\x5c\
u043d\x5cu0442\x5cu044\
b \x5cu0442\x5cu0438\x5cu\
043f\x5cu043f\x5cu043b\
\x5cu0430\x5cu0442\x5cu04\
44\x5cu043e\x5cu0440\x5cu\
043c\x5cu044b \x5cu044\
2\x5cu0438\x5cu043f\x5cu0\
43f\x5cu043e\x5cu0432\x5c\
u0435\x5cu0434\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
44f\x5cu043a\x5cu043b\x5c\
u0430\x5cu0432\x5cu043\
8\x5cu0448\x5cu0438ent\
er \x5cu0442\x5cu0438\x5c\
u043f\x5cu044d\x5cu043\
b\x5cu0435\x5cu043c\x5cu0\
435\x5cu043d\x5cu0442\x5c\
u0430\x5cu0438\x5cu043\
d\x5cu0444\x5cu043e\x5cu0\
440\x5cu043c\x5cu0430\x5c\
u0446\x5cu0438\x5cu043\
8\x5cu043e\x5cu0432\x5cu0\
44b\x5cu043f\x5cu043e\x5c\
u043b\x5cu043d\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
438\x5cu043e\x5cu0431\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu043b\x5cu0435\x5cu0\
43d\x5cu0438\x5cu044f\x5c\
u043a\x5cu043e\x5cu043\
d\x5cu0444\x5cu0438\x5cu0\
433\x5cu0443\x5cu0440\x5c\
u0430\x5cu0446\x5cu043\
8\x5cu0438\x5cu0431\x5cu0\
430\x5cu0437\x5cu044b\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445 \x5cu0443\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
35\x5cu043d\x5cu044c\x5cu\
0438\x5cu0437\x5cu043e\
\x5cu043b\x5cu044f\x5cu04\
46\x5cu0438\x5cu0438\x5cu\
0442\x5cu0440\x5cu0430\
\x5cu043d\x5cu0437\x5cu04\
30\x5cu043a\x5cu0446\x5cu\
0438\x5cu0439 \x5cu044\
5\x5cu0435\x5cu0448\x5cu0\
444\x5cu0443\x5cu043d\x5c\
u043a\x5cu0446\x5cu043\
8\x5cu044f \x5cu0447\x5cu\
0430\x5cu0441\x5cu0442\
\x5cu0438\x5cu0434\x5cu04\
30\x5cu0442\x5cu044b\x22,\
\x0atype:\x22com\x5cu043e\
\x5cu0431\x5cu044a\x5cu04\
35\x5cu043a\x5cu0442 f\
tp\x5cu0441\x5cu043e\x5cu\
0435\x5cu0434\x5cu0438\
\x5cu043d\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435 h\
ttp\x5cu0437\x5cu0430\x5c\
u043f\x5cu0440\x5cu043\
e\x5cu0441 http\x5cu04\
41\x5cu0435\x5cu0440\x5cu\
0432\x5cu0438\x5cu0441\
\x5cu043e\x5cu0442\x5cu04\
32\x5cu0435\x5cu0442 h\
ttp\x5cu0441\x5cu043e\x5c\
u0435\x5cu0434\x5cu043\
8\x5cu043d\x5cu0435\x5cu0\
43d\x5cu0438\x5cu0435 \
ws\x5cu043e\x5cu043f\x5cu\
0440\x5cu0435\x5cu0434\
\x5cu0435\x5cu043b\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044f ws\x5cu043f\x5cu0\
440\x5cu043e\x5cu043a\x5c\
u0441\x5cu0438 xbas\
e \x5cu0430\x5cu043d\x5cu\
0430\x5cu043b\x5cu0438\
\x5cu0437\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
43e\x5cu0442\x5cu0430\x5c\
u0446\x5cu0438\x5cu044\
fxs \x5cu0431\x5cu043b\
\x5cu043e\x5cu043a\x5cu04\
38\x5cu0440\x5cu043e\x5cu\
0432\x5cu043a\x5cu0430\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu043d\x5cu044b\x5cu\
0445 \x5cu0431\x5cu044\
3\x5cu0444\x5cu0435\x5cu0\
440\x5cu0434\x5cu0432\x5c\
u043e\x5cu0438\x5cu044\
7\x5cu043d\x5cu044b\x5cu0\
445\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 \x5cu0432\x5cu\
043a\x5cu043b\x5cu044e\
\x5cu0447\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435xs\
 \x5cu0432\x5cu044b\x5cu0\
440\x5cu0430\x5cu0436\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435\x5cu043a\x5cu0\
43e\x5cu043c\x5cu043f\x5c\
u043e\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
438\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 \x5cu0433\x5cu\
0435\x5cu043d\x5cu0435\
\x5cu0440\x5cu0430\x5cu04\
42\x5cu043e\x5cu0440\x5cu\
0441\x5cu043b\x5cu0443\
\x5cu0447\x5cu0430\x5cu04\
39\x5cu043d\x5cu044b\x5cu\
0445\x5cu0447\x5cu0438\
\x5cu0441\x5cu0435\x5cu04\
3b \x5cu0433\x5cu0435\x5c\
u043e\x5cu0433\x5cu044\
0\x5cu0430\x5cu0444\x5cu0\
438\x5cu0447\x5cu0435\x5c\
u0441\x5cu043a\x5cu043\
0\x5cu044f\x5cu0441\x5cu0\
445\x5cu0435\x5cu043c\x5c\
u0430 \x5cu0433\x5cu04\
35\x5cu043e\x5cu0433\x5cu\
0440\x5cu0430\x5cu0444\
\x5cu0438\x5cu0447\x5cu04\
35\x5cu0441\x5cu043a\x5cu\
0438\x5cu0435\x5cu043a\
\x5cu043e\x5cu043e\x5cu04\
40\x5cu0434\x5cu0438\x5cu\
043d\x5cu0430\x5cu0442\
\x5cu044b \x5cu0433\x5cu0\
440\x5cu0430\x5cu0444\x5c\
u0438\x5cu0447\x5cu043\
5\x5cu0441\x5cu043a\x5cu0\
430\x5cu044f\x5cu0441\x5c\
u0445\x5cu0435\x5cu043\
c\x5cu0430 \x5cu0433\x5cu\
0440\x5cu0443\x5cu043f\
\x5cu043f\x5cu0430\x5cu04\
3c\x5cu043e\x5cu0434\x5cu\
0435\x5cu043b\x5cu0438\
xs \x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0435\x5cu0440\x5cu0\
430\x5cu0441\x5cu0448\x5c\
u0438\x5cu0444\x5cu044\
0\x5cu043e\x5cu0432\x5cu0\
43a\x5cu0438\x5cu043a\x5c\
u043e\x5cu043c\x5cu043\
f\x5cu043e\x5cu043d\x5cu0\
43e\x5cu0432\x5cu043a\x5c\
u0438\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu0434\
\x5cu0432\x5cu043e\x5cu04\
38\x5cu0447\x5cu043d\x5cu\
044b\x5cu0435\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0435 \x5c\
u0434\x5cu0435\x5cu043\
d\x5cu0434\x5cu0440\x5cu0\
43e\x5cu0433\x5cu0440\x5c\
u0430\x5cu043c\x5cu043\
c\x5cu0430 \x5cu0434\x5cu\
0438\x5cu0430\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
3c\x5cu043c\x5cu0430 \x5c\
u0434\x5cu0438\x5cu043\
0\x5cu0433\x5cu0440\x5cu0\
430\x5cu043c\x5cu043c\x5c\
u0430\x5cu0433\x5cu043\
0\x5cu043d\x5cu0442\x5cu0\
430 \x5cu0434\x5cu0438\
\x5cu0430\x5cu043b\x5cu04\
3e\x5cu0433\x5cu0432\x5cu\
044b\x5cu0431\x5cu043e\
\x5cu0440\x5cu0430\x5cu04\
44\x5cu0430\x5cu0439\x5cu\
043b\x5cu0430 \x5cu043\
4\x5cu0438\x5cu0430\x5cu0\
43b\x5cu043e\x5cu0433\x5c\
u0432\x5cu044b\x5cu043\
1\x5cu043e\x5cu0440\x5cu0\
430\x5cu0446\x5cu0432\x5c\
u0435\x5cu0442\x5cu043\
0 \x5cu0434\x5cu0438\x5cu\
0430\x5cu043b\x5cu043e\
\x5cu0433\x5cu0432\x5cu04\
4b\x5cu0431\x5cu043e\x5cu\
0440\x5cu0430\x5cu0448\
\x5cu0440\x5cu0438\x5cu04\
44\x5cu0442\x5cu0430 \x5c\
u0434\x5cu0438\x5cu043\
0\x5cu043b\x5cu043e\x5cu0\
433\x5cu0440\x5cu0430\x5c\
u0441\x5cu043f\x5cu043\
8\x5cu0441\x5cu0430\x5cu0\
43d\x5cu0438\x5cu044f\x5c\
u0440\x5cu0435\x5cu043\
3\x5cu043b\x5cu0430\x5cu0\
43c\x5cu0435\x5cu043d\x5c\
u0442\x5cu043d\x5cu043\
e\x5cu0433\x5cu043e\x5cu0\
437\x5cu0430\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu044f \x5cu0434\x5cu\
0438\x5cu0430\x5cu043b\
\x5cu043e\x5cu0433\x5cu04\
40\x5cu0435\x5cu0434\x5cu\
0430\x5cu043a\x5cu0442\
\x5cu0438\x5cu0440\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
043d\x5cu0438\x5cu044f\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu043d\x5cu0434\x5cu\
0430\x5cu0440\x5cu0442\
\x5cu043d\x5cu043e\x5cu04\
33\x5cu043e\x5cu043f\x5cu\
0435\x5cu0440\x5cu0438\
\x5cu043e\x5cu0434\x5cu04\
30 \x5cu0434\x5cu0438\x5c\
u0430\x5cu043f\x5cu043\
0\x5cu0437\x5cu043e\x5cu0\
43d \x5cu0434\x5cu043e\
\x5cu043a\x5cu0443\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
0442dom \x5cu0434\x5cu\
043e\x5cu043a\x5cu0443\
\x5cu043c\x5cu0435\x5cu04\
3d\x5cu0442html \x5cu0\
434\x5cu043e\x5cu043a\x5c\
u0443\x5cu043c\x5cu043\
5\x5cu043d\x5cu0442\x5cu0\
430\x5cu0446\x5cu0438\x5c\
u044fxs \x5cu0434\x5cu\
043e\x5cu0441\x5cu0442\
\x5cu0430\x5cu0432\x5cu04\
3b\x5cu044f\x5cu0435\x5cu\
043c\x5cu043e\x5cu0435\
\x5cu0443\x5cu0432\x5cu04\
35\x5cu0434\x5cu043e\x5cu\
043c\x5cu043b\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35 \x5cu0437\x5cu0430\x5c\
u043f\x5cu0438\x5cu044\
1\x5cu044cdom \x5cu043\
7\x5cu0430\x5cu043f\x5cu0\
438\x5cu0441\x5cu044cf\
astinfoset \x5cu043\
7\x5cu0430\x5cu043f\x5cu0\
438\x5cu0441\x5cu044ch\
tml \x5cu0437\x5cu0430\
\x5cu043f\x5cu0438\x5cu04\
41\x5cu044cjson \x5cu0\
437\x5cu0430\x5cu043f\x5c\
u0438\x5cu0441\x5cu044\
cxml \x5cu0437\x5cu043\
0\x5cu043f\x5cu0438\x5cu0\
441\x5cu044czip\x5cu04\
44\x5cu0430\x5cu0439\x5cu\
043b\x5cu0430 \x5cu043\
7\x5cu0430\x5cu043f\x5cu0\
438\x5cu0441\x5cu044c\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445 \x5cu0437\x5cu0430\
\x5cu043f\x5cu0438\x5cu04\
41\x5cu044c\x5cu0442\x5cu\
0435\x5cu043a\x5cu0441\
\x5cu0442\x5cu0430 \x5cu0\
437\x5cu0430\x5cu043f\x5c\
u0438\x5cu0441\x5cu044\
c\x5cu0443\x5cu0437\x5cu0\
43b\x5cu043e\x5cu0432d\
om \x5cu0437\x5cu0430\x5c\
u043f\x5cu0440\x5cu043\
e\x5cu0441 \x5cu0437\x5cu\
0430\x5cu0449\x5cu0438\
\x5cu0449\x5cu0435\x5cu04\
3d\x5cu043d\x5cu043e\x5cu\
0435\x5cu0441\x5cu043e\
\x5cu0435\x5cu0434\x5cu04\
38\x5cu043d\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
openssl \x5cu0437\x5cu\
043d\x5cu0430\x5cu0447\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu044f\x5cu043f\x5cu\
043e\x5cu043b\x5cu0435\
\x5cu0439\x5cu0440\x5cu04\
30\x5cu0441\x5cu0448\x5cu\
0438\x5cu0444\x5cu0440\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu0438\x5cu043a\x5cu\
043e\x5cu043c\x5cu043f\
\x5cu043e\x5cu043d\x5cu04\
3e\x5cu0432\x5cu043a\x5cu\
0438\x5cu0434\x5cu0430\
\x5cu043d\x5cu043d\x5cu04\
4b\x5cu0445 \x5cu0438\x5c\
u0437\x5cu0432\x5cu043\
b\x5cu0435\x5cu0447\x5cu0\
435\x5cu043d\x5cu0438\x5c\
u0435\x5cu0442\x5cu043\
5\x5cu043a\x5cu0441\x5cu0\
442\x5cu0430 \x5cu0438\
\x5cu043c\x5cu043f\x5cu04\
3e\x5cu0440\x5cu0442xs\
 \x5cu0438\x5cu043d\x5cu0\
442\x5cu0435\x5cu0440\x5c\
u043d\x5cu0435\x5cu044\
2\x5cu043f\x5cu043e\x5cu0\
447\x5cu0442\x5cu0430 \
\x5cu0438\x5cu043d\x5cu04\
42\x5cu0435\x5cu0440\x5cu\
043d\x5cu0435\x5cu0442\
\x5cu043f\x5cu043e\x5cu04\
47\x5cu0442\x5cu043e\x5cu\
0432\x5cu043e\x5cu0435\
\x5cu0441\x5cu043e\x5cu04\
3e\x5cu0431\x5cu0449\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu0435 \x5cu0438\x5cu0\
43d\x5cu0442\x5cu0435\x5c\
u0440\x5cu043d\x5cu043\
5\x5cu0442\x5cu043f\x5cu0\
43e\x5cu0447\x5cu0442\x5c\
u043e\x5cu0432\x5cu044\
b\x5cu0439\x5cu043f\x5cu0\
440\x5cu043e\x5cu0444\x5c\
u0438\x5cu043b\x5cu044\
c \x5cu0438\x5cu043d\x5cu\
0442\x5cu0435\x5cu0440\
\x5cu043d\x5cu0435\x5cu04\
42\x5cu043f\x5cu0440\x5cu\
043e\x5cu043a\x5cu0441\
\x5cu0438 \x5cu0438\x5cu0\
43d\x5cu0442\x5cu0435\x5c\
u0440\x5cu043d\x5cu043\
5\x5cu0442\x5cu0441\x5cu0\
43e\x5cu0435\x5cu0434\x5c\
u0438\x5cu043d\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435 \x5cu0438\x5cu043d\
\x5cu0444\x5cu043e\x5cu04\
40\x5cu043c\x5cu0430\x5cu\
0446\x5cu0438\x5cu044f\
\x5cu0434\x5cu043b\x5cu04\
4f\x5cu043f\x5cu0440\x5cu\
0438\x5cu043b\x5cu043e\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu044fxs\
 \x5cu0438\x5cu0441\x5cu0\
43f\x5cu043e\x5cu043b\x5c\
u044c\x5cu0437\x5cu043\
e\x5cu0432\x5cu0430\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u0430\x5cu0442\x5cu044\
0\x5cu0438\x5cu0431\x5cu0\
443\x5cu0442\x5cu0430x\
s \x5cu0438\x5cu0441\x5cu\
043f\x5cu043e\x5cu043b\
\x5cu044c\x5cu0437\x5cu04\
3e\x5cu0432\x5cu0430\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0441\x5cu043e\x5cu04\
31\x5cu044b\x5cu0442\x5cu\
0438\x5cu044f\x5cu0436\
\x5cu0443\x5cu0440\x5cu04\
3d\x5cu0430\x5cu043b\x5cu\
0430\x5cu0440\x5cu0435\
\x5cu0433\x5cu0438\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
0430\x5cu0446\x5cu0438\
\x5cu0438 \x5cu0438\x5cu0\
441\x5cu0442\x5cu043e\x5c\
u0447\x5cu043d\x5cu043\
8\x5cu043a\x5cu0434\x5cu0\
43e\x5cu0441\x5cu0442\x5c\
u0443\x5cu043f\x5cu043\
d\x5cu044b\x5cu0445\x5cu0\
43d\x5cu0430\x5cu0441\x5c\
u0442\x5cu0440\x5cu043\
e\x5cu0435\x5cu043a\x5cu0\
43a\x5cu043e\x5cu043c\x5c\
u043f\x5cu043e\x5cu043\
d\x5cu043e\x5cu0432\x5cu0\
43a\x5cu0438\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445 \x5cu\
0438\x5cu0442\x5cu0435\
\x5cu0440\x5cu0430\x5cu04\
42\x5cu043e\x5cu0440\x5cu\
0443\x5cu0437\x5cu043b\
\x5cu043e\x5cu0432dom \
\x5cu043a\x5cu0430\x5cu04\
40\x5cu0442\x5cu0438\x5cu\
043d\x5cu043a\x5cu0430\
 \x5cu043a\x5cu0432\x5cu0\
430\x5cu043b\x5cu0438\x5c\
u0444\x5cu0438\x5cu043\
a\x5cu0430\x5cu0442\x5cu0\
43e\x5cu0440\x5cu044b\x5c\
u0434\x5cu0430\x5cu044\
2\x5cu044b \x5cu043a\x5cu\
0432\x5cu0430\x5cu043b\
\x5cu0438\x5cu0444\x5cu04\
38\x5cu043a\x5cu0430\x5cu\
0442\x5cu043e\x5cu0440\
\x5cu044b\x5cu0434\x5cu04\
32\x5cu043e\x5cu0438\x5cu\
0447\x5cu043d\x5cu044b\
\x5cu0445\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu043\
a\x5cu0432\x5cu0430\x5cu0\
43b\x5cu0438\x5cu0444\x5c\
u0438\x5cu043a\x5cu043\
0\x5cu0442\x5cu043e\x5cu0\
440\x5cu044b\x5cu0441\x5c\
u0442\x5cu0440\x5cu043\
e\x5cu043a\x5cu0438 \x5cu\
043a\x5cu0432\x5cu0430\
\x5cu043b\x5cu0438\x5cu04\
44\x5cu0438\x5cu043a\x5cu\
0430\x5cu0442\x5cu043e\
\x5cu0440\x5cu044b\x5cu04\
47\x5cu0438\x5cu0441\x5cu\
043b\x5cu0430 \x5cu043\
a\x5cu043e\x5cu043c\x5cu0\
43f\x5cu043e\x5cu043d\x5c\
u043e\x5cu0432\x5cu044\
9\x5cu0438\x5cu043a\x5cu0\
43c\x5cu0430\x5cu043a\x5c\
u0435\x5cu0442\x5cu043\
0\x5cu043a\x5cu043e\x5cu0\
43c\x5cu043f\x5cu043e\x5c\
u043d\x5cu043e\x5cu043\
2\x5cu043a\x5cu0438\x5cu0\
434\x5cu0430\x5cu043d\x5c\
u043d\x5cu044b\x5cu044\
5 \x5cu043a\x5cu043e\x5cu\
043c\x5cu043f\x5cu043e\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu0449\x5cu0438\x5cu\
043a\x5cu043d\x5cu0430\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu043e\x5cu0435\x5cu\
043a\x5cu043a\x5cu043e\
\x5cu043c\x5cu043f\x5cu04\
3e\x5cu043d\x5cu043e\x5cu\
0432\x5cu043a\x5cu0438\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu043d\x5cu044b\x5cu\
0445 \x5cu043a\x5cu043\
e\x5cu043d\x5cu0441\x5cu0\
442\x5cu0440\x5cu0443\x5c\
u043a\x5cu0442\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
430\x5cu043a\x5cu0435\x5c\
u0442\x5cu0430\x5cu043\
e\x5cu0444\x5cu043e\x5cu0\
440\x5cu043c\x5cu043b\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044f\x5cu043a\x5cu0\
43e\x5cu043c\x5cu043f\x5c\
u043e\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
438\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 \x5cu043a\x5cu\
043e\x5cu043d\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
43\x5cu043a\x5cu0442\x5cu\
043e\x5cu0440\x5cu043d\
\x5cu0430\x5cu0441\x5cu04\
42\x5cu0440\x5cu043e\x5cu\
0435\x5cu043a\x5cu043a\
\x5cu043e\x5cu043c\x5cu04\
3f\x5cu043e\x5cu043d\x5cu\
043e\x5cu0432\x5cu043a\
\x5cu0438\x5cu0434\x5cu04\
30\x5cu043d\x5cu043d\x5cu\
044b\x5cu0445 \x5cu043\
a\x5cu043e\x5cu043d\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0443\x5cu043a\x5cu044\
2\x5cu043e\x5cu0440\x5cu0\
444\x5cu043e\x5cu0440\x5c\
u043c\x5cu0430\x5cu044\
2\x5cu043d\x5cu043e\x5cu0\
439\x5cu0441\x5cu0442\x5c\
u0440\x5cu043e\x5cu043\
a\x5cu0438 \x5cu043b\x5cu\
0438\x5cu043d\x5cu0438\
\x5cu044f \x5cu043c\x5cu0\
430\x5cu043a\x5cu0435\x5c\
u0442\x5cu043a\x5cu043\
e\x5cu043c\x5cu043f\x5cu0\
43e\x5cu043d\x5cu043e\x5c\
u0432\x5cu043a\x5cu043\
8\x5cu0434\x5cu0430\x5cu0\
43d\x5cu043d\x5cu044b\x5c\
u0445 \x5cu043c\x5cu04\
30\x5cu043a\x5cu0435\x5cu\
0442\x5cu043e\x5cu0431\
\x5cu043b\x5cu0430\x5cu04\
41\x5cu0442\x5cu0438\x5cu\
043a\x5cu043e\x5cu043c\
\x5cu043f\x5cu043e\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
043a\x5cu0438\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0445 \x5c\
u043c\x5cu0430\x5cu043\
a\x5cu0435\x5cu0442\x5cu0\
43e\x5cu0444\x5cu043e\x5c\
u0440\x5cu043c\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu044f\x5cu043a\x5c\
u043e\x5cu043c\x5cu043\
f\x5cu043e\x5cu043d\x5cu0\
43e\x5cu0432\x5cu043a\x5c\
u0438\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu043c\
\x5cu0430\x5cu0441\x5cu04\
3a\x5cu0430xs \x5cu043\
c\x5cu0435\x5cu043d\x5cu0\
435\x5cu0434\x5cu0436\x5c\
u0435\x5cu0440\x5cu043\
a\x5cu0440\x5cu0438\x5cu0\
43f\x5cu0442\x5cu043e\x5c\
u0433\x5cu0440\x5cu043\
0\x5cu0444\x5cu0438\x5cu0\
438 \x5cu043d\x5cu0430\
\x5cu0431\x5cu043e\x5cu04\
40\x5cu0441\x5cu0445\x5cu\
0435\x5cu043cxml \x5cu\
043d\x5cu0430\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
3e\x5cu0439\x5cu043a\x5cu\
0438\x5cu043a\x5cu043e\
\x5cu043c\x5cu043f\x5cu04\
3e\x5cu043d\x5cu043e\x5cu\
0432\x5cu043a\x5cu0438\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu043d\x5cu044b\x5cu\
0445 \x5cu043d\x5cu043\
0\x5cu0441\x5cu0442\x5cu0\
440\x5cu043e\x5cu0439\x5c\
u043a\x5cu0438\x5cu044\
1\x5cu0435\x5cu0440\x5cu0\
438\x5cu0430\x5cu043b\x5c\
u0438\x5cu0437\x5cu043\
0\x5cu0446\x5cu0438\x5cu0\
438json \x5cu043e\x5cu\
0431\x5cu0440\x5cu0430\
\x5cu0431\x5cu043e\x5cu04\
42\x5cu043a\x5cu0430\x5cu\
043a\x5cu0430\x5cu0440\
\x5cu0442\x5cu0438\x5cu04\
3d\x5cu043e\x5cu043a \x5c\
u043e\x5cu0431\x5cu044\
0\x5cu0430\x5cu0431\x5cu0\
43e\x5cu0442\x5cu043a\x5c\
u0430\x5cu0440\x5cu043\
0\x5cu0441\x5cu0448\x5cu0\
438\x5cu0444\x5cu0440\x5c\
u043e\x5cu0432\x5cu043\
a\x5cu0438\x5cu043a\x5cu0\
43e\x5cu043c\x5cu043f\x5c\
u043e\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
438\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 \x5cu043e\x5cu\
0431\x5cu0445\x5cu043e\
\x5cu0434\x5cu0434\x5cu04\
35\x5cu0440\x5cu0435\x5cu\
0432\x5cu0430dom \x5cu\
043e\x5cu0431\x5cu044a\
\x5cu044f\x5cu0432\x5cu04\
3b\x5cu0435\x5cu043d\x5cu\
0438\x5cu0435\x5cu0430\
\x5cu0442\x5cu0440\x5cu04\
38\x5cu0431\x5cu0443\x5cu\
0442\x5cu0430xs \x5cu0\
43e\x5cu0431\x5cu044a\x5c\
u044f\x5cu0432\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu043d\x5c\
u043e\x5cu0442\x5cu043\
0\x5cu0446\x5cu0438\x5cu0\
438xs \x5cu043e\x5cu04\
31\x5cu044a\x5cu044f\x5cu\
0432\x5cu043b\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu044d\x5cu043b\x5cu\
0435\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30xs \x5cu043e\x5cu043\
f\x5cu0438\x5cu0441\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u0435\x5cu0438\x5cu044\
1\x5cu043f\x5cu043e\x5cu0\
43b\x5cu044c\x5cu0437\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu0438\x5cu0\
44f\x5cu0441\x5cu043e\x5c\
u0431\x5cu044b\x5cu044\
2\x5cu0438\x5cu044f\x5cu0\
434\x5cu043e\x5cu0441\x5c\
u0442\x5cu0443\x5cu043\
f\x5cu0436\x5cu0443\x5cu0\
440\x5cu043d\x5cu0430\x5c\
u043b\x5cu0430\x5cu044\
0\x5cu0435\x5cu0433\x5cu0\
438\x5cu0441\x5cu0442\x5c\
u0440\x5cu0430\x5cu044\
6\x5cu0438\x5cu0438 \x5cu\
043e\x5cu043f\x5cu0438\
\x5cu0441\x5cu0430\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
0438\x5cu0441\x5cu043f\
\x5cu043e\x5cu043b\x5cu04\
4c\x5cu0437\x5cu043e\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
41\x5cu043e\x5cu0431\x5cu\
044b\x5cu0442\x5cu0438\
\x5cu044f\x5cu043e\x5cu04\
42\x5cu043a\x5cu0430\x5cu\
0437\x5cu0432\x5cu0434\
\x5cu043e\x5cu0441\x5cu04\
42\x5cu0443\x5cu043f\x5cu\
0435\x5cu0436\x5cu0443\
\x5cu0440\x5cu043d\x5cu04\
30\x5cu043b\x5cu0430\x5cu\
0440\x5cu0435\x5cu0433\
\x5cu0438\x5cu0441\x5cu04\
42\x5cu0440\x5cu0430\x5cu\
0446\x5cu0438\x5cu0438\
 \x5cu043e\x5cu043f\x5cu0\
438\x5cu0441\x5cu0430\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu043e\x5cu0431\x5cu0\
440\x5cu0430\x5cu0431\x5c\
u043e\x5cu0442\x5cu043\
a\x5cu0438\x5cu0440\x5cu0\
430\x5cu0441\x5cu0448\x5c\
u0438\x5cu0444\x5cu044\
0\x5cu043e\x5cu0432\x5cu0\
43a\x5cu0438\x5cu043a\x5c\
u043e\x5cu043c\x5cu043\
f\x5cu043e\x5cu043d\x5cu0\
43e\x5cu0432\x5cu043a\x5c\
u0438\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu043e\
\x5cu043f\x5cu0438\x5cu04\
41\x5cu0430\x5cu043d\x5cu\
0438\x5cu0435\x5cu043f\
\x5cu0435\x5cu0440\x5cu04\
35\x5cu0434\x5cu0430\x5cu\
0432\x5cu0430\x5cu0435\
\x5cu043c\x5cu043e\x5cu04\
33\x5cu043e\x5cu0444\x5cu\
0430\x5cu0439\x5cu043b\
\x5cu0430 \x5cu043e\x5cu0\
43f\x5cu0438\x5cu0441\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu0435\x5cu0442\x5cu0\
438\x5cu043f\x5cu043e\x5c\
u0432 \x5cu043e\x5cu04\
3f\x5cu0440\x5cu0435\x5cu\
0434\x5cu0435\x5cu043b\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0433\x5cu\
0440\x5cu0443\x5cu043f\
\x5cu043f\x5cu044b\x5cu04\
30\x5cu0442\x5cu0440\x5cu\
0438\x5cu0431\x5cu0443\
\x5cu0442\x5cu043e\x5cu04\
32xs \x5cu043e\x5cu043\
f\x5cu0440\x5cu0435\x5cu0\
434\x5cu0435\x5cu043b\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435\x5cu0433\x5cu0\
440\x5cu0443\x5cu043f\x5c\
u043f\x5cu044b\x5cu043\
c\x5cu043e\x5cu0434\x5cu0\
435\x5cu043b\x5cu0438x\
s \x5cu043e\x5cu043f\x5cu\
0440\x5cu0435\x5cu0434\
\x5cu0435\x5cu043b\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
0435\x5cu043e\x5cu0433\
\x5cu0440\x5cu0430\x5cu04\
3d\x5cu0438\x5cu0447\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu0438\x5cu04\
34\x5cu0435\x5cu043d\x5cu\
0442\x5cu0438\x5cu0447\
\x5cu043d\x5cu043e\x5cu04\
41\x5cu0442\x5cu0438xs\
 \x5cu043e\x5cu043f\x5cu0\
440\x5cu0435\x5cu0434\x5c\
u0435\x5cu043b\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu043f\x5cu0440\x5c\
u043e\x5cu0441\x5cu044\
2\x5cu043e\x5cu0433\x5cu0\
43e\x5cu0442\x5cu0438\x5c\
u043f\x5cu0430xs \x5cu\
043e\x5cu043f\x5cu0440\
\x5cu0435\x5cu0434\x5cu04\
35\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0441\x5cu043e\x5cu04\
41\x5cu0442\x5cu0430\x5cu\
0432\x5cu043d\x5cu043e\
\x5cu0433\x5cu043e\x5cu04\
42\x5cu0438\x5cu043f\x5cu\
0430xs \x5cu043e\x5cu0\
43f\x5cu0440\x5cu0435\x5c\
u0434\x5cu0435\x5cu043\
b\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435\x5cu0442\x5c\
u0438\x5cu043f\x5cu043\
0\x5cu0434\x5cu043e\x5cu0\
43a\x5cu0443\x5cu043c\x5c\
u0435\x5cu043d\x5cu044\
2\x5cu0430dom \x5cu043\
e\x5cu043f\x5cu0440\x5cu0\
435\x5cu0434\x5cu0435\x5c\
u043b\x5cu0435\x5cu043\
d\x5cu0438\x5cu044fxpa\
thxs \x5cu043e\x5cu044\
2\x5cu0431\x5cu043e\x5cu0\
440\x5cu043a\x5cu043e\x5c\
u043c\x5cu043f\x5cu043\
e\x5cu043d\x5cu043e\x5cu0\
432\x5cu043a\x5cu0438\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445 \x5cu043f\x5cu0430\
\x5cu043a\x5cu0435\x5cu04\
42\x5cu043e\x5cu0442\x5cu\
043e\x5cu0431\x5cu0440\
\x5cu0430\x5cu0436\x5cu04\
30\x5cu0435\x5cu043c\x5cu\
044b\x5cu0445\x5cu0434\
\x5cu043e\x5cu043a\x5cu04\
43\x5cu043c\x5cu0435\x5cu\
043d\x5cu0442\x5cu043e\
\x5cu0432 \x5cu043f\x5cu0\
430\x5cu0440\x5cu0430\x5c\
u043c\x5cu0435\x5cu044\
2\x5cu0440\x5cu0432\x5cu0\
44b\x5cu0431\x5cu043e\x5c\
u0440\x5cu0430 \x5cu04\
3f\x5cu0430\x5cu0440\x5cu\
0430\x5cu043c\x5cu0435\
\x5cu0442\x5cu0440\x5cu04\
3a\x5cu043e\x5cu043c\x5cu\
043f\x5cu043e\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
3a\x5cu0438\x5cu0434\x5cu\
0430\x5cu043d\x5cu043d\
\x5cu044b\x5cu0445 \x5cu0\
43f\x5cu0430\x5cu0440\x5c\
u0430\x5cu043c\x5cu043\
5\x5cu0442\x5cu0440\x5cu0\
44b\x5cu0437\x5cu0430\x5c\
u043f\x5cu0438\x5cu044\
1\x5cu0438json \x5cu04\
3f\x5cu0430\x5cu0440\x5cu\
0430\x5cu043c\x5cu0435\
\x5cu0442\x5cu0440\x5cu04\
4b\x5cu0437\x5cu0430\x5cu\
043f\x5cu0438\x5cu0441\
\x5cu0438xml \x5cu043f\
\x5cu0430\x5cu0440\x5cu04\
30\x5cu043c\x5cu0435\x5cu\
0442\x5cu0440\x5cu044b\
\x5cu0447\x5cu0442\x5cu04\
35\x5cu043d\x5cu0438\x5cu\
044fxml \x5cu043f\x5cu\
0435\x5cu0440\x5cu0435\
\x5cu043e\x5cu043f\x5cu04\
40\x5cu0435\x5cu0434\x5cu\
0435\x5cu043b\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35xs \x5cu043f\x5cu043\
b\x5cu0430\x5cu043d\x5cu0\
438\x5cu0440\x5cu043e\x5c\
u0432\x5cu0449\x5cu043\
8\x5cu043a \x5cu043f\x5cu\
043e\x5cu043b\x5cu0435\
\x5cu0430\x5cu043d\x5cu04\
30\x5cu043b\x5cu0438\x5cu\
0437\x5cu0430\x5cu0434\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0445 \x5c\
u043f\x5cu043e\x5cu043\
b\x5cu0435\x5cu043a\x5cu0\
43e\x5cu043c\x5cu043f\x5c\
u043e\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
438\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 \x5cu043f\x5cu\
043e\x5cu0441\x5cu0442\
\x5cu0440\x5cu043e\x5cu04\
38\x5cu0442\x5cu0435\x5cu\
043b\x5cu044cdom \x5cu\
043f\x5cu043e\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
3e\x5cu0438\x5cu0442\x5cu\
0435\x5cu043b\x5cu044c\
\x5cu0437\x5cu0430\x5cu04\
3f\x5cu0440\x5cu043e\x5cu\
0441\x5cu0430 \x5cu043\
f\x5cu043e\x5cu0441\x5cu0\
442\x5cu0440\x5cu043e\x5c\
u0438\x5cu0442\x5cu043\
5\x5cu043b\x5cu044c\x5cu0\
43e\x5cu0442\x5cu0447\x5c\
u0435\x5cu0442\x5cu043\
0 \x5cu043f\x5cu043e\x5cu\
0441\x5cu0442\x5cu0440\
\x5cu043e\x5cu0438\x5cu04\
42\x5cu0435\x5cu043b\x5cu\
044c\x5cu043e\x5cu0442\
\x5cu0447\x5cu0435\x5cu04\
42\x5cu0430\x5cu0430\x5cu\
043d\x5cu0430\x5cu043b\
\x5cu0438\x5cu0437\x5cu04\
30\x5cu0434\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445 \x5cu043f\x5cu0\
43e\x5cu0441\x5cu0442\x5c\
u0440\x5cu043e\x5cu043\
8\x5cu0442\x5cu0435\x5cu0\
43b\x5cu044c\x5cu0441\x5c\
u0445\x5cu0435\x5cu043\
cxml \x5cu043f\x5cu043\
e\x5cu0442\x5cu043e\x5cu0\
43a \x5cu043f\x5cu043e\
\x5cu0442\x5cu043e\x5cu04\
3a\x5cu0432\x5cu043f\x5cu\
0430\x5cu043c\x5cu044f\
\x5cu0442\x5cu0438 \x5cu0\
43f\x5cu043e\x5cu0447\x5c\
u0442\x5cu0430 \x5cu04\
3f\x5cu043e\x5cu0447\x5cu\
0442\x5cu043e\x5cu0432\
\x5cu043e\x5cu0435\x5cu04\
41\x5cu043e\x5cu043e\x5cu\
0431\x5cu0449\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35 \x5cu043f\x5cu0440\x5c\
u0435\x5cu043e\x5cu043\
1\x5cu0440\x5cu0430\x5cu0\
437\x5cu043e\x5cu0432\x5c\
u0430\x5cu043d\x5cu043\
8\x5cu0435xsl \x5cu043\
f\x5cu0440\x5cu0435\x5cu0\
43e\x5cu0431\x5cu0440\x5c\
u0430\x5cu0437\x5cu043\
e\x5cu0432\x5cu0430\x5cu0\
43d\x5cu0438\x5cu0435\x5c\
u043a\x5cu043a\x5cu043\
0\x5cu043d\x5cu043e\x5cu0\
43d\x5cu0438\x5cu0447\x5c\
u0435\x5cu0441\x5cu043\
a\x5cu043e\x5cu043c\x5cu0\
443xml \x5cu043f\x5cu0\
440\x5cu043e\x5cu0446\x5c\
u0435\x5cu0441\x5cu044\
1\x5cu043e\x5cu0440\x5cu0\
432\x5cu044b\x5cu0432\x5c\
u043e\x5cu0434\x5cu043\
0\x5cu0440\x5cu0435\x5cu0\
437\x5cu0443\x5cu043b\x5c\
u044c\x5cu0442\x5cu043\
0\x5cu0442\x5cu0430\x5cu0\
43a\x5cu043e\x5cu043c\x5c\
u043f\x5cu043e\x5cu043\
d\x5cu043e\x5cu0432\x5cu0\
43a\x5cu0438\x5cu0434\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu044b\x5cu0445\x5cu0\
432\x5cu043a\x5cu043e\x5c\
u043b\x5cu043b\x5cu043\
5\x5cu043a\x5cu0446\x5cu0\
438\x5cu044e\x5cu0437\x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu0439 \x5cu043f\
\x5cu0440\x5cu043e\x5cu04\
46\x5cu0435\x5cu0441\x5cu\
0441\x5cu043e\x5cu0440\
\x5cu0432\x5cu044b\x5cu04\
32\x5cu043e\x5cu0434\x5cu\
0430\x5cu0440\x5cu0435\
\x5cu0437\x5cu0443\x5cu04\
3b\x5cu044c\x5cu0442\x5cu\
0430\x5cu0442\x5cu0430\
\x5cu043a\x5cu043e\x5cu04\
3c\x5cu043f\x5cu043e\x5cu\
043d\x5cu043e\x5cu0432\
\x5cu043a\x5cu0438\x5cu04\
34\x5cu0430\x5cu043d\x5cu\
043d\x5cu044b\x5cu0445\
\x5cu0432\x5cu0442\x5cu04\
30\x5cu0431\x5cu043b\x5cu\
0438\x5cu0447\x5cu043d\
\x5cu044b\x5cu0439\x5cu04\
34\x5cu043e\x5cu043a\x5cu\
0443\x5cu043c\x5cu0435\
\x5cu043d\x5cu0442 \x5cu0\
43f\x5cu0440\x5cu043e\x5c\
u0446\x5cu0435\x5cu044\
1\x5cu0441\x5cu043e\x5cu0\
440\x5cu043a\x5cu043e\x5c\
u043c\x5cu043f\x5cu043\
e\x5cu043d\x5cu043e\x5cu0\
432\x5cu043a\x5cu0438\x5c\
u0434\x5cu0430\x5cu043\
d\x5cu043d\x5cu044b\x5cu0\
445 \x5cu0440\x5cu0430\
\x5cu0437\x5cu044b\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
043e\x5cu0432\x5cu0430\
\x5cu0442\x5cu0435\x5cu04\
3b\x5cu044c\x5cu043f\x5cu\
0440\x5cu043e\x5cu0441\
\x5cu0442\x5cu0440\x5cu04\
30\x5cu043d\x5cu0441\x5cu\
0442\x5cu0432\x5cu0438\
\x5cu043c\x5cu0435\x5cu04\
3ddom \x5cu0440\x5cu04\
30\x5cu043c\x5cu043a\x5cu\
0430 \x5cu0440\x5cu043\
0\x5cu0441\x5cu043f\x5cu0\
438\x5cu0441\x5cu0430\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu0440\x5cu0435\x5cu0\
433\x5cu043b\x5cu0430\x5c\
u043c\x5cu0435\x5cu043\
d\x5cu0442\x5cu043d\x5cu0\
43e\x5cu0433\x5cu043e\x5c\
u0437\x5cu0430\x5cu043\
4\x5cu0430\x5cu043d\x5cu0\
438\x5cu044f \x5cu0440\
\x5cu0430\x5cu0441\x5cu04\
48\x5cu0438\x5cu0440\x5cu\
0435\x5cu043d\x5cu043d\
\x5cu043e\x5cu0435\x5cu04\
38\x5cu043c\x5cu044fxm\
l \x5cu0440\x5cu0435\x5cu\
0437\x5cu0443\x5cu043b\
\x5cu044c\x5cu0442\x5cu04\
30\x5cu0442\x5cu0447\x5cu\
0442\x5cu0435\x5cu043d\
\x5cu0438\x5cu044f\x5cu04\
34\x5cu0430\x5cu043d\x5cu\
043d\x5cu044b\x5cu0445\
 \x5cu0441\x5cu0432\x5cu0\
43e\x5cu0434\x5cu043d\x5c\
u0430\x5cu044f\x5cu043\
4\x5cu0438\x5cu0430\x5cu0\
433\x5cu0440\x5cu0430\x5c\
u043c\x5cu043c\x5cu043\
0 \x5cu0441\x5cu0432\x5cu\
044f\x5cu0437\x5cu044c\
\x5cu043f\x5cu0430\x5cu04\
40\x5cu0430\x5cu043c\x5cu\
0435\x5cu0442\x5cu0440\
\x5cu0430\x5cu0432\x5cu04\
4b\x5cu0431\x5cu043e\x5cu\
0440\x5cu0430 \x5cu044\
1\x5cu0432\x5cu044f\x5cu0\
437\x5cu044c\x5cu043f\x5c\
u043e\x5cu0442\x5cu043\
8\x5cu043f\x5cu0443 \x5cu\
0441\x5cu0432\x5cu044f\
\x5cu0437\x5cu044c\x5cu04\
3f\x5cu043e\x5cu0442\x5cu\
0438\x5cu043f\x5cu0443\
\x5cu043a\x5cu043e\x5cu04\
3c\x5cu043f\x5cu043e\x5cu\
043d\x5cu043e\x5cu0432\
\x5cu043a\x5cu0438\x5cu04\
34\x5cu0430\x5cu043d\x5cu\
043d\x5cu044b\x5cu0445\
 \x5cu0441\x5cu0435\x5cu0\
440\x5cu0438\x5cu0430\x5c\
u043b\x5cu0438\x5cu043\
7\x5cu0430\x5cu0442\x5cu0\
43e\x5cu0440xdto \x5cu\
0441\x5cu0435\x5cu0440\
\x5cu0442\x5cu0438\x5cu04\
44\x5cu0438\x5cu043a\x5cu\
0430\x5cu0442\x5cu043a\
\x5cu043b\x5cu0438\x5cu04\
35\x5cu043d\x5cu0442\x5cu\
0430windows \x5cu04\
41\x5cu0435\x5cu0440\x5cu\
0442\x5cu0438\x5cu0444\
\x5cu0438\x5cu043a\x5cu04\
30\x5cu0442\x5cu043a\x5cu\
043b\x5cu0438\x5cu0435\
\x5cu043d\x5cu0442\x5cu04\
30\x5cu0444\x5cu0430\x5cu\
0439\x5cu043b \x5cu044\
1\x5cu0435\x5cu0440\x5cu0\
442\x5cu0438\x5cu0444\x5c\
u0438\x5cu043a\x5cu043\
0\x5cu0442\x5cu043a\x5cu0\
440\x5cu0438\x5cu043f\x5c\
u0442\x5cu043e\x5cu043\
3\x5cu0440\x5cu0430\x5cu0\
444\x5cu0438\x5cu0438 \
\x5cu0441\x5cu0435\x5cu04\
40\x5cu0442\x5cu0438\x5cu\
0444\x5cu0438\x5cu043a\
\x5cu0430\x5cu0442\x5cu04\
4b\x5cu0443\x5cu0434\x5cu\
043e\x5cu0441\x5cu0442\
\x5cu043e\x5cu0432\x5cu04\
35\x5cu0440\x5cu044f\x5cu\
044e\x5cu0449\x5cu0438\
\x5cu0445\x5cu0446\x5cu04\
35\x5cu043d\x5cu0442\x5cu\
0440\x5cu043e\x5cu0432\
windows \x5cu0441\x5cu\
0435\x5cu0440\x5cu0442\
\x5cu0438\x5cu0444\x5cu04\
38\x5cu043a\x5cu0430\x5cu\
0442\x5cu044b\x5cu0443\
\x5cu0434\x5cu043e\x5cu04\
41\x5cu0442\x5cu043e\x5cu\
0432\x5cu0435\x5cu0440\
\x5cu044f\x5cu044e\x5cu04\
49\x5cu0438\x5cu0445\x5cu\
0446\x5cu0435\x5cu043d\
\x5cu0442\x5cu0440\x5cu04\
3e\x5cu0432\x5cu0444\x5cu\
0430\x5cu0439\x5cu043b\
 \x5cu0441\x5cu0436\x5cu0\
430\x5cu0442\x5cu0438\x5c\
u0435\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu0441\
\x5cu0438\x5cu0441\x5cu04\
42\x5cu0435\x5cu043c\x5cu\
043d\x5cu0430\x5cu044f\
\x5cu0438\x5cu043d\x5cu04\
44\x5cu043e\x5cu0440\x5cu\
043c\x5cu0430\x5cu0446\
\x5cu0438\x5cu044f \x5cu0\
441\x5cu043e\x5cu043e\x5c\
u0431\x5cu0449\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu043f\x5cu043e\x5c\
u043b\x5cu044c\x5cu043\
7\x5cu043e\x5cu0432\x5cu0\
430\x5cu0442\x5cu0435\x5c\
u043b\x5cu044e \x5cu04\
41\x5cu043e\x5cu0447\x5cu\
0435\x5cu0442\x5cu0430\
\x5cu043d\x5cu0438\x5cu04\
35\x5cu043a\x5cu043b\x5cu\
0430\x5cu0432\x5cu0438\
\x5cu0448 \x5cu0441\x5cu0\
440\x5cu0430\x5cu0432\x5c\
u043d\x5cu0435\x5cu043\
d\x5cu0438\x5cu0435\x5cu0\
437\x5cu043d\x5cu0430\x5c\
u0447\x5cu0435\x5cu043\
d\x5cu0438\x5cu0439 \x5cu\
0441\x5cu0442\x5cu0430\
\x5cu043d\x5cu0434\x5cu04\
30\x5cu0440\x5cu0442\x5cu\
043d\x5cu0430\x5cu044f\
\x5cu0434\x5cu0430\x5cu04\
42\x5cu0430\x5cu043d\x5cu\
0430\x5cu0447\x5cu0430\
\x5cu043b\x5cu0430 \x5cu0\
441\x5cu0442\x5cu0430\x5c\
u043d\x5cu0434\x5cu043\
0\x5cu0440\x5cu0442\x5cu0\
43d\x5cu044b\x5cu0439\x5c\
u043f\x5cu0435\x5cu044\
0\x5cu0438\x5cu043e\x5cu0\
434 \x5cu0441\x5cu0445\
\x5cu0435\x5cu043c\x5cu04\
30xml \x5cu0441\x5cu04\
45\x5cu0435\x5cu043c\x5cu\
0430\x5cu043a\x5cu043e\
\x5cu043c\x5cu043f\x5cu04\
3e\x5cu043d\x5cu043e\x5cu\
0432\x5cu043a\x5cu0438\
\x5cu0434\x5cu0430\x5cu04\
3d\x5cu043d\x5cu044b\x5cu\
0445 \x5cu0442\x5cu043\
0\x5cu0431\x5cu043b\x5cu0\
438\x5cu0447\x5cu043d\x5c\
u044b\x5cu0439\x5cu043\
4\x5cu043e\x5cu043a\x5cu0\
443\x5cu043c\x5cu0435\x5c\
u043d\x5cu0442 \x5cu04\
42\x5cu0435\x5cu043a\x5cu\
0441\x5cu0442\x5cu043e\
\x5cu0432\x5cu044b\x5cu04\
39\x5cu0434\x5cu043e\x5cu\
043a\x5cu0443\x5cu043c\
\x5cu0435\x5cu043d\x5cu04\
42 \x5cu0442\x5cu0435\x5c\
u0441\x5cu0442\x5cu043\
8\x5cu0440\x5cu0443\x5cu0\
435\x5cu043c\x5cu043e\x5c\
u0435\x5cu043f\x5cu044\
0\x5cu0438\x5cu043b\x5cu0\
43e\x5cu0436\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5 \x5cu0442\x5cu0438\x5cu\
043f\x5cu0434\x5cu0430\
\x5cu043d\x5cu043d\x5cu04\
4b\x5cu0445xml \x5cu04\
43\x5cu043d\x5cu0438\x5cu\
043a\x5cu0430\x5cu043b\
\x5cu044c\x5cu043d\x5cu04\
4b\x5cu0439\x5cu0438\x5cu\
0434\x5cu0435\x5cu043d\
\x5cu0442\x5cu0438\x5cu04\
44\x5cu0438\x5cu043a\x5cu\
0430\x5cu0442\x5cu043e\
\x5cu0440 \x5cu0444\x5cu0\
430\x5cu0431\x5cu0440\x5c\
u0438\x5cu043a\x5cu043\
0xdto \x5cu0444\x5cu04\
30\x5cu0439\x5cu043b \x5c\
u0444\x5cu0430\x5cu043\
9\x5cu043b\x5cu043e\x5cu0\
432\x5cu044b\x5cu0439\x5c\
u043f\x5cu043e\x5cu044\
2\x5cu043e\x5cu043a \x5cu\
0444\x5cu0430\x5cu0441\
\x5cu0435\x5cu0442\x5cu04\
34\x5cu043b\x5cu0438\x5cu\
043d\x5cu044bxs \x5cu0\
444\x5cu0430\x5cu0441\x5c\
u0435\x5cu0442\x5cu043\
a\x5cu043e\x5cu043b\x5cu0\
438\x5cu0447\x5cu0435\x5c\
u0441\x5cu0442\x5cu043\
2\x5cu0430\x5cu0440\x5cu0\
430\x5cu0437\x5cu0440\x5c\
u044f\x5cu0434\x5cu043\
e\x5cu0432\x5cu0434\x5cu0\
440\x5cu043e\x5cu0431\x5c\
u043d\x5cu043e\x5cu043\
9\x5cu0447\x5cu0430\x5cu0\
441\x5cu0442\x5cu0438x\
s \x5cu0444\x5cu0430\x5cu\
0441\x5cu0435\x5cu0442\
\x5cu043c\x5cu0430\x5cu04\
3a\x5cu0441\x5cu0438\x5cu\
043c\x5cu0430\x5cu043b\
\x5cu044c\x5cu043d\x5cu04\
3e\x5cu0433\x5cu043e\x5cu\
0432\x5cu043a\x5cu043b\
\x5cu044e\x5cu0447\x5cu04\
30\x5cu044e\x5cu0449\x5cu\
0435\x5cu0433\x5cu043e\
\x5cu0437\x5cu043d\x5cu04\
30\x5cu0447\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
xs \x5cu0444\x5cu0430\x5c\
u0441\x5cu0435\x5cu044\
2\x5cu043c\x5cu0430\x5cu0\
43a\x5cu0441\x5cu0438\x5c\
u043c\x5cu0430\x5cu043\
b\x5cu044c\x5cu043d\x5cu0\
43e\x5cu0433\x5cu043e\x5c\
u0438\x5cu0441\x5cu043\
a\x5cu043b\x5cu044e\x5cu0\
447\x5cu0430\x5cu044e\x5c\
u0449\x5cu0435\x5cu043\
3\x5cu043e\x5cu0437\x5cu0\
43d\x5cu0430\x5cu0447\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu044fxs \x5cu0444\
\x5cu0430\x5cu0441\x5cu04\
35\x5cu0442\x5cu043c\x5cu\
0430\x5cu043a\x5cu0441\
\x5cu0438\x5cu043c\x5cu04\
30\x5cu043b\x5cu044c\x5cu\
043d\x5cu043e\x5cu0439\
\x5cu0434\x5cu043b\x5cu04\
38\x5cu043d\x5cu044bxs\
 \x5cu0444\x5cu0430\x5cu0\
441\x5cu0435\x5cu0442\x5c\
u043c\x5cu0438\x5cu043\
d\x5cu0438\x5cu043c\x5cu0\
430\x5cu043b\x5cu044c\x5c\
u043d\x5cu043e\x5cu043\
3\x5cu043e\x5cu0432\x5cu0\
43a\x5cu043b\x5cu044e\x5c\
u0447\x5cu0430\x5cu044\
e\x5cu0449\x5cu0435\x5cu0\
433\x5cu043e\x5cu0437\x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu044fxs \x5cu04\
44\x5cu0430\x5cu0441\x5cu\
0435\x5cu0442\x5cu043c\
\x5cu0438\x5cu043d\x5cu04\
38\x5cu043c\x5cu0430\x5cu\
043b\x5cu044c\x5cu043d\
\x5cu043e\x5cu0433\x5cu04\
3e\x5cu0438\x5cu0441\x5cu\
043a\x5cu043b\x5cu044e\
\x5cu0447\x5cu0430\x5cu04\
4e\x5cu0449\x5cu0435\x5cu\
0433\x5cu043e\x5cu0437\
\x5cu043d\x5cu0430\x5cu04\
47\x5cu0435\x5cu043d\x5cu\
0438\x5cu044fxs \x5cu0\
444\x5cu0430\x5cu0441\x5c\
u0435\x5cu0442\x5cu043\
c\x5cu0438\x5cu043d\x5cu0\
438\x5cu043c\x5cu0430\x5c\
u043b\x5cu044c\x5cu043\
d\x5cu043e\x5cu0439\x5cu0\
434\x5cu043b\x5cu0438\x5c\
u043d\x5cu044bxs \x5cu\
0444\x5cu0430\x5cu0441\
\x5cu0435\x5cu0442\x5cu04\
3e\x5cu0431\x5cu0440\x5cu\
0430\x5cu0437\x5cu0446\
\x5cu0430xs \x5cu0444\x5c\
u0430\x5cu0441\x5cu043\
5\x5cu0442\x5cu043e\x5cu0\
431\x5cu0449\x5cu0435\x5c\
u0433\x5cu043e\x5cu043\
a\x5cu043e\x5cu043b\x5cu0\
438\x5cu0447\x5cu0435\x5c\
u0441\x5cu0442\x5cu043\
2\x5cu0430\x5cu0440\x5cu0\
430\x5cu0437\x5cu0440\x5c\
u044f\x5cu0434\x5cu043\
e\x5cu0432xs \x5cu0444\
\x5cu0430\x5cu0441\x5cu04\
35\x5cu0442\x5cu043f\x5cu\
0435\x5cu0440\x5cu0435\
\x5cu0447\x5cu0438\x5cu04\
41\x5cu043b\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
xs \x5cu0444\x5cu0430\x5c\
u0441\x5cu0435\x5cu044\
2\x5cu043f\x5cu0440\x5cu0\
43e\x5cu0431\x5cu0435\x5c\
u043b\x5cu044c\x5cu043\
d\x5cu044b\x5cu0445\x5cu0\
441\x5cu0438\x5cu043c\x5c\
u0432\x5cu043e\x5cu043\
b\x5cu043e\x5cu0432xs \
\x5cu0444\x5cu0438\x5cu04\
3b\x5cu044c\x5cu0442\x5cu\
0440\x5cu0443\x5cu0437\
\x5cu043b\x5cu043e\x5cu04\
32dom \x5cu0444\x5cu04\
3e\x5cu0440\x5cu043c\x5cu\
0430\x5cu0442\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0430\x5cu043d\x5cu\
043d\x5cu0430\x5cu044f\
\x5cu0441\x5cu0442\x5cu04\
40\x5cu043e\x5cu043a\x5cu\
0430 \x5cu0444\x5cu043\
e\x5cu0440\x5cu043c\x5cu0\
430\x5cu0442\x5cu0438\x5c\
u0440\x5cu043e\x5cu043\
2\x5cu0430\x5cu043d\x5cu0\
43d\x5cu044b\x5cu0439\x5c\
u0434\x5cu043e\x5cu043\
a\x5cu0443\x5cu043c\x5cu0\
435\x5cu043d\x5cu0442 \
\x5cu0444\x5cu0440\x5cu04\
30\x5cu0433\x5cu043c\x5cu\
0435\x5cu043d\x5cu0442\
xs \x5cu0445\x5cu0435\x5c\
u0448\x5cu0438\x5cu044\
0\x5cu043e\x5cu0432\x5cu0\
430\x5cu043d\x5cu0438\x5c\
u0435\x5cu0434\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0445 \x5cu0445\
\x5cu0440\x5cu0430\x5cu04\
3d\x5cu0438\x5cu043b\x5cu\
0438\x5cu0449\x5cu0435\
\x5cu0437\x5cu043d\x5cu04\
30\x5cu0447\x5cu0435\x5cu\
043d\x5cu0438\x5cu044f\
 \x5cu0446\x5cu0432\x5cu0\
435\x5cu0442 \x5cu0447\
\x5cu0442\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435fa\
stinfoset \x5cu0447\
\x5cu0442\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435ht\
ml \x5cu0447\x5cu0442\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435json \x5cu04\
47\x5cu0442\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
xml \x5cu0447\x5cu0442\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435zip\x5cu044\
4\x5cu0430\x5cu0439\x5cu0\
43b\x5cu0430 \x5cu0447\
\x5cu0442\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435\x5cu\
0434\x5cu0430\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
45 \x5cu0447\x5cu0442\x5c\
u0435\x5cu043d\x5cu043\
8\x5cu0435\x5cu0442\x5cu0\
435\x5cu043a\x5cu0441\x5c\
u0442\x5cu0430 \x5cu04\
47\x5cu0442\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
\x5cu0443\x5cu0437\x5cu04\
3b\x5cu043e\x5cu0432do\
m \x5cu0448\x5cu0440\x5cu\
0438\x5cu0444\x5cu0442\
 \x5cu044d\x5cu043b\x5cu0\
435\x5cu043c\x5cu0435\x5c\
u043d\x5cu0442\x5cu044\
0\x5cu0435\x5cu0437\x5cu0\
443\x5cu043b\x5cu044c\x5c\
u0442\x5cu0430\x5cu044\
2\x5cu0430\x5cu043a\x5cu0\
43e\x5cu043c\x5cu043f\x5c\
u043e\x5cu043d\x5cu043\
e\x5cu0432\x5cu043a\x5cu0\
438\x5cu0434\x5cu0430\x5c\
u043d\x5cu043d\x5cu044\
b\x5cu0445 comsafea\
rray \x5cu0434\x5cu043\
5\x5cu0440\x5cu0435\x5cu0\
432\x5cu043e\x5cu0437\x5c\
u043d\x5cu0430\x5cu044\
7\x5cu0435\x5cu043d\x5cu0\
438\x5cu0439 \x5cu043c\
\x5cu0430\x5cu0441\x5cu04\
41\x5cu0438\x5cu0432 \x5c\
u0441\x5cu043e\x5cu043\
e\x5cu0442\x5cu0432\x5cu0\
435\x5cu0442\x5cu0441\x5c\
u0442\x5cu0432\x5cu043\
8\x5cu0435 \x5cu0441\x5cu\
043f\x5cu0438\x5cu0441\
\x5cu043e\x5cu043a\x5cu04\
37\x5cu043d\x5cu0430\x5cu\
0447\x5cu0435\x5cu043d\
\x5cu0438\x5cu0439 \x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0443\x5cu043a\x5cu044\
2\x5cu0443\x5cu0440\x5cu0\
430 \x5cu0442\x5cu0430\
\x5cu0431\x5cu043b\x5cu04\
38\x5cu0446\x5cu0430\x5cu\
0437\x5cu043d\x5cu0430\
\x5cu0447\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0439 \x5c\
u0444\x5cu0438\x5cu043\
a\x5cu0441\x5cu0438\x5cu0\
440\x5cu043e\x5cu0432\x5c\
u0430\x5cu043d\x5cu043\
d\x5cu0430\x5cu044f\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0443\x5cu043a\x5cu044\
2\x5cu0443\x5cu0440\x5cu0\
430 \x5cu0444\x5cu0438\
\x5cu043a\x5cu0441\x5cu04\
38\x5cu0440\x5cu043e\x5cu\
0432\x5cu0430\x5cu043d\
\x5cu043d\x5cu043e\x5cu04\
35\x5cu0441\x5cu043e\x5cu\
043e\x5cu0442\x5cu0432\
\x5cu0435\x5cu0442\x5cu04\
41\x5cu0442\x5cu0432\x5cu\
0438\x5cu0435 \x5cu044\
4\x5cu0438\x5cu043a\x5cu0\
441\x5cu0438\x5cu0440\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0439\x5cu043c\x5c\
u0430\x5cu0441\x5cu044\
1\x5cu0438\x5cu0432 \x22,\
\x0aliteral:e},cont\
ains:[{className\
:\x22meta\x22,begin:\x22#\
|&\x22,end:\x22$\x22,keyw\
ords:{$pattern:x\
,\x0akeyword:n+\x22\x5cu0\
437\x5cu0430\x5cu0433\x5c\
u0440\x5cu0443\x5cu043\
7\x5cu0438\x5cu0442\x5cu0\
44c\x5cu0438\x5cu0437\x5c\
u0444\x5cu0430\x5cu043\
9\x5cu043b\x5cu0430 \x5cu\
0432\x5cu0435\x5cu0431\
\x5cu043a\x5cu043b\x5cu04\
38\x5cu0435\x5cu043d\x5cu\
0442 \x5cu0432\x5cu043\
c\x5cu0435\x5cu0441\x5cu0\
442\x5cu043e \x5cu0432\
\x5cu043d\x5cu0435\x5cu04\
48\x5cu043d\x5cu0435\x5cu\
0435\x5cu0441\x5cu043e\
\x5cu0435\x5cu0434\x5cu04\
38\x5cu043d\x5cu0435\x5cu\
043d\x5cu0438\x5cu0435\
 \x5cu043a\x5cu043b\x5cu0\
438\x5cu0435\x5cu043d\x5c\
u0442 \x5cu043a\x5cu04\
3e\x5cu043d\x5cu0435\x5cu\
0446\x5cu043e\x5cu0431\
\x5cu043b\x5cu0430\x5cu04\
41\x5cu0442\x5cu0438 \x5c\
u043c\x5cu043e\x5cu043\
1\x5cu0438\x5cu043b\x5cu0\
44c\x5cu043d\x5cu043e\x5c\
u0435\x5cu043f\x5cu044\
0\x5cu0438\x5cu043b\x5cu0\
43e\x5cu0436\x5cu0435\x5c\
u043d\x5cu0438\x5cu043\
5\x5cu043a\x5cu043b\x5cu0\
438\x5cu0435\x5cu043d\x5c\
u0442 \x5cu043c\x5cu04\
3e\x5cu0431\x5cu0438\x5cu\
043b\x5cu044c\x5cu043d\
\x5cu043e\x5cu0435\x5cu04\
3f\x5cu0440\x5cu0438\x5cu\
043b\x5cu043e\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435\x5cu0441\x5cu\
0435\x5cu0440\x5cu0432\
\x5cu0435\x5cu0440 \x5cu0\
43d\x5cu0430\x5cu043a\x5c\
u043b\x5cu0438\x5cu043\
5\x5cu043d\x5cu0442\x5cu0\
435 \x5cu043d\x5cu0430\
\x5cu043a\x5cu043b\x5cu04\
38\x5cu0435\x5cu043d\x5cu\
0442\x5cu0435\x5cu043d\
\x5cu0430\x5cu0441\x5cu04\
35\x5cu0440\x5cu0432\x5cu\
0435\x5cu0440\x5cu0435\
 \x5cu043d\x5cu0430\x5cu0\
43a\x5cu043b\x5cu0438\x5c\
u0435\x5cu043d\x5cu044\
2\x5cu0435\x5cu043d\x5cu0\
430\x5cu0441\x5cu0435\x5c\
u0440\x5cu0432\x5cu043\
5\x5cu0440\x5cu0435\x5cu0\
431\x5cu0435\x5cu0437\x5c\
u043a\x5cu043e\x5cu043\
d\x5cu0442\x5cu0435\x5cu0\
43a\x5cu0441\x5cu0442\x5c\
u0430 \x5cu043d\x5cu04\
30\x5cu0441\x5cu0435\x5cu\
0440\x5cu0432\x5cu0435\
\x5cu0440\x5cu0435 \x5cu0\
43d\x5cu0430\x5cu0441\x5c\
u0435\x5cu0440\x5cu043\
2\x5cu0435\x5cu0440\x5cu0\
435\x5cu0431\x5cu0435\x5c\
u0437\x5cu043a\x5cu043\
e\x5cu043d\x5cu0442\x5cu0\
435\x5cu043a\x5cu0441\x5c\
u0442\x5cu0430 \x5cu04\
3e\x5cu0431\x5cu043b\x5cu\
0430\x5cu0441\x5cu0442\
\x5cu044c \x5cu043f\x5cu0\
435\x5cu0440\x5cu0435\x5c\
u0434 \x5cu043f\x5cu04\
3e\x5cu0441\x5cu043b\x5cu\
0435 \x5cu0441\x5cu043\
5\x5cu0440\x5cu0432\x5cu0\
435\x5cu0440 \x5cu0442\
\x5cu043e\x5cu043b\x5cu04\
41\x5cu0442\x5cu044b\x5cu\
0439\x5cu043a\x5cu043b\
\x5cu0438\x5cu0435\x5cu04\
3d\x5cu0442\x5cu043e\x5cu\
0431\x5cu044b\x5cu0447\
\x5cu043d\x5cu043e\x5cu04\
35\x5cu043f\x5cu0440\x5cu\
0438\x5cu043b\x5cu043e\
\x5cu0436\x5cu0435\x5cu04\
3d\x5cu0438\x5cu0435 \x5c\
u0442\x5cu043e\x5cu043\
b\x5cu0441\x5cu0442\x5cu0\
44b\x5cu0439\x5cu043a\x5c\
u043b\x5cu0438\x5cu043\
5\x5cu043d\x5cu0442\x5cu0\
443\x5cu043f\x5cu0440\x5c\
u0430\x5cu0432\x5cu043\
b\x5cu044f\x5cu0435\x5cu0\
43c\x5cu043e\x5cu0435\x5c\
u043f\x5cu0440\x5cu043\
8\x5cu043b\x5cu043e\x5cu0\
436\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435 \x5cu04\
42\x5cu043e\x5cu043d\x5cu\
043a\x5cu0438\x5cu0439\
\x5cu043a\x5cu043b\x5cu04\
38\x5cu0435\x5cu043d\x5cu\
0442 \x22\x0a},contain\
s:[m]},{classNam\
e:\x22function\x22,var\
iants:[{\x0abegin:\x22\
\x5cu043f\x5cu0440\x5cu04\
3e\x5cu0446\x5cu0435\x5cu\
0434\x5cu0443\x5cu0440\
\x5cu0430|\x5cu0444\x5cu0\
443\x5cu043d\x5cu043a\x5c\
u0446\x5cu0438\x5cu044\
f\x22,\x0aend:\x22\x5c\x5c)\x22,\x0ak\
eywords:\x22\x5cu043f\x5c\
u0440\x5cu043e\x5cu044\
6\x5cu0435\x5cu0434\x5cu0\
443\x5cu0440\x5cu0430 \
\x5cu0444\x5cu0443\x5cu04\
3d\x5cu043a\x5cu0446\x5cu\
0438\x5cu044f\x22\x0a},{\x0a\
begin:\x22\x5cu043a\x5cu0\
43e\x5cu043d\x5cu0435\x5c\
u0446\x5cu043f\x5cu044\
0\x5cu043e\x5cu0446\x5cu0\
435\x5cu0434\x5cu0443\x5c\
u0440\x5cu044b|\x5cu04\
3a\x5cu043e\x5cu043d\x5cu\
0435\x5cu0446\x5cu0444\
\x5cu0443\x5cu043d\x5cu04\
3a\x5cu0446\x5cu0438\x5cu\
0438\x22,\x0akeywords:\
\x22\x5cu043a\x5cu043e\x5cu0\
43d\x5cu0435\x5cu0446\x5c\
u043f\x5cu0440\x5cu043\
e\x5cu0446\x5cu0435\x5cu0\
434\x5cu0443\x5cu0440\x5c\
u044b \x5cu043a\x5cu04\
3e\x5cu043d\x5cu0435\x5cu\
0446\x5cu0444\x5cu0443\
\x5cu043d\x5cu043a\x5cu04\
46\x5cu0438\x5cu0438\x22\x0a\
}],contains:[{be\
gin:\x22\x5c\x5c(\x22,end:\x22\x5c\
\x5c)\x22,endsParent:!\
0,contains:[{cla\
ssName:\x22params\x22,\
\x0abegin:x,end:\x22,\x22\
,excludeEnd:!0,e\
ndsWithParent:!0\
,keywords:{$patt\
ern:x,\x0akeyword:\x22\
\x5cu0437\x5cu043d\x5cu04\
30\x5cu0447\x22,litera\
l:e},contains:[o\
,t,a]},m]\x0a},s.in\
herit(s.TITLE_MO\
DE,{begin:x})]},\
m,{className:\x22sy\
mbol\x22,begin:\x22~\x22,\
end:\x22;|:\x22,\x0aexclu\
deEnd:!0},o,t,a,\
{match:/[;()+\x5c-:\
=,]/,className:\x22\
punctuation\x22,rel\
evance:0}]}\x0a}})(\
);hljs.registerL\
anguage(\x221c\x22,s)}\
)();/*! `abnf` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>{const a=e.\
regex,s=e.COMMEN\
T(/;/,/$/)\x0a;retu\
rn{name:\x22Augment\
ed Backus-Naur F\
orm\x22,illegal:/[!\
@#$^&',?+~`|:]/,\
\x0akeywords:[\x22ALPH\
A\x22,\x22BIT\x22,\x22CHAR\x22,\
\x22CR\x22,\x22CRLF\x22,\x22CTL\
\x22,\x22DIGIT\x22,\x22DQUOT\
E\x22,\x22HEXDIG\x22,\x22HTA\
B\x22,\x22LF\x22,\x22LWSP\x22,\x22\
OCTET\x22,\x22SP\x22,\x22VCH\
AR\x22,\x22WSP\x22],\x0acont\
ains:[{scope:\x22op\
erator\x22,match:/=\
\x5c/?/},{scope:\x22at\
tribute\x22,\x0amatch:\
a.concat(/^[a-zA\
-Z][a-zA-Z0-9-]*\
/,/(?=\x5cs*=)/)},s\
,{scope:\x22symbol\x22\
,\x0amatch:/%b[0-1]\
+(-[0-1]+|(\x5c.[0-\
1]+)+)?/},{scope\
:\x22symbol\x22,\x0amatch\
:/%d[0-9]+(-[0-9\
]+|(\x5c.[0-9]+)+)?\
/},{scope:\x22symbo\
l\x22,\x0amatch:/%x[0-\
9A-F]+(-[0-9A-F]\
+|(\x5c.[0-9A-F]+)+\
)?/},{scope:\x22sym\
bol\x22,\x0amatch:/%[s\
i](?=\x22.*\x22)/},e.Q\
UOTE_STRING_MODE\
,e.NUMBER_MODE]}\
}})()\x0a;hljs.regi\
sterLanguage(\x22ab\
nf\x22,e)})();/*! `\
accesslog` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{\x0aconst n=e.reg\
ex,a=[\x22GET\x22,\x22POS\
T\x22,\x22HEAD\x22,\x22PUT\x22,\
\x22DELETE\x22,\x22CONNEC\
T\x22,\x22OPTIONS\x22,\x22PA\
TCH\x22,\x22TRACE\x22]\x0a;r\
eturn{name:\x22Apac\
he Access Log\x22,c\
ontains:[{classN\
ame:\x22number\x22,\x0abe\
gin:/^\x5cd{1,3}\x5c.\x5c\
d{1,3}\x5c.\x5cd{1,3}\x5c\
.\x5cd{1,3}(:\x5cd{1,5\
})?\x5cb/,relevance\
:5},{\x0aclassName:\
\x22number\x22,begin:/\
\x5cb\x5cd+\x5cb/,relevan\
ce:0},{className\
:\x22string\x22,\x0abegin\
:n.concat(/\x22/,n.\
either(...a)),en\
d:/\x22/,keywords:a\
,illegal:/\x5cn/,re\
levance:5,\x0aconta\
ins:[{begin:/HTT\
P\x5c/[12]\x5c.\x5cd'/,re\
levance:5}]},{cl\
assName:\x22string\x22\
,\x0abegin:/\x5c[\x5cd[^\x5c\
]\x5cn]{8,}\x5c]/,ille\
gal:/\x5cn/,relevan\
ce:1},{className\
:\x22string\x22,\x0abegin\
:/\x5c[/,end:/\x5c]/,i\
llegal:/\x5cn/,rele\
vance:0},{classN\
ame:\x22string\x22,\x0abe\
gin:/\x22Mozilla\x5c/\x5c\
d\x5c.\x5cd \x5c(/,end:/\x22\
/,illegal:/\x5cn/,r\
elevance:3},{\x0acl\
assName:\x22string\x22\
,begin:/\x22/,end:/\
\x22/,illegal:/\x5cn/,\
relevance:0}]}}}\
)()\x0a;hljs.regist\
erLanguage(\x22acce\
sslog\x22,e)})();/*\
! `actionscript`\
 grammar compile\
d for Highlight.\
js 11.10.0 */\x0a((\
)=>{var e=(()=>{\
\x22use strict\x22;ret\
urn e=>{\x0aconst a\
=e.regex,t=/[a-z\
A-Z_$][a-zA-Z0-9\
_$]*/,n=a.concat\
(t,a.concat(\x22(\x5c\x5c\
.\x22,t,\x22)*\x22)),s={\x0a\
className:\x22rest_\
arg\x22,begin:/[.]{\
3}/,end:t,releva\
nce:10};return{\x0a\
name:\x22ActionScri\
pt\x22,aliases:[\x22as\
\x22],keywords:{\x0ake\
yword:[\x22as\x22,\x22bre\
ak\x22,\x22case\x22,\x22catc\
h\x22,\x22class\x22,\x22cons\
t\x22,\x22continue\x22,\x22d\
efault\x22,\x22delete\x22\
,\x22do\x22,\x22dynamic\x22,\
\x22each\x22,\x22else\x22,\x22e\
xtends\x22,\x22final\x22,\
\x22finally\x22,\x22for\x22,\
\x22function\x22,\x22get\x22\
,\x22if\x22,\x22implement\
s\x22,\x22import\x22,\x22in\x22\
,\x22include\x22,\x22inst\
anceof\x22,\x22interfa\
ce\x22,\x22internal\x22,\x22\
is\x22,\x22namespace\x22,\
\x22native\x22,\x22new\x22,\x22\
override\x22,\x22packa\
ge\x22,\x22private\x22,\x22p\
rotected\x22,\x22publi\
c\x22,\x22return\x22,\x22set\
\x22,\x22static\x22,\x22supe\
r\x22,\x22switch\x22,\x22thi\
s\x22,\x22throw\x22,\x22try\x22\
,\x22typeof\x22,\x22use\x22,\
\x22var\x22,\x22void\x22,\x22wh\
ile\x22,\x22with\x22],\x0ali\
teral:[\x22true\x22,\x22f\
alse\x22,\x22null\x22,\x22un\
defined\x22]},\x0acont\
ains:[e.APOS_STR\
ING_MODE,e.QUOTE\
_STRING_MODE,e.C\
_LINE_COMMENT_MO\
DE,e.C_BLOCK_COM\
MENT_MODE,e.C_NU\
MBER_MODE,{\x0amatc\
h:[/\x5cbpackage/,/\
\x5cs+/,n],classNam\
e:{1:\x22keyword\x22,3\
:\x22title.class\x22}}\
,{\x0amatch:[/\x5cb(?:\
class|interface|\
extends|implemen\
ts)/,/\x5cs+/,t],cl\
assName:{\x0a1:\x22key\
word\x22,3:\x22title.c\
lass\x22}},{classNa\
me:\x22meta\x22,beginK\
eywords:\x22import \
include\x22,\x0aend:/;\
/,keywords:{keyw\
ord:\x22import incl\
ude\x22}},{beginKey\
words:\x22function\x22\
,\x0aend:/[{;]/,exc\
ludeEnd:!0,illeg\
al:/\x5cS/,contains\
:[e.inherit(e.TI\
TLE_MODE,{\x0aclass\
Name:\x22title.func\
tion\x22}),{classNa\
me:\x22params\x22,begi\
n:/\x5c(/,end:/\x5c)/,\
\x0acontains:[e.APO\
S_STRING_MODE,e.\
QUOTE_STRING_MOD\
E,e.C_LINE_COMME\
NT_MODE,e.C_BLOC\
K_COMMENT_MODE,s\
]\x0a},{begin:a.con\
cat(/:\x5cs*/,/([*]\
|[a-zA-Z_$][a-zA\
-Z0-9_$]*)/)}]},\
e.METHOD_GUARD],\
\x0aillegal:/#/}}})\
();hljs.register\
Language(\x22action\
script\x22,e)})();/\
*! `ada` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
e=(()=>{\x22use str\
ict\x22;return e=>{\
\x0aconst n=\x22\x5c\x5cd(_|\
\x5c\x5cd)*\x22,s=\x22[eE][-\
+]?\x22+n,a=\x22\x5c\x5cb(\x22+\
n+\x22#\x5c\x5cw+(\x5c\x5c.\x5c\x5cw+\
)?#(\x22+s+\x22)?|\x22+n+\
\x22(\x5c\x5c.\x22+n+\x22)?(\x22+s\
+\x22)?)\x22,r=\x22[A-Za-\
z](_?[A-Za-z0-9.\
])*\x22,i=\x22[]\x5c\x5c{\x5c\x5c}\
%#'\x5c\x22\x22,t=e.COMME\
NT(\x22--\x22,\x22$\x22),c={\
\x0abegin:\x22\x5c\x5cs+:\x5c\x5cs\
+\x22,end:\x22\x5c\x5cs*(:=|\
;|\x5c\x5c)|=>|$)\x22,ill\
egal:i,contains:\
[{\x0abeginKeywords\
:\x22loop for decla\
re others\x22,endsP\
arent:!0},{class\
Name:\x22keyword\x22,\x0a\
beginKeywords:\x22n\
ot null constant\
 access function\
 procedure in ou\
t aliased except\
ion\x22\x0a},{classNam\
e:\x22type\x22,begin:r\
,endsParent:!0,r\
elevance:0}]};re\
turn{name:\x22Ada\x22,\
\x0acase_insensitiv\
e:!0,keywords:{\x0a\
keyword:[\x22abort\x22\
,\x22else\x22,\x22new\x22,\x22r\
eturn\x22,\x22abs\x22,\x22el\
sif\x22,\x22not\x22,\x22reve\
rse\x22,\x22abstract\x22,\
\x22end\x22,\x22accept\x22,\x22\
entry\x22,\x22select\x22,\
\x22access\x22,\x22except\
ion\x22,\x22of\x22,\x22separ\
ate\x22,\x22aliased\x22,\x22\
exit\x22,\x22or\x22,\x22some\
\x22,\x22all\x22,\x22others\x22\
,\x22subtype\x22,\x22and\x22\
,\x22for\x22,\x22out\x22,\x22sy\
nchronized\x22,\x22arr\
ay\x22,\x22function\x22,\x22\
overriding\x22,\x22at\x22\
,\x22tagged\x22,\x22gener\
ic\x22,\x22package\x22,\x22t\
ask\x22,\x22begin\x22,\x22go\
to\x22,\x22pragma\x22,\x22te\
rminate\x22,\x22body\x22,\
\x22private\x22,\x22then\x22\
,\x22if\x22,\x22procedure\
\x22,\x22type\x22,\x22case\x22,\
\x22in\x22,\x22protected\x22\
,\x22constant\x22,\x22int\
erface\x22,\x22is\x22,\x22ra\
ise\x22,\x22use\x22,\x22decl\
are\x22,\x22range\x22,\x22de\
lay\x22,\x22limited\x22,\x22\
record\x22,\x22when\x22,\x22\
delta\x22,\x22loop\x22,\x22r\
em\x22,\x22while\x22,\x22dig\
its\x22,\x22renames\x22,\x22\
with\x22,\x22do\x22,\x22mod\x22\
,\x22requeue\x22,\x22xor\x22\
],\x0aliteral:[\x22Tru\
e\x22,\x22False\x22]},con\
tains:[t,{classN\
ame:\x22string\x22,beg\
in:/\x22/,end:/\x22/,\x0a\
contains:[{begin\
:/\x22\x22/,relevance:\
0}]},{className:\
\x22string\x22,begin:/\
'.'/},{\x0aclassNam\
e:\x22number\x22,begin\
:a,relevance:0},\
{className:\x22symb\
ol\x22,begin:\x22'\x22+r}\
,{\x0aclassName:\x22ti\
tle\x22,\x0abegin:\x22(\x5c\x5c\
bwith\x5c\x5cs+)?(\x5c\x5cbp\
rivate\x5c\x5cs+)?\x5c\x5cbp\
ackage\x5c\x5cs+(\x5c\x5cbbo\
dy\x5c\x5cs+)?\x22,\x0aend:\x22\
(is|$)\x22,keywords\
:\x22package body\x22,\
excludeBegin:!0,\
excludeEnd:!0,il\
legal:i},{\x0abegin\
:\x22(\x5c\x5cb(with|over\
riding)\x5c\x5cs+)?\x5c\x5cb\
(function|proced\
ure)\x5c\x5cs+\x22,\x0aend:\x22\
(\x5c\x5cbis|\x5c\x5cbwith|\x5c\
\x5cbrenames|\x5c\x5c)\x5c\x5cs\
*;)\x22,\x0akeywords:\x22\
overriding funct\
ion procedure wi\
th is renames re\
turn\x22,returnBegi\
n:!0,\x0acontains:[\
t,{className:\x22ti\
tle\x22,\x0abegin:\x22(\x5c\x5c\
bwith\x5c\x5cs+)?\x5c\x5cb(f\
unction|procedur\
e)\x5c\x5cs+\x22,end:\x22(\x5c\x5c\
(|\x5c\x5cs+|$)\x22,\x0aexcl\
udeBegin:!0,excl\
udeEnd:!0,illega\
l:i},c,{classNam\
e:\x22type\x22,\x0abegin:\
\x22\x5c\x5cbreturn\x5c\x5cs+\x22,\
end:\x22(\x5c\x5cs+|;|$)\x22\
,keywords:\x22retur\
n\x22,excludeBegin:\
!0,\x0aexcludeEnd:!\
0,endsParent:!0,\
illegal:i}]},{cl\
assName:\x22type\x22,\x0a\
begin:\x22\x5c\x5cb(sub)?\
type\x5c\x5cs+\x22,end:\x22\x5c\
\x5cs+\x22,keywords:\x22t\
ype\x22,excludeBegi\
n:!0,illegal:i\x0a}\
,c]}}})();hljs.r\
egisterLanguage(\
\x22ada\x22,e)})();/*!\
 `angelscript` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>{const n={c\
lassName:\x22built_\
in\x22,\x0abegin:\x22\x5c\x5cb(\
void|bool|int8|i\
nt16|int32|int64\
|int|uint8|uint1\
6|uint32|uint64|\
uint|string|ref|\
array|double|flo\
at|auto|dictiona\
ry)\x22\x0a},a={classN\
ame:\x22symbol\x22,beg\
in:\x22[a-zA-Z0-9_]\
+@\x22},i={classNam\
e:\x22keyword\x22,\x0abeg\
in:\x22<\x22,end:\x22>\x22,c\
ontains:[n,a]};r\
eturn n.contains\
=[i],a.contains=\
[i],{\x0aname:\x22Ange\
lScript\x22,aliases\
:[\x22asc\x22],\x0akeywor\
ds:[\x22for\x22,\x22in|0\x22\
,\x22break\x22,\x22contin\
ue\x22,\x22while\x22,\x22do|\
0\x22,\x22return\x22,\x22if\x22\
,\x22else\x22,\x22case\x22,\x22\
switch\x22,\x22namespa\
ce\x22,\x22is\x22,\x22cast\x22,\
\x22or\x22,\x22and\x22,\x22xor\x22\
,\x22not\x22,\x22get|0\x22,\x22\
in\x22,\x22inout|10\x22,\x22\
out\x22,\x22override\x22,\
\x22set|0\x22,\x22private\
\x22,\x22public\x22,\x22cons\
t\x22,\x22default|0\x22,\x22\
final\x22,\x22shared\x22,\
\x22external\x22,\x22mixi\
n|10\x22,\x22enum\x22,\x22ty\
pedef\x22,\x22funcdef\x22\
,\x22this\x22,\x22super\x22,\
\x22import\x22,\x22from\x22,\
\x22interface\x22,\x22abs\
tract|0\x22,\x22try\x22,\x22\
catch\x22,\x22protecte\
d\x22,\x22explicit\x22,\x22p\
roperty\x22],\x0ailleg\
al:\x22(^using\x5c\x5cs+[\
A-Za-z0-9_\x5c\x5c.]+;\
$|\x5c\x5cbfunction\x5c\x5cs\
*[^\x5c\x5c(])\x22,contai\
ns:[{\x0aclassName:\
\x22string\x22,begin:\x22\
'\x22,end:\x22'\x22,illeg\
al:\x22\x5c\x5cn\x22,\x0acontai\
ns:[e.BACKSLASH_\
ESCAPE],relevanc\
e:0},{className:\
\x22string\x22,begin:'\
\x22\x22\x22',\x0aend:'\x22\x22\x22'}\
,{className:\x22str\
ing\x22,begin:'\x22',e\
nd:'\x22',illegal:\x22\
\x5c\x5cn\x22,\x0acontains:[\
e.BACKSLASH_ESCA\
PE],relevance:0\x0a\
},e.C_LINE_COMME\
NT_MODE,e.C_BLOC\
K_COMMENT_MODE,{\
className:\x22strin\
g\x22,\x0abegin:\x22^\x5c\x5cs*\
\x5c\x5c[\x22,end:\x22\x5c\x5c]\x22},\
{beginKeywords:\x22\
interface namesp\
ace\x22,end:/\x5c{/,\x0ai\
llegal:\x22[;.\x5c\x5c-]\x22\
,contains:[{clas\
sName:\x22symbol\x22,b\
egin:\x22[a-zA-Z0-9\
_]+\x22}]},{\x0abeginK\
eywords:\x22class\x22,\
end:/\x5c{/,illegal\
:\x22[;.\x5c\x5c-]\x22,conta\
ins:[{className:\
\x22symbol\x22,\x0abegin:\
\x22[a-zA-Z0-9_]+\x22,\
contains:[{begin\
:\x22[:,]\x5c\x5cs*\x22,cont\
ains:[{className\
:\x22symbol\x22,\x0abegin\
:\x22[a-zA-Z0-9_]+\x22\
}]}]}]},n,a,{cla\
ssName:\x22literal\x22\
,\x0abegin:\x22\x5c\x5cb(nul\
l|true|false)\x22},\
{className:\x22numb\
er\x22,relevance:0,\
\x0abegin:\x22(-?)(\x5c\x5cb\
0[xXbBoOdD][a-fA\
-F0-9]+|(\x5c\x5cb\x5c\x5cd+\
(\x5c\x5c.\x5c\x5cd*)?f?|\x5c\x5c.\
\x5c\x5cd+f?)([eE][-+]\
?\x5c\x5cd+f?)?)\x22\x0a}]}}\
})();hljs.regist\
erLanguage(\x22ange\
lscript\x22,e)})();\
/*! `apache` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var e=(()=>{\x22use\
 strict\x22;return \
e=>{const n={cla\
ssName:\x22number\x22,\
\x0abegin:/\x5cd{1,3}\x5c\
.\x5cd{1,3}\x5c.\x5cd{1,3\
}\x5c.\x5cd{1,3}(:\x5cd{1\
,5})?/};return{\x0a\
name:\x22Apache con\
fig\x22,aliases:[\x22a\
pacheconf\x22],case\
_insensitive:!0,\
\x0acontains:[e.HAS\
H_COMMENT_MODE,{\
className:\x22secti\
on\x22,begin:/<\x5c/?/\
,end:/>/,\x0acontai\
ns:[n,{className\
:\x22number\x22,begin:\
/:\x5cd{1,5}/\x0a},e.i\
nherit(e.QUOTE_S\
TRING_MODE,{rele\
vance:0})]},{cla\
ssName:\x22attribut\
e\x22,\x0abegin:/\x5cw+/,\
relevance:0,keyw\
ords:{\x0a_:[\x22order\
\x22,\x22deny\x22,\x22allow\x22\
,\x22setenv\x22,\x22rewri\
terule\x22,\x22rewrite\
engine\x22,\x22rewrite\
cond\x22,\x22documentr\
oot\x22,\x22sethandler\
\x22,\x22errordocument\
\x22,\x22loadmodule\x22,\x22\
options\x22,\x22header\
\x22,\x22listen\x22,\x22serv\
erroot\x22,\x22servern\
ame\x22]\x0a},starts:{\
end:/$/,relevanc\
e:0,keywords:{li\
teral:\x22on off al\
l deny allow\x22},\x0a\
contains:[{class\
Name:\x22meta\x22,begi\
n:/\x5cs\x5c[/,end:/\x5c]\
$/},{className:\x22\
variable\x22,\x0abegin\
:/[\x5c$%]\x5c{/,end:/\
\x5c}/,contains:[\x22s\
elf\x22,{className:\
\x22number\x22,begin:/\
[$%]\x5cd+/}]\x0a},n,{\
className:\x22numbe\
r\x22,begin:/\x5cb\x5cd+/\
},e.QUOTE_STRING\
_MODE]}}],illega\
l:/\x5cS/}}\x0a})();hl\
js.registerLangu\
age(\x22apache\x22,e)}\
)();/*! `applesc\
ript` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>{\x0aco\
nst t=e.regex,r=\
e.inherit(e.QUOT\
E_STRING_MODE,{i\
llegal:null}),i=\
{\x0aclassName:\x22par\
ams\x22,begin:/\x5c(/,\
end:/\x5c)/,contain\
s:[\x22self\x22,e.C_NU\
MBER_MODE,r]\x0a},n\
=e.COMMENT(/--/,\
/$/),a=[n,e.COMM\
ENT(/\x5c(\x5c*/,/\x5c*\x5c)\
/,{contains:[\x22se\
lf\x22,n]\x0a}),e.HASH\
_COMMENT_MODE];r\
eturn{name:\x22Appl\
eScript\x22,aliases\
:[\x22osascript\x22],\x0a\
keywords:{\x0akeywo\
rd:\x22about above \
after against an\
d around as at b\
ack before begin\
ning behind belo\
w beneath beside\
 between but by \
considering cont\
ain contains con\
tinue copy div d\
oes eighth else \
end equal equals\
 error every exi\
t fifth first fo\
r fourth from fr\
ont get given gl\
obal if ignoring\
 in into is it i\
ts last local me\
 middle mod my n\
inth not of on o\
nto or over prop\
 property put re\
f reference repe\
at returning scr\
ipt second set s\
eventh since six\
th some tell ten\
th that the|0 th\
en third through\
 thru timeout ti\
mes to transacti\
on try until whe\
re while whose w\
ith without\x22,\x0ali\
teral:\x22AppleScri\
pt false linefee\
d return pi quot\
e result space t\
ab true\x22,\x0abuilt_\
in:\x22alias applic\
ation boolean cl\
ass constant dat\
e file integer l\
ist number real \
record string te\
xt activate beep\
 count delay lau\
nch log offset r\
ead round run sa\
y summarize writ\
e character char\
acters contents \
day frontmost id\
 item length mon\
th name|0 paragr\
aph paragraphs r\
est reverse runn\
ing time version\
 weekday word wo\
rds year\x22\x0a},cont\
ains:[r,e.C_NUMB\
ER_MODE,{classNa\
me:\x22built_in\x22,\x0ab\
egin:t.concat(/\x5c\
b/,t.either(/cli\
pboard info/,/th\
e clipboard/,/in\
fo for/,/list (d\
isks|folder)/,/m\
ount volume/,/pa\
th to/,/(close|o\
pen for) access/\
,/(get|set) eof/\
,/current date/,\
/do shell script\
/,/get volume se\
ttings/,/random \
number/,/set vol\
ume/,/system att\
ribute/,/system \
info/,/time to G\
MT/,/(load|run|s\
tore) script/,/s\
cripting compone\
nts/,/ASCII (cha\
racter|number)/,\
/localized strin\
g/,/choose (appl\
ication|color|fi\
le|file name|fol\
der|from list|re\
mote application\
|URL)/,/display \
(alert|dialog)/)\
,/\x5cb/)\x0a},{classN\
ame:\x22built_in\x22,b\
egin:/^\x5cs*return\
\x5cb/},{className:\
\x22literal\x22,\x0abegin\
:/\x5cb(text item d\
elimiters|curren\
t application|mi\
ssing value)\x5cb/}\
,{\x0aclassName:\x22ke\
yword\x22,\x0abegin:t.\
concat(/\x5cb/,t.ei\
ther(/apart from\
/,/aside from/,/\
instead of/,/out\
 of/,/greater th\
an/,/isn't|(does\
n't|does not) (e\
qual|come before\
|come after|cont\
ain)/,/(greater|\
less) than( or e\
qual)?/,/(starts\
?|ends|begins?) \
with/,/contained\
 by/,/comes (bef\
ore|after)/,/a (\
ref|reference)/,\
/POSIX (file|pat\
h)/,/(date|time)\
 string/,/quoted\
 form/),/\x5cb/)\x0a},\
{beginKeywords:\x22\
on\x22,illegal:/[${\
=;\x5cn]/,contains:\
[e.UNDERSCORE_TI\
TLE_MODE,i]\x0a},..\
.a],illegal:/\x5c/\x5c\
/|->|=>|\x5c[\x5c[/}}}\
)();hljs.registe\
rLanguage(\x22apple\
script\x22,e)\x0a})();\
/*! `arcade` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var e=(()=>{\x22use\
 strict\x22;return \
e=>{const t=\x22[A-\
Za-z_][0-9A-Za-z\
_]*\x22,n={\x0akeyword\
:[\x22break\x22,\x22case\x22\
,\x22catch\x22,\x22contin\
ue\x22,\x22debugger\x22,\x22\
do\x22,\x22else\x22,\x22expo\
rt\x22,\x22for\x22,\x22funct\
ion\x22,\x22if\x22,\x22impor\
t\x22,\x22in\x22,\x22new\x22,\x22r\
eturn\x22,\x22switch\x22,\
\x22try\x22,\x22var\x22,\x22voi\
d\x22,\x22while\x22],\x0alit\
eral:[\x22BackSlash\
\x22,\x22DoubleQuote\x22,\
\x22ForwardSlash\x22,\x22\
Infinity\x22,\x22NaN\x22,\
\x22NewLine\x22,\x22PI\x22,\x22\
SingleQuote\x22,\x22Ta\
b\x22,\x22TextFormatti\
ng\x22,\x22false\x22,\x22nul\
l\x22,\x22true\x22,\x22undef\
ined\x22],\x0abuilt_in\
:[\x22Abs\x22,\x22Acos\x22,\x22\
All\x22,\x22Angle\x22,\x22An\
y\x22,\x22Area\x22,\x22AreaG\
eodetic\x22,\x22Array\x22\
,\x22Asin\x22,\x22Atan\x22,\x22\
Atan2\x22,\x22Attachme\
nts\x22,\x22Average\x22,\x22\
Back\x22,\x22Bearing\x22,\
\x22Boolean\x22,\x22Buffe\
r\x22,\x22BufferGeodet\
ic\x22,\x22Ceil\x22,\x22Cent\
roid\x22,\x22ChangeTim\
eZone\x22,\x22Clip\x22,\x22C\
oncatenate\x22,\x22Con\
sole\x22,\x22Constrain\
\x22,\x22Contains\x22,\x22Co\
nvertDirection\x22,\
\x22ConvexHull\x22,\x22Co\
s\x22,\x22Count\x22,\x22Cros\
ses\x22,\x22Cut\x22,\x22Date\
|0\x22,\x22DateAdd\x22,\x22D\
ateDiff\x22,\x22DateOn\
ly\x22,\x22Day\x22,\x22Decod\
e\x22,\x22DefaultValue\
\x22,\x22Densify\x22,\x22Den\
sifyGeodetic\x22,\x22D\
ictionary\x22,\x22Diff\
erence\x22,\x22Disjoin\
t\x22,\x22Distance\x22,\x22D\
istanceGeodetic\x22\
,\x22Distinct\x22,\x22Dom\
ain\x22,\x22DomainCode\
\x22,\x22DomainName\x22,\x22\
EnvelopeIntersec\
ts\x22,\x22Equals\x22,\x22Er\
ase\x22,\x22Exp\x22,\x22Expe\
cts\x22,\x22Extent\x22,\x22F\
eature\x22,\x22Feature\
Set\x22,\x22FeatureSet\
ByAssociation\x22,\x22\
FeatureSetById\x22,\
\x22FeatureSetByNam\
e\x22,\x22FeatureSetBy\
PortalItem\x22,\x22Fea\
tureSetByRelatio\
nshipClass\x22,\x22Fea\
tureSetByRelatio\
nshipName\x22,\x22Filt\
er\x22,\x22Find\x22,\x22Firs\
t|0\x22,\x22Floor\x22,\x22Fr\
omCharCode\x22,\x22Fro\
mCodePoint\x22,\x22Fro\
mJSON\x22,\x22Front\x22,\x22\
GdbVersion\x22,\x22Gen\
eralize\x22,\x22Geomet\
ry\x22,\x22GetEnvironm\
ent\x22,\x22GetFeature\
Set\x22,\x22GetFeature\
SetInfo\x22,\x22GetUse\
r\x22,\x22GroupBy\x22,\x22Gu\
id\x22,\x22HasKey\x22,\x22Ha\
sValue\x22,\x22Hash\x22,\x22\
Hour\x22,\x22IIf\x22,\x22ISO\
Month\x22,\x22ISOWeek\x22\
,\x22ISOWeekday\x22,\x22I\
SOYear\x22,\x22Include\
s\x22,\x22IndexOf\x22,\x22In\
sert\x22,\x22Intersect\
ion\x22,\x22Intersects\
\x22,\x22IsEmpty\x22,\x22IsN\
an\x22,\x22IsSelfInter\
secting\x22,\x22IsSimp\
le\x22,\x22Left|0\x22,\x22Le\
ngth\x22,\x22Length3D\x22\
,\x22LengthGeodetic\
\x22,\x22Log\x22,\x22Lower\x22,\
\x22Map\x22,\x22Max\x22,\x22Mea\
n\x22,\x22Mid\x22,\x22Millis\
econd\x22,\x22Min\x22,\x22Mi\
nute\x22,\x22Month\x22,\x22M\
ultiPartToSingle\
Part\x22,\x22Multipoin\
t\x22,\x22NearestCoord\
inate\x22,\x22NearestV\
ertex\x22,\x22NextSequ\
enceValue\x22,\x22None\
\x22,\x22Now\x22,\x22Number\x22\
,\x22Offset\x22,\x22Order\
By\x22,\x22Overlaps\x22,\x22\
Point\x22,\x22Polygon\x22\
,\x22Polyline\x22,\x22Pop\
\x22,\x22Portal\x22,\x22Pow\x22\
,\x22Proper\x22,\x22Push\x22\
,\x22Random\x22,\x22Reduc\
e\x22,\x22Relate\x22,\x22Rep\
lace\x22,\x22Resize\x22,\x22\
Reverse\x22,\x22Right|\
0\x22,\x22RingIsClockw\
ise\x22,\x22Rotate\x22,\x22R\
ound\x22,\x22Schema\x22,\x22\
Second\x22,\x22SetGeom\
etry\x22,\x22Simplify\x22\
,\x22Sin\x22,\x22Slice\x22,\x22\
Sort\x22,\x22Splice\x22,\x22\
Split\x22,\x22Sqrt\x22,\x22S\
tandardizeGuid\x22,\
\x22Stdev\x22,\x22Subtype\
Code\x22,\x22SubtypeNa\
me\x22,\x22Subtypes\x22,\x22\
Sum\x22,\x22SymmetricD\
ifference\x22,\x22Tan\x22\
,\x22Text\x22,\x22Time\x22,\x22\
TimeZone\x22,\x22TimeZ\
oneOffset\x22,\x22Time\
stamp\x22,\x22ToCharCo\
de\x22,\x22ToCodePoint\
\x22,\x22ToHex\x22,\x22ToLoc\
al\x22,\x22ToUTC\x22,\x22Tod\
ay\x22,\x22Top|0\x22,\x22Tou\
ches\x22,\x22TrackAcce\
lerationAt\x22,\x22Tra\
ckAccelerationWi\
ndow\x22,\x22TrackCurr\
entAcceleration\x22\
,\x22TrackCurrentDi\
stance\x22,\x22TrackCu\
rrentSpeed\x22,\x22Tra\
ckCurrentTime\x22,\x22\
TrackDistanceAt\x22\
,\x22TrackDistanceW\
indow\x22,\x22TrackDur\
ation\x22,\x22TrackFie\
ldWindow\x22,\x22Track\
GeometryWindow\x22,\
\x22TrackIndex\x22,\x22Tr\
ackSpeedAt\x22,\x22Tra\
ckSpeedWindow\x22,\x22\
TrackStartTime\x22,\
\x22TrackWindow\x22,\x22T\
rim\x22,\x22TypeOf\x22,\x22U\
nion\x22,\x22Upper\x22,\x22U\
rlEncode\x22,\x22Varia\
nce\x22,\x22Week\x22,\x22Wee\
kday\x22,\x22When|0\x22,\x22\
Within\x22,\x22Year|0\x22\
]\x0a},a={className\
:\x22symbol\x22,\x0abegin\
:\x22\x5c\x5c$\x22+e.regex.e\
ither(\x22aggregate\
dFeatures\x22,\x22anal\
ytic\x22,\x22config\x22,\x22\
datapoint\x22,\x22data\
store\x22,\x22editcont\
ext\x22,\x22feature\x22,\x22\
featureSet\x22,\x22fee\
dfeature\x22,\x22fence\
feature\x22,\x22fencen\
otificationtype\x22\
,\x22join\x22,\x22layer\x22,\
\x22locationupdate\x22\
,\x22map\x22,\x22measure\x22\
,\x22measure\x22,\x22orig\
inalFeature\x22,\x22re\
cord\x22,\x22reference\
\x22,\x22rowindex\x22,\x22so\
urcedatastore\x22,\x22\
sourcefeature\x22,\x22\
sourcelayer\x22,\x22ta\
rget\x22,\x22targetdat\
astore\x22,\x22targetf\
eature\x22,\x22targetl\
ayer\x22,\x22value\x22,\x22v\
iew\x22)\x0a},r={class\
Name:\x22number\x22,va\
riants:[{begin:\x22\
\x5c\x5cb(0[bB][01]+)\x22\
},{\x0abegin:\x22\x5c\x5cb(0\
[oO][0-7]+)\x22},{b\
egin:e.C_NUMBER_\
RE}],relevance:0\
},i={\x0aclassName:\
\x22subst\x22,begin:\x22\x5c\
\x5c$\x5c\x5c{\x22,end:\x22\x5c\x5c}\x22\
,keywords:n,cont\
ains:[]},o={\x0acla\
ssName:\x22string\x22,\
begin:\x22`\x22,end:\x22`\
\x22,contains:[e.BA\
CKSLASH_ESCAPE,i\
]}\x0a;i.contains=[\
e.APOS_STRING_MO\
DE,e.QUOTE_STRIN\
G_MODE,o,r,e.REG\
EXP_MODE]\x0a;const\
 s=i.contains.co\
ncat([e.C_BLOCK_\
COMMENT_MODE,e.C\
_LINE_COMMENT_MO\
DE])\x0a;return{nam\
e:\x22ArcGIS Arcade\
\x22,case_insensiti\
ve:!0,keywords:n\
,\x0acontains:[e.AP\
OS_STRING_MODE,e\
.QUOTE_STRING_MO\
DE,o,e.C_LINE_CO\
MMENT_MODE,e.C_B\
LOCK_COMMENT_MOD\
E,a,r,{\x0abegin:/[\
{,]\x5cs*/,relevanc\
e:0,contains:[{b\
egin:t+\x22\x5c\x5cs*:\x22,r\
eturnBegin:!0,\x0ar\
elevance:0,conta\
ins:[{className:\
\x22attr\x22,begin:t,r\
elevance:0}]}]},\
{\x0abegin:\x22(\x22+e.RE\
_STARTERS_RE+\x22|\x5c\
\x5cb(return)\x5c\x5cb)\x5c\x5c\
s*\x22,keywords:\x22re\
turn\x22,\x0acontains:\
[e.C_LINE_COMMEN\
T_MODE,e.C_BLOCK\
_COMMENT_MODE,e.\
REGEXP_MODE,{\x0acl\
assName:\x22functio\
n\x22,begin:\x22(\x5c\x5c(.*\
?\x5c\x5c)|\x22+t+\x22)\x5c\x5cs*=\
>\x22,returnBegin:!\
0,\x0aend:\x22\x5c\x5cs*=>\x22,\
contains:[{class\
Name:\x22params\x22,va\
riants:[{begin:t\
},{begin:/\x5c(\x5cs*\x5c\
)/\x0a},{begin:/\x5c(/\
,end:/\x5c)/,exclud\
eBegin:!0,exclud\
eEnd:!0,keywords\
:n,contains:s}]}\
]\x0a}],relevance:0\
},{beginKeywords\
:\x22function\x22,end:\
/\x5c{/,excludeEnd:\
!0,\x0acontains:[e.\
inherit(e.TITLE_\
MODE,{className:\
\x22title.function\x22\
,begin:t}),{\x0acla\
ssName:\x22params\x22,\
begin:/\x5c(/,end:/\
\x5c)/,excludeBegin\
:!0,excludeEnd:!\
0,contains:s\x0a}],\
illegal:/\x5c[|%/},\
{begin:/\x5c$[(.]/}\
],illegal:/#(?!!\
)/}}})()\x0a;hljs.r\
egisterLanguage(\
\x22arcade\x22,e)})();\
/*! `arduino` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>{const t={\x0at\
ype:[\x22boolean\x22,\x22\
byte\x22,\x22word\x22,\x22St\
ring\x22],\x0abuilt_in\
:[\x22KeyboardContr\
oller\x22,\x22MouseCon\
troller\x22,\x22Softwa\
reSerial\x22,\x22Ether\
netServer\x22,\x22Ethe\
rnetClient\x22,\x22Liq\
uidCrystal\x22,\x22Rob\
otControl\x22,\x22GSMV\
oiceCall\x22,\x22Ether\
netUDP\x22,\x22Esplora\
TFT\x22,\x22HttpClient\
\x22,\x22RobotMotor\x22,\x22\
WiFiClient\x22,\x22GSM\
Scanner\x22,\x22FileSy\
stem\x22,\x22Scheduler\
\x22,\x22GSMServer\x22,\x22Y\
unClient\x22,\x22YunSe\
rver\x22,\x22IPAddress\
\x22,\x22GSMClient\x22,\x22G\
SMModem\x22,\x22Keyboa\
rd\x22,\x22Ethernet\x22,\x22\
Console\x22,\x22GSMBan\
d\x22,\x22Esplora\x22,\x22St\
epper\x22,\x22Process\x22\
,\x22WiFiUDP\x22,\x22GSM_\
SMS\x22,\x22Mailbox\x22,\x22\
USBHost\x22,\x22Firmat\
a\x22,\x22PImage\x22,\x22Cli\
ent\x22,\x22Server\x22,\x22G\
SMPIN\x22,\x22FileIO\x22,\
\x22Bridge\x22,\x22Serial\
\x22,\x22EEPROM\x22,\x22Stre\
am\x22,\x22Mouse\x22,\x22Aud\
io\x22,\x22Servo\x22,\x22Fil\
e\x22,\x22Task\x22,\x22GPRS\x22\
,\x22WiFi\x22,\x22Wire\x22,\x22\
TFT\x22,\x22GSM\x22,\x22SPI\x22\
,\x22SD\x22],\x0a_hints:[\
\x22setup\x22,\x22loop\x22,\x22\
runShellCommandA\
synchronously\x22,\x22\
analogWriteResol\
ution\x22,\x22retrieve\
CallingNumber\x22,\x22\
printFirmwareVer\
sion\x22,\x22analogRea\
dResolution\x22,\x22se\
ndDigitalPortPai\
r\x22,\x22noListenOnLo\
calhost\x22,\x22readJo\
ystickButton\x22,\x22s\
etFirmwareVersio\
n\x22,\x22readJoystick\
Switch\x22,\x22scrollD\
isplayRight\x22,\x22ge\
tVoiceCallStatus\
\x22,\x22scrollDisplay\
Left\x22,\x22writeMicr\
oseconds\x22,\x22delay\
Microseconds\x22,\x22b\
eginTransmission\
\x22,\x22getSignalStre\
ngth\x22,\x22runAsynch\
ronously\x22,\x22getAs\
ynchronously\x22,\x22l\
istenOnLocalhost\
\x22,\x22getCurrentCar\
rier\x22,\x22readAccel\
erometer\x22,\x22messa\
geAvailable\x22,\x22se\
ndDigitalPorts\x22,\
\x22lineFollowConfi\
g\x22,\x22countryNameW\
rite\x22,\x22runShellC\
ommand\x22,\x22readStr\
ingUntil\x22,\x22rewin\
dDirectory\x22,\x22rea\
dTemperature\x22,\x22s\
etClockDivider\x22,\
\x22readLightSensor\
\x22,\x22endTransmissi\
on\x22,\x22analogRefer\
ence\x22,\x22detachInt\
errupt\x22,\x22country\
NameRead\x22,\x22attac\
hInterrupt\x22,\x22enc\
ryptionType\x22,\x22re\
adBytesUntil\x22,\x22r\
obotNameWrite\x22,\x22\
readMicrophone\x22,\
\x22robotNameRead\x22,\
\x22cityNameWrite\x22,\
\x22userNameWrite\x22,\
\x22readJoystickY\x22,\
\x22readJoystickX\x22,\
\x22mouseReleased\x22,\
\x22openNextFile\x22,\x22\
scanNetworks\x22,\x22n\
oInterrupts\x22,\x22di\
gitalWrite\x22,\x22beg\
inSpeaker\x22,\x22mous\
ePressed\x22,\x22isAct\
ionDone\x22,\x22mouseD\
ragged\x22,\x22display\
Logos\x22,\x22noAutosc\
roll\x22,\x22addParame\
ter\x22,\x22remoteNumb\
er\x22,\x22getModifier\
s\x22,\x22keyboardRead\
\x22,\x22userNameRead\x22\
,\x22waitContinue\x22,\
\x22processInput\x22,\x22\
parseCommand\x22,\x22p\
rintVersion\x22,\x22re\
adNetworks\x22,\x22wri\
teMessage\x22,\x22blin\
kVersion\x22,\x22cityN\
ameRead\x22,\x22readMe\
ssage\x22,\x22setDataM\
ode\x22,\x22parsePacke\
t\x22,\x22isListening\x22\
,\x22setBitOrder\x22,\x22\
beginPacket\x22,\x22is\
Directory\x22,\x22moto\
rsWrite\x22,\x22drawCo\
mpass\x22,\x22digitalR\
ead\x22,\x22clearScree\
n\x22,\x22serialEvent\x22\
,\x22rightToLeft\x22,\x22\
setTextSize\x22,\x22le\
ftToRight\x22,\x22requ\
estFrom\x22,\x22keyRel\
eased\x22,\x22compassR\
ead\x22,\x22analogWrit\
e\x22,\x22interrupts\x22,\
\x22WiFiServer\x22,\x22di\
sconnect\x22,\x22playM\
elody\x22,\x22parseFlo\
at\x22,\x22autoscroll\x22\
,\x22getPINUsed\x22,\x22s\
etPINUsed\x22,\x22setT\
imeout\x22,\x22sendAna\
log\x22,\x22readSlider\
\x22,\x22analogRead\x22,\x22\
beginWrite\x22,\x22cre\
ateChar\x22,\x22motors\
Stop\x22,\x22keyPresse\
d\x22,\x22tempoWrite\x22,\
\x22readButton\x22,\x22su\
bnetMask\x22,\x22debug\
Print\x22,\x22macAddre\
ss\x22,\x22writeGreen\x22\
,\x22randomSeed\x22,\x22a\
ttachGPRS\x22,\x22read\
String\x22,\x22sendStr\
ing\x22,\x22remotePort\
\x22,\x22releaseAll\x22,\x22\
mouseMoved\x22,\x22bac\
kground\x22,\x22getXCh\
ange\x22,\x22getYChang\
e\x22,\x22answerCall\x22,\
\x22getResult\x22,\x22voi\
ceCall\x22,\x22endPack\
et\x22,\x22constrain\x22,\
\x22getSocket\x22,\x22wri\
teJSON\x22,\x22getButt\
on\x22,\x22available\x22,\
\x22connected\x22,\x22fin\
dUntil\x22,\x22readByt\
es\x22,\x22exitValue\x22,\
\x22readGreen\x22,\x22wri\
teBlue\x22,\x22startLo\
op\x22,\x22IPAddress\x22,\
\x22isPressed\x22,\x22sen\
dSysex\x22,\x22pauseMo\
de\x22,\x22gatewayIP\x22,\
\x22setCursor\x22,\x22get\
OemKey\x22,\x22tuneWri\
te\x22,\x22noDisplay\x22,\
\x22loadImage\x22,\x22swi\
tchPIN\x22,\x22onReque\
st\x22,\x22onReceive\x22,\
\x22changePIN\x22,\x22pla\
yFile\x22,\x22noBuffer\
\x22,\x22parseInt\x22,\x22ov\
erflow\x22,\x22checkPI\
N\x22,\x22knobRead\x22,\x22b\
eginTFT\x22,\x22bitCle\
ar\x22,\x22updateIR\x22,\x22\
bitWrite\x22,\x22posit\
ion\x22,\x22writeRGB\x22,\
\x22highByte\x22,\x22writ\
eRed\x22,\x22setSpeed\x22\
,\x22readBlue\x22,\x22noS\
troke\x22,\x22remoteIP\
\x22,\x22transfer\x22,\x22sh\
utdown\x22,\x22hangCal\
l\x22,\x22beginSMS\x22,\x22e\
ndWrite\x22,\x22attach\
ed\x22,\x22maintain\x22,\x22\
noCursor\x22,\x22check\
Reg\x22,\x22checkPUK\x22,\
\x22shiftOut\x22,\x22isVa\
lid\x22,\x22shiftIn\x22,\x22\
pulseIn\x22,\x22connec\
t\x22,\x22println\x22,\x22lo\
calIP\x22,\x22pinMode\x22\
,\x22getIMEI\x22,\x22disp\
lay\x22,\x22noBlink\x22,\x22\
process\x22,\x22getBan\
d\x22,\x22running\x22,\x22be\
ginSD\x22,\x22drawBMP\x22\
,\x22lowByte\x22,\x22setB\
and\x22,\x22release\x22,\x22\
bitRead\x22,\x22prepar\
e\x22,\x22pointTo\x22,\x22re\
adRed\x22,\x22setMode\x22\
,\x22noFill\x22,\x22remov\
e\x22,\x22listen\x22,\x22str\
oke\x22,\x22detach\x22,\x22a\
ttach\x22,\x22noTone\x22,\
\x22exists\x22,\x22buffer\
\x22,\x22height\x22,\x22bitS\
et\x22,\x22circle\x22,\x22co\
nfig\x22,\x22cursor\x22,\x22\
random\x22,\x22IRread\x22\
,\x22setDNS\x22,\x22endSM\
S\x22,\x22getKey\x22,\x22mic\
ros\x22,\x22millis\x22,\x22b\
egin\x22,\x22print\x22,\x22w\
rite\x22,\x22ready\x22,\x22f\
lush\x22,\x22width\x22,\x22i\
sPIN\x22,\x22blink\x22,\x22c\
lear\x22,\x22press\x22,\x22m\
kdir\x22,\x22rmdir\x22,\x22c\
lose\x22,\x22point\x22,\x22y\
ield\x22,\x22image\x22,\x22B\
SSID\x22,\x22click\x22,\x22d\
elay\x22,\x22read\x22,\x22te\
xt\x22,\x22move\x22,\x22peek\
\x22,\x22beep\x22,\x22rect\x22,\
\x22line\x22,\x22open\x22,\x22s\
eek\x22,\x22fill\x22,\x22siz\
e\x22,\x22turn\x22,\x22stop\x22\
,\x22home\x22,\x22find\x22,\x22\
step\x22,\x22tone\x22,\x22sq\
rt\x22,\x22RSSI\x22,\x22SSID\
\x22,\x22end\x22,\x22bit\x22,\x22t\
an\x22,\x22cos\x22,\x22sin\x22,\
\x22pow\x22,\x22map\x22,\x22abs\
\x22,\x22max\x22,\x22min\x22,\x22g\
et\x22,\x22run\x22,\x22put\x22]\
,\x0aliteral:[\x22DIGI\
TAL_MESSAGE\x22,\x22FI\
RMATA_STRING\x22,\x22A\
NALOG_MESSAGE\x22,\x22\
REPORT_DIGITAL\x22,\
\x22REPORT_ANALOG\x22,\
\x22INPUT_PULLUP\x22,\x22\
SET_PIN_MODE\x22,\x22I\
NTERNAL2V56\x22,\x22SY\
STEM_RESET\x22,\x22LED\
_BUILTIN\x22,\x22INTER\
NAL1V1\x22,\x22SYSEX_S\
TART\x22,\x22INTERNAL\x22\
,\x22EXTERNAL\x22,\x22DEF\
AULT\x22,\x22OUTPUT\x22,\x22\
INPUT\x22,\x22HIGH\x22,\x22L\
OW\x22]\x0a},r=(e=>{co\
nst t=e.regex,r=\
e.COMMENT(\x22//\x22,\x22\
$\x22,{contains:[{b\
egin:/\x5c\x5c\x5cn/}]\x0a})\
,a=\x22decltype\x5c\x5c(a\
uto\x5c\x5c)\x22,n=\x22[a-zA\
-Z_]\x5c\x5cw*::\x22,i=\x22(\
?!struct)(\x22+a+\x22|\
\x22+t.optional(n)+\
\x22[a-zA-Z_]\x5c\x5cw*\x22+\
t.optional(\x22<[^<\
>]+>\x22)+\x22)\x22,s={\x0ac\
lassName:\x22type\x22,\
begin:\x22\x5c\x5cb[a-z\x5c\x5c\
d_]*_t\x5c\x5cb\x22},o={c\
lassName:\x22string\
\x22,variants:[{\x0abe\
gin:'(u8?|U|L)?\x22\
',end:'\x22',illega\
l:\x22\x5c\x5cn\x22,contains\
:[e.BACKSLASH_ES\
CAPE]},{\x0abegin:\x22\
(u8?|U|L)?'(\x5c\x5c\x5c\x5c\
(x[0-9A-Fa-f]{2}\
|u[0-9A-Fa-f]{4,\
8}|[0-7]{3}|\x5c\x5cS)\
|.)\x22,\x0aend:\x22'\x22,il\
legal:\x22.\x22},e.END\
_SAME_AS_BEGIN({\
\x0abegin:/(?:u8?|U\
|L)?R\x22([^()\x5c\x5c ]{\
0,16})\x5c(/,end:/\x5c\
)([^()\x5c\x5c ]{0,16}\
)\x22/})]},l={\x0aclas\
sName:\x22number\x22,v\
ariants:[{\x0abegin\
:\x22[+-]?(?:(?:[0-\
9](?:'?[0-9])*\x5c\x5c\
.(?:[0-9](?:'?[0\
-9])*)?|\x5c\x5c.[0-9]\
(?:'?[0-9])*)(?:\
[Ee][+-]?[0-9](?\
:'?[0-9])*)?|[0-\
9](?:'?[0-9])*[E\
e][+-]?[0-9](?:'\
?[0-9])*|0[Xx](?\
:[0-9A-Fa-f](?:'\
?[0-9A-Fa-f])*(?\
:\x5c\x5c.(?:[0-9A-Fa-\
f](?:'?[0-9A-Fa-\
f])*)?)?|\x5c\x5c.[0-9\
A-Fa-f](?:'?[0-9\
A-Fa-f])*)[Pp][+\
-]?[0-9](?:'?[0-\
9])*)(?:[Ff](?:1\
6|32|64|128)?|(B\
F|bf)16|[Ll]|)\x22\x0a\
},{\x0abegin:\x22[+-]?\
\x5c\x5cb(?:0[Bb][01](\
?:'?[01])*|0[Xx]\
[0-9A-Fa-f](?:'?\
[0-9A-Fa-f])*|0(\
?:'?[0-7])*|[1-9\
](?:'?[0-9])*)(?\
:[Uu](?:LL?|ll?)\
|[Uu][Zz]?|(?:LL\
?|ll?)[Uu]?|[Zz]\
[Uu]|)\x22\x0a}],relev\
ance:0},c={class\
Name:\x22meta\x22,begi\
n:/#\x5cs*[a-z]+\x5cb/\
,end:/$/,keyword\
s:{\x0akeyword:\x22if \
else elif endif \
define undef war\
ning error line \
pragma _Pragma i\
fdef ifndef incl\
ude\x22\x0a},contains:\
[{begin:/\x5c\x5c\x5cn/,r\
elevance:0},e.in\
herit(o,{classNa\
me:\x22string\x22}),{\x0a\
className:\x22strin\
g\x22,begin:/<.*?>/\
},r,e.C_BLOCK_CO\
MMENT_MODE]},d={\
\x0aclassName:\x22titl\
e\x22,begin:t.optio\
nal(n)+e.IDENT_R\
E,relevance:0\x0a},\
u=t.optional(n)+\
e.IDENT_RE+\x22\x5c\x5cs*\
\x5c\x5c(\x22,p={\x0atype:[\x22\
bool\x22,\x22char\x22,\x22ch\
ar16_t\x22,\x22char32_\
t\x22,\x22char8_t\x22,\x22do\
uble\x22,\x22float\x22,\x22i\
nt\x22,\x22long\x22,\x22shor\
t\x22,\x22void\x22,\x22wchar\
_t\x22,\x22unsigned\x22,\x22\
signed\x22,\x22const\x22,\
\x22static\x22],\x0akeywo\
rd:[\x22alignas\x22,\x22a\
lignof\x22,\x22and\x22,\x22a\
nd_eq\x22,\x22asm\x22,\x22at\
omic_cancel\x22,\x22at\
omic_commit\x22,\x22at\
omic_noexcept\x22,\x22\
auto\x22,\x22bitand\x22,\x22\
bitor\x22,\x22break\x22,\x22\
case\x22,\x22catch\x22,\x22c\
lass\x22,\x22co_await\x22\
,\x22co_return\x22,\x22co\
_yield\x22,\x22compl\x22,\
\x22concept\x22,\x22const\
_cast|10\x22,\x22const\
eval\x22,\x22constexpr\
\x22,\x22constinit\x22,\x22c\
ontinue\x22,\x22declty\
pe\x22,\x22default\x22,\x22d\
elete\x22,\x22do\x22,\x22dyn\
amic_cast|10\x22,\x22e\
lse\x22,\x22enum\x22,\x22exp\
licit\x22,\x22export\x22,\
\x22extern\x22,\x22false\x22\
,\x22final\x22,\x22for\x22,\x22\
friend\x22,\x22goto\x22,\x22\
if\x22,\x22import\x22,\x22in\
line\x22,\x22module\x22,\x22\
mutable\x22,\x22namesp\
ace\x22,\x22new\x22,\x22noex\
cept\x22,\x22not\x22,\x22not\
_eq\x22,\x22nullptr\x22,\x22\
operator\x22,\x22or\x22,\x22\
or_eq\x22,\x22override\
\x22,\x22private\x22,\x22pro\
tected\x22,\x22public\x22\
,\x22reflexpr\x22,\x22reg\
ister\x22,\x22reinterp\
ret_cast|10\x22,\x22re\
quires\x22,\x22return\x22\
,\x22sizeof\x22,\x22stati\
c_assert\x22,\x22stati\
c_cast|10\x22,\x22stru\
ct\x22,\x22switch\x22,\x22sy\
nchronized\x22,\x22tem\
plate\x22,\x22this\x22,\x22t\
hread_local\x22,\x22th\
row\x22,\x22transactio\
n_safe\x22,\x22transac\
tion_safe_dynami\
c\x22,\x22true\x22,\x22try\x22,\
\x22typedef\x22,\x22typei\
d\x22,\x22typename\x22,\x22u\
nion\x22,\x22using\x22,\x22v\
irtual\x22,\x22volatil\
e\x22,\x22while\x22,\x22xor\x22\
,\x22xor_eq\x22],\x0alite\
ral:[\x22NULL\x22,\x22fal\
se\x22,\x22nullopt\x22,\x22n\
ullptr\x22,\x22true\x22],\
built_in:[\x22_Prag\
ma\x22],\x0a_type_hint\
s:[\x22any\x22,\x22auto_p\
tr\x22,\x22barrier\x22,\x22b\
inary_semaphore\x22\
,\x22bitset\x22,\x22compl\
ex\x22,\x22condition_v\
ariable\x22,\x22condit\
ion_variable_any\
\x22,\x22counting_sema\
phore\x22,\x22deque\x22,\x22\
false_type\x22,\x22fut\
ure\x22,\x22imaginary\x22\
,\x22initializer_li\
st\x22,\x22istringstre\
am\x22,\x22jthread\x22,\x22l\
atch\x22,\x22lock_guar\
d\x22,\x22multimap\x22,\x22m\
ultiset\x22,\x22mutex\x22\
,\x22optional\x22,\x22ost\
ringstream\x22,\x22pac\
kaged_task\x22,\x22pai\
r\x22,\x22promise\x22,\x22pr\
iority_queue\x22,\x22q\
ueue\x22,\x22recursive\
_mutex\x22,\x22recursi\
ve_timed_mutex\x22,\
\x22scoped_lock\x22,\x22s\
et\x22,\x22shared_futu\
re\x22,\x22shared_lock\
\x22,\x22shared_mutex\x22\
,\x22shared_timed_m\
utex\x22,\x22shared_pt\
r\x22,\x22stack\x22,\x22stri\
ng_view\x22,\x22string\
stream\x22,\x22timed_m\
utex\x22,\x22thread\x22,\x22\
true_type\x22,\x22tupl\
e\x22,\x22unique_lock\x22\
,\x22unique_ptr\x22,\x22u\
nordered_map\x22,\x22u\
nordered_multima\
p\x22,\x22unordered_mu\
ltiset\x22,\x22unorder\
ed_set\x22,\x22variant\
\x22,\x22vector\x22,\x22weak\
_ptr\x22,\x22wstring\x22,\
\x22wstring_view\x22]\x0a\
},m={className:\x22\
function.dispatc\
h\x22,relevance:0,k\
eywords:{\x0a_hint:\
[\x22abort\x22,\x22abs\x22,\x22\
acos\x22,\x22apply\x22,\x22a\
s_const\x22,\x22asin\x22,\
\x22atan\x22,\x22atan2\x22,\x22\
calloc\x22,\x22ceil\x22,\x22\
cerr\x22,\x22cin\x22,\x22clo\
g\x22,\x22cos\x22,\x22cosh\x22,\
\x22cout\x22,\x22declval\x22\
,\x22endl\x22,\x22exchang\
e\x22,\x22exit\x22,\x22exp\x22,\
\x22fabs\x22,\x22floor\x22,\x22\
fmod\x22,\x22forward\x22,\
\x22fprintf\x22,\x22fputs\
\x22,\x22free\x22,\x22frexp\x22\
,\x22fscanf\x22,\x22futur\
e\x22,\x22invoke\x22,\x22isa\
lnum\x22,\x22isalpha\x22,\
\x22iscntrl\x22,\x22isdig\
it\x22,\x22isgraph\x22,\x22i\
slower\x22,\x22isprint\
\x22,\x22ispunct\x22,\x22iss\
pace\x22,\x22isupper\x22,\
\x22isxdigit\x22,\x22labs\
\x22,\x22launder\x22,\x22lde\
xp\x22,\x22log\x22,\x22log10\
\x22,\x22make_pair\x22,\x22m\
ake_shared\x22,\x22mak\
e_shared_for_ove\
rwrite\x22,\x22make_tu\
ple\x22,\x22make_uniqu\
e\x22,\x22malloc\x22,\x22mem\
chr\x22,\x22memcmp\x22,\x22m\
emcpy\x22,\x22memset\x22,\
\x22modf\x22,\x22move\x22,\x22p\
ow\x22,\x22printf\x22,\x22pu\
tchar\x22,\x22puts\x22,\x22r\
ealloc\x22,\x22scanf\x22,\
\x22sin\x22,\x22sinh\x22,\x22sn\
printf\x22,\x22sprintf\
\x22,\x22sqrt\x22,\x22sscanf\
\x22,\x22std\x22,\x22stderr\x22\
,\x22stdin\x22,\x22stdout\
\x22,\x22strcat\x22,\x22strc\
hr\x22,\x22strcmp\x22,\x22st\
rcpy\x22,\x22strcspn\x22,\
\x22strlen\x22,\x22strnca\
t\x22,\x22strncmp\x22,\x22st\
rncpy\x22,\x22strpbrk\x22\
,\x22strrchr\x22,\x22strs\
pn\x22,\x22strstr\x22,\x22sw\
ap\x22,\x22tan\x22,\x22tanh\x22\
,\x22terminate\x22,\x22to\
_underlying\x22,\x22to\
lower\x22,\x22toupper\x22\
,\x22vfprintf\x22,\x22vis\
it\x22,\x22vprintf\x22,\x22v\
sprintf\x22]\x0a},\x0abeg\
in:t.concat(/\x5cb/\
,/(?!decltype)/,\
/(?!if)/,/(?!for\
)/,/(?!switch)/,\
/(?!while)/,e.ID\
ENT_RE,t.lookahe\
ad(/(<[^<>]+>|)\x5c\
s*\x5c(/))\x0a},g=[m,c\
,s,r,e.C_BLOCK_C\
OMMENT_MODE,l,o]\
,_={variants:[{b\
egin:/=/,end:/;/\
},{\x0abegin:/\x5c(/,e\
nd:/\x5c)/},{beginK\
eywords:\x22new thr\
ow return else\x22,\
end:/;/}],\x0akeywo\
rds:p,contains:g\
.concat([{begin:\
/\x5c(/,end:/\x5c)/,ke\
ywords:p,\x0acontai\
ns:g.concat([\x22se\
lf\x22]),relevance:\
0}]),relevance:0\
},h={className:\x22\
function\x22,\x0abegin\
:\x22(\x22+i+\x22[\x5c\x5c*&\x5c\x5cs\
]+)+\x22+u,returnBe\
gin:!0,end:/[{;=\
]/,excludeEnd:!0\
,\x0akeywords:p,ill\
egal:/[^\x5cw\x5cs\x5c*&:\
<>.]/,contains:[\
{begin:a,keyword\
s:p,relevance:0}\
,{\x0abegin:u,retur\
nBegin:!0,contai\
ns:[d],relevance\
:0},{begin:/::/,\
relevance:0},{\x0ab\
egin:/:/,endsWit\
hParent:!0,conta\
ins:[o,l]},{rele\
vance:0,match:/,\
/},{\x0aclassName:\x22\
params\x22,begin:/\x5c\
(/,end:/\x5c)/,keyw\
ords:p,relevance\
:0,\x0acontains:[r,\
e.C_BLOCK_COMMEN\
T_MODE,o,l,s,{be\
gin:/\x5c(/,end:/\x5c)\
/,keywords:p,\x0are\
levance:0,contai\
ns:[\x22self\x22,r,e.C\
_BLOCK_COMMENT_M\
ODE,o,l,s]}]\x0a},s\
,r,e.C_BLOCK_COM\
MENT_MODE,c]};re\
turn{name:\x22C++\x22,\
\x0aaliases:[\x22cc\x22,\x22\
c++\x22,\x22h++\x22,\x22hpp\x22\
,\x22hh\x22,\x22hxx\x22,\x22cxx\
\x22],keywords:p,il\
legal:\x22</\x22,\x0aclas\
sNameAliases:{\x22f\
unction.dispatch\
\x22:\x22built_in\x22},\x0ac\
ontains:[].conca\
t(_,h,m,g,[c,{\x0ab\
egin:\x22\x5c\x5cb(deque|\
list|queue|prior\
ity_queue|pair|s\
tack|vector|map|\
set|bitset|multi\
set|multimap|uno\
rdered_map|unord\
ered_set|unorder\
ed_multiset|unor\
dered_multimap|a\
rray|tuple|optio\
nal|variant|func\
tion)\x5c\x5cs*<(?!<)\x22\
,\x0aend:\x22>\x22,keywor\
ds:p,contains:[\x22\
self\x22,s]},{begin\
:e.IDENT_RE+\x22::\x22\
,keywords:p},{\x0am\
atch:[/\x5cb(?:enum\
(?:\x5cs+(?:class|s\
truct))?|class|s\
truct|union)/,/\x5c\
s+/,/\x5cw+/],\x0aclas\
sName:{1:\x22keywor\
d\x22,3:\x22title.clas\
s\x22}}])}})(e),a=r\
.keywords\x0a;retur\
n a.type=[...a.t\
ype,...t.type],\x0a\
a.literal=[...a.\
literal,...t.lit\
eral],a.built_in\
=[...a.built_in,\
...t.built_in],\x0a\
a._hints=t._hint\
s,r.name=\x22Arduin\
o\x22,r.aliases=[\x22i\
no\x22],r.supersetO\
f=\x22cpp\x22,r}})()\x0a;\
hljs.registerLan\
guage(\x22arduino\x22,\
e)})();/*! `arma\
sm` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var s=(()\
=>{\x22use strict\x22;\
return s=>{const\
 e={\x0avariants:[s\
.COMMENT(\x22^[ \x5c\x5ct\
]*(?=#)\x22,\x22$\x22,{re\
levance:0,exclud\
eBegin:!0\x0a}),s.C\
OMMENT(\x22[;@]\x22,\x22$\
\x22,{relevance:0\x0a}\
),s.C_LINE_COMME\
NT_MODE,s.C_BLOC\
K_COMMENT_MODE]}\
;return{name:\x22AR\
M Assembly\x22,\x0acas\
e_insensitive:!0\
,aliases:[\x22arm\x22]\
,keywords:{$patt\
ern:\x22\x5c\x5c.?\x22+s.IDE\
NT_RE,\x0ameta:\x22.2b\
yte .4byte .alig\
n .ascii .asciz \
.balign .byte .c\
ode .data .else \
.end .endif .end\
m .endr .equ .er\
r .exitm .extern\
 .global .hword \
.if .ifdef .ifnd\
ef .include .irp\
 .long .macro .r\
ept .req .sectio\
n .set .skip .sp\
ace .text .word \
.arm .thumb .cod\
e16 .code32 .for\
ce_thumb .thumb_\
func .ltorg ALIA\
S ALIGN ARM AREA\
 ASSERT ATTR CN \
CODE CODE16 CODE\
32 COMMON CP DAT\
A DCB DCD DCDU D\
CDO DCFD DCFDU D\
CI DCQ DCQU DCW \
DCWU DN ELIF ELS\
E END ENDFUNC EN\
DIF ENDP ENTRY E\
QU EXPORT EXPORT\
AS EXTERN FIELD \
FILL FUNCTION GB\
LA GBLL GBLS GET\
 GLOBAL IF IMPOR\
T INCBIN INCLUDE\
 INFO KEEP LCLA \
LCLL LCLS LTORG \
MACRO MAP MEND M\
EXIT NOFP OPT PR\
ESERVE8 PROC QN \
READONLY RELOC R\
EQUIRE REQUIRE8 \
RLIST FN ROUT SE\
TA SETL SETS SN \
SPACE SUBT THUMB\
 THUMBX TTL WHIL\
E WEND \x22,\x0abuilt_\
in:\x22r0 r1 r2 r3 \
r4 r5 r6 r7 r8 r\
9 r10 r11 r12 r1\
3 r14 r15 w0 w1 \
w2 w3 w4 w5 w6 w\
7 w8 w9 w10 w11 \
w12 w13 w14 w15 \
w16 w17 w18 w19 \
w20 w21 w22 w23 \
w24 w25 w26 w27 \
w28 w29 w30 x0 x\
1 x2 x3 x4 x5 x6\
 x7 x8 x9 x10 x1\
1 x12 x13 x14 x1\
5 x16 x17 x18 x1\
9 x20 x21 x22 x2\
3 x24 x25 x26 x2\
7 x28 x29 x30 pc\
 lr sp ip sl sb \
fp a1 a2 a3 a4 v\
1 v2 v3 v4 v5 v6\
 v7 v8 f0 f1 f2 \
f3 f4 f5 f6 f7 p\
0 p1 p2 p3 p4 p5\
 p6 p7 p8 p9 p10\
 p11 p12 p13 p14\
 p15 c0 c1 c2 c3\
 c4 c5 c6 c7 c8 \
c9 c10 c11 c12 c\
13 c14 c15 q0 q1\
 q2 q3 q4 q5 q6 \
q7 q8 q9 q10 q11\
 q12 q13 q14 q15\
 cpsr_c cpsr_x c\
psr_s cpsr_f cps\
r_cx cpsr_cxs cp\
sr_xs cpsr_xsf c\
psr_sf cpsr_cxsf\
 spsr_c spsr_x s\
psr_s spsr_f sps\
r_cx spsr_cxs sp\
sr_xs spsr_xsf s\
psr_sf spsr_cxsf\
 s0 s1 s2 s3 s4 \
s5 s6 s7 s8 s9 s\
10 s11 s12 s13 s\
14 s15 s16 s17 s\
18 s19 s20 s21 s\
22 s23 s24 s25 s\
26 s27 s28 s29 s\
30 s31 d0 d1 d2 \
d3 d4 d5 d6 d7 d\
8 d9 d10 d11 d12\
 d13 d14 d15 d16\
 d17 d18 d19 d20\
 d21 d22 d23 d24\
 d25 d26 d27 d28\
 d29 d30 d31 {PC\
} {VAR} {TRUE} {\
FALSE} {OPT} {CO\
NFIG} {ENDIAN} {\
CODESIZE} {CPU} \
{FPU} {ARCHITECT\
URE} {PCSTOREOFF\
SET} {ARMASM_VER\
SION} {INTER} {R\
OPI} {RWPI} {SWS\
T} {NOSWST} . @\x22\
\x0a},contains:[{cl\
assName:\x22keyword\
\x22,\x0abegin:\x22\x5c\x5cb(ad\
c|(qd?|sh?|u[qh]\
?)?add(8|16)?|us\
ada?8|(q|sh?|u[q\
h]?)?(as|sa)x|an\
d|adrl?|sbc|rs[b\
c]|asr|b[lx]?|bl\
x|bxj|cbn?z|tb[b\
h]|bic|bfc|bfi|[\
su]bfx|bkpt|cdp2\
?|clz|clrex|cmp|\
cmn|cpsi[ed]|cps\
|setend|dbg|dmb|\
dsb|eor|isb|it[t\
e]{0,3}|lsl|lsr|\
ror|rrx|ldm(([id\
][ab])|f[ds])?|l\
dr((s|ex)?[bhd])\
?|movt?|mvn|mra|\
mar|mul|[us]mull\
|smul[bwt][bt]|s\
mu[as]d|smmul|sm\
mla|mla|umlaal|s\
mlal?([wbt][bt]|\
d)|mls|smlsl?[ds\
]|smc|svc|sev|mi\
a([bt]{2}|ph)?|m\
rr?c2?|mcrr2?|mr\
s|msr|orr|orn|pk\
h(tb|bt)|rbit|re\
v(16|sh)?|sel|[s\
u]sat(16)?|nop|p\
op|push|rfe([id]\
[ab])?|stm([id][\
ab])?|str(ex)?[b\
hd]?|(qd?)?sub|(\
sh?|q|u[qh]?)?su\
b(8|16)|[su]xt(a\
?h|a?b(16)?)|srs\
([id][ab])?|swpb\
?|swi|smi|tst|te\
q|wfe|wfi|yield)\
(eq|ne|cs|cc|mi|\
pl|vs|vc|hi|ls|g\
e|lt|gt|le|al|hs\
|lo)?[sptrx]?(?=\
\x5c\x5cs)\x22\x0a},e,s.QUOT\
E_STRING_MODE,{c\
lassName:\x22string\
\x22,begin:\x22'\x22,end:\
\x22[^\x5c\x5c\x5c\x5c]'\x22,relev\
ance:0\x0a},{classN\
ame:\x22title\x22,begi\
n:\x22\x5c\x5c|\x22,end:\x22\x5c\x5c|\
\x22,illegal:\x22\x5c\x5cn\x22,\
relevance:0},{\x0ac\
lassName:\x22number\
\x22,variants:[{beg\
in:\x22[#$=]?0x[0-9\
a-f]+\x22},{begin:\x22\
[#$=]?0b[01]+\x22\x0a}\
,{begin:\x22[#$=]\x5c\x5c\
d+\x22},{begin:\x22\x5c\x5cb\
\x5c\x5cd+\x22}],relevanc\
e:0},{className:\
\x22symbol\x22,\x0avarian\
ts:[{begin:\x22^[ \x5c\
\x5ct]*[a-z_\x5c\x5c.\x5c\x5c$]\
[a-z0-9_\x5c\x5c.\x5c\x5c$]+\
:\x22},{\x0abegin:\x22^[a\
-z_\x5c\x5c.\x5c\x5c$][a-z0-\
9_\x5c\x5c.\x5c\x5c$]+\x22},{be\
gin:\x22[=#]\x5c\x5cw+\x22}]\
,relevance:0}]}}\
})()\x0a;hljs.regis\
terLanguage(\x22arm\
asm\x22,s)})();/*! \
`asciidoc` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{const n=e.rege\
x,a=[{\x0aclassName\
:\x22strong\x22,begin:\
/\x5c*{2}([^\x5cn]+?)\x5c\
*{2}/},{classNam\
e:\x22strong\x22,\x0abegi\
n:n.concat(/\x5c*\x5c*\
/,/((\x5c*(?!\x5c*)|\x5c\x5c\
[^\x5cn]|[^*\x5cn\x5c\x5c])+\
\x5cn)+/,/(\x5c*(?!\x5c*)\
|\x5c\x5c[^\x5cn]|[^*\x5cn\x5c\x5c\
])*/,/\x5c*\x5c*/),\x0are\
levance:0},{clas\
sName:\x22strong\x22,b\
egin:/\x5cB\x5c*(\x5cS|\x5cS\
[^\x5cn]*?\x5cS)\x5c*(?!\x5c\
w)/},{\x0aclassName\
:\x22strong\x22,begin:\
/\x5c*[^\x5cs]([^\x5cn]+\x5c\
n)+([^\x5cn]+)\x5c*/}]\
,s=[{\x0aclassName:\
\x22emphasis\x22,begin\
:/_{2}([^\x5cn]+?)_\
{2}/},{className\
:\x22emphasis\x22,\x0abeg\
in:n.concat(/__/\
,/((_(?!_)|\x5c\x5c[^\x5c\
n]|[^_\x5cn\x5c\x5c])+\x5cn)\
+/,/(_(?!_)|\x5c\x5c[^\
\x5cn]|[^_\x5cn\x5c\x5c])*/,\
/__/),\x0arelevance\
:0},{className:\x22\
emphasis\x22,begin:\
/\x5cb_(\x5cS|\x5cS[^\x5cn]*\
?\x5cS)_(?!\x5cw)/},{\x0a\
className:\x22empha\
sis\x22,begin:/_[^\x5c\
s]([^\x5cn]+\x5cn)+([^\
\x5cn]+)_/},{classN\
ame:\x22emphasis\x22,\x0a\
begin:\x22\x5c\x5cB'(?!['\
\x5c\x5cs])\x22,end:\x22(\x5c\x5cn\
{2}|')\x22,contains\
:[{begin:\x22\x5c\x5c\x5c\x5c'\x5c\
\x5cw\x22,relevance:0\x0a\
}],relevance:0}]\
;return{name:\x22As\
ciiDoc\x22,aliases:\
[\x22adoc\x22],\x0acontai\
ns:[e.COMMENT(\x22^\
/{4,}\x5c\x5cn\x22,\x22\x5c\x5cn/{\
4,}$\x22,{relevance\
:10\x0a}),e.COMMENT\
(\x22^//\x22,\x22$\x22,{rele\
vance:0}),{class\
Name:\x22title\x22,beg\
in:\x22^\x5c\x5c.\x5c\x5cw.*$\x22}\
,{\x0abegin:\x22^[=\x5c\x5c*\
]{4,}\x5c\x5cn\x22,end:\x22\x5c\
\x5cn^[=\x5c\x5c*]{4,}$\x22,\
relevance:10},{c\
lassName:\x22sectio\
n\x22,\x0arelevance:10\
,variants:[{begi\
n:\x22^(={1,6})[ \x5ct\
].+?([ \x5ct]\x5c\x5c1)?$\
\x22},{\x0abegin:\x22^[^\x5c\
\x5c[\x5c\x5c]\x5c\x5cn]+?\x5c\x5cn[=\
\x5c\x5c-~\x5c\x5c^\x5c\x5c+]{2,}$\
\x22}]},{className:\
\x22meta\x22,\x0abegin:\x22^\
:.+?:\x22,end:\x22\x5c\x5cs\x22\
,excludeEnd:!0,r\
elevance:10},{cl\
assName:\x22meta\x22,\x0a\
begin:\x22^\x5c\x5c[.+?\x5c\x5c\
]$\x22,relevance:0}\
,{className:\x22quo\
te\x22,begin:\x22^_{4,\
}\x5c\x5cn\x22,\x0aend:\x22\x5c\x5cn_\
{4,}$\x22,relevance\
:10},{className:\
\x22code\x22,begin:\x22^[\
\x5c\x5c-\x5c\x5c.]{4,}\x5c\x5cn\x22,\
\x0aend:\x22\x5c\x5cn[\x5c\x5c-\x5c\x5c.\
]{4,}$\x22,relevanc\
e:10},{begin:\x22^\x5c\
\x5c+{4,}\x5c\x5cn\x22,end:\x22\
\x5c\x5cn\x5c\x5c+{4,}$\x22,\x0aco\
ntains:[{begin:\x22\
<\x22,end:\x22>\x22,subLa\
nguage:\x22xml\x22,rel\
evance:0}],relev\
ance:10},{\x0aclass\
Name:\x22bullet\x22,be\
gin:\x22^(\x5c\x5c*+|-+|\x5c\
\x5c.+|[^\x5c\x5cn]+?::)\x5c\
\x5cs+\x22},{className\
:\x22symbol\x22,\x0abegin\
:\x22^(NOTE|TIP|IMP\
ORTANT|WARNING|C\
AUTION):\x5c\x5cs+\x22,re\
levance:10},{\x0abe\
gin:/\x5c\x5c[*_`]/},{\
begin:/\x5c\x5c\x5c\x5c\x5c*{2}\
[^\x5cn]*?\x5c*{2}/},{\
begin:/\x5c\x5c\x5c\x5c_{2}[\
^\x5cn]*_{2}/},{\x0abe\
gin:/\x5c\x5c\x5c\x5c`{2}[^\x5c\
n]*`{2}/},{begin\
:/[:;}][*_`](?![\
*_`])/},...a,...\
s,{\x0aclassName:\x22s\
tring\x22,variants:\
[{begin:\x22``.+?''\
\x22},{begin:\x22`.+?'\
\x22}]},{\x0aclassName\
:\x22code\x22,begin:/`\
{2}/,end:/(\x5cn{2}\
|`{2})/},{classN\
ame:\x22code\x22,\x0abegi\
n:\x22(`.+?`|\x5c\x5c+.+?\
\x5c\x5c+)\x22,relevance:\
0},{className:\x22c\
ode\x22,begin:\x22^[ \x5c\
\x5ct]\x22,\x0aend:\x22$\x22,re\
levance:0},{begi\
n:\x22^'{3,}[ \x5c\x5ct]*\
$\x22,relevance:10}\
,{\x0abegin:\x22(link:\
)?(http|https|ft\
p|file|irc|image\
:?):\x5c\x5cS+?\x5c\x5c[[^[]\
*?\x5c\x5c]\x22,\x0areturnBe\
gin:!0,contains:\
[{begin:\x22(link|i\
mage:?):\x22,releva\
nce:0},{\x0aclassNa\
me:\x22link\x22,begin:\
\x22\x5c\x5cw\x22,end:\x22[^\x5c\x5c[\
]+\x22,relevance:0}\
,{className:\x22str\
ing\x22,\x0abegin:\x22\x5c\x5c[\
\x22,end:\x22\x5c\x5c]\x22,excl\
udeBegin:!0,excl\
udeEnd:!0,releva\
nce:0}],relevanc\
e:10}]\x0a}}})();hl\
js.registerLangu\
age(\x22asciidoc\x22,e\
)})();/*! `aspec\
tj` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>{\x0acons\
t n=e.regex,t=[\x22\
false\x22,\x22synchron\
ized\x22,\x22int\x22,\x22abs\
tract\x22,\x22float\x22,\x22\
private\x22,\x22char\x22,\
\x22boolean\x22,\x22stati\
c\x22,\x22null\x22,\x22if\x22,\x22\
const\x22,\x22for\x22,\x22tr\
ue\x22,\x22while\x22,\x22lon\
g\x22,\x22throw\x22,\x22stri\
ctfp\x22,\x22finally\x22,\
\x22protected\x22,\x22imp\
ort\x22,\x22native\x22,\x22f\
inal\x22,\x22return\x22,\x22\
void\x22,\x22enum\x22,\x22el\
se\x22,\x22extends\x22,\x22i\
mplements\x22,\x22brea\
k\x22,\x22transient\x22,\x22\
new\x22,\x22catch\x22,\x22in\
stanceof\x22,\x22byte\x22\
,\x22super\x22,\x22volati\
le\x22,\x22case\x22,\x22asse\
rt\x22,\x22short\x22,\x22pac\
kage\x22,\x22default\x22,\
\x22double\x22,\x22public\
\x22,\x22try\x22,\x22this\x22,\x22\
switch\x22,\x22continu\
e\x22,\x22throws\x22,\x22pri\
vileged\x22,\x22aspect\
Of\x22,\x22adviceexecu\
tion\x22,\x22proceed\x22,\
\x22cflowbelow\x22,\x22cf\
low\x22,\x22initializa\
tion\x22,\x22preinitia\
lization\x22,\x22stati\
cinitialization\x22\
,\x22withincode\x22,\x22t\
arget\x22,\x22within\x22,\
\x22execution\x22,\x22get\
WithinTypeName\x22,\
\x22handler\x22,\x22thisJ\
oinPoint\x22,\x22thisJ\
oinPointStaticPa\
rt\x22,\x22thisEnclosi\
ngJoinPointStati\
cPart\x22,\x22declare\x22\
,\x22parents\x22,\x22warn\
ing\x22,\x22error\x22,\x22so\
ft\x22,\x22precedence\x22\
,\x22thisAspectInst\
ance\x22],i=[\x22get\x22,\
\x22set\x22,\x22args\x22,\x22ca\
ll\x22]\x0a;return{nam\
e:\x22AspectJ\x22,keyw\
ords:t,illegal:/\
<\x5c/|#/,\x0acontains\
:[e.COMMENT(/\x5c/\x5c\
*\x5c*/,/\x5c*\x5c//,{rel\
evance:0,contain\
s:[{begin:/\x5cw+@/\
,\x0arelevance:0},{\
className:\x22docta\
g\x22,begin:/@[A-Za\
-z]+/}]\x0a}),e.C_L\
INE_COMMENT_MODE\
,e.C_BLOCK_COMME\
NT_MODE,e.APOS_S\
TRING_MODE,e.QUO\
TE_STRING_MODE,{\
\x0aclassName:\x22clas\
s\x22,beginKeywords\
:\x22aspect\x22,end:/[\
{;=]/,excludeEnd\
:!0,\x0aillegal:/[:\
;\x22\x5c[\x5c]]/,contain\
s:[{\x0abeginKeywor\
ds:\x22extends impl\
ements pertypewi\
thin perthis per\
target percflowb\
elow percflow is\
singleton\x22\x0a},e.U\
NDERSCORE_TITLE_\
MODE,{begin:/\x5c([\
^\x5c)]*/,end:/[)]+\
/,keywords:t.con\
cat(i),\x0aexcludeE\
nd:!1}]},{classN\
ame:\x22class\x22,begi\
nKeywords:\x22class\
 interface\x22,end:\
/[{;=]/,\x0aexclude\
End:!0,relevance\
:0,keywords:\x22cla\
ss interface\x22,il\
legal:/[:\x22\x5c[\x5c]]/\
,\x0acontains:[{beg\
inKeywords:\x22exte\
nds implements\x22}\
,e.UNDERSCORE_TI\
TLE_MODE]},{\x0abeg\
inKeywords:\x22poin\
tcut after befor\
e around throwin\
g returning\x22,end\
:/[)]/,\x0aexcludeE\
nd:!1,illegal:/[\
\x22\x5c[\x5c]]/,contains\
:[{\x0abegin:n.conc\
at(e.UNDERSCORE_\
IDENT_RE,/\x5cs*\x5c(/\
),returnBegin:!0\
,\x0acontains:[e.UN\
DERSCORE_TITLE_M\
ODE]}]},{begin:/\
[:]/,returnBegin\
:!0,end:/[{;]/,\x0a\
relevance:0,excl\
udeEnd:!1,keywor\
ds:t,illegal:/[\x22\
\x5c[\x5c]]/,contains:\
[{\x0abegin:n.conca\
t(e.UNDERSCORE_I\
DENT_RE,/\x5cs*\x5c(/)\
,keywords:t.conc\
at(i),relevance:\
0\x0a},e.QUOTE_STRI\
NG_MODE]},{begin\
Keywords:\x22new th\
row\x22,relevance:0\
},{\x0aclassName:\x22f\
unction\x22,\x0abegin:\
/\x5cw+ +\x5cw+(\x5c.\x5cw+)\
?\x5cs*\x5c([^\x5c)]*\x5c)\x5cs\
*((throws)[\x5cw\x5cs,\
]+)?[\x5c{;]/,retur\
nBegin:!0,\x0aend:/\
[{;=]/,keywords:\
t,excludeEnd:!0,\
contains:[{\x0abegi\
n:n.concat(e.UND\
ERSCORE_IDENT_RE\
,/\x5cs*\x5c(/),return\
Begin:!0,relevan\
ce:0,\x0acontains:[\
e.UNDERSCORE_TIT\
LE_MODE]},{class\
Name:\x22params\x22,be\
gin:/\x5c(/,end:/\x5c)\
/,\x0arelevance:0,k\
eywords:t,\x0aconta\
ins:[e.APOS_STRI\
NG_MODE,e.QUOTE_\
STRING_MODE,e.C_\
NUMBER_MODE,e.C_\
BLOCK_COMMENT_MO\
DE]\x0a},e.C_LINE_C\
OMMENT_MODE,e.C_\
BLOCK_COMMENT_MO\
DE]},e.C_NUMBER_\
MODE,{\x0aclassName\
:\x22meta\x22,begin:/@\
[A-Za-z]+/}]}}})\
();hljs.register\
Language(\x22aspect\
j\x22,e)\x0a})();/*! `\
autohotkey` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>{const a={begi\
n:\x22`[\x5c\x5cs\x5c\x5cS]\x22};r\
eturn{\x0aname:\x22Aut\
oHotkey\x22,case_in\
sensitive:!0,ali\
ases:[\x22ahk\x22],key\
words:{\x0akeyword:\
\x22Break Continue \
Critical Exit Ex\
itApp Gosub Goto\
 New OnExit Paus\
e return SetBatc\
hLines SetTimer \
Suspend Thread T\
hrow Until ahk_i\
d ahk_class ahk_\
pid ahk_exe ahk_\
group\x22,\x0aliteral:\
\x22true false NOT \
AND OR\x22,\x0abuilt_i\
n:\x22ComSpec Clipb\
oard ClipboardAl\
l ErrorLevel\x22},\x0a\
contains:[a,e.in\
herit(e.QUOTE_ST\
RING_MODE,{conta\
ins:[a]}),e.COMM\
ENT(\x22;\x22,\x22$\x22,{\x0are\
levance:0}),e.C_\
BLOCK_COMMENT_MO\
DE,{className:\x22n\
umber\x22,begin:e.N\
UMBER_RE,\x0areleva\
nce:0},{classNam\
e:\x22variable\x22,beg\
in:\x22%[a-zA-Z0-9#\
_$@]+%\x22},{\x0aclass\
Name:\x22built_in\x22,\
begin:\x22^\x5c\x5cs*\x5c\x5cw+\
\x5c\x5cs*(,|%)\x22},{cla\
ssName:\x22title\x22,v\
ariants:[{\x0abegin\
:'^[^\x5c\x5cn\x22;]+::(?\
!=)'},{begin:'^[\
^\x5c\x5cn\x22;]+:(?!=)',\
relevance:0}]},{\
\x0aclassName:\x22meta\
\x22,begin:\x22^\x5c\x5cs*#\x5c\
\x5cw+\x22,end:\x22$\x22,rel\
evance:0},{class\
Name:\x22built_in\x22,\
\x0abegin:\x22A_[a-zA-\
Z0-9]+\x22},{begin:\
\x22,\x5c\x5cs*,\x22}]}}})()\
\x0a;hljs.registerL\
anguage(\x22autohot\
key\x22,e)})();/*! \
`autoit` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
e=(()=>{\x22use str\
ict\x22;return e=>{\
const t={variant\
s:[e.COMMENT(\x22;\x22\
,\x22$\x22,{\x0arelevance\
:0\x0a}),e.COMMENT(\
\x22#cs\x22,\x22#ce\x22),e.C\
OMMENT(\x22#comment\
s-start\x22,\x22#comme\
nts-end\x22)]},r={\x0a\
begin:\x22\x5c\x5c$[A-z0-\
9_]+\x22},i={classN\
ame:\x22string\x22,var\
iants:[{begin:/\x22\
/,end:/\x22/,\x0aconta\
ins:[{begin:/\x22\x22/\
,relevance:0}]},\
{begin:/'/,end:/\
'/,contains:[{be\
gin:/''/,\x0areleva\
nce:0}]}]},n={va\
riants:[e.BINARY\
_NUMBER_MODE,e.C\
_NUMBER_MODE]};r\
eturn{\x0aname:\x22Aut\
oIt\x22,case_insens\
itive:!0,illegal\
:/\x5c/\x5c*/,keywords\
:{\x0akeyword:\x22ByRe\
f Case Const Con\
tinueCase Contin\
ueLoop Dim Do El\
se ElseIf EndFun\
c EndIf EndSelec\
t EndSwitch EndW\
ith Enum Exit Ex\
itLoop For Func \
Global If In Loc\
al Next ReDim Re\
turn Select Stat\
ic Step Switch T\
hen To Until Vol\
atile WEnd While\
 With\x22,\x0abuilt_in\
:\x22Abs ACos Adlib\
Register AdlibUn\
Register Asc Asc\
W ASin Assign AT\
an AutoItSetOpti\
on AutoItWinGetT\
itle AutoItWinSe\
tTitle Beep Bina\
ry BinaryLen Bin\
aryMid BinaryToS\
tring BitAND Bit\
NOT BitOR BitRot\
ate BitShift Bit\
XOR BlockInput B\
reak Call CDTray\
 Ceiling Chr Chr\
W ClipGet ClipPu\
t ConsoleRead Co\
nsoleWrite Conso\
leWriteError Con\
trolClick Contro\
lCommand Control\
Disable ControlE\
nable ControlFoc\
us ControlGetFoc\
us ControlGetHan\
dle ControlGetPo\
s ControlGetText\
 ControlHide Con\
trolListView Con\
trolMove Control\
Send ControlSetT\
ext ControlShow \
ControlTreeView \
Cos Dec DirCopy \
DirCreate DirGet\
Size DirMove Dir\
Remove DllCall D\
llCallAddress Dl\
lCallbackFree Dl\
lCallbackGetPtr \
DllCallbackRegis\
ter DllClose Dll\
Open DllStructCr\
eate DllStructGe\
tData DllStructG\
etPtr DllStructG\
etSize DllStruct\
SetData DriveGet\
Drive DriveGetFi\
leSystem DriveGe\
tLabel DriveGetS\
erial DriveGetTy\
pe DriveMapAdd D\
riveMapDel Drive\
MapGet DriveSetL\
abel DriveSpaceF\
ree DriveSpaceTo\
tal DriveStatus \
EnvGet EnvSet En\
vUpdate Eval Exe\
cute Exp FileCha\
ngeDir FileClose\
 FileCopy FileCr\
eateNTFSLink Fil\
eCreateShortcut \
FileDelete FileE\
xists FileFindFi\
rstFile FileFind\
NextFile FileFlu\
sh FileGetAttrib\
 FileGetEncoding\
 FileGetLongName\
 FileGetPos File\
GetShortcut File\
GetShortName Fil\
eGetSize FileGet\
Time FileGetVers\
ion FileInstall \
FileMove FileOpe\
n FileOpenDialog\
 FileRead FileRe\
adLine FileReadT\
oArray FileRecyc\
le FileRecycleEm\
pty FileSaveDial\
og FileSelectFol\
der FileSetAttri\
b FileSetEnd Fil\
eSetPos FileSetT\
ime FileWrite Fi\
leWriteLine Floo\
r FtpSetProxy Fu\
ncName GUICreate\
 GUICtrlCreateAv\
i GUICtrlCreateB\
utton GUICtrlCre\
ateCheckbox GUIC\
trlCreateCombo G\
UICtrlCreateCont\
extMenu GUICtrlC\
reateDate GUICtr\
lCreateDummy GUI\
CtrlCreateEdit G\
UICtrlCreateGrap\
hic GUICtrlCreat\
eGroup GUICtrlCr\
eateIcon GUICtrl\
CreateInput GUIC\
trlCreateLabel G\
UICtrlCreateList\
 GUICtrlCreateLi\
stView GUICtrlCr\
eateListViewItem\
 GUICtrlCreateMe\
nu GUICtrlCreate\
MenuItem GUICtrl\
CreateMonthCal G\
UICtrlCreateObj \
GUICtrlCreatePic\
 GUICtrlCreatePr\
ogress GUICtrlCr\
eateRadio GUICtr\
lCreateSlider GU\
ICtrlCreateTab G\
UICtrlCreateTabI\
tem GUICtrlCreat\
eTreeView GUICtr\
lCreateTreeViewI\
tem GUICtrlCreat\
eUpdown GUICtrlD\
elete GUICtrlGet\
Handle GUICtrlGe\
tState GUICtrlRe\
ad GUICtrlRecvMs\
g GUICtrlRegiste\
rListViewSort GU\
ICtrlSendMsg GUI\
CtrlSendToDummy \
GUICtrlSetBkColo\
r GUICtrlSetColo\
r GUICtrlSetCurs\
or GUICtrlSetDat\
a GUICtrlSetDefB\
kColor GUICtrlSe\
tDefColor GUICtr\
lSetFont GUICtrl\
SetGraphic GUICt\
rlSetImage GUICt\
rlSetLimit GUICt\
rlSetOnEvent GUI\
CtrlSetPos GUICt\
rlSetResizing GU\
ICtrlSetState GU\
ICtrlSetStyle GU\
ICtrlSetTip GUID\
elete GUIGetCurs\
orInfo GUIGetMsg\
 GUIGetStyle GUI\
RegisterMsg GUIS\
etAccelerators G\
UISetBkColor GUI\
SetCoord GUISetC\
ursor GUISetFont\
 GUISetHelp GUIS\
etIcon GUISetOnE\
vent GUISetState\
 GUISetStyle GUI\
StartGroup GUISw\
itch Hex HotKeyS\
et HttpSetProxy \
HttpSetUserAgent\
 HWnd InetClose \
InetGet InetGetI\
nfo InetGetSize \
InetRead IniDele\
te IniRead IniRe\
adSection IniRea\
dSectionNames In\
iRenameSection I\
niWrite IniWrite\
Section InputBox\
 Int IsAdmin IsA\
rray IsBinary Is\
Bool IsDeclared \
IsDllStruct IsFl\
oat IsFunc IsHWn\
d IsInt IsKeywor\
d IsNumber IsObj\
 IsPtr IsString \
Log MemGetStats \
Mod MouseClick M\
ouseClickDrag Mo\
useDown MouseGet\
Cursor MouseGetP\
os MouseMove Mou\
seUp MouseWheel \
MsgBox Number Ob\
jCreate ObjCreat\
eInterface ObjEv\
ent ObjGet ObjNa\
me OnAutoItExitR\
egister OnAutoIt\
ExitUnRegister P\
ing PixelChecksu\
m PixelGetColor \
PixelSearch Proc\
essClose Process\
Exists ProcessGe\
tStats ProcessLi\
st ProcessSetPri\
ority ProcessWai\
t ProcessWaitClo\
se ProgressOff P\
rogressOn Progre\
ssSet Ptr Random\
 RegDelete RegEn\
umKey RegEnumVal\
 RegRead RegWrit\
e Round Run RunA\
s RunAsWait RunW\
ait Send SendKee\
pActive SetError\
 SetExtended She\
llExecute ShellE\
xecuteWait Shutd\
own Sin Sleep So\
undPlay SoundSet\
WaveVolume Splas\
hImageOn SplashO\
ff SplashTextOn \
Sqrt SRandom Sta\
tusbarGetText St\
derrRead StdinWr\
ite StdioClose S\
tdoutRead String\
 StringAddCR Str\
ingCompare Strin\
gFormat StringFr\
omASCIIArray Str\
ingInStr StringI\
sAlNum StringIsA\
lpha StringIsASC\
II StringIsDigit\
 StringIsFloat S\
tringIsInt Strin\
gIsLower StringI\
sSpace StringIsU\
pper StringIsXDi\
git StringLeft S\
tringLen StringL\
ower StringMid S\
tringRegExp Stri\
ngRegExpReplace \
StringReplace St\
ringReverse Stri\
ngRight StringSp\
lit StringStripC\
R StringStripWS \
StringToASCIIArr\
ay StringToBinar\
y StringTrimLeft\
 StringTrimRight\
 StringUpper Tan\
 TCPAccept TCPCl\
oseSocket TCPCon\
nect TCPListen T\
CPNameToIP TCPRe\
cv TCPSend TCPSh\
utdown, UDPShutd\
own TCPStartup, \
UDPStartup Timer\
Diff TimerInit T\
oolTip TrayCreat\
eItem TrayCreate\
Menu TrayGetMsg \
TrayItemDelete T\
rayItemGetHandle\
 TrayItemGetStat\
e TrayItemGetTex\
t TrayItemSetOnE\
vent TrayItemSet\
State TrayItemSe\
tText TraySetCli\
ck TraySetIcon T\
raySetOnEvent Tr\
aySetPauseIcon T\
raySetState Tray\
SetToolTip TrayT\
ip UBound UDPBin\
d UDPCloseSocket\
 UDPOpen UDPRecv\
 UDPSend VarGetT\
ype WinActivate \
WinActive WinClo\
se WinExists Win\
Flash WinGetCare\
tPos WinGetClass\
List WinGetClien\
tSize WinGetHand\
le WinGetPos Win\
GetProcess WinGe\
tState WinGetTex\
t WinGetTitle Wi\
nKill WinList Wi\
nMenuSelectItem \
WinMinimizeAll W\
inMinimizeAllUnd\
o WinMove WinSet\
OnTop WinSetStat\
e WinSetTitle Wi\
nSetTrans WinWai\
t WinWaitActive \
WinWaitClose Win\
WaitNotActive\x22,\x0a\
literal:\x22True Fa\
lse And Null Not\
 Or Default\x22},co\
ntains:[t,r,i,n,\
{\x0aclassName:\x22met\
a\x22,begin:\x22#\x22,end\
:\x22$\x22,keywords:{\x0a\
keyword:[\x22EndReg\
ion\x22,\x22forcedef\x22,\
\x22forceref\x22,\x22igno\
refunc\x22,\x22include\
\x22,\x22include-once\x22\
,\x22NoTrayIcon\x22,\x22O\
nAutoItStartRegi\
ster\x22,\x22pragma\x22,\x22\
Region\x22,\x22Require\
Admin\x22,\x22Tidy_Off\
\x22,\x22Tidy_On\x22,\x22Tid\
y_Parameters\x22]\x0a}\
,contains:[{begi\
n:/\x5c\x5c\x5cn/,relevan\
ce:0},{beginKeyw\
ords:\x22include\x22,k\
eywords:{\x0akeywor\
d:\x22include\x22},end\
:\x22$\x22,contains:[i\
,{className:\x22str\
ing\x22,variants:[{\
begin:\x22<\x22,\x0aend:\x22\
>\x22},{begin:/\x22/,e\
nd:/\x22/,contains:\
[{begin:/\x22\x22/,rel\
evance:0}]},{beg\
in:/'/,\x0aend:/'/,\
contains:[{begin\
:/''/,relevance:\
0}]}]}]},i,t]},{\
className:\x22symbo\
l\x22,\x0abegin:\x22@[A-z\
0-9_]+\x22},{beginK\
eywords:\x22Func\x22,e\
nd:\x22$\x22,illegal:\x22\
\x5c\x5c$|\x5c\x5c[|%\x22,\x0acont\
ains:[e.inherit(\
e.UNDERSCORE_TIT\
LE_MODE,{classNa\
me:\x22title.functi\
on\x22}),{\x0aclassNam\
e:\x22params\x22,begin\
:\x22\x5c\x5c(\x22,end:\x22\x5c\x5c)\x22\
,contains:[r,i,n\
]}]}]}}})()\x0a;hlj\
s.registerLangua\
ge(\x22autoit\x22,e)})\
();/*! `avrasm` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var r=(()=>{\x22\
use strict\x22;retu\
rn r=>({name:\x22AV\
R Assembly\x22,\x0acas\
e_insensitive:!0\
,keywords:{$patt\
ern:\x22\x5c\x5c.?\x22+r.IDE\
NT_RE,\x0akeyword:\x22\
adc add adiw and\
 andi asr bclr b\
ld brbc brbs brc\
c brcs break bre\
q brge brhc brhs\
 brid brie brlo \
brlt brmi brne b\
rpl brsh brtc br\
ts brvc brvs bse\
t bst call cbi c\
br clc clh cli c\
ln clr cls clt c\
lv clz com cp cp\
c cpi cpse dec e\
icall eijmp elpm\
 eor fmul fmuls \
fmulsu icall ijm\
p in inc jmp ld \
ldd ldi lds lpm \
lsl lsr mov movw\
 mul muls mulsu \
neg nop or ori o\
ut pop push rcal\
l ret reti rjmp \
rol ror sbc sbr \
sbrc sbrs sec se\
h sbi sbci sbic \
sbis sbiw sei se\
n ser ses set se\
v sez sleep spm \
st std sts sub s\
ubi swap tst wdr\
\x22,\x0abuilt_in:\x22r0 \
r1 r2 r3 r4 r5 r\
6 r7 r8 r9 r10 r\
11 r12 r13 r14 r\
15 r16 r17 r18 r\
19 r20 r21 r22 r\
23 r24 r25 r26 r\
27 r28 r29 r30 r\
31 x|0 xh xl y|0\
 yh yl z|0 zh zl\
 ucsr1c udr1 ucs\
r1a ucsr1b ubrr1\
l ubrr1h ucsr0c \
ubrr0h tccr3c tc\
cr3a tccr3b tcnt\
3h tcnt3l ocr3ah\
 ocr3al ocr3bh o\
cr3bl ocr3ch ocr\
3cl icr3h icr3l \
etimsk etifr tcc\
r1c ocr1ch ocr1c\
l twcr twdr twar\
 twsr twbr oscca\
l xmcra xmcrb ei\
cra spmcsr spmcr\
 portg ddrg ping\
 portf ddrf sreg\
 sph spl xdiv ra\
mpz eicrb eimsk \
gimsk gicr eifr \
gifr timsk tifr \
mcucr mcucsr tcc\
r0 tcnt0 ocr0 as\
sr tccr1a tccr1b\
 tcnt1h tcnt1l o\
cr1ah ocr1al ocr\
1bh ocr1bl icr1h\
 icr1l tccr2 tcn\
t2 ocr2 ocdr wdt\
cr sfior eearh e\
earl eedr eecr p\
orta ddra pina p\
ortb ddrb pinb p\
ortc ddrc pinc p\
ortd ddrd pind s\
pdr spsr spcr ud\
r0 ucsr0a ucsr0b\
 ubrr0l acsr adm\
ux adcsr adch ad\
cl porte ddre pi\
ne pinf\x22,\x0ameta:\x22\
.byte .cseg .db \
.def .device .ds\
eg .dw .endmacro\
 .equ .eseg .exi\
t .include .list\
 .listmac .macro\
 .nolist .org .s\
et\x22\x0a},contains:[\
r.C_BLOCK_COMMEN\
T_MODE,r.COMMENT\
(\x22;\x22,\x22$\x22,{releva\
nce:0\x0a}),r.C_NUM\
BER_MODE,r.BINAR\
Y_NUMBER_MODE,{c\
lassName:\x22number\
\x22,\x0abegin:\x22\x5c\x5cb(\x5c\x5c\
$[a-zA-Z0-9]+|0o\
[0-7]+)\x22},r.QUOT\
E_STRING_MODE,{c\
lassName:\x22string\
\x22,\x0abegin:\x22'\x22,end\
:\x22[^\x5c\x5c\x5c\x5c]'\x22,ille\
gal:\x22[^\x5c\x5c\x5c\x5c][^']\
\x22},{className:\x22s\
ymbol\x22,\x0abegin:\x22^\
[A-Za-z0-9_.$]+:\
\x22},{className:\x22m\
eta\x22,begin:\x22#\x22,e\
nd:\x22$\x22},{\x0aclassN\
ame:\x22subst\x22,begi\
n:\x22@[0-9]+\x22}]})}\
)();hljs.registe\
rLanguage(\x22avras\
m\x22,r)})();/*! `a\
wk` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>({name\
:\x22Awk\x22,keywords:\
{\x0akeyword:\x22BEGIN\
 END if else whi\
le do for in bre\
ak continue dele\
te next nextfile\
 function func e\
xit|10\x22\x0a},contai\
ns:[{className:\x22\
variable\x22,varian\
ts:[{begin:/\x5c$[\x5c\
w\x5cd#@][\x5cw\x5cd_]*/}\
,{\x0abegin:/\x5c$\x5c{(.\
*?)\x5c}/}]},{class\
Name:\x22string\x22,co\
ntains:[e.BACKSL\
ASH_ESCAPE],\x0avar\
iants:[{begin:/(\
u|b)?r?'''/,end:\
/'''/,relevance:\
10},{begin:/(u|b\
)?r?\x22\x22\x22/,\x0aend:/\x22\
\x22\x22/,relevance:10\
},{begin:/(u|r|u\
r)'/,end:/'/,rel\
evance:10},{\x0abeg\
in:/(u|r|ur)\x22/,e\
nd:/\x22/,relevance\
:10},{begin:/(b|\
br)'/,end:/'/},{\
\x0abegin:/(b|br)\x22/\
,end:/\x22/},e.APOS\
_STRING_MODE,e.Q\
UOTE_STRING_MODE\
]\x0a},e.REGEXP_MOD\
E,e.HASH_COMMENT\
_MODE,e.NUMBER_M\
ODE]})})()\x0a;hljs\
.registerLanguag\
e(\x22awk\x22,e)})();/\
*! `axapta` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>{const t=e.UND\
ERSCORE_IDENT_RE\
,s={\x0akeyword:[\x22a\
bstract\x22,\x22as\x22,\x22a\
sc\x22,\x22avg\x22,\x22break\
\x22,\x22breakpoint\x22,\x22\
by\x22,\x22byref\x22,\x22cas\
e\x22,\x22catch\x22,\x22chan\
gecompany\x22,\x22clas\
s\x22,\x22client\x22,\x22cli\
ent\x22,\x22common\x22,\x22c\
onst\x22,\x22continue\x22\
,\x22count\x22,\x22crossc\
ompany\x22,\x22delegat\
e\x22,\x22delete_from\x22\
,\x22desc\x22,\x22display\
\x22,\x22div\x22,\x22do\x22,\x22ed\
it\x22,\x22else\x22,\x22even\
thandler\x22,\x22exist\
s\x22,\x22extends\x22,\x22fi\
nal\x22,\x22finally\x22,\x22\
firstfast\x22,\x22firs\
tonly\x22,\x22firstonl\
y1\x22,\x22firstonly10\
\x22,\x22firstonly100\x22\
,\x22firstonly1000\x22\
,\x22flush\x22,\x22for\x22,\x22\
forceliterals\x22,\x22\
forcenestedloop\x22\
,\x22forceplacehold\
ers\x22,\x22forceselec\
torder\x22,\x22forupda\
te\x22,\x22from\x22,\x22gene\
rateonly\x22,\x22group\
\x22,\x22hint\x22,\x22if\x22,\x22i\
mplements\x22,\x22in\x22,\
\x22index\x22,\x22insert_\
recordset\x22,\x22inte\
rface\x22,\x22internal\
\x22,\x22is\x22,\x22join\x22,\x22l\
ike\x22,\x22maxof\x22,\x22mi\
nof\x22,\x22mod\x22,\x22name\
space\x22,\x22new\x22,\x22ne\
xt\x22,\x22nofetch\x22,\x22n\
otexists\x22,\x22optim\
isticlock\x22,\x22orde\
r\x22,\x22outer\x22,\x22pess\
imisticlock\x22,\x22pr\
int\x22,\x22private\x22,\x22\
protected\x22,\x22publ\
ic\x22,\x22readonly\x22,\x22\
repeatableread\x22,\
\x22retry\x22,\x22return\x22\
,\x22reverse\x22,\x22sele\
ct\x22,\x22server\x22,\x22se\
tting\x22,\x22static\x22,\
\x22sum\x22,\x22super\x22,\x22s\
witch\x22,\x22this\x22,\x22t\
hrow\x22,\x22try\x22,\x22tts\
abort\x22,\x22ttsbegin\
\x22,\x22ttscommit\x22,\x22u\
nchecked\x22,\x22updat\
e_recordset\x22,\x22us\
ing\x22,\x22validtimes\
tate\x22,\x22void\x22,\x22wh\
ere\x22,\x22while\x22],\x0ab\
uilt_in:[\x22anytyp\
e\x22,\x22boolean\x22,\x22by\
te\x22,\x22char\x22,\x22cont\
ainer\x22,\x22date\x22,\x22d\
ouble\x22,\x22enum\x22,\x22g\
uid\x22,\x22int\x22,\x22int6\
4\x22,\x22long\x22,\x22real\x22\
,\x22short\x22,\x22str\x22,\x22\
utcdatetime\x22,\x22va\
r\x22],\x0aliteral:[\x22d\
efault\x22,\x22false\x22,\
\x22null\x22,\x22true\x22]},\
r={variants:[{\x0am\
atch:[/(class|in\
terface)\x5cs+/,t,/\
\x5cs+(extends|impl\
ements)\x5cs+/,t]},\
{\x0amatch:[/class\x5c\
s+/,t]}],scope:{\
2:\x22title.class\x22,\
4:\x22title.class.i\
nherited\x22},\x0akeyw\
ords:s};return{n\
ame:\x22X++\x22,aliase\
s:[\x22x++\x22],keywor\
ds:s,\x0acontains:[\
e.C_LINE_COMMENT\
_MODE,e.C_BLOCK_\
COMMENT_MODE,e.A\
POS_STRING_MODE,\
e.QUOTE_STRING_M\
ODE,e.C_NUMBER_M\
ODE,{\x0aclassName:\
\x22meta\x22,begin:\x22#\x22\
,end:\x22$\x22},r]}}})\
();hljs.register\
Language(\x22axapta\
\x22,e)\x0a})();/*! `b\
ash` grammar com\
piled for Highli\
ght.js 11.10.0 *\
/\x0a(()=>{var e=((\
)=>{\x22use strict\x22\
;return e=>{cons\
t s=e.regex,t={}\
,n={begin:/\x5c$\x5c{/\
,\x0aend:/\x5c}/,conta\
ins:[\x22self\x22,{beg\
in:/:-/,contains\
:[t]}]};Object.a\
ssign(t,{\x0aclassN\
ame:\x22variable\x22,v\
ariants:[{\x0abegin\
:s.concat(/\x5c$[\x5cw\
\x5cd#@][\x5cw\x5cd_]*/,\x22\
(?![\x5c\x5cw\x5c\x5cd])(?![\
$])\x22)},n]});cons\
t a={\x0aclassName:\
\x22subst\x22,begin:/\x5c\
$\x5c(/,end:/\x5c)/,co\
ntains:[e.BACKSL\
ASH_ESCAPE]\x0a},i=\
e.inherit(e.COMM\
ENT(),{match:[/(\
^|\x5cs)/,/#.*$/],s\
cope:{2:\x22comment\
\x22}}),c={\x0abegin:/\
<<-?\x5cs*(?=\x5cw+)/,\
starts:{contains\
:[e.END_SAME_AS_\
BEGIN({begin:/(\x5c\
w+)/,\x0aend:/(\x5cw+)\
/,className:\x22str\
ing\x22})]}},o={cla\
ssName:\x22string\x22,\
begin:/\x22/,end:/\x22\
/,\x0acontains:[e.B\
ACKSLASH_ESCAPE,\
t,a]};a.contains\
.push(o);const r\
={begin:/\x5c$?\x5c(\x5c(\
/,\x0aend:/\x5c)\x5c)/,co\
ntains:[{begin:/\
\x5cd+#[0-9a-f]+/,c\
lassName:\x22number\
\x22},e.NUMBER_MODE\
,t]\x0a},l=e.SHEBAN\
G({binary:\x22(fish\
|bash|zsh|sh|csh\
|ksh|tcsh|dash|s\
csh)\x22,relevance:\
10\x0a}),m={classNa\
me:\x22function\x22,be\
gin:/\x5cw[\x5cw\x5cd_]*\x5c\
s*\x5c(\x5cs*\x5c)\x5cs*\x5c{/,\
returnBegin:!0,\x0a\
contains:[e.inhe\
rit(e.TITLE_MODE\
,{begin:/\x5cw[\x5cw\x5cd\
_]*/})],relevanc\
e:0};return{\x0anam\
e:\x22Bash\x22,aliases\
:[\x22sh\x22,\x22zsh\x22],ke\
ywords:{$pattern\
:/\x5cb[a-z][a-z0-9\
._-]+\x5cb/,\x0akeywor\
d:[\x22if\x22,\x22then\x22,\x22\
else\x22,\x22elif\x22,\x22fi\
\x22,\x22for\x22,\x22while\x22,\
\x22until\x22,\x22in\x22,\x22do\
\x22,\x22done\x22,\x22case\x22,\
\x22esac\x22,\x22function\
\x22,\x22select\x22],\x0alit\
eral:[\x22true\x22,\x22fa\
lse\x22],\x0abuilt_in:\
[\x22break\x22,\x22cd\x22,\x22c\
ontinue\x22,\x22eval\x22,\
\x22exec\x22,\x22exit\x22,\x22e\
xport\x22,\x22getopts\x22\
,\x22hash\x22,\x22pwd\x22,\x22r\
eadonly\x22,\x22return\
\x22,\x22shift\x22,\x22test\x22\
,\x22times\x22,\x22trap\x22,\
\x22umask\x22,\x22unset\x22,\
\x22alias\x22,\x22bind\x22,\x22\
builtin\x22,\x22caller\
\x22,\x22command\x22,\x22dec\
lare\x22,\x22echo\x22,\x22en\
able\x22,\x22help\x22,\x22le\
t\x22,\x22local\x22,\x22logo\
ut\x22,\x22mapfile\x22,\x22p\
rintf\x22,\x22read\x22,\x22r\
eadarray\x22,\x22sourc\
e\x22,\x22sudo\x22,\x22type\x22\
,\x22typeset\x22,\x22ulim\
it\x22,\x22unalias\x22,\x22s\
et\x22,\x22shopt\x22,\x22aut\
oload\x22,\x22bg\x22,\x22bin\
dkey\x22,\x22bye\x22,\x22cap\
\x22,\x22chdir\x22,\x22clone\
\x22,\x22comparguments\
\x22,\x22compcall\x22,\x22co\
mpctl\x22,\x22compdesc\
ribe\x22,\x22compfiles\
\x22,\x22compgroups\x22,\x22\
compquote\x22,\x22comp\
tags\x22,\x22comptry\x22,\
\x22compvalues\x22,\x22di\
rs\x22,\x22disable\x22,\x22d\
isown\x22,\x22echotc\x22,\
\x22echoti\x22,\x22emulat\
e\x22,\x22fc\x22,\x22fg\x22,\x22fl\
oat\x22,\x22functions\x22\
,\x22getcap\x22,\x22getln\
\x22,\x22history\x22,\x22int\
eger\x22,\x22jobs\x22,\x22ki\
ll\x22,\x22limit\x22,\x22log\
\x22,\x22noglob\x22,\x22popd\
\x22,\x22print\x22,\x22pushd\
\x22,\x22pushln\x22,\x22reha\
sh\x22,\x22sched\x22,\x22set\
cap\x22,\x22setopt\x22,\x22s\
tat\x22,\x22suspend\x22,\x22\
ttyctl\x22,\x22unfunct\
ion\x22,\x22unhash\x22,\x22u\
nlimit\x22,\x22unsetop\
t\x22,\x22vared\x22,\x22wait\
\x22,\x22whence\x22,\x22wher\
e\x22,\x22which\x22,\x22zcom\
pile\x22,\x22zformat\x22,\
\x22zftp\x22,\x22zle\x22,\x22zm\
odload\x22,\x22zparseo\
pts\x22,\x22zprof\x22,\x22zp\
ty\x22,\x22zregexparse\
\x22,\x22zsocket\x22,\x22zst\
yle\x22,\x22ztcp\x22,\x22chc\
on\x22,\x22chgrp\x22,\x22cho\
wn\x22,\x22chmod\x22,\x22cp\x22\
,\x22dd\x22,\x22df\x22,\x22dir\x22\
,\x22dircolors\x22,\x22ln\
\x22,\x22ls\x22,\x22mkdir\x22,\x22\
mkfifo\x22,\x22mknod\x22,\
\x22mktemp\x22,\x22mv\x22,\x22r\
ealpath\x22,\x22rm\x22,\x22r\
mdir\x22,\x22shred\x22,\x22s\
ync\x22,\x22touch\x22,\x22tr\
uncate\x22,\x22vdir\x22,\x22\
b2sum\x22,\x22base32\x22,\
\x22base64\x22,\x22cat\x22,\x22\
cksum\x22,\x22comm\x22,\x22c\
split\x22,\x22cut\x22,\x22ex\
pand\x22,\x22fmt\x22,\x22fol\
d\x22,\x22head\x22,\x22join\x22\
,\x22md5sum\x22,\x22nl\x22,\x22\
numfmt\x22,\x22od\x22,\x22pa\
ste\x22,\x22ptx\x22,\x22pr\x22,\
\x22sha1sum\x22,\x22sha22\
4sum\x22,\x22sha256sum\
\x22,\x22sha384sum\x22,\x22s\
ha512sum\x22,\x22shuf\x22\
,\x22sort\x22,\x22split\x22,\
\x22sum\x22,\x22tac\x22,\x22tai\
l\x22,\x22tr\x22,\x22tsort\x22,\
\x22unexpand\x22,\x22uniq\
\x22,\x22wc\x22,\x22arch\x22,\x22b\
asename\x22,\x22chroot\
\x22,\x22date\x22,\x22dirnam\
e\x22,\x22du\x22,\x22echo\x22,\x22\
env\x22,\x22expr\x22,\x22fac\
tor\x22,\x22groups\x22,\x22h\
ostid\x22,\x22id\x22,\x22lin\
k\x22,\x22logname\x22,\x22ni\
ce\x22,\x22nohup\x22,\x22npr\
oc\x22,\x22pathchk\x22,\x22p\
inky\x22,\x22printenv\x22\
,\x22printf\x22,\x22pwd\x22,\
\x22readlink\x22,\x22runc\
on\x22,\x22seq\x22,\x22sleep\
\x22,\x22stat\x22,\x22stdbuf\
\x22,\x22stty\x22,\x22tee\x22,\x22\
test\x22,\x22timeout\x22,\
\x22tty\x22,\x22uname\x22,\x22u\
nlink\x22,\x22uptime\x22,\
\x22users\x22,\x22who\x22,\x22w\
hoami\x22,\x22yes\x22]\x0a},\
contains:[l,e.SH\
EBANG(),m,r,i,c,\
{match:/(\x5c/[a-z.\
_-]+)+/},o,{matc\
h:/\x5c\x5c\x22/},{\x0aclass\
Name:\x22string\x22,be\
gin:/'/,end:/'/}\
,{match:/\x5c\x5c'/},t\
]}}})()\x0a;hljs.re\
gisterLanguage(\x22\
bash\x22,e)})();/*!\
 `basic` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
E=(()=>{\x22use str\
ict\x22;return E=>(\
{name:\x22BASIC\x22,ca\
se_insensitive:!\
0,\x0aillegal:\x22^.\x22,\
keywords:{$patte\
rn:\x22[a-zA-Z][a-z\
A-Z0-9_$%!#]*\x22,\x0a\
keyword:[\x22ABS\x22,\x22\
ASC\x22,\x22AND\x22,\x22ATN\x22\
,\x22AUTO|0\x22,\x22BEEP\x22\
,\x22BLOAD|10\x22,\x22BSA\
VE|10\x22,\x22CALL\x22,\x22C\
ALLS\x22,\x22CDBL\x22,\x22CH\
AIN\x22,\x22CHDIR\x22,\x22CH\
R$|10\x22,\x22CINT\x22,\x22C\
IRCLE\x22,\x22CLEAR\x22,\x22\
CLOSE\x22,\x22CLS\x22,\x22CO\
LOR\x22,\x22COM\x22,\x22COMM\
ON\x22,\x22CONT\x22,\x22COS\x22\
,\x22CSNG\x22,\x22CSRLIN\x22\
,\x22CVD\x22,\x22CVI\x22,\x22CV\
S\x22,\x22DATA\x22,\x22DATE$\
\x22,\x22DEFDBL\x22,\x22DEFI\
NT\x22,\x22DEFSNG\x22,\x22DE\
FSTR\x22,\x22DEF|0\x22,\x22S\
EG\x22,\x22USR\x22,\x22DELET\
E\x22,\x22DIM\x22,\x22DRAW\x22,\
\x22EDIT\x22,\x22END\x22,\x22EN\
VIRON\x22,\x22ENVIRON$\
\x22,\x22EOF\x22,\x22EQV\x22,\x22E\
RASE\x22,\x22ERDEV\x22,\x22E\
RDEV$\x22,\x22ERL\x22,\x22ER\
R\x22,\x22ERROR\x22,\x22EXP\x22\
,\x22FIELD\x22,\x22FILES\x22\
,\x22FIX\x22,\x22FOR|0\x22,\x22\
FRE\x22,\x22GET\x22,\x22GOSU\
B|10\x22,\x22GOTO\x22,\x22HE\
X$\x22,\x22IF\x22,\x22THEN\x22,\
\x22ELSE|0\x22,\x22INKEY$\
\x22,\x22INP\x22,\x22INPUT\x22,\
\x22INPUT#\x22,\x22INPUT$\
\x22,\x22INSTR\x22,\x22IMP\x22,\
\x22INT\x22,\x22IOCTL\x22,\x22I\
OCTL$\x22,\x22KEY\x22,\x22ON\
\x22,\x22OFF\x22,\x22LIST\x22,\x22\
KILL\x22,\x22LEFT$\x22,\x22L\
EN\x22,\x22LET\x22,\x22LINE\x22\
,\x22LLIST\x22,\x22LOAD\x22,\
\x22LOC\x22,\x22LOCATE\x22,\x22\
LOF\x22,\x22LOG\x22,\x22LPRI\
NT\x22,\x22USING\x22,\x22LSE\
T\x22,\x22MERGE\x22,\x22MID$\
\x22,\x22MKDIR\x22,\x22MKD$\x22\
,\x22MKI$\x22,\x22MKS$\x22,\x22\
MOD\x22,\x22NAME\x22,\x22NEW\
\x22,\x22NEXT\x22,\x22NOISE\x22\
,\x22NOT\x22,\x22OCT$\x22,\x22O\
N\x22,\x22OR\x22,\x22PEN\x22,\x22P\
LAY\x22,\x22STRIG\x22,\x22OP\
EN\x22,\x22OPTION\x22,\x22BA\
SE\x22,\x22OUT\x22,\x22PAINT\
\x22,\x22PALETTE\x22,\x22PCO\
PY\x22,\x22PEEK\x22,\x22PMAP\
\x22,\x22POINT\x22,\x22POKE\x22\
,\x22POS\x22,\x22PRINT\x22,\x22\
PRINT]\x22,\x22PSET\x22,\x22\
PRESET\x22,\x22PUT\x22,\x22R\
ANDOMIZE\x22,\x22READ\x22\
,\x22REM\x22,\x22RENUM\x22,\x22\
RESET|0\x22,\x22RESTOR\
E\x22,\x22RESUME\x22,\x22RET\
URN|0\x22,\x22RIGHT$\x22,\
\x22RMDIR\x22,\x22RND\x22,\x22R\
SET\x22,\x22RUN\x22,\x22SAVE\
\x22,\x22SCREEN\x22,\x22SGN\x22\
,\x22SHELL\x22,\x22SIN\x22,\x22\
SOUND\x22,\x22SPACE$\x22,\
\x22SPC\x22,\x22SQR\x22,\x22STE\
P\x22,\x22STICK\x22,\x22STOP\
\x22,\x22STR$\x22,\x22STRING\
$\x22,\x22SWAP\x22,\x22SYSTE\
M\x22,\x22TAB\x22,\x22TAN\x22,\x22\
TIME$\x22,\x22TIMER\x22,\x22\
TROFF\x22,\x22TRON\x22,\x22T\
O\x22,\x22USR\x22,\x22VAL\x22,\x22\
VARPTR\x22,\x22VARPTR$\
\x22,\x22VIEW\x22,\x22WAIT\x22,\
\x22WHILE\x22,\x22WEND\x22,\x22\
WIDTH\x22,\x22WINDOW\x22,\
\x22WRITE\x22,\x22XOR\x22]\x0a}\
,contains:[E.QUO\
TE_STRING_MODE,E\
.COMMENT(\x22REM\x22,\x22\
$\x22,{relevance:10\
\x0a}),E.COMMENT(\x22'\
\x22,\x22$\x22,{relevance\
:0}),{className:\
\x22symbol\x22,begin:\x22\
^[0-9]+ \x22,\x0arelev\
ance:10},{classN\
ame:\x22number\x22,beg\
in:\x22\x5c\x5cb\x5c\x5cd+(\x5c\x5c.\x5c\
\x5cd+)?([edED]\x5c\x5cd+\
)?[#!]?\x22,\x0areleva\
nce:0},{classNam\
e:\x22number\x22,begin\
:\x22(&[hH][0-9a-fA\
-F]{1,4})\x22},{\x0acl\
assName:\x22number\x22\
,begin:\x22(&[oO][0\
-7]{1,6})\x22}]})})\
()\x0a;hljs.registe\
rLanguage(\x22basic\
\x22,E)})();/*! `bn\
f` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>({name:\
\x22Backus\x5cu2013Nau\
r Form\x22,\x0acontain\
s:[{className:\x22a\
ttribute\x22,begin:\
/</,end:/>/},{be\
gin:/::=/,end:/$\
/,\x0acontains:[{be\
gin:/</,end:/>/\x0a\
},e.C_LINE_COMME\
NT_MODE,e.C_BLOC\
K_COMMENT_MODE,e\
.APOS_STRING_MOD\
E,e.QUOTE_STRING\
_MODE]\x0a}]})})();\
hljs.registerLan\
guage(\x22bnf\x22,e)})\
();/*! `brainfuc\
k` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>{const \
n={className:\x22li\
teral\x22,\x0abegin:/[\
+-]+/,relevance:\
0};return{name:\x22\
Brainfuck\x22,alias\
es:[\x22bf\x22],\x0aconta\
ins:[e.COMMENT(/\
[^\x5c[\x5c]\x5c.,\x5c+\x5c-<> \
\x5cr\x5cn]/,/[\x5c[\x5c]\x5c.,\
\x5c+\x5c-<> \x5cr\x5cn]/,{c\
ontains:[{\x0amatch\
:/[ ]+[^\x5c[\x5c]\x5c.,\x5c\
+\x5c-<> \x5cr\x5cn]/,rel\
evance:0}],retur\
nEnd:!0,relevanc\
e:0}),{\x0aclassNam\
e:\x22title\x22,begin:\
\x22[\x5c\x5c[\x5c\x5c]]\x22,relev\
ance:0},{classNa\
me:\x22string\x22,\x0abeg\
in:\x22[\x5c\x5c.,]\x22,rele\
vance:0},{begin:\
/(?=\x5c+\x5c+|--)/,co\
ntains:[n]},n]}}\
})()\x0a;hljs.regis\
terLanguage(\x22bra\
infuck\x22,e)})();/\
*! `c` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;return e=>{co\
nst n=e.regex,t=\
e.COMMENT(\x22//\x22,\x22\
$\x22,{\x0acontains:[{\
begin:/\x5c\x5c\x5cn/}]\x0a}\
),a=\x22decltype\x5c\x5c(\
auto\x5c\x5c)\x22,s=\x22[a-z\
A-Z_]\x5c\x5cw*::\x22,i=\x22\
(\x22+a+\x22|\x22+n.optio\
nal(s)+\x22[a-zA-Z_\
]\x5c\x5cw*\x22+n.optiona\
l(\x22<[^<>]+>\x22)+\x22)\
\x22,r={\x0aclassName:\
\x22type\x22,variants:\
[{begin:\x22\x5c\x5cb[a-z\
\x5c\x5cd_]*_t\x5c\x5cb\x22},{\x0a\
match:/\x5cbatomic_\
[a-z]{3,6}\x5cb/}]}\
,l={className:\x22s\
tring\x22,variants:\
[{\x0abegin:'(u8?|U\
|L)?\x22',end:'\x22',i\
llegal:\x22\x5c\x5cn\x22,con\
tains:[e.BACKSLA\
SH_ESCAPE]},{\x0abe\
gin:\x22(u8?|U|L)?'\
(\x5c\x5c\x5c\x5c(x[0-9A-Fa-\
f]{2}|u[0-9A-Fa-\
f]{4,8}|[0-7]{3}\
|\x5c\x5cS)|.)\x22,\x0aend:\x22\
'\x22,illegal:\x22.\x22},\
e.END_SAME_AS_BE\
GIN({\x0abegin:/(?:\
u8?|U|L)?R\x22([^()\
\x5c\x5c ]{0,16})\x5c(/,e\
nd:/\x5c)([^()\x5c\x5c ]{\
0,16})\x22/})]},o={\
\x0aclassName:\x22numb\
er\x22,variants:[{b\
egin:\x22\x5c\x5cb(0b[01'\
]+)\x22},{\x0abegin:\x22(\
-?)\x5c\x5cb([\x5c\x5cd']+(\x5c\
\x5c.[\x5c\x5cd']*)?|\x5c\x5c.[\
\x5c\x5cd']+)((ll|LL|l\
|L)(u|U)?|(u|U)(\
ll|LL|l|L)?|f|F|\
b|B)\x22\x0a},{\x0abegin:\
\x22(-?)(\x5c\x5cb0[xX][a\
-fA-F0-9']+|(\x5c\x5cb\
[\x5c\x5cd']+(\x5c\x5c.[\x5c\x5cd'\
]*)?|\x5c\x5c.[\x5c\x5cd']+)\
([eE][-+]?[\x5c\x5cd']\
+)?)\x22\x0a}],relevan\
ce:0},c={classNa\
me:\x22meta\x22,begin:\
/#\x5cs*[a-z]+\x5cb/,e\
nd:/$/,keywords:\
{\x0akeyword:\x22if el\
se elif endif de\
fine undef warni\
ng error line pr\
agma _Pragma ifd\
ef ifndef elifde\
f elifndef inclu\
de\x22\x0a},contains:[\
{begin:/\x5c\x5c\x5cn/,re\
levance:0},e.inh\
erit(l,{classNam\
e:\x22string\x22}),{\x0ac\
lassName:\x22string\
\x22,begin:/<.*?>/}\
,t,e.C_BLOCK_COM\
MENT_MODE]},d={\x0a\
className:\x22title\
\x22,begin:n.option\
al(s)+e.IDENT_RE\
,relevance:0\x0a},_\
=n.optional(s)+e\
.IDENT_RE+\x22\x5c\x5cs*\x5c\
\x5c(\x22,u={\x0akeyword:\
[\x22asm\x22,\x22auto\x22,\x22b\
reak\x22,\x22case\x22,\x22co\
ntinue\x22,\x22default\
\x22,\x22do\x22,\x22else\x22,\x22e\
num\x22,\x22extern\x22,\x22f\
or\x22,\x22fortran\x22,\x22g\
oto\x22,\x22if\x22,\x22inlin\
e\x22,\x22register\x22,\x22r\
estrict\x22,\x22return\
\x22,\x22sizeof\x22,\x22type\
of\x22,\x22typeof_unqu\
al\x22,\x22struct\x22,\x22sw\
itch\x22,\x22typedef\x22,\
\x22union\x22,\x22volatil\
e\x22,\x22while\x22,\x22_Ali\
gnas\x22,\x22_Alignof\x22\
,\x22_Atomic\x22,\x22_Gen\
eric\x22,\x22_Noreturn\
\x22,\x22_Static_asser\
t\x22,\x22_Thread_loca\
l\x22,\x22alignas\x22,\x22al\
ignof\x22,\x22noreturn\
\x22,\x22static_assert\
\x22,\x22thread_local\x22\
,\x22_Pragma\x22],\x0atyp\
e:[\x22float\x22,\x22doub\
le\x22,\x22signed\x22,\x22un\
signed\x22,\x22int\x22,\x22s\
hort\x22,\x22long\x22,\x22ch\
ar\x22,\x22void\x22,\x22_Boo\
l\x22,\x22_BitInt\x22,\x22_C\
omplex\x22,\x22_Imagin\
ary\x22,\x22_Decimal32\
\x22,\x22_Decimal64\x22,\x22\
_Decimal96\x22,\x22_De\
cimal128\x22,\x22_Deci\
mal64x\x22,\x22_Decima\
l128x\x22,\x22_Float16\
\x22,\x22_Float32\x22,\x22_F\
loat64\x22,\x22_Float1\
28\x22,\x22_Float32x\x22,\
\x22_Float64x\x22,\x22_Fl\
oat128x\x22,\x22const\x22\
,\x22static\x22,\x22const\
expr\x22,\x22complex\x22,\
\x22bool\x22,\x22imaginar\
y\x22],\x0aliteral:\x22tr\
ue false NULL\x22,\x0a\
built_in:\x22std st\
ring wstring cin\
 cout cerr clog \
stdin stdout std\
err stringstream\
 istringstream o\
stringstream aut\
o_ptr deque list\
 queue stack vec\
tor map set pair\
 bitset multiset\
 multimap unorde\
red_set unordere\
d_map unordered_\
multiset unorder\
ed_multimap prio\
rity_queue make_\
pair array share\
d_ptr abort term\
inate abs acos a\
sin atan2 atan c\
alloc ceil cosh \
cos exit exp fab\
s floor fmod fpr\
intf fputs free \
frexp fscanf fut\
ure isalnum isal\
pha iscntrl isdi\
git isgraph islo\
wer isprint ispu\
nct isspace isup\
per isxdigit tol\
ower toupper lab\
s ldexp log10 lo\
g malloc realloc\
 memchr memcmp m\
emcpy memset mod\
f pow printf put\
char puts scanf \
sinh sin snprint\
f sprintf sqrt s\
scanf strcat str\
chr strcmp strcp\
y strcspn strlen\
 strncat strncmp\
 strncpy strpbrk\
 strrchr strspn \
strstr tanh tan \
vfprintf vprintf\
 vsprintf endl i\
nitializer_list \
unique_ptr\x22\x0a},g=\
[c,r,t,e.C_BLOCK\
_COMMENT_MODE,o,\
l],m={variants:[\
{begin:/=/,end:/\
;/},{\x0abegin:/\x5c(/\
,end:/\x5c)/},{begi\
nKeywords:\x22new t\
hrow return else\
\x22,end:/;/}],\x0akey\
words:u,contains\
:g.concat([{begi\
n:/\x5c(/,end:/\x5c)/,\
keywords:u,\x0acont\
ains:g.concat([\x22\
self\x22]),relevanc\
e:0}]),relevance\
:0},p={\x0abegin:\x22(\
\x22+i+\x22[\x5c\x5c*&\x5c\x5cs]+)\
+\x22+_,returnBegin\
:!0,end:/[{;=]/,\
excludeEnd:!0,\x0ak\
eywords:u,illega\
l:/[^\x5cw\x5cs\x5c*&:<>.\
]/,contains:[{be\
gin:a,keywords:u\
,relevance:0},{\x0a\
begin:_,returnBe\
gin:!0,contains:\
[e.inherit(d,{cl\
assName:\x22title.f\
unction\x22})],\x0arel\
evance:0},{relev\
ance:0,match:/,/\
},{className:\x22pa\
rams\x22,begin:/\x5c(/\
,end:/\x5c)/,\x0akeywo\
rds:u,relevance:\
0,contains:[t,e.\
C_BLOCK_COMMENT_\
MODE,l,o,r,{begi\
n:/\x5c(/,\x0aend:/\x5c)/\
,keywords:u,rele\
vance:0,contains\
:[\x22self\x22,t,e.C_B\
LOCK_COMMENT_MOD\
E,l,o,r]\x0a}]},r,t\
,e.C_BLOCK_COMME\
NT_MODE,c]};retu\
rn{name:\x22C\x22,alia\
ses:[\x22h\x22],keywor\
ds:u,\x0adisableAut\
odetect:!0,illeg\
al:\x22</\x22,contains\
:[].concat(m,p,g\
,[c,{\x0abegin:e.ID\
ENT_RE+\x22::\x22,keyw\
ords:u},{classNa\
me:\x22class\x22,\x0abegi\
nKeywords:\x22enum \
class struct uni\
on\x22,end:/[{;:<>=\
]/,contains:[{\x0ab\
eginKeywords:\x22fi\
nal class struct\
\x22},e.TITLE_MODE]\
}]),exports:{pre\
processor:c,\x0astr\
ings:l,keywords:\
u}}}})();hljs.re\
gisterLanguage(\x22\
c\x22,e)})();/*! `c\
al` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>{\x0acons\
t n=e.regex,a=[\x22\
div\x22,\x22mod\x22,\x22in\x22,\
\x22and\x22,\x22or\x22,\x22not\x22\
,\x22xor\x22,\x22asserter\
ror\x22,\x22begin\x22,\x22ca\
se\x22,\x22do\x22,\x22downto\
\x22,\x22else\x22,\x22end\x22,\x22\
exit\x22,\x22for\x22,\x22loc\
al\x22,\x22if\x22,\x22of\x22,\x22r\
epeat\x22,\x22then\x22,\x22t\
o\x22,\x22until\x22,\x22whil\
e\x22,\x22with\x22,\x22var\x22]\
,r=[e.C_LINE_COM\
MENT_MODE,e.COMM\
ENT(/\x5c{/,/\x5c}/,{\x0a\
relevance:0}),e.\
COMMENT(/\x5c(\x5c*/,/\
\x5c*\x5c)/,{relevance\
:10})],t={classN\
ame:\x22string\x22,\x0abe\
gin:/'/,end:/'/,\
contains:[{begin\
:/''/}]},s={clas\
sName:\x22string\x22,b\
egin:/(#\x5cd+)+/\x0a}\
,i={match:[/proc\
edure/,/\x5cs+/,/[a\
-zA-Z_][\x5cw@]*/,/\
\x5cs*/],scope:{1:\x22\
keyword\x22,\x0a3:\x22tit\
le.function\x22},co\
ntains:[{classNa\
me:\x22params\x22,begi\
n:/\x5c(/,end:/\x5c)/,\
\x0akeywords:a,cont\
ains:[t,s,e.NUMB\
ER_MODE]},...r]}\
,o={\x0amatch:[/OBJ\
ECT/,/\x5cs+/,n.eit\
her(\x22Table\x22,\x22For\
m\x22,\x22Report\x22,\x22Dat\
aport\x22,\x22Codeunit\
\x22,\x22XMLport\x22,\x22Men\
uSuite\x22,\x22Page\x22,\x22\
Query\x22),/\x5cs+/,/\x5c\
d+/,/\x5cs+(?=[^\x5cs]\
)/,/.*/,/$/],\x0are\
levance:3,scope:\
{1:\x22keyword\x22,3:\x22\
type\x22,5:\x22number\x22\
,7:\x22title\x22}};ret\
urn{\x0aname:\x22C/AL\x22\
,case_insensitiv\
e:!0,keywords:{k\
eyword:a,literal\
:\x22false true\x22},\x0a\
illegal:/\x5c/\x5c*/,c\
ontains:[{match:\
/[\x5cw]+(?=\x5c=)/,sc\
ope:\x22attribute\x22,\
relevance:0\x0a},t,\
s,{className:\x22nu\
mber\x22,begin:\x22\x5c\x5cb\
\x5c\x5cd+(\x5c\x5c.\x5c\x5cd+)?(D\
T|D|T)\x22,relevanc\
e:0},{\x0aclassName\
:\x22string\x22,begin:\
'\x22',end:'\x22'},e.N\
UMBER_MODE,o,i]}\
}})()\x0a;hljs.regi\
sterLanguage(\x22ca\
l\x22,e)})();/*! `c\
apnproto` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 t=(()=>{\x22use st\
rict\x22;return t=>\
{const n={varian\
ts:[{\x0amatch:[/(s\
truct|enum|inter\
face)/,/\x5cs+/,t.I\
DENT_RE]},{\x0amatc\
h:[/extends/,/\x5cs\
*\x5c(/,t.IDENT_RE,\
/\x5cs*\x5c)/]}],scope\
:{1:\x22keyword\x22,\x0a3\
:\x22title.class\x22}}\
;return{name:\x22Ca\
p\x5cu2019n Proto\x22,\
aliases:[\x22capnp\x22\
],keywords:{\x0akey\
word:[\x22struct\x22,\x22\
enum\x22,\x22interface\
\x22,\x22union\x22,\x22group\
\x22,\x22import\x22,\x22usin\
g\x22,\x22const\x22,\x22anno\
tation\x22,\x22extends\
\x22,\x22in\x22,\x22of\x22,\x22on\x22\
,\x22as\x22,\x22with\x22,\x22fr\
om\x22,\x22fixed\x22],\x0aty\
pe:[\x22Void\x22,\x22Bool\
\x22,\x22Int8\x22,\x22Int16\x22\
,\x22Int32\x22,\x22Int64\x22\
,\x22UInt8\x22,\x22UInt16\
\x22,\x22UInt32\x22,\x22UInt\
64\x22,\x22Float32\x22,\x22F\
loat64\x22,\x22Text\x22,\x22\
Data\x22,\x22AnyPointe\
r\x22,\x22AnyStruct\x22,\x22\
Capability\x22,\x22Lis\
t\x22],\x0aliteral:[\x22t\
rue\x22,\x22false\x22]},\x0a\
contains:[t.QUOT\
E_STRING_MODE,t.\
NUMBER_MODE,t.HA\
SH_COMMENT_MODE,\
{\x0aclassName:\x22met\
a\x22,begin:/@0x[\x5cw\
\x5cd]{16};/,illega\
l:/\x5cn/},{classNa\
me:\x22symbol\x22,\x0abeg\
in:/@\x5cd+\x5cb/},n]}\
}})();hljs.regis\
terLanguage(\x22cap\
nproto\x22,t)})();/\
*! `ceylon` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>{\x0aconst a=[\x22as\
sembly\x22,\x22module\x22\
,\x22package\x22,\x22impo\
rt\x22,\x22alias\x22,\x22cla\
ss\x22,\x22interface\x22,\
\x22object\x22,\x22given\x22\
,\x22value\x22,\x22assign\
\x22,\x22void\x22,\x22functi\
on\x22,\x22new\x22,\x22of\x22,\x22\
extends\x22,\x22satisf\
ies\x22,\x22abstracts\x22\
,\x22in\x22,\x22out\x22,\x22ret\
urn\x22,\x22break\x22,\x22co\
ntinue\x22,\x22throw\x22,\
\x22assert\x22,\x22dynami\
c\x22,\x22if\x22,\x22else\x22,\x22\
switch\x22,\x22case\x22,\x22\
for\x22,\x22while\x22,\x22tr\
y\x22,\x22catch\x22,\x22fina\
lly\x22,\x22then\x22,\x22let\
\x22,\x22this\x22,\x22outer\x22\
,\x22super\x22,\x22is\x22,\x22e\
xists\x22,\x22nonempty\
\x22],s={\x0aclassName\
:\x22subst\x22,exclude\
Begin:!0,exclude\
End:!0,begin:/``\
/,end:/``/,keywo\
rds:a,\x0arelevance\
:10},n=[{classNa\
me:\x22string\x22,begi\
n:'\x22\x22\x22',end:'\x22\x22\x22\
',relevance:10},\
{\x0aclassName:\x22str\
ing\x22,begin:'\x22',e\
nd:'\x22',contains:\
[s]},{className:\
\x22string\x22,\x0abegin:\
\x22'\x22,end:\x22'\x22},{cl\
assName:\x22number\x22\
,\x0abegin:\x22#[0-9a-\
fA-F_]+|\x5c\x5c$[01_]\
+|[0-9_]+(?:\x5c\x5c.[\
0-9_](?:[eE][+-]\
?\x5c\x5cd+)?)?[kMGTPm\
unpf]?\x22,\x0arelevan\
ce:0}];return s.\
contains=n,{name\
:\x22Ceylon\x22,keywor\
ds:{\x0akeyword:a.c\
oncat([\x22shared\x22,\
\x22abstract\x22,\x22form\
al\x22,\x22default\x22,\x22a\
ctual\x22,\x22variable\
\x22,\x22late\x22,\x22native\
\x22,\x22deprecated\x22,\x22\
final\x22,\x22sealed\x22,\
\x22annotation\x22,\x22su\
ppressWarnings\x22,\
\x22small\x22]),\x0ameta:\
[\x22doc\x22,\x22by\x22,\x22lic\
ense\x22,\x22see\x22,\x22thr\
ows\x22,\x22tagged\x22]},\
\x0aillegal:\x22\x5c\x5c$[^0\
1]|#[^0-9a-fA-F]\
\x22,\x0acontains:[e.C\
_LINE_COMMENT_MO\
DE,e.COMMENT(\x22/\x5c\
\x5c*\x22,\x22\x5c\x5c*/\x22,{cont\
ains:[\x22self\x22]}),\
{\x0aclassName:\x22met\
a\x22,begin:'@[a-z]\
\x5c\x5cw*(?::\x22[^\x22]*\x22)\
?'}].concat(n)}}\
})()\x0a;hljs.regis\
terLanguage(\x22cey\
lon\x22,e)})();/*! \
`clean` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>({\
name:\x22Clean\x22,ali\
ases:[\x22icl\x22,\x22dcl\
\x22],\x0akeywords:{\x0ak\
eyword:[\x22if\x22,\x22le\
t\x22,\x22in\x22,\x22with\x22,\x22\
where\x22,\x22case\x22,\x22o\
f\x22,\x22class\x22,\x22inst\
ance\x22,\x22otherwise\
\x22,\x22implementatio\
n\x22,\x22definition\x22,\
\x22system\x22,\x22module\
\x22,\x22from\x22,\x22import\
\x22,\x22qualified\x22,\x22a\
s\x22,\x22special\x22,\x22co\
de\x22,\x22inline\x22,\x22fo\
reign\x22,\x22export\x22,\
\x22ccall\x22,\x22stdcall\
\x22,\x22generic\x22,\x22der\
ive\x22,\x22infix\x22,\x22in\
fixl\x22,\x22infixr\x22],\
\x0abuilt_in:\x22Int R\
eal Char Bool\x22,l\
iteral:\x22True Fal\
se\x22},\x0acontains:[\
e.C_LINE_COMMENT\
_MODE,e.C_BLOCK_\
COMMENT_MODE,e.A\
POS_STRING_MODE,\
e.QUOTE_STRING_M\
ODE,e.C_NUMBER_M\
ODE,{\x0abegin:\x22->|\
<-[|:]?|#!?|>>=|\
\x5c\x5c{\x5c\x5c||\x5c\x5c|\x5c\x5c}|:=\
=|=:|<>\x22}]})})()\
\x0a;hljs.registerL\
anguage(\x22clean\x22,\
e)})();/*! `cloj\
ure` grammar com\
piled for Highli\
ght.js 11.10.0 *\
/\x0a(()=>{var e=((\
)=>{\x22use strict\x22\
;return e=>{\x0acon\
st t=\x22a-zA-Z_\x5c\x5c-\
!.?+*=<>&'\x22,n=\x22[\
#]?[\x22+t+\x22][\x22+t+\x22\
0-9/;:$#]*\x22,a=\x22d\
ef defonce defpr\
otocol defstruct\
 defmulti defmet\
hod defn- defn d\
efmacro deftype \
defrecord\x22,r={\x0a$\
pattern:n,\x0abuilt\
_in:a+\x22 cond app\
ly if-not if-let\
 if not not= =|0\
 <|0 >|0 <=|0 >=\
|0 ==|0 +|0 /|0 \
*|0 -|0 rem quot\
 neg? pos? delay\
? symbol? keywor\
d? true? false? \
integer? empty? \
coll? list? set?\
 ifn? fn? associ\
ative? sequentia\
l? sorted? count\
ed? reversible? \
number? decimal?\
 class? distinct\
? isa? float? ra\
tional? reduced?\
 ratio? odd? eve\
n? char? seq? ve\
ctor? string? ma\
p? nil? contains\
? zero? instance\
? not-every? not\
-any? libspec? -\
> ->> .. . inc c\
ompare do dotime\
s mapcat take re\
move take-while \
drop letfn drop-\
last take-last d\
rop-while while \
intern condp cas\
e reduced cycle \
split-at split-w\
ith repeat repli\
cate iterate ran\
ge merge zipmap \
declare line-seq\
 sort comparator\
 sort-by dorun d\
oall nthnext nth\
rest partition e\
val doseq await \
await-for let ag\
ent atom send se\
nd-off release-p\
ending-sends add\
-watch mapv filt\
erv remove-watch\
 agent-error res\
tart-agent set-e\
rror-handler err\
or-handler set-e\
rror-mode! error\
-mode shutdown-a\
gents quote var \
fn loop recur th\
row try monitor-\
enter monitor-ex\
it macroexpand m\
acroexpand-1 for\
 dosync and or w\
hen when-not whe\
n-let comp juxt \
partial sequence\
 memoize constan\
tly complement i\
dentity assert p\
eek pop doto pro\
xy first rest co\
ns cast coll las\
t butlast sigs r\
eify second ffir\
st fnext nfirst \
nnext meta with-\
meta ns in-ns cr\
eate-ns import r\
efer keys select\
-keys vals key v\
al rseq name nam\
espace promise i\
nto transient pe\
rsistent! conj! \
assoc! dissoc! p\
op! disj! use cl\
ass type num flo\
at double short \
byte boolean big\
int biginteger b\
igdec print-meth\
od print-dup thr\
ow-if printf for\
mat load compile\
 get-in update-i\
n pr pr-on newli\
ne flush read sl\
urp read-line su\
bvec with-open m\
emfn time re-fin\
d re-groups rand\
-int rand mod lo\
cking assert-val\
id-fdecl alias r\
esolve ref deref\
 refset swap! re\
set! set-validat\
or! compare-and-\
set! alter-meta!\
 reset-meta! com\
mute get-validat\
or alter ref-set\
 ref-history-cou\
nt ref-min-histo\
ry ref-max-histo\
ry ensure sync i\
o! new next conj\
 set! to-array f\
uture future-cal\
l into-array ase\
t gen-class redu\
ce map filter fi\
nd empty hash-ma\
p hash-set sorte\
d-map sorted-map\
-by sorted-set s\
orted-set-by vec\
 vector seq flat\
ten reverse asso\
c dissoc list di\
sj get union dif\
ference intersec\
tion extend exte\
nd-type extend-p\
rotocol int nth \
delay count conc\
at chunk chunk-b\
uffer chunk-appe\
nd chunk-first c\
hunk-rest max mi\
n dec unchecked-\
inc-int unchecke\
d-inc unchecked-\
dec-inc unchecke\
d-dec unchecked-\
negate unchecked\
-add-int uncheck\
ed-add unchecked\
-subtract-int un\
checked-subtract\
 chunk-next chun\
k-cons chunked-s\
eq? prn vary-met\
a lazy-seq sprea\
d list* str find\
-keyword keyword\
 symbol gensym f\
orce rationalize\
\x22\x0a},s={begin:n,r\
elevance:0},o={s\
cope:\x22number\x22,re\
levance:0,varian\
ts:[{\x0amatch:/[-+\
]?0[xX][0-9a-fA-\
F]+N?/},{match:/\
[-+]?0[0-7]+N?/}\
,{\x0amatch:/[-+]?[\
1-9][0-9]?[rR][0\
-9a-zA-Z]+N?/},{\
match:/[-+]?[0-9\
]+\x5c/[0-9]+N?/},{\
\x0amatch:/[-+]?[0-\
9]+((\x5c.[0-9]*([e\
E][+-]?[0-9]+)?M\
?)|([eE][+-]?[0-\
9]+M?|M))/},{\x0ama\
tch:/[-+]?([1-9]\
[0-9]*|0)N?/}]},\
c={scope:\x22charac\
ter\x22,variants:[{\
\x0amatch:/\x5c\x5co[0-3]\
?[0-7]{1,2}/},{m\
atch:/\x5c\x5cu[0-9a-f\
A-F]{4}/},{\x0amatc\
h:/\x5c\x5c(newline|sp\
ace|tab|formfeed\
|backspace|retur\
n)/},{match:/\x5c\x5c\x5c\
S/,\x0arelevance:0}\
]},i={scope:\x22reg\
ex\x22,begin:/#\x22/,e\
nd:/\x22/,contains:\
[e.BACKSLASH_ESC\
APE]\x0a},d=e.inher\
it(e.QUOTE_STRIN\
G_MODE,{illegal:\
null}),l={scope:\
\x22punctuation\x22,\x0am\
atch:/,/,relevan\
ce:0},m=e.COMMEN\
T(\x22;\x22,\x22$\x22,{relev\
ance:0}),p={\x0acla\
ssName:\x22literal\x22\
,begin:/\x5cb(true|\
false|nil)\x5cb/},u\
={\x0abegin:\x22\x5c\x5c[|(#\
::?\x22+n+\x22)?\x5c\x5c{\x22,e\
nd:\x22[\x5c\x5c]\x5c\x5c}]\x22,re\
levance:0},f={cl\
assName:\x22symbol\x22\
,\x0abegin:\x22[:]{1,2\
}\x22+n},h={begin:\x22\
\x5c\x5c(\x22,end:\x22\x5c\x5c)\x22},\
y={endsWithParen\
t:!0,relevance:0\
\x0a},g={keywords:r\
,className:\x22name\
\x22,begin:n,releva\
nce:0,starts:y\x0a}\
,b=[l,h,c,i,d,m,\
f,u,o,p,s],v={be\
ginKeywords:a,ke\
ywords:{$pattern\
:n,keyword:a},\x0ae\
nd:'(\x5c\x5c[|#|\x5c\x5cd|\x22\
|:|\x5c\x5c{|\x5c\x5c)|\x5c\x5c(|$\
)',contains:[{cl\
assName:\x22title\x22,\
begin:n,\x0arelevan\
ce:0,excludeEnd:\
!0,endsParent:!0\
}].concat(b)}\x0a;r\
eturn h.contains\
=[v,g,y],y.conta\
ins=b,u.contains\
=b,{name:\x22Clojur\
e\x22,\x0aaliases:[\x22cl\
j\x22,\x22edn\x22],illega\
l:/\x5cS/,contains:\
[l,h,c,i,d,m,f,u\
,o,p]}}})()\x0a;hlj\
s.registerLangua\
ge(\x22clojure\x22,e)}\
)();/*! `clojure\
-repl` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;return e=>({n\
ame:\x22Clojure REP\
L\x22,contains:[{\x0ac\
lassName:\x22meta.p\
rompt\x22,begin:/^(\
[\x5cw.-]+|\x5cs*#_)?=\
>/,starts:{end:/\
$/,\x0asubLanguage:\
\x22clojure\x22}}]})})\
();hljs.register\
Language(\x22clojur\
e-repl\x22,e)})();/\
*! `cmake` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>({name:\x22CMake\x22,\
aliases:[\x22cmake.\
in\x22],\x0acase_insen\
sitive:!0,keywor\
ds:{\x0akeyword:\x22br\
eak cmake_host_s\
ystem_informatio\
n cmake_minimum_\
required cmake_p\
arse_arguments c\
make_policy conf\
igure_file conti\
nue elseif else \
endforeach endfu\
nction endif end\
macro endwhile e\
xecute_process f\
ile find_file fi\
nd_library find_\
package find_pat\
h find_program f\
oreach function \
get_cmake_proper\
ty get_directory\
_property get_fi\
lename_component\
 get_property if\
 include include\
_guard list macr\
o mark_as_advanc\
ed math message \
option return se\
parate_arguments\
 set_directory_p\
roperties set_pr\
operty set site_\
name string unse\
t variable_watch\
 while add_compi\
le_definitions a\
dd_compile_optio\
ns add_custom_co\
mmand add_custom\
_target add_defi\
nitions add_depe\
ndencies add_exe\
cutable add_libr\
ary add_link_opt\
ions add_subdire\
ctory add_test a\
ux_source_direct\
ory build_comman\
d create_test_so\
urcelist define_\
property enable_\
language enable_\
testing export f\
ltk_wrap_ui get_\
source_file_prop\
erty get_target_\
property get_tes\
t_property inclu\
de_directories i\
nclude_external_\
msproject includ\
e_regular_expres\
sion install lin\
k_directories li\
nk_libraries loa\
d_cache project \
qt_wrap_cpp qt_w\
rap_ui remove_de\
finitions set_so\
urce_files_prope\
rties set_target\
_properties set_\
tests_properties\
 source_group ta\
rget_compile_def\
initions target_\
compile_features\
 target_compile_\
options target_i\
nclude_directori\
es target_link_d\
irectories targe\
t_link_libraries\
 target_link_opt\
ions target_sour\
ces try_compile \
try_run ctest_bu\
ild ctest_config\
ure ctest_covera\
ge ctest_empty_b\
inary_directory \
ctest_memcheck c\
test_read_custom\
_files ctest_run\
_script ctest_sl\
eep ctest_start \
ctest_submit cte\
st_test ctest_up\
date ctest_uploa\
d build_name exe\
c_program export\
_library_depende\
ncies install_fi\
les install_prog\
rams install_tar\
gets load_comman\
d make_directory\
 output_required\
_files remove su\
bdir_depends sub\
dirs use_mangled\
_mesa utility_so\
urce variable_re\
quires write_fil\
e qt5_use_module\
s qt5_use_packag\
e qt5_wrap_cpp o\
n off true false\
 and or not comm\
and policy targe\
t test exists is\
_newer_than is_d\
irectory is_syml\
ink is_absolute \
matches less gre\
ater equal less_\
equal greater_eq\
ual strless strg\
reater strequal \
strless_equal st\
rgreater_equal v\
ersion_less vers\
ion_greater vers\
ion_equal versio\
n_less_equal ver\
sion_greater_equ\
al in_list defin\
ed\x22\x0a},contains:[\
{className:\x22vari\
able\x22,begin:/\x5c$\x5c\
{/,end:/\x5c}/\x0a},e.\
COMMENT(/#\x5c[\x5c[/,\
/]]/),e.HASH_COM\
MENT_MODE,e.QUOT\
E_STRING_MODE,e.\
NUMBER_MODE]\x0a})}\
)();hljs.registe\
rLanguage(\x22cmake\
\x22,e)})();/*! `co\
ffeescript` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22\x0a;const e\
=[\x22as\x22,\x22in\x22,\x22of\x22\
,\x22if\x22,\x22for\x22,\x22whi\
le\x22,\x22finally\x22,\x22v\
ar\x22,\x22new\x22,\x22funct\
ion\x22,\x22do\x22,\x22retur\
n\x22,\x22void\x22,\x22else\x22\
,\x22break\x22,\x22catch\x22\
,\x22instanceof\x22,\x22w\
ith\x22,\x22throw\x22,\x22ca\
se\x22,\x22default\x22,\x22t\
ry\x22,\x22switch\x22,\x22co\
ntinue\x22,\x22typeof\x22\
,\x22delete\x22,\x22let\x22,\
\x22yield\x22,\x22const\x22,\
\x22class\x22,\x22debugge\
r\x22,\x22async\x22,\x22awai\
t\x22,\x22static\x22,\x22imp\
ort\x22,\x22from\x22,\x22exp\
ort\x22,\x22extends\x22],\
n=[\x22true\x22,\x22false\
\x22,\x22null\x22,\x22undefi\
ned\x22,\x22NaN\x22,\x22Infi\
nity\x22],r=[].conc\
at([\x22setInterval\
\x22,\x22setTimeout\x22,\x22\
clearInterval\x22,\x22\
clearTimeout\x22,\x22r\
equire\x22,\x22exports\
\x22,\x22eval\x22,\x22isFini\
te\x22,\x22isNaN\x22,\x22par\
seFloat\x22,\x22parseI\
nt\x22,\x22decodeURI\x22,\
\x22decodeURICompon\
ent\x22,\x22encodeURI\x22\
,\x22encodeURICompo\
nent\x22,\x22escape\x22,\x22\
unescape\x22],[\x22Obj\
ect\x22,\x22Function\x22,\
\x22Boolean\x22,\x22Symbo\
l\x22,\x22Math\x22,\x22Date\x22\
,\x22Number\x22,\x22BigIn\
t\x22,\x22String\x22,\x22Reg\
Exp\x22,\x22Array\x22,\x22Fl\
oat32Array\x22,\x22Flo\
at64Array\x22,\x22Int8\
Array\x22,\x22Uint8Arr\
ay\x22,\x22Uint8Clampe\
dArray\x22,\x22Int16Ar\
ray\x22,\x22Int32Array\
\x22,\x22Uint16Array\x22,\
\x22Uint32Array\x22,\x22B\
igInt64Array\x22,\x22B\
igUint64Array\x22,\x22\
Set\x22,\x22Map\x22,\x22Weak\
Set\x22,\x22WeakMap\x22,\x22\
ArrayBuffer\x22,\x22Sh\
aredArrayBuffer\x22\
,\x22Atomics\x22,\x22Data\
View\x22,\x22JSON\x22,\x22Pr\
omise\x22,\x22Generato\
r\x22,\x22GeneratorFun\
ction\x22,\x22AsyncFun\
ction\x22,\x22Reflect\x22\
,\x22Proxy\x22,\x22Intl\x22,\
\x22WebAssembly\x22],[\
\x22Error\x22,\x22EvalErr\
or\x22,\x22InternalErr\
or\x22,\x22RangeError\x22\
,\x22ReferenceError\
\x22,\x22SyntaxError\x22,\
\x22TypeError\x22,\x22URI\
Error\x22])\x0a;return\
 t=>{const a={\x0ak\
eyword:e.concat(\
[\x22then\x22,\x22unless\x22\
,\x22until\x22,\x22loop\x22,\
\x22by\x22,\x22when\x22,\x22and\
\x22,\x22or\x22,\x22is\x22,\x22isn\
t\x22,\x22not\x22]).filte\
r((i=[\x22var\x22,\x22con\
st\x22,\x22let\x22,\x22funct\
ion\x22,\x22static\x22],\x0a\
e=>!i.includes(e\
))),literal:n.co\
ncat([\x22yes\x22,\x22no\x22\
,\x22on\x22,\x22off\x22]),\x0ab\
uilt_in:r.concat\
([\x22npm\x22,\x22print\x22]\
)};var i;const s\
=\x22[A-Za-z$_][0-9\
A-Za-z$_]*\x22,o={\x0a\
className:\x22subst\
\x22,begin:/#\x5c{/,en\
d:/\x5c}/,keywords:\
a\x0a},c=[t.BINARY_\
NUMBER_MODE,t.in\
herit(t.C_NUMBER\
_MODE,{starts:{e\
nd:\x22(\x5c\x5cs*/)?\x22,\x0ar\
elevance:0}}),{c\
lassName:\x22string\
\x22,variants:[{beg\
in:/'''/,end:/''\
'/,\x0acontains:[t.\
BACKSLASH_ESCAPE\
]},{begin:/'/,en\
d:/'/,contains:[\
t.BACKSLASH_ESCA\
PE]\x0a},{begin:/\x22\x22\
\x22/,end:/\x22\x22\x22/,con\
tains:[t.BACKSLA\
SH_ESCAPE,o]},{b\
egin:/\x22/,end:/\x22/\
,\x0acontains:[t.BA\
CKSLASH_ESCAPE,o\
]}]},{className:\
\x22regexp\x22,variant\
s:[{begin:\x22///\x22,\
\x0aend:\x22///\x22,conta\
ins:[o,t.HASH_CO\
MMENT_MODE]},{be\
gin:\x22//[gim]{0,3\
}(?=\x5c\x5cW)\x22,\x0arelev\
ance:0},{begin:/\
\x5c/(?![ *]).*?(?!\
[\x5c\x5c]).\x5c/[gim]{0,\
3}(?=\x5cW)/}]},{be\
gin:\x22@\x22+s\x0a},{sub\
Language:\x22javasc\
ript\x22,excludeBeg\
in:!0,excludeEnd\
:!0,variants:[{\x0a\
begin:\x22```\x22,end:\
\x22```\x22},{begin:\x22`\
\x22,end:\x22`\x22}]}];o.\
contains=c\x0a;cons\
t l=t.inherit(t.\
TITLE_MODE,{begi\
n:s}),d=\x22(\x5c\x5c(.*\x5c\
\x5c)\x5c\x5cs*)?\x5c\x5cB[-=]>\
\x22,g={\x0aclassName:\
\x22params\x22,begin:\x22\
\x5c\x5c([^\x5c\x5c(]\x22,retur\
nBegin:!0,contai\
ns:[{begin:/\x5c(/,\
\x0aend:/\x5c)/,keywor\
ds:a,contains:[\x22\
self\x22].concat(c)\
}]},u={variants:\
[{\x0amatch:[/class\
\x5cs+/,s,/\x5cs+exten\
ds\x5cs+/,s]},{matc\
h:[/class\x5cs+/,s]\
}],scope:{\x0a2:\x22ti\
tle.class\x22,4:\x22ti\
tle.class.inheri\
ted\x22},keywords:a\
};return{\x0aname:\x22\
CoffeeScript\x22,al\
iases:[\x22coffee\x22,\
\x22cson\x22,\x22iced\x22],k\
eywords:a,illega\
l:/\x5c/\x5c*/,\x0acontai\
ns:[...c,t.COMME\
NT(\x22###\x22,\x22###\x22),\
t.HASH_COMMENT_M\
ODE,{className:\x22\
function\x22,\x0abegin\
:\x22^\x5c\x5cs*\x22+s+\x22\x5c\x5cs*\
=\x5c\x5cs*\x22+d,end:\x22[-\
=]>\x22,returnBegin\
:!0,contains:[l,\
g]},{\x0abegin:/[:\x5c\
(,=]\x5cs*/,relevan\
ce:0,contains:[{\
className:\x22funct\
ion\x22,begin:d,\x0aen\
d:\x22[-=]>\x22,return\
Begin:!0,contain\
s:[g]}]},u,{begi\
n:s+\x22:\x22,end:\x22:\x22,\
\x0areturnBegin:!0,\
returnEnd:!0,rel\
evance:0}]}}})()\
\x0a;hljs.registerL\
anguage(\x22coffees\
cript\x22,e)})();/*\
! `coq` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>({\
name:\x22Coq\x22,keywo\
rds:{\x0akeyword:[\x22\
_|0\x22,\x22as\x22,\x22at\x22,\x22\
cofix\x22,\x22else\x22,\x22e\
nd\x22,\x22exists\x22,\x22ex\
ists2\x22,\x22fix\x22,\x22fo\
r\x22,\x22forall\x22,\x22fun\
\x22,\x22if\x22,\x22IF\x22,\x22in\x22\
,\x22let\x22,\x22match\x22,\x22\
mod\x22,\x22Prop\x22,\x22ret\
urn\x22,\x22Set\x22,\x22then\
\x22,\x22Type\x22,\x22using\x22\
,\x22where\x22,\x22with\x22,\
\x22Abort\x22,\x22About\x22,\
\x22Add\x22,\x22Admit\x22,\x22A\
dmitted\x22,\x22All\x22,\x22\
Arguments\x22,\x22Assu\
mptions\x22,\x22Axiom\x22\
,\x22Back\x22,\x22BackTo\x22\
,\x22Backtrack\x22,\x22Bi\
nd\x22,\x22Blacklist\x22,\
\x22Canonical\x22,\x22Cd\x22\
,\x22Check\x22,\x22Class\x22\
,\x22Classes\x22,\x22Clos\
e\x22,\x22Coercion\x22,\x22C\
oercions\x22,\x22CoFix\
point\x22,\x22CoInduct\
ive\x22,\x22Collection\
\x22,\x22Combined\x22,\x22Co\
mpute\x22,\x22Conjectu\
re\x22,\x22Conjectures\
\x22,\x22Constant\x22,\x22co\
nstr\x22,\x22Constrain\
t\x22,\x22Constructors\
\x22,\x22Context\x22,\x22Cor\
ollary\x22,\x22CreateH\
intDb\x22,\x22Cut\x22,\x22De\
clare\x22,\x22Defined\x22\
,\x22Definition\x22,\x22D\
elimit\x22,\x22Depende\
ncies\x22,\x22Dependen\
t\x22,\x22Derive\x22,\x22Dro\
p\x22,\x22eauto\x22,\x22End\x22\
,\x22Equality\x22,\x22Eva\
l\x22,\x22Example\x22,\x22Ex\
istential\x22,\x22Exis\
tentials\x22,\x22Exist\
ing\x22,\x22Export\x22,\x22e\
xporting\x22,\x22Exter\
n\x22,\x22Extract\x22,\x22Ex\
traction\x22,\x22Fact\x22\
,\x22Field\x22,\x22Fields\
\x22,\x22File\x22,\x22Fixpoi\
nt\x22,\x22Focus\x22,\x22for\
\x22,\x22From\x22,\x22Functi\
on\x22,\x22Functional\x22\
,\x22Generalizable\x22\
,\x22Global\x22,\x22Goal\x22\
,\x22Grab\x22,\x22Grammar\
\x22,\x22Graph\x22,\x22Guard\
ed\x22,\x22Heap\x22,\x22Hint\
\x22,\x22HintDb\x22,\x22Hint\
s\x22,\x22Hypotheses\x22,\
\x22Hypothesis\x22,\x22id\
ent\x22,\x22Identity\x22,\
\x22If\x22,\x22Immediate\x22\
,\x22Implicit\x22,\x22Imp\
ort\x22,\x22Include\x22,\x22\
Inductive\x22,\x22Infi\
x\x22,\x22Info\x22,\x22Initi\
al\x22,\x22Inline\x22,\x22In\
spect\x22,\x22Instance\
\x22,\x22Instances\x22,\x22I\
ntro\x22,\x22Intros\x22,\x22\
Inversion\x22,\x22Inve\
rsion_clear\x22,\x22La\
nguage\x22,\x22Left\x22,\x22\
Lemma\x22,\x22Let\x22,\x22Li\
braries\x22,\x22Librar\
y\x22,\x22Load\x22,\x22LoadP\
ath\x22,\x22Local\x22,\x22Lo\
cate\x22,\x22Ltac\x22,\x22ML\
\x22,\x22Mode\x22,\x22Module\
\x22,\x22Modules\x22,\x22Mon\
omorphic\x22,\x22Morph\
ism\x22,\x22Next\x22,\x22NoI\
nline\x22,\x22Notation\
\x22,\x22Obligation\x22,\x22\
Obligations\x22,\x22Op\
aque\x22,\x22Open\x22,\x22Op\
timize\x22,\x22Options\
\x22,\x22Parameter\x22,\x22P\
arameters\x22,\x22Para\
metric\x22,\x22Path\x22,\x22\
Paths\x22,\x22pattern\x22\
,\x22Polymorphic\x22,\x22\
Preterm\x22,\x22Print\x22\
,\x22Printing\x22,\x22Pro\
gram\x22,\x22Projectio\
ns\x22,\x22Proof\x22,\x22Pro\
position\x22,\x22Pwd\x22,\
\x22Qed\x22,\x22Quit\x22,\x22Re\
c\x22,\x22Record\x22,\x22Rec\
ursive\x22,\x22Redirec\
t\x22,\x22Relation\x22,\x22R\
emark\x22,\x22Remove\x22,\
\x22Require\x22,\x22Reser\
ved\x22,\x22Reset\x22,\x22Re\
solve\x22,\x22Restart\x22\
,\x22Rewrite\x22,\x22Righ\
t\x22,\x22Ring\x22,\x22Rings\
\x22,\x22Save\x22,\x22Scheme\
\x22,\x22Scope\x22,\x22Scope\
s\x22,\x22Script\x22,\x22Sea\
rch\x22,\x22SearchAbou\
t\x22,\x22SearchHead\x22,\
\x22SearchPattern\x22,\
\x22SearchRewrite\x22,\
\x22Section\x22,\x22Separ\
ate\x22,\x22Set\x22,\x22Seto\
id\x22,\x22Show\x22,\x22Solv\
e\x22,\x22Sorted\x22,\x22Ste\
p\x22,\x22Strategies\x22,\
\x22Strategy\x22,\x22Stru\
cture\x22,\x22SubClass\
\x22,\x22Table\x22,\x22Table\
s\x22,\x22Tactic\x22,\x22Ter\
m\x22,\x22Test\x22,\x22Theor\
em\x22,\x22Time\x22,\x22Time\
out\x22,\x22Transparen\
t\x22,\x22Type\x22,\x22Typec\
lasses\x22,\x22Types\x22,\
\x22Undelimit\x22,\x22Und\
o\x22,\x22Unfocus\x22,\x22Un\
focused\x22,\x22Unfold\
\x22,\x22Universe\x22,\x22Un\
iverses\x22,\x22Unset\x22\
,\x22Unshelve\x22,\x22usi\
ng\x22,\x22Variable\x22,\x22\
Variables\x22,\x22Vari\
ant\x22,\x22Verbose\x22,\x22\
Visibility\x22,\x22whe\
re\x22,\x22with\x22],\x0abui\
lt_in:[\x22abstract\
\x22,\x22absurd\x22,\x22admi\
t\x22,\x22after\x22,\x22appl\
y\x22,\x22as\x22,\x22assert\x22\
,\x22assumption\x22,\x22a\
t\x22,\x22auto\x22,\x22autor\
ewrite\x22,\x22autounf\
old\x22,\x22before\x22,\x22b\
ottom\x22,\x22btauto\x22,\
\x22by\x22,\x22case\x22,\x22cas\
e_eq\x22,\x22cbn\x22,\x22cbv\
\x22,\x22change\x22,\x22clas\
sical_left\x22,\x22cla\
ssical_right\x22,\x22c\
lear\x22,\x22clearbody\
\x22,\x22cofix\x22,\x22compa\
re\x22,\x22compute\x22,\x22c\
ongruence\x22,\x22cons\
tr_eq\x22,\x22construc\
tor\x22,\x22contradict\
\x22,\x22contradiction\
\x22,\x22cut\x22,\x22cutrewr\
ite\x22,\x22cycle\x22,\x22de\
cide\x22,\x22decompose\
\x22,\x22dependent\x22,\x22d\
estruct\x22,\x22destru\
ction\x22,\x22dintuiti\
on\x22,\x22discriminat\
e\x22,\x22discrR\x22,\x22do\x22\
,\x22double\x22,\x22dtaut\
o\x22,\x22eapply\x22,\x22eas\
sumption\x22,\x22eauto\
\x22,\x22ecase\x22,\x22econs\
tructor\x22,\x22edestr\
uct\x22,\x22ediscrimin\
ate\x22,\x22eelim\x22,\x22ee\
xact\x22,\x22eexists\x22,\
\x22einduction\x22,\x22ei\
njection\x22,\x22eleft\
\x22,\x22elim\x22,\x22elimty\
pe\x22,\x22enough\x22,\x22eq\
uality\x22,\x22erewrit\
e\x22,\x22eright\x22,\x22esi\
mplify_eq\x22,\x22espl\
it\x22,\x22evar\x22,\x22exac\
t\x22,\x22exactly_once\
\x22,\x22exfalso\x22,\x22exi\
sts\x22,\x22f_equal\x22,\x22\
fail\x22,\x22field\x22,\x22f\
ield_simplify\x22,\x22\
field_simplify_e\
q\x22,\x22first\x22,\x22firs\
torder\x22,\x22fix\x22,\x22f\
old\x22,\x22fourier\x22,\x22\
functional\x22,\x22gen\
eralize\x22,\x22genera\
lizing\x22,\x22gfail\x22,\
\x22give_up\x22,\x22has_e\
var\x22,\x22hnf\x22,\x22idta\
c\x22,\x22in\x22,\x22inducti\
on\x22,\x22injection\x22,\
\x22instantiate\x22,\x22i\
ntro\x22,\x22intro_pat\
tern\x22,\x22intros\x22,\x22\
intuition\x22,\x22inve\
rsion\x22,\x22inversio\
n_clear\x22,\x22is_eva\
r\x22,\x22is_var\x22,\x22lap\
ply\x22,\x22lazy\x22,\x22lef\
t\x22,\x22lia\x22,\x22lra\x22,\x22\
move\x22,\x22native_co\
mpute\x22,\x22nia\x22,\x22ns\
atz\x22,\x22omega\x22,\x22on\
ce\x22,\x22pattern\x22,\x22p\
ose\x22,\x22progress\x22,\
\x22proof\x22,\x22psatz\x22,\
\x22quote\x22,\x22record\x22\
,\x22red\x22,\x22refine\x22,\
\x22reflexivity\x22,\x22r\
emember\x22,\x22rename\
\x22,\x22repeat\x22,\x22repl\
ace\x22,\x22revert\x22,\x22r\
evgoals\x22,\x22rewrit\
e\x22,\x22rewrite_stra\
t\x22,\x22right\x22,\x22ring\
\x22,\x22ring_simplify\
\x22,\x22rtauto\x22,\x22set\x22\
,\x22setoid_reflexi\
vity\x22,\x22setoid_re\
place\x22,\x22setoid_r\
ewrite\x22,\x22setoid_\
symmetry\x22,\x22setoi\
d_transitivity\x22,\
\x22shelve\x22,\x22shelve\
_unifiable\x22,\x22sim\
pl\x22,\x22simple\x22,\x22si\
mplify_eq\x22,\x22solv\
e\x22,\x22specialize\x22,\
\x22split\x22,\x22split_R\
abs\x22,\x22split_Rmul\
t\x22,\x22stepl\x22,\x22step\
r\x22,\x22subst\x22,\x22sum\x22\
,\x22swap\x22,\x22symmetr\
y\x22,\x22tactic\x22,\x22tau\
to\x22,\x22time\x22,\x22time\
out\x22,\x22top\x22,\x22tran\
sitivity\x22,\x22trivi\
al\x22,\x22try\x22,\x22tryif\
\x22,\x22unfold\x22,\x22unif\
y\x22,\x22until\x22,\x22usin\
g\x22,\x22vm_compute\x22,\
\x22with\x22]\x0a},contai\
ns:[e.QUOTE_STRI\
NG_MODE,e.COMMEN\
T(\x22\x5c\x5c(\x5c\x5c*\x22,\x22\x5c\x5c*\x5c\
\x5c)\x22),e.C_NUMBER_\
MODE,{\x0aclassName\
:\x22type\x22,excludeB\
egin:!0,begin:\x22\x5c\
\x5c|\x5c\x5cs*\x22,end:\x22\x5c\x5cw\
+\x22},{begin:/[-=]\
>/}]})\x0a})();hljs\
.registerLanguag\
e(\x22coq\x22,e)})();/\
*! `cos` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
e=(()=>{\x22use str\
ict\x22;return e=>(\
{name:\x22Cach\x5cxe9 \
Object Script\x22,\x0a\
case_insensitive\
:!0,aliases:[\x22cl\
s\x22],\x0akeywords:\x22p\
roperty paramete\
r class classmet\
hod clientmethod\
 extends as brea\
k catch close co\
ntinue do d|0 el\
se elseif for go\
to halt hang h|0\
 if job j|0 kill\
 k|0 lock l|0 me\
rge new open qui\
t q|0 read r|0 r\
eturn set s|0 tc\
ommit throw trol\
lback try tstart\
 use view while \
write w|0 xecute\
 x|0 zkill znspa\
ce zn ztrap zwri\
te zw zzdump zzw\
rite print zbrea\
k zinsert zload \
zprint zremove z\
save zzprint mv \
mvcall mvcrt mvd\
im mvprint zquit\
 zsync ascii\x22,\x0ac\
ontains:[{classN\
ame:\x22number\x22,beg\
in:\x22\x5c\x5cb(\x5c\x5cd+(\x5c\x5c.\
\x5c\x5cd*)?|\x5c\x5c.\x5c\x5cd+)\x22\
,relevance:0},{\x0a\
className:\x22strin\
g\x22,variants:[{be\
gin:'\x22',end:'\x22',\
contains:[{begin\
:'\x22\x22',\x0arelevance\
:0}]}]},e.C_LINE\
_COMMENT_MODE,e.\
C_BLOCK_COMMENT_\
MODE,{\x0aclassName\
:\x22comment\x22,begin\
:/;/,end:\x22$\x22,rel\
evance:0},{class\
Name:\x22built_in\x22,\
\x0abegin:/(?:\x5c$\x5c$?\
|\x5c.\x5c.)\x5c^?[a-zA-Z\
]+/},{className:\
\x22built_in\x22,\x0abegi\
n:/\x5c$\x5c$\x5c$[a-zA-Z\
]+/},{className:\
\x22built_in\x22,begin\
:/%[a-z]+(?:\x5c.[a\
-z]+)*/},{\x0aclass\
Name:\x22symbol\x22,be\
gin:/\x5c^%?[a-zA-Z\
][\x5cw]*/},{classN\
ame:\x22keyword\x22,\x0ab\
egin:/##class|##\
super|#define|#d\
im/},{begin:/&sq\
l\x5c(/,end:/\x5c)/,ex\
cludeBegin:!0,\x0ae\
xcludeEnd:!0,sub\
Language:\x22sql\x22},\
{begin:/&(js|jsc\
ript|javascript)\
</,end:/>/,\x0aexcl\
udeBegin:!0,excl\
udeEnd:!0,subLan\
guage:\x22javascrip\
t\x22},{begin:/&htm\
l<\x5cs*</,\x0aend:/>\x5c\
s*>/,subLanguage\
:\x22xml\x22}]})})();h\
ljs.registerLang\
uage(\x22cos\x22,e)})(\
);/*! `cpp` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>{const t=e.reg\
ex,a=e.COMMENT(\x22\
//\x22,\x22$\x22,{\x0acontai\
ns:[{begin:/\x5c\x5c\x5cn\
/}]\x0a}),n=\x22declty\
pe\x5c\x5c(auto\x5c\x5c)\x22,r=\
\x22[a-zA-Z_]\x5c\x5cw*::\
\x22,i=\x22(?!struct)(\
\x22+n+\x22|\x22+t.option\
al(r)+\x22[a-zA-Z_]\
\x5c\x5cw*\x22+t.optional\
(\x22<[^<>]+>\x22)+\x22)\x22\
,s={\x0aclassName:\x22\
type\x22,begin:\x22\x5c\x5cb\
[a-z\x5c\x5cd_]*_t\x5c\x5cb\x22\
},c={className:\x22\
string\x22,variants\
:[{\x0abegin:'(u8?|\
U|L)?\x22',end:'\x22',\
illegal:\x22\x5c\x5cn\x22,co\
ntains:[e.BACKSL\
ASH_ESCAPE]},{\x0ab\
egin:\x22(u8?|U|L)?\
'(\x5c\x5c\x5c\x5c(x[0-9A-Fa\
-f]{2}|u[0-9A-Fa\
-f]{4,8}|[0-7]{3\
}|\x5c\x5cS)|.)\x22,\x0aend:\
\x22'\x22,illegal:\x22.\x22}\
,e.END_SAME_AS_B\
EGIN({\x0abegin:/(?\
:u8?|U|L)?R\x22([^(\
)\x5c\x5c ]{0,16})\x5c(/,\
end:/\x5c)([^()\x5c\x5c ]\
{0,16})\x22/})]},o=\
{\x0aclassName:\x22num\
ber\x22,variants:[{\
\x0abegin:\x22[+-]?(?:\
(?:[0-9](?:'?[0-\
9])*\x5c\x5c.(?:[0-9](\
?:'?[0-9])*)?|\x5c\x5c\
.[0-9](?:'?[0-9]\
)*)(?:[Ee][+-]?[\
0-9](?:'?[0-9])*\
)?|[0-9](?:'?[0-\
9])*[Ee][+-]?[0-\
9](?:'?[0-9])*|0\
[Xx](?:[0-9A-Fa-\
f](?:'?[0-9A-Fa-\
f])*(?:\x5c\x5c.(?:[0-\
9A-Fa-f](?:'?[0-\
9A-Fa-f])*)?)?|\x5c\
\x5c.[0-9A-Fa-f](?:\
'?[0-9A-Fa-f])*)\
[Pp][+-]?[0-9](?\
:'?[0-9])*)(?:[F\
f](?:16|32|64|12\
8)?|(BF|bf)16|[L\
l]|)\x22\x0a},{\x0abegin:\
\x22[+-]?\x5c\x5cb(?:0[Bb\
][01](?:'?[01])*\
|0[Xx][0-9A-Fa-f\
](?:'?[0-9A-Fa-f\
])*|0(?:'?[0-7])\
*|[1-9](?:'?[0-9\
])*)(?:[Uu](?:LL\
?|ll?)|[Uu][Zz]?\
|(?:LL?|ll?)[Uu]\
?|[Zz][Uu]|)\x22\x0a}]\
,relevance:0},l=\
{className:\x22meta\
\x22,begin:/#\x5cs*[a-\
z]+\x5cb/,end:/$/,k\
eywords:{\x0akeywor\
d:\x22if else elif \
endif define und\
ef warning error\
 line pragma _Pr\
agma ifdef ifnde\
f include\x22\x0a},con\
tains:[{begin:/\x5c\
\x5c\x5cn/,relevance:0\
},e.inherit(c,{c\
lassName:\x22string\
\x22}),{\x0aclassName:\
\x22string\x22,begin:/\
<.*?>/},a,e.C_BL\
OCK_COMMENT_MODE\
]},u={\x0aclassName\
:\x22title\x22,begin:t\
.optional(r)+e.I\
DENT_RE,relevanc\
e:0\x0a},d=t.option\
al(r)+e.IDENT_RE\
+\x22\x5c\x5cs*\x5c\x5c(\x22,p={\x0at\
ype:[\x22bool\x22,\x22cha\
r\x22,\x22char16_t\x22,\x22c\
har32_t\x22,\x22char8_\
t\x22,\x22double\x22,\x22flo\
at\x22,\x22int\x22,\x22long\x22\
,\x22short\x22,\x22void\x22,\
\x22wchar_t\x22,\x22unsig\
ned\x22,\x22signed\x22,\x22c\
onst\x22,\x22static\x22],\
\x0akeyword:[\x22align\
as\x22,\x22alignof\x22,\x22a\
nd\x22,\x22and_eq\x22,\x22as\
m\x22,\x22atomic_cance\
l\x22,\x22atomic_commi\
t\x22,\x22atomic_noexc\
ept\x22,\x22auto\x22,\x22bit\
and\x22,\x22bitor\x22,\x22br\
eak\x22,\x22case\x22,\x22cat\
ch\x22,\x22class\x22,\x22co_\
await\x22,\x22co_retur\
n\x22,\x22co_yield\x22,\x22c\
ompl\x22,\x22concept\x22,\
\x22const_cast|10\x22,\
\x22consteval\x22,\x22con\
stexpr\x22,\x22constin\
it\x22,\x22continue\x22,\x22\
decltype\x22,\x22defau\
lt\x22,\x22delete\x22,\x22do\
\x22,\x22dynamic_cast|\
10\x22,\x22else\x22,\x22enum\
\x22,\x22explicit\x22,\x22ex\
port\x22,\x22extern\x22,\x22\
false\x22,\x22final\x22,\x22\
for\x22,\x22friend\x22,\x22g\
oto\x22,\x22if\x22,\x22impor\
t\x22,\x22inline\x22,\x22mod\
ule\x22,\x22mutable\x22,\x22\
namespace\x22,\x22new\x22\
,\x22noexcept\x22,\x22not\
\x22,\x22not_eq\x22,\x22null\
ptr\x22,\x22operator\x22,\
\x22or\x22,\x22or_eq\x22,\x22ov\
erride\x22,\x22private\
\x22,\x22protected\x22,\x22p\
ublic\x22,\x22reflexpr\
\x22,\x22register\x22,\x22re\
interpret_cast|1\
0\x22,\x22requires\x22,\x22r\
eturn\x22,\x22sizeof\x22,\
\x22static_assert\x22,\
\x22static_cast|10\x22\
,\x22struct\x22,\x22switc\
h\x22,\x22synchronized\
\x22,\x22template\x22,\x22th\
is\x22,\x22thread_loca\
l\x22,\x22throw\x22,\x22tran\
saction_safe\x22,\x22t\
ransaction_safe_\
dynamic\x22,\x22true\x22,\
\x22try\x22,\x22typedef\x22,\
\x22typeid\x22,\x22typena\
me\x22,\x22union\x22,\x22usi\
ng\x22,\x22virtual\x22,\x22v\
olatile\x22,\x22while\x22\
,\x22xor\x22,\x22xor_eq\x22]\
,\x0aliteral:[\x22NULL\
\x22,\x22false\x22,\x22nullo\
pt\x22,\x22nullptr\x22,\x22t\
rue\x22],built_in:[\
\x22_Pragma\x22],\x0a_typ\
e_hints:[\x22any\x22,\x22\
auto_ptr\x22,\x22barri\
er\x22,\x22binary_sema\
phore\x22,\x22bitset\x22,\
\x22complex\x22,\x22condi\
tion_variable\x22,\x22\
condition_variab\
le_any\x22,\x22countin\
g_semaphore\x22,\x22de\
que\x22,\x22false_type\
\x22,\x22future\x22,\x22imag\
inary\x22,\x22initiali\
zer_list\x22,\x22istri\
ngstream\x22,\x22jthre\
ad\x22,\x22latch\x22,\x22loc\
k_guard\x22,\x22multim\
ap\x22,\x22multiset\x22,\x22\
mutex\x22,\x22optional\
\x22,\x22ostringstream\
\x22,\x22packaged_task\
\x22,\x22pair\x22,\x22promis\
e\x22,\x22priority_que\
ue\x22,\x22queue\x22,\x22rec\
ursive_mutex\x22,\x22r\
ecursive_timed_m\
utex\x22,\x22scoped_lo\
ck\x22,\x22set\x22,\x22share\
d_future\x22,\x22share\
d_lock\x22,\x22shared_\
mutex\x22,\x22shared_t\
imed_mutex\x22,\x22sha\
red_ptr\x22,\x22stack\x22\
,\x22string_view\x22,\x22\
stringstream\x22,\x22t\
imed_mutex\x22,\x22thr\
ead\x22,\x22true_type\x22\
,\x22tuple\x22,\x22unique\
_lock\x22,\x22unique_p\
tr\x22,\x22unordered_m\
ap\x22,\x22unordered_m\
ultimap\x22,\x22unorde\
red_multiset\x22,\x22u\
nordered_set\x22,\x22v\
ariant\x22,\x22vector\x22\
,\x22weak_ptr\x22,\x22wst\
ring\x22,\x22wstring_v\
iew\x22]\x0a},_={class\
Name:\x22function.d\
ispatch\x22,relevan\
ce:0,keywords:{\x0a\
_hint:[\x22abort\x22,\x22\
abs\x22,\x22acos\x22,\x22app\
ly\x22,\x22as_const\x22,\x22\
asin\x22,\x22atan\x22,\x22at\
an2\x22,\x22calloc\x22,\x22c\
eil\x22,\x22cerr\x22,\x22cin\
\x22,\x22clog\x22,\x22cos\x22,\x22\
cosh\x22,\x22cout\x22,\x22de\
clval\x22,\x22endl\x22,\x22e\
xchange\x22,\x22exit\x22,\
\x22exp\x22,\x22fabs\x22,\x22fl\
oor\x22,\x22fmod\x22,\x22for\
ward\x22,\x22fprintf\x22,\
\x22fputs\x22,\x22free\x22,\x22\
frexp\x22,\x22fscanf\x22,\
\x22future\x22,\x22invoke\
\x22,\x22isalnum\x22,\x22isa\
lpha\x22,\x22iscntrl\x22,\
\x22isdigit\x22,\x22isgra\
ph\x22,\x22islower\x22,\x22i\
sprint\x22,\x22ispunct\
\x22,\x22isspace\x22,\x22isu\
pper\x22,\x22isxdigit\x22\
,\x22labs\x22,\x22launder\
\x22,\x22ldexp\x22,\x22log\x22,\
\x22log10\x22,\x22make_pa\
ir\x22,\x22make_shared\
\x22,\x22make_shared_f\
or_overwrite\x22,\x22m\
ake_tuple\x22,\x22make\
_unique\x22,\x22malloc\
\x22,\x22memchr\x22,\x22memc\
mp\x22,\x22memcpy\x22,\x22me\
mset\x22,\x22modf\x22,\x22mo\
ve\x22,\x22pow\x22,\x22print\
f\x22,\x22putchar\x22,\x22pu\
ts\x22,\x22realloc\x22,\x22s\
canf\x22,\x22sin\x22,\x22sin\
h\x22,\x22snprintf\x22,\x22s\
printf\x22,\x22sqrt\x22,\x22\
sscanf\x22,\x22std\x22,\x22s\
tderr\x22,\x22stdin\x22,\x22\
stdout\x22,\x22strcat\x22\
,\x22strchr\x22,\x22strcm\
p\x22,\x22strcpy\x22,\x22str\
cspn\x22,\x22strlen\x22,\x22\
strncat\x22,\x22strncm\
p\x22,\x22strncpy\x22,\x22st\
rpbrk\x22,\x22strrchr\x22\
,\x22strspn\x22,\x22strst\
r\x22,\x22swap\x22,\x22tan\x22,\
\x22tanh\x22,\x22terminat\
e\x22,\x22to_underlyin\
g\x22,\x22tolower\x22,\x22to\
upper\x22,\x22vfprintf\
\x22,\x22visit\x22,\x22vprin\
tf\x22,\x22vsprintf\x22]\x0a\
},\x0abegin:t.conca\
t(/\x5cb/,/(?!declt\
ype)/,/(?!if)/,/\
(?!for)/,/(?!swi\
tch)/,/(?!while)\
/,e.IDENT_RE,t.l\
ookahead(/(<[^<>\
]+>|)\x5cs*\x5c(/))\x0a},\
m=[_,l,s,a,e.C_B\
LOCK_COMMENT_MOD\
E,o,c],f={varian\
ts:[{begin:/=/,e\
nd:/;/},{\x0abegin:\
/\x5c(/,end:/\x5c)/},{\
beginKeywords:\x22n\
ew throw return \
else\x22,end:/;/}],\
\x0akeywords:p,cont\
ains:m.concat([{\
begin:/\x5c(/,end:/\
\x5c)/,keywords:p,\x0a\
contains:m.conca\
t([\x22self\x22]),rele\
vance:0}]),relev\
ance:0},g={class\
Name:\x22function\x22,\
\x0abegin:\x22(\x22+i+\x22[\x5c\
\x5c*&\x5c\x5cs]+)+\x22+d,re\
turnBegin:!0,end\
:/[{;=]/,exclude\
End:!0,\x0akeywords\
:p,illegal:/[^\x5cw\
\x5cs\x5c*&:<>.]/,cont\
ains:[{begin:n,k\
eywords:p,releva\
nce:0},{\x0abegin:d\
,returnBegin:!0,\
contains:[u],rel\
evance:0},{begin\
:/::/,relevance:\
0},{\x0abegin:/:/,e\
ndsWithParent:!0\
,contains:[c,o]}\
,{relevance:0,ma\
tch:/,/},{\x0aclass\
Name:\x22params\x22,be\
gin:/\x5c(/,end:/\x5c)\
/,keywords:p,rel\
evance:0,\x0acontai\
ns:[a,e.C_BLOCK_\
COMMENT_MODE,c,o\
,s,{begin:/\x5c(/,e\
nd:/\x5c)/,keywords\
:p,\x0arelevance:0,\
contains:[\x22self\x22\
,a,e.C_BLOCK_COM\
MENT_MODE,c,o,s]\
}]\x0a},s,a,e.C_BLO\
CK_COMMENT_MODE,\
l]};return{name:\
\x22C++\x22,\x0aaliases:[\
\x22cc\x22,\x22c++\x22,\x22h++\x22\
,\x22hpp\x22,\x22hh\x22,\x22hxx\
\x22,\x22cxx\x22],keyword\
s:p,illegal:\x22</\x22\
,\x0aclassNameAlias\
es:{\x22function.di\
spatch\x22:\x22built_i\
n\x22},\x0acontains:[]\
.concat(f,g,_,m,\
[l,{\x0abegin:\x22\x5c\x5cb(\
deque|list|queue\
|priority_queue|\
pair|stack|vecto\
r|map|set|bitset\
|multiset|multim\
ap|unordered_map\
|unordered_set|u\
nordered_multise\
t|unordered_mult\
imap|array|tuple\
|optional|varian\
t|function)\x5c\x5cs*<\
(?!<)\x22,\x0aend:\x22>\x22,\
keywords:p,conta\
ins:[\x22self\x22,s]},\
{begin:e.IDENT_R\
E+\x22::\x22,keywords:\
p},{\x0amatch:[/\x5cb(\
?:enum(?:\x5cs+(?:c\
lass|struct))?|c\
lass|struct|unio\
n)/,/\x5cs+/,/\x5cw+/]\
,\x0aclassName:{1:\x22\
keyword\x22,3:\x22titl\
e.class\x22}}])}}})\
();hljs.register\
Language(\x22cpp\x22,e\
)\x0a})();/*! `crms\
h` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>{\x0aconst\
 t=\x22group clone \
ms master locati\
on colocation or\
der fencing_topo\
logy rsc_ticket \
acl_target acl_g\
roup user role t\
ag xml\x22\x0a;return{\
name:\x22crmsh\x22,ali\
ases:[\x22crm\x22,\x22pcm\
k\x22],case_insensi\
tive:!0,keywords\
:{\x0akeyword:\x22para\
ms meta operatio\
ns op rule attri\
butes utilizatio\
n read write den\
y defined not_de\
fined in_range d\
ate spec in ref \
reference attrib\
ute type xpath v\
ersion and or lt\
 gt tag lte gte \
eq ne \x5c\x5c number \
string\x22,\x0aliteral\
:\x22Master Started\
 Slave Stopped s\
tart promote dem\
ote stop monitor\
 true false\x22\x0a},c\
ontains:[e.HASH_\
COMMENT_MODE,{be\
ginKeywords:\x22nod\
e\x22,starts:{\x0aend:\
\x22\x5c\x5cs*([\x5c\x5cw_-]+:)\
?\x22,starts:{class\
Name:\x22title\x22,end\
:\x22\x5c\x5cs*[\x5c\x5c$\x5c\x5cw_][\
\x5c\x5cw_-]*\x22}}\x0a},{be\
ginKeywords:\x22pri\
mitive rsc_templ\
ate\x22,starts:{cla\
ssName:\x22title\x22,\x0a\
end:\x22\x5c\x5cs*[\x5c\x5c$\x5c\x5cw\
_][\x5c\x5cw_-]*\x22,star\
ts:{end:\x22\x5c\x5cs*@?[\
\x5c\x5cw_][\x5c\x5cw_\x5c\x5c.:-]\
*\x22}}},{\x0abegin:\x22\x5c\
\x5cb(\x22+t.split(\x22 \x22\
).join(\x22|\x22)+\x22)\x5c\x5c\
s+\x22,keywords:t,s\
tarts:{\x0aclassNam\
e:\x22title\x22,end:\x22[\
\x5c\x5c$\x5c\x5cw_][\x5c\x5cw_-]*\
\x22}},{\x0abeginKeywo\
rds:\x22property rs\
c_defaults op_de\
faults\x22,starts:{\
className:\x22title\
\x22,\x0aend:\x22\x5c\x5cs*([\x5c\x5c\
w_-]+:)?\x22}},e.QU\
OTE_STRING_MODE,\
{className:\x22meta\
\x22,\x0abegin:\x22(ocf|s\
ystemd|service|l\
sb):[\x5c\x5cw_:-]+\x22,r\
elevance:0},{cla\
ssName:\x22number\x22,\
\x0abegin:\x22\x5c\x5cb\x5c\x5cd+(\
\x5c\x5c.\x5c\x5cd+)?(ms|s|h\
|m)?\x22,relevance:\
0},{className:\x22l\
iteral\x22,\x0abegin:\x22\
[-]?(infinity|in\
f)\x22,relevance:0}\
,{className:\x22att\
r\x22,\x0abegin:/([A-Z\
a-z$_#][\x5cw_-]+)=\
/,relevance:0},{\
className:\x22tag\x22,\
begin:\x22</?\x22,\x0aend\
:\x22/?>\x22,relevance\
:0}]}}})();hljs.\
registerLanguage\
(\x22crmsh\x22,e)})();\
/*! `crystal` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>{\x0aconst n=\x22(\
_?[ui](8|16|32|6\
4|128))?\x22,i=\x22[a-\
zA-Z_]\x5c\x5cw*[!?=]?\
|[-+~]@|<<|>>|[=\
!]~|===?|<=>|[<>\
]=?|\x5c\x5c*\x5c\x5c*|[-/+%\
^&*~|]|//|//=|&[\
-+*]=?|&\x5c\x5c*\x5c\x5c*|\x5c\
\x5c[\x5c\x5c][=?]?\x22,s=\x22[\
A-Za-z_]\x5c\x5cw*(::\x5c\
\x5cw+)*(\x5c\x5c?|!)?\x22,a\
={\x0a$pattern:\x22[a-\
zA-Z_]\x5c\x5cw*[!?=]?\
\x22,\x0akeyword:\x22abst\
ract alias annot\
ation as as? asm\
 begin break cas\
e class def do e\
lse elsif end en\
sure enum extend\
 for fun if incl\
ude instance_siz\
eof is_a? lib ma\
cro module next \
nil? of out poin\
terof private pr\
otected rescue r\
esponds_to? retu\
rn require selec\
t self sizeof st\
ruct super then \
type typeof unio\
n uninitialized \
unless until ver\
batim when while\
 with yield __DI\
R__ __END_LINE__\
 __FILE__ __LINE\
__\x22,\x0aliteral:\x22fa\
lse nil true\x22},t\
={className:\x22sub\
st\x22,begin:/#\x5c{/,\
end:/\x5c}/,keyword\
s:a\x0a},c={classNa\
me:\x22template-var\
iable\x22,variants:\
[{begin:\x22\x5c\x5c{\x5c\x5c{\x22\
,end:\x22\x5c\x5c}\x5c\x5c}\x22},{\
\x0abegin:\x22\x5c\x5c{%\x22,en\
d:\x22%\x5c\x5c}\x22}],keywo\
rds:a};function \
r(e,n){const i=[\
{begin:e,end:n}]\
\x0a;return i[0].co\
ntains=i,i}const\
 l={className:\x22s\
tring\x22,\x0acontains\
:[e.BACKSLASH_ES\
CAPE,t],variants\
:[{begin:/'/,end\
:/'/},{begin:/\x22/\
,end:/\x22/\x0a},{begi\
n:/`/,end:/`/},{\
begin:\x22%[Qwi]?\x5c\x5c\
(\x22,end:\x22\x5c\x5c)\x22,con\
tains:r(\x22\x5c\x5c(\x22,\x22\x5c\
\x5c)\x22)},{\x0abegin:\x22%\
[Qwi]?\x5c\x5c[\x22,end:\x22\
\x5c\x5c]\x22,contains:r(\
\x22\x5c\x5c[\x22,\x22\x5c\x5c]\x22)},{b\
egin:\x22%[Qwi]?\x5c\x5c{\
\x22,\x0aend:/\x5c}/,cont\
ains:r(/\x5c{/,/\x5c}/\
)},{begin:\x22%[Qwi\
]?<\x22,end:\x22>\x22,con\
tains:r(\x22<\x22,\x22>\x22)\
},{\x0abegin:\x22%[Qwi\
]?\x5c\x5c|\x22,end:\x22\x5c\x5c|\x22\
},{begin:/<<-\x5cw+\
$/,end:/^\x5cs*\x5cw+$\
/}],relevance:0}\
,b={\x0aclassName:\x22\
string\x22,variants\
:[{begin:\x22%q\x5c\x5c(\x22\
,end:\x22\x5c\x5c)\x22,conta\
ins:r(\x22\x5c\x5c(\x22,\x22\x5c\x5c)\
\x22)},{\x0abegin:\x22%q\x5c\
\x5c[\x22,end:\x22\x5c\x5c]\x22,co\
ntains:r(\x22\x5c\x5c[\x22,\x22\
\x5c\x5c]\x22)},{begin:\x22%\
q\x5c\x5c{\x22,end:/\x5c}/,\x0a\
contains:r(/\x5c{/,\
/\x5c}/)},{begin:\x22%\
q<\x22,end:\x22>\x22,cont\
ains:r(\x22<\x22,\x22>\x22)}\
,{begin:\x22%q\x5c\x5c|\x22,\
\x0aend:\x22\x5c\x5c|\x22},{beg\
in:/<<-'\x5cw+'$/,e\
nd:/^\x5cs*\x5cw+$/}],\
relevance:0},o={\
\x0abegin:\x22(?!%\x5c\x5c})\
(\x22+e.RE_STARTERS\
_RE+\x22|\x5c\x5cn|\x5c\x5cb(ca\
se|if|select|unl\
ess|until|when|w\
hile)\x5c\x5cb)\x5c\x5cs*\x22,\x0a\
keywords:\x22case i\
f select unless \
until when while\
\x22,contains:[{cla\
ssName:\x22regexp\x22,\
\x0acontains:[e.BAC\
KSLASH_ESCAPE,t]\
,variants:[{begi\
n:\x22//[a-z]*\x22,rel\
evance:0},{\x0abegi\
n:\x22/(?!\x5c\x5c/)\x22,end\
:\x22/[a-z]*\x22}]}],r\
elevance:0},g=[c\
,l,b,{className:\
\x22regexp\x22,\x0acontai\
ns:[e.BACKSLASH_\
ESCAPE,t],varian\
ts:[{begin:\x22%r\x5c\x5c\
(\x22,end:\x22\x5c\x5c)\x22,\x0aco\
ntains:r(\x22\x5c\x5c(\x22,\x22\
\x5c\x5c)\x22)},{begin:\x22%\
r\x5c\x5c[\x22,end:\x22\x5c\x5c]\x22,\
contains:r(\x22\x5c\x5c[\x22\
,\x22\x5c\x5c]\x22)},{\x0abegin\
:\x22%r\x5c\x5c{\x22,end:/\x5c}\
/,contains:r(/\x5c{\
/,/\x5c}/)},{begin:\
\x22%r<\x22,end:\x22>\x22,\x0ac\
ontains:r(\x22<\x22,\x22>\
\x22)},{begin:\x22%r\x5c\x5c\
|\x22,end:\x22\x5c\x5c|\x22}],r\
elevance:0},o,{\x0a\
className:\x22meta\x22\
,begin:\x22@\x5c\x5c[\x22,en\
d:\x22\x5c\x5c]\x22,\x0acontain\
s:[e.inherit(e.Q\
UOTE_STRING_MODE\
,{className:\x22str\
ing\x22})]},{\x0aclass\
Name:\x22variable\x22,\
\x0abegin:\x22(\x5c\x5c$\x5c\x5cW)\
|((\x5c\x5c$|@@?)(\x5c\x5cw+\
))(?=[^@$?])(?![\
A-Za-z])(?![@$?'\
])\x22\x0a},e.HASH_COM\
MENT_MODE,{class\
Name:\x22class\x22,beg\
inKeywords:\x22clas\
s module struct\x22\
,\x0aend:\x22$|;\x22,ille\
gal:/=/,contains\
:[e.HASH_COMMENT\
_MODE,e.inherit(\
e.TITLE_MODE,{\x0ab\
egin:s}),{begin:\
\x22<\x22}]},{classNam\
e:\x22class\x22,beginK\
eywords:\x22lib enu\
m union\x22,\x0aend:\x22$\
|;\x22,illegal:/=/,\
contains:[e.HASH\
_COMMENT_MODE,e.\
inherit(e.TITLE_\
MODE,{\x0abegin:s})\
]},{beginKeyword\
s:\x22annotation\x22,e\
nd:\x22$|;\x22,illegal\
:/=/,\x0acontains:[\
e.HASH_COMMENT_M\
ODE,e.inherit(e.\
TITLE_MODE,{begi\
n:s})],relevance\
:2},{\x0aclassName:\
\x22function\x22,begin\
Keywords:\x22def\x22,e\
nd:/\x5cB\x5cb/,\x0aconta\
ins:[e.inherit(e\
.TITLE_MODE,{beg\
in:i,endsParent:\
!0})]},{\x0aclassNa\
me:\x22function\x22,be\
ginKeywords:\x22fun\
 macro\x22,end:/\x5cB\x5c\
b/,\x0acontains:[e.\
inherit(e.TITLE_\
MODE,{begin:i,en\
dsParent:!0})],r\
elevance:2},{\x0acl\
assName:\x22symbol\x22\
,begin:e.UNDERSC\
ORE_IDENT_RE+\x22(!\
|\x5c\x5c?)?:\x22,relevan\
ce:0},{\x0aclassNam\
e:\x22symbol\x22,begin\
:\x22:\x22,contains:[l\
,{begin:i}],rele\
vance:0},{\x0aclass\
Name:\x22number\x22,va\
riants:[{begin:\x22\
\x5c\x5cb0b([01_]+)\x22+n\
},{begin:\x22\x5c\x5cb0o(\
[0-7_]+)\x22+n\x0a},{b\
egin:\x22\x5c\x5cb0x([A-F\
a-f0-9_]+)\x22+n},{\
\x0abegin:\x22\x5c\x5cb([1-9\
][0-9_]*[0-9]|[0\
-9])(\x5c\x5c.[0-9][0-\
9_]*)?([eE]_?[-+\
]?[0-9_]*)?(_?f(\
32|64))?(?!_)\x22\x0a}\
,{begin:\x22\x5c\x5cb([1-\
9][0-9_]*|0)\x22+n}\
],relevance:0}]\x0a\
;return t.contai\
ns=g,c.contains=\
g.slice(1),{name\
:\x22Crystal\x22,alias\
es:[\x22cr\x22],\x0akeywo\
rds:a,contains:g\
}}})();hljs.regi\
sterLanguage(\x22cr\
ystal\x22,e)})();/*\
! `csharp` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{const n={\x0akeyw\
ord:[\x22abstract\x22,\
\x22as\x22,\x22base\x22,\x22bre\
ak\x22,\x22case\x22,\x22catc\
h\x22,\x22class\x22,\x22cons\
t\x22,\x22continue\x22,\x22d\
o\x22,\x22else\x22,\x22event\
\x22,\x22explicit\x22,\x22ex\
tern\x22,\x22finally\x22,\
\x22fixed\x22,\x22for\x22,\x22f\
oreach\x22,\x22goto\x22,\x22\
if\x22,\x22implicit\x22,\x22\
in\x22,\x22interface\x22,\
\x22internal\x22,\x22is\x22,\
\x22lock\x22,\x22namespac\
e\x22,\x22new\x22,\x22operat\
or\x22,\x22out\x22,\x22overr\
ide\x22,\x22params\x22,\x22p\
rivate\x22,\x22protect\
ed\x22,\x22public\x22,\x22re\
adonly\x22,\x22record\x22\
,\x22ref\x22,\x22return\x22,\
\x22scoped\x22,\x22sealed\
\x22,\x22sizeof\x22,\x22stac\
kalloc\x22,\x22static\x22\
,\x22struct\x22,\x22switc\
h\x22,\x22this\x22,\x22throw\
\x22,\x22try\x22,\x22typeof\x22\
,\x22unchecked\x22,\x22un\
safe\x22,\x22using\x22,\x22v\
irtual\x22,\x22void\x22,\x22\
volatile\x22,\x22while\
\x22].concat([\x22add\x22\
,\x22alias\x22,\x22and\x22,\x22\
ascending\x22,\x22asyn\
c\x22,\x22await\x22,\x22by\x22,\
\x22descending\x22,\x22eq\
uals\x22,\x22from\x22,\x22ge\
t\x22,\x22global\x22,\x22gro\
up\x22,\x22init\x22,\x22into\
\x22,\x22join\x22,\x22let\x22,\x22\
nameof\x22,\x22not\x22,\x22n\
otnull\x22,\x22on\x22,\x22or\
\x22,\x22orderby\x22,\x22par\
tial\x22,\x22remove\x22,\x22\
select\x22,\x22set\x22,\x22u\
nmanaged\x22,\x22value\
|0\x22,\x22var\x22,\x22when\x22\
,\x22where\x22,\x22with\x22,\
\x22yield\x22]),\x0abuilt\
_in:[\x22bool\x22,\x22byt\
e\x22,\x22char\x22,\x22decim\
al\x22,\x22delegate\x22,\x22\
double\x22,\x22dynamic\
\x22,\x22enum\x22,\x22float\x22\
,\x22int\x22,\x22long\x22,\x22n\
int\x22,\x22nuint\x22,\x22ob\
ject\x22,\x22sbyte\x22,\x22s\
hort\x22,\x22string\x22,\x22\
ulong\x22,\x22uint\x22,\x22u\
short\x22],\x0aliteral\
:[\x22default\x22,\x22fal\
se\x22,\x22null\x22,\x22true\
\x22]},a=e.inherit(\
e.TITLE_MODE,{\x0ab\
egin:\x22[a-zA-Z](\x5c\
\x5c.?\x5c\x5cw)*\x22}),i={c\
lassName:\x22number\
\x22,variants:[{\x0abe\
gin:\x22\x5c\x5cb(0b[01']\
+)\x22},{\x0abegin:\x22(-\
?)\x5c\x5cb([\x5c\x5cd']+(\x5c\x5c\
.[\x5c\x5cd']*)?|\x5c\x5c.[\x5c\
\x5cd']+)(u|U|l|L|u\
l|UL|f|F|b|B)\x22},\
{\x0abegin:\x22(-?)(\x5c\x5c\
b0[xX][a-fA-F0-9\
']+|(\x5c\x5cb[\x5c\x5cd']+(\
\x5c\x5c.[\x5c\x5cd']*)?|\x5c\x5c.\
[\x5c\x5cd']+)([eE][-+\
]?[\x5c\x5cd']+)?)\x22\x0a}]\
,relevance:0},s=\
{className:\x22stri\
ng\x22,begin:'@\x22',e\
nd:'\x22',contains:\
[{begin:'\x22\x22'}]\x0a}\
,t=e.inherit(s,{\
illegal:/\x5cn/}),r\
={className:\x22sub\
st\x22,begin:/\x5c{/,e\
nd:/\x5c}/,\x0akeyword\
s:n},l=e.inherit\
(r,{illegal:/\x5cn/\
}),c={className:\
\x22string\x22,begin:/\
\x5c$\x22/,\x0aend:'\x22',il\
legal:/\x5cn/,conta\
ins:[{begin:/\x5c{\x5c\
{/},{begin:/\x5c}\x5c}\
/\x0a},e.BACKSLASH_\
ESCAPE,l]},o={cl\
assName:\x22string\x22\
,begin:/\x5c$@\x22/,en\
d:'\x22',contains:[\
{\x0abegin:/\x5c{\x5c{/},\
{begin:/\x5c}\x5c}/},{\
begin:'\x22\x22'},r]},\
d=e.inherit(o,{i\
llegal:/\x5cn/,\x0acon\
tains:[{begin:/\x5c\
{\x5c{/},{begin:/\x5c}\
\x5c}/},{begin:'\x22\x22'\
},l]})\x0a;r.contai\
ns=[o,c,s,e.APOS\
_STRING_MODE,e.Q\
UOTE_STRING_MODE\
,i,e.C_BLOCK_COM\
MENT_MODE],\x0al.co\
ntains=[d,c,t,e.\
APOS_STRING_MODE\
,e.QUOTE_STRING_\
MODE,i,e.inherit\
(e.C_BLOCK_COMME\
NT_MODE,{\x0aillega\
l:/\x5cn/})];const \
g={variants:[{cl\
assName:\x22string\x22\
,\x0abegin:/\x22\x22\x22(\x22*)\
(?!\x22)(.|\x5cn)*?\x22\x22\x22\
\x5c1/,relevance:1\x0a\
},o,c,s,e.APOS_S\
TRING_MODE,e.QUO\
TE_STRING_MODE]}\
,E={begin:\x22<\x22,en\
d:\x22>\x22,\x0acontains:\
[{beginKeywords:\
\x22in out\x22},a]\x0a},_\
=e.IDENT_RE+\x22(<\x22\
+e.IDENT_RE+\x22(\x5c\x5c\
s*,\x5c\x5cs*\x22+e.IDENT\
_RE+\x22)*>)?(\x5c\x5c[\x5c\x5c\
])?\x22,b={\x0abegin:\x22\
@\x22+e.IDENT_RE,re\
levance:0};retur\
n{name:\x22C#\x22,alia\
ses:[\x22cs\x22,\x22c#\x22],\
\x0akeywords:n,ille\
gal:/::/,contain\
s:[e.COMMENT(\x22//\
/\x22,\x22$\x22,{returnBe\
gin:!0,\x0acontains\
:[{className:\x22do\
ctag\x22,variants:[\
{begin:\x22///\x22,rel\
evance:0},{\x0abegi\
n:\x22\x5cx3c!--|--\x5cx3\
e\x22},{begin:\x22</?\x22\
,end:\x22>\x22}]}]\x0a}),\
e.C_LINE_COMMENT\
_MODE,e.C_BLOCK_\
COMMENT_MODE,{cl\
assName:\x22meta\x22,b\
egin:\x22#\x22,\x0aend:\x22$\
\x22,keywords:{\x0akey\
word:\x22if else el\
if endif define \
undef warning er\
ror line region \
endregion pragma\
 checksum\x22\x0a}},g,\
i,{beginKeywords\
:\x22class interfac\
e\x22,relevance:0,e\
nd:/[{;=]/,\x0aille\
gal:/[^\x5cs:,]/,co\
ntains:[{beginKe\
ywords:\x22where cl\
ass\x22\x0a},a,E,e.C_L\
INE_COMMENT_MODE\
,e.C_BLOCK_COMME\
NT_MODE]},{begin\
Keywords:\x22namesp\
ace\x22,\x0arelevance:\
0,end:/[{;=]/,il\
legal:/[^\x5cs:]/,\x0a\
contains:[a,e.C_\
LINE_COMMENT_MOD\
E,e.C_BLOCK_COMM\
ENT_MODE]},{\x0abeg\
inKeywords:\x22reco\
rd\x22,relevance:0,\
end:/[{;=]/,ille\
gal:/[^\x5cs:]/,\x0aco\
ntains:[a,E,e.C_\
LINE_COMMENT_MOD\
E,e.C_BLOCK_COMM\
ENT_MODE]},{clas\
sName:\x22meta\x22,\x0abe\
gin:\x22^\x5c\x5cs*\x5c\x5c[(?=\
[\x5c\x5cw])\x22,excludeB\
egin:!0,end:\x22\x5c\x5c]\
\x22,excludeEnd:!0,\
contains:[{\x0aclas\
sName:\x22string\x22,b\
egin:/\x22/,end:/\x22/\
}]},{\x0abeginKeywo\
rds:\x22new return \
throw await else\
\x22,relevance:0},{\
className:\x22funct\
ion\x22,\x0abegin:\x22(\x22+\
_+\x22\x5c\x5cs+)+\x22+e.IDE\
NT_RE+\x22\x5c\x5cs*(<[^=\
]+>\x5c\x5cs*)?\x5c\x5c(\x22,re\
turnBegin:!0,\x0aen\
d:/\x5cs*[{;=]/,exc\
ludeEnd:!0,keywo\
rds:n,contains:[\
{\x0abeginKeywords:\
\x22public private \
protected static\
 internal protec\
ted abstract asy\
nc extern overri\
de unsafe virtua\
l new sealed par\
tial\x22,\x0arelevance\
:0},{begin:e.IDE\
NT_RE+\x22\x5c\x5cs*(<[^=\
]+>\x5c\x5cs*)?\x5c\x5c(\x22,re\
turnBegin:!0,\x0aco\
ntains:[e.TITLE_\
MODE,E],relevanc\
e:0},{match:/\x5c(\x5c\
)/},{className:\x22\
params\x22,\x0abegin:/\
\x5c(/,end:/\x5c)/,exc\
ludeBegin:!0,exc\
ludeEnd:!0,keywo\
rds:n,relevance:\
0,\x0acontains:[g,i\
,e.C_BLOCK_COMME\
NT_MODE]\x0a},e.C_L\
INE_COMMENT_MODE\
,e.C_BLOCK_COMME\
NT_MODE]},b]}}})\
()\x0a;hljs.registe\
rLanguage(\x22cshar\
p\x22,e)})();/*! `c\
sp` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var s=(()\
=>{\x22use strict\x22;\
return s=>({name\
:\x22CSP\x22,case_inse\
nsitive:!1,\x0akeyw\
ords:{$pattern:\x22\
[a-zA-Z][a-zA-Z0\
-9_-]*\x22,\x0akeyword\
:[\x22base-uri\x22,\x22ch\
ild-src\x22,\x22connec\
t-src\x22,\x22default-\
src\x22,\x22font-src\x22,\
\x22form-action\x22,\x22f\
rame-ancestors\x22,\
\x22frame-src\x22,\x22img\
-src\x22,\x22manifest-\
src\x22,\x22media-src\x22\
,\x22object-src\x22,\x22p\
lugin-types\x22,\x22re\
port-uri\x22,\x22sandb\
ox\x22,\x22script-src\x22\
,\x22style-src\x22,\x22tr\
usted-types\x22,\x22un\
safe-hashes\x22,\x22wo\
rker-src\x22]\x0a},con\
tains:[{classNam\
e:\x22string\x22,begin\
:\x22'\x22,end:\x22'\x22},{c\
lassName:\x22attrib\
ute\x22,\x0abegin:\x22^Co\
ntent\x22,end:\x22:\x22,e\
xcludeEnd:!0}]})\
})();hljs.regist\
erLanguage(\x22csp\x22\
,s)\x0a})();/*! `cs\
s` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22\x0a;\
const e=[\x22a\x22,\x22ab\
br\x22,\x22address\x22,\x22a\
rticle\x22,\x22aside\x22,\
\x22audio\x22,\x22b\x22,\x22blo\
ckquote\x22,\x22body\x22,\
\x22button\x22,\x22canvas\
\x22,\x22caption\x22,\x22cit\
e\x22,\x22code\x22,\x22dd\x22,\x22\
del\x22,\x22details\x22,\x22\
dfn\x22,\x22div\x22,\x22dl\x22,\
\x22dt\x22,\x22em\x22,\x22field\
set\x22,\x22figcaption\
\x22,\x22figure\x22,\x22foot\
er\x22,\x22form\x22,\x22h1\x22,\
\x22h2\x22,\x22h3\x22,\x22h4\x22,\x22\
h5\x22,\x22h6\x22,\x22header\
\x22,\x22hgroup\x22,\x22html\
\x22,\x22i\x22,\x22iframe\x22,\x22\
img\x22,\x22input\x22,\x22in\
s\x22,\x22kbd\x22,\x22label\x22\
,\x22legend\x22,\x22li\x22,\x22\
main\x22,\x22mark\x22,\x22me\
nu\x22,\x22nav\x22,\x22objec\
t\x22,\x22ol\x22,\x22optgrou\
p\x22,\x22option\x22,\x22p\x22,\
\x22picture\x22,\x22q\x22,\x22q\
uote\x22,\x22samp\x22,\x22se\
ction\x22,\x22select\x22,\
\x22source\x22,\x22span\x22,\
\x22strong\x22,\x22summar\
y\x22,\x22sup\x22,\x22table\x22\
,\x22tbody\x22,\x22td\x22,\x22t\
extarea\x22,\x22tfoot\x22\
,\x22th\x22,\x22thead\x22,\x22t\
ime\x22,\x22tr\x22,\x22ul\x22,\x22\
var\x22,\x22video\x22,\x22de\
fs\x22,\x22g\x22,\x22marker\x22\
,\x22mask\x22,\x22pattern\
\x22,\x22svg\x22,\x22switch\x22\
,\x22symbol\x22,\x22feBle\
nd\x22,\x22feColorMatr\
ix\x22,\x22feComponent\
Transfer\x22,\x22feCom\
posite\x22,\x22feConvo\
lveMatrix\x22,\x22feDi\
ffuseLighting\x22,\x22\
feDisplacementMa\
p\x22,\x22feFlood\x22,\x22fe\
GaussianBlur\x22,\x22f\
eImage\x22,\x22feMerge\
\x22,\x22feMorphology\x22\
,\x22feOffset\x22,\x22feS\
pecularLighting\x22\
,\x22feTile\x22,\x22feTur\
bulence\x22,\x22linear\
Gradient\x22,\x22radia\
lGradient\x22,\x22stop\
\x22,\x22circle\x22,\x22elli\
pse\x22,\x22image\x22,\x22li\
ne\x22,\x22path\x22,\x22poly\
gon\x22,\x22polyline\x22,\
\x22rect\x22,\x22text\x22,\x22u\
se\x22,\x22textPath\x22,\x22\
tspan\x22,\x22foreignO\
bject\x22,\x22clipPath\
\x22],r=[\x22any-hover\
\x22,\x22any-pointer\x22,\
\x22aspect-ratio\x22,\x22\
color\x22,\x22color-ga\
mut\x22,\x22color-inde\
x\x22,\x22device-aspec\
t-ratio\x22,\x22device\
-height\x22,\x22device\
-width\x22,\x22display\
-mode\x22,\x22forced-c\
olors\x22,\x22grid\x22,\x22h\
eight\x22,\x22hover\x22,\x22\
inverted-colors\x22\
,\x22monochrome\x22,\x22o\
rientation\x22,\x22ove\
rflow-block\x22,\x22ov\
erflow-inline\x22,\x22\
pointer\x22,\x22prefer\
s-color-scheme\x22,\
\x22prefers-contras\
t\x22,\x22prefers-redu\
ced-motion\x22,\x22pre\
fers-reduced-tra\
nsparency\x22,\x22reso\
lution\x22,\x22scan\x22,\x22\
scripting\x22,\x22upda\
te\x22,\x22width\x22,\x22min\
-width\x22,\x22max-wid\
th\x22,\x22min-height\x22\
,\x22max-height\x22].s\
ort().reverse(),\
t=[\x22active\x22,\x22any\
-link\x22,\x22blank\x22,\x22\
checked\x22,\x22curren\
t\x22,\x22default\x22,\x22de\
fined\x22,\x22dir\x22,\x22di\
sabled\x22,\x22drop\x22,\x22\
empty\x22,\x22enabled\x22\
,\x22first\x22,\x22first-\
child\x22,\x22first-of\
-type\x22,\x22fullscre\
en\x22,\x22future\x22,\x22fo\
cus\x22,\x22focus-visi\
ble\x22,\x22focus-with\
in\x22,\x22has\x22,\x22host\x22\
,\x22host-context\x22,\
\x22hover\x22,\x22indeter\
minate\x22,\x22in-rang\
e\x22,\x22invalid\x22,\x22is\
\x22,\x22lang\x22,\x22last-c\
hild\x22,\x22last-of-t\
ype\x22,\x22left\x22,\x22lin\
k\x22,\x22local-link\x22,\
\x22not\x22,\x22nth-child\
\x22,\x22nth-col\x22,\x22nth\
-last-child\x22,\x22nt\
h-last-col\x22,\x22nth\
-last-of-type\x22,\x22\
nth-of-type\x22,\x22on\
ly-child\x22,\x22only-\
of-type\x22,\x22option\
al\x22,\x22out-of-rang\
e\x22,\x22past\x22,\x22place\
holder-shown\x22,\x22r\
ead-only\x22,\x22read-\
write\x22,\x22required\
\x22,\x22right\x22,\x22root\x22\
,\x22scope\x22,\x22target\
\x22,\x22target-within\
\x22,\x22user-invalid\x22\
,\x22valid\x22,\x22visite\
d\x22,\x22where\x22].sort\
().reverse(),i=[\
\x22after\x22,\x22backdro\
p\x22,\x22before\x22,\x22cue\
\x22,\x22cue-region\x22,\x22\
first-letter\x22,\x22f\
irst-line\x22,\x22gram\
mar-error\x22,\x22mark\
er\x22,\x22part\x22,\x22plac\
eholder\x22,\x22select\
ion\x22,\x22slotted\x22,\x22\
spelling-error\x22]\
.sort().reverse(\
),o=[\x22accent-col\
or\x22,\x22align-conte\
nt\x22,\x22align-items\
\x22,\x22align-self\x22,\x22\
alignment-baseli\
ne\x22,\x22all\x22,\x22anima\
tion\x22,\x22animation\
-delay\x22,\x22animati\
on-direction\x22,\x22a\
nimation-duratio\
n\x22,\x22animation-fi\
ll-mode\x22,\x22animat\
ion-iteration-co\
unt\x22,\x22animation-\
name\x22,\x22animation\
-play-state\x22,\x22an\
imation-timing-f\
unction\x22,\x22appear\
ance\x22,\x22backface-\
visibility\x22,\x22bac\
kground\x22,\x22backgr\
ound-attachment\x22\
,\x22background-ble\
nd-mode\x22,\x22backgr\
ound-clip\x22,\x22back\
ground-color\x22,\x22b\
ackground-image\x22\
,\x22background-ori\
gin\x22,\x22background\
-position\x22,\x22back\
ground-repeat\x22,\x22\
background-size\x22\
,\x22baseline-shift\
\x22,\x22block-size\x22,\x22\
border\x22,\x22border-\
block\x22,\x22border-b\
lock-color\x22,\x22bor\
der-block-end\x22,\x22\
border-block-end\
-color\x22,\x22border-\
block-end-style\x22\
,\x22border-block-e\
nd-width\x22,\x22borde\
r-block-start\x22,\x22\
border-block-sta\
rt-color\x22,\x22borde\
r-block-start-st\
yle\x22,\x22border-blo\
ck-start-width\x22,\
\x22border-block-st\
yle\x22,\x22border-blo\
ck-width\x22,\x22borde\
r-bottom\x22,\x22borde\
r-bottom-color\x22,\
\x22border-bottom-l\
eft-radius\x22,\x22bor\
der-bottom-right\
-radius\x22,\x22border\
-bottom-style\x22,\x22\
border-bottom-wi\
dth\x22,\x22border-col\
lapse\x22,\x22border-c\
olor\x22,\x22border-im\
age\x22,\x22border-ima\
ge-outset\x22,\x22bord\
er-image-repeat\x22\
,\x22border-image-s\
lice\x22,\x22border-im\
age-source\x22,\x22bor\
der-image-width\x22\
,\x22border-inline\x22\
,\x22border-inline-\
color\x22,\x22border-i\
nline-end\x22,\x22bord\
er-inline-end-co\
lor\x22,\x22border-inl\
ine-end-style\x22,\x22\
border-inline-en\
d-width\x22,\x22border\
-inline-start\x22,\x22\
border-inline-st\
art-color\x22,\x22bord\
er-inline-start-\
style\x22,\x22border-i\
nline-start-widt\
h\x22,\x22border-inlin\
e-style\x22,\x22border\
-inline-width\x22,\x22\
border-left\x22,\x22bo\
rder-left-color\x22\
,\x22border-left-st\
yle\x22,\x22border-lef\
t-width\x22,\x22border\
-radius\x22,\x22border\
-right\x22,\x22border-\
end-end-radius\x22,\
\x22border-end-star\
t-radius\x22,\x22borde\
r-right-color\x22,\x22\
border-right-sty\
le\x22,\x22border-righ\
t-width\x22,\x22border\
-spacing\x22,\x22borde\
r-start-end-radi\
us\x22,\x22border-star\
t-start-radius\x22,\
\x22border-style\x22,\x22\
border-top\x22,\x22bor\
der-top-color\x22,\x22\
border-top-left-\
radius\x22,\x22border-\
top-right-radius\
\x22,\x22border-top-st\
yle\x22,\x22border-top\
-width\x22,\x22border-\
width\x22,\x22bottom\x22,\
\x22box-decoration-\
break\x22,\x22box-shad\
ow\x22,\x22box-sizing\x22\
,\x22break-after\x22,\x22\
break-before\x22,\x22b\
reak-inside\x22,\x22cx\
\x22,\x22cy\x22,\x22caption-\
side\x22,\x22caret-col\
or\x22,\x22clear\x22,\x22cli\
p\x22,\x22clip-path\x22,\x22\
clip-rule\x22,\x22colo\
r\x22,\x22color-interp\
olation\x22,\x22color-\
interpolation-fi\
lters\x22,\x22color-pr\
ofile\x22,\x22color-re\
ndering\x22,\x22color-\
scheme\x22,\x22column-\
count\x22,\x22column-f\
ill\x22,\x22column-gap\
\x22,\x22column-rule\x22,\
\x22column-rule-col\
or\x22,\x22column-rule\
-style\x22,\x22column-\
rule-width\x22,\x22col\
umn-span\x22,\x22colum\
n-width\x22,\x22column\
s\x22,\x22contain\x22,\x22co\
ntent\x22,\x22content-\
visibility\x22,\x22cou\
nter-increment\x22,\
\x22counter-reset\x22,\
\x22cue\x22,\x22cue-after\
\x22,\x22cue-before\x22,\x22\
cursor\x22,\x22directi\
on\x22,\x22display\x22,\x22d\
ominant-baseline\
\x22,\x22empty-cells\x22,\
\x22enable-backgrou\
nd\x22,\x22fill\x22,\x22fill\
-opacity\x22,\x22fill-\
rule\x22,\x22filter\x22,\x22\
flex\x22,\x22flex-basi\
s\x22,\x22flex-directi\
on\x22,\x22flex-flow\x22,\
\x22flex-grow\x22,\x22fle\
x-shrink\x22,\x22flex-\
wrap\x22,\x22float\x22,\x22f\
low\x22,\x22flood-colo\
r\x22,\x22flood-opacit\
y\x22,\x22font\x22,\x22font-\
display\x22,\x22font-f\
amily\x22,\x22font-fea\
ture-settings\x22,\x22\
font-kerning\x22,\x22f\
ont-language-ove\
rride\x22,\x22font-siz\
e\x22,\x22font-size-ad\
just\x22,\x22font-smoo\
thing\x22,\x22font-str\
etch\x22,\x22font-styl\
e\x22,\x22font-synthes\
is\x22,\x22font-varian\
t\x22,\x22font-variant\
-caps\x22,\x22font-var\
iant-east-asian\x22\
,\x22font-variant-l\
igatures\x22,\x22font-\
variant-numeric\x22\
,\x22font-variant-p\
osition\x22,\x22font-v\
ariation-setting\
s\x22,\x22font-weight\x22\
,\x22gap\x22,\x22glyph-or\
ientation-horizo\
ntal\x22,\x22glyph-ori\
entation-vertica\
l\x22,\x22grid\x22,\x22grid-\
area\x22,\x22grid-auto\
-columns\x22,\x22grid-\
auto-flow\x22,\x22grid\
-auto-rows\x22,\x22gri\
d-column\x22,\x22grid-\
column-end\x22,\x22gri\
d-column-start\x22,\
\x22grid-gap\x22,\x22grid\
-row\x22,\x22grid-row-\
end\x22,\x22grid-row-s\
tart\x22,\x22grid-temp\
late\x22,\x22grid-temp\
late-areas\x22,\x22gri\
d-template-colum\
ns\x22,\x22grid-templa\
te-rows\x22,\x22hangin\
g-punctuation\x22,\x22\
height\x22,\x22hyphens\
\x22,\x22icon\x22,\x22image-\
orientation\x22,\x22im\
age-rendering\x22,\x22\
image-resolution\
\x22,\x22ime-mode\x22,\x22in\
line-size\x22,\x22inse\
t\x22,\x22inset-block\x22\
,\x22inset-block-en\
d\x22,\x22inset-block-\
start\x22,\x22inset-in\
line\x22,\x22inset-inl\
ine-end\x22,\x22inset-\
inline-start\x22,\x22i\
solation\x22,\x22kerni\
ng\x22,\x22justify-con\
tent\x22,\x22justify-i\
tems\x22,\x22justify-s\
elf\x22,\x22left\x22,\x22let\
ter-spacing\x22,\x22li\
ghting-color\x22,\x22l\
ine-break\x22,\x22line\
-height\x22,\x22list-s\
tyle\x22,\x22list-styl\
e-image\x22,\x22list-s\
tyle-position\x22,\x22\
list-style-type\x22\
,\x22marker\x22,\x22marke\
r-end\x22,\x22marker-m\
id\x22,\x22marker-star\
t\x22,\x22mask\x22,\x22margi\
n\x22,\x22margin-block\
\x22,\x22margin-block-\
end\x22,\x22margin-blo\
ck-start\x22,\x22margi\
n-bottom\x22,\x22margi\
n-inline\x22,\x22margi\
n-inline-end\x22,\x22m\
argin-inline-sta\
rt\x22,\x22margin-left\
\x22,\x22margin-right\x22\
,\x22margin-top\x22,\x22m\
arks\x22,\x22mask\x22,\x22ma\
sk-border\x22,\x22mask\
-border-mode\x22,\x22m\
ask-border-outse\
t\x22,\x22mask-border-\
repeat\x22,\x22mask-bo\
rder-slice\x22,\x22mas\
k-border-source\x22\
,\x22mask-border-wi\
dth\x22,\x22mask-clip\x22\
,\x22mask-composite\
\x22,\x22mask-image\x22,\x22\
mask-mode\x22,\x22mask\
-origin\x22,\x22mask-p\
osition\x22,\x22mask-r\
epeat\x22,\x22mask-siz\
e\x22,\x22mask-type\x22,\x22\
max-block-size\x22,\
\x22max-height\x22,\x22ma\
x-inline-size\x22,\x22\
max-width\x22,\x22min-\
block-size\x22,\x22min\
-height\x22,\x22min-in\
line-size\x22,\x22min-\
width\x22,\x22mix-blen\
d-mode\x22,\x22nav-dow\
n\x22,\x22nav-index\x22,\x22\
nav-left\x22,\x22nav-r\
ight\x22,\x22nav-up\x22,\x22\
none\x22,\x22normal\x22,\x22\
object-fit\x22,\x22obj\
ect-position\x22,\x22o\
pacity\x22,\x22order\x22,\
\x22orphans\x22,\x22outli\
ne\x22,\x22outline-col\
or\x22,\x22outline-off\
set\x22,\x22outline-st\
yle\x22,\x22outline-wi\
dth\x22,\x22overflow\x22,\
\x22overflow-wrap\x22,\
\x22overflow-x\x22,\x22ov\
erflow-y\x22,\x22paddi\
ng\x22,\x22padding-blo\
ck\x22,\x22padding-blo\
ck-end\x22,\x22padding\
-block-start\x22,\x22p\
adding-bottom\x22,\x22\
padding-inline\x22,\
\x22padding-inline-\
end\x22,\x22padding-in\
line-start\x22,\x22pad\
ding-left\x22,\x22padd\
ing-right\x22,\x22padd\
ing-top\x22,\x22page-b\
reak-after\x22,\x22pag\
e-break-before\x22,\
\x22page-break-insi\
de\x22,\x22pause\x22,\x22pau\
se-after\x22,\x22pause\
-before\x22,\x22perspe\
ctive\x22,\x22perspect\
ive-origin\x22,\x22poi\
nter-events\x22,\x22po\
sition\x22,\x22quotes\x22\
,\x22r\x22,\x22resize\x22,\x22r\
est\x22,\x22rest-after\
\x22,\x22rest-before\x22,\
\x22right\x22,\x22rotate\x22\
,\x22row-gap\x22,\x22scal\
e\x22,\x22scroll-margi\
n\x22,\x22scroll-margi\
n-block\x22,\x22scroll\
-margin-block-en\
d\x22,\x22scroll-margi\
n-block-start\x22,\x22\
scroll-margin-bo\
ttom\x22,\x22scroll-ma\
rgin-inline\x22,\x22sc\
roll-margin-inli\
ne-end\x22,\x22scroll-\
margin-inline-st\
art\x22,\x22scroll-mar\
gin-left\x22,\x22scrol\
l-margin-right\x22,\
\x22scroll-margin-t\
op\x22,\x22scroll-padd\
ing\x22,\x22scroll-pad\
ding-block\x22,\x22scr\
oll-padding-bloc\
k-end\x22,\x22scroll-p\
adding-block-sta\
rt\x22,\x22scroll-padd\
ing-bottom\x22,\x22scr\
oll-padding-inli\
ne\x22,\x22scroll-padd\
ing-inline-end\x22,\
\x22scroll-padding-\
inline-start\x22,\x22s\
croll-padding-le\
ft\x22,\x22scroll-padd\
ing-right\x22,\x22scro\
ll-padding-top\x22,\
\x22scroll-snap-ali\
gn\x22,\x22scroll-snap\
-stop\x22,\x22scroll-s\
nap-type\x22,\x22scrol\
lbar-color\x22,\x22scr\
ollbar-gutter\x22,\x22\
scrollbar-width\x22\
,\x22shape-image-th\
reshold\x22,\x22shape-\
margin\x22,\x22shape-o\
utside\x22,\x22shape-r\
endering\x22,\x22stop-\
color\x22,\x22stop-opa\
city\x22,\x22stroke\x22,\x22\
stroke-dasharray\
\x22,\x22stroke-dashof\
fset\x22,\x22stroke-li\
necap\x22,\x22stroke-l\
inejoin\x22,\x22stroke\
-miterlimit\x22,\x22st\
roke-opacity\x22,\x22s\
troke-width\x22,\x22sp\
eak\x22,\x22speak-as\x22,\
\x22src\x22,\x22tab-size\x22\
,\x22table-layout\x22,\
\x22text-anchor\x22,\x22t\
ext-align\x22,\x22text\
-align-all\x22,\x22tex\
t-align-last\x22,\x22t\
ext-combine-upri\
ght\x22,\x22text-decor\
ation\x22,\x22text-dec\
oration-color\x22,\x22\
text-decoration-\
line\x22,\x22text-deco\
ration-skip-ink\x22\
,\x22text-decoratio\
n-style\x22,\x22text-d\
ecoration-thickn\
ess\x22,\x22text-empha\
sis\x22,\x22text-empha\
sis-color\x22,\x22text\
-emphasis-positi\
on\x22,\x22text-emphas\
is-style\x22,\x22text-\
indent\x22,\x22text-ju\
stify\x22,\x22text-ori\
entation\x22,\x22text-\
overflow\x22,\x22text-\
rendering\x22,\x22text\
-shadow\x22,\x22text-t\
ransform\x22,\x22text-\
underline-offset\
\x22,\x22text-underlin\
e-position\x22,\x22top\
\x22,\x22transform\x22,\x22t\
ransform-box\x22,\x22t\
ransform-origin\x22\
,\x22transform-styl\
e\x22,\x22transition\x22,\
\x22transition-dela\
y\x22,\x22transition-d\
uration\x22,\x22transi\
tion-property\x22,\x22\
transition-timin\
g-function\x22,\x22tra\
nslate\x22,\x22unicode\
-bidi\x22,\x22vector-e\
ffect\x22,\x22vertical\
-align\x22,\x22visibil\
ity\x22,\x22voice-bala\
nce\x22,\x22voice-dura\
tion\x22,\x22voice-fam\
ily\x22,\x22voice-pitc\
h\x22,\x22voice-range\x22\
,\x22voice-rate\x22,\x22v\
oice-stress\x22,\x22vo\
ice-volume\x22,\x22whi\
te-space\x22,\x22widow\
s\x22,\x22width\x22,\x22will\
-change\x22,\x22word-b\
reak\x22,\x22word-spac\
ing\x22,\x22word-wrap\x22\
,\x22writing-mode\x22,\
\x22x\x22,\x22y\x22,\x22z-index\
\x22].sort().revers\
e()\x0a;return n=>{\
const a=n.regex,\
l=(e=>({IMPORTAN\
T:{scope:\x22meta\x22,\
begin:\x22!importan\
t\x22},\x0aBLOCK_COMME\
NT:e.C_BLOCK_COM\
MENT_MODE,HEXCOL\
OR:{scope:\x22numbe\
r\x22,\x0abegin:/#(([0\
-9a-fA-F]{3,4})|\
(([0-9a-fA-F]{2}\
){3,4}))\x5cb/},FUN\
CTION_DISPATCH:{\
\x0aclassName:\x22buil\
t_in\x22,begin:/[\x5cw\
-]+(?=\x5c()/},ATTR\
IBUTE_SELECTOR_M\
ODE:{\x0ascope:\x22sel\
ector-attr\x22,begi\
n:/\x5c[/,end:/\x5c]/,\
illegal:\x22$\x22,\x0acon\
tains:[e.APOS_ST\
RING_MODE,e.QUOT\
E_STRING_MODE]},\
CSS_NUMBER_MODE:\
{\x0ascope:\x22number\x22\
,\x0abegin:e.NUMBER\
_RE+\x22(%|em|ex|ch\
|rem|vw|vh|vmin|\
vmax|cm|mm|in|pt\
|pc|px|deg|grad|\
rad|turn|s|ms|Hz\
|kHz|dpi|dpcm|dp\
px)?\x22,\x0arelevance\
:0},CSS_VARIABLE\
:{className:\x22att\
r\x22,begin:/--[A-Z\
a-z_][A-Za-z0-9_\
-]*/}\x0a}))(n),s=[\
n.APOS_STRING_MO\
DE,n.QUOTE_STRIN\
G_MODE];return{n\
ame:\x22CSS\x22,\x0acase_\
insensitive:!0,i\
llegal:/[=|'\x5c$]/\
,keywords:{keyfr\
amePosition:\x22fro\
m to\x22},\x0aclassNam\
eAliases:{keyfra\
mePosition:\x22sele\
ctor-tag\x22},conta\
ins:[l.BLOCK_COM\
MENT,{\x0abegin:/-(\
webkit|moz|ms|o)\
-(?=[a-z])/},l.C\
SS_NUMBER_MODE,{\
\x0aclassName:\x22sele\
ctor-id\x22,begin:/\
#[A-Za-z0-9_-]+/\
,relevance:0},{\x0a\
className:\x22selec\
tor-class\x22,begin\
:\x22\x5c\x5c.[a-zA-Z-][a\
-zA-Z0-9_-]*\x22,re\
levance:0\x0a},l.AT\
TRIBUTE_SELECTOR\
_MODE,{className\
:\x22selector-pseud\
o\x22,variants:[{\x0ab\
egin:\x22:(\x22+t.join\
(\x22|\x22)+\x22)\x22},{begi\
n:\x22:(:)?(\x22+i.joi\
n(\x22|\x22)+\x22)\x22}]},l.\
CSS_VARIABLE,{\x0ac\
lassName:\x22attrib\
ute\x22,begin:\x22\x5c\x5cb(\
\x22+o.join(\x22|\x22)+\x22)\
\x5c\x5cb\x22},{begin:/:/\
,end:/[;}{]/,\x0aco\
ntains:[l.BLOCK_\
COMMENT,l.HEXCOL\
OR,l.IMPORTANT,l\
.CSS_NUMBER_MODE\
,...s,{\x0abegin:/(\
url|data-uri)\x5c(/\
,end:/\x5c)/,releva\
nce:0,keywords:{\
built_in:\x22url da\
ta-uri\x22\x0a},contai\
ns:[...s,{classN\
ame:\x22string\x22,beg\
in:/[^)]/,endsWi\
thParent:!0,\x0aexc\
ludeEnd:!0}]},l.\
FUNCTION_DISPATC\
H]},{begin:a.loo\
kahead(/@/),end:\
\x22[{;]\x22,\x0arelevanc\
e:0,illegal:/:/,\
contains:[{class\
Name:\x22keyword\x22,b\
egin:/@-?\x5cw[\x5cw]*\
(-\x5cw+)*/\x0a},{begi\
n:/\x5cs/,endsWithP\
arent:!0,exclude\
End:!0,relevance\
:0,keywords:{\x0a$p\
attern:/[a-z-]+/\
,keyword:\x22and or\
 not only\x22,attri\
bute:r.join(\x22 \x22)\
},contains:[{\x0abe\
gin:/[a-z-]+(?=:\
)/,className:\x22at\
tribute\x22},...s,l\
.CSS_NUMBER_MODE\
]}]},{\x0aclassName\
:\x22selector-tag\x22,\
begin:\x22\x5c\x5cb(\x22+e.j\
oin(\x22|\x22)+\x22)\x5c\x5cb\x22}\
]}}})()\x0a;hljs.re\
gisterLanguage(\x22\
css\x22,e)})();/*! \
`d` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>{const\
 a={\x0a$pattern:e.\
UNDERSCORE_IDENT\
_RE,\x0akeyword:\x22ab\
stract alias ali\
gn asm assert au\
to body break by\
te case cast cat\
ch class const c\
ontinue debug de\
fault delete dep\
recated do else \
enum export exte\
rn final finally\
 for foreach for\
each_reverse|10 \
goto if immutabl\
e import in inou\
t int interface \
invariant is laz\
y macro mixin mo\
dule new nothrow\
 out override pa\
ckage pragma pri\
vate protected p\
ublic pure ref r\
eturn scope shar\
ed static struct\
 super switch sy\
nchronized templ\
ate this throw t\
ry typedef typei\
d typeof union u\
nittest version \
void volatile wh\
ile with __FILE_\
_ __LINE__ __gsh\
ared|10 __thread\
 __traits __DATE\
__ __EOF__ __TIM\
E__ __TIMESTAMP_\
_ __VENDOR__ __V\
ERSION__\x22,\x0abuilt\
_in:\x22bool cdoubl\
e cent cfloat ch\
ar creal dchar d\
elegate double d\
string float fun\
ction idouble if\
loat ireal long \
real short strin\
g ubyte ucent ui\
nt ulong ushort \
wchar wstring\x22,\x0a\
literal:\x22false n\
ull true\x22\x0a},n=\x22(\
0|[1-9][\x5c\x5cd_]*)\x22\
,t=\x22(0|[1-9][\x5c\x5cd\
_]*|\x5c\x5cd[\x5c\x5cd_]*|[\
\x5c\x5cd_]+?\x5c\x5cd)\x22,r=\x22\
([\x5c\x5cda-fA-F][\x5c\x5cd\
a-fA-F_]*|_[\x5c\x5cda\
-fA-F][\x5c\x5cda-fA-F\
_]*)\x22,i=\x22([eE][+\
-]?\x22+t+\x22)\x22,s=\x22(\x22\
+n+\x22|0[bB][01_]+\
|0[xX]\x22+r+\x22)\x22,l=\
\x22\x5c\x5c\x5c\x5c(['\x5c\x22\x5c\x5c?\x5c\x5c\x5c\
\x5cabfnrtv]|u[\x5c\x5cdA\
-Fa-f]{4}|[0-7]{\
1,3}|x[\x5c\x5cdA-Fa-f\
]{2}|U[\x5c\x5cdA-Fa-f\
]{8})|&[a-zA-Z\x5c\x5c\
d]{2,};\x22,c={\x0acla\
ssName:\x22number\x22,\
begin:\x22\x5c\x5cb\x22+s+\x22(\
L|u|U|Lu|LU|uL|U\
L)?\x22,relevance:0\
},_={\x0aclassName:\
\x22number\x22,\x0abegin:\
\x22\x5c\x5cb(((0[xX](\x22+r\
+\x22\x5c\x5c.\x22+r+\x22|\x5c\x5c.?\x22\
+r+\x22)[pP][+-]?\x22+\
t+\x22)|(\x22+t+\x22(\x5c\x5c.\x5c\
\x5cd*|\x22+i+\x22)|\x5c\x5cd+\x5c\
\x5c.\x22+t+\x22|\x5c\x5c.\x22+n+i\
+\x22?))([fF]|L|i|[\
fF]i|Li)?|\x22+s+\x22(\
i|[fF]i|Li))\x22,\x0ar\
elevance:0},d={c\
lassName:\x22string\
\x22,begin:\x22'(\x22+l+\x22\
|.)\x22,end:\x22'\x22,ill\
egal:\x22.\x22},o={\x0acl\
assName:\x22string\x22\
,begin:'\x22',conta\
ins:[{begin:l,re\
levance:0}],end:\
'\x22[cwd]?'\x0a},u=e.\
COMMENT(\x22\x5c\x5c/\x5c\x5c+\x22\
,\x22\x5c\x5c+\x5c\x5c/\x22,{conta\
ins:[\x22self\x22],rel\
evance:10});retu\
rn{\x0aname:\x22D\x22,key\
words:a,contains\
:[e.C_LINE_COMME\
NT_MODE,e.C_BLOC\
K_COMMENT_MODE,u\
,{\x0aclassName:\x22st\
ring\x22,begin:'x\x22[\
\x5c\x5cda-fA-F\x5c\x5cs\x5c\x5cn\x5c\
\x5cr]*\x22[cwd]?',rel\
evance:10},o,{\x0ac\
lassName:\x22string\
\x22,begin:'[rq]\x22',\
end:'\x22[cwd]?',re\
levance:5},{clas\
sName:\x22string\x22,\x0a\
begin:\x22`\x22,end:\x22`\
[cwd]?\x22},{classN\
ame:\x22string\x22,beg\
in:'q\x22\x5c\x5c{',end:'\
\x5c\x5c}\x22'},_,c,d,{\x0ac\
lassName:\x22meta\x22,\
begin:\x22^#!\x22,end:\
\x22$\x22,relevance:5}\
,{className:\x22met\
a\x22,\x0abegin:\x22#(lin\
e)\x22,end:\x22$\x22,rele\
vance:5},{classN\
ame:\x22keyword\x22,\x0ab\
egin:\x22@[a-zA-Z_]\
[a-zA-Z_\x5c\x5cd]*\x22}]\
}}})();hljs.regi\
sterLanguage(\x22d\x22\
,e)})();/*! `dar\
t` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>{const \
n={className:\x22su\
bst\x22,variants:[{\
\x0abegin:\x22\x5c\x5c$[A-Za\
-z0-9_]+\x22}]},a={\
className:\x22subst\
\x22,variants:[{beg\
in:/\x5c$\x5c{/,\x0aend:/\
\x5c}/}],keywords:\x22\
true false null \
this is new supe\
r\x22},t={className\
:\x22string\x22,\x0avaria\
nts:[{begin:\x22r''\
'\x22,end:\x22'''\x22},{b\
egin:'r\x22\x22\x22',end:\
'\x22\x22\x22'},{begin:\x22r\
'\x22,end:\x22'\x22,\x0aille\
gal:\x22\x5c\x5cn\x22},{begi\
n:'r\x22',end:'\x22',i\
llegal:\x22\x5c\x5cn\x22},{b\
egin:\x22'''\x22,end:\x22\
'''\x22,\x0acontains:[\
e.BACKSLASH_ESCA\
PE,n,a]},{begin:\
'\x22\x22\x22',end:'\x22\x22\x22',\
\x0acontains:[e.BAC\
KSLASH_ESCAPE,n,\
a]},{begin:\x22'\x22,e\
nd:\x22'\x22,illegal:\x22\
\x5c\x5cn\x22,\x0acontains:[\
e.BACKSLASH_ESCA\
PE,n,a]},{begin:\
'\x22',end:'\x22',ille\
gal:\x22\x5c\x5cn\x22,\x0aconta\
ins:[e.BACKSLASH\
_ESCAPE,n,a]}]};\
a.contains=[e.C_\
NUMBER_MODE,t]\x0a;\
const i=[\x22Compar\
able\x22,\x22DateTime\x22\
,\x22Duration\x22,\x22Fun\
ction\x22,\x22Iterable\
\x22,\x22Iterator\x22,\x22Li\
st\x22,\x22Map\x22,\x22Match\
\x22,\x22Object\x22,\x22Patt\
ern\x22,\x22RegExp\x22,\x22S\
et\x22,\x22Stopwatch\x22,\
\x22String\x22,\x22String\
Buffer\x22,\x22StringS\
ink\x22,\x22Symbol\x22,\x22T\
ype\x22,\x22Uri\x22,\x22bool\
\x22,\x22double\x22,\x22int\x22\
,\x22num\x22,\x22Element\x22\
,\x22ElementList\x22],\
r=i.map((e=>e+\x22?\
\x22))\x0a;return{name\
:\x22Dart\x22,keywords\
:{\x0akeyword:[\x22abs\
tract\x22,\x22as\x22,\x22ass\
ert\x22,\x22async\x22,\x22aw\
ait\x22,\x22base\x22,\x22bre\
ak\x22,\x22case\x22,\x22catc\
h\x22,\x22class\x22,\x22cons\
t\x22,\x22continue\x22,\x22c\
ovariant\x22,\x22defau\
lt\x22,\x22deferred\x22,\x22\
do\x22,\x22dynamic\x22,\x22e\
lse\x22,\x22enum\x22,\x22exp\
ort\x22,\x22extends\x22,\x22\
extension\x22,\x22exte\
rnal\x22,\x22factory\x22,\
\x22false\x22,\x22final\x22,\
\x22finally\x22,\x22for\x22,\
\x22Function\x22,\x22get\x22\
,\x22hide\x22,\x22if\x22,\x22im\
plements\x22,\x22impor\
t\x22,\x22in\x22,\x22interfa\
ce\x22,\x22is\x22,\x22late\x22,\
\x22library\x22,\x22mixin\
\x22,\x22new\x22,\x22null\x22,\x22\
on\x22,\x22operator\x22,\x22\
part\x22,\x22required\x22\
,\x22rethrow\x22,\x22retu\
rn\x22,\x22sealed\x22,\x22se\
t\x22,\x22show\x22,\x22stati\
c\x22,\x22super\x22,\x22swit\
ch\x22,\x22sync\x22,\x22this\
\x22,\x22throw\x22,\x22true\x22\
,\x22try\x22,\x22typedef\x22\
,\x22var\x22,\x22void\x22,\x22w\
hen\x22,\x22while\x22,\x22wi\
th\x22,\x22yield\x22],\x0abu\
ilt_in:i.concat(\
r).concat([\x22Neve\
r\x22,\x22Null\x22,\x22dynam\
ic\x22,\x22print\x22,\x22doc\
ument\x22,\x22querySel\
ector\x22,\x22querySel\
ectorAll\x22,\x22windo\
w\x22]),\x0a$pattern:/\
[A-Za-z][A-Za-z0\
-9_]*\x5c??/},\x0acont\
ains:[t,e.COMMEN\
T(/\x5c/\x5c*\x5c*(?!\x5c/)/\
,/\x5c*\x5c//,{subLang\
uage:\x22markdown\x22,\
relevance:0\x0a}),e\
.COMMENT(/\x5c/{3,}\
 ?/,/$/,{contain\
s:[{subLanguage:\
\x22markdown\x22,begin\
:\x22.\x22,\x0aend:\x22$\x22,re\
levance:0}]}),e.\
C_LINE_COMMENT_M\
ODE,e.C_BLOCK_CO\
MMENT_MODE,{\x0acla\
ssName:\x22class\x22,b\
eginKeywords:\x22cl\
ass interface\x22,e\
nd:/\x5c{/,excludeE\
nd:!0,\x0acontains:\
[{beginKeywords:\
\x22extends impleme\
nts\x22},e.UNDERSCO\
RE_TITLE_MODE]\x0a}\
,e.C_NUMBER_MODE\
,{className:\x22met\
a\x22,begin:\x22@[A-Za\
-z]+\x22},{begin:\x22=\
>\x22}]}}})()\x0a;hljs\
.registerLanguag\
e(\x22dart\x22,e)})();\
/*! `delphi` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var e=(()=>{\x22use\
 strict\x22;return \
e=>{\x0aconst a=[\x22e\
xports\x22,\x22registe\
r\x22,\x22file\x22,\x22shl\x22,\
\x22array\x22,\x22record\x22\
,\x22property\x22,\x22for\
\x22,\x22mod\x22,\x22while\x22,\
\x22set\x22,\x22ally\x22,\x22la\
bel\x22,\x22uses\x22,\x22rai\
se\x22,\x22not\x22,\x22store\
d\x22,\x22class\x22,\x22safe\
call\x22,\x22var\x22,\x22int\
erface\x22,\x22or\x22,\x22pr\
ivate\x22,\x22static\x22,\
\x22exit\x22,\x22index\x22,\x22\
inherited\x22,\x22to\x22,\
\x22else\x22,\x22stdcall\x22\
,\x22override\x22,\x22shr\
\x22,\x22asm\x22,\x22far\x22,\x22r\
esourcestring\x22,\x22\
finalization\x22,\x22p\
acked\x22,\x22virtual\x22\
,\x22out\x22,\x22and\x22,\x22pr\
otected\x22,\x22librar\
y\x22,\x22do\x22,\x22xorwrit\
e\x22,\x22goto\x22,\x22near\x22\
,\x22function\x22,\x22end\
\x22,\x22div\x22,\x22overloa\
d\x22,\x22object\x22,\x22uni\
t\x22,\x22begin\x22,\x22stri\
ng\x22,\x22on\x22,\x22inline\
\x22,\x22repeat\x22,\x22unti\
l\x22,\x22destructor\x22,\
\x22write\x22,\x22message\
\x22,\x22program\x22,\x22wit\
h\x22,\x22read\x22,\x22initi\
alization\x22,\x22exce\
pt\x22,\x22default\x22,\x22n\
il\x22,\x22if\x22,\x22case\x22,\
\x22cdecl\x22,\x22in\x22,\x22do\
wnto\x22,\x22threadvar\
\x22,\x22of\x22,\x22try\x22,\x22pa\
scal\x22,\x22const\x22,\x22e\
xternal\x22,\x22constr\
uctor\x22,\x22type\x22,\x22p\
ublic\x22,\x22then\x22,\x22i\
mplementation\x22,\x22\
finally\x22,\x22publis\
hed\x22,\x22procedure\x22\
,\x22absolute\x22,\x22rei\
ntroduce\x22,\x22opera\
tor\x22,\x22as\x22,\x22is\x22,\x22\
abstract\x22,\x22alias\
\x22,\x22assembler\x22,\x22b\
itpacked\x22,\x22break\
\x22,\x22continue\x22,\x22cp\
pdecl\x22,\x22cvar\x22,\x22e\
numerator\x22,\x22expe\
rimental\x22,\x22platf\
orm\x22,\x22deprecated\
\x22,\x22unimplemented\
\x22,\x22dynamic\x22,\x22exp\
ort\x22,\x22far16\x22,\x22fo\
rward\x22,\x22generic\x22\
,\x22helper\x22,\x22imple\
ments\x22,\x22interrup\
t\x22,\x22iochecks\x22,\x22l\
ocal\x22,\x22name\x22,\x22no\
default\x22,\x22noretu\
rn\x22,\x22nostackfram\
e\x22,\x22oldfpccall\x22,\
\x22otherwise\x22,\x22sav\
eregisters\x22,\x22sof\
tfloat\x22,\x22special\
ize\x22,\x22strict\x22,\x22u\
naligned\x22,\x22varar\
gs\x22],r=[e.C_LINE\
_COMMENT_MODE,e.\
COMMENT(/\x5c{/,/\x5c}\
/,{\x0arelevance:0}\
),e.COMMENT(/\x5c(\x5c\
*/,/\x5c*\x5c)/,{relev\
ance:10})],t={cl\
assName:\x22meta\x22,\x0a\
variants:[{begin\
:/\x5c{\x5c$/,end:/\x5c}/\
},{begin:/\x5c(\x5c*\x5c$\
/,end:/\x5c*\x5c)/}]},\
n={\x0aclassName:\x22s\
tring\x22,begin:/'/\
,end:/'/,contain\
s:[{begin:/''/}]\
},s={\x0aclassName:\
\x22string\x22,variant\
s:[{match:/#\x5cd[\x5c\
d_]*/},{\x0amatch:/\
#\x5c$[\x5cdA-Fa-f][\x5cd\
A-Fa-f_]*/},{mat\
ch:/#&[0-7][0-7_\
]*/},{\x0amatch:/#%\
[01][01_]*/}]},i\
={begin:e.IDENT_\
RE+\x22\x5c\x5cs*=\x5c\x5cs*cla\
ss\x5c\x5cs*\x5c\x5c(\x22,\x0aretu\
rnBegin:!0,conta\
ins:[e.TITLE_MOD\
E]},c={className\
:\x22function\x22,\x0abeg\
inKeywords:\x22func\
tion constructor\
 destructor proc\
edure\x22,end:/[:;]\
/,\x0akeywords:\x22fun\
ction constructo\
r|10 destructor|\
10 procedure|10\x22\
,\x0acontains:[e.TI\
TLE_MODE,{classN\
ame:\x22params\x22,beg\
in:/\x5c(/,end:/\x5c)/\
,keywords:a,\x0acon\
tains:[n,s,t].co\
ncat(r)},t].conc\
at(r)};return{na\
me:\x22Delphi\x22,\x0aali\
ases:[\x22dpr\x22,\x22dfm\
\x22,\x22pas\x22,\x22pascal\x22\
],case_insensiti\
ve:!0,keywords:a\
,\x0aillegal:/\x22|\x5c$[\
G-Zg-z]|\x5c/\x5c*|<\x5c/\
|\x5c|/,contains:[n\
,s,{className:\x22n\
umber\x22,\x0arelevanc\
e:0,variants:[{m\
atch:/\x5cb\x5cd[\x5cd_]*\
(\x5c.\x5cd[\x5cd_]*)?/},\
{match:/\x5c$[\x5cdA-F\
a-f_]+/\x0a},{match\
:/\x5c$/,relevance:\
0},{match:/&[0-7\
][0-7_]*/},{matc\
h:/%[01_]+/},{ma\
tch:/%/,\x0arelevan\
ce:0}]},i,c,t].c\
oncat(r)}}})();h\
ljs.registerLang\
uage(\x22delphi\x22,e)\
})();/*! `diff` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{const a=e\
.regex;return{na\
me:\x22Diff\x22,\x0aalias\
es:[\x22patch\x22],con\
tains:[{classNam\
e:\x22meta\x22,relevan\
ce:10,\x0amatch:a.e\
ither(/^@@ +-\x5cd+\
,\x5cd+ +\x5c+\x5cd+,\x5cd+ \
+@@/,/^\x5c*\x5c*\x5c* +\x5c\
d+,\x5cd+ +\x5c*\x5c*\x5c*\x5c*\
$/,/^--- +\x5cd+,\x5cd\
+ +----$/)\x0a},{cl\
assName:\x22comment\
\x22,variants:[{\x0abe\
gin:a.either(/In\
dex: /,/^index/,\
/={3,}/,/^-{3}/,\
/^\x5c*{3} /,/^\x5c+{3\
}/,/^diff --git/\
),\x0aend:/$/},{mat\
ch:/^\x5c*{15}$/}]}\
,{className:\x22add\
ition\x22,begin:/^\x5c\
+/,end:/$/},{\x0acl\
assName:\x22deletio\
n\x22,begin:/^-/,en\
d:/$/},{classNam\
e:\x22addition\x22,beg\
in:/^!/,\x0aend:/$/\
}]}}})();hljs.re\
gisterLanguage(\x22\
diff\x22,e)})();/*!\
 `django` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
{const t={begin:\
/\x5c|[A-Za-z]+:?/,\
\x0akeywords:{\x0aname\
:\x22truncatewords \
removetags lineb\
reaksbr yesno ge\
t_digit timesinc\
e random stripta\
gs filesizeforma\
t escape linebre\
aks length_is lj\
ust rjust cut ur\
lize fix_ampersa\
nds title floatf\
ormat capfirst p\
print divisibleb\
y add make_list \
unordered_list u\
rlencode timeunt\
il urlizetrunc w\
ordcount stringf\
ormat linenumber\
s slice date dic\
tsort dictsortre\
versed default_i\
f_none pluralize\
 lower join cent\
er default trunc\
atewords_html up\
per length phone\
2numeric wordwra\
p time addslashe\
s slugify first \
escapejs force_e\
scape iriencode \
last safe safese\
q truncatechars \
localize unlocal\
ize localtime ut\
c timezone\x22\x0a},co\
ntains:[e.QUOTE_\
STRING_MODE,e.AP\
OS_STRING_MODE]}\
;return{name:\x22Dj\
ango\x22,\x0aaliases:[\
\x22jinja\x22],case_in\
sensitive:!0,sub\
Language:\x22xml\x22,\x0a\
contains:[e.COMM\
ENT(/\x5c{%\x5cs*comme\
nt\x5cs*%\x5c}/,/\x5c{%\x5cs\
*endcomment\x5cs*%\x5c\
}/),e.COMMENT(/\x5c\
{#/,/#\x5c}/),{\x0acla\
ssName:\x22template\
-tag\x22,begin:/\x5c{%\
/,end:/%\x5c}/,cont\
ains:[{className\
:\x22name\x22,\x0abegin:/\
\x5cw+/,keywords:{\x0a\
name:\x22comment en\
dcomment load te\
mplatetag ifchan\
ged endifchanged\
 if endif firsto\
f for endfor ifn\
otequal endifnot\
equal widthratio\
 extends include\
 spaceless endsp\
aceless regroup \
ifequal endifequ\
al ssi now with \
cycle url filter\
 endfilter debug\
 block endblock \
else autoescape \
endautoescape cs\
rf_token empty e\
lif endwith stat\
ic trans blocktr\
ans endblocktran\
s get_static_pre\
fix get_media_pr\
efix plural get_\
current_language\
 language get_av\
ailable_language\
s get_current_la\
nguage_bidi get_\
language_info ge\
t_language_info_\
list localize en\
dlocalize localt\
ime endlocaltime\
 timezone endtim\
ezone get_curren\
t_timezone verba\
tim\x22\x0a},starts:{e\
ndsWithParent:!0\
,keywords:\x22in by\
 as\x22,contains:[t\
],relevance:0}}]\
},{\x0aclassName:\x22t\
emplate-variable\
\x22,begin:/\x5c{\x5c{/,e\
nd:/\x5c}\x5c}/,contai\
ns:[t]}]}}})()\x0a;\
hljs.registerLan\
guage(\x22django\x22,e\
)})();/*! `dns` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var d=(()=>{\x22\
use strict\x22;retu\
rn d=>({name:\x22DN\
S Zone\x22,\x0aaliases\
:[\x22bind\x22,\x22zone\x22]\
,\x0akeywords:[\x22IN\x22\
,\x22A\x22,\x22AAAA\x22,\x22AFS\
DB\x22,\x22APL\x22,\x22CAA\x22,\
\x22CDNSKEY\x22,\x22CDS\x22,\
\x22CERT\x22,\x22CNAME\x22,\x22\
DHCID\x22,\x22DLV\x22,\x22DN\
AME\x22,\x22DNSKEY\x22,\x22D\
S\x22,\x22HIP\x22,\x22IPSECK\
EY\x22,\x22KEY\x22,\x22KX\x22,\x22\
LOC\x22,\x22MX\x22,\x22NAPTR\
\x22,\x22NS\x22,\x22NSEC\x22,\x22N\
SEC3\x22,\x22NSEC3PARA\
M\x22,\x22PTR\x22,\x22RRSIG\x22\
,\x22RP\x22,\x22SIG\x22,\x22SOA\
\x22,\x22SRV\x22,\x22SSHFP\x22,\
\x22TA\x22,\x22TKEY\x22,\x22TLS\
A\x22,\x22TSIG\x22,\x22TXT\x22]\
,\x0acontains:[d.CO\
MMENT(\x22;\x22,\x22$\x22,{r\
elevance:0}),{cl\
assName:\x22meta\x22,\x0a\
begin:/^\x5c$(TTL|G\
ENERATE|INCLUDE|\
ORIGIN)\x5cb/},{cla\
ssName:\x22number\x22,\
\x0abegin:\x22((([0-9A\
-Fa-f]{1,4}:){7}\
([0-9A-Fa-f]{1,4\
}|:))|(([0-9A-Fa\
-f]{1,4}:){6}(:[\
0-9A-Fa-f]{1,4}|\
((25[0-5]|2[0-4]\
\x5c\x5cd|1\x5c\x5cd\x5c\x5cd|[1-9\
]?\x5c\x5cd)(\x5c\x5c.(25[0-\
5]|2[0-4]\x5c\x5cd|1\x5c\x5c\
d\x5c\x5cd|[1-9]?\x5c\x5cd))\
{3})|:))|(([0-9A\
-Fa-f]{1,4}:){5}\
(((:[0-9A-Fa-f]{\
1,4}){1,2})|:((2\
5[0-5]|2[0-4]\x5c\x5cd\
|1\x5c\x5cd\x5c\x5cd|[1-9]?\x5c\
\x5cd)(\x5c\x5c.(25[0-5]|\
2[0-4]\x5c\x5cd|1\x5c\x5cd\x5c\x5c\
d|[1-9]?\x5c\x5cd)){3}\
)|:))|(([0-9A-Fa\
-f]{1,4}:){4}(((\
:[0-9A-Fa-f]{1,4\
}){1,3})|((:[0-9\
A-Fa-f]{1,4})?:(\
(25[0-5]|2[0-4]\x5c\
\x5cd|1\x5c\x5cd\x5c\x5cd|[1-9]\
?\x5c\x5cd)(\x5c\x5c.(25[0-5\
]|2[0-4]\x5c\x5cd|1\x5c\x5cd\
\x5c\x5cd|[1-9]?\x5c\x5cd)){\
3}))|:))|(([0-9A\
-Fa-f]{1,4}:){3}\
(((:[0-9A-Fa-f]{\
1,4}){1,4})|((:[\
0-9A-Fa-f]{1,4})\
{0,2}:((25[0-5]|\
2[0-4]\x5c\x5cd|1\x5c\x5cd\x5c\x5c\
d|[1-9]?\x5c\x5cd)(\x5c\x5c.\
(25[0-5]|2[0-4]\x5c\
\x5cd|1\x5c\x5cd\x5c\x5cd|[1-9]\
?\x5c\x5cd)){3}))|:))|\
(([0-9A-Fa-f]{1,\
4}:){2}(((:[0-9A\
-Fa-f]{1,4}){1,5\
})|((:[0-9A-Fa-f\
]{1,4}){0,3}:((2\
5[0-5]|2[0-4]\x5c\x5cd\
|1\x5c\x5cd\x5c\x5cd|[1-9]?\x5c\
\x5cd)(\x5c\x5c.(25[0-5]|\
2[0-4]\x5c\x5cd|1\x5c\x5cd\x5c\x5c\
d|[1-9]?\x5c\x5cd)){3}\
))|:))|(([0-9A-F\
a-f]{1,4}:){1}((\
(:[0-9A-Fa-f]{1,\
4}){1,6})|((:[0-\
9A-Fa-f]{1,4}){0\
,4}:((25[0-5]|2[\
0-4]\x5c\x5cd|1\x5c\x5cd\x5c\x5cd|\
[1-9]?\x5c\x5cd)(\x5c\x5c.(2\
5[0-5]|2[0-4]\x5c\x5cd\
|1\x5c\x5cd\x5c\x5cd|[1-9]?\x5c\
\x5cd)){3}))|:))|(:\
(((:[0-9A-Fa-f]{\
1,4}){1,7})|((:[\
0-9A-Fa-f]{1,4})\
{0,5}:((25[0-5]|\
2[0-4]\x5c\x5cd|1\x5c\x5cd\x5c\x5c\
d|[1-9]?\x5c\x5cd)(\x5c\x5c.\
(25[0-5]|2[0-4]\x5c\
\x5cd|1\x5c\x5cd\x5c\x5cd|[1-9]\
?\x5c\x5cd)){3}))|:)))\
\x5c\x5cb\x22\x0a},{classNam\
e:\x22number\x22,\x0abegi\
n:\x22((25[0-5]|(2[\
0-4]|1{0,1}[0-9]\
){0,1}[0-9]).){3\
,3}(25[0-5]|(2[0\
-4]|1{0,1}[0-9])\
{0,1}[0-9])\x5c\x5cb\x22\x0a\
},d.inherit(d.NU\
MBER_MODE,{begin\
:/\x5cb\x5cd+[dhwm]?/}\
)]})})()\x0a;hljs.r\
egisterLanguage(\
\x22dns\x22,d)})();/*!\
 `dockerfile` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>({name:\x22Dock\
erfile\x22,aliases:\
[\x22docker\x22],\x0acase\
_insensitive:!0,\
\x0akeywords:[\x22from\
\x22,\x22maintainer\x22,\x22\
expose\x22,\x22env\x22,\x22a\
rg\x22,\x22user\x22,\x22onbu\
ild\x22,\x22stopsignal\
\x22],\x0acontains:[e.\
HASH_COMMENT_MOD\
E,e.APOS_STRING_\
MODE,e.QUOTE_STR\
ING_MODE,e.NUMBE\
R_MODE,{\x0abeginKe\
ywords:\x22run cmd \
entrypoint volum\
e add copy workd\
ir label healthc\
heck shell\x22,\x0asta\
rts:{end:/[^\x5c\x5c]$\
/,subLanguage:\x22b\
ash\x22}}],illegal:\
\x22</\x22})})()\x0a;hljs\
.registerLanguag\
e(\x22dockerfile\x22,e\
)})();/*! `dos` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{const r=e\
.COMMENT(/^\x5cs*@?\
rem\x5cb/,/$/,{\x0arel\
evance:10});retu\
rn{name:\x22Batch f\
ile (DOS)\x22,alias\
es:[\x22bat\x22,\x22cmd\x22]\
,\x0acase_insensiti\
ve:!0,illegal:/\x5c\
/\x5c*/,keywords:{\x0a\
keyword:[\x22if\x22,\x22e\
lse\x22,\x22goto\x22,\x22for\
\x22,\x22in\x22,\x22do\x22,\x22cal\
l\x22,\x22exit\x22,\x22not\x22,\
\x22exist\x22,\x22errorle\
vel\x22,\x22defined\x22,\x22\
equ\x22,\x22neq\x22,\x22lss\x22\
,\x22leq\x22,\x22gtr\x22,\x22ge\
q\x22],\x0abuilt_in:[\x22\
prn\x22,\x22nul\x22,\x22lpt3\
\x22,\x22lpt2\x22,\x22lpt1\x22,\
\x22con\x22,\x22com4\x22,\x22co\
m3\x22,\x22com2\x22,\x22com1\
\x22,\x22aux\x22,\x22shift\x22,\
\x22cd\x22,\x22dir\x22,\x22echo\
\x22,\x22setlocal\x22,\x22en\
dlocal\x22,\x22set\x22,\x22p\
ause\x22,\x22copy\x22,\x22ap\
pend\x22,\x22assoc\x22,\x22a\
t\x22,\x22attrib\x22,\x22bre\
ak\x22,\x22cacls\x22,\x22cd\x22\
,\x22chcp\x22,\x22chdir\x22,\
\x22chkdsk\x22,\x22chkntf\
s\x22,\x22cls\x22,\x22cmd\x22,\x22\
color\x22,\x22comp\x22,\x22c\
ompact\x22,\x22convert\
\x22,\x22date\x22,\x22dir\x22,\x22\
diskcomp\x22,\x22diskc\
opy\x22,\x22doskey\x22,\x22e\
rase\x22,\x22fs\x22,\x22find\
\x22,\x22findstr\x22,\x22for\
mat\x22,\x22ftype\x22,\x22gr\
aftabl\x22,\x22help\x22,\x22\
keyb\x22,\x22label\x22,\x22m\
d\x22,\x22mkdir\x22,\x22mode\
\x22,\x22more\x22,\x22move\x22,\
\x22path\x22,\x22pause\x22,\x22\
print\x22,\x22popd\x22,\x22p\
ushd\x22,\x22promt\x22,\x22r\
d\x22,\x22recover\x22,\x22re\
m\x22,\x22rename\x22,\x22rep\
lace\x22,\x22restore\x22,\
\x22rmdir\x22,\x22shift\x22,\
\x22sort\x22,\x22start\x22,\x22\
subst\x22,\x22time\x22,\x22t\
itle\x22,\x22tree\x22,\x22ty\
pe\x22,\x22ver\x22,\x22verif\
y\x22,\x22vol\x22,\x22ping\x22,\
\x22net\x22,\x22ipconfig\x22\
,\x22taskkill\x22,\x22xco\
py\x22,\x22ren\x22,\x22del\x22]\
\x0a},contains:[{cl\
assName:\x22variabl\
e\x22,begin:/%%[^ ]\
|%[^ ]+?%|![^ ]+\
?!/},{\x0aclassName\
:\x22function\x22,begi\
n:\x22^\x5c\x5cs*[A-Za-z.\
_?][A-Za-z0-9_$#\
@~.?]*(:|\x5c\x5cs+lab\
el)\x22,\x0aend:\x22goto:\
eof\x22,contains:[e\
.inherit(e.TITLE\
_MODE,{\x0abegin:\x22(\
[_a-zA-Z]\x5c\x5cw*\x5c\x5c.\
)*([_a-zA-Z]\x5c\x5cw*\
:)?[_a-zA-Z]\x5c\x5cw*\
\x22}),r]},{\x0aclassN\
ame:\x22number\x22,beg\
in:\x22\x5c\x5cb\x5c\x5cd+\x22,rel\
evance:0},r]}}})\
()\x0a;hljs.registe\
rLanguage(\x22dos\x22,\
e)})();/*! `dsco\
nfig` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>({ke\
ywords:\x22dsconfig\
\x22,contains:[{\x0acl\
assName:\x22keyword\
\x22,begin:\x22^dsconf\
ig\x22,end:/\x5cs/,exc\
ludeEnd:!0,relev\
ance:10},{\x0aclass\
Name:\x22built_in\x22,\
begin:/(list|cre\
ate|get|set|dele\
te)-(\x5cw+)/,end:/\
\x5cs/,\x0aexcludeEnd:\
!0,illegal:\x22!@#$\
%^&*()\x22,relevanc\
e:10},{className\
:\x22built_in\x22,\x0abeg\
in:/--(\x5cw+)/,end\
:/\x5cs/,excludeEnd\
:!0},{className:\
\x22string\x22,begin:/\
\x22/,end:/\x22/},{\x0acl\
assName:\x22string\x22\
,begin:/'/,end:/\
'/},{className:\x22\
string\x22,begin:/[\
\x5cw\x5c-?]+:\x5cw+/,\x0aen\
d:/\x5cW/,relevance\
:0},{className:\x22\
string\x22,begin:/\x5c\
w+(\x5c-\x5cw+)*/,end:\
/(?=\x5cW)/,\x0areleva\
nce:0},e.HASH_CO\
MMENT_MODE]})})(\
);hljs.registerL\
anguage(\x22dsconfi\
g\x22,e)})();/*! `d\
ts` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>{const\
 a={className:\x22s\
tring\x22,\x0avariants\
:[e.inherit(e.QU\
OTE_STRING_MODE,\
{begin:'((u8?|U)\
|L)?\x22'}),{\x0abegin\
:'(u8?|U)?R\x22',en\
d:'\x22',contains:[\
e.BACKSLASH_ESCA\
PE]},{begin:\x22'\x5c\x5c\
\x5c\x5c?.\x22,\x0aend:\x22'\x22,i\
llegal:\x22.\x22}]},n=\
{className:\x22numb\
er\x22,variants:[{\x0a\
begin:\x22\x5c\x5cb(\x5c\x5cd+(\
\x5c\x5c.\x5c\x5cd*)?|\x5c\x5c.\x5c\x5cd\
+)(u|U|l|L|ul|UL\
|f|F)\x22},{begin:e\
.C_NUMBER_RE}],\x0a\
relevance:0},s={\
className:\x22meta\x22\
,begin:\x22#\x22,end:\x22\
$\x22,keywords:{\x0ake\
yword:\x22if else e\
lif endif define\
 undef ifdef ifn\
def\x22},contains:[\
{begin:/\x5c\x5c\x5cn/,\x0ar\
elevance:0},{beg\
inKeywords:\x22incl\
ude\x22,end:\x22$\x22,key\
words:{keyword:\x22\
include\x22},\x0aconta\
ins:[e.inherit(a\
,{className:\x22str\
ing\x22}),{classNam\
e:\x22string\x22,begin\
:\x22<\x22,\x0aend:\x22>\x22,il\
legal:\x22\x5c\x5cn\x22}]},a\
,e.C_LINE_COMMEN\
T_MODE,e.C_BLOCK\
_COMMENT_MODE]},\
i={\x0aclassName:\x22v\
ariable\x22,begin:/\
&[a-z\x5cd_]*\x5cb/};r\
eturn{name:\x22Devi\
ce Tree\x22,contain\
s:[{\x0aclassName:\x22\
title.class\x22,beg\
in:/^\x5c/(?=\x5cs*\x5c{)\
/,relevance:10},\
i,{\x0aclassName:\x22k\
eyword\x22,begin:\x22/\
[a-z][a-z\x5c\x5cd-]*/\
\x22},{className:\x22s\
ymbol\x22,\x0abegin:\x22^\
\x5c\x5cs*[a-zA-Z_][a-\
zA-Z\x5c\x5cd_]*:\x22},{c\
lassName:\x22title.\
class\x22,\x0abegin:/[\
a-zA-Z_][a-zA-Z\x5c\
d_@-]*(?=\x5cs\x5c{)/,\
relevance:.2},{r\
elevance:0,\x0amatc\
h:[/[a-z][a-z-,]\
+/,/\x5cs*/,/=/],sc\
ope:{1:\x22attr\x22,3:\
\x22operator\x22}},{\x0am\
atch:/[a-z][a-z-\
,]+(?=;)/,releva\
nce:0,scope:\x22att\
r\x22},{className:\x22\
params\x22,\x0arelevan\
ce:0,begin:\x22<\x22,e\
nd:\x22>\x22,contains:\
[n,i]\x0a},e.C_LINE\
_COMMENT_MODE,e.\
C_BLOCK_COMMENT_\
MODE,n,a,s,{scop\
e:\x22punctuation\x22,\
\x0arelevance:0,mat\
ch:/\x5c};|[;{}]/},\
{begin:e.IDENT_R\
E+\x22::\x22,keywords:\
\x22\x22}]}}})()\x0a;hljs\
.registerLanguag\
e(\x22dts\x22,e)})();/\
*! `dust` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
({name:\x22Dust\x22,al\
iases:[\x22dst\x22],\x0ac\
ase_insensitive:\
!0,subLanguage:\x22\
xml\x22,contains:[{\
className:\x22templ\
ate-tag\x22,\x0abegin:\
/\x5c{[#\x5c/]/,end:/\x5c\
}/,illegal:/;/,c\
ontains:[{classN\
ame:\x22name\x22,\x0abegi\
n:/[a-zA-Z\x5c.-]+/\
,starts:{endsWit\
hParent:!0,relev\
ance:0,\x0acontains\
:[e.QUOTE_STRING\
_MODE]}}]},{clas\
sName:\x22template-\
variable\x22,begin:\
/\x5c{/,\x0aend:/\x5c}/,i\
llegal:/;/,keywo\
rds:\x22if eq ne lt\
 lte gt gte sele\
ct default math \
sep\x22}]\x0a})})();hl\
js.registerLangu\
age(\x22dust\x22,e)})(\
);/*! `ebnf` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var e=(()=>{\x22use\
 strict\x22;return \
e=>{const a=e.CO\
MMENT(/\x5c(\x5c*/,/\x5c*\
\x5c)/)\x0a;return{nam\
e:\x22Extended Back\
us-Naur Form\x22,il\
legal:/\x5cS/,conta\
ins:[a,{\x0aclassNa\
me:\x22attribute\x22,b\
egin:/^[ ]*[a-zA\
-Z]+([\x5cs_-]+[a-z\
A-Z]+)*/},{begin\
:/=/,\x0aend:/[.;]/\
,contains:[a,{cl\
assName:\x22meta\x22,b\
egin:/\x5c?.*\x5c?/},{\
className:\x22strin\
g\x22,\x0avariants:[e.\
APOS_STRING_MODE\
,e.QUOTE_STRING_\
MODE,{begin:\x22`\x22,\
end:\x22`\x22}]}]}]}}}\
)()\x0a;hljs.regist\
erLanguage(\x22ebnf\
\x22,e)})();/*! `el\
ixir` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>{\x0aco\
nst n=e.regex,a=\
\x22[a-zA-Z_][a-zA-\
Z0-9_.]*(!|\x5c\x5c?)?\
\x22,i={$pattern:a,\
\x0akeyword:[\x22after\
\x22,\x22alias\x22,\x22and\x22,\
\x22case\x22,\x22catch\x22,\x22\
cond\x22,\x22defstruct\
\x22,\x22defguard\x22,\x22do\
\x22,\x22else\x22,\x22end\x22,\x22\
fn\x22,\x22for\x22,\x22if\x22,\x22\
import\x22,\x22in\x22,\x22no\
t\x22,\x22or\x22,\x22quote\x22,\
\x22raise\x22,\x22receive\
\x22,\x22require\x22,\x22rer\
aise\x22,\x22rescue\x22,\x22\
try\x22,\x22unless\x22,\x22u\
nquote\x22,\x22unquote\
_splicing\x22,\x22use\x22\
,\x22when\x22,\x22with|0\x22\
],\x0aliteral:[\x22fal\
se\x22,\x22nil\x22,\x22true\x22\
]},s={className:\
\x22subst\x22,begin:/#\
\x5c{/,end:/\x5c}/,\x0ake\
ywords:i},c={mat\
ch:/\x5c\x5c[\x5cs\x5cS]/,sc\
ope:\x22char.escape\
\x22,relevance:0\x0a},\
r=\x22[/|([{<\x5c\x22']\x22,\
t=[{begin:/\x22/,en\
d:/\x22/},{begin:/'\
/,end:/'/},{begi\
n:/\x5c//,\x0aend:/\x5c//\
},{begin:/\x5c|/,en\
d:/\x5c|/},{begin:/\
\x5c(/,end:/\x5c)/},{b\
egin:/\x5c[/,end:/\x5c\
]/},{\x0abegin:/\x5c{/\
,end:/\x5c}/},{begi\
n:/</,end:/>/}],\
d=e=>({scope:\x22ch\
ar.escape\x22,\x0abegi\
n:n.concat(/\x5c\x5c/,\
e),relevance:0})\
,o={className:\x22s\
tring\x22,\x0abegin:\x22~\
[a-z](?=\x22+r+\x22)\x22,\
contains:t.map((\
n=>e.inherit(n,{\
contains:[d(n.en\
d),c,s]\x0a})))},b=\
{className:\x22stri\
ng\x22,begin:\x22~[A-Z\
](?=\x22+r+\x22)\x22,\x0acon\
tains:t.map((n=>\
e.inherit(n,{con\
tains:[d(n.end)]\
})))},g={classNa\
me:\x22regex\x22,\x0avari\
ants:[{begin:\x22~r\
(?=\x22+r+\x22)\x22,conta\
ins:t.map((a=>e.\
inherit(a,{\x0aend:\
n.concat(a.end,/\
[uismxfU]{0,7}/)\
,contains:[d(a.e\
nd),c,s]})))},{\x0a\
begin:\x22~R(?=\x22+r+\
\x22)\x22,contains:t.m\
ap((a=>e.inherit\
(a,{\x0aend:n.conca\
t(a.end,/[uismxf\
U]{0,7}/),contai\
ns:[d(a.end)]}))\
)}]},l={\x0aclassNa\
me:\x22string\x22,cont\
ains:[e.BACKSLAS\
H_ESCAPE,s],vari\
ants:[{begin:/\x22\x22\
\x22/,\x0aend:/\x22\x22\x22/},{\
begin:/'''/,end:\
/'''/},{begin:/~\
S\x22\x22\x22/,end:/\x22\x22\x22/,\
contains:[]},{\x0ab\
egin:/~S\x22/,end:/\
\x22/,contains:[]},\
{begin:/~S'''/,e\
nd:/'''/,contain\
s:[]},{\x0abegin:/~\
S'/,end:/'/,cont\
ains:[]},{begin:\
/'/,end:/'/},{be\
gin:/\x22/,end:/\x22/}\
]},m={\x0aclassName\
:\x22function\x22,begi\
nKeywords:\x22def d\
efp defmacro def\
macrop\x22,end:/\x5cB\x5c\
b/,\x0acontains:[e.\
inherit(e.TITLE_\
MODE,{begin:a,en\
dsParent:!0})]},\
u=e.inherit(m,{\x0a\
className:\x22class\
\x22,beginKeywords:\
\x22defimpl defmodu\
le defprotocol d\
efrecord\x22,\x0aend:/\
\x5cbdo\x5cb|$|;/}),f=\
[l,g,b,o,e.HASH_\
COMMENT_MODE,u,m\
,{begin:\x22::\x22},{\x0a\
className:\x22symbo\
l\x22,begin:\x22:(?![\x5c\
\x5cs:])\x22,contains:\
[l,{\x0abegin:\x22[a-z\
A-Z_]\x5c\x5cw*[!?=]?|\
[-+~]@|<<|>>|=~|\
===?|<=>|[<>]=?|\
\x5c\x5c*\x5c\x5c*|[-/+%^&*~\
`|]|\x5c\x5c[\x5c\x5c]=?\x22\x0a}]\
,relevance:0},{c\
lassName:\x22symbol\
\x22,begin:a+\x22:(?!:\
)\x22,relevance:0},\
{\x0aclassName:\x22tit\
le.class\x22,begin:\
/(\x5cb[A-Z][a-zA-Z\
0-9_]+)/,relevan\
ce:0},{\x0aclassNam\
e:\x22number\x22,\x0abegi\
n:\x22(\x5c\x5cb0o[0-7_]+\
)|(\x5c\x5cb0b[01_]+)|\
(\x5c\x5cb0x[0-9a-fA-F\
_]+)|(-?\x5c\x5cb[0-9]\
[0-9_]*(\x5c\x5c.[0-9_\
]+([eE][-+]?[0-9\
]+)?)?)\x22,\x0areleva\
nce:0},{classNam\
e:\x22variable\x22,beg\
in:\x22(\x5c\x5c$\x5c\x5cW)|((\x5c\
\x5c$|@@?)(\x5c\x5cw+))\x22}\
]\x0a;return s.cont\
ains=f,{name:\x22El\
ixir\x22,aliases:[\x22\
ex\x22,\x22exs\x22],keywo\
rds:i,contains:f\
}}\x0a})();hljs.reg\
isterLanguage(\x22e\
lixir\x22,e)})();/*\
! `elm` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>{c\
onst n={\x0avariant\
s:[e.COMMENT(\x22--\
\x22,\x22$\x22),e.COMMENT\
(/\x5c{-/,/-\x5c}/,{co\
ntains:[\x22self\x22]}\
)]},i={\x0aclassNam\
e:\x22type\x22,begin:\x22\
\x5c\x5cb[A-Z][\x5c\x5cw']*\x22\
,relevance:0},s=\
{begin:\x22\x5c\x5c(\x22,end\
:\x22\x5c\x5c)\x22,\x0aillegal:\
'\x22',contains:[{c\
lassName:\x22type\x22,\
\x0abegin:\x22\x5c\x5cb[A-Z]\
[\x5c\x5cw]*(\x5c\x5c((\x5c\x5c.\x5c\x5c\
.|,|\x5c\x5cw+)\x5c\x5c))?\x22}\
,n]};return{name\
:\x22Elm\x22,\x0akeywords\
:[\x22let\x22,\x22in\x22,\x22if\
\x22,\x22then\x22,\x22else\x22,\
\x22case\x22,\x22of\x22,\x22whe\
re\x22,\x22module\x22,\x22im\
port\x22,\x22exposing\x22\
,\x22type\x22,\x22alias\x22,\
\x22as\x22,\x22infix\x22,\x22in\
fixl\x22,\x22infixr\x22,\x22\
port\x22,\x22effect\x22,\x22\
command\x22,\x22subscr\
iption\x22],\x0acontai\
ns:[{beginKeywor\
ds:\x22port effect \
module\x22,end:\x22exp\
osing\x22,\x0akeywords\
:\x22port effect mo\
dule where comma\
nd subscription \
exposing\x22,\x0aconta\
ins:[s,n],illega\
l:\x22\x5c\x5cW\x5c\x5c.|;\x22},{b\
egin:\x22import\x22,en\
d:\x22$\x22,\x0akeywords:\
\x22import as expos\
ing\x22,contains:[s\
,n],illegal:\x22\x5c\x5cW\
\x5c\x5c.|;\x22},{begin:\x22\
type\x22,\x0aend:\x22$\x22,k\
eywords:\x22type al\
ias\x22,contains:[i\
,s,{begin:/\x5c{/,e\
nd:/\x5c}/,\x0acontain\
s:s.contains},n]\
},{beginKeywords\
:\x22infix infixl i\
nfixr\x22,end:\x22$\x22,\x0a\
contains:[e.C_NU\
MBER_MODE,n]},{b\
egin:\x22port\x22,end:\
\x22$\x22,keywords:\x22po\
rt\x22,contains:[n]\
\x0a},{className:\x22s\
tring\x22,begin:\x22'\x5c\
\x5c\x5c\x5c?.\x22,end:\x22'\x22,i\
llegal:\x22.\x22\x0a},e.Q\
UOTE_STRING_MODE\
,e.C_NUMBER_MODE\
,i,e.inherit(e.T\
ITLE_MODE,{\x0abegi\
n:\x22^[_a-z][\x5c\x5cw']\
*\x22}),n,{begin:\x22-\
>|<-\x22}],illegal:\
/;/}}})()\x0a;hljs.\
registerLanguage\
(\x22elm\x22,e)})();/*\
! `erb` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>({\
name:\x22ERB\x22,subLa\
nguage:\x22xml\x22,\x0aco\
ntains:[e.COMMEN\
T(\x22<%#\x22,\x22%>\x22),{b\
egin:\x22<%[%=-]?\x22,\
end:\x22[%-]?%>\x22,\x0as\
ubLanguage:\x22ruby\
\x22,excludeBegin:!\
0,excludeEnd:!0}\
]})})()\x0a;hljs.re\
gisterLanguage(\x22\
erb\x22,e)})();/*! \
`erlang` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
e=(()=>{\x22use str\
ict\x22;return e=>{\
\x0aconst n=\x22[a-z']\
[a-zA-Z0-9_']*\x22,\
r=\x22(\x22+n+\x22:\x22+n+\x22|\
\x22+n+\x22)\x22,a={\x0akeyw\
ord:\x22after and a\
ndalso|10 band b\
egin bnot bor bs\
l bzr bxor case \
catch cond div e\
nd fun if let no\
t of orelse|10 q\
uery receive rem\
 try when xor\x22,\x0a\
literal:\x22false t\
rue\x22},i=e.COMMEN\
T(\x22%\x22,\x22$\x22),s={cl\
assName:\x22number\x22\
,\x0abegin:\x22\x5c\x5cb(\x5c\x5cd\
+(_\x5c\x5cd+)*#[a-fA-\
F0-9]+(_[a-fA-F0\
-9]+)*|\x5c\x5cd+(_\x5c\x5cd\
+)*(\x5c\x5c.\x5c\x5cd+(_\x5c\x5cd\
+)*)?([eE][-+]?\x5c\
\x5cd+)?)\x22,\x0arelevan\
ce:0},c={begin:\x22\
fun\x5c\x5cs+\x22+n+\x22/\x5c\x5cd\
+\x22},t={begin:r+\x22\
\x5c\x5c(\x22,end:\x22\x5c\x5c)\x22,\x0a\
returnBegin:!0,r\
elevance:0,conta\
ins:[{begin:r,re\
levance:0},{begi\
n:\x22\x5c\x5c(\x22,\x0aend:\x22\x5c\x5c\
)\x22,endsWithParen\
t:!0,returnEnd:!\
0,relevance:0}]}\
,d={begin:/\x5c{/,e\
nd:/\x5c}/,\x0arelevan\
ce:0},o={begin:\x22\
\x5c\x5cb_([A-Z][A-Za-\
z0-9_]*)?\x22,relev\
ance:0},l={\x0abegi\
n:\x22[A-Z][a-zA-Z0\
-9_]*\x22,relevance\
:0},g={begin:\x22#\x22\
+e.UNDERSCORE_ID\
ENT_RE,\x0arelevanc\
e:0,returnBegin:\
!0,contains:[{be\
gin:\x22#\x22+e.UNDERS\
CORE_IDENT_RE,\x0ar\
elevance:0},{beg\
in:/\x5c{/,end:/\x5c}/\
,relevance:0}]},\
b={scope:\x22string\
\x22,\x0amatch:/\x5c$(\x5c\x5c(\
[^0-9]|[0-9]{1,3\
}|)|.)/},E={\x0abeg\
inKeywords:\x22fun \
receive if try c\
ase\x22,end:\x22end\x22,k\
eywords:a}\x0a;E.co\
ntains=[i,c,e.in\
herit(e.APOS_STR\
ING_MODE,{classN\
ame:\x22\x22\x0a}),E,t,e.\
QUOTE_STRING_MOD\
E,s,d,o,l,g,b]\x0a;\
const u=[i,c,E,t\
,e.QUOTE_STRING_\
MODE,s,d,o,l,g,b\
]\x0a;t.contains[1]\
.contains=u,d.co\
ntains=u,g.conta\
ins[1].contains=\
u;const _={\x0aclas\
sName:\x22params\x22,b\
egin:\x22\x5c\x5c(\x22,end:\x22\
\x5c\x5c)\x22,contains:u}\
;return{name:\x22Er\
lang\x22,\x0aaliases:[\
\x22erl\x22],keywords:\
a,illegal:\x22(</|\x5c\
\x5c*=|\x5c\x5c+=|-=|/\x5c\x5c*\
|\x5c\x5c*/|\x5c\x5c(\x5c\x5c*|\x5c\x5c*\
\x5c\x5c))\x22,\x0acontains:\
[{className:\x22fun\
ction\x22,begin:\x22^\x22\
+n+\x22\x5c\x5cs*\x5c\x5c(\x22,end\
:\x22->\x22,returnBegi\
n:!0,\x0aillegal:\x22\x5c\
\x5c(|#|//|/\x5c\x5c*|\x5c\x5c\x5c\
\x5c|:|;\x22,contains:\
[_,e.inherit(e.T\
ITLE_MODE,{begin\
:n})],\x0astarts:{e\
nd:\x22;|\x5c\x5c.\x22,keywo\
rds:a,contains:u\
}},i,{begin:\x22^-\x22\
,end:\x22\x5c\x5c.\x22,relev\
ance:0,\x0aexcludeE\
nd:!0,returnBegi\
n:!0,keywords:{$\
pattern:\x22-\x22+e.ID\
ENT_RE,\x0akeyword:\
[\x22-module\x22,\x22-rec\
ord\x22,\x22-undef\x22,\x22-\
export\x22,\x22-ifdef\x22\
,\x22-ifndef\x22,\x22-aut\
hor\x22,\x22-copyright\
\x22,\x22-doc\x22,\x22-vsn\x22,\
\x22-import\x22,\x22-incl\
ude\x22,\x22-include_l\
ib\x22,\x22-compile\x22,\x22\
-define\x22,\x22-else\x22\
,\x22-endif\x22,\x22-file\
\x22,\x22-behaviour\x22,\x22\
-behavior\x22,\x22-spe\
c\x22].map((e=>e+\x22|\
1.5\x22)).join(\x22 \x22)\
\x0a},contains:[_]}\
,s,e.QUOTE_STRIN\
G_MODE,g,o,l,d,b\
,{begin:/\x5c.$/}]}\
}})()\x0a;hljs.regi\
sterLanguage(\x22er\
lang\x22,e)})();/*!\
 `erlang-repl` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>{const n=e.\
regex;return{\x0ana\
me:\x22Erlang REPL\x22\
,keywords:{built\
_in:\x22spawn spawn\
_link self\x22,\x0akey\
word:\x22after and \
andalso|10 band \
begin bnot bor b\
sl bsr bxor case\
 catch cond div \
end fun if let n\
ot of or orelse|\
10 query receive\
 rem try when xo\
r\x22\x0a},contains:[{\
className:\x22meta.\
prompt\x22,begin:\x22^\
[0-9]+> \x22,releva\
nce:10\x0a},e.COMME\
NT(\x22%\x22,\x22$\x22),{cla\
ssName:\x22number\x22,\
\x0abegin:\x22\x5c\x5cb(\x5c\x5cd+\
(_\x5c\x5cd+)*#[a-fA-F\
0-9]+(_[a-fA-F0-\
9]+)*|\x5c\x5cd+(_\x5c\x5cd+\
)*(\x5c\x5c.\x5c\x5cd+(_\x5c\x5cd+\
)*)?([eE][-+]?\x5c\x5c\
d+)?)\x22,\x0arelevanc\
e:0},e.APOS_STRI\
NG_MODE,e.QUOTE_\
STRING_MODE,{\x0abe\
gin:n.concat(/\x5c?\
(::)?/,/([A-Z]\x5cw\
*)/,/((::)[A-Z]\x5c\
w*)*/)},{begin:\x22\
->\x22},{\x0abegin:\x22ok\
\x22},{begin:\x22!\x22},{\
\x0abegin:\x22(\x5c\x5cb[a-z\
'][a-zA-Z0-9_']*\
:[a-z'][a-zA-Z0-\
9_']*)|(\x5c\x5cb[a-z'\
][a-zA-Z0-9_']*)\
\x22,\x0arelevance:0},\
{begin:\x22[A-Z][a-\
zA-Z0-9_']*\x22,rel\
evance:0}]}}})()\
\x0a;hljs.registerL\
anguage(\x22erlang-\
repl\x22,e)})();/*!\
 `excel` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
E=(()=>{\x22use str\
ict\x22;return E=>(\
{name:\x22Excel for\
mulae\x22,\x0aaliases:\
[\x22xlsx\x22,\x22xls\x22],c\
ase_insensitive:\
!0,keywords:{$pa\
ttern:/[a-zA-Z][\
\x5cw\x5c.]*/,\x0abuilt_i\
n:[\x22ABS\x22,\x22ACCRIN\
T\x22,\x22ACCRINTM\x22,\x22A\
COS\x22,\x22ACOSH\x22,\x22AC\
OT\x22,\x22ACOTH\x22,\x22AGG\
REGATE\x22,\x22ADDRESS\
\x22,\x22AMORDEGRC\x22,\x22A\
MORLINC\x22,\x22AND\x22,\x22\
ARABIC\x22,\x22AREAS\x22,\
\x22ASC\x22,\x22ASIN\x22,\x22AS\
INH\x22,\x22ATAN\x22,\x22ATA\
N2\x22,\x22ATANH\x22,\x22AVE\
DEV\x22,\x22AVERAGE\x22,\x22\
AVERAGEA\x22,\x22AVERA\
GEIF\x22,\x22AVERAGEIF\
S\x22,\x22BAHTTEXT\x22,\x22B\
ASE\x22,\x22BESSELI\x22,\x22\
BESSELJ\x22,\x22BESSEL\
K\x22,\x22BESSELY\x22,\x22BE\
TADIST\x22,\x22BETA.DI\
ST\x22,\x22BETAINV\x22,\x22B\
ETA.INV\x22,\x22BIN2DE\
C\x22,\x22BIN2HEX\x22,\x22BI\
N2OCT\x22,\x22BINOMDIS\
T\x22,\x22BINOM.DIST\x22,\
\x22BINOM.DIST.RANG\
E\x22,\x22BINOM.INV\x22,\x22\
BITAND\x22,\x22BITLSHI\
FT\x22,\x22BITOR\x22,\x22BIT\
RSHIFT\x22,\x22BITXOR\x22\
,\x22CALL\x22,\x22CEILING\
\x22,\x22CEILING.MATH\x22\
,\x22CEILING.PRECIS\
E\x22,\x22CELL\x22,\x22CHAR\x22\
,\x22CHIDIST\x22,\x22CHII\
NV\x22,\x22CHITEST\x22,\x22C\
HISQ.DIST\x22,\x22CHIS\
Q.DIST.RT\x22,\x22CHIS\
Q.INV\x22,\x22CHISQ.IN\
V.RT\x22,\x22CHISQ.TES\
T\x22,\x22CHOOSE\x22,\x22CLE\
AN\x22,\x22CODE\x22,\x22COLU\
MN\x22,\x22COLUMNS\x22,\x22C\
OMBIN\x22,\x22COMBINA\x22\
,\x22COMPLEX\x22,\x22CONC\
AT\x22,\x22CONCATENATE\
\x22,\x22CONFIDENCE\x22,\x22\
CONFIDENCE.NORM\x22\
,\x22CONFIDENCE.T\x22,\
\x22CONVERT\x22,\x22CORRE\
L\x22,\x22COS\x22,\x22COSH\x22,\
\x22COT\x22,\x22COTH\x22,\x22CO\
UNT\x22,\x22COUNTA\x22,\x22C\
OUNTBLANK\x22,\x22COUN\
TIF\x22,\x22COUNTIFS\x22,\
\x22COUPDAYBS\x22,\x22COU\
PDAYS\x22,\x22COUPDAYS\
NC\x22,\x22COUPNCD\x22,\x22C\
OUPNUM\x22,\x22COUPPCD\
\x22,\x22COVAR\x22,\x22COVAR\
IANCE.P\x22,\x22COVARI\
ANCE.S\x22,\x22CRITBIN\
OM\x22,\x22CSC\x22,\x22CSCH\x22\
,\x22CUBEKPIMEMBER\x22\
,\x22CUBEMEMBER\x22,\x22C\
UBEMEMBERPROPERT\
Y\x22,\x22CUBERANKEDME\
MBER\x22,\x22CUBESET\x22,\
\x22CUBESETCOUNT\x22,\x22\
CUBEVALUE\x22,\x22CUMI\
PMT\x22,\x22CUMPRINC\x22,\
\x22DATE\x22,\x22DATEDIF\x22\
,\x22DATEVALUE\x22,\x22DA\
VERAGE\x22,\x22DAY\x22,\x22D\
AYS\x22,\x22DAYS360\x22,\x22\
DB\x22,\x22DBCS\x22,\x22DCOU\
NT\x22,\x22DCOUNTA\x22,\x22D\
DB\x22,\x22DEC2BIN\x22,\x22D\
EC2HEX\x22,\x22DEC2OCT\
\x22,\x22DECIMAL\x22,\x22DEG\
REES\x22,\x22DELTA\x22,\x22D\
EVSQ\x22,\x22DGET\x22,\x22DI\
SC\x22,\x22DMAX\x22,\x22DMIN\
\x22,\x22DOLLAR\x22,\x22DOLL\
ARDE\x22,\x22DOLLARFR\x22\
,\x22DPRODUCT\x22,\x22DST\
DEV\x22,\x22DSTDEVP\x22,\x22\
DSUM\x22,\x22DURATION\x22\
,\x22DVAR\x22,\x22DVARP\x22,\
\x22EDATE\x22,\x22EFFECT\x22\
,\x22ENCODEURL\x22,\x22EO\
MONTH\x22,\x22ERF\x22,\x22ER\
F.PRECISE\x22,\x22ERFC\
\x22,\x22ERFC.PRECISE\x22\
,\x22ERROR.TYPE\x22,\x22E\
UROCONVERT\x22,\x22EVE\
N\x22,\x22EXACT\x22,\x22EXP\x22\
,\x22EXPON.DIST\x22,\x22E\
XPONDIST\x22,\x22FACT\x22\
,\x22FACTDOUBLE\x22,\x22F\
ALSE|0\x22,\x22F.DIST\x22\
,\x22FDIST\x22,\x22F.DIST\
.RT\x22,\x22FILTERXML\x22\
,\x22FIND\x22,\x22FINDB\x22,\
\x22F.INV\x22,\x22F.INV.R\
T\x22,\x22FINV\x22,\x22FISHE\
R\x22,\x22FISHERINV\x22,\x22\
FIXED\x22,\x22FLOOR\x22,\x22\
FLOOR.MATH\x22,\x22FLO\
OR.PRECISE\x22,\x22FOR\
ECAST\x22,\x22FORECAST\
.ETS\x22,\x22FORECAST.\
ETS.CONFINT\x22,\x22FO\
RECAST.ETS.SEASO\
NALITY\x22,\x22FORECAS\
T.ETS.STAT\x22,\x22FOR\
ECAST.LINEAR\x22,\x22F\
ORMULATEXT\x22,\x22FRE\
QUENCY\x22,\x22F.TEST\x22\
,\x22FTEST\x22,\x22FV\x22,\x22F\
VSCHEDULE\x22,\x22GAMM\
A\x22,\x22GAMMA.DIST\x22,\
\x22GAMMADIST\x22,\x22GAM\
MA.INV\x22,\x22GAMMAIN\
V\x22,\x22GAMMALN\x22,\x22GA\
MMALN.PRECISE\x22,\x22\
GAUSS\x22,\x22GCD\x22,\x22GE\
OMEAN\x22,\x22GESTEP\x22,\
\x22GETPIVOTDATA\x22,\x22\
GROWTH\x22,\x22HARMEAN\
\x22,\x22HEX2BIN\x22,\x22HEX\
2DEC\x22,\x22HEX2OCT\x22,\
\x22HLOOKUP\x22,\x22HOUR\x22\
,\x22HYPERLINK\x22,\x22HY\
PGEOM.DIST\x22,\x22HYP\
GEOMDIST\x22,\x22IF\x22,\x22\
IFERROR\x22,\x22IFNA\x22,\
\x22IFS\x22,\x22IMABS\x22,\x22I\
MAGINARY\x22,\x22IMARG\
UMENT\x22,\x22IMCONJUG\
ATE\x22,\x22IMCOS\x22,\x22IM\
COSH\x22,\x22IMCOT\x22,\x22I\
MCSC\x22,\x22IMCSCH\x22,\x22\
IMDIV\x22,\x22IMEXP\x22,\x22\
IMLN\x22,\x22IMLOG10\x22,\
\x22IMLOG2\x22,\x22IMPOWE\
R\x22,\x22IMPRODUCT\x22,\x22\
IMREAL\x22,\x22IMSEC\x22,\
\x22IMSECH\x22,\x22IMSIN\x22\
,\x22IMSINH\x22,\x22IMSQR\
T\x22,\x22IMSUB\x22,\x22IMSU\
M\x22,\x22IMTAN\x22,\x22INDE\
X\x22,\x22INDIRECT\x22,\x22I\
NFO\x22,\x22INT\x22,\x22INTE\
RCEPT\x22,\x22INTRATE\x22\
,\x22IPMT\x22,\x22IRR\x22,\x22I\
SBLANK\x22,\x22ISERR\x22,\
\x22ISERROR\x22,\x22ISEVE\
N\x22,\x22ISFORMULA\x22,\x22\
ISLOGICAL\x22,\x22ISNA\
\x22,\x22ISNONTEXT\x22,\x22I\
SNUMBER\x22,\x22ISODD\x22\
,\x22ISREF\x22,\x22ISTEXT\
\x22,\x22ISO.CEILING\x22,\
\x22ISOWEEKNUM\x22,\x22IS\
PMT\x22,\x22JIS\x22,\x22KURT\
\x22,\x22LARGE\x22,\x22LCM\x22,\
\x22LEFT\x22,\x22LEFTB\x22,\x22\
LEN\x22,\x22LENB\x22,\x22LIN\
EST\x22,\x22LN\x22,\x22LOG\x22,\
\x22LOG10\x22,\x22LOGEST\x22\
,\x22LOGINV\x22,\x22LOGNO\
RM.DIST\x22,\x22LOGNOR\
MDIST\x22,\x22LOGNORM.\
INV\x22,\x22LOOKUP\x22,\x22L\
OWER\x22,\x22MATCH\x22,\x22M\
AX\x22,\x22MAXA\x22,\x22MAXI\
FS\x22,\x22MDETERM\x22,\x22M\
DURATION\x22,\x22MEDIA\
N\x22,\x22MID\x22,\x22MIDBs\x22\
,\x22MIN\x22,\x22MINIFS\x22,\
\x22MINA\x22,\x22MINUTE\x22,\
\x22MINVERSE\x22,\x22MIRR\
\x22,\x22MMULT\x22,\x22MOD\x22,\
\x22MODE\x22,\x22MODE.MUL\
T\x22,\x22MODE.SNGL\x22,\x22\
MONTH\x22,\x22MROUND\x22,\
\x22MULTINOMIAL\x22,\x22M\
UNIT\x22,\x22N\x22,\x22NA\x22,\x22\
NEGBINOM.DIST\x22,\x22\
NEGBINOMDIST\x22,\x22N\
ETWORKDAYS\x22,\x22NET\
WORKDAYS.INTL\x22,\x22\
NOMINAL\x22,\x22NORM.D\
IST\x22,\x22NORMDIST\x22,\
\x22NORMINV\x22,\x22NORM.\
INV\x22,\x22NORM.S.DIS\
T\x22,\x22NORMSDIST\x22,\x22\
NORM.S.INV\x22,\x22NOR\
MSINV\x22,\x22NOT\x22,\x22NO\
W\x22,\x22NPER\x22,\x22NPV\x22,\
\x22NUMBERVALUE\x22,\x22O\
CT2BIN\x22,\x22OCT2DEC\
\x22,\x22OCT2HEX\x22,\x22ODD\
\x22,\x22ODDFPRICE\x22,\x22O\
DDFYIELD\x22,\x22ODDLP\
RICE\x22,\x22ODDLYIELD\
\x22,\x22OFFSET\x22,\x22OR\x22,\
\x22PDURATION\x22,\x22PEA\
RSON\x22,\x22PERCENTIL\
E.EXC\x22,\x22PERCENTI\
LE.INC\x22,\x22PERCENT\
ILE\x22,\x22PERCENTRAN\
K.EXC\x22,\x22PERCENTR\
ANK.INC\x22,\x22PERCEN\
TRANK\x22,\x22PERMUT\x22,\
\x22PERMUTATIONA\x22,\x22\
PHI\x22,\x22PHONETIC\x22,\
\x22PI\x22,\x22PMT\x22,\x22POIS\
SON.DIST\x22,\x22POISS\
ON\x22,\x22POWER\x22,\x22PPM\
T\x22,\x22PRICE\x22,\x22PRIC\
EDISC\x22,\x22PRICEMAT\
\x22,\x22PROB\x22,\x22PRODUC\
T\x22,\x22PROPER\x22,\x22PV\x22\
,\x22QUARTILE\x22,\x22QUA\
RTILE.EXC\x22,\x22QUAR\
TILE.INC\x22,\x22QUOTI\
ENT\x22,\x22RADIANS\x22,\x22\
RAND\x22,\x22RANDBETWE\
EN\x22,\x22RANK.AVG\x22,\x22\
RANK.EQ\x22,\x22RANK\x22,\
\x22RATE\x22,\x22RECEIVED\
\x22,\x22REGISTER.ID\x22,\
\x22REPLACE\x22,\x22REPLA\
CEB\x22,\x22REPT\x22,\x22RIG\
HT\x22,\x22RIGHTB\x22,\x22RO\
MAN\x22,\x22ROUND\x22,\x22RO\
UNDDOWN\x22,\x22ROUNDU\
P\x22,\x22ROW\x22,\x22ROWS\x22,\
\x22RRI\x22,\x22RSQ\x22,\x22RTD\
\x22,\x22SEARCH\x22,\x22SEAR\
CHB\x22,\x22SEC\x22,\x22SECH\
\x22,\x22SECOND\x22,\x22SERI\
ESSUM\x22,\x22SHEET\x22,\x22\
SHEETS\x22,\x22SIGN\x22,\x22\
SIN\x22,\x22SINH\x22,\x22SKE\
W\x22,\x22SKEW.P\x22,\x22SLN\
\x22,\x22SLOPE\x22,\x22SMALL\
\x22,\x22SQL.REQUEST\x22,\
\x22SQRT\x22,\x22SQRTPI\x22,\
\x22STANDARDIZE\x22,\x22S\
TDEV\x22,\x22STDEV.P\x22,\
\x22STDEV.S\x22,\x22STDEV\
A\x22,\x22STDEVP\x22,\x22STD\
EVPA\x22,\x22STEYX\x22,\x22S\
UBSTITUTE\x22,\x22SUBT\
OTAL\x22,\x22SUM\x22,\x22SUM\
IF\x22,\x22SUMIFS\x22,\x22SU\
MPRODUCT\x22,\x22SUMSQ\
\x22,\x22SUMX2MY2\x22,\x22SU\
MX2PY2\x22,\x22SUMXMY2\
\x22,\x22SWITCH\x22,\x22SYD\x22\
,\x22T\x22,\x22TAN\x22,\x22TANH\
\x22,\x22TBILLEQ\x22,\x22TBI\
LLPRICE\x22,\x22TBILLY\
IELD\x22,\x22T.DIST\x22,\x22\
T.DIST.2T\x22,\x22T.DI\
ST.RT\x22,\x22TDIST\x22,\x22\
TEXT\x22,\x22TEXTJOIN\x22\
,\x22TIME\x22,\x22TIMEVAL\
UE\x22,\x22T.INV\x22,\x22T.I\
NV.2T\x22,\x22TINV\x22,\x22T\
ODAY\x22,\x22TRANSPOSE\
\x22,\x22TREND\x22,\x22TRIM\x22\
,\x22TRIMMEAN\x22,\x22TRU\
E|0\x22,\x22TRUNC\x22,\x22T.\
TEST\x22,\x22TTEST\x22,\x22T\
YPE\x22,\x22UNICHAR\x22,\x22\
UNICODE\x22,\x22UPPER\x22\
,\x22VALUE\x22,\x22VAR\x22,\x22\
VAR.P\x22,\x22VAR.S\x22,\x22\
VARA\x22,\x22VARP\x22,\x22VA\
RPA\x22,\x22VDB\x22,\x22VLOO\
KUP\x22,\x22WEBSERVICE\
\x22,\x22WEEKDAY\x22,\x22WEE\
KNUM\x22,\x22WEIBULL\x22,\
\x22WEIBULL.DIST\x22,\x22\
WORKDAY\x22,\x22WORKDA\
Y.INTL\x22,\x22XIRR\x22,\x22\
XNPV\x22,\x22XOR\x22,\x22YEA\
R\x22,\x22YEARFRAC\x22,\x22Y\
IELD\x22,\x22YIELDDISC\
\x22,\x22YIELDMAT\x22,\x22Z.\
TEST\x22,\x22ZTEST\x22]\x0a}\
,contains:[{begi\
n:/^=/,end:/[^=]\
/,returnEnd:!0,i\
llegal:/=/,relev\
ance:10},{\x0aclass\
Name:\x22symbol\x22,be\
gin:/\x5cb[A-Z]{1,2\
}\x5cd+\x5cb/,end:/[^\x5c\
d]/,excludeEnd:!\
0,\x0arelevance:0},\
{className:\x22symb\
ol\x22,begin:/[A-Z]\
{0,2}\x5cd*:[A-Z]{0\
,2}\x5cd*/,relevanc\
e:0\x0a},E.BACKSLAS\
H_ESCAPE,E.QUOTE\
_STRING_MODE,{cl\
assName:\x22number\x22\
,\x0abegin:E.NUMBER\
_RE+\x22(%)?\x22,relev\
ance:0},E.COMMEN\
T(/\x5cbN\x5c(/,/\x5c)/,{\
excludeBegin:!0,\
\x0aexcludeEnd:!0,i\
llegal:/\x5cn/})]})\
})();hljs.regist\
erLanguage(\x22exce\
l\x22,E)})();/*! `f\
ix` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>({name\
:\x22FIX\x22,contains:\
[{\x0abegin:/[^\x5cu24\
01\x5cu0001]+/,end:\
/[\x5cu2401\x5cu0001]/\
,excludeEnd:!0,r\
eturnBegin:!0,\x0ar\
eturnEnd:!1,cont\
ains:[{begin:/([\
^\x5cu2401\x5cu0001=]+\
)/,end:/=([^\x5cu24\
01\x5cu0001=]+)/,\x0ar\
eturnEnd:!0,retu\
rnBegin:!1,class\
Name:\x22attr\x22},{be\
gin:/=/,end:/([\x5c\
u2401\x5cu0001])/,\x0a\
excludeEnd:!0,ex\
cludeBegin:!0,cl\
assName:\x22string\x22\
}]}],case_insens\
itive:!0})})()\x0a;\
hljs.registerLan\
guage(\x22fix\x22,e)})\
();/*! `flix` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>({name:\x22Flix\
\x22,keywords:{\x0akey\
word:[\x22case\x22,\x22cl\
ass\x22,\x22def\x22,\x22else\
\x22,\x22enum\x22,\x22if\x22,\x22i\
mpl\x22,\x22import\x22,\x22i\
n\x22,\x22lat\x22,\x22rel\x22,\x22\
index\x22,\x22let\x22,\x22ma\
tch\x22,\x22namespace\x22\
,\x22switch\x22,\x22type\x22\
,\x22yield\x22,\x22with\x22]\
,\x0aliteral:[\x22true\
\x22,\x22false\x22]},\x0acon\
tains:[e.C_LINE_\
COMMENT_MODE,e.C\
_BLOCK_COMMENT_M\
ODE,{className:\x22\
string\x22,\x0abegin:/\
'(.|\x5c\x5c[xXuU][a-z\
A-Z0-9]+)'/},{cl\
assName:\x22string\x22\
,variants:[{begi\
n:'\x22',\x0aend:'\x22'}]\
},{className:\x22fu\
nction\x22,beginKey\
words:\x22def\x22,end:\
/[:={\x5c[(\x5cn;]/,\x0ae\
xcludeEnd:!0,con\
tains:[{classNam\
e:\x22title\x22,releva\
nce:0,\x0abegin:/[^\
0-9\x5cn\x5ct \x22'(),.`{\
}\x5c[\x5c]:;][^\x5cn\x5ct \x22\
'(),.`{}\x5c[\x5c]:;]+\
|[^0-9\x5cn\x5ct \x22'(),\
.`{}\x5c[\x5c]:;=]/\x0a}]\
},e.C_NUMBER_MOD\
E]})})();hljs.re\
gisterLanguage(\x22\
flix\x22,e)})();/*!\
 `fortran` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{const n=e.rege\
x,a={\x0avariants:[\
e.COMMENT(\x22!\x22,\x22$\
\x22,{relevance:0})\
,e.COMMENT(\x22^C[ \
]\x22,\x22$\x22,{relevanc\
e:0\x0a}),e.COMMENT\
(\x22^C$\x22,\x22$\x22,{rele\
vance:0})]\x0a},t=/\
(_[a-z_\x5cd]+)?/,i\
=/([de][+-]?\x5cd+)\
?/,c={className:\
\x22number\x22,variant\
s:[{\x0abegin:n.con\
cat(/\x5cb\x5cd+/,/\x5c.(\
\x5cd*)/,i,t)},{beg\
in:n.concat(/\x5cb\x5c\
d+/,i,t)},{\x0abegi\
n:n.concat(/\x5c.\x5cd\
+/,i,t)}],releva\
nce:0},o={classN\
ame:\x22function\x22,\x0a\
beginKeywords:\x22s\
ubroutine functi\
on program\x22,ille\
gal:\x22[${=\x5c\x5cn]\x22,\x0a\
contains:[e.UNDE\
RSCORE_TITLE_MOD\
E,{className:\x22pa\
rams\x22,begin:\x22\x5c\x5c(\
\x22,end:\x22\x5c\x5c)\x22}]}\x0a;\
return{name:\x22For\
tran\x22,case_insen\
sitive:!0,aliase\
s:[\x22f90\x22,\x22f95\x22],\
keywords:{\x0a$patt\
ern:/\x5cb[a-z][a-z\
0-9_]+\x5cb|\x5c.[a-z]\
[a-z0-9_]+\x5c./,\x0ak\
eyword:[\x22kind\x22,\x22\
do\x22,\x22concurrent\x22\
,\x22local\x22,\x22shared\
\x22,\x22while\x22,\x22priva\
te\x22,\x22call\x22,\x22intr\
insic\x22,\x22where\x22,\x22\
elsewhere\x22,\x22type\
\x22,\x22endtype\x22,\x22end\
module\x22,\x22endsele\
ct\x22,\x22endinterfac\
e\x22,\x22end\x22,\x22enddo\x22\
,\x22endif\x22,\x22if\x22,\x22f\
orall\x22,\x22endforal\
l\x22,\x22only\x22,\x22conta\
ins\x22,\x22default\x22,\x22\
return\x22,\x22stop\x22,\x22\
then\x22,\x22block\x22,\x22e\
ndblock\x22,\x22endass\
ociate\x22,\x22public\x22\
,\x22subroutine|10\x22\
,\x22function\x22,\x22pro\
gram\x22,\x22.and.\x22,\x22.\
or.\x22,\x22.not.\x22,\x22.l\
e.\x22,\x22.eq.\x22,\x22.ge.\
\x22,\x22.gt.\x22,\x22.lt.\x22,\
\x22goto\x22,\x22save\x22,\x22e\
lse\x22,\x22use\x22,\x22modu\
le\x22,\x22select\x22,\x22ca\
se\x22,\x22access\x22,\x22bl\
ank\x22,\x22direct\x22,\x22e\
xist\x22,\x22file\x22,\x22fm\
t\x22,\x22form\x22,\x22forma\
tted\x22,\x22iostat\x22,\x22\
name\x22,\x22named\x22,\x22n\
extrec\x22,\x22number\x22\
,\x22opened\x22,\x22rec\x22,\
\x22recl\x22,\x22sequenti\
al\x22,\x22status\x22,\x22un\
formatted\x22,\x22unit\
\x22,\x22continue\x22,\x22fo\
rmat\x22,\x22pause\x22,\x22c\
ycle\x22,\x22exit\x22,\x22c_\
null_char\x22,\x22c_al\
ert\x22,\x22c_backspac\
e\x22,\x22c_form_feed\x22\
,\x22flush\x22,\x22wait\x22,\
\x22decimal\x22,\x22round\
\x22,\x22iomsg\x22,\x22synch\
ronous\x22,\x22nopass\x22\
,\x22non_overridabl\
e\x22,\x22pass\x22,\x22prote\
cted\x22,\x22volatile\x22\
,\x22abstract\x22,\x22ext\
ends\x22,\x22import\x22,\x22\
non_intrinsic\x22,\x22\
value\x22,\x22deferred\
\x22,\x22generic\x22,\x22fin\
al\x22,\x22enumerator\x22\
,\x22class\x22,\x22associ\
ate\x22,\x22bind\x22,\x22enu\
m\x22,\x22c_int\x22,\x22c_sh\
ort\x22,\x22c_long\x22,\x22c\
_long_long\x22,\x22c_s\
igned_char\x22,\x22c_s\
ize_t\x22,\x22c_int8_t\
\x22,\x22c_int16_t\x22,\x22c\
_int32_t\x22,\x22c_int\
64_t\x22,\x22c_int_lea\
st8_t\x22,\x22c_int_le\
ast16_t\x22,\x22c_int_\
least32_t\x22,\x22c_in\
t_least64_t\x22,\x22c_\
int_fast8_t\x22,\x22c_\
int_fast16_t\x22,\x22c\
_int_fast32_t\x22,\x22\
c_int_fast64_t\x22,\
\x22c_intmax_t\x22,\x22C_\
intptr_t\x22,\x22c_flo\
at\x22,\x22c_double\x22,\x22\
c_long_double\x22,\x22\
c_float_complex\x22\
,\x22c_double_compl\
ex\x22,\x22c_long_doub\
le_complex\x22,\x22c_b\
ool\x22,\x22c_char\x22,\x22c\
_null_ptr\x22,\x22c_nu\
ll_funptr\x22,\x22c_ne\
w_line\x22,\x22c_carri\
age_return\x22,\x22c_h\
orizontal_tab\x22,\x22\
c_vertical_tab\x22,\
\x22iso_c_binding\x22,\
\x22c_loc\x22,\x22c_funlo\
c\x22,\x22c_associated\
\x22,\x22c_f_pointer\x22,\
\x22c_ptr\x22,\x22c_funpt\
r\x22,\x22iso_fortran_\
env\x22,\x22character_\
storage_size\x22,\x22e\
rror_unit\x22,\x22file\
_storage_size\x22,\x22\
input_unit\x22,\x22ios\
tat_end\x22,\x22iostat\
_eor\x22,\x22numeric_s\
torage_size\x22,\x22ou\
tput_unit\x22,\x22c_f_\
procpointer\x22,\x22ie\
ee_arithmetic\x22,\x22\
ieee_support_und\
erflow_control\x22,\
\x22ieee_get_underf\
low_mode\x22,\x22ieee_\
set_underflow_mo\
de\x22,\x22newunit\x22,\x22c\
ontiguous\x22,\x22recu\
rsive\x22,\x22pad\x22,\x22po\
sition\x22,\x22action\x22\
,\x22delim\x22,\x22readwr\
ite\x22,\x22eor\x22,\x22adva\
nce\x22,\x22nml\x22,\x22inte\
rface\x22,\x22procedur\
e\x22,\x22namelist\x22,\x22i\
nclude\x22,\x22sequenc\
e\x22,\x22elemental\x22,\x22\
pure\x22,\x22impure\x22,\x22\
integer\x22,\x22real\x22,\
\x22character\x22,\x22com\
plex\x22,\x22logical\x22,\
\x22codimension\x22,\x22d\
imension\x22,\x22alloc\
atable|10\x22,\x22para\
meter\x22,\x22external\
\x22,\x22implicit|10\x22,\
\x22none\x22,\x22double\x22,\
\x22precision\x22,\x22ass\
ign\x22,\x22intent\x22,\x22o\
ptional\x22,\x22pointe\
r\x22,\x22target\x22,\x22in\x22\
,\x22out\x22,\x22common\x22,\
\x22equivalence\x22,\x22d\
ata\x22],\x0aliteral:[\
\x22.False.\x22,\x22.True\
.\x22],\x0abuilt_in:[\x22\
alog\x22,\x22alog10\x22,\x22\
amax0\x22,\x22amax1\x22,\x22\
amin0\x22,\x22amin1\x22,\x22\
amod\x22,\x22cabs\x22,\x22cc\
os\x22,\x22cexp\x22,\x22clog\
\x22,\x22csin\x22,\x22csqrt\x22\
,\x22dabs\x22,\x22dacos\x22,\
\x22dasin\x22,\x22datan\x22,\
\x22datan2\x22,\x22dcos\x22,\
\x22dcosh\x22,\x22ddim\x22,\x22\
dexp\x22,\x22dint\x22,\x22dl\
og\x22,\x22dlog10\x22,\x22dm\
ax1\x22,\x22dmin1\x22,\x22dm\
od\x22,\x22dnint\x22,\x22dsi\
gn\x22,\x22dsin\x22,\x22dsin\
h\x22,\x22dsqrt\x22,\x22dtan\
\x22,\x22dtanh\x22,\x22float\
\x22,\x22iabs\x22,\x22idim\x22,\
\x22idint\x22,\x22idnint\x22\
,\x22ifix\x22,\x22isign\x22,\
\x22max0\x22,\x22max1\x22,\x22m\
in0\x22,\x22min1\x22,\x22sng\
l\x22,\x22algama\x22,\x22cda\
bs\x22,\x22cdcos\x22,\x22cde\
xp\x22,\x22cdlog\x22,\x22cds\
in\x22,\x22cdsqrt\x22,\x22cq\
abs\x22,\x22cqcos\x22,\x22cq\
exp\x22,\x22cqlog\x22,\x22cq\
sin\x22,\x22cqsqrt\x22,\x22d\
cmplx\x22,\x22dconjg\x22,\
\x22derf\x22,\x22derfc\x22,\x22\
dfloat\x22,\x22dgamma\x22\
,\x22dimag\x22,\x22dlgama\
\x22,\x22iqint\x22,\x22qabs\x22\
,\x22qacos\x22,\x22qasin\x22\
,\x22qatan\x22,\x22qatan2\
\x22,\x22qcmplx\x22,\x22qcon\
jg\x22,\x22qcos\x22,\x22qcos\
h\x22,\x22qdim\x22,\x22qerf\x22\
,\x22qerfc\x22,\x22qexp\x22,\
\x22qgamma\x22,\x22qimag\x22\
,\x22qlgama\x22,\x22qlog\x22\
,\x22qlog10\x22,\x22qmax1\
\x22,\x22qmin1\x22,\x22qmod\x22\
,\x22qnint\x22,\x22qsign\x22\
,\x22qsin\x22,\x22qsinh\x22,\
\x22qsqrt\x22,\x22qtan\x22,\x22\
qtanh\x22,\x22abs\x22,\x22ac\
os\x22,\x22aimag\x22,\x22ain\
t\x22,\x22anint\x22,\x22asin\
\x22,\x22atan\x22,\x22atan2\x22\
,\x22char\x22,\x22cmplx\x22,\
\x22conjg\x22,\x22cos\x22,\x22c\
osh\x22,\x22exp\x22,\x22icha\
r\x22,\x22index\x22,\x22int\x22\
,\x22log\x22,\x22log10\x22,\x22\
max\x22,\x22min\x22,\x22nint\
\x22,\x22sign\x22,\x22sin\x22,\x22\
sinh\x22,\x22sqrt\x22,\x22ta\
n\x22,\x22tanh\x22,\x22print\
\x22,\x22write\x22,\x22dim\x22,\
\x22lge\x22,\x22lgt\x22,\x22lle\
\x22,\x22llt\x22,\x22mod\x22,\x22n\
ullify\x22,\x22allocat\
e\x22,\x22deallocate\x22,\
\x22adjustl\x22,\x22adjus\
tr\x22,\x22all\x22,\x22alloc\
ated\x22,\x22any\x22,\x22ass\
ociated\x22,\x22bit_si\
ze\x22,\x22btest\x22,\x22cei\
ling\x22,\x22count\x22,\x22c\
shift\x22,\x22date_and\
_time\x22,\x22digits\x22,\
\x22dot_product\x22,\x22e\
oshift\x22,\x22epsilon\
\x22,\x22exponent\x22,\x22fl\
oor\x22,\x22fraction\x22,\
\x22huge\x22,\x22iand\x22,\x22i\
bclr\x22,\x22ibits\x22,\x22i\
bset\x22,\x22ieor\x22,\x22io\
r\x22,\x22ishft\x22,\x22ishf\
tc\x22,\x22lbound\x22,\x22le\
n_trim\x22,\x22matmul\x22\
,\x22maxexponent\x22,\x22\
maxloc\x22,\x22maxval\x22\
,\x22merge\x22,\x22minexp\
onent\x22,\x22minloc\x22,\
\x22minval\x22,\x22modulo\
\x22,\x22mvbits\x22,\x22near\
est\x22,\x22pack\x22,\x22pre\
sent\x22,\x22product\x22,\
\x22radix\x22,\x22random_\
number\x22,\x22random_\
seed\x22,\x22range\x22,\x22r\
epeat\x22,\x22reshape\x22\
,\x22rrspacing\x22,\x22sc\
ale\x22,\x22scan\x22,\x22sel\
ected_int_kind\x22,\
\x22selected_real_k\
ind\x22,\x22set_expone\
nt\x22,\x22shape\x22,\x22siz\
e\x22,\x22spacing\x22,\x22sp\
read\x22,\x22sum\x22,\x22sys\
tem_clock\x22,\x22tiny\
\x22,\x22transpose\x22,\x22t\
rim\x22,\x22ubound\x22,\x22u\
npack\x22,\x22verify\x22,\
\x22achar\x22,\x22iachar\x22\
,\x22transfer\x22,\x22dbl\
e\x22,\x22entry\x22,\x22dpro\
d\x22,\x22cpu_time\x22,\x22c\
ommand_argument_\
count\x22,\x22get_comm\
and\x22,\x22get_comman\
d_argument\x22,\x22get\
_environment_var\
iable\x22,\x22is_iosta\
t_end\x22,\x22ieee_ari\
thmetic\x22,\x22ieee_s\
upport_underflow\
_control\x22,\x22ieee_\
get_underflow_mo\
de\x22,\x22ieee_set_un\
derflow_mode\x22,\x22i\
s_iostat_eor\x22,\x22m\
ove_alloc\x22,\x22new_\
line\x22,\x22selected_\
char_kind\x22,\x22same\
_type_as\x22,\x22exten\
ds_type_of\x22,\x22aco\
sh\x22,\x22asinh\x22,\x22ata\
nh\x22,\x22bessel_j0\x22,\
\x22bessel_j1\x22,\x22bes\
sel_jn\x22,\x22bessel_\
y0\x22,\x22bessel_y1\x22,\
\x22bessel_yn\x22,\x22erf\
\x22,\x22erfc\x22,\x22erfc_s\
caled\x22,\x22gamma\x22,\x22\
log_gamma\x22,\x22hypo\
t\x22,\x22norm2\x22,\x22atom\
ic_define\x22,\x22atom\
ic_ref\x22,\x22execute\
_command_line\x22,\x22\
leadz\x22,\x22trailz\x22,\
\x22storage_size\x22,\x22\
merge_bits\x22,\x22bge\
\x22,\x22bgt\x22,\x22ble\x22,\x22b\
lt\x22,\x22dshiftl\x22,\x22d\
shiftr\x22,\x22findloc\
\x22,\x22iall\x22,\x22iany\x22,\
\x22iparity\x22,\x22image\
_index\x22,\x22lcoboun\
d\x22,\x22ucobound\x22,\x22m\
askl\x22,\x22maskr\x22,\x22n\
um_images\x22,\x22pari\
ty\x22,\x22popcnt\x22,\x22po\
ppar\x22,\x22shifta\x22,\x22\
shiftl\x22,\x22shiftr\x22\
,\x22this_image\x22,\x22s\
ync\x22,\x22change\x22,\x22t\
eam\x22,\x22co_broadca\
st\x22,\x22co_max\x22,\x22co\
_min\x22,\x22co_sum\x22,\x22\
co_reduce\x22]\x0a},il\
legal:/\x5c/\x5c*/,con\
tains:[{classNam\
e:\x22string\x22,relev\
ance:0,\x0avariants\
:[e.APOS_STRING_\
MODE,e.QUOTE_STR\
ING_MODE]},o,{be\
gin:/^C\x5cs*=(?!=)\
/,\x0arelevance:0},\
a,c]}}})();hljs.\
registerLanguage\
(\x22fortran\x22,e)})(\
);/*! `fsharp` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;funct\
ion e(e){\x0areturn\
 RegExp(e.replac\
e(/[-/\x5c\x5c^$*+?.()\
|[\x5c]{}]/g,\x22\x5c\x5c$&\x22\
),\x22m\x22)}function \
n(e){\x0areturn e?\x22\
string\x22==typeof \
e?e:e.source:nul\
l}function t(e){\
return i(\x22(?=\x22,e\
,\x22)\x22)}\x0afunction \
i(...e){return e\
.map((e=>n(e))).\
join(\x22\x22)}functio\
n a(...e){const \
t=(e=>{\x0aconst n=\
e[e.length-1]\x0a;r\
eturn\x22object\x22==t\
ypeof n&&n.const\
ructor===Object?\
(e.splice(e.leng\
th-1,1),n):{}\x0a})\
(e);return\x22(\x22+(t\
.capture?\x22\x22:\x22?:\x22\
)+e.map((e=>n(e)\
)).join(\x22|\x22)+\x22)\x22\
}return n=>{\x0acon\
st r={scope:\x22key\
word\x22,match:/\x5cb(\
yield|return|let\
|do|match|use)!/\
\x0a},o=[\x22bool\x22,\x22by\
te\x22,\x22sbyte\x22,\x22int\
8\x22,\x22int16\x22,\x22int3\
2\x22,\x22uint8\x22,\x22uint\
16\x22,\x22uint32\x22,\x22in\
t\x22,\x22uint\x22,\x22int64\
\x22,\x22uint64\x22,\x22nati\
veint\x22,\x22unativei\
nt\x22,\x22decimal\x22,\x22f\
loat\x22,\x22double\x22,\x22\
float32\x22,\x22single\
\x22,\x22char\x22,\x22string\
\x22,\x22unit\x22,\x22bigint\
\x22,\x22option\x22,\x22vopt\
ion\x22,\x22list\x22,\x22arr\
ay\x22,\x22seq\x22,\x22byref\
\x22,\x22exn\x22,\x22inref\x22,\
\x22nativeptr\x22,\x22obj\
\x22,\x22outref\x22,\x22void\
ptr\x22,\x22Result\x22],s\
={\x0akeyword:[\x22abs\
tract\x22,\x22and\x22,\x22as\
\x22,\x22assert\x22,\x22base\
\x22,\x22begin\x22,\x22class\
\x22,\x22default\x22,\x22del\
egate\x22,\x22do\x22,\x22don\
e\x22,\x22downcast\x22,\x22d\
ownto\x22,\x22elif\x22,\x22e\
lse\x22,\x22end\x22,\x22exce\
ption\x22,\x22extern\x22,\
\x22finally\x22,\x22fixed\
\x22,\x22for\x22,\x22fun\x22,\x22f\
unction\x22,\x22global\
\x22,\x22if\x22,\x22in\x22,\x22inh\
erit\x22,\x22inline\x22,\x22\
interface\x22,\x22inte\
rnal\x22,\x22lazy\x22,\x22le\
t\x22,\x22match\x22,\x22memb\
er\x22,\x22module\x22,\x22mu\
table\x22,\x22namespac\
e\x22,\x22new\x22,\x22of\x22,\x22o\
pen\x22,\x22or\x22,\x22overr\
ide\x22,\x22private\x22,\x22\
public\x22,\x22rec\x22,\x22r\
eturn\x22,\x22static\x22,\
\x22struct\x22,\x22then\x22,\
\x22to\x22,\x22try\x22,\x22type\
\x22,\x22upcast\x22,\x22use\x22\
,\x22val\x22,\x22void\x22,\x22w\
hen\x22,\x22while\x22,\x22wi\
th\x22,\x22yield\x22],\x0ali\
teral:[\x22true\x22,\x22f\
alse\x22,\x22null\x22,\x22So\
me\x22,\x22None\x22,\x22Ok\x22,\
\x22Error\x22,\x22infinit\
y\x22,\x22infinityf\x22,\x22\
nan\x22,\x22nanf\x22],\x0abu\
ilt_in:[\x22not\x22,\x22r\
ef\x22,\x22raise\x22,\x22rer\
aise\x22,\x22dict\x22,\x22re\
adOnlyDict\x22,\x22set\
\x22,\x22get\x22,\x22enum\x22,\x22\
sizeof\x22,\x22typeof\x22\
,\x22typedefof\x22,\x22na\
meof\x22,\x22nullArg\x22,\
\x22invalidArg\x22,\x22in\
validOp\x22,\x22id\x22,\x22f\
st\x22,\x22snd\x22,\x22ignor\
e\x22,\x22lock\x22,\x22using\
\x22,\x22box\x22,\x22unbox\x22,\
\x22tryUnbox\x22,\x22prin\
tf\x22,\x22printfn\x22,\x22s\
printf\x22,\x22eprintf\
\x22,\x22eprintfn\x22,\x22fp\
rintf\x22,\x22fprintfn\
\x22,\x22failwith\x22,\x22fa\
ilwithf\x22],\x0a\x22vari\
able.constant\x22:[\
\x22__LINE__\x22,\x22__SO\
URCE_DIRECTORY__\
\x22,\x22__SOURCE_FILE\
__\x22]},c={\x0avarian\
ts:[n.COMMENT(/\x5c\
(\x5c*(?!\x5c))/,/\x5c*\x5c)\
/,{contains:[\x22se\
lf\x22]\x0a}),n.C_LINE\
_COMMENT_MODE]},\
l={scope:\x22variab\
le\x22,begin:/``/,e\
nd:/``/\x0a},u=/\x5cB(\
'|\x5c^)/,p={scope:\
\x22symbol\x22,variant\
s:[{match:i(u,/`\
`.*?``/)},{\x0amatc\
h:i(u,n.UNDERSCO\
RE_IDENT_RE)}],r\
elevance:0},f=({\
includeEqual:n})\
=>{let r\x0a;r=n?\x22!\
%&*+-/<=>@^|~?\x22:\
\x22!%&*+-/<>@^|~?\x22\
\x0a;const o=i(\x22[\x22,\
...Array.from(r)\
.map(e),\x22]\x22),s=a\
(o,/\x5c./),c=i(s,t\
(s)),l=a(i(c,s,\x22\
*\x22),i(o,\x22+\x22))\x0a;r\
eturn{scope:\x22ope\
rator\x22,match:a(l\
,/:\x5c?>/,/:\x5c?/,/:\
>/,/:=/,/::?/,/\x5c\
$/),\x0arelevance:0\
}},d=f({includeE\
qual:!0}),b=f({i\
ncludeEqual:!1})\
,g=(e,r)=>({\x0abeg\
in:i(e,t(i(/\x5cs*/\
,a(/\x5cw/,/'/,/\x5c^/\
,/#/,/``/,/\x5c(/,/\
{\x5c|/)))),beginSc\
ope:r,\x0aend:t(a(/\
\x5cn/,/=/)),releva\
nce:0,keywords:n\
.inherit(s,{type\
:o}),\x0acontains:[\
c,p,n.inherit(l,\
{scope:null}),b]\
\x0a}),m=g(/:/,\x22ope\
rator\x22),h=g(/\x5cbo\
f\x5cb/,\x22keyword\x22),\
y={\x0abegin:[/(^|\x5c\
s+)/,/type/,/\x5cs+\
/,/[a-zA-Z_](\x5cw|\
')*/],beginScope\
:{2:\x22keyword\x22,\x0a4\
:\x22title.class\x22},\
end:t(/\x5c(|=|$/),\
keywords:s,conta\
ins:[c,n.inherit\
(l,{scope:null\x0a}\
),p,{scope:\x22oper\
ator\x22,match:/<|>\
/},m]},E={scope:\
\x22computation-exp\
ression\x22,\x0amatch:\
/\x5cb[_a-z]\x5cw*(?=\x5c\
s*\x5c{)/},_={\x0abegi\
n:[/^\x5cs*/,i(/#/,\
a(\x22if\x22,\x22else\x22,\x22e\
ndif\x22,\x22line\x22,\x22no\
warn\x22,\x22light\x22,\x22r\
\x22,\x22i\x22,\x22I\x22,\x22load\x22\
,\x22time\x22,\x22help\x22,\x22\
quit\x22)),/\x5cb/],\x0ab\
eginScope:{2:\x22me\
ta\x22},end:t(/\x5cs|$\
/)},v={\x0avariants\
:[n.BINARY_NUMBE\
R_MODE,n.C_NUMBE\
R_MODE]},w={scop\
e:\x22string\x22,begin\
:/\x22/,\x0aend:/\x22/,co\
ntains:[n.BACKSL\
ASH_ESCAPE]},A={\
scope:\x22string\x22,b\
egin:/@\x22/,end:/\x22\
/,\x0acontains:[{ma\
tch:/\x22\x22/},n.BACK\
SLASH_ESCAPE]},S\
={scope:\x22string\x22\
,begin:/\x22\x22\x22/,\x0aen\
d:/\x22\x22\x22/,relevanc\
e:2},C={scope:\x22s\
ubst\x22,begin:/\x5c{/\
,end:/\x5c}/,keywor\
ds:s},O={\x0ascope:\
\x22string\x22,begin:/\
\x5c$\x22/,end:/\x22/,con\
tains:[{match:/\x5c\
{\x5c{/},{match:/\x5c}\
\x5c}/\x0a},n.BACKSLAS\
H_ESCAPE,C]},R={\
scope:\x22string\x22,b\
egin:/(\x5c$@|@\x5c$)\x22\
/,end:/\x22/,\x0aconta\
ins:[{match:/\x5c{\x5c\
{/},{match:/\x5c}\x5c}\
/},{match:/\x22\x22/},\
n.BACKSLASH_ESCA\
PE,C]},k={\x0ascope\
:\x22string\x22,begin:\
/\x5c$\x22\x22\x22/,end:/\x22\x22\x22\
/,contains:[{mat\
ch:/\x5c{\x5c{/},{matc\
h:/\x5c}\x5c}/\x0a},C],re\
levance:2},x={sc\
ope:\x22string\x22,\x0ama\
tch:i(/'/,a(/[^\x5c\
\x5c']/,/\x5c\x5c(?:.|\x5cd{\
3}|x[a-fA-F\x5cd]{2\
}|u[a-fA-F\x5cd]{4}\
|U[a-fA-F\x5cd]{8})\
/),/'/)\x0a};return\
 C.contains=[R,O\
,A,w,x,r,c,l,m,E\
,_,v,p,d],{name:\
\x22F#\x22,\x0aaliases:[\x22\
fs\x22,\x22f#\x22],keywor\
ds:s,illegal:/\x5c/\
\x5c*/,classNameAli\
ases:{\x0a\x22computat\
ion-expression\x22:\
\x22keyword\x22},conta\
ins:[r,{variants\
:[k,R,O,S,A,w,x]\
\x0a},c,l,y,{scope:\
\x22meta\x22,begin:/\x5c[\
</,end:/>\x5c]/,rel\
evance:2,contain\
s:[l,S,A,w,x,v]\x0a\
},h,m,E,_,v,p,d]\
}}})();hljs.regi\
sterLanguage(\x22fs\
harp\x22,e)})();/*!\
 `gams` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>{c\
onst a=e.regex,n\
={\x0akeyword:\x22abor\
t acronym acrony\
ms alias all and\
 assign binary c\
ard diag display\
 else eq file fi\
les for free ge \
gt if integer le\
 loop lt maximiz\
ing minimizing m\
odel models ne n\
egative no not o\
ption options or\
 ord positive pr\
od put putpage p\
uttl repeat same\
as semicont semi\
int smax smin so\
lve sos1 sos2 su\
m system table t\
hen until using \
while xor yes\x22,\x0a\
literal:\x22eps inf\
 na\x22,\x0abuilt_in:\x22\
abs arccos arcsi\
n arctan arctan2\
 Beta betaReg bi\
nomial ceil cent\
ropy cos cosh cv\
Power div div0 e\
Dist entropy err\
orf execSeed exp\
 fact floor frac\
 gamma gammaReg \
log logBeta logG\
amma log10 log2 \
mapVal max min m\
od ncpCM ncpF nc\
pVUpow ncpVUsin \
normal pi poly p\
ower randBinomia\
l randLinear ran\
dTriangle round \
rPower sigmoid s\
ign signPower si\
n sinh slexp sll\
og10 slrec sqexp\
 sqlog10 sqr sqr\
ec sqrt tan tanh\
 trunc uniform u\
niformInt vcPowe\
r bool_and bool_\
eqv bool_imp boo\
l_not bool_or bo\
ol_xor ifThen re\
l_eq rel_ge rel_\
gt rel_le rel_lt\
 rel_ne gday gdo\
w ghour gleap gm\
illisec gminute \
gmonth gsecond g\
year jdate jnow \
jstart jtime err\
orLevel execErro\
r gamsRelease ga\
msVersion handle\
Collect handleDe\
lete handleStatu\
s handleSubmit h\
eapFree heapLimi\
t heapSize jobHa\
ndle jobKill job\
Status jobTermin\
ate licenseLevel\
 licenseStatus m\
axExecError slee\
p timeClose time\
Comp timeElapsed\
 timeExec timeSt\
art\x22\x0a},i={classN\
ame:\x22symbol\x22,var\
iants:[{begin:/=\
[lgenxc]=/},{beg\
in:/\x5c$/}]},s={\x0ac\
lassName:\x22commen\
t\x22,variants:[{be\
gin:\x22'\x22,end:\x22'\x22}\
,{begin:'\x22',end:\
'\x22'}],\x0aillegal:\x22\
\x5c\x5cn\x22,contains:[e\
.BACKSLASH_ESCAP\
E]},o={begin:\x22/\x22\
,end:\x22/\x22,keyword\
s:n,\x0acontains:[s\
,e.C_LINE_COMMEN\
T_MODE,e.C_BLOCK\
_COMMENT_MODE,e.\
QUOTE_STRING_MOD\
E,e.APOS_STRING_\
MODE,e.C_NUMBER_\
MODE]\x0a},t=/[a-z0\
-9&#*=?@\x5c\x5c><:,()\
$[\x5c]_.{}!+%^-]+/\
,r={\x0abegin:/[a-z\
][a-z0-9_]*(\x5c([a\
-z0-9_, ]*\x5c))?[ \
\x5ct]+/,excludeBeg\
in:!0,end:\x22$\x22,\x0ae\
ndsWithParent:!0\
,contains:[s,o,{\
className:\x22comme\
nt\x22,\x0abegin:a.con\
cat(t,a.anyNumbe\
rOfTimes(a.conca\
t(/[ ]+/,t))),re\
levance:0}]};ret\
urn{\x0aname:\x22GAMS\x22\
,aliases:[\x22gms\x22]\
,case_insensitiv\
e:!0,keywords:n,\
\x0acontains:[e.COM\
MENT(/^\x5c$ontext/\
,/^\x5c$offtext/),{\
className:\x22meta\x22\
,\x0abegin:\x22^\x5c\x5c$[a-\
z0-9]+\x22,end:\x22$\x22,\
returnBegin:!0,c\
ontains:[{classN\
ame:\x22keyword\x22,\x0ab\
egin:\x22^\x5c\x5c$[a-z0-\
9]+\x22}]\x0a},e.COMME\
NT(\x22^\x5c\x5c*\x22,\x22$\x22),e\
.C_LINE_COMMENT_\
MODE,e.C_BLOCK_C\
OMMENT_MODE,e.QU\
OTE_STRING_MODE,\
e.APOS_STRING_MO\
DE,{\x0abeginKeywor\
ds:\x22set sets par\
ameter parameter\
s variable varia\
bles scalar scal\
ars equation equ\
ations\x22,\x0aend:\x22;\x22\
,\x0acontains:[e.CO\
MMENT(\x22^\x5c\x5c*\x22,\x22$\x22\
),e.C_LINE_COMME\
NT_MODE,e.C_BLOC\
K_COMMENT_MODE,e\
.QUOTE_STRING_MO\
DE,e.APOS_STRING\
_MODE,o,r]\x0a},{be\
ginKeywords:\x22tab\
le\x22,end:\x22;\x22,retu\
rnBegin:!0,conta\
ins:[{\x0abeginKeyw\
ords:\x22table\x22,end\
:\x22$\x22,contains:[r\
]\x0a},e.COMMENT(\x22^\
\x5c\x5c*\x22,\x22$\x22),e.C_LI\
NE_COMMENT_MODE,\
e.C_BLOCK_COMMEN\
T_MODE,e.QUOTE_S\
TRING_MODE,e.APO\
S_STRING_MODE,e.\
C_NUMBER_MODE]\x0a}\
,{className:\x22fun\
ction\x22,begin:/^[\
a-z][a-z0-9_,\x5c-+\
' ()$]+\x5c.{2}/,re\
turnBegin:!0,\x0aco\
ntains:[{classNa\
me:\x22title\x22,begin\
:/^[a-z0-9_]+/},\
{className:\x22para\
ms\x22,\x0abegin:/\x5c(/,\
end:/\x5c)/,exclude\
Begin:!0,exclude\
End:!0},i]},e.C_\
NUMBER_MODE,i]}}\
})()\x0a;hljs.regis\
terLanguage(\x22gam\
s\x22,e)})();/*! `g\
auss` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>{con\
st t={\x0akeyword:\x22\
bool break call \
callexe checkint\
errupt clear cle\
arg closeall cls\
 comlog compile \
continue create \
debug declare de\
lete disable dli\
brary dllcall do\
 dos ed edit els\
e elseif enable \
end endfor endif\
 endp endo error\
log errorlogat e\
xpr external fn \
for format goto \
gosub graph if k\
eyword let lib l\
ibrary line load\
 loadarray loade\
xe loadf loadk l\
oadm loadp loads\
 loadx local loc\
ate loopnextinde\
x lprint lpwidth\
 lshow matrix ms\
ym ndpclex new o\
pen output outwi\
dth plot plotsym\
 pop prcsn print\
 printdos proc p\
ush retp return \
rndcon rndmod rn\
dmult rndseed ru\
n save saveall s\
creen scroll set\
array show spars\
e stop string st\
ruct system trac\
e trap threadfor\
 threadendfor th\
readbegin thread\
join threadstat \
threadend until \
use while winpri\
nt ne ge le gt l\
t and xor or not\
 eq eqv\x22,\x0abuilt_\
in:\x22abs acf acon\
cat aeye amax am\
ean AmericanBino\
mCall AmericanBi\
nomCall_Greeks A\
mericanBinomCall\
_ImpVol American\
BinomPut America\
nBinomPut_Greeks\
 AmericanBinomPu\
t_ImpVol America\
nBSCall American\
BSCall_Greeks Am\
ericanBSCall_Imp\
Vol AmericanBSPu\
t AmericanBSPut_\
Greeks AmericanB\
SPut_ImpVol amin\
 amult annotatio\
nGetDefaults ann\
otationSetBkd an\
notationSetFont \
annotationSetLin\
eColor annotatio\
nSetLineStyle an\
notationSetLineT\
hickness annualT\
radingDays arcco\
s arcsin areshap\
e arrayalloc arr\
ayindex arrayini\
t arraytomat asc\
iiload asclabel \
astd astds asum \
atan atan2 atran\
spose axmargin b\
alance band band\
chol bandcholsol\
 bandltsol bandr\
v bandsolpd bar \
base10 begwind b\
esselj bessely b\
eta box boxcox c\
dfBeta cdfBetaIn\
v cdfBinomial cd\
fBinomialInv cdf\
Bvn cdfBvn2 cdfB\
vn2e cdfCauchy c\
dfCauchyInv cdfC\
hic cdfChii cdfC\
hinc cdfChincInv\
 cdfExp cdfExpIn\
v cdfFc cdfFnc c\
dfFncInv cdfGam \
cdfGenPareto cdf\
HyperGeo cdfLapl\
ace cdfLaplaceIn\
v cdfLogistic cd\
fLogisticInv cdf\
mControlCreate c\
dfMvn cdfMvn2e c\
dfMvnce cdfMvne \
cdfMvt2e cdfMvtc\
e cdfMvte cdfN c\
dfN2 cdfNc cdfNe\
gBinomial cdfNeg\
BinomialInv cdfN\
i cdfPoisson cdf\
PoissonInv cdfRa\
yleigh cdfRaylei\
ghInv cdfTc cdfT\
ci cdfTnc cdfTvn\
 cdfWeibull cdfW\
eibullInv cdir c\
eil ChangeDir ch\
dir chiBarSquare\
 chol choldn cho\
lsol cholup chrs\
 close code cols\
 colsf combinate\
 combinated comp\
lex con cond con\
j cons ConScore \
contour conv con\
vertsatostr conv\
ertstrtosa corrm\
 corrms corrvc c\
orrx corrxs cos \
cosh counts coun\
twts crossprd cr\
out croutp csrco\
l csrlin csvRead\
M csvReadSA cump\
rodc cumsumc cur\
ve cvtos datacre\
ate datacreateco\
mplex datalist d\
ataload dataloop\
 dataopen datasa\
ve date datestr \
datestring dates\
trymd dayinyr da\
yofweek dbAddDat\
abase dbClose db\
Commit dbCreateQ\
uery dbExecQuery\
 dbGetConnectOpt\
ions dbGetDataba\
seName dbGetDriv\
erName dbGetDriv\
ers dbGetHostNam\
e dbGetLastError\
Num dbGetLastErr\
orText dbGetNume\
ricalPrecPolicy \
dbGetPassword db\
GetPort dbGetTab\
leHeaders dbGetT\
ables dbGetUserN\
ame dbHasFeature\
 dbIsDriverAvail\
able dbIsOpen db\
IsOpenError dbOp\
en dbQueryBindVa\
lue dbQueryClear\
 dbQueryCols dbQ\
ueryExecPrepared\
 dbQueryFetchAll\
M dbQueryFetchAl\
lSA dbQueryFetch\
OneM dbQueryFetc\
hOneSA dbQueryFi\
nish dbQueryGetB\
oundValue dbQuer\
yGetBoundValues \
dbQueryGetField \
dbQueryGetLastEr\
rorNum dbQueryGe\
tLastErrorText d\
bQueryGetLastIns\
ertID dbQueryGet\
LastQuery dbQuer\
yGetPosition dbQ\
ueryIsActive dbQ\
ueryIsForwardOnl\
y dbQueryIsNull \
dbQueryIsSelect \
dbQueryIsValid d\
bQueryPrepare db\
QueryRows dbQuer\
ySeek dbQuerySee\
kFirst dbQuerySe\
ekLast dbQuerySe\
ekNext dbQuerySe\
ekPrevious dbQue\
rySetForwardOnly\
 dbRemoveDatabas\
e dbRollback dbS\
etConnectOptions\
 dbSetDatabaseNa\
me dbSetHostName\
 dbSetNumericalP\
recPolicy dbSetP\
ort dbSetUserNam\
e dbTransaction \
DeleteFile delif\
 delrows denseTo\
Sp denseToSpRE d\
enToZero design \
det detl dfft df\
fti diag diagrv \
digamma doswin D\
OSWinCloseall DO\
SWinOpen dotfeq \
dotfeqmt dotfge \
dotfgemt dotfgt \
dotfgtmt dotfle \
dotflemt dotflt \
dotfltmt dotfne \
dotfnemt draw dr\
op dsCreate dsta\
t dstatmt dstatm\
tControlCreate d\
tdate dtday dtti\
me dttodtv dttos\
tr dttoutc dtvno\
rmal dtvtodt dtv\
toutc dummy dumm\
ybr dummydn eig \
eigh eighv eigv \
elapsedTradingDa\
ys endwind envge\
t eof eqSolve eq\
Solvemt eqSolvem\
tControlCreate e\
qSolvemtOutCreat\
e eqSolveset erf\
 erfc erfccplx e\
rfcplx error etd\
ays ethsec etstr\
 EuropeanBinomCa\
ll EuropeanBinom\
Call_Greeks Euro\
peanBinomCall_Im\
pVol EuropeanBin\
omPut EuropeanBi\
nomPut_Greeks Eu\
ropeanBinomPut_I\
mpVol EuropeanBS\
Call EuropeanBSC\
all_Greeks Europ\
eanBSCall_ImpVol\
 EuropeanBSPut E\
uropeanBSPut_Gre\
eks EuropeanBSPu\
t_ImpVol exctsmp\
l exec execbg ex\
p extern eye fch\
eckerr fclearerr\
 feq feqmt fflus\
h fft ffti fftm \
fftmi fftn fge f\
gemt fgets fgets\
a fgetsat fgetst\
 fgt fgtmt filei\
nfo filesa fle f\
lemt floor flt f\
ltmt fmod fne fn\
emt fonts fopen \
formatcv formatn\
v fputs fputst f\
seek fstrerror f\
tell ftocv ftos \
ftostrC gamma ga\
mmacplx gammaii \
gausset gdaAppen\
d gdaCreate gdaD\
Stat gdaDStatMat\
 gdaGetIndex gda\
GetName gdaGetNa\
mes gdaGetOrders\
 gdaGetType gdaG\
etTypes gdaGetVa\
rInfo gdaIsCplx \
gdaLoad gdaPack \
gdaRead gdaReadB\
yIndex gdaReadSo\
me gdaReadSparse\
 gdaReadStruct g\
daReportVarInfo \
gdaSave gdaUpdat\
e gdaUpdateAndPa\
ck gdaVars gdaWr\
ite gdaWrite32 g\
daWriteSome geta\
rray getdims get\
f getGAUSShome g\
etmatrix getmatr\
ix4D getname get\
namef getNextTra\
dingDay getNextW\
eekDay getnr get\
orders getpath g\
etPreviousTradin\
gDay getPrevious\
WeekDay getRow g\
etscalar3D getsc\
alar4D getTrRow \
getwind glm grad\
cplx gradMT grad\
MTm gradMTT grad\
MTTm gradp graph\
prt graphset has\
imag header head\
ermt hess hessMT\
 hessMTg hessMTg\
w hessMTm hessMT\
mw hessMTT hessM\
TTg hessMTTgw he\
ssMTTm hessMTw h\
essp hist histf \
histp hsec imag \
indcv indexcat i\
ndices indices2 \
indicesf indices\
fn indnv indsav \
integrate1d inte\
grateControlCrea\
te intgrat2 intg\
rat3 inthp1 inth\
p2 inthp3 inthp4\
 inthpControlCre\
ate intquad1 int\
quad2 intquad3 i\
ntrleav intrleav\
sa intrsect ints\
imp inv invpd in\
vswp iscplx iscp\
lxf isden isinfn\
anmiss ismiss ke\
y keyav keyw lag\
 lag1 lagn lapEi\
ghb lapEighi lap\
Eighvb lapEighvi\
 lapgEig lapgEig\
h lapgEighv lapg\
Eigv lapgSchur l\
apgSvdcst lapgSv\
ds lapgSvdst lap\
Svdcusv lapSvds \
lapSvdusv ldlp l\
dlsol linSolve l\
istwise ln lncdf\
bvn lncdfbvn2 ln\
cdfmvn lncdfn ln\
cdfn2 lncdfnc ln\
fact lngammacplx\
 lnpdfmvn lnpdfm\
vt lnpdfn lnpdft\
 loadd loadstruc\
t loadwind loess\
 loessmt loessmt\
ControlCreate lo\
g loglog logx lo\
gy lower lowmat \
lowmat1 ltrisol \
lu lusol machEps\
ilon make makeva\
rs makewind marg\
in matalloc mati\
nit mattoarray m\
axbytes maxc max\
indc maxv maxvec\
 mbesselei mbess\
elei0 mbesselei1\
 mbesseli mbesse\
li0 mbesseli1 me\
anc median merge\
by mergevar minc\
 minindc minv mi\
ss missex missrv\
 moment momentd \
movingave moving\
aveExpwgt moving\
aveWgt nextindex\
 nextn nextnevn \
nextwind ntos nu\
ll null1 numComb\
inations ols ols\
mt olsmtControlC\
reate olsqr olsq\
r2 olsqrmt ones \
optn optnevn ort\
h outtyp pacf pa\
ckedToSp packr p\
arse pause pdfCa\
uchy pdfChi pdfE\
xp pdfGenPareto \
pdfHyperGeo pdfL\
aplace pdfLogist\
ic pdfn pdfPoiss\
on pdfRayleigh p\
dfWeibull pi pin\
v pinvmt plotAdd\
Arrow plotAddBar\
 plotAddBox plot\
AddHist plotAddH\
istF plotAddHist\
P plotAddPolar p\
lotAddScatter pl\
otAddShape plotA\
ddTextbox plotAd\
dTS plotAddXY pl\
otArea plotBar p\
lotBox plotClear\
Layout plotConto\
ur plotCustomLay\
out plotGetDefau\
lts plotHist plo\
tHistF plotHistP\
 plotLayout plot\
LogLog plotLogX \
plotLogY plotOpe\
nWindow plotPola\
r plotSave plotS\
catter plotSetAx\
esPen plotSetBar\
 plotSetBarFill \
plotSetBarStacke\
d plotSetBkdColo\
r plotSetFill pl\
otSetGrid plotSe\
tLegend plotSetL\
ineColor plotSet\
LineStyle plotSe\
tLineSymbol plot\
SetLineThickness\
 plotSetNewWindo\
w plotSetTitle p\
lotSetWhichYAxis\
 plotSetXAxisSho\
w plotSetXLabel \
plotSetXRange pl\
otSetXTicInterva\
l plotSetXTicLab\
el plotSetYAxisS\
how plotSetYLabe\
l plotSetYRange \
plotSetZAxisShow\
 plotSetZLabel p\
lotSurface plotT\
S plotXY polar p\
olychar polyeval\
 polygamma polyi\
nt polymake poly\
mat polymroot po\
lymult polyroot \
pqgwin previousi\
ndex princomp pr\
intfm printfmt p\
rodc psi putarra\
y putf putvals p\
vCreate pvGetInd\
ex pvGetParNames\
 pvGetParVector \
pvLength pvList \
pvPack pvPacki p\
vPackm pvPackmi \
pvPacks pvPacksi\
 pvPacksm pvPack\
smi pvPutParVect\
or pvTest pvUnpa\
ck QNewton QNewt\
onmt QNewtonmtCo\
ntrolCreate QNew\
tonmtOutCreate Q\
NewtonSet QProg \
QProgmt QProgmtI\
nCreate qqr qqre\
 qqrep qr qre qr\
ep qrsol qrtsol \
qtyr qtyre qtyre\
p quantile quant\
iled qyr qyre qy\
rep qz rank rank\
indx readr real \
reclassify recla\
ssifyCuts recode\
 recserar recser\
cp recserrc reru\
n rescale reshap\
e rets rev rfft \
rffti rfftip rff\
tn rfftnp rfftp \
rndBernoulli rnd\
Beta rndBinomial\
 rndCauchy rndCh\
iSquare rndCon r\
ndCreateState rn\
dExp rndGamma rn\
dGeo rndGumbel r\
ndHyperGeo rndi \
rndKMbeta rndKMg\
am rndKMi rndKMn\
 rndKMnb rndKMp \
rndKMu rndKMvm r\
ndLaplace rndLCb\
eta rndLCgam rnd\
LCi rndLCn rndLC\
nb rndLCp rndLCu\
 rndLCvm rndLogN\
orm rndMTu rndMV\
n rndMVt rndn rn\
dnb rndNegBinomi\
al rndp rndPoiss\
on rndRayleigh r\
ndStateSkip rndu\
 rndvm rndWeibul\
l rndWishart rot\
ater round rows \
rowsf rref sampl\
eData satostrC s\
aved saveStruct \
savewind scale s\
cale3d scalerr s\
calinfnanmiss sc\
almiss schtoc sc\
hur searchsource\
path seekr selec\
t selif seqa seq\
m setdif setdifs\
a setvars setvwr\
mode setwind she\
ll shiftr sin si\
ngleindex sinh s\
leep solpd sortc\
 sortcc sortd so\
rthc sorthcc sor\
tind sortindc so\
rtmc sortr sortr\
c spBiconjGradSo\
l spChol spConjG\
radSol spCreate \
spDenseSubmat sp\
DiagRvMat spEigv\
 spEye spLDL spl\
ine spLU spNumNZ\
E spOnes spreadS\
heetReadM spread\
SheetReadSA spre\
adSheetWrite spS\
cale spSubmat sp\
ToDense spTrTDen\
se spTScalar spZ\
eros sqpSolve sq\
pSolveMT sqpSolv\
eMTControlCreate\
 sqpSolveMTlagra\
ngeCreate sqpSol\
veMToutCreate sq\
pSolveSet sqrt s\
tatements stdc s\
tdsc stocv stof \
strcombine strin\
dx strlen strput\
 strrindx strsec\
t strsplit strsp\
litPad strtodt s\
trtof strtofcplx\
 strtriml strtri\
mr strtrunc strt\
runcl strtruncpa\
d strtruncr subm\
at subscat subst\
ute subvec sumc \
sumr surface svd\
 svd1 svd2 svdcu\
sv svds svdusv s\
ysstate tab tan \
tanh tempname ti\
me timedt timest\
r timeutc title \
tkf2eps tkf2ps t\
ocart todaydt to\
eplitz token top\
olar trapchk tri\
gamma trimr trun\
c type typecv ty\
pef union unions\
a uniqindx uniqi\
ndxsa unique uni\
quesa upmat upma\
t1 upper utctodt\
 utctodtv utriso\
l vals varCovMS \
varCovXS varget \
vargetl varmall \
varmares varput \
varputl vartypef\
 vcm vcms vcx vc\
xs vec vech vecr\
 vector vget vie\
w viewxyz vlist \
vnamecv volume v\
put vread vtypec\
v wait waitc wal\
kindex where win\
dow writer xlabe\
l xlsGetSheetCou\
nt xlsGetSheetSi\
ze xlsGetSheetTy\
pes xlsMakeRange\
 xlsReadM xlsRea\
dSA xlsWrite xls\
WriteM xlsWriteS\
A xpnd xtics xy \
xyz ylabel ytics\
 zeros zeta zlab\
el ztics cdfEmpi\
rical dot h5crea\
te h5open h5read\
 h5readAttribute\
 h5write h5write\
Attribute ldl pl\
otAddErrorBar pl\
otAddSurface plo\
tCDFEmpirical pl\
otSetColormap pl\
otSetContourLabe\
ls plotSetLegend\
Font plotSetText\
Interpreter plot\
SetXTicCount plo\
tSetYTicCount pl\
otSetZLevels pow\
erm strjoin sylv\
ester strtrim\x22,\x0a\
literal:\x22DB_AFTE\
R_LAST_ROW DB_AL\
L_TABLES DB_BATC\
H_OPERATIONS DB_\
BEFORE_FIRST_ROW\
 DB_BLOB DB_EVEN\
T_NOTIFICATIONS \
DB_FINISH_QUERY \
DB_HIGH_PRECISIO\
N DB_LAST_INSERT\
_ID DB_LOW_PRECI\
SION_DOUBLE DB_L\
OW_PRECISION_INT\
32 DB_LOW_PRECIS\
ION_INT64 DB_LOW\
_PRECISION_NUMBE\
RS DB_MULTIPLE_R\
ESULT_SETS DB_NA\
MED_PLACEHOLDERS\
 DB_POSITIONAL_P\
LACEHOLDERS DB_P\
REPARED_QUERIES \
DB_QUERY_SIZE DB\
_SIMPLE_LOCKING \
DB_SYSTEM_TABLES\
 DB_TABLES DB_TR\
ANSACTIONS DB_UN\
ICODE DB_VIEWS _\
_STDIN __STDOUT \
__STDERR __FILE_\
DIR\x22\x0a},a=e.COMME\
NT(\x22@\x22,\x22@\x22),r={c\
lassName:\x22meta\x22,\
begin:\x22#\x22,end:\x22$\
\x22,keywords:{\x0akey\
word:\x22define def\
inecs|10 undef i\
fdef ifndef ifli\
ght ifdllcall if\
mac ifos2win ifu\
nix else endif l\
ineson linesoff \
srcfile srcline\x22\
\x0a},contains:[{be\
gin:/\x5c\x5c\x5cn/,relev\
ance:0},{beginKe\
ywords:\x22include\x22\
,end:\x22$\x22,\x0akeywor\
ds:{keyword:\x22inc\
lude\x22},contains:\
[{className:\x22str\
ing\x22,begin:'\x22',e\
nd:'\x22',\x0aillegal:\
\x22\x5c\x5cn\x22}]},e.C_LIN\
E_COMMENT_MODE,e\
.C_BLOCK_COMMENT\
_MODE,a]},n={\x0abe\
gin:/\x5cbstruct\x5cs+\
/,end:/\x5cs/,keywo\
rds:\x22struct\x22,con\
tains:[{classNam\
e:\x22type\x22,\x0abegin:\
e.UNDERSCORE_IDE\
NT_RE,relevance:\
0}]},s=[{classNa\
me:\x22params\x22,begi\
n:/\x5c(/,\x0aend:/\x5c)/\
,excludeBegin:!0\
,excludeEnd:!0,e\
ndsWithParent:!0\
,relevance:0,con\
tains:[{\x0aclassNa\
me:\x22literal\x22,beg\
in:/\x5c.\x5c.\x5c./},e.C\
_NUMBER_MODE,e.C\
_BLOCK_COMMENT_M\
ODE,a,n]\x0a}],o={c\
lassName:\x22title\x22\
,begin:e.UNDERSC\
ORE_IDENT_RE,rel\
evance:0},d=(t,r\
,n)=>{\x0aconst d=e\
.inherit({classN\
ame:\x22function\x22,b\
eginKeywords:t,e\
nd:r,excludeEnd:\
!0,\x0acontains:[].\
concat(s)},{})\x0a;\
return d.contain\
s.push(o),d.cont\
ains.push(e.C_NU\
MBER_MODE),\x0ad.co\
ntains.push(e.C_\
BLOCK_COMMENT_MO\
DE),d.contains.p\
ush(a),d},l={\x0acl\
assName:\x22built_i\
n\x22,begin:\x22\x5c\x5cb(\x22+\
t.built_in.split\
(\x22 \x22).join(\x22|\x22)+\
\x22)\x5c\x5cb\x22},i={\x0aclas\
sName:\x22string\x22,b\
egin:'\x22',end:'\x22'\
,contains:[e.BAC\
KSLASH_ESCAPE],r\
elevance:0\x0a},c={\
begin:e.UNDERSCO\
RE_IDENT_RE+\x22\x5c\x5cs\
*\x5c\x5c(\x22,returnBegi\
n:!0,keywords:t,\
\x0arelevance:0,con\
tains:[{beginKey\
words:t.keyword}\
,l,{className:\x22b\
uilt_in\x22,\x0abegin:\
e.UNDERSCORE_IDE\
NT_RE,relevance:\
0}]},p={begin:/\x5c\
(/,end:/\x5c)/,rele\
vance:0,\x0akeyword\
s:{built_in:t.bu\
ilt_in,literal:t\
.literal},\x0aconta\
ins:[e.C_NUMBER_\
MODE,e.C_BLOCK_C\
OMMENT_MODE,a,l,\
c,i,\x22self\x22]}\x0a;re\
turn c.contains.\
push(p),{name:\x22G\
AUSS\x22,aliases:[\x22\
gss\x22],case_insen\
sitive:!0,\x0akeywo\
rds:t,illegal:/(\
\x5c{[%#]|[%#]\x5c}| <\
- )/,\x0acontains:[\
e.C_NUMBER_MODE,\
e.C_LINE_COMMENT\
_MODE,e.C_BLOCK_\
COMMENT_MODE,a,i\
,r,{\x0aclassName:\x22\
keyword\x22,\x0abegin:\
/\x5cbexternal (mat\
rix|string|array\
|sparse matrix|s\
truct|proc|keywo\
rd|fn)/\x0a},d(\x22pro\
c keyword\x22,\x22;\x22),\
d(\x22fn\x22,\x22=\x22),{beg\
inKeywords:\x22for \
threadfor\x22,end:/\
;/,\x0arelevance:0,\
contains:[e.C_BL\
OCK_COMMENT_MODE\
,a,p]},{variants\
:[{\x0abegin:e.UNDE\
RSCORE_IDENT_RE+\
\x22\x5c\x5c.\x22+e.UNDERSCO\
RE_IDENT_RE},{\x0ab\
egin:e.UNDERSCOR\
E_IDENT_RE+\x22\x5c\x5cs*\
=\x22}],relevance:0\
},c,n]}}})()\x0a;hl\
js.registerLangu\
age(\x22gauss\x22,e)})\
();/*! `gcode` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>{const a=e.\
inherit(e.C_NUMB\
ER_MODE,{\x0abegin:\
\x22([-+]?((\x5c\x5c.\x5c\x5cd+\
)|(\x5c\x5cd+)(\x5c\x5c.\x5c\x5cd*\
)?))|\x22+e.C_NUMBE\
R_RE\x0a}),n=[e.C_L\
INE_COMMENT_MODE\
,e.C_BLOCK_COMME\
NT_MODE,e.COMMEN\
T(/\x5c(/,/\x5c)/),a,e\
.inherit(e.APOS_\
STRING_MODE,{\x0ail\
legal:null}),e.i\
nherit(e.QUOTE_S\
TRING_MODE,{ille\
gal:null}),{clas\
sName:\x22name\x22,\x0abe\
gin:\x22([G])([0-9]\
+\x5c\x5c.?[0-9]?)\x22},{\
className:\x22name\x22\
,\x0abegin:\x22([M])([\
0-9]+\x5c\x5c.?[0-9]?)\
\x22},{className:\x22a\
ttr\x22,begin:\x22(VC|\
VS|#)\x22,\x0aend:\x22(\x5c\x5c\
d+)\x22},{className\
:\x22attr\x22,begin:\x22(\
VZOFX|VZOFY|VZOF\
Z)\x22},{\x0aclassName\
:\x22built_in\x22,\x0abeg\
in:\x22(ATAN|ABS|AC\
OS|ASIN|SIN|COS|\
EXP|FIX|FUP|ROUN\
D|LN|TAN)(\x5c\x5c[)\x22,\
contains:[a],\x0aen\
d:\x22\x5c\x5c]\x22},{classN\
ame:\x22symbol\x22,var\
iants:[{begin:\x22N\
\x22,end:\x22\x5c\x5cd+\x22,ill\
egal:\x22\x5c\x5cW\x22}]}]\x0a;\
return{name:\x22G-c\
ode (ISO 6983)\x22,\
aliases:[\x22nc\x22],c\
ase_insensitive:\
!0,keywords:{\x0a$p\
attern:\x22[A-Z_][A\
-Z0-9_.]*\x22,\x0akeyw\
ord:\x22IF DO WHILE\
 ENDWHILE CALL E\
NDIF SUB ENDSUB \
GOTO REPEAT ENDR\
EPEAT EQ LT GT N\
E GE LE OR XOR\x22\x0a\
},contains:[{cla\
ssName:\x22meta\x22,be\
gin:\x22%\x22},{classN\
ame:\x22meta\x22,begin\
:\x22([O])([0-9]+)\x22\
\x0a}].concat(n)}}}\
)();hljs.registe\
rLanguage(\x22gcode\
\x22,e)})();/*! `gh\
erkin` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;return e=>({n\
ame:\x22Gherkin\x22,al\
iases:[\x22feature\x22\
],\x0akeywords:\x22Fea\
ture Background \
Ability Business\
 Need Scenario S\
cenarios Scenari\
o Outline Scenar\
io Template Exam\
ples Given And T\
hen But When\x22,\x0ac\
ontains:[{classN\
ame:\x22symbol\x22,beg\
in:\x22\x5c\x5c*\x22,relevan\
ce:0},{className\
:\x22meta\x22,\x0abegin:\x22\
@[^@\x5c\x5cs]+\x22},{beg\
in:\x22\x5c\x5c|\x22,end:\x22\x5c\x5c\
|\x5c\x5cw*$\x22,contains\
:[{className:\x22st\
ring\x22,\x0abegin:\x22[^\
|]+\x22}]},{classNa\
me:\x22variable\x22,be\
gin:\x22<\x22,end:\x22>\x22}\
,e.HASH_COMMENT_\
MODE,{\x0aclassName\
:\x22string\x22,begin:\
'\x22\x22\x22',end:'\x22\x22\x22'}\
,e.QUOTE_STRING_\
MODE]})})()\x0a;hlj\
s.registerLangua\
ge(\x22gherkin\x22,e)}\
)();/*! `glsl` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>({name:\x22GLS\
L\x22,keywords:{\x0ake\
yword:\x22break con\
tinue discard do\
 else for if ret\
urn while switch\
 case default at\
tribute binding \
buffer ccw centr\
oid centroid var\
ying coherent co\
lumn_major const\
 cw depth_any de\
pth_greater dept\
h_less depth_unc\
hanged early_fra\
gment_tests equa\
l_spacing flat f\
ractional_even_s\
pacing fractiona\
l_odd_spacing hi\
ghp in index ino\
ut invariant inv\
ocations isoline\
s layout line_st\
rip lines lines_\
adjacency local_\
size_x local_siz\
e_y local_size_z\
 location lowp m\
ax_vertices medi\
ump noperspectiv\
e offset origin_\
upper_left out p\
acked patch pixe\
l_center_integer\
 point_mode poin\
ts precise preci\
sion quads r11f_\
g11f_b10f r16 r1\
6_snorm r16f r16\
i r16ui r32f r32\
i r32ui r8 r8_sn\
orm r8i r8ui rea\
donly restrict r\
g16 rg16_snorm r\
g16f rg16i rg16u\
i rg32f rg32i rg\
32ui rg8 rg8_sno\
rm rg8i rg8ui rg\
b10_a2 rgb10_a2u\
i rgba16 rgba16_\
snorm rgba16f rg\
ba16i rgba16ui r\
gba32f rgba32i r\
gba32ui rgba8 rg\
ba8_snorm rgba8i\
 rgba8ui row_maj\
or sample shared\
 smooth std140 s\
td430 stream tri\
angle_strip tria\
ngles triangles_\
adjacency unifor\
m varying vertic\
es volatile writ\
eonly\x22,\x0atype:\x22at\
omic_uint bool b\
vec2 bvec3 bvec4\
 dmat2 dmat2x2 d\
mat2x3 dmat2x4 d\
mat3 dmat3x2 dma\
t3x3 dmat3x4 dma\
t4 dmat4x2 dmat4\
x3 dmat4x4 doubl\
e dvec2 dvec3 dv\
ec4 float iimage\
1D iimage1DArray\
 iimage2D iimage\
2DArray iimage2D\
MS iimage2DMSArr\
ay iimage2DRect \
iimage3D iimageB\
uffer iimageCube\
 iimageCubeArray\
 image1D image1D\
Array image2D im\
age2DArray image\
2DMS image2DMSAr\
ray image2DRect \
image3D imageBuf\
fer imageCube im\
ageCubeArray int\
 isampler1D isam\
pler1DArray isam\
pler2D isampler2\
DArray isampler2\
DMS isampler2DMS\
Array isampler2D\
Rect isampler3D \
isamplerBuffer i\
samplerCube isam\
plerCubeArray iv\
ec2 ivec3 ivec4 \
mat2 mat2x2 mat2\
x3 mat2x4 mat3 m\
at3x2 mat3x3 mat\
3x4 mat4 mat4x2 \
mat4x3 mat4x4 sa\
mpler1D sampler1\
DArray sampler1D\
ArrayShadow samp\
ler1DShadow samp\
ler2D sampler2DA\
rray sampler2DAr\
rayShadow sample\
r2DMS sampler2DM\
SArray sampler2D\
Rect sampler2DRe\
ctShadow sampler\
2DShadow sampler\
3D samplerBuffer\
 samplerCube sam\
plerCubeArray sa\
mplerCubeArraySh\
adow samplerCube\
Shadow image1D u\
image1DArray uim\
age2D uimage2DAr\
ray uimage2DMS u\
image2DMSArray u\
image2DRect uima\
ge3D uimageBuffe\
r uimageCube uim\
ageCubeArray uin\
t usampler1D usa\
mpler1DArray usa\
mpler2D usampler\
2DArray usampler\
2DMS usampler2DM\
SArray usampler2\
DRect usampler3D\
 samplerBuffer u\
samplerCube usam\
plerCubeArray uv\
ec2 uvec3 uvec4 \
vec2 vec3 vec4 v\
oid\x22,\x0abuilt_in:\x22\
gl_MaxAtomicCoun\
terBindings gl_M\
axAtomicCounterB\
ufferSize gl_Max\
ClipDistances gl\
_MaxClipPlanes g\
l_MaxCombinedAto\
micCounterBuffer\
s gl_MaxCombined\
AtomicCounters g\
l_MaxCombinedIma\
geUniforms gl_Ma\
xCombinedImageUn\
itsAndFragmentOu\
tputs gl_MaxComb\
inedTextureImage\
Units gl_MaxComp\
uteAtomicCounter\
Buffers gl_MaxCo\
mputeAtomicCount\
ers gl_MaxComput\
eImageUniforms g\
l_MaxComputeText\
ureImageUnits gl\
_MaxComputeUnifo\
rmComponents gl_\
MaxComputeWorkGr\
oupCount gl_MaxC\
omputeWorkGroupS\
ize gl_MaxDrawBu\
ffers gl_MaxFrag\
mentAtomicCounte\
rBuffers gl_MaxF\
ragmentAtomicCou\
nters gl_MaxFrag\
mentImageUniform\
s gl_MaxFragment\
InputComponents \
gl_MaxFragmentIn\
putVectors gl_Ma\
xFragmentUniform\
Components gl_Ma\
xFragmentUniform\
Vectors gl_MaxGe\
ometryAtomicCoun\
terBuffers gl_Ma\
xGeometryAtomicC\
ounters gl_MaxGe\
ometryImageUnifo\
rms gl_MaxGeomet\
ryInputComponent\
s gl_MaxGeometry\
OutputComponents\
 gl_MaxGeometryO\
utputVertices gl\
_MaxGeometryText\
ureImageUnits gl\
_MaxGeometryTota\
lOutputComponent\
s gl_MaxGeometry\
UniformComponent\
s gl_MaxGeometry\
VaryingComponent\
s gl_MaxImageSam\
ples gl_MaxImage\
Units gl_MaxLigh\
ts gl_MaxPatchVe\
rtices gl_MaxPro\
gramTexelOffset \
gl_MaxTessContro\
lAtomicCounterBu\
ffers gl_MaxTess\
ControlAtomicCou\
nters gl_MaxTess\
ControlImageUnif\
orms gl_MaxTessC\
ontrolInputCompo\
nents gl_MaxTess\
ControlOutputCom\
ponents gl_MaxTe\
ssControlTexture\
ImageUnits gl_Ma\
xTessControlTota\
lOutputComponent\
s gl_MaxTessCont\
rolUniformCompon\
ents gl_MaxTessE\
valuationAtomicC\
ounterBuffers gl\
_MaxTessEvaluati\
onAtomicCounters\
 gl_MaxTessEvalu\
ationImageUnifor\
ms gl_MaxTessEva\
luationInputComp\
onents gl_MaxTes\
sEvaluationOutpu\
tComponents gl_M\
axTessEvaluation\
TextureImageUnit\
s gl_MaxTessEval\
uationUniformCom\
ponents gl_MaxTe\
ssGenLevel gl_Ma\
xTessPatchCompon\
ents gl_MaxTextu\
reCoords gl_MaxT\
extureImageUnits\
 gl_MaxTextureUn\
its gl_MaxVaryin\
gComponents gl_M\
axVaryingFloats \
gl_MaxVaryingVec\
tors gl_MaxVerte\
xAtomicCounterBu\
ffers gl_MaxVert\
exAtomicCounters\
 gl_MaxVertexAtt\
ribs gl_MaxVerte\
xImageUniforms g\
l_MaxVertexOutpu\
tComponents gl_M\
axVertexOutputVe\
ctors gl_MaxVert\
exTextureImageUn\
its gl_MaxVertex\
UniformComponent\
s gl_MaxVertexUn\
iformVectors gl_\
MaxViewports gl_\
MinProgramTexelO\
ffset gl_BackCol\
or gl_BackLightM\
odelProduct gl_B\
ackLightProduct \
gl_BackMaterial \
gl_BackSecondary\
Color gl_ClipDis\
tance gl_ClipPla\
ne gl_ClipVertex\
 gl_Color gl_Dep\
thRange gl_EyePl\
aneQ gl_EyePlane\
R gl_EyePlaneS g\
l_EyePlaneT gl_F\
og gl_FogCoord g\
l_FogFragCoord g\
l_FragColor gl_F\
ragCoord gl_Frag\
Data gl_FragDept\
h gl_FrontColor \
gl_FrontFacing g\
l_FrontLightMode\
lProduct gl_Fron\
tLightProduct gl\
_FrontMaterial g\
l_FrontSecondary\
Color gl_GlobalI\
nvocationID gl_I\
nstanceID gl_Inv\
ocationID gl_Lay\
er gl_LightModel\
 gl_LightSource \
gl_LocalInvocati\
onID gl_LocalInv\
ocationIndex gl_\
ModelViewMatrix \
gl_ModelViewMatr\
ixInverse gl_Mod\
elViewMatrixInve\
rseTranspose gl_\
ModelViewMatrixT\
ranspose gl_Mode\
lViewProjectionM\
atrix gl_ModelVi\
ewProjectionMatr\
ixInverse gl_Mod\
elViewProjection\
MatrixInverseTra\
nspose gl_ModelV\
iewProjectionMat\
rixTranspose gl_\
MultiTexCoord0 g\
l_MultiTexCoord1\
 gl_MultiTexCoor\
d2 gl_MultiTexCo\
ord3 gl_MultiTex\
Coord4 gl_MultiT\
exCoord5 gl_Mult\
iTexCoord6 gl_Mu\
ltiTexCoord7 gl_\
Normal gl_Normal\
Matrix gl_Normal\
Scale gl_NumSamp\
les gl_NumWorkGr\
oups gl_ObjectPl\
aneQ gl_ObjectPl\
aneR gl_ObjectPl\
aneS gl_ObjectPl\
aneT gl_PatchVer\
ticesIn gl_Point\
 gl_PointCoord g\
l_PointSize gl_P\
osition gl_Primi\
tiveID gl_Primit\
iveIDIn gl_Proje\
ctionMatrix gl_P\
rojectionMatrixI\
nverse gl_Projec\
tionMatrixInvers\
eTranspose gl_Pr\
ojectionMatrixTr\
anspose gl_Sampl\
eID gl_SampleMas\
k gl_SampleMaskI\
n gl_SamplePosit\
ion gl_Secondary\
Color gl_TessCoo\
rd gl_TessLevelI\
nner gl_TessLeve\
lOuter gl_TexCoo\
rd gl_TextureEnv\
Color gl_Texture\
Matrix gl_Textur\
eMatrixInverse g\
l_TextureMatrixI\
nverseTranspose \
gl_TextureMatrix\
Transpose gl_Ver\
tex gl_VertexID \
gl_ViewportIndex\
 gl_WorkGroupID \
gl_WorkGroupSize\
 gl_in gl_out Em\
itStreamVertex E\
mitVertex EndPri\
mitive EndStream\
Primitive abs ac\
os acosh all any\
 asin asinh atan\
 atanh atomicAdd\
 atomicAnd atomi\
cCompSwap atomic\
Counter atomicCo\
unterDecrement a\
tomicCounterIncr\
ement atomicExch\
ange atomicMax a\
tomicMin atomicO\
r atomicXor barr\
ier bitCount bit\
fieldExtract bit\
fieldInsert bitf\
ieldReverse ceil\
 clamp cos cosh \
cross dFdx dFdy \
degrees determin\
ant distance dot\
 equal exp exp2 \
faceforward find\
LSB findMSB floa\
tBitsToInt float\
BitsToUint floor\
 fma fract frexp\
 ftransform fwid\
th greaterThan g\
reaterThanEqual \
groupMemoryBarri\
er imageAtomicAd\
d imageAtomicAnd\
 imageAtomicComp\
Swap imageAtomic\
Exchange imageAt\
omicMax imageAto\
micMin imageAtom\
icOr imageAtomic\
Xor imageLoad im\
ageSize imageSto\
re imulExtended \
intBitsToFloat i\
nterpolateAtCent\
roid interpolate\
AtOffset interpo\
lateAtSample inv\
erse inversesqrt\
 isinf isnan lde\
xp length lessTh\
an lessThanEqual\
 log log2 matrix\
CompMult max mem\
oryBarrier memor\
yBarrierAtomicCo\
unter memoryBarr\
ierBuffer memory\
BarrierImage mem\
oryBarrierShared\
 min mix mod mod\
f noise1 noise2 \
noise3 noise4 no\
rmalize not notE\
qual outerProduc\
t packDouble2x32\
 packHalf2x16 pa\
ckSnorm2x16 pack\
Snorm4x8 packUno\
rm2x16 packUnorm\
4x8 pow radians \
reflect refract \
round roundEven \
shadow1D shadow1\
DLod shadow1DPro\
j shadow1DProjLo\
d shadow2D shado\
w2DLod shadow2DP\
roj shadow2DProj\
Lod sign sin sin\
h smoothstep sqr\
t step tan tanh \
texelFetch texel\
FetchOffset text\
ure texture1D te\
xture1DLod textu\
re1DProj texture\
1DProjLod textur\
e2D texture2DLod\
 texture2DProj t\
exture2DProjLod \
texture3D textur\
e3DLod texture3D\
Proj texture3DPr\
ojLod textureCub\
e textureCubeLod\
 textureGather t\
extureGatherOffs\
et textureGather\
Offsets textureG\
rad textureGradO\
ffset textureLod\
 textureLodOffse\
t textureOffset \
textureProj text\
ureProjGrad text\
ureProjGradOffse\
t textureProjLod\
 textureProjLodO\
ffset texturePro\
jOffset textureQ\
ueryLevels textu\
reQueryLod textu\
reSize transpose\
 trunc uaddCarry\
 uintBitsToFloat\
 umulExtended un\
packDouble2x32 u\
npackHalf2x16 un\
packSnorm2x16 un\
packSnorm4x8 unp\
ackUnorm2x16 unp\
ackUnorm4x8 usub\
Borrow\x22,\x0aliteral\
:\x22true false\x22},i\
llegal:'\x22',\x0acont\
ains:[e.C_LINE_C\
OMMENT_MODE,e.C_\
BLOCK_COMMENT_MO\
DE,e.C_NUMBER_MO\
DE,{\x0aclassName:\x22\
meta\x22,begin:\x22#\x22,\
end:\x22$\x22}]})})();\
hljs.registerLan\
guage(\x22glsl\x22,e)}\
)();/*! `gml` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>({name:\x22GML\x22\
,case_insensitiv\
e:!1,\x0akeywords:{\
\x0akeyword:[\x22#endr\
egion\x22,\x22#macro\x22,\
\x22#region\x22,\x22and\x22,\
\x22begin\x22,\x22break\x22,\
\x22case\x22,\x22construc\
tor\x22,\x22continue\x22,\
\x22default\x22,\x22delet\
e\x22,\x22div\x22,\x22do\x22,\x22e\
lse\x22,\x22end\x22,\x22enum\
\x22,\x22exit\x22,\x22for\x22,\x22\
function\x22,\x22globa\
lvar\x22,\x22if\x22,\x22mod\x22\
,\x22new\x22,\x22not\x22,\x22or\
\x22,\x22repeat\x22,\x22retu\
rn\x22,\x22static\x22,\x22sw\
itch\x22,\x22then\x22,\x22un\
til\x22,\x22var\x22,\x22whil\
e\x22,\x22with\x22,\x22xor\x22]\
,\x0abuilt_in:[\x22abs\
\x22,\x22alarm_get\x22,\x22a\
larm_set\x22,\x22angle\
_difference\x22,\x22an\
imcurve_channel_\
evaluate\x22,\x22animc\
urve_channel_new\
\x22,\x22animcurve_cre\
ate\x22,\x22animcurve_\
destroy\x22,\x22animcu\
rve_exists\x22,\x22ani\
mcurve_get\x22,\x22ani\
mcurve_get_chann\
el\x22,\x22animcurve_g\
et_channel_index\
\x22,\x22animcurve_poi\
nt_new\x22,\x22ansi_ch\
ar\x22,\x22application\
_get_position\x22,\x22\
application_surf\
ace_draw_enable\x22\
,\x22application_su\
rface_enable\x22,\x22a\
pplication_surfa\
ce_is_enabled\x22,\x22\
arccos\x22,\x22arcsin\x22\
,\x22arctan\x22,\x22arcta\
n2\x22,\x22array_all\x22,\
\x22array_any\x22,\x22arr\
ay_concat\x22,\x22arra\
y_contains\x22,\x22arr\
ay_contains_ext\x22\
,\x22array_copy\x22,\x22a\
rray_copy_while\x22\
,\x22array_create\x22,\
\x22array_create_ex\
t\x22,\x22array_delete\
\x22,\x22array_equals\x22\
,\x22array_filter\x22,\
\x22array_filter_ex\
t\x22,\x22array_find_i\
ndex\x22,\x22array_fir\
st\x22,\x22array_forea\
ch\x22,\x22array_get\x22,\
\x22array_get_index\
\x22,\x22array_insert\x22\
,\x22array_intersec\
tion\x22,\x22array_las\
t\x22,\x22array_length\
\x22,\x22array_map\x22,\x22a\
rray_map_ext\x22,\x22a\
rray_pop\x22,\x22array\
_push\x22,\x22array_re\
duce\x22,\x22array_res\
ize\x22,\x22array_reve\
rse\x22,\x22array_reve\
rse_ext\x22,\x22array_\
set\x22,\x22array_shuf\
fle\x22,\x22array_shuf\
fle_ext\x22,\x22array_\
sort\x22,\x22array_uni\
on\x22,\x22array_uniqu\
e\x22,\x22array_unique\
_ext\x22,\x22asset_add\
_tags\x22,\x22asset_cl\
ear_tags\x22,\x22asset\
_get_ids\x22,\x22asset\
_get_index\x22,\x22ass\
et_get_tags\x22,\x22as\
set_get_type\x22,\x22a\
sset_has_any_tag\
\x22,\x22asset_has_tag\
s\x22,\x22asset_remove\
_tags\x22,\x22audio_bu\
s_clear_emitters\
\x22,\x22audio_bus_cre\
ate\x22,\x22audio_bus_\
get_emitters\x22,\x22a\
udio_channel_num\
\x22,\x22audio_create_\
buffer_sound\x22,\x22a\
udio_create_play\
_queue\x22,\x22audio_c\
reate_stream\x22,\x22a\
udio_create_sync\
_group\x22,\x22audio_d\
ebug\x22,\x22audio_des\
troy_stream\x22,\x22au\
dio_destroy_sync\
_group\x22,\x22audio_e\
ffect_create\x22,\x22a\
udio_emitter_bus\
\x22,\x22audio_emitter\
_create\x22,\x22audio_\
emitter_exists\x22,\
\x22audio_emitter_f\
alloff\x22,\x22audio_e\
mitter_free\x22,\x22au\
dio_emitter_gain\
\x22,\x22audio_emitter\
_get_bus\x22,\x22audio\
_emitter_get_gai\
n\x22,\x22audio_emitte\
r_get_listener_m\
ask\x22,\x22audio_emit\
ter_get_pitch\x22,\x22\
audio_emitter_ge\
t_vx\x22,\x22audio_emi\
tter_get_vy\x22,\x22au\
dio_emitter_get_\
vz\x22,\x22audio_emitt\
er_get_x\x22,\x22audio\
_emitter_get_y\x22,\
\x22audio_emitter_g\
et_z\x22,\x22audio_emi\
tter_pitch\x22,\x22aud\
io_emitter_posit\
ion\x22,\x22audio_emit\
ter_set_listener\
_mask\x22,\x22audio_em\
itter_velocity\x22,\
\x22audio_exists\x22,\x22\
audio_falloff_se\
t_model\x22,\x22audio_\
free_buffer_soun\
d\x22,\x22audio_free_p\
lay_queue\x22,\x22audi\
o_get_listener_c\
ount\x22,\x22audio_get\
_listener_info\x22,\
\x22audio_get_liste\
ner_mask\x22,\x22audio\
_get_master_gain\
\x22,\x22audio_get_nam\
e\x22,\x22audio_get_re\
corder_count\x22,\x22a\
udio_get_recorde\
r_info\x22,\x22audio_g\
et_type\x22,\x22audio_\
group_get_assets\
\x22,\x22audio_group_g\
et_gain\x22,\x22audio_\
group_is_loaded\x22\
,\x22audio_group_lo\
ad\x22,\x22audio_group\
_load_progress\x22,\
\x22audio_group_nam\
e\x22,\x22audio_group_\
set_gain\x22,\x22audio\
_group_stop_all\x22\
,\x22audio_group_un\
load\x22,\x22audio_is_\
paused\x22,\x22audio_i\
s_playing\x22,\x22audi\
o_listener_get_d\
ata\x22,\x22audio_list\
ener_orientation\
\x22,\x22audio_listene\
r_position\x22,\x22aud\
io_listener_set_\
orientation\x22,\x22au\
dio_listener_set\
_position\x22,\x22audi\
o_listener_set_v\
elocity\x22,\x22audio_\
listener_velocit\
y\x22,\x22audio_master\
_gain\x22,\x22audio_pa\
use_all\x22,\x22audio_\
pause_sound\x22,\x22au\
dio_pause_sync_g\
roup\x22,\x22audio_pla\
y_in_sync_group\x22\
,\x22audio_play_sou\
nd\x22,\x22audio_play_\
sound_at\x22,\x22audio\
_play_sound_ext\x22\
,\x22audio_play_sou\
nd_on\x22,\x22audio_qu\
eue_sound\x22,\x22audi\
o_resume_all\x22,\x22a\
udio_resume_soun\
d\x22,\x22audio_resume\
_sync_group\x22,\x22au\
dio_set_listener\
_mask\x22,\x22audio_se\
t_master_gain\x22,\x22\
audio_sound_gain\
\x22,\x22audio_sound_g\
et_audio_group\x22,\
\x22audio_sound_get\
_gain\x22,\x22audio_so\
und_get_listener\
_mask\x22,\x22audio_so\
und_get_loop\x22,\x22a\
udio_sound_get_l\
oop_end\x22,\x22audio_\
sound_get_loop_s\
tart\x22,\x22audio_sou\
nd_get_pitch\x22,\x22a\
udio_sound_get_t\
rack_position\x22,\x22\
audio_sound_is_p\
layable\x22,\x22audio_\
sound_length\x22,\x22a\
udio_sound_loop\x22\
,\x22audio_sound_lo\
op_end\x22,\x22audio_s\
ound_loop_start\x22\
,\x22audio_sound_pi\
tch\x22,\x22audio_soun\
d_set_listener_m\
ask\x22,\x22audio_soun\
d_set_track_posi\
tion\x22,\x22audio_sta\
rt_recording\x22,\x22a\
udio_start_sync_\
group\x22,\x22audio_st\
op_all\x22,\x22audio_s\
top_recording\x22,\x22\
audio_stop_sound\
\x22,\x22audio_stop_sy\
nc_group\x22,\x22audio\
_sync_group_debu\
g\x22,\x22audio_sync_g\
roup_get_track_p\
os\x22,\x22audio_sync_\
group_is_paused\x22\
,\x22audio_sync_gro\
up_is_playing\x22,\x22\
audio_system_is_\
available\x22,\x22audi\
o_system_is_init\
ialised\x22,\x22base64\
_decode\x22,\x22base64\
_encode\x22,\x22bool\x22,\
\x22browser_input_c\
apture\x22,\x22buffer_\
async_group_begi\
n\x22,\x22buffer_async\
_group_end\x22,\x22buf\
fer_async_group_\
option\x22,\x22buffer_\
base64_decode\x22,\x22\
buffer_base64_de\
code_ext\x22,\x22buffe\
r_base64_encode\x22\
,\x22buffer_compres\
s\x22,\x22buffer_copy\x22\
,\x22buffer_copy_fr\
om_vertex_buffer\
\x22,\x22buffer_copy_s\
tride\x22,\x22buffer_c\
rc32\x22,\x22buffer_cr\
eate\x22,\x22buffer_cr\
eate_from_vertex\
_buffer\x22,\x22buffer\
_create_from_ver\
tex_buffer_ext\x22,\
\x22buffer_decompre\
ss\x22,\x22buffer_dele\
te\x22,\x22buffer_exis\
ts\x22,\x22buffer_fill\
\x22,\x22buffer_get_ad\
dress\x22,\x22buffer_g\
et_alignment\x22,\x22b\
uffer_get_size\x22,\
\x22buffer_get_surf\
ace\x22,\x22buffer_get\
_type\x22,\x22buffer_l\
oad\x22,\x22buffer_loa\
d_async\x22,\x22buffer\
_load_ext\x22,\x22buff\
er_load_partial\x22\
,\x22buffer_md5\x22,\x22b\
uffer_peek\x22,\x22buf\
fer_poke\x22,\x22buffe\
r_read\x22,\x22buffer_\
resize\x22,\x22buffer_\
save\x22,\x22buffer_sa\
ve_async\x22,\x22buffe\
r_save_ext\x22,\x22buf\
fer_seek\x22,\x22buffe\
r_set_surface\x22,\x22\
buffer_set_used_\
size\x22,\x22buffer_sh\
a1\x22,\x22buffer_size\
of\x22,\x22buffer_tell\
\x22,\x22buffer_write\x22\
,\x22call_cancel\x22,\x22\
call_later\x22,\x22cam\
era_apply\x22,\x22came\
ra_copy_transfor\
ms\x22,\x22camera_crea\
te\x22,\x22camera_crea\
te_view\x22,\x22camera\
_destroy\x22,\x22camer\
a_get_active\x22,\x22c\
amera_get_begin_\
script\x22,\x22camera_\
get_default\x22,\x22ca\
mera_get_end_scr\
ipt\x22,\x22camera_get\
_proj_mat\x22,\x22came\
ra_get_update_sc\
ript\x22,\x22camera_ge\
t_view_angle\x22,\x22c\
amera_get_view_b\
order_x\x22,\x22camera\
_get_view_border\
_y\x22,\x22camera_get_\
view_height\x22,\x22ca\
mera_get_view_ma\
t\x22,\x22camera_get_v\
iew_speed_x\x22,\x22ca\
mera_get_view_sp\
eed_y\x22,\x22camera_g\
et_view_target\x22,\
\x22camera_get_view\
_width\x22,\x22camera_\
get_view_x\x22,\x22cam\
era_get_view_y\x22,\
\x22camera_set_begi\
n_script\x22,\x22camer\
a_set_default\x22,\x22\
camera_set_end_s\
cript\x22,\x22camera_s\
et_proj_mat\x22,\x22ca\
mera_set_update_\
script\x22,\x22camera_\
set_view_angle\x22,\
\x22camera_set_view\
_border\x22,\x22camera\
_set_view_mat\x22,\x22\
camera_set_view_\
pos\x22,\x22camera_set\
_view_size\x22,\x22cam\
era_set_view_spe\
ed\x22,\x22camera_set_\
view_target\x22,\x22ce\
il\x22,\x22choose\x22,\x22ch\
r\x22,\x22clamp\x22,\x22clic\
kable_add\x22,\x22clic\
kable_add_ext\x22,\x22\
clickable_change\
\x22,\x22clickable_cha\
nge_ext\x22,\x22clicka\
ble_delete\x22,\x22cli\
ckable_exists\x22,\x22\
clickable_set_st\
yle\x22,\x22clipboard_\
get_text\x22,\x22clipb\
oard_has_text\x22,\x22\
clipboard_set_te\
xt\x22,\x22cloud_file_\
save\x22,\x22cloud_str\
ing_save\x22,\x22cloud\
_synchronise\x22,\x22c\
ode_is_compiled\x22\
,\x22collision_circ\
le\x22,\x22collision_c\
ircle_list\x22,\x22col\
lision_ellipse\x22,\
\x22collision_ellip\
se_list\x22,\x22collis\
ion_line\x22,\x22colli\
sion_line_list\x22,\
\x22collision_point\
\x22,\x22collision_poi\
nt_list\x22,\x22collis\
ion_rectangle\x22,\x22\
collision_rectan\
gle_list\x22,\x22color\
_get_blue\x22,\x22colo\
r_get_green\x22,\x22co\
lor_get_hue\x22,\x22co\
lor_get_red\x22,\x22co\
lor_get_saturati\
on\x22,\x22color_get_v\
alue\x22,\x22colour_ge\
t_blue\x22,\x22colour_\
get_green\x22,\x22colo\
ur_get_hue\x22,\x22col\
our_get_red\x22,\x22co\
lour_get_saturat\
ion\x22,\x22colour_get\
_value\x22,\x22cos\x22,\x22d\
arccos\x22,\x22darcsin\
\x22,\x22darctan\x22,\x22dar\
ctan2\x22,\x22date_com\
pare_date\x22,\x22date\
_compare_datetim\
e\x22,\x22date_compare\
_time\x22,\x22date_cre\
ate_datetime\x22,\x22d\
ate_current_date\
time\x22,\x22date_date\
_of\x22,\x22date_date_\
string\x22,\x22date_da\
tetime_string\x22,\x22\
date_day_span\x22,\x22\
date_days_in_mon\
th\x22,\x22date_days_i\
n_year\x22,\x22date_ge\
t_day\x22,\x22date_get\
_day_of_year\x22,\x22d\
ate_get_hour\x22,\x22d\
ate_get_hour_of_\
year\x22,\x22date_get_\
minute\x22,\x22date_ge\
t_minute_of_year\
\x22,\x22date_get_mont\
h\x22,\x22date_get_sec\
ond\x22,\x22date_get_s\
econd_of_year\x22,\x22\
date_get_timezon\
e\x22,\x22date_get_wee\
k\x22,\x22date_get_wee\
kday\x22,\x22date_get_\
year\x22,\x22date_hour\
_span\x22,\x22date_inc\
_day\x22,\x22date_inc_\
hour\x22,\x22date_inc_\
minute\x22,\x22date_in\
c_month\x22,\x22date_i\
nc_second\x22,\x22date\
_inc_week\x22,\x22date\
_inc_year\x22,\x22date\
_is_today\x22,\x22date\
_leap_year\x22,\x22dat\
e_minute_span\x22,\x22\
date_month_span\x22\
,\x22date_second_sp\
an\x22,\x22date_set_ti\
mezone\x22,\x22date_ti\
me_of\x22,\x22date_tim\
e_string\x22,\x22date_\
valid_datetime\x22,\
\x22date_week_span\x22\
,\x22date_year_span\
\x22,\x22db_to_lin\x22,\x22d\
bg_add_font_glyp\
hs\x22,\x22dbg_button\x22\
,\x22dbg_checkbox\x22,\
\x22dbg_color\x22,\x22dbg\
_colour\x22,\x22dbg_dr\
op_down\x22,\x22dbg_sa\
me_line\x22,\x22dbg_se\
ction\x22,\x22dbg_sect\
ion_delete\x22,\x22dbg\
_section_exists\x22\
,\x22dbg_slider\x22,\x22d\
bg_slider_int\x22,\x22\
dbg_sprite\x22,\x22dbg\
_text\x22,\x22dbg_text\
_input\x22,\x22dbg_vie\
w\x22,\x22dbg_view_del\
ete\x22,\x22dbg_view_e\
xists\x22,\x22dbg_watc\
h\x22,\x22dcos\x22,\x22debug\
_event\x22,\x22debug_g\
et_callstack\x22,\x22d\
egtorad\x22,\x22device\
_get_tilt_x\x22,\x22de\
vice_get_tilt_y\x22\
,\x22device_get_til\
t_z\x22,\x22device_is_\
keypad_open\x22,\x22de\
vice_mouse_check\
_button\x22,\x22device\
_mouse_check_but\
ton_pressed\x22,\x22de\
vice_mouse_check\
_button_released\
\x22,\x22device_mouse_\
dbclick_enable\x22,\
\x22device_mouse_ra\
w_x\x22,\x22device_mou\
se_raw_y\x22,\x22devic\
e_mouse_x\x22,\x22devi\
ce_mouse_x_to_gu\
i\x22,\x22device_mouse\
_y\x22,\x22device_mous\
e_y_to_gui\x22,\x22dir\
ectory_create\x22,\x22\
directory_destro\
y\x22,\x22directory_ex\
ists\x22,\x22display_g\
et_dpi_x\x22,\x22displ\
ay_get_dpi_y\x22,\x22d\
isplay_get_frequ\
ency\x22,\x22display_g\
et_gui_height\x22,\x22\
display_get_gui_\
width\x22,\x22display_\
get_height\x22,\x22dis\
play_get_orienta\
tion\x22,\x22display_g\
et_sleep_margin\x22\
,\x22display_get_ti\
ming_method\x22,\x22di\
splay_get_width\x22\
,\x22display_mouse_\
get_x\x22,\x22display_\
mouse_get_y\x22,\x22di\
splay_mouse_set\x22\
,\x22display_reset\x22\
,\x22display_set_gu\
i_maximise\x22,\x22dis\
play_set_gui_max\
imize\x22,\x22display_\
set_gui_size\x22,\x22d\
isplay_set_sleep\
_margin\x22,\x22displa\
y_set_timing_met\
hod\x22,\x22display_se\
t_ui_visibility\x22\
,\x22distance_to_ob\
ject\x22,\x22distance_\
to_point\x22,\x22dot_p\
roduct\x22,\x22dot_pro\
duct_3d\x22,\x22dot_pr\
oduct_3d_normali\
sed\x22,\x22dot_produc\
t_3d_normalized\x22\
,\x22dot_product_no\
rmalised\x22,\x22dot_p\
roduct_normalize\
d\x22,\x22draw_arrow\x22,\
\x22draw_button\x22,\x22d\
raw_circle\x22,\x22dra\
w_circle_color\x22,\
\x22draw_circle_col\
our\x22,\x22draw_clear\
\x22,\x22draw_clear_al\
pha\x22,\x22draw_ellip\
se\x22,\x22draw_ellips\
e_color\x22,\x22draw_e\
llipse_colour\x22,\x22\
draw_enable_draw\
event\x22,\x22draw_ena\
ble_skeleton_ble\
ndmodes\x22,\x22draw_e\
nable_swf_aa\x22,\x22d\
raw_flush\x22,\x22draw\
_get_alpha\x22,\x22dra\
w_get_color\x22,\x22dr\
aw_get_colour\x22,\x22\
draw_get_enable_\
skeleton_blendmo\
des\x22,\x22draw_get_f\
ont\x22,\x22draw_get_h\
align\x22,\x22draw_get\
_lighting\x22,\x22draw\
_get_swf_aa_leve\
l\x22,\x22draw_get_val\
ign\x22,\x22draw_getpi\
xel\x22,\x22draw_getpi\
xel_ext\x22,\x22draw_h\
ealthbar\x22,\x22draw_\
highscore\x22,\x22draw\
_light_define_am\
bient\x22,\x22draw_lig\
ht_define_direct\
ion\x22,\x22draw_light\
_define_point\x22,\x22\
draw_light_enabl\
e\x22,\x22draw_light_g\
et\x22,\x22draw_light_\
get_ambient\x22,\x22dr\
aw_line\x22,\x22draw_l\
ine_color\x22,\x22draw\
_line_colour\x22,\x22d\
raw_line_width\x22,\
\x22draw_line_width\
_color\x22,\x22draw_li\
ne_width_colour\x22\
,\x22draw_path\x22,\x22dr\
aw_point\x22,\x22draw_\
point_color\x22,\x22dr\
aw_point_colour\x22\
,\x22draw_primitive\
_begin\x22,\x22draw_pr\
imitive_begin_te\
xture\x22,\x22draw_pri\
mitive_end\x22,\x22dra\
w_rectangle\x22,\x22dr\
aw_rectangle_col\
or\x22,\x22draw_rectan\
gle_colour\x22,\x22dra\
w_roundrect\x22,\x22dr\
aw_roundrect_col\
or\x22,\x22draw_roundr\
ect_color_ext\x22,\x22\
draw_roundrect_c\
olour\x22,\x22draw_rou\
ndrect_colour_ex\
t\x22,\x22draw_roundre\
ct_ext\x22,\x22draw_se\
lf\x22,\x22draw_set_al\
pha\x22,\x22draw_set_c\
ircle_precision\x22\
,\x22draw_set_color\
\x22,\x22draw_set_colo\
ur\x22,\x22draw_set_fo\
nt\x22,\x22draw_set_ha\
lign\x22,\x22draw_set_\
lighting\x22,\x22draw_\
set_swf_aa_level\
\x22,\x22draw_set_vali\
gn\x22,\x22draw_skelet\
on\x22,\x22draw_skelet\
on_collision\x22,\x22d\
raw_skeleton_ins\
tance\x22,\x22draw_ske\
leton_time\x22,\x22dra\
w_sprite\x22,\x22draw_\
sprite_ext\x22,\x22dra\
w_sprite_general\
\x22,\x22draw_sprite_p\
art\x22,\x22draw_sprit\
e_part_ext\x22,\x22dra\
w_sprite_pos\x22,\x22d\
raw_sprite_stret\
ched\x22,\x22draw_spri\
te_stretched_ext\
\x22,\x22draw_sprite_t\
iled\x22,\x22draw_spri\
te_tiled_ext\x22,\x22d\
raw_surface\x22,\x22dr\
aw_surface_ext\x22,\
\x22draw_surface_ge\
neral\x22,\x22draw_sur\
face_part\x22,\x22draw\
_surface_part_ex\
t\x22,\x22draw_surface\
_stretched\x22,\x22dra\
w_surface_stretc\
hed_ext\x22,\x22draw_s\
urface_tiled\x22,\x22d\
raw_surface_tile\
d_ext\x22,\x22draw_tex\
t\x22,\x22draw_text_co\
lor\x22,\x22draw_text_\
colour\x22,\x22draw_te\
xt_ext\x22,\x22draw_te\
xt_ext_color\x22,\x22d\
raw_text_ext_col\
our\x22,\x22draw_text_\
ext_transformed\x22\
,\x22draw_text_ext_\
transformed_colo\
r\x22,\x22draw_text_ex\
t_transformed_co\
lour\x22,\x22draw_text\
_transformed\x22,\x22d\
raw_text_transfo\
rmed_color\x22,\x22dra\
w_text_transform\
ed_colour\x22,\x22draw\
_texture_flush\x22,\
\x22draw_tile\x22,\x22dra\
w_tilemap\x22,\x22draw\
_triangle\x22,\x22draw\
_triangle_color\x22\
,\x22draw_triangle_\
colour\x22,\x22draw_ve\
rtex\x22,\x22draw_vert\
ex_color\x22,\x22draw_\
vertex_colour\x22,\x22\
draw_vertex_text\
ure\x22,\x22draw_verte\
x_texture_color\x22\
,\x22draw_vertex_te\
xture_colour\x22,\x22d\
s_exists\x22,\x22ds_gr\
id_add\x22,\x22ds_grid\
_add_disk\x22,\x22ds_g\
rid_add_grid_reg\
ion\x22,\x22ds_grid_ad\
d_region\x22,\x22ds_gr\
id_clear\x22,\x22ds_gr\
id_copy\x22,\x22ds_gri\
d_create\x22,\x22ds_gr\
id_destroy\x22,\x22ds_\
grid_get\x22,\x22ds_gr\
id_get_disk_max\x22\
,\x22ds_grid_get_di\
sk_mean\x22,\x22ds_gri\
d_get_disk_min\x22,\
\x22ds_grid_get_dis\
k_sum\x22,\x22ds_grid_\
get_max\x22,\x22ds_gri\
d_get_mean\x22,\x22ds_\
grid_get_min\x22,\x22d\
s_grid_get_sum\x22,\
\x22ds_grid_height\x22\
,\x22ds_grid_multip\
ly\x22,\x22ds_grid_mul\
tiply_disk\x22,\x22ds_\
grid_multiply_gr\
id_region\x22,\x22ds_g\
rid_multiply_reg\
ion\x22,\x22ds_grid_re\
ad\x22,\x22ds_grid_res\
ize\x22,\x22ds_grid_se\
t\x22,\x22ds_grid_set_\
disk\x22,\x22ds_grid_s\
et_grid_region\x22,\
\x22ds_grid_set_reg\
ion\x22,\x22ds_grid_sh\
uffle\x22,\x22ds_grid_\
sort\x22,\x22ds_grid_t\
o_mp_grid\x22,\x22ds_g\
rid_value_disk_e\
xists\x22,\x22ds_grid_\
value_disk_x\x22,\x22d\
s_grid_value_dis\
k_y\x22,\x22ds_grid_va\
lue_exists\x22,\x22ds_\
grid_value_x\x22,\x22d\
s_grid_value_y\x22,\
\x22ds_grid_width\x22,\
\x22ds_grid_write\x22,\
\x22ds_list_add\x22,\x22d\
s_list_clear\x22,\x22d\
s_list_copy\x22,\x22ds\
_list_create\x22,\x22d\
s_list_delete\x22,\x22\
ds_list_destroy\x22\
,\x22ds_list_empty\x22\
,\x22ds_list_find_i\
ndex\x22,\x22ds_list_f\
ind_value\x22,\x22ds_l\
ist_insert\x22,\x22ds_\
list_is_list\x22,\x22d\
s_list_is_map\x22,\x22\
ds_list_mark_as_\
list\x22,\x22ds_list_m\
ark_as_map\x22,\x22ds_\
list_read\x22,\x22ds_l\
ist_replace\x22,\x22ds\
_list_set\x22,\x22ds_l\
ist_shuffle\x22,\x22ds\
_list_size\x22,\x22ds_\
list_sort\x22,\x22ds_l\
ist_write\x22,\x22ds_m\
ap_add\x22,\x22ds_map_\
add_list\x22,\x22ds_ma\
p_add_map\x22,\x22ds_m\
ap_clear\x22,\x22ds_ma\
p_copy\x22,\x22ds_map_\
create\x22,\x22ds_map_\
delete\x22,\x22ds_map_\
destroy\x22,\x22ds_map\
_empty\x22,\x22ds_map_\
exists\x22,\x22ds_map_\
find_first\x22,\x22ds_\
map_find_last\x22,\x22\
ds_map_find_next\
\x22,\x22ds_map_find_p\
revious\x22,\x22ds_map\
_find_value\x22,\x22ds\
_map_is_list\x22,\x22d\
s_map_is_map\x22,\x22d\
s_map_keys_to_ar\
ray\x22,\x22ds_map_rea\
d\x22,\x22ds_map_repla\
ce\x22,\x22ds_map_repl\
ace_list\x22,\x22ds_ma\
p_replace_map\x22,\x22\
ds_map_secure_lo\
ad\x22,\x22ds_map_secu\
re_load_buffer\x22,\
\x22ds_map_secure_s\
ave\x22,\x22ds_map_sec\
ure_save_buffer\x22\
,\x22ds_map_set\x22,\x22d\
s_map_size\x22,\x22ds_\
map_values_to_ar\
ray\x22,\x22ds_map_wri\
te\x22,\x22ds_priority\
_add\x22,\x22ds_priori\
ty_change_priori\
ty\x22,\x22ds_priority\
_clear\x22,\x22ds_prio\
rity_copy\x22,\x22ds_p\
riority_create\x22,\
\x22ds_priority_del\
ete_max\x22,\x22ds_pri\
ority_delete_min\
\x22,\x22ds_priority_d\
elete_value\x22,\x22ds\
_priority_destro\
y\x22,\x22ds_priority_\
empty\x22,\x22ds_prior\
ity_find_max\x22,\x22d\
s_priority_find_\
min\x22,\x22ds_priorit\
y_find_priority\x22\
,\x22ds_priority_re\
ad\x22,\x22ds_priority\
_size\x22,\x22ds_prior\
ity_write\x22,\x22ds_q\
ueue_clear\x22,\x22ds_\
queue_copy\x22,\x22ds_\
queue_create\x22,\x22d\
s_queue_dequeue\x22\
,\x22ds_queue_destr\
oy\x22,\x22ds_queue_em\
pty\x22,\x22ds_queue_e\
nqueue\x22,\x22ds_queu\
e_head\x22,\x22ds_queu\
e_read\x22,\x22ds_queu\
e_size\x22,\x22ds_queu\
e_tail\x22,\x22ds_queu\
e_write\x22,\x22ds_set\
_precision\x22,\x22ds_\
stack_clear\x22,\x22ds\
_stack_copy\x22,\x22ds\
_stack_create\x22,\x22\
ds_stack_destroy\
\x22,\x22ds_stack_empt\
y\x22,\x22ds_stack_pop\
\x22,\x22ds_stack_push\
\x22,\x22ds_stack_read\
\x22,\x22ds_stack_size\
\x22,\x22ds_stack_top\x22\
,\x22ds_stack_write\
\x22,\x22dsin\x22,\x22dtan\x22,\
\x22effect_clear\x22,\x22\
effect_create_ab\
ove\x22,\x22effect_cre\
ate_below\x22,\x22effe\
ct_create_depth\x22\
,\x22effect_create_\
layer\x22,\x22environm\
ent_get_variable\
\x22,\x22event_inherit\
ed\x22,\x22event_perfo\
rm\x22,\x22event_perfo\
rm_async\x22,\x22event\
_perform_object\x22\
,\x22event_user\x22,\x22e\
xception_unhandl\
ed_handler\x22,\x22exp\
\x22,\x22extension_exi\
sts\x22,\x22extension_\
get_option_count\
\x22,\x22extension_get\
_option_names\x22,\x22\
extension_get_op\
tion_value\x22,\x22ext\
ension_get_optio\
ns\x22,\x22extension_g\
et_version\x22,\x22ext\
ernal_call\x22,\x22ext\
ernal_define\x22,\x22e\
xternal_free\x22,\x22f\
ile_attributes\x22,\
\x22file_bin_close\x22\
,\x22file_bin_open\x22\
,\x22file_bin_posit\
ion\x22,\x22file_bin_r\
ead_byte\x22,\x22file_\
bin_rewrite\x22,\x22fi\
le_bin_seek\x22,\x22fi\
le_bin_size\x22,\x22fi\
le_bin_write_byt\
e\x22,\x22file_copy\x22,\x22\
file_delete\x22,\x22fi\
le_exists\x22,\x22file\
_find_close\x22,\x22fi\
le_find_first\x22,\x22\
file_find_next\x22,\
\x22file_rename\x22,\x22f\
ile_text_close\x22,\
\x22file_text_eof\x22,\
\x22file_text_eoln\x22\
,\x22file_text_open\
_append\x22,\x22file_t\
ext_open_from_st\
ring\x22,\x22file_text\
_open_read\x22,\x22fil\
e_text_open_writ\
e\x22,\x22file_text_re\
ad_real\x22,\x22file_t\
ext_read_string\x22\
,\x22file_text_read\
ln\x22,\x22file_text_w\
rite_real\x22,\x22file\
_text_write_stri\
ng\x22,\x22file_text_w\
riteln\x22,\x22filenam\
e_change_ext\x22,\x22f\
ilename_dir\x22,\x22fi\
lename_drive\x22,\x22f\
ilename_ext\x22,\x22fi\
lename_name\x22,\x22fi\
lename_path\x22,\x22fl\
oor\x22,\x22font_add\x22,\
\x22font_add_enable\
_aa\x22,\x22font_add_g\
et_enable_aa\x22,\x22f\
ont_add_sprite\x22,\
\x22font_add_sprite\
_ext\x22,\x22font_cach\
e_glyph\x22,\x22font_d\
elete\x22,\x22font_ena\
ble_effects\x22,\x22fo\
nt_enable_sdf\x22,\x22\
font_exists\x22,\x22fo\
nt_get_bold\x22,\x22fo\
nt_get_first\x22,\x22f\
ont_get_fontname\
\x22,\x22font_get_info\
\x22,\x22font_get_ital\
ic\x22,\x22font_get_la\
st\x22,\x22font_get_na\
me\x22,\x22font_get_sd\
f_enabled\x22,\x22font\
_get_sdf_spread\x22\
,\x22font_get_size\x22\
,\x22font_get_textu\
re\x22,\x22font_get_uv\
s\x22,\x22font_replace\
_sprite\x22,\x22font_r\
eplace_sprite_ex\
t\x22,\x22font_sdf_spr\
ead\x22,\x22font_set_c\
ache_size\x22,\x22frac\
\x22,\x22fx_create\x22,\x22f\
x_get_name\x22,\x22fx_\
get_parameter\x22,\x22\
fx_get_parameter\
_names\x22,\x22fx_get_\
parameters\x22,\x22fx_\
get_single_layer\
\x22,\x22fx_set_parame\
ter\x22,\x22fx_set_par\
ameters\x22,\x22fx_set\
_single_layer\x22,\x22\
game_change\x22,\x22ga\
me_end\x22,\x22game_ge\
t_speed\x22,\x22game_l\
oad\x22,\x22game_load_\
buffer\x22,\x22game_re\
start\x22,\x22game_sav\
e\x22,\x22game_save_bu\
ffer\x22,\x22game_set_\
speed\x22,\x22gamepad_\
axis_count\x22,\x22gam\
epad_axis_value\x22\
,\x22gamepad_button\
_check\x22,\x22gamepad\
_button_check_pr\
essed\x22,\x22gamepad_\
button_check_rel\
eased\x22,\x22gamepad_\
button_count\x22,\x22g\
amepad_button_va\
lue\x22,\x22gamepad_ge\
t_axis_deadzone\x22\
,\x22gamepad_get_bu\
tton_threshold\x22,\
\x22gamepad_get_des\
cription\x22,\x22gamep\
ad_get_device_co\
unt\x22,\x22gamepad_ge\
t_guid\x22,\x22gamepad\
_get_mapping\x22,\x22g\
amepad_get_optio\
n\x22,\x22gamepad_hat_\
count\x22,\x22gamepad_\
hat_value\x22,\x22game\
pad_is_connected\
\x22,\x22gamepad_is_su\
pported\x22,\x22gamepa\
d_remove_mapping\
\x22,\x22gamepad_set_a\
xis_deadzone\x22,\x22g\
amepad_set_butto\
n_threshold\x22,\x22ga\
mepad_set_color\x22\
,\x22gamepad_set_co\
lour\x22,\x22gamepad_s\
et_option\x22,\x22game\
pad_set_vibratio\
n\x22,\x22gamepad_test\
_mapping\x22,\x22gc_co\
llect\x22,\x22gc_enabl\
e\x22,\x22gc_get_stats\
\x22,\x22gc_get_target\
_frame_time\x22,\x22gc\
_is_enabled\x22,\x22gc\
_target_frame_ti\
me\x22,\x22gesture_dou\
ble_tap_distance\
\x22,\x22gesture_doubl\
e_tap_time\x22,\x22ges\
ture_drag_distan\
ce\x22,\x22gesture_dra\
g_time\x22,\x22gesture\
_flick_speed\x22,\x22g\
esture_get_doubl\
e_tap_distance\x22,\
\x22gesture_get_dou\
ble_tap_time\x22,\x22g\
esture_get_drag_\
distance\x22,\x22gestu\
re_get_drag_time\
\x22,\x22gesture_get_f\
lick_speed\x22,\x22ges\
ture_get_pinch_a\
ngle_away\x22,\x22gest\
ure_get_pinch_an\
gle_towards\x22,\x22ge\
sture_get_pinch_\
distance\x22,\x22gestu\
re_get_rotate_an\
gle\x22,\x22gesture_ge\
t_rotate_time\x22,\x22\
gesture_get_tap_\
count\x22,\x22gesture_\
pinch_angle_away\
\x22,\x22gesture_pinch\
_angle_towards\x22,\
\x22gesture_pinch_d\
istance\x22,\x22gestur\
e_rotate_angle\x22,\
\x22gesture_rotate_\
time\x22,\x22gesture_t\
ap_count\x22,\x22get_i\
nteger\x22,\x22get_int\
eger_async\x22,\x22get\
_login_async\x22,\x22g\
et_open_filename\
\x22,\x22get_open_file\
name_ext\x22,\x22get_s\
ave_filename\x22,\x22g\
et_save_filename\
_ext\x22,\x22get_strin\
g\x22,\x22get_string_a\
sync\x22,\x22get_timer\
\x22,\x22gif_add_surfa\
ce\x22,\x22gif_open\x22,\x22\
gif_save\x22,\x22gif_s\
ave_buffer\x22,\x22gml\
_pragma\x22,\x22gml_re\
lease_mode\x22,\x22gpu\
_get_alphatesten\
able\x22,\x22gpu_get_a\
lphatestref\x22,\x22gp\
u_get_blendenabl\
e\x22,\x22gpu_get_blen\
dmode\x22,\x22gpu_get_\
blendmode_dest\x22,\
\x22gpu_get_blendmo\
de_destalpha\x22,\x22g\
pu_get_blendmode\
_ext\x22,\x22gpu_get_b\
lendmode_ext_sep\
alpha\x22,\x22gpu_get_\
blendmode_src\x22,\x22\
gpu_get_blendmod\
e_srcalpha\x22,\x22gpu\
_get_colorwritee\
nable\x22,\x22gpu_get_\
colourwriteenabl\
e\x22,\x22gpu_get_cull\
mode\x22,\x22gpu_get_d\
epth\x22,\x22gpu_get_f\
og\x22,\x22gpu_get_sta\
te\x22,\x22gpu_get_tex\
_filter\x22,\x22gpu_ge\
t_tex_filter_ext\
\x22,\x22gpu_get_tex_m\
ax_aniso\x22,\x22gpu_g\
et_tex_max_aniso\
_ext\x22,\x22gpu_get_t\
ex_max_mip\x22,\x22gpu\
_get_tex_max_mip\
_ext\x22,\x22gpu_get_t\
ex_min_mip\x22,\x22gpu\
_get_tex_min_mip\
_ext\x22,\x22gpu_get_t\
ex_mip_bias\x22,\x22gp\
u_get_tex_mip_bi\
as_ext\x22,\x22gpu_get\
_tex_mip_enable\x22\
,\x22gpu_get_tex_mi\
p_enable_ext\x22,\x22g\
pu_get_tex_mip_f\
ilter\x22,\x22gpu_get_\
tex_mip_filter_e\
xt\x22,\x22gpu_get_tex\
_repeat\x22,\x22gpu_ge\
t_tex_repeat_ext\
\x22,\x22gpu_get_texfi\
lter\x22,\x22gpu_get_t\
exfilter_ext\x22,\x22g\
pu_get_texrepeat\
\x22,\x22gpu_get_texre\
peat_ext\x22,\x22gpu_g\
et_zfunc\x22,\x22gpu_g\
et_ztestenable\x22,\
\x22gpu_get_zwritee\
nable\x22,\x22gpu_pop_\
state\x22,\x22gpu_push\
_state\x22,\x22gpu_set\
_alphatestenable\
\x22,\x22gpu_set_alpha\
testref\x22,\x22gpu_se\
t_blendenable\x22,\x22\
gpu_set_blendmod\
e\x22,\x22gpu_set_blen\
dmode_ext\x22,\x22gpu_\
set_blendmode_ex\
t_sepalpha\x22,\x22gpu\
_set_colorwritee\
nable\x22,\x22gpu_set_\
colourwriteenabl\
e\x22,\x22gpu_set_cull\
mode\x22,\x22gpu_set_d\
epth\x22,\x22gpu_set_f\
og\x22,\x22gpu_set_sta\
te\x22,\x22gpu_set_tex\
_filter\x22,\x22gpu_se\
t_tex_filter_ext\
\x22,\x22gpu_set_tex_m\
ax_aniso\x22,\x22gpu_s\
et_tex_max_aniso\
_ext\x22,\x22gpu_set_t\
ex_max_mip\x22,\x22gpu\
_set_tex_max_mip\
_ext\x22,\x22gpu_set_t\
ex_min_mip\x22,\x22gpu\
_set_tex_min_mip\
_ext\x22,\x22gpu_set_t\
ex_mip_bias\x22,\x22gp\
u_set_tex_mip_bi\
as_ext\x22,\x22gpu_set\
_tex_mip_enable\x22\
,\x22gpu_set_tex_mi\
p_enable_ext\x22,\x22g\
pu_set_tex_mip_f\
ilter\x22,\x22gpu_set_\
tex_mip_filter_e\
xt\x22,\x22gpu_set_tex\
_repeat\x22,\x22gpu_se\
t_tex_repeat_ext\
\x22,\x22gpu_set_texfi\
lter\x22,\x22gpu_set_t\
exfilter_ext\x22,\x22g\
pu_set_texrepeat\
\x22,\x22gpu_set_texre\
peat_ext\x22,\x22gpu_s\
et_zfunc\x22,\x22gpu_s\
et_ztestenable\x22,\
\x22gpu_set_zwritee\
nable\x22,\x22handle_p\
arse\x22,\x22highscore\
_add\x22,\x22highscore\
_clear\x22,\x22highsco\
re_name\x22,\x22highsc\
ore_value\x22,\x22http\
_get\x22,\x22http_get_\
file\x22,\x22http_get_\
request_crossori\
gin\x22,\x22http_post_\
string\x22,\x22http_re\
quest\x22,\x22http_set\
_request_crossor\
igin\x22,\x22iap_acqui\
re\x22,\x22iap_activat\
e\x22,\x22iap_consume\x22\
,\x22iap_enumerate_\
products\x22,\x22iap_p\
roduct_details\x22,\
\x22iap_purchase_de\
tails\x22,\x22iap_rest\
ore_all\x22,\x22iap_st\
atus\x22,\x22ini_close\
\x22,\x22ini_key_delet\
e\x22,\x22ini_key_exis\
ts\x22,\x22ini_open\x22,\x22\
ini_open_from_st\
ring\x22,\x22ini_read_\
real\x22,\x22ini_read_\
string\x22,\x22ini_sec\
tion_delete\x22,\x22in\
i_section_exists\
\x22,\x22ini_write_rea\
l\x22,\x22ini_write_st\
ring\x22,\x22instance_\
activate_all\x22,\x22i\
nstance_activate\
_layer\x22,\x22instanc\
e_activate_objec\
t\x22,\x22instance_act\
ivate_region\x22,\x22i\
nstance_change\x22,\
\x22instance_copy\x22,\
\x22instance_create\
_depth\x22,\x22instanc\
e_create_layer\x22,\
\x22instance_deacti\
vate_all\x22,\x22insta\
nce_deactivate_l\
ayer\x22,\x22instance_\
deactivate_objec\
t\x22,\x22instance_dea\
ctivate_region\x22,\
\x22instance_destro\
y\x22,\x22instance_exi\
sts\x22,\x22instance_f\
ind\x22,\x22instance_f\
urthest\x22,\x22instan\
ce_id_get\x22,\x22inst\
ance_nearest\x22,\x22i\
nstance_number\x22,\
\x22instance_place\x22\
,\x22instance_place\
_list\x22,\x22instance\
_position\x22,\x22inst\
ance_position_li\
st\x22,\x22instanceof\x22\
,\x22int64\x22,\x22io_cle\
ar\x22,\x22irandom\x22,\x22i\
random_range\x22,\x22i\
s_array\x22,\x22is_boo\
l\x22,\x22is_callable\x22\
,\x22is_debug_overl\
ay_open\x22,\x22is_han\
dle\x22,\x22is_infinit\
y\x22,\x22is_instanceo\
f\x22,\x22is_int32\x22,\x22i\
s_int64\x22,\x22is_key\
board_used_debug\
_overlay\x22,\x22is_me\
thod\x22,\x22is_mouse_\
over_debug_overl\
ay\x22,\x22is_nan\x22,\x22is\
_numeric\x22,\x22is_pt\
r\x22,\x22is_real\x22,\x22is\
_string\x22,\x22is_str\
uct\x22,\x22is_undefin\
ed\x22,\x22json_decode\
\x22,\x22json_encode\x22,\
\x22json_parse\x22,\x22js\
on_stringify\x22,\x22k\
eyboard_check\x22,\x22\
keyboard_check_d\
irect\x22,\x22keyboard\
_check_pressed\x22,\
\x22keyboard_check_\
released\x22,\x22keybo\
ard_clear\x22,\x22keyb\
oard_get_map\x22,\x22k\
eyboard_get_numl\
ock\x22,\x22keyboard_k\
ey_press\x22,\x22keybo\
ard_key_release\x22\
,\x22keyboard_set_m\
ap\x22,\x22keyboard_se\
t_numlock\x22,\x22keyb\
oard_unset_map\x22,\
\x22keyboard_virtua\
l_height\x22,\x22keybo\
ard_virtual_hide\
\x22,\x22keyboard_virt\
ual_show\x22,\x22keybo\
ard_virtual_stat\
us\x22,\x22layer_add_i\
nstance\x22,\x22layer_\
background_alpha\
\x22,\x22layer_backgro\
und_blend\x22,\x22laye\
r_background_cha\
nge\x22,\x22layer_back\
ground_create\x22,\x22\
layer_background\
_destroy\x22,\x22layer\
_background_exis\
ts\x22,\x22layer_backg\
round_get_alpha\x22\
,\x22layer_backgrou\
nd_get_blend\x22,\x22l\
ayer_background_\
get_htiled\x22,\x22lay\
er_background_ge\
t_id\x22,\x22layer_bac\
kground_get_inde\
x\x22,\x22layer_backgr\
ound_get_speed\x22,\
\x22layer_backgroun\
d_get_sprite\x22,\x22l\
ayer_background_\
get_stretch\x22,\x22la\
yer_background_g\
et_visible\x22,\x22lay\
er_background_ge\
t_vtiled\x22,\x22layer\
_background_get_\
xscale\x22,\x22layer_b\
ackground_get_ys\
cale\x22,\x22layer_bac\
kground_htiled\x22,\
\x22layer_backgroun\
d_index\x22,\x22layer_\
background_speed\
\x22,\x22layer_backgro\
und_sprite\x22,\x22lay\
er_background_st\
retch\x22,\x22layer_ba\
ckground_visible\
\x22,\x22layer_backgro\
und_vtiled\x22,\x22lay\
er_background_xs\
cale\x22,\x22layer_bac\
kground_yscale\x22,\
\x22layer_clear_fx\x22\
,\x22layer_create\x22,\
\x22layer_depth\x22,\x22l\
ayer_destroy\x22,\x22l\
ayer_destroy_ins\
tances\x22,\x22layer_e\
lement_move\x22,\x22la\
yer_enable_fx\x22,\x22\
layer_exists\x22,\x22l\
ayer_force_draw_\
depth\x22,\x22layer_fx\
_is_enabled\x22,\x22la\
yer_get_all\x22,\x22la\
yer_get_all_elem\
ents\x22,\x22layer_get\
_depth\x22,\x22layer_g\
et_element_layer\
\x22,\x22layer_get_ele\
ment_type\x22,\x22laye\
r_get_forced_dep\
th\x22,\x22layer_get_f\
x\x22,\x22layer_get_hs\
peed\x22,\x22layer_get\
_id\x22,\x22layer_get_\
id_at_depth\x22,\x22la\
yer_get_name\x22,\x22l\
ayer_get_script_\
begin\x22,\x22layer_ge\
t_script_end\x22,\x22l\
ayer_get_shader\x22\
,\x22layer_get_targ\
et_room\x22,\x22layer_\
get_visible\x22,\x22la\
yer_get_vspeed\x22,\
\x22layer_get_x\x22,\x22l\
ayer_get_y\x22,\x22lay\
er_has_instance\x22\
,\x22layer_hspeed\x22,\
\x22layer_instance_\
get_instance\x22,\x22l\
ayer_is_draw_dep\
th_forced\x22,\x22laye\
r_reset_target_r\
oom\x22,\x22layer_scri\
pt_begin\x22,\x22layer\
_script_end\x22,\x22la\
yer_sequence_ang\
le\x22,\x22layer_seque\
nce_create\x22,\x22lay\
er_sequence_dest\
roy\x22,\x22layer_sequ\
ence_exists\x22,\x22la\
yer_sequence_get\
_angle\x22,\x22layer_s\
equence_get_head\
dir\x22,\x22layer_sequ\
ence_get_headpos\
\x22,\x22layer_sequenc\
e_get_instance\x22,\
\x22layer_sequence_\
get_length\x22,\x22lay\
er_sequence_get_\
sequence\x22,\x22layer\
_sequence_get_sp\
eedscale\x22,\x22layer\
_sequence_get_x\x22\
,\x22layer_sequence\
_get_xscale\x22,\x22la\
yer_sequence_get\
_y\x22,\x22layer_seque\
nce_get_yscale\x22,\
\x22layer_sequence_\
headdir\x22,\x22layer_\
sequence_headpos\
\x22,\x22layer_sequenc\
e_is_finished\x22,\x22\
layer_sequence_i\
s_paused\x22,\x22layer\
_sequence_pause\x22\
,\x22layer_sequence\
_play\x22,\x22layer_se\
quence_speedscal\
e\x22,\x22layer_sequen\
ce_x\x22,\x22layer_seq\
uence_xscale\x22,\x22l\
ayer_sequence_y\x22\
,\x22layer_sequence\
_yscale\x22,\x22layer_\
set_fx\x22,\x22layer_s\
et_target_room\x22,\
\x22layer_set_visib\
le\x22,\x22layer_shade\
r\x22,\x22layer_sprite\
_alpha\x22,\x22layer_s\
prite_angle\x22,\x22la\
yer_sprite_blend\
\x22,\x22layer_sprite_\
change\x22,\x22layer_s\
prite_create\x22,\x22l\
ayer_sprite_dest\
roy\x22,\x22layer_spri\
te_exists\x22,\x22laye\
r_sprite_get_alp\
ha\x22,\x22layer_sprit\
e_get_angle\x22,\x22la\
yer_sprite_get_b\
lend\x22,\x22layer_spr\
ite_get_id\x22,\x22lay\
er_sprite_get_in\
dex\x22,\x22layer_spri\
te_get_speed\x22,\x22l\
ayer_sprite_get_\
sprite\x22,\x22layer_s\
prite_get_x\x22,\x22la\
yer_sprite_get_x\
scale\x22,\x22layer_sp\
rite_get_y\x22,\x22lay\
er_sprite_get_ys\
cale\x22,\x22layer_spr\
ite_index\x22,\x22laye\
r_sprite_speed\x22,\
\x22layer_sprite_x\x22\
,\x22layer_sprite_x\
scale\x22,\x22layer_sp\
rite_y\x22,\x22layer_s\
prite_yscale\x22,\x22l\
ayer_tile_alpha\x22\
,\x22layer_tile_ble\
nd\x22,\x22layer_tile_\
change\x22,\x22layer_t\
ile_create\x22,\x22lay\
er_tile_destroy\x22\
,\x22layer_tile_exi\
sts\x22,\x22layer_tile\
_get_alpha\x22,\x22lay\
er_tile_get_blen\
d\x22,\x22layer_tile_g\
et_region\x22,\x22laye\
r_tile_get_sprit\
e\x22,\x22layer_tile_g\
et_visible\x22,\x22lay\
er_tile_get_x\x22,\x22\
layer_tile_get_x\
scale\x22,\x22layer_ti\
le_get_y\x22,\x22layer\
_tile_get_yscale\
\x22,\x22layer_tile_re\
gion\x22,\x22layer_til\
e_visible\x22,\x22laye\
r_tile_x\x22,\x22layer\
_tile_xscale\x22,\x22l\
ayer_tile_y\x22,\x22la\
yer_tile_yscale\x22\
,\x22layer_tilemap_\
create\x22,\x22layer_t\
ilemap_destroy\x22,\
\x22layer_tilemap_e\
xists\x22,\x22layer_ti\
lemap_get_id\x22,\x22l\
ayer_vspeed\x22,\x22la\
yer_x\x22,\x22layer_y\x22\
,\x22lengthdir_x\x22,\x22\
lengthdir_y\x22,\x22le\
rp\x22,\x22lin_to_db\x22,\
\x22ln\x22,\x22load_csv\x22,\
\x22log10\x22,\x22log2\x22,\x22\
logn\x22,\x22make_colo\
r_hsv\x22,\x22make_col\
or_rgb\x22,\x22make_co\
lour_hsv\x22,\x22make_\
colour_rgb\x22,\x22mat\
h_get_epsilon\x22,\x22\
math_set_epsilon\
\x22,\x22matrix_build\x22\
,\x22matrix_build_i\
dentity\x22,\x22matrix\
_build_lookat\x22,\x22\
matrix_build_pro\
jection_ortho\x22,\x22\
matrix_build_pro\
jection_perspect\
ive\x22,\x22matrix_bui\
ld_projection_pe\
rspective_fov\x22,\x22\
matrix_get\x22,\x22mat\
rix_multiply\x22,\x22m\
atrix_set\x22,\x22matr\
ix_stack_clear\x22,\
\x22matrix_stack_is\
_empty\x22,\x22matrix_\
stack_pop\x22,\x22matr\
ix_stack_push\x22,\x22\
matrix_stack_set\
\x22,\x22matrix_stack_\
top\x22,\x22matrix_tra\
nsform_vertex\x22,\x22\
max\x22,\x22md5_file\x22,\
\x22md5_string_unic\
ode\x22,\x22md5_string\
_utf8\x22,\x22mean\x22,\x22m\
edian\x22,\x22merge_co\
lor\x22,\x22merge_colo\
ur\x22,\x22method\x22,\x22me\
thod_call\x22,\x22meth\
od_get_index\x22,\x22m\
ethod_get_self\x22,\
\x22min\x22,\x22motion_ad\
d\x22,\x22motion_set\x22,\
\x22mouse_check_but\
ton\x22,\x22mouse_chec\
k_button_pressed\
\x22,\x22mouse_check_b\
utton_released\x22,\
\x22mouse_clear\x22,\x22m\
ouse_wheel_down\x22\
,\x22mouse_wheel_up\
\x22,\x22move_and_coll\
ide\x22,\x22move_bounc\
e_all\x22,\x22move_bou\
nce_solid\x22,\x22move\
_contact_all\x22,\x22m\
ove_contact_soli\
d\x22,\x22move_outside\
_all\x22,\x22move_outs\
ide_solid\x22,\x22move\
_random\x22,\x22move_s\
nap\x22,\x22move_towar\
ds_point\x22,\x22move_\
wrap\x22,\x22mp_grid_a\
dd_cell\x22,\x22mp_gri\
d_add_instances\x22\
,\x22mp_grid_add_re\
ctangle\x22,\x22mp_gri\
d_clear_all\x22,\x22mp\
_grid_clear_cell\
\x22,\x22mp_grid_clear\
_rectangle\x22,\x22mp_\
grid_create\x22,\x22mp\
_grid_destroy\x22,\x22\
mp_grid_draw\x22,\x22m\
p_grid_get_cell\x22\
,\x22mp_grid_path\x22,\
\x22mp_grid_to_ds_g\
rid\x22,\x22mp_linear_\
path\x22,\x22mp_linear\
_path_object\x22,\x22m\
p_linear_step\x22,\x22\
mp_linear_step_o\
bject\x22,\x22mp_poten\
tial_path\x22,\x22mp_p\
otential_path_ob\
ject\x22,\x22mp_potent\
ial_settings\x22,\x22m\
p_potential_step\
\x22,\x22mp_potential_\
step_object\x22,\x22na\
meof\x22,\x22network_c\
onnect\x22,\x22network\
_connect_async\x22,\
\x22network_connect\
_raw\x22,\x22network_c\
onnect_raw_async\
\x22,\x22network_creat\
e_server\x22,\x22netwo\
rk_create_server\
_raw\x22,\x22network_c\
reate_socket\x22,\x22n\
etwork_create_so\
cket_ext\x22,\x22netwo\
rk_destroy\x22,\x22net\
work_resolve\x22,\x22n\
etwork_send_broa\
dcast\x22,\x22network_\
send_packet\x22,\x22ne\
twork_send_raw\x22,\
\x22network_send_ud\
p\x22,\x22network_send\
_udp_raw\x22,\x22netwo\
rk_set_config\x22,\x22\
network_set_time\
out\x22,\x22object_exi\
sts\x22,\x22object_get\
_mask\x22,\x22object_g\
et_name\x22,\x22object\
_get_parent\x22,\x22ob\
ject_get_persist\
ent\x22,\x22object_get\
_physics\x22,\x22objec\
t_get_solid\x22,\x22ob\
ject_get_sprite\x22\
,\x22object_get_vis\
ible\x22,\x22object_is\
_ancestor\x22,\x22obje\
ct_set_mask\x22,\x22ob\
ject_set_persist\
ent\x22,\x22object_set\
_solid\x22,\x22object_\
set_sprite\x22,\x22obj\
ect_set_visible\x22\
,\x22ord\x22,\x22os_check\
_permission\x22,\x22os\
_get_config\x22,\x22os\
_get_info\x22,\x22os_g\
et_language\x22,\x22os\
_get_region\x22,\x22os\
_is_network_conn\
ected\x22,\x22os_is_pa\
used\x22,\x22os_lock_o\
rientation\x22,\x22os_\
powersave_enable\
\x22,\x22os_request_pe\
rmission\x22,\x22os_se\
t_orientation_lo\
ck\x22,\x22parameter_c\
ount\x22,\x22parameter\
_string\x22,\x22part_e\
mitter_burst\x22,\x22p\
art_emitter_clea\
r\x22,\x22part_emitter\
_create\x22,\x22part_e\
mitter_delay\x22,\x22p\
art_emitter_dest\
roy\x22,\x22part_emitt\
er_destroy_all\x22,\
\x22part_emitter_en\
able\x22,\x22part_emit\
ter_exists\x22,\x22par\
t_emitter_interv\
al\x22,\x22part_emitte\
r_region\x22,\x22part_\
emitter_relative\
\x22,\x22part_emitter_\
stream\x22,\x22part_pa\
rticles_burst\x22,\x22\
part_particles_c\
lear\x22,\x22part_part\
icles_count\x22,\x22pa\
rt_particles_cre\
ate\x22,\x22part_parti\
cles_create_colo\
r\x22,\x22part_particl\
es_create_colour\
\x22,\x22part_system_a\
ngle\x22,\x22part_syst\
em_automatic_dra\
w\x22,\x22part_system_\
automatic_update\
\x22,\x22part_system_c\
lear\x22,\x22part_syst\
em_color\x22,\x22part_\
system_colour\x22,\x22\
part_system_crea\
te\x22,\x22part_system\
_create_layer\x22,\x22\
part_system_dept\
h\x22,\x22part_system_\
destroy\x22,\x22part_s\
ystem_draw_order\
\x22,\x22part_system_d\
rawit\x22,\x22part_sys\
tem_exists\x22,\x22par\
t_system_get_inf\
o\x22,\x22part_system_\
get_layer\x22,\x22part\
_system_global_s\
pace\x22,\x22part_syst\
em_layer\x22,\x22part_\
system_position\x22\
,\x22part_system_up\
date\x22,\x22part_type\
_alpha1\x22,\x22part_t\
ype_alpha2\x22,\x22par\
t_type_alpha3\x22,\x22\
part_type_blend\x22\
,\x22part_type_clea\
r\x22,\x22part_type_co\
lor1\x22,\x22part_type\
_color2\x22,\x22part_t\
ype_color3\x22,\x22par\
t_type_color_hsv\
\x22,\x22part_type_col\
or_mix\x22,\x22part_ty\
pe_color_rgb\x22,\x22p\
art_type_colour1\
\x22,\x22part_type_col\
our2\x22,\x22part_type\
_colour3\x22,\x22part_\
type_colour_hsv\x22\
,\x22part_type_colo\
ur_mix\x22,\x22part_ty\
pe_colour_rgb\x22,\x22\
part_type_create\
\x22,\x22part_type_dea\
th\x22,\x22part_type_d\
estroy\x22,\x22part_ty\
pe_direction\x22,\x22p\
art_type_exists\x22\
,\x22part_type_grav\
ity\x22,\x22part_type_\
life\x22,\x22part_type\
_orientation\x22,\x22p\
art_type_scale\x22,\
\x22part_type_shape\
\x22,\x22part_type_siz\
e\x22,\x22part_type_si\
ze_x\x22,\x22part_type\
_size_y\x22,\x22part_t\
ype_speed\x22,\x22part\
_type_sprite\x22,\x22p\
art_type_step\x22,\x22\
part_type_subima\
ge\x22,\x22particle_ex\
ists\x22,\x22particle_\
get_info\x22,\x22path_\
add\x22,\x22path_add_p\
oint\x22,\x22path_appe\
nd\x22,\x22path_assign\
\x22,\x22path_change_p\
oint\x22,\x22path_clea\
r_points\x22,\x22path_\
delete\x22,\x22path_de\
lete_point\x22,\x22pat\
h_duplicate\x22,\x22pa\
th_end\x22,\x22path_ex\
ists\x22,\x22path_flip\
\x22,\x22path_get_clos\
ed\x22,\x22path_get_ki\
nd\x22,\x22path_get_le\
ngth\x22,\x22path_get_\
name\x22,\x22path_get_\
number\x22,\x22path_ge\
t_point_speed\x22,\x22\
path_get_point_x\
\x22,\x22path_get_poin\
t_y\x22,\x22path_get_p\
recision\x22,\x22path_\
get_speed\x22,\x22path\
_get_x\x22,\x22path_ge\
t_y\x22,\x22path_inser\
t_point\x22,\x22path_m\
irror\x22,\x22path_res\
cale\x22,\x22path_reve\
rse\x22,\x22path_rotat\
e\x22,\x22path_set_clo\
sed\x22,\x22path_set_k\
ind\x22,\x22path_set_p\
recision\x22,\x22path_\
shift\x22,\x22path_sta\
rt\x22,\x22physics_app\
ly_angular_impul\
se\x22,\x22physics_app\
ly_force\x22,\x22physi\
cs_apply_impulse\
\x22,\x22physics_apply\
_local_force\x22,\x22p\
hysics_apply_loc\
al_impulse\x22,\x22phy\
sics_apply_torqu\
e\x22,\x22physics_draw\
_debug\x22,\x22physics\
_fixture_add_poi\
nt\x22,\x22physics_fix\
ture_bind\x22,\x22phys\
ics_fixture_bind\
_ext\x22,\x22physics_f\
ixture_create\x22,\x22\
physics_fixture_\
delete\x22,\x22physics\
_fixture_set_ang\
ular_damping\x22,\x22p\
hysics_fixture_s\
et_awake\x22,\x22physi\
cs_fixture_set_b\
ox_shape\x22,\x22physi\
cs_fixture_set_c\
hain_shape\x22,\x22phy\
sics_fixture_set\
_circle_shape\x22,\x22\
physics_fixture_\
set_collision_gr\
oup\x22,\x22physics_fi\
xture_set_densit\
y\x22,\x22physics_fixt\
ure_set_edge_sha\
pe\x22,\x22physics_fix\
ture_set_frictio\
n\x22,\x22physics_fixt\
ure_set_kinemati\
c\x22,\x22physics_fixt\
ure_set_linear_d\
amping\x22,\x22physics\
_fixture_set_pol\
ygon_shape\x22,\x22phy\
sics_fixture_set\
_restitution\x22,\x22p\
hysics_fixture_s\
et_sensor\x22,\x22phys\
ics_get_density\x22\
,\x22physics_get_fr\
iction\x22,\x22physics\
_get_restitution\
\x22,\x22physics_joint\
_delete\x22,\x22physic\
s_joint_distance\
_create\x22,\x22physic\
s_joint_enable_m\
otor\x22,\x22physics_j\
oint_friction_cr\
eate\x22,\x22physics_j\
oint_gear_create\
\x22,\x22physics_joint\
_get_value\x22,\x22phy\
sics_joint_prism\
atic_create\x22,\x22ph\
ysics_joint_pull\
ey_create\x22,\x22phys\
ics_joint_revolu\
te_create\x22,\x22phys\
ics_joint_rope_c\
reate\x22,\x22physics_\
joint_set_value\x22\
,\x22physics_joint_\
weld_create\x22,\x22ph\
ysics_joint_whee\
l_create\x22,\x22physi\
cs_mass_properti\
es\x22,\x22physics_par\
ticle_count\x22,\x22ph\
ysics_particle_c\
reate\x22,\x22physics_\
particle_delete\x22\
,\x22physics_partic\
le_delete_region\
_box\x22,\x22physics_p\
article_delete_r\
egion_circle\x22,\x22p\
hysics_particle_\
delete_region_po\
ly\x22,\x22physics_par\
ticle_draw\x22,\x22phy\
sics_particle_dr\
aw_ext\x22,\x22physics\
_particle_get_da\
mping\x22,\x22physics_\
particle_get_dat\
a\x22,\x22physics_part\
icle_get_data_pa\
rticle\x22,\x22physics\
_particle_get_de\
nsity\x22,\x22physics_\
particle_get_gra\
vity_scale\x22,\x22phy\
sics_particle_ge\
t_group_flags\x22,\x22\
physics_particle\
_get_max_count\x22,\
\x22physics_particl\
e_get_radius\x22,\x22p\
hysics_particle_\
group_add_point\x22\
,\x22physics_partic\
le_group_begin\x22,\
\x22physics_particl\
e_group_box\x22,\x22ph\
ysics_particle_g\
roup_circle\x22,\x22ph\
ysics_particle_g\
roup_count\x22,\x22phy\
sics_particle_gr\
oup_delete\x22,\x22phy\
sics_particle_gr\
oup_end\x22,\x22physic\
s_particle_group\
_get_ang_vel\x22,\x22p\
hysics_particle_\
group_get_angle\x22\
,\x22physics_partic\
le_group_get_cen\
tre_x\x22,\x22physics_\
particle_group_g\
et_centre_y\x22,\x22ph\
ysics_particle_g\
roup_get_data\x22,\x22\
physics_particle\
_group_get_inert\
ia\x22,\x22physics_par\
ticle_group_get_\
mass\x22,\x22physics_p\
article_group_ge\
t_vel_x\x22,\x22physic\
s_particle_group\
_get_vel_y\x22,\x22phy\
sics_particle_gr\
oup_get_x\x22,\x22phys\
ics_particle_gro\
up_get_y\x22,\x22physi\
cs_particle_grou\
p_join\x22,\x22physics\
_particle_group_\
polygon\x22,\x22physic\
s_particle_set_c\
ategory_flags\x22,\x22\
physics_particle\
_set_damping\x22,\x22p\
hysics_particle_\
set_density\x22,\x22ph\
ysics_particle_s\
et_flags\x22,\x22physi\
cs_particle_set_\
gravity_scale\x22,\x22\
physics_particle\
_set_group_flags\
\x22,\x22physics_parti\
cle_set_max_coun\
t\x22,\x22physics_part\
icle_set_radius\x22\
,\x22physics_pause_\
enable\x22,\x22physics\
_remove_fixture\x22\
,\x22physics_set_de\
nsity\x22,\x22physics_\
set_friction\x22,\x22p\
hysics_set_resti\
tution\x22,\x22physics\
_test_overlap\x22,\x22\
physics_world_cr\
eate\x22,\x22physics_w\
orld_draw_debug\x22\
,\x22physics_world_\
gravity\x22,\x22physic\
s_world_update_i\
terations\x22,\x22phys\
ics_world_update\
_speed\x22,\x22place_e\
mpty\x22,\x22place_fre\
e\x22,\x22place_meetin\
g\x22,\x22place_snappe\
d\x22,\x22point_direct\
ion\x22,\x22point_dist\
ance\x22,\x22point_dis\
tance_3d\x22,\x22point\
_in_circle\x22,\x22poi\
nt_in_rectangle\x22\
,\x22point_in_trian\
gle\x22,\x22position_c\
hange\x22,\x22position\
_destroy\x22,\x22posit\
ion_empty\x22,\x22posi\
tion_meeting\x22,\x22p\
ower\x22,\x22ptr\x22,\x22rad\
todeg\x22,\x22random\x22,\
\x22random_get_seed\
\x22,\x22random_range\x22\
,\x22random_set_see\
d\x22,\x22randomise\x22,\x22\
randomize\x22,\x22real\
\x22,\x22rectangle_in_\
circle\x22,\x22rectang\
le_in_rectangle\x22\
,\x22rectangle_in_t\
riangle\x22,\x22ref_cr\
eate\x22,\x22rollback_\
chat\x22,\x22rollback_\
create_game\x22,\x22ro\
llback_define_ex\
tra_network_late\
ncy\x22,\x22rollback_d\
efine_input\x22,\x22ro\
llback_define_in\
put_frame_delay\x22\
,\x22rollback_defin\
e_mock_input\x22,\x22r\
ollback_define_p\
layer\x22,\x22rollback\
_display_events\x22\
,\x22rollback_get_i\
nfo\x22,\x22rollback_g\
et_input\x22,\x22rollb\
ack_get_player_p\
refs\x22,\x22rollback_\
join_game\x22,\x22roll\
back_leave_game\x22\
,\x22rollback_set_p\
layer_prefs\x22,\x22ro\
llback_start_gam\
e\x22,\x22rollback_syn\
c_on_frame\x22,\x22rol\
lback_use_late_j\
oin\x22,\x22rollback_u\
se_manual_start\x22\
,\x22rollback_use_p\
layer_prefs\x22,\x22ro\
llback_use_rando\
m_input\x22,\x22room_a\
dd\x22,\x22room_assign\
\x22,\x22room_duplicat\
e\x22,\x22room_exists\x22\
,\x22room_get_camer\
a\x22,\x22room_get_inf\
o\x22,\x22room_get_nam\
e\x22,\x22room_get_vie\
wport\x22,\x22room_got\
o\x22,\x22room_goto_ne\
xt\x22,\x22room_goto_p\
revious\x22,\x22room_i\
nstance_add\x22,\x22ro\
om_instance_clea\
r\x22,\x22room_next\x22,\x22\
room_previous\x22,\x22\
room_restart\x22,\x22r\
oom_set_camera\x22,\
\x22room_set_height\
\x22,\x22room_set_pers\
istent\x22,\x22room_se\
t_view_enabled\x22,\
\x22room_set_viewpo\
rt\x22,\x22room_set_wi\
dth\x22,\x22round\x22,\x22sc\
heduler_resoluti\
on_get\x22,\x22schedul\
er_resolution_se\
t\x22,\x22screen_save\x22\
,\x22screen_save_pa\
rt\x22,\x22script_exec\
ute\x22,\x22script_exe\
cute_ext\x22,\x22scrip\
t_exists\x22,\x22scrip\
t_get_name\x22,\x22seq\
uence_create\x22,\x22s\
equence_destroy\x22\
,\x22sequence_exist\
s\x22,\x22sequence_get\
\x22,\x22sequence_get_\
objects\x22,\x22sequen\
ce_instance_over\
ride_object\x22,\x22se\
quence_keyframe_\
new\x22,\x22sequence_k\
eyframedata_new\x22\
,\x22sequence_track\
_new\x22,\x22sha1_file\
\x22,\x22sha1_string_u\
nicode\x22,\x22sha1_st\
ring_utf8\x22,\x22shad\
er_current\x22,\x22sha\
der_enable_corne\
r_id\x22,\x22shader_ge\
t_name\x22,\x22shader_\
get_sampler_inde\
x\x22,\x22shader_get_u\
niform\x22,\x22shader_\
is_compiled\x22,\x22sh\
ader_reset\x22,\x22sha\
der_set\x22,\x22shader\
_set_uniform_f\x22,\
\x22shader_set_unif\
orm_f_array\x22,\x22sh\
ader_set_uniform\
_f_buffer\x22,\x22shad\
er_set_uniform_i\
\x22,\x22shader_set_un\
iform_i_array\x22,\x22\
shader_set_unifo\
rm_matrix\x22,\x22shad\
er_set_uniform_m\
atrix_array\x22,\x22sh\
aders_are_suppor\
ted\x22,\x22shop_leave\
_rating\x22,\x22show_d\
ebug_message\x22,\x22s\
how_debug_messag\
e_ext\x22,\x22show_deb\
ug_overlay\x22,\x22sho\
w_error\x22,\x22show_m\
essage\x22,\x22show_me\
ssage_async\x22,\x22sh\
ow_question\x22,\x22sh\
ow_question_asyn\
c\x22,\x22sign\x22,\x22sin\x22,\
\x22skeleton_animat\
ion_clear\x22,\x22skel\
eton_animation_g\
et\x22,\x22skeleton_an\
imation_get_dura\
tion\x22,\x22skeleton_\
animation_get_ev\
ent_frames\x22,\x22ske\
leton_animation_\
get_ext\x22,\x22skelet\
on_animation_get\
_frame\x22,\x22skeleto\
n_animation_get_\
frames\x22,\x22skeleto\
n_animation_get_\
position\x22,\x22skele\
ton_animation_is\
_finished\x22,\x22skel\
eton_animation_i\
s_looping\x22,\x22skel\
eton_animation_l\
ist\x22,\x22skeleton_a\
nimation_mix\x22,\x22s\
keleton_animatio\
n_set\x22,\x22skeleton\
_animation_set_e\
xt\x22,\x22skeleton_an\
imation_set_fram\
e\x22,\x22skeleton_ani\
mation_set_posit\
ion\x22,\x22skeleton_a\
ttachment_create\
\x22,\x22skeleton_atta\
chment_create_co\
lor\x22,\x22skeleton_a\
ttachment_create\
_colour\x22,\x22skelet\
on_attachment_de\
stroy\x22,\x22skeleton\
_attachment_exis\
ts\x22,\x22skeleton_at\
tachment_get\x22,\x22s\
keleton_attachme\
nt_replace\x22,\x22ske\
leton_attachment\
_replace_color\x22,\
\x22skeleton_attach\
ment_replace_col\
our\x22,\x22skeleton_a\
ttachment_set\x22,\x22\
skeleton_bone_da\
ta_get\x22,\x22skeleto\
n_bone_data_set\x22\
,\x22skeleton_bone_\
list\x22,\x22skeleton_\
bone_state_get\x22,\
\x22skeleton_bone_s\
tate_set\x22,\x22skele\
ton_collision_dr\
aw_set\x22,\x22skeleto\
n_find_slot\x22,\x22sk\
eleton_get_bound\
s\x22,\x22skeleton_get\
_minmax\x22,\x22skelet\
on_get_num_bound\
s\x22,\x22skeleton_ski\
n_create\x22,\x22skele\
ton_skin_get\x22,\x22s\
keleton_skin_lis\
t\x22,\x22skeleton_ski\
n_set\x22,\x22skeleton\
_slot_alpha_get\x22\
,\x22skeleton_slot_\
color_get\x22,\x22skel\
eton_slot_color_\
set\x22,\x22skeleton_s\
lot_colour_get\x22,\
\x22skeleton_slot_c\
olour_set\x22,\x22skel\
eton_slot_data\x22,\
\x22skeleton_slot_d\
ata_instance\x22,\x22s\
keleton_slot_lis\
t\x22,\x22sprite_add\x22,\
\x22sprite_add_ext\x22\
,\x22sprite_add_fro\
m_surface\x22,\x22spri\
te_assign\x22,\x22spri\
te_collision_mas\
k\x22,\x22sprite_creat\
e_from_surface\x22,\
\x22sprite_delete\x22,\
\x22sprite_duplicat\
e\x22,\x22sprite_exist\
s\x22,\x22sprite_flush\
\x22,\x22sprite_flush_\
multi\x22,\x22sprite_g\
et_bbox_bottom\x22,\
\x22sprite_get_bbox\
_left\x22,\x22sprite_g\
et_bbox_mode\x22,\x22s\
prite_get_bbox_r\
ight\x22,\x22sprite_ge\
t_bbox_top\x22,\x22spr\
ite_get_height\x22,\
\x22sprite_get_info\
\x22,\x22sprite_get_na\
me\x22,\x22sprite_get_\
nineslice\x22,\x22spri\
te_get_number\x22,\x22\
sprite_get_speed\
\x22,\x22sprite_get_sp\
eed_type\x22,\x22sprit\
e_get_texture\x22,\x22\
sprite_get_tpe\x22,\
\x22sprite_get_uvs\x22\
,\x22sprite_get_wid\
th\x22,\x22sprite_get_\
xoffset\x22,\x22sprite\
_get_yoffset\x22,\x22s\
prite_merge\x22,\x22sp\
rite_nineslice_c\
reate\x22,\x22sprite_p\
refetch\x22,\x22sprite\
_prefetch_multi\x22\
,\x22sprite_replace\
\x22,\x22sprite_save\x22,\
\x22sprite_save_str\
ip\x22,\x22sprite_set_\
alpha_from_sprit\
e\x22,\x22sprite_set_b\
box\x22,\x22sprite_set\
_bbox_mode\x22,\x22spr\
ite_set_cache_si\
ze\x22,\x22sprite_set_\
cache_size_ext\x22,\
\x22sprite_set_nine\
slice\x22,\x22sprite_s\
et_offset\x22,\x22spri\
te_set_speed\x22,\x22s\
qr\x22,\x22sqrt\x22,\x22stat\
ic_get\x22,\x22static_\
set\x22,\x22string\x22,\x22s\
tring_byte_at\x22,\x22\
string_byte_leng\
th\x22,\x22string_char\
_at\x22,\x22string_con\
cat\x22,\x22string_con\
cat_ext\x22,\x22string\
_copy\x22,\x22string_c\
ount\x22,\x22string_de\
lete\x22,\x22string_di\
gits\x22,\x22string_en\
ds_with\x22,\x22string\
_ext\x22,\x22string_fo\
reach\x22,\x22string_f\
ormat\x22,\x22string_h\
ash_to_newline\x22,\
\x22string_height\x22,\
\x22string_height_e\
xt\x22,\x22string_inse\
rt\x22,\x22string_join\
\x22,\x22string_join_e\
xt\x22,\x22string_last\
_pos\x22,\x22string_la\
st_pos_ext\x22,\x22str\
ing_length\x22,\x22str\
ing_letters\x22,\x22st\
ring_lettersdigi\
ts\x22,\x22string_lowe\
r\x22,\x22string_ord_a\
t\x22,\x22string_pos\x22,\
\x22string_pos_ext\x22\
,\x22string_repeat\x22\
,\x22string_replace\
\x22,\x22string_replac\
e_all\x22,\x22string_s\
et_byte_at\x22,\x22str\
ing_split\x22,\x22stri\
ng_split_ext\x22,\x22s\
tring_starts_wit\
h\x22,\x22string_trim\x22\
,\x22string_trim_en\
d\x22,\x22string_trim_\
start\x22,\x22string_u\
pper\x22,\x22string_wi\
dth\x22,\x22string_wid\
th_ext\x22,\x22struct_\
exists\x22,\x22struct_\
foreach\x22,\x22struct\
_get\x22,\x22struct_ge\
t_from_hash\x22,\x22st\
ruct_get_names\x22,\
\x22struct_names_co\
unt\x22,\x22struct_rem\
ove\x22,\x22struct_set\
\x22,\x22struct_set_fr\
om_hash\x22,\x22surfac\
e_copy\x22,\x22surface\
_copy_part\x22,\x22sur\
face_create\x22,\x22su\
rface_create_ext\
\x22,\x22surface_depth\
_disable\x22,\x22surfa\
ce_exists\x22,\x22surf\
ace_format_is_su\
pported\x22,\x22surfac\
e_free\x22,\x22surface\
_get_depth_disab\
le\x22,\x22surface_get\
_format\x22,\x22surfac\
e_get_height\x22,\x22s\
urface_get_targe\
t\x22,\x22surface_get_\
target_ext\x22,\x22sur\
face_get_texture\
\x22,\x22surface_get_w\
idth\x22,\x22surface_g\
etpixel\x22,\x22surfac\
e_getpixel_ext\x22,\
\x22surface_reset_t\
arget\x22,\x22surface_\
resize\x22,\x22surface\
_save\x22,\x22surface_\
save_part\x22,\x22surf\
ace_set_target\x22,\
\x22surface_set_tar\
get_ext\x22,\x22tag_ge\
t_asset_ids\x22,\x22ta\
g_get_assets\x22,\x22t\
an\x22,\x22texture_deb\
ug_messages\x22,\x22te\
xture_flush\x22,\x22te\
xture_get_height\
\x22,\x22texture_get_t\
exel_height\x22,\x22te\
xture_get_texel_\
width\x22,\x22texture_\
get_uvs\x22,\x22textur\
e_get_width\x22,\x22te\
xture_global_sca\
le\x22,\x22texture_is_\
ready\x22,\x22texture_\
prefetch\x22,\x22textu\
re_set_stage\x22,\x22t\
exturegroup_get_\
fonts\x22,\x22textureg\
roup_get_names\x22,\
\x22texturegroup_ge\
t_sprites\x22,\x22text\
uregroup_get_sta\
tus\x22,\x22texturegro\
up_get_textures\x22\
,\x22texturegroup_g\
et_tilesets\x22,\x22te\
xturegroup_load\x22\
,\x22texturegroup_s\
et_mode\x22,\x22textur\
egroup_unload\x22,\x22\
tile_get_empty\x22,\
\x22tile_get_flip\x22,\
\x22tile_get_index\x22\
,\x22tile_get_mirro\
r\x22,\x22tile_get_rot\
ate\x22,\x22tile_set_e\
mpty\x22,\x22tile_set_\
flip\x22,\x22tile_set_\
index\x22,\x22tile_set\
_mirror\x22,\x22tile_s\
et_rotate\x22,\x22tile\
map_clear\x22,\x22tile\
map_get\x22,\x22tilema\
p_get_at_pixel\x22,\
\x22tilemap_get_cel\
l_x_at_pixel\x22,\x22t\
ilemap_get_cell_\
y_at_pixel\x22,\x22til\
emap_get_frame\x22,\
\x22tilemap_get_glo\
bal_mask\x22,\x22tilem\
ap_get_height\x22,\x22\
tilemap_get_mask\
\x22,\x22tilemap_get_t\
ile_height\x22,\x22til\
emap_get_tile_wi\
dth\x22,\x22tilemap_ge\
t_tileset\x22,\x22tile\
map_get_width\x22,\x22\
tilemap_get_x\x22,\x22\
tilemap_get_y\x22,\x22\
tilemap_set\x22,\x22ti\
lemap_set_at_pix\
el\x22,\x22tilemap_set\
_global_mask\x22,\x22t\
ilemap_set_heigh\
t\x22,\x22tilemap_set_\
mask\x22,\x22tilemap_s\
et_width\x22,\x22tilem\
ap_tileset\x22,\x22til\
emap_x\x22,\x22tilemap\
_y\x22,\x22tileset_get\
_info\x22,\x22tileset_\
get_name\x22,\x22tiles\
et_get_texture\x22,\
\x22tileset_get_uvs\
\x22,\x22time_bpm_to_s\
econds\x22,\x22time_se\
conds_to_bpm\x22,\x22t\
ime_source_creat\
e\x22,\x22time_source_\
destroy\x22,\x22time_s\
ource_exists\x22,\x22t\
ime_source_get_c\
hildren\x22,\x22time_s\
ource_get_parent\
\x22,\x22time_source_g\
et_period\x22,\x22time\
_source_get_reps\
_completed\x22,\x22tim\
e_source_get_rep\
s_remaining\x22,\x22ti\
me_source_get_st\
ate\x22,\x22time_sourc\
e_get_time_remai\
ning\x22,\x22time_sour\
ce_get_units\x22,\x22t\
ime_source_pause\
\x22,\x22time_source_r\
econfigure\x22,\x22tim\
e_source_reset\x22,\
\x22time_source_res\
ume\x22,\x22time_sourc\
e_start\x22,\x22time_s\
ource_stop\x22,\x22tim\
eline_add\x22,\x22time\
line_clear\x22,\x22tim\
eline_delete\x22,\x22t\
imeline_exists\x22,\
\x22timeline_get_na\
me\x22,\x22timeline_ma\
x_moment\x22,\x22timel\
ine_moment_add_s\
cript\x22,\x22timeline\
_moment_clear\x22,\x22\
timeline_size\x22,\x22\
typeof\x22,\x22url_get\
_domain\x22,\x22url_op\
en\x22,\x22url_open_ex\
t\x22,\x22url_open_ful\
l\x22,\x22uwp_device_t\
ouchscreen_avail\
able\x22,\x22uwp_livet\
ile_badge_clear\x22\
,\x22uwp_livetile_b\
adge_notificatio\
n\x22,\x22uwp_livetile\
_notification_be\
gin\x22,\x22uwp_liveti\
le_notification_\
end\x22,\x22uwp_liveti\
le_notification_\
expiry\x22,\x22uwp_liv\
etile_notificati\
on_image_add\x22,\x22u\
wp_livetile_noti\
fication_seconda\
ry_begin\x22,\x22uwp_l\
ivetile_notifica\
tion_tag\x22,\x22uwp_l\
ivetile_notifica\
tion_template_ad\
d\x22,\x22uwp_livetile\
_notification_te\
xt_add\x22,\x22uwp_liv\
etile_queue_enab\
le\x22,\x22uwp_livetil\
e_tile_clear\x22,\x22u\
wp_secondarytile\
_badge_clear\x22,\x22u\
wp_secondarytile\
_badge_notificat\
ion\x22,\x22uwp_second\
arytile_delete\x22,\
\x22uwp_secondaryti\
le_pin\x22,\x22uwp_sec\
ondarytile_tile_\
clear\x22,\x22variable\
_clone\x22,\x22variabl\
e_get_hash\x22,\x22var\
iable_global_exi\
sts\x22,\x22variable_g\
lobal_get\x22,\x22vari\
able_global_set\x22\
,\x22variable_insta\
nce_exists\x22,\x22var\
iable_instance_g\
et\x22,\x22variable_in\
stance_get_names\
\x22,\x22variable_inst\
ance_names_count\
\x22,\x22variable_inst\
ance_set\x22,\x22varia\
ble_struct_exist\
s\x22,\x22variable_str\
uct_get\x22,\x22variab\
le_struct_get_na\
mes\x22,\x22variable_s\
truct_names_coun\
t\x22,\x22variable_str\
uct_remove\x22,\x22var\
iable_struct_set\
\x22,\x22vertex_argb\x22,\
\x22vertex_begin\x22,\x22\
vertex_color\x22,\x22v\
ertex_colour\x22,\x22v\
ertex_create_buf\
fer\x22,\x22vertex_cre\
ate_buffer_ext\x22,\
\x22vertex_create_b\
uffer_from_buffe\
r\x22,\x22vertex_creat\
e_buffer_from_bu\
ffer_ext\x22,\x22verte\
x_delete_buffer\x22\
,\x22vertex_end\x22,\x22v\
ertex_float1\x22,\x22v\
ertex_float2\x22,\x22v\
ertex_float3\x22,\x22v\
ertex_float4\x22,\x22v\
ertex_format_add\
_color\x22,\x22vertex_\
format_add_colou\
r\x22,\x22vertex_forma\
t_add_custom\x22,\x22v\
ertex_format_add\
_normal\x22,\x22vertex\
_format_add_posi\
tion\x22,\x22vertex_fo\
rmat_add_positio\
n_3d\x22,\x22vertex_fo\
rmat_add_texcoor\
d\x22,\x22vertex_forma\
t_begin\x22,\x22vertex\
_format_delete\x22,\
\x22vertex_format_e\
nd\x22,\x22vertex_form\
at_get_info\x22,\x22ve\
rtex_freeze\x22,\x22ve\
rtex_get_buffer_\
size\x22,\x22vertex_ge\
t_number\x22,\x22verte\
x_normal\x22,\x22verte\
x_position\x22,\x22ver\
tex_position_3d\x22\
,\x22vertex_submit\x22\
,\x22vertex_submit_\
ext\x22,\x22vertex_tex\
coord\x22,\x22vertex_u\
byte4\x22,\x22vertex_u\
pdate_buffer_fro\
m_buffer\x22,\x22verte\
x_update_buffer_\
from_vertex\x22,\x22vi\
deo_close\x22,\x22vide\
o_draw\x22,\x22video_e\
nable_loop\x22,\x22vid\
eo_get_duration\x22\
,\x22video_get_form\
at\x22,\x22video_get_p\
osition\x22,\x22video_\
get_status\x22,\x22vid\
eo_get_volume\x22,\x22\
video_is_looping\
\x22,\x22video_open\x22,\x22\
video_pause\x22,\x22vi\
deo_resume\x22,\x22vid\
eo_seek_to\x22,\x22vid\
eo_set_volume\x22,\x22\
view_get_camera\x22\
,\x22view_get_hport\
\x22,\x22view_get_surf\
ace_id\x22,\x22view_ge\
t_visible\x22,\x22view\
_get_wport\x22,\x22vie\
w_get_xport\x22,\x22vi\
ew_get_yport\x22,\x22v\
iew_set_camera\x22,\
\x22view_set_hport\x22\
,\x22view_set_surfa\
ce_id\x22,\x22view_set\
_visible\x22,\x22view_\
set_wport\x22,\x22view\
_set_xport\x22,\x22vie\
w_set_yport\x22,\x22vi\
rtual_key_add\x22,\x22\
virtual_key_dele\
te\x22,\x22virtual_key\
_hide\x22,\x22virtual_\
key_show\x22,\x22wallp\
aper_set_config\x22\
,\x22wallpaper_set_\
subscriptions\x22,\x22\
weak_ref_alive\x22,\
\x22weak_ref_any_al\
ive\x22,\x22weak_ref_c\
reate\x22,\x22window_c\
enter\x22,\x22window_d\
evice\x22,\x22window_e\
nable_borderless\
_fullscreen\x22,\x22wi\
ndow_get_borderl\
ess_fullscreen\x22,\
\x22window_get_capt\
ion\x22,\x22window_get\
_color\x22,\x22window_\
get_colour\x22,\x22win\
dow_get_cursor\x22,\
\x22window_get_full\
screen\x22,\x22window_\
get_height\x22,\x22win\
dow_get_showbord\
er\x22,\x22window_get_\
visible_rects\x22,\x22\
window_get_width\
\x22,\x22window_get_x\x22\
,\x22window_get_y\x22,\
\x22window_handle\x22,\
\x22window_has_focu\
s\x22,\x22window_mouse\
_get_delta_x\x22,\x22w\
indow_mouse_get_\
delta_y\x22,\x22window\
_mouse_get_locke\
d\x22,\x22window_mouse\
_get_x\x22,\x22window_\
mouse_get_y\x22,\x22wi\
ndow_mouse_set\x22,\
\x22window_mouse_se\
t_locked\x22,\x22windo\
w_set_caption\x22,\x22\
window_set_color\
\x22,\x22window_set_co\
lour\x22,\x22window_se\
t_cursor\x22,\x22windo\
w_set_fullscreen\
\x22,\x22window_set_ma\
x_height\x22,\x22windo\
w_set_max_width\x22\
,\x22window_set_min\
_height\x22,\x22window\
_set_min_width\x22,\
\x22window_set_posi\
tion\x22,\x22window_se\
t_rectangle\x22,\x22wi\
ndow_set_showbor\
der\x22,\x22window_set\
_size\x22,\x22window_v\
iew_mouse_get_x\x22\
,\x22window_view_mo\
use_get_y\x22,\x22wind\
ow_views_mouse_g\
et_x\x22,\x22window_vi\
ews_mouse_get_y\x22\
,\x22winphone_tile_\
background_color\
\x22,\x22winphone_tile\
_background_colo\
ur\x22,\x22zip_add_fil\
e\x22,\x22zip_create\x22,\
\x22zip_save\x22,\x22zip_\
unzip\x22,\x22zip_unzi\
p_async\x22],\x0asymbo\
l:[\x22AudioEffect\x22\
,\x22AudioEffectTyp\
e\x22,\x22AudioLFOType\
\x22,\x22GM_build_date\
\x22,\x22GM_build_type\
\x22,\x22GM_is_sandbox\
ed\x22,\x22GM_project_\
filename\x22,\x22GM_ru\
ntime_version\x22,\x22\
GM_version\x22,\x22NaN\
\x22,\x22_GMFILE_\x22,\x22_G\
MFUNCTION_\x22,\x22_GM\
LINE_\x22,\x22alignmen\
tH\x22,\x22alignmentV\x22\
,\x22all\x22,\x22animcurv\
etype_bezier\x22,\x22a\
nimcurvetype_cat\
mullrom\x22,\x22animcu\
rvetype_linear\x22,\
\x22asset_animation\
curve\x22,\x22asset_fo\
nt\x22,\x22asset_objec\
t\x22,\x22asset_path\x22,\
\x22asset_room\x22,\x22as\
set_script\x22,\x22ass\
et_sequence\x22,\x22as\
set_shader\x22,\x22ass\
et_sound\x22,\x22asset\
_sprite\x22,\x22asset_\
tiles\x22,\x22asset_ti\
meline\x22,\x22asset_u\
nknown\x22,\x22audio_3\
D\x22,\x22audio_bus_ma\
in\x22,\x22audio_fallo\
ff_exponent_dist\
ance\x22,\x22audio_fal\
loff_exponent_di\
stance_clamped\x22,\
\x22audio_falloff_e\
xponent_distance\
_scaled\x22,\x22audio_\
falloff_inverse_\
distance\x22,\x22audio\
_falloff_inverse\
_distance_clampe\
d\x22,\x22audio_fallof\
f_inverse_distan\
ce_scaled\x22,\x22audi\
o_falloff_linear\
_distance\x22,\x22audi\
o_falloff_linear\
_distance_clampe\
d\x22,\x22audio_fallof\
f_none\x22,\x22audio_m\
ono\x22,\x22audio_ster\
eo\x22,\x22bboxkind_di\
amond\x22,\x22bboxkind\
_ellipse\x22,\x22bboxk\
ind_precise\x22,\x22bb\
oxkind_rectangul\
ar\x22,\x22bboxmode_au\
tomatic\x22,\x22bboxmo\
de_fullimage\x22,\x22b\
boxmode_manual\x22,\
\x22bm_add\x22,\x22bm_des\
t_alpha\x22,\x22bm_des\
t_color\x22,\x22bm_des\
t_colour\x22,\x22bm_in\
v_dest_alpha\x22,\x22b\
m_inv_dest_color\
\x22,\x22bm_inv_dest_c\
olour\x22,\x22bm_inv_s\
rc_alpha\x22,\x22bm_in\
v_src_color\x22,\x22bm\
_inv_src_colour\x22\
,\x22bm_max\x22,\x22bm_no\
rmal\x22,\x22bm_one\x22,\x22\
bm_src_alpha\x22,\x22b\
m_src_alpha_sat\x22\
,\x22bm_src_color\x22,\
\x22bm_src_colour\x22,\
\x22bm_subtract\x22,\x22b\
m_zero\x22,\x22browser\
_chrome\x22,\x22browse\
r_edge\x22,\x22browser\
_firefox\x22,\x22brows\
er_ie\x22,\x22browser_\
ie_mobile\x22,\x22brow\
ser_not_a_browse\
r\x22,\x22browser_oper\
a\x22,\x22browser_safa\
ri\x22,\x22browser_saf\
ari_mobile\x22,\x22bro\
wser_tizen\x22,\x22bro\
wser_unknown\x22,\x22b\
rowser_windows_s\
tore\x22,\x22buffer_bo\
ol\x22,\x22buffer_f16\x22\
,\x22buffer_f32\x22,\x22b\
uffer_f64\x22,\x22buff\
er_fast\x22,\x22buffer\
_fixed\x22,\x22buffer_\
grow\x22,\x22buffer_s1\
6\x22,\x22buffer_s32\x22,\
\x22buffer_s8\x22,\x22buf\
fer_seek_end\x22,\x22b\
uffer_seek_relat\
ive\x22,\x22buffer_see\
k_start\x22,\x22buffer\
_string\x22,\x22buffer\
_text\x22,\x22buffer_u\
16\x22,\x22buffer_u32\x22\
,\x22buffer_u64\x22,\x22b\
uffer_u8\x22,\x22buffe\
r_vbuffer\x22,\x22buff\
er_wrap\x22,\x22c_aqua\
\x22,\x22c_black\x22,\x22c_b\
lue\x22,\x22c_dkgray\x22,\
\x22c_dkgrey\x22,\x22c_fu\
chsia\x22,\x22c_gray\x22,\
\x22c_green\x22,\x22c_gre\
y\x22,\x22c_lime\x22,\x22c_l\
tgray\x22,\x22c_ltgrey\
\x22,\x22c_maroon\x22,\x22c_\
navy\x22,\x22c_olive\x22,\
\x22c_orange\x22,\x22c_pu\
rple\x22,\x22c_red\x22,\x22c\
_silver\x22,\x22c_teal\
\x22,\x22c_white\x22,\x22c_y\
ellow\x22,\x22cache_di\
rectory\x22,\x22charac\
terSpacing\x22,\x22cmp\
func_always\x22,\x22cm\
pfunc_equal\x22,\x22cm\
pfunc_greater\x22,\x22\
cmpfunc_greatere\
qual\x22,\x22cmpfunc_l\
ess\x22,\x22cmpfunc_le\
ssequal\x22,\x22cmpfun\
c_never\x22,\x22cmpfun\
c_notequal\x22,\x22cor\
eColor\x22,\x22coreCol\
our\x22,\x22cr_appstar\
t\x22,\x22cr_arrow\x22,\x22c\
r_beam\x22,\x22cr_cros\
s\x22,\x22cr_default\x22,\
\x22cr_drag\x22,\x22cr_ha\
ndpoint\x22,\x22cr_hou\
rglass\x22,\x22cr_none\
\x22,\x22cr_size_all\x22,\
\x22cr_size_nesw\x22,\x22\
cr_size_ns\x22,\x22cr_\
size_nwse\x22,\x22cr_s\
ize_we\x22,\x22cr_upar\
row\x22,\x22cull_clock\
wise\x22,\x22cull_coun\
terclockwise\x22,\x22c\
ull_noculling\x22,\x22\
device_emulator\x22\
,\x22device_ios_ipa\
d\x22,\x22device_ios_i\
pad_retina\x22,\x22dev\
ice_ios_iphone\x22,\
\x22device_ios_ipho\
ne5\x22,\x22device_ios\
_iphone6\x22,\x22devic\
e_ios_iphone6plu\
s\x22,\x22device_ios_i\
phone_retina\x22,\x22d\
evice_ios_unknow\
n\x22,\x22device_table\
t\x22,\x22display_land\
scape\x22,\x22display_\
landscape_flippe\
d\x22,\x22display_port\
rait\x22,\x22display_p\
ortrait_flipped\x22\
,\x22dll_cdecl\x22,\x22dl\
l_stdcall\x22,\x22drop\
ShadowEnabled\x22,\x22\
dropShadowEnable\
d\x22,\x22ds_type_grid\
\x22,\x22ds_type_list\x22\
,\x22ds_type_map\x22,\x22\
ds_type_priority\
\x22,\x22ds_type_queue\
\x22,\x22ds_type_stack\
\x22,\x22ef_cloud\x22,\x22ef\
_ellipse\x22,\x22ef_ex\
plosion\x22,\x22ef_fir\
ework\x22,\x22ef_flare\
\x22,\x22ef_rain\x22,\x22ef_\
ring\x22,\x22ef_smoke\x22\
,\x22ef_smokeup\x22,\x22e\
f_snow\x22,\x22ef_spar\
k\x22,\x22ef_star\x22,\x22ef\
fectsEnabled\x22,\x22e\
ffectsEnabled\x22,\x22\
ev_alarm\x22,\x22ev_an\
imation_end\x22,\x22ev\
_animation_event\
\x22,\x22ev_animation_\
update\x22,\x22ev_asyn\
c_audio_playback\
\x22,\x22ev_async_audi\
o_playback_ended\
\x22,\x22ev_async_audi\
o_recording\x22,\x22ev\
_async_dialog\x22,\x22\
ev_async_push_no\
tification\x22,\x22ev_\
async_save_load\x22\
,\x22ev_async_save_\
load\x22,\x22ev_async_\
social\x22,\x22ev_asyn\
c_system_event\x22,\
\x22ev_async_web\x22,\x22\
ev_async_web_clo\
ud\x22,\x22ev_async_we\
b_iap\x22,\x22ev_async\
_web_image_load\x22\
,\x22ev_async_web_n\
etworking\x22,\x22ev_a\
sync_web_steam\x22,\
\x22ev_audio_playba\
ck\x22,\x22ev_audio_pl\
ayback_ended\x22,\x22e\
v_audio_recordin\
g\x22,\x22ev_boundary\x22\
,\x22ev_boundary_vi\
ew0\x22,\x22ev_boundar\
y_view1\x22,\x22ev_bou\
ndary_view2\x22,\x22ev\
_boundary_view3\x22\
,\x22ev_boundary_vi\
ew4\x22,\x22ev_boundar\
y_view5\x22,\x22ev_bou\
ndary_view6\x22,\x22ev\
_boundary_view7\x22\
,\x22ev_broadcast_m\
essage\x22,\x22ev_clea\
nup\x22,\x22ev_collisi\
on\x22,\x22ev_create\x22,\
\x22ev_destroy\x22,\x22ev\
_dialog_async\x22,\x22\
ev_draw\x22,\x22ev_dra\
w_begin\x22,\x22ev_dra\
w_end\x22,\x22ev_draw_\
normal\x22,\x22ev_draw\
_post\x22,\x22ev_draw_\
pre\x22,\x22ev_end_of_\
path\x22,\x22ev_game_e\
nd\x22,\x22ev_game_sta\
rt\x22,\x22ev_gesture\x22\
,\x22ev_gesture_dou\
ble_tap\x22,\x22ev_ges\
ture_drag_end\x22,\x22\
ev_gesture_drag_\
start\x22,\x22ev_gestu\
re_dragging\x22,\x22ev\
_gesture_flick\x22,\
\x22ev_gesture_pinc\
h_end\x22,\x22ev_gestu\
re_pinch_in\x22,\x22ev\
_gesture_pinch_o\
ut\x22,\x22ev_gesture_\
pinch_start\x22,\x22ev\
_gesture_rotate_\
end\x22,\x22ev_gesture\
_rotate_start\x22,\x22\
ev_gesture_rotat\
ing\x22,\x22ev_gesture\
_tap\x22,\x22ev_global\
_gesture_double_\
tap\x22,\x22ev_global_\
gesture_drag_end\
\x22,\x22ev_global_ges\
ture_drag_start\x22\
,\x22ev_global_gest\
ure_dragging\x22,\x22e\
v_global_gesture\
_flick\x22,\x22ev_glob\
al_gesture_pinch\
_end\x22,\x22ev_global\
_gesture_pinch_i\
n\x22,\x22ev_global_ge\
sture_pinch_out\x22\
,\x22ev_global_gest\
ure_pinch_start\x22\
,\x22ev_global_gest\
ure_rotate_end\x22,\
\x22ev_global_gestu\
re_rotate_start\x22\
,\x22ev_global_gest\
ure_rotating\x22,\x22e\
v_global_gesture\
_tap\x22,\x22ev_global\
_left_button\x22,\x22e\
v_global_left_pr\
ess\x22,\x22ev_global_\
left_release\x22,\x22e\
v_global_middle_\
button\x22,\x22ev_glob\
al_middle_press\x22\
,\x22ev_global_midd\
le_release\x22,\x22ev_\
global_right_but\
ton\x22,\x22ev_global_\
right_press\x22,\x22ev\
_global_right_re\
lease\x22,\x22ev_gui\x22,\
\x22ev_gui_begin\x22,\x22\
ev_gui_end\x22,\x22ev_\
joystick1_button\
1\x22,\x22ev_joystick1\
_button2\x22,\x22ev_jo\
ystick1_button3\x22\
,\x22ev_joystick1_b\
utton4\x22,\x22ev_joys\
tick1_button5\x22,\x22\
ev_joystick1_but\
ton6\x22,\x22ev_joysti\
ck1_button7\x22,\x22ev\
_joystick1_butto\
n8\x22,\x22ev_joystick\
1_down\x22,\x22ev_joys\
tick1_left\x22,\x22ev_\
joystick1_right\x22\
,\x22ev_joystick1_u\
p\x22,\x22ev_joystick2\
_button1\x22,\x22ev_jo\
ystick2_button2\x22\
,\x22ev_joystick2_b\
utton3\x22,\x22ev_joys\
tick2_button4\x22,\x22\
ev_joystick2_but\
ton5\x22,\x22ev_joysti\
ck2_button6\x22,\x22ev\
_joystick2_butto\
n7\x22,\x22ev_joystick\
2_button8\x22,\x22ev_j\
oystick2_down\x22,\x22\
ev_joystick2_lef\
t\x22,\x22ev_joystick2\
_right\x22,\x22ev_joys\
tick2_up\x22,\x22ev_ke\
yboard\x22,\x22ev_keyp\
ress\x22,\x22ev_keyrel\
ease\x22,\x22ev_left_b\
utton\x22,\x22ev_left_\
press\x22,\x22ev_left_\
release\x22,\x22ev_mid\
dle_button\x22,\x22ev_\
middle_press\x22,\x22e\
v_middle_release\
\x22,\x22ev_mouse\x22,\x22ev\
_mouse_enter\x22,\x22e\
v_mouse_leave\x22,\x22\
ev_mouse_wheel_d\
own\x22,\x22ev_mouse_w\
heel_up\x22,\x22ev_no_\
button\x22,\x22ev_no_m\
ore_health\x22,\x22ev_\
no_more_lives\x22,\x22\
ev_other\x22,\x22ev_ou\
tside\x22,\x22ev_outsi\
de_view0\x22,\x22ev_ou\
tside_view1\x22,\x22ev\
_outside_view2\x22,\
\x22ev_outside_view\
3\x22,\x22ev_outside_v\
iew4\x22,\x22ev_outsid\
e_view5\x22,\x22ev_out\
side_view6\x22,\x22ev_\
outside_view7\x22,\x22\
ev_pre_create\x22,\x22\
ev_push_notifica\
tion\x22,\x22ev_right_\
button\x22,\x22ev_righ\
t_press\x22,\x22ev_rig\
ht_release\x22,\x22ev_\
room_end\x22,\x22ev_ro\
om_start\x22,\x22ev_so\
cial\x22,\x22ev_step\x22,\
\x22ev_step_begin\x22,\
\x22ev_step_end\x22,\x22e\
v_step_normal\x22,\x22\
ev_system_event\x22\
,\x22ev_trigger\x22,\x22e\
v_user0\x22,\x22ev_use\
r1\x22,\x22ev_user10\x22,\
\x22ev_user11\x22,\x22ev_\
user12\x22,\x22ev_user\
13\x22,\x22ev_user14\x22,\
\x22ev_user15\x22,\x22ev_\
user2\x22,\x22ev_user3\
\x22,\x22ev_user4\x22,\x22ev\
_user5\x22,\x22ev_user\
6\x22,\x22ev_user7\x22,\x22e\
v_user8\x22,\x22ev_use\
r9\x22,\x22ev_web_asyn\
c\x22,\x22ev_web_cloud\
\x22,\x22ev_web_iap\x22,\x22\
ev_web_image_loa\
d\x22,\x22ev_web_netwo\
rking\x22,\x22ev_web_s\
ound_load\x22,\x22ev_w\
eb_steam\x22,\x22fa_ar\
chive\x22,\x22fa_botto\
m\x22,\x22fa_center\x22,\x22\
fa_directory\x22,\x22f\
a_hidden\x22,\x22fa_le\
ft\x22,\x22fa_middle\x22,\
\x22fa_none\x22,\x22fa_re\
adonly\x22,\x22fa_righ\
t\x22,\x22fa_sysfile\x22,\
\x22fa_top\x22,\x22fa_vol\
umeid\x22,\x22false\x22,\x22\
frameSizeX\x22,\x22fra\
meSizeY\x22,\x22gamesp\
eed_fps\x22,\x22gamesp\
eed_microseconds\
\x22,\x22global\x22,\x22glow\
Color\x22,\x22glowColo\
ur\x22,\x22glowEnabled\
\x22,\x22glowEnabled\x22,\
\x22glowEnd\x22,\x22glowS\
tart\x22,\x22gp_axis_a\
cceleration_x\x22,\x22\
gp_axis_accelera\
tion_y\x22,\x22gp_axis\
_acceleration_z\x22\
,\x22gp_axis_angula\
r_velocity_x\x22,\x22g\
p_axis_angular_v\
elocity_y\x22,\x22gp_a\
xis_angular_velo\
city_z\x22,\x22gp_axis\
_orientation_w\x22,\
\x22gp_axis_orienta\
tion_x\x22,\x22gp_axis\
_orientation_y\x22,\
\x22gp_axis_orienta\
tion_z\x22,\x22gp_axis\
lh\x22,\x22gp_axislv\x22,\
\x22gp_axisrh\x22,\x22gp_\
axisrv\x22,\x22gp_face\
1\x22,\x22gp_face2\x22,\x22g\
p_face3\x22,\x22gp_fac\
e4\x22,\x22gp_padd\x22,\x22g\
p_padl\x22,\x22gp_padr\
\x22,\x22gp_padu\x22,\x22gp_\
select\x22,\x22gp_shou\
lderl\x22,\x22gp_shoul\
derlb\x22,\x22gp_shoul\
derr\x22,\x22gp_should\
errb\x22,\x22gp_start\x22\
,\x22gp_stickl\x22,\x22gp\
_stickr\x22,\x22iap_av\
ailable\x22,\x22iap_ca\
nceled\x22,\x22iap_ev_\
consume\x22,\x22iap_ev\
_product\x22,\x22iap_e\
v_purchase\x22,\x22iap\
_ev_restore\x22,\x22ia\
p_ev_storeload\x22,\
\x22iap_failed\x22,\x22ia\
p_purchased\x22,\x22ia\
p_refunded\x22,\x22iap\
_status_availabl\
e\x22,\x22iap_status_l\
oading\x22,\x22iap_sta\
tus_processing\x22,\
\x22iap_status_rest\
oring\x22,\x22iap_stat\
us_unavailable\x22,\
\x22iap_status_unin\
itialised\x22,\x22iap_\
storeload_failed\
\x22,\x22iap_storeload\
_ok\x22,\x22iap_unavai\
lable\x22,\x22infinity\
\x22,\x22kbv_autocapit\
alize_characters\
\x22,\x22kbv_autocapit\
alize_none\x22,\x22kbv\
_autocapitalize_\
sentences\x22,\x22kbv_\
autocapitalize_w\
ords\x22,\x22kbv_retur\
nkey_continue\x22,\x22\
kbv_returnkey_de\
fault\x22,\x22kbv_retu\
rnkey_done\x22,\x22kbv\
_returnkey_emerg\
ency\x22,\x22kbv_retur\
nkey_go\x22,\x22kbv_re\
turnkey_google\x22,\
\x22kbv_returnkey_j\
oin\x22,\x22kbv_return\
key_next\x22,\x22kbv_r\
eturnkey_route\x22,\
\x22kbv_returnkey_s\
earch\x22,\x22kbv_retu\
rnkey_send\x22,\x22kbv\
_returnkey_yahoo\
\x22,\x22kbv_type_asci\
i\x22,\x22kbv_type_def\
ault\x22,\x22kbv_type_\
email\x22,\x22kbv_type\
_numbers\x22,\x22kbv_t\
ype_phone\x22,\x22kbv_\
type_phone_name\x22\
,\x22kbv_type_url\x22,\
\x22layerelementtyp\
e_background\x22,\x22l\
ayerelementtype_\
instance\x22,\x22layer\
elementtype_oldt\
ilemap\x22,\x22layerel\
ementtype_partic\
lesystem\x22,\x22layer\
elementtype_sequ\
ence\x22,\x22layerelem\
enttype_sprite\x22,\
\x22layerelementtyp\
e_tile\x22,\x22layerel\
ementtype_tilema\
p\x22,\x22layerelement\
type_undefined\x22,\
\x22leaderboard_typ\
e_number\x22,\x22leade\
rboard_type_time\
_mins_secs\x22,\x22lig\
httype_dir\x22,\x22lig\
httype_point\x22,\x22l\
ineSpacing\x22,\x22m_a\
xisx\x22,\x22m_axisx_g\
ui\x22,\x22m_axisy\x22,\x22m\
_axisy_gui\x22,\x22m_s\
croll_down\x22,\x22m_s\
croll_up\x22,\x22matri\
x_projection\x22,\x22m\
atrix_view\x22,\x22mat\
rix_world\x22,\x22mb_a\
ny\x22,\x22mb_left\x22,\x22m\
b_middle\x22,\x22mb_no\
ne\x22,\x22mb_right\x22,\x22\
mb_side1\x22,\x22mb_si\
de2\x22,\x22mip_marked\
only\x22,\x22mip_off\x22,\
\x22mip_on\x22,\x22networ\
k_config_avoid_t\
ime_wait\x22,\x22netwo\
rk_config_connec\
t_timeout\x22,\x22netw\
ork_config_disab\
le_multicast\x22,\x22n\
etwork_config_di\
sable_reliable_u\
dp\x22,\x22network_con\
fig_enable_multi\
cast\x22,\x22network_c\
onfig_enable_rel\
iable_udp\x22,\x22netw\
ork_config_use_n\
on_blocking_sock\
et\x22,\x22network_con\
fig_websocket_pr\
otocol\x22,\x22network\
_connect_active\x22\
,\x22network_connec\
t_blocking\x22,\x22net\
work_connect_non\
blocking\x22,\x22netwo\
rk_connect_none\x22\
,\x22network_connec\
t_passive\x22,\x22netw\
ork_send_binary\x22\
,\x22network_send_t\
ext\x22,\x22network_so\
cket_bluetooth\x22,\
\x22network_socket_\
tcp\x22,\x22network_so\
cket_udp\x22,\x22netwo\
rk_socket_ws\x22,\x22n\
etwork_socket_ws\
s\x22,\x22network_type\
_connect\x22,\x22netwo\
rk_type_data\x22,\x22n\
etwork_type_disc\
onnect\x22,\x22network\
_type_down\x22,\x22net\
work_type_non_bl\
ocking_connect\x22,\
\x22network_type_up\
\x22,\x22network_type_\
up_failed\x22,\x22nine\
slice_blank\x22,\x22ni\
neslice_bottom\x22,\
\x22nineslice_cente\
r\x22,\x22nineslice_ce\
ntre\x22,\x22nineslice\
_hide\x22,\x22nineslic\
e_left\x22,\x22ninesli\
ce_mirror\x22,\x22nine\
slice_repeat\x22,\x22n\
ineslice_right\x22,\
\x22nineslice_stret\
ch\x22,\x22nineslice_t\
op\x22,\x22noone\x22,\x22of_\
challenge_lose\x22,\
\x22of_challenge_ti\
e\x22,\x22of_challenge\
_win\x22,\x22os_androi\
d\x22,\x22os_gdk\x22,\x22os_\
gxgames\x22,\x22os_ios\
\x22,\x22os_linux\x22,\x22os\
_macosx\x22,\x22os_ope\
ragx\x22,\x22os_permis\
sion_denied\x22,\x22os\
_permission_deni\
ed_dont_request\x22\
,\x22os_permission_\
granted\x22,\x22os_ps3\
\x22,\x22os_ps4\x22,\x22os_p\
s5\x22,\x22os_psvita\x22,\
\x22os_switch\x22,\x22os_\
tvos\x22,\x22os_unknow\
n\x22,\x22os_uwp\x22,\x22os_\
win8native\x22,\x22os_\
windows\x22,\x22os_win\
phone\x22,\x22os_xboxo\
ne\x22,\x22os_xboxseri\
esxs\x22,\x22other\x22,\x22o\
utlineColor\x22,\x22ou\
tlineColour\x22,\x22ou\
tlineDist\x22,\x22outl\
ineEnabled\x22,\x22out\
lineEnabled\x22,\x22pa\
ragraphSpacing\x22,\
\x22path_action_con\
tinue\x22,\x22path_act\
ion_restart\x22,\x22pa\
th_action_revers\
e\x22,\x22path_action_\
stop\x22,\x22phy_debug\
_render_aabb\x22,\x22p\
hy_debug_render_\
collision_pairs\x22\
,\x22phy_debug_rend\
er_coms\x22,\x22phy_de\
bug_render_core_\
shapes\x22,\x22phy_deb\
ug_render_joints\
\x22,\x22phy_debug_ren\
der_obb\x22,\x22phy_de\
bug_render_shape\
s\x22,\x22phy_joint_an\
chor_1_x\x22,\x22phy_j\
oint_anchor_1_y\x22\
,\x22phy_joint_anch\
or_2_x\x22,\x22phy_joi\
nt_anchor_2_y\x22,\x22\
phy_joint_angle\x22\
,\x22phy_joint_angl\
e_limits\x22,\x22phy_j\
oint_damping_rat\
io\x22,\x22phy_joint_f\
requency\x22,\x22phy_j\
oint_length_1\x22,\x22\
phy_joint_length\
_2\x22,\x22phy_joint_l\
ower_angle_limit\
\x22,\x22phy_joint_max\
_force\x22,\x22phy_joi\
nt_max_length\x22,\x22\
phy_joint_max_mo\
tor_force\x22,\x22phy_\
joint_max_motor_\
torque\x22,\x22phy_joi\
nt_max_torque\x22,\x22\
phy_joint_motor_\
force\x22,\x22phy_join\
t_motor_speed\x22,\x22\
phy_joint_motor_\
torque\x22,\x22phy_joi\
nt_reaction_forc\
e_x\x22,\x22phy_joint_\
reaction_force_y\
\x22,\x22phy_joint_rea\
ction_torque\x22,\x22p\
hy_joint_speed\x22,\
\x22phy_joint_trans\
lation\x22,\x22phy_joi\
nt_upper_angle_l\
imit\x22,\x22phy_parti\
cle_data_flag_ca\
tegory\x22,\x22phy_par\
ticle_data_flag_\
color\x22,\x22phy_part\
icle_data_flag_c\
olour\x22,\x22phy_part\
icle_data_flag_p\
osition\x22,\x22phy_pa\
rticle_data_flag\
_typeflags\x22,\x22phy\
_particle_data_f\
lag_velocity\x22,\x22p\
hy_particle_flag\
_colormixing\x22,\x22p\
hy_particle_flag\
_colourmixing\x22,\x22\
phy_particle_fla\
g_elastic\x22,\x22phy_\
particle_flag_po\
wder\x22,\x22phy_parti\
cle_flag_spring\x22\
,\x22phy_particle_f\
lag_tensile\x22,\x22ph\
y_particle_flag_\
viscous\x22,\x22phy_pa\
rticle_flag_wall\
\x22,\x22phy_particle_\
flag_water\x22,\x22phy\
_particle_flag_z\
ombie\x22,\x22phy_part\
icle_group_flag_\
rigid\x22,\x22phy_part\
icle_group_flag_\
solid\x22,\x22pi\x22,\x22poi\
nter_invalid\x22,\x22p\
ointer_null\x22,\x22pr\
_linelist\x22,\x22pr_l\
inestrip\x22,\x22pr_po\
intlist\x22,\x22pr_tri\
anglefan\x22,\x22pr_tr\
ianglelist\x22,\x22pr_\
trianglestrip\x22,\x22\
ps_distr_gaussia\
n\x22,\x22ps_distr_inv\
gaussian\x22,\x22ps_di\
str_linear\x22,\x22ps_\
mode_burst\x22,\x22ps_\
mode_stream\x22,\x22ps\
_shape_diamond\x22,\
\x22ps_shape_ellips\
e\x22,\x22ps_shape_lin\
e\x22,\x22ps_shape_rec\
tangle\x22,\x22pt_shap\
e_circle\x22,\x22pt_sh\
ape_cloud\x22,\x22pt_s\
hape_disk\x22,\x22pt_s\
hape_explosion\x22,\
\x22pt_shape_flare\x22\
,\x22pt_shape_line\x22\
,\x22pt_shape_pixel\
\x22,\x22pt_shape_ring\
\x22,\x22pt_shape_smok\
e\x22,\x22pt_shape_sno\
w\x22,\x22pt_shape_spa\
rk\x22,\x22pt_shape_sp\
here\x22,\x22pt_shape_\
square\x22,\x22pt_shap\
e_star\x22,\x22rollbac\
k_chat_message\x22,\
\x22rollback_connec\
t_error\x22,\x22rollba\
ck_connect_info\x22\
,\x22rollback_conne\
cted_to_peer\x22,\x22r\
ollback_connecti\
on_rejected\x22,\x22ro\
llback_disconnec\
ted_from_peer\x22,\x22\
rollback_end_gam\
e\x22,\x22rollback_gam\
e_full\x22,\x22rollbac\
k_game_info\x22,\x22ro\
llback_game_inte\
rrupted\x22,\x22rollba\
ck_game_resumed\x22\
,\x22rollback_high_\
latency\x22,\x22rollba\
ck_player_prefs\x22\
,\x22rollback_proto\
col_rejected\x22,\x22r\
ollback_synchron\
ized_with_peer\x22,\
\x22rollback_synchr\
onizing_with_pee\
r\x22,\x22self\x22,\x22seqau\
diokey_loop\x22,\x22se\
qaudiokey_onesho\
t\x22,\x22seqdir_left\x22\
,\x22seqdir_right\x22,\
\x22seqinterpolatio\
n_assign\x22,\x22seqin\
terpolation_lerp\
\x22,\x22seqplay_loop\x22\
,\x22seqplay_onesho\
t\x22,\x22seqplay_ping\
pong\x22,\x22seqtextke\
y_bottom\x22,\x22seqte\
xtkey_center\x22,\x22s\
eqtextkey_justif\
y\x22,\x22seqtextkey_l\
eft\x22,\x22seqtextkey\
_middle\x22,\x22seqtex\
tkey_right\x22,\x22seq\
textkey_top\x22,\x22se\
qtracktype_audio\
\x22,\x22seqtracktype_\
bool\x22,\x22seqtrackt\
ype_clipmask\x22,\x22s\
eqtracktype_clip\
mask_mask\x22,\x22seqt\
racktype_clipmas\
k_subject\x22,\x22seqt\
racktype_color\x22,\
\x22seqtracktype_co\
lour\x22,\x22seqtrackt\
ype_empty\x22,\x22seqt\
racktype_graphic\
\x22,\x22seqtracktype_\
group\x22,\x22seqtrack\
type_instance\x22,\x22\
seqtracktype_mes\
sage\x22,\x22seqtrackt\
ype_moment\x22,\x22seq\
tracktype_partic\
lesystem\x22,\x22seqtr\
acktype_real\x22,\x22s\
eqtracktype_sequ\
ence\x22,\x22seqtrackt\
ype_spriteframes\
\x22,\x22seqtracktype_\
string\x22,\x22seqtrac\
ktype_text\x22,\x22sha\
dowColor\x22,\x22shado\
wColour\x22,\x22shadow\
OffsetX\x22,\x22shadow\
OffsetY\x22,\x22shadow\
Softness\x22,\x22sprit\
e_add_ext_error_\
cancelled\x22,\x22spri\
te_add_ext_error\
_decompressfaile\
d\x22,\x22sprite_add_e\
xt_error_loadfai\
led\x22,\x22sprite_add\
_ext_error_setup\
failed\x22,\x22sprite_\
add_ext_error_sp\
ritenotfound\x22,\x22s\
prite_add_ext_er\
ror_unknown\x22,\x22sp\
ritespeed_frames\
pergameframe\x22,\x22s\
pritespeed_frame\
spersecond\x22,\x22sur\
face_r16float\x22,\x22\
surface_r32float\
\x22,\x22surface_r8uno\
rm\x22,\x22surface_rg8\
unorm\x22,\x22surface_\
rgba16float\x22,\x22su\
rface_rgba32floa\
t\x22,\x22surface_rgba\
4unorm\x22,\x22surface\
_rgba8unorm\x22,\x22te\
xturegroup_statu\
s_fetched\x22,\x22text\
uregroup_status_\
loaded\x22,\x22texture\
group_status_loa\
ding\x22,\x22texturegr\
oup_status_unloa\
ded\x22,\x22tf_anisotr\
opic\x22,\x22tf_linear\
\x22,\x22tf_point\x22,\x22th\
ickness\x22,\x22tile_f\
lip\x22,\x22tile_index\
_mask\x22,\x22tile_mir\
ror\x22,\x22tile_rotat\
e\x22,\x22time_source_\
expire_after\x22,\x22t\
ime_source_expir\
e_nearest\x22,\x22time\
_source_game\x22,\x22t\
ime_source_globa\
l\x22,\x22time_source_\
state_active\x22,\x22t\
ime_source_state\
_initial\x22,\x22time_\
source_state_pau\
sed\x22,\x22time_sourc\
e_state_stopped\x22\
,\x22time_source_un\
its_frames\x22,\x22tim\
e_source_units_s\
econds\x22,\x22timezon\
e_local\x22,\x22timezo\
ne_utc\x22,\x22tm_coun\
tvsyncs\x22,\x22tm_sle\
ep\x22,\x22tm_systemti\
ming\x22,\x22true\x22,\x22ty\
_real\x22,\x22ty_strin\
g\x22,\x22undefined\x22,\x22\
vertex_type_colo\
r\x22,\x22vertex_type_\
colour\x22,\x22vertex_\
type_float1\x22,\x22ve\
rtex_type_float2\
\x22,\x22vertex_type_f\
loat3\x22,\x22vertex_t\
ype_float4\x22,\x22ver\
tex_type_ubyte4\x22\
,\x22vertex_usage_b\
inormal\x22,\x22vertex\
_usage_blendindi\
ces\x22,\x22vertex_usa\
ge_blendweight\x22,\
\x22vertex_usage_co\
lor\x22,\x22vertex_usa\
ge_colour\x22,\x22vert\
ex_usage_depth\x22,\
\x22vertex_usage_fo\
g\x22,\x22vertex_usage\
_normal\x22,\x22vertex\
_usage_position\x22\
,\x22vertex_usage_p\
size\x22,\x22vertex_us\
age_sample\x22,\x22ver\
tex_usage_tangen\
t\x22,\x22vertex_usage\
_texcoord\x22,\x22vide\
o_format_rgba\x22,\x22\
video_format_yuv\
\x22,\x22video_status_\
closed\x22,\x22video_s\
tatus_paused\x22,\x22v\
ideo_status_play\
ing\x22,\x22video_stat\
us_preparing\x22,\x22v\
k_add\x22,\x22vk_alt\x22,\
\x22vk_anykey\x22,\x22vk_\
backspace\x22,\x22vk_c\
ontrol\x22,\x22vk_deci\
mal\x22,\x22vk_delete\x22\
,\x22vk_divide\x22,\x22vk\
_down\x22,\x22vk_end\x22,\
\x22vk_enter\x22,\x22vk_e\
scape\x22,\x22vk_f1\x22,\x22\
vk_f10\x22,\x22vk_f11\x22\
,\x22vk_f12\x22,\x22vk_f2\
\x22,\x22vk_f3\x22,\x22vk_f4\
\x22,\x22vk_f5\x22,\x22vk_f6\
\x22,\x22vk_f7\x22,\x22vk_f8\
\x22,\x22vk_f9\x22,\x22vk_ho\
me\x22,\x22vk_insert\x22,\
\x22vk_lalt\x22,\x22vk_lc\
ontrol\x22,\x22vk_left\
\x22,\x22vk_lshift\x22,\x22v\
k_multiply\x22,\x22vk_\
nokey\x22,\x22vk_numpa\
d0\x22,\x22vk_numpad1\x22\
,\x22vk_numpad2\x22,\x22v\
k_numpad3\x22,\x22vk_n\
umpad4\x22,\x22vk_nump\
ad5\x22,\x22vk_numpad6\
\x22,\x22vk_numpad7\x22,\x22\
vk_numpad8\x22,\x22vk_\
numpad9\x22,\x22vk_pag\
edown\x22,\x22vk_pageu\
p\x22,\x22vk_pause\x22,\x22v\
k_printscreen\x22,\x22\
vk_ralt\x22,\x22vk_rco\
ntrol\x22,\x22vk_retur\
n\x22,\x22vk_right\x22,\x22v\
k_rshift\x22,\x22vk_sh\
ift\x22,\x22vk_space\x22,\
\x22vk_subtract\x22,\x22v\
k_tab\x22,\x22vk_up\x22,\x22\
wallpaper_config\
\x22,\x22wallpaper_sub\
scription_data\x22,\
\x22wrap\x22],\x0a\x22variab\
le.language\x22:[\x22a\
larm\x22,\x22applicati\
on_surface\x22,\x22arg\
ument\x22,\x22argument\
0\x22,\x22argument1\x22,\x22\
argument2\x22,\x22argu\
ment3\x22,\x22argument\
4\x22,\x22argument5\x22,\x22\
argument6\x22,\x22argu\
ment7\x22,\x22argument\
8\x22,\x22argument9\x22,\x22\
argument10\x22,\x22arg\
ument11\x22,\x22argume\
nt12\x22,\x22argument1\
3\x22,\x22argument14\x22,\
\x22argument15\x22,\x22ar\
gument_count\x22,\x22a\
sync_load\x22,\x22back\
ground_color\x22,\x22b\
ackground_colour\
\x22,\x22background_sh\
owcolor\x22,\x22backgr\
ound_showcolour\x22\
,\x22bbox_bottom\x22,\x22\
bbox_left\x22,\x22bbox\
_right\x22,\x22bbox_to\
p\x22,\x22browser_heig\
ht\x22,\x22browser_wid\
th\x22,\x22colour?Colo\
urTrack\x22,\x22curren\
t_day\x22,\x22current_\
hour\x22,\x22current_m\
inute\x22,\x22current_\
month\x22,\x22current_\
second\x22,\x22current\
_time\x22,\x22current_\
weekday\x22,\x22curren\
t_year\x22,\x22cursor_\
sprite\x22,\x22debug_m\
ode\x22,\x22delta_time\
\x22,\x22depth\x22,\x22direc\
tion\x22,\x22display_a\
a\x22,\x22drawn_by_seq\
uence\x22,\x22event_ac\
tion\x22,\x22event_dat\
a\x22,\x22event_number\
\x22,\x22event_object\x22\
,\x22event_type\x22,\x22f\
ont_texture_page\
_size\x22,\x22fps\x22,\x22fp\
s_real\x22,\x22frictio\
n\x22,\x22game_display\
_name\x22,\x22game_id\x22\
,\x22game_project_n\
ame\x22,\x22game_save_\
id\x22,\x22gravity\x22,\x22g\
ravity_direction\
\x22,\x22health\x22,\x22hspe\
ed\x22,\x22iap_data\x22,\x22\
id\x22,\x22image_alpha\
\x22,\x22image_angle\x22,\
\x22image_blend\x22,\x22i\
mage_index\x22,\x22ima\
ge_number\x22,\x22imag\
e_speed\x22,\x22image_\
xscale\x22,\x22image_y\
scale\x22,\x22in_colli\
sion_tree\x22,\x22in_s\
equence\x22,\x22instan\
ce_count\x22,\x22insta\
nce_id\x22,\x22keyboar\
d_key\x22,\x22keyboard\
_lastchar\x22,\x22keyb\
oard_lastkey\x22,\x22k\
eyboard_string\x22,\
\x22layer\x22,\x22lives\x22,\
\x22longMessage\x22,\x22m\
anaged\x22,\x22mask_in\
dex\x22,\x22message\x22,\x22\
mouse_button\x22,\x22m\
ouse_lastbutton\x22\
,\x22mouse_x\x22,\x22mous\
e_y\x22,\x22object_ind\
ex\x22,\x22os_browser\x22\
,\x22os_device\x22,\x22os\
_type\x22,\x22os_versi\
on\x22,\x22path_endact\
ion\x22,\x22path_index\
\x22,\x22path_orientat\
ion\x22,\x22path_posit\
ion\x22,\x22path_posit\
ionprevious\x22,\x22pa\
th_scale\x22,\x22path_\
speed\x22,\x22persiste\
nt\x22,\x22phy_active\x22\
,\x22phy_angular_da\
mping\x22,\x22phy_angu\
lar_velocity\x22,\x22p\
hy_bullet\x22,\x22phy_\
col_normal_x\x22,\x22p\
hy_col_normal_y\x22\
,\x22phy_collision_\
points\x22,\x22phy_col\
lision_x\x22,\x22phy_c\
ollision_y\x22,\x22phy\
_com_x\x22,\x22phy_com\
_y\x22,\x22phy_dynamic\
\x22,\x22phy_fixed_rot\
ation\x22,\x22phy_iner\
tia\x22,\x22phy_kinema\
tic\x22,\x22phy_linear\
_damping\x22,\x22phy_l\
inear_velocity_x\
\x22,\x22phy_linear_ve\
locity_y\x22,\x22phy_m\
ass\x22,\x22phy_positi\
on_x\x22,\x22phy_posit\
ion_xprevious\x22,\x22\
phy_position_y\x22,\
\x22phy_position_yp\
revious\x22,\x22phy_ro\
tation\x22,\x22phy_sle\
eping\x22,\x22phy_spee\
d\x22,\x22phy_speed_x\x22\
,\x22phy_speed_y\x22,\x22\
player_avatar_sp\
rite\x22,\x22player_av\
atar_url\x22,\x22playe\
r_id\x22,\x22player_lo\
cal\x22,\x22player_typ\
e\x22,\x22player_user_\
id\x22,\x22program_dir\
ectory\x22,\x22rollbac\
k_api_server\x22,\x22r\
ollback_confirme\
d_frame\x22,\x22rollba\
ck_current_frame\
\x22,\x22rollback_even\
t_id\x22,\x22rollback_\
event_param\x22,\x22ro\
llback_game_runn\
ing\x22,\x22room\x22,\x22roo\
m_first\x22,\x22room_h\
eight\x22,\x22room_las\
t\x22,\x22room_persist\
ent\x22,\x22room_speed\
\x22,\x22room_width\x22,\x22\
score\x22,\x22script\x22,\
\x22sequence_instan\
ce\x22,\x22solid\x22,\x22spe\
ed\x22,\x22sprite_heig\
ht\x22,\x22sprite_inde\
x\x22,\x22sprite_width\
\x22,\x22sprite_xoffse\
t\x22,\x22sprite_yoffs\
et\x22,\x22stacktrace\x22\
,\x22temp_directory\
\x22,\x22timeline_inde\
x\x22,\x22timeline_loo\
p\x22,\x22timeline_pos\
ition\x22,\x22timeline\
_running\x22,\x22timel\
ine_speed\x22,\x22view\
_camera\x22,\x22view_c\
urrent\x22,\x22view_en\
abled\x22,\x22view_hpo\
rt\x22,\x22view_surfac\
e_id\x22,\x22view_visi\
ble\x22,\x22view_wport\
\x22,\x22view_xport\x22,\x22\
view_yport\x22,\x22vis\
ible\x22,\x22vspeed\x22,\x22\
webgl_enabled\x22,\x22\
working_director\
y\x22,\x22x\x22,\x22xpreviou\
s\x22,\x22xstart\x22,\x22y\x22,\
\x22yprevious\x22,\x22yst\
art\x22]\x0a},\x0acontain\
s:[e.C_LINE_COMM\
ENT_MODE,e.C_BLO\
CK_COMMENT_MODE,\
e.APOS_STRING_MO\
DE,e.QUOTE_STRIN\
G_MODE,e.C_NUMBE\
R_MODE]\x0a})})();h\
ljs.registerLang\
uage(\x22gml\x22,e)})(\
);/*! `go` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{const a={\x0akeyw\
ord:[\x22break\x22,\x22ca\
se\x22,\x22chan\x22,\x22cons\
t\x22,\x22continue\x22,\x22d\
efault\x22,\x22defer\x22,\
\x22else\x22,\x22fallthro\
ugh\x22,\x22for\x22,\x22func\
\x22,\x22go\x22,\x22goto\x22,\x22i\
f\x22,\x22import\x22,\x22int\
erface\x22,\x22map\x22,\x22p\
ackage\x22,\x22range\x22,\
\x22return\x22,\x22select\
\x22,\x22struct\x22,\x22swit\
ch\x22,\x22type\x22,\x22var\x22\
],\x0atype:[\x22bool\x22,\
\x22byte\x22,\x22complex6\
4\x22,\x22complex128\x22,\
\x22error\x22,\x22float32\
\x22,\x22float64\x22,\x22int\
8\x22,\x22int16\x22,\x22int3\
2\x22,\x22int64\x22,\x22stri\
ng\x22,\x22uint8\x22,\x22uin\
t16\x22,\x22uint32\x22,\x22u\
int64\x22,\x22int\x22,\x22ui\
nt\x22,\x22uintptr\x22,\x22r\
une\x22],\x0aliteral:[\
\x22true\x22,\x22false\x22,\x22\
iota\x22,\x22nil\x22],\x0abu\
ilt_in:[\x22append\x22\
,\x22cap\x22,\x22close\x22,\x22\
complex\x22,\x22copy\x22,\
\x22imag\x22,\x22len\x22,\x22ma\
ke\x22,\x22new\x22,\x22panic\
\x22,\x22print\x22,\x22print\
ln\x22,\x22real\x22,\x22reco\
ver\x22,\x22delete\x22]\x0a}\
;return{name:\x22Go\
\x22,aliases:[\x22gola\
ng\x22],keywords:a,\
illegal:\x22</\x22,\x0aco\
ntains:[e.C_LINE\
_COMMENT_MODE,e.\
C_BLOCK_COMMENT_\
MODE,{className:\
\x22string\x22,\x0avarian\
ts:[e.QUOTE_STRI\
NG_MODE,e.APOS_S\
TRING_MODE,{begi\
n:\x22`\x22,end:\x22`\x22}]}\
,{\x0aclassName:\x22nu\
mber\x22,variants:[\
{\x0amatch:/-?\x5cb0[x\
X]\x5c.[a-fA-F0-9](\
_?[a-fA-F0-9])*[\
pP][+-]?\x5cd(_?\x5cd)\
*i?/,relevance:0\
\x0a},{\x0amatch:/-?\x5cb\
0[xX](_?[a-fA-F0\
-9])+((\x5c.([a-fA-\
F0-9](_?[a-fA-F0\
-9])*)?)?[pP][+-\
]?\x5cd(_?\x5cd)*)?i?/\
,\x0arelevance:0},{\
match:/-?\x5cb0[oO]\
(_?[0-7])*i?/,re\
levance:0},{\x0amat\
ch:/-?\x5c.\x5cd(_?\x5cd)\
*([eE][+-]?\x5cd(_?\
\x5cd)*)?i?/,releva\
nce:0},{\x0amatch:/\
-?\x5cb\x5cd(_?\x5cd)*(\x5c.\
(\x5cd(_?\x5cd)*)?)?([\
eE][+-]?\x5cd(_?\x5cd)\
*)?i?/,relevance\
:0}]},{\x0abegin:/:\
=/},{className:\x22\
function\x22,beginK\
eywords:\x22func\x22,e\
nd:\x22\x5c\x5cs*(\x5c\x5c{|$)\x22\
,\x0aexcludeEnd:!0,\
contains:[e.TITL\
E_MODE,{classNam\
e:\x22params\x22,begin\
:/\x5c(/,end:/\x5c)/,\x0a\
endsParent:!0,ke\
ywords:a,illegal\
:/[\x22']/}]}]}}})(\
);hljs.registerL\
anguage(\x22go\x22,e)\x0a\
})();/*! `golo` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>({name:\x22Go\
lo\x22,keywords:{\x0ak\
eyword:[\x22println\
\x22,\x22readln\x22,\x22prin\
t\x22,\x22import\x22,\x22mod\
ule\x22,\x22function\x22,\
\x22local\x22,\x22return\x22\
,\x22let\x22,\x22var\x22,\x22wh\
ile\x22,\x22for\x22,\x22fore\
ach\x22,\x22times\x22,\x22in\
\x22,\x22case\x22,\x22when\x22,\
\x22match\x22,\x22with\x22,\x22\
break\x22,\x22continue\
\x22,\x22augment\x22,\x22aug\
mentation\x22,\x22each\
\x22,\x22find\x22,\x22filter\
\x22,\x22reduce\x22,\x22if\x22,\
\x22then\x22,\x22else\x22,\x22o\
therwise\x22,\x22try\x22,\
\x22catch\x22,\x22finally\
\x22,\x22raise\x22,\x22throw\
\x22,\x22orIfNull\x22,\x22Dy\
namicObject|10\x22,\
\x22DynamicVariable\
\x22,\x22struct\x22,\x22Obse\
rvable\x22,\x22map\x22,\x22s\
et\x22,\x22vector\x22,\x22li\
st\x22,\x22array\x22],\x0ali\
teral:[\x22true\x22,\x22f\
alse\x22,\x22null\x22]},\x0a\
contains:[e.HASH\
_COMMENT_MODE,e.\
QUOTE_STRING_MOD\
E,e.C_NUMBER_MOD\
E,{\x0aclassName:\x22m\
eta\x22,begin:\x22@[A-\
Za-z]+\x22}]})})();\
hljs.registerLan\
guage(\x22golo\x22,e)}\
)();/*! `gradle`\
 grammar compile\
d for Highlight.\
js 11.10.0 */\x0a((\
)=>{var e=(()=>{\
\x22use strict\x22;ret\
urn e=>({name:\x22G\
radle\x22,case_inse\
nsitive:!0,\x0akeyw\
ords:[\x22task\x22,\x22pr\
oject\x22,\x22allproje\
cts\x22,\x22subproject\
s\x22,\x22artifacts\x22,\x22\
buildscript\x22,\x22co\
nfigurations\x22,\x22d\
ependencies\x22,\x22re\
positories\x22,\x22sou\
rceSets\x22,\x22descri\
ption\x22,\x22delete\x22,\
\x22from\x22,\x22into\x22,\x22i\
nclude\x22,\x22exclude\
\x22,\x22source\x22,\x22clas\
spath\x22,\x22destinat\
ionDir\x22,\x22include\
s\x22,\x22options\x22,\x22so\
urceCompatibilit\
y\x22,\x22targetCompat\
ibility\x22,\x22group\x22\
,\x22flatDir\x22,\x22doLa\
st\x22,\x22doFirst\x22,\x22f\
latten\x22,\x22todir\x22,\
\x22fromdir\x22,\x22ant\x22,\
\x22def\x22,\x22abstract\x22\
,\x22break\x22,\x22case\x22,\
\x22catch\x22,\x22continu\
e\x22,\x22default\x22,\x22do\
\x22,\x22else\x22,\x22extend\
s\x22,\x22final\x22,\x22fina\
lly\x22,\x22for\x22,\x22if\x22,\
\x22implements\x22,\x22in\
stanceof\x22,\x22nativ\
e\x22,\x22new\x22,\x22privat\
e\x22,\x22protected\x22,\x22\
public\x22,\x22return\x22\
,\x22static\x22,\x22switc\
h\x22,\x22synchronized\
\x22,\x22throw\x22,\x22throw\
s\x22,\x22transient\x22,\x22\
try\x22,\x22volatile\x22,\
\x22while\x22,\x22strictf\
p\x22,\x22package\x22,\x22im\
port\x22,\x22false\x22,\x22n\
ull\x22,\x22super\x22,\x22th\
is\x22,\x22true\x22,\x22antl\
rtask\x22,\x22checksty\
le\x22,\x22codenarc\x22,\x22\
copy\x22,\x22boolean\x22,\
\x22byte\x22,\x22char\x22,\x22c\
lass\x22,\x22double\x22,\x22\
float\x22,\x22int\x22,\x22in\
terface\x22,\x22long\x22,\
\x22short\x22,\x22void\x22,\x22\
compile\x22,\x22runTim\
e\x22,\x22file\x22,\x22fileT\
ree\x22,\x22abs\x22,\x22any\x22\
,\x22append\x22,\x22asLis\
t\x22,\x22asWritable\x22,\
\x22call\x22,\x22collect\x22\
,\x22compareTo\x22,\x22co\
unt\x22,\x22div\x22,\x22dump\
\x22,\x22each\x22,\x22eachBy\
te\x22,\x22eachFile\x22,\x22\
eachLine\x22,\x22every\
\x22,\x22find\x22,\x22findAl\
l\x22,\x22flatten\x22,\x22ge\
tAt\x22,\x22getErr\x22,\x22g\
etIn\x22,\x22getOut\x22,\x22\
getText\x22,\x22grep\x22,\
\x22immutable\x22,\x22inj\
ect\x22,\x22inspect\x22,\x22\
intersect\x22,\x22invo\
keMethods\x22,\x22isCa\
se\x22,\x22join\x22,\x22left\
Shift\x22,\x22minus\x22,\x22\
multiply\x22,\x22newIn\
putStream\x22,\x22newO\
utputStream\x22,\x22ne\
wPrintWriter\x22,\x22n\
ewReader\x22,\x22newWr\
iter\x22,\x22next\x22,\x22pl\
us\x22,\x22pop\x22,\x22power\
\x22,\x22previous\x22,\x22pr\
int\x22,\x22println\x22,\x22\
push\x22,\x22putAt\x22,\x22r\
ead\x22,\x22readBytes\x22\
,\x22readLines\x22,\x22re\
verse\x22,\x22reverseE\
ach\x22,\x22round\x22,\x22si\
ze\x22,\x22sort\x22,\x22spli\
tEachLine\x22,\x22step\
\x22,\x22subMap\x22,\x22time\
s\x22,\x22toInteger\x22,\x22\
toList\x22,\x22tokeniz\
e\x22,\x22upto\x22,\x22waitF\
orOrKill\x22,\x22withP\
rintWriter\x22,\x22wit\
hReader\x22,\x22withSt\
ream\x22,\x22withWrite\
r\x22,\x22withWriterAp\
pend\x22,\x22write\x22,\x22w\
riteLine\x22],\x0acont\
ains:[e.C_LINE_C\
OMMENT_MODE,e.C_\
BLOCK_COMMENT_MO\
DE,e.APOS_STRING\
_MODE,e.QUOTE_ST\
RING_MODE,e.NUMB\
ER_MODE,e.REGEXP\
_MODE]\x0a})})();hl\
js.registerLangu\
age(\x22gradle\x22,e)}\
)();/*! `graphql\
` grammar compil\
ed for Highlight\
.js 11.10.0 */\x0a(\
()=>{var e=(()=>\
{\x22use strict\x22;re\
turn e=>{const a\
=e.regex;return{\
name:\x22GraphQL\x22,\x0a\
aliases:[\x22gql\x22],\
case_insensitive\
:!0,disableAutod\
etect:!1,keyword\
s:{\x0akeyword:[\x22qu\
ery\x22,\x22mutation\x22,\
\x22subscription\x22,\x22\
type\x22,\x22input\x22,\x22s\
chema\x22,\x22directiv\
e\x22,\x22interface\x22,\x22\
union\x22,\x22scalar\x22,\
\x22fragment\x22,\x22enum\
\x22,\x22on\x22],\x0aliteral\
:[\x22true\x22,\x22false\x22\
,\x22null\x22]},\x0aconta\
ins:[e.HASH_COMM\
ENT_MODE,e.QUOTE\
_STRING_MODE,e.N\
UMBER_MODE,{\x0asco\
pe:\x22punctuation\x22\
,match:/[.]{3}/,\
relevance:0},{sc\
ope:\x22punctuation\
\x22,\x0abegin:/[\x5c!\x5c(\x5c\
)\x5c:\x5c=\x5c[\x5c]\x5c{\x5c|\x5c}]\
{1}/,relevance:0\
},{scope:\x22variab\
le\x22,begin:/\x5c$/,\x0a\
end:/\x5cW/,exclude\
End:!0,relevance\
:0},{scope:\x22meta\
\x22,match:/@\x5cw+/,e\
xcludeEnd:!0},{\x0a\
scope:\x22symbol\x22,b\
egin:a.concat(/[\
_A-Za-z][_0-9A-Z\
a-z]*/,a.lookahe\
ad(/\x5cs*:/)),\x0arel\
evance:0}],illeg\
al:[/[;<']/,/BEG\
IN/]}}})();hljs.\
registerLanguage\
(\x22graphql\x22,e)\x0a})\
();/*! `groovy` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;func\
tion e(e,a={}){r\
eturn a.variants\
=e,a}\x0areturn a=>\
{\x0aconst n=a.rege\
x,t=\x22[A-Za-z0-9_\
$]+\x22,r=e([a.C_LI\
NE_COMMENT_MODE,\
a.C_BLOCK_COMMEN\
T_MODE,a.COMMENT\
(\x22/\x5c\x5c*\x5c\x5c*\x22,\x22\x5c\x5c*/\
\x22,{\x0arelevance:0,\
contains:[{begin\
:/\x5cw+@/,relevanc\
e:0},{className:\
\x22doctag\x22,\x0abegin:\
\x22@[A-Za-z]+\x22}]})\
]),s={className:\
\x22regexp\x22,begin:/\
~?\x5c/[^\x5c/\x5cn]+\x5c//,\
\x0acontains:[a.BAC\
KSLASH_ESCAPE]\x0a}\
,i=e([a.BINARY_N\
UMBER_MODE,a.C_N\
UMBER_MODE]),l=e\
([{begin:/\x22\x22\x22/,e\
nd:/\x22\x22\x22/},{\x0abegi\
n:/'''/,end:/'''\
/},{begin:\x22\x5c\x5c$/\x22\
,end:\x22/\x5c\x5c$\x22,rele\
vance:10\x0a},a.APO\
S_STRING_MODE,a.\
QUOTE_STRING_MOD\
E],{className:\x22s\
tring\x22}),c={\x0amat\
ch:[/(class|inte\
rface|trait|enum\
|record|extends|\
implements)/,/\x5cs\
+/,a.UNDERSCORE_\
IDENT_RE],\x0ascope\
:{1:\x22keyword\x22,3:\
\x22title.class\x22}};\
return{name:\x22Gro\
ovy\x22,keywords:{\x0a\
\x22variable.langua\
ge\x22:\x22this super\x22\
,literal:\x22true f\
alse null\x22,\x0atype\
:[\x22byte\x22,\x22short\x22\
,\x22char\x22,\x22int\x22,\x22l\
ong\x22,\x22boolean\x22,\x22\
float\x22,\x22double\x22,\
\x22void\x22],\x0akeyword\
:[\x22def\x22,\x22as\x22,\x22in\
\x22,\x22assert\x22,\x22trai\
t\x22,\x22abstract\x22,\x22s\
tatic\x22,\x22volatile\
\x22,\x22transient\x22,\x22p\
ublic\x22,\x22private\x22\
,\x22protected\x22,\x22sy\
nchronized\x22,\x22fin\
al\x22,\x22class\x22,\x22int\
erface\x22,\x22enum\x22,\x22\
if\x22,\x22else\x22,\x22for\x22\
,\x22while\x22,\x22switch\
\x22,\x22case\x22,\x22break\x22\
,\x22default\x22,\x22cont\
inue\x22,\x22throw\x22,\x22t\
hrows\x22,\x22try\x22,\x22ca\
tch\x22,\x22finally\x22,\x22\
implements\x22,\x22ext\
ends\x22,\x22new\x22,\x22imp\
ort\x22,\x22package\x22,\x22\
return\x22,\x22instanc\
eof\x22,\x22var\x22]\x0a},co\
ntains:[a.SHEBAN\
G({binary:\x22groov\
y\x22,relevance:10}\
),r,l,s,i,c,{\x0acl\
assName:\x22meta\x22,b\
egin:\x22@[A-Za-z]+\
\x22,relevance:0},{\
className:\x22attr\x22\
,\x0abegin:t+\x22[ \x5ct]\
*:\x22,relevance:0}\
,{begin:/\x5c?/,end\
:/:/,relevance:0\
,\x0acontains:[r,l,\
s,i,\x22self\x22]},{cl\
assName:\x22symbol\x22\
,\x0abegin:\x22^[ \x5ct]*\
\x22+n.lookahead(t+\
\x22:\x22),excludeBegi\
n:!0,end:t+\x22:\x22,r\
elevance:0}],\x0ail\
legal:/#|<\x5c//}}}\
)();hljs.registe\
rLanguage(\x22groov\
y\x22,e)})();/*! `h\
aml` grammar com\
piled for Highli\
ght.js 11.10.0 *\
/\x0a(()=>{var e=((\
)=>{\x22use strict\x22\
;return e=>({nam\
e:\x22HAML\x22,case_in\
sensitive:!0,\x0aco\
ntains:[{classNa\
me:\x22meta\x22,\x0abegin\
:\x22^!!!( (5|1\x5c\x5c.1\
|Strict|Frameset\
|Basic|Mobile|RD\
Fa|XML\x5c\x5cb.*))?$\x22\
,\x0arelevance:10},\
e.COMMENT(\x22^\x5c\x5cs*\
(!=#|=#|-#|/).*$\
\x22,null,{relevanc\
e:0}),{\x0abegin:\x22^\
\x5c\x5cs*(-|=|!=)(?!#\
)\x22,end:/$/,subLa\
nguage:\x22ruby\x22,ex\
cludeBegin:!0,\x0ae\
xcludeEnd:!0},{c\
lassName:\x22tag\x22,b\
egin:\x22^\x5c\x5cs*%\x22,co\
ntains:[{\x0aclassN\
ame:\x22selector-ta\
g\x22,begin:\x22\x5c\x5cw+\x22}\
,{className:\x22sel\
ector-id\x22,begin:\
\x22#[\x5c\x5cw-]+\x22\x0a},{cl\
assName:\x22selecto\
r-class\x22,begin:\x22\
\x5c\x5c.[\x5c\x5cw-]+\x22},{be\
gin:/\x5c{\x5cs*/,end:\
/\x5cs*\x5c}/,\x0acontain\
s:[{begin:\x22:\x5c\x5cw+\
\x5c\x5cs*=>\x22,end:\x22,\x5c\x5c\
s+\x22,returnBegin:\
!0,endsWithParen\
t:!0,\x0acontains:[\
{className:\x22attr\
\x22,begin:\x22:\x5c\x5cw+\x22\x0a\
},e.APOS_STRING_\
MODE,e.QUOTE_STR\
ING_MODE,{begin:\
\x22\x5c\x5cw+\x22,relevance\
:0}]}]},{\x0abegin:\
\x22\x5c\x5c(\x5c\x5cs*\x22,end:\x22\x5c\
\x5cs*\x5c\x5c)\x22,excludeE\
nd:!0,contains:[\
{begin:\x22\x5c\x5cw+\x5c\x5cs*\
=\x22,\x0aend:\x22\x5c\x5cs+\x22,r\
eturnBegin:!0,en\
dsWithParent:!0,\
contains:[{class\
Name:\x22attr\x22,\x0abeg\
in:\x22\x5c\x5cw+\x22,releva\
nce:0},e.APOS_ST\
RING_MODE,e.QUOT\
E_STRING_MODE,{b\
egin:\x22\x5c\x5cw+\x22,\x0arel\
evance:0}]}]}]},\
{begin:\x22^\x5c\x5cs*[=~\
]\x5c\x5cs*\x22},{begin:/\
#\x5c{/,end:/\x5c}/,\x0as\
ubLanguage:\x22ruby\
\x22,excludeBegin:!\
0,excludeEnd:!0}\
]})})()\x0a;hljs.re\
gisterLanguage(\x22\
haml\x22,e)})();/*!\
 `handlebars` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>{const a=e.r\
egex,n={\x0a$patter\
n:/[\x5cw.\x5c/]+/,\x0abu\
ilt_in:[\x22action\x22\
,\x22bindattr\x22,\x22col\
lection\x22,\x22compon\
ent\x22,\x22concat\x22,\x22d\
ebugger\x22,\x22each\x22,\
\x22each-in\x22,\x22get\x22,\
\x22hash\x22,\x22if\x22,\x22in\x22\
,\x22input\x22,\x22link-t\
o\x22,\x22loc\x22,\x22log\x22,\x22\
lookup\x22,\x22mut\x22,\x22o\
utlet\x22,\x22partial\x22\
,\x22query-params\x22,\
\x22render\x22,\x22templa\
te\x22,\x22textarea\x22,\x22\
unbound\x22,\x22unless\
\x22,\x22view\x22,\x22with\x22,\
\x22yield\x22]\x0a},t=/\x5c[\
\x5c]|\x5c[[^\x5c]]+\x5c]/,s\
=/[^\x5cs!\x22#%&'()*+\
,.\x5c/;<=>@\x5c[\x5c\x5c\x5c]^\
`{|}~]+/,i=a.eit\
her(/\x22\x22|\x22[^\x22]+\x22/\
,/''|'[^']+'/,t,\
s),r=a.concat(a.\
optional(/\x5c.|\x5c.\x5c\
/|\x5c//),i,a.anyNu\
mberOfTimes(a.co\
ncat(/(\x5c.|\x5c/)/,i\
))),l=a.concat(\x22\
(\x22,t,\x22|\x22,s,\x22)(?=\
=)\x22),c={\x0abegin:r\
},o=e.inherit(c,\
{keywords:{$patt\
ern:/[\x5cw.\x5c/]+/,\x0a\
literal:[\x22true\x22,\
\x22false\x22,\x22undefin\
ed\x22,\x22null\x22]}}),m\
={begin:/\x5c(/,end\
:/\x5c)/},d={\x0aclass\
Name:\x22attr\x22,begi\
n:l,relevance:0,\
starts:{begin:/=\
/,end:/=/,starts\
:{\x0acontains:[e.N\
UMBER_MODE,e.QUO\
TE_STRING_MODE,e\
.APOS_STRING_MOD\
E,o,m]}}},g={\x0aco\
ntains:[e.NUMBER\
_MODE,e.QUOTE_ST\
RING_MODE,e.APOS\
_STRING_MODE,{be\
gin:/as\x5cs+\x5c|/,\x0ak\
eywords:{keyword\
:\x22as\x22},end:/\x5c|/,\
contains:[{begin\
:/\x5cw+/}]},d,o,m]\
,returnEnd:!0\x0a},\
b=e.inherit(c,{c\
lassName:\x22name\x22,\
keywords:n,start\
s:e.inherit(g,{e\
nd:/\x5c)/})})\x0a;m.c\
ontains=[b];cons\
t u=e.inherit(c,\
{keywords:n,clas\
sName:\x22name\x22,\x0ast\
arts:e.inherit(g\
,{end:/\x5c}\x5c}/})})\
,h=e.inherit(c,{\
keywords:n,class\
Name:\x22name\x22\x0a}),N\
=e.inherit(c,{cl\
assName:\x22name\x22,k\
eywords:n,starts\
:e.inherit(g,{en\
d:/\x5c}\x5c}/})})\x0a;re\
turn{name:\x22Handl\
ebars\x22,\x0aaliases:\
[\x22hbs\x22,\x22html.hbs\
\x22,\x22html.handleba\
rs\x22,\x22htmlbars\x22],\
case_insensitive\
:!0,\x0asubLanguage\
:\x22xml\x22,contains:\
[{begin:/\x5c\x5c\x5c{\x5c{/\
,skip:!0},{begin\
:/\x5c\x5c\x5c\x5c(?=\x5c{\x5c{)/,\
\x0askip:!0},e.COMM\
ENT(/\x5c{\x5c{!--/,/-\
-\x5c}\x5c}/),e.COMMEN\
T(/\x5c{\x5c{!/,/\x5c}\x5c}/\
),{\x0aclassName:\x22t\
emplate-tag\x22,beg\
in:/\x5c{\x5c{\x5c{\x5c{(?!\x5c\
/)/,end:/\x5c}\x5c}\x5c}\x5c\
}/,contains:[u],\
\x0astarts:{end:/\x5c{\
\x5c{\x5c{\x5c{\x5c//,return\
End:!0,subLangua\
ge:\x22xml\x22}},{\x0acla\
ssName:\x22template\
-tag\x22,begin:/\x5c{\x5c\
{\x5c{\x5c{\x5c//,end:/\x5c}\
\x5c}\x5c}\x5c}/,contains\
:[h]},{\x0aclassNam\
e:\x22template-tag\x22\
,begin:/\x5c{\x5c{#/,e\
nd:/\x5c}\x5c}/,contai\
ns:[u]},{\x0aclassN\
ame:\x22template-ta\
g\x22,begin:/\x5c{\x5c{(?\
=else\x5c}\x5c})/,end:\
/\x5c}\x5c}/,keywords:\
\x22else\x22},{\x0aclassN\
ame:\x22template-ta\
g\x22,begin:/\x5c{\x5c{(?\
=else if)/,end:/\
\x5c}\x5c}/,keywords:\x22\
else if\x22\x0a},{clas\
sName:\x22template-\
tag\x22,begin:/\x5c{\x5c{\
\x5c//,end:/\x5c}\x5c}/,c\
ontains:[h]},{\x0ac\
lassName:\x22templa\
te-variable\x22,beg\
in:/\x5c{\x5c{\x5c{/,end:\
/\x5c}\x5c}\x5c}/,contain\
s:[N]},{\x0aclassNa\
me:\x22template-var\
iable\x22,begin:/\x5c{\
\x5c{/,end:/\x5c}\x5c}/,c\
ontains:[N]}]}}}\
)()\x0a;hljs.regist\
erLanguage(\x22hand\
lebars\x22,e)})();/\
*! `haskell` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var e=(()=>{\x22use\
 strict\x22;return \
e=>{\x0aconst n=\x22([\
0-9]_*)+\x22,a=\x22([0\
-9a-fA-F]_*)+\x22,i\
=\x22([!#$%&*+.\x5c\x5c/<\
=>?@\x5c\x5c\x5c\x5c^~-]|(?!\
([(),;\x5c\x5c[\x5c\x5c]`|{}\
]|[_:\x5c\x22']))(\x5c\x5cp{\
S}|\x5c\x5cp{P}))\x22,s={\
\x0avariants:[e.COM\
MENT(\x22--+\x22,\x22$\x22),\
e.COMMENT(/\x5c{-/,\
/-\x5c}/,{contains:\
[\x22self\x22]})]},l={\
\x0aclassName:\x22meta\
\x22,begin:/\x5c{-#/,e\
nd:/#-\x5c}/},t={cl\
assName:\x22meta\x22,b\
egin:\x22^#\x22,end:\x22$\
\x22\x0a},c={className\
:\x22type\x22,begin:\x22\x5c\
\x5cb[A-Z][\x5c\x5cw']*\x22,\
relevance:0},r={\
begin:\x22\x5c\x5c(\x22,\x0aend\
:\x22\x5c\x5c)\x22,illegal:'\
\x22',contains:[l,t\
,{className:\x22typ\
e\x22,\x0abegin:\x22\x5c\x5cb[A\
-Z][\x5c\x5cw]*(\x5c\x5c((\x5c\x5c\
.\x5c\x5c.|,|\x5c\x5cw+)\x5c\x5c))\
?\x22},e.inherit(e.\
TITLE_MODE,{\x0abeg\
in:\x22[_a-z][\x5c\x5cw']\
*\x22}),s]},o={clas\
sName:\x22number\x22,r\
elevance:0,varia\
nts:[{\x0amatch:`\x5c\x5c\
b(${n})(\x5c\x5c.(${n}\
))?([eE][+-]?(${\
n}))?\x5c\x5cb`},{\x0amat\
ch:`\x5c\x5cb0[xX]_*($\
{a})(\x5c\x5c.(${a}))?\
([pP][+-]?(${n})\
)?\x5c\x5cb`},{\x0amatch:\
\x22\x5c\x5cb0[oO](([0-7]\
_*)+)\x5c\x5cb\x22},{matc\
h:\x22\x5c\x5cb0[bB](([01\
]_*)+)\x5c\x5cb\x22}]};re\
turn{\x0aname:\x22Hask\
ell\x22,aliases:[\x22h\
s\x22],\x0akeywords:\x22l\
et in if then el\
se case of where\
 do module impor\
t hiding qualifi\
ed type data new\
type deriving cl\
ass instance as \
default infix in\
fixl infixr fore\
ign export ccall\
 stdcall cpluspl\
us jvm dotnet sa\
fe unsafe family\
 forall mdo proc\
 rec\x22,\x0aunicodeRe\
gex:!0,contains:\
[{beginKeywords:\
\x22module\x22,end:\x22wh\
ere\x22,\x0akeywords:\x22\
module where\x22,co\
ntains:[r,s],ill\
egal:\x22\x5c\x5cW\x5c\x5c.|;\x22}\
,{\x0abegin:\x22\x5c\x5cbimp\
ort\x5c\x5cb\x22,end:\x22$\x22,\
keywords:\x22import\
 qualified as hi\
ding\x22,\x0acontains:\
[r,s],illegal:\x22\x5c\
\x5cW\x5c\x5c.|;\x22},{class\
Name:\x22class\x22,\x0abe\
gin:\x22^(\x5c\x5cs*)?(cl\
ass|instance)\x5c\x5cb\
\x22,end:\x22where\x22,\x0ak\
eywords:\x22class f\
amily instance w\
here\x22,contains:[\
c,r,s]},{classNa\
me:\x22class\x22,\x0abegi\
n:\x22\x5c\x5cb(data|(new\
)?type)\x5c\x5cb\x22,end:\
\x22$\x22,\x0akeywords:\x22d\
ata family type \
newtype deriving\
\x22,contains:[l,c,\
r,{begin:/\x5c{/,\x0ae\
nd:/\x5c}/,contains\
:r.contains},s]}\
,{beginKeywords:\
\x22default\x22,end:\x22$\
\x22,\x0acontains:[c,r\
,s]},{beginKeywo\
rds:\x22infix infix\
l infixr\x22,end:\x22$\
\x22,\x0acontains:[e.C\
_NUMBER_MODE,s]}\
,{begin:\x22\x5c\x5cbfore\
ign\x5c\x5cb\x22,end:\x22$\x22,\
\x0akeywords:\x22forei\
gn import export\
 ccall stdcall c\
plusplus jvm dot\
net safe unsafe\x22\
,\x0acontains:[c,e.\
QUOTE_STRING_MOD\
E,s]},{className\
:\x22meta\x22,\x0abegin:\x22\
#!\x5c\x5c/usr\x5c\x5c/bin\x5c\x5c\
/env runhaskell\x22\
,end:\x22$\x22},l,t,{s\
cope:\x22string\x22,\x0ab\
egin:/'(?=\x5c\x5c?.')\
/,end:/'/,contai\
ns:[{scope:\x22char\
.escape\x22,match:/\
\x5c\x5c./}]\x0a},e.QUOTE\
_STRING_MODE,o,c\
,e.inherit(e.TIT\
LE_MODE,{begin:\x22\
^[_a-z][\x5c\x5cw']*\x22}\
),{\x0abegin:`(?!-)\
${i}--+|--+(?!-)\
${i}`},s,{begin:\
\x22->|<-\x22}]}}})()\x0a\
;hljs.registerLa\
nguage(\x22haskell\x22\
,e)})();/*! `hax\
e` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>({name:\
\x22Haxe\x22,aliases:[\
\x22hx\x22],keywords:{\
\x0akeyword:\x22abstra\
ct break case ca\
st catch continu\
e default do dyn\
amic else enum e\
xtern final for \
function here if\
 import in inlin\
e is macro never\
 new override pa\
ckage private ge\
t set public ret\
urn static super\
 switch this thr\
ow trace try typ\
edef untyped usi\
ng var while Int\
 Float String Bo\
ol Dynamic Void \
Array \x22,\x0abuilt_i\
n:\x22trace this\x22,l\
iteral:\x22true fal\
se null _\x22},cont\
ains:[{\x0aclassNam\
e:\x22string\x22,begin\
:\x22'\x22,end:\x22'\x22,con\
tains:[e.BACKSLA\
SH_ESCAPE,{\x0aclas\
sName:\x22subst\x22,be\
gin:/\x5c$\x5c{/,end:/\
\x5c}/},{className:\
\x22subst\x22,begin:/\x5c\
$/,\x0aend:/\x5cW\x5c}/}]\
},e.QUOTE_STRING\
_MODE,e.C_LINE_C\
OMMENT_MODE,e.C_\
BLOCK_COMMENT_MO\
DE,{\x0aclassName:\x22\
number\x22,\x0abegin:/\
(-?)(\x5cb0[xX][a-f\
A-F0-9_]+|(\x5cb\x5cd+\
(\x5c.[\x5cd_]*)?|\x5c.[\x5c\
d_]+)(([eE][-+]?\
\x5cd+)|i32|u32|i64\
|f64)?)/,\x0areleva\
nce:0},{classNam\
e:\x22variable\x22,beg\
in:\x22\x5c\x5c$[a-zA-Z_$\
][a-zA-Z0-9_$]*\x22\
},{\x0aclassName:\x22m\
eta\x22,begin:/@:?/\
,end:/\x5c(|$/,excl\
udeEnd:!0},{clas\
sName:\x22meta\x22,\x0abe\
gin:\x22#\x22,end:\x22$\x22,\
keywords:{keywor\
d:\x22if else elsei\
f end error\x22}},{\
\x0aclassName:\x22type\
\x22,begin:/:[ \x5ct]*\
/,end:/[^A-Za-z0\
-9_ \x5ct\x5c->]/,excl\
udeBegin:!0,\x0aexc\
ludeEnd:!0,relev\
ance:0},{classNa\
me:\x22type\x22,begin:\
/:[ \x5ct]*/,end:/\x5c\
W/,\x0aexcludeBegin\
:!0,excludeEnd:!\
0},{className:\x22t\
ype\x22,beginKeywor\
ds:\x22new\x22,end:/\x5cW\
/,\x0aexcludeBegin:\
!0,excludeEnd:!0\
},{className:\x22ti\
tle.class\x22,begin\
Keywords:\x22enum\x22,\
\x0aend:/\x5c{/,contai\
ns:[e.TITLE_MODE\
]},{className:\x22t\
itle.class\x22,\x0abeg\
in:\x22\x5c\x5cbabstract\x5c\
\x5cb(?=\x5c\x5cs*\x22+e.IDE\
NT_RE+\x22\x5c\x5cs*\x5c\x5c()\x22\
,end:/[\x5c{$]/,con\
tains:[{\x0aclassNa\
me:\x22type\x22,begin:\
/\x5c(/,end:/\x5c)/,ex\
cludeBegin:!0,ex\
cludeEnd:!0},{\x0ac\
lassName:\x22type\x22,\
begin:/from +/,e\
nd:/\x5cW/,excludeB\
egin:!0,excludeE\
nd:!0},{\x0aclassNa\
me:\x22type\x22,begin:\
/to +/,end:/\x5cW/,\
excludeBegin:!0,\
excludeEnd:!0\x0a},\
e.TITLE_MODE],ke\
ywords:{keyword:\
\x22abstract from t\
o\x22}},{className:\
\x22title.class\x22,\x0ab\
egin:/\x5cb(class|i\
nterface) +/,end\
:/[\x5c{$]/,exclude\
End:!0,\x0akeywords\
:\x22class interfac\
e\x22,contains:[{cl\
assName:\x22keyword\
\x22,\x0abegin:/\x5cb(ext\
ends|implements)\
 +/,keywords:\x22ex\
tends implements\
\x22,contains:[{\x0acl\
assName:\x22type\x22,b\
egin:e.IDENT_RE,\
relevance:0}]},e\
.TITLE_MODE]},{\x0a\
className:\x22title\
.function\x22,begin\
Keywords:\x22functi\
on\x22,end:/\x5c(/,exc\
ludeEnd:!0,\x0aille\
gal:/\x5cS/,contain\
s:[e.TITLE_MODE]\
}],illegal:/<\x5c//\
})})()\x0a;hljs.reg\
isterLanguage(\x22h\
axe\x22,e)})();/*! \
`hsp` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>({na\
me:\x22HSP\x22,case_in\
sensitive:!0,\x0ake\
ywords:{$pattern\
:/[\x5cw._]+/,\x0akeyw\
ord:\x22goto gosub \
return break rep\
eat loop continu\
e wait await dim\
 sdim foreach di\
mtype dup dupptr\
 end stop newmod\
 delmod mref run\
 exgoto on mcall\
 assert logmes n\
ewlab resume yie\
ld onexit onerro\
r onkey onclick \
oncmd exist dele\
te mkdir chdir d\
irlist bload bsa\
ve bcopy memfile\
 if else poke wp\
oke lpoke getstr\
 chdpm memexpand\
 memcpy memset n\
otesel noteadd n\
otedel noteload \
notesave randomi\
ze noteunsel not\
eget split strre\
p setease button\
 chgdisp exec di\
alog mmload mmpl\
ay mmstop mci ps\
et pget syscolor\
 mes print title\
 pos circle cls \
font sysfont obj\
size picload col\
or palcolor pale\
tte redraw width\
 gsel gcopy gzoo\
m gmode bmpsave \
hsvcolor getkey \
listbox chkbox c\
ombox input mesb\
ox buffer screen\
 bgscr mouse obj\
sel groll line c\
lrobj boxf objpr\
m objmode stick \
grect grotate gs\
quare gradf obji\
mage objskip obj\
enable celload c\
eldiv celput new\
com querycom del\
com cnvstow comr\
es axobj winobj \
sendmsg comevent\
 comevarg sarray\
conv callfunc cn\
vwtos comevdisp \
libptr system hs\
pstat hspver sta\
t cnt err strsiz\
e looplev sublev\
 iparam wparam l\
param refstr ref\
dval int rnd str\
len length lengt\
h2 length3 lengt\
h4 vartype getti\
me peek wpeek lp\
eek varptr varus\
e noteinfo instr\
 abs limit getea\
se str strmid st\
rf getpath strtr\
im sin cos tan a\
tan sqrt double \
absf expf logf l\
imitf powf getea\
sef mousex mouse\
y mousew hwnd hi\
nstance hdc ginf\
o objinfo dirinf\
o sysinfo thismo\
d __hspver__ __h\
sp30__ __date__ \
__time__ __line_\
_ __file__ _debu\
g __hspdef__ and\
 or xor not scre\
en_normal screen\
_palette screen_\
hide screen_fixe\
dsize screen_too\
l screen_frame g\
mode_gdi gmode_m\
em gmode_rgb0 gm\
ode_alpha gmode_\
rgb0alpha gmode_\
add gmode_sub gm\
ode_pixela ginfo\
_mx ginfo_my gin\
fo_act ginfo_sel\
 ginfo_wx1 ginfo\
_wy1 ginfo_wx2 g\
info_wy2 ginfo_v\
x ginfo_vy ginfo\
_sizex ginfo_siz\
ey ginfo_winx gi\
nfo_winy ginfo_m\
esx ginfo_mesy g\
info_r ginfo_g g\
info_b ginfo_pal\
use ginfo_dispx \
ginfo_dispy ginf\
o_cx ginfo_cy gi\
nfo_intid ginfo_\
newid ginfo_sx g\
info_sy objinfo_\
mode objinfo_bms\
cr objinfo_hwnd \
notemax notesize\
 dir_cur dir_exe\
 dir_win dir_sys\
 dir_cmdline dir\
_desktop dir_myd\
oc dir_tv font_n\
ormal font_bold \
font_italic font\
_underline font_\
strikeout font_a\
ntialias objmode\
_normal objmode_\
guifont objmode_\
usefont gsquare_\
grad msgothic ms\
mincho do until \
while wend for n\
ext _break _cont\
inue switch case\
 default swbreak\
 swend ddim ldim\
 alloc m_pi rad2\
deg deg2rad ease\
_linear ease_qua\
d_in ease_quad_o\
ut ease_quad_ino\
ut ease_cubic_in\
 ease_cubic_out \
ease_cubic_inout\
 ease_quartic_in\
 ease_quartic_ou\
t ease_quartic_i\
nout ease_bounce\
_in ease_bounce_\
out ease_bounce_\
inout ease_shake\
_in ease_shake_o\
ut ease_shake_in\
out ease_loop\x22\x0a}\
,\x0acontains:[e.C_\
LINE_COMMENT_MOD\
E,e.C_BLOCK_COMM\
ENT_MODE,e.QUOTE\
_STRING_MODE,e.A\
POS_STRING_MODE,\
{\x0aclassName:\x22str\
ing\x22,begin:/\x5c{\x22/\
,end:/\x22\x5c}/,conta\
ins:[e.BACKSLASH\
_ESCAPE]\x0a},e.COM\
MENT(\x22;\x22,\x22$\x22,{re\
levance:0}),{cla\
ssName:\x22meta\x22,be\
gin:\x22#\x22,end:\x22$\x22,\
\x0akeywords:{\x0akeyw\
ord:\x22addion cfun\
c cmd cmpopt com\
func const defcf\
unc deffunc defi\
ne else endif en\
um epack func gl\
obal if ifdef if\
ndef include mod\
cfunc modfunc mo\
dinit modterm mo\
dule pack packop\
t regcmd runtime\
 undef usecom us\
elib\x22\x0a},contains\
:[e.inherit(e.QU\
OTE_STRING_MODE,\
{className:\x22stri\
ng\x22\x0a}),e.NUMBER_\
MODE,e.C_NUMBER_\
MODE,e.C_LINE_CO\
MMENT_MODE,e.C_B\
LOCK_COMMENT_MOD\
E]\x0a},{className:\
\x22symbol\x22,begin:\x22\
^\x5c\x5c*(\x5c\x5cw+|@)\x22},e\
.NUMBER_MODE,e.C\
_NUMBER_MODE]})}\
)()\x0a;hljs.regist\
erLanguage(\x22hsp\x22\
,e)})();/*! `htt\
p` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>{const \
n=\x22HTTP/([32]|1\x5c\
\x5c.[01])\x22,a={\x0acla\
ssName:\x22attribut\
e\x22,\x0abegin:e.rege\
x.concat(\x22^\x22,/[A\
-Za-z][A-Za-z0-9\
-]*/,\x22(?=\x5c\x5c:\x5c\x5cs)\
\x22),starts:{\x0acont\
ains:[{className\
:\x22punctuation\x22,b\
egin:/: /,releva\
nce:0,starts:{en\
d:\x22$\x22,\x0arelevance\
:0}}]}},s=[a,{be\
gin:\x22\x5c\x5cn\x5c\x5cn\x22,sta\
rts:{subLanguage\
:[],endsWithPare\
nt:!0}\x0a}];return\
{name:\x22HTTP\x22,ali\
ases:[\x22https\x22],i\
llegal:/\x5cS/,cont\
ains:[{\x0abegin:\x22^\
(?=\x22+n+\x22 \x5c\x5cd{3})\
\x22,end:/$/,contai\
ns:[{className:\x22\
meta\x22,begin:n},{\
\x0aclassName:\x22numb\
er\x22,begin:\x22\x5c\x5cb\x5c\x5c\
d{3}\x5c\x5cb\x22}],start\
s:{end:/\x5cb\x5cB/,il\
legal:/\x5cS/,\x0acont\
ains:s}},{begin:\
\x22(?=^[A-Z]+ (.*?\
) \x22+n+\x22$)\x22,end:/\
$/,contains:[{\x0ac\
lassName:\x22string\
\x22,begin:\x22 \x22,end:\
\x22 \x22,excludeBegin\
:!0,excludeEnd:!\
0},{\x0aclassName:\x22\
meta\x22,begin:n},{\
className:\x22keywo\
rd\x22,begin:\x22[A-Z]\
+\x22}],starts:{\x0aen\
d:/\x5cb\x5cB/,illegal\
:/\x5cS/,contains:s\
}},e.inherit(a,{\
relevance:0})]}}\
})()\x0a;hljs.regis\
terLanguage(\x22htt\
p\x22,e)})();/*! `h\
y` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>{\x0aconst\
 a=\x22a-zA-Z_\x5c\x5c-!.\
?+*=<>&#'\x22,t=\x22[\x22\
+a+\x22][\x22+a+\x220-9/;\
:]*\x22,i={$pattern\
:t,\x0abuilt_in:\x22!=\
 % %= & &= * ** \
**= *= *map + +=\
 , --build-class\
-- --import-- -=\
 . / // //= /= <\
 << <<= <= = > >\
= >> >>= @ @= ^ \
^= abs accumulat\
e all and any ap\
-compose ap-doti\
mes ap-each ap-e\
ach-while ap-fil\
ter ap-first ap-\
if ap-last ap-ma\
p ap-map-when ap\
-pipe ap-reduce \
ap-reject apply \
as-> ascii asser\
t assoc bin brea\
k butlast callab\
le calling-modul\
e-name car case \
cdr chain chr co\
ll? combinations\
 compile compres\
s cond cons cons\
? continue count\
 curry cut cycle\
 dec def default\
-method defclass\
 defmacro defmac\
ro-alias defmacr\
o/g! defmain def\
method defmulti \
defn defn-alias \
defnc defnr defr\
eader defseq del\
 delattr delete-\
route dict-comp \
dir disassemble \
dispatch-reader-\
macro distinct d\
ivmod do doto dr\
op drop-last dro\
p-while empty? e\
nd-sequence eval\
 eval-and-compil\
e eval-when-comp\
ile even? every?\
 except exec fil\
ter first flatte\
n float? fn fnc \
fnr for for* for\
mat fraction gen\
expr gensym get \
getattr global g\
lobals group-by \
hasattr hash hex\
 id identity if \
if* if-not if-py\
thon2 import in \
inc input instan\
ce? integer inte\
ger-char? intege\
r? interleave in\
terpose is is-co\
ll is-cons is-em\
pty is-even is-e\
very is-float is\
-instance is-int\
eger is-integer-\
char is-iterable\
 is-iterator is-\
keyword is-neg i\
s-none is-not is\
-numeric is-odd \
is-pos is-string\
 is-symbol is-ze\
ro isinstance is\
lice issubclass \
iter iterable? i\
terate iterator?\
 keyword keyword\
? lambda last le\
n let lif lif-no\
t list* list-com\
p locals loop ma\
cro-error macroe\
xpand macroexpan\
d-1 macroexpand-\
all map max merg\
e-with method-de\
corator min mult\
i-decorator mult\
icombinations na\
me neg? next non\
e? nonlocal not \
not-in not? nth \
numeric? oct odd\
? open or ord pa\
rtition permutat\
ions pos? post-r\
oute postwalk po\
w prewalk print \
product profile/\
calls profile/cp\
u put-route quas\
iquote quote rai\
se range read re\
ad-str recursive\
-replace reduce \
remove repeat re\
peatedly repr re\
quire rest round\
 route route-wit\
h-methods rwm se\
cond seq set-com\
p setattr setv s\
ome sorted strin\
g string? sum sw\
itch symbol? tak\
e take-nth take-\
while tee try un\
less unquote unq\
uote-splicing va\
rs walk when whi\
le with with* wi\
th-decorator wit\
h-gensyms xi xor\
 yield yield-fro\
m zero? zip zip-\
longest | |= ~\x22\x0a\
},r={begin:t,rel\
evance:0},n={cla\
ssName:\x22number\x22,\
begin:\x22[-+]?\x5c\x5cd+\
(\x5c\x5c.\x5c\x5cd+)?\x22,\x0arel\
evance:0},s=e.in\
herit(e.QUOTE_ST\
RING_MODE,{illeg\
al:null\x0a}),o=e.C\
OMMENT(\x22;\x22,\x22$\x22,{\
relevance:0}),l=\
{className:\x22lite\
ral\x22,\x0abegin:/\x5cb(\
[Tt]rue|[Ff]alse\
|nil|None)\x5cb/},c\
={begin:\x22[\x5c\x5c[\x5c\x5c{\
]\x22,end:\x22[\x5c\x5c]\x5c\x5c}]\
\x22,\x0arelevance:0},\
d={className:\x22co\
mment\x22,begin:\x22\x5c\x5c\
^\x22+t\x0a},m=e.COMME\
NT(\x22\x5c\x5c^\x5c\x5c{\x22,\x22\x5c\x5c}\
\x22),p={className:\
\x22symbol\x22,begin:\x22\
[:]{1,2}\x22+t},u={\
\x0abegin:\x22\x5c\x5c(\x22,end\
:\x22\x5c\x5c)\x22},f={endsW\
ithParent:!0,rel\
evance:0},g={cla\
ssName:\x22name\x22,\x0ar\
elevance:0,keywo\
rds:i,begin:t,st\
arts:f},h=[u,s,d\
,m,o,p,c,n,l,r]\x0a\
;return u.contai\
ns=[e.COMMENT(\x22c\
omment\x22,\x22\x22),g,f]\
,f.contains=h,c.\
contains=h,{\x0anam\
e:\x22Hy\x22,aliases:[\
\x22hylang\x22],illega\
l:/\x5cS/,\x0acontains\
:[e.SHEBANG(),u,\
s,d,m,o,p,c,n,l]\
}}})();hljs.regi\
sterLanguage(\x22hy\
\x22,e)\x0a})();/*! `i\
nform7` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>({\
name:\x22Inform 7\x22,\
aliases:[\x22i7\x22],\x0a\
case_insensitive\
:!0,keywords:{\x0ak\
eyword:\x22thing ro\
om person man wo\
man animal conta\
iner supporter b\
ackdrop door sce\
nery open closed\
 locked inside g\
ender is are say\
 understand kind\
 of rule\x22\x0a},cont\
ains:[{className\
:\x22string\x22,begin:\
'\x22',end:'\x22',rele\
vance:0,contains\
:[{\x0aclassName:\x22s\
ubst\x22,begin:\x22\x5c\x5c[\
\x22,end:\x22\x5c\x5c]\x22}]},{\
className:\x22secti\
on\x22,\x0abegin:/^(Vo\
lume|Book|Part|C\
hapter|Section|T\
able)\x5cb/,end:\x22$\x22\
},{\x0abegin:/^(Che\
ck|Carry out|Rep\
ort|Instead of|T\
o|Rule|When|Befo\
re|After)\x5cb/,\x0aen\
d:\x22:\x22,contains:[\
{begin:\x22\x5c\x5c(This\x22\
,end:\x22\x5c\x5c)\x22}]},{c\
lassName:\x22commen\
t\x22,\x0abegin:\x22\x5c\x5c[\x22,\
end:\x22\x5c\x5c]\x22,contai\
ns:[\x22self\x22]}]})}\
)()\x0a;hljs.regist\
erLanguage(\x22info\
rm7\x22,e)})();/*! \
`ini` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>{con\
st n=e.regex,a={\
className:\x22numbe\
r\x22,\x0arelevance:0,\
variants:[{begin\
:/([+-]+)?[\x5cd]+_\
[\x5cd_]+/},{begin:\
e.NUMBER_RE}]\x0a},\
s=e.COMMENT();s.\
variants=[{begin\
:/;/,end:/$/},{b\
egin:/#/,end:/$/\
}];const i={\x0acla\
ssName:\x22variable\
\x22,variants:[{beg\
in:/\x5c$[\x5cw\x5cd\x22][\x5cw\
\x5cd_]*/},{begin:/\
\x5c$\x5c{(.*?)\x5c}/\x0a}]}\
,t={className:\x22l\
iteral\x22,begin:/\x5c\
bon|off|true|fal\
se|yes|no\x5cb/},r=\
{\x0aclassName:\x22str\
ing\x22,contains:[e\
.BACKSLASH_ESCAP\
E],variants:[{be\
gin:\x22'''\x22,\x0aend:\x22\
'''\x22,relevance:1\
0},{begin:'\x22\x22\x22',\
end:'\x22\x22\x22',releva\
nce:10},{begin:'\
\x22',end:'\x22'\x0a},{be\
gin:\x22'\x22,end:\x22'\x22}\
]},l={begin:/\x5c[/\
,end:/\x5c]/,contai\
ns:[s,t,i,r,a,\x22s\
elf\x22],\x0arelevance\
:0},c=n.either(/\
[A-Za-z0-9_-]+/,\
/\x22(\x5c\x5c\x22|[^\x22])*\x22/,\
/'[^']*'/);retur\
n{\x0aname:\x22TOML, a\
lso INI\x22,aliases\
:[\x22toml\x22],case_i\
nsensitive:!0,il\
legal:/\x5cS/,\x0acont\
ains:[s,{classNa\
me:\x22section\x22,beg\
in:/\x5c[+/,end:/\x5c]\
+/},{\x0abegin:n.co\
ncat(c,\x22(\x5c\x5cs*\x5c\x5c.\
\x5c\x5cs*\x22,c,\x22)*\x22,n.l\
ookahead(/\x5cs*=\x5cs\
*[^#\x5cs]/)),\x0aclas\
sName:\x22attr\x22,sta\
rts:{end:/$/,con\
tains:[s,l,t,i,r\
,a]}}]}}})()\x0a;hl\
js.registerLangu\
age(\x22ini\x22,e)})()\
;/*! `irpf90` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>{\x0aconst n=e.\
regex,t=/(_[a-z_\
\x5cd]+)?/,a=/([de]\
[+-]?\x5cd+)?/,i={c\
lassName:\x22number\
\x22,\x0avariants:[{be\
gin:n.concat(/\x5cb\
\x5cd+/,/\x5c.(\x5cd*)/,a\
,t)},{begin:n.co\
ncat(/\x5cb\x5cd+/,a,t\
)\x0a},{begin:n.con\
cat(/\x5c.\x5cd+/,a,t)\
}],relevance:0};\
return{name:\x22IRP\
F90\x22,\x0acase_insen\
sitive:!0,keywor\
ds:{literal:\x22.Fa\
lse. .True.\x22,\x0ake\
yword:\x22kind do w\
hile private cal\
l intrinsic wher\
e elsewhere type\
 endtype endmodu\
le endselect end\
interface end en\
ddo endif if for\
all endforall on\
ly contains defa\
ult return stop \
then public subr\
outine|10 functi\
on program .and.\
 .or. .not. .le.\
 .eq. .ge. .gt. \
.lt. goto save e\
lse use module s\
elect case acces\
s blank direct e\
xist file fmt fo\
rm formatted ios\
tat name named n\
extrec number op\
ened rec recl se\
quential status \
unformatted unit\
 continue format\
 pause cycle exi\
t c_null_char c_\
alert c_backspac\
e c_form_feed fl\
ush wait decimal\
 round iomsg syn\
chronous nopass \
non_overridable \
pass protected v\
olatile abstract\
 extends import \
non_intrinsic va\
lue deferred gen\
eric final enume\
rator class asso\
ciate bind enum \
c_int c_short c_\
long c_long_long\
 c_signed_char c\
_size_t c_int8_t\
 c_int16_t c_int\
32_t c_int64_t c\
_int_least8_t c_\
int_least16_t c_\
int_least32_t c_\
int_least64_t c_\
int_fast8_t c_in\
t_fast16_t c_int\
_fast32_t c_int_\
fast64_t c_intma\
x_t C_intptr_t c\
_float c_double \
c_long_double c_\
float_complex c_\
double_complex c\
_long_double_com\
plex c_bool c_ch\
ar c_null_ptr c_\
null_funptr c_ne\
w_line c_carriag\
e_return c_horiz\
ontal_tab c_vert\
ical_tab iso_c_b\
inding c_loc c_f\
unloc c_associat\
ed  c_f_pointer \
c_ptr c_funptr i\
so_fortran_env c\
haracter_storage\
_size error_unit\
 file_storage_si\
ze input_unit io\
stat_end iostat_\
eor numeric_stor\
age_size output_\
unit c_f_procpoi\
nter ieee_arithm\
etic ieee_suppor\
t_underflow_cont\
rol ieee_get_und\
erflow_mode ieee\
_set_underflow_m\
ode newunit cont\
iguous recursive\
 pad position ac\
tion delim readw\
rite eor advance\
 nml interface p\
rocedure namelis\
t include sequen\
ce elemental pur\
e integer real c\
haracter complex\
 logical dimensi\
on allocatable|1\
0 parameter exte\
rnal implicit|10\
 none double pre\
cision assign in\
tent optional po\
inter target in \
out common equiv\
alence data begi\
n_provider &begi\
n_provider end_p\
rovider begin_sh\
ell end_shell be\
gin_template end\
_template subst \
assert touch sof\
t_touch provide \
no_dep free irp_\
if irp_else irp_\
endif irp_write \
irp_read\x22,\x0abuilt\
_in:\x22alog alog10\
 amax0 amax1 ami\
n0 amin1 amod ca\
bs ccos cexp clo\
g csin csqrt dab\
s dacos dasin da\
tan datan2 dcos \
dcosh ddim dexp \
dint dlog dlog10\
 dmax1 dmin1 dmo\
d dnint dsign ds\
in dsinh dsqrt d\
tan dtanh float \
iabs idim idint \
idnint ifix isig\
n max0 max1 min0\
 min1 sngl algam\
a cdabs cdcos cd\
exp cdlog cdsin \
cdsqrt cqabs cqc\
os cqexp cqlog c\
qsin cqsqrt dcmp\
lx dconjg derf d\
erfc dfloat dgam\
ma dimag dlgama \
iqint qabs qacos\
 qasin qatan qat\
an2 qcmplx qconj\
g qcos qcosh qdi\
m qerf qerfc qex\
p qgamma qimag q\
lgama qlog qlog1\
0 qmax1 qmin1 qm\
od qnint qsign q\
sin qsinh qsqrt \
qtan qtanh abs a\
cos aimag aint a\
nint asin atan a\
tan2 char cmplx \
conjg cos cosh e\
xp ichar index i\
nt log log10 max\
 min nint sign s\
in sinh sqrt tan\
 tanh print writ\
e dim lge lgt ll\
e llt mod nullif\
y allocate deall\
ocate adjustl ad\
justr all alloca\
ted any associat\
ed bit_size btes\
t ceiling count \
cshift date_and_\
time digits dot_\
product eoshift \
epsilon exponent\
 floor fraction \
huge iand ibclr \
ibits ibset ieor\
 ior ishft ishft\
c lbound len_tri\
m matmul maxexpo\
nent maxloc maxv\
al merge minexpo\
nent minloc minv\
al modulo mvbits\
 nearest pack pr\
esent product ra\
dix random_numbe\
r random_seed ra\
nge repeat resha\
pe rrspacing sca\
le scan selected\
_int_kind select\
ed_real_kind set\
_exponent shape \
size spacing spr\
ead sum system_c\
lock tiny transp\
ose trim ubound \
unpack verify ac\
har iachar trans\
fer dble entry d\
prod cpu_time co\
mmand_argument_c\
ount get_command\
 get_command_arg\
ument get_enviro\
nment_variable i\
s_iostat_end iee\
e_arithmetic iee\
e_support_underf\
low_control ieee\
_get_underflow_m\
ode ieee_set_und\
erflow_mode is_i\
ostat_eor move_a\
lloc new_line se\
lected_char_kind\
 same_type_as ex\
tends_type_of ac\
osh asinh atanh \
bessel_j0 bessel\
_j1 bessel_jn be\
ssel_y0 bessel_y\
1 bessel_yn erf \
erfc erfc_scaled\
 gamma log_gamma\
 hypot norm2 ato\
mic_define atomi\
c_ref execute_co\
mmand_line leadz\
 trailz storage_\
size merge_bits \
bge bgt ble blt \
dshiftl dshiftr \
findloc iall ian\
y iparity image_\
index lcobound u\
cobound maskl ma\
skr num_images p\
arity popcnt pop\
par shifta shift\
l shiftr this_im\
age IRP_ALIGN ir\
p_here\x22\x0a},illega\
l:/\x5c/\x5c*/,contain\
s:[e.inherit(e.A\
POS_STRING_MODE,\
{className:\x22stri\
ng\x22,\x0arelevance:0\
}),e.inherit(e.Q\
UOTE_STRING_MODE\
,{className:\x22str\
ing\x22,relevance:0\
}),{\x0aclassName:\x22\
function\x22,beginK\
eywords:\x22subrout\
ine function pro\
gram\x22,\x0aillegal:\x22\
[${=\x5c\x5cn]\x22,contai\
ns:[e.UNDERSCORE\
_TITLE_MODE,{cla\
ssName:\x22params\x22,\
\x0abegin:\x22\x5c\x5c(\x22,end\
:\x22\x5c\x5c)\x22}]},e.COMM\
ENT(\x22!\x22,\x22$\x22,{rel\
evance:0\x0a}),e.CO\
MMENT(\x22begin_doc\
\x22,\x22end_doc\x22,{rel\
evance:10}),i]}}\
})()\x0a;hljs.regis\
terLanguage(\x22irp\
f90\x22,e)})();/*! \
`isbl` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var S=\
(()=>{\x22use stric\
t\x22;return S=>{\x0ac\
onst E=\x22[A-Za-z\x5c\
u0410-\x5cu042f\x5cu04\
30-\x5cu044f\x5cu0451\x5c\
u0401_!][A-Za-z\x5c\
u0410-\x5cu042f\x5cu04\
30-\x5cu044f\x5cu0451\x5c\
u0401_0-9]*\x22,_={\
\x0aclassName:\x22numb\
er\x22,begin:S.NUMB\
ER_RE,relevance:\
0},T={className:\
\x22string\x22,\x0avarian\
ts:[{begin:'\x22',e\
nd:'\x22'},{begin:\x22\
'\x22,end:\x22'\x22}]},R=\
{className:\x22doct\
ag\x22,\x0abegin:\x22\x5c\x5cb(\
?:TODO|DONE|BEGI\
N|END|STUB|CHG|F\
IXME|NOTE|BUG|XX\
X)\x5c\x5cb\x22,relevance\
:0\x0a},O={variants\
:[{className:\x22co\
mment\x22,begin:\x22//\
\x22,end:\x22$\x22,releva\
nce:0,\x0acontains:\
[S.PHRASAL_WORDS\
_MODE,R]},{class\
Name:\x22comment\x22,b\
egin:\x22/\x5c\x5c*\x22,end:\
\x22\x5c\x5c*/\x22,\x0arelevanc\
e:0,contains:[S.\
PHRASAL_WORDS_MO\
DE,R]}]},C={$pat\
tern:E,\x0akeyword:\
\x22and \x5cu0438 else\
 \x5cu0438\x5cu043d\x5cu0\
430\x5cu0447\x5cu0435 \
endexcept endfin\
ally endforeach \
\x5cu043a\x5cu043e\x5cu04\
3d\x5cu0435\x5cu0446\x5cu\
0432\x5cu0441\x5cu0435\
 endif \x5cu043a\x5cu0\
43e\x5cu043d\x5cu0435\x5c\
u0446\x5cu0435\x5cu044\
1\x5cu043b\x5cu0438 en\
dwhile \x5cu043a\x5cu0\
43e\x5cu043d\x5cu0435\x5c\
u0446\x5cu043f\x5cu043\
e\x5cu043a\x5cu0430 ex\
cept exitfor fin\
ally foreach \x5cu0\
432\x5cu0441\x5cu0435 \
if \x5cu0435\x5cu0441\x5c\
u043b\x5cu0438 in \x5c\
u0432 not \x5cu043d\
\x5cu0435 or \x5cu0438\
\x5cu043b\x5cu0438 try\
 while \x5cu043f\x5cu0\
43e\x5cu043a\x5cu0430 \
\x22,\x0abuilt_in:\x22SYS\
RES_CONST_ACCES_\
RIGHT_TYPE_EDIT \
SYSRES_CONST_ACC\
ES_RIGHT_TYPE_FU\
LL SYSRES_CONST_\
ACCES_RIGHT_TYPE\
_VIEW SYSRES_CON\
ST_ACCESS_MODE_R\
EQUISITE_CODE SY\
SRES_CONST_ACCES\
S_NO_ACCESS_VIEW\
 SYSRES_CONST_AC\
CESS_NO_ACCESS_V\
IEW_CODE SYSRES_\
CONST_ACCESS_RIG\
HTS_ADD_REQUISIT\
E_CODE SYSRES_CO\
NST_ACCESS_RIGHT\
S_ADD_REQUISITE_\
YES_CODE SYSRES_\
CONST_ACCESS_RIG\
HTS_CHANGE_REQUI\
SITE_CODE SYSRES\
_CONST_ACCESS_RI\
GHTS_CHANGE_REQU\
ISITE_YES_CODE S\
YSRES_CONST_ACCE\
SS_RIGHTS_DELETE\
_REQUISITE_CODE \
SYSRES_CONST_ACC\
ESS_RIGHTS_DELET\
E_REQUISITE_YES_\
CODE SYSRES_CONS\
T_ACCESS_RIGHTS_\
EXECUTE_REQUISIT\
E_CODE SYSRES_CO\
NST_ACCESS_RIGHT\
S_EXECUTE_REQUIS\
ITE_YES_CODE SYS\
RES_CONST_ACCESS\
_RIGHTS_NO_ACCES\
S_REQUISITE_CODE\
 SYSRES_CONST_AC\
CESS_RIGHTS_NO_A\
CCESS_REQUISITE_\
YES_CODE SYSRES_\
CONST_ACCESS_RIG\
HTS_RATIFY_REQUI\
SITE_CODE SYSRES\
_CONST_ACCESS_RI\
GHTS_RATIFY_REQU\
ISITE_YES_CODE S\
YSRES_CONST_ACCE\
SS_RIGHTS_REQUIS\
ITE_CODE SYSRES_\
CONST_ACCESS_RIG\
HTS_VIEW SYSRES_\
CONST_ACCESS_RIG\
HTS_VIEW_CODE SY\
SRES_CONST_ACCES\
S_RIGHTS_VIEW_RE\
QUISITE_CODE SYS\
RES_CONST_ACCESS\
_RIGHTS_VIEW_REQ\
UISITE_YES_CODE \
SYSRES_CONST_ACC\
ESS_TYPE_CHANGE \
SYSRES_CONST_ACC\
ESS_TYPE_CHANGE_\
CODE SYSRES_CONS\
T_ACCESS_TYPE_EX\
ISTS SYSRES_CONS\
T_ACCESS_TYPE_EX\
ISTS_CODE SYSRES\
_CONST_ACCESS_TY\
PE_FULL SYSRES_C\
ONST_ACCESS_TYPE\
_FULL_CODE SYSRE\
S_CONST_ACCESS_T\
YPE_VIEW SYSRES_\
CONST_ACCESS_TYP\
E_VIEW_CODE SYSR\
ES_CONST_ACTION_\
TYPE_ABORT SYSRE\
S_CONST_ACTION_T\
YPE_ACCEPT SYSRE\
S_CONST_ACTION_T\
YPE_ACCESS_RIGHT\
S SYSRES_CONST_A\
CTION_TYPE_ADD_A\
TTACHMENT SYSRES\
_CONST_ACTION_TY\
PE_CHANGE_CARD S\
YSRES_CONST_ACTI\
ON_TYPE_CHANGE_K\
IND SYSRES_CONST\
_ACTION_TYPE_CHA\
NGE_STORAGE SYSR\
ES_CONST_ACTION_\
TYPE_CONTINUE SY\
SRES_CONST_ACTIO\
N_TYPE_COPY SYSR\
ES_CONST_ACTION_\
TYPE_CREATE SYSR\
ES_CONST_ACTION_\
TYPE_CREATE_VERS\
ION SYSRES_CONST\
_ACTION_TYPE_DEL\
ETE SYSRES_CONST\
_ACTION_TYPE_DEL\
ETE_ATTACHMENT S\
YSRES_CONST_ACTI\
ON_TYPE_DELETE_V\
ERSION SYSRES_CO\
NST_ACTION_TYPE_\
DISABLE_DELEGATE\
_ACCESS_RIGHTS S\
YSRES_CONST_ACTI\
ON_TYPE_ENABLE_D\
ELEGATE_ACCESS_R\
IGHTS SYSRES_CON\
ST_ACTION_TYPE_E\
NCRYPTION_BY_CER\
TIFICATE SYSRES_\
CONST_ACTION_TYP\
E_ENCRYPTION_BY_\
CERTIFICATE_AND_\
PASSWORD SYSRES_\
CONST_ACTION_TYP\
E_ENCRYPTION_BY_\
PASSWORD SYSRES_\
CONST_ACTION_TYP\
E_EXPORT_WITH_LO\
CK SYSRES_CONST_\
ACTION_TYPE_EXPO\
RT_WITHOUT_LOCK \
SYSRES_CONST_ACT\
ION_TYPE_IMPORT_\
WITH_UNLOCK SYSR\
ES_CONST_ACTION_\
TYPE_IMPORT_WITH\
OUT_UNLOCK SYSRE\
S_CONST_ACTION_T\
YPE_LIFE_CYCLE_S\
TAGE SYSRES_CONS\
T_ACTION_TYPE_LO\
CK SYSRES_CONST_\
ACTION_TYPE_LOCK\
_FOR_SERVER SYSR\
ES_CONST_ACTION_\
TYPE_LOCK_MODIFY\
 SYSRES_CONST_AC\
TION_TYPE_MARK_A\
S_READED SYSRES_\
CONST_ACTION_TYP\
E_MARK_AS_UNREAD\
ED SYSRES_CONST_\
ACTION_TYPE_MODI\
FY SYSRES_CONST_\
ACTION_TYPE_MODI\
FY_CARD SYSRES_C\
ONST_ACTION_TYPE\
_MOVE_TO_ARCHIVE\
 SYSRES_CONST_AC\
TION_TYPE_OFF_EN\
CRYPTION SYSRES_\
CONST_ACTION_TYP\
E_PASSWORD_CHANG\
E SYSRES_CONST_A\
CTION_TYPE_PERFO\
RM SYSRES_CONST_\
ACTION_TYPE_RECO\
VER_FROM_LOCAL_C\
OPY SYSRES_CONST\
_ACTION_TYPE_RES\
TART SYSRES_CONS\
T_ACTION_TYPE_RE\
STORE_FROM_ARCHI\
VE SYSRES_CONST_\
ACTION_TYPE_REVI\
SION SYSRES_CONS\
T_ACTION_TYPE_SE\
ND_BY_MAIL SYSRE\
S_CONST_ACTION_T\
YPE_SIGN SYSRES_\
CONST_ACTION_TYP\
E_START SYSRES_C\
ONST_ACTION_TYPE\
_UNLOCK SYSRES_C\
ONST_ACTION_TYPE\
_UNLOCK_FROM_SER\
VER SYSRES_CONST\
_ACTION_TYPE_VER\
SION_STATE SYSRE\
S_CONST_ACTION_T\
YPE_VERSION_VISI\
BILITY SYSRES_CO\
NST_ACTION_TYPE_\
VIEW SYSRES_CONS\
T_ACTION_TYPE_VI\
EW_SHADOW_COPY S\
YSRES_CONST_ACTI\
ON_TYPE_WORKFLOW\
_DESCRIPTION_MOD\
IFY SYSRES_CONST\
_ACTION_TYPE_WRI\
TE_HISTORY SYSRE\
S_CONST_ACTIVE_V\
ERSION_STATE_PIC\
K_VALUE SYSRES_C\
ONST_ADD_REFEREN\
CE_MODE_NAME SYS\
RES_CONST_ADDITI\
ON_REQUISITE_COD\
E SYSRES_CONST_A\
DDITIONAL_PARAMS\
_REQUISITE_CODE \
SYSRES_CONST_ADI\
TIONAL_JOB_END_D\
ATE_REQUISITE_NA\
ME SYSRES_CONST_\
ADITIONAL_JOB_RE\
AD_REQUISITE_NAM\
E SYSRES_CONST_A\
DITIONAL_JOB_STA\
RT_DATE_REQUISIT\
E_NAME SYSRES_CO\
NST_ADITIONAL_JO\
B_STATE_REQUISIT\
E_NAME SYSRES_CO\
NST_ADMINISTRATI\
ON_HISTORY_ADDIN\
G_USER_TO_GROUP_\
ACTION SYSRES_CO\
NST_ADMINISTRATI\
ON_HISTORY_ADDIN\
G_USER_TO_GROUP_\
ACTION_CODE SYSR\
ES_CONST_ADMINIS\
TRATION_HISTORY_\
CREATION_COMP_AC\
TION SYSRES_CONS\
T_ADMINISTRATION\
_HISTORY_CREATIO\
N_COMP_ACTION_CO\
DE SYSRES_CONST_\
ADMINISTRATION_H\
ISTORY_CREATION_\
GROUP_ACTION SYS\
RES_CONST_ADMINI\
STRATION_HISTORY\
_CREATION_GROUP_\
ACTION_CODE SYSR\
ES_CONST_ADMINIS\
TRATION_HISTORY_\
CREATION_USER_AC\
TION SYSRES_CONS\
T_ADMINISTRATION\
_HISTORY_CREATIO\
N_USER_ACTION_CO\
DE SYSRES_CONST_\
ADMINISTRATION_H\
ISTORY_DATABASE_\
USER_CREATION SY\
SRES_CONST_ADMIN\
ISTRATION_HISTOR\
Y_DATABASE_USER_\
CREATION_ACTION \
SYSRES_CONST_ADM\
INISTRATION_HIST\
ORY_DATABASE_USE\
R_DELETION SYSRE\
S_CONST_ADMINIST\
RATION_HISTORY_D\
ATABASE_USER_DEL\
ETION_ACTION SYS\
RES_CONST_ADMINI\
STRATION_HISTORY\
_DELETION_COMP_A\
CTION SYSRES_CON\
ST_ADMINISTRATIO\
N_HISTORY_DELETI\
ON_COMP_ACTION_C\
ODE SYSRES_CONST\
_ADMINISTRATION_\
HISTORY_DELETION\
_GROUP_ACTION SY\
SRES_CONST_ADMIN\
ISTRATION_HISTOR\
Y_DELETION_GROUP\
_ACTION_CODE SYS\
RES_CONST_ADMINI\
STRATION_HISTORY\
_DELETION_USER_A\
CTION SYSRES_CON\
ST_ADMINISTRATIO\
N_HISTORY_DELETI\
ON_USER_ACTION_C\
ODE SYSRES_CONST\
_ADMINISTRATION_\
HISTORY_DELETION\
_USER_FROM_GROUP\
_ACTION SYSRES_C\
ONST_ADMINISTRAT\
ION_HISTORY_DELE\
TION_USER_FROM_G\
ROUP_ACTION_CODE\
 SYSRES_CONST_AD\
MINISTRATION_HIS\
TORY_GRANTING_FI\
LTERER_ACTION SY\
SRES_CONST_ADMIN\
ISTRATION_HISTOR\
Y_GRANTING_FILTE\
RER_ACTION_CODE \
SYSRES_CONST_ADM\
INISTRATION_HIST\
ORY_GRANTING_FIL\
TERER_RESTRICTIO\
N_ACTION SYSRES_\
CONST_ADMINISTRA\
TION_HISTORY_GRA\
NTING_FILTERER_R\
ESTRICTION_ACTIO\
N_CODE SYSRES_CO\
NST_ADMINISTRATI\
ON_HISTORY_GRANT\
ING_PRIVILEGE_AC\
TION SYSRES_CONS\
T_ADMINISTRATION\
_HISTORY_GRANTIN\
G_PRIVILEGE_ACTI\
ON_CODE SYSRES_C\
ONST_ADMINISTRAT\
ION_HISTORY_GRAN\
TING_RIGHTS_ACTI\
ON SYSRES_CONST_\
ADMINISTRATION_H\
ISTORY_GRANTING_\
RIGHTS_ACTION_CO\
DE SYSRES_CONST_\
ADMINISTRATION_H\
ISTORY_IS_MAIN_S\
ERVER_CHANGED_AC\
TION SYSRES_CONS\
T_ADMINISTRATION\
_HISTORY_IS_MAIN\
_SERVER_CHANGED_\
ACTION_CODE SYSR\
ES_CONST_ADMINIS\
TRATION_HISTORY_\
IS_PUBLIC_CHANGE\
D_ACTION SYSRES_\
CONST_ADMINISTRA\
TION_HISTORY_IS_\
PUBLIC_CHANGED_A\
CTION_CODE SYSRE\
S_CONST_ADMINIST\
RATION_HISTORY_R\
EMOVING_FILTERER\
_ACTION SYSRES_C\
ONST_ADMINISTRAT\
ION_HISTORY_REMO\
VING_FILTERER_AC\
TION_CODE SYSRES\
_CONST_ADMINISTR\
ATION_HISTORY_RE\
MOVING_FILTERER_\
RESTRICTION_ACTI\
ON SYSRES_CONST_\
ADMINISTRATION_H\
ISTORY_REMOVING_\
FILTERER_RESTRIC\
TION_ACTION_CODE\
 SYSRES_CONST_AD\
MINISTRATION_HIS\
TORY_REMOVING_PR\
IVILEGE_ACTION S\
YSRES_CONST_ADMI\
NISTRATION_HISTO\
RY_REMOVING_PRIV\
ILEGE_ACTION_COD\
E SYSRES_CONST_A\
DMINISTRATION_HI\
STORY_REMOVING_R\
IGHTS_ACTION SYS\
RES_CONST_ADMINI\
STRATION_HISTORY\
_REMOVING_RIGHTS\
_ACTION_CODE SYS\
RES_CONST_ADMINI\
STRATION_HISTORY\
_SERVER_LOGIN_CR\
EATION SYSRES_CO\
NST_ADMINISTRATI\
ON_HISTORY_SERVE\
R_LOGIN_CREATION\
_ACTION SYSRES_C\
ONST_ADMINISTRAT\
ION_HISTORY_SERV\
ER_LOGIN_DELETIO\
N SYSRES_CONST_A\
DMINISTRATION_HI\
STORY_SERVER_LOG\
IN_DELETION_ACTI\
ON SYSRES_CONST_\
ADMINISTRATION_H\
ISTORY_UPDATING_\
CATEGORY_ACTION \
SYSRES_CONST_ADM\
INISTRATION_HIST\
ORY_UPDATING_CAT\
EGORY_ACTION_COD\
E SYSRES_CONST_A\
DMINISTRATION_HI\
STORY_UPDATING_C\
OMP_TITLE_ACTION\
 SYSRES_CONST_AD\
MINISTRATION_HIS\
TORY_UPDATING_CO\
MP_TITLE_ACTION_\
CODE SYSRES_CONS\
T_ADMINISTRATION\
_HISTORY_UPDATIN\
G_FULL_NAME_ACTI\
ON SYSRES_CONST_\
ADMINISTRATION_H\
ISTORY_UPDATING_\
FULL_NAME_ACTION\
_CODE SYSRES_CON\
ST_ADMINISTRATIO\
N_HISTORY_UPDATI\
NG_GROUP_ACTION \
SYSRES_CONST_ADM\
INISTRATION_HIST\
ORY_UPDATING_GRO\
UP_ACTION_CODE S\
YSRES_CONST_ADMI\
NISTRATION_HISTO\
RY_UPDATING_PARE\
NT_GROUP_ACTION \
SYSRES_CONST_ADM\
INISTRATION_HIST\
ORY_UPDATING_PAR\
ENT_GROUP_ACTION\
_CODE SYSRES_CON\
ST_ADMINISTRATIO\
N_HISTORY_UPDATI\
NG_USER_AUTH_TYP\
E_ACTION SYSRES_\
CONST_ADMINISTRA\
TION_HISTORY_UPD\
ATING_USER_AUTH_\
TYPE_ACTION_CODE\
 SYSRES_CONST_AD\
MINISTRATION_HIS\
TORY_UPDATING_US\
ER_LOGIN_ACTION \
SYSRES_CONST_ADM\
INISTRATION_HIST\
ORY_UPDATING_USE\
R_LOGIN_ACTION_C\
ODE SYSRES_CONST\
_ADMINISTRATION_\
HISTORY_UPDATING\
_USER_STATUS_ACT\
ION SYSRES_CONST\
_ADMINISTRATION_\
HISTORY_UPDATING\
_USER_STATUS_ACT\
ION_CODE SYSRES_\
CONST_ADMINISTRA\
TION_HISTORY_USE\
R_PASSWORD_CHANG\
E SYSRES_CONST_A\
DMINISTRATION_HI\
STORY_USER_PASSW\
ORD_CHANGE_ACTIO\
N SYSRES_CONST_A\
LL_ACCEPT_CONDIT\
ION_RUS SYSRES_C\
ONST_ALL_USERS_G\
ROUP SYSRES_CONS\
T_ALL_USERS_GROU\
P_NAME SYSRES_CO\
NST_ALL_USERS_SE\
RVER_GROUP_NAME \
SYSRES_CONST_ALL\
OWED_ACCESS_TYPE\
_CODE SYSRES_CON\
ST_ALLOWED_ACCES\
S_TYPE_NAME SYSR\
ES_CONST_APP_VIE\
WER_TYPE_REQUISI\
TE_CODE SYSRES_C\
ONST_APPROVING_S\
IGNATURE_NAME SY\
SRES_CONST_APPRO\
VING_SIGNATURE_R\
EQUISITE_CODE SY\
SRES_CONST_ASSIS\
TANT_SUBSTITUE_T\
YPE SYSRES_CONST\
_ASSISTANT_SUBST\
ITUE_TYPE_CODE S\
YSRES_CONST_ATTA\
CH_TYPE_COMPONEN\
T_TOKEN SYSRES_C\
ONST_ATTACH_TYPE\
_DOC SYSRES_CONS\
T_ATTACH_TYPE_ED\
OC SYSRES_CONST_\
ATTACH_TYPE_FOLD\
ER SYSRES_CONST_\
ATTACH_TYPE_JOB \
SYSRES_CONST_ATT\
ACH_TYPE_REFEREN\
CE SYSRES_CONST_\
ATTACH_TYPE_TASK\
 SYSRES_CONST_AU\
TH_ENCODED_PASSW\
ORD SYSRES_CONST\
_AUTH_ENCODED_PA\
SSWORD_CODE SYSR\
ES_CONST_AUTH_NO\
VELL SYSRES_CONS\
T_AUTH_PASSWORD \
SYSRES_CONST_AUT\
H_PASSWORD_CODE \
SYSRES_CONST_AUT\
H_WINDOWS SYSRES\
_CONST_AUTHENTIC\
ATING_SIGNATURE_\
NAME SYSRES_CONS\
T_AUTHENTICATING\
_SIGNATURE_REQUI\
SITE_CODE SYSRES\
_CONST_AUTO_ENUM\
_METHOD_FLAG SYS\
RES_CONST_AUTO_N\
UMERATION_CODE S\
YSRES_CONST_AUTO\
_STRONG_ENUM_MET\
HOD_FLAG SYSRES_\
CONST_AUTOTEXT_N\
AME_REQUISITE_CO\
DE SYSRES_CONST_\
AUTOTEXT_TEXT_RE\
QUISITE_CODE SYS\
RES_CONST_AUTOTE\
XT_USAGE_ALL SYS\
RES_CONST_AUTOTE\
XT_USAGE_ALL_COD\
E SYSRES_CONST_A\
UTOTEXT_USAGE_SI\
GN SYSRES_CONST_\
AUTOTEXT_USAGE_S\
IGN_CODE SYSRES_\
CONST_AUTOTEXT_U\
SAGE_WORK SYSRES\
_CONST_AUTOTEXT_\
USAGE_WORK_CODE \
SYSRES_CONST_AUT\
OTEXT_USE_ANYWHE\
RE_CODE SYSRES_C\
ONST_AUTOTEXT_US\
E_ON_SIGNING_COD\
E SYSRES_CONST_A\
UTOTEXT_USE_ON_W\
ORK_CODE SYSRES_\
CONST_BEGIN_DATE\
_REQUISITE_CODE \
SYSRES_CONST_BLA\
CK_LIFE_CYCLE_ST\
AGE_FONT_COLOR S\
YSRES_CONST_BLUE\
_LIFE_CYCLE_STAG\
E_FONT_COLOR SYS\
RES_CONST_BTN_PA\
RT SYSRES_CONST_\
CALCULATED_ROLE_\
TYPE_CODE SYSRES\
_CONST_CALL_TYPE\
_VARIABLE_BUTTON\
_VALUE SYSRES_CO\
NST_CALL_TYPE_VA\
RIABLE_PROGRAM_V\
ALUE SYSRES_CONS\
T_CANCEL_MESSAGE\
_FUNCTION_RESULT\
 SYSRES_CONST_CA\
RD_PART SYSRES_C\
ONST_CARD_REFERE\
NCE_MODE_NAME SY\
SRES_CONST_CERTI\
FICATE_TYPE_REQU\
ISITE_ENCRYPT_VA\
LUE SYSRES_CONST\
_CERTIFICATE_TYP\
E_REQUISITE_SIGN\
_AND_ENCRYPT_VAL\
UE SYSRES_CONST_\
CERTIFICATE_TYPE\
_REQUISITE_SIGN_\
VALUE SYSRES_CON\
ST_CHECK_PARAM_V\
ALUE_DATE_PARAM_\
TYPE SYSRES_CONS\
T_CHECK_PARAM_VA\
LUE_FLOAT_PARAM_\
TYPE SYSRES_CONS\
T_CHECK_PARAM_VA\
LUE_INTEGER_PARA\
M_TYPE SYSRES_CO\
NST_CHECK_PARAM_\
VALUE_PICK_PARAM\
_TYPE SYSRES_CON\
ST_CHECK_PARAM_V\
ALUE_REEFRENCE_P\
ARAM_TYPE SYSRES\
_CONST_CLOSED_RE\
CORD_FLAG_VALUE_\
FEMININE SYSRES_\
CONST_CLOSED_REC\
ORD_FLAG_VALUE_M\
ASCULINE SYSRES_\
CONST_CODE_COMPO\
NENT_TYPE_ADMIN \
SYSRES_CONST_COD\
E_COMPONENT_TYPE\
_DEVELOPER SYSRE\
S_CONST_CODE_COM\
PONENT_TYPE_DOCS\
 SYSRES_CONST_CO\
DE_COMPONENT_TYP\
E_EDOC_CARDS SYS\
RES_CONST_CODE_C\
OMPONENT_TYPE_EX\
TERNAL_EXECUTABL\
E SYSRES_CONST_C\
ODE_COMPONENT_TY\
PE_OTHER SYSRES_\
CONST_CODE_COMPO\
NENT_TYPE_REFERE\
NCE SYSRES_CONST\
_CODE_COMPONENT_\
TYPE_REPORT SYSR\
ES_CONST_CODE_CO\
MPONENT_TYPE_SCR\
IPT SYSRES_CONST\
_CODE_COMPONENT_\
TYPE_URL SYSRES_\
CONST_CODE_REQUI\
SITE_ACCESS SYSR\
ES_CONST_CODE_RE\
QUISITE_CODE SYS\
RES_CONST_CODE_R\
EQUISITE_COMPONE\
NT SYSRES_CONST_\
CODE_REQUISITE_D\
ESCRIPTION SYSRE\
S_CONST_CODE_REQ\
UISITE_EXCLUDE_C\
OMPONENT SYSRES_\
CONST_CODE_REQUI\
SITE_RECORD SYSR\
ES_CONST_COMMENT\
_REQ_CODE SYSRES\
_CONST_COMMON_SE\
TTINGS_REQUISITE\
_CODE SYSRES_CON\
ST_COMP_CODE_GRD\
 SYSRES_CONST_CO\
MPONENT_GROUP_TY\
PE_REQUISITE_COD\
E SYSRES_CONST_C\
OMPONENT_TYPE_AD\
MIN_COMPONENTS S\
YSRES_CONST_COMP\
ONENT_TYPE_DEVEL\
OPER_COMPONENTS \
SYSRES_CONST_COM\
PONENT_TYPE_DOCS\
 SYSRES_CONST_CO\
MPONENT_TYPE_EDO\
C_CARDS SYSRES_C\
ONST_COMPONENT_T\
YPE_EDOCS SYSRES\
_CONST_COMPONENT\
_TYPE_EXTERNAL_E\
XECUTABLE SYSRES\
_CONST_COMPONENT\
_TYPE_OTHER SYSR\
ES_CONST_COMPONE\
NT_TYPE_REFERENC\
E_TYPES SYSRES_C\
ONST_COMPONENT_T\
YPE_REFERENCES S\
YSRES_CONST_COMP\
ONENT_TYPE_REPOR\
TS SYSRES_CONST_\
COMPONENT_TYPE_S\
CRIPTS SYSRES_CO\
NST_COMPONENT_TY\
PE_URL SYSRES_CO\
NST_COMPONENTS_R\
EMOTE_SERVERS_VI\
EW_CODE SYSRES_C\
ONST_CONDITION_B\
LOCK_DESCRIPTION\
 SYSRES_CONST_CO\
NST_FIRM_STATUS_\
COMMON SYSRES_CO\
NST_CONST_FIRM_S\
TATUS_INDIVIDUAL\
 SYSRES_CONST_CO\
NST_NEGATIVE_VAL\
UE SYSRES_CONST_\
CONST_POSITIVE_V\
ALUE SYSRES_CONS\
T_CONST_SERVER_S\
TATUS_DONT_REPLI\
CATE SYSRES_CONS\
T_CONST_SERVER_S\
TATUS_REPLICATE \
SYSRES_CONST_CON\
TENTS_REQUISITE_\
CODE SYSRES_CONS\
T_DATA_TYPE_BOOL\
EAN SYSRES_CONST\
_DATA_TYPE_DATE \
SYSRES_CONST_DAT\
A_TYPE_FLOAT SYS\
RES_CONST_DATA_T\
YPE_INTEGER SYSR\
ES_CONST_DATA_TY\
PE_PICK SYSRES_C\
ONST_DATA_TYPE_R\
EFERENCE SYSRES_\
CONST_DATA_TYPE_\
STRING SYSRES_CO\
NST_DATA_TYPE_TE\
XT SYSRES_CONST_\
DATA_TYPE_VARIAN\
T SYSRES_CONST_D\
ATE_CLOSE_REQ_CO\
DE SYSRES_CONST_\
DATE_FORMAT_DATE\
_ONLY_CHAR SYSRE\
S_CONST_DATE_OPE\
N_REQ_CODE SYSRE\
S_CONST_DATE_REQ\
UISITE SYSRES_CO\
NST_DATE_REQUISI\
TE_CODE SYSRES_C\
ONST_DATE_REQUIS\
ITE_NAME SYSRES_\
CONST_DATE_REQUI\
SITE_TYPE SYSRES\
_CONST_DATE_TYPE\
_CHAR SYSRES_CON\
ST_DATETIME_FORM\
AT_VALUE SYSRES_\
CONST_DEA_ACCESS\
_RIGHTS_ACTION_C\
ODE SYSRES_CONST\
_DESCRIPTION_LOC\
ALIZE_ID_REQUISI\
TE_CODE SYSRES_C\
ONST_DESCRIPTION\
_REQUISITE_CODE \
SYSRES_CONST_DET\
1_PART SYSRES_CO\
NST_DET2_PART SY\
SRES_CONST_DET3_\
PART SYSRES_CONS\
T_DET4_PART SYSR\
ES_CONST_DET5_PA\
RT SYSRES_CONST_\
DET6_PART SYSRES\
_CONST_DETAIL_DA\
TASET_KEY_REQUIS\
ITE_CODE SYSRES_\
CONST_DETAIL_PIC\
K_REQUISITE_CODE\
 SYSRES_CONST_DE\
TAIL_REQ_CODE SY\
SRES_CONST_DO_NO\
T_USE_ACCESS_TYP\
E_CODE SYSRES_CO\
NST_DO_NOT_USE_A\
CCESS_TYPE_NAME \
SYSRES_CONST_DO_\
NOT_USE_ON_VIEW_\
ACCESS_TYPE_CODE\
 SYSRES_CONST_DO\
_NOT_USE_ON_VIEW\
_ACCESS_TYPE_NAM\
E SYSRES_CONST_D\
OCUMENT_STORAGES\
_CODE SYSRES_CON\
ST_DOCUMENT_TEMP\
LATES_TYPE_NAME \
SYSRES_CONST_DOU\
BLE_REQUISITE_CO\
DE SYSRES_CONST_\
EDITOR_CLOSE_FIL\
E_OBSERV_TYPE_CO\
DE SYSRES_CONST_\
EDITOR_CLOSE_PRO\
CESS_OBSERV_TYPE\
_CODE SYSRES_CON\
ST_EDITOR_TYPE_R\
EQUISITE_CODE SY\
SRES_CONST_EDITO\
RS_APPLICATION_N\
AME_REQUISITE_CO\
DE SYSRES_CONST_\
EDITORS_CREATE_S\
EVERAL_PROCESSES\
_REQUISITE_CODE \
SYSRES_CONST_EDI\
TORS_EXTENSION_R\
EQUISITE_CODE SY\
SRES_CONST_EDITO\
RS_OBSERVER_BY_P\
ROCESS_TYPE SYSR\
ES_CONST_EDITORS\
_REFERENCE_CODE \
SYSRES_CONST_EDI\
TORS_REPLACE_SPE\
C_CHARS_REQUISIT\
E_CODE SYSRES_CO\
NST_EDITORS_USE_\
PLUGINS_REQUISIT\
E_CODE SYSRES_CO\
NST_EDITORS_VIEW\
_DOCUMENT_OPENED\
_TO_EDIT_CODE SY\
SRES_CONST_EDOC_\
CARD_TYPE_REQUIS\
ITE_CODE SYSRES_\
CONST_EDOC_CARD_\
TYPES_LINK_REQUI\
SITE_CODE SYSRES\
_CONST_EDOC_CERT\
IFICATE_AND_PASS\
WORD_ENCODE_CODE\
 SYSRES_CONST_ED\
OC_CERTIFICATE_E\
NCODE_CODE SYSRE\
S_CONST_EDOC_DAT\
E_REQUISITE_CODE\
 SYSRES_CONST_ED\
OC_KIND_REFERENC\
E_CODE SYSRES_CO\
NST_EDOC_KINDS_B\
Y_TEMPLATE_ACTIO\
N_CODE SYSRES_CO\
NST_EDOC_MANAGE_\
ACCESS_CODE SYSR\
ES_CONST_EDOC_NO\
NE_ENCODE_CODE S\
YSRES_CONST_EDOC\
_NUMBER_REQUISIT\
E_CODE SYSRES_CO\
NST_EDOC_PASSWOR\
D_ENCODE_CODE SY\
SRES_CONST_EDOC_\
READONLY_ACCESS_\
CODE SYSRES_CONS\
T_EDOC_SHELL_LIF\
E_TYPE_VIEW_VALU\
E SYSRES_CONST_E\
DOC_SIZE_RESTRIC\
TION_PRIORITY_RE\
QUISITE_CODE SYS\
RES_CONST_EDOC_S\
TORAGE_CHECK_ACC\
ESS_RIGHTS_REQUI\
SITE_CODE SYSRES\
_CONST_EDOC_STOR\
AGE_COMPUTER_NAM\
E_REQUISITE_CODE\
 SYSRES_CONST_ED\
OC_STORAGE_DATAB\
ASE_NAME_REQUISI\
TE_CODE SYSRES_C\
ONST_EDOC_STORAG\
E_EDIT_IN_STORAG\
E_REQUISITE_CODE\
 SYSRES_CONST_ED\
OC_STORAGE_LOCAL\
_PATH_REQUISITE_\
CODE SYSRES_CONS\
T_EDOC_STORAGE_S\
HARED_SOURCE_NAM\
E_REQUISITE_CODE\
 SYSRES_CONST_ED\
OC_TEMPLATE_REQU\
ISITE_CODE SYSRE\
S_CONST_EDOC_TYP\
ES_REFERENCE_COD\
E SYSRES_CONST_E\
DOC_VERSION_ACTI\
VE_STAGE_CODE SY\
SRES_CONST_EDOC_\
VERSION_DESIGN_S\
TAGE_CODE SYSRES\
_CONST_EDOC_VERS\
ION_OBSOLETE_STA\
GE_CODE SYSRES_C\
ONST_EDOC_WRITE_\
ACCES_CODE SYSRE\
S_CONST_EDOCUMEN\
T_CARD_REQUISITE\
S_REFERENCE_CODE\
_SELECTED_REQUIS\
ITE SYSRES_CONST\
_ENCODE_CERTIFIC\
ATE_TYPE_CODE SY\
SRES_CONST_END_D\
ATE_REQUISITE_CO\
DE SYSRES_CONST_\
ENUMERATION_TYPE\
_REQUISITE_CODE \
SYSRES_CONST_EXE\
CUTE_ACCESS_RIGH\
TS_TYPE_CODE SYS\
RES_CONST_EXECUT\
IVE_FILE_STORAGE\
_TYPE SYSRES_CON\
ST_EXIST_CONST S\
YSRES_CONST_EXIS\
T_VALUE SYSRES_C\
ONST_EXPORT_LOCK\
_TYPE_ASK SYSRES\
_CONST_EXPORT_LO\
CK_TYPE_WITH_LOC\
K SYSRES_CONST_E\
XPORT_LOCK_TYPE_\
WITHOUT_LOCK SYS\
RES_CONST_EXPORT\
_VERSION_TYPE_AS\
K SYSRES_CONST_E\
XPORT_VERSION_TY\
PE_LAST SYSRES_C\
ONST_EXPORT_VERS\
ION_TYPE_LAST_AC\
TIVE SYSRES_CONS\
T_EXTENSION_REQU\
ISITE_CODE SYSRE\
S_CONST_FILTER_N\
AME_REQUISITE_CO\
DE SYSRES_CONST_\
FILTER_REQUISITE\
_CODE SYSRES_CON\
ST_FILTER_TYPE_C\
OMMON_CODE SYSRE\
S_CONST_FILTER_T\
YPE_COMMON_NAME \
SYSRES_CONST_FIL\
TER_TYPE_USER_CO\
DE SYSRES_CONST_\
FILTER_TYPE_USER\
_NAME SYSRES_CON\
ST_FILTER_VALUE_\
REQUISITE_NAME S\
YSRES_CONST_FLOA\
T_NUMBER_FORMAT_\
CHAR SYSRES_CONS\
T_FLOAT_REQUISIT\
E_TYPE SYSRES_CO\
NST_FOLDER_AUTHO\
R_VALUE SYSRES_C\
ONST_FOLDER_KIND\
_ANY_OBJECTS SYS\
RES_CONST_FOLDER\
_KIND_COMPONENTS\
 SYSRES_CONST_FO\
LDER_KIND_EDOCS \
SYSRES_CONST_FOL\
DER_KIND_JOBS SY\
SRES_CONST_FOLDE\
R_KIND_TASKS SYS\
RES_CONST_FOLDER\
_TYPE_COMMON SYS\
RES_CONST_FOLDER\
_TYPE_COMPONENT \
SYSRES_CONST_FOL\
DER_TYPE_FAVORIT\
ES SYSRES_CONST_\
FOLDER_TYPE_INBO\
X SYSRES_CONST_F\
OLDER_TYPE_OUTBO\
X SYSRES_CONST_F\
OLDER_TYPE_QUICK\
_LAUNCH SYSRES_C\
ONST_FOLDER_TYPE\
_SEARCH SYSRES_C\
ONST_FOLDER_TYPE\
_SHORTCUTS SYSRE\
S_CONST_FOLDER_T\
YPE_USER SYSRES_\
CONST_FROM_DICTI\
ONARY_ENUM_METHO\
D_FLAG SYSRES_CO\
NST_FULL_SUBSTIT\
UTE_TYPE SYSRES_\
CONST_FULL_SUBST\
ITUTE_TYPE_CODE \
SYSRES_CONST_FUN\
CTION_CANCEL_RES\
ULT SYSRES_CONST\
_FUNCTION_CATEGO\
RY_SYSTEM SYSRES\
_CONST_FUNCTION_\
CATEGORY_USER SY\
SRES_CONST_FUNCT\
ION_FAILURE_RESU\
LT SYSRES_CONST_\
FUNCTION_SAVE_RE\
SULT SYSRES_CONS\
T_GENERATED_REQU\
ISITE SYSRES_CON\
ST_GREEN_LIFE_CY\
CLE_STAGE_FONT_C\
OLOR SYSRES_CONS\
T_GROUP_ACCOUNT_\
TYPE_VALUE_CODE \
SYSRES_CONST_GRO\
UP_CATEGORY_NORM\
AL_CODE SYSRES_C\
ONST_GROUP_CATEG\
ORY_NORMAL_NAME \
SYSRES_CONST_GRO\
UP_CATEGORY_SERV\
ICE_CODE SYSRES_\
CONST_GROUP_CATE\
GORY_SERVICE_NAM\
E SYSRES_CONST_G\
ROUP_COMMON_CATE\
GORY_FIELD_VALUE\
 SYSRES_CONST_GR\
OUP_FULL_NAME_RE\
QUISITE_CODE SYS\
RES_CONST_GROUP_\
NAME_REQUISITE_C\
ODE SYSRES_CONST\
_GROUP_RIGHTS_T_\
REQUISITE_CODE S\
YSRES_CONST_GROU\
P_SERVER_CODES_R\
EQUISITE_CODE SY\
SRES_CONST_GROUP\
_SERVER_NAME_REQ\
UISITE_CODE SYSR\
ES_CONST_GROUP_S\
ERVICE_CATEGORY_\
FIELD_VALUE SYSR\
ES_CONST_GROUP_U\
SER_REQUISITE_CO\
DE SYSRES_CONST_\
GROUPS_REFERENCE\
_CODE SYSRES_CON\
ST_GROUPS_REQUIS\
ITE_CODE SYSRES_\
CONST_HIDDEN_MOD\
E_NAME SYSRES_CO\
NST_HIGH_LVL_REQ\
UISITE_CODE SYSR\
ES_CONST_HISTORY\
_ACTION_CREATE_C\
ODE SYSRES_CONST\
_HISTORY_ACTION_\
DELETE_CODE SYSR\
ES_CONST_HISTORY\
_ACTION_EDIT_COD\
E SYSRES_CONST_H\
OUR_CHAR SYSRES_\
CONST_ID_REQUISI\
TE_CODE SYSRES_C\
ONST_IDSPS_REQUI\
SITE_CODE SYSRES\
_CONST_IMAGE_MOD\
E_COLOR SYSRES_C\
ONST_IMAGE_MODE_\
GREYSCALE SYSRES\
_CONST_IMAGE_MOD\
E_MONOCHROME SYS\
RES_CONST_IMPORT\
ANCE_HIGH SYSRES\
_CONST_IMPORTANC\
E_LOW SYSRES_CON\
ST_IMPORTANCE_NO\
RMAL SYSRES_CONS\
T_IN_DESIGN_VERS\
ION_STATE_PICK_V\
ALUE SYSRES_CONS\
T_INCOMING_WORK_\
RULE_TYPE_CODE S\
YSRES_CONST_INT_\
REQUISITE SYSRES\
_CONST_INT_REQUI\
SITE_TYPE SYSRES\
_CONST_INTEGER_N\
UMBER_FORMAT_CHA\
R SYSRES_CONST_I\
NTEGER_TYPE_CHAR\
 SYSRES_CONST_IS\
_GENERATED_REQUI\
SITE_NEGATIVE_VA\
LUE SYSRES_CONST\
_IS_PUBLIC_ROLE_\
REQUISITE_CODE S\
YSRES_CONST_IS_R\
EMOTE_USER_NEGAT\
IVE_VALUE SYSRES\
_CONST_IS_REMOTE\
_USER_POSITIVE_V\
ALUE SYSRES_CONS\
T_IS_STORED_REQU\
ISITE_NEGATIVE_V\
ALUE SYSRES_CONS\
T_IS_STORED_REQU\
ISITE_STORED_VAL\
UE SYSRES_CONST_\
ITALIC_LIFE_CYCL\
E_STAGE_DRAW_STY\
LE SYSRES_CONST_\
JOB_BLOCK_DESCRI\
PTION SYSRES_CON\
ST_JOB_KIND_CONT\
ROL_JOB SYSRES_C\
ONST_JOB_KIND_JO\
B SYSRES_CONST_J\
OB_KIND_NOTICE S\
YSRES_CONST_JOB_\
STATE_ABORTED SY\
SRES_CONST_JOB_S\
TATE_COMPLETE SY\
SRES_CONST_JOB_S\
TATE_WORKING SYS\
RES_CONST_KIND_R\
EQUISITE_CODE SY\
SRES_CONST_KIND_\
REQUISITE_NAME S\
YSRES_CONST_KIND\
S_CREATE_SHADOW_\
COPIES_REQUISITE\
_CODE SYSRES_CON\
ST_KINDS_DEFAULT\
_EDOC_LIFE_STAGE\
_REQUISITE_CODE \
SYSRES_CONST_KIN\
DS_EDOC_ALL_TEPL\
ATES_ALLOWED_REQ\
UISITE_CODE SYSR\
ES_CONST_KINDS_E\
DOC_ALLOW_LIFE_C\
YCLE_STAGE_CHANG\
ING_REQUISITE_CO\
DE SYSRES_CONST_\
KINDS_EDOC_ALLOW\
_MULTIPLE_ACTIVE\
_VERSIONS_REQUIS\
ITE_CODE SYSRES_\
CONST_KINDS_EDOC\
_SHARE_ACCES_RIG\
HTS_BY_DEFAULT_C\
ODE SYSRES_CONST\
_KINDS_EDOC_TEMP\
LATE_REQUISITE_C\
ODE SYSRES_CONST\
_KINDS_EDOC_TYPE\
_REQUISITE_CODE \
SYSRES_CONST_KIN\
DS_SIGNERS_REQUI\
SITES_CODE SYSRE\
S_CONST_KOD_INPU\
T_TYPE SYSRES_CO\
NST_LAST_UPDATE_\
DATE_REQUISITE_C\
ODE SYSRES_CONST\
_LIFE_CYCLE_STAR\
T_STAGE_REQUISIT\
E_CODE SYSRES_CO\
NST_LILAC_LIFE_C\
YCLE_STAGE_FONT_\
COLOR SYSRES_CON\
ST_LINK_OBJECT_K\
IND_COMPONENT SY\
SRES_CONST_LINK_\
OBJECT_KIND_DOCU\
MENT SYSRES_CONS\
T_LINK_OBJECT_KI\
ND_EDOC SYSRES_C\
ONST_LINK_OBJECT\
_KIND_FOLDER SYS\
RES_CONST_LINK_O\
BJECT_KIND_JOB S\
YSRES_CONST_LINK\
_OBJECT_KIND_REF\
ERENCE SYSRES_CO\
NST_LINK_OBJECT_\
KIND_TASK SYSRES\
_CONST_LINK_REF_\
TYPE_REQUISITE_C\
ODE SYSRES_CONST\
_LIST_REFERENCE_\
MODE_NAME SYSRES\
_CONST_LOCALIZAT\
ION_DICTIONARY_M\
AIN_VIEW_CODE SY\
SRES_CONST_MAIN_\
VIEW_CODE SYSRES\
_CONST_MANUAL_EN\
UM_METHOD_FLAG S\
YSRES_CONST_MAST\
ER_COMP_TYPE_REQ\
UISITE_CODE SYSR\
ES_CONST_MASTER_\
TABLE_REC_ID_REQ\
UISITE_CODE SYSR\
ES_CONST_MAXIMIZ\
ED_MODE_NAME SYS\
RES_CONST_ME_VAL\
UE SYSRES_CONST_\
MESSAGE_ATTENTIO\
N_CAPTION SYSRES\
_CONST_MESSAGE_C\
ONFIRMATION_CAPT\
ION SYSRES_CONST\
_MESSAGE_ERROR_C\
APTION SYSRES_CO\
NST_MESSAGE_INFO\
RMATION_CAPTION \
SYSRES_CONST_MIN\
IMIZED_MODE_NAME\
 SYSRES_CONST_MI\
NUTE_CHAR SYSRES\
_CONST_MODULE_RE\
QUISITE_CODE SYS\
RES_CONST_MONITO\
RING_BLOCK_DESCR\
IPTION SYSRES_CO\
NST_MONTH_FORMAT\
_VALUE SYSRES_CO\
NST_NAME_LOCALIZ\
E_ID_REQUISITE_C\
ODE SYSRES_CONST\
_NAME_REQUISITE_\
CODE SYSRES_CONS\
T_NAME_SINGULAR_\
REQUISITE_CODE S\
YSRES_CONST_NAME\
AN_INPUT_TYPE SY\
SRES_CONST_NEGAT\
IVE_PICK_VALUE S\
YSRES_CONST_NEGA\
TIVE_VALUE SYSRE\
S_CONST_NO SYSRE\
S_CONST_NO_PICK_\
VALUE SYSRES_CON\
ST_NO_SIGNATURE_\
REQUISITE_CODE S\
YSRES_CONST_NO_V\
ALUE SYSRES_CONS\
T_NONE_ACCESS_RI\
GHTS_TYPE_CODE S\
YSRES_CONST_NONO\
PERATING_RECORD_\
FLAG_VALUE SYSRE\
S_CONST_NONOPERA\
TING_RECORD_FLAG\
_VALUE_MASCULINE\
 SYSRES_CONST_NO\
RMAL_ACCESS_RIGH\
TS_TYPE_CODE SYS\
RES_CONST_NORMAL\
_LIFE_CYCLE_STAG\
E_DRAW_STYLE SYS\
RES_CONST_NORMAL\
_MODE_NAME SYSRE\
S_CONST_NOT_ALLO\
WED_ACCESS_TYPE_\
CODE SYSRES_CONS\
T_NOT_ALLOWED_AC\
CESS_TYPE_NAME S\
YSRES_CONST_NOTE\
_REQUISITE_CODE \
SYSRES_CONST_NOT\
ICE_BLOCK_DESCRI\
PTION SYSRES_CON\
ST_NUM_REQUISITE\
 SYSRES_CONST_NU\
M_STR_REQUISITE_\
CODE SYSRES_CONS\
T_NUMERATION_AUT\
O_NOT_STRONG SYS\
RES_CONST_NUMERA\
TION_AUTO_STRONG\
 SYSRES_CONST_NU\
MERATION_FROM_DI\
CTONARY SYSRES_C\
ONST_NUMERATION_\
MANUAL SYSRES_CO\
NST_NUMERIC_TYPE\
_CHAR SYSRES_CON\
ST_NUMREQ_REQUIS\
ITE_CODE SYSRES_\
CONST_OBSOLETE_V\
ERSION_STATE_PIC\
K_VALUE SYSRES_C\
ONST_OPERATING_R\
ECORD_FLAG_VALUE\
 SYSRES_CONST_OP\
ERATING_RECORD_F\
LAG_VALUE_CODE S\
YSRES_CONST_OPER\
ATING_RECORD_FLA\
G_VALUE_FEMININE\
 SYSRES_CONST_OP\
ERATING_RECORD_F\
LAG_VALUE_MASCUL\
INE SYSRES_CONST\
_OPTIONAL_FORM_C\
OMP_REQCODE_PREF\
IX SYSRES_CONST_\
ORANGE_LIFE_CYCL\
E_STAGE_FONT_COL\
OR SYSRES_CONST_\
ORIGINALREF_REQU\
ISITE_CODE SYSRE\
S_CONST_OURFIRM_\
REF_CODE SYSRES_\
CONST_OURFIRM_RE\
QUISITE_CODE SYS\
RES_CONST_OURFIR\
M_VAR SYSRES_CON\
ST_OUTGOING_WORK\
_RULE_TYPE_CODE \
SYSRES_CONST_PIC\
K_NEGATIVE_RESUL\
T SYSRES_CONST_P\
ICK_POSITIVE_RES\
ULT SYSRES_CONST\
_PICK_REQUISITE \
SYSRES_CONST_PIC\
K_REQUISITE_TYPE\
 SYSRES_CONST_PI\
CK_TYPE_CHAR SYS\
RES_CONST_PLAN_S\
TATUS_REQUISITE_\
CODE SYSRES_CONS\
T_PLATFORM_VERSI\
ON_COMMENT SYSRE\
S_CONST_PLUGINS_\
SETTINGS_DESCRIP\
TION_REQUISITE_C\
ODE SYSRES_CONST\
_POSITIVE_PICK_V\
ALUE SYSRES_CONS\
T_POWER_TO_CREAT\
E_ACTION_CODE SY\
SRES_CONST_POWER\
_TO_SIGN_ACTION_\
CODE SYSRES_CONS\
T_PRIORITY_REQUI\
SITE_CODE SYSRES\
_CONST_QUALIFIED\
_TASK_TYPE SYSRE\
S_CONST_QUALIFIE\
D_TASK_TYPE_CODE\
 SYSRES_CONST_RE\
CSTAT_REQUISITE_\
CODE SYSRES_CONS\
T_RED_LIFE_CYCLE\
_STAGE_FONT_COLO\
R SYSRES_CONST_R\
EF_ID_T_REF_TYPE\
_REQUISITE_CODE \
SYSRES_CONST_REF\
_REQUISITE SYSRE\
S_CONST_REF_REQU\
ISITE_TYPE SYSRE\
S_CONST_REF_REQU\
ISITES_REFERENCE\
_CODE_SELECTED_R\
EQUISITE SYSRES_\
CONST_REFERENCE_\
RECORD_HISTORY_C\
REATE_ACTION_COD\
E SYSRES_CONST_R\
EFERENCE_RECORD_\
HISTORY_DELETE_A\
CTION_CODE SYSRE\
S_CONST_REFERENC\
E_RECORD_HISTORY\
_MODIFY_ACTION_C\
ODE SYSRES_CONST\
_REFERENCE_TYPE_\
CHAR SYSRES_CONS\
T_REFERENCE_TYPE\
_REQUISITE_NAME \
SYSRES_CONST_REF\
ERENCES_ADD_PARA\
MS_REQUISITE_COD\
E SYSRES_CONST_R\
EFERENCES_DISPLA\
Y_REQUISITE_REQU\
ISITE_CODE SYSRE\
S_CONST_REMOTE_S\
ERVER_STATUS_WOR\
KING SYSRES_CONS\
T_REMOTE_SERVER_\
TYPE_MAIN SYSRES\
_CONST_REMOTE_SE\
RVER_TYPE_SECOND\
ARY SYSRES_CONST\
_REMOTE_USER_FLA\
G_VALUE_CODE SYS\
RES_CONST_REPORT\
_APP_EDITOR_INTE\
RNAL SYSRES_CONS\
T_REPORT_BASE_RE\
PORT_ID_REQUISIT\
E_CODE SYSRES_CO\
NST_REPORT_BASE_\
REPORT_REQUISITE\
_CODE SYSRES_CON\
ST_REPORT_SCRIPT\
_REQUISITE_CODE \
SYSRES_CONST_REP\
ORT_TEMPLATE_REQ\
UISITE_CODE SYSR\
ES_CONST_REPORT_\
VIEWER_CODE_REQU\
ISITE_CODE SYSRE\
S_CONST_REQ_ALLO\
W_COMPONENT_DEFA\
ULT_VALUE SYSRES\
_CONST_REQ_ALLOW\
_RECORD_DEFAULT_\
VALUE SYSRES_CON\
ST_REQ_ALLOW_SER\
VER_COMPONENT_DE\
FAULT_VALUE SYSR\
ES_CONST_REQ_MOD\
E_AVAILABLE_CODE\
 SYSRES_CONST_RE\
Q_MODE_EDIT_CODE\
 SYSRES_CONST_RE\
Q_MODE_HIDDEN_CO\
DE SYSRES_CONST_\
REQ_MODE_NOT_AVA\
ILABLE_CODE SYSR\
ES_CONST_REQ_MOD\
E_VIEW_CODE SYSR\
ES_CONST_REQ_NUM\
BER_REQUISITE_CO\
DE SYSRES_CONST_\
REQ_SECTION_VALU\
E SYSRES_CONST_R\
EQ_TYPE_VALUE SY\
SRES_CONST_REQUI\
SITE_FORMAT_BY_U\
NIT SYSRES_CONST\
_REQUISITE_FORMA\
T_DATE_FULL SYSR\
ES_CONST_REQUISI\
TE_FORMAT_DATE_T\
IME SYSRES_CONST\
_REQUISITE_FORMA\
T_LEFT SYSRES_CO\
NST_REQUISITE_FO\
RMAT_RIGHT SYSRE\
S_CONST_REQUISIT\
E_FORMAT_WITHOUT\
_UNIT SYSRES_CON\
ST_REQUISITE_NUM\
BER_REQUISITE_CO\
DE SYSRES_CONST_\
REQUISITE_SECTIO\
N_ACTIONS SYSRES\
_CONST_REQUISITE\
_SECTION_BUTTON \
SYSRES_CONST_REQ\
UISITE_SECTION_B\
UTTONS SYSRES_CO\
NST_REQUISITE_SE\
CTION_CARD SYSRE\
S_CONST_REQUISIT\
E_SECTION_TABLE \
SYSRES_CONST_REQ\
UISITE_SECTION_T\
ABLE10 SYSRES_CO\
NST_REQUISITE_SE\
CTION_TABLE11 SY\
SRES_CONST_REQUI\
SITE_SECTION_TAB\
LE12 SYSRES_CONS\
T_REQUISITE_SECT\
ION_TABLE13 SYSR\
ES_CONST_REQUISI\
TE_SECTION_TABLE\
14 SYSRES_CONST_\
REQUISITE_SECTIO\
N_TABLE15 SYSRES\
_CONST_REQUISITE\
_SECTION_TABLE16\
 SYSRES_CONST_RE\
QUISITE_SECTION_\
TABLE17 SYSRES_C\
ONST_REQUISITE_S\
ECTION_TABLE18 S\
YSRES_CONST_REQU\
ISITE_SECTION_TA\
BLE19 SYSRES_CON\
ST_REQUISITE_SEC\
TION_TABLE2 SYSR\
ES_CONST_REQUISI\
TE_SECTION_TABLE\
20 SYSRES_CONST_\
REQUISITE_SECTIO\
N_TABLE21 SYSRES\
_CONST_REQUISITE\
_SECTION_TABLE22\
 SYSRES_CONST_RE\
QUISITE_SECTION_\
TABLE23 SYSRES_C\
ONST_REQUISITE_S\
ECTION_TABLE24 S\
YSRES_CONST_REQU\
ISITE_SECTION_TA\
BLE3 SYSRES_CONS\
T_REQUISITE_SECT\
ION_TABLE4 SYSRE\
S_CONST_REQUISIT\
E_SECTION_TABLE5\
 SYSRES_CONST_RE\
QUISITE_SECTION_\
TABLE6 SYSRES_CO\
NST_REQUISITE_SE\
CTION_TABLE7 SYS\
RES_CONST_REQUIS\
ITE_SECTION_TABL\
E8 SYSRES_CONST_\
REQUISITE_SECTIO\
N_TABLE9 SYSRES_\
CONST_REQUISITES\
_PSEUDOREFERENCE\
_REQUISITE_NUMBE\
R_REQUISITE_CODE\
 SYSRES_CONST_RI\
GHT_ALIGNMENT_CO\
DE SYSRES_CONST_\
ROLES_REFERENCE_\
CODE SYSRES_CONS\
T_ROUTE_STEP_AFT\
ER_RUS SYSRES_CO\
NST_ROUTE_STEP_A\
ND_CONDITION_RUS\
 SYSRES_CONST_RO\
UTE_STEP_OR_COND\
ITION_RUS SYSRES\
_CONST_ROUTE_TYP\
E_COMPLEX SYSRES\
_CONST_ROUTE_TYP\
E_PARALLEL SYSRE\
S_CONST_ROUTE_TY\
PE_SERIAL SYSRES\
_CONST_SBDATASET\
DESC_NEGATIVE_VA\
LUE SYSRES_CONST\
_SBDATASETDESC_P\
OSITIVE_VALUE SY\
SRES_CONST_SBVIE\
WSDESC_POSITIVE_\
VALUE SYSRES_CON\
ST_SCRIPT_BLOCK_\
DESCRIPTION SYSR\
ES_CONST_SEARCH_\
BY_TEXT_REQUISIT\
E_CODE SYSRES_CO\
NST_SEARCHES_COM\
PONENT_CONTENT S\
YSRES_CONST_SEAR\
CHES_CRITERIA_AC\
TION_NAME SYSRES\
_CONST_SEARCHES_\
EDOC_CONTENT SYS\
RES_CONST_SEARCH\
ES_FOLDER_CONTEN\
T SYSRES_CONST_S\
EARCHES_JOB_CONT\
ENT SYSRES_CONST\
_SEARCHES_REFERE\
NCE_CODE SYSRES_\
CONST_SEARCHES_T\
ASK_CONTENT SYSR\
ES_CONST_SECOND_\
CHAR SYSRES_CONS\
T_SECTION_REQUIS\
ITE_ACTIONS_VALU\
E SYSRES_CONST_S\
ECTION_REQUISITE\
_CARD_VALUE SYSR\
ES_CONST_SECTION\
_REQUISITE_CODE \
SYSRES_CONST_SEC\
TION_REQUISITE_D\
ETAIL_1_VALUE SY\
SRES_CONST_SECTI\
ON_REQUISITE_DET\
AIL_2_VALUE SYSR\
ES_CONST_SECTION\
_REQUISITE_DETAI\
L_3_VALUE SYSRES\
_CONST_SECTION_R\
EQUISITE_DETAIL_\
4_VALUE SYSRES_C\
ONST_SECTION_REQ\
UISITE_DETAIL_5_\
VALUE SYSRES_CON\
ST_SECTION_REQUI\
SITE_DETAIL_6_VA\
LUE SYSRES_CONST\
_SELECT_REFERENC\
E_MODE_NAME SYSR\
ES_CONST_SELECT_\
TYPE_SELECTABLE \
SYSRES_CONST_SEL\
ECT_TYPE_SELECTA\
BLE_ONLY_CHILD S\
YSRES_CONST_SELE\
CT_TYPE_SELECTAB\
LE_WITH_CHILD SY\
SRES_CONST_SELEC\
T_TYPE_UNSLECTAB\
LE SYSRES_CONST_\
SERVER_TYPE_MAIN\
 SYSRES_CONST_SE\
RVICE_USER_CATEG\
ORY_FIELD_VALUE \
SYSRES_CONST_SET\
TINGS_USER_REQUI\
SITE_CODE SYSRES\
_CONST_SIGNATURE\
_AND_ENCODE_CERT\
IFICATE_TYPE_COD\
E SYSRES_CONST_S\
IGNATURE_CERTIFI\
CATE_TYPE_CODE S\
YSRES_CONST_SING\
ULAR_TITLE_REQUI\
SITE_CODE SYSRES\
_CONST_SQL_SERVE\
R_AUTHENTIFICATI\
ON_FLAG_VALUE_CO\
DE SYSRES_CONST_\
SQL_SERVER_ENCOD\
E_AUTHENTIFICATI\
ON_FLAG_VALUE_CO\
DE SYSRES_CONST_\
STANDART_ROUTE_R\
EFERENCE_CODE SY\
SRES_CONST_STAND\
ART_ROUTE_REFERE\
NCE_COMMENT_REQU\
ISITE_CODE SYSRE\
S_CONST_STANDART\
_ROUTES_GROUPS_R\
EFERENCE_CODE SY\
SRES_CONST_STATE\
_REQ_NAME SYSRES\
_CONST_STATE_REQ\
UISITE_ACTIVE_VA\
LUE SYSRES_CONST\
_STATE_REQUISITE\
_CLOSED_VALUE SY\
SRES_CONST_STATE\
_REQUISITE_CODE \
SYSRES_CONST_STA\
TIC_ROLE_TYPE_CO\
DE SYSRES_CONST_\
STATUS_PLAN_DEFA\
ULT_VALUE SYSRES\
_CONST_STATUS_VA\
LUE_AUTOCLEANING\
 SYSRES_CONST_ST\
ATUS_VALUE_BLUE_\
SQUARE SYSRES_CO\
NST_STATUS_VALUE\
_COMPLETE SYSRES\
_CONST_STATUS_VA\
LUE_GREEN_SQUARE\
 SYSRES_CONST_ST\
ATUS_VALUE_ORANG\
E_SQUARE SYSRES_\
CONST_STATUS_VAL\
UE_PURPLE_SQUARE\
 SYSRES_CONST_ST\
ATUS_VALUE_RED_S\
QUARE SYSRES_CON\
ST_STATUS_VALUE_\
SUSPEND SYSRES_C\
ONST_STATUS_VALU\
E_YELLOW_SQUARE \
SYSRES_CONST_STD\
ROUTE_SHOW_TO_US\
ERS_REQUISITE_CO\
DE SYSRES_CONST_\
STORAGE_TYPE_FIL\
E SYSRES_CONST_S\
TORAGE_TYPE_SQL_\
SERVER SYSRES_CO\
NST_STR_REQUISIT\
E SYSRES_CONST_S\
TRIKEOUT_LIFE_CY\
CLE_STAGE_DRAW_S\
TYLE SYSRES_CONS\
T_STRING_FORMAT_\
LEFT_ALIGN_CHAR \
SYSRES_CONST_STR\
ING_FORMAT_RIGHT\
_ALIGN_CHAR SYSR\
ES_CONST_STRING_\
REQUISITE_CODE S\
YSRES_CONST_STRI\
NG_REQUISITE_TYP\
E SYSRES_CONST_S\
TRING_TYPE_CHAR \
SYSRES_CONST_SUB\
STITUTES_PSEUDOR\
EFERENCE_CODE SY\
SRES_CONST_SUBTA\
SK_BLOCK_DESCRIP\
TION SYSRES_CONS\
T_SYSTEM_SETTING\
_CURRENT_USER_PA\
RAM_VALUE SYSRES\
_CONST_SYSTEM_SE\
TTING_EMPTY_VALU\
E_PARAM_VALUE SY\
SRES_CONST_SYSTE\
M_VERSION_COMMEN\
T SYSRES_CONST_T\
ASK_ACCESS_TYPE_\
ALL SYSRES_CONST\
_TASK_ACCESS_TYP\
E_ALL_MEMBERS SY\
SRES_CONST_TASK_\
ACCESS_TYPE_MANU\
AL SYSRES_CONST_\
TASK_ENCODE_TYPE\
_CERTIFICATION S\
YSRES_CONST_TASK\
_ENCODE_TYPE_CER\
TIFICATION_AND_P\
ASSWORD SYSRES_C\
ONST_TASK_ENCODE\
_TYPE_NONE SYSRE\
S_CONST_TASK_ENC\
ODE_TYPE_PASSWOR\
D SYSRES_CONST_T\
ASK_ROUTE_ALL_CO\
NDITION SYSRES_C\
ONST_TASK_ROUTE_\
AND_CONDITION SY\
SRES_CONST_TASK_\
ROUTE_OR_CONDITI\
ON SYSRES_CONST_\
TASK_STATE_ABORT\
ED SYSRES_CONST_\
TASK_STATE_COMPL\
ETE SYSRES_CONST\
_TASK_STATE_CONT\
INUED SYSRES_CON\
ST_TASK_STATE_CO\
NTROL SYSRES_CON\
ST_TASK_STATE_IN\
IT SYSRES_CONST_\
TASK_STATE_WORKI\
NG SYSRES_CONST_\
TASK_TITLE SYSRE\
S_CONST_TASK_TYP\
ES_GROUPS_REFERE\
NCE_CODE SYSRES_\
CONST_TASK_TYPES\
_REFERENCE_CODE \
SYSRES_CONST_TEM\
PLATES_REFERENCE\
_CODE SYSRES_CON\
ST_TEST_DATE_REQ\
UISITE_NAME SYSR\
ES_CONST_TEST_DE\
V_DATABASE_NAME \
SYSRES_CONST_TES\
T_DEV_SYSTEM_COD\
E SYSRES_CONST_T\
EST_EDMS_DATABAS\
E_NAME SYSRES_CO\
NST_TEST_EDMS_MA\
IN_CODE SYSRES_C\
ONST_TEST_EDMS_M\
AIN_DB_NAME SYSR\
ES_CONST_TEST_ED\
MS_SECOND_CODE S\
YSRES_CONST_TEST\
_EDMS_SECOND_DB_\
NAME SYSRES_CONS\
T_TEST_EDMS_SYST\
EM_CODE SYSRES_C\
ONST_TEST_NUMERI\
C_REQUISITE_NAME\
 SYSRES_CONST_TE\
XT_REQUISITE SYS\
RES_CONST_TEXT_R\
EQUISITE_CODE SY\
SRES_CONST_TEXT_\
REQUISITE_TYPE S\
YSRES_CONST_TEXT\
_TYPE_CHAR SYSRE\
S_CONST_TYPE_COD\
E_REQUISITE_CODE\
 SYSRES_CONST_TY\
PE_REQUISITE_COD\
E SYSRES_CONST_U\
NDEFINED_LIFE_CY\
CLE_STAGE_FONT_C\
OLOR SYSRES_CONS\
T_UNITS_SECTION_\
ID_REQUISITE_COD\
E SYSRES_CONST_U\
NITS_SECTION_REQ\
UISITE_CODE SYSR\
ES_CONST_UNOPERA\
TING_RECORD_FLAG\
_VALUE_CODE SYSR\
ES_CONST_UNSTORE\
D_DATA_REQUISITE\
_CODE SYSRES_CON\
ST_UNSTORED_DATA\
_REQUISITE_NAME \
SYSRES_CONST_USE\
_ACCESS_TYPE_COD\
E SYSRES_CONST_U\
SE_ACCESS_TYPE_N\
AME SYSRES_CONST\
_USER_ACCOUNT_TY\
PE_VALUE_CODE SY\
SRES_CONST_USER_\
ADDITIONAL_INFOR\
MATION_REQUISITE\
_CODE SYSRES_CON\
ST_USER_AND_GROU\
P_ID_FROM_PSEUDO\
REFERENCE_REQUIS\
ITE_CODE SYSRES_\
CONST_USER_CATEG\
ORY_NORMAL SYSRE\
S_CONST_USER_CER\
TIFICATE_REQUISI\
TE_CODE SYSRES_C\
ONST_USER_CERTIF\
ICATE_STATE_REQU\
ISITE_CODE SYSRE\
S_CONST_USER_CER\
TIFICATE_SUBJECT\
_NAME_REQUISITE_\
CODE SYSRES_CONS\
T_USER_CERTIFICA\
TE_THUMBPRINT_RE\
QUISITE_CODE SYS\
RES_CONST_USER_C\
OMMON_CATEGORY S\
YSRES_CONST_USER\
_COMMON_CATEGORY\
_CODE SYSRES_CON\
ST_USER_FULL_NAM\
E_REQUISITE_CODE\
 SYSRES_CONST_US\
ER_GROUP_TYPE_RE\
QUISITE_CODE SYS\
RES_CONST_USER_L\
OGIN_REQUISITE_C\
ODE SYSRES_CONST\
_USER_REMOTE_CON\
TROLLER_REQUISIT\
E_CODE SYSRES_CO\
NST_USER_REMOTE_\
SYSTEM_REQUISITE\
_CODE SYSRES_CON\
ST_USER_RIGHTS_T\
_REQUISITE_CODE \
SYSRES_CONST_USE\
R_SERVER_NAME_RE\
QUISITE_CODE SYS\
RES_CONST_USER_S\
ERVICE_CATEGORY \
SYSRES_CONST_USE\
R_SERVICE_CATEGO\
RY_CODE SYSRES_C\
ONST_USER_STATUS\
_ADMINISTRATOR_C\
ODE SYSRES_CONST\
_USER_STATUS_ADM\
INISTRATOR_NAME \
SYSRES_CONST_USE\
R_STATUS_DEVELOP\
ER_CODE SYSRES_C\
ONST_USER_STATUS\
_DEVELOPER_NAME \
SYSRES_CONST_USE\
R_STATUS_DISABLE\
D_CODE SYSRES_CO\
NST_USER_STATUS_\
DISABLED_NAME SY\
SRES_CONST_USER_\
STATUS_SYSTEM_DE\
VELOPER_CODE SYS\
RES_CONST_USER_S\
TATUS_USER_CODE \
SYSRES_CONST_USE\
R_STATUS_USER_NA\
ME SYSRES_CONST_\
USER_STATUS_USER\
_NAME_DEPRECATED\
 SYSRES_CONST_US\
ER_TYPE_FIELD_VA\
LUE_USER SYSRES_\
CONST_USER_TYPE_\
REQUISITE_CODE S\
YSRES_CONST_USER\
S_CONTROLLER_REQ\
UISITE_CODE SYSR\
ES_CONST_USERS_I\
S_MAIN_SERVER_RE\
QUISITE_CODE SYS\
RES_CONST_USERS_\
REFERENCE_CODE S\
YSRES_CONST_USER\
S_REGISTRATION_C\
ERTIFICATES_ACTI\
ON_NAME SYSRES_C\
ONST_USERS_REQUI\
SITE_CODE SYSRES\
_CONST_USERS_SYS\
TEM_REQUISITE_CO\
DE SYSRES_CONST_\
USERS_USER_ACCES\
S_RIGHTS_TYPR_RE\
QUISITE_CODE SYS\
RES_CONST_USERS_\
USER_AUTHENTICAT\
ION_REQUISITE_CO\
DE SYSRES_CONST_\
USERS_USER_COMPO\
NENT_REQUISITE_C\
ODE SYSRES_CONST\
_USERS_USER_GROU\
P_REQUISITE_CODE\
 SYSRES_CONST_US\
ERS_VIEW_CERTIFI\
CATES_ACTION_NAM\
E SYSRES_CONST_V\
IEW_DEFAULT_CODE\
 SYSRES_CONST_VI\
EW_DEFAULT_NAME \
SYSRES_CONST_VIE\
WER_REQUISITE_CO\
DE SYSRES_CONST_\
WAITING_BLOCK_DE\
SCRIPTION SYSRES\
_CONST_WIZARD_FO\
RM_LABEL_TEST_ST\
RING  SYSRES_CON\
ST_WIZARD_QUERY_\
PARAM_HEIGHT_ETA\
LON_STRING SYSRE\
S_CONST_WIZARD_R\
EFERENCE_COMMENT\
_REQUISITE_CODE \
SYSRES_CONST_WOR\
K_RULES_DESCRIPT\
ION_REQUISITE_CO\
DE SYSRES_CONST_\
WORK_TIME_CALEND\
AR_REFERENCE_COD\
E SYSRES_CONST_W\
ORK_WORKFLOW_HAR\
D_ROUTE_TYPE_VAL\
UE SYSRES_CONST_\
WORK_WORKFLOW_HA\
RD_ROUTE_TYPE_VA\
LUE_CODE SYSRES_\
CONST_WORK_WORKF\
LOW_HARD_ROUTE_T\
YPE_VALUE_CODE_R\
US SYSRES_CONST_\
WORK_WORKFLOW_SO\
FT_ROUTE_TYPE_VA\
LUE_CODE_RUS SYS\
RES_CONST_WORKFL\
OW_ROUTE_TYPR_HA\
RD SYSRES_CONST_\
WORKFLOW_ROUTE_T\
YPR_SOFT SYSRES_\
CONST_XML_ENCODI\
NG SYSRES_CONST_\
XREC_STAT_REQUIS\
ITE_CODE SYSRES_\
CONST_XRECID_FIE\
LD_NAME SYSRES_C\
ONST_YES SYSRES_\
CONST_YES_NO_2_R\
EQUISITE_CODE SY\
SRES_CONST_YES_N\
O_REQUISITE_CODE\
 SYSRES_CONST_YE\
S_NO_T_REF_TYPE_\
REQUISITE_CODE S\
YSRES_CONST_YES_\
PICK_VALUE SYSRE\
S_CONST_YES_VALU\
E CR FALSE nil N\
O_VALUE NULL TAB\
 TRUE YES_VALUE \
ADMINISTRATORS_G\
ROUP_NAME CUSTOM\
IZERS_GROUP_NAME\
 DEVELOPERS_GROU\
P_NAME SERVICE_U\
SERS_GROUP_NAME \
DECISION_BLOCK_F\
IRST_OPERAND_PRO\
PERTY DECISION_B\
LOCK_NAME_PROPER\
TY DECISION_BLOC\
K_OPERATION_PROP\
ERTY DECISION_BL\
OCK_RESULT_TYPE_\
PROPERTY DECISIO\
N_BLOCK_SECOND_O\
PERAND_PROPERTY \
ANY_FILE_EXTENTI\
ON COMPRESSED_DO\
CUMENT_EXTENSION\
 EXTENDED_DOCUME\
NT_EXTENSION SHO\
RT_COMPRESSED_DO\
CUMENT_EXTENSION\
 SHORT_EXTENDED_\
DOCUMENT_EXTENSI\
ON JOB_BLOCK_ABO\
RT_DEADLINE_PROP\
ERTY JOB_BLOCK_A\
FTER_FINISH_EVEN\
T JOB_BLOCK_AFTE\
R_QUERY_PARAMETE\
RS_EVENT JOB_BLO\
CK_ATTACHMENT_PR\
OPERTY JOB_BLOCK\
_ATTACHMENTS_RIG\
HTS_GROUP_PROPER\
TY JOB_BLOCK_ATT\
ACHMENTS_RIGHTS_\
TYPE_PROPERTY JO\
B_BLOCK_BEFORE_Q\
UERY_PARAMETERS_\
EVENT JOB_BLOCK_\
BEFORE_START_EVE\
NT JOB_BLOCK_CRE\
ATED_JOBS_PROPER\
TY JOB_BLOCK_DEA\
DLINE_PROPERTY J\
OB_BLOCK_EXECUTI\
ON_RESULTS_PROPE\
RTY JOB_BLOCK_IS\
_PARALLEL_PROPER\
TY JOB_BLOCK_IS_\
RELATIVE_ABORT_D\
EADLINE_PROPERTY\
 JOB_BLOCK_IS_RE\
LATIVE_DEADLINE_\
PROPERTY JOB_BLO\
CK_JOB_TEXT_PROP\
ERTY JOB_BLOCK_N\
AME_PROPERTY JOB\
_BLOCK_NEED_SIGN\
_ON_PERFORM_PROP\
ERTY JOB_BLOCK_P\
ERFORMER_PROPERT\
Y JOB_BLOCK_RELA\
TIVE_ABORT_DEADL\
INE_TYPE_PROPERT\
Y JOB_BLOCK_RELA\
TIVE_DEADLINE_TY\
PE_PROPERTY JOB_\
BLOCK_SUBJECT_PR\
OPERTY ENGLISH_L\
ANGUAGE_CODE RUS\
SIAN_LANGUAGE_CO\
DE smHidden smMa\
ximized smMinimi\
zed smNormal wmN\
o wmYes COMPONEN\
T_TOKEN_LINK_KIN\
D DOCUMENT_LINK_\
KIND EDOCUMENT_L\
INK_KIND FOLDER_\
LINK_KIND JOB_LI\
NK_KIND REFERENC\
E_LINK_KIND TASK\
_LINK_KIND COMPO\
NENT_TOKEN_LOCK_\
TYPE EDOCUMENT_V\
ERSION_LOCK_TYPE\
 MONITOR_BLOCK_A\
FTER_FINISH_EVEN\
T MONITOR_BLOCK_\
BEFORE_START_EVE\
NT MONITOR_BLOCK\
_DEADLINE_PROPER\
TY MONITOR_BLOCK\
_INTERVAL_PROPER\
TY MONITOR_BLOCK\
_INTERVAL_TYPE_P\
ROPERTY MONITOR_\
BLOCK_IS_RELATIV\
E_DEADLINE_PROPE\
RTY MONITOR_BLOC\
K_NAME_PROPERTY \
MONITOR_BLOCK_RE\
LATIVE_DEADLINE_\
TYPE_PROPERTY MO\
NITOR_BLOCK_SEAR\
CH_SCRIPT_PROPER\
TY NOTICE_BLOCK_\
AFTER_FINISH_EVE\
NT NOTICE_BLOCK_\
ATTACHMENT_PROPE\
RTY NOTICE_BLOCK\
_ATTACHMENTS_RIG\
HTS_GROUP_PROPER\
TY NOTICE_BLOCK_\
ATTACHMENTS_RIGH\
TS_TYPE_PROPERTY\
 NOTICE_BLOCK_BE\
FORE_START_EVENT\
 NOTICE_BLOCK_CR\
EATED_NOTICES_PR\
OPERTY NOTICE_BL\
OCK_DEADLINE_PRO\
PERTY NOTICE_BLO\
CK_IS_RELATIVE_D\
EADLINE_PROPERTY\
 NOTICE_BLOCK_NA\
ME_PROPERTY NOTI\
CE_BLOCK_NOTICE_\
TEXT_PROPERTY NO\
TICE_BLOCK_PERFO\
RMER_PROPERTY NO\
TICE_BLOCK_RELAT\
IVE_DEADLINE_TYP\
E_PROPERTY NOTIC\
E_BLOCK_SUBJECT_\
PROPERTY dseAfte\
rCancel dseAfter\
Close dseAfterDe\
lete dseAfterDel\
eteOutOfTransact\
ion dseAfterInse\
rt dseAfterOpen \
dseAfterScroll d\
seAfterUpdate ds\
eAfterUpdateOutO\
fTransaction dse\
BeforeCancel dse\
BeforeClose dseB\
eforeDelete dseB\
eforeDetailUpdat\
e dseBeforeInser\
t dseBeforeOpen \
dseBeforeUpdate \
dseOnAnyRequisit\
eChange dseOnClo\
seRecord dseOnDe\
leteError dseOnO\
penRecord dseOnP\
repareUpdate dse\
OnUpdateError ds\
eOnUpdateRatifie\
dRecord dseOnVal\
idDelete dseOnVa\
lidUpdate reOnCh\
ange reOnChangeV\
alues SELECTION_\
BEGIN_ROUTE_EVEN\
T SELECTION_END_\
ROUTE_EVENT CURR\
ENT_PERIOD_IS_RE\
QUIRED PREVIOUS_\
CARD_TYPE_NAME S\
HOW_RECORD_PROPE\
RTIES_FORM ACCES\
S_RIGHTS_SETTING\
_DIALOG_CODE ADM\
INISTRATOR_USER_\
CODE ANALYTIC_RE\
PORT_TYPE asrtHi\
deLocal asrtHide\
Remote CALCULATE\
D_ROLE_TYPE_CODE\
 COMPONENTS_REFE\
RENCE_DEVELOPER_\
VIEW_CODE DCTS_T\
EST_PROTOCOLS_FO\
LDER_PATH E_EDOC\
_VERSION_ALREADY\
_APPROVINGLY_SIG\
NED E_EDOC_VERSI\
ON_ALREADY_APPRO\
VINGLY_SIGNED_BY\
_USER E_EDOC_VER\
SION_ALREDY_SIGN\
ED E_EDOC_VERSIO\
N_ALREDY_SIGNED_\
BY_USER EDOC_TYP\
ES_CODE_REQUISIT\
E_FIELD_NAME EDO\
CUMENTS_ALIAS_NA\
ME FILES_FOLDER_\
PATH FILTER_OPER\
ANDS_DELIMITER F\
ILTER_OPERATIONS\
_DELIMITER FORMC\
ARD_NAME FORMLIS\
T_NAME GET_EXTEN\
DED_DOCUMENT_EXT\
ENSION_CREATION_\
MODE GET_EXTENDE\
D_DOCUMENT_EXTEN\
SION_IMPORT_MODE\
 INTEGRATED_REPO\
RT_TYPE IS_BUILD\
ER_APPLICATION_R\
OLE IS_BUILDER_A\
PPLICATION_ROLE2\
 IS_BUILDER_USER\
S ISBSYSDEV LOG_\
FOLDER_PATH mbCa\
ncel mbNo mbNoTo\
All mbOK mbYes m\
bYesToAll MEMORY\
_DATASET_DESRIPT\
IONS_FILENAME mr\
No mrNoToAll mrY\
es mrYesToAll MU\
LTIPLE_SELECT_DI\
ALOG_CODE NONOPE\
RATING_RECORD_FL\
AG_FEMININE NONO\
PERATING_RECORD_\
FLAG_MASCULINE O\
PERATING_RECORD_\
FLAG_FEMININE OP\
ERATING_RECORD_F\
LAG_MASCULINE PR\
OFILING_SETTINGS\
_COMMON_SETTINGS\
_CODE_VALUE PROG\
RAM_INITIATED_LO\
OKUP_ACTION ratD\
elete ratEdit ra\
tInsert REPORT_T\
YPE REQUIRED_PIC\
K_VALUES_VARIABL\
E rmCard rmList \
SBRTE_PROGID_DEV\
 SBRTE_PROGID_RE\
LEASE STATIC_ROL\
E_TYPE_CODE SUPP\
RESS_EMPTY_TEMPL\
ATE_CREATION SYS\
TEM_USER_CODE UP\
DATE_DIALOG_DATA\
SET USED_IN_OBJE\
CT_HINT_PARAM US\
ER_INITIATED_LOO\
KUP_ACTION USER_\
NAME_FORMAT USER\
_SELECTION_RESTR\
ICTIONS WORKFLOW\
_TEST_PROTOCOLS_\
FOLDER_PATH ELS_\
SUBTYPE_CONTROL_\
NAME ELS_FOLDER_\
KIND_CONTROL_NAM\
E REPEAT_PROCESS\
_CURRENT_OBJECT_\
EXCEPTION_NAME P\
RIVILEGE_COMPONE\
NT_FULL_ACCESS P\
RIVILEGE_DEVELOP\
MENT_EXPORT PRIV\
ILEGE_DEVELOPMEN\
T_IMPORT PRIVILE\
GE_DOCUMENT_DELE\
TE PRIVILEGE_ESD\
 PRIVILEGE_FOLDE\
R_DELETE PRIVILE\
GE_MANAGE_ACCESS\
_RIGHTS PRIVILEG\
E_MANAGE_REPLICA\
TION PRIVILEGE_M\
ANAGE_SESSION_SE\
RVER PRIVILEGE_O\
BJECT_FULL_ACCES\
S PRIVILEGE_OBJE\
CT_VIEW PRIVILEG\
E_RESERVE_LICENS\
E PRIVILEGE_SYST\
EM_CUSTOMIZE PRI\
VILEGE_SYSTEM_DE\
VELOP PRIVILEGE_\
SYSTEM_INSTALL P\
RIVILEGE_TASK_DE\
LETE PRIVILEGE_U\
SER_PLUGIN_SETTI\
NGS_CUSTOMIZE PR\
IVILEGES_PSEUDOR\
EFERENCE_CODE AC\
CESS_TYPES_PSEUD\
OREFERENCE_CODE \
ALL_AVAILABLE_CO\
MPONENTS_PSEUDOR\
EFERENCE_CODE AL\
L_AVAILABLE_PRIV\
ILEGES_PSEUDOREF\
ERENCE_CODE ALL_\
REPLICATE_COMPON\
ENTS_PSEUDOREFER\
ENCE_CODE AVAILA\
BLE_DEVELOPERS_C\
OMPONENTS_PSEUDO\
REFERENCE_CODE C\
OMPONENTS_PSEUDO\
REFERENCE_CODE F\
ILTRATER_SETTING\
S_CONFLICTS_PSEU\
DOREFERENCE_CODE\
 GROUPS_PSEUDORE\
FERENCE_CODE REC\
EIVE_PROTOCOL_PS\
EUDOREFERENCE_CO\
DE REFERENCE_REQ\
UISITE_PSEUDOREF\
ERENCE_CODE REFE\
RENCE_REQUISITES\
_PSEUDOREFERENCE\
_CODE REFTYPES_P\
SEUDOREFERENCE_C\
ODE REPLICATION_\
SEANCES_DIARY_PS\
EUDOREFERENCE_CO\
DE SEND_PROTOCOL\
_PSEUDOREFERENCE\
_CODE SUBSTITUTE\
S_PSEUDOREFERENC\
E_CODE SYSTEM_SE\
TTINGS_PSEUDOREF\
ERENCE_CODE UNIT\
S_PSEUDOREFERENC\
E_CODE USERS_PSE\
UDOREFERENCE_COD\
E VIEWERS_PSEUDO\
REFERENCE_CODE C\
ERTIFICATE_TYPE_\
ENCRYPT CERTIFIC\
ATE_TYPE_SIGN CE\
RTIFICATE_TYPE_S\
IGN_AND_ENCRYPT \
STORAGE_TYPE_FIL\
E STORAGE_TYPE_N\
AS_CIFS STORAGE_\
TYPE_SAPERION ST\
ORAGE_TYPE_SQL_S\
ERVER COMPTYPE2_\
REQUISITE_DOCUME\
NTS_VALUE COMPTY\
PE2_REQUISITE_TA\
SKS_VALUE COMPTY\
PE2_REQUISITE_FO\
LDERS_VALUE COMP\
TYPE2_REQUISITE_\
REFERENCES_VALUE\
 SYSREQ_CODE SYS\
REQ_COMPTYPE2 SY\
SREQ_CONST_AVAIL\
ABLE_FOR_WEB SYS\
REQ_CONST_COMMON\
_CODE SYSREQ_CON\
ST_COMMON_VALUE \
SYSREQ_CONST_FIR\
M_CODE SYSREQ_CO\
NST_FIRM_STATUS \
SYSREQ_CONST_FIR\
M_VALUE SYSREQ_C\
ONST_SERVER_STAT\
US SYSREQ_CONTEN\
TS SYSREQ_DATE_O\
PEN SYSREQ_DATE_\
CLOSE SYSREQ_DES\
CRIPTION SYSREQ_\
DESCRIPTION_LOCA\
LIZE_ID SYSREQ_D\
OUBLE SYSREQ_EDO\
C_ACCESS_TYPE SY\
SREQ_EDOC_AUTHOR\
 SYSREQ_EDOC_CRE\
ATED SYSREQ_EDOC\
_DELEGATE_RIGHTS\
_REQUISITE_CODE \
SYSREQ_EDOC_EDIT\
OR SYSREQ_EDOC_E\
NCODE_TYPE SYSRE\
Q_EDOC_ENCRYPTIO\
N_PLUGIN_NAME SY\
SREQ_EDOC_ENCRYP\
TION_PLUGIN_VERS\
ION SYSREQ_EDOC_\
EXPORT_DATE SYSR\
EQ_EDOC_EXPORTER\
 SYSREQ_EDOC_KIN\
D SYSREQ_EDOC_LI\
FE_STAGE_NAME SY\
SREQ_EDOC_LOCKED\
_FOR_SERVER_CODE\
 SYSREQ_EDOC_MOD\
IFIED SYSREQ_EDO\
C_NAME SYSREQ_ED\
OC_NOTE SYSREQ_E\
DOC_QUALIFIED_ID\
 SYSREQ_EDOC_SES\
SION_KEY SYSREQ_\
EDOC_SESSION_KEY\
_ENCRYPTION_PLUG\
IN_NAME SYSREQ_E\
DOC_SESSION_KEY_\
ENCRYPTION_PLUGI\
N_VERSION SYSREQ\
_EDOC_SIGNATURE_\
TYPE SYSREQ_EDOC\
_SIGNED SYSREQ_E\
DOC_STORAGE SYSR\
EQ_EDOC_STORAGES\
_ARCHIVE_STORAGE\
 SYSREQ_EDOC_STO\
RAGES_CHECK_RIGH\
TS SYSREQ_EDOC_S\
TORAGES_COMPUTER\
_NAME SYSREQ_EDO\
C_STORAGES_EDIT_\
IN_STORAGE SYSRE\
Q_EDOC_STORAGES_\
EXECUTIVE_STORAG\
E SYSREQ_EDOC_ST\
ORAGES_FUNCTION \
SYSREQ_EDOC_STOR\
AGES_INITIALIZED\
 SYSREQ_EDOC_STO\
RAGES_LOCAL_PATH\
 SYSREQ_EDOC_STO\
RAGES_SAPERION_D\
ATABASE_NAME SYS\
REQ_EDOC_STORAGE\
S_SEARCH_BY_TEXT\
 SYSREQ_EDOC_STO\
RAGES_SERVER_NAM\
E SYSREQ_EDOC_ST\
ORAGES_SHARED_SO\
URCE_NAME SYSREQ\
_EDOC_STORAGES_T\
YPE SYSREQ_EDOC_\
TEXT_MODIFIED SY\
SREQ_EDOC_TYPE_A\
CT_CODE SYSREQ_E\
DOC_TYPE_ACT_DES\
CRIPTION SYSREQ_\
EDOC_TYPE_ACT_DE\
SCRIPTION_LOCALI\
ZE_ID SYSREQ_EDO\
C_TYPE_ACT_ON_EX\
ECUTE SYSREQ_EDO\
C_TYPE_ACT_ON_EX\
ECUTE_EXISTS SYS\
REQ_EDOC_TYPE_AC\
T_SECTION SYSREQ\
_EDOC_TYPE_ADD_P\
ARAMS SYSREQ_EDO\
C_TYPE_COMMENT S\
YSREQ_EDOC_TYPE_\
EVENT_TEXT SYSRE\
Q_EDOC_TYPE_NAME\
_IN_SINGULAR SYS\
REQ_EDOC_TYPE_NA\
ME_IN_SINGULAR_L\
OCALIZE_ID SYSRE\
Q_EDOC_TYPE_NAME\
_LOCALIZE_ID SYS\
REQ_EDOC_TYPE_NU\
MERATION_METHOD \
SYSREQ_EDOC_TYPE\
_PSEUDO_REQUISIT\
E_CODE SYSREQ_ED\
OC_TYPE_REQ_CODE\
 SYSREQ_EDOC_TYP\
E_REQ_DESCRIPTIO\
N SYSREQ_EDOC_TY\
PE_REQ_DESCRIPTI\
ON_LOCALIZE_ID S\
YSREQ_EDOC_TYPE_\
REQ_IS_LEADING S\
YSREQ_EDOC_TYPE_\
REQ_IS_REQUIRED \
SYSREQ_EDOC_TYPE\
_REQ_NUMBER SYSR\
EQ_EDOC_TYPE_REQ\
_ON_CHANGE SYSRE\
Q_EDOC_TYPE_REQ_\
ON_CHANGE_EXISTS\
 SYSREQ_EDOC_TYP\
E_REQ_ON_SELECT \
SYSREQ_EDOC_TYPE\
_REQ_ON_SELECT_K\
IND SYSREQ_EDOC_\
TYPE_REQ_SECTION\
 SYSREQ_EDOC_TYP\
E_VIEW_CARD SYSR\
EQ_EDOC_TYPE_VIE\
W_CODE SYSREQ_ED\
OC_TYPE_VIEW_COM\
MENT SYSREQ_EDOC\
_TYPE_VIEW_IS_MA\
IN SYSREQ_EDOC_T\
YPE_VIEW_NAME SY\
SREQ_EDOC_TYPE_V\
IEW_NAME_LOCALIZ\
E_ID SYSREQ_EDOC\
_VERSION_AUTHOR \
SYSREQ_EDOC_VERS\
ION_CRC SYSREQ_E\
DOC_VERSION_DATA\
 SYSREQ_EDOC_VER\
SION_EDITOR SYSR\
EQ_EDOC_VERSION_\
EXPORT_DATE SYSR\
EQ_EDOC_VERSION_\
EXPORTER SYSREQ_\
EDOC_VERSION_HID\
DEN SYSREQ_EDOC_\
VERSION_LIFE_STA\
GE SYSREQ_EDOC_V\
ERSION_MODIFIED \
SYSREQ_EDOC_VERS\
ION_NOTE SYSREQ_\
EDOC_VERSION_SIG\
NATURE_TYPE SYSR\
EQ_EDOC_VERSION_\
SIGNED SYSREQ_ED\
OC_VERSION_SIZE \
SYSREQ_EDOC_VERS\
ION_SOURCE SYSRE\
Q_EDOC_VERSION_T\
EXT_MODIFIED SYS\
REQ_EDOCKIND_DEF\
AULT_VERSION_STA\
TE_CODE SYSREQ_F\
OLDER_KIND SYSRE\
Q_FUNC_CATEGORY \
SYSREQ_FUNC_COMM\
ENT SYSREQ_FUNC_\
GROUP SYSREQ_FUN\
C_GROUP_COMMENT \
SYSREQ_FUNC_GROU\
P_NUMBER SYSREQ_\
FUNC_HELP SYSREQ\
_FUNC_PARAM_DEF_\
VALUE SYSREQ_FUN\
C_PARAM_IDENT SY\
SREQ_FUNC_PARAM_\
NUMBER SYSREQ_FU\
NC_PARAM_TYPE SY\
SREQ_FUNC_TEXT S\
YSREQ_GROUP_CATE\
GORY SYSREQ_ID S\
YSREQ_LAST_UPDAT\
E SYSREQ_LEADER_\
REFERENCE SYSREQ\
_LINE_NUMBER SYS\
REQ_MAIN_RECORD_\
ID SYSREQ_NAME S\
YSREQ_NAME_LOCAL\
IZE_ID SYSREQ_NO\
TE SYSREQ_ORIGIN\
AL_RECORD SYSREQ\
_OUR_FIRM SYSREQ\
_PROFILING_SETTI\
NGS_BATCH_LOGING\
 SYSREQ_PROFILIN\
G_SETTINGS_BATCH\
_SIZE SYSREQ_PRO\
FILING_SETTINGS_\
PROFILING_ENABLE\
D SYSREQ_PROFILI\
NG_SETTINGS_SQL_\
PROFILING_ENABLE\
D SYSREQ_PROFILI\
NG_SETTINGS_STAR\
T_LOGGED SYSREQ_\
RECORD_STATUS SY\
SREQ_REF_REQ_FIE\
LD_NAME SYSREQ_R\
EF_REQ_FORMAT SY\
SREQ_REF_REQ_GEN\
ERATED SYSREQ_RE\
F_REQ_LENGTH SYS\
REQ_REF_REQ_PREC\
ISION SYSREQ_REF\
_REQ_REFERENCE S\
YSREQ_REF_REQ_SE\
CTION SYSREQ_REF\
_REQ_STORED SYSR\
EQ_REF_REQ_TOKEN\
S SYSREQ_REF_REQ\
_TYPE SYSREQ_REF\
_REQ_VIEW SYSREQ\
_REF_TYPE_ACT_CO\
DE SYSREQ_REF_TY\
PE_ACT_DESCRIPTI\
ON SYSREQ_REF_TY\
PE_ACT_DESCRIPTI\
ON_LOCALIZE_ID S\
YSREQ_REF_TYPE_A\
CT_ON_EXECUTE SY\
SREQ_REF_TYPE_AC\
T_ON_EXECUTE_EXI\
STS SYSREQ_REF_T\
YPE_ACT_SECTION \
SYSREQ_REF_TYPE_\
ADD_PARAMS SYSRE\
Q_REF_TYPE_COMME\
NT SYSREQ_REF_TY\
PE_COMMON_SETTIN\
GS SYSREQ_REF_TY\
PE_DISPLAY_REQUI\
SITE_NAME SYSREQ\
_REF_TYPE_EVENT_\
TEXT SYSREQ_REF_\
TYPE_MAIN_LEADIN\
G_REF SYSREQ_REF\
_TYPE_NAME_IN_SI\
NGULAR SYSREQ_RE\
F_TYPE_NAME_IN_S\
INGULAR_LOCALIZE\
_ID SYSREQ_REF_T\
YPE_NAME_LOCALIZ\
E_ID SYSREQ_REF_\
TYPE_NUMERATION_\
METHOD SYSREQ_RE\
F_TYPE_REQ_CODE \
SYSREQ_REF_TYPE_\
REQ_DESCRIPTION \
SYSREQ_REF_TYPE_\
REQ_DESCRIPTION_\
LOCALIZE_ID SYSR\
EQ_REF_TYPE_REQ_\
IS_CONTROL SYSRE\
Q_REF_TYPE_REQ_I\
S_FILTER SYSREQ_\
REF_TYPE_REQ_IS_\
LEADING SYSREQ_R\
EF_TYPE_REQ_IS_R\
EQUIRED SYSREQ_R\
EF_TYPE_REQ_NUMB\
ER SYSREQ_REF_TY\
PE_REQ_ON_CHANGE\
 SYSREQ_REF_TYPE\
_REQ_ON_CHANGE_E\
XISTS SYSREQ_REF\
_TYPE_REQ_ON_SEL\
ECT SYSREQ_REF_T\
YPE_REQ_ON_SELEC\
T_KIND SYSREQ_RE\
F_TYPE_REQ_SECTI\
ON SYSREQ_REF_TY\
PE_VIEW_CARD SYS\
REQ_REF_TYPE_VIE\
W_CODE SYSREQ_RE\
F_TYPE_VIEW_COMM\
ENT SYSREQ_REF_T\
YPE_VIEW_IS_MAIN\
 SYSREQ_REF_TYPE\
_VIEW_NAME SYSRE\
Q_REF_TYPE_VIEW_\
NAME_LOCALIZE_ID\
 SYSREQ_REFERENC\
E_TYPE_ID SYSREQ\
_STATE SYSREQ_ST\
AT\x5cu0415 SYSREQ_\
SYSTEM_SETTINGS_\
VALUE SYSREQ_TYP\
E SYSREQ_UNIT SY\
SREQ_UNIT_ID SYS\
REQ_USER_GROUPS_\
GROUP_FULL_NAME \
SYSREQ_USER_GROU\
PS_GROUP_NAME SY\
SREQ_USER_GROUPS\
_GROUP_SERVER_NA\
ME SYSREQ_USERS_\
ACCESS_RIGHTS SY\
SREQ_USERS_AUTHE\
NTICATION SYSREQ\
_USERS_CATEGORY \
SYSREQ_USERS_COM\
PONENT SYSREQ_US\
ERS_COMPONENT_US\
ER_IS_PUBLIC SYS\
REQ_USERS_DOMAIN\
 SYSREQ_USERS_FU\
LL_USER_NAME SYS\
REQ_USERS_GROUP \
SYSREQ_USERS_IS_\
MAIN_SERVER SYSR\
EQ_USERS_LOGIN S\
YSREQ_USERS_REFE\
RENCE_USER_IS_PU\
BLIC SYSREQ_USER\
S_STATUS SYSREQ_\
USERS_USER_CERTI\
FICATE SYSREQ_US\
ERS_USER_CERTIFI\
CATE_INFO SYSREQ\
_USERS_USER_CERT\
IFICATE_PLUGIN_N\
AME SYSREQ_USERS\
_USER_CERTIFICAT\
E_PLUGIN_VERSION\
 SYSREQ_USERS_US\
ER_CERTIFICATE_S\
TATE SYSREQ_USER\
S_USER_CERTIFICA\
TE_SUBJECT_NAME \
SYSREQ_USERS_USE\
R_CERTIFICATE_TH\
UMBPRINT SYSREQ_\
USERS_USER_DEFAU\
LT_CERTIFICATE S\
YSREQ_USERS_USER\
_DESCRIPTION SYS\
REQ_USERS_USER_G\
LOBAL_NAME SYSRE\
Q_USERS_USER_LOG\
IN SYSREQ_USERS_\
USER_MAIN_SERVER\
 SYSREQ_USERS_US\
ER_TYPE SYSREQ_W\
ORK_RULES_FOLDER\
_ID RESULT_VAR_N\
AME RESULT_VAR_N\
AME_ENG AUTO_NUM\
ERATION_RULE_ID \
CANT_CHANGE_ID_R\
EQUISITE_RULE_ID\
 CANT_CHANGE_OUR\
FIRM_REQUISITE_R\
ULE_ID CHECK_CHA\
NGING_REFERENCE_\
RECORD_USE_RULE_\
ID CHECK_CODE_RE\
QUISITE_RULE_ID \
CHECK_DELETING_R\
EFERENCE_RECORD_\
USE_RULE_ID CHEC\
K_FILTRATER_CHAN\
GES_RULE_ID CHEC\
K_RECORD_INTERVA\
L_RULE_ID CHECK_\
REFERENCE_INTERV\
AL_RULE_ID CHECK\
_REQUIRED_DATA_F\
ULLNESS_RULE_ID \
CHECK_REQUIRED_R\
EQUISITES_FULLNE\
SS_RULE_ID MAKE_\
RECORD_UNRATIFIE\
D_RULE_ID RESTOR\
E_AUTO_NUMERATIO\
N_RULE_ID SET_FI\
RM_CONTEXT_FROM_\
RECORD_RULE_ID S\
ET_FIRST_RECORD_\
IN_LIST_FORM_RUL\
E_ID SET_IDSPS_V\
ALUE_RULE_ID SET\
_NEXT_CODE_VALUE\
_RULE_ID SET_OUR\
FIRM_BOUNDS_RULE\
_ID SET_OURFIRM_\
REQUISITE_RULE_I\
D SCRIPT_BLOCK_A\
FTER_FINISH_EVEN\
T SCRIPT_BLOCK_B\
EFORE_START_EVEN\
T SCRIPT_BLOCK_E\
XECUTION_RESULTS\
_PROPERTY SCRIPT\
_BLOCK_NAME_PROP\
ERTY SCRIPT_BLOC\
K_SCRIPT_PROPERT\
Y SUBTASK_BLOCK_\
ABORT_DEADLINE_P\
ROPERTY SUBTASK_\
BLOCK_AFTER_FINI\
SH_EVENT SUBTASK\
_BLOCK_ASSIGN_PA\
RAMS_EVENT SUBTA\
SK_BLOCK_ATTACHM\
ENTS_PROPERTY SU\
BTASK_BLOCK_ATTA\
CHMENTS_RIGHTS_G\
ROUP_PROPERTY SU\
BTASK_BLOCK_ATTA\
CHMENTS_RIGHTS_T\
YPE_PROPERTY SUB\
TASK_BLOCK_BEFOR\
E_START_EVENT SU\
BTASK_BLOCK_CREA\
TED_TASK_PROPERT\
Y SUBTASK_BLOCK_\
CREATION_EVENT S\
UBTASK_BLOCK_DEA\
DLINE_PROPERTY S\
UBTASK_BLOCK_IMP\
ORTANCE_PROPERTY\
 SUBTASK_BLOCK_I\
NITIATOR_PROPERT\
Y SUBTASK_BLOCK_\
IS_RELATIVE_ABOR\
T_DEADLINE_PROPE\
RTY SUBTASK_BLOC\
K_IS_RELATIVE_DE\
ADLINE_PROPERTY \
SUBTASK_BLOCK_JO\
BS_TYPE_PROPERTY\
 SUBTASK_BLOCK_N\
AME_PROPERTY SUB\
TASK_BLOCK_PARAL\
LEL_ROUTE_PROPER\
TY SUBTASK_BLOCK\
_PERFORMERS_PROP\
ERTY SUBTASK_BLO\
CK_RELATIVE_ABOR\
T_DEADLINE_TYPE_\
PROPERTY SUBTASK\
_BLOCK_RELATIVE_\
DEADLINE_TYPE_PR\
OPERTY SUBTASK_B\
LOCK_REQUIRE_SIG\
N_PROPERTY SUBTA\
SK_BLOCK_STANDAR\
D_ROUTE_PROPERTY\
 SUBTASK_BLOCK_S\
TART_EVENT SUBTA\
SK_BLOCK_STEP_CO\
NTROL_PROPERTY S\
UBTASK_BLOCK_SUB\
JECT_PROPERTY SU\
BTASK_BLOCK_TASK\
_CONTROL_PROPERT\
Y SUBTASK_BLOCK_\
TEXT_PROPERTY SU\
BTASK_BLOCK_UNLO\
CK_ATTACHMENTS_O\
N_STOP_PROPERTY \
SUBTASK_BLOCK_US\
E_STANDARD_ROUTE\
_PROPERTY SUBTAS\
K_BLOCK_WAIT_FOR\
_TASK_COMPLETE_P\
ROPERTY SYSCOMP_\
CONTROL_JOBS SYS\
COMP_FOLDERS SYS\
COMP_JOBS SYSCOM\
P_NOTICES SYSCOM\
P_TASKS SYSDLG_C\
REATE_EDOCUMENT \
SYSDLG_CREATE_ED\
OCUMENT_VERSION \
SYSDLG_CURRENT_P\
ERIOD SYSDLG_EDI\
T_FUNCTION_HELP \
SYSDLG_EDOCUMENT\
_KINDS_FOR_TEMPL\
ATE SYSDLG_EXPOR\
T_MULTIPLE_EDOCU\
MENTS SYSDLG_EXP\
ORT_SINGLE_EDOCU\
MENT SYSDLG_IMPO\
RT_EDOCUMENT SYS\
DLG_MULTIPLE_SEL\
ECT SYSDLG_SETUP\
_ACCESS_RIGHTS S\
YSDLG_SETUP_DEFA\
ULT_RIGHTS SYSDL\
G_SETUP_FILTER_C\
ONDITION SYSDLG_\
SETUP_SIGN_RIGHT\
S SYSDLG_SETUP_T\
ASK_OBSERVERS SY\
SDLG_SETUP_TASK_\
ROUTE SYSDLG_SET\
UP_USERS_LIST SY\
SDLG_SIGN_EDOCUM\
ENT SYSDLG_SIGN_\
MULTIPLE_EDOCUME\
NTS SYSREF_ACCES\
S_RIGHTS_TYPES S\
YSREF_ADMINISTRA\
TION_HISTORY SYS\
REF_ALL_AVAILABL\
E_COMPONENTS SYS\
REF_ALL_AVAILABL\
E_PRIVILEGES SYS\
REF_ALL_REPLICAT\
ING_COMPONENTS S\
YSREF_AVAILABLE_\
DEVELOPERS_COMPO\
NENTS SYSREF_CAL\
ENDAR_EVENTS SYS\
REF_COMPONENT_TO\
KEN_HISTORY SYSR\
EF_COMPONENT_TOK\
ENS SYSREF_COMPO\
NENTS SYSREF_CON\
STANTS SYSREF_DA\
TA_RECEIVE_PROTO\
COL SYSREF_DATA_\
SEND_PROTOCOL SY\
SREF_DIALOGS SYS\
REF_DIALOGS_REQU\
ISITES SYSREF_ED\
ITORS SYSREF_EDO\
C_CARDS SYSREF_E\
DOC_TYPES SYSREF\
_EDOCUMENT_CARD_\
REQUISITES SYSRE\
F_EDOCUMENT_CARD\
_TYPES SYSREF_ED\
OCUMENT_CARD_TYP\
ES_REFERENCE SYS\
REF_EDOCUMENT_CA\
RDS SYSREF_EDOCU\
MENT_HISTORY SYS\
REF_EDOCUMENT_KI\
NDS SYSREF_EDOCU\
MENT_REQUISITES \
SYSREF_EDOCUMENT\
_SIGNATURES SYSR\
EF_EDOCUMENT_TEM\
PLATES SYSREF_ED\
OCUMENT_TEXT_STO\
RAGES SYSREF_EDO\
CUMENT_VIEWS SYS\
REF_FILTERER_SET\
UP_CONFLICTS SYS\
REF_FILTRATER_SE\
TTING_CONFLICTS \
SYSREF_FOLDER_HI\
STORY SYSREF_FOL\
DERS SYSREF_FUNC\
TION_GROUPS SYSR\
EF_FUNCTION_PARA\
MS SYSREF_FUNCTI\
ONS SYSREF_JOB_H\
ISTORY SYSREF_LI\
NKS SYSREF_LOCAL\
IZATION_DICTIONA\
RY SYSREF_LOCALI\
ZATION_LANGUAGES\
 SYSREF_MODULES \
SYSREF_PRIVILEGE\
S SYSREF_RECORD_\
HISTORY SYSREF_R\
EFERENCE_REQUISI\
TES SYSREF_REFER\
ENCE_TYPE_VIEWS \
SYSREF_REFERENCE\
_TYPES SYSREF_RE\
FERENCES SYSREF_\
REFERENCES_REQUI\
SITES SYSREF_REM\
OTE_SERVERS SYSR\
EF_REPLICATION_S\
ESSIONS_LOG SYSR\
EF_REPLICATION_S\
ESSIONS_PROTOCOL\
 SYSREF_REPORTS \
SYSREF_ROLES SYS\
REF_ROUTE_BLOCK_\
GROUPS SYSREF_RO\
UTE_BLOCKS SYSRE\
F_SCRIPTS SYSREF\
_SEARCHES SYSREF\
_SERVER_EVENTS S\
YSREF_SERVER_EVE\
NTS_HISTORY SYSR\
EF_STANDARD_ROUT\
E_GROUPS SYSREF_\
STANDARD_ROUTES \
SYSREF_STATUSES \
SYSREF_SYSTEM_SE\
TTINGS SYSREF_TA\
SK_HISTORY SYSRE\
F_TASK_KIND_GROU\
PS SYSREF_TASK_K\
INDS SYSREF_TASK\
_RIGHTS SYSREF_T\
ASK_SIGNATURES S\
YSREF_TASKS SYSR\
EF_UNITS SYSREF_\
USER_GROUPS SYSR\
EF_USER_GROUPS_R\
EFERENCE SYSREF_\
USER_SUBSTITUTIO\
N SYSREF_USERS S\
YSREF_USERS_REFE\
RENCE SYSREF_VIE\
WERS SYSREF_WORK\
ING_TIME_CALENDA\
RS ACCESS_RIGHTS\
_TABLE_NAME EDMS\
_ACCESS_TABLE_NA\
ME EDOC_TYPES_TA\
BLE_NAME TEST_DE\
V_DB_NAME TEST_D\
EV_SYSTEM_CODE T\
EST_EDMS_DB_NAME\
 TEST_EDMS_MAIN_\
CODE TEST_EDMS_M\
AIN_DB_NAME TEST\
_EDMS_SECOND_COD\
E TEST_EDMS_SECO\
ND_DB_NAME TEST_\
EDMS_SYSTEM_CODE\
 TEST_ISB5_MAIN_\
CODE TEST_ISB5_S\
ECOND_CODE TEST_\
SQL_SERVER_2005_\
NAME TEST_SQL_SE\
RVER_NAME ATTENT\
ION_CAPTION cbsC\
ommandLinks cbsD\
efault CONFIRMAT\
ION_CAPTION ERRO\
R_CAPTION INFORM\
ATION_CAPTION mr\
Cancel mrOk EDOC\
_VERSION_ACTIVE_\
STAGE_CODE EDOC_\
VERSION_DESIGN_S\
TAGE_CODE EDOC_V\
ERSION_OBSOLETE_\
STAGE_CODE cpDat\
aEnciphermentEna\
bled cpDigitalSi\
gnatureEnabled c\
pID cpIssuer cpP\
luginVersion cpS\
erial cpSubjectN\
ame cpSubjSimple\
Name cpValidFrom\
Date cpValidToDa\
te ISBL_SYNTAX N\
O_SYNTAX XML_SYN\
TAX WAIT_BLOCK_A\
FTER_FINISH_EVEN\
T WAIT_BLOCK_BEF\
ORE_START_EVENT \
WAIT_BLOCK_DEADL\
INE_PROPERTY WAI\
T_BLOCK_IS_RELAT\
IVE_DEADLINE_PRO\
PERTY WAIT_BLOCK\
_NAME_PROPERTY W\
AIT_BLOCK_RELATI\
VE_DEADLINE_TYPE\
_PROPERTY SYSRES\
_COMMON SYSRES_C\
ONST SYSRES_MBFU\
NC SYSRES_SBDATA\
 SYSRES_SBGUI SY\
SRES_SBINTF SYSR\
ES_SBREFDSC SYSR\
ES_SQLERRORS SYS\
RES_SYSCOMP atUs\
er atGroup atRol\
e aemEnabledAlwa\
ys aemDisabledAl\
ways aemEnabledO\
nBrowse aemEnabl\
edOnEdit aemDisa\
bledOnBrowseEmpt\
y apBegin apEnd \
alLeft alRight a\
smNever asmNoBut\
Customize asmAsL\
astTime asmYesBu\
tCustomize asmAl\
ways cirCommon c\
irRevoked ctSign\
ature ctEncode c\
tSignatureEncode\
 clbUnchecked cl\
bChecked clbGray\
ed ceISB ceAlway\
s ceNever ctDocu\
ment ctReference\
 ctScript ctUnkn\
own ctReport ctD\
ialog ctFunction\
 ctFolder ctEDoc\
ument ctTask ctJ\
ob ctNotice ctCo\
ntrolJob cfInter\
nal cfDisplay ci\
Unspecified ciWr\
ite ciRead ckFol\
der ckEDocument \
ckTask ckJob ckC\
omponentToken ck\
Any ckReference \
ckScript ckRepor\
t ckDialog ctISB\
LEditor ctBevel \
ctButton ctCheck\
ListBox ctComboB\
ox ctComboEdit c\
tGrid ctDBCheckB\
ox ctDBComboBox \
ctDBEdit ctDBEll\
ipsis ctDBMemo c\
tDBNavigator ctD\
BRadioGroup ctDB\
StatusLabel ctEd\
it ctGroupBox ct\
InplaceHint ctMe\
mo ctPanel ctLis\
tBox ctRadioButt\
on ctRichEdit ct\
TabSheet ctWebBr\
owser ctImage ct\
HyperLink ctLabe\
l ctDBMultiEllip\
sis ctRibbon ctR\
ichView ctInnerP\
anel ctPanelGrou\
p ctBitButton cc\
tDate cctInteger\
 cctNumeric cctP\
ick cctReference\
 cctString cctTe\
xt cltInternal c\
ltPrimary cltGUI\
 dseBeforeOpen d\
seAfterOpen dseB\
eforeClose dseAf\
terClose dseOnVa\
lidDelete dseBef\
oreDelete dseAft\
erDelete dseAfte\
rDeleteOutOfTran\
saction dseOnDel\
eteError dseBefo\
reInsert dseAfte\
rInsert dseOnVal\
idUpdate dseBefo\
reUpdate dseOnUp\
dateRatifiedReco\
rd dseAfterUpdat\
e dseAfterUpdate\
OutOfTransaction\
 dseOnUpdateErro\
r dseAfterScroll\
 dseOnOpenRecord\
 dseOnCloseRecor\
d dseBeforeCance\
l dseAfterCancel\
 dseOnUpdateDead\
lockError dseBef\
oreDetailUpdate \
dseOnPrepareUpda\
te dseOnAnyRequi\
siteChange dssEd\
it dssInsert dss\
Browse dssInActi\
ve dftDate dftSh\
ortDate dftDateT\
ime dftTimeStamp\
 dotDays dotHour\
s dotMinutes dot\
Seconds dtkndLoc\
al dtkndUTC arNo\
ne arView arEdit\
 arFull ddaView \
ddaEdit emLock e\
mEdit emSign emE\
xportWithLock em\
ImportWithUnlock\
 emChangeVersion\
Note emOpenForMo\
dify emChangeLif\
eStage emDelete \
emCreateVersion \
emImport emUnloc\
kExportedWithLoc\
k emStart emAbor\
t emReInit emMar\
kAsReaded emMark\
AsUnreaded emPer\
form emAccept em\
Resume emChangeR\
ights emEditRout\
e emEditObserver\
 emRecoveryFromL\
ocalCopy emChang\
eWorkAccessType \
emChangeEncodeTy\
peToCertificate \
emChangeEncodeTy\
peToPassword emC\
hangeEncodeTypeT\
oNone emChangeEn\
codeTypeToCertif\
icatePassword em\
ChangeStandardRo\
ute emGetText em\
OpenForView emMo\
veToStorage emCr\
eateObject emCha\
ngeVersionHidden\
 emDeleteVersion\
 emChangeLifeCyc\
leStage emApprov\
ingSign emExport\
 emContinue emLo\
ckFromEdit emUnL\
ockForEdit emLoc\
kForServer emUnl\
ockFromServer em\
DelegateAccessRi\
ghts emReEncode \
ecotFile ecotPro\
cess eaGet eaCop\
y eaCreate eaCre\
ateStandardRoute\
 edltAll edltNot\
hing edltQuery e\
ssmText essmCard\
 esvtLast esvtLa\
stActive esvtSpe\
cified edsfExecu\
tive edsfArchive\
 edstSQLServer e\
dstFile edvstNon\
e edvstEDocument\
VersionCopy edvs\
tFile edvstTempl\
ate edvstScanned\
File vsDefault v\
sDesign vsActive\
 vsObsolete etNo\
ne etCertificate\
 etPassword etCe\
rtificatePasswor\
d ecException ec\
Warning ecInform\
ation estAll est\
ApprovingOnly ev\
tLast evtLastAct\
ive evtQuery fdt\
String fdtNumeri\
c fdtInteger fdt\
Date fdtText fdt\
Unknown fdtWideS\
tring fdtLargeIn\
teger ftInbox ft\
Outbox ftFavorit\
es ftCommonFolde\
r ftUserFolder f\
tComponents ftQu\
ickLaunch ftShor\
tcuts ftSearch g\
rhAuto grhX1 grh\
X2 grhX3 hltText\
 hltRTF hltHTML \
iffBMP iffJPEG i\
ffMultiPageTIFF \
iffSinglePageTIF\
F iffTIFF iffPNG\
 im8bGrayscale i\
m24bRGB im1bMono\
chrome itBMP itJ\
PEG itWMF itPNG \
ikhInformation i\
khWarning ikhErr\
or ikhNoIcon icU\
nknown icScript \
icFunction icInt\
egratedReport ic\
AnalyticReport i\
cDataSetEventHan\
dler icActionHan\
dler icFormEvent\
Handler icLookUp\
EventHandler icR\
equisiteChangeEv\
entHandler icBef\
oreSearchEventHa\
ndler icRoleCalc\
ulation icSelect\
RouteEventHandle\
r icBlockPropert\
yCalculation icB\
lockQueryParamsE\
ventHandler icCh\
angeSearchResult\
EventHandler icB\
lockEventHandler\
 icSubTaskInitEv\
entHandler icEDo\
cDataSetEventHan\
dler icEDocLookU\
pEventHandler ic\
EDocActionHandle\
r icEDocFormEven\
tHandler icEDocR\
equisiteChangeEv\
entHandler icStr\
ucturedConversio\
nRule icStructur\
edConversionEven\
tBefore icStruct\
uredConversionEv\
entAfter icWizar\
dEventHandler ic\
WizardFinishEven\
tHandler icWizar\
dStepEventHandle\
r icWizardStepFi\
nishEventHandler\
 icWizardActionE\
nableEventHandle\
r icWizardAction\
ExecuteEventHand\
ler icCreateJobs\
Handler icCreate\
NoticesHandler i\
cBeforeLookUpEve\
ntHandler icAfte\
rLookUpEventHand\
ler icTaskAbortE\
ventHandler icWo\
rkflowBlockActio\
nHandler icDialo\
gDataSetEventHan\
dler icDialogAct\
ionHandler icDia\
logLookUpEventHa\
ndler icDialogRe\
quisiteChangeEve\
ntHandler icDial\
ogFormEventHandl\
er icDialogValid\
CloseEventHandle\
r icBlockFormEve\
ntHandler icTask\
FormEventHandler\
 icReferenceMeth\
od icEDocMethod \
icDialogMethod i\
cProcessMessageH\
andler isShow is\
Hide isByUserSet\
tings jkJob jkNo\
tice jkControlJo\
b jtInner jtLeft\
 jtRight jtFull \
jtCross lbpAbove\
 lbpBelow lbpLef\
t lbpRight eltPe\
rConnection eltP\
erUser sfcUndefi\
ned sfcBlack sfc\
Green sfcRed sfc\
Blue sfcOrange s\
fcLilac sfsItali\
c sfsStrikeout s\
fsNormal ldctSta\
ndardRoute ldctW\
izard ldctScript\
 ldctFunction ld\
ctRouteBlock ldc\
tIntegratedRepor\
t ldctAnalyticRe\
port ldctReferen\
ceType ldctEDocu\
mentType ldctDia\
log ldctServerEv\
ents mrcrtNone m\
rcrtUser mrcrtMa\
ximal mrcrtCusto\
m vtEqual vtGrea\
terOrEqual vtLes\
sOrEqual vtRange\
 rdYesterday rdT\
oday rdTomorrow \
rdThisWeek rdThi\
sMonth rdThisYea\
r rdNextMonth rd\
NextWeek rdLastW\
eek rdLastMonth \
rdWindow rdFile \
rdPrinter rdtStr\
ing rdtNumeric r\
dtInteger rdtDat\
e rdtReference r\
dtAccount rdtTex\
t rdtPick rdtUnk\
nown rdtLargeInt\
eger rdtDocument\
 reOnChange reOn\
ChangeValues ttG\
lobal ttLocal tt\
User ttSystem ss\
mBrowse ssmSelec\
t ssmMultiSelect\
 ssmBrowseModal \
smSelect smLike \
smCard stNone st\
Authenticating s\
tApproving sctSt\
ring sctStream s\
stAnsiSort sstNa\
turalSort svtEqu\
al svtContain so\
atString soatNum\
eric soatInteger\
 soatDatetime so\
atReferenceRecor\
d soatText soatP\
ick soatBoolean \
soatEDocument so\
atAccount soatIn\
tegerCollection \
soatNumericColle\
ction soatString\
Collection soatP\
ickCollection so\
atDatetimeCollec\
tion soatBoolean\
Collection soatR\
eferenceRecordCo\
llection soatEDo\
cumentCollection\
 soatAccountColl\
ection soatConte\
nts soatUnknown \
tarAbortByUser t\
arAbortByWorkflo\
wException tvtAl\
lWords tvtExactP\
hrase tvtAnyWord\
 usNone usComple\
ted usRedSquare \
usBlueSquare usY\
ellowSquare usGr\
eenSquare usOran\
geSquare usPurpl\
eSquare usFollow\
Up utUnknown utU\
ser utDeveloper \
utAdministrator \
utSystemDevelope\
r utDisconnected\
 btAnd btDetailA\
nd btOr btNotOr \
btOnly vmView vm\
Select vmNavigat\
ion vsmSingle vs\
mMultiple vsmMul\
tipleCheck vsmNo\
Selection wfatPr\
evious wfatNext \
wfatCancel wfatF\
inish wfepUndefi\
ned wfepText3 wf\
epText6 wfepText\
9 wfepSpinEdit w\
fepDropDown wfep\
RadioGroup wfepF\
lag wfepText12 w\
fepText15 wfepTe\
xt18 wfepText21 \
wfepText24 wfepT\
ext27 wfepText30\
 wfepRadioGroupC\
olumn1 wfepRadio\
GroupColumn2 wfe\
pRadioGroupColum\
n3 wfetQueryPara\
meter wfetText w\
fetDelimiter wfe\
tLabel wptString\
 wptInteger wptN\
umeric wptBoolea\
n wptDateTime wp\
tPick wptText wp\
tUser wptUserLis\
t wptEDocumentIn\
fo wptEDocumentI\
nfoList wptRefer\
enceRecordInfo w\
ptReferenceRecor\
dInfoList wptFol\
derInfo wptTaskI\
nfo wptContents \
wptFileName wptD\
ate wsrComplete \
wsrGoNext wsrGoP\
revious wsrCusto\
m wsrCancel wsrG\
oFinal wstForm w\
stEDocument wstT\
askCard wstRefer\
enceRecordCard w\
stFinal waAll wa\
Performers waMan\
ual wsbStart wsb\
Finish wsbNotice\
 wsbStep wsbDeci\
sion wsbWait wsb\
Monitor wsbScrip\
t wsbConnector w\
sbSubTask wsbLif\
eCycleStage wsbP\
ause wdtInteger \
wdtFloat wdtStri\
ng wdtPick wdtDa\
teTime wdtBoolea\
n wdtTask wdtJob\
 wdtFolder wdtED\
ocument wdtRefer\
enceRecord wdtUs\
er wdtGroup wdtR\
ole wdtIntegerCo\
llection wdtFloa\
tCollection wdtS\
tringCollection \
wdtPickCollectio\
n wdtDateTimeCol\
lection wdtBoole\
anCollection wdt\
TaskCollection w\
dtJobCollection \
wdtFolderCollect\
ion wdtEDocument\
Collection wdtRe\
ferenceRecordCol\
lection wdtUserC\
ollection wdtGro\
upCollection wdt\
RoleCollection w\
dtContents wdtUs\
erList wdtSearch\
Description wdtD\
eadLine wdtPickS\
et wdtAccountCol\
lection wiLow wi\
Normal wiHigh wr\
tSoft wrtHard ws\
Init wsRunning w\
sDone wsControll\
ed wsAborted wsC\
ontinued wtmFull\
 wtmFromCurrent \
wtmOnlyCurrent \x22\
,\x0aclass:\x22AltStat\
e Application Ca\
llType Component\
Tokens CreatedJo\
bs CreatedNotice\
s ControlState D\
ialogResult Dial\
ogs EDocuments E\
DocumentVersionS\
ource Folders Gl\
obalIDs Job Jobs\
 InputValue Look\
UpReference Look\
UpRequisiteNames\
 LookUpSearch Ob\
ject ParentCompo\
nent Processes R\
eferences Requis\
ite ReportName R\
eports Result Sc\
ripts Searches S\
electedAttachmen\
ts SelectedItems\
 SelectMode Send\
er ServerEvents \
ServiceFactory S\
hiftState SubTas\
k SystemDialogs \
Tasks Wizard Wiz\
ards Work \x5cu0412\
\x5cu044b\x5cu0437\x5cu04\
3e\x5cu0432\x5cu0421\x5cu\
043f\x5cu043e\x5cu0441\
\x5cu043e\x5cu0431 \x5cu0\
418\x5cu043c\x5cu044f\x5c\
u041e\x5cu0442\x5cu044\
7\x5cu0435\x5cu0442\x5cu0\
430 \x5cu0420\x5cu0435\
\x5cu043a\x5cu0432\x5cu04\
17\x5cu043d\x5cu0430\x5cu\
0447 \x22,\x0aliteral:\
\x22null true false\
 nil \x22},I={begin\
:\x22\x5c\x5c.\x5c\x5cs*\x22+S.UND\
ERSCORE_IDENT_RE\
,\x0akeywords:C,rel\
evance:0},N={cla\
ssName:\x22type\x22,\x0ab\
egin:\x22:[ \x5c\x5ct]*(I\
Application|IAcc\
essRights|IAccou\
ntRepository|IAc\
countSelectionRe\
strictions|IActi\
on|IActionList|I\
AdministrationHi\
storyDescription\
|IAnchors|IAppli\
cation|IArchiveI\
nfo|IAttachment|\
IAttachmentList|\
ICheckListBox|IC\
heckPointedList|\
IColumn|ICompone\
nt|IComponentDes\
cription|ICompon\
entToken|ICompon\
entTokenFactory|\
IComponentTokenI\
nfo|ICompRecordI\
nfo|IConnection|\
IContents|IContr\
ol|IControlJob|I\
ControlJobInfo|I\
ControlList|ICry\
pto|ICrypto2|ICu\
stomJob|ICustomJ\
obInfo|ICustomLi\
stBox|ICustomObj\
ectWizardStep|IC\
ustomWork|ICusto\
mWorkInfo|IDataS\
et|IDataSetAcces\
sInfo|IDataSigne\
r|IDateCriterion\
|IDateRequisite|\
IDateRequisiteDe\
scription|IDateV\
alue|IDeaAccessR\
ights|IDeaObject\
Info|IDevelopmen\
tComponentLock|I\
Dialog|IDialogFa\
ctory|IDialogPic\
kRequisiteItems|\
IDialogsFactory|\
IDICSFactory|IDo\
cRequisite|IDocu\
mentInfo|IDualLi\
stDialog|IECerti\
ficate|IECertifi\
cateInfo|IECerti\
ficates|IEditCon\
trol|IEditorForm\
|IEdmsExplorer|I\
EdmsObject|IEdms\
ObjectDescriptio\
n|IEdmsObjectFac\
tory|IEdmsObject\
Info|IEDocument|\
IEDocumentAccess\
Rights|IEDocumen\
tDescription|IED\
ocumentEditor|IE\
DocumentFactory|\
IEDocumentInfo|I\
EDocumentStorage\
|IEDocumentVersi\
on|IEDocumentVer\
sionListDialog|I\
EDocumentVersion\
Source|IEDocumen\
tWizardStep|IEDo\
cVerSignature|IE\
DocVersionState|\
IEnabledMode|IEn\
codeProvider|IEn\
crypter|IEvent|I\
EventList|IExcep\
tion|IExternalEv\
ents|IExternalHa\
ndler|IFactory|I\
Field|IFileDialo\
g|IFolder|IFolde\
rDescription|IFo\
lderDialog|IFold\
erFactory|IFolde\
rInfo|IForEach|I\
Form|IFormTitle|\
IFormWizardStep|\
IGlobalIDFactory\
|IGlobalIDInfo|I\
Grid|IHasher|IHi\
storyDescription\
|IHyperLinkContr\
ol|IImageButton|\
IImageControl|II\
nnerPanel|IInpla\
ceHint|IIntegerC\
riterion|IIntege\
rList|IIntegerRe\
quisite|IInteger\
Value|IISBLEdito\
rForm|IJob|IJobD\
escription|IJobF\
actory|IJobForm|\
IJobInfo|ILabelC\
ontrol|ILargeInt\
egerCriterion|IL\
argeIntegerRequi\
site|ILargeInteg\
erValue|ILicense\
Info|ILifeCycleS\
tage|IList|IList\
Box|ILocalIDInfo\
|ILocalization|I\
Lock|IMemoryData\
Set|IMessagingFa\
ctory|IMetadataR\
epository|INotic\
e|INoticeInfo|IN\
umericCriterion|\
INumericRequisit\
e|INumericValue|\
IObject|IObjectD\
escription|IObje\
ctImporter|IObje\
ctInfo|IObserver\
|IPanelGroup|IPi\
ckCriterion|IPic\
kProperty|IPickR\
equisite|IPickRe\
quisiteDescripti\
on|IPickRequisit\
eItem|IPickRequi\
siteItems|IPickV\
alue|IPrivilege|\
IPrivilegeList|I\
Process|IProcess\
Factory|IProcess\
Message|IProgres\
s|IProperty|IPro\
pertyChangeEvent\
|IQuery|IReferen\
ce|IReferenceCri\
terion|IReferenc\
eEnabledMode|IRe\
ferenceFactory|I\
ReferenceHistory\
Description|IRef\
erenceInfo|IRefe\
renceRecordCardW\
izardStep|IRefer\
enceRequisiteDes\
cription|IRefere\
ncesFactory|IRef\
erenceValue|IRef\
Requisite|IRepor\
t|IReportFactory\
|IRequisite|IReq\
uisiteDescriptio\
n|IRequisiteDesc\
riptionList|IReq\
uisiteFactory|IR\
ichEdit|IRouteSt\
ep|IRule|IRuleLi\
st|ISchemeBlock|\
IScript|IScriptF\
actory|ISearchCr\
iteria|ISearchCr\
iterion|ISearchD\
escription|ISear\
chFactory|ISearc\
hFolderInfo|ISea\
rchForObjectDesc\
ription|ISearchR\
esultRestriction\
s|ISecuredContex\
t|ISelectDialog|\
IServerEvent|ISe\
rverEventFactory\
|IServiceDialog|\
IServiceFactory|\
ISignature|ISign\
Provider|ISignPr\
ovider2|ISignPro\
vider3|ISimpleCr\
iterion|IStringC\
riterion|IString\
List|IStringRequ\
isite|IStringReq\
uisiteDescriptio\
n|IStringValue|I\
SystemDialogsFac\
tory|ISystemInfo\
|ITabSheet|ITask\
|ITaskAbortReaso\
nInfo|ITaskCardW\
izardStep|ITaskD\
escription|ITask\
Factory|ITaskInf\
o|ITaskRoute|ITe\
xtCriterion|ITex\
tRequisite|IText\
Value|ITreeListS\
electDialog|IUse\
r|IUserList|IVal\
ue|IView|IWebBro\
wserControl|IWiz\
ard|IWizardActio\
n|IWizardFactory\
|IWizardFormElem\
ent|IWizardParam\
|IWizardPickPara\
m|IWizardReferen\
ceParam|IWizardS\
tep|IWorkAccessR\
ights|IWorkDescr\
iption|IWorkflow\
AskableParam|IWo\
rkflowAskablePar\
ams|IWorkflowBlo\
ck|IWorkflowBloc\
kResult|IWorkflo\
wEnabledMode|IWo\
rkflowParam|IWor\
kflowPickParam|I\
WorkflowReferenc\
eParam|IWorkStat\
e|IWorkTreeCusto\
mNode|IWorkTreeJ\
obNode|IWorkTree\
TaskNode|IXMLEdi\
torForm|SBCrypto\
)\x22,\x0aend:\x22[ \x5c\x5ct]*\
=\x22,excludeEnd:!0\
},A={className:\x22\
variable\x22,keywor\
ds:C,begin:E,\x0are\
levance:0,contai\
ns:[N,I]\x0a},e=\x22[A\
-Za-z\x5cu0410-\x5cu04\
2f\x5cu0430-\x5cu044f\x5c\
u0451\x5cu0401_][A-\
Za-z\x5cu0410-\x5cu042\
f\x5cu0430-\x5cu044f\x5cu\
0451\x5cu0401_0-9]*\
\x5c\x5c(\x22\x0a;return{nam\
e:\x22ISBL\x22,case_in\
sensitive:!0,key\
words:C,\x0aillegal\
:\x22\x5c\x5c$|\x5c\x5c?|%|,|;$\
|~|#|@|</\x22,conta\
ins:[{className:\
\x22function\x22,begin\
:e,\x0aend:\x22\x5c\x5c)$\x22,r\
eturnBegin:!0,ke\
ywords:C,illegal\
:\x22[\x5c\x5c[\x5c\x5c]\x5c\x5c|\x5c\x5c$\x5c\
\x5c?%,~#@]\x22,\x0aconta\
ins:[{className:\
\x22title\x22,keywords\
:{$pattern:E,\x0abu\
ilt_in:\x22AddSubSt\
ring AdjustLineB\
reaks AmountInWo\
rds Analysis Arr\
ayDimCount Array\
HighBound ArrayL\
owBound ArrayOf \
ArrayReDim Asser\
t Assigned Begin\
OfMonth BeginOfP\
eriod BuildProfi\
lingOperationAna\
lysis CallProced\
ure CanReadFile \
CArrayElement CD\
ataSetRequisite \
ChangeDate Chang\
eReferenceDatase\
t Char CharPos C\
heckParam CheckP\
aramValue Compar\
eStrings Constan\
tExists ControlS\
tate ConvertDate\
Str Copy CopyFil\
e CreateArray Cr\
eateCachedRefere\
nce CreateConnec\
tion CreateDialo\
g CreateDualList\
Dialog CreateEdi\
tor CreateExcept\
ion CreateFile C\
reateFolderDialo\
g CreateInputDia\
log CreateLinkFi\
le CreateList Cr\
eateLock CreateM\
emoryDataSet Cre\
ateObject Create\
OpenDialog Creat\
eProgress Create\
Query CreateRefe\
rence CreateRepo\
rt CreateSaveDia\
log CreateScript\
 CreateSQLPivotF\
unction CreateSt\
ringList CreateT\
reeListSelectDia\
log CSelectSQL C\
SQL CSubString C\
urrentUserID Cur\
rentUserName Cur\
rentVersion Data\
SetLocateEx Date\
Diff DateTimeDif\
f DateToStr DayO\
fWeek DeleteFile\
 DirectoryExists\
 DisableCheckAcc\
essRights Disabl\
eCheckFullShowin\
gRestriction Dis\
ableMassTaskSend\
ingRestrictions \
DropTable DupeSt\
ring EditText En\
ableCheckAccessR\
ights EnableChec\
kFullShowingRest\
riction EnableMa\
ssTaskSendingRes\
trictions EndOfM\
onth EndOfPeriod\
 ExceptionExists\
 ExceptionsOff E\
xceptionsOn Exec\
ute ExecuteProce\
ss Exit ExpandEn\
vironmentVariabl\
es ExtractFileDr\
ive ExtractFileE\
xt ExtractFileNa\
me ExtractFilePa\
th ExtractParams\
 FileExists File\
Size FindFile Fi\
ndSubString Firm\
Context ForceDir\
ectories Format \
FormatDate Forma\
tNumeric FormatS\
QLDate FormatStr\
ing FreeExceptio\
n GetComponent G\
etComponentLaunc\
hParam GetConsta\
nt GetLastExcept\
ion GetReference\
Record GetRefTyp\
eByRefID GetTabl\
eID GetTempFolde\
r IfThen In Inde\
xOf InputDialog \
InputDialogEx In\
teractiveMode Is\
FileLocked IsGra\
phicFile IsNumer\
ic Length LoadSt\
ring LoadStringF\
mt LocalTimeToUT\
C LowerCase Max \
MessageBox Messa\
geBoxEx MimeDeco\
deBinary MimeDec\
odeString MimeEn\
codeBinary MimeE\
ncodeString Min \
MoneyInWords Mov\
eFile NewID Now \
OpenFile Ord Pre\
cision Raise Rea\
dCertificateFrom\
File ReadFile Re\
ferenceCodeByID \
ReferenceNumber \
ReferenceRequisi\
teMode Reference\
RequisiteValue R\
egionDateSetting\
s RegionNumberSe\
ttings RegionTim\
eSettings RegRea\
d RegWrite Renam\
eFile Replace Ro\
und SelectServer\
Code SelectSQL S\
erverDateTime Se\
tConstant SetMan\
agedFolderFields\
State ShowConsta\
ntsInputDialog S\
howMessage Sleep\
 Split SQL SQL2X\
LSTAB SQLProfili\
ngSendReport Str\
ToDate SubString\
 SubStringCount \
SystemSetting Ti\
me TimeDiff Toda\
y Transliterate \
Trim UpperCase U\
serStatus UTCToL\
ocalTime Validat\
eXML VarIsClear \
VarIsEmpty VarIs\
Null WorkTimeDif\
f WriteFile Writ\
eFileEx WriteObj\
ectHistory \x5cu041\
0\x5cu043d\x5cu0430\x5cu0\
43b\x5cu0438\x5cu0437 \
\x5cu0411\x5cu0430\x5cu04\
37\x5cu0430\x5cu0414\x5cu\
0430\x5cu043d\x5cu043d\
\x5cu044b\x5cu0445 \x5cu0\
411\x5cu043b\x5cu043e\x5c\
u043a\x5cu0415\x5cu044\
1\x5cu0442\x5cu044c \x5cu\
0411\x5cu043b\x5cu043e\
\x5cu043a\x5cu0415\x5cu04\
41\x5cu0442\x5cu044c\x5cu\
0420\x5cu0430\x5cu0441\
\x5cu0448 \x5cu0411\x5cu0\
43b\x5cu043e\x5cu043a\x5c\
u0418\x5cu043d\x5cu044\
4\x5cu043e \x5cu0411\x5cu\
043b\x5cu043e\x5cu043a\
\x5cu0421\x5cu043d\x5cu04\
4f\x5cu0442\x5cu044c \x5c\
u0411\x5cu043b\x5cu043\
e\x5cu043a\x5cu0421\x5cu0\
43d\x5cu044f\x5cu0442\x5c\
u044c\x5cu0420\x5cu043\
0\x5cu0441\x5cu0448 \x5cu\
0411\x5cu043b\x5cu043e\
\x5cu043a\x5cu0423\x5cu04\
41\x5cu0442\x5cu0430\x5cu\
043d\x5cu043e\x5cu0432\
\x5cu0438\x5cu0442\x5cu04\
4c \x5cu0412\x5cu0432\x5c\
u043e\x5cu0434 \x5cu04\
12\x5cu0432\x5cu043e\x5cu\
0434\x5cu041c\x5cu0435\
\x5cu043d\x5cu044e \x5cu0\
412\x5cu0435\x5cu0434\x5c\
u0421 \x5cu0412\x5cu04\
35\x5cu0434\x5cu0421\x5cu\
043f\x5cu0440 \x5cu041\
2\x5cu0435\x5cu0440\x5cu0\
445\x5cu043d\x5cu044f\x5c\
u044f\x5cu0413\x5cu044\
0\x5cu0430\x5cu043d\x5cu0\
438\x5cu0446\x5cu0430\x5c\
u041c\x5cu0430\x5cu044\
1\x5cu0441\x5cu0438\x5cu0\
432\x5cu0430 \x5cu0412\
\x5cu043d\x5cu0435\x5cu04\
48\x5cu041f\x5cu0440\x5cu\
043e\x5cu0433\x5cu0440\
 \x5cu0412\x5cu043e\x5cu0\
441\x5cu0441\x5cu0442 \
\x5cu0412\x5cu0440\x5cu04\
35\x5cu043c\x5cu0435\x5cu\
043d\x5cu043d\x5cu0430\
\x5cu044f\x5cu041f\x5cu04\
30\x5cu043f\x5cu043a\x5cu\
0430 \x5cu0412\x5cu044\
0\x5cu0435\x5cu043c\x5cu0\
44f \x5cu0412\x5cu044b\
\x5cu0431\x5cu043e\x5cu04\
40SQL \x5cu0412\x5cu04\
4b\x5cu0431\x5cu0440\x5cu\
0430\x5cu0442\x5cu044c\
\x5cu0417\x5cu0430\x5cu04\
3f\x5cu0438\x5cu0441\x5cu\
044c \x5cu0412\x5cu044\
b\x5cu0434\x5cu0435\x5cu0\
43b\x5cu0438\x5cu0442\x5c\
u044c\x5cu0421\x5cu044\
2\x5cu0440 \x5cu0412\x5cu\
044b\x5cu0437\x5cu0432\
\x5cu0430\x5cu0442\x5cu04\
4c \x5cu0412\x5cu044b\x5c\
u043f\x5cu043e\x5cu043\
b\x5cu043d\x5cu0438\x5cu0\
442\x5cu044c \x5cu0412\
\x5cu044b\x5cu043f\x5cu04\
1f\x5cu0440\x5cu043e\x5cu\
0433\x5cu0440 \x5cu041\
3\x5cu0440\x5cu0430\x5cu0\
444\x5cu0438\x5cu0447\x5c\
u0435\x5cu0441\x5cu043\
a\x5cu0438\x5cu0439\x5cu0\
424\x5cu0430\x5cu0439\x5c\
u043b \x5cu0413\x5cu04\
40\x5cu0443\x5cu043f\x5cu\
043f\x5cu0430\x5cu0414\
\x5cu043e\x5cu043f\x5cu04\
3e\x5cu043b\x5cu043d\x5cu\
0438\x5cu0442\x5cu0435\
\x5cu043b\x5cu044c\x5cu04\
3d\x5cu043e \x5cu0414\x5c\
u0430\x5cu0442\x5cu043\
0\x5cu0412\x5cu0440\x5cu0\
435\x5cu043c\x5cu044f\x5c\
u0421\x5cu0435\x5cu044\
0\x5cu0432 \x5cu0414\x5cu\
0435\x5cu043d\x5cu044c\
\x5cu041d\x5cu0435\x5cu04\
34\x5cu0435\x5cu043b\x5cu\
0438 \x5cu0414\x5cu043\
8\x5cu0430\x5cu043b\x5cu0\
43e\x5cu0433\x5cu0414\x5c\
u0430\x5cu041d\x5cu043\
5\x5cu0442 \x5cu0414\x5cu\
043b\x5cu0438\x5cu043d\
\x5cu0430\x5cu0421\x5cu04\
42\x5cu0440 \x5cu0414\x5c\
u043e\x5cu0431\x5cu041\
f\x5cu043e\x5cu0434\x5cu0\
441\x5cu0442\x5cu0440 \
\x5cu0415\x5cu041f\x5cu04\
43\x5cu0441\x5cu0442\x5cu\
043e \x5cu0415\x5cu044\
1\x5cu043b\x5cu0438\x5cu0\
422\x5cu043e \x5cu0415\
\x5cu0427\x5cu0438\x5cu04\
41\x5cu043b\x5cu043e \x5c\
u0417\x5cu0430\x5cu043\
c\x5cu041f\x5cu043e\x5cu0\
434\x5cu0441\x5cu0442\x5c\
u0440 \x5cu0417\x5cu04\
30\x5cu043f\x5cu0438\x5cu\
0441\x5cu044c\x5cu0421\
\x5cu043f\x5cu0440\x5cu04\
30\x5cu0432\x5cu043e\x5cu\
0447\x5cu043d\x5cu0438\
\x5cu043a\x5cu0430 \x5cu0\
417\x5cu043d\x5cu0430\x5c\
u0447\x5cu041f\x5cu043\
e\x5cu043b\x5cu044f\x5cu0\
421\x5cu043f\x5cu0440 \
\x5cu0418\x5cu0414\x5cu04\
22\x5cu0438\x5cu043f\x5cu\
0421\x5cu043f\x5cu0440\
 \x5cu0418\x5cu0437\x5cu0\
432\x5cu043b\x5cu0435\x5c\
u0447\x5cu044c\x5cu041\
4\x5cu0438\x5cu0441\x5cu0\
43a \x5cu0418\x5cu0437\
\x5cu0432\x5cu043b\x5cu04\
35\x5cu0447\x5cu044c\x5cu\
0418\x5cu043c\x5cu044f\
\x5cu0424\x5cu0430\x5cu04\
39\x5cu043b\x5cu0430 \x5c\
u0418\x5cu0437\x5cu043\
2\x5cu043b\x5cu0435\x5cu0\
447\x5cu044c\x5cu041f\x5c\
u0443\x5cu0442\x5cu044\
c \x5cu0418\x5cu0437\x5cu\
0432\x5cu043b\x5cu0435\
\x5cu0447\x5cu044c\x5cu04\
20\x5cu0430\x5cu0441\x5cu\
0448\x5cu0438\x5cu0440\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435 \x5cu0418\x5c\
u0437\x5cu043c\x5cu041\
4\x5cu0430\x5cu0442 \x5cu\
0418\x5cu0437\x5cu043c\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
0420\x5cu0430\x5cu0437\
\x5cu043c\x5cu0435\x5cu04\
40\x5cu041c\x5cu0430\x5cu\
0441\x5cu0441\x5cu0438\
\x5cu0432\x5cu0430 \x5cu0\
418\x5cu0437\x5cu043c\x5c\
u0435\x5cu0440\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
439\x5cu041c\x5cu0430\x5c\
u0441\x5cu0441\x5cu043\
8\x5cu0432\x5cu0430 \x5cu\
0418\x5cu043c\x5cu044f\
\x5cu041e\x5cu0440\x5cu04\
33 \x5cu0418\x5cu043c\x5c\
u044f\x5cu041f\x5cu043\
e\x5cu043b\x5cu044f\x5cu0\
421\x5cu043f\x5cu0440 \
\x5cu0418\x5cu043d\x5cu04\
34\x5cu0435\x5cu043a\x5cu\
0441 \x5cu0418\x5cu043\
d\x5cu0434\x5cu0438\x5cu0\
43a\x5cu0430\x5cu0442\x5c\
u043e\x5cu0440\x5cu041\
7\x5cu0430\x5cu043a\x5cu0\
440\x5cu044b\x5cu0442\x5c\
u044c \x5cu0418\x5cu04\
3d\x5cu0434\x5cu0438\x5cu\
043a\x5cu0430\x5cu0442\
\x5cu043e\x5cu0440\x5cu04\
1e\x5cu0442\x5cu043a\x5cu\
0440\x5cu044b\x5cu0442\
\x5cu044c \x5cu0418\x5cu0\
43d\x5cu0434\x5cu0438\x5c\
u043a\x5cu0430\x5cu044\
2\x5cu043e\x5cu0440\x5cu0\
428\x5cu0430\x5cu0433 \
\x5cu0418\x5cu043d\x5cu04\
42\x5cu0435\x5cu0440\x5cu\
0430\x5cu043a\x5cu0442\
\x5cu0438\x5cu0432\x5cu04\
3d\x5cu044b\x5cu0439\x5cu\
0420\x5cu0435\x5cu0436\
\x5cu0438\x5cu043c \x5cu0\
418\x5cu0442\x5cu043e\x5c\
u0433\x5cu0422\x5cu043\
1\x5cu043b\x5cu0421\x5cu0\
43f\x5cu0440 \x5cu041a\
\x5cu043e\x5cu0434\x5cu04\
12\x5cu0438\x5cu0434\x5cu\
0412\x5cu0435\x5cu0434\
\x5cu0421\x5cu043f\x5cu04\
40 \x5cu041a\x5cu043e\x5c\
u0434\x5cu0412\x5cu043\
8\x5cu0434\x5cu0421\x5cu0\
43f\x5cu0440\x5cu041f\x5c\
u043e\x5cu0418\x5cu041\
4 \x5cu041a\x5cu043e\x5cu\
0434\x5cu041f\x5cu043e\
Analit \x5cu041a\x5cu0\
43e\x5cu0434\x5cu0421\x5c\
u0438\x5cu043c\x5cu043\
2\x5cu043e\x5cu043b\x5cu0\
430 \x5cu041a\x5cu043e\
\x5cu0434\x5cu0421\x5cu04\
3f\x5cu0440 \x5cu041a\x5c\
u043e\x5cu043b\x5cu041\
f\x5cu043e\x5cu0434\x5cu0\
441\x5cu0442\x5cu0440 \
\x5cu041a\x5cu043e\x5cu04\
3b\x5cu041f\x5cu0440\x5cu\
043e\x5cu043f \x5cu041\
a\x5cu043e\x5cu043d\x5cu0\
41c\x5cu0435\x5cu0441 \
\x5cu041a\x5cu043e\x5cu04\
3d\x5cu0441\x5cu0442 \x5c\
u041a\x5cu043e\x5cu043\
d\x5cu0441\x5cu0442\x5cu0\
415\x5cu0441\x5cu0442\x5c\
u044c \x5cu041a\x5cu04\
3e\x5cu043d\x5cu0441\x5cu\
0442\x5cu0417\x5cu043d\
\x5cu0430\x5cu0447 \x5cu0\
41a\x5cu043e\x5cu043d\x5c\
u0422\x5cu0440\x5cu043\
0\x5cu043d \x5cu041a\x5cu\
043e\x5cu043f\x5cu0438\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0430\x5cu0442\x5cu\
044c\x5cu0424\x5cu0430\
\x5cu0439\x5cu043b \x5cu0\
41a\x5cu043e\x5cu043f\x5c\
u0438\x5cu044f\x5cu042\
1\x5cu0442\x5cu0440 \x5cu\
041a\x5cu041f\x5cu0435\
\x5cu0440\x5cu0438\x5cu04\
3e\x5cu0434 \x5cu041a\x5c\
u0421\x5cu0442\x5cu044\
0\x5cu0422\x5cu0431\x5cu0\
43b\x5cu0421\x5cu043f\x5c\
u0440 \x5cu041c\x5cu04\
30\x5cu043a\x5cu0441 \x5c\
u041c\x5cu0430\x5cu043\
a\x5cu0441\x5cu0421\x5cu0\
442\x5cu0440\x5cu0422\x5c\
u0431\x5cu043b\x5cu042\
1\x5cu043f\x5cu0440 \x5cu\
041c\x5cu0430\x5cu0441\
\x5cu0441\x5cu0438\x5cu04\
32 \x5cu041c\x5cu0435\x5c\
u043d\x5cu044e \x5cu04\
1c\x5cu0435\x5cu043d\x5cu\
044e\x5cu0420\x5cu0430\
\x5cu0441\x5cu0448 \x5cu0\
41c\x5cu0438\x5cu043d \
\x5cu041d\x5cu0430\x5cu04\
31\x5cu043e\x5cu0440\x5cu\
0414\x5cu0430\x5cu043d\
\x5cu043d\x5cu044b\x5cu04\
45\x5cu041d\x5cu0430\x5cu\
0439\x5cu0442\x5cu0438\
\x5cu0420\x5cu0430\x5cu04\
41\x5cu0448 \x5cu041d\x5c\
u0430\x5cu0438\x5cu043\
c\x5cu0412\x5cu0438\x5cu0\
434\x5cu0421\x5cu043f\x5c\
u0440 \x5cu041d\x5cu04\
30\x5cu0438\x5cu043c\x5cu\
041f\x5cu043eAnalit\
 \x5cu041d\x5cu0430\x5cu0\
438\x5cu043c\x5cu0421\x5c\
u043f\x5cu0440 \x5cu04\
1d\x5cu0430\x5cu0441\x5cu\
0442\x5cu0440\x5cu043e\
\x5cu0438\x5cu0442\x5cu04\
4c\x5cu041f\x5cu0435\x5cu\
0440\x5cu0435\x5cu0432\
\x5cu043e\x5cu0434\x5cu04\
4b\x5cu0421\x5cu0442\x5cu\
0440\x5cu043e\x5cu043a\
 \x5cu041d\x5cu0430\x5cu0\
447\x5cu041c\x5cu0435\x5c\
u0441 \x5cu041d\x5cu04\
30\x5cu0447\x5cu0422\x5cu\
0440\x5cu0430\x5cu043d\
 \x5cu041d\x5cu0438\x5cu0\
436\x5cu043d\x5cu044f\x5c\
u044f\x5cu0413\x5cu044\
0\x5cu0430\x5cu043d\x5cu0\
438\x5cu0446\x5cu0430\x5c\
u041c\x5cu0430\x5cu044\
1\x5cu0441\x5cu0438\x5cu0\
432\x5cu0430 \x5cu041d\
\x5cu043e\x5cu043c\x5cu04\
35\x5cu0440\x5cu0421\x5cu\
043f\x5cu0440 \x5cu041\
d\x5cu041f\x5cu0435\x5cu0\
440\x5cu0438\x5cu043e\x5c\
u0434 \x5cu041e\x5cu04\
3a\x5cu043d\x5cu043e \x5c\
u041e\x5cu043a\x5cu044\
0 \x5cu041e\x5cu043a\x5cu\
0440\x5cu0443\x5cu0436\
\x5cu0435\x5cu043d\x5cu04\
38\x5cu0435 \x5cu041e\x5c\
u0442\x5cu043b\x5cu041\
8\x5cu043d\x5cu0444\x5cu0\
414\x5cu043e\x5cu0431\x5c\
u0430\x5cu0432\x5cu043\
8\x5cu0442\x5cu044c \x5cu\
041e\x5cu0442\x5cu043b\
\x5cu0418\x5cu043d\x5cu04\
44\x5cu0423\x5cu0434\x5cu\
0430\x5cu043b\x5cu0438\
\x5cu0442\x5cu044c \x5cu0\
41e\x5cu0442\x5cu0447\x5c\
u0435\x5cu0442 \x5cu04\
1e\x5cu0442\x5cu0447\x5cu\
0435\x5cu0442\x5cu0410\
\x5cu043d\x5cu0430\x5cu04\
3b \x5cu041e\x5cu0442\x5c\
u0447\x5cu0435\x5cu044\
2\x5cu0418\x5cu043d\x5cu0\
442 \x5cu041f\x5cu0430\
\x5cu043f\x5cu043a\x5cu04\
30\x5cu0421\x5cu0443\x5cu\
0449\x5cu0435\x5cu0441\
\x5cu0442\x5cu0432\x5cu04\
43\x5cu0435\x5cu0442 \x5c\
u041f\x5cu0430\x5cu044\
3\x5cu0437\x5cu0430 \x5cu\
041f\x5cu0412\x5cu044b\
\x5cu0431\x5cu043e\x5cu04\
40SQL \x5cu041f\x5cu04\
35\x5cu0440\x5cu0435\x5cu\
0438\x5cu043c\x5cu0435\
\x5cu043d\x5cu043e\x5cu04\
32\x5cu0430\x5cu0442\x5cu\
044c\x5cu0424\x5cu0430\
\x5cu0439\x5cu043b \x5cu0\
41f\x5cu0435\x5cu0440\x5c\
u0435\x5cu043c\x5cu043\
5\x5cu043d\x5cu043d\x5cu0\
44b\x5cu0435 \x5cu041f\
\x5cu0435\x5cu0440\x5cu04\
35\x5cu043c\x5cu0435\x5cu\
0441\x5cu0442\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
24\x5cu0430\x5cu0439\x5cu\
043b \x5cu041f\x5cu043\
e\x5cu0434\x5cu0441\x5cu0\
442\x5cu0440 \x5cu041f\
\x5cu043e\x5cu0438\x5cu04\
41\x5cu043a\x5cu041f\x5cu\
043e\x5cu0434\x5cu0441\
\x5cu0442\x5cu0440 \x5cu0\
41f\x5cu043e\x5cu0438\x5c\
u0441\x5cu043a\x5cu042\
1\x5cu0442\x5cu0440 \x5cu\
041f\x5cu043e\x5cu043b\
\x5cu0443\x5cu0447\x5cu04\
38\x5cu0442\x5cu044c\x5cu\
0418\x5cu0414\x5cu0422\
\x5cu0430\x5cu0431\x5cu04\
3b\x5cu0438\x5cu0446\x5cu\
044b \x5cu041f\x5cu043\
e\x5cu043b\x5cu044c\x5cu0\
437\x5cu043e\x5cu0432\x5c\
u0430\x5cu0442\x5cu043\
5\x5cu043b\x5cu044c\x5cu0\
414\x5cu043e\x5cu043f\x5c\
u043e\x5cu043b\x5cu043\
d\x5cu0438\x5cu0442\x5cu0\
435\x5cu043b\x5cu044c\x5c\
u043d\x5cu043e \x5cu04\
1f\x5cu043e\x5cu043b\x5cu\
044c\x5cu0437\x5cu043e\
\x5cu0432\x5cu0430\x5cu04\
42\x5cu0435\x5cu043b\x5cu\
044c\x5cu0418\x5cu0414\
 \x5cu041f\x5cu043e\x5cu0\
43b\x5cu044c\x5cu0437\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu0442\x5cu0435\x5cu0\
43b\x5cu044c\x5cu0418\x5c\
u043c\x5cu044f \x5cu04\
1f\x5cu043e\x5cu043b\x5cu\
044c\x5cu0437\x5cu043e\
\x5cu0432\x5cu0430\x5cu04\
42\x5cu0435\x5cu043b\x5cu\
044c\x5cu0421\x5cu0442\
\x5cu0430\x5cu0442\x5cu04\
43\x5cu0441 \x5cu041f\x5c\
u0440\x5cu0435\x5cu044\
0\x5cu0432\x5cu0430\x5cu0\
442\x5cu044c \x5cu041f\
\x5cu0440\x5cu043e\x5cu04\
32\x5cu0435\x5cu0440\x5cu\
0438\x5cu0442\x5cu044c\
\x5cu041f\x5cu0430\x5cu04\
40\x5cu0430\x5cu043c\x5cu\
0435\x5cu0442\x5cu0440\
 \x5cu041f\x5cu0440\x5cu0\
43e\x5cu0432\x5cu0435\x5c\
u0440\x5cu0438\x5cu044\
2\x5cu044c\x5cu041f\x5cu0\
430\x5cu0440\x5cu0430\x5c\
u043c\x5cu0435\x5cu044\
2\x5cu0440\x5cu0417\x5cu0\
43d\x5cu0430\x5cu0447 \
\x5cu041f\x5cu0440\x5cu04\
3e\x5cu0432\x5cu0435\x5cu\
0440\x5cu0438\x5cu0442\
\x5cu044c\x5cu0423\x5cu04\
41\x5cu043b\x5cu043e\x5cu\
0432\x5cu0438\x5cu0435\
 \x5cu0420\x5cu0430\x5cu0\
437\x5cu0431\x5cu0421\x5c\
u0442\x5cu0440 \x5cu04\
20\x5cu0430\x5cu0437\x5cu\
043d\x5cu0412\x5cu0440\
\x5cu0435\x5cu043c\x5cu04\
4f \x5cu0420\x5cu0430\x5c\
u0437\x5cu043d\x5cu041\
4\x5cu0430\x5cu0442 \x5cu\
0420\x5cu0430\x5cu0437\
\x5cu043d\x5cu0414\x5cu04\
30\x5cu0442\x5cu0430\x5cu\
0412\x5cu0440\x5cu0435\
\x5cu043c\x5cu044f \x5cu0\
420\x5cu0430\x5cu0437\x5c\
u043d\x5cu0420\x5cu043\
0\x5cu0431\x5cu0412\x5cu0\
440\x5cu0435\x5cu043c\x5c\
u044f \x5cu0420\x5cu04\
35\x5cu0433\x5cu0423\x5cu\
0441\x5cu0442\x5cu0412\
\x5cu0440\x5cu0435\x5cu04\
3c \x5cu0420\x5cu0435\x5c\
u0433\x5cu0423\x5cu044\
1\x5cu0442\x5cu0414\x5cu0\
430\x5cu0442 \x5cu0420\
\x5cu0435\x5cu0433\x5cu04\
23\x5cu0441\x5cu0442\x5cu\
0427\x5cu0441\x5cu043b\
 \x5cu0420\x5cu0435\x5cu0\
434\x5cu0422\x5cu0435\x5c\
u043a\x5cu0441\x5cu044\
2 \x5cu0420\x5cu0435\x5cu\
0435\x5cu0441\x5cu0442\
\x5cu0440\x5cu0417\x5cu04\
30\x5cu043f\x5cu0438\x5cu\
0441\x5cu044c \x5cu042\
0\x5cu0435\x5cu0435\x5cu0\
441\x5cu0442\x5cu0440\x5c\
u0421\x5cu043f\x5cu043\
8\x5cu0441\x5cu043e\x5cu0\
43a\x5cu0418\x5cu043c\x5c\
u0435\x5cu043d\x5cu041\
f\x5cu0430\x5cu0440\x5cu0\
430\x5cu043c \x5cu0420\
\x5cu0435\x5cu0435\x5cu04\
41\x5cu0442\x5cu0440\x5cu\
0427\x5cu0442\x5cu0435\
\x5cu043d\x5cu0438\x5cu04\
35 \x5cu0420\x5cu0435\x5c\
u043a\x5cu0432\x5cu042\
1\x5cu043f\x5cu0440 \x5cu\
0420\x5cu0435\x5cu043a\
\x5cu0432\x5cu0421\x5cu04\
3f\x5cu0440\x5cu041f\x5cu\
0440 \x5cu0421\x5cu043\
5\x5cu0433\x5cu043e\x5cu0\
434\x5cu043d\x5cu044f \
\x5cu0421\x5cu0435\x5cu04\
39\x5cu0447\x5cu0430\x5cu\
0441 \x5cu0421\x5cu043\
5\x5cu0440\x5cu0432\x5cu0\
435\x5cu0440 \x5cu0421\
\x5cu0435\x5cu0440\x5cu04\
32\x5cu0435\x5cu0440\x5cu\
041f\x5cu0440\x5cu043e\
\x5cu0446\x5cu0435\x5cu04\
41\x5cu0441\x5cu0418\x5cu\
0414 \x5cu0421\x5cu043\
5\x5cu0440\x5cu0442\x5cu0\
438\x5cu0444\x5cu0438\x5c\
u043a\x5cu0430\x5cu044\
2\x5cu0424\x5cu0430\x5cu0\
439\x5cu043b\x5cu0421\x5c\
u0447\x5cu0438\x5cu044\
2\x5cu0430\x5cu0442\x5cu0\
44c \x5cu0421\x5cu0436\
\x5cu041f\x5cu0440\x5cu04\
3e\x5cu0431 \x5cu0421\x5c\
u0438\x5cu043c\x5cu043\
2\x5cu043e\x5cu043b \x5cu\
0421\x5cu0438\x5cu0441\
\x5cu0442\x5cu0435\x5cu04\
3c\x5cu0430\x5cu0414\x5cu\
0438\x5cu0440\x5cu0435\
\x5cu043a\x5cu0442\x5cu04\
43\x5cu043c\x5cu041a\x5cu\
043e\x5cu0434 \x5cu042\
1\x5cu0438\x5cu0441\x5cu0\
442\x5cu0435\x5cu043c\x5c\
u0430\x5cu0418\x5cu043\
d\x5cu0444\x5cu043e\x5cu0\
440\x5cu043c\x5cu0430\x5c\
u0446\x5cu0438\x5cu044\
f \x5cu0421\x5cu0438\x5cu\
0441\x5cu0442\x5cu0435\
\x5cu043c\x5cu0430\x5cu04\
1a\x5cu043e\x5cu0434 \x5c\
u0421\x5cu043e\x5cu043\
4\x5cu0435\x5cu0440\x5cu0\
436\x5cu0438\x5cu0442 \
\x5cu0421\x5cu043e\x5cu04\
35\x5cu0434\x5cu0438\x5cu\
043d\x5cu0435\x5cu043d\
\x5cu0438\x5cu0435\x5cu04\
17\x5cu0430\x5cu043a\x5cu\
0440\x5cu044b\x5cu0442\
\x5cu044c \x5cu0421\x5cu0\
43e\x5cu0435\x5cu0434\x5c\
u0438\x5cu043d\x5cu043\
5\x5cu043d\x5cu0438\x5cu0\
435\x5cu041e\x5cu0442\x5c\
u043a\x5cu0440\x5cu044\
b\x5cu0442\x5cu044c \x5cu\
0421\x5cu043e\x5cu0437\
\x5cu0434\x5cu0430\x5cu04\
42\x5cu044c\x5cu0414\x5cu\
0438\x5cu0430\x5cu043b\
\x5cu043e\x5cu0433 \x5cu0\
421\x5cu043e\x5cu0437\x5c\
u0434\x5cu0430\x5cu044\
2\x5cu044c\x5cu0414\x5cu0\
438\x5cu0430\x5cu043b\x5c\
u043e\x5cu0433\x5cu041\
2\x5cu044b\x5cu0431\x5cu0\
43e\x5cu0440\x5cu0430\x5c\
u0418\x5cu0437\x5cu041\
4\x5cu0432\x5cu0443\x5cu0\
445\x5cu0421\x5cu043f\x5c\
u0438\x5cu0441\x5cu043\
a\x5cu043e\x5cu0432 \x5cu\
0421\x5cu043e\x5cu0437\
\x5cu0434\x5cu0430\x5cu04\
42\x5cu044c\x5cu0414\x5cu\
0438\x5cu0430\x5cu043b\
\x5cu043e\x5cu0433\x5cu04\
12\x5cu044b\x5cu0431\x5cu\
043e\x5cu0440\x5cu0430\
\x5cu041f\x5cu0430\x5cu04\
3f\x5cu043a\x5cu0438 \x5c\
u0421\x5cu043e\x5cu043\
7\x5cu0434\x5cu0430\x5cu0\
442\x5cu044c\x5cu0414\x5c\
u0438\x5cu0430\x5cu043\
b\x5cu043e\x5cu0433\x5cu0\
41e\x5cu0442\x5cu043a\x5c\
u0440\x5cu044b\x5cu044\
2\x5cu0438\x5cu044f\x5cu0\
424\x5cu0430\x5cu0439\x5c\
u043b\x5cu0430 \x5cu04\
21\x5cu043e\x5cu0437\x5cu\
0434\x5cu0430\x5cu0442\
\x5cu044c\x5cu0414\x5cu04\
38\x5cu0430\x5cu043b\x5cu\
043e\x5cu0433\x5cu0421\
\x5cu043e\x5cu0445\x5cu04\
40\x5cu0430\x5cu043d\x5cu\
0435\x5cu043d\x5cu0438\
\x5cu044f\x5cu0424\x5cu04\
30\x5cu0439\x5cu043b\x5cu\
0430 \x5cu0421\x5cu043\
e\x5cu0437\x5cu0434\x5cu0\
430\x5cu0442\x5cu044c\x5c\
u0417\x5cu0430\x5cu043\
f\x5cu0440\x5cu043e\x5cu0\
441 \x5cu0421\x5cu043e\
\x5cu0437\x5cu0434\x5cu04\
30\x5cu0442\x5cu044c\x5cu\
0418\x5cu043d\x5cu0434\
\x5cu0438\x5cu043a\x5cu04\
30\x5cu0442\x5cu043e\x5cu\
0440 \x5cu0421\x5cu043\
e\x5cu0437\x5cu0434\x5cu0\
430\x5cu0442\x5cu044c\x5c\
u0418\x5cu0441\x5cu043\
a\x5cu043b\x5cu044e\x5cu0\
447\x5cu0435\x5cu043d\x5c\
u0438\x5cu0435 \x5cu04\
21\x5cu043e\x5cu0437\x5cu\
0434\x5cu0430\x5cu0442\
\x5cu044c\x5cu041a\x5cu04\
4d\x5cu0448\x5cu0438\x5cu\
0440\x5cu043e\x5cu0432\
\x5cu0430\x5cu043d\x5cu04\
3d\x5cu044b\x5cu0439\x5cu\
0421\x5cu043f\x5cu0440\
\x5cu0430\x5cu0432\x5cu04\
3e\x5cu0447\x5cu043d\x5cu\
0438\x5cu043a \x5cu042\
1\x5cu043e\x5cu0437\x5cu0\
434\x5cu0430\x5cu0442\x5c\
u044c\x5cu041c\x5cu043\
0\x5cu0441\x5cu0441\x5cu0\
438\x5cu0432 \x5cu0421\
\x5cu043e\x5cu0437\x5cu04\
34\x5cu0430\x5cu0442\x5cu\
044c\x5cu041d\x5cu0430\
\x5cu0431\x5cu043e\x5cu04\
40\x5cu0414\x5cu0430\x5cu\
043d\x5cu043d\x5cu044b\
\x5cu0445 \x5cu0421\x5cu0\
43e\x5cu0437\x5cu0434\x5c\
u0430\x5cu0442\x5cu044\
c\x5cu041e\x5cu0431\x5cu0\
44a\x5cu0435\x5cu043a\x5c\
u0442 \x5cu0421\x5cu04\
3e\x5cu0437\x5cu0434\x5cu\
0430\x5cu0442\x5cu044c\
\x5cu041e\x5cu0442\x5cu04\
47\x5cu0435\x5cu0442 \x5c\
u0421\x5cu043e\x5cu043\
7\x5cu0434\x5cu0430\x5cu0\
442\x5cu044c\x5cu041f\x5c\
u0430\x5cu043f\x5cu043\
a\x5cu0443 \x5cu0421\x5cu\
043e\x5cu0437\x5cu0434\
\x5cu0430\x5cu0442\x5cu04\
4c\x5cu0420\x5cu0435\x5cu\
0434\x5cu0430\x5cu043a\
\x5cu0442\x5cu043e\x5cu04\
40 \x5cu0421\x5cu043e\x5c\
u0437\x5cu0434\x5cu043\
0\x5cu0442\x5cu044c\x5cu0\
421\x5cu043e\x5cu0435\x5c\
u0434\x5cu0438\x5cu043\
d\x5cu0435\x5cu043d\x5cu0\
438\x5cu0435 \x5cu0421\
\x5cu043e\x5cu0437\x5cu04\
34\x5cu0430\x5cu0442\x5cu\
044c\x5cu0421\x5cu043f\
\x5cu0438\x5cu0441\x5cu04\
3e\x5cu043a \x5cu0421\x5c\
u043e\x5cu0437\x5cu043\
4\x5cu0430\x5cu0442\x5cu0\
44c\x5cu0421\x5cu043f\x5c\
u0438\x5cu0441\x5cu043\
e\x5cu043a\x5cu0421\x5cu0\
442\x5cu0440\x5cu043e\x5c\
u043a \x5cu0421\x5cu04\
3e\x5cu0437\x5cu0434\x5cu\
0430\x5cu0442\x5cu044c\
\x5cu0421\x5cu043f\x5cu04\
40\x5cu0430\x5cu0432\x5cu\
043e\x5cu0447\x5cu043d\
\x5cu0438\x5cu043a \x5cu0\
421\x5cu043e\x5cu0437\x5c\
u0434\x5cu0430\x5cu044\
2\x5cu044c\x5cu0421\x5cu0\
446\x5cu0435\x5cu043d\x5c\
u0430\x5cu0440\x5cu043\
8\x5cu0439 \x5cu0421\x5cu\
043e\x5cu0437\x5cu0434\
\x5cu0421\x5cu043f\x5cu04\
40 \x5cu0421\x5cu043e\x5c\
u0441\x5cu0442\x5cu042\
1\x5cu043f\x5cu0440 \x5cu\
0421\x5cu043e\x5cu0445\
\x5cu0440 \x5cu0421\x5cu0\
43e\x5cu0445\x5cu0440\x5c\
u0421\x5cu043f\x5cu044\
0 \x5cu0421\x5cu043f\x5cu\
0438\x5cu0441\x5cu043e\
\x5cu043a\x5cu0421\x5cu04\
38\x5cu0441\x5cu0442\x5cu\
0435\x5cu043c \x5cu042\
1\x5cu043f\x5cu0440 \x5cu\
0421\x5cu043f\x5cu0440\
\x5cu0430\x5cu0432\x5cu04\
3e\x5cu0447\x5cu043d\x5cu\
0438\x5cu043a \x5cu042\
1\x5cu043f\x5cu0440\x5cu0\
411\x5cu043b\x5cu043e\x5c\
u043a\x5cu0415\x5cu044\
1\x5cu0442\x5cu044c \x5cu\
0421\x5cu043f\x5cu0440\
\x5cu0411\x5cu043b\x5cu04\
3e\x5cu043a\x5cu0421\x5cu\
043d\x5cu044f\x5cu0442\
\x5cu044c \x5cu0421\x5cu0\
43f\x5cu0440\x5cu0411\x5c\
u043b\x5cu043e\x5cu043\
a\x5cu0421\x5cu043d\x5cu0\
44f\x5cu0442\x5cu044c\x5c\
u0420\x5cu0430\x5cu044\
1\x5cu0448 \x5cu0421\x5cu\
043f\x5cu0440\x5cu0411\
\x5cu043b\x5cu043e\x5cu04\
3a\x5cu0423\x5cu0441\x5cu\
0442\x5cu0430\x5cu043d\
\x5cu043e\x5cu0432\x5cu04\
38\x5cu0442\x5cu044c \x5c\
u0421\x5cu043f\x5cu044\
0\x5cu0418\x5cu0437\x5cu0\
43c\x5cu041d\x5cu0430\x5c\
u0431\x5cu0414\x5cu043\
0\x5cu043d \x5cu0421\x5cu\
043f\x5cu0440\x5cu041a\
\x5cu043e\x5cu0434 \x5cu0\
421\x5cu043f\x5cu0440\x5c\
u041d\x5cu043e\x5cu043\
c\x5cu0435\x5cu0440 \x5cu\
0421\x5cu043f\x5cu0440\
\x5cu041e\x5cu0431\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
0438\x5cu0442\x5cu044c\
 \x5cu0421\x5cu043f\x5cu0\
440\x5cu041e\x5cu0442\x5c\
u043a\x5cu0440\x5cu044\
b\x5cu0442\x5cu044c \x5cu\
0421\x5cu043f\x5cu0440\
\x5cu041e\x5cu0442\x5cu04\
3c\x5cu0435\x5cu043d\x5cu\
0438\x5cu0442\x5cu044c\
 \x5cu0421\x5cu043f\x5cu0\
440\x5cu041f\x5cu0430\x5c\
u0440\x5cu0430\x5cu043\
c \x5cu0421\x5cu043f\x5cu\
0440\x5cu041f\x5cu043e\
\x5cu043b\x5cu0435\x5cu04\
17\x5cu043d\x5cu0430\x5cu\
0447 \x5cu0421\x5cu043\
f\x5cu0440\x5cu041f\x5cu0\
43e\x5cu043b\x5cu0435\x5c\
u0418\x5cu043c\x5cu044\
f \x5cu0421\x5cu043f\x5cu\
0440\x5cu0420\x5cu0435\
\x5cu043a\x5cu0432 \x5cu0\
421\x5cu043f\x5cu0440\x5c\
u0420\x5cu0435\x5cu043\
a\x5cu0432\x5cu0412\x5cu0\
432\x5cu0435\x5cu0434\x5c\
u0417\x5cu043d \x5cu04\
21\x5cu043f\x5cu0440\x5cu\
0420\x5cu0435\x5cu043a\
\x5cu0432\x5cu041d\x5cu04\
3e\x5cu0432\x5cu044b\x5cu\
0435 \x5cu0421\x5cu043\
f\x5cu0440\x5cu0420\x5cu0\
435\x5cu043a\x5cu0432\x5c\
u041f\x5cu0440 \x5cu04\
21\x5cu043f\x5cu0440\x5cu\
0420\x5cu0435\x5cu043a\
\x5cu0432\x5cu041f\x5cu04\
40\x5cu0435\x5cu0434\x5cu\
0417\x5cu043d \x5cu042\
1\x5cu043f\x5cu0440\x5cu0\
420\x5cu0435\x5cu043a\x5c\
u0432\x5cu0420\x5cu043\
5\x5cu0436\x5cu0438\x5cu0\
43c \x5cu0421\x5cu043f\
\x5cu0440\x5cu0420\x5cu04\
35\x5cu043a\x5cu0432\x5cu\
0422\x5cu0438\x5cu043f\
\x5cu0422\x5cu0435\x5cu04\
3a\x5cu0441\x5cu0442 \x5c\
u0421\x5cu043f\x5cu044\
0\x5cu0421\x5cu043e\x5cu0\
437\x5cu0434\x5cu0430\x5c\
u0442\x5cu044c \x5cu04\
21\x5cu043f\x5cu0440\x5cu\
0421\x5cu043e\x5cu0441\
\x5cu0442 \x5cu0421\x5cu0\
43f\x5cu0440\x5cu0421\x5c\
u043e\x5cu0445\x5cu044\
0\x5cu0430\x5cu043d\x5cu0\
438\x5cu0442\x5cu044c \
\x5cu0421\x5cu043f\x5cu04\
40\x5cu0422\x5cu0431\x5cu\
043b\x5cu0418\x5cu0442\
\x5cu043e\x5cu0433 \x5cu0\
421\x5cu043f\x5cu0440\x5c\
u0422\x5cu0431\x5cu043\
b\x5cu0421\x5cu0442\x5cu0\
440 \x5cu0421\x5cu043f\
\x5cu0440\x5cu0422\x5cu04\
31\x5cu043b\x5cu0421\x5cu\
0442\x5cu0440\x5cu041a\
\x5cu043e\x5cu043b \x5cu0\
421\x5cu043f\x5cu0440\x5c\
u0422\x5cu0431\x5cu043\
b\x5cu0421\x5cu0442\x5cu0\
440\x5cu041c\x5cu0430\x5c\
u043a\x5cu0441 \x5cu04\
21\x5cu043f\x5cu0440\x5cu\
0422\x5cu0431\x5cu043b\
\x5cu0421\x5cu0442\x5cu04\
40\x5cu041c\x5cu0438\x5cu\
043d \x5cu0421\x5cu043\
f\x5cu0440\x5cu0422\x5cu0\
431\x5cu043b\x5cu0421\x5c\
u0442\x5cu0440\x5cu041\
f\x5cu0440\x5cu0435\x5cu0\
434 \x5cu0421\x5cu043f\
\x5cu0440\x5cu0422\x5cu04\
31\x5cu043b\x5cu0421\x5cu\
0442\x5cu0440\x5cu0421\
\x5cu043b\x5cu0435\x5cu04\
34 \x5cu0421\x5cu043f\x5c\
u0440\x5cu0422\x5cu043\
1\x5cu043b\x5cu0421\x5cu0\
442\x5cu0440\x5cu0421\x5c\
u043e\x5cu0437\x5cu043\
4 \x5cu0421\x5cu043f\x5cu\
0440\x5cu0422\x5cu0431\
\x5cu043b\x5cu0421\x5cu04\
42\x5cu0440\x5cu0423\x5cu\
0434 \x5cu0421\x5cu043\
f\x5cu0440\x5cu0422\x5cu0\
435\x5cu043a\x5cu041f\x5c\
u0440\x5cu0435\x5cu043\
4\x5cu0441\x5cu0442 \x5cu\
0421\x5cu043f\x5cu0440\
\x5cu0423\x5cu0434\x5cu04\
30\x5cu043b\x5cu0438\x5cu\
0442\x5cu044c \x5cu042\
1\x5cu0440\x5cu0430\x5cu0\
432\x5cu043d\x5cu0438\x5c\
u0442\x5cu044c\x5cu042\
1\x5cu0442\x5cu0440 \x5cu\
0421\x5cu0442\x5cu0440\
\x5cu0412\x5cu0435\x5cu04\
40\x5cu0445\x5cu0420\x5cu\
0435\x5cu0433\x5cu0438\
\x5cu0441\x5cu0442\x5cu04\
40 \x5cu0421\x5cu0442\x5c\
u0440\x5cu041d\x5cu043\
8\x5cu0436\x5cu043d\x5cu0\
420\x5cu0435\x5cu0433\x5c\
u0438\x5cu0441\x5cu044\
2\x5cu0440 \x5cu0421\x5cu\
0442\x5cu0440\x5cu0422\
\x5cu0431\x5cu043b\x5cu04\
21\x5cu043f\x5cu0440 \x5c\
u0421\x5cu0443\x5cu043\
c\x5cu041f\x5cu0440\x5cu0\
43e\x5cu043f \x5cu0421\
\x5cu0446\x5cu0435\x5cu04\
3d\x5cu0430\x5cu0440\x5cu\
0438\x5cu0439 \x5cu042\
1\x5cu0446\x5cu0435\x5cu0\
43d\x5cu0430\x5cu0440\x5c\
u0438\x5cu0439\x5cu041\
f\x5cu0430\x5cu0440\x5cu0\
430\x5cu043c \x5cu0422\
\x5cu0435\x5cu043a\x5cu04\
12\x5cu0435\x5cu0440\x5cu\
0441\x5cu0438\x5cu044f\
 \x5cu0422\x5cu0435\x5cu0\
43a\x5cu041e\x5cu0440\x5c\
u0433 \x5cu0422\x5cu04\
3e\x5cu0447\x5cu043d \x5c\
u0422\x5cu0440\x5cu043\
0\x5cu043d \x5cu0422\x5cu\
0440\x5cu0430\x5cu043d\
\x5cu0441\x5cu043b\x5cu04\
38\x5cu0442\x5cu0435\x5cu\
0440\x5cu0430\x5cu0446\
\x5cu0438\x5cu044f \x5cu0\
423\x5cu0434\x5cu0430\x5c\
u043b\x5cu0438\x5cu044\
2\x5cu044c\x5cu0422\x5cu0\
430\x5cu0431\x5cu043b\x5c\
u0438\x5cu0446\x5cu044\
3 \x5cu0423\x5cu0434\x5cu\
0430\x5cu043b\x5cu0438\
\x5cu0442\x5cu044c\x5cu04\
24\x5cu0430\x5cu0439\x5cu\
043b \x5cu0423\x5cu043\
4\x5cu0421\x5cu043f\x5cu0\
440 \x5cu0423\x5cu0434\
\x5cu0421\x5cu0442\x5cu04\
40\x5cu0422\x5cu0431\x5cu\
043b\x5cu0421\x5cu043f\
\x5cu0440 \x5cu0423\x5cu0\
441\x5cu0442 \x5cu0423\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu043d\x5cu043e\x5cu\
0432\x5cu043a\x5cu0438\
\x5cu041a\x5cu043e\x5cu04\
3d\x5cu0441\x5cu0442\x5cu\
0430\x5cu043d\x5cu0442\
 \x5cu0424\x5cu0430\x5cu0\
439\x5cu043b\x5cu0410\x5c\
u0442\x5cu0440\x5cu043\
8\x5cu0431\x5cu0443\x5cu0\
442\x5cu0421\x5cu0447\x5c\
u0438\x5cu0442\x5cu043\
0\x5cu0442\x5cu044c \x5cu\
0424\x5cu0430\x5cu0439\
\x5cu043b\x5cu0410\x5cu04\
42\x5cu0440\x5cu0438\x5cu\
0431\x5cu0443\x5cu0442\
\x5cu0423\x5cu0441\x5cu04\
42\x5cu0430\x5cu043d\x5cu\
043e\x5cu0432\x5cu0438\
\x5cu0442\x5cu044c \x5cu0\
424\x5cu0430\x5cu0439\x5c\
u043b\x5cu0412\x5cu044\
0\x5cu0435\x5cu043c\x5cu0\
44f \x5cu0424\x5cu0430\
\x5cu0439\x5cu043b\x5cu04\
12\x5cu0440\x5cu0435\x5cu\
043c\x5cu044f\x5cu0423\
\x5cu0441\x5cu0442\x5cu04\
30\x5cu043d\x5cu043e\x5cu\
0432\x5cu0438\x5cu0442\
\x5cu044c \x5cu0424\x5cu0\
430\x5cu0439\x5cu043b\x5c\
u0412\x5cu044b\x5cu043\
1\x5cu0440\x5cu0430\x5cu0\
442\x5cu044c \x5cu0424\
\x5cu0430\x5cu0439\x5cu04\
3b\x5cu0417\x5cu0430\x5cu\
043d\x5cu044f\x5cu0442\
 \x5cu0424\x5cu0430\x5cu0\
439\x5cu043b\x5cu0417\x5c\
u0430\x5cu043f\x5cu043\
8\x5cu0441\x5cu0430\x5cu0\
442\x5cu044c \x5cu0424\
\x5cu0430\x5cu0439\x5cu04\
3b\x5cu0418\x5cu0441\x5cu\
043a\x5cu0430\x5cu0442\
\x5cu044c \x5cu0424\x5cu0\
430\x5cu0439\x5cu043b\x5c\
u041a\x5cu043e\x5cu043\
f\x5cu0438\x5cu0440\x5cu0\
43e\x5cu0432\x5cu0430\x5c\
u0442\x5cu044c \x5cu04\
24\x5cu0430\x5cu0439\x5cu\
043b\x5cu041c\x5cu043e\
\x5cu0436\x5cu043d\x5cu04\
3e\x5cu0427\x5cu0438\x5cu\
0442\x5cu0430\x5cu0442\
\x5cu044c \x5cu0424\x5cu0\
430\x5cu0439\x5cu043b\x5c\
u041e\x5cu0442\x5cu043\
a\x5cu0440\x5cu044b\x5cu0\
442\x5cu044c \x5cu0424\
\x5cu0430\x5cu0439\x5cu04\
3b\x5cu041f\x5cu0435\x5cu\
0440\x5cu0435\x5cu0438\
\x5cu043c\x5cu0435\x5cu04\
3d\x5cu043e\x5cu0432\x5cu\
0430\x5cu0442\x5cu044c\
 \x5cu0424\x5cu0430\x5cu0\
439\x5cu043b\x5cu041f\x5c\
u0435\x5cu0440\x5cu043\
5\x5cu043a\x5cu043e\x5cu0\
434\x5cu0438\x5cu0440\x5c\
u043e\x5cu0432\x5cu043\
0\x5cu0442\x5cu044c \x5cu\
0424\x5cu0430\x5cu0439\
\x5cu043b\x5cu041f\x5cu04\
35\x5cu0440\x5cu0435\x5cu\
043c\x5cu0435\x5cu0441\
\x5cu0442\x5cu0438\x5cu04\
42\x5cu044c \x5cu0424\x5c\
u0430\x5cu0439\x5cu043\
b\x5cu041f\x5cu0440\x5cu0\
43e\x5cu0441\x5cu043c\x5c\
u043e\x5cu0442\x5cu044\
0\x5cu0435\x5cu0442\x5cu0\
44c \x5cu0424\x5cu0430\
\x5cu0439\x5cu043b\x5cu04\
20\x5cu0430\x5cu0437\x5cu\
043c\x5cu0435\x5cu0440\
 \x5cu0424\x5cu0430\x5cu0\
439\x5cu043b\x5cu0421\x5c\
u043e\x5cu0437\x5cu043\
4\x5cu0430\x5cu0442\x5cu0\
44c \x5cu0424\x5cu0430\
\x5cu0439\x5cu043b\x5cu04\
21\x5cu0441\x5cu044b\x5cu\
043b\x5cu043a\x5cu0430\
\x5cu0421\x5cu043e\x5cu04\
37\x5cu0434\x5cu0430\x5cu\
0442\x5cu044c \x5cu042\
4\x5cu0430\x5cu0439\x5cu0\
43b\x5cu0421\x5cu0443\x5c\
u0449\x5cu0435\x5cu044\
1\x5cu0442\x5cu0432\x5cu0\
443\x5cu0435\x5cu0442 \
\x5cu0424\x5cu0430\x5cu04\
39\x5cu043b\x5cu0421\x5cu\
0447\x5cu0438\x5cu0442\
\x5cu0430\x5cu0442\x5cu04\
4c \x5cu0424\x5cu0430\x5c\
u0439\x5cu043b\x5cu042\
3\x5cu0434\x5cu0430\x5cu0\
43b\x5cu0438\x5cu0442\x5c\
u044c \x5cu0424\x5cu04\
3c\x5cu0442SQL\x5cu041\
4\x5cu0430\x5cu0442 \x5cu\
0424\x5cu043c\x5cu0442\
\x5cu0414\x5cu0430\x5cu04\
42 \x5cu0424\x5cu043c\x5c\
u0442\x5cu0421\x5cu044\
2\x5cu0440 \x5cu0424\x5cu\
043c\x5cu0442\x5cu0427\
\x5cu0441\x5cu043b \x5cu0\
424\x5cu043e\x5cu0440\x5c\
u043c\x5cu0430\x5cu044\
2 \x5cu0426\x5cu041c\x5cu\
0430\x5cu0441\x5cu0441\
\x5cu0438\x5cu0432\x5cu04\
2d\x5cu043b\x5cu0435\x5cu\
043c\x5cu0435\x5cu043d\
\x5cu0442 \x5cu0426\x5cu0\
41d\x5cu0430\x5cu0431\x5c\
u043e\x5cu0440\x5cu041\
4\x5cu0430\x5cu043d\x5cu0\
43d\x5cu044b\x5cu0445\x5c\
u0420\x5cu0435\x5cu043\
a\x5cu0432\x5cu0438\x5cu0\
437\x5cu0438\x5cu0442 \
\x5cu0426\x5cu041f\x5cu04\
3e\x5cu0434\x5cu0441\x5cu\
0442\x5cu0440 \x22\x0a},b\
egin:e,end:\x22\x5c\x5c(\x22\
,returnBegin:!0,\
excludeEnd:!0},I\
,A,T,_,O]},N,I,A\
,T,_,O]}}})()\x0a;h\
ljs.registerLang\
uage(\x22isbl\x22,S)})\
();/*! `java` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22\x0a;var e\
=\x22[0-9](_*[0-9])\
*\x22,a=`\x5c\x5c.(${e})`\
,n=\x22[0-9a-fA-F](\
_*[0-9a-fA-F])*\x22\
,s={\x0aclassName:\x22\
number\x22,variants\
:[{\x0abegin:`(\x5c\x5cb(\
${e})((${a})|\x5c\x5c.\
)?|(${a}))[eE][+\
-]?(${e})[fFdD]?\
\x5c\x5cb`},{\x0abegin:`\x5c\
\x5cb(${e})((${a})[\
fFdD]?\x5c\x5cb|\x5c\x5c.([f\
FdD]\x5c\x5cb)?)`},{be\
gin:`(${a})[fFdD\
]?\x5c\x5cb`\x0a},{begin:\
`\x5c\x5cb(${e})[fFdD]\
\x5c\x5cb`},{\x0abegin:`\x5c\
\x5cb0[xX]((${n})\x5c\x5c\
.?|(${n})?\x5c\x5c.(${\
n}))[pP][+-]?(${\
e})[fFdD]?\x5c\x5cb`},\
{\x0abegin:\x22\x5c\x5cb(0|[\
1-9](_*[0-9])*)[\
lL]?\x5c\x5cb\x22},{begin\
:`\x5c\x5cb0[xX](${n})\
[lL]?\x5c\x5cb`},{\x0abeg\
in:\x22\x5c\x5cb0(_*[0-7]\
)*[lL]?\x5c\x5cb\x22},{be\
gin:\x22\x5c\x5cb0[bB][01\
](_*[01])*[lL]?\x5c\
\x5cb\x22}],\x0arelevance\
:0};function t(e\
,a,n){return-1==\
=n?\x22\x22:e.replace(\
a,(s=>t(e,a,n-1)\
))}\x0areturn e=>{\x0a\
const a=e.regex,\
n=\x22[\x5cxc0-\x5cu02b8a\
-zA-Z_$][\x5cxc0-\x5cu\
02b8a-zA-Z_$0-9]\
*\x22,i=n+t(\x22(?:<\x22+\
n+\x22~~~(?:\x5c\x5cs*,\x5c\x5c\
s*\x22+n+\x22~~~)*>)?\x22\
,/~~~/g,2),r={\x0ak\
eyword:[\x22synchro\
nized\x22,\x22abstract\
\x22,\x22private\x22,\x22var\
\x22,\x22static\x22,\x22if\x22,\
\x22const \x22,\x22for\x22,\x22\
while\x22,\x22strictfp\
\x22,\x22finally\x22,\x22pro\
tected\x22,\x22import\x22\
,\x22native\x22,\x22final\
\x22,\x22void\x22,\x22enum\x22,\
\x22else\x22,\x22break\x22,\x22\
transient\x22,\x22catc\
h\x22,\x22instanceof\x22,\
\x22volatile\x22,\x22case\
\x22,\x22assert\x22,\x22pack\
age\x22,\x22default\x22,\x22\
public\x22,\x22try\x22,\x22s\
witch\x22,\x22continue\
\x22,\x22throws\x22,\x22prot\
ected\x22,\x22public\x22,\
\x22private\x22,\x22modul\
e\x22,\x22requires\x22,\x22e\
xports\x22,\x22do\x22,\x22se\
aled\x22,\x22yield\x22,\x22p\
ermits\x22,\x22goto\x22],\
\x0aliteral:[\x22false\
\x22,\x22true\x22,\x22null\x22]\
,\x0atype:[\x22char\x22,\x22\
boolean\x22,\x22long\x22,\
\x22float\x22,\x22int\x22,\x22b\
yte\x22,\x22short\x22,\x22do\
uble\x22],\x0abuilt_in\
:[\x22super\x22,\x22this\x22\
]},l={className:\
\x22meta\x22,begin:\x22@\x22\
+n,contains:[{\x0ab\
egin:/\x5c(/,end:/\x5c\
)/,contains:[\x22se\
lf\x22]}]},c={class\
Name:\x22params\x22,be\
gin:/\x5c(/,\x0aend:/\x5c\
)/,keywords:r,re\
levance:0,contai\
ns:[e.C_BLOCK_CO\
MMENT_MODE],ends\
Parent:!0}\x0a;retu\
rn{name:\x22Java\x22,a\
liases:[\x22jsp\x22],k\
eywords:r,illega\
l:/<\x5c/|#/,\x0aconta\
ins:[e.COMMENT(\x22\
/\x5c\x5c*\x5c\x5c*\x22,\x22\x5c\x5c*/\x22,\
{relevance:0,con\
tains:[{begin:/\x5c\
w+@/,\x0arelevance:\
0},{className:\x22d\
octag\x22,begin:\x22@[\
A-Za-z]+\x22}]}),{\x0a\
begin:/import ja\
va\x5c.[a-z]+\x5c./,ke\
ywords:\x22import\x22,\
relevance:2\x0a},e.\
C_LINE_COMMENT_M\
ODE,e.C_BLOCK_CO\
MMENT_MODE,{begi\
n:/\x22\x22\x22/,end:/\x22\x22\x22\
/,\x0aclassName:\x22st\
ring\x22,contains:[\
e.BACKSLASH_ESCA\
PE]\x0a},e.APOS_STR\
ING_MODE,e.QUOTE\
_STRING_MODE,{\x0am\
atch:[/\x5cb(?:clas\
s|interface|enum\
|extends|impleme\
nts|new)/,/\x5cs+/,\
n],className:{\x0a1\
:\x22keyword\x22,3:\x22ti\
tle.class\x22}},{ma\
tch:/non-sealed/\
,scope:\x22keyword\x22\
},{\x0abegin:[a.con\
cat(/(?!else)/,n\
),/\x5cs+/,n,/\x5cs+/,\
/=(?!=)/],classN\
ame:{1:\x22type\x22,\x0a3\
:\x22variable\x22,5:\x22o\
perator\x22}},{begi\
n:[/record/,/\x5cs+\
/,n],className:{\
1:\x22keyword\x22,\x0a3:\x22\
title.class\x22},co\
ntains:[c,e.C_LI\
NE_COMMENT_MODE,\
e.C_BLOCK_COMMEN\
T_MODE]},{\x0abegin\
Keywords:\x22new th\
row return else\x22\
,relevance:0},{\x0a\
begin:[\x22(?:\x22+i+\x22\
\x5c\x5cs+)\x22,e.UNDERSC\
ORE_IDENT_RE,/\x5cs\
*(?=\x5c()/],classN\
ame:{\x0a2:\x22title.f\
unction\x22},keywor\
ds:r,contains:[{\
className:\x22param\
s\x22,begin:/\x5c(/,\x0ae\
nd:/\x5c)/,keywords\
:r,relevance:0,\x0a\
contains:[l,e.AP\
OS_STRING_MODE,e\
.QUOTE_STRING_MO\
DE,s,e.C_BLOCK_C\
OMMENT_MODE]\x0a},e\
.C_LINE_COMMENT_\
MODE,e.C_BLOCK_C\
OMMENT_MODE]},s,\
l]}}})()\x0a;hljs.r\
egisterLanguage(\
\x22java\x22,e)})();/*\
! `javascript` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22\x0a;cons\
t e=\x22[A-Za-z$_][\
0-9A-Za-z$_]*\x22,n\
=[\x22as\x22,\x22in\x22,\x22of\x22\
,\x22if\x22,\x22for\x22,\x22whi\
le\x22,\x22finally\x22,\x22v\
ar\x22,\x22new\x22,\x22funct\
ion\x22,\x22do\x22,\x22retur\
n\x22,\x22void\x22,\x22else\x22\
,\x22break\x22,\x22catch\x22\
,\x22instanceof\x22,\x22w\
ith\x22,\x22throw\x22,\x22ca\
se\x22,\x22default\x22,\x22t\
ry\x22,\x22switch\x22,\x22co\
ntinue\x22,\x22typeof\x22\
,\x22delete\x22,\x22let\x22,\
\x22yield\x22,\x22const\x22,\
\x22class\x22,\x22debugge\
r\x22,\x22async\x22,\x22awai\
t\x22,\x22static\x22,\x22imp\
ort\x22,\x22from\x22,\x22exp\
ort\x22,\x22extends\x22],\
a=[\x22true\x22,\x22false\
\x22,\x22null\x22,\x22undefi\
ned\x22,\x22NaN\x22,\x22Infi\
nity\x22],t=[\x22Objec\
t\x22,\x22Function\x22,\x22B\
oolean\x22,\x22Symbol\x22\
,\x22Math\x22,\x22Date\x22,\x22\
Number\x22,\x22BigInt\x22\
,\x22String\x22,\x22RegEx\
p\x22,\x22Array\x22,\x22Floa\
t32Array\x22,\x22Float\
64Array\x22,\x22Int8Ar\
ray\x22,\x22Uint8Array\
\x22,\x22Uint8ClampedA\
rray\x22,\x22Int16Arra\
y\x22,\x22Int32Array\x22,\
\x22Uint16Array\x22,\x22U\
int32Array\x22,\x22Big\
Int64Array\x22,\x22Big\
Uint64Array\x22,\x22Se\
t\x22,\x22Map\x22,\x22WeakSe\
t\x22,\x22WeakMap\x22,\x22Ar\
rayBuffer\x22,\x22Shar\
edArrayBuffer\x22,\x22\
Atomics\x22,\x22DataVi\
ew\x22,\x22JSON\x22,\x22Prom\
ise\x22,\x22Generator\x22\
,\x22GeneratorFunct\
ion\x22,\x22AsyncFunct\
ion\x22,\x22Reflect\x22,\x22\
Proxy\x22,\x22Intl\x22,\x22W\
ebAssembly\x22],s=[\
\x22Error\x22,\x22EvalErr\
or\x22,\x22InternalErr\
or\x22,\x22RangeError\x22\
,\x22ReferenceError\
\x22,\x22SyntaxError\x22,\
\x22TypeError\x22,\x22URI\
Error\x22],r=[\x22setI\
nterval\x22,\x22setTim\
eout\x22,\x22clearInte\
rval\x22,\x22clearTime\
out\x22,\x22require\x22,\x22\
exports\x22,\x22eval\x22,\
\x22isFinite\x22,\x22isNa\
N\x22,\x22parseFloat\x22,\
\x22parseInt\x22,\x22deco\
deURI\x22,\x22decodeUR\
IComponent\x22,\x22enc\
odeURI\x22,\x22encodeU\
RIComponent\x22,\x22es\
cape\x22,\x22unescape\x22\
],c=[\x22arguments\x22\
,\x22this\x22,\x22super\x22,\
\x22console\x22,\x22windo\
w\x22,\x22document\x22,\x22l\
ocalStorage\x22,\x22se\
ssionStorage\x22,\x22m\
odule\x22,\x22global\x22]\
,i=[].concat(r,t\
,s)\x0a;return o=>{\
const l=o.regex,\
b=e,d={begin:/<[\
A-Za-z0-9\x5c\x5c._:-]\
+/,\x0aend:/\x5c/[A-Za\
-z0-9\x5c\x5c._:-]+>|\x5c\
/>/,isTrulyOpeni\
ngTag:(e,n)=>{\x0ac\
onst a=e[0].leng\
th+e.index,t=e.i\
nput[a]\x0a;if(\x22<\x22=\
==t||\x22,\x22===t)ret\
urn void n.ignor\
eMatch();let s\x0a;\
\x22>\x22===t&&(((e,{a\
fter:n})=>{const\
 a=\x22</\x22+e[0].sli\
ce(1)\x0a;return-1!\
==e.input.indexO\
f(a,n)})(e,{afte\
r:a})||n.ignoreM\
atch())\x0a;const r\
=e.input.substri\
ng(a)\x0a;((s=r.mat\
ch(/^\x5cs*=/))||(s\
=r.match(/^\x5cs+ex\
tends\x5cs+/))&&0==\
=s.index)&&n.ign\
oreMatch()\x0a}},g=\
{$pattern:e,keyw\
ord:n,literal:a,\
built_in:i,\x22vari\
able.language\x22:c\
\x0a},u=\x22[0-9](_?[0\
-9])*\x22,m=`\x5c\x5c.(${\
u})`,E=\x220|[1-9](\
_?[0-9])*|0[0-7]\
*[89][0-9]*\x22,A={\
\x0aclassName:\x22numb\
er\x22,variants:[{\x0a\
begin:`(\x5c\x5cb(${E}\
)((${m})|\x5c\x5c.)?|(\
${m}))[eE][+-]?(\
${u})\x5c\x5cb`},{\x0abeg\
in:`\x5c\x5cb(${E})\x5c\x5cb\
((${m})\x5c\x5cb|\x5c\x5c.)?\
|(${m})\x5c\x5cb`},{\x0ab\
egin:\x22\x5c\x5cb(0|[1-9\
](_?[0-9])*)n\x5c\x5cb\
\x22},{\x0abegin:\x22\x5c\x5cb0\
[xX][0-9a-fA-F](\
_?[0-9a-fA-F])*n\
?\x5c\x5cb\x22},{\x0abegin:\x22\
\x5c\x5cb0[bB][0-1](_?\
[0-1])*n?\x5c\x5cb\x22},{\
begin:\x22\x5c\x5cb0[oO][\
0-7](_?[0-7])*n?\
\x5c\x5cb\x22},{\x0abegin:\x22\x5c\
\x5cb0[0-7]+n?\x5c\x5cb\x22}\
],relevance:0},y\
={className:\x22sub\
st\x22,begin:\x22\x5c\x5c$\x5c\x5c\
{\x22,\x0aend:\x22\x5c\x5c}\x22,ke\
ywords:g,contain\
s:[]},h={begin:\x22\
.?html`\x22,end:\x22\x22,\
starts:{end:\x22`\x22,\
\x0areturnEnd:!1,co\
ntains:[o.BACKSL\
ASH_ESCAPE,y],su\
bLanguage:\x22xml\x22}\
},N={\x0abegin:\x22.?c\
ss`\x22,end:\x22\x22,star\
ts:{end:\x22`\x22,retu\
rnEnd:!1,\x0acontai\
ns:[o.BACKSLASH_\
ESCAPE,y],subLan\
guage:\x22css\x22}},_=\
{begin:\x22.?gql`\x22,\
end:\x22\x22,\x0astarts:{\
end:\x22`\x22,returnEn\
d:!1,contains:[o\
.BACKSLASH_ESCAP\
E,y],\x0asubLanguag\
e:\x22graphql\x22}},f=\
{className:\x22stri\
ng\x22,begin:\x22`\x22,en\
d:\x22`\x22,\x0acontains:\
[o.BACKSLASH_ESC\
APE,y]},p={class\
Name:\x22comment\x22,\x0a\
variants:[o.COMM\
ENT(/\x5c/\x5c*\x5c*(?!\x5c/\
)/,\x22\x5c\x5c*/\x22,{relev\
ance:0,contains:\
[{\x0abegin:\x22(?=@[A\
-Za-z]+)\x22,releva\
nce:0,contains:[\
{className:\x22doct\
ag\x22,\x0abegin:\x22@[A-\
Za-z]+\x22},{classN\
ame:\x22type\x22,begin\
:\x22\x5c\x5c{\x22,end:\x22\x5c\x5c}\x22\
,excludeEnd:!0,\x0a\
excludeBegin:!0,\
relevance:0},{cl\
assName:\x22variabl\
e\x22,begin:b+\x22(?=\x5c\
\x5cs*(-)|$)\x22,\x0aends\
Parent:!0,releva\
nce:0},{begin:/(\
?=[^\x5cn])\x5cs/,rele\
vance:0}]}]\x0a}),o\
.C_BLOCK_COMMENT\
_MODE,o.C_LINE_C\
OMMENT_MODE]\x0a},v\
=[o.APOS_STRING_\
MODE,o.QUOTE_STR\
ING_MODE,h,N,_,f\
,{match:/\x5c$\x5cd+/}\
,A]\x0a;y.contains=\
v.concat({begin:\
/\x5c{/,end:/\x5c}/,ke\
ywords:g,contain\
s:[\x22self\x22].conca\
t(v)\x0a});const S=\
[].concat(p,y.co\
ntains),w=S.conc\
at([{begin:/(\x5cs*\
)\x5c(/,end:/\x5c)/,\x0ak\
eywords:g,contai\
ns:[\x22self\x22].conc\
at(S)}]),R={clas\
sName:\x22params\x22,b\
egin:/(\x5cs*)\x5c(/,\x0a\
end:/\x5c)/,exclude\
Begin:!0,exclude\
End:!0,keywords:\
g,contains:w},O=\
{variants:[{\x0amat\
ch:[/class/,/\x5cs+\
/,b,/\x5cs+/,/exten\
ds/,/\x5cs+/,l.conc\
at(b,\x22(\x22,l.conca\
t(/\x5c./,b),\x22)*\x22)]\
,\x0ascope:{1:\x22keyw\
ord\x22,3:\x22title.cl\
ass\x22,5:\x22keyword\x22\
,7:\x22title.class.\
inherited\x22}},{\x0am\
atch:[/class/,/\x5c\
s+/,b],scope:{1:\
\x22keyword\x22,3:\x22tit\
le.class\x22}}]},k=\
{relevance:0,\x0ama\
tch:l.either(/\x5cb\
JSON/,/\x5cb[A-Z][a\
-z]+([A-Z][a-z]*\
|\x5cd)*/,/\x5cb[A-Z]{\
2,}([A-Z][a-z]+|\
\x5cd)+([A-Z][a-z]*\
)*/,/\x5cb[A-Z]{2,}\
[a-z]+([A-Z][a-z\
]+|\x5cd)*([A-Z][a-\
z]*)*/),\x0aclassNa\
me:\x22title.class\x22\
,keywords:{_:[..\
.t,...s]}},I={va\
riants:[{\x0amatch:\
[/function/,/\x5cs+\
/,b,/(?=\x5cs*\x5c()/]\
},{match:[/funct\
ion/,/\x5cs*(?=\x5c()/\
]}],\x0aclassName:{\
1:\x22keyword\x22,3:\x22t\
itle.function\x22},\
label:\x22func.def\x22\
,contains:[R],\x0ai\
llegal:/%/},x={\x0a\
match:l.concat(/\
\x5cb/,(T=[...r,\x22su\
per\x22,\x22import\x22].m\
ap((e=>e+\x22\x5c\x5cs*\x5c\x5c\
(\x22)),\x0al.concat(\x22\
(?!\x22,T.join(\x22|\x22)\
,\x22)\x22)),b,l.looka\
head(/\x5cs*\x5c(/)),\x0a\
className:\x22title\
.function\x22,relev\
ance:0};var T;co\
nst C={\x0abegin:l.\
concat(/\x5c./,l.lo\
okahead(l.concat\
(b,/(?![0-9A-Za-\
z$_(])/))),end:b\
,\x0aexcludeBegin:!\
0,keywords:\x22prot\
otype\x22,className\
:\x22property\x22,rele\
vance:0},M={\x0amat\
ch:[/get|set/,/\x5c\
s+/,b,/(?=\x5c()/],\
className:{1:\x22ke\
yword\x22,3:\x22title.\
function\x22},\x0acont\
ains:[{begin:/\x5c(\
\x5c)/},R]\x0a},B=\x22(\x5c\x5c\
([^()]*(\x5c\x5c([^()]\
*(\x5c\x5c([^()]*\x5c\x5c)[^\
()]*)*\x5c\x5c)[^()]*)\
*\x5c\x5c)|\x22+o.UNDERSC\
ORE_IDENT_RE+\x22)\x5c\
\x5cs*=>\x22,$={\x0amatch\
:[/const|var|let\
/,/\x5cs+/,b,/\x5cs*/,\
/=\x5cs*/,/(async\x5cs\
*)?/,l.lookahead\
(B)],\x0akeywords:\x22\
async\x22,className\
:{1:\x22keyword\x22,3:\
\x22title.function\x22\
},contains:[R]}\x0a\
;return{name:\x22Ja\
vaScript\x22,aliase\
s:[\x22js\x22,\x22jsx\x22,\x22m\
js\x22,\x22cjs\x22],keywo\
rds:g,exports:{\x0a\
PARAMS_CONTAINS:\
w,CLASS_REFERENC\
E:k},illegal:/#(\
?![$_A-z])/,\x0acon\
tains:[o.SHEBANG\
({label:\x22shebang\
\x22,binary:\x22node\x22,\
relevance:5}),{\x0a\
label:\x22use_stric\
t\x22,className:\x22me\
ta\x22,relevance:10\
,\x0abegin:/^\x5cs*['\x22\
]use (strict|asm\
)['\x22]/\x0a},o.APOS_\
STRING_MODE,o.QU\
OTE_STRING_MODE,\
h,N,_,f,p,{match\
:/\x5c$\x5cd+/},A,k,{\x0a\
className:\x22attr\x22\
,begin:b+l.looka\
head(\x22:\x22),releva\
nce:0},$,{\x0abegin\
:\x22(\x22+o.RE_STARTE\
RS_RE+\x22|\x5c\x5cb(case\
|return|throw)\x5c\x5c\
b)\x5c\x5cs*\x22,\x0akeyword\
s:\x22return throw \
case\x22,relevance:\
0,contains:[p,o.\
REGEXP_MODE,{\x0acl\
assName:\x22functio\
n\x22,begin:B,retur\
nBegin:!0,end:\x22\x5c\
\x5cs*=>\x22,contains:\
[{\x0aclassName:\x22pa\
rams\x22,variants:[\
{begin:o.UNDERSC\
ORE_IDENT_RE,rel\
evance:0},{\x0aclas\
sName:null,begin\
:/\x5c(\x5cs*\x5c)/,skip:\
!0},{begin:/(\x5cs*\
)\x5c(/,end:/\x5c)/,\x0ae\
xcludeBegin:!0,e\
xcludeEnd:!0,key\
words:g,contains\
:w}]}]},{begin:/\
,/,relevance:0\x0a}\
,{match:/\x5cs+/,re\
levance:0},{vari\
ants:[{begin:\x22<>\
\x22,end:\x22</>\x22},{\x0am\
atch:/<[A-Za-z0-\
9\x5c\x5c._:-]+\x5cs*\x5c/>/\
},{begin:d.begin\
,\x0a\x22on:begin\x22:d.i\
sTrulyOpeningTag\
,end:d.end}],sub\
Language:\x22xml\x22,c\
ontains:[{\x0abegin\
:d.begin,end:d.e\
nd,skip:!0,conta\
ins:[\x22self\x22]}]}]\
},I,{\x0abeginKeywo\
rds:\x22while if sw\
itch catch for\x22}\
,{\x0abegin:\x22\x5c\x5cb(?!\
function)\x22+o.UND\
ERSCORE_IDENT_RE\
+\x22\x5c\x5c([^()]*(\x5c\x5c([\
^()]*(\x5c\x5c([^()]*\x5c\
\x5c)[^()]*)*\x5c\x5c)[^(\
)]*)*\x5c\x5c)\x5c\x5cs*\x5c\x5c{\x22\
,\x0areturnBegin:!0\
,label:\x22func.def\
\x22,contains:[R,o.\
inherit(o.TITLE_\
MODE,{begin:b,\x0ac\
lassName:\x22title.\
function\x22})]},{m\
atch:/\x5c.\x5c.\x5c./,re\
levance:0},C,{ma\
tch:\x22\x5c\x5c$\x22+b,\x0arel\
evance:0},{match\
:[/\x5cbconstructor\
(?=\x5cs*\x5c()/],clas\
sName:{1:\x22title.\
function\x22},\x0acont\
ains:[R]},x,{rel\
evance:0,match:/\
\x5cb[A-Z][A-Z_0-9]\
+\x5cb/,\x0aclassName:\
\x22variable.consta\
nt\x22},O,M,{match:\
/\x5c$[(.]/}]}}})()\
\x0a;hljs.registerL\
anguage(\x22javascr\
ipt\x22,e)})();/*! \
`jboss-cli` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>({name:\x22JBoss \
CLI\x22,\x0aaliases:[\x22\
wildfly-cli\x22],ke\
ywords:{$pattern\
:\x22[a-z-]+\x22,\x0akeyw\
ord:\x22alias batch\
 cd clear comman\
d connect connec\
tion-factory con\
nection-info dat\
a-source deploy \
deployment-info \
deployment-overl\
ay echo echo-dmr\
 help history if\
 jdbc-driver-inf\
o jms-queue|20 j\
ms-topic|20 ls p\
atch pwd quit re\
ad-attribute rea\
d-operation relo\
ad rollout-plan \
run-batch set sh\
utdown try unali\
as undeploy unse\
t version xa-dat\
a-source\x22,\x0aliter\
al:\x22true false\x22}\
,contains:[e.HAS\
H_COMMENT_MODE,e\
.QUOTE_STRING_MO\
DE,{\x0aclassName:\x22\
params\x22,begin:/-\
-[\x5cw\x5c-=\x5c/]+/},{c\
lassName:\x22functi\
on\x22,\x0abegin:/:[\x5cw\
\x5c-.]+/,relevance\
:0},{className:\x22\
string\x22,begin:/\x5c\
B([\x5c/.])[\x5cw\x5c-.\x5c/\
=]+/\x0a},{classNam\
e:\x22params\x22,begin\
:/\x5c(/,end:/\x5c)/,c\
ontains:[{begin:\
/[\x5cw-]+ *=/,\x0aret\
urnBegin:!0,rele\
vance:0,contains\
:[{className:\x22at\
tr\x22,begin:/[\x5cw-]\
+/}]}],\x0arelevanc\
e:0}]})})();hljs\
.registerLanguag\
e(\x22jboss-cli\x22,e)\
})();/*! `json` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{const a=[\
\x22true\x22,\x22false\x22,\x22\
null\x22],s={\x0ascope\
:\x22literal\x22,begin\
Keywords:a.join(\
\x22 \x22)};return{nam\
e:\x22JSON\x22,aliases\
:[\x22jsonc\x22],\x0akeyw\
ords:{literal:a}\
,contains:[{clas\
sName:\x22attr\x22,\x0abe\
gin:/\x22(\x5c\x5c.|[^\x5c\x5c\x22\
\x5cr\x5cn])*\x22(?=\x5cs*:)\
/,relevance:1.01\
},{match:/[{}[\x5c]\
,:]/,\x0aclassName:\
\x22punctuation\x22,re\
levance:0\x0a},e.QU\
OTE_STRING_MODE,\
s,e.C_NUMBER_MOD\
E,e.C_LINE_COMME\
NT_MODE,e.C_BLOC\
K_COMMENT_MODE],\
\x0aillegal:\x22\x5c\x5cS\x22}}\
})();hljs.regist\
erLanguage(\x22json\
\x22,e)})();/*! `ju\
lia` grammar com\
piled for Highli\
ght.js 11.10.0 *\
/\x0a(()=>{var e=((\
)=>{\x22use strict\x22\
;return e=>{\x0acon\
st r=\x22[A-Za-z_\x5c\x5c\
u00A1-\x5c\x5cuFFFF][A\
-Za-z_0-9\x5c\x5cu00A1\
-\x5c\x5cuFFFF]*\x22,t={$\
pattern:r,\x0akeywo\
rd:[\x22baremodule\x22\
,\x22begin\x22,\x22break\x22\
,\x22catch\x22,\x22ccall\x22\
,\x22const\x22,\x22contin\
ue\x22,\x22do\x22,\x22else\x22,\
\x22elseif\x22,\x22end\x22,\x22\
export\x22,\x22false\x22,\
\x22finally\x22,\x22for\x22,\
\x22function\x22,\x22glob\
al\x22,\x22if\x22,\x22import\
\x22,\x22in\x22,\x22isa\x22,\x22le\
t\x22,\x22local\x22,\x22macr\
o\x22,\x22module\x22,\x22quo\
te\x22,\x22return\x22,\x22tr\
ue\x22,\x22try\x22,\x22using\
\x22,\x22where\x22,\x22while\
\x22],\x0aliteral:[\x22AR\
GS\x22,\x22C_NULL\x22,\x22DE\
POT_PATH\x22,\x22ENDIA\
N_BOM\x22,\x22ENV\x22,\x22In\
f\x22,\x22Inf16\x22,\x22Inf3\
2\x22,\x22Inf64\x22,\x22Inse\
rtionSort\x22,\x22LOAD\
_PATH\x22,\x22MergeSor\
t\x22,\x22NaN\x22,\x22NaN16\x22\
,\x22NaN32\x22,\x22NaN64\x22\
,\x22PROGRAM_FILE\x22,\
\x22QuickSort\x22,\x22Rou\
ndDown\x22,\x22RoundFr\
omZero\x22,\x22RoundNe\
arest\x22,\x22RoundNea\
restTiesAway\x22,\x22R\
oundNearestTiesU\
p\x22,\x22RoundToZero\x22\
,\x22RoundUp\x22,\x22VERS\
ION|0\x22,\x22devnull\x22\
,\x22false\x22,\x22im\x22,\x22m\
issing\x22,\x22nothing\
\x22,\x22pi\x22,\x22stderr\x22,\
\x22stdin\x22,\x22stdout\x22\
,\x22true\x22,\x22undef\x22,\
\x22\x5cu03c0\x22,\x22\x5cu212f\
\x22],\x0abuilt_in:[\x22A\
bstractArray\x22,\x22A\
bstractChannel\x22,\
\x22AbstractChar\x22,\x22\
AbstractDict\x22,\x22A\
bstractDisplay\x22,\
\x22AbstractFloat\x22,\
\x22AbstractIrratio\
nal\x22,\x22AbstractMa\
trix\x22,\x22AbstractR\
ange\x22,\x22AbstractS\
et\x22,\x22AbstractStr\
ing\x22,\x22AbstractUn\
itRange\x22,\x22Abstra\
ctVecOrMat\x22,\x22Abs\
tractVector\x22,\x22An\
y\x22,\x22ArgumentErro\
r\x22,\x22Array\x22,\x22Asse\
rtionError\x22,\x22Big\
Float\x22,\x22BigInt\x22,\
\x22BitArray\x22,\x22BitM\
atrix\x22,\x22BitSet\x22,\
\x22BitVector\x22,\x22Boo\
l\x22,\x22BoundsError\x22\
,\x22CapturedExcept\
ion\x22,\x22CartesianI\
ndex\x22,\x22Cartesian\
Indices\x22,\x22Cchar\x22\
,\x22Cdouble\x22,\x22Cflo\
at\x22,\x22Channel\x22,\x22C\
har\x22,\x22Cint\x22,\x22Cin\
tmax_t\x22,\x22Clong\x22,\
\x22Clonglong\x22,\x22Cmd\
\x22,\x22Colon\x22,\x22Compl\
ex\x22,\x22ComplexF16\x22\
,\x22ComplexF32\x22,\x22C\
omplexF64\x22,\x22Comp\
ositeException\x22,\
\x22Condition\x22,\x22Cpt\
rdiff_t\x22,\x22Cshort\
\x22,\x22Csize_t\x22,\x22Css\
ize_t\x22,\x22Cstring\x22\
,\x22Cuchar\x22,\x22Cuint\
\x22,\x22Cuintmax_t\x22,\x22\
Culong\x22,\x22Culongl\
ong\x22,\x22Cushort\x22,\x22\
Cvoid\x22,\x22Cwchar_t\
\x22,\x22Cwstring\x22,\x22Da\
taType\x22,\x22DenseAr\
ray\x22,\x22DenseMatri\
x\x22,\x22DenseVecOrMa\
t\x22,\x22DenseVector\x22\
,\x22Dict\x22,\x22Dimensi\
onMismatch\x22,\x22Dim\
s\x22,\x22DivideError\x22\
,\x22DomainError\x22,\x22\
EOFError\x22,\x22Enum\x22\
,\x22ErrorException\
\x22,\x22Exception\x22,\x22E\
xponentialBackOf\
f\x22,\x22Expr\x22,\x22Float\
16\x22,\x22Float32\x22,\x22F\
loat64\x22,\x22Functio\
n\x22,\x22GlobalRef\x22,\x22\
HTML\x22,\x22IO\x22,\x22IOBu\
ffer\x22,\x22IOContext\
\x22,\x22IOStream\x22,\x22Id\
Dict\x22,\x22IndexCart\
esian\x22,\x22IndexLin\
ear\x22,\x22IndexStyle\
\x22,\x22InexactError\x22\
,\x22InitError\x22,\x22In\
t\x22,\x22Int128\x22,\x22Int\
16\x22,\x22Int32\x22,\x22Int\
64\x22,\x22Int8\x22,\x22Inte\
ger\x22,\x22InterruptE\
xception\x22,\x22Inval\
idStateException\
\x22,\x22Irrational\x22,\x22\
KeyError\x22,\x22LinRa\
nge\x22,\x22LineNumber\
Node\x22,\x22LinearInd\
ices\x22,\x22LoadError\
\x22,\x22MIME\x22,\x22Matrix\
\x22,\x22Method\x22,\x22Meth\
odError\x22,\x22Missin\
g\x22,\x22MissingExcep\
tion\x22,\x22Module\x22,\x22\
NTuple\x22,\x22NamedTu\
ple\x22,\x22Nothing\x22,\x22\
Number\x22,\x22Ordinal\
Range\x22,\x22OutOfMem\
oryError\x22,\x22Overf\
lowError\x22,\x22Pair\x22\
,\x22PartialQuickSo\
rt\x22,\x22PermutedDim\
sArray\x22,\x22Pipe\x22,\x22\
ProcessFailedExc\
eption\x22,\x22Ptr\x22,\x22Q\
uoteNode\x22,\x22Ratio\
nal\x22,\x22RawFD\x22,\x22Re\
adOnlyMemoryErro\
r\x22,\x22Real\x22,\x22Reent\
rantLock\x22,\x22Ref\x22,\
\x22Regex\x22,\x22RegexMa\
tch\x22,\x22RoundingMo\
de\x22,\x22Segmentatio\
nFault\x22,\x22Set\x22,\x22S\
igned\x22,\x22Some\x22,\x22S\
tackOverflowErro\
r\x22,\x22StepRange\x22,\x22\
StepRangeLen\x22,\x22S\
tridedArray\x22,\x22St\
ridedMatrix\x22,\x22St\
ridedVecOrMat\x22,\x22\
StridedVector\x22,\x22\
String\x22,\x22StringI\
ndexError\x22,\x22SubA\
rray\x22,\x22SubString\
\x22,\x22SubstitutionS\
tring\x22,\x22Symbol\x22,\
\x22SystemError\x22,\x22T\
ask\x22,\x22TaskFailed\
Exception\x22,\x22Text\
\x22,\x22TextDisplay\x22,\
\x22Timer\x22,\x22Tuple\x22,\
\x22Type\x22,\x22TypeErro\
r\x22,\x22TypeVar\x22,\x22UI\
nt\x22,\x22UInt128\x22,\x22U\
Int16\x22,\x22UInt32\x22,\
\x22UInt64\x22,\x22UInt8\x22\
,\x22UndefInitializ\
er\x22,\x22UndefKeywor\
dError\x22,\x22UndefRe\
fError\x22,\x22UndefVa\
rError\x22,\x22Union\x22,\
\x22UnionAll\x22,\x22Unit\
Range\x22,\x22Unsigned\
\x22,\x22Val\x22,\x22Vararg\x22\
,\x22VecElement\x22,\x22V\
ecOrMat\x22,\x22Vector\
\x22,\x22VersionNumber\
\x22,\x22WeakKeyDict\x22,\
\x22WeakRef\x22]\x0a},n={\
keywords:t,illeg\
al:/<\x5c//},a={cla\
ssName:\x22subst\x22,b\
egin:/\x5c$\x5c(/,end:\
/\x5c)/,\x0akeywords:t\
},i={className:\x22\
variable\x22,begin:\
\x22\x5c\x5c$\x22+r},o={clas\
sName:\x22string\x22,\x0a\
contains:[e.BACK\
SLASH_ESCAPE,a,i\
],variants:[{beg\
in:/\x5cw*\x22\x22\x22/,end:\
/\x22\x22\x22\x5cw*/,\x0areleva\
nce:10},{begin:/\
\x5cw*\x22/,end:/\x22\x5cw*/\
}]},s={className\
:\x22string\x22,\x0aconta\
ins:[e.BACKSLASH\
_ESCAPE,a,i],beg\
in:\x22`\x22,end:\x22`\x22},\
l={className:\x22me\
ta\x22,\x0abegin:\x22@\x22+r\
};return n.name=\
\x22Julia\x22,n.contai\
ns=[{className:\x22\
number\x22,\x0abegin:/\
(\x5cb0x[\x5cd_]*(\x5c.[\x5c\
d_]*)?|0x\x5c.\x5cd[\x5cd\
_]*)p[-+]?\x5cd+|\x5cb\
0[box][a-fA-F0-9\
][a-fA-F0-9_]*|(\
\x5cb\x5cd[\x5cd_]*(\x5c.[\x5cd\
_]*)?|\x5c.\x5cd[\x5cd_]*\
)([eEfF][-+]?\x5cd+\
)?/,\x0arelevance:0\
},{className:\x22st\
ring\x22,begin:/'(.\
|\x5c\x5c[xXuU][a-zA-Z\
0-9]+)'/},o,s,l,\
{\x0aclassName:\x22com\
ment\x22,variants:[\
{begin:\x22#=\x22,end:\
\x22=#\x22,relevance:1\
0},{begin:\x22#\x22,\x0ae\
nd:\x22$\x22}]},e.HASH\
_COMMENT_MODE,{c\
lassName:\x22keywor\
d\x22,\x0abegin:\x22\x5c\x5cb((\
(abstract|primit\
ive)\x5c\x5cs+)type|(m\
utable\x5c\x5cs+)?stru\
ct)\x5c\x5cb\x22},{begin:\
/<:/\x0a}],a.contai\
ns=n.contains,n}\
})();hljs.regist\
erLanguage(\x22juli\
a\x22,e)})();/*! `j\
ulia-repl` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r a=(()=>{\x22use s\
trict\x22;return a=\
>({name:\x22Julia R\
EPL\x22,contains:[{\
\x0aclassName:\x22meta\
.prompt\x22,begin:/\
^julia>/,relevan\
ce:10,starts:{en\
d:/^(?![ ]{6})/,\
\x0asubLanguage:\x22ju\
lia\x22}}],aliases:\
[\x22jldoctest\x22]})}\
)()\x0a;hljs.regist\
erLanguage(\x22juli\
a-repl\x22,a)})();/\
*! `kotlin` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22\x0a;var e=\x22\
[0-9](_*[0-9])*\x22\
,n=`\x5c\x5c.(${e})`,a\
=\x22[0-9a-fA-F](_*\
[0-9a-fA-F])*\x22,i\
={\x0aclassName:\x22nu\
mber\x22,variants:[\
{\x0abegin:`(\x5c\x5cb(${\
e})((${n})|\x5c\x5c.)?\
|(${n}))[eE][+-]\
?(${e})[fFdD]?\x5c\x5c\
b`},{\x0abegin:`\x5c\x5cb\
(${e})((${n})[fF\
dD]?\x5c\x5cb|\x5c\x5c.([fFd\
D]\x5c\x5cb)?)`},{begi\
n:`(${n})[fFdD]?\
\x5c\x5cb`\x0a},{begin:`\x5c\
\x5cb(${e})[fFdD]\x5c\x5c\
b`},{\x0abegin:`\x5c\x5cb\
0[xX]((${a})\x5c\x5c.?\
|(${a})?\x5c\x5c.(${a}\
))[pP][+-]?(${e}\
)[fFdD]?\x5c\x5cb`},{\x0a\
begin:\x22\x5c\x5cb(0|[1-\
9](_*[0-9])*)[lL\
]?\x5c\x5cb\x22},{begin:`\
\x5c\x5cb0[xX](${a})[l\
L]?\x5c\x5cb`},{\x0abegin\
:\x22\x5c\x5cb0(_*[0-7])*\
[lL]?\x5c\x5cb\x22},{begi\
n:\x22\x5c\x5cb0[bB][01](\
_*[01])*[lL]?\x5c\x5cb\
\x22}],\x0arelevance:0\
};return e=>{con\
st n={\x0akeyword:\x22\
abstract as val \
var vararg get s\
et class object \
open private pro\
tected public no\
inline crossinli\
ne dynamic final\
 enum if else do\
 while for when \
throw try catch \
finally import p\
ackage is in fun\
 override compan\
ion reified inli\
ne lateinit init\
 interface annot\
ation data seale\
d internal infix\
 operator out by\
 constructor sup\
er tailrec where\
 const inner sus\
pend typealias e\
xternal expect a\
ctual\x22,\x0abuilt_in\
:\x22Byte Short Cha\
r Int Long Boole\
an Float Double \
Void Unit Nothin\
g\x22,\x0aliteral:\x22tru\
e false null\x22},a\
={className:\x22sym\
bol\x22,begin:e.UND\
ERSCORE_IDENT_RE\
+\x22@\x22\x0a},s={classN\
ame:\x22subst\x22,begi\
n:/\x5c$\x5c{/,end:/\x5c}\
/,contains:[e.C_\
NUMBER_MODE]},t=\
{\x0aclassName:\x22var\
iable\x22,begin:\x22\x5c\x5c\
$\x22+e.UNDERSCORE_\
IDENT_RE},r={cla\
ssName:\x22string\x22,\
\x0avariants:[{begi\
n:'\x22\x22\x22',end:'\x22\x22\x22\
(?=[^\x22])',contai\
ns:[t,s]},{begin\
:\x22'\x22,end:\x22'\x22,\x0ail\
legal:/\x5cn/,conta\
ins:[e.BACKSLASH\
_ESCAPE]},{begin\
:'\x22',end:'\x22',ill\
egal:/\x5cn/,\x0aconta\
ins:[e.BACKSLASH\
_ESCAPE,t,s]}]};\
s.contains.push(\
r);const l={\x0acla\
ssName:\x22meta\x22,\x0ab\
egin:\x22@(?:file|p\
roperty|field|ge\
t|set|receiver|p\
aram|setparam|de\
legate)\x5c\x5cs*:(?:\x5c\
\x5cs*\x22+e.UNDERSCOR\
E_IDENT_RE+\x22)?\x22\x0a\
},c={className:\x22\
meta\x22,begin:\x22@\x22+\
e.UNDERSCORE_IDE\
NT_RE,contains:[\
{begin:/\x5c(/,\x0aend\
:/\x5c)/,contains:[\
e.inherit(r,{cla\
ssName:\x22string\x22}\
),\x22self\x22]}]\x0a},o=\
i,b=e.COMMENT(\x22/\
\x5c\x5c*\x22,\x22\x5c\x5c*/\x22,{con\
tains:[e.C_BLOCK\
_COMMENT_MODE]})\
,E={\x0avariants:[{\
className:\x22type\x22\
,begin:e.UNDERSC\
ORE_IDENT_RE},{b\
egin:/\x5c(/,end:/\x5c\
)/,\x0acontains:[]}\
]},d=E;return d.\
variants[1].cont\
ains=[E],E.varia\
nts[1].contains=\
[d],\x0a{name:\x22Kotl\
in\x22,aliases:[\x22kt\
\x22,\x22kts\x22],keyword\
s:n,\x0acontains:[e\
.COMMENT(\x22/\x5c\x5c*\x5c\x5c\
*\x22,\x22\x5c\x5c*/\x22,{relev\
ance:0,contains:\
[{className:\x22doc\
tag\x22,\x0abegin:\x22@[A\
-Za-z]+\x22}]}),e.C\
_LINE_COMMENT_MO\
DE,b,{className:\
\x22keyword\x22,\x0abegin\
:/\x5cb(break|conti\
nue|return|this)\
\x5cb/,starts:{cont\
ains:[{className\
:\x22symbol\x22,\x0abegin\
:/@\x5cw+/}]}},a,l,\
c,{className:\x22fu\
nction\x22,beginKey\
words:\x22fun\x22,end:\
\x22[(]|$\x22,\x0areturnB\
egin:!0,excludeE\
nd:!0,keywords:n\
,relevance:5,con\
tains:[{\x0abegin:e\
.UNDERSCORE_IDEN\
T_RE+\x22\x5c\x5cs*\x5c\x5c(\x22,r\
eturnBegin:!0,re\
levance:0,\x0aconta\
ins:[e.UNDERSCOR\
E_TITLE_MODE]},{\
className:\x22type\x22\
,begin:/</,end:/\
>/,\x0akeywords:\x22re\
ified\x22,relevance\
:0},{className:\x22\
params\x22,begin:/\x5c\
(/,end:/\x5c)/,\x0aend\
sParent:!0,keywo\
rds:n,relevance:\
0,contains:[{beg\
in:/:/,end:/[=,\x5c\
/]/,\x0aendsWithPar\
ent:!0,contains:\
[E,e.C_LINE_COMM\
ENT_MODE,b],rele\
vance:0\x0a},e.C_LI\
NE_COMMENT_MODE,\
b,l,c,r,e.C_NUMB\
ER_MODE]},b]},{\x0a\
begin:[/class|in\
terface|trait/,/\
\x5cs+/,e.UNDERSCOR\
E_IDENT_RE],begi\
nScope:{\x0a3:\x22titl\
e.class\x22},keywor\
ds:\x22class interf\
ace trait\x22,end:/\
[:\x5c{(]|$/,exclud\
eEnd:!0,\x0aillegal\
:\x22extends implem\
ents\x22,contains:[\
{\x0abeginKeywords:\
\x22public protecte\
d internal priva\
te constructor\x22\x0a\
},e.UNDERSCORE_T\
ITLE_MODE,{class\
Name:\x22type\x22,begi\
n:/</,end:/>/,ex\
cludeBegin:!0,\x0ae\
xcludeEnd:!0,rel\
evance:0},{class\
Name:\x22type\x22,begi\
n:/[,:]\x5cs*/,end:\
/[<\x5c(,){\x5cs]|$/,\x0a\
excludeBegin:!0,\
returnEnd:!0},l,\
c]},r,{className\
:\x22meta\x22,begin:\x22^\
#!/usr/bin/env\x22,\
\x0aend:\x22$\x22,illegal\
:\x22\x5cn\x22},o]}}})();\
hljs.registerLan\
guage(\x22kotlin\x22,e\
)})();/*! `lasso\
` grammar compil\
ed for Highlight\
.js 11.10.0 */\x0a(\
()=>{var e=(()=>\
{\x22use strict\x22;re\
turn e=>{\x0aconst \
a=\x22[a-zA-Z_][\x5c\x5cw\
.]*\x22,n=\x22<\x5c\x5c?(las\
so(script)?|=)\x22,\
r=\x22\x5c\x5c]|\x5c\x5c?>\x22,t={\
\x0a$pattern:a+\x22|&[\
lg]t;\x22,\x0aliteral:\
\x22true false none\
 minimal full al\
l void and or no\
t bw nbw ew new \
cn ncn lt lte gt\
 gte eq neq rx n\
rx ft\x22,\x0abuilt_in\
:\x22array date dec\
imal duration in\
teger map pair s\
tring tag xml nu\
ll boolean bytes\
 keyword list lo\
cale queue set s\
tack staticarray\
 local var varia\
ble global data \
self inherited c\
urrentcapture gi\
venblock\x22,\x0akeywo\
rd:\x22cache databa\
se_names databas\
e_schemanames da\
tabase_tablename\
s define_tag def\
ine_type email_b\
atch encode_set \
html_comment han\
dle handle_error\
 header if inlin\
e iterate ljax_t\
arget link link_\
currentaction li\
nk_currentgroup \
link_currentreco\
rd link_detail l\
ink_firstgroup l\
ink_firstrecord \
link_lastgroup l\
ink_lastrecord l\
ink_nextgroup li\
nk_nextrecord li\
nk_prevgroup lin\
k_prevrecord log\
 loop namespace_\
using output_non\
e portal private\
 protect records\
 referer referre\
r repeating resu\
ltset rows searc\
h_args search_ar\
guments select s\
ort_args sort_ar\
guments thread_a\
tomic value_list\
 while abort cas\
e else fail_if f\
ail_ifnot fail i\
f_empty if_false\
 if_null if_true\
 loop_abort loop\
_continue loop_c\
ount params para\
ms_up return ret\
urn_value run_ch\
ildren soap_defi\
netag soap_lastr\
equest soap_last\
response tag_nam\
e ascending aver\
age by define de\
scending do equa\
ls frozen group \
handle_failure i\
mport in into jo\
in let match max\
 min on order pa\
rent protected p\
rovide public re\
quire returnhome\
 skip split_thre\
ad sum take thre\
ad to trait type\
 where with yiel\
d yieldhome\x22\x0a},s\
=e.COMMENT(\x22\x5cx3c\
!--\x22,\x22--\x5cx3e\x22,{r\
elevance:0}),i={\
className:\x22meta\x22\
,\x0abegin:\x22\x5c\x5c[nopr\
ocess\x5c\x5c]\x22,starts\
:{end:\x22\x5c\x5c[/nopro\
cess\x5c\x5c]\x22,returnE\
nd:!0,contains:[\
s]\x0a}},l={classNa\
me:\x22meta\x22,begin:\
\x22\x5c\x5c[/noprocess|\x22\
+n},o={className\
:\x22symbol\x22,\x0abegin\
:\x22'\x22+a+\x22'\x22\x0a},c=[\
e.C_LINE_COMMENT\
_MODE,e.C_BLOCK_\
COMMENT_MODE,e.i\
nherit(e.C_NUMBE\
R_MODE,{\x0abegin:e\
.C_NUMBER_RE+\x22|(\
-?infinity|NaN)\x5c\
\x5cb\x22}),e.inherit(\
e.APOS_STRING_MO\
DE,{\x0aillegal:nul\
l}),e.inherit(e.\
QUOTE_STRING_MOD\
E,{illegal:null}\
),{\x0aclassName:\x22s\
tring\x22,begin:\x22`\x22\
,end:\x22`\x22},{varia\
nts:[{begin:\x22[#$\
]\x22+a},{begin:\x22#\x22\
,\x0aend:\x22\x5c\x5cd+\x22,ill\
egal:\x22\x5c\x5cW\x22}]},{c\
lassName:\x22type\x22,\
begin:\x22::\x5c\x5cs*\x22,e\
nd:a,illegal:\x22\x5c\x5c\
W\x22\x0a},{className:\
\x22params\x22,variant\
s:[{begin:\x22-(?!i\
nfinity)\x22+a,rele\
vance:0},{\x0abegin\
:\x22(\x5c\x5c.\x5c\x5c.\x5c\x5c.)\x22}]\
},{begin:/(->|\x5c.\
)\x5cs*/,relevance:\
0,contains:[o]},\
{\x0aclassName:\x22cla\
ss\x22,beginKeyword\
s:\x22define\x22,retur\
nEnd:!0,end:\x22\x5c\x5c(\
|=>\x22,\x0acontains:[\
e.inherit(e.TITL\
E_MODE,{begin:a+\
\x22(=(?!>))?|[-+*/\
%](?!>)\x22})]}];re\
turn{\x0aname:\x22Lass\
o\x22,aliases:[\x22ls\x22\
,\x22lassoscript\x22],\
case_insensitive\
:!0,keywords:t,\x0a\
contains:[{class\
Name:\x22meta\x22,begi\
n:r,relevance:0,\
starts:{end:\x22\x5c\x5c[\
|\x22+n,\x0areturnEnd:\
!0,relevance:0,c\
ontains:[s]}},i,\
l,{className:\x22me\
ta\x22,\x0abegin:\x22\x5c\x5c[n\
o_square_bracket\
s\x22,starts:{end:\x22\
\x5c\x5c[/no_square_br\
ackets\x5c\x5c]\x22,\x0akeyw\
ords:t,contains:\
[{className:\x22met\
a\x22,begin:r,relev\
ance:0,starts:{\x0a\
end:\x22\x5c\x5c[noproces\
s\x5c\x5c]|\x22+n,returnE\
nd:!0,contains:[\
s]}},i,l].concat\
(c)}},{\x0aclassNam\
e:\x22meta\x22,begin:\x22\
\x5c\x5c[\x22,relevance:0\
},{className:\x22me\
ta\x22,begin:\x22^#!\x22,\
\x0aend:\x22lasso9$\x22,r\
elevance:10}].co\
ncat(c)}}})();hl\
js.registerLangu\
age(\x22lasso\x22,e)\x0a}\
)();/*! `latex` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{const n=[\
{begin:/\x5c^{6}[0-\
9a-f]{6}/},{\x0abeg\
in:/\x5c^{5}[0-9a-f\
]{5}/},{begin:/\x5c\
^{4}[0-9a-f]{4}/\
},{begin:/\x5c^{3}[\
0-9a-f]{3}/\x0a},{b\
egin:/\x5c^{2}[0-9a\
-f]{2}/},{begin:\
/\x5c^{2}[\x5cu0000-\x5cu\
007f]/}],a=[{\x0acl\
assName:\x22keyword\
\x22,begin:/\x5c\x5c/,rel\
evance:0,contain\
s:[{endsParent:!\
0,\x0abegin:e.regex\
.either(...[\x22(?:\
NeedsTeXFormat|R\
equirePackage|Ge\
tIdInfo)\x22,\x22Provi\
des(?:Expl)?(?:P\
ackage|Class|Fil\
e)\x22,\x22(?:DeclareO\
ption|ProcessOpt\
ions)\x22,\x22(?:docum\
entclass|usepack\
age|input|includ\
e)\x22,\x22makeat(?:le\
tter|other)\x22,\x22Ex\
plSyntax(?:On|Of\
f)\x22,\x22(?:new|rene\
w|provide)?comma\
nd\x22,\x22(?:re)newen\
vironment\x22,\x22(?:N\
ew|Renew|Provide\
|Declare)(?:Expa\
ndable)?Document\
Command\x22,\x22(?:New\
|Renew|Provide|D\
eclare)DocumentE\
nvironment\x22,\x22(?:\
(?:e|g|x)?def|le\
t)\x22,\x22(?:begin|en\
d)\x22,\x22(?:part|cha\
pter|(?:sub){0,2\
}section|(?:sub)\
?paragraph)\x22,\x22ca\
ption\x22,\x22(?:label\
|(?:eq|page|name\
)?ref|(?:paren|f\
oot|super)?cite)\
\x22,\x22(?:alpha|beta\
|[Gg]amma|[Dd]el\
ta|(?:var)?epsil\
on|zeta|eta|[Tt]\
heta|vartheta)\x22,\
\x22(?:iota|(?:var)\
?kappa|[Ll]ambda\
|mu|nu|[Xx]i|[Pp\
]i|varpi|(?:var)\
rho)\x22,\x22(?:[Ss]ig\
ma|varsigma|tau|\
[Uu]psilon|[Pp]h\
i|varphi|chi|[Pp\
]si|[Oo]mega)\x22,\x22\
(?:frac|sum|prod\
|lim|infty|times\
|sqrt|leq|geq|le\
ft|right|middle|\
[bB]igg?)\x22,\x22(?:[\
lr]angle|q?quad|\
[lcvdi]?dots|d?d\
ot|hat|tilde|bar\
)\x22].map((e=>e+\x22(\
?![a-zA-Z@:_])\x22)\
))\x0a},{endsParent\
:!0,\x0abegin:RegEx\
p([\x22(?:__)?[a-zA\
-Z]{2,}_[a-zA-Z]\
(?:_?[a-zA-Z])+:\
[a-zA-Z]*\x22,\x22[lgc\
]__?[a-zA-Z](?:_\
?[a-zA-Z])*_[a-z\
A-Z]{2,}\x22,\x22[qs]_\
_?[a-zA-Z](?:_?[\
a-zA-Z])+\x22,\x22use(\
?:_i)?:[a-zA-Z]*\
\x22,\x22(?:else|fi|or\
):\x22,\x22(?:if|cs|ex\
p):w\x22,\x22(?:hbox|v\
box):n\x22,\x22::[a-zA\
-Z]_unbraced\x22,\x22:\
:[a-zA-Z:]\x22].map\
((e=>e+\x22(?![a-zA\
-Z:_])\x22)).join(\x22\
|\x22))\x0a},{endsPare\
nt:!0,variants:n\
},{endsParent:!0\
,relevance:0,var\
iants:[{\x0abegin:/\
[a-zA-Z@]+/},{be\
gin:/[^a-zA-Z@]?\
/}]}]},{classNam\
e:\x22params\x22,relev\
ance:0,\x0abegin:/#\
+\x5cd?/},{variants\
:n},{className:\x22\
built_in\x22,releva\
nce:0,begin:/[$&\
^_]/},{\x0aclassNam\
e:\x22meta\x22,begin:/\
% ?!(T[eE]X|tex|\
BIB|bib)/,end:\x22$\
\x22,relevance:10\x0a}\
,e.COMMENT(\x22%\x22,\x22\
$\x22,{relevance:0}\
)],i={begin:/\x5c{/\
,end:/\x5c}/,releva\
nce:0,\x0acontains:\
[\x22self\x22,...a]},t\
=e.inherit(i,{re\
levance:0,endsPa\
rent:!0,\x0acontain\
s:[i,...a]}),r={\
begin:/\x5c[/,end:/\
\x5c]/,endsParent:!\
0,relevance:0,\x0ac\
ontains:[i,...a]\
},s={begin:/\x5cs+/\
,relevance:0},c=\
[t],l=[r],o=(e,n\
)=>({\x0acontains:[\
s],starts:{relev\
ance:0,contains:\
e,starts:n}}),d=\
(e,n)=>({\x0abegin:\
\x22\x5c\x5c\x5c\x5c\x22+e+\x22(?![a-\
zA-Z@:_])\x22,keywo\
rds:{$pattern:/\x5c\
\x5c[a-zA-Z]+/,keyw\
ord:\x22\x5c\x5c\x22+e\x0a},rel\
evance:0,contain\
s:[s],starts:n})\
,g=(n,a)=>e.inhe\
rit({\x0abegin:\x22\x5c\x5c\x5c\
\x5cbegin(?=[ \x5ct]*(\
\x5c\x5cr?\x5c\x5cn[ \x5ct]*)?\x5c\
\x5c{\x22+n+\x22\x5c\x5c})\x22,key\
words:{\x0a$pattern\
:/\x5c\x5c[a-zA-Z]+/,k\
eyword:\x22\x5c\x5cbegin\x22\
},relevance:0\x0a},\
o(c,a)),m=(n=\x22st\
ring\x22)=>e.END_SA\
ME_AS_BEGIN({cla\
ssName:n,begin:/\
(.|\x5cr?\x5cn)/,\x0aend:\
/(.|\x5cr?\x5cn)/,excl\
udeBegin:!0,excl\
udeEnd:!0,endsPa\
rent:!0}),b=e=>(\
{\x0aclassName:\x22str\
ing\x22,end:\x22(?=\x5c\x5c\x5c\
\x5cend\x5c\x5c{\x22+e+\x22\x5c\x5c})\
\x22}),p=(e=\x22string\
\x22)=>({relevance:\
0,\x0abegin:/\x5c{/,st\
arts:{endsParent\
:!0,contains:[{c\
lassName:e,end:/\
(?=\x5c})/,\x0aendsPar\
ent:!0,contains:\
[{begin:/\x5c{/,end\
:/\x5c}/,relevance:\
0,contains:[\x22sel\
f\x22]}]}]}\x0a});retu\
rn{name:\x22LaTeX\x22,\
aliases:[\x22tex\x22],\
\x0acontains:[...[\x22\
verb\x22,\x22lstinline\
\x22].map((e=>d(e,{\
contains:[m()]})\
)),d(\x22mint\x22,o(c,\
{\x0acontains:[m()]\
})),d(\x22mintinlin\
e\x22,o(c,{contains\
:[p(),m()]})),d(\
\x22url\x22,{\x0acontains\
:[p(\x22link\x22),p(\x22l\
ink\x22)]}),d(\x22hype\
rref\x22,{contains:\
[p(\x22link\x22)]\x0a}),d\
(\x22href\x22,o(l,{con\
tains:[p(\x22link\x22)\
]\x0a})),...[].conc\
at(...[\x22\x22,\x22\x5c\x5c*\x22]\
.map((e=>[g(\x22ver\
batim\x22+e,b(\x22verb\
atim\x22+e)),g(\x22fil\
econtents\x22+e,o(c\
,b(\x22filecontents\
\x22+e))),...[\x22\x22,\x22B\
\x22,\x22L\x22].map((n=>g\
(n+\x22Verbatim\x22+e,\
o(l,b(n+\x22Verbati\
m\x22+e)))))]))),g(\
\x22minted\x22,o(l,o(c\
,b(\x22minted\x22)))),\
...a]\x0a}}})();hlj\
s.registerLangua\
ge(\x22latex\x22,e)})(\
);/*! `ldif` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var a=(()=>{\x22use\
 strict\x22;return \
a=>({name:\x22LDIF\x22\
,contains:[{\x0acla\
ssName:\x22attribut\
e\x22,match:\x22^dn(?=\
:)\x22,relevance:10\
},{className:\x22at\
tribute\x22,\x0amatch:\
\x22^\x5c\x5cw+(?=:)\x22},{c\
lassName:\x22litera\
l\x22,match:\x22^-\x22},a\
.HASH_COMMENT_MO\
DE]})})()\x0a;hljs.\
registerLanguage\
(\x22ldif\x22,a)})();/\
*! `leaf` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 a=(()=>{\x22use st\
rict\x22;return a=>\
{\x0aconst e=/([A-Z\
a-z_][A-Za-z_0-9\
]*)?/,n={scope:\x22\
params\x22,begin:/\x5c\
(/,\x0aend:/\x5c)(?=\x5c:\
?)/,endsParent:!\
0,relevance:7,co\
ntains:[{scope:\x22\
string\x22,begin:'\x22\
',\x0aend:'\x22'},{sco\
pe:\x22keyword\x22,mat\
ch:\x22true|false|i\
n\x22},{scope:\x22vari\
able\x22,\x0amatch:/[A\
-Za-z_][A-Za-z_0\
-9]*/},{scope:\x22o\
perator\x22,\x0amatch:\
/\x5c+|\x5c-|\x5c*|\x5c/|\x5c%|\
\x5c=\x5c=|\x5c=|\x5c!|\x5c>|\x5c<\
|\x5c&\x5c&|\x5c|\x5c|/}]},t\
={match:[e,/(?=\x5c\
()/],\x0ascope:{1:\x22\
keyword\x22},contai\
ns:[n]};return n\
.contains.unshif\
t(t),{name:\x22Leaf\
\x22,\x0acontains:[{ma\
tch:[/#+/,e,/(?=\
\x5c()/],scope:{1:\x22\
punctuation\x22,2:\x22\
keyword\x22},starts\
:{\x0acontains:[{ma\
tch:/\x5c:/,scope:\x22\
punctuation\x22}]},\
contains:[n]},{m\
atch:[/#+/,e,/:?\
/],\x0ascope:{1:\x22pu\
nctuation\x22,2:\x22ke\
yword\x22,3:\x22punctu\
ation\x22}}]}}})()\x0a\
;hljs.registerLa\
nguage(\x22leaf\x22,a)\
})();/*! `less` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22\x0a;con\
st e=[\x22a\x22,\x22abbr\x22\
,\x22address\x22,\x22arti\
cle\x22,\x22aside\x22,\x22au\
dio\x22,\x22b\x22,\x22blockq\
uote\x22,\x22body\x22,\x22bu\
tton\x22,\x22canvas\x22,\x22\
caption\x22,\x22cite\x22,\
\x22code\x22,\x22dd\x22,\x22del\
\x22,\x22details\x22,\x22dfn\
\x22,\x22div\x22,\x22dl\x22,\x22dt\
\x22,\x22em\x22,\x22fieldset\
\x22,\x22figcaption\x22,\x22\
figure\x22,\x22footer\x22\
,\x22form\x22,\x22h1\x22,\x22h2\
\x22,\x22h3\x22,\x22h4\x22,\x22h5\x22\
,\x22h6\x22,\x22header\x22,\x22\
hgroup\x22,\x22html\x22,\x22\
i\x22,\x22iframe\x22,\x22img\
\x22,\x22input\x22,\x22ins\x22,\
\x22kbd\x22,\x22label\x22,\x22l\
egend\x22,\x22li\x22,\x22mai\
n\x22,\x22mark\x22,\x22menu\x22\
,\x22nav\x22,\x22object\x22,\
\x22ol\x22,\x22optgroup\x22,\
\x22option\x22,\x22p\x22,\x22pi\
cture\x22,\x22q\x22,\x22quot\
e\x22,\x22samp\x22,\x22secti\
on\x22,\x22select\x22,\x22so\
urce\x22,\x22span\x22,\x22st\
rong\x22,\x22summary\x22,\
\x22sup\x22,\x22table\x22,\x22t\
body\x22,\x22td\x22,\x22text\
area\x22,\x22tfoot\x22,\x22t\
h\x22,\x22thead\x22,\x22time\
\x22,\x22tr\x22,\x22ul\x22,\x22var\
\x22,\x22video\x22,\x22defs\x22\
,\x22g\x22,\x22marker\x22,\x22m\
ask\x22,\x22pattern\x22,\x22\
svg\x22,\x22switch\x22,\x22s\
ymbol\x22,\x22feBlend\x22\
,\x22feColorMatrix\x22\
,\x22feComponentTra\
nsfer\x22,\x22feCompos\
ite\x22,\x22feConvolve\
Matrix\x22,\x22feDiffu\
seLighting\x22,\x22feD\
isplacementMap\x22,\
\x22feFlood\x22,\x22feGau\
ssianBlur\x22,\x22feIm\
age\x22,\x22feMerge\x22,\x22\
feMorphology\x22,\x22f\
eOffset\x22,\x22feSpec\
ularLighting\x22,\x22f\
eTile\x22,\x22feTurbul\
ence\x22,\x22linearGra\
dient\x22,\x22radialGr\
adient\x22,\x22stop\x22,\x22\
circle\x22,\x22ellipse\
\x22,\x22image\x22,\x22line\x22\
,\x22path\x22,\x22polygon\
\x22,\x22polyline\x22,\x22re\
ct\x22,\x22text\x22,\x22use\x22\
,\x22textPath\x22,\x22tsp\
an\x22,\x22foreignObje\
ct\x22,\x22clipPath\x22],\
t=[\x22any-hover\x22,\x22\
any-pointer\x22,\x22as\
pect-ratio\x22,\x22col\
or\x22,\x22color-gamut\
\x22,\x22color-index\x22,\
\x22device-aspect-r\
atio\x22,\x22device-he\
ight\x22,\x22device-wi\
dth\x22,\x22display-mo\
de\x22,\x22forced-colo\
rs\x22,\x22grid\x22,\x22heig\
ht\x22,\x22hover\x22,\x22inv\
erted-colors\x22,\x22m\
onochrome\x22,\x22orie\
ntation\x22,\x22overfl\
ow-block\x22,\x22overf\
low-inline\x22,\x22poi\
nter\x22,\x22prefers-c\
olor-scheme\x22,\x22pr\
efers-contrast\x22,\
\x22prefers-reduced\
-motion\x22,\x22prefer\
s-reduced-transp\
arency\x22,\x22resolut\
ion\x22,\x22scan\x22,\x22scr\
ipting\x22,\x22update\x22\
,\x22width\x22,\x22min-wi\
dth\x22,\x22max-width\x22\
,\x22min-height\x22,\x22m\
ax-height\x22].sort\
().reverse(),r=[\
\x22active\x22,\x22any-li\
nk\x22,\x22blank\x22,\x22che\
cked\x22,\x22current\x22,\
\x22default\x22,\x22defin\
ed\x22,\x22dir\x22,\x22disab\
led\x22,\x22drop\x22,\x22emp\
ty\x22,\x22enabled\x22,\x22f\
irst\x22,\x22first-chi\
ld\x22,\x22first-of-ty\
pe\x22,\x22fullscreen\x22\
,\x22future\x22,\x22focus\
\x22,\x22focus-visible\
\x22,\x22focus-within\x22\
,\x22has\x22,\x22host\x22,\x22h\
ost-context\x22,\x22ho\
ver\x22,\x22indetermin\
ate\x22,\x22in-range\x22,\
\x22invalid\x22,\x22is\x22,\x22\
lang\x22,\x22last-chil\
d\x22,\x22last-of-type\
\x22,\x22left\x22,\x22link\x22,\
\x22local-link\x22,\x22no\
t\x22,\x22nth-child\x22,\x22\
nth-col\x22,\x22nth-la\
st-child\x22,\x22nth-l\
ast-col\x22,\x22nth-la\
st-of-type\x22,\x22nth\
-of-type\x22,\x22only-\
child\x22,\x22only-of-\
type\x22,\x22optional\x22\
,\x22out-of-range\x22,\
\x22past\x22,\x22placehol\
der-shown\x22,\x22read\
-only\x22,\x22read-wri\
te\x22,\x22required\x22,\x22\
right\x22,\x22root\x22,\x22s\
cope\x22,\x22target\x22,\x22\
target-within\x22,\x22\
user-invalid\x22,\x22v\
alid\x22,\x22visited\x22,\
\x22where\x22].sort().\
reverse(),i=[\x22af\
ter\x22,\x22backdrop\x22,\
\x22before\x22,\x22cue\x22,\x22\
cue-region\x22,\x22fir\
st-letter\x22,\x22firs\
t-line\x22,\x22grammar\
-error\x22,\x22marker\x22\
,\x22part\x22,\x22placeho\
lder\x22,\x22selection\
\x22,\x22slotted\x22,\x22spe\
lling-error\x22].so\
rt().reverse(),o\
=[\x22accent-color\x22\
,\x22align-content\x22\
,\x22align-items\x22,\x22\
align-self\x22,\x22ali\
gnment-baseline\x22\
,\x22all\x22,\x22animatio\
n\x22,\x22animation-de\
lay\x22,\x22animation-\
direction\x22,\x22anim\
ation-duration\x22,\
\x22animation-fill-\
mode\x22,\x22animation\
-iteration-count\
\x22,\x22animation-nam\
e\x22,\x22animation-pl\
ay-state\x22,\x22anima\
tion-timing-func\
tion\x22,\x22appearanc\
e\x22,\x22backface-vis\
ibility\x22,\x22backgr\
ound\x22,\x22backgroun\
d-attachment\x22,\x22b\
ackground-blend-\
mode\x22,\x22backgroun\
d-clip\x22,\x22backgro\
und-color\x22,\x22back\
ground-image\x22,\x22b\
ackground-origin\
\x22,\x22background-po\
sition\x22,\x22backgro\
und-repeat\x22,\x22bac\
kground-size\x22,\x22b\
aseline-shift\x22,\x22\
block-size\x22,\x22bor\
der\x22,\x22border-blo\
ck\x22,\x22border-bloc\
k-color\x22,\x22border\
-block-end\x22,\x22bor\
der-block-end-co\
lor\x22,\x22border-blo\
ck-end-style\x22,\x22b\
order-block-end-\
width\x22,\x22border-b\
lock-start\x22,\x22bor\
der-block-start-\
color\x22,\x22border-b\
lock-start-style\
\x22,\x22border-block-\
start-width\x22,\x22bo\
rder-block-style\
\x22,\x22border-block-\
width\x22,\x22border-b\
ottom\x22,\x22border-b\
ottom-color\x22,\x22bo\
rder-bottom-left\
-radius\x22,\x22border\
-bottom-right-ra\
dius\x22,\x22border-bo\
ttom-style\x22,\x22bor\
der-bottom-width\
\x22,\x22border-collap\
se\x22,\x22border-colo\
r\x22,\x22border-image\
\x22,\x22border-image-\
outset\x22,\x22border-\
image-repeat\x22,\x22b\
order-image-slic\
e\x22,\x22border-image\
-source\x22,\x22border\
-image-width\x22,\x22b\
order-inline\x22,\x22b\
order-inline-col\
or\x22,\x22border-inli\
ne-end\x22,\x22border-\
inline-end-color\
\x22,\x22border-inline\
-end-style\x22,\x22bor\
der-inline-end-w\
idth\x22,\x22border-in\
line-start\x22,\x22bor\
der-inline-start\
-color\x22,\x22border-\
inline-start-sty\
le\x22,\x22border-inli\
ne-start-width\x22,\
\x22border-inline-s\
tyle\x22,\x22border-in\
line-width\x22,\x22bor\
der-left\x22,\x22borde\
r-left-color\x22,\x22b\
order-left-style\
\x22,\x22border-left-w\
idth\x22,\x22border-ra\
dius\x22,\x22border-ri\
ght\x22,\x22border-end\
-end-radius\x22,\x22bo\
rder-end-start-r\
adius\x22,\x22border-r\
ight-color\x22,\x22bor\
der-right-style\x22\
,\x22border-right-w\
idth\x22,\x22border-sp\
acing\x22,\x22border-s\
tart-end-radius\x22\
,\x22border-start-s\
tart-radius\x22,\x22bo\
rder-style\x22,\x22bor\
der-top\x22,\x22border\
-top-color\x22,\x22bor\
der-top-left-rad\
ius\x22,\x22border-top\
-right-radius\x22,\x22\
border-top-style\
\x22,\x22border-top-wi\
dth\x22,\x22border-wid\
th\x22,\x22bottom\x22,\x22bo\
x-decoration-bre\
ak\x22,\x22box-shadow\x22\
,\x22box-sizing\x22,\x22b\
reak-after\x22,\x22bre\
ak-before\x22,\x22brea\
k-inside\x22,\x22cx\x22,\x22\
cy\x22,\x22caption-sid\
e\x22,\x22caret-color\x22\
,\x22clear\x22,\x22clip\x22,\
\x22clip-path\x22,\x22cli\
p-rule\x22,\x22color\x22,\
\x22color-interpola\
tion\x22,\x22color-int\
erpolation-filte\
rs\x22,\x22color-profi\
le\x22,\x22color-rende\
ring\x22,\x22color-sch\
eme\x22,\x22column-cou\
nt\x22,\x22column-fill\
\x22,\x22column-gap\x22,\x22\
column-rule\x22,\x22co\
lumn-rule-color\x22\
,\x22column-rule-st\
yle\x22,\x22column-rul\
e-width\x22,\x22column\
-span\x22,\x22column-w\
idth\x22,\x22columns\x22,\
\x22contain\x22,\x22conte\
nt\x22,\x22content-vis\
ibility\x22,\x22counte\
r-increment\x22,\x22co\
unter-reset\x22,\x22cu\
e\x22,\x22cue-after\x22,\x22\
cue-before\x22,\x22cur\
sor\x22,\x22direction\x22\
,\x22display\x22,\x22domi\
nant-baseline\x22,\x22\
empty-cells\x22,\x22en\
able-background\x22\
,\x22fill\x22,\x22fill-op\
acity\x22,\x22fill-rul\
e\x22,\x22filter\x22,\x22fle\
x\x22,\x22flex-basis\x22,\
\x22flex-direction\x22\
,\x22flex-flow\x22,\x22fl\
ex-grow\x22,\x22flex-s\
hrink\x22,\x22flex-wra\
p\x22,\x22float\x22,\x22flow\
\x22,\x22flood-color\x22,\
\x22flood-opacity\x22,\
\x22font\x22,\x22font-dis\
play\x22,\x22font-fami\
ly\x22,\x22font-featur\
e-settings\x22,\x22fon\
t-kerning\x22,\x22font\
-language-overri\
de\x22,\x22font-size\x22,\
\x22font-size-adjus\
t\x22,\x22font-smoothi\
ng\x22,\x22font-stretc\
h\x22,\x22font-style\x22,\
\x22font-synthesis\x22\
,\x22font-variant\x22,\
\x22font-variant-ca\
ps\x22,\x22font-varian\
t-east-asian\x22,\x22f\
ont-variant-liga\
tures\x22,\x22font-var\
iant-numeric\x22,\x22f\
ont-variant-posi\
tion\x22,\x22font-vari\
ation-settings\x22,\
\x22font-weight\x22,\x22g\
ap\x22,\x22glyph-orien\
tation-horizonta\
l\x22,\x22glyph-orient\
ation-vertical\x22,\
\x22grid\x22,\x22grid-are\
a\x22,\x22grid-auto-co\
lumns\x22,\x22grid-aut\
o-flow\x22,\x22grid-au\
to-rows\x22,\x22grid-c\
olumn\x22,\x22grid-col\
umn-end\x22,\x22grid-c\
olumn-start\x22,\x22gr\
id-gap\x22,\x22grid-ro\
w\x22,\x22grid-row-end\
\x22,\x22grid-row-star\
t\x22,\x22grid-templat\
e\x22,\x22grid-templat\
e-areas\x22,\x22grid-t\
emplate-columns\x22\
,\x22grid-template-\
rows\x22,\x22hanging-p\
unctuation\x22,\x22hei\
ght\x22,\x22hyphens\x22,\x22\
icon\x22,\x22image-ori\
entation\x22,\x22image\
-rendering\x22,\x22ima\
ge-resolution\x22,\x22\
ime-mode\x22,\x22inlin\
e-size\x22,\x22inset\x22,\
\x22inset-block\x22,\x22i\
nset-block-end\x22,\
\x22inset-block-sta\
rt\x22,\x22inset-inlin\
e\x22,\x22inset-inline\
-end\x22,\x22inset-inl\
ine-start\x22,\x22isol\
ation\x22,\x22kerning\x22\
,\x22justify-conten\
t\x22,\x22justify-item\
s\x22,\x22justify-self\
\x22,\x22left\x22,\x22letter\
-spacing\x22,\x22light\
ing-color\x22,\x22line\
-break\x22,\x22line-he\
ight\x22,\x22list-styl\
e\x22,\x22list-style-i\
mage\x22,\x22list-styl\
e-position\x22,\x22lis\
t-style-type\x22,\x22m\
arker\x22,\x22marker-e\
nd\x22,\x22marker-mid\x22\
,\x22marker-start\x22,\
\x22mask\x22,\x22margin\x22,\
\x22margin-block\x22,\x22\
margin-block-end\
\x22,\x22margin-block-\
start\x22,\x22margin-b\
ottom\x22,\x22margin-i\
nline\x22,\x22margin-i\
nline-end\x22,\x22marg\
in-inline-start\x22\
,\x22margin-left\x22,\x22\
margin-right\x22,\x22m\
argin-top\x22,\x22mark\
s\x22,\x22mask\x22,\x22mask-\
border\x22,\x22mask-bo\
rder-mode\x22,\x22mask\
-border-outset\x22,\
\x22mask-border-rep\
eat\x22,\x22mask-borde\
r-slice\x22,\x22mask-b\
order-source\x22,\x22m\
ask-border-width\
\x22,\x22mask-clip\x22,\x22m\
ask-composite\x22,\x22\
mask-image\x22,\x22mas\
k-mode\x22,\x22mask-or\
igin\x22,\x22mask-posi\
tion\x22,\x22mask-repe\
at\x22,\x22mask-size\x22,\
\x22mask-type\x22,\x22max\
-block-size\x22,\x22ma\
x-height\x22,\x22max-i\
nline-size\x22,\x22max\
-width\x22,\x22min-blo\
ck-size\x22,\x22min-he\
ight\x22,\x22min-inlin\
e-size\x22,\x22min-wid\
th\x22,\x22mix-blend-m\
ode\x22,\x22nav-down\x22,\
\x22nav-index\x22,\x22nav\
-left\x22,\x22nav-righ\
t\x22,\x22nav-up\x22,\x22non\
e\x22,\x22normal\x22,\x22obj\
ect-fit\x22,\x22object\
-position\x22,\x22opac\
ity\x22,\x22order\x22,\x22or\
phans\x22,\x22outline\x22\
,\x22outline-color\x22\
,\x22outline-offset\
\x22,\x22outline-style\
\x22,\x22outline-width\
\x22,\x22overflow\x22,\x22ov\
erflow-wrap\x22,\x22ov\
erflow-x\x22,\x22overf\
low-y\x22,\x22padding\x22\
,\x22padding-block\x22\
,\x22padding-block-\
end\x22,\x22padding-bl\
ock-start\x22,\x22padd\
ing-bottom\x22,\x22pad\
ding-inline\x22,\x22pa\
dding-inline-end\
\x22,\x22padding-inlin\
e-start\x22,\x22paddin\
g-left\x22,\x22padding\
-right\x22,\x22padding\
-top\x22,\x22page-brea\
k-after\x22,\x22page-b\
reak-before\x22,\x22pa\
ge-break-inside\x22\
,\x22pause\x22,\x22pause-\
after\x22,\x22pause-be\
fore\x22,\x22perspecti\
ve\x22,\x22perspective\
-origin\x22,\x22pointe\
r-events\x22,\x22posit\
ion\x22,\x22quotes\x22,\x22r\
\x22,\x22resize\x22,\x22rest\
\x22,\x22rest-after\x22,\x22\
rest-before\x22,\x22ri\
ght\x22,\x22rotate\x22,\x22r\
ow-gap\x22,\x22scale\x22,\
\x22scroll-margin\x22,\
\x22scroll-margin-b\
lock\x22,\x22scroll-ma\
rgin-block-end\x22,\
\x22scroll-margin-b\
lock-start\x22,\x22scr\
oll-margin-botto\
m\x22,\x22scroll-margi\
n-inline\x22,\x22scrol\
l-margin-inline-\
end\x22,\x22scroll-mar\
gin-inline-start\
\x22,\x22scroll-margin\
-left\x22,\x22scroll-m\
argin-right\x22,\x22sc\
roll-margin-top\x22\
,\x22scroll-padding\
\x22,\x22scroll-paddin\
g-block\x22,\x22scroll\
-padding-block-e\
nd\x22,\x22scroll-padd\
ing-block-start\x22\
,\x22scroll-padding\
-bottom\x22,\x22scroll\
-padding-inline\x22\
,\x22scroll-padding\
-inline-end\x22,\x22sc\
roll-padding-inl\
ine-start\x22,\x22scro\
ll-padding-left\x22\
,\x22scroll-padding\
-right\x22,\x22scroll-\
padding-top\x22,\x22sc\
roll-snap-align\x22\
,\x22scroll-snap-st\
op\x22,\x22scroll-snap\
-type\x22,\x22scrollba\
r-color\x22,\x22scroll\
bar-gutter\x22,\x22scr\
ollbar-width\x22,\x22s\
hape-image-thres\
hold\x22,\x22shape-mar\
gin\x22,\x22shape-outs\
ide\x22,\x22shape-rend\
ering\x22,\x22stop-col\
or\x22,\x22stop-opacit\
y\x22,\x22stroke\x22,\x22str\
oke-dasharray\x22,\x22\
stroke-dashoffse\
t\x22,\x22stroke-linec\
ap\x22,\x22stroke-line\
join\x22,\x22stroke-mi\
terlimit\x22,\x22strok\
e-opacity\x22,\x22stro\
ke-width\x22,\x22speak\
\x22,\x22speak-as\x22,\x22sr\
c\x22,\x22tab-size\x22,\x22t\
able-layout\x22,\x22te\
xt-anchor\x22,\x22text\
-align\x22,\x22text-al\
ign-all\x22,\x22text-a\
lign-last\x22,\x22text\
-combine-upright\
\x22,\x22text-decorati\
on\x22,\x22text-decora\
tion-color\x22,\x22tex\
t-decoration-lin\
e\x22,\x22text-decorat\
ion-skip-ink\x22,\x22t\
ext-decoration-s\
tyle\x22,\x22text-deco\
ration-thickness\
\x22,\x22text-emphasis\
\x22,\x22text-emphasis\
-color\x22,\x22text-em\
phasis-position\x22\
,\x22text-emphasis-\
style\x22,\x22text-ind\
ent\x22,\x22text-justi\
fy\x22,\x22text-orient\
ation\x22,\x22text-ove\
rflow\x22,\x22text-ren\
dering\x22,\x22text-sh\
adow\x22,\x22text-tran\
sform\x22,\x22text-und\
erline-offset\x22,\x22\
text-underline-p\
osition\x22,\x22top\x22,\x22\
transform\x22,\x22tran\
sform-box\x22,\x22tran\
sform-origin\x22,\x22t\
ransform-style\x22,\
\x22transition\x22,\x22tr\
ansition-delay\x22,\
\x22transition-dura\
tion\x22,\x22transitio\
n-property\x22,\x22tra\
nsition-timing-f\
unction\x22,\x22transl\
ate\x22,\x22unicode-bi\
di\x22,\x22vector-effe\
ct\x22,\x22vertical-al\
ign\x22,\x22visibility\
\x22,\x22voice-balance\
\x22,\x22voice-duratio\
n\x22,\x22voice-family\
\x22,\x22voice-pitch\x22,\
\x22voice-range\x22,\x22v\
oice-rate\x22,\x22voic\
e-stress\x22,\x22voice\
-volume\x22,\x22white-\
space\x22,\x22widows\x22,\
\x22width\x22,\x22will-ch\
ange\x22,\x22word-brea\
k\x22,\x22word-spacing\
\x22,\x22word-wrap\x22,\x22w\
riting-mode\x22,\x22x\x22\
,\x22y\x22,\x22z-index\x22].\
sort().reverse()\
,n=r.concat(i).s\
ort().reverse()\x0a\
;return a=>{cons\
t l=(e=>({IMPORT\
ANT:{scope:\x22meta\
\x22,begin:\x22!import\
ant\x22},\x0aBLOCK_COM\
MENT:e.C_BLOCK_C\
OMMENT_MODE,HEXC\
OLOR:{scope:\x22num\
ber\x22,\x0abegin:/#((\
[0-9a-fA-F]{3,4}\
)|(([0-9a-fA-F]{\
2}){3,4}))\x5cb/},F\
UNCTION_DISPATCH\
:{\x0aclassName:\x22bu\
ilt_in\x22,begin:/[\
\x5cw-]+(?=\x5c()/},AT\
TRIBUTE_SELECTOR\
_MODE:{\x0ascope:\x22s\
elector-attr\x22,be\
gin:/\x5c[/,end:/\x5c]\
/,illegal:\x22$\x22,\x0ac\
ontains:[e.APOS_\
STRING_MODE,e.QU\
OTE_STRING_MODE]\
},CSS_NUMBER_MOD\
E:{\x0ascope:\x22numbe\
r\x22,\x0abegin:e.NUMB\
ER_RE+\x22(%|em|ex|\
ch|rem|vw|vh|vmi\
n|vmax|cm|mm|in|\
pt|pc|px|deg|gra\
d|rad|turn|s|ms|\
Hz|kHz|dpi|dpcm|\
dppx)?\x22,\x0arelevan\
ce:0},CSS_VARIAB\
LE:{className:\x22a\
ttr\x22,begin:/--[A\
-Za-z_][A-Za-z0-\
9_-]*/}\x0a}))(a),s\
=n,d=\x22[\x5c\x5cw-]+\x22,c\
=\x22(\x22+d+\x22|@\x5c\x5c{\x22+d\
+\x22\x5c\x5c})\x22,g=[],b=[\
],m=e=>({\x0aclassN\
ame:\x22string\x22,beg\
in:\x22~?\x22+e+\x22.*?\x22+\
e}),p=(e,t,r)=>(\
{className:e,beg\
in:t,\x0arelevance:\
r}),f={$pattern:\
/[a-z-]+/,keywor\
d:\x22and or not on\
ly\x22,\x0aattribute:t\
.join(\x22 \x22)},u={b\
egin:\x22\x5c\x5c(\x22,end:\x22\
\x5c\x5c)\x22,contains:b,\
keywords:f,\x0arele\
vance:0}\x0a;b.push\
(a.C_LINE_COMMEN\
T_MODE,a.C_BLOCK\
_COMMENT_MODE,m(\
\x22'\x22),m('\x22'),l.CS\
S_NUMBER_MODE,{\x0a\
begin:\x22(url|data\
-uri)\x5c\x5c(\x22,starts\
:{className:\x22str\
ing\x22,end:\x22[\x5c\x5c)\x5c\x5c\
n]\x22,\x0aexcludeEnd:\
!0}\x0a},l.HEXCOLOR\
,u,p(\x22variable\x22,\
\x22@@?\x22+d,10),p(\x22v\
ariable\x22,\x22@\x5c\x5c{\x22+\
d+\x22\x5c\x5c}\x22),p(\x22buil\
t_in\x22,\x22~?`[^`]*?\
`\x22),{\x0aclassName:\
\x22attribute\x22,begi\
n:d+\x22\x5c\x5cs*:\x22,end:\
\x22:\x22,returnBegin:\
!0,excludeEnd:!0\
\x0a},l.IMPORTANT,{\
beginKeywords:\x22a\
nd not\x22},l.FUNCT\
ION_DISPATCH);co\
nst h=b.concat({\
\x0abegin:/\x5c{/,end:\
/\x5c}/,contains:g}\
),k={beginKeywor\
ds:\x22when\x22,endsWi\
thParent:!0,\x0acon\
tains:[{beginKey\
words:\x22and not\x22}\
].concat(b)},v={\
begin:c+\x22\x5c\x5cs*:\x22,\
\x0areturnBegin:!0,\
end:/[;}]/,relev\
ance:0,contains:\
[{begin:/-(webki\
t|moz|ms|o)-/\x0a},\
l.CSS_VARIABLE,{\
className:\x22attri\
bute\x22,begin:\x22\x5c\x5cb\
(\x22+o.join(\x22|\x22)+\x22\
)\x5c\x5cb\x22,\x0aend:/(?=:\
)/,starts:{endsW\
ithParent:!0,ill\
egal:\x22[<=$]\x22,rel\
evance:0,contain\
s:b}}]\x0a},y={clas\
sName:\x22keyword\x22,\
\x0abegin:\x22@(import\
|media|charset|f\
ont-face|(-[a-z]\
+-)?keyframes|su\
pports|document|\
namespace|page|v\
iewport|host)\x5c\x5cb\
\x22,\x0astarts:{end:\x22\
[;{}]\x22,keywords:\
f,returnEnd:!0,c\
ontains:b,releva\
nce:0}},w={\x0aclas\
sName:\x22variable\x22\
,variants:[{begi\
n:\x22@\x22+d+\x22\x5c\x5cs*:\x22,\
relevance:15},{b\
egin:\x22@\x22+d\x0a}],st\
arts:{end:\x22[;}]\x22\
,returnEnd:!0,co\
ntains:h}},x={va\
riants:[{\x0abegin:\
\x22[\x5c\x5c.#:&\x5c\x5c[>]\x22,e\
nd:\x22[;{}]\x22},{beg\
in:c,end:/\x5c{/}],\
returnBegin:!0,\x0a\
returnEnd:!0,ill\
egal:\x22[<='$\x5c\x22]\x22,\
relevance:0,\x0acon\
tains:[a.C_LINE_\
COMMENT_MODE,a.C\
_BLOCK_COMMENT_M\
ODE,k,p(\x22keyword\
\x22,\x22all\x5c\x5cb\x22),p(\x22v\
ariable\x22,\x22@\x5c\x5c{\x22+\
d+\x22\x5c\x5c}\x22),{\x0abegin\
:\x22\x5c\x5cb(\x22+e.join(\x22\
|\x22)+\x22)\x5c\x5cb\x22,class\
Name:\x22selector-t\
ag\x22\x0a},l.CSS_NUMB\
ER_MODE,p(\x22selec\
tor-tag\x22,c,0),p(\
\x22selector-id\x22,\x22#\
\x22+c),p(\x22selector\
-class\x22,\x22\x5c\x5c.\x22+c,\
0),p(\x22selector-t\
ag\x22,\x22&\x22,0),l.ATT\
RIBUTE_SELECTOR_\
MODE,{\x0aclassName\
:\x22selector-pseud\
o\x22,begin:\x22:(\x22+r.\
join(\x22|\x22)+\x22)\x22},{\
\x0aclassName:\x22sele\
ctor-pseudo\x22,beg\
in:\x22:(:)?(\x22+i.jo\
in(\x22|\x22)+\x22)\x22},{be\
gin:/\x5c(/,\x0aend:/\x5c\
)/,relevance:0,c\
ontains:h},{begi\
n:\x22!important\x22},\
l.FUNCTION_DISPA\
TCH]},_={\x0abegin:\
d+\x22:(:)?\x22+`(${s.\
join(\x22|\x22)})`,ret\
urnBegin:!0,cont\
ains:[x]}\x0a;retur\
n g.push(a.C_LIN\
E_COMMENT_MODE,a\
.C_BLOCK_COMMENT\
_MODE,y,w,_,v,x,\
k,l.FUNCTION_DIS\
PATCH),\x0a{name:\x22L\
ess\x22,case_insens\
itive:!0,illegal\
:\x22[=>'/<($\x5c\x22]\x22,c\
ontains:g}}})()\x0a\
;hljs.registerLa\
nguage(\x22less\x22,e)\
})();/*! `lisp` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{\x0aconst n=\
\x22[a-zA-Z_\x5c\x5c-+\x5c\x5c*\
\x5c\x5c/<=>&#][a-zA-Z\
0-9_\x5c\x5c-+*\x5c\x5c/<=>&\
#!]*\x22,a=\x22\x5c\x5c|[^]*\
?\x5c\x5c|\x22,i=\x22(-|\x5c\x5c+)\
?\x5c\x5cd+(\x5c\x5c.\x5c\x5cd+|\x5c\x5c\
/\x5c\x5cd+)?((d|e|f|l\
|s|D|E|F|L|S)(\x5c\x5c\
+|-)?\x5c\x5cd+)?\x22,s={\
\x0aclassName:\x22lite\
ral\x22,begin:\x22\x5c\x5cb(\
t{1}|nil)\x5c\x5cb\x22},l\
={className:\x22num\
ber\x22,variants:[{\
\x0abegin:i,relevan\
ce:0},{begin:\x22#(\
b|B)[0-1]+(/[0-1\
]+)?\x22},{\x0abegin:\x22\
#(o|O)[0-7]+(/[0\
-7]+)?\x22},{begin:\
\x22#(x|X)[0-9a-fA-\
F]+(/[0-9a-fA-F]\
+)?\x22},{\x0abegin:\x22#\
(c|C)\x5c\x5c(\x22+i+\x22 +\x22\
+i,end:\x22\x5c\x5c)\x22}]},\
b=e.inherit(e.QU\
OTE_STRING_MODE,\
{\x0aillegal:null})\
,g=e.COMMENT(\x22;\x22\
,\x22$\x22,{relevance:\
0}),r={begin:\x22\x5c\x5c\
*\x22,end:\x22\x5c\x5c*\x22},t=\
{\x0aclassName:\x22sym\
bol\x22,begin:\x22[:&]\
\x22+n},c={begin:n,\
relevance:0},d={\
begin:a},o={\x0acon\
tains:[l,b,r,t,{\
begin:\x22\x5c\x5c(\x22,end:\
\x22\x5c\x5c)\x22,contains:[\
\x22self\x22,s,b,l,c]}\
,c],\x0avariants:[{\
begin:\x22['`]\x5c\x5c(\x22,\
end:\x22\x5c\x5c)\x22},{begi\
n:\x22\x5c\x5c(quote \x22,en\
d:\x22\x5c\x5c)\x22,keywords\
:{\x0aname:\x22quote\x22}\
},{begin:\x22'\x22+a}]\
},v={variants:[{\
begin:\x22'\x22+n},{\x0ab\
egin:\x22#'\x22+n+\x22(::\
\x22+n+\x22)*\x22}]},m={b\
egin:\x22\x5c\x5c(\x5c\x5cs*\x22,e\
nd:\x22\x5c\x5c)\x22},u={end\
sWithParent:!0,\x0a\
relevance:0};ret\
urn m.contains=[\
{className:\x22name\
\x22,variants:[{beg\
in:n,relevance:0\
\x0a},{begin:a}]},u\
],u.contains=[o,\
v,m,s,l,b,g,r,t,\
d,c],{name:\x22Lisp\
\x22,illegal:/\x5cS/,\x0a\
contains:[l,e.SH\
EBANG(),s,b,g,o,\
v,m,c]}}})();hlj\
s.registerLangua\
ge(\x22lisp\x22,e)\x0a})(\
);/*! `livecodes\
erver` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;return e=>{co\
nst r={className\
:\x22variable\x22,\x0avar\
iants:[{begin:\x22\x5c\
\x5cb([gtps][A-Z]{1\
}[a-zA-Z0-9]*)(\x5c\
\x5c[.+\x5c\x5c])?(?:\x5c\x5cs*\
?)\x22},{\x0abegin:\x22\x5c\x5c\
$_[A-Z]+\x22}],rele\
vance:0\x0a},t=[e.C\
_BLOCK_COMMENT_M\
ODE,e.HASH_COMME\
NT_MODE,e.COMMEN\
T(\x22--\x22,\x22$\x22),e.CO\
MMENT(\x22[^:]//\x22,\x22\
$\x22)],a=e.inherit\
(e.TITLE_MODE,{\x0a\
variants:[{begin\
:\x22\x5c\x5cb_*rig[A-Z][\
A-Za-z0-9_\x5c\x5c-]*\x22\
},{begin:\x22\x5c\x5cb_[a\
-z0-9\x5c\x5c-]+\x22}]\x0a})\
,o=e.inherit(e.T\
ITLE_MODE,{begin\
:\x22\x5c\x5cb([A-Za-z0-9\
_\x5c\x5c-]+)\x5c\x5cb\x22});re\
turn{\x0aname:\x22Live\
Code\x22,case_insen\
sitive:!1,keywor\
ds:{\x0akeyword:\x22$_\
COOKIE $_FILES $\
_GET $_GET_BINAR\
Y $_GET_RAW $_PO\
ST $_POST_BINARY\
 $_POST_RAW $_SE\
SSION $_SERVER c\
odepoint codepoi\
nts segment segm\
ents codeunit co\
deunits sentence\
 sentences trueW\
ord trueWords pa\
ragraph after by\
te bytes english\
 the until http \
forever descendi\
ng using line re\
al8 with seventh\
 for stdout fina\
lly element word\
 words fourth be\
fore black ninth\
 sixth character\
s chars stderr u\
Int1 uInt1s uInt\
2 uInt2s stdin s\
tring lines rela\
tive rel any fif\
th items from mi\
ddle mid at else\
 of catch then t\
hird it file mil\
liseconds second\
s second secs se\
c int1 int1s int\
4 int4s internet\
 int2 int2s norm\
al text item las\
t long detailed \
effective uInt4 \
uInt4s repeat en\
d repeat URL in \
try into switch \
to words https t\
oken binfile eac\
h tenth as ticks\
 tick system rea\
l4 by dateItems \
without char cha\
racter ascending\
 eighth whole da\
teTime numeric s\
hort first ftp i\
nteger abbreviat\
ed abbr abbrev p\
rivate case whil\
e if div mod wra\
p and or bitAnd \
bitNot bitOr bit\
Xor among not in\
 a an within con\
tains ends with \
begins the keys \
of keys\x22,\x0alitera\
l:\x22SIX TEN FORMF\
EED NINE ZERO NO\
NE SPACE FOUR FA\
LSE COLON CRLF P\
I COMMA ENDOFFIL\
E EOF EIGHT FIVE\
 QUOTE EMPTY ONE\
 TRUE RETURN CR \
LINEFEED RIGHT B\
ACKSLASH NULL SE\
VEN TAB THREE TW\
O six ten formfe\
ed nine zero non\
e space four fal\
se colon crlf pi\
 comma endoffile\
 eof eight five \
quote empty one \
true return cr l\
inefeed right ba\
ckslash null sev\
en tab three two\
 RIVERSION RISTA\
TE FILE_READ_MOD\
E FILE_WRITE_MOD\
E FILE_WRITE_MOD\
E DIR_WRITE_MODE\
 FILE_READ_UMASK\
 FILE_WRITE_UMAS\
K DIR_READ_UMASK\
 DIR_WRITE_UMASK\
\x22,\x0abuilt_in:\x22put\
 abs acos aliasR\
eference annuity\
 arrayDecode arr\
ayEncode asin at\
an atan2 average\
 avg avgDev base\
64Decode base64E\
ncode baseConver\
t binaryDecode b\
inaryEncode byte\
Offset byteToNum\
 cachedURL cache\
dURLs charToNum \
cipherNames code\
pointOffset code\
pointProperty co\
depointToNum cod\
eunitOffset comm\
andNames compoun\
d compress const\
antNames cos dat\
e dateFormat dec\
ompress differen\
ce directories d\
iskSpace DNSServ\
ers exp exp1 exp\
2 exp10 extents \
files flushEvent\
s folders format\
 functionNames g\
eometricMean glo\
bal globals hasM\
emory harmonicMe\
an hostAddress h\
ostAddressToName\
 hostName hostNa\
meToAddress isNu\
mber ISOToMac it\
emOffset keys le\
n length libURLE\
rrorData libUrlF\
ormData libURLft\
pCommand libURLL\
astHTTPHeaders l\
ibURLLastRHHeade\
rs libUrlMultipa\
rtFormAddPart li\
bUrlMultipartFor\
mData libURLVers\
ion lineOffset l\
n ln1 localNames\
 log log2 log10 \
longFilePath low\
er macToISO matc\
hChunk matchText\
 matrixMultiply \
max md5Digest me\
dian merge messa\
geAuthentication\
Code messageDige\
st millisec mill\
isecs millisecon\
d milliseconds m\
in monthNames na\
tiveCharToNum no\
rmalizeText num \
number numToByte\
 numToChar numTo\
Codepoint numToN\
ativeChar offset\
 open openfiles \
openProcesses op\
enProcessIDs ope\
nSockets paragra\
phOffset paramCo\
unt param params\
 peerAddress pen\
dingMessages pla\
tform popStdDev \
populationStanda\
rdDeviation popu\
lationVariance p\
opVariance proce\
ssID random rand\
omBytes replaceT\
ext result revCr\
eateXMLTree revC\
reateXMLTreeFrom\
File revCurrentR\
ecord revCurrent\
RecordIsFirst re\
vCurrentRecordIs\
Last revDatabase\
ColumnCount revD\
atabaseColumnIsN\
ull revDatabaseC\
olumnLengths rev\
DatabaseColumnNa\
mes revDatabaseC\
olumnNamed revDa\
tabaseColumnNumb\
ered revDatabase\
ColumnTypes revD\
atabaseConnectRe\
sult revDatabase\
Cursors revDatab\
aseID revDatabas\
eTableNames revD\
atabaseType revD\
ataFromQuery rev\
db_closeCursor r\
evdb_columnbynum\
ber revdb_column\
count revdb_colu\
mnisnull revdb_c\
olumnlengths rev\
db_columnnames r\
evdb_columntypes\
 revdb_commit re\
vdb_connect revd\
b_connections re\
vdb_connectioner\
r revdb_currentr\
ecord revdb_curs\
orconnection rev\
db_cursorerr rev\
db_cursors revdb\
_dbtype revdb_di\
sconnect revdb_e\
xecute revdb_ise\
of revdb_isbof r\
evdb_movefirst r\
evdb_movelast re\
vdb_movenext rev\
db_moveprev revd\
b_query revdb_qu\
erylist revdb_re\
cordcount revdb_\
rollback revdb_t\
ablenames revGet\
DatabaseDriverPa\
th revNumberOfRe\
cords revOpenDat\
abase revOpenDat\
abases revQueryD\
atabase revQuery\
DatabaseBlob rev\
QueryResult revQ\
ueryIsAtStart re\
vQueryIsAtEnd re\
vUnixFromMacPath\
 revXMLAttribute\
 revXMLAttribute\
s revXMLAttribut\
eValues revXMLCh\
ildContents revX\
MLChildNames rev\
XMLCreateTreeFro\
mFileWithNamespa\
ces revXMLCreate\
TreeWithNamespac\
es revXMLDataFro\
mXPathQuery revX\
MLEvaluateXPath \
revXMLFirstChild\
 revXMLMatchingN\
ode revXMLNextSi\
bling revXMLNode\
Contents revXMLN\
umberOfChildren \
revXMLParent rev\
XMLPreviousSibli\
ng revXMLRootNod\
e revXMLRPC_Crea\
teRequest revXML\
RPC_Documents re\
vXMLRPC_Error re\
vXMLRPC_GetHost \
revXMLRPC_GetMet\
hod revXMLRPC_Ge\
tParam revXMLTex\
t revXMLRPC_Exec\
ute revXMLRPC_Ge\
tParamCount revX\
MLRPC_GetParamNo\
de revXMLRPC_Get\
ParamType revXML\
RPC_GetPath revX\
MLRPC_GetPort re\
vXMLRPC_GetProto\
col revXMLRPC_Ge\
tRequest revXMLR\
PC_GetResponse r\
evXMLRPC_GetSock\
et revXMLTree re\
vXMLTrees revXML\
ValidateDTD revZ\
ipDescribeItem r\
evZipEnumerateIt\
ems revZipOpenAr\
chives round sam\
pVariance sec se\
cs seconds sente\
nceOffset sha1Di\
gest shell short\
FilePath sin spe\
cialFolderPath s\
qrt standardDevi\
ation statRound \
stdDev sum sysEr\
ror systemVersio\
n tan tempName t\
extDecode textEn\
code tick ticks \
time to tokenOff\
set toLower toUp\
per transpose tr\
uewordOffset tru\
nc uniDecode uni\
Encode upper URL\
Decode URLEncode\
 URLStatus uuid \
value variableNa\
mes variance ver\
sion waitDepth w\
eekdayNames word\
Offset xsltApply\
Stylesheet xsltA\
pplyStylesheetFr\
omFile xsltLoadS\
tylesheet xsltLo\
adStylesheetFrom\
File add breakpo\
int cancel clear\
 local variable \
file word line f\
older directory \
URL close socket\
 process combine\
 constant conver\
t create new ali\
as folder direct\
ory decrypt dele\
te variable word\
 line folder dir\
ectory URL dispa\
tch divide do en\
crypt filter get\
 include interse\
ct kill libURLDo\
wnloadToFile lib\
URLFollowHttpRed\
irects libURLftp\
Upload libURLftp\
UploadFile libUR\
LresetAll libUrl\
SetAuthCallback \
libURLSetDriver \
libURLSetCustomH\
TTPHeaders libUr\
lSetExpect100 li\
bURLSetFTPListCo\
mmand libURLSetF\
TPMode libURLSet\
FTPStopTime libU\
RLSetStatusCallb\
ack load extensi\
on loadedExtensi\
ons multiply soc\
ket prepare proc\
ess post seek re\
l relative read \
from process ren\
ame replace requ\
ire resetAll res\
olve revAddXMLNo\
de revAppendXML \
revCloseCursor r\
evCloseDatabase \
revCommitDatabas\
e revCopyFile re\
vCopyFolder revC\
opyXMLNode revDe\
leteFolder revDe\
leteXMLNode revD\
eleteAllXMLTrees\
 revDeleteXMLTre\
e revExecuteSQL \
revGoURL revInse\
rtXMLNode revMov\
eFolder revMoveT\
oFirstRecord rev\
MoveToLastRecord\
 revMoveToNextRe\
cord revMoveToPr\
eviousRecord rev\
MoveToRecord rev\
MoveXMLNode revP\
utIntoXMLNode re\
vRollBackDatabas\
e revSetDatabase\
DriverPath revSe\
tXMLAttribute re\
vXMLRPC_AddParam\
 revXMLRPC_Delet\
eAllDocuments re\
vXMLAddDTD revXM\
LRPC_Free revXML\
RPC_FreeAll revX\
MLRPC_DeleteDocu\
ment revXMLRPC_D\
eleteParam revXM\
LRPC_SetHost rev\
XMLRPC_SetMethod\
 revXMLRPC_SetPo\
rt revXMLRPC_Set\
Protocol revXMLR\
PC_SetSocket rev\
ZipAddItemWithDa\
ta revZipAddItem\
WithFile revZipA\
ddUncompressedIt\
emWithData revZi\
pAddUncompressed\
ItemWithFile rev\
ZipCancel revZip\
CloseArchive rev\
ZipDeleteItem re\
vZipExtractItemT\
oFile revZipExtr\
actItemToVariabl\
e revZipSetProgr\
essCallback revZ\
ipRenameItem rev\
ZipReplaceItemWi\
thData revZipRep\
laceItemWithFile\
 revZipOpenArchi\
ve send set sort\
 split start sto\
p subtract symme\
tric union unloa\
d vectorDotProdu\
ct wait write\x22\x0a}\
,contains:[r,{cl\
assName:\x22keyword\
\x22,begin:\x22\x5c\x5cbend\x5c\
\x5csif\x5c\x5cb\x22},{\x0aclas\
sName:\x22function\x22\
,beginKeywords:\x22\
function\x22,end:\x22$\
\x22,\x0acontains:[r,o\
,e.APOS_STRING_M\
ODE,e.QUOTE_STRI\
NG_MODE,e.BINARY\
_NUMBER_MODE,e.C\
_NUMBER_MODE,a]\x0a\
},{className:\x22fu\
nction\x22,begin:\x22\x5c\
\x5cbend\x5c\x5cs+\x22,end:\x22\
$\x22,keywords:\x22end\
\x22,\x0acontains:[o,a\
],relevance:0},{\
beginKeywords:\x22c\
ommand on\x22,end:\x22\
$\x22,\x0acontains:[r,\
o,e.APOS_STRING_\
MODE,e.QUOTE_STR\
ING_MODE,e.BINAR\
Y_NUMBER_MODE,e.\
C_NUMBER_MODE,a]\
\x0a},{className:\x22m\
eta\x22,variants:[{\
begin:\x22<\x5c\x5c?(rev|\
lc|livecode)\x22,re\
levance:10},{\x0abe\
gin:\x22<\x5c\x5c?\x22},{beg\
in:\x22\x5c\x5c?>\x22}]\x0a},e.\
APOS_STRING_MODE\
,e.QUOTE_STRING_\
MODE,e.BINARY_NU\
MBER_MODE,e.C_NU\
MBER_MODE,a].con\
cat(t),\x0aillegal:\
\x22;$|^\x5c\x5c[|^=|&|\x5c\x5c\
{\x22}}})();hljs.re\
gisterLanguage(\x22\
livecodeserver\x22,\
e)})();/*! `live\
script` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22\x0a;const e=[\x22a\
s\x22,\x22in\x22,\x22of\x22,\x22if\
\x22,\x22for\x22,\x22while\x22,\
\x22finally\x22,\x22var\x22,\
\x22new\x22,\x22function\x22\
,\x22do\x22,\x22return\x22,\x22\
void\x22,\x22else\x22,\x22br\
eak\x22,\x22catch\x22,\x22in\
stanceof\x22,\x22with\x22\
,\x22throw\x22,\x22case\x22,\
\x22default\x22,\x22try\x22,\
\x22switch\x22,\x22contin\
ue\x22,\x22typeof\x22,\x22de\
lete\x22,\x22let\x22,\x22yie\
ld\x22,\x22const\x22,\x22cla\
ss\x22,\x22debugger\x22,\x22\
async\x22,\x22await\x22,\x22\
static\x22,\x22import\x22\
,\x22from\x22,\x22export\x22\
,\x22extends\x22],n=[\x22\
true\x22,\x22false\x22,\x22n\
ull\x22,\x22undefined\x22\
,\x22NaN\x22,\x22Infinity\
\x22],a=[].concat([\
\x22setInterval\x22,\x22s\
etTimeout\x22,\x22clea\
rInterval\x22,\x22clea\
rTimeout\x22,\x22requi\
re\x22,\x22exports\x22,\x22e\
val\x22,\x22isFinite\x22,\
\x22isNaN\x22,\x22parseFl\
oat\x22,\x22parseInt\x22,\
\x22decodeURI\x22,\x22dec\
odeURIComponent\x22\
,\x22encodeURI\x22,\x22en\
codeURIComponent\
\x22,\x22escape\x22,\x22unes\
cape\x22],[\x22Object\x22\
,\x22Function\x22,\x22Boo\
lean\x22,\x22Symbol\x22,\x22\
Math\x22,\x22Date\x22,\x22Nu\
mber\x22,\x22BigInt\x22,\x22\
String\x22,\x22RegExp\x22\
,\x22Array\x22,\x22Float3\
2Array\x22,\x22Float64\
Array\x22,\x22Int8Arra\
y\x22,\x22Uint8Array\x22,\
\x22Uint8ClampedArr\
ay\x22,\x22Int16Array\x22\
,\x22Int32Array\x22,\x22U\
int16Array\x22,\x22Uin\
t32Array\x22,\x22BigIn\
t64Array\x22,\x22BigUi\
nt64Array\x22,\x22Set\x22\
,\x22Map\x22,\x22WeakSet\x22\
,\x22WeakMap\x22,\x22Arra\
yBuffer\x22,\x22Shared\
ArrayBuffer\x22,\x22At\
omics\x22,\x22DataView\
\x22,\x22JSON\x22,\x22Promis\
e\x22,\x22Generator\x22,\x22\
GeneratorFunctio\
n\x22,\x22AsyncFunctio\
n\x22,\x22Reflect\x22,\x22Pr\
oxy\x22,\x22Intl\x22,\x22Web\
Assembly\x22],[\x22Err\
or\x22,\x22EvalError\x22,\
\x22InternalError\x22,\
\x22RangeError\x22,\x22Re\
ferenceError\x22,\x22S\
yntaxError\x22,\x22Typ\
eError\x22,\x22URIErro\
r\x22])\x0a;return t=>\
{const r={\x0akeywo\
rd:e.concat([\x22th\
en\x22,\x22unless\x22,\x22un\
til\x22,\x22loop\x22,\x22of\x22\
,\x22by\x22,\x22when\x22,\x22an\
d\x22,\x22or\x22,\x22is\x22,\x22is\
nt\x22,\x22not\x22,\x22it\x22,\x22\
that\x22,\x22otherwise\
\x22,\x22from\x22,\x22to\x22,\x22t\
il\x22,\x22fallthrough\
\x22,\x22case\x22,\x22enum\x22,\
\x22native\x22,\x22list\x22,\
\x22map\x22,\x22__hasProp\
\x22,\x22__extends\x22,\x22_\
_slice\x22,\x22__bind\x22\
,\x22__indexOf\x22]),\x0a\
literal:n.concat\
([\x22yes\x22,\x22no\x22,\x22on\
\x22,\x22off\x22,\x22it\x22,\x22th\
at\x22,\x22void\x22]),\x0abu\
ilt_in:a.concat(\
[\x22npm\x22,\x22print\x22])\
\x0a},s=\x22[A-Za-z$_]\
(?:-[0-9A-Za-z$_\
]|[0-9A-Za-z$_])\
*\x22,i=t.inherit(t\
.TITLE_MODE,{\x0abe\
gin:s}),o={class\
Name:\x22subst\x22,beg\
in:/#\x5c{/,end:/\x5c}\
/,keywords:r},c=\
{\x0aclassName:\x22sub\
st\x22,begin:/#[A-Z\
a-z$_]/,end:/(?:\
-[0-9A-Za-z$_]|[\
0-9A-Za-z$_])*/,\
\x0akeywords:r},l=[\
t.BINARY_NUMBER_\
MODE,{className:\
\x22number\x22,\x0abegin:\
\x22(\x5c\x5cb0[xX][a-fA-\
F0-9_]+)|(\x5c\x5cb\x5c\x5cd\
(\x5c\x5cd|_\x5c\x5cd)*(\x5c\x5c.(\
\x5c\x5cd(\x5c\x5cd|_\x5c\x5cd)*)?\
)?(_*[eE]([-+]\x5c\x5c\
d(_\x5c\x5cd|\x5c\x5cd)*)?)?\
[_a-z]*)\x22,\x0arelev\
ance:0,starts:{e\
nd:\x22(\x5c\x5cs*/)?\x22,re\
levance:0}},{cla\
ssName:\x22string\x22,\
variants:[{\x0abegi\
n:/'''/,end:/'''\
/,contains:[t.BA\
CKSLASH_ESCAPE]}\
,{begin:/'/,end:\
/'/,\x0acontains:[t\
.BACKSLASH_ESCAP\
E]},{begin:/\x22\x22\x22/\
,end:/\x22\x22\x22/,\x0acont\
ains:[t.BACKSLAS\
H_ESCAPE,o,c]},{\
begin:/\x22/,end:/\x22\
/,\x0acontains:[t.B\
ACKSLASH_ESCAPE,\
o,c]},{begin:/\x5c\x5c\
/,end:/(\x5cs|$)/,e\
xcludeEnd:!0}]},\
{\x0aclassName:\x22reg\
exp\x22,variants:[{\
begin:\x22//\x22,end:\x22\
//[gim]*\x22,\x0aconta\
ins:[o,t.HASH_CO\
MMENT_MODE]},{\x0ab\
egin:/\x5c/(?![ *])\
(\x5c\x5c.|[^\x5c\x5c\x5cn])*?\x5c\
/[gim]*(?=\x5cW)/}]\
},{begin:\x22@\x22+s},\
{begin:\x22``\x22,\x0aend\
:\x22``\x22,excludeBeg\
in:!0,excludeEnd\
:!0,subLanguage:\
\x22javascript\x22}];o\
.contains=l\x0a;con\
st d={className:\
\x22params\x22,begin:\x22\
\x5c\x5c(\x22,returnBegin\
:!0,contains:[{b\
egin:/\x5c(/,\x0aend:/\
\x5c)/,keywords:r,c\
ontains:[\x22self\x22]\
.concat(l)}]},g=\
{variants:[{\x0amat\
ch:[/class\x5cs+/,s\
,/\x5cs+extends\x5cs+/\
,s]},{match:[/cl\
ass\x5cs+/,s]}],sco\
pe:{\x0a2:\x22title.cl\
ass\x22,4:\x22title.cl\
ass.inherited\x22},\
keywords:r};retu\
rn{name:\x22LiveScr\
ipt\x22,\x0aaliases:[\x22\
ls\x22],keywords:r,\
illegal:/\x5c/\x5c*/,\x0a\
contains:l.conca\
t([t.COMMENT(\x22\x5c\x5c\
/\x5c\x5c*\x22,\x22\x5c\x5c*\x5c\x5c/\x22),\
t.HASH_COMMENT_M\
ODE,{\x0abegin:\x22(#=\
>|=>|\x5c\x5c|>>|-?->|\
!->)\x22},{classNam\
e:\x22function\x22,con\
tains:[i,d],\x0aret\
urnBegin:!0,vari\
ants:[{\x0abegin:\x22(\
\x22+s+\x22\x5c\x5cs*(?:=|:=\
)\x5c\x5cs*)?(\x5c\x5c(.*\x5c\x5c)\
\x5c\x5cs*)?\x5c\x5cB->\x5c\x5c*?\x22\
,end:\x22->\x5c\x5c*?\x22},{\
\x0abegin:\x22(\x22+s+\x22\x5c\x5c\
s*(?:=|:=)\x5c\x5cs*)?\
!?(\x5c\x5c(.*\x5c\x5c)\x5c\x5cs*)\
?\x5c\x5cB[-~]{1,2}>\x5c\x5c\
*?\x22,\x0aend:\x22[-~]{1\
,2}>\x5c\x5c*?\x22},{\x0abeg\
in:\x22(\x22+s+\x22\x5c\x5cs*(?\
:=|:=)\x5c\x5cs*)?(\x5c\x5c(\
.*\x5c\x5c)\x5c\x5cs*)?\x5c\x5cB!?\
[-~]{1,2}>\x5c\x5c*?\x22,\
\x0aend:\x22!?[-~]{1,2\
}>\x5c\x5c*?\x22}]},g,{be\
gin:s+\x22:\x22,end:\x22:\
\x22,returnBegin:!0\
,returnEnd:!0,\x0ar\
elevance:0}])}}}\
)();hljs.registe\
rLanguage(\x22lives\
cript\x22,e)})();/*\
! `llvm` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
e=(()=>{\x22use str\
ict\x22;return e=>{\
\x0aconst a=e.regex\
,n=/([-a-zA-Z$._\
][\x5cw$.-]*)/,t={c\
lassName:\x22variab\
le\x22,variants:[{\x0a\
begin:a.concat(/\
%/,n)},{begin:/%\
\x5cd+/},{begin:/#\x5c\
d+/}]},i={classN\
ame:\x22title\x22,\x0avar\
iants:[{begin:a.\
concat(/@/,n)},{\
begin:/@\x5cd+/},{b\
egin:a.concat(/!\
/,n)},{\x0abegin:a.\
concat(/!\x5cd+/,n)\
},{begin:/!\x5cd+/}\
]};return{name:\x22\
LLVM IR\x22,keyword\
s:{\x0akeyword:\x22beg\
in end true fals\
e declare define\
 global constant\
 private linker_\
private internal\
 available_exter\
nally linkonce l\
inkonce_odr weak\
 weak_odr append\
ing dllimport dl\
lexport common d\
efault hidden pr\
otected extern_w\
eak external thr\
ead_local zeroin\
itializer undef \
null to tail tar\
get triple datal\
ayout volatile n\
uw nsw nnan ninf\
 nsz arcp fast e\
xact inbounds al\
ign addrspace se\
ction alias modu\
le asm sideeffec\
t gc dbg linker_\
private_weak att\
ributes blockadd\
ress initialexec\
 localdynamic lo\
calexec prefix u\
nnamed_addr ccc \
fastcc coldcc x8\
6_stdcallcc x86_\
fastcallcc arm_a\
pcscc arm_aapcsc\
c arm_aapcs_vfpc\
c ptx_device ptx\
_kernel intel_oc\
l_bicc msp430_in\
trcc spir_func s\
pir_kernel x86_6\
4_sysvcc x86_64_\
win64cc x86_this\
callcc cc c sign\
ext zeroext inre\
g sret nounwind \
noreturn noalias\
 nocapture byval\
 nest readnone r\
eadonly inlinehi\
nt noinline alwa\
ysinline optsize\
 ssp sspreq nore\
dzone noimplicit\
float naked buil\
tin cold nobuilt\
in noduplicate n\
onlazybind optno\
ne returns_twice\
 sanitize_addres\
s sanitize_memor\
y sanitize_threa\
d sspstrong uwta\
ble returned typ\
e opaque eq ne s\
lt sgt sle sge u\
lt ugt ule uge o\
eq one olt ogt o\
le oge ord uno u\
eq une x acq_rel\
 acquire alignst\
ack atomic catch\
 cleanup filter \
inteldialect max\
 min monotonic n\
and personality \
release seq_cst \
singlethread uma\
x umin unordered\
 xchg add fadd s\
ub fsub mul fmul\
 udiv sdiv fdiv \
urem srem frem s\
hl lshr ashr and\
 or xor icmp fcm\
p phi call trunc\
 zext sext fptru\
nc fpext uitofp \
sitofp fptoui fp\
tosi inttoptr pt\
rtoint bitcast a\
ddrspacecast sel\
ect va_arg ret b\
r switch invoke \
unwind unreachab\
le indirectbr la\
ndingpad resume \
malloc alloca fr\
ee load store ge\
telementptr extr\
actelement inser\
telement shuffle\
vector getresult\
 extractvalue in\
sertvalue atomic\
rmw cmpxchg fenc\
e argmemonly\x22,\x0at\
ype:\x22void half b\
float float doub\
le fp128 x86_fp8\
0 ppc_fp128 x86_\
amx x86_mmx ptr \
label token meta\
data opaque\x22\x0a},c\
ontains:[{classN\
ame:\x22type\x22,begin\
:/\x5cbi\x5cd+(?=\x5cs|\x5cb\
)/},e.COMMENT(/;\
\x5cs*$/,null,{\x0arel\
evance:0}),e.COM\
MENT(/;/,/$/),{c\
lassName:\x22string\
\x22,begin:/\x22/,end:\
/\x22/,\x0acontains:[{\
className:\x22char.\
escape\x22,match:/\x5c\
\x5c\x5cd\x5cd/}]},i,{cla\
ssName:\x22punctuat\
ion\x22,\x0arelevance:\
0,begin:/,/},{cl\
assName:\x22operato\
r\x22,relevance:0,b\
egin:/=/},t,{\x0acl\
assName:\x22symbol\x22\
,variants:[{begi\
n:/^\x5cs*[a-z]+:/}\
],relevance:0},{\
\x0aclassName:\x22numb\
er\x22,variants:[{b\
egin:/[su]?0[xX]\
[KMLHR]?[a-fA-F0\
-9]+/},{\x0abegin:/\
[-+]?\x5cd+(?:[.]\x5cd\
+)?(?:[eE][-+]?\x5c\
d+(?:[.]\x5cd+)?)?/\
}],relevance:0}]\
}}})()\x0a;hljs.reg\
isterLanguage(\x22l\
lvm\x22,e)})();/*! \
`lsl` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var E=(\
()=>{\x22use strict\
\x22;return E=>{con\
st T={className:\
\x22number\x22,\x0areleva\
nce:0,begin:E.C_\
NUMBER_RE};retur\
n{name:\x22LSL (Lin\
den Scripting La\
nguage)\x22,\x0aillega\
l:\x22:\x22,contains:[\
{className:\x22stri\
ng\x22,begin:'\x22',en\
d:'\x22',contains:[\
{\x0aclassName:\x22sub\
st\x22,begin:/\x5c\x5c[tn\
\x22\x5c\x5c]/}]},{classN\
ame:\x22comment\x22,\x0av\
ariants:[E.COMME\
NT(\x22//\x22,\x22$\x22),E.C\
OMMENT(\x22/\x5c\x5c*\x22,\x22\x5c\
\x5c*/\x22)],relevance\
:0},T,{\x0aclassNam\
e:\x22section\x22,vari\
ants:[{begin:\x22\x5c\x5c\
b(state|default)\
\x5c\x5cb\x22},{\x0abegin:\x22\x5c\
\x5cb(state_(entry|\
exit)|touch(_(st\
art|end))?|(land\
_)?collision(_(s\
tart|end))?|time\
r|listen|(no_)?s\
ensor|control|(n\
ot_)?at_(rot_)?t\
arget|money|emai\
l|experience_per\
missions(_denied\
)?|run_time_perm\
issions|changed|\
attach|dataserve\
r|moving_(start|\
end)|link_messag\
e|(on|object)_re\
z|remote_data|ht\
tp_re(sponse|que\
st)|path_update|\
transaction_resu\
lt)\x5c\x5cb\x22\x0a}]},{cla\
ssName:\x22built_in\
\x22,\x0abegin:\x22\x5c\x5cb(ll\
(AgentInExperien\
ce|(Create|DataS\
ize|Delete|KeyCo\
unt|Keys|Read|Up\
date)KeyValue|Ge\
tExperience(Deta\
ils|ErrorMessage\
)|ReturnObjectsB\
y(ID|Owner)|Json\
(2List|[GS]etVal\
ue|ValueType)|Si\
n|Cos|Tan|Atan2|\
Sqrt|Pow|Abs|Fab\
s|Frand|Floor|Ce\
il|Round|Vec(Mag\
|Norm|Dist)|Rot(\
Between|2(Euler|\
Fwd|Left|Up))|(E\
uler|Axes)2Rot|W\
hisper|(Region|O\
wner)?Say|Shout|\
Listen(Control|R\
emove)?|Sensor(R\
epeat|Remove)?|D\
etected(Name|Key\
|Owner|Type|Pos|\
Vel|Grab|Rot|Gro\
up|LinkNumber)|D\
ie|Ground|Wind|(\
[GS]et)(Animatio\
nOverride|Memory\
Limit|PrimMediaP\
arams|ParcelMusi\
cURL|Object(Desc\
|Name)|PhysicsMa\
terial|Status|Sc\
ale|Color|Alpha|\
Texture|Pos|Rot|\
Force|Torque)|Re\
setAnimationOver\
ride|(Scale|Offs\
et|Rotate)Textur\
e|(Rot)?Target(R\
emove)?|(Stop)?M\
oveToTarget|Appl\
y(Rotational)?Im\
pulse|Set(Keyfra\
medMotion|Conten\
tType|RegionPos|\
(Angular)?Veloci\
ty|Buoyancy|Hove\
rHeight|ForceAnd\
Torque|TimerEven\
t|ScriptState|Da\
mage|TextureAnim\
|Sound(Queueing|\
Radius)|Vehicle(\
Type|(Float|Vect\
or|Rotation)Para\
m)|(Touch|Sit)?T\
ext|Camera(Eye|A\
t)Offset|Primiti\
veParams|ClickAc\
tion|Link(Alpha|\
Color|PrimitiveP\
arams(Fast)?|Tex\
ture(Anim)?|Came\
ra|Media)|Remote\
ScriptAccessPin|\
PayPrice|LocalRo\
t)|ScaleByFactor\
|Get((Max|Min)Sc\
aleFactor|Closes\
tNavPoint|Static\
Path|SimStats|En\
v|PrimitiveParam\
s|Link(Primitive\
Params|Number(Of\
Sides)?|Key|Name\
|Media)|HTTPHead\
er|FreeURLs|Obje\
ct(Details|PermM\
ask|PrimCount)|P\
arcel(MaxPrims|D\
etails|Prim(Coun\
t|Owners))|Attac\
hed(List)?|(SPMa\
x|Free|Used)Memo\
ry|Region(Name|T\
imeDilation|FPS|\
Corner|AgentCoun\
t)|Root(Position\
|Rotation)|UnixT\
ime|(Parcel|Regi\
on)Flags|(Wall|G\
MT)clock|Simulat\
orHostname|Bound\
ingBox|Geometric\
Center|Creator|N\
umberOf(Prims|No\
tecardLines|Side\
s)|Animation(Lis\
t)?|(Camera|Loca\
l)(Pos|Rot)|Vel|\
Accel|Omega|Time\
(stamp|OfDay)|(O\
bject|CenterOf)?\
Mass|MassMKS|Ene\
rgy|Owner|(Owner\
)?Key|SunDirecti\
on|Texture(Offse\
t|Scale|Rot)|Inv\
entory(Number|Na\
me|Key|Type|Crea\
tor|PermMask)|Pe\
rmissions(Key)?|\
StartParameter|L\
ist(Length|Entry\
Type)|Date|Agent\
(Size|Info|Langu\
age|List)|LandOw\
nerAt|NotecardLi\
ne|Script(Name|S\
tate))|(Get|Rese\
t|GetAndReset)Ti\
me|PlaySound(Sla\
ve)?|LoopSound(M\
aster|Slave)?|(T\
rigger|Stop|Prel\
oad)Sound|((Get|\
Delete)Sub|Inser\
t)String|To(Uppe\
r|Lower)|Give(In\
ventoryList|Mone\
y)|RezObject|(St\
op)?LookAt|Sleep\
|CollisionFilter\
|(Take|Release)C\
ontrols|DetachFr\
omAvatar|AttachT\
oAvatar(Temp)?|I\
nstantMessage|(G\
etNext)?Email|St\
opHover|MinEvent\
Delay|RotLookAt|\
String(Length|Tr\
im)|(Start|Stop)\
Animation|Target\
Omega|Request(Ex\
perience)?Permis\
sions|(Create|Br\
eak)Link|BreakAl\
lLinks|(Give|Rem\
ove)Inventory|Wa\
ter|PassTouches|\
Request(Agent|In\
ventory)Data|Tel\
eportAgent(Home|\
GlobalCoords)?|M\
odifyLand|Collis\
ionSound|ResetSc\
ript|MessageLink\
ed|PushObject|Pa\
ssCollisions|Axi\
sAngle2Rot|Rot2(\
Axis|Angle)|A(co\
s|sin)|AngleBetw\
een|AllowInvento\
ryDrop|SubString\
Index|List2(CSV|\
Integer|Json|Flo\
at|String|Key|Ve\
ctor|Rot|List(St\
rided)?)|DeleteS\
ubList|List(Stat\
istics|Sort|Rand\
omize|(Insert|Fi\
nd|Replace)List)\
|EdgeOfWorld|Adj\
ustSoundVolume|K\
ey2Name|TriggerS\
oundLimited|Ejec\
tFromLand|(CSV|P\
arseString)2List\
|OverMyLand|Same\
Group|UnSit|Grou\
nd(Slope|Normal|\
Contour)|GroundR\
epel|(Set|Remove\
)VehicleFlags|Si\
tOnLink|(AvatarO\
n)?(Link)?SitTar\
get|Script(Dange\
r|Profiler)|Dial\
og|VolumeDetect|\
ResetOtherScript\
|RemoteLoadScrip\
tPin|(Open|Close\
)RemoteDataChann\
el|SendRemoteDat\
a|RemoteDataRepl\
y|(Integer|Strin\
g)ToBase64|XorBa\
se64|Log(10)?|Ba\
se64To(String|In\
teger)|ParseStri\
ngKeepNulls|RezA\
tRoot|RequestSim\
ulatorData|Force\
Mouselook|(Load|\
Release|(E|Une)s\
cape)URL|ParcelM\
edia(CommandList\
|Query)|ModPow|M\
apDestination|(R\
emoveFrom|AddTo|\
Reset)Land(Pass|\
Ban)List|(Set|Cl\
ear)CameraParams\
|HTTP(Request|Re\
sponse)|TextBox|\
DetectedTouch(UV\
|Face|Pos|(N|Bin\
)ormal|ST)|(MD5|\
SHA1|DumpList2)S\
tring|Request(Se\
cure)?URL|Clear(\
Prim|Link)Media|\
(Link)?ParticleS\
ystem|(Get|Reque\
st)(Username|Dis\
playName)|Region\
SayTo|CastRay|Ge\
nerateKey|Transf\
erLindenDollars|\
ManageEstateAcce\
ss|(Create|Delet\
e)Character|Exec\
CharacterCmd|Eva\
de|FleeFrom|Navi\
gateTo|PatrolPoi\
nts|Pursue|Updat\
eCharacter|Wande\
rWithin))\x5c\x5cb\x22\x0a},\
{className:\x22lite\
ral\x22,variants:[{\
\x0abegin:\x22\x5c\x5cb(PI|T\
WO_PI|PI_BY_TWO|\
DEG_TO_RAD|RAD_T\
O_DEG|SQRT2)\x5c\x5cb\x22\
},{\x0abegin:\x22\x5c\x5cb(X\
P_ERROR_(EXPERIE\
NCES_DISABLED|EX\
PERIENCE_(DISABL\
ED|SUSPENDED)|IN\
VALID_(EXPERIENC\
E|PARAMETERS)|KE\
Y_NOT_FOUND|MATU\
RITY_EXCEEDED|NO\
NE|NOT_(FOUND|PE\
RMITTED(_LAND)?)\
|NO_EXPERIENCE|Q\
UOTA_EXCEEDED|RE\
TRY_UPDATE|STORA\
GE_EXCEPTION|STO\
RE_DISABLED|THRO\
TTLED|UNKNOWN_ER\
ROR)|JSON_APPEND\
|STATUS_(PHYSICS\
|ROTATE_[XYZ]|PH\
ANTOM|SANDBOX|BL\
OCK_GRAB(_OBJECT\
)?|(DIE|RETURN)_\
AT_EDGE|CAST_SHA\
DOWS|OK|MALFORME\
D_PARAMS|TYPE_MI\
SMATCH|BOUNDS_ER\
ROR|NOT_(FOUND|S\
UPPORTED)|INTERN\
AL_ERROR|WHITELI\
ST_FAILED)|AGENT\
(_(BY_(LEGACY_|U\
SER)NAME|FLYING|\
ATTACHMENTS|SCRI\
PTED|MOUSELOOK|S\
ITTING|ON_OBJECT\
|AWAY|WALKING|IN\
_AIR|TYPING|CROU\
CHING|BUSY|ALWAY\
S_RUN|AUTOPILOT|\
LIST_(PARCEL(_OW\
NER)?|REGION)))?\
|CAMERA_(PITCH|D\
ISTANCE|BEHINDNE\
SS_(ANGLE|LAG)|(\
FOCUS|POSITION)(\
_(THRESHOLD|LOCK\
ED|LAG))?|FOCUS_\
OFFSET|ACTIVE)|A\
NIM_ON|LOOP|REVE\
RSE|PING_PONG|SM\
OOTH|ROTATE|SCAL\
E|ALL_SIDES|LINK\
_(ROOT|SET|ALL_(\
OTHERS|CHILDREN)\
|THIS)|ACTIVE|PA\
SS(IVE|_(ALWAYS|\
IF_NOT_HANDLED|N\
EVER))|SCRIPTED|\
CONTROL_(FWD|BAC\
K|(ROT_)?(LEFT|R\
IGHT)|UP|DOWN|(M\
L_)?LBUTTON)|PER\
MISSION_(RETURN_\
OBJECTS|DEBIT|OV\
ERRIDE_ANIMATION\
S|SILENT_ESTATE_\
MANAGEMENT|TAKE_\
CONTROLS|TRIGGER\
_ANIMATION|ATTAC\
H|CHANGE_LINKS|(\
CONTROL|TRACK)_C\
AMERA|TELEPORT)|\
INVENTORY_(TEXTU\
RE|SOUND|OBJECT|\
SCRIPT|LANDMARK|\
CLOTHING|NOTECAR\
D|BODYPART|ANIMA\
TION|GESTURE|ALL\
|NONE)|CHANGED_(\
INVENTORY|COLOR|\
SHAPE|SCALE|TEXT\
URE|LINK|ALLOWED\
_DROP|OWNER|REGI\
ON(_START)?|TELE\
PORT|MEDIA)|OBJE\
CT_(CLICK_ACTION\
|HOVER_HEIGHT|LA\
ST_OWNER_ID|(PHY\
SICS|SERVER|STRE\
AMING)_COST|UNKN\
OWN_DETAIL|CHARA\
CTER_TIME|PHANTO\
M|PHYSICS|TEMP_(\
ATTACHED|ON_REZ)\
|NAME|DESC|POS|P\
RIM_(COUNT|EQUIV\
ALENCE)|RETURN_(\
PARCEL(_OWNER)?|\
REGION)|REZZER_K\
EY|ROO?T|VELOCIT\
Y|OMEGA|OWNER|GR\
OUP(_TAG)?|CREAT\
OR|ATTACHED_(POI\
NT|SLOTS_AVAILAB\
LE)|RENDER_WEIGH\
T|(BODY_SHAPE|PA\
THFINDING)_TYPE|\
(RUNNING|TOTAL)_\
SCRIPT_COUNT|TOT\
AL_INVENTORY_COU\
NT|SCRIPT_(MEMOR\
Y|TIME))|TYPE_(I\
NTEGER|FLOAT|STR\
ING|KEY|VECTOR|R\
OTATION|INVALID)\
|(DEBUG|PUBLIC)_\
CHANNEL|ATTACH_(\
AVATAR_CENTER|CH\
EST|HEAD|BACK|PE\
LVIS|MOUTH|CHIN|\
NECK|NOSE|BELLY|\
[LR](SHOULDER|HA\
ND|FOOT|EAR|EYE|\
[UL](ARM|LEG)|HI\
P)|(LEFT|RIGHT)_\
PEC|HUD_(CENTER_\
[12]|TOP_(RIGHT|\
CENTER|LEFT)|BOT\
TOM(_(RIGHT|LEFT\
))?)|[LR]HAND_RI\
NG1|TAIL_(BASE|T\
IP)|[LR]WING|FAC\
E_(JAW|[LR]EAR|[\
LR]EYE|TOUNGE)|G\
ROIN|HIND_[LR]FO\
OT)|LAND_(LEVEL|\
RAISE|LOWER|SMOO\
TH|NOISE|REVERT)\
|DATA_(ONLINE|NA\
ME|BORN|SIM_(POS\
|STATUS|RATING)|\
PAYINFO)|PAYMENT\
_INFO_(ON_FILE|U\
SED)|REMOTE_DATA\
_(CHANNEL|REQUES\
T|REPLY)|PSYS_(P\
ART_(BF_(ZERO|ON\
E(_MINUS_(DEST_C\
OLOR|SOURCE_(ALP\
HA|COLOR)))?|DES\
T_COLOR|SOURCE_(\
ALPHA|COLOR))|BL\
END_FUNC_(DEST|S\
OURCE)|FLAGS|(ST\
ART|END)_(COLOR|\
ALPHA|SCALE|GLOW\
)|MAX_AGE|(RIBBO\
N|WIND|INTERP_(C\
OLOR|SCALE)|BOUN\
CE|FOLLOW_(SRC|V\
ELOCITY)|TARGET_\
(POS|LINEAR)|EMI\
SSIVE)_MASK)|SRC\
_(MAX_AGE|PATTER\
N|ANGLE_(BEGIN|E\
ND)|BURST_(RATE|\
PART_COUNT|RADIU\
S|SPEED_(MIN|MAX\
))|ACCEL|TEXTURE\
|TARGET_KEY|OMEG\
A|PATTERN_(DROP|\
EXPLODE|ANGLE(_C\
ONE(_EMPTY)?)?))\
)|VEHICLE_(REFER\
ENCE_FRAME|TYPE_\
(NONE|SLED|CAR|B\
OAT|AIRPLANE|BAL\
LOON)|(LINEAR|AN\
GULAR)_(FRICTION\
_TIMESCALE|MOTOR\
_DIRECTION)|LINE\
AR_MOTOR_OFFSET|\
HOVER_(HEIGHT|EF\
FICIENCY|TIMESCA\
LE)|BUOYANCY|(LI\
NEAR|ANGULAR)_(D\
EFLECTION_(EFFIC\
IENCY|TIMESCALE)\
|MOTOR_(DECAY_)?\
TIMESCALE)|VERTI\
CAL_ATTRACTION_(\
EFFICIENCY|TIMES\
CALE)|BANKING_(E\
FFICIENCY|MIX|TI\
MESCALE)|FLAG_(N\
O_DEFLECTION_UP|\
LIMIT_(ROLL_ONLY\
|MOTOR_UP)|HOVER\
_((WATER|TERRAIN\
|UP)_ONLY|GLOBAL\
_HEIGHT)|MOUSELO\
OK_(STEER|BANK)|\
CAMERA_DECOUPLED\
))|PRIM_(ALLOW_U\
NSIT|ALPHA_MODE(\
_(BLEND|EMISSIVE\
|MASK|NONE))?|NO\
RMAL|SPECULAR|TY\
PE(_(BOX|CYLINDE\
R|PRISM|SPHERE|T\
ORUS|TUBE|RING|S\
CULPT))?|HOLE_(D\
EFAULT|CIRCLE|SQ\
UARE|TRIANGLE)|M\
ATERIAL(_(STONE|\
METAL|GLASS|WOOD\
|FLESH|PLASTIC|R\
UBBER))?|SHINY_(\
NONE|LOW|MEDIUM|\
HIGH)|BUMP_(NONE\
|BRIGHT|DARK|WOO\
D|BARK|BRICKS|CH\
ECKER|CONCRETE|T\
ILE|STONE|DISKS|\
GRAVEL|BLOBS|SID\
ING|LARGETILE|ST\
UCCO|SUCTION|WEA\
VE)|TEXGEN_(DEFA\
ULT|PLANAR)|SCRI\
PTED_SIT_ONLY|SC\
ULPT_(TYPE_(SPHE\
RE|TORUS|PLANE|C\
YLINDER|MASK)|FL\
AG_(MIRROR|INVER\
T))|PHYSICS(_(SH\
APE_(CONVEX|NONE\
|PRIM|TYPE)))?|(\
POS|ROT)_LOCAL|S\
LICE|TEXT|FLEXIB\
LE|POINT_LIGHT|T\
EMP_ON_REZ|PHANT\
OM|POSITION|SIT_\
TARGET|SIZE|ROTA\
TION|TEXTURE|NAM\
E|OMEGA|DESC|LIN\
K_TARGET|COLOR|B\
UMP_SHINY|FULLBR\
IGHT|TEXGEN|GLOW\
|MEDIA_(ALT_IMAG\
E_ENABLE|CONTROL\
S|(CURRENT|HOME)\
_URL|AUTO_(LOOP|\
PLAY|SCALE|ZOOM)\
|FIRST_CLICK_INT\
ERACT|(WIDTH|HEI\
GHT)_PIXELS|WHIT\
ELIST(_ENABLE)?|\
PERMS_(INTERACT|\
CONTROL)|PARAM_M\
AX|CONTROLS_(STA\
NDARD|MINI)|PERM\
_(NONE|OWNER|GRO\
UP|ANYONE)|MAX_(\
URL_LENGTH|WHITE\
LIST_(SIZE|COUNT\
)|(WIDTH|HEIGHT)\
_PIXELS)))|MASK_\
(BASE|OWNER|GROU\
P|EVERYONE|NEXT)\
|PERM_(TRANSFER|\
MODIFY|COPY|MOVE\
|ALL)|PARCEL_(ME\
DIA_COMMAND_(STO\
P|PAUSE|PLAY|LOO\
P|TEXTURE|URL|TI\
ME|AGENT|UNLOAD|\
AUTO_ALIGN|TYPE|\
SIZE|DESC|LOOP_S\
ET)|FLAG_(ALLOW_\
(FLY|(GROUP_)?SC\
RIPTS|LANDMARK|T\
ERRAFORM|DAMAGE|\
CREATE_(GROUP_)?\
OBJECTS)|USE_(AC\
CESS_(GROUP|LIST\
)|BAN_LIST|LAND_\
PASS_LIST)|LOCAL\
_SOUND_ONLY|REST\
RICT_PUSHOBJECT|\
ALLOW_(GROUP|ALL\
)_OBJECT_ENTRY)|\
COUNT_(TOTAL|OWN\
ER|GROUP|OTHER|S\
ELECTED|TEMP)|DE\
TAILS_(NAME|DESC\
|OWNER|GROUP|ARE\
A|ID|SEE_AVATARS\
))|LIST_STAT_(MA\
X|MIN|MEAN|MEDIA\
N|STD_DEV|SUM(_S\
QUARES)?|NUM_COU\
NT|GEOMETRIC_MEA\
N|RANGE)|PAY_(HI\
DE|DEFAULT)|REGI\
ON_FLAG_(ALLOW_D\
AMAGE|FIXED_SUN|\
BLOCK_TERRAFORM|\
SANDBOX|DISABLE_\
(COLLISIONS|PHYS\
ICS)|BLOCK_FLY|A\
LLOW_DIRECT_TELE\
PORT|RESTRICT_PU\
SHOBJECT)|HTTP_(\
METHOD|MIMETYPE|\
BODY_(MAXLENGTH|\
TRUNCATED)|CUSTO\
M_HEADER|PRAGMA_\
NO_CACHE|VERBOSE\
_THROTTLE|VERIFY\
_CERT)|SIT_(INVA\
LID_(AGENT|LINK_\
OBJECT)|NO(T_EXP\
ERIENCE|_(ACCESS\
|EXPERIENCE_PERM\
ISSION|SIT_TARGE\
T)))|STRING_(TRI\
M(_(HEAD|TAIL))?\
)|CLICK_ACTION_(\
NONE|TOUCH|SIT|B\
UY|PAY|OPEN(_MED\
IA)?|PLAY|ZOOM)|\
TOUCH_INVALID_FA\
CE|PROFILE_(NONE\
|SCRIPT_MEMORY)|\
RC_(DATA_FLAGS|D\
ETECT_PHANTOM|GE\
T_(LINK_NUM|NORM\
AL|ROOT_KEY)|MAX\
_HITS|REJECT_(TY\
PES|AGENTS|(NON)\
?PHYSICAL|LAND))\
|RCERR_(CAST_TIM\
E_EXCEEDED|SIM_P\
ERF_LOW|UNKNOWN)\
|ESTATE_ACCESS_(\
ALLOWED_(AGENT|G\
ROUP)_(ADD|REMOV\
E)|BANNED_AGENT_\
(ADD|REMOVE))|DE\
NSITY|FRICTION|R\
ESTITUTION|GRAVI\
TY_MULTIPLIER|KF\
M_(COMMAND|CMD_(\
PLAY|STOP|PAUSE)\
|MODE|FORWARD|LO\
OP|PING_PONG|REV\
ERSE|DATA|ROTATI\
ON|TRANSLATION)|\
ERR_(GENERIC|PAR\
CEL_PERMISSIONS|\
MALFORMED_PARAMS\
|RUNTIME_PERMISS\
IONS|THROTTLED)|\
CHARACTER_(CMD_(\
(SMOOTH_)?STOP|J\
UMP)|DESIRED_(TU\
RN_)?SPEED|RADIU\
S|STAY_WITHIN_PA\
RCEL|LENGTH|ORIE\
NTATION|ACCOUNT_\
FOR_SKIPPED_FRAM\
ES|AVOIDANCE_MOD\
E|TYPE(_([ABCD]|\
NONE))?|MAX_(DEC\
EL|TURN_RADIUS|(\
ACCEL|SPEED)))|P\
URSUIT_(OFFSET|F\
UZZ_FACTOR|GOAL_\
TOLERANCE|INTERC\
EPT)|REQUIRE_LIN\
E_OF_SIGHT|FORCE\
_DIRECT_PATH|VER\
TICAL|HORIZONTAL\
|AVOID_(CHARACTE\
RS|DYNAMIC_OBSTA\
CLES|NONE)|PU_(E\
VADE_(HIDDEN|SPO\
TTED)|FAILURE_(D\
YNAMIC_PATHFINDI\
NG_DISABLED|INVA\
LID_(GOAL|START)\
|NO_(NAVMESH|VAL\
ID_DESTINATION)|\
OTHER|TARGET_GON\
E|(PARCEL_)?UNRE\
ACHABLE)|(GOAL|S\
LOWDOWN_DISTANCE\
)_REACHED)|TRAVE\
RSAL_TYPE(_(FAST\
|NONE|SLOW))?|CO\
NTENT_TYPE_(ATOM\
|FORM|HTML|JSON|\
LLSD|RSS|TEXT|XH\
TML|XML)|GCNP_(R\
ADIUS|STATIC)|(P\
ATROL|WANDER)_PA\
USE_AT_WAYPOINTS\
|OPT_(AVATAR|CHA\
RACTER|EXCLUSION\
_VOLUME|LEGACY_L\
INKSET|MATERIAL_\
VOLUME|OTHER|STA\
TIC_OBSTACLE|WAL\
KABLE)|SIM_STAT_\
PCT_CHARS_STEPPE\
D)\x5c\x5cb\x22\x0a},{begin:\
\x22\x5c\x5cb(FALSE|TRUE)\
\x5c\x5cb\x22},{begin:\x22\x5c\x5c\
b(ZERO_ROTATION)\
\x5c\x5cb\x22},{\x0abegin:\x22\x5c\
\x5cb(EOF|JSON_(ARR\
AY|DELETE|FALSE|\
INVALID|NULL|NUM\
BER|OBJECT|STRIN\
G|TRUE)|NULL_KEY\
|TEXTURE_(BLANK|\
DEFAULT|MEDIA|PL\
YWOOD|TRANSPAREN\
T)|URL_REQUEST_(\
GRANTED|DENIED))\
\x5c\x5cb\x22\x0a},{begin:\x22\x5c\
\x5cb(ZERO_VECTOR|T\
OUCH_INVALID_(TE\
XCOORD|VECTOR))\x5c\
\x5cb\x22}]},{\x0aclassNa\
me:\x22type\x22,\x0abegin\
:\x22\x5c\x5cb(integer|fl\
oat|string|key|v\
ector|quaternion\
|rotation|list)\x5c\
\x5cb\x22}]}}})()\x0a;hlj\
s.registerLangua\
ge(\x22lsl\x22,E)})();\
/*! `lua` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
{const t=\x22\x5c\x5c[=*\x5c\
\x5c[\x22,a=\x22\x5c\x5c]=*\x5c\x5c]\x22\
,n={\x0abegin:t,end\
:a,contains:[\x22se\
lf\x22]\x0a},o=[e.COMM\
ENT(\x22--(?!\x22+t+\x22)\
\x22,\x22$\x22),e.COMMENT\
(\x22--\x22+t,a,{conta\
ins:[n],relevanc\
e:10\x0a})];return{\
name:\x22Lua\x22,keywo\
rds:{$pattern:e.\
UNDERSCORE_IDENT\
_RE,\x0aliteral:\x22tr\
ue false nil\x22,\x0ak\
eyword:\x22and brea\
k do else elseif\
 end for goto if\
 in local not or\
 repeat return t\
hen until while\x22\
,\x0abuilt_in:\x22_G _\
ENV _VERSION __i\
ndex __newindex \
__mode __call __\
metatable __tost\
ring __len __gc \
__add __sub __mu\
l __div __mod __\
pow __concat __u\
nm __eq __lt __l\
e assert collect\
garbage dofile e\
rror getfenv get\
metatable ipairs\
 load loadfile l\
oadstring module\
 next pairs pcal\
l print rawequal\
 rawget rawset r\
equire select se\
tfenv setmetatab\
le tonumber tost\
ring type unpack\
 xpcall arg self\
 coroutine resum\
e yield status w\
rap create runni\
ng debug getupva\
lue debug sethoo\
k getmetatable g\
ethook setmetata\
ble setlocal tra\
ceback setfenv g\
etinfo setupvalu\
e getlocal getre\
gistry getfenv i\
o lines write cl\
ose flush open o\
utput type read \
stderr stdin inp\
ut stdout popen \
tmpfile math log\
 max acos huge l\
dexp pi cos tanh\
 pow deg tan cos\
h sinh random ra\
ndomseed frexp c\
eil floor rad ab\
s sqrt modf asin\
 min mod fmod lo\
g10 atan2 exp si\
n atan os exit s\
etlocale date ge\
tenv difftime re\
move time clock \
tmpname rename e\
xecute package p\
reload loadlib l\
oaded loaders cp\
ath config path \
seeall string su\
b upper len gfin\
d rep find match\
 char dump gmatc\
h reverse byte f\
ormat gsub lower\
 table setn inse\
rt getn foreachi\
 maxn foreach co\
ncat sort remove\
\x22\x0a},contains:o.c\
oncat([{classNam\
e:\x22function\x22,beg\
inKeywords:\x22func\
tion\x22,end:\x22\x5c\x5c)\x22,\
\x0acontains:[e.inh\
erit(e.TITLE_MOD\
E,{\x0abegin:\x22([_a-\
zA-Z]\x5c\x5cw*\x5c\x5c.)*([\
_a-zA-Z]\x5c\x5cw*:)?[\
_a-zA-Z]\x5c\x5cw*\x22}),\
{className:\x22para\
ms\x22,\x0abegin:\x22\x5c\x5c(\x22\
,endsWithParent:\
!0,contains:o}].\
concat(o)\x0a},e.C_\
NUMBER_MODE,e.AP\
OS_STRING_MODE,e\
.QUOTE_STRING_MO\
DE,{className:\x22s\
tring\x22,\x0abegin:t,\
end:a,contains:[\
n],relevance:5}]\
)}}})();hljs.reg\
isterLanguage(\x22l\
ua\x22,e)\x0a})();/*! \
`makefile` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{const i={class\
Name:\x22variable\x22,\
\x0avariants:[{begi\
n:\x22\x5c\x5c$\x5c\x5c(\x22+e.UND\
ERSCORE_IDENT_RE\
+\x22\x5c\x5c)\x22,\x0acontains\
:[e.BACKSLASH_ES\
CAPE]},{begin:/\x5c\
$[@%<?\x5c^\x5c+\x5c*]/}]\
},a={className:\x22\
string\x22,\x0abegin:/\
\x22/,end:/\x22/,conta\
ins:[e.BACKSLASH\
_ESCAPE,i]},n={c\
lassName:\x22variab\
le\x22,\x0abegin:/\x5c$\x5c(\
[\x5cw-]+\x5cs/,end:/\x5c\
)/,keywords:{\x0abu\
ilt_in:\x22subst pa\
tsubst strip fin\
dstring filter f\
ilter-out sort w\
ord wordlist fir\
stword lastword \
dir notdir suffi\
x basename addsu\
ffix addprefix j\
oin wildcard rea\
lpath abspath er\
ror warning shel\
l origin flavor \
foreach if or an\
d call eval file\
 value\x22\x0a},contai\
ns:[i]},s={begin\
:\x22^\x22+e.UNDERSCOR\
E_IDENT_RE+\x22\x5c\x5cs*\
(?=[:+?]?=)\x22},r=\
{\x0aclassName:\x22sec\
tion\x22,begin:/^[^\
\x5cs]+:/,end:/$/,c\
ontains:[i]};ret\
urn{\x0aname:\x22Makef\
ile\x22,aliases:[\x22m\
k\x22,\x22mak\x22,\x22make\x22]\
,keywords:{$patt\
ern:/[\x5cw-]+/,\x0ake\
yword:\x22define en\
def undefine ifd\
ef ifndef ifeq i\
fneq else endif \
include -include\
 sinclude overri\
de export unexpo\
rt private vpath\
\x22\x0a},contains:[e.\
HASH_COMMENT_MOD\
E,i,a,n,s,{class\
Name:\x22meta\x22,begi\
n:/^\x5c.PHONY:/,\x0ae\
nd:/$/,keywords:\
{$pattern:/[\x5c.\x5cw\
]+/,keyword:\x22.PH\
ONY\x22}},r]}}})()\x0a\
;hljs.registerLa\
nguage(\x22makefile\
\x22,e)})();/*! `ma\
rkdown` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>{c\
onst n={begin:/<\
\x5c/?[A-Za-z_]/,\x0ae\
nd:\x22>\x22,subLangua\
ge:\x22xml\x22,relevan\
ce:0},a={variant\
s:[{begin:/\x5c[.+?\
\x5c]\x5c[.*?\x5c]/,\x0arele\
vance:0},{\x0abegin\
:/\x5c[.+?\x5c]\x5c(((dat\
a|javascript|mai\
lto):|(?:http|ft\
p)s?:\x5c/\x5c/).*?\x5c)/\
,\x0arelevance:2},{\
\x0abegin:e.regex.c\
oncat(/\x5c[.+?\x5c]\x5c(\
/,/[A-Za-z][A-Za\
-z0-9+.-]*/,/:\x5c/\
\x5c/.*?\x5c)/),\x0arelev\
ance:2},{begin:/\
\x5c[.+?\x5c]\x5c([./?&#]\
.*?\x5c)/,relevance\
:1},{\x0abegin:/\x5c[.\
*?\x5c]\x5c(.*?\x5c)/,rel\
evance:0}],retur\
nBegin:!0,contai\
ns:[{match:/\x5c[(?\
=\x5c])/\x0a},{classNa\
me:\x22string\x22,rele\
vance:0,begin:\x22\x5c\
\x5c[\x22,end:\x22\x5c\x5c]\x22,ex\
cludeBegin:!0,\x0ar\
eturnEnd:!0},{cl\
assName:\x22link\x22,r\
elevance:0,begin\
:\x22\x5c\x5c]\x5c\x5c(\x22,end:\x22\x5c\
\x5c)\x22,\x0aexcludeBegi\
n:!0,excludeEnd:\
!0},{className:\x22\
symbol\x22,relevanc\
e:0,begin:\x22\x5c\x5c]\x5c\x5c\
[\x22,\x0aend:\x22\x5c\x5c]\x22,ex\
cludeBegin:!0,ex\
cludeEnd:!0}]},i\
={className:\x22str\
ong\x22,contains:[]\
,\x0avariants:[{beg\
in:/_{2}(?!\x5cs)/,\
end:/_{2}/},{beg\
in:/\x5c*{2}(?!\x5cs)/\
,end:/\x5c*{2}/}]\x0a}\
,s={className:\x22e\
mphasis\x22,contain\
s:[],variants:[{\
begin:/\x5c*(?![*\x5cs\
])/,end:/\x5c*/},{\x0a\
begin:/_(?![_\x5cs]\
)/,end:/_/,relev\
ance:0}]},c=e.in\
herit(i,{contain\
s:[]\x0a}),t=e.inhe\
rit(s,{contains:\
[]});i.contains.\
push(t),s.contai\
ns.push(c)\x0a;let \
g=[n,a];return[i\
,s,c,t].forEach(\
(e=>{e.contains=\
e.contains.conca\
t(g)\x0a})),g=g.con\
cat(i,s),{name:\x22\
Markdown\x22,aliase\
s:[\x22md\x22,\x22mkdown\x22\
,\x22mkd\x22],contains\
:[{\x0aclassName:\x22s\
ection\x22,variants\
:[{begin:\x22^#{1,6\
}\x22,end:\x22$\x22,conta\
ins:g},{\x0abegin:\x22\
(?=^.+?\x5c\x5cn[=-]{2\
,}$)\x22,contains:[\
{begin:\x22^[=-]*$\x22\
},{begin:\x22^\x22,end\
:\x22\x5c\x5cn\x22,\x0acontains\
:g}]}]},n,{class\
Name:\x22bullet\x22,be\
gin:\x22^[ \x5ct]*([*+\
-]|(\x5c\x5cd+\x5c\x5c.))(?=\
\x5c\x5cs+)\x22,\x0aend:\x22\x5c\x5cs\
+\x22,excludeEnd:!0\
},i,s,{className\
:\x22quote\x22,begin:\x22\
^>\x5c\x5cs+\x22,contains\
:g,\x0aend:\x22$\x22},{cl\
assName:\x22code\x22,v\
ariants:[{begin:\
\x22(`{3,})[^`](.|\x5c\
\x5cn)*?\x5c\x5c1`*[ ]*\x22}\
,{\x0abegin:\x22(~{3,}\
)[^~](.|\x5c\x5cn)*?\x5c\x5c\
1~*[ ]*\x22},{begin\
:\x22```\x22,end:\x22```+\
[ ]*$\x22},{\x0abegin:\
\x22~~~\x22,end:\x22~~~+[\
 ]*$\x22},{begin:\x22`\
.+?`\x22},{begin:\x22(\
?=^( {4}|\x5c\x5ct))\x22,\
\x0acontains:[{begi\
n:\x22^( {4}|\x5c\x5ct)\x22,\
end:\x22(\x5c\x5cn)$\x22}],r\
elevance:0}]},{\x0a\
begin:\x22^[-\x5c\x5c*]{3\
,}\x22,end:\x22$\x22},a,{\
begin:/^\x5c[[^\x5cn]+\
\x5c]:/,returnBegin\
:!0,contains:[{\x0a\
className:\x22symbo\
l\x22,begin:/\x5c[/,en\
d:/\x5c]/,excludeBe\
gin:!0,excludeEn\
d:!0},{\x0aclassNam\
e:\x22link\x22,begin:/\
:\x5cs*/,end:/$/,ex\
cludeBegin:!0}]}\
,{scope:\x22literal\
\x22,\x0amatch:/&([a-z\
A-Z0-9]+|#[0-9]{\
1,7}|#[Xx][0-9a-\
fA-F]{1,6});/}]}\
}})()\x0a;hljs.regi\
sterLanguage(\x22ma\
rkdown\x22,e)})();/\
*! `mathematica`\
 grammar compile\
d for Highlight.\
js 11.10.0 */\x0a((\
)=>{var e=(()=>{\
\x22use strict\x22\x0a;co\
nst e=[\x22AASTrian\
gle\x22,\x22AbelianGro\
up\x22,\x22Abort\x22,\x22Abo\
rtKernels\x22,\x22Abor\
tProtect\x22,\x22Abort\
ScheduledTask\x22,\x22\
Above\x22,\x22Abs\x22,\x22Ab\
sArg\x22,\x22AbsArgPlo\
t\x22,\x22Absolute\x22,\x22A\
bsoluteCorrelati\
on\x22,\x22AbsoluteCor\
relationFunction\
\x22,\x22AbsoluteCurre\
ntValue\x22,\x22Absolu\
teDashing\x22,\x22Abso\
luteFileName\x22,\x22A\
bsoluteOptions\x22,\
\x22AbsolutePointSi\
ze\x22,\x22AbsoluteThi\
ckness\x22,\x22Absolut\
eTime\x22,\x22Absolute\
Timing\x22,\x22Accepta\
nceThreshold\x22,\x22A\
ccountingForm\x22,\x22\
Accumulate\x22,\x22Acc\
uracy\x22,\x22Accuracy\
Goal\x22,\x22AcousticA\
bsorbingValue\x22,\x22\
AcousticImpedanc\
eValue\x22,\x22Acousti\
cNormalVelocityV\
alue\x22,\x22AcousticP\
DEComponent\x22,\x22Ac\
ousticPressureCo\
ndition\x22,\x22Acoust\
icRadiationValue\
\x22,\x22AcousticSound\
HardValue\x22,\x22Acou\
sticSoundSoftCon\
dition\x22,\x22ActionD\
elay\x22,\x22ActionMen\
u\x22,\x22ActionMenuBo\
x\x22,\x22ActionMenuBo\
xOptions\x22,\x22Activ\
ate\x22,\x22Active\x22,\x22A\
ctiveClassificat\
ion\x22,\x22ActiveClas\
sificationObject\
\x22,\x22ActiveItem\x22,\x22\
ActivePrediction\
\x22,\x22ActivePredict\
ionObject\x22,\x22Acti\
veStyle\x22,\x22Acycli\
cGraphQ\x22,\x22AddOnH\
elpPath\x22,\x22AddSid\
es\x22,\x22AddTo\x22,\x22Add\
ToSearchIndex\x22,\x22\
AddUsers\x22,\x22Adjac\
encyGraph\x22,\x22Adja\
cencyList\x22,\x22Adja\
cencyMatrix\x22,\x22Ad\
jacentMeshCells\x22\
,\x22Adjugate\x22,\x22Adj\
ustmentBox\x22,\x22Adj\
ustmentBoxOption\
s\x22,\x22AdjustTimeSe\
riesForecast\x22,\x22A\
dministrativeDiv\
isionData\x22,\x22Affi\
neHalfSpace\x22,\x22Af\
fineSpace\x22,\x22Affi\
neStateSpaceMode\
l\x22,\x22AffineTransf\
orm\x22,\x22After\x22,\x22Ag\
gregatedEntityCl\
ass\x22,\x22Aggregatio\
nLayer\x22,\x22Aircraf\
tData\x22,\x22AirportD\
ata\x22,\x22AirPressur\
eData\x22,\x22AirSound\
Attenuation\x22,\x22Ai\
rTemperatureData\
\x22,\x22AiryAi\x22,\x22Airy\
AiPrime\x22,\x22AiryAi\
Zero\x22,\x22AiryBi\x22,\x22\
AiryBiPrime\x22,\x22Ai\
ryBiZero\x22,\x22Algeb\
raicIntegerQ\x22,\x22A\
lgebraicNumber\x22,\
\x22AlgebraicNumber\
Denominator\x22,\x22Al\
gebraicNumberNor\
m\x22,\x22AlgebraicNum\
berPolynomial\x22,\x22\
AlgebraicNumberT\
race\x22,\x22Algebraic\
Rules\x22,\x22Algebrai\
cRulesData\x22,\x22Alg\
ebraics\x22,\x22Algebr\
aicUnitQ\x22,\x22Align\
ment\x22,\x22Alignment\
Marker\x22,\x22Alignme\
ntPoint\x22,\x22All\x22,\x22\
AllowAdultConten\
t\x22,\x22AllowChatSer\
vices\x22,\x22AllowedC\
loudExtraParamet\
ers\x22,\x22AllowedClo\
udParameterExten\
sions\x22,\x22AllowedD\
imensions\x22,\x22Allo\
wedFrequencyRang\
e\x22,\x22AllowedHeads\
\x22,\x22AllowGroupClo\
se\x22,\x22AllowIncomp\
lete\x22,\x22AllowInli\
neCells\x22,\x22AllowK\
ernelInitializat\
ion\x22,\x22AllowLoose\
Grammar\x22,\x22AllowR\
everseGroupClose\
\x22,\x22AllowScriptLe\
velChange\x22,\x22Allo\
wVersionUpdate\x22,\
\x22AllTrue\x22,\x22Alpha\
bet\x22,\x22Alphabetic\
Order\x22,\x22Alphabet\
icSort\x22,\x22AlphaCh\
annel\x22,\x22Alternat\
eImage\x22,\x22Alterna\
tingFactorial\x22,\x22\
AlternatingGroup\
\x22,\x22AlternativeHy\
pothesis\x22,\x22Alter\
natives\x22,\x22Altitu\
deMethod\x22,\x22Ambie\
ntLight\x22,\x22Ambigu\
ityFunction\x22,\x22Am\
biguityList\x22,\x22An\
alytic\x22,\x22Anatomy\
Data\x22,\x22AnatomyFo\
rm\x22,\x22AnatomyPlot\
3D\x22,\x22AnatomySkin\
Style\x22,\x22AnatomyS\
tyling\x22,\x22Anchore\
dSearch\x22,\x22And\x22,\x22\
AndersonDarlingT\
est\x22,\x22AngerJ\x22,\x22A\
ngleBisector\x22,\x22A\
ngleBracket\x22,\x22An\
glePath\x22,\x22AngleP\
ath3D\x22,\x22AngleVec\
tor\x22,\x22AngularGau\
ge\x22,\x22Animate\x22,\x22A\
nimatedImage\x22,\x22A\
nimationCycleOff\
set\x22,\x22AnimationC\
ycleRepetitions\x22\
,\x22AnimationDirec\
tion\x22,\x22Animation\
DisplayTime\x22,\x22An\
imationRate\x22,\x22An\
imationRepetitio\
ns\x22,\x22AnimationRu\
nning\x22,\x22Animatio\
nRunTime\x22,\x22Anima\
tionTimeIndex\x22,\x22\
AnimationVideo\x22,\
\x22Animator\x22,\x22Anim\
atorBox\x22,\x22Animat\
orBoxOptions\x22,\x22A\
nimatorElements\x22\
,\x22Annotate\x22,\x22Ann\
otation\x22,\x22Annota\
tionDelete\x22,\x22Ann\
otationKeys\x22,\x22An\
notationRules\x22,\x22\
AnnotationValue\x22\
,\x22Annuity\x22,\x22Annu\
ityDue\x22,\x22Annulus\
\x22,\x22AnomalyDetect\
ion\x22,\x22AnomalyDet\
ector\x22,\x22AnomalyD\
etectorFunction\x22\
,\x22Anonymous\x22,\x22An\
tialiasing\x22,\x22Ant\
ihermitian\x22,\x22Ant\
ihermitianMatrix\
Q\x22,\x22Antisymmetri\
c\x22,\x22Antisymmetri\
cMatrixQ\x22,\x22Anton\
yms\x22,\x22AnyOrder\x22,\
\x22AnySubset\x22,\x22Any\
True\x22,\x22Apart\x22,\x22A\
partSquareFree\x22,\
\x22APIFunction\x22,\x22A\
ppearance\x22,\x22Appe\
aranceElements\x22,\
\x22AppearanceRules\
\x22,\x22AppellF1\x22,\x22Ap\
pend\x22,\x22AppendChe\
ck\x22,\x22AppendLayer\
\x22,\x22AppendTo\x22,\x22Ap\
plication\x22,\x22Appl\
y\x22,\x22ApplyReactio\
n\x22,\x22ApplySides\x22,\
\x22ApplyTo\x22,\x22ArcCo\
s\x22,\x22ArcCosh\x22,\x22Ar\
cCot\x22,\x22ArcCoth\x22,\
\x22ArcCsc\x22,\x22ArcCsc\
h\x22,\x22ArcCurvature\
\x22,\x22ARCHProcess\x22,\
\x22ArcLength\x22,\x22Arc\
Sec\x22,\x22ArcSech\x22,\x22\
ArcSin\x22,\x22ArcSinD\
istribution\x22,\x22Ar\
cSinh\x22,\x22ArcTan\x22,\
\x22ArcTanh\x22,\x22Area\x22\
,\x22Arg\x22,\x22ArgMax\x22,\
\x22ArgMin\x22,\x22Argume\
ntCountQ\x22,\x22Argum\
entsOptions\x22,\x22AR\
IMAProcess\x22,\x22Ari\
thmeticGeometric\
Mean\x22,\x22ARMAProce\
ss\x22,\x22Around\x22,\x22Ar\
oundReplace\x22,\x22AR\
Process\x22,\x22Array\x22\
,\x22ArrayComponent\
s\x22,\x22ArrayDepth\x22,\
\x22ArrayFilter\x22,\x22A\
rrayFlatten\x22,\x22Ar\
rayMesh\x22,\x22ArrayP\
ad\x22,\x22ArrayPlot\x22,\
\x22ArrayPlot3D\x22,\x22A\
rrayQ\x22,\x22ArrayRed\
uce\x22,\x22ArrayResam\
ple\x22,\x22ArrayResha\
pe\x22,\x22ArrayRules\x22\
,\x22Arrays\x22,\x22Arrow\
\x22,\x22Arrow3DBox\x22,\x22\
ArrowBox\x22,\x22Arrow\
heads\x22,\x22ASATrian\
gle\x22,\x22Ask\x22,\x22AskA\
ppend\x22,\x22AskConfi\
rm\x22,\x22AskDisplay\x22\
,\x22AskedQ\x22,\x22Asked\
Value\x22,\x22AskFunct\
ion\x22,\x22AskState\x22,\
\x22AskTemplateDisp\
lay\x22,\x22AspectRati\
o\x22,\x22AspectRatioF\
ixed\x22,\x22Assert\x22,\x22\
AssessmentFuncti\
on\x22,\x22AssessmentR\
esultObject\x22,\x22As\
sociateTo\x22,\x22Asso\
ciation\x22,\x22Associ\
ationFormat\x22,\x22As\
sociationMap\x22,\x22A\
ssociationQ\x22,\x22As\
sociationThread\x22\
,\x22AssumeDetermin\
istic\x22,\x22Assuming\
\x22,\x22Assumptions\x22,\
\x22AstroAngularSep\
aration\x22,\x22AstroB\
ackground\x22,\x22Astr\
oCenter\x22,\x22AstroD\
istance\x22,\x22AstroG\
raphics\x22,\x22AstroG\
ridLines\x22,\x22Astro\
GridLinesStyle\x22,\
\x22AstronomicalDat\
a\x22,\x22AstroPositio\
n\x22,\x22AstroProject\
ion\x22,\x22AstroRange\
\x22,\x22AstroRangePad\
ding\x22,\x22AstroRefe\
renceFrame\x22,\x22Ast\
roStyling\x22,\x22Astr\
oZoomLevel\x22,\x22Asy\
mptotic\x22,\x22Asympt\
oticDSolveValue\x22\
,\x22AsymptoticEqua\
l\x22,\x22AsymptoticEq\
uivalent\x22,\x22Asymp\
toticExpectation\
\x22,\x22AsymptoticGre\
ater\x22,\x22Asymptoti\
cGreaterEqual\x22,\x22\
AsymptoticIntegr\
ate\x22,\x22Asymptotic\
Less\x22,\x22Asymptoti\
cLessEqual\x22,\x22Asy\
mptoticOutputTra\
cker\x22,\x22Asymptoti\
cProbability\x22,\x22A\
symptoticProduct\
\x22,\x22AsymptoticRSo\
lveValue\x22,\x22Asymp\
toticSolve\x22,\x22Asy\
mptoticSum\x22,\x22Asy\
nchronous\x22,\x22Asyn\
chronousTaskObje\
ct\x22,\x22Asynchronou\
sTasks\x22,\x22Atom\x22,\x22\
AtomCoordinates\x22\
,\x22AtomCount\x22,\x22At\
omDiagramCoordin\
ates\x22,\x22AtomLabel\
s\x22,\x22AtomLabelSty\
le\x22,\x22AtomList\x22,\x22\
AtomQ\x22,\x22AttachCe\
ll\x22,\x22AttachedCel\
l\x22,\x22AttentionLay\
er\x22,\x22Attributes\x22\
,\x22Audio\x22,\x22AudioA\
mplify\x22,\x22AudioAn\
notate\x22,\x22AudioAn\
notationLookup\x22,\
\x22AudioBlockMap\x22,\
\x22AudioCapture\x22,\x22\
AudioChannelAssi\
gnment\x22,\x22AudioCh\
annelCombine\x22,\x22A\
udioChannelMix\x22,\
\x22AudioChannels\x22,\
\x22AudioChannelSep\
arate\x22,\x22AudioDat\
a\x22,\x22AudioDelay\x22,\
\x22AudioDelete\x22,\x22A\
udioDevice\x22,\x22Aud\
ioDistance\x22,\x22Aud\
ioEncoding\x22,\x22Aud\
ioFade\x22,\x22AudioFr\
equencyShift\x22,\x22A\
udioGenerator\x22,\x22\
AudioIdentify\x22,\x22\
AudioInputDevice\
\x22,\x22AudioInsert\x22,\
\x22AudioInstanceQ\x22\
,\x22AudioIntervals\
\x22,\x22AudioJoin\x22,\x22A\
udioLabel\x22,\x22Audi\
oLength\x22,\x22AudioL\
ocalMeasurements\
\x22,\x22AudioLooping\x22\
,\x22AudioLoudness\x22\
,\x22AudioMeasureme\
nts\x22,\x22AudioNorma\
lize\x22,\x22AudioOutp\
utDevice\x22,\x22Audio\
Overlay\x22,\x22AudioP\
ad\x22,\x22AudioPan\x22,\x22\
AudioPartition\x22,\
\x22AudioPause\x22,\x22Au\
dioPitchShift\x22,\x22\
AudioPlay\x22,\x22Audi\
oPlot\x22,\x22AudioQ\x22,\
\x22AudioRecord\x22,\x22A\
udioReplace\x22,\x22Au\
dioResample\x22,\x22Au\
dioReverb\x22,\x22Audi\
oReverse\x22,\x22Audio\
SampleRate\x22,\x22Aud\
ioSpectralMap\x22,\x22\
AudioSpectralTra\
nsformation\x22,\x22Au\
dioSplit\x22,\x22Audio\
Stop\x22,\x22AudioStre\
am\x22,\x22AudioStream\
s\x22,\x22AudioTimeStr\
etch\x22,\x22AudioTrac\
kApply\x22,\x22AudioTr\
ackSelection\x22,\x22A\
udioTrim\x22,\x22Audio\
Type\x22,\x22Augmented\
Polyhedron\x22,\x22Aug\
mentedSymmetricP\
olynomial\x22,\x22Auth\
enticate\x22,\x22Authe\
ntication\x22,\x22Auth\
enticationDialog\
\x22,\x22AutoAction\x22,\x22\
Autocomplete\x22,\x22A\
utocompletionFun\
ction\x22,\x22AutoCopy\
\x22,\x22Autocorrelati\
onTest\x22,\x22AutoDel\
ete\x22,\x22AutoEvalua\
teEvents\x22,\x22AutoG\
eneratedPackage\x22\
,\x22AutoIndent\x22,\x22A\
utoIndentSpacing\
s\x22,\x22AutoItalicWo\
rds\x22,\x22AutoloadPa\
th\x22,\x22AutoMatch\x22,\
\x22Automatic\x22,\x22Aut\
omaticImageSize\x22\
,\x22AutoMultiplica\
tionSymbol\x22,\x22Aut\
oNumberFormattin\
g\x22,\x22AutoOpenNote\
books\x22,\x22AutoOpen\
Palettes\x22,\x22AutoO\
peratorRendering\
s\x22,\x22AutoQuoteCha\
racters\x22,\x22AutoRe\
freshed\x22,\x22AutoRe\
move\x22,\x22AutorunSe\
quencing\x22,\x22AutoS\
caling\x22,\x22AutoScr\
oll\x22,\x22AutoSpacin\
g\x22,\x22AutoStyleOpt\
ions\x22,\x22AutoStyle\
Words\x22,\x22AutoSubm\
itting\x22,\x22Axes\x22,\x22\
AxesEdge\x22,\x22AxesL\
abel\x22,\x22AxesOrigi\
n\x22,\x22AxesStyle\x22,\x22\
AxiomaticTheory\x22\
,\x22Axis\x22,\x22Axis3DB\
ox\x22,\x22Axis3DBoxOp\
tions\x22,\x22AxisBox\x22\
,\x22AxisBoxOptions\
\x22,\x22AxisLabel\x22,\x22A\
xisObject\x22,\x22Axis\
Style\x22,\x22BabyMons\
terGroupB\x22,\x22Back\
\x22,\x22BackFaceColor\
\x22,\x22BackFaceGlowC\
olor\x22,\x22BackFaceO\
pacity\x22,\x22BackFac\
eSpecularColor\x22,\
\x22BackFaceSpecula\
rExponent\x22,\x22Back\
FaceSurfaceAppea\
rance\x22,\x22BackFace\
Texture\x22,\x22Backgr\
ound\x22,\x22Backgroun\
dAppearance\x22,\x22Ba\
ckgroundTasksSet\
tings\x22,\x22Backslas\
h\x22,\x22Backsubstitu\
tion\x22,\x22Backward\x22\
,\x22Ball\x22,\x22Band\x22,\x22\
BandpassFilter\x22,\
\x22BandstopFilter\x22\
,\x22BarabasiAlbert\
GraphDistributio\
n\x22,\x22BarChart\x22,\x22B\
arChart3D\x22,\x22Barc\
odeImage\x22,\x22Barco\
deRecognize\x22,\x22Ba\
ringhausHenzeTes\
t\x22,\x22BarLegend\x22,\x22\
BarlowProschanIm\
portance\x22,\x22Barne\
sG\x22,\x22BarOrigin\x22,\
\x22BarSpacing\x22,\x22Ba\
rtlettHannWindow\
\x22,\x22BartlettWindo\
w\x22,\x22BaseDecode\x22,\
\x22BaseEncode\x22,\x22Ba\
seForm\x22,\x22Baselin\
e\x22,\x22BaselinePosi\
tion\x22,\x22BaseStyle\
\x22,\x22BasicRecurren\
tLayer\x22,\x22BatchNo\
rmalizationLayer\
\x22,\x22BatchSize\x22,\x22B\
atesDistribution\
\x22,\x22BattleLemarie\
Wavelet\x22,\x22Bayesi\
anMaximization\x22,\
\x22BayesianMaximiz\
ationObject\x22,\x22Ba\
yesianMinimizati\
on\x22,\x22BayesianMin\
imizationObject\x22\
,\x22Because\x22,\x22Beck\
mannDistribution\
\x22,\x22Beep\x22,\x22Before\
\x22,\x22Begin\x22,\x22Begin\
DialogPacket\x22,\x22B\
eginPackage\x22,\x22Be\
llB\x22,\x22BellY\x22,\x22Be\
low\x22,\x22BenfordDis\
tribution\x22,\x22Beni\
niDistribution\x22,\
\x22BenktanderGibra\
tDistribution\x22,\x22\
BenktanderWeibul\
lDistribution\x22,\x22\
BernoulliB\x22,\x22Ber\
noulliDistributi\
on\x22,\x22BernoulliGr\
aphDistribution\x22\
,\x22BernoulliProce\
ss\x22,\x22BernsteinBa\
sis\x22,\x22BesagL\x22,\x22B\
esselFilterModel\
\x22,\x22BesselI\x22,\x22Bes\
selJ\x22,\x22BesselJZe\
ro\x22,\x22BesselK\x22,\x22B\
esselY\x22,\x22BesselY\
Zero\x22,\x22Beta\x22,\x22Be\
taBinomialDistri\
bution\x22,\x22BetaDis\
tribution\x22,\x22Beta\
NegativeBinomial\
Distribution\x22,\x22B\
etaPrimeDistribu\
tion\x22,\x22BetaRegul\
arized\x22,\x22Between\
\x22,\x22BetweennessCe\
ntrality\x22,\x22Bevel\
ed\x22,\x22BeveledPoly\
hedron\x22,\x22BezierC\
urve\x22,\x22BezierCur\
ve3DBox\x22,\x22Bezier\
Curve3DBoxOption\
s\x22,\x22BezierCurveB\
ox\x22,\x22BezierCurve\
BoxOptions\x22,\x22Bez\
ierFunction\x22,\x22Bi\
lateralFilter\x22,\x22\
BilateralLaplace\
Transform\x22,\x22Bila\
teralZTransform\x22\
,\x22Binarize\x22,\x22Bin\
aryDeserialize\x22,\
\x22BinaryDistance\x22\
,\x22BinaryFormat\x22,\
\x22BinaryImageQ\x22,\x22\
BinaryRead\x22,\x22Bin\
aryReadList\x22,\x22Bi\
narySerialize\x22,\x22\
BinaryWrite\x22,\x22Bi\
nCounts\x22,\x22BinLis\
ts\x22,\x22BinnedVario\
gramList\x22,\x22Binom\
ial\x22,\x22BinomialDi\
stribution\x22,\x22Bin\
omialPointProces\
s\x22,\x22BinomialProc\
ess\x22,\x22BinormalDi\
stribution\x22,\x22Bio\
rthogonalSplineW\
avelet\x22,\x22BioSequ\
ence\x22,\x22BioSequen\
ceBackTranslateL\
ist\x22,\x22BioSequenc\
eComplement\x22,\x22Bi\
oSequenceInstanc\
es\x22,\x22BioSequence\
Modify\x22,\x22BioSequ\
encePlot\x22,\x22BioSe\
quenceQ\x22,\x22BioSeq\
uenceReverseComp\
lement\x22,\x22BioSequ\
enceTranscribe\x22,\
\x22BioSequenceTran\
slate\x22,\x22Bipartit\
eGraphQ\x22,\x22Biquad\
raticFilterModel\
\x22,\x22BirnbaumImpor\
tance\x22,\x22Birnbaum\
SaundersDistribu\
tion\x22,\x22BitAnd\x22,\x22\
BitClear\x22,\x22BitGe\
t\x22,\x22BitLength\x22,\x22\
BitNot\x22,\x22BitOr\x22,\
\x22BitRate\x22,\x22BitSe\
t\x22,\x22BitShiftLeft\
\x22,\x22BitShiftRight\
\x22,\x22BitXor\x22,\x22Biwe\
ightLocation\x22,\x22B\
iweightMidvarian\
ce\x22,\x22Black\x22,\x22Bla\
ckmanHarrisWindo\
w\x22,\x22BlackmanNutt\
allWindow\x22,\x22Blac\
kmanWindow\x22,\x22Bla\
nk\x22,\x22BlankForm\x22,\
\x22BlankNullSequen\
ce\x22,\x22BlankSequen\
ce\x22,\x22Blend\x22,\x22Blo\
ck\x22,\x22BlockchainA\
ddressData\x22,\x22Blo\
ckchainBase\x22,\x22Bl\
ockchainBlockDat\
a\x22,\x22BlockchainCo\
ntractValue\x22,\x22Bl\
ockchainData\x22,\x22B\
lockchainGet\x22,\x22B\
lockchainKeyEnco\
de\x22,\x22BlockchainP\
ut\x22,\x22BlockchainT\
okenData\x22,\x22Block\
chainTransaction\
\x22,\x22BlockchainTra\
nsactionData\x22,\x22B\
lockchainTransac\
tionSign\x22,\x22Block\
chainTransaction\
Submit\x22,\x22BlockDi\
agonalMatrix\x22,\x22B\
lockLowerTriangu\
larMatrix\x22,\x22Bloc\
kMap\x22,\x22BlockRand\
om\x22,\x22BlockUpperT\
riangularMatrix\x22\
,\x22BlomqvistBeta\x22\
,\x22BlomqvistBetaT\
est\x22,\x22Blue\x22,\x22Blu\
r\x22,\x22Blurring\x22,\x22B\
odePlot\x22,\x22Bohman\
Window\x22,\x22Bold\x22,\x22\
Bond\x22,\x22BondCount\
\x22,\x22BondLabels\x22,\x22\
BondLabelStyle\x22,\
\x22BondList\x22,\x22Bond\
Q\x22,\x22Bookmarks\x22,\x22\
Boole\x22,\x22BooleanC\
onsecutiveFuncti\
on\x22,\x22BooleanConv\
ert\x22,\x22BooleanCou\
ntingFunction\x22,\x22\
BooleanFunction\x22\
,\x22BooleanGraph\x22,\
\x22BooleanMaxterms\
\x22,\x22BooleanMinimi\
ze\x22,\x22BooleanMint\
erms\x22,\x22BooleanQ\x22\
,\x22BooleanRegion\x22\
,\x22Booleans\x22,\x22Boo\
leanStrings\x22,\x22Bo\
oleanTable\x22,\x22Boo\
leanVariables\x22,\x22\
BorderDimensions\
\x22,\x22BorelTannerDi\
stribution\x22,\x22Bot\
tom\x22,\x22BottomHatT\
ransform\x22,\x22Bound\
aryDiscretizeGra\
phics\x22,\x22Boundary\
DiscretizeRegion\
\x22,\x22BoundaryMesh\x22\
,\x22BoundaryMeshRe\
gion\x22,\x22BoundaryM\
eshRegionQ\x22,\x22Bou\
ndaryStyle\x22,\x22Bou\
ndedRegionQ\x22,\x22Bo\
undingRegion\x22,\x22B\
ounds\x22,\x22Box\x22,\x22Bo\
xBaselineShift\x22,\
\x22BoxData\x22,\x22BoxDi\
mensions\x22,\x22Boxed\
\x22,\x22Boxes\x22,\x22BoxFo\
rm\x22,\x22BoxFormForm\
atTypes\x22,\x22BoxFra\
me\x22,\x22BoxID\x22,\x22Box\
Margins\x22,\x22BoxMat\
rix\x22,\x22BoxObject\x22\
,\x22BoxRatios\x22,\x22Bo\
xRotation\x22,\x22BoxR\
otationPoint\x22,\x22B\
oxStyle\x22,\x22BoxWhi\
skerChart\x22,\x22Bra\x22\
,\x22BracketingBar\x22\
,\x22BraKet\x22,\x22BrayC\
urtisDistance\x22,\x22\
BreadthFirstScan\
\x22,\x22Break\x22,\x22Bridg\
eData\x22,\x22Brightne\
ssEqualize\x22,\x22Bro\
adcastStationDat\
a\x22,\x22Brown\x22,\x22Brow\
nForsytheTest\x22,\x22\
BrownianBridgePr\
ocess\x22,\x22BrowserC\
ategory\x22,\x22BSplin\
eBasis\x22,\x22BSpline\
Curve\x22,\x22BSplineC\
urve3DBox\x22,\x22BSpl\
ineCurve3DBoxOpt\
ions\x22,\x22BSplineCu\
rveBox\x22,\x22BSpline\
CurveBoxOptions\x22\
,\x22BSplineFunctio\
n\x22,\x22BSplineSurfa\
ce\x22,\x22BSplineSurf\
ace3DBox\x22,\x22BSpli\
neSurface3DBoxOp\
tions\x22,\x22BubbleCh\
art\x22,\x22BubbleChar\
t3D\x22,\x22BubbleScal\
e\x22,\x22BubbleSizes\x22\
,\x22BuckyballGraph\
\x22,\x22BuildCompiled\
Component\x22,\x22Buil\
dingData\x22,\x22Bulle\
tGauge\x22,\x22Busines\
sDayQ\x22,\x22Butterfl\
yGraph\x22,\x22Butterw\
orthFilterModel\x22\
,\x22Button\x22,\x22Butto\
nBar\x22,\x22ButtonBox\
\x22,\x22ButtonBoxOpti\
ons\x22,\x22ButtonCell\
\x22,\x22ButtonContent\
s\x22,\x22ButtonData\x22,\
\x22ButtonEvaluator\
\x22,\x22ButtonExpanda\
ble\x22,\x22ButtonFram\
e\x22,\x22ButtonFuncti\
on\x22,\x22ButtonMargi\
ns\x22,\x22ButtonMinHe\
ight\x22,\x22ButtonNot\
e\x22,\x22ButtonNotebo\
ok\x22,\x22ButtonSourc\
e\x22,\x22ButtonStyle\x22\
,\x22ButtonStyleMen\
uListing\x22,\x22Byte\x22\
,\x22ByteArray\x22,\x22By\
teArrayFormat\x22,\x22\
ByteArrayFormatQ\
\x22,\x22ByteArrayQ\x22,\x22\
ByteArrayToStrin\
g\x22,\x22ByteCount\x22,\x22\
ByteOrdering\x22,\x22C\
\x22,\x22CachedValue\x22,\
\x22CacheGraphics\x22,\
\x22CachePersistenc\
e\x22,\x22CalendarConv\
ert\x22,\x22CalendarDa\
ta\x22,\x22CalendarTyp\
e\x22,\x22Callout\x22,\x22Ca\
lloutMarker\x22,\x22Ca\
lloutStyle\x22,\x22Cal\
lPacket\x22,\x22Canber\
raDistance\x22,\x22Can\
cel\x22,\x22CancelButt\
on\x22,\x22Candlestick\
Chart\x22,\x22Canonica\
lGraph\x22,\x22Canonic\
alizePolygon\x22,\x22C\
anonicalizePolyh\
edron\x22,\x22Canonica\
lizeRegion\x22,\x22Can\
onicalName\x22,\x22Can\
onicalWarpingCor\
respondence\x22,\x22Ca\
nonicalWarpingDi\
stance\x22,\x22CantorM\
esh\x22,\x22CantorStai\
rcase\x22,\x22Canvas\x22,\
\x22Cap\x22,\x22CapForm\x22,\
\x22CapitalDifferen\
tialD\x22,\x22Capitali\
ze\x22,\x22CapsuleShap\
e\x22,\x22CaptureRunni\
ng\x22,\x22CaputoD\x22,\x22C\
ardinalBSplineBa\
sis\x22,\x22CarlemanLi\
nearize\x22,\x22Carlso\
nRC\x22,\x22CarlsonRD\x22\
,\x22CarlsonRE\x22,\x22Ca\
rlsonRF\x22,\x22Carlso\
nRG\x22,\x22CarlsonRJ\x22\
,\x22CarlsonRK\x22,\x22Ca\
rlsonRM\x22,\x22Carmic\
haelLambda\x22,\x22Cas\
eOrdering\x22,\x22Case\
s\x22,\x22CaseSensitiv\
e\x22,\x22Cashflow\x22,\x22C\
asoratian\x22,\x22Cast\
\x22,\x22Catalan\x22,\x22Cat\
alanNumber\x22,\x22Cat\
ch\x22,\x22Categorical\
Distribution\x22,\x22C\
atenate\x22,\x22Catena\
teLayer\x22,\x22Cauchy\
Distribution\x22,\x22C\
auchyMatrix\x22,\x22Ca\
uchyPointProcess\
\x22,\x22CauchyWindow\x22\
,\x22CayleyGraph\x22,\x22\
CDF\x22,\x22CDFDeploy\x22\
,\x22CDFInformation\
\x22,\x22CDFWavelet\x22,\x22\
Ceiling\x22,\x22Celest\
ialSystem\x22,\x22Cell\
\x22,\x22CellAutoOverw\
rite\x22,\x22CellBasel\
ine\x22,\x22CellBoundi\
ngBox\x22,\x22CellBrac\
ketOptions\x22,\x22Cel\
lChangeTimes\x22,\x22C\
ellContents\x22,\x22Ce\
llContext\x22,\x22Cell\
Dingbat\x22,\x22CellDi\
ngbatMargin\x22,\x22Ce\
llDynamicExpress\
ion\x22,\x22CellEditDu\
plicate\x22,\x22CellEl\
ementsBoundingBo\
x\x22,\x22CellElementS\
pacings\x22,\x22CellEp\
ilog\x22,\x22CellEvalu\
ationDuplicate\x22,\
\x22CellEvaluationF\
unction\x22,\x22CellEv\
aluationLanguage\
\x22,\x22CellEventActi\
ons\x22,\x22CellFrame\x22\
,\x22CellFrameColor\
\x22,\x22CellFrameLabe\
lMargins\x22,\x22CellF\
rameLabels\x22,\x22Cel\
lFrameMargins\x22,\x22\
CellFrameStyle\x22,\
\x22CellGroup\x22,\x22Cel\
lGroupData\x22,\x22Cel\
lGrouping\x22,\x22Cell\
GroupingRules\x22,\x22\
CellHorizontalSc\
rolling\x22,\x22CellID\
\x22,\x22CellInsertion\
PointCell\x22,\x22Cell\
Label\x22,\x22CellLabe\
lAutoDelete\x22,\x22Ce\
llLabelMargins\x22,\
\x22CellLabelPositi\
oning\x22,\x22CellLabe\
lStyle\x22,\x22CellLab\
elTemplate\x22,\x22Cel\
lMargins\x22,\x22CellO\
bject\x22,\x22CellOpen\
\x22,\x22CellPrint\x22,\x22C\
ellProlog\x22,\x22Cell\
s\x22,\x22CellSize\x22,\x22C\
ellStyle\x22,\x22CellT\
ags\x22,\x22CellTrayPo\
sition\x22,\x22CellTra\
yWidgets\x22,\x22Cellu\
larAutomaton\x22,\x22C\
ensoredDistribut\
ion\x22,\x22Censoring\x22\
,\x22Center\x22,\x22Cente\
rArray\x22,\x22CenterD\
ot\x22,\x22CenteredInt\
erval\x22,\x22CentralF\
eature\x22,\x22Central\
Moment\x22,\x22Central\
MomentGenerating\
Function\x22,\x22Cepst\
rogram\x22,\x22Cepstro\
gramArray\x22,\x22Ceps\
trumArray\x22,\x22CFor\
m\x22,\x22Champernowne\
Number\x22,\x22ChangeO\
ptions\x22,\x22Channel\
Base\x22,\x22ChannelBr\
okerAction\x22,\x22Cha\
nnelDatabin\x22,\x22Ch\
annelHistoryLeng\
th\x22,\x22ChannelList\
en\x22,\x22ChannelList\
ener\x22,\x22ChannelLi\
steners\x22,\x22Channe\
lListenerWait\x22,\x22\
ChannelObject\x22,\x22\
ChannelPreSendFu\
nction\x22,\x22Channel\
ReceiverFunction\
\x22,\x22ChannelSend\x22,\
\x22ChannelSubscrib\
ers\x22,\x22ChanVeseBi\
narize\x22,\x22Charact\
er\x22,\x22CharacterCo\
unts\x22,\x22Character\
Encoding\x22,\x22Chara\
cterEncodingsPat\
h\x22,\x22Characterist\
icFunction\x22,\x22Cha\
racteristicPolyn\
omial\x22,\x22Characte\
rName\x22,\x22Characte\
rNormalize\x22,\x22Cha\
racterRange\x22,\x22Ch\
aracters\x22,\x22Chart\
BaseStyle\x22,\x22Char\
tElementData\x22,\x22C\
hartElementDataF\
unction\x22,\x22ChartE\
lementFunction\x22,\
\x22ChartElements\x22,\
\x22ChartLabels\x22,\x22C\
hartLayout\x22,\x22Cha\
rtLegends\x22,\x22Char\
tStyle\x22,\x22Chebysh\
ev1FilterModel\x22,\
\x22Chebyshev2Filte\
rModel\x22,\x22Chebysh\
evDistance\x22,\x22Che\
byshevT\x22,\x22Chebys\
hevU\x22,\x22Check\x22,\x22C\
heckAbort\x22,\x22Chec\
kAll\x22,\x22CheckArgu\
ments\x22,\x22Checkbox\
\x22,\x22CheckboxBar\x22,\
\x22CheckboxBox\x22,\x22C\
heckboxBoxOption\
s\x22,\x22ChemicalConv\
ert\x22,\x22ChemicalDa\
ta\x22,\x22ChemicalFor\
mula\x22,\x22ChemicalI\
nstance\x22,\x22Chemic\
alReaction\x22,\x22Che\
ssboardDistance\x22\
,\x22ChiDistributio\
n\x22,\x22ChineseRemai\
nder\x22,\x22ChiSquare\
Distribution\x22,\x22C\
hoiceButtons\x22,\x22C\
hoiceDialog\x22,\x22Ch\
oleskyDecomposit\
ion\x22,\x22Chop\x22,\x22Chr\
omaticityPlot\x22,\x22\
ChromaticityPlot\
3D\x22,\x22ChromaticPo\
lynomial\x22,\x22Circl\
e\x22,\x22CircleBox\x22,\x22\
CircleDot\x22,\x22Circ\
leMinus\x22,\x22Circle\
Plus\x22,\x22CirclePoi\
nts\x22,\x22CircleThro\
ugh\x22,\x22CircleTime\
s\x22,\x22CirculantGra\
ph\x22,\x22CircularArc\
Through\x22,\x22Circul\
arOrthogonalMatr\
ixDistribution\x22,\
\x22CircularQuatern\
ionMatrixDistrib\
ution\x22,\x22Circular\
RealMatrixDistri\
bution\x22,\x22Circula\
rSymplecticMatri\
xDistribution\x22,\x22\
CircularUnitaryM\
atrixDistributio\
n\x22,\x22Circumsphere\
\x22,\x22CityData\x22,\x22Cl\
assifierFunction\
\x22,\x22ClassifierInf\
ormation\x22,\x22Class\
ifierMeasurement\
s\x22,\x22ClassifierMe\
asurementsObject\
\x22,\x22Classify\x22,\x22Cl\
assPriors\x22,\x22Clea\
r\x22,\x22ClearAll\x22,\x22C\
learAttributes\x22,\
\x22ClearCookies\x22,\x22\
ClearPermissions\
\x22,\x22ClearSystemCa\
che\x22,\x22ClebschGor\
dan\x22,\x22ClickPane\x22\
,\x22ClickToCopy\x22,\x22\
ClickToCopyEnabl\
ed\x22,\x22Clip\x22,\x22Clip\
boardNotebook\x22,\x22\
ClipFill\x22,\x22Clipp\
ingStyle\x22,\x22ClipP\
lanes\x22,\x22ClipPlan\
esStyle\x22,\x22ClipRa\
nge\x22,\x22Clock\x22,\x22Cl\
ockGauge\x22,\x22Clock\
wiseContourInteg\
ral\x22,\x22Close\x22,\x22Cl\
osed\x22,\x22CloseKern\
els\x22,\x22ClosenessC\
entrality\x22,\x22Clos\
ing\x22,\x22ClosingAut\
oSave\x22,\x22ClosingE\
vent\x22,\x22CloudAcco\
untData\x22,\x22CloudB\
ase\x22,\x22CloudConne\
ct\x22,\x22CloudConnec\
tions\x22,\x22CloudDep\
loy\x22,\x22CloudDirec\
tory\x22,\x22CloudDisc\
onnect\x22,\x22CloudEv\
aluate\x22,\x22CloudEx\
port\x22,\x22CloudExpr\
ession\x22,\x22CloudEx\
pressions\x22,\x22Clou\
dFunction\x22,\x22Clou\
dGet\x22,\x22CloudImpo\
rt\x22,\x22CloudLoggin\
gData\x22,\x22CloudObj\
ect\x22,\x22CloudObjec\
tInformation\x22,\x22C\
loudObjectInform\
ationData\x22,\x22Clou\
dObjectNameForma\
t\x22,\x22CloudObjects\
\x22,\x22CloudObjectUR\
LType\x22,\x22CloudPub\
lish\x22,\x22CloudPut\x22\
,\x22CloudRendering\
Method\x22,\x22CloudSa\
ve\x22,\x22CloudShare\x22\
,\x22CloudSubmit\x22,\x22\
CloudSymbol\x22,\x22Cl\
oudUnshare\x22,\x22Clo\
udUserID\x22,\x22Clust\
erClassify\x22,\x22Clu\
sterDissimilarit\
yFunction\x22,\x22Clus\
teringComponents\
\x22,\x22ClusteringMea\
surements\x22,\x22Clus\
teringTree\x22,\x22CMY\
KColor\x22,\x22Coarse\x22\
,\x22CodeAssistOpti\
ons\x22,\x22Coefficien\
t\x22,\x22CoefficientA\
rrays\x22,\x22Coeffici\
entDomain\x22,\x22Coef\
ficientList\x22,\x22Co\
efficientRules\x22,\
\x22CoifletWavelet\x22\
,\x22Collect\x22,\x22Coll\
inearPoints\x22,\x22Co\
lon\x22,\x22ColonForm\x22\
,\x22ColorBalance\x22,\
\x22ColorCombine\x22,\x22\
ColorConvert\x22,\x22C\
olorCoverage\x22,\x22C\
olorData\x22,\x22Color\
DataFunction\x22,\x22C\
olorDetect\x22,\x22Col\
orDistance\x22,\x22Col\
orFunction\x22,\x22Col\
orFunctionBinnin\
g\x22,\x22ColorFunctio\
nScaling\x22,\x22Color\
ize\x22,\x22ColorNegat\
e\x22,\x22ColorOutput\x22\
,\x22ColorProfileDa\
ta\x22,\x22ColorQ\x22,\x22Co\
lorQuantize\x22,\x22Co\
lorReplace\x22,\x22Col\
orRules\x22,\x22ColorS\
electorSettings\x22\
,\x22ColorSeparate\x22\
,\x22ColorSetter\x22,\x22\
ColorSetterBox\x22,\
\x22ColorSetterBoxO\
ptions\x22,\x22ColorSl\
ider\x22,\x22ColorsNea\
r\x22,\x22ColorSpace\x22,\
\x22ColorToneMappin\
g\x22,\x22Column\x22,\x22Col\
umnAlignments\x22,\x22\
ColumnBackground\
s\x22,\x22ColumnForm\x22,\
\x22ColumnLines\x22,\x22C\
olumnsEqual\x22,\x22Co\
lumnSpacings\x22,\x22C\
olumnWidths\x22,\x22Co\
mbinatorB\x22,\x22Comb\
inatorC\x22,\x22Combin\
atorI\x22,\x22Combinat\
orK\x22,\x22Combinator\
S\x22,\x22CombinatorW\x22\
,\x22CombinatorY\x22,\x22\
CombinedEntityCl\
ass\x22,\x22CombinerFu\
nction\x22,\x22CometDa\
ta\x22,\x22CommonDefau\
ltFormatTypes\x22,\x22\
Commonest\x22,\x22Comm\
onestFilter\x22,\x22Co\
mmonName\x22,\x22Commo\
nUnits\x22,\x22Communi\
tyBoundaryStyle\x22\
,\x22CommunityGraph\
Plot\x22,\x22Community\
Labels\x22,\x22Communi\
tyRegionStyle\x22,\x22\
CompanyData\x22,\x22Co\
mpatibleUnitQ\x22,\x22\
CompilationOptio\
ns\x22,\x22Compilation\
Target\x22,\x22Compile\
\x22,\x22Compiled\x22,\x22Co\
mpiledCodeFuncti\
on\x22,\x22CompiledCom\
ponent\x22,\x22Compile\
dExpressionDecla\
ration\x22,\x22Compile\
dFunction\x22,\x22Comp\
iledLayer\x22,\x22Comp\
ilerCallback\x22,\x22C\
ompilerEnvironme\
nt\x22,\x22CompilerEnv\
ironmentAppend\x22,\
\x22CompilerEnviron\
mentAppendTo\x22,\x22C\
ompilerEnvironme\
ntObject\x22,\x22Compi\
lerOptions\x22,\x22Com\
plement\x22,\x22Comple\
mentedEntityClas\
s\x22,\x22CompleteGrap\
h\x22,\x22CompleteGrap\
hQ\x22,\x22CompleteInt\
egral\x22,\x22Complete\
KaryTree\x22,\x22Compl\
etionsListPacket\
\x22,\x22Complex\x22,\x22Com\
plexArrayPlot\x22,\x22\
ComplexContourPl\
ot\x22,\x22Complexes\x22,\
\x22ComplexExpand\x22,\
\x22ComplexInfinity\
\x22,\x22ComplexityFun\
ction\x22,\x22ComplexL\
istPlot\x22,\x22Comple\
xPlot\x22,\x22ComplexP\
lot3D\x22,\x22ComplexR\
egionPlot\x22,\x22Comp\
lexStreamPlot\x22,\x22\
ComplexVectorPlo\
t\x22,\x22ComponentMea\
surements\x22,\x22Comp\
onentwiseContext\
Menu\x22,\x22Compose\x22,\
\x22ComposeList\x22,\x22C\
omposeSeries\x22,\x22C\
ompositeQ\x22,\x22Comp\
osition\x22,\x22Compou\
ndElement\x22,\x22Comp\
oundExpression\x22,\
\x22CompoundPoisson\
Distribution\x22,\x22C\
ompoundPoissonPr\
ocess\x22,\x22Compound\
RenewalProcess\x22,\
\x22Compress\x22,\x22Comp\
ressedData\x22,\x22Com\
pressionLevel\x22,\x22\
ComputeUncertain\
ty\x22,\x22ConcaveHull\
Mesh\x22,\x22Condition\
\x22,\x22ConditionalEx\
pression\x22,\x22Condi\
tioned\x22,\x22Cone\x22,\x22\
ConeBox\x22,\x22Confid\
enceLevel\x22,\x22Conf\
idenceRange\x22,\x22Co\
nfidenceTransfor\
m\x22,\x22Configuratio\
nPath\x22,\x22Confirm\x22\
,\x22ConfirmAssert\x22\
,\x22ConfirmBy\x22,\x22Co\
nfirmMatch\x22,\x22Con\
firmQuiet\x22,\x22Conf\
ormationMethod\x22,\
\x22ConformAudio\x22,\x22\
ConformImages\x22,\x22\
Congruent\x22,\x22Coni\
cGradientFilling\
\x22,\x22ConicHullRegi\
on\x22,\x22ConicHullRe\
gion3DBox\x22,\x22Coni\
cHullRegion3DBox\
Options\x22,\x22ConicH\
ullRegionBox\x22,\x22C\
onicHullRegionBo\
xOptions\x22,\x22Conic\
Optimization\x22,\x22C\
onjugate\x22,\x22Conju\
gateTranspose\x22,\x22\
Conjunction\x22,\x22Co\
nnect\x22,\x22Connecte\
dComponents\x22,\x22Co\
nnectedGraphComp\
onents\x22,\x22Connect\
edGraphQ\x22,\x22Conne\
ctedMeshComponen\
ts\x22,\x22ConnectedMo\
leculeComponents\
\x22,\x22ConnectedMole\
culeQ\x22,\x22Connecti\
onSettings\x22,\x22Con\
nectLibraryCallb\
ackFunction\x22,\x22Co\
nnectSystemModel\
Components\x22,\x22Con\
nectSystemModelC\
ontroller\x22,\x22Conn\
esWindow\x22,\x22Conov\
erTest\x22,\x22Conserv\
ativeConvectionP\
DETerm\x22,\x22Console\
Message\x22,\x22Consta\
nt\x22,\x22ConstantArr\
ay\x22,\x22ConstantArr\
ayLayer\x22,\x22Consta\
ntImage\x22,\x22Consta\
ntPlusLayer\x22,\x22Co\
nstantRegionQ\x22,\x22\
Constants\x22,\x22Cons\
tantTimesLayer\x22,\
\x22ConstellationDa\
ta\x22,\x22Constrained\
Max\x22,\x22Constraine\
dMin\x22,\x22Construct\
\x22,\x22Containing\x22,\x22\
ContainsAll\x22,\x22Co\
ntainsAny\x22,\x22Cont\
ainsExactly\x22,\x22Co\
ntainsNone\x22,\x22Con\
tainsOnly\x22,\x22Cont\
entDetectorFunct\
ion\x22,\x22ContentFie\
ldOptions\x22,\x22Cont\
entLocationFunct\
ion\x22,\x22ContentObj\
ect\x22,\x22ContentPad\
ding\x22,\x22ContentsB\
oundingBox\x22,\x22Con\
tentSelectable\x22,\
\x22ContentSize\x22,\x22C\
ontext\x22,\x22Context\
Menu\x22,\x22Contexts\x22\
,\x22ContextToFileN\
ame\x22,\x22Continuati\
on\x22,\x22Continue\x22,\x22\
ContinuedFractio\
n\x22,\x22ContinuedFra\
ctionK\x22,\x22Continu\
ousAction\x22,\x22Cont\
inuousMarkovProc\
ess\x22,\x22Continuous\
Task\x22,\x22Continuou\
sTimeModelQ\x22,\x22Co\
ntinuousWaveletD\
ata\x22,\x22Continuous\
WaveletTransform\
\x22,\x22ContourDetect\
\x22,\x22ContourGraphi\
cs\x22,\x22ContourInte\
gral\x22,\x22ContourLa\
bels\x22,\x22ContourLi\
nes\x22,\x22ContourPlo\
t\x22,\x22ContourPlot3\
D\x22,\x22Contours\x22,\x22C\
ontourShading\x22,\x22\
ContourSmoothing\
\x22,\x22ContourStyle\x22\
,\x22Contraharmonic\
Mean\x22,\x22Contrasti\
veLossLayer\x22,\x22Co\
ntrol\x22,\x22ControlA\
ctive\x22,\x22ControlA\
lignment\x22,\x22Contr\
olGroupContentsB\
ox\x22,\x22Controllabi\
lityGramian\x22,\x22Co\
ntrollabilityMat\
rix\x22,\x22Controllab\
leDecomposition\x22\
,\x22ControllableMo\
delQ\x22,\x22Controlle\
rDuration\x22,\x22Cont\
rollerInformatio\
n\x22,\x22ControllerIn\
formationData\x22,\x22\
ControllerLinkin\
g\x22,\x22ControllerMa\
nipulate\x22,\x22Contr\
ollerMethod\x22,\x22Co\
ntrollerPath\x22,\x22C\
ontrollerState\x22,\
\x22ControlPlacemen\
t\x22,\x22ControlsRend\
ering\x22,\x22ControlT\
ype\x22,\x22Convection\
PDETerm\x22,\x22Conver\
gents\x22,\x22Conversi\
onOptions\x22,\x22Conv\
ersionRules\x22,\x22Co\
nvertToPostScrip\
t\x22,\x22ConvertToPos\
tScriptPacket\x22,\x22\
ConvexHullMesh\x22,\
\x22ConvexHullRegio\
n\x22,\x22ConvexOptimi\
zation\x22,\x22ConvexP\
olygonQ\x22,\x22Convex\
PolyhedronQ\x22,\x22Co\
nvexRegionQ\x22,\x22Co\
nvolutionLayer\x22,\
\x22Convolve\x22,\x22Conw\
ayGroupCo1\x22,\x22Con\
wayGroupCo2\x22,\x22Co\
nwayGroupCo3\x22,\x22C\
ookieFunction\x22,\x22\
Cookies\x22,\x22Coordi\
nateBoundingBox\x22\
,\x22CoordinateBoun\
dingBoxArray\x22,\x22C\
oordinateBounds\x22\
,\x22CoordinateBoun\
dsArray\x22,\x22Coordi\
nateChartData\x22,\x22\
CoordinatesToolO\
ptions\x22,\x22Coordin\
ateTransform\x22,\x22C\
oordinateTransfo\
rmData\x22,\x22Coplana\
rPoints\x22,\x22Coprim\
eQ\x22,\x22Coproduct\x22,\
\x22CopulaDistribut\
ion\x22,\x22Copyable\x22,\
\x22CopyDatabin\x22,\x22C\
opyDirectory\x22,\x22C\
opyFile\x22,\x22CopyFu\
nction\x22,\x22CopyTag\
\x22,\x22CopyToClipboa\
rd\x22,\x22CoreNilpote\
ntDecomposition\x22\
,\x22CornerFilter\x22,\
\x22CornerNeighbors\
\x22,\x22Correlation\x22,\
\x22CorrelationDist\
ance\x22,\x22Correlati\
onFunction\x22,\x22Cor\
relationTest\x22,\x22C\
os\x22,\x22Cosh\x22,\x22Cosh\
Integral\x22,\x22Cosin\
eDistance\x22,\x22Cosi\
neWindow\x22,\x22CosIn\
tegral\x22,\x22Cot\x22,\x22C\
oth\x22,\x22CoulombF\x22,\
\x22CoulombG\x22,\x22Coul\
ombH1\x22,\x22CoulombH\
2\x22,\x22Count\x22,\x22Coun\
tDistinct\x22,\x22Coun\
tDistinctBy\x22,\x22Co\
unterAssignments\
\x22,\x22CounterBox\x22,\x22\
CounterBoxOption\
s\x22,\x22CounterClock\
wiseContourInteg\
ral\x22,\x22CounterEva\
luator\x22,\x22Counter\
Function\x22,\x22Count\
erIncrements\x22,\x22C\
ounterStyle\x22,\x22Co\
unterStyleMenuLi\
sting\x22,\x22CountRoo\
ts\x22,\x22CountryData\
\x22,\x22Counts\x22,\x22Coun\
tsBy\x22,\x22Covarianc\
e\x22,\x22CovarianceEs\
timatorFunction\x22\
,\x22CovarianceFunc\
tion\x22,\x22CoxianDis\
tribution\x22,\x22CoxI\
ngersollRossProc\
ess\x22,\x22CoxModel\x22,\
\x22CoxModelFit\x22,\x22C\
ramerVonMisesTes\
t\x22,\x22CreateArchiv\
e\x22,\x22CreateCellID\
\x22,\x22CreateChannel\
\x22,\x22CreateCloudEx\
pression\x22,\x22Creat\
eCompilerEnviron\
ment\x22,\x22CreateDat\
abin\x22,\x22CreateDat\
aStructure\x22,\x22Cre\
ateDataSystemMod\
el\x22,\x22CreateDialo\
g\x22,\x22CreateDirect\
ory\x22,\x22CreateDocu\
ment\x22,\x22CreateFil\
e\x22,\x22CreateInterm\
ediateDirectorie\
s\x22,\x22CreateLicens\
eEntitlement\x22,\x22C\
reateManagedLibr\
aryExpression\x22,\x22\
CreateNotebook\x22,\
\x22CreatePacletArc\
hive\x22,\x22CreatePal\
ette\x22,\x22CreatePer\
missionsGroup\x22,\x22\
CreateScheduledT\
ask\x22,\x22CreateSear\
chIndex\x22,\x22Create\
SystemModel\x22,\x22Cr\
eateTemporary\x22,\x22\
CreateTypeInstan\
ce\x22,\x22CreateUUID\x22\
,\x22CreateWindow\x22,\
\x22CriterionFuncti\
on\x22,\x22Criticality\
FailureImportanc\
e\x22,\x22CriticalityS\
uccessImportance\
\x22,\x22CriticalSecti\
on\x22,\x22Cross\x22,\x22Cro\
ssEntropyLossLay\
er\x22,\x22CrossingCou\
nt\x22,\x22CrossingDet\
ect\x22,\x22CrossingPo\
lygon\x22,\x22CrossMat\
rix\x22,\x22Csc\x22,\x22Csch\
\x22,\x22CSGRegion\x22,\x22C\
SGRegionQ\x22,\x22CSGR\
egionTree\x22,\x22CTCL\
ossLayer\x22,\x22Cube\x22\
,\x22CubeRoot\x22,\x22Cub\
ics\x22,\x22Cuboid\x22,\x22C\
uboidBox\x22,\x22Cuboi\
dBoxOptions\x22,\x22Cu\
mulant\x22,\x22Cumulan\
tGeneratingFunct\
ion\x22,\x22Cumulative\
FeatureImpactPlo\
t\x22,\x22Cup\x22,\x22CupCap\
\x22,\x22Curl\x22,\x22CurlyD\
oubleQuote\x22,\x22Cur\
lyQuote\x22,\x22Curren\
cyConvert\x22,\x22Curr\
entDate\x22,\x22Curren\
tImage\x22,\x22Current\
NotebookImage\x22,\x22\
CurrentScreenIma\
ge\x22,\x22CurrentValu\
e\x22,\x22Curry\x22,\x22Curr\
yApplied\x22,\x22Curva\
tureFlowFilter\x22,\
\x22CurveClosed\x22,\x22C\
yan\x22,\x22CycleGraph\
\x22,\x22CycleIndexPol\
ynomial\x22,\x22Cycles\
\x22,\x22CyclicGroup\x22,\
\x22Cyclotomic\x22,\x22Cy\
linder\x22,\x22Cylinde\
rBox\x22,\x22CylinderB\
oxOptions\x22,\x22Cyli\
ndricalDecomposi\
tion\x22,\x22Cylindric\
alDecompositionF\
unction\x22,\x22D\x22,\x22Da\
gumDistribution\x22\
,\x22DamData\x22,\x22Dame\
rauLevenshteinDi\
stance\x22,\x22Damping\
Factor\x22,\x22Darker\x22\
,\x22Dashed\x22,\x22Dashi\
ng\x22,\x22DatabaseCon\
nect\x22,\x22DatabaseD\
isconnect\x22,\x22Data\
baseReference\x22,\x22\
Databin\x22,\x22Databi\
nAdd\x22,\x22DatabinRe\
move\x22,\x22Databins\x22\
,\x22DatabinSubmit\x22\
,\x22DatabinUpload\x22\
,\x22DataCompressio\
n\x22,\x22DataDistribu\
tion\x22,\x22DataRange\
\x22,\x22DataReversed\x22\
,\x22Dataset\x22,\x22Data\
setDisplayPanel\x22\
,\x22DatasetTheme\x22,\
\x22DataStructure\x22,\
\x22DataStructureQ\x22\
,\x22Date\x22,\x22DateBou\
nds\x22,\x22Dated\x22,\x22Da\
teDelimiters\x22,\x22D\
ateDifference\x22,\x22\
DatedUnit\x22,\x22Date\
Format\x22,\x22DateFun\
ction\x22,\x22DateGran\
ularity\x22,\x22DateHi\
stogram\x22,\x22DateIn\
terval\x22,\x22DateLis\
t\x22,\x22DateListLogP\
lot\x22,\x22DateListPl\
ot\x22,\x22DateListSte\
pPlot\x22,\x22DateObje\
ct\x22,\x22DateObjectQ\
\x22,\x22DateOverlapsQ\
\x22,\x22DatePattern\x22,\
\x22DatePlus\x22,\x22Date\
Range\x22,\x22DateRedu\
ction\x22,\x22DateScal\
e\x22,\x22DateSelect\x22,\
\x22DateString\x22,\x22Da\
teTicksFormat\x22,\x22\
DateValue\x22,\x22Date\
WithinQ\x22,\x22Daubec\
hiesWavelet\x22,\x22Da\
visDistribution\x22\
,\x22DawsonF\x22,\x22DayC\
ount\x22,\x22DayCountC\
onvention\x22,\x22DayH\
emisphere\x22,\x22Dayl\
ightQ\x22,\x22DayMatch\
Q\x22,\x22DayName\x22,\x22Da\
yNightTerminator\
\x22,\x22DayPlus\x22,\x22Day\
Range\x22,\x22DayRound\
\x22,\x22DeBruijnGraph\
\x22,\x22DeBruijnSeque\
nce\x22,\x22Debug\x22,\x22De\
bugTag\x22,\x22Decapit\
alize\x22,\x22Decimal\x22\
,\x22DecimalForm\x22,\x22\
DeclareCompiledC\
omponent\x22,\x22Decla\
reKnownSymbols\x22,\
\x22DeclarePackage\x22\
,\x22Decompose\x22,\x22De\
convolutionLayer\
\x22,\x22Decrement\x22,\x22D\
ecrypt\x22,\x22Decrypt\
File\x22,\x22DedekindE\
ta\x22,\x22DeepSpacePr\
obeData\x22,\x22Defaul\
t\x22,\x22Default2DToo\
l\x22,\x22Default3DToo\
l\x22,\x22DefaultAttac\
hedCellStyle\x22,\x22D\
efaultAxesStyle\x22\
,\x22DefaultBaseSty\
le\x22,\x22DefaultBoxS\
tyle\x22,\x22DefaultBu\
tton\x22,\x22DefaultCo\
lor\x22,\x22DefaultCon\
trolPlacement\x22,\x22\
DefaultDockedCel\
lStyle\x22,\x22Default\
DuplicateCellSty\
le\x22,\x22DefaultDura\
tion\x22,\x22DefaultEl\
ement\x22,\x22DefaultF\
aceGridsStyle\x22,\x22\
DefaultFieldHint\
Style\x22,\x22DefaultF\
ont\x22,\x22DefaultFon\
tProperties\x22,\x22De\
faultFormatType\x22\
,\x22DefaultFrameSt\
yle\x22,\x22DefaultFra\
meTicksStyle\x22,\x22D\
efaultGridLinesS\
tyle\x22,\x22DefaultIn\
lineFormatType\x22,\
\x22DefaultInputFor\
matType\x22,\x22Defaul\
tLabelStyle\x22,\x22De\
faultMenuStyle\x22,\
\x22DefaultNaturalL\
anguage\x22,\x22Defaul\
tNewCellStyle\x22,\x22\
DefaultNewInline\
CellStyle\x22,\x22Defa\
ultNotebook\x22,\x22De\
faultOptions\x22,\x22D\
efaultOutputForm\
atType\x22,\x22Default\
PrintPrecision\x22,\
\x22DefaultStyle\x22,\x22\
DefaultStyleDefi\
nitions\x22,\x22Defaul\
tTextFormatType\x22\
,\x22DefaultTextInl\
ineFormatType\x22,\x22\
DefaultTicksStyl\
e\x22,\x22DefaultToolt\
ipStyle\x22,\x22Defaul\
tValue\x22,\x22Default\
Values\x22,\x22Defer\x22,\
\x22DefineExternal\x22\
,\x22DefineInputStr\
eamMethod\x22,\x22Defi\
neOutputStreamMe\
thod\x22,\x22DefineRes\
ourceFunction\x22,\x22\
Definition\x22,\x22Deg\
ree\x22,\x22DegreeCent\
rality\x22,\x22DegreeG\
raphDistribution\
\x22,\x22DegreeLexicog\
raphic\x22,\x22DegreeR\
everseLexicograp\
hic\x22,\x22DEigensyst\
em\x22,\x22DEigenvalue\
s\x22,\x22Deinitializa\
tion\x22,\x22Del\x22,\x22Del\
aunayMesh\x22,\x22Dela\
yed\x22,\x22Deletable\x22\
,\x22Delete\x22,\x22Delet\
eAdjacentDuplica\
tes\x22,\x22DeleteAnom\
alies\x22,\x22DeleteBo\
rderComponents\x22,\
\x22DeleteCases\x22,\x22D\
eleteChannel\x22,\x22D\
eleteCloudExpres\
sion\x22,\x22DeleteCon\
tents\x22,\x22DeleteDi\
rectory\x22,\x22Delete\
Duplicates\x22,\x22Del\
eteDuplicatesBy\x22\
,\x22DeleteElements\
\x22,\x22DeleteFile\x22,\x22\
DeleteMissing\x22,\x22\
DeleteObject\x22,\x22D\
eletePermissions\
Key\x22,\x22DeleteSear\
chIndex\x22,\x22Delete\
SmallComponents\x22\
,\x22DeleteStopword\
s\x22,\x22DeleteWithCo\
ntents\x22,\x22Deletio\
nWarning\x22,\x22Delim\
itedArray\x22,\x22Deli\
mitedSequence\x22,\x22\
Delimiter\x22,\x22Deli\
miterAutoMatchin\
g\x22,\x22DelimiterFla\
shTime\x22,\x22Delimit\
erMatching\x22,\x22Del\
imiters\x22,\x22Delive\
ryFunction\x22,\x22Den\
drogram\x22,\x22Denomi\
nator\x22,\x22DensityG\
raphics\x22,\x22Densit\
yHistogram\x22,\x22Den\
sityPlot\x22,\x22Densi\
tyPlot3D\x22,\x22Depen\
dentVariables\x22,\x22\
Deploy\x22,\x22Deploye\
d\x22,\x22Depth\x22,\x22Dept\
hFirstScan\x22,\x22Der\
ivative\x22,\x22Deriva\
tiveFilter\x22,\x22Der\
ivativePDETerm\x22,\
\x22DerivedKey\x22,\x22De\
scriptorStateSpa\
ce\x22,\x22DesignMatri\
x\x22,\x22DestroyAfter\
Evaluation\x22,\x22Det\
\x22,\x22DeviceClose\x22,\
\x22DeviceConfigure\
\x22,\x22DeviceExecute\
\x22,\x22DeviceExecute\
Asynchronous\x22,\x22D\
eviceObject\x22,\x22De\
viceOpen\x22,\x22Devic\
eOpenQ\x22,\x22DeviceR\
ead\x22,\x22DeviceRead\
Buffer\x22,\x22DeviceR\
eadLatest\x22,\x22Devi\
ceReadList\x22,\x22Dev\
iceReadTimeSerie\
s\x22,\x22Devices\x22,\x22De\
viceStreams\x22,\x22De\
viceWrite\x22,\x22Devi\
ceWriteBuffer\x22,\x22\
DGaussianWavelet\
\x22,\x22DiacriticalPo\
sitioning\x22,\x22Diag\
onal\x22,\x22Diagonali\
zableMatrixQ\x22,\x22D\
iagonalMatrix\x22,\x22\
DiagonalMatrixQ\x22\
,\x22Dialog\x22,\x22Dialo\
gIndent\x22,\x22Dialog\
Input\x22,\x22DialogLe\
vel\x22,\x22DialogNote\
book\x22,\x22DialogPro\
log\x22,\x22DialogRetu\
rn\x22,\x22DialogSymbo\
ls\x22,\x22Diamond\x22,\x22D\
iamondMatrix\x22,\x22D\
iceDissimilarity\
\x22,\x22DictionaryLoo\
kup\x22,\x22Dictionary\
WordQ\x22,\x22Differen\
ceDelta\x22,\x22Differ\
enceOrder\x22,\x22Diff\
erenceQuotient\x22,\
\x22DifferenceRoot\x22\
,\x22DifferenceRoot\
Reduce\x22,\x22Differe\
nces\x22,\x22Different\
ialD\x22,\x22Different\
ialRoot\x22,\x22Differ\
entialRootReduce\
\x22,\x22Differentiato\
rFilter\x22,\x22Diffus\
ionPDETerm\x22,\x22Dig\
gleGatesPointPro\
cess\x22,\x22DiggleGra\
ttonPointProcess\
\x22,\x22DigitalSignat\
ure\x22,\x22DigitBlock\
\x22,\x22DigitBlockMin\
imum\x22,\x22DigitChar\
acter\x22,\x22DigitCou\
nt\x22,\x22DigitQ\x22,\x22Di\
hedralAngle\x22,\x22Di\
hedralGroup\x22,\x22Di\
lation\x22,\x22Dimensi\
onalCombinations\
\x22,\x22DimensionalMe\
shComponents\x22,\x22D\
imensionReduce\x22,\
\x22DimensionReduce\
rFunction\x22,\x22Dime\
nsionReduction\x22,\
\x22Dimensions\x22,\x22Di\
racComb\x22,\x22DiracD\
elta\x22,\x22DirectedE\
dge\x22,\x22DirectedEd\
ges\x22,\x22DirectedGr\
aph\x22,\x22DirectedGr\
aphQ\x22,\x22DirectedI\
nfinity\x22,\x22Direct\
ion\x22,\x22Directiona\
lLight\x22,\x22Directi\
ve\x22,\x22Directory\x22,\
\x22DirectoryName\x22,\
\x22DirectoryQ\x22,\x22Di\
rectoryStack\x22,\x22D\
irichletBeta\x22,\x22D\
irichletCharacte\
r\x22,\x22DirichletCon\
dition\x22,\x22Dirichl\
etConvolve\x22,\x22Dir\
ichletDistributi\
on\x22,\x22DirichletEt\
a\x22,\x22DirichletL\x22,\
\x22DirichletLambda\
\x22,\x22DirichletTran\
sform\x22,\x22Dirichle\
tWindow\x22,\x22Disabl\
eConsolePrintPac\
ket\x22,\x22DisableFor\
matting\x22,\x22Discre\
teAsymptotic\x22,\x22D\
iscreteChirpZTra\
nsform\x22,\x22Discret\
eConvolve\x22,\x22Disc\
reteDelta\x22,\x22Disc\
reteHadamardTran\
sform\x22,\x22Discrete\
Indicator\x22,\x22Disc\
reteInputOutputM\
odel\x22,\x22DiscreteL\
imit\x22,\x22DiscreteL\
QEstimatorGains\x22\
,\x22DiscreteLQRegu\
latorGains\x22,\x22Dis\
creteLyapunovSol\
ve\x22,\x22DiscreteMar\
kovProcess\x22,\x22Dis\
creteMaxLimit\x22,\x22\
DiscreteMinLimit\
\x22,\x22DiscretePlot\x22\
,\x22DiscretePlot3D\
\x22,\x22DiscreteRatio\
\x22,\x22DiscreteRicca\
tiSolve\x22,\x22Discre\
teShift\x22,\x22Discre\
teTimeModelQ\x22,\x22D\
iscreteUniformDi\
stribution\x22,\x22Dis\
creteVariables\x22,\
\x22DiscreteWavelet\
Data\x22,\x22DiscreteW\
aveletPacketTran\
sform\x22,\x22Discrete\
WaveletTransform\
\x22,\x22DiscretizeGra\
phics\x22,\x22Discreti\
zeRegion\x22,\x22Discr\
iminant\x22,\x22Disjoi\
ntQ\x22,\x22Disjunctio\
n\x22,\x22Disk\x22,\x22DiskB\
ox\x22,\x22DiskBoxOpti\
ons\x22,\x22DiskMatrix\
\x22,\x22DiskSegment\x22,\
\x22Dispatch\x22,\x22Disp\
atchQ\x22,\x22Dispersi\
onEstimatorFunct\
ion\x22,\x22Display\x22,\x22\
DisplayAllSteps\x22\
,\x22DisplayEndPack\
et\x22,\x22DisplayForm\
\x22,\x22DisplayFuncti\
on\x22,\x22DisplayPack\
et\x22,\x22DisplayRule\
s\x22,\x22DisplayStrin\
g\x22,\x22DisplayTempo\
rary\x22,\x22DisplayWi\
th\x22,\x22DisplayWith\
Ref\x22,\x22DisplayWit\
hVariable\x22,\x22Dist\
anceFunction\x22,\x22D\
istanceMatrix\x22,\x22\
DistanceTransfor\
m\x22,\x22Distribute\x22,\
\x22Distributed\x22,\x22D\
istributedContex\
ts\x22,\x22DistributeD\
efinitions\x22,\x22Dis\
tributionChart\x22,\
\x22DistributionDom\
ain\x22,\x22Distributi\
onFitTest\x22,\x22Dist\
ributionParamete\
rAssumptions\x22,\x22D\
istributionParam\
eterQ\x22,\x22Ditherin\
g\x22,\x22Div\x22,\x22Diverg\
ence\x22,\x22Divide\x22,\x22\
DivideBy\x22,\x22Divid\
ers\x22,\x22DivideSide\
s\x22,\x22Divisible\x22,\x22\
Divisors\x22,\x22Divis\
orSigma\x22,\x22Diviso\
rSum\x22,\x22DMSList\x22,\
\x22DMSString\x22,\x22Do\x22\
,\x22DockedCell\x22,\x22D\
ockedCells\x22,\x22Doc\
umentGenerator\x22,\
\x22DocumentGenerat\
orInformation\x22,\x22\
DocumentGenerato\
rInformationData\
\x22,\x22DocumentGener\
ators\x22,\x22Document\
Notebook\x22,\x22Docum\
entWeightingRule\
s\x22,\x22Dodecahedron\
\x22,\x22DomainRegistr\
ationInformation\
\x22,\x22DominantColor\
s\x22,\x22DominatorTre\
eGraph\x22,\x22Dominat\
orVertexList\x22,\x22D\
OSTextFormat\x22,\x22D\
ot\x22,\x22DotDashed\x22,\
\x22DotEqual\x22,\x22DotL\
ayer\x22,\x22DotPlusLa\
yer\x22,\x22Dotted\x22,\x22D\
oubleBracketingB\
ar\x22,\x22DoubleConto\
urIntegral\x22,\x22Dou\
bleDownArrow\x22,\x22D\
oubleLeftArrow\x22,\
\x22DoubleLeftRight\
Arrow\x22,\x22DoubleLe\
ftTee\x22,\x22DoubleLo\
ngLeftArrow\x22,\x22Do\
ubleLongLeftRigh\
tArrow\x22,\x22DoubleL\
ongRightArrow\x22,\x22\
DoubleRightArrow\
\x22,\x22DoubleRightTe\
e\x22,\x22DoubleUpArro\
w\x22,\x22DoubleUpDown\
Arrow\x22,\x22DoubleVe\
rticalBar\x22,\x22Doub\
lyInfinite\x22,\x22Dow\
n\x22,\x22DownArrow\x22,\x22\
DownArrowBar\x22,\x22D\
ownArrowUpArrow\x22\
,\x22DownLeftRightV\
ector\x22,\x22DownLeft\
TeeVector\x22,\x22Down\
LeftVector\x22,\x22Dow\
nLeftVectorBar\x22,\
\x22DownRightTeeVec\
tor\x22,\x22DownRightV\
ector\x22,\x22DownRigh\
tVectorBar\x22,\x22Dow\
nsample\x22,\x22DownTe\
e\x22,\x22DownTeeArrow\
\x22,\x22DownValues\x22,\x22\
DownValuesFuncti\
on\x22,\x22DragAndDrop\
\x22,\x22DrawBackFaces\
\x22,\x22DrawEdges\x22,\x22D\
rawFrontFaces\x22,\x22\
DrawHighlighted\x22\
,\x22DrazinInverse\x22\
,\x22Drop\x22,\x22Dropout\
Layer\x22,\x22DropShad\
owing\x22,\x22DSolve\x22,\
\x22DSolveChangeVar\
iables\x22,\x22DSolveV\
alue\x22,\x22Dt\x22,\x22Dual\
LinearProgrammin\
g\x22,\x22DualPlanarGr\
aph\x22,\x22DualPolyhe\
dron\x22,\x22DualSyste\
msModel\x22,\x22DumpGe\
t\x22,\x22DumpSave\x22,\x22D\
uplicateFreeQ\x22,\x22\
Duration\x22,\x22Dynam\
ic\x22,\x22DynamicBox\x22\
,\x22DynamicBoxOpti\
ons\x22,\x22DynamicEva\
luationTimeout\x22,\
\x22DynamicGeoGraph\
ics\x22,\x22DynamicIma\
ge\x22,\x22DynamicLoca\
tion\x22,\x22DynamicMo\
dule\x22,\x22DynamicMo\
duleBox\x22,\x22Dynami\
cModuleBoxOption\
s\x22,\x22DynamicModul\
eParent\x22,\x22Dynami\
cModuleValues\x22,\x22\
DynamicName\x22,\x22Dy\
namicNamespace\x22,\
\x22DynamicReferenc\
e\x22,\x22DynamicSetti\
ng\x22,\x22DynamicUpda\
ting\x22,\x22DynamicWr\
apper\x22,\x22DynamicW\
rapperBox\x22,\x22Dyna\
micWrapperBoxOpt\
ions\x22,\x22E\x22,\x22Earth\
ImpactData\x22,\x22Ear\
thquakeData\x22,\x22Ec\
centricityCentra\
lity\x22,\x22Echo\x22,\x22Ec\
hoEvaluation\x22,\x22E\
choFunction\x22,\x22Ec\
hoLabel\x22,\x22EchoTi\
ming\x22,\x22EclipseTy\
pe\x22,\x22EdgeAdd\x22,\x22E\
dgeBetweennessCe\
ntrality\x22,\x22EdgeC\
apacity\x22,\x22EdgeCa\
pForm\x22,\x22EdgeChro\
maticNumber\x22,\x22Ed\
geColor\x22,\x22EdgeCo\
nnectivity\x22,\x22Edg\
eContract\x22,\x22Edge\
Cost\x22,\x22EdgeCount\
\x22,\x22EdgeCoverQ\x22,\x22\
EdgeCycleMatrix\x22\
,\x22EdgeDashing\x22,\x22\
EdgeDelete\x22,\x22Edg\
eDetect\x22,\x22EdgeFo\
rm\x22,\x22EdgeIndex\x22,\
\x22EdgeJoinForm\x22,\x22\
EdgeLabeling\x22,\x22E\
dgeLabels\x22,\x22Edge\
LabelStyle\x22,\x22Edg\
eList\x22,\x22EdgeOpac\
ity\x22,\x22EdgeQ\x22,\x22Ed\
geRenderingFunct\
ion\x22,\x22EdgeRules\x22\
,\x22EdgeShapeFunct\
ion\x22,\x22EdgeStyle\x22\
,\x22EdgeTaggedGrap\
h\x22,\x22EdgeTaggedGr\
aphQ\x22,\x22EdgeTags\x22\
,\x22EdgeThickness\x22\
,\x22EdgeTransitive\
GraphQ\x22,\x22EdgeVal\
ueRange\x22,\x22EdgeVa\
lueSizes\x22,\x22EdgeW\
eight\x22,\x22EdgeWeig\
htedGraphQ\x22,\x22Edi\
table\x22,\x22EditButt\
onSettings\x22,\x22Edi\
tCellTagsSetting\
s\x22,\x22EditDistance\
\x22,\x22EffectiveInte\
rest\x22,\x22Eigensyst\
em\x22,\x22Eigenvalues\
\x22,\x22EigenvectorCe\
ntrality\x22,\x22Eigen\
vectors\x22,\x22Elemen\
t\x22,\x22ElementData\x22\
,\x22ElementwiseLay\
er\x22,\x22ElidedForms\
\x22,\x22Eliminate\x22,\x22E\
liminationOrder\x22\
,\x22Ellipsoid\x22,\x22El\
lipticE\x22,\x22Ellipt\
icExp\x22,\x22Elliptic\
ExpPrime\x22,\x22Ellip\
ticF\x22,\x22EllipticF\
ilterModel\x22,\x22Ell\
ipticK\x22,\x22Ellipti\
cLog\x22,\x22EllipticN\
omeQ\x22,\x22EllipticP\
i\x22,\x22EllipticRedu\
cedHalfPeriods\x22,\
\x22EllipticTheta\x22,\
\x22EllipticThetaPr\
ime\x22,\x22EmbedCode\x22\
,\x22EmbeddedHTML\x22,\
\x22EmbeddedService\
\x22,\x22EmbeddedSQLEn\
tityClass\x22,\x22Embe\
ddedSQLExpressio\
n\x22,\x22EmbeddingLay\
er\x22,\x22EmbeddingOb\
ject\x22,\x22EmitSound\
\x22,\x22EmphasizeSynt\
axErrors\x22,\x22Empir\
icalDistribution\
\x22,\x22Empty\x22,\x22Empty\
GraphQ\x22,\x22EmptyRe\
gion\x22,\x22EmptySpac\
eF\x22,\x22EnableConso\
lePrintPacket\x22,\x22\
Enabled\x22,\x22Enclos\
e\x22,\x22Encode\x22,\x22Enc\
rypt\x22,\x22Encrypted\
Object\x22,\x22Encrypt\
File\x22,\x22End\x22,\x22End\
Add\x22,\x22EndDialogP\
acket\x22,\x22EndOfBuf\
fer\x22,\x22EndOfFile\x22\
,\x22EndOfLine\x22,\x22En\
dOfString\x22,\x22EndP\
ackage\x22,\x22EngineE\
nvironment\x22,\x22Eng\
ineeringForm\x22,\x22E\
nter\x22,\x22EnterExpr\
essionPacket\x22,\x22E\
nterTextPacket\x22,\
\x22Entity\x22,\x22Entity\
Class\x22,\x22EntityCl\
assList\x22,\x22Entity\
Copies\x22,\x22EntityF\
unction\x22,\x22Entity\
Group\x22,\x22EntityIn\
stance\x22,\x22EntityL\
ist\x22,\x22EntityPref\
etch\x22,\x22EntityPro\
perties\x22,\x22Entity\
Property\x22,\x22Entit\
yPropertyClass\x22,\
\x22EntityRegister\x22\
,\x22EntityStore\x22,\x22\
EntityStores\x22,\x22E\
ntityTypeName\x22,\x22\
EntityUnregister\
\x22,\x22EntityValue\x22,\
\x22Entropy\x22,\x22Entro\
pyFilter\x22,\x22Envir\
onment\x22,\x22Epilog\x22\
,\x22EpilogFunction\
\x22,\x22Equal\x22,\x22Equal\
Columns\x22,\x22EqualR\
ows\x22,\x22EqualTilde\
\x22,\x22EqualTo\x22,\x22Equ\
atedTo\x22,\x22Equilib\
rium\x22,\x22Equirippl\
eFilterKernel\x22,\x22\
Equivalent\x22,\x22Erf\
\x22,\x22Erfc\x22,\x22Erfi\x22,\
\x22ErlangB\x22,\x22Erlan\
gC\x22,\x22ErlangDistr\
ibution\x22,\x22Erosio\
n\x22,\x22ErrorBox\x22,\x22E\
rrorBoxOptions\x22,\
\x22ErrorNorm\x22,\x22Err\
orPacket\x22,\x22Error\
sDialogSettings\x22\
,\x22EscapeRadius\x22,\
\x22EstimatedBackgr\
ound\x22,\x22Estimated\
Distribution\x22,\x22E\
stimatedPointNor\
mals\x22,\x22Estimated\
PointProcess\x22,\x22E\
stimatedProcess\x22\
,\x22EstimatedVario\
gramModel\x22,\x22Esti\
matorGains\x22,\x22Est\
imatorRegulator\x22\
,\x22EuclideanDista\
nce\x22,\x22EulerAngle\
s\x22,\x22EulerCharact\
eristic\x22,\x22EulerE\
\x22,\x22EulerGamma\x22,\x22\
EulerianGraphQ\x22,\
\x22EulerMatrix\x22,\x22E\
ulerPhi\x22,\x22Evalua\
table\x22,\x22Evaluate\
\x22,\x22Evaluated\x22,\x22E\
valuatePacket\x22,\x22\
EvaluateSchedule\
dTask\x22,\x22Evaluati\
onBox\x22,\x22Evaluati\
onCell\x22,\x22Evaluat\
ionCompletionAct\
ion\x22,\x22Evaluation\
Data\x22,\x22Evaluatio\
nElements\x22,\x22Eval\
uationEnvironmen\
t\x22,\x22EvaluationMo\
de\x22,\x22EvaluationM\
onitor\x22,\x22Evaluat\
ionNotebook\x22,\x22Ev\
aluationObject\x22,\
\x22EvaluationOrder\
\x22,\x22EvaluationPri\
vileges\x22,\x22Evalua\
tionRateLimit\x22,\x22\
Evaluator\x22,\x22Eval\
uatorNames\x22,\x22Eve\
nQ\x22,\x22EventData\x22,\
\x22EventEvaluator\x22\
,\x22EventHandler\x22,\
\x22EventHandlerTag\
\x22,\x22EventLabels\x22,\
\x22EventSeries\x22,\x22E\
xactBlackmanWind\
ow\x22,\x22ExactNumber\
Q\x22,\x22ExactRootIso\
lation\x22,\x22Example\
Data\x22,\x22Except\x22,\x22\
ExcludedContexts\
\x22,\x22ExcludedForms\
\x22,\x22ExcludedLines\
\x22,\x22ExcludedPhysi\
calQuantities\x22,\x22\
ExcludePods\x22,\x22Ex\
clusions\x22,\x22Exclu\
sionsStyle\x22,\x22Exi\
sts\x22,\x22Exit\x22,\x22Exi\
tDialog\x22,\x22Exopla\
netData\x22,\x22Exp\x22,\x22\
Expand\x22,\x22ExpandA\
ll\x22,\x22ExpandDenom\
inator\x22,\x22ExpandF\
ileName\x22,\x22Expand\
Numerator\x22,\x22Expe\
ctation\x22,\x22Expect\
ationE\x22,\x22Expecte\
dValue\x22,\x22ExpGamm\
aDistribution\x22,\x22\
ExpIntegralE\x22,\x22E\
xpIntegralEi\x22,\x22E\
xpirationDate\x22,\x22\
Exponent\x22,\x22Expon\
entFunction\x22,\x22Ex\
ponentialDistrib\
ution\x22,\x22Exponent\
ialFamily\x22,\x22Expo\
nentialGeneratin\
gFunction\x22,\x22Expo\
nentialMovingAve\
rage\x22,\x22Exponenti\
alPowerDistribut\
ion\x22,\x22ExponentPo\
sition\x22,\x22Exponen\
tStep\x22,\x22Export\x22,\
\x22ExportAutoRepla\
cements\x22,\x22Export\
ByteArray\x22,\x22Expo\
rtForm\x22,\x22ExportP\
acket\x22,\x22ExportSt\
ring\x22,\x22Expressio\
n\x22,\x22ExpressionCe\
ll\x22,\x22ExpressionG\
raph\x22,\x22Expressio\
nPacket\x22,\x22Expres\
sionTree\x22,\x22Expre\
ssionUUID\x22,\x22ExpT\
oTrig\x22,\x22Extended\
EntityClass\x22,\x22Ex\
tendedGCD\x22,\x22Exte\
nsion\x22,\x22ExtentEl\
ementFunction\x22,\x22\
ExtentMarkers\x22,\x22\
ExtentSize\x22,\x22Ext\
ernalBundle\x22,\x22Ex\
ternalCall\x22,\x22Ext\
ernalDataCharact\
erEncoding\x22,\x22Ext\
ernalEvaluate\x22,\x22\
ExternalFunction\
\x22,\x22ExternalFunct\
ionName\x22,\x22Extern\
alIdentifier\x22,\x22E\
xternalObject\x22,\x22\
ExternalOptions\x22\
,\x22ExternalSessio\
nObject\x22,\x22Extern\
alSessions\x22,\x22Ext\
ernalStorageBase\
\x22,\x22ExternalStora\
geDownload\x22,\x22Ext\
ernalStorageGet\x22\
,\x22ExternalStorag\
eObject\x22,\x22Extern\
alStoragePut\x22,\x22E\
xternalStorageUp\
load\x22,\x22ExternalT\
ypeSignature\x22,\x22E\
xternalValue\x22,\x22E\
xtract\x22,\x22Extract\
Archive\x22,\x22Extrac\
tLayer\x22,\x22Extract\
PacletArchive\x22,\x22\
ExtremeValueDist\
ribution\x22,\x22FaceA\
lign\x22,\x22FaceForm\x22\
,\x22FaceGrids\x22,\x22Fa\
ceGridsStyle\x22,\x22F\
aceRecognize\x22,\x22F\
acialFeatures\x22,\x22\
Factor\x22,\x22FactorC\
omplete\x22,\x22Factor\
ial\x22,\x22Factorial2\
\x22,\x22FactorialMome\
nt\x22,\x22FactorialMo\
mentGeneratingFu\
nction\x22,\x22Factori\
alPower\x22,\x22Factor\
Integer\x22,\x22Factor\
List\x22,\x22FactorSqu\
areFree\x22,\x22Factor\
SquareFreeList\x22,\
\x22FactorTerms\x22,\x22F\
actorTermsList\x22,\
\x22Fail\x22,\x22Failure\x22\
,\x22FailureAction\x22\
,\x22FailureDistrib\
ution\x22,\x22FailureQ\
\x22,\x22False\x22,\x22Farey\
Sequence\x22,\x22FARIM\
AProcess\x22,\x22Featu\
reDistance\x22,\x22Fea\
tureExtract\x22,\x22Fe\
atureExtraction\x22\
,\x22FeatureExtract\
or\x22,\x22FeatureExtr\
actorFunction\x22,\x22\
FeatureImpactPlo\
t\x22,\x22FeatureNames\
\x22,\x22FeatureNeares\
t\x22,\x22FeatureSpace\
Plot\x22,\x22FeatureSp\
acePlot3D\x22,\x22Feat\
ureTypes\x22,\x22Featu\
reValueDependenc\
yPlot\x22,\x22FeatureV\
alueImpactPlot\x22,\
\x22FEDisableConsol\
ePrintPacket\x22,\x22F\
eedbackLinearize\
\x22,\x22FeedbackSecto\
r\x22,\x22FeedbackSect\
orStyle\x22,\x22Feedba\
ckType\x22,\x22FEEnabl\
eConsolePrintPac\
ket\x22,\x22FetalGrowt\
hData\x22,\x22Fibonacc\
i\x22,\x22Fibonorial\x22,\
\x22FieldCompletion\
Function\x22,\x22Field\
Hint\x22,\x22FieldHint\
Style\x22,\x22FieldMas\
ked\x22,\x22FieldSize\x22\
,\x22File\x22,\x22FileBas\
eName\x22,\x22FileByte\
Count\x22,\x22FileConv\
ert\x22,\x22FileDate\x22,\
\x22FileExistsQ\x22,\x22F\
ileExtension\x22,\x22F\
ileFormat\x22,\x22File\
FormatProperties\
\x22,\x22FileFormatQ\x22,\
\x22FileHandler\x22,\x22F\
ileHash\x22,\x22FileIn\
formation\x22,\x22File\
Name\x22,\x22FileNameD\
epth\x22,\x22FileNameD\
ialogSettings\x22,\x22\
FileNameDrop\x22,\x22F\
ileNameForms\x22,\x22F\
ileNameJoin\x22,\x22Fi\
leNames\x22,\x22FileNa\
meSetter\x22,\x22FileN\
ameSplit\x22,\x22FileN\
ameTake\x22,\x22FileNa\
meToFormatList\x22,\
\x22FilePrint\x22,\x22Fil\
eSize\x22,\x22FileSyst\
emMap\x22,\x22FileSyst\
emScan\x22,\x22FileSys\
temTree\x22,\x22FileTe\
mplate\x22,\x22FileTem\
plateApply\x22,\x22Fil\
eType\x22,\x22FilledCu\
rve\x22,\x22FilledCurv\
eBox\x22,\x22FilledCur\
veBoxOptions\x22,\x22F\
illedTorus\x22,\x22Fil\
lForm\x22,\x22Filling\x22\
,\x22FillingStyle\x22,\
\x22FillingTransfor\
m\x22,\x22FilteredEnti\
tyClass\x22,\x22Filter\
Rules\x22,\x22Financia\
lBond\x22,\x22Financia\
lData\x22,\x22Financia\
lDerivative\x22,\x22Fi\
nancialIndicator\
\x22,\x22Find\x22,\x22FindAn\
omalies\x22,\x22FindAr\
gMax\x22,\x22FindArgMi\
n\x22,\x22FindChannels\
\x22,\x22FindClique\x22,\x22\
FindClusters\x22,\x22F\
indCookies\x22,\x22Fin\
dCurvePath\x22,\x22Fin\
dCycle\x22,\x22FindDev\
ices\x22,\x22FindDistr\
ibution\x22,\x22FindDi\
stributionParame\
ters\x22,\x22FindDivis\
ions\x22,\x22FindEdgeC\
oloring\x22,\x22FindEd\
geCover\x22,\x22FindEd\
geCut\x22,\x22FindEdge\
IndependentPaths\
\x22,\x22FindEquationa\
lProof\x22,\x22FindEul\
erianCycle\x22,\x22Fin\
dExternalEvaluat\
ors\x22,\x22FindFaces\x22\
,\x22FindFile\x22,\x22Fin\
dFit\x22,\x22FindFormu\
la\x22,\x22FindFundame\
ntalCycles\x22,\x22Fin\
dGeneratingFunct\
ion\x22,\x22FindGeoLoc\
ation\x22,\x22FindGeom\
etricConjectures\
\x22,\x22FindGeometric\
Transform\x22,\x22Find\
GraphCommunities\
\x22,\x22FindGraphIsom\
orphism\x22,\x22FindGr\
aphPartition\x22,\x22F\
indHamiltonianCy\
cle\x22,\x22FindHamilt\
onianPath\x22,\x22Find\
HiddenMarkovStat\
es\x22,\x22FindImageTe\
xt\x22,\x22FindIndepen\
dentEdgeSet\x22,\x22Fi\
ndIndependentVer\
texSet\x22,\x22FindIns\
tance\x22,\x22FindInte\
gerNullVector\x22,\x22\
FindIsomers\x22,\x22Fi\
ndIsomorphicSubg\
raph\x22,\x22FindKClan\
\x22,\x22FindKClique\x22,\
\x22FindKClub\x22,\x22Fin\
dKPlex\x22,\x22FindLib\
rary\x22,\x22FindLinea\
rRecurrence\x22,\x22Fi\
ndList\x22,\x22FindMat\
chingColor\x22,\x22Fin\
dMaximum\x22,\x22FindM\
aximumCut\x22,\x22Find\
MaximumFlow\x22,\x22Fi\
ndMaxValue\x22,\x22Fin\
dMeshDefects\x22,\x22F\
indMinimum\x22,\x22Fin\
dMinimumCostFlow\
\x22,\x22FindMinimumCu\
t\x22,\x22FindMinValue\
\x22,\x22FindMoleculeS\
ubstructure\x22,\x22Fi\
ndPath\x22,\x22FindPea\
ks\x22,\x22FindPermuta\
tion\x22,\x22FindPlana\
rColoring\x22,\x22Find\
PointProcessPara\
meters\x22,\x22FindPos\
tmanTour\x22,\x22FindP\
rocessParameters\
\x22,\x22FindRegionTra\
nsform\x22,\x22FindRep\
eat\x22,\x22FindRoot\x22,\
\x22FindSequenceFun\
ction\x22,\x22FindSett\
ings\x22,\x22FindShort\
estPath\x22,\x22FindSh\
ortestTour\x22,\x22Fin\
dSpanningTree\x22,\x22\
FindSubgraphIsom\
orphism\x22,\x22FindSy\
stemModelEquilib\
rium\x22,\x22FindTextu\
alAnswer\x22,\x22FindT\
hreshold\x22,\x22FindT\
ransientRepeat\x22,\
\x22FindVertexColor\
ing\x22,\x22FindVertex\
Cover\x22,\x22FindVert\
exCut\x22,\x22FindVert\
exIndependentPat\
hs\x22,\x22Fine\x22,\x22Fini\
shDynamic\x22,\x22Fini\
teAbelianGroupCo\
unt\x22,\x22FiniteGrou\
pCount\x22,\x22FiniteG\
roupData\x22,\x22First\
\x22,\x22FirstCase\x22,\x22F\
irstPassageTimeD\
istribution\x22,\x22Fi\
rstPosition\x22,\x22Fi\
scherGroupFi22\x22,\
\x22FischerGroupFi2\
3\x22,\x22FischerGroup\
Fi24Prime\x22,\x22Fish\
erHypergeometric\
Distribution\x22,\x22F\
isherRatioTest\x22,\
\x22FisherZDistribu\
tion\x22,\x22Fit\x22,\x22Fit\
All\x22,\x22FitRegular\
ization\x22,\x22Fitted\
Model\x22,\x22FixedOrd\
er\x22,\x22FixedPoint\x22\
,\x22FixedPointList\
\x22,\x22FlashSelectio\
n\x22,\x22Flat\x22,\x22FlatS\
hading\x22,\x22Flatten\
\x22,\x22FlattenAt\x22,\x22F\
lattenLayer\x22,\x22Fl\
atTopWindow\x22,\x22Fl\
ightData\x22,\x22FlipV\
iew\x22,\x22Floor\x22,\x22Fl\
owPolynomial\x22,\x22F\
old\x22,\x22FoldList\x22,\
\x22FoldPair\x22,\x22Fold\
PairList\x22,\x22FoldW\
hile\x22,\x22FoldWhile\
List\x22,\x22FollowRed\
irects\x22,\x22Font\x22,\x22\
FontColor\x22,\x22Font\
Family\x22,\x22FontFor\
m\x22,\x22FontName\x22,\x22F\
ontOpacity\x22,\x22Fon\
tPostScriptName\x22\
,\x22FontProperties\
\x22,\x22FontReencodin\
g\x22,\x22FontSize\x22,\x22F\
ontSlant\x22,\x22FontS\
ubstitutions\x22,\x22F\
ontTracking\x22,\x22Fo\
ntVariations\x22,\x22F\
ontWeight\x22,\x22For\x22\
,\x22ForAll\x22,\x22ForAl\
lType\x22,\x22ForceVer\
sionInstall\x22,\x22Fo\
rmat\x22,\x22FormatRul\
es\x22,\x22FormatType\x22\
,\x22FormatTypeAuto\
Convert\x22,\x22Format\
Values\x22,\x22FormBox\
\x22,\x22FormBoxOption\
s\x22,\x22FormControl\x22\
,\x22FormFunction\x22,\
\x22FormLayoutFunct\
ion\x22,\x22FormObject\
\x22,\x22FormPage\x22,\x22Fo\
rmProtectionMeth\
od\x22,\x22FormTheme\x22,\
\x22FormulaData\x22,\x22F\
ormulaLookup\x22,\x22F\
ortranForm\x22,\x22For\
ward\x22,\x22ForwardBa\
ckward\x22,\x22Forward\
CloudCredentials\
\x22,\x22Fourier\x22,\x22Fou\
rierCoefficient\x22\
,\x22FourierCosCoef\
ficient\x22,\x22Fourie\
rCosSeries\x22,\x22Fou\
rierCosTransform\
\x22,\x22FourierDCT\x22,\x22\
FourierDCTFilter\
\x22,\x22FourierDCTMat\
rix\x22,\x22FourierDST\
\x22,\x22FourierDSTMat\
rix\x22,\x22FourierMat\
rix\x22,\x22FourierPar\
ameters\x22,\x22Fourie\
rSequenceTransfo\
rm\x22,\x22FourierSeri\
es\x22,\x22FourierSinC\
oefficient\x22,\x22Fou\
rierSinSeries\x22,\x22\
FourierSinTransf\
orm\x22,\x22FourierTra\
nsform\x22,\x22Fourier\
TrigSeries\x22,\x22Fox\
H\x22,\x22FoxHReduce\x22,\
\x22FractionalBrown\
ianMotionProcess\
\x22,\x22FractionalD\x22,\
\x22FractionalGauss\
ianNoiseProcess\x22\
,\x22FractionalPart\
\x22,\x22FractionBox\x22,\
\x22FractionBoxOpti\
ons\x22,\x22FractionLi\
ne\x22,\x22Frame\x22,\x22Fra\
meBox\x22,\x22FrameBox\
Options\x22,\x22Framed\
\x22,\x22FrameInset\x22,\x22\
FrameLabel\x22,\x22Fra\
meless\x22,\x22FrameLi\
stVideo\x22,\x22FrameM\
argins\x22,\x22FrameRa\
te\x22,\x22FrameStyle\x22\
,\x22FrameTicks\x22,\x22F\
rameTicksStyle\x22,\
\x22FRatioDistribut\
ion\x22,\x22FrechetDis\
tribution\x22,\x22Free\
Q\x22,\x22FrenetSerret\
System\x22,\x22Frequen\
cySamplingFilter\
Kernel\x22,\x22Fresnel\
C\x22,\x22FresnelF\x22,\x22F\
resnelG\x22,\x22Fresne\
lS\x22,\x22Friday\x22,\x22Fr\
obeniusNumber\x22,\x22\
FrobeniusSolve\x22,\
\x22FromAbsoluteTim\
e\x22,\x22FromCharacte\
rCode\x22,\x22FromCoef\
ficientRules\x22,\x22F\
romContinuedFrac\
tion\x22,\x22FromDate\x22\
,\x22FromDateString\
\x22,\x22FromDigits\x22,\x22\
FromDMS\x22,\x22FromEn\
tity\x22,\x22FromJulia\
nDate\x22,\x22FromLett\
erNumber\x22,\x22FromP\
olarCoordinates\x22\
,\x22FromRawPointer\
\x22,\x22FromRomanNume\
ral\x22,\x22FromSpheri\
calCoordinates\x22,\
\x22FromUnixTime\x22,\x22\
Front\x22,\x22FrontEnd\
DynamicExpressio\
n\x22,\x22FrontEndEven\
tActions\x22,\x22Front\
EndExecute\x22,\x22Fro\
ntEndObject\x22,\x22Fr\
ontEndResource\x22,\
\x22FrontEndResourc\
eString\x22,\x22FrontE\
ndStackSize\x22,\x22Fr\
ontEndToken\x22,\x22Fr\
ontEndTokenExecu\
te\x22,\x22FrontEndVal\
ueCache\x22,\x22FrontE\
ndVersion\x22,\x22Fron\
tFaceColor\x22,\x22Fro\
ntFaceGlowColor\x22\
,\x22FrontFaceOpaci\
ty\x22,\x22FrontFaceSp\
ecularColor\x22,\x22Fr\
ontFaceSpecularE\
xponent\x22,\x22FrontF\
aceSurfaceAppear\
ance\x22,\x22FrontFace\
Texture\x22,\x22Full\x22,\
\x22FullAxes\x22,\x22Full\
Definition\x22,\x22Ful\
lForm\x22,\x22FullGrap\
hics\x22,\x22FullInfor\
mationOutputRegu\
lator\x22,\x22FullOpti\
ons\x22,\x22FullRegion\
\x22,\x22FullSimplify\x22\
,\x22Function\x22,\x22Fun\
ctionAnalytic\x22,\x22\
FunctionBijectiv\
e\x22,\x22FunctionComp\
ile\x22,\x22FunctionCo\
mpileExport\x22,\x22Fu\
nctionCompileExp\
ortByteArray\x22,\x22F\
unctionCompileEx\
portLibrary\x22,\x22Fu\
nctionCompileExp\
ortString\x22,\x22Func\
tionContinuous\x22,\
\x22FunctionConvexi\
ty\x22,\x22FunctionDec\
laration\x22,\x22Funct\
ionDiscontinuiti\
es\x22,\x22FunctionDom\
ain\x22,\x22FunctionEx\
pand\x22,\x22FunctionI\
njective\x22,\x22Funct\
ionInterpolation\
\x22,\x22FunctionLayer\
\x22,\x22FunctionMerom\
orphic\x22,\x22Functio\
nMonotonicity\x22,\x22\
FunctionPeriod\x22,\
\x22FunctionPoles\x22,\
\x22FunctionRange\x22,\
\x22FunctionSign\x22,\x22\
FunctionSingular\
ities\x22,\x22Function\
Space\x22,\x22Function\
Surjective\x22,\x22Fus\
sellVeselyImport\
ance\x22,\x22GaborFilt\
er\x22,\x22GaborMatrix\
\x22,\x22GaborWavelet\x22\
,\x22GainMargins\x22,\x22\
GainPhaseMargins\
\x22,\x22GalaxyData\x22,\x22\
GalleryView\x22,\x22Ga\
mma\x22,\x22GammaDistr\
ibution\x22,\x22GammaR\
egularized\x22,\x22Gap\
Penalty\x22,\x22GARCHP\
rocess\x22,\x22GatedRe\
currentLayer\x22,\x22G\
ather\x22,\x22GatherBy\
\x22,\x22GaugeFaceElem\
entFunction\x22,\x22Ga\
ugeFaceStyle\x22,\x22G\
augeFrameElement\
Function\x22,\x22Gauge\
FrameSize\x22,\x22Gaug\
eFrameStyle\x22,\x22Ga\
ugeLabels\x22,\x22Gaug\
eMarkers\x22,\x22Gauge\
Style\x22,\x22Gaussian\
Filter\x22,\x22Gaussia\
nIntegers\x22,\x22Gaus\
sianMatrix\x22,\x22Gau\
ssianOrthogonalM\
atrixDistributio\
n\x22,\x22GaussianSymp\
lecticMatrixDist\
ribution\x22,\x22Gauss\
ianUnitaryMatrix\
Distribution\x22,\x22G\
aussianWindow\x22,\x22\
GCD\x22,\x22Gegenbauer\
C\x22,\x22General\x22,\x22Ge\
neralizedLinearM\
odelFit\x22,\x22Genera\
teAsymmetricKeyP\
air\x22,\x22GenerateCo\
nditions\x22,\x22Gener\
atedAssetFormat\x22\
,\x22GeneratedAsset\
Location\x22,\x22Gener\
atedCell\x22,\x22Gener\
atedCellStyles\x22,\
\x22GeneratedDocume\
ntBinding\x22,\x22Gene\
rateDerivedKey\x22,\
\x22GenerateDigital\
Signature\x22,\x22Gene\
rateDocument\x22,\x22G\
eneratedParamete\
rs\x22,\x22GeneratedQu\
antityMagnitudes\
\x22,\x22GenerateFileS\
ignature\x22,\x22Gener\
ateHTTPResponse\x22\
,\x22GenerateSecure\
dAuthenticationK\
ey\x22,\x22GenerateSym\
metricKey\x22,\x22Gene\
ratingFunction\x22,\
\x22GeneratorDescri\
ption\x22,\x22Generato\
rHistoryLength\x22,\
\x22GeneratorOutput\
Type\x22,\x22Generic\x22,\
\x22GenericCylindri\
calDecomposition\
\x22,\x22GenomeData\x22,\x22\
GenomeLookup\x22,\x22G\
eoAntipode\x22,\x22Geo\
Area\x22,\x22GeoArrayS\
ize\x22,\x22GeoBackgro\
und\x22,\x22GeoBoundar\
y\x22,\x22GeoBoundingB\
ox\x22,\x22GeoBounds\x22,\
\x22GeoBoundsRegion\
\x22,\x22GeoBoundsRegi\
onBoundary\x22,\x22Geo\
BubbleChart\x22,\x22Ge\
oCenter\x22,\x22GeoCir\
cle\x22,\x22GeoContour\
Plot\x22,\x22GeoDensit\
yPlot\x22,\x22Geodesic\
Closing\x22,\x22Geodes\
icDilation\x22,\x22Geo\
desicErosion\x22,\x22G\
eodesicOpening\x22,\
\x22GeodesicPolyhed\
ron\x22,\x22GeoDestina\
tion\x22,\x22GeodesyDa\
ta\x22,\x22GeoDirectio\
n\x22,\x22GeoDisk\x22,\x22Ge\
oDisplacement\x22,\x22\
GeoDistance\x22,\x22Ge\
oDistanceList\x22,\x22\
GeoElevationData\
\x22,\x22GeoEntities\x22,\
\x22GeoGraphics\x22,\x22G\
eoGraphPlot\x22,\x22Ge\
oGraphValuePlot\x22\
,\x22GeogravityMode\
lData\x22,\x22GeoGridD\
irectionDifferen\
ce\x22,\x22GeoGridLine\
s\x22,\x22GeoGridLines\
Style\x22,\x22GeoGridP\
osition\x22,\x22GeoGri\
dRange\x22,\x22GeoGrid\
RangePadding\x22,\x22G\
eoGridUnitArea\x22,\
\x22GeoGridUnitDist\
ance\x22,\x22GeoGridVe\
ctor\x22,\x22GeoGroup\x22\
,\x22GeoHemisphere\x22\
,\x22GeoHemisphereB\
oundary\x22,\x22GeoHis\
togram\x22,\x22GeoIden\
tify\x22,\x22GeoImage\x22\
,\x22GeoLabels\x22,\x22Ge\
oLength\x22,\x22GeoLis\
tPlot\x22,\x22GeoLocat\
ion\x22,\x22Geological\
PeriodData\x22,\x22Geo\
magneticModelDat\
a\x22,\x22GeoMarker\x22,\x22\
GeometricAsserti\
on\x22,\x22GeometricBr\
ownianMotionProc\
ess\x22,\x22GeometricD\
istribution\x22,\x22Ge\
ometricMean\x22,\x22Ge\
ometricMeanFilte\
r\x22,\x22GeometricOpt\
imization\x22,\x22Geom\
etricScene\x22,\x22Geo\
metricStep\x22,\x22Geo\
metricStylingRul\
es\x22,\x22GeometricTe\
st\x22,\x22GeometricTr\
ansformation\x22,\x22G\
eometricTransfor\
mation3DBox\x22,\x22Ge\
ometricTransform\
ation3DBoxOption\
s\x22,\x22GeometricTra\
nsformationBox\x22,\
\x22GeometricTransf\
ormationBoxOptio\
ns\x22,\x22GeoModel\x22,\x22\
GeoNearest\x22,\x22Geo\
OrientationData\x22\
,\x22GeoPath\x22,\x22GeoP\
olygon\x22,\x22GeoPosi\
tion\x22,\x22GeoPositi\
onENU\x22,\x22GeoPosit\
ionXYZ\x22,\x22GeoProj\
ection\x22,\x22GeoProj\
ectionData\x22,\x22Geo\
Range\x22,\x22GeoRange\
Padding\x22,\x22GeoReg\
ionValuePlot\x22,\x22G\
eoResolution\x22,\x22G\
eoScaleBar\x22,\x22Geo\
Server\x22,\x22GeoSmoo\
thHistogram\x22,\x22Ge\
oStreamPlot\x22,\x22Ge\
oStyling\x22,\x22GeoSt\
ylingImageFuncti\
on\x22,\x22GeoVariant\x22\
,\x22GeoVector\x22,\x22Ge\
oVectorENU\x22,\x22Geo\
VectorPlot\x22,\x22Geo\
VectorXYZ\x22,\x22GeoV\
isibleRegion\x22,\x22G\
eoVisibleRegionB\
oundary\x22,\x22GeoWit\
hinQ\x22,\x22GeoZoomLe\
vel\x22,\x22GestureHan\
dler\x22,\x22GestureHa\
ndlerTag\x22,\x22Get\x22,\
\x22GetContext\x22,\x22Ge\
tEnvironment\x22,\x22G\
etFileName\x22,\x22Get\
LinebreakInforma\
tionPacket\x22,\x22Gib\
bsPointProcess\x22,\
\x22Glaisher\x22,\x22Glob\
alClusteringCoef\
ficient\x22,\x22Global\
Preferences\x22,\x22Gl\
obalSession\x22,\x22Gl\
ow\x22,\x22GoldenAngle\
\x22,\x22GoldenRatio\x22,\
\x22GompertzMakeham\
Distribution\x22,\x22G\
oochShading\x22,\x22Go\
odmanKruskalGamm\
a\x22,\x22GoodmanKrusk\
alGammaTest\x22,\x22Go\
to\x22,\x22GouraudShad\
ing\x22,\x22Grad\x22,\x22Gra\
dient\x22,\x22Gradient\
Filter\x22,\x22Gradien\
tFittedMesh\x22,\x22Gr\
adientOrientatio\
nFilter\x22,\x22Gramma\
rApply\x22,\x22Grammar\
Rules\x22,\x22GrammarT\
oken\x22,\x22Graph\x22,\x22G\
raph3D\x22,\x22GraphAs\
sortativity\x22,\x22Gr\
aphAutomorphismG\
roup\x22,\x22GraphCent\
er\x22,\x22GraphComple\
ment\x22,\x22GraphData\
\x22,\x22GraphDensity\x22\
,\x22GraphDiameter\x22\
,\x22GraphDifferenc\
e\x22,\x22GraphDisjoin\
tUnion\x22,\x22GraphDi\
stance\x22,\x22GraphDi\
stanceMatrix\x22,\x22G\
raphEmbedding\x22,\x22\
GraphHighlight\x22,\
\x22GraphHighlightS\
tyle\x22,\x22GraphHub\x22\
,\x22Graphics\x22,\x22Gra\
phics3D\x22,\x22Graphi\
cs3DBox\x22,\x22Graphi\
cs3DBoxOptions\x22,\
\x22GraphicsArray\x22,\
\x22GraphicsBaselin\
e\x22,\x22GraphicsBox\x22\
,\x22GraphicsBoxOpt\
ions\x22,\x22GraphicsC\
olor\x22,\x22GraphicsC\
olumn\x22,\x22Graphics\
Complex\x22,\x22Graphi\
csComplex3DBox\x22,\
\x22GraphicsComplex\
3DBoxOptions\x22,\x22G\
raphicsComplexBo\
x\x22,\x22GraphicsComp\
lexBoxOptions\x22,\x22\
GraphicsContents\
\x22,\x22GraphicsData\x22\
,\x22GraphicsGrid\x22,\
\x22GraphicsGridBox\
\x22,\x22GraphicsGroup\
\x22,\x22GraphicsGroup\
3DBox\x22,\x22Graphics\
Group3DBoxOption\
s\x22,\x22GraphicsGrou\
pBox\x22,\x22GraphicsG\
roupBoxOptions\x22,\
\x22GraphicsGroupin\
g\x22,\x22GraphicsHigh\
lightColor\x22,\x22Gra\
phicsRow\x22,\x22Graph\
icsSpacing\x22,\x22Gra\
phicsStyle\x22,\x22Gra\
phIntersection\x22,\
\x22GraphJoin\x22,\x22Gra\
phLayerLabels\x22,\x22\
GraphLayers\x22,\x22Gr\
aphLayerStyle\x22,\x22\
GraphLayout\x22,\x22Gr\
aphLinkEfficienc\
y\x22,\x22GraphPeriphe\
ry\x22,\x22GraphPlot\x22,\
\x22GraphPlot3D\x22,\x22G\
raphPower\x22,\x22Grap\
hProduct\x22,\x22Graph\
PropertyDistribu\
tion\x22,\x22GraphQ\x22,\x22\
GraphRadius\x22,\x22Gr\
aphReciprocity\x22,\
\x22GraphRoot\x22,\x22Gra\
phStyle\x22,\x22GraphS\
um\x22,\x22GraphTree\x22,\
\x22GraphUnion\x22,\x22Gr\
ay\x22,\x22GrayLevel\x22,\
\x22Greater\x22,\x22Great\
erEqual\x22,\x22Greate\
rEqualLess\x22,\x22Gre\
aterEqualThan\x22,\x22\
GreaterFullEqual\
\x22,\x22GreaterGreate\
r\x22,\x22GreaterLess\x22\
,\x22GreaterSlantEq\
ual\x22,\x22GreaterTha\
n\x22,\x22GreaterTilde\
\x22,\x22GreekStyle\x22,\x22\
Green\x22,\x22GreenFun\
ction\x22,\x22Grid\x22,\x22G\
ridBaseline\x22,\x22Gr\
idBox\x22,\x22GridBoxA\
lignment\x22,\x22GridB\
oxBackground\x22,\x22G\
ridBoxDividers\x22,\
\x22GridBoxFrame\x22,\x22\
GridBoxItemSize\x22\
,\x22GridBoxItemSty\
le\x22,\x22GridBoxOpti\
ons\x22,\x22GridBoxSpa\
cings\x22,\x22GridCrea\
tionSettings\x22,\x22G\
ridDefaultElemen\
t\x22,\x22GridElementS\
tyleOptions\x22,\x22Gr\
idFrame\x22,\x22GridFr\
ameMargins\x22,\x22Gri\
dGraph\x22,\x22GridLin\
es\x22,\x22GridLinesSt\
yle\x22,\x22GridVideo\x22\
,\x22GroebnerBasis\x22\
,\x22GroupActionBas\
e\x22,\x22GroupBy\x22,\x22Gr\
oupCentralizer\x22,\
\x22GroupElementFro\
mWord\x22,\x22GroupEle\
mentPosition\x22,\x22G\
roupElementQ\x22,\x22G\
roupElements\x22,\x22G\
roupElementToWor\
d\x22,\x22GroupGenerat\
ors\x22,\x22Groupings\x22\
,\x22GroupMultiplic\
ationTable\x22,\x22Gro\
upOpenerColor\x22,\x22\
GroupOpenerInsid\
eFrame\x22,\x22GroupOr\
bits\x22,\x22GroupOrde\
r\x22,\x22GroupPageBre\
akWithin\x22,\x22Group\
SetwiseStabilize\
r\x22,\x22GroupStabili\
zer\x22,\x22GroupStabi\
lizerChain\x22,\x22Gro\
upTogetherGroupi\
ng\x22,\x22GroupTogeth\
erNestedGrouping\
\x22,\x22GrowCutCompon\
ents\x22,\x22Gudermann\
ian\x22,\x22GuidedFilt\
er\x22,\x22GumbelDistr\
ibution\x22,\x22HaarWa\
velet\x22,\x22Hadamard\
Matrix\x22,\x22HalfLin\
e\x22,\x22HalfNormalDi\
stribution\x22,\x22Hal\
fPlane\x22,\x22HalfSpa\
ce\x22,\x22HalftoneSha\
ding\x22,\x22Hamiltoni\
anGraphQ\x22,\x22Hammi\
ngDistance\x22,\x22Ham\
mingWindow\x22,\x22Han\
dlerFunctions\x22,\x22\
HandlerFunctions\
Keys\x22,\x22HankelH1\x22\
,\x22HankelH2\x22,\x22Han\
kelMatrix\x22,\x22Hank\
elTransform\x22,\x22Ha\
nnPoissonWindow\x22\
,\x22HannWindow\x22,\x22H\
aradaNortonGroup\
HN\x22,\x22HararyGraph\
\x22,\x22HardcorePoint\
Process\x22,\x22Harmon\
icMean\x22,\x22Harmoni\
cMeanFilter\x22,\x22Ha\
rmonicNumber\x22,\x22H\
ash\x22,\x22HatchFilli\
ng\x22,\x22HatchShadin\
g\x22,\x22Haversine\x22,\x22\
HazardFunction\x22,\
\x22Head\x22,\x22HeadComp\
ose\x22,\x22HeaderAlig\
nment\x22,\x22HeaderBa\
ckground\x22,\x22Heade\
rDisplayFunction\
\x22,\x22HeaderLines\x22,\
\x22Headers\x22,\x22Heade\
rSize\x22,\x22HeaderSt\
yle\x22,\x22Heads\x22,\x22He\
atFluxValue\x22,\x22He\
atInsulationValu\
e\x22,\x22HeatOutflowV\
alue\x22,\x22HeatRadia\
tionValue\x22,\x22Heat\
SymmetryValue\x22,\x22\
HeatTemperatureC\
ondition\x22,\x22HeatT\
ransferPDECompon\
ent\x22,\x22HeatTransf\
erValue\x22,\x22Heavis\
ideLambda\x22,\x22Heav\
isidePi\x22,\x22Heavis\
ideTheta\x22,\x22HeldG\
roupHe\x22,\x22HeldPar\
t\x22,\x22HelmholtzPDE\
Component\x22,\x22Help\
BrowserLookup\x22,\x22\
HelpBrowserNoteb\
ook\x22,\x22HelpBrowse\
rSettings\x22,\x22Help\
ViewerSettings\x22,\
\x22Here\x22,\x22HermiteD\
ecomposition\x22,\x22H\
ermiteH\x22,\x22Hermit\
ian\x22,\x22HermitianM\
atrixQ\x22,\x22Hessenb\
ergDecomposition\
\x22,\x22Hessian\x22,\x22Heu\
nB\x22,\x22HeunBPrime\x22\
,\x22HeunC\x22,\x22HeunCP\
rime\x22,\x22HeunD\x22,\x22H\
eunDPrime\x22,\x22Heun\
G\x22,\x22HeunGPrime\x22,\
\x22HeunT\x22,\x22HeunTPr\
ime\x22,\x22Hexadecima\
lCharacter\x22,\x22Hex\
ahedron\x22,\x22Hexahe\
dronBox\x22,\x22Hexahe\
dronBoxOptions\x22,\
\x22HiddenItems\x22,\x22H\
iddenMarkovProce\
ss\x22,\x22HiddenSurfa\
ce\x22,\x22Highlighted\
\x22,\x22HighlightGrap\
h\x22,\x22HighlightIma\
ge\x22,\x22HighlightMe\
sh\x22,\x22HighlightSt\
ring\x22,\x22HighpassF\
ilter\x22,\x22HigmanSi\
msGroupHS\x22,\x22Hilb\
ertCurve\x22,\x22Hilbe\
rtFilter\x22,\x22Hilbe\
rtMatrix\x22,\x22Histo\
gram\x22,\x22Histogram\
3D\x22,\x22HistogramDi\
stribution\x22,\x22His\
togramList\x22,\x22His\
togramPointDensi\
ty\x22,\x22HistogramTr\
ansform\x22,\x22Histog\
ramTransformInte\
rpolation\x22,\x22Hist\
oricalPeriodData\
\x22,\x22HitMissTransf\
orm\x22,\x22HITSCentra\
lity\x22,\x22HjorthDis\
tribution\x22,\x22Hodg\
eDual\x22,\x22Hoeffdin\
gD\x22,\x22HoeffdingDT\
est\x22,\x22Hold\x22,\x22Hol\
dAll\x22,\x22HoldAllCo\
mplete\x22,\x22HoldCom\
plete\x22,\x22HoldFirs\
t\x22,\x22HoldForm\x22,\x22H\
oldPattern\x22,\x22Hol\
dRest\x22,\x22HolidayC\
alendar\x22,\x22HomeDi\
rectory\x22,\x22HomePa\
ge\x22,\x22Horizontal\x22\
,\x22HorizontalForm\
\x22,\x22HorizontalGau\
ge\x22,\x22HorizontalS\
crollPosition\x22,\x22\
HornerForm\x22,\x22Hos\
tLookup\x22,\x22Hotell\
ingTSquareDistri\
bution\x22,\x22HoytDis\
tribution\x22,\x22HTML\
Save\x22,\x22HTTPError\
Response\x22,\x22HTTPR\
edirect\x22,\x22HTTPRe\
quest\x22,\x22HTTPRequ\
estData\x22,\x22HTTPRe\
sponse\x22,\x22Hue\x22,\x22H\
umanGrowthData\x22,\
\x22HumpDownHump\x22,\x22\
HumpEqual\x22,\x22Hurw\
itzLerchPhi\x22,\x22Hu\
rwitzZeta\x22,\x22Hype\
rbolicDistributi\
on\x22,\x22HypercubeGr\
aph\x22,\x22Hyperexpon\
entialDistributi\
on\x22,\x22Hyperfactor\
ial\x22,\x22Hypergeome\
tric0F1\x22,\x22Hyperg\
eometric0F1Regul\
arized\x22,\x22Hyperge\
ometric1F1\x22,\x22Hyp\
ergeometric1F1Re\
gularized\x22,\x22Hype\
rgeometric2F1\x22,\x22\
Hypergeometric2F\
1Regularized\x22,\x22H\
ypergeometricDis\
tribution\x22,\x22Hype\
rgeometricPFQ\x22,\x22\
HypergeometricPF\
QRegularized\x22,\x22H\
ypergeometricU\x22,\
\x22Hyperlink\x22,\x22Hyp\
erlinkAction\x22,\x22H\
yperlinkCreation\
Settings\x22,\x22Hyper\
plane\x22,\x22Hyphenat\
ion\x22,\x22Hyphenatio\
nOptions\x22,\x22Hypoe\
xponentialDistri\
bution\x22,\x22Hypothe\
sisTestData\x22,\x22I\x22\
,\x22IconData\x22,\x22Ico\
nize\x22,\x22IconizedO\
bject\x22,\x22IconRule\
s\x22,\x22Icosahedron\x22\
,\x22Identity\x22,\x22Ide\
ntityMatrix\x22,\x22If\
\x22,\x22IfCompiled\x22,\x22\
IgnoreCase\x22,\x22Ign\
oreDiacritics\x22,\x22\
IgnoreIsotopes\x22,\
\x22IgnorePunctuati\
on\x22,\x22IgnoreSpell\
Check\x22,\x22IgnoreSt\
ereochemistry\x22,\x22\
IgnoringInactive\
\x22,\x22Im\x22,\x22Image\x22,\x22\
Image3D\x22,\x22Image3\
DProjection\x22,\x22Im\
age3DSlices\x22,\x22Im\
ageAccumulate\x22,\x22\
ImageAdd\x22,\x22Image\
Adjust\x22,\x22ImageAl\
ign\x22,\x22ImageApply\
\x22,\x22ImageApplyInd\
exed\x22,\x22ImageAspe\
ctRatio\x22,\x22ImageA\
ssemble\x22,\x22ImageA\
ugmentationLayer\
\x22,\x22ImageBounding\
Boxes\x22,\x22ImageCac\
he\x22,\x22ImageCacheV\
alid\x22,\x22ImageCapt\
ure\x22,\x22ImageCaptu\
reFunction\x22,\x22Ima\
geCases\x22,\x22ImageC\
hannels\x22,\x22ImageC\
lip\x22,\x22ImageColla\
ge\x22,\x22ImageColorS\
pace\x22,\x22ImageComp\
ose\x22,\x22ImageConta\
insQ\x22,\x22ImageCont\
ents\x22,\x22ImageConv\
olve\x22,\x22ImageCooc\
currence\x22,\x22Image\
Corners\x22,\x22ImageC\
orrelate\x22,\x22Image\
CorrespondingPoi\
nts\x22,\x22ImageCrop\x22\
,\x22ImageData\x22,\x22Im\
ageDeconvolve\x22,\x22\
ImageDemosaic\x22,\x22\
ImageDifference\x22\
,\x22ImageDimension\
s\x22,\x22ImageDisplac\
ements\x22,\x22ImageDi\
stance\x22,\x22ImageEd\
itMode\x22,\x22ImageEf\
fect\x22,\x22ImageExpo\
sureCombine\x22,\x22Im\
ageFeatureTrack\x22\
,\x22ImageFileApply\
\x22,\x22ImageFileFilt\
er\x22,\x22ImageFileSc\
an\x22,\x22ImageFilter\
\x22,\x22ImageFocusCom\
bine\x22,\x22ImageFore\
stingComponents\x22\
,\x22ImageFormattin\
gWidth\x22,\x22ImageFo\
rwardTransformat\
ion\x22,\x22ImageGraph\
ics\x22,\x22ImageHisto\
gram\x22,\x22ImageIden\
tify\x22,\x22ImageInst\
anceQ\x22,\x22ImageKey\
points\x22,\x22ImageLa\
bels\x22,\x22ImageLege\
nds\x22,\x22ImageLevel\
s\x22,\x22ImageLines\x22,\
\x22ImageMargins\x22,\x22\
ImageMarker\x22,\x22Im\
ageMarkers\x22,\x22Ima\
geMeasurements\x22,\
\x22ImageMesh\x22,\x22Ima\
geMultiply\x22,\x22Ima\
geOffset\x22,\x22Image\
Pad\x22,\x22ImagePaddi\
ng\x22,\x22ImagePartit\
ion\x22,\x22ImagePerio\
dogram\x22,\x22ImagePe\
rspectiveTransfo\
rmation\x22,\x22ImageP\
osition\x22,\x22ImageP\
reviewFunction\x22,\
\x22ImagePyramid\x22,\x22\
ImagePyramidAppl\
y\x22,\x22ImageQ\x22,\x22Ima\
geRangeCache\x22,\x22I\
mageRecolor\x22,\x22Im\
ageReflect\x22,\x22Ima\
geRegion\x22,\x22Image\
Resize\x22,\x22ImageRe\
solution\x22,\x22Image\
Restyle\x22,\x22ImageR\
otate\x22,\x22ImageRot\
ated\x22,\x22ImageSali\
encyFilter\x22,\x22Ima\
geScaled\x22,\x22Image\
Scan\x22,\x22ImageSize\
\x22,\x22ImageSizeActi\
on\x22,\x22ImageSizeCa\
che\x22,\x22ImageSizeM\
ultipliers\x22,\x22Ima\
geSizeRaw\x22,\x22Imag\
eStitch\x22,\x22ImageS\
ubtract\x22,\x22ImageT\
ake\x22,\x22ImageTrans\
formation\x22,\x22Imag\
eTrim\x22,\x22ImageTyp\
e\x22,\x22ImageValue\x22,\
\x22ImageValuePosit\
ions\x22,\x22ImageVect\
orscopePlot\x22,\x22Im\
ageWaveformPlot\x22\
,\x22ImagingDevice\x22\
,\x22ImplicitD\x22,\x22Im\
plicitRegion\x22,\x22I\
mplies\x22,\x22Import\x22\
,\x22ImportAutoRepl\
acements\x22,\x22Impor\
tByteArray\x22,\x22Imp\
ortedObject\x22,\x22Im\
portOptions\x22,\x22Im\
portString\x22,\x22Imp\
rovementImportan\
ce\x22,\x22In\x22,\x22Inacti\
vate\x22,\x22Inactive\x22\
,\x22InactiveStyle\x22\
,\x22IncidenceGraph\
\x22,\x22IncidenceList\
\x22,\x22IncidenceMatr\
ix\x22,\x22IncludeArom\
aticBonds\x22,\x22Incl\
udeConstantBasis\
\x22,\x22IncludedConte\
xts\x22,\x22IncludeDef\
initions\x22,\x22Inclu\
deDirectories\x22,\x22\
IncludeFileExten\
sion\x22,\x22IncludeGe\
neratorTasks\x22,\x22I\
ncludeHydrogens\x22\
,\x22IncludeInflect\
ions\x22,\x22IncludeMe\
taInformation\x22,\x22\
IncludePods\x22,\x22In\
cludeQuantities\x22\
,\x22IncludeRelated\
Tables\x22,\x22Include\
SingularSolution\
s\x22,\x22IncludeSingu\
larTerm\x22,\x22Includ\
eWindowTimes\x22,\x22I\
ncrement\x22,\x22Indef\
initeMatrixQ\x22,\x22I\
ndent\x22,\x22Indentin\
gNewlineSpacings\
\x22,\x22IndentMaxFrac\
tion\x22,\x22Independe\
nceTest\x22,\x22Indepe\
ndentEdgeSetQ\x22,\x22\
IndependentPhysi\
calQuantity\x22,\x22In\
dependentUnit\x22,\x22\
IndependentUnitD\
imension\x22,\x22Indep\
endentVertexSetQ\
\x22,\x22Indeterminate\
\x22,\x22Indeterminate\
Threshold\x22,\x22Inde\
xCreationOptions\
\x22,\x22Indexed\x22,\x22Ind\
exEdgeTaggedGrap\
h\x22,\x22IndexGraph\x22,\
\x22IndexTag\x22,\x22Ineq\
uality\x22,\x22InertEv\
aluate\x22,\x22InertEx\
pression\x22,\x22Inexa\
ctNumberQ\x22,\x22Inex\
actNumbers\x22,\x22Inf\
initeFuture\x22,\x22In\
finiteLine\x22,\x22Inf\
initeLineThrough\
\x22,\x22InfinitePast\x22\
,\x22InfinitePlane\x22\
,\x22Infinity\x22,\x22Inf\
ix\x22,\x22InflationAd\
just\x22,\x22Inflation\
Method\x22,\x22Informa\
tion\x22,\x22Informati\
onData\x22,\x22Informa\
tionDataGrid\x22,\x22I\
nherited\x22,\x22Inher\
itScope\x22,\x22Inhomo\
geneousPoissonPo\
intProcess\x22,\x22Inh\
omogeneousPoisso\
nProcess\x22,\x22Initi\
alEvaluationHist\
ory\x22,\x22Initializa\
tion\x22,\x22Initializ\
ationCell\x22,\x22Init\
ializationCellEv\
aluation\x22,\x22Initi\
alizationCellWar\
ning\x22,\x22Initializ\
ationObject\x22,\x22In\
itializationObje\
cts\x22,\x22Initializa\
tionValue\x22,\x22Init\
ialize\x22,\x22Initial\
Seeding\x22,\x22Inline\
CounterAssignmen\
ts\x22,\x22InlineCount\
erIncrements\x22,\x22I\
nlineRules\x22,\x22Inn\
er\x22,\x22InnerPolygo\
n\x22,\x22InnerPolyhed\
ron\x22,\x22Inpaint\x22,\x22\
Input\x22,\x22InputAli\
ases\x22,\x22InputAssu\
mptions\x22,\x22InputA\
utoReplacements\x22\
,\x22InputField\x22,\x22I\
nputFieldBox\x22,\x22I\
nputFieldBoxOpti\
ons\x22,\x22InputForm\x22\
,\x22InputGrouping\x22\
,\x22InputNamePacke\
t\x22,\x22InputNoteboo\
k\x22,\x22InputPacket\x22\
,\x22InputPorts\x22,\x22I\
nputSettings\x22,\x22I\
nputStream\x22,\x22Inp\
utString\x22,\x22Input\
StringPacket\x22,\x22I\
nputToBoxFormPac\
ket\x22,\x22Insert\x22,\x22I\
nsertionFunction\
\x22,\x22InsertionPoin\
tObject\x22,\x22Insert\
Linebreaks\x22,\x22Ins\
ertResults\x22,\x22Ins\
et\x22,\x22Inset3DBox\x22\
,\x22Inset3DBoxOpti\
ons\x22,\x22InsetBox\x22,\
\x22InsetBoxOptions\
\x22,\x22Insphere\x22,\x22In\
stall\x22,\x22InstallS\
ervice\x22,\x22Instanc\
eNormalizationLa\
yer\x22,\x22InString\x22,\
\x22Integer\x22,\x22Integ\
erDigits\x22,\x22Integ\
erExponent\x22,\x22Int\
egerLength\x22,\x22Int\
egerName\x22,\x22Integ\
erPart\x22,\x22Integer\
Partitions\x22,\x22Int\
egerQ\x22,\x22IntegerR\
everse\x22,\x22Integer\
s\x22,\x22IntegerStrin\
g\x22,\x22Integral\x22,\x22I\
ntegrate\x22,\x22Integ\
rateChangeVariab\
les\x22,\x22Interactiv\
e\x22,\x22InteractiveT\
radingChart\x22,\x22In\
terfaceSwitched\x22\
,\x22Interlaced\x22,\x22I\
nterleaving\x22,\x22In\
ternallyBalanced\
Decomposition\x22,\x22\
InterpolatingFun\
ction\x22,\x22Interpol\
atingPolynomial\x22\
,\x22Interpolation\x22\
,\x22InterpolationO\
rder\x22,\x22Interpola\
tionPoints\x22,\x22Int\
erpolationPrecis\
ion\x22,\x22Interpreta\
tion\x22,\x22Interpret\
ationBox\x22,\x22Inter\
pretationBoxOpti\
ons\x22,\x22Interpreta\
tionFunction\x22,\x22I\
nterpreter\x22,\x22Int\
erpretTemplate\x22,\
\x22InterquartileRa\
nge\x22,\x22Interrupt\x22\
,\x22InterruptSetti\
ngs\x22,\x22Intersecte\
dEntityClass\x22,\x22I\
ntersectingQ\x22,\x22I\
ntersection\x22,\x22In\
terval\x22,\x22Interva\
lIntersection\x22,\x22\
IntervalMarkers\x22\
,\x22IntervalMarker\
sStyle\x22,\x22Interva\
lMemberQ\x22,\x22Inter\
valSlider\x22,\x22Inte\
rvalUnion\x22,\x22Into\
\x22,\x22Inverse\x22,\x22Inv\
erseBetaRegulari\
zed\x22,\x22InverseBil\
ateralLaplaceTra\
nsform\x22,\x22Inverse\
BilateralZTransf\
orm\x22,\x22InverseCDF\
\x22,\x22InverseChiSqu\
areDistribution\x22\
,\x22InverseContinu\
ousWaveletTransf\
orm\x22,\x22InverseDis\
tanceTransform\x22,\
\x22InverseElliptic\
NomeQ\x22,\x22InverseE\
rf\x22,\x22InverseErfc\
\x22,\x22InverseFourie\
r\x22,\x22InverseFouri\
erCosTransform\x22,\
\x22InverseFourierS\
equenceTransform\
\x22,\x22InverseFourie\
rSinTransform\x22,\x22\
InverseFourierTr\
ansform\x22,\x22Invers\
eFunction\x22,\x22Inve\
rseFunctions\x22,\x22I\
nverseGammaDistr\
ibution\x22,\x22Invers\
eGammaRegularize\
d\x22,\x22InverseGauss\
ianDistribution\x22\
,\x22InverseGuderma\
nnian\x22,\x22InverseH\
ankelTransform\x22,\
\x22InverseHaversin\
e\x22,\x22InverseImage\
Pyramid\x22,\x22Invers\
eJacobiCD\x22,\x22Inve\
rseJacobiCN\x22,\x22In\
verseJacobiCS\x22,\x22\
InverseJacobiDC\x22\
,\x22InverseJacobiD\
N\x22,\x22InverseJacob\
iDS\x22,\x22InverseJac\
obiNC\x22,\x22InverseJ\
acobiND\x22,\x22Invers\
eJacobiNS\x22,\x22Inve\
rseJacobiSC\x22,\x22In\
verseJacobiSD\x22,\x22\
InverseJacobiSN\x22\
,\x22InverseLaplace\
Transform\x22,\x22Inve\
rseMellinTransfo\
rm\x22,\x22InversePerm\
utation\x22,\x22Invers\
eRadon\x22,\x22Inverse\
RadonTransform\x22,\
\x22InverseSeries\x22,\
\x22InverseShortTim\
eFourier\x22,\x22Inver\
seSpectrogram\x22,\x22\
InverseSurvivalF\
unction\x22,\x22Invers\
eTransformedRegi\
on\x22,\x22InverseWave\
letTransform\x22,\x22I\
nverseWeierstras\
sP\x22,\x22InverseWish\
artMatrixDistrib\
ution\x22,\x22InverseZ\
Transform\x22,\x22Invi\
sible\x22,\x22Invisibl\
eApplication\x22,\x22I\
nvisibleTimes\x22,\x22\
IPAddress\x22,\x22Irre\
duciblePolynomia\
lQ\x22,\x22IslandData\x22\
,\x22IsolatingInter\
val\x22,\x22Isomorphic\
GraphQ\x22,\x22Isomorp\
hicSubgraphQ\x22,\x22I\
sotopeData\x22,\x22Ita\
lic\x22,\x22Item\x22,\x22Ite\
mAspectRatio\x22,\x22I\
temBox\x22,\x22ItemBox\
Options\x22,\x22ItemDi\
splayFunction\x22,\x22\
ItemSize\x22,\x22ItemS\
tyle\x22,\x22ItoProces\
s\x22,\x22JaccardDissi\
milarity\x22,\x22Jacob\
iAmplitude\x22,\x22Jac\
obian\x22,\x22JacobiCD\
\x22,\x22JacobiCN\x22,\x22Ja\
cobiCS\x22,\x22JacobiD\
C\x22,\x22JacobiDN\x22,\x22J\
acobiDS\x22,\x22Jacobi\
Epsilon\x22,\x22Jacobi\
NC\x22,\x22JacobiND\x22,\x22\
JacobiNS\x22,\x22Jacob\
iP\x22,\x22JacobiSC\x22,\x22\
JacobiSD\x22,\x22Jacob\
iSN\x22,\x22JacobiSymb\
ol\x22,\x22JacobiZeta\x22\
,\x22JacobiZN\x22,\x22Jan\
koGroupJ1\x22,\x22Jank\
oGroupJ2\x22,\x22Janko\
GroupJ3\x22,\x22JankoG\
roupJ4\x22,\x22JarqueB\
eraALMTest\x22,\x22Joh\
nsonDistribution\
\x22,\x22Join\x22,\x22JoinAc\
ross\x22,\x22Joined\x22,\x22\
JoinedCurve\x22,\x22Jo\
inedCurveBox\x22,\x22J\
oinedCurveBoxOpt\
ions\x22,\x22JoinForm\x22\
,\x22JordanDecompos\
ition\x22,\x22JordanMo\
delDecomposition\
\x22,\x22JulianDate\x22,\x22\
JuliaSetBoettche\
r\x22,\x22JuliaSetIter\
ationCount\x22,\x22Jul\
iaSetPlot\x22,\x22Juli\
aSetPoints\x22,\x22K\x22,\
\x22KagiChart\x22,\x22Kai\
serBesselWindow\x22\
,\x22KaiserWindow\x22,\
\x22KalmanEstimator\
\x22,\x22KalmanFilter\x22\
,\x22KarhunenLoeveD\
ecomposition\x22,\x22K\
aryTree\x22,\x22KatzCe\
ntrality\x22,\x22KCore\
Components\x22,\x22KDi\
stribution\x22,\x22KEd\
geConnectedCompo\
nents\x22,\x22KEdgeCon\
nectedGraphQ\x22,\x22K\
eepExistingVersi\
on\x22,\x22KelvinBei\x22,\
\x22KelvinBer\x22,\x22Kel\
vinKei\x22,\x22KelvinK\
er\x22,\x22KendallTau\x22\
,\x22KendallTauTest\
\x22,\x22KernelConfigu\
ration\x22,\x22KernelE\
xecute\x22,\x22KernelF\
unction\x22,\x22Kernel\
MixtureDistribut\
ion\x22,\x22KernelObje\
ct\x22,\x22Kernels\x22,\x22K\
et\x22,\x22Key\x22,\x22KeyCo\
llisionFunction\x22\
,\x22KeyComplement\x22\
,\x22KeyDrop\x22,\x22KeyD\
ropFrom\x22,\x22KeyExi\
stsQ\x22,\x22KeyFreeQ\x22\
,\x22KeyIntersectio\
n\x22,\x22KeyMap\x22,\x22Key\
MemberQ\x22,\x22Keypoi\
ntStrength\x22,\x22Key\
s\x22,\x22KeySelect\x22,\x22\
KeySort\x22,\x22KeySor\
tBy\x22,\x22KeyTake\x22,\x22\
KeyUnion\x22,\x22KeyVa\
lueMap\x22,\x22KeyValu\
ePattern\x22,\x22Khinc\
hin\x22,\x22KillProces\
s\x22,\x22KirchhoffGra\
ph\x22,\x22KirchhoffMa\
trix\x22,\x22KleinInva\
riantJ\x22,\x22Knapsac\
kSolve\x22,\x22KnightT\
ourGraph\x22,\x22KnotD\
ata\x22,\x22KnownUnitQ\
\x22,\x22KochCurve\x22,\x22K\
olmogorovSmirnov\
Test\x22,\x22Kronecker\
Delta\x22,\x22Kronecke\
rModelDecomposit\
ion\x22,\x22KroneckerP\
roduct\x22,\x22Kroneck\
erSymbol\x22,\x22Kuipe\
rTest\x22,\x22Kumarasw\
amyDistribution\x22\
,\x22Kurtosis\x22,\x22Kuw\
aharaFilter\x22,\x22KV\
ertexConnectedCo\
mponents\x22,\x22KVert\
exConnectedGraph\
Q\x22,\x22LABColor\x22,\x22L\
abel\x22,\x22Labeled\x22,\
\x22LabeledSlider\x22,\
\x22LabelingFunctio\
n\x22,\x22LabelingSize\
\x22,\x22LabelStyle\x22,\x22\
LabelVisibility\x22\
,\x22LaguerreL\x22,\x22La\
keData\x22,\x22LambdaC\
omponents\x22,\x22Lamb\
ertW\x22,\x22LameC\x22,\x22L\
ameCPrime\x22,\x22Lame\
EigenvalueA\x22,\x22La\
meEigenvalueB\x22,\x22\
LameS\x22,\x22LameSPri\
me\x22,\x22LaminaData\x22\
,\x22LanczosWindow\x22\
,\x22LandauDistribu\
tion\x22,\x22Language\x22\
,\x22LanguageCatego\
ry\x22,\x22LanguageDat\
a\x22,\x22LanguageIden\
tify\x22,\x22LanguageO\
ptions\x22,\x22Laplace\
Distribution\x22,\x22L\
aplaceTransform\x22\
,\x22Laplacian\x22,\x22La\
placianFilter\x22,\x22\
LaplacianGaussia\
nFilter\x22,\x22Laplac\
ianPDETerm\x22,\x22Lar\
ge\x22,\x22Larger\x22,\x22La\
st\x22,\x22Latitude\x22,\x22\
LatitudeLongitud\
e\x22,\x22LatticeData\x22\
,\x22LatticeReduce\x22\
,\x22Launch\x22,\x22Launc\
hKernels\x22,\x22Layer\
edGraphPlot\x22,\x22La\
yeredGraphPlot3D\
\x22,\x22LayerSizeFunc\
tion\x22,\x22LayoutInf\
ormation\x22,\x22LCHCo\
lor\x22,\x22LCM\x22,\x22Lead\
erSize\x22,\x22LeafCou\
nt\x22,\x22LeapVariant\
\x22,\x22LeapYearQ\x22,\x22L\
earnDistribution\
\x22,\x22LearnedDistri\
bution\x22,\x22Learnin\
gRate\x22,\x22Learning\
RateMultipliers\x22\
,\x22LeastSquares\x22,\
\x22LeastSquaresFil\
terKernel\x22,\x22Left\
\x22,\x22LeftArrow\x22,\x22L\
eftArrowBar\x22,\x22Le\
ftArrowRightArro\
w\x22,\x22LeftDownTeeV\
ector\x22,\x22LeftDown\
Vector\x22,\x22LeftDow\
nVectorBar\x22,\x22Lef\
tRightArrow\x22,\x22Le\
ftRightVector\x22,\x22\
LeftTee\x22,\x22LeftTe\
eArrow\x22,\x22LeftTee\
Vector\x22,\x22LeftTri\
angle\x22,\x22LeftTria\
ngleBar\x22,\x22LeftTr\
iangleEqual\x22,\x22Le\
ftUpDownVector\x22,\
\x22LeftUpTeeVector\
\x22,\x22LeftUpVector\x22\
,\x22LeftUpVectorBa\
r\x22,\x22LeftVector\x22,\
\x22LeftVectorBar\x22,\
\x22LegendAppearanc\
e\x22,\x22Legended\x22,\x22L\
egendFunction\x22,\x22\
LegendLabel\x22,\x22Le\
gendLayout\x22,\x22Leg\
endMargins\x22,\x22Leg\
endMarkers\x22,\x22Leg\
endMarkerSize\x22,\x22\
LegendreP\x22,\x22Lege\
ndreQ\x22,\x22Legendre\
Type\x22,\x22Length\x22,\x22\
LengthWhile\x22,\x22Le\
rchPhi\x22,\x22Less\x22,\x22\
LessEqual\x22,\x22Less\
EqualGreater\x22,\x22L\
essEqualThan\x22,\x22L\
essFullEqual\x22,\x22L\
essGreater\x22,\x22Les\
sLess\x22,\x22LessSlan\
tEqual\x22,\x22LessTha\
n\x22,\x22LessTilde\x22,\x22\
LetterCharacter\x22\
,\x22LetterCounts\x22,\
\x22LetterNumber\x22,\x22\
LetterQ\x22,\x22Level\x22\
,\x22LeveneTest\x22,\x22L\
eviCivitaTensor\x22\
,\x22LevyDistributi\
on\x22,\x22Lexicograph\
ic\x22,\x22Lexicograph\
icOrder\x22,\x22Lexico\
graphicSort\x22,\x22Li\
braryDataType\x22,\x22\
LibraryFunction\x22\
,\x22LibraryFunctio\
nDeclaration\x22,\x22L\
ibraryFunctionEr\
ror\x22,\x22LibraryFun\
ctionInformation\
\x22,\x22LibraryFuncti\
onLoad\x22,\x22Library\
FunctionUnload\x22,\
\x22LibraryLoad\x22,\x22L\
ibraryUnload\x22,\x22L\
icenseEntitlemen\
tObject\x22,\x22Licens\
eEntitlements\x22,\x22\
LicenseID\x22,\x22Lice\
nsingSettings\x22,\x22\
LiftingFilterDat\
a\x22,\x22LiftingWavel\
etTransform\x22,\x22Li\
ghtBlue\x22,\x22LightB\
rown\x22,\x22LightCyan\
\x22,\x22Lighter\x22,\x22Lig\
htGray\x22,\x22LightGr\
een\x22,\x22Lighting\x22,\
\x22LightingAngle\x22,\
\x22LightMagenta\x22,\x22\
LightOrange\x22,\x22Li\
ghtPink\x22,\x22LightP\
urple\x22,\x22LightRed\
\x22,\x22LightSources\x22\
,\x22LightYellow\x22,\x22\
Likelihood\x22,\x22Lim\
it\x22,\x22LimitsPosit\
ioning\x22,\x22LimitsP\
ositioningTokens\
\x22,\x22LindleyDistri\
bution\x22,\x22Line\x22,\x22\
Line3DBox\x22,\x22Line\
3DBoxOptions\x22,\x22L\
inearFilter\x22,\x22Li\
nearFractionalOp\
timization\x22,\x22Lin\
earFractionalTra\
nsform\x22,\x22LinearG\
radientFilling\x22,\
\x22LinearGradientI\
mage\x22,\x22Linearizi\
ngTransformation\
Data\x22,\x22LinearLay\
er\x22,\x22LinearModel\
Fit\x22,\x22LinearOffs\
etFunction\x22,\x22Lin\
earOptimization\x22\
,\x22LinearProgramm\
ing\x22,\x22LinearRecu\
rrence\x22,\x22LinearS\
olve\x22,\x22LinearSol\
veFunction\x22,\x22Lin\
eBox\x22,\x22LineBoxOp\
tions\x22,\x22LineBrea\
k\x22,\x22LinebreakAdj\
ustments\x22,\x22LineB\
reakChart\x22,\x22Line\
breakSemicolonWe\
ighting\x22,\x22LineBr\
eakWithin\x22,\x22Line\
Color\x22,\x22LineGrap\
h\x22,\x22LineIndent\x22,\
\x22LineIndentMaxFr\
action\x22,\x22LineInt\
egralConvolution\
Plot\x22,\x22LineInteg\
ralConvolutionSc\
ale\x22,\x22LineLegend\
\x22,\x22LineOpacity\x22,\
\x22LineSpacing\x22,\x22L\
ineWrapParts\x22,\x22L\
inkActivate\x22,\x22Li\
nkClose\x22,\x22LinkCo\
nnect\x22,\x22LinkConn\
ectedQ\x22,\x22LinkCre\
ate\x22,\x22LinkError\x22\
,\x22LinkFlush\x22,\x22Li\
nkFunction\x22,\x22Lin\
kHost\x22,\x22LinkInte\
rrupt\x22,\x22LinkLaun\
ch\x22,\x22LinkMode\x22,\x22\
LinkObject\x22,\x22Lin\
kOpen\x22,\x22LinkOpti\
ons\x22,\x22LinkPatter\
ns\x22,\x22LinkProtoco\
l\x22,\x22LinkRankCent\
rality\x22,\x22LinkRea\
d\x22,\x22LinkReadHeld\
\x22,\x22LinkReadyQ\x22,\x22\
Links\x22,\x22LinkServ\
ice\x22,\x22LinkWrite\x22\
,\x22LinkWriteHeld\x22\
,\x22LiouvilleLambd\
a\x22,\x22List\x22,\x22Lista\
ble\x22,\x22ListAnimat\
e\x22,\x22ListContourP\
lot\x22,\x22ListContou\
rPlot3D\x22,\x22ListCo\
nvolve\x22,\x22ListCor\
relate\x22,\x22ListCur\
vePathPlot\x22,\x22Lis\
tDeconvolve\x22,\x22Li\
stDensityPlot\x22,\x22\
ListDensityPlot3\
D\x22,\x22Listen\x22,\x22Lis\
tFormat\x22,\x22ListFo\
urierSequenceTra\
nsform\x22,\x22ListInt\
erpolation\x22,\x22Lis\
tLineIntegralCon\
volutionPlot\x22,\x22L\
istLinePlot\x22,\x22Li\
stLinePlot3D\x22,\x22L\
istLogLinearPlot\
\x22,\x22ListLogLogPlo\
t\x22,\x22ListLogPlot\x22\
,\x22ListPicker\x22,\x22L\
istPickerBox\x22,\x22L\
istPickerBoxBack\
ground\x22,\x22ListPic\
kerBoxOptions\x22,\x22\
ListPlay\x22,\x22ListP\
lot\x22,\x22ListPlot3D\
\x22,\x22ListPointPlot\
3D\x22,\x22ListPolarPl\
ot\x22,\x22ListQ\x22,\x22Lis\
tSliceContourPlo\
t3D\x22,\x22ListSliceD\
ensityPlot3D\x22,\x22L\
istSliceVectorPl\
ot3D\x22,\x22ListStepP\
lot\x22,\x22ListStream\
DensityPlot\x22,\x22Li\
stStreamPlot\x22,\x22L\
istStreamPlot3D\x22\
,\x22ListSurfacePlo\
t3D\x22,\x22ListVector\
DensityPlot\x22,\x22Li\
stVectorDisplace\
mentPlot\x22,\x22ListV\
ectorDisplacemen\
tPlot3D\x22,\x22ListVe\
ctorPlot\x22,\x22ListV\
ectorPlot3D\x22,\x22Li\
stZTransform\x22,\x22L\
iteral\x22,\x22Literal\
Search\x22,\x22Literal\
Type\x22,\x22LoadCompi\
ledComponent\x22,\x22L\
ocalAdaptiveBina\
rize\x22,\x22LocalCach\
e\x22,\x22LocalCluster\
ingCoefficient\x22,\
\x22LocalEvaluate\x22,\
\x22LocalizeDefinit\
ions\x22,\x22LocalizeV\
ariables\x22,\x22Local\
Object\x22,\x22LocalOb\
jects\x22,\x22LocalRes\
ponseNormalizati\
onLayer\x22,\x22LocalS\
ubmit\x22,\x22LocalSym\
bol\x22,\x22LocalTime\x22\
,\x22LocalTimeZone\x22\
,\x22LocationEquiva\
lenceTest\x22,\x22Loca\
tionTest\x22,\x22Locat\
or\x22,\x22LocatorAuto\
Create\x22,\x22Locator\
Box\x22,\x22LocatorBox\
Options\x22,\x22Locato\
rCentering\x22,\x22Loc\
atorPane\x22,\x22Locat\
orPaneBox\x22,\x22Loca\
torPaneBoxOption\
s\x22,\x22LocatorRegio\
n\x22,\x22Locked\x22,\x22Log\
\x22,\x22Log10\x22,\x22Log2\x22\
,\x22LogBarnesG\x22,\x22L\
ogGamma\x22,\x22LogGam\
maDistribution\x22,\
\x22LogicalExpand\x22,\
\x22LogIntegral\x22,\x22L\
ogisticDistribut\
ion\x22,\x22LogisticSi\
gmoid\x22,\x22LogitMod\
elFit\x22,\x22LogLikel\
ihood\x22,\x22LogLinea\
rPlot\x22,\x22LogLogis\
ticDistribution\x22\
,\x22LogLogPlot\x22,\x22L\
ogMultinormalDis\
tribution\x22,\x22LogN\
ormalDistributio\
n\x22,\x22LogPlot\x22,\x22Lo\
gRankTest\x22,\x22LogS\
eriesDistributio\
n\x22,\x22LongEqual\x22,\x22\
Longest\x22,\x22Longes\
tCommonSequence\x22\
,\x22LongestCommonS\
equencePositions\
\x22,\x22LongestCommon\
Subsequence\x22,\x22Lo\
ngestCommonSubse\
quencePositions\x22\
,\x22LongestMatch\x22,\
\x22LongestOrderedS\
equence\x22,\x22LongFo\
rm\x22,\x22Longitude\x22,\
\x22LongLeftArrow\x22,\
\x22LongLeftRightAr\
row\x22,\x22LongRightA\
rrow\x22,\x22LongShort\
TermMemoryLayer\x22\
,\x22Lookup\x22,\x22Loopb\
ack\x22,\x22LoopFreeGr\
aphQ\x22,\x22Looping\x22,\
\x22LossFunction\x22,\x22\
LowerCaseQ\x22,\x22Low\
erLeftArrow\x22,\x22Lo\
werRightArrow\x22,\x22\
LowerTriangulari\
ze\x22,\x22LowerTriang\
ularMatrix\x22,\x22Low\
erTriangularMatr\
ixQ\x22,\x22LowpassFil\
ter\x22,\x22LQEstimato\
rGains\x22,\x22LQGRegu\
lator\x22,\x22LQOutput\
RegulatorGains\x22,\
\x22LQRegulatorGain\
s\x22,\x22LUBackSubsti\
tution\x22,\x22LucasL\x22\
,\x22LuccioSamiComp\
onents\x22,\x22LUDecom\
position\x22,\x22Lunar\
Eclipse\x22,\x22LUVCol\
or\x22,\x22LyapunovSol\
ve\x22,\x22LyonsGroupL\
y\x22,\x22MachineID\x22,\x22\
MachineName\x22,\x22Ma\
chineNumberQ\x22,\x22M\
achinePrecision\x22\
,\x22MacintoshSyste\
mPageSetup\x22,\x22Mag\
enta\x22,\x22Magnifica\
tion\x22,\x22Magnify\x22,\
\x22MailAddressVali\
dation\x22,\x22MailExe\
cute\x22,\x22MailFolde\
r\x22,\x22MailItem\x22,\x22M\
ailReceiverFunct\
ion\x22,\x22MailRespon\
seFunction\x22,\x22Mai\
lSearch\x22,\x22MailSe\
rverConnect\x22,\x22Ma\
ilServerConnecti\
on\x22,\x22MailSetting\
s\x22,\x22MainSolve\x22,\x22\
MaintainDynamicC\
aches\x22,\x22Majority\
\x22,\x22MakeBoxes\x22,\x22M\
akeExpression\x22,\x22\
MakeRules\x22,\x22Mana\
gedLibraryExpres\
sionID\x22,\x22Managed\
LibraryExpressio\
nQ\x22,\x22MandelbrotS\
etBoettcher\x22,\x22Ma\
ndelbrotSetDista\
nce\x22,\x22Mandelbrot\
SetIterationCoun\
t\x22,\x22MandelbrotSe\
tMemberQ\x22,\x22Mande\
lbrotSetPlot\x22,\x22M\
angoldtLambda\x22,\x22\
ManhattanDistanc\
e\x22,\x22Manipulate\x22,\
\x22Manipulator\x22,\x22M\
annedSpaceMissio\
nData\x22,\x22MannWhit\
neyTest\x22,\x22Mantis\
saExponent\x22,\x22Man\
ual\x22,\x22Map\x22,\x22MapA\
ll\x22,\x22MapApply\x22,\x22\
MapAt\x22,\x22MapIndex\
ed\x22,\x22MAProcess\x22,\
\x22MapThread\x22,\x22Mar\
chenkoPasturDist\
ribution\x22,\x22Marcu\
mQ\x22,\x22MardiaCombi\
nedTest\x22,\x22Mardia\
KurtosisTest\x22,\x22M\
ardiaSkewnessTes\
t\x22,\x22MarginalDist\
ribution\x22,\x22Marko\
vProcessProperti\
es\x22,\x22Masking\x22,\x22M\
assConcentration\
Condition\x22,\x22Mass\
FluxValue\x22,\x22Mass\
ImpermeableBound\
aryValue\x22,\x22MassO\
utflowValue\x22,\x22Ma\
ssSymmetryValue\x22\
,\x22MassTransferVa\
lue\x22,\x22MassTransp\
ortPDEComponent\x22\
,\x22MatchingDissim\
ilarity\x22,\x22MatchL\
ocalNameQ\x22,\x22Matc\
hLocalNames\x22,\x22Ma\
tchQ\x22,\x22Material\x22\
,\x22MaterialShadin\
g\x22,\x22MaternPointP\
rocess\x22,\x22Mathema\
ticalFunctionDat\
a\x22,\x22MathematicaN\
otation\x22,\x22Mathie\
uC\x22,\x22MathieuChar\
acteristicA\x22,\x22Ma\
thieuCharacteris\
ticB\x22,\x22MathieuCh\
aracteristicExpo\
nent\x22,\x22MathieuCP\
rime\x22,\x22MathieuGr\
oupM11\x22,\x22Mathieu\
GroupM12\x22,\x22Mathi\
euGroupM22\x22,\x22Mat\
hieuGroupM23\x22,\x22M\
athieuGroupM24\x22,\
\x22MathieuS\x22,\x22Math\
ieuSPrime\x22,\x22Math\
MLForm\x22,\x22MathMLT\
ext\x22,\x22Matrices\x22,\
\x22MatrixExp\x22,\x22Mat\
rixForm\x22,\x22Matrix\
Function\x22,\x22Matri\
xLog\x22,\x22MatrixNor\
malDistribution\x22\
,\x22MatrixPlot\x22,\x22M\
atrixPower\x22,\x22Mat\
rixPropertyDistr\
ibution\x22,\x22Matrix\
Q\x22,\x22MatrixRank\x22,\
\x22MatrixTDistribu\
tion\x22,\x22Max\x22,\x22Max\
Bend\x22,\x22MaxCellMe\
asure\x22,\x22MaxColor\
Distance\x22,\x22MaxDa\
te\x22,\x22MaxDetect\x22,\
\x22MaxDisplayedChi\
ldren\x22,\x22MaxDurat\
ion\x22,\x22MaxExtraBa\
ndwidths\x22,\x22MaxEx\
traConditions\x22,\x22\
MaxFeatureDispla\
cement\x22,\x22MaxFeat\
ures\x22,\x22MaxFilter\
\x22,\x22MaximalBy\x22,\x22M\
aximize\x22,\x22MaxIte\
ms\x22,\x22MaxIteratio\
ns\x22,\x22MaxLimit\x22,\x22\
MaxMemoryUsed\x22,\x22\
MaxMixtureKernel\
s\x22,\x22MaxOverlapFr\
action\x22,\x22MaxPlot\
Points\x22,\x22MaxPoin\
ts\x22,\x22MaxRecursio\
n\x22,\x22MaxStableDis\
tribution\x22,\x22MaxS\
tepFraction\x22,\x22Ma\
xSteps\x22,\x22MaxStep\
Size\x22,\x22MaxTraini\
ngRounds\x22,\x22MaxVa\
lue\x22,\x22MaxwellDis\
tribution\x22,\x22MaxW\
ordGap\x22,\x22McLaugh\
linGroupMcL\x22,\x22Me\
an\x22,\x22MeanAbsolut\
eLossLayer\x22,\x22Mea\
nAround\x22,\x22MeanCl\
usteringCoeffici\
ent\x22,\x22MeanDegree\
Connectivity\x22,\x22M\
eanDeviation\x22,\x22M\
eanFilter\x22,\x22Mean\
GraphDistance\x22,\x22\
MeanNeighborDegr\
ee\x22,\x22MeanPointDe\
nsity\x22,\x22MeanShif\
t\x22,\x22MeanShiftFil\
ter\x22,\x22MeanSquare\
dLossLayer\x22,\x22Med\
ian\x22,\x22MedianDevi\
ation\x22,\x22MedianFi\
lter\x22,\x22MedicalTe\
stData\x22,\x22Medium\x22\
,\x22MeijerG\x22,\x22Meij\
erGReduce\x22,\x22Meix\
nerDistribution\x22\
,\x22MellinConvolve\
\x22,\x22MellinTransfo\
rm\x22,\x22MemberQ\x22,\x22M\
emoryAvailable\x22,\
\x22MemoryConstrain\
ed\x22,\x22MemoryConst\
raint\x22,\x22MemoryIn\
Use\x22,\x22MengerMesh\
\x22,\x22Menu\x22,\x22MenuAp\
pearance\x22,\x22MenuC\
ommandKey\x22,\x22Menu\
Evaluator\x22,\x22Menu\
Item\x22,\x22MenuList\x22\
,\x22MenuPacket\x22,\x22M\
enuSortingValue\x22\
,\x22MenuStyle\x22,\x22Me\
nuView\x22,\x22Merge\x22,\
\x22MergeDifference\
s\x22,\x22MergingFunct\
ion\x22,\x22MersennePr\
imeExponent\x22,\x22Me\
rsennePrimeExpon\
entQ\x22,\x22Mesh\x22,\x22Me\
shCellCentroid\x22,\
\x22MeshCellCount\x22,\
\x22MeshCellHighlig\
ht\x22,\x22MeshCellInd\
ex\x22,\x22MeshCellLab\
el\x22,\x22MeshCellMar\
ker\x22,\x22MeshCellMe\
asure\x22,\x22MeshCell\
Quality\x22,\x22MeshCe\
lls\x22,\x22MeshCellSh\
apeFunction\x22,\x22Me\
shCellStyle\x22,\x22Me\
shConnectivityGr\
aph\x22,\x22MeshCoordi\
nates\x22,\x22MeshFunc\
tions\x22,\x22MeshPrim\
itives\x22,\x22MeshQua\
lityGoal\x22,\x22MeshR\
ange\x22,\x22MeshRefin\
ementFunction\x22,\x22\
MeshRegion\x22,\x22Mes\
hRegionQ\x22,\x22MeshS\
hading\x22,\x22MeshSty\
le\x22,\x22Message\x22,\x22M\
essageDialog\x22,\x22M\
essageList\x22,\x22Mes\
sageName\x22,\x22Messa\
geObject\x22,\x22Messa\
geOptions\x22,\x22Mess\
agePacket\x22,\x22Mess\
ages\x22,\x22MessagesN\
otebook\x22,\x22MetaCh\
aracters\x22,\x22MetaI\
nformation\x22,\x22Met\
eorShowerData\x22,\x22\
Method\x22,\x22MethodO\
ptions\x22,\x22Mexican\
HatWavelet\x22,\x22Mey\
erWavelet\x22,\x22Midp\
oint\x22,\x22MIMETypeT\
oFormatList\x22,\x22Mi\
n\x22,\x22MinColorDist\
ance\x22,\x22MinDate\x22,\
\x22MinDetect\x22,\x22Min\
eralData\x22,\x22MinFi\
lter\x22,\x22MinimalBy\
\x22,\x22MinimalPolyno\
mial\x22,\x22MinimalSt\
ateSpaceModel\x22,\x22\
Minimize\x22,\x22Minim\
umTimeIncrement\x22\
,\x22MinIntervalSiz\
e\x22,\x22MinkowskiQue\
stionMark\x22,\x22MinL\
imit\x22,\x22MinMax\x22,\x22\
MinorPlanetData\x22\
,\x22Minors\x22,\x22MinPo\
intSeparation\x22,\x22\
MinRecursion\x22,\x22M\
inSize\x22,\x22MinStab\
leDistribution\x22,\
\x22Minus\x22,\x22MinusPl\
us\x22,\x22MinValue\x22,\x22\
Missing\x22,\x22Missin\
gBehavior\x22,\x22Miss\
ingDataMethod\x22,\x22\
MissingDataRules\
\x22,\x22MissingQ\x22,\x22Mi\
ssingString\x22,\x22Mi\
ssingStyle\x22,\x22Mis\
singValuePattern\
\x22,\x22MissingValueS\
ynthesis\x22,\x22Mitta\
gLefflerE\x22,\x22Mixe\
dFractionParts\x22,\
\x22MixedGraphQ\x22,\x22M\
ixedMagnitude\x22,\x22\
MixedRadix\x22,\x22Mix\
edRadixQuantity\x22\
,\x22MixedUnit\x22,\x22Mi\
xtureDistributio\
n\x22,\x22Mod\x22,\x22Modal\x22\
,\x22Mode\x22,\x22ModelPr\
edictiveControll\
er\x22,\x22Modular\x22,\x22M\
odularInverse\x22,\x22\
ModularLambda\x22,\x22\
Module\x22,\x22Modulus\
\x22,\x22MoebiusMu\x22,\x22M\
olecule\x22,\x22Molecu\
leAlign\x22,\x22Molecu\
leContainsQ\x22,\x22Mo\
leculeDraw\x22,\x22Mol\
eculeEquivalentQ\
\x22,\x22MoleculeFreeQ\
\x22,\x22MoleculeGraph\
\x22,\x22MoleculeMatch\
Q\x22,\x22MoleculeMaxi\
mumCommonSubstru\
cture\x22,\x22Molecule\
Modify\x22,\x22Molecul\
eName\x22,\x22Molecule\
Pattern\x22,\x22Molecu\
lePlot\x22,\x22Molecul\
ePlot3D\x22,\x22Molecu\
leProperty\x22,\x22Mol\
eculeQ\x22,\x22Molecul\
eRecognize\x22,\x22Mol\
eculeSubstructur\
eCount\x22,\x22Molecul\
eValue\x22,\x22Moment\x22\
,\x22MomentConvert\x22\
,\x22MomentEvaluate\
\x22,\x22MomentGenerat\
ingFunction\x22,\x22Mo\
mentOfInertia\x22,\x22\
Monday\x22,\x22Monitor\
\x22,\x22MonomialList\x22\
,\x22MonomialOrder\x22\
,\x22MonsterGroupM\x22\
,\x22MoonPhase\x22,\x22Mo\
onPosition\x22,\x22Mor\
letWavelet\x22,\x22Mor\
phologicalBinari\
ze\x22,\x22Morphologic\
alBranchPoints\x22,\
\x22MorphologicalCo\
mponents\x22,\x22Morph\
ologicalEulerNum\
ber\x22,\x22Morphologi\
calGraph\x22,\x22Morph\
ologicalPerimete\
r\x22,\x22Morphologica\
lTransform\x22,\x22Mor\
talityData\x22,\x22Mos\
t\x22,\x22MountainData\
\x22,\x22MouseAnnotati\
on\x22,\x22MouseAppear\
ance\x22,\x22MouseAppe\
aranceTag\x22,\x22Mous\
eButtons\x22,\x22Mouse\
over\x22,\x22MousePoin\
terNote\x22,\x22MouseP\
osition\x22,\x22MovieD\
ata\x22,\x22MovingAver\
age\x22,\x22MovingMap\x22\
,\x22MovingMedian\x22,\
\x22MoyalDistributi\
on\x22,\x22MultiaxisAr\
rangement\x22,\x22Mult\
icolumn\x22,\x22Multie\
dgeStyle\x22,\x22Multi\
graphQ\x22,\x22Multila\
unchWarning\x22,\x22Mu\
ltiLetterItalics\
\x22,\x22MultiLetterSt\
yle\x22,\x22MultilineF\
unction\x22,\x22Multin\
omial\x22,\x22Multinom\
ialDistribution\x22\
,\x22MultinormalDis\
tribution\x22,\x22Mult\
iplicativeOrder\x22\
,\x22Multiplicity\x22,\
\x22MultiplySides\x22,\
\x22MultiscriptBoxO\
ptions\x22,\x22Multise\
lection\x22,\x22Multiv\
ariateHypergeome\
tricDistribution\
\x22,\x22MultivariateP\
oissonDistributi\
on\x22,\x22Multivariat\
eTDistribution\x22,\
\x22N\x22,\x22NakagamiDis\
tribution\x22,\x22Name\
Q\x22,\x22Names\x22,\x22Name\
spaceBox\x22,\x22Names\
paceBoxOptions\x22,\
\x22Nand\x22,\x22NArgMax\x22\
,\x22NArgMin\x22,\x22NBer\
noulliB\x22,\x22NBodyS\
imulation\x22,\x22NBod\
ySimulationData\x22\
,\x22NCache\x22,\x22NCapu\
toD\x22,\x22NDEigensys\
tem\x22,\x22NDEigenval\
ues\x22,\x22NDSolve\x22,\x22\
NDSolveValue\x22,\x22N\
earest\x22,\x22Nearest\
Function\x22,\x22Neare\
stMeshCells\x22,\x22Ne\
arestNeighborG\x22,\
\x22NearestNeighbor\
Graph\x22,\x22NearestT\
o\x22,\x22NebulaData\x22,\
\x22NeedlemanWunsch\
Similarity\x22,\x22Nee\
ds\x22,\x22Negative\x22,\x22\
NegativeBinomial\
Distribution\x22,\x22N\
egativeDefiniteM\
atrixQ\x22,\x22Negativ\
eIntegers\x22,\x22Nega\
tivelyOrientedPo\
ints\x22,\x22NegativeM\
ultinomialDistri\
bution\x22,\x22Negativ\
eRationals\x22,\x22Neg\
ativeReals\x22,\x22Neg\
ativeSemidefinit\
eMatrixQ\x22,\x22Neigh\
borhoodData\x22,\x22Ne\
ighborhoodGraph\x22\
,\x22Nest\x22,\x22NestedG\
reaterGreater\x22,\x22\
NestedLessLess\x22,\
\x22NestedScriptRul\
es\x22,\x22NestGraph\x22,\
\x22NestList\x22,\x22Nest\
Tree\x22,\x22NestWhile\
\x22,\x22NestWhileList\
\x22,\x22NetAppend\x22,\x22N\
etArray\x22,\x22NetArr\
ayLayer\x22,\x22NetBid\
irectionalOperat\
or\x22,\x22NetChain\x22,\x22\
NetDecoder\x22,\x22Net\
Delete\x22,\x22NetDrop\
\x22,\x22NetEncoder\x22,\x22\
NetEvaluationMod\
e\x22,\x22NetExternalO\
bject\x22,\x22NetExtra\
ct\x22,\x22NetFlatten\x22\
,\x22NetFoldOperato\
r\x22,\x22NetGANOperat\
or\x22,\x22NetGraph\x22,\x22\
NetInformation\x22,\
\x22NetInitialize\x22,\
\x22NetInsert\x22,\x22Net\
InsertSharedArra\
ys\x22,\x22NetJoin\x22,\x22N\
etMapOperator\x22,\x22\
NetMapThreadOper\
ator\x22,\x22NetMeasur\
ements\x22,\x22NetMode\
l\x22,\x22NetNestOpera\
tor\x22,\x22NetPairEmb\
eddingOperator\x22,\
\x22NetPort\x22,\x22NetPo\
rtGradient\x22,\x22Net\
Prepend\x22,\x22NetRen\
ame\x22,\x22NetReplace\
\x22,\x22NetReplacePar\
t\x22,\x22NetSharedArr\
ay\x22,\x22NetStateObj\
ect\x22,\x22NetTake\x22,\x22\
NetTrain\x22,\x22NetTr\
ainResultsObject\
\x22,\x22NetUnfold\x22,\x22N\
etworkPacketCapt\
ure\x22,\x22NetworkPac\
ketRecording\x22,\x22N\
etworkPacketReco\
rdingDuring\x22,\x22Ne\
tworkPacketTrace\
\x22,\x22NeumannValue\x22\
,\x22NevilleThetaC\x22\
,\x22NevilleThetaD\x22\
,\x22NevilleThetaN\x22\
,\x22NevilleThetaS\x22\
,\x22NewPrimitiveSt\
yle\x22,\x22NExpectati\
on\x22,\x22Next\x22,\x22Next\
Cell\x22,\x22NextDate\x22\
,\x22NextPrime\x22,\x22Ne\
xtScheduledTaskT\
ime\x22,\x22NeymanScot\
tPointProcess\x22,\x22\
NFractionalD\x22,\x22N\
HoldAll\x22,\x22NHoldF\
irst\x22,\x22NHoldRest\
\x22,\x22NicholsGridLi\
nes\x22,\x22NicholsPlo\
t\x22,\x22NightHemisph\
ere\x22,\x22NIntegrate\
\x22,\x22NMaximize\x22,\x22N\
MaxValue\x22,\x22NMini\
mize\x22,\x22NMinValue\
\x22,\x22NominalScale\x22\
,\x22NominalVariabl\
es\x22,\x22NonAssociat\
ive\x22,\x22Noncentral\
BetaDistribution\
\x22,\x22NoncentralChi\
SquareDistributi\
on\x22,\x22NoncentralF\
RatioDistributio\
n\x22,\x22NoncentralSt\
udentTDistributi\
on\x22,\x22NonCommutat\
iveMultiply\x22,\x22No\
nConstants\x22,\x22Non\
dimensionalizati\
onTransform\x22,\x22No\
ne\x22,\x22NoneTrue\x22,\x22\
NonlinearModelFi\
t\x22,\x22NonlinearSta\
teSpaceModel\x22,\x22N\
onlocalMeansFilt\
er\x22,\x22NonNegative\
\x22,\x22NonNegativeIn\
tegers\x22,\x22NonNega\
tiveRationals\x22,\x22\
NonNegativeReals\
\x22,\x22NonPositive\x22,\
\x22NonPositiveInte\
gers\x22,\x22NonPositi\
veRationals\x22,\x22No\
nPositiveReals\x22,\
\x22Nor\x22,\x22NorlundB\x22\
,\x22Norm\x22,\x22Normal\x22\
,\x22NormalDistribu\
tion\x22,\x22NormalGro\
uping\x22,\x22Normaliz\
ationLayer\x22,\x22Nor\
malize\x22,\x22Normali\
zed\x22,\x22Normalized\
SquaredEuclidean\
Distance\x22,\x22Norma\
lMatrixQ\x22,\x22Norma\
lsFunction\x22,\x22Nor\
mFunction\x22,\x22Not\x22\
,\x22NotCongruent\x22,\
\x22NotCupCap\x22,\x22Not\
DoubleVerticalBa\
r\x22,\x22Notebook\x22,\x22N\
otebookApply\x22,\x22N\
otebookAutoSave\x22\
,\x22NotebookBrowse\
Directory\x22,\x22Note\
bookClose\x22,\x22Note\
bookConvertSetti\
ngs\x22,\x22NotebookCr\
eate\x22,\x22NotebookD\
efault\x22,\x22Noteboo\
kDelete\x22,\x22Notebo\
okDirectory\x22,\x22No\
tebookDynamicExp\
ression\x22,\x22Notebo\
okEvaluate\x22,\x22Not\
ebookEventAction\
s\x22,\x22NotebookFile\
Name\x22,\x22NotebookF\
ind\x22,\x22NotebookGe\
t\x22,\x22NotebookImpo\
rt\x22,\x22NotebookInf\
ormation\x22,\x22Noteb\
ookInterfaceObje\
ct\x22,\x22NotebookLoc\
ate\x22,\x22NotebookOb\
ject\x22,\x22NotebookO\
pen\x22,\x22NotebookPa\
th\x22,\x22NotebookPri\
nt\x22,\x22NotebookPut\
\x22,\x22NotebookRead\x22\
,\x22Notebooks\x22,\x22No\
tebookSave\x22,\x22Not\
ebookSelection\x22,\
\x22NotebooksMenu\x22,\
\x22NotebookTemplat\
e\x22,\x22NotebookWrit\
e\x22,\x22NotElement\x22,\
\x22NotEqualTilde\x22,\
\x22NotExists\x22,\x22Not\
Greater\x22,\x22NotGre\
aterEqual\x22,\x22NotG\
reaterFullEqual\x22\
,\x22NotGreaterGrea\
ter\x22,\x22NotGreater\
Less\x22,\x22NotGreate\
rSlantEqual\x22,\x22No\
tGreaterTilde\x22,\x22\
Nothing\x22,\x22NotHum\
pDownHump\x22,\x22NotH\
umpEqual\x22,\x22Notif\
icationFunction\x22\
,\x22NotLeftTriangl\
e\x22,\x22NotLeftTrian\
gleBar\x22,\x22NotLeft\
TriangleEqual\x22,\x22\
NotLess\x22,\x22NotLes\
sEqual\x22,\x22NotLess\
FullEqual\x22,\x22NotL\
essGreater\x22,\x22Not\
LessLess\x22,\x22NotLe\
ssSlantEqual\x22,\x22N\
otLessTilde\x22,\x22No\
tNestedGreaterGr\
eater\x22,\x22NotNeste\
dLessLess\x22,\x22NotP\
recedes\x22,\x22NotPre\
cedesEqual\x22,\x22Not\
PrecedesSlantEqu\
al\x22,\x22NotPrecedes\
Tilde\x22,\x22NotRever\
seElement\x22,\x22NotR\
ightTriangle\x22,\x22N\
otRightTriangleB\
ar\x22,\x22NotRightTri\
angleEqual\x22,\x22Not\
SquareSubset\x22,\x22N\
otSquareSubsetEq\
ual\x22,\x22NotSquareS\
uperset\x22,\x22NotSqu\
areSupersetEqual\
\x22,\x22NotSubset\x22,\x22N\
otSubsetEqual\x22,\x22\
NotSucceeds\x22,\x22No\
tSucceedsEqual\x22,\
\x22NotSucceedsSlan\
tEqual\x22,\x22NotSucc\
eedsTilde\x22,\x22NotS\
uperset\x22,\x22NotSup\
ersetEqual\x22,\x22Not\
Tilde\x22,\x22NotTilde\
Equal\x22,\x22NotTilde\
FullEqual\x22,\x22NotT\
ildeTilde\x22,\x22NotV\
erticalBar\x22,\x22Now\
\x22,\x22NoWhitespace\x22\
,\x22NProbability\x22,\
\x22NProduct\x22,\x22NPro\
ductFactors\x22,\x22NR\
oots\x22,\x22NSolve\x22,\x22\
NSolveValues\x22,\x22N\
Sum\x22,\x22NSumTerms\x22\
,\x22NuclearExplosi\
onData\x22,\x22Nuclear\
ReactorData\x22,\x22Nu\
ll\x22,\x22NullRecords\
\x22,\x22NullSpace\x22,\x22N\
ullWords\x22,\x22Numbe\
r\x22,\x22NumberCompos\
e\x22,\x22NumberDecomp\
ose\x22,\x22NumberDigi\
t\x22,\x22NumberExpand\
\x22,\x22NumberFieldCl\
assNumber\x22,\x22Numb\
erFieldDiscrimin\
ant\x22,\x22NumberFiel\
dFundamentalUnit\
s\x22,\x22NumberFieldI\
ntegralBasis\x22,\x22N\
umberFieldNormRe\
presentatives\x22,\x22\
NumberFieldRegul\
ator\x22,\x22NumberFie\
ldRootsOfUnity\x22,\
\x22NumberFieldSign\
ature\x22,\x22NumberFo\
rm\x22,\x22NumberForma\
t\x22,\x22NumberLinePl\
ot\x22,\x22NumberMarks\
\x22,\x22NumberMultipl\
ier\x22,\x22NumberPadd\
ing\x22,\x22NumberPoin\
t\x22,\x22NumberQ\x22,\x22Nu\
mberSeparator\x22,\x22\
NumberSigns\x22,\x22Nu\
mberString\x22,\x22Num\
erator\x22,\x22Numerat\
orDenominator\x22,\x22\
NumericalOrder\x22,\
\x22NumericalSort\x22,\
\x22NumericArray\x22,\x22\
NumericArrayQ\x22,\x22\
NumericArrayType\
\x22,\x22NumericFuncti\
on\x22,\x22NumericQ\x22,\x22\
NuttallWindow\x22,\x22\
NValues\x22,\x22Nyquis\
tGridLines\x22,\x22Nyq\
uistPlot\x22,\x22O\x22,\x22O\
bjectExistsQ\x22,\x22O\
bservabilityGram\
ian\x22,\x22Observabil\
ityMatrix\x22,\x22Obse\
rvableDecomposit\
ion\x22,\x22Observable\
ModelQ\x22,\x22OceanDa\
ta\x22,\x22Octahedron\x22\
,\x22OddQ\x22,\x22Off\x22,\x22O\
ffset\x22,\x22OLEData\x22\
,\x22On\x22,\x22ONanGroup\
ON\x22,\x22Once\x22,\x22OneI\
dentity\x22,\x22Opacit\
y\x22,\x22OpacityFunct\
ion\x22,\x22OpacityFun\
ctionScaling\x22,\x22O\
pen\x22,\x22OpenAppend\
\x22,\x22Opener\x22,\x22Open\
erBox\x22,\x22OpenerBo\
xOptions\x22,\x22Opene\
rView\x22,\x22OpenFunc\
tionInspectorPac\
ket\x22,\x22Opening\x22,\x22\
OpenRead\x22,\x22OpenS\
pecialOptions\x22,\x22\
OpenTemporary\x22,\x22\
OpenWrite\x22,\x22Oper\
ate\x22,\x22OperatingS\
ystem\x22,\x22Operator\
Applied\x22,\x22Optimu\
mFlowData\x22,\x22Opti\
onal\x22,\x22OptionalE\
lement\x22,\x22OptionI\
nspectorSettings\
\x22,\x22OptionQ\x22,\x22Opt\
ions\x22,\x22OptionsPa\
cket\x22,\x22OptionsPa\
ttern\x22,\x22OptionVa\
lue\x22,\x22OptionValu\
eBox\x22,\x22OptionVal\
ueBoxOptions\x22,\x22O\
r\x22,\x22Orange\x22,\x22Ord\
er\x22,\x22OrderDistri\
bution\x22,\x22Ordered\
Q\x22,\x22Ordering\x22,\x22O\
rderingBy\x22,\x22Orde\
ringLayer\x22,\x22Orde\
rless\x22,\x22Orderles\
sPatternSequence\
\x22,\x22OrdinalScale\x22\
,\x22OrnsteinUhlenb\
eckProcess\x22,\x22Ort\
hogonalize\x22,\x22Ort\
hogonalMatrixQ\x22,\
\x22Out\x22,\x22Outer\x22,\x22O\
uterPolygon\x22,\x22Ou\
terPolyhedron\x22,\x22\
OutputAutoOverwr\
ite\x22,\x22OutputCont\
rollabilityMatri\
x\x22,\x22OutputContro\
llableModelQ\x22,\x22O\
utputForm\x22,\x22Outp\
utFormData\x22,\x22Out\
putGrouping\x22,\x22Ou\
tputMathEditExpr\
ession\x22,\x22OutputN\
amePacket\x22,\x22Outp\
utPorts\x22,\x22Output\
Response\x22,\x22Outpu\
tSizeLimit\x22,\x22Out\
putStream\x22,\x22Over\
\x22,\x22OverBar\x22,\x22Ove\
rDot\x22,\x22Overflow\x22\
,\x22OverHat\x22,\x22Over\
laps\x22,\x22Overlay\x22,\
\x22OverlayBox\x22,\x22Ov\
erlayBoxOptions\x22\
,\x22OverlayVideo\x22,\
\x22Overscript\x22,\x22Ov\
erscriptBox\x22,\x22Ov\
erscriptBoxOptio\
ns\x22,\x22OverTilde\x22,\
\x22OverVector\x22,\x22Ov\
erwriteTarget\x22,\x22\
OwenT\x22,\x22OwnValue\
s\x22,\x22Package\x22,\x22Pa\
ckingMethod\x22,\x22Pa\
ckPaclet\x22,\x22Pacle\
tDataRebuild\x22,\x22P\
acletDirectoryAd\
d\x22,\x22PacletDirect\
oryLoad\x22,\x22Paclet\
DirectoryRemove\x22\
,\x22PacletDirector\
yUnload\x22,\x22Paclet\
Disable\x22,\x22Paclet\
Enable\x22,\x22PacletF\
ind\x22,\x22PacletFind\
Remote\x22,\x22PacletI\
nformation\x22,\x22Pac\
letInstall\x22,\x22Pac\
letInstallSubmit\
\x22,\x22PacletNewerQ\x22\
,\x22PacletObject\x22,\
\x22PacletObjectQ\x22,\
\x22PacletSite\x22,\x22Pa\
cletSiteObject\x22,\
\x22PacletSiteRegis\
ter\x22,\x22PacletSite\
s\x22,\x22PacletSiteUn\
register\x22,\x22Pacle\
tSiteUpdate\x22,\x22Pa\
cletSymbol\x22,\x22Pac\
letUninstall\x22,\x22P\
acletUpdate\x22,\x22Pa\
ddedForm\x22,\x22Paddi\
ng\x22,\x22PaddingLaye\
r\x22,\x22PaddingSize\x22\
,\x22PadeApproximan\
t\x22,\x22PadLeft\x22,\x22Pa\
dRight\x22,\x22PageBre\
akAbove\x22,\x22PageBr\
eakBelow\x22,\x22PageB\
reakWithin\x22,\x22Pag\
eFooterLines\x22,\x22P\
ageFooters\x22,\x22Pag\
eHeaderLines\x22,\x22P\
ageHeaders\x22,\x22Pag\
eHeight\x22,\x22PageRa\
nkCentrality\x22,\x22P\
ageTheme\x22,\x22PageW\
idth\x22,\x22Paginatio\
n\x22,\x22PairCorrelat\
ionG\x22,\x22PairedBar\
Chart\x22,\x22PairedHi\
stogram\x22,\x22Paired\
SmoothHistogram\x22\
,\x22PairedTTest\x22,\x22\
PairedZTest\x22,\x22Pa\
letteNotebook\x22,\x22\
PalettePath\x22,\x22Pa\
lettesMenuSettin\
gs\x22,\x22PalindromeQ\
\x22,\x22Pane\x22,\x22PaneBo\
x\x22,\x22PaneBoxOptio\
ns\x22,\x22Panel\x22,\x22Pan\
elBox\x22,\x22PanelBox\
Options\x22,\x22Panele\
d\x22,\x22PaneSelector\
\x22,\x22PaneSelectorB\
ox\x22,\x22PaneSelecto\
rBoxOptions\x22,\x22Pa\
perWidth\x22,\x22Parab\
olicCylinderD\x22,\x22\
ParagraphIndent\x22\
,\x22ParagraphSpaci\
ng\x22,\x22ParallelArr\
ay\x22,\x22ParallelAxi\
sPlot\x22,\x22Parallel\
Combine\x22,\x22Parall\
elDo\x22,\x22Parallele\
piped\x22,\x22Parallel\
Evaluate\x22,\x22Paral\
lelization\x22,\x22Par\
allelize\x22,\x22Paral\
lelKernels\x22,\x22Par\
allelMap\x22,\x22Paral\
lelNeeds\x22,\x22Paral\
lelogram\x22,\x22Paral\
lelProduct\x22,\x22Par\
allelSubmit\x22,\x22Pa\
rallelSum\x22,\x22Para\
llelTable\x22,\x22Para\
llelTry\x22,\x22Parame\
ter\x22,\x22ParameterE\
stimator\x22,\x22Param\
eterMixtureDistr\
ibution\x22,\x22Parame\
terVariables\x22,\x22P\
arametricConvexO\
ptimization\x22,\x22Pa\
rametricFunction\
\x22,\x22ParametricNDS\
olve\x22,\x22Parametri\
cNDSolveValue\x22,\x22\
ParametricPlot\x22,\
\x22ParametricPlot3\
D\x22,\x22ParametricRa\
mpLayer\x22,\x22Parame\
tricRegion\x22,\x22Par\
entBox\x22,\x22ParentC\
ell\x22,\x22ParentConn\
ect\x22,\x22ParentDire\
ctory\x22,\x22ParentEd\
geLabel\x22,\x22Parent\
EdgeLabelFunctio\
n\x22,\x22ParentEdgeLa\
belStyle\x22,\x22Paren\
tEdgeShapeFuncti\
on\x22,\x22ParentEdgeS\
tyle\x22,\x22ParentEdg\
eStyleFunction\x22,\
\x22ParentForm\x22,\x22Pa\
renthesize\x22,\x22Par\
entList\x22,\x22Parent\
Notebook\x22,\x22Paret\
oDistribution\x22,\x22\
ParetoPickandsDi\
stribution\x22,\x22Par\
kData\x22,\x22Part\x22,\x22P\
artBehavior\x22,\x22Pa\
rtialCorrelation\
Function\x22,\x22Parti\
alD\x22,\x22ParticleAc\
celeratorData\x22,\x22\
ParticleData\x22,\x22P\
artition\x22,\x22Parti\
tionGranularity\x22\
,\x22PartitionsP\x22,\x22\
PartitionsQ\x22,\x22Pa\
rtLayer\x22,\x22PartOf\
Speech\x22,\x22PartPro\
tection\x22,\x22Parzen\
Window\x22,\x22PascalD\
istribution\x22,\x22Pa\
ssEventsDown\x22,\x22P\
assEventsUp\x22,\x22Pa\
ste\x22,\x22PasteAutoQ\
uoteCharacters\x22,\
\x22PasteBoxFormInl\
ineCells\x22,\x22Paste\
Button\x22,\x22Path\x22,\x22\
PathGraph\x22,\x22Path\
GraphQ\x22,\x22Pattern\
\x22,\x22PatternFillin\
g\x22,\x22PatternReact\
ion\x22,\x22PatternSeq\
uence\x22,\x22PatternT\
est\x22,\x22PauliMatri\
x\x22,\x22PaulWavelet\x22\
,\x22Pause\x22,\x22Paused\
Time\x22,\x22PDF\x22,\x22Pea\
kDetect\x22,\x22PeanoC\
urve\x22,\x22PearsonCh\
iSquareTest\x22,\x22Pe\
arsonCorrelation\
Test\x22,\x22PearsonDi\
stribution\x22,\x22Pen\
ttinenPointProce\
ss\x22,\x22PercentForm\
\x22,\x22PerfectNumber\
\x22,\x22PerfectNumber\
Q\x22,\x22PerformanceG\
oal\x22,\x22Perimeter\x22\
,\x22PeriodicBounda\
ryCondition\x22,\x22Pe\
riodicInterpolat\
ion\x22,\x22Periodogra\
m\x22,\x22PeriodogramA\
rray\x22,\x22Permanent\
\x22,\x22Permissions\x22,\
\x22PermissionsGrou\
p\x22,\x22PermissionsG\
roupMemberQ\x22,\x22Pe\
rmissionsGroups\x22\
,\x22PermissionsKey\
\x22,\x22PermissionsKe\
ys\x22,\x22Permutation\
Cycles\x22,\x22Permuta\
tionCyclesQ\x22,\x22Pe\
rmutationGroup\x22,\
\x22PermutationLeng\
th\x22,\x22Permutation\
List\x22,\x22Permutati\
onListQ\x22,\x22Permut\
ationMatrix\x22,\x22Pe\
rmutationMax\x22,\x22P\
ermutationMin\x22,\x22\
PermutationOrder\
\x22,\x22PermutationPo\
wer\x22,\x22Permutatio\
nProduct\x22,\x22Permu\
tationReplace\x22,\x22\
Permutations\x22,\x22P\
ermutationSuppor\
t\x22,\x22Permute\x22,\x22Pe\
ronaMalikFilter\x22\
,\x22Perpendicular\x22\
,\x22PerpendicularB\
isector\x22,\x22Persis\
tenceLocation\x22,\x22\
PersistenceTime\x22\
,\x22PersistentObje\
ct\x22,\x22PersistentO\
bjects\x22,\x22Persist\
entSymbol\x22,\x22Pers\
istentValue\x22,\x22Pe\
rsonData\x22,\x22PERTD\
istribution\x22,\x22Pe\
tersenGraph\x22,\x22Ph\
aseMargins\x22,\x22Pha\
seRange\x22,\x22PhongS\
hading\x22,\x22Physica\
lSystemData\x22,\x22Pi\
\x22,\x22Pick\x22,\x22Picked\
Elements\x22,\x22PickM\
ode\x22,\x22PIDData\x22,\x22\
PIDDerivativeFil\
ter\x22,\x22PIDFeedfor\
ward\x22,\x22PIDTune\x22,\
\x22Piecewise\x22,\x22Pie\
cewiseExpand\x22,\x22P\
ieChart\x22,\x22PieCha\
rt3D\x22,\x22PillaiTra\
ce\x22,\x22PillaiTrace\
Test\x22,\x22PingTime\x22\
,\x22Pink\x22,\x22PitchRe\
cognize\x22,\x22Pivoti\
ng\x22,\x22PixelConstr\
ained\x22,\x22PixelVal\
ue\x22,\x22PixelValueP\
ositions\x22,\x22Place\
d\x22,\x22Placeholder\x22\
,\x22PlaceholderLay\
er\x22,\x22Placeholder\
Replace\x22,\x22Plain\x22\
,\x22PlanarAngle\x22,\x22\
PlanarFaceList\x22,\
\x22PlanarGraph\x22,\x22P\
lanarGraphQ\x22,\x22Pl\
anckRadiationLaw\
\x22,\x22PlaneCurveDat\
a\x22,\x22PlanetaryMoo\
nData\x22,\x22PlanetDa\
ta\x22,\x22PlantData\x22,\
\x22Play\x22,\x22Playback\
Settings\x22,\x22PlayR\
ange\x22,\x22Plot\x22,\x22Pl\
ot3D\x22,\x22Plot3Matr\
ix\x22,\x22PlotDivisio\
n\x22,\x22PlotJoined\x22,\
\x22PlotLabel\x22,\x22Plo\
tLabels\x22,\x22PlotLa\
yout\x22,\x22PlotLegen\
ds\x22,\x22PlotMarkers\
\x22,\x22PlotPoints\x22,\x22\
PlotRange\x22,\x22Plot\
RangeClipping\x22,\x22\
PlotRangeClipPla\
nesStyle\x22,\x22PlotR\
angePadding\x22,\x22Pl\
otRegion\x22,\x22PlotS\
tyle\x22,\x22PlotTheme\
\x22,\x22Pluralize\x22,\x22P\
lus\x22,\x22PlusMinus\x22\
,\x22Pochhammer\x22,\x22P\
odStates\x22,\x22PodWi\
dth\x22,\x22Point\x22,\x22Po\
int3DBox\x22,\x22Point\
3DBoxOptions\x22,\x22P\
ointBox\x22,\x22PointB\
oxOptions\x22,\x22Poin\
tCountDistributi\
on\x22,\x22PointDensit\
y\x22,\x22PointDensity\
Function\x22,\x22Point\
FigureChart\x22,\x22Po\
intLegend\x22,\x22Poin\
tLight\x22,\x22PointPr\
ocessEstimator\x22,\
\x22PointProcessFit\
Test\x22,\x22PointProc\
essParameterAssu\
mptions\x22,\x22PointP\
rocessParameterQ\
\x22,\x22PointSize\x22,\x22P\
ointStatisticFun\
ction\x22,\x22PointVal\
uePlot\x22,\x22Poisson\
ConsulDistributi\
on\x22,\x22PoissonDist\
ribution\x22,\x22Poiss\
onPDEComponent\x22,\
\x22PoissonPointPro\
cess\x22,\x22PoissonPr\
ocess\x22,\x22PoissonW\
indow\x22,\x22PolarAxe\
s\x22,\x22PolarAxesOri\
gin\x22,\x22PolarGridL\
ines\x22,\x22PolarPlot\
\x22,\x22PolarTicks\x22,\x22\
PoleZeroMarkers\x22\
,\x22PolyaAeppliDis\
tribution\x22,\x22Poly\
Gamma\x22,\x22Polygon\x22\
,\x22Polygon3DBox\x22,\
\x22Polygon3DBoxOpt\
ions\x22,\x22Polygonal\
Number\x22,\x22Polygon\
Angle\x22,\x22PolygonB\
ox\x22,\x22PolygonBoxO\
ptions\x22,\x22Polygon\
Coordinates\x22,\x22Po\
lygonDecompositi\
on\x22,\x22PolygonHole\
Scale\x22,\x22PolygonI\
ntersections\x22,\x22P\
olygonScale\x22,\x22Po\
lyhedron\x22,\x22Polyh\
edronAngle\x22,\x22Pol\
yhedronBox\x22,\x22Pol\
yhedronBoxOption\
s\x22,\x22PolyhedronCo\
ordinates\x22,\x22Poly\
hedronData\x22,\x22Pol\
yhedronDecomposi\
tion\x22,\x22Polyhedro\
nGenus\x22,\x22PolyLog\
\x22,\x22PolynomialExp\
ressionQ\x22,\x22Polyn\
omialExtendedGCD\
\x22,\x22PolynomialFor\
m\x22,\x22PolynomialGC\
D\x22,\x22PolynomialLC\
M\x22,\x22PolynomialMo\
d\x22,\x22PolynomialQ\x22\
,\x22PolynomialQuot\
ient\x22,\x22Polynomia\
lQuotientRemaind\
er\x22,\x22PolynomialR\
educe\x22,\x22Polynomi\
alRemainder\x22,\x22Po\
lynomials\x22,\x22Poly\
nomialSumOfSquar\
esList\x22,\x22Pooling\
Layer\x22,\x22PopupMen\
u\x22,\x22PopupMenuBox\
\x22,\x22PopupMenuBoxO\
ptions\x22,\x22PopupVi\
ew\x22,\x22PopupWindow\
\x22,\x22Position\x22,\x22Po\
sitionIndex\x22,\x22Po\
sitionLargest\x22,\x22\
PositionSmallest\
\x22,\x22Positive\x22,\x22Po\
sitiveDefiniteMa\
trixQ\x22,\x22Positive\
Integers\x22,\x22Posit\
ivelyOrientedPoi\
nts\x22,\x22PositiveRa\
tionals\x22,\x22Positi\
veReals\x22,\x22Positi\
veSemidefiniteMa\
trixQ\x22,\x22Possible\
ZeroQ\x22,\x22Postfix\x22\
,\x22PostScript\x22,\x22P\
ower\x22,\x22PowerDist\
ribution\x22,\x22Power\
Expand\x22,\x22PowerMo\
d\x22,\x22PowerModList\
\x22,\x22PowerRange\x22,\x22\
PowerSpectralDen\
sity\x22,\x22PowersRep\
resentations\x22,\x22P\
owerSymmetricPol\
ynomial\x22,\x22Preced\
ence\x22,\x22Precedenc\
eForm\x22,\x22Precedes\
\x22,\x22PrecedesEqual\
\x22,\x22PrecedesSlant\
Equal\x22,\x22Precedes\
Tilde\x22,\x22Precisio\
n\x22,\x22PrecisionGoa\
l\x22,\x22PreDecrement\
\x22,\x22Predict\x22,\x22Pre\
dictionRoot\x22,\x22Pr\
edictorFunction\x22\
,\x22PredictorInfor\
mation\x22,\x22Predict\
orMeasurements\x22,\
\x22PredictorMeasur\
ementsObject\x22,\x22P\
reemptProtect\x22,\x22\
PreferencesPath\x22\
,\x22PreferencesSet\
tings\x22,\x22Prefix\x22,\
\x22PreIncrement\x22,\x22\
Prepend\x22,\x22Prepen\
dLayer\x22,\x22Prepend\
To\x22,\x22Preprocessi\
ngRules\x22,\x22Preser\
veColor\x22,\x22Preser\
veImageOptions\x22,\
\x22Previous\x22,\x22Prev\
iousCell\x22,\x22Previ\
ousDate\x22,\x22PriceG\
raphDistribution\
\x22,\x22PrimaryPlaceh\
older\x22,\x22Prime\x22,\x22\
PrimeNu\x22,\x22PrimeO\
mega\x22,\x22PrimePi\x22,\
\x22PrimePowerQ\x22,\x22P\
rimeQ\x22,\x22Primes\x22,\
\x22PrimeZetaP\x22,\x22Pr\
imitivePolynomia\
lQ\x22,\x22PrimitiveRo\
ot\x22,\x22PrimitiveRo\
otList\x22,\x22Princip\
alComponents\x22,\x22P\
rincipalValue\x22,\x22\
Print\x22,\x22Printabl\
eASCIIQ\x22,\x22PrintA\
ction\x22,\x22PrintFor\
m\x22,\x22PrintingCopi\
es\x22,\x22PrintingOpt\
ions\x22,\x22PrintingP\
ageRange\x22,\x22Print\
ingStartingPageN\
umber\x22,\x22Printing\
StyleEnvironment\
\x22,\x22Printout3D\x22,\x22\
Printout3DPrevie\
wer\x22,\x22PrintPreci\
sion\x22,\x22PrintTemp\
orary\x22,\x22Prism\x22,\x22\
PrismBox\x22,\x22Prism\
BoxOptions\x22,\x22Pri\
vateCellOptions\x22\
,\x22PrivateEvaluat\
ionOptions\x22,\x22Pri\
vateFontOptions\x22\
,\x22PrivateFrontEn\
dOptions\x22,\x22Priva\
teKey\x22,\x22PrivateN\
otebookOptions\x22,\
\x22PrivatePaths\x22,\x22\
Probability\x22,\x22Pr\
obabilityDistrib\
ution\x22,\x22Probabil\
ityPlot\x22,\x22Probab\
ilityPr\x22,\x22Probab\
ilityScalePlot\x22,\
\x22ProbitModelFit\x22\
,\x22ProcessConnect\
ion\x22,\x22ProcessDir\
ectory\x22,\x22Process\
Environment\x22,\x22Pr\
ocesses\x22,\x22Proces\
sEstimator\x22,\x22Pro\
cessInformation\x22\
,\x22ProcessObject\x22\
,\x22ProcessParamet\
erAssumptions\x22,\x22\
ProcessParameter\
Q\x22,\x22ProcessState\
Domain\x22,\x22Process\
Status\x22,\x22Process\
TimeDomain\x22,\x22Pro\
duct\x22,\x22ProductDi\
stribution\x22,\x22Pro\
ductLog\x22,\x22Progre\
ssIndicator\x22,\x22Pr\
ogressIndicatorB\
ox\x22,\x22ProgressInd\
icatorBoxOptions\
\x22,\x22ProgressRepor\
ting\x22,\x22Projectio\
n\x22,\x22Prolog\x22,\x22Pro\
mptForm\x22,\x22ProofO\
bject\x22,\x22Propagat\
eAborts\x22,\x22Proper\
ties\x22,\x22Property\x22\
,\x22PropertyList\x22,\
\x22PropertyValue\x22,\
\x22Proportion\x22,\x22Pr\
oportional\x22,\x22Pro\
tect\x22,\x22Protected\
\x22,\x22ProteinData\x22,\
\x22Pruning\x22,\x22Pseud\
oInverse\x22,\x22Psych\
rometricProperty\
Data\x22,\x22PublicKey\
\x22,\x22PublisherID\x22,\
\x22PulsarData\x22,\x22Pu\
nctuationCharact\
er\x22,\x22Purple\x22,\x22Pu\
t\x22,\x22PutAppend\x22,\x22\
Pyramid\x22,\x22Pyrami\
dBox\x22,\x22PyramidBo\
xOptions\x22,\x22QBino\
mial\x22,\x22QFactoria\
l\x22,\x22QGamma\x22,\x22QHy\
pergeometricPFQ\x22\
,\x22QnDispersion\x22,\
\x22QPochhammer\x22,\x22Q\
PolyGamma\x22,\x22QRDe\
composition\x22,\x22Qu\
adraticIrrationa\
lQ\x22,\x22QuadraticOp\
timization\x22,\x22Qua\
ntile\x22,\x22Quantile\
Plot\x22,\x22Quantity\x22\
,\x22QuantityArray\x22\
,\x22QuantityDistri\
bution\x22,\x22Quantit\
yForm\x22,\x22Quantity\
Magnitude\x22,\x22Quan\
tityQ\x22,\x22Quantity\
Unit\x22,\x22QuantityV\
ariable\x22,\x22Quanti\
tyVariableCanoni\
calUnit\x22,\x22Quanti\
tyVariableDimens\
ions\x22,\x22QuantityV\
ariableIdentifie\
r\x22,\x22QuantityVari\
ablePhysicalQuan\
tity\x22,\x22Quartics\x22\
,\x22QuartileDeviat\
ion\x22,\x22Quartiles\x22\
,\x22QuartileSkewne\
ss\x22,\x22Query\x22,\x22Que\
stionGenerator\x22,\
\x22QuestionInterfa\
ce\x22,\x22QuestionObj\
ect\x22,\x22QuestionSe\
lector\x22,\x22Queuein\
gNetworkProcess\x22\
,\x22QueueingProces\
s\x22,\x22QueuePropert\
ies\x22,\x22Quiet\x22,\x22Qu\
ietEcho\x22,\x22Quit\x22,\
\x22Quotient\x22,\x22Quot\
ientRemainder\x22,\x22\
RadialAxisPlot\x22,\
\x22RadialGradientF\
illing\x22,\x22RadialG\
radientImage\x22,\x22R\
adialityCentrali\
ty\x22,\x22RadicalBox\x22\
,\x22RadicalBoxOpti\
ons\x22,\x22RadioButto\
n\x22,\x22RadioButtonB\
ar\x22,\x22RadioButton\
Box\x22,\x22RadioButto\
nBoxOptions\x22,\x22Ra\
don\x22,\x22RadonTrans\
form\x22,\x22Ramanujan\
Tau\x22,\x22RamanujanT\
auL\x22,\x22RamanujanT\
auTheta\x22,\x22Ramanu\
janTauZ\x22,\x22Ramp\x22,\
\x22Random\x22,\x22Random\
ArrayLayer\x22,\x22Ran\
domChoice\x22,\x22Rand\
omColor\x22,\x22Random\
Complex\x22,\x22Random\
Date\x22,\x22RandomEnt\
ity\x22,\x22RandomFunc\
tion\x22,\x22RandomGen\
eratorState\x22,\x22Ra\
ndomGeoPosition\x22\
,\x22RandomGraph\x22,\x22\
RandomImage\x22,\x22Ra\
ndomInstance\x22,\x22R\
andomInteger\x22,\x22R\
andomPermutation\
\x22,\x22RandomPoint\x22,\
\x22RandomPointConf\
iguration\x22,\x22Rand\
omPolygon\x22,\x22Rand\
omPolyhedron\x22,\x22R\
andomPrime\x22,\x22Ran\
domReal\x22,\x22Random\
Sample\x22,\x22RandomS\
eed\x22,\x22RandomSeed\
ing\x22,\x22RandomTime\
\x22,\x22RandomTree\x22,\x22\
RandomVariate\x22,\x22\
RandomWalkProces\
s\x22,\x22RandomWord\x22,\
\x22Range\x22,\x22RangeFi\
lter\x22,\x22RangeSpec\
ification\x22,\x22Rank\
edMax\x22,\x22RankedMi\
n\x22,\x22RarerProbabi\
lity\x22,\x22Raster\x22,\x22\
Raster3D\x22,\x22Raste\
r3DBox\x22,\x22Raster3\
DBoxOptions\x22,\x22Ra\
sterArray\x22,\x22Rast\
erBox\x22,\x22RasterBo\
xOptions\x22,\x22Raste\
rize\x22,\x22RasterSiz\
e\x22,\x22Rational\x22,\x22R\
ationalExpressio\
nQ\x22,\x22RationalFun\
ctions\x22,\x22Rationa\
lize\x22,\x22Rationals\
\x22,\x22Ratios\x22,\x22RawA\
rray\x22,\x22RawBoxes\x22\
,\x22RawData\x22,\x22RawM\
edium\x22,\x22Rayleigh\
Distribution\x22,\x22R\
e\x22,\x22ReactionBala\
nce\x22,\x22ReactionBa\
lancedQ\x22,\x22Reacti\
onPDETerm\x22,\x22Read\
\x22,\x22ReadByteArray\
\x22,\x22ReadLine\x22,\x22Re\
adList\x22,\x22ReadPro\
tected\x22,\x22ReadStr\
ing\x22,\x22Real\x22,\x22Rea\
lAbs\x22,\x22RealBlock\
DiagonalForm\x22,\x22R\
ealDigits\x22,\x22Real\
Exponent\x22,\x22Reals\
\x22,\x22RealSign\x22,\x22Re\
ap\x22,\x22RebuildPacl\
etData\x22,\x22Recalib\
rationFunction\x22,\
\x22RecognitionPrio\
r\x22,\x22RecognitionT\
hreshold\x22,\x22Recon\
structionMesh\x22,\x22\
Record\x22,\x22RecordL\
ists\x22,\x22RecordSep\
arators\x22,\x22Rectan\
gle\x22,\x22RectangleB\
ox\x22,\x22RectangleBo\
xOptions\x22,\x22Recta\
ngleChart\x22,\x22Rect\
angleChart3D\x22,\x22R\
ectangularRepeat\
ingElement\x22,\x22Rec\
urrenceFilter\x22,\x22\
RecurrenceTable\x22\
,\x22RecurringDigit\
sForm\x22,\x22Red\x22,\x22Re\
duce\x22,\x22RefBox\x22,\x22\
ReferenceLineSty\
le\x22,\x22ReferenceMa\
rkers\x22,\x22Referenc\
eMarkerStyle\x22,\x22R\
efine\x22,\x22Reflecti\
onMatrix\x22,\x22Refle\
ctionTransform\x22,\
\x22Refresh\x22,\x22Refre\
shRate\x22,\x22Region\x22\
,\x22RegionBinarize\
\x22,\x22RegionBoundar\
y\x22,\x22RegionBounda\
ryStyle\x22,\x22Region\
Bounds\x22,\x22RegionC\
entroid\x22,\x22Region\
Congruent\x22,\x22Regi\
onConvert\x22,\x22Regi\
onDifference\x22,\x22R\
egionDilation\x22,\x22\
RegionDimension\x22\
,\x22RegionDisjoint\
\x22,\x22RegionDistanc\
e\x22,\x22RegionDistan\
ceFunction\x22,\x22Reg\
ionEmbeddingDime\
nsion\x22,\x22RegionEq\
ual\x22,\x22RegionEros\
ion\x22,\x22RegionFill\
ingStyle\x22,\x22Regio\
nFit\x22,\x22RegionFun\
ction\x22,\x22RegionIm\
age\x22,\x22RegionInte\
rsection\x22,\x22Regio\
nMeasure\x22,\x22Regio\
nMember\x22,\x22Region\
MemberFunction\x22,\
\x22RegionMoment\x22,\x22\
RegionNearest\x22,\x22\
RegionNearestFun\
ction\x22,\x22RegionPl\
ot\x22,\x22RegionPlot3\
D\x22,\x22RegionProduc\
t\x22,\x22RegionQ\x22,\x22Re\
gionResize\x22,\x22Reg\
ionSimilar\x22,\x22Reg\
ionSize\x22,\x22Region\
SymmetricDiffere\
nce\x22,\x22RegionUnio\
n\x22,\x22RegionWithin\
\x22,\x22RegisterExter\
nalEvaluator\x22,\x22R\
egularExpression\
\x22,\x22Regularizatio\
n\x22,\x22RegularlySam\
pledQ\x22,\x22RegularP\
olygon\x22,\x22ReIm\x22,\x22\
ReImLabels\x22,\x22ReI\
mPlot\x22,\x22ReImStyl\
e\x22,\x22Reinstall\x22,\x22\
RelationalDataba\
se\x22,\x22RelationGra\
ph\x22,\x22Release\x22,\x22R\
eleaseHold\x22,\x22Rel\
iabilityDistribu\
tion\x22,\x22ReliefIma\
ge\x22,\x22ReliefPlot\x22\
,\x22RemoteAuthoriz\
ationCaching\x22,\x22R\
emoteBatchJobAbo\
rt\x22,\x22RemoteBatch\
JobObject\x22,\x22Remo\
teBatchJobs\x22,\x22Re\
moteBatchMapSubm\
it\x22,\x22RemoteBatch\
SubmissionEnviro\
nment\x22,\x22RemoteBa\
tchSubmit\x22,\x22Remo\
teConnect\x22,\x22Remo\
teConnectionObje\
ct\x22,\x22RemoteEvalu\
ate\x22,\x22RemoteFile\
\x22,\x22RemoteInputFi\
les\x22,\x22RemoteKern\
elObject\x22,\x22Remot\
eProviderSetting\
s\x22,\x22RemoteRun\x22,\x22\
RemoteRunProcess\
\x22,\x22RemovalCondit\
ions\x22,\x22Remove\x22,\x22\
RemoveAlphaChann\
el\x22,\x22RemoveAsync\
hronousTask\x22,\x22Re\
moveAudioStream\x22\
,\x22RemoveBackgrou\
nd\x22,\x22RemoveChann\
elListener\x22,\x22Rem\
oveChannelSubscr\
ibers\x22,\x22Removed\x22\
,\x22RemoveDiacriti\
cs\x22,\x22RemoveInput\
StreamMethod\x22,\x22R\
emoveOutputStrea\
mMethod\x22,\x22Remove\
Property\x22,\x22Remov\
eScheduledTask\x22,\
\x22RemoveUsers\x22,\x22R\
emoveVideoStream\
\x22,\x22RenameDirecto\
ry\x22,\x22RenameFile\x22\
,\x22RenderAll\x22,\x22Re\
nderingOptions\x22,\
\x22RenewalProcess\x22\
,\x22RenkoChart\x22,\x22R\
epairMesh\x22,\x22Repe\
ated\x22,\x22RepeatedN\
ull\x22,\x22RepeatedSt\
ring\x22,\x22RepeatedT\
iming\x22,\x22Repeatin\
gElement\x22,\x22Repla\
ce\x22,\x22ReplaceAll\x22\
,\x22ReplaceAt\x22,\x22Re\
placeHeldPart\x22,\x22\
ReplaceImageValu\
e\x22,\x22ReplaceList\x22\
,\x22ReplacePart\x22,\x22\
ReplacePixelValu\
e\x22,\x22ReplaceRepea\
ted\x22,\x22ReplicateL\
ayer\x22,\x22RequiredP\
hysicalQuantitie\
s\x22,\x22Resampling\x22,\
\x22ResamplingAlgor\
ithmData\x22,\x22Resam\
plingMethod\x22,\x22Re\
scale\x22,\x22Rescalin\
gTransform\x22,\x22Res\
etDirectory\x22,\x22Re\
setScheduledTask\
\x22,\x22ReshapeLayer\x22\
,\x22Residue\x22,\x22Resi\
dueSum\x22,\x22ResizeL\
ayer\x22,\x22Resolve\x22,\
\x22ResolveContextA\
liases\x22,\x22Resourc\
eAcquire\x22,\x22Resou\
rceData\x22,\x22Resour\
ceFunction\x22,\x22Res\
ourceObject\x22,\x22Re\
sourceRegister\x22,\
\x22ResourceRemove\x22\
,\x22ResourceSearch\
\x22,\x22ResourceSubmi\
ssionObject\x22,\x22Re\
sourceSubmit\x22,\x22R\
esourceSystemBas\
e\x22,\x22ResourceSyst\
emPath\x22,\x22Resourc\
eUpdate\x22,\x22Resour\
ceVersion\x22,\x22Resp\
onseForm\x22,\x22Rest\x22\
,\x22RestartInterva\
l\x22,\x22Restricted\x22,\
\x22Resultant\x22,\x22Res\
umePacket\x22,\x22Retu\
rn\x22,\x22ReturnCreat\
esNewCell\x22,\x22Retu\
rnEntersInput\x22,\x22\
ReturnExpression\
Packet\x22,\x22ReturnI\
nputFormPacket\x22,\
\x22ReturnPacket\x22,\x22\
ReturnReceiptFun\
ction\x22,\x22ReturnTe\
xtPacket\x22,\x22Rever\
se\x22,\x22ReverseAppl\
ied\x22,\x22ReverseBio\
rthogonalSplineW\
avelet\x22,\x22Reverse\
Element\x22,\x22Revers\
eEquilibrium\x22,\x22R\
everseGraph\x22,\x22Re\
verseSort\x22,\x22Reve\
rseSortBy\x22,\x22Reve\
rseUpEquilibrium\
\x22,\x22RevolutionAxi\
s\x22,\x22RevolutionPl\
ot3D\x22,\x22RGBColor\x22\
,\x22RiccatiSolve\x22,\
\x22RiceDistributio\
n\x22,\x22RidgeFilter\x22\
,\x22RiemannR\x22,\x22Rie\
mannSiegelTheta\x22\
,\x22RiemannSiegelZ\
\x22,\x22RiemannXi\x22,\x22R\
iffle\x22,\x22Right\x22,\x22\
RightArrow\x22,\x22Rig\
htArrowBar\x22,\x22Rig\
htArrowLeftArrow\
\x22,\x22RightComposit\
ion\x22,\x22RightCoset\
Representative\x22,\
\x22RightDownTeeVec\
tor\x22,\x22RightDownV\
ector\x22,\x22RightDow\
nVectorBar\x22,\x22Rig\
htTee\x22,\x22RightTee\
Arrow\x22,\x22RightTee\
Vector\x22,\x22RightTr\
iangle\x22,\x22RightTr\
iangleBar\x22,\x22Righ\
tTriangleEqual\x22,\
\x22RightUpDownVect\
or\x22,\x22RightUpTeeV\
ector\x22,\x22RightUpV\
ector\x22,\x22RightUpV\
ectorBar\x22,\x22Right\
Vector\x22,\x22RightVe\
ctorBar\x22,\x22Ripley\
K\x22,\x22RipleyRasson\
Region\x22,\x22RiskAch\
ievementImportan\
ce\x22,\x22RiskReducti\
onImportance\x22,\x22R\
obustConvexOptim\
ization\x22,\x22Rogers\
TanimotoDissimil\
arity\x22,\x22RollPitc\
hYawAngles\x22,\x22Rol\
lPitchYawMatrix\x22\
,\x22RomanNumeral\x22,\
\x22Root\x22,\x22RootAppr\
oximant\x22,\x22RootIn\
tervals\x22,\x22RootLo\
cusPlot\x22,\x22RootMe\
anSquare\x22,\x22RootO\
fUnityQ\x22,\x22RootRe\
duce\x22,\x22Roots\x22,\x22R\
ootSum\x22,\x22RootTre\
e\x22,\x22Rotate\x22,\x22Rot\
ateLabel\x22,\x22Rotat\
eLeft\x22,\x22RotateRi\
ght\x22,\x22RotationAc\
tion\x22,\x22RotationB\
ox\x22,\x22RotationBox\
Options\x22,\x22Rotati\
onMatrix\x22,\x22Rotat\
ionTransform\x22,\x22R\
ound\x22,\x22RoundImpl\
ies\x22,\x22RoundingRa\
dius\x22,\x22Row\x22,\x22Row\
Alignments\x22,\x22Row\
Backgrounds\x22,\x22Ro\
wBox\x22,\x22RowHeight\
s\x22,\x22RowLines\x22,\x22R\
owMinHeight\x22,\x22Ro\
wReduce\x22,\x22RowsEq\
ual\x22,\x22RowSpacing\
s\x22,\x22RSolve\x22,\x22RSo\
lveValue\x22,\x22Rudin\
Shapiro\x22,\x22Rudval\
isGroupRu\x22,\x22Rule\
\x22,\x22RuleCondition\
\x22,\x22RuleDelayed\x22,\
\x22RuleForm\x22,\x22Rule\
Plot\x22,\x22RulerUnit\
s\x22,\x22RulesTree\x22,\x22\
Run\x22,\x22RunProcess\
\x22,\x22RunScheduledT\
ask\x22,\x22RunThrough\
\x22,\x22RuntimeAttrib\
utes\x22,\x22RuntimeOp\
tions\x22,\x22RussellR\
aoDissimilarity\x22\
,\x22SameAs\x22,\x22SameQ\
\x22,\x22SameTest\x22,\x22Sa\
meTestProperties\
\x22,\x22SampledEntity\
Class\x22,\x22SampleDe\
pth\x22,\x22SampledSou\
ndFunction\x22,\x22Sam\
pledSoundList\x22,\x22\
SampleRate\x22,\x22Sam\
plingPeriod\x22,\x22SA\
RIMAProcess\x22,\x22SA\
RMAProcess\x22,\x22SAS\
Triangle\x22,\x22Satel\
liteData\x22,\x22Satis\
fiabilityCount\x22,\
\x22SatisfiabilityI\
nstances\x22,\x22Satis\
fiableQ\x22,\x22Saturd\
ay\x22,\x22Save\x22,\x22Save\
able\x22,\x22SaveAutoD\
elete\x22,\x22SaveConn\
ection\x22,\x22SaveDef\
initions\x22,\x22Savit\
zkyGolayMatrix\x22,\
\x22SawtoothWave\x22,\x22\
Scale\x22,\x22Scaled\x22,\
\x22ScaleDivisions\x22\
,\x22ScaledMousePos\
ition\x22,\x22ScaleOri\
gin\x22,\x22ScalePaddi\
ng\x22,\x22ScaleRanges\
\x22,\x22ScaleRangeSty\
le\x22,\x22ScalingFunc\
tions\x22,\x22ScalingM\
atrix\x22,\x22ScalingT\
ransform\x22,\x22Scan\x22\
,\x22ScheduledTask\x22\
,\x22ScheduledTaskA\
ctiveQ\x22,\x22Schedul\
edTaskInformatio\
n\x22,\x22ScheduledTas\
kInformationData\
\x22,\x22ScheduledTask\
Object\x22,\x22Schedul\
edTasks\x22,\x22SchurD\
ecomposition\x22,\x22S\
cientificForm\x22,\x22\
ScientificNotati\
onThreshold\x22,\x22Sc\
orerGi\x22,\x22ScorerG\
iPrime\x22,\x22ScorerH\
i\x22,\x22ScorerHiPrim\
e\x22,\x22ScreenRectan\
gle\x22,\x22ScreenStyl\
eEnvironment\x22,\x22S\
criptBaselineShi\
fts\x22,\x22ScriptForm\
\x22,\x22ScriptLevel\x22,\
\x22ScriptMinSize\x22,\
\x22ScriptRules\x22,\x22S\
criptSizeMultipl\
iers\x22,\x22Scrollbar\
s\x22,\x22ScrollingOpt\
ions\x22,\x22ScrollPos\
ition\x22,\x22SearchAd\
justment\x22,\x22Searc\
hIndexObject\x22,\x22S\
earchIndices\x22,\x22S\
earchQueryString\
\x22,\x22SearchResultO\
bject\x22,\x22Sec\x22,\x22Se\
ch\x22,\x22SechDistrib\
ution\x22,\x22SecondOr\
derConeOptimizat\
ion\x22,\x22SectionGro\
uping\x22,\x22SectorCh\
art\x22,\x22SectorChar\
t3D\x22,\x22SectorOrig\
in\x22,\x22SectorSpaci\
ng\x22,\x22SecuredAuth\
enticationKey\x22,\x22\
SecuredAuthentic\
ationKeys\x22,\x22Secu\
rityCertificate\x22\
,\x22SeedRandom\x22,\x22S\
elect\x22,\x22Selectab\
le\x22,\x22SelectCompo\
nents\x22,\x22Selected\
Cells\x22,\x22Selected\
Notebook\x22,\x22Selec\
tFirst\x22,\x22Selecti\
on\x22,\x22SelectionAn\
imate\x22,\x22Selectio\
nCell\x22,\x22Selectio\
nCellCreateCell\x22\
,\x22SelectionCellD\
efaultStyle\x22,\x22Se\
lectionCellParen\
tStyle\x22,\x22Selecti\
onCreateCell\x22,\x22S\
electionDebugger\
Tag\x22,\x22SelectionE\
valuate\x22,\x22Select\
ionEvaluateCreat\
eCell\x22,\x22Selectio\
nMove\x22,\x22Selectio\
nPlaceholder\x22,\x22S\
electWithContent\
s\x22,\x22SelfLoops\x22,\x22\
SelfLoopStyle\x22,\x22\
SemanticImport\x22,\
\x22SemanticImportS\
tring\x22,\x22Semantic\
Interpretation\x22,\
\x22SemialgebraicCo\
mponentInstances\
\x22,\x22SemidefiniteO\
ptimization\x22,\x22Se\
ndMail\x22,\x22SendMes\
sage\x22,\x22Sequence\x22\
,\x22SequenceAlignm\
ent\x22,\x22SequenceAt\
tentionLayer\x22,\x22S\
equenceCases\x22,\x22S\
equenceCount\x22,\x22S\
equenceFold\x22,\x22Se\
quenceFoldList\x22,\
\x22SequenceForm\x22,\x22\
SequenceHold\x22,\x22S\
equenceIndicesLa\
yer\x22,\x22SequenceLa\
stLayer\x22,\x22Sequen\
ceMostLayer\x22,\x22Se\
quencePosition\x22,\
\x22SequencePredict\
\x22,\x22SequencePredi\
ctorFunction\x22,\x22S\
equenceReplace\x22,\
\x22SequenceRestLay\
er\x22,\x22SequenceRev\
erseLayer\x22,\x22Sequ\
enceSplit\x22,\x22Seri\
es\x22,\x22SeriesCoeff\
icient\x22,\x22SeriesD\
ata\x22,\x22SeriesTerm\
Goal\x22,\x22ServiceCo\
nnect\x22,\x22ServiceD\
isconnect\x22,\x22Serv\
iceExecute\x22,\x22Ser\
viceObject\x22,\x22Ser\
viceRequest\x22,\x22Se\
rviceResponse\x22,\x22\
ServiceSubmit\x22,\x22\
SessionSubmit\x22,\x22\
SessionTime\x22,\x22Se\
t\x22,\x22SetAccuracy\x22\
,\x22SetAlphaChanne\
l\x22,\x22SetAttribute\
s\x22,\x22Setbacks\x22,\x22S\
etCloudDirectory\
\x22,\x22SetCookies\x22,\x22\
SetDelayed\x22,\x22Set\
Directory\x22,\x22SetE\
nvironment\x22,\x22Set\
FileDate\x22,\x22SetFi\
leFormatProperti\
es\x22,\x22SetOptions\x22\
,\x22SetOptionsPack\
et\x22,\x22SetPermissi\
ons\x22,\x22SetPrecisi\
on\x22,\x22SetProperty\
\x22,\x22SetSecuredAut\
henticationKey\x22,\
\x22SetSelectedNote\
book\x22,\x22SetShared\
Function\x22,\x22SetSh\
aredVariable\x22,\x22S\
etStreamPosition\
\x22,\x22SetSystemMode\
l\x22,\x22SetSystemOpt\
ions\x22,\x22Setter\x22,\x22\
SetterBar\x22,\x22Sett\
erBox\x22,\x22SetterBo\
xOptions\x22,\x22Setti\
ng\x22,\x22SetUsers\x22,\x22\
Shading\x22,\x22Shallo\
w\x22,\x22ShannonWavel\
et\x22,\x22ShapiroWilk\
Test\x22,\x22Share\x22,\x22S\
haringList\x22,\x22Sha\
rpen\x22,\x22ShearingM\
atrix\x22,\x22Shearing\
Transform\x22,\x22Shel\
lRegion\x22,\x22ShenCa\
stanMatrix\x22,\x22Shi\
ftedGompertzDist\
ribution\x22,\x22Shift\
RegisterSequence\
\x22,\x22Short\x22,\x22Short\
DownArrow\x22,\x22Shor\
test\x22,\x22ShortestM\
atch\x22,\x22ShortestP\
athFunction\x22,\x22Sh\
ortLeftArrow\x22,\x22S\
hortRightArrow\x22,\
\x22ShortTimeFourie\
r\x22,\x22ShortTimeFou\
rierData\x22,\x22Short\
UpArrow\x22,\x22Show\x22,\
\x22ShowAutoConvert\
\x22,\x22ShowAutoSpell\
Check\x22,\x22ShowAuto\
Styles\x22,\x22ShowCel\
lBracket\x22,\x22ShowC\
ellLabel\x22,\x22ShowC\
ellTags\x22,\x22ShowCl\
osedCellArea\x22,\x22S\
howCodeAssist\x22,\x22\
ShowContents\x22,\x22S\
howControls\x22,\x22Sh\
owCursorTracker\x22\
,\x22ShowGroupOpenC\
loseIcon\x22,\x22ShowG\
roupOpener\x22,\x22Sho\
wInvisibleCharac\
ters\x22,\x22ShowPageB\
reaks\x22,\x22ShowPred\
ictiveInterface\x22\
,\x22ShowSelection\x22\
,\x22ShowShortBoxFo\
rm\x22,\x22ShowSpecial\
Characters\x22,\x22Sho\
wStringCharacter\
s\x22,\x22ShowSyntaxSt\
yles\x22,\x22Shrinking\
Delay\x22,\x22ShrinkWr\
apBoundingBox\x22,\x22\
SiderealTime\x22,\x22S\
iegelTheta\x22,\x22Sie\
gelTukeyTest\x22,\x22S\
ierpinskiCurve\x22,\
\x22SierpinskiMesh\x22\
,\x22Sign\x22,\x22Signatu\
re\x22,\x22SignedRankT\
est\x22,\x22SignedRegi\
onDistance\x22,\x22Sig\
nificanceLevel\x22,\
\x22SignPadding\x22,\x22S\
ignTest\x22,\x22Simila\
rityRules\x22,\x22Simp\
leGraph\x22,\x22Simple\
GraphQ\x22,\x22SimpleP\
olygonQ\x22,\x22Simple\
PolyhedronQ\x22,\x22Si\
mplex\x22,\x22Simplify\
\x22,\x22Sin\x22,\x22Sinc\x22,\x22\
SinghMaddalaDist\
ribution\x22,\x22Singl\
eEvaluation\x22,\x22Si\
ngleLetterItalic\
s\x22,\x22SingleLetter\
Style\x22,\x22Singular\
ValueDecompositi\
on\x22,\x22SingularVal\
ueList\x22,\x22Singula\
rValuePlot\x22,\x22Sin\
gularValues\x22,\x22Si\
nh\x22,\x22SinhIntegra\
l\x22,\x22SinIntegral\x22\
,\x22SixJSymbol\x22,\x22S\
keleton\x22,\x22Skelet\
onTransform\x22,\x22Sk\
ellamDistributio\
n\x22,\x22Skewness\x22,\x22S\
kewNormalDistrib\
ution\x22,\x22SkinStyl\
e\x22,\x22Skip\x22,\x22Slice\
ContourPlot3D\x22,\x22\
SliceDensityPlot\
3D\x22,\x22SliceDistri\
bution\x22,\x22SliceVe\
ctorPlot3D\x22,\x22Sli\
der\x22,\x22Slider2D\x22,\
\x22Slider2DBox\x22,\x22S\
lider2DBoxOption\
s\x22,\x22SliderBox\x22,\x22\
SliderBoxOptions\
\x22,\x22SlideShowVide\
o\x22,\x22SlideView\x22,\x22\
Slot\x22,\x22SlotSeque\
nce\x22,\x22Small\x22,\x22Sm\
allCircle\x22,\x22Smal\
ler\x22,\x22SmithDecom\
position\x22,\x22Smith\
DelayCompensator\
\x22,\x22SmithWaterman\
Similarity\x22,\x22Smo\
othDensityHistog\
ram\x22,\x22SmoothHist\
ogram\x22,\x22SmoothHi\
stogram3D\x22,\x22Smoo\
thKernelDistribu\
tion\x22,\x22SmoothPoi\
ntDensity\x22,\x22SnDi\
spersion\x22,\x22Snipp\
et\x22,\x22SnippetsVid\
eo\x22,\x22SnubPolyhed\
ron\x22,\x22SocialMedi\
aData\x22,\x22Socket\x22,\
\x22SocketConnect\x22,\
\x22SocketListen\x22,\x22\
SocketListener\x22,\
\x22SocketObject\x22,\x22\
SocketOpen\x22,\x22Soc\
ketReadMessage\x22,\
\x22SocketReadyQ\x22,\x22\
Sockets\x22,\x22Socket\
WaitAll\x22,\x22Socket\
WaitNext\x22,\x22Softm\
axLayer\x22,\x22SokalS\
neathDissimilari\
ty\x22,\x22SolarEclips\
e\x22,\x22SolarSystemF\
eatureData\x22,\x22Sol\
arTime\x22,\x22SolidAn\
gle\x22,\x22SolidBound\
aryLoadValue\x22,\x22S\
olidData\x22,\x22Solid\
DisplacementCond\
ition\x22,\x22SolidFix\
edCondition\x22,\x22So\
lidMechanicsPDEC\
omponent\x22,\x22Solid\
MechanicsStrain\x22\
,\x22SolidMechanics\
Stress\x22,\x22SolidRe\
gionQ\x22,\x22Solve\x22,\x22\
SolveAlways\x22,\x22So\
lveDelayed\x22,\x22Sol\
veValues\x22,\x22Sort\x22\
,\x22SortBy\x22,\x22Sorte\
dBy\x22,\x22SortedEnti\
tyClass\x22,\x22Sound\x22\
,\x22SoundAndGraphi\
cs\x22,\x22SoundNote\x22,\
\x22SoundVolume\x22,\x22S\
ourceLink\x22,\x22Sour\
cePDETerm\x22,\x22Sow\x22\
,\x22Space\x22,\x22SpaceC\
urveData\x22,\x22Space\
Form\x22,\x22Spacer\x22,\x22\
Spacings\x22,\x22Span\x22\
,\x22SpanAdjustment\
s\x22,\x22SpanCharacte\
rRounding\x22,\x22Span\
FromAbove\x22,\x22Span\
FromBoth\x22,\x22SpanF\
romLeft\x22,\x22SpanLi\
neThickness\x22,\x22Sp\
anMaxSize\x22,\x22Span\
MinSize\x22,\x22Spanni\
ngCharacters\x22,\x22S\
panSymmetric\x22,\x22S\
parseArray\x22,\x22Spa\
rseArrayQ\x22,\x22Spat\
ialBinnedPointDa\
ta\x22,\x22SpatialBoun\
daryCorrection\x22,\
\x22SpatialEstimate\
\x22,\x22SpatialEstima\
torFunction\x22,\x22Sp\
atialGraphDistri\
bution\x22,\x22Spatial\
J\x22,\x22SpatialMedia\
n\x22,\x22SpatialNoise\
Level\x22,\x22SpatialO\
bservationRegion\
Q\x22,\x22SpatialPoint\
Data\x22,\x22SpatialPo\
intSelect\x22,\x22Spat\
ialRandomnessTes\
t\x22,\x22SpatialTrans\
formationLayer\x22,\
\x22SpatialTrendFun\
ction\x22,\x22Speak\x22,\x22\
SpeakerMatchQ\x22,\x22\
SpearmanRankTest\
\x22,\x22SpearmanRho\x22,\
\x22SpeciesData\x22,\x22S\
pecificityGoal\x22,\
\x22SpectralLineDat\
a\x22,\x22Spectrogram\x22\
,\x22SpectrogramArr\
ay\x22,\x22Specularity\
\x22,\x22SpeechCases\x22,\
\x22SpeechInterpret\
er\x22,\x22SpeechRecog\
nize\x22,\x22SpeechSyn\
thesize\x22,\x22Spelli\
ngCorrection\x22,\x22S\
pellingCorrectio\
nList\x22,\x22Spelling\
Dictionaries\x22,\x22S\
pellingDictionar\
iesPath\x22,\x22Spelli\
ngOptions\x22,\x22Sphe\
re\x22,\x22SphereBox\x22,\
\x22SphereBoxOption\
s\x22,\x22SpherePoints\
\x22,\x22SphericalBess\
elJ\x22,\x22SphericalB\
esselY\x22,\x22Spheric\
alHankelH1\x22,\x22Sph\
ericalHankelH2\x22,\
\x22SphericalHarmon\
icY\x22,\x22SphericalP\
lot3D\x22,\x22Spherica\
lRegion\x22,\x22Spheri\
calShell\x22,\x22Spher\
oidalEigenvalue\x22\
,\x22SpheroidalJoin\
ingFactor\x22,\x22Sphe\
roidalPS\x22,\x22Spher\
oidalPSPrime\x22,\x22S\
pheroidalQS\x22,\x22Sp\
heroidalQSPrime\x22\
,\x22SpheroidalRadi\
alFactor\x22,\x22Spher\
oidalS1\x22,\x22Sphero\
idalS1Prime\x22,\x22Sp\
heroidalS2\x22,\x22Sph\
eroidalS2Prime\x22,\
\x22Splice\x22,\x22Splice\
dDistribution\x22,\x22\
SplineClosed\x22,\x22S\
plineDegree\x22,\x22Sp\
lineKnots\x22,\x22Spli\
neWeights\x22,\x22Spli\
t\x22,\x22SplitBy\x22,\x22Sp\
okenString\x22,\x22Spo\
tLight\x22,\x22Sqrt\x22,\x22\
SqrtBox\x22,\x22SqrtBo\
xOptions\x22,\x22Squar\
e\x22,\x22SquaredEucli\
deanDistance\x22,\x22S\
quareFreeQ\x22,\x22Squ\
areIntersection\x22\
,\x22SquareMatrixQ\x22\
,\x22SquareRepeatin\
gElement\x22,\x22Squar\
esR\x22,\x22SquareSubs\
et\x22,\x22SquareSubse\
tEqual\x22,\x22SquareS\
uperset\x22,\x22Square\
SupersetEqual\x22,\x22\
SquareUnion\x22,\x22Sq\
uareWave\x22,\x22SSSTr\
iangle\x22,\x22Stabili\
tyMargins\x22,\x22Stab\
ilityMarginsStyl\
e\x22,\x22StableDistri\
bution\x22,\x22Stack\x22,\
\x22StackBegin\x22,\x22St\
ackComplete\x22,\x22St\
ackedDateListPlo\
t\x22,\x22StackedListP\
lot\x22,\x22StackInhib\
it\x22,\x22StadiumShap\
e\x22,\x22StandardAtmo\
sphereData\x22,\x22Sta\
ndardDeviation\x22,\
\x22StandardDeviati\
onFilter\x22,\x22Stand\
ardForm\x22,\x22Standa\
rdize\x22,\x22Standard\
ized\x22,\x22StandardO\
ceanData\x22,\x22Stand\
byDistribution\x22,\
\x22Star\x22,\x22StarClus\
terData\x22,\x22StarDa\
ta\x22,\x22StarGraph\x22,\
\x22StartAsynchrono\
usTask\x22,\x22StartEx\
ternalSession\x22,\x22\
StartingStepSize\
\x22,\x22StartOfLine\x22,\
\x22StartOfString\x22,\
\x22StartProcess\x22,\x22\
StartScheduledTa\
sk\x22,\x22StartupSoun\
d\x22,\x22StartWebSess\
ion\x22,\x22StateDimen\
sions\x22,\x22StateFee\
dbackGains\x22,\x22Sta\
teOutputEstimato\
r\x22,\x22StateRespons\
e\x22,\x22StateSpaceMo\
del\x22,\x22StateSpace\
Realization\x22,\x22St\
ateSpaceTransfor\
m\x22,\x22StateTransfo\
rmationLinearize\
\x22,\x22StationaryDis\
tribution\x22,\x22Stat\
ionaryWaveletPac\
ketTransform\x22,\x22S\
tationaryWavelet\
Transform\x22,\x22Stat\
usArea\x22,\x22StatusC\
entrality\x22,\x22Step\
Monitor\x22,\x22Stereo\
chemistryElement\
s\x22,\x22StieltjesGam\
ma\x22,\x22StippleShad\
ing\x22,\x22StirlingS1\
\x22,\x22StirlingS2\x22,\x22\
StopAsynchronous\
Task\x22,\x22StoppingP\
owerData\x22,\x22StopS\
cheduledTask\x22,\x22S\
trataVariables\x22,\
\x22StratonovichPro\
cess\x22,\x22StraussHa\
rdcorePointProce\
ss\x22,\x22StraussPoin\
tProcess\x22,\x22Strea\
mColorFunction\x22,\
\x22StreamColorFunc\
tionScaling\x22,\x22St\
reamDensityPlot\x22\
,\x22StreamMarkers\x22\
,\x22StreamPlot\x22,\x22S\
treamPlot3D\x22,\x22St\
reamPoints\x22,\x22Str\
eamPosition\x22,\x22St\
reams\x22,\x22StreamSc\
ale\x22,\x22StreamStyl\
e\x22,\x22StrictInequa\
lities\x22,\x22String\x22\
,\x22StringBreak\x22,\x22\
StringByteCount\x22\
,\x22StringCases\x22,\x22\
StringContainsQ\x22\
,\x22StringCount\x22,\x22\
StringDelete\x22,\x22S\
tringDrop\x22,\x22Stri\
ngEndsQ\x22,\x22String\
Expression\x22,\x22Str\
ingExtract\x22,\x22Str\
ingForm\x22,\x22String\
Format\x22,\x22StringF\
ormatQ\x22,\x22StringF\
reeQ\x22,\x22StringIns\
ert\x22,\x22StringJoin\
\x22,\x22StringLength\x22\
,\x22StringMatchQ\x22,\
\x22StringPadLeft\x22,\
\x22StringPadRight\x22\
,\x22StringPart\x22,\x22S\
tringPartition\x22,\
\x22StringPosition\x22\
,\x22StringQ\x22,\x22Stri\
ngRepeat\x22,\x22Strin\
gReplace\x22,\x22Strin\
gReplaceList\x22,\x22S\
tringReplacePart\
\x22,\x22StringReverse\
\x22,\x22StringRiffle\x22\
,\x22StringRotateLe\
ft\x22,\x22StringRotat\
eRight\x22,\x22StringS\
keleton\x22,\x22String\
Split\x22,\x22StringSt\
artsQ\x22,\x22StringTa\
ke\x22,\x22StringTakeD\
rop\x22,\x22StringTemp\
late\x22,\x22StringToB\
yteArray\x22,\x22Strin\
gToStream\x22,\x22Stri\
ngTrim\x22,\x22StripBo\
xes\x22,\x22StripOnInp\
ut\x22,\x22StripStyleO\
nPaste\x22,\x22StripWr\
apperBoxes\x22,\x22Str\
okeForm\x22,\x22Struck\
through\x22,\x22Struct\
uralImportance\x22,\
\x22StructuredArray\
\x22,\x22StructuredArr\
ayHeadQ\x22,\x22Struct\
uredSelection\x22,\x22\
StruveH\x22,\x22Struve\
L\x22,\x22Stub\x22,\x22Stude\
ntTDistribution\x22\
,\x22Style\x22,\x22StyleB\
ox\x22,\x22StyleBoxAut\
oDelete\x22,\x22StyleD\
ata\x22,\x22StyleDefin\
itions\x22,\x22StyleFo\
rm\x22,\x22StyleHints\x22\
,\x22StyleKeyMappin\
g\x22,\x22StyleMenuLis\
ting\x22,\x22StyleName\
DialogSettings\x22,\
\x22StyleNames\x22,\x22St\
ylePrint\x22,\x22Style\
SheetPath\x22,\x22Subd\
ivide\x22,\x22Subfacto\
rial\x22,\x22Subgraph\x22\
,\x22SubMinus\x22,\x22Sub\
Plus\x22,\x22Subresult\
antPolynomialRem\
ainders\x22,\x22Subres\
ultantPolynomial\
s\x22,\x22Subresultant\
s\x22,\x22Subscript\x22,\x22\
SubscriptBox\x22,\x22S\
ubscriptBoxOptio\
ns\x22,\x22Subscripted\
\x22,\x22Subsequences\x22\
,\x22Subset\x22,\x22Subse\
tCases\x22,\x22SubsetC\
ount\x22,\x22SubsetEqu\
al\x22,\x22SubsetMap\x22,\
\x22SubsetPosition\x22\
,\x22SubsetQ\x22,\x22Subs\
etReplace\x22,\x22Subs\
ets\x22,\x22SubStar\x22,\x22\
SubstitutionSyst\
em\x22,\x22Subsuperscr\
ipt\x22,\x22Subsupersc\
riptBox\x22,\x22Subsup\
erscriptBoxOptio\
ns\x22,\x22SubtitleEnc\
oding\x22,\x22Subtitle\
TrackSelection\x22,\
\x22Subtract\x22,\x22Subt\
ractFrom\x22,\x22Subtr\
actSides\x22,\x22SubVa\
lues\x22,\x22Succeeds\x22\
,\x22SucceedsEqual\x22\
,\x22SucceedsSlantE\
qual\x22,\x22SucceedsT\
ilde\x22,\x22Success\x22,\
\x22SuchThat\x22,\x22Sum\x22\
,\x22SumConvergence\
\x22,\x22SummationLaye\
r\x22,\x22Sunday\x22,\x22Sun\
Position\x22,\x22Sunri\
se\x22,\x22Sunset\x22,\x22Su\
perDagger\x22,\x22Supe\
rMinus\x22,\x22Superno\
vaData\x22,\x22SuperPl\
us\x22,\x22Superscript\
\x22,\x22SuperscriptBo\
x\x22,\x22SuperscriptB\
oxOptions\x22,\x22Supe\
rset\x22,\x22SupersetE\
qual\x22,\x22SuperStar\
\x22,\x22Surd\x22,\x22SurdFo\
rm\x22,\x22SurfaceAppe\
arance\x22,\x22Surface\
Area\x22,\x22SurfaceCo\
lor\x22,\x22SurfaceDat\
a\x22,\x22SurfaceGraph\
ics\x22,\x22SurvivalDi\
stribution\x22,\x22Sur\
vivalFunction\x22,\x22\
SurvivalModel\x22,\x22\
SurvivalModelFit\
\x22,\x22SuspendPacket\
\x22,\x22SuzukiDistrib\
ution\x22,\x22SuzukiGr\
oupSuz\x22,\x22SwatchL\
egend\x22,\x22Switch\x22,\
\x22Symbol\x22,\x22Symbol\
Name\x22,\x22SymletWav\
elet\x22,\x22Symmetric\
\x22,\x22SymmetricDiff\
erence\x22,\x22Symmetr\
icGroup\x22,\x22Symmet\
ricKey\x22,\x22Symmetr\
icMatrixQ\x22,\x22Symm\
etricPolynomial\x22\
,\x22SymmetricReduc\
tion\x22,\x22Symmetriz\
e\x22,\x22SymmetrizedA\
rray\x22,\x22Symmetriz\
edArrayRules\x22,\x22S\
ymmetrizedDepend\
entComponents\x22,\x22\
SymmetrizedIndep\
endentComponents\
\x22,\x22SymmetrizedRe\
placePart\x22,\x22Sync\
hronousInitializ\
ation\x22,\x22Synchron\
ousUpdating\x22,\x22Sy\
nonyms\x22,\x22Syntax\x22\
,\x22SyntaxForm\x22,\x22S\
yntaxInformation\
\x22,\x22SyntaxLength\x22\
,\x22SyntaxPacket\x22,\
\x22SyntaxQ\x22,\x22Synth\
esizeMissingValu\
es\x22,\x22SystemCrede\
ntial\x22,\x22SystemCr\
edentialData\x22,\x22S\
ystemCredentialK\
ey\x22,\x22SystemCrede\
ntialKeys\x22,\x22Syst\
emCredentialStor\
eObject\x22,\x22System\
DialogInput\x22,\x22Sy\
stemException\x22,\x22\
SystemGet\x22,\x22Syst\
emHelpPath\x22,\x22Sys\
temInformation\x22,\
\x22SystemInformati\
onData\x22,\x22SystemI\
nstall\x22,\x22SystemM\
odel\x22,\x22SystemMod\
eler\x22,\x22SystemMod\
elExamples\x22,\x22Sys\
temModelLineariz\
e\x22,\x22SystemModelM\
easurements\x22,\x22Sy\
stemModelParamet\
ricSimulate\x22,\x22Sy\
stemModelPlot\x22,\x22\
SystemModelProgr\
essReporting\x22,\x22S\
ystemModelReliab\
ility\x22,\x22SystemMo\
dels\x22,\x22SystemMod\
elSimulate\x22,\x22Sys\
temModelSimulate\
Sensitivity\x22,\x22Sy\
stemModelSimulat\
ionData\x22,\x22System\
Open\x22,\x22SystemOpt\
ions\x22,\x22SystemPro\
cessData\x22,\x22Syste\
mProcesses\x22,\x22Sys\
temsConnectionsM\
odel\x22,\x22SystemsMo\
delControllerDat\
a\x22,\x22SystemsModel\
Delay\x22,\x22SystemsM\
odelDelayApproxi\
mate\x22,\x22SystemsMo\
delDelete\x22,\x22Syst\
emsModelDimensio\
ns\x22,\x22SystemsMode\
lExtract\x22,\x22Syste\
msModelFeedbackC\
onnect\x22,\x22Systems\
ModelLabels\x22,\x22Sy\
stemsModelLinear\
ity\x22,\x22SystemsMod\
elMerge\x22,\x22System\
sModelOrder\x22,\x22Sy\
stemsModelParall\
elConnect\x22,\x22Syst\
emsModelSeriesCo\
nnect\x22,\x22SystemsM\
odelStateFeedbac\
kConnect\x22,\x22Syste\
msModelVectorRel\
ativeOrders\x22,\x22Sy\
stemStub\x22,\x22Syste\
mTest\x22,\x22Tab\x22,\x22Ta\
bFilling\x22,\x22Table\
\x22,\x22TableAlignmen\
ts\x22,\x22TableDepth\x22\
,\x22TableDirection\
s\x22,\x22TableForm\x22,\x22\
TableHeadings\x22,\x22\
TableSpacing\x22,\x22T\
ableView\x22,\x22Table\
ViewBox\x22,\x22TableV\
iewBoxAlignment\x22\
,\x22TableViewBoxBa\
ckground\x22,\x22Table\
ViewBoxHeaders\x22,\
\x22TableViewBoxIte\
mSize\x22,\x22TableVie\
wBoxItemStyle\x22,\x22\
TableViewBoxOpti\
ons\x22,\x22TabSpacing\
s\x22,\x22TabView\x22,\x22Ta\
bViewBox\x22,\x22TabVi\
ewBoxOptions\x22,\x22T\
agBox\x22,\x22TagBoxNo\
te\x22,\x22TagBoxOptio\
ns\x22,\x22TaggingRule\
s\x22,\x22TagSet\x22,\x22Tag\
SetDelayed\x22,\x22Tag\
Style\x22,\x22TagUnset\
\x22,\x22Take\x22,\x22TakeDr\
op\x22,\x22TakeLargest\
\x22,\x22TakeLargestBy\
\x22,\x22TakeList\x22,\x22Ta\
keSmallest\x22,\x22Tak\
eSmallestBy\x22,\x22Ta\
keWhile\x22,\x22Tally\x22\
,\x22Tan\x22,\x22Tanh\x22,\x22T\
argetDevice\x22,\x22Ta\
rgetFunctions\x22,\x22\
TargetSystem\x22,\x22T\
argetUnits\x22,\x22Tas\
kAbort\x22,\x22TaskExe\
cute\x22,\x22TaskObjec\
t\x22,\x22TaskRemove\x22,\
\x22TaskResume\x22,\x22Ta\
sks\x22,\x22TaskSuspen\
d\x22,\x22TaskWait\x22,\x22T\
autologyQ\x22,\x22Tele\
graphProcess\x22,\x22T\
emplateApply\x22,\x22T\
emplateArgBox\x22,\x22\
TemplateBox\x22,\x22Te\
mplateBoxOptions\
\x22,\x22TemplateEvalu\
ate\x22,\x22TemplateEx\
pression\x22,\x22Templ\
ateIf\x22,\x22Template\
Object\x22,\x22Templat\
eSequence\x22,\x22Temp\
lateSlot\x22,\x22Templ\
ateSlotSequence\x22\
,\x22TemplateUneval\
uated\x22,\x22Template\
Verbatim\x22,\x22Templ\
ateWith\x22,\x22Tempor\
alData\x22,\x22Tempora\
lRegularity\x22,\x22Te\
mporary\x22,\x22Tempor\
aryVariable\x22,\x22Te\
nsorContract\x22,\x22T\
ensorDimensions\x22\
,\x22TensorExpand\x22,\
\x22TensorProduct\x22,\
\x22TensorQ\x22,\x22Tenso\
rRank\x22,\x22TensorRe\
duce\x22,\x22TensorSym\
metry\x22,\x22TensorTr\
anspose\x22,\x22Tensor\
Wedge\x22,\x22Terminat\
edEvaluation\x22,\x22T\
ernaryListPlot\x22,\
\x22TernaryPlotCorn\
ers\x22,\x22TestID\x22,\x22T\
estReport\x22,\x22Test\
ReportObject\x22,\x22T\
estResultObject\x22\
,\x22Tetrahedron\x22,\x22\
TetrahedronBox\x22,\
\x22TetrahedronBoxO\
ptions\x22,\x22TeXForm\
\x22,\x22TeXSave\x22,\x22Tex\
t\x22,\x22Text3DBox\x22,\x22\
Text3DBoxOptions\
\x22,\x22TextAlignment\
\x22,\x22TextBand\x22,\x22Te\
xtBoundingBox\x22,\x22\
TextBox\x22,\x22TextCa\
ses\x22,\x22TextCell\x22,\
\x22TextClipboardTy\
pe\x22,\x22TextContent\
s\x22,\x22TextData\x22,\x22T\
extElement\x22,\x22Tex\
tForm\x22,\x22TextGrid\
\x22,\x22TextJustifica\
tion\x22,\x22TextLine\x22\
,\x22TextPacket\x22,\x22T\
extParagraph\x22,\x22T\
extPosition\x22,\x22Te\
xtRecognize\x22,\x22Te\
xtSearch\x22,\x22TextS\
earchReport\x22,\x22Te\
xtSentences\x22,\x22Te\
xtString\x22,\x22TextS\
tructure\x22,\x22TextS\
tyle\x22,\x22TextTrans\
lation\x22,\x22Texture\
\x22,\x22TextureCoordi\
nateFunction\x22,\x22T\
extureCoordinate\
Scaling\x22,\x22TextWo\
rds\x22,\x22Therefore\x22\
,\x22ThermodynamicD\
ata\x22,\x22Thermomete\
rGauge\x22,\x22Thick\x22,\
\x22Thickness\x22,\x22Thi\
n\x22,\x22Thinning\x22,\x22T\
hisLink\x22,\x22Thomas\
PointProcess\x22,\x22T\
hompsonGroupTh\x22,\
\x22Thread\x22,\x22Thread\
ed\x22,\x22ThreadingLa\
yer\x22,\x22ThreeJSymb\
ol\x22,\x22Threshold\x22,\
\x22Through\x22,\x22Throw\
\x22,\x22ThueMorse\x22,\x22T\
humbnail\x22,\x22Thurs\
day\x22,\x22TickDirect\
ion\x22,\x22TickLabelO\
rientation\x22,\x22Tic\
kLabelPositionin\
g\x22,\x22TickLabels\x22,\
\x22TickLengths\x22,\x22T\
ickPositions\x22,\x22T\
icks\x22,\x22TicksStyl\
e\x22,\x22TideData\x22,\x22T\
ilde\x22,\x22TildeEqua\
l\x22,\x22TildeFullEqu\
al\x22,\x22TildeTilde\x22\
,\x22TimeConstraine\
d\x22,\x22TimeConstrai\
nt\x22,\x22TimeDirecti\
on\x22,\x22TimeFormat\x22\
,\x22TimeGoal\x22,\x22Tim\
elinePlot\x22,\x22Time\
Object\x22,\x22TimeObj\
ectQ\x22,\x22TimeRemai\
ning\x22,\x22Times\x22,\x22T\
imesBy\x22,\x22TimeSer\
ies\x22,\x22TimeSeries\
Aggregate\x22,\x22Time\
SeriesForecast\x22,\
\x22TimeSeriesInser\
t\x22,\x22TimeSeriesIn\
vertibility\x22,\x22Ti\
meSeriesMap\x22,\x22Ti\
meSeriesMapThrea\
d\x22,\x22TimeSeriesMo\
del\x22,\x22TimeSeries\
ModelFit\x22,\x22TimeS\
eriesResample\x22,\x22\
TimeSeriesRescal\
e\x22,\x22TimeSeriesSh\
ift\x22,\x22TimeSeries\
Thread\x22,\x22TimeSer\
iesWindow\x22,\x22Time\
System\x22,\x22TimeSys\
temConvert\x22,\x22Tim\
eUsed\x22,\x22TimeValu\
e\x22,\x22TimeWarpingC\
orrespondence\x22,\x22\
TimeWarpingDista\
nce\x22,\x22TimeZone\x22,\
\x22TimeZoneConvert\
\x22,\x22TimeZoneOffse\
t\x22,\x22Timing\x22,\x22Tin\
y\x22,\x22TitleGroupin\
g\x22,\x22TitsGroupT\x22,\
\x22ToBoxes\x22,\x22ToCha\
racterCode\x22,\x22ToC\
olor\x22,\x22ToContinu\
ousTimeModel\x22,\x22T\
oDate\x22,\x22Today\x22,\x22\
ToDiscreteTimeMo\
del\x22,\x22ToEntity\x22,\
\x22ToeplitzMatrix\x22\
,\x22ToExpression\x22,\
\x22ToFileName\x22,\x22To\
gether\x22,\x22Toggle\x22\
,\x22ToggleFalse\x22,\x22\
Toggler\x22,\x22Toggle\
rBar\x22,\x22TogglerBo\
x\x22,\x22TogglerBoxOp\
tions\x22,\x22ToHeldEx\
pression\x22,\x22ToInv\
ertibleTimeSerie\
s\x22,\x22TokenWords\x22,\
\x22Tolerance\x22,\x22ToL\
owerCase\x22,\x22Tomor\
row\x22,\x22ToNumberFi\
eld\x22,\x22TooBig\x22,\x22T\
ooltip\x22,\x22Tooltip\
Box\x22,\x22TooltipBox\
Options\x22,\x22Toolti\
pDelay\x22,\x22Tooltip\
Style\x22,\x22ToonShad\
ing\x22,\x22Top\x22,\x22TopH\
atTransform\x22,\x22To\
PolarCoordinates\
\x22,\x22TopologicalSo\
rt\x22,\x22ToRadicals\x22\
,\x22ToRawPointer\x22,\
\x22ToRules\x22,\x22Torus\
\x22,\x22TorusGraph\x22,\x22\
ToSphericalCoord\
inates\x22,\x22ToStrin\
g\x22,\x22Total\x22,\x22Tota\
lHeight\x22,\x22TotalL\
ayer\x22,\x22TotalVari\
ationFilter\x22,\x22To\
talWidth\x22,\x22Touch\
Position\x22,\x22Touch\
screenAutoZoom\x22,\
\x22TouchscreenCont\
rolPlacement\x22,\x22T\
oUpperCase\x22,\x22Tou\
rVideo\x22,\x22Tr\x22,\x22Tr\
ace\x22,\x22TraceAbove\
\x22,\x22TraceAction\x22,\
\x22TraceBackward\x22,\
\x22TraceDepth\x22,\x22Tr\
aceDialog\x22,\x22Trac\
eForward\x22,\x22Trace\
Internal\x22,\x22Trace\
Level\x22,\x22TraceOff\
\x22,\x22TraceOn\x22,\x22Tra\
ceOriginal\x22,\x22Tra\
cePrint\x22,\x22TraceS\
can\x22,\x22TrackCellC\
hangeTimes\x22,\x22Tra\
ckedSymbols\x22,\x22Tr\
ackingFunction\x22,\
\x22TracyWidomDistr\
ibution\x22,\x22Tradin\
gChart\x22,\x22Traditi\
onalForm\x22,\x22Tradi\
tionalFunctionNo\
tation\x22,\x22Traditi\
onalNotation\x22,\x22T\
raditionalOrder\x22\
,\x22TrainImageCont\
entDetector\x22,\x22Tr\
ainingProgressCh\
eckpointing\x22,\x22Tr\
ainingProgressFu\
nction\x22,\x22Trainin\
gProgressMeasure\
ments\x22,\x22Training\
ProgressReportin\
g\x22,\x22TrainingStop\
pingCriterion\x22,\x22\
TrainingUpdateSc\
hedule\x22,\x22TrainTe\
xtContentDetecto\
r\x22,\x22TransferFunc\
tionCancel\x22,\x22Tra\
nsferFunctionExp\
and\x22,\x22TransferFu\
nctionFactor\x22,\x22T\
ransferFunctionM\
odel\x22,\x22TransferF\
unctionPoles\x22,\x22T\
ransferFunctionT\
ransform\x22,\x22Trans\
ferFunctionZeros\
\x22,\x22Transformatio\
nClass\x22,\x22Transfo\
rmationFunction\x22\
,\x22Transformation\
Functions\x22,\x22Tran\
sformationMatrix\
\x22,\x22TransformedDi\
stribution\x22,\x22Tra\
nsformedField\x22,\x22\
TransformedProce\
ss\x22,\x22Transformed\
Region\x22,\x22Transit\
ionDirection\x22,\x22T\
ransitionDuratio\
n\x22,\x22TransitionEf\
fect\x22,\x22Transitiv\
eClosureGraph\x22,\x22\
TransitiveReduct\
ionGraph\x22,\x22Trans\
late\x22,\x22Translati\
onOptions\x22,\x22Tran\
slationTransform\
\x22,\x22Transliterate\
\x22,\x22Transparent\x22,\
\x22TransparentColo\
r\x22,\x22Transpose\x22,\x22\
TransposeLayer\x22,\
\x22TrapEnterKey\x22,\x22\
TrapSelection\x22,\x22\
TravelDirections\
\x22,\x22TravelDirecti\
onsData\x22,\x22Travel\
Distance\x22,\x22Trave\
lDistanceList\x22,\x22\
TravelMethod\x22,\x22T\
ravelTime\x22,\x22Tree\
\x22,\x22TreeCases\x22,\x22T\
reeChildren\x22,\x22Tr\
eeCount\x22,\x22TreeDa\
ta\x22,\x22TreeDelete\x22\
,\x22TreeDepth\x22,\x22Tr\
eeElementCoordin\
ates\x22,\x22TreeEleme\
ntLabel\x22,\x22TreeEl\
ementLabelFuncti\
on\x22,\x22TreeElement\
LabelStyle\x22,\x22Tre\
eElementShape\x22,\x22\
TreeElementShape\
Function\x22,\x22TreeE\
lementSize\x22,\x22Tre\
eElementSizeFunc\
tion\x22,\x22TreeEleme\
ntStyle\x22,\x22TreeEl\
ementStyleFuncti\
on\x22,\x22TreeExpress\
ion\x22,\x22TreeExtrac\
t\x22,\x22TreeFold\x22,\x22T\
reeForm\x22,\x22TreeGr\
aph\x22,\x22TreeGraphQ\
\x22,\x22TreeInsert\x22,\x22\
TreeLayout\x22,\x22Tre\
eLeafCount\x22,\x22Tre\
eLeafQ\x22,\x22TreeLea\
ves\x22,\x22TreeLevel\x22\
,\x22TreeMap\x22,\x22Tree\
MapAt\x22,\x22TreeOutl\
ine\x22,\x22TreePlot\x22,\
\x22TreePosition\x22,\x22\
TreeQ\x22,\x22TreeRepl\
acePart\x22,\x22TreeRu\
les\x22,\x22TreeScan\x22,\
\x22TreeSelect\x22,\x22Tr\
eeSize\x22,\x22TreeTra\
versalOrder\x22,\x22Tr\
endStyle\x22,\x22Trian\
gle\x22,\x22TriangleCe\
nter\x22,\x22TriangleC\
onstruct\x22,\x22Trian\
gleMeasurement\x22,\
\x22TriangleWave\x22,\x22\
TriangularDistri\
bution\x22,\x22Triangu\
lateMesh\x22,\x22Trig\x22\
,\x22TrigExpand\x22,\x22T\
rigFactor\x22,\x22Trig\
FactorList\x22,\x22Tri\
gger\x22,\x22TrigReduc\
e\x22,\x22TrigToExp\x22,\x22\
TrimmedMean\x22,\x22Tr\
immedVariance\x22,\x22\
TropicalStormDat\
a\x22,\x22True\x22,\x22TrueQ\
\x22,\x22TruncatedDist\
ribution\x22,\x22Trunc\
atedPolyhedron\x22,\
\x22TsallisQExponen\
tialDistribution\
\x22,\x22TsallisQGauss\
ianDistribution\x22\
,\x22TTest\x22,\x22Tube\x22,\
\x22TubeBezierCurve\
Box\x22,\x22TubeBezier\
CurveBoxOptions\x22\
,\x22TubeBox\x22,\x22Tube\
BoxOptions\x22,\x22Tub\
eBSplineCurveBox\
\x22,\x22TubeBSplineCu\
rveBoxOptions\x22,\x22\
Tuesday\x22,\x22TukeyL\
ambdaDistributio\
n\x22,\x22TukeyWindow\x22\
,\x22TunnelData\x22,\x22T\
uples\x22,\x22TuranGra\
ph\x22,\x22TuringMachi\
ne\x22,\x22TuttePolyno\
mial\x22,\x22TwoWayRul\
e\x22,\x22Typed\x22,\x22Type\
Declaration\x22,\x22Ty\
peEvaluate\x22,\x22Typ\
eHint\x22,\x22TypeOf\x22,\
\x22TypeSpecifier\x22,\
\x22UnateQ\x22,\x22Uncomp\
ress\x22,\x22Unconstra\
inedParameters\x22,\
\x22Undefined\x22,\x22Und\
erBar\x22,\x22Underflo\
w\x22,\x22Underlined\x22,\
\x22Underoverscript\
\x22,\x22Underoverscri\
ptBox\x22,\x22Underove\
rscriptBoxOption\
s\x22,\x22Underscript\x22\
,\x22UnderscriptBox\
\x22,\x22UnderscriptBo\
xOptions\x22,\x22Under\
seaFeatureData\x22,\
\x22UndirectedEdge\x22\
,\x22UndirectedGrap\
h\x22,\x22UndirectedGr\
aphQ\x22,\x22UndoOptio\
ns\x22,\x22UndoTracked\
Variables\x22,\x22Uneq\
ual\x22,\x22UnequalTo\x22\
,\x22Unevaluated\x22,\x22\
UniformDistribut\
ion\x22,\x22UniformGra\
phDistribution\x22,\
\x22UniformPolyhedr\
on\x22,\x22UniformSumD\
istribution\x22,\x22Un\
install\x22,\x22Union\x22\
,\x22UnionedEntityC\
lass\x22,\x22UnionPlus\
\x22,\x22Unique\x22,\x22Uniq\
ueElements\x22,\x22Uni\
taryMatrixQ\x22,\x22Un\
itBox\x22,\x22UnitConv\
ert\x22,\x22UnitDimens\
ions\x22,\x22Unitize\x22,\
\x22UnitRootTest\x22,\x22\
UnitSimplify\x22,\x22U\
nitStep\x22,\x22UnitSy\
stem\x22,\x22UnitTrian\
gle\x22,\x22UnitVector\
\x22,\x22UnitVectorLay\
er\x22,\x22UnityDimens\
ions\x22,\x22UniverseM\
odelData\x22,\x22Unive\
rsityData\x22,\x22Unix\
Time\x22,\x22Unlabeled\
Tree\x22,\x22UnmanageO\
bject\x22,\x22Unprotec\
t\x22,\x22UnregisterEx\
ternalEvaluator\x22\
,\x22UnsameQ\x22,\x22Unsa\
vedVariables\x22,\x22U\
nset\x22,\x22UnsetShar\
ed\x22,\x22Until\x22,\x22Unt\
rackedVariables\x22\
,\x22Up\x22,\x22UpArrow\x22,\
\x22UpArrowBar\x22,\x22Up\
ArrowDownArrow\x22,\
\x22Update\x22,\x22Update\
DynamicObjects\x22,\
\x22UpdateDynamicOb\
jectsSynchronous\
\x22,\x22UpdateInterva\
l\x22,\x22UpdatePaclet\
Sites\x22,\x22UpdateSe\
archIndex\x22,\x22UpDo\
wnArrow\x22,\x22UpEqui\
librium\x22,\x22UpperC\
aseQ\x22,\x22UpperLeft\
Arrow\x22,\x22UpperRig\
htArrow\x22,\x22UpperT\
riangularize\x22,\x22U\
pperTriangularMa\
trix\x22,\x22UpperTria\
ngularMatrixQ\x22,\x22\
Upsample\x22,\x22UpSet\
\x22,\x22UpSetDelayed\x22\
,\x22UpTee\x22,\x22UpTeeA\
rrow\x22,\x22UpTo\x22,\x22Up\
Values\x22,\x22URL\x22,\x22U\
RLBuild\x22,\x22URLDec\
ode\x22,\x22URLDispatc\
her\x22,\x22URLDownloa\
d\x22,\x22URLDownloadS\
ubmit\x22,\x22URLEncod\
e\x22,\x22URLExecute\x22,\
\x22URLExpand\x22,\x22URL\
Fetch\x22,\x22URLFetch\
Asynchronous\x22,\x22U\
RLParse\x22,\x22URLQue\
ryDecode\x22,\x22URLQu\
eryEncode\x22,\x22URLR\
ead\x22,\x22URLRespons\
eTime\x22,\x22URLSave\x22\
,\x22URLSaveAsynchr\
onous\x22,\x22URLShort\
en\x22,\x22URLSubmit\x22,\
\x22UseEmbeddedLibr\
ary\x22,\x22UseGraphic\
sRange\x22,\x22UserDef\
inedWavelet\x22,\x22Us\
ing\x22,\x22UsingFront\
End\x22,\x22UtilityFun\
ction\x22,\x22V2Get\x22,\x22\
ValenceErrorHand\
ling\x22,\x22ValenceFi\
lling\x22,\x22Validati\
onLength\x22,\x22Valid\
ationSet\x22,\x22Value\
Box\x22,\x22ValueBoxOp\
tions\x22,\x22ValueDim\
ensions\x22,\x22ValueF\
orm\x22,\x22ValuePrepr\
ocessingFunction\
\x22,\x22ValueQ\x22,\x22Valu\
es\x22,\x22ValuesData\x22\
,\x22VandermondeMat\
rix\x22,\x22Variables\x22\
,\x22Variance\x22,\x22Var\
ianceEquivalence\
Test\x22,\x22VarianceE\
stimatorFunction\
\x22,\x22VarianceGamma\
Distribution\x22,\x22V\
arianceGammaPoin\
tProcess\x22,\x22Varia\
nceTest\x22,\x22Variog\
ramFunction\x22,\x22Va\
riogramModel\x22,\x22V\
ectorAngle\x22,\x22Vec\
torAround\x22,\x22Vect\
orAspectRatio\x22,\x22\
VectorColorFunct\
ion\x22,\x22VectorColo\
rFunctionScaling\
\x22,\x22VectorDensity\
Plot\x22,\x22VectorDis\
placementPlot\x22,\x22\
VectorDisplaceme\
ntPlot3D\x22,\x22Vecto\
rGlyphData\x22,\x22Vec\
torGreater\x22,\x22Vec\
torGreaterEqual\x22\
,\x22VectorLess\x22,\x22V\
ectorLessEqual\x22,\
\x22VectorMarkers\x22,\
\x22VectorPlot\x22,\x22Ve\
ctorPlot3D\x22,\x22Vec\
torPoints\x22,\x22Vect\
orQ\x22,\x22VectorRang\
e\x22,\x22Vectors\x22,\x22Ve\
ctorScale\x22,\x22Vect\
orScaling\x22,\x22Vect\
orSizes\x22,\x22Vector\
Style\x22,\x22Vee\x22,\x22Ve\
rbatim\x22,\x22Verbose\
\x22,\x22VerificationT\
est\x22,\x22VerifyConv\
ergence\x22,\x22Verify\
DerivedKey\x22,\x22Ver\
ifyDigitalSignat\
ure\x22,\x22VerifyFile\
Signature\x22,\x22Veri\
fyInterpretation\
\x22,\x22VerifySecurit\
yCertificates\x22,\x22\
VerifySolutions\x22\
,\x22VerifyTestAssu\
mptions\x22,\x22Versio\
nedPreferences\x22,\
\x22VertexAdd\x22,\x22Ver\
texCapacity\x22,\x22Ve\
rtexChromaticNum\
ber\x22,\x22VertexColo\
rs\x22,\x22VertexCompo\
nent\x22,\x22VertexCon\
nectivity\x22,\x22Vert\
exContract\x22,\x22Ver\
texCoordinateRul\
es\x22,\x22VertexCoord\
inates\x22,\x22VertexC\
orrelationSimila\
rity\x22,\x22VertexCos\
ineSimilarity\x22,\x22\
VertexCount\x22,\x22Ve\
rtexCoverQ\x22,\x22Ver\
texDataCoordinat\
es\x22,\x22VertexDegre\
e\x22,\x22VertexDelete\
\x22,\x22VertexDiceSim\
ilarity\x22,\x22Vertex\
Eccentricity\x22,\x22V\
ertexInComponent\
\x22,\x22VertexInCompo\
nentGraph\x22,\x22Vert\
exInDegree\x22,\x22Ver\
texIndex\x22,\x22Verte\
xJaccardSimilari\
ty\x22,\x22VertexLabel\
ing\x22,\x22VertexLabe\
ls\x22,\x22VertexLabel\
Style\x22,\x22VertexLi\
st\x22,\x22VertexNorma\
ls\x22,\x22VertexOutCo\
mponent\x22,\x22Vertex\
OutComponentGrap\
h\x22,\x22VertexOutDeg\
ree\x22,\x22VertexQ\x22,\x22\
VertexRenderingF\
unction\x22,\x22Vertex\
Replace\x22,\x22Vertex\
Shape\x22,\x22VertexSh\
apeFunction\x22,\x22Ve\
rtexSize\x22,\x22Verte\
xStyle\x22,\x22VertexT\
extureCoordinate\
s\x22,\x22VertexTransi\
tiveGraphQ\x22,\x22Ver\
texWeight\x22,\x22Vert\
exWeightedGraphQ\
\x22,\x22Vertical\x22,\x22Ve\
rticalBar\x22,\x22Vert\
icalForm\x22,\x22Verti\
calGauge\x22,\x22Verti\
calSeparator\x22,\x22V\
erticalSlider\x22,\x22\
VerticalTilde\x22,\x22\
Video\x22,\x22VideoCap\
ture\x22,\x22VideoComb\
ine\x22,\x22VideoDelet\
e\x22,\x22VideoEncodin\
g\x22,\x22VideoExtract\
Frames\x22,\x22VideoFr\
ameList\x22,\x22VideoF\
rameMap\x22,\x22VideoG\
enerator\x22,\x22Video\
Insert\x22,\x22VideoIn\
tervals\x22,\x22VideoJ\
oin\x22,\x22VideoMap\x22,\
\x22VideoMapList\x22,\x22\
VideoMapTimeSeri\
es\x22,\x22VideoPaddin\
g\x22,\x22VideoPause\x22,\
\x22VideoPlay\x22,\x22Vid\
eoQ\x22,\x22VideoRecor\
d\x22,\x22VideoReplace\
\x22,\x22VideoScreenCa\
pture\x22,\x22VideoSpl\
it\x22,\x22VideoStop\x22,\
\x22VideoStream\x22,\x22V\
ideoStreams\x22,\x22Vi\
deoTimeStretch\x22,\
\x22VideoTrackSelec\
tion\x22,\x22VideoTran\
scode\x22,\x22VideoTra\
nsparency\x22,\x22Vide\
oTrim\x22,\x22ViewAngl\
e\x22,\x22ViewCenter\x22,\
\x22ViewMatrix\x22,\x22Vi\
ewPoint\x22,\x22ViewPo\
intSelectorSetti\
ngs\x22,\x22ViewPort\x22,\
\x22ViewProjection\x22\
,\x22ViewRange\x22,\x22Vi\
ewVector\x22,\x22ViewV\
ertical\x22,\x22Virtua\
lGroupData\x22,\x22Vis\
ible\x22,\x22VisibleCe\
ll\x22,\x22VoiceStyleD\
ata\x22,\x22VoigtDistr\
ibution\x22,\x22Volcan\
oData\x22,\x22Volume\x22,\
\x22VonMisesDistrib\
ution\x22,\x22VoronoiM\
esh\x22,\x22WaitAll\x22,\x22\
WaitAsynchronous\
Task\x22,\x22WaitNext\x22\
,\x22WaitUntil\x22,\x22Wa\
kebyDistribution\
\x22,\x22WalleniusHype\
rgeometricDistri\
bution\x22,\x22WaringY\
uleDistribution\x22\
,\x22WarpingCorresp\
ondence\x22,\x22Warpin\
gDistance\x22,\x22Wate\
rshedComponents\x22\
,\x22WatsonUSquareT\
est\x22,\x22WattsStrog\
atzGraphDistribu\
tion\x22,\x22WaveletBe\
stBasis\x22,\x22Wavele\
tFilterCoefficie\
nts\x22,\x22WaveletIma\
gePlot\x22,\x22Wavelet\
ListPlot\x22,\x22Wavel\
etMapIndexed\x22,\x22W\
aveletMatrixPlot\
\x22,\x22WaveletPhi\x22,\x22\
WaveletPsi\x22,\x22Wav\
eletScale\x22,\x22Wave\
letScalogram\x22,\x22W\
aveletThreshold\x22\
,\x22WavePDECompone\
nt\x22,\x22WeaklyConne\
ctedComponents\x22,\
\x22WeaklyConnected\
GraphComponents\x22\
,\x22WeaklyConnecte\
dGraphQ\x22,\x22WeakSt\
ationarity\x22,\x22Wea\
therData\x22,\x22Weath\
erForecastData\x22,\
\x22WebAudioSearch\x22\
,\x22WebColumn\x22,\x22We\
bElementObject\x22,\
\x22WeberE\x22,\x22WebExe\
cute\x22,\x22WebImage\x22\
,\x22WebImageSearch\
\x22,\x22WebItem\x22,\x22Web\
PageMetaInformat\
ion\x22,\x22WebRow\x22,\x22W\
ebSearch\x22,\x22WebSe\
ssionObject\x22,\x22We\
bSessions\x22,\x22WebW\
indowObject\x22,\x22We\
dge\x22,\x22Wednesday\x22\
,\x22WeibullDistrib\
ution\x22,\x22Weierstr\
assE1\x22,\x22Weierstr\
assE2\x22,\x22Weierstr\
assE3\x22,\x22Weierstr\
assEta1\x22,\x22Weiers\
trassEta2\x22,\x22Weie\
rstrassEta3\x22,\x22We\
ierstrassHalfPer\
iods\x22,\x22Weierstra\
ssHalfPeriodW1\x22,\
\x22WeierstrassHalf\
PeriodW2\x22,\x22Weier\
strassHalfPeriod\
W3\x22,\x22Weierstrass\
InvariantG2\x22,\x22We\
ierstrassInvaria\
ntG3\x22,\x22Weierstra\
ssInvariants\x22,\x22W\
eierstrassP\x22,\x22We\
ierstrassPPrime\x22\
,\x22WeierstrassSig\
ma\x22,\x22Weierstrass\
Zeta\x22,\x22WeightedA\
djacencyGraph\x22,\x22\
WeightedAdjacenc\
yMatrix\x22,\x22Weight\
edData\x22,\x22Weighte\
dGraphQ\x22,\x22Weight\
s\x22,\x22WelchWindow\x22\
,\x22WheelGraph\x22,\x22W\
henEvent\x22,\x22Which\
\x22,\x22While\x22,\x22White\
\x22,\x22WhiteNoisePro\
cess\x22,\x22WhitePoin\
t\x22,\x22Whitespace\x22,\
\x22WhitespaceChara\
cter\x22,\x22Whittaker\
M\x22,\x22WhittakerW\x22,\
\x22WholeCellGroupO\
pener\x22,\x22WienerFi\
lter\x22,\x22WienerPro\
cess\x22,\x22WignerD\x22,\
\x22WignerSemicircl\
eDistribution\x22,\x22\
WikidataData\x22,\x22W\
ikidataSearch\x22,\x22\
WikipediaData\x22,\x22\
WikipediaSearch\x22\
,\x22WilksW\x22,\x22Wilks\
WTest\x22,\x22WindDire\
ctionData\x22,\x22Wind\
ingCount\x22,\x22Windi\
ngPolygon\x22,\x22Wind\
owClickSelect\x22,\x22\
WindowElements\x22,\
\x22WindowFloating\x22\
,\x22WindowFrame\x22,\x22\
WindowFrameEleme\
nts\x22,\x22WindowMarg\
ins\x22,\x22WindowMova\
ble\x22,\x22WindowOpac\
ity\x22,\x22WindowPers\
istentStyles\x22,\x22W\
indowSelected\x22,\x22\
WindowSize\x22,\x22Win\
dowStatusArea\x22,\x22\
WindowTitle\x22,\x22Wi\
ndowToolbars\x22,\x22W\
indowWidth\x22,\x22Win\
dSpeedData\x22,\x22Win\
dVectorData\x22,\x22Wi\
nsorizedMean\x22,\x22W\
insorizedVarianc\
e\x22,\x22WishartMatri\
xDistribution\x22,\x22\
With\x22,\x22WithClean\
up\x22,\x22WithLock\x22,\x22\
WolframAlpha\x22,\x22W\
olframAlphaDate\x22\
,\x22WolframAlphaQu\
antity\x22,\x22Wolfram\
AlphaResult\x22,\x22Wo\
lframCloudSettin\
gs\x22,\x22WolframLang\
uageData\x22,\x22Word\x22\
,\x22WordBoundary\x22,\
\x22WordCharacter\x22,\
\x22WordCloud\x22,\x22Wor\
dCount\x22,\x22WordCou\
nts\x22,\x22WordData\x22,\
\x22WordDefinition\x22\
,\x22WordFrequency\x22\
,\x22WordFrequencyD\
ata\x22,\x22WordList\x22,\
\x22WordOrientation\
\x22,\x22WordSearch\x22,\x22\
WordSelectionFun\
ction\x22,\x22WordSepa\
rators\x22,\x22WordSpa\
cings\x22,\x22WordStem\
\x22,\x22WordTranslati\
on\x22,\x22WorkingPrec\
ision\x22,\x22WrapArou\
nd\x22,\x22Write\x22,\x22Wri\
teLine\x22,\x22WriteSt\
ring\x22,\x22Wronskian\
\x22,\x22XMLElement\x22,\x22\
XMLObject\x22,\x22XMLT\
emplate\x22,\x22Xnor\x22,\
\x22Xor\x22,\x22XYZColor\x22\
,\x22Yellow\x22,\x22Yeste\
rday\x22,\x22YuleDissi\
milarity\x22,\x22Zerni\
keR\x22,\x22ZeroSymmet\
ric\x22,\x22ZeroTest\x22,\
\x22ZeroWidthTimes\x22\
,\x22Zeta\x22,\x22ZetaZer\
o\x22,\x22ZIPCodeData\x22\
,\x22ZipfDistributi\
on\x22,\x22ZoomCenter\x22\
,\x22ZoomFactor\x22,\x22Z\
Test\x22,\x22ZTransfor\
m\x22,\x22$Aborted\x22,\x22$\
ActivationGroupI\
D\x22,\x22$ActivationK\
ey\x22,\x22$Activation\
UserRegistered\x22,\
\x22$AddOnsDirector\
y\x22,\x22$AllowDataUp\
dates\x22,\x22$AllowEx\
ternalChannelFun\
ctions\x22,\x22$AllowI\
nternet\x22,\x22$Asser\
tFunction\x22,\x22$Ass\
umptions\x22,\x22$Asyn\
chronousTask\x22,\x22$\
AudioDecoders\x22,\x22\
$AudioEncoders\x22,\
\x22$AudioInputDevi\
ces\x22,\x22$AudioOutp\
utDevices\x22,\x22$Bas\
eDirectory\x22,\x22$Ba\
sePacletsDirecto\
ry\x22,\x22$BatchInput\
\x22,\x22$BatchOutput\x22\
,\x22$BlockchainBas\
e\x22,\x22$BoxForms\x22,\x22\
$ByteOrdering\x22,\x22\
$CacheBaseDirect\
ory\x22,\x22$Canceled\x22\
,\x22$ChannelBase\x22,\
\x22$CharacterEncod\
ing\x22,\x22$Character\
Encodings\x22,\x22$Clo\
udAccountName\x22,\x22\
$CloudBase\x22,\x22$Cl\
oudConnected\x22,\x22$\
CloudConnection\x22\
,\x22$CloudCreditsA\
vailable\x22,\x22$Clou\
dEvaluation\x22,\x22$C\
loudExpressionBa\
se\x22,\x22$CloudObjec\
tNameFormat\x22,\x22$C\
loudObjectURLTyp\
e\x22,\x22$CloudRootDi\
rectory\x22,\x22$Cloud\
SymbolBase\x22,\x22$Cl\
oudUserID\x22,\x22$Clo\
udUserUUID\x22,\x22$Cl\
oudVersion\x22,\x22$Cl\
oudVersionNumber\
\x22,\x22$CloudWolfram\
EngineVersionNum\
ber\x22,\x22$CommandLi\
ne\x22,\x22$Compilatio\
nTarget\x22,\x22$Compi\
lerEnvironment\x22,\
\x22$ConditionHold\x22\
,\x22$ConfiguredKer\
nels\x22,\x22$Context\x22\
,\x22$ContextAliase\
s\x22,\x22$ContextPath\
\x22,\x22$ControlActiv\
eSetting\x22,\x22$Cook\
ies\x22,\x22$CookieSto\
re\x22,\x22$CreationDa\
te\x22,\x22$Cryptograp\
hicEllipticCurve\
Names\x22,\x22$Current\
Link\x22,\x22$CurrentT\
ask\x22,\x22$CurrentWe\
bSession\x22,\x22$Data\
Structures\x22,\x22$Da\
teStringFormat\x22,\
\x22$DefaultAudioIn\
putDevice\x22,\x22$Def\
aultAudioOutputD\
evice\x22,\x22$Default\
Font\x22,\x22$DefaultF\
rontEnd\x22,\x22$Defau\
ltImagingDevice\x22\
,\x22$DefaultKernel\
s\x22,\x22$DefaultLoca\
lBase\x22,\x22$Default\
LocalKernel\x22,\x22$D\
efaultMailbox\x22,\x22\
$DefaultNetworkI\
nterface\x22,\x22$Defa\
ultPath\x22,\x22$Defau\
ltProxyRules\x22,\x22$\
DefaultRemoteBat\
chSubmissionEnvi\
ronment\x22,\x22$Defau\
ltRemoteKernel\x22,\
\x22$DefaultSystemC\
redentialStore\x22,\
\x22$Display\x22,\x22$Dis\
playFunction\x22,\x22$\
DistributedConte\
xts\x22,\x22$DynamicEv\
aluation\x22,\x22$Echo\
\x22,\x22$EmbedCodeEnv\
ironments\x22,\x22$Emb\
eddableServices\x22\
,\x22$EntityStores\x22\
,\x22$Epilog\x22,\x22$Eva\
luationCloudBase\
\x22,\x22$EvaluationCl\
oudObject\x22,\x22$Eva\
luationEnvironme\
nt\x22,\x22$ExportForm\
ats\x22,\x22$ExternalI\
dentifierTypes\x22,\
\x22$ExternalStorag\
eBase\x22,\x22$Failed\x22\
,\x22$FinancialData\
Source\x22,\x22$FontFa\
milies\x22,\x22$Format\
Type\x22,\x22$FrontEnd\
\x22,\x22$FrontEndSess\
ion\x22,\x22$Generated\
AssetLocation\x22,\x22\
$GeoEntityTypes\x22\
,\x22$GeoLocation\x22,\
\x22$GeoLocationCit\
y\x22,\x22$GeoLocation\
Country\x22,\x22$GeoLo\
cationPrecision\x22\
,\x22$GeoLocationSo\
urce\x22,\x22$HistoryL\
ength\x22,\x22$HomeDir\
ectory\x22,\x22$HTMLEx\
portRules\x22,\x22$HTT\
PCookies\x22,\x22$HTTP\
Request\x22,\x22$Ignor\
eEOF\x22,\x22$ImageFor\
mattingWidth\x22,\x22$\
ImageResolution\x22\
,\x22$ImagingDevice\
\x22,\x22$ImagingDevic\
es\x22,\x22$ImportForm\
ats\x22,\x22$IncomingM\
ailSettings\x22,\x22$I\
nitialDirectory\x22\
,\x22$Initializatio\
n\x22,\x22$Initializat\
ionContexts\x22,\x22$I\
nput\x22,\x22$InputFil\
eName\x22,\x22$InputSt\
reamMethods\x22,\x22$I\
nspector\x22,\x22$Inst\
allationDate\x22,\x22$\
InstallationDire\
ctory\x22,\x22$Interfa\
ceEnvironment\x22,\x22\
$InterpreterType\
s\x22,\x22$IterationLi\
mit\x22,\x22$KernelCou\
nt\x22,\x22$KernelID\x22,\
\x22$Language\x22,\x22$La\
unchDirectory\x22,\x22\
$LibraryPath\x22,\x22$\
LicenseExpiratio\
nDate\x22,\x22$License\
ID\x22,\x22$LicensePro\
cesses\x22,\x22$Licens\
eServer\x22,\x22$Licen\
seSubprocesses\x22,\
\x22$LicenseType\x22,\x22\
$Line\x22,\x22$Linked\x22\
,\x22$LinkSupported\
\x22,\x22$LoadedFiles\x22\
,\x22$LocalBase\x22,\x22$\
LocalSymbolBase\x22\
,\x22$MachineAddres\
ses\x22,\x22$MachineDo\
main\x22,\x22$MachineD\
omains\x22,\x22$Machin\
eEpsilon\x22,\x22$Mach\
ineID\x22,\x22$Machine\
Name\x22,\x22$MachineP\
recision\x22,\x22$Mach\
ineType\x22,\x22$MaxDi\
splayedChildren\x22\
,\x22$MaxExtraPreci\
sion\x22,\x22$MaxLicen\
seProcesses\x22,\x22$M\
axLicenseSubproc\
esses\x22,\x22$MaxMach\
ineNumber\x22,\x22$Max\
Number\x22,\x22$MaxPie\
cewiseCases\x22,\x22$M\
axPrecision\x22,\x22$M\
axRootDegree\x22,\x22$\
MessageGroups\x22,\x22\
$MessageList\x22,\x22$\
MessagePrePrint\x22\
,\x22$Messages\x22,\x22$M\
inMachineNumber\x22\
,\x22$MinNumber\x22,\x22$\
MinorReleaseNumb\
er\x22,\x22$MinPrecisi\
on\x22,\x22$MobilePhon\
e\x22,\x22$ModuleNumbe\
r\x22,\x22$NetworkConn\
ected\x22,\x22$Network\
Interfaces\x22,\x22$Ne\
tworkLicense\x22,\x22$\
NewMessage\x22,\x22$Ne\
wSymbol\x22,\x22$Noteb\
ookInlineStorage\
Limit\x22,\x22$Noteboo\
ks\x22,\x22$NoValue\x22,\x22\
$NumberMarks\x22,\x22$\
Off\x22,\x22$Operating\
System\x22,\x22$Output\
\x22,\x22$OutputForms\x22\
,\x22$OutputSizeLim\
it\x22,\x22$OutputStre\
amMethods\x22,\x22$Pac\
kages\x22,\x22$ParentL\
ink\x22,\x22$ParentPro\
cessID\x22,\x22$Passwo\
rdFile\x22,\x22$PatchL\
evelID\x22,\x22$Path\x22,\
\x22$PathnameSepara\
tor\x22,\x22$Performan\
ceGoal\x22,\x22$Permis\
sions\x22,\x22$Permiss\
ionsGroupBase\x22,\x22\
$PersistenceBase\
\x22,\x22$PersistenceP\
ath\x22,\x22$PipeSuppo\
rted\x22,\x22$PlotThem\
e\x22,\x22$Post\x22,\x22$Pre\
\x22,\x22$PreferencesD\
irectory\x22,\x22$PreI\
nitialization\x22,\x22\
$PrePrint\x22,\x22$Pre\
Read\x22,\x22$PrintFor\
ms\x22,\x22$PrintLiter\
al\x22,\x22$Printout3D\
Previewer\x22,\x22$Pro\
cessID\x22,\x22$Proces\
sorCount\x22,\x22$Proc\
essorType\x22,\x22$Pro\
ductInformation\x22\
,\x22$ProgramName\x22,\
\x22$ProgressReport\
ing\x22,\x22$Publisher\
ID\x22,\x22$RandomGene\
ratorState\x22,\x22$Ra\
ndomState\x22,\x22$Rec\
ursionLimit\x22,\x22$R\
egisteredDeviceC\
lasses\x22,\x22$Regist\
eredUserName\x22,\x22$\
ReleaseNumber\x22,\x22\
$RequesterAddres\
s\x22,\x22$RequesterCl\
oudUserID\x22,\x22$Req\
uesterCloudUserU\
UID\x22,\x22$Requester\
WolframID\x22,\x22$Req\
uesterWolframUUI\
D\x22,\x22$ResourceSys\
temBase\x22,\x22$Resou\
rceSystemPath\x22,\x22\
$RootDirectory\x22,\
\x22$ScheduledTask\x22\
,\x22$ScriptCommand\
Line\x22,\x22$ScriptIn\
putString\x22,\x22$Sec\
uredAuthenticati\
onKeyTokens\x22,\x22$S\
erviceCreditsAva\
ilable\x22,\x22$Servic\
es\x22,\x22$SessionID\x22\
,\x22$SetParentLink\
\x22,\x22$SharedFuncti\
ons\x22,\x22$SharedVar\
iables\x22,\x22$SoundD\
isplay\x22,\x22$SoundD\
isplayFunction\x22,\
\x22$SourceLink\x22,\x22$\
SSHAuthenticatio\
n\x22,\x22$SubtitleDec\
oders\x22,\x22$Subtitl\
eEncoders\x22,\x22$Sum\
maryBoxDataSizeL\
imit\x22,\x22$Suppress\
InputFormHeads\x22,\
\x22$SynchronousEva\
luation\x22,\x22$Synta\
xHandler\x22,\x22$Syst\
em\x22,\x22$SystemChar\
acterEncoding\x22,\x22\
$SystemCredentia\
lStore\x22,\x22$System\
ID\x22,\x22$SystemMemo\
ry\x22,\x22$SystemShel\
l\x22,\x22$SystemTimeZ\
one\x22,\x22$SystemWor\
dLength\x22,\x22$Targe\
tSystems\x22,\x22$Temp\
latePath\x22,\x22$Temp\
oraryDirectory\x22,\
\x22$TemporaryPrefi\
x\x22,\x22$TestFileNam\
e\x22,\x22$TextStyle\x22,\
\x22$TimedOut\x22,\x22$Ti\
meUnit\x22,\x22$TimeZo\
ne\x22,\x22$TimeZoneEn\
tity\x22,\x22$TopDirec\
tory\x22,\x22$TraceOff\
\x22,\x22$TraceOn\x22,\x22$T\
racePattern\x22,\x22$T\
racePostAction\x22,\
\x22$TracePreAction\
\x22,\x22$UnitSystem\x22,\
\x22$Urgent\x22,\x22$User\
AddOnsDirectory\x22\
,\x22$UserAgentLang\
uages\x22,\x22$UserAge\
ntMachine\x22,\x22$Use\
rAgentName\x22,\x22$Us\
erAgentOperating\
System\x22,\x22$UserAg\
entString\x22,\x22$Use\
rAgentVersion\x22,\x22\
$UserBaseDirecto\
ry\x22,\x22$UserBasePa\
cletsDirectory\x22,\
\x22$UserDocumentsD\
irectory\x22,\x22$User\
name\x22,\x22$UserName\
\x22,\x22$UserURLBase\x22\
,\x22$Version\x22,\x22$Ve\
rsionNumber\x22,\x22$V\
ideoDecoders\x22,\x22$\
VideoEncoders\x22,\x22\
$VoiceStyles\x22,\x22$\
WolframDocuments\
Directory\x22,\x22$Wol\
framID\x22,\x22$Wolfra\
mUUID\x22]\x0a;return \
t=>{\x0aconst i=t.r\
egex,o=i.either(\
i.concat(/([2-9]\
|[1-2]\x5cd|[3][0-5\
])\x5c^\x5c^/,/(\x5cw*\x5c.\x5c\
w+|\x5cw+\x5c.\x5cw*|\x5cw+)\
/),/(\x5cd*\x5c.\x5cd+|\x5cd\
+\x5c.\x5cd*|\x5cd+)/),a=\
i.either(/``[+-]\
?(\x5cd*\x5c.\x5cd+|\x5cd+\x5c.\
\x5cd*|\x5cd+)/,/`([+-\
]?(\x5cd*\x5c.\x5cd+|\x5cd+\x5c\
.\x5cd*|\x5cd+))?/),n=\
{\x0aclassName:\x22num\
ber\x22,relevance:0\
,\x0abegin:i.concat\
(o,i.optional(a)\
,i.optional(/\x5c*\x5c\
^[+-]?\x5cd+/))\x0a},r\
=/[a-zA-Z$][a-zA\
-Z0-9$]*/,l=new \
Set(e),s={varian\
ts:[{\x0aclassName:\
\x22builtin-symbol\x22\
,begin:r,\x22on:beg\
in\x22:(e,t)=>{\x0al.h\
as(e[0])||t.igno\
reMatch()}},{cla\
ssName:\x22symbol\x22,\
relevance:0,begi\
n:r}]},c={\x0aclass\
Name:\x22message-na\
me\x22,relevance:0,\
begin:i.concat(\x22\
::\x22,r)};return{\x0a\
name:\x22Mathematic\
a\x22,aliases:[\x22mma\
\x22,\x22wl\x22],classNam\
eAliases:{brace:\
\x22punctuation\x22,\x0ap\
attern:\x22type\x22,sl\
ot:\x22type\x22,symbol\
:\x22variable\x22,\x22nam\
ed-character\x22:\x22v\
ariable\x22,\x0a\x22built\
in-symbol\x22:\x22buil\
t_in\x22,\x22message-n\
ame\x22:\x22string\x22},\x0a\
contains:[t.COMM\
ENT(/\x5c(\x5c*/,/\x5c*\x5c)\
/,{contains:[\x22se\
lf\x22]}),{classNam\
e:\x22pattern\x22,\x0arel\
evance:0,begin:/\
([a-zA-Z$][a-zA-\
Z0-9$]*)?_+([a-z\
A-Z$][a-zA-Z0-9$\
]*)?/},{\x0aclassNa\
me:\x22slot\x22,releva\
nce:0,begin:/#[a\
-zA-Z$][a-zA-Z0-\
9$]*|#+[0-9]?/},\
c,s,{\x0aclassName:\
\x22named-character\
\x22,begin:/\x5c\x5c\x5c[[$a\
-zA-Z][$a-zA-Z0-\
9]+\x5c]/\x0a},t.QUOTE\
_STRING_MODE,n,{\
className:\x22opera\
tor\x22,relevance:0\
,\x0abegin:/[+\x5c-*/,\
;.:@~=><&|_`'^?!\
%]+/},{className\
:\x22brace\x22,relevan\
ce:0,\x0abegin:/[[\x5c\
](){}]/}]}}})();\
hljs.registerLan\
guage(\x22mathemati\
ca\x22,e)})();/*! `\
matlab` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>{c\
onst a=\x22('|\x5c\x5c.')\
+\x22,s={relevance:\
0,\x0acontains:[{be\
gin:a}]};return{\
name:\x22Matlab\x22,ke\
ywords:{\x0akeyword\
:\x22arguments brea\
k case catch cla\
ssdef continue e\
lse elseif end e\
numeration event\
s for function g\
lobal if methods\
 otherwise parfo\
r persistent pro\
perties return s\
pmd switch try w\
hile\x22,\x0abuilt_in:\
\x22sin sind sinh a\
sin asind asinh \
cos cosd cosh ac\
os acosd acosh t\
an tand tanh ata\
n atand atan2 at\
anh sec secd sec\
h asec asecd ase\
ch csc cscd csch\
 acsc acscd acsc\
h cot cotd coth \
acot acotd acoth\
 hypot exp expm1\
 log log1p log10\
 log2 pow2 realp\
ow reallog reals\
qrt sqrt nthroot\
 nextpow2 abs an\
gle complex conj\
 imag real unwra\
p isreal cplxpai\
r fix floor ceil\
 round mod rem s\
ign airy besselj\
 bessely besselh\
 besseli besselk\
 beta betainc be\
taln ellipj elli\
pke erf erfc erf\
cx erfinv expint\
 gamma gammainc \
gammaln psi lege\
ndre cross dot f\
actor isprime pr\
imes gcd lcm rat\
 rats perms ncho\
osek factorial c\
art2sph cart2pol\
 pol2cart sph2ca\
rt hsv2rgb rgb2h\
sv zeros ones ey\
e repmat rand ra\
ndn linspace log\
space freqspace \
meshgrid accumar\
ray size length \
ndims numel disp\
 isempty isequal\
 isequalwithequa\
lnans cat reshap\
e diag blkdiag t\
ril triu fliplr \
flipud flipdim r\
ot90 find sub2in\
d ind2sub bsxfun\
 ndgrid permute \
ipermute shiftdi\
m circshift sque\
eze isscalar isv\
ector ans eps re\
almax realmin pi\
 i|0 inf nan isn\
an isinf isfinit\
e j|0 why compan\
 gallery hadamar\
d hankel hilb in\
vhilb magic pasc\
al rosser toepli\
tz vander wilkin\
son max min nanm\
ax nanmin mean n\
anmean type tabl\
e readtable writ\
etable sortrows \
sort figure plot\
 plot3 scatter s\
catter3 cellfun \
legend intersect\
 ismember procru\
stes hold num2ce\
ll \x22\x0a},illegal:'\
(//|\x22|#|/\x5c\x5c*|\x5c\x5cs\
+/\x5c\x5cw+)',contain\
s:[{className:\x22f\
unction\x22,\x0abeginK\
eywords:\x22functio\
n\x22,end:\x22$\x22,conta\
ins:[e.UNDERSCOR\
E_TITLE_MODE,{\x0ac\
lassName:\x22params\
\x22,variants:[{beg\
in:\x22\x5c\x5c(\x22,end:\x22\x5c\x5c\
)\x22},{begin:\x22\x5c\x5c[\x22\
,end:\x22\x5c\x5c]\x22}]}]\x0a}\
,{className:\x22bui\
lt_in\x22,begin:/tr\
ue|false/,releva\
nce:0,starts:s},\
{\x0abegin:\x22[a-zA-Z\
][a-zA-Z_0-9]*\x22+\
a,relevance:0},{\
className:\x22numbe\
r\x22,\x0abegin:e.C_NU\
MBER_RE,relevanc\
e:0,starts:s},{c\
lassName:\x22string\
\x22,begin:\x22'\x22,end:\
\x22'\x22,\x0acontains:[{\
begin:\x22''\x22}]},{b\
egin:/\x5c]|\x5c}|\x5c)/,\
relevance:0,star\
ts:s},{\x0aclassNam\
e:\x22string\x22,begin\
:'\x22',end:'\x22',con\
tains:[{begin:'\x22\
\x22'}],starts:s\x0a},\
e.COMMENT(\x22^\x5c\x5cs*\
%\x5c\x5c{\x5c\x5cs*$\x22,\x22^\x5c\x5cs\
*%\x5c\x5c}\x5c\x5cs*$\x22),e.C\
OMMENT(\x22%\x22,\x22$\x22)]\
}}})()\x0a;hljs.reg\
isterLanguage(\x22m\
atlab\x22,e)})();/*\
! `maxima` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>({name:\x22Maxima\x22\
,keywords:{\x0a$pat\
tern:\x22[A-Za-z_%]\
[0-9A-Za-z_%]*\x22,\
\x0akeyword:\x22if the\
n else elseif fo\
r thru do while \
unless step in a\
nd or not\x22,\x0alite\
ral:\x22true false \
unknown inf minf\
 ind und %e %i %\
pi %phi %gamma\x22,\
\x0abuilt_in:\x22 abas\
ep abs absint ab\
solute_real_time\
 acos acosh acot\
 acoth acsc acsc\
h activate addco\
l add_edge add_e\
dges addmatrices\
 addrow add_vert\
ex add_vertices \
adjacency_matrix\
 adjoin adjoint \
af agd airy airy\
_ai airy_bi airy\
_dai airy_dbi al\
gsys alg_type al\
ias allroots alp\
hacharp alphanum\
ericp amortizati\
on %and annuity_\
fv annuity_pv an\
tid antidiff Ant\
iDifference appe\
nd appendfile ap\
ply apply1 apply\
2 applyb1 apropo\
s args arit_amor\
tization arithme\
tic arithsum arr\
ay arrayapply ar\
rayinfo arraymak\
e arraysetapply \
ascii asec asech\
 asin asinh aski\
nteger asksign a\
ssoc assoc_legen\
dre_p assoc_lege\
ndre_q assume as\
sume_external_by\
te_order asympa \
at atan atan2 at\
anh atensimp ato\
m atvalue augcoe\
fmatrix augmente\
d_lagrangian_met\
hod av average_d\
egree backtrace \
bars barsplot ba\
rsplot_descripti\
on base64 base64\
_decode bashindi\
ces batch batchl\
oad bc2 bdvac be\
lln benefit_cost\
 bern bernpoly b\
ernstein_approx \
bernstein_expand\
 bernstein_poly \
bessel bessel_i \
bessel_j bessel_\
k bessel_simplif\
y bessel_y beta \
beta_incomplete \
beta_incomplete_\
generalized beta\
_incomplete_regu\
larized bezout b\
fallroots bffac \
bf_find_root bf_\
fmin_cobyla bfhz\
eta bfloat bfloa\
tp bfpsi bfpsi0 \
bfzeta biconnect\
ed_components bi\
metric binomial \
bipartition bloc\
k blockmatrixp b\
ode_gain bode_ph\
ase bothcoef box\
 boxplot boxplot\
_description bre\
ak bug_report bu\
ild_info|10 buil\
dq build_sample \
burn cabs canfor\
m canten cardina\
lity carg cartan\
 cartesian_produ\
ct catch cauchy_\
matrix cbffac cd\
f_bernoulli cdf_\
beta cdf_binomia\
l cdf_cauchy cdf\
_chi2 cdf_contin\
uous_uniform cdf\
_discrete_unifor\
m cdf_exp cdf_f \
cdf_gamma cdf_ge\
neral_finite_dis\
crete cdf_geomet\
ric cdf_gumbel c\
df_hypergeometri\
c cdf_laplace cd\
f_logistic cdf_l\
ognormal cdf_neg\
ative_binomial c\
df_noncentral_ch\
i2 cdf_noncentra\
l_student_t cdf_\
normal cdf_paret\
o cdf_poisson cd\
f_rank_sum cdf_r\
ayleigh cdf_sign\
ed_rank cdf_stud\
ent_t cdf_weibul\
l cdisplay ceili\
ng central_momen\
t cequal cequali\
gnore cf cfdisre\
p cfexpand cgeod\
esic cgreaterp c\
greaterpignore c\
hangename change\
var chaosgame ch\
arat charfun cha\
rfun2 charlist c\
harp charpoly ch\
dir chebyshev_t \
chebyshev_u chec\
kdiv check_overl\
aps chinese chol\
esky christof ch\
romatic_index ch\
romatic_number c\
int circulant_gr\
aph clear_edge_w\
eight clear_rule\
s clear_vertex_l\
abel clebsch_gor\
dan clebsch_grap\
h clessp clesspi\
gnore close clos\
efile cmetric co\
eff coefmatrix c\
ograd col collap\
se collectterms \
columnop columns\
pace columnswap \
columnvector com\
bination combine\
 comp2pui compar\
e compfile compi\
le compile_file \
complement_graph\
 complete_bipart\
ite_graph comple\
te_graph complex\
_number_p compon\
ents compose_fun\
ctions concan co\
ncat conjugate c\
onmetderiv conne\
cted_components \
connect_vertices\
 cons constant c\
onstantp constit\
uent constvalue \
cont2part conten\
t continuous_fre\
q contortion con\
tour_plot contra\
ct contract_edge\
 contragrad cont\
rib_ode convert \
coord copy copy_\
file copy_graph \
copylist copymat\
rix cor cos cosh\
 cot coth cov co\
v1 covdiff covec\
t covers crc24su\
m create_graph c\
reate_list csc c\
sch csetup cspli\
ne ctaylor ct_co\
ordsys ctransfor\
m ctranspose cub\
e_graph cuboctah\
edron_graph cunl\
isp cv cycle_dig\
raph cycle_graph\
 cylindrical day\
s360 dblint deac\
tivate declare d\
eclare_constvalu\
e declare_dimens\
ions declare_fun\
damental_dimensi\
ons declare_fund\
amental_units de\
clare_qty declar\
e_translated dec\
lare_unit_conver\
sion declare_uni\
ts declare_weigh\
ts decsym defcon\
 define define_a\
lt_display defin\
e_variable defin\
t defmatch defru\
le defstruct def\
taylor degree_se\
quence del delet\
e deleten delta \
demo demoivre de\
nom depends deri\
vdegree derivlis\
t describe desol\
ve determinant d\
float dgauss_a d\
gauss_b dgeev dg\
emm dgeqrf dgesv\
 dgesvd diag dia\
gmatrix diag_mat\
rix diagmatrixp \
diameter diff di\
gitcharp dimacs_\
export dimacs_im\
port dimension d\
imensionless dim\
ensions dimensio\
ns_as_list direc\
t directory disc\
rete_freq disjoi\
n disjointp diso\
late disp dispco\
n dispform dispf\
un dispJordan di\
splay disprule d\
ispterms distrib\
 divide divisors\
 divsum dkummer_\
m dkummer_u dlan\
ge dodecahedron_\
graph dotproduct\
 dotsimp dpart d\
raw draw2d draw3\
d drawdf draw_fi\
le draw_graph ds\
calar echelon ed\
ge_coloring edge\
_connectivity ed\
ges eigens_by_ja\
cobi eigenvalues\
 eigenvectors ei\
ghth einstein ei\
vals eivects ela\
psed_real_time e\
lapsed_run_time \
ele2comp ele2pol\
ynome ele2pui el\
em elementp elev\
ation_grid elim \
elim_allbut elim\
inate eliminate_\
using ellipse el\
liptic_e ellipti\
c_ec elliptic_eu\
 elliptic_f elli\
ptic_kc elliptic\
_pi ematrix empt\
y_graph emptyp e\
ndcons entermatr\
ix entertensor e\
ntier equal equa\
lp equiv_classes\
 erf erfc erf_ge\
neralized erfi e\
rrcatch error er\
rormsg errors eu\
ler ev eval_stri\
ng evenp every e\
volution evoluti\
on2d evundiff ex\
ample exp expand\
 expandwrt expan\
dwrt_factored ex\
pint expintegral\
_chi expintegral\
_ci expintegral_\
e expintegral_e1\
 expintegral_ei \
expintegral_e_si\
mplify expintegr\
al_li expintegra\
l_shi expintegra\
l_si explicit ex\
plose exponentia\
lize express exp\
t exsec extdiff \
extract_linear_e\
quations extrema\
l_subset ezgcd %\
f f90 facsum fac\
tcomb factor fac\
torfacsum factor\
ial factorout fa\
ctorsum facts fa\
st_central_eleme\
nts fast_linsolv\
e fasttimes feat\
urep fernfale ff\
t fib fibtophi f\
ifth filename_me\
rge file_search \
file_type fillar\
ray findde find_\
root find_root_a\
bs find_root_err\
or find_root_rel\
 first fix flatt\
en flength float\
 floatnump floor\
 flower_snark fl\
ush flush1deriv \
flushd flushnd f\
lush_output fmin\
_cobyla forget f\
ortran fourcos f\
ourexpand fourie\
r fourier_elim f\
ourint fourintco\
s fourintsin fou\
rsimp foursin fo\
urth fposition f\
rame_bracket fre\
eof freshline fr\
esnel_c fresnel_\
s from_adjacency\
_matrix frucht_g\
raph full_listif\
y fullmap fullma\
pl fullratsimp f\
ullratsubst full\
setify funcsolve\
 fundamental_dim\
ensions fundamen\
tal_units fundef\
 funmake funp fv\
 g0 g1 gamma gam\
ma_greek gamma_i\
ncomplete gamma_\
incomplete_gener\
alized gamma_inc\
omplete_regulari\
zed gauss gauss_\
a gauss_b gaussp\
rob gcd gcdex gc\
divide gcfac gcf\
actor gd general\
ized_lambert_w g\
enfact gen_lague\
rre genmatrix ge\
nsym geo_amortiz\
ation geo_annuit\
y_fv geo_annuity\
_pv geomap geome\
tric geometric_m\
ean geosum get g\
etcurrentdirecto\
ry get_edge_weig\
ht getenv get_lu\
_factors get_out\
put_stream_strin\
g get_pixel get_\
plot_option get_\
tex_environment \
get_tex_environm\
ent_default get_\
vertex_label gfa\
ctor gfactorsum \
ggf girth global\
_variances gn gn\
uplot_close gnup\
lot_replot gnupl\
ot_reset gnuplot\
_restart gnuplot\
_start go Gosper\
 GosperSum gr2d \
gr3d gradef gram\
schmidt graph6_d\
ecode graph6_enc\
ode graph6_expor\
t graph6_import \
graph_center gra\
ph_charpoly grap\
h_eigenvalues gr\
aph_flow graph_o\
rder graph_perip\
hery graph_produ\
ct graph_size gr\
aph_union great_\
rhombicosidodeca\
hedron_graph gre\
at_rhombicubocta\
hedron_graph gri\
d_graph grind gr\
obner_basis grot\
zch_graph hamilt\
on_cycle hamilto\
n_path hankel ha\
nkel_1 hankel_2 \
harmonic harmoni\
c_mean hav heawo\
od_graph hermite\
 hessian hgfred \
hilbertmap hilbe\
rt_matrix hipow \
histogram histog\
ram_description \
hodge horner hyp\
ergeometric i0 i\
1 %ibes ic1 ic2 \
ic_convert ichr1\
 ichr2 icosahedr\
on_graph icosido\
decahedron_graph\
 icurvature iden\
t identfor ident\
ity idiff idim i\
dummy ieqn %if i\
factors iframes \
ifs igcdex igeod\
esic_coords ilt \
image imagpart i\
metric implicit \
implicit_derivat\
ive implicit_plo\
t indexed_tensor\
 indices induced\
_subgraph infere\
ncep inference_r\
esult infix info\
_display init_at\
ensor init_ctens\
or in_neighbors \
innerproduct inp\
art inprod inrt \
integerp integer\
_partitions inte\
grate intersect \
intersection int\
ervalp intopois \
intosum invarian\
t1 invariant2 in\
verse_fft invers\
e_jacobi_cd inve\
rse_jacobi_cn in\
verse_jacobi_cs \
inverse_jacobi_d\
c inverse_jacobi\
_dn inverse_jaco\
bi_ds inverse_ja\
cobi_nc inverse_\
jacobi_nd invers\
e_jacobi_ns inve\
rse_jacobi_sc in\
verse_jacobi_sd \
inverse_jacobi_s\
n invert invert_\
by_adjoint inver\
t_by_lu inv_mod \
irr is is_biconn\
ected is_biparti\
te is_connected \
is_digraph is_ed\
ge_in_graph is_g\
raph is_graph_or\
_digraph ishow i\
s_isomorphic iso\
late isomorphism\
 is_planar isqrt\
 isreal_p is_sco\
nnected is_tree \
is_vertex_in_gra\
ph items_inferen\
ce %j j0 j1 jaco\
bi jacobian jaco\
bi_cd jacobi_cn \
jacobi_cs jacobi\
_dc jacobi_dn ja\
cobi_ds jacobi_n\
c jacobi_nd jaco\
bi_ns jacobi_p j\
acobi_sc jacobi_\
sd jacobi_sn JF \
jn join jordan j\
ulia julia_set j\
ulia_sin %k kdel\
s kdelta kill ki\
llcontext kostka\
 kron_delta kron\
ecker_product ku\
mmer_m kummer_u \
kurtosis kurtosi\
s_bernoulli kurt\
osis_beta kurtos\
is_binomial kurt\
osis_chi2 kurtos\
is_continuous_un\
iform kurtosis_d\
iscrete_uniform \
kurtosis_exp kur\
tosis_f kurtosis\
_gamma kurtosis_\
general_finite_d\
iscrete kurtosis\
_geometric kurto\
sis_gumbel kurto\
sis_hypergeometr\
ic kurtosis_lapl\
ace kurtosis_log\
istic kurtosis_l\
ognormal kurtosi\
s_negative_binom\
ial kurtosis_non\
central_chi2 kur\
tosis_noncentral\
_student_t kurto\
sis_normal kurto\
sis_pareto kurto\
sis_poisson kurt\
osis_rayleigh ku\
rtosis_student_t\
 kurtosis_weibul\
l label labels l\
agrange laguerre\
 lambda lambert_\
w laplace laplac\
ian_matrix last \
lbfgs lc2kdt lch\
arp lc_l lcm lc_\
u ldefint ldisp \
ldisplay legendr\
e_p legendre_q l\
einstein length \
let letrules let\
simp levi_civita\
 lfreeof lgtreil\
lis lhs li liedi\
ff limit Lindste\
dt linear linear\
interpol linear_\
program linear_r\
egression line_g\
raph linsolve li\
starray list_cor\
relations listif\
y list_matrix_en\
tries list_nc_mo\
nomials listofte\
ns listofvars li\
stp lmax lmin lo\
ad loadfile loca\
l locate_matrix_\
entry log logcon\
tract log_gamma \
lopow lorentz_ga\
uge lowercasep l\
part lratsubst l\
reduce lriemann \
lsquares_estimat\
es lsquares_esti\
mates_approximat\
e lsquares_estim\
ates_exact lsqua\
res_mse lsquares\
_residual_mse ls\
quares_residuals\
 lsum ltreillis \
lu_backsub lucas\
 lu_factor %m ma\
croexpand macroe\
xpand1 make_arra\
y makebox makefa\
ct makegamma mak\
e_graph make_lev\
el_picture makel\
ist makeOrders m\
ake_poly_contine\
nt make_poly_cou\
ntry make_polygo\
n make_random_st\
ate make_rgb_pic\
ture makeset mak\
e_string_input_s\
tream make_strin\
g_output_stream \
make_transform m\
andelbrot mandel\
brot_set map map\
atom maplist mat\
chdeclare matchf\
ix mat_cond mat_\
fullunblocker ma\
t_function mathm\
l_display mat_no\
rm matrix matrix\
map matrixp matr\
ix_size mattrace\
 mat_trace mat_u\
nblocker max max\
_clique max_degr\
ee max_flow maxi\
mize_lp max_inde\
pendent_set max_\
matching maybe m\
d5sum mean mean_\
bernoulli mean_b\
eta mean_binomia\
l mean_chi2 mean\
_continuous_unif\
orm mean_deviati\
on mean_discrete\
_uniform mean_ex\
p mean_f mean_ga\
mma mean_general\
_finite_discrete\
 mean_geometric \
mean_gumbel mean\
_hypergeometric \
mean_laplace mea\
n_logistic mean_\
lognormal mean_n\
egative_binomial\
 mean_noncentral\
_chi2 mean_nonce\
ntral_student_t \
mean_normal mean\
_pareto mean_poi\
sson mean_raylei\
gh mean_student_\
t mean_weibull m\
edian median_dev\
iation member me\
sh metricexpanda\
ll mgf1_sha1 min\
 min_degree min_\
edge_cut minfact\
orial minimalPol\
y minimize_lp mi\
nimum_spanning_t\
ree minor minpac\
k_lsquares minpa\
ck_solve min_ver\
tex_cover min_ve\
rtex_cut mkdir m\
newton mod mode_\
declare mode_ide\
ntity ModeMatrix\
 moebius mon2sch\
ur mono monomial\
_dimensions mult\
ibernstein_poly \
multi_display_fo\
r_texinfo multi_\
elem multinomial\
 multinomial_coe\
ff multi_orbit m\
ultiplot_mode mu\
lti_pui multsym \
multthru myciels\
ki_graph nary na\
tural_unit nc_de\
gree ncexpt ncha\
rpoly negative_p\
icture neighbors\
 new newcontext \
newdet new_graph\
 newline newton \
new_variable nex\
t_prime nicedumm\
ies niceindices \
ninth nofix nona\
rray noncentral_\
moment nonmetric\
ity nonnegintege\
rp nonscalarp no\
nzeroandfreeof n\
otequal nounify \
nptetrad npv nro\
ots nterms nterm\
st nthroot nulli\
ty nullspace num\
 numbered_bounda\
ries numberp num\
ber_to_octets nu\
m_distinct_parti\
tions numerval n\
umfactor num_par\
titions nusum nz\
eta nzetai nzeta\
r octets_to_numb\
er octets_to_oid\
 odd_girth oddp \
ode2 ode_check o\
delin oid_to_oct\
ets op opena ope\
na_binary openr \
openr_binary ope\
nw openw_binary \
operatorp opsubs\
t optimize %or o\
rbit orbits orde\
rgreat ordergrea\
tp orderless ord\
erlessp orthogon\
al_complement or\
thopoly_recur or\
thopoly_weight o\
utermap out_neig\
hbors outofpois \
pade parabolic_c\
ylinder_d parame\
tric parametric_\
surface parg par\
Gosper parse_str\
ing parse_timeda\
te part part2con\
t partfrac parti\
tion partition_s\
et partpol path_\
digraph path_gra\
ph pathname_dire\
ctory pathname_n\
ame pathname_typ\
e pdf_bernoulli \
pdf_beta pdf_bin\
omial pdf_cauchy\
 pdf_chi2 pdf_co\
ntinuous_uniform\
 pdf_discrete_un\
iform pdf_exp pd\
f_f pdf_gamma pd\
f_general_finite\
_discrete pdf_ge\
ometric pdf_gumb\
el pdf_hypergeom\
etric pdf_laplac\
e pdf_logistic p\
df_lognormal pdf\
_negative_binomi\
al pdf_noncentra\
l_chi2 pdf_nonce\
ntral_student_t \
pdf_normal pdf_p\
areto pdf_poisso\
n pdf_rank_sum p\
df_rayleigh pdf_\
signed_rank pdf_\
student_t pdf_we\
ibull pearson_sk\
ewness permanent\
 permut permutat\
ion permutations\
 petersen_graph \
petrov pickapart\
 picture_equalp \
picturep piechar\
t piechart_descr\
iption planar_em\
bedding playback\
 plog plot2d plo\
t3d plotdf plote\
q plsquares poch\
hammer points po\
isdiff poisexpt \
poisint poismap \
poisplus poissim\
p poissubst pois\
times poistrim p\
olar polarform p\
olartorect polar\
_to_xy poly_add \
poly_buchberger \
poly_buchberger_\
criterion poly_c\
olon_ideal poly_\
content polydeco\
mp poly_depends_\
p poly_eliminati\
on_ideal poly_ex\
act_divide poly_\
expand poly_expt\
 poly_gcd polygo\
n poly_grobner p\
oly_grobner_equa\
l poly_grobner_m\
ember poly_grobn\
er_subsetp poly_\
ideal_intersecti\
on poly_ideal_po\
lysaturation pol\
y_ideal_polysatu\
ration1 poly_ide\
al_saturation po\
ly_ideal_saturat\
ion1 poly_lcm po\
ly_minimization \
polymod poly_mul\
tiply polynome2e\
le polynomialp p\
oly_normal_form \
poly_normalize p\
oly_normalize_li\
st poly_polysatu\
ration_extension\
 poly_primitive_\
part poly_pseudo\
_divide poly_red\
uced_grobner pol\
y_reduction poly\
_saturation_exte\
nsion poly_s_pol\
ynomial poly_sub\
tract polytocomp\
anion pop postfi\
x potential powe\
r_mod powerserie\
s powerset prefi\
x prev_prime pri\
mep primes princ\
ipal_components \
print printf pri\
ntfile print_gra\
ph printpois pri\
ntprops prodrac \
product properti\
es propvars psi \
psubst ptriangul\
arize pui pui2co\
mp pui2ele pui2p\
olynome pui_dire\
ct puireduc push\
 put pv qput qra\
nge qty quad_con\
trol quad_qag qu\
ad_qagi quad_qag\
p quad_qags quad\
_qawc quad_qawf \
quad_qawo quad_q\
aws quadrilatera\
l quantile quant\
ile_bernoulli qu\
antile_beta quan\
tile_binomial qu\
antile_cauchy qu\
antile_chi2 quan\
tile_continuous_\
uniform quantile\
_discrete_unifor\
m quantile_exp q\
uantile_f quanti\
le_gamma quantil\
e_general_finite\
_discrete quanti\
le_geometric qua\
ntile_gumbel qua\
ntile_hypergeome\
tric quantile_la\
place quantile_l\
ogistic quantile\
_lognormal quant\
ile_negative_bin\
omial quantile_n\
oncentral_chi2 q\
uantile_noncentr\
al_student_t qua\
ntile_normal qua\
ntile_pareto qua\
ntile_poisson qu\
antile_rayleigh \
quantile_student\
_t quantile_weib\
ull quartile_ske\
wness quit qunit\
 quotient racah_\
v racah_w radcan\
 radius random r\
andom_bernoulli \
random_beta rand\
om_binomial rand\
om_bipartite_gra\
ph random_cauchy\
 random_chi2 ran\
dom_continuous_u\
niform random_di\
graph random_dis\
crete_uniform ra\
ndom_exp random_\
f random_gamma r\
andom_general_fi\
nite_discrete ra\
ndom_geometric r\
andom_graph rand\
om_graph1 random\
_gumbel random_h\
ypergeometric ra\
ndom_laplace ran\
dom_logistic ran\
dom_lognormal ra\
ndom_negative_bi\
nomial random_ne\
twork random_non\
central_chi2 ran\
dom_noncentral_s\
tudent_t random_\
normal random_pa\
reto random_perm\
utation random_p\
oisson random_ra\
yleigh random_re\
gular_graph rand\
om_student_t ran\
dom_tournament r\
andom_tree rando\
m_weibull range \
rank rat ratcoef\
 ratdenom ratdif\
f ratdisrep rate\
xpand ratinterpo\
l rational ratio\
nalize ratnumer \
ratnump ratp rat\
simp ratsubst ra\
tvars ratweight \
read read_array \
read_binary_arra\
y read_binary_li\
st read_binary_m\
atrix readbyte r\
eadchar read_has\
hed_array readli\
ne read_list rea\
d_matrix read_ne\
sted_list readon\
ly read_xpm real\
_imagpart_to_con\
jugate realpart \
realroots rearra\
y rectangle rect\
form rectform_lo\
g_if_constant re\
cttopolar rediff\
 reduce_consts r\
educe_order regi\
on region_bounda\
ries region_boun\
daries_plus rem \
remainder remarr\
ay rembox remcom\
ps remcon remcoo\
rd remfun remfun\
ction remlet rem\
ove remove_const\
value remove_dim\
ensions remove_e\
dge remove_funda\
mental_dimension\
s remove_fundame\
ntal_units remov\
e_plot_option re\
move_vertex remp\
art remrule rems\
ym remvalue rena\
me rename_file r\
eset reset_displ\
ays residue reso\
lvante resolvant\
e_alternee1 reso\
lvante_bipartite\
 resolvante_died\
rale resolvante_\
klein resolvante\
_klein3 resolvan\
te_produit_sym r\
esolvante_unitai\
re resolvante_vi\
erer rest result\
ant return revea\
l reverse revert\
 revert2 rgb2lev\
el rhs ricci rie\
mann rinvariant \
risch rk rmdir r\
ncombine romberg\
 room rootscontr\
act round row ro\
wop rowswap rred\
uce run_testsuit\
e %s save saving\
 scalarp scaled_\
bessel_i scaled_\
bessel_i0 scaled\
_bessel_i1 scale\
factors scanmap \
scatterplot scat\
terplot_descript\
ion scene schur2\
comp sconcat sco\
py scsimp scurva\
ture sdowncase s\
ec sech second s\
equal sequaligno\
re set_alt_displ\
ay setdifference\
 set_draw_defaul\
ts set_edge_weig\
ht setelmx seteq\
ualp setify setp\
 set_partitions \
set_plot_option \
set_prompt set_r\
andom_state set_\
tex_environment \
set_tex_environm\
ent_default setu\
nits setup_autol\
oad set_up_dot_s\
implifications s\
et_vertex_label \
seventh sexplode\
 sf sha1sum sha2\
56sum shortest_p\
ath shortest_wei\
ghted_path show \
showcomps showra\
tvars sierpinski\
ale sierpinskima\
p sign signum si\
milaritytransfor\
m simp_inequalit\
y simplify_sum s\
implode simpmetd\
eriv simtran sin\
 sinh sinsert si\
nvertcase sixth \
skewness skewnes\
s_bernoulli skew\
ness_beta skewne\
ss_binomial skew\
ness_chi2 skewne\
ss_continuous_un\
iform skewness_d\
iscrete_uniform \
skewness_exp ske\
wness_f skewness\
_gamma skewness_\
general_finite_d\
iscrete skewness\
_geometric skewn\
ess_gumbel skewn\
ess_hypergeometr\
ic skewness_lapl\
ace skewness_log\
istic skewness_l\
ognormal skewnes\
s_negative_binom\
ial skewness_non\
central_chi2 ske\
wness_noncentral\
_student_t skewn\
ess_normal skewn\
ess_pareto skewn\
ess_poisson skew\
ness_rayleigh sk\
ewness_student_t\
 skewness_weibul\
l slength smake \
small_rhombicosi\
dodecahedron_gra\
ph small_rhombic\
uboctahedron_gra\
ph smax smin smi\
smatch snowmap s\
nub_cube_graph s\
nub_dodecahedron\
_graph solve sol\
ve_rec solve_rec\
_rat some somrac\
 sort sparse6_de\
code sparse6_enc\
ode sparse6_expo\
rt sparse6_impor\
t specint spheri\
cal spherical_be\
ssel_j spherical\
_bessel_y spheri\
cal_hankel1 sphe\
rical_hankel2 sp\
herical_harmonic\
 spherical_to_xy\
z splice split s\
position sprint \
sqfr sqrt sqrtde\
nest sremove sre\
movefirst srever\
se ssearch ssort\
 sstatus ssubst \
ssubstfirst stai\
rcase standardiz\
e standardize_in\
verse_trig starp\
lot starplot_des\
cription status \
std std1 std_ber\
noulli std_beta \
std_binomial std\
_chi2 std_contin\
uous_uniform std\
_discrete_unifor\
m std_exp std_f \
std_gamma std_ge\
neral_finite_dis\
crete std_geomet\
ric std_gumbel s\
td_hypergeometri\
c std_laplace st\
d_logistic std_l\
ognormal std_neg\
ative_binomial s\
td_noncentral_ch\
i2 std_noncentra\
l_student_t std_\
normal std_paret\
o std_poisson st\
d_rayleigh std_s\
tudent_t std_wei\
bull stemplot st\
irling stirling1\
 stirling2 strim\
 striml strimr s\
tring stringout \
stringp strong_c\
omponents struve\
_h struve_l subl\
is sublist subli\
st_indices subma\
trix subsample s\
ubset subsetp su\
bst substinpart \
subst_parallel s\
ubstpart substri\
ng subvar subvar\
p sum sumcontrac\
t summand_to_rec\
 supcase supcont\
ext symbolp symm\
difference symme\
tricp system tak\
e_channel take_i\
nference tan tan\
h taylor taylori\
nfo taylorp tayl\
or_simplifier ta\
ytorat tcl_outpu\
t tcontract tell\
rat tellsimp tel\
lsimpafter tente\
x tenth test_mea\
n test_means_dif\
ference test_nor\
mality test_prop\
ortion test_prop\
ortions_differen\
ce test_rank_sum\
 test_sign test_\
signed_rank test\
_variance test_v\
ariance_ratio te\
x tex1 tex_displ\
ay texput %th th\
ird throw time t\
imedate timer ti\
mer_info tldefin\
t tlimit todd_co\
xeter toeplitz t\
okens to_lisp to\
pological_sort t\
o_poly to_poly_s\
olve totaldisrep\
 totalfourier to\
tient tpartpol t\
race tracematrix\
 trace_options t\
ransform_sample \
translate transl\
ate_file transpo\
se treefale tree\
_reduce treillis\
 treinat triangl\
e triangularize \
trigexpand trigr\
at trigreduce tr\
igsimp trunc tru\
ncate truncated_\
cube_graph trunc\
ated_dodecahedro\
n_graph truncate\
d_icosahedron_gr\
aph truncated_te\
trahedron_graph \
tr_warnings_get \
tube tutte_graph\
 ueivects uforge\
t ultraspherical\
 underlying_grap\
h undiff union u\
nique uniteigenv\
ectors unitp uni\
ts unit_step uni\
tvector unorder \
unsum untellrat \
untimer untrace \
uppercasep uricc\
i uriemann uvect\
 vandermonde_mat\
rix var var1 var\
_bernoulli var_b\
eta var_binomial\
 var_chi2 var_co\
ntinuous_uniform\
 var_discrete_un\
iform var_exp va\
r_f var_gamma va\
r_general_finite\
_discrete var_ge\
ometric var_gumb\
el var_hypergeom\
etric var_laplac\
e var_logistic v\
ar_lognormal var\
_negative_binomi\
al var_noncentra\
l_chi2 var_nonce\
ntral_student_t \
var_normal var_p\
areto var_poisso\
n var_rayleigh v\
ar_student_t var\
_weibull vector \
vectorpotential \
vectorsimp verbi\
fy vers vertex_c\
oloring vertex_c\
onnectivity vert\
ex_degree vertex\
_distance vertex\
_eccentricity ve\
rtex_in_degree v\
ertex_out_degree\
 vertices vertic\
es_to_cycle vert\
ices_to_path %w \
weyl wheel_graph\
 wiener_index wi\
gner_3j wigner_6\
j wigner_9j with\
_stdout write_bi\
nary_data writeb\
yte write_data w\
ritefile wronski\
an xreduce xthru\
 %y Zeilberger z\
eroequiv zerofor\
 zeromatrix zero\
matrixp zeta zge\
ev zheev zlange \
zn_add_table zn_\
carmichael_lambd\
a zn_characteris\
tic_factors zn_d\
eterminant zn_fa\
ctor_generators \
zn_invert_by_lu \
zn_log zn_mult_t\
able absboxchar \
activecontexts a\
dapt_depth addit\
ive adim aform a\
lgebraic algepsi\
lon algexact ali\
ases allbut all_\
dotsimp_denoms a\
llocation allsym\
 alphabetic anim\
ation antisymmet\
ric arrays askex\
p assume_pos ass\
ume_pos_pred ass\
umescalar asymbo\
l atomgrad atrig\
1 axes axis_3d a\
xis_bottom axis_\
left axis_right \
axis_top azimuth\
 background back\
ground_color bac\
ksubst berlefact\
 bernstein_expli\
cit besselexpand\
 beta_args_sum_t\
o_integer beta_e\
xpand bftorat bf\
trunc bindtest b\
order boundaries\
_array box boxch\
ar breakup %c ca\
pping cauchysum \
cbrange cbtics c\
enter cflength c\
frame_flag cnonm\
et_flag color co\
lor_bar color_ba\
r_tics colorbox \
columns commutat\
ive complex cone\
 context context\
s contour contou\
r_levels cosnpif\
lag ctaypov ctay\
pt ctayswitch ct\
ayvar ct_coords \
ctorsion_flag ct\
rgsimp cube curr\
ent_let_rule_pac\
kage cylinder da\
ta_file_name deb\
ugmode decreasin\
g default_let_ru\
le_package delay\
 dependencies de\
rivabbrev derivs\
ubst detout diag\
metric diff dim \
dimensions dispf\
lag display2d|10\
 display_format_\
internal distrib\
ute_over doallmx\
ops domain domxe\
xpt domxmxops do\
mxnctimes dontfa\
ctor doscmxops d\
oscmxplus dot0ns\
csimp dot0simp d\
ot1simp dotassoc\
 dotconstrules d\
otdistrib dotexp\
tsimp dotident d\
otscrules draw_g\
raph_program dra\
w_realpart edge_\
color edge_color\
ing edge_partiti\
on edge_type edg\
e_width %edispfl\
ag elevation %em\
ode endphi endth\
eta engineering_\
format_floats en\
hanced3d %enumer\
 epsilon_lp erff\
lag erf_represen\
tation errormsg \
error_size error\
_syms error_type\
 %e_to_numlog ev\
al even evenfun \
evflag evfun ev_\
point expandwrt_\
denom expintexpa\
nd expintrep exp\
on expop exptdis\
pflag exptisolat\
e exptsubst face\
xpand facsum_com\
bine factlim fac\
torflag factoria\
l_expand factors\
_only fb feature\
 features file_n\
ame file_output_\
append file_sear\
ch_demo file_sea\
rch_lisp file_se\
arch_maxima|10 f\
ile_search_tests\
 file_search_usa\
ge file_type_lis\
p file_type_maxi\
ma|10 fill_color\
 fill_density fi\
lled_func fixed_\
vertices flipfla\
g float2bf font \
font_size fortin\
dent fortspaces \
fpprec fpprintpr\
ec functions gam\
ma_expand gammal\
im gdet genindex\
 gensumnum GGFCF\
MAX GGFINFINITY \
globalsolve gnup\
lot_command gnup\
lot_curve_styles\
 gnuplot_curve_t\
itles gnuplot_de\
fault_term_comma\
nd gnuplot_dumb_\
term_command gnu\
plot_file_args g\
nuplot_file_name\
 gnuplot_out_fil\
e gnuplot_pdf_te\
rm_command gnupl\
ot_pm3d gnuplot_\
png_term_command\
 gnuplot_postamb\
le gnuplot_pream\
ble gnuplot_ps_t\
erm_command gnup\
lot_svg_term_com\
mand gnuplot_ter\
m gnuplot_view_a\
rgs Gosper_in_Ze\
ilberger gradefs\
 grid grid2d gri\
nd halfangles he\
ad_angle head_bo\
th head_length h\
ead_type height \
hypergeometric_r\
epresentation %i\
args ibase icc1 \
icc2 icounter id\
ummyx ieqnprint \
ifb ifc1 ifc2 if\
g ifgi ifr ifram\
e_bracket_form i\
fri igeowedge_fl\
ag ikt1 ikt2 ima\
ginary inchar in\
creasing infeval\
 infinity inflag\
 infolists inm i\
nmc1 inmc2 intan\
alysis integer i\
ntegervalued int\
egrate_use_roots\
of integration_c\
onstant integrat\
ion_constant_cou\
nter interpolate\
_color intfaclim\
 ip_grid ip_grid\
_in irrational i\
solate_wrt_times\
 iterations itr \
julia_parameter \
%k1 %k2 keepfloa\
t key key_pos ki\
nvariant kt labe\
l label_alignmen\
t label_orientat\
ion labels lasso\
ciative lbfgs_nc\
orrections lbfgs\
_nfeval_max left\
just legend letr\
at let_rule_pack\
ages lfg lg lhos\
pitallim limsubs\
t linear linear_\
solver linechar \
linel|10 linenum\
 line_type linew\
idth line_width \
linsolve_params \
linsolvewarn lis\
pdisp listarith \
listconstvars li\
stdummyvars lmxc\
har load_pathnam\
e loadprint loga\
bs logarc logcb \
logconcoeffp log\
expand lognegint\
 logsimp logx lo\
gx_secondary log\
y logy_secondary\
 logz lriem m1pb\
ranch macroexpan\
sion macros main\
var manual_demo \
maperror mapprin\
t matrix_element\
_add matrix_elem\
ent_mult matrix_\
element_transpos\
e maxapplydepth \
maxapplyheight m\
axima_tempdir|10\
 maxima_userdir|\
10 maxnegex MAX_\
ORD maxposex max\
psifracdenom max\
psifracnum maxps\
inegint maxpsipo\
sint maxtayorder\
 mesh_lines_colo\
r method mod_big\
_prime mode_chec\
k_errorp mode_ch\
eckp mode_check_\
warnp mod_test m\
od_threshold mod\
ular_linear_solv\
er modulus multi\
plicative multip\
licities myoptio\
ns nary negdistr\
ib negsumdispfla\
g newline newton\
epsilon newtonma\
xiter nextlayerf\
actor niceindice\
spref nm nmc noe\
val nolabels non\
egative_lp nonin\
teger nonscalar \
noun noundisp no\
uns np npi ntick\
s ntrig numer nu\
mer_pbranch obas\
e odd oddfun opa\
city oppropertie\
s opsubst optimp\
refix optionset \
orientation orig\
in orthopoly_ret\
urns_intervals o\
utative outchar \
packagefile pale\
tte partswitch p\
df_file pfeforma\
t phiresolution \
%piargs piece pi\
vot_count_sx piv\
ot_max_sx plot_f\
ormat plot_optio\
ns plot_realpart\
 png_file pochha\
mmer_max_index p\
oints pointsize \
point_size point\
s_joined point_t\
ype poislim pois\
son poly_coeffic\
ient_ring poly_e\
limination_order\
 polyfactor poly\
_grobner_algorit\
hm poly_grobner_\
debug poly_monom\
ial_order poly_p\
rimary_eliminati\
on_order poly_re\
turn_term_list p\
oly_secondary_el\
imination_order \
poly_top_reducti\
on_only posfun p\
osition powerdis\
p pred prederror\
 primep_number_o\
f_tests product_\
use_gamma progra\
m programmode pr\
omote_float_to_b\
igfloat prompt p\
roportional_axes\
 props psexpand \
ps_file radexpan\
d radius radsubs\
tflag rassociati\
ve ratalgdenom r\
atchristof ratde\
nomdivide ratein\
stein ratepsilon\
 ratfac rational\
 ratmx ratprint \
ratriemann ratsi\
mpexpons ratvars\
witch ratweights\
 ratweyl ratwtlv\
l real realonly \
redraw refcheck \
resolution resta\
rt resultant ric\
 riem rmxchar %r\
num_list romberg\
abs rombergit ro\
mbergmin romberg\
tol rootsconmode\
 rootsepsilon ru\
n_viewer same_xy\
 same_xyz savede\
f savefactors sc\
alar scalarmatri\
xp scale scale_l\
p setcheck setch\
eckbreak setval \
show_edge_color \
show_edges show_\
edge_type show_e\
dge_width show_i\
d show_label sho\
wtime show_verte\
x_color show_ver\
tex_size show_ve\
rtex_type show_v\
ertices show_wei\
ght simp simplif\
ied_output simpl\
ify_products sim\
pproduct simpsum\
 sinnpiflag solv\
edecomposes solv\
eexplicit solvef\
actors solvenull\
warn solveradcan\
 solvetrigwarn s\
pace sparse sphe\
re spring_embedd\
ing_depth sqrtdi\
spflag stardisp \
startphi startth\
eta stats_numer \
stringdisp struc\
tures style subl\
is_apply_lambda \
subnumsimp sumex\
pand sumsplitfac\
t surface surfac\
e_hide svg_file \
symmetric tab ta\
ylordepth taylor\
_logexpand taylo\
r_order_coeffici\
ents taylor_trun\
cate_polynomials\
 tensorkill term\
inal testsuite_f\
iles thetaresolu\
tion timer_deval\
ue title tlimswi\
tch tr track tra\
nscompile transf\
orm transform_xy\
 translate_fast_\
arrays transpare\
nt transrun tr_a\
rray_as_ref tr_b\
ound_function_ap\
plyp tr_file_tty\
_messagesp tr_fl\
oat_can_branch_c\
omplex tr_functi\
on_call_default \
trigexpandplus t\
rigexpandtimes t\
riginverses trig\
sign trivial_sol\
utions tr_numer \
tr_optimize_max_\
loop tr_semicomp\
ile tr_state_var\
s tr_warn_bad_fu\
nction_calls tr_\
warn_fexpr tr_wa\
rn_meval tr_warn\
_mode tr_warn_un\
declared tr_warn\
_undefined_varia\
ble tstep ttyoff\
 tube_extremes u\
fg ug %unitexpan\
d unit_vectors u\
ric uriem use_fa\
st_arrays user_p\
reamble usersetu\
nits values vect\
_cross verbose v\
ertex_color vert\
ex_coloring vert\
ex_partition ver\
tex_size vertex_\
type view warnin\
gs weyl width wi\
ndowname windowt\
itle wired_surfa\
ce wireframe xax\
is xaxis_color x\
axis_secondary x\
axis_type xaxis_\
width xlabel xla\
bel_secondary xl\
ength xrange xra\
nge_secondary xt\
ics xtics_axis x\
tics_rotate xtic\
s_rotate_seconda\
ry xtics_seconda\
ry xtics_seconda\
ry_axis xu_grid \
x_voxel xy_file \
xyplane xy_scale\
 yaxis yaxis_col\
or yaxis_seconda\
ry yaxis_type ya\
xis_width ylabel\
 ylabel_secondar\
y ylength yrange\
 yrange_secondar\
y ytics ytics_ax\
is ytics_rotate \
ytics_rotate_sec\
ondary ytics_sec\
ondary ytics_sec\
ondary_axis yv_g\
rid y_voxel yx_r\
atio zaxis zaxis\
_color zaxis_typ\
e zaxis_width ze\
roa zerob zerobe\
rn zeta%pi zlabe\
l zlabel_rotate \
zlength zmin zn_\
primroot_limit z\
n_primroot_prete\
st\x22,\x0asymbol:\x22_ _\
_ %|0 %%|0\x22},con\
tains:[{classNam\
e:\x22comment\x22,begi\
n:\x22/\x5c\x5c*\x22,end:\x22\x5c\x5c\
*/\x22,\x0acontains:[\x22\
self\x22]},e.QUOTE_\
STRING_MODE,{cla\
ssName:\x22number\x22,\
relevance:0,\x0avar\
iants:[{begin:\x22\x5c\
\x5cb(\x5c\x5cd+|\x5c\x5cd+\x5c\x5c.|\
\x5c\x5c.\x5c\x5cd+|\x5c\x5cd+\x5c\x5c.\x5c\
\x5cd+)[Ee][-+]?\x5c\x5cd\
+\x5c\x5cb\x22},{\x0abegin:\x22\
\x5c\x5cb(\x5c\x5cd+|\x5c\x5cd+\x5c\x5c.\
|\x5c\x5c.\x5c\x5cd+|\x5c\x5cd+\x5c\x5c.\
\x5c\x5cd+)[Bb][-+]?\x5c\x5c\
d+\x5c\x5cb\x22,relevance\
:10},{\x0abegin:\x22\x5c\x5c\
b(\x5c\x5c.\x5c\x5cd+|\x5c\x5cd+\x5c\x5c\
.\x5c\x5cd+)\x5c\x5cb\x22},{beg\
in:\x22\x5c\x5cb(\x5c\x5cd+|0[0\
-9A-Za-z]+)\x5c\x5c.?\x5c\
\x5cb\x22}]\x0a}],illegal\
:/@/})})();hljs.\
registerLanguage\
(\x22maxima\x22,e)})()\
;/*! `mel` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>({name:\x22MEL\x22,\x0ak\
eywords:\x22int flo\
at string vector\
 matrix if else \
switch case defa\
ult while do for\
 in break contin\
ue global proc r\
eturn about abs \
addAttr addAttri\
buteEditorNodeHe\
lp addDynamic ad\
dNewShelfTab add\
PP addPanelCateg\
ory addPrefixToN\
ame advanceToNex\
tDrivenKey affec\
tedNet affects a\
imConstraint air\
 alias aliasAttr\
 align alignCtx \
alignCurve align\
Surface allViewF\
it ambientLight \
angle angleBetwe\
en animCone anim\
CurveEditor anim\
Display animView\
 annotate append\
StringArray appl\
icationName appl\
yAttrPreset appl\
yTake arcLenDimC\
ontext arcLength\
Dimension arclen\
 arrayMapper art\
3dPaintCtx artAt\
trCtx artAttrPai\
ntVertexCtx artA\
ttrSkinPaintCtx \
artAttrTool artB\
uildPaintMenu ar\
tFluidAttrCtx ar\
tPuttyCtx artSel\
ectCtx artSetPai\
ntCtx artUserPai\
ntCtx assignComm\
and assignInputD\
evice assignView\
portFactories at\
tachCurve attach\
DeviceAttr attac\
hSurface attrCol\
orSliderGrp attr\
Compatibility at\
trControlGrp att\
rEnumOptionMenu \
attrEnumOptionMe\
nuGrp attrFieldG\
rp attrFieldSlid\
erGrp attrNaviga\
tionControlGrp a\
ttrPresetEditWin\
 attributeExists\
 attributeInfo a\
ttributeMenu att\
ributeQuery auto\
Keyframe autoPla\
ce bakeClip bake\
FluidShading bak\
ePartialHistory \
bakeResults bake\
Simulation basen\
ame basenameEx b\
atchRender besse\
l bevel bevelPlu\
s binMembership \
bindSkin blend2 \
blendShape blend\
ShapeEditor blen\
dShapePanel blen\
dTwoAttr blindDa\
taType boneLatti\
ce boundary boxD\
ollyCtx boxZoomC\
tx bufferCurve b\
uildBookmarkMenu\
 buildKeyframeMe\
nu button button\
Manip CBG cacheF\
ile cacheFileCom\
bine cacheFileMe\
rge cacheFileTra\
ck camera camera\
View canCreateMa\
nip canvas capit\
alizeString catc\
h catchQuiet cei\
l changeSubdivCo\
mponentDisplayLe\
vel changeSubdiv\
Region channelBo\
x character char\
acterMap charact\
erOutlineEditor \
characterize chd\
ir checkBox chec\
kBoxGrp checkDef\
aultRenderGlobal\
s choice circle \
circularFillet c\
lamp clear clear\
Cache clip clipE\
ditor clipEditor\
CurrentTimeCtx c\
lipSchedule clip\
SchedulerOutline\
r clipTrimBefore\
 closeCurve clos\
eSurface cluster\
 cmdFileOutput c\
mdScrollFieldExe\
cuter cmdScrollF\
ieldReporter cmd\
Shell coarsenSub\
divSelectionList\
 collision color\
 colorAtPoint co\
lorEditor colorI\
ndex colorIndexS\
liderGrp colorSl\
iderButtonGrp co\
lorSliderGrp col\
umnLayout comman\
dEcho commandLin\
e commandPort co\
mpactHairSystem \
componentEditor \
compositingInter\
op computePolyse\
tVolume conditio\
n cone confirmDi\
alog connectAttr\
 connectControl \
connectDynamic c\
onnectJoint conn\
ectionInfo const\
rain constrainVa\
lue construction\
History containe\
r containsMultib\
yte contextInfo \
control convertF\
romOldLayers con\
vertIffToPsd con\
vertLightmap con\
vertSolidTx conv\
ertTessellation \
convertUnit copy\
Array copyFlexor\
 copyKey copySki\
nWeights cos cpB\
utton cpCache cp\
ClothSet cpColli\
sion cpConstrain\
t cpConvClothToM\
esh cpForces cpG\
etSolverAttr cpP\
anel cpProperty \
cpRigidCollision\
Filter cpSeam cp\
SetEdit cpSetSol\
verAttr cpSolver\
 cpSolverTypes c\
pTool cpUpdateCl\
othUVs createDis\
playLayer create\
DrawCtx createEd\
itor createLayer\
edPsdFile create\
MotionField crea\
teNewShelf creat\
eNode createRend\
erLayer createSu\
bdivRegion cross\
 crossProduct ct\
xAbort ctxComple\
tion ctxEditMode\
 ctxTraverse cur\
rentCtx currentT\
ime currentTimeC\
tx currentUnit c\
urve curveAddPtC\
tx curveCVCtx cu\
rveEPCtx curveEd\
itorCtx curveInt\
ersect curveMove\
EPCtx curveOnSur\
face curveSketch\
Ctx cutKey cycle\
Check cylinder d\
agPose date defa\
ultLightListChec\
kBox defaultNavi\
gation defineDat\
aServer defineVi\
rtualDevice defo\
rmer deg_to_rad \
delete deleteAtt\
r deleteShadingG\
roupsAndMaterial\
s deleteShelfTab\
 deleteUI delete\
UnusedBrushes de\
lrandstr detachC\
urve detachDevic\
eAttr detachSurf\
ace deviceEditor\
 devicePanel dgI\
nfo dgdirty dgev\
al dgtimer dimWh\
en directKeyCtx \
directionalLight\
 dirmap dirname \
disable disconne\
ctAttr disconnec\
tJoint diskCache\
 displacementToP\
oly displayAffec\
ted displayColor\
 displayCull dis\
playLevelOfDetai\
l displayPref di\
splayRGBColor di\
splaySmoothness \
displayStats dis\
playString displ\
aySurface distan\
ceDimContext dis\
tanceDimension d\
oBlur dolly doll\
yCtx dopeSheetEd\
itor dot dotProd\
uct doubleProfil\
eBirailSurface d\
rag dragAttrCont\
ext draggerConte\
xt dropoffLocato\
r duplicate dupl\
icateCurve dupli\
cateSurface dynC\
ache dynControl \
dynExport dynExp\
ression dynGloba\
ls dynPaintEdito\
r dynParticleCtx\
 dynPref dynRelE\
dPanel dynRelEdi\
tor dynamicLoad \
editAttrLimits e\
ditDisplayLayerG\
lobals editDispl\
ayLayerMembers e\
ditRenderLayerAd\
justment editRen\
derLayerGlobals \
editRenderLayerM\
embers editor ed\
itorTemplate eff\
ector emit emitt\
er enableDevice \
encodeString end\
String endsWith \
env equivalent e\
quivalentTol erf\
 error eval eval\
Deferred evalEch\
o event exactWor\
ldBoundingBox ex\
clusiveLightChec\
kBox exec execut\
eForEachObject e\
xists exp expres\
sion expressionE\
ditorListen exte\
ndCurve extendSu\
rface extrude fc\
heck fclose feof\
 fflush fgetline\
 fgetword file f\
ileBrowserDialog\
 fileDialog file\
Extension fileIn\
fo filetest fill\
etCurve filter f\
ilterCurve filte\
rExpand filterSt\
udioImport findA\
llIntersections \
findAnimCurves f\
indKeyframe find\
MenuItem findRel\
atedSkinCluster \
finder firstPare\
ntOf fitBspline \
flexor floatEq f\
loatField floatF\
ieldGrp floatScr\
ollBar floatSlid\
er floatSlider2 \
floatSliderButto\
nGrp floatSlider\
Grp floor flow f\
luidCacheInfo fl\
uidEmitter fluid\
VoxelInfo flushU\
ndo fmod fontDia\
log fopen formLa\
yout format fpri\
nt frameLayout f\
read freeFormFil\
let frewind from\
NativePath fwrit\
e gamma gauss ge\
ometryConstraint\
 getApplicationV\
ersionAsFloat ge\
tAttr getClassif\
ication getDefau\
ltBrush getFileL\
ist getFluidAttr\
 getInputDeviceR\
ange getMayaPane\
lTypes getModifi\
ers getPanel get\
ParticleAttr get\
PluginResource g\
etenv getpid glR\
ender glRenderEd\
itor globalStitc\
h gmatch goal go\
toBindPose grabC\
olor gradientCon\
trol gradientCon\
trolNoAttr graph\
DollyCtx graphSe\
lectContext grap\
hTrackCtx gravit\
y grid gridLayou\
t group groupObj\
ectsByName HfAdd\
AttractorToAS Hf\
AssignAS HfBuild\
EqualMap HfBuild\
FurFiles HfBuild\
FurImages HfCanc\
elAFR HfConnectA\
SToHF HfCreateAt\
tractor HfDelete\
AS HfEditAS HfPe\
rformCreateAS Hf\
RemoveAttractorF\
romAS HfSelectAt\
tached HfSelectA\
ttractors HfUnAs\
signAS hardenPoi\
ntCurve hardware\
 hardwareRenderP\
anel headsUpDisp\
lay headsUpMessa\
ge help helpLine\
 hermite hide hi\
lite hitTest hot\
Box hotkey hotke\
yCheck hsv_to_rg\
b hudButton hudS\
lider hudSliderB\
utton hwReflecti\
onMap hwRender h\
wRenderLoad hype\
rGraph hyperPane\
l hyperShade hyp\
ot iconTextButto\
n iconTextCheckB\
ox iconTextRadio\
Button iconTextR\
adioCollection i\
conTextScrollLis\
t iconTextStatic\
Label ikHandle i\
kHandleCtx ikHan\
dleDisplayScale \
ikSolver ikSplin\
eHandleCtx ikSys\
tem ikSystemInfo\
 ikfkDisplayMeth\
od illustratorCu\
rves image imfPl\
ugins inheritTra\
nsform insertJoi\
nt insertJointCt\
x insertKeyCtx i\
nsertKnotCurve i\
nsertKnotSurface\
 instance instan\
ceable instancer\
 intField intFie\
ldGrp intScrollB\
ar intSlider int\
SliderGrp interT\
oUI internalVar \
intersect iprEng\
ine isAnimCurve \
isConnected isDi\
rty isParentOf i\
sSameObject isTr\
ue isValidObject\
Name isValidStri\
ng isValidUiName\
 isolateSelect i\
temFilter itemFi\
lterAttr itemFil\
terRender itemFi\
lterType joint j\
ointCluster join\
tCtx jointDispla\
yScale jointLatt\
ice keyTangent k\
eyframe keyframe\
Outliner keyfram\
eRegionCurrentTi\
meCtx keyframeRe\
gionDirectKeyCtx\
 keyframeRegionD\
ollyCtx keyframe\
RegionInsertKeyC\
tx keyframeRegio\
nMoveKeyCtx keyf\
rameRegionScaleK\
eyCtx keyframeRe\
gionSelectKeyCtx\
 keyframeRegionS\
etKeyCtx keyfram\
eRegionTrackCtx \
keyframeStats la\
ssoContext latti\
ce latticeDeform\
KeyCtx launch la\
unchImageEditor \
layerButton laye\
redShaderPort la\
yeredTexturePort\
 layout layoutDi\
alog lightList l\
ightListEditor l\
ightListPanel li\
ghtlink lineInte\
rsection linearP\
recision linstep\
 listAnimatable \
listAttr listCam\
eras listConnect\
ions listDeviceA\
ttachments listH\
istory listInput\
DeviceAxes listI\
nputDeviceButton\
s listInputDevic\
es listMenuAnnot\
ation listNodeTy\
pes listPanelCat\
egories listRela\
tives listSets l\
istTransforms li\
stUnselected lis\
terEditor loadFl\
uid loadNewShelf\
 loadPlugin load\
PluginLanguageRe\
sources loadPref\
Objects localize\
dPanelLabel lock\
Node loft log lo\
ngNameOf lookThr\
u ls lsThroughFi\
lter lsType lsUI\
 Mayatomr mag ma\
keIdentity makeL\
ive makePaintabl\
e makeRoll makeS\
ingleSurface mak\
eTubeOn makebot \
manipMoveContext\
 manipMoveLimits\
Ctx manipOptions\
 manipRotateCont\
ext manipRotateL\
imitsCtx manipSc\
aleContext manip\
ScaleLimitsCtx m\
arker match max \
memory menu menu\
BarLayout menuEd\
itor menuItem me\
nuItemToShelf me\
nuSet menuSetPre\
f messageLine mi\
n minimizeApp mi\
rrorJoint modelC\
urrentTimeCtx mo\
delEditor modelP\
anel mouse movIn\
 movOut move mov\
eIKtoFK moveKeyC\
tx moveVertexAlo\
ngDirection mult\
iProfileBirailSu\
rface mute nPart\
icle nameCommand\
 nameField names\
pace namespaceIn\
fo newPanelItems\
 newton nodeCast\
 nodeIconButton \
nodeOutliner nod\
ePreset nodeType\
 noise nonLinear\
 normalConstrain\
t normalize nurb\
sBoolean nurbsCo\
pyUVSet nurbsCub\
e nurbsEditUV nu\
rbsPlane nurbsSe\
lect nurbsSquare\
 nurbsToPoly nur\
bsToPolygonsPref\
 nurbsToSubdiv n\
urbsToSubdivPref\
 nurbsUVSet nurb\
sViewDirectionVe\
ctor objExists o\
bjectCenter obje\
ctLayer objectTy\
pe objectTypeUI \
obsoleteProc oce\
anNurbsPreviewPl\
ane offsetCurve \
offsetCurveOnSur\
face offsetSurfa\
ce openGLExtensi\
on openMayaPref \
optionMenu optio\
nMenuGrp optionV\
ar orbit orbitCt\
x orientConstrai\
nt outlinerEdito\
r outlinerPanel \
overrideModifier\
 paintEffectsDis\
play pairBlend p\
alettePort paneL\
ayout panel pane\
lConfiguration p\
anelHistory para\
mDimContext para\
mDimension param\
Locator parent p\
arentConstraint \
particle particl\
eExists particle\
Instancer partic\
leRenderInfo par\
tition pasteKey \
pathAnimation pa\
use pclose perce\
nt performanceOp\
tions pfxstrokes\
 pickWalk pictur\
e pixelMove plan\
arSrf plane play\
 playbackOptions\
 playblast plugA\
ttr plugNode plu\
ginInfo pluginRe\
sourceUtil point\
Constraint point\
CurveConstraint \
pointLight point\
MatrixMult point\
OnCurve pointOnS\
urface pointPosi\
tion poleVectorC\
onstraint polyAp\
pend polyAppendF\
acetCtx polyAppe\
ndVertex polyAut\
oProjection poly\
AverageNormal po\
lyAverageVertex \
polyBevel polyBl\
endColor polyBli\
ndData polyBoolO\
p polyBridgeEdge\
 polyCacheMonito\
r polyCheck poly\
ChipOff polyClip\
board polyCloseB\
order polyCollap\
seEdge polyColla\
pseFacet polyCol\
orBlindData poly\
ColorDel polyCol\
orPerVertex poly\
ColorSet polyCom\
pare polyCone po\
lyCopyUV polyCre\
ase polyCreaseCt\
x polyCreateFace\
t polyCreateFace\
tCtx polyCube po\
lyCut polyCutCtx\
 polyCylinder po\
lyCylindricalPro\
jection polyDelE\
dge polyDelFacet\
 polyDelVertex p\
olyDuplicateAndC\
onnect polyDupli\
cateEdge polyEdi\
tUV polyEditUVSh\
ell polyEvaluate\
 polyExtrudeEdge\
 polyExtrudeFace\
t polyExtrudeVer\
tex polyFlipEdge\
 polyFlipUV poly\
ForceUV polyGeoS\
ampler polyHelix\
 polyInfo polyIn\
stallAction poly\
LayoutUV polyLis\
tComponentConver\
sion polyMapCut \
polyMapDel polyM\
apSew polyMapSew\
Move polyMergeEd\
ge polyMergeEdge\
Ctx polyMergeFac\
et polyMergeFace\
tCtx polyMergeUV\
 polyMergeVertex\
 polyMirrorFace \
polyMoveEdge pol\
yMoveFacet polyM\
oveFacetUV polyM\
oveUV polyMoveVe\
rtex polyNormal \
polyNormalPerVer\
tex polyNormaliz\
eUV polyOptUvs p\
olyOptions polyO\
utput polyPipe p\
olyPlanarProject\
ion polyPlane po\
lyPlatonicSolid \
polyPoke polyPri\
mitive polyPrism\
 polyProjection \
polyPyramid poly\
Quad polyQueryBl\
indData polyRedu\
ce polySelect po\
lySelectConstrai\
nt polySelectCon\
straintMonitor p\
olySelectCtx pol\
ySelectEditCtx p\
olySeparate poly\
SetToFaceNormal \
polySewEdge poly\
ShortestPathCtx \
polySmooth polyS\
oftEdge polySphe\
re polySpherical\
Projection polyS\
plit polySplitCt\
x polySplitEdge \
polySplitRing po\
lySplitVertex po\
lyStraightenUVBo\
rder polySubdivi\
deEdge polySubdi\
videFacet polyTo\
Subdiv polyTorus\
 polyTransfer po\
lyTriangulate po\
lyUVSet polyUnit\
e polyWedgeFace \
popen popupMenu \
pose pow preload\
RefEd print prog\
ressBar progress\
Window projFileV\
iewer projectCur\
ve projectTangen\
t projectionCont\
ext projectionMa\
nip promptDialog\
 propModCtx prop\
Move psdChannelO\
utliner psdEditT\
extureFile psdEx\
port psdTextureF\
ile putenv pwd p\
ython querySubdi\
v quit rad_to_de\
g radial radioBu\
tton radioButton\
Grp radioCollect\
ion radioMenuIte\
mCollection ramp\
ColorPort rand r\
andomizeFollicle\
s randstate rang\
eControl readTak\
e rebuildCurve r\
ebuildSurface re\
cordAttr recordD\
evice redo refer\
ence referenceEd\
it referenceQuer\
y refineSubdivSe\
lectionList refr\
esh refreshAE re\
gisterPluginReso\
urce rehash relo\
adImage removeJo\
int removeMultiI\
nstance removePa\
nelCategory rena\
me renameAttr re\
nameSelectionLis\
t renameUI rende\
r renderGlobalsN\
ode renderInfo r\
enderLayerButton\
 renderLayerPare\
nt renderLayerPo\
stProcess render\
LayerUnparent re\
nderManip render\
Partition render\
QualityNode rend\
erSettings rende\
rThumbnailUpdate\
 renderWindowEdi\
tor renderWindow\
SelectContext re\
nderer reorder r\
eorderDeformers \
requires reroot \
resampleFluid re\
setAE resetPfxTo\
PolyCamera reset\
Tool resolutionN\
ode retarget rev\
erseCurve revers\
eSurface revolve\
 rgb_to_hsv rigi\
dBody rigidSolve\
r roll rollCtx r\
ootOf rot rotate\
 rotationInterpo\
lation roundCons\
tantRadius rowCo\
lumnLayout rowLa\
yout runTimeComm\
and runup sample\
Image saveAllShe\
lves saveAttrPre\
set saveFluid sa\
veImage saveInit\
ialState saveMen\
u savePrefObject\
s savePrefs save\
Shelf saveToolSe\
ttings scale sca\
leBrushBrightnes\
s scaleComponent\
s scaleConstrain\
t scaleKey scale\
KeyCtx sceneEdit\
or sceneUIReplac\
ement scmh scrip\
tCtx scriptEdito\
rInfo scriptJob \
scriptNode scrip\
tTable scriptToS\
helf scriptedPan\
el scriptedPanel\
Type scrollField\
 scrollLayout sc\
ulpt searchPathA\
rray seed selLoa\
dSettings select\
 selectContext s\
electCurveCV sel\
ectKey selectKey\
Ctx selectKeyfra\
meRegionCtx sele\
ctMode selectPre\
f selectPriority\
 selectType sele\
ctedNodes select\
ionConnection se\
parator setAttr \
setAttrEnumResou\
rce setAttrMappi\
ng setAttrNiceNa\
meResource setCo\
nstraintRestPosi\
tion setDefaultS\
hadingGroup setD\
rivenKeyframe se\
tDynamic setEdit\
Ctx setEditor se\
tFluidAttr setFo\
cus setInfinity \
setInputDeviceMa\
pping setKeyCtx \
setKeyPath setKe\
yframe setKeyfra\
meBlendshapeTarg\
etWts setMenuMod\
e setNodeNiceNam\
eResource setNod\
eTypeFlag setPar\
ent setParticleA\
ttr setPfxToPoly\
Camera setPlugin\
Resource setProj\
ect setStampDens\
ity setStartupMe\
ssage setState s\
etToolTo setUITe\
mplate setXformM\
anip sets shadin\
gConnection shad\
ingGeometryRelCt\
x shadingLightRe\
lCtx shadingNetw\
orkCompare shadi\
ngNode shapeComp\
are shelfButton \
shelfLayout shel\
fTabLayout shell\
Field shortNameO\
f showHelp showH\
idden showManipC\
tx showSelection\
InTitle showShad\
ingGroupAttrEdit\
or showWindow si\
gn simplify sin \
singleProfileBir\
ailSurface size \
sizeBytes skinCl\
uster skinPercen\
t smoothCurve sm\
oothTangentSurfa\
ce smoothstep sn\
ap2to2 snapKey s\
napMode snapToge\
therCtx snapshot\
 soft softMod so\
ftModCtx sort so\
und soundControl\
 source spaceLoc\
ator sphere sphr\
and spotLight sp\
otLightPreviewPo\
rt spreadSheetEd\
itor spring sqrt\
 squareSurface s\
rtContext stackT\
race startString\
 startsWith stit\
chAndExplodeShel\
l stitchSurface \
stitchSurfacePoi\
nts strcmp strin\
gArrayCatenate s\
tringArrayContai\
ns stringArrayCo\
unt stringArrayI\
nsertAtIndex str\
ingArrayIntersec\
tor stringArrayR\
emove stringArra\
yRemoveAtIndex s\
tringArrayRemove\
Duplicates strin\
gArrayRemoveExac\
t stringArrayToS\
tring stringToSt\
ringArray strip \
stripPrefixFromN\
ame stroke subdA\
utoProjection su\
bdCleanTopology \
subdCollapse sub\
dDuplicateAndCon\
nect subdEditUV \
subdListComponen\
tConversion subd\
MapCut subdMapSe\
wMove subdMatchT\
opology subdMirr\
or subdToBlind s\
ubdToPoly subdTr\
ansferUVsToCache\
 subdiv subdivCr\
ease subdivDispl\
aySmoothness sub\
stitute substitu\
teAllString subs\
tituteGeometry s\
ubstring surface\
 surfaceSampler \
surfaceShaderLis\
t swatchDisplayP\
ort switchTable \
symbolButton sym\
bolCheckBox sysF\
ile system tabLa\
yout tan tangent\
Constraint texLa\
tticeDeformConte\
xt texManipConte\
xt texMoveContex\
t texMoveUVShell\
Context texRotat\
eContext texScal\
eContext texSele\
ctContext texSel\
ectShortestPathC\
tx texSmudgeUVCo\
ntext texWinTool\
Ctx text textCur\
ves textField te\
xtFieldButtonGrp\
 textFieldGrp te\
xtManip textScro\
llList textToShe\
lf textureDispla\
cePlane textureH\
airColor texture\
PlacementContext\
 textureWindow t\
hreadCount three\
PointArcCtx time\
Control timePort\
 timerX toNative\
Path toggle togg\
leAxis toggleWin\
dowVisibility to\
kenize tokenizeL\
ist tolerance to\
lower toolButton\
 toolCollection \
toolDropped tool\
HasOptions toolP\
ropertyWindow to\
rus toupper trac\
e track trackCtx\
 transferAttribu\
tes transformCom\
pare transformLi\
mits translator \
trim trunc trunc\
ateFluidCache tr\
uncateHairCache \
tumble tumbleCtx\
 turbulence twoP\
ointArcCtx uiRes\
 uiTemplate unas\
signInputDevice \
undo undoInfo un\
group uniform un\
it unloadPlugin \
untangleUV untit\
ledFileName untr\
im upAxis update\
AE userCtx uvLin\
k uvSnapshot val\
idateShelfName v\
ectorize view2dT\
oolCtx viewCamer\
a viewClipPlane \
viewFit viewHead\
On viewLookAt vi\
ewManip viewPlac\
e viewSet visor \
volumeAxis vorte\
x waitCursor war\
ning webBrowser \
webBrowserPrefs \
whatIs window wi\
ndowPref wire wi\
reContext worksp\
ace wrinkle wrin\
kleContext write\
Take xbmLangPath\
List xform\x22,\x0aill\
egal:\x22</\x22,contai\
ns:[e.C_NUMBER_M\
ODE,e.APOS_STRIN\
G_MODE,e.QUOTE_S\
TRING_MODE,{\x0acla\
ssName:\x22string\x22,\
begin:\x22`\x22,end:\x22`\
\x22,contains:[e.BA\
CKSLASH_ESCAPE]}\
,{\x0abegin:/[$%@](\
\x5c^\x5cw\x5cb|#\x5cw+|[^\x5cs\
\x5cw{]|\x5c{\x5cw+\x5c}|\x5cw+\
)/\x0a},e.C_LINE_CO\
MMENT_MODE,e.C_B\
LOCK_COMMENT_MOD\
E]})})()\x0a;hljs.r\
egisterLanguage(\
\x22mel\x22,e)})();/*!\
 `mercury` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{\x0aconst i=e.COM\
MENT(\x22%\x22,\x22$\x22),n=\
e.inherit(e.APOS\
_STRING_MODE,{re\
levance:0\x0a}),r=e\
.inherit(e.QUOTE\
_STRING_MODE,{re\
levance:0})\x0a;ret\
urn r.contains=r\
.contains.slice(\
),r.contains.pus\
h({className:\x22su\
bst\x22,\x0abegin:\x22\x5c\x5c\x5c\
\x5c[abfnrtv]\x5c\x5c|\x5c\x5c\x5c\
\x5cx[0-9a-fA-F]*\x5c\x5c\
\x5c\x5c\x5c\x5c|%[-+# *.0-9\
]*[dioxXucsfeEgG\
p]\x22,\x0arelevance:0\
}),{name:\x22Mercur\
y\x22,aliases:[\x22m\x22,\
\x22moo\x22],keywords:\
{\x0akeyword:\x22modul\
e use_module imp\
ort_module inclu\
de_module end_mo\
dule initialise \
mutable initiali\
ze finalize fina\
lise interface i\
mplementation pr\
ed mode func typ\
e inst solver an\
y_pred any_func \
is semidet det n\
ondet multi erro\
neous failure cc\
_nondet cc_multi\
 typeclass insta\
nce where pragma\
 promise externa\
l trace atomic o\
r_else require_c\
omplete_switch r\
equire_det requi\
re_semidet requi\
re_multi require\
_nondet require_\
cc_multi require\
_cc_nondet requi\
re_erroneous req\
uire_failure\x22,\x0am\
eta:\x22inline no_i\
nline type_spec \
source_file fact\
_table obsolete \
memo loop_check \
minimal_model te\
rminates does_no\
t_terminate chec\
k_termination pr\
omise_equivalent\
_clauses foreign\
_proc foreign_de\
cl foreign_code \
foreign_type for\
eign_import_modu\
le foreign_expor\
t_enum foreign_e\
xport foreign_en\
um may_call_merc\
ury will_not_cal\
l_mercury thread\
_safe not_thread\
_safe maybe_thre\
ad_safe promise_\
pure promise_sem\
ipure tabled_for\
_io local untrai\
led trailed atta\
ch_to_io_state c\
an_pass_as_mercu\
ry_type stable w\
ill_not_throw_ex\
ception may_modi\
fy_trail will_no\
t_modify_trail m\
ay_duplicate may\
_not_duplicate a\
ffects_liveness \
does_not_affect_\
liveness doesnt_\
affect_liveness \
no_sharing unkno\
wn_sharing shari\
ng\x22,\x0abuilt_in:\x22s\
ome all not if t\
hen else true fa\
il false try cat\
ch catch_any sem\
idet_true semide\
t_false semidet_\
fail impure_true\
 impure semipure\
\x22\x0a},contains:[{c\
lassName:\x22built_\
in\x22,variants:[{b\
egin:\x22<=>\x22},{beg\
in:\x22<=\x22,\x0arelevan\
ce:0},{begin:\x22=>\
\x22,relevance:0},{\
begin:\x22/\x5c\x5c\x5c\x5c\x22},{\
begin:\x22\x5c\x5c\x5c\x5c/\x22}]}\
,{\x0aclassName:\x22bu\
ilt_in\x22,variants\
:[{begin:\x22:-\x5c\x5c|-\
-\x5cx3e\x22},{begin:\x22\
=\x22,relevance:0}]\
\x0a},i,e.C_BLOCK_C\
OMMENT_MODE,{cla\
ssName:\x22number\x22,\
begin:\x220'.\x5c\x5c|0[b\
ox][0-9a-fA-F]*\x22\
\x0a},e.NUMBER_MODE\
,n,r,{begin:/:-/\
},{begin:/\x5c.$/}]\
}}})()\x0a;hljs.reg\
isterLanguage(\x22m\
ercury\x22,e)})();/\
*! `mipsasm` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var e=(()=>{\x22use\
 strict\x22;return \
e=>({name:\x22MIPS \
Assembly\x22,\x0acase_\
insensitive:!0,a\
liases:[\x22mips\x22],\
keywords:{$patte\
rn:\x22\x5c\x5c.?\x22+e.IDEN\
T_RE,\x0ameta:\x22.2by\
te .4byte .align\
 .ascii .asciz .\
balign .byte .co\
de .data .else .\
end .endif .endm\
 .endr .equ .err\
 .exitm .extern \
.global .hword .\
if .ifdef .ifnde\
f .include .irp \
.long .macro .re\
pt .req .section\
 .set .skip .spa\
ce .text .word .\
ltorg \x22,\x0abuilt_i\
n:\x22$0 $1 $2 $3 $\
4 $5 $6 $7 $8 $9\
 $10 $11 $12 $13\
 $14 $15 $16 $17\
 $18 $19 $20 $21\
 $22 $23 $24 $25\
 $26 $27 $28 $29\
 $30 $31 zero at\
 v0 v1 a0 a1 a2 \
a3 a4 a5 a6 a7 t\
0 t1 t2 t3 t4 t5\
 t6 t7 t8 t9 s0 \
s1 s2 s3 s4 s5 s\
6 s7 s8 k0 k1 gp\
 sp fp ra $f0 $f\
1 $f2 $f2 $f4 $f\
5 $f6 $f7 $f8 $f\
9 $f10 $f11 $f12\
 $f13 $f14 $f15 \
$f16 $f17 $f18 $\
f19 $f20 $f21 $f\
22 $f23 $f24 $f2\
5 $f26 $f27 $f28\
 $f29 $f30 $f31 \
Context Random E\
ntryLo0 EntryLo1\
 Context PageMas\
k Wired EntryHi \
HWREna BadVAddr \
Count Compare SR\
 IntCtl SRSCtl S\
RSMap Cause EPC \
PRId EBase Confi\
g Config1 Config\
2 Config3 LLAddr\
 Debug DEPC DESA\
VE CacheErr ECC \
ErrorEPC TagLo D\
ataLo TagHi Data\
Hi WatchLo Watch\
Hi PerfCtl PerfC\
nt \x22\x0a},contains:\
[{className:\x22key\
word\x22,\x0abegin:\x22\x5c\x5c\
b(addi?u?|andi?|\
b(al)?|beql?|bge\
z(al)?l?|bgtzl?|\
blezl?|bltz(al)?\
l?|bnel?|cl[oz]|\
divu?|ext|ins|j(\
al)?|jalr(\x5c\x5c.hb)\
?|jr(\x5c\x5c.hb)?|lbu\
?|lhu?|ll|lui|lw\
[lr]?|maddu?|mfh\
i|mflo|movn|movz\
|move|msubu?|mth\
i|mtlo|mul|multu\
?|nop|nor|ori?|r\
otrv?|sb|sc|se[b\
h]|sh|sllv?|slti\
?u?|srav?|srlv?|\
subu?|sw[lr]?|xo\
ri?|wsbh|abs\x5c\x5c.[\
sd]|add\x5c\x5c.[sd]|a\
lnv.ps|bc1[ft]l?\
|c\x5c\x5c.(s?f|un|u?e\
q|[ou]lt|[ou]le|\
ngle?|seq|l[et]|\
ng[et])\x5c\x5c.[sd]|(\
ceil|floor|round\
|trunc)\x5c\x5c.[lw]\x5c\x5c\
.[sd]|cfc1|cvt\x5c\x5c\
.d\x5c\x5c.[lsw]|cvt\x5c\x5c\
.l\x5c\x5c.[dsw]|cvt\x5c\x5c\
.ps\x5c\x5c.s|cvt\x5c\x5c.s\x5c\
\x5c.[dlw]|cvt\x5c\x5c.s\x5c\
\x5c.p[lu]|cvt\x5c\x5c.w\x5c\
\x5c.[dls]|div\x5c\x5c.[d\
s]|ldx?c1|luxc1|\
lwx?c1|madd\x5c\x5c.[s\
d]|mfc1|mov[fntz\
]?\x5c\x5c.[ds]|msub\x5c\x5c\
.[sd]|mth?c1|mul\
\x5c\x5c.[ds]|neg\x5c\x5c.[d\
s]|nmadd\x5c\x5c.[ds]|\
nmsub\x5c\x5c.[ds]|p[l\
u][lu]\x5c\x5c.ps|reci\
p\x5c\x5c.fmt|r?sqrt\x5c\x5c\
.[ds]|sdx?c1|sub\
\x5c\x5c.[ds]|suxc1|sw\
x?c1|break|cache\
|d?eret|[de]i|eh\
b|mfc0|mtc0|paus\
e|prefx?|rdhwr|r\
dpgpr|sdbbp|ssno\
p|synci?|syscall\
|teqi?|tgei?u?|t\
lb(p|r|w[ir])|tl\
ti?u?|tnei?|wait\
|wrpgpr)\x22,\x0aend:\x22\
\x5c\x5cs\x22\x0a},e.COMMENT\
(\x22[;#](?!\x5c\x5cs*$)\x22\
,\x22$\x22),e.C_BLOCK_\
COMMENT_MODE,e.Q\
UOTE_STRING_MODE\
,{\x0aclassName:\x22st\
ring\x22,begin:\x22'\x22,\
end:\x22[^\x5c\x5c\x5c\x5c]'\x22,r\
elevance:0},{cla\
ssName:\x22title\x22,\x0a\
begin:\x22\x5c\x5c|\x22,end:\
\x22\x5c\x5c|\x22,illegal:\x22\x5c\
\x5cn\x22,relevance:0}\
,{className:\x22num\
ber\x22,variants:[{\
\x0abegin:\x220x[0-9a-\
f]+\x22},{begin:\x22\x5c\x5c\
b-?\x5c\x5cd+\x22}],relev\
ance:0},{classNa\
me:\x22symbol\x22,\x0avar\
iants:[{begin:\x22^\
\x5c\x5cs*[a-z_\x5c\x5c.\x5c\x5c$]\
[a-z0-9_\x5c\x5c.\x5c\x5c$]+\
:\x22},{begin:\x22^\x5c\x5cs\
*[0-9]+:\x22},{\x0abeg\
in:\x22[0-9]+[bf]\x22}\
],relevance:0}],\
illegal:/\x5c//})})\
()\x0a;hljs.registe\
rLanguage(\x22mipsa\
sm\x22,e)})();/*! `\
mizar` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;return e=>({n\
ame:\x22Mizar\x22,\x0akey\
words:\x22environ v\
ocabularies nota\
tions constructo\
rs definitions r\
egistrations the\
orems schemes re\
quirements begin\
 end definition \
registration clu\
ster existence p\
red func defpred\
 deffunc theorem\
 proof let take \
assume then thus\
 hence ex for st\
 holds consider \
reconsider such \
that and in prov\
ided of as from \
be being by mean\
s equals implies\
 iff redefine de\
fine now not or \
attr is mode sup\
pose per cases s\
et thesis contra\
diction scheme r\
eserve struct co\
rrectness compat\
ibility coherenc\
e symmetry assym\
etry reflexivity\
 irreflexivity c\
onnectedness uni\
queness commutat\
ivity idempotenc\
e involutiveness\
 projectivity\x22,\x0a\
contains:[e.COMM\
ENT(\x22::\x22,\x22$\x22)]})\
})();hljs.regist\
erLanguage(\x22miza\
r\x22,e)})();/*! `m\
ojolicious` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>({name:\x22Mojoli\
cious\x22,subLangua\
ge:\x22xml\x22,\x0acontai\
ns:[{className:\x22\
meta\x22,begin:\x22^__\
(END|DATA)__$\x22},\
{\x0abegin:\x22^\x5c\x5cs*%{\
1,2}={0,2}\x22,end:\
\x22$\x22,subLanguage:\
\x22perl\x22},{begin:\x22\
<%{1,2}={0,2}\x22,\x0a\
end:\x22={0,1}%>\x22,s\
ubLanguage:\x22perl\
\x22,excludeBegin:!\
0,excludeEnd:!0}\
]})})()\x0a;hljs.re\
gisterLanguage(\x22\
mojolicious\x22,e)}\
)();/*! `monkey`\
 grammar compile\
d for Highlight.\
js 11.10.0 */\x0a((\
)=>{var e=(()=>{\
\x22use strict\x22;ret\
urn e=>{const n=\
{className:\x22numb\
er\x22,\x0arelevance:0\
,variants:[{begi\
n:\x22[$][a-fA-F0-9\
]+\x22},e.NUMBER_MO\
DE]},a={variants\
:[{\x0amatch:[/(fun\
ction|method)/,/\
\x5cs+/,e.UNDERSCOR\
E_IDENT_RE]}],sc\
ope:{1:\x22keyword\x22\
,\x0a3:\x22title.funct\
ion\x22}},t={varian\
ts:[{\x0amatch:[/(c\
lass|interface|e\
xtends|implement\
s)/,/\x5cs+/,e.UNDE\
RSCORE_IDENT_RE]\
}],\x0ascope:{1:\x22ke\
yword\x22,3:\x22title.\
class\x22}};return{\
name:\x22Monkey\x22,ca\
se_insensitive:!\
0,\x0akeywords:{\x0ake\
yword:[\x22public\x22,\
\x22private\x22,\x22prope\
rty\x22,\x22continue\x22,\
\x22exit\x22,\x22extern\x22,\
\x22new\x22,\x22try\x22,\x22cat\
ch\x22,\x22eachin\x22,\x22no\
t\x22,\x22abstract\x22,\x22f\
inal\x22,\x22select\x22,\x22\
case\x22,\x22default\x22,\
\x22const\x22,\x22local\x22,\
\x22global\x22,\x22field\x22\
,\x22end\x22,\x22if\x22,\x22the\
n\x22,\x22else\x22,\x22elsei\
f\x22,\x22endif\x22,\x22whil\
e\x22,\x22wend\x22,\x22repea\
t\x22,\x22until\x22,\x22fore\
ver\x22,\x22for\x22,\x22to\x22,\
\x22step\x22,\x22next\x22,\x22r\
eturn\x22,\x22module\x22,\
\x22inline\x22,\x22throw\x22\
,\x22import\x22,\x22and\x22,\
\x22or\x22,\x22shl\x22,\x22shr\x22\
,\x22mod\x22],\x0abuilt_i\
n:[\x22DebugLog\x22,\x22D\
ebugStop\x22,\x22Error\
\x22,\x22Print\x22,\x22ACos\x22\
,\x22ACosr\x22,\x22ASin\x22,\
\x22ASinr\x22,\x22ATan\x22,\x22\
ATan2\x22,\x22ATan2r\x22,\
\x22ATanr\x22,\x22Abs\x22,\x22A\
bs\x22,\x22Ceil\x22,\x22Clam\
p\x22,\x22Clamp\x22,\x22Cos\x22\
,\x22Cosr\x22,\x22Exp\x22,\x22F\
loor\x22,\x22Log\x22,\x22Max\
\x22,\x22Max\x22,\x22Min\x22,\x22M\
in\x22,\x22Pow\x22,\x22Sgn\x22,\
\x22Sgn\x22,\x22Sin\x22,\x22Sin\
r\x22,\x22Sqrt\x22,\x22Tan\x22,\
\x22Tanr\x22,\x22Seed\x22,\x22P\
I\x22,\x22HALFPI\x22,\x22TWO\
PI\x22],\x0aliteral:[\x22\
true\x22,\x22false\x22,\x22n\
ull\x22]},illegal:/\
\x5c/\x5c*/,\x0acontains:\
[e.COMMENT(\x22#rem\
\x22,\x22#end\x22),e.COMM\
ENT(\x22'\x22,\x22$\x22,{rel\
evance:0}),a,t,{\
\x0aclassName:\x22vari\
able.language\x22,b\
egin:/\x5cb(self|su\
per)\x5cb/},{classN\
ame:\x22meta\x22,\x0abegi\
n:/\x5cs*#/,end:\x22$\x22\
,keywords:{keywo\
rd:\x22if else else\
if endif end the\
n\x22}},{\x0amatch:[/^\
\x5cs*/,/strict\x5cb/]\
,scope:{2:\x22meta\x22\
}},{beginKeyword\
s:\x22alias\x22,end:\x22=\
\x22,\x0acontains:[e.U\
NDERSCORE_TITLE_\
MODE]},e.QUOTE_S\
TRING_MODE,n]}}}\
)()\x0a;hljs.regist\
erLanguage(\x22monk\
ey\x22,e)})();/*! `\
moonscript` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>{const n={\x0akey\
word:\x22if then no\
t for in while d\
o return else el\
seif break conti\
nue switch and o\
r unless when cl\
ass extends supe\
r local import e\
xport from using\
\x22,\x0aliteral:\x22true\
 false nil\x22,\x0abui\
lt_in:\x22_G _VERSI\
ON assert collec\
tgarbage dofile \
error getfenv ge\
tmetatable ipair\
s load loadfile \
loadstring modul\
e next pairs pca\
ll print rawequa\
l rawget rawset \
require select s\
etfenv setmetata\
ble tonumber tos\
tring type unpac\
k xpcall corouti\
ne debug io math\
 os package stri\
ng table\x22\x0a},s=\x22[\
A-Za-z$_][0-9A-Z\
a-z$_]*\x22,a={clas\
sName:\x22subst\x22,be\
gin:/#\x5c{/,end:/\x5c\
}/,\x0akeywords:n},\
t=[e.inherit(e.C\
_NUMBER_MODE,{st\
arts:{end:\x22(\x5c\x5cs*\
/)?\x22,relevance:0\
}\x0a}),{className:\
\x22string\x22,variant\
s:[{begin:/'/,en\
d:/'/,\x0acontains:\
[e.BACKSLASH_ESC\
APE]},{begin:/\x22/\
,end:/\x22/,\x0acontai\
ns:[e.BACKSLASH_\
ESCAPE,a]}]},{cl\
assName:\x22built_i\
n\x22,begin:\x22@__\x22+e\
.IDENT_RE\x0a},{beg\
in:\x22@\x22+e.IDENT_R\
E},{begin:e.IDEN\
T_RE+\x22\x5c\x5c\x5c\x5c\x22+e.ID\
ENT_RE}];a.conta\
ins=t\x0a;const i=e\
.inherit(e.TITLE\
_MODE,{begin:s})\
,r=\x22(\x5c\x5c(.*\x5c\x5c)\x5c\x5cs\
*)?\x5c\x5cB[-=]>\x22,l={\
\x0aclassName:\x22para\
ms\x22,begin:\x22\x5c\x5c([^\
\x5c\x5c(]\x22,returnBegi\
n:!0,contains:[{\
begin:/\x5c(/,\x0aend:\
/\x5c)/,keywords:n,\
contains:[\x22self\x22\
].concat(t)}]};r\
eturn{name:\x22Moon\
Script\x22,\x0aaliases\
:[\x22moon\x22],keywor\
ds:n,illegal:/\x5c/\
\x5c*/,\x0acontains:t.\
concat([e.COMMEN\
T(\x22--\x22,\x22$\x22),{cla\
ssName:\x22function\
\x22,\x0abegin:\x22^\x5c\x5cs*\x22\
+s+\x22\x5c\x5cs*=\x5c\x5cs*\x22+r\
,end:\x22[-=]>\x22,ret\
urnBegin:!0,cont\
ains:[i,l]},{\x0abe\
gin:/[\x5c(,:=]\x5cs*/\
,relevance:0,con\
tains:[{classNam\
e:\x22function\x22,beg\
in:r,\x0aend:\x22[-=]>\
\x22,returnBegin:!0\
,contains:[l]}]}\
,{className:\x22cla\
ss\x22,\x0abeginKeywor\
ds:\x22class\x22,end:\x22\
$\x22,illegal:/[:=\x22\
\x5c[\x5c]]/,contains:\
[{\x0abeginKeywords\
:\x22extends\x22,endsW\
ithParent:!0,ill\
egal:/[:=\x22\x5c[\x5c]]/\
,contains:[i]},i\
]\x0a},{className:\x22\
name\x22,begin:s+\x22:\
\x22,end:\x22:\x22,return\
Begin:!0,returnE\
nd:!0,relevance:\
0\x0a}])}}})();hljs\
.registerLanguag\
e(\x22moonscript\x22,e\
)})();/*! `n1ql`\
 grammar compile\
d for Highlight.\
js 11.10.0 */\x0a((\
)=>{var e=(()=>{\
\x22use strict\x22;ret\
urn e=>({name:\x22N\
1QL\x22,case_insens\
itive:!0,\x0acontai\
ns:[{\x0abeginKeywo\
rds:\x22build creat\
e index delete d\
rop explain infe\
r|10 insert merg\
e prepare select\
 update upsert|1\
0\x22,\x0aend:/;/,keyw\
ords:{\x0akeyword:[\
\x22all\x22,\x22alter\x22,\x22a\
nalyze\x22,\x22and\x22,\x22a\
ny\x22,\x22array\x22,\x22as\x22\
,\x22asc\x22,\x22begin\x22,\x22\
between\x22,\x22binary\
\x22,\x22boolean\x22,\x22bre\
ak\x22,\x22bucket\x22,\x22bu\
ild\x22,\x22by\x22,\x22call\x22\
,\x22case\x22,\x22cast\x22,\x22\
cluster\x22,\x22collat\
e\x22,\x22collection\x22,\
\x22commit\x22,\x22connec\
t\x22,\x22continue\x22,\x22c\
orrelate\x22,\x22cover\
\x22,\x22create\x22,\x22data\
base\x22,\x22dataset\x22,\
\x22datastore\x22,\x22dec\
lare\x22,\x22decrement\
\x22,\x22delete\x22,\x22deri\
ved\x22,\x22desc\x22,\x22des\
cribe\x22,\x22distinct\
\x22,\x22do\x22,\x22drop\x22,\x22e\
ach\x22,\x22element\x22,\x22\
else\x22,\x22end\x22,\x22eve\
ry\x22,\x22except\x22,\x22ex\
clude\x22,\x22execute\x22\
,\x22exists\x22,\x22expla\
in\x22,\x22fetch\x22,\x22fir\
st\x22,\x22flatten\x22,\x22f\
or\x22,\x22force\x22,\x22fro\
m\x22,\x22function\x22,\x22g\
rant\x22,\x22group\x22,\x22g\
si\x22,\x22having\x22,\x22if\
\x22,\x22ignore\x22,\x22ilik\
e\x22,\x22in\x22,\x22include\
\x22,\x22increment\x22,\x22i\
ndex\x22,\x22infer\x22,\x22i\
nline\x22,\x22inner\x22,\x22\
insert\x22,\x22interse\
ct\x22,\x22into\x22,\x22is\x22,\
\x22join\x22,\x22key\x22,\x22ke\
ys\x22,\x22keyspace\x22,\x22\
known\x22,\x22last\x22,\x22l\
eft\x22,\x22let\x22,\x22lett\
ing\x22,\x22like\x22,\x22lim\
it\x22,\x22lsm\x22,\x22map\x22,\
\x22mapping\x22,\x22match\
ed\x22,\x22materialize\
d\x22,\x22merge\x22,\x22minu\
s\x22,\x22namespace\x22,\x22\
nest\x22,\x22not\x22,\x22num\
ber\x22,\x22object\x22,\x22o\
ffset\x22,\x22on\x22,\x22opt\
ion\x22,\x22or\x22,\x22order\
\x22,\x22outer\x22,\x22over\x22\
,\x22parse\x22,\x22partit\
ion\x22,\x22password\x22,\
\x22path\x22,\x22pool\x22,\x22p\
repare\x22,\x22primary\
\x22,\x22private\x22,\x22pri\
vilege\x22,\x22procedu\
re\x22,\x22public\x22,\x22ra\
w\x22,\x22realm\x22,\x22redu\
ce\x22,\x22rename\x22,\x22re\
turn\x22,\x22returning\
\x22,\x22revoke\x22,\x22righ\
t\x22,\x22role\x22,\x22rollb\
ack\x22,\x22satisfies\x22\
,\x22schema\x22,\x22selec\
t\x22,\x22self\x22,\x22semi\x22\
,\x22set\x22,\x22show\x22,\x22s\
ome\x22,\x22start\x22,\x22st\
atistics\x22,\x22strin\
g\x22,\x22system\x22,\x22the\
n\x22,\x22to\x22,\x22transac\
tion\x22,\x22trigger\x22,\
\x22truncate\x22,\x22unde\
r\x22,\x22union\x22,\x22uniq\
ue\x22,\x22unknown\x22,\x22u\
nnest\x22,\x22unset\x22,\x22\
update\x22,\x22upsert\x22\
,\x22use\x22,\x22user\x22,\x22u\
sing\x22,\x22validate\x22\
,\x22value\x22,\x22valued\
\x22,\x22values\x22,\x22via\x22\
,\x22view\x22,\x22when\x22,\x22\
where\x22,\x22while\x22,\x22\
with\x22,\x22within\x22,\x22\
work\x22,\x22xor\x22],\x0ali\
teral:[\x22true\x22,\x22f\
alse\x22,\x22null\x22,\x22mi\
ssing|5\x22],\x0abuilt\
_in:[\x22array_agg\x22\
,\x22array_append\x22,\
\x22array_concat\x22,\x22\
array_contains\x22,\
\x22array_count\x22,\x22a\
rray_distinct\x22,\x22\
array_ifnull\x22,\x22a\
rray_length\x22,\x22ar\
ray_max\x22,\x22array_\
min\x22,\x22array_posi\
tion\x22,\x22array_pre\
pend\x22,\x22array_put\
\x22,\x22array_range\x22,\
\x22array_remove\x22,\x22\
array_repeat\x22,\x22a\
rray_replace\x22,\x22a\
rray_reverse\x22,\x22a\
rray_sort\x22,\x22arra\
y_sum\x22,\x22avg\x22,\x22co\
unt\x22,\x22max\x22,\x22min\x22\
,\x22sum\x22,\x22greatest\
\x22,\x22least\x22,\x22ifmis\
sing\x22,\x22ifmissing\
ornull\x22,\x22ifnull\x22\
,\x22missingif\x22,\x22nu\
llif\x22,\x22ifinf\x22,\x22i\
fnan\x22,\x22ifnanorin\
f\x22,\x22naninf\x22,\x22neg\
infif\x22,\x22posinfif\
\x22,\x22clock_millis\x22\
,\x22clock_str\x22,\x22da\
te_add_millis\x22,\x22\
date_add_str\x22,\x22d\
ate_diff_millis\x22\
,\x22date_diff_str\x22\
,\x22date_part_mill\
is\x22,\x22date_part_s\
tr\x22,\x22date_trunc_\
millis\x22,\x22date_tr\
unc_str\x22,\x22durati\
on_to_str\x22,\x22mill\
is\x22,\x22str_to_mill\
is\x22,\x22millis_to_s\
tr\x22,\x22millis_to_u\
tc\x22,\x22millis_to_z\
one_name\x22,\x22now_m\
illis\x22,\x22now_str\x22\
,\x22str_to_duratio\
n\x22,\x22str_to_utc\x22,\
\x22str_to_zone_nam\
e\x22,\x22decode_json\x22\
,\x22encode_json\x22,\x22\
encoded_size\x22,\x22p\
oly_length\x22,\x22bas\
e64\x22,\x22base64_enc\
ode\x22,\x22base64_dec\
ode\x22,\x22meta\x22,\x22uui\
d\x22,\x22abs\x22,\x22acos\x22,\
\x22asin\x22,\x22atan\x22,\x22a\
tan2\x22,\x22ceil\x22,\x22co\
s\x22,\x22degrees\x22,\x22e\x22\
,\x22exp\x22,\x22ln\x22,\x22log\
\x22,\x22floor\x22,\x22pi\x22,\x22\
power\x22,\x22radians\x22\
,\x22random\x22,\x22round\
\x22,\x22sign\x22,\x22sin\x22,\x22\
sqrt\x22,\x22tan\x22,\x22tru\
nc\x22,\x22object_leng\
th\x22,\x22object_name\
s\x22,\x22object_pairs\
\x22,\x22object_inner_\
pairs\x22,\x22object_v\
alues\x22,\x22object_i\
nner_values\x22,\x22ob\
ject_add\x22,\x22objec\
t_put\x22,\x22object_r\
emove\x22,\x22object_u\
nwrap\x22,\x22regexp_c\
ontains\x22,\x22regexp\
_like\x22,\x22regexp_p\
osition\x22,\x22regexp\
_replace\x22,\x22conta\
ins\x22,\x22initcap\x22,\x22\
length\x22,\x22lower\x22,\
\x22ltrim\x22,\x22positio\
n\x22,\x22repeat\x22,\x22rep\
lace\x22,\x22rtrim\x22,\x22s\
plit\x22,\x22substr\x22,\x22\
title\x22,\x22trim\x22,\x22u\
pper\x22,\x22isarray\x22,\
\x22isatom\x22,\x22isbool\
ean\x22,\x22isnumber\x22,\
\x22isobject\x22,\x22isst\
ring\x22,\x22type\x22,\x22to\
array\x22,\x22toatom\x22,\
\x22toboolean\x22,\x22ton\
umber\x22,\x22toobject\
\x22,\x22tostring\x22]\x0a},\
contains:[{class\
Name:\x22string\x22,be\
gin:\x22'\x22,end:\x22'\x22,\
contains:[e.BACK\
SLASH_ESCAPE]\x0a},\
{className:\x22stri\
ng\x22,begin:'\x22',en\
d:'\x22',contains:[\
e.BACKSLASH_ESCA\
PE]},{\x0aclassName\
:\x22symbol\x22,begin:\
\x22`\x22,end:\x22`\x22,cont\
ains:[e.BACKSLAS\
H_ESCAPE]\x0a},e.C_\
NUMBER_MODE,e.C_\
BLOCK_COMMENT_MO\
DE]},e.C_BLOCK_C\
OMMENT_MODE]})})\
()\x0a;hljs.registe\
rLanguage(\x22n1ql\x22\
,e)})();/*! `nes\
tedtext` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
t=(()=>{\x22use str\
ict\x22;return t=>(\
{name:\x22Nested Te\
xt\x22,aliases:[\x22nt\
\x22],\x0acontains:[t.\
inherit(t.HASH_C\
OMMENT_MODE,{beg\
in:/^\x5cs*(?=#)/,e\
xcludeBegin:!0})\
,{\x0avariants:[{ma\
tch:[/^\x5cs*/,/-/,\
/[ ]/,/.*$/]},{m\
atch:[/^\x5cs*/,/-$\
/]}],className:{\
\x0a2:\x22bullet\x22,4:\x22s\
tring\x22}},{match:\
[/^\x5cs*/,/>/,/[ ]\
/,/.*$/],classNa\
me:{\x0a2:\x22punctuat\
ion\x22,4:\x22string\x22}\
},{match:[/^\x5cs*(\
?=\x5cS)/,/[^:]+/,/\
:\x5cs*/,/$/],\x0aclas\
sName:{2:\x22attrib\
ute\x22,3:\x22punctuat\
ion\x22}},{\x0amatch:[\
/^\x5cs*(?=\x5cS)/,/[^\
:]*[^: ]/,/[ ]*:\
/,/[ ]/,/.*$/],c\
lassName:{2:\x22att\
ribute\x22,\x0a3:\x22punc\
tuation\x22,5:\x22stri\
ng\x22}}]})})();hlj\
s.registerLangua\
ge(\x22nestedtext\x22,\
t)})();/*! `ngin\
x` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>{const \
n=e.regex,a={\x0acl\
assName:\x22variabl\
e\x22,variants:[{be\
gin:/\x5c$\x5cd+/},{be\
gin:/\x5c$\x5c{\x5cw+\x5c}/}\
,{\x0abegin:n.conca\
t(/[$@]/,e.UNDER\
SCORE_IDENT_RE)}\
]},s={endsWithPa\
rent:!0,keywords\
:{\x0a$pattern:/[a-\
z_]{2,}|\x5c/dev\x5c/p\
oll/,\x0aliteral:[\x22\
on\x22,\x22off\x22,\x22yes\x22,\
\x22no\x22,\x22true\x22,\x22fal\
se\x22,\x22none\x22,\x22bloc\
ked\x22,\x22debug\x22,\x22in\
fo\x22,\x22notice\x22,\x22wa\
rn\x22,\x22error\x22,\x22cri\
t\x22,\x22select\x22,\x22bre\
ak\x22,\x22last\x22,\x22perm\
anent\x22,\x22redirect\
\x22,\x22kqueue\x22,\x22rtsi\
g\x22,\x22epoll\x22,\x22poll\
\x22,\x22/dev/poll\x22]\x0a}\
,relevance:0,ill\
egal:\x22=>\x22,contai\
ns:[e.HASH_COMME\
NT_MODE,{classNa\
me:\x22string\x22,\x0acon\
tains:[e.BACKSLA\
SH_ESCAPE,a],var\
iants:[{begin:/\x22\
/,end:/\x22/},{begi\
n:/'/,end:/'/\x0a}]\
},{begin:\x22([a-z]\
+):/\x22,end:\x22\x5c\x5cs\x22,\
endsWithParent:!\
0,excludeEnd:!0,\
contains:[a]\x0a},{\
className:\x22regex\
p\x22,contains:[e.B\
ACKSLASH_ESCAPE,\
a],variants:[{be\
gin:\x22\x5c\x5cs\x5c\x5c^\x22,\x0aen\
d:\x22\x5c\x5cs|\x5c\x5c{|;\x22,re\
turnEnd:!0},{beg\
in:\x22~\x5c\x5c*?\x5c\x5cs+\x22,e\
nd:\x22\x5c\x5cs|\x5c\x5c{|;\x22,r\
eturnEnd:!0},{\x0ab\
egin:\x22\x5c\x5c*(\x5c\x5c.[a-\
z\x5c\x5c-]+)+\x22},{begi\
n:\x22([a-z\x5c\x5c-]+\x5c\x5c.\
)+\x5c\x5c*\x22}]},{class\
Name:\x22number\x22,\x0ab\
egin:\x22\x5c\x5cb\x5c\x5cd{1,3\
}\x5c\x5c.\x5c\x5cd{1,3}\x5c\x5c.\x5c\
\x5cd{1,3}\x5c\x5c.\x5c\x5cd{1,\
3}(:\x5c\x5cd{1,5})?\x5c\x5c\
b\x22},{\x0aclassName:\
\x22number\x22,begin:\x22\
\x5c\x5cb\x5c\x5cd+[kKmMgGds\
hdwy]?\x5c\x5cb\x22,relev\
ance:0},a]};retu\
rn{\x0aname:\x22Nginx \
config\x22,aliases:\
[\x22nginxconf\x22],co\
ntains:[e.HASH_C\
OMMENT_MODE,{\x0abe\
ginKeywords:\x22ups\
tream location\x22,\
end:/;|\x5c{/,conta\
ins:s.contains,k\
eywords:{\x0asectio\
n:\x22upstream loca\
tion\x22}},{classNa\
me:\x22section\x22,\x0abe\
gin:n.concat(e.U\
NDERSCORE_IDENT_\
RE+n.lookahead(/\
\x5cs+\x5c{/)),relevan\
ce:0},{\x0abegin:n.\
lookahead(e.UNDE\
RSCORE_IDENT_RE+\
\x22\x5c\x5cs\x22),end:\x22;|\x5c\x5c\
{\x22,contains:[{\x0ac\
lassName:\x22attrib\
ute\x22,begin:e.UND\
ERSCORE_IDENT_RE\
,starts:s}],rele\
vance:0}],\x0ailleg\
al:\x22[^\x5c\x5cs\x5c\x5c}\x5c\x5c{]\
\x22}}})();hljs.reg\
isterLanguage(\x22n\
ginx\x22,e)})();/*!\
 `nim` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;return e=>({n\
ame:\x22Nim\x22,keywor\
ds:{\x0akeyword:[\x22a\
ddr\x22,\x22and\x22,\x22as\x22,\
\x22asm\x22,\x22bind\x22,\x22bl\
ock\x22,\x22break\x22,\x22ca\
se\x22,\x22cast\x22,\x22cons\
t\x22,\x22continue\x22,\x22c\
onverter\x22,\x22disca\
rd\x22,\x22distinct\x22,\x22\
div\x22,\x22do\x22,\x22elif\x22\
,\x22else\x22,\x22end\x22,\x22e\
num\x22,\x22except\x22,\x22e\
xport\x22,\x22finally\x22\
,\x22for\x22,\x22from\x22,\x22f\
unc\x22,\x22generic\x22,\x22\
guarded\x22,\x22if\x22,\x22i\
mport\x22,\x22in\x22,\x22inc\
lude\x22,\x22interface\
\x22,\x22is\x22,\x22isnot\x22,\x22\
iterator\x22,\x22let\x22,\
\x22macro\x22,\x22method\x22\
,\x22mixin\x22,\x22mod\x22,\x22\
nil\x22,\x22not\x22,\x22noti\
n\x22,\x22object\x22,\x22of\x22\
,\x22or\x22,\x22out\x22,\x22pro\
c\x22,\x22ptr\x22,\x22raise\x22\
,\x22ref\x22,\x22return\x22,\
\x22shared\x22,\x22shl\x22,\x22\
shr\x22,\x22static\x22,\x22t\
emplate\x22,\x22try\x22,\x22\
tuple\x22,\x22type\x22,\x22u\
sing\x22,\x22var\x22,\x22whe\
n\x22,\x22while\x22,\x22with\
\x22,\x22without\x22,\x22xor\
\x22,\x22yield\x22],\x0alite\
ral:[\x22true\x22,\x22fal\
se\x22],\x0atype:[\x22int\
\x22,\x22int8\x22,\x22int16\x22\
,\x22int32\x22,\x22int64\x22\
,\x22uint\x22,\x22uint8\x22,\
\x22uint16\x22,\x22uint32\
\x22,\x22uint64\x22,\x22floa\
t\x22,\x22float32\x22,\x22fl\
oat64\x22,\x22bool\x22,\x22c\
har\x22,\x22string\x22,\x22c\
string\x22,\x22pointer\
\x22,\x22expr\x22,\x22stmt\x22,\
\x22void\x22,\x22auto\x22,\x22a\
ny\x22,\x22range\x22,\x22arr\
ay\x22,\x22openarray\x22,\
\x22varargs\x22,\x22seq\x22,\
\x22set\x22,\x22clong\x22,\x22c\
ulong\x22,\x22cchar\x22,\x22\
cschar\x22,\x22cshort\x22\
,\x22cint\x22,\x22csize\x22,\
\x22clonglong\x22,\x22cfl\
oat\x22,\x22cdouble\x22,\x22\
clongdouble\x22,\x22cu\
char\x22,\x22cushort\x22,\
\x22cuint\x22,\x22culongl\
ong\x22,\x22cstringarr\
ay\x22,\x22semistatic\x22\
],\x0abuilt_in:[\x22st\
din\x22,\x22stdout\x22,\x22s\
tderr\x22,\x22result\x22]\
},contains:[{cla\
ssName:\x22meta\x22,\x0ab\
egin:/\x5c{\x5c./,end:\
/\x5c.\x5c}/,relevance\
:10},{className:\
\x22string\x22,begin:/\
[a-zA-Z]\x5cw*\x22/,\x0ae\
nd:/\x22/,contains:\
[{begin:/\x22\x22/}]},\
{className:\x22stri\
ng\x22,begin:/([a-z\
A-Z]\x5cw*)?\x22\x22\x22/,\x0ae\
nd:/\x22\x22\x22/},e.QUOT\
E_STRING_MODE,{c\
lassName:\x22type\x22,\
begin:/\x5cb[A-Z]\x5cw\
+\x5cb/,\x0arelevance:\
0},{className:\x22n\
umber\x22,relevance\
:0,variants:[{\x0ab\
egin:/\x5cb(0[xX][0\
-9a-fA-F][_0-9a-\
fA-F]*)('?[iIuU]\
(8|16|32|64))?/}\
,{\x0abegin:/\x5cb(0o[\
0-7][_0-7]*)('?[\
iIuUfF](8|16|32|\
64))?/},{\x0abegin:\
/\x5cb(0(b|B)[01][_\
01]*)('?[iIuUfF]\
(8|16|32|64))?/}\
,{\x0abegin:/\x5cb(\x5cd[\
_\x5cd]*)('?[iIuUfF\
](8|16|32|64))?/\
}]},e.HASH_COMME\
NT_MODE]})})()\x0a;\
hljs.registerLan\
guage(\x22nim\x22,e)})\
();/*! `nix` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var e=(()=>{\x22use\
 strict\x22;return \
e=>{const n={\x0ake\
yword:[\x22rec\x22,\x22wi\
th\x22,\x22let\x22,\x22in\x22,\x22\
inherit\x22,\x22assert\
\x22,\x22if\x22,\x22else\x22,\x22t\
hen\x22],\x0aliteral:[\
\x22true\x22,\x22false\x22,\x22\
or\x22,\x22and\x22,\x22null\x22\
],\x0abuilt_in:[\x22im\
port\x22,\x22abort\x22,\x22b\
aseNameOf\x22,\x22dirO\
f\x22,\x22isNull\x22,\x22bui\
ltins\x22,\x22map\x22,\x22re\
moveAttrs\x22,\x22thro\
w\x22,\x22toString\x22,\x22d\
erivation\x22]\x0a},s=\
{className:\x22subs\
t\x22,begin:/\x5c$\x5c{/,\
end:/\x5c}/,keyword\
s:n},a={classNam\
e:\x22string\x22,\x0acont\
ains:[{className\
:\x22char.escape\x22,b\
egin:/''\x5c$/},s],\
variants:[{begin\
:\x22''\x22,\x0aend:\x22''\x22}\
,{begin:'\x22',end:\
'\x22'}]\x0a},i=[e.NUM\
BER_MODE,e.HASH_\
COMMENT_MODE,e.C\
_BLOCK_COMMENT_M\
ODE,a,{\x0abegin:/[\
a-zA-Z0-9-_]+(\x5cs\
*=)/,returnBegin\
:!0,relevance:0,\
contains:[{\x0aclas\
sName:\x22attr\x22,beg\
in:/\x5cS+/,relevan\
ce:.2}]}];return\
 s.contains=i,{n\
ame:\x22Nix\x22,\x0aalias\
es:[\x22nixos\x22],key\
words:n,contains\
:i}}})();hljs.re\
gisterLanguage(\x22\
nix\x22,e)\x0a})();/*!\
 `node-repl` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var a=(()=>{\x22use\
 strict\x22;return \
a=>({name:\x22Node \
REPL\x22,contains:[\
{\x0aclassName:\x22met\
a.prompt\x22,starts\
:{end:/ |$/,star\
ts:{end:\x22$\x22,\x0asub\
Language:\x22javasc\
ript\x22}},variants\
:[{begin:/^>(?=[\
 ]|$)/},{\x0abegin:\
/^\x5c.\x5c.\x5c.(?=[ ]|$\
)/}]}]})})();hlj\
s.registerLangua\
ge(\x22node-repl\x22,a\
)})();/*! `nsis`\
 grammar compile\
d for Highlight.\
js 11.10.0 */\x0a((\
)=>{var e=(()=>{\
\x22use strict\x22;ret\
urn e=>{const t=\
e.regex,n={\x0aclas\
sName:\x22variable.\
constant\x22,\x0abegin\
:t.concat(/\x5c$/,t\
.either(\x22ADMINTO\
OLS\x22,\x22APPDATA\x22,\x22\
CDBURN_AREA\x22,\x22CM\
DLINE\x22,\x22COMMONFI\
LES32\x22,\x22COMMONFI\
LES64\x22,\x22COMMONFI\
LES\x22,\x22COOKIES\x22,\x22\
DESKTOP\x22,\x22DOCUME\
NTS\x22,\x22EXEDIR\x22,\x22E\
XEFILE\x22,\x22EXEPATH\
\x22,\x22FAVORITES\x22,\x22F\
ONTS\x22,\x22HISTORY\x22,\
\x22HWNDPARENT\x22,\x22IN\
STDIR\x22,\x22INTERNET\
_CACHE\x22,\x22LANGUAG\
E\x22,\x22LOCALAPPDATA\
\x22,\x22MUSIC\x22,\x22NETHO\
OD\x22,\x22OUTDIR\x22,\x22PI\
CTURES\x22,\x22PLUGINS\
DIR\x22,\x22PRINTHOOD\x22\
,\x22PROFILE\x22,\x22PROG\
RAMFILES32\x22,\x22PRO\
GRAMFILES64\x22,\x22PR\
OGRAMFILES\x22,\x22QUI\
CKLAUNCH\x22,\x22RECEN\
T\x22,\x22RESOURCES_LO\
CALIZED\x22,\x22RESOUR\
CES\x22,\x22SENDTO\x22,\x22S\
MPROGRAMS\x22,\x22SMST\
ARTUP\x22,\x22STARTMEN\
U\x22,\x22SYSDIR\x22,\x22TEM\
P\x22,\x22TEMPLATES\x22,\x22\
VIDEOS\x22,\x22WINDIR\x22\
))\x0a},i={classNam\
e:\x22variable\x22,beg\
in:/\x5c$+\x5c{[\x5c!\x5cw.:\
-]+\x5c}/},r={class\
Name:\x22variable\x22,\
\x0abegin:/\x5c$+\x5cw[\x5cw\
\x5c.]*/,illegal:/\x5c\
(\x5c)\x5c{\x5c}/},a={cla\
ssName:\x22variable\
\x22,\x0abegin:/\x5c$+\x5c([\
\x5cw^.:!-]+\x5c)/},l=\
{className:\x22para\
ms\x22,\x0abegin:t.eit\
her(\x22ARCHIVE\x22,\x22F\
ILE_ATTRIBUTE_AR\
CHIVE\x22,\x22FILE_ATT\
RIBUTE_NORMAL\x22,\x22\
FILE_ATTRIBUTE_O\
FFLINE\x22,\x22FILE_AT\
TRIBUTE_READONLY\
\x22,\x22FILE_ATTRIBUT\
E_SYSTEM\x22,\x22FILE_\
ATTRIBUTE_TEMPOR\
ARY\x22,\x22HKCR\x22,\x22HKC\
U\x22,\x22HKDD\x22,\x22HKEY_\
CLASSES_ROOT\x22,\x22H\
KEY_CURRENT_CONF\
IG\x22,\x22HKEY_CURREN\
T_USER\x22,\x22HKEY_DY\
N_DATA\x22,\x22HKEY_LO\
CAL_MACHINE\x22,\x22HK\
EY_PERFORMANCE_D\
ATA\x22,\x22HKEY_USERS\
\x22,\x22HKLM\x22,\x22HKPD\x22,\
\x22HKU\x22,\x22IDABORT\x22,\
\x22IDCANCEL\x22,\x22IDIG\
NORE\x22,\x22IDNO\x22,\x22ID\
OK\x22,\x22IDRETRY\x22,\x22I\
DYES\x22,\x22MB_ABORTR\
ETRYIGNORE\x22,\x22MB_\
DEFBUTTON1\x22,\x22MB_\
DEFBUTTON2\x22,\x22MB_\
DEFBUTTON3\x22,\x22MB_\
DEFBUTTON4\x22,\x22MB_\
ICONEXCLAMATION\x22\
,\x22MB_ICONINFORMA\
TION\x22,\x22MB_ICONQU\
ESTION\x22,\x22MB_ICON\
STOP\x22,\x22MB_OK\x22,\x22M\
B_OKCANCEL\x22,\x22MB_\
RETRYCANCEL\x22,\x22MB\
_RIGHT\x22,\x22MB_RTLR\
EADING\x22,\x22MB_SETF\
OREGROUND\x22,\x22MB_T\
OPMOST\x22,\x22MB_USER\
ICON\x22,\x22MB_YESNO\x22\
,\x22NORMAL\x22,\x22OFFLI\
NE\x22,\x22READONLY\x22,\x22\
SHCTX\x22,\x22SHELL_CO\
NTEXT\x22,\x22SYSTEM|T\
EMPORARY\x22)\x0a},s={\
className:\x22keywo\
rd\x22,\x0abegin:t.con\
cat(/!/,t.either\
(\x22addincludedir\x22\
,\x22addplugindir\x22,\
\x22appendfile\x22,\x22as\
sert\x22,\x22cd\x22,\x22defi\
ne\x22,\x22delfile\x22,\x22e\
cho\x22,\x22else\x22,\x22end\
if\x22,\x22error\x22,\x22exe\
cute\x22,\x22finalize\x22\
,\x22getdllversion\x22\
,\x22gettlbversion\x22\
,\x22if\x22,\x22ifdef\x22,\x22i\
fmacrodef\x22,\x22ifma\
crondef\x22,\x22ifndef\
\x22,\x22include\x22,\x22ins\
ertmacro\x22,\x22macro\
\x22,\x22macroend\x22,\x22ma\
kensis\x22,\x22packhdr\
\x22,\x22searchparse\x22,\
\x22searchreplace\x22,\
\x22system\x22,\x22tempfi\
le\x22,\x22undef\x22,\x22uni\
nstfinalize\x22,\x22ve\
rbose\x22,\x22warning\x22\
))\x0a},o={classNam\
e:\x22string\x22,varia\
nts:[{begin:'\x22',\
end:'\x22'},{begin:\
\x22'\x22,end:\x22'\x22},{\x0ab\
egin:\x22`\x22,end:\x22`\x22\
}],illegal:/\x5cn/,\
contains:[{class\
Name:\x22char.escap\
e\x22,\x0abegin:/\x5c$(\x5c\x5c\
[nrt]|\x5c$)/},n,i,\
r,a]},S={\x0amatch:\
[/Function/,/\x5cs+\
/,t.concat(/(\x5c.)\
?/,e.IDENT_RE)],\
scope:{1:\x22keywor\
d\x22,\x0a3:\x22title.fun\
ction\x22}};return{\
name:\x22NSIS\x22,case\
_insensitive:!0,\
keywords:{\x0akeywo\
rd:[\x22Abort\x22,\x22Add\
BrandingImage\x22,\x22\
AddSize\x22,\x22AllowR\
ootDirInstall\x22,\x22\
AllowSkipFiles\x22,\
\x22AutoCloseWindow\
\x22,\x22BGFont\x22,\x22BGGr\
adient\x22,\x22Brandin\
gText\x22,\x22BringToF\
ront\x22,\x22Call\x22,\x22Ca\
llInstDLL\x22,\x22Capt\
ion\x22,\x22ChangeUI\x22,\
\x22CheckBitmap\x22,\x22C\
learErrors\x22,\x22Com\
pletedText\x22,\x22Com\
ponentText\x22,\x22Cop\
yFiles\x22,\x22CRCChec\
k\x22,\x22CreateDirect\
ory\x22,\x22CreateFont\
\x22,\x22CreateShortCu\
t\x22,\x22Delete\x22,\x22Del\
eteINISec\x22,\x22Dele\
teINIStr\x22,\x22Delet\
eRegKey\x22,\x22Delete\
RegValue\x22,\x22Detai\
lPrint\x22,\x22Details\
ButtonText\x22,\x22Dir\
Text\x22,\x22DirVar\x22,\x22\
DirVerify\x22,\x22Enab\
leWindow\x22,\x22EnumR\
egKey\x22,\x22EnumRegV\
alue\x22,\x22Exch\x22,\x22Ex\
ec\x22,\x22ExecShell\x22,\
\x22ExecShellWait\x22,\
\x22ExecWait\x22,\x22Expa\
ndEnvStrings\x22,\x22F\
ile\x22,\x22FileBufSiz\
e\x22,\x22FileClose\x22,\x22\
FileErrorText\x22,\x22\
FileOpen\x22,\x22FileR\
ead\x22,\x22FileReadBy\
te\x22,\x22FileReadUTF\
16LE\x22,\x22FileReadW\
ord\x22,\x22FileWriteU\
TF16LE\x22,\x22FileSee\
k\x22,\x22FileWrite\x22,\x22\
FileWriteByte\x22,\x22\
FileWriteWord\x22,\x22\
FindClose\x22,\x22Find\
First\x22,\x22FindNext\
\x22,\x22FindWindow\x22,\x22\
FlushINI\x22,\x22GetCu\
rInstType\x22,\x22GetC\
urrentAddress\x22,\x22\
GetDlgItem\x22,\x22Get\
DLLVersion\x22,\x22Get\
DLLVersionLocal\x22\
,\x22GetErrorLevel\x22\
,\x22GetFileTime\x22,\x22\
GetFileTimeLocal\
\x22,\x22GetFullPathNa\
me\x22,\x22GetFunction\
Address\x22,\x22GetIns\
tDirError\x22,\x22GetK\
nownFolderPath\x22,\
\x22GetLabelAddress\
\x22,\x22GetTempFileNa\
me\x22,\x22GetWinVer\x22,\
\x22Goto\x22,\x22HideWind\
ow\x22,\x22Icon\x22,\x22IfAb\
ort\x22,\x22IfErrors\x22,\
\x22IfFileExists\x22,\x22\
IfRebootFlag\x22,\x22I\
fRtlLanguage\x22,\x22I\
fShellVarContext\
All\x22,\x22IfSilent\x22,\
\x22InitPluginsDir\x22\
,\x22InstallButtonT\
ext\x22,\x22InstallCol\
ors\x22,\x22InstallDir\
\x22,\x22InstallDirReg\
Key\x22,\x22InstProgre\
ssFlags\x22,\x22InstTy\
pe\x22,\x22InstTypeGet\
Text\x22,\x22InstTypeS\
etText\x22,\x22Int64Cm\
p\x22,\x22Int64CmpU\x22,\x22\
Int64Fmt\x22,\x22IntCm\
p\x22,\x22IntCmpU\x22,\x22In\
tFmt\x22,\x22IntOp\x22,\x22I\
ntPtrCmp\x22,\x22IntPt\
rCmpU\x22,\x22IntPtrOp\
\x22,\x22IsWindow\x22,\x22La\
ngString\x22,\x22Licen\
seBkColor\x22,\x22Lice\
nseData\x22,\x22Licens\
eForceSelection\x22\
,\x22LicenseLangStr\
ing\x22,\x22LicenseTex\
t\x22,\x22LoadAndSetIm\
age\x22,\x22LoadLangua\
geFile\x22,\x22LockWin\
dow\x22,\x22LogSet\x22,\x22L\
ogText\x22,\x22Manifes\
tDPIAware\x22,\x22Mani\
festLongPathAwar\
e\x22,\x22ManifestMaxV\
ersionTested\x22,\x22M\
anifestSupported\
OS\x22,\x22MessageBox\x22\
,\x22MiscButtonText\
\x22,\x22Name|0\x22,\x22Nop\x22\
,\x22OutFile\x22,\x22Page\
\x22,\x22PageCallbacks\
\x22,\x22PEAddResource\
\x22,\x22PEDllCharacte\
ristics\x22,\x22PERemo\
veResource\x22,\x22PES\
ubsysVer\x22,\x22Pop\x22,\
\x22Push\x22,\x22Quit\x22,\x22R\
eadEnvStr\x22,\x22Read\
INIStr\x22,\x22ReadReg\
DWORD\x22,\x22ReadRegS\
tr\x22,\x22Reboot\x22,\x22Re\
gDLL\x22,\x22Rename\x22,\x22\
RequestExecution\
Level\x22,\x22ReserveF\
ile\x22,\x22Return\x22,\x22R\
MDir\x22,\x22SearchPat\
h\x22,\x22SectionGetFl\
ags\x22,\x22SectionGet\
InstTypes\x22,\x22Sect\
ionGetSize\x22,\x22Sec\
tionGetText\x22,\x22Se\
ctionIn\x22,\x22Sectio\
nSetFlags\x22,\x22Sect\
ionSetInstTypes\x22\
,\x22SectionSetSize\
\x22,\x22SectionSetTex\
t\x22,\x22SendMessage\x22\
,\x22SetAutoClose\x22,\
\x22SetBrandingImag\
e\x22,\x22SetCompress\x22\
,\x22SetCompressor\x22\
,\x22SetCompressorD\
ictSize\x22,\x22SetCtl\
Colors\x22,\x22SetCurI\
nstType\x22,\x22SetDat\
ablockOptimize\x22,\
\x22SetDateSave\x22,\x22S\
etDetailsPrint\x22,\
\x22SetDetailsView\x22\
,\x22SetErrorLevel\x22\
,\x22SetErrors\x22,\x22Se\
tFileAttributes\x22\
,\x22SetFont\x22,\x22SetO\
utPath\x22,\x22SetOver\
write\x22,\x22SetReboo\
tFlag\x22,\x22SetRegVi\
ew\x22,\x22SetShellVar\
Context\x22,\x22SetSil\
ent\x22,\x22ShowInstDe\
tails\x22,\x22ShowUnin\
stDetails\x22,\x22Show\
Window\x22,\x22SilentI\
nstall\x22,\x22SilentU\
nInstall\x22,\x22Sleep\
\x22,\x22SpaceTexts\x22,\x22\
StrCmp\x22,\x22StrCmpS\
\x22,\x22StrCpy\x22,\x22StrL\
en\x22,\x22SubCaption\x22\
,\x22Unicode\x22,\x22Unin\
stallButtonText\x22\
,\x22UninstallCapti\
on\x22,\x22UninstallIc\
on\x22,\x22UninstallSu\
bCaption\x22,\x22Unins\
tallText\x22,\x22Unins\
tPage\x22,\x22UnRegDLL\
\x22,\x22Var\x22,\x22VIAddVe\
rsionKey\x22,\x22VIFil\
eVersion\x22,\x22VIPro\
ductVersion\x22,\x22Wi\
ndowIcon\x22,\x22Write\
INIStr\x22,\x22WriteRe\
gBin\x22,\x22WriteRegD\
WORD\x22,\x22WriteRegE\
xpandStr\x22,\x22Write\
RegMultiStr\x22,\x22Wr\
iteRegNone\x22,\x22Wri\
teRegStr\x22,\x22Write\
Uninstaller\x22,\x22XP\
Style\x22],\x0aliteral\
:[\x22admin\x22,\x22all\x22,\
\x22auto\x22,\x22both\x22,\x22b\
ottom\x22,\x22bzip2\x22,\x22\
colored\x22,\x22compon\
ents\x22,\x22current\x22,\
\x22custom\x22,\x22direct\
ory\x22,\x22false\x22,\x22fo\
rce\x22,\x22hide\x22,\x22hig\
hest\x22,\x22ifdiff\x22,\x22\
ifnewer\x22,\x22instfi\
les\x22,\x22lastused\x22,\
\x22leave\x22,\x22left\x22,\x22\
license\x22,\x22liston\
ly\x22,\x22lzma\x22,\x22neve\
rshow\x22,\x22none\x22,\x22n\
ormal\x22,\x22notset\x22,\
\x22off\x22,\x22on\x22,\x22open\
\x22,\x22print\x22,\x22right\
\x22,\x22show\x22,\x22silent\
\x22,\x22silentlog\x22,\x22s\
mooth\x22,\x22textonly\
\x22,\x22top\x22,\x22true\x22,\x22\
try\x22,\x22un.compone\
nts\x22,\x22un.custom\x22\
,\x22un.directory\x22,\
\x22un.instfiles\x22,\x22\
un.license\x22,\x22uni\
nstConfirm\x22,\x22use\
r\x22,\x22Win10\x22,\x22Win7\
\x22,\x22Win8\x22,\x22WinVis\
ta\x22,\x22zlib\x22]\x0a},co\
ntains:[e.HASH_C\
OMMENT_MODE,e.C_\
BLOCK_COMMENT_MO\
DE,e.COMMENT(\x22;\x22\
,\x22$\x22,{\x0arelevance\
:0}),{match:[/Va\
r/,/\x5cs+/,/(?:\x5c/G\
LOBAL\x5cs+)?/,/[A-\
Za-z][\x5cw.]*/],sc\
ope:{\x0a1:\x22keyword\
\x22,3:\x22params\x22,4:\x22\
variable\x22}},S,{\x0a\
beginKeywords:\x22F\
unction PageEx S\
ection SectionGr\
oup FunctionEnd \
SectionEnd\x22\x0a},o,\
s,i,r,a,l,{class\
Name:\x22title.func\
tion\x22,begin:/\x5cw+\
::\x5cw+/},e.NUMBER\
_MODE]}}})()\x0a;hl\
js.registerLangu\
age(\x22nsis\x22,e)})(\
);/*! `objective\
c` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>{const \
n=/[a-zA-Z@][a-z\
A-Z0-9_]*/,_={\x0a$\
pattern:n,keywor\
d:[\x22@interface\x22,\
\x22@class\x22,\x22@proto\
col\x22,\x22@implement\
ation\x22]}\x0a;return\
{name:\x22Objective\
-C\x22,\x0aaliases:[\x22m\
m\x22,\x22objc\x22,\x22obj-c\
\x22,\x22obj-c++\x22,\x22obj\
ective-c++\x22],key\
words:{\x0a\x22variabl\
e.language\x22:[\x22th\
is\x22,\x22super\x22],$pa\
ttern:n,\x0akeyword\
:[\x22while\x22,\x22expor\
t\x22,\x22sizeof\x22,\x22typ\
edef\x22,\x22const\x22,\x22s\
truct\x22,\x22for\x22,\x22un\
ion\x22,\x22volatile\x22,\
\x22static\x22,\x22mutabl\
e\x22,\x22if\x22,\x22do\x22,\x22re\
turn\x22,\x22goto\x22,\x22en\
um\x22,\x22else\x22,\x22brea\
k\x22,\x22extern\x22,\x22asm\
\x22,\x22case\x22,\x22defaul\
t\x22,\x22register\x22,\x22e\
xplicit\x22,\x22typena\
me\x22,\x22switch\x22,\x22co\
ntinue\x22,\x22inline\x22\
,\x22readonly\x22,\x22ass\
ign\x22,\x22readwrite\x22\
,\x22self\x22,\x22@synchr\
onized\x22,\x22id\x22,\x22ty\
peof\x22,\x22nonatomic\
\x22,\x22IBOutlet\x22,\x22IB\
Action\x22,\x22strong\x22\
,\x22weak\x22,\x22copy\x22,\x22\
in\x22,\x22out\x22,\x22inout\
\x22,\x22bycopy\x22,\x22byre\
f\x22,\x22oneway\x22,\x22__s\
trong\x22,\x22__weak\x22,\
\x22__block\x22,\x22__aut\
oreleasing\x22,\x22@pr\
ivate\x22,\x22@protect\
ed\x22,\x22@public\x22,\x22@\
try\x22,\x22@property\x22\
,\x22@end\x22,\x22@throw\x22\
,\x22@catch\x22,\x22@fina\
lly\x22,\x22@autorelea\
sepool\x22,\x22@synthe\
size\x22,\x22@dynamic\x22\
,\x22@selector\x22,\x22@o\
ptional\x22,\x22@requi\
red\x22,\x22@encode\x22,\x22\
@package\x22,\x22@impo\
rt\x22,\x22@defs\x22,\x22@co\
mpatibility_alia\
s\x22,\x22__bridge\x22,\x22_\
_bridge_transfer\
\x22,\x22__bridge_reta\
ined\x22,\x22__bridge_\
retain\x22,\x22__covar\
iant\x22,\x22__contrav\
ariant\x22,\x22__kindo\
f\x22,\x22_Nonnull\x22,\x22_\
Nullable\x22,\x22_Null\
_unspecified\x22,\x22_\
_FUNCTION__\x22,\x22__\
PRETTY_FUNCTION_\
_\x22,\x22__attribute_\
_\x22,\x22getter\x22,\x22set\
ter\x22,\x22retain\x22,\x22u\
nsafe_unretained\
\x22,\x22nonnull\x22,\x22nul\
lable\x22,\x22null_uns\
pecified\x22,\x22null_\
resettable\x22,\x22cla\
ss\x22,\x22instancetyp\
e\x22,\x22NS_DESIGNATE\
D_INITIALIZER\x22,\x22\
NS_UNAVAILABLE\x22,\
\x22NS_REQUIRES_SUP\
ER\x22,\x22NS_RETURNS_\
INNER_POINTER\x22,\x22\
NS_INLINE\x22,\x22NS_A\
VAILABLE\x22,\x22NS_DE\
PRECATED\x22,\x22NS_EN\
UM\x22,\x22NS_OPTIONS\x22\
,\x22NS_SWIFT_UNAVA\
ILABLE\x22,\x22NS_ASSU\
ME_NONNULL_BEGIN\
\x22,\x22NS_ASSUME_NON\
NULL_END\x22,\x22NS_RE\
FINED_FOR_SWIFT\x22\
,\x22NS_SWIFT_NAME\x22\
,\x22NS_SWIFT_NOTHR\
OW\x22,\x22NS_DURING\x22,\
\x22NS_HANDLER\x22,\x22NS\
_ENDHANDLER\x22,\x22NS\
_VALUERETURN\x22,\x22N\
S_VOIDRETURN\x22],\x0a\
literal:[\x22false\x22\
,\x22true\x22,\x22FALSE\x22,\
\x22TRUE\x22,\x22nil\x22,\x22YE\
S\x22,\x22NO\x22,\x22NULL\x22],\
\x0abuilt_in:[\x22disp\
atch_once_t\x22,\x22di\
spatch_queue_t\x22,\
\x22dispatch_sync\x22,\
\x22dispatch_async\x22\
,\x22dispatch_once\x22\
],\x0atype:[\x22int\x22,\x22\
float\x22,\x22char\x22,\x22u\
nsigned\x22,\x22signed\
\x22,\x22short\x22,\x22long\x22\
,\x22double\x22,\x22wchar\
_t\x22,\x22unichar\x22,\x22v\
oid\x22,\x22bool\x22,\x22BOO\
L\x22,\x22id|0\x22,\x22_Bool\
\x22]\x0a},illegal:\x22</\
\x22,contains:[{cla\
ssName:\x22built_in\
\x22,\x0abegin:\x22\x5c\x5cb(AV\
|CA|CF|CG|CI|CL|\
CM|CN|CT|MK|MP|M\
TK|MTL|NS|SCN|SK\
|UI|WK|XC)\x5c\x5cw+\x22\x0a\
},e.C_LINE_COMME\
NT_MODE,e.C_BLOC\
K_COMMENT_MODE,e\
.C_NUMBER_MODE,e\
.QUOTE_STRING_MO\
DE,e.APOS_STRING\
_MODE,{\x0aclassNam\
e:\x22string\x22,varia\
nts:[{begin:'@\x22'\
,end:'\x22',illegal\
:\x22\x5c\x5cn\x22,\x0acontains\
:[e.BACKSLASH_ES\
CAPE]}]},{classN\
ame:\x22meta\x22,begin\
:/#\x5cs*[a-z]+\x5cb/,\
end:/$/,\x0akeyword\
s:{\x0akeyword:\x22if \
else elif endif \
define undef war\
ning error line \
pragma ifdef ifn\
def include\x22\x0a},c\
ontains:[{begin:\
/\x5c\x5c\x5cn/,relevance\
:0},e.inherit(e.\
QUOTE_STRING_MOD\
E,{\x0aclassName:\x22s\
tring\x22}),{classN\
ame:\x22string\x22,beg\
in:/<.*?>/,end:/\
$/,illegal:\x22\x5c\x5cn\x22\
\x0a},e.C_LINE_COMM\
ENT_MODE,e.C_BLO\
CK_COMMENT_MODE]\
},{className:\x22cl\
ass\x22,\x0abegin:\x22(\x22+\
_.keyword.join(\x22\
|\x22)+\x22)\x5c\x5cb\x22,end:/\
(\x5c{|$)/,excludeE\
nd:!0,keywords:_\
,\x0acontains:[e.UN\
DERSCORE_TITLE_M\
ODE]},{begin:\x22\x5c\x5c\
.\x22+e.UNDERSCORE_\
IDENT_RE,\x0areleva\
nce:0}]}}})();hl\
js.registerLangu\
age(\x22objectivec\x22\
,e)})();/*! `oca\
ml` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>({name\
:\x22OCaml\x22,aliases\
:[\x22ml\x22],\x0akeyword\
s:{$pattern:\x22[a-\
z_]\x5c\x5cw*!?\x22,\x0akeyw\
ord:\x22and as asse\
rt asr begin cla\
ss constraint do\
 done downto els\
e end exception \
external for fun\
 function functo\
r if in include \
inherit! inherit\
 initializer lan\
d lazy let lor l\
sl lsr lxor matc\
h method!|10 met\
hod mod module m\
utable new objec\
t of open! open \
or private rec s\
ig struct then t\
o try type val! \
val virtual when\
 while with pars\
er value\x22,\x0abuilt\
_in:\x22array bool \
bytes char exn|5\
 float int int32\
 int64 list lazy\
_t|5 nativeint|5\
 string unit in_\
channel out_chan\
nel ref\x22,\x0alitera\
l:\x22true false\x22},\
illegal:/\x5c/\x5c/|>>\
/,contains:[{cla\
ssName:\x22literal\x22\
,\x0abegin:\x22\x5c\x5c[(\x5c\x5c|\
\x5c\x5c|)?\x5c\x5c]|\x5c\x5c(\x5c\x5c)\x22\
,relevance:0},e.\
COMMENT(\x22\x5c\x5c(\x5c\x5c*\x22\
,\x22\x5c\x5c*\x5c\x5c)\x22,{\x0acont\
ains:[\x22self\x22]}),\
{className:\x22symb\
ol\x22,begin:\x22'[A-Z\
a-z_](?!')[\x5c\x5cw']\
*\x22},{\x0aclassName:\
\x22type\x22,begin:\x22`[\
A-Z][\x5c\x5cw']*\x22},{c\
lassName:\x22type\x22,\
\x0abegin:\x22\x5c\x5cb[A-Z]\
[\x5c\x5cw']*\x22,relevan\
ce:0},{begin:\x22[a\
-z_]\x5c\x5cw*'[\x5c\x5cw']*\
\x22,relevance:0\x0a},\
e.inherit(e.APOS\
_STRING_MODE,{cl\
assName:\x22string\x22\
,relevance:0\x0a}),\
e.inherit(e.QUOT\
E_STRING_MODE,{i\
llegal:null}),{c\
lassName:\x22number\
\x22,\x0abegin:\x22\x5c\x5cb(0[\
xX][a-fA-F0-9_]+\
[Lln]?|0[oO][0-7\
_]+[Lln]?|0[bB][\
01_]+[Lln]?|[0-9\
][0-9_]*([Lln]|(\
\x5c\x5c.[0-9_]*)?([eE\
][-+]?[0-9_]+)?)\
?)\x22,\x0arelevance:0\
},{begin:/->/}]}\
)})();hljs.regis\
terLanguage(\x22oca\
ml\x22,e)})();/*! `\
openscad` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
{const n={classN\
ame:\x22keyword\x22,\x0ab\
egin:\x22\x5c\x5c$(f[asn]\
|t|vp[rtd]|child\
ren)\x22},r={classN\
ame:\x22number\x22,\x0abe\
gin:\x22\x5c\x5cb\x5c\x5cd+(\x5c\x5c.\
\x5c\x5cd+)?(e-?\x5c\x5cd+)?\
\x22,relevance:0\x0a},\
s=e.inherit(e.QU\
OTE_STRING_MODE,\
{illegal:null}),\
a={className:\x22fu\
nction\x22,\x0abeginKe\
ywords:\x22module f\
unction\x22,end:/=|\
\x5c{/,contains:[{c\
lassName:\x22params\
\x22,\x0abegin:\x22\x5c\x5c(\x22,e\
nd:\x22\x5c\x5c)\x22,contain\
s:[\x22self\x22,r,s,n,\
{className:\x22lite\
ral\x22,\x0abegin:\x22fal\
se|true|PI|undef\
\x22}]},e.UNDERSCOR\
E_TITLE_MODE]};r\
eturn{name:\x22Open\
SCAD\x22,\x0aaliases:[\
\x22scad\x22],keywords\
:{\x0akeyword:\x22func\
tion module incl\
ude use for inte\
rsection_for if \
else \x5c\x5c%\x22,\x0aliter\
al:\x22false true P\
I undef\x22,\x0abuilt_\
in:\x22circle squar\
e polygon text s\
phere cube cylin\
der polyhedron t\
ranslate rotate \
scale resize mir\
ror multmatrix c\
olor offset hull\
 minkowski union\
 difference inte\
rsection abs sig\
n sin cos tan ac\
os asin atan ata\
n2 floor round c\
eil ln log pow s\
qrt exp rands mi\
n max concat loo\
kup str chr sear\
ch version versi\
on_num norm cros\
s parent_module \
echo import impo\
rt_dxf dxf_linea\
r_extrude linear\
_extrude rotate_\
extrude surface \
projection rende\
r children dxf_c\
ross dxf_dim let\
 assign\x22\x0a},conta\
ins:[e.C_LINE_CO\
MMENT_MODE,e.C_B\
LOCK_COMMENT_MOD\
E,r,{className:\x22\
meta\x22,\x0akeywords:\
{keyword:\x22includ\
e use\x22},begin:\x22i\
nclude|use <\x22,en\
d:\x22>\x22},s,n,{\x0abeg\
in:\x22[*!#%]\x22,rele\
vance:0},a]}}})(\
);hljs.registerL\
anguage(\x22opensca\
d\x22,e)})();/*! `o\
xygene` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var e\
=(()=>{\x22use stri\
ct\x22;return e=>{c\
onst t={$pattern\
:/\x5c.?\x5cw+/,\x0akeywo\
rd:\x22abstract add\
 and array as as\
c aspect assembl\
y async begin br\
eak block by cas\
e class concat c\
onst copy constr\
uctor continue c\
reate default de\
legate desc dist\
inct div do down\
to dynamic each \
else empty end e\
nsure enum equal\
s event except e\
xit extension ex\
ternal false fin\
al finalize fina\
lizer finally fl\
ags for forward \
from function fu\
ture global grou\
p has if impleme\
ntation implemen\
ts implies in in\
dex inherited in\
line interface i\
nto invariants i\
s iterator join \
locked locking l\
oop matching met\
hod mod module n\
amespace nested \
new nil not noti\
fy nullable of o\
ld on operator o\
r order out over\
ride parallel pa\
rams partial pin\
ned private proc\
edure property p\
rotected public \
queryable raise \
read readonly re\
cord reintroduce\
 remove repeat r\
equire result re\
verse sealed sel\
ect self sequenc\
e set shl shr sk\
ip static step s\
oft take then to\
 true try tuple \
type union unit \
unsafe until use\
s using var virt\
ual raises volat\
ile where while \
with write xor y\
ield await mappe\
d deprecated std\
call cdecl pasca\
l register safec\
all overload lib\
rary platform re\
ference packed s\
trict published \
autoreleasepool \
selector strong \
weak unretained\x22\
\x0a},r=e.COMMENT(/\
\x5c{/,/\x5c}/,{releva\
nce:0}),n=e.COMM\
ENT(\x22\x5c\x5c(\x5c\x5c*\x22,\x22\x5c\x5c\
*\x5c\x5c)\x22,{\x0arelevanc\
e:10}),a={classN\
ame:\x22string\x22,beg\
in:\x22'\x22,end:\x22'\x22,c\
ontains:[{begin:\
\x22''\x22}]\x0a},i={clas\
sName:\x22string\x22,b\
egin:\x22(#\x5c\x5cd+)+\x22}\
,s={\x0abeginKeywor\
ds:\x22function con\
structor destruc\
tor procedure me\
thod\x22,end:\x22[:;]\x22\
,\x0akeywords:\x22func\
tion constructor\
|10 destructor|1\
0 procedure|10 m\
ethod|10\x22,\x0aconta\
ins:[e.inherit(e\
.TITLE_MODE,{sco\
pe:\x22title.functi\
on\x22}),{className\
:\x22params\x22,\x0abegin\
:\x22\x5c\x5c(\x22,end:\x22\x5c\x5c)\x22\
,keywords:t,cont\
ains:[a,i]},r,n]\
};return{name:\x22O\
xygene\x22,\x0acase_in\
sensitive:!0,key\
words:t,illegal:\
'(\x22|\x5c\x5c$[G-Zg-z]|\
\x5c\x5c/\x5c\x5c*|</|=>|->)\
',\x0acontains:[r,n\
,e.C_LINE_COMMEN\
T_MODE,a,i,e.NUM\
BER_MODE,s,{scop\
e:\x22punctuation\x22,\
\x0amatch:/;/,relev\
ance:0}]}}})();h\
ljs.registerLang\
uage(\x22oxygene\x22,e\
)})();/*! `parse\
r3` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>{const\
 a=e.COMMENT(/\x5c{\
/,/\x5c}/,{\x0acontain\
s:[\x22self\x22]});ret\
urn{name:\x22Parser\
3\x22,subLanguage:\x22\
xml\x22,relevance:0\
,\x0acontains:[e.CO\
MMENT(\x22^#\x22,\x22$\x22),\
e.COMMENT(/\x5c^rem\
\x5c{/,/\x5c}/,{releva\
nce:10,\x0acontains\
:[a]}),{classNam\
e:\x22meta\x22,begin:\x22\
^@(?:BASE|USE|CL\
ASS|OPTIONS)$\x22,\x0a\
relevance:10},{c\
lassName:\x22title\x22\
,\x0abegin:\x22@[\x5c\x5cw\x5c\x5c\
-]+\x5c\x5c[[\x5c\x5cw^;\x5c\x5c-]\
*\x5c\x5c](?:\x5c\x5c[[\x5c\x5cw^;\
\x5c\x5c-]*\x5c\x5c])?(?:.*)\
$\x22},{\x0aclassName:\
\x22variable\x22,begin\
:/\x5c$\x5c{?[\x5cw\x5c-.:]+\
\x5c}?/},{className\
:\x22keyword\x22,\x0abegi\
n:/\x5c^[\x5cw\x5c-.:]+/}\
,{className:\x22num\
ber\x22,begin:\x22\x5c\x5c^#\
[0-9a-fA-F]+\x22\x0a},\
e.C_NUMBER_MODE]\
}}})();hljs.regi\
sterLanguage(\x22pa\
rser3\x22,e)})();/*\
! `perl` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
e=(()=>{\x22use str\
ict\x22;return e=>{\
\x0aconst n=e.regex\
,t=/[dualxmsipng\
r]{0,12}/,s={$pa\
ttern:/[\x5cw.]+/,\x0a\
keyword:\x22abs acc\
ept alarm and at\
an2 bind binmode\
 bless break cal\
ler chdir chmod \
chomp chop chown\
 chr chroot clas\
s close closedir\
 connect continu\
e cos crypt dbmc\
lose dbmopen def\
ined delete die \
do dump each els\
e elsif endgrent\
 endhostent endn\
etent endprotoen\
t endpwent endse\
rvent eof eval e\
xec exists exit \
exp fcntl field \
fileno flock for\
 foreach fork fo\
rmat formline ge\
tc getgrent getg\
rgid getgrnam ge\
thostbyaddr geth\
ostbyname gethos\
tent getlogin ge\
tnetbyaddr getne\
tbyname getneten\
t getpeername ge\
tpgrp getpriorit\
y getprotobyname\
 getprotobynumbe\
r getprotoent ge\
tpwent getpwnam \
getpwuid getserv\
byname getservby\
port getservent \
getsockname gets\
ockopt given glo\
b gmtime goto gr\
ep gt hex if ind\
ex int ioctl joi\
n keys kill last\
 lc lcfirst leng\
th link listen l\
ocal localtime l\
og lstat lt ma m\
ap method mkdir \
msgctl msgget ms\
grcv msgsnd my n\
e next no not oc\
t open opendir o\
r ord our pack p\
ackage pipe pop \
pos print printf\
 prototype push \
q|0 qq quotemeta\
 qw qx rand read\
 readdir readlin\
e readlink readp\
ipe recv redo re\
f rename require\
 reset return re\
verse rewinddir \
rindex rmdir say\
 scalar seek see\
kdir select semc\
tl semget semop \
send setgrent se\
thostent setnete\
nt setpgrp setpr\
iority setprotoe\
nt setpwent sets\
ervent setsockop\
t shift shmctl s\
hmget shmread sh\
mwrite shutdown \
sin sleep socket\
 socketpair sort\
 splice split sp\
rintf sqrt srand\
 stat state stud\
y sub substr sym\
link syscall sys\
open sysread sys\
seek system sysw\
rite tell telldi\
r tie tied time \
times tr truncat\
e uc ucfirst uma\
sk undef unless \
unlink unpack un\
shift untie unti\
l use utime valu\
es vec wait wait\
pid wantarray wa\
rn when while wr\
ite x|0 xor y|0\x22\
\x0a},r={className:\
\x22subst\x22,begin:\x22[\
$@]\x5c\x5c{\x22,end:\x22\x5c\x5c}\
\x22,keywords:s},a=\
{begin:/->\x5c{/,\x0ae\
nd:/\x5c}/},i={scop\
e:\x22attr\x22,match:/\
\x5cs+:\x5cs*\x5cw+(\x5cs*\x5c(\
.*?\x5c))?/},c={sco\
pe:\x22variable\x22,\x0av\
ariants:[{begin:\
/\x5c$\x5cd/},{\x0abegin:\
n.concat(/[$%@](\
?!\x22)(\x5c^\x5cw\x5cb|#\x5cw+\
(::\x5cw+)*|\x5c{\x5cw+\x5c}\
|\x5cw+(::\x5cw*)*)/,\x22\
(?![A-Za-z])(?![\
@$%])\x22)\x0a},{begin\
:/[$%@](?!\x22)[^\x5cs\
\x5cw{=]|\x5c$=/,relev\
ance:0}],contain\
s:[i]},o={\x0aclass\
Name:\x22number\x22,va\
riants:[{match:/\
0?\x5c.[0-9][0-9_]+\
\x5cb/},{\x0amatch:/\x5cb\
v?(0|[1-9][0-9_]\
*(\x5c.[0-9_]+)?|[1\
-9][0-9_]*)\x5cb/},\
{\x0amatch:/\x5cb0[0-7\
][0-7_]*\x5cb/},{ma\
tch:/\x5cb0x[0-9a-f\
A-F][0-9a-fA-F_]\
*\x5cb/},{\x0amatch:/\x5c\
b0b[0-1][0-1_]*\x5c\
b/}],relevance:0\
\x0a},l=[e.BACKSLAS\
H_ESCAPE,r,c],g=\
[/!/,/\x5c//,/\x5c|/,/\
\x5c?/,/'/,/\x22/,/#/]\
,d=(e,s,r=\x22\x5c\x5c1\x22)\
=>{\x0aconst a=\x22\x5c\x5c1\
\x22===r?r:n.concat\
(r,s)\x0a;return n.\
concat(n.concat(\
\x22(?:\x22,e,\x22)\x22),s,/\
(?:\x5c\x5c.|[^\x5c\x5c\x5c/])*\
?/,a,/(?:\x5c\x5c.|[^\x5c\
\x5c\x5c/])*?/,r,t)\x0a},\
m=(e,s,r)=>n.con\
cat(n.concat(\x22(?\
:\x22,e,\x22)\x22),s,/(?:\
\x5c\x5c.|[^\x5c\x5c\x5c/])*?/,\
r,t),p=[c,e.HASH\
_COMMENT_MODE,e.\
COMMENT(/^=\x5cw/,/\
=cut/,{\x0aendsWith\
Parent:!0}),a,{c\
lassName:\x22string\
\x22,contains:l,var\
iants:[{\x0abegin:\x22\
q[qwxr]?\x5c\x5cs*\x5c\x5c(\x22\
,end:\x22\x5c\x5c)\x22,relev\
ance:5},{begin:\x22\
q[qwxr]?\x5c\x5cs*\x5c\x5c[\x22\
,\x0aend:\x22\x5c\x5c]\x22,rele\
vance:5},{begin:\
\x22q[qwxr]?\x5c\x5cs*\x5c\x5c{\
\x22,end:\x22\x5c\x5c}\x22,rele\
vance:5},{\x0abegin\
:\x22q[qwxr]?\x5c\x5cs*\x5c\x5c\
|\x22,end:\x22\x5c\x5c|\x22,rel\
evance:5},{begin\
:\x22q[qwxr]?\x5c\x5cs*<\x22\
,end:\x22>\x22,\x0areleva\
nce:5},{begin:\x22q\
w\x5c\x5cs+q\x22,end:\x22q\x22,\
relevance:5},{be\
gin:\x22'\x22,end:\x22'\x22,\
\x0acontains:[e.BAC\
KSLASH_ESCAPE]},\
{begin:'\x22',end:'\
\x22'},{begin:\x22`\x22,e\
nd:\x22`\x22,\x0acontains\
:[e.BACKSLASH_ES\
CAPE]},{begin:/\x5c\
{\x5cw+\x5c}/,relevanc\
e:0},{\x0abegin:\x22-?\
\x5c\x5cw+\x5c\x5cs*=>\x22,rele\
vance:0}]},o,{\x0ab\
egin:\x22(\x5c\x5c/\x5c\x5c/|\x22+\
e.RE_STARTERS_RE\
+\x22|\x5c\x5cb(split|ret\
urn|print|revers\
e|grep)\x5c\x5cb)\x5c\x5cs*\x22\
,\x0akeywords:\x22spli\
t return print r\
everse grep\x22,rel\
evance:0,\x0acontai\
ns:[e.HASH_COMME\
NT_MODE,{classNa\
me:\x22regexp\x22,vari\
ants:[{\x0abegin:d(\
\x22s|tr|y\x22,n.eithe\
r(...g,{capture:\
!0}))},{begin:d(\
\x22s|tr|y\x22,\x22\x5c\x5c(\x22,\x22\
\x5c\x5c)\x22)},{\x0abegin:d\
(\x22s|tr|y\x22,\x22\x5c\x5c[\x22,\
\x22\x5c\x5c]\x22)},{begin:d\
(\x22s|tr|y\x22,\x22\x5c\x5c{\x22,\
\x22\x5c\x5c}\x22)}],relevan\
ce:2},{\x0aclassNam\
e:\x22regexp\x22,varia\
nts:[{begin:/(m|\
qr)\x5c/\x5c//,relevan\
ce:0},{\x0abegin:m(\
\x22(?:m|qr)?\x22,/\x5c//\
,/\x5c//)},{begin:m\
(\x22m|qr\x22,n.either\
(...g,{capture:!\
0\x0a}),/\x5c1/)},{beg\
in:m(\x22m|qr\x22,/\x5c(/\
,/\x5c)/)},{begin:m\
(\x22m|qr\x22,/\x5c[/,/\x5c]\
/)},{\x0abegin:m(\x22m\
|qr\x22,/\x5c{/,/\x5c}/)}\
]}]},{className:\
\x22function\x22,begin\
Keywords:\x22sub me\
thod\x22,\x0aend:\x22(\x5c\x5cs\
*\x5c\x5c(.*?\x5c\x5c))?[;{]\
\x22,excludeEnd:!0,\
relevance:5,cont\
ains:[e.TITLE_MO\
DE,i]\x0a},{classNa\
me:\x22class\x22,begin\
Keywords:\x22class\x22\
,end:\x22[;{]\x22,excl\
udeEnd:!0,releva\
nce:5,\x0acontains:\
[e.TITLE_MODE,i,\
o]},{begin:\x22-\x5c\x5cw\
\x5c\x5cb\x22,relevance:0\
},{begin:\x22^__DAT\
A__$\x22,\x0aend:\x22^__E\
ND__$\x22,subLangua\
ge:\x22mojolicious\x22\
,contains:[{begi\
n:\x22^@@.*\x22,end:\x22$\
\x22,\x0aclassName:\x22co\
mment\x22}]}];retur\
n r.contains=p,a\
.contains=p,{nam\
e:\x22Perl\x22,\x0aaliase\
s:[\x22pl\x22,\x22pm\x22],ke\
ywords:s,contain\
s:p}}})();hljs.r\
egisterLanguage(\
\x22perl\x22,e)\x0a})();/\
*! `pf` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var t\
=(()=>{\x22use stri\
ct\x22;return t=>({\
name:\x22Packet Fil\
ter config\x22,\x0aali\
ases:[\x22pf.conf\x22]\
,keywords:{$patt\
ern:/[a-z0-9_<>-\
]+/,\x0abuilt_in:\x22b\
lock match pass \
load anchor|5 an\
tispoof|10 set t\
able\x22,\x0akeyword:\x22\
in out log quick\
 on rdomain inet\
 inet6 proto fro\
m port os to rou\
te allow-opts di\
vert-packet dive\
rt-reply divert-\
to flags group i\
cmp-type icmp6-t\
ype label once p\
robability recie\
ved-on rtable pr\
io queue tos tag\
 tagged user kee\
p fragment for o\
s drop af-to|10 \
binat-to|10 nat-\
to|10 rdr-to|10 \
bitmask least-st\
ats random round\
-robin source-ha\
sh static-port d\
up-to reply-to r\
oute-to parent b\
andwidth default\
 min max qlimit \
block-policy deb\
ug fingerprints \
hostid limit log\
interface optimi\
zation reassembl\
e ruleset-optimi\
zation basic non\
e profile skip s\
tate-defaults st\
ate-policy timeo\
ut const counter\
s persist no mod\
ulate synproxy s\
tate|5 floating \
if-bound no-sync\
 pflow|10 sloppy\
 source-track gl\
obal rule max-sr\
c-nodes max-src-\
states max-src-c\
onn max-src-conn\
-rate overload f\
lush scrub|5 max\
-mss min-ttl no-\
df|10 random-id\x22\
,\x0aliteral:\x22all a\
ny no-route self\
 urpf-failed egr\
ess|5 unknown\x22},\
\x0acontains:[t.HAS\
H_COMMENT_MODE,t\
.NUMBER_MODE,t.Q\
UOTE_STRING_MODE\
,{\x0aclassName:\x22va\
riable\x22,begin:/\x5c\
$[\x5cw\x5cd#@][\x5cw\x5cd_]\
*/,relevance:0},\
{\x0aclassName:\x22var\
iable\x22,begin:/<(\
?!\x5c/)/,end:/>/}]\
})})()\x0a;hljs.reg\
isterLanguage(\x22p\
f\x22,t)})();/*! `p\
gsql` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var E=(\
()=>{\x22use strict\
\x22;return E=>{\x0aco\
nst T=E.COMMENT(\
\x22--\x22,\x22$\x22),N=\x22\x5c\x5c$\
([a-zA-Z_]?|[a-z\
A-Z_][a-zA-Z_0-9\
]*)\x5c\x5c$\x22,A=\x22BIGIN\
T INT8 BIGSERIAL\
 SERIAL8 BIT VAR\
YING VARBIT BOOL\
EAN BOOL BOX BYT\
EA CHARACTER CHA\
R VARCHAR CIDR C\
IRCLE DATE DOUBL\
E PRECISION FLOA\
T8 FLOAT INET IN\
TEGER INT INT4 I\
NTERVAL JSON JSO\
NB LINE LSEG|10 \
MACADDR MACADDR8\
 MONEY NUMERIC D\
EC DECIMAL PATH \
POINT POLYGON RE\
AL FLOAT4 SMALLI\
NT INT2 SMALLSER\
IAL|10 SERIAL2|1\
0 SERIAL|10 SERI\
AL4|10 TEXT TIME\
 ZONE TIMETZ|10 \
TIMESTAMP TIMEST\
AMPTZ|10 TSQUERY\
|10 TSVECTOR|10 \
TXID_SNAPSHOT|10\
 UUID XML NATION\
AL NCHAR INT4RAN\
GE|10 INT8RANGE|\
10 NUMRANGE|10 T\
SRANGE|10 TSTZRA\
NGE|10 DATERANGE\
|10 ANYELEMENT A\
NYARRAY ANYNONAR\
RAY ANYENUM ANYR\
ANGE CSTRING INT\
ERNAL RECORD PG_\
DDL_COMMAND VOID\
 UNKNOWN OPAQUE \
REFCURSOR NAME O\
ID REGPROC|10 RE\
GPROCEDURE|10 RE\
GOPER|10 REGOPER\
ATOR|10 REGCLASS\
|10 REGTYPE|10 R\
EGROLE|10 REGNAM\
ESPACE|10 REGCON\
FIG|10 REGDICTIO\
NARY|10 \x22,R=A.tr\
im().split(\x22 \x22).\
map((E=>E.split(\
\x22|\x22)[0])).join(\x22\
|\x22),I=\x22ARRAY_AGG\
 AVG BIT_AND BIT\
_OR BOOL_AND BOO\
L_OR COUNT EVERY\
 JSON_AGG JSONB_\
AGG JSON_OBJECT_\
AGG JSONB_OBJECT\
_AGG MAX MIN MOD\
E STRING_AGG SUM\
 XMLAGG CORR COV\
AR_POP COVAR_SAM\
P REGR_AVGX REGR\
_AVGY REGR_COUNT\
 REGR_INTERCEPT \
REGR_R2 REGR_SLO\
PE REGR_SXX REGR\
_SXY REGR_SYY ST\
DDEV STDDEV_POP \
STDDEV_SAMP VARI\
ANCE VAR_POP VAR\
_SAMP PERCENTILE\
_CONT PERCENTILE\
_DISC ROW_NUMBER\
 RANK DENSE_RANK\
 PERCENT_RANK CU\
ME_DIST NTILE LA\
G LEAD FIRST_VAL\
UE LAST_VALUE NT\
H_VALUE NUM_NONN\
ULLS NUM_NULLS A\
BS CBRT CEIL CEI\
LING DEGREES DIV\
 EXP FLOOR LN LO\
G MOD PI POWER R\
ADIANS ROUND SCA\
LE SIGN SQRT TRU\
NC WIDTH_BUCKET \
RANDOM SETSEED A\
COS ACOSD ASIN A\
SIND ATAN ATAND \
ATAN2 ATAN2D COS\
 COSD COT COTD S\
IN SIND TAN TAND\
 BIT_LENGTH CHAR\
_LENGTH CHARACTE\
R_LENGTH LOWER O\
CTET_LENGTH OVER\
LAY POSITION SUB\
STRING TREAT TRI\
M UPPER ASCII BT\
RIM CHR CONCAT C\
ONCAT_WS CONVERT\
 CONVERT_FROM CO\
NVERT_TO DECODE \
ENCODE INITCAP L\
EFT LENGTH LPAD \
LTRIM MD5 PARSE_\
IDENT PG_CLIENT_\
ENCODING QUOTE_I\
DENT|10 QUOTE_LI\
TERAL|10 QUOTE_N\
ULLABLE|10 REGEX\
P_MATCH REGEXP_M\
ATCHES REGEXP_RE\
PLACE REGEXP_SPL\
IT_TO_ARRAY REGE\
XP_SPLIT_TO_TABL\
E REPEAT REPLACE\
 REVERSE RIGHT R\
PAD RTRIM SPLIT_\
PART STRPOS SUBS\
TR TO_ASCII TO_H\
EX TRANSLATE OCT\
ET_LENGTH GET_BI\
T GET_BYTE SET_B\
IT SET_BYTE TO_C\
HAR TO_DATE TO_N\
UMBER TO_TIMESTA\
MP AGE CLOCK_TIM\
ESTAMP|10 DATE_P\
ART DATE_TRUNC I\
SFINITE JUSTIFY_\
DAYS JUSTIFY_HOU\
RS JUSTIFY_INTER\
VAL MAKE_DATE MA\
KE_INTERVAL|10 M\
AKE_TIME MAKE_TI\
MESTAMP|10 MAKE_\
TIMESTAMPTZ|10 N\
OW STATEMENT_TIM\
ESTAMP|10 TIMEOF\
DAY TRANSACTION_\
TIMESTAMP|10 ENU\
M_FIRST ENUM_LAS\
T ENUM_RANGE ARE\
A CENTER DIAMETE\
R HEIGHT ISCLOSE\
D ISOPEN NPOINTS\
 PCLOSE POPEN RA\
DIUS WIDTH BOX B\
OUND_BOX CIRCLE \
LINE LSEG PATH P\
OLYGON ABBREV BR\
OADCAST HOST HOS\
TMASK MASKLEN NE\
TMASK NETWORK SE\
T_MASKLEN TEXT I\
NET_SAME_FAMILY \
INET_MERGE MACAD\
DR8_SET7BIT ARRA\
Y_TO_TSVECTOR GE\
T_CURRENT_TS_CON\
FIG NUMNODE PLAI\
NTO_TSQUERY PHRA\
SETO_TSQUERY WEB\
SEARCH_TO_TSQUER\
Y QUERYTREE SETW\
EIGHT STRIP TO_T\
SQUERY TO_TSVECT\
OR JSON_TO_TSVEC\
TOR JSONB_TO_TSV\
ECTOR TS_DELETE \
TS_FILTER TS_HEA\
DLINE TS_RANK TS\
_RANK_CD TS_REWR\
ITE TSQUERY_PHRA\
SE TSVECTOR_TO_A\
RRAY TSVECTOR_UP\
DATE_TRIGGER TSV\
ECTOR_UPDATE_TRI\
GGER_COLUMN XMLC\
OMMENT XMLCONCAT\
 XMLELEMENT XMLF\
OREST XMLPI XMLR\
OOT XMLEXISTS XM\
L_IS_WELL_FORMED\
 XML_IS_WELL_FOR\
MED_DOCUMENT XML\
_IS_WELL_FORMED_\
CONTENT XPATH XP\
ATH_EXISTS XMLTA\
BLE XMLNAMESPACE\
S TABLE_TO_XML T\
ABLE_TO_XMLSCHEM\
A TABLE_TO_XML_A\
ND_XMLSCHEMA QUE\
RY_TO_XML QUERY_\
TO_XMLSCHEMA QUE\
RY_TO_XML_AND_XM\
LSCHEMA CURSOR_T\
O_XML CURSOR_TO_\
XMLSCHEMA SCHEMA\
_TO_XML SCHEMA_T\
O_XMLSCHEMA SCHE\
MA_TO_XML_AND_XM\
LSCHEMA DATABASE\
_TO_XML DATABASE\
_TO_XMLSCHEMA DA\
TABASE_TO_XML_AN\
D_XMLSCHEMA XMLA\
TTRIBUTES TO_JSO\
N TO_JSONB ARRAY\
_TO_JSON ROW_TO_\
JSON JSON_BUILD_\
ARRAY JSONB_BUIL\
D_ARRAY JSON_BUI\
LD_OBJECT JSONB_\
BUILD_OBJECT JSO\
N_OBJECT JSONB_O\
BJECT JSON_ARRAY\
_LENGTH JSONB_AR\
RAY_LENGTH JSON_\
EACH JSONB_EACH \
JSON_EACH_TEXT J\
SONB_EACH_TEXT J\
SON_EXTRACT_PATH\
 JSONB_EXTRACT_P\
ATH JSON_OBJECT_\
KEYS JSONB_OBJEC\
T_KEYS JSON_POPU\
LATE_RECORD JSON\
B_POPULATE_RECOR\
D JSON_POPULATE_\
RECORDSET JSONB_\
POPULATE_RECORDS\
ET JSON_ARRAY_EL\
EMENTS JSONB_ARR\
AY_ELEMENTS JSON\
_ARRAY_ELEMENTS_\
TEXT JSONB_ARRAY\
_ELEMENTS_TEXT J\
SON_TYPEOF JSONB\
_TYPEOF JSON_TO_\
RECORD JSONB_TO_\
RECORD JSON_TO_R\
ECORDSET JSONB_T\
O_RECORDSET JSON\
_STRIP_NULLS JSO\
NB_STRIP_NULLS J\
SONB_SET JSONB_I\
NSERT JSONB_PRET\
TY CURRVAL LASTV\
AL NEXTVAL SETVA\
L COALESCE NULLI\
F GREATEST LEAST\
 ARRAY_APPEND AR\
RAY_CAT ARRAY_ND\
IMS ARRAY_DIMS A\
RRAY_FILL ARRAY_\
LENGTH ARRAY_LOW\
ER ARRAY_POSITIO\
N ARRAY_POSITION\
S ARRAY_PREPEND \
ARRAY_REMOVE ARR\
AY_REPLACE ARRAY\
_TO_STRING ARRAY\
_UPPER CARDINALI\
TY STRING_TO_ARR\
AY UNNEST ISEMPT\
Y LOWER_INC UPPE\
R_INC LOWER_INF \
UPPER_INF RANGE_\
MERGE GENERATE_S\
ERIES GENERATE_S\
UBSCRIPTS CURREN\
T_DATABASE CURRE\
NT_QUERY CURRENT\
_SCHEMA|10 CURRE\
NT_SCHEMAS|10 IN\
ET_CLIENT_ADDR I\
NET_CLIENT_PORT \
INET_SERVER_ADDR\
 INET_SERVER_POR\
T ROW_SECURITY_A\
CTIVE FORMAT_TYP\
E TO_REGCLASS TO\
_REGPROC TO_REGP\
ROCEDURE TO_REGO\
PER TO_REGOPERAT\
OR TO_REGTYPE TO\
_REGNAMESPACE TO\
_REGROLE COL_DES\
CRIPTION OBJ_DES\
CRIPTION SHOBJ_D\
ESCRIPTION TXID_\
CURRENT TXID_CUR\
RENT_IF_ASSIGNED\
 TXID_CURRENT_SN\
APSHOT TXID_SNAP\
SHOT_XIP TXID_SN\
APSHOT_XMAX TXID\
_SNAPSHOT_XMIN T\
XID_VISIBLE_IN_S\
NAPSHOT TXID_STA\
TUS CURRENT_SETT\
ING SET_CONFIG B\
RIN_SUMMARIZE_NE\
W_VALUES BRIN_SU\
MMARIZE_RANGE BR\
IN_DESUMMARIZE_R\
ANGE GIN_CLEAN_P\
ENDING_LIST SUPP\
RESS_REDUNDANT_U\
PDATES_TRIGGER L\
O_FROM_BYTEA LO_\
PUT LO_GET LO_CR\
EAT LO_CREATE LO\
_UNLINK LO_IMPOR\
T LO_EXPORT LORE\
AD LOWRITE GROUP\
ING CAST\x22.split(\
\x22 \x22).map((E=>E.s\
plit(\x22|\x22)[0])).j\
oin(\x22|\x22)\x0a;return\
{name:\x22PostgreSQ\
L\x22,aliases:[\x22pos\
tgres\x22,\x22postgres\
ql\x22],supersetOf:\
\x22sql\x22,\x0acase_inse\
nsitive:!0,keywo\
rds:{\x0akeyword:\x22A\
BORT ALTER ANALY\
ZE BEGIN CALL CH\
ECKPOINT|10 CLOS\
E CLUSTER COMMEN\
T COMMIT COPY CR\
EATE DEALLOCATE \
DECLARE DELETE D\
ISCARD DO DROP E\
ND EXECUTE EXPLA\
IN FETCH GRANT I\
MPORT INSERT LIS\
TEN LOAD LOCK MO\
VE NOTIFY PREPAR\
E REASSIGN|10 RE\
FRESH REINDEX RE\
LEASE RESET REVO\
KE ROLLBACK SAVE\
POINT SECURITY S\
ELECT SET SHOW S\
TART TRUNCATE UN\
LISTEN|10 UPDATE\
 VACUUM|10 VALUE\
S AGGREGATE COLL\
ATION CONVERSION\
|10 DATABASE DEF\
AULT PRIVILEGES \
DOMAIN TRIGGER E\
XTENSION FOREIGN\
 WRAPPER|10 TABL\
E FUNCTION GROUP\
 LANGUAGE LARGE \
OBJECT MATERIALI\
ZED VIEW OPERATO\
R CLASS FAMILY P\
OLICY PUBLICATIO\
N|10 ROLE RULE S\
CHEMA SEQUENCE S\
ERVER STATISTICS\
 SUBSCRIPTION SY\
STEM TABLESPACE \
CONFIGURATION DI\
CTIONARY PARSER \
TEMPLATE TYPE US\
ER MAPPING PREPA\
RED ACCESS METHO\
D CAST AS TRANSF\
ORM TRANSACTION \
OWNED TO INTO SE\
SSION AUTHORIZAT\
ION INDEX PROCED\
URE ASSERTION AL\
L ANALYSE AND AN\
Y ARRAY ASC ASYM\
METRIC|10 BOTH C\
ASE CHECK COLLAT\
E COLUMN CONCURR\
ENTLY|10 CONSTRA\
INT CROSS DEFERR\
ABLE RANGE DESC \
DISTINCT ELSE EX\
CEPT FOR FREEZE|\
10 FROM FULL HAV\
ING ILIKE IN INI\
TIALLY INNER INT\
ERSECT IS ISNULL\
 JOIN LATERAL LE\
ADING LIKE LIMIT\
 NATURAL NOT NOT\
NULL NULL OFFSET\
 ON ONLY OR ORDE\
R OUTER OVERLAPS\
 PLACING PRIMARY\
 REFERENCES RETU\
RNING SIMILAR SO\
ME SYMMETRIC TAB\
LESAMPLE THEN TR\
AILING UNION UNI\
QUE USING VARIAD\
IC|10 VERBOSE WH\
EN WHERE WINDOW \
WITH BY RETURNS \
INOUT OUT SETOF|\
10 IF STRICT CUR\
RENT CONTINUE OW\
NER LOCATION OVE\
R PARTITION WITH\
IN BETWEEN ESCAP\
E EXTERNAL INVOK\
ER DEFINER WORK \
RENAME VERSION C\
ONNECTION CONNEC\
T TABLES TEMP TE\
MPORARY FUNCTION\
S SEQUENCES TYPE\
S SCHEMAS OPTION\
 CASCADE RESTRIC\
T ADD ADMIN EXIS\
TS VALID VALIDAT\
E ENABLE DISABLE\
 REPLICA|10 ALWA\
YS PASSING COLUM\
NS PATH REF VALU\
E OVERRIDING IMM\
UTABLE STABLE VO\
LATILE BEFORE AF\
TER EACH ROW PRO\
CEDURAL ROUTINE \
NO HANDLER VALID\
ATOR OPTIONS STO\
RAGE OIDS|10 WIT\
HOUT INHERIT DEP\
ENDS CALLED INPU\
T LEAKPROOF|10 C\
OST ROWS NOWAIT \
SEARCH UNTIL ENC\
RYPTED|10 PASSWO\
RD CONFLICT|10 I\
NSTEAD INHERITS \
CHARACTERISTICS \
WRITE CURSOR ALS\
O STATEMENT SHAR\
E EXCLUSIVE INLI\
NE ISOLATION REP\
EATABLE READ COM\
MITTED SERIALIZA\
BLE UNCOMMITTED \
LOCAL GLOBAL SQL\
 PROCEDURES RECU\
RSIVE SNAPSHOT R\
OLLUP CUBE TRUST\
ED|10 INCLUDE FO\
LLOWING PRECEDIN\
G UNBOUNDED RANG\
E GROUPS UNENCRY\
PTED|10 SYSID FO\
RMAT DELIMITER H\
EADER QUOTE ENCO\
DING FILTER OFF \
FORCE_QUOTE FORC\
E_NOT_NULL FORCE\
_NULL COSTS BUFF\
ERS TIMING SUMMA\
RY DISABLE_PAGE_\
SKIPPING RESTART\
 CYCLE GENERATED\
 IDENTITY DEFERR\
ED IMMEDIATE LEV\
EL LOGGED UNLOGG\
ED OF NOTHING NO\
NE EXCLUDE ATTRI\
BUTE USAGE ROUTI\
NES TRUE FALSE N\
AN INFINITY ALIA\
S BEGIN CONSTANT\
 DECLARE END EXC\
EPTION RETURN PE\
RFORM|10 RAISE G\
ET DIAGNOSTICS S\
TACKED|10 FOREAC\
H LOOP ELSIF EXI\
T WHILE REVERSE \
SLICE DEBUG LOG \
INFO NOTICE WARN\
ING ASSERT OPEN \
SUPERUSER NOSUPE\
RUSER CREATEDB N\
OCREATEDB CREATE\
ROLE NOCREATEROL\
E INHERIT NOINHE\
RIT LOGIN NOLOGI\
N REPLICATION NO\
REPLICATION BYPA\
SSRLS NOBYPASSRL\
S \x22,\x0abuilt_in:\x22C\
URRENT_TIME CURR\
ENT_TIMESTAMP CU\
RRENT_USER CURRE\
NT_CATALOG|10 CU\
RRENT_DATE LOCAL\
TIME LOCALTIMEST\
AMP CURRENT_ROLE\
|10 CURRENT_SCHE\
MA|10 SESSION_US\
ER PUBLIC FOUND \
NEW OLD TG_NAME|\
10 TG_WHEN|10 TG\
_LEVEL|10 TG_OP|\
10 TG_RELID|10 T\
G_RELNAME|10 TG_\
TABLE_NAME|10 TG\
_TABLE_SCHEMA|10\
 TG_NARGS|10 TG_\
ARGV|10 TG_EVENT\
|10 TG_TAG|10 RO\
W_COUNT RESULT_O\
ID|10 PG_CONTEXT\
|10 RETURNED_SQL\
STATE COLUMN_NAM\
E CONSTRAINT_NAM\
E PG_DATATYPE_NA\
ME|10 MESSAGE_TE\
XT TABLE_NAME SC\
HEMA_NAME PG_EXC\
EPTION_DETAIL|10\
 PG_EXCEPTION_HI\
NT|10 PG_EXCEPTI\
ON_CONTEXT|10 SQ\
LSTATE SQLERRM|1\
0 SUCCESSFUL_COM\
PLETION WARNING \
DYNAMIC_RESULT_S\
ETS_RETURNED IMP\
LICIT_ZERO_BIT_P\
ADDING NULL_VALU\
E_ELIMINATED_IN_\
SET_FUNCTION PRI\
VILEGE_NOT_GRANT\
ED PRIVILEGE_NOT\
_REVOKED STRING_\
DATA_RIGHT_TRUNC\
ATION DEPRECATED\
_FEATURE NO_DATA\
 NO_ADDITIONAL_D\
YNAMIC_RESULT_SE\
TS_RETURNED SQL_\
STATEMENT_NOT_YE\
T_COMPLETE CONNE\
CTION_EXCEPTION \
CONNECTION_DOES_\
NOT_EXIST CONNEC\
TION_FAILURE SQL\
CLIENT_UNABLE_TO\
_ESTABLISH_SQLCO\
NNECTION SQLSERV\
ER_REJECTED_ESTA\
BLISHMENT_OF_SQL\
CONNECTION TRANS\
ACTION_RESOLUTIO\
N_UNKNOWN PROTOC\
OL_VIOLATION TRI\
GGERED_ACTION_EX\
CEPTION FEATURE_\
NOT_SUPPORTED IN\
VALID_TRANSACTIO\
N_INITIATION LOC\
ATOR_EXCEPTION I\
NVALID_LOCATOR_S\
PECIFICATION INV\
ALID_GRANTOR INV\
ALID_GRANT_OPERA\
TION INVALID_ROL\
E_SPECIFICATION \
DIAGNOSTICS_EXCE\
PTION STACKED_DI\
AGNOSTICS_ACCESS\
ED_WITHOUT_ACTIV\
E_HANDLER CASE_N\
OT_FOUND CARDINA\
LITY_VIOLATION D\
ATA_EXCEPTION AR\
RAY_SUBSCRIPT_ER\
ROR CHARACTER_NO\
T_IN_REPERTOIRE \
DATETIME_FIELD_O\
VERFLOW DIVISION\
_BY_ZERO ERROR_I\
N_ASSIGNMENT ESC\
APE_CHARACTER_CO\
NFLICT INDICATOR\
_OVERFLOW INTERV\
AL_FIELD_OVERFLO\
W INVALID_ARGUME\
NT_FOR_LOGARITHM\
 INVALID_ARGUMEN\
T_FOR_NTILE_FUNC\
TION INVALID_ARG\
UMENT_FOR_NTH_VA\
LUE_FUNCTION INV\
ALID_ARGUMENT_FO\
R_POWER_FUNCTION\
 INVALID_ARGUMEN\
T_FOR_WIDTH_BUCK\
ET_FUNCTION INVA\
LID_CHARACTER_VA\
LUE_FOR_CAST INV\
ALID_DATETIME_FO\
RMAT INVALID_ESC\
APE_CHARACTER IN\
VALID_ESCAPE_OCT\
ET INVALID_ESCAP\
E_SEQUENCE NONST\
ANDARD_USE_OF_ES\
CAPE_CHARACTER I\
NVALID_INDICATOR\
_PARAMETER_VALUE\
 INVALID_PARAMET\
ER_VALUE INVALID\
_REGULAR_EXPRESS\
ION INVALID_ROW_\
COUNT_IN_LIMIT_C\
LAUSE INVALID_RO\
W_COUNT_IN_RESUL\
T_OFFSET_CLAUSE \
INVALID_TABLESAM\
PLE_ARGUMENT INV\
ALID_TABLESAMPLE\
_REPEAT INVALID_\
TIME_ZONE_DISPLA\
CEMENT_VALUE INV\
ALID_USE_OF_ESCA\
PE_CHARACTER MOS\
T_SPECIFIC_TYPE_\
MISMATCH NULL_VA\
LUE_NOT_ALLOWED \
NULL_VALUE_NO_IN\
DICATOR_PARAMETE\
R NUMERIC_VALUE_\
OUT_OF_RANGE SEQ\
UENCE_GENERATOR_\
LIMIT_EXCEEDED S\
TRING_DATA_LENGT\
H_MISMATCH STRIN\
G_DATA_RIGHT_TRU\
NCATION SUBSTRIN\
G_ERROR TRIM_ERR\
OR UNTERMINATED_\
C_STRING ZERO_LE\
NGTH_CHARACTER_S\
TRING FLOATING_P\
OINT_EXCEPTION I\
NVALID_TEXT_REPR\
ESENTATION INVAL\
ID_BINARY_REPRES\
ENTATION BAD_COP\
Y_FILE_FORMAT UN\
TRANSLATABLE_CHA\
RACTER NOT_AN_XM\
L_DOCUMENT INVAL\
ID_XML_DOCUMENT \
INVALID_XML_CONT\
ENT INVALID_XML_\
COMMENT INVALID_\
XML_PROCESSING_I\
NSTRUCTION INTEG\
RITY_CONSTRAINT_\
VIOLATION RESTRI\
CT_VIOLATION NOT\
_NULL_VIOLATION \
FOREIGN_KEY_VIOL\
ATION UNIQUE_VIO\
LATION CHECK_VIO\
LATION EXCLUSION\
_VIOLATION INVAL\
ID_CURSOR_STATE \
INVALID_TRANSACT\
ION_STATE ACTIVE\
_SQL_TRANSACTION\
 BRANCH_TRANSACT\
ION_ALREADY_ACTI\
VE HELD_CURSOR_R\
EQUIRES_SAME_ISO\
LATION_LEVEL INA\
PPROPRIATE_ACCES\
S_MODE_FOR_BRANC\
H_TRANSACTION IN\
APPROPRIATE_ISOL\
ATION_LEVEL_FOR_\
BRANCH_TRANSACTI\
ON NO_ACTIVE_SQL\
_TRANSACTION_FOR\
_BRANCH_TRANSACT\
ION READ_ONLY_SQ\
L_TRANSACTION SC\
HEMA_AND_DATA_ST\
ATEMENT_MIXING_N\
OT_SUPPORTED NO_\
ACTIVE_SQL_TRANS\
ACTION IN_FAILED\
_SQL_TRANSACTION\
 IDLE_IN_TRANSAC\
TION_SESSION_TIM\
EOUT INVALID_SQL\
_STATEMENT_NAME \
TRIGGERED_DATA_C\
HANGE_VIOLATION \
INVALID_AUTHORIZ\
ATION_SPECIFICAT\
ION INVALID_PASS\
WORD DEPENDENT_P\
RIVILEGE_DESCRIP\
TORS_STILL_EXIST\
 DEPENDENT_OBJEC\
TS_STILL_EXIST I\
NVALID_TRANSACTI\
ON_TERMINATION S\
QL_ROUTINE_EXCEP\
TION FUNCTION_EX\
ECUTED_NO_RETURN\
_STATEMENT MODIF\
YING_SQL_DATA_NO\
T_PERMITTED PROH\
IBITED_SQL_STATE\
MENT_ATTEMPTED R\
EADING_SQL_DATA_\
NOT_PERMITTED IN\
VALID_CURSOR_NAM\
E EXTERNAL_ROUTI\
NE_EXCEPTION CON\
TAINING_SQL_NOT_\
PERMITTED MODIFY\
ING_SQL_DATA_NOT\
_PERMITTED PROHI\
BITED_SQL_STATEM\
ENT_ATTEMPTED RE\
ADING_SQL_DATA_N\
OT_PERMITTED EXT\
ERNAL_ROUTINE_IN\
VOCATION_EXCEPTI\
ON INVALID_SQLST\
ATE_RETURNED NUL\
L_VALUE_NOT_ALLO\
WED TRIGGER_PROT\
OCOL_VIOLATED SR\
F_PROTOCOL_VIOLA\
TED EVENT_TRIGGE\
R_PROTOCOL_VIOLA\
TED SAVEPOINT_EX\
CEPTION INVALID_\
SAVEPOINT_SPECIF\
ICATION INVALID_\
CATALOG_NAME INV\
ALID_SCHEMA_NAME\
 TRANSACTION_ROL\
LBACK TRANSACTIO\
N_INTEGRITY_CONS\
TRAINT_VIOLATION\
 SERIALIZATION_F\
AILURE STATEMENT\
_COMPLETION_UNKN\
OWN DEADLOCK_DET\
ECTED SYNTAX_ERR\
OR_OR_ACCESS_RUL\
E_VIOLATION SYNT\
AX_ERROR INSUFFI\
CIENT_PRIVILEGE \
CANNOT_COERCE GR\
OUPING_ERROR WIN\
DOWING_ERROR INV\
ALID_RECURSION I\
NVALID_FOREIGN_K\
EY INVALID_NAME \
NAME_TOO_LONG RE\
SERVED_NAME DATA\
TYPE_MISMATCH IN\
DETERMINATE_DATA\
TYPE COLLATION_M\
ISMATCH INDETERM\
INATE_COLLATION \
WRONG_OBJECT_TYP\
E GENERATED_ALWA\
YS UNDEFINED_COL\
UMN UNDEFINED_FU\
NCTION UNDEFINED\
_TABLE UNDEFINED\
_PARAMETER UNDEF\
INED_OBJECT DUPL\
ICATE_COLUMN DUP\
LICATE_CURSOR DU\
PLICATE_DATABASE\
 DUPLICATE_FUNCT\
ION DUPLICATE_PR\
EPARED_STATEMENT\
 DUPLICATE_SCHEM\
A DUPLICATE_TABL\
E DUPLICATE_ALIA\
S DUPLICATE_OBJE\
CT AMBIGUOUS_COL\
UMN AMBIGUOUS_FU\
NCTION AMBIGUOUS\
_PARAMETER AMBIG\
UOUS_ALIAS INVAL\
ID_COLUMN_REFERE\
NCE INVALID_COLU\
MN_DEFINITION IN\
VALID_CURSOR_DEF\
INITION INVALID_\
DATABASE_DEFINIT\
ION INVALID_FUNC\
TION_DEFINITION \
INVALID_PREPARED\
_STATEMENT_DEFIN\
ITION INVALID_SC\
HEMA_DEFINITION \
INVALID_TABLE_DE\
FINITION INVALID\
_OBJECT_DEFINITI\
ON WITH_CHECK_OP\
TION_VIOLATION I\
NSUFFICIENT_RESO\
URCES DISK_FULL \
OUT_OF_MEMORY TO\
O_MANY_CONNECTIO\
NS CONFIGURATION\
_LIMIT_EXCEEDED \
PROGRAM_LIMIT_EX\
CEEDED STATEMENT\
_TOO_COMPLEX TOO\
_MANY_COLUMNS TO\
O_MANY_ARGUMENTS\
 OBJECT_NOT_IN_P\
REREQUISITE_STAT\
E OBJECT_IN_USE \
CANT_CHANGE_RUNT\
IME_PARAM LOCK_N\
OT_AVAILABLE OPE\
RATOR_INTERVENTI\
ON QUERY_CANCELE\
D ADMIN_SHUTDOWN\
 CRASH_SHUTDOWN \
CANNOT_CONNECT_N\
OW DATABASE_DROP\
PED SYSTEM_ERROR\
 IO_ERROR UNDEFI\
NED_FILE DUPLICA\
TE_FILE SNAPSHOT\
_TOO_OLD CONFIG_\
FILE_ERROR LOCK_\
FILE_EXISTS FDW_\
ERROR FDW_COLUMN\
_NAME_NOT_FOUND \
FDW_DYNAMIC_PARA\
METER_VALUE_NEED\
ED FDW_FUNCTION_\
SEQUENCE_ERROR F\
DW_INCONSISTENT_\
DESCRIPTOR_INFOR\
MATION FDW_INVAL\
ID_ATTRIBUTE_VAL\
UE FDW_INVALID_C\
OLUMN_NAME FDW_I\
NVALID_COLUMN_NU\
MBER FDW_INVALID\
_DATA_TYPE FDW_I\
NVALID_DATA_TYPE\
_DESCRIPTORS FDW\
_INVALID_DESCRIP\
TOR_FIELD_IDENTI\
FIER FDW_INVALID\
_HANDLE FDW_INVA\
LID_OPTION_INDEX\
 FDW_INVALID_OPT\
ION_NAME FDW_INV\
ALID_STRING_LENG\
TH_OR_BUFFER_LEN\
GTH FDW_INVALID_\
STRING_FORMAT FD\
W_INVALID_USE_OF\
_NULL_POINTER FD\
W_TOO_MANY_HANDL\
ES FDW_OUT_OF_ME\
MORY FDW_NO_SCHE\
MAS FDW_OPTION_N\
AME_NOT_FOUND FD\
W_REPLY_HANDLE F\
DW_SCHEMA_NOT_FO\
UND FDW_TABLE_NO\
T_FOUND FDW_UNAB\
LE_TO_CREATE_EXE\
CUTION FDW_UNABL\
E_TO_CREATE_REPL\
Y FDW_UNABLE_TO_\
ESTABLISH_CONNEC\
TION PLPGSQL_ERR\
OR RAISE_EXCEPTI\
ON NO_DATA_FOUND\
 TOO_MANY_ROWS A\
SSERT_FAILURE IN\
TERNAL_ERROR DAT\
A_CORRUPTED INDE\
X_CORRUPTED \x22\x0a},\
illegal:/:==|\x5cW\x5c\
s*\x5c(\x5c*|(^|\x5cs)\x5c$[\
a-z]|\x5c{\x5c{|[a-z]:\
\x5cs*$|\x5c.\x5c.\x5c.|TO:|\
DO:/,\x0acontains:[\
{className:\x22keyw\
ord\x22,variants:[{\
begin:/\x5cbTEXT\x5cs*\
SEARCH\x5cb/},{\x0abeg\
in:/\x5cb(PRIMARY|F\
OREIGN|FOR(\x5cs+NO\
)?)\x5cs+KEY\x5cb/},{\x0a\
begin:/\x5cbPARALLE\
L\x5cs+(UNSAFE|REST\
RICTED|SAFE)\x5cb/}\
,{\x0abegin:/\x5cbSTOR\
AGE\x5cs+(PLAIN|EXT\
ERNAL|EXTENDED|M\
AIN)\x5cb/},{\x0abegin\
:/\x5cbMATCH\x5cs+(FUL\
L|PARTIAL|SIMPLE\
)\x5cb/},{begin:/\x5cb\
NULLS\x5cs+(FIRST|L\
AST)\x5cb/},{\x0abegin\
:/\x5cbEVENT\x5cs+TRIG\
GER\x5cb/},{begin:/\
\x5cb(MAPPING|OR)\x5cs\
+REPLACE\x5cb/},{\x0ab\
egin:/\x5cb(FROM|TO\
)\x5cs+(PROGRAM|STD\
IN|STDOUT)\x5cb/},{\
\x0abegin:/\x5cb(SHARE\
|EXCLUSIVE)\x5cs+MO\
DE\x5cb/},{\x0abegin:/\
\x5cb(LEFT|RIGHT)\x5cs\
+(OUTER\x5cs+)?JOIN\
\x5cb/},{\x0abegin:/\x5cb\
(FETCH|MOVE)\x5cs+(\
NEXT|PRIOR|FIRST\
|LAST|ABSOLUTE|R\
ELATIVE|FORWARD|\
BACKWARD)\x5cb/\x0a},{\
begin:/\x5cbPRESERV\
E\x5cs+ROWS\x5cb/},{be\
gin:/\x5cbDISCARD\x5cs\
+PLANS\x5cb/},{\x0abeg\
in:/\x5cbREFERENCIN\
G\x5cs+(OLD|NEW)\x5cb/\
},{begin:/\x5cbSKIP\
\x5cs+LOCKED\x5cb/},{\x0a\
begin:/\x5cbGROUPIN\
G\x5cs+SETS\x5cb/},{\x0ab\
egin:/\x5cb(BINARY|\
INSENSITIVE|SCRO\
LL|NO\x5cs+SCROLL)\x5c\
s+(CURSOR|FOR)\x5cb\
/},{\x0abegin:/\x5cb(W\
ITH|WITHOUT)\x5cs+H\
OLD\x5cb/},{\x0abegin:\
/\x5cbWITH\x5cs+(CASCA\
DED|LOCAL)\x5cs+CHE\
CK\x5cs+OPTION\x5cb/},\
{\x0abegin:/\x5cbEXCLU\
DE\x5cs+(TIES|NO\x5cs+\
OTHERS)\x5cb/},{\x0abe\
gin:/\x5cbFORMAT\x5cs+\
(TEXT|XML|JSON|Y\
AML)\x5cb/},{\x0abegin\
:/\x5cbSET\x5cs+((SESS\
ION|LOCAL)\x5cs+)?N\
AMES\x5cb/},{begin:\
/\x5cbIS\x5cs+(NOT\x5cs+)\
?UNKNOWN\x5cb/\x0a},{b\
egin:/\x5cbSECURITY\
\x5cs+LABEL\x5cb/},{be\
gin:/\x5cbSTANDALON\
E\x5cs+(YES|NO|NO\x5cs\
+VALUE)\x5cb/\x0a},{be\
gin:/\x5cbWITH\x5cs+(N\
O\x5cs+)?DATA\x5cb/},{\
begin:/\x5cb(FOREIG\
N|SET)\x5cs+DATA\x5cb/\
},{\x0abegin:/\x5cbSET\
\x5cs+(CATALOG|CONS\
TRAINTS)\x5cb/},{be\
gin:/\x5cb(WITH|FOR\
)\x5cs+ORDINALITY\x5cb\
/},{\x0abegin:/\x5cbIS\
\x5cs+(NOT\x5cs+)?DOCU\
MENT\x5cb/},{\x0abegin\
:/\x5cbXML\x5cs+OPTION\
\x5cs+(DOCUMENT|CON\
TENT)\x5cb/},{\x0abegi\
n:/\x5cb(STRIP|PRES\
ERVE)\x5cs+WHITESPA\
CE\x5cb/},{\x0abegin:/\
\x5cbNO\x5cs+(ACTION|M\
AXVALUE|MINVALUE\
)\x5cb/},{\x0abegin:/\x5c\
bPARTITION\x5cs+BY\x5c\
s+(RANGE|LIST|HA\
SH)\x5cb/},{begin:/\
\x5cbAT\x5cs+TIME\x5cs+ZO\
NE\x5cb/},{\x0abegin:/\
\x5cbGRANTED\x5cs+BY\x5cb\
/},{begin:/\x5cbRET\
URN\x5cs+(QUERY|NEX\
T)\x5cb/},{\x0abegin:/\
\x5cb(ATTACH|DETACH\
)\x5cs+PARTITION\x5cb/\
},{\x0abegin:/\x5cbFOR\
CE\x5cs+ROW\x5cs+LEVEL\
\x5cs+SECURITY\x5cb/},\
{\x0abegin:/\x5cb(INCL\
UDING|EXCLUDING)\
\x5cs+(COMMENTS|CON\
STRAINTS|DEFAULT\
S|IDENTITY|INDEX\
ES|STATISTICS|ST\
ORAGE|ALL)\x5cb/\x0a},\
{begin:/\x5cbAS\x5cs+(\
ASSIGNMENT|IMPLI\
CIT|PERMISSIVE|R\
ESTRICTIVE|ENUM|\
RANGE)\x5cb/}]},{\x0ab\
egin:/\x5cb(FORMAT|\
FAMILY|VERSION)\x5c\
s*\x5c(/},{begin:/\x5c\
bINCLUDE\x5cs*\x5c(/,\x0a\
keywords:\x22INCLUD\
E\x22},{begin:/\x5cbRA\
NGE(?!\x5cs*(BETWEE\
N|UNBOUNDED|CURR\
ENT|[-0-9]+))/\x0a}\
,{\x0abegin:/\x5cb(VER\
SION|OWNER|TEMPL\
ATE|TABLESPACE|C\
ONNECTION\x5cs+LIMI\
T|PROCEDURE|REST\
RICT|JOIN|PARSER\
|COPY|START|END|\
COLLATION|INPUT|\
ANALYZE|STORAGE|\
LIKE|DEFAULT|DEL\
IMITER|ENCODING|\
COLUMN|CONSTRAIN\
T|TABLE|SCHEMA)\x5c\
s*=/\x0a},{begin:/\x5c\
b(PG_\x5cw+?|HAS_[A\
-Z_]+_PRIVILEGE)\
\x5cb/,relevance:10\
},{\x0abegin:/\x5cbEXT\
RACT\x5cs*\x5c(/,end:/\
\x5cbFROM\x5cb/,return\
End:!0,keywords:\
{\x0atype:\x22CENTURY \
DAY DECADE DOW D\
OY EPOCH HOUR IS\
ODOW ISOYEAR MIC\
ROSECONDS MILLEN\
NIUM MILLISECOND\
S MINUTE MONTH Q\
UARTER SECOND TI\
MEZONE TIMEZONE_\
HOUR TIMEZONE_MI\
NUTE WEEK YEAR\x22\x0a\
}},{begin:/\x5cb(XM\
LELEMENT|XMLPI)\x5c\
s*\x5c(\x5cs*NAME/,key\
words:{keyword:\x22\
NAME\x22}},{\x0abegin:\
/\x5cb(XMLPARSE|XML\
SERIALIZE)\x5cs*\x5c(\x5c\
s*(DOCUMENT|CONT\
ENT)/,keywords:{\
\x0akeyword:\x22DOCUME\
NT CONTENT\x22}},{b\
eginKeywords:\x22CA\
CHE INCREMENT MA\
XVALUE MINVALUE\x22\
,\x0aend:E.C_NUMBER\
_RE,returnEnd:!0\
,keywords:\x22BY CA\
CHE INCREMENT MA\
XVALUE MINVALUE\x22\
\x0a},{className:\x22t\
ype\x22,begin:/\x5cb(W\
ITH|WITHOUT)\x5cs+T\
IME\x5cs+ZONE\x5cb/},{\
className:\x22type\x22\
,\x0abegin:/\x5cbINTER\
VAL\x5cs+(YEAR|MONT\
H|DAY|HOUR|MINUT\
E|SECOND)(\x5cs+TO\x5c\
s+(MONTH|HOUR|MI\
NUTE|SECOND))?\x5cb\
/\x0a},{\x0abegin:/\x5cbR\
ETURNS\x5cs+(LANGUA\
GE_HANDLER|TRIGG\
ER|EVENT_TRIGGER\
|FDW_HANDLER|IND\
EX_AM_HANDLER|TS\
M_HANDLER)\x5cb/,\x0ak\
eywords:{keyword\
:\x22RETURNS\x22,\x0atype\
:\x22LANGUAGE_HANDL\
ER TRIGGER EVENT\
_TRIGGER FDW_HAN\
DLER INDEX_AM_HA\
NDLER TSM_HANDLE\
R\x22\x0a}},{begin:\x22\x5c\x5c\
b(\x22+I+\x22)\x5c\x5cs*\x5c\x5c(\x22\
},{begin:\x22\x5c\x5c.(\x22+\
R+\x22)\x5c\x5cb\x22},{\x0abegi\
n:\x22\x5c\x5cb(\x22+R+\x22)\x5c\x5cs\
+PATH\x5c\x5cb\x22,keywor\
ds:{keyword:\x22PAT\
H\x22,\x0atype:A.repla\
ce(\x22PATH \x22,\x22\x22)}}\
,{className:\x22typ\
e\x22,begin:\x22\x5c\x5cb(\x22+\
R+\x22)\x5c\x5cb\x22},{\x0aclas\
sName:\x22string\x22,b\
egin:\x22'\x22,end:\x22'\x22\
,contains:[{begi\
n:\x22''\x22}]},{\x0aclas\
sName:\x22string\x22,b\
egin:\x22(e|E|u&|U&\
)'\x22,end:\x22'\x22,cont\
ains:[{begin:\x22\x5c\x5c\
\x5c\x5c.\x22}],\x0arelevanc\
e:10},E.END_SAME\
_AS_BEGIN({begin\
:N,end:N,contain\
s:[{\x0asubLanguage\
:[\x22pgsql\x22,\x22perl\x22\
,\x22python\x22,\x22tcl\x22,\
\x22r\x22,\x22lua\x22,\x22java\x22\
,\x22php\x22,\x22ruby\x22,\x22b\
ash\x22,\x22scheme\x22,\x22x\
ml\x22,\x22json\x22],\x0aend\
sWithParent:!0}]\
}),{begin:'\x22',en\
d:'\x22',contains:[\
{begin:'\x22\x22'}]\x0a},\
E.C_NUMBER_MODE,\
E.C_BLOCK_COMMEN\
T_MODE,T,{classN\
ame:\x22meta\x22,varia\
nts:[{\x0abegin:\x22%(\
ROW)?TYPE\x22,relev\
ance:10},{begin:\
\x22\x5c\x5c$\x5c\x5cd+\x22},{begi\
n:\x22^#\x5c\x5cw\x22,end:\x22$\
\x22}]},{\x0aclassName\
:\x22symbol\x22,begin:\
\x22<<\x5c\x5cs*[a-zA-Z_]\
[a-zA-Z_0-9$]*\x5c\x5c\
s*>>\x22,relevance:\
10}]}}\x0a})();hljs\
.registerLanguag\
e(\x22pgsql\x22,E)})()\
;/*! `php` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{\x0aconst t=e.reg\
ex,a=/(?![A-Za-z\
0-9])(?![$])/,r=\
t.concat(/[a-zA-\
Z_\x5cx7f-\x5cxff][a-z\
A-Z0-9_\x5cx7f-\x5cxff\
]*/,a),n=t.conca\
t(/(\x5c\x5c?[A-Z][a-z\
0-9_\x5cx7f-\x5cxff]+|\
\x5c\x5c?[A-Z]+(?=[A-Z\
][a-z0-9_\x5cx7f-\x5cx\
ff])){1,}/,a),o=\
{\x0ascope:\x22variabl\
e\x22,match:\x22\x5c\x5c$+\x22+\
r},c={scope:\x22sub\
st\x22,variants:[{b\
egin:/\x5c$\x5cw+/},{\x0a\
begin:/\x5c{\x5c$/,end\
:/\x5c}/}]},i=e.inh\
erit(e.APOS_STRI\
NG_MODE,{illegal\
:null\x0a}),s=\x22[ \x5ct\
\x5cn]\x22,l={scope:\x22s\
tring\x22,variants:\
[e.inherit(e.QUO\
TE_STRING_MODE,{\
\x0aillegal:null,co\
ntains:e.QUOTE_S\
TRING_MODE.conta\
ins.concat(c)}),\
i,{\x0abegin:/<<<[ \
\x5ct]*(?:(\x5cw+)|\x22(\x5c\
w+)\x22)\x5cn/,end:/[ \
\x5ct]*(\x5cw+)\x5cb/,\x0aco\
ntains:e.QUOTE_S\
TRING_MODE.conta\
ins.concat(c),\x22o\
n:begin\x22:(e,t)=>\
{\x0at.data._beginM\
atch=e[1]||e[2]}\
,\x22on:end\x22:(e,t)=\
>{\x0at.data._begin\
Match!==e[1]&&t.\
ignoreMatch()}},\
e.END_SAME_AS_BE\
GIN({\x0abegin:/<<<\
[ \x5ct]*'(\x5cw+)'\x5cn/\
,end:/[ \x5ct]*(\x5cw+\
)\x5cb/})]},d={scop\
e:\x22number\x22,varia\
nts:[{\x0abegin:\x22\x5c\x5c\
b0[bB][01]+(?:_[\
01]+)*\x5c\x5cb\x22},{beg\
in:\x22\x5c\x5cb0[oO][0-7\
]+(?:_[0-7]+)*\x5c\x5c\
b\x22},{\x0abegin:\x22\x5c\x5cb\
0[xX][\x5c\x5cda-fA-F]\
+(?:_[\x5c\x5cda-fA-F]\
+)*\x5c\x5cb\x22},{\x0abegin\
:\x22(?:\x5c\x5cb\x5c\x5cd+(?:_\
\x5c\x5cd+)*(\x5c\x5c.(?:\x5c\x5cd\
+(?:_\x5c\x5cd+)*))?|\x5c\
\x5cB\x5c\x5c.\x5c\x5cd+)(?:[eE\
][+-]?\x5c\x5cd+)?\x22\x0a}]\
,relevance:0\x0a},_\
=[\x22false\x22,\x22null\x22\
,\x22true\x22],p=[\x22__C\
LASS__\x22,\x22__DIR__\
\x22,\x22__FILE__\x22,\x22__\
FUNCTION__\x22,\x22__C\
OMPILER_HALT_OFF\
SET__\x22,\x22__LINE__\
\x22,\x22__METHOD__\x22,\x22\
__NAMESPACE__\x22,\x22\
__TRAIT__\x22,\x22die\x22\
,\x22echo\x22,\x22exit\x22,\x22\
include\x22,\x22includ\
e_once\x22,\x22print\x22,\
\x22require\x22,\x22requi\
re_once\x22,\x22array\x22\
,\x22abstract\x22,\x22and\
\x22,\x22as\x22,\x22binary\x22,\
\x22bool\x22,\x22boolean\x22\
,\x22break\x22,\x22callab\
le\x22,\x22case\x22,\x22catc\
h\x22,\x22class\x22,\x22clon\
e\x22,\x22const\x22,\x22cont\
inue\x22,\x22declare\x22,\
\x22default\x22,\x22do\x22,\x22\
double\x22,\x22else\x22,\x22\
elseif\x22,\x22empty\x22,\
\x22enddeclare\x22,\x22en\
dfor\x22,\x22endforeac\
h\x22,\x22endif\x22,\x22ends\
witch\x22,\x22endwhile\
\x22,\x22enum\x22,\x22eval\x22,\
\x22extends\x22,\x22final\
\x22,\x22finally\x22,\x22flo\
at\x22,\x22for\x22,\x22forea\
ch\x22,\x22from\x22,\x22glob\
al\x22,\x22goto\x22,\x22if\x22,\
\x22implements\x22,\x22in\
stanceof\x22,\x22inste\
adof\x22,\x22int\x22,\x22int\
eger\x22,\x22interface\
\x22,\x22isset\x22,\x22itera\
ble\x22,\x22list\x22,\x22mat\
ch|0\x22,\x22mixed\x22,\x22n\
ew\x22,\x22never\x22,\x22obj\
ect\x22,\x22or\x22,\x22priva\
te\x22,\x22protected\x22,\
\x22public\x22,\x22readon\
ly\x22,\x22real\x22,\x22retu\
rn\x22,\x22string\x22,\x22sw\
itch\x22,\x22throw\x22,\x22t\
rait\x22,\x22try\x22,\x22uns\
et\x22,\x22use\x22,\x22var\x22,\
\x22void\x22,\x22while\x22,\x22\
xor\x22,\x22yield\x22],b=\
[\x22Error|0\x22,\x22Appe\
ndIterator\x22,\x22Arg\
umentCountError\x22\
,\x22ArithmeticErro\
r\x22,\x22ArrayIterato\
r\x22,\x22ArrayObject\x22\
,\x22AssertionError\
\x22,\x22BadFunctionCa\
llException\x22,\x22Ba\
dMethodCallExcep\
tion\x22,\x22CachingIt\
erator\x22,\x22Callbac\
kFilterIterator\x22\
,\x22CompileError\x22,\
\x22Countable\x22,\x22Dir\
ectoryIterator\x22,\
\x22DivisionByZeroE\
rror\x22,\x22DomainExc\
eption\x22,\x22EmptyIt\
erator\x22,\x22ErrorEx\
ception\x22,\x22Except\
ion\x22,\x22Filesystem\
Iterator\x22,\x22Filte\
rIterator\x22,\x22Glob\
Iterator\x22,\x22Infin\
iteIterator\x22,\x22In\
validArgumentExc\
eption\x22,\x22Iterato\
rIterator\x22,\x22Leng\
thException\x22,\x22Li\
mitIterator\x22,\x22Lo\
gicException\x22,\x22M\
ultipleIterator\x22\
,\x22NoRewindIterat\
or\x22,\x22OutOfBounds\
Exception\x22,\x22OutO\
fRangeException\x22\
,\x22OuterIterator\x22\
,\x22OverflowExcept\
ion\x22,\x22ParentIter\
ator\x22,\x22ParseErro\
r\x22,\x22RangeExcepti\
on\x22,\x22RecursiveAr\
rayIterator\x22,\x22Re\
cursiveCachingIt\
erator\x22,\x22Recursi\
veCallbackFilter\
Iterator\x22,\x22Recur\
siveDirectoryIte\
rator\x22,\x22Recursiv\
eFilterIterator\x22\
,\x22RecursiveItera\
tor\x22,\x22RecursiveI\
teratorIterator\x22\
,\x22RecursiveRegex\
Iterator\x22,\x22Recur\
siveTreeIterator\
\x22,\x22RegexIterator\
\x22,\x22RuntimeExcept\
ion\x22,\x22SeekableIt\
erator\x22,\x22SplDoub\
lyLinkedList\x22,\x22S\
plFileInfo\x22,\x22Spl\
FileObject\x22,\x22Spl\
FixedArray\x22,\x22Spl\
Heap\x22,\x22SplMaxHea\
p\x22,\x22SplMinHeap\x22,\
\x22SplObjectStorag\
e\x22,\x22SplObserver\x22\
,\x22SplPriorityQue\
ue\x22,\x22SplQueue\x22,\x22\
SplStack\x22,\x22SplSu\
bject\x22,\x22SplTempF\
ileObject\x22,\x22Type\
Error\x22,\x22Underflo\
wException\x22,\x22Une\
xpectedValueExce\
ption\x22,\x22Unhandle\
dMatchError\x22,\x22Ar\
rayAccess\x22,\x22Back\
edEnum\x22,\x22Closure\
\x22,\x22Fiber\x22,\x22Gener\
ator\x22,\x22Iterator\x22\
,\x22IteratorAggreg\
ate\x22,\x22Serializab\
le\x22,\x22Stringable\x22\
,\x22Throwable\x22,\x22Tr\
aversable\x22,\x22Unit\
Enum\x22,\x22WeakRefer\
ence\x22,\x22WeakMap\x22,\
\x22Directory\x22,\x22__P\
HP_Incomplete_Cl\
ass\x22,\x22parent\x22,\x22p\
hp_user_filter\x22,\
\x22self\x22,\x22static\x22,\
\x22stdClass\x22],E={\x0a\
keyword:p,litera\
l:(e=>{const t=[\
];return e.forEa\
ch((e=>{\x0at.push(\
e),e.toLowerCase\
()===e?t.push(e.\
toUpperCase()):t\
.push(e.toLowerC\
ase())\x0a})),t})(_\
),built_in:b},u=\
e=>e.map((e=>e.r\
eplace(/\x5c|\x5cd+$/,\
\x22\x22))),g={variant\
s:[{\x0amatch:[/new\
/,t.concat(s,\x22+\x22\
),t.concat(\x22(?!\x22\
,u(b).join(\x22\x5c\x5cb|\
\x22),\x22\x5c\x5cb)\x22),n],sc\
ope:{\x0a1:\x22keyword\
\x22,4:\x22title.class\
\x22}}]},h=t.concat\
(r,\x22\x5c\x5cb(?!\x5c\x5c()\x22)\
,m={variants:[{\x0a\
match:[t.concat(\
/::/,t.lookahead\
(/(?!class\x5cb)/))\
,h],scope:{2:\x22va\
riable.constant\x22\
\x0a}},{match:[/::/\
,/class/],scope:\
{2:\x22variable.lan\
guage\x22}},{\x0amatch\
:[n,t.concat(/::\
/,t.lookahead(/(\
?!class\x5cb)/)),h]\
,scope:{1:\x22title\
.class\x22,\x0a3:\x22vari\
able.constant\x22}}\
,{match:[n,t.con\
cat(\x22::\x22,t.looka\
head(/(?!class\x5cb\
)/))],\x0ascope:{1:\
\x22title.class\x22}},\
{match:[n,/::/,/\
class/],scope:{1\
:\x22title.class\x22,\x0a\
3:\x22variable.lang\
uage\x22}}]},I={sco\
pe:\x22attr\x22,\x0amatch\
:t.concat(r,t.lo\
okahead(\x22:\x22),t.l\
ookahead(/(?!::)\
/))},f={relevanc\
e:0,\x0abegin:/\x5c(/,\
end:/\x5c)/,keyword\
s:E,contains:[I,\
o,m,e.C_BLOCK_CO\
MMENT_MODE,l,d,g\
]\x0a},O={relevance\
:0,\x0amatch:[/\x5cb/,\
t.concat(\x22(?!fn\x5c\
\x5cb|function\x5c\x5cb|\x22\
,u(p).join(\x22\x5c\x5cb|\
\x22),\x22|\x22,u(b).join\
(\x22\x5c\x5cb|\x22),\x22\x5c\x5cb)\x22)\
,r,t.concat(s,\x22*\
\x22),t.lookahead(/\
(?=\x5c()/)],\x0ascope\
:{3:\x22title.funct\
ion.invoke\x22},con\
tains:[f]};f.con\
tains.push(O)\x0a;c\
onst v=[I,m,e.C_\
BLOCK_COMMENT_MO\
DE,l,d,g];return\
{case_insensitiv\
e:!1,\x0akeywords:E\
,contains:[{begi\
n:t.concat(/#\x5c[\x5c\
s*/,n),beginScop\
e:\x22meta\x22,end:/]/\
,\x0aendScope:\x22meta\
\x22,keywords:{lite\
ral:_,keyword:[\x22\
new\x22,\x22array\x22]},c\
ontains:[{\x0abegin\
:/\x5c[/,end:/]/,ke\
ywords:{literal:\
_,keyword:[\x22new\x22\
,\x22array\x22]},\x0acont\
ains:[\x22self\x22,...\
v]},...v,{scope:\
\x22meta\x22,match:n}]\
\x0a},e.HASH_COMMEN\
T_MODE,e.COMMENT\
(\x22//\x22,\x22$\x22),e.COM\
MENT(\x22/\x5c\x5c*\x22,\x22\x5c\x5c*\
/\x22,{contains:[{\x0a\
scope:\x22doctag\x22,m\
atch:\x22@[A-Za-z]+\
\x22}]}),{match:/__\
halt_compiler\x5c(\x5c\
);/,\x0akeywords:\x22_\
_halt_compiler\x22,\
starts:{scope:\x22c\
omment\x22,end:e.MA\
TCH_NOTHING_RE,\x0a\
contains:[{match\
:/\x5c?>/,scope:\x22me\
ta\x22,endsParent:!\
0}]}},{scope:\x22me\
ta\x22,variants:[{\x0a\
begin:/<\x5c?php/,r\
elevance:10},{be\
gin:/<\x5c?=/},{beg\
in:/<\x5c?/,relevan\
ce:.1},{\x0abegin:/\
\x5c?>/}]},{scope:\x22\
variable.languag\
e\x22,match:/\x5c$this\
\x5cb/},o,O,m,{\x0amat\
ch:[/const/,/\x5cs/\
,r],scope:{1:\x22ke\
yword\x22,3:\x22variab\
le.constant\x22}},g\
,{\x0ascope:\x22functi\
on\x22,relevance:0,\
beginKeywords:\x22f\
n function\x22,end:\
/[;{]/,\x0aexcludeE\
nd:!0,illegal:\x22[\
$%\x5c\x5c[]\x22,contains\
:[{beginKeywords\
:\x22use\x22\x0a},e.UNDER\
SCORE_TITLE_MODE\
,{begin:\x22=>\x22,end\
sParent:!0},{sco\
pe:\x22params\x22,\x0abeg\
in:\x22\x5c\x5c(\x22,end:\x22\x5c\x5c\
)\x22,excludeBegin:\
!0,excludeEnd:!0\
,keywords:E,\x0acon\
tains:[\x22self\x22,o,\
m,e.C_BLOCK_COMM\
ENT_MODE,l,d]}]}\
,{scope:\x22class\x22,\
variants:[{\x0abegi\
nKeywords:\x22enum\x22\
,illegal:/[($\x22]/\
},{beginKeywords\
:\x22class interfac\
e trait\x22,\x0aillega\
l:/[:($\x22]/}],rel\
evance:0,end:/\x5c{\
/,excludeEnd:!0,\
contains:[{\x0abegi\
nKeywords:\x22exten\
ds implements\x22},\
e.UNDERSCORE_TIT\
LE_MODE]},{\x0abegi\
nKeywords:\x22names\
pace\x22,relevance:\
0,end:\x22;\x22,illega\
l:/[.']/,\x0acontai\
ns:[e.inherit(e.\
UNDERSCORE_TITLE\
_MODE,{scope:\x22ti\
tle.class\x22})]},{\
\x0abeginKeywords:\x22\
use\x22,relevance:0\
,end:\x22;\x22,contain\
s:[{\x0amatch:/\x5cb(a\
s|const|function\
)\x5cb/,scope:\x22keyw\
ord\x22},e.UNDERSCO\
RE_TITLE_MODE]},\
l,d]}\x0a}})();hljs\
.registerLanguag\
e(\x22php\x22,e)})();/\
*! `php-template\
` grammar compil\
ed for Highlight\
.js 11.10.0 */\x0a(\
()=>{var n=(()=>\
{\x22use strict\x22;re\
turn n=>({name:\x22\
PHP template\x22,\x0as\
ubLanguage:\x22xml\x22\
,contains:[{begi\
n:/<\x5c?(php|=)?/,\
end:/\x5c?>/,subLan\
guage:\x22php\x22,\x0acon\
tains:[{begin:\x22/\
\x5c\x5c*\x22,end:\x22\x5c\x5c*/\x22,\
skip:!0},{begin:\
'b\x22',end:'\x22',ski\
p:!0},{\x0abegin:\x22b\
'\x22,end:\x22'\x22,skip:\
!0},n.inherit(n.\
APOS_STRING_MODE\
,{illegal:null,\x0a\
className:null,c\
ontains:null,ski\
p:!0}),n.inherit\
(n.QUOTE_STRING_\
MODE,{\x0aillegal:n\
ull,className:nu\
ll,contains:null\
,skip:!0})]}]})}\
)()\x0a;hljs.regist\
erLanguage(\x22php-\
template\x22,n)})()\
;/*! `plaintext`\
 grammar compile\
d for Highlight.\
js 11.10.0 */\x0a((\
)=>{var t=(()=>{\
\x22use strict\x22;ret\
urn t=>({name:\x22P\
lain text\x22,\x0aalia\
ses:[\x22text\x22,\x22txt\
\x22],disableAutode\
tect:!0})})()\x0a;h\
ljs.registerLang\
uage(\x22plaintext\x22\
,t)})();/*! `pon\
y` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>({name:\
\x22Pony\x22,keywords:\
{\x0akeyword:\x22actor\
 addressof and a\
s be break class\
 compile_error c\
ompile_intrinsic\
 consume continu\
e delegate diges\
tof do else else\
if embed end err\
or for fun if if\
def in interface\
 is isnt lambda \
let match new no\
t object or prim\
itive recover re\
peat return stru\
ct then trait tr\
y type until use\
 var where while\
 with xor\x22,\x0ameta\
:\x22iso val tag tr\
n box ref\x22,liter\
al:\x22this false t\
rue\x22},contains:[\
{\x0aclassName:\x22typ\
e\x22,begin:\x22\x5c\x5cb_?[\
A-Z][\x5c\x5cw]*\x22,rele\
vance:0},{classN\
ame:\x22string\x22,\x0abe\
gin:'\x22\x22\x22',end:'\x22\
\x22\x22',relevance:10\
},{className:\x22st\
ring\x22,begin:'\x22',\
end:'\x22',\x0acontain\
s:[e.BACKSLASH_E\
SCAPE]},{classNa\
me:\x22string\x22,begi\
n:\x22'\x22,end:\x22'\x22,\x0ac\
ontains:[e.BACKS\
LASH_ESCAPE],rel\
evance:0},{begin\
:e.IDENT_RE+\x22'\x22,\
relevance:0},{\x0ac\
lassName:\x22number\
\x22,\x0abegin:\x22(-?)(\x5c\
\x5cb0[xX][a-fA-F0-\
9]+|\x5c\x5cb0[bB][01]\
+|(\x5c\x5cb\x5c\x5cd+(_\x5c\x5cd+\
)?(\x5c\x5c.\x5c\x5cd*)?|\x5c\x5c.\
\x5c\x5cd+)([eE][-+]?\x5c\
\x5cd+)?)\x22,\x0arelevan\
ce:0},e.C_LINE_C\
OMMENT_MODE,e.C_\
BLOCK_COMMENT_MO\
DE]})})()\x0a;hljs.\
registerLanguage\
(\x22pony\x22,e)})();/\
*! `powershell` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{const n={\
$pattern:/-?[A-z\
\x5c.\x5c-]+\x5cb/,\x0akeywo\
rd:\x22if else fore\
ach return do wh\
ile until elseif\
 begin for trap \
data dynamicpara\
m end break thro\
w param continue\
 finally in swit\
ch exit filter t\
ry process catch\
 hidden static p\
arameter\x22,\x0abuilt\
_in:\x22ac asnp cat\
 cd CFS chdir cl\
c clear clhy cli\
 clp cls clv cns\
n compare copy c\
p cpi cpp curl c\
vpa dbp del diff\
 dir dnsn ebp ec\
ho|0 epal epcsv \
epsn erase etsn \
exsn fc fhx fl f\
t fw gal gbp gc \
gcb gci gcm gcs \
gdr gerr ghy gi \
gin gjb gl gm gm\
o gp gps gpv gro\
up gsn gsnp gsv \
gtz gu gv gwmi h\
 history icm iex\
 ihy ii ipal ipc\
sv ipmo ipsn irm\
 ise iwmi iwr ki\
ll lp ls man md \
measure mi mount\
 move mp mv nal \
ndr ni nmo npssc\
 nsn nv ogv oh p\
opd ps pushd pwd\
 r rbp rcjb rcsn\
 rd rdr ren ri r\
jb rm rmdir rmo \
rni rnp rp rsn r\
snp rujb rv rvpa\
 rwmi sajb sal s\
aps sasv sbp sc \
scb select set s\
hcm si sl sleep \
sls sort sp spjb\
 spps spsv start\
 stz sujb sv swm\
i tee trcm type \
wget where wjb w\
rite\x22\x0a},s={begin\
:\x22`[\x5c\x5cs\x5c\x5cS]\x22,rel\
evance:0},i={cla\
ssName:\x22variable\
\x22,variants:[{\x0abe\
gin:/\x5c$\x5cB/},{cla\
ssName:\x22keyword\x22\
,begin:/\x5c$this/}\
,{begin:/\x5c$[\x5cw\x5cd\
][\x5cw\x5cd_:]*/}]\x0a},\
a={className:\x22st\
ring\x22,variants:[\
{begin:/\x22/,end:/\
\x22/},{begin:/@\x22/,\
end:/^\x22@/}],\x0acon\
tains:[s,i,{clas\
sName:\x22variable\x22\
,begin:/\x5c$[A-z]/\
,end:/[^A-z]/}]}\
,t={\x0aclassName:\x22\
string\x22,variants\
:[{begin:/'/,end\
:/'/},{begin:/@'\
/,end:/^'@/}]\x0a},\
r=e.inherit(e.CO\
MMENT(null,null)\
,{variants:[{beg\
in:/#/,end:/$/},\
{begin:/<#/,\x0aend\
:/#>/}],contains\
:[{className:\x22do\
ctag\x22,variants:[\
{\x0abegin:/\x5c.(syno\
psis|description\
|example|inputs|\
outputs|notes|li\
nk|component|rol\
e|functionality)\
/\x0a},{\x0abegin:/\x5c.(\
parameter|forwar\
dhelptargetname|\
forwardhelpcateg\
ory|remotehelpru\
nspace|externalh\
elp)\x5cs+\x5cS+/\x0a}]}]\
}),c={className:\
\x22class\x22,beginKey\
words:\x22class enu\
m\x22,end:/\x5cs*[{]/,\
\x0aexcludeEnd:!0,r\
elevance:0,conta\
ins:[e.TITLE_MOD\
E]},l={className\
:\x22function\x22,\x0abeg\
in:/function\x5cs+/\
,end:/\x5cs*\x5c{|$/,e\
xcludeEnd:!0,ret\
urnBegin:!0,rele\
vance:0,\x0acontain\
s:[{begin:\x22funct\
ion\x22,relevance:0\
,className:\x22keyw\
ord\x22},{className\
:\x22title\x22,\x0abegin:\
/\x5cw[\x5cw\x5cd]*((-)[\x5c\
w\x5cd]+)*/,relevan\
ce:0},{begin:/\x5c(\
/,end:/\x5c)/,\x0aclas\
sName:\x22params\x22,r\
elevance:0,conta\
ins:[i]}]},o={be\
gin:/using\x5cs/,en\
d:/$/,\x0areturnBeg\
in:!0,contains:[\
a,t,{className:\x22\
keyword\x22,\x0abegin:\
/(using|assembly\
|command|module|\
namespace|type)/\
}]},p={\x0aclassNam\
e:\x22function\x22,beg\
in:/\x5c[.*\x5c]\x5cs*[\x5cw\
]+[ ]??\x5c(/,end:/\
$/,returnBegin:!\
0,\x0arelevance:0,c\
ontains:[{classN\
ame:\x22keyword\x22,\x0ab\
egin:\x22(\x22.concat(\
n.keyword.toStri\
ng().replace(/\x5cs\
/g,\x22|\x22),\x22)\x5c\x5cb\x22),\
endsParent:!0,\x0ar\
elevance:0},e.in\
herit(e.TITLE_MO\
DE,{endsParent:!\
0})]\x0a},g=[p,r,s,\
e.NUMBER_MODE,a,\
t,{className:\x22bu\
ilt_in\x22,variants\
:[{\x0abegin:\x22(Add|\
Clear|Close|Copy\
|Enter|Exit|Find\
|Format|Get|Hide\
|Join|Lock|Move|\
New|Open|Optimiz\
e|Pop|Push|Redo|\
Remove|Rename|Re\
set|Resize|Searc\
h|Select|Set|Sho\
w|Skip|Split|Ste\
p|Switch|Undo|Un\
lock|Watch|Backu\
p|Checkpoint|Com\
pare|Compress|Co\
nvert|ConvertFro\
m|ConvertTo|Dism\
ount|Edit|Expand\
|Export|Group|Im\
port|Initialize|\
Limit|Merge|Moun\
t|Out|Publish|Re\
store|Save|Sync|\
Unpublish|Update\
|Approve|Assert|\
Build|Complete|C\
onfirm|Deny|Depl\
oy|Disable|Enabl\
e|Install|Invoke\
|Register|Reques\
t|Restart|Resume\
|Start|Stop|Subm\
it|Suspend|Unins\
tall|Unregister|\
Wait|Debug|Measu\
re|Ping|Repair|R\
esolve|Test|Trac\
e|Connect|Discon\
nect|Read|Receiv\
e|Send|Write|Blo\
ck|Grant|Protect\
|Revoke|Unblock|\
Unprotect|Use|Fo\
rEach|Sort|Tee|W\
here)+(-)[\x5c\x5cw\x5c\x5cd\
]+\x22\x0a}]},i,{class\
Name:\x22literal\x22,b\
egin:/\x5c$(null|tr\
ue|false)\x5cb/},{\x0a\
className:\x22selec\
tor-tag\x22,begin:/\
@\x5cB/,relevance:0\
}],m={begin:/\x5c[/\
,end:/\x5c]/,\x0aexclu\
deBegin:!0,exclu\
deEnd:!0,relevan\
ce:0,contains:[]\
.concat(\x22self\x22,g\
,{\x0abegin:\x22(strin\
g|char|byte|int|\
long|bool|decima\
l|single|double|\
DateTime|xml|arr\
ay|hashtable|voi\
d)\x22,\x0aclassName:\x22\
built_in\x22,releva\
nce:0},{classNam\
e:\x22type\x22,begin:/\
[\x5c.\x5cw\x5cd]+/,\x0arele\
vance:0})};retur\
n p.contains.uns\
hift(m),{name:\x22P\
owerShell\x22,\x0aalia\
ses:[\x22pwsh\x22,\x22ps\x22\
,\x22ps1\x22],case_ins\
ensitive:!0,keyw\
ords:n,\x0acontains\
:g.concat(c,l,o,\
{variants:[{clas\
sName:\x22operator\x22\
,\x0abegin:\x22(-and|-\
as|-band|-bnot|-\
bor|-bxor|-cases\
ensitive|-cconta\
ins|-ceq|-cge|-c\
gt|-cle|-clike|-\
clt|-cmatch|-cne\
|-cnotcontains|-\
cnotlike|-cnotma\
tch|-contains|-c\
replace|-csplit|\
-eq|-exact|-f|-f\
ile|-ge|-gt|-ico\
ntains|-ieq|-ige\
|-igt|-ile|-ilik\
e|-ilt|-imatch|-\
in|-ine|-inotcon\
tains|-inotlike|\
-inotmatch|-irep\
lace|-is|-isnot|\
-isplit|-join|-l\
e|-like|-lt|-mat\
ch|-ne|-not|-not\
contains|-notin|\
-notlike|-notmat\
ch|-or|-regex|-r\
eplace|-shl|-shr\
|-split|-wildcar\
d|-xor)\x5c\x5cb\x22\x0a},{c\
lassName:\x22litera\
l\x22,begin:/(-){1,\
2}[\x5cw\x5cd-]+/,rele\
vance:0}]},m)}}}\
)()\x0a;hljs.regist\
erLanguage(\x22powe\
rshell\x22,e)})();/\
*! `processing` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{\x0aconst t=\
e.regex,a=[\x22disp\
layHeight\x22,\x22disp\
layWidth\x22,\x22mouse\
Y\x22,\x22mouseX\x22,\x22mou\
sePressed\x22,\x22pmou\
seX\x22,\x22pmouseY\x22,\x22\
key\x22,\x22keyCode\x22,\x22\
pixels\x22,\x22focused\
\x22,\x22frameCount\x22,\x22\
frameRate\x22,\x22heig\
ht\x22,\x22width\x22,\x22siz\
e\x22,\x22createGraphi\
cs\x22,\x22beginDraw\x22,\
\x22createShape\x22,\x22l\
oadShape\x22,\x22PShap\
e\x22,\x22arc\x22,\x22ellips\
e\x22,\x22line\x22,\x22point\
\x22,\x22quad\x22,\x22rect\x22,\
\x22triangle\x22,\x22bezi\
er\x22,\x22bezierDetai\
l\x22,\x22bezierPoint\x22\
,\x22bezierTangent\x22\
,\x22curve\x22,\x22curveD\
etail\x22,\x22curvePoi\
nt\x22,\x22curveTangen\
t\x22,\x22curveTightne\
ss\x22,\x22shape\x22,\x22sha\
peMode\x22,\x22beginCo\
ntour\x22,\x22beginSha\
pe\x22,\x22bezierVerte\
x\x22,\x22curveVertex\x22\
,\x22endContour\x22,\x22e\
ndShape\x22,\x22quadra\
ticVertex\x22,\x22vert\
ex\x22,\x22ellipseMode\
\x22,\x22noSmooth\x22,\x22re\
ctMode\x22,\x22smooth\x22\
,\x22strokeCap\x22,\x22st\
rokeJoin\x22,\x22strok\
eWeight\x22,\x22mouseC\
licked\x22,\x22mouseDr\
agged\x22,\x22mouseMov\
ed\x22,\x22mousePresse\
d\x22,\x22mouseRelease\
d\x22,\x22mouseWheel\x22,\
\x22keyPressed\x22,\x22ke\
yPressedkeyRelea\
sed\x22,\x22keyTyped\x22,\
\x22print\x22,\x22println\
\x22,\x22save\x22,\x22saveFr\
ame\x22,\x22day\x22,\x22hour\
\x22,\x22millis\x22,\x22minu\
te\x22,\x22month\x22,\x22sec\
ond\x22,\x22year\x22,\x22bac\
kground\x22,\x22clear\x22\
,\x22colorMode\x22,\x22fi\
ll\x22,\x22noFill\x22,\x22no\
Stroke\x22,\x22stroke\x22\
,\x22alpha\x22,\x22blue\x22,\
\x22brightness\x22,\x22co\
lor\x22,\x22green\x22,\x22hu\
e\x22,\x22lerpColor\x22,\x22\
red\x22,\x22saturation\
\x22,\x22modelX\x22,\x22mode\
lY\x22,\x22modelZ\x22,\x22sc\
reenX\x22,\x22screenY\x22\
,\x22screenZ\x22,\x22ambi\
ent\x22,\x22emissive\x22,\
\x22shininess\x22,\x22spe\
cular\x22,\x22add\x22,\x22cr\
eateImage\x22,\x22begi\
nCamera\x22,\x22camera\
\x22,\x22endCamera\x22,\x22f\
rustum\x22,\x22ortho\x22,\
\x22perspective\x22,\x22p\
rintCamera\x22,\x22pri\
ntProjection\x22,\x22c\
ursor\x22,\x22frameRat\
e\x22,\x22noCursor\x22,\x22e\
xit\x22,\x22loop\x22,\x22noL\
oop\x22,\x22popStyle\x22,\
\x22pushStyle\x22,\x22red\
raw\x22,\x22binary\x22,\x22b\
oolean\x22,\x22byte\x22,\x22\
char\x22,\x22float\x22,\x22h\
ex\x22,\x22int\x22,\x22str\x22,\
\x22unbinary\x22,\x22unhe\
x\x22,\x22join\x22,\x22match\
\x22,\x22matchAll\x22,\x22nf\
\x22,\x22nfc\x22,\x22nfp\x22,\x22n\
fs\x22,\x22split\x22,\x22spl\
itTokens\x22,\x22trim\x22\
,\x22append\x22,\x22array\
Copy\x22,\x22concat\x22,\x22\
expand\x22,\x22reverse\
\x22,\x22shorten\x22,\x22sor\
t\x22,\x22splice\x22,\x22sub\
set\x22,\x22box\x22,\x22sphe\
re\x22,\x22sphereDetai\
l\x22,\x22createInput\x22\
,\x22createReader\x22,\
\x22loadBytes\x22,\x22loa\
dJSONArray\x22,\x22loa\
dJSONObject\x22,\x22lo\
adStrings\x22,\x22load\
Table\x22,\x22loadXML\x22\
,\x22open\x22,\x22parseXM\
L\x22,\x22saveTable\x22,\x22\
selectFolder\x22,\x22s\
electInput\x22,\x22beg\
inRaw\x22,\x22beginRec\
ord\x22,\x22createOutp\
ut\x22,\x22createWrite\
r\x22,\x22endRaw\x22,\x22end\
Record\x22,\x22PrintWr\
itersaveBytes\x22,\x22\
saveJSONArray\x22,\x22\
saveJSONObject\x22,\
\x22saveStream\x22,\x22sa\
veStrings\x22,\x22save\
XML\x22,\x22selectOutp\
ut\x22,\x22popMatrix\x22,\
\x22printMatrix\x22,\x22p\
ushMatrix\x22,\x22rese\
tMatrix\x22,\x22rotate\
\x22,\x22rotateX\x22,\x22rot\
ateY\x22,\x22rotateZ\x22,\
\x22scale\x22,\x22shearX\x22\
,\x22shearY\x22,\x22trans\
late\x22,\x22ambientLi\
ght\x22,\x22directiona\
lLight\x22,\x22lightFa\
lloff\x22,\x22lights\x22,\
\x22lightSpecular\x22,\
\x22noLights\x22,\x22norm\
al\x22,\x22pointLight\x22\
,\x22spotLight\x22,\x22im\
age\x22,\x22imageMode\x22\
,\x22loadImage\x22,\x22no\
Tint\x22,\x22requestIm\
age\x22,\x22tint\x22,\x22tex\
ture\x22,\x22textureMo\
de\x22,\x22textureWrap\
\x22,\x22blend\x22,\x22copy\x22\
,\x22filter\x22,\x22get\x22,\
\x22loadPixels\x22,\x22se\
t\x22,\x22updatePixels\
\x22,\x22blendMode\x22,\x22l\
oadShader\x22,\x22PSha\
derresetShader\x22,\
\x22shader\x22,\x22create\
Font\x22,\x22loadFont\x22\
,\x22text\x22,\x22textFon\
t\x22,\x22textAlign\x22,\x22\
textLeading\x22,\x22te\
xtMode\x22,\x22textSiz\
e\x22,\x22textWidth\x22,\x22\
textAscent\x22,\x22tex\
tDescent\x22,\x22abs\x22,\
\x22ceil\x22,\x22constrai\
n\x22,\x22dist\x22,\x22exp\x22,\
\x22floor\x22,\x22lerp\x22,\x22\
log\x22,\x22mag\x22,\x22map\x22\
,\x22max\x22,\x22min\x22,\x22no\
rm\x22,\x22pow\x22,\x22round\
\x22,\x22sq\x22,\x22sqrt\x22,\x22a\
cos\x22,\x22asin\x22,\x22ata\
n\x22,\x22atan2\x22,\x22cos\x22\
,\x22degrees\x22,\x22radi\
ans\x22,\x22sin\x22,\x22tan\x22\
,\x22noise\x22,\x22noiseD\
etail\x22,\x22noiseSee\
d\x22,\x22random\x22,\x22ran\
domGaussian\x22,\x22ra\
ndomSeed\x22],r=e.I\
DENT_RE,s={\x0avari\
ants:[{match:t.c\
oncat(t.either(.\
..a),t.lookahead\
(/\x5cs*\x5c(/)),\x0aclas\
sName:\x22built_in\x22\
},{relevance:0,\x0a\
match:t.concat(/\
\x5cb(?!for|if|whil\
e)/,r,t.lookahea\
d(/\x5cs*\x5c(/)),\x0acla\
ssName:\x22title.fu\
nction\x22}]},o={ma\
tch:[/new\x5cs+/,r]\
,className:{1:\x22k\
eyword\x22,\x0a2:\x22clas\
s.title\x22}},i={re\
levance:0,match:\
[/\x5c./,r],classNa\
me:{2:\x22property\x22\
}},n={\x0avariants:\
[{match:[/class/\
,/\x5cs+/,r,/\x5cs+/,/\
extends/,/\x5cs+/,r\
]},{\x0amatch:[/cla\
ss/,/\x5cs+/,r]}],c\
lassName:{1:\x22key\
word\x22,3:\x22title.c\
lass\x22,5:\x22keyword\
\x22,\x0a7:\x22title.clas\
s.inherited\x22}};r\
eturn{name:\x22Proc\
essing\x22,aliases:\
[\x22pde\x22],keywords\
:{\x0akeyword:[\x22abs\
tract\x22,\x22assert\x22,\
\x22break\x22,\x22case\x22,\x22\
catch\x22,\x22const\x22,\x22\
continue\x22,\x22defau\
lt\x22,\x22else\x22,\x22enum\
\x22,\x22final\x22,\x22final\
ly\x22,\x22for\x22,\x22if\x22,\x22\
import\x22,\x22instanc\
eof\x22,\x22long\x22,\x22nat\
ive\x22,\x22new\x22,\x22pack\
age\x22,\x22private\x22,\x22\
private\x22,\x22protec\
ted\x22,\x22protected\x22\
,\x22public\x22,\x22publi\
c\x22,\x22return\x22,\x22sta\
tic\x22,\x22strictfp\x22,\
\x22switch\x22,\x22synchr\
onized\x22,\x22throw\x22,\
\x22throws\x22,\x22transi\
ent\x22,\x22try\x22,\x22void\
\x22,\x22volatile\x22,\x22wh\
ile\x22],\x0aliteral:\x22\
P2D P3D HALF_PI \
PI QUARTER_PI TA\
U TWO_PI null tr\
ue false\x22,\x0atitle\
:\x22setup draw\x22,va\
riable:\x22super th\
is\x22,\x0abuilt_in:[.\
..a,\x22BufferedRea\
der\x22,\x22PVector\x22,\x22\
PFont\x22,\x22PImage\x22,\
\x22PGraphics\x22,\x22Has\
hMap\x22,\x22String\x22,\x22\
Array\x22,\x22FloatDic\
t\x22,\x22ArrayList\x22,\x22\
FloatList\x22,\x22IntD\
ict\x22,\x22IntList\x22,\x22\
JSONArray\x22,\x22JSON\
Object\x22,\x22Object\x22\
,\x22StringDict\x22,\x22S\
tringList\x22,\x22Tabl\
e\x22,\x22TableRow\x22,\x22X\
ML\x22],\x0atype:[\x22boo\
lean\x22,\x22byte\x22,\x22ch\
ar\x22,\x22color\x22,\x22dou\
ble\x22,\x22float\x22,\x22in\
t\x22,\x22long\x22,\x22short\
\x22]},\x0acontains:[n\
,o,s,i,e.C_LINE_\
COMMENT_MODE,e.C\
_BLOCK_COMMENT_M\
ODE,e.APOS_STRIN\
G_MODE,e.QUOTE_S\
TRING_MODE,e.C_N\
UMBER_MODE]\x0a}}})\
();hljs.register\
Language(\x22proces\
sing\x22,e)})();/*!\
 `profile` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>({name:\x22Python \
profiler\x22,\x0aconta\
ins:[e.C_NUMBER_\
MODE,{begin:\x22[a-\
zA-Z_][\x5c\x5cda-zA-Z\
_]+\x5c\x5c.[\x5c\x5cda-zA-Z\
_]{1,3}\x22,\x0aend:\x22:\
\x22,excludeEnd:!0}\
,{begin:\x22(ncalls\
|tottime|cumtime\
)\x22,end:\x22$\x22,\x0akeyw\
ords:\x22ncalls tot\
time|10 cumtime|\
10 filename\x22,rel\
evance:10},{\x0abeg\
in:\x22function cal\
ls\x22,end:\x22$\x22,cont\
ains:[e.C_NUMBER\
_MODE],relevance\
:10\x0a},e.APOS_STR\
ING_MODE,e.QUOTE\
_STRING_MODE,{cl\
assName:\x22string\x22\
,begin:\x22\x5c\x5c(\x22,\x0aen\
d:\x22\x5c\x5c)$\x22,exclude\
Begin:!0,exclude\
End:!0,relevance\
:0}]})})()\x0a;hljs\
.registerLanguag\
e(\x22profile\x22,e)})\
();/*! `prolog` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var n=(()=>{\x22\
use strict\x22;retu\
rn n=>{const e={\
begin:/\x5c(/,end:/\
\x5c)/,\x0arelevance:0\
},a={begin:/\x5c[/,\
end:/\x5c]/},s={cla\
ssName:\x22comment\x22\
,begin:/%/,end:/\
$/,\x0acontains:[n.\
PHRASAL_WORDS_MO\
DE]},i={classNam\
e:\x22string\x22,begin\
:/`/,end:/`/,\x0aco\
ntains:[n.BACKSL\
ASH_ESCAPE]},g=[\
{begin:/[a-z][A-\
Za-z0-9_]*/,rele\
vance:0},{\x0aclass\
Name:\x22symbol\x22,va\
riants:[{begin:/\
[A-Z][a-zA-Z0-9_\
]*/},{\x0abegin:/_[\
A-Za-z0-9_]*/}],\
relevance:0},e,{\
begin:/:-/\x0a},a,s\
,n.C_BLOCK_COMME\
NT_MODE,n.QUOTE_\
STRING_MODE,n.AP\
OS_STRING_MODE,i\
,{\x0aclassName:\x22st\
ring\x22,begin:/0'(\
\x5c\x5c'|.)/},{classN\
ame:\x22string\x22,beg\
in:/0'\x5c\x5cs/\x0a},n.C\
_NUMBER_MODE];re\
turn e.contains=\
g,a.contains=g,{\
name:\x22Prolog\x22,\x0ac\
ontains:g.concat\
([{begin:/\x5c.$/}]\
)}}})();hljs.reg\
isterLanguage(\x22p\
rolog\x22,n)})();/*\
! `properties` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>{\x0aconst n=\x22\
[ \x5c\x5ct\x5c\x5cf]*\x22,t=n+\
\x22[:=]\x22+n,s=\x22[ \x5c\x5c\
t\x5c\x5cf]+\x22,a=\x22([^\x5c\x5c\
\x5c\x5c:= \x5c\x5ct\x5c\x5cf\x5c\x5cn]|\
\x5c\x5c\x5c\x5c.)+\x22,r={\x0aend\
:\x22(\x22+t+\x22|\x22+s+\x22)\x22\
,relevance:0,sta\
rts:{className:\x22\
string\x22,end:/$/,\
relevance:0,\x0acon\
tains:[{begin:\x22\x5c\
\x5c\x5c\x5c\x5c\x5c\x5c\x5c\x22},{begin\
:\x22\x5c\x5c\x5c\x5c\x5c\x5cn\x22}]}};r\
eturn{name:\x22.pro\
perties\x22,\x0adisabl\
eAutodetect:!0,c\
ase_insensitive:\
!0,illegal:/\x5cS/,\
\x0acontains:[e.COM\
MENT(\x22^\x5c\x5cs*[!#]\x22\
,\x22$\x22),{returnBeg\
in:!0,variants:[\
{begin:a+t},{\x0abe\
gin:a+s}],contai\
ns:[{className:\x22\
attr\x22,begin:a,en\
dsParent:!0}],st\
arts:r},{\x0aclassN\
ame:\x22attr\x22,begin\
:a+n+\x22$\x22}]}}})()\
;hljs.registerLa\
nguage(\x22properti\
es\x22,e)\x0a})();/*! \
`protobuf` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{const s={\x0amatc\
h:[/(message|enu\
m|service)\x5cs+/,e\
.IDENT_RE],scope\
:{1:\x22keyword\x22,\x0a2\
:\x22title.class\x22}}\
;return{name:\x22Pr\
otocol Buffers\x22,\
aliases:[\x22proto\x22\
],keywords:{\x0akey\
word:[\x22package\x22,\
\x22import\x22,\x22option\
\x22,\x22optional\x22,\x22re\
quired\x22,\x22repeate\
d\x22,\x22group\x22,\x22oneo\
f\x22],\x0atype:[\x22doub\
le\x22,\x22float\x22,\x22int\
32\x22,\x22int64\x22,\x22uin\
t32\x22,\x22uint64\x22,\x22s\
int32\x22,\x22sint64\x22,\
\x22fixed32\x22,\x22fixed\
64\x22,\x22sfixed32\x22,\x22\
sfixed64\x22,\x22bool\x22\
,\x22string\x22,\x22bytes\
\x22],\x0aliteral:[\x22tr\
ue\x22,\x22false\x22]},\x0ac\
ontains:[e.QUOTE\
_STRING_MODE,e.N\
UMBER_MODE,e.C_L\
INE_COMMENT_MODE\
,e.C_BLOCK_COMME\
NT_MODE,s,{\x0aclas\
sName:\x22function\x22\
,beginKeywords:\x22\
rpc\x22,end:/[{;]/,\
excludeEnd:!0,\x0ak\
eywords:\x22rpc ret\
urns\x22},{begin:/^\
\x5cs*[A-Z_]+(?=\x5cs*\
=[^\x5cn]+;$)/}]}}}\
)()\x0a;hljs.regist\
erLanguage(\x22prot\
obuf\x22,e)})();/*!\
 `puppet` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
{\x0aconst s=e.COMM\
ENT(\x22#\x22,\x22$\x22),r=\x22\
([A-Za-z_]|::)(\x5c\
\x5cw|::)*\x22,a=e.inh\
erit(e.TITLE_MOD\
E,{\x0abegin:r}),n=\
{className:\x22vari\
able\x22,begin:\x22\x5c\x5c$\
\x22+r},i={classNam\
e:\x22string\x22,\x0acont\
ains:[e.BACKSLAS\
H_ESCAPE,n],vari\
ants:[{begin:/'/\
,end:/'/},{begin\
:/\x22/,end:/\x22/\x0a}]}\
;return{name:\x22Pu\
ppet\x22,aliases:[\x22\
pp\x22],contains:[s\
,n,i,{beginKeywo\
rds:\x22class\x22,\x0aend\
:\x22\x5c\x5c{|;\x22,illegal\
:/=/,contains:[a\
,s]},{beginKeywo\
rds:\x22define\x22,end\
:/\x5c{/,\x0acontains:\
[{className:\x22sec\
tion\x22,begin:e.ID\
ENT_RE,endsParen\
t:!0}]},{\x0abegin:\
e.IDENT_RE+\x22\x5c\x5cs+\
\x5c\x5c{\x22,returnBegin\
:!0,end:/\x5cS/,con\
tains:[{\x0aclassNa\
me:\x22keyword\x22,beg\
in:e.IDENT_RE,re\
levance:.2},{beg\
in:/\x5c{/,end:/\x5c}/\
,\x0akeywords:{\x0akey\
word:\x22and case d\
efault else elsi\
f false if in im\
port enherits no\
de or true undef\
 unless main set\
tings $string \x22,\
\x0aliteral:\x22alias \
audit before log\
level noop requi\
re subscribe tag\
 owner ensure gr\
oup mode name|0 \
changes context \
force incl lens \
load_path onlyif\
 provider return\
s root show_diff\
 type_check en_a\
ddress ip_addres\
s realname comma\
nd environment h\
our monute month\
 monthday specia\
l target weekday\
 creates cwd ogo\
utput refresh re\
freshonly tries \
try_sleep umask \
backup checksum \
content ctime fo\
rce ignore links\
 mtime purge rec\
urse recurselimi\
t replace selinu\
x_ignore_default\
s selrange selro\
le seltype selus\
er source souirc\
e_permissions so\
urceselect valid\
ate_cmd validate\
_replacement all\
owdupe attribute\
_membership auth\
_membership forc\
elocal gid ia_lo\
ad_module member\
s system host_al\
iases ip allowed\
_trunk_vlans des\
cription device_\
url duplex encap\
sulation etherch\
annel native_vla\
n speed principa\
ls allow_root au\
th_class auth_ty\
pe authenticate_\
user k_of_n mech\
anisms rule sess\
ion_owner shared\
 options device \
fstype enable ha\
srestart directo\
ry present absen\
t link atboot bl\
ockdevice device\
 dump pass remou\
nts poller_tag u\
se message withp\
ath adminfile al\
low_virtual allo\
wcdrom category \
configfiles flav\
or install_optio\
ns instance pack\
age_settings pla\
tform responsefi\
le status uninst\
all_options vend\
or unless_system\
_user unless_uid\
 binary control \
flags hasstatus \
manifest pattern\
 restart running\
 start stop allo\
wdupe auths expi\
ry gid groups ho\
me iterations ke\
y_membership key\
s managehome mem\
bership password\
 password_max_ag\
e password_min_a\
ge profile_membe\
rship profiles p\
roject purge_ssh\
_keys role_membe\
rship roles salt\
 shell uid baseu\
rl cost descr en\
abled enablegrou\
ps exclude failo\
vermethod gpgche\
ck gpgkey http_c\
aching include i\
ncludepkgs keepa\
live metadata_ex\
pire metalink mi\
rrorlist priorit\
y protect proxy \
proxy_password p\
roxy_username re\
po_gpgcheck s3_e\
nabled skip_if_u\
navailable sslca\
cert sslclientce\
rt sslclientkey \
sslverify mounte\
d\x22,\x0abuilt_in:\x22ar\
chitecture augea\
sversion blockde\
vices boardmanuf\
acturer boardpro\
ductname boardse\
rialnumber cfkey\
 dhcp_servers do\
main ec2_ ec2_us\
erdata facterver\
sion filesystems\
 ldom fqdn gid h\
ardwareisa hardw\
aremodel hostnam\
e id|0 interface\
s ipaddress ipad\
dress_ ipaddress\
6 ipaddress6_ ip\
hostnumber is_vi\
rtual kernel ker\
nelmajversion ke\
rnelrelease kern\
elversion kernel\
release kernelve\
rsion lsbdistcod\
ename lsbdistdes\
cription lsbdist\
id lsbdistreleas\
e lsbmajdistrele\
ase lsbminordist\
release lsbrelea\
se macaddress ma\
caddress_ macosx\
_buildversion ma\
cosx_productname\
 macosx_productv\
ersion macosx_pr\
oductverson_majo\
r macosx_product\
version_minor ma\
nufacturer memor\
yfree memorysize\
 netmask metmask\
_ network_ opera\
tingsystem opera\
tingsystemmajrel\
ease operatingsy\
stemrelease osfa\
mily partitions \
path physicalpro\
cessorcount proc\
essor processorc\
ount productname\
 ps puppetversio\
n rubysitedir ru\
byversion selinu\
x selinux_config\
_mode selinux_co\
nfig_policy seli\
nux_current_mode\
 selinux_current\
_mode selinux_en\
forced selinux_p\
olicyversion ser\
ialnumber sp_ ss\
hdsakey sshecdsa\
key sshrsakey sw\
apencrypted swap\
free swapsize ti\
mezone type uniq\
ueid uptime upti\
me_days uptime_h\
ours uptime_seco\
nds uuid virtual\
 vlans xendomain\
s zfs_version zo\
nenae zones zpoo\
l_version\x22\x0a},rel\
evance:0,contain\
s:[i,s,{begin:\x22[\
a-zA-Z_]+\x5c\x5cs*=>\x22\
,returnBegin:!0,\
end:\x22=>\x22,\x0acontai\
ns:[{className:\x22\
attr\x22,begin:e.ID\
ENT_RE}]},{class\
Name:\x22number\x22,\x0ab\
egin:\x22(\x5c\x5cb0[0-7_\
]+)|(\x5c\x5cb0x[0-9a-\
fA-F_]+)|(\x5c\x5cb[1-\
9][0-9_]*(\x5c\x5c.[0-\
9_]+)?)|[0_]\x5c\x5cb\x22\
,\x0arelevance:0},n\
]}],relevance:0}\
]}}})();hljs.reg\
isterLanguage(\x22p\
uppet\x22,e)})();/*\
! `purebasic` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>({name:\x22Pure\
BASIC\x22,\x0aaliases:\
[\x22pb\x22,\x22pbi\x22],\x0ake\
ywords:\x22Align An\
d Array As Break\
 CallDebugger Ca\
se CompilerCase \
CompilerDefault \
CompilerElse Com\
pilerElseIf Comp\
ilerEndIf Compil\
erEndSelect Comp\
ilerError Compil\
erIf CompilerSel\
ect CompilerWarn\
ing Continue Dat\
a DataSection De\
bug DebugLevel D\
eclare DeclareC \
DeclareCDLL Decl\
areDLL DeclareMo\
dule Default Def\
ine Dim DisableA\
SM DisableDebugg\
er DisableExplic\
it Else ElseIf E\
nableASM EnableD\
ebugger EnableEx\
plicit End EndDa\
taSection EndDec\
lareModule EndEn\
umeration EndIf \
EndImport EndInt\
erface EndMacro \
EndModule EndPro\
cedure EndSelect\
 EndStructure En\
dStructureUnion \
EndWith Enumerat\
ion EnumerationB\
inary Extends Fa\
keReturn For For\
Each ForEver Glo\
bal Gosub Goto I\
f Import ImportC\
 IncludeBinary I\
ncludeFile Inclu\
dePath Interface\
 List Macro Macr\
oExpandedCount M\
ap Module NewLis\
t NewMap Next No\
t Or Procedure P\
rocedureC Proced\
ureCDLL Procedur\
eDLL ProcedureRe\
turn Protected P\
rototype Prototy\
peC ReDim Read R\
epeat Restore Re\
turn Runtime Sel\
ect Shared Stati\
c Step Structure\
 StructureUnion \
Swap Threaded To\
 UndefineMacro U\
ntil Until  Unus\
eModule UseModul\
e Wend While Wit\
h XIncludeFile X\
Or\x22,\x0acontains:[e\
.COMMENT(\x22;\x22,\x22$\x22\
,{relevance:0}),\
{className:\x22func\
tion\x22,\x0abegin:\x22\x5c\x5c\
b(Procedure|Decl\
are)(C|CDLL|DLL)\
?\x5c\x5cb\x22,end:\x22\x5c\x5c(\x22,\
excludeEnd:!0,\x0ar\
eturnBegin:!0,co\
ntains:[{classNa\
me:\x22keyword\x22,\x0abe\
gin:\x22(Procedure|\
Declare)(C|CDLL|\
DLL)?\x22,excludeEn\
d:!0},{className\
:\x22type\x22,\x0abegin:\x22\
\x5c\x5c.\x5c\x5cw*\x22},e.UNDE\
RSCORE_TITLE_MOD\
E]},{className:\x22\
string\x22,begin:'(\
~)?\x22',\x0aend:'\x22',i\
llegal:\x22\x5c\x5cn\x22},{c\
lassName:\x22symbol\
\x22,begin:\x22#[a-zA-\
Z_]\x5c\x5cw*\x5c\x5c$?\x22}]})\
})()\x0a;hljs.regis\
terLanguage(\x22pur\
ebasic\x22,e)})();/\
*! `python` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>{\x0aconst n=e.re\
gex,a=/[\x5cp{XID_S\
tart}_]\x5cp{XID_Co\
ntinue}*/u,s=[\x22a\
nd\x22,\x22as\x22,\x22assert\
\x22,\x22async\x22,\x22await\
\x22,\x22break\x22,\x22case\x22\
,\x22class\x22,\x22contin\
ue\x22,\x22def\x22,\x22del\x22,\
\x22elif\x22,\x22else\x22,\x22e\
xcept\x22,\x22finally\x22\
,\x22for\x22,\x22from\x22,\x22g\
lobal\x22,\x22if\x22,\x22imp\
ort\x22,\x22in\x22,\x22is\x22,\x22\
lambda\x22,\x22match\x22,\
\x22nonlocal|10\x22,\x22n\
ot\x22,\x22or\x22,\x22pass\x22,\
\x22raise\x22,\x22return\x22\
,\x22try\x22,\x22while\x22,\x22\
with\x22,\x22yield\x22],t\
={\x0a$pattern:/[A-\
Za-z]\x5cw+|__\x5cw+__\
/,keyword:s,\x0abui\
lt_in:[\x22__import\
__\x22,\x22abs\x22,\x22all\x22,\
\x22any\x22,\x22ascii\x22,\x22b\
in\x22,\x22bool\x22,\x22brea\
kpoint\x22,\x22bytearr\
ay\x22,\x22bytes\x22,\x22cal\
lable\x22,\x22chr\x22,\x22cl\
assmethod\x22,\x22comp\
ile\x22,\x22complex\x22,\x22\
delattr\x22,\x22dict\x22,\
\x22dir\x22,\x22divmod\x22,\x22\
enumerate\x22,\x22eval\
\x22,\x22exec\x22,\x22filter\
\x22,\x22float\x22,\x22forma\
t\x22,\x22frozenset\x22,\x22\
getattr\x22,\x22global\
s\x22,\x22hasattr\x22,\x22ha\
sh\x22,\x22help\x22,\x22hex\x22\
,\x22id\x22,\x22input\x22,\x22i\
nt\x22,\x22isinstance\x22\
,\x22issubclass\x22,\x22i\
ter\x22,\x22len\x22,\x22list\
\x22,\x22locals\x22,\x22map\x22\
,\x22max\x22,\x22memoryvi\
ew\x22,\x22min\x22,\x22next\x22\
,\x22object\x22,\x22oct\x22,\
\x22open\x22,\x22ord\x22,\x22po\
w\x22,\x22print\x22,\x22prop\
erty\x22,\x22range\x22,\x22r\
epr\x22,\x22reversed\x22,\
\x22round\x22,\x22set\x22,\x22s\
etattr\x22,\x22slice\x22,\
\x22sorted\x22,\x22static\
method\x22,\x22str\x22,\x22s\
um\x22,\x22super\x22,\x22tup\
le\x22,\x22type\x22,\x22vars\
\x22,\x22zip\x22],\x0alitera\
l:[\x22__debug__\x22,\x22\
Ellipsis\x22,\x22False\
\x22,\x22None\x22,\x22NotImp\
lemented\x22,\x22True\x22\
],\x0atype:[\x22Any\x22,\x22\
Callable\x22,\x22Corou\
tine\x22,\x22Dict\x22,\x22Li\
st\x22,\x22Literal\x22,\x22G\
eneric\x22,\x22Optiona\
l\x22,\x22Sequence\x22,\x22S\
et\x22,\x22Tuple\x22,\x22Typ\
e\x22,\x22Union\x22]\x0a},i=\
{className:\x22meta\
\x22,begin:/^(>>>|\x5c\
.\x5c.\x5c.) /},r={cla\
ssName:\x22subst\x22,b\
egin:/\x5c{/,\x0aend:/\
\x5c}/,keywords:t,i\
llegal:/#/},l={b\
egin:/\x5c{\x5c{/,rele\
vance:0},o={\x0acla\
ssName:\x22string\x22,\
contains:[e.BACK\
SLASH_ESCAPE],va\
riants:[{\x0abegin:\
/([uU]|[bB]|[rR]\
|[bB][rR]|[rR][b\
B])?'''/,end:/''\
'/,\x0acontains:[e.\
BACKSLASH_ESCAPE\
,i],relevance:10\
},{\x0abegin:/([uU]\
|[bB]|[rR]|[bB][\
rR]|[rR][bB])?\x22\x22\
\x22/,end:/\x22\x22\x22/,\x0aco\
ntains:[e.BACKSL\
ASH_ESCAPE,i],re\
levance:10},{\x0abe\
gin:/([fF][rR]|[\
rR][fF]|[fF])'''\
/,end:/'''/,\x0acon\
tains:[e.BACKSLA\
SH_ESCAPE,i,l,r]\
},{begin:/([fF][\
rR]|[rR][fF]|[fF\
])\x22\x22\x22/,\x0aend:/\x22\x22\x22\
/,contains:[e.BA\
CKSLASH_ESCAPE,i\
,l,r]},{begin:/(\
[uU]|[rR])'/,end\
:/'/,\x0arelevance:\
10},{begin:/([uU\
]|[rR])\x22/,end:/\x22\
/,relevance:10},\
{\x0abegin:/([bB]|[\
bB][rR]|[rR][bB]\
)'/,end:/'/},{be\
gin:/([bB]|[bB][\
rR]|[rR][bB])\x22/,\
\x0aend:/\x22/},{begin\
:/([fF][rR]|[rR]\
[fF]|[fF])'/,end\
:/'/,\x0acontains:[\
e.BACKSLASH_ESCA\
PE,l,r]},{begin:\
/([fF][rR]|[rR][\
fF]|[fF])\x22/,end:\
/\x22/,\x0acontains:[e\
.BACKSLASH_ESCAP\
E,l,r]},e.APOS_S\
TRING_MODE,e.QUO\
TE_STRING_MODE]\x0a\
},b=\x22[0-9](_?[0-\
9])*\x22,c=`(\x5c\x5cb(${\
b}))?\x5c\x5c.(${b})|\x5c\
\x5cb(${b})\x5c\x5c.`,d=\x22\
\x5c\x5cb|\x22+s.join(\x22|\x22\
),g={\x0aclassName:\
\x22number\x22,relevan\
ce:0,variants:[{\
\x0abegin:`(\x5c\x5cb(${b\
})|(${c}))[eE][+\
-]?(${b})[jJ]?(?\
=${d})`},{begin:\
`(${c})[jJ]?`},{\
\x0abegin:`\x5c\x5cb([1-9\
](_?[0-9])*|0+(_\
?0)*)[lLjJ]?(?=$\
{d})`},{\x0abegin:`\
\x5c\x5cb0[bB](_?[01])\
+[lL]?(?=${d})`}\
,{begin:`\x5c\x5cb0[oO\
](_?[0-7])+[lL]?\
(?=${d})`\x0a},{beg\
in:`\x5c\x5cb0[xX](_?[\
0-9a-fA-F])+[lL]\
?(?=${d})`},{beg\
in:`\x5c\x5cb(${b})[jJ\
](?=${d})`\x0a}]},p\
={className:\x22com\
ment\x22,begin:n.lo\
okahead(/# type:\
/),end:/$/,keywo\
rds:t,\x0acontains:\
[{begin:/# type:\
/},{begin:/#/,en\
d:/\x5cb\x5cB/,endsWit\
hParent:!0}]},m=\
{\x0aclassName:\x22par\
ams\x22,variants:[{\
className:\x22\x22,beg\
in:/\x5c(\x5cs*\x5c)/,ski\
p:!0},{begin:/\x5c(\
/,\x0aend:/\x5c)/,excl\
udeBegin:!0,excl\
udeEnd:!0,keywor\
ds:t,\x0acontains:[\
\x22self\x22,i,g,o,e.H\
ASH_COMMENT_MODE\
]}]};return r.co\
ntains=[o,g,i],{\
\x0aname:\x22Python\x22,a\
liases:[\x22py\x22,\x22gy\
p\x22,\x22ipython\x22],un\
icodeRegex:!0,ke\
ywords:t,\x0aillega\
l:/(<\x5c/|\x5c?)|=>/,\
contains:[i,g,{s\
cope:\x22variable.l\
anguage\x22,match:/\
\x5cbself\x5cb/\x0a},{beg\
inKeywords:\x22if\x22,\
relevance:0},{ma\
tch:/\x5cbor\x5cb/,sco\
pe:\x22keyword\x22\x0a},o\
,p,e.HASH_COMMEN\
T_MODE,{match:[/\
\x5cbdef/,/\x5cs+/,a],\
scope:{1:\x22keywor\
d\x22,\x0a3:\x22title.fun\
ction\x22},contains\
:[m]},{variants:\
[{\x0amatch:[/\x5cbcla\
ss/,/\x5cs+/,a,/\x5cs*\
/,/\x5c(\x5cs*/,a,/\x5cs*\
\x5c)/]},{match:[/\x5c\
bclass/,/\x5cs+/,a]\
}],\x0ascope:{1:\x22ke\
yword\x22,3:\x22title.\
class\x22,6:\x22title.\
class.inherited\x22\
}},{\x0aclassName:\x22\
meta\x22,begin:/^[\x5c\
t ]*@/,end:/(?=#\
)|$/,contains:[g\
,m,o]}]}}})()\x0a;h\
ljs.registerLang\
uage(\x22python\x22,e)\
})();/*! `python\
-repl` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var a=\
(()=>{\x22use stric\
t\x22;return a=>({a\
liases:[\x22pycon\x22]\
,contains:[{\x0acla\
ssName:\x22meta.pro\
mpt\x22,starts:{end\
:/ |$/,starts:{e\
nd:\x22$\x22,subLangua\
ge:\x22python\x22}\x0a},v\
ariants:[{begin:\
/^>>>(?=[ ]|$)/}\
,{begin:/^\x5c.\x5c.\x5c.\
(?=[ ]|$)/}]}]})\
})()\x0a;hljs.regis\
terLanguage(\x22pyt\
hon-repl\x22,a)})()\
;/*! `q` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
e=(()=>{\x22use str\
ict\x22;return e=>(\
{name:\x22Q\x22,aliase\
s:[\x22k\x22,\x22kdb\x22],\x0ak\
eywords:{$patter\
n:/(`?)[A-Za-z0-\
9_]+\x5cb/,\x0akeyword\
:\x22do while selec\
t delete by upda\
te from\x22,literal\
:\x220b 1b\x22,\x0abuilt_\
in:\x22neg not null\
 string reciproc\
al floor ceiling\
 signum mod xbar\
 xlog and or eac\
h scan over prio\
r mmu lsq inv md\
5 ltime gtime co\
unt first var de\
v med cov cor al\
l any rand sums \
prds mins maxs f\
ills deltas rati\
os avgs differ p\
rev next rank re\
verse iasc idesc\
 asc desc msum m\
count mavg mdev \
xrank mmin mmax \
xprev rotate dis\
tinct group wher\
e flip type key \
til get value at\
tr cut set upser\
t raze union int\
er except cross \
sv vs sublist en\
list read0 read1\
 hopen hclose hd\
el hsym hcount p\
each system ltri\
m rtrim trim low\
er upper ssr vie\
w tables views c\
ols xcols keys x\
key xcol xasc xd\
esc fkeys meta l\
j aj aj0 ij pj a\
sof uj ww wj wj1\
 fby xgroup ungr\
oup ej save load\
 rsave rload sho\
w csv parse eval\
 min max avg wav\
g wsum sin cos t\
an sum\x22,\x0atype:\x22`\
float `double in\
t `timestamp `ti\
mespan `datetime\
 `time `boolean \
`symbol `char `b\
yte `short `long\
 `real `month `d\
ate `minute `sec\
ond `guid\x22\x0a},con\
tains:[e.C_LINE_\
COMMENT_MODE,e.Q\
UOTE_STRING_MODE\
,e.C_NUMBER_MODE\
]})})()\x0a;hljs.re\
gisterLanguage(\x22\
q\x22,e)})();/*! `q\
ml` grammar comp\
iled for Highlig\
ht.js 11.10.0 */\
\x0a(()=>{var e=(()\
=>{\x22use strict\x22;\
return e=>{\x0acons\
t n=\x22[a-zA-Z_][a\
-zA-Z0-9\x5c\x5c._]*\x22,\
r={className:\x22at\
tribute\x22,begin:\x22\
\x5c\x5cbid\x5c\x5cs*:\x22,\x0asta\
rts:{className:\x22\
string\x22,end:n,re\
turnEnd:!1}},a={\
begin:n+\x22\x5c\x5cs*:\x22,\
\x0areturnBegin:!0,\
contains:[{class\
Name:\x22attribute\x22\
,begin:n,end:\x22\x5c\x5c\
s*:\x22,\x0aexcludeEnd\
:!0,relevance:0}\
],relevance:0},t\
={begin:e.regex.\
concat(n,/\x5cs*\x5c{/\
),\x0aend:/\x5c{/,retu\
rnBegin:!0,relev\
ance:0,contains:\
[e.inherit(e.TIT\
LE_MODE,{begin:n\
})]\x0a};return{nam\
e:\x22QML\x22,aliases:\
[\x22qt\x22],case_inse\
nsitive:!1,keywo\
rds:{\x0akeyword:\x22i\
n of on if for w\
hile finally var\
 new function do\
 return void els\
e break catch in\
stanceof with th\
row case default\
 try this switch\
 continue typeof\
 delete let yiel\
d const export s\
uper debugger as\
 async await imp\
ort\x22,\x0aliteral:\x22t\
rue false null u\
ndefined NaN Inf\
inity\x22,\x0abuilt_in\
:\x22eval isFinite \
isNaN parseFloat\
 parseInt decode\
URI decodeURICom\
ponent encodeURI\
 encodeURICompon\
ent escape unesc\
ape Object Funct\
ion Boolean Erro\
r EvalError Inte\
rnalError RangeE\
rror ReferenceEr\
ror StopIteratio\
n SyntaxError Ty\
peError URIError\
 Number Math Dat\
e String RegExp \
Array Float32Arr\
ay Float64Array \
Int16Array Int32\
Array Int8Array \
Uint16Array Uint\
32Array Uint8Arr\
ay Uint8ClampedA\
rray ArrayBuffer\
 DataView JSON I\
ntl arguments re\
quire module con\
sole window docu\
ment Symbol Set \
Map WeakSet Weak\
Map Proxy Reflec\
t Behavior bool \
color coordinate\
 date double enu\
meration font ge\
ocircle georecta\
ngle geoshape in\
t list matrix4x4\
 parent point qu\
aternion real re\
ct size string u\
rl variant vecto\
r2d vector3d vec\
tor4d Promise\x22\x0a}\
,contains:[{clas\
sName:\x22meta\x22,beg\
in:/^\x5cs*['\x22]use \
(strict|asm)['\x22]\
/\x0a},e.APOS_STRIN\
G_MODE,e.QUOTE_S\
TRING_MODE,{clas\
sName:\x22string\x22,b\
egin:\x22`\x22,end:\x22`\x22\
,\x0acontains:[e.BA\
CKSLASH_ESCAPE,{\
className:\x22subst\
\x22,begin:\x22\x5c\x5c$\x5c\x5c{\x22\
,end:\x22\x5c\x5c}\x22}]\x0a},e\
.C_LINE_COMMENT_\
MODE,e.C_BLOCK_C\
OMMENT_MODE,{cla\
ssName:\x22number\x22,\
variants:[{\x0abegi\
n:\x22\x5c\x5cb(0[bB][01]\
+)\x22},{begin:\x22\x5c\x5cb\
(0[oO][0-7]+)\x22},\
{begin:e.C_NUMBE\
R_RE}],\x0arelevanc\
e:0},{begin:\x22(\x22+\
e.RE_STARTERS_RE\
+\x22|\x5c\x5cb(case|retu\
rn|throw)\x5c\x5cb)\x5c\x5cs\
*\x22,\x0akeywords:\x22re\
turn throw case\x22\
,\x0acontains:[e.C_\
LINE_COMMENT_MOD\
E,e.C_BLOCK_COMM\
ENT_MODE,e.REGEX\
P_MODE,{begin:/<\
/,\x0aend:/>\x5cs*[);\x5c\
]]/,relevance:0,\
subLanguage:\x22xml\
\x22}],relevance:0}\
,{\x0aclassName:\x22ke\
yword\x22,begin:\x22\x5c\x5c\
bsignal\x5c\x5cb\x22,star\
ts:{className:\x22s\
tring\x22,\x0aend:\x22(\x5c\x5c\
(|:|=|;|,|//|/\x5c\x5c\
*|$)\x22,returnEnd:\
!0}},{className:\
\x22keyword\x22,\x0abegin\
:\x22\x5c\x5cbproperty\x5c\x5cb\
\x22,starts:{classN\
ame:\x22string\x22,end\
:\x22(:|=|;|,|//|/\x5c\
\x5c*|$)\x22,\x0areturnEn\
d:!0}},{classNam\
e:\x22function\x22,beg\
inKeywords:\x22func\
tion\x22,end:/\x5c{/,\x0a\
excludeEnd:!0,co\
ntains:[e.inheri\
t(e.TITLE_MODE,{\
begin:/[A-Za-z$_\
][0-9A-Za-z$_]*/\
\x0a}),{className:\x22\
params\x22,begin:/\x5c\
(/,end:/\x5c)/,excl\
udeBegin:!0,excl\
udeEnd:!0,\x0aconta\
ins:[e.C_LINE_CO\
MMENT_MODE,e.C_B\
LOCK_COMMENT_MOD\
E]}],illegal:/\x5c[\
|%/},{\x0abegin:\x22\x5c\x5c\
.\x22+e.IDENT_RE,re\
levance:0},r,a,t\
],illegal:/#/}}}\
)()\x0a;hljs.regist\
erLanguage(\x22qml\x22\
,e)})();/*! `r` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{\x0aconst a=\
e.regex,n=/(?:(?\
:[a-zA-Z]|\x5c.[._a\
-zA-Z])[._a-zA-Z\
0-9]*)|\x5c.(?!\x5cd)/\
,i=a.either(/0[x\
X][0-9a-fA-F]+\x5c.\
[0-9a-fA-F]*[pP]\
[+-]?\x5cd+i?/,/0[x\
X][0-9a-fA-F]+(?\
:[pP][+-]?\x5cd+)?[\
Li]?/,/(?:\x5cd+(?:\
\x5c.\x5cd*)?|\x5c.\x5cd+)(?\
:[eE][+-]?\x5cd+)?[\
Li]?/),s=/[=!<>:\
]=|\x5c|\x5c||&&|:::?|\
<-|<<-|->>|->|\x5c|\
>|[-+*\x5c/?!$&|:<=\
>@^~]|\x5c*\x5c*/,t=a.\
either(/[()]/,/[\
{}]/,/\x5c[\x5c[/,/[[\x5c\
]]/,/\x5c\x5c/,/,/)\x0a;r\
eturn{name:\x22R\x22,k\
eywords:{$patter\
n:n,\x0akeyword:\x22fu\
nction if in bre\
ak next repeat e\
lse for while\x22,\x0a\
literal:\x22NULL NA\
 TRUE FALSE Inf \
NaN NA_integer_|\
10 NA_real_|10 N\
A_character_|10 \
NA_complex_|10\x22,\
\x0abuilt_in:\x22LETTE\
RS letters month\
.abb month.name \
pi T F abs acos \
acosh all any an\
yNA Arg as.call \
as.character as.\
complex as.doubl\
e as.environment\
 as.integer as.l\
ogical as.null.d\
efault as.numeri\
c as.raw asin as\
inh atan atanh a\
ttr attributes b\
aseenv browser c\
 call ceiling cl\
ass Conj cos cos\
h cospi cummax c\
ummin cumprod cu\
msum digamma dim\
 dimnames emptye\
nv exp expressio\
n floor forceAnd\
Call gamma gc.ti\
me globalenv Im \
interactive invi\
sible is.array i\
s.atomic is.call\
 is.character is\
.complex is.doub\
le is.environmen\
t is.expression \
is.finite is.fun\
ction is.infinit\
e is.integer is.\
language is.list\
 is.logical is.m\
atrix is.na is.n\
ame is.nan is.nu\
ll is.numeric is\
.object is.pairl\
ist is.raw is.re\
cursive is.singl\
e is.symbol lazy\
LoadDBfetch leng\
th lgamma list l\
og max min missi\
ng Mod names nar\
gs nzchar oldCla\
ss on.exit pos.t\
o.env proc.time \
prod quote range\
 Re rep retracem\
em return round \
seq_along seq_le\
n seq.int sign s\
ignif sin sinh s\
inpi sqrt standa\
rdGeneric substi\
tute sum switch \
tan tanh tanpi t\
racemem trigamma\
 trunc unclass u\
ntracemem UseMet\
hod xtfrm\x22\x0a},con\
tains:[e.COMMENT\
(/#'/,/$/,{conta\
ins:[{scope:\x22doc\
tag\x22,match:/@exa\
mples/,\x0astarts:{\
end:a.lookahead(\
a.either(/\x5cn^#'\x5c\
s*(?=@[a-zA-Z]+)\
/,/\x5cn^(?!#')/)),\
\x0aendsParent:!0}}\
,{scope:\x22doctag\x22\
,begin:\x22@param\x22,\
end:/$/,contains\
:[{\x0ascope:\x22varia\
ble\x22,variants:[{\
match:n},{match:\
/`(?:\x5c\x5c.|[^`\x5c\x5c])\
+`/}],endsParent\
:!0\x0a}]},{scope:\x22\
doctag\x22,match:/@\
[a-zA-Z]+/},{sco\
pe:\x22keyword\x22,mat\
ch:/\x5c\x5c[a-zA-Z]+/\
}]\x0a}),e.HASH_COM\
MENT_MODE,{scope\
:\x22string\x22,contai\
ns:[e.BACKSLASH_\
ESCAPE],\x0avariant\
s:[e.END_SAME_AS\
_BEGIN({begin:/[\
rR]\x22(-*)\x5c(/,end:\
/\x5c)(-*)\x22/\x0a}),e.E\
ND_SAME_AS_BEGIN\
({begin:/[rR]\x22(-\
*)\x5c{/,end:/\x5c}(-*\
)\x22/\x0a}),e.END_SAM\
E_AS_BEGIN({begi\
n:/[rR]\x22(-*)\x5c[/,\
end:/\x5c](-*)\x22/\x0a})\
,e.END_SAME_AS_B\
EGIN({begin:/[rR\
]'(-*)\x5c(/,end:/\x5c\
)(-*)'/\x0a}),e.END\
_SAME_AS_BEGIN({\
begin:/[rR]'(-*)\
\x5c{/,end:/\x5c}(-*)'\
/\x0a}),e.END_SAME_\
AS_BEGIN({begin:\
/[rR]'(-*)\x5c[/,en\
d:/\x5c](-*)'/}),{b\
egin:'\x22',end:'\x22'\
,\x0arelevance:0},{\
begin:\x22'\x22,end:\x22'\
\x22,relevance:0}]}\
,{relevance:0,va\
riants:[{scope:{\
\x0a1:\x22operator\x22,2:\
\x22number\x22},match:\
[s,i]},{scope:{1\
:\x22operator\x22,2:\x22n\
umber\x22},\x0amatch:[\
/%[^%]*%/,i]},{s\
cope:{1:\x22punctua\
tion\x22,2:\x22number\x22\
},match:[t,i]},{\
scope:{\x0a2:\x22numbe\
r\x22},match:[/[^a-\
zA-Z0-9._]|^/,i]\
}]},{scope:{3:\x22o\
perator\x22},\x0amatch\
:[n,/\x5cs+/,/<-/,/\
\x5cs+/]},{scope:\x22o\
perator\x22,relevan\
ce:0,variants:[{\
match:s},{\x0amatch\
:/%[^%]*%/}]},{s\
cope:\x22punctuatio\
n\x22,relevance:0,m\
atch:t},{begin:\x22\
`\x22,end:\x22`\x22,\x0acont\
ains:[{begin:/\x5c\x5c\
./}]}]}}})();hlj\
s.registerLangua\
ge(\x22r\x22,e)})();/*\
! `reasonml` gra\
mmar compiled fo\
r Highlight.js 1\
1.10.0 */\x0a(()=>{\
var e=(()=>{\x22use\
 strict\x22;return \
e=>({name:\x22Reaso\
nML\x22,aliases:[\x22r\
e\x22],\x0akeywords:{$\
pattern:/[a-z_]\x5c\
w*!?/,\x0akeyword:[\
\x22and\x22,\x22as\x22,\x22asr\x22\
,\x22assert\x22,\x22begin\
\x22,\x22class\x22,\x22const\
raint\x22,\x22do\x22,\x22don\
e\x22,\x22downto\x22,\x22els\
e\x22,\x22end\x22,\x22esfun\x22\
,\x22exception\x22,\x22ex\
ternal\x22,\x22for\x22,\x22f\
un\x22,\x22function\x22,\x22\
functor\x22,\x22if\x22,\x22i\
n\x22,\x22include\x22,\x22in\
herit\x22,\x22initiali\
zer\x22,\x22land\x22,\x22laz\
y\x22,\x22let\x22,\x22lor\x22,\x22\
lsl\x22,\x22lsr\x22,\x22lxor\
\x22,\x22mod\x22,\x22module\x22\
,\x22mutable\x22,\x22new\x22\
,\x22nonrec\x22,\x22objec\
t\x22,\x22of\x22,\x22open\x22,\x22\
or\x22,\x22pri\x22,\x22pub\x22,\
\x22rec\x22,\x22sig\x22,\x22str\
uct\x22,\x22switch\x22,\x22t\
hen\x22,\x22to\x22,\x22try\x22,\
\x22type\x22,\x22val\x22,\x22vi\
rtual\x22,\x22when\x22,\x22w\
hile\x22,\x22with\x22],\x0ab\
uilt_in:[\x22array\x22\
,\x22bool\x22,\x22bytes\x22,\
\x22char\x22,\x22exn|5\x22,\x22\
float\x22,\x22int\x22,\x22in\
t32\x22,\x22int64\x22,\x22li\
st\x22,\x22lazy_t|5\x22,\x22\
nativeint|5\x22,\x22re\
f\x22,\x22string\x22,\x22uni\
t\x22],\x0aliteral:[\x22t\
rue\x22,\x22false\x22]},i\
llegal:/(:-|:=|\x5c\
$\x5c{|\x5c+=)/,contai\
ns:[{scope:\x22lite\
ral\x22,\x0amatch:/\x5c[(\
\x5c|\x5c|)?\x5c]|\x5c(\x5c)/,r\
elevance:0\x0a},e.C\
_LINE_COMMENT_MO\
DE,e.COMMENT(/\x5c/\
\x5c*/,/\x5c*\x5c//,{ille\
gal:/^(#,\x5c/\x5c/)/}\
),{\x0ascope:\x22symbo\
l\x22,match:/\x5c'[A-Z\
a-z_](?!\x5c')[\x5cw\x5c'\
]*/},{scope:\x22typ\
e\x22,\x0amatch:/`[A-Z\
][\x5cw\x5c']*/},{scop\
e:\x22type\x22,match:/\
\x5cb[A-Z][\x5cw\x5c']*/,\
relevance:0},{\x0am\
atch:/[a-z_]\x5cw*\x5c\
'[\x5cw\x5c']*/,releva\
nce:0},{scope:\x22o\
perator\x22,\x0amatch:\
/\x5cs+(\x5c|\x5c||\x5c+[\x5c+\x5c\
.]?|\x5c*[\x5c*\x5c/\x5c.]?|\
\x5c/[\x5c.]?|\x5c.\x5c.\x5c.|\x5c\
|>|&&|===?)\x5cs+/,\
\x0arelevance:0},e.\
inherit(e.APOS_S\
TRING_MODE,{scop\
e:\x22string\x22,relev\
ance:0\x0a}),e.inhe\
rit(e.QUOTE_STRI\
NG_MODE,{illegal\
:null}),{scope:\x22\
number\x22,variants\
:[{\x0amatch:/\x5cb0[x\
X][a-fA-F0-9_]+[\
Lln]?/},{match:/\
\x5cb0[oO][0-7_]+[L\
ln]?/},{\x0amatch:/\
\x5cb0[bB][01_]+[Ll\
n]?/},{\x0amatch:/\x5c\
b[0-9][0-9_]*([L\
ln]|(\x5c.[0-9_]*)?\
([eE][-+]?[0-9_]\
+)?)/}],relevanc\
e:0}]})\x0a})();hlj\
s.registerLangua\
ge(\x22reasonml\x22,e)\
})();/*! `rib` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>({name:\x22Ren\
derMan RIB\x22,\x0akey\
words:\x22ArchiveRe\
cord AreaLightSo\
urce Atmosphere \
Attribute Attrib\
uteBegin Attribu\
teEnd Basis Begi\
n Blobby Bound C\
lipping Clipping\
Plane Color Colo\
rSamples ConcatT\
ransform Cone Co\
ordinateSystem C\
oordSysTransform\
 CropWindow Curv\
es Cylinder Dept\
hOfField Detail \
DetailRange Disk\
 Displacement Di\
splay End ErrorH\
andler Exposure \
Exterior Format \
FrameAspectRatio\
 FrameBegin Fram\
eEnd GeneralPoly\
gon GeometricApp\
roximation Geome\
try Hider Hyperb\
oloid Identity I\
lluminate Imager\
 Interior LightS\
ource MakeCubeFa\
ceEnvironment Ma\
keLatLongEnviron\
ment MakeShadow \
MakeTexture Matt\
e MotionBegin Mo\
tionEnd NuPatch \
ObjectBegin Obje\
ctEnd ObjectInst\
ance Opacity Opt\
ion Orientation \
Paraboloid Patch\
 PatchMesh Persp\
ective PixelFilt\
er PixelSamples \
PixelVariance Po\
ints PointsGener\
alPolygons Point\
sPolygons Polygo\
n Procedural Pro\
jection Quantize\
 ReadArchive Rel\
ativeDetail Reve\
rseOrientation R\
otate Scale Scre\
enWindow Shading\
Interpolation Sh\
adingRate Shutte\
r Sides Skew Sol\
idBegin SolidEnd\
 Sphere Subdivis\
ionMesh Surface \
TextureCoordinat\
es Torus Transfo\
rm TransformBegi\
n TransformEnd T\
ransformPoints T\
ranslate TrimCur\
ve WorldBegin Wo\
rldEnd\x22,\x0aillegal\
:\x22</\x22,\x0acontains:\
[e.HASH_COMMENT_\
MODE,e.C_NUMBER_\
MODE,e.APOS_STRI\
NG_MODE,e.QUOTE_\
STRING_MODE]\x0a})}\
)();hljs.registe\
rLanguage(\x22rib\x22,\
e)})();/*! `robo\
conf` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>{con\
st n=\x22[a-zA-Z-_]\
[^\x5c\x5cn{]+\x5c\x5c{\x22,a={\
\x0aclassName:\x22attr\
ibute\x22,begin:/[a\
-zA-Z-_]+/,end:/\
\x5cs*:/,excludeEnd\
:!0,starts:{\x0aend\
:\x22;\x22,relevance:0\
,contains:[{clas\
sName:\x22variable\x22\
,begin:/\x5c.[a-zA-\
Z-_]+/},{\x0aclassN\
ame:\x22keyword\x22,be\
gin:/\x5c(optional\x5c\
)/}]}};return{na\
me:\x22Roboconf\x22,\x0aa\
liases:[\x22graph\x22,\
\x22instances\x22],cas\
e_insensitive:!0\
,keywords:\x22impor\
t\x22,contains:[{\x0ab\
egin:\x22^facet \x22+n\
,end:/\x5c}/,keywor\
ds:\x22facet\x22,conta\
ins:[a,e.HASH_CO\
MMENT_MODE]},{\x0ab\
egin:\x22^\x5c\x5cs*insta\
nce of \x22+n,end:/\
\x5c}/,\x0akeywords:\x22n\
ame count channe\
ls instance-data\
 instance-state \
instance of\x22,\x0ail\
legal:/\x5cS/,conta\
ins:[\x22self\x22,a,e.\
HASH_COMMENT_MOD\
E]},{begin:\x22^\x22+n\
,end:/\x5c}/,\x0aconta\
ins:[a,e.HASH_CO\
MMENT_MODE]},e.H\
ASH_COMMENT_MODE\
]}}})()\x0a;hljs.re\
gisterLanguage(\x22\
roboconf\x22,e)})()\
;/*! `routeros` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{\x0aconst r=\
\x22foreach do whil\
e for if from to\
 step else on-er\
ror and or not i\
n\x22,n=\x22true false\
 yes no nothing \
nil null\x22,i={\x0acl\
assName:\x22variabl\
e\x22,variants:[{be\
gin:/\x5c$[\x5cw\x5cd#@][\
\x5cw\x5cd_]*/},{begin\
:/\x5c$\x5c{(.*?)\x5c}/\x0a}\
]},s={className:\
\x22string\x22,begin:/\
\x22/,end:/\x22/,conta\
ins:[e.BACKSLASH\
_ESCAPE,i,{\x0aclas\
sName:\x22variable\x22\
,begin:/\x5c$\x5c(/,en\
d:/\x5c)/,contains:\
[e.BACKSLASH_ESC\
APE]}]},t={\x0aclas\
sName:\x22string\x22,b\
egin:/'/,end:/'/\
};return{name:\x22M\
ikroTik RouterOS\
 script\x22,\x0aaliase\
s:[\x22mikrotik\x22],c\
ase_insensitive:\
!0,keywords:{$pa\
ttern:/:?[\x5cw-]+/\
,\x0aliteral:n,\x0akey\
word:r+\x22 :\x22+r.sp\
lit(\x22 \x22).join(\x22 \
:\x22)+\x22 :\x22+\x22global\
 local beep dela\
y put len typeof\
 pick log time s\
et find environm\
ent terminal err\
or execute parse\
 resolve toarray\
 tobool toid toi\
p toip6 tonum to\
str totime\x22.spli\
t(\x22 \x22).join(\x22 :\x22\
)\x0a},contains:[{v\
ariants:[{begin:\
/\x5c/\x5c*/,end:/\x5c*\x5c/\
/},{begin:/\x5c/\x5c//\
,end:/$/},{\x0abegi\
n:/<\x5c//,end:/>/}\
],illegal:/./},e\
.COMMENT(\x22^#\x22,\x22$\
\x22),s,t,i,{\x0abegin\
:/[\x5cw-]+=([^\x5cs{}\
[\x5c]()>]+)/,relev\
ance:0,returnBeg\
in:!0,contains:[\
{\x0aclassName:\x22att\
ribute\x22,begin:/[\
^=]+/},{begin:/=\
/,endsWithParent\
:!0,relevance:0,\
\x0acontains:[s,t,i\
,{className:\x22lit\
eral\x22,begin:\x22\x5c\x5cb\
(\x22+n.split(\x22 \x22).\
join(\x22|\x22)+\x22)\x5c\x5cb\x22\
\x0a},{begin:/(\x22[^\x22\
]*\x22|[^\x5cs{}[\x5c]]+)\
/}]}]},{classNam\
e:\x22number\x22,begin\
:/\x5c*[0-9a-fA-F]+\
/\x0a},{\x0abegin:\x22\x5c\x5cb\
(add|remove|enab\
le|disable|set|g\
et|print|export|\
edit|find|run|de\
bug|error|info|w\
arning)([\x5c\x5cs[(\x5c\x5c\
]|])\x22,\x0areturnBeg\
in:!0,contains:[\
{className:\x22buil\
t_in\x22,begin:/\x5cw+\
/}]},{\x0aclassName\
:\x22built_in\x22,vari\
ants:[{\x0abegin:\x22(\
\x5c\x5c.\x5c\x5c./|/|\x5c\x5cs)((\
traffic-flow|tra\
ffic-generator|f\
irewall|schedule\
r|aaa|accounting\
|address-list|ad\
dress|align|area\
|bandwidth-serve\
r|bfd|bgp|bridge\
|client|clock|co\
mmunity|config|c\
onnection|consol\
e|customer|defau\
lt|dhcp-client|d\
hcp-server|disco\
very|dns|e-mail|\
ethernet|filter|\
firmware|gps|gra\
phing|group|hard\
ware|health|hots\
pot|identity|igm\
p-proxy|incoming\
|instance|interf\
ace|ip|ipsec|ipv\
6|irq|l2tp-serve\
r|lcd|ldp|loggin\
g|mac-server|mac\
-winbox|mangle|m\
anual|mirror|mme\
|mpls|nat|nd|nei\
ghbor|network|no\
te|ntp|ospf|ospf\
-v3|ovpn-server|\
page|peer|pim|pi\
ng|policy|pool|p\
ort|ppp|pppoe-cl\
ient|pptp-server\
|prefix|profile|\
proposal|proxy|q\
ueue|radius|reso\
urce|rip|ripng|r\
oute|routing|scr\
een|script|secur\
ity-profiles|ser\
ver|service|serv\
ice-port|setting\
s|shares|smb|sms\
|sniffer|snmp|sn\
ooper|socks|sstp\
-server|system|t\
ool|tracking|typ\
e|upgrade|upnp|u\
ser-manager|user\
s|user|vlan|secr\
et|vrrp|watchdog\
|web-access|wire\
less|pptp|pppoe|\
lan|wan|layer7-p\
rotocol|lease|si\
mple|raw);?\x5c\x5cs)+\
\x22\x0a},{begin:/\x5c.\x5c.\
/,relevance:0}]}\
]}}})();hljs.reg\
isterLanguage(\x22r\
outeros\x22,e)})();\
/*! `rsl` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
{const t={\x0amatch\
:[/(surface|disp\
lacement|light|v\
olume|imager)/,/\
\x5cs+/,e.IDENT_RE]\
,scope:{\x0a1:\x22keyw\
ord\x22,3:\x22title.cl\
ass\x22}};return{na\
me:\x22RenderMan RS\
L\x22,keywords:{\x0ake\
yword:[\x22while\x22,\x22\
for\x22,\x22if\x22,\x22do\x22,\x22\
return\x22,\x22else\x22,\x22\
break\x22,\x22extern\x22,\
\x22continue\x22],\x0abui\
lt_in:[\x22abs\x22,\x22ac\
os\x22,\x22ambient\x22,\x22a\
rea\x22,\x22asin\x22,\x22ata\
n\x22,\x22atmosphere\x22,\
\x22attribute\x22,\x22cal\
culatenormal\x22,\x22c\
eil\x22,\x22cellnoise\x22\
,\x22clamp\x22,\x22comp\x22,\
\x22concat\x22,\x22cos\x22,\x22\
degrees\x22,\x22depth\x22\
,\x22Deriv\x22,\x22diffus\
e\x22,\x22distance\x22,\x22D\
u\x22,\x22Dv\x22,\x22environ\
ment\x22,\x22exp\x22,\x22fac\
eforward\x22,\x22filte\
rstep\x22,\x22floor\x22,\x22\
format\x22,\x22fresnel\
\x22,\x22incident\x22,\x22le\
ngth\x22,\x22lightsour\
ce\x22,\x22log\x22,\x22match\
\x22,\x22max\x22,\x22min\x22,\x22m\
od\x22,\x22noise\x22,\x22nor\
malize\x22,\x22ntransf\
orm\x22,\x22opposite\x22,\
\x22option\x22,\x22phong\x22\
,\x22pnoise\x22,\x22pow\x22,\
\x22printf\x22,\x22ptline\
d\x22,\x22radians\x22,\x22ra\
ndom\x22,\x22reflect\x22,\
\x22refract\x22,\x22rende\
rinfo\x22,\x22round\x22,\x22\
setcomp\x22,\x22setxco\
mp\x22,\x22setycomp\x22,\x22\
setzcomp\x22,\x22shado\
w\x22,\x22sign\x22,\x22sin\x22,\
\x22smoothstep\x22,\x22sp\
ecular\x22,\x22specula\
rbrdf\x22,\x22spline\x22,\
\x22sqrt\x22,\x22step\x22,\x22t\
an\x22,\x22texture\x22,\x22t\
extureinfo\x22,\x22tra\
ce\x22,\x22transform\x22,\
\x22vtransform\x22,\x22xc\
omp\x22,\x22ycomp\x22,\x22zc\
omp\x22],\x0atype:[\x22ma\
trix\x22,\x22float\x22,\x22c\
olor\x22,\x22point\x22,\x22n\
ormal\x22,\x22vector\x22]\
},illegal:\x22</\x22,\x0a\
contains:[e.C_LI\
NE_COMMENT_MODE,\
e.C_BLOCK_COMMEN\
T_MODE,e.QUOTE_S\
TRING_MODE,e.APO\
S_STRING_MODE,e.\
C_NUMBER_MODE,{\x0a\
className:\x22meta\x22\
,begin:\x22#\x22,end:\x22\
$\x22},t,{\x0abeginKey\
words:\x22illuminat\
e illuminance ga\
ther\x22,end:\x22\x5c\x5c(\x22}\
]}}})()\x0a;hljs.re\
gisterLanguage(\x22\
rsl\x22,e)})();/*! \
`ruby` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;return e=>{\x0ac\
onst n=e.regex,a\
=\x22([a-zA-Z_]\x5c\x5cw*\
[!?=]?|[-+~]@|<<\
|>>|=~|===?|<=>|\
[<>]=?|\x5c\x5c*\x5c\x5c*|[-\
/+%^&*~`|]|\x5c\x5c[\x5c\x5c\
]=?)\x22,s=n.either\
(/\x5cb([A-Z]+[a-z0\
-9]+)+/,/\x5cb([A-Z\
]+[a-z0-9]+)+[A-\
Z]+/),i=n.concat\
(s,/(::\x5cw+)*/),t\
={\x0a\x22variable.con\
stant\x22:[\x22__FILE_\
_\x22,\x22__LINE__\x22,\x22_\
_ENCODING__\x22],\x0a\x22\
variable.languag\
e\x22:[\x22self\x22,\x22supe\
r\x22],\x0akeyword:[\x22a\
lias\x22,\x22and\x22,\x22beg\
in\x22,\x22BEGIN\x22,\x22bre\
ak\x22,\x22case\x22,\x22clas\
s\x22,\x22defined\x22,\x22do\
\x22,\x22else\x22,\x22elsif\x22\
,\x22end\x22,\x22END\x22,\x22en\
sure\x22,\x22for\x22,\x22if\x22\
,\x22in\x22,\x22module\x22,\x22\
next\x22,\x22not\x22,\x22or\x22\
,\x22redo\x22,\x22require\
\x22,\x22rescue\x22,\x22retr\
y\x22,\x22return\x22,\x22the\
n\x22,\x22undef\x22,\x22unle\
ss\x22,\x22until\x22,\x22whe\
n\x22,\x22while\x22,\x22yiel\
d\x22,\x22include\x22,\x22ex\
tend\x22,\x22prepend\x22,\
\x22public\x22,\x22privat\
e\x22,\x22protected\x22,\x22\
raise\x22,\x22throw\x22],\
\x0abuilt_in:[\x22proc\
\x22,\x22lambda\x22,\x22attr\
_accessor\x22,\x22attr\
_reader\x22,\x22attr_w\
riter\x22,\x22define_m\
ethod\x22,\x22private_\
constant\x22,\x22modul\
e_function\x22],\x0ali\
teral:[\x22true\x22,\x22f\
alse\x22,\x22nil\x22]},c=\
{className:\x22doct\
ag\x22,begin:\x22@[A-Z\
a-z]+\x22},r={\x0abegi\
n:\x22#<\x22,end:\x22>\x22},\
b=[e.COMMENT(\x22#\x22\
,\x22$\x22,{contains:[\
c]\x0a}),e.COMMENT(\
\x22^=begin\x22,\x22^=end\
\x22,{contains:[c],\
relevance:10\x0a}),\
e.COMMENT(\x22^__EN\
D__\x22,e.MATCH_NOT\
HING_RE)],l={cla\
ssName:\x22subst\x22,b\
egin:/#\x5c{/,\x0aend:\
/\x5c}/,keywords:t}\
,d={className:\x22s\
tring\x22,contains:\
[e.BACKSLASH_ESC\
APE,l],\x0avariants\
:[{begin:/'/,end\
:/'/},{begin:/\x22/\
,end:/\x22/},{begin\
:/`/,end:/`/},{\x0a\
begin:/%[qQwWx]?\
\x5c(/,end:/\x5c)/},{b\
egin:/%[qQwWx]?\x5c\
[/,end:/\x5c]/},{\x0ab\
egin:/%[qQwWx]?\x5c\
{/,end:/\x5c}/},{be\
gin:/%[qQwWx]?</\
,end:/>/},{begin\
:/%[qQwWx]?\x5c//,\x0a\
end:/\x5c//},{begin\
:/%[qQwWx]?%/,en\
d:/%/},{begin:/%\
[qQwWx]?-/,end:/\
-/},{\x0abegin:/%[q\
QwWx]?\x5c|/,end:/\x5c\
|/},{begin:/\x5cB\x5c?\
(\x5c\x5c\x5cd{1,3})/},{\x0a\
begin:/\x5cB\x5c?(\x5c\x5cx[\
A-Fa-f0-9]{1,2})\
/},{begin:/\x5cB\x5c?(\
\x5c\x5cu\x5c{?[A-Fa-f0-9\
]{1,6}\x5c}?)/},{\x0ab\
egin:/\x5cB\x5c?(\x5c\x5cM-\x5c\
\x5cC-|\x5c\x5cM-\x5c\x5cc|\x5c\x5cc\x5c\
\x5cM-|\x5c\x5cM-|\x5c\x5cC-\x5c\x5cM\
-)[\x5cx20-\x5cx7e]/},\
{\x0abegin:/\x5cB\x5c?\x5c\x5c(\
c|C-)[\x5cx20-\x5cx7e]\
/},{begin:/\x5cB\x5c?\x5c\
\x5c?\x5cS/},{\x0abegin:n\
.concat(/<<[-~]?\
'?/,n.lookahead(\
/(\x5cw+)(?=\x5cW)[^\x5cn\
]*\x5cn(?:[^\x5cn]*\x5cn)\
*?\x5cs*\x5c1\x5cb/)),\x0aco\
ntains:[e.END_SA\
ME_AS_BEGIN({beg\
in:/(\x5cw+)/,end:/\
(\x5cw+)/,\x0acontains\
:[e.BACKSLASH_ES\
CAPE,l]})]}]},o=\
\x22[0-9](_?[0-9])*\
\x22,g={className:\x22\
number\x22,\x0arelevan\
ce:0,variants:[{\
\x0abegin:`\x5c\x5cb([1-9\
](_?[0-9])*|0)(\x5c\
\x5c.(${o}))?([eE][\
+-]?(${o})|r)?i?\
\x5c\x5cb`},{\x0abegin:\x22\x5c\
\x5cb0[dD][0-9](_?[\
0-9])*r?i?\x5c\x5cb\x22},\
{begin:\x22\x5c\x5cb0[bB]\
[0-1](_?[0-1])*r\
?i?\x5c\x5cb\x22\x0a},{begin\
:\x22\x5c\x5cb0[oO][0-7](\
_?[0-7])*r?i?\x5c\x5cb\
\x22},{\x0abegin:\x22\x5c\x5cb0\
[xX][0-9a-fA-F](\
_?[0-9a-fA-F])*r\
?i?\x5c\x5cb\x22},{\x0abegin\
:\x22\x5c\x5cb0(_?[0-7])+\
r?i?\x5c\x5cb\x22}]},_={v\
ariants:[{match:\
/\x5c(\x5c)/},{\x0aclassN\
ame:\x22params\x22,beg\
in:/\x5c(/,end:/(?=\
\x5c))/,excludeBegi\
n:!0,endsParent:\
!0,\x0akeywords:t}]\
},u=[d,{variants\
:[{match:[/class\
\x5cs+/,i,/\x5cs+<\x5cs+/\
,i]},{\x0amatch:[/\x5c\
b(class|module)\x5c\
s+/,i]}],scope:{\
2:\x22title.class\x22,\
\x0a4:\x22title.class.\
inherited\x22},keyw\
ords:t},{match:[\
/(include|extend\
)\x5cs+/,i],scope:{\
\x0a2:\x22title.class\x22\
},keywords:t},{r\
elevance:0,match\
:[i,/\x5c.new[. (]/\
],scope:{\x0a1:\x22tit\
le.class\x22}},{rel\
evance:0,match:/\
\x5cb[A-Z][A-Z_0-9]\
+\x5cb/,\x0aclassName:\
\x22variable.consta\
nt\x22},{relevance:\
0,match:s,scope:\
\x22title.class\x22},{\
\x0amatch:[/def/,/\x5c\
s+/,a],scope:{1:\
\x22keyword\x22,3:\x22tit\
le.function\x22},co\
ntains:[_]},{\x0abe\
gin:e.IDENT_RE+\x22\
::\x22},{className:\
\x22symbol\x22,\x0abegin:\
e.UNDERSCORE_IDE\
NT_RE+\x22(!|\x5c\x5c?)?:\
\x22,relevance:0},{\
className:\x22symbo\
l\x22,\x0abegin:\x22:(?!\x5c\
\x5cs)\x22,contains:[d\
,{begin:a}],rele\
vance:0},g,{clas\
sName:\x22variable\x22\
,\x0abegin:\x22(\x5c\x5c$\x5c\x5cW\
)|((\x5c\x5c$|@@?)(\x5c\x5cw\
+))(?=[^@$?])(?!\
[A-Za-z])(?![@$?\
'])\x22},{\x0aclassNam\
e:\x22params\x22,begin\
:/\x5c|/,end:/\x5c|/,e\
xcludeBegin:!0,e\
xcludeEnd:!0,\x0are\
levance:0,keywor\
ds:t},{begin:\x22(\x22\
+e.RE_STARTERS_R\
E+\x22|unless)\x5c\x5cs*\x22\
,\x0akeywords:\x22unle\
ss\x22,contains:[{c\
lassName:\x22regexp\
\x22,contains:[e.BA\
CKSLASH_ESCAPE,l\
],\x0aillegal:/\x5cn/,\
variants:[{begin\
:\x22/\x22,end:\x22/[a-z]\
*\x22},{begin:/%r\x5c{\
/,end:/\x5c}[a-z]*/\
},{\x0abegin:\x22%r\x5c\x5c(\
\x22,end:\x22\x5c\x5c)[a-z]*\
\x22},{begin:\x22%r!\x22,\
end:\x22![a-z]*\x22},{\
begin:\x22%r\x5c\x5c[\x22,\x0ae\
nd:\x22\x5c\x5c][a-z]*\x22}]\
}].concat(r,b),r\
elevance:0}].con\
cat(r,b)\x0a;l.cont\
ains=u,_.contain\
s=u;const m=[{be\
gin:/^\x5cs*=>/,sta\
rts:{end:\x22$\x22,con\
tains:u}\x0a},{clas\
sName:\x22meta.prom\
pt\x22,\x0abegin:\x22^([>\
?]>|[\x5c\x5cw#]+\x5c\x5c(\x5c\x5c\
w+\x5c\x5c):\x5c\x5cd+:\x5c\x5cd+[\
>*]|(\x5c\x5cw+-)?\x5c\x5cd+\
\x5c\x5c.\x5c\x5cd+\x5c\x5c.\x5c\x5cd+(p\
\x5c\x5cd+)?[^\x5c\x5cd][^>]\
+>)(?=[ ])\x22,\x0asta\
rts:{end:\x22$\x22,key\
words:t,contains\
:u}}];return b.u\
nshift(r),{name:\
\x22Ruby\x22,\x0aaliases:\
[\x22rb\x22,\x22gemspec\x22,\
\x22podspec\x22,\x22thor\x22\
,\x22irb\x22],keywords\
:t,illegal:/\x5c/\x5c*\
/,\x0acontains:[e.S\
HEBANG({binary:\x22\
ruby\x22})].concat(\
m).concat(b).con\
cat(u)}}})()\x0a;hl\
js.registerLangu\
age(\x22ruby\x22,e)})(\
);/*! `ruleslang\
uage` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var T=(\
()=>{\x22use strict\
\x22;return T=>({na\
me:\x22Oracle Rules\
 Language\x22,\x0akeyw\
ords:{\x0akeyword:\x22\
BILL_PERIOD BILL\
_START BILL_STOP\
 RS_EFFECTIVE_ST\
ART RS_EFFECTIVE\
_STOP RS_JURIS_C\
ODE RS_OPCO_CODE\
 INTDADDATTRIBUT\
E|5 INTDADDVMSG|\
5 INTDBLOCKOP|5 \
INTDBLOCKOPNA|5 \
INTDCLOSE|5 INTD\
COUNT|5 INTDCOUN\
TSTATUSCODE|5 IN\
TDCREATEMASK|5 I\
NTDCREATEDAYMASK\
|5 INTDCREATEFAC\
TORMASK|5 INTDCR\
EATEHANDLE|5 INT\
DCREATEOVERRIDED\
AYMASK|5 INTDCRE\
ATEOVERRIDEMASK|\
5 INTDCREATESTAT\
USCODEMASK|5 INT\
DCREATETOUPERIOD\
|5 INTDDELETE|5 \
INTDDIPTEST|5 IN\
TDEXPORT|5 INTDG\
ETERRORCODE|5 IN\
TDGETERRORMESSAG\
E|5 INTDISEQUAL|\
5 INTDJOIN|5 INT\
DLOAD|5 INTDLOAD\
ACTUALCUT|5 INTD\
LOADDATES|5 INTD\
LOADHIST|5 INTDL\
OADLIST|5 INTDLO\
ADLISTDATES|5 IN\
TDLOADLISTENERGY\
|5 INTDLOADLISTH\
IST|5 INTDLOADRE\
LATEDCHANNEL|5 I\
NTDLOADSP|5 INTD\
LOADSTAGING|5 IN\
TDLOADUOM|5 INTD\
LOADUOMDATES|5 I\
NTDLOADUOMHIST|5\
 INTDLOADVERSION\
|5 INTDOPEN|5 IN\
TDREADFIRST|5 IN\
TDREADNEXT|5 INT\
DRECCOUNT|5 INTD\
RELEASE|5 INTDRE\
PLACE|5 INTDROLL\
AVG|5 INTDROLLPE\
AK|5 INTDSCALARO\
P|5 INTDSCALE|5 \
INTDSETATTRIBUTE\
|5 INTDSETDSTPAR\
TICIPANT|5 INTDS\
ETSTRING|5 INTDS\
ETVALUE|5 INTDSE\
TVALUESTATUS|5 I\
NTDSHIFTSTARTTIM\
E|5 INTDSMOOTH|5\
 INTDSORT|5 INTD\
SPIKETEST|5 INTD\
SUBSET|5 INTDTOU\
|5 INTDTOURELEAS\
E|5 INTDTOUVALUE\
|5 INTDUPDATESTA\
TS|5 INTDVALUE|5\
 STDEV INTDDELET\
EEX|5 INTDLOADEX\
ACTUAL|5 INTDLOA\
DEXCUT|5 INTDLOA\
DEXDATES|5 INTDL\
OADEX|5 INTDLOAD\
EXRELATEDCHANNEL\
|5 INTDSAVEEX|5 \
MVLOAD|5 MVLOADA\
CCT|5 MVLOADACCT\
DATES|5 MVLOADAC\
CTHIST|5 MVLOADD\
ATES|5 MVLOADHIS\
T|5 MVLOADLIST|5\
 MVLOADLISTDATES\
|5 MVLOADLISTHIS\
T|5 IF FOR NEXT \
DONE SELECT END \
CALL ABORT CLEAR\
 CHANNEL FACTOR \
LIST NUMBER OVER\
RIDE SET WEEK DI\
STRIBUTIONNODE E\
LSE WHEN THEN OT\
HERWISE IENUM CS\
V INCLUDE LEAVE \
RIDER SAVE DELET\
E NOVALUE SECTIO\
N WARN SAVE_UPDA\
TE DETERMINANT L\
ABEL REPORT REVE\
NUE EACH IN FROM\
 TOTAL CHARGE BL\
OCK AND OR CSV_F\
ILE RATE_CODE AU\
XILIARY_DEMAND U\
IDACCOUNT RS BIL\
L_PERIOD_SELECT \
HOURS_PER_MONTH \
INTD_ERROR_STOP \
SEASON_SCHEDULE_\
NAME ACCOUNTFACT\
OR ARRAYUPPERBOU\
ND CALLSTOREDPRO\
C GETADOCONNECTI\
ON GETCONNECT GE\
TDATASOURCE GETQ\
UALIFIER GETUSER\
ID HASVALUE LIST\
COUNT LISTOP LIS\
TUPDATE LISTVALU\
E PRORATEFACTOR \
RSPRORATE SETBIN\
PATH SETDBMONITO\
R WQ_OPEN BILLIN\
GHOURS DATE DATE\
FROMFLOAT DATETI\
MEFROMSTRING DAT\
ETIMETOSTRING DA\
TETOFLOAT DAY DA\
YDIFF DAYNAME DB\
DATETIME HOUR MI\
NUTE MONTH MONTH\
DIFF MONTHHOURS \
MONTHNAME ROUNDD\
ATE SAMEWEEKDAYL\
ASTYEAR SECOND W\
EEKDAY WEEKDIFF \
YEAR YEARDAY YEA\
RSTR COMPSUM HIS\
TCOUNT HISTMAX H\
ISTMIN HISTMINNZ\
 HISTVALUE MAXNR\
ANGE MAXRANGE MI\
NRANGE COMPIKVA \
COMPKVA COMPKVAR\
FROMKQKW COMPLF \
IDATTR FLAG LF2K\
W LF2KWH MAXKW P\
OWERFACTOR READI\
NG2USAGE AVGSEAS\
ON MAXSEASON MON\
THLYMERGE SEASON\
VALUE SUMSEASON \
ACCTREADDATES AC\
CTTABLELOAD CONF\
IGADD CONFIGGET \
CREATEOBJECT CRE\
ATEREPORT EMAILC\
LIENT EXPBLKMDMU\
SAGE EXPMDMUSAGE\
 EXPORT_USAGE FA\
CTORINEFFECT GET\
USERSPECIFIEDSTO\
P INEFFECT ISHOL\
IDAY RUNRATE SAV\
E_PROFILE SETREP\
ORTTITLE USEREXI\
T WATFORRUNRATE \
TO TABLE ACOS AS\
IN ATAN ATAN2 BI\
TAND CEIL COS CO\
SECANT COSH COTA\
NGENT DIVQUOT DI\
VREM EXP FABS FL\
OOR FMOD FREPM F\
REXPN LOG LOG10 \
MAX MAXN MIN MIN\
NZ MODF POW ROUN\
D ROUND2VALUE RO\
UNDINT SECANT SI\
N SINH SQROOT TA\
N TANH FLOAT2STR\
ING FLOAT2STRING\
NC INSTR LEFT LE\
N LTRIM MID RIGH\
T RTRIM STRING S\
TRINGNC TOLOWER \
TOUPPER TRIM NUM\
DAYS READ_DATE S\
TAGING\x22,\x0abuilt_i\
n:\x22IDENTIFIER OP\
TIONS XML_ELEMEN\
T XML_OP XML_ELE\
MENT_OF DOMDOCCR\
EATE DOMDOCLOADF\
ILE DOMDOCLOADXM\
L DOMDOCSAVEFILE\
 DOMDOCGETROOT D\
OMDOCADDPI DOMNO\
DEGETNAME DOMNOD\
EGETTYPE DOMNODE\
GETVALUE DOMNODE\
GETCHILDCT DOMNO\
DEGETFIRSTCHILD \
DOMNODEGETSIBLIN\
G DOMNODECREATEC\
HILDELEMENT DOMN\
ODESETATTRIBUTE \
DOMNODEGETCHILDE\
LEMENTCT DOMNODE\
GETFIRSTCHILDELE\
MENT DOMNODEGETS\
IBLINGELEMENT DO\
MNODEGETATTRIBUT\
ECT DOMNODEGETAT\
TRIBUTEI DOMNODE\
GETATTRIBUTEBYNA\
ME DOMNODEGETBYN\
AME\x22\x0a},\x0acontains\
:[T.C_LINE_COMME\
NT_MODE,T.C_BLOC\
K_COMMENT_MODE,T\
.APOS_STRING_MOD\
E,T.QUOTE_STRING\
_MODE,T.C_NUMBER\
_MODE,{\x0aclassNam\
e:\x22literal\x22,vari\
ants:[{begin:\x22#\x5c\
\x5cs+\x22,relevance:0\
},{begin:\x22#[a-zA\
-Z .]+\x22\x0a}]}]})})\
();hljs.register\
Language(\x22rulesl\
anguage\x22,T)})();\
/*! `rust` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{\x0aconst t=e.reg\
ex,n=/(r#)?/,a=t\
.concat(n,e.UNDE\
RSCORE_IDENT_RE)\
,i=t.concat(n,e.\
IDENT_RE),r={\x0acl\
assName:\x22title.f\
unction.invoke\x22,\
relevance:0,\x0abeg\
in:t.concat(/\x5cb/\
,/(?!let|for|whi\
le|if|else|match\
\x5cb)/,i,t.lookahe\
ad(/\x5cs*\x5c(/))\x0a},s\
=\x22([ui](8|16|32|\
64|128|size)|f(3\
2|64))?\x22,l=[\x22dro\
p \x22,\x22Copy\x22,\x22Send\
\x22,\x22Sized\x22,\x22Sync\x22\
,\x22Drop\x22,\x22Fn\x22,\x22Fn\
Mut\x22,\x22FnOnce\x22,\x22T\
oOwned\x22,\x22Clone\x22,\
\x22Debug\x22,\x22Partial\
Eq\x22,\x22PartialOrd\x22\
,\x22Eq\x22,\x22Ord\x22,\x22AsR\
ef\x22,\x22AsMut\x22,\x22Int\
o\x22,\x22From\x22,\x22Defau\
lt\x22,\x22Iterator\x22,\x22\
Extend\x22,\x22IntoIte\
rator\x22,\x22DoubleEn\
dedIterator\x22,\x22Ex\
actSizeIterator\x22\
,\x22SliceConcatExt\
\x22,\x22ToString\x22,\x22as\
sert!\x22,\x22assert_e\
q!\x22,\x22bitflags!\x22,\
\x22bytes!\x22,\x22cfg!\x22,\
\x22col!\x22,\x22concat!\x22\
,\x22concat_idents!\
\x22,\x22debug_assert!\
\x22,\x22debug_assert_\
eq!\x22,\x22env!\x22,\x22epr\
intln!\x22,\x22panic!\x22\
,\x22file!\x22,\x22format\
!\x22,\x22format_args!\
\x22,\x22include_bytes\
!\x22,\x22include_str!\
\x22,\x22line!\x22,\x22local\
_data_key!\x22,\x22mod\
ule_path!\x22,\x22opti\
on_env!\x22,\x22print!\
\x22,\x22println!\x22,\x22se\
lect!\x22,\x22stringif\
y!\x22,\x22try!\x22,\x22unim\
plemented!\x22,\x22unr\
eachable!\x22,\x22vec!\
\x22,\x22write!\x22,\x22writ\
eln!\x22,\x22macro_rul\
es!\x22,\x22assert_ne!\
\x22,\x22debug_assert_\
ne!\x22],o=[\x22i8\x22,\x22i\
16\x22,\x22i32\x22,\x22i64\x22,\
\x22i128\x22,\x22isize\x22,\x22\
u8\x22,\x22u16\x22,\x22u32\x22,\
\x22u64\x22,\x22u128\x22,\x22us\
ize\x22,\x22f32\x22,\x22f64\x22\
,\x22str\x22,\x22char\x22,\x22b\
ool\x22,\x22Box\x22,\x22Opti\
on\x22,\x22Result\x22,\x22St\
ring\x22,\x22Vec\x22]\x0a;re\
turn{name:\x22Rust\x22\
,aliases:[\x22rs\x22],\
keywords:{$patte\
rn:e.IDENT_RE+\x22!\
?\x22,type:o,\x0akeywo\
rd:[\x22abstract\x22,\x22\
as\x22,\x22async\x22,\x22awa\
it\x22,\x22become\x22,\x22bo\
x\x22,\x22break\x22,\x22cons\
t\x22,\x22continue\x22,\x22c\
rate\x22,\x22do\x22,\x22dyn\x22\
,\x22else\x22,\x22enum\x22,\x22\
extern\x22,\x22false\x22,\
\x22final\x22,\x22fn\x22,\x22fo\
r\x22,\x22if\x22,\x22impl\x22,\x22\
in\x22,\x22let\x22,\x22loop\x22\
,\x22macro\x22,\x22match\x22\
,\x22mod\x22,\x22move\x22,\x22m\
ut\x22,\x22override\x22,\x22\
priv\x22,\x22pub\x22,\x22ref\
\x22,\x22return\x22,\x22self\
\x22,\x22Self\x22,\x22static\
\x22,\x22struct\x22,\x22supe\
r\x22,\x22trait\x22,\x22true\
\x22,\x22try\x22,\x22type\x22,\x22\
typeof\x22,\x22union\x22,\
\x22unsafe\x22,\x22unsize\
d\x22,\x22use\x22,\x22virtua\
l\x22,\x22where\x22,\x22whil\
e\x22,\x22yield\x22],\x0alit\
eral:[\x22true\x22,\x22fa\
lse\x22,\x22Some\x22,\x22Non\
e\x22,\x22Ok\x22,\x22Err\x22],b\
uilt_in:l},illeg\
al:\x22</\x22,\x0acontain\
s:[e.C_LINE_COMM\
ENT_MODE,e.COMME\
NT(\x22/\x5c\x5c*\x22,\x22\x5c\x5c*/\x22\
,{contains:[\x22sel\
f\x22]\x0a}),e.inherit\
(e.QUOTE_STRING_\
MODE,{begin:/b?\x22\
/,illegal:null})\
,{\x0aclassName:\x22st\
ring\x22,variants:[\
{begin:/b?r(#*)\x22\
(.|\x5cn)*?\x22\x5c1(?!#)\
/},{\x0abegin:/b?'\x5c\
\x5c?(x\x5cw{2}|u\x5cw{4}\
|U\x5cw{8}|.)'/}]},\
{className:\x22symb\
ol\x22,\x0abegin:/'[a-\
zA-Z_][a-zA-Z0-9\
_]*/},{className\
:\x22number\x22,varian\
ts:[{\x0abegin:\x22\x5c\x5cb\
0b([01_]+)\x22+s},{\
begin:\x22\x5c\x5cb0o([0-\
7_]+)\x22+s},{\x0abegi\
n:\x22\x5c\x5cb0x([A-Fa-f\
0-9_]+)\x22+s},{\x0abe\
gin:\x22\x5c\x5cb(\x5c\x5cd[\x5c\x5cd\
_]*(\x5c\x5c.[0-9_]+)?\
([eE][+-]?[0-9_]\
+)?)\x22+s}],releva\
nce:0},{\x0abegin:[\
/fn/,/\x5cs+/,a],cl\
assName:{1:\x22keyw\
ord\x22,3:\x22title.fu\
nction\x22}},{\x0aclas\
sName:\x22meta\x22,beg\
in:\x22#!?\x5c\x5c[\x22,end:\
\x22\x5c\x5c]\x22,contains:[\
{className:\x22stri\
ng\x22,\x0abegin:/\x22/,e\
nd:/\x22/,contains:\
[e.BACKSLASH_ESC\
APE]}]},{\x0abegin:\
[/let/,/\x5cs+/,/(?\
:mut\x5cs+)?/,a],cl\
assName:{1:\x22keyw\
ord\x22,3:\x22keyword\x22\
,\x0a4:\x22variable\x22}}\
,{begin:[/for/,/\
\x5cs+/,a,/\x5cs+/,/in\
/],className:{1:\
\x22keyword\x22,\x0a3:\x22va\
riable\x22,5:\x22keywo\
rd\x22}},{begin:[/t\
ype/,/\x5cs+/,a],cl\
assName:{1:\x22keyw\
ord\x22,\x0a3:\x22title.c\
lass\x22}},{begin:[\
/(?:trait|enum|s\
truct|union|impl\
|for)/,/\x5cs+/,a],\
\x0aclassName:{1:\x22k\
eyword\x22,3:\x22title\
.class\x22}},{begin\
:e.IDENT_RE+\x22::\x22\
,keywords:{\x0akeyw\
ord:\x22Self\x22,built\
_in:l,type:o}},{\
className:\x22punct\
uation\x22,begin:\x22-\
>\x22},r]}}})()\x0a;hl\
js.registerLangu\
age(\x22rust\x22,e)})(\
);/*! `sas` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>{const t=e.reg\
ex;return{name:\x22\
SAS\x22,\x0acase_insen\
sitive:!0,keywor\
ds:{\x0aliteral:[\x22n\
ull\x22,\x22missing\x22,\x22\
_all_\x22,\x22_automat\
ic_\x22,\x22_character\
_\x22,\x22_infile_\x22,\x22_\
n_\x22,\x22_name_\x22,\x22_n\
ull_\x22,\x22_numeric_\
\x22,\x22_user_\x22,\x22_web\
out_\x22],\x0akeyword:\
[\x22do\x22,\x22if\x22,\x22then\
\x22,\x22else\x22,\x22end\x22,\x22\
until\x22,\x22while\x22,\x22\
abort\x22,\x22array\x22,\x22\
attrib\x22,\x22by\x22,\x22ca\
ll\x22,\x22cards\x22,\x22car\
ds4\x22,\x22catname\x22,\x22\
continue\x22,\x22datal\
ines\x22,\x22datalines\
4\x22,\x22delete\x22,\x22del\
im\x22,\x22delimiter\x22,\
\x22display\x22,\x22dm\x22,\x22\
drop\x22,\x22endsas\x22,\x22\
error\x22,\x22file\x22,\x22f\
ilename\x22,\x22footno\
te\x22,\x22format\x22,\x22go\
to\x22,\x22in\x22,\x22infile\
\x22,\x22informat\x22,\x22in\
put\x22,\x22keep\x22,\x22lab\
el\x22,\x22leave\x22,\x22len\
gth\x22,\x22libname\x22,\x22\
link\x22,\x22list\x22,\x22lo\
stcard\x22,\x22merge\x22,\
\x22missing\x22,\x22modif\
y\x22,\x22options\x22,\x22ou\
tput\x22,\x22out\x22,\x22pag\
e\x22,\x22put\x22,\x22redire\
ct\x22,\x22remove\x22,\x22re\
name\x22,\x22replace\x22,\
\x22retain\x22,\x22return\
\x22,\x22select\x22,\x22set\x22\
,\x22skip\x22,\x22startsa\
s\x22,\x22stop\x22,\x22title\
\x22,\x22update\x22,\x22wait\
sas\x22,\x22where\x22,\x22wi\
ndow\x22,\x22x|0\x22,\x22sys\
task\x22,\x22add\x22,\x22and\
\x22,\x22alter\x22,\x22as\x22,\x22\
cascade\x22,\x22check\x22\
,\x22create\x22,\x22delet\
e\x22,\x22describe\x22,\x22d\
istinct\x22,\x22drop\x22,\
\x22foreign\x22,\x22from\x22\
,\x22group\x22,\x22having\
\x22,\x22index\x22,\x22inser\
t\x22,\x22into\x22,\x22in\x22,\x22\
key\x22,\x22like\x22,\x22mes\
sage\x22,\x22modify\x22,\x22\
msgtype\x22,\x22not\x22,\x22\
null\x22,\x22on\x22,\x22or\x22,\
\x22order\x22,\x22primary\
\x22,\x22references\x22,\x22\
reset\x22,\x22restrict\
\x22,\x22select\x22,\x22set\x22\
,\x22table\x22,\x22unique\
\x22,\x22update\x22,\x22vali\
date\x22,\x22view\x22,\x22wh\
ere\x22]\x0a},contains\
:[{className:\x22ke\
yword\x22,begin:/^\x5c\
s*(proc [\x5cw\x5cd_]+\
|data|run|quit)[\
\x5cs;]/\x0a},{classNa\
me:\x22variable\x22,be\
gin:/&[a-zA-Z_&]\
[a-zA-Z0-9_]*\x5c.?\
/},{\x0abegin:[/^\x5cs\
*/,/datalines;|c\
ards;/,/(?:.*\x5cn)\
+/,/^\x5cs*;\x5cs*$/],\
className:{\x0a2:\x22k\
eyword\x22,3:\x22strin\
g\x22}},{\x0abegin:[/%\
mend|%macro/,/\x5cs\
+/,/[a-zA-Z_&][a\
-zA-Z0-9_]*/],cl\
assName:{1:\x22buil\
t_in\x22,\x0a3:\x22title.\
function\x22}},{cla\
ssName:\x22built_in\
\x22,\x0abegin:\x22%\x22+t.e\
ither(\x22bquote\x22,\x22\
nrbquote\x22,\x22cmpre\
s\x22,\x22qcmpres\x22,\x22co\
mpstor\x22,\x22datatyp\
\x22,\x22display\x22,\x22do\x22\
,\x22else\x22,\x22end\x22,\x22e\
val\x22,\x22global\x22,\x22g\
oto\x22,\x22if\x22,\x22index\
\x22,\x22input\x22,\x22keyde\
f\x22,\x22label\x22,\x22left\
\x22,\x22length\x22,\x22let\x22\
,\x22local\x22,\x22lowcas\
e\x22,\x22macro\x22,\x22mend\
\x22,\x22nrbquote\x22,\x22nr\
quote\x22,\x22nrstr\x22,\x22\
put\x22,\x22qcmpres\x22,\x22\
qleft\x22,\x22qlowcase\
\x22,\x22qscan\x22,\x22qsubs\
tr\x22,\x22qsysfunc\x22,\x22\
qtrim\x22,\x22quote\x22,\x22\
qupcase\x22,\x22scan\x22,\
\x22str\x22,\x22substr\x22,\x22\
superq\x22,\x22syscall\
\x22,\x22sysevalf\x22,\x22sy\
sexec\x22,\x22sysfunc\x22\
,\x22sysget\x22,\x22syslp\
ut\x22,\x22sysprod\x22,\x22s\
ysrc\x22,\x22sysrput\x22,\
\x22then\x22,\x22to\x22,\x22tri\
m\x22,\x22unquote\x22,\x22un\
til\x22,\x22upcase\x22,\x22v\
erify\x22,\x22while\x22,\x22\
window\x22)\x0a},{clas\
sName:\x22title.fun\
ction\x22,begin:/%[\
a-zA-Z_][a-zA-Z_\
0-9]*/},{\x0aclassN\
ame:\x22meta\x22,\x0abegi\
n:t.either(\x22abs\x22\
,\x22addr\x22,\x22airy\x22,\x22\
arcos\x22,\x22arsin\x22,\x22\
atan\x22,\x22attrc\x22,\x22a\
ttrn\x22,\x22band\x22,\x22be\
tainv\x22,\x22blshift\x22\
,\x22bnot\x22,\x22bor\x22,\x22b\
rshift\x22,\x22bxor\x22,\x22\
byte\x22,\x22cdf\x22,\x22cei\
l\x22,\x22cexist\x22,\x22cin\
v\x22,\x22close\x22,\x22cnon\
ct\x22,\x22collate\x22,\x22c\
ompbl\x22,\x22compound\
\x22,\x22compress\x22,\x22co\
s\x22,\x22cosh\x22,\x22css\x22,\
\x22curobs\x22,\x22cv\x22,\x22d\
accdb\x22,\x22daccdbsl\
\x22,\x22daccsl\x22,\x22dacc\
syd\x22,\x22dacctab\x22,\x22\
dairy\x22,\x22date\x22,\x22d\
atejul\x22,\x22datepar\
t\x22,\x22datetime\x22,\x22d\
ay\x22,\x22dclose\x22,\x22de\
pdb\x22,\x22depdbsl\x22,\x22\
depdbsl\x22,\x22depsl\x22\
,\x22depsl\x22,\x22depsyd\
\x22,\x22depsyd\x22,\x22dept\
ab\x22,\x22deptab\x22,\x22de\
quote\x22,\x22dhms\x22,\x22d\
if\x22,\x22digamma\x22,\x22d\
im\x22,\x22dinfo\x22,\x22dnu\
m\x22,\x22dopen\x22,\x22dopt\
name\x22,\x22doptnum\x22,\
\x22dread\x22,\x22dropnot\
e\x22,\x22dsname\x22,\x22erf\
\x22,\x22erfc\x22,\x22exist\x22\
,\x22exp\x22,\x22fappend\x22\
,\x22fclose\x22,\x22fcol\x22\
,\x22fdelete\x22,\x22fetc\
h\x22,\x22fetchobs\x22,\x22f\
exist\x22,\x22fget\x22,\x22f\
ileexist\x22,\x22filen\
ame\x22,\x22fileref\x22,\x22\
finfo\x22,\x22finv\x22,\x22f\
ipname\x22,\x22fipname\
l\x22,\x22fipstate\x22,\x22f\
loor\x22,\x22fnonct\x22,\x22\
fnote\x22,\x22fopen\x22,\x22\
foptname\x22,\x22foptn\
um\x22,\x22fpoint\x22,\x22fp\
os\x22,\x22fput\x22,\x22frea\
d\x22,\x22frewind\x22,\x22fr\
len\x22,\x22fsep\x22,\x22fuz\
z\x22,\x22fwrite\x22,\x22gam\
inv\x22,\x22gamma\x22,\x22ge\
toption\x22,\x22getvar\
c\x22,\x22getvarn\x22,\x22hb\
ound\x22,\x22hms\x22,\x22hos\
thelp\x22,\x22hour\x22,\x22i\
bessel\x22,\x22index\x22,\
\x22indexc\x22,\x22indexw\
\x22,\x22input\x22,\x22input\
c\x22,\x22inputn\x22,\x22int\
\x22,\x22intck\x22,\x22intnx\
\x22,\x22intrr\x22,\x22irr\x22,\
\x22jbessel\x22,\x22julda\
te\x22,\x22kurtosis\x22,\x22\
lag\x22,\x22lbound\x22,\x22l\
eft\x22,\x22length\x22,\x22l\
gamma\x22,\x22libname\x22\
,\x22libref\x22,\x22log\x22,\
\x22log10\x22,\x22log2\x22,\x22\
logpdf\x22,\x22logpmf\x22\
,\x22logsdf\x22,\x22lowca\
se\x22,\x22max\x22,\x22mdy\x22,\
\x22mean\x22,\x22min\x22,\x22mi\
nute\x22,\x22mod\x22,\x22mon\
th\x22,\x22mopen\x22,\x22mor\
t\x22,\x22n\x22,\x22netpv\x22,\x22\
nmiss\x22,\x22normal\x22,\
\x22note\x22,\x22npv\x22,\x22op\
en\x22,\x22ordinal\x22,\x22p\
athname\x22,\x22pdf\x22,\x22\
peek\x22,\x22peekc\x22,\x22p\
mf\x22,\x22point\x22,\x22poi\
sson\x22,\x22poke\x22,\x22pr\
obbeta\x22,\x22probbnm\
l\x22,\x22probchi\x22,\x22pr\
obf\x22,\x22probgam\x22,\x22\
probhypr\x22,\x22probi\
t\x22,\x22probnegb\x22,\x22p\
robnorm\x22,\x22probt\x22\
,\x22put\x22,\x22putc\x22,\x22p\
utn\x22,\x22qtr\x22,\x22quot\
e\x22,\x22ranbin\x22,\x22ran\
cau\x22,\x22ranexp\x22,\x22r\
angam\x22,\x22range\x22,\x22\
rank\x22,\x22rannor\x22,\x22\
ranpoi\x22,\x22rantbl\x22\
,\x22rantri\x22,\x22ranun\
i\x22,\x22repeat\x22,\x22res\
olve\x22,\x22reverse\x22,\
\x22rewind\x22,\x22right\x22\
,\x22round\x22,\x22saving\
\x22,\x22scan\x22,\x22sdf\x22,\x22\
second\x22,\x22sign\x22,\x22\
sin\x22,\x22sinh\x22,\x22ske\
wness\x22,\x22soundex\x22\
,\x22spedis\x22,\x22sqrt\x22\
,\x22std\x22,\x22stderr\x22,\
\x22stfips\x22,\x22stname\
\x22,\x22stnamel\x22,\x22sub\
str\x22,\x22sum\x22,\x22symg\
et\x22,\x22sysget\x22,\x22sy\
smsg\x22,\x22sysprod\x22,\
\x22sysrc\x22,\x22system\x22\
,\x22tan\x22,\x22tanh\x22,\x22t\
ime\x22,\x22timepart\x22,\
\x22tinv\x22,\x22tnonct\x22,\
\x22today\x22,\x22transla\
te\x22,\x22tranwrd\x22,\x22t\
rigamma\x22,\x22trim\x22,\
\x22trimn\x22,\x22trunc\x22,\
\x22uniform\x22,\x22upcas\
e\x22,\x22uss\x22,\x22var\x22,\x22\
varfmt\x22,\x22varinfm\
t\x22,\x22varlabel\x22,\x22v\
arlen\x22,\x22varname\x22\
,\x22varnum\x22,\x22varra\
y\x22,\x22varrayx\x22,\x22va\
rtype\x22,\x22verify\x22,\
\x22vformat\x22,\x22vform\
atd\x22,\x22vformatdx\x22\
,\x22vformatn\x22,\x22vfo\
rmatnx\x22,\x22vformat\
w\x22,\x22vformatwx\x22,\x22\
vformatx\x22,\x22vinar\
ray\x22,\x22vinarrayx\x22\
,\x22vinformat\x22,\x22vi\
nformatd\x22,\x22vinfo\
rmatdx\x22,\x22vinform\
atn\x22,\x22vinformatn\
x\x22,\x22vinformatw\x22,\
\x22vinformatwx\x22,\x22v\
informatx\x22,\x22vlab\
el\x22,\x22vlabelx\x22,\x22v\
length\x22,\x22vlength\
x\x22,\x22vname\x22,\x22vnam\
ex\x22,\x22vtype\x22,\x22vty\
pex\x22,\x22weekday\x22,\x22\
year\x22,\x22yyq\x22,\x22zip\
fips\x22,\x22zipname\x22,\
\x22zipnamel\x22,\x22zips\
tate\x22)+\x22(?=\x5c\x5c()\x22\
\x0a},{className:\x22s\
tring\x22,variants:\
[e.APOS_STRING_M\
ODE,e.QUOTE_STRI\
NG_MODE]\x0a},e.COM\
MENT(\x22\x5c\x5c*\x22,\x22;\x22),\
e.C_BLOCK_COMMEN\
T_MODE]}}})()\x0a;h\
ljs.registerLang\
uage(\x22sas\x22,e)})(\
);/*! `scala` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>{const n=e.r\
egex,a={classNam\
e:\x22subst\x22,\x0avaria\
nts:[{begin:\x22\x5c\x5c$\
[A-Za-z0-9_]+\x22},\
{begin:/\x5c$\x5c{/,en\
d:/\x5c}/}]},s={\x0acl\
assName:\x22string\x22\
,variants:[{begi\
n:'\x22\x22\x22',end:'\x22\x22\x22\
'},{begin:'\x22',en\
d:'\x22',\x0aillegal:\x22\
\x5c\x5cn\x22,contains:[e\
.BACKSLASH_ESCAP\
E]},{begin:'[a-z\
]+\x22',end:'\x22',\x0ail\
legal:\x22\x5c\x5cn\x22,cont\
ains:[e.BACKSLAS\
H_ESCAPE,a]},{cl\
assName:\x22string\x22\
,\x0abegin:'[a-z]+\x22\
\x22\x22',end:'\x22\x22\x22',co\
ntains:[a],relev\
ance:10}]},i={cl\
assName:\x22type\x22,\x0a\
begin:\x22\x5c\x5cb[A-Z][\
A-Za-z0-9_]*\x22,re\
levance:0},t={cl\
assName:\x22title\x22,\
\x0abegin:/[^0-9\x5cn\x5c\
t \x22'(),.`{}\x5c[\x5c]:\
;][^\x5cn\x5ct \x22'(),.`\
{}\x5c[\x5c]:;]+|[^0-9\
\x5cn\x5ct \x22'(),.`{}\x5c[\
\x5c]:;=]/,\x0arelevan\
ce:0},c={classNa\
me:\x22class\x22,begin\
Keywords:\x22class \
object trait typ\
e\x22,\x0aend:/[:={\x5c[\x5c\
n;]/,excludeEnd:\
!0,\x0acontains:[e.\
C_LINE_COMMENT_M\
ODE,e.C_BLOCK_CO\
MMENT_MODE,{\x0abeg\
inKeywords:\x22exte\
nds with\x22,releva\
nce:10},{begin:/\
\x5c[/,end:/\x5c]/,exc\
ludeBegin:!0,\x0aex\
cludeEnd:!0,rele\
vance:0,\x0acontain\
s:[i,e.C_LINE_CO\
MMENT_MODE,e.C_B\
LOCK_COMMENT_MOD\
E]},{className:\x22\
params\x22,\x0abegin:/\
\x5c(/,end:/\x5c)/,exc\
ludeBegin:!0,exc\
ludeEnd:!0,relev\
ance:0,\x0acontains\
:[i,e.C_LINE_COM\
MENT_MODE,e.C_BL\
OCK_COMMENT_MODE\
]},t]},r={\x0aclass\
Name:\x22function\x22,\
beginKeywords:\x22d\
ef\x22,end:n.lookah\
ead(/[:={\x5c[(\x5cn;]\
/),\x0acontains:[t]\
};return{name:\x22S\
cala\x22,keywords:{\
literal:\x22true fa\
lse null\x22,\x0akeywo\
rd:\x22type yield l\
azy override def\
 with val var se\
aled abstract pr\
ivate trait obje\
ct if then forSo\
me for while do \
throw finally pr\
otected extends \
import final ret\
urn else break n\
ew catch super c\
lass case packag\
e default try th\
is match continu\
e throws implici\
t export enum gi\
ven transparent\x22\
\x0a},contains:[{be\
gin:[\x22//>\x22,/\x5cs+/\
,/using/,/\x5cs+/,/\
\x5cS+/],beginScope\
:{1:\x22comment\x22,\x0a3\
:\x22keyword\x22,5:\x22ty\
pe\x22},end:/$/,con\
tains:[{classNam\
e:\x22string\x22,begin\
:/\x5cS+/}]\x0a},e.C_L\
INE_COMMENT_MODE\
,e.C_BLOCK_COMME\
NT_MODE,s,i,r,c,\
e.C_NUMBER_MODE,\
{\x0abegin:[/^\x5cs*/,\
\x22extension\x22,/\x5cs+\
(?=[[(])/],begin\
Scope:{2:\x22keywor\
d\x22}},{\x0abegin:[/^\
\x5cs*/,/end/,/\x5cs+/\
,/(extension\x5cb)?\
/],beginScope:{2\
:\x22keyword\x22,4:\x22ke\
yword\x22}\x0a},{match\
:/\x5c.inline\x5cb/},{\
begin:/\x5cbinline(\
?=\x5cs)/,keywords:\
\x22inline\x22},{\x0abegi\
n:[/\x5c(\x5cs*/,/usin\
g/,/\x5cs+(?!\x5c))/],\
beginScope:{2:\x22k\
eyword\x22}},{class\
Name:\x22meta\x22,\x0abeg\
in:\x22@[A-Za-z]+\x22}\
]}}})();hljs.reg\
isterLanguage(\x22s\
cala\x22,e)})();/*!\
 `scheme` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
{\x0aconst t=\x22[^\x5c\x5c(\
\x5c\x5c)\x5c\x5c[\x5c\x5c]\x5c\x5c{\x5c\x5c}\x5c\
\x22,'`;#|\x5c\x5c\x5c\x5c\x5c\x5cs]+\
\x22,n=\x22(-|\x5c\x5c+)?\x5c\x5cd\
+([./]\x5c\x5cd+)?\x22,r=\
{\x0a$pattern:t,\x0abu\
ilt_in:\x22case-lam\
bda call/cc clas\
s define-class e\
xit-handler fiel\
d import inherit\
 init-field inte\
rface let*-value\
s let-values let\
/ec mixin opt-la\
mbda override pr\
otect provide pu\
blic rename requ\
ire require-for-\
syntax syntax sy\
ntax-case syntax\
-error unit/sig \
unless when with\
-syntax and begi\
n call-with-curr\
ent-continuation\
 call-with-input\
-file call-with-\
output-file case\
 cond define def\
ine-syntax delay\
 do dynamic-wind\
 else for-each i\
f lambda let let\
* let-syntax let\
rec letrec-synta\
x map or syntax-\
rules ' * + , ,@\
 - ... / ; < <= \
= => > >= ` abs \
acos angle appen\
d apply asin ass\
oc assq assv ata\
n boolean? caar \
cadr call-with-i\
nput-file call-w\
ith-output-file \
call-with-values\
 car cdddar cddd\
dr cdr ceiling c\
har->integer cha\
r-alphabetic? ch\
ar-ci<=? char-ci\
<? char-ci=? cha\
r-ci>=? char-ci>\
? char-downcase \
char-lower-case?\
 char-numeric? c\
har-ready? char-\
upcase char-uppe\
r-case? char-whi\
tespace? char<=?\
 char<? char=? c\
har>=? char>? ch\
ar? close-input-\
port close-outpu\
t-port complex? \
cons cos current\
-input-port curr\
ent-output-port \
denominator disp\
lay eof-object? \
eq? equal? eqv? \
eval even? exact\
->inexact exact?\
 exp expt floor \
force gcd imag-p\
art inexact->exa\
ct inexact? inpu\
t-port? integer-\
>char integer? i\
nteraction-envir\
onment lcm lengt\
h list list->str\
ing list->vector\
 list-ref list-t\
ail list? load l\
og magnitude mak\
e-polar make-rec\
tangular make-st\
ring make-vector\
 max member memq\
 memv min modulo\
 negative? newli\
ne not null-envi\
ronment null? nu\
mber->string num\
ber? numerator o\
dd? open-input-f\
ile open-output-\
file output-port\
? pair? peek-cha\
r port? positive\
? procedure? qua\
siquote quote qu\
otient rational?\
 rationalize rea\
d read-char real\
-part real? rema\
inder reverse ro\
und scheme-repor\
t-environment se\
t! set-car! set-\
cdr! sin sqrt st\
ring string->lis\
t string->number\
 string->symbol \
string-append st\
ring-ci<=? strin\
g-ci<? string-ci\
=? string-ci>=? \
string-ci>? stri\
ng-copy string-f\
ill! string-leng\
th string-ref st\
ring-set! string\
<=? string<? str\
ing=? string>=? \
string>? string?\
 substring symbo\
l->string symbol\
? tan transcript\
-off transcript-\
on truncate valu\
es vector vector\
->list vector-fi\
ll! vector-lengt\
h vector-ref vec\
tor-set! with-in\
put-from-file wi\
th-output-to-fil\
e write write-ch\
ar zero?\x22\x0a},a={c\
lassName:\x22litera\
l\x22,begin:\x22(#t|#f\
|#\x5c\x5c\x5c\x5c\x22+t+\x22|#\x5c\x5c\x5c\
\x5c.)\x22},i={\x0aclassN\
ame:\x22number\x22,var\
iants:[{begin:n,\
relevance:0},{be\
gin:n+\x22[+\x5c\x5c-]\x22+n\
+\x22i\x22,\x0arelevance:\
0},{begin:\x22#b[0-\
1]+(/[0-1]+)?\x22},\
{begin:\x22#o[0-7]+\
(/[0-7]+)?\x22},{\x0ab\
egin:\x22#x[0-9a-f]\
+(/[0-9a-f]+)?\x22}\
]},c=e.QUOTE_STR\
ING_MODE,s=[e.CO\
MMENT(\x22;\x22,\x22$\x22,{\x0a\
relevance:0}),e.\
COMMENT(\x22#\x5c\x5c|\x22,\x22\
\x5c\x5c|#\x22)],l={begin\
:t,relevance:0},\
o={\x0aclassName:\x22s\
ymbol\x22,begin:\x22'\x22\
+t},g={endsWithP\
arent:!0,relevan\
ce:0},u={variant\
s:[{\x0abegin:/'/},\
{begin:\x22`\x22}],con\
tains:[{begin:\x22\x5c\
\x5c(\x22,end:\x22\x5c\x5c)\x22,\x0ac\
ontains:[\x22self\x22,\
a,c,i,l,o]}]},d=\
{className:\x22name\
\x22,relevance:0,be\
gin:t,\x0akeywords:\
r},p={variants:[\
{begin:\x22\x5c\x5c(\x22,end\
:\x22\x5c\x5c)\x22},{begin:\x22\
\x5c\x5c[\x22,end:\x22\x5c\x5c]\x22}]\
,\x0acontains:[{beg\
in:/lambda/,ends\
WithParent:!0,re\
turnBegin:!0,con\
tains:[d,{\x0aendsP\
arent:!0,variant\
s:[{begin:/\x5c(/,e\
nd:/\x5c)/},{begin:\
/\x5c[/,end:/\x5c]/}],\
\x0acontains:[l]}]}\
,d,g]};return g.\
contains=[a,i,c,\
l,o,u,p].concat(\
s),{\x0aname:\x22Schem\
e\x22,aliases:[\x22scm\
\x22],illegal:/\x5cS/,\
\x0acontains:[e.SHE\
BANG(),i,c,o,u,p\
].concat(s)}}})(\
)\x0a;hljs.register\
Language(\x22scheme\
\x22,e)})();/*! `sc\
ilab` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>{con\
st n=[e.C_NUMBER\
_MODE,{\x0aclassNam\
e:\x22string\x22,begin\
:\x22'|\x5c\x22\x22,end:\x22'|\x5c\
\x22\x22,contains:[e.B\
ACKSLASH_ESCAPE,\
{\x0abegin:\x22''\x22}]}]\
;return{name:\x22Sc\
ilab\x22,aliases:[\x22\
sci\x22],keywords:{\
$pattern:/%?\x5cw+/\
,\x0akeyword:\x22abort\
 break case clea\
r catch continue\
 do elseif else \
endfunction end \
for function glo\
bal if pause ret\
urn resume selec\
t try then while\
\x22,\x0aliteral:\x22%f %\
F %t %T %pi %eps\
 %inf %nan %e %i\
 %z %s\x22,\x0abuilt_i\
n:\x22abs and acos \
asin atan ceil c\
d chdir clearglo\
bal cosh cos cum\
prod deff disp e\
rror exec execst\
r exists exp eye\
 gettext floor f\
printf fread fso\
lve imag isdef i\
sempty isinfisna\
n isvector laste\
rror length load\
 linspace list l\
istfiles log10 l\
og2 log max min \
msprintf mclose \
mopen ones or pa\
thconvert poly p\
rintf prod pwd r\
and real round s\
inh sin size gso\
rt sprintf sqrt \
strcat strcmps t\
ring sum system \
tanh tan type ty\
pename warning z\
eros matrix\x22\x0a},i\
llegal:'(\x22|#|/\x5c\x5c\
*|\x5c\x5cs+/\x5c\x5cw+)',co\
ntains:[{classNa\
me:\x22function\x22,\x0ab\
eginKeywords:\x22fu\
nction\x22,end:\x22$\x22,\
contains:[e.UNDE\
RSCORE_TITLE_MOD\
E,{\x0aclassName:\x22p\
arams\x22,begin:\x22\x5c\x5c\
(\x22,end:\x22\x5c\x5c)\x22}]},\
{\x0abegin:\x22[a-zA-Z\
_][a-zA-Z_0-9]*[\
\x5c\x5c.']+\x22,relevanc\
e:0},{begin:\x22\x5c\x5c[\
\x22,\x0aend:\x22\x5c\x5c][\x5c\x5c.'\
]*\x22,relevance:0,\
contains:n},e.CO\
MMENT(\x22//\x22,\x22$\x22)]\
.concat(n)}}})()\
\x0a;hljs.registerL\
anguage(\x22scilab\x22\
,e)})();/*! `scs\
s` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22\x0a;\
const e=[\x22a\x22,\x22ab\
br\x22,\x22address\x22,\x22a\
rticle\x22,\x22aside\x22,\
\x22audio\x22,\x22b\x22,\x22blo\
ckquote\x22,\x22body\x22,\
\x22button\x22,\x22canvas\
\x22,\x22caption\x22,\x22cit\
e\x22,\x22code\x22,\x22dd\x22,\x22\
del\x22,\x22details\x22,\x22\
dfn\x22,\x22div\x22,\x22dl\x22,\
\x22dt\x22,\x22em\x22,\x22field\
set\x22,\x22figcaption\
\x22,\x22figure\x22,\x22foot\
er\x22,\x22form\x22,\x22h1\x22,\
\x22h2\x22,\x22h3\x22,\x22h4\x22,\x22\
h5\x22,\x22h6\x22,\x22header\
\x22,\x22hgroup\x22,\x22html\
\x22,\x22i\x22,\x22iframe\x22,\x22\
img\x22,\x22input\x22,\x22in\
s\x22,\x22kbd\x22,\x22label\x22\
,\x22legend\x22,\x22li\x22,\x22\
main\x22,\x22mark\x22,\x22me\
nu\x22,\x22nav\x22,\x22objec\
t\x22,\x22ol\x22,\x22optgrou\
p\x22,\x22option\x22,\x22p\x22,\
\x22picture\x22,\x22q\x22,\x22q\
uote\x22,\x22samp\x22,\x22se\
ction\x22,\x22select\x22,\
\x22source\x22,\x22span\x22,\
\x22strong\x22,\x22summar\
y\x22,\x22sup\x22,\x22table\x22\
,\x22tbody\x22,\x22td\x22,\x22t\
extarea\x22,\x22tfoot\x22\
,\x22th\x22,\x22thead\x22,\x22t\
ime\x22,\x22tr\x22,\x22ul\x22,\x22\
var\x22,\x22video\x22,\x22de\
fs\x22,\x22g\x22,\x22marker\x22\
,\x22mask\x22,\x22pattern\
\x22,\x22svg\x22,\x22switch\x22\
,\x22symbol\x22,\x22feBle\
nd\x22,\x22feColorMatr\
ix\x22,\x22feComponent\
Transfer\x22,\x22feCom\
posite\x22,\x22feConvo\
lveMatrix\x22,\x22feDi\
ffuseLighting\x22,\x22\
feDisplacementMa\
p\x22,\x22feFlood\x22,\x22fe\
GaussianBlur\x22,\x22f\
eImage\x22,\x22feMerge\
\x22,\x22feMorphology\x22\
,\x22feOffset\x22,\x22feS\
pecularLighting\x22\
,\x22feTile\x22,\x22feTur\
bulence\x22,\x22linear\
Gradient\x22,\x22radia\
lGradient\x22,\x22stop\
\x22,\x22circle\x22,\x22elli\
pse\x22,\x22image\x22,\x22li\
ne\x22,\x22path\x22,\x22poly\
gon\x22,\x22polyline\x22,\
\x22rect\x22,\x22text\x22,\x22u\
se\x22,\x22textPath\x22,\x22\
tspan\x22,\x22foreignO\
bject\x22,\x22clipPath\
\x22],r=[\x22any-hover\
\x22,\x22any-pointer\x22,\
\x22aspect-ratio\x22,\x22\
color\x22,\x22color-ga\
mut\x22,\x22color-inde\
x\x22,\x22device-aspec\
t-ratio\x22,\x22device\
-height\x22,\x22device\
-width\x22,\x22display\
-mode\x22,\x22forced-c\
olors\x22,\x22grid\x22,\x22h\
eight\x22,\x22hover\x22,\x22\
inverted-colors\x22\
,\x22monochrome\x22,\x22o\
rientation\x22,\x22ove\
rflow-block\x22,\x22ov\
erflow-inline\x22,\x22\
pointer\x22,\x22prefer\
s-color-scheme\x22,\
\x22prefers-contras\
t\x22,\x22prefers-redu\
ced-motion\x22,\x22pre\
fers-reduced-tra\
nsparency\x22,\x22reso\
lution\x22,\x22scan\x22,\x22\
scripting\x22,\x22upda\
te\x22,\x22width\x22,\x22min\
-width\x22,\x22max-wid\
th\x22,\x22min-height\x22\
,\x22max-height\x22].s\
ort().reverse(),\
t=[\x22active\x22,\x22any\
-link\x22,\x22blank\x22,\x22\
checked\x22,\x22curren\
t\x22,\x22default\x22,\x22de\
fined\x22,\x22dir\x22,\x22di\
sabled\x22,\x22drop\x22,\x22\
empty\x22,\x22enabled\x22\
,\x22first\x22,\x22first-\
child\x22,\x22first-of\
-type\x22,\x22fullscre\
en\x22,\x22future\x22,\x22fo\
cus\x22,\x22focus-visi\
ble\x22,\x22focus-with\
in\x22,\x22has\x22,\x22host\x22\
,\x22host-context\x22,\
\x22hover\x22,\x22indeter\
minate\x22,\x22in-rang\
e\x22,\x22invalid\x22,\x22is\
\x22,\x22lang\x22,\x22last-c\
hild\x22,\x22last-of-t\
ype\x22,\x22left\x22,\x22lin\
k\x22,\x22local-link\x22,\
\x22not\x22,\x22nth-child\
\x22,\x22nth-col\x22,\x22nth\
-last-child\x22,\x22nt\
h-last-col\x22,\x22nth\
-last-of-type\x22,\x22\
nth-of-type\x22,\x22on\
ly-child\x22,\x22only-\
of-type\x22,\x22option\
al\x22,\x22out-of-rang\
e\x22,\x22past\x22,\x22place\
holder-shown\x22,\x22r\
ead-only\x22,\x22read-\
write\x22,\x22required\
\x22,\x22right\x22,\x22root\x22\
,\x22scope\x22,\x22target\
\x22,\x22target-within\
\x22,\x22user-invalid\x22\
,\x22valid\x22,\x22visite\
d\x22,\x22where\x22].sort\
().reverse(),i=[\
\x22after\x22,\x22backdro\
p\x22,\x22before\x22,\x22cue\
\x22,\x22cue-region\x22,\x22\
first-letter\x22,\x22f\
irst-line\x22,\x22gram\
mar-error\x22,\x22mark\
er\x22,\x22part\x22,\x22plac\
eholder\x22,\x22select\
ion\x22,\x22slotted\x22,\x22\
spelling-error\x22]\
.sort().reverse(\
),o=[\x22accent-col\
or\x22,\x22align-conte\
nt\x22,\x22align-items\
\x22,\x22align-self\x22,\x22\
alignment-baseli\
ne\x22,\x22all\x22,\x22anima\
tion\x22,\x22animation\
-delay\x22,\x22animati\
on-direction\x22,\x22a\
nimation-duratio\
n\x22,\x22animation-fi\
ll-mode\x22,\x22animat\
ion-iteration-co\
unt\x22,\x22animation-\
name\x22,\x22animation\
-play-state\x22,\x22an\
imation-timing-f\
unction\x22,\x22appear\
ance\x22,\x22backface-\
visibility\x22,\x22bac\
kground\x22,\x22backgr\
ound-attachment\x22\
,\x22background-ble\
nd-mode\x22,\x22backgr\
ound-clip\x22,\x22back\
ground-color\x22,\x22b\
ackground-image\x22\
,\x22background-ori\
gin\x22,\x22background\
-position\x22,\x22back\
ground-repeat\x22,\x22\
background-size\x22\
,\x22baseline-shift\
\x22,\x22block-size\x22,\x22\
border\x22,\x22border-\
block\x22,\x22border-b\
lock-color\x22,\x22bor\
der-block-end\x22,\x22\
border-block-end\
-color\x22,\x22border-\
block-end-style\x22\
,\x22border-block-e\
nd-width\x22,\x22borde\
r-block-start\x22,\x22\
border-block-sta\
rt-color\x22,\x22borde\
r-block-start-st\
yle\x22,\x22border-blo\
ck-start-width\x22,\
\x22border-block-st\
yle\x22,\x22border-blo\
ck-width\x22,\x22borde\
r-bottom\x22,\x22borde\
r-bottom-color\x22,\
\x22border-bottom-l\
eft-radius\x22,\x22bor\
der-bottom-right\
-radius\x22,\x22border\
-bottom-style\x22,\x22\
border-bottom-wi\
dth\x22,\x22border-col\
lapse\x22,\x22border-c\
olor\x22,\x22border-im\
age\x22,\x22border-ima\
ge-outset\x22,\x22bord\
er-image-repeat\x22\
,\x22border-image-s\
lice\x22,\x22border-im\
age-source\x22,\x22bor\
der-image-width\x22\
,\x22border-inline\x22\
,\x22border-inline-\
color\x22,\x22border-i\
nline-end\x22,\x22bord\
er-inline-end-co\
lor\x22,\x22border-inl\
ine-end-style\x22,\x22\
border-inline-en\
d-width\x22,\x22border\
-inline-start\x22,\x22\
border-inline-st\
art-color\x22,\x22bord\
er-inline-start-\
style\x22,\x22border-i\
nline-start-widt\
h\x22,\x22border-inlin\
e-style\x22,\x22border\
-inline-width\x22,\x22\
border-left\x22,\x22bo\
rder-left-color\x22\
,\x22border-left-st\
yle\x22,\x22border-lef\
t-width\x22,\x22border\
-radius\x22,\x22border\
-right\x22,\x22border-\
end-end-radius\x22,\
\x22border-end-star\
t-radius\x22,\x22borde\
r-right-color\x22,\x22\
border-right-sty\
le\x22,\x22border-righ\
t-width\x22,\x22border\
-spacing\x22,\x22borde\
r-start-end-radi\
us\x22,\x22border-star\
t-start-radius\x22,\
\x22border-style\x22,\x22\
border-top\x22,\x22bor\
der-top-color\x22,\x22\
border-top-left-\
radius\x22,\x22border-\
top-right-radius\
\x22,\x22border-top-st\
yle\x22,\x22border-top\
-width\x22,\x22border-\
width\x22,\x22bottom\x22,\
\x22box-decoration-\
break\x22,\x22box-shad\
ow\x22,\x22box-sizing\x22\
,\x22break-after\x22,\x22\
break-before\x22,\x22b\
reak-inside\x22,\x22cx\
\x22,\x22cy\x22,\x22caption-\
side\x22,\x22caret-col\
or\x22,\x22clear\x22,\x22cli\
p\x22,\x22clip-path\x22,\x22\
clip-rule\x22,\x22colo\
r\x22,\x22color-interp\
olation\x22,\x22color-\
interpolation-fi\
lters\x22,\x22color-pr\
ofile\x22,\x22color-re\
ndering\x22,\x22color-\
scheme\x22,\x22column-\
count\x22,\x22column-f\
ill\x22,\x22column-gap\
\x22,\x22column-rule\x22,\
\x22column-rule-col\
or\x22,\x22column-rule\
-style\x22,\x22column-\
rule-width\x22,\x22col\
umn-span\x22,\x22colum\
n-width\x22,\x22column\
s\x22,\x22contain\x22,\x22co\
ntent\x22,\x22content-\
visibility\x22,\x22cou\
nter-increment\x22,\
\x22counter-reset\x22,\
\x22cue\x22,\x22cue-after\
\x22,\x22cue-before\x22,\x22\
cursor\x22,\x22directi\
on\x22,\x22display\x22,\x22d\
ominant-baseline\
\x22,\x22empty-cells\x22,\
\x22enable-backgrou\
nd\x22,\x22fill\x22,\x22fill\
-opacity\x22,\x22fill-\
rule\x22,\x22filter\x22,\x22\
flex\x22,\x22flex-basi\
s\x22,\x22flex-directi\
on\x22,\x22flex-flow\x22,\
\x22flex-grow\x22,\x22fle\
x-shrink\x22,\x22flex-\
wrap\x22,\x22float\x22,\x22f\
low\x22,\x22flood-colo\
r\x22,\x22flood-opacit\
y\x22,\x22font\x22,\x22font-\
display\x22,\x22font-f\
amily\x22,\x22font-fea\
ture-settings\x22,\x22\
font-kerning\x22,\x22f\
ont-language-ove\
rride\x22,\x22font-siz\
e\x22,\x22font-size-ad\
just\x22,\x22font-smoo\
thing\x22,\x22font-str\
etch\x22,\x22font-styl\
e\x22,\x22font-synthes\
is\x22,\x22font-varian\
t\x22,\x22font-variant\
-caps\x22,\x22font-var\
iant-east-asian\x22\
,\x22font-variant-l\
igatures\x22,\x22font-\
variant-numeric\x22\
,\x22font-variant-p\
osition\x22,\x22font-v\
ariation-setting\
s\x22,\x22font-weight\x22\
,\x22gap\x22,\x22glyph-or\
ientation-horizo\
ntal\x22,\x22glyph-ori\
entation-vertica\
l\x22,\x22grid\x22,\x22grid-\
area\x22,\x22grid-auto\
-columns\x22,\x22grid-\
auto-flow\x22,\x22grid\
-auto-rows\x22,\x22gri\
d-column\x22,\x22grid-\
column-end\x22,\x22gri\
d-column-start\x22,\
\x22grid-gap\x22,\x22grid\
-row\x22,\x22grid-row-\
end\x22,\x22grid-row-s\
tart\x22,\x22grid-temp\
late\x22,\x22grid-temp\
late-areas\x22,\x22gri\
d-template-colum\
ns\x22,\x22grid-templa\
te-rows\x22,\x22hangin\
g-punctuation\x22,\x22\
height\x22,\x22hyphens\
\x22,\x22icon\x22,\x22image-\
orientation\x22,\x22im\
age-rendering\x22,\x22\
image-resolution\
\x22,\x22ime-mode\x22,\x22in\
line-size\x22,\x22inse\
t\x22,\x22inset-block\x22\
,\x22inset-block-en\
d\x22,\x22inset-block-\
start\x22,\x22inset-in\
line\x22,\x22inset-inl\
ine-end\x22,\x22inset-\
inline-start\x22,\x22i\
solation\x22,\x22kerni\
ng\x22,\x22justify-con\
tent\x22,\x22justify-i\
tems\x22,\x22justify-s\
elf\x22,\x22left\x22,\x22let\
ter-spacing\x22,\x22li\
ghting-color\x22,\x22l\
ine-break\x22,\x22line\
-height\x22,\x22list-s\
tyle\x22,\x22list-styl\
e-image\x22,\x22list-s\
tyle-position\x22,\x22\
list-style-type\x22\
,\x22marker\x22,\x22marke\
r-end\x22,\x22marker-m\
id\x22,\x22marker-star\
t\x22,\x22mask\x22,\x22margi\
n\x22,\x22margin-block\
\x22,\x22margin-block-\
end\x22,\x22margin-blo\
ck-start\x22,\x22margi\
n-bottom\x22,\x22margi\
n-inline\x22,\x22margi\
n-inline-end\x22,\x22m\
argin-inline-sta\
rt\x22,\x22margin-left\
\x22,\x22margin-right\x22\
,\x22margin-top\x22,\x22m\
arks\x22,\x22mask\x22,\x22ma\
sk-border\x22,\x22mask\
-border-mode\x22,\x22m\
ask-border-outse\
t\x22,\x22mask-border-\
repeat\x22,\x22mask-bo\
rder-slice\x22,\x22mas\
k-border-source\x22\
,\x22mask-border-wi\
dth\x22,\x22mask-clip\x22\
,\x22mask-composite\
\x22,\x22mask-image\x22,\x22\
mask-mode\x22,\x22mask\
-origin\x22,\x22mask-p\
osition\x22,\x22mask-r\
epeat\x22,\x22mask-siz\
e\x22,\x22mask-type\x22,\x22\
max-block-size\x22,\
\x22max-height\x22,\x22ma\
x-inline-size\x22,\x22\
max-width\x22,\x22min-\
block-size\x22,\x22min\
-height\x22,\x22min-in\
line-size\x22,\x22min-\
width\x22,\x22mix-blen\
d-mode\x22,\x22nav-dow\
n\x22,\x22nav-index\x22,\x22\
nav-left\x22,\x22nav-r\
ight\x22,\x22nav-up\x22,\x22\
none\x22,\x22normal\x22,\x22\
object-fit\x22,\x22obj\
ect-position\x22,\x22o\
pacity\x22,\x22order\x22,\
\x22orphans\x22,\x22outli\
ne\x22,\x22outline-col\
or\x22,\x22outline-off\
set\x22,\x22outline-st\
yle\x22,\x22outline-wi\
dth\x22,\x22overflow\x22,\
\x22overflow-wrap\x22,\
\x22overflow-x\x22,\x22ov\
erflow-y\x22,\x22paddi\
ng\x22,\x22padding-blo\
ck\x22,\x22padding-blo\
ck-end\x22,\x22padding\
-block-start\x22,\x22p\
adding-bottom\x22,\x22\
padding-inline\x22,\
\x22padding-inline-\
end\x22,\x22padding-in\
line-start\x22,\x22pad\
ding-left\x22,\x22padd\
ing-right\x22,\x22padd\
ing-top\x22,\x22page-b\
reak-after\x22,\x22pag\
e-break-before\x22,\
\x22page-break-insi\
de\x22,\x22pause\x22,\x22pau\
se-after\x22,\x22pause\
-before\x22,\x22perspe\
ctive\x22,\x22perspect\
ive-origin\x22,\x22poi\
nter-events\x22,\x22po\
sition\x22,\x22quotes\x22\
,\x22r\x22,\x22resize\x22,\x22r\
est\x22,\x22rest-after\
\x22,\x22rest-before\x22,\
\x22right\x22,\x22rotate\x22\
,\x22row-gap\x22,\x22scal\
e\x22,\x22scroll-margi\
n\x22,\x22scroll-margi\
n-block\x22,\x22scroll\
-margin-block-en\
d\x22,\x22scroll-margi\
n-block-start\x22,\x22\
scroll-margin-bo\
ttom\x22,\x22scroll-ma\
rgin-inline\x22,\x22sc\
roll-margin-inli\
ne-end\x22,\x22scroll-\
margin-inline-st\
art\x22,\x22scroll-mar\
gin-left\x22,\x22scrol\
l-margin-right\x22,\
\x22scroll-margin-t\
op\x22,\x22scroll-padd\
ing\x22,\x22scroll-pad\
ding-block\x22,\x22scr\
oll-padding-bloc\
k-end\x22,\x22scroll-p\
adding-block-sta\
rt\x22,\x22scroll-padd\
ing-bottom\x22,\x22scr\
oll-padding-inli\
ne\x22,\x22scroll-padd\
ing-inline-end\x22,\
\x22scroll-padding-\
inline-start\x22,\x22s\
croll-padding-le\
ft\x22,\x22scroll-padd\
ing-right\x22,\x22scro\
ll-padding-top\x22,\
\x22scroll-snap-ali\
gn\x22,\x22scroll-snap\
-stop\x22,\x22scroll-s\
nap-type\x22,\x22scrol\
lbar-color\x22,\x22scr\
ollbar-gutter\x22,\x22\
scrollbar-width\x22\
,\x22shape-image-th\
reshold\x22,\x22shape-\
margin\x22,\x22shape-o\
utside\x22,\x22shape-r\
endering\x22,\x22stop-\
color\x22,\x22stop-opa\
city\x22,\x22stroke\x22,\x22\
stroke-dasharray\
\x22,\x22stroke-dashof\
fset\x22,\x22stroke-li\
necap\x22,\x22stroke-l\
inejoin\x22,\x22stroke\
-miterlimit\x22,\x22st\
roke-opacity\x22,\x22s\
troke-width\x22,\x22sp\
eak\x22,\x22speak-as\x22,\
\x22src\x22,\x22tab-size\x22\
,\x22table-layout\x22,\
\x22text-anchor\x22,\x22t\
ext-align\x22,\x22text\
-align-all\x22,\x22tex\
t-align-last\x22,\x22t\
ext-combine-upri\
ght\x22,\x22text-decor\
ation\x22,\x22text-dec\
oration-color\x22,\x22\
text-decoration-\
line\x22,\x22text-deco\
ration-skip-ink\x22\
,\x22text-decoratio\
n-style\x22,\x22text-d\
ecoration-thickn\
ess\x22,\x22text-empha\
sis\x22,\x22text-empha\
sis-color\x22,\x22text\
-emphasis-positi\
on\x22,\x22text-emphas\
is-style\x22,\x22text-\
indent\x22,\x22text-ju\
stify\x22,\x22text-ori\
entation\x22,\x22text-\
overflow\x22,\x22text-\
rendering\x22,\x22text\
-shadow\x22,\x22text-t\
ransform\x22,\x22text-\
underline-offset\
\x22,\x22text-underlin\
e-position\x22,\x22top\
\x22,\x22transform\x22,\x22t\
ransform-box\x22,\x22t\
ransform-origin\x22\
,\x22transform-styl\
e\x22,\x22transition\x22,\
\x22transition-dela\
y\x22,\x22transition-d\
uration\x22,\x22transi\
tion-property\x22,\x22\
transition-timin\
g-function\x22,\x22tra\
nslate\x22,\x22unicode\
-bidi\x22,\x22vector-e\
ffect\x22,\x22vertical\
-align\x22,\x22visibil\
ity\x22,\x22voice-bala\
nce\x22,\x22voice-dura\
tion\x22,\x22voice-fam\
ily\x22,\x22voice-pitc\
h\x22,\x22voice-range\x22\
,\x22voice-rate\x22,\x22v\
oice-stress\x22,\x22vo\
ice-volume\x22,\x22whi\
te-space\x22,\x22widow\
s\x22,\x22width\x22,\x22will\
-change\x22,\x22word-b\
reak\x22,\x22word-spac\
ing\x22,\x22word-wrap\x22\
,\x22writing-mode\x22,\
\x22x\x22,\x22y\x22,\x22z-index\
\x22].sort().revers\
e()\x0a;return n=>{\
const a=(e=>({IM\
PORTANT:{scope:\x22\
meta\x22,begin:\x22!im\
portant\x22},\x0aBLOCK\
_COMMENT:e.C_BLO\
CK_COMMENT_MODE,\
HEXCOLOR:{scope:\
\x22number\x22,\x0abegin:\
/#(([0-9a-fA-F]{\
3,4})|(([0-9a-fA\
-F]{2}){3,4}))\x5cb\
/},FUNCTION_DISP\
ATCH:{\x0aclassName\
:\x22built_in\x22,begi\
n:/[\x5cw-]+(?=\x5c()/\
},ATTRIBUTE_SELE\
CTOR_MODE:{\x0ascop\
e:\x22selector-attr\
\x22,begin:/\x5c[/,end\
:/\x5c]/,illegal:\x22$\
\x22,\x0acontains:[e.A\
POS_STRING_MODE,\
e.QUOTE_STRING_M\
ODE]},CSS_NUMBER\
_MODE:{\x0ascope:\x22n\
umber\x22,\x0abegin:e.\
NUMBER_RE+\x22(%|em\
|ex|ch|rem|vw|vh\
|vmin|vmax|cm|mm\
|in|pt|pc|px|deg\
|grad|rad|turn|s\
|ms|Hz|kHz|dpi|d\
pcm|dppx)?\x22,\x0arel\
evance:0},CSS_VA\
RIABLE:{classNam\
e:\x22attr\x22,begin:/\
--[A-Za-z_][A-Za\
-z0-9_-]*/}\x0a}))(\
n),l=i,s=t,d=\x22@[\
a-z-]+\x22,c={class\
Name:\x22variable\x22,\
\x0abegin:\x22(\x5c\x5c$[a-z\
A-Z-][a-zA-Z0-9_\
-]*)\x5c\x5cb\x22,relevan\
ce:0};return{nam\
e:\x22SCSS\x22,\x0acase_i\
nsensitive:!0,il\
legal:\x22[=/|']\x22,\x0a\
contains:[n.C_LI\
NE_COMMENT_MODE,\
n.C_BLOCK_COMMEN\
T_MODE,a.CSS_NUM\
BER_MODE,{\x0aclass\
Name:\x22selector-i\
d\x22,begin:\x22#[A-Za\
-z0-9_-]+\x22,relev\
ance:0},{\x0aclassN\
ame:\x22selector-cl\
ass\x22,begin:\x22\x5c\x5c.[\
A-Za-z0-9_-]+\x22,r\
elevance:0\x0a},a.A\
TTRIBUTE_SELECTO\
R_MODE,{classNam\
e:\x22selector-tag\x22\
,\x0abegin:\x22\x5c\x5cb(\x22+e\
.join(\x22|\x22)+\x22)\x5c\x5cb\
\x22,relevance:0},{\
className:\x22selec\
tor-pseudo\x22,\x0abeg\
in:\x22:(\x22+s.join(\x22\
|\x22)+\x22)\x22},{classN\
ame:\x22selector-ps\
eudo\x22,\x0abegin:\x22:(\
:)?(\x22+l.join(\x22|\x22\
)+\x22)\x22},c,{begin:\
/\x5c(/,end:/\x5c)/,\x0ac\
ontains:[a.CSS_N\
UMBER_MODE]},a.C\
SS_VARIABLE,{cla\
ssName:\x22attribut\
e\x22,\x0abegin:\x22\x5c\x5cb(\x22\
+o.join(\x22|\x22)+\x22)\x5c\
\x5cb\x22},{\x0abegin:\x22\x5c\x5c\
b(whitespace|wai\
t|w-resize|visib\
le|vertical-text\
|vertical-ideogr\
aphic|uppercase|\
upper-roman|uppe\
r-alpha|underlin\
e|transparent|to\
p|thin|thick|tex\
t|text-top|text-\
bottom|tb-rl|tab\
le-header-group|\
table-footer-gro\
up|sw-resize|sup\
er|strict|static\
|square|solid|sm\
all-caps|separat\
e|se-resize|scro\
ll|s-resize|rtl|\
row-resize|ridge\
|right|repeat|re\
peat-y|repeat-x|\
relative|progres\
s|pointer|overli\
ne|outside|outse\
t|oblique|nowrap\
|not-allowed|nor\
mal|none|nw-resi\
ze|no-repeat|no-\
drop|newspaper|n\
e-resize|n-resiz\
e|move|middle|me\
dium|ltr|lr-tb|l\
owercase|lower-r\
oman|lower-alpha\
|loose|list-item\
|line|line-throu\
gh|line-edge|lig\
hter|left|keep-a\
ll|justify|itali\
c|inter-word|int\
er-ideograph|ins\
ide|inset|inline\
|inline-block|in\
herit|inactive|i\
deograph-space|i\
deograph-parenth\
esis|ideograph-n\
umeric|ideograph\
-alpha|horizonta\
l|hidden|help|ha\
nd|groove|fixed|\
ellipsis|e-resiz\
e|double|dotted|\
distribute|distr\
ibute-space|dist\
ribute-letter|di\
stribute-all-lin\
es|disc|disabled\
|default|decimal\
|dashed|crosshai\
r|collapse|col-r\
esize|circle|cha\
r|center|capital\
ize|break-word|b\
reak-all|bottom|\
both|bolder|bold\
|block|bidi-over\
ride|below|basel\
ine|auto|always|\
all-scroll|absol\
ute|table|table-\
cell)\x5c\x5cb\x22\x0a},{beg\
in:/:/,end:/[;}{\
]/,relevance:0,\x0a\
contains:[a.BLOC\
K_COMMENT,c,a.HE\
XCOLOR,a.CSS_NUM\
BER_MODE,n.QUOTE\
_STRING_MODE,n.A\
POS_STRING_MODE,\
a.IMPORTANT,a.FU\
NCTION_DISPATCH]\
\x0a},{begin:\x22@(pag\
e|font-face)\x22,ke\
ywords:{$pattern\
:d,keyword:\x22@pag\
e @font-face\x22}},\
{\x0abegin:\x22@\x22,end:\
\x22[{;]\x22,returnBeg\
in:!0,keywords:{\
$pattern:/[a-z-]\
+/,\x0akeyword:\x22and\
 or not only\x22,at\
tribute:r.join(\x22\
 \x22)},contains:[{\
begin:d,\x0aclassNa\
me:\x22keyword\x22},{b\
egin:/[a-z-]+(?=\
:)/,className:\x22a\
ttribute\x22\x0a},c,n.\
QUOTE_STRING_MOD\
E,n.APOS_STRING_\
MODE,a.HEXCOLOR,\
a.CSS_NUMBER_MOD\
E]\x0a},a.FUNCTION_\
DISPATCH]}}})();\
hljs.registerLan\
guage(\x22scss\x22,e)}\
)();/*! `shell` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var s=(()=>{\x22\
use strict\x22;retu\
rn s=>({name:\x22Sh\
ell Session\x22,\x0aal\
iases:[\x22console\x22\
,\x22shellsession\x22]\
,contains:[{clas\
sName:\x22meta.prom\
pt\x22,\x0abegin:/^\x5cs{\
0,3}[/~\x5cw\x5cd[\x5c]()\
@-]*[>%$#][ ]?/,\
starts:{end:/[^\x5c\
\x5c](?=\x5cs*$)/,\x0asub\
Language:\x22bash\x22}\
}]})})();hljs.re\
gisterLanguage(\x22\
shell\x22,s)})();/*\
! `smali` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
{\x0aconst n=[\x22add\x22\
,\x22and\x22,\x22cmp\x22,\x22cm\
pg\x22,\x22cmpl\x22,\x22cons\
t\x22,\x22div\x22,\x22double\
\x22,\x22float\x22,\x22goto\x22\
,\x22if\x22,\x22int\x22,\x22lon\
g\x22,\x22move\x22,\x22mul\x22,\
\x22neg\x22,\x22new\x22,\x22nop\
\x22,\x22not\x22,\x22or\x22,\x22re\
m\x22,\x22return\x22,\x22shl\
\x22,\x22shr\x22,\x22sput\x22,\x22\
sub\x22,\x22throw\x22,\x22us\
hr\x22,\x22xor\x22]\x0a;retu\
rn{name:\x22Smali\x22,\
contains:[{class\
Name:\x22string\x22,be\
gin:'\x22',end:'\x22',\
relevance:0\x0a},e.\
COMMENT(\x22#\x22,\x22$\x22,\
{relevance:0}),{\
className:\x22keywo\
rd\x22,variants:[{\x0a\
begin:\x22\x5c\x5cs*\x5c\x5c.en\
d\x5c\x5cs[a-zA-Z0-9]*\
\x22},{begin:\x22^[ ]*\
\x5c\x5c.[a-zA-Z]*\x22,re\
levance:0},{\x0abeg\
in:\x22\x5c\x5cs:[a-zA-Z_\
0-9]*\x22,relevance\
:0},{\x0abegin:\x22\x5c\x5cs\
(transient|const\
ructor|abstract|\
final|synthetic|\
public|private|p\
rotected|static|\
bridge|system)\x22\x0a\
}]},{className:\x22\
built_in\x22,varian\
ts:[{begin:\x22\x5c\x5cs(\
\x22+n.join(\x22|\x22)+\x22)\
\x5c\x5cs\x22},{\x0abegin:\x22\x5c\
\x5cs(\x22+n.join(\x22|\x22)\
+\x22)((-|/)[a-zA-Z\
0-9]+)+\x5c\x5cs\x22,rele\
vance:10},{\x0abegi\
n:\x22\x5c\x5cs(aget|aput\
|array|check|exe\
cute|fill|filled\
|goto/16|goto/32\
|iget|instance|i\
nvoke|iput|monit\
or|packed|sget|s\
parse)((-|/)[a-z\
A-Z0-9]+)*\x5c\x5cs\x22,\x0a\
relevance:10}]},\
{className:\x22clas\
s\x22,begin:\x22L[^(;:\
\x5cn]*;\x22,relevance\
:0},{\x0abegin:\x22[vp\
][0-9]+\x22}]}}})()\
;hljs.registerLa\
nguage(\x22smali\x22,e\
)})();/*! `small\
talk` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>{con\
st n=\x22[a-z][a-zA\
-Z0-9_]*\x22,a={\x0acl\
assName:\x22string\x22\
,begin:\x22\x5c\x5c$.{1}\x22\
},s={className:\x22\
symbol\x22,\x0abegin:\x22\
#\x22+e.UNDERSCORE_\
IDENT_RE};return\
{name:\x22Smalltalk\
\x22,aliases:[\x22st\x22]\
,\x0akeywords:[\x22sel\
f\x22,\x22super\x22,\x22nil\x22\
,\x22true\x22,\x22false\x22,\
\x22thisContext\x22],\x0a\
contains:[e.COMM\
ENT('\x22','\x22'),e.A\
POS_STRING_MODE,\
{className:\x22type\
\x22,\x0abegin:\x22\x5c\x5cb[A-\
Z][A-Za-z0-9_]*\x22\
,relevance:0},{b\
egin:n+\x22:\x22,relev\
ance:0\x0a},e.C_NUM\
BER_MODE,s,a,{be\
gin:\x22\x5c\x5c|[ ]*\x22+n+\
\x22([ ]+\x22+n+\x22)*[ ]\
*\x5c\x5c|\x22,returnBegi\
n:!0,\x0aend:/\x5c|/,i\
llegal:/\x5cS/,cont\
ains:[{begin:\x22(\x5c\
\x5c|[ ]*)?\x22+n}]},{\
begin:\x22#\x5c\x5c(\x22,\x0aen\
d:\x22\x5c\x5c)\x22,contains\
:[e.APOS_STRING_\
MODE,a,e.C_NUMBE\
R_MODE,s]}]}}})(\
)\x0a;hljs.register\
Language(\x22smallt\
alk\x22,e)})();/*! \
`sml` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>({na\
me:\x22SML (Standar\
d ML)\x22,\x0aaliases:\
[\x22ml\x22],keywords:\
{$pattern:\x22[a-z_\
]\x5c\x5cw*!?\x22,\x0akeywor\
d:\x22abstype and a\
ndalso as case d\
atatype do else \
end eqtype excep\
tion fn fun func\
tor handle if in\
 include infix i\
nfixr let local \
nonfix of op ope\
n orelse raise r\
ec sharing sig s\
ignature struct \
structure then t\
ype val with wit\
htype where whil\
e\x22,\x0abuilt_in:\x22ar\
ray bool char ex\
n int list optio\
n order real ref\
 string substrin\
g vector unit wo\
rd\x22,\x0aliteral:\x22tr\
ue false NONE SO\
ME LESS EQUAL GR\
EATER nil\x22},ille\
gal:/\x5c/\x5c/|>>/,\x0ac\
ontains:[{classN\
ame:\x22literal\x22,be\
gin:/\x5c[(\x5c|\x5c|)?\x5c]\
|\x5c(\x5c)/,relevance\
:0\x0a},e.COMMENT(\x22\
\x5c\x5c(\x5c\x5c*\x22,\x22\x5c\x5c*\x5c\x5c)\x22\
,{contains:[\x22sel\
f\x22]}),{className\
:\x22symbol\x22,\x0abegin\
:\x22'[A-Za-z_](?!'\
)[\x5c\x5cw']*\x22},{clas\
sName:\x22type\x22,beg\
in:\x22`[A-Z][\x5c\x5cw']\
*\x22},{\x0aclassName:\
\x22type\x22,begin:\x22\x5c\x5c\
b[A-Z][\x5c\x5cw']*\x22,r\
elevance:0},{\x0abe\
gin:\x22[a-z_]\x5c\x5cw*'\
[\x5c\x5cw']*\x22},e.inhe\
rit(e.APOS_STRIN\
G_MODE,{classNam\
e:\x22string\x22,\x0arele\
vance:0}),e.inhe\
rit(e.QUOTE_STRI\
NG_MODE,{illegal\
:null}),{classNa\
me:\x22number\x22,\x0abeg\
in:\x22\x5c\x5cb(0[xX][a-\
fA-F0-9_]+[Lln]?\
|0[oO][0-7_]+[Ll\
n]?|0[bB][01_]+[\
Lln]?|[0-9][0-9_\
]*([Lln]|(\x5c\x5c.[0-\
9_]*)?([eE][-+]?\
[0-9_]+)?)?)\x22,\x0ar\
elevance:0},{beg\
in:/[-=]>/}]})})\
();hljs.register\
Language(\x22sml\x22,e\
)})();/*! `sqf` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{const t={\
className:\x22strin\
g\x22,\x0avariants:[{b\
egin:'\x22',end:'\x22'\
,contains:[{begi\
n:'\x22\x22',relevance\
:0}]},{begin:\x22'\x22\
,\x0aend:\x22'\x22,contai\
ns:[{begin:\x22''\x22,\
relevance:0}]}]}\
,a={className:\x22m\
eta\x22,\x0abegin:/#\x5cs\
*[a-z]+\x5cb/,end:/\
$/,\x0akeywords:\x22de\
fine undef ifdef\
 ifndef else end\
if include if\x22,c\
ontains:[{\x0abegin\
:/\x5c\x5c\x5cn/,relevanc\
e:0},e.inherit(t\
,{className:\x22str\
ing\x22}),{begin:/<\
[^\x5cn>]*>/,\x0aend:/\
$/,illegal:\x22\x5c\x5cn\x22\
},e.C_LINE_COMME\
NT_MODE,e.C_BLOC\
K_COMMENT_MODE]}\
;return{\x0aname:\x22S\
QF\x22,case_insensi\
tive:!0,keywords\
:{\x0akeyword:[\x22bre\
ak\x22,\x22breakWith\x22,\
\x22breakOut\x22,\x22brea\
kTo\x22,\x22case\x22,\x22cat\
ch\x22,\x22continue\x22,\x22\
continueWith\x22,\x22d\
efault\x22,\x22do\x22,\x22el\
se\x22,\x22exit\x22,\x22exit\
With\x22,\x22for\x22,\x22for\
Each\x22,\x22from\x22,\x22if\
\x22,\x22local\x22,\x22priva\
te\x22,\x22switch\x22,\x22st\
ep\x22,\x22then\x22,\x22thro\
w\x22,\x22to\x22,\x22try\x22,\x22w\
aitUntil\x22,\x22while\
\x22,\x22with\x22],\x0abuilt\
_in:[\x22abs\x22,\x22accT\
ime\x22,\x22acos\x22,\x22act\
ion\x22,\x22actionIDs\x22\
,\x22actionKeys\x22,\x22a\
ctionKeysEx\x22,\x22ac\
tionKeysImages\x22,\
\x22actionKeysNames\
\x22,\x22actionKeysNam\
esArray\x22,\x22action\
Name\x22,\x22actionPar\
ams\x22,\x22activateAd\
dons\x22,\x22activated\
Addons\x22,\x22activat\
eKey\x22,\x22activeTit\
leEffectParams\x22,\
\x22add3DENConnecti\
on\x22,\x22add3DENEven\
tHandler\x22,\x22add3D\
ENLayer\x22,\x22addAct\
ion\x22,\x22addBackpac\
k\x22,\x22addBackpackC\
argo\x22,\x22addBackpa\
ckCargoGlobal\x22,\x22\
addBackpackGloba\
l\x22,\x22addBinocular\
Item\x22,\x22addCamSha\
ke\x22,\x22addCuratorA\
ddons\x22,\x22addCurat\
orCameraArea\x22,\x22a\
ddCuratorEditabl\
eObjects\x22,\x22addCu\
ratorEditingArea\
\x22,\x22addCuratorPoi\
nts\x22,\x22addEditorO\
bject\x22,\x22addEvent\
Handler\x22,\x22addFor\
ce\x22,\x22addForceGen\
eratorRTD\x22,\x22addG\
oggles\x22,\x22addGrou\
pIcon\x22,\x22addHandg\
unItem\x22,\x22addHead\
gear\x22,\x22addItem\x22,\
\x22addItemCargo\x22,\x22\
addItemCargoGlob\
al\x22,\x22addItemPool\
\x22,\x22addItemToBack\
pack\x22,\x22addItemTo\
Uniform\x22,\x22addIte\
mToVest\x22,\x22addLiv\
eStats\x22,\x22addMaga\
zine\x22,\x22addMagazi\
neAmmoCargo\x22,\x22ad\
dMagazineCargo\x22,\
\x22addMagazineCarg\
oGlobal\x22,\x22addMag\
azineGlobal\x22,\x22ad\
dMagazinePool\x22,\x22\
addMagazines\x22,\x22a\
ddMagazineTurret\
\x22,\x22addMenu\x22,\x22add\
MenuItem\x22,\x22addMi\
ssionEventHandle\
r\x22,\x22addMPEventHa\
ndler\x22,\x22addMusic\
EventHandler\x22,\x22a\
ddonFiles\x22,\x22addO\
wnedMine\x22,\x22addPl\
ayerScores\x22,\x22add\
PrimaryWeaponIte\
m\x22,\x22addPublicVar\
iableEventHandle\
r\x22,\x22addRating\x22,\x22\
addResources\x22,\x22a\
ddScore\x22,\x22addSco\
reSide\x22,\x22addSeco\
ndaryWeaponItem\x22\
,\x22addSwitchableU\
nit\x22,\x22addTeamMem\
ber\x22,\x22addToRemai\
nsCollector\x22,\x22ad\
dTorque\x22,\x22addUni\
form\x22,\x22addUserAc\
tionEventHandler\
\x22,\x22addVehicle\x22,\x22\
addVest\x22,\x22addWay\
point\x22,\x22addWeapo\
n\x22,\x22addWeaponCar\
go\x22,\x22addWeaponCa\
rgoGlobal\x22,\x22addW\
eaponGlobal\x22,\x22ad\
dWeaponItem\x22,\x22ad\
dWeaponPool\x22,\x22ad\
dWeaponTurret\x22,\x22\
addWeaponWithAtt\
achmentsCargo\x22,\x22\
addWeaponWithAtt\
achmentsCargoGlo\
bal\x22,\x22admin\x22,\x22ag\
ent\x22,\x22agents\x22,\x22A\
GLToASL\x22,\x22aimedA\
tTarget\x22,\x22aimPos\
\x22,\x22airDensityCur\
veRTD\x22,\x22airDensi\
tyRTD\x22,\x22airplane\
Throttle\x22,\x22airpo\
rtSide\x22,\x22AISFini\
shHeal\x22,\x22alive\x22,\
\x22all3DENEntities\
\x22,\x22allActiveTitl\
eEffects\x22,\x22allAd\
donsInfo\x22,\x22allAi\
rports\x22,\x22allCont\
rols\x22,\x22allCurato\
rs\x22,\x22allCutLayer\
s\x22,\x22allDead\x22,\x22al\
lDeadMen\x22,\x22allDi\
aryRecords\x22,\x22all\
DiarySubjects\x22,\x22\
allDisplays\x22,\x22al\
lEnv3DSoundSourc\
es\x22,\x22allGroups\x22,\
\x22allLODs\x22,\x22allMa\
pMarkers\x22,\x22allMi\
nes\x22,\x22allMission\
Objects\x22,\x22allObj\
ects\x22,\x22allow3DMo\
de\x22,\x22allowCrewIn\
Immobile\x22,\x22allow\
CuratorLogicIgno\
reAreas\x22,\x22allowD\
amage\x22,\x22allowDam\
mage\x22,\x22allowedSe\
rvice\x22,\x22allowFil\
eOperations\x22,\x22al\
lowFleeing\x22,\x22all\
owGetIn\x22,\x22allowS\
ervice\x22,\x22allowSp\
rint\x22,\x22allPlayer\
s\x22,\x22allSimpleObj\
ects\x22,\x22allSites\x22\
,\x22allTurrets\x22,\x22a\
llUnits\x22,\x22allUni\
tsUAV\x22,\x22allUsers\
\x22,\x22allVariables\x22\
,\x22ambientTempera\
ture\x22,\x22ammo\x22,\x22am\
moOnPylon\x22,\x22and\x22\
,\x22animate\x22,\x22anim\
ateBay\x22,\x22animate\
Door\x22,\x22animatePy\
lon\x22,\x22animateSou\
rce\x22,\x22animationN\
ames\x22,\x22animation\
Phase\x22,\x22animatio\
nSourcePhase\x22,\x22a\
nimationState\x22,\x22\
apertureParams\x22,\
\x22append\x22,\x22apply\x22\
,\x22armoryPoints\x22,\
\x22arrayIntersect\x22\
,\x22asin\x22,\x22ASLToAG\
L\x22,\x22ASLToATL\x22,\x22a\
ssert\x22,\x22assignAs\
Cargo\x22,\x22assignAs\
CargoIndex\x22,\x22ass\
ignAsCommander\x22,\
\x22assignAsDriver\x22\
,\x22assignAsGunner\
\x22,\x22assignAsTurre\
t\x22,\x22assignCurato\
r\x22,\x22assignedCarg\
o\x22,\x22assignedComm\
ander\x22,\x22assigned\
Driver\x22,\x22assigne\
dGroup\x22,\x22assigne\
dGunner\x22,\x22assign\
edItems\x22,\x22assign\
edTarget\x22,\x22assig\
nedTeam\x22,\x22assign\
edVehicle\x22,\x22assi\
gnedVehicleRole\x22\
,\x22assignedVehicl\
es\x22,\x22assignItem\x22\
,\x22assignTeam\x22,\x22a\
ssignToAirport\x22,\
\x22atan\x22,\x22atan2\x22,\x22\
atg\x22,\x22ATLToASL\x22,\
\x22attachedObject\x22\
,\x22attachedObject\
s\x22,\x22attachedTo\x22,\
\x22attachObject\x22,\x22\
attachTo\x22,\x22attac\
kEnabled\x22,\x22awake\
\x22,\x22backpack\x22,\x22ba\
ckpackCargo\x22,\x22ba\
ckpackContainer\x22\
,\x22backpackItems\x22\
,\x22backpackMagazi\
nes\x22,\x22backpackSp\
aceFor\x22,\x22behavio\
ur\x22,\x22benchmark\x22,\
\x22bezierInterpola\
tion\x22,\x22binocular\
\x22,\x22binocularItem\
s\x22,\x22binocularMag\
azine\x22,\x22bounding\
Box\x22,\x22boundingBo\
xReal\x22,\x22bounding\
Center\x22,\x22brakesD\
isabled\x22,\x22briefi\
ngName\x22,\x22buildin\
gExit\x22,\x22building\
Pos\x22,\x22buldozer_E\
nableRoadDiag\x22,\x22\
buldozer_IsEnabl\
edRoadDiag\x22,\x22bul\
dozer_LoadNewRoa\
ds\x22,\x22buldozer_re\
loadOperMap\x22,\x22bu\
ttonAction\x22,\x22but\
tonSetAction\x22,\x22c\
adetMode\x22,\x22calcu\
latePath\x22,\x22calcu\
latePlayerVisibi\
lityByFriendly\x22,\
\x22call\x22,\x22callExte\
nsion\x22,\x22camComma\
nd\x22,\x22camCommit\x22,\
\x22camCommitPrepar\
ed\x22,\x22camCommitte\
d\x22,\x22camConstucti\
onSetParams\x22,\x22ca\
mCreate\x22,\x22camDes\
troy\x22,\x22cameraEff\
ect\x22,\x22cameraEffe\
ctEnableHUD\x22,\x22ca\
meraInterest\x22,\x22c\
ameraOn\x22,\x22camera\
View\x22,\x22campaignC\
onfigFile\x22,\x22camP\
reload\x22,\x22camPrel\
oaded\x22,\x22camPrepa\
reBank\x22,\x22camPrep\
areDir\x22,\x22camPrep\
areDive\x22,\x22camPre\
pareFocus\x22,\x22camP\
repareFov\x22,\x22camP\
repareFovRange\x22,\
\x22camPreparePos\x22,\
\x22camPrepareRelPo\
s\x22,\x22camPrepareTa\
rget\x22,\x22camSetBan\
k\x22,\x22camSetDir\x22,\x22\
camSetDive\x22,\x22cam\
SetFocus\x22,\x22camSe\
tFov\x22,\x22camSetFov\
Range\x22,\x22camSetPo\
s\x22,\x22camSetRelPos\
\x22,\x22camSetTarget\x22\
,\x22camTarget\x22,\x22ca\
mUseNVG\x22,\x22canAdd\
\x22,\x22canAddItemToB\
ackpack\x22,\x22canAdd\
ItemToUniform\x22,\x22\
canAddItemToVest\
\x22,\x22cancelSimpleT\
askDestination\x22,\
\x22canDeployWeapon\
\x22,\x22canFire\x22,\x22can\
Move\x22,\x22canSlingL\
oad\x22,\x22canStand\x22,\
\x22canSuspend\x22,\x22ca\
nTriggerDynamicS\
imulation\x22,\x22canU\
nloadInCombat\x22,\x22\
canVehicleCargo\x22\
,\x22captive\x22,\x22capt\
iveNum\x22,\x22cbCheck\
ed\x22,\x22cbSetChecke\
d\x22,\x22ceil\x22,\x22chann\
elEnabled\x22,\x22chea\
tsEnabled\x22,\x22chec\
kAIFeature\x22,\x22che\
ckVisibility\x22,\x22c\
lassName\x22,\x22clear\
3DENAttribute\x22,\x22\
clear3DENInvento\
ry\x22,\x22clearAllIte\
msFromBackpack\x22,\
\x22clearBackpackCa\
rgo\x22,\x22clearBackp\
ackCargoGlobal\x22,\
\x22clearForcesRTD\x22\
,\x22clearGroupIcon\
s\x22,\x22clearItemCar\
go\x22,\x22clearItemCa\
rgoGlobal\x22,\x22clea\
rItemPool\x22,\x22clea\
rMagazineCargo\x22,\
\x22clearMagazineCa\
rgoGlobal\x22,\x22clea\
rMagazinePool\x22,\x22\
clearOverlay\x22,\x22c\
learRadio\x22,\x22clea\
rWeaponCargo\x22,\x22c\
learWeaponCargoG\
lobal\x22,\x22clearWea\
ponPool\x22,\x22client\
Owner\x22,\x22closeDia\
log\x22,\x22closeDispl\
ay\x22,\x22closeOverla\
y\x22,\x22collapseObje\
ctTree\x22,\x22collect\
3DENHistory\x22,\x22co\
llectiveRTD\x22,\x22co\
llisionDisabledW\
ith\x22,\x22combatBeha\
viour\x22,\x22combatMo\
de\x22,\x22commandArti\
lleryFire\x22,\x22comm\
andChat\x22,\x22comman\
der\x22,\x22commandFir\
e\x22,\x22commandFollo\
w\x22,\x22commandFSM\x22,\
\x22commandGetOut\x22,\
\x22commandingMenu\x22\
,\x22commandMove\x22,\x22\
commandRadio\x22,\x22c\
ommandStop\x22,\x22com\
mandSuppressiveF\
ire\x22,\x22commandTar\
get\x22,\x22commandWat\
ch\x22,\x22comment\x22,\x22c\
ommitOverlay\x22,\x22c\
ompatibleItems\x22,\
\x22compatibleMagaz\
ines\x22,\x22compile\x22,\
\x22compileFinal\x22,\x22\
compileScript\x22,\x22\
completedFSM\x22,\x22c\
omposeText\x22,\x22con\
figClasses\x22,\x22con\
figFile\x22,\x22config\
Hierarchy\x22,\x22conf\
igName\x22,\x22configO\
f\x22,\x22configProper\
ties\x22,\x22configSou\
rceAddonList\x22,\x22c\
onfigSourceMod\x22,\
\x22configSourceMod\
List\x22,\x22confirmSe\
nsorTarget\x22,\x22con\
nectTerminalToUA\
V\x22,\x22connectToSer\
ver\x22,\x22controlsGr\
oupCtrl\x22,\x22conver\
sationDisabled\x22,\
\x22copyFromClipboa\
rd\x22,\x22copyToClipb\
oard\x22,\x22copyWaypo\
ints\x22,\x22cos\x22,\x22cou\
nt\x22,\x22countEnemy\x22\
,\x22countFriendly\x22\
,\x22countSide\x22,\x22co\
untType\x22,\x22countU\
nknown\x22,\x22create3\
DENComposition\x22,\
\x22create3DENEntit\
y\x22,\x22createAgent\x22\
,\x22createCenter\x22,\
\x22createDialog\x22,\x22\
createDiaryLink\x22\
,\x22createDiaryRec\
ord\x22,\x22createDiar\
ySubject\x22,\x22creat\
eDisplay\x22,\x22creat\
eGearDialog\x22,\x22cr\
eateGroup\x22,\x22crea\
teGuardedPoint\x22,\
\x22createHashMap\x22,\
\x22createHashMapFr\
omArray\x22,\x22create\
Location\x22,\x22creat\
eMarker\x22,\x22create\
MarkerLocal\x22,\x22cr\
eateMenu\x22,\x22creat\
eMine\x22,\x22createMi\
ssionDisplay\x22,\x22c\
reateMPCampaignD\
isplay\x22,\x22createS\
impleObject\x22,\x22cr\
eateSimpleTask\x22,\
\x22createSite\x22,\x22cr\
eateSoundSource\x22\
,\x22createTask\x22,\x22c\
reateTeam\x22,\x22crea\
teTrigger\x22,\x22crea\
teUnit\x22,\x22createV\
ehicle\x22,\x22createV\
ehicleCrew\x22,\x22cre\
ateVehicleLocal\x22\
,\x22crew\x22,\x22ctAddHe\
ader\x22,\x22ctAddRow\x22\
,\x22ctClear\x22,\x22ctCu\
rSel\x22,\x22ctData\x22,\x22\
ctFindHeaderRows\
\x22,\x22ctFindRowHead\
er\x22,\x22ctHeaderCon\
trols\x22,\x22ctHeader\
Count\x22,\x22ctRemove\
Headers\x22,\x22ctRemo\
veRows\x22,\x22ctrlAct\
ivate\x22,\x22ctrlAddE\
ventHandler\x22,\x22ct\
rlAngle\x22,\x22ctrlAn\
imateModel\x22,\x22ctr\
lAnimationPhaseM\
odel\x22,\x22ctrlAt\x22,\x22\
ctrlAutoScrollDe\
lay\x22,\x22ctrlAutoSc\
rollRewind\x22,\x22ctr\
lAutoScrollSpeed\
\x22,\x22ctrlBackgroun\
dColor\x22,\x22ctrlChe\
cked\x22,\x22ctrlClass\
Name\x22,\x22ctrlCommi\
t\x22,\x22ctrlCommitte\
d\x22,\x22ctrlCreate\x22,\
\x22ctrlDelete\x22,\x22ct\
rlEnable\x22,\x22ctrlE\
nabled\x22,\x22ctrlFad\
e\x22,\x22ctrlFontHeig\
ht\x22,\x22ctrlForegro\
undColor\x22,\x22ctrlH\
TMLLoaded\x22,\x22ctrl\
IDC\x22,\x22ctrlIDD\x22,\x22\
ctrlMapAnimAdd\x22,\
\x22ctrlMapAnimClea\
r\x22,\x22ctrlMapAnimC\
ommit\x22,\x22ctrlMapA\
nimDone\x22,\x22ctrlMa\
pCursor\x22,\x22ctrlMa\
pMouseOver\x22,\x22ctr\
lMapPosition\x22,\x22c\
trlMapScale\x22,\x22ct\
rlMapScreenToWor\
ld\x22,\x22ctrlMapSetP\
osition\x22,\x22ctrlMa\
pWorldToScreen\x22,\
\x22ctrlModel\x22,\x22ctr\
lModelDirAndUp\x22,\
\x22ctrlModelScale\x22\
,\x22ctrlMousePosit\
ion\x22,\x22ctrlParent\
\x22,\x22ctrlParentCon\
trolsGroup\x22,\x22ctr\
lPosition\x22,\x22ctrl\
RemoveAllEventHa\
ndlers\x22,\x22ctrlRem\
oveEventHandler\x22\
,\x22ctrlScale\x22,\x22ct\
rlScrollValues\x22,\
\x22ctrlSetActiveCo\
lor\x22,\x22ctrlSetAng\
le\x22,\x22ctrlSetAuto\
ScrollDelay\x22,\x22ct\
rlSetAutoScrollR\
ewind\x22,\x22ctrlSetA\
utoScrollSpeed\x22,\
\x22ctrlSetBackgrou\
ndColor\x22,\x22ctrlSe\
tChecked\x22,\x22ctrlS\
etDisabledColor\x22\
,\x22ctrlSetEventHa\
ndler\x22,\x22ctrlSetF\
ade\x22,\x22ctrlSetFoc\
us\x22,\x22ctrlSetFont\
\x22,\x22ctrlSetFontH1\
\x22,\x22ctrlSetFontH1\
B\x22,\x22ctrlSetFontH\
2\x22,\x22ctrlSetFontH\
2B\x22,\x22ctrlSetFont\
H3\x22,\x22ctrlSetFont\
H3B\x22,\x22ctrlSetFon\
tH4\x22,\x22ctrlSetFon\
tH4B\x22,\x22ctrlSetFo\
ntH5\x22,\x22ctrlSetFo\
ntH5B\x22,\x22ctrlSetF\
ontH6\x22,\x22ctrlSetF\
ontH6B\x22,\x22ctrlSet\
FontHeight\x22,\x22ctr\
lSetFontHeightH1\
\x22,\x22ctrlSetFontHe\
ightH2\x22,\x22ctrlSet\
FontHeightH3\x22,\x22c\
trlSetFontHeight\
H4\x22,\x22ctrlSetFont\
HeightH5\x22,\x22ctrlS\
etFontHeightH6\x22,\
\x22ctrlSetFontHeig\
htSecondary\x22,\x22ct\
rlSetFontP\x22,\x22ctr\
lSetFontPB\x22,\x22ctr\
lSetFontSecondar\
y\x22,\x22ctrlSetForeg\
roundColor\x22,\x22ctr\
lSetModel\x22,\x22ctrl\
SetModelDirAndUp\
\x22,\x22ctrlSetModelS\
cale\x22,\x22ctrlSetMo\
usePosition\x22,\x22ct\
rlSetPixelPrecis\
ion\x22,\x22ctrlSetPos\
ition\x22,\x22ctrlSetP\
ositionH\x22,\x22ctrlS\
etPositionW\x22,\x22ct\
rlSetPositionX\x22,\
\x22ctrlSetPosition\
Y\x22,\x22ctrlSetScale\
\x22,\x22ctrlSetScroll\
Values\x22,\x22ctrlSet\
Shadow\x22,\x22ctrlSet\
StructuredText\x22,\
\x22ctrlSetText\x22,\x22c\
trlSetTextColor\x22\
,\x22ctrlSetTextCol\
orSecondary\x22,\x22ct\
rlSetTextSeconda\
ry\x22,\x22ctrlSetText\
Selection\x22,\x22ctrl\
SetTooltip\x22,\x22ctr\
lSetTooltipColor\
Box\x22,\x22ctrlSetToo\
ltipColorShade\x22,\
\x22ctrlSetTooltipC\
olorText\x22,\x22ctrlS\
etTooltipMaxWidt\
h\x22,\x22ctrlSetURL\x22,\
\x22ctrlSetURLOverl\
ayMode\x22,\x22ctrlSha\
dow\x22,\x22ctrlShow\x22,\
\x22ctrlShown\x22,\x22ctr\
lStyle\x22,\x22ctrlTex\
t\x22,\x22ctrlTextColo\
r\x22,\x22ctrlTextHeig\
ht\x22,\x22ctrlTextSec\
ondary\x22,\x22ctrlTex\
tSelection\x22,\x22ctr\
lTextWidth\x22,\x22ctr\
lTooltip\x22,\x22ctrlT\
ype\x22,\x22ctrlURL\x22,\x22\
ctrlURLOverlayMo\
de\x22,\x22ctrlVisible\
\x22,\x22ctRowControls\
\x22,\x22ctRowCount\x22,\x22\
ctSetCurSel\x22,\x22ct\
SetData\x22,\x22ctSetH\
eaderTemplate\x22,\x22\
ctSetRowTemplate\
\x22,\x22ctSetValue\x22,\x22\
ctValue\x22,\x22curato\
rAddons\x22,\x22curato\
rCamera\x22,\x22curato\
rCameraArea\x22,\x22cu\
ratorCameraAreaC\
eiling\x22,\x22curator\
Coef\x22,\x22curatorEd\
itableObjects\x22,\x22\
curatorEditingAr\
ea\x22,\x22curatorEdit\
ingAreaType\x22,\x22cu\
ratorMouseOver\x22,\
\x22curatorPoints\x22,\
\x22curatorRegister\
edObjects\x22,\x22cura\
torSelected\x22,\x22cu\
ratorWaypointCos\
t\x22,\x22current3DENO\
peration\x22,\x22curre\
ntChannel\x22,\x22curr\
entCommand\x22,\x22cur\
rentMagazine\x22,\x22c\
urrentMagazineDe\
tail\x22,\x22currentMa\
gazineDetailTurr\
et\x22,\x22currentMaga\
zineTurret\x22,\x22cur\
rentMuzzle\x22,\x22cur\
rentNamespace\x22,\x22\
currentPilot\x22,\x22c\
urrentTask\x22,\x22cur\
rentTasks\x22,\x22curr\
entThrowable\x22,\x22c\
urrentVisionMode\
\x22,\x22currentWaypoi\
nt\x22,\x22currentWeap\
on\x22,\x22currentWeap\
onMode\x22,\x22current\
WeaponTurret\x22,\x22c\
urrentZeroing\x22,\x22\
cursorObject\x22,\x22c\
ursorTarget\x22,\x22cu\
stomChat\x22,\x22custo\
mRadio\x22,\x22customW\
aypointPosition\x22\
,\x22cutFadeOut\x22,\x22c\
utObj\x22,\x22cutRsc\x22,\
\x22cutText\x22,\x22damag\
e\x22,\x22date\x22,\x22dateT\
oNumber\x22,\x22dayTim\
e\x22,\x22deActivateKe\
y\x22,\x22debriefingTe\
xt\x22,\x22debugFSM\x22,\x22\
debugLog\x22,\x22decay\
GraphValues\x22,\x22de\
g\x22,\x22delete3DENEn\
tities\x22,\x22deleteA\
t\x22,\x22deleteCenter\
\x22,\x22deleteCollect\
ion\x22,\x22deleteEdit\
orObject\x22,\x22delet\
eGroup\x22,\x22deleteG\
roupWhenEmpty\x22,\x22\
deleteIdentity\x22,\
\x22deleteLocation\x22\
,\x22deleteMarker\x22,\
\x22deleteMarkerLoc\
al\x22,\x22deleteRange\
\x22,\x22deleteResourc\
es\x22,\x22deleteSite\x22\
,\x22deleteStatus\x22,\
\x22deleteTeam\x22,\x22de\
leteVehicle\x22,\x22de\
leteVehicleCrew\x22\
,\x22deleteWaypoint\
\x22,\x22detach\x22,\x22dete\
ctedMines\x22,\x22diag\
_activeMissionFS\
Ms\x22,\x22diag_active\
Scripts\x22,\x22diag_a\
ctiveSQFScripts\x22\
,\x22diag_activeSQS\
Scripts\x22,\x22diag_a\
llMissionEventHa\
ndlers\x22,\x22diag_ca\
ptureFrame\x22,\x22dia\
g_captureFrameTo\
File\x22,\x22diag_capt\
ureSlowFrame\x22,\x22d\
iag_codePerforma\
nce\x22,\x22diag_delta\
Time\x22,\x22diag_draw\
mode\x22,\x22diag_dump\
CalltraceToLog\x22,\
\x22diag_dumpScript\
Assembly\x22,\x22diag_\
dumpTerrainSynth\
\x22,\x22diag_dynamicS\
imulationEnd\x22,\x22d\
iag_enable\x22,\x22dia\
g_enabled\x22,\x22diag\
_exportConfig\x22,\x22\
diag_exportTerra\
inSVG\x22,\x22diag_fps\
\x22,\x22diag_fpsmin\x22,\
\x22diag_frameno\x22,\x22\
diag_getTerrainS\
egmentOffset\x22,\x22d\
iag_lightNewLoad\
\x22,\x22diag_list\x22,\x22d\
iag_localized\x22,\x22\
diag_log\x22,\x22diag_\
logSlowFrame\x22,\x22d\
iag_mergeConfigF\
ile\x22,\x22diag_recor\
dTurretLimits\x22,\x22\
diag_resetFSM\x22,\x22\
diag_resetshapes\
\x22,\x22diag_scope\x22,\x22\
diag_setLightNew\
\x22,\x22diag_stacktra\
ce\x22,\x22diag_tickTi\
me\x22,\x22diag_toggle\
\x22,\x22dialog\x22,\x22diar\
ySubjectExists\x22,\
\x22didJIP\x22,\x22didJIP\
Owner\x22,\x22difficul\
ty\x22,\x22difficultyE\
nabled\x22,\x22difficu\
ltyEnabledRTD\x22,\x22\
difficultyOption\
\x22,\x22direction\x22,\x22d\
irectionStabiliz\
ationEnabled\x22,\x22d\
irectSay\x22,\x22disab\
leAI\x22,\x22disableBr\
akes\x22,\x22disableCo\
llisionWith\x22,\x22di\
sableConversatio\
n\x22,\x22disableDebri\
efingStats\x22,\x22dis\
ableMapIndicator\
s\x22,\x22disableNVGEq\
uipment\x22,\x22disabl\
eRemoteSensors\x22,\
\x22disableSerializ\
ation\x22,\x22disableT\
IEquipment\x22,\x22dis\
ableUAVConnectab\
ility\x22,\x22disableU\
serInput\x22,\x22displ\
ayAddEventHandle\
r\x22,\x22displayChild\
\x22,\x22displayCtrl\x22,\
\x22displayParent\x22,\
\x22displayRemoveAl\
lEventHandlers\x22,\
\x22displayRemoveEv\
entHandler\x22,\x22dis\
playSetEventHand\
ler\x22,\x22displayUni\
queName\x22,\x22displa\
yUpdate\x22,\x22dissol\
veTeam\x22,\x22distanc\
e\x22,\x22distance2D\x22,\
\x22distanceSqr\x22,\x22d\
istributionRegio\
n\x22,\x22do3DENAction\
\x22,\x22doArtilleryFi\
re\x22,\x22doFire\x22,\x22do\
Follow\x22,\x22doFSM\x22,\
\x22doGetOut\x22,\x22doMo\
ve\x22,\x22doorPhase\x22,\
\x22doStop\x22,\x22doSupp\
ressiveFire\x22,\x22do\
Target\x22,\x22doWatch\
\x22,\x22drawArrow\x22,\x22d\
rawEllipse\x22,\x22dra\
wIcon\x22,\x22drawIcon\
3D\x22,\x22drawLaser\x22,\
\x22drawLine\x22,\x22draw\
Line3D\x22,\x22drawLin\
k\x22,\x22drawLocation\
\x22,\x22drawPolygon\x22,\
\x22drawRectangle\x22,\
\x22drawTriangle\x22,\x22\
driver\x22,\x22drop\x22,\x22\
dynamicSimulatio\
nDistance\x22,\x22dyna\
micSimulationDis\
tanceCoef\x22,\x22dyna\
micSimulationEna\
bled\x22,\x22dynamicSi\
mulationSystemEn\
abled\x22,\x22echo\x22,\x22e\
dit3DENMissionAt\
tributes\x22,\x22editO\
bject\x22,\x22editorSe\
tEventHandler\x22,\x22\
effectiveCommand\
er\x22,\x22elevatePeri\
scope\x22,\x22emptyPos\
itions\x22,\x22enableA\
I\x22,\x22enableAIFeat\
ure\x22,\x22enableAimP\
recision\x22,\x22enabl\
eAttack\x22,\x22enable\
AudioFeature\x22,\x22e\
nableAutoStartUp\
RTD\x22,\x22enableAuto\
TrimRTD\x22,\x22enable\
CamShake\x22,\x22enabl\
eCaustics\x22,\x22enab\
leChannel\x22,\x22enab\
leCollisionWith\x22\
,\x22enableCopilot\x22\
,\x22enableDebriefi\
ngStats\x22,\x22enable\
DiagLegend\x22,\x22ena\
bleDirectionStab\
ilization\x22,\x22enab\
leDynamicSimulat\
ion\x22,\x22enableDyna\
micSimulationSys\
tem\x22,\x22enableEndD\
ialog\x22,\x22enableEn\
gineArtillery\x22,\x22\
enableEnvironmen\
t\x22,\x22enableFatigu\
e\x22,\x22enableGunLig\
hts\x22,\x22enableInfo\
PanelComponent\x22,\
\x22enableIRLasers\x22\
,\x22enableMimics\x22,\
\x22enablePersonTur\
ret\x22,\x22enableRadi\
o\x22,\x22enableReload\
\x22,\x22enableRopeAtt\
ach\x22,\x22enableSatN\
ormalOnDetail\x22,\x22\
enableSaving\x22,\x22e\
nableSentences\x22,\
\x22enableSimulatio\
n\x22,\x22enableSimula\
tionGlobal\x22,\x22ena\
bleStamina\x22,\x22ena\
bleStressDamage\x22\
,\x22enableTeamSwit\
ch\x22,\x22enableTraff\
ic\x22,\x22enableUAVCo\
nnectability\x22,\x22e\
nableUAVWaypoint\
s\x22,\x22enableVehicl\
eCargo\x22,\x22enableV\
ehicleSensor\x22,\x22e\
nableWeaponDisas\
sembly\x22,\x22endLoad\
ingScreen\x22,\x22endM\
ission\x22,\x22engineO\
n\x22,\x22enginesIsOnR\
TD\x22,\x22enginesPowe\
rRTD\x22,\x22enginesRp\
mRTD\x22,\x22enginesTo\
rqueRTD\x22,\x22entiti\
es\x22,\x22environment\
Enabled\x22,\x22enviro\
nmentVolume\x22,\x22eq\
uipmentDisabled\x22\
,\x22estimatedEndSe\
rverTime\x22,\x22estim\
atedTimeLeft\x22,\x22e\
valObjectArgumen\
t\x22,\x22everyBackpac\
k\x22,\x22everyContain\
er\x22,\x22exec\x22,\x22exec\
EditorScript\x22,\x22e\
xecFSM\x22,\x22execVM\x22\
,\x22exp\x22,\x22expected\
Destination\x22,\x22ex\
portJIPMessages\x22\
,\x22eyeDirection\x22,\
\x22eyePos\x22,\x22face\x22,\
\x22faction\x22,\x22fadeE\
nvironment\x22,\x22fad\
eMusic\x22,\x22fadeRad\
io\x22,\x22fadeSound\x22,\
\x22fadeSpeech\x22,\x22fa\
ilMission\x22,\x22file\
Exists\x22,\x22fillWea\
ponsFromPool\x22,\x22f\
ind\x22,\x22findAny\x22,\x22\
findCover\x22,\x22find\
Display\x22,\x22findEd\
itorObject\x22,\x22fin\
dEmptyPosition\x22,\
\x22findEmptyPositi\
onReady\x22,\x22findIf\
\x22,\x22findNearestEn\
emy\x22,\x22finishMiss\
ionInit\x22,\x22finite\
\x22,\x22fire\x22,\x22fireAt\
Target\x22,\x22firstBa\
ckpack\x22,\x22flag\x22,\x22\
flagAnimationPha\
se\x22,\x22flagOwner\x22,\
\x22flagSide\x22,\x22flag\
Texture\x22,\x22flatte\
n\x22,\x22fleeing\x22,\x22fl\
oor\x22,\x22flyInHeigh\
t\x22,\x22flyInHeightA\
SL\x22,\x22focusedCtrl\
\x22,\x22fog\x22,\x22fogFore\
cast\x22,\x22fogParams\
\x22,\x22forceAddUnifo\
rm\x22,\x22forceAtPosi\
tionRTD\x22,\x22forceC\
adetDifficulty\x22,\
\x22forcedMap\x22,\x22for\
ceEnd\x22,\x22forceFla\
gTexture\x22,\x22force\
FollowRoad\x22,\x22for\
ceGeneratorRTD\x22,\
\x22forceMap\x22,\x22forc\
eRespawn\x22,\x22force\
Speed\x22,\x22forceUni\
code\x22,\x22forceWalk\
\x22,\x22forceWeaponFi\
re\x22,\x22forceWeathe\
rChange\x22,\x22forEac\
hMember\x22,\x22forEac\
hMemberAgent\x22,\x22f\
orEachMemberTeam\
\x22,\x22forgetTarget\x22\
,\x22format\x22,\x22forma\
tion\x22,\x22formation\
Direction\x22,\x22form\
ationLeader\x22,\x22fo\
rmationMembers\x22,\
\x22formationPositi\
on\x22,\x22formationTa\
sk\x22,\x22formatText\x22\
,\x22formLeader\x22,\x22f\
reeExtension\x22,\x22f\
reeLook\x22,\x22fromEd\
itor\x22,\x22fuel\x22,\x22fu\
llCrew\x22,\x22gearIDC\
AmmoCount\x22,\x22gear\
SlotAmmoCount\x22,\x22\
gearSlotData\x22,\x22g\
estureState\x22,\x22ge\
t\x22,\x22get3DENActio\
nState\x22,\x22get3DEN\
Attribute\x22,\x22get3\
DENCamera\x22,\x22get3\
DENConnections\x22,\
\x22get3DENEntity\x22,\
\x22get3DENEntityID\
\x22,\x22get3DENGrid\x22,\
\x22get3DENIconsVis\
ible\x22,\x22get3DENLa\
yerEntities\x22,\x22ge\
t3DENLinesVisibl\
e\x22,\x22get3DENMissi\
onAttribute\x22,\x22ge\
t3DENMouseOver\x22,\
\x22get3DENSelected\
\x22,\x22getAimingCoef\
\x22,\x22getAllEnv3DSo\
undControllers\x22,\
\x22getAllEnvSoundC\
ontrollers\x22,\x22get\
AllHitPointsDama\
ge\x22,\x22getAllOwned\
Mines\x22,\x22getAllPy\
lonsInfo\x22,\x22getAl\
lSoundController\
s\x22,\x22getAllUnitTr\
aits\x22,\x22getAmmoCa\
rgo\x22,\x22getAnimAim\
Precision\x22,\x22getA\
nimSpeedCoef\x22,\x22g\
etArray\x22,\x22getArt\
illeryAmmo\x22,\x22get\
ArtilleryCompute\
rSettings\x22,\x22getA\
rtilleryETA\x22,\x22ge\
tAssetDLCInfo\x22,\x22\
getAssignedCurat\
orLogic\x22,\x22getAss\
ignedCuratorUnit\
\x22,\x22getAttackTarg\
et\x22,\x22getAudioOpt\
ionVolumes\x22,\x22get\
BackpackCargo\x22,\x22\
getBleedingRemai\
ning\x22,\x22getBurnin\
gValue\x22,\x22getCalc\
ulatePlayerVisib\
ilityByFriendly\x22\
,\x22getCameraViewD\
irection\x22,\x22getCa\
rgoIndex\x22,\x22getCe\
nterOfMass\x22,\x22get\
ClientState\x22,\x22ge\
tClientStateNumb\
er\x22,\x22getCompatib\
lePylonMagazines\
\x22,\x22getConnectedU\
AV\x22,\x22getConnecte\
dUAVUnit\x22,\x22getCo\
ntainerMaxLoad\x22,\
\x22getCorpse\x22,\x22get\
CruiseControl\x22,\x22\
getCursorObjectP\
arams\x22,\x22getCusto\
mAimCoef\x22,\x22getCu\
stomSoundControl\
ler\x22,\x22getCustomS\
oundControllerCo\
unt\x22,\x22getDammage\
\x22,\x22getDebriefing\
Text\x22,\x22getDescri\
ption\x22,\x22getDir\x22,\
\x22getDirVisual\x22,\x22\
getDiverState\x22,\x22\
getDLCAssetsUsag\
e\x22,\x22getDLCAssets\
UsageByName\x22,\x22ge\
tDLCs\x22,\x22getDLCUs\
ageTime\x22,\x22getEdi\
torCamera\x22,\x22getE\
ditorMode\x22,\x22getE\
ditorObjectScope\
\x22,\x22getElevationO\
ffset\x22,\x22getEngin\
eTargetRPMRTD\x22,\x22\
getEnv3DSoundCon\
troller\x22,\x22getEnv\
SoundController\x22\
,\x22getEventHandle\
rInfo\x22,\x22getFatig\
ue\x22,\x22getFieldMan\
ualStartPage\x22,\x22g\
etForcedFlagText\
ure\x22,\x22getForcedS\
peed\x22,\x22getFriend\
\x22,\x22getFSMVariabl\
e\x22,\x22getFuelCargo\
\x22,\x22getGraphValue\
s\x22,\x22getGroupIcon\
\x22,\x22getGroupIconP\
arams\x22,\x22getGroup\
Icons\x22,\x22getHideF\
rom\x22,\x22getHit\x22,\x22g\
etHitIndex\x22,\x22get\
HitPointDamage\x22,\
\x22getItemCargo\x22,\x22\
getLighting\x22,\x22ge\
tLightingAt\x22,\x22ge\
tLoadedModsInfo\x22\
,\x22getMagazineCar\
go\x22,\x22getMarkerCo\
lor\x22,\x22getMarkerP\
os\x22,\x22getMarkerSi\
ze\x22,\x22getMarkerTy\
pe\x22,\x22getMass\x22,\x22g\
etMissionConfig\x22\
,\x22getMissionConf\
igValue\x22,\x22getMis\
sionDLCs\x22,\x22getMi\
ssionLayerEntiti\
es\x22,\x22getMissionL\
ayers\x22,\x22getMissi\
onPath\x22,\x22getMode\
lInfo\x22,\x22getMouse\
Position\x22,\x22getMu\
sicPlayedTime\x22,\x22\
getNumber\x22,\x22getO\
bjectArgument\x22,\x22\
getObjectChildre\
n\x22,\x22getObjectDLC\
\x22,\x22getObjectFOV\x22\
,\x22getObjectID\x22,\x22\
getObjectMateria\
ls\x22,\x22getObjectPr\
oxy\x22,\x22getObjectS\
cale\x22,\x22getObject\
Textures\x22,\x22getOb\
jectType\x22,\x22getOb\
jectViewDistance\
\x22,\x22getOpticsMode\
\x22,\x22getOrDefault\x22\
,\x22getOrDefaultCa\
ll\x22,\x22getOxygenRe\
maining\x22,\x22getPer\
sonUsedDLCs\x22,\x22ge\
tPilotCameraDire\
ction\x22,\x22getPilot\
CameraPosition\x22,\
\x22getPilotCameraR\
otation\x22,\x22getPil\
otCameraTarget\x22,\
\x22getPiPViewDista\
nce\x22,\x22getPlateNu\
mber\x22,\x22getPlayer\
Channel\x22,\x22getPla\
yerID\x22,\x22getPlaye\
rScores\x22,\x22getPla\
yerUID\x22,\x22getPlay\
erVoNVolume\x22,\x22ge\
tPos\x22,\x22getPosASL\
\x22,\x22getPosASLVisu\
al\x22,\x22getPosASLW\x22\
,\x22getPosATL\x22,\x22ge\
tPosATLVisual\x22,\x22\
getPosVisual\x22,\x22g\
etPosWorld\x22,\x22get\
PosWorldVisual\x22,\
\x22getPylonMagazin\
es\x22,\x22getRelDir\x22,\
\x22getRelPos\x22,\x22get\
RemoteSensorsDis\
abled\x22,\x22getRepai\
rCargo\x22,\x22getReso\
lution\x22,\x22getRoad\
Info\x22,\x22getRotorB\
rakeRTD\x22,\x22getSen\
sorTargets\x22,\x22get\
SensorThreats\x22,\x22\
getShadowDistanc\
e\x22,\x22getShotParen\
ts\x22,\x22getSlingLoa\
d\x22,\x22getSoundCont\
roller\x22,\x22getSoun\
dControllerResul\
t\x22,\x22getSpeed\x22,\x22g\
etStamina\x22,\x22getS\
tatValue\x22,\x22getSt\
eamFriendsServer\
s\x22,\x22getSubtitleO\
ptions\x22,\x22getSupp\
ression\x22,\x22getTer\
rainGrid\x22,\x22getTe\
rrainHeight\x22,\x22ge\
tTerrainHeightAS\
L\x22,\x22getTerrainIn\
fo\x22,\x22getText\x22,\x22g\
etTextRaw\x22,\x22getT\
extureInfo\x22,\x22get\
TextWidth\x22,\x22getT\
iParameters\x22,\x22ge\
tTotalDLCUsageTi\
me\x22,\x22getTrimOffs\
etRTD\x22,\x22getTurre\
tLimits\x22,\x22getTur\
retOpticsMode\x22,\x22\
getUnitFreefallI\
nfo\x22,\x22getUnitLoa\
dout\x22,\x22getUnitTr\
ait\x22,\x22getUnloadI\
nCombat\x22,\x22getUse\
rInfo\x22,\x22getUserM\
FDText\x22,\x22getUser\
MFDValue\x22,\x22getVa\
riable\x22,\x22getVehi\
cleCargo\x22,\x22getVe\
hicleTiPars\x22,\x22ge\
tWeaponCargo\x22,\x22g\
etWeaponSway\x22,\x22g\
etWingsOrientati\
onRTD\x22,\x22getWings\
PositionRTD\x22,\x22ge\
tWPPos\x22,\x22glanceA\
t\x22,\x22globalChat\x22,\
\x22globalRadio\x22,\x22g\
oggles\x22,\x22goto\x22,\x22\
group\x22,\x22groupCha\
t\x22,\x22groupFromNet\
Id\x22,\x22groupIconSe\
lectable\x22,\x22group\
IconsVisible\x22,\x22g\
roupID\x22,\x22groupOw\
ner\x22,\x22groupRadio\
\x22,\x22groups\x22,\x22grou\
pSelectedUnits\x22,\
\x22groupSelectUnit\
\x22,\x22gunner\x22,\x22gust\
s\x22,\x22halt\x22,\x22handg\
unItems\x22,\x22handgu\
nMagazine\x22,\x22hand\
gunWeapon\x22,\x22hand\
sHit\x22,\x22hashValue\
\x22,\x22hasInterface\x22\
,\x22hasPilotCamera\
\x22,\x22hasWeapon\x22,\x22h\
cAllGroups\x22,\x22hcG\
roupParams\x22,\x22hcL\
eader\x22,\x22hcRemove\
AllGroups\x22,\x22hcRe\
moveGroup\x22,\x22hcSe\
lected\x22,\x22hcSelec\
tGroup\x22,\x22hcSetGr\
oup\x22,\x22hcShowBar\x22\
,\x22hcShownBar\x22,\x22h\
eadgear\x22,\x22hideBo\
dy\x22,\x22hideObject\x22\
,\x22hideObjectGlob\
al\x22,\x22hideSelecti\
on\x22,\x22hint\x22,\x22hint\
C\x22,\x22hintCadet\x22,\x22\
hintSilent\x22,\x22hmd\
\x22,\x22hostMission\x22,\
\x22htmlLoad\x22,\x22HUDM\
ovementLevels\x22,\x22\
humidity\x22,\x22image\
\x22,\x22importAllGrou\
ps\x22,\x22importance\x22\
,\x22in\x22,\x22inArea\x22,\x22\
inAreaArray\x22,\x22in\
capacitatedState\
\x22,\x22inflame\x22,\x22inf\
lamed\x22,\x22infoPane\
l\x22,\x22infoPanelCom\
ponentEnabled\x22,\x22\
infoPanelCompone\
nts\x22,\x22infoPanels\
\x22,\x22inGameUISetEv\
entHandler\x22,\x22inh\
eritsFrom\x22,\x22init\
AmbientLife\x22,\x22in\
Polygon\x22,\x22inputA\
ction\x22,\x22inputCon\
troller\x22,\x22inputM\
ouse\x22,\x22inRangeOf\
Artillery\x22,\x22inse\
rt\x22,\x22insertEdito\
rObject\x22,\x22inters\
ect\x22,\x22is3DEN\x22,\x22i\
s3DENMultiplayer\
\x22,\x22is3DENPreview\
\x22,\x22isAbleToBreat\
he\x22,\x22isActionMen\
uVisible\x22,\x22isAge\
nt\x22,\x22isAimPrecis\
ionEnabled\x22,\x22isA\
llowedCrewInImmo\
bile\x22,\x22isArray\x22,\
\x22isAutoHoverOn\x22,\
\x22isAutonomous\x22,\x22\
isAutoStartUpEna\
bledRTD\x22,\x22isAuto\
test\x22,\x22isAutoTri\
mOnRTD\x22,\x22isAwake\
\x22,\x22isBleeding\x22,\x22\
isBurning\x22,\x22isCl\
ass\x22,\x22isCollisio\
nLightOn\x22,\x22isCop\
ilotEnabled\x22,\x22is\
DamageAllowed\x22,\x22\
isDedicated\x22,\x22is\
DLCAvailable\x22,\x22i\
sEngineOn\x22,\x22isEq\
ualRef\x22,\x22isEqual\
To\x22,\x22isEqualType\
\x22,\x22isEqualTypeAl\
l\x22,\x22isEqualTypeA\
ny\x22,\x22isEqualType\
Array\x22,\x22isEqualT\
ypeParams\x22,\x22isFi\
lePatchingEnable\
d\x22,\x22isFinal\x22,\x22is\
FlashlightOn\x22,\x22i\
sFlatEmpty\x22,\x22isF\
orcedWalk\x22,\x22isFo\
rmationLeader\x22,\x22\
isGameFocused\x22,\x22\
isGamePaused\x22,\x22i\
sGroupDeletedWhe\
nEmpty\x22,\x22isHidde\
n\x22,\x22isInRemainsC\
ollector\x22,\x22isIns\
tructorFigureEna\
bled\x22,\x22isIRLaser\
On\x22,\x22isKeyActive\
\x22,\x22isKindOf\x22,\x22is\
LaserOn\x22,\x22isLigh\
tOn\x22,\x22isLocalize\
d\x22,\x22isManualFire\
\x22,\x22isMarkedForCo\
llection\x22,\x22isMis\
sionProfileNames\
paceLoaded\x22,\x22isM\
ultiplayer\x22,\x22isM\
ultiplayerSolo\x22,\
\x22isNil\x22,\x22isNotEq\
ualRef\x22,\x22isNotEq\
ualTo\x22,\x22isNull\x22,\
\x22isNumber\x22,\x22isOb\
jectHidden\x22,\x22isO\
bjectRTD\x22,\x22isOnR\
oad\x22,\x22isPiPEnabl\
ed\x22,\x22isPlayer\x22,\x22\
isRealTime\x22,\x22isR\
emoteExecuted\x22,\x22\
isRemoteExecuted\
JIP\x22,\x22isSaving\x22,\
\x22isSensorTargetC\
onfirmed\x22,\x22isSer\
ver\x22,\x22isShowing3\
DIcons\x22,\x22isSimpl\
eObject\x22,\x22isSpri\
ntAllowed\x22,\x22isSt\
aminaEnabled\x22,\x22i\
sSteamMission\x22,\x22\
isSteamOverlayEn\
abled\x22,\x22isStream\
FriendlyUIEnable\
d\x22,\x22isStressDama\
geEnabled\x22,\x22isTe\
xt\x22,\x22isTouchingG\
round\x22,\x22isTurned\
Out\x22,\x22isTutHints\
Enabled\x22,\x22isUAVC\
onnectable\x22,\x22isU\
AVConnected\x22,\x22is\
UIContext\x22,\x22isUn\
iformAllowed\x22,\x22i\
sVehicleCargo\x22,\x22\
isVehicleRadarOn\
\x22,\x22isVehicleSens\
orEnabled\x22,\x22isWa\
lking\x22,\x22isWeapon\
Deployed\x22,\x22isWea\
ponRested\x22,\x22item\
Cargo\x22,\x22items\x22,\x22\
itemsWithMagazin\
es\x22,\x22join\x22,\x22join\
As\x22,\x22joinAsSilen\
t\x22,\x22joinSilent\x22,\
\x22joinString\x22,\x22kb\
AddDatabase\x22,\x22kb\
AddDatabaseTarge\
ts\x22,\x22kbAddTopic\x22\
,\x22kbHasTopic\x22,\x22k\
bReact\x22,\x22kbRemov\
eTopic\x22,\x22kbTell\x22\
,\x22kbWasSaid\x22,\x22ke\
yImage\x22,\x22keyName\
\x22,\x22keys\x22,\x22knowsA\
bout\x22,\x22land\x22,\x22la\
ndAt\x22,\x22landResul\
t\x22,\x22language\x22,\x22l\
aserTarget\x22,\x22lbA\
dd\x22,\x22lbClear\x22,\x22l\
bColor\x22,\x22lbColor\
Right\x22,\x22lbCurSel\
\x22,\x22lbData\x22,\x22lbDe\
lete\x22,\x22lbIsSelec\
ted\x22,\x22lbPicture\x22\
,\x22lbPictureRight\
\x22,\x22lbSelection\x22,\
\x22lbSetColor\x22,\x22lb\
SetColorRight\x22,\x22\
lbSetCurSel\x22,\x22lb\
SetData\x22,\x22lbSetP\
icture\x22,\x22lbSetPi\
ctureColor\x22,\x22lbS\
etPictureColorDi\
sabled\x22,\x22lbSetPi\
ctureColorSelect\
ed\x22,\x22lbSetPictur\
eRight\x22,\x22lbSetPi\
ctureRightColor\x22\
,\x22lbSetPictureRi\
ghtColorDisabled\
\x22,\x22lbSetPictureR\
ightColorSelecte\
d\x22,\x22lbSetSelectC\
olor\x22,\x22lbSetSele\
ctColorRight\x22,\x22l\
bSetSelected\x22,\x22l\
bSetText\x22,\x22lbSet\
TextRight\x22,\x22lbSe\
tTooltip\x22,\x22lbSet\
Value\x22,\x22lbSize\x22,\
\x22lbSort\x22,\x22lbSort\
By\x22,\x22lbSortByVal\
ue\x22,\x22lbText\x22,\x22lb\
TextRight\x22,\x22lbTo\
oltip\x22,\x22lbValue\x22\
,\x22leader\x22,\x22leade\
rboardDeInit\x22,\x22l\
eaderboardGetRow\
s\x22,\x22leaderboardI\
nit\x22,\x22leaderboar\
dRequestRowsFrie\
nds\x22,\x22leaderboar\
dRequestRowsGlob\
al\x22,\x22leaderboard\
RequestRowsGloba\
lAroundUser\x22,\x22le\
aderboardsReques\
tUploadScore\x22,\x22l\
eaderboardsReque\
stUploadScoreKee\
pBest\x22,\x22leaderbo\
ardState\x22,\x22leave\
Vehicle\x22,\x22librar\
yCredits\x22,\x22libra\
ryDisclaimers\x22,\x22\
lifeState\x22,\x22ligh\
tAttachObject\x22,\x22\
lightDetachObjec\
t\x22,\x22lightIsOn\x22,\x22\
lightnings\x22,\x22lim\
itSpeed\x22,\x22linear\
Conversion\x22,\x22lin\
eIntersects\x22,\x22li\
neIntersectsObjs\
\x22,\x22lineIntersect\
sSurfaces\x22,\x22line\
IntersectsWith\x22,\
\x22linkItem\x22,\x22list\
\x22,\x22listObjects\x22,\
\x22listRemoteTarge\
ts\x22,\x22listVehicle\
Sensors\x22,\x22ln\x22,\x22l\
nbAddArray\x22,\x22lnb\
AddColumn\x22,\x22lnbA\
ddRow\x22,\x22lnbClear\
\x22,\x22lnbColor\x22,\x22ln\
bColorRight\x22,\x22ln\
bCurSelRow\x22,\x22lnb\
Data\x22,\x22lnbDelete\
Column\x22,\x22lnbDele\
teRow\x22,\x22lnbGetCo\
lumnsPosition\x22,\x22\
lnbPicture\x22,\x22lnb\
PictureRight\x22,\x22l\
nbSetColor\x22,\x22lnb\
SetColorRight\x22,\x22\
lnbSetColumnsPos\
\x22,\x22lnbSetCurSelR\
ow\x22,\x22lnbSetData\x22\
,\x22lnbSetPicture\x22\
,\x22lnbSetPictureC\
olor\x22,\x22lnbSetPic\
tureColorRight\x22,\
\x22lnbSetPictureCo\
lorSelected\x22,\x22ln\
bSetPictureColor\
SelectedRight\x22,\x22\
lnbSetPictureRig\
ht\x22,\x22lnbSetText\x22\
,\x22lnbSetTextRigh\
t\x22,\x22lnbSetToolti\
p\x22,\x22lnbSetValue\x22\
,\x22lnbSize\x22,\x22lnbS\
ort\x22,\x22lnbSortBy\x22\
,\x22lnbSortByValue\
\x22,\x22lnbText\x22,\x22lnb\
TextRight\x22,\x22lnbV\
alue\x22,\x22load\x22,\x22lo\
adAbs\x22,\x22loadBack\
pack\x22,\x22loadConfi\
g\x22,\x22loadFile\x22,\x22l\
oadGame\x22,\x22loadId\
entity\x22,\x22loadMag\
azine\x22,\x22loadOver\
lay\x22,\x22loadStatus\
\x22,\x22loadUniform\x22,\
\x22loadVest\x22,\x22loca\
lize\x22,\x22localName\
space\x22,\x22location\
Position\x22,\x22lock\x22\
,\x22lockCameraTo\x22,\
\x22lockCargo\x22,\x22loc\
kDriver\x22,\x22locked\
\x22,\x22lockedCameraT\
o\x22,\x22lockedCargo\x22\
,\x22lockedDriver\x22,\
\x22lockedInventory\
\x22,\x22lockedTurret\x22\
,\x22lockIdentity\x22,\
\x22lockInventory\x22,\
\x22lockTurret\x22,\x22lo\
ckWp\x22,\x22log\x22,\x22log\
Entities\x22,\x22logNe\
twork\x22,\x22logNetwo\
rkTerminate\x22,\x22lo\
okAt\x22,\x22lookAtPos\
\x22,\x22magazineCargo\
\x22,\x22magazines\x22,\x22m\
agazinesAllTurre\
ts\x22,\x22magazinesAm\
mo\x22,\x22magazinesAm\
moCargo\x22,\x22magazi\
nesAmmoFull\x22,\x22ma\
gazinesDetail\x22,\x22\
magazinesDetailB\
ackpack\x22,\x22magazi\
nesDetailUniform\
\x22,\x22magazinesDeta\
ilVest\x22,\x22magazin\
esTurret\x22,\x22magaz\
ineTurretAmmo\x22,\x22\
mapAnimAdd\x22,\x22map\
AnimClear\x22,\x22mapA\
nimCommit\x22,\x22mapA\
nimDone\x22,\x22mapCen\
terOnCamera\x22,\x22ma\
pGridPosition\x22,\x22\
markAsFinishedOn\
Steam\x22,\x22markerAl\
pha\x22,\x22markerBrus\
h\x22,\x22markerChanne\
l\x22,\x22markerColor\x22\
,\x22markerDir\x22,\x22ma\
rkerPolyline\x22,\x22m\
arkerPos\x22,\x22marke\
rShadow\x22,\x22marker\
Shape\x22,\x22markerSi\
ze\x22,\x22markerText\x22\
,\x22markerType\x22,\x22m\
atrixMultiply\x22,\x22\
matrixTranspose\x22\
,\x22max\x22,\x22maxLoad\x22\
,\x22members\x22,\x22menu\
Action\x22,\x22menuAdd\
\x22,\x22menuChecked\x22,\
\x22menuClear\x22,\x22men\
uCollapse\x22,\x22menu\
Data\x22,\x22menuDelet\
e\x22,\x22menuEnable\x22,\
\x22menuEnabled\x22,\x22m\
enuExpand\x22,\x22menu\
Hover\x22,\x22menuPict\
ure\x22,\x22menuSetAct\
ion\x22,\x22menuSetChe\
ck\x22,\x22menuSetData\
\x22,\x22menuSetPictur\
e\x22,\x22menuSetShort\
cut\x22,\x22menuSetTex\
t\x22,\x22menuSetURL\x22,\
\x22menuSetValue\x22,\x22\
menuShortcut\x22,\x22m\
enuShortcutText\x22\
,\x22menuSize\x22,\x22men\
uSort\x22,\x22menuText\
\x22,\x22menuURL\x22,\x22men\
uValue\x22,\x22merge\x22,\
\x22min\x22,\x22mineActiv\
e\x22,\x22mineDetected\
By\x22,\x22missileTarg\
et\x22,\x22missileTarg\
etPos\x22,\x22missionC\
onfigFile\x22,\x22miss\
ionDifficulty\x22,\x22\
missionEnd\x22,\x22mis\
sionName\x22,\x22missi\
onNameSource\x22,\x22m\
issionNamespace\x22\
,\x22missionProfile\
Namespace\x22,\x22miss\
ionStart\x22,\x22missi\
onVersion\x22,\x22mod\x22\
,\x22modelToWorld\x22,\
\x22modelToWorldVis\
ual\x22,\x22modelToWor\
ldVisualWorld\x22,\x22\
modelToWorldWorl\
d\x22,\x22modParams\x22,\x22\
moonIntensity\x22,\x22\
moonPhase\x22,\x22mora\
le\x22,\x22move\x22,\x22move\
3DENCamera\x22,\x22mov\
eInAny\x22,\x22moveInC\
argo\x22,\x22moveInCom\
mander\x22,\x22moveInD\
river\x22,\x22moveInGu\
nner\x22,\x22moveInTur\
ret\x22,\x22moveObject\
ToEnd\x22,\x22moveOut\x22\
,\x22moveTime\x22,\x22mov\
eTo\x22,\x22moveToComp\
leted\x22,\x22moveToFa\
iled\x22,\x22musicVolu\
me\x22,\x22name\x22,\x22name\
dProperties\x22,\x22na\
meSound\x22,\x22nearEn\
tities\x22,\x22nearest\
Building\x22,\x22neare\
stLocation\x22,\x22nea\
restLocations\x22,\x22\
nearestLocationW\
ithDubbing\x22,\x22nea\
restMines\x22,\x22near\
estObject\x22,\x22near\
estObjects\x22,\x22nea\
restTerrainObjec\
ts\x22,\x22nearObjects\
\x22,\x22nearObjectsRe\
ady\x22,\x22nearRoads\x22\
,\x22nearSupplies\x22,\
\x22nearTargets\x22,\x22n\
eedReload\x22,\x22need\
Service\x22,\x22netId\x22\
,\x22netObjNull\x22,\x22n\
ewOverlay\x22,\x22next\
MenuItemIndex\x22,\x22\
nextWeatherChang\
e\x22,\x22nMenuItems\x22,\
\x22not\x22,\x22numberOfE\
nginesRTD\x22,\x22numb\
erToDate\x22,\x22objec\
tCurators\x22,\x22obje\
ctFromNetId\x22,\x22ob\
jectParent\x22,\x22obj\
Status\x22,\x22onBrief\
ingGroup\x22,\x22onBri\
efingNotes\x22,\x22onB\
riefingPlan\x22,\x22on\
BriefingTeamSwit\
ch\x22,\x22onCommandMo\
deChanged\x22,\x22onDo\
ubleClick\x22,\x22onEa\
chFrame\x22,\x22onGrou\
pIconClick\x22,\x22onG\
roupIconOverEnte\
r\x22,\x22onGroupIconO\
verLeave\x22,\x22onHCG\
roupSelectionCha\
nged\x22,\x22onMapSing\
leClick\x22,\x22onPlay\
erConnected\x22,\x22on\
PlayerDisconnect\
ed\x22,\x22onPreloadFi\
nished\x22,\x22onPrelo\
adStarted\x22,\x22onSh\
owNewObject\x22,\x22on\
TeamSwitch\x22,\x22ope\
nCuratorInterfac\
e\x22,\x22openDLCPage\x22\
,\x22openGPS\x22,\x22open\
Map\x22,\x22openSteamA\
pp\x22,\x22openYoutube\
Video\x22,\x22or\x22,\x22ord\
erGetIn\x22,\x22overca\
st\x22,\x22overcastFor\
ecast\x22,\x22owner\x22,\x22\
param\x22,\x22params\x22,\
\x22parseNumber\x22,\x22p\
arseSimpleArray\x22\
,\x22parseText\x22,\x22pa\
rsingNamespace\x22,\
\x22particlesQualit\
y\x22,\x22periscopeEle\
vation\x22,\x22pickWea\
ponPool\x22,\x22pitch\x22\
,\x22pixelGrid\x22,\x22pi\
xelGridBase\x22,\x22pi\
xelGridNoUIScale\
\x22,\x22pixelH\x22,\x22pixe\
lW\x22,\x22playableSlo\
tsNumber\x22,\x22playa\
bleUnits\x22,\x22playA\
ction\x22,\x22playActi\
onNow\x22,\x22player\x22,\
\x22playerRespawnTi\
me\x22,\x22playerSide\x22\
,\x22playersNumber\x22\
,\x22playGesture\x22,\x22\
playMission\x22,\x22pl\
ayMove\x22,\x22playMov\
eNow\x22,\x22playMusic\
\x22,\x22playScriptedM\
ission\x22,\x22playSou\
nd\x22,\x22playSound3D\
\x22,\x22playSoundUI\x22,\
\x22pose\x22,\x22position\
\x22,\x22positionCamer\
aToWorld\x22,\x22posSc\
reenToWorld\x22,\x22po\
sWorldToScreen\x22,\
\x22ppEffectAdjust\x22\
,\x22ppEffectCommit\
\x22,\x22ppEffectCommi\
tted\x22,\x22ppEffectC\
reate\x22,\x22ppEffect\
Destroy\x22,\x22ppEffe\
ctEnable\x22,\x22ppEff\
ectEnabled\x22,\x22ppE\
ffectForceInNVG\x22\
,\x22precision\x22,\x22pr\
eloadCamera\x22,\x22pr\
eloadObject\x22,\x22pr\
eloadSound\x22,\x22pre\
loadTitleObj\x22,\x22p\
reloadTitleRsc\x22,\
\x22preprocessFile\x22\
,\x22preprocessFile\
LineNumbers\x22,\x22pr\
imaryWeapon\x22,\x22pr\
imaryWeaponItems\
\x22,\x22primaryWeapon\
Magazine\x22,\x22prior\
ity\x22,\x22processDia\
ryLink\x22,\x22product\
Version\x22,\x22profil\
eName\x22,\x22profileN\
amespace\x22,\x22profi\
leNameSteam\x22,\x22pr\
ogressLoadingScr\
een\x22,\x22progressPo\
sition\x22,\x22progres\
sSetPosition\x22,\x22p\
ublicVariable\x22,\x22\
publicVariableCl\
ient\x22,\x22publicVar\
iableServer\x22,\x22pu\
shBack\x22,\x22pushBac\
kUnique\x22,\x22putWea\
ponPool\x22,\x22queryI\
temsPool\x22,\x22query\
MagazinePool\x22,\x22q\
ueryWeaponPool\x22,\
\x22rad\x22,\x22radioChan\
nelAdd\x22,\x22radioCh\
annelCreate\x22,\x22ra\
dioChannelInfo\x22,\
\x22radioChannelRem\
ove\x22,\x22radioChann\
elSetCallSign\x22,\x22\
radioChannelSetL\
abel\x22,\x22radioEnab\
led\x22,\x22radioVolum\
e\x22,\x22rain\x22,\x22rainb\
ow\x22,\x22rainParams\x22\
,\x22random\x22,\x22rank\x22\
,\x22rankId\x22,\x22ratin\
g\x22,\x22rectangular\x22\
,\x22regexFind\x22,\x22re\
gexMatch\x22,\x22regex\
Replace\x22,\x22regist\
eredTasks\x22,\x22regi\
sterTask\x22,\x22reloa\
d\x22,\x22reloadEnable\
d\x22,\x22remoteContro\
l\x22,\x22remoteExec\x22,\
\x22remoteExecCall\x22\
,\x22remoteExecuted\
Owner\x22,\x22remove3D\
ENConnection\x22,\x22r\
emove3DENEventHa\
ndler\x22,\x22remove3D\
ENLayer\x22,\x22remove\
Action\x22,\x22removeA\
ll3DENEventHandl\
ers\x22,\x22removeAllA\
ctions\x22,\x22removeA\
llAssignedItems\x22\
,\x22removeAllBinoc\
ularItems\x22,\x22remo\
veAllContainers\x22\
,\x22removeAllCurat\
orAddons\x22,\x22remov\
eAllCuratorCamer\
aAreas\x22,\x22removeA\
llCuratorEditing\
Areas\x22,\x22removeAl\
lEventHandlers\x22,\
\x22removeAllHandgu\
nItems\x22,\x22removeA\
llItems\x22,\x22remove\
AllItemsWithMaga\
zines\x22,\x22removeAl\
lMissionEventHan\
dlers\x22,\x22removeAl\
lMPEventHandlers\
\x22,\x22removeAllMusi\
cEventHandlers\x22,\
\x22removeAllOwnedM\
ines\x22,\x22removeAll\
PrimaryWeaponIte\
ms\x22,\x22removeAllSe\
condaryWeaponIte\
ms\x22,\x22removeAllUs\
erActionEventHan\
dlers\x22,\x22removeAl\
lWeapons\x22,\x22remov\
eBackpack\x22,\x22remo\
veBackpackGlobal\
\x22,\x22removeBinocul\
arItem\x22,\x22removeC\
uratorAddons\x22,\x22r\
emoveCuratorCame\
raArea\x22,\x22removeC\
uratorEditableOb\
jects\x22,\x22removeCu\
ratorEditingArea\
\x22,\x22removeDiaryRe\
cord\x22,\x22removeDia\
rySubject\x22,\x22remo\
veDrawIcon\x22,\x22rem\
oveDrawLinks\x22,\x22r\
emoveEventHandle\
r\x22,\x22removeFromRe\
mainsCollector\x22,\
\x22removeGoggles\x22,\
\x22removeGroupIcon\
\x22,\x22removeHandgun\
Item\x22,\x22removeHea\
dgear\x22,\x22removeIt\
em\x22,\x22removeItemF\
romBackpack\x22,\x22re\
moveItemFromUnif\
orm\x22,\x22removeItem\
FromVest\x22,\x22remov\
eItems\x22,\x22removeM\
agazine\x22,\x22remove\
MagazineGlobal\x22,\
\x22removeMagazines\
\x22,\x22removeMagazin\
esTurret\x22,\x22remov\
eMagazineTurret\x22\
,\x22removeMenuItem\
\x22,\x22removeMission\
EventHandler\x22,\x22r\
emoveMPEventHand\
ler\x22,\x22removeMusi\
cEventHandler\x22,\x22\
removeOwnedMine\x22\
,\x22removePrimaryW\
eaponItem\x22,\x22remo\
veSecondaryWeapo\
nItem\x22,\x22removeSi\
mpleTask\x22,\x22remov\
eSwitchableUnit\x22\
,\x22removeTeamMemb\
er\x22,\x22removeUnifo\
rm\x22,\x22removeUserA\
ctionEventHandle\
r\x22,\x22removeVest\x22,\
\x22removeWeapon\x22,\x22\
removeWeaponAtta\
chmentCargo\x22,\x22re\
moveWeaponCargo\x22\
,\x22removeWeaponGl\
obal\x22,\x22removeWea\
ponTurret\x22,\x22repo\
rtRemoteTarget\x22,\
\x22requiredVersion\
\x22,\x22resetCamShake\
\x22,\x22resetSubgroup\
Direction\x22,\x22resi\
ze\x22,\x22resources\x22,\
\x22respawnVehicle\x22\
,\x22restartEditorC\
amera\x22,\x22reveal\x22,\
\x22revealMine\x22,\x22re\
verse\x22,\x22reversed\
MouseY\x22,\x22roadAt\x22\
,\x22roadsConnected\
To\x22,\x22roleDescrip\
tion\x22,\x22ropeAttac\
hedObjects\x22,\x22rop\
eAttachedTo\x22,\x22ro\
peAttachEnabled\x22\
,\x22ropeAttachTo\x22,\
\x22ropeCreate\x22,\x22ro\
peCut\x22,\x22ropeDest\
roy\x22,\x22ropeDetach\
\x22,\x22ropeEndPositi\
on\x22,\x22ropeLength\x22\
,\x22ropes\x22,\x22ropesA\
ttachedTo\x22,\x22rope\
Segments\x22,\x22ropeU\
nwind\x22,\x22ropeUnwo\
und\x22,\x22rotorsForc\
esRTD\x22,\x22rotorsRp\
mRTD\x22,\x22round\x22,\x22r\
unInitScript\x22,\x22s\
afeZoneH\x22,\x22safeZ\
oneW\x22,\x22safeZoneW\
Abs\x22,\x22safeZoneX\x22\
,\x22safeZoneXAbs\x22,\
\x22safeZoneY\x22,\x22sav\
e3DENInventory\x22,\
\x22saveGame\x22,\x22save\
Identity\x22,\x22saveJ\
oysticks\x22,\x22saveM\
issionProfileNam\
espace\x22,\x22saveOve\
rlay\x22,\x22saveProfi\
leNamespace\x22,\x22sa\
veStatus\x22,\x22saveV\
ar\x22,\x22savingEnabl\
ed\x22,\x22say\x22,\x22say2D\
\x22,\x22say3D\x22,\x22scope\
Name\x22,\x22score\x22,\x22s\
coreSide\x22,\x22scree\
nshot\x22,\x22screenTo\
World\x22,\x22scriptDo\
ne\x22,\x22scriptName\x22\
,\x22scudState\x22,\x22se\
condaryWeapon\x22,\x22\
secondaryWeaponI\
tems\x22,\x22secondary\
WeaponMagazine\x22,\
\x22select\x22,\x22select\
BestPlaces\x22,\x22sel\
ectDiarySubject\x22\
,\x22selectedEditor\
Objects\x22,\x22select\
EditorObject\x22,\x22s\
electionNames\x22,\x22\
selectionPositio\
n\x22,\x22selectionVec\
torDirAndUp\x22,\x22se\
lectLeader\x22,\x22sel\
ectMax\x22,\x22selectM\
in\x22,\x22selectNoPla\
yer\x22,\x22selectPlay\
er\x22,\x22selectRando\
m\x22,\x22selectRandom\
Weighted\x22,\x22selec\
tWeapon\x22,\x22select\
WeaponTurret\x22,\x22s\
endAUMessage\x22,\x22s\
endSimpleCommand\
\x22,\x22sendTask\x22,\x22se\
ndTaskResult\x22,\x22s\
endUDPMessage\x22,\x22\
sentencesEnabled\
\x22,\x22serverCommand\
\x22,\x22serverCommand\
Available\x22,\x22serv\
erCommandExecuta\
ble\x22,\x22serverName\
\x22,\x22serverNamespa\
ce\x22,\x22serverTime\x22\
,\x22set\x22,\x22set3DENA\
ttribute\x22,\x22set3D\
ENAttributes\x22,\x22s\
et3DENGrid\x22,\x22set\
3DENIconsVisible\
\x22,\x22set3DENLayer\x22\
,\x22set3DENLinesVi\
sible\x22,\x22set3DENL\
ogicType\x22,\x22set3D\
ENMissionAttribu\
te\x22,\x22set3DENMiss\
ionAttributes\x22,\x22\
set3DENModelsVis\
ible\x22,\x22set3DENOb\
jectType\x22,\x22set3D\
ENSelected\x22,\x22set\
AccTime\x22,\x22setAct\
ualCollectiveRTD\
\x22,\x22setAirplaneTh\
rottle\x22,\x22setAirp\
ortSide\x22,\x22setAmm\
o\x22,\x22setAmmoCargo\
\x22,\x22setAmmoOnPylo\
n\x22,\x22setAnimSpeed\
Coef\x22,\x22setApertu\
re\x22,\x22setAperture\
New\x22,\x22setArmoryP\
oints\x22,\x22setAttri\
butes\x22,\x22setAuton\
omous\x22,\x22setBehav\
iour\x22,\x22setBehavi\
ourStrong\x22,\x22setB\
leedingRemaining\
\x22,\x22setBrakesRTD\x22\
,\x22setCameraInter\
est\x22,\x22setCamShak\
eDefParams\x22,\x22set\
CamShakeParams\x22,\
\x22setCamUseTi\x22,\x22s\
etCaptive\x22,\x22setC\
enterOfMass\x22,\x22se\
tCollisionLight\x22\
,\x22setCombatBehav\
iour\x22,\x22setCombat\
Mode\x22,\x22setCompas\
sOscillation\x22,\x22s\
etConvoySeparati\
on\x22,\x22setCruiseCo\
ntrol\x22,\x22setCurat\
orCameraAreaCeil\
ing\x22,\x22setCurator\
Coef\x22,\x22setCurato\
rEditingAreaType\
\x22,\x22setCuratorWay\
pointCost\x22,\x22setC\
urrentChannel\x22,\x22\
setCurrentTask\x22,\
\x22setCurrentWaypo\
int\x22,\x22setCustomA\
imCoef\x22,\x22SetCust\
omMissionData\x22,\x22\
setCustomSoundCo\
ntroller\x22,\x22setCu\
stomWeightRTD\x22,\x22\
setDamage\x22,\x22setD\
ammage\x22,\x22setDate\
\x22,\x22setDebriefing\
Text\x22,\x22setDefaul\
tCamera\x22,\x22setDes\
tination\x22,\x22setDe\
tailMapBlendPars\
\x22,\x22setDiaryRecor\
dText\x22,\x22setDiary\
SubjectPicture\x22,\
\x22setDir\x22,\x22setDir\
ection\x22,\x22setDraw\
Icon\x22,\x22setDriveO\
nPath\x22,\x22setDropI\
nterval\x22,\x22setDyn\
amicSimulationDi\
stance\x22,\x22setDyna\
micSimulationDis\
tanceCoef\x22,\x22setE\
ditorMode\x22,\x22setE\
ditorObjectScope\
\x22,\x22setEffectCond\
ition\x22,\x22setEffec\
tiveCommander\x22,\x22\
setEngineRpmRTD\x22\
,\x22setFace\x22,\x22setF\
aceanimation\x22,\x22s\
etFatigue\x22,\x22setF\
eatureType\x22,\x22set\
FlagAnimationPha\
se\x22,\x22setFlagOwne\
r\x22,\x22setFlagSide\x22\
,\x22setFlagTexture\
\x22,\x22setFog\x22,\x22setF\
orceGeneratorRTD\
\x22,\x22setFormation\x22\
,\x22setFormationTa\
sk\x22,\x22setFormDir\x22\
,\x22setFriend\x22,\x22se\
tFromEditor\x22,\x22se\
tFSMVariable\x22,\x22s\
etFuel\x22,\x22setFuel\
Cargo\x22,\x22setGroup\
Icon\x22,\x22setGroupI\
conParams\x22,\x22setG\
roupIconsSelecta\
ble\x22,\x22setGroupIc\
onsVisible\x22,\x22set\
Groupid\x22,\x22setGro\
upIdGlobal\x22,\x22set\
GroupOwner\x22,\x22set\
Gusts\x22,\x22setHideB\
ehind\x22,\x22setHit\x22,\
\x22setHitIndex\x22,\x22s\
etHitPointDamage\
\x22,\x22setHorizonPar\
allaxCoef\x22,\x22setH\
UDMovementLevels\
\x22,\x22setHumidity\x22,\
\x22setIdentity\x22,\x22s\
etImportance\x22,\x22s\
etInfoPanel\x22,\x22se\
tLeader\x22,\x22setLig\
htAmbient\x22,\x22setL\
ightAttenuation\x22\
,\x22setLightBright\
ness\x22,\x22setLightC\
olor\x22,\x22setLightC\
onePars\x22,\x22setLig\
htDayLight\x22,\x22set\
LightFlareMaxDis\
tance\x22,\x22setLight\
FlareSize\x22,\x22setL\
ightIntensity\x22,\x22\
setLightIR\x22,\x22set\
Lightnings\x22,\x22set\
LightUseFlare\x22,\x22\
setLightVolumeSh\
ape\x22,\x22setLocalWi\
ndParams\x22,\x22setMa\
gazineTurretAmmo\
\x22,\x22setMarkerAlph\
a\x22,\x22setMarkerAlp\
haLocal\x22,\x22setMar\
kerBrush\x22,\x22setMa\
rkerBrushLocal\x22,\
\x22setMarkerColor\x22\
,\x22setMarkerColor\
Local\x22,\x22setMarke\
rDir\x22,\x22setMarker\
DirLocal\x22,\x22setMa\
rkerPolyline\x22,\x22s\
etMarkerPolyline\
Local\x22,\x22setMarke\
rPos\x22,\x22setMarker\
PosLocal\x22,\x22setMa\
rkerShadow\x22,\x22set\
MarkerShadowLoca\
l\x22,\x22setMarkerSha\
pe\x22,\x22setMarkerSh\
apeLocal\x22,\x22setMa\
rkerSize\x22,\x22setMa\
rkerSizeLocal\x22,\x22\
setMarkerText\x22,\x22\
setMarkerTextLoc\
al\x22,\x22setMarkerTy\
pe\x22,\x22setMarkerTy\
peLocal\x22,\x22setMas\
s\x22,\x22setMaxLoad\x22,\
\x22setMimic\x22,\x22setM\
issileTarget\x22,\x22s\
etMissileTargetP\
os\x22,\x22setMousePos\
ition\x22,\x22setMusic\
Effect\x22,\x22setMusi\
cEventHandler\x22,\x22\
setName\x22,\x22setNam\
eSound\x22,\x22setObje\
ctArguments\x22,\x22se\
tObjectMaterial\x22\
,\x22setObjectMater\
ialGlobal\x22,\x22setO\
bjectProxy\x22,\x22set\
ObjectScale\x22,\x22se\
tObjectTexture\x22,\
\x22setObjectTextur\
eGlobal\x22,\x22setObj\
ectViewDistance\x22\
,\x22setOpticsMode\x22\
,\x22setOvercast\x22,\x22\
setOwner\x22,\x22setOx\
ygenRemaining\x22,\x22\
setParticleCircl\
e\x22,\x22setParticleC\
lass\x22,\x22setPartic\
leFire\x22,\x22setPart\
icleParams\x22,\x22set\
ParticleRandom\x22,\
\x22setPilotCameraD\
irection\x22,\x22setPi\
lotCameraRotatio\
n\x22,\x22setPilotCame\
raTarget\x22,\x22setPi\
lotLight\x22,\x22setPi\
PEffect\x22,\x22setPiP\
ViewDistance\x22,\x22s\
etPitch\x22,\x22setPla\
teNumber\x22,\x22setPl\
ayable\x22,\x22setPlay\
erRespawnTime\x22,\x22\
setPlayerVoNVolu\
me\x22,\x22setPos\x22,\x22se\
tPosASL\x22,\x22setPos\
ASL2\x22,\x22setPosASL\
W\x22,\x22setPosATL\x22,\x22\
setPosition\x22,\x22se\
tPosWorld\x22,\x22setP\
ylonLoadout\x22,\x22se\
tPylonsPriority\x22\
,\x22setRadioMsg\x22,\x22\
setRain\x22,\x22setRai\
nbow\x22,\x22setRandom\
Lip\x22,\x22setRank\x22,\x22\
setRectangular\x22,\
\x22setRepairCargo\x22\
,\x22setRotorBrakeR\
TD\x22,\x22setShadowDi\
stance\x22,\x22setShot\
Parents\x22,\x22setSid\
e\x22,\x22setSimpleTas\
kAlwaysVisible\x22,\
\x22setSimpleTaskCu\
stomData\x22,\x22setSi\
mpleTaskDescript\
ion\x22,\x22setSimpleT\
askDestination\x22,\
\x22setSimpleTaskTa\
rget\x22,\x22setSimple\
TaskType\x22,\x22setSi\
mulWeatherLayers\
\x22,\x22setSize\x22,\x22set\
Skill\x22,\x22setSling\
Load\x22,\x22setSoundE\
ffect\x22,\x22setSpeak\
er\x22,\x22setSpeech\x22,\
\x22setSpeedMode\x22,\x22\
setStamina\x22,\x22set\
StaminaScheme\x22,\x22\
setStatValue\x22,\x22s\
etSuppression\x22,\x22\
setSystemOfUnits\
\x22,\x22setTargetAge\x22\
,\x22setTaskMarkerO\
ffset\x22,\x22setTaskR\
esult\x22,\x22setTaskS\
tate\x22,\x22setTerrai\
nGrid\x22,\x22setTerra\
inHeight\x22,\x22setTe\
xt\x22,\x22setTimeMult\
iplier\x22,\x22setTiPa\
rameter\x22,\x22setTit\
leEffect\x22,\x22setTo\
wParent\x22,\x22setTra\
fficDensity\x22,\x22se\
tTrafficDistance\
\x22,\x22setTrafficGap\
\x22,\x22setTrafficSpe\
ed\x22,\x22setTriggerA\
ctivation\x22,\x22setT\
riggerArea\x22,\x22set\
TriggerInterval\x22\
,\x22setTriggerStat\
ements\x22,\x22setTrig\
gerText\x22,\x22setTri\
ggerTimeout\x22,\x22se\
tTriggerType\x22,\x22s\
etTurretLimits\x22,\
\x22setTurretOptics\
Mode\x22,\x22setType\x22,\
\x22setUnconscious\x22\
,\x22setUnitAbility\
\x22,\x22setUnitCombat\
Mode\x22,\x22setUnitFr\
eefallHeight\x22,\x22s\
etUnitLoadout\x22,\x22\
setUnitPos\x22,\x22set\
UnitPosWeak\x22,\x22se\
tUnitRank\x22,\x22setU\
nitRecoilCoeffic\
ient\x22,\x22setUnitTr\
ait\x22,\x22setUnloadI\
nCombat\x22,\x22setUse\
rActionText\x22,\x22se\
tUserMFDText\x22,\x22s\
etUserMFDValue\x22,\
\x22setVariable\x22,\x22s\
etVectorDir\x22,\x22se\
tVectorDirAndUp\x22\
,\x22setVectorUp\x22,\x22\
setVehicleAmmo\x22,\
\x22setVehicleAmmoD\
ef\x22,\x22setVehicleA\
rmor\x22,\x22setVehicl\
eCargo\x22,\x22setVehi\
cleId\x22,\x22setVehic\
leLock\x22,\x22setVehi\
clePosition\x22,\x22se\
tVehicleRadar\x22,\x22\
setVehicleReceiv\
eRemoteTargets\x22,\
\x22setVehicleRepor\
tOwnPosition\x22,\x22s\
etVehicleReportR\
emoteTargets\x22,\x22s\
etVehicleTiPars\x22\
,\x22setVehicleVarN\
ame\x22,\x22setVelocit\
y\x22,\x22setVelocityM\
odelSpace\x22,\x22setV\
elocityTransform\
ation\x22,\x22setViewD\
istance\x22,\x22setVis\
ibleIfTreeCollap\
sed\x22,\x22setWantedR\
PMRTD\x22,\x22setWaves\
\x22,\x22setWaypointBe\
haviour\x22,\x22setWay\
pointCombatMode\x22\
,\x22setWaypointCom\
pletionRadius\x22,\x22\
setWaypointDescr\
iption\x22,\x22setWayp\
ointForceBehavio\
ur\x22,\x22setWaypoint\
Formation\x22,\x22setW\
aypointHousePosi\
tion\x22,\x22setWaypoi\
ntLoiterAltitude\
\x22,\x22setWaypointLo\
iterRadius\x22,\x22set\
WaypointLoiterTy\
pe\x22,\x22setWaypoint\
Name\x22,\x22setWaypoi\
ntPosition\x22,\x22set\
WaypointScript\x22,\
\x22setWaypointSpee\
d\x22,\x22setWaypointS\
tatements\x22,\x22setW\
aypointTimeout\x22,\
\x22setWaypointType\
\x22,\x22setWaypointVi\
sible\x22,\x22setWeapo\
nReloadingTime\x22,\
\x22setWeaponZeroin\
g\x22,\x22setWind\x22,\x22se\
tWindDir\x22,\x22setWi\
ndForce\x22,\x22setWin\
dStr\x22,\x22setWingFo\
rceScaleRTD\x22,\x22se\
tWPPos\x22,\x22show3DI\
cons\x22,\x22showChat\x22\
,\x22showCinemaBord\
er\x22,\x22showCommand\
ingMenu\x22,\x22showCo\
mpass\x22,\x22showCura\
torCompass\x22,\x22sho\
wGps\x22,\x22showHUD\x22,\
\x22showLegend\x22,\x22sh\
owMap\x22,\x22shownArt\
illeryComputer\x22,\
\x22shownChat\x22,\x22sho\
wnCompass\x22,\x22show\
nCuratorCompass\x22\
,\x22showNewEditorO\
bject\x22,\x22shownGps\
\x22,\x22shownHUD\x22,\x22sh\
ownMap\x22,\x22shownPa\
d\x22,\x22shownRadio\x22,\
\x22shownScoretable\
\x22,\x22shownSubtitle\
s\x22,\x22shownUAVFeed\
\x22,\x22shownWarrant\x22\
,\x22shownWatch\x22,\x22s\
howPad\x22,\x22showRad\
io\x22,\x22showScoreta\
ble\x22,\x22showSubtit\
les\x22,\x22showUAVFee\
d\x22,\x22showWarrant\x22\
,\x22showWatch\x22,\x22sh\
owWaypoint\x22,\x22sho\
wWaypoints\x22,\x22sid\
e\x22,\x22sideChat\x22,\x22s\
ideRadio\x22,\x22simpl\
eTasks\x22,\x22simulat\
ionEnabled\x22,\x22sim\
ulCloudDensity\x22,\
\x22simulCloudOcclu\
sion\x22,\x22simulInCl\
ouds\x22,\x22simulWeat\
herSync\x22,\x22sin\x22,\x22\
size\x22,\x22sizeOf\x22,\x22\
skill\x22,\x22skillFin\
al\x22,\x22skipTime\x22,\x22\
sleep\x22,\x22sliderPo\
sition\x22,\x22sliderR\
ange\x22,\x22sliderSet\
Position\x22,\x22slide\
rSetRange\x22,\x22slid\
erSetSpeed\x22,\x22sli\
derSpeed\x22,\x22sling\
LoadAssistantSho\
wn\x22,\x22soldierMaga\
zines\x22,\x22someAmmo\
\x22,\x22sort\x22,\x22soundV\
olume\x22,\x22spawn\x22,\x22\
speaker\x22,\x22speech\
Volume\x22,\x22speed\x22,\
\x22speedMode\x22,\x22spl\
itString\x22,\x22sqrt\x22\
,\x22squadParams\x22,\x22\
stance\x22,\x22startLo\
adingScreen\x22,\x22st\
op\x22,\x22stopEngineR\
TD\x22,\x22stopped\x22,\x22s\
tr\x22,\x22sunOrMoon\x22,\
\x22supportInfo\x22,\x22s\
uppressFor\x22,\x22sur\
faceIsWater\x22,\x22su\
rfaceNormal\x22,\x22su\
rfaceTexture\x22,\x22s\
urfaceType\x22,\x22swi\
mInDepth\x22,\x22switc\
hableUnits\x22,\x22swi\
tchAction\x22,\x22swit\
chCamera\x22,\x22switc\
hGesture\x22,\x22switc\
hLight\x22,\x22switchM\
ove\x22,\x22synchroniz\
edObjects\x22,\x22sync\
hronizedTriggers\
\x22,\x22synchronizedW\
aypoints\x22,\x22synch\
ronizeObjectsAdd\
\x22,\x22synchronizeOb\
jectsRemove\x22,\x22sy\
nchronizeTrigger\
\x22,\x22synchronizeWa\
ypoint\x22,\x22systemC\
hat\x22,\x22systemOfUn\
its\x22,\x22systemTime\
\x22,\x22systemTimeUTC\
\x22,\x22tan\x22,\x22targetK\
nowledge\x22,\x22targe\
ts\x22,\x22targetsAggr\
egate\x22,\x22targetsQ\
uery\x22,\x22taskAlway\
sVisible\x22,\x22taskC\
hildren\x22,\x22taskCo\
mpleted\x22,\x22taskCu\
stomData\x22,\x22taskD\
escription\x22,\x22tas\
kDestination\x22,\x22t\
askHint\x22,\x22taskMa\
rkerOffset\x22,\x22tas\
kName\x22,\x22taskPare\
nt\x22,\x22taskResult\x22\
,\x22taskState\x22,\x22ta\
skType\x22,\x22teamMem\
ber\x22,\x22teamName\x22,\
\x22teams\x22,\x22teamSwi\
tch\x22,\x22teamSwitch\
Enabled\x22,\x22teamTy\
pe\x22,\x22terminate\x22,\
\x22terrainIntersec\
t\x22,\x22terrainInter\
sectASL\x22,\x22terrai\
nIntersectAtASL\x22\
,\x22text\x22,\x22textLog\
\x22,\x22textLogFormat\
\x22,\x22tg\x22,\x22time\x22,\x22t\
imeMultiplier\x22,\x22\
titleCut\x22,\x22title\
FadeOut\x22,\x22titleO\
bj\x22,\x22titleRsc\x22,\x22\
titleText\x22,\x22toAr\
ray\x22,\x22toFixed\x22,\x22\
toLower\x22,\x22toLowe\
rANSI\x22,\x22toString\
\x22,\x22toUpper\x22,\x22toU\
pperANSI\x22,\x22trigg\
erActivated\x22,\x22tr\
iggerActivation\x22\
,\x22triggerAmmo\x22,\x22\
triggerArea\x22,\x22tr\
iggerAttachedVeh\
icle\x22,\x22triggerAt\
tachObject\x22,\x22tri\
ggerAttachVehicl\
e\x22,\x22triggerDynam\
icSimulation\x22,\x22t\
riggerInterval\x22,\
\x22triggerStatemen\
ts\x22,\x22triggerText\
\x22,\x22triggerTimeou\
t\x22,\x22triggerTimeo\
utCurrent\x22,\x22trig\
gerType\x22,\x22trim\x22,\
\x22turretLocal\x22,\x22t\
urretOwner\x22,\x22tur\
retUnit\x22,\x22tvAdd\x22\
,\x22tvClear\x22,\x22tvCo\
llapse\x22,\x22tvColla\
pseAll\x22,\x22tvCount\
\x22,\x22tvCurSel\x22,\x22tv\
Data\x22,\x22tvDelete\x22\
,\x22tvExpand\x22,\x22tvE\
xpandAll\x22,\x22tvIsS\
elected\x22,\x22tvPict\
ure\x22,\x22tvPictureR\
ight\x22,\x22tvSelecti\
on\x22,\x22tvSetColor\x22\
,\x22tvSetCurSel\x22,\x22\
tvSetData\x22,\x22tvSe\
tPicture\x22,\x22tvSet\
PictureColor\x22,\x22t\
vSetPictureColor\
Disabled\x22,\x22tvSet\
PictureColorSele\
cted\x22,\x22tvSetPict\
ureRight\x22,\x22tvSet\
PictureRightColo\
r\x22,\x22tvSetPicture\
RightColorDisabl\
ed\x22,\x22tvSetPictur\
eRightColorSelec\
ted\x22,\x22tvSetSelec\
tColor\x22,\x22tvSetSe\
lected\x22,\x22tvSetTe\
xt\x22,\x22tvSetToolti\
p\x22,\x22tvSetValue\x22,\
\x22tvSort\x22,\x22tvSort\
All\x22,\x22tvSortByVa\
lue\x22,\x22tvSortByVa\
lueAll\x22,\x22tvText\x22\
,\x22tvTooltip\x22,\x22tv\
Value\x22,\x22type\x22,\x22t\
ypeName\x22,\x22typeOf\
\x22,\x22UAVControl\x22,\x22\
uiNamespace\x22,\x22ui\
Sleep\x22,\x22unassign\
Curator\x22,\x22unassi\
gnItem\x22,\x22unassig\
nTeam\x22,\x22unassign\
Vehicle\x22,\x22underw\
ater\x22,\x22uniform\x22,\
\x22uniformContaine\
r\x22,\x22uniformItems\
\x22,\x22uniformMagazi\
nes\x22,\x22uniqueUnit\
Items\x22,\x22unitAddo\
ns\x22,\x22unitAimPosi\
tion\x22,\x22unitAimPo\
sitionVisual\x22,\x22u\
nitBackpack\x22,\x22un\
itCombatMode\x22,\x22u\
nitIsUAV\x22,\x22unitP\
os\x22,\x22unitReady\x22,\
\x22unitRecoilCoeff\
icient\x22,\x22units\x22,\
\x22unitsBelowHeigh\
t\x22,\x22unitTurret\x22,\
\x22unlinkItem\x22,\x22un\
lockAchievement\x22\
,\x22unregisterTask\
\x22,\x22updateDrawIco\
n\x22,\x22updateMenuIt\
em\x22,\x22updateObjec\
tTree\x22,\x22useAIOpe\
rMapObstructionT\
est\x22,\x22useAISteer\
ingComponent\x22,\x22u\
seAudioTimeForMo\
ves\x22,\x22userInputD\
isabled\x22,\x22values\
\x22,\x22vectorAdd\x22,\x22v\
ectorCos\x22,\x22vecto\
rCrossProduct\x22,\x22\
vectorDiff\x22,\x22vec\
torDir\x22,\x22vectorD\
irVisual\x22,\x22vecto\
rDistance\x22,\x22vect\
orDistanceSqr\x22,\x22\
vectorDotProduct\
\x22,\x22vectorFromTo\x22\
,\x22vectorLinearCo\
nversion\x22,\x22vecto\
rMagnitude\x22,\x22vec\
torMagnitudeSqr\x22\
,\x22vectorModelToW\
orld\x22,\x22vectorMod\
elToWorldVisual\x22\
,\x22vectorMultiply\
\x22,\x22vectorNormali\
zed\x22,\x22vectorUp\x22,\
\x22vectorUpVisual\x22\
,\x22vectorWorldToM\
odel\x22,\x22vectorWor\
ldToModelVisual\x22\
,\x22vehicle\x22,\x22vehi\
cleCargoEnabled\x22\
,\x22vehicleChat\x22,\x22\
vehicleMoveInfo\x22\
,\x22vehicleRadio\x22,\
\x22vehicleReceiveR\
emoteTargets\x22,\x22v\
ehicleReportOwnP\
osition\x22,\x22vehicl\
eReportRemoteTar\
gets\x22,\x22vehicles\x22\
,\x22vehicleVarName\
\x22,\x22velocity\x22,\x22ve\
locityModelSpace\
\x22,\x22verifySignatu\
re\x22,\x22vest\x22,\x22vest\
Container\x22,\x22vest\
Items\x22,\x22vestMaga\
zines\x22,\x22viewDist\
ance\x22,\x22visibleCo\
mpass\x22,\x22visibleG\
ps\x22,\x22visibleMap\x22\
,\x22visiblePositio\
n\x22,\x22visiblePosit\
ionASL\x22,\x22visible\
Scoretable\x22,\x22vis\
ibleWatch\x22,\x22wave\
s\x22,\x22waypointAtta\
chedObject\x22,\x22way\
pointAttachedVeh\
icle\x22,\x22waypointA\
ttachObject\x22,\x22wa\
ypointAttachVehi\
cle\x22,\x22waypointBe\
haviour\x22,\x22waypoi\
ntCombatMode\x22,\x22w\
aypointCompletio\
nRadius\x22,\x22waypoi\
ntDescription\x22,\x22\
waypointForceBeh\
aviour\x22,\x22waypoin\
tFormation\x22,\x22way\
pointHousePositi\
on\x22,\x22waypointLoi\
terAltitude\x22,\x22wa\
ypointLoiterRadi\
us\x22,\x22waypointLoi\
terType\x22,\x22waypoi\
ntName\x22,\x22waypoin\
tPosition\x22,\x22wayp\
oints\x22,\x22waypoint\
Script\x22,\x22waypoin\
tsEnabledUAV\x22,\x22w\
aypointShow\x22,\x22wa\
ypointSpeed\x22,\x22wa\
ypointStatements\
\x22,\x22waypointTimeo\
ut\x22,\x22waypointTim\
eoutCurrent\x22,\x22wa\
ypointType\x22,\x22way\
pointVisible\x22,\x22w\
eaponAccessories\
\x22,\x22weaponAccesso\
riesCargo\x22,\x22weap\
onCargo\x22,\x22weapon\
Direction\x22,\x22weap\
onInertia\x22,\x22weap\
onLowered\x22,\x22weap\
onReloadingTime\x22\
,\x22weapons\x22,\x22weap\
onsInfo\x22,\x22weapon\
sItems\x22,\x22weapons\
ItemsCargo\x22,\x22wea\
ponState\x22,\x22weapo\
nsTurret\x22,\x22weigh\
tRTD\x22,\x22WFSideTex\
t\x22,\x22wind\x22,\x22windD\
ir\x22,\x22windRTD\x22,\x22w\
indStr\x22,\x22wingsFo\
rcesRTD\x22,\x22worldN\
ame\x22,\x22worldSize\x22\
,\x22worldToModel\x22,\
\x22worldToModelVis\
ual\x22,\x22worldToScr\
een\x22],\x0aliteral:[\
\x22blufor\x22,\x22civili\
an\x22,\x22configNull\x22\
,\x22controlNull\x22,\x22\
displayNull\x22,\x22di\
aryRecordNull\x22,\x22\
east\x22,\x22endl\x22,\x22fa\
lse\x22,\x22grpNull\x22,\x22\
independent\x22,\x22li\
neBreak\x22,\x22locati\
onNull\x22,\x22nil\x22,\x22o\
bjNull\x22,\x22opfor\x22,\
\x22pi\x22,\x22resistance\
\x22,\x22scriptNull\x22,\x22\
sideAmbientLife\x22\
,\x22sideEmpty\x22,\x22si\
deEnemy\x22,\x22sideFr\
iendly\x22,\x22sideLog\
ic\x22,\x22sideUnknown\
\x22,\x22taskNull\x22,\x22te\
amMemberNull\x22,\x22t\
rue\x22,\x22west\x22]\x0a},c\
ontains:[e.C_LIN\
E_COMMENT_MODE,e\
.C_BLOCK_COMMENT\
_MODE,e.NUMBER_M\
ODE,{\x0aclassName:\
\x22variable\x22,begin\
:/\x5cb_+[a-zA-Z]\x5cw\
*/},{className:\x22\
title\x22,\x0abegin:/[\
a-zA-Z][a-zA-Z_0\
-9]*_fnc_[a-zA-Z\
_0-9]+/},t,a],\x0ai\
llegal:[/\x5c$[^a-f\
A-F0-9]/,/\x5cw\x5c$/,\
/\x5c?/,/@/,/ \x5c| /,\
/[a-zA-Z_]\x5c./,/\x5c\
:\x5c=/,/\x5c[\x5c:/]}}\x0a}\
)();hljs.registe\
rLanguage(\x22sqf\x22,\
e)})();/*! `sql`\
 grammar compile\
d for Highlight.\
js 11.10.0 */\x0a((\
)=>{var e=(()=>{\
\x22use strict\x22;ret\
urn e=>{\x0aconst r\
=e.regex,t=e.COM\
MENT(\x22--\x22,\x22$\x22),n\
=[\x22true\x22,\x22false\x22\
,\x22unknown\x22],a=[\x22\
bigint\x22,\x22binary\x22\
,\x22blob\x22,\x22boolean\
\x22,\x22char\x22,\x22charac\
ter\x22,\x22clob\x22,\x22dat\
e\x22,\x22dec\x22,\x22decflo\
at\x22,\x22decimal\x22,\x22f\
loat\x22,\x22int\x22,\x22int\
eger\x22,\x22interval\x22\
,\x22nchar\x22,\x22nclob\x22\
,\x22national\x22,\x22num\
eric\x22,\x22real\x22,\x22ro\
w\x22,\x22smallint\x22,\x22t\
ime\x22,\x22timestamp\x22\
,\x22varchar\x22,\x22vary\
ing\x22,\x22varbinary\x22\
],i=[\x22abs\x22,\x22acos\
\x22,\x22array_agg\x22,\x22a\
sin\x22,\x22atan\x22,\x22avg\
\x22,\x22cast\x22,\x22ceil\x22,\
\x22ceiling\x22,\x22coale\
sce\x22,\x22corr\x22,\x22cos\
\x22,\x22cosh\x22,\x22count\x22\
,\x22covar_pop\x22,\x22co\
var_samp\x22,\x22cume_\
dist\x22,\x22dense_ran\
k\x22,\x22deref\x22,\x22elem\
ent\x22,\x22exp\x22,\x22extr\
act\x22,\x22first_valu\
e\x22,\x22floor\x22,\x22json\
_array\x22,\x22json_ar\
rayagg\x22,\x22json_ex\
ists\x22,\x22json_obje\
ct\x22,\x22json_object\
agg\x22,\x22json_query\
\x22,\x22json_table\x22,\x22\
json_table_primi\
tive\x22,\x22json_valu\
e\x22,\x22lag\x22,\x22last_v\
alue\x22,\x22lead\x22,\x22li\
stagg\x22,\x22ln\x22,\x22log\
\x22,\x22log10\x22,\x22lower\
\x22,\x22max\x22,\x22min\x22,\x22m\
od\x22,\x22nth_value\x22,\
\x22ntile\x22,\x22nullif\x22\
,\x22percent_rank\x22,\
\x22percentile_cont\
\x22,\x22percentile_di\
sc\x22,\x22position\x22,\x22\
position_regex\x22,\
\x22power\x22,\x22rank\x22,\x22\
regr_avgx\x22,\x22regr\
_avgy\x22,\x22regr_cou\
nt\x22,\x22regr_interc\
ept\x22,\x22regr_r2\x22,\x22\
regr_slope\x22,\x22reg\
r_sxx\x22,\x22regr_sxy\
\x22,\x22regr_syy\x22,\x22ro\
w_number\x22,\x22sin\x22,\
\x22sinh\x22,\x22sqrt\x22,\x22s\
tddev_pop\x22,\x22stdd\
ev_samp\x22,\x22substr\
ing\x22,\x22substring_\
regex\x22,\x22sum\x22,\x22ta\
n\x22,\x22tanh\x22,\x22trans\
late\x22,\x22translate\
_regex\x22,\x22treat\x22,\
\x22trim\x22,\x22trim_arr\
ay\x22,\x22unnest\x22,\x22up\
per\x22,\x22value_of\x22,\
\x22var_pop\x22,\x22var_s\
amp\x22,\x22width_buck\
et\x22],s=[\x22create \
table\x22,\x22insert i\
nto\x22,\x22primary ke\
y\x22,\x22foreign key\x22\
,\x22not null\x22,\x22alt\
er table\x22,\x22add c\
onstraint\x22,\x22grou\
ping sets\x22,\x22on o\
verflow\x22,\x22charac\
ter set\x22,\x22respec\
t nulls\x22,\x22ignore\
 nulls\x22,\x22nulls f\
irst\x22,\x22nulls las\
t\x22,\x22depth first\x22\
,\x22breadth first\x22\
],o=i,c=[\x22abs\x22,\x22\
acos\x22,\x22all\x22,\x22all\
ocate\x22,\x22alter\x22,\x22\
and\x22,\x22any\x22,\x22are\x22\
,\x22array\x22,\x22array_\
agg\x22,\x22array_max_\
cardinality\x22,\x22as\
\x22,\x22asensitive\x22,\x22\
asin\x22,\x22asymmetri\
c\x22,\x22at\x22,\x22atan\x22,\x22\
atomic\x22,\x22authori\
zation\x22,\x22avg\x22,\x22b\
egin\x22,\x22begin_fra\
me\x22,\x22begin_parti\
tion\x22,\x22between\x22,\
\x22bigint\x22,\x22binary\
\x22,\x22blob\x22,\x22boolea\
n\x22,\x22both\x22,\x22by\x22,\x22\
call\x22,\x22called\x22,\x22\
cardinality\x22,\x22ca\
scaded\x22,\x22case\x22,\x22\
cast\x22,\x22ceil\x22,\x22ce\
iling\x22,\x22char\x22,\x22c\
har_length\x22,\x22cha\
racter\x22,\x22charact\
er_length\x22,\x22chec\
k\x22,\x22classifier\x22,\
\x22clob\x22,\x22close\x22,\x22\
coalesce\x22,\x22colla\
te\x22,\x22collect\x22,\x22c\
olumn\x22,\x22commit\x22,\
\x22condition\x22,\x22con\
nect\x22,\x22constrain\
t\x22,\x22contains\x22,\x22c\
onvert\x22,\x22copy\x22,\x22\
corr\x22,\x22correspon\
ding\x22,\x22cos\x22,\x22cos\
h\x22,\x22count\x22,\x22cova\
r_pop\x22,\x22covar_sa\
mp\x22,\x22create\x22,\x22cr\
oss\x22,\x22cube\x22,\x22cum\
e_dist\x22,\x22current\
\x22,\x22current_catal\
og\x22,\x22current_dat\
e\x22,\x22current_defa\
ult_transform_gr\
oup\x22,\x22current_pa\
th\x22,\x22current_rol\
e\x22,\x22current_row\x22\
,\x22current_schema\
\x22,\x22current_time\x22\
,\x22current_timest\
amp\x22,\x22current_pa\
th\x22,\x22current_rol\
e\x22,\x22current_tran\
sform_group_for_\
type\x22,\x22current_u\
ser\x22,\x22cursor\x22,\x22c\
ycle\x22,\x22date\x22,\x22da\
y\x22,\x22deallocate\x22,\
\x22dec\x22,\x22decimal\x22,\
\x22decfloat\x22,\x22decl\
are\x22,\x22default\x22,\x22\
define\x22,\x22delete\x22\
,\x22dense_rank\x22,\x22d\
eref\x22,\x22describe\x22\
,\x22deterministic\x22\
,\x22disconnect\x22,\x22d\
istinct\x22,\x22double\
\x22,\x22drop\x22,\x22dynami\
c\x22,\x22each\x22,\x22eleme\
nt\x22,\x22else\x22,\x22empt\
y\x22,\x22end\x22,\x22end_fr\
ame\x22,\x22end_partit\
ion\x22,\x22end-exec\x22,\
\x22equals\x22,\x22escape\
\x22,\x22every\x22,\x22excep\
t\x22,\x22exec\x22,\x22execu\
te\x22,\x22exists\x22,\x22ex\
p\x22,\x22external\x22,\x22e\
xtract\x22,\x22false\x22,\
\x22fetch\x22,\x22filter\x22\
,\x22first_value\x22,\x22\
float\x22,\x22floor\x22,\x22\
for\x22,\x22foreign\x22,\x22\
frame_row\x22,\x22free\
\x22,\x22from\x22,\x22full\x22,\
\x22function\x22,\x22fusi\
on\x22,\x22get\x22,\x22globa\
l\x22,\x22grant\x22,\x22grou\
p\x22,\x22grouping\x22,\x22g\
roups\x22,\x22having\x22,\
\x22hold\x22,\x22hour\x22,\x22i\
dentity\x22,\x22in\x22,\x22i\
ndicator\x22,\x22initi\
al\x22,\x22inner\x22,\x22ino\
ut\x22,\x22insensitive\
\x22,\x22insert\x22,\x22int\x22\
,\x22integer\x22,\x22inte\
rsect\x22,\x22intersec\
tion\x22,\x22interval\x22\
,\x22into\x22,\x22is\x22,\x22jo\
in\x22,\x22json_array\x22\
,\x22json_arrayagg\x22\
,\x22json_exists\x22,\x22\
json_object\x22,\x22js\
on_objectagg\x22,\x22j\
son_query\x22,\x22json\
_table\x22,\x22json_ta\
ble_primitive\x22,\x22\
json_value\x22,\x22lag\
\x22,\x22language\x22,\x22la\
rge\x22,\x22last_value\
\x22,\x22lateral\x22,\x22lea\
d\x22,\x22leading\x22,\x22le\
ft\x22,\x22like\x22,\x22like\
_regex\x22,\x22listagg\
\x22,\x22ln\x22,\x22local\x22,\x22\
localtime\x22,\x22loca\
ltimestamp\x22,\x22log\
\x22,\x22log10\x22,\x22lower\
\x22,\x22match\x22,\x22match\
_number\x22,\x22match_\
recognize\x22,\x22matc\
hes\x22,\x22max\x22,\x22memb\
er\x22,\x22merge\x22,\x22met\
hod\x22,\x22min\x22,\x22minu\
te\x22,\x22mod\x22,\x22modif\
ies\x22,\x22module\x22,\x22m\
onth\x22,\x22multiset\x22\
,\x22national\x22,\x22nat\
ural\x22,\x22nchar\x22,\x22n\
clob\x22,\x22new\x22,\x22no\x22\
,\x22none\x22,\x22normali\
ze\x22,\x22not\x22,\x22nth_v\
alue\x22,\x22ntile\x22,\x22n\
ull\x22,\x22nullif\x22,\x22n\
umeric\x22,\x22octet_l\
ength\x22,\x22occurren\
ces_regex\x22,\x22of\x22,\
\x22offset\x22,\x22old\x22,\x22\
omit\x22,\x22on\x22,\x22one\x22\
,\x22only\x22,\x22open\x22,\x22\
or\x22,\x22order\x22,\x22out\
\x22,\x22outer\x22,\x22over\x22\
,\x22overlaps\x22,\x22ove\
rlay\x22,\x22parameter\
\x22,\x22partition\x22,\x22p\
attern\x22,\x22per\x22,\x22p\
ercent\x22,\x22percent\
_rank\x22,\x22percenti\
le_cont\x22,\x22percen\
tile_disc\x22,\x22peri\
od\x22,\x22portion\x22,\x22p\
osition\x22,\x22positi\
on_regex\x22,\x22power\
\x22,\x22precedes\x22,\x22pr\
ecision\x22,\x22prepar\
e\x22,\x22primary\x22,\x22pr\
ocedure\x22,\x22ptf\x22,\x22\
range\x22,\x22rank\x22,\x22r\
eads\x22,\x22real\x22,\x22re\
cursive\x22,\x22ref\x22,\x22\
references\x22,\x22ref\
erencing\x22,\x22regr_\
avgx\x22,\x22regr_avgy\
\x22,\x22regr_count\x22,\x22\
regr_intercept\x22,\
\x22regr_r2\x22,\x22regr_\
slope\x22,\x22regr_sxx\
\x22,\x22regr_sxy\x22,\x22re\
gr_syy\x22,\x22release\
\x22,\x22result\x22,\x22retu\
rn\x22,\x22returns\x22,\x22r\
evoke\x22,\x22right\x22,\x22\
rollback\x22,\x22rollu\
p\x22,\x22row\x22,\x22row_nu\
mber\x22,\x22rows\x22,\x22ru\
nning\x22,\x22savepoin\
t\x22,\x22scope\x22,\x22scro\
ll\x22,\x22search\x22,\x22se\
cond\x22,\x22seek\x22,\x22se\
lect\x22,\x22sensitive\
\x22,\x22session_user\x22\
,\x22set\x22,\x22show\x22,\x22s\
imilar\x22,\x22sin\x22,\x22s\
inh\x22,\x22skip\x22,\x22sma\
llint\x22,\x22some\x22,\x22s\
pecific\x22,\x22specif\
ictype\x22,\x22sql\x22,\x22s\
qlexception\x22,\x22sq\
lstate\x22,\x22sqlwarn\
ing\x22,\x22sqrt\x22,\x22sta\
rt\x22,\x22static\x22,\x22st\
ddev_pop\x22,\x22stdde\
v_samp\x22,\x22submult\
iset\x22,\x22subset\x22,\x22\
substring\x22,\x22subs\
tring_regex\x22,\x22su\
cceeds\x22,\x22sum\x22,\x22s\
ymmetric\x22,\x22syste\
m\x22,\x22system_time\x22\
,\x22system_user\x22,\x22\
table\x22,\x22tablesam\
ple\x22,\x22tan\x22,\x22tanh\
\x22,\x22then\x22,\x22time\x22,\
\x22timestamp\x22,\x22tim\
ezone_hour\x22,\x22tim\
ezone_minute\x22,\x22t\
o\x22,\x22trailing\x22,\x22t\
ranslate\x22,\x22trans\
late_regex\x22,\x22tra\
nslation\x22,\x22treat\
\x22,\x22trigger\x22,\x22tri\
m\x22,\x22trim_array\x22,\
\x22true\x22,\x22truncate\
\x22,\x22uescape\x22,\x22uni\
on\x22,\x22unique\x22,\x22un\
known\x22,\x22unnest\x22,\
\x22update\x22,\x22upper\x22\
,\x22user\x22,\x22using\x22,\
\x22value\x22,\x22values\x22\
,\x22value_of\x22,\x22var\
_pop\x22,\x22var_samp\x22\
,\x22varbinary\x22,\x22va\
rchar\x22,\x22varying\x22\
,\x22versioning\x22,\x22w\
hen\x22,\x22whenever\x22,\
\x22where\x22,\x22width_b\
ucket\x22,\x22window\x22,\
\x22with\x22,\x22within\x22,\
\x22without\x22,\x22year\x22\
,\x22add\x22,\x22asc\x22,\x22co\
llation\x22,\x22desc\x22,\
\x22final\x22,\x22first\x22,\
\x22last\x22,\x22view\x22].f\
ilter((e=>!i.inc\
ludes(e))),l={\x0ab\
egin:r.concat(/\x5c\
b/,r.either(...o\
),/\x5cs*\x5c(/),relev\
ance:0,keywords:\
{built_in:o}}\x0a;r\
eturn{name:\x22SQL\x22\
,case_insensitiv\
e:!0,illegal:/[{\
}]|<\x5c//,keywords\
:{\x0a$pattern:/\x5cb[\
\x5cw\x5c.]+/,keyword:\
((e,{exceptions:\
r,when:t}={})=>{\
const n=t\x0a;retur\
n r=r||[],e.map(\
(e=>e.match(/\x5c|\x5c\
d+$/)||r.include\
s(e)?e:n(e)?e+\x22|\
0\x22:e))\x0a})(c,{whe\
n:e=>e.length<3}\
),literal:n,type\
:a,\x0abuilt_in:[\x22c\
urrent_catalog\x22,\
\x22current_date\x22,\x22\
current_default_\
transform_group\x22\
,\x22current_path\x22,\
\x22current_role\x22,\x22\
current_schema\x22,\
\x22current_transfo\
rm_group_for_typ\
e\x22,\x22current_user\
\x22,\x22session_user\x22\
,\x22system_time\x22,\x22\
system_user\x22,\x22cu\
rrent_time\x22,\x22loc\
altime\x22,\x22current\
_timestamp\x22,\x22loc\
altimestamp\x22]\x0a},\
contains:[{begin\
:r.either(...s),\
relevance:0,keyw\
ords:{$pattern:/\
[\x5cw\x5c.]+/,\x0akeywor\
d:c.concat(s),li\
teral:n,type:a}}\
,{className:\x22typ\
e\x22,\x0abegin:r.eith\
er(\x22double preci\
sion\x22,\x22large obj\
ect\x22,\x22with timez\
one\x22,\x22without ti\
mezone\x22)\x0a},l,{cl\
assName:\x22variabl\
e\x22,begin:/@[a-z0\
-9][a-z0-9_]*/},\
{className:\x22stri\
ng\x22,\x0avariants:[{\
begin:/'/,end:/'\
/,contains:[{beg\
in:/''/}]}]},{be\
gin:/\x22/,end:/\x22/,\
\x0acontains:[{begi\
n:/\x22\x22/}]},e.C_NU\
MBER_MODE,e.C_BL\
OCK_COMMENT_MODE\
,t,{\x0aclassName:\x22\
operator\x22,begin:\
/[-+*/=%^~]|&&?|\
\x5c|\x5c|?|!=?|<(?:=>\
?|<|>)?|>[>=]?/,\
\x0arelevance:0}]}}\
})();hljs.regist\
erLanguage(\x22sql\x22\
,e)})();/*! `sta\
n` grammar compi\
led for Highligh\
t.js 11.10.0 */\x0a\
(()=>{var e=(()=\
>{\x22use strict\x22;r\
eturn e=>{\x0aconst\
 _=e.regex,o=[\x22b\
ernoulli\x22,\x22berno\
ulli_logit\x22,\x22ber\
noulli_logit_glm\
\x22,\x22beta\x22,\x22beta_b\
inomial\x22,\x22beta_p\
roportion\x22,\x22bino\
mial\x22,\x22binomial_\
logit\x22,\x22categori\
cal\x22,\x22categorica\
l_logit\x22,\x22catego\
rical_logit_glm\x22\
,\x22cauchy\x22,\x22chi_s\
quare\x22,\x22dirichle\
t\x22,\x22discrete_ran\
ge\x22,\x22double_expo\
nential\x22,\x22exp_mo\
d_normal\x22,\x22expon\
ential\x22,\x22frechet\
\x22,\x22gamma\x22,\x22gauss\
ian_dlm_obs\x22,\x22gu\
mbel\x22,\x22hmm_laten\
t\x22,\x22hypergeometr\
ic\x22,\x22inv_chi_squ\
are\x22,\x22inv_gamma\x22\
,\x22inv_wishart\x22,\x22\
inv_wishart_chol\
esky\x22,\x22lkj_corr\x22\
,\x22lkj_corr_chole\
sky\x22,\x22logistic\x22,\
\x22loglogistic\x22,\x22l\
ognormal\x22,\x22multi\
_gp\x22,\x22multi_gp_c\
holesky\x22,\x22multin\
omial\x22,\x22multinom\
ial_logit\x22,\x22mult\
i_normal\x22,\x22multi\
_normal_cholesky\
\x22,\x22multi_normal_\
prec\x22,\x22multi_stu\
dent_cholesky_t\x22\
,\x22multi_student_\
t\x22,\x22multi_studen\
t_t_cholesky\x22,\x22n\
eg_binomial\x22,\x22ne\
g_binomial_2\x22,\x22n\
eg_binomial_2_lo\
g\x22,\x22neg_binomial\
_2_log_glm\x22,\x22nor\
mal\x22,\x22normal_id_\
glm\x22,\x22ordered_lo\
gistic\x22,\x22ordered\
_logistic_glm\x22,\x22\
ordered_probit\x22,\
\x22pareto\x22,\x22pareto\
_type_2\x22,\x22poisso\
n\x22,\x22poisson_log\x22\
,\x22poisson_log_gl\
m\x22,\x22rayleigh\x22,\x22s\
caled_inv_chi_sq\
uare\x22,\x22skew_doub\
le_exponential\x22,\
\x22skew_normal\x22,\x22s\
td_normal\x22,\x22std_\
normal_log\x22,\x22stu\
dent_t\x22,\x22uniform\
\x22,\x22von_mises\x22,\x22w\
eibull\x22,\x22wiener\x22\
,\x22wishart\x22,\x22wish\
art_cholesky\x22],t\
=e.COMMENT(/\x5c/\x5c*\
/,/\x5c*\x5c//,{\x0arelev\
ance:0,contains:\
[{scope:\x22doctag\x22\
,match:/@(return\
|param)/}]}),r={\
\x0ascope:\x22meta\x22,be\
gin:/#include\x5cb/\
,end:/$/,contain\
s:[{match:/[a-z]\
[a-z-._]+/,\x0ascop\
e:\x22string\x22},e.C_\
LINE_COMMENT_MOD\
E]\x0a},i=[\x22lower\x22,\
\x22upper\x22,\x22offset\x22\
,\x22multiplier\x22];r\
eturn{name:\x22Stan\
\x22,\x0aaliases:[\x22sta\
nfuncs\x22],keyword\
s:{$pattern:e.ID\
ENT_RE,\x0atitle:[\x22\
functions\x22,\x22mode\
l\x22,\x22data\x22,\x22param\
eters\x22,\x22quantiti\
es\x22,\x22transformed\
\x22,\x22generated\x22],\x0a\
type:[\x22array\x22,\x22t\
uple\x22,\x22complex\x22,\
\x22int\x22,\x22real\x22,\x22ve\
ctor\x22,\x22complex_v\
ector\x22,\x22ordered\x22\
,\x22positive_order\
ed\x22,\x22simplex\x22,\x22u\
nit_vector\x22,\x22row\
_vector\x22,\x22comple\
x_row_vector\x22,\x22m\
atrix\x22,\x22complex_\
matrix\x22,\x22cholesk\
y_factor_corr|10\
\x22,\x22cholesky_fact\
or_cov|10\x22,\x22corr\
_matrix|10\x22,\x22cov\
_matrix|10\x22,\x22voi\
d\x22],\x0akeyword:[\x22f\
or\x22,\x22in\x22,\x22if\x22,\x22e\
lse\x22,\x22while\x22,\x22br\
eak\x22,\x22continue\x22,\
\x22return\x22],\x0abuilt\
_in:[\x22abs\x22,\x22acos\
\x22,\x22acosh\x22,\x22add_d\
iag\x22,\x22algebra_so\
lver\x22,\x22algebra_s\
olver_newton\x22,\x22a\
ppend_array\x22,\x22ap\
pend_col\x22,\x22appen\
d_row\x22,\x22asin\x22,\x22a\
sinh\x22,\x22atan\x22,\x22at\
an2\x22,\x22atanh\x22,\x22be\
ssel_first_kind\x22\
,\x22bessel_second_\
kind\x22,\x22binary_lo\
g_loss\x22,\x22block\x22,\
\x22cbrt\x22,\x22ceil\x22,\x22c\
hol2inv\x22,\x22choles\
ky_decompose\x22,\x22c\
hoose\x22,\x22col\x22,\x22co\
ls\x22,\x22columns_dot\
_product\x22,\x22colum\
ns_dot_self\x22,\x22co\
mplex_schur_deco\
mpose\x22,\x22complex_\
schur_decompose_\
t\x22,\x22complex_schu\
r_decompose_u\x22,\x22\
conj\x22,\x22cos\x22,\x22cos\
h\x22,\x22cov_exp_quad\
\x22,\x22crossprod\x22,\x22c\
sr_extract\x22,\x22csr\
_extract_u\x22,\x22csr\
_extract_v\x22,\x22csr\
_extract_w\x22,\x22csr\
_matrix_times_ve\
ctor\x22,\x22csr_to_de\
nse_matrix\x22,\x22cum\
ulative_sum\x22,\x22da\
e\x22,\x22dae_tol\x22,\x22de\
terminant\x22,\x22diag\
_matrix\x22,\x22diagon\
al\x22,\x22diag_post_m\
ultiply\x22,\x22diag_p\
re_multiply\x22,\x22di\
gamma\x22,\x22dims\x22,\x22d\
istance\x22,\x22dot_pr\
oduct\x22,\x22dot_self\
\x22,\x22eigendecompos\
e\x22,\x22eigendecompo\
se_sym\x22,\x22eigenva\
lues\x22,\x22eigenvalu\
es_sym\x22,\x22eigenve\
ctors\x22,\x22eigenvec\
tors_sym\x22,\x22erf\x22,\
\x22erfc\x22,\x22exp\x22,\x22ex\
p2\x22,\x22expm1\x22,\x22fal\
ling_factorial\x22,\
\x22fdim\x22,\x22fft\x22,\x22ff\
t2\x22,\x22floor\x22,\x22fma\
\x22,\x22fmax\x22,\x22fmin\x22,\
\x22fmod\x22,\x22gamma_p\x22\
,\x22gamma_q\x22,\x22gene\
ralized_inverse\x22\
,\x22get_imag\x22,\x22get\
_real\x22,\x22head\x22,\x22h\
mm_hidden_state_\
prob\x22,\x22hmm_margi\
nal\x22,\x22hypot\x22,\x22id\
entity_matrix\x22,\x22\
inc_beta\x22,\x22integ\
rate_1d\x22,\x22integr\
ate_ode\x22,\x22integr\
ate_ode_adams\x22,\x22\
integrate_ode_bd\
f\x22,\x22integrate_od\
e_rk45\x22,\x22int_ste\
p\x22,\x22inv\x22,\x22inv_cl\
oglog\x22,\x22inv_erfc\
\x22,\x22inverse\x22,\x22inv\
erse_spd\x22,\x22inv_f\
ft\x22,\x22inv_fft2\x22,\x22\
inv_inc_beta\x22,\x22i\
nv_logit\x22,\x22inv_P\
hi\x22,\x22inv_sqrt\x22,\x22\
inv_square\x22,\x22is_\
inf\x22,\x22is_nan\x22,\x22l\
ambert_w0\x22,\x22lamb\
ert_wm1\x22,\x22lbeta\x22\
,\x22lchoose\x22,\x22ldex\
p\x22,\x22lgamma\x22,\x22lin\
spaced_array\x22,\x22l\
inspaced_int_arr\
ay\x22,\x22linspaced_r\
ow_vector\x22,\x22lins\
paced_vector\x22,\x22l\
mgamma\x22,\x22lmultip\
ly\x22,\x22log\x22,\x22log1m\
\x22,\x22log1m_exp\x22,\x22l\
og1m_inv_logit\x22,\
\x22log1p\x22,\x22log1p_e\
xp\x22,\x22log_determi\
nant\x22,\x22log_diff_\
exp\x22,\x22log_fallin\
g_factorial\x22,\x22lo\
g_inv_logit\x22,\x22lo\
g_inv_logit_diff\
\x22,\x22logit\x22,\x22log_m\
ix\x22,\x22log_modifie\
d_bessel_first_k\
ind\x22,\x22log_rising\
_factorial\x22,\x22log\
_softmax\x22,\x22log_s\
um_exp\x22,\x22machine\
_precision\x22,\x22map\
_rect\x22,\x22matrix_e\
xp\x22,\x22matrix_exp_\
multiply\x22,\x22matri\
x_power\x22,\x22max\x22,\x22\
mdivide_left_spd\
\x22,\x22mdivide_left_\
tri_low\x22,\x22mdivid\
e_right_spd\x22,\x22md\
ivide_right_tri_\
low\x22,\x22mean\x22,\x22min\
\x22,\x22modified_bess\
el_first_kind\x22,\x22\
modified_bessel_\
second_kind\x22,\x22mu\
ltiply_lower_tri\
_self_transpose\x22\
,\x22negative_infin\
ity\x22,\x22norm\x22,\x22nor\
m1\x22,\x22norm2\x22,\x22not\
_a_number\x22,\x22num_\
elements\x22,\x22ode_a\
dams\x22,\x22ode_adams\
_tol\x22,\x22ode_adjoi\
nt_tol_ctl\x22,\x22ode\
_bdf\x22,\x22ode_bdf_t\
ol\x22,\x22ode_ckrk\x22,\x22\
ode_ckrk_tol\x22,\x22o\
de_rk45\x22,\x22ode_rk\
45_tol\x22,\x22one_hot\
_array\x22,\x22one_hot\
_int_array\x22,\x22one\
_hot_row_vector\x22\
,\x22one_hot_vector\
\x22,\x22ones_array\x22,\x22\
ones_int_array\x22,\
\x22ones_row_vector\
\x22,\x22ones_vector\x22,\
\x22owens_t\x22,\x22Phi\x22,\
\x22Phi_approx\x22,\x22po\
lar\x22,\x22positive_i\
nfinity\x22,\x22pow\x22,\x22\
print\x22,\x22prod\x22,\x22p\
roj\x22,\x22qr\x22,\x22qr_Q\x22\
,\x22qr_R\x22,\x22qr_thin\
\x22,\x22qr_thin_Q\x22,\x22q\
r_thin_R\x22,\x22quad_\
form\x22,\x22quad_form\
_diag\x22,\x22quad_for\
m_sym\x22,\x22quantile\
\x22,\x22rank\x22,\x22reduce\
_sum\x22,\x22reject\x22,\x22\
rep_array\x22,\x22rep_\
matrix\x22,\x22rep_row\
_vector\x22,\x22rep_ve\
ctor\x22,\x22reverse\x22,\
\x22rising_factoria\
l\x22,\x22round\x22,\x22row\x22\
,\x22rows\x22,\x22rows_do\
t_product\x22,\x22rows\
_dot_self\x22,\x22scal\
e_matrix_exp_mul\
tiply\x22,\x22sd\x22,\x22seg\
ment\x22,\x22sin\x22,\x22sin\
gular_values\x22,\x22s\
inh\x22,\x22size\x22,\x22sof\
tmax\x22,\x22sort_asc\x22\
,\x22sort_desc\x22,\x22so\
rt_indices_asc\x22,\
\x22sort_indices_de\
sc\x22,\x22sqrt\x22,\x22squa\
re\x22,\x22squared_dis\
tance\x22,\x22step\x22,\x22s\
ub_col\x22,\x22sub_row\
\x22,\x22sum\x22,\x22svd\x22,\x22s\
vd_U\x22,\x22svd_V\x22,\x22s\
ymmetrize_from_l\
ower_tri\x22,\x22tail\x22\
,\x22tan\x22,\x22tanh\x22,\x22t\
arget\x22,\x22tcrosspr\
od\x22,\x22tgamma\x22,\x22to\
_array_1d\x22,\x22to_a\
rray_2d\x22,\x22to_com\
plex\x22,\x22to_int\x22,\x22\
to_matrix\x22,\x22to_r\
ow_vector\x22,\x22to_v\
ector\x22,\x22trace\x22,\x22\
trace_gen_quad_f\
orm\x22,\x22trace_quad\
_form\x22,\x22trigamma\
\x22,\x22trunc\x22,\x22unifo\
rm_simplex\x22,\x22var\
iance\x22,\x22zeros_ar\
ray\x22,\x22zeros_int_\
array\x22,\x22zeros_ro\
w_vector\x22]\x0a},con\
tains:[e.C_LINE_\
COMMENT_MODE,r,e\
.HASH_COMMENT_MO\
DE,t,{scope:\x22bui\
lt_in\x22,\x0amatch:/\x5c\
s(pi|e|sqrt2|log\
2|log10)(?=\x5c()/,\
relevance:0},{\x0am\
atch:_.concat(/[\
<,]\x5cs*/,_.either\
(...i),/\x5cs*=/),k\
eywords:i},{scop\
e:\x22keyword\x22,\x0amat\
ch:/\x5cbtarget(?=\x5c\
s*\x5c+=)/},{\x0amatch\
:[/~\x5cs*/,_.eithe\
r(...o),/(?:\x5c(\x5c)\
)/,/\x5cs*T(?=\x5cs*\x5c[\
)/],scope:{2:\x22bu\
ilt_in\x22,\x0a4:\x22keyw\
ord\x22}},{scope:\x22b\
uilt_in\x22,keyword\
s:o,\x0abegin:_.con\
cat(/\x5cw*/,_.eith\
er(...o),/(_lpdf\
|_lupdf|_lpmf|_c\
df|_lcdf|_lccdf|\
_qf)(?=\x5cs*[\x5c(.*\x5c\
)])/)\x0a},{begin:[\
/~/,/\x5cs*/,_.conc\
at(_.either(...o\
),/(?=\x5cs*[\x5c(.*\x5c)\
])/)],scope:{\x0a3:\
\x22built_in\x22}},{\x0ab\
egin:[/~/,/\x5cs*\x5cw\
+(?=\x5cs*[\x5c(.*\x5c)])\
/,\x22(?!.*/\x5cb(\x22+_.\
either(...o)+\x22)\x5c\
b)\x22],scope:{\x0a2:\x22\
title.function\x22}\
},{scope:\x22title.\
function\x22,\x0abegin\
:/\x5cw*(_lpdf|_lup\
df|_lpmf|_cdf|_l\
cdf|_lccdf|_qf)(\
?=\x5cs*[\x5c(.*\x5c)])/}\
,{\x0ascope:\x22number\
\x22,\x0amatch:_.conca\
t(/(?:\x5cb\x5cd+(?:_\x5c\
d+)*(?:\x5c.(?:\x5cd+(\
?:_\x5cd+)*)?)?|\x5cB\x5c\
.\x5cd+(?:_\x5cd+)*)/,\
/(?:[eE][+-]?\x5cd+\
(?:_\x5cd+)*)?i?(?!\
\x5cw)/),\x0arelevance\
:0},{scope:\x22stri\
ng\x22,begin:/\x22/,en\
d:/\x22/}]}}})()\x0a;h\
ljs.registerLang\
uage(\x22stan\x22,e)})\
();/*! `stata` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>({name:\x22Sta\
ta\x22,aliases:[\x22do\
\x22,\x22ado\x22],\x0acase_i\
nsensitive:!0,\x0ak\
eywords:\x22if else\
 in foreach for \
forv forva forva\
l forvalu forval\
ue forvalues by \
bys bysort xi qu\
ietly qui captur\
e about ac ac_7 \
acprplot acprplo\
t_7 adjust ado a\
dopath adoupdate\
 alpha ameans an\
 ano anov anova \
anova_estat anov\
a_terms anovadef\
 aorder ap app a\
ppe appen append\
 arch arch_dr ar\
ch_estat arch_p \
archlm areg areg\
_p args arima ar\
ima_dr arima_est\
at arima_p as as\
mprobit asmprobi\
t_estat asmprobi\
t_lf asmprobit_m\
fx__dlg asmprobi\
t_p ass asse ass\
er assert avplot\
 avplot_7 avplot\
s avplots_7 bcsk\
ew0 bgodfrey bia\
s binreg bip0_lf\
 biplot bipp_lf \
bipr_lf bipr_p b\
iprobit bitest b\
itesti bitowt bl\
ogit bmemsize bo\
ot bootsamp boot\
strap bootstrap_\
8 boxco_l boxco_\
p boxcox boxcox_\
6 boxcox_p bprob\
it br break brie\
r bro brow brows\
 browse brr brrs\
tat bs bs_7 bsam\
pl_w bsample bsa\
mple_7 bsqreg bs\
tat bstat_7 bsta\
t_8 bstrap bstra\
p_7 bubble bubbl\
eplot ca ca_esta\
t ca_p cabiplot \
camat canon cano\
n_8 canon_8_p ca\
non_estat canon_\
p cap caprojecti\
on capt captu ca\
ptur capture cat\
 cc cchart cchar\
t_7 cci cd censo\
bs_table centile\
 cf char chdir c\
heckdlgfiles che\
ckestimationsamp\
le checkhlpfiles\
 checksum chelp \
ci cii cl class \
classutil clear \
cli clis clist c\
lo clog clog_lf \
clog_p clogi clo\
gi_sw clogit clo\
git_lf clogit_p \
clogitp clogl_sw\
 cloglog cloneva\
r clslistarray c\
luster cluster_m\
easures cluster_\
stop cluster_tre\
e cluster_tree_8\
 clustermat cmdl\
og cnr cnre cnre\
g cnreg_p cnreg_\
sw cnsreg codebo\
ok collaps4 coll\
apse colormult_n\
b colormult_nw c\
ompare compress \
conf confi confi\
r confirm conren\
 cons const cons\
tr constra const\
rai constrain co\
nstraint continu\
e contract copy \
copyright copyso\
urce cor corc co\
rr corr2data cor\
r_anti corr_kmo \
corr_smc corre c\
orrel correla co\
rrelat correlate\
 corrgram cou co\
un count cox cox\
_p cox_sw coxbas\
e coxhaz coxvar \
cprplot cprplot_\
7 crc cret cretu\
 cretur creturn \
cross cs cscript\
 cscript_log csi\
 ct ct_is ctset \
ctst_5 ctst_st c\
ttost cumsp cums\
p_7 cumul cusum \
cusum_7 cutil d|\
0 datasig datasi\
gn datasigna dat\
asignat datasign\
atu datasignatur\
 datasignature d\
atetof db dbeta \
de dec deco deco\
d decode deff de\
s desc descr des\
cri describ desc\
ribe destring df\
beta dfgls dfull\
er di di_g dir d\
irstats dis disc\
ard disp disp_re\
s disp_s displ d\
ispla display di\
stinct do doe do\
ed doedi doedit \
dotplot dotplot_\
7 dprobit drawno\
rm drop ds ds_ut\
il dstdize dupli\
cates durbina dw\
stat dydx e|0 ed\
 edi edit egen e\
ivreg emdef en e\
nc enco encod en\
code eq erase er\
eg ereg_lf ereg_\
p ereg_sw ereghe\
t ereghet_glf er\
eghet_glf_sh ere\
ghet_gp ereghet_\
ilf ereghet_ilf_\
sh ereghet_ip er\
et eretu eretur \
ereturn err erro\
 error esize est\
 est_cfexist est\
_cfname est_clic\
kable est_expand\
 est_hold est_ta\
ble est_unhold e\
st_unholdok esta\
t estat_default \
estat_summ estat\
_vce_only esti e\
stimates etodow \
etof etomdy ex e\
xi exit expand e\
xpandcl fac fact\
 facto factor fa\
ctor_estat facto\
r_p factor_pca_r\
otated factor_ro\
tate factormat f\
cast fcast_compu\
te fcast_graph f\
dades fdadesc fd\
adescr fdadescri\
 fdadescrib fdad\
escribe fdasav f\
dasave fdause fh\
_st file open fi\
le read file clo\
se file filefilt\
er fillin find_h\
lp_file findfile\
 findit findit_7\
 fit fl fli flis\
 flist for5_0 fo\
rest forestplot \
form forma forma\
t fpredict frac_\
154 frac_adj fra\
c_chk frac_cox f\
rac_ddp frac_dis\
 frac_dv frac_in\
 frac_mun frac_p\
p frac_pq frac_p\
v frac_wgt frac_\
xo fracgen fracp\
lot fracplot_7 f\
racpoly fracpred\
 fron_ex fron_hn\
 fron_p fron_tn \
fron_tn2 frontie\
r ftodate ftoe f\
tomdy ftowdate f\
unnel funnelplot\
 g|0 gamhet_glf \
gamhet_gp gamhet\
_ilf gamhet_ip g\
amma gamma_d2 ga\
mma_p gamma_sw g\
ammahet gdi_hexa\
gon gdi_spokes g\
e gen gene gener\
 genera generat \
generate genrank\
 genstd genvmean\
 gettoken gl gla\
dder gladder_7 g\
lim_l01 glim_l02\
 glim_l03 glim_l\
04 glim_l05 glim\
_l06 glim_l07 gl\
im_l08 glim_l09 \
glim_l10 glim_l1\
1 glim_l12 glim_\
lf glim_mu glim_\
nw1 glim_nw2 gli\
m_nw3 glim_p gli\
m_v1 glim_v2 gli\
m_v3 glim_v4 gli\
m_v5 glim_v6 gli\
m_v7 glm glm_6 g\
lm_p glm_sw glmp\
red glo glob glo\
ba global glogit\
 glogit_8 glogit\
_p gmeans gnbre_\
lf gnbreg gnbreg\
_5 gnbreg_p gomp\
_lf gompe_sw gom\
per_p gompertz g\
ompertzhet gomph\
et_glf gomphet_g\
lf_sh gomphet_gp\
 gomphet_ilf gom\
phet_ilf_sh gomp\
het_ip gphdot gp\
hpen gphprint gp\
refs gprobi_p gp\
robit gprobit_8 \
gr gr7 gr_copy g\
r_current gr_db \
gr_describe gr_d\
ir gr_draw gr_dr\
aw_replay gr_dro\
p gr_edit gr_edi\
tviewopts gr_exa\
mple gr_example2\
 gr_export gr_pr\
int gr_qscheme g\
r_query gr_read \
gr_rename gr_rep\
lay gr_save gr_s\
et gr_setscheme \
gr_table gr_undo\
 gr_use graph gr\
aph7 grebar grei\
gen greigen_7 gr\
eigen_8 grmeanby\
 grmeanby_7 gs_f\
ileinfo gs_filet\
ype gs_graphinfo\
 gs_stat gsort g\
wood h|0 hadimvo\
 hareg hausman h\
aver he heck_d2 \
heckma_p heckman\
 heckp_lf heckpr\
_p heckprob hel \
help hereg hetpr\
_lf hetpr_p hetp\
rob hettest hexd\
ump hilite hist \
hist_7 histogram\
 hlogit hlu hmea\
ns hotel hotelli\
ng hprobit hreg \
hsearch icd9 icd\
9_ff icd9p iis i\
mpute imtest inb\
ase include inf \
infi infil infil\
e infix inp inpu\
 input ins inshe\
et insp inspe in\
spec inspect int\
eg inten intreg \
intreg_7 intreg_\
p intrg2_ll intr\
g_ll intrg_ll2 i\
polate iqreg ir \
irf irf_create i\
rfm iri is_svy i\
s_svysum isid is\
tdize ivprob_1_l\
f ivprob_lf ivpr\
obit ivprobit_p \
ivreg ivreg_foot\
note ivtob_1_lf \
ivtob_lf ivtobit\
 ivtobit_p jackk\
nife jacknife jk\
nife jknife_6 jk\
nife_8 jkstat jo\
inby kalarma1 ka\
p kap_3 kapmeier\
 kappa kapwgt kd\
ensity kdensity_\
7 keep ksm ksmir\
nov ktau kwallis\
 l|0 la lab labb\
e labbeplot labe\
 label labelbook\
 ladder levels l\
evelsof leverage\
 lfit lfit_p li \
lincom line link\
test lis list ll\
oghet_glf lloghe\
t_glf_sh lloghet\
_gp lloghet_ilf \
lloghet_ilf_sh l\
loghet_ip llogi_\
sw llogis_p llog\
ist llogistic ll\
ogistichet lnorm\
_lf lnorm_sw lno\
rma_p lnormal ln\
ormalhet lnormhe\
t_glf lnormhet_g\
lf_sh lnormhet_g\
p lnormhet_ilf l\
normhet_ilf_sh l\
normhet_ip lnske\
w0 loadingplot l\
oc loca local lo\
g logi logis_lf \
logistic logisti\
c_p logit logit_\
estat logit_p lo\
glogs logrank lo\
neway lookfor lo\
okup lowess lowe\
ss_7 lpredict lr\
ecomp lroc lroc_\
7 lrtest ls lsen\
s lsens_7 lsens_\
x lstat ltable l\
table_7 ltriang \
lv lvr2plot lvr2\
plot_7 m|0 ma ma\
c macr macro mak\
ecns man manova \
manova_estat man\
ova_p manovatest\
 mantel mark mar\
kin markout mark\
sample mat mat_c\
app mat_order ma\
t_put_rr mat_rap\
p mata mata_clea\
r mata_describe \
mata_drop mata_m\
atdescribe mata_\
matsave mata_mat\
use mata_memory \
mata_mlib mata_m\
osave mata_renam\
e mata_which mat\
alabel matcproc \
matlist matname \
matr matri matri\
x matrix_input__\
dlg matstrik mcc\
 mcci md0_ md1_ \
md1debug_ md2_ m\
d2debug_ mds mds\
_estat mds_p mds\
config mdslong m\
dsmat mdsshepard\
 mdytoe mdytof m\
e_derd mean mean\
s median memory \
memsize menl meq\
parse mer merg m\
erge meta mfp mf\
x mhelp mhodds m\
inbound mixed_ll\
 mixed_ll_reparm\
 mkassert mkdir \
mkmat mkspline m\
l ml_5 ml_adjs m\
l_bhhhs ml_c_d m\
l_check ml_clear\
 ml_cnt ml_debug\
 ml_defd ml_e0 m\
l_e0_bfgs ml_e0_\
cycle ml_e0_dfp \
ml_e0i ml_e1 ml_\
e1_bfgs ml_e1_bh\
hh ml_e1_cycle m\
l_e1_dfp ml_e2 m\
l_e2_cycle ml_eb\
fg0 ml_ebfr0 ml_\
ebfr1 ml_ebh0q m\
l_ebhh0 ml_ebhr0\
 ml_ebr0i ml_ecr\
0i ml_edfp0 ml_e\
dfr0 ml_edfr1 ml\
_edr0i ml_eds ml\
_eer0i ml_egr0i \
ml_elf ml_elf_bf\
gs ml_elf_bhhh m\
l_elf_cycle ml_e\
lf_dfp ml_elfi m\
l_elfs ml_enr0i \
ml_enrr0 ml_erdu\
0 ml_erdu0_bfgs \
ml_erdu0_bhhh ml\
_erdu0_bhhhq ml_\
erdu0_cycle ml_e\
rdu0_dfp ml_erdu\
0_nrbfgs ml_exde\
 ml_footnote ml_\
geqnr ml_grad0 m\
l_graph ml_hbhhh\
 ml_hd0 ml_hold \
ml_init ml_inv m\
l_log ml_max ml_\
mlout ml_mlout_8\
 ml_model ml_nb0\
 ml_opt ml_p ml_\
plot ml_query ml\
_rdgrd ml_repor \
ml_s_e ml_score \
ml_searc ml_tech\
nique ml_unhold \
mleval mlf_ mlma\
tbysum mlmatsum \
mlog mlogi mlogi\
t mlogit_footnot\
e mlogit_p mlopt\
s mlsum mlvecsum\
 mnl0_ mor more \
mov move mprobit\
 mprobit_lf mpro\
bit_p mrdu0_ mrd\
u1_ mvdecode mve\
ncode mvreg mvre\
g_estat n|0 nbre\
g nbreg_al nbreg\
_lf nbreg_p nbre\
g_sw nestreg net\
 newey newey_7 n\
ewey_p news nl n\
l_7 nl_9 nl_9_p \
nl_p nl_p_7 nlco\
m nlcom_p nlexp2\
 nlexp2_7 nlexp2\
a nlexp2a_7 nlex\
p3 nlexp3_7 nlgo\
m3 nlgom3_7 nlgo\
m4 nlgom4_7 nlin\
it nllog3 nllog3\
_7 nllog4 nllog4\
_7 nlog_rd nlogi\
t nlogit_p nlogi\
tgen nlogittree \
nlpred no nobrea\
k noi nois noisi\
 noisil noisily \
note notes notes\
_dlg nptrend num\
label numlist od\
bc old_ver olo o\
log ologi ologi_\
sw ologit ologit\
_p ologitp on on\
e onew onewa one\
way op_colnm op_\
comp op_diff op_\
inv op_str opr o\
pro oprob oprob_\
sw oprobi oprobi\
_p oprobit oprob\
itp opts_exclusi\
ve order orthog \
orthpoly ou out \
outf outfi outfi\
l outfile outs o\
utsh outshe outs\
hee outsheet ovt\
est pac pac_7 pa\
lette parse pars\
e_dissim pause p\
ca pca_8 pca_dis\
play pca_estat p\
ca_p pca_rotate \
pcamat pchart pc\
hart_7 pchi pchi\
_7 pcorr pctile \
pentium pergram \
pergram_7 permut\
e permute_8 pers\
onal peto_st pkc\
ollapse pkcross \
pkequiv pkexamin\
e pkexamine_7 pk\
shape pksumm pks\
umm_7 pl plo plo\
t plugin pnorm p\
norm_7 poisgof p\
oiss_lf poiss_sw\
 poisso_p poisso\
n poisson_estat \
post postclose p\
ostfile postutil\
 pperron pr prai\
s prais_e prais_\
e2 prais_p predi\
ct predictnl pre\
serve print pro \
prob probi probi\
t probit_estat p\
robit_p proc_tim\
e procoverlay pr\
ocrustes procrus\
tes_estat procru\
stes_p profiler \
prog progr progr\
a program prop p\
roportion prtest\
 prtesti pwcorr \
pwd q\x5c\x5cs qby qby\
s qchi qchi_7 ql\
adder qladder_7 \
qnorm qnorm_7 qq\
plot qqplot_7 qr\
eg qreg_c qreg_p\
 qreg_sw qu quad\
chk quantile qua\
ntile_7 que quer\
 query range ran\
ksum ratio rchar\
t rchart_7 rcof \
recast reclink r\
ecode reg reg3 r\
eg3_p regdw regr\
 regre regre_p2 \
regres regres_p \
regress regress_\
estat regriv_p r\
emap ren rena re\
nam rename renpf\
ix repeat replac\
e report reshape\
 restore ret ret\
u retur return r\
m rmdir robvar r\
occomp roccomp_7\
 roccomp_8 rocf_\
lf rocfit rocfit\
_8 rocgold rocpl\
ot rocplot_7 roc\
tab roctab_7 rol\
ling rologit rol\
ogit_p rot rota \
rotat rotate rot\
atemat rreg rreg\
_p ru run runtes\
t rvfplot rvfplo\
t_7 rvpplot rvpp\
lot_7 sa safesum\
 sample sampsi s\
av save savedres\
ults saveold sc \
sca scal scala s\
calar scatter sc\
m_mine sco scob_\
lf scob_p scobi_\
sw scobit scor s\
core scoreplot s\
coreplot_help sc\
ree screeplot sc\
reeplot_help sdt\
est sdtesti se s\
earch separate s\
eperate serrbar \
serrbar_7 serset\
 set set_default\
s sfrancia sh sh\
e shel shell she\
whart shewhart_7\
 signestimations\
ample signrank s\
igntest simul si\
mul_7 simulate s\
imulate_8 sktest\
 sleep slogit sl\
ogit_d2 slogit_p\
 smooth snapspan\
 so sor sort spe\
arman spikeplot \
spikeplot_7 spik\
eplt spline_x sp\
lit sqreg sqreg_\
p sret sretu sre\
tur sreturn ssc \
st st_ct st_hc s\
t_hcd st_hcd_sh \
st_is st_issys s\
t_note st_promo \
st_set st_show s\
t_smpl st_subid \
stack statsby st\
atsby_8 stbase s\
tci stci_7 stcox\
 stcox_estat stc\
ox_fr stcox_fr_l\
l stcox_p stcox_\
sw stcoxkm stcox\
km_7 stcstat stc\
urv stcurve stcu\
rve_7 stdes stem\
 stepwise stereg\
 stfill stgen st\
ir stjoin stmc s\
tmh stphplot stp\
hplot_7 stphtest\
 stphtest_7 stpt\
ime strate strat\
e_7 streg streg_\
sw streset sts s\
ts_7 stset stspl\
it stsum sttocc \
sttoct stvary st\
weib su suest su\
est_8 sum summ s\
umma summar summ\
ari summariz sum\
marize sunflower\
 sureg survcurv \
survsum svar sva\
r_p svmat svy sv\
y_disp svy_dreg \
svy_est svy_est_\
7 svy_estat svy_\
get svy_gnbreg_p\
 svy_head svy_he\
ader svy_heckman\
_p svy_heckprob_\
p svy_intreg_p s\
vy_ivreg_p svy_l\
ogistic_p svy_lo\
git_p svy_mlogit\
_p svy_nbreg_p s\
vy_ologit_p svy_\
oprobit_p svy_po\
isson_p svy_prob\
it_p svy_regress\
_p svy_sub svy_s\
ub_7 svy_x svy_x\
_7 svy_x_p svyde\
s svydes_8 svyge\
n svygnbreg svyh\
eckman svyheckpr\
ob svyintreg svy\
intreg_7 svyintr\
g svyivreg svylc\
 svylog_p svylog\
it svymarkout sv\
ymarkout_8 svyme\
an svymlog svyml\
ogit svynbreg sv\
yolog svyologit \
svyoprob svyopro\
bit svyopts svyp\
ois svypois_7 sv\
ypoisson svyprob\
it svyprobt svyp\
rop svyprop_7 sv\
yratio svyreg sv\
yreg_p svyregres\
s svyset svyset_\
7 svyset_8 svyta\
b svytab_7 svyte\
st svytotal sw s\
w_8 swcnreg swco\
x swereg swilk s\
wlogis swlogit s\
wologit swoprbt \
swpois swprobit \
swqreg swtobit s\
wweib symmetry s\
ymmi symplot sym\
plot_7 syntax sy\
sdescribe sysdir\
 sysuse szroeter\
 ta tab tab1 tab\
2 tab_or tabd ta\
bdi tabdis tabdi\
sp tabi table ta\
bodds tabodds_7 \
tabstat tabu tab\
ul tabula tabula\
t tabulate te te\
mpfile tempname \
tempvar tes test\
 testnl testparm\
 teststd tetrach\
oric time_it tim\
er tis tob tobi \
tobit tobit_p to\
bit_sw token tok\
eni tokeniz toke\
nize tostring to\
tal translate tr\
anslator transma\
p treat_ll treat\
r_p treatreg tri\
m trimfill trnb_\
cons trnb_mean t\
rpoiss_d2 trunc_\
ll truncr_p trun\
creg tsappend ts\
et tsfill tsline\
 tsline_ex tsrep\
ort tsrevar tsrl\
ine tsset tssmoo\
th tsunab ttest \
ttesti tut_chk t\
ut_wait tutorial\
 tw tware_st two\
 twoway twoway__\
fpfit_serset two\
way__function_ge\
n twoway__histog\
ram_gen twoway__\
ipoint_serset tw\
oway__ipoints_se\
rset twoway__kde\
nsity_gen twoway\
__lfit_serset tw\
oway__normgen_ge\
n twoway__pci_se\
rset twoway__qfi\
t_serset twoway_\
_scatteri_serset\
 twoway__sunflow\
er_gen twoway_ks\
m_serset ty typ \
type typeof u|0 \
unab unabbrev un\
abcmd update us \
use uselabel var\
 var_mkcompanion\
 var_p varbasic \
varfcast vargran\
ger varirf varir\
f_add varirf_cgr\
aph varirf_creat\
e varirf_ctable \
varirf_describe \
varirf_dir varir\
f_drop varirf_er\
ase varirf_graph\
 varirf_ograph v\
arirf_rename var\
irf_set varirf_t\
able varlist var\
lmar varnorm var\
soc varstable va\
rstable_w varsta\
ble_w2 varwle vc\
e vec vec_fevd v\
ec_mkphi vec_p v\
ec_p_w vecirf_cr\
eate veclmar vec\
lmar_w vecnorm v\
ecnorm_w vecrank\
 vecstable verin\
st vers versi ve\
rsio version vie\
w viewsource vif\
 vwls wdatetof w\
ebdescribe webse\
ek webuse weib1_\
lf weib2_lf weib\
_lf weib_lf0 wei\
bhet_glf weibhet\
_glf_sh weibhet_\
glfa weibhet_glf\
a_sh weibhet_gp \
weibhet_ilf weib\
het_ilf_sh weibh\
et_ilfa weibhet_\
ilfa_sh weibhet_\
ip weibu_sw weib\
ul_p weibull wei\
bull_c weibull_s\
 weibullhet wh w\
help whi which w\
hil while wilc_s\
t wilcoxon win w\
ind windo window\
 winexec wntestb\
 wntestb_7 wntes\
tq xchart xchart\
_7 xcorr xcorr_7\
 xi xi_6 xmlsav \
xmlsave xmluse x\
pose xsh xshe xs\
hel xshell xt_ii\
s xt_tis xtab_p \
xtabond xtbin_p \
xtclog xtcloglog\
 xtcloglog_8 xtc\
loglog_d2 xtclog\
log_pa_p xtclogl\
og_re_p xtcnt_p \
xtcorr xtdata xt\
des xtfront_p xt\
frontier xtgee x\
tgee_elink xtgee\
_estat xtgee_mak\
eivar xtgee_p xt\
gee_plink xtgls \
xtgls_p xthaus x\
thausman xtht_p \
xthtaylor xtile \
xtint_p xtintreg\
 xtintreg_8 xtin\
treg_d2 xtintreg\
_p xtivp_1 xtivp\
_2 xtivreg xtlin\
e xtline_ex xtlo\
git xtlogit_8 xt\
logit_d2 xtlogit\
_fe_p xtlogit_pa\
_p xtlogit_re_p \
xtmixed xtmixed_\
estat xtmixed_p \
xtnb_fe xtnb_lf \
xtnbreg xtnbreg_\
pa_p xtnbreg_ref\
e_p xtpcse xtpcs\
e_p xtpois xtpoi\
sson xtpoisson_d\
2 xtpoisson_pa_p\
 xtpoisson_refe_\
p xtpred xtprobi\
t xtprobit_8 xtp\
robit_d2 xtprobi\
t_re_p xtps_fe x\
tps_lf xtps_ren \
xtps_ren_8 xtrar\
_p xtrc xtrc_p x\
trchh xtrefe_p x\
treg xtreg_be xt\
reg_fe xtreg_ml \
xtreg_pa_p xtreg\
_re xtregar xtre\
re_p xtset xtsf_\
ll xtsf_llti xts\
um xttab xttest0\
 xttobit xttobit\
_8 xttobit_p xtt\
rans yx yxview__\
barlike_draw yxv\
iew_area_draw yx\
view_bar_draw yx\
view_dot_draw yx\
view_dropline_dr\
aw yxview_functi\
on_draw yxview_i\
arrow_draw yxvie\
w_ilabels_draw y\
xview_normal_dra\
w yxview_pcarrow\
_draw yxview_pcb\
arrow_draw yxvie\
w_pccapsym_draw \
yxview_pcscatter\
_draw yxview_pcs\
pike_draw yxview\
_rarea_draw yxvi\
ew_rbar_draw yxv\
iew_rbarm_draw y\
xview_rcap_draw \
yxview_rcapsym_d\
raw yxview_rconn\
ected_draw yxvie\
w_rline_draw yxv\
iew_rscatter_dra\
w yxview_rspike_\
draw yxview_spik\
e_draw yxview_su\
nflower_draw zap\
_s zinb zinb_llf\
 zinb_plf zip zi\
p_llf zip_p zip_\
plf zt_ct_5 zt_h\
c_5 zt_hcd_5 zt_\
is_5 zt_iss_5 zt\
_sho_5 zt_smp_5 \
ztbase_5 ztcox_5\
 ztdes_5 ztereg_\
5 ztfill_5 ztgen\
_5 ztir_5 ztjoin\
_5 ztnb ztnb_p z\
tp ztp_p zts_5 z\
tset_5 ztspli_5 \
ztsum_5 zttoct_5\
 ztvary_5 ztweib\
_5\x22,\x0acontains:[{\
className:\x22symbo\
l\x22,begin:/`[a-zA\
-Z0-9_]+'/},{cla\
ssName:\x22variable\
\x22,\x0abegin:/\x5c$\x5c{?[\
a-zA-Z0-9_]+\x5c}?/\
,relevance:0},{c\
lassName:\x22string\
\x22,variants:[{\x0abe\
gin:'`\x22[^\x5cr\x5cn]*?\
\x22\x5c''},{begin:'\x22[\
^\x5cr\x5cn\x22]*\x22'}]},{c\
lassName:\x22built_\
in\x22,\x0avariants:[{\
\x0abegin:\x22\x5c\x5cb(abs|\
acos|asin|atan|a\
tan2|atanh|ceil|\
cloglog|comb|cos\
|digamma|exp|flo\
or|invcloglog|in\
vlogit|ln|lnfact\
|lnfactorial|lng\
amma|log|log10|m\
ax|min|mod|reldi\
f|round|sign|sin\
|sqrt|sum|tan|ta\
nh|trigamma|trun\
c|betaden|Binomi\
al|binorm|binorm\
al|chi2|chi2tail\
|dgammapda|dgamm\
apdada|dgammapda\
dx|dgammapdx|dga\
mmapdxdx|F|Fden|\
Ftail|gammaden|g\
ammap|ibeta|invb\
inomial|invchi2|\
invchi2tail|invF\
|invFtail|invgam\
map|invibeta|inv\
nchi2|invnFtail|\
invnibeta|invnor\
m|invnormal|invt\
tail|nbetaden|nc\
hi2|nFden|nFtail\
|nibeta|norm|nor\
mal|normalden|no\
rmd|npnchi2|tden\
|ttail|uniform|a\
bbrev|char|index\
|indexnot|length\
|lower|ltrim|mat\
ch|plural|proper\
|real|regexm|reg\
exr|regexs|rever\
se|rtrim|string|\
strlen|strlower|\
strltrim|strmatc\
h|strofreal|strp\
os|strproper|str\
reverse|strrtrim\
|strtrim|struppe\
r|subinstr|subin\
word|substr|trim\
|upper|word|word\
count|_caller|au\
tocode|byteorder\
|chop|clip|cond|\
e|epsdouble|epsf\
loat|group|inlis\
t|inrange|irecod\
e|matrix|maxbyte\
|maxdouble|maxfl\
oat|maxint|maxlo\
ng|mi|minbyte|mi\
ndouble|minfloat\
|minint|minlong|\
missing|r|recode\
|replay|return|s\
|scalar|d|date|d\
ay|dow|doy|halfy\
ear|mdy|month|qu\
arter|week|year|\
d|daily|dofd|dof\
h|dofm|dofq|dofw\
|dofy|h|halfyear\
ly|hofd|m|mofd|m\
onthly|q|qofd|qu\
arterly|tin|twit\
hin|w|weekly|wof\
d|y|yearly|yh|ym\
|yofd|yq|yw|chol\
esky|colnumb|col\
sof|corr|det|dia\
g|diag0cnt|el|ge\
t|hadamard|I|inv\
|invsym|issym|is\
symmetric|J|matm\
issing|matunifor\
m|mreldif|nullma\
t|rownumb|rowsof\
|sweep|syminv|tr\
ace|vec|vecdiag)\
(?=\x5c\x5c()\x22\x0a}]},e.C\
OMMENT(\x22^[ \x5ct]*\x5c\
\x5c*.*$\x22,!1),e.C_L\
INE_COMMENT_MODE\
,e.C_BLOCK_COMME\
NT_MODE]\x0a})})();\
hljs.registerLan\
guage(\x22stata\x22,e)\
})();/*! `step21\
` grammar compil\
ed for Highlight\
.js 11.10.0 */\x0a(\
()=>{var e=(()=>\
{\x22use strict\x22;re\
turn e=>({name:\x22\
STEP Part 21\x22,\x0aa\
liases:[\x22p21\x22,\x22s\
tep\x22,\x22stp\x22],case\
_insensitive:!0,\
keywords:{\x0a$patt\
ern:\x22[A-Z_][A-Z0\
-9_.]*\x22,keyword:\
[\x22HEADER\x22,\x22ENDSE\
C\x22,\x22DATA\x22]},cont\
ains:[{\x0aclassNam\
e:\x22meta\x22,begin:\x22\
ISO-10303-21;\x22,r\
elevance:10},{cl\
assName:\x22meta\x22,\x0a\
begin:\x22END-ISO-1\
0303-21;\x22,releva\
nce:10\x0a},e.C_LIN\
E_COMMENT_MODE,e\
.C_BLOCK_COMMENT\
_MODE,e.COMMENT(\
\x22/\x5c\x5c*\x5c\x5c*!\x22,\x22\x5c\x5c*/\
\x22),e.C_NUMBER_MO\
DE,e.inherit(e.A\
POS_STRING_MODE,\
{\x0aillegal:null})\
,e.inherit(e.QUO\
TE_STRING_MODE,{\
illegal:null}),{\
\x0aclassName:\x22stri\
ng\x22,begin:\x22'\x22,en\
d:\x22'\x22},{classNam\
e:\x22symbol\x22,varia\
nts:[{begin:\x22#\x22,\
\x0aend:\x22\x5c\x5cd+\x22,ille\
gal:\x22\x5c\x5cW\x22}]}]})}\
)();hljs.registe\
rLanguage(\x22step2\
1\x22,e)})();/*! `s\
tylus` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22\x0a;const e=[\x22a\x22\
,\x22abbr\x22,\x22address\
\x22,\x22article\x22,\x22asi\
de\x22,\x22audio\x22,\x22b\x22,\
\x22blockquote\x22,\x22bo\
dy\x22,\x22button\x22,\x22ca\
nvas\x22,\x22caption\x22,\
\x22cite\x22,\x22code\x22,\x22d\
d\x22,\x22del\x22,\x22detail\
s\x22,\x22dfn\x22,\x22div\x22,\x22\
dl\x22,\x22dt\x22,\x22em\x22,\x22f\
ieldset\x22,\x22figcap\
tion\x22,\x22figure\x22,\x22\
footer\x22,\x22form\x22,\x22\
h1\x22,\x22h2\x22,\x22h3\x22,\x22h\
4\x22,\x22h5\x22,\x22h6\x22,\x22he\
ader\x22,\x22hgroup\x22,\x22\
html\x22,\x22i\x22,\x22ifram\
e\x22,\x22img\x22,\x22input\x22\
,\x22ins\x22,\x22kbd\x22,\x22la\
bel\x22,\x22legend\x22,\x22l\
i\x22,\x22main\x22,\x22mark\x22\
,\x22menu\x22,\x22nav\x22,\x22o\
bject\x22,\x22ol\x22,\x22opt\
group\x22,\x22option\x22,\
\x22p\x22,\x22picture\x22,\x22q\
\x22,\x22quote\x22,\x22samp\x22\
,\x22section\x22,\x22sele\
ct\x22,\x22source\x22,\x22sp\
an\x22,\x22strong\x22,\x22su\
mmary\x22,\x22sup\x22,\x22ta\
ble\x22,\x22tbody\x22,\x22td\
\x22,\x22textarea\x22,\x22tf\
oot\x22,\x22th\x22,\x22thead\
\x22,\x22time\x22,\x22tr\x22,\x22u\
l\x22,\x22var\x22,\x22video\x22\
,\x22defs\x22,\x22g\x22,\x22mar\
ker\x22,\x22mask\x22,\x22pat\
tern\x22,\x22svg\x22,\x22swi\
tch\x22,\x22symbol\x22,\x22f\
eBlend\x22,\x22feColor\
Matrix\x22,\x22feCompo\
nentTransfer\x22,\x22f\
eComposite\x22,\x22feC\
onvolveMatrix\x22,\x22\
feDiffuseLightin\
g\x22,\x22feDisplaceme\
ntMap\x22,\x22feFlood\x22\
,\x22feGaussianBlur\
\x22,\x22feImage\x22,\x22feM\
erge\x22,\x22feMorphol\
ogy\x22,\x22feOffset\x22,\
\x22feSpecularLight\
ing\x22,\x22feTile\x22,\x22f\
eTurbulence\x22,\x22li\
nearGradient\x22,\x22r\
adialGradient\x22,\x22\
stop\x22,\x22circle\x22,\x22\
ellipse\x22,\x22image\x22\
,\x22line\x22,\x22path\x22,\x22\
polygon\x22,\x22polyli\
ne\x22,\x22rect\x22,\x22text\
\x22,\x22use\x22,\x22textPat\
h\x22,\x22tspan\x22,\x22fore\
ignObject\x22,\x22clip\
Path\x22],t=[\x22any-h\
over\x22,\x22any-point\
er\x22,\x22aspect-rati\
o\x22,\x22color\x22,\x22colo\
r-gamut\x22,\x22color-\
index\x22,\x22device-a\
spect-ratio\x22,\x22de\
vice-height\x22,\x22de\
vice-width\x22,\x22dis\
play-mode\x22,\x22forc\
ed-colors\x22,\x22grid\
\x22,\x22height\x22,\x22hove\
r\x22,\x22inverted-col\
ors\x22,\x22monochrome\
\x22,\x22orientation\x22,\
\x22overflow-block\x22\
,\x22overflow-inlin\
e\x22,\x22pointer\x22,\x22pr\
efers-color-sche\
me\x22,\x22prefers-con\
trast\x22,\x22prefers-\
reduced-motion\x22,\
\x22prefers-reduced\
-transparency\x22,\x22\
resolution\x22,\x22sca\
n\x22,\x22scripting\x22,\x22\
update\x22,\x22width\x22,\
\x22min-width\x22,\x22max\
-width\x22,\x22min-hei\
ght\x22,\x22max-height\
\x22].sort().revers\
e(),r=[\x22active\x22,\
\x22any-link\x22,\x22blan\
k\x22,\x22checked\x22,\x22cu\
rrent\x22,\x22default\x22\
,\x22defined\x22,\x22dir\x22\
,\x22disabled\x22,\x22dro\
p\x22,\x22empty\x22,\x22enab\
led\x22,\x22first\x22,\x22fi\
rst-child\x22,\x22firs\
t-of-type\x22,\x22full\
screen\x22,\x22future\x22\
,\x22focus\x22,\x22focus-\
visible\x22,\x22focus-\
within\x22,\x22has\x22,\x22h\
ost\x22,\x22host-conte\
xt\x22,\x22hover\x22,\x22ind\
eterminate\x22,\x22in-\
range\x22,\x22invalid\x22\
,\x22is\x22,\x22lang\x22,\x22la\
st-child\x22,\x22last-\
of-type\x22,\x22left\x22,\
\x22link\x22,\x22local-li\
nk\x22,\x22not\x22,\x22nth-c\
hild\x22,\x22nth-col\x22,\
\x22nth-last-child\x22\
,\x22nth-last-col\x22,\
\x22nth-last-of-typ\
e\x22,\x22nth-of-type\x22\
,\x22only-child\x22,\x22o\
nly-of-type\x22,\x22op\
tional\x22,\x22out-of-\
range\x22,\x22past\x22,\x22p\
laceholder-shown\
\x22,\x22read-only\x22,\x22r\
ead-write\x22,\x22requ\
ired\x22,\x22right\x22,\x22r\
oot\x22,\x22scope\x22,\x22ta\
rget\x22,\x22target-wi\
thin\x22,\x22user-inva\
lid\x22,\x22valid\x22,\x22vi\
sited\x22,\x22where\x22].\
sort().reverse()\
,i=[\x22after\x22,\x22bac\
kdrop\x22,\x22before\x22,\
\x22cue\x22,\x22cue-regio\
n\x22,\x22first-letter\
\x22,\x22first-line\x22,\x22\
grammar-error\x22,\x22\
marker\x22,\x22part\x22,\x22\
placeholder\x22,\x22se\
lection\x22,\x22slotte\
d\x22,\x22spelling-err\
or\x22].sort().reve\
rse(),o=[\x22accent\
-color\x22,\x22align-c\
ontent\x22,\x22align-i\
tems\x22,\x22align-sel\
f\x22,\x22alignment-ba\
seline\x22,\x22all\x22,\x22a\
nimation\x22,\x22anima\
tion-delay\x22,\x22ani\
mation-direction\
\x22,\x22animation-dur\
ation\x22,\x22animatio\
n-fill-mode\x22,\x22an\
imation-iteratio\
n-count\x22,\x22animat\
ion-name\x22,\x22anima\
tion-play-state\x22\
,\x22animation-timi\
ng-function\x22,\x22ap\
pearance\x22,\x22backf\
ace-visibility\x22,\
\x22background\x22,\x22ba\
ckground-attachm\
ent\x22,\x22background\
-blend-mode\x22,\x22ba\
ckground-clip\x22,\x22\
background-color\
\x22,\x22background-im\
age\x22,\x22background\
-origin\x22,\x22backgr\
ound-position\x22,\x22\
background-repea\
t\x22,\x22background-s\
ize\x22,\x22baseline-s\
hift\x22,\x22block-siz\
e\x22,\x22border\x22,\x22bor\
der-block\x22,\x22bord\
er-block-color\x22,\
\x22border-block-en\
d\x22,\x22border-block\
-end-color\x22,\x22bor\
der-block-end-st\
yle\x22,\x22border-blo\
ck-end-width\x22,\x22b\
order-block-star\
t\x22,\x22border-block\
-start-color\x22,\x22b\
order-block-star\
t-style\x22,\x22border\
-block-start-wid\
th\x22,\x22border-bloc\
k-style\x22,\x22border\
-block-width\x22,\x22b\
order-bottom\x22,\x22b\
order-bottom-col\
or\x22,\x22border-bott\
om-left-radius\x22,\
\x22border-bottom-r\
ight-radius\x22,\x22bo\
rder-bottom-styl\
e\x22,\x22border-botto\
m-width\x22,\x22border\
-collapse\x22,\x22bord\
er-color\x22,\x22borde\
r-image\x22,\x22border\
-image-outset\x22,\x22\
border-image-rep\
eat\x22,\x22border-ima\
ge-slice\x22,\x22borde\
r-image-source\x22,\
\x22border-image-wi\
dth\x22,\x22border-inl\
ine\x22,\x22border-inl\
ine-color\x22,\x22bord\
er-inline-end\x22,\x22\
border-inline-en\
d-color\x22,\x22border\
-inline-end-styl\
e\x22,\x22border-inlin\
e-end-width\x22,\x22bo\
rder-inline-star\
t\x22,\x22border-inlin\
e-start-color\x22,\x22\
border-inline-st\
art-style\x22,\x22bord\
er-inline-start-\
width\x22,\x22border-i\
nline-style\x22,\x22bo\
rder-inline-widt\
h\x22,\x22border-left\x22\
,\x22border-left-co\
lor\x22,\x22border-lef\
t-style\x22,\x22border\
-left-width\x22,\x22bo\
rder-radius\x22,\x22bo\
rder-right\x22,\x22bor\
der-end-end-radi\
us\x22,\x22border-end-\
start-radius\x22,\x22b\
order-right-colo\
r\x22,\x22border-right\
-style\x22,\x22border-\
right-width\x22,\x22bo\
rder-spacing\x22,\x22b\
order-start-end-\
radius\x22,\x22border-\
start-start-radi\
us\x22,\x22border-styl\
e\x22,\x22border-top\x22,\
\x22border-top-colo\
r\x22,\x22border-top-l\
eft-radius\x22,\x22bor\
der-top-right-ra\
dius\x22,\x22border-to\
p-style\x22,\x22border\
-top-width\x22,\x22bor\
der-width\x22,\x22bott\
om\x22,\x22box-decorat\
ion-break\x22,\x22box-\
shadow\x22,\x22box-siz\
ing\x22,\x22break-afte\
r\x22,\x22break-before\
\x22,\x22break-inside\x22\
,\x22cx\x22,\x22cy\x22,\x22capt\
ion-side\x22,\x22caret\
-color\x22,\x22clear\x22,\
\x22clip\x22,\x22clip-pat\
h\x22,\x22clip-rule\x22,\x22\
color\x22,\x22color-in\
terpolation\x22,\x22co\
lor-interpolatio\
n-filters\x22,\x22colo\
r-profile\x22,\x22colo\
r-rendering\x22,\x22co\
lor-scheme\x22,\x22col\
umn-count\x22,\x22colu\
mn-fill\x22,\x22column\
-gap\x22,\x22column-ru\
le\x22,\x22column-rule\
-color\x22,\x22column-\
rule-style\x22,\x22col\
umn-rule-width\x22,\
\x22column-span\x22,\x22c\
olumn-width\x22,\x22co\
lumns\x22,\x22contain\x22\
,\x22content\x22,\x22cont\
ent-visibility\x22,\
\x22counter-increme\
nt\x22,\x22counter-res\
et\x22,\x22cue\x22,\x22cue-a\
fter\x22,\x22cue-befor\
e\x22,\x22cursor\x22,\x22dir\
ection\x22,\x22display\
\x22,\x22dominant-base\
line\x22,\x22empty-cel\
ls\x22,\x22enable-back\
ground\x22,\x22fill\x22,\x22\
fill-opacity\x22,\x22f\
ill-rule\x22,\x22filte\
r\x22,\x22flex\x22,\x22flex-\
basis\x22,\x22flex-dir\
ection\x22,\x22flex-fl\
ow\x22,\x22flex-grow\x22,\
\x22flex-shrink\x22,\x22f\
lex-wrap\x22,\x22float\
\x22,\x22flow\x22,\x22flood-\
color\x22,\x22flood-op\
acity\x22,\x22font\x22,\x22f\
ont-display\x22,\x22fo\
nt-family\x22,\x22font\
-feature-setting\
s\x22,\x22font-kerning\
\x22,\x22font-language\
-override\x22,\x22font\
-size\x22,\x22font-siz\
e-adjust\x22,\x22font-\
smoothing\x22,\x22font\
-stretch\x22,\x22font-\
style\x22,\x22font-syn\
thesis\x22,\x22font-va\
riant\x22,\x22font-var\
iant-caps\x22,\x22font\
-variant-east-as\
ian\x22,\x22font-varia\
nt-ligatures\x22,\x22f\
ont-variant-nume\
ric\x22,\x22font-varia\
nt-position\x22,\x22fo\
nt-variation-set\
tings\x22,\x22font-wei\
ght\x22,\x22gap\x22,\x22glyp\
h-orientation-ho\
rizontal\x22,\x22glyph\
-orientation-ver\
tical\x22,\x22grid\x22,\x22g\
rid-area\x22,\x22grid-\
auto-columns\x22,\x22g\
rid-auto-flow\x22,\x22\
grid-auto-rows\x22,\
\x22grid-column\x22,\x22g\
rid-column-end\x22,\
\x22grid-column-sta\
rt\x22,\x22grid-gap\x22,\x22\
grid-row\x22,\x22grid-\
row-end\x22,\x22grid-r\
ow-start\x22,\x22grid-\
template\x22,\x22grid-\
template-areas\x22,\
\x22grid-template-c\
olumns\x22,\x22grid-te\
mplate-rows\x22,\x22ha\
nging-punctuatio\
n\x22,\x22height\x22,\x22hyp\
hens\x22,\x22icon\x22,\x22im\
age-orientation\x22\
,\x22image-renderin\
g\x22,\x22image-resolu\
tion\x22,\x22ime-mode\x22\
,\x22inline-size\x22,\x22\
inset\x22,\x22inset-bl\
ock\x22,\x22inset-bloc\
k-end\x22,\x22inset-bl\
ock-start\x22,\x22inse\
t-inline\x22,\x22inset\
-inline-end\x22,\x22in\
set-inline-start\
\x22,\x22isolation\x22,\x22k\
erning\x22,\x22justify\
-content\x22,\x22justi\
fy-items\x22,\x22justi\
fy-self\x22,\x22left\x22,\
\x22letter-spacing\x22\
,\x22lighting-color\
\x22,\x22line-break\x22,\x22\
line-height\x22,\x22li\
st-style\x22,\x22list-\
style-image\x22,\x22li\
st-style-positio\
n\x22,\x22list-style-t\
ype\x22,\x22marker\x22,\x22m\
arker-end\x22,\x22mark\
er-mid\x22,\x22marker-\
start\x22,\x22mask\x22,\x22m\
argin\x22,\x22margin-b\
lock\x22,\x22margin-bl\
ock-end\x22,\x22margin\
-block-start\x22,\x22m\
argin-bottom\x22,\x22m\
argin-inline\x22,\x22m\
argin-inline-end\
\x22,\x22margin-inline\
-start\x22,\x22margin-\
left\x22,\x22margin-ri\
ght\x22,\x22margin-top\
\x22,\x22marks\x22,\x22mask\x22\
,\x22mask-border\x22,\x22\
mask-border-mode\
\x22,\x22mask-border-o\
utset\x22,\x22mask-bor\
der-repeat\x22,\x22mas\
k-border-slice\x22,\
\x22mask-border-sou\
rce\x22,\x22mask-borde\
r-width\x22,\x22mask-c\
lip\x22,\x22mask-compo\
site\x22,\x22mask-imag\
e\x22,\x22mask-mode\x22,\x22\
mask-origin\x22,\x22ma\
sk-position\x22,\x22ma\
sk-repeat\x22,\x22mask\
-size\x22,\x22mask-typ\
e\x22,\x22max-block-si\
ze\x22,\x22max-height\x22\
,\x22max-inline-siz\
e\x22,\x22max-width\x22,\x22\
min-block-size\x22,\
\x22min-height\x22,\x22mi\
n-inline-size\x22,\x22\
min-width\x22,\x22mix-\
blend-mode\x22,\x22nav\
-down\x22,\x22nav-inde\
x\x22,\x22nav-left\x22,\x22n\
av-right\x22,\x22nav-u\
p\x22,\x22none\x22,\x22norma\
l\x22,\x22object-fit\x22,\
\x22object-position\
\x22,\x22opacity\x22,\x22ord\
er\x22,\x22orphans\x22,\x22o\
utline\x22,\x22outline\
-color\x22,\x22outline\
-offset\x22,\x22outlin\
e-style\x22,\x22outlin\
e-width\x22,\x22overfl\
ow\x22,\x22overflow-wr\
ap\x22,\x22overflow-x\x22\
,\x22overflow-y\x22,\x22p\
adding\x22,\x22padding\
-block\x22,\x22padding\
-block-end\x22,\x22pad\
ding-block-start\
\x22,\x22padding-botto\
m\x22,\x22padding-inli\
ne\x22,\x22padding-inl\
ine-end\x22,\x22paddin\
g-inline-start\x22,\
\x22padding-left\x22,\x22\
padding-right\x22,\x22\
padding-top\x22,\x22pa\
ge-break-after\x22,\
\x22page-break-befo\
re\x22,\x22page-break-\
inside\x22,\x22pause\x22,\
\x22pause-after\x22,\x22p\
ause-before\x22,\x22pe\
rspective\x22,\x22pers\
pective-origin\x22,\
\x22pointer-events\x22\
,\x22position\x22,\x22quo\
tes\x22,\x22r\x22,\x22resize\
\x22,\x22rest\x22,\x22rest-a\
fter\x22,\x22rest-befo\
re\x22,\x22right\x22,\x22rot\
ate\x22,\x22row-gap\x22,\x22\
scale\x22,\x22scroll-m\
argin\x22,\x22scroll-m\
argin-block\x22,\x22sc\
roll-margin-bloc\
k-end\x22,\x22scroll-m\
argin-block-star\
t\x22,\x22scroll-margi\
n-bottom\x22,\x22scrol\
l-margin-inline\x22\
,\x22scroll-margin-\
inline-end\x22,\x22scr\
oll-margin-inlin\
e-start\x22,\x22scroll\
-margin-left\x22,\x22s\
croll-margin-rig\
ht\x22,\x22scroll-marg\
in-top\x22,\x22scroll-\
padding\x22,\x22scroll\
-padding-block\x22,\
\x22scroll-padding-\
block-end\x22,\x22scro\
ll-padding-block\
-start\x22,\x22scroll-\
padding-bottom\x22,\
\x22scroll-padding-\
inline\x22,\x22scroll-\
padding-inline-e\
nd\x22,\x22scroll-padd\
ing-inline-start\
\x22,\x22scroll-paddin\
g-left\x22,\x22scroll-\
padding-right\x22,\x22\
scroll-padding-t\
op\x22,\x22scroll-snap\
-align\x22,\x22scroll-\
snap-stop\x22,\x22scro\
ll-snap-type\x22,\x22s\
crollbar-color\x22,\
\x22scrollbar-gutte\
r\x22,\x22scrollbar-wi\
dth\x22,\x22shape-imag\
e-threshold\x22,\x22sh\
ape-margin\x22,\x22sha\
pe-outside\x22,\x22sha\
pe-rendering\x22,\x22s\
top-color\x22,\x22stop\
-opacity\x22,\x22strok\
e\x22,\x22stroke-dasha\
rray\x22,\x22stroke-da\
shoffset\x22,\x22strok\
e-linecap\x22,\x22stro\
ke-linejoin\x22,\x22st\
roke-miterlimit\x22\
,\x22stroke-opacity\
\x22,\x22stroke-width\x22\
,\x22speak\x22,\x22speak-\
as\x22,\x22src\x22,\x22tab-s\
ize\x22,\x22table-layo\
ut\x22,\x22text-anchor\
\x22,\x22text-align\x22,\x22\
text-align-all\x22,\
\x22text-align-last\
\x22,\x22text-combine-\
upright\x22,\x22text-d\
ecoration\x22,\x22text\
-decoration-colo\
r\x22,\x22text-decorat\
ion-line\x22,\x22text-\
decoration-skip-\
ink\x22,\x22text-decor\
ation-style\x22,\x22te\
xt-decoration-th\
ickness\x22,\x22text-e\
mphasis\x22,\x22text-e\
mphasis-color\x22,\x22\
text-emphasis-po\
sition\x22,\x22text-em\
phasis-style\x22,\x22t\
ext-indent\x22,\x22tex\
t-justify\x22,\x22text\
-orientation\x22,\x22t\
ext-overflow\x22,\x22t\
ext-rendering\x22,\x22\
text-shadow\x22,\x22te\
xt-transform\x22,\x22t\
ext-underline-of\
fset\x22,\x22text-unde\
rline-position\x22,\
\x22top\x22,\x22transform\
\x22,\x22transform-box\
\x22,\x22transform-ori\
gin\x22,\x22transform-\
style\x22,\x22transiti\
on\x22,\x22transition-\
delay\x22,\x22transiti\
on-duration\x22,\x22tr\
ansition-propert\
y\x22,\x22transition-t\
iming-function\x22,\
\x22translate\x22,\x22uni\
code-bidi\x22,\x22vect\
or-effect\x22,\x22vert\
ical-align\x22,\x22vis\
ibility\x22,\x22voice-\
balance\x22,\x22voice-\
duration\x22,\x22voice\
-family\x22,\x22voice-\
pitch\x22,\x22voice-ra\
nge\x22,\x22voice-rate\
\x22,\x22voice-stress\x22\
,\x22voice-volume\x22,\
\x22white-space\x22,\x22w\
idows\x22,\x22width\x22,\x22\
will-change\x22,\x22wo\
rd-break\x22,\x22word-\
spacing\x22,\x22word-w\
rap\x22,\x22writing-mo\
de\x22,\x22x\x22,\x22y\x22,\x22z-i\
ndex\x22].sort().re\
verse()\x0a;return \
n=>{const a=(e=>\
({IMPORTANT:{sco\
pe:\x22meta\x22,begin:\
\x22!important\x22},\x0aB\
LOCK_COMMENT:e.C\
_BLOCK_COMMENT_M\
ODE,HEXCOLOR:{sc\
ope:\x22number\x22,\x0abe\
gin:/#(([0-9a-fA\
-F]{3,4})|(([0-9\
a-fA-F]{2}){3,4}\
))\x5cb/},FUNCTION_\
DISPATCH:{\x0aclass\
Name:\x22built_in\x22,\
begin:/[\x5cw-]+(?=\
\x5c()/},ATTRIBUTE_\
SELECTOR_MODE:{\x0a\
scope:\x22selector-\
attr\x22,begin:/\x5c[/\
,end:/\x5c]/,illega\
l:\x22$\x22,\x0acontains:\
[e.APOS_STRING_M\
ODE,e.QUOTE_STRI\
NG_MODE]},CSS_NU\
MBER_MODE:{\x0ascop\
e:\x22number\x22,\x0abegi\
n:e.NUMBER_RE+\x22(\
%|em|ex|ch|rem|v\
w|vh|vmin|vmax|c\
m|mm|in|pt|pc|px\
|deg|grad|rad|tu\
rn|s|ms|Hz|kHz|d\
pi|dpcm|dppx)?\x22,\
\x0arelevance:0},CS\
S_VARIABLE:{clas\
sName:\x22attr\x22,beg\
in:/--[A-Za-z_][\
A-Za-z0-9_-]*/}\x0a\
}))(n),l={classN\
ame:\x22variable\x22,b\
egin:\x22\x5c\x5c$\x22+n.IDE\
NT_RE},s=\x22(?=[.\x5c\
\x5cs\x5c\x5cn[:,(])\x22\x0a;re\
turn{name:\x22Stylu\
s\x22,aliases:[\x22sty\
l\x22],case_insensi\
tive:!1,\x0akeyword\
s:\x22if else for i\
n\x22,\x0aillegal:\x22(\x5c\x5c\
?|(\x5c\x5cbReturn\x5c\x5cb)\
|(\x5c\x5cbEnd\x5c\x5cb)|(\x5c\x5c\
bend\x5c\x5cb)|(\x5c\x5cbdef\
\x5c\x5cb)|;|#\x5c\x5cs|\x5c\x5c*\x5c\
\x5cs|===\x5c\x5cs|\x5c\x5c||%)\
\x22,\x0acontains:[n.Q\
UOTE_STRING_MODE\
,n.APOS_STRING_M\
ODE,n.C_LINE_COM\
MENT_MODE,n.C_BL\
OCK_COMMENT_MODE\
,a.HEXCOLOR,{\x0abe\
gin:\x22\x5c\x5c.[a-zA-Z]\
[a-zA-Z0-9_-]*\x22+\
s,className:\x22sel\
ector-class\x22},{\x0a\
begin:\x22#[a-zA-Z]\
[a-zA-Z0-9_-]*\x22+\
s,className:\x22sel\
ector-id\x22},{\x0abeg\
in:\x22\x5c\x5cb(\x22+e.join\
(\x22|\x22)+\x22)\x22+s,clas\
sName:\x22selector-\
tag\x22},{\x0aclassNam\
e:\x22selector-pseu\
do\x22,begin:\x22&?:(\x22\
+r.join(\x22|\x22)+\x22)\x22\
+s},{\x0aclassName:\
\x22selector-pseudo\
\x22,begin:\x22&?:(:)?\
(\x22+i.join(\x22|\x22)+\x22\
)\x22+s\x0a},a.ATTRIBU\
TE_SELECTOR_MODE\
,{className:\x22key\
word\x22,begin:/@me\
dia/,starts:{\x0aen\
d:/[{;}]/,keywor\
ds:{$pattern:/[a\
-z-]+/,keyword:\x22\
and or not only\x22\
,\x0aattribute:t.jo\
in(\x22 \x22)},contain\
s:[a.CSS_NUMBER_\
MODE]}},{classNa\
me:\x22keyword\x22,\x0abe\
gin:\x22@((-(o|moz|\
ms|webkit)-)?(ch\
arset|css|debug|\
extend|font-face\
|for|import|incl\
ude|keyframes|me\
dia|mixin|page|w\
arn|while))\x5c\x5cb\x22\x0a\
},l,a.CSS_NUMBER\
_MODE,{className\
:\x22function\x22,\x0abeg\
in:\x22^[a-zA-Z][a-\
zA-Z0-9_-]*\x5c\x5c(.*\
\x5c\x5c)\x22,illegal:\x22[\x5c\
\x5cn]\x22,returnBegin\
:!0,\x0acontains:[{\
className:\x22title\
\x22,begin:\x22\x5c\x5cb[a-z\
A-Z][a-zA-Z0-9_-\
]*\x22},{\x0aclassName\
:\x22params\x22,begin:\
/\x5c(/,end:/\x5c)/,\x0ac\
ontains:[a.HEXCO\
LOR,l,n.APOS_STR\
ING_MODE,a.CSS_N\
UMBER_MODE,n.QUO\
TE_STRING_MODE]\x0a\
}]},a.CSS_VARIAB\
LE,{className:\x22a\
ttribute\x22,begin:\
\x22\x5c\x5cb(\x22+o.join(\x22|\
\x22)+\x22)\x5c\x5cb\x22,\x0astart\
s:{end:/;|$/,\x0aco\
ntains:[a.HEXCOL\
OR,l,n.APOS_STRI\
NG_MODE,n.QUOTE_\
STRING_MODE,a.CS\
S_NUMBER_MODE,n.\
C_BLOCK_COMMENT_\
MODE,a.IMPORTANT\
,a.FUNCTION_DISP\
ATCH],\x0aillegal:/\
\x5c./,relevance:0}\
},a.FUNCTION_DIS\
PATCH]}}})()\x0a;hl\
js.registerLangu\
age(\x22stylus\x22,e)}\
)();/*! `subunit\
` grammar compil\
ed for Highlight\
.js 11.10.0 */\x0a(\
()=>{var s=(()=>\
{\x22use strict\x22;re\
turn s=>({name:\x22\
SubUnit\x22,case_in\
sensitive:!0,\x0aco\
ntains:[{classNa\
me:\x22string\x22,begi\
n:\x22\x5c\x5c[\x5cn(multipa\
rt)?\x22,end:\x22\x5c\x5c]\x5cn\
\x22},{\x0aclassName:\x22\
string\x22,begin:\x22\x5c\
\x5cd{4}-\x5c\x5cd{2}-\x5c\x5cd\
{2}(\x5c\x5cs+)\x5c\x5cd{2}:\
\x5c\x5cd{2}:\x5c\x5cd{2}.\x5c\x5c\
d+Z\x22\x0a},{classNam\
e:\x22string\x22,begin\
:\x22(\x5c\x5c+|-)\x5c\x5cd+\x22},\
{className:\x22keyw\
ord\x22,relevance:1\
0,\x0avariants:[{\x0ab\
egin:\x22^(test|tes\
ting|success|suc\
cessful|failure|\
error|skip|xfail\
|uxsuccess)(:?)\x5c\
\x5cs+(test)?\x22\x0a},{b\
egin:\x22^progress(\
:?)(\x5c\x5cs+)?(pop|p\
ush)?\x22},{begin:\x22\
^tags:\x22},{begin:\
\x22^time:\x22}]}]\x0a})}\
)();hljs.registe\
rLanguage(\x22subun\
it\x22,s)})();/*! `\
swift` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;function e(e)\
{\x0areturn e?\x22stri\
ng\x22==typeof e?e:\
e.source:null}fu\
nction n(e){retu\
rn t(\x22(?=\x22,e,\x22)\x22\
)}\x0afunction t(..\
.n){return n.map\
((n=>e(n))).join\
(\x22\x22)}function a(\
...n){const t=(e\
=>{\x0aconst n=e[e.\
length-1]\x0a;retur\
n\x22object\x22==typeo\
f n&&n.construct\
or===Object?(e.s\
plice(e.length-1\
,1),n):{}\x0a})(n);\
return\x22(\x22+(t.cap\
ture?\x22\x22:\x22?:\x22)+n.\
map((n=>e(n))).j\
oin(\x22|\x22)+\x22)\x22}\x0aco\
nst i=e=>t(/\x5cb/,\
e,/\x5cw$/.test(e)?\
/\x5cb/:/\x5cB/),s=[\x22P\
rotocol\x22,\x22Type\x22]\
.map(i),c=[\x22init\
\x22,\x22self\x22].map(i)\
,u=[\x22Any\x22,\x22Self\x22\
],o=[\x22actor\x22,\x22an\
y\x22,\x22associatedty\
pe\x22,\x22async\x22,\x22awa\
it\x22,/as\x5c?/,/as!/\
,\x22as\x22,\x22borrowing\
\x22,\x22break\x22,\x22case\x22\
,\x22catch\x22,\x22class\x22\
,\x22consume\x22,\x22cons\
uming\x22,\x22continue\
\x22,\x22convenience\x22,\
\x22copy\x22,\x22default\x22\
,\x22defer\x22,\x22deinit\
\x22,\x22didSet\x22,\x22dist\
ributed\x22,\x22do\x22,\x22d\
ynamic\x22,\x22each\x22,\x22\
else\x22,\x22enum\x22,\x22ex\
tension\x22,\x22fallth\
rough\x22,/filepriv\
ate\x5c(set\x5c)/,\x22fil\
eprivate\x22,\x22final\
\x22,\x22for\x22,\x22func\x22,\x22\
get\x22,\x22guard\x22,\x22if\
\x22,\x22import\x22,\x22indi\
rect\x22,\x22infix\x22,/i\
nit\x5c?/,/init!/,\x22\
inout\x22,/internal\
\x5c(set\x5c)/,\x22intern\
al\x22,\x22in\x22,\x22is\x22,\x22i\
solated\x22,\x22noniso\
lated\x22,\x22lazy\x22,\x22l\
et\x22,\x22macro\x22,\x22mut\
ating\x22,\x22nonmutat\
ing\x22,/open\x5c(set\x5c\
)/,\x22open\x22,\x22opera\
tor\x22,\x22optional\x22,\
\x22override\x22,\x22pack\
age\x22,\x22postfix\x22,\x22\
precedencegroup\x22\
,\x22prefix\x22,/priva\
te\x5c(set\x5c)/,\x22priv\
ate\x22,\x22protocol\x22,\
/public\x5c(set\x5c)/,\
\x22public\x22,\x22repeat\
\x22,\x22required\x22,\x22re\
throws\x22,\x22return\x22\
,\x22set\x22,\x22some\x22,\x22s\
tatic\x22,\x22struct\x22,\
\x22subscript\x22,\x22sup\
er\x22,\x22switch\x22,\x22th\
rows\x22,\x22throw\x22,/t\
ry\x5c?/,/try!/,\x22tr\
y\x22,\x22typealias\x22,/\
unowned\x5c(safe\x5c)/\
,/unowned\x5c(unsaf\
e\x5c)/,\x22unowned\x22,\x22\
var\x22,\x22weak\x22,\x22whe\
re\x22,\x22while\x22,\x22wil\
lSet\x22],r=[\x22false\
\x22,\x22nil\x22,\x22true\x22],\
l=[\x22assignment\x22,\
\x22associativity\x22,\
\x22higherThan\x22,\x22le\
ft\x22,\x22lowerThan\x22,\
\x22none\x22,\x22right\x22],\
m=[\x22#colorLitera\
l\x22,\x22#column\x22,\x22#d\
sohandle\x22,\x22#else\
\x22,\x22#elseif\x22,\x22#en\
dif\x22,\x22#error\x22,\x22#\
file\x22,\x22#fileID\x22,\
\x22#fileLiteral\x22,\x22\
#filePath\x22,\x22#fun\
ction\x22,\x22#if\x22,\x22#i\
mageLiteral\x22,\x22#k\
eyPath\x22,\x22#line\x22,\
\x22#selector\x22,\x22#so\
urceLocation\x22,\x22#\
warning\x22],p=[\x22ab\
s\x22,\x22all\x22,\x22any\x22,\x22\
assert\x22,\x22asserti\
onFailure\x22,\x22debu\
gPrint\x22,\x22dump\x22,\x22\
fatalError\x22,\x22get\
VaList\x22,\x22isKnown\
UniquelyReferenc\
ed\x22,\x22max\x22,\x22min\x22,\
\x22numericCast\x22,\x22p\
ointwiseMax\x22,\x22po\
intwiseMin\x22,\x22pre\
condition\x22,\x22prec\
onditionFailure\x22\
,\x22print\x22,\x22readLi\
ne\x22,\x22repeatEleme\
nt\x22,\x22sequence\x22,\x22\
stride\x22,\x22swap\x22,\x22\
swift_unboxFromS\
wiftValueWithTyp\
e\x22,\x22transcode\x22,\x22\
type\x22,\x22unsafeBit\
Cast\x22,\x22unsafeDow\
ncast\x22,\x22withExte\
ndedLifetime\x22,\x22w\
ithUnsafeMutable\
Pointer\x22,\x22withUn\
safePointer\x22,\x22wi\
thVaList\x22,\x22witho\
utActuallyEscapi\
ng\x22,\x22zip\x22],d=a(/\
[/=\x5c-+!*%<>&|^~?\
]/,/[\x5cu00A1-\x5cu00\
A7]/,/[\x5cu00A9\x5cu0\
0AB]/,/[\x5cu00AC\x5cu\
00AE]/,/[\x5cu00B0\x5c\
u00B1]/,/[\x5cu00B6\
\x5cu00BB\x5cu00BF\x5cu00\
D7\x5cu00F7]/,/[\x5cu2\
016-\x5cu2017]/,/[\x5c\
u2020-\x5cu2027]/,/\
[\x5cu2030-\x5cu203E]/\
,/[\x5cu2041-\x5cu2053\
]/,/[\x5cu2055-\x5cu20\
5E]/,/[\x5cu2190-\x5cu\
23FF]/,/[\x5cu2500-\
\x5cu2775]/,/[\x5cu279\
4-\x5cu2BFF]/,/[\x5cu2\
E00-\x5cu2E7F]/,/[\x5c\
u3001-\x5cu3003]/,/\
[\x5cu3008-\x5cu3020]/\
,/[\x5cu3030]/),F=a\
(d,/[\x5cu0300-\x5cu03\
6F]/,/[\x5cu1DC0-\x5cu\
1DFF]/,/[\x5cu20D0-\
\x5cu20FF]/,/[\x5cuFE0\
0-\x5cuFE0F]/,/[\x5cuF\
E20-\x5cuFE2F]/),b=\
t(d,F,\x22*\x22),h=a(/\
[a-zA-Z_]/,/[\x5cu0\
0A8\x5cu00AA\x5cu00AD\x5c\
u00AF\x5cu00B2-\x5cu00\
B5\x5cu00B7-\x5cu00BA]\
/,/[\x5cu00BC-\x5cu00B\
E\x5cu00C0-\x5cu00D6\x5cu\
00D8-\x5cu00F6\x5cu00F\
8-\x5cu00FF]/,/[\x5cu0\
100-\x5cu02FF\x5cu0370\
-\x5cu167F\x5cu1681-\x5cu\
180D\x5cu180F-\x5cu1DB\
F]/,/[\x5cu1E00-\x5cu1\
FFF]/,/[\x5cu200B-\x5c\
u200D\x5cu202A-\x5cu20\
2E\x5cu203F-\x5cu2040\x5c\
u2054\x5cu2060-\x5cu20\
6F]/,/[\x5cu2070-\x5cu\
20CF\x5cu2100-\x5cu218\
F\x5cu2460-\x5cu24FF\x5cu\
2776-\x5cu2793]/,/[\
\x5cu2C00-\x5cu2DFF\x5cu2\
E80-\x5cu2FFF]/,/[\x5c\
u3004-\x5cu3007\x5cu30\
21-\x5cu302F\x5cu3031-\
\x5cu303F\x5cu3040-\x5cuD\
7FF]/,/[\x5cuF900-\x5c\
uFD3D\x5cuFD40-\x5cuFD\
CF\x5cuFDF0-\x5cuFE1F\x5c\
uFE30-\x5cuFE44]/,/\
[\x5cuFE47-\x5cuFEFE\x5cu\
FF00-\x5cuFFFD]/),f\
=a(h,/\x5cd/,/[\x5cu03\
00-\x5cu036F\x5cu1DC0-\
\x5cu1DFF\x5cu20D0-\x5cu2\
0FF\x5cuFE20-\x5cuFE2F\
]/),w=t(h,f,\x22*\x22)\
,g=t(/[A-Z]/,f,\x22\
*\x22),y=[\x22attached\
\x22,\x22autoclosure\x22,\
t(/convention\x5c(/\
,a(\x22swift\x22,\x22bloc\
k\x22,\x22c\x22),/\x5c)/),\x22d\
iscardableResult\
\x22,\x22dynamicCallab\
le\x22,\x22dynamicMemb\
erLookup\x22,\x22escap\
ing\x22,\x22freestandi\
ng\x22,\x22frozen\x22,\x22GK\
Inspectable\x22,\x22IB\
Action\x22,\x22IBDesig\
nable\x22,\x22IBInspec\
table\x22,\x22IBOutlet\
\x22,\x22IBSegueAction\
\x22,\x22inlinable\x22,\x22m\
ain\x22,\x22nonobjc\x22,\x22\
NSApplicationMai\
n\x22,\x22NSCopying\x22,\x22\
NSManaged\x22,t(/ob\
jc\x5c(/,w,/\x5c)/),\x22o\
bjc\x22,\x22objcMember\
s\x22,\x22propertyWrap\
per\x22,\x22requires_s\
tored_property_i\
nits\x22,\x22resultBui\
lder\x22,\x22Sendable\x22\
,\x22testable\x22,\x22UIA\
pplicationMain\x22,\
\x22unchecked\x22,\x22unk\
nown\x22,\x22usableFro\
mInline\x22,\x22warn_u\
nqualified_acces\
s\x22],v=[\x22iOS\x22,\x22iO\
SApplicationExte\
nsion\x22,\x22macOS\x22,\x22\
macOSApplication\
Extension\x22,\x22macC\
atalyst\x22,\x22macCat\
alystApplication\
Extension\x22,\x22watc\
hOS\x22,\x22watchOSApp\
licationExtensio\
n\x22,\x22tvOS\x22,\x22tvOSA\
pplicationExtens\
ion\x22,\x22swift\x22]\x0a;r\
eturn e=>{const \
d={match:/\x5cs+/,r\
elevance:0},h=e.\
COMMENT(\x22/\x5c\x5c*\x22,\x22\
\x5c\x5c*/\x22,{\x0acontains\
:[\x22self\x22]}),E=[e\
.C_LINE_COMMENT_\
MODE,h],A={match\
:[/\x5c./,a(...s,..\
.c)],\x0aclassName:\
{2:\x22keyword\x22}},C\
={match:t(/\x5c./,a\
(...o)),relevanc\
e:0\x0a},k=o.filter\
((e=>\x22string\x22==t\
ypeof e)).concat\
([\x22_|0\x22]),N={var\
iants:[{\x0aclassNa\
me:\x22keyword\x22,\x0ama\
tch:a(...o.filte\
r((e=>\x22string\x22!=\
typeof e)).conca\
t(u).map(i),...c\
)}]},S={\x0a$patter\
n:a(/\x5cb\x5cw+/,/#\x5cw\
+/),keyword:k.co\
ncat(m),literal:\
r},B=[A,C,N],D=[\
{\x0amatch:t(/\x5c./,a\
(...p)),relevanc\
e:0},{className:\
\x22built_in\x22,\x0amatc\
h:t(/\x5cb/,a(...p)\
,/(?=\x5c()/)}],_={\
match:/->/,relev\
ance:0},M=[_,{\x0ac\
lassName:\x22operat\
or\x22,relevance:0,\
variants:[{match\
:b},{match:`\x5c\x5c.(\
\x5c\x5c.|${F})+`}]\x0a}]\
,x=\x22([0-9]_*)+\x22,\
L=\x22([0-9a-fA-F]_\
*)+\x22,$={classNam\
e:\x22number\x22,relev\
ance:0,\x0avariants\
:[{match:`\x5c\x5cb(${\
x})(\x5c\x5c.(${x}))?(\
[eE][+-]?(${x}))\
?\x5c\x5cb`},{\x0amatch:`\
\x5c\x5cb0x(${L})(\x5c\x5c.(\
${L}))?([pP][+-]\
?(${x}))?\x5c\x5cb`},{\
match:/\x5cb0o([0-7\
]_*)+\x5cb/\x0a},{matc\
h:/\x5cb0b([01]_*)+\
\x5cb/}]},I=(e=\x22\x22)=\
>({className:\x22su\
bst\x22,variants:[{\
\x0amatch:t(/\x5c\x5c/,e,\
/[0\x5c\x5ctnr\x22']/)},{\
match:t(/\x5c\x5c/,e,/\
u\x5c{[0-9a-fA-F]{1\
,8}\x5c}/)}]\x0a}),O=(\
e=\x22\x22)=>({classNa\
me:\x22subst\x22,match\
:t(/\x5c\x5c/,e,/[\x5ct ]\
*(?:[\x5cr\x5cn]|\x5cr\x5cn)\
/)\x0a}),P=(e=\x22\x22)=>\
({className:\x22sub\
st\x22,label:\x22inter\
pol\x22,begin:t(/\x5c\x5c\
/,e,/\x5c(/),end:/\x5c\
)/\x0a}),j=(e=\x22\x22)=>\
({begin:t(e,/\x22\x22\x22\
/),end:t(/\x22\x22\x22/,e\
),contains:[I(e)\
,O(e),P(e)]\x0a}),K\
=(e=\x22\x22)=>({begin\
:t(e,/\x22/),end:t(\
/\x22/,e),contains:\
[I(e),P(e)]}),T=\
{\x0aclassName:\x22str\
ing\x22,\x0avariants:[\
j(),j(\x22#\x22),j(\x22##\
\x22),j(\x22###\x22),K(),\
K(\x22#\x22),K(\x22##\x22),K\
(\x22###\x22)]\x0a},q=[e.\
BACKSLASH_ESCAPE\
,{begin:/\x5c[/,end\
:/\x5c]/,relevance:\
0,\x0acontains:[e.B\
ACKSLASH_ESCAPE]\
}],U={begin:/\x5c/[\
^\x5cs](?=[^/\x5cn]*\x5c/\
)/,end:/\x5c//,\x0acon\
tains:q},z=e=>{c\
onst n=t(e,/\x5c//)\
,a=t(/\x5c//,e);ret\
urn{begin:n,end:\
a,\x0acontains:[...\
q,{scope:\x22commen\
t\x22,begin:`#(?!.*\
${a})`,end:/$/}]\
}},V={\x0ascope:\x22re\
gexp\x22,variants:[\
z(\x22###\x22),z(\x22##\x22)\
,z(\x22#\x22),U]},W={m\
atch:t(/`/,w,/`/\
)\x0a},Z=[W,{classN\
ame:\x22variable\x22,m\
atch:/\x5c$\x5cd+/},{c\
lassName:\x22variab\
le\x22,\x0amatch:`\x5c\x5c$$\
{f}+`}],G=[{matc\
h:/(@|#(un)?)ava\
ilable/,scope:\x22k\
eyword\x22,starts:{\
\x0acontains:[{begi\
n:/\x5c(/,end:/\x5c)/,\
keywords:v,conta\
ins:[...M,$,T]}]\
}},{\x0ascope:\x22keyw\
ord\x22,match:t(/@/\
,a(...y),n(a(/\x5c(\
/,/\x5cs+/)))},{sco\
pe:\x22meta\x22,\x0amatch\
:t(/@/,w)}],H={m\
atch:n(/\x5cb[A-Z]/\
),relevance:0,co\
ntains:[{classNa\
me:\x22type\x22,\x0amatch\
:t(/(AV|CA|CF|CG\
|CI|CL|CM|CN|CT|\
MK|MP|MTK|MTL|NS\
|SCN|SK|UI|WK|XC\
)/,f,\x22+\x22)\x0a},{cla\
ssName:\x22type\x22,ma\
tch:g,relevance:\
0},{match:/[?!]+\
/,relevance:0},{\
\x0amatch:/\x5c.\x5c.\x5c./,\
relevance:0},{ma\
tch:t(/\x5cs+&\x5cs+/,\
n(g)),relevance:\
0}]},R={\x0abegin:/\
</,end:/>/,keywo\
rds:S,contains:[\
...E,...B,...G,_\
,H]};H.contains.\
push(R)\x0a;const X\
={begin:/\x5c(/,end\
:/\x5c)/,relevance:\
0,keywords:S,con\
tains:[\x22self\x22,{\x0a\
match:t(w,/\x5cs*:/\
),keywords:\x22_|0\x22\
,relevance:0\x0a},.\
..E,V,...B,...D,\
...M,$,T,...Z,..\
.G,H]},J={begin:\
/</,end:/>/,\x0akey\
words:\x22repeat ea\
ch\x22,contains:[..\
.E,H]},Q={begin:\
/\x5c(/,end:/\x5c)/,ke\
ywords:S,\x0acontai\
ns:[{begin:a(n(t\
(w,/\x5cs*:/)),n(t(\
w,/\x5cs+/,w,/\x5cs*:/\
))),end:/:/,rele\
vance:0,\x0acontain\
s:[{className:\x22k\
eyword\x22,match:/\x5c\
b_\x5cb/},{classNam\
e:\x22params\x22,match\
:w}]\x0a},...E,...B\
,...M,$,T,...G,H\
,X],endsParent:!\
0,illegal:/[\x22']/\
},Y={\x0amatch:[/(f\
unc|macro)/,/\x5cs+\
/,a(W.match,w,b)\
],className:{1:\x22\
keyword\x22,\x0a3:\x22tit\
le.function\x22},co\
ntains:[J,Q,d],i\
llegal:[/\x5c[/,/%/\
]},ee={\x0amatch:[/\
\x5cb(?:subscript|i\
nit[?!]?)/,/\x5cs*(\
?=[<(])/],classN\
ame:{1:\x22keyword\x22\
},\x0acontains:[J,Q\
,d],illegal:/\x5c[|\
%/},ne={match:[/\
operator/,/\x5cs+/,\
b],className:{\x0a1\
:\x22keyword\x22,3:\x22ti\
tle\x22}},te={begin\
:[/precedencegro\
up/,/\x5cs+/,g],cla\
ssName:{\x0a1:\x22keyw\
ord\x22,3:\x22title\x22},\
contains:[H],key\
words:[...l,...r\
],end:/}/},ae={\x0a\
begin:[/(struct|\
protocol|class|e\
xtension|enum|ac\
tor)/,/\x5cs+/,w,/\x5c\
s*/],\x0abeginScope\
:{1:\x22keyword\x22,3:\
\x22title.class\x22},k\
eywords:S,contai\
ns:[J,...B,{begi\
n:/:/,\x0aend:/\x5c{/,\
keywords:S,conta\
ins:[{scope:\x22tit\
le.class.inherit\
ed\x22,match:g},...\
B],\x0arelevance:0}\
]};for(const e o\
f T.variants){\x0ac\
onst n=e.contain\
s.find((e=>\x22inte\
rpol\x22===e.label)\
);n.keywords=S\x0a;\
const t=[...B,..\
.D,...M,$,T,...Z\
];n.contains=[..\
.t,{begin:/\x5c(/,e\
nd:/\x5c)/,\x0acontain\
s:[\x22self\x22,...t]}\
]}return{name:\x22S\
wift\x22,keywords:S\
,\x0acontains:[...E\
,Y,ee,ae,ne,te,{\
beginKeywords:\x22i\
mport\x22,end:/$/,c\
ontains:[...E],\x0a\
relevance:0},V,.\
..B,...D,...M,$,\
T,...Z,...G,H,X]\
}}})()\x0a;hljs.reg\
isterLanguage(\x22s\
wift\x22,e)})();/*!\
 `taggerscript` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>({name:\x22Ta\
gger Script\x22,con\
tains:[{\x0aclassNa\
me:\x22comment\x22,beg\
in:/\x5c$noop\x5c(/,en\
d:/\x5c)/,contains:\
[{begin:/\x5c\x5c[()]/\
},{\x0abegin:/\x5c(/,e\
nd:/\x5c)/,contains\
:[{begin:/\x5c\x5c[()]\
/},\x22self\x22]}],rel\
evance:10},{\x0acla\
ssName:\x22keyword\x22\
,begin:/\x5c$[_a-zA\
-Z0-9]+(?=\x5c()/},\
{className:\x22vari\
able\x22,\x0abegin:/%[\
_a-zA-Z0-9:]+%/}\
,{className:\x22sym\
bol\x22,begin:/\x5c\x5c[\x5c\
\x5cnt$%,()]/},{\x0acl\
assName:\x22symbol\x22\
,begin:/\x5c\x5cu[a-fA\
-F0-9]{4}/}]})})\
()\x0a;hljs.registe\
rLanguage(\x22tagge\
rscript\x22,e)})();\
/*! `tap` gramma\
r compiled for H\
ighlight.js 11.1\
0.0 */\x0a(()=>{var\
 e=(()=>{\x22use st\
rict\x22;return e=>\
({name:\x22Test Any\
thing Protocol\x22,\
\x0acase_insensitiv\
e:!0,contains:[e\
.HASH_COMMENT_MO\
DE,{className:\x22m\
eta\x22,variants:[{\
\x0abegin:\x22^TAP ver\
sion (\x5c\x5cd+)$\x22},{\
begin:\x22^1\x5c\x5c.\x5c\x5c.(\
\x5c\x5cd+)$\x22}]},{begi\
n:/---$/,\x0aend:\x22\x5c\
\x5c.\x5c\x5c.\x5c\x5c.$\x22,subLa\
nguage:\x22yaml\x22,re\
levance:0},{clas\
sName:\x22number\x22,\x0a\
begin:\x22 (\x5c\x5cd+) \x22\
},{className:\x22sy\
mbol\x22,variants:[\
{begin:\x22^ok\x22},{b\
egin:\x22^not ok\x22}]\
\x0a}]})})();hljs.r\
egisterLanguage(\
\x22tap\x22,e)})();/*!\
 `tcl` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22;return e=>{\x0ac\
onst a=e.regex,t\
=/[a-zA-Z_][a-zA\
-Z0-9_]*/,r={cla\
ssName:\x22number\x22,\
\x0avariants:[e.BIN\
ARY_NUMBER_MODE,\
e.C_NUMBER_MODE]\
};return{name:\x22T\
cl\x22,\x0aaliases:[\x22t\
k\x22],\x0akeywords:[\x22\
after\x22,\x22append\x22,\
\x22apply\x22,\x22array\x22,\
\x22auto_execok\x22,\x22a\
uto_import\x22,\x22aut\
o_load\x22,\x22auto_mk\
index\x22,\x22auto_mki\
ndex_old\x22,\x22auto_\
qualify\x22,\x22auto_r\
eset\x22,\x22bgerror\x22,\
\x22binary\x22,\x22break\x22\
,\x22catch\x22,\x22cd\x22,\x22c\
han\x22,\x22clock\x22,\x22cl\
ose\x22,\x22concat\x22,\x22c\
ontinue\x22,\x22dde\x22,\x22\
dict\x22,\x22encoding\x22\
,\x22eof\x22,\x22error\x22,\x22\
eval\x22,\x22exec\x22,\x22ex\
it\x22,\x22expr\x22,\x22fblo\
cked\x22,\x22fconfigur\
e\x22,\x22fcopy\x22,\x22file\
\x22,\x22fileevent\x22,\x22f\
ilename\x22,\x22flush\x22\
,\x22for\x22,\x22foreach\x22\
,\x22format\x22,\x22gets\x22\
,\x22glob\x22,\x22global\x22\
,\x22history\x22,\x22http\
\x22,\x22if\x22,\x22incr\x22,\x22i\
nfo\x22,\x22interp\x22,\x22j\
oin\x22,\x22lappend|10\
\x22,\x22lassign|10\x22,\x22\
lindex|10\x22,\x22lins\
ert|10\x22,\x22list\x22,\x22\
llength|10\x22,\x22loa\
d\x22,\x22lrange|10\x22,\x22\
lrepeat|10\x22,\x22lre\
place|10\x22,\x22lreve\
rse|10\x22,\x22lsearch\
|10\x22,\x22lset|10\x22,\x22\
lsort|10\x22,\x22mathf\
unc\x22,\x22mathop\x22,\x22m\
emory\x22,\x22msgcat\x22,\
\x22namespace\x22,\x22ope\
n\x22,\x22package\x22,\x22pa\
rray\x22,\x22pid\x22,\x22pkg\
::create\x22,\x22pkg_m\
kIndex\x22,\x22platfor\
m\x22,\x22platform::sh\
ell\x22,\x22proc\x22,\x22put\
s\x22,\x22pwd\x22,\x22read\x22,\
\x22refchan\x22,\x22regex\
p\x22,\x22registry\x22,\x22r\
egsub|10\x22,\x22renam\
e\x22,\x22return\x22,\x22saf\
e\x22,\x22scan\x22,\x22seek\x22\
,\x22set\x22,\x22socket\x22,\
\x22source\x22,\x22split\x22\
,\x22string\x22,\x22subst\
\x22,\x22switch\x22,\x22tcl_\
endOfWord\x22,\x22tcl_\
findLibrary\x22,\x22tc\
l_startOfNextWor\
d\x22,\x22tcl_startOfP\
reviousWord\x22,\x22tc\
l_wordBreakAfter\
\x22,\x22tcl_wordBreak\
Before\x22,\x22tcltest\
\x22,\x22tclvars\x22,\x22tel\
l\x22,\x22time\x22,\x22tm\x22,\x22\
trace\x22,\x22unknown\x22\
,\x22unload\x22,\x22unset\
\x22,\x22update\x22,\x22uple\
vel\x22,\x22upvar\x22,\x22va\
riable\x22,\x22vwait\x22,\
\x22while\x22],\x0acontai\
ns:[e.COMMENT(\x22;\
[ \x5c\x5ct]*#\x22,\x22$\x22),e\
.COMMENT(\x22^[ \x5c\x5ct\
]*#\x22,\x22$\x22),{\x0abegi\
nKeywords:\x22proc\x22\
,end:\x22[\x5c\x5c{]\x22,exc\
ludeEnd:!0,conta\
ins:[{className:\
\x22title\x22,\x0abegin:\x22\
[ \x5c\x5ct\x5c\x5cn\x5c\x5cr]+(::\
)?[a-zA-Z_]((::)\
?[a-zA-Z0-9_])*\x22\
,end:\x22[ \x5c\x5ct\x5c\x5cn\x5c\x5c\
r]\x22,\x0aendsWithPar\
ent:!0,excludeEn\
d:!0}]},{classNa\
me:\x22variable\x22,va\
riants:[{\x0abegin:\
a.concat(/\x5c$/,a.\
optional(/::/),t\
,\x22(::\x22,t,\x22)*\x22)},\
{\x0abegin:\x22\x5c\x5c$\x5c\x5c{(\
::)?[a-zA-Z_]((:\
:)?[a-zA-Z0-9_])\
*\x22,end:\x22\x5c\x5c}\x22,con\
tains:[r]}]},{\x0ac\
lassName:\x22string\
\x22,contains:[e.BA\
CKSLASH_ESCAPE],\
\x0avariants:[e.inh\
erit(e.QUOTE_STR\
ING_MODE,{illega\
l:null})]},r]}}}\
)()\x0a;hljs.regist\
erLanguage(\x22tcl\x22\
,e)})();/*! `thr\
ift` grammar com\
piled for Highli\
ght.js 11.10.0 *\
/\x0a(()=>{var e=((\
)=>{\x22use strict\x22\
;return e=>{\x0acon\
st t=[\x22bool\x22,\x22by\
te\x22,\x22i16\x22,\x22i32\x22,\
\x22i64\x22,\x22double\x22,\x22\
string\x22,\x22binary\x22\
];return{\x0aname:\x22\
Thrift\x22,keywords\
:{\x0akeyword:[\x22nam\
espace\x22,\x22const\x22,\
\x22typedef\x22,\x22struc\
t\x22,\x22enum\x22,\x22servi\
ce\x22,\x22exception\x22,\
\x22void\x22,\x22oneway\x22,\
\x22set\x22,\x22list\x22,\x22ma\
p\x22,\x22required\x22,\x22o\
ptional\x22],\x0atype:\
t,literal:\x22true \
false\x22},\x0acontain\
s:[e.QUOTE_STRIN\
G_MODE,e.NUMBER_\
MODE,e.C_LINE_CO\
MMENT_MODE,e.C_B\
LOCK_COMMENT_MOD\
E,{\x0aclassName:\x22c\
lass\x22,beginKeywo\
rds:\x22struct enum\
 service excepti\
on\x22,end:/\x5c{/,\x0ail\
legal:/\x5cn/,conta\
ins:[e.inherit(e\
.TITLE_MODE,{sta\
rts:{endsWithPar\
ent:!0,\x0aexcludeE\
nd:!0}})]},{begi\
n:\x22\x5c\x5cb(set|list|\
map)\x5c\x5cs*<\x22,keywo\
rds:{\x0atype:[...t\
,\x22set\x22,\x22list\x22,\x22m\
ap\x22]},end:\x22>\x22,co\
ntains:[\x22self\x22]}\
]}}})()\x0a;hljs.re\
gisterLanguage(\x22\
thrift\x22,e)})();/\
*! `tp` grammar \
compiled for Hig\
hlight.js 11.10.\
0 */\x0a(()=>{var O\
=(()=>{\x22use stri\
ct\x22;return O=>{c\
onst e={classNam\
e:\x22number\x22,\x0abegi\
n:\x22[1-9][0-9]*\x22,\
relevance:0},R={\
className:\x22symbo\
l\x22,begin:\x22:[^\x5c\x5c]\
]+\x22};return{\x0anam\
e:\x22TP\x22,keywords:\
{\x0akeyword:[\x22ABOR\
T\x22,\x22ACC\x22,\x22ADJUST\
\x22,\x22AND\x22,\x22AP_LD\x22,\
\x22BREAK\x22,\x22CALL\x22,\x22\
CNT\x22,\x22COL\x22,\x22COND\
ITION\x22,\x22CONFIG\x22,\
\x22DA\x22,\x22DB\x22,\x22DIV\x22,\
\x22DETECT\x22,\x22ELSE\x22,\
\x22END\x22,\x22ENDFOR\x22,\x22\
ERR_NUM\x22,\x22ERROR_\
PROG\x22,\x22FINE\x22,\x22FO\
R\x22,\x22GP\x22,\x22GUARD\x22,\
\x22INC\x22,\x22IF\x22,\x22JMP\x22\
,\x22LINEAR_MAX_SPE\
ED\x22,\x22LOCK\x22,\x22MOD\x22\
,\x22MONITOR\x22,\x22OFFS\
ET\x22,\x22Offset\x22,\x22OR\
\x22,\x22OVERRIDE\x22,\x22PA\
USE\x22,\x22PREG\x22,\x22PTH\
\x22,\x22RT_LD\x22,\x22RUN\x22,\
\x22SELECT\x22,\x22SKIP\x22,\
\x22Skip\x22,\x22TA\x22,\x22TB\x22\
,\x22TO\x22,\x22TOOL_OFFS\
ET\x22,\x22Tool_Offset\
\x22,\x22UF\x22,\x22UT\x22,\x22UFR\
AME_NUM\x22,\x22UTOOL_\
NUM\x22,\x22UNLOCK\x22,\x22W\
AIT\x22,\x22X\x22,\x22Y\x22,\x22Z\x22\
,\x22W\x22,\x22P\x22,\x22R\x22,\x22ST\
RLEN\x22,\x22SUBSTR\x22,\x22\
FINDSTR\x22,\x22VOFFSE\
T\x22,\x22PROG\x22,\x22ATTR\x22\
,\x22MN\x22,\x22POS\x22],\x0ali\
teral:[\x22ON\x22,\x22OFF\
\x22,\x22max_speed\x22,\x22L\
POS\x22,\x22JPOS\x22,\x22ENA\
BLE\x22,\x22DISABLE\x22,\x22\
START\x22,\x22STOP\x22,\x22R\
ESET\x22]\x0a},contain\
s:[{className:\x22b\
uilt_in\x22,\x0abegin:\
\x22(AR|P|PAYLOAD|P\
R|R|SR|RSR|LBL|V\
R|UALM|MESSAGE|U\
TOOL|UFRAME|TIME\
R|TIMER_OVERFLOW\
|JOINT_MAX_SPEED\
|RESUME_PROG|DIA\
G_REC)\x5c\x5c[\x22,\x0aend:\
\x22\x5c\x5c]\x22,contains:[\
\x22self\x22,e,R]},{cl\
assName:\x22built_i\
n\x22,\x0abegin:\x22(AI|A\
O|DI|DO|F|RI|RO|\
UI|UO|GI|GO|SI|S\
O)\x5c\x5c[\x22,end:\x22\x5c\x5c]\x22\
,\x0acontains:[\x22sel\
f\x22,e,O.QUOTE_STR\
ING_MODE,R]},{cl\
assName:\x22keyword\
\x22,\x0abegin:\x22/(PROG\
|ATTR|MN|POS|END\
)\x5c\x5cb\x22},{classNam\
e:\x22keyword\x22,\x0abeg\
in:\x22(CALL|RUN|PO\
INT_LOGIC|LBL)\x5c\x5c\
b\x22},{className:\x22\
keyword\x22,\x0abegin:\
\x22\x5c\x5cb(ACC|CNT|Ski\
p|Offset|PSPD|RT\
_LD|AP_LD|Tool_O\
ffset)\x22},{\x0aclass\
Name:\x22number\x22,\x0ab\
egin:\x22\x5c\x5cd+(sec|m\
sec|mm/sec|cm/mi\
n|inch/min|deg/s\
ec|mm|in|cm)?\x5c\x5cb\
\x22,relevance:0\x0a},\
O.COMMENT(\x22//\x22,\x22\
[;$]\x22),O.COMMENT\
(\x22!\x22,\x22[;$]\x22),O.C\
OMMENT(\x22--eg:\x22,\x22\
$\x22),O.QUOTE_STRI\
NG_MODE,{\x0aclassN\
ame:\x22string\x22,beg\
in:\x22'\x22,end:\x22'\x22},\
O.C_NUMBER_MODE,\
{className:\x22vari\
able\x22,\x0abegin:\x22\x5c\x5c\
$[A-Za-z0-9_]+\x22}\
]}}})();hljs.reg\
isterLanguage(\x22t\
p\x22,O)})();/*! `t\
wig` grammar com\
piled for Highli\
ght.js 11.10.0 *\
/\x0a(()=>{var e=((\
)=>{\x22use strict\x22\
;return e=>{\x0acon\
st a=e.regex,t=[\
\x22absolute_url\x22,\x22\
asset|0\x22,\x22asset_\
version\x22,\x22attrib\
ute\x22,\x22block\x22,\x22co\
nstant\x22,\x22control\
ler|0\x22,\x22country_\
timezones\x22,\x22csrf\
_token\x22,\x22cycle\x22,\
\x22date\x22,\x22dump\x22,\x22e\
xpression\x22,\x22form\
|0\x22,\x22form_end\x22,\x22\
form_errors\x22,\x22fo\
rm_help\x22,\x22form_l\
abel\x22,\x22form_rest\
\x22,\x22form_row\x22,\x22fo\
rm_start\x22,\x22form_\
widget\x22,\x22html_cl\
asses\x22,\x22include\x22\
,\x22is_granted\x22,\x22l\
ogout_path\x22,\x22log\
out_url\x22,\x22max\x22,\x22\
min\x22,\x22parent\x22,\x22p\
ath|0\x22,\x22random\x22,\
\x22range\x22,\x22relativ\
e_path\x22,\x22render\x22\
,\x22render_esi\x22,\x22s\
ource\x22,\x22template\
_from_string\x22,\x22u\
rl|0\x22]\x0a;let r=[\x22\
apply\x22,\x22autoesca\
pe\x22,\x22block\x22,\x22cac\
he\x22,\x22deprecated\x22\
,\x22do\x22,\x22embed\x22,\x22e\
xtends\x22,\x22filter\x22\
,\x22flush\x22,\x22for\x22,\x22\
form_theme\x22,\x22fro\
m\x22,\x22if\x22,\x22import\x22\
,\x22include\x22,\x22macr\
o\x22,\x22sandbox\x22,\x22se\
t\x22,\x22stopwatch\x22,\x22\
trans\x22,\x22trans_de\
fault_domain\x22,\x22t\
ranschoice\x22,\x22use\
\x22,\x22verbatim\x22,\x22wi\
th\x22]\x0a;r=r.concat\
(r.map((e=>\x22end\x22\
+e)));const n={s\
cope:\x22string\x22,va\
riants:[{begin:/\
'/,\x0aend:/'/},{be\
gin:/\x22/,end:/\x22/}\
]},o={scope:\x22num\
ber\x22,match:/\x5cd+/\
},s={begin:/\x5c(/,\
\x0aend:/\x5c)/,exclud\
eBegin:!0,exclud\
eEnd:!0,contains\
:[n,o]},c={\x0abegi\
nKeywords:t.join\
(\x22 \x22),keywords:{\
name:t},relevanc\
e:0,contains:[s]\
},m={\x0amatch:/\x5c|(\
?=[A-Za-z_]+:?)/\
,beginScope:\x22pun\
ctuation\x22,releva\
nce:0,contains:[\
{\x0amatch:/[A-Za-z\
_]+:?/,\x0akeywords\
:[\x22abs\x22,\x22abbr_cl\
ass\x22,\x22abbr_metho\
d\x22,\x22batch\x22,\x22capi\
talize\x22,\x22column\x22\
,\x22convert_encodi\
ng\x22,\x22country_nam\
e\x22,\x22currency_nam\
e\x22,\x22currency_sym\
bol\x22,\x22data_uri\x22,\
\x22date\x22,\x22date_mod\
ify\x22,\x22default\x22,\x22\
escape\x22,\x22file_ex\
cerpt\x22,\x22file_lin\
k\x22,\x22file_relativ\
e\x22,\x22filter\x22,\x22fir\
st\x22,\x22format\x22,\x22fo\
rmat_args\x22,\x22form\
at_args_as_text\x22\
,\x22format_currenc\
y\x22,\x22format_date\x22\
,\x22format_datetim\
e\x22,\x22format_file\x22\
,\x22format_file_fr\
om_text\x22,\x22format\
_number\x22,\x22format\
_time\x22,\x22html_to_\
markdown\x22,\x22human\
ize\x22,\x22inky_to_ht\
ml\x22,\x22inline_css\x22\
,\x22join\x22,\x22json_en\
code\x22,\x22keys\x22,\x22la\
nguage_name\x22,\x22la\
st\x22,\x22length\x22,\x22lo\
cale_name\x22,\x22lowe\
r\x22,\x22map\x22,\x22markdo\
wn\x22,\x22markdown_to\
_html\x22,\x22merge\x22,\x22\
nl2br\x22,\x22number_f\
ormat\x22,\x22raw\x22,\x22re\
duce\x22,\x22replace\x22,\
\x22reverse\x22,\x22round\
\x22,\x22slice\x22,\x22slug\x22\
,\x22sort\x22,\x22spacele\
ss\x22,\x22split\x22,\x22str\
iptags\x22,\x22timezon\
e_name\x22,\x22title\x22,\
\x22trans\x22,\x22transch\
oice\x22,\x22trim\x22,\x22u|\
0\x22,\x22upper\x22,\x22url_\
encode\x22,\x22yaml_du\
mp\x22,\x22yaml_encode\
\x22]\x0a}]},i=(e,{rel\
evance:t})=>({be\
ginScope:{1:\x22tem\
plate-tag\x22,3:\x22na\
me\x22},\x0arelevance:\
t||2,endScope:\x22t\
emplate-tag\x22,beg\
in:[/\x5c{%/,/\x5cs*/,\
a.either(...e)],\
\x0aend:/%\x5c}/,keywo\
rds:\x22in\x22,contain\
s:[m,c,n,o]}),l=\
i(r,{relevance:2\
\x0a}),_=i([/[a-z_]\
+/],{relevance:1\
});return{name:\x22\
Twig\x22,aliases:[\x22\
craftcms\x22],\x0acase\
_insensitive:!0,\
subLanguage:\x22xml\
\x22,contains:[e.CO\
MMENT(/\x5c{#/,/#\x5c}\
/),l,_,{\x0aclassNa\
me:\x22template-var\
iable\x22,begin:/\x5c{\
\x5c{/,end:/\x5c}\x5c}/,c\
ontains:[\x22self\x22,\
m,c,n,o]\x0a}]}}})(\
);hljs.registerL\
anguage(\x22twig\x22,e\
)})();/*! `types\
cript` grammar c\
ompiled for High\
light.js 11.10.0\
 */\x0a(()=>{var e=\
(()=>{\x22use stric\
t\x22\x0a;const e=\x22[A-\
Za-z$_][0-9A-Za-\
z$_]*\x22,n=[\x22as\x22,\x22\
in\x22,\x22of\x22,\x22if\x22,\x22f\
or\x22,\x22while\x22,\x22fin\
ally\x22,\x22var\x22,\x22new\
\x22,\x22function\x22,\x22do\
\x22,\x22return\x22,\x22void\
\x22,\x22else\x22,\x22break\x22\
,\x22catch\x22,\x22instan\
ceof\x22,\x22with\x22,\x22th\
row\x22,\x22case\x22,\x22def\
ault\x22,\x22try\x22,\x22swi\
tch\x22,\x22continue\x22,\
\x22typeof\x22,\x22delete\
\x22,\x22let\x22,\x22yield\x22,\
\x22const\x22,\x22class\x22,\
\x22debugger\x22,\x22asyn\
c\x22,\x22await\x22,\x22stat\
ic\x22,\x22import\x22,\x22fr\
om\x22,\x22export\x22,\x22ex\
tends\x22],a=[\x22true\
\x22,\x22false\x22,\x22null\x22\
,\x22undefined\x22,\x22Na\
N\x22,\x22Infinity\x22],t\
=[\x22Object\x22,\x22Func\
tion\x22,\x22Boolean\x22,\
\x22Symbol\x22,\x22Math\x22,\
\x22Date\x22,\x22Number\x22,\
\x22BigInt\x22,\x22String\
\x22,\x22RegExp\x22,\x22Arra\
y\x22,\x22Float32Array\
\x22,\x22Float64Array\x22\
,\x22Int8Array\x22,\x22Ui\
nt8Array\x22,\x22Uint8\
ClampedArray\x22,\x22I\
nt16Array\x22,\x22Int3\
2Array\x22,\x22Uint16A\
rray\x22,\x22Uint32Arr\
ay\x22,\x22BigInt64Arr\
ay\x22,\x22BigUint64Ar\
ray\x22,\x22Set\x22,\x22Map\x22\
,\x22WeakSet\x22,\x22Weak\
Map\x22,\x22ArrayBuffe\
r\x22,\x22SharedArrayB\
uffer\x22,\x22Atomics\x22\
,\x22DataView\x22,\x22JSO\
N\x22,\x22Promise\x22,\x22Ge\
nerator\x22,\x22Genera\
torFunction\x22,\x22As\
yncFunction\x22,\x22Re\
flect\x22,\x22Proxy\x22,\x22\
Intl\x22,\x22WebAssemb\
ly\x22],s=[\x22Error\x22,\
\x22EvalError\x22,\x22Int\
ernalError\x22,\x22Ran\
geError\x22,\x22Refere\
nceError\x22,\x22Synta\
xError\x22,\x22TypeErr\
or\x22,\x22URIError\x22],\
r=[\x22setInterval\x22\
,\x22setTimeout\x22,\x22c\
learInterval\x22,\x22c\
learTimeout\x22,\x22re\
quire\x22,\x22exports\x22\
,\x22eval\x22,\x22isFinit\
e\x22,\x22isNaN\x22,\x22pars\
eFloat\x22,\x22parseIn\
t\x22,\x22decodeURI\x22,\x22\
decodeURICompone\
nt\x22,\x22encodeURI\x22,\
\x22encodeURICompon\
ent\x22,\x22escape\x22,\x22u\
nescape\x22],c=[\x22ar\
guments\x22,\x22this\x22,\
\x22super\x22,\x22console\
\x22,\x22window\x22,\x22docu\
ment\x22,\x22localStor\
age\x22,\x22sessionSto\
rage\x22,\x22module\x22,\x22\
global\x22],i=[].co\
ncat(r,t,s)\x0a;fun\
ction o(o){const\
 l=o.regex,d=e,b\
={begin:/<[A-Za-\
z0-9\x5c\x5c._:-]+/,\x0ae\
nd:/\x5c/[A-Za-z0-9\
\x5c\x5c._:-]+>|\x5c/>/,i\
sTrulyOpeningTag\
:(e,n)=>{\x0aconst \
a=e[0].length+e.\
index,t=e.input[\
a]\x0a;if(\x22<\x22===t||\
\x22,\x22===t)return v\
oid n.ignoreMatc\
h();let s\x0a;\x22>\x22==\
=t&&(((e,{after:\
n})=>{const a=\x22<\
/\x22+e[0].slice(1)\
\x0a;return-1!==e.i\
nput.indexOf(a,n\
)})(e,{after:a})\
||n.ignoreMatch(\
))\x0a;const r=e.in\
put.substring(a)\
\x0a;((s=r.match(/^\
\x5cs*=/))||(s=r.ma\
tch(/^\x5cs+extends\
\x5cs+/))&&0===s.in\
dex)&&n.ignoreMa\
tch()\x0a}},g={$pat\
tern:e,keyword:n\
,literal:a,built\
_in:i,\x22variable.\
language\x22:c\x0a},u=\
\x22[0-9](_?[0-9])*\
\x22,m=`\x5c\x5c.(${u})`,\
E=\x220|[1-9](_?[0-\
9])*|0[0-7]*[89]\
[0-9]*\x22,A={\x0aclas\
sName:\x22number\x22,v\
ariants:[{\x0abegin\
:`(\x5c\x5cb(${E})((${\
m})|\x5c\x5c.)?|(${m})\
)[eE][+-]?(${u})\
\x5c\x5cb`},{\x0abegin:`\x5c\
\x5cb(${E})\x5c\x5cb((${m\
})\x5c\x5cb|\x5c\x5c.)?|(${m\
})\x5c\x5cb`},{\x0abegin:\
\x22\x5c\x5cb(0|[1-9](_?[\
0-9])*)n\x5c\x5cb\x22},{\x0a\
begin:\x22\x5c\x5cb0[xX][\
0-9a-fA-F](_?[0-\
9a-fA-F])*n?\x5c\x5cb\x22\
},{\x0abegin:\x22\x5c\x5cb0[\
bB][0-1](_?[0-1]\
)*n?\x5c\x5cb\x22},{begin\
:\x22\x5c\x5cb0[oO][0-7](\
_?[0-7])*n?\x5c\x5cb\x22}\
,{\x0abegin:\x22\x5c\x5cb0[0\
-7]+n?\x5c\x5cb\x22}],rel\
evance:0},y={cla\
ssName:\x22subst\x22,b\
egin:\x22\x5c\x5c$\x5c\x5c{\x22,\x0ae\
nd:\x22\x5c\x5c}\x22,keyword\
s:g,contains:[]}\
,p={begin:\x22.?htm\
l`\x22,end:\x22\x22,start\
s:{end:\x22`\x22,\x0aretu\
rnEnd:!1,contain\
s:[o.BACKSLASH_E\
SCAPE,y],subLang\
uage:\x22xml\x22}},N={\
\x0abegin:\x22.?css`\x22,\
end:\x22\x22,starts:{e\
nd:\x22`\x22,returnEnd\
:!1,\x0acontains:[o\
.BACKSLASH_ESCAP\
E,y],subLanguage\
:\x22css\x22}},f={begi\
n:\x22.?gql`\x22,end:\x22\
\x22,\x0astarts:{end:\x22\
`\x22,returnEnd:!1,\
contains:[o.BACK\
SLASH_ESCAPE,y],\
\x0asubLanguage:\x22gr\
aphql\x22}},_={clas\
sName:\x22string\x22,b\
egin:\x22`\x22,end:\x22`\x22\
,\x0acontains:[o.BA\
CKSLASH_ESCAPE,y\
]},h={className:\
\x22comment\x22,\x0avaria\
nts:[o.COMMENT(/\
\x5c/\x5c*\x5c*(?!\x5c/)/,\x22\x5c\
\x5c*/\x22,{relevance:\
0,contains:[{\x0abe\
gin:\x22(?=@[A-Za-z\
]+)\x22,relevance:0\
,contains:[{clas\
sName:\x22doctag\x22,\x0a\
begin:\x22@[A-Za-z]\
+\x22},{className:\x22\
type\x22,begin:\x22\x5c\x5c{\
\x22,end:\x22\x5c\x5c}\x22,excl\
udeEnd:!0,\x0aexclu\
deBegin:!0,relev\
ance:0},{classNa\
me:\x22variable\x22,be\
gin:d+\x22(?=\x5c\x5cs*(-\
)|$)\x22,\x0aendsParen\
t:!0,relevance:0\
},{begin:/(?=[^\x5c\
n])\x5cs/,relevance\
:0}]}]\x0a}),o.C_BL\
OCK_COMMENT_MODE\
,o.C_LINE_COMMEN\
T_MODE]\x0a},S=[o.A\
POS_STRING_MODE,\
o.QUOTE_STRING_M\
ODE,p,N,f,_,{mat\
ch:/\x5c$\x5cd+/},A]\x0a;\
y.contains=S.con\
cat({begin:/\x5c{/,\
end:/\x5c}/,keyword\
s:g,contains:[\x22s\
elf\x22].concat(S)\x0a\
});const v=[].co\
ncat(h,y.contain\
s),w=v.concat([{\
begin:/(\x5cs*)\x5c(/,\
end:/\x5c)/,\x0akeywor\
ds:g,contains:[\x22\
self\x22].concat(v)\
}]),R={className\
:\x22params\x22,begin:\
/(\x5cs*)\x5c(/,\x0aend:/\
\x5c)/,excludeBegin\
:!0,excludeEnd:!\
0,keywords:g,con\
tains:w},k={vari\
ants:[{\x0amatch:[/\
class/,/\x5cs+/,d,/\
\x5cs+/,/extends/,/\
\x5cs+/,l.concat(d,\
\x22(\x22,l.concat(/\x5c.\
/,d),\x22)*\x22)],\x0asco\
pe:{1:\x22keyword\x22,\
3:\x22title.class\x22,\
5:\x22keyword\x22,7:\x22t\
itle.class.inher\
ited\x22}},{\x0amatch:\
[/class/,/\x5cs+/,d\
],scope:{1:\x22keyw\
ord\x22,3:\x22title.cl\
ass\x22}}]},x={rele\
vance:0,\x0amatch:l\
.either(/\x5cbJSON/\
,/\x5cb[A-Z][a-z]+(\
[A-Z][a-z]*|\x5cd)*\
/,/\x5cb[A-Z]{2,}([\
A-Z][a-z]+|\x5cd)+(\
[A-Z][a-z]*)*/,/\
\x5cb[A-Z]{2,}[a-z]\
+([A-Z][a-z]+|\x5cd\
)*([A-Z][a-z]*)*\
/),\x0aclassName:\x22t\
itle.class\x22,keyw\
ords:{_:[...t,..\
.s]}},O={variant\
s:[{\x0amatch:[/fun\
ction/,/\x5cs+/,d,/\
(?=\x5cs*\x5c()/]},{ma\
tch:[/function/,\
/\x5cs*(?=\x5c()/]}],\x0a\
className:{1:\x22ke\
yword\x22,3:\x22title.\
function\x22},label\
:\x22func.def\x22,cont\
ains:[R],\x0aillega\
l:/%/},I={\x0amatch\
:l.concat(/\x5cb/,(\
C=[...r,\x22super\x22,\
\x22import\x22].map((e\
=>e+\x22\x5c\x5cs*\x5c\x5c(\x22)),\
\x0al.concat(\x22(?!\x22,\
C.join(\x22|\x22),\x22)\x22)\
),d,l.lookahead(\
/\x5cs*\x5c(/)),\x0aclass\
Name:\x22title.func\
tion\x22,relevance:\
0};var C;const T\
={\x0abegin:l.conca\
t(/\x5c./,l.lookahe\
ad(l.concat(d,/(\
?![0-9A-Za-z$_(]\
)/))),end:d,\x0aexc\
ludeBegin:!0,key\
words:\x22prototype\
\x22,className:\x22pro\
perty\x22,relevance\
:0},M={\x0amatch:[/\
get|set/,/\x5cs+/,d\
,/(?=\x5c()/],class\
Name:{1:\x22keyword\
\x22,3:\x22title.funct\
ion\x22},\x0acontains:\
[{begin:/\x5c(\x5c)/},\
R]\x0a},B=\x22(\x5c\x5c([^()\
]*(\x5c\x5c([^()]*(\x5c\x5c(\
[^()]*\x5c\x5c)[^()]*)\
*\x5c\x5c)[^()]*)*\x5c\x5c)|\
\x22+o.UNDERSCORE_I\
DENT_RE+\x22)\x5c\x5cs*=>\
\x22,$={\x0amatch:[/co\
nst|var|let/,/\x5cs\
+/,d,/\x5cs*/,/=\x5cs*\
/,/(async\x5cs*)?/,\
l.lookahead(B)],\
\x0akeywords:\x22async\
\x22,className:{1:\x22\
keyword\x22,3:\x22titl\
e.function\x22},con\
tains:[R]}\x0a;retu\
rn{name:\x22JavaScr\
ipt\x22,aliases:[\x22j\
s\x22,\x22jsx\x22,\x22mjs\x22,\x22\
cjs\x22],keywords:g\
,exports:{\x0aPARAM\
S_CONTAINS:w,CLA\
SS_REFERENCE:x},\
illegal:/#(?![$_\
A-z])/,\x0acontains\
:[o.SHEBANG({lab\
el:\x22shebang\x22,bin\
ary:\x22node\x22,relev\
ance:5}),{\x0alabel\
:\x22use_strict\x22,cl\
assName:\x22meta\x22,r\
elevance:10,\x0abeg\
in:/^\x5cs*['\x22]use \
(strict|asm)['\x22]\
/\x0a},o.APOS_STRIN\
G_MODE,o.QUOTE_S\
TRING_MODE,p,N,f\
,_,h,{match:/\x5c$\x5c\
d+/},A,x,{\x0aclass\
Name:\x22attr\x22,begi\
n:d+l.lookahead(\
\x22:\x22),relevance:0\
},$,{\x0abegin:\x22(\x22+\
o.RE_STARTERS_RE\
+\x22|\x5c\x5cb(case|retu\
rn|throw)\x5c\x5cb)\x5c\x5cs\
*\x22,\x0akeywords:\x22re\
turn throw case\x22\
,relevance:0,con\
tains:[h,o.REGEX\
P_MODE,{\x0aclassNa\
me:\x22function\x22,be\
gin:B,returnBegi\
n:!0,end:\x22\x5c\x5cs*=>\
\x22,contains:[{\x0acl\
assName:\x22params\x22\
,variants:[{begi\
n:o.UNDERSCORE_I\
DENT_RE,relevanc\
e:0},{\x0aclassName\
:null,begin:/\x5c(\x5c\
s*\x5c)/,skip:!0},{\
begin:/(\x5cs*)\x5c(/,\
end:/\x5c)/,\x0aexclud\
eBegin:!0,exclud\
eEnd:!0,keywords\
:g,contains:w}]}\
]},{begin:/,/,re\
levance:0\x0a},{mat\
ch:/\x5cs+/,relevan\
ce:0},{variants:\
[{begin:\x22<>\x22,end\
:\x22</>\x22},{\x0amatch:\
/<[A-Za-z0-9\x5c\x5c._\
:-]+\x5cs*\x5c/>/},{be\
gin:b.begin,\x0a\x22on\
:begin\x22:b.isTrul\
yOpeningTag,end:\
b.end}],subLangu\
age:\x22xml\x22,contai\
ns:[{\x0abegin:b.be\
gin,end:b.end,sk\
ip:!0,contains:[\
\x22self\x22]}]}]},O,{\
\x0abeginKeywords:\x22\
while if switch \
catch for\x22},{\x0abe\
gin:\x22\x5c\x5cb(?!funct\
ion)\x22+o.UNDERSCO\
RE_IDENT_RE+\x22\x5c\x5c(\
[^()]*(\x5c\x5c([^()]*\
(\x5c\x5c([^()]*\x5c\x5c)[^(\
)]*)*\x5c\x5c)[^()]*)*\
\x5c\x5c)\x5c\x5cs*\x5c\x5c{\x22,\x0aret\
urnBegin:!0,labe\
l:\x22func.def\x22,con\
tains:[R,o.inher\
it(o.TITLE_MODE,\
{begin:d,\x0aclassN\
ame:\x22title.funct\
ion\x22})]},{match:\
/\x5c.\x5c.\x5c./,relevan\
ce:0},T,{match:\x22\
\x5c\x5c$\x22+d,\x0arelevanc\
e:0},{match:[/\x5cb\
constructor(?=\x5cs\
*\x5c()/],className\
:{1:\x22title.funct\
ion\x22},\x0acontains:\
[R]},I,{relevanc\
e:0,match:/\x5cb[A-\
Z][A-Z_0-9]+\x5cb/,\
\x0aclassName:\x22vari\
able.constant\x22},\
k,M,{match:/\x5c$[(\
.]/}]}}return t=\
>{\x0aconst s=o(t),\
r=e,l=[\x22any\x22,\x22vo\
id\x22,\x22number\x22,\x22bo\
olean\x22,\x22string\x22,\
\x22object\x22,\x22never\x22\
,\x22symbol\x22,\x22bigin\
t\x22,\x22unknown\x22],d=\
{\x0abegin:[/namesp\
ace/,/\x5cs+/,t.IDE\
NT_RE],beginScop\
e:{1:\x22keyword\x22,3\
:\x22title.class\x22}\x0a\
},b={beginKeywor\
ds:\x22interface\x22,e\
nd:/\x5c{/,excludeE\
nd:!0,keywords:{\
\x0akeyword:\x22interf\
ace extends\x22,bui\
lt_in:l},contain\
s:[s.exports.CLA\
SS_REFERENCE]\x0a},\
g={$pattern:e,\x0ak\
eyword:n.concat(\
[\x22type\x22,\x22interfa\
ce\x22,\x22public\x22,\x22pr\
ivate\x22,\x22protecte\
d\x22,\x22implements\x22,\
\x22declare\x22,\x22abstr\
act\x22,\x22readonly\x22,\
\x22enum\x22,\x22override\
\x22,\x22satisfies\x22]),\
\x0aliteral:a,built\
_in:i.concat(l),\
\x22variable.langua\
ge\x22:c},u={classN\
ame:\x22meta\x22,\x0abegi\
n:\x22@\x22+r},m=(e,n,\
a)=>{const t=e.c\
ontains.findInde\
x((e=>e.label===\
n))\x0a;if(-1===t)t\
hrow Error(\x22can \
not find mode to\
 replace\x22);e.con\
tains.splice(t,1\
,a)}\x0a;Object.ass\
ign(s.keywords,g\
),s.exports.PARA\
MS_CONTAINS.push\
(u)\x0a;const E=s.c\
ontains.find((e=\
>\x22attr\x22===e.clas\
sName))\x0a;return \
s.exports.PARAMS\
_CONTAINS.push([\
s.exports.CLASS_\
REFERENCE,E]),\x0as\
.contains=s.cont\
ains.concat([u,d\
,b]),\x0am(s,\x22sheba\
ng\x22,t.SHEBANG())\
,m(s,\x22use_strict\
\x22,{className:\x22me\
ta\x22,relevance:10\
,\x0abegin:/^\x5cs*['\x22\
]use strict['\x22]/\
\x0a}),s.contains.f\
ind((e=>\x22func.de\
f\x22===e.label)).r\
elevance=0,Objec\
t.assign(s,{\x0anam\
e:\x22TypeScript\x22,a\
liases:[\x22ts\x22,\x22ts\
x\x22,\x22mts\x22,\x22cts\x22]}\
),s}})()\x0a;hljs.r\
egisterLanguage(\
\x22typescript\x22,e)}\
)();/*! `vala` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>({name:\x22Val\
a\x22,keywords:{\x0ake\
yword:\x22char ucha\
r unichar int ui\
nt long ulong sh\
ort ushort int8 \
int16 int32 int6\
4 uint8 uint16 u\
int32 uint64 flo\
at double bool s\
truct enum strin\
g void weak unow\
ned owned async \
signal static ab\
stract interface\
 override virtua\
l delegate if wh\
ile do for forea\
ch else switch c\
ase break defaul\
t return try cat\
ch public privat\
e protected inte\
rnal using new t\
his get set cons\
t stdout stdin s\
tderr var\x22,\x0abuil\
t_in:\x22DBus GLib \
CCode Gee Object\
 Gtk Posix\x22,lite\
ral:\x22false true \
null\x22},\x0acontains\
:[{className:\x22cl\
ass\x22,beginKeywor\
ds:\x22class interf\
ace namespace\x22,e\
nd:/\x5c{/,\x0aexclude\
End:!0,illegal:\x22\
[^,:\x5c\x5cn\x5c\x5cs\x5c\x5c.]\x22,\
contains:[e.UNDE\
RSCORE_TITLE_MOD\
E]\x0a},e.C_LINE_CO\
MMENT_MODE,e.C_B\
LOCK_COMMENT_MOD\
E,{className:\x22st\
ring\x22,begin:'\x22\x22\x22\
',\x0aend:'\x22\x22\x22',rel\
evance:5},e.APOS\
_STRING_MODE,e.Q\
UOTE_STRING_MODE\
,e.C_NUMBER_MODE\
,{\x0aclassName:\x22me\
ta\x22,begin:\x22^#\x22,e\
nd:\x22$\x22}]})})();h\
ljs.registerLang\
uage(\x22vala\x22,e)})\
();/*! `vbnet` g\
rammar compiled \
for Highlight.js\
 11.10.0 */\x0a(()=\
>{var e=(()=>{\x22u\
se strict\x22;retur\
n e=>{\x0aconst n=e\
.regex,t=/\x5cd{1,2\
}\x5c/\x5cd{1,2}\x5c/\x5cd{4\
}/,a=/\x5cd{4}-\x5cd{1\
,2}-\x5cd{1,2}/,i=/\
(\x5cd|1[012])(:\x5cd+\
){0,2} *(AM|PM)/\
,s=/\x5cd{1,2}(:\x5cd{\
1,2}){1,2}/,r={\x0a\
className:\x22liter\
al\x22,variants:[{b\
egin:n.concat(/#\
 */,n.either(a,t\
),/ *#/)},{\x0abegi\
n:n.concat(/# */\
,s,/ *#/)},{begi\
n:n.concat(/# */\
,i,/ *#/)},{\x0abeg\
in:n.concat(/# *\
/,n.either(a,t),\
/ +/,n.either(i,\
s),/ *#/)}]\x0a},l=\
e.COMMENT(/'''/,\
/$/,{contains:[{\
className:\x22docta\
g\x22,begin:/<\x5c/?/,\
end:/>/}]\x0a}),o=e\
.COMMENT(null,/$\
/,{variants:[{be\
gin:/'/},{begin:\
/([\x5ct ]|^)REM(?=\
\x5cs)/}]})\x0a;return\
{name:\x22Visual Ba\
sic .NET\x22,aliase\
s:[\x22vb\x22],case_in\
sensitive:!0,\x0acl\
assNameAliases:{\
label:\x22symbol\x22},\
keywords:{\x0akeywo\
rd:\x22addhandler a\
lias aggregate a\
nsi as async ass\
embly auto binar\
y by byref byval\
 call case catch\
 class compare c\
onst continue cu\
stom declare def\
ault delegate di\
m distinct do ea\
ch equals else e\
lseif end enum e\
rase error event\
 exit explicit f\
inally for frien\
d from function \
get global goto \
group handles if\
 implements impo\
rts in inherits \
interface into i\
terator join key\
 let lib loop me\
 mid module must\
inherit mustover\
ride mybase mycl\
ass namespace na\
rrowing new next\
 notinheritable \
notoverridable o\
f off on operato\
r option optiona\
l order overload\
s overridable ov\
errides paramarr\
ay partial prese\
rve private prop\
erty protected p\
ublic raiseevent\
 readonly redim \
removehandler re\
sume return sele\
ct set shadows s\
hared skip stati\
c step stop stru\
cture strict sub\
 synclock take t\
ext then throw t\
o try unicode un\
til using when w\
here while widen\
ing with witheve\
nts writeonly yi\
eld\x22,\x0abuilt_in:\x22\
addressof and an\
dalso await dire\
ctcast gettype g\
etxmlnamespace i\
s isfalse isnot \
istrue like mod \
nameof new not o\
r orelse trycast\
 typeof xor cboo\
l cbyte cchar cd\
ate cdbl cdec ci\
nt clng cobj csb\
yte cshort csng \
cstr cuint culng\
 cushort\x22,\x0atype:\
\x22boolean byte ch\
ar date decimal \
double integer l\
ong object sbyte\
 short single st\
ring uinteger ul\
ong ushort\x22,\x0alit\
eral:\x22true false\
 nothing\x22},\x0aille\
gal:\x22//|\x5c\x5c{|\x5c\x5c}|\
endif|gosub|vari\
ant|wend|^\x5c\x5c$ \x22,\
contains:[{\x0aclas\
sName:\x22string\x22,b\
egin:/\x22(\x22\x22|[^/n]\
)\x22C\x5cb/},{classNa\
me:\x22string\x22,begi\
n:/\x22/,\x0aend:/\x22/,i\
llegal:/\x5cn/,cont\
ains:[{begin:/\x22\x22\
/}]},r,{classNam\
e:\x22number\x22,relev\
ance:0,\x0avariants\
:[{begin:/\x5cb\x5cd[\x5c\
d_]*((\x5c.[\x5cd_]+(E\
[+-]?[\x5cd_]+)?)|(\
E[+-]?[\x5cd_]+))[R\
FD@!#]?/\x0a},{begi\
n:/\x5cb\x5cd[\x5cd_]*((U\
?[SIL])|[%&])?/}\
,{begin:/&H[\x5cdA-\
F_]+((U?[SIL])|[\
%&])?/},{\x0abegin:\
/&O[0-7_]+((U?[S\
IL])|[%&])?/},{b\
egin:/&B[01_]+((\
U?[SIL])|[%&])?/\
}]},{\x0aclassName:\
\x22label\x22,begin:/^\
\x5cw+:/},l,o,{clas\
sName:\x22meta\x22,\x0abe\
gin:/[\x5ct ]*#(con\
st|disable|else|\
elseif|enable|en\
d|externalsource\
|if|region)\x5cb/,\x0a\
end:/$/,keywords\
:{\x0akeyword:\x22cons\
t disable else e\
lseif enable end\
 externalsource \
if region then\x22}\
,\x0acontains:[o]}]\
}}})();hljs.regi\
sterLanguage(\x22vb\
net\x22,e)})();/*! \
`vbscript` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{\x0aconst t=e.reg\
ex,r=[\x22lcase\x22,\x22m\
onth\x22,\x22vartype\x22,\
\x22instrrev\x22,\x22ubou\
nd\x22,\x22setlocale\x22,\
\x22getobject\x22,\x22rgb\
\x22,\x22getref\x22,\x22stri\
ng\x22,\x22weekdayname\
\x22,\x22rnd\x22,\x22dateadd\
\x22,\x22monthname\x22,\x22n\
ow\x22,\x22day\x22,\x22minut\
e\x22,\x22isarray\x22,\x22cb\
ool\x22,\x22round\x22,\x22fo\
rmatcurrency\x22,\x22c\
onversions\x22,\x22csn\
g\x22,\x22timevalue\x22,\x22\
second\x22,\x22year\x22,\x22\
space\x22,\x22abs\x22,\x22cl\
ng\x22,\x22timeserial\x22\
,\x22fixs\x22,\x22len\x22,\x22a\
sc\x22,\x22isempty\x22,\x22m\
aths\x22,\x22dateseria\
l\x22,\x22atn\x22,\x22timer\x22\
,\x22isobject\x22,\x22fil\
ter\x22,\x22weekday\x22,\x22\
datevalue\x22,\x22ccur\
\x22,\x22isdate\x22,\x22inst\
r\x22,\x22datediff\x22,\x22f\
ormatdatetime\x22,\x22\
replace\x22,\x22isnull\
\x22,\x22right\x22,\x22sgn\x22,\
\x22array\x22,\x22snumeri\
c\x22,\x22log\x22,\x22cdbl\x22,\
\x22hex\x22,\x22chr\x22,\x22lbo\
und\x22,\x22msgbox\x22,\x22u\
case\x22,\x22getlocale\
\x22,\x22cos\x22,\x22cdate\x22,\
\x22cbyte\x22,\x22rtrim\x22,\
\x22join\x22,\x22hour\x22,\x22o\
ct\x22,\x22typename\x22,\x22\
trim\x22,\x22strcomp\x22,\
\x22int\x22,\x22createobj\
ect\x22,\x22loadpictur\
e\x22,\x22tan\x22,\x22format\
number\x22,\x22mid\x22,\x22s\
plit\x22,\x22cint\x22,\x22si\
n\x22,\x22datepart\x22,\x22l\
trim\x22,\x22sqr\x22,\x22tim\
e\x22,\x22derived\x22,\x22ev\
al\x22,\x22date\x22,\x22form\
atpercent\x22,\x22exp\x22\
,\x22inputbox\x22,\x22lef\
t\x22,\x22ascw\x22,\x22chrw\x22\
,\x22regexp\x22,\x22cstr\x22\
,\x22err\x22]\x0a;return{\
name:\x22VBScript\x22,\
aliases:[\x22vbs\x22],\
case_insensitive\
:!0,keywords:{\x0ak\
eyword:[\x22call\x22,\x22\
class\x22,\x22const\x22,\x22\
dim\x22,\x22do\x22,\x22loop\x22\
,\x22erase\x22,\x22execut\
e\x22,\x22executegloba\
l\x22,\x22exit\x22,\x22for\x22,\
\x22each\x22,\x22next\x22,\x22f\
unction\x22,\x22if\x22,\x22t\
hen\x22,\x22else\x22,\x22on\x22\
,\x22error\x22,\x22option\
\x22,\x22explicit\x22,\x22ne\
w\x22,\x22private\x22,\x22pr\
operty\x22,\x22let\x22,\x22g\
et\x22,\x22public\x22,\x22ra\
ndomize\x22,\x22redim\x22\
,\x22rem\x22,\x22select\x22,\
\x22case\x22,\x22set\x22,\x22st\
op\x22,\x22sub\x22,\x22while\
\x22,\x22wend\x22,\x22with\x22,\
\x22end\x22,\x22to\x22,\x22else\
if\x22,\x22is\x22,\x22or\x22,\x22x\
or\x22,\x22and\x22,\x22not\x22,\
\x22class_initializ\
e\x22,\x22class_termin\
ate\x22,\x22default\x22,\x22\
preserve\x22,\x22in\x22,\x22\
me\x22,\x22byval\x22,\x22byr\
ef\x22,\x22step\x22,\x22resu\
me\x22,\x22goto\x22],\x0abui\
lt_in:[\x22server\x22,\
\x22response\x22,\x22requ\
est\x22,\x22scriptengi\
ne\x22,\x22scriptengin\
ebuildversion\x22,\x22\
scriptenginemino\
rversion\x22,\x22scrip\
tenginemajorvers\
ion\x22],\x0aliteral:[\
\x22true\x22,\x22false\x22,\x22\
null\x22,\x22nothing\x22,\
\x22empty\x22]},illega\
l:\x22//\x22,contains:\
[{\x0abegin:t.conca\
t(t.either(...r)\
,\x22\x5c\x5cs*\x5c\x5c(\x22),rele\
vance:0,keywords\
:{built_in:r}\x0a},\
e.inherit(e.QUOT\
E_STRING_MODE,{c\
ontains:[{begin:\
'\x22\x22'}]}),e.COMME\
NT(/'/,/$/,{\x0arel\
evance:0}),e.C_N\
UMBER_MODE]}}})(\
);hljs.registerL\
anguage(\x22vbscrip\
t\x22,e)})();/*! `v\
bscript-html` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>({name:\x22VBSc\
ript in HTML\x22,\x0as\
ubLanguage:\x22xml\x22\
,contains:[{begi\
n:\x22<%\x22,end:\x22%>\x22,\
subLanguage:\x22vbs\
cript\x22}]})})()\x0a;\
hljs.registerLan\
guage(\x22vbscript-\
html\x22,e)})();/*!\
 `verilog` gramm\
ar compiled for \
Highlight.js 11.\
10.0 */\x0a(()=>{va\
r e=(()=>{\x22use s\
trict\x22;return e=\
>{\x0aconst n=e.reg\
ex,t=[\x22begin_key\
words\x22,\x22celldefi\
ne\x22,\x22default_net\
type\x22,\x22default_d\
ecay_time\x22,\x22defa\
ult_trireg_stren\
gth\x22,\x22define\x22,\x22d\
elay_mode_distri\
buted\x22,\x22delay_mo\
de_path\x22,\x22delay_\
mode_unit\x22,\x22dela\
y_mode_zero\x22,\x22el\
se\x22,\x22elsif\x22,\x22end\
_keywords\x22,\x22endc\
elldefine\x22,\x22endi\
f\x22,\x22ifdef\x22,\x22ifnd\
ef\x22,\x22include\x22,\x22l\
ine\x22,\x22nounconnec\
ted_drive\x22,\x22prag\
ma\x22,\x22resetall\x22,\x22\
timescale\x22,\x22unco\
nnected_drive\x22,\x22\
undef\x22,\x22undefine\
all\x22]\x0a;return{na\
me:\x22Verilog\x22,ali\
ases:[\x22v\x22,\x22sv\x22,\x22\
svh\x22],case_insen\
sitive:!1,keywor\
ds:{\x0a$pattern:/\x5c\
$?[\x5cw]+(\x5c$[\x5cw]+)\
*/,\x0akeyword:[\x22ac\
cept_on\x22,\x22alias\x22\
,\x22always\x22,\x22alway\
s_comb\x22,\x22always_\
ff\x22,\x22always_latc\
h\x22,\x22and\x22,\x22assert\
\x22,\x22assign\x22,\x22assu\
me\x22,\x22automatic\x22,\
\x22before\x22,\x22begin\x22\
,\x22bind\x22,\x22bins\x22,\x22\
binsof\x22,\x22bit\x22,\x22b\
reak\x22,\x22buf|0\x22,\x22b\
ufif0\x22,\x22bufif1\x22,\
\x22byte\x22,\x22case\x22,\x22c\
asex\x22,\x22casez\x22,\x22c\
ell\x22,\x22chandle\x22,\x22\
checker\x22,\x22class\x22\
,\x22clocking\x22,\x22cmo\
s\x22,\x22config\x22,\x22con\
st\x22,\x22constraint\x22\
,\x22context\x22,\x22cont\
inue\x22,\x22cover\x22,\x22c\
overgroup\x22,\x22cove\
rpoint\x22,\x22cross\x22,\
\x22deassign\x22,\x22defa\
ult\x22,\x22defparam\x22,\
\x22design\x22,\x22disabl\
e\x22,\x22dist\x22,\x22do\x22,\x22\
edge\x22,\x22else\x22,\x22en\
d\x22,\x22endcase\x22,\x22en\
dchecker\x22,\x22endcl\
ass\x22,\x22endclockin\
g\x22,\x22endconfig\x22,\x22\
endfunction\x22,\x22en\
dgenerate\x22,\x22endg\
roup\x22,\x22endinterf\
ace\x22,\x22endmodule\x22\
,\x22endpackage\x22,\x22e\
ndprimitive\x22,\x22en\
dprogram\x22,\x22endpr\
operty\x22,\x22endspec\
ify\x22,\x22endsequenc\
e\x22,\x22endtable\x22,\x22e\
ndtask\x22,\x22enum\x22,\x22\
event\x22,\x22eventual\
ly\x22,\x22expect\x22,\x22ex\
port\x22,\x22extends\x22,\
\x22extern\x22,\x22final\x22\
,\x22first_match\x22,\x22\
for\x22,\x22force\x22,\x22fo\
reach\x22,\x22forever\x22\
,\x22fork\x22,\x22forkjoi\
n\x22,\x22function\x22,\x22g\
enerate|5\x22,\x22genv\
ar\x22,\x22global\x22,\x22hi\
ghz0\x22,\x22highz1\x22,\x22\
if\x22,\x22iff\x22,\x22ifnon\
e\x22,\x22ignore_bins\x22\
,\x22illegal_bins\x22,\
\x22implements\x22,\x22im\
plies\x22,\x22import\x22,\
\x22incdir\x22,\x22includ\
e\x22,\x22initial\x22,\x22in\
out\x22,\x22input\x22,\x22in\
side\x22,\x22instance\x22\
,\x22int\x22,\x22integer\x22\
,\x22interconnect\x22,\
\x22interface\x22,\x22int\
ersect\x22,\x22join\x22,\x22\
join_any\x22,\x22join_\
none\x22,\x22large\x22,\x22l\
et\x22,\x22liblist\x22,\x22l\
ibrary\x22,\x22local\x22,\
\x22localparam\x22,\x22lo\
gic\x22,\x22longint\x22,\x22\
macromodule\x22,\x22ma\
tches\x22,\x22medium\x22,\
\x22modport\x22,\x22modul\
e\x22,\x22nand\x22,\x22neged\
ge\x22,\x22nettype\x22,\x22n\
ew\x22,\x22nexttime\x22,\x22\
nmos\x22,\x22nor\x22,\x22nos\
howcancelled\x22,\x22n\
ot\x22,\x22notif0\x22,\x22no\
tif1\x22,\x22or\x22,\x22outp\
ut\x22,\x22package\x22,\x22p\
acked\x22,\x22paramete\
r\x22,\x22pmos\x22,\x22posed\
ge\x22,\x22primitive\x22,\
\x22priority\x22,\x22prog\
ram\x22,\x22property\x22,\
\x22protected\x22,\x22pul\
l0\x22,\x22pull1\x22,\x22pul\
ldown\x22,\x22pullup\x22,\
\x22pulsestyle_onde\
tect\x22,\x22pulsestyl\
e_onevent\x22,\x22pure\
\x22,\x22rand\x22,\x22randc\x22\
,\x22randcase\x22,\x22ran\
dsequence\x22,\x22rcmo\
s\x22,\x22real\x22,\x22realt\
ime\x22,\x22ref\x22,\x22reg\x22\
,\x22reject_on\x22,\x22re\
lease\x22,\x22repeat\x22,\
\x22restrict\x22,\x22retu\
rn\x22,\x22rnmos\x22,\x22rpm\
os\x22,\x22rtran\x22,\x22rtr\
anif0\x22,\x22rtranif1\
\x22,\x22s_always\x22,\x22s_\
eventually\x22,\x22s_n\
exttime\x22,\x22s_unti\
l\x22,\x22s_until_with\
\x22,\x22scalared\x22,\x22se\
quence\x22,\x22shortin\
t\x22,\x22shortreal\x22,\x22\
showcancelled\x22,\x22\
signed\x22,\x22small\x22,\
\x22soft\x22,\x22solve\x22,\x22\
specify\x22,\x22specpa\
ram\x22,\x22static\x22,\x22s\
tring\x22,\x22strong\x22,\
\x22strong0\x22,\x22stron\
g1\x22,\x22struct\x22,\x22su\
per\x22,\x22supply0\x22,\x22\
supply1\x22,\x22sync_a\
ccept_on\x22,\x22sync_\
reject_on\x22,\x22tabl\
e\x22,\x22tagged\x22,\x22tas\
k\x22,\x22this\x22,\x22throu\
ghout\x22,\x22time\x22,\x22t\
imeprecision\x22,\x22t\
imeunit\x22,\x22tran\x22,\
\x22tranif0\x22,\x22trani\
f1\x22,\x22tri\x22,\x22tri0\x22\
,\x22tri1\x22,\x22triand\x22\
,\x22trior\x22,\x22trireg\
\x22,\x22type\x22,\x22typede\
f\x22,\x22union\x22,\x22uniq\
ue\x22,\x22unique0\x22,\x22u\
nsigned\x22,\x22until\x22\
,\x22until_with\x22,\x22u\
ntyped\x22,\x22use\x22,\x22u\
wire\x22,\x22var\x22,\x22vec\
tored\x22,\x22virtual\x22\
,\x22void\x22,\x22wait\x22,\x22\
wait_order\x22,\x22wan\
d\x22,\x22weak\x22,\x22weak0\
\x22,\x22weak1\x22,\x22while\
\x22,\x22wildcard\x22,\x22wi\
re\x22,\x22with\x22,\x22with\
in\x22,\x22wor\x22,\x22xnor\x22\
,\x22xor\x22],\x0aliteral\
:[\x22null\x22],\x0abuilt\
_in:[\x22$finish\x22,\x22\
$stop\x22,\x22$exit\x22,\x22\
$fatal\x22,\x22$error\x22\
,\x22$warning\x22,\x22$in\
fo\x22,\x22$realtime\x22,\
\x22$time\x22,\x22$printt\
imescale\x22,\x22$bits\
toreal\x22,\x22$bitsto\
shortreal\x22,\x22$ito\
r\x22,\x22$signed\x22,\x22$c\
ast\x22,\x22$bits\x22,\x22$s\
time\x22,\x22$timeform\
at\x22,\x22$realtobits\
\x22,\x22$shortrealtob\
its\x22,\x22$rtoi\x22,\x22$u\
nsigned\x22,\x22$asser\
ton\x22,\x22$assertkil\
l\x22,\x22$assertpasso\
n\x22,\x22$assertfailo\
n\x22,\x22$assertnonva\
cuouson\x22,\x22$asser\
toff\x22,\x22$assertco\
ntrol\x22,\x22$assertp\
assoff\x22,\x22$assert\
failoff\x22,\x22$asser\
tvacuousoff\x22,\x22$i\
sunbounded\x22,\x22$sa\
mpled\x22,\x22$fell\x22,\x22\
$changed\x22,\x22$past\
_gclk\x22,\x22$fell_gc\
lk\x22,\x22$changed_gc\
lk\x22,\x22$rising_gcl\
k\x22,\x22$steady_gclk\
\x22,\x22$coverage_con\
trol\x22,\x22$coverage\
_get\x22,\x22$coverage\
_save\x22,\x22$set_cov\
erage_db_name\x22,\x22\
$rose\x22,\x22$stable\x22\
,\x22$past\x22,\x22$rose_\
gclk\x22,\x22$stable_g\
clk\x22,\x22$future_gc\
lk\x22,\x22$falling_gc\
lk\x22,\x22$changing_g\
clk\x22,\x22$display\x22,\
\x22$coverage_get_m\
ax\x22,\x22$coverage_m\
erge\x22,\x22$get_cove\
rage\x22,\x22$load_cov\
erage_db\x22,\x22$type\
name\x22,\x22$unpacked\
_dimensions\x22,\x22$l\
eft\x22,\x22$low\x22,\x22$in\
crement\x22,\x22$clog2\
\x22,\x22$ln\x22,\x22$log10\x22\
,\x22$exp\x22,\x22$sqrt\x22,\
\x22$pow\x22,\x22$floor\x22,\
\x22$ceil\x22,\x22$sin\x22,\x22\
$cos\x22,\x22$tan\x22,\x22$c\
ountbits\x22,\x22$oneh\
ot\x22,\x22$isunknown\x22\
,\x22$fatal\x22,\x22$warn\
ing\x22,\x22$dimension\
s\x22,\x22$right\x22,\x22$hi\
gh\x22,\x22$size\x22,\x22$as\
in\x22,\x22$acos\x22,\x22$at\
an\x22,\x22$atan2\x22,\x22$h\
ypot\x22,\x22$sinh\x22,\x22$\
cosh\x22,\x22$tanh\x22,\x22$\
asinh\x22,\x22$acosh\x22,\
\x22$atanh\x22,\x22$count\
ones\x22,\x22$onehot0\x22\
,\x22$error\x22,\x22$info\
\x22,\x22$random\x22,\x22$di\
st_chi_square\x22,\x22\
$dist_erlang\x22,\x22$\
dist_exponential\
\x22,\x22$dist_normal\x22\
,\x22$dist_poisson\x22\
,\x22$dist_t\x22,\x22$dis\
t_uniform\x22,\x22$q_i\
nitialize\x22,\x22$q_r\
emove\x22,\x22$q_exam\x22\
,\x22$async$and$arr\
ay\x22,\x22$async$nand\
$array\x22,\x22$async$\
or$array\x22,\x22$asyn\
c$nor$array\x22,\x22$s\
ync$and$array\x22,\x22\
$sync$nand$array\
\x22,\x22$sync$or$arra\
y\x22,\x22$sync$nor$ar\
ray\x22,\x22$q_add\x22,\x22$\
q_full\x22,\x22$psprin\
tf\x22,\x22$async$and$\
plane\x22,\x22$async$n\
and$plane\x22,\x22$asy\
nc$or$plane\x22,\x22$a\
sync$nor$plane\x22,\
\x22$sync$and$plane\
\x22,\x22$sync$nand$pl\
ane\x22,\x22$sync$or$p\
lane\x22,\x22$sync$nor\
$plane\x22,\x22$system\
\x22,\x22$display\x22,\x22$d\
isplayb\x22,\x22$displ\
ayh\x22,\x22$displayo\x22\
,\x22$strobe\x22,\x22$str\
obeb\x22,\x22$strobeh\x22\
,\x22$strobeo\x22,\x22$wr\
ite\x22,\x22$readmemb\x22\
,\x22$readmemh\x22,\x22$w\
ritememh\x22,\x22$valu\
e$plusargs\x22,\x22$du\
mpvars\x22,\x22$dumpon\
\x22,\x22$dumplimit\x22,\x22\
$dumpports\x22,\x22$du\
mpportson\x22,\x22$dum\
pportslimit\x22,\x22$w\
riteb\x22,\x22$writeh\x22\
,\x22$writeo\x22,\x22$mon\
itor\x22,\x22$monitorb\
\x22,\x22$monitorh\x22,\x22$\
monitoro\x22,\x22$writ\
ememb\x22,\x22$dumpfil\
e\x22,\x22$dumpoff\x22,\x22$\
dumpall\x22,\x22$dumpf\
lush\x22,\x22$dumpport\
soff\x22,\x22$dumpport\
sall\x22,\x22$dumpport\
sflush\x22,\x22$fclose\
\x22,\x22$fdisplay\x22,\x22$\
fdisplayb\x22,\x22$fdi\
splayh\x22,\x22$fdispl\
ayo\x22,\x22$fstrobe\x22,\
\x22$fstrobeb\x22,\x22$fs\
trobeh\x22,\x22$fstrob\
eo\x22,\x22$swrite\x22,\x22$\
swriteb\x22,\x22$swrit\
eh\x22,\x22$swriteo\x22,\x22\
$fscanf\x22,\x22$fread\
\x22,\x22$fseek\x22,\x22$ffl\
ush\x22,\x22$feof\x22,\x22$f\
open\x22,\x22$fwrite\x22,\
\x22$fwriteb\x22,\x22$fwr\
iteh\x22,\x22$fwriteo\x22\
,\x22$fmonitor\x22,\x22$f\
monitorb\x22,\x22$fmon\
itorh\x22,\x22$fmonito\
ro\x22,\x22$sformat\x22,\x22\
$sformatf\x22,\x22$fge\
tc\x22,\x22$ungetc\x22,\x22$\
fgets\x22,\x22$sscanf\x22\
,\x22$rewind\x22,\x22$fte\
ll\x22,\x22$ferror\x22]\x0a}\
,contains:[e.C_B\
LOCK_COMMENT_MOD\
E,e.C_LINE_COMME\
NT_MODE,e.QUOTE_\
STRING_MODE,{\x0asc\
ope:\x22number\x22,con\
tains:[e.BACKSLA\
SH_ESCAPE],varia\
nts:[{\x0abegin:/\x5cb\
((\x5cd+'([bhodBHOD\
]))[0-9xzXZa-fA-\
F_]+)/},{\x0abegin:\
/\x5cB(('([bhodBHOD\
]))[0-9xzXZa-fA-\
F_]+)/},{begin:/\
\x5cb[0-9][0-9_]*/,\
\x0arelevance:0}]},\
{scope:\x22variable\
\x22,variants:[{beg\
in:\x22#\x5c\x5c((?!param\
eter).+\x5c\x5c)\x22},{\x0ab\
egin:\x22\x5c\x5c.\x5c\x5cw+\x22,r\
elevance:0}]},{s\
cope:\x22variable.c\
onstant\x22,\x0amatch:\
n.concat(/`/,n.e\
ither(\x22__FILE__\x22\
,\x22__LINE__\x22))},{\
scope:\x22meta\x22,\x0abe\
gin:n.concat(/`/\
,n.either(...t))\
,end:/$|\x5c/\x5c/|\x5c/\x5c\
*/,returnEnd:!0,\
keywords:t}]}}\x0a}\
)();hljs.registe\
rLanguage(\x22veril\
og\x22,e)})();/*! `\
vhdl` grammar co\
mpiled for Highl\
ight.js 11.10.0 \
*/\x0a(()=>{var e=(\
()=>{\x22use strict\
\x22;return e=>{\x0aco\
nst r=\x22\x5c\x5cd(_|\x5c\x5cd\
)*\x22,t=\x22[eE][-+]?\
\x22+r,n=\x22\x5c\x5cb(\x22+r+\x22\
#\x5c\x5cw+(\x5c\x5c.\x5c\x5cw+)?#\
(\x22+t+\x22)?|\x22+r+\x22(\x5c\
\x5c.\x22+r+\x22)?(\x22+t+\x22)\
?)\x22\x0a;return{name\
:\x22VHDL\x22,case_ins\
ensitive:!0,keyw\
ords:{\x0akeyword:[\
\x22abs\x22,\x22access\x22,\x22\
after\x22,\x22alias\x22,\x22\
all\x22,\x22and\x22,\x22arch\
itecture\x22,\x22array\
\x22,\x22assert\x22,\x22assu\
me\x22,\x22assume_guar\
antee\x22,\x22attribut\
e\x22,\x22begin\x22,\x22bloc\
k\x22,\x22body\x22,\x22buffe\
r\x22,\x22bus\x22,\x22case\x22,\
\x22component\x22,\x22con\
figuration\x22,\x22con\
stant\x22,\x22context\x22\
,\x22cover\x22,\x22discon\
nect\x22,\x22downto\x22,\x22\
default\x22,\x22else\x22,\
\x22elsif\x22,\x22end\x22,\x22e\
ntity\x22,\x22exit\x22,\x22f\
airness\x22,\x22file\x22,\
\x22for\x22,\x22force\x22,\x22f\
unction\x22,\x22genera\
te\x22,\x22generic\x22,\x22g\
roup\x22,\x22guarded\x22,\
\x22if\x22,\x22impure\x22,\x22i\
n\x22,\x22inertial\x22,\x22i\
nout\x22,\x22is\x22,\x22labe\
l\x22,\x22library\x22,\x22li\
nkage\x22,\x22literal\x22\
,\x22loop\x22,\x22map\x22,\x22m\
od\x22,\x22nand\x22,\x22new\x22\
,\x22next\x22,\x22nor\x22,\x22n\
ot\x22,\x22null\x22,\x22of\x22,\
\x22on\x22,\x22open\x22,\x22or\x22\
,\x22others\x22,\x22out\x22,\
\x22package\x22,\x22param\
eter\x22,\x22port\x22,\x22po\
stponed\x22,\x22proced\
ure\x22,\x22process\x22,\x22\
property\x22,\x22prote\
cted\x22,\x22pure\x22,\x22ra\
nge\x22,\x22record\x22,\x22r\
egister\x22,\x22reject\
\x22,\x22release\x22,\x22rem\
\x22,\x22report\x22,\x22rest\
rict\x22,\x22restrict_\
guarantee\x22,\x22retu\
rn\x22,\x22rol\x22,\x22ror\x22,\
\x22select\x22,\x22sequen\
ce\x22,\x22severity\x22,\x22\
shared\x22,\x22signal\x22\
,\x22sla\x22,\x22sll\x22,\x22sr\
a\x22,\x22srl\x22,\x22strong\
\x22,\x22subtype\x22,\x22the\
n\x22,\x22to\x22,\x22transpo\
rt\x22,\x22type\x22,\x22unaf\
fected\x22,\x22units\x22,\
\x22until\x22,\x22use\x22,\x22v\
ariable\x22,\x22view\x22,\
\x22vmode\x22,\x22vprop\x22,\
\x22vunit\x22,\x22wait\x22,\x22\
when\x22,\x22while\x22,\x22w\
ith\x22,\x22xnor\x22,\x22xor\
\x22],\x0abuilt_in:[\x22b\
oolean\x22,\x22bit\x22,\x22c\
haracter\x22,\x22integ\
er\x22,\x22time\x22,\x22dela\
y_length\x22,\x22natur\
al\x22,\x22positive\x22,\x22\
string\x22,\x22bit_vec\
tor\x22,\x22file_open_\
kind\x22,\x22file_open\
_status\x22,\x22std_lo\
gic\x22,\x22std_logic_\
vector\x22,\x22unsigne\
d\x22,\x22signed\x22,\x22boo\
lean_vector\x22,\x22in\
teger_vector\x22,\x22s\
td_ulogic\x22,\x22std_\
ulogic_vector\x22,\x22\
unresolved_unsig\
ned\x22,\x22u_unsigned\
\x22,\x22unresolved_si\
gned\x22,\x22u_signed\x22\
,\x22real_vector\x22,\x22\
time_vector\x22],\x0al\
iteral:[\x22false\x22,\
\x22true\x22,\x22note\x22,\x22w\
arning\x22,\x22error\x22,\
\x22failure\x22,\x22line\x22\
,\x22text\x22,\x22side\x22,\x22\
width\x22]\x0a},illega\
l:/\x5c{/,\x0acontains\
:[e.C_BLOCK_COMM\
ENT_MODE,e.COMME\
NT(\x22--\x22,\x22$\x22),e.Q\
UOTE_STRING_MODE\
,{\x0aclassName:\x22nu\
mber\x22,begin:n,re\
levance:0},{clas\
sName:\x22string\x22,\x0a\
begin:\x22'(U|X|0|1\
|Z|W|L|H|-)'\x22,co\
ntains:[e.BACKSL\
ASH_ESCAPE]},{\x0ac\
lassName:\x22symbol\
\x22,begin:\x22'[A-Za-\
z](_?[A-Za-z0-9]\
)*\x22,\x0acontains:[e\
.BACKSLASH_ESCAP\
E]}]}}})();hljs.\
registerLanguage\
(\x22vhdl\x22,e)})();/\
*! `vim` grammar\
 compiled for Hi\
ghlight.js 11.10\
.0 */\x0a(()=>{var \
e=(()=>{\x22use str\
ict\x22;return e=>(\
{name:\x22Vim Scrip\
t\x22,keywords:{\x0a$p\
attern:/[!#@\x5cw]+\
/,\x0akeyword:\x22N|0 \
P|0 X|0 a|0 ab a\
bc abo al am an|\
0 ar arga argd a\
rge argdo argg a\
rgl argu as au a\
ug aun b|0 bN ba\
 bad bd be bel b\
f bl bm bn bo bp\
 br brea breaka \
breakd breakl br\
o bufdo buffers \
bun bw c|0 cN cN\
f ca cabc caddb \
cad caddf cal ca\
t cb cc ccl cd c\
e cex cf cfir cg\
etb cgete cg cha\
nges chd che che\
ckt cl cla clo c\
m cmapc cme cn c\
new cnf cno cnor\
ea cnoreme co co\
l colo com comc \
comp con conf co\
pe cp cpf cq cr \
cs cst cu cuna c\
unme cw delm deb\
 debugg delc del\
f dif diffg diff\
o diffp diffpu d\
iffs diffthis di\
g di dl dell dj \
dli do doautoa d\
p dr ds dsp e|0 \
ea ec echoe echo\
h echom echon el\
 elsei em en end\
fo endf endt end\
w ene ex exe exi\
 exu f|0 files f\
ilet fin fina fi\
ni fir fix fo fo\
ldc foldd folddo\
c foldo for fu g\
o gr grepa gu gv\
 ha helpf helpg \
helpt hi hid his\
 ia iabc if ij i\
l im imapc ime i\
no inorea inorem\
e int is isp iu \
iuna iunme j|0 j\
u k|0 keepa kee \
keepj lN lNf l|0\
 lad laddb laddf\
 la lan lat lb l\
c lch lcl lcs le\
 lefta let lex l\
f lfir lgetb lge\
te lg lgr lgrepa\
 lh ll lla lli l\
mak lm lmapc lne\
 lnew lnf ln loa\
dk lo loc lockv \
lol lope lp lpf \
lr ls lt lu lua \
luad luaf lv lvi\
mgrepa lw m|0 ma\
 mak map mapc ma\
rks mat me menut\
 mes mk mks mksp\
 mkv mkvie mod m\
z mzf nbc nb nbs\
 new nm nmapc nm\
e nn nnoreme noa\
 no noh norea no\
reme norm nu nun\
 nunme ol o|0 om\
 omapc ome on on\
o onoreme opt ou\
 ounme ow p|0 pr\
ofd prof pro pro\
mptr pc ped pe p\
erld po popu pp \
pre prev ps pt p\
tN ptf ptj ptl p\
tn ptp ptr pts p\
u pw py3 python3\
 py3d py3f py py\
d pyf quita qa r\
ec red redi redr\
 redraws reg res\
 ret retu rew ri\
 rightb rub ruby\
d rubyf rund ru \
rv sN san sa sal\
 sav sb sbN sba \
sbf sbl sbm sbn \
sbp sbr scrip sc\
ripte scs se set\
f setg setl sf s\
fir sh sim sig s\
il sl sla sm sma\
p smapc sme sn s\
ni sno snor snor\
eme sor so spell\
d spe spelli spe\
llr spellu spell\
w sp spr sre st \
sta startg start\
r star stopi stj\
 sts sun sunm su\
nme sus sv sw sy\
 synti sync tN t\
abN tabc tabdo t\
abe tabf tabfir \
tabl tabm tabnew\
 tabn tabo tabp \
tabr tabs tab ta\
 tags tc tcld tc\
lf te tf th tj t\
l tm tn to tp tr\
 try ts tu u|0 u\
ndoj undol una u\
nh unl unlo unm \
unme uns up ve v\
erb vert vim vim\
grepa vi viu vie\
 vm vmapc vme vn\
e vn vnoreme vs \
vu vunme windo w\
|0 wN wa wh wi w\
inc winp wn wp w\
q wqa ws wu wv x\
|0 xa xmapc xm x\
me xn xnoreme xu\
 xunme y|0 z|0 ~\
 Next Print appe\
nd abbreviate ab\
clear aboveleft \
all amenu anorem\
enu args argadd \
argdelete argedi\
t argglobal argl\
ocal argument as\
cii autocmd augr\
oup aunmenu buff\
er bNext ball ba\
dd bdelete behav\
e belowright bfi\
rst blast bmodif\
ied bnext botrig\
ht bprevious bre\
wind break break\
add breakdel bre\
aklist browse bu\
nload bwipeout c\
hange cNext cNfi\
le cabbrev cabcl\
ear caddbuffer c\
addexpr caddfile\
 call catch cbuf\
fer cclose cente\
r cexpr cfile cf\
irst cgetbuffer \
cgetexpr cgetfil\
e chdir checkpat\
h checktime clis\
t clast close cm\
ap cmapclear cme\
nu cnext cnewer \
cnfile cnoremap \
cnoreabbrev cnor\
emenu copy colde\
r colorscheme co\
mmand comclear c\
ompiler continue\
 confirm copen c\
previous cpfile \
cquit crewind cs\
cope cstag cunma\
p cunabbrev cunm\
enu cwindow dele\
te delmarks debu\
g debuggreedy de\
lcommand delfunc\
tion diffupdate \
diffget diffoff \
diffpatch diffpu\
t diffsplit digr\
aphs display del\
etel djump dlist\
 doautocmd doaut\
oall deletep dro\
p dsearch dsplit\
 edit earlier ec\
ho echoerr echoh\
l echomsg else e\
lseif emenu endi\
f endfor endfunc\
tion endtry endw\
hile enew execut\
e exit exusage f\
ile filetype fin\
d finally finish\
 first fixdel fo\
ld foldclose fol\
ddoopen folddocl\
osed foldopen fu\
nction global go\
to grep grepadd \
gui gvim hardcop\
y help helpfind \
helpgrep helptag\
s highlight hide\
 history insert \
iabbrev iabclear\
 ijump ilist ima\
p imapclear imen\
u inoremap inore\
abbrev inoremenu\
 intro isearch i\
split iunmap iun\
abbrev iunmenu j\
oin jumps keepal\
t keepmarks keep\
jumps lNext lNfi\
le list laddexpr\
 laddbuffer ladd\
file last langua\
ge later lbuffer\
 lcd lchdir lclo\
se lcscope left \
leftabove lexpr \
lfile lfirst lge\
tbuffer lgetexpr\
 lgetfile lgrep \
lgrepadd lhelpgr\
ep llast llist l\
make lmap lmapcl\
ear lnext lnewer\
 lnfile lnoremap\
 loadkeymap load\
view lockmarks l\
ockvar lolder lo\
pen lprevious lp\
file lrewind lta\
g lunmap luado l\
uafile lvimgrep \
lvimgrepadd lwin\
dow move mark ma\
ke mapclear matc\
h menu menutrans\
late messages mk\
exrc mksession m\
kspell mkvimrc m\
kview mode mzsch\
eme mzfile nbclo\
se nbkey nbsart \
next nmap nmapcl\
ear nmenu nnorem\
ap nnoremenu noa\
utocmd noremap n\
ohlsearch noreab\
brev noremenu no\
rmal number nunm\
ap nunmenu oldfi\
les open omap om\
apclear omenu on\
ly onoremap onor\
emenu options ou\
nmap ounmenu own\
syntax print pro\
fdel profile pro\
mptfind promptre\
pl pclose pedit \
perl perldo pop \
popup ppop prese\
rve previous pse\
arch ptag ptNext\
 ptfirst ptjump \
ptlast ptnext pt\
previous ptrewin\
d ptselect put p\
wd py3do py3file\
 python pydo pyf\
ile quit quitall\
 qall read recov\
er redo redir re\
draw redrawstatu\
s registers resi\
ze retab return \
rewind right rig\
htbelow ruby rub\
ydo rubyfile run\
do runtime rvimi\
nfo substitute s\
Next sandbox sar\
gument sall save\
as sbuffer sbNex\
t sball sbfirst \
sblast sbmodifie\
d sbnext sbprevi\
ous sbrewind scr\
iptnames scripte\
ncoding scscope \
set setfiletype \
setglobal setloc\
al sfind sfirst \
shell simalt sig\
n silent sleep s\
last smagic smap\
clear smenu snex\
t sniff snomagic\
 snoremap snorem\
enu sort source \
spelldump spellg\
ood spellinfo sp\
ellrepall spellu\
ndo spellwrong s\
plit sprevious s\
rewind stop stag\
 startgreplace s\
tartreplace star\
tinsert stopinse\
rt stjump stsele\
ct sunhide sunma\
p sunmenu suspen\
d sview swapname\
 syntax syntime \
syncbind tNext t\
abNext tabclose \
tabedit tabfind \
tabfirst tablast\
 tabmove tabnext\
 tabonly tabprev\
ious tabrewind t\
ag tcl tcldo tcl\
file tearoff tfi\
rst throw tjump \
tlast tmenu tnex\
t topleft tprevi\
ous trewind tsel\
ect tunmenu undo\
 undojoin undoli\
st unabbreviate \
unhide unlet unl\
ockvar unmap unm\
enu unsilent upd\
ate vglobal vers\
ion verbose vert\
ical vimgrep vim\
grepadd visual v\
iusage view vmap\
 vmapclear vmenu\
 vnew vnoremap v\
noremenu vsplit \
vunmap vunmenu w\
rite wNext wall \
while winsize wi\
ncmd winpos wnex\
t wprevious wqal\
l wsverb wundo w\
viminfo xit xall\
 xmapclear xmap \
xmenu xnoremap x\
noremenu xunmap \
xunmenu yank\x22,\x0ab\
uilt_in:\x22synIDtr\
ans atan2 range \
matcharg did_fil\
etype asin feedk\
eys xor argv com\
plete_check add \
getwinposx getqf\
list getwinposy \
screencol clearm\
atches empty ext\
end getcmdpos mz\
eval garbagecoll\
ect setreg ceil \
sqrt diff_hlID i\
nputsecret get g\
etfperm getpid f\
ilewritable shif\
twidth max sinh \
isdirectory synI\
D system inputre\
store winline at\
an visualmode in\
putlist tabpagew\
innr round getre\
gtype mapcheck h\
asmapto histdel \
argidx findfile \
sha256 exists to\
upper getcmdline\
 taglist string \
getmatches bufnr\
 strftime winwid\
th bufexists str\
trans tabpagebuf\
list setcmdpos r\
emote_read print\
f setloclist get\
pos getline bufw\
innr float2nr le\
n getcmdtype dif\
f_filler luaeval\
 resolve libcall\
nr foldclosedend\
 reverse filter \
has_key bufname \
str2float strlen\
 setline getchar\
mod setbufvar in\
dex searchpos sh\
ellescape undofi\
le foldclosed se\
tqflist bufliste\
d strchars str2n\
r virtcol floor \
remove undotree \
remote_expr winh\
eight gettabwinv\
ar reltime curso\
r tabpagenr find\
dir localtime ac\
os getloclist se\
arch tanh matche\
nd rename gettab\
var strdisplaywi\
dth type abs py3\
eval setwinvar t\
olower wildmenum\
ode log10 spells\
uggest bufloaded\
 synconcealed ne\
xtnonblank serve\
r2client complet\
e settabwinvar e\
xecutable input \
wincol setmatche\
s getftype hlID \
inputsave search\
pair or screenro\
w line settabvar\
 histadd deepcop\
y strpart remote\
_peek and eval g\
etftime submatch\
 screenchar wins\
aveview matchadd\
 mkdir screenatt\
r getfontname li\
bcall reltimestr\
 getfsize winnr \
invert pow getbu\
fline byte2line \
soundfold repeat\
 fnameescape tag\
files sin strwid\
th spellbadword \
trunc maparg log\
 lispindent host\
name setpos glob\
path remote_fore\
ground getchar s\
ynIDattr fnamemo\
dify cscope_conn\
ection stridx wi\
nbufnr indent mi\
n complete_add n\
r2char searchpai\
rpos inputdialog\
 values matchlis\
t items hlexists\
 strridx browsed\
ir expand fmod p\
athshorten line2\
byte argc count \
getwinvar glob f\
oldtextresult ge\
treg foreground \
cosh matchdelete\
 has char2nr sim\
plify histget se\
archdecl iconv w\
inrestcmd pumvis\
ible writefile f\
oldlevel hasloca\
ldir keys cos ma\
tchstr foldtext \
histnr tan tempn\
ame getcwd bytei\
dx getbufvar isl\
ocked escape eve\
nthandler remote\
_send serverlist\
 winrestview syn\
stack pyeval pre\
vnonblank readfi\
le cindent filer\
eadable changenr\
 exp\x22\x0a},illegal:\
/;/,contains:[e.\
NUMBER_MODE,{cla\
ssName:\x22string\x22,\
begin:\x22'\x22,end:\x22'\
\x22,\x0aillegal:\x22\x5c\x5cn\x22\
},{className:\x22st\
ring\x22,begin:/\x22(\x5c\
\x5c\x22|\x5cn\x5c\x5c|[^\x22\x5cn])*\
\x22/\x0a},e.COMMENT('\
\x22',\x22$\x22),{classNa\
me:\x22variable\x22,be\
gin:/[bwtglsav]:\
[\x5cw\x5cd_]+/},{\x0abeg\
in:[/\x5cb(?:functi\
on|function!)/,/\
\x5cs+/,e.IDENT_RE]\
,className:{1:\x22k\
eyword\x22,\x0a3:\x22titl\
e\x22},end:\x22$\x22,rele\
vance:0,contains\
:[{className:\x22pa\
rams\x22,begin:\x22\x5c\x5c(\
\x22,\x0aend:\x22\x5c\x5c)\x22}]},\
{className:\x22symb\
ol\x22,begin:/<[\x5cw-\
]+>/}]})})()\x0a;hl\
js.registerLangu\
age(\x22vim\x22,e)})()\
;/*! `wasm` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar e=(()=>{\x22use \
strict\x22;return e\
=>{e.regex;const\
 a=e.COMMENT(/\x5c(\
;/,/;\x5c)/)\x0a;retur\
n a.contains.pus\
h(\x22self\x22),{name:\
\x22WebAssembly\x22,ke\
ywords:{$pattern\
:/[\x5cw.]+/,\x0akeywo\
rd:[\x22anyfunc\x22,\x22b\
lock\x22,\x22br\x22,\x22br_i\
f\x22,\x22br_table\x22,\x22c\
all\x22,\x22call_indir\
ect\x22,\x22data\x22,\x22dro\
p\x22,\x22elem\x22,\x22else\x22\
,\x22end\x22,\x22export\x22,\
\x22func\x22,\x22global.g\
et\x22,\x22global.set\x22\
,\x22local.get\x22,\x22lo\
cal.set\x22,\x22local.\
tee\x22,\x22get_global\
\x22,\x22get_local\x22,\x22g\
lobal\x22,\x22if\x22,\x22imp\
ort\x22,\x22local\x22,\x22lo\
op\x22,\x22memory\x22,\x22me\
mory.grow\x22,\x22memo\
ry.size\x22,\x22module\
\x22,\x22mut\x22,\x22nop\x22,\x22o\
ffset\x22,\x22param\x22,\x22\
result\x22,\x22return\x22\
,\x22select\x22,\x22set_g\
lobal\x22,\x22set_loca\
l\x22,\x22start\x22,\x22tabl\
e\x22,\x22tee_local\x22,\x22\
then\x22,\x22type\x22,\x22un\
reachable\x22]\x0a},co\
ntains:[e.COMMEN\
T(/;;/,/$/),a,{m\
atch:[/(?:offset\
|align)/,/\x5cs*/,/\
=/],\x0aclassName:{\
1:\x22keyword\x22,3:\x22o\
perator\x22}},{clas\
sName:\x22variable\x22\
,begin:/\x5c$[\x5cw_]+\
/},{\x0amatch:/(\x5c((\
?!;)|\x5c))+/,class\
Name:\x22punctuatio\
n\x22,relevance:0},\
{\x0abegin:[/(?:fun\
c|call|call_indi\
rect)/,/\x5cs+/,/\x5c$\
[^\x5cs)]+/],classN\
ame:{1:\x22keyword\x22\
,\x0a3:\x22title.funct\
ion\x22}},e.QUOTE_S\
TRING_MODE,{matc\
h:/(i32|i64|f32|\
f64)(?!\x5c.)/,\x0acla\
ssName:\x22type\x22},{\
className:\x22keywo\
rd\x22,\x0amatch:/\x5cb(f\
32|f64|i32|i64)(\
?:\x5c.(?:abs|add|a\
nd|ceil|clz|cons\
t|convert_[su]\x5c/\
i(?:32|64)|copys\
ign|ctz|demote\x5c/\
f64|div(?:_[su])\
?|eqz?|extend_[s\
u]\x5c/i32|floor|ge\
(?:_[su])?|gt(?:\
_[su])?|le(?:_[s\
u])?|load(?:(?:8\
|16|32)_[su])?|l\
t(?:_[su])?|max|\
min|mul|nearest|\
neg?|or|popcnt|p\
romote\x5c/f32|rein\
terpret\x5c/[fi](?:\
32|64)|rem_[su]|\
rot[lr]|shl|shr_\
[su]|store(?:8|1\
6|32)?|sqrt|sub|\
trunc(?:_[su]\x5c/f\
(?:32|64))?|wrap\
\x5c/i64|xor))\x5cb/\x0a}\
,{className:\x22num\
ber\x22,relevance:0\
,\x0amatch:/[+-]?\x5cb\
(?:\x5cd(?:_?\x5cd)*(?\
:\x5c.\x5cd(?:_?\x5cd)*)?\
(?:[eE][+-]?\x5cd(?\
:_?\x5cd)*)?|0x[\x5cda\
-fA-F](?:_?[\x5cda-\
fA-F])*(?:\x5c.[\x5cda\
-fA-F](?:_?[\x5cda-\
fA-D])*)?(?:[pP]\
[+-]?\x5cd(?:_?\x5cd)*\
)?)\x5cb|\x5cbinf\x5cb|\x5cb\
nan(?::0x[\x5cda-fA\
-F](?:_?[\x5cda-fA-\
D])*)?\x5cb/\x0a}]}}})\
();hljs.register\
Language(\x22wasm\x22,\
e)})();/*! `wren\
` grammar compil\
ed for Highlight\
.js 11.10.0 */\x0a(\
()=>{var e=(()=>\
{\x22use strict\x22;re\
turn e=>{\x0aconst \
a=e.regex,s=/[a-\
zA-Z]\x5cw*/,t=[\x22as\
\x22,\x22break\x22,\x22class\
\x22,\x22construct\x22,\x22c\
ontinue\x22,\x22else\x22,\
\x22for\x22,\x22foreign\x22,\
\x22if\x22,\x22import\x22,\x22i\
n\x22,\x22is\x22,\x22return\x22\
,\x22static\x22,\x22var\x22,\
\x22while\x22],c=[\x22tru\
e\x22,\x22false\x22,\x22null\
\x22],n=[\x22this\x22,\x22su\
per\x22],r=[\x22-\x22,\x22~\x22\
,/\x5c*/,\x22%\x22,/\x5c.\x5c.\x5c\
./,/\x5c.\x5c./,/\x5c+/,\x22\
<<\x22,\x22>>\x22,\x22>=\x22,\x22<\
=\x22,\x22<\x22,\x22>\x22,/\x5c^/,\
/!=/,/!/,/\x5cbis\x5cb\
/,\x22==\x22,\x22&&\x22,\x22&\x22,\
/\x5c|\x5c|/,/\x5c|/,/\x5c?:\
/,\x22=\x22],i={\x0arelev\
ance:0,match:a.c\
oncat(/\x5cb(?!(if|\
while|for|else|s\
uper)\x5cb)/,s,/(?=\
\x5cs*[({])/),\x0aclas\
sName:\x22title.fun\
ction\x22},o={\x0amatc\
h:a.concat(a.eit\
her(a.concat(/\x5cb\
(?!(if|while|for\
|else|super)\x5cb)/\
,s),a.either(...\
r)),/(?=\x5cs*\x5c([^)\
]+\x5c)\x5cs*\x5c{)/),\x0acl\
assName:\x22title.f\
unction\x22,starts:\
{contains:[{begi\
n:/\x5c(/,end:/\x5c)/,\
contains:[{\x0arele\
vance:0,scope:\x22p\
arams\x22,match:s}]\
}]}},l={variants\
:[{\x0amatch:[/clas\
s\x5cs+/,s,/\x5cs+is\x5cs\
+/,s]},{match:[/\
class\x5cs+/,s]}],s\
cope:{\x0a2:\x22title.\
class\x22,4:\x22title.\
class.inherited\x22\
},keywords:t},m=\
{relevance:0,\x0ama\
tch:a.either(...\
r),className:\x22op\
erator\x22},b={clas\
sName:\x22property\x22\
,\x0abegin:a.concat\
(/\x5c./,a.lookahea\
d(s)),end:s,excl\
udeBegin:!0,rele\
vance:0},h={\x0arel\
evance:0,match:a\
.concat(/\x5cb_/,s)\
,scope:\x22variable\
\x22},p={relevance:\
0,\x0amatch:/\x5cb[A-Z\
]+[a-z]+([A-Z]+[\
a-z]+)*/,scope:\x22\
title.class\x22,key\
words:{\x0a_:[\x22Bool\
\x22,\x22Class\x22,\x22Fiber\
\x22,\x22Fn\x22,\x22List\x22,\x22M\
ap\x22,\x22Null\x22,\x22Num\x22\
,\x22Object\x22,\x22Range\
\x22,\x22Sequence\x22,\x22St\
ring\x22,\x22System\x22]\x0a\
}},u=e.C_NUMBER_\
MODE,g={match:[s\
,/\x5cs*/,/=/,/\x5cs*/\
,/\x5c(/,s,/\x5c)\x5cs*\x5c{\
/],scope:{\x0a1:\x22ti\
tle.function\x22,3:\
\x22operator\x22,6:\x22pa\
rams\x22}},d=e.COMM\
ENT(/\x5c/\x5c*\x5c*/,/\x5c*\
\x5c//,{\x0acontains:[\
{match:/@[a-z]+/\
,scope:\x22doctag\x22}\
,\x22self\x22]}),v={sc\
ope:\x22subst\x22,\x0abeg\
in:/%\x5c(/,end:/\x5c)\
/,contains:[u,p,\
i,h,m]},f={scope\
:\x22string\x22,begin:\
/\x22/,end:/\x22/,\x0acon\
tains:[v,{scope:\
\x22char.escape\x22,va\
riants:[{match:/\
\x5c\x5c\x5c\x5c|\x5c\x5c[\x220%abefn\
rtv]/},{\x0amatch:/\
\x5c\x5cx[0-9A-F]{2}/}\
,{match:/\x5c\x5cu[0-9\
A-F]{4}/},{match\
:/\x5c\x5cU[0-9A-F]{8}\
/}]}]}\x0a;v.contai\
ns.push(f);const\
 N=[...t,...n,..\
.c],_={relevance\
:0,\x0amatch:a.conc\
at(\x22\x5c\x5cb(?!\x22,N.jo\
in(\x22|\x22),\x22\x5c\x5cb)\x22,/\
[a-zA-Z_]\x5cw*(?:[\
?!]|\x5cb)/),\x0aclass\
Name:\x22variable\x22}\
;return{name:\x22Wr\
en\x22,keywords:{ke\
yword:t,\x0a\x22variab\
le.language\x22:n,l\
iteral:c},contai\
ns:[{scope:\x22comm\
ent\x22,variants:[{\
\x0abegin:[/#!?/,/[\
A-Za-z_]+(?=\x5c()/\
],beginScope:{},\
keywords:{litera\
l:c},contains:[]\
,\x0aend:/\x5c)/},{beg\
in:[/#!?/,/[A-Za\
-z_]+/],beginSco\
pe:{},end:/$/}]}\
,u,f,{\x0aclassName\
:\x22string\x22,begin:\
/\x22\x22\x22/,end:/\x22\x22\x22/\x0a\
},d,e.C_LINE_COM\
MENT_MODE,e.C_BL\
OCK_COMMENT_MODE\
,p,l,g,o,i,m,h,b\
,_]}}})()\x0a;hljs.\
registerLanguage\
(\x22wren\x22,e)})();/\
*! `x86asm` gram\
mar compiled for\
 Highlight.js 11\
.10.0 */\x0a(()=>{v\
ar s=(()=>{\x22use \
strict\x22;return s\
=>({name:\x22Intel \
x86 Assembly\x22,\x0ac\
ase_insensitive:\
!0,keywords:{$pa\
ttern:\x22[.%]?\x22+s.\
IDENT_RE,\x0akeywor\
d:\x22lock rep repe\
 repz repne repn\
z xaquire xrelea\
se bnd nobnd aaa\
 aad aam aas adc\
 add and arpl bb\
0_reset bb1_rese\
t bound bsf bsr \
bswap bt btc btr\
 bts call cbw cd\
q cdqe clc cld c\
li clts cmc cmp \
cmpsb cmpsd cmps\
q cmpsw cmpxchg \
cmpxchg486 cmpxc\
hg8b cmpxchg16b \
cpuid cpu_read c\
pu_write cqo cwd\
 cwde daa das de\
c div dmint emms\
 enter equ f2xm1\
 fabs fadd faddp\
 fbld fbstp fchs\
 fclex fcmovb fc\
movbe fcmove fcm\
ovnb fcmovnbe fc\
movne fcmovnu fc\
movu fcom fcomi \
fcomip fcomp fco\
mpp fcos fdecstp\
 fdisi fdiv fdiv\
p fdivr fdivrp f\
emms feni ffree \
ffreep fiadd fic\
om ficomp fidiv \
fidivr fild fimu\
l fincstp finit \
fist fistp fistt\
p fisub fisubr f\
ld fld1 fldcw fl\
denv fldl2e fldl\
2t fldlg2 fldln2\
 fldpi fldz fmul\
 fmulp fnclex fn\
disi fneni fnini\
t fnop fnsave fn\
stcw fnstenv fns\
tsw fpatan fprem\
 fprem1 fptan fr\
ndint frstor fsa\
ve fscale fsetpm\
 fsin fsincos fs\
qrt fst fstcw fs\
tenv fstp fstsw \
fsub fsubp fsubr\
 fsubrp ftst fuc\
om fucomi fucomi\
p fucomp fucompp\
 fxam fxch fxtra\
ct fyl2x fyl2xp1\
 hlt ibts icebp \
idiv imul in inc\
 incbin insb ins\
d insw int int01\
 int1 int03 int3\
 into invd invpc\
id invlpg invlpg\
a iret iretd ire\
tq iretw jcxz je\
cxz jrcxz jmp jm\
pe lahf lar lds \
lea leave les lf\
ence lfs lgdt lg\
s lidt lldt lmsw\
 loadall loadall\
286 lodsb lodsd \
lodsq lodsw loop\
 loope loopne lo\
opnz loopz lsl l\
ss ltr mfence mo\
nitor mov movd m\
ovq movsb movsd \
movsq movsw movs\
x movsxd movzx m\
ul mwait neg nop\
 not or out outs\
b outsd outsw pa\
ckssdw packsswb \
packuswb paddb p\
addd paddsb padd\
siw paddsw paddu\
sb paddusw paddw\
 pand pandn paus\
e paveb pavgusb \
pcmpeqb pcmpeqd \
pcmpeqw pcmpgtb \
pcmpgtd pcmpgtw \
pdistib pf2id pf\
acc pfadd pfcmpe\
q pfcmpge pfcmpg\
t pfmax pfmin pf\
mul pfrcp pfrcpi\
t1 pfrcpit2 pfrs\
qit1 pfrsqrt pfs\
ub pfsubr pi2fd \
pmachriw pmaddwd\
 pmagw pmulhriw \
pmulhrwa pmulhrw\
c pmulhw pmullw \
pmvgezb pmvlzb p\
mvnzb pmvzb pop \
popa popad popaw\
 popf popfd popf\
q popfw por pref\
etch prefetchw p\
slld psllq psllw\
 psrad psraw psr\
ld psrlq psrlw p\
subb psubd psubs\
b psubsiw psubsw\
 psubusb psubusw\
 psubw punpckhbw\
 punpckhdq punpc\
khwd punpcklbw p\
unpckldq punpckl\
wd push pusha pu\
shad pushaw push\
f pushfd pushfq \
pushfw pxor rcl \
rcr rdshr rdmsr \
rdpmc rdtsc rdts\
cp ret retf retn\
 rol ror rdm rsd\
c rsldt rsm rsts\
 sahf sal salc s\
ar sbb scasb sca\
sd scasq scasw s\
fence sgdt shl s\
hld shr shrd sid\
t sldt skinit sm\
i smint smintold\
 smsw stc std st\
i stosb stosd st\
osq stosw str su\
b svdc svldt svt\
s swapgs syscall\
 sysenter sysexi\
t sysret test ud\
0 ud1 ud2b ud2 u\
d2a umov verr ve\
rw fwait wbinvd \
wrshr wrmsr xadd\
 xbts xchg xlatb\
 xlat xor cmove \
cmovz cmovne cmo\
vnz cmova cmovnb\
e cmovae cmovnb \
cmovb cmovnae cm\
ovbe cmovna cmov\
g cmovnle cmovge\
 cmovnl cmovl cm\
ovnge cmovle cmo\
vng cmovc cmovnc\
 cmovo cmovno cm\
ovs cmovns cmovp\
 cmovpe cmovnp c\
movpo je jz jne \
jnz ja jnbe jae \
jnb jb jnae jbe \
jna jg jnle jge \
jnl jl jnge jle \
jng jc jnc jo jn\
o js jns jpo jnp\
 jpe jp sete set\
z setne setnz se\
ta setnbe setae \
setnb setnc setb\
 setnae setcset \
setbe setna setg\
 setnle setge se\
tnl setl setnge \
setle setng sets\
 setns seto setn\
o setpe setp set\
po setnp addps a\
ddss andnps andp\
s cmpeqps cmpeqs\
s cmpleps cmples\
s cmpltps cmplts\
s cmpneqps cmpne\
qss cmpnleps cmp\
nless cmpnltps c\
mpnltss cmpordps\
 cmpordss cmpuno\
rdps cmpunordss \
cmpps cmpss comi\
ss cvtpi2ps cvtp\
s2pi cvtsi2ss cv\
tss2si cvttps2pi\
 cvttss2si divps\
 divss ldmxcsr m\
axps maxss minps\
 minss movaps mo\
vhps movlhps mov\
lps movhlps movm\
skps movntps mov\
ss movups mulps \
mulss orps rcpps\
 rcpss rsqrtps r\
sqrtss shufps sq\
rtps sqrtss stmx\
csr subps subss \
ucomiss unpckhps\
 unpcklps xorps \
fxrstor fxrstor6\
4 fxsave fxsave6\
4 xgetbv xsetbv \
xsave xsave64 xs\
aveopt xsaveopt6\
4 xrstor xrstor6\
4 prefetchnta pr\
efetcht0 prefetc\
ht1 prefetcht2 m\
askmovq movntq p\
avgb pavgw pextr\
w pinsrw pmaxsw \
pmaxub pminsw pm\
inub pmovmskb pm\
ulhuw psadbw psh\
ufw pf2iw pfnacc\
 pfpnacc pi2fw p\
swapd maskmovdqu\
 clflush movntdq\
 movnti movntpd \
movdqa movdqu mo\
vdq2q movq2dq pa\
ddq pmuludq pshu\
fd pshufhw pshuf\
lw pslldq psrldq\
 psubq punpckhqd\
q punpcklqdq add\
pd addsd andnpd \
andpd cmpeqpd cm\
peqsd cmplepd cm\
plesd cmpltpd cm\
pltsd cmpneqpd c\
mpneqsd cmpnlepd\
 cmpnlesd cmpnlt\
pd cmpnltsd cmpo\
rdpd cmpordsd cm\
punordpd cmpunor\
dsd cmppd comisd\
 cvtdq2pd cvtdq2\
ps cvtpd2dq cvtp\
d2pi cvtpd2ps cv\
tpi2pd cvtps2dq \
cvtps2pd cvtsd2s\
i cvtsd2ss cvtsi\
2sd cvtss2sd cvt\
tpd2pi cvttpd2dq\
 cvttps2dq cvtts\
d2si divpd divsd\
 maxpd maxsd min\
pd minsd movapd \
movhpd movlpd mo\
vmskpd movupd mu\
lpd mulsd orpd s\
hufpd sqrtpd sqr\
tsd subpd subsd \
ucomisd unpckhpd\
 unpcklpd xorpd \
addsubpd addsubp\
s haddpd haddps \
hsubpd hsubps ld\
dqu movddup movs\
hdup movsldup cl\
gi stgi vmcall v\
mclear vmfunc vm\
launch vmload vm\
mcall vmptrld vm\
ptrst vmread vmr\
esume vmrun vmsa\
ve vmwrite vmxof\
f vmxon invept i\
nvvpid pabsb pab\
sw pabsd palignr\
 phaddw phaddd p\
haddsw phsubw ph\
subd phsubsw pma\
ddubsw pmulhrsw \
pshufb psignb ps\
ignw psignd extr\
q insertq movnts\
d movntss lzcnt \
blendpd blendps \
blendvpd blendvp\
s dppd dpps extr\
actps insertps m\
ovntdqa mpsadbw \
packusdw pblendv\
b pblendw pcmpeq\
q pextrb pextrd \
pextrq phminposu\
w pinsrb pinsrd \
pinsrq pmaxsb pm\
axsd pmaxud pmax\
uw pminsb pminsd\
 pminud pminuw p\
movsxbw pmovsxbd\
 pmovsxbq pmovsx\
wd pmovsxwq pmov\
sxdq pmovzxbw pm\
ovzxbd pmovzxbq \
pmovzxwd pmovzxw\
q pmovzxdq pmuld\
q pmulld ptest r\
oundpd roundps r\
oundsd roundss c\
rc32 pcmpestri p\
cmpestrm pcmpist\
ri pcmpistrm pcm\
pgtq popcnt gets\
ec pfrcpv pfrsqr\
tv movbe aesenc \
aesenclast aesde\
c aesdeclast aes\
imc aeskeygenass\
ist vaesenc vaes\
enclast vaesdec \
vaesdeclast vaes\
imc vaeskeygenas\
sist vaddpd vadd\
ps vaddsd vaddss\
 vaddsubpd vadds\
ubps vandpd vand\
ps vandnpd vandn\
ps vblendpd vble\
ndps vblendvpd v\
blendvps vbroadc\
astss vbroadcast\
sd vbroadcastf12\
8 vcmpeq_ospd vc\
mpeqpd vcmplt_os\
pd vcmpltpd vcmp\
le_ospd vcmplepd\
 vcmpunord_qpd v\
cmpunordpd vcmpn\
eq_uqpd vcmpneqp\
d vcmpnlt_uspd v\
cmpnltpd vcmpnle\
_uspd vcmpnlepd \
vcmpord_qpd vcmp\
ordpd vcmpeq_uqp\
d vcmpnge_uspd v\
cmpngepd vcmpngt\
_uspd vcmpngtpd \
vcmpfalse_oqpd v\
cmpfalsepd vcmpn\
eq_oqpd vcmpge_o\
spd vcmpgepd vcm\
pgt_ospd vcmpgtp\
d vcmptrue_uqpd \
vcmptruepd vcmpl\
t_oqpd vcmple_oq\
pd vcmpunord_spd\
 vcmpneq_uspd vc\
mpnlt_uqpd vcmpn\
le_uqpd vcmpord_\
spd vcmpeq_uspd \
vcmpnge_uqpd vcm\
pngt_uqpd vcmpfa\
lse_ospd vcmpneq\
_ospd vcmpge_oqp\
d vcmpgt_oqpd vc\
mptrue_uspd vcmp\
pd vcmpeq_osps v\
cmpeqps vcmplt_o\
sps vcmpltps vcm\
ple_osps vcmplep\
s vcmpunord_qps \
vcmpunordps vcmp\
neq_uqps vcmpneq\
ps vcmpnlt_usps \
vcmpnltps vcmpnl\
e_usps vcmpnleps\
 vcmpord_qps vcm\
pordps vcmpeq_uq\
ps vcmpnge_usps \
vcmpngeps vcmpng\
t_usps vcmpngtps\
 vcmpfalse_oqps \
vcmpfalseps vcmp\
neq_oqps vcmpge_\
osps vcmpgeps vc\
mpgt_osps vcmpgt\
ps vcmptrue_uqps\
 vcmptrueps vcmp\
lt_oqps vcmple_o\
qps vcmpunord_sp\
s vcmpneq_usps v\
cmpnlt_uqps vcmp\
nle_uqps vcmpord\
_sps vcmpeq_usps\
 vcmpnge_uqps vc\
mpngt_uqps vcmpf\
alse_osps vcmpne\
q_osps vcmpge_oq\
ps vcmpgt_oqps v\
cmptrue_usps vcm\
pps vcmpeq_ossd \
vcmpeqsd vcmplt_\
ossd vcmpltsd vc\
mple_ossd vcmple\
sd vcmpunord_qsd\
 vcmpunordsd vcm\
pneq_uqsd vcmpne\
qsd vcmpnlt_ussd\
 vcmpnltsd vcmpn\
le_ussd vcmpnles\
d vcmpord_qsd vc\
mpordsd vcmpeq_u\
qsd vcmpnge_ussd\
 vcmpngesd vcmpn\
gt_ussd vcmpngts\
d vcmpfalse_oqsd\
 vcmpfalsesd vcm\
pneq_oqsd vcmpge\
_ossd vcmpgesd v\
cmpgt_ossd vcmpg\
tsd vcmptrue_uqs\
d vcmptruesd vcm\
plt_oqsd vcmple_\
oqsd vcmpunord_s\
sd vcmpneq_ussd \
vcmpnlt_uqsd vcm\
pnle_uqsd vcmpor\
d_ssd vcmpeq_uss\
d vcmpnge_uqsd v\
cmpngt_uqsd vcmp\
false_ossd vcmpn\
eq_ossd vcmpge_o\
qsd vcmpgt_oqsd \
vcmptrue_ussd vc\
mpsd vcmpeq_osss\
 vcmpeqss vcmplt\
_osss vcmpltss v\
cmple_osss vcmpl\
ess vcmpunord_qs\
s vcmpunordss vc\
mpneq_uqss vcmpn\
eqss vcmpnlt_uss\
s vcmpnltss vcmp\
nle_usss vcmpnle\
ss vcmpord_qss v\
cmpordss vcmpeq_\
uqss vcmpnge_uss\
s vcmpngess vcmp\
ngt_usss vcmpngt\
ss vcmpfalse_oqs\
s vcmpfalsess vc\
mpneq_oqss vcmpg\
e_osss vcmpgess \
vcmpgt_osss vcmp\
gtss vcmptrue_uq\
ss vcmptruess vc\
mplt_oqss vcmple\
_oqss vcmpunord_\
sss vcmpneq_usss\
 vcmpnlt_uqss vc\
mpnle_uqss vcmpo\
rd_sss vcmpeq_us\
ss vcmpnge_uqss \
vcmpngt_uqss vcm\
pfalse_osss vcmp\
neq_osss vcmpge_\
oqss vcmpgt_oqss\
 vcmptrue_usss v\
cmpss vcomisd vc\
omiss vcvtdq2pd \
vcvtdq2ps vcvtpd\
2dq vcvtpd2ps vc\
vtps2dq vcvtps2p\
d vcvtsd2si vcvt\
sd2ss vcvtsi2sd \
vcvtsi2ss vcvtss\
2sd vcvtss2si vc\
vttpd2dq vcvttps\
2dq vcvttsd2si v\
cvttss2si vdivpd\
 vdivps vdivsd v\
divss vdppd vdpp\
s vextractf128 v\
extractps vhaddp\
d vhaddps vhsubp\
d vhsubps vinser\
tf128 vinsertps \
vlddqu vldqqu vl\
dmxcsr vmaskmovd\
qu vmaskmovps vm\
askmovpd vmaxpd \
vmaxps vmaxsd vm\
axss vminpd vmin\
ps vminsd vminss\
 vmovapd vmovaps\
 vmovd vmovq vmo\
vddup vmovdqa vm\
ovqqa vmovdqu vm\
ovqqu vmovhlps v\
movhpd vmovhps v\
movlhps vmovlpd \
vmovlps vmovmskp\
d vmovmskps vmov\
ntdq vmovntqq vm\
ovntdqa vmovntpd\
 vmovntps vmovsd\
 vmovshdup vmovs\
ldup vmovss vmov\
upd vmovups vmps\
adbw vmulpd vmul\
ps vmulsd vmulss\
 vorpd vorps vpa\
bsb vpabsw vpabs\
d vpacksswb vpac\
kssdw vpackuswb \
vpackusdw vpaddb\
 vpaddw vpaddd v\
paddq vpaddsb vp\
addsw vpaddusb v\
paddusw vpalignr\
 vpand vpandn vp\
avgb vpavgw vpbl\
endvb vpblendw v\
pcmpestri vpcmpe\
strm vpcmpistri \
vpcmpistrm vpcmp\
eqb vpcmpeqw vpc\
mpeqd vpcmpeqq v\
pcmpgtb vpcmpgtw\
 vpcmpgtd vpcmpg\
tq vpermilpd vpe\
rmilps vperm2f12\
8 vpextrb vpextr\
w vpextrd vpextr\
q vphaddw vphadd\
d vphaddsw vphmi\
nposuw vphsubw v\
phsubd vphsubsw \
vpinsrb vpinsrw \
vpinsrd vpinsrq \
vpmaddwd vpmaddu\
bsw vpmaxsb vpma\
xsw vpmaxsd vpma\
xub vpmaxuw vpma\
xud vpminsb vpmi\
nsw vpminsd vpmi\
nub vpminuw vpmi\
nud vpmovmskb vp\
movsxbw vpmovsxb\
d vpmovsxbq vpmo\
vsxwd vpmovsxwq \
vpmovsxdq vpmovz\
xbw vpmovzxbd vp\
movzxbq vpmovzxw\
d vpmovzxwq vpmo\
vzxdq vpmulhuw v\
pmulhrsw vpmulhw\
 vpmullw vpmulld\
 vpmuludq vpmuld\
q vpor vpsadbw v\
pshufb vpshufd v\
pshufhw vpshuflw\
 vpsignb vpsignw\
 vpsignd vpslldq\
 vpsrldq vpsllw \
vpslld vpsllq vp\
sraw vpsrad vpsr\
lw vpsrld vpsrlq\
 vptest vpsubb v\
psubw vpsubd vps\
ubq vpsubsb vpsu\
bsw vpsubusb vps\
ubusw vpunpckhbw\
 vpunpckhwd vpun\
pckhdq vpunpckhq\
dq vpunpcklbw vp\
unpcklwd vpunpck\
ldq vpunpcklqdq \
vpxor vrcpps vrc\
pss vrsqrtps vrs\
qrtss vroundpd v\
roundps vroundsd\
 vroundss vshufp\
d vshufps vsqrtp\
d vsqrtps vsqrts\
d vsqrtss vstmxc\
sr vsubpd vsubps\
 vsubsd vsubss v\
testps vtestpd v\
ucomisd vucomiss\
 vunpckhpd vunpc\
khps vunpcklpd v\
unpcklps vxorpd \
vxorps vzeroall \
vzeroupper pclmu\
llqlqdq pclmulhq\
lqdq pclmullqhqd\
q pclmulhqhqdq p\
clmulqdq vpclmul\
lqlqdq vpclmulhq\
lqdq vpclmullqhq\
dq vpclmulhqhqdq\
 vpclmulqdq vfma\
dd132ps vfmadd13\
2pd vfmadd312ps \
vfmadd312pd vfma\
dd213ps vfmadd21\
3pd vfmadd123ps \
vfmadd123pd vfma\
dd231ps vfmadd23\
1pd vfmadd321ps \
vfmadd321pd vfma\
ddsub132ps vfmad\
dsub132pd vfmadd\
sub312ps vfmadds\
ub312pd vfmaddsu\
b213ps vfmaddsub\
213pd vfmaddsub1\
23ps vfmaddsub12\
3pd vfmaddsub231\
ps vfmaddsub231p\
d vfmaddsub321ps\
 vfmaddsub321pd \
vfmsub132ps vfms\
ub132pd vfmsub31\
2ps vfmsub312pd \
vfmsub213ps vfms\
ub213pd vfmsub12\
3ps vfmsub123pd \
vfmsub231ps vfms\
ub231pd vfmsub32\
1ps vfmsub321pd \
vfmsubadd132ps v\
fmsubadd132pd vf\
msubadd312ps vfm\
subadd312pd vfms\
ubadd213ps vfmsu\
badd213pd vfmsub\
add123ps vfmsuba\
dd123pd vfmsubad\
d231ps vfmsubadd\
231pd vfmsubadd3\
21ps vfmsubadd32\
1pd vfnmadd132ps\
 vfnmadd132pd vf\
nmadd312ps vfnma\
dd312pd vfnmadd2\
13ps vfnmadd213p\
d vfnmadd123ps v\
fnmadd123pd vfnm\
add231ps vfnmadd\
231pd vfnmadd321\
ps vfnmadd321pd \
vfnmsub132ps vfn\
msub132pd vfnmsu\
b312ps vfnmsub31\
2pd vfnmsub213ps\
 vfnmsub213pd vf\
nmsub123ps vfnms\
ub123pd vfnmsub2\
31ps vfnmsub231p\
d vfnmsub321ps v\
fnmsub321pd vfma\
dd132ss vfmadd13\
2sd vfmadd312ss \
vfmadd312sd vfma\
dd213ss vfmadd21\
3sd vfmadd123ss \
vfmadd123sd vfma\
dd231ss vfmadd23\
1sd vfmadd321ss \
vfmadd321sd vfms\
ub132ss vfmsub13\
2sd vfmsub312ss \
vfmsub312sd vfms\
ub213ss vfmsub21\
3sd vfmsub123ss \
vfmsub123sd vfms\
ub231ss vfmsub23\
1sd vfmsub321ss \
vfmsub321sd vfnm\
add132ss vfnmadd\
132sd vfnmadd312\
ss vfnmadd312sd \
vfnmadd213ss vfn\
madd213sd vfnmad\
d123ss vfnmadd12\
3sd vfnmadd231ss\
 vfnmadd231sd vf\
nmadd321ss vfnma\
dd321sd vfnmsub1\
32ss vfnmsub132s\
d vfnmsub312ss v\
fnmsub312sd vfnm\
sub213ss vfnmsub\
213sd vfnmsub123\
ss vfnmsub123sd \
vfnmsub231ss vfn\
msub231sd vfnmsu\
b321ss vfnmsub32\
1sd rdfsbase rdg\
sbase rdrand wrf\
sbase wrgsbase v\
cvtph2ps vcvtps2\
ph adcx adox rds\
eed clac stac xs\
tore xcryptecb x\
cryptcbc xcryptc\
tr xcryptcfb xcr\
yptofb montmul x\
sha1 xsha256 llw\
pcb slwpcb lwpva\
l lwpins vfmaddp\
d vfmaddps vfmad\
dsd vfmaddss vfm\
addsubpd vfmadds\
ubps vfmsubaddpd\
 vfmsubaddps vfm\
subpd vfmsubps v\
fmsubsd vfmsubss\
 vfnmaddpd vfnma\
ddps vfnmaddsd v\
fnmaddss vfnmsub\
pd vfnmsubps vfn\
msubsd vfnmsubss\
 vfrczpd vfrczps\
 vfrczsd vfrczss\
 vpcmov vpcomb v\
pcomd vpcomq vpc\
omub vpcomud vpc\
omuq vpcomuw vpc\
omw vphaddbd vph\
addbq vphaddbw v\
phadddq vphaddub\
d vphaddubq vpha\
ddubw vphaddudq \
vphadduwd vphadd\
uwq vphaddwd vph\
addwq vphsubbw v\
phsubdq vphsubwd\
 vpmacsdd vpmacs\
dqh vpmacsdql vp\
macssdd vpmacssd\
qh vpmacssdql vp\
macsswd vpmacssw\
w vpmacswd vpmac\
sww vpmadcsswd v\
pmadcswd vpperm \
vprotb vprotd vp\
rotq vprotw vpsh\
ab vpshad vpshaq\
 vpshaw vpshlb v\
pshld vpshlq vps\
hlw vbroadcasti1\
28 vpblendd vpbr\
oadcastb vpbroad\
castw vpbroadcas\
td vpbroadcastq \
vpermd vpermpd v\
permps vpermq vp\
erm2i128 vextrac\
ti128 vinserti12\
8 vpmaskmovd vpm\
askmovq vpsllvd \
vpsllvq vpsravd \
vpsrlvd vpsrlvq \
vgatherdpd vgath\
erqpd vgatherdps\
 vgatherqps vpga\
therdd vpgatherq\
d vpgatherdq vpg\
atherqq xabort x\
begin xend xtest\
 andn bextr blci\
 blcic blsi blsi\
c blcfill blsfil\
l blcmsk blsmsk \
blsr blcs bzhi m\
ulx pdep pext ro\
rx sarx shlx shr\
x tzcnt tzmsk t1\
mskc valignd val\
ignq vblendmpd v\
blendmps vbroadc\
astf32x4 vbroadc\
astf64x4 vbroadc\
asti32x4 vbroadc\
asti64x4 vcompre\
sspd vcompressps\
 vcvtpd2udq vcvt\
ps2udq vcvtsd2us\
i vcvtss2usi vcv\
ttpd2udq vcvttps\
2udq vcvttsd2usi\
 vcvttss2usi vcv\
tudq2pd vcvtudq2\
ps vcvtusi2sd vc\
vtusi2ss vexpand\
pd vexpandps vex\
tractf32x4 vextr\
actf64x4 vextrac\
ti32x4 vextracti\
64x4 vfixupimmpd\
 vfixupimmps vfi\
xupimmsd vfixupi\
mmss vgetexppd v\
getexpps vgetexp\
sd vgetexpss vge\
tmantpd vgetmant\
ps vgetmantsd vg\
etmantss vinsert\
f32x4 vinsertf64\
x4 vinserti32x4 \
vinserti64x4 vmo\
vdqa32 vmovdqa64\
 vmovdqu32 vmovd\
qu64 vpabsq vpan\
dd vpandnd vpand\
nq vpandq vpblen\
dmd vpblendmq vp\
cmpltd vpcmpled \
vpcmpneqd vpcmpn\
ltd vpcmpnled vp\
cmpd vpcmpltq vp\
cmpleq vpcmpneqq\
 vpcmpnltq vpcmp\
nleq vpcmpq vpcm\
pequd vpcmpltud \
vpcmpleud vpcmpn\
equd vpcmpnltud \
vpcmpnleud vpcmp\
ud vpcmpequq vpc\
mpltuq vpcmpleuq\
 vpcmpnequq vpcm\
pnltuq vpcmpnleu\
q vpcmpuq vpcomp\
ressd vpcompress\
q vpermi2d vperm\
i2pd vpermi2ps v\
permi2q vpermt2d\
 vpermt2pd vperm\
t2ps vpermt2q vp\
expandd vpexpand\
q vpmaxsq vpmaxu\
q vpminsq vpminu\
q vpmovdb vpmovd\
w vpmovqb vpmovq\
d vpmovqw vpmovs\
db vpmovsdw vpmo\
vsqb vpmovsqd vp\
movsqw vpmovusdb\
 vpmovusdw vpmov\
usqb vpmovusqd v\
pmovusqw vpord v\
porq vprold vpro\
lq vprolvd vprol\
vq vprord vprorq\
 vprorvd vprorvq\
 vpscatterdd vps\
catterdq vpscatt\
erqd vpscatterqq\
 vpsraq vpsravq \
vpternlogd vpter\
nlogq vptestmd v\
ptestmq vptestnm\
d vptestnmq vpxo\
rd vpxorq vrcp14\
pd vrcp14ps vrcp\
14sd vrcp14ss vr\
ndscalepd vrndsc\
aleps vrndscales\
d vrndscaless vr\
sqrt14pd vrsqrt1\
4ps vrsqrt14sd v\
rsqrt14ss vscale\
fpd vscalefps vs\
calefsd vscalefs\
s vscatterdpd vs\
catterdps vscatt\
erqpd vscatterqp\
s vshuff32x4 vsh\
uff64x2 vshufi32\
x4 vshufi64x2 ka\
ndnw kandw kmovw\
 knotw kortestw \
korw kshiftlw ks\
hiftrw kunpckbw \
kxnorw kxorw vpb\
roadcastmb2q vpb\
roadcastmw2d vpc\
onflictd vpconfl\
ictq vplzcntd vp\
lzcntq vexp2pd v\
exp2ps vrcp28pd \
vrcp28ps vrcp28s\
d vrcp28ss vrsqr\
t28pd vrsqrt28ps\
 vrsqrt28sd vrsq\
rt28ss vgatherpf\
0dpd vgatherpf0d\
ps vgatherpf0qpd\
 vgatherpf0qps v\
gatherpf1dpd vga\
therpf1dps vgath\
erpf1qpd vgather\
pf1qps vscatterp\
f0dpd vscatterpf\
0dps vscatterpf0\
qpd vscatterpf0q\
ps vscatterpf1dp\
d vscatterpf1dps\
 vscatterpf1qpd \
vscatterpf1qps p\
refetchwt1 bndmk\
 bndcl bndcu bnd\
cn bndmov bndldx\
 bndstx sha1rnds\
4 sha1nexte sha1\
msg1 sha1msg2 sh\
a256rnds2 sha256\
msg1 sha256msg2 \
hint_nop0 hint_n\
op1 hint_nop2 hi\
nt_nop3 hint_nop\
4 hint_nop5 hint\
_nop6 hint_nop7 \
hint_nop8 hint_n\
op9 hint_nop10 h\
int_nop11 hint_n\
op12 hint_nop13 \
hint_nop14 hint_\
nop15 hint_nop16\
 hint_nop17 hint\
_nop18 hint_nop1\
9 hint_nop20 hin\
t_nop21 hint_nop\
22 hint_nop23 hi\
nt_nop24 hint_no\
p25 hint_nop26 h\
int_nop27 hint_n\
op28 hint_nop29 \
hint_nop30 hint_\
nop31 hint_nop32\
 hint_nop33 hint\
_nop34 hint_nop3\
5 hint_nop36 hin\
t_nop37 hint_nop\
38 hint_nop39 hi\
nt_nop40 hint_no\
p41 hint_nop42 h\
int_nop43 hint_n\
op44 hint_nop45 \
hint_nop46 hint_\
nop47 hint_nop48\
 hint_nop49 hint\
_nop50 hint_nop5\
1 hint_nop52 hin\
t_nop53 hint_nop\
54 hint_nop55 hi\
nt_nop56 hint_no\
p57 hint_nop58 h\
int_nop59 hint_n\
op60 hint_nop61 \
hint_nop62 hint_\
nop63\x22,\x0abuilt_in\
:\x22ip eip rip al \
ah bl bh cl ch d\
l dh sil dil bpl\
 spl r8b r9b r10\
b r11b r12b r13b\
 r14b r15b ax bx\
 cx dx si di bp \
sp r8w r9w r10w \
r11w r12w r13w r\
14w r15w eax ebx\
 ecx edx esi edi\
 ebp esp eip r8d\
 r9d r10d r11d r\
12d r13d r14d r1\
5d rax rbx rcx r\
dx rsi rdi rbp r\
sp r8 r9 r10 r11\
 r12 r13 r14 r15\
 cs ds es fs gs \
ss st st0 st1 st\
2 st3 st4 st5 st\
6 st7 mm0 mm1 mm\
2 mm3 mm4 mm5 mm\
6 mm7 xmm0  xmm1\
  xmm2  xmm3  xm\
m4  xmm5  xmm6  \
xmm7  xmm8  xmm9\
 xmm10  xmm11 xm\
m12 xmm13 xmm14 \
xmm15 xmm16 xmm1\
7 xmm18 xmm19 xm\
m20 xmm21 xmm22 \
xmm23 xmm24 xmm2\
5 xmm26 xmm27 xm\
m28 xmm29 xmm30 \
xmm31 ymm0  ymm1\
  ymm2  ymm3  ym\
m4  ymm5  ymm6  \
ymm7  ymm8  ymm9\
 ymm10  ymm11 ym\
m12 ymm13 ymm14 \
ymm15 ymm16 ymm1\
7 ymm18 ymm19 ym\
m20 ymm21 ymm22 \
ymm23 ymm24 ymm2\
5 ymm26 ymm27 ym\
m28 ymm29 ymm30 \
ymm31 zmm0  zmm1\
  zmm2  zmm3  zm\
m4  zmm5  zmm6  \
zmm7  zmm8  zmm9\
 zmm10  zmm11 zm\
m12 zmm13 zmm14 \
zmm15 zmm16 zmm1\
7 zmm18 zmm19 zm\
m20 zmm21 zmm22 \
zmm23 zmm24 zmm2\
5 zmm26 zmm27 zm\
m28 zmm29 zmm30 \
zmm31 k0 k1 k2 k\
3 k4 k5 k6 k7 bn\
d0 bnd1 bnd2 bnd\
3 cr0 cr1 cr2 cr\
3 cr4 cr8 dr0 dr\
1 dr2 dr3 dr8 tr\
3 tr4 tr5 tr6 tr\
7 r0 r1 r2 r3 r4\
 r5 r6 r7 r0b r1\
b r2b r3b r4b r5\
b r6b r7b r0w r1\
w r2w r3w r4w r5\
w r6w r7w r0d r1\
d r2d r3d r4d r5\
d r6d r7d r0h r1\
h r2h r3h r0l r1\
l r2l r3l r4l r5\
l r6l r7l r8l r9\
l r10l r11l r12l\
 r13l r14l r15l \
db dw dd dq dt d\
dq do dy dz resb\
 resw resd resq \
rest resdq reso \
resy resz incbin\
 equ times byte \
word dword qword\
 nosplit rel abs\
 seg wrt strict \
near far a32 ptr\
\x22,\x0ameta:\x22%define\
 %xdefine %+ %un\
def %defstr %def\
tok %assign %str\
cat %strlen %sub\
str %rotate %eli\
f %else %endif %\
if %ifmacro %ifc\
tx %ifidn %ifidn\
i %ifid %ifnum %\
ifstr %iftoken %\
ifempty %ifenv %\
error %warning %\
fatal %rep %endr\
ep %include %pus\
h %pop %repl %pa\
thsearch %depend\
 %use %arg %stac\
ksize %local %li\
ne %comment %end\
comment .nolist \
__FILE__ __LINE_\
_ __SECT__  __BI\
TS__ __OUTPUT_FO\
RMAT__ __DATE__ \
__TIME__ __DATE_\
NUM__ __TIME_NUM\
__ __UTC_DATE__ \
__UTC_TIME__ __U\
TC_DATE_NUM__ __\
UTC_TIME_NUM__  \
__PASS__ struc e\
ndstruc istruc a\
t iend align ali\
gnb sectalign da\
z nodaz up down \
zero default opt\
ion assume publi\
c bits use16 use\
32 use64 default\
 section segment\
 absolute extern\
 global common c\
pu float __utf16\
__ __utf16le__ _\
_utf16be__ __utf\
32__ __utf32le__\
 __utf32be__ __f\
loat8__ __float1\
6__ __float32__ \
__float64__ __fl\
oat80m__ __float\
80e__ __float128\
l__ __float128h_\
_ __Infinity__ _\
_QNaN__ __SNaN__\
 Inf NaN QNaN SN\
aN float8 float1\
6 float32 float6\
4 float80m float\
80e float128l fl\
oat128h __FLOAT_\
DAZ__ __FLOAT_RO\
UND__ __FLOAT__\x22\
\x0a},contains:[s.C\
OMMENT(\x22;\x22,\x22$\x22,{\
relevance:0}),{c\
lassName:\x22number\
\x22,variants:[{\x0abe\
gin:\x22\x5c\x5cb(?:([0-9\
][0-9_]*)?\x5c\x5c.[0-\
9_]*(?:[eE][+-]?\
[0-9_]+)?|(0[Xx]\
)?[0-9][0-9_]*(\x5c\
\x5c.[0-9_]*)?(?:[p\
P](?:[+-]?[0-9_]\
+)?)?)\x5c\x5cb\x22,\x0arele\
vance:0},{begin:\
\x22\x5c\x5c$[0-9][0-9A-F\
a-f]*\x22,relevance\
:0},{\x0abegin:\x22\x5c\x5cb\
(?:[0-9A-Fa-f][0\
-9A-Fa-f_]*[Hh]|\
[0-9][0-9_]*[DdT\
t]?|[0-7][0-7_]*\
[QqOo]|[0-1][0-1\
_]*[BbYy])\x5c\x5cb\x22\x0a}\
,{\x0abegin:\x22\x5c\x5cb(?:\
0[Xx][0-9A-Fa-f_\
]+|0[DdTt][0-9_]\
+|0[QqOo][0-7_]+\
|0[BbYy][0-1_]+)\
\x5c\x5cb\x22\x0a}]},s.QUOTE\
_STRING_MODE,{cl\
assName:\x22string\x22\
,variants:[{begi\
n:\x22'\x22,end:\x22[^\x5c\x5c\x5c\
\x5c]'\x22\x0a},{begin:\x22`\
\x22,end:\x22[^\x5c\x5c\x5c\x5c]`\x22\
}],relevance:0},\
{className:\x22symb\
ol\x22,variants:[{\x0a\
begin:\x22^\x5c\x5cs*[A-Z\
a-z._?][A-Za-z0-\
9_$#@~.?]*(:|\x5c\x5cs\
+label)\x22},{\x0abegi\
n:\x22^\x5c\x5cs*%%[A-Za-\
z0-9_$#@~.?]*:\x22}\
],relevance:0},{\
className:\x22subst\
\x22,\x0abegin:\x22%[0-9]\
+\x22,relevance:0},\
{className:\x22subs\
t\x22,begin:\x22%!S+\x22,\
relevance:0},{\x0ac\
lassName:\x22meta\x22,\
begin:/^\x5cs*\x5c.[\x5cw\
_-]+/}]})})();hl\
js.registerLangu\
age(\x22x86asm\x22,s)\x0a\
})();/*! `xl` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>{const t={\x0a$\
pattern:/[a-zA-Z\
][a-zA-Z0-9_?]*/\
,\x0akeyword:[\x22if\x22,\
\x22then\x22,\x22else\x22,\x22d\
o\x22,\x22while\x22,\x22unti\
l\x22,\x22for\x22,\x22loop\x22,\
\x22import\x22,\x22with\x22,\
\x22is\x22,\x22as\x22,\x22where\
\x22,\x22when\x22,\x22by\x22,\x22d\
ata\x22,\x22constant\x22,\
\x22integer\x22,\x22real\x22\
,\x22text\x22,\x22name\x22,\x22\
boolean\x22,\x22symbol\
\x22,\x22infix\x22,\x22prefi\
x\x22,\x22postfix\x22,\x22bl\
ock\x22,\x22tree\x22],\x0ali\
teral:[\x22true\x22,\x22f\
alse\x22,\x22nil\x22],\x0abu\
ilt_in:[\x22in\x22,\x22mo\
d\x22,\x22rem\x22,\x22and\x22,\x22\
or\x22,\x22xor\x22,\x22not\x22,\
\x22abs\x22,\x22sign\x22,\x22fl\
oor\x22,\x22ceil\x22,\x22sqr\
t\x22,\x22sin\x22,\x22cos\x22,\x22\
tan\x22,\x22asin\x22,\x22aco\
s\x22,\x22atan\x22,\x22exp\x22,\
\x22expm1\x22,\x22log\x22,\x22l\
og2\x22,\x22log10\x22,\x22lo\
g1p\x22,\x22pi\x22,\x22at\x22,\x22\
text_length\x22,\x22te\
xt_range\x22,\x22text_\
find\x22,\x22text_repl\
ace\x22,\x22contains\x22,\
\x22page\x22,\x22slide\x22,\x22\
basic_slide\x22,\x22ti\
tle_slide\x22,\x22titl\
e\x22,\x22subtitle\x22,\x22f\
ade_in\x22,\x22fade_ou\
t\x22,\x22fade_at\x22,\x22cl\
ear_color\x22,\x22colo\
r\x22,\x22line_color\x22,\
\x22line_width\x22,\x22te\
xture_wrap\x22,\x22tex\
ture_transform\x22,\
\x22texture\x22,\x22scale\
_?x\x22,\x22scale_?y\x22,\
\x22scale_?z?\x22,\x22tra\
nslate_?x\x22,\x22tran\
slate_?y\x22,\x22trans\
late_?z?\x22,\x22rotat\
e_?x\x22,\x22rotate_?y\
\x22,\x22rotate_?z?\x22,\x22\
rectangle\x22,\x22circ\
le\x22,\x22ellipse\x22,\x22s\
phere\x22,\x22path\x22,\x22l\
ine_to\x22,\x22move_to\
\x22,\x22quad_to\x22,\x22cur\
ve_to\x22,\x22theme\x22,\x22\
background\x22,\x22con\
tents\x22,\x22locally\x22\
,\x22time\x22,\x22mouse_?\
x\x22,\x22mouse_?y\x22,\x22m\
ouse_buttons\x22].c\
oncat([\x22ObjectLo\
ader\x22,\x22Animate\x22,\
\x22MovieCredits\x22,\x22\
Slides\x22,\x22Filters\
\x22,\x22Shading\x22,\x22Mat\
erials\x22,\x22LensFla\
re\x22,\x22Mapping\x22,\x22V\
LCAudioVideo\x22,\x22S\
tereoDecoder\x22,\x22P\
ointCloud\x22,\x22Netw\
orkAccess\x22,\x22Remo\
teControl\x22,\x22RegE\
xp\x22,\x22ChromaKey\x22,\
\x22Snowfall\x22,\x22Node\
JS\x22,\x22Speech\x22,\x22Ch\
arts\x22])\x0a},a={cla\
ssName:\x22string\x22,\
begin:'\x22',end:'\x22\
',illegal:\x22\x5c\x5cn\x22}\
,n={\x0abeginKeywor\
ds:\x22import\x22,end:\
\x22$\x22,keywords:t,c\
ontains:[a]},o={\
className:\x22funct\
ion\x22,\x0abegin:/[a-\
z][^\x5cn]*->/,retu\
rnBegin:!0,end:/\
->/,contains:[e.\
inherit(e.TITLE_\
MODE,{\x0astarts:{e\
ndsWithParent:!0\
,keywords:t}})]}\
;return{name:\x22XL\
\x22,aliases:[\x22tao\x22\
],\x0akeywords:t,co\
ntains:[e.C_LINE\
_COMMENT_MODE,e.\
C_BLOCK_COMMENT_\
MODE,a,{\x0aclassNa\
me:\x22string\x22,begi\
n:\x22'\x22,end:\x22'\x22,il\
legal:\x22\x5c\x5cn\x22},{cl\
assName:\x22string\x22\
,\x0abegin:\x22<<\x22,end\
:\x22>>\x22},o,n,{clas\
sName:\x22number\x22,\x0a\
begin:\x22[0-9]+#[0\
-9A-Z_]+(\x5c\x5c.[0-9\
-A-Z_]+)?#?([Ee]\
[+-]?[0-9]+)?\x22},\
e.NUMBER_MODE]}}\
\x0a})();hljs.regis\
terLanguage(\x22xl\x22\
,e)})();/*! `xml\
` grammar compil\
ed for Highlight\
.js 11.10.0 */\x0a(\
()=>{var e=(()=>\
{\x22use strict\x22;re\
turn e=>{\x0aconst \
a=e.regex,n=a.co\
ncat(/[\x5cp{L}_]/u\
,a.optional(/[\x5cp\
{L}0-9_.-]*:/u),\
/[\x5cp{L}0-9_.-]*/\
u),s={\x0aclassName\
:\x22symbol\x22,begin:\
/&[a-z]+;|&#[0-9\
]+;|&#x[a-f0-9]+\
;/},t={begin:/\x5cs\
/,\x0acontains:[{cl\
assName:\x22keyword\
\x22,begin:/#?[a-z_\
][a-z1-9_-]+/,il\
legal:/\x5cn/}]\x0a},i\
=e.inherit(t,{be\
gin:/\x5c(/,end:/\x5c)\
/}),c=e.inherit(\
e.APOS_STRING_MO\
DE,{\x0aclassName:\x22\
string\x22}),l=e.in\
herit(e.QUOTE_ST\
RING_MODE,{class\
Name:\x22string\x22}),\
r={\x0aendsWithPare\
nt:!0,illegal:/<\
/,relevance:0,co\
ntains:[{classNa\
me:\x22attr\x22,\x0abegin\
:/[\x5cp{L}0-9._:-]\
+/u,relevance:0}\
,{begin:/=\x5cs*/,r\
elevance:0,conta\
ins:[{\x0aclassName\
:\x22string\x22,endsPa\
rent:!0,variants\
:[{begin:/\x22/,end\
:/\x22/,contains:[s\
]},{\x0abegin:/'/,e\
nd:/'/,contains:\
[s]},{begin:/[^\x5c\
s\x22'=<>`]+/}]}]}]\
};return{\x0aname:\x22\
HTML, XML\x22,\x0aalia\
ses:[\x22html\x22,\x22xht\
ml\x22,\x22rss\x22,\x22atom\x22\
,\x22xjb\x22,\x22xsd\x22,\x22xs\
l\x22,\x22plist\x22,\x22wsf\x22\
,\x22svg\x22],\x0acase_in\
sensitive:!0,uni\
codeRegex:!0,con\
tains:[{classNam\
e:\x22meta\x22,begin:/\
<![a-z]/,\x0aend:/>\
/,relevance:10,c\
ontains:[t,l,c,i\
,{begin:/\x5c[/,end\
:/\x5c]/,contains:[\
{\x0aclassName:\x22met\
a\x22,begin:/<![a-z\
]/,end:/>/,conta\
ins:[t,i,l,c]}]}\
]\x0a},e.COMMENT(/<\
!--/,/-->/,{rele\
vance:10}),{begi\
n:/<!\x5c[CDATA\x5c[/,\
end:/\x5c]\x5c]>/,\x0arel\
evance:10},s,{cl\
assName:\x22meta\x22,e\
nd:/\x5c?>/,variant\
s:[{begin:/<\x5c?xm\
l/,\x0arelevance:10\
,contains:[l]},{\
begin:/<\x5c?[a-z][\
a-z0-9]+/}]},{cl\
assName:\x22tag\x22,\x0ab\
egin:/<style(?=\x5c\
s|>)/,end:/>/,ke\
ywords:{name:\x22st\
yle\x22},contains:[\
r],starts:{\x0aend:\
/<\x5c/style>/,retu\
rnEnd:!0,subLang\
uage:[\x22css\x22,\x22xml\
\x22]}},{className:\
\x22tag\x22,\x0abegin:/<s\
cript(?=\x5cs|>)/,e\
nd:/>/,keywords:\
{name:\x22script\x22},\
contains:[r],sta\
rts:{\x0aend:/<\x5c/sc\
ript>/,returnEnd\
:!0,subLanguage:\
[\x22javascript\x22,\x22h\
andlebars\x22,\x22xml\x22\
]}},{\x0aclassName:\
\x22tag\x22,begin:/<>|\
<\x5c/>/},{classNam\
e:\x22tag\x22,\x0abegin:a\
.concat(/</,a.lo\
okahead(a.concat\
(n,a.either(/\x5c/>\
/,/>/,/\x5cs/)))),\x0a\
end:/\x5c/?>/,conta\
ins:[{className:\
\x22name\x22,begin:n,r\
elevance:0,start\
s:r}]},{\x0aclassNa\
me:\x22tag\x22,begin:a\
.concat(/<\x5c//,a.\
lookahead(a.conc\
at(n,/>/))),cont\
ains:[{\x0aclassNam\
e:\x22name\x22,begin:n\
,relevance:0},{b\
egin:/>/,relevan\
ce:0,endsParent:\
!0}]}]}}\x0a})();hl\
js.registerLangu\
age(\x22xml\x22,e)})()\
;/*! `xquery` gr\
ammar compiled f\
or Highlight.js \
11.10.0 */\x0a(()=>\
{var e=(()=>{\x22us\
e strict\x22;return\
 e=>({name:\x22XQue\
ry\x22,\x0aaliases:[\x22x\
path\x22,\x22xq\x22,\x22xqm\x22\
],case_insensiti\
ve:!1,\x0aillegal:/\
(proc)|(abstract\
)|(extends)|(unt\
il)|(#)/,keyword\
s:{\x0a$pattern:/[a\
-zA-Z$][a-zA-Z0-\
9_:-]*/,\x0akeyword\
:[\x22module\x22,\x22sche\
ma\x22,\x22namespace\x22,\
\x22boundary-space\x22\
,\x22preserve\x22,\x22no-\
preserve\x22,\x22strip\
\x22,\x22default\x22,\x22col\
lation\x22,\x22base-ur\
i\x22,\x22ordering\x22,\x22c\
ontext\x22,\x22decimal\
-format\x22,\x22decima\
l-separator\x22,\x22co\
py-namespaces\x22,\x22\
empty-sequence\x22,\
\x22except\x22,\x22expone\
nt-separator\x22,\x22e\
xternal\x22,\x22groupi\
ng-separator\x22,\x22i\
nherit\x22,\x22no-inhe\
rit\x22,\x22lax\x22,\x22minu\
s-sign\x22,\x22per-mil\
le\x22,\x22percent\x22,\x22s\
chema-attribute\x22\
,\x22schema-element\
\x22,\x22strict\x22,\x22unor\
dered\x22,\x22zero-dig\
it\x22,\x22declare\x22,\x22i\
mport\x22,\x22option\x22,\
\x22function\x22,\x22vali\
date\x22,\x22variable\x22\
,\x22for\x22,\x22at\x22,\x22in\x22\
,\x22let\x22,\x22where\x22,\x22\
order\x22,\x22group\x22,\x22\
by\x22,\x22return\x22,\x22if\
\x22,\x22then\x22,\x22else\x22,\
\x22tumbling\x22,\x22slid\
ing\x22,\x22window\x22,\x22s\
tart\x22,\x22when\x22,\x22on\
ly\x22,\x22end\x22,\x22previ\
ous\x22,\x22next\x22,\x22sta\
ble\x22,\x22ascending\x22\
,\x22descending\x22,\x22a\
llowing\x22,\x22empty\x22\
,\x22greatest\x22,\x22lea\
st\x22,\x22some\x22,\x22ever\
y\x22,\x22satisfies\x22,\x22\
switch\x22,\x22case\x22,\x22\
typeswitch\x22,\x22try\
\x22,\x22catch\x22,\x22and\x22,\
\x22or\x22,\x22to\x22,\x22union\
\x22,\x22intersect\x22,\x22i\
nstance\x22,\x22of\x22,\x22t\
reat\x22,\x22as\x22,\x22cast\
able\x22,\x22cast\x22,\x22ma\
p\x22,\x22array\x22,\x22dele\
te\x22,\x22insert\x22,\x22in\
to\x22,\x22replace\x22,\x22v\
alue\x22,\x22rename\x22,\x22\
copy\x22,\x22modify\x22,\x22\
update\x22],\x0atype:[\
\x22item\x22,\x22document\
-node\x22,\x22node\x22,\x22a\
ttribute\x22,\x22docum\
ent\x22,\x22element\x22,\x22\
comment\x22,\x22namesp\
ace\x22,\x22namespace-\
node\x22,\x22processin\
g-instruction\x22,\x22\
text\x22,\x22construct\
ion\x22,\x22xs:anyAtom\
icType\x22,\x22xs:unty\
pedAtomic\x22,\x22xs:d\
uration\x22,\x22xs:tim\
e\x22,\x22xs:decimal\x22,\
\x22xs:float\x22,\x22xs:d\
ouble\x22,\x22xs:gYear\
Month\x22,\x22xs:gYear\
\x22,\x22xs:gMonthDay\x22\
,\x22xs:gMonth\x22,\x22xs\
:gDay\x22,\x22xs:boole\
an\x22,\x22xs:base64Bi\
nary\x22,\x22xs:hexBin\
ary\x22,\x22xs:anyURI\x22\
,\x22xs:QName\x22,\x22xs:\
NOTATION\x22,\x22xs:da\
teTime\x22,\x22xs:date\
TimeStamp\x22,\x22xs:d\
ate\x22,\x22xs:string\x22\
,\x22xs:normalizedS\
tring\x22,\x22xs:token\
\x22,\x22xs:language\x22,\
\x22xs:NMTOKEN\x22,\x22xs\
:Name\x22,\x22xs:NCNam\
e\x22,\x22xs:ID\x22,\x22xs:I\
DREF\x22,\x22xs:ENTITY\
\x22,\x22xs:integer\x22,\x22\
xs:nonPositiveIn\
teger\x22,\x22xs:negat\
iveInteger\x22,\x22xs:\
long\x22,\x22xs:int\x22,\x22\
xs:short\x22,\x22xs:by\
te\x22,\x22xs:nonNegat\
iveInteger\x22,\x22xs:\
unisignedLong\x22,\x22\
xs:unsignedInt\x22,\
\x22xs:unsignedShor\
t\x22,\x22xs:unsignedB\
yte\x22,\x22xs:positiv\
eInteger\x22,\x22xs:ye\
arMonthDuration\x22\
,\x22xs:dayTimeDura\
tion\x22],\x0aliteral:\
[\x22eq\x22,\x22ne\x22,\x22lt\x22,\
\x22le\x22,\x22gt\x22,\x22ge\x22,\x22\
is\x22,\x22self::\x22,\x22ch\
ild::\x22,\x22descenda\
nt::\x22,\x22descendan\
t-or-self::\x22,\x22at\
tribute::\x22,\x22foll\
owing::\x22,\x22follow\
ing-sibling::\x22,\x22\
parent::\x22,\x22ances\
tor::\x22,\x22ancestor\
-or-self::\x22,\x22pre\
ceding::\x22,\x22prece\
ding-sibling::\x22,\
\x22NaN\x22]\x0a},contain\
s:[{className:\x22v\
ariable\x22,begin:/\
[$][\x5cw\x5c-:]+/},{c\
lassName:\x22built_\
in\x22,\x0avariants:[{\
begin:/\x5cbarray:/\
,\x0aend:/(?:append\
|filter|flatten|\
fold-(?:left|rig\
ht)|for-each(?:-\
pair)?|get|head|\
insert-before|jo\
in|put|remove|re\
verse|size|sort|\
subarray|tail)\x5cb\
/\x0a},{begin:/\x5cbma\
p:/,\x0aend:/(?:con\
tains|entry|find\
|for-each|get|ke\
ys|merge|put|rem\
ove|size)\x5cb/},{\x0a\
begin:/\x5cbmath:/,\
\x0aend:/(?:a(?:cos\
|sin|tan[2]?)|co\
s|exp(?:10)?|log\
(?:10)?|pi|pow|s\
in|sqrt|tan)\x5cb/\x0a\
},{begin:/\x5cbop:/\
,end:/\x5c(/,exclud\
eEnd:!0},{begin:\
/\x5cbfn:/,end:/\x5c(/\
,excludeEnd:!0\x0a}\
,{\x0abegin:/[^</$:\
'\x22-]\x5cb(?:abs|acc\
umulator-(?:afte\
r|before)|adjust\
-(?:date(?:Time)\
?|time)-to-timez\
one|analyze-stri\
ng|apply|availab\
le-(?:environmen\
t-variables|syst\
em-properties)|a\
vg|base-uri|bool\
ean|ceiling|code\
points?-(?:equal\
|to-string)|coll\
ation-key|collec\
tion|compare|con\
cat|contains(?:-\
token)?|copy-of|\
count|current(?:\
-)?(?:date(?:Tim\
e)?|time|group(?\
:ing-key)?|outpu\
t-uri|merge-(?:g\
roup|key))?data|\
dateTime|days?-f\
rom-(?:date(?:Ti\
me)?|duration)|d\
eep-equal|defaul\
t-(?:collation|l\
anguage)|distinc\
t-values|documen\
t(?:-uri)?|doc(?\
:-available)?|el\
ement-(?:availab\
le|with-id)|empt\
y|encode-for-uri\
|ends-with|envir\
onment-variable|\
error|escape-htm\
l-uri|exactly-on\
e|exists|false|f\
ilter|floor|fold\
-(?:left|right)|\
for-each(?:-pair\
)?|format-(?:dat\
e(?:Time)?|time|\
integer|number)|\
function-(?:arit\
y|available|look\
up|name)|generat\
e-id|has-childre\
n|head|hours-fro\
m-(?:dateTime|du\
ration|time)|id(\
?:ref)?|implicit\
-timezone|in-sco\
pe-prefixes|inde\
x-of|innermost|i\
nsert-before|iri\
-to-uri|json-(?:\
doc|to-xml)|key|\
lang|last|load-x\
query-module|loc\
al-name(?:-from-\
QName)?|(?:lower\
|upper)-case|mat\
ches|max|minutes\
-from-(?:dateTim\
e|duration|time)\
|min|months?-fro\
m-(?:date(?:Time\
)?|duration)|nam\
e(?:space-uri-?(\
?:for-prefix|fro\
m-QName)?)?|nill\
ed|node-name|nor\
malize-(?:space|\
unicode)|not|num\
ber|one-or-more|\
outermost|parse-\
(?:ietf-date|jso\
n)|path|position\
|(?:prefix-from-\
)?QName|random-n\
umber-generator|\
regex-group|remo\
ve|replace|resol\
ve-(?:QName|uri)\
|reverse|root|ro\
und(?:-half-to-e\
ven)?|seconds-fr\
om-(?:dateTime|d\
uration|time)|sn\
apshot|sort|star\
ts-with|static-b\
ase-uri|stream-a\
vailable|string-\
?(?:join|length|\
to-codepoints)?|\
subsequence|subs\
tring-?(?:after|\
before)?|sum|sys\
tem-property|tai\
l|timezone-from-\
(?:date(?:Time)?\
|time)|tokenize|\
trace|trans(?:fo\
rm|late)|true|ty\
pe-available|uno\
rdered|unparsed-\
(?:entity|text)?\
-?(?:public-id|u\
ri|available|lin\
es)?|uri-collect\
ion|xml-to-json|\
years?-from-(?:d\
ate(?:Time)?|dur\
ation)|zero-or-o\
ne)\x5cb/\x0a},{begin:\
/\x5cblocal:/,end:/\
\x5c(/,excludeEnd:!\
0},{begin:/\x5cbzip\
:/,\x0aend:/(?:zip-\
file|(?:xml|html\
|text|binary)-en\
try| (?:update-)\
?entries)\x5cb/},{\x0a\
begin:/\x5cb(?:util\
|db|functx|app|x\
dmp|xmldb):/,end\
:/\x5c(/,excludeEnd\
:!0}]},{\x0aclassNa\
me:\x22string\x22,vari\
ants:[{begin:/\x22/\
,end:/\x22/,contain\
s:[{begin:/\x22\x22/,\x0a\
relevance:0}]},{\
begin:/'/,end:/'\
/,contains:[{beg\
in:/''/,relevanc\
e:0}]}]},{\x0aclass\
Name:\x22number\x22,\x0ab\
egin:/(\x5cb0[0-7_]\
+)|(\x5cb0x[0-9a-fA\
-F_]+)|(\x5cb[1-9][\
0-9_]*(\x5c.[0-9_]+\
)?)|[0_]\x5cb/,\x0arel\
evance:0},{class\
Name:\x22comment\x22,b\
egin:/\x5c(:/,end:/\
:\x5c)/,relevance:1\
0,contains:[{\x0acl\
assName:\x22doctag\x22\
,begin:/@\x5cw+/}]}\
,{className:\x22met\
a\x22,begin:/%[\x5cw\x5c-\
:]+/},{\x0aclassNam\
e:\x22title\x22,begin:\
/\x5cbxquery versio\
n \x22[13]\x5c.[01]\x22\x5cs\
?(?:encoding \x22.+\
\x22)?/,\x0aend:/;/},{\
\x0abeginKeywords:\x22\
element attribut\
e comment docume\
nt processing-in\
struction\x22,\x0aend:\
/\x5c{/,excludeEnd:\
!0},{begin:/<([\x5c\
w._:-]+)(\x5cs+\x5cS*=\
('|\x22).*('|\x22))?>/\
,\x0aend:/(\x5c/[\x5cw._:\
-]+>)/,subLangua\
ge:\x22xml\x22,contain\
s:[{begin:/\x5c{/,e\
nd:/\x5c}/,\x0asubLang\
uage:\x22xquery\x22},\x22\
self\x22]}]})})();h\
ljs.registerLang\
uage(\x22xquery\x22,e)\
})();/*! `yaml` \
grammar compiled\
 for Highlight.j\
s 11.10.0 */\x0a(()\
=>{var e=(()=>{\x22\
use strict\x22;retu\
rn e=>{\x0aconst n=\
\x22true false yes \
no null\x22,a=\x22[\x5c\x5cw\
#;/?:@&=+$,.~*'(\
)[\x5c\x5c]]+\x22,s={\x0acla\
ssName:\x22string\x22,\
relevance:0,vari\
ants:[{begin:/'/\
,end:/'/},{begin\
:/\x22/,end:/\x22/\x0a},{\
begin:/\x5cS+/}],co\
ntains:[e.BACKSL\
ASH_ESCAPE,{clas\
sName:\x22template-\
variable\x22,\x0avaria\
nts:[{begin:/\x5c{\x5c\
{/,end:/\x5c}\x5c}/},{\
begin:/%\x5c{/,end:\
/\x5c}/}]}]},i=e.in\
herit(s,{\x0avarian\
ts:[{begin:/'/,e\
nd:/'/},{begin:/\
\x22/,end:/\x22/},{beg\
in:/[^\x5cs,{}[\x5c]]+\
/}]}),l={\x0aend:\x22,\
\x22,endsWithParent\
:!0,excludeEnd:!\
0,keywords:n,rel\
evance:0},t={beg\
in:/\x5c{/,\x0aend:/\x5c}\
/,contains:[l],i\
llegal:\x22\x5c\x5cn\x22,rel\
evance:0},g={beg\
in:\x22\x5c\x5c[\x22,end:\x22\x5c\x5c\
]\x22,\x0acontains:[l]\
,illegal:\x22\x5c\x5cn\x22,r\
elevance:0},b=[{\
className:\x22attr\x22\
,variants:[{\x0abeg\
in:/\x5cw[\x5cw :()\x5c./\
-]*:(?=[ \x5ct]|$)/\
},{begin:/\x22\x5cw[\x5cw\
 :()\x5c./-]*\x22:(?=[\
 \x5ct]|$)/},{\x0abegi\
n:/'\x5cw[\x5cw :()\x5c./\
-]*':(?=[ \x5ct]|$)\
/}]},{className:\
\x22meta\x22,begin:\x22^-\
--\x5c\x5cs*$\x22,\x0areleva\
nce:10},{classNa\
me:\x22string\x22,\x0abeg\
in:\x22[\x5c\x5c|>]([1-9]\
?[+-])?[ ]*\x5c\x5cn( \
+)[^ ][^\x5c\x5cn]*\x5c\x5cn\
(\x5c\x5c2[^\x5c\x5cn]+\x5c\x5cn?)\
*\x22},{\x0abegin:\x22<%[\
%=-]?\x22,end:\x22[%-]\
?%>\x22,subLanguage\
:\x22ruby\x22,excludeB\
egin:!0,excludeE\
nd:!0,\x0arelevance\
:0},{className:\x22\
type\x22,begin:\x22!\x5c\x5c\
w+!\x22+a},{classNa\
me:\x22type\x22,\x0abegin\
:\x22!<\x22+a+\x22>\x22},{cl\
assName:\x22type\x22,b\
egin:\x22!\x22+a},{cla\
ssName:\x22type\x22,be\
gin:\x22!!\x22+a\x0a},{cl\
assName:\x22meta\x22,b\
egin:\x22&\x22+e.UNDER\
SCORE_IDENT_RE+\x22\
$\x22},{className:\x22\
meta\x22,\x0abegin:\x22\x5c\x5c\
*\x22+e.UNDERSCORE_\
IDENT_RE+\x22$\x22},{c\
lassName:\x22bullet\
\x22,begin:\x22-(?=[ ]\
|$)\x22,\x0arelevance:\
0},e.HASH_COMMEN\
T_MODE,{beginKey\
words:n,keywords\
:{literal:n}},{\x0a\
className:\x22numbe\
r\x22,\x0abegin:\x22\x5c\x5cb[0\
-9]{4}(-[0-9][0-\
9]){0,2}([Tt \x5c\x5ct\
][0-9][0-9]?(:[0\
-9][0-9]){2})?(\x5c\
\x5c.[0-9]*)?([ \x5c\x5ct\
])*(Z|[-+][0-9][\
0-9]?(:[0-9][0-9\
])?)?\x5c\x5cb\x22\x0a},{cla\
ssName:\x22number\x22,\
begin:e.C_NUMBER\
_RE+\x22\x5c\x5cb\x22,releva\
nce:0},t,g,s],r=\
[...b]\x0a;return r\
.pop(),r.push(i)\
,l.contains=r,{n\
ame:\x22YAML\x22,case_\
insensitive:!0,\x0a\
aliases:[\x22yml\x22],\
contains:b}}})()\
;hljs.registerLa\
nguage(\x22yaml\x22,e)\
})();/*! `zephir\
` grammar compil\
ed for Highlight\
.js 11.10.0 */\x0a(\
()=>{var e=(()=>\
{\x22use strict\x22;re\
turn e=>{const n\
={className:\x22str\
ing\x22,\x0acontains:[\
e.BACKSLASH_ESCA\
PE],variants:[e.\
inherit(e.APOS_S\
TRING_MODE,{\x0aill\
egal:null}),e.in\
herit(e.QUOTE_ST\
RING_MODE,{illeg\
al:null})]\x0a},a=e\
.UNDERSCORE_TITL\
E_MODE,s={varian\
ts:[e.BINARY_NUM\
BER_MODE,e.C_NUM\
BER_MODE]\x0a},i=\x22n\
amespace class i\
nterface use ext\
ends function re\
turn abstract fi\
nal public prote\
cted private sta\
tic deprecated t\
hrow try catch E\
xception echo em\
pty isset instan\
ceof unset let v\
ar new const sel\
f require if els\
e elseif switch \
case default do \
while loop for c\
ontinue break li\
kely unlikely __\
LINE__ __FILE__ \
__DIR__ __FUNCTI\
ON__ __CLASS__ _\
_TRAIT__ __METHO\
D__ __NAMESPACE_\
_ array boolean \
float double int\
eger object reso\
urce string char\
 long unsigned b\
ool int uint ulo\
ng uchar true fa\
lse null undefin\
ed\x22\x0a;return{name\
:\x22Zephir\x22,aliase\
s:[\x22zep\x22],keywor\
ds:i,\x0acontains:[\
e.C_LINE_COMMENT\
_MODE,e.COMMENT(\
/\x5c/\x5c*/,/\x5c*\x5c//,{c\
ontains:[{\x0aclass\
Name:\x22doctag\x22,be\
gin:/@[A-Za-z]+/\
}]}),{className:\
\x22string\x22,\x0abegin:\
/<<<['\x22]?\x5cw+['\x22]\
?$/,end:/^\x5cw+;/,\
contains:[e.BACK\
SLASH_ESCAPE]},{\
\x0abegin:/(::|->)+\
[a-zA-Z_\x5cx7f-\x5cxf\
f][a-zA-Z0-9_\x5cx7\
f-\x5cxff]*/},{clas\
sName:\x22function\x22\
,\x0abeginKeywords:\
\x22function fn\x22,en\
d:/[;{]/,exclude\
End:!0,illegal:/\
\x5c$|\x5c[|%/,\x0acontai\
ns:[a,{className\
:\x22params\x22,begin:\
/\x5c(/,end:/\x5c)/,ke\
ywords:i,\x0acontai\
ns:[\x22self\x22,e.C_B\
LOCK_COMMENT_MOD\
E,n,s]}]},{class\
Name:\x22class\x22,\x0abe\
ginKeywords:\x22cla\
ss interface\x22,en\
d:/\x5c{/,excludeEn\
d:!0,illegal:/[:\
($\x22]/,\x0acontains:\
[{beginKeywords:\
\x22extends impleme\
nts\x22},a]},{begin\
Keywords:\x22namesp\
ace\x22,\x0aend:/;/,il\
legal:/[.']/,con\
tains:[a]},{begi\
nKeywords:\x22use\x22,\
end:/;/,contains\
:[a]\x0a},{begin:/=\
>/},n,s]}}})();h\
ljs.registerLang\
uage(\x22zephir\x22,e)\
})();\
\x00\x00\x08\xe9\
/\
**\x0a * Minified b\
y jsDelivr using\
 Terser v5.37.0.\
\x0a * Original fil\
e: /npm/markdown\
-it-katex@2.0.3/\
index.js\x0a *\x0a * D\
o NOT use SRI wi\
th dynamically g\
enerated files! \
More information\
: https://www.js\
delivr.com/using\
-sri-with-dynami\
c-files\x0a */\x0a\x22use\
 strict\x22;var kat\
ex=require(\x22kate\
x\x22);function isV\
alidDelim(r,e){v\
ar n,t,i=r.posMa\
x,c=!0,o=!0;retu\
rn n=e>0?r.src.c\
harCodeAt(e-1):-\
1,t=e+1<=i?r.src\
.charCodeAt(e+1)\
:-1,(32===n||9==\
=n||t>=48&&t<=57\
)&&(o=!1),32!==t\
&&9!==t||(c=!1),\
{can_open:c,can_\
close:o}}functio\
n math_inline(r,\
e){var n,t,i,c;i\
f(\x22$\x22!==r.src[r.\
pos])return!1;if\
(!isValidDelim(r\
,r.pos).can_open\
)return e||(r.pe\
nding+=\x22$\x22),r.po\
s+=1,!0;for(t=n=\
r.pos+1;-1!==(t=\
r.src.indexOf(\x22$\
\x22,t));){for(c=t-\
1;\x22\x5c\x5c\x22===r.src[c\
];)c-=1;if((t-c)\
%2==1)break;t+=1\
}return-1===t?(e\
||(r.pending+=\x22$\
\x22),r.pos=n,!0):t\
-n==0?(e||(r.pen\
ding+=\x22$$\x22),r.po\
s=n+1,!0):isVali\
dDelim(r,t).can_\
close?(e||((i=r.\
push(\x22math_inlin\
e\x22,\x22math\x22,0)).ma\
rkup=\x22$\x22,i.conte\
nt=r.src.slice(n\
,t)),r.pos=t+1,!\
0):(e||(r.pendin\
g+=\x22$\x22),r.pos=n,\
!0)}function mat\
h_block(r,e,n,t)\
{var i,c,o,s,a,l\
=!1,u=r.bMarks[e\
]+r.tShift[e],p=\
r.eMarks[e];if(u\
+2>p)return!1;if\
(\x22$$\x22!==r.src.sl\
ice(u,u+2))retur\
n!1;if(u+=2,i=r.\
src.slice(u,p),t\
)return!0;for(\x22$\
$\x22===i.trim().sl\
ice(-2)&&(i=i.tr\
im().slice(0,-2)\
,l=!0),o=e;!l&&!\
(++o>=n)&&!((u=r\
.bMarks[o]+r.tSh\
ift[o])<(p=r.eMa\
rks[o])&&r.tShif\
t[o]<r.blkIndent\
);)\x22$$\x22===r.src.\
slice(u,p).trim(\
).slice(-2)&&(s=\
r.src.slice(0,p)\
.lastIndexOf(\x22$$\
\x22),c=r.src.slice\
(u,s),l=!0);retu\
rn r.line=o+1,(a\
=r.push(\x22math_bl\
ock\x22,\x22math\x22,0)).\
block=!0,a.conte\
nt=(i&&i.trim()?\
i+\x22\x5cn\x22:\x22\x22)+r.get\
Lines(e+1,o,r.tS\
hift[e],!0)+(c&&\
c.trim()?c:\x22\x22),a\
.map=[e,r.line],\
a.markup=\x22$$\x22,!0\
}module.exports=\
function(r,e){e=\
e||{};r.inline.r\
uler.after(\x22esca\
pe\x22,\x22math_inline\
\x22,math_inline),r\
.block.ruler.aft\
er(\x22blockquote\x22,\
\x22math_block\x22,mat\
h_block,{alt:[\x22p\
aragraph\x22,\x22refer\
ence\x22,\x22blockquot\
e\x22,\x22list\x22]}),r.r\
enderer.rules.ma\
th_inline=functi\
on(r,n){return f\
unction(r){e.dis\
playMode=!1;try{\
return katex.ren\
derToString(r,e)\
}catch(n){return\
 e.throwOnError&\
&console.log(n),\
r}}(r[n].content\
)},r.renderer.ru\
les.math_block=f\
unction(r,n){ret\
urn function(r){\
e.displayMode=!0\
;try{return\x22<p>\x22\
+katex.renderToS\
tring(r,e)+\x22</p>\
\x22}catch(n){retur\
n e.throwOnError\
&&console.log(n)\
,r}}(r[n].conten\
t)+\x22\x5cn\x22}};\x0a//# s\
ourceMappingURL=\
/sm/6d5601e1352e\
1d22c8e8b51ab0e3\
ea84e698545be1c6\
30b38738659c8eee\
c9f1.map\
\x00\x049\x15\
!\
function(e,t){\x22o\
bject\x22==typeof e\
xports&&\x22object\x22\
==typeof module?\
module.exports=t\
():\x22function\x22==t\
ypeof define&&de\
fine.amd?define(\
[],t):\x22object\x22==\
typeof exports?e\
xports.katex=t()\
:e.katex=t()}(\x22u\
ndefined\x22!=typeo\
f self?self:this\
,(function(){ret\
urn function(){\x22\
use strict\x22;var \
e={d:function(t,\
r){for(var n in \
r)e.o(r,n)&&!e.o\
(t,n)&&Object.de\
fineProperty(t,n\
,{enumerable:!0,\
get:r[n]})},o:fu\
nction(e,t){retu\
rn Object.protot\
ype.hasOwnProper\
ty.call(e,t)}},t\
={};e.d(t,{defau\
lt:function(){re\
turn na}});var r\
=function e(t,r)\
{this.position=v\
oid 0;var n,a=\x22K\
aTeX parse error\
: \x22+t,i=r&&r.loc\
;if(i&&i.start<=\
i.end){var o=i.l\
exer.input;n=i.s\
tart;var s=i.end\
;n===o.length?a+\
=\x22 at end of inp\
ut: \x22:a+=\x22 at po\
sition \x22+(n+1)+\x22\
: \x22;var l=o.slic\
e(n,s).replace(/\
[^]/g,\x22$&\x5cu0332\x22\
);a+=(n>15?\x22\x5cu20\
26\x22+o.slice(n-15\
,n):o.slice(0,n)\
)+l+(s+15<o.leng\
th?o.slice(s,s+1\
5)+\x22\x5cu2026\x22:o.sl\
ice(s))}var h=ne\
w Error(a);retur\
n h.name=\x22ParseE\
rror\x22,h.__proto_\
_=e.prototype,h.\
position=n,h};r.\
prototype.__prot\
o__=Error.protot\
ype;var n=r,a=/(\
[A-Z])/g,i={\x22&\x22:\
\x22&amp;\x22,\x22>\x22:\x22&gt\
;\x22,\x22<\x22:\x22&lt;\x22,'\x22\
':\x22&quot;\x22,\x22'\x22:\x22\
&#x27;\x22},o=/[&><\
\x22']/g;var s=func\
tion e(t){return\
\x22ordgroup\x22===t.t\
ype||\x22color\x22===t\
.type?1===t.body\
.length?e(t.body\
[0]):t:\x22font\x22===\
t.type?e(t.body)\
:t},l={contains:\
function(e,t){re\
turn-1!==e.index\
Of(t)},deflt:fun\
ction(e,t){retur\
n void 0===e?t:e\
},escape:functio\
n(e){return Stri\
ng(e).replace(o,\
(function(e){ret\
urn i[e]}))},hyp\
henate:function(\
e){return e.repl\
ace(a,\x22-$1\x22).toL\
owerCase()},getB\
aseElem:s,isChar\
acterBox:functio\
n(e){var t=s(e);\
return\x22mathord\x22=\
==t.type||\x22texto\
rd\x22===t.type||\x22a\
tom\x22===t.type},p\
rotocolFromUrl:f\
unction(e){var t\
=/^\x5cs*([^\x5c\x5c/#]*?\
)(?::|&#0*58|&#x\
0*3a)/i.exec(e);\
return null!=t?t\
[1]:\x22_relative\x22}\
},h={displayMode\
:{type:\x22boolean\x22\
,description:\x22Re\
nder math in dis\
play mode, which\
 puts the math i\
n display style \
(so \x5c\x5cint and \x5c\x5c\
sum are large, f\
or example), and\
 centers the mat\
h on the page on\
 its own line.\x22,\
cli:\x22-d, --displ\
ay-mode\x22},output\
:{type:{enum:[\x22h\
tmlAndMathml\x22,\x22h\
tml\x22,\x22mathml\x22]},\
description:\x22Det\
ermines the mark\
up language of t\
he output.\x22,cli:\
\x22-F, --format <t\
ype>\x22},leqno:{ty\
pe:\x22boolean\x22,des\
cription:\x22Render\
 display math in\
 leqno style (le\
ft-justified tag\
s).\x22},fleqn:{typ\
e:\x22boolean\x22,desc\
ription:\x22Render \
display math flu\
sh left.\x22},throw\
OnError:{type:\x22b\
oolean\x22,default:\
!0,cli:\x22-t, --no\
-throw-on-error\x22\
,cliDescription:\
\x22Render errors (\
in the color giv\
en by --error-co\
lor) instead of \
throwing a Parse\
Error exception \
when encounterin\
g an error.\x22},er\
rorColor:{type:\x22\
string\x22,default:\
\x22#cc0000\x22,cli:\x22-\
c, --error-color\
 <color>\x22,cliDes\
cription:\x22A colo\
r string given i\
n the format 'rg\
b' or 'rrggbb' (\
no #). This opti\
on determines th\
e color of error\
s rendered by th\
e -t option.\x22,cl\
iProcessor:funct\
ion(e){return\x22#\x22\
+e}},macros:{typ\
e:\x22object\x22,cli:\x22\
-m, --macro <def\
>\x22,cliDescriptio\
n:\x22Define custom\
 macro of the fo\
rm '\x5c\x5cfoo:expans\
ion' (use multip\
le -m arguments \
for multiple mac\
ros).\x22,cliDefaul\
t:[],cliProcesso\
r:function(e,t){\
return t.push(e)\
,t}},minRuleThic\
kness:{type:\x22num\
ber\x22,description\
:\x22Specifies a mi\
nimum thickness,\
 in ems, for fra\
ction lines, `\x5c\x5c\
sqrt` top lines,\
 `{array}` verti\
cal lines, `\x5c\x5chl\
ine`, `\x5c\x5chdashli\
ne`, `\x5c\x5cunderlin\
e`, `\x5c\x5coverline`\
, and the border\
s of `\x5c\x5cfbox`, `\
\x5c\x5cboxed`, and `\x5c\
\x5cfcolorbox`.\x22,pr\
ocessor:function\
(e){return Math.\
max(0,e)},cli:\x22-\
-min-rule-thickn\
ess <size>\x22,cliP\
rocessor:parseFl\
oat},colorIsText\
Color:{type:\x22boo\
lean\x22,descriptio\
n:\x22Makes \x5c\x5ccolor\
 behave like LaT\
eX's 2-argument \
\x5c\x5ctextcolor, ins\
tead of LaTeX's \
one-argument \x5c\x5cc\
olor mode change\
.\x22,cli:\x22-b, --co\
lor-is-text-colo\
r\x22},strict:{type\
:[{enum:[\x22warn\x22,\
\x22ignore\x22,\x22error\x22\
]},\x22boolean\x22,\x22fu\
nction\x22],descrip\
tion:\x22Turn on st\
rict / LaTeX fai\
thfulness mode, \
which throws an \
error if the inp\
ut uses features\
 that are not su\
pported by LaTeX\
.\x22,cli:\x22-S, --st\
rict\x22,cliDefault\
:!1},trust:{type\
:[\x22boolean\x22,\x22fun\
ction\x22],descript\
ion:\x22Trust the i\
nput, enabling a\
ll HTML features\
 such as \x5c\x5curl.\x22\
,cli:\x22-T, --trus\
t\x22},maxSize:{typ\
e:\x22number\x22,defau\
lt:1/0,descripti\
on:\x22If non-zero,\
 all user-specif\
ied sizes, e.g. \
in \x5c\x5crule{500em}\
{500em}, will be\
 capped to maxSi\
ze ems. Otherwis\
e, elements and \
spaces can be ar\
bitrarily large\x22\
,processor:funct\
ion(e){return Ma\
th.max(0,e)},cli\
:\x22-s, --max-size\
 <n>\x22,cliProcess\
or:parseInt},max\
Expand:{type:\x22nu\
mber\x22,default:1e\
3,description:\x22L\
imit the number \
of macro expansi\
ons to the speci\
fied number, to \
prevent e.g. inf\
inite macro loop\
s. If set to Inf\
inity, the macro\
 expander will t\
ry to fully expa\
nd as in LaTeX.\x22\
,processor:funct\
ion(e){return Ma\
th.max(0,e)},cli\
:\x22-e, --max-expa\
nd <n>\x22,cliProce\
ssor:function(e)\
{return\x22Infinity\
\x22===e?1/0:parseI\
nt(e)}},globalGr\
oup:{type:\x22boole\
an\x22,cli:!1}};fun\
ction c(e){if(e.\
default)return e\
.default;var t=e\
.type,r=Array.is\
Array(t)?t[0]:t;\
if(\x22string\x22!=typ\
eof r)return r.e\
num[0];switch(r)\
{case\x22boolean\x22:r\
eturn!1;case\x22str\
ing\x22:return\x22\x22;ca\
se\x22number\x22:retur\
n 0;case\x22object\x22\
:return{}}}var m\
=function(){func\
tion e(e){for(va\
r t in this.disp\
layMode=void 0,t\
his.output=void \
0,this.leqno=voi\
d 0,this.fleqn=v\
oid 0,this.throw\
OnError=void 0,t\
his.errorColor=v\
oid 0,this.macro\
s=void 0,this.mi\
nRuleThickness=v\
oid 0,this.color\
IsTextColor=void\
 0,this.strict=v\
oid 0,this.trust\
=void 0,this.max\
Size=void 0,this\
.maxExpand=void \
0,this.globalGro\
up=void 0,e=e||{\
},h)if(h.hasOwnP\
roperty(t)){var \
r=h[t];this[t]=v\
oid 0!==e[t]?r.p\
rocessor?r.proce\
ssor(e[t]):e[t]:\
c(r)}}var t=e.pr\
ototype;return t\
.reportNonstrict\
=function(e,t,r)\
{var a=this.stri\
ct;if(\x22function\x22\
==typeof a&&(a=a\
(e,t,r)),a&&\x22ign\
ore\x22!==a){if(!0=\
==a||\x22error\x22===a\
)throw new n(\x22La\
TeX-incompatible\
 input and stric\
t mode is set to\
 'error': \x22+t+\x22 \
[\x22+e+\x22]\x22,r);\x22war\
n\x22===a?\x22undefine\
d\x22!=typeof conso\
le&&console.warn\
(\x22LaTeX-incompat\
ible input and s\
trict mode is se\
t to 'warn': \x22+t\
+\x22 [\x22+e+\x22]\x22):\x22un\
defined\x22!=typeof\
 console&&consol\
e.warn(\x22LaTeX-in\
compatible input\
 and strict mode\
 is set to unrec\
ognized '\x22+a+\x22':\
 \x22+t+\x22 [\x22+e+\x22]\x22)\
}},t.useStrictBe\
havior=function(\
e,t,r){var n=thi\
s.strict;if(\x22fun\
ction\x22==typeof n\
)try{n=n(e,t,r)}\
catch(e){n=\x22erro\
r\x22}return!(!n||\x22\
ignore\x22===n)&&(!\
0===n||\x22error\x22==\
=n||(\x22warn\x22===n?\
(\x22undefined\x22!=ty\
peof console&&co\
nsole.warn(\x22LaTe\
X-incompatible i\
nput and strict \
mode is set to '\
warn': \x22+t+\x22 [\x22+\
e+\x22]\x22),!1):(\x22und\
efined\x22!=typeof \
console&&console\
.warn(\x22LaTeX-inc\
ompatible input \
and strict mode \
is set to unreco\
gnized '\x22+n+\x22': \
\x22+t+\x22 [\x22+e+\x22]\x22),\
!1)))},t.isTrust\
ed=function(e){e\
.url&&!e.protoco\
l&&(e.protocol=l\
.protocolFromUrl\
(e.url));var t=\x22\
function\x22==typeo\
f this.trust?thi\
s.trust(e):this.\
trust;return Boo\
lean(t)},e}(),u=\
function(){funct\
ion e(e,t,r){thi\
s.id=void 0,this\
.size=void 0,thi\
s.cramped=void 0\
,this.id=e,this.\
size=t,this.cram\
ped=r}var t=e.pr\
ototype;return t\
.sup=function(){\
return p[d[this.\
id]]},t.sub=func\
tion(){return p[\
f[this.id]]},t.f\
racNum=function(\
){return p[g[thi\
s.id]]},t.fracDe\
n=function(){ret\
urn p[v[this.id]\
]},t.cramp=funct\
ion(){return p[b\
[this.id]]},t.te\
xt=function(){re\
turn p[y[this.id\
]]},t.isTight=fu\
nction(){return \
this.size>=2},e}\
(),p=[new u(0,0,\
!1),new u(1,0,!0\
),new u(2,1,!1),\
new u(3,1,!0),ne\
w u(4,2,!1),new \
u(5,2,!0),new u(\
6,3,!1),new u(7,\
3,!0)],d=[4,5,4,\
5,6,7,6,7],f=[5,\
5,5,5,7,7,7,7],g\
=[2,3,4,5,6,7,6,\
7],v=[3,3,5,5,7,\
7,7,7],b=[1,1,3,\
3,5,5,7,7],y=[0,\
1,2,3,2,3,2,3],x\
={DISPLAY:p[0],T\
EXT:p[2],SCRIPT:\
p[4],SCRIPTSCRIP\
T:p[6]},w=[{name\
:\x22latin\x22,blocks:\
[[256,591],[768,\
879]]},{name:\x22cy\
rillic\x22,blocks:[\
[1024,1279]]},{n\
ame:\x22armenian\x22,b\
locks:[[1328,142\
3]]},{name:\x22brah\
mic\x22,blocks:[[23\
04,4255]]},{name\
:\x22georgian\x22,bloc\
ks:[[4256,4351]]\
},{name:\x22cjk\x22,bl\
ocks:[[12288,125\
43],[19968,40879\
],[65280,65376]]\
},{name:\x22hangul\x22\
,blocks:[[44032,\
55215]]}];var k=\
[];function S(e)\
{for(var t=0;t<k\
.length;t+=2)if(\
e>=k[t]&&e<=k[t+\
1])return!0;retu\
rn!1}w.forEach((\
function(e){retu\
rn e.blocks.forE\
ach((function(e)\
{return k.push.a\
pply(k,e)}))}));\
var M=80,z={doub\
leleftarrow:\x22M26\
2 157\x5cnl10-10c34\
-36 62.7-77 86-1\
23 3.3-8 5-13.3 \
5-16 0-5.3-6.7-8\
-20-8-7.3\x5cn 0-12\
.2.5-14.5 1.5-2.\
3 1-4.8 4.5-7.5 \
10.5-49.3 97.3-1\
21.7 169.3-217 2\
16-28\x5cn 14-57.3 \
25-88 33-6.7 2-1\
1 3.8-13 5.5-2 1\
.7-3 4.2-3 7.5s1\
 5.8 3 7.5\x5cnc2 1\
.7 6.3 3.5 13 5.\
5 68 17.3 128.2 \
47.8 180.5 91.5 \
52.3 43.7 93.8 9\
6.2 124.5\x5cn 157.\
5 9.3 8 15.3 12.\
3 18 13h6c12-.7 \
18-4 18-10 0-2-1\
.7-7-5-15-23.3-4\
6-52-87\x5cn-86-123\
l-10-10h399738v-\
40H218c328 0 0 0\
 0 0l-10-8c-26.7\
-20-65.7-43-117-\
69 2.7\x5cn-2 6-3.7\
 10-5 36.7-16 72\
.3-37.3 107-64l1\
0-8h399782v-40z\x5c\
nm8 0v40h399730v\
-40zm0 194v40h39\
9730v-40z\x22,doubl\
erightarrow:\x22M39\
9738 392l\x5cn-10 1\
0c-34 36-62.7 77\
-86 123-3.3 8-5 \
13.3-5 16 0 5.3 \
6.7 8 20 8 7.3 0\
 12.2-.5\x5cn 14.5-\
1.5 2.3-1 4.8-4.\
5 7.5-10.5 49.3-\
97.3 121.7-169.3\
 217-216 28-14 5\
7.3-25 88\x5cn-33 6\
.7-2 11-3.8 13-5\
.5 2-1.7 3-4.2 3\
-7.5s-1-5.8-3-7.\
5c-2-1.7-6.3-3.5\
-13-5.5-68\x5cn-17.\
3-128.2-47.8-180\
.5-91.5-52.3-43.\
7-93.8-96.2-124.\
5-157.5-9.3-8-15\
.3-12.3-18\x5cn-13h\
-6c-12 .7-18 4-1\
8 10 0 2 1.7 7 5\
 15 23.3 46 52 8\
7 86 123l10 10H0\
v40h399782\x5cnc-32\
8 0 0 0 0 0l10 8\
c26.7 20 65.7 43\
 117 69-2.7 2-6 \
3.7-10 5-36.7 16\
-72.3 37.3\x5cn-107\
 64l-10 8H0v40zM\
0 157v40h399730v\
-40zm0 194v40h39\
9730v-40z\x22,lefta\
rrow:\x22M400000 24\
1H110l3-3c68.7-5\
2.7 113.7-120\x5cn \
135-202 4-14.7 6\
-23 6-25 0-7.3-7\
-11-21-11-8 0-13\
.2.8-15.5 2.5-2.\
3 1.7-4.2 5.8\x5cn-\
5.5 12.5-1.3 4.7\
-2.7 10.3-4 17-1\
2 48.7-34.8 92-6\
8.5 130S65.3 228\
.3 18 247\x5cnc-10 \
4-16 7.7-18 11 0\
 8.7 6 14.3 18 1\
7 47.3 18.7 87.8\
 47 121.5 85S196\
 441.3 208\x5cn 490\
c.7 2 1.3 5 2 9s\
1.2 6.7 1.5 8c.3\
 1.3 1 3.3 2 6s2\
.2 4.5 3.5 5.5c1\
.3 1 3.3\x5cn 1.8 6\
 2.5s6 1 10 1c14\
 0 21-3.7 21-11 \
0-2-2-10.3-6-25-\
20-79.3-65-146.7\
-135-202\x5cn l-3-3\
h399890zM100 241\
v40h399900v-40z\x22\
,leftbrace:\x22M6 5\
48l-6-6v-35l6-11\
c56-104 135.3-18\
1.3 238-232 57.3\
-28.7 117\x5cn-45 1\
79-50h399577v120\
H403c-43.3 7-81 \
15-113 26-100.7 \
33-179.7 91-237 \
174-2.7\x5cn 5-6 9-\
10 13-.7 1-7.3 1\
-20 1H6z\x22,leftbr\
aceunder:\x22M0 6l6\
-6h17c12.688 0 1\
9.313.3 20 1 4 4\
 7.313 8.3 10 13\
\x5cn 35.313 51.3 8\
0.813 93.8 136.5\
 127.5 55.688 33\
.7 117.188 55.8 \
184.5 66.5.688\x5cn\
 0 2 .3 4 1 18.6\
88 2.7 76 4.3 17\
2 5h399450v120H4\
29l-6-1c-124.688\
-8-235-61.7\x5cn-33\
1-161C60.687 138\
.7 32.312 99.3 7\
 54L0 41V6z\x22,lef\
tgroup:\x22M400000 \
80\x5cnH435C64 80 1\
68.3 229.4 21 26\
0c-5.9 1.2-18 0-\
18 0-2 0-3-1-3-3\
v-38C76 61 257 0\
\x5cn 435 0h399565z\
\x22,leftgroupunder\
:\x22M400000 262\x5cnH\
435C64 262 168.3\
 112.6 21 82c-5.\
9-1.2-18 0-18 0-\
2 0-3 1-3 3v38c7\
6 158 257 219\x5cn \
435 219h399565z\x22\
,leftharpoon:\x22M0\
 267c.7 5.3 3 10\
 7 14h399993v-40\
H93c3.3\x5cn-3.3 10\
.2-9.5 20.5-18.5\
s17.8-15.8 22.5-\
20.5c50.7-52 88-\
110.3 112-175 4-\
11.3 5\x5cn-18.3 3-\
21-1.3-4-7.3-6-1\
8-6-8 0-13 .7-15\
 2s-4.7 6.7-8 16\
c-42 98.7-107.3 \
174.7\x5cn-196 228-\
6.7 4.7-10.7 8-1\
2 10-1.3 2-2 5.7\
-2 11zm100-26v40\
h399900v-40z\x22,le\
ftharpoonplus:\x22M\
0 267c.7 5.3 3 1\
0 7 14h399993v-4\
0H93c3.3-3.3 10.\
2-9.5\x5cn 20.5-18.\
5s17.8-15.8 22.5\
-20.5c50.7-52 88\
-110.3 112-175 4\
-11.3 5-18.3 3-2\
1-1.3\x5cn-4-7.3-6-\
18-6-8 0-13 .7-1\
5 2s-4.7 6.7-8 1\
6c-42 98.7-107.3\
 174.7-196 228-6\
.7 4.7\x5cn-10.7 8-\
12 10-1.3 2-2 5.\
7-2 11zm100-26v4\
0h399900v-40zM0 \
435v40h400000v-4\
0z\x5cnm0 0v40h4000\
00v-40z\x22,lefthar\
poondown:\x22M7 241\
c-4 4-6.333 8.66\
7-7 14 0 5.333.6\
67 9 2 11s5.333\x5c\
n 5.333 12 10c90\
.667 54 156 130 \
196 228 3.333 10\
.667 6.333 16.33\
3 9 17 2 .667 5\x5c\
n 1 9 1h5c10.667\
 0 16.667-2 18-6\
 2-2.667 1-9.667\
-3-21-32-87.333-\
82.667-157.667\x5cn\
-152-211l-3-3h39\
9907v-40zM93 281\
 H400000 v-40L7 \
241z\x22,leftharpoo\
ndownplus:\x22M7 43\
5c-4 4-6.3 8.7-7\
 14 0 5.3.7 9 2 \
11s5.3 5.3 12\x5cn \
10c90.7 54 156 1\
30 196 228 3.3 1\
0.7 6.3 16.3 9 1\
7 2 .7 5 1 9 1h5\
c10.7 0 16.7\x5cn-2\
 18-6 2-2.7 1-9.\
7-3-21-32-87.3-8\
2.7-157.7-152-21\
1l-3-3h399907v-4\
0H7zm93 0\x5cnv40h3\
99900v-40zM0 241\
v40h399900v-40zm\
0 0v40h399900v-4\
0z\x22,lefthook:\x22M4\
00000 281 H103s-\
33-11.2-61-33.5S\
0 197.3 0 164s14\
.2-61.2 42.5\x5cn-8\
3.5C70.8 58.2 10\
4 47 142 47 c16.\
7 0 25 6.7 25 20\
 0 12-8.7 18.7-2\
6 20-40 3.3\x5cn-68\
.7 15.7-86 37-10\
 12-15 25.3-15 4\
0 0 22.7 9.8 40.\
7 29.5 54 19.7 1\
3.3 43.5 21\x5cn 71\
.5 23h399859zM10\
3 281v-40h399897\
v40z\x22,leftlinese\
gment:\x22M40 281 V\
428 H0 V94 H40 V\
241 H400000 v40z\
\x5cnM40 281 V428 H\
0 V94 H40 V241 H\
400000 v40z\x22,lef\
tmapsto:\x22M40 281\
 V448H0V74H40V24\
1H400000v40z\x5cnM4\
0 281 V448H0V74H\
40V241H400000v40\
z\x22,leftToFrom:\x22M\
0 147h400000v40H\
0zm0 214c68 40 1\
15.7 95.7 143 16\
7h22c15.3 0 23\x5cn\
-.3 23-1 0-1.3-5\
.3-13.7-16-37-18\
-35.3-41.3-69-70\
-101l-7-8h399905\
v-40H95l7-8\x5cnc28\
.7-32 52-65.7 70\
-101 10.7-23.3 1\
6-35.7 16-37 0-.\
7-7.7-1-23-1h-22\
C115.7 265.3\x5cn 6\
8 321 0 361zm0-1\
74v-40h399900v40\
zm100 154v40h399\
900v-40z\x22,longeq\
ual:\x22M0 50 h4000\
00 v40H0z m0 194\
h40000v40H0z\x5cnM0\
 50 h400000 v40H\
0z m0 194h40000v\
40H0z\x22,midbrace:\
\x22M200428 334\x5cnc-\
100.7-8.3-195.3-\
44-280-108-55.3-\
42-101.7-93-139-\
153l-9-14c-2.7 4\
-5.7 8.7-9 14\x5cn-\
53.3 86.7-123.7 \
153-211 199-66.7\
 36-137.3 56.3-2\
12 62H0V214h1995\
68c178.3-11.7\x5cn \
311.7-78.3 403-2\
01 6-8 9.7-12 11\
-12 .7-.7 6.7-1 \
18-1s17.3.3 18 1\
c1.3 0 5 4 11\x5cn \
12 44.7 59.3 101\
.3 106.3 170 141\
s145.3 54.3 229 \
60h199572v120z\x22,\
midbraceunder:\x22M\
199572 214\x5cnc100\
.7 8.3 195.3 44 \
280 108 55.3 42 \
101.7 93 139 153\
l9 14c2.7-4 5.7-\
8.7 9-14\x5cn 53.3-\
86.7 123.7-153 2\
11-199 66.7-36 1\
37.3-56.3 212-62\
h199568v120H2004\
32c-178.3\x5cn 11.7\
-311.7 78.3-403 \
201-6 8-9.7 12-1\
1 12-.7.7-6.7 1-\
18 1s-17.3-.3-18\
-1c-1.3 0\x5cn-5-4-\
11-12-44.7-59.3-\
101.3-106.3-170-\
141s-145.3-54.3-\
229-60H0V214z\x22,o\
iintSize1:\x22M512.\
6 71.6c272.6 0 3\
20.3 106.8 320.3\
 178.2 0 70.8-47\
.7 177.6\x5cn-320.3\
 177.6S193.1 320\
.6 193.1 249.8c0\
-71.4 46.9-178.2\
 319.5-178.2z\x5cnm\
368.1 178.2c0-86\
.4-60.9-215.4-36\
8.1-215.4-306.4 \
0-367.3 129-367.\
3 215.4 0 85.8\x5cn\
60.9 214.8 367.3\
 214.8 307.2 0 3\
68.1-129 368.1-2\
14.8z\x22,oiintSize\
2:\x22M757.8 100.1c\
384.7 0 451.1 13\
7.6 451.1 230 0 \
91.3-66.4 228.8\x5c\
n-451.1 228.8-38\
6.3 0-452.7-137.\
5-452.7-228.8 0-\
92.4 66.4-230 45\
2.7-230z\x5cnm502.4\
 230c0-111.2-82.\
4-277.2-502.4-27\
7.2s-504 166-504\
 277.2\x5cnc0 110 8\
4 276 504 276s50\
2.4-166 502.4-27\
6z\x22,oiiintSize1:\
\x22M681.4 71.6c408\
.9 0 480.5 106.8\
 480.5 178.2 0 7\
0.8-71.6 177.6\x5cn\
-480.5 177.6S202\
.1 320.6 202.1 2\
49.8c0-71.4 70.5\
-178.2 479.3-178\
.2z\x5cnm525.8 178.\
2c0-86.4-86.8-21\
5.4-525.7-215.4-\
437.9 0-524.7 12\
9-524.7 215.4 0\x5c\
n85.8 86.8 214.8\
 524.7 214.8 438\
.9 0 525.7-129 5\
25.7-214.8z\x22,oii\
intSize2:\x22M1021.\
2 53c603.6 0 707\
.8 165.8 707.8 2\
77.2 0 110-104.2\
 275.8\x5cn-707.8 2\
75.8-606 0-710.2\
-165.8-710.2-275\
.8C311 218.8 415\
.2 53 1021.2 53z\
\x5cnm770.4 277.1c0\
-131.2-126.4-327\
.6-770.5-327.6S2\
48.4 198.9 248.4\
 330.1\x5cnc0 130 1\
28.8 326.4 772.7\
 326.4s770.5-196\
.4 770.5-326.4z\x22\
,rightarrow:\x22M0 \
241v40h399891c-4\
7.3 35.3-84 78-1\
10 128\x5cn-16.7 32\
-27.7 63.7-33 95\
 0 1.3-.2 2.7-.5\
 4-.3 1.3-.5 2.3\
-.5 3 0 7.3 6.7 \
11 20\x5cn 11 8 0 1\
3.2-.8 15.5-2.5 \
2.3-1.7 4.2-5.5 \
5.5-11.5 2-13.3 \
5.7-27 11-41 14.\
7-44.7\x5cn 39-84.5\
 73-119.5s73.7-6\
0.2 119-75.5c6-2\
 9-5.7 9-11s-3-9\
-9-11c-45.3-15.3\
-85\x5cn-40.5-119-7\
5.5s-58.3-74.8-7\
3-119.5c-4.7-14-\
8.3-27.3-11-40-1\
.3-6.7-3.2-10.8-\
5.5\x5cn-12.5-2.3-1\
.7-7.5-2.5-15.5-\
2.5-14 0-21 3.7-\
21 11 0 2 2 10.3\
 6 25 20.7 83.3 \
67\x5cn 151.7 139 2\
05zm0 0v40h39990\
0v-40z\x22,rightbra\
ce:\x22M400000 542l\
\x5cn-6 6h-17c-12.7\
 0-19.3-.3-20-1-\
4-4-7.3-8.3-10-1\
3-35.3-51.3-80.8\
-93.8-136.5-127.\
5\x5cns-117.2-55.8-\
184.5-66.5c-.7 0\
-2-.3-4-1-18.7-2\
.7-76-4.3-172-5H\
0V214h399571l6 1\
\x5cnc124.7 8 235 6\
1.7 331 161 31.3\
 33.3 59.7 72.7 \
85 118l7 13v35z\x22\
,rightbraceunder\
:\x22M399994 0l6 6v\
35l-6 11c-56 104\
-135.3 181.3-238\
 232-57.3\x5cn 28.7\
-117 45-179 50H-\
300V214h399897c4\
3.3-7 81-15 113-\
26 100.7-33 179.\
7-91 237\x5cn-174 2\
.7-5 6-9 10-13 .\
7-1 7.3-1 20-1h1\
7z\x22,rightgroup:\x22\
M0 80h399565c371\
 0 266.7 149.4 4\
14 180 5.9 1.2 1\
8 0 18 0 2 0\x5cn 3\
-1 3-3v-38c-76-1\
58-257-219-435-2\
19H0z\x22,rightgrou\
punder:\x22M0 262h3\
99565c371 0 266.\
7-149.4 414-180 \
5.9-1.2 18 0 18\x5c\
n 0 2 0 3 1 3 3v\
38c-76 158-257 2\
19-435 219H0z\x22,r\
ightharpoon:\x22M0 \
241v40h399993c4.\
7-4.7 7-9.3 7-14\
 0-9.3\x5cn-3.7-15.\
3-11-18-92.7-56.\
7-159-133.7-199-\
231-3.3-9.3-6-14\
.7-8-16-2-1.3-7-\
2-15-2\x5cn-10.7 0-\
16.7 2-18 6-2 2.\
7-1 9.7 3 21 15.\
3 42 36.7 81.8 6\
4 119.5 27.3 37.\
7 58\x5cn 69.2 92 9\
4.5zm0 0v40h3999\
00v-40z\x22,rightha\
rpoonplus:\x22M0 24\
1v40h399993c4.7-\
4.7 7-9.3 7-14 0\
-9.3-3.7-15.3-11\
\x5cn-18-92.7-56.7-\
159-133.7-199-23\
1-3.3-9.3-6-14.7\
-8-16-2-1.3-7-2-\
15-2-10.7 0-16.7\
\x5cn 2-18 6-2 2.7-\
1 9.7 3 21 15.3 \
42 36.7 81.8 64 \
119.5 27.3 37.7 \
58 69.2 92 94.5z\
\x5cnm0 0v40h399900\
v-40z m100 194v4\
0h399900v-40zm0 \
0v40h399900v-40z\
\x22,rightharpoondo\
wn:\x22M399747 511c\
0 7.3 6.7 11 20 \
11 8 0 13-.8 15-\
2.5s4.7-6.8\x5cn 8-\
15.5c40-94 99.3-\
166.3 178-217 13\
.3-8 20.3-12.3 2\
1-13 5.3-3.3 8.5\
-5.8 9.5\x5cn-7.5 1\
-1.7 1.5-5.2 1.5\
-10.5s-2.3-10.3-\
7-15H0v40h399908\
c-34 25.3-64.7 5\
7-92 95\x5cn-27.3 3\
8-48.7 77.7-64 1\
19-3.3 8.7-5 14-\
5 16zM0 241v40h3\
99900v-40z\x22,righ\
tharpoondownplus\
:\x22M399747 705c0 \
7.3 6.7 11 20 11\
 8 0 13-.8\x5cn 15-\
2.5s4.7-6.8 8-15\
.5c40-94 99.3-16\
6.3 178-217 13.3\
-8 20.3-12.3 21-\
13 5.3-3.3\x5cn 8.5\
-5.8 9.5-7.5 1-1\
.7 1.5-5.2 1.5-1\
0.5s-2.3-10.3-7-\
15H0v40h399908c-\
34 25.3\x5cn-64.7 5\
7-92 95-27.3 38-\
48.7 77.7-64 119\
-3.3 8.7-5 14-5 \
16zM0 435v40h399\
900v-40z\x5cnm0-194\
v40h400000v-40zm\
0 0v40h400000v-4\
0z\x22,righthook:\x22M\
399859 241c-764 \
0 0 0 0 0 40-3.3\
 68.7-15.7 86-37\
 10-12 15-25.3\x5cn\
 15-40 0-22.7-9.\
8-40.7-29.5-54-1\
9.7-13.3-43.5-21\
-71.5-23-17.3-1.\
3-26-8-26-20 0\x5cn\
-13.3 8.7-20 26-\
20 38 0 71 11.2 \
99 33.5 0 0 7 5.\
6 21 16.7 14 11.\
2 21 33.5 21\x5cn 6\
6.8s-14 61.2-42 \
83.5c-28 22.3-61\
 33.5-99 33.5L0 \
241z M0 281v-40h\
399859v40z\x22,righ\
tlinesegment:\x22M3\
99960 241 V94 h4\
0 V428 h-40 V281\
 H0 v-40z\x5cnM3999\
60 241 V94 h40 V\
428 h-40 V281 H0\
 v-40z\x22,rightToF\
rom:\x22M400000 167\
c-70.7-42-118-97\
.7-142-167h-23c-\
15.3 0-23 .3-23\x5c\
n 1 0 1.3 5.3 13\
.7 16 37 18 35.3\
 41.3 69 70 101l\
7 8H0v40h399905l\
-7 8c-28.7 32\x5cn-\
52 65.7-70 101-1\
0.7 23.3-16 35.7\
-16 37 0 .7 7.7 \
1 23 1h23c24-69.\
3 71.3-125 142\x5cn\
-167z M100 147v4\
0h399900v-40zM0 \
341v40h399900v-4\
0z\x22,twoheadlefta\
rrow:\x22M0 167c68 \
40\x5cn 115.7 95.7 \
143 167h22c15.3 \
0 23-.3 23-1 0-1\
.3-5.3-13.7-16-3\
7-18-35.3-41.3-6\
9\x5cn-70-101l-7-8h\
125l9 7c50.7 39.\
3 85 86 103 140h\
46c0-4.7-6.3-18.\
7-19-42-18-35.3\x5c\
n-40-67.3-66-96l\
-9-9h399716v-40H\
284l9-9c26-28.7 \
48-60.7 66-96 12\
.7-23.333 19\x5cn-3\
7.333 19-42h-46c\
-18 54-52.3 100.\
7-103 140l-9 7H9\
5l7-8c28.7-32 52\
-65.7 70-101\x5cn 1\
0.7-23.333 16-35\
.7 16-37 0-.7-7.\
7-1-23-1h-22C115\
.7 71.3 68 127 0\
 167z\x22,twoheadri\
ghtarrow:\x22M40000\
0 167\x5cnc-68-40-1\
15.7-95.7-143-16\
7h-22c-15.3 0-23\
 .3-23 1 0 1.3 5\
.3 13.7 16 37 18\
 35.3\x5cn 41.3 69 \
70 101l7 8h-125l\
-9-7c-50.7-39.3-\
85-86-103-140h-4\
6c0 4.7 6.3 18.7\
 19 42\x5cn 18 35.3\
 40 67.3 66 96l9\
 9H0v40h399716l-\
9 9c-26 28.7-48 \
60.7-66 96-12.7 \
23.333\x5cn-19 37.3\
33-19 42h46c18-5\
4 52.3-100.7 103\
-140l9-7h125l-7 \
8c-28.7 32-52 65\
.7-70\x5cn 101-10.7\
 23.333-16 35.7-\
16 37 0 .7 7.7 1\
 23 1h22c27.3-71\
.3 75-127 143-16\
7z\x22,tilde1:\x22M200\
 55.538c-77 0-16\
8 73.953-177 73.\
953-3 0-7\x5cn-2.17\
5-9-5.437L2 97c-\
1-2-2-4-2-6 0-4 \
2-7 5-9l20-12C11\
6 12 171 0 207 0\
c86 0\x5cn 114 68 1\
91 68 78 0 168-6\
8 177-68 4 0 7 2\
 9 5l12 19c1 2.1\
75 2 4.35 2 6.52\
5 0\x5cn 4.35-2 7.6\
13-5 9.788l-19 1\
3.05c-92 63.077-\
116.937 75.308-1\
83 76.128\x5cn-68.2\
67.847-113-73.95\
2-191-73.952z\x22,t\
ilde2:\x22M344 55.2\
66c-142 0-300.63\
8 81.316-311.5 8\
6.418\x5cn-8.01 3.7\
62-22.5 10.91-23\
.5 5.562L1 120c-\
1-2-1-3-1-4 0-5 \
3-9 8-10l18.4-9C\
160.9\x5cn 31.9 283\
 0 358 0c148 0 1\
88 122 331 122s3\
14-97 326-97c4 0\
 8 2 10 7l7 21.1\
14\x5cnc1 2.14 1 3.\
21 1 4.28 0 5.34\
7-3 9.626-7 10.6\
96l-22.3 12.622C\
852.6 158.372 75\
1\x5cn 181.476 676 \
181.476c-149 0-1\
89-126.21-332-12\
6.21z\x22,tilde3:\x22M\
786 59C457 59 32\
 175.242 13 175.\
242c-6 0-10-3.45\
7\x5cn-11-10.37L.15\
 138c-1-7 3-12 1\
0-13l19.2-6.4C37\
8.4 40.7 634.3 0\
 804.3 0c337 0\x5cn\
 411.8 157 746.8\
 157 328 0 754-1\
12 773-112 5 0 1\
0 3 11 9l1 14.07\
5c1 8.066-.697\x5cn\
 16.595-6.697 17\
.492l-21.052 7.3\
1c-367.9 98.146-\
609.15 122.696-7\
78.15 122.696\x5cn \
-338 0-409-156.5\
73-744-156.573z\x22\
,tilde4:\x22M786 58\
C457 58 32 177.4\
87 13 177.487c-6\
 0-10-3.345\x5cn-11\
-10.035L.15 143c\
-1-7 3-12 10-13l\
22-6.7C381.2 35 \
637.15 0 807.15 \
0c337 0 409\x5cn 17\
7 744 177 328 0 \
754-127 773-127 \
5 0 10 3 11 9l1 \
14.794c1 7.805-3\
 13.38-9\x5cn 14.49\
5l-20.7 5.574c-3\
66.85 99.79-607.\
3 139.372-776.3 \
139.372-338 0-40\
9\x5cn -175.236-744\
-175.236z\x22,vec:\x22\
M377 20c0-5.333 \
1.833-10 5.5-14S\
391 0 397 0c4.66\
7 0 8.667 1.667 \
12 5\x5cn3.333 2.66\
7 6.667 9 10 19 \
6.667 24.667 20.\
333 43.667 41 57\
 7.333 4.667 11\x5c\
n10.667 11 18 0 \
6-1 10-3 12s-6.6\
67 5-14 9c-28.66\
7 14.667-53.667 \
35.667-75 63\x5cn-1\
.333 1.333-3.167\
 3.5-5.5 6.5s-4 \
4.833-5 5.5c-1 .\
667-2.5 1.333-4.\
5 2s-4.333 1\x5cn-7\
 1c-4.667 0-9.16\
7-1.833-13.5-5.5\
S337 184 337 178\
c0-12.667 15.667\
-32.333 47-59\x5cnH\
213l-171-1c-8.66\
7-6-13-12.333-13\
-19 0-4.667 4.33\
3-11.333 13-20h3\
59\x5cnc-16-25.333-\
24-45-24-59z\x22,wi\
dehat1:\x22M529 0h5\
l519 115c5 1 9 5\
 9 10 0 1-1 2-1 \
3l-4 22\x5cnc-1 5-5\
 9-11 9h-2L532 6\
7 19 159h-2c-5 0\
-9-4-11-9l-5-22c\
-1-6 2-12 8-13z\x22\
,widehat2:\x22M1181\
 0h2l1171 176c6 \
0 10 5 10 11l-2 \
23c-1 6-5 10\x5cn-1\
1 10h-1L1182 67 \
15 220h-1c-6 0-1\
0-4-11-10l-2-23c\
-1-6 4-11 10-11z\
\x22,widehat3:\x22M118\
1 0h2l1171 236c6\
 0 10 5 10 11l-2\
 23c-1 6-5 10\x5cn-\
11 10h-1L1182 67\
 15 280h-1c-6 0-\
10-4-11-10l-2-23\
c-1-6 4-11 10-11\
z\x22,widehat4:\x22M11\
81 0h2l1171 296c\
6 0 10 5 10 11l-\
2 23c-1 6-5 10\x5cn\
-11 10h-1L1182 6\
7 15 340h-1c-6 0\
-10-4-11-10l-2-2\
3c-1-6 4-11 10-1\
1z\x22,widecheck1:\x22\
M529,159h5l519,-\
115c5,-1,9,-5,9,\
-10c0,-1,-1,-2,-\
1,-3l-4,-22c-1,\x5c\
n-5,-5,-9,-11,-9\
h-2l-512,92l-513\
,-92h-2c-5,0,-9,\
4,-11,9l-5,22c-1\
,6,2,12,8,13z\x22,w\
idecheck2:\x22M1181\
,220h2l1171,-176\
c6,0,10,-5,10,-1\
1l-2,-23c-1,-6,-\
5,-10,\x5cn-11,-10h\
-1l-1168,153l-11\
67,-153h-1c-6,0,\
-10,4,-11,10l-2,\
23c-1,6,4,11,10,\
11z\x22,widecheck3:\
\x22M1181,280h2l117\
1,-236c6,0,10,-5\
,10,-11l-2,-23c-\
1,-6,-5,-10,\x5cn-1\
1,-10h-1l-1168,2\
13l-1167,-213h-1\
c-6,0,-10,4,-11,\
10l-2,23c-1,6,4,\
11,10,11z\x22,widec\
heck4:\x22M1181,340\
h2l1171,-296c6,0\
,10,-5,10,-11l-2\
,-23c-1,-6,-5,-1\
0,\x5cn-11,-10h-1l-\
1168,273l-1167,-\
273h-1c-6,0,-10,\
4,-11,10l-2,23c-\
1,6,4,11,10,11z\x22\
,baraboveleftarr\
ow:\x22M400000 620h\
-399890l3 -3c68.\
7 -52.7 113.7 -1\
20 135 -202\x5cnc4 \
-14.7 6 -23 6 -2\
5c0 -7.3 -7 -11 \
-21 -11c-8 0 -13\
.2 0.8 -15.5 2.5\
\x5cnc-2.3 1.7 -4.2\
 5.8 -5.5 12.5c-\
1.3 4.7 -2.7 10.\
3 -4 17c-12 48.7\
 -34.8 92 -68.5 \
130\x5cns-74.2 66.3\
 -121.5 85c-10 4\
 -16 7.7 -18 11c\
0 8.7 6 14.3 18 \
17c47.3 18.7 87.\
8 47\x5cn121.5 85s5\
6.5 81.3 68.5 13\
0c0.7 2 1.3 5 2 \
9s1.2 6.7 1.5 8c\
0.3 1.3 1 3.3 2 \
6\x5cns2.2 4.5 3.5 \
5.5c1.3 1 3.3 1.\
8 6 2.5s6 1 10 1\
c14 0 21 -3.7 21\
 -11\x5cnc0 -2 -2 -\
10.3 -6 -25c-20 \
-79.3 -65 -146.7\
 -135 -202l-3 -3\
h399890z\x5cnM100 6\
20v40h399900v-40\
z M0 241v40h3999\
00v-40zM0 241v40\
h399900v-40z\x22,ri\
ghtarrowabovebar\
:\x22M0 241v40h3998\
91c-47.3 35.3-84\
 78-110 128-16.7\
 32\x5cn-27.7 63.7-\
33 95 0 1.3-.2 2\
.7-.5 4-.3 1.3-.\
5 2.3-.5 3 0 7.3\
 6.7 11 20 11 8 \
0\x5cn13.2-.8 15.5-\
2.5 2.3-1.7 4.2-\
5.5 5.5-11.5 2-1\
3.3 5.7-27 11-41\
 14.7-44.7 39\x5cn-\
84.5 73-119.5s73\
.7-60.2 119-75.5\
c6-2 9-5.7 9-11s\
-3-9-9-11c-45.3-\
15.3-85-40.5\x5cn-1\
19-75.5s-58.3-74\
.8-73-119.5c-4.7\
-14-8.3-27.3-11-\
40-1.3-6.7-3.2-1\
0.8-5.5\x5cn-12.5-2\
.3-1.7-7.5-2.5-1\
5.5-2.5-14 0-21 \
3.7-21 11 0 2 2 \
10.3 6 25 20.7 8\
3.3 67\x5cn151.7 13\
9 205zm96 379h39\
9894v40H0zm0 0h3\
99904v40H0z\x22,bar\
aboveshortleftha\
rpoon:\x22M507,435c\
-4,4,-6.3,8.7,-7\
,14c0,5.3,0.7,9,\
2,11\x5cnc1.3,2,5.3\
,5.3,12,10c90.7,\
54,156,130,196,2\
28c3.3,10.7,6.3,\
16.3,9,17\x5cnc2,0.\
7,5,1,9,1c0,0,5,\
0,5,0c10.7,0,16.\
7,-2,18,-6c2,-2.\
7,1,-9.7,-3,-21\x5c\
nc-32,-87.3,-82.\
7,-157.7,-152,-2\
11c0,0,-3,-3,-3,\
-3l399351,0l0,-4\
0\x5cnc-398570,0,-3\
99437,0,-399437,\
0z M593 435 v40 \
H399500 v-40z\x5cnM\
0 281 v-40 H3999\
08 v40z M0 281 v\
-40 H399908 v40z\
\x22,rightharpoonab\
oveshortbar:\x22M0,\
241 l0,40c399126\
,0,399993,0,3999\
93,0\x5cnc4.7,-4.7,\
7,-9.3,7,-14c0,-\
9.3,-3.7,-15.3,-\
11,-18c-92.7,-56\
.7,-159,-133.7,-\
199,\x5cn-231c-3.3,\
-9.3,-6,-14.7,-8\
,-16c-2,-1.3,-7,\
-2,-15,-2c-10.7,\
0,-16.7,2,-18,6\x5c\
nc-2,2.7,-1,9.7,\
3,21c15.3,42,36.\
7,81.8,64,119.5c\
27.3,37.7,58,69.\
2,92,94.5z\x5cnM0 2\
41 v40 H399908 v\
-40z M0 475 v-40\
 H399500 v40z M0\
 475 v-40 H39950\
0 v40z\x22,shortbar\
aboveleftharpoon\
:\x22M7,435c-4,4,-6\
.3,8.7,-7,14c0,5\
.3,0.7,9,2,11\x5cnc\
1.3,2,5.3,5.3,12\
,10c90.7,54,156,\
130,196,228c3.3,\
10.7,6.3,16.3,9,\
17c2,0.7,5,1,9,\x5c\
n1c0,0,5,0,5,0c1\
0.7,0,16.7,-2,18\
,-6c2,-2.7,1,-9.\
7,-3,-21c-32,-87\
.3,-82.7,-157.7,\
\x5cn-152,-211c0,0,\
-3,-3,-3,-3l3999\
07,0l0,-40c-3991\
26,0,-399993,0,-\
399993,0z\x5cnM93 4\
35 v40 H400000 v\
-40z M500 241 v4\
0 H400000 v-40z \
M500 241 v40 H40\
0000 v-40z\x22,shor\
trightharpoonabo\
vebar:\x22M53,241l0\
,40c398570,0,399\
437,0,399437,0\x5cn\
c4.7,-4.7,7,-9.3\
,7,-14c0,-9.3,-3\
.7,-15.3,-11,-18\
c-92.7,-56.7,-15\
9,-133.7,-199,\x5cn\
-231c-3.3,-9.3,-\
6,-14.7,-8,-16c-\
2,-1.3,-7,-2,-15\
,-2c-10.7,0,-16.\
7,2,-18,6\x5cnc-2,2\
.7,-1,9.7,3,21c1\
5.3,42,36.7,81.8\
,64,119.5c27.3,3\
7.7,58,69.2,92,9\
4.5z\x5cnM500 241 v\
40 H399408 v-40z\
 M500 435 v40 H4\
00000 v-40z\x22},A=\
function(){funct\
ion e(e){this.ch\
ildren=void 0,th\
is.classes=void \
0,this.height=vo\
id 0,this.depth=\
void 0,this.maxF\
ontSize=void 0,t\
his.style=void 0\
,this.children=e\
,this.classes=[]\
,this.height=0,t\
his.depth=0,this\
.maxFontSize=0,t\
his.style={}}var\
 t=e.prototype;r\
eturn t.hasClass\
=function(e){ret\
urn l.contains(t\
his.classes,e)},\
t.toNode=functio\
n(){for(var e=do\
cument.createDoc\
umentFragment(),\
t=0;t<this.child\
ren.length;t++)e\
.appendChild(thi\
s.children[t].to\
Node());return e\
},t.toMarkup=fun\
ction(){for(var \
e=\x22\x22,t=0;t<this.\
children.length;\
t++)e+=this.chil\
dren[t].toMarkup\
();return e},t.t\
oText=function()\
{var e=function(\
e){return e.toTe\
xt()};return thi\
s.children.map(e\
).join(\x22\x22)},e}()\
,T={\x22AMS-Regular\
\x22:{32:[0,0,0,0,.\
25],65:[0,.68889\
,0,0,.72222],66:\
[0,.68889,0,0,.6\
6667],67:[0,.688\
89,0,0,.72222],6\
8:[0,.68889,0,0,\
.72222],69:[0,.6\
8889,0,0,.66667]\
,70:[0,.68889,0,\
0,.61111],71:[0,\
.68889,0,0,.7777\
8],72:[0,.68889,\
0,0,.77778],73:[\
0,.68889,0,0,.38\
889],74:[.16667,\
.68889,0,0,.5],7\
5:[0,.68889,0,0,\
.77778],76:[0,.6\
8889,0,0,.66667]\
,77:[0,.68889,0,\
0,.94445],78:[0,\
.68889,0,0,.7222\
2],79:[.16667,.6\
8889,0,0,.77778]\
,80:[0,.68889,0,\
0,.61111],81:[.1\
6667,.68889,0,0,\
.77778],82:[0,.6\
8889,0,0,.72222]\
,83:[0,.68889,0,\
0,.55556],84:[0,\
.68889,0,0,.6666\
7],85:[0,.68889,\
0,0,.72222],86:[\
0,.68889,0,0,.72\
222],87:[0,.6888\
9,0,0,1],88:[0,.\
68889,0,0,.72222\
],89:[0,.68889,0\
,0,.72222],90:[0\
,.68889,0,0,.666\
67],107:[0,.6888\
9,0,0,.55556],16\
0:[0,0,0,0,.25],\
165:[0,.675,.025\
,0,.75],174:[.15\
559,.69224,0,0,.\
94666],240:[0,.6\
8889,0,0,.55556]\
,295:[0,.68889,0\
,0,.54028],710:[\
0,.825,0,0,2.333\
34],732:[0,.9,0,\
0,2.33334],770:[\
0,.825,0,0,2.333\
34],771:[0,.9,0,\
0,2.33334],989:[\
.08167,.58167,0,\
0,.77778],1008:[\
0,.43056,.04028,\
0,.66667],8245:[\
0,.54986,0,0,.27\
5],8463:[0,.6888\
9,0,0,.54028],84\
87:[0,.68889,0,0\
,.72222],8498:[0\
,.68889,0,0,.555\
56],8502:[0,.688\
89,0,0,.66667],8\
503:[0,.68889,0,\
0,.44445],8504:[\
0,.68889,0,0,.66\
667],8513:[0,.68\
889,0,0,.63889],\
8592:[-.03598,.4\
6402,0,0,.5],859\
4:[-.03598,.4640\
2,0,0,.5],8602:[\
-.13313,.36687,0\
,0,1],8603:[-.13\
313,.36687,0,0,1\
],8606:[.01354,.\
52239,0,0,1],860\
8:[.01354,.52239\
,0,0,1],8610:[.0\
1354,.52239,0,0,\
1.11111],8611:[.\
01354,.52239,0,0\
,1.11111],8619:[\
0,.54986,0,0,1],\
8620:[0,.54986,0\
,0,1],8621:[-.13\
313,.37788,0,0,1\
.38889],8622:[-.\
13313,.36687,0,0\
,1],8624:[0,.692\
24,0,0,.5],8625:\
[0,.69224,0,0,.5\
],8630:[0,.43056\
,0,0,1],8631:[0,\
.43056,0,0,1],86\
34:[.08198,.5819\
8,0,0,.77778],86\
35:[.08198,.5819\
8,0,0,.77778],86\
38:[.19444,.6922\
4,0,0,.41667],86\
39:[.19444,.6922\
4,0,0,.41667],86\
42:[.19444,.6922\
4,0,0,.41667],86\
43:[.19444,.6922\
4,0,0,.41667],86\
44:[.1808,.675,0\
,0,1],8646:[.180\
8,.675,0,0,1],86\
47:[.1808,.675,0\
,0,1],8648:[.194\
44,.69224,0,0,.8\
3334],8649:[.180\
8,.675,0,0,1],86\
50:[.19444,.6922\
4,0,0,.83334],86\
51:[.01354,.5223\
9,0,0,1],8652:[.\
01354,.52239,0,0\
,1],8653:[-.1331\
3,.36687,0,0,1],\
8654:[-.13313,.3\
6687,0,0,1],8655\
:[-.13313,.36687\
,0,0,1],8666:[.1\
3667,.63667,0,0,\
1],8667:[.13667,\
.63667,0,0,1],86\
69:[-.13313,.377\
88,0,0,1],8672:[\
-.064,.437,0,0,1\
.334],8674:[-.06\
4,.437,0,0,1.334\
],8705:[0,.825,0\
,0,.5],8708:[0,.\
68889,0,0,.55556\
],8709:[.08167,.\
58167,0,0,.77778\
],8717:[0,.43056\
,0,0,.42917],872\
2:[-.03598,.4640\
2,0,0,.5],8724:[\
.08198,.69224,0,\
0,.77778],8726:[\
.08167,.58167,0,\
0,.77778],8733:[\
0,.69224,0,0,.77\
778],8736:[0,.69\
224,0,0,.72222],\
8737:[0,.69224,0\
,0,.72222],8738:\
[.03517,.52239,0\
,0,.72222],8739:\
[.08167,.58167,0\
,0,.22222],8740:\
[.25142,.74111,0\
,0,.27778],8741:\
[.08167,.58167,0\
,0,.38889],8742:\
[.25142,.74111,0\
,0,.5],8756:[0,.\
69224,0,0,.66667\
],8757:[0,.69224\
,0,0,.66667],876\
4:[-.13313,.3668\
7,0,0,.77778],87\
65:[-.13313,.377\
88,0,0,.77778],8\
769:[-.13313,.36\
687,0,0,.77778],\
8770:[-.03625,.4\
6375,0,0,.77778]\
,8774:[.30274,.7\
9383,0,0,.77778]\
,8776:[-.01688,.\
48312,0,0,.77778\
],8778:[.08167,.\
58167,0,0,.77778\
],8782:[.06062,.\
54986,0,0,.77778\
],8783:[.06062,.\
54986,0,0,.77778\
],8785:[.08198,.\
58198,0,0,.77778\
],8786:[.08198,.\
58198,0,0,.77778\
],8787:[.08198,.\
58198,0,0,.77778\
],8790:[0,.69224\
,0,0,.77778],879\
1:[.22958,.72958\
,0,0,.77778],879\
6:[.08198,.91667\
,0,0,.77778],880\
6:[.25583,.75583\
,0,0,.77778],880\
7:[.25583,.75583\
,0,0,.77778],880\
8:[.25142,.75726\
,0,0,.77778],880\
9:[.25142,.75726\
,0,0,.77778],881\
2:[.25583,.75583\
,0,0,.5],8814:[.\
20576,.70576,0,0\
,.77778],8815:[.\
20576,.70576,0,0\
,.77778],8816:[.\
30274,.79383,0,0\
,.77778],8817:[.\
30274,.79383,0,0\
,.77778],8818:[.\
22958,.72958,0,0\
,.77778],8819:[.\
22958,.72958,0,0\
,.77778],8822:[.\
1808,.675,0,0,.7\
7778],8823:[.180\
8,.675,0,0,.7777\
8],8828:[.13667,\
.63667,0,0,.7777\
8],8829:[.13667,\
.63667,0,0,.7777\
8],8830:[.22958,\
.72958,0,0,.7777\
8],8831:[.22958,\
.72958,0,0,.7777\
8],8832:[.20576,\
.70576,0,0,.7777\
8],8833:[.20576,\
.70576,0,0,.7777\
8],8840:[.30274,\
.79383,0,0,.7777\
8],8841:[.30274,\
.79383,0,0,.7777\
8],8842:[.13597,\
.63597,0,0,.7777\
8],8843:[.13597,\
.63597,0,0,.7777\
8],8847:[.03517,\
.54986,0,0,.7777\
8],8848:[.03517,\
.54986,0,0,.7777\
8],8858:[.08198,\
.58198,0,0,.7777\
8],8859:[.08198,\
.58198,0,0,.7777\
8],8861:[.08198,\
.58198,0,0,.7777\
8],8862:[0,.675,\
0,0,.77778],8863\
:[0,.675,0,0,.77\
778],8864:[0,.67\
5,0,0,.77778],88\
65:[0,.675,0,0,.\
77778],8872:[0,.\
69224,0,0,.61111\
],8873:[0,.69224\
,0,0,.72222],887\
4:[0,.69224,0,0,\
.88889],8876:[0,\
.68889,0,0,.6111\
1],8877:[0,.6888\
9,0,0,.61111],88\
78:[0,.68889,0,0\
,.72222],8879:[0\
,.68889,0,0,.722\
22],8882:[.03517\
,.54986,0,0,.777\
78],8883:[.03517\
,.54986,0,0,.777\
78],8884:[.13667\
,.63667,0,0,.777\
78],8885:[.13667\
,.63667,0,0,.777\
78],8888:[0,.549\
86,0,0,1.11111],\
8890:[.19444,.43\
056,0,0,.55556],\
8891:[.19444,.69\
224,0,0,.61111],\
8892:[.19444,.69\
224,0,0,.61111],\
8901:[0,.54986,0\
,0,.27778],8903:\
[.08167,.58167,0\
,0,.77778],8905:\
[.08167,.58167,0\
,0,.77778],8906:\
[.08167,.58167,0\
,0,.77778],8907:\
[0,.69224,0,0,.7\
7778],8908:[0,.6\
9224,0,0,.77778]\
,8909:[-.03598,.\
46402,0,0,.77778\
],8910:[0,.54986\
,0,0,.76042],891\
1:[0,.54986,0,0,\
.76042],8912:[.0\
3517,.54986,0,0,\
.77778],8913:[.0\
3517,.54986,0,0,\
.77778],8914:[0,\
.54986,0,0,.6666\
7],8915:[0,.5498\
6,0,0,.66667],89\
16:[0,.69224,0,0\
,.66667],8918:[.\
0391,.5391,0,0,.\
77778],8919:[.03\
91,.5391,0,0,.77\
778],8920:[.0351\
7,.54986,0,0,1.3\
3334],8921:[.035\
17,.54986,0,0,1.\
33334],8922:[.38\
569,.88569,0,0,.\
77778],8923:[.38\
569,.88569,0,0,.\
77778],8926:[.13\
667,.63667,0,0,.\
77778],8927:[.13\
667,.63667,0,0,.\
77778],8928:[.30\
274,.79383,0,0,.\
77778],8929:[.30\
274,.79383,0,0,.\
77778],8934:[.23\
222,.74111,0,0,.\
77778],8935:[.23\
222,.74111,0,0,.\
77778],8936:[.23\
222,.74111,0,0,.\
77778],8937:[.23\
222,.74111,0,0,.\
77778],8938:[.20\
576,.70576,0,0,.\
77778],8939:[.20\
576,.70576,0,0,.\
77778],8940:[.30\
274,.79383,0,0,.\
77778],8941:[.30\
274,.79383,0,0,.\
77778],8994:[.19\
444,.69224,0,0,.\
77778],8995:[.19\
444,.69224,0,0,.\
77778],9416:[.15\
559,.69224,0,0,.\
90222],9484:[0,.\
69224,0,0,.5],94\
88:[0,.69224,0,0\
,.5],9492:[0,.37\
788,0,0,.5],9496\
:[0,.37788,0,0,.\
5],9585:[.19444,\
.68889,0,0,.8888\
9],9586:[.19444,\
.74111,0,0,.8888\
9],9632:[0,.675,\
0,0,.77778],9633\
:[0,.675,0,0,.77\
778],9650:[0,.54\
986,0,0,.72222],\
9651:[0,.54986,0\
,0,.72222],9654:\
[.03517,.54986,0\
,0,.77778],9660:\
[0,.54986,0,0,.7\
2222],9661:[0,.5\
4986,0,0,.72222]\
,9664:[.03517,.5\
4986,0,0,.77778]\
,9674:[.11111,.6\
9224,0,0,.66667]\
,9733:[.19444,.6\
9224,0,0,.94445]\
,10003:[0,.69224\
,0,0,.83334],100\
16:[0,.69224,0,0\
,.83334],10731:[\
.11111,.69224,0,\
0,.66667],10846:\
[.19444,.75583,0\
,0,.61111],10877\
:[.13667,.63667,\
0,0,.77778],1087\
8:[.13667,.63667\
,0,0,.77778],108\
85:[.25583,.7558\
3,0,0,.77778],10\
886:[.25583,.755\
83,0,0,.77778],1\
0887:[.13597,.63\
597,0,0,.77778],\
10888:[.13597,.6\
3597,0,0,.77778]\
,10889:[.26167,.\
75726,0,0,.77778\
],10890:[.26167,\
.75726,0,0,.7777\
8],10891:[.48256\
,.98256,0,0,.777\
78],10892:[.4825\
6,.98256,0,0,.77\
778],10901:[.136\
67,.63667,0,0,.7\
7778],10902:[.13\
667,.63667,0,0,.\
77778],10933:[.2\
5142,.75726,0,0,\
.77778],10934:[.\
25142,.75726,0,0\
,.77778],10935:[\
.26167,.75726,0,\
0,.77778],10936:\
[.26167,.75726,0\
,0,.77778],10937\
:[.26167,.75726,\
0,0,.77778],1093\
8:[.26167,.75726\
,0,0,.77778],109\
49:[.25583,.7558\
3,0,0,.77778],10\
950:[.25583,.755\
83,0,0,.77778],1\
0955:[.28481,.79\
383,0,0,.77778],\
10956:[.28481,.7\
9383,0,0,.77778]\
,57350:[.08167,.\
58167,0,0,.22222\
],57351:[.08167,\
.58167,0,0,.3888\
9],57352:[.08167\
,.58167,0,0,.777\
78],57353:[0,.43\
056,.04028,0,.66\
667],57356:[.251\
42,.75726,0,0,.7\
7778],57357:[.25\
142,.75726,0,0,.\
77778],57358:[.4\
1951,.91951,0,0,\
.77778],57359:[.\
30274,.79383,0,0\
,.77778],57360:[\
.30274,.79383,0,\
0,.77778],57361:\
[.41951,.91951,0\
,0,.77778],57366\
:[.25142,.75726,\
0,0,.77778],5736\
7:[.25142,.75726\
,0,0,.77778],573\
68:[.25142,.7572\
6,0,0,.77778],57\
369:[.25142,.757\
26,0,0,.77778],5\
7370:[.13597,.63\
597,0,0,.77778],\
57371:[.13597,.6\
3597,0,0,.77778]\
},\x22Caligraphic-R\
egular\x22:{32:[0,0\
,0,0,.25],65:[0,\
.68333,0,.19445,\
.79847],66:[0,.6\
8333,.03041,.138\
89,.65681],67:[0\
,.68333,.05834,.\
13889,.52653],68\
:[0,.68333,.0277\
8,.08334,.77139]\
,69:[0,.68333,.0\
8944,.11111,.527\
78],70:[0,.68333\
,.09931,.11111,.\
71875],71:[.0972\
2,.68333,.0593,.\
11111,.59487],72\
:[0,.68333,.0096\
5,.11111,.84452]\
,73:[0,.68333,.0\
7382,0,.54452],7\
4:[.09722,.68333\
,.18472,.16667,.\
67778],75:[0,.68\
333,.01445,.0555\
6,.76195],76:[0,\
.68333,0,.13889,\
.68972],77:[0,.6\
8333,0,.13889,1.\
2009],78:[0,.683\
33,.14736,.08334\
,.82049],79:[0,.\
68333,.02778,.11\
111,.79611],80:[\
0,.68333,.08222,\
.08334,.69556],8\
1:[.09722,.68333\
,0,.11111,.81667\
],82:[0,.68333,0\
,.08334,.8475],8\
3:[0,.68333,.075\
,.13889,.60556],\
84:[0,.68333,.25\
417,0,.54464],85\
:[0,.68333,.0993\
1,.08334,.62583]\
,86:[0,.68333,.0\
8222,0,.61278],8\
7:[0,.68333,.082\
22,.08334,.98778\
],88:[0,.68333,.\
14643,.13889,.71\
33],89:[.09722,.\
68333,.08222,.08\
334,.66834],90:[\
0,.68333,.07944,\
.13889,.72473],1\
60:[0,0,0,0,.25]\
},\x22Fraktur-Regul\
ar\x22:{32:[0,0,0,0\
,.25],33:[0,.691\
41,0,0,.29574],3\
4:[0,.69141,0,0,\
.21471],38:[0,.6\
9141,0,0,.73786]\
,39:[0,.69141,0,\
0,.21201],40:[.2\
4982,.74947,0,0,\
.38865],41:[.249\
82,.74947,0,0,.3\
8865],42:[0,.621\
19,0,0,.27764],4\
3:[.08319,.58283\
,0,0,.75623],44:\
[0,.10803,0,0,.2\
7764],45:[.08319\
,.58283,0,0,.756\
23],46:[0,.10803\
,0,0,.27764],47:\
[.24982,.74947,0\
,0,.50181],48:[0\
,.47534,0,0,.501\
81],49:[0,.47534\
,0,0,.50181],50:\
[0,.47534,0,0,.5\
0181],51:[.18906\
,.47534,0,0,.501\
81],52:[.18906,.\
47534,0,0,.50181\
],53:[.18906,.47\
534,0,0,.50181],\
54:[0,.69141,0,0\
,.50181],55:[.18\
906,.47534,0,0,.\
50181],56:[0,.69\
141,0,0,.50181],\
57:[.18906,.4753\
4,0,0,.50181],58\
:[0,.47534,0,0,.\
21606],59:[.1260\
4,.47534,0,0,.21\
606],61:[-.13099\
,.36866,0,0,.756\
23],63:[0,.69141\
,0,0,.36245],65:\
[0,.69141,0,0,.7\
176],66:[0,.6914\
1,0,0,.88397],67\
:[0,.69141,0,0,.\
61254],68:[0,.69\
141,0,0,.83158],\
69:[0,.69141,0,0\
,.66278],70:[.12\
604,.69141,0,0,.\
61119],71:[0,.69\
141,0,0,.78539],\
72:[.06302,.6914\
1,0,0,.7203],73:\
[0,.69141,0,0,.5\
5448],74:[.12604\
,.69141,0,0,.552\
31],75:[0,.69141\
,0,0,.66845],76:\
[0,.69141,0,0,.6\
6602],77:[0,.691\
41,0,0,1.04953],\
78:[0,.69141,0,0\
,.83212],79:[0,.\
69141,0,0,.82699\
],80:[.18906,.69\
141,0,0,.82753],\
81:[.03781,.6914\
1,0,0,.82699],82\
:[0,.69141,0,0,.\
82807],83:[0,.69\
141,0,0,.82861],\
84:[0,.69141,0,0\
,.66899],85:[0,.\
69141,0,0,.64576\
],86:[0,.69141,0\
,0,.83131],87:[0\
,.69141,0,0,1.04\
602],88:[0,.6914\
1,0,0,.71922],89\
:[.18906,.69141,\
0,0,.83293],90:[\
.12604,.69141,0,\
0,.60201],91:[.2\
4982,.74947,0,0,\
.27764],93:[.249\
82,.74947,0,0,.2\
7764],94:[0,.691\
41,0,0,.49965],9\
7:[0,.47534,0,0,\
.50046],98:[0,.6\
9141,0,0,.51315]\
,99:[0,.47534,0,\
0,.38946],100:[0\
,.62119,0,0,.498\
57],101:[0,.4753\
4,0,0,.40053],10\
2:[.18906,.69141\
,0,0,.32626],103\
:[.18906,.47534,\
0,0,.5037],104:[\
.18906,.69141,0,\
0,.52126],105:[0\
,.69141,0,0,.278\
99],106:[0,.6914\
1,0,0,.28088],10\
7:[0,.69141,0,0,\
.38946],108:[0,.\
69141,0,0,.27953\
],109:[0,.47534,\
0,0,.76676],110:\
[0,.47534,0,0,.5\
2666],111:[0,.47\
534,0,0,.48885],\
112:[.18906,.523\
96,0,0,.50046],1\
13:[.18906,.4753\
4,0,0,.48912],11\
4:[0,.47534,0,0,\
.38919],115:[0,.\
47534,0,0,.44266\
],116:[0,.62119,\
0,0,.33301],117:\
[0,.47534,0,0,.5\
172],118:[0,.523\
96,0,0,.5118],11\
9:[0,.52396,0,0,\
.77351],120:[.18\
906,.47534,0,0,.\
38865],121:[.189\
06,.47534,0,0,.4\
9884],122:[.1890\
6,.47534,0,0,.39\
054],160:[0,0,0,\
0,.25],8216:[0,.\
69141,0,0,.21471\
],8217:[0,.69141\
,0,0,.21471],581\
12:[0,.62119,0,0\
,.49749],58113:[\
0,.62119,0,0,.49\
83],58114:[.1890\
6,.69141,0,0,.33\
328],58115:[.189\
06,.69141,0,0,.3\
2923],58116:[.18\
906,.47534,0,0,.\
50343],58117:[0,\
.69141,0,0,.3330\
1],58118:[0,.621\
19,0,0,.33409],5\
8119:[0,.47534,0\
,0,.50073]},\x22Mai\
n-Bold\x22:{32:[0,0\
,0,0,.25],33:[0,\
.69444,0,0,.35],\
34:[0,.69444,0,0\
,.60278],35:[.19\
444,.69444,0,0,.\
95833],36:[.0555\
6,.75,0,0,.575],\
37:[.05556,.75,0\
,0,.95833],38:[0\
,.69444,0,0,.894\
44],39:[0,.69444\
,0,0,.31944],40:\
[.25,.75,0,0,.44\
722],41:[.25,.75\
,0,0,.44722],42:\
[0,.75,0,0,.575]\
,43:[.13333,.633\
33,0,0,.89444],4\
4:[.19444,.15556\
,0,0,.31944],45:\
[0,.44444,0,0,.3\
8333],46:[0,.155\
56,0,0,.31944],4\
7:[.25,.75,0,0,.\
575],48:[0,.6444\
4,0,0,.575],49:[\
0,.64444,0,0,.57\
5],50:[0,.64444,\
0,0,.575],51:[0,\
.64444,0,0,.575]\
,52:[0,.64444,0,\
0,.575],53:[0,.6\
4444,0,0,.575],5\
4:[0,.64444,0,0,\
.575],55:[0,.644\
44,0,0,.575],56:\
[0,.64444,0,0,.5\
75],57:[0,.64444\
,0,0,.575],58:[0\
,.44444,0,0,.319\
44],59:[.19444,.\
44444,0,0,.31944\
],60:[.08556,.58\
556,0,0,.89444],\
61:[-.10889,.391\
11,0,0,.89444],6\
2:[.08556,.58556\
,0,0,.89444],63:\
[0,.69444,0,0,.5\
4305],64:[0,.694\
44,0,0,.89444],6\
5:[0,.68611,0,0,\
.86944],66:[0,.6\
8611,0,0,.81805]\
,67:[0,.68611,0,\
0,.83055],68:[0,\
.68611,0,0,.8819\
4],69:[0,.68611,\
0,0,.75555],70:[\
0,.68611,0,0,.72\
361],71:[0,.6861\
1,0,0,.90416],72\
:[0,.68611,0,0,.\
9],73:[0,.68611,\
0,0,.43611],74:[\
0,.68611,0,0,.59\
444],75:[0,.6861\
1,0,0,.90138],76\
:[0,.68611,0,0,.\
69166],77:[0,.68\
611,0,0,1.09166]\
,78:[0,.68611,0,\
0,.9],79:[0,.686\
11,0,0,.86388],8\
0:[0,.68611,0,0,\
.78611],81:[.194\
44,.68611,0,0,.8\
6388],82:[0,.686\
11,0,0,.8625],83\
:[0,.68611,0,0,.\
63889],84:[0,.68\
611,0,0,.8],85:[\
0,.68611,0,0,.88\
472],86:[0,.6861\
1,.01597,0,.8694\
4],87:[0,.68611,\
.01597,0,1.18888\
],88:[0,.68611,0\
,0,.86944],89:[0\
,.68611,.02875,0\
,.86944],90:[0,.\
68611,0,0,.70277\
],91:[.25,.75,0,\
0,.31944],92:[.2\
5,.75,0,0,.575],\
93:[.25,.75,0,0,\
.31944],94:[0,.6\
9444,0,0,.575],9\
5:[.31,.13444,.0\
3194,0,.575],97:\
[0,.44444,0,0,.5\
5902],98:[0,.694\
44,0,0,.63889],9\
9:[0,.44444,0,0,\
.51111],100:[0,.\
69444,0,0,.63889\
],101:[0,.44444,\
0,0,.52708],102:\
[0,.69444,.10903\
,0,.35139],103:[\
.19444,.44444,.0\
1597,0,.575],104\
:[0,.69444,0,0,.\
63889],105:[0,.6\
9444,0,0,.31944]\
,106:[.19444,.69\
444,0,0,.35139],\
107:[0,.69444,0,\
0,.60694],108:[0\
,.69444,0,0,.319\
44],109:[0,.4444\
4,0,0,.95833],11\
0:[0,.44444,0,0,\
.63889],111:[0,.\
44444,0,0,.575],\
112:[.19444,.444\
44,0,0,.63889],1\
13:[.19444,.4444\
4,0,0,.60694],11\
4:[0,.44444,0,0,\
.47361],115:[0,.\
44444,0,0,.45361\
],116:[0,.63492,\
0,0,.44722],117:\
[0,.44444,0,0,.6\
3889],118:[0,.44\
444,.01597,0,.60\
694],119:[0,.444\
44,.01597,0,.830\
55],120:[0,.4444\
4,0,0,.60694],12\
1:[.19444,.44444\
,.01597,0,.60694\
],122:[0,.44444,\
0,0,.51111],123:\
[.25,.75,0,0,.57\
5],124:[.25,.75,\
0,0,.31944],125:\
[.25,.75,0,0,.57\
5],126:[.35,.344\
44,0,0,.575],160\
:[0,0,0,0,.25],1\
63:[0,.69444,0,0\
,.86853],168:[0,\
.69444,0,0,.575]\
,172:[0,.44444,0\
,0,.76666],176:[\
0,.69444,0,0,.86\
944],177:[.13333\
,.63333,0,0,.894\
44],184:[.17014,\
0,0,0,.51111],19\
8:[0,.68611,0,0,\
1.04166],215:[.1\
3333,.63333,0,0,\
.89444],216:[.04\
861,.73472,0,0,.\
89444],223:[0,.6\
9444,0,0,.59722]\
,230:[0,.44444,0\
,0,.83055],247:[\
.13333,.63333,0,\
0,.89444],248:[.\
09722,.54167,0,0\
,.575],305:[0,.4\
4444,0,0,.31944]\
,338:[0,.68611,0\
,0,1.16944],339:\
[0,.44444,0,0,.8\
9444],567:[.1944\
4,.44444,0,0,.35\
139],710:[0,.694\
44,0,0,.575],711\
:[0,.63194,0,0,.\
575],713:[0,.596\
11,0,0,.575],714\
:[0,.69444,0,0,.\
575],715:[0,.694\
44,0,0,.575],728\
:[0,.69444,0,0,.\
575],729:[0,.694\
44,0,0,.31944],7\
30:[0,.69444,0,0\
,.86944],732:[0,\
.69444,0,0,.575]\
,733:[0,.69444,0\
,0,.575],915:[0,\
.68611,0,0,.6916\
6],916:[0,.68611\
,0,0,.95833],920\
:[0,.68611,0,0,.\
89444],923:[0,.6\
8611,0,0,.80555]\
,926:[0,.68611,0\
,0,.76666],928:[\
0,.68611,0,0,.9]\
,931:[0,.68611,0\
,0,.83055],933:[\
0,.68611,0,0,.89\
444],934:[0,.686\
11,0,0,.83055],9\
36:[0,.68611,0,0\
,.89444],937:[0,\
.68611,0,0,.8305\
5],8211:[0,.4444\
4,.03194,0,.575]\
,8212:[0,.44444,\
.03194,0,1.14999\
],8216:[0,.69444\
,0,0,.31944],821\
7:[0,.69444,0,0,\
.31944],8220:[0,\
.69444,0,0,.6027\
8],8221:[0,.6944\
4,0,0,.60278],82\
24:[.19444,.6944\
4,0,0,.51111],82\
25:[.19444,.6944\
4,0,0,.51111],82\
42:[0,.55556,0,0\
,.34444],8407:[0\
,.72444,.15486,0\
,.575],8463:[0,.\
69444,0,0,.66759\
],8465:[0,.69444\
,0,0,.83055],846\
7:[0,.69444,0,0,\
.47361],8472:[.1\
9444,.44444,0,0,\
.74027],8476:[0,\
.69444,0,0,.8305\
5],8501:[0,.6944\
4,0,0,.70277],85\
92:[-.10889,.391\
11,0,0,1.14999],\
8593:[.19444,.69\
444,0,0,.575],85\
94:[-.10889,.391\
11,0,0,1.14999],\
8595:[.19444,.69\
444,0,0,.575],85\
96:[-.10889,.391\
11,0,0,1.14999],\
8597:[.25,.75,0,\
0,.575],8598:[.1\
9444,.69444,0,0,\
1.14999],8599:[.\
19444,.69444,0,0\
,1.14999],8600:[\
.19444,.69444,0,\
0,1.14999],8601:\
[.19444,.69444,0\
,0,1.14999],8636\
:[-.10889,.39111\
,0,0,1.14999],86\
37:[-.10889,.391\
11,0,0,1.14999],\
8640:[-.10889,.3\
9111,0,0,1.14999\
],8641:[-.10889,\
.39111,0,0,1.149\
99],8656:[-.1088\
9,.39111,0,0,1.1\
4999],8657:[.194\
44,.69444,0,0,.7\
0277],8658:[-.10\
889,.39111,0,0,1\
.14999],8659:[.1\
9444,.69444,0,0,\
.70277],8660:[-.\
10889,.39111,0,0\
,1.14999],8661:[\
.25,.75,0,0,.702\
77],8704:[0,.694\
44,0,0,.63889],8\
706:[0,.69444,.0\
6389,0,.62847],8\
707:[0,.69444,0,\
0,.63889],8709:[\
.05556,.75,0,0,.\
575],8711:[0,.68\
611,0,0,.95833],\
8712:[.08556,.58\
556,0,0,.76666],\
8715:[.08556,.58\
556,0,0,.76666],\
8722:[.13333,.63\
333,0,0,.89444],\
8723:[.13333,.63\
333,0,0,.89444],\
8725:[.25,.75,0,\
0,.575],8726:[.2\
5,.75,0,0,.575],\
8727:[-.02778,.4\
7222,0,0,.575],8\
728:[-.02639,.47\
361,0,0,.575],87\
29:[-.02639,.473\
61,0,0,.575],873\
0:[.18,.82,0,0,.\
95833],8733:[0,.\
44444,0,0,.89444\
],8734:[0,.44444\
,0,0,1.14999],87\
36:[0,.69224,0,0\
,.72222],8739:[.\
25,.75,0,0,.3194\
4],8741:[.25,.75\
,0,0,.575],8743:\
[0,.55556,0,0,.7\
6666],8744:[0,.5\
5556,0,0,.76666]\
,8745:[0,.55556,\
0,0,.76666],8746\
:[0,.55556,0,0,.\
76666],8747:[.19\
444,.69444,.1277\
8,0,.56875],8764\
:[-.10889,.39111\
,0,0,.89444],876\
8:[.19444,.69444\
,0,0,.31944],877\
1:[.00222,.50222\
,0,0,.89444],877\
3:[.027,.638,0,0\
,.894],8776:[.02\
444,.52444,0,0,.\
89444],8781:[.00\
222,.50222,0,0,.\
89444],8801:[.00\
222,.50222,0,0,.\
89444],8804:[.19\
667,.69667,0,0,.\
89444],8805:[.19\
667,.69667,0,0,.\
89444],8810:[.08\
556,.58556,0,0,1\
.14999],8811:[.0\
8556,.58556,0,0,\
1.14999],8826:[.\
08556,.58556,0,0\
,.89444],8827:[.\
08556,.58556,0,0\
,.89444],8834:[.\
08556,.58556,0,0\
,.89444],8835:[.\
08556,.58556,0,0\
,.89444],8838:[.\
19667,.69667,0,0\
,.89444],8839:[.\
19667,.69667,0,0\
,.89444],8846:[0\
,.55556,0,0,.766\
66],8849:[.19667\
,.69667,0,0,.894\
44],8850:[.19667\
,.69667,0,0,.894\
44],8851:[0,.555\
56,0,0,.76666],8\
852:[0,.55556,0,\
0,.76666],8853:[\
.13333,.63333,0,\
0,.89444],8854:[\
.13333,.63333,0,\
0,.89444],8855:[\
.13333,.63333,0,\
0,.89444],8856:[\
.13333,.63333,0,\
0,.89444],8857:[\
.13333,.63333,0,\
0,.89444],8866:[\
0,.69444,0,0,.70\
277],8867:[0,.69\
444,0,0,.70277],\
8868:[0,.69444,0\
,0,.89444],8869:\
[0,.69444,0,0,.8\
9444],8900:[-.02\
639,.47361,0,0,.\
575],8901:[-.026\
39,.47361,0,0,.3\
1944],8902:[-.02\
778,.47222,0,0,.\
575],8968:[.25,.\
75,0,0,.51111],8\
969:[.25,.75,0,0\
,.51111],8970:[.\
25,.75,0,0,.5111\
1],8971:[.25,.75\
,0,0,.51111],899\
4:[-.13889,.3611\
1,0,0,1.14999],8\
995:[-.13889,.36\
111,0,0,1.14999]\
,9651:[.19444,.6\
9444,0,0,1.02222\
],9657:[-.02778,\
.47222,0,0,.575]\
,9661:[.19444,.6\
9444,0,0,1.02222\
],9667:[-.02778,\
.47222,0,0,.575]\
,9711:[.19444,.6\
9444,0,0,1.14999\
],9824:[.12963,.\
69444,0,0,.89444\
],9825:[.12963,.\
69444,0,0,.89444\
],9826:[.12963,.\
69444,0,0,.89444\
],9827:[.12963,.\
69444,0,0,.89444\
],9837:[0,.75,0,\
0,.44722],9838:[\
.19444,.69444,0,\
0,.44722],9839:[\
.19444,.69444,0,\
0,.44722],10216:\
[.25,.75,0,0,.44\
722],10217:[.25,\
.75,0,0,.44722],\
10815:[0,.68611,\
0,0,.9],10927:[.\
19667,.69667,0,0\
,.89444],10928:[\
.19667,.69667,0,\
0,.89444],57376:\
[.19444,.69444,0\
,0,0]},\x22Main-Bol\
dItalic\x22:{32:[0,\
0,0,0,.25],33:[0\
,.69444,.11417,0\
,.38611],34:[0,.\
69444,.07939,0,.\
62055],35:[.1944\
4,.69444,.06833,\
0,.94444],37:[.0\
5556,.75,.12861,\
0,.94444],38:[0,\
.69444,.08528,0,\
.88555],39:[0,.6\
9444,.12945,0,.3\
5555],40:[.25,.7\
5,.15806,0,.4733\
3],41:[.25,.75,.\
03306,0,.47333],\
42:[0,.75,.14333\
,0,.59111],43:[.\
10333,.60333,.03\
306,0,.88555],44\
:[.19444,.14722,\
0,0,.35555],45:[\
0,.44444,.02611,\
0,.41444],46:[0,\
.14722,0,0,.3555\
5],47:[.25,.75,.\
15806,0,.59111],\
48:[0,.64444,.13\
167,0,.59111],49\
:[0,.64444,.1316\
7,0,.59111],50:[\
0,.64444,.13167,\
0,.59111],51:[0,\
.64444,.13167,0,\
.59111],52:[.194\
44,.64444,.13167\
,0,.59111],53:[0\
,.64444,.13167,0\
,.59111],54:[0,.\
64444,.13167,0,.\
59111],55:[.1944\
4,.64444,.13167,\
0,.59111],56:[0,\
.64444,.13167,0,\
.59111],57:[0,.6\
4444,.13167,0,.5\
9111],58:[0,.444\
44,.06695,0,.355\
55],59:[.19444,.\
44444,.06695,0,.\
35555],61:[-.108\
89,.39111,.06833\
,0,.88555],63:[0\
,.69444,.11472,0\
,.59111],64:[0,.\
69444,.09208,0,.\
88555],65:[0,.68\
611,0,0,.86555],\
66:[0,.68611,.09\
92,0,.81666],67:\
[0,.68611,.14208\
,0,.82666],68:[0\
,.68611,.09062,0\
,.87555],69:[0,.\
68611,.11431,0,.\
75666],70:[0,.68\
611,.12903,0,.72\
722],71:[0,.6861\
1,.07347,0,.8952\
7],72:[0,.68611,\
.17208,0,.8961],\
73:[0,.68611,.15\
681,0,.47166],74\
:[0,.68611,.145,\
0,.61055],75:[0,\
.68611,.14208,0,\
.89499],76:[0,.6\
8611,0,0,.69777]\
,77:[0,.68611,.1\
7208,0,1.07277],\
78:[0,.68611,.17\
208,0,.8961],79:\
[0,.68611,.09062\
,0,.85499],80:[0\
,.68611,.0992,0,\
.78721],81:[.194\
44,.68611,.09062\
,0,.85499],82:[0\
,.68611,.02559,0\
,.85944],83:[0,.\
68611,.11264,0,.\
64999],84:[0,.68\
611,.12903,0,.79\
61],85:[0,.68611\
,.17208,0,.88083\
],86:[0,.68611,.\
18625,0,.86555],\
87:[0,.68611,.18\
625,0,1.15999],8\
8:[0,.68611,.156\
81,0,.86555],89:\
[0,.68611,.19803\
,0,.86555],90:[0\
,.68611,.14208,0\
,.70888],91:[.25\
,.75,.1875,0,.35\
611],93:[.25,.75\
,.09972,0,.35611\
],94:[0,.69444,.\
06709,0,.59111],\
95:[.31,.13444,.\
09811,0,.59111],\
97:[0,.44444,.09\
426,0,.59111],98\
:[0,.69444,.0786\
1,0,.53222],99:[\
0,.44444,.05222,\
0,.53222],100:[0\
,.69444,.10861,0\
,.59111],101:[0,\
.44444,.085,0,.5\
3222],102:[.1944\
4,.69444,.21778,\
0,.4],103:[.1944\
4,.44444,.105,0,\
.53222],104:[0,.\
69444,.09426,0,.\
59111],105:[0,.6\
9326,.11387,0,.3\
5555],106:[.1944\
4,.69326,.1672,0\
,.35555],107:[0,\
.69444,.11111,0,\
.53222],108:[0,.\
69444,.10861,0,.\
29666],109:[0,.4\
4444,.09426,0,.9\
4444],110:[0,.44\
444,.09426,0,.64\
999],111:[0,.444\
44,.07861,0,.591\
11],112:[.19444,\
.44444,.07861,0,\
.59111],113:[.19\
444,.44444,.105,\
0,.53222],114:[0\
,.44444,.11111,0\
,.50167],115:[0,\
.44444,.08167,0,\
.48694],116:[0,.\
63492,.09639,0,.\
385],117:[0,.444\
44,.09426,0,.620\
55],118:[0,.4444\
4,.11111,0,.5322\
2],119:[0,.44444\
,.11111,0,.76777\
],120:[0,.44444,\
.12583,0,.56055]\
,121:[.19444,.44\
444,.105,0,.5616\
6],122:[0,.44444\
,.13889,0,.49055\
],126:[.35,.3444\
4,.11472,0,.5911\
1],160:[0,0,0,0,\
.25],168:[0,.694\
44,.11473,0,.591\
11],176:[0,.6944\
4,0,0,.94888],18\
4:[.17014,0,0,0,\
.53222],198:[0,.\
68611,.11431,0,1\
.02277],216:[.04\
861,.73472,.0906\
2,0,.88555],223:\
[.19444,.69444,.\
09736,0,.665],23\
0:[0,.44444,.085\
,0,.82666],248:[\
.09722,.54167,.0\
9458,0,.59111],3\
05:[0,.44444,.09\
426,0,.35555],33\
8:[0,.68611,.114\
31,0,1.14054],33\
9:[0,.44444,.085\
,0,.82666],567:[\
.19444,.44444,.0\
4611,0,.385],710\
:[0,.69444,.0670\
9,0,.59111],711:\
[0,.63194,.08271\
,0,.59111],713:[\
0,.59444,.10444,\
0,.59111],714:[0\
,.69444,.08528,0\
,.59111],715:[0,\
.69444,0,0,.5911\
1],728:[0,.69444\
,.10333,0,.59111\
],729:[0,.69444,\
.12945,0,.35555]\
,730:[0,.69444,0\
,0,.94888],732:[\
0,.69444,.11472,\
0,.59111],733:[0\
,.69444,.11472,0\
,.59111],915:[0,\
.68611,.12903,0,\
.69777],916:[0,.\
68611,0,0,.94444\
],920:[0,.68611,\
.09062,0,.88555]\
,923:[0,.68611,0\
,0,.80666],926:[\
0,.68611,.15092,\
0,.76777],928:[0\
,.68611,.17208,0\
,.8961],931:[0,.\
68611,.11431,0,.\
82666],933:[0,.6\
8611,.10778,0,.8\
8555],934:[0,.68\
611,.05632,0,.82\
666],936:[0,.686\
11,.10778,0,.885\
55],937:[0,.6861\
1,.0992,0,.82666\
],8211:[0,.44444\
,.09811,0,.59111\
],8212:[0,.44444\
,.09811,0,1.1822\
1],8216:[0,.6944\
4,.12945,0,.3555\
5],8217:[0,.6944\
4,.12945,0,.3555\
5],8220:[0,.6944\
4,.16772,0,.6205\
5],8221:[0,.6944\
4,.07939,0,.6205\
5]},\x22Main-Italic\
\x22:{32:[0,0,0,0,.\
25],33:[0,.69444\
,.12417,0,.30667\
],34:[0,.69444,.\
06961,0,.51444],\
35:[.19444,.6944\
4,.06616,0,.8177\
7],37:[.05556,.7\
5,.13639,0,.8177\
7],38:[0,.69444,\
.09694,0,.76666]\
,39:[0,.69444,.1\
2417,0,.30667],4\
0:[.25,.75,.1619\
4,0,.40889],41:[\
.25,.75,.03694,0\
,.40889],42:[0,.\
75,.14917,0,.511\
11],43:[.05667,.\
56167,.03694,0,.\
76666],44:[.1944\
4,.10556,0,0,.30\
667],45:[0,.4305\
6,.02826,0,.3577\
8],46:[0,.10556,\
0,0,.30667],47:[\
.25,.75,.16194,0\
,.51111],48:[0,.\
64444,.13556,0,.\
51111],49:[0,.64\
444,.13556,0,.51\
111],50:[0,.6444\
4,.13556,0,.5111\
1],51:[0,.64444,\
.13556,0,.51111]\
,52:[.19444,.644\
44,.13556,0,.511\
11],53:[0,.64444\
,.13556,0,.51111\
],54:[0,.64444,.\
13556,0,.51111],\
55:[.19444,.6444\
4,.13556,0,.5111\
1],56:[0,.64444,\
.13556,0,.51111]\
,57:[0,.64444,.1\
3556,0,.51111],5\
8:[0,.43056,.058\
2,0,.30667],59:[\
.19444,.43056,.0\
582,0,.30667],61\
:[-.13313,.36687\
,.06616,0,.76666\
],63:[0,.69444,.\
1225,0,.51111],6\
4:[0,.69444,.095\
97,0,.76666],65:\
[0,.68333,0,0,.7\
4333],66:[0,.683\
33,.10257,0,.703\
89],67:[0,.68333\
,.14528,0,.71555\
],68:[0,.68333,.\
09403,0,.755],69\
:[0,.68333,.1202\
8,0,.67833],70:[\
0,.68333,.13305,\
0,.65277],71:[0,\
.68333,.08722,0,\
.77361],72:[0,.6\
8333,.16389,0,.7\
4333],73:[0,.683\
33,.15806,0,.385\
55],74:[0,.68333\
,.14028,0,.525],\
75:[0,.68333,.14\
528,0,.76888],76\
:[0,.68333,0,0,.\
62722],77:[0,.68\
333,.16389,0,.89\
666],78:[0,.6833\
3,.16389,0,.7433\
3],79:[0,.68333,\
.09403,0,.76666]\
,80:[0,.68333,.1\
0257,0,.67833],8\
1:[.19444,.68333\
,.09403,0,.76666\
],82:[0,.68333,.\
03868,0,.72944],\
83:[0,.68333,.11\
972,0,.56222],84\
:[0,.68333,.1330\
5,0,.71555],85:[\
0,.68333,.16389,\
0,.74333],86:[0,\
.68333,.18361,0,\
.74333],87:[0,.6\
8333,.18361,0,.9\
9888],88:[0,.683\
33,.15806,0,.743\
33],89:[0,.68333\
,.19383,0,.74333\
],90:[0,.68333,.\
14528,0,.61333],\
91:[.25,.75,.187\
5,0,.30667],93:[\
.25,.75,.10528,0\
,.30667],94:[0,.\
69444,.06646,0,.\
51111],95:[.31,.\
12056,.09208,0,.\
51111],97:[0,.43\
056,.07671,0,.51\
111],98:[0,.6944\
4,.06312,0,.46],\
99:[0,.43056,.05\
653,0,.46],100:[\
0,.69444,.10333,\
0,.51111],101:[0\
,.43056,.07514,0\
,.46],102:[.1944\
4,.69444,.21194,\
0,.30667],103:[.\
19444,.43056,.08\
847,0,.46],104:[\
0,.69444,.07671,\
0,.51111],105:[0\
,.65536,.1019,0,\
.30667],106:[.19\
444,.65536,.1446\
7,0,.30667],107:\
[0,.69444,.10764\
,0,.46],108:[0,.\
69444,.10333,0,.\
25555],109:[0,.4\
3056,.07671,0,.8\
1777],110:[0,.43\
056,.07671,0,.56\
222],111:[0,.430\
56,.06312,0,.511\
11],112:[.19444,\
.43056,.06312,0,\
.51111],113:[.19\
444,.43056,.0884\
7,0,.46],114:[0,\
.43056,.10764,0,\
.42166],115:[0,.\
43056,.08208,0,.\
40889],116:[0,.6\
1508,.09486,0,.3\
3222],117:[0,.43\
056,.07671,0,.53\
666],118:[0,.430\
56,.10764,0,.46]\
,119:[0,.43056,.\
10764,0,.66444],\
120:[0,.43056,.1\
2042,0,.46389],1\
21:[.19444,.4305\
6,.08847,0,.4855\
5],122:[0,.43056\
,.12292,0,.40889\
],126:[.35,.3178\
6,.11585,0,.5111\
1],160:[0,0,0,0,\
.25],168:[0,.667\
86,.10474,0,.511\
11],176:[0,.6944\
4,0,0,.83129],18\
4:[.17014,0,0,0,\
.46],198:[0,.683\
33,.12028,0,.882\
77],216:[.04861,\
.73194,.09403,0,\
.76666],223:[.19\
444,.69444,.1051\
4,0,.53666],230:\
[0,.43056,.07514\
,0,.71555],248:[\
.09722,.52778,.0\
9194,0,.51111],3\
38:[0,.68333,.12\
028,0,.98499],33\
9:[0,.43056,.075\
14,0,.71555],710\
:[0,.69444,.0664\
6,0,.51111],711:\
[0,.62847,.08295\
,0,.51111],713:[\
0,.56167,.10333,\
0,.51111],714:[0\
,.69444,.09694,0\
,.51111],715:[0,\
.69444,0,0,.5111\
1],728:[0,.69444\
,.10806,0,.51111\
],729:[0,.66786,\
.11752,0,.30667]\
,730:[0,.69444,0\
,0,.83129],732:[\
0,.66786,.11585,\
0,.51111],733:[0\
,.69444,.1225,0,\
.51111],915:[0,.\
68333,.13305,0,.\
62722],916:[0,.6\
8333,0,0,.81777]\
,920:[0,.68333,.\
09403,0,.76666],\
923:[0,.68333,0,\
0,.69222],926:[0\
,.68333,.15294,0\
,.66444],928:[0,\
.68333,.16389,0,\
.74333],931:[0,.\
68333,.12028,0,.\
71555],933:[0,.6\
8333,.11111,0,.7\
6666],934:[0,.68\
333,.05986,0,.71\
555],936:[0,.683\
33,.11111,0,.766\
66],937:[0,.6833\
3,.10257,0,.7155\
5],8211:[0,.4305\
6,.09208,0,.5111\
1],8212:[0,.4305\
6,.09208,0,1.022\
22],8216:[0,.694\
44,.12417,0,.306\
67],8217:[0,.694\
44,.12417,0,.306\
67],8220:[0,.694\
44,.1685,0,.5144\
4],8221:[0,.6944\
4,.06961,0,.5144\
4],8463:[0,.6888\
9,0,0,.54028]},\x22\
Main-Regular\x22:{3\
2:[0,0,0,0,.25],\
33:[0,.69444,0,0\
,.27778],34:[0,.\
69444,0,0,.5],35\
:[.19444,.69444,\
0,0,.83334],36:[\
.05556,.75,0,0,.\
5],37:[.05556,.7\
5,0,0,.83334],38\
:[0,.69444,0,0,.\
77778],39:[0,.69\
444,0,0,.27778],\
40:[.25,.75,0,0,\
.38889],41:[.25,\
.75,0,0,.38889],\
42:[0,.75,0,0,.5\
],43:[.08333,.58\
333,0,0,.77778],\
44:[.19444,.1055\
6,0,0,.27778],45\
:[0,.43056,0,0,.\
33333],46:[0,.10\
556,0,0,.27778],\
47:[.25,.75,0,0,\
.5],48:[0,.64444\
,0,0,.5],49:[0,.\
64444,0,0,.5],50\
:[0,.64444,0,0,.\
5],51:[0,.64444,\
0,0,.5],52:[0,.6\
4444,0,0,.5],53:\
[0,.64444,0,0,.5\
],54:[0,.64444,0\
,0,.5],55:[0,.64\
444,0,0,.5],56:[\
0,.64444,0,0,.5]\
,57:[0,.64444,0,\
0,.5],58:[0,.430\
56,0,0,.27778],5\
9:[.19444,.43056\
,0,0,.27778],60:\
[.0391,.5391,0,0\
,.77778],61:[-.1\
3313,.36687,0,0,\
.77778],62:[.039\
1,.5391,0,0,.777\
78],63:[0,.69444\
,0,0,.47222],64:\
[0,.69444,0,0,.7\
7778],65:[0,.683\
33,0,0,.75],66:[\
0,.68333,0,0,.70\
834],67:[0,.6833\
3,0,0,.72222],68\
:[0,.68333,0,0,.\
76389],69:[0,.68\
333,0,0,.68056],\
70:[0,.68333,0,0\
,.65278],71:[0,.\
68333,0,0,.78472\
],72:[0,.68333,0\
,0,.75],73:[0,.6\
8333,0,0,.36111]\
,74:[0,.68333,0,\
0,.51389],75:[0,\
.68333,0,0,.7777\
8],76:[0,.68333,\
0,0,.625],77:[0,\
.68333,0,0,.9166\
7],78:[0,.68333,\
0,0,.75],79:[0,.\
68333,0,0,.77778\
],80:[0,.68333,0\
,0,.68056],81:[.\
19444,.68333,0,0\
,.77778],82:[0,.\
68333,0,0,.73611\
],83:[0,.68333,0\
,0,.55556],84:[0\
,.68333,0,0,.722\
22],85:[0,.68333\
,0,0,.75],86:[0,\
.68333,.01389,0,\
.75],87:[0,.6833\
3,.01389,0,1.027\
78],88:[0,.68333\
,0,0,.75],89:[0,\
.68333,.025,0,.7\
5],90:[0,.68333,\
0,0,.61111],91:[\
.25,.75,0,0,.277\
78],92:[.25,.75,\
0,0,.5],93:[.25,\
.75,0,0,.27778],\
94:[0,.69444,0,0\
,.5],95:[.31,.12\
056,.02778,0,.5]\
,97:[0,.43056,0,\
0,.5],98:[0,.694\
44,0,0,.55556],9\
9:[0,.43056,0,0,\
.44445],100:[0,.\
69444,0,0,.55556\
],101:[0,.43056,\
0,0,.44445],102:\
[0,.69444,.07778\
,0,.30556],103:[\
.19444,.43056,.0\
1389,0,.5],104:[\
0,.69444,0,0,.55\
556],105:[0,.667\
86,0,0,.27778],1\
06:[.19444,.6678\
6,0,0,.30556],10\
7:[0,.69444,0,0,\
.52778],108:[0,.\
69444,0,0,.27778\
],109:[0,.43056,\
0,0,.83334],110:\
[0,.43056,0,0,.5\
5556],111:[0,.43\
056,0,0,.5],112:\
[.19444,.43056,0\
,0,.55556],113:[\
.19444,.43056,0,\
0,.52778],114:[0\
,.43056,0,0,.391\
67],115:[0,.4305\
6,0,0,.39445],11\
6:[0,.61508,0,0,\
.38889],117:[0,.\
43056,0,0,.55556\
],118:[0,.43056,\
.01389,0,.52778]\
,119:[0,.43056,.\
01389,0,.72222],\
120:[0,.43056,0,\
0,.52778],121:[.\
19444,.43056,.01\
389,0,.52778],12\
2:[0,.43056,0,0,\
.44445],123:[.25\
,.75,0,0,.5],124\
:[.25,.75,0,0,.2\
7778],125:[.25,.\
75,0,0,.5],126:[\
.35,.31786,0,0,.\
5],160:[0,0,0,0,\
.25],163:[0,.694\
44,0,0,.76909],1\
67:[.19444,.6944\
4,0,0,.44445],16\
8:[0,.66786,0,0,\
.5],172:[0,.4305\
6,0,0,.66667],17\
6:[0,.69444,0,0,\
.75],177:[.08333\
,.58333,0,0,.777\
78],182:[.19444,\
.69444,0,0,.6111\
1],184:[.17014,0\
,0,0,.44445],198\
:[0,.68333,0,0,.\
90278],215:[.083\
33,.58333,0,0,.7\
7778],216:[.0486\
1,.73194,0,0,.77\
778],223:[0,.694\
44,0,0,.5],230:[\
0,.43056,0,0,.72\
222],247:[.08333\
,.58333,0,0,.777\
78],248:[.09722,\
.52778,0,0,.5],3\
05:[0,.43056,0,0\
,.27778],338:[0,\
.68333,0,0,1.013\
89],339:[0,.4305\
6,0,0,.77778],56\
7:[.19444,.43056\
,0,0,.30556],710\
:[0,.69444,0,0,.\
5],711:[0,.62847\
,0,0,.5],713:[0,\
.56778,0,0,.5],7\
14:[0,.69444,0,0\
,.5],715:[0,.694\
44,0,0,.5],728:[\
0,.69444,0,0,.5]\
,729:[0,.66786,0\
,0,.27778],730:[\
0,.69444,0,0,.75\
],732:[0,.66786,\
0,0,.5],733:[0,.\
69444,0,0,.5],91\
5:[0,.68333,0,0,\
.625],916:[0,.68\
333,0,0,.83334],\
920:[0,.68333,0,\
0,.77778],923:[0\
,.68333,0,0,.694\
45],926:[0,.6833\
3,0,0,.66667],92\
8:[0,.68333,0,0,\
.75],931:[0,.683\
33,0,0,.72222],9\
33:[0,.68333,0,0\
,.77778],934:[0,\
.68333,0,0,.7222\
2],936:[0,.68333\
,0,0,.77778],937\
:[0,.68333,0,0,.\
72222],8211:[0,.\
43056,.02778,0,.\
5],8212:[0,.4305\
6,.02778,0,1],82\
16:[0,.69444,0,0\
,.27778],8217:[0\
,.69444,0,0,.277\
78],8220:[0,.694\
44,0,0,.5],8221:\
[0,.69444,0,0,.5\
],8224:[.19444,.\
69444,0,0,.44445\
],8225:[.19444,.\
69444,0,0,.44445\
],8230:[0,.123,0\
,0,1.172],8242:[\
0,.55556,0,0,.27\
5],8407:[0,.7144\
4,.15382,0,.5],8\
463:[0,.68889,0,\
0,.54028],8465:[\
0,.69444,0,0,.72\
222],8467:[0,.69\
444,0,.11111,.41\
667],8472:[.1944\
4,.43056,0,.1111\
1,.63646],8476:[\
0,.69444,0,0,.72\
222],8501:[0,.69\
444,0,0,.61111],\
8592:[-.13313,.3\
6687,0,0,1],8593\
:[.19444,.69444,\
0,0,.5],8594:[-.\
13313,.36687,0,0\
,1],8595:[.19444\
,.69444,0,0,.5],\
8596:[-.13313,.3\
6687,0,0,1],8597\
:[.25,.75,0,0,.5\
],8598:[.19444,.\
69444,0,0,1],859\
9:[.19444,.69444\
,0,0,1],8600:[.1\
9444,.69444,0,0,\
1],8601:[.19444,\
.69444,0,0,1],86\
14:[.011,.511,0,\
0,1],8617:[.011,\
.511,0,0,1.126],\
8618:[.011,.511,\
0,0,1.126],8636:\
[-.13313,.36687,\
0,0,1],8637:[-.1\
3313,.36687,0,0,\
1],8640:[-.13313\
,.36687,0,0,1],8\
641:[-.13313,.36\
687,0,0,1],8652:\
[.011,.671,0,0,1\
],8656:[-.13313,\
.36687,0,0,1],86\
57:[.19444,.6944\
4,0,0,.61111],86\
58:[-.13313,.366\
87,0,0,1],8659:[\
.19444,.69444,0,\
0,.61111],8660:[\
-.13313,.36687,0\
,0,1],8661:[.25,\
.75,0,0,.61111],\
8704:[0,.69444,0\
,0,.55556],8706:\
[0,.69444,.05556\
,.08334,.5309],8\
707:[0,.69444,0,\
0,.55556],8709:[\
.05556,.75,0,0,.\
5],8711:[0,.6833\
3,0,0,.83334],87\
12:[.0391,.5391,\
0,0,.66667],8715\
:[.0391,.5391,0,\
0,.66667],8722:[\
.08333,.58333,0,\
0,.77778],8723:[\
.08333,.58333,0,\
0,.77778],8725:[\
.25,.75,0,0,.5],\
8726:[.25,.75,0,\
0,.5],8727:[-.03\
472,.46528,0,0,.\
5],8728:[-.05555\
,.44445,0,0,.5],\
8729:[-.05555,.4\
4445,0,0,.5],873\
0:[.2,.8,0,0,.83\
334],8733:[0,.43\
056,0,0,.77778],\
8734:[0,.43056,0\
,0,1],8736:[0,.6\
9224,0,0,.72222]\
,8739:[.25,.75,0\
,0,.27778],8741:\
[.25,.75,0,0,.5]\
,8743:[0,.55556,\
0,0,.66667],8744\
:[0,.55556,0,0,.\
66667],8745:[0,.\
55556,0,0,.66667\
],8746:[0,.55556\
,0,0,.66667],874\
7:[.19444,.69444\
,.11111,0,.41667\
],8764:[-.13313,\
.36687,0,0,.7777\
8],8768:[.19444,\
.69444,0,0,.2777\
8],8771:[-.03625\
,.46375,0,0,.777\
78],8773:[-.022,\
.589,0,0,.778],8\
776:[-.01688,.48\
312,0,0,.77778],\
8781:[-.03625,.4\
6375,0,0,.77778]\
,8784:[-.133,.67\
3,0,0,.778],8801\
:[-.03625,.46375\
,0,0,.77778],880\
4:[.13597,.63597\
,0,0,.77778],880\
5:[.13597,.63597\
,0,0,.77778],881\
0:[.0391,.5391,0\
,0,1],8811:[.039\
1,.5391,0,0,1],8\
826:[.0391,.5391\
,0,0,.77778],882\
7:[.0391,.5391,0\
,0,.77778],8834:\
[.0391,.5391,0,0\
,.77778],8835:[.\
0391,.5391,0,0,.\
77778],8838:[.13\
597,.63597,0,0,.\
77778],8839:[.13\
597,.63597,0,0,.\
77778],8846:[0,.\
55556,0,0,.66667\
],8849:[.13597,.\
63597,0,0,.77778\
],8850:[.13597,.\
63597,0,0,.77778\
],8851:[0,.55556\
,0,0,.66667],885\
2:[0,.55556,0,0,\
.66667],8853:[.0\
8333,.58333,0,0,\
.77778],8854:[.0\
8333,.58333,0,0,\
.77778],8855:[.0\
8333,.58333,0,0,\
.77778],8856:[.0\
8333,.58333,0,0,\
.77778],8857:[.0\
8333,.58333,0,0,\
.77778],8866:[0,\
.69444,0,0,.6111\
1],8867:[0,.6944\
4,0,0,.61111],88\
68:[0,.69444,0,0\
,.77778],8869:[0\
,.69444,0,0,.777\
78],8872:[.249,.\
75,0,0,.867],890\
0:[-.05555,.4444\
5,0,0,.5],8901:[\
-.05555,.44445,0\
,0,.27778],8902:\
[-.03472,.46528,\
0,0,.5],8904:[.0\
05,.505,0,0,.9],\
8942:[.03,.903,0\
,0,.278],8943:[-\
.19,.313,0,0,1.1\
72],8945:[-.1,.8\
23,0,0,1.282],89\
68:[.25,.75,0,0,\
.44445],8969:[.2\
5,.75,0,0,.44445\
],8970:[.25,.75,\
0,0,.44445],8971\
:[.25,.75,0,0,.4\
4445],8994:[-.14\
236,.35764,0,0,1\
],8995:[-.14236,\
.35764,0,0,1],91\
36:[.244,.744,0,\
0,.412],9137:[.2\
44,.745,0,0,.412\
],9651:[.19444,.\
69444,0,0,.88889\
],9657:[-.03472,\
.46528,0,0,.5],9\
661:[.19444,.694\
44,0,0,.88889],9\
667:[-.03472,.46\
528,0,0,.5],9711\
:[.19444,.69444,\
0,0,1],9824:[.12\
963,.69444,0,0,.\
77778],9825:[.12\
963,.69444,0,0,.\
77778],9826:[.12\
963,.69444,0,0,.\
77778],9827:[.12\
963,.69444,0,0,.\
77778],9837:[0,.\
75,0,0,.38889],9\
838:[.19444,.694\
44,0,0,.38889],9\
839:[.19444,.694\
44,0,0,.38889],1\
0216:[.25,.75,0,\
0,.38889],10217:\
[.25,.75,0,0,.38\
889],10222:[.244\
,.744,0,0,.412],\
10223:[.244,.745\
,0,0,.412],10229\
:[.011,.511,0,0,\
1.609],10230:[.0\
11,.511,0,0,1.63\
8],10231:[.011,.\
511,0,0,1.859],1\
0232:[.024,.525,\
0,0,1.609],10233\
:[.024,.525,0,0,\
1.638],10234:[.0\
24,.525,0,0,1.85\
8],10236:[.011,.\
511,0,0,1.638],1\
0815:[0,.68333,0\
,0,.75],10927:[.\
13597,.63597,0,0\
,.77778],10928:[\
.13597,.63597,0,\
0,.77778],57376:\
[.19444,.69444,0\
,0,0]},\x22Math-Bol\
dItalic\x22:{32:[0,\
0,0,0,.25],48:[0\
,.44444,0,0,.575\
],49:[0,.44444,0\
,0,.575],50:[0,.\
44444,0,0,.575],\
51:[.19444,.4444\
4,0,0,.575],52:[\
.19444,.44444,0,\
0,.575],53:[.194\
44,.44444,0,0,.5\
75],54:[0,.64444\
,0,0,.575],55:[.\
19444,.44444,0,0\
,.575],56:[0,.64\
444,0,0,.575],57\
:[.19444,.44444,\
0,0,.575],65:[0,\
.68611,0,0,.8694\
4],66:[0,.68611,\
.04835,0,.8664],\
67:[0,.68611,.06\
979,0,.81694],68\
:[0,.68611,.0319\
4,0,.93812],69:[\
0,.68611,.05451,\
0,.81007],70:[0,\
.68611,.15972,0,\
.68889],71:[0,.6\
8611,0,0,.88673]\
,72:[0,.68611,.0\
8229,0,.98229],7\
3:[0,.68611,.077\
78,0,.51111],74:\
[0,.68611,.10069\
,0,.63125],75:[0\
,.68611,.06979,0\
,.97118],76:[0,.\
68611,0,0,.75555\
],77:[0,.68611,.\
11424,0,1.14201]\
,78:[0,.68611,.1\
1424,0,.95034],7\
9:[0,.68611,.031\
94,0,.83666],80:\
[0,.68611,.15972\
,0,.72309],81:[.\
19444,.68611,0,0\
,.86861],82:[0,.\
68611,.00421,0,.\
87235],83:[0,.68\
611,.05382,0,.69\
271],84:[0,.6861\
1,.15972,0,.6366\
3],85:[0,.68611,\
.11424,0,.80027]\
,86:[0,.68611,.2\
5555,0,.67778],8\
7:[0,.68611,.159\
72,0,1.09305],88\
:[0,.68611,.0777\
8,0,.94722],89:[\
0,.68611,.25555,\
0,.67458],90:[0,\
.68611,.06979,0,\
.77257],97:[0,.4\
4444,0,0,.63287]\
,98:[0,.69444,0,\
0,.52083],99:[0,\
.44444,0,0,.5134\
2],100:[0,.69444\
,0,0,.60972],101\
:[0,.44444,0,0,.\
55361],102:[.194\
44,.69444,.11042\
,0,.56806],103:[\
.19444,.44444,.0\
3704,0,.5449],10\
4:[0,.69444,0,0,\
.66759],105:[0,.\
69326,0,0,.4048]\
,106:[.19444,.69\
326,.0622,0,.470\
83],107:[0,.6944\
4,.01852,0,.6037\
],108:[0,.69444,\
.0088,0,.34815],\
109:[0,.44444,0,\
0,1.0324],110:[0\
,.44444,0,0,.712\
96],111:[0,.4444\
4,0,0,.58472],11\
2:[.19444,.44444\
,0,0,.60092],113\
:[.19444,.44444,\
.03704,0,.54213]\
,114:[0,.44444,.\
03194,0,.5287],1\
15:[0,.44444,0,0\
,.53125],116:[0,\
.63492,0,0,.4152\
8],117:[0,.44444\
,0,0,.68102],118\
:[0,.44444,.0370\
4,0,.56666],119:\
[0,.44444,.02778\
,0,.83148],120:[\
0,.44444,0,0,.65\
903],121:[.19444\
,.44444,.03704,0\
,.59028],122:[0,\
.44444,.04213,0,\
.55509],160:[0,0\
,0,0,.25],915:[0\
,.68611,.15972,0\
,.65694],916:[0,\
.68611,0,0,.9583\
3],920:[0,.68611\
,.03194,0,.86722\
],923:[0,.68611,\
0,0,.80555],926:\
[0,.68611,.07458\
,0,.84125],928:[\
0,.68611,.08229,\
0,.98229],931:[0\
,.68611,.05451,0\
,.88507],933:[0,\
.68611,.15972,0,\
.67083],934:[0,.\
68611,0,0,.76666\
],936:[0,.68611,\
.11653,0,.71402]\
,937:[0,.68611,.\
04835,0,.8789],9\
45:[0,.44444,0,0\
,.76064],946:[.1\
9444,.69444,.034\
03,0,.65972],947\
:[.19444,.44444,\
.06389,0,.59003]\
,948:[0,.69444,.\
03819,0,.52222],\
949:[0,.44444,0,\
0,.52882],950:[.\
19444,.69444,.06\
215,0,.50833],95\
1:[.19444,.44444\
,.03704,0,.6],95\
2:[0,.69444,.031\
94,0,.5618],953:\
[0,.44444,0,0,.4\
1204],954:[0,.44\
444,0,0,.66759],\
955:[0,.69444,0,\
0,.67083],956:[.\
19444,.44444,0,0\
,.70787],957:[0,\
.44444,.06898,0,\
.57685],958:[.19\
444,.69444,.0302\
1,0,.50833],959:\
[0,.44444,0,0,.5\
8472],960:[0,.44\
444,.03704,0,.68\
241],961:[.19444\
,.44444,0,0,.611\
8],962:[.09722,.\
44444,.07917,0,.\
42361],963:[0,.4\
4444,.03704,0,.6\
8588],964:[0,.44\
444,.13472,0,.52\
083],965:[0,.444\
44,.03704,0,.630\
55],966:[.19444,\
.44444,0,0,.7472\
2],967:[.19444,.\
44444,0,0,.71805\
],968:[.19444,.6\
9444,.03704,0,.7\
5833],969:[0,.44\
444,.03704,0,.71\
782],977:[0,.694\
44,0,0,.69155],9\
81:[.19444,.6944\
4,0,0,.7125],982\
:[0,.44444,.0319\
4,0,.975],1009:[\
.19444,.44444,0,\
0,.6118],1013:[0\
,.44444,0,0,.483\
33],57649:[0,.44\
444,0,0,.39352],\
57911:[.19444,.4\
4444,0,0,.43889]\
},\x22Math-Italic\x22:\
{32:[0,0,0,0,.25\
],48:[0,.43056,0\
,0,.5],49:[0,.43\
056,0,0,.5],50:[\
0,.43056,0,0,.5]\
,51:[.19444,.430\
56,0,0,.5],52:[.\
19444,.43056,0,0\
,.5],53:[.19444,\
.43056,0,0,.5],5\
4:[0,.64444,0,0,\
.5],55:[.19444,.\
43056,0,0,.5],56\
:[0,.64444,0,0,.\
5],57:[.19444,.4\
3056,0,0,.5],65:\
[0,.68333,0,.138\
89,.75],66:[0,.6\
8333,.05017,.083\
34,.75851],67:[0\
,.68333,.07153,.\
08334,.71472],68\
:[0,.68333,.0277\
8,.05556,.82792]\
,69:[0,.68333,.0\
5764,.08334,.738\
2],70:[0,.68333,\
.13889,.08334,.6\
4306],71:[0,.683\
33,0,.08334,.786\
25],72:[0,.68333\
,.08125,.05556,.\
83125],73:[0,.68\
333,.07847,.1111\
1,.43958],74:[0,\
.68333,.09618,.1\
6667,.55451],75:\
[0,.68333,.07153\
,.05556,.84931],\
76:[0,.68333,0,.\
02778,.68056],77\
:[0,.68333,.1090\
3,.08334,.97014]\
,78:[0,.68333,.1\
0903,.08334,.803\
47],79:[0,.68333\
,.02778,.08334,.\
76278],80:[0,.68\
333,.13889,.0833\
4,.64201],81:[.1\
9444,.68333,0,.0\
8334,.79056],82:\
[0,.68333,.00773\
,.08334,.75929],\
83:[0,.68333,.05\
764,.08334,.6132\
],84:[0,.68333,.\
13889,.08334,.58\
438],85:[0,.6833\
3,.10903,.02778,\
.68278],86:[0,.6\
8333,.22222,0,.5\
8333],87:[0,.683\
33,.13889,0,.944\
45],88:[0,.68333\
,.07847,.08334,.\
82847],89:[0,.68\
333,.22222,0,.58\
056],90:[0,.6833\
3,.07153,.08334,\
.68264],97:[0,.4\
3056,0,0,.52859]\
,98:[0,.69444,0,\
0,.42917],99:[0,\
.43056,0,.05556,\
.43276],100:[0,.\
69444,0,.16667,.\
52049],101:[0,.4\
3056,0,.05556,.4\
6563],102:[.1944\
4,.69444,.10764,\
.16667,.48959],1\
03:[.19444,.4305\
6,.03588,.02778,\
.47697],104:[0,.\
69444,0,0,.57616\
],105:[0,.65952,\
0,0,.34451],106:\
[.19444,.65952,.\
05724,0,.41181],\
107:[0,.69444,.0\
3148,0,.5206],10\
8:[0,.69444,.019\
68,.08334,.29838\
],109:[0,.43056,\
0,0,.87801],110:\
[0,.43056,0,0,.6\
0023],111:[0,.43\
056,0,.05556,.48\
472],112:[.19444\
,.43056,0,.08334\
,.50313],113:[.1\
9444,.43056,.035\
88,.08334,.44641\
],114:[0,.43056,\
.02778,.05556,.4\
5116],115:[0,.43\
056,0,.05556,.46\
875],116:[0,.615\
08,0,.08334,.361\
11],117:[0,.4305\
6,0,.02778,.5724\
6],118:[0,.43056\
,.03588,.02778,.\
48472],119:[0,.4\
3056,.02691,.083\
34,.71592],120:[\
0,.43056,0,.0277\
8,.57153],121:[.\
19444,.43056,.03\
588,.05556,.4902\
8],122:[0,.43056\
,.04398,.05556,.\
46505],160:[0,0,\
0,0,.25],915:[0,\
.68333,.13889,.0\
8334,.61528],916\
:[0,.68333,0,.16\
667,.83334],920:\
[0,.68333,.02778\
,.08334,.76278],\
923:[0,.68333,0,\
.16667,.69445],9\
26:[0,.68333,.07\
569,.08334,.7423\
6],928:[0,.68333\
,.08125,.05556,.\
83125],931:[0,.6\
8333,.05764,.083\
34,.77986],933:[\
0,.68333,.13889,\
.05556,.58333],9\
34:[0,.68333,0,.\
08334,.66667],93\
6:[0,.68333,.11,\
.05556,.61222],9\
37:[0,.68333,.05\
017,.08334,.7724\
],945:[0,.43056,\
.0037,.02778,.63\
97],946:[.19444,\
.69444,.05278,.0\
8334,.56563],947\
:[.19444,.43056,\
.05556,0,.51773]\
,948:[0,.69444,.\
03785,.05556,.44\
444],949:[0,.430\
56,0,.08334,.466\
32],950:[.19444,\
.69444,.07378,.0\
8334,.4375],951:\
[.19444,.43056,.\
03588,.05556,.49\
653],952:[0,.694\
44,.02778,.08334\
,.46944],953:[0,\
.43056,0,.05556,\
.35394],954:[0,.\
43056,0,0,.57616\
],955:[0,.69444,\
0,0,.58334],956:\
[.19444,.43056,0\
,.02778,.60255],\
957:[0,.43056,.0\
6366,.02778,.493\
98],958:[.19444,\
.69444,.04601,.1\
1111,.4375],959:\
[0,.43056,0,.055\
56,.48472],960:[\
0,.43056,.03588,\
0,.57003],961:[.\
19444,.43056,0,.\
08334,.51702],96\
2:[.09722,.43056\
,.07986,.08334,.\
36285],963:[0,.4\
3056,.03588,0,.5\
7141],964:[0,.43\
056,.1132,.02778\
,.43715],965:[0,\
.43056,.03588,.0\
2778,.54028],966\
:[.19444,.43056,\
0,.08334,.65417]\
,967:[.19444,.43\
056,0,.05556,.62\
569],968:[.19444\
,.69444,.03588,.\
11111,.65139],96\
9:[0,.43056,.035\
88,0,.62245],977\
:[0,.69444,0,.08\
334,.59144],981:\
[.19444,.69444,0\
,.08334,.59583],\
982:[0,.43056,.0\
2778,0,.82813],1\
009:[.19444,.430\
56,0,.08334,.517\
02],1013:[0,.430\
56,0,.05556,.405\
9],57649:[0,.430\
56,0,.02778,.322\
46],57911:[.1944\
4,.43056,0,.0833\
4,.38403]},\x22Sans\
Serif-Bold\x22:{32:\
[0,0,0,0,.25],33\
:[0,.69444,0,0,.\
36667],34:[0,.69\
444,0,0,.55834],\
35:[.19444,.6944\
4,0,0,.91667],36\
:[.05556,.75,0,0\
,.55],37:[.05556\
,.75,0,0,1.02912\
],38:[0,.69444,0\
,0,.83056],39:[0\
,.69444,0,0,.305\
56],40:[.25,.75,\
0,0,.42778],41:[\
.25,.75,0,0,.427\
78],42:[0,.75,0,\
0,.55],43:[.1166\
7,.61667,0,0,.85\
556],44:[.10556,\
.13056,0,0,.3055\
6],45:[0,.45833,\
0,0,.36667],46:[\
0,.13056,0,0,.30\
556],47:[.25,.75\
,0,0,.55],48:[0,\
.69444,0,0,.55],\
49:[0,.69444,0,0\
,.55],50:[0,.694\
44,0,0,.55],51:[\
0,.69444,0,0,.55\
],52:[0,.69444,0\
,0,.55],53:[0,.6\
9444,0,0,.55],54\
:[0,.69444,0,0,.\
55],55:[0,.69444\
,0,0,.55],56:[0,\
.69444,0,0,.55],\
57:[0,.69444,0,0\
,.55],58:[0,.458\
33,0,0,.30556],5\
9:[.10556,.45833\
,0,0,.30556],61:\
[-.09375,.40625,\
0,0,.85556],63:[\
0,.69444,0,0,.51\
945],64:[0,.6944\
4,0,0,.73334],65\
:[0,.69444,0,0,.\
73334],66:[0,.69\
444,0,0,.73334],\
67:[0,.69444,0,0\
,.70278],68:[0,.\
69444,0,0,.79445\
],69:[0,.69444,0\
,0,.64167],70:[0\
,.69444,0,0,.611\
11],71:[0,.69444\
,0,0,.73334],72:\
[0,.69444,0,0,.7\
9445],73:[0,.694\
44,0,0,.33056],7\
4:[0,.69444,0,0,\
.51945],75:[0,.6\
9444,0,0,.76389]\
,76:[0,.69444,0,\
0,.58056],77:[0,\
.69444,0,0,.9777\
8],78:[0,.69444,\
0,0,.79445],79:[\
0,.69444,0,0,.79\
445],80:[0,.6944\
4,0,0,.70278],81\
:[.10556,.69444,\
0,0,.79445],82:[\
0,.69444,0,0,.70\
278],83:[0,.6944\
4,0,0,.61111],84\
:[0,.69444,0,0,.\
73334],85:[0,.69\
444,0,0,.76389],\
86:[0,.69444,.01\
528,0,.73334],87\
:[0,.69444,.0152\
8,0,1.03889],88:\
[0,.69444,0,0,.7\
3334],89:[0,.694\
44,.0275,0,.7333\
4],90:[0,.69444,\
0,0,.67223],91:[\
.25,.75,0,0,.343\
06],93:[.25,.75,\
0,0,.34306],94:[\
0,.69444,0,0,.55\
],95:[.35,.10833\
,.03056,0,.55],9\
7:[0,.45833,0,0,\
.525],98:[0,.694\
44,0,0,.56111],9\
9:[0,.45833,0,0,\
.48889],100:[0,.\
69444,0,0,.56111\
],101:[0,.45833,\
0,0,.51111],102:\
[0,.69444,.07639\
,0,.33611],103:[\
.19444,.45833,.0\
1528,0,.55],104:\
[0,.69444,0,0,.5\
6111],105:[0,.69\
444,0,0,.25556],\
106:[.19444,.694\
44,0,0,.28611],1\
07:[0,.69444,0,0\
,.53056],108:[0,\
.69444,0,0,.2555\
6],109:[0,.45833\
,0,0,.86667],110\
:[0,.45833,0,0,.\
56111],111:[0,.4\
5833,0,0,.55],11\
2:[.19444,.45833\
,0,0,.56111],113\
:[.19444,.45833,\
0,0,.56111],114:\
[0,.45833,.01528\
,0,.37222],115:[\
0,.45833,0,0,.42\
167],116:[0,.589\
29,0,0,.40417],1\
17:[0,.45833,0,0\
,.56111],118:[0,\
.45833,.01528,0,\
.5],119:[0,.4583\
3,.01528,0,.7444\
5],120:[0,.45833\
,0,0,.5],121:[.1\
9444,.45833,.015\
28,0,.5],122:[0,\
.45833,0,0,.4763\
9],126:[.35,.344\
44,0,0,.55],160:\
[0,0,0,0,.25],16\
8:[0,.69444,0,0,\
.55],176:[0,.694\
44,0,0,.73334],1\
80:[0,.69444,0,0\
,.55],184:[.1701\
4,0,0,0,.48889],\
305:[0,.45833,0,\
0,.25556],567:[.\
19444,.45833,0,0\
,.28611],710:[0,\
.69444,0,0,.55],\
711:[0,.63542,0,\
0,.55],713:[0,.6\
3778,0,0,.55],72\
8:[0,.69444,0,0,\
.55],729:[0,.694\
44,0,0,.30556],7\
30:[0,.69444,0,0\
,.73334],732:[0,\
.69444,0,0,.55],\
733:[0,.69444,0,\
0,.55],915:[0,.6\
9444,0,0,.58056]\
,916:[0,.69444,0\
,0,.91667],920:[\
0,.69444,0,0,.85\
556],923:[0,.694\
44,0,0,.67223],9\
26:[0,.69444,0,0\
,.73334],928:[0,\
.69444,0,0,.7944\
5],931:[0,.69444\
,0,0,.79445],933\
:[0,.69444,0,0,.\
85556],934:[0,.6\
9444,0,0,.79445]\
,936:[0,.69444,0\
,0,.85556],937:[\
0,.69444,0,0,.79\
445],8211:[0,.45\
833,.03056,0,.55\
],8212:[0,.45833\
,.03056,0,1.1000\
1],8216:[0,.6944\
4,0,0,.30556],82\
17:[0,.69444,0,0\
,.30556],8220:[0\
,.69444,0,0,.558\
34],8221:[0,.694\
44,0,0,.55834]},\
\x22SansSerif-Itali\
c\x22:{32:[0,0,0,0,\
.25],33:[0,.6944\
4,.05733,0,.3194\
5],34:[0,.69444,\
.00316,0,.5],35:\
[.19444,.69444,.\
05087,0,.83334],\
36:[.05556,.75,.\
11156,0,.5],37:[\
.05556,.75,.0312\
6,0,.83334],38:[\
0,.69444,.03058,\
0,.75834],39:[0,\
.69444,.07816,0,\
.27778],40:[.25,\
.75,.13164,0,.38\
889],41:[.25,.75\
,.02536,0,.38889\
],42:[0,.75,.117\
75,0,.5],43:[.08\
333,.58333,.0253\
6,0,.77778],44:[\
.125,.08333,0,0,\
.27778],45:[0,.4\
4444,.01946,0,.3\
3333],46:[0,.083\
33,0,0,.27778],4\
7:[.25,.75,.1316\
4,0,.5],48:[0,.6\
5556,.11156,0,.5\
],49:[0,.65556,.\
11156,0,.5],50:[\
0,.65556,.11156,\
0,.5],51:[0,.655\
56,.11156,0,.5],\
52:[0,.65556,.11\
156,0,.5],53:[0,\
.65556,.11156,0,\
.5],54:[0,.65556\
,.11156,0,.5],55\
:[0,.65556,.1115\
6,0,.5],56:[0,.6\
5556,.11156,0,.5\
],57:[0,.65556,.\
11156,0,.5],58:[\
0,.44444,.02502,\
0,.27778],59:[.1\
25,.44444,.02502\
,0,.27778],61:[-\
.13,.37,.05087,0\
,.77778],63:[0,.\
69444,.11809,0,.\
47222],64:[0,.69\
444,.07555,0,.66\
667],65:[0,.6944\
4,0,0,.66667],66\
:[0,.69444,.0829\
3,0,.66667],67:[\
0,.69444,.11983,\
0,.63889],68:[0,\
.69444,.07555,0,\
.72223],69:[0,.6\
9444,.11983,0,.5\
9722],70:[0,.694\
44,.13372,0,.569\
45],71:[0,.69444\
,.11983,0,.66667\
],72:[0,.69444,.\
08094,0,.70834],\
73:[0,.69444,.13\
372,0,.27778],74\
:[0,.69444,.0809\
4,0,.47222],75:[\
0,.69444,.11983,\
0,.69445],76:[0,\
.69444,0,0,.5416\
7],77:[0,.69444,\
.08094,0,.875],7\
8:[0,.69444,.080\
94,0,.70834],79:\
[0,.69444,.07555\
,0,.73611],80:[0\
,.69444,.08293,0\
,.63889],81:[.12\
5,.69444,.07555,\
0,.73611],82:[0,\
.69444,.08293,0,\
.64584],83:[0,.6\
9444,.09205,0,.5\
5556],84:[0,.694\
44,.13372,0,.680\
56],85:[0,.69444\
,.08094,0,.6875]\
,86:[0,.69444,.1\
615,0,.66667],87\
:[0,.69444,.1615\
,0,.94445],88:[0\
,.69444,.13372,0\
,.66667],89:[0,.\
69444,.17261,0,.\
66667],90:[0,.69\
444,.11983,0,.61\
111],91:[.25,.75\
,.15942,0,.28889\
],93:[.25,.75,.0\
8719,0,.28889],9\
4:[0,.69444,.079\
9,0,.5],95:[.35,\
.09444,.08616,0,\
.5],97:[0,.44444\
,.00981,0,.48056\
],98:[0,.69444,.\
03057,0,.51667],\
99:[0,.44444,.08\
336,0,.44445],10\
0:[0,.69444,.094\
83,0,.51667],101\
:[0,.44444,.0677\
8,0,.44445],102:\
[0,.69444,.21705\
,0,.30556],103:[\
.19444,.44444,.1\
0836,0,.5],104:[\
0,.69444,.01778,\
0,.51667],105:[0\
,.67937,.09718,0\
,.23889],106:[.1\
9444,.67937,.091\
62,0,.26667],107\
:[0,.69444,.0833\
6,0,.48889],108:\
[0,.69444,.09483\
,0,.23889],109:[\
0,.44444,.01778,\
0,.79445],110:[0\
,.44444,.01778,0\
,.51667],111:[0,\
.44444,.06613,0,\
.5],112:[.19444,\
.44444,.0389,0,.\
51667],113:[.194\
44,.44444,.04169\
,0,.51667],114:[\
0,.44444,.10836,\
0,.34167],115:[0\
,.44444,.0778,0,\
.38333],116:[0,.\
57143,.07225,0,.\
36111],117:[0,.4\
4444,.04169,0,.5\
1667],118:[0,.44\
444,.10836,0,.46\
111],119:[0,.444\
44,.10836,0,.683\
34],120:[0,.4444\
4,.09169,0,.4611\
1],121:[.19444,.\
44444,.10836,0,.\
46111],122:[0,.4\
4444,.08752,0,.4\
3472],126:[.35,.\
32659,.08826,0,.\
5],160:[0,0,0,0,\
.25],168:[0,.679\
37,.06385,0,.5],\
176:[0,.69444,0,\
0,.73752],184:[.\
17014,0,0,0,.444\
45],305:[0,.4444\
4,.04169,0,.2388\
9],567:[.19444,.\
44444,.04169,0,.\
26667],710:[0,.6\
9444,.0799,0,.5]\
,711:[0,.63194,.\
08432,0,.5],713:\
[0,.60889,.08776\
,0,.5],714:[0,.6\
9444,.09205,0,.5\
],715:[0,.69444,\
0,0,.5],728:[0,.\
69444,.09483,0,.\
5],729:[0,.67937\
,.07774,0,.27778\
],730:[0,.69444,\
0,0,.73752],732:\
[0,.67659,.08826\
,0,.5],733:[0,.6\
9444,.09205,0,.5\
],915:[0,.69444,\
.13372,0,.54167]\
,916:[0,.69444,0\
,0,.83334],920:[\
0,.69444,.07555,\
0,.77778],923:[0\
,.69444,0,0,.611\
11],926:[0,.6944\
4,.12816,0,.6666\
7],928:[0,.69444\
,.08094,0,.70834\
],931:[0,.69444,\
.11983,0,.72222]\
,933:[0,.69444,.\
09031,0,.77778],\
934:[0,.69444,.0\
4603,0,.72222],9\
36:[0,.69444,.09\
031,0,.77778],93\
7:[0,.69444,.082\
93,0,.72222],821\
1:[0,.44444,.086\
16,0,.5],8212:[0\
,.44444,.08616,0\
,1],8216:[0,.694\
44,.07816,0,.277\
78],8217:[0,.694\
44,.07816,0,.277\
78],8220:[0,.694\
44,.14205,0,.5],\
8221:[0,.69444,.\
00316,0,.5]},\x22Sa\
nsSerif-Regular\x22\
:{32:[0,0,0,0,.2\
5],33:[0,.69444,\
0,0,.31945],34:[\
0,.69444,0,0,.5]\
,35:[.19444,.694\
44,0,0,.83334],3\
6:[.05556,.75,0,\
0,.5],37:[.05556\
,.75,0,0,.83334]\
,38:[0,.69444,0,\
0,.75834],39:[0,\
.69444,0,0,.2777\
8],40:[.25,.75,0\
,0,.38889],41:[.\
25,.75,0,0,.3888\
9],42:[0,.75,0,0\
,.5],43:[.08333,\
.58333,0,0,.7777\
8],44:[.125,.083\
33,0,0,.27778],4\
5:[0,.44444,0,0,\
.33333],46:[0,.0\
8333,0,0,.27778]\
,47:[.25,.75,0,0\
,.5],48:[0,.6555\
6,0,0,.5],49:[0,\
.65556,0,0,.5],5\
0:[0,.65556,0,0,\
.5],51:[0,.65556\
,0,0,.5],52:[0,.\
65556,0,0,.5],53\
:[0,.65556,0,0,.\
5],54:[0,.65556,\
0,0,.5],55:[0,.6\
5556,0,0,.5],56:\
[0,.65556,0,0,.5\
],57:[0,.65556,0\
,0,.5],58:[0,.44\
444,0,0,.27778],\
59:[.125,.44444,\
0,0,.27778],61:[\
-.13,.37,0,0,.77\
778],63:[0,.6944\
4,0,0,.47222],64\
:[0,.69444,0,0,.\
66667],65:[0,.69\
444,0,0,.66667],\
66:[0,.69444,0,0\
,.66667],67:[0,.\
69444,0,0,.63889\
],68:[0,.69444,0\
,0,.72223],69:[0\
,.69444,0,0,.597\
22],70:[0,.69444\
,0,0,.56945],71:\
[0,.69444,0,0,.6\
6667],72:[0,.694\
44,0,0,.70834],7\
3:[0,.69444,0,0,\
.27778],74:[0,.6\
9444,0,0,.47222]\
,75:[0,.69444,0,\
0,.69445],76:[0,\
.69444,0,0,.5416\
7],77:[0,.69444,\
0,0,.875],78:[0,\
.69444,0,0,.7083\
4],79:[0,.69444,\
0,0,.73611],80:[\
0,.69444,0,0,.63\
889],81:[.125,.6\
9444,0,0,.73611]\
,82:[0,.69444,0,\
0,.64584],83:[0,\
.69444,0,0,.5555\
6],84:[0,.69444,\
0,0,.68056],85:[\
0,.69444,0,0,.68\
75],86:[0,.69444\
,.01389,0,.66667\
],87:[0,.69444,.\
01389,0,.94445],\
88:[0,.69444,0,0\
,.66667],89:[0,.\
69444,.025,0,.66\
667],90:[0,.6944\
4,0,0,.61111],91\
:[.25,.75,0,0,.2\
8889],93:[.25,.7\
5,0,0,.28889],94\
:[0,.69444,0,0,.\
5],95:[.35,.0944\
4,.02778,0,.5],9\
7:[0,.44444,0,0,\
.48056],98:[0,.6\
9444,0,0,.51667]\
,99:[0,.44444,0,\
0,.44445],100:[0\
,.69444,0,0,.516\
67],101:[0,.4444\
4,0,0,.44445],10\
2:[0,.69444,.069\
44,0,.30556],103\
:[.19444,.44444,\
.01389,0,.5],104\
:[0,.69444,0,0,.\
51667],105:[0,.6\
7937,0,0,.23889]\
,106:[.19444,.67\
937,0,0,.26667],\
107:[0,.69444,0,\
0,.48889],108:[0\
,.69444,0,0,.238\
89],109:[0,.4444\
4,0,0,.79445],11\
0:[0,.44444,0,0,\
.51667],111:[0,.\
44444,0,0,.5],11\
2:[.19444,.44444\
,0,0,.51667],113\
:[.19444,.44444,\
0,0,.51667],114:\
[0,.44444,.01389\
,0,.34167],115:[\
0,.44444,0,0,.38\
333],116:[0,.571\
43,0,0,.36111],1\
17:[0,.44444,0,0\
,.51667],118:[0,\
.44444,.01389,0,\
.46111],119:[0,.\
44444,.01389,0,.\
68334],120:[0,.4\
4444,0,0,.46111]\
,121:[.19444,.44\
444,.01389,0,.46\
111],122:[0,.444\
44,0,0,.43472],1\
26:[.35,.32659,0\
,0,.5],160:[0,0,\
0,0,.25],168:[0,\
.67937,0,0,.5],1\
76:[0,.69444,0,0\
,.66667],184:[.1\
7014,0,0,0,.4444\
5],305:[0,.44444\
,0,0,.23889],567\
:[.19444,.44444,\
0,0,.26667],710:\
[0,.69444,0,0,.5\
],711:[0,.63194,\
0,0,.5],713:[0,.\
60889,0,0,.5],71\
4:[0,.69444,0,0,\
.5],715:[0,.6944\
4,0,0,.5],728:[0\
,.69444,0,0,.5],\
729:[0,.67937,0,\
0,.27778],730:[0\
,.69444,0,0,.666\
67],732:[0,.6765\
9,0,0,.5],733:[0\
,.69444,0,0,.5],\
915:[0,.69444,0,\
0,.54167],916:[0\
,.69444,0,0,.833\
34],920:[0,.6944\
4,0,0,.77778],92\
3:[0,.69444,0,0,\
.61111],926:[0,.\
69444,0,0,.66667\
],928:[0,.69444,\
0,0,.70834],931:\
[0,.69444,0,0,.7\
2222],933:[0,.69\
444,0,0,.77778],\
934:[0,.69444,0,\
0,.72222],936:[0\
,.69444,0,0,.777\
78],937:[0,.6944\
4,0,0,.72222],82\
11:[0,.44444,.02\
778,0,.5],8212:[\
0,.44444,.02778,\
0,1],8216:[0,.69\
444,0,0,.27778],\
8217:[0,.69444,0\
,0,.27778],8220:\
[0,.69444,0,0,.5\
],8221:[0,.69444\
,0,0,.5]},\x22Scrip\
t-Regular\x22:{32:[\
0,0,0,0,.25],65:\
[0,.7,.22925,0,.\
80253],66:[0,.7,\
.04087,0,.90757]\
,67:[0,.7,.1689,\
0,.66619],68:[0,\
.7,.09371,0,.774\
43],69:[0,.7,.18\
583,0,.56162],70\
:[0,.7,.13634,0,\
.89544],71:[0,.7\
,.17322,0,.60961\
],72:[0,.7,.2969\
4,0,.96919],73:[\
0,.7,.19189,0,.8\
0907],74:[.27778\
,.7,.19189,0,1.0\
5159],75:[0,.7,.\
31259,0,.91364],\
76:[0,.7,.19189,\
0,.87373],77:[0,\
.7,.15981,0,1.08\
031],78:[0,.7,.3\
525,0,.9015],79:\
[0,.7,.08078,0,.\
73787],80:[0,.7,\
.08078,0,1.01262\
],81:[0,.7,.0330\
5,0,.88282],82:[\
0,.7,.06259,0,.8\
5],83:[0,.7,.191\
89,0,.86767],84:\
[0,.7,.29087,0,.\
74697],85:[0,.7,\
.25815,0,.79996]\
,86:[0,.7,.27523\
,0,.62204],87:[0\
,.7,.27523,0,.80\
532],88:[0,.7,.2\
6006,0,.94445],8\
9:[0,.7,.2939,0,\
.70961],90:[0,.7\
,.24037,0,.8212]\
,160:[0,0,0,0,.2\
5]},\x22Size1-Regul\
ar\x22:{32:[0,0,0,0\
,.25],40:[.35001\
,.85,0,0,.45834]\
,41:[.35001,.85,\
0,0,.45834],47:[\
.35001,.85,0,0,.\
57778],91:[.3500\
1,.85,0,0,.41667\
],92:[.35001,.85\
,0,0,.57778],93:\
[.35001,.85,0,0,\
.41667],123:[.35\
001,.85,0,0,.583\
34],125:[.35001,\
.85,0,0,.58334],\
160:[0,0,0,0,.25\
],710:[0,.72222,\
0,0,.55556],732:\
[0,.72222,0,0,.5\
5556],770:[0,.72\
222,0,0,.55556],\
771:[0,.72222,0,\
0,.55556],8214:[\
-99e-5,.601,0,0,\
.77778],8593:[1e\
-5,.6,0,0,.66667\
],8595:[1e-5,.6,\
0,0,.66667],8657\
:[1e-5,.6,0,0,.7\
7778],8659:[1e-5\
,.6,0,0,.77778],\
8719:[.25001,.75\
,0,0,.94445],872\
0:[.25001,.75,0,\
0,.94445],8721:[\
.25001,.75,0,0,1\
.05556],8730:[.3\
5001,.85,0,0,1],\
8739:[-.00599,.6\
06,0,0,.33333],8\
741:[-.00599,.60\
6,0,0,.55556],87\
47:[.30612,.805,\
.19445,0,.47222]\
,8748:[.306,.805\
,.19445,0,.47222\
],8749:[.306,.80\
5,.19445,0,.4722\
2],8750:[.30612,\
.805,.19445,0,.4\
7222],8896:[.250\
01,.75,0,0,.8333\
4],8897:[.25001,\
.75,0,0,.83334],\
8898:[.25001,.75\
,0,0,.83334],889\
9:[.25001,.75,0,\
0,.83334],8968:[\
.35001,.85,0,0,.\
47222],8969:[.35\
001,.85,0,0,.472\
22],8970:[.35001\
,.85,0,0,.47222]\
,8971:[.35001,.8\
5,0,0,.47222],91\
68:[-99e-5,.601,\
0,0,.66667],1021\
6:[.35001,.85,0,\
0,.47222],10217:\
[.35001,.85,0,0,\
.47222],10752:[.\
25001,.75,0,0,1.\
11111],10753:[.2\
5001,.75,0,0,1.1\
1111],10754:[.25\
001,.75,0,0,1.11\
111],10756:[.250\
01,.75,0,0,.8333\
4],10758:[.25001\
,.75,0,0,.83334]\
},\x22Size2-Regular\
\x22:{32:[0,0,0,0,.\
25],40:[.65002,1\
.15,0,0,.59722],\
41:[.65002,1.15,\
0,0,.59722],47:[\
.65002,1.15,0,0,\
.81111],91:[.650\
02,1.15,0,0,.472\
22],92:[.65002,1\
.15,0,0,.81111],\
93:[.65002,1.15,\
0,0,.47222],123:\
[.65002,1.15,0,0\
,.66667],125:[.6\
5002,1.15,0,0,.6\
6667],160:[0,0,0\
,0,.25],710:[0,.\
75,0,0,1],732:[0\
,.75,0,0,1],770:\
[0,.75,0,0,1],77\
1:[0,.75,0,0,1],\
8719:[.55001,1.0\
5,0,0,1.27778],8\
720:[.55001,1.05\
,0,0,1.27778],87\
21:[.55001,1.05,\
0,0,1.44445],873\
0:[.65002,1.15,0\
,0,1],8747:[.862\
25,1.36,.44445,0\
,.55556],8748:[.\
862,1.36,.44445,\
0,.55556],8749:[\
.862,1.36,.44445\
,0,.55556],8750:\
[.86225,1.36,.44\
445,0,.55556],88\
96:[.55001,1.05,\
0,0,1.11111],889\
7:[.55001,1.05,0\
,0,1.11111],8898\
:[.55001,1.05,0,\
0,1.11111],8899:\
[.55001,1.05,0,0\
,1.11111],8968:[\
.65002,1.15,0,0,\
.52778],8969:[.6\
5002,1.15,0,0,.5\
2778],8970:[.650\
02,1.15,0,0,.527\
78],8971:[.65002\
,1.15,0,0,.52778\
],10216:[.65002,\
1.15,0,0,.61111]\
,10217:[.65002,1\
.15,0,0,.61111],\
10752:[.55001,1.\
05,0,0,1.51112],\
10753:[.55001,1.\
05,0,0,1.51112],\
10754:[.55001,1.\
05,0,0,1.51112],\
10756:[.55001,1.\
05,0,0,1.11111],\
10758:[.55001,1.\
05,0,0,1.11111]}\
,\x22Size3-Regular\x22\
:{32:[0,0,0,0,.2\
5],40:[.95003,1.\
45,0,0,.73611],4\
1:[.95003,1.45,0\
,0,.73611],47:[.\
95003,1.45,0,0,1\
.04445],91:[.950\
03,1.45,0,0,.527\
78],92:[.95003,1\
.45,0,0,1.04445]\
,93:[.95003,1.45\
,0,0,.52778],123\
:[.95003,1.45,0,\
0,.75],125:[.950\
03,1.45,0,0,.75]\
,160:[0,0,0,0,.2\
5],710:[0,.75,0,\
0,1.44445],732:[\
0,.75,0,0,1.4444\
5],770:[0,.75,0,\
0,1.44445],771:[\
0,.75,0,0,1.4444\
5],8730:[.95003,\
1.45,0,0,1],8968\
:[.95003,1.45,0,\
0,.58334],8969:[\
.95003,1.45,0,0,\
.58334],8970:[.9\
5003,1.45,0,0,.5\
8334],8971:[.950\
03,1.45,0,0,.583\
34],10216:[.9500\
3,1.45,0,0,.75],\
10217:[.95003,1.\
45,0,0,.75]},\x22Si\
ze4-Regular\x22:{32\
:[0,0,0,0,.25],4\
0:[1.25003,1.75,\
0,0,.79167],41:[\
1.25003,1.75,0,0\
,.79167],47:[1.2\
5003,1.75,0,0,1.\
27778],91:[1.250\
03,1.75,0,0,.583\
34],92:[1.25003,\
1.75,0,0,1.27778\
],93:[1.25003,1.\
75,0,0,.58334],1\
23:[1.25003,1.75\
,0,0,.80556],125\
:[1.25003,1.75,0\
,0,.80556],160:[\
0,0,0,0,.25],710\
:[0,.825,0,0,1.8\
889],732:[0,.825\
,0,0,1.8889],770\
:[0,.825,0,0,1.8\
889],771:[0,.825\
,0,0,1.8889],873\
0:[1.25003,1.75,\
0,0,1],8968:[1.2\
5003,1.75,0,0,.6\
3889],8969:[1.25\
003,1.75,0,0,.63\
889],8970:[1.250\
03,1.75,0,0,.638\
89],8971:[1.2500\
3,1.75,0,0,.6388\
9],9115:[.64502,\
1.155,0,0,.875],\
9116:[1e-5,.6,0,\
0,.875],9117:[.6\
4502,1.155,0,0,.\
875],9118:[.6450\
2,1.155,0,0,.875\
],9119:[1e-5,.6,\
0,0,.875],9120:[\
.64502,1.155,0,0\
,.875],9121:[.64\
502,1.155,0,0,.6\
6667],9122:[-99e\
-5,.601,0,0,.666\
67],9123:[.64502\
,1.155,0,0,.6666\
7],9124:[.64502,\
1.155,0,0,.66667\
],9125:[-99e-5,.\
601,0,0,.66667],\
9126:[.64502,1.1\
55,0,0,.66667],9\
127:[1e-5,.9,0,0\
,.88889],9128:[.\
65002,1.15,0,0,.\
88889],9129:[.90\
001,0,0,0,.88889\
],9130:[0,.3,0,0\
,.88889],9131:[1\
e-5,.9,0,0,.8888\
9],9132:[.65002,\
1.15,0,0,.88889]\
,9133:[.90001,0,\
0,0,.88889],9143\
:[.88502,.915,0,\
0,1.05556],10216\
:[1.25003,1.75,0\
,0,.80556],10217\
:[1.25003,1.75,0\
,0,.80556],57344\
:[-.00499,.605,0\
,0,1.05556],5734\
5:[-.00499,.605,\
0,0,1.05556],576\
80:[0,.12,0,0,.4\
5],57681:[0,.12,\
0,0,.45],57682:[\
0,.12,0,0,.45],5\
7683:[0,.12,0,0,\
.45]},\x22Typewrite\
r-Regular\x22:{32:[\
0,0,0,0,.525],33\
:[0,.61111,0,0,.\
525],34:[0,.6111\
1,0,0,.525],35:[\
0,.61111,0,0,.52\
5],36:[.08333,.6\
9444,0,0,.525],3\
7:[.08333,.69444\
,0,0,.525],38:[0\
,.61111,0,0,.525\
],39:[0,.61111,0\
,0,.525],40:[.08\
333,.69444,0,0,.\
525],41:[.08333,\
.69444,0,0,.525]\
,42:[0,.52083,0,\
0,.525],43:[-.08\
056,.53055,0,0,.\
525],44:[.13889,\
.125,0,0,.525],4\
5:[-.08056,.5305\
5,0,0,.525],46:[\
0,.125,0,0,.525]\
,47:[.08333,.694\
44,0,0,.525],48:\
[0,.61111,0,0,.5\
25],49:[0,.61111\
,0,0,.525],50:[0\
,.61111,0,0,.525\
],51:[0,.61111,0\
,0,.525],52:[0,.\
61111,0,0,.525],\
53:[0,.61111,0,0\
,.525],54:[0,.61\
111,0,0,.525],55\
:[0,.61111,0,0,.\
525],56:[0,.6111\
1,0,0,.525],57:[\
0,.61111,0,0,.52\
5],58:[0,.43056,\
0,0,.525],59:[.1\
3889,.43056,0,0,\
.525],60:[-.0555\
6,.55556,0,0,.52\
5],61:[-.19549,.\
41562,0,0,.525],\
62:[-.05556,.555\
56,0,0,.525],63:\
[0,.61111,0,0,.5\
25],64:[0,.61111\
,0,0,.525],65:[0\
,.61111,0,0,.525\
],66:[0,.61111,0\
,0,.525],67:[0,.\
61111,0,0,.525],\
68:[0,.61111,0,0\
,.525],69:[0,.61\
111,0,0,.525],70\
:[0,.61111,0,0,.\
525],71:[0,.6111\
1,0,0,.525],72:[\
0,.61111,0,0,.52\
5],73:[0,.61111,\
0,0,.525],74:[0,\
.61111,0,0,.525]\
,75:[0,.61111,0,\
0,.525],76:[0,.6\
1111,0,0,.525],7\
7:[0,.61111,0,0,\
.525],78:[0,.611\
11,0,0,.525],79:\
[0,.61111,0,0,.5\
25],80:[0,.61111\
,0,0,.525],81:[.\
13889,.61111,0,0\
,.525],82:[0,.61\
111,0,0,.525],83\
:[0,.61111,0,0,.\
525],84:[0,.6111\
1,0,0,.525],85:[\
0,.61111,0,0,.52\
5],86:[0,.61111,\
0,0,.525],87:[0,\
.61111,0,0,.525]\
,88:[0,.61111,0,\
0,.525],89:[0,.6\
1111,0,0,.525],9\
0:[0,.61111,0,0,\
.525],91:[.08333\
,.69444,0,0,.525\
],92:[.08333,.69\
444,0,0,.525],93\
:[.08333,.69444,\
0,0,.525],94:[0,\
.61111,0,0,.525]\
,95:[.09514,0,0,\
0,.525],96:[0,.6\
1111,0,0,.525],9\
7:[0,.43056,0,0,\
.525],98:[0,.611\
11,0,0,.525],99:\
[0,.43056,0,0,.5\
25],100:[0,.6111\
1,0,0,.525],101:\
[0,.43056,0,0,.5\
25],102:[0,.6111\
1,0,0,.525],103:\
[.22222,.43056,0\
,0,.525],104:[0,\
.61111,0,0,.525]\
,105:[0,.61111,0\
,0,.525],106:[.2\
2222,.61111,0,0,\
.525],107:[0,.61\
111,0,0,.525],10\
8:[0,.61111,0,0,\
.525],109:[0,.43\
056,0,0,.525],11\
0:[0,.43056,0,0,\
.525],111:[0,.43\
056,0,0,.525],11\
2:[.22222,.43056\
,0,0,.525],113:[\
.22222,.43056,0,\
0,.525],114:[0,.\
43056,0,0,.525],\
115:[0,.43056,0,\
0,.525],116:[0,.\
55358,0,0,.525],\
117:[0,.43056,0,\
0,.525],118:[0,.\
43056,0,0,.525],\
119:[0,.43056,0,\
0,.525],120:[0,.\
43056,0,0,.525],\
121:[.22222,.430\
56,0,0,.525],122\
:[0,.43056,0,0,.\
525],123:[.08333\
,.69444,0,0,.525\
],124:[.08333,.6\
9444,0,0,.525],1\
25:[.08333,.6944\
4,0,0,.525],126:\
[0,.61111,0,0,.5\
25],127:[0,.6111\
1,0,0,.525],160:\
[0,0,0,0,.525],1\
76:[0,.61111,0,0\
,.525],184:[.194\
45,0,0,0,.525],3\
05:[0,.43056,0,0\
,.525],567:[.222\
22,.43056,0,0,.5\
25],711:[0,.5659\
7,0,0,.525],713:\
[0,.56555,0,0,.5\
25],714:[0,.6111\
1,0,0,.525],715:\
[0,.61111,0,0,.5\
25],728:[0,.6111\
1,0,0,.525],730:\
[0,.61111,0,0,.5\
25],770:[0,.6111\
1,0,0,.525],771:\
[0,.61111,0,0,.5\
25],776:[0,.6111\
1,0,0,.525],915:\
[0,.61111,0,0,.5\
25],916:[0,.6111\
1,0,0,.525],920:\
[0,.61111,0,0,.5\
25],923:[0,.6111\
1,0,0,.525],926:\
[0,.61111,0,0,.5\
25],928:[0,.6111\
1,0,0,.525],931:\
[0,.61111,0,0,.5\
25],933:[0,.6111\
1,0,0,.525],934:\
[0,.61111,0,0,.5\
25],936:[0,.6111\
1,0,0,.525],937:\
[0,.61111,0,0,.5\
25],8216:[0,.611\
11,0,0,.525],821\
7:[0,.61111,0,0,\
.525],8242:[0,.6\
1111,0,0,.525],9\
251:[.11111,.219\
44,0,0,.525]}},B\
={slant:[.25,.25\
,.25],space:[0,0\
,0],stretch:[0,0\
,0],shrink:[0,0,\
0],xHeight:[.431\
,.431,.431],quad\
:[1,1.171,1.472]\
,extraSpace:[0,0\
,0],num1:[.677,.\
732,.925],num2:[\
.394,.384,.387],\
num3:[.444,.471,\
.504],denom1:[.6\
86,.752,1.025],d\
enom2:[.345,.344\
,.532],sup1:[.41\
3,.503,.504],sup\
2:[.363,.431,.40\
4],sup3:[.289,.2\
86,.294],sub1:[.\
15,.143,.2],sub2\
:[.247,.286,.4],\
supDrop:[.386,.3\
53,.494],subDrop\
:[.05,.071,.1],d\
elim1:[2.39,1.7,\
1.98],delim2:[1.\
01,1.157,1.42],a\
xisHeight:[.25,.\
25,.25],defaultR\
uleThickness:[.0\
4,.049,.049],big\
OpSpacing1:[.111\
,.111,.111],bigO\
pSpacing2:[.166,\
.166,.166],bigOp\
Spacing3:[.2,.2,\
.2],bigOpSpacing\
4:[.6,.611,.611]\
,bigOpSpacing5:[\
.1,.143,.143],sq\
rtRuleThickness:\
[.04,.04,.04],pt\
PerEm:[10,10,10]\
,doubleRuleSep:[\
.2,.2,.2],arrayR\
uleWidth:[.04,.0\
4,.04],fboxsep:[\
.3,.3,.3],fboxru\
le:[.04,.04,.04]\
},C={\x22\x5cxc5\x22:\x22A\x22,\
\x22\x5cxd0\x22:\x22D\x22,\x22\x5cxde\
\x22:\x22o\x22,\x22\x5cxe5\x22:\x22a\x22\
,\x22\x5cxf0\x22:\x22d\x22,\x22\x5cxf\
e\x22:\x22o\x22,\x22\x5cu0410\x22:\
\x22A\x22,\x22\x5cu0411\x22:\x22B\x22\
,\x22\x5cu0412\x22:\x22B\x22,\x22\x5c\
u0413\x22:\x22F\x22,\x22\x5cu04\
14\x22:\x22A\x22,\x22\x5cu0415\x22\
:\x22E\x22,\x22\x5cu0416\x22:\x22K\
\x22,\x22\x5cu0417\x22:\x223\x22,\x22\
\x5cu0418\x22:\x22N\x22,\x22\x5cu0\
419\x22:\x22N\x22,\x22\x5cu041a\
\x22:\x22K\x22,\x22\x5cu041b\x22:\x22\
N\x22,\x22\x5cu041c\x22:\x22M\x22,\
\x22\x5cu041d\x22:\x22H\x22,\x22\x5cu\
041e\x22:\x22O\x22,\x22\x5cu041\
f\x22:\x22N\x22,\x22\x5cu0420\x22:\
\x22P\x22,\x22\x5cu0421\x22:\x22C\x22\
,\x22\x5cu0422\x22:\x22T\x22,\x22\x5c\
u0423\x22:\x22y\x22,\x22\x5cu04\
24\x22:\x22O\x22,\x22\x5cu0425\x22\
:\x22X\x22,\x22\x5cu0426\x22:\x22U\
\x22,\x22\x5cu0427\x22:\x22h\x22,\x22\
\x5cu0428\x22:\x22W\x22,\x22\x5cu0\
429\x22:\x22W\x22,\x22\x5cu042a\
\x22:\x22B\x22,\x22\x5cu042b\x22:\x22\
X\x22,\x22\x5cu042c\x22:\x22B\x22,\
\x22\x5cu042d\x22:\x223\x22,\x22\x5cu\
042e\x22:\x22X\x22,\x22\x5cu042\
f\x22:\x22R\x22,\x22\x5cu0430\x22:\
\x22a\x22,\x22\x5cu0431\x22:\x22b\x22\
,\x22\x5cu0432\x22:\x22a\x22,\x22\x5c\
u0433\x22:\x22r\x22,\x22\x5cu04\
34\x22:\x22y\x22,\x22\x5cu0435\x22\
:\x22e\x22,\x22\x5cu0436\x22:\x22m\
\x22,\x22\x5cu0437\x22:\x22e\x22,\x22\
\x5cu0438\x22:\x22n\x22,\x22\x5cu0\
439\x22:\x22n\x22,\x22\x5cu043a\
\x22:\x22n\x22,\x22\x5cu043b\x22:\x22\
n\x22,\x22\x5cu043c\x22:\x22m\x22,\
\x22\x5cu043d\x22:\x22n\x22,\x22\x5cu\
043e\x22:\x22o\x22,\x22\x5cu043\
f\x22:\x22n\x22,\x22\x5cu0440\x22:\
\x22p\x22,\x22\x5cu0441\x22:\x22c\x22\
,\x22\x5cu0442\x22:\x22o\x22,\x22\x5c\
u0443\x22:\x22y\x22,\x22\x5cu04\
44\x22:\x22b\x22,\x22\x5cu0445\x22\
:\x22x\x22,\x22\x5cu0446\x22:\x22n\
\x22,\x22\x5cu0447\x22:\x22n\x22,\x22\
\x5cu0448\x22:\x22w\x22,\x22\x5cu0\
449\x22:\x22w\x22,\x22\x5cu044a\
\x22:\x22a\x22,\x22\x5cu044b\x22:\x22\
m\x22,\x22\x5cu044c\x22:\x22a\x22,\
\x22\x5cu044d\x22:\x22e\x22,\x22\x5cu\
044e\x22:\x22m\x22,\x22\x5cu044\
f\x22:\x22r\x22};function\
 N(e,t,r){if(!T[\
t])throw new Err\
or(\x22Font metrics\
 not found for f\
ont: \x22+t+\x22.\x22);va\
r n=e.charCodeAt\
(0),a=T[t][n];if\
(!a&&e[0]in C&&(\
n=C[e[0]].charCo\
deAt(0),a=T[t][n\
]),a||\x22text\x22!==r\
||S(n)&&(a=T[t][\
77]),a)return{de\
pth:a[0],height:\
a[1],italic:a[2]\
,skew:a[3],width\
:a[4]}}var q={};\
var I=[[1,1,1],[\
2,1,1],[3,1,1],[\
4,2,1],[5,2,1],[\
6,3,1],[7,4,2],[\
8,6,3],[9,7,6],[\
10,8,7],[11,10,9\
]],R=[.5,.6,.7,.\
8,.9,1,1.2,1.44,\
1.728,2.074,2.48\
8],H=function(e,\
t){return t.size\
<2?e:I[e-1][t.si\
ze-1]},O=functio\
n(){function e(t\
){this.style=voi\
d 0,this.color=v\
oid 0,this.size=\
void 0,this.text\
Size=void 0,this\
.phantom=void 0,\
this.font=void 0\
,this.fontFamily\
=void 0,this.fon\
tWeight=void 0,t\
his.fontShape=vo\
id 0,this.sizeMu\
ltiplier=void 0,\
this.maxSize=voi\
d 0,this.minRule\
Thickness=void 0\
,this._fontMetri\
cs=void 0,this.s\
tyle=t.style,thi\
s.color=t.color,\
this.size=t.size\
||e.BASESIZE,thi\
s.textSize=t.tex\
tSize||this.size\
,this.phantom=!!\
t.phantom,this.f\
ont=t.font||\x22\x22,t\
his.fontFamily=t\
.fontFamily||\x22\x22,\
this.fontWeight=\
t.fontWeight||\x22\x22\
,this.fontShape=\
t.fontShape||\x22\x22,\
this.sizeMultipl\
ier=R[this.size-\
1],this.maxSize=\
t.maxSize,this.m\
inRuleThickness=\
t.minRuleThickne\
ss,this._fontMet\
rics=void 0}var \
t=e.prototype;re\
turn t.extend=fu\
nction(t){var r=\
{style:this.styl\
e,size:this.size\
,textSize:this.t\
extSize,color:th\
is.color,phantom\
:this.phantom,fo\
nt:this.font,fon\
tFamily:this.fon\
tFamily,fontWeig\
ht:this.fontWeig\
ht,fontShape:thi\
s.fontShape,maxS\
ize:this.maxSize\
,minRuleThicknes\
s:this.minRuleTh\
ickness};for(var\
 n in t)t.hasOwn\
Property(n)&&(r[\
n]=t[n]);return \
new e(r)},t.havi\
ngStyle=function\
(e){return this.\
style===e?this:t\
his.extend({styl\
e:e,size:H(this.\
textSize,e)})},t\
.havingCrampedSt\
yle=function(){r\
eturn this.havin\
gStyle(this.styl\
e.cramp())},t.ha\
vingSize=functio\
n(e){return this\
.size===e&&this.\
textSize===e?thi\
s:this.extend({s\
tyle:this.style.\
text(),size:e,te\
xtSize:e,sizeMul\
tiplier:R[e-1]})\
},t.havingBaseSt\
yle=function(t){\
t=t||this.style.\
text();var r=H(e\
.BASESIZE,t);ret\
urn this.size===\
r&&this.textSize\
===e.BASESIZE&&t\
his.style===t?th\
is:this.extend({\
style:t,size:r})\
},t.havingBaseSi\
zing=function(){\
var e;switch(thi\
s.style.id){case\
 4:case 5:e=3;br\
eak;case 6:case \
7:e=1;break;defa\
ult:e=6}return t\
his.extend({styl\
e:this.style.tex\
t(),size:e})},t.\
withColor=functi\
on(e){return thi\
s.extend({color:\
e})},t.withPhant\
om=function(){re\
turn this.extend\
({phantom:!0})},\
t.withFont=funct\
ion(e){return th\
is.extend({font:\
e})},t.withTextF\
ontFamily=functi\
on(e){return thi\
s.extend({fontFa\
mily:e,font:\x22\x22})\
},t.withTextFont\
Weight=function(\
e){return this.e\
xtend({fontWeigh\
t:e,font:\x22\x22})},t\
.withTextFontSha\
pe=function(e){r\
eturn this.exten\
d({fontShape:e,f\
ont:\x22\x22})},t.sizi\
ngClasses=functi\
on(e){return e.s\
ize!==this.size?\
[\x22sizing\x22,\x22reset\
-size\x22+e.size,\x22s\
ize\x22+this.size]:\
[]},t.baseSizing\
Classes=function\
(){return this.s\
ize!==e.BASESIZE\
?[\x22sizing\x22,\x22rese\
t-size\x22+this.siz\
e,\x22size\x22+e.BASES\
IZE]:[]},t.fontM\
etrics=function(\
){return this._f\
ontMetrics||(thi\
s._fontMetrics=f\
unction(e){var t\
;if(!q[t=e>=5?0:\
e>=3?1:2]){var r\
=q[t]={cssEmPerM\
u:B.quad[t]/18};\
for(var n in B)B\
.hasOwnProperty(\
n)&&(r[n]=B[n][t\
])}return q[t]}(\
this.size)),this\
._fontMetrics},t\
.getColor=functi\
on(){return this\
.phantom?\x22transp\
arent\x22:this.colo\
r},e}();O.BASESI\
ZE=6;var E=O,L={\
pt:1,mm:7227/254\
0,cm:7227/254,in\
:72.27,bp:1.0037\
5,pc:12,dd:1238/\
1157,cc:14856/11\
57,nd:685/642,nc\
:1370/107,sp:1/6\
5536,px:1.00375}\
,D={ex:!0,em:!0,\
mu:!0},V=functio\
n(e){return\x22stri\
ng\x22!=typeof e&&(\
e=e.unit),e in L\
||e in D||\x22ex\x22==\
=e},P=function(e\
,t){var r;if(e.u\
nit in L)r=L[e.u\
nit]/t.fontMetri\
cs().ptPerEm/t.s\
izeMultiplier;el\
se if(\x22mu\x22===e.u\
nit)r=t.fontMetr\
ics().cssEmPerMu\
;else{var a;if(a\
=t.style.isTight\
()?t.havingStyle\
(t.style.text())\
:t,\x22ex\x22===e.unit\
)r=a.fontMetrics\
().xHeight;else{\
if(\x22em\x22!==e.unit\
)throw new n(\x22In\
valid unit: '\x22+e\
.unit+\x22'\x22);r=a.f\
ontMetrics().qua\
d}a!==t&&(r*=a.s\
izeMultiplier/t.\
sizeMultiplier)}\
return Math.min(\
e.number*r,t.max\
Size)},F=functio\
n(e){return+e.to\
Fixed(4)+\x22em\x22},G\
=function(e){ret\
urn e.filter((fu\
nction(e){return\
 e})).join(\x22 \x22)}\
,U=function(e,t,\
r){if(this.class\
es=e||[],this.at\
tributes={},this\
.height=0,this.d\
epth=0,this.maxF\
ontSize=0,this.s\
tyle=r||{},t){t.\
style.isTight()&\
&this.classes.pu\
sh(\x22mtight\x22);var\
 n=t.getColor();\
n&&(this.style.c\
olor=n)}},Y=func\
tion(e){var t=do\
cument.createEle\
ment(e);for(var \
r in t.className\
=G(this.classes)\
,this.style)this\
.style.hasOwnPro\
perty(r)&&(t.sty\
le[r]=this.style\
[r]);for(var n i\
n this.attribute\
s)this.attribute\
s.hasOwnProperty\
(n)&&t.setAttrib\
ute(n,this.attri\
butes[n]);for(va\
r a=0;a<this.chi\
ldren.length;a++\
)t.appendChild(t\
his.children[a].\
toNode());return\
 t},X=function(e\
){var t=\x22<\x22+e;th\
is.classes.lengt\
h&&(t+=' class=\x22\
'+l.escape(G(thi\
s.classes))+'\x22')\
;var r=\x22\x22;for(va\
r n in this.styl\
e)this.style.has\
OwnProperty(n)&&\
(r+=l.hyphenate(\
n)+\x22:\x22+this.styl\
e[n]+\x22;\x22);for(va\
r a in r&&(t+=' \
style=\x22'+l.escap\
e(r)+'\x22'),this.a\
ttributes)this.a\
ttributes.hasOwn\
Property(a)&&(t+\
=\x22 \x22+a+'=\x22'+l.es\
cape(this.attrib\
utes[a])+'\x22');t+\
=\x22>\x22;for(var i=0\
;i<this.children\
.length;i++)t+=t\
his.children[i].\
toMarkup();retur\
n t+=\x22</\x22+e+\x22>\x22}\
,W=function(){fu\
nction e(e,t,r,n\
){this.children=\
void 0,this.attr\
ibutes=void 0,th\
is.classes=void \
0,this.height=vo\
id 0,this.depth=\
void 0,this.widt\
h=void 0,this.ma\
xFontSize=void 0\
,this.style=void\
 0,U.call(this,e\
,r,n),this.child\
ren=t||[]}var t=\
e.prototype;retu\
rn t.setAttribut\
e=function(e,t){\
this.attributes[\
e]=t},t.hasClass\
=function(e){ret\
urn l.contains(t\
his.classes,e)},\
t.toNode=functio\
n(){return Y.cal\
l(this,\x22span\x22)},\
t.toMarkup=funct\
ion(){return X.c\
all(this,\x22span\x22)\
},e}(),_=functio\
n(){function e(e\
,t,r,n){this.chi\
ldren=void 0,thi\
s.attributes=voi\
d 0,this.classes\
=void 0,this.hei\
ght=void 0,this.\
depth=void 0,thi\
s.maxFontSize=vo\
id 0,this.style=\
void 0,U.call(th\
is,t,n),this.chi\
ldren=r||[],this\
.setAttribute(\x22h\
ref\x22,e)}var t=e.\
prototype;return\
 t.setAttribute=\
function(e,t){th\
is.attributes[e]\
=t},t.hasClass=f\
unction(e){retur\
n l.contains(thi\
s.classes,e)},t.\
toNode=function(\
){return Y.call(\
this,\x22a\x22)},t.toM\
arkup=function()\
{return X.call(t\
his,\x22a\x22)},e}(),j\
=function(){func\
tion e(e,t,r){th\
is.src=void 0,th\
is.alt=void 0,th\
is.classes=void \
0,this.height=vo\
id 0,this.depth=\
void 0,this.maxF\
ontSize=void 0,t\
his.style=void 0\
,this.alt=t,this\
.src=e,this.clas\
ses=[\x22mord\x22],thi\
s.style=r}var t=\
e.prototype;retu\
rn t.hasClass=fu\
nction(e){return\
 l.contains(this\
.classes,e)},t.t\
oNode=function()\
{var e=document.\
createElement(\x22i\
mg\x22);for(var t i\
n e.src=this.src\
,e.alt=this.alt,\
e.className=\x22mor\
d\x22,this.style)th\
is.style.hasOwnP\
roperty(t)&&(e.s\
tyle[t]=this.sty\
le[t]);return e}\
,t.toMarkup=func\
tion(){var e=\x22<i\
mg  src='\x22+this.\
src+\x22 'alt='\x22+th\
is.alt+\x22' \x22,t=\x22\x22\
;for(var r in th\
is.style)this.st\
yle.hasOwnProper\
ty(r)&&(t+=l.hyp\
henate(r)+\x22:\x22+th\
is.style[r]+\x22;\x22)\
;return t&&(e+='\
 style=\x22'+l.esca\
pe(t)+'\x22'),e+=\x22'\
/>\x22},e}(),$={\x22\x5cx\
ee\x22:\x22\x5cu0131\x5cu030\
2\x22,\x22\x5cxef\x22:\x22\x5cu013\
1\x5cu0308\x22,\x22\x5cxed\x22:\
\x22\x5cu0131\x5cu0301\x22,\x22\
\x5cxec\x22:\x22\x5cu0131\x5cu0\
300\x22},Z=function\
(){function e(e,\
t,r,n,a,i,o,s){t\
his.text=void 0,\
this.height=void\
 0,this.depth=vo\
id 0,this.italic\
=void 0,this.ske\
w=void 0,this.wi\
dth=void 0,this.\
maxFontSize=void\
 0,this.classes=\
void 0,this.styl\
e=void 0,this.te\
xt=e,this.height\
=t||0,this.depth\
=r||0,this.itali\
c=n||0,this.skew\
=a||0,this.width\
=i||0,this.class\
es=o||[],this.st\
yle=s||{},this.m\
axFontSize=0;var\
 l=function(e){f\
or(var t=0;t<w.l\
ength;t++)for(va\
r r=w[t],n=0;n<r\
.blocks.length;n\
++){var a=r.bloc\
ks[n];if(e>=a[0]\
&&e<=a[1])return\
 r.name}return n\
ull}(this.text.c\
harCodeAt(0));l&\
&this.classes.pu\
sh(l+\x22_fallback\x22\
),/[\x5cxee\x5cxef\x5cxed\
\x5cxec]/.test(this\
.text)&&(this.te\
xt=$[this.text])\
}var t=e.prototy\
pe;return t.hasC\
lass=function(e)\
{return l.contai\
ns(this.classes,\
e)},t.toNode=fun\
ction(){var e=do\
cument.createTex\
tNode(this.text)\
,t=null;for(var \
r in this.italic\
>0&&((t=document\
.createElement(\x22\
span\x22)).style.ma\
rginRight=F(this\
.italic)),this.c\
lasses.length>0&\
&((t=t||document\
.createElement(\x22\
span\x22)).classNam\
e=G(this.classes\
)),this.style)th\
is.style.hasOwnP\
roperty(r)&&((t=\
t||document.crea\
teElement(\x22span\x22\
)).style[r]=this\
.style[r]);retur\
n t?(t.appendChi\
ld(e),t):e},t.to\
Markup=function(\
){var e=!1,t=\x22<s\
pan\x22;this.classe\
s.length&&(e=!0,\
t+=' class=\x22',t+\
=l.escape(G(this\
.classes)),t+='\x22\
');var r=\x22\x22;for(\
var n in this.it\
alic>0&&(r+=\x22mar\
gin-right:\x22+this\
.italic+\x22em;\x22),t\
his.style)this.s\
tyle.hasOwnPrope\
rty(n)&&(r+=l.hy\
phenate(n)+\x22:\x22+t\
his.style[n]+\x22;\x22\
);r&&(e=!0,t+=' \
style=\x22'+l.escap\
e(r)+'\x22');var a=\
l.escape(this.te\
xt);return e?(t+\
=\x22>\x22,t+=a,t+=\x22</\
span>\x22):a},e}(),\
K=function(){fun\
ction e(e,t){thi\
s.children=void \
0,this.attribute\
s=void 0,this.ch\
ildren=e||[],thi\
s.attributes=t||\
{}}var t=e.proto\
type;return t.to\
Node=function(){\
var e=document.c\
reateElementNS(\x22\
http://www.w3.or\
g/2000/svg\x22,\x22svg\
\x22);for(var t in \
this.attributes)\
Object.prototype\
.hasOwnProperty.\
call(this.attrib\
utes,t)&&e.setAt\
tribute(t,this.a\
ttributes[t]);fo\
r(var r=0;r<this\
.children.length\
;r++)e.appendChi\
ld(this.children\
[r].toNode());re\
turn e},t.toMark\
up=function(){va\
r e='<svg xmlns=\
\x22http://www.w3.o\
rg/2000/svg\x22';fo\
r(var t in this.\
attributes)Objec\
t.prototype.hasO\
wnProperty.call(\
this.attributes,\
t)&&(e+=\x22 \x22+t+\x22=\
'\x22+this.attribut\
es[t]+\x22'\x22);e+=\x22>\
\x22;for(var r=0;r<\
this.children.le\
ngth;r++)e+=this\
.children[r].toM\
arkup();return e\
+=\x22</svg>\x22},e}()\
,J=function(){fu\
nction e(e,t){th\
is.pathName=void\
 0,this.alternat\
e=void 0,this.pa\
thName=e,this.al\
ternate=t}var t=\
e.prototype;retu\
rn t.toNode=func\
tion(){var e=doc\
ument.createElem\
entNS(\x22http://ww\
w.w3.org/2000/sv\
g\x22,\x22path\x22);retur\
n this.alternate\
?e.setAttribute(\
\x22d\x22,this.alterna\
te):e.setAttribu\
te(\x22d\x22,z[this.pa\
thName]),e},t.to\
Markup=function(\
){return this.al\
ternate?\x22<path d\
='\x22+this.alterna\
te+\x22'/>\x22:\x22<path \
d='\x22+z[this.path\
Name]+\x22'/>\x22},e}(\
),Q=function(){f\
unction e(e){thi\
s.attributes=voi\
d 0,this.attribu\
tes=e||{}}var t=\
e.prototype;retu\
rn t.toNode=func\
tion(){var e=doc\
ument.createElem\
entNS(\x22http://ww\
w.w3.org/2000/sv\
g\x22,\x22line\x22);for(v\
ar t in this.att\
ributes)Object.p\
rototype.hasOwnP\
roperty.call(thi\
s.attributes,t)&\
&e.setAttribute(\
t,this.attribute\
s[t]);return e},\
t.toMarkup=funct\
ion(){var e=\x22<li\
ne\x22;for(var t in\
 this.attributes\
)Object.prototyp\
e.hasOwnProperty\
.call(this.attri\
butes,t)&&(e+=\x22 \
\x22+t+\x22='\x22+this.at\
tributes[t]+\x22'\x22)\
;return e+=\x22/>\x22}\
,e}();function e\
e(e){if(e instan\
ceof Z)return e;\
throw new Error(\
\x22Expected symbol\
Node but got \x22+S\
tring(e)+\x22.\x22)}va\
r te={bin:1,clos\
e:1,inner:1,open\
:1,punct:1,rel:1\
},re={\x22accent-to\
ken\x22:1,mathord:1\
,\x22op-token\x22:1,sp\
acing:1,textord:\
1},ne={math:{},t\
ext:{}},ae=ne;fu\
nction ie(e,t,r,\
n,a,i){ne[e][a]=\
{font:t,group:r,\
replace:n},i&&n&\
&(ne[e][n]=ne[e]\
[a])}var oe=\x22mat\
h\x22,se=\x22text\x22,le=\
\x22main\x22,he=\x22ams\x22,\
ce=\x22accent-token\
\x22,me=\x22bin\x22,ue=\x22c\
lose\x22,pe=\x22inner\x22\
,de=\x22mathord\x22,fe\
=\x22op-token\x22,ge=\x22\
open\x22,ve=\x22punct\x22\
,be=\x22rel\x22,ye=\x22sp\
acing\x22,xe=\x22texto\
rd\x22;ie(oe,le,be,\
\x22\x5cu2261\x22,\x22\x5c\x5cequi\
v\x22,!0),ie(oe,le,\
be,\x22\x5cu227a\x22,\x22\x5c\x5cp\
rec\x22,!0),ie(oe,l\
e,be,\x22\x5cu227b\x22,\x22\x5c\
\x5csucc\x22,!0),ie(oe\
,le,be,\x22\x5cu223c\x22,\
\x22\x5c\x5csim\x22,!0),ie(o\
e,le,be,\x22\x5cu22a5\x22\
,\x22\x5c\x5cperp\x22),ie(oe\
,le,be,\x22\x5cu2aaf\x22,\
\x22\x5c\x5cpreceq\x22,!0),i\
e(oe,le,be,\x22\x5cu2a\
b0\x22,\x22\x5c\x5csucceq\x22,!\
0),ie(oe,le,be,\x22\
\x5cu2243\x22,\x22\x5c\x5csimeq\
\x22,!0),ie(oe,le,b\
e,\x22\x5cu2223\x22,\x22\x5c\x5cmi\
d\x22,!0),ie(oe,le,\
be,\x22\x5cu226a\x22,\x22\x5c\x5cl\
l\x22,!0),ie(oe,le,\
be,\x22\x5cu226b\x22,\x22\x5c\x5cg\
g\x22,!0),ie(oe,le,\
be,\x22\x5cu224d\x22,\x22\x5c\x5ca\
symp\x22,!0),ie(oe,\
le,be,\x22\x5cu2225\x22,\x22\
\x5c\x5cparallel\x22),ie(\
oe,le,be,\x22\x5cu22c8\
\x22,\x22\x5c\x5cbowtie\x22,!0)\
,ie(oe,le,be,\x22\x5cu\
2323\x22,\x22\x5c\x5csmile\x22,\
!0),ie(oe,le,be,\
\x22\x5cu2291\x22,\x22\x5c\x5csqsu\
bseteq\x22,!0),ie(o\
e,le,be,\x22\x5cu2292\x22\
,\x22\x5c\x5csqsupseteq\x22,\
!0),ie(oe,le,be,\
\x22\x5cu2250\x22,\x22\x5c\x5cdote\
q\x22,!0),ie(oe,le,\
be,\x22\x5cu2322\x22,\x22\x5c\x5cf\
rown\x22,!0),ie(oe,\
le,be,\x22\x5cu220b\x22,\x22\
\x5c\x5cni\x22,!0),ie(oe,\
le,be,\x22\x5cu221d\x22,\x22\
\x5c\x5cpropto\x22,!0),ie\
(oe,le,be,\x22\x5cu22a\
2\x22,\x22\x5c\x5cvdash\x22,!0)\
,ie(oe,le,be,\x22\x5cu\
22a3\x22,\x22\x5c\x5cdashv\x22,\
!0),ie(oe,le,be,\
\x22\x5cu220b\x22,\x22\x5c\x5cowns\
\x22),ie(oe,le,ve,\x22\
.\x22,\x22\x5c\x5cldotp\x22),ie\
(oe,le,ve,\x22\x5cu22c\
5\x22,\x22\x5c\x5ccdotp\x22),ie\
(oe,le,xe,\x22#\x22,\x22\x5c\
\x5c#\x22),ie(se,le,xe\
,\x22#\x22,\x22\x5c\x5c#\x22),ie(o\
e,le,xe,\x22&\x22,\x22\x5c\x5c&\
\x22),ie(se,le,xe,\x22\
&\x22,\x22\x5c\x5c&\x22),ie(oe,\
le,xe,\x22\x5cu2135\x22,\x22\
\x5c\x5caleph\x22,!0),ie(\
oe,le,xe,\x22\x5cu2200\
\x22,\x22\x5c\x5cforall\x22,!0)\
,ie(oe,le,xe,\x22\x5cu\
210f\x22,\x22\x5c\x5chbar\x22,!\
0),ie(oe,le,xe,\x22\
\x5cu2203\x22,\x22\x5c\x5cexist\
s\x22,!0),ie(oe,le,\
xe,\x22\x5cu2207\x22,\x22\x5c\x5cn\
abla\x22,!0),ie(oe,\
le,xe,\x22\x5cu266d\x22,\x22\
\x5c\x5cflat\x22,!0),ie(o\
e,le,xe,\x22\x5cu2113\x22\
,\x22\x5c\x5cell\x22,!0),ie(\
oe,le,xe,\x22\x5cu266e\
\x22,\x22\x5c\x5cnatural\x22,!0\
),ie(oe,le,xe,\x22\x5c\
u2663\x22,\x22\x5c\x5cclubsu\
it\x22,!0),ie(oe,le\
,xe,\x22\x5cu2118\x22,\x22\x5c\x5c\
wp\x22,!0),ie(oe,le\
,xe,\x22\x5cu266f\x22,\x22\x5c\x5c\
sharp\x22,!0),ie(oe\
,le,xe,\x22\x5cu2662\x22,\
\x22\x5c\x5cdiamondsuit\x22,\
!0),ie(oe,le,xe,\
\x22\x5cu211c\x22,\x22\x5c\x5cRe\x22,\
!0),ie(oe,le,xe,\
\x22\x5cu2661\x22,\x22\x5c\x5chear\
tsuit\x22,!0),ie(oe\
,le,xe,\x22\x5cu2111\x22,\
\x22\x5c\x5cIm\x22,!0),ie(oe\
,le,xe,\x22\x5cu2660\x22,\
\x22\x5c\x5cspadesuit\x22,!0\
),ie(oe,le,xe,\x22\x5c\
xa7\x22,\x22\x5c\x5cS\x22,!0),i\
e(se,le,xe,\x22\x5cxa7\
\x22,\x22\x5c\x5cS\x22),ie(oe,l\
e,xe,\x22\x5cxb6\x22,\x22\x5c\x5cP\
\x22,!0),ie(se,le,x\
e,\x22\x5cxb6\x22,\x22\x5c\x5cP\x22),\
ie(oe,le,xe,\x22\x5cu2\
020\x22,\x22\x5c\x5cdag\x22),ie\
(se,le,xe,\x22\x5cu202\
0\x22,\x22\x5c\x5cdag\x22),ie(s\
e,le,xe,\x22\x5cu2020\x22\
,\x22\x5c\x5ctextdagger\x22)\
,ie(oe,le,xe,\x22\x5cu\
2021\x22,\x22\x5c\x5cddag\x22),\
ie(se,le,xe,\x22\x5cu2\
021\x22,\x22\x5c\x5cddag\x22),i\
e(se,le,xe,\x22\x5cu20\
21\x22,\x22\x5c\x5ctextdagge\
rdbl\x22),ie(oe,le,\
ue,\x22\x5cu23b1\x22,\x22\x5c\x5cr\
moustache\x22,!0),i\
e(oe,le,ge,\x22\x5cu23\
b0\x22,\x22\x5c\x5clmoustach\
e\x22,!0),ie(oe,le,\
ue,\x22\x5cu27ef\x22,\x22\x5c\x5cr\
group\x22,!0),ie(oe\
,le,ge,\x22\x5cu27ee\x22,\
\x22\x5c\x5clgroup\x22,!0),i\
e(oe,le,me,\x22\x5cu22\
13\x22,\x22\x5c\x5cmp\x22,!0),i\
e(oe,le,me,\x22\x5cu22\
96\x22,\x22\x5c\x5cominus\x22,!\
0),ie(oe,le,me,\x22\
\x5cu228e\x22,\x22\x5c\x5cuplus\
\x22,!0),ie(oe,le,m\
e,\x22\x5cu2293\x22,\x22\x5c\x5csq\
cap\x22,!0),ie(oe,l\
e,me,\x22\x5cu2217\x22,\x22\x5c\
\x5cast\x22),ie(oe,le,\
me,\x22\x5cu2294\x22,\x22\x5c\x5cs\
qcup\x22,!0),ie(oe,\
le,me,\x22\x5cu25ef\x22,\x22\
\x5c\x5cbigcirc\x22,!0),i\
e(oe,le,me,\x22\x5cu22\
19\x22,\x22\x5c\x5cbullet\x22,!\
0),ie(oe,le,me,\x22\
\x5cu2021\x22,\x22\x5c\x5cddagg\
er\x22),ie(oe,le,me\
,\x22\x5cu2240\x22,\x22\x5c\x5cwr\x22\
,!0),ie(oe,le,me\
,\x22\x5cu2a3f\x22,\x22\x5c\x5cama\
lg\x22),ie(oe,le,me\
,\x22&\x22,\x22\x5c\x5cAnd\x22),ie\
(oe,le,be,\x22\x5cu27f\
5\x22,\x22\x5c\x5clongleftar\
row\x22,!0),ie(oe,l\
e,be,\x22\x5cu21d0\x22,\x22\x5c\
\x5cLeftarrow\x22,!0),\
ie(oe,le,be,\x22\x5cu2\
7f8\x22,\x22\x5c\x5cLongleft\
arrow\x22,!0),ie(oe\
,le,be,\x22\x5cu27f6\x22,\
\x22\x5c\x5clongrightarro\
w\x22,!0),ie(oe,le,\
be,\x22\x5cu21d2\x22,\x22\x5c\x5cR\
ightarrow\x22,!0),i\
e(oe,le,be,\x22\x5cu27\
f9\x22,\x22\x5c\x5cLongright\
arrow\x22,!0),ie(oe\
,le,be,\x22\x5cu2194\x22,\
\x22\x5c\x5cleftrightarro\
w\x22,!0),ie(oe,le,\
be,\x22\x5cu27f7\x22,\x22\x5c\x5cl\
ongleftrightarro\
w\x22,!0),ie(oe,le,\
be,\x22\x5cu21d4\x22,\x22\x5c\x5cL\
eftrightarrow\x22,!\
0),ie(oe,le,be,\x22\
\x5cu27fa\x22,\x22\x5c\x5cLongl\
eftrightarrow\x22,!\
0),ie(oe,le,be,\x22\
\x5cu21a6\x22,\x22\x5c\x5cmapst\
o\x22,!0),ie(oe,le,\
be,\x22\x5cu27fc\x22,\x22\x5c\x5cl\
ongmapsto\x22,!0),i\
e(oe,le,be,\x22\x5cu21\
97\x22,\x22\x5c\x5cnearrow\x22,\
!0),ie(oe,le,be,\
\x22\x5cu21a9\x22,\x22\x5c\x5chook\
leftarrow\x22,!0),i\
e(oe,le,be,\x22\x5cu21\
aa\x22,\x22\x5c\x5chookright\
arrow\x22,!0),ie(oe\
,le,be,\x22\x5cu2198\x22,\
\x22\x5c\x5csearrow\x22,!0),\
ie(oe,le,be,\x22\x5cu2\
1bc\x22,\x22\x5c\x5cleftharp\
oonup\x22,!0),ie(oe\
,le,be,\x22\x5cu21c0\x22,\
\x22\x5c\x5crightharpoonu\
p\x22,!0),ie(oe,le,\
be,\x22\x5cu2199\x22,\x22\x5c\x5cs\
warrow\x22,!0),ie(o\
e,le,be,\x22\x5cu21bd\x22\
,\x22\x5c\x5cleftharpoond\
own\x22,!0),ie(oe,l\
e,be,\x22\x5cu21c1\x22,\x22\x5c\
\x5crightharpoondow\
n\x22,!0),ie(oe,le,\
be,\x22\x5cu2196\x22,\x22\x5c\x5cn\
warrow\x22,!0),ie(o\
e,le,be,\x22\x5cu21cc\x22\
,\x22\x5c\x5crightlefthar\
poons\x22,!0),ie(oe\
,he,be,\x22\x5cu226e\x22,\
\x22\x5c\x5cnless\x22,!0),ie\
(oe,he,be,\x22\x5cue01\
0\x22,\x22\x5c\x5c@nleqslant\
\x22),ie(oe,he,be,\x22\
\x5cue011\x22,\x22\x5c\x5c@nleq\
q\x22),ie(oe,he,be,\
\x22\x5cu2a87\x22,\x22\x5c\x5clneq\
\x22,!0),ie(oe,he,b\
e,\x22\x5cu2268\x22,\x22\x5c\x5cln\
eqq\x22,!0),ie(oe,h\
e,be,\x22\x5cue00c\x22,\x22\x5c\
\x5c@lvertneqq\x22),ie\
(oe,he,be,\x22\x5cu22e\
6\x22,\x22\x5c\x5clnsim\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
2a89\x22,\x22\x5c\x5clnappro\
x\x22,!0),ie(oe,he,\
be,\x22\x5cu2280\x22,\x22\x5c\x5cn\
prec\x22,!0),ie(oe,\
he,be,\x22\x5cu22e0\x22,\x22\
\x5c\x5cnpreceq\x22,!0),i\
e(oe,he,be,\x22\x5cu22\
e8\x22,\x22\x5c\x5cprecnsim\x22\
,!0),ie(oe,he,be\
,\x22\x5cu2ab9\x22,\x22\x5c\x5cpre\
cnapprox\x22,!0),ie\
(oe,he,be,\x22\x5cu224\
1\x22,\x22\x5c\x5cnsim\x22,!0),\
ie(oe,he,be,\x22\x5cue\
006\x22,\x22\x5c\x5c@nshortm\
id\x22),ie(oe,he,be\
,\x22\x5cu2224\x22,\x22\x5c\x5cnmi\
d\x22,!0),ie(oe,he,\
be,\x22\x5cu22ac\x22,\x22\x5c\x5cn\
vdash\x22,!0),ie(oe\
,he,be,\x22\x5cu22ad\x22,\
\x22\x5c\x5cnvDash\x22,!0),i\
e(oe,he,be,\x22\x5cu22\
ea\x22,\x22\x5c\x5cntriangle\
left\x22),ie(oe,he,\
be,\x22\x5cu22ec\x22,\x22\x5c\x5cn\
trianglelefteq\x22,\
!0),ie(oe,he,be,\
\x22\x5cu228a\x22,\x22\x5c\x5csubs\
etneq\x22,!0),ie(oe\
,he,be,\x22\x5cue01a\x22,\
\x22\x5c\x5c@varsubsetneq\
\x22),ie(oe,he,be,\x22\
\x5cu2acb\x22,\x22\x5c\x5csubse\
tneqq\x22,!0),ie(oe\
,he,be,\x22\x5cue017\x22,\
\x22\x5c\x5c@varsubsetneq\
q\x22),ie(oe,he,be,\
\x22\x5cu226f\x22,\x22\x5c\x5cngtr\
\x22,!0),ie(oe,he,b\
e,\x22\x5cue00f\x22,\x22\x5c\x5c@n\
geqslant\x22),ie(oe\
,he,be,\x22\x5cue00e\x22,\
\x22\x5c\x5c@ngeqq\x22),ie(o\
e,he,be,\x22\x5cu2a88\x22\
,\x22\x5c\x5cgneq\x22,!0),ie\
(oe,he,be,\x22\x5cu226\
9\x22,\x22\x5c\x5cgneqq\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
e00d\x22,\x22\x5c\x5c@gvertn\
eqq\x22),ie(oe,he,b\
e,\x22\x5cu22e7\x22,\x22\x5c\x5cgn\
sim\x22,!0),ie(oe,h\
e,be,\x22\x5cu2a8a\x22,\x22\x5c\
\x5cgnapprox\x22,!0),i\
e(oe,he,be,\x22\x5cu22\
81\x22,\x22\x5c\x5cnsucc\x22,!0\
),ie(oe,he,be,\x22\x5c\
u22e1\x22,\x22\x5c\x5cnsucce\
q\x22,!0),ie(oe,he,\
be,\x22\x5cu22e9\x22,\x22\x5c\x5cs\
uccnsim\x22,!0),ie(\
oe,he,be,\x22\x5cu2aba\
\x22,\x22\x5c\x5csuccnapprox\
\x22,!0),ie(oe,he,b\
e,\x22\x5cu2246\x22,\x22\x5c\x5cnc\
ong\x22,!0),ie(oe,h\
e,be,\x22\x5cue007\x22,\x22\x5c\
\x5c@nshortparallel\
\x22),ie(oe,he,be,\x22\
\x5cu2226\x22,\x22\x5c\x5cnpara\
llel\x22,!0),ie(oe,\
he,be,\x22\x5cu22af\x22,\x22\
\x5c\x5cnVDash\x22,!0),ie\
(oe,he,be,\x22\x5cu22e\
b\x22,\x22\x5c\x5cntriangler\
ight\x22),ie(oe,he,\
be,\x22\x5cu22ed\x22,\x22\x5c\x5cn\
trianglerighteq\x22\
,!0),ie(oe,he,be\
,\x22\x5cue018\x22,\x22\x5c\x5c@ns\
upseteqq\x22),ie(oe\
,he,be,\x22\x5cu228b\x22,\
\x22\x5c\x5csupsetneq\x22,!0\
),ie(oe,he,be,\x22\x5c\
ue01b\x22,\x22\x5c\x5c@varsu\
psetneq\x22),ie(oe,\
he,be,\x22\x5cu2acc\x22,\x22\
\x5c\x5csupsetneqq\x22,!0\
),ie(oe,he,be,\x22\x5c\
ue019\x22,\x22\x5c\x5c@varsu\
psetneqq\x22),ie(oe\
,he,be,\x22\x5cu22ae\x22,\
\x22\x5c\x5cnVdash\x22,!0),i\
e(oe,he,be,\x22\x5cu2a\
b5\x22,\x22\x5c\x5cprecneqq\x22\
,!0),ie(oe,he,be\
,\x22\x5cu2ab6\x22,\x22\x5c\x5csuc\
cneqq\x22,!0),ie(oe\
,he,be,\x22\x5cue016\x22,\
\x22\x5c\x5c@nsubseteqq\x22)\
,ie(oe,he,me,\x22\x5cu\
22b4\x22,\x22\x5c\x5cunlhd\x22)\
,ie(oe,he,me,\x22\x5cu\
22b5\x22,\x22\x5c\x5cunrhd\x22)\
,ie(oe,he,be,\x22\x5cu\
219a\x22,\x22\x5c\x5cnleftar\
row\x22,!0),ie(oe,h\
e,be,\x22\x5cu219b\x22,\x22\x5c\
\x5cnrightarrow\x22,!0\
),ie(oe,he,be,\x22\x5c\
u21cd\x22,\x22\x5c\x5cnLefta\
rrow\x22,!0),ie(oe,\
he,be,\x22\x5cu21cf\x22,\x22\
\x5c\x5cnRightarrow\x22,!\
0),ie(oe,he,be,\x22\
\x5cu21ae\x22,\x22\x5c\x5cnleft\
rightarrow\x22,!0),\
ie(oe,he,be,\x22\x5cu2\
1ce\x22,\x22\x5c\x5cnLeftrig\
htarrow\x22,!0),ie(\
oe,he,be,\x22\x5cu25b3\
\x22,\x22\x5c\x5cvartriangle\
\x22),ie(oe,he,xe,\x22\
\x5cu210f\x22,\x22\x5c\x5chslas\
h\x22),ie(oe,he,xe,\
\x22\x5cu25bd\x22,\x22\x5c\x5ctria\
ngledown\x22),ie(oe\
,he,xe,\x22\x5cu25ca\x22,\
\x22\x5c\x5clozenge\x22),ie(\
oe,he,xe,\x22\x5cu24c8\
\x22,\x22\x5c\x5ccircledS\x22),\
ie(oe,he,xe,\x22\x5cxa\
e\x22,\x22\x5c\x5ccircledR\x22)\
,ie(se,he,xe,\x22\x5cx\
ae\x22,\x22\x5c\x5ccircledR\x22\
),ie(oe,he,xe,\x22\x5c\
u2221\x22,\x22\x5c\x5cmeasur\
edangle\x22,!0),ie(\
oe,he,xe,\x22\x5cu2204\
\x22,\x22\x5c\x5cnexists\x22),i\
e(oe,he,xe,\x22\x5cu21\
27\x22,\x22\x5c\x5cmho\x22),ie(\
oe,he,xe,\x22\x5cu2132\
\x22,\x22\x5c\x5cFinv\x22,!0),i\
e(oe,he,xe,\x22\x5cu21\
41\x22,\x22\x5c\x5cGame\x22,!0)\
,ie(oe,he,xe,\x22\x5cu\
2035\x22,\x22\x5c\x5cbackpri\
me\x22),ie(oe,he,xe\
,\x22\x5cu25b2\x22,\x22\x5c\x5cbla\
cktriangle\x22),ie(\
oe,he,xe,\x22\x5cu25bc\
\x22,\x22\x5c\x5cblacktriang\
ledown\x22),ie(oe,h\
e,xe,\x22\x5cu25a0\x22,\x22\x5c\
\x5cblacksquare\x22),i\
e(oe,he,xe,\x22\x5cu29\
eb\x22,\x22\x5c\x5cblackloze\
nge\x22),ie(oe,he,x\
e,\x22\x5cu2605\x22,\x22\x5c\x5cbi\
gstar\x22),ie(oe,he\
,xe,\x22\x5cu2222\x22,\x22\x5c\x5c\
sphericalangle\x22,\
!0),ie(oe,he,xe,\
\x22\x5cu2201\x22,\x22\x5c\x5ccomp\
lement\x22,!0),ie(o\
e,he,xe,\x22\x5cxf0\x22,\x22\
\x5c\x5ceth\x22,!0),ie(se\
,le,xe,\x22\x5cxf0\x22,\x22\x5c\
xf0\x22),ie(oe,he,x\
e,\x22\x5cu2571\x22,\x22\x5c\x5cdi\
agup\x22),ie(oe,he,\
xe,\x22\x5cu2572\x22,\x22\x5c\x5cd\
iagdown\x22),ie(oe,\
he,xe,\x22\x5cu25a1\x22,\x22\
\x5c\x5csquare\x22),ie(oe\
,he,xe,\x22\x5cu25a1\x22,\
\x22\x5c\x5cBox\x22),ie(oe,h\
e,xe,\x22\x5cu25ca\x22,\x22\x5c\
\x5cDiamond\x22),ie(oe\
,he,xe,\x22\x5cxa5\x22,\x22\x5c\
\x5cyen\x22,!0),ie(se,\
he,xe,\x22\x5cxa5\x22,\x22\x5c\x5c\
yen\x22,!0),ie(oe,h\
e,xe,\x22\x5cu2713\x22,\x22\x5c\
\x5ccheckmark\x22,!0),\
ie(se,he,xe,\x22\x5cu2\
713\x22,\x22\x5c\x5ccheckmar\
k\x22),ie(oe,he,xe,\
\x22\x5cu2136\x22,\x22\x5c\x5cbeth\
\x22,!0),ie(oe,he,x\
e,\x22\x5cu2138\x22,\x22\x5c\x5cda\
leth\x22,!0),ie(oe,\
he,xe,\x22\x5cu2137\x22,\x22\
\x5c\x5cgimel\x22,!0),ie(\
oe,he,xe,\x22\x5cu03dd\
\x22,\x22\x5c\x5cdigamma\x22,!0\
),ie(oe,he,xe,\x22\x5c\
u03f0\x22,\x22\x5c\x5cvarkap\
pa\x22),ie(oe,he,ge\
,\x22\x5cu250c\x22,\x22\x5c\x5c@ul\
corner\x22,!0),ie(o\
e,he,ue,\x22\x5cu2510\x22\
,\x22\x5c\x5c@urcorner\x22,!\
0),ie(oe,he,ge,\x22\
\x5cu2514\x22,\x22\x5c\x5c@llco\
rner\x22,!0),ie(oe,\
he,ue,\x22\x5cu2518\x22,\x22\
\x5c\x5c@lrcorner\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
2266\x22,\x22\x5c\x5cleqq\x22,!\
0),ie(oe,he,be,\x22\
\x5cu2a7d\x22,\x22\x5c\x5cleqsl\
ant\x22,!0),ie(oe,h\
e,be,\x22\x5cu2a95\x22,\x22\x5c\
\x5ceqslantless\x22,!0\
),ie(oe,he,be,\x22\x5c\
u2272\x22,\x22\x5c\x5clesssi\
m\x22,!0),ie(oe,he,\
be,\x22\x5cu2a85\x22,\x22\x5c\x5cl\
essapprox\x22,!0),i\
e(oe,he,be,\x22\x5cu22\
4a\x22,\x22\x5c\x5capproxeq\x22\
,!0),ie(oe,he,me\
,\x22\x5cu22d6\x22,\x22\x5c\x5cles\
sdot\x22),ie(oe,he,\
be,\x22\x5cu22d8\x22,\x22\x5c\x5cl\
ll\x22,!0),ie(oe,he\
,be,\x22\x5cu2276\x22,\x22\x5c\x5c\
lessgtr\x22,!0),ie(\
oe,he,be,\x22\x5cu22da\
\x22,\x22\x5c\x5clesseqgtr\x22,\
!0),ie(oe,he,be,\
\x22\x5cu2a8b\x22,\x22\x5c\x5cless\
eqqgtr\x22,!0),ie(o\
e,he,be,\x22\x5cu2251\x22\
,\x22\x5c\x5cdoteqdot\x22),i\
e(oe,he,be,\x22\x5cu22\
53\x22,\x22\x5c\x5crisingdot\
seq\x22,!0),ie(oe,h\
e,be,\x22\x5cu2252\x22,\x22\x5c\
\x5cfallingdotseq\x22,\
!0),ie(oe,he,be,\
\x22\x5cu223d\x22,\x22\x5c\x5cback\
sim\x22,!0),ie(oe,h\
e,be,\x22\x5cu22cd\x22,\x22\x5c\
\x5cbacksimeq\x22,!0),\
ie(oe,he,be,\x22\x5cu2\
ac5\x22,\x22\x5c\x5csubseteq\
q\x22,!0),ie(oe,he,\
be,\x22\x5cu22d0\x22,\x22\x5c\x5cS\
ubset\x22,!0),ie(oe\
,he,be,\x22\x5cu228f\x22,\
\x22\x5c\x5csqsubset\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
227c\x22,\x22\x5c\x5cpreccur\
lyeq\x22,!0),ie(oe,\
he,be,\x22\x5cu22de\x22,\x22\
\x5c\x5ccurlyeqprec\x22,!\
0),ie(oe,he,be,\x22\
\x5cu227e\x22,\x22\x5c\x5cprecs\
im\x22,!0),ie(oe,he\
,be,\x22\x5cu2ab7\x22,\x22\x5c\x5c\
precapprox\x22,!0),\
ie(oe,he,be,\x22\x5cu2\
2b2\x22,\x22\x5c\x5cvartrian\
gleleft\x22),ie(oe,\
he,be,\x22\x5cu22b4\x22,\x22\
\x5c\x5ctrianglelefteq\
\x22),ie(oe,he,be,\x22\
\x5cu22a8\x22,\x22\x5c\x5cvDash\
\x22,!0),ie(oe,he,b\
e,\x22\x5cu22aa\x22,\x22\x5c\x5cVv\
dash\x22,!0),ie(oe,\
he,be,\x22\x5cu2323\x22,\x22\
\x5c\x5csmallsmile\x22),i\
e(oe,he,be,\x22\x5cu23\
22\x22,\x22\x5c\x5csmallfrow\
n\x22),ie(oe,he,be,\
\x22\x5cu224f\x22,\x22\x5c\x5cbump\
eq\x22,!0),ie(oe,he\
,be,\x22\x5cu224e\x22,\x22\x5c\x5c\
Bumpeq\x22,!0),ie(o\
e,he,be,\x22\x5cu2267\x22\
,\x22\x5c\x5cgeqq\x22,!0),ie\
(oe,he,be,\x22\x5cu2a7\
e\x22,\x22\x5c\x5cgeqslant\x22,\
!0),ie(oe,he,be,\
\x22\x5cu2a96\x22,\x22\x5c\x5ceqsl\
antgtr\x22,!0),ie(o\
e,he,be,\x22\x5cu2273\x22\
,\x22\x5c\x5cgtrsim\x22,!0),\
ie(oe,he,be,\x22\x5cu2\
a86\x22,\x22\x5c\x5cgtrappro\
x\x22,!0),ie(oe,he,\
me,\x22\x5cu22d7\x22,\x22\x5c\x5cg\
trdot\x22),ie(oe,he\
,be,\x22\x5cu22d9\x22,\x22\x5c\x5c\
ggg\x22,!0),ie(oe,h\
e,be,\x22\x5cu2277\x22,\x22\x5c\
\x5cgtrless\x22,!0),ie\
(oe,he,be,\x22\x5cu22d\
b\x22,\x22\x5c\x5cgtreqless\x22\
,!0),ie(oe,he,be\
,\x22\x5cu2a8c\x22,\x22\x5c\x5cgtr\
eqqless\x22,!0),ie(\
oe,he,be,\x22\x5cu2256\
\x22,\x22\x5c\x5ceqcirc\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
2257\x22,\x22\x5c\x5ccirceq\x22\
,!0),ie(oe,he,be\
,\x22\x5cu225c\x22,\x22\x5c\x5ctri\
angleq\x22,!0),ie(o\
e,he,be,\x22\x5cu223c\x22\
,\x22\x5c\x5cthicksim\x22),i\
e(oe,he,be,\x22\x5cu22\
48\x22,\x22\x5c\x5cthickappr\
ox\x22),ie(oe,he,be\
,\x22\x5cu2ac6\x22,\x22\x5c\x5csup\
seteqq\x22,!0),ie(o\
e,he,be,\x22\x5cu22d1\x22\
,\x22\x5c\x5cSupset\x22,!0),\
ie(oe,he,be,\x22\x5cu2\
290\x22,\x22\x5c\x5csqsupset\
\x22,!0),ie(oe,he,b\
e,\x22\x5cu227d\x22,\x22\x5c\x5csu\
cccurlyeq\x22,!0),i\
e(oe,he,be,\x22\x5cu22\
df\x22,\x22\x5c\x5ccurlyeqsu\
cc\x22,!0),ie(oe,he\
,be,\x22\x5cu227f\x22,\x22\x5c\x5c\
succsim\x22,!0),ie(\
oe,he,be,\x22\x5cu2ab8\
\x22,\x22\x5c\x5csuccapprox\x22\
,!0),ie(oe,he,be\
,\x22\x5cu22b3\x22,\x22\x5c\x5cvar\
triangleright\x22),\
ie(oe,he,be,\x22\x5cu2\
2b5\x22,\x22\x5c\x5ctriangle\
righteq\x22),ie(oe,\
he,be,\x22\x5cu22a9\x22,\x22\
\x5c\x5cVdash\x22,!0),ie(\
oe,he,be,\x22\x5cu2223\
\x22,\x22\x5c\x5cshortmid\x22),\
ie(oe,he,be,\x22\x5cu2\
225\x22,\x22\x5c\x5cshortpar\
allel\x22),ie(oe,he\
,be,\x22\x5cu226c\x22,\x22\x5c\x5c\
between\x22,!0),ie(\
oe,he,be,\x22\x5cu22d4\
\x22,\x22\x5c\x5cpitchfork\x22,\
!0),ie(oe,he,be,\
\x22\x5cu221d\x22,\x22\x5c\x5cvarp\
ropto\x22),ie(oe,he\
,be,\x22\x5cu25c0\x22,\x22\x5c\x5c\
blacktrianglelef\
t\x22),ie(oe,he,be,\
\x22\x5cu2234\x22,\x22\x5c\x5cther\
efore\x22,!0),ie(oe\
,he,be,\x22\x5cu220d\x22,\
\x22\x5c\x5cbackepsilon\x22)\
,ie(oe,he,be,\x22\x5cu\
25b6\x22,\x22\x5c\x5cblacktr\
iangleright\x22),ie\
(oe,he,be,\x22\x5cu223\
5\x22,\x22\x5c\x5cbecause\x22,!\
0),ie(oe,he,be,\x22\
\x5cu22d8\x22,\x22\x5c\x5cllles\
s\x22),ie(oe,he,be,\
\x22\x5cu22d9\x22,\x22\x5c\x5cgggt\
r\x22),ie(oe,he,me,\
\x22\x5cu22b2\x22,\x22\x5c\x5clhd\x22\
),ie(oe,he,me,\x22\x5c\
u22b3\x22,\x22\x5c\x5crhd\x22),\
ie(oe,he,be,\x22\x5cu2\
242\x22,\x22\x5c\x5ceqsim\x22,!\
0),ie(oe,le,be,\x22\
\x5cu22c8\x22,\x22\x5c\x5cJoin\x22\
),ie(oe,he,be,\x22\x5c\
u2251\x22,\x22\x5c\x5cDoteq\x22\
,!0),ie(oe,he,me\
,\x22\x5cu2214\x22,\x22\x5c\x5cdot\
plus\x22,!0),ie(oe,\
he,me,\x22\x5cu2216\x22,\x22\
\x5c\x5csmallsetminus\x22\
),ie(oe,he,me,\x22\x5c\
u22d2\x22,\x22\x5c\x5cCap\x22,!\
0),ie(oe,he,me,\x22\
\x5cu22d3\x22,\x22\x5c\x5cCup\x22,\
!0),ie(oe,he,me,\
\x22\x5cu2a5e\x22,\x22\x5c\x5cdoub\
lebarwedge\x22,!0),\
ie(oe,he,me,\x22\x5cu2\
29f\x22,\x22\x5c\x5cboxminus\
\x22,!0),ie(oe,he,m\
e,\x22\x5cu229e\x22,\x22\x5c\x5cbo\
xplus\x22,!0),ie(oe\
,he,me,\x22\x5cu22c7\x22,\
\x22\x5c\x5cdivideontimes\
\x22,!0),ie(oe,he,m\
e,\x22\x5cu22c9\x22,\x22\x5c\x5clt\
imes\x22,!0),ie(oe,\
he,me,\x22\x5cu22ca\x22,\x22\
\x5c\x5crtimes\x22,!0),ie\
(oe,he,me,\x22\x5cu22c\
b\x22,\x22\x5c\x5cleftthreet\
imes\x22,!0),ie(oe,\
he,me,\x22\x5cu22cc\x22,\x22\
\x5c\x5crightthreetime\
s\x22,!0),ie(oe,he,\
me,\x22\x5cu22cf\x22,\x22\x5c\x5cc\
urlywedge\x22,!0),i\
e(oe,he,me,\x22\x5cu22\
ce\x22,\x22\x5c\x5ccurlyvee\x22\
,!0),ie(oe,he,me\
,\x22\x5cu229d\x22,\x22\x5c\x5ccir\
cleddash\x22,!0),ie\
(oe,he,me,\x22\x5cu229\
b\x22,\x22\x5c\x5ccircledast\
\x22,!0),ie(oe,he,m\
e,\x22\x5cu22c5\x22,\x22\x5c\x5cce\
nterdot\x22),ie(oe,\
he,me,\x22\x5cu22ba\x22,\x22\
\x5c\x5cintercal\x22,!0),\
ie(oe,he,me,\x22\x5cu2\
2d2\x22,\x22\x5c\x5cdoubleca\
p\x22),ie(oe,he,me,\
\x22\x5cu22d3\x22,\x22\x5c\x5cdoub\
lecup\x22),ie(oe,he\
,me,\x22\x5cu22a0\x22,\x22\x5c\x5c\
boxtimes\x22,!0),ie\
(oe,he,be,\x22\x5cu21e\
2\x22,\x22\x5c\x5cdashrighta\
rrow\x22,!0),ie(oe,\
he,be,\x22\x5cu21e0\x22,\x22\
\x5c\x5cdashleftarrow\x22\
,!0),ie(oe,he,be\
,\x22\x5cu21c7\x22,\x22\x5c\x5clef\
tleftarrows\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
21c6\x22,\x22\x5c\x5cleftrig\
htarrows\x22,!0),ie\
(oe,he,be,\x22\x5cu21d\
a\x22,\x22\x5c\x5cLleftarrow\
\x22,!0),ie(oe,he,b\
e,\x22\x5cu219e\x22,\x22\x5c\x5ctw\
oheadleftarrow\x22,\
!0),ie(oe,he,be,\
\x22\x5cu21a2\x22,\x22\x5c\x5cleft\
arrowtail\x22,!0),i\
e(oe,he,be,\x22\x5cu21\
ab\x22,\x22\x5c\x5clooparrow\
left\x22,!0),ie(oe,\
he,be,\x22\x5cu21cb\x22,\x22\
\x5c\x5cleftrightharpo\
ons\x22,!0),ie(oe,h\
e,be,\x22\x5cu21b6\x22,\x22\x5c\
\x5ccurvearrowleft\x22\
,!0),ie(oe,he,be\
,\x22\x5cu21ba\x22,\x22\x5c\x5ccir\
clearrowleft\x22,!0\
),ie(oe,he,be,\x22\x5c\
u21b0\x22,\x22\x5c\x5cLsh\x22,!\
0),ie(oe,he,be,\x22\
\x5cu21c8\x22,\x22\x5c\x5cupupa\
rrows\x22,!0),ie(oe\
,he,be,\x22\x5cu21bf\x22,\
\x22\x5c\x5cupharpoonleft\
\x22,!0),ie(oe,he,b\
e,\x22\x5cu21c3\x22,\x22\x5c\x5cdo\
wnharpoonleft\x22,!\
0),ie(oe,le,be,\x22\
\x5cu22b6\x22,\x22\x5c\x5corigo\
f\x22,!0),ie(oe,le,\
be,\x22\x5cu22b7\x22,\x22\x5c\x5ci\
mageof\x22,!0),ie(o\
e,he,be,\x22\x5cu22b8\x22\
,\x22\x5c\x5cmultimap\x22,!0\
),ie(oe,he,be,\x22\x5c\
u21ad\x22,\x22\x5c\x5cleftri\
ghtsquigarrow\x22,!\
0),ie(oe,he,be,\x22\
\x5cu21c9\x22,\x22\x5c\x5cright\
rightarrows\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
21c4\x22,\x22\x5c\x5crightle\
ftarrows\x22,!0),ie\
(oe,he,be,\x22\x5cu21a\
0\x22,\x22\x5c\x5ctwoheadrig\
htarrow\x22,!0),ie(\
oe,he,be,\x22\x5cu21a3\
\x22,\x22\x5c\x5crightarrowt\
ail\x22,!0),ie(oe,h\
e,be,\x22\x5cu21ac\x22,\x22\x5c\
\x5clooparrowright\x22\
,!0),ie(oe,he,be\
,\x22\x5cu21b7\x22,\x22\x5c\x5ccur\
vearrowright\x22,!0\
),ie(oe,he,be,\x22\x5c\
u21bb\x22,\x22\x5c\x5ccircle\
arrowright\x22,!0),\
ie(oe,he,be,\x22\x5cu2\
1b1\x22,\x22\x5c\x5cRsh\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
21ca\x22,\x22\x5c\x5cdowndow\
narrows\x22,!0),ie(\
oe,he,be,\x22\x5cu21be\
\x22,\x22\x5c\x5cupharpoonri\
ght\x22,!0),ie(oe,h\
e,be,\x22\x5cu21c2\x22,\x22\x5c\
\x5cdownharpoonrigh\
t\x22,!0),ie(oe,he,\
be,\x22\x5cu21dd\x22,\x22\x5c\x5cr\
ightsquigarrow\x22,\
!0),ie(oe,he,be,\
\x22\x5cu21dd\x22,\x22\x5c\x5clead\
sto\x22),ie(oe,he,b\
e,\x22\x5cu21db\x22,\x22\x5c\x5cRr\
ightarrow\x22,!0),i\
e(oe,he,be,\x22\x5cu21\
be\x22,\x22\x5c\x5crestricti\
on\x22),ie(oe,le,xe\
,\x22\x5cu2018\x22,\x22`\x22),i\
e(oe,le,xe,\x22$\x22,\x22\
\x5c\x5c$\x22),ie(se,le,x\
e,\x22$\x22,\x22\x5c\x5c$\x22),ie(\
se,le,xe,\x22$\x22,\x22\x5c\x5c\
textdollar\x22),ie(\
oe,le,xe,\x22%\x22,\x22\x5c\x5c\
%\x22),ie(se,le,xe,\
\x22%\x22,\x22\x5c\x5c%\x22),ie(oe\
,le,xe,\x22_\x22,\x22\x5c\x5c_\x22\
),ie(se,le,xe,\x22_\
\x22,\x22\x5c\x5c_\x22),ie(se,l\
e,xe,\x22_\x22,\x22\x5c\x5ctext\
underscore\x22),ie(\
oe,le,xe,\x22\x5cu2220\
\x22,\x22\x5c\x5cangle\x22,!0),\
ie(oe,le,xe,\x22\x5cu2\
21e\x22,\x22\x5c\x5cinfty\x22,!\
0),ie(oe,le,xe,\x22\
\x5cu2032\x22,\x22\x5c\x5cprime\
\x22),ie(oe,le,xe,\x22\
\x5cu25b3\x22,\x22\x5c\x5ctrian\
gle\x22),ie(oe,le,x\
e,\x22\x5cu0393\x22,\x22\x5c\x5cGa\
mma\x22,!0),ie(oe,l\
e,xe,\x22\x5cu0394\x22,\x22\x5c\
\x5cDelta\x22,!0),ie(o\
e,le,xe,\x22\x5cu0398\x22\
,\x22\x5c\x5cTheta\x22,!0),i\
e(oe,le,xe,\x22\x5cu03\
9b\x22,\x22\x5c\x5cLambda\x22,!\
0),ie(oe,le,xe,\x22\
\x5cu039e\x22,\x22\x5c\x5cXi\x22,!\
0),ie(oe,le,xe,\x22\
\x5cu03a0\x22,\x22\x5c\x5cPi\x22,!\
0),ie(oe,le,xe,\x22\
\x5cu03a3\x22,\x22\x5c\x5cSigma\
\x22,!0),ie(oe,le,x\
e,\x22\x5cu03a5\x22,\x22\x5c\x5cUp\
silon\x22,!0),ie(oe\
,le,xe,\x22\x5cu03a6\x22,\
\x22\x5c\x5cPhi\x22,!0),ie(o\
e,le,xe,\x22\x5cu03a8\x22\
,\x22\x5c\x5cPsi\x22,!0),ie(\
oe,le,xe,\x22\x5cu03a9\
\x22,\x22\x5c\x5cOmega\x22,!0),\
ie(oe,le,xe,\x22A\x22,\
\x22\x5cu0391\x22),ie(oe,\
le,xe,\x22B\x22,\x22\x5cu039\
2\x22),ie(oe,le,xe,\
\x22E\x22,\x22\x5cu0395\x22),ie\
(oe,le,xe,\x22Z\x22,\x22\x5c\
u0396\x22),ie(oe,le\
,xe,\x22H\x22,\x22\x5cu0397\x22\
),ie(oe,le,xe,\x22I\
\x22,\x22\x5cu0399\x22),ie(o\
e,le,xe,\x22K\x22,\x22\x5cu0\
39a\x22),ie(oe,le,x\
e,\x22M\x22,\x22\x5cu039c\x22),\
ie(oe,le,xe,\x22N\x22,\
\x22\x5cu039d\x22),ie(oe,\
le,xe,\x22O\x22,\x22\x5cu039\
f\x22),ie(oe,le,xe,\
\x22P\x22,\x22\x5cu03a1\x22),ie\
(oe,le,xe,\x22T\x22,\x22\x5c\
u03a4\x22),ie(oe,le\
,xe,\x22X\x22,\x22\x5cu03a7\x22\
),ie(oe,le,xe,\x22\x5c\
xac\x22,\x22\x5c\x5cneg\x22,!0)\
,ie(oe,le,xe,\x22\x5cx\
ac\x22,\x22\x5c\x5clnot\x22),ie\
(oe,le,xe,\x22\x5cu22a\
4\x22,\x22\x5c\x5ctop\x22),ie(o\
e,le,xe,\x22\x5cu22a5\x22\
,\x22\x5c\x5cbot\x22),ie(oe,\
le,xe,\x22\x5cu2205\x22,\x22\
\x5c\x5cemptyset\x22),ie(\
oe,he,xe,\x22\x5cu2205\
\x22,\x22\x5c\x5cvarnothing\x22\
),ie(oe,le,de,\x22\x5c\
u03b1\x22,\x22\x5c\x5calpha\x22\
,!0),ie(oe,le,de\
,\x22\x5cu03b2\x22,\x22\x5c\x5cbet\
a\x22,!0),ie(oe,le,\
de,\x22\x5cu03b3\x22,\x22\x5c\x5cg\
amma\x22,!0),ie(oe,\
le,de,\x22\x5cu03b4\x22,\x22\
\x5c\x5cdelta\x22,!0),ie(\
oe,le,de,\x22\x5cu03f5\
\x22,\x22\x5c\x5cepsilon\x22,!0\
),ie(oe,le,de,\x22\x5c\
u03b6\x22,\x22\x5c\x5czeta\x22,\
!0),ie(oe,le,de,\
\x22\x5cu03b7\x22,\x22\x5c\x5ceta\x22\
,!0),ie(oe,le,de\
,\x22\x5cu03b8\x22,\x22\x5c\x5cthe\
ta\x22,!0),ie(oe,le\
,de,\x22\x5cu03b9\x22,\x22\x5c\x5c\
iota\x22,!0),ie(oe,\
le,de,\x22\x5cu03ba\x22,\x22\
\x5c\x5ckappa\x22,!0),ie(\
oe,le,de,\x22\x5cu03bb\
\x22,\x22\x5c\x5clambda\x22,!0)\
,ie(oe,le,de,\x22\x5cu\
03bc\x22,\x22\x5c\x5cmu\x22,!0)\
,ie(oe,le,de,\x22\x5cu\
03bd\x22,\x22\x5c\x5cnu\x22,!0)\
,ie(oe,le,de,\x22\x5cu\
03be\x22,\x22\x5c\x5cxi\x22,!0)\
,ie(oe,le,de,\x22\x5cu\
03bf\x22,\x22\x5c\x5comicron\
\x22,!0),ie(oe,le,d\
e,\x22\x5cu03c0\x22,\x22\x5c\x5cpi\
\x22,!0),ie(oe,le,d\
e,\x22\x5cu03c1\x22,\x22\x5c\x5crh\
o\x22,!0),ie(oe,le,\
de,\x22\x5cu03c3\x22,\x22\x5c\x5cs\
igma\x22,!0),ie(oe,\
le,de,\x22\x5cu03c4\x22,\x22\
\x5c\x5ctau\x22,!0),ie(oe\
,le,de,\x22\x5cu03c5\x22,\
\x22\x5c\x5cupsilon\x22,!0),\
ie(oe,le,de,\x22\x5cu0\
3d5\x22,\x22\x5c\x5cphi\x22,!0)\
,ie(oe,le,de,\x22\x5cu\
03c7\x22,\x22\x5c\x5cchi\x22,!0\
),ie(oe,le,de,\x22\x5c\
u03c8\x22,\x22\x5c\x5cpsi\x22,!\
0),ie(oe,le,de,\x22\
\x5cu03c9\x22,\x22\x5c\x5comega\
\x22,!0),ie(oe,le,d\
e,\x22\x5cu03b5\x22,\x22\x5c\x5cva\
repsilon\x22,!0),ie\
(oe,le,de,\x22\x5cu03d\
1\x22,\x22\x5c\x5cvartheta\x22,\
!0),ie(oe,le,de,\
\x22\x5cu03d6\x22,\x22\x5c\x5cvarp\
i\x22,!0),ie(oe,le,\
de,\x22\x5cu03f1\x22,\x22\x5c\x5cv\
arrho\x22,!0),ie(oe\
,le,de,\x22\x5cu03c2\x22,\
\x22\x5c\x5cvarsigma\x22,!0)\
,ie(oe,le,de,\x22\x5cu\
03c6\x22,\x22\x5c\x5cvarphi\x22\
,!0),ie(oe,le,me\
,\x22\x5cu2217\x22,\x22*\x22,!0\
),ie(oe,le,me,\x22+\
\x22,\x22+\x22),ie(oe,le,\
me,\x22\x5cu2212\x22,\x22-\x22,\
!0),ie(oe,le,me,\
\x22\x5cu22c5\x22,\x22\x5c\x5ccdot\
\x22,!0),ie(oe,le,m\
e,\x22\x5cu2218\x22,\x22\x5c\x5cci\
rc\x22,!0),ie(oe,le\
,me,\x22\x5cxf7\x22,\x22\x5c\x5cdi\
v\x22,!0),ie(oe,le,\
me,\x22\x5cxb1\x22,\x22\x5c\x5cpm\x22\
,!0),ie(oe,le,me\
,\x22\x5cxd7\x22,\x22\x5c\x5ctimes\
\x22,!0),ie(oe,le,m\
e,\x22\x5cu2229\x22,\x22\x5c\x5cca\
p\x22,!0),ie(oe,le,\
me,\x22\x5cu222a\x22,\x22\x5c\x5cc\
up\x22,!0),ie(oe,le\
,me,\x22\x5cu2216\x22,\x22\x5c\x5c\
setminus\x22,!0),ie\
(oe,le,me,\x22\x5cu222\
7\x22,\x22\x5c\x5cland\x22),ie(\
oe,le,me,\x22\x5cu2228\
\x22,\x22\x5c\x5clor\x22),ie(oe\
,le,me,\x22\x5cu2227\x22,\
\x22\x5c\x5cwedge\x22,!0),ie\
(oe,le,me,\x22\x5cu222\
8\x22,\x22\x5c\x5cvee\x22,!0),i\
e(oe,le,xe,\x22\x5cu22\
1a\x22,\x22\x5c\x5csurd\x22),ie\
(oe,le,ge,\x22\x5cu27e\
8\x22,\x22\x5c\x5clangle\x22,!0\
),ie(oe,le,ge,\x22\x5c\
u2223\x22,\x22\x5c\x5clvert\x22\
),ie(oe,le,ge,\x22\x5c\
u2225\x22,\x22\x5c\x5clVert\x22\
),ie(oe,le,ue,\x22?\
\x22,\x22?\x22),ie(oe,le,\
ue,\x22!\x22,\x22!\x22),ie(o\
e,le,ue,\x22\x5cu27e9\x22\
,\x22\x5c\x5crangle\x22,!0),\
ie(oe,le,ue,\x22\x5cu2\
223\x22,\x22\x5c\x5crvert\x22),\
ie(oe,le,ue,\x22\x5cu2\
225\x22,\x22\x5c\x5crVert\x22),\
ie(oe,le,be,\x22=\x22,\
\x22=\x22),ie(oe,le,be\
,\x22:\x22,\x22:\x22),ie(oe,\
le,be,\x22\x5cu2248\x22,\x22\
\x5c\x5capprox\x22,!0),ie\
(oe,le,be,\x22\x5cu224\
5\x22,\x22\x5c\x5ccong\x22,!0),\
ie(oe,le,be,\x22\x5cu2\
265\x22,\x22\x5c\x5cge\x22),ie(\
oe,le,be,\x22\x5cu2265\
\x22,\x22\x5c\x5cgeq\x22,!0),ie\
(oe,le,be,\x22\x5cu219\
0\x22,\x22\x5c\x5cgets\x22),ie(\
oe,le,be,\x22>\x22,\x22\x5c\x5c\
gt\x22,!0),ie(oe,le\
,be,\x22\x5cu2208\x22,\x22\x5c\x5c\
in\x22,!0),ie(oe,le\
,be,\x22\x5cue020\x22,\x22\x5c\x5c\
@not\x22),ie(oe,le,\
be,\x22\x5cu2282\x22,\x22\x5c\x5cs\
ubset\x22,!0),ie(oe\
,le,be,\x22\x5cu2283\x22,\
\x22\x5c\x5csupset\x22,!0),i\
e(oe,le,be,\x22\x5cu22\
86\x22,\x22\x5c\x5csubseteq\x22\
,!0),ie(oe,le,be\
,\x22\x5cu2287\x22,\x22\x5c\x5csup\
seteq\x22,!0),ie(oe\
,he,be,\x22\x5cu2288\x22,\
\x22\x5c\x5cnsubseteq\x22,!0\
),ie(oe,he,be,\x22\x5c\
u2289\x22,\x22\x5c\x5cnsupse\
teq\x22,!0),ie(oe,l\
e,be,\x22\x5cu22a8\x22,\x22\x5c\
\x5cmodels\x22),ie(oe,\
le,be,\x22\x5cu2190\x22,\x22\
\x5c\x5cleftarrow\x22,!0)\
,ie(oe,le,be,\x22\x5cu\
2264\x22,\x22\x5c\x5cle\x22),ie\
(oe,le,be,\x22\x5cu226\
4\x22,\x22\x5c\x5cleq\x22,!0),i\
e(oe,le,be,\x22<\x22,\x22\
\x5c\x5clt\x22,!0),ie(oe,\
le,be,\x22\x5cu2192\x22,\x22\
\x5c\x5crightarrow\x22,!0\
),ie(oe,le,be,\x22\x5c\
u2192\x22,\x22\x5c\x5cto\x22),i\
e(oe,he,be,\x22\x5cu22\
71\x22,\x22\x5c\x5cngeq\x22,!0)\
,ie(oe,he,be,\x22\x5cu\
2270\x22,\x22\x5c\x5cnleq\x22,!\
0),ie(oe,le,ye,\x22\
\x5cxa0\x22,\x22\x5c\x5c \x22),ie(\
oe,le,ye,\x22\x5cxa0\x22,\
\x22\x5c\x5cspace\x22),ie(oe\
,le,ye,\x22\x5cxa0\x22,\x22\x5c\
\x5cnobreakspace\x22),\
ie(se,le,ye,\x22\x5cxa\
0\x22,\x22\x5c\x5c \x22),ie(se,\
le,ye,\x22\x5cxa0\x22,\x22 \x22\
),ie(se,le,ye,\x22\x5c\
xa0\x22,\x22\x5c\x5cspace\x22),\
ie(se,le,ye,\x22\x5cxa\
0\x22,\x22\x5c\x5cnobreakspa\
ce\x22),ie(oe,le,ye\
,null,\x22\x5c\x5cnobreak\
\x22),ie(oe,le,ye,n\
ull,\x22\x5c\x5callowbrea\
k\x22),ie(oe,le,ve,\
\x22,\x22,\x22,\x22),ie(oe,l\
e,ve,\x22;\x22,\x22;\x22),ie\
(oe,he,me,\x22\x5cu22b\
c\x22,\x22\x5c\x5cbarwedge\x22,\
!0),ie(oe,he,me,\
\x22\x5cu22bb\x22,\x22\x5c\x5cveeb\
ar\x22,!0),ie(oe,le\
,me,\x22\x5cu2299\x22,\x22\x5c\x5c\
odot\x22,!0),ie(oe,\
le,me,\x22\x5cu2295\x22,\x22\
\x5c\x5coplus\x22,!0),ie(\
oe,le,me,\x22\x5cu2297\
\x22,\x22\x5c\x5cotimes\x22,!0)\
,ie(oe,le,xe,\x22\x5cu\
2202\x22,\x22\x5c\x5cpartial\
\x22,!0),ie(oe,le,m\
e,\x22\x5cu2298\x22,\x22\x5c\x5cos\
lash\x22,!0),ie(oe,\
he,me,\x22\x5cu229a\x22,\x22\
\x5c\x5ccircledcirc\x22,!\
0),ie(oe,he,me,\x22\
\x5cu22a1\x22,\x22\x5c\x5cboxdo\
t\x22,!0),ie(oe,le,\
me,\x22\x5cu25b3\x22,\x22\x5c\x5cb\
igtriangleup\x22),i\
e(oe,le,me,\x22\x5cu25\
bd\x22,\x22\x5c\x5cbigtriang\
ledown\x22),ie(oe,l\
e,me,\x22\x5cu2020\x22,\x22\x5c\
\x5cdagger\x22),ie(oe,\
le,me,\x22\x5cu22c4\x22,\x22\
\x5c\x5cdiamond\x22),ie(o\
e,le,me,\x22\x5cu22c6\x22\
,\x22\x5c\x5cstar\x22),ie(oe\
,le,me,\x22\x5cu25c3\x22,\
\x22\x5c\x5ctriangleleft\x22\
),ie(oe,le,me,\x22\x5c\
u25b9\x22,\x22\x5c\x5ctriang\
leright\x22),ie(oe,\
le,ge,\x22{\x22,\x22\x5c\x5c{\x22)\
,ie(se,le,xe,\x22{\x22\
,\x22\x5c\x5c{\x22),ie(se,le\
,xe,\x22{\x22,\x22\x5c\x5ctextb\
raceleft\x22),ie(oe\
,le,ue,\x22}\x22,\x22\x5c\x5c}\x22\
),ie(se,le,xe,\x22}\
\x22,\x22\x5c\x5c}\x22),ie(se,l\
e,xe,\x22}\x22,\x22\x5c\x5ctext\
braceright\x22),ie(\
oe,le,ge,\x22{\x22,\x22\x5c\x5c\
lbrace\x22),ie(oe,l\
e,ue,\x22}\x22,\x22\x5c\x5crbra\
ce\x22),ie(oe,le,ge\
,\x22[\x22,\x22\x5c\x5clbrack\x22,\
!0),ie(se,le,xe,\
\x22[\x22,\x22\x5c\x5clbrack\x22,!\
0),ie(oe,le,ue,\x22\
]\x22,\x22\x5c\x5crbrack\x22,!0\
),ie(se,le,xe,\x22]\
\x22,\x22\x5c\x5crbrack\x22,!0)\
,ie(oe,le,ge,\x22(\x22\
,\x22\x5c\x5clparen\x22,!0),\
ie(oe,le,ue,\x22)\x22,\
\x22\x5c\x5crparen\x22,!0),i\
e(se,le,xe,\x22<\x22,\x22\
\x5c\x5ctextless\x22,!0),\
ie(se,le,xe,\x22>\x22,\
\x22\x5c\x5ctextgreater\x22,\
!0),ie(oe,le,ge,\
\x22\x5cu230a\x22,\x22\x5c\x5clflo\
or\x22,!0),ie(oe,le\
,ue,\x22\x5cu230b\x22,\x22\x5c\x5c\
rfloor\x22,!0),ie(o\
e,le,ge,\x22\x5cu2308\x22\
,\x22\x5c\x5clceil\x22,!0),i\
e(oe,le,ue,\x22\x5cu23\
09\x22,\x22\x5c\x5crceil\x22,!0\
),ie(oe,le,xe,\x22\x5c\
\x5c\x22,\x22\x5c\x5cbackslash\x22\
),ie(oe,le,xe,\x22\x5c\
u2223\x22,\x22|\x22),ie(o\
e,le,xe,\x22\x5cu2223\x22\
,\x22\x5c\x5cvert\x22),ie(se\
,le,xe,\x22|\x22,\x22\x5c\x5cte\
xtbar\x22,!0),ie(oe\
,le,xe,\x22\x5cu2225\x22,\
\x22\x5c\x5c|\x22),ie(oe,le,\
xe,\x22\x5cu2225\x22,\x22\x5c\x5cV\
ert\x22),ie(se,le,x\
e,\x22\x5cu2225\x22,\x22\x5c\x5cte\
xtbardbl\x22),ie(se\
,le,xe,\x22~\x22,\x22\x5c\x5cte\
xtasciitilde\x22),i\
e(se,le,xe,\x22\x5c\x5c\x22,\
\x22\x5c\x5ctextbackslash\
\x22),ie(se,le,xe,\x22\
^\x22,\x22\x5c\x5ctextasciic\
ircum\x22),ie(oe,le\
,be,\x22\x5cu2191\x22,\x22\x5c\x5c\
uparrow\x22,!0),ie(\
oe,le,be,\x22\x5cu21d1\
\x22,\x22\x5c\x5cUparrow\x22,!0\
),ie(oe,le,be,\x22\x5c\
u2193\x22,\x22\x5c\x5cdownar\
row\x22,!0),ie(oe,l\
e,be,\x22\x5cu21d3\x22,\x22\x5c\
\x5cDownarrow\x22,!0),\
ie(oe,le,be,\x22\x5cu2\
195\x22,\x22\x5c\x5cupdownar\
row\x22,!0),ie(oe,l\
e,be,\x22\x5cu21d5\x22,\x22\x5c\
\x5cUpdownarrow\x22,!0\
),ie(oe,le,fe,\x22\x5c\
u2210\x22,\x22\x5c\x5ccoprod\
\x22),ie(oe,le,fe,\x22\
\x5cu22c1\x22,\x22\x5c\x5cbigve\
e\x22),ie(oe,le,fe,\
\x22\x5cu22c0\x22,\x22\x5c\x5cbigw\
edge\x22),ie(oe,le,\
fe,\x22\x5cu2a04\x22,\x22\x5c\x5cb\
iguplus\x22),ie(oe,\
le,fe,\x22\x5cu22c2\x22,\x22\
\x5c\x5cbigcap\x22),ie(oe\
,le,fe,\x22\x5cu22c3\x22,\
\x22\x5c\x5cbigcup\x22),ie(o\
e,le,fe,\x22\x5cu222b\x22\
,\x22\x5c\x5cint\x22),ie(oe,\
le,fe,\x22\x5cu222b\x22,\x22\
\x5c\x5cintop\x22),ie(oe,\
le,fe,\x22\x5cu222c\x22,\x22\
\x5c\x5ciint\x22),ie(oe,l\
e,fe,\x22\x5cu222d\x22,\x22\x5c\
\x5ciiint\x22),ie(oe,l\
e,fe,\x22\x5cu220f\x22,\x22\x5c\
\x5cprod\x22),ie(oe,le\
,fe,\x22\x5cu2211\x22,\x22\x5c\x5c\
sum\x22),ie(oe,le,f\
e,\x22\x5cu2a02\x22,\x22\x5c\x5cbi\
gotimes\x22),ie(oe,\
le,fe,\x22\x5cu2a01\x22,\x22\
\x5c\x5cbigoplus\x22),ie(\
oe,le,fe,\x22\x5cu2a00\
\x22,\x22\x5c\x5cbigodot\x22),i\
e(oe,le,fe,\x22\x5cu22\
2e\x22,\x22\x5c\x5coint\x22),ie\
(oe,le,fe,\x22\x5cu222\
f\x22,\x22\x5c\x5coiint\x22),ie\
(oe,le,fe,\x22\x5cu223\
0\x22,\x22\x5c\x5coiiint\x22),i\
e(oe,le,fe,\x22\x5cu2a\
06\x22,\x22\x5c\x5cbigsqcup\x22\
),ie(oe,le,fe,\x22\x5c\
u222b\x22,\x22\x5c\x5csmalli\
nt\x22),ie(se,le,pe\
,\x22\x5cu2026\x22,\x22\x5c\x5ctex\
tellipsis\x22),ie(o\
e,le,pe,\x22\x5cu2026\x22\
,\x22\x5c\x5cmathellipsis\
\x22),ie(se,le,pe,\x22\
\x5cu2026\x22,\x22\x5c\x5cldots\
\x22,!0),ie(oe,le,p\
e,\x22\x5cu2026\x22,\x22\x5c\x5cld\
ots\x22,!0),ie(oe,l\
e,pe,\x22\x5cu22ef\x22,\x22\x5c\
\x5c@cdots\x22,!0),ie(\
oe,le,pe,\x22\x5cu22f1\
\x22,\x22\x5c\x5cddots\x22,!0),\
ie(oe,le,xe,\x22\x5cu2\
2ee\x22,\x22\x5c\x5cvarvdots\
\x22),ie(oe,le,ce,\x22\
\x5cu02ca\x22,\x22\x5c\x5cacute\
\x22),ie(oe,le,ce,\x22\
\x5cu02cb\x22,\x22\x5c\x5cgrave\
\x22),ie(oe,le,ce,\x22\
\x5cxa8\x22,\x22\x5c\x5cddot\x22),\
ie(oe,le,ce,\x22~\x22,\
\x22\x5c\x5ctilde\x22),ie(oe\
,le,ce,\x22\x5cu02c9\x22,\
\x22\x5c\x5cbar\x22),ie(oe,l\
e,ce,\x22\x5cu02d8\x22,\x22\x5c\
\x5cbreve\x22),ie(oe,l\
e,ce,\x22\x5cu02c7\x22,\x22\x5c\
\x5ccheck\x22),ie(oe,l\
e,ce,\x22^\x22,\x22\x5c\x5chat\x22\
),ie(oe,le,ce,\x22\x5c\
u20d7\x22,\x22\x5c\x5cvec\x22),\
ie(oe,le,ce,\x22\x5cu0\
2d9\x22,\x22\x5c\x5cdot\x22),ie\
(oe,le,ce,\x22\x5cu02d\
a\x22,\x22\x5c\x5cmathring\x22)\
,ie(oe,le,de,\x22\x5cu\
e131\x22,\x22\x5c\x5c@imath\x22\
),ie(oe,le,de,\x22\x5c\
ue237\x22,\x22\x5c\x5c@jmath\
\x22),ie(oe,le,xe,\x22\
\x5cu0131\x22,\x22\x5cu0131\x22\
),ie(oe,le,xe,\x22\x5c\
u0237\x22,\x22\x5cu0237\x22)\
,ie(se,le,xe,\x22\x5cu\
0131\x22,\x22\x5c\x5ci\x22,!0),\
ie(se,le,xe,\x22\x5cu0\
237\x22,\x22\x5c\x5cj\x22,!0),i\
e(se,le,xe,\x22\x5cxdf\
\x22,\x22\x5c\x5css\x22,!0),ie(\
se,le,xe,\x22\x5cxe6\x22,\
\x22\x5c\x5cae\x22,!0),ie(se\
,le,xe,\x22\x5cu0153\x22,\
\x22\x5c\x5coe\x22,!0),ie(se\
,le,xe,\x22\x5cxf8\x22,\x22\x5c\
\x5co\x22,!0),ie(se,le\
,xe,\x22\x5cxc6\x22,\x22\x5c\x5cAE\
\x22,!0),ie(se,le,x\
e,\x22\x5cu0152\x22,\x22\x5c\x5cOE\
\x22,!0),ie(se,le,x\
e,\x22\x5cxd8\x22,\x22\x5c\x5cO\x22,!\
0),ie(se,le,ce,\x22\
\x5cu02ca\x22,\x22\x5c\x5c'\x22),i\
e(se,le,ce,\x22\x5cu02\
cb\x22,\x22\x5c\x5c`\x22),ie(se\
,le,ce,\x22\x5cu02c6\x22,\
\x22\x5c\x5c^\x22),ie(se,le,\
ce,\x22\x5cu02dc\x22,\x22\x5c\x5c~\
\x22),ie(se,le,ce,\x22\
\x5cu02c9\x22,\x22\x5c\x5c=\x22),i\
e(se,le,ce,\x22\x5cu02\
d8\x22,\x22\x5c\x5cu\x22),ie(se\
,le,ce,\x22\x5cu02d9\x22,\
\x22\x5c\x5c.\x22),ie(se,le,\
ce,\x22\x5cxb8\x22,\x22\x5c\x5cc\x22)\
,ie(se,le,ce,\x22\x5cu\
02da\x22,\x22\x5c\x5cr\x22),ie(\
se,le,ce,\x22\x5cu02c7\
\x22,\x22\x5c\x5cv\x22),ie(se,l\
e,ce,\x22\x5cxa8\x22,'\x5c\x5c\x22\
'),ie(se,le,ce,\x22\
\x5cu02dd\x22,\x22\x5c\x5cH\x22),i\
e(se,le,ce,\x22\x5cu25\
ef\x22,\x22\x5c\x5ctextcircl\
ed\x22);var we={\x22--\
\x22:!0,\x22---\x22:!0,\x22`\
`\x22:!0,\x22''\x22:!0};i\
e(se,le,xe,\x22\x5cu20\
13\x22,\x22--\x22,!0),ie(\
se,le,xe,\x22\x5cu2013\
\x22,\x22\x5c\x5ctextendash\x22\
),ie(se,le,xe,\x22\x5c\
u2014\x22,\x22---\x22,!0)\
,ie(se,le,xe,\x22\x5cu\
2014\x22,\x22\x5c\x5ctextemd\
ash\x22),ie(se,le,x\
e,\x22\x5cu2018\x22,\x22`\x22,!\
0),ie(se,le,xe,\x22\
\x5cu2018\x22,\x22\x5c\x5ctextq\
uoteleft\x22),ie(se\
,le,xe,\x22\x5cu2019\x22,\
\x22'\x22,!0),ie(se,le\
,xe,\x22\x5cu2019\x22,\x22\x5c\x5c\
textquoteright\x22)\
,ie(se,le,xe,\x22\x5cu\
201c\x22,\x22``\x22,!0),i\
e(se,le,xe,\x22\x5cu20\
1c\x22,\x22\x5c\x5ctextquote\
dblleft\x22),ie(se,\
le,xe,\x22\x5cu201d\x22,\x22\
''\x22,!0),ie(se,le\
,xe,\x22\x5cu201d\x22,\x22\x5c\x5c\
textquotedblrigh\
t\x22),ie(oe,le,xe,\
\x22\x5cxb0\x22,\x22\x5c\x5cdegree\
\x22,!0),ie(se,le,x\
e,\x22\x5cxb0\x22,\x22\x5c\x5cdegr\
ee\x22),ie(se,le,xe\
,\x22\x5cxb0\x22,\x22\x5c\x5ctextd\
egree\x22,!0),ie(oe\
,le,xe,\x22\x5cxa3\x22,\x22\x5c\
\x5cpounds\x22),ie(oe,\
le,xe,\x22\x5cxa3\x22,\x22\x5c\x5c\
mathsterling\x22,!0\
),ie(se,le,xe,\x22\x5c\
xa3\x22,\x22\x5c\x5cpounds\x22)\
,ie(se,le,xe,\x22\x5cx\
a3\x22,\x22\x5c\x5ctextsterl\
ing\x22,!0),ie(oe,h\
e,xe,\x22\x5cu2720\x22,\x22\x5c\
\x5cmaltese\x22),ie(se\
,he,xe,\x22\x5cu2720\x22,\
\x22\x5c\x5cmaltese\x22);for\
(var ke='0123456\
789/@.\x22',Se=0;Se\
<ke.length;Se++)\
{var Me=ke.charA\
t(Se);ie(oe,le,x\
e,Me,Me)}for(var\
 ze='0123456789!\
@*()-=+\x22;:?/.,',\
Ae=0;Ae<ze.lengt\
h;Ae++){var Te=z\
e.charAt(Ae);ie(\
se,le,xe,Te,Te)}\
for(var Be=\x22ABCD\
EFGHIJKLMNOPQRST\
UVWXYZabcdefghij\
klmnopqrstuvwxyz\
\x22,Ce=0;Ce<Be.len\
gth;Ce++){var Ne\
=Be.charAt(Ce);i\
e(oe,le,de,Ne,Ne\
),ie(se,le,xe,Ne\
,Ne)}ie(oe,he,xe\
,\x22C\x22,\x22\x5cu2102\x22),i\
e(se,he,xe,\x22C\x22,\x22\
\x5cu2102\x22),ie(oe,h\
e,xe,\x22H\x22,\x22\x5cu210d\
\x22),ie(se,he,xe,\x22\
H\x22,\x22\x5cu210d\x22),ie(\
oe,he,xe,\x22N\x22,\x22\x5cu\
2115\x22),ie(se,he,\
xe,\x22N\x22,\x22\x5cu2115\x22)\
,ie(oe,he,xe,\x22P\x22\
,\x22\x5cu2119\x22),ie(se\
,he,xe,\x22P\x22,\x22\x5cu21\
19\x22),ie(oe,he,xe\
,\x22Q\x22,\x22\x5cu211a\x22),i\
e(se,he,xe,\x22Q\x22,\x22\
\x5cu211a\x22),ie(oe,h\
e,xe,\x22R\x22,\x22\x5cu211d\
\x22),ie(se,he,xe,\x22\
R\x22,\x22\x5cu211d\x22),ie(\
oe,he,xe,\x22Z\x22,\x22\x5cu\
2124\x22),ie(se,he,\
xe,\x22Z\x22,\x22\x5cu2124\x22)\
,ie(oe,le,de,\x22h\x22\
,\x22\x5cu210e\x22),ie(se\
,le,de,\x22h\x22,\x22\x5cu21\
0e\x22);for(var qe=\
\x22\x22,Ie=0;Ie<Be.le\
ngth;Ie++){var R\
e=Be.charAt(Ie);\
ie(oe,le,de,Re,q\
e=String.fromCha\
rCode(55349,5632\
0+Ie)),ie(se,le,\
xe,Re,qe),ie(oe,\
le,de,Re,qe=Stri\
ng.fromCharCode(\
55349,56372+Ie))\
,ie(se,le,xe,Re,\
qe),ie(oe,le,de,\
Re,qe=String.fro\
mCharCode(55349,\
56424+Ie)),ie(se\
,le,xe,Re,qe),ie\
(oe,le,de,Re,qe=\
String.fromCharC\
ode(55349,56580+\
Ie)),ie(se,le,xe\
,Re,qe),ie(oe,le\
,de,Re,qe=String\
.fromCharCode(55\
349,56736+Ie)),i\
e(se,le,xe,Re,qe\
),ie(oe,le,de,Re\
,qe=String.fromC\
harCode(55349,56\
788+Ie)),ie(se,l\
e,xe,Re,qe),ie(o\
e,le,de,Re,qe=St\
ring.fromCharCod\
e(55349,56840+Ie\
)),ie(se,le,xe,R\
e,qe),ie(oe,le,d\
e,Re,qe=String.f\
romCharCode(5534\
9,56944+Ie)),ie(\
se,le,xe,Re,qe),\
Ie<26&&(ie(oe,le\
,de,Re,qe=String\
.fromCharCode(55\
349,56632+Ie)),i\
e(se,le,xe,Re,qe\
),ie(oe,le,de,Re\
,qe=String.fromC\
harCode(55349,56\
476+Ie)),ie(se,l\
e,xe,Re,qe))}ie(\
oe,le,de,\x22k\x22,qe=\
String.fromCharC\
ode(55349,56668)\
),ie(se,le,xe,\x22k\
\x22,qe);for(var He\
=0;He<10;He++){v\
ar Oe=He.toStrin\
g();ie(oe,le,de,\
Oe,qe=String.fro\
mCharCode(55349,\
57294+He)),ie(se\
,le,xe,Oe,qe),ie\
(oe,le,de,Oe,qe=\
String.fromCharC\
ode(55349,57314+\
He)),ie(se,le,xe\
,Oe,qe),ie(oe,le\
,de,Oe,qe=String\
.fromCharCode(55\
349,57324+He)),i\
e(se,le,xe,Oe,qe\
),ie(oe,le,de,Oe\
,qe=String.fromC\
harCode(55349,57\
334+He)),ie(se,l\
e,xe,Oe,qe)}for(\
var Ee=\x22\x5cxd0\x5cxde\
\x5cxfe\x22,Le=0;Le<Ee\
.length;Le++){va\
r De=Ee.charAt(L\
e);ie(oe,le,de,D\
e,De),ie(se,le,x\
e,De,De)}var Ve=\
[[\x22mathbf\x22,\x22text\
bf\x22,\x22Main-Bold\x22]\
,[\x22mathbf\x22,\x22text\
bf\x22,\x22Main-Bold\x22]\
,[\x22mathnormal\x22,\x22\
textit\x22,\x22Math-It\
alic\x22],[\x22mathnor\
mal\x22,\x22textit\x22,\x22M\
ath-Italic\x22],[\x22b\
oldsymbol\x22,\x22bold\
symbol\x22,\x22Main-Bo\
ldItalic\x22],[\x22bol\
dsymbol\x22,\x22boldsy\
mbol\x22,\x22Main-Bold\
Italic\x22],[\x22maths\
cr\x22,\x22textscr\x22,\x22S\
cript-Regular\x22],\
[\x22\x22,\x22\x22,\x22\x22],[\x22\x22,\x22\
\x22,\x22\x22],[\x22\x22,\x22\x22,\x22\x22]\
,[\x22mathfrak\x22,\x22te\
xtfrak\x22,\x22Fraktur\
-Regular\x22],[\x22mat\
hfrak\x22,\x22textfrak\
\x22,\x22Fraktur-Regul\
ar\x22],[\x22mathbb\x22,\x22\
textbb\x22,\x22AMS-Reg\
ular\x22],[\x22mathbb\x22\
,\x22textbb\x22,\x22AMS-R\
egular\x22],[\x22\x22,\x22\x22,\
\x22\x22],[\x22\x22,\x22\x22,\x22\x22],[\
\x22mathsf\x22,\x22textsf\
\x22,\x22SansSerif-Reg\
ular\x22],[\x22mathsf\x22\
,\x22textsf\x22,\x22SansS\
erif-Regular\x22],[\
\x22mathboldsf\x22,\x22te\
xtboldsf\x22,\x22SansS\
erif-Bold\x22],[\x22ma\
thboldsf\x22,\x22textb\
oldsf\x22,\x22SansSeri\
f-Bold\x22],[\x22mathi\
tsf\x22,\x22textitsf\x22,\
\x22SansSerif-Itali\
c\x22],[\x22mathitsf\x22,\
\x22textitsf\x22,\x22Sans\
Serif-Italic\x22],[\
\x22\x22,\x22\x22,\x22\x22],[\x22\x22,\x22\x22\
,\x22\x22],[\x22mathtt\x22,\x22\
texttt\x22,\x22Typewri\
ter-Regular\x22],[\x22\
mathtt\x22,\x22texttt\x22\
,\x22Typewriter-Reg\
ular\x22]],Pe=[[\x22ma\
thbf\x22,\x22textbf\x22,\x22\
Main-Bold\x22],[\x22\x22,\
\x22\x22,\x22\x22],[\x22mathsf\x22\
,\x22textsf\x22,\x22SansS\
erif-Regular\x22],[\
\x22mathboldsf\x22,\x22te\
xtboldsf\x22,\x22SansS\
erif-Bold\x22],[\x22ma\
thtt\x22,\x22texttt\x22,\x22\
Typewriter-Regul\
ar\x22]],Fe=functio\
n(e,t,r){return \
ae[r][e]&&ae[r][\
e].replace&&(e=a\
e[r][e].replace)\
,{value:e,metric\
s:N(e,t,r)}},Ge=\
function(e,t,r,n\
,a){var i,o=Fe(e\
,t,r),s=o.metric\
s;if(e=o.value,s\
){var l=s.italic\
;(\x22text\x22===r||n&\
&\x22mathit\x22===n.fo\
nt)&&(l=0),i=new\
 Z(e,s.height,s.\
depth,l,s.skew,s\
.width,a)}else\x22u\
ndefined\x22!=typeo\
f console&&conso\
le.warn(\x22No char\
acter metrics fo\
r '\x22+e+\x22' in sty\
le '\x22+t+\x22' and m\
ode '\x22+r+\x22'\x22),i=\
new Z(e,0,0,0,0,\
0,a);if(n){i.max\
FontSize=n.sizeM\
ultiplier,n.styl\
e.isTight()&&i.c\
lasses.push(\x22mti\
ght\x22);var h=n.ge\
tColor();h&&(i.s\
tyle.color=h)}re\
turn i},Ue=funct\
ion(e,t){if(G(e.\
classes)!==G(t.c\
lasses)||e.skew!\
==t.skew||e.maxF\
ontSize!==t.maxF\
ontSize)return!1\
;if(1===e.classe\
s.length){var r=\
e.classes[0];if(\
\x22mbin\x22===r||\x22mor\
d\x22===r)return!1}\
for(var n in e.s\
tyle)if(e.style.\
hasOwnProperty(n\
)&&e.style[n]!==\
t.style[n])retur\
n!1;for(var a in\
 t.style)if(t.st\
yle.hasOwnProper\
ty(a)&&e.style[a\
]!==t.style[a])r\
eturn!1;return!0\
},Ye=function(e)\
{for(var t=0,r=0\
,n=0,a=0;a<e.chi\
ldren.length;a++\
){var i=e.childr\
en[a];i.height>t\
&&(t=i.height),i\
.depth>r&&(r=i.d\
epth),i.maxFontS\
ize>n&&(n=i.maxF\
ontSize)}e.heigh\
t=t,e.depth=r,e.\
maxFontSize=n},X\
e=function(e,t,r\
,n){var a=new W(\
e,t,r,n);return \
Ye(a),a},We=func\
tion(e,t,r,n){re\
turn new W(e,t,r\
,n)},_e=function\
(e){var t=new A(\
e);return Ye(t),\
t},je=function(e\
,t,r){var n=\x22\x22;s\
witch(e){case\x22am\
srm\x22:n=\x22AMS\x22;bre\
ak;case\x22textrm\x22:\
n=\x22Main\x22;break;c\
ase\x22textsf\x22:n=\x22S\
ansSerif\x22;break;\
case\x22texttt\x22:n=\x22\
Typewriter\x22;brea\
k;default:n=e}re\
turn n+\x22-\x22+(\x22tex\
tbf\x22===t&&\x22texti\
t\x22===r?\x22BoldItal\
ic\x22:\x22textbf\x22===t\
?\x22Bold\x22:\x22textit\x22\
===t?\x22Italic\x22:\x22R\
egular\x22)},$e={ma\
thbf:{variant:\x22b\
old\x22,fontName:\x22M\
ain-Bold\x22},mathr\
m:{variant:\x22norm\
al\x22,fontName:\x22Ma\
in-Regular\x22},tex\
tit:{variant:\x22it\
alic\x22,fontName:\x22\
Main-Italic\x22},ma\
thit:{variant:\x22i\
talic\x22,fontName:\
\x22Main-Italic\x22},m\
athnormal:{varia\
nt:\x22italic\x22,font\
Name:\x22Math-Itali\
c\x22},mathbb:{vari\
ant:\x22double-stru\
ck\x22,fontName:\x22AM\
S-Regular\x22},math\
cal:{variant:\x22sc\
ript\x22,fontName:\x22\
Caligraphic-Regu\
lar\x22},mathfrak:{\
variant:\x22fraktur\
\x22,fontName:\x22Frak\
tur-Regular\x22},ma\
thscr:{variant:\x22\
script\x22,fontName\
:\x22Script-Regular\
\x22},mathsf:{varia\
nt:\x22sans-serif\x22,\
fontName:\x22SansSe\
rif-Regular\x22},ma\
thtt:{variant:\x22m\
onospace\x22,fontNa\
me:\x22Typewriter-R\
egular\x22}},Ze={ve\
c:[\x22vec\x22,.471,.7\
14],oiintSize1:[\
\x22oiintSize1\x22,.95\
7,.499],oiintSiz\
e2:[\x22oiintSize2\x22\
,1.472,.659],oii\
intSize1:[\x22oiiin\
tSize1\x22,1.304,.4\
99],oiiintSize2:\
[\x22oiiintSize2\x22,1\
.98,.659]},Ke={f\
ontMap:$e,makeSy\
mbol:Ge,mathsym:\
function(e,t,r,n\
){return void 0=\
==n&&(n=[]),\x22bol\
dsymbol\x22===r.fon\
t&&Fe(e,\x22Main-Bo\
ld\x22,t).metrics?G\
e(e,\x22Main-Bold\x22,\
t,r,n.concat([\x22m\
athbf\x22])):\x22\x5c\x5c\x22==\
=e||\x22main\x22===ae[\
t][e].font?Ge(e,\
\x22Main-Regular\x22,t\
,r,n):Ge(e,\x22AMS-\
Regular\x22,t,r,n.c\
oncat([\x22amsrm\x22])\
)},makeSpan:Xe,m\
akeSvgSpan:We,ma\
keLineSpan:funct\
ion(e,t,r){var n\
=Xe([e],[],t);re\
turn n.height=Ma\
th.max(r||t.font\
Metrics().defaul\
tRuleThickness,t\
.minRuleThicknes\
s),n.style.borde\
rBottomWidth=F(n\
.height),n.maxFo\
ntSize=1,n},make\
Anchor:function(\
e,t,r,n){var a=n\
ew _(e,t,r,n);re\
turn Ye(a),a},ma\
keFragment:_e,wr\
apFragment:funct\
ion(e,t){return \
e instanceof A?X\
e([],[e],t):e},m\
akeVList:functio\
n(e,t){for(var r\
=function(e){if(\
\x22individualShift\
\x22===e.positionTy\
pe){for(var t=e.\
children,r=[t[0]\
],n=-t[0].shift-\
t[0].elem.depth,\
a=n,i=1;i<t.leng\
th;i++){var o=-t\
[i].shift-a-t[i]\
.elem.depth,s=o-\
(t[i-1].elem.hei\
ght+t[i-1].elem.\
depth);a+=o,r.pu\
sh({type:\x22kern\x22,\
size:s}),r.push(\
t[i])}return{chi\
ldren:r,depth:n}\
}var l;if(\x22top\x22=\
==e.positionType\
){for(var h=e.po\
sitionData,c=0;c\
<e.children.leng\
th;c++){var m=e.\
children[c];h-=\x22\
kern\x22===m.type?m\
.size:m.elem.hei\
ght+m.elem.depth\
}l=h}else if(\x22bo\
ttom\x22===e.positi\
onType)l=-e.posi\
tionData;else{va\
r u=e.children[0\
];if(\x22elem\x22!==u.\
type)throw new E\
rror('First chil\
d must have type\
 \x22elem\x22.');if(\x22s\
hift\x22===e.positi\
onType)l=-u.elem\
.depth-e.positio\
nData;else{if(\x22f\
irstBaseline\x22!==\
e.positionType)t\
hrow new Error(\x22\
Invalid position\
Type \x22+e.positio\
nType+\x22.\x22);l=-u.\
elem.depth}}retu\
rn{children:e.ch\
ildren,depth:l}}\
(e),n=r.children\
,a=r.depth,i=0,o\
=0;o<n.length;o+\
+){var s=n[o];if\
(\x22elem\x22===s.type\
){var l=s.elem;i\
=Math.max(i,l.ma\
xFontSize,l.heig\
ht)}}i+=2;var h=\
Xe([\x22pstrut\x22],[]\
);h.style.height\
=F(i);for(var c=\
[],m=a,u=a,p=a,d\
=0;d<n.length;d+\
+){var f=n[d];if\
(\x22kern\x22===f.type\
)p+=f.size;else{\
var g=f.elem,v=f\
.wrapperClasses|\
|[],b=f.wrapperS\
tyle||{},y=Xe(v,\
[h,g],void 0,b);\
y.style.top=F(-i\
-p-g.depth),f.ma\
rginLeft&&(y.sty\
le.marginLeft=f.\
marginLeft),f.ma\
rginRight&&(y.st\
yle.marginRight=\
f.marginRight),c\
.push(y),p+=g.he\
ight+g.depth}m=M\
ath.min(m,p),u=M\
ath.max(u,p)}var\
 x,w=Xe([\x22vlist\x22\
],c);if(w.style.\
height=F(u),m<0)\
{var k=Xe([],[])\
,S=Xe([\x22vlist\x22],\
[k]);S.style.hei\
ght=F(-m);var M=\
Xe([\x22vlist-s\x22],[\
new Z(\x22\x5cu200b\x22)]\
);x=[Xe([\x22vlist-\
r\x22],[w,M]),Xe([\x22\
vlist-r\x22],[S])]}\
else x=[Xe([\x22vli\
st-r\x22],[w])];var\
 z=Xe([\x22vlist-t\x22\
],x);return 2===\
x.length&&z.clas\
ses.push(\x22vlist-\
t2\x22),z.height=u,\
z.depth=-m,z},ma\
keOrd:function(e\
,t,r){var a=e.mo\
de,i=e.text,o=[\x22\
mord\x22],s=\x22math\x22=\
==a||\x22text\x22===a&\
&t.font,l=s?t.fo\
nt:t.fontFamily;\
if(55349===i.cha\
rCodeAt(0)){var \
h=function(e,t){\
var r=1024*(e.ch\
arCodeAt(0)-5529\
6)+(e.charCodeAt\
(1)-56320)+65536\
,a=\x22math\x22===t?0:\
1;if(119808<=r&&\
r<120484){var i=\
Math.floor((r-11\
9808)/26);return\
[Ve[i][2],Ve[i][\
a]]}if(120782<=r\
&&r<=120831){var\
 o=Math.floor((r\
-120782)/10);ret\
urn[Pe[o][2],Pe[\
o][a]]}if(120485\
===r||120486===r\
)return[Ve[0][2]\
,Ve[0][a]];if(12\
0486<r&&r<120782\
)return[\x22\x22,\x22\x22];t\
hrow new n(\x22Unsu\
pported characte\
r: \x22+e)}(i,a),c=\
h[0],m=h[1];retu\
rn Ge(i,c,a,t,o.\
concat(m))}if(l)\
{var u,p;if(\x22bol\
dsymbol\x22===l){va\
r d=function(e,t\
,r,n,a){return\x22t\
extord\x22!==a&&Fe(\
e,\x22Math-BoldItal\
ic\x22,t).metrics?{\
fontName:\x22Math-B\
oldItalic\x22,fontC\
lass:\x22boldsymbol\
\x22}:{fontName:\x22Ma\
in-Bold\x22,fontCla\
ss:\x22mathbf\x22}}(i,\
a,0,0,r);u=d.fon\
tName,p=[d.fontC\
lass]}else s?(u=\
$e[l].fontName,p\
=[l]):(u=je(l,t.\
fontWeight,t.fon\
tShape),p=[l,t.f\
ontWeight,t.font\
Shape]);if(Fe(i,\
u,a).metrics)ret\
urn Ge(i,u,a,t,o\
.concat(p));if(w\
e.hasOwnProperty\
(i)&&\x22Typewriter\
\x22===u.slice(0,10\
)){for(var f=[],\
g=0;g<i.length;g\
++)f.push(Ge(i[g\
],u,a,t,o.concat\
(p)));return _e(\
f)}}if(\x22mathord\x22\
===r)return Ge(i\
,\x22Math-Italic\x22,a\
,t,o.concat([\x22ma\
thnormal\x22]));if(\
\x22textord\x22===r){v\
ar v=ae[a][i]&&a\
e[a][i].font;if(\
\x22ams\x22===v){var b\
=je(\x22amsrm\x22,t.fo\
ntWeight,t.fontS\
hape);return Ge(\
i,b,a,t,o.concat\
(\x22amsrm\x22,t.fontW\
eight,t.fontShap\
e))}if(\x22main\x22!==\
v&&v){var y=je(v\
,t.fontWeight,t.\
fontShape);retur\
n Ge(i,y,a,t,o.c\
oncat(y,t.fontWe\
ight,t.fontShape\
))}var x=je(\x22tex\
trm\x22,t.fontWeigh\
t,t.fontShape);r\
eturn Ge(i,x,a,t\
,o.concat(t.font\
Weight,t.fontSha\
pe))}throw new E\
rror(\x22unexpected\
 type: \x22+r+\x22 in \
makeOrd\x22)},makeG\
lue:function(e,t\
){var r=Xe([\x22msp\
ace\x22],[],t),n=P(\
e,t);return r.st\
yle.marginRight=\
F(n),r},staticSv\
g:function(e,t){\
var r=Ze[e],n=r[\
0],a=r[1],i=r[2]\
,o=new J(n),s=ne\
w K([o],{width:F\
(a),height:F(i),\
style:\x22width:\x22+F\
(a),viewBox:\x220 0\
 \x22+1e3*a+\x22 \x22+1e3\
*i,preserveAspec\
tRatio:\x22xMinYMin\
\x22}),l=We([\x22overl\
ay\x22],[s],t);retu\
rn l.height=i,l.\
style.height=F(i\
),l.style.width=\
F(a),l},svgData:\
Ze,tryCombineCha\
rs:function(e){f\
or(var t=0;t<e.l\
ength-1;t++){var\
 r=e[t],n=e[t+1]\
;r instanceof Z&\
&n instanceof Z&\
&Ue(r,n)&&(r.tex\
t+=n.text,r.heig\
ht=Math.max(r.he\
ight,n.height),r\
.depth=Math.max(\
r.depth,n.depth)\
,r.italic=n.ital\
ic,e.splice(t+1,\
1),t--)}return e\
}},Je={number:3,\
unit:\x22mu\x22},Qe={n\
umber:4,unit:\x22mu\
\x22},et={number:5,\
unit:\x22mu\x22},tt={m\
ord:{mop:Je,mbin\
:Qe,mrel:et,minn\
er:Je},mop:{mord\
:Je,mop:Je,mrel:\
et,minner:Je},mb\
in:{mord:Qe,mop:\
Qe,mopen:Qe,minn\
er:Qe},mrel:{mor\
d:et,mop:et,mope\
n:et,minner:et},\
mopen:{},mclose:\
{mop:Je,mbin:Qe,\
mrel:et,minner:J\
e},mpunct:{mord:\
Je,mop:Je,mrel:e\
t,mopen:Je,mclos\
e:Je,mpunct:Je,m\
inner:Je},minner\
:{mord:Je,mop:Je\
,mbin:Qe,mrel:et\
,mopen:Je,mpunct\
:Je,minner:Je}},\
rt={mord:{mop:Je\
},mop:{mord:Je,m\
op:Je},mbin:{},m\
rel:{},mopen:{},\
mclose:{mop:Je},\
mpunct:{},minner\
:{mop:Je}},nt={}\
,at={},it={};fun\
ction ot(e){for(\
var t=e.type,r=e\
.names,n=e.props\
,a=e.handler,i=e\
.htmlBuilder,o=e\
.mathmlBuilder,s\
={type:t,numArgs\
:n.numArgs,argTy\
pes:n.argTypes,a\
llowedInArgument\
:!!n.allowedInAr\
gument,allowedIn\
Text:!!n.allowed\
InText,allowedIn\
Math:void 0===n.\
allowedInMath||n\
.allowedInMath,n\
umOptionalArgs:n\
.numOptionalArgs\
||0,infix:!!n.in\
fix,primitive:!!\
n.primitive,hand\
ler:a},l=0;l<r.l\
ength;++l)nt[r[l\
]]=s;t&&(i&&(at[\
t]=i),o&&(it[t]=\
o))}function st(\
e){ot({type:e.ty\
pe,names:[],prop\
s:{numArgs:0},ha\
ndler:function()\
{throw new Error\
(\x22Should never b\
e called.\x22)},htm\
lBuilder:e.htmlB\
uilder,mathmlBui\
lder:e.mathmlBui\
lder})}var lt=fu\
nction(e){return\
\x22ordgroup\x22===e.t\
ype&&1===e.body.\
length?e.body[0]\
:e},ht=function(\
e){return\x22ordgro\
up\x22===e.type?e.b\
ody:[e]},ct=Ke.m\
akeSpan,mt=[\x22lef\
tmost\x22,\x22mbin\x22,\x22m\
open\x22,\x22mrel\x22,\x22mo\
p\x22,\x22mpunct\x22],ut=\
[\x22rightmost\x22,\x22mr\
el\x22,\x22mclose\x22,\x22mp\
unct\x22],pt={displ\
ay:x.DISPLAY,tex\
t:x.TEXT,script:\
x.SCRIPT,scripts\
cript:x.SCRIPTSC\
RIPT},dt={mord:\x22\
mord\x22,mop:\x22mop\x22,\
mbin:\x22mbin\x22,mrel\
:\x22mrel\x22,mopen:\x22m\
open\x22,mclose:\x22mc\
lose\x22,mpunct:\x22mp\
unct\x22,minner:\x22mi\
nner\x22},ft=functi\
on(e,t,r,n){void\
 0===n&&(n=[null\
,null]);for(var \
a=[],i=0;i<e.len\
gth;i++){var o=w\
t(e[i],t);if(o i\
nstanceof A){var\
 s=o.children;a.\
push.apply(a,s)}\
else a.push(o)}i\
f(Ke.tryCombineC\
hars(a),!r)retur\
n a;var h=t;if(1\
===e.length){var\
 c=e[0];\x22sizing\x22\
===c.type?h=t.ha\
vingSize(c.size)\
:\x22styling\x22===c.t\
ype&&(h=t.having\
Style(pt[c.style\
]))}var m=ct([n[\
0]||\x22leftmost\x22],\
[],t),u=ct([n[1]\
||\x22rightmost\x22],[\
],t),p=\x22root\x22===\
r;return gt(a,(f\
unction(e,t){var\
 r=t.classes[0],\
n=e.classes[0];\x22\
mbin\x22===r&&l.con\
tains(ut,n)?t.cl\
asses[0]=\x22mord\x22:\
\x22mbin\x22===n&&l.co\
ntains(mt,r)&&(e\
.classes[0]=\x22mor\
d\x22)}),{node:m},u\
,p),gt(a,(functi\
on(e,t){var r=yt\
(t),n=yt(e),a=r&\
&n?e.hasClass(\x22m\
tight\x22)?rt[r][n]\
:tt[r][n]:null;i\
f(a)return Ke.ma\
keGlue(a,h)}),{n\
ode:m},u,p),a},g\
t=function e(t,r\
,n,a,i){a&&t.pus\
h(a);for(var o=0\
;o<t.length;o++)\
{var s=t[o],l=vt\
(s);if(l)e(l.chi\
ldren,r,n,null,i\
);else{var h=!s.\
hasClass(\x22mspace\
\x22);if(h){var c=r\
(s,n.node);c&&(n\
.insertAfter?n.i\
nsertAfter(c):(t\
.unshift(c),o++)\
)}h?n.node=s:i&&\
s.hasClass(\x22newl\
ine\x22)&&(n.node=c\
t([\x22leftmost\x22]))\
,n.insertAfter=f\
unction(e){retur\
n function(r){t.\
splice(e+1,0,r),\
o++}}(o)}}a&&t.p\
op()},vt=functio\
n(e){return e in\
stanceof A||e in\
stanceof _||e in\
stanceof W&&e.ha\
sClass(\x22enclosin\
g\x22)?e:null},bt=f\
unction e(t,r){v\
ar n=vt(t);if(n)\
{var a=n.childre\
n;if(a.length){i\
f(\x22right\x22===r)re\
turn e(a[a.lengt\
h-1],\x22right\x22);if\
(\x22left\x22===r)retu\
rn e(a[0],\x22left\x22\
)}}return t},yt=\
function(e,t){re\
turn e?(t&&(e=bt\
(e,t)),dt[e.clas\
ses[0]]||null):n\
ull},xt=function\
(e,t){var r=[\x22nu\
lldelimiter\x22].co\
ncat(e.baseSizin\
gClasses());retu\
rn ct(t.concat(r\
))},wt=function(\
e,t,r){if(!e)ret\
urn ct();if(at[e\
.type]){var a=at\
[e.type](e,t);if\
(r&&t.size!==r.s\
ize){a=ct(t.sizi\
ngClasses(r),[a]\
,t);var i=t.size\
Multiplier/r.siz\
eMultiplier;a.he\
ight*=i,a.depth*\
=i}return a}thro\
w new n(\x22Got gro\
up of unknown ty\
pe: '\x22+e.type+\x22'\
\x22)};function kt(\
e,t){var r=ct([\x22\
base\x22],e,t),n=ct\
([\x22strut\x22]);retu\
rn n.style.heigh\
t=F(r.height+r.d\
epth),r.depth&&(\
n.style.vertical\
Align=F(-r.depth\
)),r.children.un\
shift(n),r}funct\
ion St(e,t){var \
r=null;1===e.len\
gth&&\x22tag\x22===e[0\
].type&&(r=e[0].\
tag,e=e[0].body)\
;var n,a=ft(e,t,\
\x22root\x22);2===a.le\
ngth&&a[1].hasCl\
ass(\x22tag\x22)&&(n=a\
.pop());for(var \
i,o=[],s=[],l=0;\
l<a.length;l++)i\
f(s.push(a[l]),a\
[l].hasClass(\x22mb\
in\x22)||a[l].hasCl\
ass(\x22mrel\x22)||a[l\
].hasClass(\x22allo\
wbreak\x22)){for(va\
r h=!1;l<a.lengt\
h-1&&a[l+1].hasC\
lass(\x22mspace\x22)&&\
!a[l+1].hasClass\
(\x22newline\x22);)l++\
,s.push(a[l]),a[\
l].hasClass(\x22nob\
reak\x22)&&(h=!0);h\
||(o.push(kt(s,t\
)),s=[])}else a[\
l].hasClass(\x22new\
line\x22)&&(s.pop()\
,s.length>0&&(o.\
push(kt(s,t)),s=\
[]),o.push(a[l])\
);s.length>0&&o.\
push(kt(s,t)),r?\
((i=kt(ft(r,t,!0\
))).classes=[\x22ta\
g\x22],o.push(i)):n\
&&o.push(n);var \
c=ct([\x22katex-htm\
l\x22],o);if(c.setA\
ttribute(\x22aria-h\
idden\x22,\x22true\x22),i\
){var m=i.childr\
en[0];m.style.he\
ight=F(c.height+\
c.depth),c.depth\
&&(m.style.verti\
calAlign=F(-c.de\
pth))}return c}f\
unction Mt(e){re\
turn new A(e)}va\
r zt=function(){\
function e(e,t,r\
){this.type=void\
 0,this.attribut\
es=void 0,this.c\
hildren=void 0,t\
his.classes=void\
 0,this.type=e,t\
his.attributes={\
},this.children=\
t||[],this.class\
es=r||[]}var t=e\
.prototype;retur\
n t.setAttribute\
=function(e,t){t\
his.attributes[e\
]=t},t.getAttrib\
ute=function(e){\
return this.attr\
ibutes[e]},t.toN\
ode=function(){v\
ar e=document.cr\
eateElementNS(\x22h\
ttp://www.w3.org\
/1998/Math/MathM\
L\x22,this.type);fo\
r(var t in this.\
attributes)Objec\
t.prototype.hasO\
wnProperty.call(\
this.attributes,\
t)&&e.setAttribu\
te(t,this.attrib\
utes[t]);this.cl\
asses.length>0&&\
(e.className=G(t\
his.classes));fo\
r(var r=0;r<this\
.children.length\
;r++)e.appendChi\
ld(this.children\
[r].toNode());re\
turn e},t.toMark\
up=function(){va\
r e=\x22<\x22+this.typ\
e;for(var t in t\
his.attributes)O\
bject.prototype.\
hasOwnProperty.c\
all(this.attribu\
tes,t)&&(e+=\x22 \x22+\
t+'=\x22',e+=l.esca\
pe(this.attribut\
es[t]),e+='\x22');t\
his.classes.leng\
th>0&&(e+=' clas\
s =\x22'+l.escape(G\
(this.classes))+\
'\x22'),e+=\x22>\x22;for(\
var r=0;r<this.c\
hildren.length;r\
++)e+=this.child\
ren[r].toMarkup(\
);return e+=\x22</\x22\
+this.type+\x22>\x22},\
t.toText=functio\
n(){return this.\
children.map((fu\
nction(e){return\
 e.toText()})).j\
oin(\x22\x22)},e}(),At\
=function(){func\
tion e(e){this.t\
ext=void 0,this.\
text=e}var t=e.p\
rototype;return \
t.toNode=functio\
n(){return docum\
ent.createTextNo\
de(this.text)},t\
.toMarkup=functi\
on(){return l.es\
cape(this.toText\
())},t.toText=fu\
nction(){return \
this.text},e}(),\
Tt={MathNode:zt,\
TextNode:At,Spac\
eNode:function()\
{function e(e){t\
his.width=void 0\
,this.character=\
void 0,this.widt\
h=e,this.charact\
er=e>=.05555&&e<\
=.05556?\x22\x5cu200a\x22\
:e>=.1666&&e<=.1\
667?\x22\x5cu2009\x22:e>=\
.2222&&e<=.2223?\
\x22\x5cu2005\x22:e>=.277\
7&&e<=.2778?\x22\x5cu2\
005\x5cu200a\x22:e>=-.\
05556&&e<=-.0555\
5?\x22\x5cu200a\x5cu2063\x22\
:e>=-.1667&&e<=-\
.1666?\x22\x5cu2009\x5cu2\
063\x22:e>=-.2223&&\
e<=-.2222?\x22\x5cu205\
f\x5cu2063\x22:e>=-.27\
78&&e<=-.2777?\x22\x5c\
u2005\x5cu2063\x22:nul\
l}var t=e.protot\
ype;return t.toN\
ode=function(){i\
f(this.character\
)return document\
.createTextNode(\
this.character);\
var e=document.c\
reateElementNS(\x22\
http://www.w3.or\
g/1998/Math/Math\
ML\x22,\x22mspace\x22);re\
turn e.setAttrib\
ute(\x22width\x22,F(th\
is.width)),e},t.\
toMarkup=functio\
n(){return this.\
character?\x22<mtex\
t>\x22+this.charact\
er+\x22</mtext>\x22:'<\
mspace width=\x22'+\
F(this.width)+'\x22\
/>'},t.toText=fu\
nction(){return \
this.character?t\
his.character:\x22 \
\x22},e}(),newDocum\
entFragment:Mt},\
Bt=function(e,t,\
r){return!ae[t][\
e]||!ae[t][e].re\
place||55349===e\
.charCodeAt(0)||\
we.hasOwnPropert\
y(e)&&r&&(r.font\
Family&&\x22tt\x22===r\
.fontFamily.slic\
e(4,6)||r.font&&\
\x22tt\x22===r.font.sl\
ice(4,6))||(e=ae\
[t][e].replace),\
new Tt.TextNode(\
e)},Ct=function(\
e){return 1===e.\
length?e[0]:new \
Tt.MathNode(\x22mro\
w\x22,e)},Nt=functi\
on(e,t){if(\x22text\
tt\x22===t.fontFami\
ly)return\x22monosp\
ace\x22;if(\x22textsf\x22\
===t.fontFamily)\
return\x22textit\x22==\
=t.fontShape&&\x22t\
extbf\x22===t.fontW\
eight?\x22sans-seri\
f-bold-italic\x22:\x22\
textit\x22===t.font\
Shape?\x22sans-seri\
f-italic\x22:\x22textb\
f\x22===t.fontWeigh\
t?\x22bold-sans-ser\
if\x22:\x22sans-serif\x22\
;if(\x22textit\x22===t\
.fontShape&&\x22tex\
tbf\x22===t.fontWei\
ght)return\x22bold-\
italic\x22;if(\x22text\
it\x22===t.fontShap\
e)return\x22italic\x22\
;if(\x22textbf\x22===t\
.fontWeight)retu\
rn\x22bold\x22;var r=t\
.font;if(!r||\x22ma\
thnormal\x22===r)re\
turn null;var n=\
e.mode;if(\x22mathi\
t\x22===r)return\x22it\
alic\x22;if(\x22boldsy\
mbol\x22===r)return\
\x22textord\x22===e.ty\
pe?\x22bold\x22:\x22bold-\
italic\x22;if(\x22math\
bf\x22===r)return\x22b\
old\x22;if(\x22mathbb\x22\
===r)return\x22doub\
le-struck\x22;if(\x22m\
athfrak\x22===r)ret\
urn\x22fraktur\x22;if(\
\x22mathscr\x22===r||\x22\
mathcal\x22===r)ret\
urn\x22script\x22;if(\x22\
mathsf\x22===r)retu\
rn\x22sans-serif\x22;i\
f(\x22mathtt\x22===r)r\
eturn\x22monospace\x22\
;var a=e.text;re\
turn l.contains(\
[\x22\x5c\x5cimath\x22,\x22\x5c\x5cjm\
ath\x22],a)?null:(a\
e[n][a]&&ae[n][a\
].replace&&(a=ae\
[n][a].replace),\
N(a,Ke.fontMap[r\
].fontName,n)?Ke\
.fontMap[r].vari\
ant:null)},qt=fu\
nction(e,t,r){if\
(1===e.length){v\
ar n=Rt(e[0],t);\
return r&&n inst\
anceof zt&&\x22mo\x22=\
==n.type&&(n.set\
Attribute(\x22lspac\
e\x22,\x220em\x22),n.setA\
ttribute(\x22rspace\
\x22,\x220em\x22)),[n]}fo\
r(var a,i=[],o=0\
;o<e.length;o++)\
{var s=Rt(e[o],t\
);if(s instanceo\
f zt&&a instance\
of zt){if(\x22mtext\
\x22===s.type&&\x22mte\
xt\x22===a.type&&s.\
getAttribute(\x22ma\
thvariant\x22)===a.\
getAttribute(\x22ma\
thvariant\x22)){var\
 l;(l=a.children\
).push.apply(l,s\
.children);conti\
nue}if(\x22mn\x22===s.\
type&&\x22mn\x22===a.t\
ype){var h;(h=a.\
children).push.a\
pply(h,s.childre\
n);continue}if(\x22\
mi\x22===s.type&&1=\
==s.children.len\
gth&&\x22mn\x22===a.ty\
pe){var c=s.chil\
dren[0];if(c ins\
tanceof At&&\x22.\x22=\
==c.text){var m;\
(m=a.children).p\
ush.apply(m,s.ch\
ildren);continue\
}}else if(\x22mi\x22==\
=a.type&&1===a.c\
hildren.length){\
var u=a.children\
[0];if(u instanc\
eof At&&\x22\x5cu0338\x22\
===u.text&&(\x22mo\x22\
===s.type||\x22mi\x22=\
==s.type||\x22mn\x22==\
=s.type)){var p=\
s.children[0];p \
instanceof At&&p\
.text.length>0&&\
(p.text=p.text.s\
lice(0,1)+\x22\x5cu033\
8\x22+p.text.slice(\
1),i.pop())}}}i.\
push(s),a=s}retu\
rn i},It=functio\
n(e,t,r){return \
Ct(qt(e,t,r))},R\
t=function(e,t){\
if(!e)return new\
 Tt.MathNode(\x22mr\
ow\x22);if(it[e.typ\
e])return it[e.t\
ype](e,t);throw \
new n(\x22Got group\
 of unknown type\
: '\x22+e.type+\x22'\x22)\
};function Ht(e,\
t,r,n,a){var i,o\
=qt(e,r);i=1===o\
.length&&o[0]ins\
tanceof zt&&l.co\
ntains([\x22mrow\x22,\x22\
mtable\x22],o[0].ty\
pe)?o[0]:new Tt.\
MathNode(\x22mrow\x22,\
o);var s=new Tt.\
MathNode(\x22annota\
tion\x22,[new Tt.Te\
xtNode(t)]);s.se\
tAttribute(\x22enco\
ding\x22,\x22applicati\
on/x-tex\x22);var h\
=new Tt.MathNode\
(\x22semantics\x22,[i,\
s]),c=new Tt.Mat\
hNode(\x22math\x22,[h]\
);c.setAttribute\
(\x22xmlns\x22,\x22http:/\
/www.w3.org/1998\
/Math/MathML\x22),n\
&&c.setAttribute\
(\x22display\x22,\x22bloc\
k\x22);var m=a?\x22kat\
ex\x22:\x22katex-mathm\
l\x22;return Ke.mak\
eSpan([m],[c])}v\
ar Ot=function(e\
){return new E({\
style:e.displayM\
ode?x.DISPLAY:x.\
TEXT,maxSize:e.m\
axSize,minRuleTh\
ickness:e.minRul\
eThickness})},Et\
=function(e,t){i\
f(t.displayMode)\
{var r=[\x22katex-d\
isplay\x22];t.leqno\
&&r.push(\x22leqno\x22\
),t.fleqn&&r.pus\
h(\x22fleqn\x22),e=Ke.\
makeSpan(r,[e])}\
return e},Lt=fun\
ction(e,t,r){var\
 n,a=Ot(r);if(\x22m\
athml\x22===r.outpu\
t)return Ht(e,t,\
a,r.displayMode,\
!0);if(\x22html\x22===\
r.output){var i=\
St(e,a);n=Ke.mak\
eSpan([\x22katex\x22],\
[i])}else{var o=\
Ht(e,t,a,r.displ\
ayMode,!1),s=St(\
e,a);n=Ke.makeSp\
an([\x22katex\x22],[o,\
s])}return Et(n,\
r)},Dt={widehat:\
\x22^\x22,widecheck:\x22\x5c\
u02c7\x22,widetilde\
:\x22~\x22,utilde:\x22~\x22,\
overleftarrow:\x22\x5c\
u2190\x22,underleft\
arrow:\x22\x5cu2190\x22,x\
leftarrow:\x22\x5cu219\
0\x22,overrightarro\
w:\x22\x5cu2192\x22,under\
rightarrow:\x22\x5cu21\
92\x22,xrightarrow:\
\x22\x5cu2192\x22,underbr\
ace:\x22\x5cu23df\x22,ove\
rbrace:\x22\x5cu23de\x22,\
overgroup:\x22\x5cu23e\
0\x22,undergroup:\x22\x5c\
u23e1\x22,overleftr\
ightarrow:\x22\x5cu219\
4\x22,underleftrigh\
tarrow:\x22\x5cu2194\x22,\
xleftrightarrow:\
\x22\x5cu2194\x22,Overrig\
htarrow:\x22\x5cu21d2\x22\
,xRightarrow:\x22\x5cu\
21d2\x22,overleftha\
rpoon:\x22\x5cu21bc\x22,x\
leftharpoonup:\x22\x5c\
u21bc\x22,overright\
harpoon:\x22\x5cu21c0\x22\
,xrightharpoonup\
:\x22\x5cu21c0\x22,xLefta\
rrow:\x22\x5cu21d0\x22,xL\
eftrightarrow:\x22\x5c\
u21d4\x22,xhookleft\
arrow:\x22\x5cu21a9\x22,x\
hookrightarrow:\x22\
\x5cu21aa\x22,xmapsto:\
\x22\x5cu21a6\x22,xrighth\
arpoondown:\x22\x5cu21\
c1\x22,xleftharpoon\
down:\x22\x5cu21bd\x22,xr\
ightleftharpoons\
:\x22\x5cu21cc\x22,xleftr\
ightharpoons:\x22\x5cu\
21cb\x22,xtwoheadle\
ftarrow:\x22\x5cu219e\x22\
,xtwoheadrightar\
row:\x22\x5cu21a0\x22,xlo\
ngequal:\x22=\x22,xtof\
rom:\x22\x5cu21c4\x22,xri\
ghtleftarrows:\x22\x5c\
u21c4\x22,xrightequ\
ilibrium:\x22\x5cu21cc\
\x22,xleftequilibri\
um:\x22\x5cu21cb\x22,\x22\x5c\x5cc\
drightarrow\x22:\x22\x5cu\
2192\x22,\x22\x5c\x5ccdlefta\
rrow\x22:\x22\x5cu2190\x22,\x22\
\x5c\x5ccdlongequal\x22:\x22\
=\x22},Vt={overrigh\
tarrow:[[\x22righta\
rrow\x22],.888,522,\
\x22xMaxYMin\x22],over\
leftarrow:[[\x22lef\
tarrow\x22],.888,52\
2,\x22xMinYMin\x22],un\
derrightarrow:[[\
\x22rightarrow\x22],.8\
88,522,\x22xMaxYMin\
\x22],underleftarro\
w:[[\x22leftarrow\x22]\
,.888,522,\x22xMinY\
Min\x22],xrightarro\
w:[[\x22rightarrow\x22\
],1.469,522,\x22xMa\
xYMin\x22],\x22\x5c\x5ccdrig\
htarrow\x22:[[\x22righ\
tarrow\x22],3,522,\x22\
xMaxYMin\x22],xleft\
arrow:[[\x22leftarr\
ow\x22],1.469,522,\x22\
xMinYMin\x22],\x22\x5c\x5ccd\
leftarrow\x22:[[\x22le\
ftarrow\x22],3,522,\
\x22xMinYMin\x22],Over\
rightarrow:[[\x22do\
ublerightarrow\x22]\
,.888,560,\x22xMaxY\
Min\x22],xRightarro\
w:[[\x22doubleright\
arrow\x22],1.526,56\
0,\x22xMaxYMin\x22],xL\
eftarrow:[[\x22doub\
leleftarrow\x22],1.\
526,560,\x22xMinYMi\
n\x22],overleftharp\
oon:[[\x22leftharpo\
on\x22],.888,522,\x22x\
MinYMin\x22],xlefth\
arpoonup:[[\x22left\
harpoon\x22],.888,5\
22,\x22xMinYMin\x22],x\
leftharpoondown:\
[[\x22leftharpoondo\
wn\x22],.888,522,\x22x\
MinYMin\x22],overri\
ghtharpoon:[[\x22ri\
ghtharpoon\x22],.88\
8,522,\x22xMaxYMin\x22\
],xrightharpoonu\
p:[[\x22rightharpoo\
n\x22],.888,522,\x22xM\
axYMin\x22],xrighth\
arpoondown:[[\x22ri\
ghtharpoondown\x22]\
,.888,522,\x22xMaxY\
Min\x22],xlongequal\
:[[\x22longequal\x22],\
.888,334,\x22xMinYM\
in\x22],\x22\x5c\x5ccdlongeq\
ual\x22:[[\x22longequa\
l\x22],3,334,\x22xMinY\
Min\x22],xtwoheadle\
ftarrow:[[\x22twohe\
adleftarrow\x22],.8\
88,334,\x22xMinYMin\
\x22],xtwoheadright\
arrow:[[\x22twohead\
rightarrow\x22],.88\
8,334,\x22xMaxYMin\x22\
],overleftrighta\
rrow:[[\x22leftarro\
w\x22,\x22rightarrow\x22]\
,.888,522],overb\
race:[[\x22leftbrac\
e\x22,\x22midbrace\x22,\x22r\
ightbrace\x22],1.6,\
548],underbrace:\
[[\x22leftbraceunde\
r\x22,\x22midbraceunde\
r\x22,\x22rightbraceun\
der\x22],1.6,548],u\
nderleftrightarr\
ow:[[\x22leftarrow\x22\
,\x22rightarrow\x22],.\
888,522],xleftri\
ghtarrow:[[\x22left\
arrow\x22,\x22rightarr\
ow\x22],1.75,522],x\
Leftrightarrow:[\
[\x22doubleleftarro\
w\x22,\x22doublerighta\
rrow\x22],1.75,560]\
,xrightleftharpo\
ons:[[\x22leftharpo\
ondownplus\x22,\x22rig\
htharpoonplus\x22],\
1.75,716],xleftr\
ightharpoons:[[\x22\
leftharpoonplus\x22\
,\x22rightharpoondo\
wnplus\x22],1.75,71\
6],xhookleftarro\
w:[[\x22leftarrow\x22,\
\x22righthook\x22],1.0\
8,522],xhookrigh\
tarrow:[[\x22leftho\
ok\x22,\x22rightarrow\x22\
],1.08,522],over\
linesegment:[[\x22l\
eftlinesegment\x22,\
\x22rightlinesegmen\
t\x22],.888,522],un\
derlinesegment:[\
[\x22leftlinesegmen\
t\x22,\x22rightlineseg\
ment\x22],.888,522]\
,overgroup:[[\x22le\
ftgroup\x22,\x22rightg\
roup\x22],.888,342]\
,undergroup:[[\x22l\
eftgroupunder\x22,\x22\
rightgroupunder\x22\
],.888,342],xmap\
sto:[[\x22leftmapst\
o\x22,\x22rightarrow\x22]\
,1.5,522],xtofro\
m:[[\x22leftToFrom\x22\
,\x22rightToFrom\x22],\
1.75,528],xright\
leftarrows:[[\x22ba\
raboveleftarrow\x22\
,\x22rightarrowabov\
ebar\x22],1.75,901]\
,xrightequilibri\
um:[[\x22barabovesh\
ortleftharpoon\x22,\
\x22rightharpoonabo\
veshortbar\x22],1.7\
5,716],xleftequi\
librium:[[\x22short\
baraboveleftharp\
oon\x22,\x22shortright\
harpoonabovebar\x22\
],1.75,716]},Pt=\
function(e,t,r,n\
,a){var i,o=e.he\
ight+e.depth+r+n\
;if(/fbox|color|\
angl/.test(t)){i\
f(i=Ke.makeSpan(\
[\x22stretchy\x22,t],[\
],a),\x22fbox\x22===t)\
{var s=a.color&&\
a.getColor();s&&\
(i.style.borderC\
olor=s)}}else{va\
r l=[];/^[bx]can\
cel$/.test(t)&&l\
.push(new Q({x1:\
\x220\x22,y1:\x220\x22,x2:\x221\
00%\x22,y2:\x22100%\x22,\x22\
stroke-width\x22:\x220\
.046em\x22})),/^x?c\
ancel$/.test(t)&\
&l.push(new Q({x\
1:\x220\x22,y1:\x22100%\x22,\
x2:\x22100%\x22,y2:\x220\x22\
,\x22stroke-width\x22:\
\x220.046em\x22}));var\
 h=new K(l,{widt\
h:\x22100%\x22,height:\
F(o)});i=Ke.make\
SvgSpan([],[h],a\
)}return i.heigh\
t=o,i.style.heig\
ht=F(o),i},Ft=fu\
nction(e){var t=\
new Tt.MathNode(\
\x22mo\x22,[new Tt.Tex\
tNode(Dt[e.repla\
ce(/^\x5c\x5c/,\x22\x22)])])\
;return t.setAtt\
ribute(\x22stretchy\
\x22,\x22true\x22),t},Gt=\
function(e,t){va\
r r=function(){v\
ar r=4e5,n=e.lab\
el.slice(1);if(l\
.contains([\x22wide\
hat\x22,\x22widecheck\x22\
,\x22widetilde\x22,\x22ut\
ilde\x22],n)){var a\
,i,o,s=\x22ordgroup\
\x22===(d=e.base).t\
ype?d.body.lengt\
h:1;if(s>5)\x22wide\
hat\x22===n||\x22widec\
heck\x22===n?(a=420\
,r=2364,o=.42,i=\
n+\x224\x22):(a=312,r=\
2340,o=.34,i=\x22ti\
lde4\x22);else{var \
h=[1,1,2,2,3,3][\
s];\x22widehat\x22===n\
||\x22widecheck\x22===\
n?(r=[0,1062,236\
4,2364,2364][h],\
a=[0,239,300,360\
,420][h],o=[0,.2\
4,.3,.3,.36,.42]\
[h],i=n+h):(r=[0\
,600,1033,2339,2\
340][h],a=[0,260\
,286,306,312][h]\
,o=[0,.26,.286,.\
3,.306,.34][h],i\
=\x22tilde\x22+h)}var \
c=new J(i),m=new\
 K([c],{width:\x221\
00%\x22,height:F(o)\
,viewBox:\x220 0 \x22+\
r+\x22 \x22+a,preserve\
AspectRatio:\x22non\
e\x22});return{span\
:Ke.makeSvgSpan(\
[],[m],t),minWid\
th:0,height:o}}v\
ar u,p,d,f=[],g=\
Vt[n],v=g[0],b=g\
[1],y=g[2],x=y/1\
e3,w=v.length;if\
(1===w)u=[\x22hide-\
tail\x22],p=[g[3]];\
else if(2===w)u=\
[\x22halfarrow-left\
\x22,\x22halfarrow-rig\
ht\x22],p=[\x22xMinYMi\
n\x22,\x22xMaxYMin\x22];e\
lse{if(3!==w)thr\
ow new Error(\x22Co\
rrect katexImage\
sData or update \
code here to sup\
port\x5cn          \
          \x22+w+\x22 \
children.\x22);u=[\x22\
brace-left\x22,\x22bra\
ce-center\x22,\x22brac\
e-right\x22],p=[\x22xM\
inYMin\x22,\x22xMidYMi\
n\x22,\x22xMaxYMin\x22]}f\
or(var k=0;k<w;k\
++){var S=new J(\
v[k]),M=new K([S\
],{width:\x22400em\x22\
,height:F(x),vie\
wBox:\x220 0 \x22+r+\x22 \
\x22+y,preserveAspe\
ctRatio:p[k]+\x22 s\
lice\x22}),z=Ke.mak\
eSvgSpan([u[k]],\
[M],t);if(1===w)\
return{span:z,mi\
nWidth:b,height:\
x};z.style.heigh\
t=F(x),f.push(z)\
}return{span:Ke.\
makeSpan([\x22stret\
chy\x22],f,t),minWi\
dth:b,height:x}}\
(),n=r.span,a=r.\
minWidth,i=r.hei\
ght;return n.hei\
ght=i,n.style.he\
ight=F(i),a>0&&(\
n.style.minWidth\
=F(a)),n};functi\
on Ut(e,t){if(!e\
||e.type!==t)thr\
ow new Error(\x22Ex\
pected node of t\
ype \x22+t+\x22, but g\
ot \x22+(e?\x22node of\
 type \x22+e.type:S\
tring(e)));retur\
n e}function Yt(\
e){var t=Xt(e);i\
f(!t)throw new E\
rror(\x22Expected n\
ode of symbol gr\
oup type, but go\
t \x22+(e?\x22node of \
type \x22+e.type:St\
ring(e)));return\
 t}function Xt(e\
){return e&&(\x22at\
om\x22===e.type||re\
.hasOwnProperty(\
e.type))?e:null}\
var Wt=function(\
e,t){var r,n,a;e\
&&\x22supsub\x22===e.t\
ype?(r=(n=Ut(e.b\
ase,\x22accent\x22)).b\
ase,e.base=r,a=f\
unction(e){if(e \
instanceof W)ret\
urn e;throw new \
Error(\x22Expected \
span<HtmlDomNode\
> but got \x22+Stri\
ng(e)+\x22.\x22)}(wt(e\
,t)),e.base=n):r\
=(n=Ut(e,\x22accent\
\x22)).base;var i=w\
t(r,t.havingCram\
pedStyle()),o=0;\
if(n.isShifty&&l\
.isCharacterBox(\
r)){var s=l.getB\
aseElem(r);o=ee(\
wt(s,t.havingCra\
mpedStyle())).sk\
ew}var h,c=\x22\x5c\x5cc\x22\
===n.label,m=c?i\
.height+i.depth:\
Math.min(i.heigh\
t,t.fontMetrics(\
).xHeight);if(n.\
isStretchy)h=Gt(\
n,t),h=Ke.makeVL\
ist({positionTyp\
e:\x22firstBaseline\
\x22,children:[{typ\
e:\x22elem\x22,elem:i}\
,{type:\x22elem\x22,el\
em:h,wrapperClas\
ses:[\x22svg-align\x22\
],wrapperStyle:o\
>0?{width:\x22calc(\
100% - \x22+F(2*o)+\
\x22)\x22,marginLeft:F\
(2*o)}:void 0}]}\
,t);else{var u,p\
;\x22\x5c\x5cvec\x22===n.lab\
el?(u=Ke.staticS\
vg(\x22vec\x22,t),p=Ke\
.svgData.vec[1])\
:((u=ee(u=Ke.mak\
eOrd({mode:n.mod\
e,text:n.label},\
t,\x22textord\x22))).i\
talic=0,p=u.widt\
h,c&&(m+=u.depth\
)),h=Ke.makeSpan\
([\x22accent-body\x22]\
,[u]);var d=\x22\x5c\x5ct\
extcircled\x22===n.\
label;d&&(h.clas\
ses.push(\x22accent\
-full\x22),m=i.heig\
ht);var f=o;d||(\
f-=p/2),h.style.\
left=F(f),\x22\x5c\x5ctex\
tcircled\x22===n.la\
bel&&(h.style.to\
p=\x22.2em\x22),h=Ke.m\
akeVList({positi\
onType:\x22firstBas\
eline\x22,children:\
[{type:\x22elem\x22,el\
em:i},{type:\x22ker\
n\x22,size:-m},{typ\
e:\x22elem\x22,elem:h}\
]},t)}var g=Ke.m\
akeSpan([\x22mord\x22,\
\x22accent\x22],[h],t)\
;return a?(a.chi\
ldren[0]=g,a.hei\
ght=Math.max(g.h\
eight,a.height),\
a.classes[0]=\x22mo\
rd\x22,a):g},_t=fun\
ction(e,t){var r\
=e.isStretchy?Ft\
(e.label):new Tt\
.MathNode(\x22mo\x22,[\
Bt(e.label,e.mod\
e)]),n=new Tt.Ma\
thNode(\x22mover\x22,[\
Rt(e.base,t),r])\
;return n.setAtt\
ribute(\x22accent\x22,\
\x22true\x22),n},jt=ne\
w RegExp([\x22\x5c\x5cacu\
te\x22,\x22\x5c\x5cgrave\x22,\x22\x5c\
\x5cddot\x22,\x22\x5c\x5ctilde\x22\
,\x22\x5c\x5cbar\x22,\x22\x5c\x5cbrev\
e\x22,\x22\x5c\x5ccheck\x22,\x22\x5c\x5c\
hat\x22,\x22\x5c\x5cvec\x22,\x22\x5c\x5c\
dot\x22,\x22\x5c\x5cmathring\
\x22].map((function\
(e){return\x22\x5c\x5c\x22+e\
})).join(\x22|\x22));o\
t({type:\x22accent\x22\
,names:[\x22\x5c\x5cacute\
\x22,\x22\x5c\x5cgrave\x22,\x22\x5c\x5cd\
dot\x22,\x22\x5c\x5ctilde\x22,\x22\
\x5c\x5cbar\x22,\x22\x5c\x5cbreve\x22\
,\x22\x5c\x5ccheck\x22,\x22\x5c\x5cha\
t\x22,\x22\x5c\x5cvec\x22,\x22\x5c\x5cdo\
t\x22,\x22\x5c\x5cmathring\x22,\
\x22\x5c\x5cwidecheck\x22,\x22\x5c\
\x5cwidehat\x22,\x22\x5c\x5cwid\
etilde\x22,\x22\x5c\x5coverr\
ightarrow\x22,\x22\x5c\x5cov\
erleftarrow\x22,\x22\x5c\x5c\
Overrightarrow\x22,\
\x22\x5c\x5coverleftright\
arrow\x22,\x22\x5c\x5covergr\
oup\x22,\x22\x5c\x5coverline\
segment\x22,\x22\x5c\x5cover\
leftharpoon\x22,\x22\x5c\x5c\
overrightharpoon\
\x22],props:{numArg\
s:1},handler:fun\
ction(e,t){var r\
=lt(t[0]),n=!jt.\
test(e.funcName)\
,a=!n||\x22\x5c\x5cwideha\
t\x22===e.funcName|\
|\x22\x5c\x5cwidetilde\x22==\
=e.funcName||\x22\x5c\x5c\
widecheck\x22===e.f\
uncName;return{t\
ype:\x22accent\x22,mod\
e:e.parser.mode,\
label:e.funcName\
,isStretchy:n,is\
Shifty:a,base:r}\
},htmlBuilder:Wt\
,mathmlBuilder:_\
t}),ot({type:\x22ac\
cent\x22,names:[\x22\x5c\x5c\
'\x22,\x22\x5c\x5c`\x22,\x22\x5c\x5c^\x22,\x22\
\x5c\x5c~\x22,\x22\x5c\x5c=\x22,\x22\x5c\x5cu\x22\
,\x22\x5c\x5c.\x22,'\x5c\x5c\x22',\x22\x5c\x5c\
c\x22,\x22\x5c\x5cr\x22,\x22\x5c\x5cH\x22,\x22\
\x5c\x5cv\x22,\x22\x5c\x5ctextcirc\
led\x22],props:{num\
Args:1,allowedIn\
Text:!0,allowedI\
nMath:!0,argType\
s:[\x22primitive\x22]}\
,handler:functio\
n(e,t){var r=t[0\
],n=e.parser.mod\
e;return\x22math\x22==\
=n&&(e.parser.se\
ttings.reportNon\
strict(\x22mathVsTe\
xtAccents\x22,\x22LaTe\
X's accent \x22+e.f\
uncName+\x22 works \
only in text mod\
e\x22),n=\x22text\x22),{t\
ype:\x22accent\x22,mod\
e:n,label:e.func\
Name,isStretchy:\
!1,isShifty:!0,b\
ase:r}},htmlBuil\
der:Wt,mathmlBui\
lder:_t}),ot({ty\
pe:\x22accentUnder\x22\
,names:[\x22\x5c\x5cunder\
leftarrow\x22,\x22\x5c\x5cun\
derrightarrow\x22,\x22\
\x5c\x5cunderleftright\
arrow\x22,\x22\x5c\x5cunderg\
roup\x22,\x22\x5c\x5cunderli\
nesegment\x22,\x22\x5c\x5cut\
ilde\x22],props:{nu\
mArgs:1},handler\
:function(e,t){v\
ar r=e.parser,n=\
e.funcName,a=t[0\
];return{type:\x22a\
ccentUnder\x22,mode\
:r.mode,label:n,\
base:a}},htmlBui\
lder:function(e,\
t){var r=wt(e.ba\
se,t),n=Gt(e,t),\
a=\x22\x5c\x5cutilde\x22===e\
.label?.12:0,i=K\
e.makeVList({pos\
itionType:\x22top\x22,\
positionData:r.h\
eight,children:[\
{type:\x22elem\x22,ele\
m:n,wrapperClass\
es:[\x22svg-align\x22]\
},{type:\x22kern\x22,s\
ize:a},{type:\x22el\
em\x22,elem:r}]},t)\
;return Ke.makeS\
pan([\x22mord\x22,\x22acc\
entunder\x22],[i],t\
)},mathmlBuilder\
:function(e,t){v\
ar r=Ft(e.label)\
,n=new Tt.MathNo\
de(\x22munder\x22,[Rt(\
e.base,t),r]);re\
turn n.setAttrib\
ute(\x22accentunder\
\x22,\x22true\x22),n}});v\
ar $t=function(e\
){var t=new Tt.M\
athNode(\x22mpadded\
\x22,e?[e]:[]);retu\
rn t.setAttribut\
e(\x22width\x22,\x22+0.6e\
m\x22),t.setAttribu\
te(\x22lspace\x22,\x220.3\
em\x22),t};ot({type\
:\x22xArrow\x22,names:\
[\x22\x5c\x5cxleftarrow\x22,\
\x22\x5c\x5cxrightarrow\x22,\
\x22\x5c\x5cxLeftarrow\x22,\x22\
\x5c\x5cxRightarrow\x22,\x22\
\x5c\x5cxleftrightarro\
w\x22,\x22\x5c\x5cxLeftright\
arrow\x22,\x22\x5c\x5cxhookl\
eftarrow\x22,\x22\x5c\x5cxho\
okrightarrow\x22,\x22\x5c\
\x5cxmapsto\x22,\x22\x5c\x5cxri\
ghtharpoondown\x22,\
\x22\x5c\x5cxrightharpoon\
up\x22,\x22\x5c\x5cxleftharp\
oondown\x22,\x22\x5c\x5cxlef\
tharpoonup\x22,\x22\x5c\x5cx\
rightleftharpoon\
s\x22,\x22\x5c\x5cxleftright\
harpoons\x22,\x22\x5c\x5cxlo\
ngequal\x22,\x22\x5c\x5cxtwo\
headrightarrow\x22,\
\x22\x5c\x5cxtwoheadlefta\
rrow\x22,\x22\x5c\x5cxtofrom\
\x22,\x22\x5c\x5cxrightlefta\
rrows\x22,\x22\x5c\x5cxright\
equilibrium\x22,\x22\x5c\x5c\
xleftequilibrium\
\x22,\x22\x5c\x5c\x5c\x5ccdrightar\
row\x22,\x22\x5c\x5c\x5c\x5ccdleft\
arrow\x22,\x22\x5c\x5c\x5c\x5ccdlo\
ngequal\x22],props:\
{numArgs:1,numOp\
tionalArgs:1},ha\
ndler:function(e\
,t,r){var n=e.pa\
rser,a=e.funcNam\
e;return{type:\x22x\
Arrow\x22,mode:n.mo\
de,label:a,body:\
t[0],below:r[0]}\
},htmlBuilder:fu\
nction(e,t){var \
r,n=t.style,a=t.\
havingStyle(n.su\
p()),i=Ke.wrapFr\
agment(wt(e.body\
,a,t),t),o=\x22\x5c\x5cx\x22\
===e.label.slice\
(0,2)?\x22x\x22:\x22cd\x22;i\
.classes.push(o+\
\x22-arrow-pad\x22),e.\
below&&(a=t.havi\
ngStyle(n.sub())\
,(r=Ke.wrapFragm\
ent(wt(e.below,a\
,t),t)).classes.\
push(o+\x22-arrow-p\
ad\x22));var s,l=Gt\
(e,t),h=-t.fontM\
etrics().axisHei\
ght+.5*l.height,\
c=-t.fontMetrics\
().axisHeight-.5\
*l.height-.111;i\
f((i.depth>.25||\
\x22\x5c\x5cxleftequilibr\
ium\x22===e.label)&\
&(c-=i.depth),r)\
{var m=-t.fontMe\
trics().axisHeig\
ht+r.height+.5*l\
.height+.111;s=K\
e.makeVList({pos\
itionType:\x22indiv\
idualShift\x22,chil\
dren:[{type:\x22ele\
m\x22,elem:i,shift:\
c},{type:\x22elem\x22,\
elem:l,shift:h},\
{type:\x22elem\x22,ele\
m:r,shift:m}]},t\
)}else s=Ke.make\
VList({positionT\
ype:\x22individualS\
hift\x22,children:[\
{type:\x22elem\x22,ele\
m:i,shift:c},{ty\
pe:\x22elem\x22,elem:l\
,shift:h}]},t);r\
eturn s.children\
[0].children[0].\
children[1].clas\
ses.push(\x22svg-al\
ign\x22),Ke.makeSpa\
n([\x22mrel\x22,\x22x-arr\
ow\x22],[s],t)},mat\
hmlBuilder:funct\
ion(e,t){var r,n\
=Ft(e.label);if(\
n.setAttribute(\x22\
minsize\x22,\x22x\x22===e\
.label.charAt(0)\
?\x221.75em\x22:\x223.0em\
\x22),e.body){var a\
=$t(Rt(e.body,t)\
);if(e.below){va\
r i=$t(Rt(e.belo\
w,t));r=new Tt.M\
athNode(\x22mundero\
ver\x22,[n,i,a])}el\
se r=new Tt.Math\
Node(\x22mover\x22,[n,\
a])}else if(e.be\
low){var o=$t(Rt\
(e.below,t));r=n\
ew Tt.MathNode(\x22\
munder\x22,[n,o])}e\
lse r=$t(),r=new\
 Tt.MathNode(\x22mo\
ver\x22,[n,r]);retu\
rn r}});var Zt=K\
e.makeSpan;funct\
ion Kt(e,t){var \
r=ft(e.body,t,!0\
);return Zt([e.m\
class],r,t)}func\
tion Jt(e,t){var\
 r,n=qt(e.body,t\
);return\x22minner\x22\
===e.mclass?r=ne\
w Tt.MathNode(\x22m\
padded\x22,n):\x22mord\
\x22===e.mclass?e.i\
sCharacterBox?(r\
=n[0]).type=\x22mi\x22\
:r=new Tt.MathNo\
de(\x22mi\x22,n):(e.is\
CharacterBox?(r=\
n[0]).type=\x22mo\x22:\
r=new Tt.MathNod\
e(\x22mo\x22,n),\x22mbin\x22\
===e.mclass?(r.a\
ttributes.lspace\
=\x220.22em\x22,r.attr\
ibutes.rspace=\x220\
.22em\x22):\x22mpunct\x22\
===e.mclass?(r.a\
ttributes.lspace\
=\x220em\x22,r.attribu\
tes.rspace=\x220.17\
em\x22):\x22mopen\x22===e\
.mclass||\x22mclose\
\x22===e.mclass?(r.\
attributes.lspac\
e=\x220em\x22,r.attrib\
utes.rspace=\x220em\
\x22):\x22minner\x22===e.\
mclass&&(r.attri\
butes.lspace=\x220.\
0556em\x22,r.attrib\
utes.width=\x22+0.1\
111em\x22)),r}ot({t\
ype:\x22mclass\x22,nam\
es:[\x22\x5c\x5cmathord\x22,\
\x22\x5c\x5cmathbin\x22,\x22\x5c\x5cm\
athrel\x22,\x22\x5c\x5cmatho\
pen\x22,\x22\x5c\x5cmathclos\
e\x22,\x22\x5c\x5cmathpunct\x22\
,\x22\x5c\x5cmathinner\x22],\
props:{numArgs:1\
,primitive:!0},h\
andler:function(\
e,t){var r=e.par\
ser,n=e.funcName\
,a=t[0];return{t\
ype:\x22mclass\x22,mod\
e:r.mode,mclass:\
\x22m\x22+n.slice(5),b\
ody:ht(a),isChar\
acterBox:l.isCha\
racterBox(a)}},h\
tmlBuilder:Kt,ma\
thmlBuilder:Jt})\
;var Qt=function\
(e){var t=\x22ordgr\
oup\x22===e.type&&e\
.body.length?e.b\
ody[0]:e;return\x22\
atom\x22!==t.type||\
\x22bin\x22!==t.family\
&&\x22rel\x22!==t.fami\
ly?\x22mord\x22:\x22m\x22+t.\
family};ot({type\
:\x22mclass\x22,names:\
[\x22\x5c\x5c@binrel\x22],pr\
ops:{numArgs:2},\
handler:function\
(e,t){return{typ\
e:\x22mclass\x22,mode:\
e.parser.mode,mc\
lass:Qt(t[0]),bo\
dy:ht(t[1]),isCh\
aracterBox:l.isC\
haracterBox(t[1]\
)}}}),ot({type:\x22\
mclass\x22,names:[\x22\
\x5c\x5cstackrel\x22,\x22\x5c\x5co\
verset\x22,\x22\x5c\x5cunder\
set\x22],props:{num\
Args:2},handler:\
function(e,t){va\
r r,n=e.parser,a\
=e.funcName,i=t[\
1],o=t[0];r=\x22\x5c\x5cs\
tackrel\x22!==a?Qt(\
i):\x22mrel\x22;var s=\
{type:\x22op\x22,mode:\
i.mode,limits:!0\
,alwaysHandleSup\
Sub:!0,parentIsS\
upSub:!1,symbol:\
!1,suppressBaseS\
hift:\x22\x5c\x5cstackrel\
\x22!==a,body:ht(i)\
},h={type:\x22supsu\
b\x22,mode:o.mode,b\
ase:s,sup:\x22\x5c\x5cund\
erset\x22===a?null:\
o,sub:\x22\x5c\x5cunderse\
t\x22===a?o:null};r\
eturn{type:\x22mcla\
ss\x22,mode:n.mode,\
mclass:r,body:[h\
],isCharacterBox\
:l.isCharacterBo\
x(h)}},htmlBuild\
er:Kt,mathmlBuil\
der:Jt}),ot({typ\
e:\x22pmb\x22,names:[\x22\
\x5c\x5cpmb\x22],props:{n\
umArgs:1,allowed\
InText:!0},handl\
er:function(e,t)\
{return{type:\x22pm\
b\x22,mode:e.parser\
.mode,mclass:Qt(\
t[0]),body:ht(t[\
0])}},htmlBuilde\
r:function(e,t){\
var r=ft(e.body,\
t,!0),n=Ke.makeS\
pan([e.mclass],r\
,t);return n.sty\
le.textShadow=\x220\
.02em 0.01em 0.0\
4px\x22,n},mathmlBu\
ilder:function(e\
,t){var r=qt(e.b\
ody,t),n=new Tt.\
MathNode(\x22mstyle\
\x22,r);return n.se\
tAttribute(\x22styl\
e\x22,\x22text-shadow:\
 0.02em 0.01em 0\
.04px\x22),n}});var\
 er={\x22>\x22:\x22\x5c\x5c\x5c\x5ccd\
rightarrow\x22,\x22<\x22:\
\x22\x5c\x5c\x5c\x5ccdleftarrow\
\x22,\x22=\x22:\x22\x5c\x5c\x5c\x5ccdlon\
gequal\x22,A:\x22\x5c\x5cupa\
rrow\x22,V:\x22\x5c\x5cdowna\
rrow\x22,\x22|\x22:\x22\x5c\x5cVer\
t\x22,\x22.\x22:\x22no arrow\
\x22},tr=function(e\
){return\x22textord\
\x22===e.type&&\x22@\x22=\
==e.text};functi\
on rr(e,t,r){var\
 n=er[e];switch(\
n){case\x22\x5c\x5c\x5c\x5ccdri\
ghtarrow\x22:case\x22\x5c\
\x5c\x5c\x5ccdleftarrow\x22:\
return r.callFun\
ction(n,[t[0]],[\
t[1]]);case\x22\x5c\x5cup\
arrow\x22:case\x22\x5c\x5cdo\
wnarrow\x22:var a={\
type:\x22atom\x22,text\
:n,mode:\x22math\x22,f\
amily:\x22rel\x22},i={\
type:\x22ordgroup\x22,\
mode:\x22math\x22,body\
:[r.callFunction\
(\x22\x5c\x5c\x5c\x5ccdleft\x22,[t\
[0]],[]),r.callF\
unction(\x22\x5c\x5cBig\x22,\
[a],[]),r.callFu\
nction(\x22\x5c\x5c\x5c\x5ccdri\
ght\x22,[t[1]],[])]\
};return r.callF\
unction(\x22\x5c\x5c\x5c\x5ccdp\
arent\x22,[i],[]);c\
ase\x22\x5c\x5c\x5c\x5ccdlongeq\
ual\x22:return r.ca\
llFunction(\x22\x5c\x5c\x5c\x5c\
cdlongequal\x22,[],\
[]);case\x22\x5c\x5cVert\x22\
:return r.callFu\
nction(\x22\x5c\x5cBig\x22,[\
{type:\x22textord\x22,\
text:\x22\x5c\x5cVert\x22,mo\
de:\x22math\x22}],[]);\
default:return{t\
ype:\x22textord\x22,te\
xt:\x22 \x22,mode:\x22mat\
h\x22}}}ot({type:\x22c\
dlabel\x22,names:[\x22\
\x5c\x5c\x5c\x5ccdleft\x22,\x22\x5c\x5c\x5c\
\x5ccdright\x22],props\
:{numArgs:1},han\
dler:function(e,\
t){var r=e.parse\
r,n=e.funcName;r\
eturn{type:\x22cdla\
bel\x22,mode:r.mode\
,side:n.slice(4)\
,label:t[0]}},ht\
mlBuilder:functi\
on(e,t){var r=t.\
havingStyle(t.st\
yle.sup()),n=Ke.\
wrapFragment(wt(\
e.label,r,t),t);\
return n.classes\
.push(\x22cd-label-\
\x22+e.side),n.styl\
e.bottom=F(.8-n.\
depth),n.height=\
0,n.depth=0,n},m\
athmlBuilder:fun\
ction(e,t){var r\
=new Tt.MathNode\
(\x22mrow\x22,[Rt(e.la\
bel,t)]);return(\
r=new Tt.MathNod\
e(\x22mpadded\x22,[r])\
).setAttribute(\x22\
width\x22,\x220\x22),\x22lef\
t\x22===e.side&&r.s\
etAttribute(\x22lsp\
ace\x22,\x22-1width\x22),\
r.setAttribute(\x22\
voffset\x22,\x220.7em\x22\
),(r=new Tt.Math\
Node(\x22mstyle\x22,[r\
])).setAttribute\
(\x22displaystyle\x22,\
\x22false\x22),r.setAt\
tribute(\x22scriptl\
evel\x22,\x221\x22),r}}),\
ot({type:\x22cdlabe\
lparent\x22,names:[\
\x22\x5c\x5c\x5c\x5ccdparent\x22],\
props:{numArgs:1\
},handler:functi\
on(e,t){return{t\
ype:\x22cdlabelpare\
nt\x22,mode:e.parse\
r.mode,fragment:\
t[0]}},htmlBuild\
er:function(e,t)\
{var r=Ke.wrapFr\
agment(wt(e.frag\
ment,t),t);retur\
n r.classes.push\
(\x22cd-vert-arrow\x22\
),r},mathmlBuild\
er:function(e,t)\
{return new Tt.M\
athNode(\x22mrow\x22,[\
Rt(e.fragment,t)\
])}}),ot({type:\x22\
textord\x22,names:[\
\x22\x5c\x5c@char\x22],props\
:{numArgs:1,allo\
wedInText:!0},ha\
ndler:function(e\
,t){for(var r=e.\
parser,a=Ut(t[0]\
,\x22ordgroup\x22).bod\
y,i=\x22\x22,o=0;o<a.l\
ength;o++){i+=Ut\
(a[o],\x22textord\x22)\
.text}var s,l=pa\
rseInt(i);if(isN\
aN(l))throw new \
n(\x22\x5c\x5c@char has n\
on-numeric argum\
ent \x22+i);if(l<0|\
|l>=1114111)thro\
w new n(\x22\x5c\x5c@char\
 with invalid co\
de point \x22+i);re\
turn l<=65535?s=\
String.fromCharC\
ode(l):(l-=65536\
,s=String.fromCh\
arCode(55296+(l>\
>10),56320+(1023\
&l))),{type:\x22tex\
tord\x22,mode:r.mod\
e,text:s}}});var\
 nr=function(e,t\
){var r=ft(e.bod\
y,t.withColor(e.\
color),!1);retur\
n Ke.makeFragmen\
t(r)},ar=functio\
n(e,t){var r=qt(\
e.body,t.withCol\
or(e.color)),n=n\
ew Tt.MathNode(\x22\
mstyle\x22,r);retur\
n n.setAttribute\
(\x22mathcolor\x22,e.c\
olor),n};ot({typ\
e:\x22color\x22,names:\
[\x22\x5c\x5ctextcolor\x22],\
props:{numArgs:2\
,allowedInText:!\
0,argTypes:[\x22col\
or\x22,\x22original\x22]}\
,handler:functio\
n(e,t){var r=e.p\
arser,n=Ut(t[0],\
\x22color-token\x22).c\
olor,a=t[1];retu\
rn{type:\x22color\x22,\
mode:r.mode,colo\
r:n,body:ht(a)}}\
,htmlBuilder:nr,\
mathmlBuilder:ar\
}),ot({type:\x22col\
or\x22,names:[\x22\x5c\x5cco\
lor\x22],props:{num\
Args:1,allowedIn\
Text:!0,argTypes\
:[\x22color\x22]},hand\
ler:function(e,t\
){var r=e.parser\
,n=e.breakOnToke\
nText,a=Ut(t[0],\
\x22color-token\x22).c\
olor;r.gullet.ma\
cros.set(\x22\x5c\x5ccurr\
ent@color\x22,a);va\
r i=r.parseExpre\
ssion(!0,n);retu\
rn{type:\x22color\x22,\
mode:r.mode,colo\
r:a,body:i}},htm\
lBuilder:nr,math\
mlBuilder:ar}),o\
t({type:\x22cr\x22,nam\
es:[\x22\x5c\x5c\x5c\x5c\x22],prop\
s:{numArgs:0,num\
OptionalArgs:0,a\
llowedInText:!0}\
,handler:functio\
n(e,t,r){var n=e\
.parser,a=\x22[\x22===\
n.gullet.future(\
).text?n.parseSi\
zeGroup(!0):null\
,i=!n.settings.d\
isplayMode||!n.s\
ettings.useStric\
tBehavior(\x22newLi\
neInDisplayMode\x22\
,\x22In LaTeX, \x5c\x5c\x5c\x5c\
 or \x5c\x5cnewline do\
es nothing in di\
splay mode\x22);ret\
urn{type:\x22cr\x22,mo\
de:n.mode,newLin\
e:i,size:a&&Ut(a\
,\x22size\x22).value}}\
,htmlBuilder:fun\
ction(e,t){var r\
=Ke.makeSpan([\x22m\
space\x22],[],t);re\
turn e.newLine&&\
(r.classes.push(\
\x22newline\x22),e.siz\
e&&(r.style.marg\
inTop=F(P(e.size\
,t)))),r},mathml\
Builder:function\
(e,t){var r=new \
Tt.MathNode(\x22msp\
ace\x22);return e.n\
ewLine&&(r.setAt\
tribute(\x22linebre\
ak\x22,\x22newline\x22),e\
.size&&r.setAttr\
ibute(\x22height\x22,F\
(P(e.size,t)))),\
r}});var ir={\x22\x5c\x5c\
global\x22:\x22\x5c\x5cgloba\
l\x22,\x22\x5c\x5clong\x22:\x22\x5c\x5c\x5c\
\x5cgloballong\x22,\x22\x5c\x5c\
\x5c\x5cgloballong\x22:\x22\x5c\
\x5c\x5c\x5cgloballong\x22,\x22\
\x5c\x5cdef\x22:\x22\x5c\x5cgdef\x22,\
\x22\x5c\x5cgdef\x22:\x22\x5c\x5cgdef\
\x22,\x22\x5c\x5cedef\x22:\x22\x5c\x5cxd\
ef\x22,\x22\x5c\x5cxdef\x22:\x22\x5c\x5c\
xdef\x22,\x22\x5c\x5clet\x22:\x22\x5c\
\x5c\x5c\x5cgloballet\x22,\x22\x5c\
\x5cfuturelet\x22:\x22\x5c\x5c\x5c\
\x5cglobalfuture\x22},\
or=function(e){v\
ar t=e.text;if(/\
^(?:[\x5c\x5c{}$&#^_]|\
EOF)$/.test(t))t\
hrow new n(\x22Expe\
cted a control s\
equence\x22,e);retu\
rn t},sr=functio\
n(e,t,r,n){var a\
=e.gullet.macros\
.get(r.text);nul\
l==a&&(r.noexpan\
d=!0,a={tokens:[\
r],numArgs:0,une\
xpandable:!e.gul\
let.isExpandable\
(r.text)}),e.gul\
let.macros.set(t\
,a,n)};ot({type:\
\x22internal\x22,names\
:[\x22\x5c\x5cglobal\x22,\x22\x5c\x5c\
long\x22,\x22\x5c\x5c\x5c\x5cgloba\
llong\x22],props:{n\
umArgs:0,allowed\
InText:!0},handl\
er:function(e){v\
ar t=e.parser,r=\
e.funcName;t.con\
sumeSpaces();var\
 a=t.fetch();if(\
ir[a.text])retur\
n\x22\x5c\x5cglobal\x22!==r&\
&\x22\x5c\x5c\x5c\x5cgloballong\
\x22!==r||(a.text=i\
r[a.text]),Ut(t.\
parseFunction(),\
\x22internal\x22);thro\
w new n(\x22Invalid\
 token after mac\
ro prefix\x22,a)}})\
,ot({type:\x22inter\
nal\x22,names:[\x22\x5c\x5cd\
ef\x22,\x22\x5c\x5cgdef\x22,\x22\x5c\x5c\
edef\x22,\x22\x5c\x5cxdef\x22],\
props:{numArgs:0\
,allowedInText:!\
0,primitive:!0},\
handler:function\
(e){var t=e.pars\
er,r=e.funcName,\
a=t.gullet.popTo\
ken(),i=a.text;i\
f(/^(?:[\x5c\x5c{}$&#^\
_]|EOF)$/.test(i\
))throw new n(\x22E\
xpected a contro\
l sequence\x22,a);f\
or(var o,s=0,l=[\
[]];\x22{\x22!==t.gull\
et.future().text\
;)if(\x22#\x22===(a=t.\
gullet.popToken(\
)).text){if(\x22{\x22=\
==t.gullet.futur\
e().text){o=t.gu\
llet.future(),l[\
s].push(\x22{\x22);bre\
ak}if(a=t.gullet\
.popToken(),!/^[\
1-9]$/.test(a.te\
xt))throw new n(\
'Invalid argumen\
t number \x22'+a.te\
xt+'\x22');if(parse\
Int(a.text)!==s+\
1)throw new n('A\
rgument number \x22\
'+a.text+'\x22 out \
of order');s++,l\
.push([])}else{i\
f(\x22EOF\x22===a.text\
)throw new n(\x22Ex\
pected a macro d\
efinition\x22);l[s]\
.push(a.text)}va\
r h=t.gullet.con\
sumeArg().tokens\
;return o&&h.uns\
hift(o),\x22\x5c\x5cedef\x22\
!==r&&\x22\x5c\x5cxdef\x22!=\
=r||(h=t.gullet.\
expandTokens(h))\
.reverse(),t.gul\
let.macros.set(i\
,{tokens:h,numAr\
gs:s,delimiters:\
l},r===ir[r]),{t\
ype:\x22internal\x22,m\
ode:t.mode}}}),o\
t({type:\x22interna\
l\x22,names:[\x22\x5c\x5clet\
\x22,\x22\x5c\x5c\x5c\x5cgloballet\
\x22],props:{numArg\
s:0,allowedInTex\
t:!0,primitive:!\
0},handler:funct\
ion(e){var t=e.p\
arser,r=e.funcNa\
me,n=or(t.gullet\
.popToken());t.g\
ullet.consumeSpa\
ces();var a=func\
tion(e){var t=e.\
gullet.popToken(\
);return\x22=\x22===t.\
text&&\x22 \x22===(t=e\
.gullet.popToken\
()).text&&(t=e.g\
ullet.popToken()\
),t}(t);return s\
r(t,n,a,\x22\x5c\x5c\x5c\x5cglo\
ballet\x22===r),{ty\
pe:\x22internal\x22,mo\
de:t.mode}}}),ot\
({type:\x22internal\
\x22,names:[\x22\x5c\x5cfutu\
relet\x22,\x22\x5c\x5c\x5c\x5cglob\
alfuture\x22],props\
:{numArgs:0,allo\
wedInText:!0,pri\
mitive:!0},handl\
er:function(e){v\
ar t=e.parser,r=\
e.funcName,n=or(\
t.gullet.popToke\
n()),a=t.gullet.\
popToken(),i=t.g\
ullet.popToken()\
;return sr(t,n,i\
,\x22\x5c\x5c\x5c\x5cglobalfutu\
re\x22===r),t.gulle\
t.pushToken(i),t\
.gullet.pushToke\
n(a),{type:\x22inte\
rnal\x22,mode:t.mod\
e}}});var lr=fun\
ction(e,t,r){var\
 n=N(ae.math[e]&\
&ae.math[e].repl\
ace||e,t,r);if(!\
n)throw new Erro\
r(\x22Unsupported s\
ymbol \x22+e+\x22 and \
font size \x22+t+\x22.\
\x22);return n},hr=\
function(e,t,r,n\
){var a=r.having\
BaseStyle(t),i=K\
e.makeSpan(n.con\
cat(a.sizingClas\
ses(r)),[e],r),o\
=a.sizeMultiplie\
r/r.sizeMultipli\
er;return i.heig\
ht*=o,i.depth*=o\
,i.maxFontSize=a\
.sizeMultiplier,\
i},cr=function(e\
,t,r){var n=t.ha\
vingBaseStyle(r)\
,a=(1-t.sizeMult\
iplier/n.sizeMul\
tiplier)*t.fontM\
etrics().axisHei\
ght;e.classes.pu\
sh(\x22delimcenter\x22\
),e.style.top=F(\
a),e.height-=a,e\
.depth+=a},mr=fu\
nction(e,t,r,n,a\
,i){var o=functi\
on(e,t,r,n){retu\
rn Ke.makeSymbol\
(e,\x22Size\x22+t+\x22-Re\
gular\x22,r,n)}(e,t\
,a,n),s=hr(Ke.ma\
keSpan([\x22delimsi\
zing\x22,\x22size\x22+t],\
[o],n),x.TEXT,n,\
i);return r&&cr(\
s,n,x.TEXT),s},u\
r=function(e,t,r\
){var n;return n\
=\x22Size1-Regular\x22\
===t?\x22delim-size\
1\x22:\x22delim-size4\x22\
,{type:\x22elem\x22,el\
em:Ke.makeSpan([\
\x22delimsizinginne\
r\x22,n],[Ke.makeSp\
an([],[Ke.makeSy\
mbol(e,t,r)])])}\
},pr=function(e,\
t,r){var n=T[\x22Si\
ze4-Regular\x22][e.\
charCodeAt(0)]?T\
[\x22Size4-Regular\x22\
][e.charCodeAt(0\
)][4]:T[\x22Size1-R\
egular\x22][e.charC\
odeAt(0)][4],a=n\
ew J(\x22inner\x22,fun\
ction(e,t){switc\
h(e){case\x22\x5cu239c\
\x22:return\x22M291 0 \
H417 V\x22+t+\x22 H291\
z M291 0 H417 V\x22\
+t+\x22 H291z\x22;case\
\x22\x5cu2223\x22:return\x22\
M145 0 H188 V\x22+t\
+\x22 H145z M145 0 \
H188 V\x22+t+\x22 H145\
z\x22;case\x22\x5cu2225\x22:\
return\x22M145 0 H1\
88 V\x22+t+\x22 H145z \
M145 0 H188 V\x22+t\
+\x22 H145zM367 0 H\
410 V\x22+t+\x22 H367z\
 M367 0 H410 V\x22+\
t+\x22 H367z\x22;case\x22\
\x5cu239f\x22:return\x22M\
457 0 H583 V\x22+t+\
\x22 H457z M457 0 H\
583 V\x22+t+\x22 H457z\
\x22;case\x22\x5cu23a2\x22:r\
eturn\x22M319 0 H40\
3 V\x22+t+\x22 H319z M\
319 0 H403 V\x22+t+\
\x22 H319z\x22;case\x22\x5cu\
23a5\x22:return\x22M26\
3 0 H347 V\x22+t+\x22 \
H263z M263 0 H34\
7 V\x22+t+\x22 H263z\x22;\
case\x22\x5cu23aa\x22:ret\
urn\x22M384 0 H504 \
V\x22+t+\x22 H384z M38\
4 0 H504 V\x22+t+\x22 \
H384z\x22;case\x22\x5cu23\
d0\x22:return\x22M312 \
0 H355 V\x22+t+\x22 H3\
12z M312 0 H355 \
V\x22+t+\x22 H312z\x22;ca\
se\x22\x5cu2016\x22:retur\
n\x22M257 0 H300 V\x22\
+t+\x22 H257z M257 \
0 H300 V\x22+t+\x22 H2\
57zM478 0 H521 V\
\x22+t+\x22 H478z M478\
 0 H521 V\x22+t+\x22 H\
478z\x22;default:re\
turn\x22\x22}}(e,Math.\
round(1e3*t))),i\
=new K([a],{widt\
h:F(n),height:F(\
t),style:\x22width:\
\x22+F(n),viewBox:\x22\
0 0 \x22+1e3*n+\x22 \x22+\
Math.round(1e3*t\
),preserveAspect\
Ratio:\x22xMinYMin\x22\
}),o=Ke.makeSvgS\
pan([],[i],r);re\
turn o.height=t,\
o.style.height=F\
(t),o.style.widt\
h=F(n),{type:\x22el\
em\x22,elem:o}},dr=\
{type:\x22kern\x22,siz\
e:-.008},fr=[\x22|\x22\
,\x22\x5c\x5clvert\x22,\x22\x5c\x5crv\
ert\x22,\x22\x5c\x5cvert\x22],g\
r=[\x22\x5c\x5c|\x22,\x22\x5c\x5clVer\
t\x22,\x22\x5c\x5crVert\x22,\x22\x5c\x5c\
Vert\x22],vr=functi\
on(e,t,r,n,a,i){\
var o,s,h,c,m=\x22\x22\
,u=0;o=h=c=e,s=n\
ull;var p=\x22Size1\
-Regular\x22;\x22\x5c\x5cupa\
rrow\x22===e?h=c=\x22\x5c\
u23d0\x22:\x22\x5c\x5cUparro\
w\x22===e?h=c=\x22\x5cu20\
16\x22:\x22\x5c\x5cdownarrow\
\x22===e?o=h=\x22\x5cu23d\
0\x22:\x22\x5c\x5cDownarrow\x22\
===e?o=h=\x22\x5cu2016\
\x22:\x22\x5c\x5cupdownarrow\
\x22===e?(o=\x22\x5c\x5cupar\
row\x22,h=\x22\x5cu23d0\x22,\
c=\x22\x5c\x5cdownarrow\x22)\
:\x22\x5c\x5cUpdownarrow\x22\
===e?(o=\x22\x5c\x5cUparr\
ow\x22,h=\x22\x5cu2016\x22,c\
=\x22\x5c\x5cDownarrow\x22):\
l.contains(fr,e)\
?(h=\x22\x5cu2223\x22,m=\x22\
vert\x22,u=333):l.c\
ontains(gr,e)?(h\
=\x22\x5cu2225\x22,m=\x22dou\
blevert\x22,u=556):\
\x22[\x22===e||\x22\x5c\x5clbra\
ck\x22===e?(o=\x22\x5cu23\
a1\x22,h=\x22\x5cu23a2\x22,c\
=\x22\x5cu23a3\x22,p=\x22Siz\
e4-Regular\x22,m=\x22l\
brack\x22,u=667):\x22]\
\x22===e||\x22\x5c\x5crbrack\
\x22===e?(o=\x22\x5cu23a4\
\x22,h=\x22\x5cu23a5\x22,c=\x22\
\x5cu23a6\x22,p=\x22Size4\
-Regular\x22,m=\x22rbr\
ack\x22,u=667):\x22\x5c\x5cl\
floor\x22===e||\x22\x5cu2\
30a\x22===e?(h=o=\x22\x5c\
u23a2\x22,c=\x22\x5cu23a3\
\x22,p=\x22Size4-Regul\
ar\x22,m=\x22lfloor\x22,u\
=667):\x22\x5c\x5clceil\x22=\
==e||\x22\x5cu2308\x22===\
e?(o=\x22\x5cu23a1\x22,h=\
c=\x22\x5cu23a2\x22,p=\x22Si\
ze4-Regular\x22,m=\x22\
lceil\x22,u=667):\x22\x5c\
\x5crfloor\x22===e||\x22\x5c\
u230b\x22===e?(h=o=\
\x22\x5cu23a5\x22,c=\x22\x5cu23\
a6\x22,p=\x22Size4-Reg\
ular\x22,m=\x22rfloor\x22\
,u=667):\x22\x5c\x5crceil\
\x22===e||\x22\x5cu2309\x22=\
==e?(o=\x22\x5cu23a4\x22,\
h=c=\x22\x5cu23a5\x22,p=\x22\
Size4-Regular\x22,m\
=\x22rceil\x22,u=667):\
\x22(\x22===e||\x22\x5c\x5clpar\
en\x22===e?(o=\x22\x5cu23\
9b\x22,h=\x22\x5cu239c\x22,c\
=\x22\x5cu239d\x22,p=\x22Siz\
e4-Regular\x22,m=\x22l\
paren\x22,u=875):\x22)\
\x22===e||\x22\x5c\x5crparen\
\x22===e?(o=\x22\x5cu239e\
\x22,h=\x22\x5cu239f\x22,c=\x22\
\x5cu23a0\x22,p=\x22Size4\
-Regular\x22,m=\x22rpa\
ren\x22,u=875):\x22\x5c\x5c{\
\x22===e||\x22\x5c\x5clbrace\
\x22===e?(o=\x22\x5cu23a7\
\x22,s=\x22\x5cu23a8\x22,c=\x22\
\x5cu23a9\x22,h=\x22\x5cu23a\
a\x22,p=\x22Size4-Regu\
lar\x22):\x22\x5c\x5c}\x22===e|\
|\x22\x5c\x5crbrace\x22===e?\
(o=\x22\x5cu23ab\x22,s=\x22\x5c\
u23ac\x22,c=\x22\x5cu23ad\
\x22,h=\x22\x5cu23aa\x22,p=\x22\
Size4-Regular\x22):\
\x22\x5c\x5clgroup\x22===e||\
\x22\x5cu27ee\x22===e?(o=\
\x22\x5cu23a7\x22,c=\x22\x5cu23\
a9\x22,h=\x22\x5cu23aa\x22,p\
=\x22Size4-Regular\x22\
):\x22\x5c\x5crgroup\x22===e\
||\x22\x5cu27ef\x22===e?(\
o=\x22\x5cu23ab\x22,c=\x22\x5cu\
23ad\x22,h=\x22\x5cu23aa\x22\
,p=\x22Size4-Regula\
r\x22):\x22\x5c\x5clmoustach\
e\x22===e||\x22\x5cu23b0\x22\
===e?(o=\x22\x5cu23a7\x22\
,c=\x22\x5cu23ad\x22,h=\x22\x5c\
u23aa\x22,p=\x22Size4-\
Regular\x22):\x22\x5c\x5crmo\
ustache\x22!==e&&\x22\x5c\
u23b1\x22!==e||(o=\x22\
\x5cu23ab\x22,c=\x22\x5cu23a\
9\x22,h=\x22\x5cu23aa\x22,p=\
\x22Size4-Regular\x22)\
;var d=lr(o,p,a)\
,f=d.height+d.de\
pth,g=lr(h,p,a),\
v=g.height+g.dep\
th,b=lr(c,p,a),y\
=b.height+b.dept\
h,w=0,k=1;if(nul\
l!==s){var S=lr(\
s,p,a);w=S.heigh\
t+S.depth,k=2}va\
r M=f+y+w,z=M+Ma\
th.max(0,Math.ce\
il((t-M)/(k*v)))\
*k*v,A=n.fontMet\
rics().axisHeigh\
t;r&&(A*=n.sizeM\
ultiplier);var T\
=z/2-A,B=[];if(m\
.length>0){var C\
=z-f-y,N=Math.ro\
und(1e3*z),q=fun\
ction(e,t){switc\
h(e){case\x22lbrack\
\x22:return\x22M403 17\
59 V84 H666 V0 H\
319 V1759 v\x22+t+\x22\
 v1759 h347 v-84\
\x5cnH403z M403 175\
9 V0 H319 V1759 \
v\x22+t+\x22 v1759 h84\
z\x22;case\x22rbrack\x22:\
return\x22M347 1759\
 V0 H0 V84 H263 \
V1759 v\x22+t+\x22 v17\
59 H0 v84 H347z\x5c\
nM347 1759 V0 H2\
63 V1759 v\x22+t+\x22 \
v1759 h84z\x22;case\
\x22vert\x22:return\x22M1\
45 15 v585 v\x22+t+\
\x22 v585 c2.667,10\
,9.667,15,21,15\x5c\
nc10,0,16.667,-5\
,20,-15 v-585 v\x22\
+-t+\x22 v-585 c-2.\
667,-10,-9.667,-\
15,-21,-15\x5cnc-10\
,0,-16.667,5,-20\
,15z M188 15 H14\
5 v585 v\x22+t+\x22 v5\
85 h43z\x22;case\x22do\
ublevert\x22:return\
\x22M145 15 v585 v\x22\
+t+\x22 v585 c2.667\
,10,9.667,15,21,\
15\x5cnc10,0,16.667\
,-5,20,-15 v-585\
 v\x22+-t+\x22 v-585 c\
-2.667,-10,-9.66\
7,-15,-21,-15\x5cnc\
-10,0,-16.667,5,\
-20,15z M188 15 \
H145 v585 v\x22+t+\x22\
 v585 h43z\x5cnM367\
 15 v585 v\x22+t+\x22 \
v585 c2.667,10,9\
.667,15,21,15\x5cnc\
10,0,16.667,-5,2\
0,-15 v-585 v\x22+-\
t+\x22 v-585 c-2.66\
7,-10,-9.667,-15\
,-21,-15\x5cnc-10,0\
,-16.667,5,-20,1\
5z M410 15 H367 \
v585 v\x22+t+\x22 v585\
 h43z\x22;case\x22lflo\
or\x22:return\x22M319 \
602 V0 H403 V602\
 v\x22+t+\x22 v1715 h2\
63 v84 H319z\x5cnMM\
319 602 V0 H403 \
V602 v\x22+t+\x22 v171\
5 H319z\x22;case\x22rf\
loor\x22:return\x22M31\
9 602 V0 H403 V6\
02 v\x22+t+\x22 v1799 \
H0 v-84 H319z\x5cnM\
M319 602 V0 H403\
 V602 v\x22+t+\x22 v17\
15 H319z\x22;case\x22l\
ceil\x22:return\x22M40\
3 1759 V84 H666 \
V0 H319 V1759 v\x22\
+t+\x22 v602 h84z\x5cn\
M403 1759 V0 H31\
9 V1759 v\x22+t+\x22 v\
602 h84z\x22;case\x22r\
ceil\x22:return\x22M34\
7 1759 V0 H0 V84\
 H263 V1759 v\x22+t\
+\x22 v602 h84z\x5cnM3\
47 1759 V0 h-84 \
V1759 v\x22+t+\x22 v60\
2 h84z\x22;case\x22lpa\
ren\x22:return\x22M863\
,9c0,-2,-2,-5,-6\
,-9c0,0,-17,0,-1\
7,0c-12.7,0,-19.\
3,0.3,-20,1\x5cnc-5\
.3,5.3,-10.3,11,\
-15,17c-242.7,29\
4.7,-395.3,682,-\
458,1162c-21.3,1\
63.3,-33.3,349,\x5c\
n-36,557 l0,\x22+(t\
+84)+\x22c0.2,6,0,2\
6,0,60c2,159.3,1\
0,310.7,24,454c5\
3.3,528,210,\x5cn94\
9.7,470,1265c4.7\
,6,9.7,11.7,15,1\
7c0.7,0.7,7,1,19\
,1c0,0,18,0,18,0\
c4,-4,6,-7,6,-9\x5c\
nc0,-2.7,-3.3,-8\
.7,-10,-18c-135.\
3,-192.7,-235.5,\
-414.3,-300.5,-6\
65c-65,-250.7,-1\
02.5,\x5cn-544.7,-1\
12.5,-882c-2,-10\
4,-3,-167,-3,-18\
9\x5cnl0,-\x22+(t+92)+\
\x22c0,-162.7,5.7,-\
314,17,-454c20.7\
,-272,63.7,-513,\
129,-723c65.3,\x5cn\
-210,155.3,-396.\
3,270,-559c6.7,-\
9.3,10,-15.3,10,\
-18z\x22;case\x22rpare\
n\x22:return\x22M76,0c\
-16.7,0,-25,3,-2\
5,9c0,2,2,6.3,6,\
13c21.3,28.7,42.\
3,60.3,\x5cn63,95c9\
6.7,156.7,172.8,\
332.5,228.5,527.\
5c55.7,195,92.8,\
416.5,111.5,664.\
5\x5cnc11.3,139.3,1\
7,290.7,17,454c0\
,28,1.7,43,3.3,4\
5l0,\x22+(t+9)+\x22\x5cnc\
-3,4,-3.3,16.7,-\
3.3,38c0,162,-5.\
7,313.7,-17,455c\
-18.7,248,-55.8,\
469.3,-111.5,664\
\x5cnc-55.7,194.7,-\
131.8,370.3,-228\
.5,527c-20.7,34.\
7,-41.7,66.3,-63\
,95c-2,3.3,-4,7,\
-6,11\x5cnc0,7.3,5.\
7,11,17,11c0,0,1\
1,0,11,0c9.3,0,1\
4.3,-0.3,15,-1c5\
.3,-5.3,10.3,-11\
,15,-17\x5cnc242.7,\
-294.7,395.3,-68\
1.7,458,-1161c21\
.3,-164.7,33.3,-\
350.7,36,-558\x5cnl\
0,-\x22+(t+144)+\x22c-\
2,-159.3,-10,-31\
0.7,-24,-454c-53\
.3,-528,-210,-94\
9.7,\x5cn-470,-1265\
c-4.7,-6,-9.7,-1\
1.7,-15,-17c-0.7\
,-0.7,-6.7,-1,-1\
8,-1z\x22;default:t\
hrow new Error(\x22\
Unknown stretchy\
 delimiter.\x22)}}(\
m,Math.round(1e3\
*C)),I=new J(m,q\
),R=(u/1e3).toFi\
xed(3)+\x22em\x22,H=(N\
/1e3).toFixed(3)\
+\x22em\x22,O=new K([I\
],{width:R,heigh\
t:H,viewBox:\x220 0\
 \x22+u+\x22 \x22+N}),E=K\
e.makeSvgSpan([]\
,[O],n);E.height\
=N/1e3,E.style.w\
idth=R,E.style.h\
eight=H,B.push({\
type:\x22elem\x22,elem\
:E})}else{if(B.p\
ush(ur(c,p,a)),B\
.push(dr),null==\
=s){var L=z-f-y+\
.016;B.push(pr(h\
,L,n))}else{var \
D=(z-f-y-w)/2+.0\
16;B.push(pr(h,D\
,n)),B.push(dr),\
B.push(ur(s,p,a)\
),B.push(dr),B.p\
ush(pr(h,D,n))}B\
.push(dr),B.push\
(ur(o,p,a))}var \
V=n.havingBaseSt\
yle(x.TEXT),P=Ke\
.makeVList({posi\
tionType:\x22bottom\
\x22,positionData:T\
,children:B},V);\
return hr(Ke.mak\
eSpan([\x22delimsiz\
ing\x22,\x22mult\x22],[P]\
,V),x.TEXT,n,i)}\
,br=.08,yr=funct\
ion(e,t,r,n,a){v\
ar i=function(e,\
t,r){t*=1e3;var \
n=\x22\x22;switch(e){c\
ase\x22sqrtMain\x22:n=\
function(e,t){re\
turn\x22M95,\x22+(622+\
e+t)+\x22\x5cnc-2.7,0,\
-7.17,-2.7,-13.5\
,-8c-5.8,-5.3,-9\
.5,-10,-9.5,-14\x5c\
nc0,-2,0.3,-3.3,\
1,-4c1.3,-2.7,23\
.83,-20.7,67.5,-\
54\x5cnc44.2,-33.3,\
65.8,-50.3,66.5,\
-51c1.3,-1.3,3,-\
2,5,-2c4.7,0,8.7\
,3.3,12,10\x5cns173\
,378,173,378c0.7\
,0,35.3,-71,104,\
-213c68.7,-142,1\
37.5,-285,206.5,\
-429\x5cnc69,-144,1\
04.5,-217.7,106.\
5,-221\x5cnl\x22+e/2.0\
75+\x22 -\x22+e+\x22\x5cnc5.\
3,-9.3,12,-14,20\
,-14\x5cnH400000v\x22+\
(40+e)+\x22H845.272\
4\x5cns-225.272,467\
,-225.272,467s-2\
35,486,-235,486c\
-2.7,4.7,-9,7,-1\
9,7\x5cnc-6,0,-10,-\
1,-12,-3s-194,-4\
22,-194,-422s-65\
,47,-65,47z\x5cnM\x22+\
(834+e)+\x22 \x22+t+\x22h\
400000v\x22+(40+e)+\
\x22h-400000z\x22}(t,M\
);break;case\x22sqr\
tSize1\x22:n=functi\
on(e,t){return\x22M\
263,\x22+(601+e+t)+\
\x22c0.7,0,18,39.7,\
52,119\x5cnc34,79.3\
,68.167,158.7,10\
2.5,238c34.3,79.\
3,51.8,119.3,52.\
5,120\x5cnc340,-704\
.7,510.7,-1060.3\
,512,-1067\x5cnl\x22+e\
/2.084+\x22 -\x22+e+\x22\x5c\
nc4.7,-7.3,11,-1\
1,19,-11\x5cnH40000\
v\x22+(40+e)+\x22H1012\
.3\x5cns-271.3,567,\
-271.3,567c-38.7\
,80.7,-84,175,-1\
36,283c-52,108,-\
89.167,185.3,-11\
1.5,232\x5cnc-22.3,\
46.7,-33.8,70.3,\
-34.5,71c-4.7,4.\
7,-12.3,7,-23,7s\
-12,-1,-12,-1\x5cns\
-109,-253,-109,-\
253c-72.7,-168,-\
109.3,-252,-110,\
-252c-10.7,8,-22\
,16.7,-34,26\x5cnc-\
22,17.3,-33.3,26\
,-34,26s-26,-26,\
-26,-26s76,-59,7\
6,-59s76,-60,76,\
-60z\x5cnM\x22+(1001+e\
)+\x22 \x22+t+\x22h400000\
v\x22+(40+e)+\x22h-400\
000z\x22}(t,M);brea\
k;case\x22sqrtSize2\
\x22:n=function(e,t\
){return\x22M983 \x22+\
(10+e+t)+\x22\x5cnl\x22+e\
/3.13+\x22 -\x22+e+\x22\x5cn\
c4,-6.7,10,-10,1\
8,-10 H400000v\x22+\
(40+e)+\x22\x5cnH1013.\
1s-83.4,268,-264\
.1,840c-180.7,57\
2,-277,876.3,-28\
9,913c-4.7,4.7,-\
12.7,7,-24,7\x5cns-\
12,0,-12,0c-1.3,\
-3.3,-3.7,-11.7,\
-7,-25c-35.3,-12\
5.3,-106.7,-373.\
3,-214,-744\x5cnc-1\
0,12,-21,25,-33,\
39s-32,39,-32,39\
c-6,-5.3,-15,-14\
,-27,-26s25,-30,\
25,-30\x5cnc26.7,-3\
2.7,52,-63,76,-9\
1s52,-60,52,-60s\
208,722,208,722\x5c\
nc56,-175.3,126.\
3,-397.3,211,-66\
6c84.7,-268.7,15\
3.8,-488.2,207.5\
,-658.5\x5cnc53.7,-\
170.3,84.5,-266.\
8,92.5,-289.5z\x5cn\
M\x22+(1001+e)+\x22 \x22+\
t+\x22h400000v\x22+(40\
+e)+\x22h-400000z\x22}\
(t,M);break;case\
\x22sqrtSize3\x22:n=fu\
nction(e,t){retu\
rn\x22M424,\x22+(2398+\
e+t)+\x22\x5cnc-1.3,-0\
.7,-38.5,-172,-1\
11.5,-514c-73,-3\
42,-109.8,-513.3\
,-110.5,-514\x5cnc0\
,-2,-10.7,14.3,-\
32,49c-4.7,7.3,-\
9.8,15.7,-15.5,2\
5c-5.7,9.3,-9.8,\
16,-12.5,20\x5cns-5\
,7,-5,7c-4,-3.3,\
-8.3,-7.7,-13,-1\
3s-13,-13,-13,-1\
3s76,-122,76,-12\
2s77,-121,77,-12\
1\x5cns209,968,209,\
968c0,-2,84.7,-3\
61.7,254,-1079c1\
69.3,-717.3,254.\
7,-1077.7,256,-1\
081\x5cnl\x22+e/4.223+\
\x22 -\x22+e+\x22c4,-6.7,\
10,-10,18,-10 H4\
00000\x5cnv\x22+(40+e)\
+\x22H1014.6\x5cns-87.\
3,378.7,-272.6,1\
166c-185.3,787.3\
,-279.3,1182.3,-\
282,1185\x5cnc-2,6,\
-10,9,-24,9\x5cnc-8\
,0,-12,-0.7,-12,\
-2z M\x22+(1001+e)+\
\x22 \x22+t+\x22\x5cnh400000\
v\x22+(40+e)+\x22h-400\
000z\x22}(t,M);brea\
k;case\x22sqrtSize4\
\x22:n=function(e,t\
){return\x22M473,\x22+\
(2713+e+t)+\x22\x5cnc3\
39.3,-1799.3,509\
.3,-2700,510,-27\
02 l\x22+e/5.298+\x22 \
-\x22+e+\x22\x5cnc3.3,-7.\
3,9.3,-11,18,-11\
 H400000v\x22+(40+e\
)+\x22H1017.7\x5cns-90\
.5,478,-276.2,14\
66c-185.7,988,-2\
79.5,1483,-281.5\
,1485c-2,6,-10,9\
,-24,9\x5cnc-8,0,-1\
2,-0.7,-12,-2c0,\
-1.3,-5.3,-32,-1\
6,-92c-50.7,-293\
.3,-119.7,-693.3\
,-207,-1200\x5cnc0,\
-1.3,-5.3,8.7,-1\
6,30c-10.7,21.3,\
-21.3,42.7,-32,6\
4s-16,33,-16,33s\
-26,-26,-26,-26\x5c\
ns76,-153,76,-15\
3s77,-151,77,-15\
1c0.7,0.7,35.7,2\
02,105,604c67.3,\
400.7,102,602.7,\
104,\x5cn606zM\x22+(10\
01+e)+\x22 \x22+t+\x22h40\
0000v\x22+(40+e)+\x22H\
1017.7z\x22}(t,M);b\
reak;case\x22sqrtTa\
ll\x22:n=function(e\
,t,r){return\x22M70\
2 \x22+(e+t)+\x22H4000\
00\x22+(40+e)+\x22\x5cnH7\
42v\x22+(r-54-t-e)+\
\x22l-4 4-4 4c-.667\
.7 -2 1.5-4 2.5s\
-4.167 1.833-6.5\
 2.5-5.5 1-9.5 1\
\x5cnh-12l-28-84c-1\
6.667-52-96.667 \
-294.333-240-727\
l-212 -643 -85 1\
70\x5cnc-4-3.333-8.\
333-7.667-13 -13\
l-13-13l77-155 7\
7-156c66 199.333\
 139 419.667\x5cn21\
9 661 l218 661zM\
702 \x22+t+\x22H400000\
v\x22+(40+e)+\x22H742z\
\x22}(t,M,r)}return\
 n}(e,n,r),o=new\
 J(e,i),s=new K(\
[o],{width:\x22400e\
m\x22,height:F(t),v\
iewBox:\x220 0 4000\
00 \x22+r,preserveA\
spectRatio:\x22xMin\
YMin slice\x22});re\
turn Ke.makeSvgS\
pan([\x22hide-tail\x22\
],[s],a)},xr=[\x22(\
\x22,\x22\x5c\x5clparen\x22,\x22)\x22\
,\x22\x5c\x5crparen\x22,\x22[\x22,\
\x22\x5c\x5clbrack\x22,\x22]\x22,\x22\
\x5c\x5crbrack\x22,\x22\x5c\x5c{\x22,\
\x22\x5c\x5clbrace\x22,\x22\x5c\x5c}\x22\
,\x22\x5c\x5crbrace\x22,\x22\x5c\x5cl\
floor\x22,\x22\x5c\x5crfloor\
\x22,\x22\x5cu230a\x22,\x22\x5cu23\
0b\x22,\x22\x5c\x5clceil\x22,\x22\x5c\
\x5crceil\x22,\x22\x5cu2308\x22\
,\x22\x5cu2309\x22,\x22\x5c\x5csur\
d\x22],wr=[\x22\x5c\x5cuparr\
ow\x22,\x22\x5c\x5cdownarrow\
\x22,\x22\x5c\x5cupdownarrow\
\x22,\x22\x5c\x5cUparrow\x22,\x22\x5c\
\x5cDownarrow\x22,\x22\x5c\x5cU\
pdownarrow\x22,\x22|\x22,\
\x22\x5c\x5c|\x22,\x22\x5c\x5cvert\x22,\x22\
\x5c\x5cVert\x22,\x22\x5c\x5clvert\
\x22,\x22\x5c\x5crvert\x22,\x22\x5c\x5cl\
Vert\x22,\x22\x5c\x5crVert\x22,\
\x22\x5c\x5clgroup\x22,\x22\x5c\x5crg\
roup\x22,\x22\x5cu27ee\x22,\x22\
\x5cu27ef\x22,\x22\x5c\x5clmous\
tache\x22,\x22\x5c\x5crmoust\
ache\x22,\x22\x5cu23b0\x22,\x22\
\x5cu23b1\x22],kr=[\x22<\x22\
,\x22>\x22,\x22\x5c\x5clangle\x22,\
\x22\x5c\x5crangle\x22,\x22/\x22,\x22\
\x5c\x5cbackslash\x22,\x22\x5c\x5c\
lt\x22,\x22\x5c\x5cgt\x22],Sr=[\
0,1.2,1.8,2.4,3]\
,Mr=[{type:\x22smal\
l\x22,style:x.SCRIP\
TSCRIPT},{type:\x22\
small\x22,style:x.S\
CRIPT},{type:\x22sm\
all\x22,style:x.TEX\
T},{type:\x22large\x22\
,size:1},{type:\x22\
large\x22,size:2},{\
type:\x22large\x22,siz\
e:3},{type:\x22larg\
e\x22,size:4}],zr=[\
{type:\x22small\x22,st\
yle:x.SCRIPTSCRI\
PT},{type:\x22small\
\x22,style:x.SCRIPT\
},{type:\x22small\x22,\
style:x.TEXT},{t\
ype:\x22stack\x22}],Ar\
=[{type:\x22small\x22,\
style:x.SCRIPTSC\
RIPT},{type:\x22sma\
ll\x22,style:x.SCRI\
PT},{type:\x22small\
\x22,style:x.TEXT},\
{type:\x22large\x22,si\
ze:1},{type:\x22lar\
ge\x22,size:2},{typ\
e:\x22large\x22,size:3\
},{type:\x22large\x22,\
size:4},{type:\x22s\
tack\x22}],Tr=funct\
ion(e){if(\x22small\
\x22===e.type)retur\
n\x22Main-Regular\x22;\
if(\x22large\x22===e.t\
ype)return\x22Size\x22\
+e.size+\x22-Regula\
r\x22;if(\x22stack\x22===\
e.type)return\x22Si\
ze4-Regular\x22;thr\
ow new Error(\x22Ad\
d support for de\
lim type '\x22+e.ty\
pe+\x22' here.\x22)},B\
r=function(e,t,r\
,n){for(var a=Ma\
th.min(2,3-n.sty\
le.size);a<r.len\
gth&&\x22stack\x22!==r\
[a].type;a++){va\
r i=lr(e,Tr(r[a]\
),\x22math\x22),o=i.he\
ight+i.depth;if(\
\x22small\x22===r[a].t\
ype&&(o*=n.havin\
gBaseStyle(r[a].\
style).sizeMulti\
plier),o>t)retur\
n r[a]}return r[\
r.length-1]},Cr=\
function(e,t,r,n\
,a,i){var o;\x22<\x22=\
==e||\x22\x5c\x5clt\x22===e|\
|\x22\x5cu27e8\x22===e?e=\
\x22\x5c\x5clangle\x22:\x22>\x22!=\
=e&&\x22\x5c\x5cgt\x22!==e&&\
\x22\x5cu27e9\x22!==e||(e\
=\x22\x5c\x5crangle\x22),o=l\
.contains(kr,e)?\
Mr:l.contains(xr\
,e)?Ar:zr;var s=\
Br(e,t,o,n);retu\
rn\x22small\x22===s.ty\
pe?function(e,t,\
r,n,a,i){var o=K\
e.makeSymbol(e,\x22\
Main-Regular\x22,a,\
n),s=hr(o,t,n,i)\
;return r&&cr(s,\
n,t),s}(e,s.styl\
e,r,n,a,i):\x22larg\
e\x22===s.type?mr(e\
,s.size,r,n,a,i)\
:vr(e,t,r,n,a,i)\
},Nr={sqrtImage:\
function(e,t){va\
r r,n,a=t.having\
BaseSizing(),i=B\
r(\x22\x5c\x5csurd\x22,e*a.s\
izeMultiplier,Ar\
,a),o=a.sizeMult\
iplier,s=Math.ma\
x(0,t.minRuleThi\
ckness-t.fontMet\
rics().sqrtRuleT\
hickness),l=0,h=\
0,c=0;return\x22sma\
ll\x22===i.type?(e<\
1?o=1:e<1.4&&(o=\
.7),h=(1+s)/o,(r\
=yr(\x22sqrtMain\x22,l\
=(1+s+br)/o,c=1e\
3+1e3*s+80,s,t))\
.style.minWidth=\
\x220.853em\x22,n=.833\
/o):\x22large\x22===i.\
type?(c=1080*Sr[\
i.size],h=(Sr[i.\
size]+s)/o,l=(Sr\
[i.size]+s+br)/o\
,(r=yr(\x22sqrtSize\
\x22+i.size,l,c,s,t\
)).style.minWidt\
h=\x221.02em\x22,n=1/o\
):(l=e+s+br,h=e+\
s,c=Math.floor(1\
e3*e+s)+80,(r=yr\
(\x22sqrtTall\x22,l,c,\
s,t)).style.minW\
idth=\x220.742em\x22,n\
=1.056),r.height\
=h,r.style.heigh\
t=F(l),{span:r,a\
dvanceWidth:n,ru\
leWidth:(t.fontM\
etrics().sqrtRul\
eThickness+s)*o}\
},sizedDelim:fun\
ction(e,t,r,a,i)\
{if(\x22<\x22===e||\x22\x5c\x5c\
lt\x22===e||\x22\x5cu27e8\
\x22===e?e=\x22\x5c\x5clangl\
e\x22:\x22>\x22!==e&&\x22\x5c\x5cg\
t\x22!==e&&\x22\x5cu27e9\x22\
!==e||(e=\x22\x5c\x5crang\
le\x22),l.contains(\
xr,e)||l.contain\
s(kr,e))return m\
r(e,t,!1,r,a,i);\
if(l.contains(wr\
,e))return vr(e,\
Sr[t],!1,r,a,i);\
throw new n(\x22Ill\
egal delimiter: \
'\x22+e+\x22'\x22)},sizeT\
oMaxHeight:Sr,cu\
stomSizedDelim:C\
r,leftRightDelim\
:function(e,t,r,\
n,a,i){var o=n.f\
ontMetrics().axi\
sHeight*n.sizeMu\
ltiplier,s=5/n.f\
ontMetrics().ptP\
erEm,l=Math.max(\
t-o,r+o),h=Math.\
max(l/500*901,2*\
l-s);return Cr(e\
,h,!0,n,a,i)}},q\
r={\x22\x5c\x5cbigl\x22:{mcl\
ass:\x22mopen\x22,size\
:1},\x22\x5c\x5cBigl\x22:{mc\
lass:\x22mopen\x22,siz\
e:2},\x22\x5c\x5cbiggl\x22:{\
mclass:\x22mopen\x22,s\
ize:3},\x22\x5c\x5cBiggl\x22\
:{mclass:\x22mopen\x22\
,size:4},\x22\x5c\x5cbigr\
\x22:{mclass:\x22mclos\
e\x22,size:1},\x22\x5c\x5cBi\
gr\x22:{mclass:\x22mcl\
ose\x22,size:2},\x22\x5c\x5c\
biggr\x22:{mclass:\x22\
mclose\x22,size:3},\
\x22\x5c\x5cBiggr\x22:{mclas\
s:\x22mclose\x22,size:\
4},\x22\x5c\x5cbigm\x22:{mcl\
ass:\x22mrel\x22,size:\
1},\x22\x5c\x5cBigm\x22:{mcl\
ass:\x22mrel\x22,size:\
2},\x22\x5c\x5cbiggm\x22:{mc\
lass:\x22mrel\x22,size\
:3},\x22\x5c\x5cBiggm\x22:{m\
class:\x22mrel\x22,siz\
e:4},\x22\x5c\x5cbig\x22:{mc\
lass:\x22mord\x22,size\
:1},\x22\x5c\x5cBig\x22:{mcl\
ass:\x22mord\x22,size:\
2},\x22\x5c\x5cbigg\x22:{mcl\
ass:\x22mord\x22,size:\
3},\x22\x5c\x5cBigg\x22:{mcl\
ass:\x22mord\x22,size:\
4}},Ir=[\x22(\x22,\x22\x5c\x5cl\
paren\x22,\x22)\x22,\x22\x5c\x5crp\
aren\x22,\x22[\x22,\x22\x5c\x5clbr\
ack\x22,\x22]\x22,\x22\x5c\x5crbra\
ck\x22,\x22\x5c\x5c{\x22,\x22\x5c\x5clbr\
ace\x22,\x22\x5c\x5c}\x22,\x22\x5c\x5crb\
race\x22,\x22\x5c\x5clfloor\x22\
,\x22\x5c\x5crfloor\x22,\x22\x5cu2\
30a\x22,\x22\x5cu230b\x22,\x22\x5c\
\x5clceil\x22,\x22\x5c\x5crceil\
\x22,\x22\x5cu2308\x22,\x22\x5cu23\
09\x22,\x22<\x22,\x22>\x22,\x22\x5c\x5cl\
angle\x22,\x22\x5cu27e8\x22,\
\x22\x5c\x5crangle\x22,\x22\x5cu27\
e9\x22,\x22\x5c\x5clt\x22,\x22\x5c\x5cgt\
\x22,\x22\x5c\x5clvert\x22,\x22\x5c\x5cr\
vert\x22,\x22\x5c\x5clVert\x22,\
\x22\x5c\x5crVert\x22,\x22\x5c\x5clgr\
oup\x22,\x22\x5c\x5crgroup\x22,\
\x22\x5cu27ee\x22,\x22\x5cu27ef\
\x22,\x22\x5c\x5clmoustache\x22\
,\x22\x5c\x5crmoustache\x22,\
\x22\x5cu23b0\x22,\x22\x5cu23b1\
\x22,\x22/\x22,\x22\x5c\x5cbacksla\
sh\x22,\x22|\x22,\x22\x5c\x5cvert\x22\
,\x22\x5c\x5c|\x22,\x22\x5c\x5cVert\x22,\
\x22\x5c\x5cuparrow\x22,\x22\x5c\x5cU\
parrow\x22,\x22\x5c\x5cdowna\
rrow\x22,\x22\x5c\x5cDownarr\
ow\x22,\x22\x5c\x5cupdownarr\
ow\x22,\x22\x5c\x5cUpdownarr\
ow\x22,\x22.\x22];functio\
n Rr(e,t){var r=\
Xt(e);if(r&&l.co\
ntains(Ir,r.text\
))return r;throw\
 new n(r?\x22Invali\
d delimiter '\x22+r\
.text+\x22' after '\
\x22+t.funcName+\x22'\x22\
:\x22Invalid delimi\
ter type '\x22+e.ty\
pe+\x22'\x22,e)}functi\
on Hr(e){if(!e.b\
ody)throw new Er\
ror(\x22Bug: The le\
ftright ParseNod\
e wasn't fully p\
arsed.\x22)}ot({typ\
e:\x22delimsizing\x22,\
names:[\x22\x5c\x5cbigl\x22,\
\x22\x5c\x5cBigl\x22,\x22\x5c\x5cbigg\
l\x22,\x22\x5c\x5cBiggl\x22,\x22\x5c\x5c\
bigr\x22,\x22\x5c\x5cBigr\x22,\x22\
\x5c\x5cbiggr\x22,\x22\x5c\x5cBigg\
r\x22,\x22\x5c\x5cbigm\x22,\x22\x5c\x5cB\
igm\x22,\x22\x5c\x5cbiggm\x22,\x22\
\x5c\x5cBiggm\x22,\x22\x5c\x5cbig\x22\
,\x22\x5c\x5cBig\x22,\x22\x5c\x5cbigg\
\x22,\x22\x5c\x5cBigg\x22],prop\
s:{numArgs:1,arg\
Types:[\x22primitiv\
e\x22]},handler:fun\
ction(e,t){var r\
=Rr(t[0],e);retu\
rn{type:\x22delimsi\
zing\x22,mode:e.par\
ser.mode,size:qr\
[e.funcName].siz\
e,mclass:qr[e.fu\
ncName].mclass,d\
elim:r.text}},ht\
mlBuilder:functi\
on(e,t){return\x22.\
\x22===e.delim?Ke.m\
akeSpan([e.mclas\
s]):Nr.sizedDeli\
m(e.delim,e.size\
,t,e.mode,[e.mcl\
ass])},mathmlBui\
lder:function(e)\
{var t=[];\x22.\x22!==\
e.delim&&t.push(\
Bt(e.delim,e.mod\
e));var r=new Tt\
.MathNode(\x22mo\x22,t\
);\x22mopen\x22===e.mc\
lass||\x22mclose\x22==\
=e.mclass?r.setA\
ttribute(\x22fence\x22\
,\x22true\x22):r.setAt\
tribute(\x22fence\x22,\
\x22false\x22),r.setAt\
tribute(\x22stretch\
y\x22,\x22true\x22);var n\
=F(Nr.sizeToMaxH\
eight[e.size]);r\
eturn r.setAttri\
bute(\x22minsize\x22,n\
),r.setAttribute\
(\x22maxsize\x22,n),r}\
}),ot({type:\x22lef\
tright-right\x22,na\
mes:[\x22\x5c\x5cright\x22],\
props:{numArgs:1\
,primitive:!0},h\
andler:function(\
e,t){var r=e.par\
ser.gullet.macro\
s.get(\x22\x5c\x5ccurrent\
@color\x22);if(r&&\x22\
string\x22!=typeof \
r)throw new n(\x22\x5c\
\x5ccurrent@color s\
et to non-string\
 in \x5c\x5cright\x22);re\
turn{type:\x22leftr\
ight-right\x22,mode\
:e.parser.mode,d\
elim:Rr(t[0],e).\
text,color:r}}})\
,ot({type:\x22leftr\
ight\x22,names:[\x22\x5c\x5c\
left\x22],props:{nu\
mArgs:1,primitiv\
e:!0},handler:fu\
nction(e,t){var \
r=Rr(t[0],e),n=e\
.parser;++n.left\
rightDepth;var a\
=n.parseExpressi\
on(!1);--n.leftr\
ightDepth,n.expe\
ct(\x22\x5c\x5cright\x22,!1)\
;var i=Ut(n.pars\
eFunction(),\x22lef\
tright-right\x22);r\
eturn{type:\x22left\
right\x22,mode:n.mo\
de,body:a,left:r\
.text,right:i.de\
lim,rightColor:i\
.color}},htmlBui\
lder:function(e,\
t){Hr(e);for(var\
 r,n,a=ft(e.body\
,t,!0,[\x22mopen\x22,\x22\
mclose\x22]),i=0,o=\
0,s=!1,l=0;l<a.l\
ength;l++)a[l].i\
sMiddle?s=!0:(i=\
Math.max(a[l].he\
ight,i),o=Math.m\
ax(a[l].depth,o)\
);if(i*=t.sizeMu\
ltiplier,o*=t.si\
zeMultiplier,r=\x22\
.\x22===e.left?xt(t\
,[\x22mopen\x22]):Nr.l\
eftRightDelim(e.\
left,i,o,t,e.mod\
e,[\x22mopen\x22]),a.u\
nshift(r),s)for(\
var h=1;h<a.leng\
th;h++){var c=a[\
h].isMiddle;c&&(\
a[h]=Nr.leftRigh\
tDelim(c.delim,i\
,o,c.options,e.m\
ode,[]))}if(\x22.\x22=\
==e.right)n=xt(t\
,[\x22mclose\x22]);els\
e{var m=e.rightC\
olor?t.withColor\
(e.rightColor):t\
;n=Nr.leftRightD\
elim(e.right,i,o\
,m,e.mode,[\x22mclo\
se\x22])}return a.p\
ush(n),Ke.makeSp\
an([\x22minner\x22],a,\
t)},mathmlBuilde\
r:function(e,t){\
Hr(e);var r=qt(e\
.body,t);if(\x22.\x22!\
==e.left){var n=\
new Tt.MathNode(\
\x22mo\x22,[Bt(e.left,\
e.mode)]);n.setA\
ttribute(\x22fence\x22\
,\x22true\x22),r.unshi\
ft(n)}if(\x22.\x22!==e\
.right){var a=ne\
w Tt.MathNode(\x22m\
o\x22,[Bt(e.right,e\
.mode)]);a.setAt\
tribute(\x22fence\x22,\
\x22true\x22),e.rightC\
olor&&a.setAttri\
bute(\x22mathcolor\x22\
,e.rightColor),r\
.push(a)}return \
Ct(r)}}),ot({typ\
e:\x22middle\x22,names\
:[\x22\x5c\x5cmiddle\x22],pr\
ops:{numArgs:1,p\
rimitive:!0},han\
dler:function(e,\
t){var r=Rr(t[0]\
,e);if(!e.parser\
.leftrightDepth)\
throw new n(\x22\x5c\x5cm\
iddle without pr\
eceding \x5c\x5cleft\x22,\
r);return{type:\x22\
middle\x22,mode:e.p\
arser.mode,delim\
:r.text}},htmlBu\
ilder:function(e\
,t){var r;if(\x22.\x22\
===e.delim)r=xt(\
t,[]);else{r=Nr.\
sizedDelim(e.del\
im,1,t,e.mode,[]\
);var n={delim:e\
.delim,options:t\
};r.isMiddle=n}r\
eturn r},mathmlB\
uilder:function(\
e,t){var r=\x22\x5c\x5cve\
rt\x22===e.delim||\x22\
|\x22===e.delim?Bt(\
\x22|\x22,\x22text\x22):Bt(e\
.delim,e.mode),n\
=new Tt.MathNode\
(\x22mo\x22,[r]);retur\
n n.setAttribute\
(\x22fence\x22,\x22true\x22)\
,n.setAttribute(\
\x22lspace\x22,\x220.05em\
\x22),n.setAttribut\
e(\x22rspace\x22,\x220.05\
em\x22),n}});var Or\
=function(e,t){v\
ar r,n,a,i=Ke.wr\
apFragment(wt(e.\
body,t),t),o=e.l\
abel.slice(1),s=\
t.sizeMultiplier\
,h=0,c=l.isChara\
cterBox(e.body);\
if(\x22sout\x22===o)(r\
=Ke.makeSpan([\x22s\
tretchy\x22,\x22sout\x22]\
)).height=t.font\
Metrics().defaul\
tRuleThickness/s\
,h=-.5*t.fontMet\
rics().xHeight;e\
lse if(\x22phase\x22==\
=o){var m=P({num\
ber:.6,unit:\x22pt\x22\
},t),u=P({number\
:.35,unit:\x22ex\x22},\
t);s/=t.havingBa\
seSizing().sizeM\
ultiplier;var p=\
i.height+i.depth\
+m+u;i.style.pad\
dingLeft=F(p/2+m\
);var d=Math.flo\
or(1e3*p*s),f=\x22M\
400000 \x22+(n=d)+\x22\
 H0 L\x22+n/2+\x22 0 l\
65 45 L145 \x22+(n-\
80)+\x22 H400000z\x22,\
g=new K([new J(\x22\
phase\x22,f)],{widt\
h:\x22400em\x22,height\
:F(d/1e3),viewBo\
x:\x220 0 400000 \x22+\
d,preserveAspect\
Ratio:\x22xMinYMin \
slice\x22});(r=Ke.m\
akeSvgSpan([\x22hid\
e-tail\x22],[g],t))\
.style.height=F(\
p),h=i.depth+m+u\
}else{/cancel/.t\
est(o)?c||i.clas\
ses.push(\x22cancel\
-pad\x22):\x22angl\x22===\
o?i.classes.push\
(\x22anglpad\x22):i.cl\
asses.push(\x22boxp\
ad\x22);var v=0,b=0\
,y=0;/box/.test(\
o)?(y=Math.max(t\
.fontMetrics().f\
boxrule,t.minRul\
eThickness),b=v=\
t.fontMetrics().\
fboxsep+(\x22colorb\
ox\x22===o?0:y)):\x22a\
ngl\x22===o?(v=4*(y\
=Math.max(t.font\
Metrics().defaul\
tRuleThickness,t\
.minRuleThicknes\
s)),b=Math.max(0\
,.25-i.depth)):b\
=v=c?.2:0,r=Pt(i\
,o,v,b,t),/fbox|\
boxed|fcolorbox/\
.test(o)?(r.styl\
e.borderStyle=\x22s\
olid\x22,r.style.bo\
rderWidth=F(y)):\
\x22angl\x22===o&&.049\
!==y&&(r.style.b\
orderTopWidth=F(\
y),r.style.borde\
rRightWidth=F(y)\
),h=i.depth+b,e.\
backgroundColor&\
&(r.style.backgr\
oundColor=e.back\
groundColor,e.bo\
rderColor&&(r.st\
yle.borderColor=\
e.borderColor))}\
if(e.backgroundC\
olor)a=Ke.makeVL\
ist({positionTyp\
e:\x22individualShi\
ft\x22,children:[{t\
ype:\x22elem\x22,elem:\
r,shift:h},{type\
:\x22elem\x22,elem:i,s\
hift:0}]},t);els\
e{var x=/cancel|\
phase/.test(o)?[\
\x22svg-align\x22]:[];\
a=Ke.makeVList({\
positionType:\x22in\
dividualShift\x22,c\
hildren:[{type:\x22\
elem\x22,elem:i,shi\
ft:0},{type:\x22ele\
m\x22,elem:r,shift:\
h,wrapperClasses\
:x}]},t)}return/\
cancel/.test(o)&\
&(a.height=i.hei\
ght,a.depth=i.de\
pth),/cancel/.te\
st(o)&&!c?Ke.mak\
eSpan([\x22mord\x22,\x22c\
ancel-lap\x22],[a],\
t):Ke.makeSpan([\
\x22mord\x22],[a],t)},\
Er=function(e,t)\
{var r=0,n=new T\
t.MathNode(e.lab\
el.indexOf(\x22colo\
rbox\x22)>-1?\x22mpadd\
ed\x22:\x22menclose\x22,[\
Rt(e.body,t)]);s\
witch(e.label){c\
ase\x22\x5c\x5ccancel\x22:n.\
setAttribute(\x22no\
tation\x22,\x22updiago\
nalstrike\x22);brea\
k;case\x22\x5c\x5cbcancel\
\x22:n.setAttribute\
(\x22notation\x22,\x22dow\
ndiagonalstrike\x22\
);break;case\x22\x5c\x5cp\
hase\x22:n.setAttri\
bute(\x22notation\x22,\
\x22phasorangle\x22);b\
reak;case\x22\x5c\x5csout\
\x22:n.setAttribute\
(\x22notation\x22,\x22hor\
izontalstrike\x22);\
break;case\x22\x5c\x5cfbo\
x\x22:n.setAttribut\
e(\x22notation\x22,\x22bo\
x\x22);break;case\x22\x5c\
\x5cangl\x22:n.setAttr\
ibute(\x22notation\x22\
,\x22actuarial\x22);br\
eak;case\x22\x5c\x5cfcolo\
rbox\x22:case\x22\x5c\x5ccol\
orbox\x22:if(r=t.fo\
ntMetrics().fbox\
sep*t.fontMetric\
s().ptPerEm,n.se\
tAttribute(\x22widt\
h\x22,\x22+\x22+2*r+\x22pt\x22)\
,n.setAttribute(\
\x22height\x22,\x22+\x22+2*r\
+\x22pt\x22),n.setAttr\
ibute(\x22lspace\x22,r\
+\x22pt\x22),n.setAttr\
ibute(\x22voffset\x22,\
r+\x22pt\x22),\x22\x5c\x5cfcolo\
rbox\x22===e.label)\
{var a=Math.max(\
t.fontMetrics().\
fboxrule,t.minRu\
leThickness);n.s\
etAttribute(\x22sty\
le\x22,\x22border: \x22+a\
+\x22em solid \x22+Str\
ing(e.borderColo\
r))}break;case\x22\x5c\
\x5cxcancel\x22:n.setA\
ttribute(\x22notati\
on\x22,\x22updiagonals\
trike downdiagon\
alstrike\x22)}retur\
n e.backgroundCo\
lor&&n.setAttrib\
ute(\x22mathbackgro\
und\x22,e.backgroun\
dColor),n};ot({t\
ype:\x22enclose\x22,na\
mes:[\x22\x5c\x5ccolorbox\
\x22],props:{numArg\
s:2,allowedInTex\
t:!0,argTypes:[\x22\
color\x22,\x22text\x22]},\
handler:function\
(e,t,r){var n=e.\
parser,a=e.funcN\
ame,i=Ut(t[0],\x22c\
olor-token\x22).col\
or,o=t[1];return\
{type:\x22enclose\x22,\
mode:n.mode,labe\
l:a,backgroundCo\
lor:i,body:o}},h\
tmlBuilder:Or,ma\
thmlBuilder:Er})\
,ot({type:\x22enclo\
se\x22,names:[\x22\x5c\x5cfc\
olorbox\x22],props:\
{numArgs:3,allow\
edInText:!0,argT\
ypes:[\x22color\x22,\x22c\
olor\x22,\x22text\x22]},h\
andler:function(\
e,t,r){var n=e.p\
arser,a=e.funcNa\
me,i=Ut(t[0],\x22co\
lor-token\x22).colo\
r,o=Ut(t[1],\x22col\
or-token\x22).color\
,s=t[2];return{t\
ype:\x22enclose\x22,mo\
de:n.mode,label:\
a,backgroundColo\
r:o,borderColor:\
i,body:s}},htmlB\
uilder:Or,mathml\
Builder:Er}),ot(\
{type:\x22enclose\x22,\
names:[\x22\x5c\x5cfbox\x22]\
,props:{numArgs:\
1,argTypes:[\x22hbo\
x\x22],allowedInTex\
t:!0},handler:fu\
nction(e,t){retu\
rn{type:\x22enclose\
\x22,mode:e.parser.\
mode,label:\x22\x5c\x5cfb\
ox\x22,body:t[0]}}}\
),ot({type:\x22encl\
ose\x22,names:[\x22\x5c\x5cc\
ancel\x22,\x22\x5c\x5cbcance\
l\x22,\x22\x5c\x5cxcancel\x22,\x22\
\x5c\x5csout\x22,\x22\x5c\x5cphase\
\x22],props:{numArg\
s:1},handler:fun\
ction(e,t){var r\
=e.parser,n=e.fu\
ncName,a=t[0];re\
turn{type:\x22enclo\
se\x22,mode:r.mode,\
label:n,body:a}}\
,htmlBuilder:Or,\
mathmlBuilder:Er\
}),ot({type:\x22enc\
lose\x22,names:[\x22\x5c\x5c\
angl\x22],props:{nu\
mArgs:1,argTypes\
:[\x22hbox\x22],allowe\
dInText:!1},hand\
ler:function(e,t\
){return{type:\x22e\
nclose\x22,mode:e.p\
arser.mode,label\
:\x22\x5c\x5cangl\x22,body:t\
[0]}}});var Lr={\
};function Dr(e)\
{for(var t=e.typ\
e,r=e.names,n=e.\
props,a=e.handle\
r,i=e.htmlBuilde\
r,o=e.mathmlBuil\
der,s={type:t,nu\
mArgs:n.numArgs|\
|0,allowedInText\
:!1,numOptionalA\
rgs:0,handler:a}\
,l=0;l<r.length;\
++l)Lr[r[l]]=s;i\
&&(at[t]=i),o&&(\
it[t]=o)}var Vr=\
{};function Pr(e\
,t){Vr[e]=t}var \
Fr=function(){fu\
nction e(e,t,r){\
this.lexer=void \
0,this.start=voi\
d 0,this.end=voi\
d 0,this.lexer=e\
,this.start=t,th\
is.end=r}return \
e.range=function\
(t,r){return r?t\
&&t.loc&&r.loc&&\
t.loc.lexer===r.\
loc.lexer?new e(\
t.loc.lexer,t.lo\
c.start,r.loc.en\
d):null:t&&t.loc\
},e}(),Gr=functi\
on(){function e(\
e,t){this.text=v\
oid 0,this.loc=v\
oid 0,this.noexp\
and=void 0,this.\
treatAsRelax=voi\
d 0,this.text=e,\
this.loc=t}retur\
n e.prototype.ra\
nge=function(t,r\
){return new e(r\
,Fr.range(this,t\
))},e}();functio\
n Ur(e){var t=[]\
;e.consumeSpaces\
();var r=e.fetch\
().text;for(\x22\x5c\x5cr\
elax\x22===r&&(e.co\
nsume(),e.consum\
eSpaces(),r=e.fe\
tch().text);\x22\x5c\x5ch\
line\x22===r||\x22\x5c\x5chd\
ashline\x22===r;)e.\
consume(),t.push\
(\x22\x5c\x5chdashline\x22==\
=r),e.consumeSpa\
ces(),r=e.fetch(\
).text;return t}\
var Yr=function(\
e){if(!e.parser.\
settings.display\
Mode)throw new n\
(\x22{\x22+e.envName+\x22\
} can be used on\
ly in display mo\
de.\x22)};function \
Xr(e){if(-1===e.\
indexOf(\x22ed\x22))re\
turn-1===e.index\
Of(\x22*\x22)}function\
 Wr(e,t,r){var a\
=t.hskipBeforeAn\
dAfter,i=t.addJo\
t,o=t.cols,s=t.a\
rraystretch,l=t.\
colSeparationTyp\
e,h=t.autoTag,c=\
t.singleRow,m=t.\
emptySingleRow,u\
=t.maxNumCols,p=\
t.leqno;if(e.gul\
let.beginGroup()\
,c||e.gullet.mac\
ros.set(\x22\x5c\x5ccr\x22,\x22\
\x5c\x5c\x5c\x5c\x5c\x5crelax\x22),!s\
){var d=e.gullet\
.expandMacroAsTe\
xt(\x22\x5c\x5carraystret\
ch\x22);if(null==d)\
s=1;else if(!(s=\
parseFloat(d))||\
s<0)throw new n(\
\x22Invalid \x5c\x5carray\
stretch: \x22+d)}e.\
gullet.beginGrou\
p();var f=[],g=[\
f],v=[],b=[],y=n\
ull!=h?[]:void 0\
;function x(){h&\
&e.gullet.macros\
.set(\x22\x5c\x5c@eqnsw\x22,\
\x221\x22,!0)}function\
 w(){y&&(e.gulle\
t.macros.get(\x22\x5c\x5c\
df@tag\x22)?(y.push\
(e.subparse([new\
 Gr(\x22\x5c\x5cdf@tag\x22)]\
)),e.gullet.macr\
os.set(\x22\x5c\x5cdf@tag\
\x22,void 0,!0)):y.\
push(Boolean(h)&\
&\x221\x22===e.gullet.\
macros.get(\x22\x5c\x5c@e\
qnsw\x22)))}for(x()\
,b.push(Ur(e));;\
){var k=e.parseE\
xpression(!1,c?\x22\
\x5c\x5cend\x22:\x22\x5c\x5c\x5c\x5c\x22);e\
.gullet.endGroup\
(),e.gullet.begi\
nGroup(),k={type\
:\x22ordgroup\x22,mode\
:e.mode,body:k},\
r&&(k={type:\x22sty\
ling\x22,mode:e.mod\
e,style:r,body:[\
k]}),f.push(k);v\
ar S=e.fetch().t\
ext;if(\x22&\x22===S){\
if(u&&f.length==\
=u){if(c||l)thro\
w new n(\x22Too man\
y tab characters\
: &\x22,e.nextToken\
);e.settings.rep\
ortNonstrict(\x22te\
xtEnv\x22,\x22Too few \
columns specifie\
d in the {array}\
 column argument\
.\x22)}e.consume()}\
else{if(\x22\x5c\x5cend\x22=\
==S){w(),1===f.l\
ength&&\x22styling\x22\
===k.type&&0===k\
.body[0].body.le\
ngth&&(g.length>\
1||!m)&&g.pop(),\
b.length<g.lengt\
h+1&&b.push([]);\
break}if(\x22\x5c\x5c\x5c\x5c\x22!\
==S)throw new n(\
\x22Expected & or \x5c\
\x5c\x5c\x5c or \x5c\x5ccr or \x5c\
\x5cend\x22,e.nextToke\
n);e.consume();v\
ar M=void 0;\x22 \x22!\
==e.gullet.futur\
e().text&&(M=e.p\
arseSizeGroup(!0\
)),v.push(M?M.va\
lue:null),w(),b.\
push(Ur(e)),f=[]\
,g.push(f),x()}}\
return e.gullet.\
endGroup(),e.gul\
let.endGroup(),{\
type:\x22array\x22,mod\
e:e.mode,addJot:\
i,arraystretch:s\
,body:g,cols:o,r\
owGaps:v,hskipBe\
foreAndAfter:a,h\
LinesBeforeRow:b\
,colSeparationTy\
pe:l,tags:y,leqn\
o:p}}function _r\
(e){return\x22d\x22===\
e.slice(0,1)?\x22di\
splay\x22:\x22text\x22}va\
r jr=function(e,\
t){var r,a,i=e.b\
ody.length,o=e.h\
LinesBeforeRow,s\
=0,h=new Array(i\
),c=[],m=Math.ma\
x(t.fontMetrics(\
).arrayRuleWidth\
,t.minRuleThickn\
ess),u=1/t.fontM\
etrics().ptPerEm\
,p=5*u;e.colSepa\
rationType&&\x22sma\
ll\x22===e.colSepar\
ationType&&(p=t.\
havingStyle(x.SC\
RIPT).sizeMultip\
lier/t.sizeMulti\
plier*.2778);var\
 d=\x22CD\x22===e.colS\
eparationType?P(\
{number:3,unit:\x22\
ex\x22},t):12*u,f=3\
*u,g=e.arraystre\
tch*d,v=.7*g,b=.\
3*g,y=0;function\
 w(e){for(var t=\
0;t<e.length;++t\
)t>0&&(y+=.25),c\
.push({pos:y,isD\
ashed:e[t]})}for\
(w(o[0]),r=0;r<e\
.body.length;++r\
){var k=e.body[r\
],S=v,M=b;s<k.le\
ngth&&(s=k.lengt\
h);var z=new Arr\
ay(k.length);for\
(a=0;a<k.length;\
++a){var A=wt(k[\
a],t);M<A.depth&\
&(M=A.depth),S<A\
.height&&(S=A.he\
ight),z[a]=A}var\
 T=e.rowGaps[r],\
B=0;T&&(B=P(T,t)\
)>0&&(M<(B+=b)&&\
(M=B),B=0),e.add\
Jot&&(M+=f),z.he\
ight=S,z.depth=M\
,y+=S,z.pos=y,y+\
=M+B,h[r]=z,w(o[\
r+1])}var C,N,q=\
y/2+t.fontMetric\
s().axisHeight,I\
=e.cols||[],R=[]\
,H=[];if(e.tags&\
&e.tags.some((fu\
nction(e){return\
 e})))for(r=0;r<\
i;++r){var O=h[r\
],E=O.pos-q,L=e.\
tags[r],D=void 0\
;(D=!0===L?Ke.ma\
keSpan([\x22eqn-num\
\x22],[],t):!1===L?\
Ke.makeSpan([],[\
],t):Ke.makeSpan\
([],ft(L,t,!0),t\
)).depth=O.depth\
,D.height=O.heig\
ht,H.push({type:\
\x22elem\x22,elem:D,sh\
ift:E})}for(a=0,\
N=0;a<s||N<I.len\
gth;++a,++N){for\
(var V=I[N]||{},\
G=!0;\x22separator\x22\
===V.type;){if(G\
||((C=Ke.makeSpa\
n([\x22arraycolsep\x22\
],[])).style.wid\
th=F(t.fontMetri\
cs().doubleRuleS\
ep),R.push(C)),\x22\
|\x22!==V.separator\
&&\x22:\x22!==V.separa\
tor)throw new n(\
\x22Invalid separat\
or type: \x22+V.sep\
arator);var U=\x22|\
\x22===V.separator?\
\x22solid\x22:\x22dashed\x22\
,Y=Ke.makeSpan([\
\x22vertical-separa\
tor\x22],[],t);Y.st\
yle.height=F(y),\
Y.style.borderRi\
ghtWidth=F(m),Y.\
style.borderRigh\
tStyle=U,Y.style\
.margin=\x220 \x22+F(-\
m/2);var X=y-q;X\
&&(Y.style.verti\
calAlign=F(-X)),\
R.push(Y),V=I[++\
N]||{},G=!1}if(!\
(a>=s)){var W=vo\
id 0;(a>0||e.hsk\
ipBeforeAndAfter\
)&&0!==(W=l.defl\
t(V.pregap,p))&&\
((C=Ke.makeSpan(\
[\x22arraycolsep\x22],\
[])).style.width\
=F(W),R.push(C))\
;var _=[];for(r=\
0;r<i;++r){var j\
=h[r],$=j[a];if(\
$){var Z=j.pos-q\
;$.depth=j.depth\
,$.height=j.heig\
ht,_.push({type:\
\x22elem\x22,elem:$,sh\
ift:Z})}}_=Ke.ma\
keVList({positio\
nType:\x22individua\
lShift\x22,children\
:_},t),_=Ke.make\
Span([\x22col-align\
-\x22+(V.align||\x22c\x22\
)],[_]),R.push(_\
),(a<s-1||e.hski\
pBeforeAndAfter)\
&&0!==(W=l.deflt\
(V.postgap,p))&&\
((C=Ke.makeSpan(\
[\x22arraycolsep\x22],\
[])).style.width\
=F(W),R.push(C))\
}}if(h=Ke.makeSp\
an([\x22mtable\x22],R)\
,c.length>0){for\
(var K=Ke.makeLi\
neSpan(\x22hline\x22,t\
,m),J=Ke.makeLin\
eSpan(\x22hdashline\
\x22,t,m),Q=[{type:\
\x22elem\x22,elem:h,sh\
ift:0}];c.length\
>0;){var ee=c.po\
p(),te=ee.pos-q;\
ee.isDashed?Q.pu\
sh({type:\x22elem\x22,\
elem:J,shift:te}\
):Q.push({type:\x22\
elem\x22,elem:K,shi\
ft:te})}h=Ke.mak\
eVList({position\
Type:\x22individual\
Shift\x22,children:\
Q},t)}if(0===H.l\
ength)return Ke.\
makeSpan([\x22mord\x22\
],[h],t);var re=\
Ke.makeVList({po\
sitionType:\x22indi\
vidualShift\x22,chi\
ldren:H},t);retu\
rn re=Ke.makeSpa\
n([\x22tag\x22],[re],t\
),Ke.makeFragmen\
t([h,re])},$r={c\
:\x22center \x22,l:\x22le\
ft \x22,r:\x22right \x22}\
,Zr=function(e,t\
){for(var r=[],n\
=new Tt.MathNode\
(\x22mtd\x22,[],[\x22mtr-\
glue\x22]),a=new Tt\
.MathNode(\x22mtd\x22,\
[],[\x22mml-eqn-num\
\x22]),i=0;i<e.body\
.length;i++){for\
(var o=e.body[i]\
,s=[],l=0;l<o.le\
ngth;l++)s.push(\
new Tt.MathNode(\
\x22mtd\x22,[Rt(o[l],t\
)]));e.tags&&e.t\
ags[i]&&(s.unshi\
ft(n),s.push(n),\
e.leqno?s.unshif\
t(a):s.push(a)),\
r.push(new Tt.Ma\
thNode(\x22mtr\x22,s))\
}var h=new Tt.Ma\
thNode(\x22mtable\x22,\
r),c=.5===e.arra\
ystretch?.1:.16+\
e.arraystretch-1\
+(e.addJot?.09:0\
);h.setAttribute\
(\x22rowspacing\x22,F(\
c));var m=\x22\x22,u=\x22\
\x22;if(e.cols&&e.c\
ols.length>0){va\
r p=e.cols,d=\x22\x22,\
f=!1,g=0,v=p.len\
gth;\x22separator\x22=\
==p[0].type&&(m+\
=\x22top \x22,g=1),\x22se\
parator\x22===p[p.l\
ength-1].type&&(\
m+=\x22bottom \x22,v-=\
1);for(var b=g;b\
<v;b++)\x22align\x22==\
=p[b].type?(u+=$\
r[p[b].align],f&\
&(d+=\x22none \x22),f=\
!0):\x22separator\x22=\
==p[b].type&&f&&\
(d+=\x22|\x22===p[b].s\
eparator?\x22solid \
\x22:\x22dashed \x22,f=!1\
);h.setAttribute\
(\x22columnalign\x22,u\
.trim()),/[sd]/.\
test(d)&&h.setAt\
tribute(\x22columnl\
ines\x22,d.trim())}\
if(\x22align\x22===e.c\
olSeparationType\
){for(var y=e.co\
ls||[],x=\x22\x22,w=1;\
w<y.length;w++)x\
+=w%2?\x220em \x22:\x221e\
m \x22;h.setAttribu\
te(\x22columnspacin\
g\x22,x.trim())}els\
e\x22alignat\x22===e.c\
olSeparationType\
||\x22gather\x22===e.c\
olSeparationType\
?h.setAttribute(\
\x22columnspacing\x22,\
\x220em\x22):\x22small\x22==\
=e.colSeparation\
Type?h.setAttrib\
ute(\x22columnspaci\
ng\x22,\x220.2778em\x22):\
\x22CD\x22===e.colSepa\
rationType?h.set\
Attribute(\x22colum\
nspacing\x22,\x220.5em\
\x22):h.setAttribut\
e(\x22columnspacing\
\x22,\x221em\x22);var k=\x22\
\x22,S=e.hLinesBefo\
reRow;m+=S[0].le\
ngth>0?\x22left \x22:\x22\
\x22,m+=S[S.length-\
1].length>0?\x22rig\
ht \x22:\x22\x22;for(var \
M=1;M<S.length-1\
;M++)k+=0===S[M]\
.length?\x22none \x22:\
S[M][0]?\x22dashed \
\x22:\x22solid \x22;retur\
n/[sd]/.test(k)&\
&h.setAttribute(\
\x22rowlines\x22,k.tri\
m()),\x22\x22!==m&&(h=\
new Tt.MathNode(\
\x22menclose\x22,[h]))\
.setAttribute(\x22n\
otation\x22,m.trim(\
)),e.arraystretc\
h&&e.arraystretc\
h<1&&(h=new Tt.M\
athNode(\x22mstyle\x22\
,[h])).setAttrib\
ute(\x22scriptlevel\
\x22,\x221\x22),h},Kr=fun\
ction(e,t){-1===\
e.envName.indexO\
f(\x22ed\x22)&&Yr(e);v\
ar r,a=[],i=e.en\
vName.indexOf(\x22a\
t\x22)>-1?\x22alignat\x22\
:\x22align\x22,o=\x22spli\
t\x22===e.envName,s\
=Wr(e.parser,{co\
ls:a,addJot:!0,a\
utoTag:o?void 0:\
Xr(e.envName),em\
ptySingleRow:!0,\
colSeparationTyp\
e:i,maxNumCols:o\
?2:void 0,leqno:\
e.parser.setting\
s.leqno},\x22displa\
y\x22),l=0,h={type:\
\x22ordgroup\x22,mode:\
e.mode,body:[]};\
if(t[0]&&\x22ordgro\
up\x22===t[0].type)\
{for(var c=\x22\x22,m=\
0;m<t[0].body.le\
ngth;m++){c+=Ut(\
t[0].body[m],\x22te\
xtord\x22).text}r=N\
umber(c),l=2*r}v\
ar u=!l;s.body.f\
orEach((function\
(e){for(var t=1;\
t<e.length;t+=2)\
{var a=Ut(e[t],\x22\
styling\x22);Ut(a.b\
ody[0],\x22ordgroup\
\x22).body.unshift(\
h)}if(u)l<e.leng\
th&&(l=e.length)\
;else{var i=e.le\
ngth/2;if(r<i)th\
row new n(\x22Too m\
any math in a ro\
w: expected \x22+r+\
\x22, but got \x22+i,e\
[0])}}));for(var\
 p=0;p<l;++p){va\
r d=\x22r\x22,f=0;p%2=\
=1?d=\x22l\x22:p>0&&u&\
&(f=1),a[p]={typ\
e:\x22align\x22,align:\
d,pregap:f,postg\
ap:0}}return s.c\
olSeparationType\
=u?\x22align\x22:\x22alig\
nat\x22,s};Dr({type\
:\x22array\x22,names:[\
\x22array\x22,\x22darray\x22\
],props:{numArgs\
:1},handler:func\
tion(e,t){var r=\
(Xt(t[0])?[t[0]]\
:Ut(t[0],\x22ordgro\
up\x22).body).map((\
function(e){var \
t=Yt(e).text;if(\
-1!==\x22lcr\x22.index\
Of(t))return{typ\
e:\x22align\x22,align:\
t};if(\x22|\x22===t)re\
turn{type:\x22separ\
ator\x22,separator:\
\x22|\x22};if(\x22:\x22===t)\
return{type:\x22sep\
arator\x22,separato\
r:\x22:\x22};throw new\
 n(\x22Unknown colu\
mn alignment: \x22+\
t,e)})),a={cols:\
r,hskipBeforeAnd\
After:!0,maxNumC\
ols:r.length};re\
turn Wr(e.parser\
,a,_r(e.envName)\
)},htmlBuilder:j\
r,mathmlBuilder:\
Zr}),Dr({type:\x22a\
rray\x22,names:[\x22ma\
trix\x22,\x22pmatrix\x22,\
\x22bmatrix\x22,\x22Bmatr\
ix\x22,\x22vmatrix\x22,\x22V\
matrix\x22,\x22matrix*\
\x22,\x22pmatrix*\x22,\x22bm\
atrix*\x22,\x22Bmatrix\
*\x22,\x22vmatrix*\x22,\x22V\
matrix*\x22],props:\
{numArgs:0},hand\
ler:function(e){\
var t={matrix:nu\
ll,pmatrix:[\x22(\x22,\
\x22)\x22],bmatrix:[\x22[\
\x22,\x22]\x22],Bmatrix:[\
\x22\x5c\x5c{\x22,\x22\x5c\x5c}\x22],vma\
trix:[\x22|\x22,\x22|\x22],V\
matrix:[\x22\x5c\x5cVert\x22\
,\x22\x5c\x5cVert\x22]}[e.en\
vName.replace(\x22*\
\x22,\x22\x22)],r=\x22c\x22,a={\
hskipBeforeAndAf\
ter:!1,cols:[{ty\
pe:\x22align\x22,align\
:r}]};if(\x22*\x22===e\
.envName.charAt(\
e.envName.length\
-1)){var i=e.par\
ser;if(i.consume\
Spaces(),\x22[\x22===i\
.fetch().text){i\
f(i.consume(),i.\
consumeSpaces(),\
r=i.fetch().text\
,-1===\x22lcr\x22.inde\
xOf(r))throw new\
 n(\x22Expected l o\
r c or r\x22,i.next\
Token);i.consume\
(),i.consumeSpac\
es(),i.expect(\x22]\
\x22),i.consume(),a\
.cols=[{type:\x22al\
ign\x22,align:r}]}}\
var o=Wr(e.parse\
r,a,_r(e.envName\
)),s=Math.max.ap\
ply(Math,[0].con\
cat(o.body.map((\
function(e){retu\
rn e.length}))))\
;return o.cols=n\
ew Array(s).fill\
({type:\x22align\x22,a\
lign:r}),t?{type\
:\x22leftright\x22,mod\
e:e.mode,body:[o\
],left:t[0],righ\
t:t[1],rightColo\
r:void 0}:o},htm\
lBuilder:jr,math\
mlBuilder:Zr}),D\
r({type:\x22array\x22,\
names:[\x22smallmat\
rix\x22],props:{num\
Args:0},handler:\
function(e){var \
t=Wr(e.parser,{a\
rraystretch:.5},\
\x22script\x22);return\
 t.colSeparation\
Type=\x22small\x22,t},\
htmlBuilder:jr,m\
athmlBuilder:Zr}\
),Dr({type:\x22arra\
y\x22,names:[\x22subar\
ray\x22],props:{num\
Args:1},handler:\
function(e,t){va\
r r=(Xt(t[0])?[t\
[0]]:Ut(t[0],\x22or\
dgroup\x22).body).m\
ap((function(e){\
var t=Yt(e).text\
;if(-1!==\x22lc\x22.in\
dexOf(t))return{\
type:\x22align\x22,ali\
gn:t};throw new \
n(\x22Unknown colum\
n alignment: \x22+t\
,e)}));if(r.leng\
th>1)throw new n\
(\x22{subarray} can\
 contain only on\
e column\x22);var a\
={cols:r,hskipBe\
foreAndAfter:!1,\
arraystretch:.5}\
;if((a=Wr(e.pars\
er,a,\x22script\x22)).\
body.length>0&&a\
.body[0].length>\
1)throw new n(\x22{\
subarray} can co\
ntain only one c\
olumn\x22);return a\
},htmlBuilder:jr\
,mathmlBuilder:Z\
r}),Dr({type:\x22ar\
ray\x22,names:[\x22cas\
es\x22,\x22dcases\x22,\x22rc\
ases\x22,\x22drcases\x22]\
,props:{numArgs:\
0},handler:funct\
ion(e){var t=Wr(\
e.parser,{arrays\
tretch:1.2,cols:\
[{type:\x22align\x22,a\
lign:\x22l\x22,pregap:\
0,postgap:1},{ty\
pe:\x22align\x22,align\
:\x22l\x22,pregap:0,po\
stgap:0}]},_r(e.\
envName));return\
{type:\x22leftright\
\x22,mode:e.mode,bo\
dy:[t],left:e.en\
vName.indexOf(\x22r\
\x22)>-1?\x22.\x22:\x22\x5c\x5c{\x22,\
right:e.envName.\
indexOf(\x22r\x22)>-1?\
\x22\x5c\x5c}\x22:\x22.\x22,rightC\
olor:void 0}},ht\
mlBuilder:jr,mat\
hmlBuilder:Zr}),\
Dr({type:\x22array\x22\
,names:[\x22align\x22,\
\x22align*\x22,\x22aligne\
d\x22,\x22split\x22],prop\
s:{numArgs:0},ha\
ndler:Kr,htmlBui\
lder:jr,mathmlBu\
ilder:Zr}),Dr({t\
ype:\x22array\x22,name\
s:[\x22gathered\x22,\x22g\
ather\x22,\x22gather*\x22\
],props:{numArgs\
:0},handler:func\
tion(e){l.contai\
ns([\x22gather\x22,\x22ga\
ther*\x22],e.envNam\
e)&&Yr(e);var t=\
{cols:[{type:\x22al\
ign\x22,align:\x22c\x22}]\
,addJot:!0,colSe\
parationType:\x22ga\
ther\x22,autoTag:Xr\
(e.envName),empt\
ySingleRow:!0,le\
qno:e.parser.set\
tings.leqno};ret\
urn Wr(e.parser,\
t,\x22display\x22)},ht\
mlBuilder:jr,mat\
hmlBuilder:Zr}),\
Dr({type:\x22array\x22\
,names:[\x22alignat\
\x22,\x22alignat*\x22,\x22al\
ignedat\x22],props:\
{numArgs:1},hand\
ler:Kr,htmlBuild\
er:jr,mathmlBuil\
der:Zr}),Dr({typ\
e:\x22array\x22,names:\
[\x22equation\x22,\x22equ\
ation*\x22],props:{\
numArgs:0},handl\
er:function(e){Y\
r(e);var t={auto\
Tag:Xr(e.envName\
),emptySingleRow\
:!0,singleRow:!0\
,maxNumCols:1,le\
qno:e.parser.set\
tings.leqno};ret\
urn Wr(e.parser,\
t,\x22display\x22)},ht\
mlBuilder:jr,mat\
hmlBuilder:Zr}),\
Dr({type:\x22array\x22\
,names:[\x22CD\x22],pr\
ops:{numArgs:0},\
handler:function\
(e){return Yr(e)\
,function(e){var\
 t=[];for(e.gull\
et.beginGroup(),\
e.gullet.macros.\
set(\x22\x5c\x5ccr\x22,\x22\x5c\x5c\x5c\x5c\
\x5c\x5crelax\x22),e.gull\
et.beginGroup();\
;){t.push(e.pars\
eExpression(!1,\x22\
\x5c\x5c\x5c\x5c\x22)),e.gullet\
.endGroup(),e.gu\
llet.beginGroup(\
);var r=e.fetch(\
).text;if(\x22&\x22!==\
r&&\x22\x5c\x5c\x5c\x5c\x22!==r){i\
f(\x22\x5c\x5cend\x22===r){0\
===t[t.length-1]\
.length&&t.pop()\
;break}throw new\
 n(\x22Expected \x5c\x5c\x5c\
\x5c or \x5c\x5ccr or \x5c\x5ce\
nd\x22,e.nextToken)\
}e.consume()}for\
(var a,i,o=[],s=\
[o],l=0;l<t.leng\
th;l++){for(var \
h=t[l],c={type:\x22\
styling\x22,body:[]\
,mode:\x22math\x22,sty\
le:\x22display\x22},m=\
0;m<h.length;m++\
)if(tr(h[m])){o.\
push(c);var u=Yt\
(h[m+=1]).text,p\
=new Array(2);if\
(p[0]={type:\x22ord\
group\x22,mode:\x22mat\
h\x22,body:[]},p[1]\
={type:\x22ordgroup\
\x22,mode:\x22math\x22,bo\
dy:[]},\x22=|.\x22.ind\
exOf(u)>-1);else\
{if(!(\x22<>AV\x22.ind\
exOf(u)>-1))thro\
w new n('Expecte\
d one of \x22<>AV=|\
.\x22 after @',h[m]\
);for(var d=0;d<\
2;d++){for(var f\
=!0,g=m+1;g<h.le\
ngth;g++){if(i=u\
,(\x22mathord\x22===(a\
=h[g]).type||\x22at\
om\x22===a.type)&&a\
.text===i){f=!1,\
m=g;break}if(tr(\
h[g]))throw new \
n(\x22Missing a \x22+u\
+\x22 character to \
complete a CD ar\
row.\x22,h[g]);p[d]\
.body.push(h[g])\
}if(f)throw new \
n(\x22Missing a \x22+u\
+\x22 character to \
complete a CD ar\
row.\x22,h[m])}}var\
 v={type:\x22stylin\
g\x22,body:[rr(u,p,\
e)],mode:\x22math\x22,\
style:\x22display\x22}\
;o.push(v),c={ty\
pe:\x22styling\x22,bod\
y:[],mode:\x22math\x22\
,style:\x22display\x22\
}}else c.body.pu\
sh(h[m]);l%2==0?\
o.push(c):o.shif\
t(),o=[],s.push(\
o)}return e.gull\
et.endGroup(),e.\
gullet.endGroup(\
),{type:\x22array\x22,\
mode:\x22math\x22,body\
:s,arraystretch:\
1,addJot:!0,rowG\
aps:[null],cols:\
new Array(s[0].l\
ength).fill({typ\
e:\x22align\x22,align:\
\x22c\x22,pregap:.25,p\
ostgap:.25}),col\
SeparationType:\x22\
CD\x22,hLinesBefore\
Row:new Array(s.\
length+1).fill([\
])}}(e.parser)},\
htmlBuilder:jr,m\
athmlBuilder:Zr}\
),Pr(\x22\x5c\x5cnonumber\
\x22,\x22\x5c\x5cgdef\x5c\x5c@eqns\
w{0}\x22),Pr(\x22\x5c\x5cnot\
ag\x22,\x22\x5c\x5cnonumber\x22\
),ot({type:\x22text\
\x22,names:[\x22\x5c\x5chlin\
e\x22,\x22\x5c\x5chdashline\x22\
],props:{numArgs\
:0,allowedInText\
:!0,allowedInMat\
h:!0},handler:fu\
nction(e,t){thro\
w new n(e.funcNa\
me+\x22 valid only \
within array env\
ironment\x22)}});va\
r Jr=Lr;ot({type\
:\x22environment\x22,n\
ames:[\x22\x5c\x5cbegin\x22,\
\x22\x5c\x5cend\x22],props:{\
numArgs:1,argTyp\
es:[\x22text\x22]},han\
dler:function(e,\
t){var r=e.parse\
r,a=e.funcName,i\
=t[0];if(\x22ordgro\
up\x22!==i.type)thr\
ow new n(\x22Invali\
d environment na\
me\x22,i);for(var o\
=\x22\x22,s=0;s<i.body\
.length;++s)o+=U\
t(i.body[s],\x22tex\
tord\x22).text;if(\x22\
\x5c\x5cbegin\x22===a){if\
(!Jr.hasOwnPrope\
rty(o))throw new\
 n(\x22No such envi\
ronment: \x22+o,i);\
var l=Jr[o],h=r.\
parseArguments(\x22\
\x5c\x5cbegin{\x22+o+\x22}\x22,\
l),c=h.args,m=h.\
optArgs,u={mode:\
r.mode,envName:o\
,parser:r},p=l.h\
andler(u,c,m);r.\
expect(\x22\x5c\x5cend\x22,!\
1);var d=r.nextT\
oken,f=Ut(r.pars\
eFunction(),\x22env\
ironment\x22);if(f.\
name!==o)throw n\
ew n(\x22Mismatch: \
\x5c\x5cbegin{\x22+o+\x22} m\
atched by \x5c\x5cend{\
\x22+f.name+\x22}\x22,d);\
return p}return{\
type:\x22environmen\
t\x22,mode:r.mode,n\
ame:o,nameGroup:\
i}}});var Qr=fun\
ction(e,t){var r\
=e.font,n=t.with\
Font(r);return w\
t(e.body,n)},en=\
function(e,t){va\
r r=e.font,n=t.w\
ithFont(r);retur\
n Rt(e.body,n)},\
tn={\x22\x5c\x5cBbb\x22:\x22\x5c\x5cm\
athbb\x22,\x22\x5c\x5cbold\x22:\
\x22\x5c\x5cmathbf\x22,\x22\x5c\x5cfr\
ak\x22:\x22\x5c\x5cmathfrak\x22\
,\x22\x5c\x5cbm\x22:\x22\x5c\x5cbolds\
ymbol\x22};ot({type\
:\x22font\x22,names:[\x22\
\x5c\x5cmathrm\x22,\x22\x5c\x5cmat\
hit\x22,\x22\x5c\x5cmathbf\x22,\
\x22\x5c\x5cmathnormal\x22,\x22\
\x5c\x5cmathbb\x22,\x22\x5c\x5cmat\
hcal\x22,\x22\x5c\x5cmathfra\
k\x22,\x22\x5c\x5cmathscr\x22,\x22\
\x5c\x5cmathsf\x22,\x22\x5c\x5cmat\
htt\x22,\x22\x5c\x5cBbb\x22,\x22\x5c\x5c\
bold\x22,\x22\x5c\x5cfrak\x22],\
props:{numArgs:1\
,allowedInArgume\
nt:!0},handler:f\
unction(e,t){var\
 r=e.parser,n=e.\
funcName,a=lt(t[\
0]),i=n;return i\
 in tn&&(i=tn[i]\
),{type:\x22font\x22,m\
ode:r.mode,font:\
i.slice(1),body:\
a}},htmlBuilder:\
Qr,mathmlBuilder\
:en}),ot({type:\x22\
mclass\x22,names:[\x22\
\x5c\x5cboldsymbol\x22,\x22\x5c\
\x5cbm\x22],props:{num\
Args:1},handler:\
function(e,t){va\
r r=e.parser,n=t\
[0],a=l.isCharac\
terBox(n);return\
{type:\x22mclass\x22,m\
ode:r.mode,mclas\
s:Qt(n),body:[{t\
ype:\x22font\x22,mode:\
r.mode,font:\x22bol\
dsymbol\x22,body:n}\
],isCharacterBox\
:a}}}),ot({type:\
\x22font\x22,names:[\x22\x5c\
\x5crm\x22,\x22\x5c\x5csf\x22,\x22\x5c\x5ct\
t\x22,\x22\x5c\x5cbf\x22,\x22\x5c\x5cit\x22\
,\x22\x5c\x5ccal\x22],props:\
{numArgs:0,allow\
edInText:!0},han\
dler:function(e,\
t){var r=e.parse\
r,n=e.funcName,a\
=e.breakOnTokenT\
ext,i=r.mode,o=r\
.parseExpression\
(!0,a);return{ty\
pe:\x22font\x22,mode:i\
,font:\x22math\x22+n.s\
lice(1),body:{ty\
pe:\x22ordgroup\x22,mo\
de:r.mode,body:o\
}}},htmlBuilder:\
Qr,mathmlBuilder\
:en});var rn=fun\
ction(e,t){var r\
=t;return\x22displa\
y\x22===e?r=r.id>=x\
.SCRIPT.id?r.tex\
t():x.DISPLAY:\x22t\
ext\x22===e&&r.size\
===x.DISPLAY.siz\
e?r=x.TEXT:\x22scri\
pt\x22===e?r=x.SCRI\
PT:\x22scriptscript\
\x22===e&&(r=x.SCRI\
PTSCRIPT),r},nn=\
function(e,t){va\
r r,n=rn(e.size,\
t.style),a=n.fra\
cNum(),i=n.fracD\
en();r=t.havingS\
tyle(a);var o=wt\
(e.numer,r,t);if\
(e.continued){va\
r s=8.5/t.fontMe\
trics().ptPerEm,\
l=3.5/t.fontMetr\
ics().ptPerEm;o.\
height=o.height<\
s?s:o.height,o.d\
epth=o.depth<l?l\
:o.depth}r=t.hav\
ingStyle(i);var \
h,c,m,u,p,d,f,g,\
v,b,y=wt(e.denom\
,r,t);if(e.hasBa\
rLine?(e.barSize\
?(c=P(e.barSize,\
t),h=Ke.makeLine\
Span(\x22frac-line\x22\
,t,c)):h=Ke.make\
LineSpan(\x22frac-l\
ine\x22,t),c=h.heig\
ht,m=h.height):(\
h=null,c=0,m=t.f\
ontMetrics().def\
aultRuleThicknes\
s),n.size===x.DI\
SPLAY.size||\x22dis\
play\x22===e.size?(\
u=t.fontMetrics(\
).num1,p=c>0?3*m\
:7*m,d=t.fontMet\
rics().denom1):(\
c>0?(u=t.fontMet\
rics().num2,p=m)\
:(u=t.fontMetric\
s().num3,p=3*m),\
d=t.fontMetrics(\
).denom2),h){var\
 w=t.fontMetrics\
().axisHeight;u-\
o.depth-(w+.5*c)\
<p&&(u+=p-(u-o.d\
epth-(w+.5*c))),\
w-.5*c-(y.height\
-d)<p&&(d+=p-(w-\
.5*c-(y.height-d\
)));var k=-(w-.5\
*c);f=Ke.makeVLi\
st({positionType\
:\x22individualShif\
t\x22,children:[{ty\
pe:\x22elem\x22,elem:y\
,shift:d},{type:\
\x22elem\x22,elem:h,sh\
ift:k},{type:\x22el\
em\x22,elem:o,shift\
:-u}]},t)}else{v\
ar S=u-o.depth-(\
y.height-d);S<p&\
&(u+=.5*(p-S),d+\
=.5*(p-S)),f=Ke.\
makeVList({posit\
ionType:\x22individ\
ualShift\x22,childr\
en:[{type:\x22elem\x22\
,elem:y,shift:d}\
,{type:\x22elem\x22,el\
em:o,shift:-u}]}\
,t)}return r=t.h\
avingStyle(n),f.\
height*=r.sizeMu\
ltiplier/t.sizeM\
ultiplier,f.dept\
h*=r.sizeMultipl\
ier/t.sizeMultip\
lier,g=n.size===\
x.DISPLAY.size?t\
.fontMetrics().d\
elim1:n.size===x\
.SCRIPTSCRIPT.si\
ze?t.havingStyle\
(x.SCRIPT).fontM\
etrics().delim2:\
t.fontMetrics().\
delim2,v=null==e\
.leftDelim?xt(t,\
[\x22mopen\x22]):Nr.cu\
stomSizedDelim(e\
.leftDelim,g,!0,\
t.havingStyle(n)\
,e.mode,[\x22mopen\x22\
]),b=e.continued\
?Ke.makeSpan([])\
:null==e.rightDe\
lim?xt(t,[\x22mclos\
e\x22]):Nr.customSi\
zedDelim(e.right\
Delim,g,!0,t.hav\
ingStyle(n),e.mo\
de,[\x22mclose\x22]),K\
e.makeSpan([\x22mor\
d\x22].concat(r.siz\
ingClasses(t)),[\
v,Ke.makeSpan([\x22\
mfrac\x22],[f]),b],\
t)},an=function(\
e,t){var r=new T\
t.MathNode(\x22mfra\
c\x22,[Rt(e.numer,t\
),Rt(e.denom,t)]\
);if(e.hasBarLin\
e){if(e.barSize)\
{var n=P(e.barSi\
ze,t);r.setAttri\
bute(\x22linethickn\
ess\x22,F(n))}}else\
 r.setAttribute(\
\x22linethickness\x22,\
\x220px\x22);var a=rn(\
e.size,t.style);\
if(a.size!==t.st\
yle.size){r=new \
Tt.MathNode(\x22mst\
yle\x22,[r]);var i=\
a.size===x.DISPL\
AY.size?\x22true\x22:\x22\
false\x22;r.setAttr\
ibute(\x22displayst\
yle\x22,i),r.setAtt\
ribute(\x22scriptle\
vel\x22,\x220\x22)}if(nul\
l!=e.leftDelim||\
null!=e.rightDel\
im){var o=[];if(\
null!=e.leftDeli\
m){var s=new Tt.\
MathNode(\x22mo\x22,[n\
ew Tt.TextNode(e\
.leftDelim.repla\
ce(\x22\x5c\x5c\x22,\x22\x22))]);s\
.setAttribute(\x22f\
ence\x22,\x22true\x22),o.\
push(s)}if(o.pus\
h(r),null!=e.rig\
htDelim){var l=n\
ew Tt.MathNode(\x22\
mo\x22,[new Tt.Text\
Node(e.rightDeli\
m.replace(\x22\x5c\x5c\x22,\x22\
\x22))]);l.setAttri\
bute(\x22fence\x22,\x22tr\
ue\x22),o.push(l)}r\
eturn Ct(o)}retu\
rn r};ot({type:\x22\
genfrac\x22,names:[\
\x22\x5c\x5cdfrac\x22,\x22\x5c\x5cfra\
c\x22,\x22\x5c\x5ctfrac\x22,\x22\x5c\x5c\
dbinom\x22,\x22\x5c\x5cbinom\
\x22,\x22\x5c\x5ctbinom\x22,\x22\x5c\x5c\
\x5c\x5catopfrac\x22,\x22\x5c\x5c\x5c\
\x5cbracefrac\x22,\x22\x5c\x5c\x5c\
\x5cbrackfrac\x22],pro\
ps:{numArgs:2,al\
lowedInArgument:\
!0},handler:func\
tion(e,t){var r,\
n=e.parser,a=e.f\
uncName,i=t[0],o\
=t[1],s=null,l=n\
ull,h=\x22auto\x22;swi\
tch(a){case\x22\x5c\x5cdf\
rac\x22:case\x22\x5c\x5cfrac\
\x22:case\x22\x5c\x5ctfrac\x22:\
r=!0;break;case\x22\
\x5c\x5c\x5c\x5catopfrac\x22:r=\
!1;break;case\x22\x5c\x5c\
dbinom\x22:case\x22\x5c\x5cb\
inom\x22:case\x22\x5c\x5ctbi\
nom\x22:r=!1,s=\x22(\x22,\
l=\x22)\x22;break;case\
\x22\x5c\x5c\x5c\x5cbracefrac\x22:\
r=!1,s=\x22\x5c\x5c{\x22,l=\x22\
\x5c\x5c}\x22;break;case\x22\
\x5c\x5c\x5c\x5cbrackfrac\x22:r\
=!1,s=\x22[\x22,l=\x22]\x22;\
break;default:th\
row new Error(\x22U\
nrecognized genf\
rac command\x22)}sw\
itch(a){case\x22\x5c\x5cd\
frac\x22:case\x22\x5c\x5cdbi\
nom\x22:h=\x22display\x22\
;break;case\x22\x5c\x5ctf\
rac\x22:case\x22\x5c\x5ctbin\
om\x22:h=\x22text\x22}ret\
urn{type:\x22genfra\
c\x22,mode:n.mode,c\
ontinued:!1,nume\
r:i,denom:o,hasB\
arLine:r,leftDel\
im:s,rightDelim:\
l,size:h,barSize\
:null}},htmlBuil\
der:nn,mathmlBui\
lder:an}),ot({ty\
pe:\x22genfrac\x22,nam\
es:[\x22\x5c\x5ccfrac\x22],p\
rops:{numArgs:2}\
,handler:functio\
n(e,t){var r=e.p\
arser,n=(e.funcN\
ame,t[0]),a=t[1]\
;return{type:\x22ge\
nfrac\x22,mode:r.mo\
de,continued:!0,\
numer:n,denom:a,\
hasBarLine:!0,le\
ftDelim:null,rig\
htDelim:null,siz\
e:\x22display\x22,barS\
ize:null}}}),ot(\
{type:\x22infix\x22,na\
mes:[\x22\x5c\x5cover\x22,\x22\x5c\
\x5cchoose\x22,\x22\x5c\x5catop\
\x22,\x22\x5c\x5cbrace\x22,\x22\x5c\x5cb\
rack\x22],props:{nu\
mArgs:0,infix:!0\
},handler:functi\
on(e){var t,r=e.\
parser,n=e.funcN\
ame,a=e.token;sw\
itch(n){case\x22\x5c\x5co\
ver\x22:t=\x22\x5c\x5cfrac\x22;\
break;case\x22\x5c\x5ccho\
ose\x22:t=\x22\x5c\x5cbinom\x22\
;break;case\x22\x5c\x5cat\
op\x22:t=\x22\x5c\x5c\x5c\x5catopf\
rac\x22;break;case\x22\
\x5c\x5cbrace\x22:t=\x22\x5c\x5c\x5c\x5c\
bracefrac\x22;break\
;case\x22\x5c\x5cbrack\x22:t\
=\x22\x5c\x5c\x5c\x5cbrackfrac\x22\
;break;default:t\
hrow new Error(\x22\
Unrecognized inf\
ix genfrac comma\
nd\x22)}return{type\
:\x22infix\x22,mode:r.\
mode,replaceWith\
:t,token:a}}});v\
ar on=[\x22display\x22\
,\x22text\x22,\x22script\x22\
,\x22scriptscript\x22]\
,sn=function(e){\
var t=null;retur\
n e.length>0&&(t\
=\x22.\x22===(t=e)?nul\
l:t),t};ot({type\
:\x22genfrac\x22,names\
:[\x22\x5c\x5cgenfrac\x22],p\
rops:{numArgs:6,\
allowedInArgumen\
t:!0,argTypes:[\x22\
math\x22,\x22math\x22,\x22si\
ze\x22,\x22text\x22,\x22math\
\x22,\x22math\x22]},handl\
er:function(e,t)\
{var r,n=e.parse\
r,a=t[4],i=t[5],\
o=lt(t[0]),s=\x22at\
om\x22===o.type&&\x22o\
pen\x22===o.family?\
sn(o.text):null,\
l=lt(t[1]),h=\x22at\
om\x22===l.type&&\x22c\
lose\x22===l.family\
?sn(l.text):null\
,c=Ut(t[2],\x22size\
\x22),m=null;r=!!c.\
isBlank||(m=c.va\
lue).number>0;va\
r u=\x22auto\x22,p=t[3\
];if(\x22ordgroup\x22=\
==p.type){if(p.b\
ody.length>0){va\
r d=Ut(p.body[0]\
,\x22textord\x22);u=on\
[Number(d.text)]\
}}else p=Ut(p,\x22t\
extord\x22),u=on[Nu\
mber(p.text)];re\
turn{type:\x22genfr\
ac\x22,mode:n.mode,\
numer:a,denom:i,\
continued:!1,has\
BarLine:r,barSiz\
e:m,leftDelim:s,\
rightDelim:h,siz\
e:u}},htmlBuilde\
r:nn,mathmlBuild\
er:an}),ot({type\
:\x22infix\x22,names:[\
\x22\x5c\x5cabove\x22],props\
:{numArgs:1,argT\
ypes:[\x22size\x22],in\
fix:!0},handler:\
function(e,t){va\
r r=e.parser,n=(\
e.funcName,e.tok\
en);return{type:\
\x22infix\x22,mode:r.m\
ode,replaceWith:\
\x22\x5c\x5c\x5c\x5cabovefrac\x22,\
size:Ut(t[0],\x22si\
ze\x22).value,token\
:n}}}),ot({type:\
\x22genfrac\x22,names:\
[\x22\x5c\x5c\x5c\x5cabovefrac\x22\
],props:{numArgs\
:3,argTypes:[\x22ma\
th\x22,\x22size\x22,\x22math\
\x22]},handler:func\
tion(e,t){var r=\
e.parser,n=(e.fu\
ncName,t[0]),a=f\
unction(e){if(!e\
)throw new Error\
(\x22Expected non-n\
ull, but got \x22+S\
tring(e));return\
 e}(Ut(t[1],\x22inf\
ix\x22).size),i=t[2\
],o=a.number>0;r\
eturn{type:\x22genf\
rac\x22,mode:r.mode\
,numer:n,denom:i\
,continued:!1,ha\
sBarLine:o,barSi\
ze:a,leftDelim:n\
ull,rightDelim:n\
ull,size:\x22auto\x22}\
},htmlBuilder:nn\
,mathmlBuilder:a\
n});var ln=funct\
ion(e,t){var r,n\
,a=t.style;\x22sups\
ub\x22===e.type?(r=\
e.sup?wt(e.sup,t\
.havingStyle(a.s\
up()),t):wt(e.su\
b,t.havingStyle(\
a.sub()),t),n=Ut\
(e.base,\x22horizBr\
ace\x22)):n=Ut(e,\x22h\
orizBrace\x22);var \
i,o=wt(n.base,t.\
havingBaseStyle(\
x.DISPLAY)),s=Gt\
(n,t);if(n.isOve\
r?(i=Ke.makeVLis\
t({positionType:\
\x22firstBaseline\x22,\
children:[{type:\
\x22elem\x22,elem:o},{\
type:\x22kern\x22,size\
:.1},{type:\x22elem\
\x22,elem:s}]},t)).\
children[0].chil\
dren[0].children\
[1].classes.push\
(\x22svg-align\x22):(i\
=Ke.makeVList({p\
ositionType:\x22bot\
tom\x22,positionDat\
a:o.depth+.1+s.h\
eight,children:[\
{type:\x22elem\x22,ele\
m:s},{type:\x22kern\
\x22,size:.1},{type\
:\x22elem\x22,elem:o}]\
},t)).children[0\
].children[0].ch\
ildren[0].classe\
s.push(\x22svg-alig\
n\x22),r){var l=Ke.\
makeSpan([\x22mord\x22\
,n.isOver?\x22mover\
\x22:\x22munder\x22],[i],\
t);i=n.isOver?Ke\
.makeVList({posi\
tionType:\x22firstB\
aseline\x22,childre\
n:[{type:\x22elem\x22,\
elem:l},{type:\x22k\
ern\x22,size:.2},{t\
ype:\x22elem\x22,elem:\
r}]},t):Ke.makeV\
List({positionTy\
pe:\x22bottom\x22,posi\
tionData:l.depth\
+.2+r.height+r.d\
epth,children:[{\
type:\x22elem\x22,elem\
:r},{type:\x22kern\x22\
,size:.2},{type:\
\x22elem\x22,elem:l}]}\
,t)}return Ke.ma\
keSpan([\x22mord\x22,n\
.isOver?\x22mover\x22:\
\x22munder\x22],[i],t)\
};ot({type:\x22hori\
zBrace\x22,names:[\x22\
\x5c\x5coverbrace\x22,\x22\x5c\x5c\
underbrace\x22],pro\
ps:{numArgs:1},h\
andler:function(\
e,t){var r=e.par\
ser,n=e.funcName\
;return{type:\x22ho\
rizBrace\x22,mode:r\
.mode,label:n,is\
Over:/^\x5c\x5cover/.t\
est(n),base:t[0]\
}},htmlBuilder:l\
n,mathmlBuilder:\
function(e,t){va\
r r=Ft(e.label);\
return new Tt.Ma\
thNode(e.isOver?\
\x22mover\x22:\x22munder\x22\
,[Rt(e.base,t),r\
])}}),ot({type:\x22\
href\x22,names:[\x22\x5c\x5c\
href\x22],props:{nu\
mArgs:2,argTypes\
:[\x22url\x22,\x22origina\
l\x22],allowedInTex\
t:!0},handler:fu\
nction(e,t){var \
r=e.parser,n=t[1\
],a=Ut(t[0],\x22url\
\x22).url;return r.\
settings.isTrust\
ed({command:\x22\x5c\x5ch\
ref\x22,url:a})?{ty\
pe:\x22href\x22,mode:r\
.mode,href:a,bod\
y:ht(n)}:r.forma\
tUnsupportedCmd(\
\x22\x5c\x5chref\x22)},htmlB\
uilder:function(\
e,t){var r=ft(e.\
body,t,!1);retur\
n Ke.makeAnchor(\
e.href,[],r,t)},\
mathmlBuilder:fu\
nction(e,t){var \
r=It(e.body,t);r\
eturn r instance\
of zt||(r=new zt\
(\x22mrow\x22,[r])),r.\
setAttribute(\x22hr\
ef\x22,e.href),r}})\
,ot({type:\x22href\x22\
,names:[\x22\x5c\x5curl\x22]\
,props:{numArgs:\
1,argTypes:[\x22url\
\x22],allowedInText\
:!0},handler:fun\
ction(e,t){var r\
=e.parser,n=Ut(t\
[0],\x22url\x22).url;i\
f(!r.settings.is\
Trusted({command\
:\x22\x5c\x5curl\x22,url:n})\
)return r.format\
UnsupportedCmd(\x22\
\x5c\x5curl\x22);for(var \
a=[],i=0;i<n.len\
gth;i++){var o=n\
[i];\x22~\x22===o&&(o=\
\x22\x5c\x5ctextasciitild\
e\x22),a.push({type\
:\x22textord\x22,mode:\
\x22text\x22,text:o})}\
var s={type:\x22tex\
t\x22,mode:r.mode,f\
ont:\x22\x5c\x5ctexttt\x22,b\
ody:a};return{ty\
pe:\x22href\x22,mode:r\
.mode,href:n,bod\
y:ht(s)}}}),ot({\
type:\x22hbox\x22,name\
s:[\x22\x5c\x5chbox\x22],pro\
ps:{numArgs:1,ar\
gTypes:[\x22text\x22],\
allowedInText:!0\
,primitive:!0},h\
andler:function(\
e,t){return{type\
:\x22hbox\x22,mode:e.p\
arser.mode,body:\
ht(t[0])}},htmlB\
uilder:function(\
e,t){var r=ft(e.\
body,t,!1);retur\
n Ke.makeFragmen\
t(r)},mathmlBuil\
der:function(e,t\
){return new Tt.\
MathNode(\x22mrow\x22,\
qt(e.body,t))}})\
,ot({type:\x22html\x22\
,names:[\x22\x5c\x5chtmlC\
lass\x22,\x22\x5c\x5chtmlId\x22\
,\x22\x5c\x5chtmlStyle\x22,\x22\
\x5c\x5chtmlData\x22],pro\
ps:{numArgs:2,ar\
gTypes:[\x22raw\x22,\x22o\
riginal\x22],allowe\
dInText:!0},hand\
ler:function(e,t\
){var r,a=e.pars\
er,i=e.funcName,\
o=(e.token,Ut(t[\
0],\x22raw\x22).string\
),s=t[1];a.setti\
ngs.strict&&a.se\
ttings.reportNon\
strict(\x22htmlExte\
nsion\x22,\x22HTML ext\
ension is disabl\
ed on strict mod\
e\x22);var l={};swi\
tch(i){case\x22\x5c\x5cht\
mlClass\x22:l.class\
=o,r={command:\x22\x5c\
\x5chtmlClass\x22,clas\
s:o};break;case\x22\
\x5c\x5chtmlId\x22:l.id=o\
,r={command:\x22\x5c\x5ch\
tmlId\x22,id:o};bre\
ak;case\x22\x5c\x5chtmlSt\
yle\x22:l.style=o,r\
={command:\x22\x5c\x5chtm\
lStyle\x22,style:o}\
;break;case\x22\x5c\x5cht\
mlData\x22:for(var \
h=o.split(\x22,\x22),c\
=0;c<h.length;c+\
+){var m=h[c].sp\
lit(\x22=\x22);if(2!==\
m.length)throw n\
ew n(\x22Error pars\
ing key-value fo\
r \x5c\x5chtmlData\x22);l\
[\x22data-\x22+m[0].tr\
im()]=m[1].trim(\
)}r={command:\x22\x5c\x5c\
htmlData\x22,attrib\
utes:l};break;de\
fault:throw new \
Error(\x22Unrecogni\
zed html command\
\x22)}return a.sett\
ings.isTrusted(r\
)?{type:\x22html\x22,m\
ode:a.mode,attri\
butes:l,body:ht(\
s)}:a.formatUnsu\
pportedCmd(i)},h\
tmlBuilder:funct\
ion(e,t){var r=f\
t(e.body,t,!1),n\
=[\x22enclosing\x22];e\
.attributes.clas\
s&&n.push.apply(\
n,e.attributes.c\
lass.trim().spli\
t(/\x5cs+/));var a=\
Ke.makeSpan(n,r,\
t);for(var i in \
e.attributes)\x22cl\
ass\x22!==i&&e.attr\
ibutes.hasOwnPro\
perty(i)&&a.setA\
ttribute(i,e.att\
ributes[i]);retu\
rn a},mathmlBuil\
der:function(e,t\
){return It(e.bo\
dy,t)}}),ot({typ\
e:\x22htmlmathml\x22,n\
ames:[\x22\x5c\x5chtml@ma\
thml\x22],props:{nu\
mArgs:2,allowedI\
nText:!0},handle\
r:function(e,t){\
return{type:\x22htm\
lmathml\x22,mode:e.\
parser.mode,html\
:ht(t[0]),mathml\
:ht(t[1])}},html\
Builder:function\
(e,t){var r=ft(e\
.html,t,!1);retu\
rn Ke.makeFragme\
nt(r)},mathmlBui\
lder:function(e,\
t){return It(e.m\
athml,t)}});var \
hn=function(e){i\
f(/^[-+]? *(\x5cd+(\
\x5c.\x5cd*)?|\x5c.\x5cd+)$/\
.test(e))return{\
number:+e,unit:\x22\
bp\x22};var t=/([-+\
]?) *(\x5cd+(?:\x5c.\x5cd\
*)?|\x5c.\x5cd+) *([a-\
z]{2})/.exec(e);\
if(!t)throw new \
n(\x22Invalid size:\
 '\x22+e+\x22' in \x5c\x5cin\
cludegraphics\x22);\
var r={number:+(\
t[1]+t[2]),unit:\
t[3]};if(!V(r))t\
hrow new n(\x22Inva\
lid unit: '\x22+r.u\
nit+\x22' in \x5c\x5cincl\
udegraphics.\x22);r\
eturn r};ot({typ\
e:\x22includegraphi\
cs\x22,names:[\x22\x5c\x5cin\
cludegraphics\x22],\
props:{numArgs:1\
,numOptionalArgs\
:1,argTypes:[\x22ra\
w\x22,\x22url\x22],allowe\
dInText:!1},hand\
ler:function(e,t\
,r){var a=e.pars\
er,i={number:0,u\
nit:\x22em\x22},o={num\
ber:.9,unit:\x22em\x22\
},s={number:0,un\
it:\x22em\x22},l=\x22\x22;if\
(r[0])for(var h=\
Ut(r[0],\x22raw\x22).s\
tring.split(\x22,\x22)\
,c=0;c<h.length;\
c++){var m=h[c].\
split(\x22=\x22);if(2=\
==m.length){var \
u=m[1].trim();sw\
itch(m[0].trim()\
){case\x22alt\x22:l=u;\
break;case\x22width\
\x22:i=hn(u);break;\
case\x22height\x22:o=h\
n(u);break;case\x22\
totalheight\x22:s=h\
n(u);break;defau\
lt:throw new n(\x22\
Invalid key: '\x22+\
m[0]+\x22' in \x5c\x5cinc\
ludegraphics.\x22)}\
}}var p=Ut(t[0],\
\x22url\x22).url;retur\
n\x22\x22===l&&(l=(l=(\
l=p).replace(/^.\
*[\x5c\x5c/]/,\x22\x22)).sub\
string(0,l.lastI\
ndexOf(\x22.\x22))),a.\
settings.isTrust\
ed({command:\x22\x5c\x5ci\
ncludegraphics\x22,\
url:p})?{type:\x22i\
ncludegraphics\x22,\
mode:a.mode,alt:\
l,width:i,height\
:o,totalheight:s\
,src:p}:a.format\
UnsupportedCmd(\x22\
\x5c\x5cincludegraphic\
s\x22)},htmlBuilder\
:function(e,t){v\
ar r=P(e.height,\
t),n=0;e.totalhe\
ight.number>0&&(\
n=P(e.totalheigh\
t,t)-r);var a=0;\
e.width.number>0\
&&(a=P(e.width,t\
));var i={height\
:F(r+n)};a>0&&(i\
.width=F(a)),n>0\
&&(i.verticalAli\
gn=F(-n));var o=\
new j(e.src,e.al\
t,i);return o.he\
ight=r,o.depth=n\
,o},mathmlBuilde\
r:function(e,t){\
var r=new Tt.Mat\
hNode(\x22mglyph\x22,[\
]);r.setAttribut\
e(\x22alt\x22,e.alt);v\
ar n=P(e.height,\
t),a=0;if(e.tota\
lheight.number>0\
&&(a=P(e.totalhe\
ight,t)-n,r.setA\
ttribute(\x22valign\
\x22,F(-a))),r.setA\
ttribute(\x22height\
\x22,F(n+a)),e.widt\
h.number>0){var \
i=P(e.width,t);r\
.setAttribute(\x22w\
idth\x22,F(i))}retu\
rn r.setAttribut\
e(\x22src\x22,e.src),r\
}}),ot({type:\x22ke\
rn\x22,names:[\x22\x5c\x5cke\
rn\x22,\x22\x5c\x5cmkern\x22,\x22\x5c\
\x5chskip\x22,\x22\x5c\x5cmskip\
\x22],props:{numArg\
s:1,argTypes:[\x22s\
ize\x22],primitive:\
!0,allowedInText\
:!0},handler:fun\
ction(e,t){var r\
=e.parser,n=e.fu\
ncName,a=Ut(t[0]\
,\x22size\x22);if(r.se\
ttings.strict){v\
ar i=\x22m\x22===n[1],\
o=\x22mu\x22===a.value\
.unit;i?(o||r.se\
ttings.reportNon\
strict(\x22mathVsTe\
xtUnits\x22,\x22LaTeX'\
s \x22+n+\x22 supports\
 only mu units, \
not \x22+a.value.un\
it+\x22 units\x22),\x22ma\
th\x22!==r.mode&&r.\
settings.reportN\
onstrict(\x22mathVs\
TextUnits\x22,\x22LaTe\
X's \x22+n+\x22 works \
only in math mod\
e\x22)):o&&r.settin\
gs.reportNonstri\
ct(\x22mathVsTextUn\
its\x22,\x22LaTeX's \x22+\
n+\x22 doesn't supp\
ort mu units\x22)}r\
eturn{type:\x22kern\
\x22,mode:r.mode,di\
mension:a.value}\
},htmlBuilder:fu\
nction(e,t){retu\
rn Ke.makeGlue(e\
.dimension,t)},m\
athmlBuilder:fun\
ction(e,t){var r\
=P(e.dimension,t\
);return new Tt.\
SpaceNode(r)}}),\
ot({type:\x22lap\x22,n\
ames:[\x22\x5c\x5cmathlla\
p\x22,\x22\x5c\x5cmathrlap\x22,\
\x22\x5c\x5cmathclap\x22],pr\
ops:{numArgs:1,a\
llowedInText:!0}\
,handler:functio\
n(e,t){var r=e.p\
arser,n=e.funcNa\
me,a=t[0];return\
{type:\x22lap\x22,mode\
:r.mode,alignmen\
t:n.slice(5),bod\
y:a}},htmlBuilde\
r:function(e,t){\
var r;\x22clap\x22===e\
.alignment?(r=Ke\
.makeSpan([],[wt\
(e.body,t)]),r=K\
e.makeSpan([\x22inn\
er\x22],[r],t)):r=K\
e.makeSpan([\x22inn\
er\x22],[wt(e.body,\
t)]);var n=Ke.ma\
keSpan([\x22fix\x22],[\
]),a=Ke.makeSpan\
([e.alignment],[\
r,n],t),i=Ke.mak\
eSpan([\x22strut\x22])\
;return i.style.\
height=F(a.heigh\
t+a.depth),a.dep\
th&&(i.style.ver\
ticalAlign=F(-a.\
depth)),a.childr\
en.unshift(i),a=\
Ke.makeSpan([\x22th\
inbox\x22],[a],t),K\
e.makeSpan([\x22mor\
d\x22,\x22vbox\x22],[a],t\
)},mathmlBuilder\
:function(e,t){v\
ar r=new Tt.Math\
Node(\x22mpadded\x22,[\
Rt(e.body,t)]);i\
f(\x22rlap\x22!==e.ali\
gnment){var n=\x22l\
lap\x22===e.alignme\
nt?\x22-1\x22:\x22-0.5\x22;r\
.setAttribute(\x22l\
space\x22,n+\x22width\x22\
)}return r.setAt\
tribute(\x22width\x22,\
\x220px\x22),r}}),ot({\
type:\x22styling\x22,n\
ames:[\x22\x5c\x5c(\x22,\x22$\x22]\
,props:{numArgs:\
0,allowedInText:\
!0,allowedInMath\
:!1},handler:fun\
ction(e,t){var r\
=e.funcName,n=e.\
parser,a=n.mode;\
n.switchMode(\x22ma\
th\x22);var i=\x22\x5c\x5c(\x22\
===r?\x22\x5c\x5c)\x22:\x22$\x22,o\
=n.parseExpressi\
on(!1,i);return \
n.expect(i),n.sw\
itchMode(a),{typ\
e:\x22styling\x22,mode\
:n.mode,style:\x22t\
ext\x22,body:o}}}),\
ot({type:\x22text\x22,\
names:[\x22\x5c\x5c)\x22,\x22\x5c\x5c\
]\x22],props:{numAr\
gs:0,allowedInTe\
xt:!0,allowedInM\
ath:!1},handler:\
function(e,t){th\
row new n(\x22Misma\
tched \x22+e.funcNa\
me)}});var cn=fu\
nction(e,t){swit\
ch(t.style.size)\
{case x.DISPLAY.\
size:return e.di\
splay;case x.TEX\
T.size:return e.\
text;case x.SCRI\
PT.size:return e\
.script;case x.S\
CRIPTSCRIPT.size\
:return e.script\
script;default:r\
eturn e.text}};o\
t({type:\x22mathcho\
ice\x22,names:[\x22\x5c\x5cm\
athchoice\x22],prop\
s:{numArgs:4,pri\
mitive:!0},handl\
er:function(e,t)\
{return{type:\x22ma\
thchoice\x22,mode:e\
.parser.mode,dis\
play:ht(t[0]),te\
xt:ht(t[1]),scri\
pt:ht(t[2]),scri\
ptscript:ht(t[3]\
)}},htmlBuilder:\
function(e,t){va\
r r=cn(e,t),n=ft\
(r,t,!1);return \
Ke.makeFragment(\
n)},mathmlBuilde\
r:function(e,t){\
var r=cn(e,t);re\
turn It(r,t)}});\
var mn=function(\
e,t,r,n,a,i,o){e\
=Ke.makeSpan([],\
[e]);var s,h,c,m\
=r&&l.isCharacte\
rBox(r);if(t){va\
r u=wt(t,n.havin\
gStyle(a.sup()),\
n);h={elem:u,ker\
n:Math.max(n.fon\
tMetrics().bigOp\
Spacing1,n.fontM\
etrics().bigOpSp\
acing3-u.depth)}\
}if(r){var p=wt(\
r,n.havingStyle(\
a.sub()),n);s={e\
lem:p,kern:Math.\
max(n.fontMetric\
s().bigOpSpacing\
2,n.fontMetrics(\
).bigOpSpacing4-\
p.height)}}if(h&\
&s){var d=n.font\
Metrics().bigOpS\
pacing5+s.elem.h\
eight+s.elem.dep\
th+s.kern+e.dept\
h+o;c=Ke.makeVLi\
st({positionType\
:\x22bottom\x22,positi\
onData:d,childre\
n:[{type:\x22kern\x22,\
size:n.fontMetri\
cs().bigOpSpacin\
g5},{type:\x22elem\x22\
,elem:s.elem,mar\
ginLeft:F(-i)},{\
type:\x22kern\x22,size\
:s.kern},{type:\x22\
elem\x22,elem:e},{t\
ype:\x22kern\x22,size:\
h.kern},{type:\x22e\
lem\x22,elem:h.elem\
,marginLeft:F(i)\
},{type:\x22kern\x22,s\
ize:n.fontMetric\
s().bigOpSpacing\
5}]},n)}else if(\
s){var f=e.heigh\
t-o;c=Ke.makeVLi\
st({positionType\
:\x22top\x22,positionD\
ata:f,children:[\
{type:\x22kern\x22,siz\
e:n.fontMetrics(\
).bigOpSpacing5}\
,{type:\x22elem\x22,el\
em:s.elem,margin\
Left:F(-i)},{typ\
e:\x22kern\x22,size:s.\
kern},{type:\x22ele\
m\x22,elem:e}]},n)}\
else{if(!h)retur\
n e;var g=e.dept\
h+o;c=Ke.makeVLi\
st({positionType\
:\x22bottom\x22,positi\
onData:g,childre\
n:[{type:\x22elem\x22,\
elem:e},{type:\x22k\
ern\x22,size:h.kern\
},{type:\x22elem\x22,e\
lem:h.elem,margi\
nLeft:F(i)},{typ\
e:\x22kern\x22,size:n.\
fontMetrics().bi\
gOpSpacing5}]},n\
)}var v=[c];if(s\
&&0!==i&&!m){var\
 b=Ke.makeSpan([\
\x22mspace\x22],[],n);\
b.style.marginRi\
ght=F(i),v.unshi\
ft(b)}return Ke.\
makeSpan([\x22mop\x22,\
\x22op-limits\x22],v,n\
)},un=[\x22\x5c\x5csmalli\
nt\x22],pn=function\
(e,t){var r,n,a,\
i=!1;\x22supsub\x22===\
e.type?(r=e.sup,\
n=e.sub,a=Ut(e.b\
ase,\x22op\x22),i=!0):\
a=Ut(e,\x22op\x22);var\
 o,s=t.style,h=!\
1;if(s.size===x.\
DISPLAY.size&&a.\
symbol&&!l.conta\
ins(un,a.name)&&\
(h=!0),a.symbol)\
{var c=h?\x22Size2-\
Regular\x22:\x22Size1-\
Regular\x22,m=\x22\x22;if\
(\x22\x5c\x5coiint\x22!==a.n\
ame&&\x22\x5c\x5coiiint\x22!\
==a.name||(m=a.n\
ame.slice(1),a.n\
ame=\x22oiint\x22===m?\
\x22\x5c\x5ciint\x22:\x22\x5c\x5ciiin\
t\x22),o=Ke.makeSym\
bol(a.name,c,\x22ma\
th\x22,t,[\x22mop\x22,\x22op\
-symbol\x22,h?\x22larg\
e-op\x22:\x22small-op\x22\
]),m.length>0){v\
ar u=o.italic,p=\
Ke.staticSvg(m+\x22\
Size\x22+(h?\x222\x22:\x221\x22\
),t);o=Ke.makeVL\
ist({positionTyp\
e:\x22individualShi\
ft\x22,children:[{t\
ype:\x22elem\x22,elem:\
o,shift:0},{type\
:\x22elem\x22,elem:p,s\
hift:h?.08:0}]},\
t),a.name=\x22\x5c\x5c\x22+m\
,o.classes.unshi\
ft(\x22mop\x22),o.ital\
ic=u}}else if(a.\
body){var d=ft(a\
.body,t,!0);1===\
d.length&&d[0]in\
stanceof Z?(o=d[\
0]).classes[0]=\x22\
mop\x22:o=Ke.makeSp\
an([\x22mop\x22],d,t)}\
else{for(var f=[\
],g=1;g<a.name.l\
ength;g++)f.push\
(Ke.mathsym(a.na\
me[g],a.mode,t))\
;o=Ke.makeSpan([\
\x22mop\x22],f,t)}var \
v=0,b=0;return(o\
 instanceof Z||\x22\
\x5c\x5coiint\x22===a.nam\
e||\x22\x5c\x5coiiint\x22===\
a.name)&&!a.supp\
ressBaseShift&&(\
v=(o.height-o.de\
pth)/2-t.fontMet\
rics().axisHeigh\
t,b=o.italic),i?\
mn(o,r,n,t,s,b,v\
):(v&&(o.style.p\
osition=\x22relativ\
e\x22,o.style.top=F\
(v)),o)},dn=func\
tion(e,t){var r;\
if(e.symbol)r=ne\
w zt(\x22mo\x22,[Bt(e.\
name,e.mode)]),l\
.contains(un,e.n\
ame)&&r.setAttri\
bute(\x22largeop\x22,\x22\
false\x22);else if(\
e.body)r=new zt(\
\x22mo\x22,qt(e.body,t\
));else{r=new zt\
(\x22mi\x22,[new At(e.\
name.slice(1))])\
;var n=new zt(\x22m\
o\x22,[Bt(\x22\x5cu2061\x22,\
\x22text\x22)]);r=e.pa\
rentIsSupSub?new\
 zt(\x22mrow\x22,[r,n]\
):Mt([r,n])}retu\
rn r},fn={\x22\x5cu220\
f\x22:\x22\x5c\x5cprod\x22,\x22\x5cu2\
210\x22:\x22\x5c\x5ccoprod\x22,\
\x22\x5cu2211\x22:\x22\x5c\x5csum\x22\
,\x22\x5cu22c0\x22:\x22\x5c\x5cbig\
wedge\x22,\x22\x5cu22c1\x22:\
\x22\x5c\x5cbigvee\x22,\x22\x5cu22\
c2\x22:\x22\x5c\x5cbigcap\x22,\x22\
\x5cu22c3\x22:\x22\x5c\x5cbigcu\
p\x22,\x22\x5cu2a00\x22:\x22\x5c\x5cb\
igodot\x22,\x22\x5cu2a01\x22\
:\x22\x5c\x5cbigoplus\x22,\x22\x5c\
u2a02\x22:\x22\x5c\x5cbigoti\
mes\x22,\x22\x5cu2a04\x22:\x22\x5c\
\x5cbiguplus\x22,\x22\x5cu2a\
06\x22:\x22\x5c\x5cbigsqcup\x22\
};ot({type:\x22op\x22,\
names:[\x22\x5c\x5ccoprod\
\x22,\x22\x5c\x5cbigvee\x22,\x22\x5c\x5c\
bigwedge\x22,\x22\x5c\x5cbig\
uplus\x22,\x22\x5c\x5cbigcap\
\x22,\x22\x5c\x5cbigcup\x22,\x22\x5c\x5c\
intop\x22,\x22\x5c\x5cprod\x22,\
\x22\x5c\x5csum\x22,\x22\x5c\x5cbigot\
imes\x22,\x22\x5c\x5cbigoplu\
s\x22,\x22\x5c\x5cbigodot\x22,\x22\
\x5c\x5cbigsqcup\x22,\x22\x5c\x5cs\
mallint\x22,\x22\x5cu220f\
\x22,\x22\x5cu2210\x22,\x22\x5cu22\
11\x22,\x22\x5cu22c0\x22,\x22\x5cu\
22c1\x22,\x22\x5cu22c2\x22,\x22\
\x5cu22c3\x22,\x22\x5cu2a00\x22\
,\x22\x5cu2a01\x22,\x22\x5cu2a0\
2\x22,\x22\x5cu2a04\x22,\x22\x5cu2\
a06\x22],props:{num\
Args:0},handler:\
function(e,t){va\
r r=e.parser,n=e\
.funcName;return\
 1===n.length&&(\
n=fn[n]),{type:\x22\
op\x22,mode:r.mode,\
limits:!0,parent\
IsSupSub:!1,symb\
ol:!0,name:n}},h\
tmlBuilder:pn,ma\
thmlBuilder:dn})\
,ot({type:\x22op\x22,n\
ames:[\x22\x5c\x5cmathop\x22\
],props:{numArgs\
:1,primitive:!0}\
,handler:functio\
n(e,t){var r=e.p\
arser,n=t[0];ret\
urn{type:\x22op\x22,mo\
de:r.mode,limits\
:!1,parentIsSupS\
ub:!1,symbol:!1,\
body:ht(n)}},htm\
lBuilder:pn,math\
mlBuilder:dn});v\
ar gn={\x22\x5cu222b\x22:\
\x22\x5c\x5cint\x22,\x22\x5cu222c\x22\
:\x22\x5c\x5ciint\x22,\x22\x5cu222\
d\x22:\x22\x5c\x5ciiint\x22,\x22\x5cu\
222e\x22:\x22\x5c\x5coint\x22,\x22\
\x5cu222f\x22:\x22\x5c\x5coiint\
\x22,\x22\x5cu2230\x22:\x22\x5c\x5coi\
iint\x22};ot({type:\
\x22op\x22,names:[\x22\x5c\x5ca\
rcsin\x22,\x22\x5c\x5carccos\
\x22,\x22\x5c\x5carctan\x22,\x22\x5c\x5c\
arctg\x22,\x22\x5c\x5carcctg\
\x22,\x22\x5c\x5carg\x22,\x22\x5c\x5cch\x22\
,\x22\x5c\x5ccos\x22,\x22\x5c\x5ccose\
c\x22,\x22\x5c\x5ccosh\x22,\x22\x5c\x5cc\
ot\x22,\x22\x5c\x5ccotg\x22,\x22\x5c\x5c\
coth\x22,\x22\x5c\x5ccsc\x22,\x22\x5c\
\x5cctg\x22,\x22\x5c\x5ccth\x22,\x22\x5c\
\x5cdeg\x22,\x22\x5c\x5cdim\x22,\x22\x5c\
\x5cexp\x22,\x22\x5c\x5chom\x22,\x22\x5c\
\x5cker\x22,\x22\x5c\x5clg\x22,\x22\x5c\x5c\
ln\x22,\x22\x5c\x5clog\x22,\x22\x5c\x5cs\
ec\x22,\x22\x5c\x5csin\x22,\x22\x5c\x5cs\
inh\x22,\x22\x5c\x5csh\x22,\x22\x5c\x5ct\
an\x22,\x22\x5c\x5ctanh\x22,\x22\x5c\x5c\
tg\x22,\x22\x5c\x5cth\x22],prop\
s:{numArgs:0},ha\
ndler:function(e\
){var t=e.parser\
,r=e.funcName;re\
turn{type:\x22op\x22,m\
ode:t.mode,limit\
s:!1,parentIsSup\
Sub:!1,symbol:!1\
,name:r}},htmlBu\
ilder:pn,mathmlB\
uilder:dn}),ot({\
type:\x22op\x22,names:\
[\x22\x5c\x5cdet\x22,\x22\x5c\x5cgcd\x22\
,\x22\x5c\x5cinf\x22,\x22\x5c\x5clim\x22\
,\x22\x5c\x5cmax\x22,\x22\x5c\x5cmin\x22\
,\x22\x5c\x5cPr\x22,\x22\x5c\x5csup\x22]\
,props:{numArgs:\
0},handler:funct\
ion(e){var t=e.p\
arser,r=e.funcNa\
me;return{type:\x22\
op\x22,mode:t.mode,\
limits:!0,parent\
IsSupSub:!1,symb\
ol:!1,name:r}},h\
tmlBuilder:pn,ma\
thmlBuilder:dn})\
,ot({type:\x22op\x22,n\
ames:[\x22\x5c\x5cint\x22,\x22\x5c\
\x5ciint\x22,\x22\x5c\x5ciiint\x22\
,\x22\x5c\x5coint\x22,\x22\x5c\x5coii\
nt\x22,\x22\x5c\x5coiiint\x22,\x22\
\x5cu222b\x22,\x22\x5cu222c\x22\
,\x22\x5cu222d\x22,\x22\x5cu222\
e\x22,\x22\x5cu222f\x22,\x22\x5cu2\
230\x22],props:{num\
Args:0},handler:\
function(e){var \
t=e.parser,r=e.f\
uncName;return 1\
===r.length&&(r=\
gn[r]),{type:\x22op\
\x22,mode:t.mode,li\
mits:!1,parentIs\
SupSub:!1,symbol\
:!0,name:r}},htm\
lBuilder:pn,math\
mlBuilder:dn});v\
ar vn=function(e\
,t){var r,n,a,i,\
o=!1;if(\x22supsub\x22\
===e.type?(r=e.s\
up,n=e.sub,a=Ut(\
e.base,\x22operator\
name\x22),o=!0):a=U\
t(e,\x22operatornam\
e\x22),a.body.lengt\
h>0){for(var s=a\
.body.map((funct\
ion(e){var t=e.t\
ext;return\x22strin\
g\x22==typeof t?{ty\
pe:\x22textord\x22,mod\
e:e.mode,text:t}\
:e})),l=ft(s,t.w\
ithFont(\x22mathrm\x22\
),!0),h=0;h<l.le\
ngth;h++){var c=\
l[h];c instanceo\
f Z&&(c.text=c.t\
ext.replace(/\x5cu2\
212/,\x22-\x22).replac\
e(/\x5cu2217/,\x22*\x22))\
}i=Ke.makeSpan([\
\x22mop\x22],l,t)}else\
 i=Ke.makeSpan([\
\x22mop\x22],[],t);ret\
urn o?mn(i,r,n,t\
,t.style,0,0):i}\
;function bn(e,t\
,r){for(var n=ft\
(e,t,!1),a=t.siz\
eMultiplier/r.si\
zeMultiplier,i=0\
;i<n.length;i++)\
{var o=n[i].clas\
ses.indexOf(\x22siz\
ing\x22);o<0?Array.\
prototype.push.a\
pply(n[i].classe\
s,t.sizingClasse\
s(r)):n[i].class\
es[o+1]===\x22reset\
-size\x22+t.size&&(\
n[i].classes[o+1\
]=\x22reset-size\x22+r\
.size),n[i].heig\
ht*=a,n[i].depth\
*=a}return Ke.ma\
keFragment(n)}ot\
({type:\x22operator\
name\x22,names:[\x22\x5c\x5c\
operatorname@\x22,\x22\
\x5c\x5coperatornamewi\
thlimits\x22],props\
:{numArgs:1},han\
dler:function(e,\
t){var r=e.parse\
r,n=e.funcName,a\
=t[0];return{typ\
e:\x22operatorname\x22\
,mode:r.mode,bod\
y:ht(a),alwaysHa\
ndleSupSub:\x22\x5c\x5cop\
eratornamewithli\
mits\x22===n,limits\
:!1,parentIsSupS\
ub:!1}},htmlBuil\
der:vn,mathmlBui\
lder:function(e,\
t){for(var r=qt(\
e.body,t.withFon\
t(\x22mathrm\x22)),n=!\
0,a=0;a<r.length\
;a++){var i=r[a]\
;if(i instanceof\
 Tt.SpaceNode);e\
lse if(i instanc\
eof Tt.MathNode)\
switch(i.type){c\
ase\x22mi\x22:case\x22mn\x22\
:case\x22ms\x22:case\x22m\
space\x22:case\x22mtex\
t\x22:break;case\x22mo\
\x22:var o=i.childr\
en[0];1===i.chil\
dren.length&&o i\
nstanceof Tt.Tex\
tNode?o.text=o.t\
ext.replace(/\x5cu2\
212/,\x22-\x22).replac\
e(/\x5cu2217/,\x22*\x22):\
n=!1;break;defau\
lt:n=!1}else n=!\
1}if(n){var s=r.\
map((function(e)\
{return e.toText\
()})).join(\x22\x22);r\
=[new Tt.TextNod\
e(s)]}var l=new \
Tt.MathNode(\x22mi\x22\
,r);l.setAttribu\
te(\x22mathvariant\x22\
,\x22normal\x22);var h\
=new Tt.MathNode\
(\x22mo\x22,[Bt(\x22\x5cu206\
1\x22,\x22text\x22)]);ret\
urn e.parentIsSu\
pSub?new Tt.Math\
Node(\x22mrow\x22,[l,h\
]):Tt.newDocumen\
tFragment([l,h])\
}}),Pr(\x22\x5c\x5coperat\
orname\x22,\x22\x5c\x5c@ifst\
ar\x5c\x5coperatorname\
withlimits\x5c\x5coper\
atorname@\x22),st({\
type:\x22ordgroup\x22,\
htmlBuilder:func\
tion(e,t){return\
 e.semisimple?Ke\
.makeFragment(ft\
(e.body,t,!1)):K\
e.makeSpan([\x22mor\
d\x22],ft(e.body,t,\
!0),t)},mathmlBu\
ilder:function(e\
,t){return It(e.\
body,t,!0)}}),ot\
({type:\x22overline\
\x22,names:[\x22\x5c\x5cover\
line\x22],props:{nu\
mArgs:1},handler\
:function(e,t){v\
ar r=e.parser,n=\
t[0];return{type\
:\x22overline\x22,mode\
:r.mode,body:n}}\
,htmlBuilder:fun\
ction(e,t){var r\
=wt(e.body,t.hav\
ingCrampedStyle(\
)),n=Ke.makeLine\
Span(\x22overline-l\
ine\x22,t),a=t.font\
Metrics().defaul\
tRuleThickness,i\
=Ke.makeVList({p\
ositionType:\x22fir\
stBaseline\x22,chil\
dren:[{type:\x22ele\
m\x22,elem:r},{type\
:\x22kern\x22,size:3*a\
},{type:\x22elem\x22,e\
lem:n},{type:\x22ke\
rn\x22,size:a}]},t)\
;return Ke.makeS\
pan([\x22mord\x22,\x22ove\
rline\x22],[i],t)},\
mathmlBuilder:fu\
nction(e,t){var \
r=new Tt.MathNod\
e(\x22mo\x22,[new Tt.T\
extNode(\x22\x5cu203e\x22\
)]);r.setAttribu\
te(\x22stretchy\x22,\x22t\
rue\x22);var n=new \
Tt.MathNode(\x22mov\
er\x22,[Rt(e.body,t\
),r]);return n.s\
etAttribute(\x22acc\
ent\x22,\x22true\x22),n}}\
),ot({type:\x22phan\
tom\x22,names:[\x22\x5c\x5cp\
hantom\x22],props:{\
numArgs:1,allowe\
dInText:!0},hand\
ler:function(e,t\
){var r=e.parser\
,n=t[0];return{t\
ype:\x22phantom\x22,mo\
de:r.mode,body:h\
t(n)}},htmlBuild\
er:function(e,t)\
{var r=ft(e.body\
,t.withPhantom()\
,!1);return Ke.m\
akeFragment(r)},\
mathmlBuilder:fu\
nction(e,t){var \
r=qt(e.body,t);r\
eturn new Tt.Mat\
hNode(\x22mphantom\x22\
,r)}}),ot({type:\
\x22hphantom\x22,names\
:[\x22\x5c\x5chphantom\x22],\
props:{numArgs:1\
,allowedInText:!\
0},handler:funct\
ion(e,t){var r=e\
.parser,n=t[0];r\
eturn{type:\x22hpha\
ntom\x22,mode:r.mod\
e,body:n}},htmlB\
uilder:function(\
e,t){var r=Ke.ma\
keSpan([],[wt(e.\
body,t.withPhant\
om())]);if(r.hei\
ght=0,r.depth=0,\
r.children)for(v\
ar n=0;n<r.child\
ren.length;n++)r\
.children[n].hei\
ght=0,r.children\
[n].depth=0;retu\
rn r=Ke.makeVLis\
t({positionType:\
\x22firstBaseline\x22,\
children:[{type:\
\x22elem\x22,elem:r}]}\
,t),Ke.makeSpan(\
[\x22mord\x22],[r],t)}\
,mathmlBuilder:f\
unction(e,t){var\
 r=qt(ht(e.body)\
,t),n=new Tt.Mat\
hNode(\x22mphantom\x22\
,r),a=new Tt.Mat\
hNode(\x22mpadded\x22,\
[n]);return a.se\
tAttribute(\x22heig\
ht\x22,\x220px\x22),a.set\
Attribute(\x22depth\
\x22,\x220px\x22),a}}),ot\
({type:\x22vphantom\
\x22,names:[\x22\x5c\x5cvpha\
ntom\x22],props:{nu\
mArgs:1,allowedI\
nText:!0},handle\
r:function(e,t){\
var r=e.parser,n\
=t[0];return{typ\
e:\x22vphantom\x22,mod\
e:r.mode,body:n}\
},htmlBuilder:fu\
nction(e,t){var \
r=Ke.makeSpan([\x22\
inner\x22],[wt(e.bo\
dy,t.withPhantom\
())]),n=Ke.makeS\
pan([\x22fix\x22],[]);\
return Ke.makeSp\
an([\x22mord\x22,\x22rlap\
\x22],[r,n],t)},mat\
hmlBuilder:funct\
ion(e,t){var r=q\
t(ht(e.body),t),\
n=new Tt.MathNod\
e(\x22mphantom\x22,r),\
a=new Tt.MathNod\
e(\x22mpadded\x22,[n])\
;return a.setAtt\
ribute(\x22width\x22,\x22\
0px\x22),a}}),ot({t\
ype:\x22raisebox\x22,n\
ames:[\x22\x5c\x5craisebo\
x\x22],props:{numAr\
gs:2,argTypes:[\x22\
size\x22,\x22hbox\x22],al\
lowedInText:!0},\
handler:function\
(e,t){var r=e.pa\
rser,n=Ut(t[0],\x22\
size\x22).value,a=t\
[1];return{type:\
\x22raisebox\x22,mode:\
r.mode,dy:n,body\
:a}},htmlBuilder\
:function(e,t){v\
ar r=wt(e.body,t\
),n=P(e.dy,t);re\
turn Ke.makeVLis\
t({positionType:\
\x22shift\x22,position\
Data:-n,children\
:[{type:\x22elem\x22,e\
lem:r}]},t)},mat\
hmlBuilder:funct\
ion(e,t){var r=n\
ew Tt.MathNode(\x22\
mpadded\x22,[Rt(e.b\
ody,t)]),n=e.dy.\
number+e.dy.unit\
;return r.setAtt\
ribute(\x22voffset\x22\
,n),r}}),ot({typ\
e:\x22internal\x22,nam\
es:[\x22\x5c\x5crelax\x22],p\
rops:{numArgs:0,\
allowedInText:!0\
},handler:functi\
on(e){return{typ\
e:\x22internal\x22,mod\
e:e.parser.mode}\
}}),ot({type:\x22ru\
le\x22,names:[\x22\x5c\x5cru\
le\x22],props:{numA\
rgs:2,numOptiona\
lArgs:1,argTypes\
:[\x22size\x22,\x22size\x22,\
\x22size\x22]},handler\
:function(e,t,r)\
{var n=e.parser,\
a=r[0],i=Ut(t[0]\
,\x22size\x22),o=Ut(t[\
1],\x22size\x22);retur\
n{type:\x22rule\x22,mo\
de:n.mode,shift:\
a&&Ut(a,\x22size\x22).\
value,width:i.va\
lue,height:o.val\
ue}},htmlBuilder\
:function(e,t){v\
ar r=Ke.makeSpan\
([\x22mord\x22,\x22rule\x22]\
,[],t),n=P(e.wid\
th,t),a=P(e.heig\
ht,t),i=e.shift?\
P(e.shift,t):0;r\
eturn r.style.bo\
rderRightWidth=F\
(n),r.style.bord\
erTopWidth=F(a),\
r.style.bottom=F\
(i),r.width=n,r.\
height=a+i,r.dep\
th=-i,r.maxFontS\
ize=1.125*a*t.si\
zeMultiplier,r},\
mathmlBuilder:fu\
nction(e,t){var \
r=P(e.width,t),n\
=P(e.height,t),a\
=e.shift?P(e.shi\
ft,t):0,i=t.colo\
r&&t.getColor()|\
|\x22black\x22,o=new T\
t.MathNode(\x22mspa\
ce\x22);o.setAttrib\
ute(\x22mathbackgro\
und\x22,i),o.setAtt\
ribute(\x22width\x22,F\
(r)),o.setAttrib\
ute(\x22height\x22,F(n\
));var s=new Tt.\
MathNode(\x22mpadde\
d\x22,[o]);return a\
>=0?s.setAttribu\
te(\x22height\x22,F(a)\
):(s.setAttribut\
e(\x22height\x22,F(a))\
,s.setAttribute(\
\x22depth\x22,F(-a))),\
s.setAttribute(\x22\
voffset\x22,F(a)),s\
}});var yn=[\x22\x5c\x5ct\
iny\x22,\x22\x5c\x5csixptsiz\
e\x22,\x22\x5c\x5cscriptsize\
\x22,\x22\x5c\x5cfootnotesiz\
e\x22,\x22\x5c\x5csmall\x22,\x22\x5c\x5c\
normalsize\x22,\x22\x5c\x5cl\
arge\x22,\x22\x5c\x5cLarge\x22,\
\x22\x5c\x5cLARGE\x22,\x22\x5c\x5chug\
e\x22,\x22\x5c\x5cHuge\x22];ot(\
{type:\x22sizing\x22,n\
ames:yn,props:{n\
umArgs:0,allowed\
InText:!0},handl\
er:function(e,t)\
{var r=e.breakOn\
TokenText,n=e.fu\
ncName,a=e.parse\
r,i=a.parseExpre\
ssion(!1,r);retu\
rn{type:\x22sizing\x22\
,mode:a.mode,siz\
e:yn.indexOf(n)+\
1,body:i}},htmlB\
uilder:function(\
e,t){var r=t.hav\
ingSize(e.size);\
return bn(e.body\
,r,t)},mathmlBui\
lder:function(e,\
t){var r=t.havin\
gSize(e.size),n=\
qt(e.body,r),a=n\
ew Tt.MathNode(\x22\
mstyle\x22,n);retur\
n a.setAttribute\
(\x22mathsize\x22,F(r.\
sizeMultiplier))\
,a}}),ot({type:\x22\
smash\x22,names:[\x22\x5c\
\x5csmash\x22],props:{\
numArgs:1,numOpt\
ionalArgs:1,allo\
wedInText:!0},ha\
ndler:function(e\
,t,r){var n=e.pa\
rser,a=!1,i=!1,o\
=r[0]&&Ut(r[0],\x22\
ordgroup\x22);if(o)\
for(var s=\x22\x22,l=0\
;l<o.body.length\
;++l){if(\x22t\x22===(\
s=o.body[l].text\
))a=!0;else{if(\x22\
b\x22!==s){a=!1,i=!\
1;break}i=!0}}el\
se a=!0,i=!0;var\
 h=t[0];return{t\
ype:\x22smash\x22,mode\
:n.mode,body:h,s\
mashHeight:a,sma\
shDepth:i}},html\
Builder:function\
(e,t){var r=Ke.m\
akeSpan([],[wt(e\
.body,t)]);if(!e\
.smashHeight&&!e\
.smashDepth)retu\
rn r;if(e.smashH\
eight&&(r.height\
=0,r.children))f\
or(var n=0;n<r.c\
hildren.length;n\
++)r.children[n]\
.height=0;if(e.s\
mashDepth&&(r.de\
pth=0,r.children\
))for(var a=0;a<\
r.children.lengt\
h;a++)r.children\
[a].depth=0;var \
i=Ke.makeVList({\
positionType:\x22fi\
rstBaseline\x22,chi\
ldren:[{type:\x22el\
em\x22,elem:r}]},t)\
;return Ke.makeS\
pan([\x22mord\x22],[i]\
,t)},mathmlBuild\
er:function(e,t)\
{var r=new Tt.Ma\
thNode(\x22mpadded\x22\
,[Rt(e.body,t)])\
;return e.smashH\
eight&&r.setAttr\
ibute(\x22height\x22,\x22\
0px\x22),e.smashDep\
th&&r.setAttribu\
te(\x22depth\x22,\x220px\x22\
),r}}),ot({type:\
\x22sqrt\x22,names:[\x22\x5c\
\x5csqrt\x22],props:{n\
umArgs:1,numOpti\
onalArgs:1},hand\
ler:function(e,t\
,r){var n=e.pars\
er,a=r[0],i=t[0]\
;return{type:\x22sq\
rt\x22,mode:n.mode,\
body:i,index:a}}\
,htmlBuilder:fun\
ction(e,t){var r\
=wt(e.body,t.hav\
ingCrampedStyle(\
));0===r.height&\
&(r.height=t.fon\
tMetrics().xHeig\
ht),r=Ke.wrapFra\
gment(r,t);var n\
=t.fontMetrics()\
.defaultRuleThic\
kness,a=n;t.styl\
e.id<x.TEXT.id&&\
(a=t.fontMetrics\
().xHeight);var \
i=n+a/4,o=r.heig\
ht+r.depth+i+n,s\
=Nr.sqrtImage(o,\
t),l=s.span,h=s.\
ruleWidth,c=s.ad\
vanceWidth,m=l.h\
eight-h;m>r.heig\
ht+r.depth+i&&(i\
=(i+m-r.height-r\
.depth)/2);var u\
=l.height-r.heig\
ht-i-h;r.style.p\
addingLeft=F(c);\
var p=Ke.makeVLi\
st({positionType\
:\x22firstBaseline\x22\
,children:[{type\
:\x22elem\x22,elem:r,w\
rapperClasses:[\x22\
svg-align\x22]},{ty\
pe:\x22kern\x22,size:-\
(r.height+u)},{t\
ype:\x22elem\x22,elem:\
l},{type:\x22kern\x22,\
size:h}]},t);if(\
e.index){var d=t\
.havingStyle(x.S\
CRIPTSCRIPT),f=w\
t(e.index,d,t),g\
=.6*(p.height-p.\
depth),v=Ke.make\
VList({positionT\
ype:\x22shift\x22,posi\
tionData:-g,chil\
dren:[{type:\x22ele\
m\x22,elem:f}]},t),\
b=Ke.makeSpan([\x22\
root\x22],[v]);retu\
rn Ke.makeSpan([\
\x22mord\x22,\x22sqrt\x22],[\
b,p],t)}return K\
e.makeSpan([\x22mor\
d\x22,\x22sqrt\x22],[p],t\
)},mathmlBuilder\
:function(e,t){v\
ar r=e.body,n=e.\
index;return n?n\
ew Tt.MathNode(\x22\
mroot\x22,[Rt(r,t),\
Rt(n,t)]):new Tt\
.MathNode(\x22msqrt\
\x22,[Rt(r,t)])}});\
var xn={display:\
x.DISPLAY,text:x\
.TEXT,script:x.S\
CRIPT,scriptscri\
pt:x.SCRIPTSCRIP\
T};ot({type:\x22sty\
ling\x22,names:[\x22\x5c\x5c\
displaystyle\x22,\x22\x5c\
\x5ctextstyle\x22,\x22\x5c\x5cs\
criptstyle\x22,\x22\x5c\x5cs\
criptscriptstyle\
\x22],props:{numArg\
s:0,allowedInTex\
t:!0,primitive:!\
0},handler:funct\
ion(e,t){var r=e\
.breakOnTokenTex\
t,n=e.funcName,a\
=e.parser,i=a.pa\
rseExpression(!0\
,r),o=n.slice(1,\
n.length-5);retu\
rn{type:\x22styling\
\x22,mode:a.mode,st\
yle:o,body:i}},h\
tmlBuilder:funct\
ion(e,t){var r=x\
n[e.style],n=t.h\
avingStyle(r).wi\
thFont(\x22\x22);retur\
n bn(e.body,n,t)\
},mathmlBuilder:\
function(e,t){va\
r r=xn[e.style],\
n=t.havingStyle(\
r),a=qt(e.body,n\
),i=new Tt.MathN\
ode(\x22mstyle\x22,a),\
o={display:[\x220\x22,\
\x22true\x22],text:[\x220\
\x22,\x22false\x22],scrip\
t:[\x221\x22,\x22false\x22],\
scriptscript:[\x222\
\x22,\x22false\x22]}[e.st\
yle];return i.se\
tAttribute(\x22scri\
ptlevel\x22,o[0]),i\
.setAttribute(\x22d\
isplaystyle\x22,o[1\
]),i}});var wn=f\
unction(e,t){var\
 r=e.base;return\
 r?\x22op\x22===r.type\
?r.limits&&(t.st\
yle.size===x.DIS\
PLAY.size||r.alw\
aysHandleSupSub)\
?pn:null:\x22operat\
orname\x22===r.type\
?r.alwaysHandleS\
upSub&&(t.style.\
size===x.DISPLAY\
.size||r.limits)\
?vn:null:\x22accent\
\x22===r.type?l.isC\
haracterBox(r.ba\
se)?Wt:null:\x22hor\
izBrace\x22===r.typ\
e&&!e.sub===r.is\
Over?ln:null:nul\
l};st({type:\x22sup\
sub\x22,htmlBuilder\
:function(e,t){v\
ar r=wn(e,t);if(\
r)return r(e,t);\
var n,a,i,o=e.ba\
se,s=e.sup,h=e.s\
ub,c=wt(o,t),m=t\
.fontMetrics(),u\
=0,p=0,d=o&&l.is\
CharacterBox(o);\
if(s){var f=t.ha\
vingStyle(t.styl\
e.sup());n=wt(s,\
f,t),d||(u=c.hei\
ght-f.fontMetric\
s().supDrop*f.si\
zeMultiplier/t.s\
izeMultiplier)}i\
f(h){var g=t.hav\
ingStyle(t.style\
.sub());a=wt(h,g\
,t),d||(p=c.dept\
h+g.fontMetrics(\
).subDrop*g.size\
Multiplier/t.siz\
eMultiplier)}i=t\
.style===x.DISPL\
AY?m.sup1:t.styl\
e.cramped?m.sup3\
:m.sup2;var v,b=\
t.sizeMultiplier\
,y=F(.5/m.ptPerE\
m/b),w=null;if(a\
){var k=e.base&&\
\x22op\x22===e.base.ty\
pe&&e.base.name&\
&(\x22\x5c\x5coiint\x22===e.\
base.name||\x22\x5c\x5coi\
iint\x22===e.base.n\
ame);(c instance\
of Z||k)&&(w=F(-\
c.italic))}if(n&\
&a){u=Math.max(u\
,i,n.depth+.25*m\
.xHeight),p=Math\
.max(p,m.sub2);v\
ar S=4*m.default\
RuleThickness;if\
(u-n.depth-(a.he\
ight-p)<S){p=S-(\
u-n.depth)+a.hei\
ght;var M=.8*m.x\
Height-(u-n.dept\
h);M>0&&(u+=M,p-\
=M)}var z=[{type\
:\x22elem\x22,elem:a,s\
hift:p,marginRig\
ht:y,marginLeft:\
w},{type:\x22elem\x22,\
elem:n,shift:-u,\
marginRight:y}];\
v=Ke.makeVList({\
positionType:\x22in\
dividualShift\x22,c\
hildren:z},t)}el\
se if(a){p=Math.\
max(p,m.sub1,a.h\
eight-.8*m.xHeig\
ht);var A=[{type\
:\x22elem\x22,elem:a,m\
arginLeft:w,marg\
inRight:y}];v=Ke\
.makeVList({posi\
tionType:\x22shift\x22\
,positionData:p,\
children:A},t)}e\
lse{if(!n)throw \
new Error(\x22supsu\
b must have eith\
er sup or sub.\x22)\
;u=Math.max(u,i,\
n.depth+.25*m.xH\
eight),v=Ke.make\
VList({positionT\
ype:\x22shift\x22,posi\
tionData:-u,chil\
dren:[{type:\x22ele\
m\x22,elem:n,margin\
Right:y}]},t)}va\
r T=yt(c,\x22right\x22\
)||\x22mord\x22;return\
 Ke.makeSpan([T]\
,[c,Ke.makeSpan(\
[\x22msupsub\x22],[v])\
],t)},mathmlBuil\
der:function(e,t\
){var r,n=!1;e.b\
ase&&\x22horizBrace\
\x22===e.base.type&\
&!!e.sup===e.bas\
e.isOver&&(n=!0,\
r=e.base.isOver)\
,!e.base||\x22op\x22!=\
=e.base.type&&\x22o\
peratorname\x22!==e\
.base.type||(e.b\
ase.parentIsSupS\
ub=!0);var a,i=[\
Rt(e.base,t)];if\
(e.sub&&i.push(R\
t(e.sub,t)),e.su\
p&&i.push(Rt(e.s\
up,t)),n)a=r?\x22mo\
ver\x22:\x22munder\x22;el\
se if(e.sub)if(e\
.sup){var o=e.ba\
se;a=o&&\x22op\x22===o\
.type&&o.limits&\
&t.style===x.DIS\
PLAY||o&&\x22operat\
orname\x22===o.type\
&&o.alwaysHandle\
SupSub&&(t.style\
===x.DISPLAY||o.\
limits)?\x22mundero\
ver\x22:\x22msubsup\x22}e\
lse{var s=e.base\
;a=s&&\x22op\x22===s.t\
ype&&s.limits&&(\
t.style===x.DISP\
LAY||s.alwaysHan\
dleSupSub)||s&&\x22\
operatorname\x22===\
s.type&&s.always\
HandleSupSub&&(s\
.limits||t.style\
===x.DISPLAY)?\x22m\
under\x22:\x22msub\x22}el\
se{var l=e.base;\
a=l&&\x22op\x22===l.ty\
pe&&l.limits&&(t\
.style===x.DISPL\
AY||l.alwaysHand\
leSupSub)||l&&\x22o\
peratorname\x22===l\
.type&&l.alwaysH\
andleSupSub&&(l.\
limits||t.style=\
==x.DISPLAY)?\x22mo\
ver\x22:\x22msup\x22}retu\
rn new Tt.MathNo\
de(a,i)}}),st({t\
ype:\x22atom\x22,htmlB\
uilder:function(\
e,t){return Ke.m\
athsym(e.text,e.\
mode,t,[\x22m\x22+e.fa\
mily])},mathmlBu\
ilder:function(e\
,t){var r=new Tt\
.MathNode(\x22mo\x22,[\
Bt(e.text,e.mode\
)]);if(\x22bin\x22===e\
.family){var n=N\
t(e,t);\x22bold-ita\
lic\x22===n&&r.setA\
ttribute(\x22mathva\
riant\x22,n)}else\x22p\
unct\x22===e.family\
?r.setAttribute(\
\x22separator\x22,\x22tru\
e\x22):\x22open\x22!==e.f\
amily&&\x22close\x22!=\
=e.family||r.set\
Attribute(\x22stret\
chy\x22,\x22false\x22);re\
turn r}});var kn\
={mi:\x22italic\x22,mn\
:\x22normal\x22,mtext:\
\x22normal\x22};st({ty\
pe:\x22mathord\x22,htm\
lBuilder:functio\
n(e,t){return Ke\
.makeOrd(e,t,\x22ma\
thord\x22)},mathmlB\
uilder:function(\
e,t){var r=new T\
t.MathNode(\x22mi\x22,\
[Bt(e.text,e.mod\
e,t)]),n=Nt(e,t)\
||\x22italic\x22;retur\
n n!==kn[r.type]\
&&r.setAttribute\
(\x22mathvariant\x22,n\
),r}}),st({type:\
\x22textord\x22,htmlBu\
ilder:function(e\
,t){return Ke.ma\
keOrd(e,t,\x22texto\
rd\x22)},mathmlBuil\
der:function(e,t\
){var r,n=Bt(e.t\
ext,e.mode,t),a=\
Nt(e,t)||\x22normal\
\x22;return r=\x22text\
\x22===e.mode?new T\
t.MathNode(\x22mtex\
t\x22,[n]):/[0-9]/.\
test(e.text)?new\
 Tt.MathNode(\x22mn\
\x22,[n]):\x22\x5c\x5cprime\x22\
===e.text?new Tt\
.MathNode(\x22mo\x22,[\
n]):new Tt.MathN\
ode(\x22mi\x22,[n]),a!\
==kn[r.type]&&r.\
setAttribute(\x22ma\
thvariant\x22,a),r}\
});var Sn={\x22\x5c\x5cno\
break\x22:\x22nobreak\x22\
,\x22\x5c\x5callowbreak\x22:\
\x22allowbreak\x22},Mn\
={\x22 \x22:{},\x22\x5c\x5c \x22:{\
},\x22~\x22:{className\
:\x22nobreak\x22},\x22\x5c\x5cs\
pace\x22:{},\x22\x5c\x5cnobr\
eakspace\x22:{class\
Name:\x22nobreak\x22}}\
;st({type:\x22spaci\
ng\x22,htmlBuilder:\
function(e,t){if\
(Mn.hasOwnProper\
ty(e.text)){var \
r=Mn[e.text].cla\
ssName||\x22\x22;if(\x22t\
ext\x22===e.mode){v\
ar a=Ke.makeOrd(\
e,t,\x22textord\x22);r\
eturn a.classes.\
push(r),a}return\
 Ke.makeSpan([\x22m\
space\x22,r],[Ke.ma\
thsym(e.text,e.m\
ode,t)],t)}if(Sn\
.hasOwnProperty(\
e.text))return K\
e.makeSpan([\x22msp\
ace\x22,Sn[e.text]]\
,[],t);throw new\
 n('Unknown type\
 of space \x22'+e.t\
ext+'\x22')},mathml\
Builder:function\
(e,t){if(!Mn.has\
OwnProperty(e.te\
xt)){if(Sn.hasOw\
nProperty(e.text\
))return new Tt.\
MathNode(\x22mspace\
\x22);throw new n('\
Unknown type of \
space \x22'+e.text+\
'\x22')}return new \
Tt.MathNode(\x22mte\
xt\x22,[new Tt.Text\
Node(\x22\x5cxa0\x22)])}}\
);var zn=functio\
n(){var e=new Tt\
.MathNode(\x22mtd\x22,\
[]);return e.set\
Attribute(\x22width\
\x22,\x2250%\x22),e};st({\
type:\x22tag\x22,mathm\
lBuilder:functio\
n(e,t){var r=new\
 Tt.MathNode(\x22mt\
able\x22,[new Tt.Ma\
thNode(\x22mtr\x22,[zn\
(),new Tt.MathNo\
de(\x22mtd\x22,[It(e.b\
ody,t)]),zn(),ne\
w Tt.MathNode(\x22m\
td\x22,[It(e.tag,t)\
])])]);return r.\
setAttribute(\x22wi\
dth\x22,\x22100%\x22),r}}\
);var An={\x22\x5c\x5ctex\
t\x22:void 0,\x22\x5c\x5ctex\
trm\x22:\x22textrm\x22,\x22\x5c\
\x5ctextsf\x22:\x22textsf\
\x22,\x22\x5c\x5ctexttt\x22:\x22te\
xttt\x22,\x22\x5c\x5ctextnor\
mal\x22:\x22textrm\x22},T\
n={\x22\x5c\x5ctextbf\x22:\x22t\
extbf\x22,\x22\x5c\x5ctextmd\
\x22:\x22textmd\x22},Bn={\
\x22\x5c\x5ctextit\x22:\x22text\
it\x22,\x22\x5c\x5ctextup\x22:\x22\
textup\x22},Cn=func\
tion(e,t){var r=\
e.font;return r?\
An[r]?t.withText\
FontFamily(An[r]\
):Tn[r]?t.withTe\
xtFontWeight(Tn[\
r]):t.withTextFo\
ntShape(Bn[r]):t\
};ot({type:\x22text\
\x22,names:[\x22\x5c\x5ctext\
\x22,\x22\x5c\x5ctextrm\x22,\x22\x5c\x5c\
textsf\x22,\x22\x5c\x5ctextt\
t\x22,\x22\x5c\x5ctextnormal\
\x22,\x22\x5c\x5ctextbf\x22,\x22\x5c\x5c\
textmd\x22,\x22\x5c\x5ctexti\
t\x22,\x22\x5c\x5ctextup\x22],p\
rops:{numArgs:1,\
argTypes:[\x22text\x22\
],allowedInArgum\
ent:!0,allowedIn\
Text:!0},handler\
:function(e,t){v\
ar r=e.parser,n=\
e.funcName,a=t[0\
];return{type:\x22t\
ext\x22,mode:r.mode\
,body:ht(a),font\
:n}},htmlBuilder\
:function(e,t){v\
ar r=Cn(e,t),n=f\
t(e.body,r,!0);r\
eturn Ke.makeSpa\
n([\x22mord\x22,\x22text\x22\
],n,r)},mathmlBu\
ilder:function(e\
,t){var r=Cn(e,t\
);return It(e.bo\
dy,r)}}),ot({typ\
e:\x22underline\x22,na\
mes:[\x22\x5c\x5cunderlin\
e\x22],props:{numAr\
gs:1,allowedInTe\
xt:!0},handler:f\
unction(e,t){ret\
urn{type:\x22underl\
ine\x22,mode:e.pars\
er.mode,body:t[0\
]}},htmlBuilder:\
function(e,t){va\
r r=wt(e.body,t)\
,n=Ke.makeLineSp\
an(\x22underline-li\
ne\x22,t),a=t.fontM\
etrics().default\
RuleThickness,i=\
Ke.makeVList({po\
sitionType:\x22top\x22\
,positionData:r.\
height,children:\
[{type:\x22kern\x22,si\
ze:a},{type:\x22ele\
m\x22,elem:n},{type\
:\x22kern\x22,size:3*a\
},{type:\x22elem\x22,e\
lem:r}]},t);retu\
rn Ke.makeSpan([\
\x22mord\x22,\x22underlin\
e\x22],[i],t)},math\
mlBuilder:functi\
on(e,t){var r=ne\
w Tt.MathNode(\x22m\
o\x22,[new Tt.TextN\
ode(\x22\x5cu203e\x22)]);\
r.setAttribute(\x22\
stretchy\x22,\x22true\x22\
);var n=new Tt.M\
athNode(\x22munder\x22\
,[Rt(e.body,t),r\
]);return n.setA\
ttribute(\x22accent\
under\x22,\x22true\x22),n\
}}),ot({type:\x22vc\
enter\x22,names:[\x22\x5c\
\x5cvcenter\x22],props\
:{numArgs:1,argT\
ypes:[\x22original\x22\
],allowedInText:\
!1},handler:func\
tion(e,t){return\
{type:\x22vcenter\x22,\
mode:e.parser.mo\
de,body:t[0]}},h\
tmlBuilder:funct\
ion(e,t){var r=w\
t(e.body,t),n=t.\
fontMetrics().ax\
isHeight,a=.5*(r\
.height-n-(r.dep\
th+n));return Ke\
.makeVList({posi\
tionType:\x22shift\x22\
,positionData:a,\
children:[{type:\
\x22elem\x22,elem:r}]}\
,t)},mathmlBuild\
er:function(e,t)\
{return new Tt.M\
athNode(\x22mpadded\
\x22,[Rt(e.body,t)]\
,[\x22vcenter\x22])}})\
,ot({type:\x22verb\x22\
,names:[\x22\x5c\x5cverb\x22\
],props:{numArgs\
:0,allowedInText\
:!0},handler:fun\
ction(e,t,r){thr\
ow new n(\x22\x5c\x5cverb\
 ended by end of\
 line instead of\
 matching delimi\
ter\x22)},htmlBuild\
er:function(e,t)\
{for(var r=Nn(e)\
,n=[],a=t.having\
Style(t.style.te\
xt()),i=0;i<r.le\
ngth;i++){var o=\
r[i];\x22~\x22===o&&(o\
=\x22\x5c\x5ctextasciitil\
de\x22),n.push(Ke.m\
akeSymbol(o,\x22Typ\
ewriter-Regular\x22\
,e.mode,a,[\x22mord\
\x22,\x22texttt\x22]))}re\
turn Ke.makeSpan\
([\x22mord\x22,\x22text\x22]\
.concat(a.sizing\
Classes(t)),Ke.t\
ryCombineChars(n\
),a)},mathmlBuil\
der:function(e,t\
){var r=new Tt.T\
extNode(Nn(e)),n\
=new Tt.MathNode\
(\x22mtext\x22,[r]);re\
turn n.setAttrib\
ute(\x22mathvariant\
\x22,\x22monospace\x22),n\
}});var Nn=funct\
ion(e){return e.\
body.replace(/ /\
g,e.star?\x22\x5cu2423\
\x22:\x22\x5cxa0\x22)},qn=nt\
,In=\x22[ \x5cr\x5cn\x5ct]\x22,\
Rn=\x22(\x5c\x5c\x5c\x5c[a-zA-Z\
@]+)\x22+In+\x22*\x22,Hn=\
\x22[\x5cu0300-\x5cu036f]\
\x22,On=new RegExp(\
Hn+\x22+$\x22),En=\x22(\x22+\
In+\x22+)|\x5c\x5c\x5c\x5c(\x5cn|[\
 \x5cr\x5ct]+\x5cn?)[ \x5cr\x5c\
t]*|([!-\x5c\x5c[\x5c\x5c]-\x5c\
u2027\x5cu202a-\x5cud7\
ff\x5cuf900-\x5cuffff]\
\x22+Hn+\x22*|[\x5cud800-\
\x5cudbff][\x5cudc00-\x5c\
udfff]\x22+Hn+\x22*|\x5c\x5c\
\x5c\x5cverb\x5c\x5c*([^]).*\
?\x5c\x5c4|\x5c\x5c\x5c\x5cverb([^\
*a-zA-Z]).*?\x5c\x5c5|\
\x22+Rn+\x22|\x5c\x5c\x5c\x5c[^\x5cud\
800-\x5cudfff])\x22,Ln\
=function(){func\
tion e(e,t){this\
.input=void 0,th\
is.settings=void\
 0,this.tokenReg\
ex=void 0,this.c\
atcodes=void 0,t\
his.input=e,this\
.settings=t,this\
.tokenRegex=new \
RegExp(En,\x22g\x22),t\
his.catcodes={\x22%\
\x22:14,\x22~\x22:13}}var\
 t=e.prototype;r\
eturn t.setCatco\
de=function(e,t)\
{this.catcodes[e\
]=t},t.lex=funct\
ion(){var e=this\
.input,t=this.to\
kenRegex.lastInd\
ex;if(t===e.leng\
th)return new Gr\
(\x22EOF\x22,new Fr(th\
is,t,t));var r=t\
his.tokenRegex.e\
xec(e);if(null==\
=r||r.index!==t)\
throw new n(\x22Une\
xpected characte\
r: '\x22+e[t]+\x22'\x22,n\
ew Gr(e[t],new F\
r(this,t,t+1)));\
var a=r[6]||r[3]\
||(r[2]?\x22\x5c\x5c \x22:\x22 \
\x22);if(14===this.\
catcodes[a]){var\
 i=e.indexOf(\x22\x5cn\
\x22,this.tokenRege\
x.lastIndex);ret\
urn-1===i?(this.\
tokenRegex.lastI\
ndex=e.length,th\
is.settings.repo\
rtNonstrict(\x22com\
mentAtEnd\x22,\x22% co\
mment has no ter\
minating newline\
; LaTeX would fa\
il because of co\
mmenting the end\
 of math mode (e\
.g. $)\x22)):this.t\
okenRegex.lastIn\
dex=i+1,this.lex\
()}return new Gr\
(a,new Fr(this,t\
,this.tokenRegex\
.lastIndex))},e}\
(),Dn=function()\
{function e(e,t)\
{void 0===e&&(e=\
{}),void 0===t&&\
(t={}),this.curr\
ent=void 0,this.\
builtins=void 0,\
this.undefStack=\
void 0,this.curr\
ent=t,this.built\
ins=e,this.undef\
Stack=[]}var t=e\
.prototype;retur\
n t.beginGroup=f\
unction(){this.u\
ndefStack.push({\
})},t.endGroup=f\
unction(){if(0==\
=this.undefStack\
.length)throw ne\
w n(\x22Unbalanced \
namespace destru\
ction: attempt t\
o pop global nam\
espace; please r\
eport this as a \
bug\x22);var e=this\
.undefStack.pop(\
);for(var t in e\
)e.hasOwnPropert\
y(t)&&(null==e[t\
]?delete this.cu\
rrent[t]:this.cu\
rrent[t]=e[t])},\
t.endGroups=func\
tion(){for(;this\
.undefStack.leng\
th>0;)this.endGr\
oup()},t.has=fun\
ction(e){return \
this.current.has\
OwnProperty(e)||\
this.builtins.ha\
sOwnProperty(e)}\
,t.get=function(\
e){return this.c\
urrent.hasOwnPro\
perty(e)?this.cu\
rrent[e]:this.bu\
iltins[e]},t.set\
=function(e,t,r)\
{if(void 0===r&&\
(r=!1),r){for(va\
r n=0;n<this.und\
efStack.length;n\
++)delete this.u\
ndefStack[n][e];\
this.undefStack.\
length>0&&(this.\
undefStack[this.\
undefStack.lengt\
h-1][e]=t)}else{\
var a=this.undef\
Stack[this.undef\
Stack.length-1];\
a&&!a.hasOwnProp\
erty(e)&&(a[e]=t\
his.current[e])}\
null==t?delete t\
his.current[e]:t\
his.current[e]=t\
},e}(),Vn=Vr;Pr(\
\x22\x5c\x5cnoexpand\x22,(fu\
nction(e){var t=\
e.popToken();ret\
urn e.isExpandab\
le(t.text)&&(t.n\
oexpand=!0,t.tre\
atAsRelax=!0),{t\
okens:[t],numArg\
s:0}})),Pr(\x22\x5c\x5cex\
pandafter\x22,(func\
tion(e){var t=e.\
popToken();retur\
n e.expandOnce(!\
0),{tokens:[t],n\
umArgs:0}})),Pr(\
\x22\x5c\x5c@firstoftwo\x22,\
(function(e){ret\
urn{tokens:e.con\
sumeArgs(2)[0],n\
umArgs:0}})),Pr(\
\x22\x5c\x5c@secondoftwo\x22\
,(function(e){re\
turn{tokens:e.co\
nsumeArgs(2)[1],\
numArgs:0}})),Pr\
(\x22\x5c\x5c@ifnextchar\x22\
,(function(e){va\
r t=e.consumeArg\
s(3);e.consumeSp\
aces();var r=e.f\
uture();return 1\
===t[0].length&&\
t[0][0].text===r\
.text?{tokens:t[\
1],numArgs:0}:{t\
okens:t[2],numAr\
gs:0}})),Pr(\x22\x5c\x5c@\
ifstar\x22,\x22\x5c\x5c@ifne\
xtchar *{\x5c\x5c@firs\
toftwo{#1}}\x22),Pr\
(\x22\x5c\x5cTextOrMath\x22,\
(function(e){var\
 t=e.consumeArgs\
(2);return\x22text\x22\
===e.mode?{token\
s:t[0],numArgs:0\
}:{tokens:t[1],n\
umArgs:0}}));var\
 Pn={0:0,1:1,2:2\
,3:3,4:4,5:5,6:6\
,7:7,8:8,9:9,a:1\
0,A:10,b:11,B:11\
,c:12,C:12,d:13,\
D:13,e:14,E:14,f\
:15,F:15};Pr(\x22\x5c\x5c\
char\x22,(function(\
e){var t,r=e.pop\
Token(),a=\x22\x22;if(\
\x22'\x22===r.text)t=8\
,r=e.popToken();\
else if('\x22'===r.\
text)t=16,r=e.po\
pToken();else if\
(\x22`\x22===r.text)if\
(\x22\x5c\x5c\x22===(r=e.pop\
Token()).text[0]\
)a=r.text.charCo\
deAt(1);else{if(\
\x22EOF\x22===r.text)t\
hrow new n(\x22\x5c\x5cch\
ar` missing argu\
ment\x22);a=r.text.\
charCodeAt(0)}el\
se t=10;if(t){if\
(null==(a=Pn[r.t\
ext])||a>=t)thro\
w new n(\x22Invalid\
 base-\x22+t+\x22 digi\
t \x22+r.text);for(\
var i;null!=(i=P\
n[e.future().tex\
t])&&i<t;)a*=t,a\
+=i,e.popToken()\
}return\x22\x5c\x5c@char{\
\x22+a+\x22}\x22}));var F\
n=function(e,t,r\
){var a=e.consum\
eArg().tokens;if\
(1!==a.length)th\
row new n(\x22\x5c\x5cnew\
command's first \
argument must be\
 a macro name\x22);\
var i=a[0].text,\
o=e.isDefined(i)\
;if(o&&!t)throw \
new n(\x22\x5c\x5cnewcomm\
and{\x22+i+\x22} attem\
pting to redefin\
e \x22+i+\x22; use \x5c\x5cr\
enewcommand\x22);if\
(!o&&!r)throw ne\
w n(\x22\x5c\x5crenewcomm\
and{\x22+i+\x22} when \
command \x22+i+\x22 do\
es not yet exist\
; use \x5c\x5cnewcomma\
nd\x22);var s=0;if(\
1===(a=e.consume\
Arg().tokens).le\
ngth&&\x22[\x22===a[0]\
.text){for(var l\
=\x22\x22,h=e.expandNe\
xtToken();\x22]\x22!==\
h.text&&\x22EOF\x22!==\
h.text;)l+=h.tex\
t,h=e.expandNext\
Token();if(!l.ma\
tch(/^\x5cs*[0-9]+\x5c\
s*$/))throw new \
n(\x22Invalid numbe\
r of arguments: \
\x22+l);s=parseInt(\
l),a=e.consumeAr\
g().tokens}retur\
n e.macros.set(i\
,{tokens:a,numAr\
gs:s}),\x22\x22};Pr(\x22\x5c\
\x5cnewcommand\x22,(fu\
nction(e){return\
 Fn(e,!1,!0)})),\
Pr(\x22\x5c\x5crenewcomma\
nd\x22,(function(e)\
{return Fn(e,!0,\
!1)})),Pr(\x22\x5c\x5cpro\
videcommand\x22,(fu\
nction(e){return\
 Fn(e,!0,!0)})),\
Pr(\x22\x5c\x5cmessage\x22,(\
function(e){var \
t=e.consumeArgs(\
1)[0];return con\
sole.log(t.rever\
se().map((functi\
on(e){return e.t\
ext})).join(\x22\x22))\
,\x22\x22})),Pr(\x22\x5c\x5cerr\
message\x22,(functi\
on(e){var t=e.co\
nsumeArgs(1)[0];\
return console.e\
rror(t.reverse()\
.map((function(e\
){return e.text}\
)).join(\x22\x22)),\x22\x22}\
)),Pr(\x22\x5c\x5cshow\x22,(\
function(e){var \
t=e.popToken(),r\
=t.text;return c\
onsole.log(t,e.m\
acros.get(r),qn[\
r],ae.math[r],ae\
.text[r]),\x22\x22})),\
Pr(\x22\x5c\x5cbgroup\x22,\x22{\
\x22),Pr(\x22\x5c\x5cegroup\x22\
,\x22}\x22),Pr(\x22~\x22,\x22\x5c\x5c\
nobreakspace\x22),P\
r(\x22\x5c\x5clq\x22,\x22`\x22),Pr\
(\x22\x5c\x5crq\x22,\x22'\x22),Pr(\
\x22\x5c\x5caa\x22,\x22\x5c\x5cr a\x22),\
Pr(\x22\x5c\x5cAA\x22,\x22\x5c\x5cr A\
\x22),Pr(\x22\x5c\x5ctextcop\
yright\x22,\x22\x5c\x5chtml@\
mathml{\x5c\x5ctextcir\
cled{c}}{\x5c\x5cchar`\
\x5cxa9}\x22),Pr(\x22\x5c\x5cco\
pyright\x22,\x22\x5c\x5cText\
OrMath{\x5c\x5ctextcop\
yright}{\x5c\x5ctext{\x5c\
\x5ctextcopyright}}\
\x22),Pr(\x22\x5c\x5ctextreg\
istered\x22,\x22\x5c\x5chtml\
@mathml{\x5c\x5ctextci\
rcled{\x5c\x5cscriptsi\
ze R}}{\x5c\x5cchar`\x5cx\
ae}\x22),Pr(\x22\x5cu212c\
\x22,\x22\x5c\x5cmathscr{B}\x22\
),Pr(\x22\x5cu2130\x22,\x22\x5c\
\x5cmathscr{E}\x22),Pr\
(\x22\x5cu2131\x22,\x22\x5c\x5cmat\
hscr{F}\x22),Pr(\x22\x5cu\
210b\x22,\x22\x5c\x5cmathscr\
{H}\x22),Pr(\x22\x5cu2110\
\x22,\x22\x5c\x5cmathscr{I}\x22\
),Pr(\x22\x5cu2112\x22,\x22\x5c\
\x5cmathscr{L}\x22),Pr\
(\x22\x5cu2133\x22,\x22\x5c\x5cmat\
hscr{M}\x22),Pr(\x22\x5cu\
211b\x22,\x22\x5c\x5cmathscr\
{R}\x22),Pr(\x22\x5cu212d\
\x22,\x22\x5c\x5cmathfrak{C}\
\x22),Pr(\x22\x5cu210c\x22,\x22\
\x5c\x5cmathfrak{H}\x22),\
Pr(\x22\x5cu2128\x22,\x22\x5c\x5cm\
athfrak{Z}\x22),Pr(\
\x22\x5c\x5cBbbk\x22,\x22\x5c\x5cBbb{\
k}\x22),Pr(\x22\x5cxb7\x22,\x22\
\x5c\x5ccdotp\x22),Pr(\x22\x5c\x5c\
llap\x22,\x22\x5c\x5cmathlla\
p{\x5c\x5ctextrm{#1}}\x22\
),Pr(\x22\x5c\x5crlap\x22,\x22\x5c\
\x5cmathrlap{\x5c\x5ctext\
rm{#1}}\x22),Pr(\x22\x5c\x5c\
clap\x22,\x22\x5c\x5cmathcla\
p{\x5c\x5ctextrm{#1}}\x22\
),Pr(\x22\x5c\x5cmathstru\
t\x22,\x22\x5c\x5cvphantom{(\
}\x22),Pr(\x22\x5c\x5cunderb\
ar\x22,\x22\x5c\x5cunderline\
{\x5c\x5ctext{#1}}\x22),P\
r(\x22\x5c\x5cnot\x22,'\x5c\x5chtm\
l@mathml{\x5c\x5cmathr\
el{\x5c\x5cmathrlap\x5c\x5c@\
not}}{\x5c\x5cchar\x22338\
}'),Pr(\x22\x5c\x5cneq\x22,\x22\
\x5c\x5chtml@mathml{\x5c\x5c\
mathrel{\x5c\x5cnot=}}\
{\x5c\x5cmathrel{\x5c\x5ccha\
r`\x5cu2260}}\x22),Pr(\
\x22\x5c\x5cne\x22,\x22\x5c\x5cneq\x22),\
Pr(\x22\x5cu2260\x22,\x22\x5c\x5cn\
eq\x22),Pr(\x22\x5c\x5cnotin\
\x22,\x22\x5c\x5chtml@mathml\
{\x5c\x5cmathrel{{\x5c\x5cin\
}\x5c\x5cmathllap{/\x5c\x5cm\
skip1mu}}}{\x5c\x5cmat\
hrel{\x5c\x5cchar`\x5cu22\
09}}\x22),Pr(\x22\x5cu220\
9\x22,\x22\x5c\x5cnotin\x22),Pr\
(\x22\x5cu2258\x22,\x22\x5c\x5chtm\
l@mathml{\x5c\x5cmathr\
el{=\x5c\x5ckern{-1em}\
\x5c\x5craisebox{0.4em\
}{$\x5c\x5cscriptsize\x5c\
\x5cfrown$}}}{\x5c\x5cmat\
hrel{\x5c\x5cchar`\x5cu22\
58}}\x22),Pr(\x22\x5cu225\
9\x22,\x22\x5c\x5chtml@mathm\
l{\x5c\x5cstackrel{\x5c\x5ct\
iny\x5c\x5cwedge}{=}}{\
\x5c\x5cmathrel{\x5c\x5cchar\
`\x5cu2258}}\x22),Pr(\x22\
\x5cu225a\x22,\x22\x5c\x5chtml@\
mathml{\x5c\x5cstackre\
l{\x5c\x5ctiny\x5c\x5cvee}{=\
}}{\x5c\x5cmathrel{\x5c\x5cc\
har`\x5cu225a}}\x22),P\
r(\x22\x5cu225b\x22,\x22\x5c\x5cht\
ml@mathml{\x5c\x5cstac\
krel{\x5c\x5cscriptsiz\
e\x5c\x5cstar}{=}}{\x5c\x5cm\
athrel{\x5c\x5cchar`\x5cu\
225b}}\x22),Pr(\x22\x5cu2\
25d\x22,\x22\x5c\x5chtml@mat\
hml{\x5c\x5cstackrel{\x5c\
\x5ctiny\x5c\x5cmathrm{de\
f}}{=}}{\x5c\x5cmathre\
l{\x5c\x5cchar`\x5cu225d}\
}\x22),Pr(\x22\x5cu225e\x22,\
\x22\x5c\x5chtml@mathml{\x5c\
\x5cstackrel{\x5c\x5ctiny\
\x5c\x5cmathrm{m}}{=}}\
{\x5c\x5cmathrel{\x5c\x5ccha\
r`\x5cu225e}}\x22),Pr(\
\x22\x5cu225f\x22,\x22\x5c\x5chtml\
@mathml{\x5c\x5cstackr\
el{\x5c\x5ctiny?}{=}}{\
\x5c\x5cmathrel{\x5c\x5cchar\
`\x5cu225f}}\x22),Pr(\x22\
\x5cu27c2\x22,\x22\x5c\x5cperp\x22\
),Pr(\x22\x5cu203c\x22,\x22\x5c\
\x5cmathclose{!\x5c\x5cmk\
ern-0.8mu!}\x22),Pr\
(\x22\x5cu220c\x22,\x22\x5c\x5cnot\
ni\x22),Pr(\x22\x5cu231c\x22\
,\x22\x5c\x5culcorner\x22),P\
r(\x22\x5cu231d\x22,\x22\x5c\x5cur\
corner\x22),Pr(\x22\x5cu2\
31e\x22,\x22\x5c\x5cllcorner\
\x22),Pr(\x22\x5cu231f\x22,\x22\
\x5c\x5clrcorner\x22),Pr(\
\x22\x5cxa9\x22,\x22\x5c\x5ccopyri\
ght\x22),Pr(\x22\x5cxae\x22,\
\x22\x5c\x5ctextregistere\
d\x22),Pr(\x22\x5cufe0f\x22,\
\x22\x5c\x5ctextregistere\
d\x22),Pr(\x22\x5c\x5culcorn\
er\x22,'\x5c\x5chtml@math\
ml{\x5c\x5c@ulcorner}{\
\x5c\x5cmathop{\x5c\x5cchar\x22\
231c}}'),Pr(\x22\x5c\x5cu\
rcorner\x22,'\x5c\x5chtml\
@mathml{\x5c\x5c@urcor\
ner}{\x5c\x5cmathop{\x5c\x5c\
char\x22231d}}'),Pr\
(\x22\x5c\x5cllcorner\x22,'\x5c\
\x5chtml@mathml{\x5c\x5c@\
llcorner}{\x5c\x5cmath\
op{\x5c\x5cchar\x22231e}}\
'),Pr(\x22\x5c\x5clrcorne\
r\x22,'\x5c\x5chtml@mathm\
l{\x5c\x5c@lrcorner}{\x5c\
\x5cmathop{\x5c\x5cchar\x222\
31f}}'),Pr(\x22\x5c\x5cvd\
ots\x22,\x22\x5c\x5cmathord{\
\x5c\x5cvarvdots\x5c\x5crule\
{0pt}{15pt}}\x22),P\
r(\x22\x5cu22ee\x22,\x22\x5c\x5cvd\
ots\x22),Pr(\x22\x5c\x5cvarG\
amma\x22,\x22\x5c\x5cmathit{\
\x5c\x5cGamma}\x22),Pr(\x22\x5c\
\x5cvarDelta\x22,\x22\x5c\x5cma\
thit{\x5c\x5cDelta}\x22),\
Pr(\x22\x5c\x5cvarTheta\x22,\
\x22\x5c\x5cmathit{\x5c\x5cThet\
a}\x22),Pr(\x22\x5c\x5cvarLa\
mbda\x22,\x22\x5c\x5cmathit{\
\x5c\x5cLambda}\x22),Pr(\x22\
\x5c\x5cvarXi\x22,\x22\x5c\x5cmath\
it{\x5c\x5cXi}\x22),Pr(\x22\x5c\
\x5cvarPi\x22,\x22\x5c\x5cmathi\
t{\x5c\x5cPi}\x22),Pr(\x22\x5c\x5c\
varSigma\x22,\x22\x5c\x5cmat\
hit{\x5c\x5cSigma}\x22),P\
r(\x22\x5c\x5cvarUpsilon\x22\
,\x22\x5c\x5cmathit{\x5c\x5cUps\
ilon}\x22),Pr(\x22\x5c\x5cva\
rPhi\x22,\x22\x5c\x5cmathit{\
\x5c\x5cPhi}\x22),Pr(\x22\x5c\x5cv\
arPsi\x22,\x22\x5c\x5cmathit\
{\x5c\x5cPsi}\x22),Pr(\x22\x5c\x5c\
varOmega\x22,\x22\x5c\x5cmat\
hit{\x5c\x5cOmega}\x22),P\
r(\x22\x5c\x5csubstack\x22,\x22\
\x5c\x5cbegin{subarray\
}{c}#1\x5c\x5cend{suba\
rray}\x22),Pr(\x22\x5c\x5cco\
lon\x22,\x22\x5c\x5cnobreak\x5c\
\x5cmskip2mu\x5c\x5cmathp\
unct{}\x5c\x5cmathchoi\
ce{\x5c\x5cmkern-3mu}{\
\x5c\x5cmkern-3mu}{}{}\
{:}\x5c\x5cmskip6mu\x5c\x5cr\
elax\x22),Pr(\x22\x5c\x5cbox\
ed\x22,\x22\x5c\x5cfbox{$\x5c\x5cd\
isplaystyle{#1}$\
}\x22),Pr(\x22\x5c\x5ciff\x22,\x22\
\x5c\x5cDOTSB\x5c\x5c;\x5c\x5cLong\
leftrightarrow\x5c\x5c\
;\x22),Pr(\x22\x5c\x5cimplie\
s\x22,\x22\x5c\x5cDOTSB\x5c\x5c;\x5c\x5c\
Longrightarrow\x5c\x5c\
;\x22),Pr(\x22\x5c\x5cimplie\
dby\x22,\x22\x5c\x5cDOTSB\x5c\x5c;\
\x5c\x5cLongleftarrow\x5c\
\x5c;\x22);var Gn={\x22,\x22\
:\x22\x5c\x5cdotsc\x22,\x22\x5c\x5cno\
t\x22:\x22\x5c\x5cdotsb\x22,\x22+\x22\
:\x22\x5c\x5cdotsb\x22,\x22=\x22:\x22\
\x5c\x5cdotsb\x22,\x22<\x22:\x22\x5c\x5c\
dotsb\x22,\x22>\x22:\x22\x5c\x5cdo\
tsb\x22,\x22-\x22:\x22\x5c\x5cdots\
b\x22,\x22*\x22:\x22\x5c\x5cdotsb\x22\
,\x22:\x22:\x22\x5c\x5cdotsb\x22,\x22\
\x5c\x5cDOTSB\x22:\x22\x5c\x5cdots\
b\x22,\x22\x5c\x5ccoprod\x22:\x22\x5c\
\x5cdotsb\x22,\x22\x5c\x5cbigve\
e\x22:\x22\x5c\x5cdotsb\x22,\x22\x5c\x5c\
bigwedge\x22:\x22\x5c\x5cdot\
sb\x22,\x22\x5c\x5cbiguplus\x22\
:\x22\x5c\x5cdotsb\x22,\x22\x5c\x5cbi\
gcap\x22:\x22\x5c\x5cdotsb\x22,\
\x22\x5c\x5cbigcup\x22:\x22\x5c\x5cdo\
tsb\x22,\x22\x5c\x5cprod\x22:\x22\x5c\
\x5cdotsb\x22,\x22\x5c\x5csum\x22:\
\x22\x5c\x5cdotsb\x22,\x22\x5c\x5cbig\
otimes\x22:\x22\x5c\x5cdotsb\
\x22,\x22\x5c\x5cbigoplus\x22:\x22\
\x5c\x5cdotsb\x22,\x22\x5c\x5cbigo\
dot\x22:\x22\x5c\x5cdotsb\x22,\x22\
\x5c\x5cbigsqcup\x22:\x22\x5c\x5cd\
otsb\x22,\x22\x5c\x5cAnd\x22:\x22\x5c\
\x5cdotsb\x22,\x22\x5c\x5clongr\
ightarrow\x22:\x22\x5c\x5cdo\
tsb\x22,\x22\x5c\x5cLongrigh\
tarrow\x22:\x22\x5c\x5cdotsb\
\x22,\x22\x5c\x5clongleftarr\
ow\x22:\x22\x5c\x5cdotsb\x22,\x22\x5c\
\x5cLongleftarrow\x22:\
\x22\x5c\x5cdotsb\x22,\x22\x5c\x5clon\
gleftrightarrow\x22\
:\x22\x5c\x5cdotsb\x22,\x22\x5c\x5cLo\
ngleftrightarrow\
\x22:\x22\x5c\x5cdotsb\x22,\x22\x5c\x5cm\
apsto\x22:\x22\x5c\x5cdotsb\x22\
,\x22\x5c\x5clongmapsto\x22:\
\x22\x5c\x5cdotsb\x22,\x22\x5c\x5choo\
krightarrow\x22:\x22\x5c\x5c\
dotsb\x22,\x22\x5c\x5cdoteq\x22\
:\x22\x5c\x5cdotsb\x22,\x22\x5c\x5cma\
thbin\x22:\x22\x5c\x5cdotsb\x22\
,\x22\x5c\x5cmathrel\x22:\x22\x5c\x5c\
dotsb\x22,\x22\x5c\x5crelbar\
\x22:\x22\x5c\x5cdotsb\x22,\x22\x5c\x5cR\
elbar\x22:\x22\x5c\x5cdotsb\x22\
,\x22\x5c\x5cxrightarrow\x22\
:\x22\x5c\x5cdotsb\x22,\x22\x5c\x5cxl\
eftarrow\x22:\x22\x5c\x5cdot\
sb\x22,\x22\x5c\x5cDOTSI\x22:\x22\x5c\
\x5cdotsi\x22,\x22\x5c\x5cint\x22:\
\x22\x5c\x5cdotsi\x22,\x22\x5c\x5coin\
t\x22:\x22\x5c\x5cdotsi\x22,\x22\x5c\x5c\
iint\x22:\x22\x5c\x5cdotsi\x22,\
\x22\x5c\x5ciiint\x22:\x22\x5c\x5cdot\
si\x22,\x22\x5c\x5ciiiint\x22:\x22\
\x5c\x5cdotsi\x22,\x22\x5c\x5cidot\
sint\x22:\x22\x5c\x5cdotsi\x22,\
\x22\x5c\x5cDOTSX\x22:\x22\x5c\x5cdot\
sx\x22};Pr(\x22\x5c\x5cdots\x22\
,(function(e){va\
r t=\x22\x5c\x5cdotso\x22,r=\
e.expandAfterFut\
ure().text;retur\
n r in Gn?t=Gn[r\
]:(\x22\x5c\x5cnot\x22===r.s\
lice(0,4)||r in \
ae.math&&l.conta\
ins([\x22bin\x22,\x22rel\x22\
],ae.math[r].gro\
up))&&(t=\x22\x5c\x5cdots\
b\x22),t}));var Un=\
{\x22)\x22:!0,\x22]\x22:!0,\x22\
\x5c\x5crbrack\x22:!0,\x22\x5c\x5c\
}\x22:!0,\x22\x5c\x5crbrace\x22\
:!0,\x22\x5c\x5crangle\x22:!\
0,\x22\x5c\x5crceil\x22:!0,\x22\
\x5c\x5crfloor\x22:!0,\x22\x5c\x5c\
rgroup\x22:!0,\x22\x5c\x5crm\
oustache\x22:!0,\x22\x5c\x5c\
right\x22:!0,\x22\x5c\x5cbig\
r\x22:!0,\x22\x5c\x5cbiggr\x22:\
!0,\x22\x5c\x5cBigr\x22:!0,\x22\
\x5c\x5cBiggr\x22:!0,$:!0\
,\x22;\x22:!0,\x22.\x22:!0,\x22\
,\x22:!0};Pr(\x22\x5c\x5cdot\
so\x22,(function(e)\
{return e.future\
().text in Un?\x22\x5c\
\x5cldots\x5c\x5c,\x22:\x22\x5c\x5cld\
ots\x22})),Pr(\x22\x5c\x5cdo\
tsc\x22,(function(e\
){var t=e.future\
().text;return t\
 in Un&&\x22,\x22!==t?\
\x22\x5c\x5cldots\x5c\x5c,\x22:\x22\x5c\x5c\
ldots\x22})),Pr(\x22\x5c\x5c\
cdots\x22,(function\
(e){return e.fut\
ure().text in Un\
?\x22\x5c\x5c@cdots\x5c\x5c,\x22:\x22\
\x5c\x5c@cdots\x22})),Pr(\
\x22\x5c\x5cdotsb\x22,\x22\x5c\x5ccdo\
ts\x22),Pr(\x22\x5c\x5cdotsm\
\x22,\x22\x5c\x5ccdots\x22),Pr(\
\x22\x5c\x5cdotsi\x22,\x22\x5c\x5c!\x5c\x5c\
cdots\x22),Pr(\x22\x5c\x5cdo\
tsx\x22,\x22\x5c\x5cldots\x5c\x5c,\
\x22),Pr(\x22\x5c\x5cDOTSI\x22,\
\x22\x5c\x5crelax\x22),Pr(\x22\x5c\
\x5cDOTSB\x22,\x22\x5c\x5crelax\
\x22),Pr(\x22\x5c\x5cDOTSX\x22,\
\x22\x5c\x5crelax\x22),Pr(\x22\x5c\
\x5ctmspace\x22,\x22\x5c\x5cTex\
tOrMath{\x5c\x5ckern#1\
#3}{\x5c\x5cmskip#1#2}\
\x5c\x5crelax\x22),Pr(\x22\x5c\x5c\
,\x22,\x22\x5c\x5ctmspace+{3\
mu}{.1667em}\x22),P\
r(\x22\x5c\x5cthinspace\x22,\
\x22\x5c\x5c,\x22),Pr(\x22\x5c\x5c>\x22,\
\x22\x5c\x5cmskip{4mu}\x22),\
Pr(\x22\x5c\x5c:\x22,\x22\x5c\x5ctmsp\
ace+{4mu}{.2222e\
m}\x22),Pr(\x22\x5c\x5cmedsp\
ace\x22,\x22\x5c\x5c:\x22),Pr(\x22\
\x5c\x5c;\x22,\x22\x5c\x5ctmspace+\
{5mu}{.2777em}\x22)\
,Pr(\x22\x5c\x5cthickspac\
e\x22,\x22\x5c\x5c;\x22),Pr(\x22\x5c\x5c\
!\x22,\x22\x5c\x5ctmspace-{3\
mu}{.1667em}\x22),P\
r(\x22\x5c\x5cnegthinspac\
e\x22,\x22\x5c\x5c!\x22),Pr(\x22\x5c\x5c\
negmedspace\x22,\x22\x5c\x5c\
tmspace-{4mu}{.2\
222em}\x22),Pr(\x22\x5c\x5cn\
egthickspace\x22,\x22\x5c\
\x5ctmspace-{5mu}{.\
277em}\x22),Pr(\x22\x5c\x5ce\
nspace\x22,\x22\x5c\x5ckern.\
5em \x22),Pr(\x22\x5c\x5cens\
kip\x22,\x22\x5c\x5chskip.5e\
m\x5c\x5crelax\x22),Pr(\x22\x5c\
\x5cquad\x22,\x22\x5c\x5chskip1\
em\x5c\x5crelax\x22),Pr(\x22\
\x5c\x5cqquad\x22,\x22\x5c\x5chski\
p2em\x5c\x5crelax\x22),Pr\
(\x22\x5c\x5ctag\x22,\x22\x5c\x5c@ifs\
tar\x5c\x5ctag@literal\
\x5c\x5ctag@paren\x22),Pr\
(\x22\x5c\x5ctag@paren\x22,\x22\
\x5c\x5ctag@literal{({\
#1})}\x22),Pr(\x22\x5c\x5cta\
g@literal\x22,(func\
tion(e){if(e.mac\
ros.get(\x22\x5c\x5cdf@ta\
g\x22))throw new n(\
\x22Multiple \x5c\x5ctag\x22\
);return\x22\x5c\x5cgdef\x5c\
\x5cdf@tag{\x5c\x5ctext{#\
1}}\x22})),Pr(\x22\x5c\x5cbm\
od\x22,\x22\x5c\x5cmathchoic\
e{\x5c\x5cmskip1mu}{\x5c\x5c\
mskip1mu}{\x5c\x5cmski\
p5mu}{\x5c\x5cmskip5mu\
}\x5c\x5cmathbin{\x5c\x5crm \
mod}\x5c\x5cmathchoice\
{\x5c\x5cmskip1mu}{\x5c\x5cm\
skip1mu}{\x5c\x5cmskip\
5mu}{\x5c\x5cmskip5mu}\
\x22),Pr(\x22\x5c\x5cpod\x22,\x22\x5c\
\x5callowbreak\x5c\x5cmat\
hchoice{\x5c\x5cmkern1\
8mu}{\x5c\x5cmkern8mu}\
{\x5c\x5cmkern8mu}{\x5c\x5cm\
kern8mu}(#1)\x22),P\
r(\x22\x5c\x5cpmod\x22,\x22\x5c\x5cpo\
d{{\x5c\x5crm mod}\x5c\x5cmk\
ern6mu#1}\x22),Pr(\x22\
\x5c\x5cmod\x22,\x22\x5c\x5callowb\
reak\x5c\x5cmathchoice\
{\x5c\x5cmkern18mu}{\x5c\x5c\
mkern12mu}{\x5c\x5cmke\
rn12mu}{\x5c\x5cmkern1\
2mu}{\x5c\x5crm mod}\x5c\x5c\
,\x5c\x5c,#1\x22),Pr(\x22\x5c\x5cn\
ewline\x22,\x22\x5c\x5c\x5c\x5c\x5c\x5cr\
elax\x22),Pr(\x22\x5c\x5cTeX\
\x22,\x22\x5c\x5ctextrm{\x5c\x5cht\
ml@mathml{T\x5c\x5cker\
n-.1667em\x5c\x5craise\
box{-.5ex}{E}\x5c\x5ck\
ern-.125emX}{TeX\
}}\x22);var Yn=F(T[\
\x22Main-Regular\x22][\
\x22T\x22.charCodeAt(0\
)][1]-.7*T[\x22Main\
-Regular\x22][\x22A\x22.c\
harCodeAt(0)][1]\
);Pr(\x22\x5c\x5cLaTeX\x22,\x22\
\x5c\x5ctextrm{\x5c\x5chtml@\
mathml{L\x5c\x5ckern-.\
36em\x5c\x5craisebox{\x22\
+Yn+\x22}{\x5c\x5cscripts\
tyle A}\x5c\x5ckern-.1\
5em\x5c\x5cTeX}{LaTeX}\
}\x22),Pr(\x22\x5c\x5cKaTeX\x22\
,\x22\x5c\x5ctextrm{\x5c\x5chtm\
l@mathml{K\x5c\x5ckern\
-.17em\x5c\x5craisebox\
{\x22+Yn+\x22}{\x5c\x5cscrip\
tstyle A}\x5c\x5ckern-\
.15em\x5c\x5cTeX}{KaTe\
X}}\x22),Pr(\x22\x5c\x5chspa\
ce\x22,\x22\x5c\x5c@ifstar\x5c\x5c\
@hspacer\x5c\x5c@hspac\
e\x22),Pr(\x22\x5c\x5c@hspac\
e\x22,\x22\x5c\x5chskip #1\x5c\x5c\
relax\x22),Pr(\x22\x5c\x5c@h\
spacer\x22,\x22\x5c\x5crule{\
0pt}{0pt}\x5c\x5chskip\
 #1\x5c\x5crelax\x22),Pr(\
\x22\x5c\x5cordinarycolon\
\x22,\x22:\x22),Pr(\x22\x5c\x5cvce\
ntcolon\x22,\x22\x5c\x5cmath\
rel{\x5c\x5cmathop\x5c\x5cor\
dinarycolon}\x22),P\
r(\x22\x5c\x5cdblcolon\x22,'\
\x5c\x5chtml@mathml{\x5c\x5c\
mathrel{\x5c\x5cvcentc\
olon\x5c\x5cmathrel{\x5c\x5c\
mkern-.9mu}\x5c\x5cvce\
ntcolon}}{\x5c\x5cmath\
op{\x5c\x5cchar\x222237}}\
'),Pr(\x22\x5c\x5ccoloneq\
q\x22,'\x5c\x5chtml@mathm\
l{\x5c\x5cmathrel{\x5c\x5cvc\
entcolon\x5c\x5cmathre\
l{\x5c\x5cmkern-1.2mu}\
=}}{\x5c\x5cmathop{\x5c\x5cc\
har\x222254}}'),Pr(\
\x22\x5c\x5cColoneqq\x22,'\x5c\x5c\
html@mathml{\x5c\x5cma\
threl{\x5c\x5cdblcolon\
\x5c\x5cmathrel{\x5c\x5cmker\
n-1.2mu}=}}{\x5c\x5cma\
thop{\x5c\x5cchar\x222237\
\x5c\x5cchar\x223d}}'),Pr\
(\x22\x5c\x5ccoloneq\x22,'\x5c\x5c\
html@mathml{\x5c\x5cma\
threl{\x5c\x5cvcentcol\
on\x5c\x5cmathrel{\x5c\x5cmk\
ern-1.2mu}\x5c\x5cmath\
rel{-}}}{\x5c\x5cmatho\
p{\x5c\x5cchar\x223a\x5c\x5ccha\
r\x222212}}'),Pr(\x22\x5c\
\x5cColoneq\x22,'\x5c\x5chtm\
l@mathml{\x5c\x5cmathr\
el{\x5c\x5cdblcolon\x5c\x5cm\
athrel{\x5c\x5cmkern-1\
.2mu}\x5c\x5cmathrel{-\
}}}{\x5c\x5cmathop{\x5c\x5cc\
har\x222237\x5c\x5cchar\x222\
212}}'),Pr(\x22\x5c\x5ceq\
qcolon\x22,'\x5c\x5chtml@\
mathml{\x5c\x5cmathrel\
{=\x5c\x5cmathrel{\x5c\x5cmk\
ern-1.2mu}\x5c\x5cvcen\
tcolon}}{\x5c\x5cmatho\
p{\x5c\x5cchar\x222255}}'\
),Pr(\x22\x5c\x5cEqqcolon\
\x22,'\x5c\x5chtml@mathml\
{\x5c\x5cmathrel{=\x5c\x5cma\
threl{\x5c\x5cmkern-1.\
2mu}\x5c\x5cdblcolon}}\
{\x5c\x5cmathop{\x5c\x5cchar\
\x223d\x5c\x5cchar\x222237}}\
'),Pr(\x22\x5c\x5ceqcolon\
\x22,'\x5c\x5chtml@mathml\
{\x5c\x5cmathrel{\x5c\x5cmat\
hrel{-}\x5c\x5cmathrel\
{\x5c\x5cmkern-1.2mu}\x5c\
\x5cvcentcolon}}{\x5c\x5c\
mathop{\x5c\x5cchar\x2222\
39}}'),Pr(\x22\x5c\x5cEqc\
olon\x22,'\x5c\x5chtml@ma\
thml{\x5c\x5cmathrel{\x5c\
\x5cmathrel{-}\x5c\x5cmat\
hrel{\x5c\x5cmkern-1.2\
mu}\x5c\x5cdblcolon}}{\
\x5c\x5cmathop{\x5c\x5cchar\x22\
2212\x5c\x5cchar\x222237}\
}'),Pr(\x22\x5c\x5ccolona\
pprox\x22,'\x5c\x5chtml@m\
athml{\x5c\x5cmathrel{\
\x5c\x5cvcentcolon\x5c\x5cma\
threl{\x5c\x5cmkern-1.\
2mu}\x5c\x5capprox}}{\x5c\
\x5cmathop{\x5c\x5cchar\x223\
a\x5c\x5cchar\x222248}}')\
,Pr(\x22\x5c\x5cColonappr\
ox\x22,'\x5c\x5chtml@math\
ml{\x5c\x5cmathrel{\x5c\x5cd\
blcolon\x5c\x5cmathrel\
{\x5c\x5cmkern-1.2mu}\x5c\
\x5capprox}}{\x5c\x5cmath\
op{\x5c\x5cchar\x222237\x5c\x5c\
char\x222248}}'),Pr\
(\x22\x5c\x5ccolonsim\x22,'\x5c\
\x5chtml@mathml{\x5c\x5cm\
athrel{\x5c\x5cvcentco\
lon\x5c\x5cmathrel{\x5c\x5cm\
kern-1.2mu}\x5c\x5csim\
}}{\x5c\x5cmathop{\x5c\x5cch\
ar\x223a\x5c\x5cchar\x22223c\
}}'),Pr(\x22\x5c\x5cColon\
sim\x22,'\x5c\x5chtml@mat\
hml{\x5c\x5cmathrel{\x5c\x5c\
dblcolon\x5c\x5cmathre\
l{\x5c\x5cmkern-1.2mu}\
\x5c\x5csim}}{\x5c\x5cmathop\
{\x5c\x5cchar\x222237\x5c\x5cch\
ar\x22223c}}'),Pr(\x22\
\x5cu2237\x22,\x22\x5c\x5cdblco\
lon\x22),Pr(\x22\x5cu2239\
\x22,\x22\x5c\x5ceqcolon\x22),P\
r(\x22\x5cu2254\x22,\x22\x5c\x5cco\
loneqq\x22),Pr(\x22\x5cu2\
255\x22,\x22\x5c\x5ceqqcolon\
\x22),Pr(\x22\x5cu2a74\x22,\x22\
\x5c\x5cColoneqq\x22),Pr(\
\x22\x5c\x5cratio\x22,\x22\x5c\x5cvce\
ntcolon\x22),Pr(\x22\x5c\x5c\
coloncolon\x22,\x22\x5c\x5cd\
blcolon\x22),Pr(\x22\x5c\x5c\
colonequals\x22,\x22\x5c\x5c\
coloneqq\x22),Pr(\x22\x5c\
\x5ccoloncolonequal\
s\x22,\x22\x5c\x5cColoneqq\x22)\
,Pr(\x22\x5c\x5cequalscol\
on\x22,\x22\x5c\x5ceqqcolon\x22\
),Pr(\x22\x5c\x5cequalsco\
loncolon\x22,\x22\x5c\x5cEqq\
colon\x22),Pr(\x22\x5c\x5cco\
lonminus\x22,\x22\x5c\x5ccol\
oneq\x22),Pr(\x22\x5c\x5ccol\
oncolonminus\x22,\x22\x5c\
\x5cColoneq\x22),Pr(\x22\x5c\
\x5cminuscolon\x22,\x22\x5c\x5c\
eqcolon\x22),Pr(\x22\x5c\x5c\
minuscoloncolon\x22\
,\x22\x5c\x5cEqcolon\x22),Pr\
(\x22\x5c\x5ccoloncolonap\
prox\x22,\x22\x5c\x5cColonap\
prox\x22),Pr(\x22\x5c\x5ccol\
oncolonsim\x22,\x22\x5c\x5cC\
olonsim\x22),Pr(\x22\x5c\x5c\
simcolon\x22,\x22\x5c\x5cmat\
hrel{\x5c\x5csim\x5c\x5cmath\
rel{\x5c\x5cmkern-1.2m\
u}\x5c\x5cvcentcolon}\x22\
),Pr(\x22\x5c\x5csimcolon\
colon\x22,\x22\x5c\x5cmathre\
l{\x5c\x5csim\x5c\x5cmathrel\
{\x5c\x5cmkern-1.2mu}\x5c\
\x5cdblcolon}\x22),Pr(\
\x22\x5c\x5capproxcolon\x22,\
\x22\x5c\x5cmathrel{\x5c\x5capp\
rox\x5c\x5cmathrel{\x5c\x5cm\
kern-1.2mu}\x5c\x5cvce\
ntcolon}\x22),Pr(\x22\x5c\
\x5capproxcoloncolo\
n\x22,\x22\x5c\x5cmathrel{\x5c\x5c\
approx\x5c\x5cmathrel{\
\x5c\x5cmkern-1.2mu}\x5c\x5c\
dblcolon}\x22),Pr(\x22\
\x5c\x5cnotni\x22,\x22\x5c\x5chtml\
@mathml{\x5c\x5cnot\x5c\x5cn\
i}{\x5c\x5cmathrel{\x5c\x5cc\
har`\x5cu220c}}\x22),P\
r(\x22\x5c\x5climsup\x22,\x22\x5c\x5c\
DOTSB\x5c\x5coperatorn\
ame*{lim\x5c\x5c,sup}\x22\
),Pr(\x22\x5c\x5climinf\x22,\
\x22\x5c\x5cDOTSB\x5c\x5coperat\
orname*{lim\x5c\x5c,in\
f}\x22),Pr(\x22\x5c\x5cinjli\
m\x22,\x22\x5c\x5cDOTSB\x5c\x5cope\
ratorname*{inj\x5c\x5c\
,lim}\x22),Pr(\x22\x5c\x5cpr\
ojlim\x22,\x22\x5c\x5cDOTSB\x5c\
\x5coperatorname*{p\
roj\x5c\x5c,lim}\x22),Pr(\
\x22\x5c\x5cvarlimsup\x22,\x22\x5c\
\x5cDOTSB\x5c\x5coperator\
name*{\x5c\x5coverline\
{lim}}\x22),Pr(\x22\x5c\x5cv\
arliminf\x22,\x22\x5c\x5cDOT\
SB\x5c\x5coperatorname\
*{\x5c\x5cunderline{li\
m}}\x22),Pr(\x22\x5c\x5cvari\
njlim\x22,\x22\x5c\x5cDOTSB\x5c\
\x5coperatorname*{\x5c\
\x5cunderrightarrow\
{lim}}\x22),Pr(\x22\x5c\x5cv\
arprojlim\x22,\x22\x5c\x5cDO\
TSB\x5c\x5coperatornam\
e*{\x5c\x5cunderleftar\
row{lim}}\x22),Pr(\x22\
\x5c\x5cgvertneqq\x22,\x22\x5c\x5c\
html@mathml{\x5c\x5c@g\
vertneqq}{\x5cu2269\
}\x22),Pr(\x22\x5c\x5clvertn\
eqq\x22,\x22\x5c\x5chtml@mat\
hml{\x5c\x5c@lvertneqq\
}{\x5cu2268}\x22),Pr(\x22\
\x5c\x5cngeqq\x22,\x22\x5c\x5chtml\
@mathml{\x5c\x5c@ngeqq\
}{\x5cu2271}\x22),Pr(\x22\
\x5c\x5cngeqslant\x22,\x22\x5c\x5c\
html@mathml{\x5c\x5c@n\
geqslant}{\x5cu2271\
}\x22),Pr(\x22\x5c\x5cnleqq\x22\
,\x22\x5c\x5chtml@mathml{\
\x5c\x5c@nleqq}{\x5cu2270\
}\x22),Pr(\x22\x5c\x5cnleqsl\
ant\x22,\x22\x5c\x5chtml@mat\
hml{\x5c\x5c@nleqslant\
}{\x5cu2270}\x22),Pr(\x22\
\x5c\x5cnshortmid\x22,\x22\x5c\x5c\
html@mathml{\x5c\x5c@n\
shortmid}{\x5cu2224\
}\x22),Pr(\x22\x5c\x5cnshort\
parallel\x22,\x22\x5c\x5chtm\
l@mathml{\x5c\x5c@nsho\
rtparallel}{\x5cu22\
26}\x22),Pr(\x22\x5c\x5cnsub\
seteqq\x22,\x22\x5c\x5chtml@\
mathml{\x5c\x5c@nsubse\
teqq}{\x5cu2288}\x22),\
Pr(\x22\x5c\x5cnsupseteqq\
\x22,\x22\x5c\x5chtml@mathml\
{\x5c\x5c@nsupseteqq}{\
\x5cu2289}\x22),Pr(\x22\x5c\x5c\
varsubsetneq\x22,\x22\x5c\
\x5chtml@mathml{\x5c\x5c@\
varsubsetneq}{\x5cu\
228a}\x22),Pr(\x22\x5c\x5cva\
rsubsetneqq\x22,\x22\x5c\x5c\
html@mathml{\x5c\x5c@v\
arsubsetneqq}{\x5cu\
2acb}\x22),Pr(\x22\x5c\x5cva\
rsupsetneq\x22,\x22\x5c\x5ch\
tml@mathml{\x5c\x5c@va\
rsupsetneq}{\x5cu22\
8b}\x22),Pr(\x22\x5c\x5cvars\
upsetneqq\x22,\x22\x5c\x5cht\
ml@mathml{\x5c\x5c@var\
supsetneqq}{\x5cu2a\
cc}\x22),Pr(\x22\x5c\x5cimat\
h\x22,\x22\x5c\x5chtml@mathm\
l{\x5c\x5c@imath}{\x5cu01\
31}\x22),Pr(\x22\x5c\x5cjmat\
h\x22,\x22\x5c\x5chtml@mathm\
l{\x5c\x5c@jmath}{\x5cu02\
37}\x22),Pr(\x22\x5c\x5cllbr\
acket\x22,\x22\x5c\x5chtml@m\
athml{\x5c\x5cmathopen\
{[\x5c\x5cmkern-3.2mu[\
}}{\x5c\x5cmathopen{\x5c\x5c\
char`\x5cu27e6}}\x22),\
Pr(\x22\x5c\x5crrbracket\x22\
,\x22\x5c\x5chtml@mathml{\
\x5c\x5cmathclose{]\x5c\x5cm\
kern-3.2mu]}}{\x5c\x5c\
mathclose{\x5c\x5cchar\
`\x5cu27e7}}\x22),Pr(\x22\
\x5cu27e6\x22,\x22\x5c\x5cllbra\
cket\x22),Pr(\x22\x5cu27e\
7\x22,\x22\x5c\x5crrbracket\x22\
),Pr(\x22\x5c\x5clBrace\x22,\
\x22\x5c\x5chtml@mathml{\x5c\
\x5cmathopen{\x5c\x5c{\x5c\x5cm\
kern-3.2mu[}}{\x5c\x5c\
mathopen{\x5c\x5cchar`\
\x5cu2983}}\x22),Pr(\x22\x5c\
\x5crBrace\x22,\x22\x5c\x5chtml\
@mathml{\x5c\x5cmathcl\
ose{]\x5c\x5cmkern-3.2\
mu\x5c\x5c}}}{\x5c\x5cmathcl\
ose{\x5c\x5cchar`\x5cu298\
4}}\x22),Pr(\x22\x5cu2983\
\x22,\x22\x5c\x5clBrace\x22),Pr\
(\x22\x5cu2984\x22,\x22\x5c\x5crBr\
ace\x22),Pr(\x22\x5c\x5cminu\
so\x22,\x22\x5c\x5cmathbin{\x5c\
\x5chtml@mathml{{\x5c\x5c\
mathrlap{\x5c\x5cmathc\
hoice{\x5c\x5ckern{0.1\
45em}}{\x5c\x5ckern{0.\
145em}}{\x5c\x5ckern{0\
.1015em}}{\x5c\x5ckern\
{0.0725em}}\x5c\x5ccir\
c}{-}}}{\x5c\x5cchar`\x5c\
u29b5}}\x22),Pr(\x22\x5cu\
29b5\x22,\x22\x5c\x5cminuso\x22\
),Pr(\x22\x5c\x5cdarr\x22,\x22\x5c\
\x5cdownarrow\x22),Pr(\
\x22\x5c\x5cdArr\x22,\x22\x5c\x5cDown\
arrow\x22),Pr(\x22\x5c\x5cDa\
rr\x22,\x22\x5c\x5cDownarrow\
\x22),Pr(\x22\x5c\x5clang\x22,\x22\
\x5c\x5clangle\x22),Pr(\x22\x5c\
\x5crang\x22,\x22\x5c\x5crangle\
\x22),Pr(\x22\x5c\x5cuarr\x22,\x22\
\x5c\x5cuparrow\x22),Pr(\x22\
\x5c\x5cuArr\x22,\x22\x5c\x5cUparr\
ow\x22),Pr(\x22\x5c\x5cUarr\x22\
,\x22\x5c\x5cUparrow\x22),Pr\
(\x22\x5c\x5cN\x22,\x22\x5c\x5cmathbb\
{N}\x22),Pr(\x22\x5c\x5cR\x22,\x22\
\x5c\x5cmathbb{R}\x22),Pr\
(\x22\x5c\x5cZ\x22,\x22\x5c\x5cmathbb\
{Z}\x22),Pr(\x22\x5c\x5calef\
\x22,\x22\x5c\x5caleph\x22),Pr(\
\x22\x5c\x5calefsym\x22,\x22\x5c\x5ca\
leph\x22),Pr(\x22\x5c\x5cAlp\
ha\x22,\x22\x5c\x5cmathrm{A}\
\x22),Pr(\x22\x5c\x5cBeta\x22,\x22\
\x5c\x5cmathrm{B}\x22),Pr\
(\x22\x5c\x5cbull\x22,\x22\x5c\x5cbul\
let\x22),Pr(\x22\x5c\x5cChi\x22\
,\x22\x5c\x5cmathrm{X}\x22),\
Pr(\x22\x5c\x5cclubs\x22,\x22\x5c\x5c\
clubsuit\x22),Pr(\x22\x5c\
\x5ccnums\x22,\x22\x5c\x5cmathb\
b{C}\x22),Pr(\x22\x5c\x5cCom\
plex\x22,\x22\x5c\x5cmathbb{\
C}\x22),Pr(\x22\x5c\x5cDagge\
r\x22,\x22\x5c\x5cddagger\x22),\
Pr(\x22\x5c\x5cdiamonds\x22,\
\x22\x5c\x5cdiamondsuit\x22)\
,Pr(\x22\x5c\x5cempty\x22,\x22\x5c\
\x5cemptyset\x22),Pr(\x22\
\x5c\x5cEpsilon\x22,\x22\x5c\x5cma\
thrm{E}\x22),Pr(\x22\x5c\x5c\
Eta\x22,\x22\x5c\x5cmathrm{H\
}\x22),Pr(\x22\x5c\x5cexist\x22\
,\x22\x5c\x5cexists\x22),Pr(\
\x22\x5c\x5charr\x22,\x22\x5c\x5cleft\
rightarrow\x22),Pr(\
\x22\x5c\x5chArr\x22,\x22\x5c\x5cLeft\
rightarrow\x22),Pr(\
\x22\x5c\x5cHarr\x22,\x22\x5c\x5cLeft\
rightarrow\x22),Pr(\
\x22\x5c\x5chearts\x22,\x22\x5c\x5che\
artsuit\x22),Pr(\x22\x5c\x5c\
image\x22,\x22\x5c\x5cIm\x22),P\
r(\x22\x5c\x5cinfin\x22,\x22\x5c\x5ci\
nfty\x22),Pr(\x22\x5c\x5cIot\
a\x22,\x22\x5c\x5cmathrm{I}\x22\
),Pr(\x22\x5c\x5cisin\x22,\x22\x5c\
\x5cin\x22),Pr(\x22\x5c\x5cKapp\
a\x22,\x22\x5c\x5cmathrm{K}\x22\
),Pr(\x22\x5c\x5clarr\x22,\x22\x5c\
\x5cleftarrow\x22),Pr(\
\x22\x5c\x5clArr\x22,\x22\x5c\x5cLeft\
arrow\x22),Pr(\x22\x5c\x5cLa\
rr\x22,\x22\x5c\x5cLeftarrow\
\x22),Pr(\x22\x5c\x5clrarr\x22,\
\x22\x5c\x5cleftrightarro\
w\x22),Pr(\x22\x5c\x5clrArr\x22\
,\x22\x5c\x5cLeftrightarr\
ow\x22),Pr(\x22\x5c\x5cLrarr\
\x22,\x22\x5c\x5cLeftrightar\
row\x22),Pr(\x22\x5c\x5cMu\x22,\
\x22\x5c\x5cmathrm{M}\x22),P\
r(\x22\x5c\x5cnatnums\x22,\x22\x5c\
\x5cmathbb{N}\x22),Pr(\
\x22\x5c\x5cNu\x22,\x22\x5c\x5cmathrm\
{N}\x22),Pr(\x22\x5c\x5cOmic\
ron\x22,\x22\x5c\x5cmathrm{O\
}\x22),Pr(\x22\x5c\x5cplusmn\
\x22,\x22\x5c\x5cpm\x22),Pr(\x22\x5c\x5c\
rarr\x22,\x22\x5c\x5crightar\
row\x22),Pr(\x22\x5c\x5crArr\
\x22,\x22\x5c\x5cRightarrow\x22\
),Pr(\x22\x5c\x5cRarr\x22,\x22\x5c\
\x5cRightarrow\x22),Pr\
(\x22\x5c\x5creal\x22,\x22\x5c\x5cRe\x22\
),Pr(\x22\x5c\x5creals\x22,\x22\
\x5c\x5cmathbb{R}\x22),Pr\
(\x22\x5c\x5cReals\x22,\x22\x5c\x5cma\
thbb{R}\x22),Pr(\x22\x5c\x5c\
Rho\x22,\x22\x5c\x5cmathrm{P\
}\x22),Pr(\x22\x5c\x5csdot\x22,\
\x22\x5c\x5ccdot\x22),Pr(\x22\x5c\x5c\
sect\x22,\x22\x5c\x5cS\x22),Pr(\
\x22\x5c\x5cspades\x22,\x22\x5c\x5csp\
adesuit\x22),Pr(\x22\x5c\x5c\
sub\x22,\x22\x5c\x5csubset\x22)\
,Pr(\x22\x5c\x5csube\x22,\x22\x5c\x5c\
subseteq\x22),Pr(\x22\x5c\
\x5csupe\x22,\x22\x5c\x5csupset\
eq\x22),Pr(\x22\x5c\x5cTau\x22,\
\x22\x5c\x5cmathrm{T}\x22),P\
r(\x22\x5c\x5cthetasym\x22,\x22\
\x5c\x5cvartheta\x22),Pr(\
\x22\x5c\x5cweierp\x22,\x22\x5c\x5cwp\
\x22),Pr(\x22\x5c\x5cZeta\x22,\x22\
\x5c\x5cmathrm{Z}\x22),Pr\
(\x22\x5c\x5cargmin\x22,\x22\x5c\x5cD\
OTSB\x5c\x5coperatorna\
me*{arg\x5c\x5c,min}\x22)\
,Pr(\x22\x5c\x5cargmax\x22,\x22\
\x5c\x5cDOTSB\x5c\x5coperato\
rname*{arg\x5c\x5c,max\
}\x22),Pr(\x22\x5c\x5cplim\x22,\
\x22\x5c\x5cDOTSB\x5c\x5cmathop\
{\x5c\x5coperatorname{\
plim}}\x5c\x5climits\x22)\
,Pr(\x22\x5c\x5cbra\x22,\x22\x5c\x5cm\
athinner{\x5c\x5clangl\
e{#1}|}\x22),Pr(\x22\x5c\x5c\
ket\x22,\x22\x5c\x5cmathinne\
r{|{#1}\x5c\x5crangle}\
\x22),Pr(\x22\x5c\x5cbraket\x22\
,\x22\x5c\x5cmathinner{\x5c\x5c\
langle{#1}\x5c\x5crang\
le}\x22),Pr(\x22\x5c\x5cBra\x22\
,\x22\x5c\x5cleft\x5c\x5clangle\
#1\x5c\x5cright|\x22),Pr(\
\x22\x5c\x5cKet\x22,\x22\x5c\x5cleft|\
#1\x5c\x5cright\x5c\x5crangl\
e\x22);var Xn=funct\
ion(e){return fu\
nction(t){var r=\
t.consumeArg().t\
okens,n=t.consum\
eArg().tokens,a=\
t.consumeArg().t\
okens,i=t.consum\
eArg().tokens,o=\
t.macros.get(\x22|\x22\
),s=t.macros.get\
(\x22\x5c\x5c|\x22);t.macros\
.beginGroup();va\
r l=function(t){\
return function(\
r){e&&(r.macros.\
set(\x22|\x22,o),a.len\
gth&&r.macros.se\
t(\x22\x5c\x5c|\x22,s));var \
i=t;!t&&a.length\
&&(\x22|\x22===r.futur\
e().text&&(r.pop\
Token(),i=!0));r\
eturn{tokens:i?a\
:n,numArgs:0}}};\
t.macros.set(\x22|\x22\
,l(!1)),a.length\
&&t.macros.set(\x22\
\x5c\x5c|\x22,l(!0));var \
h=t.consumeArg()\
.tokens,c=t.expa\
ndTokens([].conc\
at(i,h,r));retur\
n t.macros.endGr\
oup(),{tokens:c.\
reverse(),numArg\
s:0}}};Pr(\x22\x5c\x5cbra\
@ket\x22,Xn(!1)),Pr\
(\x22\x5c\x5cbra@set\x22,Xn(\
!0)),Pr(\x22\x5c\x5cBrake\
t\x22,\x22\x5c\x5cbra@ket{\x5c\x5c\
left\x5c\x5clangle}{\x5c\x5c\
,\x5c\x5cmiddle\x5c\x5cvert\x5c\
\x5c,}{\x5c\x5c,\x5c\x5cmiddle\x5c\
\x5cvert\x5c\x5c,}{\x5c\x5crigh\
t\x5c\x5crangle}\x22),Pr(\
\x22\x5c\x5cSet\x22,\x22\x5c\x5cbra@s\
et{\x5c\x5cleft\x5c\x5c{\x5c\x5c:}\
{\x5c\x5c;\x5c\x5cmiddle\x5c\x5cve\
rt\x5c\x5c;}{\x5c\x5c;\x5c\x5cmidd\
le\x5c\x5cVert\x5c\x5c;}{\x5c\x5c:\
\x5c\x5cright\x5c\x5c}}\x22),Pr\
(\x22\x5c\x5cset\x22,\x22\x5c\x5cbra@\
set{\x5c\x5c{\x5c\x5c,}{\x5c\x5cmi\
d}{}{\x5c\x5c,\x5c\x5c}}\x22),P\
r(\x22\x5c\x5cangln\x22,\x22{\x5c\x5c\
angl n}\x22),Pr(\x22\x5c\x5c\
blue\x22,\x22\x5c\x5ctextcol\
or{##6495ed}{#1}\
\x22),Pr(\x22\x5c\x5corange\x22\
,\x22\x5c\x5ctextcolor{##\
ffa500}{#1}\x22),Pr\
(\x22\x5c\x5cpink\x22,\x22\x5c\x5ctex\
tcolor{##ff00af}\
{#1}\x22),Pr(\x22\x5c\x5cred\
\x22,\x22\x5c\x5ctextcolor{#\
#df0030}{#1}\x22),P\
r(\x22\x5c\x5cgreen\x22,\x22\x5c\x5ct\
extcolor{##28ae7\
b}{#1}\x22),Pr(\x22\x5c\x5cg\
ray\x22,\x22\x5c\x5ctextcolo\
r{gray}{#1}\x22),Pr\
(\x22\x5c\x5cpurple\x22,\x22\x5c\x5ct\
extcolor{##9d38b\
d}{#1}\x22),Pr(\x22\x5c\x5cb\
lueA\x22,\x22\x5c\x5ctextcol\
or{##ccfaff}{#1}\
\x22),Pr(\x22\x5c\x5cblueB\x22,\
\x22\x5c\x5ctextcolor{##8\
0f6ff}{#1}\x22),Pr(\
\x22\x5c\x5cblueC\x22,\x22\x5c\x5ctex\
tcolor{##63d9ea}\
{#1}\x22),Pr(\x22\x5c\x5cblu\
eD\x22,\x22\x5c\x5ctextcolor\
{##11accd}{#1}\x22)\
,Pr(\x22\x5c\x5cblueE\x22,\x22\x5c\
\x5ctextcolor{##0c7\
f99}{#1}\x22),Pr(\x22\x5c\
\x5ctealA\x22,\x22\x5c\x5ctextc\
olor{##94fff5}{#\
1}\x22),Pr(\x22\x5c\x5ctealB\
\x22,\x22\x5c\x5ctextcolor{#\
#26edd5}{#1}\x22),P\
r(\x22\x5c\x5ctealC\x22,\x22\x5c\x5ct\
extcolor{##01d1c\
1}{#1}\x22),Pr(\x22\x5c\x5ct\
ealD\x22,\x22\x5c\x5ctextcol\
or{##01a995}{#1}\
\x22),Pr(\x22\x5c\x5ctealE\x22,\
\x22\x5c\x5ctextcolor{##2\
08170}{#1}\x22),Pr(\
\x22\x5c\x5cgreenA\x22,\x22\x5c\x5cte\
xtcolor{##b6ffb0\
}{#1}\x22),Pr(\x22\x5c\x5cgr\
eenB\x22,\x22\x5c\x5ctextcol\
or{##8af281}{#1}\
\x22),Pr(\x22\x5c\x5cgreenC\x22\
,\x22\x5c\x5ctextcolor{##\
74cf70}{#1}\x22),Pr\
(\x22\x5c\x5cgreenD\x22,\x22\x5c\x5ct\
extcolor{##1fab5\
4}{#1}\x22),Pr(\x22\x5c\x5cg\
reenE\x22,\x22\x5c\x5ctextco\
lor{##0d923f}{#1\
}\x22),Pr(\x22\x5c\x5cgoldA\x22\
,\x22\x5c\x5ctextcolor{##\
ffd0a9}{#1}\x22),Pr\
(\x22\x5c\x5cgoldB\x22,\x22\x5c\x5cte\
xtcolor{##ffbb71\
}{#1}\x22),Pr(\x22\x5c\x5cgo\
ldC\x22,\x22\x5c\x5ctextcolo\
r{##ff9c39}{#1}\x22\
),Pr(\x22\x5c\x5cgoldD\x22,\x22\
\x5c\x5ctextcolor{##e0\
7d10}{#1}\x22),Pr(\x22\
\x5c\x5cgoldE\x22,\x22\x5c\x5ctext\
color{##a75a05}{\
#1}\x22),Pr(\x22\x5c\x5credA\
\x22,\x22\x5c\x5ctextcolor{#\
#fca9a9}{#1}\x22),P\
r(\x22\x5c\x5credB\x22,\x22\x5c\x5cte\
xtcolor{##ff8482\
}{#1}\x22),Pr(\x22\x5c\x5cre\
dC\x22,\x22\x5c\x5ctextcolor\
{##f9685d}{#1}\x22)\
,Pr(\x22\x5c\x5credD\x22,\x22\x5c\x5c\
textcolor{##e84d\
39}{#1}\x22),Pr(\x22\x5c\x5c\
redE\x22,\x22\x5c\x5ctextcol\
or{##bc2612}{#1}\
\x22),Pr(\x22\x5c\x5cmaroonA\
\x22,\x22\x5c\x5ctextcolor{#\
#ffbde0}{#1}\x22),P\
r(\x22\x5c\x5cmaroonB\x22,\x22\x5c\
\x5ctextcolor{##ff9\
2c6}{#1}\x22),Pr(\x22\x5c\
\x5cmaroonC\x22,\x22\x5c\x5ctex\
tcolor{##ed5fa6}\
{#1}\x22),Pr(\x22\x5c\x5cmar\
oonD\x22,\x22\x5c\x5ctextcol\
or{##ca337c}{#1}\
\x22),Pr(\x22\x5c\x5cmaroonE\
\x22,\x22\x5c\x5ctextcolor{#\
#9e034e}{#1}\x22),P\
r(\x22\x5c\x5cpurpleA\x22,\x22\x5c\
\x5ctextcolor{##ddd\
7ff}{#1}\x22),Pr(\x22\x5c\
\x5cpurpleB\x22,\x22\x5c\x5ctex\
tcolor{##c6b9fc}\
{#1}\x22),Pr(\x22\x5c\x5cpur\
pleC\x22,\x22\x5c\x5ctextcol\
or{##aa87ff}{#1}\
\x22),Pr(\x22\x5c\x5cpurpleD\
\x22,\x22\x5c\x5ctextcolor{#\
#7854ab}{#1}\x22),P\
r(\x22\x5c\x5cpurpleE\x22,\x22\x5c\
\x5ctextcolor{##543\
b78}{#1}\x22),Pr(\x22\x5c\
\x5cmintA\x22,\x22\x5c\x5ctextc\
olor{##f5f9e8}{#\
1}\x22),Pr(\x22\x5c\x5cmintB\
\x22,\x22\x5c\x5ctextcolor{#\
#edf2df}{#1}\x22),P\
r(\x22\x5c\x5cmintC\x22,\x22\x5c\x5ct\
extcolor{##e0e5c\
c}{#1}\x22),Pr(\x22\x5c\x5cg\
rayA\x22,\x22\x5c\x5ctextcol\
or{##f6f7f7}{#1}\
\x22),Pr(\x22\x5c\x5cgrayB\x22,\
\x22\x5c\x5ctextcolor{##f\
0f1f2}{#1}\x22),Pr(\
\x22\x5c\x5cgrayC\x22,\x22\x5c\x5ctex\
tcolor{##e3e5e6}\
{#1}\x22),Pr(\x22\x5c\x5cgra\
yD\x22,\x22\x5c\x5ctextcolor\
{##d6d8da}{#1}\x22)\
,Pr(\x22\x5c\x5cgrayE\x22,\x22\x5c\
\x5ctextcolor{##bab\
ec2}{#1}\x22),Pr(\x22\x5c\
\x5cgrayF\x22,\x22\x5c\x5ctextc\
olor{##888d93}{#\
1}\x22),Pr(\x22\x5c\x5cgrayG\
\x22,\x22\x5c\x5ctextcolor{#\
#626569}{#1}\x22),P\
r(\x22\x5c\x5cgrayH\x22,\x22\x5c\x5ct\
extcolor{##3b3e4\
0}{#1}\x22),Pr(\x22\x5c\x5cg\
rayI\x22,\x22\x5c\x5ctextcol\
or{##21242c}{#1}\
\x22),Pr(\x22\x5c\x5ckaBlue\x22\
,\x22\x5c\x5ctextcolor{##\
314453}{#1}\x22),Pr\
(\x22\x5c\x5ckaGreen\x22,\x22\x5c\x5c\
textcolor{##71B3\
07}{#1}\x22);var Wn\
={\x22^\x22:!0,_:!0,\x22\x5c\
\x5climits\x22:!0,\x22\x5c\x5cn\
olimits\x22:!0},_n=\
function(){funct\
ion e(e,t,r){thi\
s.settings=void \
0,this.expansion\
Count=void 0,thi\
s.lexer=void 0,t\
his.macros=void \
0,this.stack=voi\
d 0,this.mode=vo\
id 0,this.settin\
gs=t,this.expans\
ionCount=0,this.\
feed(e),this.mac\
ros=new Dn(Vn,t.\
macros),this.mod\
e=r,this.stack=[\
]}var t=e.protot\
ype;return t.fee\
d=function(e){th\
is.lexer=new Ln(\
e,this.settings)\
},t.switchMode=f\
unction(e){this.\
mode=e},t.beginG\
roup=function(){\
this.macros.begi\
nGroup()},t.endG\
roup=function(){\
this.macros.endG\
roup()},t.endGro\
ups=function(){t\
his.macros.endGr\
oups()},t.future\
=function(){retu\
rn 0===this.stac\
k.length&&this.p\
ushToken(this.le\
xer.lex()),this.\
stack[this.stack\
.length-1]},t.po\
pToken=function(\
){return this.fu\
ture(),this.stac\
k.pop()},t.pushT\
oken=function(e)\
{this.stack.push\
(e)},t.pushToken\
s=function(e){va\
r t;(t=this.stac\
k).push.apply(t,\
e)},t.scanArgume\
nt=function(e){v\
ar t,r,n;if(e){i\
f(this.consumeSp\
aces(),\x22[\x22!==thi\
s.future().text)\
return null;t=th\
is.popToken();va\
r a=this.consume\
Arg([\x22]\x22]);n=a.t\
okens,r=a.end}el\
se{var i=this.co\
nsumeArg();n=i.t\
okens,t=i.start,\
r=i.end}return t\
his.pushToken(ne\
w Gr(\x22EOF\x22,r.loc\
)),this.pushToke\
ns(n),t.range(r,\
\x22\x22)},t.consumeSp\
aces=function(){\
for(;;){if(\x22 \x22!=\
=this.future().t\
ext)break;this.s\
tack.pop()}},t.c\
onsumeArg=functi\
on(e){var t=[],r\
=e&&e.length>0;r\
||this.consumeSp\
aces();var a,i=t\
his.future(),o=0\
,s=0;do{if(a=thi\
s.popToken(),t.p\
ush(a),\x22{\x22===a.t\
ext)++o;else if(\
\x22}\x22===a.text){if\
(-1===--o)throw \
new n(\x22Extra }\x22,\
a)}else if(\x22EOF\x22\
===a.text)throw \
new n(\x22Unexpecte\
d end of input i\
n a macro argume\
nt, expected '\x22+\
(e&&r?e[s]:\x22}\x22)+\
\x22'\x22,a);if(e&&r)i\
f((0===o||1===o&\
&\x22{\x22===e[s])&&a.\
text===e[s]){if(\
++s===e.length){\
t.splice(-s,s);b\
reak}}else s=0}w\
hile(0!==o||r);r\
eturn\x22{\x22===i.tex\
t&&\x22}\x22===t[t.len\
gth-1].text&&(t.\
pop(),t.shift())\
,t.reverse(),{to\
kens:t,start:i,e\
nd:a}},t.consume\
Args=function(e,\
t){if(t){if(t.le\
ngth!==e+1)throw\
 new n(\x22The leng\
th of delimiters\
 doesn't match t\
he number of arg\
s!\x22);for(var r=t\
[0],a=0;a<r.leng\
th;a++){var i=th\
is.popToken();if\
(r[a]!==i.text)t\
hrow new n(\x22Use \
of the macro doe\
sn't match its d\
efinition\x22,i)}}f\
or(var o=[],s=0;\
s<e;s++)o.push(t\
his.consumeArg(t\
&&t[s+1]).tokens\
);return o},t.ex\
pandOnce=functio\
n(e){var t=this.\
popToken(),r=t.t\
ext,a=t.noexpand\
?null:this._getE\
xpansion(r);if(n\
ull==a||e&&a.une\
xpandable){if(e&\
&null==a&&\x22\x5c\x5c\x22==\
=r[0]&&!this.isD\
efined(r))throw \
new n(\x22Undefined\
 control sequenc\
e: \x22+r);return t\
his.pushToken(t)\
,!1}if(this.expa\
nsionCount++,thi\
s.expansionCount\
>this.settings.m\
axExpand)throw n\
ew n(\x22Too many e\
xpansions: infin\
ite loop or need\
 to increase max\
Expand setting\x22)\
;var i=a.tokens,\
o=this.consumeAr\
gs(a.numArgs,a.d\
elimiters);if(a.\
numArgs)for(var \
s=(i=i.slice()).\
length-1;s>=0;--\
s){var l=i[s];if\
(\x22#\x22===l.text){i\
f(0===s)throw ne\
w n(\x22Incomplete \
placeholder at e\
nd of macro body\
\x22,l);if(\x22#\x22===(l\
=i[--s]).text)i.\
splice(s+1,1);el\
se{if(!/^[1-9]$/\
.test(l.text))th\
row new n(\x22Not a\
 valid argument \
number\x22,l);var h\
;(h=i).splice.ap\
ply(h,[s,2].conc\
at(o[+l.text-1])\
)}}}return this.\
pushTokens(i),i.\
length},t.expand\
AfterFuture=func\
tion(){return th\
is.expandOnce(),\
this.future()},t\
.expandNextToken\
=function(){for(\
;;)if(!1===this.\
expandOnce()){va\
r e=this.stack.p\
op();return e.tr\
eatAsRelax&&(e.t\
ext=\x22\x5c\x5crelax\x22),e\
}throw new Error\
},t.expandMacro=\
function(e){retu\
rn this.macros.h\
as(e)?this.expan\
dTokens([new Gr(\
e)]):void 0},t.e\
xpandTokens=func\
tion(e){var t=[]\
,r=this.stack.le\
ngth;for(this.pu\
shTokens(e);this\
.stack.length>r;\
)if(!1===this.ex\
pandOnce(!0)){va\
r n=this.stack.p\
op();n.treatAsRe\
lax&&(n.noexpand\
=!1,n.treatAsRel\
ax=!1),t.push(n)\
}return t},t.exp\
andMacroAsText=f\
unction(e){var t\
=this.expandMacr\
o(e);return t?t.\
map((function(e)\
{return e.text})\
).join(\x22\x22):t},t.\
_getExpansion=fu\
nction(e){var t=\
this.macros.get(\
e);if(null==t)re\
turn t;if(1===e.\
length){var r=th\
is.lexer.catcode\
s[e];if(null!=r&\
&13!==r)return}v\
ar n=\x22function\x22=\
=typeof t?t(this\
):t;if(\x22string\x22=\
=typeof n){var a\
=0;if(-1!==n.ind\
exOf(\x22#\x22))for(va\
r i=n.replace(/#\
#/g,\x22\x22);-1!==i.i\
ndexOf(\x22#\x22+(a+1)\
);)++a;for(var o\
=new Ln(n,this.s\
ettings),s=[],l=\
o.lex();\x22EOF\x22!==\
l.text;)s.push(l\
),l=o.lex();retu\
rn s.reverse(),{\
tokens:s,numArgs\
:a}}return n},t.\
isDefined=functi\
on(e){return thi\
s.macros.has(e)|\
|qn.hasOwnProper\
ty(e)||ae.math.h\
asOwnProperty(e)\
||ae.text.hasOwn\
Property(e)||Wn.\
hasOwnProperty(e\
)},t.isExpandabl\
e=function(e){va\
r t=this.macros.\
get(e);return nu\
ll!=t?\x22string\x22==\
typeof t||\x22funct\
ion\x22==typeof t||\
!t.unexpandable:\
qn.hasOwnPropert\
y(e)&&!qn[e].pri\
mitive},e}(),jn=\
/^[\x5cu208a\x5cu208b\x5c\
u208c\x5cu208d\x5cu208\
e\x5cu2080\x5cu2081\x5cu2\
082\x5cu2083\x5cu2084\x5c\
u2085\x5cu2086\x5cu208\
7\x5cu2088\x5cu2089\x5cu2\
090\x5cu2091\x5cu2095\x5c\
u1d62\x5cu2c7c\x5cu209\
6\x5cu2097\x5cu2098\x5cu2\
099\x5cu2092\x5cu209a\x5c\
u1d63\x5cu209b\x5cu209\
c\x5cu1d64\x5cu1d65\x5cu2\
093\x5cu1d66\x5cu1d67\x5c\
u1d68\x5cu1d69\x5cu1d6\
a]/,$n=Object.fr\
eeze({\x22\x5cu208a\x22:\x22\
+\x22,\x22\x5cu208b\x22:\x22-\x22,\
\x22\x5cu208c\x22:\x22=\x22,\x22\x5cu\
208d\x22:\x22(\x22,\x22\x5cu208\
e\x22:\x22)\x22,\x22\x5cu2080\x22:\
\x220\x22,\x22\x5cu2081\x22:\x221\x22\
,\x22\x5cu2082\x22:\x222\x22,\x22\x5c\
u2083\x22:\x223\x22,\x22\x5cu20\
84\x22:\x224\x22,\x22\x5cu2085\x22\
:\x225\x22,\x22\x5cu2086\x22:\x226\
\x22,\x22\x5cu2087\x22:\x227\x22,\x22\
\x5cu2088\x22:\x228\x22,\x22\x5cu2\
089\x22:\x229\x22,\x22\x5cu2090\
\x22:\x22a\x22,\x22\x5cu2091\x22:\x22\
e\x22,\x22\x5cu2095\x22:\x22h\x22,\
\x22\x5cu1d62\x22:\x22i\x22,\x22\x5cu\
2c7c\x22:\x22j\x22,\x22\x5cu209\
6\x22:\x22k\x22,\x22\x5cu2097\x22:\
\x22l\x22,\x22\x5cu2098\x22:\x22m\x22\
,\x22\x5cu2099\x22:\x22n\x22,\x22\x5c\
u2092\x22:\x22o\x22,\x22\x5cu20\
9a\x22:\x22p\x22,\x22\x5cu1d63\x22\
:\x22r\x22,\x22\x5cu209b\x22:\x22s\
\x22,\x22\x5cu209c\x22:\x22t\x22,\x22\
\x5cu1d64\x22:\x22u\x22,\x22\x5cu1\
d65\x22:\x22v\x22,\x22\x5cu2093\
\x22:\x22x\x22,\x22\x5cu1d66\x22:\x22\
\x5cu03b2\x22,\x22\x5cu1d67\x22\
:\x22\x5cu03b3\x22,\x22\x5cu1d6\
8\x22:\x22\x5cu03c1\x22,\x22\x5cu1\
d69\x22:\x22\x5cu03d5\x22,\x22\x5c\
u1d6a\x22:\x22\x5cu03c7\x22,\
\x22\x5cu207a\x22:\x22+\x22,\x22\x5cu\
207b\x22:\x22-\x22,\x22\x5cu207\
c\x22:\x22=\x22,\x22\x5cu207d\x22:\
\x22(\x22,\x22\x5cu207e\x22:\x22)\x22\
,\x22\x5cu2070\x22:\x220\x22,\x22\x5c\
xb9\x22:\x221\x22,\x22\x5cxb2\x22:\
\x222\x22,\x22\x5cxb3\x22:\x223\x22,\x22\
\x5cu2074\x22:\x224\x22,\x22\x5cu2\
075\x22:\x225\x22,\x22\x5cu2076\
\x22:\x226\x22,\x22\x5cu2077\x22:\x22\
7\x22,\x22\x5cu2078\x22:\x228\x22,\
\x22\x5cu2079\x22:\x229\x22,\x22\x5cu\
1d2c\x22:\x22A\x22,\x22\x5cu1d2\
e\x22:\x22B\x22,\x22\x5cu1d30\x22:\
\x22D\x22,\x22\x5cu1d31\x22:\x22E\x22\
,\x22\x5cu1d33\x22:\x22G\x22,\x22\x5c\
u1d34\x22:\x22H\x22,\x22\x5cu1d\
35\x22:\x22I\x22,\x22\x5cu1d36\x22\
:\x22J\x22,\x22\x5cu1d37\x22:\x22K\
\x22,\x22\x5cu1d38\x22:\x22L\x22,\x22\
\x5cu1d39\x22:\x22M\x22,\x22\x5cu1\
d3a\x22:\x22N\x22,\x22\x5cu1d3c\
\x22:\x22O\x22,\x22\x5cu1d3e\x22:\x22\
P\x22,\x22\x5cu1d3f\x22:\x22R\x22,\
\x22\x5cu1d40\x22:\x22T\x22,\x22\x5cu\
1d41\x22:\x22U\x22,\x22\x5cu2c7\
d\x22:\x22V\x22,\x22\x5cu1d42\x22:\
\x22W\x22,\x22\x5cu1d43\x22:\x22a\x22\
,\x22\x5cu1d47\x22:\x22b\x22,\x22\x5c\
u1d9c\x22:\x22c\x22,\x22\x5cu1d\
48\x22:\x22d\x22,\x22\x5cu1d49\x22\
:\x22e\x22,\x22\x5cu1da0\x22:\x22f\
\x22,\x22\x5cu1d4d\x22:\x22g\x22,\x22\
\x5cu02b0\x22:\x22h\x22,\x22\x5cu2\
071\x22:\x22i\x22,\x22\x5cu02b2\
\x22:\x22j\x22,\x22\x5cu1d4f\x22:\x22\
k\x22,\x22\x5cu02e1\x22:\x22l\x22,\
\x22\x5cu1d50\x22:\x22m\x22,\x22\x5cu\
207f\x22:\x22n\x22,\x22\x5cu1d5\
2\x22:\x22o\x22,\x22\x5cu1d56\x22:\
\x22p\x22,\x22\x5cu02b3\x22:\x22r\x22\
,\x22\x5cu02e2\x22:\x22s\x22,\x22\x5c\
u1d57\x22:\x22t\x22,\x22\x5cu1d\
58\x22:\x22u\x22,\x22\x5cu1d5b\x22\
:\x22v\x22,\x22\x5cu02b7\x22:\x22w\
\x22,\x22\x5cu02e3\x22:\x22x\x22,\x22\
\x5cu02b8\x22:\x22y\x22,\x22\x5cu1\
dbb\x22:\x22z\x22,\x22\x5cu1d5d\
\x22:\x22\x5cu03b2\x22,\x22\x5cu1d\
5e\x22:\x22\x5cu03b3\x22,\x22\x5cu\
1d5f\x22:\x22\x5cu03b4\x22,\x22\
\x5cu1d60\x22:\x22\x5cu03d5\x22\
,\x22\x5cu1d61\x22:\x22\x5cu03c\
7\x22,\x22\x5cu1dbf\x22:\x22\x5cu0\
3b8\x22}),Zn={\x22\x5cu03\
01\x22:{text:\x22\x5c\x5c'\x22,\
math:\x22\x5c\x5cacute\x22},\
\x22\x5cu0300\x22:{text:\x22\
\x5c\x5c`\x22,math:\x22\x5c\x5cgra\
ve\x22},\x22\x5cu0308\x22:{t\
ext:'\x5c\x5c\x22',math:\x22\
\x5c\x5cddot\x22},\x22\x5cu0303\
\x22:{text:\x22\x5c\x5c~\x22,ma\
th:\x22\x5c\x5ctilde\x22},\x22\x5c\
u0304\x22:{text:\x22\x5c\x5c\
=\x22,math:\x22\x5c\x5cbar\x22}\
,\x22\x5cu0306\x22:{text:\
\x22\x5c\x5cu\x22,math:\x22\x5c\x5cbr\
eve\x22},\x22\x5cu030c\x22:{\
text:\x22\x5c\x5cv\x22,math:\
\x22\x5c\x5ccheck\x22},\x22\x5cu03\
02\x22:{text:\x22\x5c\x5c^\x22,\
math:\x22\x5c\x5chat\x22},\x22\x5c\
u0307\x22:{text:\x22\x5c\x5c\
.\x22,math:\x22\x5c\x5cdot\x22}\
,\x22\x5cu030a\x22:{text:\
\x22\x5c\x5cr\x22,math:\x22\x5c\x5cma\
thring\x22},\x22\x5cu030b\
\x22:{text:\x22\x5c\x5cH\x22},\x22\
\x5cu0327\x22:{text:\x22\x5c\
\x5cc\x22}},Kn={\x22\x5cxe1\x22\
:\x22a\x5cu0301\x22,\x22\x5cxe0\
\x22:\x22a\x5cu0300\x22,\x22\x5cxe\
4\x22:\x22a\x5cu0308\x22,\x22\x5cu\
01df\x22:\x22a\x5cu0308\x5cu\
0304\x22,\x22\x5cxe3\x22:\x22a\x5c\
u0303\x22,\x22\x5cu0101\x22:\
\x22a\x5cu0304\x22,\x22\x5cu010\
3\x22:\x22a\x5cu0306\x22,\x22\x5cu\
1eaf\x22:\x22a\x5cu0306\x5cu\
0301\x22,\x22\x5cu1eb1\x22:\x22\
a\x5cu0306\x5cu0300\x22,\x22\
\x5cu1eb5\x22:\x22a\x5cu0306\
\x5cu0303\x22,\x22\x5cu01ce\x22\
:\x22a\x5cu030c\x22,\x22\x5cxe2\
\x22:\x22a\x5cu0302\x22,\x22\x5cu1\
ea5\x22:\x22a\x5cu0302\x5cu0\
301\x22,\x22\x5cu1ea7\x22:\x22a\
\x5cu0302\x5cu0300\x22,\x22\x5c\
u1eab\x22:\x22a\x5cu0302\x5c\
u0303\x22,\x22\x5cu0227\x22:\
\x22a\x5cu0307\x22,\x22\x5cu01e\
1\x22:\x22a\x5cu0307\x5cu030\
4\x22,\x22\x5cxe5\x22:\x22a\x5cu03\
0a\x22,\x22\x5cu01fb\x22:\x22a\x5c\
u030a\x5cu0301\x22,\x22\x5cu\
1e03\x22:\x22b\x5cu0307\x22,\
\x22\x5cu0107\x22:\x22c\x5cu030\
1\x22,\x22\x5cu1e09\x22:\x22c\x5cu\
0327\x5cu0301\x22,\x22\x5cu0\
10d\x22:\x22c\x5cu030c\x22,\x22\
\x5cu0109\x22:\x22c\x5cu0302\
\x22,\x22\x5cu010b\x22:\x22c\x5cu0\
307\x22,\x22\x5cxe7\x22:\x22c\x5cu\
0327\x22,\x22\x5cu010f\x22:\x22\
d\x5cu030c\x22,\x22\x5cu1e0b\
\x22:\x22d\x5cu0307\x22,\x22\x5cu1\
e11\x22:\x22d\x5cu0327\x22,\x22\
\x5cxe9\x22:\x22e\x5cu0301\x22,\
\x22\x5cxe8\x22:\x22e\x5cu0300\x22\
,\x22\x5cxeb\x22:\x22e\x5cu0308\
\x22,\x22\x5cu1ebd\x22:\x22e\x5cu0\
303\x22,\x22\x5cu0113\x22:\x22e\
\x5cu0304\x22,\x22\x5cu1e17\x22\
:\x22e\x5cu0304\x5cu0301\x22\
,\x22\x5cu1e15\x22:\x22e\x5cu03\
04\x5cu0300\x22,\x22\x5cu011\
5\x22:\x22e\x5cu0306\x22,\x22\x5cu\
1e1d\x22:\x22e\x5cu0327\x5cu\
0306\x22,\x22\x5cu011b\x22:\x22\
e\x5cu030c\x22,\x22\x5cxea\x22:\
\x22e\x5cu0302\x22,\x22\x5cu1eb\
f\x22:\x22e\x5cu0302\x5cu030\
1\x22,\x22\x5cu1ec1\x22:\x22e\x5cu\
0302\x5cu0300\x22,\x22\x5cu1\
ec5\x22:\x22e\x5cu0302\x5cu0\
303\x22,\x22\x5cu0117\x22:\x22e\
\x5cu0307\x22,\x22\x5cu0229\x22\
:\x22e\x5cu0327\x22,\x22\x5cu1e\
1f\x22:\x22f\x5cu0307\x22,\x22\x5c\
u01f5\x22:\x22g\x5cu0301\x22\
,\x22\x5cu1e21\x22:\x22g\x5cu03\
04\x22,\x22\x5cu011f\x22:\x22g\x5c\
u0306\x22,\x22\x5cu01e7\x22:\
\x22g\x5cu030c\x22,\x22\x5cu011\
d\x22:\x22g\x5cu0302\x22,\x22\x5cu\
0121\x22:\x22g\x5cu0307\x22,\
\x22\x5cu0123\x22:\x22g\x5cu032\
7\x22,\x22\x5cu1e27\x22:\x22h\x5cu\
0308\x22,\x22\x5cu021f\x22:\x22\
h\x5cu030c\x22,\x22\x5cu0125\
\x22:\x22h\x5cu0302\x22,\x22\x5cu1\
e23\x22:\x22h\x5cu0307\x22,\x22\
\x5cu1e29\x22:\x22h\x5cu0327\
\x22,\x22\x5cxed\x22:\x22i\x5cu030\
1\x22,\x22\x5cxec\x22:\x22i\x5cu03\
00\x22,\x22\x5cxef\x22:\x22i\x5cu0\
308\x22,\x22\x5cu1e2f\x22:\x22i\
\x5cu0308\x5cu0301\x22,\x22\x5c\
u0129\x22:\x22i\x5cu0303\x22\
,\x22\x5cu012b\x22:\x22i\x5cu03\
04\x22,\x22\x5cu012d\x22:\x22i\x5c\
u0306\x22,\x22\x5cu01d0\x22:\
\x22i\x5cu030c\x22,\x22\x5cxee\x22\
:\x22i\x5cu0302\x22,\x22\x5cu01\
f0\x22:\x22j\x5cu030c\x22,\x22\x5c\
u0135\x22:\x22j\x5cu0302\x22\
,\x22\x5cu1e31\x22:\x22k\x5cu03\
01\x22,\x22\x5cu01e9\x22:\x22k\x5c\
u030c\x22,\x22\x5cu0137\x22:\
\x22k\x5cu0327\x22,\x22\x5cu013\
a\x22:\x22l\x5cu0301\x22,\x22\x5cu\
013e\x22:\x22l\x5cu030c\x22,\
\x22\x5cu013c\x22:\x22l\x5cu032\
7\x22,\x22\x5cu1e3f\x22:\x22m\x5cu\
0301\x22,\x22\x5cu1e41\x22:\x22\
m\x5cu0307\x22,\x22\x5cu0144\
\x22:\x22n\x5cu0301\x22,\x22\x5cu0\
1f9\x22:\x22n\x5cu0300\x22,\x22\
\x5cxf1\x22:\x22n\x5cu0303\x22,\
\x22\x5cu0148\x22:\x22n\x5cu030\
c\x22,\x22\x5cu1e45\x22:\x22n\x5cu\
0307\x22,\x22\x5cu0146\x22:\x22\
n\x5cu0327\x22,\x22\x5cxf3\x22:\
\x22o\x5cu0301\x22,\x22\x5cxf2\x22\
:\x22o\x5cu0300\x22,\x22\x5cxf6\
\x22:\x22o\x5cu0308\x22,\x22\x5cu0\
22b\x22:\x22o\x5cu0308\x5cu0\
304\x22,\x22\x5cxf5\x22:\x22o\x5cu\
0303\x22,\x22\x5cu1e4d\x22:\x22\
o\x5cu0303\x5cu0301\x22,\x22\
\x5cu1e4f\x22:\x22o\x5cu0303\
\x5cu0308\x22,\x22\x5cu022d\x22\
:\x22o\x5cu0303\x5cu0304\x22\
,\x22\x5cu014d\x22:\x22o\x5cu03\
04\x22,\x22\x5cu1e53\x22:\x22o\x5c\
u0304\x5cu0301\x22,\x22\x5cu\
1e51\x22:\x22o\x5cu0304\x5cu\
0300\x22,\x22\x5cu014f\x22:\x22\
o\x5cu0306\x22,\x22\x5cu01d2\
\x22:\x22o\x5cu030c\x22,\x22\x5cxf\
4\x22:\x22o\x5cu0302\x22,\x22\x5cu\
1ed1\x22:\x22o\x5cu0302\x5cu\
0301\x22,\x22\x5cu1ed3\x22:\x22\
o\x5cu0302\x5cu0300\x22,\x22\
\x5cu1ed7\x22:\x22o\x5cu0302\
\x5cu0303\x22,\x22\x5cu022f\x22\
:\x22o\x5cu0307\x22,\x22\x5cu02\
31\x22:\x22o\x5cu0307\x5cu03\
04\x22,\x22\x5cu0151\x22:\x22o\x5c\
u030b\x22,\x22\x5cu1e55\x22:\
\x22p\x5cu0301\x22,\x22\x5cu1e5\
7\x22:\x22p\x5cu0307\x22,\x22\x5cu\
0155\x22:\x22r\x5cu0301\x22,\
\x22\x5cu0159\x22:\x22r\x5cu030\
c\x22,\x22\x5cu1e59\x22:\x22r\x5cu\
0307\x22,\x22\x5cu0157\x22:\x22\
r\x5cu0327\x22,\x22\x5cu015b\
\x22:\x22s\x5cu0301\x22,\x22\x5cu1\
e65\x22:\x22s\x5cu0301\x5cu0\
307\x22,\x22\x5cu0161\x22:\x22s\
\x5cu030c\x22,\x22\x5cu1e67\x22\
:\x22s\x5cu030c\x5cu0307\x22\
,\x22\x5cu015d\x22:\x22s\x5cu03\
02\x22,\x22\x5cu1e61\x22:\x22s\x5c\
u0307\x22,\x22\x5cu015f\x22:\
\x22s\x5cu0327\x22,\x22\x5cu1e9\
7\x22:\x22t\x5cu0308\x22,\x22\x5cu\
0165\x22:\x22t\x5cu030c\x22,\
\x22\x5cu1e6b\x22:\x22t\x5cu030\
7\x22,\x22\x5cu0163\x22:\x22t\x5cu\
0327\x22,\x22\x5cxfa\x22:\x22u\x5c\
u0301\x22,\x22\x5cxf9\x22:\x22u\
\x5cu0300\x22,\x22\x5cxfc\x22:\x22\
u\x5cu0308\x22,\x22\x5cu01d8\
\x22:\x22u\x5cu0308\x5cu0301\
\x22,\x22\x5cu01dc\x22:\x22u\x5cu0\
308\x5cu0300\x22,\x22\x5cu01\
d6\x22:\x22u\x5cu0308\x5cu03\
04\x22,\x22\x5cu01da\x22:\x22u\x5c\
u0308\x5cu030c\x22,\x22\x5cu\
0169\x22:\x22u\x5cu0303\x22,\
\x22\x5cu1e79\x22:\x22u\x5cu030\
3\x5cu0301\x22,\x22\x5cu016b\
\x22:\x22u\x5cu0304\x22,\x22\x5cu1\
e7b\x22:\x22u\x5cu0304\x5cu0\
308\x22,\x22\x5cu016d\x22:\x22u\
\x5cu0306\x22,\x22\x5cu01d4\x22\
:\x22u\x5cu030c\x22,\x22\x5cxfb\
\x22:\x22u\x5cu0302\x22,\x22\x5cu0\
16f\x22:\x22u\x5cu030a\x22,\x22\
\x5cu0171\x22:\x22u\x5cu030b\
\x22,\x22\x5cu1e7d\x22:\x22v\x5cu0\
303\x22,\x22\x5cu1e83\x22:\x22w\
\x5cu0301\x22,\x22\x5cu1e81\x22\
:\x22w\x5cu0300\x22,\x22\x5cu1e\
85\x22:\x22w\x5cu0308\x22,\x22\x5c\
u0175\x22:\x22w\x5cu0302\x22\
,\x22\x5cu1e87\x22:\x22w\x5cu03\
07\x22,\x22\x5cu1e98\x22:\x22w\x5c\
u030a\x22,\x22\x5cu1e8d\x22:\
\x22x\x5cu0308\x22,\x22\x5cu1e8\
b\x22:\x22x\x5cu0307\x22,\x22\x5cx\
fd\x22:\x22y\x5cu0301\x22,\x22\x5c\
u1ef3\x22:\x22y\x5cu0300\x22\
,\x22\x5cxff\x22:\x22y\x5cu0308\
\x22,\x22\x5cu1ef9\x22:\x22y\x5cu0\
303\x22,\x22\x5cu0233\x22:\x22y\
\x5cu0304\x22,\x22\x5cu0177\x22\
:\x22y\x5cu0302\x22,\x22\x5cu1e\
8f\x22:\x22y\x5cu0307\x22,\x22\x5c\
u1e99\x22:\x22y\x5cu030a\x22\
,\x22\x5cu017a\x22:\x22z\x5cu03\
01\x22,\x22\x5cu017e\x22:\x22z\x5c\
u030c\x22,\x22\x5cu1e91\x22:\
\x22z\x5cu0302\x22,\x22\x5cu017\
c\x22:\x22z\x5cu0307\x22,\x22\x5cx\
c1\x22:\x22A\x5cu0301\x22,\x22\x5c\
xc0\x22:\x22A\x5cu0300\x22,\x22\
\x5cxc4\x22:\x22A\x5cu0308\x22,\
\x22\x5cu01de\x22:\x22A\x5cu030\
8\x5cu0304\x22,\x22\x5cxc3\x22:\
\x22A\x5cu0303\x22,\x22\x5cu010\
0\x22:\x22A\x5cu0304\x22,\x22\x5cu\
0102\x22:\x22A\x5cu0306\x22,\
\x22\x5cu1eae\x22:\x22A\x5cu030\
6\x5cu0301\x22,\x22\x5cu1eb0\
\x22:\x22A\x5cu0306\x5cu0300\
\x22,\x22\x5cu1eb4\x22:\x22A\x5cu0\
306\x5cu0303\x22,\x22\x5cu01\
cd\x22:\x22A\x5cu030c\x22,\x22\x5c\
xc2\x22:\x22A\x5cu0302\x22,\x22\
\x5cu1ea4\x22:\x22A\x5cu0302\
\x5cu0301\x22,\x22\x5cu1ea6\x22\
:\x22A\x5cu0302\x5cu0300\x22\
,\x22\x5cu1eaa\x22:\x22A\x5cu03\
02\x5cu0303\x22,\x22\x5cu022\
6\x22:\x22A\x5cu0307\x22,\x22\x5cu\
01e0\x22:\x22A\x5cu0307\x5cu\
0304\x22,\x22\x5cxc5\x22:\x22A\x5c\
u030a\x22,\x22\x5cu01fa\x22:\
\x22A\x5cu030a\x5cu0301\x22,\
\x22\x5cu1e02\x22:\x22B\x5cu030\
7\x22,\x22\x5cu0106\x22:\x22C\x5cu\
0301\x22,\x22\x5cu1e08\x22:\x22\
C\x5cu0327\x5cu0301\x22,\x22\
\x5cu010c\x22:\x22C\x5cu030c\
\x22,\x22\x5cu0108\x22:\x22C\x5cu0\
302\x22,\x22\x5cu010a\x22:\x22C\
\x5cu0307\x22,\x22\x5cxc7\x22:\x22\
C\x5cu0327\x22,\x22\x5cu010e\
\x22:\x22D\x5cu030c\x22,\x22\x5cu1\
e0a\x22:\x22D\x5cu0307\x22,\x22\
\x5cu1e10\x22:\x22D\x5cu0327\
\x22,\x22\x5cxc9\x22:\x22E\x5cu030\
1\x22,\x22\x5cxc8\x22:\x22E\x5cu03\
00\x22,\x22\x5cxcb\x22:\x22E\x5cu0\
308\x22,\x22\x5cu1ebc\x22:\x22E\
\x5cu0303\x22,\x22\x5cu0112\x22\
:\x22E\x5cu0304\x22,\x22\x5cu1e\
16\x22:\x22E\x5cu0304\x5cu03\
01\x22,\x22\x5cu1e14\x22:\x22E\x5c\
u0304\x5cu0300\x22,\x22\x5cu\
0114\x22:\x22E\x5cu0306\x22,\
\x22\x5cu1e1c\x22:\x22E\x5cu032\
7\x5cu0306\x22,\x22\x5cu011a\
\x22:\x22E\x5cu030c\x22,\x22\x5cxc\
a\x22:\x22E\x5cu0302\x22,\x22\x5cu\
1ebe\x22:\x22E\x5cu0302\x5cu\
0301\x22,\x22\x5cu1ec0\x22:\x22\
E\x5cu0302\x5cu0300\x22,\x22\
\x5cu1ec4\x22:\x22E\x5cu0302\
\x5cu0303\x22,\x22\x5cu0116\x22\
:\x22E\x5cu0307\x22,\x22\x5cu02\
28\x22:\x22E\x5cu0327\x22,\x22\x5c\
u1e1e\x22:\x22F\x5cu0307\x22\
,\x22\x5cu01f4\x22:\x22G\x5cu03\
01\x22,\x22\x5cu1e20\x22:\x22G\x5c\
u0304\x22,\x22\x5cu011e\x22:\
\x22G\x5cu0306\x22,\x22\x5cu01e\
6\x22:\x22G\x5cu030c\x22,\x22\x5cu\
011c\x22:\x22G\x5cu0302\x22,\
\x22\x5cu0120\x22:\x22G\x5cu030\
7\x22,\x22\x5cu0122\x22:\x22G\x5cu\
0327\x22,\x22\x5cu1e26\x22:\x22\
H\x5cu0308\x22,\x22\x5cu021e\
\x22:\x22H\x5cu030c\x22,\x22\x5cu0\
124\x22:\x22H\x5cu0302\x22,\x22\
\x5cu1e22\x22:\x22H\x5cu0307\
\x22,\x22\x5cu1e28\x22:\x22H\x5cu0\
327\x22,\x22\x5cxcd\x22:\x22I\x5cu\
0301\x22,\x22\x5cxcc\x22:\x22I\x5c\
u0300\x22,\x22\x5cxcf\x22:\x22I\
\x5cu0308\x22,\x22\x5cu1e2e\x22\
:\x22I\x5cu0308\x5cu0301\x22\
,\x22\x5cu0128\x22:\x22I\x5cu03\
03\x22,\x22\x5cu012a\x22:\x22I\x5c\
u0304\x22,\x22\x5cu012c\x22:\
\x22I\x5cu0306\x22,\x22\x5cu01c\
f\x22:\x22I\x5cu030c\x22,\x22\x5cx\
ce\x22:\x22I\x5cu0302\x22,\x22\x5c\
u0130\x22:\x22I\x5cu0307\x22\
,\x22\x5cu0134\x22:\x22J\x5cu03\
02\x22,\x22\x5cu1e30\x22:\x22K\x5c\
u0301\x22,\x22\x5cu01e8\x22:\
\x22K\x5cu030c\x22,\x22\x5cu013\
6\x22:\x22K\x5cu0327\x22,\x22\x5cu\
0139\x22:\x22L\x5cu0301\x22,\
\x22\x5cu013d\x22:\x22L\x5cu030\
c\x22,\x22\x5cu013b\x22:\x22L\x5cu\
0327\x22,\x22\x5cu1e3e\x22:\x22\
M\x5cu0301\x22,\x22\x5cu1e40\
\x22:\x22M\x5cu0307\x22,\x22\x5cu0\
143\x22:\x22N\x5cu0301\x22,\x22\
\x5cu01f8\x22:\x22N\x5cu0300\
\x22,\x22\x5cxd1\x22:\x22N\x5cu030\
3\x22,\x22\x5cu0147\x22:\x22N\x5cu\
030c\x22,\x22\x5cu1e44\x22:\x22\
N\x5cu0307\x22,\x22\x5cu0145\
\x22:\x22N\x5cu0327\x22,\x22\x5cxd\
3\x22:\x22O\x5cu0301\x22,\x22\x5cx\
d2\x22:\x22O\x5cu0300\x22,\x22\x5c\
xd6\x22:\x22O\x5cu0308\x22,\x22\
\x5cu022a\x22:\x22O\x5cu0308\
\x5cu0304\x22,\x22\x5cxd5\x22:\x22\
O\x5cu0303\x22,\x22\x5cu1e4c\
\x22:\x22O\x5cu0303\x5cu0301\
\x22,\x22\x5cu1e4e\x22:\x22O\x5cu0\
303\x5cu0308\x22,\x22\x5cu02\
2c\x22:\x22O\x5cu0303\x5cu03\
04\x22,\x22\x5cu014c\x22:\x22O\x5c\
u0304\x22,\x22\x5cu1e52\x22:\
\x22O\x5cu0304\x5cu0301\x22,\
\x22\x5cu1e50\x22:\x22O\x5cu030\
4\x5cu0300\x22,\x22\x5cu014e\
\x22:\x22O\x5cu0306\x22,\x22\x5cu0\
1d1\x22:\x22O\x5cu030c\x22,\x22\
\x5cxd4\x22:\x22O\x5cu0302\x22,\
\x22\x5cu1ed0\x22:\x22O\x5cu030\
2\x5cu0301\x22,\x22\x5cu1ed2\
\x22:\x22O\x5cu0302\x5cu0300\
\x22,\x22\x5cu1ed6\x22:\x22O\x5cu0\
302\x5cu0303\x22,\x22\x5cu02\
2e\x22:\x22O\x5cu0307\x22,\x22\x5c\
u0230\x22:\x22O\x5cu0307\x5c\
u0304\x22,\x22\x5cu0150\x22:\
\x22O\x5cu030b\x22,\x22\x5cu1e5\
4\x22:\x22P\x5cu0301\x22,\x22\x5cu\
1e56\x22:\x22P\x5cu0307\x22,\
\x22\x5cu0154\x22:\x22R\x5cu030\
1\x22,\x22\x5cu0158\x22:\x22R\x5cu\
030c\x22,\x22\x5cu1e58\x22:\x22\
R\x5cu0307\x22,\x22\x5cu0156\
\x22:\x22R\x5cu0327\x22,\x22\x5cu0\
15a\x22:\x22S\x5cu0301\x22,\x22\
\x5cu1e64\x22:\x22S\x5cu0301\
\x5cu0307\x22,\x22\x5cu0160\x22\
:\x22S\x5cu030c\x22,\x22\x5cu1e\
66\x22:\x22S\x5cu030c\x5cu03\
07\x22,\x22\x5cu015c\x22:\x22S\x5c\
u0302\x22,\x22\x5cu1e60\x22:\
\x22S\x5cu0307\x22,\x22\x5cu015\
e\x22:\x22S\x5cu0327\x22,\x22\x5cu\
0164\x22:\x22T\x5cu030c\x22,\
\x22\x5cu1e6a\x22:\x22T\x5cu030\
7\x22,\x22\x5cu0162\x22:\x22T\x5cu\
0327\x22,\x22\x5cxda\x22:\x22U\x5c\
u0301\x22,\x22\x5cxd9\x22:\x22U\
\x5cu0300\x22,\x22\x5cxdc\x22:\x22\
U\x5cu0308\x22,\x22\x5cu01d7\
\x22:\x22U\x5cu0308\x5cu0301\
\x22,\x22\x5cu01db\x22:\x22U\x5cu0\
308\x5cu0300\x22,\x22\x5cu01\
d5\x22:\x22U\x5cu0308\x5cu03\
04\x22,\x22\x5cu01d9\x22:\x22U\x5c\
u0308\x5cu030c\x22,\x22\x5cu\
0168\x22:\x22U\x5cu0303\x22,\
\x22\x5cu1e78\x22:\x22U\x5cu030\
3\x5cu0301\x22,\x22\x5cu016a\
\x22:\x22U\x5cu0304\x22,\x22\x5cu1\
e7a\x22:\x22U\x5cu0304\x5cu0\
308\x22,\x22\x5cu016c\x22:\x22U\
\x5cu0306\x22,\x22\x5cu01d3\x22\
:\x22U\x5cu030c\x22,\x22\x5cxdb\
\x22:\x22U\x5cu0302\x22,\x22\x5cu0\
16e\x22:\x22U\x5cu030a\x22,\x22\
\x5cu0170\x22:\x22U\x5cu030b\
\x22,\x22\x5cu1e7c\x22:\x22V\x5cu0\
303\x22,\x22\x5cu1e82\x22:\x22W\
\x5cu0301\x22,\x22\x5cu1e80\x22\
:\x22W\x5cu0300\x22,\x22\x5cu1e\
84\x22:\x22W\x5cu0308\x22,\x22\x5c\
u0174\x22:\x22W\x5cu0302\x22\
,\x22\x5cu1e86\x22:\x22W\x5cu03\
07\x22,\x22\x5cu1e8c\x22:\x22X\x5c\
u0308\x22,\x22\x5cu1e8a\x22:\
\x22X\x5cu0307\x22,\x22\x5cxdd\x22\
:\x22Y\x5cu0301\x22,\x22\x5cu1e\
f2\x22:\x22Y\x5cu0300\x22,\x22\x5c\
u0178\x22:\x22Y\x5cu0308\x22\
,\x22\x5cu1ef8\x22:\x22Y\x5cu03\
03\x22,\x22\x5cu0232\x22:\x22Y\x5c\
u0304\x22,\x22\x5cu0176\x22:\
\x22Y\x5cu0302\x22,\x22\x5cu1e8\
e\x22:\x22Y\x5cu0307\x22,\x22\x5cu\
0179\x22:\x22Z\x5cu0301\x22,\
\x22\x5cu017d\x22:\x22Z\x5cu030\
c\x22,\x22\x5cu1e90\x22:\x22Z\x5cu\
0302\x22,\x22\x5cu017b\x22:\x22\
Z\x5cu0307\x22,\x22\x5cu03ac\
\x22:\x22\x5cu03b1\x5cu0301\x22\
,\x22\x5cu1f70\x22:\x22\x5cu03b\
1\x5cu0300\x22,\x22\x5cu1fb1\
\x22:\x22\x5cu03b1\x5cu0304\x22\
,\x22\x5cu1fb0\x22:\x22\x5cu03b\
1\x5cu0306\x22,\x22\x5cu03ad\
\x22:\x22\x5cu03b5\x5cu0301\x22\
,\x22\x5cu1f72\x22:\x22\x5cu03b\
5\x5cu0300\x22,\x22\x5cu03ae\
\x22:\x22\x5cu03b7\x5cu0301\x22\
,\x22\x5cu1f74\x22:\x22\x5cu03b\
7\x5cu0300\x22,\x22\x5cu03af\
\x22:\x22\x5cu03b9\x5cu0301\x22\
,\x22\x5cu1f76\x22:\x22\x5cu03b\
9\x5cu0300\x22,\x22\x5cu03ca\
\x22:\x22\x5cu03b9\x5cu0308\x22\
,\x22\x5cu0390\x22:\x22\x5cu03b\
9\x5cu0308\x5cu0301\x22,\x22\
\x5cu1fd2\x22:\x22\x5cu03b9\x5c\
u0308\x5cu0300\x22,\x22\x5cu\
1fd1\x22:\x22\x5cu03b9\x5cu0\
304\x22,\x22\x5cu1fd0\x22:\x22\x5c\
u03b9\x5cu0306\x22,\x22\x5cu\
03cc\x22:\x22\x5cu03bf\x5cu0\
301\x22,\x22\x5cu1f78\x22:\x22\x5c\
u03bf\x5cu0300\x22,\x22\x5cu\
03cd\x22:\x22\x5cu03c5\x5cu0\
301\x22,\x22\x5cu1f7a\x22:\x22\x5c\
u03c5\x5cu0300\x22,\x22\x5cu\
03cb\x22:\x22\x5cu03c5\x5cu0\
308\x22,\x22\x5cu03b0\x22:\x22\x5c\
u03c5\x5cu0308\x5cu030\
1\x22,\x22\x5cu1fe2\x22:\x22\x5cu0\
3c5\x5cu0308\x5cu0300\x22\
,\x22\x5cu1fe1\x22:\x22\x5cu03c\
5\x5cu0304\x22,\x22\x5cu1fe0\
\x22:\x22\x5cu03c5\x5cu0306\x22\
,\x22\x5cu03ce\x22:\x22\x5cu03c\
9\x5cu0301\x22,\x22\x5cu1f7c\
\x22:\x22\x5cu03c9\x5cu0300\x22\
,\x22\x5cu038e\x22:\x22\x5cu03a\
5\x5cu0301\x22,\x22\x5cu1fea\
\x22:\x22\x5cu03a5\x5cu0300\x22\
,\x22\x5cu03ab\x22:\x22\x5cu03a\
5\x5cu0308\x22,\x22\x5cu1fe9\
\x22:\x22\x5cu03a5\x5cu0304\x22\
,\x22\x5cu1fe8\x22:\x22\x5cu03a\
5\x5cu0306\x22,\x22\x5cu038f\
\x22:\x22\x5cu03a9\x5cu0301\x22\
,\x22\x5cu1ffa\x22:\x22\x5cu03a\
9\x5cu0300\x22},Jn=fun\
ction(){function\
 e(e,t){this.mod\
e=void 0,this.gu\
llet=void 0,this\
.settings=void 0\
,this.leftrightD\
epth=void 0,this\
.nextToken=void \
0,this.mode=\x22mat\
h\x22,this.gullet=n\
ew _n(e,t,this.m\
ode),this.settin\
gs=t,this.leftri\
ghtDepth=0}var t\
=e.prototype;ret\
urn t.expect=fun\
ction(e,t){if(vo\
id 0===t&&(t=!0)\
,this.fetch().te\
xt!==e)throw new\
 n(\x22Expected '\x22+\
e+\x22', got '\x22+thi\
s.fetch().text+\x22\
'\x22,this.fetch())\
;t&&this.consume\
()},t.consume=fu\
nction(){this.ne\
xtToken=null},t.\
fetch=function()\
{return null==th\
is.nextToken&&(t\
his.nextToken=th\
is.gullet.expand\
NextToken()),thi\
s.nextToken},t.s\
witchMode=functi\
on(e){this.mode=\
e,this.gullet.sw\
itchMode(e)},t.p\
arse=function(){\
this.settings.gl\
obalGroup||this.\
gullet.beginGrou\
p(),this.setting\
s.colorIsTextCol\
or&&this.gullet.\
macros.set(\x22\x5c\x5cco\
lor\x22,\x22\x5c\x5ctextcolo\
r\x22);try{var e=th\
is.parseExpressi\
on(!1);return th\
is.expect(\x22EOF\x22)\
,this.settings.g\
lobalGroup||this\
.gullet.endGroup\
(),e}finally{thi\
s.gullet.endGrou\
ps()}},t.subpars\
e=function(e){va\
r t=this.nextTok\
en;this.consume(\
),this.gullet.pu\
shToken(new Gr(\x22\
}\x22)),this.gullet\
.pushTokens(e);v\
ar r=this.parseE\
xpression(!1);re\
turn this.expect\
(\x22}\x22),this.nextT\
oken=t,r},t.pars\
eExpression=func\
tion(t,r){for(va\
r n=[];;){\x22math\x22\
===this.mode&&th\
is.consumeSpaces\
();var a=this.fe\
tch();if(-1!==e.\
endOfExpression.\
indexOf(a.text))\
break;if(r&&a.te\
xt===r)break;if(\
t&&qn[a.text]&&q\
n[a.text].infix)\
break;var i=this\
.parseAtom(r);if\
(!i)break;\x22inter\
nal\x22!==i.type&&n\
.push(i)}return\x22\
text\x22===this.mod\
e&&this.formLiga\
tures(n),this.ha\
ndleInfixNodes(n\
)},t.handleInfix\
Nodes=function(e\
){for(var t,r=-1\
,a=0;a<e.length;\
a++)if(\x22infix\x22==\
=e[a].type){if(-\
1!==r)throw new \
n(\x22only one infi\
x operator per g\
roup\x22,e[a].token\
);r=a,t=e[a].rep\
laceWith}if(-1!=\
=r&&t){var i,o,s\
=e.slice(0,r),l=\
e.slice(r+1);ret\
urn i=1===s.leng\
th&&\x22ordgroup\x22==\
=s[0].type?s[0]:\
{type:\x22ordgroup\x22\
,mode:this.mode,\
body:s},o=1===l.\
length&&\x22ordgrou\
p\x22===l[0].type?l\
[0]:{type:\x22ordgr\
oup\x22,mode:this.m\
ode,body:l},[\x22\x5c\x5c\
\x5c\x5cabovefrac\x22===t\
?this.callFuncti\
on(t,[i,e[r],o],\
[]):this.callFun\
ction(t,[i,o],[]\
)]}return e},t.h\
andleSupSubscrip\
t=function(e){va\
r t=this.fetch()\
,r=t.text;this.c\
onsume(),this.co\
nsumeSpaces();va\
r a=this.parseGr\
oup(e);if(!a)thr\
ow new n(\x22Expect\
ed group after '\
\x22+r+\x22'\x22,t);retur\
n a},t.formatUns\
upportedCmd=func\
tion(e){for(var \
t=[],r=0;r<e.len\
gth;r++)t.push({\
type:\x22textord\x22,m\
ode:\x22text\x22,text:\
e[r]});var n={ty\
pe:\x22text\x22,mode:t\
his.mode,body:t}\
;return{type:\x22co\
lor\x22,mode:this.m\
ode,color:this.s\
ettings.errorCol\
or,body:[n]}},t.\
parseAtom=functi\
on(t){var r,a,i=\
this.parseGroup(\
\x22atom\x22,t);if(\x22te\
xt\x22===this.mode)\
return i;for(;;)\
{this.consumeSpa\
ces();var o=this\
.fetch();if(\x22\x5c\x5cl\
imits\x22===o.text|\
|\x22\x5c\x5cnolimits\x22===\
o.text){if(i&&\x22o\
p\x22===i.type){var\
 s=\x22\x5c\x5climits\x22===\
o.text;i.limits=\
s,i.alwaysHandle\
SupSub=!0}else{i\
f(!i||\x22operatorn\
ame\x22!==i.type)th\
row new n(\x22Limit\
 controls must f\
ollow a math ope\
rator\x22,o);i.alwa\
ysHandleSupSub&&\
(i.limits=\x22\x5c\x5clim\
its\x22===o.text)}t\
his.consume()}el\
se if(\x22^\x22===o.te\
xt){if(r)throw n\
ew n(\x22Double sup\
erscript\x22,o);r=t\
his.handleSupSub\
script(\x22superscr\
ipt\x22)}else if(\x22_\
\x22===o.text){if(a\
)throw new n(\x22Do\
uble subscript\x22,\
o);a=this.handle\
SupSubscript(\x22su\
bscript\x22)}else i\
f(\x22'\x22===o.text){\
if(r)throw new n\
(\x22Double supersc\
ript\x22,o);var l={\
type:\x22textord\x22,m\
ode:this.mode,te\
xt:\x22\x5c\x5cprime\x22},h=\
[l];for(this.con\
sume();\x22'\x22===thi\
s.fetch().text;)\
h.push(l),this.c\
onsume();\x22^\x22===t\
his.fetch().text\
&&h.push(this.ha\
ndleSupSubscript\
(\x22superscript\x22))\
,r={type:\x22ordgro\
up\x22,mode:this.mo\
de,body:h}}else{\
if(!$n[o.text])b\
reak;var c=$n[o.\
text],m=jn.test(\
o.text);for(this\
.consume();;){va\
r u=this.fetch()\
.text;if(!$n[u])\
break;if(jn.test\
(u)!==m)break;th\
is.consume(),c+=\
$n[u]}var p=new \
e(c,this.setting\
s).parse();m?a={\
type:\x22ordgroup\x22,\
mode:\x22math\x22,body\
:p}:r={type:\x22ord\
group\x22,mode:\x22mat\
h\x22,body:p}}}retu\
rn r||a?{type:\x22s\
upsub\x22,mode:this\
.mode,base:i,sup\
:r,sub:a}:i},t.p\
arseFunction=fun\
ction(e,t){var r\
=this.fetch(),a=\
r.text,i=qn[a];i\
f(!i)return null\
;if(this.consume\
(),t&&\x22atom\x22!==t\
&&!i.allowedInAr\
gument)throw new\
 n(\x22Got function\
 '\x22+a+\x22' with no\
 arguments\x22+(t?\x22\
 as \x22+t:\x22\x22),r);i\
f(\x22text\x22===this.\
mode&&!i.allowed\
InText)throw new\
 n(\x22Can't use fu\
nction '\x22+a+\x22' i\
n text mode\x22,r);\
if(\x22math\x22===this\
.mode&&!1===i.al\
lowedInMath)thro\
w new n(\x22Can't u\
se function '\x22+a\
+\x22' in math mode\
\x22,r);var o=this.\
parseArguments(a\
,i),s=o.args,l=o\
.optArgs;return \
this.callFunctio\
n(a,s,l,r,e)},t.\
callFunction=fun\
ction(e,t,r,a,i)\
{var o={funcName\
:e,parser:this,t\
oken:a,breakOnTo\
kenText:i},s=qn[\
e];if(s&&s.handl\
er)return s.hand\
ler(o,t,r);throw\
 new n(\x22No funct\
ion handler for \
\x22+e)},t.parseArg\
uments=function(\
e,t){var r=t.num\
Args+t.numOption\
alArgs;if(0===r)\
return{args:[],o\
ptArgs:[]};for(v\
ar a=[],i=[],o=0\
;o<r;o++){var s=\
t.argTypes&&t.ar\
gTypes[o],l=o<t.\
numOptionalArgs;\
(t.primitive&&nu\
ll==s||\x22sqrt\x22===\
t.type&&1===o&&n\
ull==i[0])&&(s=\x22\
primitive\x22);var \
h=this.parseGrou\
pOfType(\x22argumen\
t to '\x22+e+\x22'\x22,s,\
l);if(l)i.push(h\
);else{if(null==\
h)throw new n(\x22N\
ull argument, pl\
ease report this\
 as a bug\x22);a.pu\
sh(h)}}return{ar\
gs:a,optArgs:i}}\
,t.parseGroupOfT\
ype=function(e,t\
,r){switch(t){ca\
se\x22color\x22:return\
 this.parseColor\
Group(r);case\x22si\
ze\x22:return this.\
parseSizeGroup(r\
);case\x22url\x22:retu\
rn this.parseUrl\
Group(r);case\x22ma\
th\x22:case\x22text\x22:r\
eturn this.parse\
ArgumentGroup(r,\
t);case\x22hbox\x22:va\
r a=this.parseAr\
gumentGroup(r,\x22t\
ext\x22);return nul\
l!=a?{type:\x22styl\
ing\x22,mode:a.mode\
,body:[a],style:\
\x22text\x22}:null;cas\
e\x22raw\x22:var i=thi\
s.parseStringGro\
up(\x22raw\x22,r);retu\
rn null!=i?{type\
:\x22raw\x22,mode:\x22tex\
t\x22,string:i.text\
}:null;case\x22prim\
itive\x22:if(r)thro\
w new n(\x22A primi\
tive argument ca\
nnot be optional\
\x22);var o=this.pa\
rseGroup(e);if(n\
ull==o)throw new\
 n(\x22Expected gro\
up as \x22+e,this.f\
etch());return o\
;case\x22original\x22:\
case null:case v\
oid 0:return thi\
s.parseArgumentG\
roup(r);default:\
throw new n(\x22Unk\
nown group type \
as \x22+e,this.fetc\
h())}},t.consume\
Spaces=function(\
){for(;\x22 \x22===thi\
s.fetch().text;)\
this.consume()},\
t.parseStringGro\
up=function(e,t)\
{var r=this.gull\
et.scanArgument(\
t);if(null==r)re\
turn null;for(va\
r n,a=\x22\x22;\x22EOF\x22!=\
=(n=this.fetch()\
).text;)a+=n.tex\
t,this.consume()\
;return this.con\
sume(),r.text=a,\
r},t.parseRegexG\
roup=function(e,\
t){for(var r,a=t\
his.fetch(),i=a,\
o=\x22\x22;\x22EOF\x22!==(r=\
this.fetch()).te\
xt&&e.test(o+r.t\
ext);)o+=(i=r).t\
ext,this.consume\
();if(\x22\x22===o)thr\
ow new n(\x22Invali\
d \x22+t+\x22: '\x22+a.te\
xt+\x22'\x22,a);return\
 a.range(i,o)},t\
.parseColorGroup\
=function(e){var\
 t=this.parseStr\
ingGroup(\x22color\x22\
,e);if(null==t)r\
eturn null;var r\
=/^(#[a-f0-9]{3}\
|#?[a-f0-9]{6}|[\
a-z]+)$/i.exec(t\
.text);if(!r)thr\
ow new n(\x22Invali\
d color: '\x22+t.te\
xt+\x22'\x22,t);var a=\
r[0];return/^[0-\
9a-f]{6}$/i.test\
(a)&&(a=\x22#\x22+a),{\
type:\x22color-toke\
n\x22,mode:this.mod\
e,color:a}},t.pa\
rseSizeGroup=fun\
ction(e){var t,r\
=!1;if(this.gull\
et.consumeSpaces\
(),!(t=e||\x22{\x22===\
this.gullet.futu\
re().text?this.p\
arseStringGroup(\
\x22size\x22,e):this.p\
arseRegexGroup(/\
^[-+]? *(?:$|\x5cd+\
|\x5cd+\x5c.\x5cd*|\x5c.\x5cd*)\
 *[a-z]{0,2} *$/\
,\x22size\x22)))return\
 null;e||0!==t.t\
ext.length||(t.t\
ext=\x220pt\x22,r=!0);\
var a=/([-+]?) *\
(\x5cd+(?:\x5c.\x5cd*)?|\x5c\
.\x5cd+) *([a-z]{2}\
)/.exec(t.text);\
if(!a)throw new \
n(\x22Invalid size:\
 '\x22+t.text+\x22'\x22,t\
);var i={number:\
+(a[1]+a[2]),uni\
t:a[3]};if(!V(i)\
)throw new n(\x22In\
valid unit: '\x22+i\
.unit+\x22'\x22,t);ret\
urn{type:\x22size\x22,\
mode:this.mode,v\
alue:i,isBlank:r\
}},t.parseUrlGro\
up=function(e){t\
his.gullet.lexer\
.setCatcode(\x22%\x22,\
13),this.gullet.\
lexer.setCatcode\
(\x22~\x22,12);var t=t\
his.parseStringG\
roup(\x22url\x22,e);if\
(this.gullet.lex\
er.setCatcode(\x22%\
\x22,14),this.gulle\
t.lexer.setCatco\
de(\x22~\x22,13),null=\
=t)return null;v\
ar r=t.text.repl\
ace(/\x5c\x5c([#$%&~_^\
{}])/g,\x22$1\x22);ret\
urn{type:\x22url\x22,m\
ode:this.mode,ur\
l:r}},t.parseArg\
umentGroup=funct\
ion(e,t){var r=t\
his.gullet.scanA\
rgument(e);if(nu\
ll==r)return nul\
l;var n=this.mod\
e;t&&this.switch\
Mode(t),this.gul\
let.beginGroup()\
;var a=this.pars\
eExpression(!1,\x22\
EOF\x22);this.expec\
t(\x22EOF\x22),this.gu\
llet.endGroup();\
var i={type:\x22ord\
group\x22,mode:this\
.mode,loc:r.loc,\
body:a};return t\
&&this.switchMod\
e(n),i},t.parseG\
roup=function(e,\
t){var r,a=this.\
fetch(),i=a.text\
;if(\x22{\x22===i||\x22\x5c\x5c\
begingroup\x22===i)\
{this.consume();\
var o=\x22{\x22===i?\x22}\
\x22:\x22\x5c\x5cendgroup\x22;t\
his.gullet.begin\
Group();var s=th\
is.parseExpressi\
on(!1,o),l=this.\
fetch();this.exp\
ect(o),this.gull\
et.endGroup(),r=\
{type:\x22ordgroup\x22\
,mode:this.mode,\
loc:Fr.range(a,l\
),body:s,semisim\
ple:\x22\x5c\x5cbegingrou\
p\x22===i||void 0}}\
else if(null==(r\
=this.parseFunct\
ion(t,e)||this.p\
arseSymbol())&&\x22\
\x5c\x5c\x22===i[0]&&!Wn.\
hasOwnProperty(i\
)){if(this.setti\
ngs.throwOnError\
)throw new n(\x22Un\
defined control \
sequence: \x22+i,a)\
;r=this.formatUn\
supportedCmd(i),\
this.consume()}r\
eturn r},t.formL\
igatures=functio\
n(e){for(var t=e\
.length-1,r=0;r<\
t;++r){var n=e[r\
],a=n.text;\x22-\x22==\
=a&&\x22-\x22===e[r+1]\
.text&&(r+1<t&&\x22\
-\x22===e[r+2].text\
?(e.splice(r,3,{\
type:\x22textord\x22,m\
ode:\x22text\x22,loc:F\
r.range(n,e[r+2]\
),text:\x22---\x22}),t\
-=2):(e.splice(r\
,2,{type:\x22textor\
d\x22,mode:\x22text\x22,l\
oc:Fr.range(n,e[\
r+1]),text:\x22--\x22}\
),t-=1)),\x22'\x22!==a\
&&\x22`\x22!==a||e[r+1\
].text!==a||(e.s\
plice(r,2,{type:\
\x22textord\x22,mode:\x22\
text\x22,loc:Fr.ran\
ge(n,e[r+1]),tex\
t:a+a}),t-=1)}},\
t.parseSymbol=fu\
nction(){var e=t\
his.fetch(),t=e.\
text;if(/^\x5c\x5cverb\
[^a-zA-Z]/.test(\
t)){this.consume\
();var r=t.slice\
(5),a=\x22*\x22===r.ch\
arAt(0);if(a&&(r\
=r.slice(1)),r.l\
ength<2||r.charA\
t(0)!==r.slice(-\
1))throw new n(\x22\
\x5c\x5cverb assertion\
 failed --\x5cn    \
                \
please report wh\
at input caused \
this bug\x22);retur\
n{type:\x22verb\x22,mo\
de:\x22text\x22,body:r\
=r.slice(1,-1),s\
tar:a}}Kn.hasOwn\
Property(t[0])&&\
!ae[this.mode][t\
[0]]&&(this.sett\
ings.strict&&\x22ma\
th\x22===this.mode&\
&this.settings.r\
eportNonstrict(\x22\
unicodeTextInMat\
hMode\x22,'Accented\
 Unicode text ch\
aracter \x22'+t[0]+\
'\x22 used in math \
mode',e),t=Kn[t[\
0]]+t.slice(1));\
var i,o=On.exec(\
t);if(o&&(\x22i\x22===\
(t=t.substring(0\
,o.index))?t=\x22\x5cu\
0131\x22:\x22j\x22===t&&(\
t=\x22\x5cu0237\x22)),ae[\
this.mode][t]){t\
his.settings.str\
ict&&\x22math\x22===th\
is.mode&&Ee.inde\
xOf(t)>=0&&this.\
settings.reportN\
onstrict(\x22unicod\
eTextInMathMode\x22\
,'Latin-1/Unicod\
e text character\
 \x22'+t[0]+'\x22 used\
 in math mode',e\
);var s,l=ae[thi\
s.mode][t].group\
,h=Fr.range(e);i\
f(te.hasOwnPrope\
rty(l)){var c=l;\
s={type:\x22atom\x22,m\
ode:this.mode,fa\
mily:c,loc:h,tex\
t:t}}else s={typ\
e:l,mode:this.mo\
de,loc:h,text:t}\
;i=s}else{if(!(t\
.charCodeAt(0)>=\
128))return null\
;this.settings.s\
trict&&(S(t.char\
CodeAt(0))?\x22math\
\x22===this.mode&&t\
his.settings.rep\
ortNonstrict(\x22un\
icodeTextInMathM\
ode\x22,'Unicode te\
xt character \x22'+\
t[0]+'\x22 used in \
math mode',e):th\
is.settings.repo\
rtNonstrict(\x22unk\
nownSymbol\x22,'Unr\
ecognized Unicod\
e character \x22'+t\
[0]+'\x22 ('+t.char\
CodeAt(0)+\x22)\x22,e)\
),i={type:\x22texto\
rd\x22,mode:\x22text\x22,\
loc:Fr.range(e),\
text:t}}if(this.\
consume(),o)for(\
var m=0;m<o[0].l\
ength;m++){var u\
=o[0][m];if(!Zn[\
u])throw new n(\x22\
Unknown accent '\
 \x22+u+\x22'\x22,e);var \
p=Zn[u][this.mod\
e]||Zn[u].text;i\
f(!p)throw new n\
(\x22Accent \x22+u+\x22 u\
nsupported in \x22+\
this.mode+\x22 mode\
\x22,e);i={type:\x22ac\
cent\x22,mode:this.\
mode,loc:Fr.rang\
e(e),label:p,isS\
tretchy:!1,isShi\
fty:!0,base:i}}r\
eturn i},e}();Jn\
.endOfExpression\
=[\x22}\x22,\x22\x5c\x5cendgrou\
p\x22,\x22\x5c\x5cend\x22,\x22\x5c\x5cri\
ght\x22,\x22&\x22];var Qn\
=function(e,t){i\
f(!(\x22string\x22==ty\
peof e||e instan\
ceof String))thr\
ow new TypeError\
(\x22KaTeX can only\
 parse string ty\
ped expression\x22)\
;var r=new Jn(e,\
t);delete r.gull\
et.macros.curren\
t[\x22\x5c\x5cdf@tag\x22];va\
r a=r.parse();if\
(delete r.gullet\
.macros.current[\
\x22\x5c\x5ccurrent@color\
\x22],delete r.gull\
et.macros.curren\
t[\x22\x5c\x5ccolor\x22],r.g\
ullet.macros.get\
(\x22\x5c\x5cdf@tag\x22)){if\
(!t.displayMode)\
throw new n(\x22\x5c\x5ct\
ag works only in\
 display equatio\
ns\x22);a=[{type:\x22t\
ag\x22,mode:\x22text\x22,\
body:a,tag:r.sub\
parse([new Gr(\x22\x5c\
\x5cdf@tag\x22)])}]}re\
turn a},ea=funct\
ion(e,t,r){t.tex\
tContent=\x22\x22;var \
n=ra(e,r).toNode\
();t.appendChild\
(n)};\x22undefined\x22\
!=typeof documen\
t&&\x22CSS1Compat\x22!\
==document.compa\
tMode&&(\x22undefin\
ed\x22!=typeof cons\
ole&&console.war\
n(\x22Warning: KaTe\
X doesn't work i\
n quirks mode. M\
ake sure your we\
bsite has a suit\
able doctype.\x22),\
ea=function(){th\
row new n(\x22KaTeX\
 doesn't work in\
 quirks mode.\x22)}\
);var ta=functio\
n(e,t,r){if(r.th\
rowOnError||!(e \
instanceof n))th\
row e;var a=Ke.m\
akeSpan([\x22katex-\
error\x22],[new Z(t\
)]);return a.set\
Attribute(\x22title\
\x22,e.toString()),\
a.setAttribute(\x22\
style\x22,\x22color:\x22+\
r.errorColor),a}\
,ra=function(e,t\
){var r=new m(t)\
;try{var n=Qn(e,\
r);return Lt(n,e\
,r)}catch(t){ret\
urn ta(t,e,r)}},\
na={version:\x220.1\
6.7\x22,render:ea,r\
enderToString:fu\
nction(e,t){retu\
rn ra(e,t).toMar\
kup()},ParseErro\
r:n,SETTINGS_SCH\
EMA:h,__parse:fu\
nction(e,t){var \
r=new m(t);retur\
n Qn(e,r)},__ren\
derToDomTree:ra,\
__renderToHTMLTr\
ee:function(e,t)\
{var r=new m(t);\
try{return funct\
ion(e,t,r){var n\
=St(e,Ot(r)),a=K\
e.makeSpan([\x22kat\
ex\x22],[n]);return\
 Et(a,r)}(Qn(e,r\
),0,r)}catch(t){\
return ta(t,e,r)\
}},__setFontMetr\
ics:function(e,t\
){T[e]=t},__defi\
neSymbol:ie,__de\
fineFunction:ot,\
__defineMacro:Pr\
,__domTree:{Span\
:W,Anchor:_,Symb\
olNode:Z,SvgNode\
:K,PathNode:J,Li\
neNode:Q}};retur\
n t=t.default}()\
}));\
\x00\x03\xfe,\
/\
/ v2.6.41 | 2025\
-09-11\x0a//\x0a// PyG\
PT Runtime for s\
treaming Markdow\
n in QWebEngine:\
\x0a//\x0a// - QWebCha\
nnel bridge\x0a// -\
 markdown-it wit\
h lightweight Ka\
TeX placeholders\
 (no KaTeX compu\
te on hot path)\x0a\
// - Dollar-deli\
miter math place\
holders: $...$ a\
nd $$...$$ (fina\
lize/idle/always\
 policy)\x0a// - No\
 inline hljs com\
pute in renderer\
 (final code hig\
hlight is deferr\
ed)\x0a// - Lightwe\
ight, scroll+rAF\
 driven hljs hig\
hlighting (no In\
tersectionObserv\
er)\x0a// - Async, \
chunked KaTeX re\
ndering to avoid\
 UI stalls\x0a// - \
Async, scheduled\
 tail promotion \
(small chunks, U\
I-friendly)\x0a// -\
 Adaptive snapsh\
ots, minimal DOM\
 churn\x0a// - Incr\
emental, non-fli\
ckering code str\
eaming (frozen p\
refix + plain ta\
il)\x0a// - Unified\
, bridge-based l\
ogger injected t\
o all subsystems\
 (no console)\x0a//\
 - Buffered logg\
ing with auto-fl\
ush when QWebCha\
nnel is ready\x0a//\
 - Runtime promo\
tion of language\
 from first-line\
 directive durin\
g streaming\x0a// -\
 Stream-aware re\
nderer (no linki\
fy on hot path) \
and plain-text s\
treaming fallbac\
k for huge code\x0a\
// - Final highl\
ight hard caps t\
o avoid OOM on g\
igantic blocks\x0a/\
/ - Precomputed \
code meta (len/h\
ead/tail and nl \
for full rendere\
r) to avoid heav\
y .textContent r\
eads\x0a\x0a(function \
() {\x0a  'use stri\
ct';\x0a\x0a  // =====\
================\
================\
================\
================\
=====\x0a  // 0) Ut\
ils & Config\x0a  /\
/ ==============\
================\
================\
================\
============\x0a\x0a  \
// Small helper \
utilities used a\
cross the runtim\
e.\x0a  class Utils\
 {\x0a    static g(\
name, dflt) { re\
turn (typeof win\
dow[name] !== 'u\
ndefined') ? win\
dow[name] : dflt\
; }\x0a    static n\
ow() { return (t\
ypeof performanc\
e !== 'undefined\
' && performance\
.now) ? performa\
nce.now() : Date\
.now(); }\x0a\x0a    /\
/ reuse a single\
 detached elemen\
t to reduce allo\
cations on hot p\
ath.\x0a    static \
escapeHtml(s) {\x0a\
      const d = \
Utils._escDiv ||\
 (Utils._escDiv \
= document.creat\
eElement('div'))\
;\x0a      d.textCo\
ntent = String(s\
 ?? '');\x0a      r\
eturn d.innerHTM\
L;\x0a    }\x0a\x0a    st\
atic countNewlin\
es(s) {\x0a      if\
 (!s) return 0;\x0a\
      let c = 0,\
 i = -1; while (\
(i = s.indexOf('\
\x5cn', i + 1)) !==\
 -1) c++;\x0a      \
return c;\x0a    }\x0a\
    static reEsc\
ape(s) { return \
String(s).replac\
e(/[.*+?^${}()|[\
\x5c]\x5c\x5c]/g, '\x5c\x5c$&')\
; }\x0a    // Sched\
ule a function i\
n idle time (fal\
ls back to setTi\
meout).\x0a    stat\
ic idle(fn, time\
out) {\x0a      if \
('requestIdleCal\
lback' in window\
) return request\
IdleCallback(fn,\
 { timeout: time\
out || 800 });\x0a \
     return setT\
imeout(fn, 50);\x0a\
    }\x0a    // Can\
cel idle callbac\
k if possible (s\
afe for fallback\
).\x0a    static ca\
ncelIdle(id) {\x0a \
     try {\x0a     \
   if ('cancelId\
leCallback' in w\
indow) cancelIdl\
eCallback(id);\x0a \
       else clea\
rTimeout(id);\x0a  \
    } catch (_) \
{}\x0a    }\x0a    sta\
tic get SE() { r\
eturn document.s\
crollingElement \
|| document.docu\
mentElement; }\x0a\x0a\
    // shared UT\
F-8 decoder to a\
void per-call al\
locations.\x0a    s\
tatic utf8Decode\
(bytes) {\x0a      \
if (!Utils._td) \
Utils._td = new \
TextDecoder('utf\
-8');\x0a      retu\
rn Utils._td.dec\
ode(bytes);\x0a    \
}\x0a  }\x0a\x0a  // ====\
================\
================\
================\
================\
======\x0a  // 0.1)\
 Unified Logger \
(bridge-based; i\
njected into cla\
sses)\x0a  // =====\
================\
================\
================\
================\
=====\x0a\x0a  class L\
ogger {\x0a    // P\
roduction-grade \
logger with queu\
e, batch sending\
 and soft caps.\x0a\
    constructor(\
cfg) {\x0a      thi\
s.cfg = cfg || {\
 LOG: {} };\x0a    \
  // Queue holds\
 strings waiting\
 to be sent over\
 QWebChannel to \
Python.\x0a      th\
is.queue = [];\x0a \
     this.queueB\
ytes = 0; // app\
roximate UTF-16 \
bytes (chars * 2\
)\x0a      this.arm\
ed = false;\x0a    \
  this.maxTries \
= 480; // ~8s @6\
0fps\x0a      this.\
tries = 0;\x0a     \
 this.bridge = n\
ull;\x0a      this.\
_idleId = 0;    \
          // idl\
e-callback handl\
e (backoff-frien\
dly)\x0a      this.\
_lastFlushQueueB\
ytes = 0; // las\
t known bytes to\
 detect progress\
\x0a\x0a      // Centr\
alized rAF manag\
er handle (bound\
 by runtime); fa\
lls back to micr\
otasks until ava\
ilable.\x0a      th\
is.raf = null;\x0a \
     this._rafSc\
heduled = false;\
\x0a      this._raf\
Key = { t: 'Logg\
er:tick' };\x0a\x0a   \
   // Soft limit\
s; tunable from \
window.* if desi\
red.\x0a      const\
 L = this.cfg.LO\
G || {};\x0a      t\
his.MAX_QUEUE = \
Utils.g('LOG_MAX\
_QUEUE', L.MAX_Q\
UEUE ?? 400);\x0a  \
    this.MAX_BYT\
ES = Utils.g('LO\
G_MAX_BYTES', L.\
MAX_BYTES ?? 256\
 * 1024); // 256\
 KiB\x0a      this.\
BATCH_MAX = Util\
s.g('LOG_BATCH_M\
AX', L.BATCH_MAX\
 ?? 64);\x0a      t\
his.RATE_LIMIT_P\
ER_SEC = Utils.g\
('LOG_RATE_LIMIT\
_PER_SEC', L.RAT\
E_LIMIT_PER_SEC \
?? 0); // 0 == o\
ff\x0a      this._r\
lWindowMs = 1000\
;\x0a      this._rl\
Count = 0;\x0a     \
 this._rlWindowS\
tart = Utils.now\
();\x0a    }\x0a    //\
 Connect a QWebC\
hannel bridge an\
d flush any pend\
ing messages.\x0a  \
  bindBridge(bri\
dge) {\x0a      thi\
s.bridge = bridg\
e || null;\x0a     \
 // When bridge \
arrives, flush p\
ending messages \
respecting caps.\
\x0a      this.flus\
h();\x0a    }\x0a    /\
/ Bind RafManage\
r instance \xe2\x80\x93 e\
nsures no direct\
 requestAnimatio\
nFrame usage in \
Logger.\x0a    bind\
Raf(raf) {\x0a     \
 this.raf = raf \
|| null;\x0a    }\x0a \
   // Check if d\
ebug logging is \
enabled for a na\
mespace.\x0a    isE\
nabled(ns) {\x0a   \
   if (!ns) retu\
rn !!(window.STR\
EAM_DEBUG || win\
dow.MD_LANG_DEBU\
G);\x0a      const \
key1 = ns + '_DE\
BUG';\x0a      cons\
t key2 = ns.toUp\
perCase() + '_DE\
BUG';\x0a      retu\
rn !!(window[key\
1] || window[key\
2] || window.STR\
EAM_DEBUG || win\
dow.MD_LANG_DEBU\
G);\x0a    }\x0a    //\
 Pretty-view str\
ing (safe trunca\
tion for logs).\x0a\
    pv(s, n = 12\
0) {\x0a      if (!\
s) return '';\x0a  \
    s = String(s\
);\x0a      if (s.l\
ength <= n) retu\
rn s.replace(/\x5cn\
/g, '\x5c\x5cn');\x0a    \
  const k = Math\
.floor(n / 2);\x0a \
     return (s.s\
lice(0, k) + ' \xe2\
\x80\xa6 ' + s.slice(-\
k)).replace(/\x5cn/\
g, '\x5c\x5cn');\x0a    }\
\x0a    j(o) { try \
{ return JSON.st\
ringify(o); } ca\
tch (_) { try { \
return String(o)\
; } catch(__){ r\
eturn '[unserial\
izable]'; } } }\x0a\
    _emit(msg) {\
\x0a      // Attemp\
t batch-capable \
sink if provided\
 by the bridge\x0a \
     try {\x0a     \
   if (this.brid\
ge) {\x0a          \
if (typeof this.\
bridge.log_batch\
 === 'function')\
 { this.bridge.l\
og_batch([String\
(msg)]); return \
true; }\x0a        \
  if (typeof thi\
s.bridge.logBatc\
h === 'function'\
) { this.bridge.\
logBatch([String\
(msg)]); return \
true; }\x0a        \
  if (typeof thi\
s.bridge.log ===\
 'function') { t\
his.bridge.log(S\
tring(msg)); ret\
urn true; }\x0a    \
    }\x0a        if\
 (window.runtime\
 && runtime.brid\
ge && typeof run\
time.bridge.log \
=== 'function') \
{\x0a          runt\
ime.bridge.log(S\
tring(msg)); ret\
urn true;\x0a      \
  }\x0a      } catc\
h (_) {}\x0a      r\
eturn false;\x0a   \
 }\x0a    _emitBatc\
h(arr) {\x0a      t\
ry {\x0a        if \
(!arr || !arr.le\
ngth) return 0;\x0a\
        // Prefe\
r batch API if p\
resent; otherwis\
e fallback to pe\
r-line\x0a        i\
f (this.bridge &\
& typeof this.br\
idge.log_batch =\
== 'function') {\
 this.bridge.log\
_batch(arr.map(S\
tring)); return \
arr.length; }\x0a  \
      if (this.b\
ridge && typeof \
this.bridge.logB\
atch === 'functi\
on') { this.brid\
ge.logBatch(arr.\
map(String)); re\
turn arr.length;\
 }\x0a        if (t\
his.bridge && ty\
peof this.bridge\
.log === 'functi\
on') {\x0a         \
 for (let i = 0;\
 i < arr.length;\
 i++) this.bridg\
e.log(String(arr\
[i]));\x0a         \
 return arr.leng\
th;\x0a        }\x0a  \
    } catch (_) \
{}\x0a      return \
0;\x0a    }\x0a    _ma\
ybeDropForCaps(l\
en) {\x0a      // H\
ard cap queue si\
ze and memory to\
 guard Python pr\
ocess via QWebCh\
annel.\x0a      if \
(this.queue.leng\
th <= this.MAX_Q\
UEUE && this.que\
ueBytes <= this.\
MAX_BYTES) retur\
n;\x0a      const t\
argetLen = Math.\
floor(this.MAX_Q\
UEUE * 0.8);\x0a   \
   const targetB\
ytes = Math.floo\
r(this.MAX_BYTES\
 * 0.8);\x0a      /\
/ Drop oldest fi\
rst; keep newest\
 events\x0a      wh\
ile ((this.queue\
.length > target\
Len || this.queu\
eBytes > targetB\
ytes) && this.qu\
eue.length) {\x0a  \
      const remo\
ved = this.queue\
.shift();\x0a      \
  this.queueByte\
s -= (removed ? \
removed.length *\
 2 : 0);\x0a      }\
\x0a      // Add on\
e synthetic noti\
ce to indicate d\
ropped logs (avo\
id recursion)\x0a  \
    const notice\
 = '[LOGGER] que\
ue trimmed due t\
o caps';\x0a      t\
his.queue.unshif\
t(notice);\x0a     \
 this.queueBytes\
 += notice.lengt\
h * 2;\x0a    }\x0a   \
 _passRateLimit(\
) {\x0a      if (!t\
his.RATE_LIMIT_P\
ER_SEC || this.R\
ATE_LIMIT_PER_SE\
C <= 0) return t\
rue;\x0a      const\
 now = Utils.now\
();\x0a      if (no\
w - this._rlWind\
owStart > this._\
rlWindowMs) {\x0a  \
      this._rlWi\
ndowStart = now;\
\x0a        this._r\
lCount = 0;\x0a    \
  }\x0a      if (th\
is._rlCount >= t\
his.RATE_LIMIT_P\
ER_SEC) return f\
alse;\x0a      retu\
rn true;\x0a    }\x0a \
   // Push a log\
 line; try immed\
iate send, other\
wise enqueue.\x0a  \
  log(text) {\x0a  \
    const msg = \
String(text);\x0a  \
    // If bridge\
 is available, t\
ry immediate emi\
t to avoid queue\
ing overhead.\x0a  \
    if (this.bri\
dge && (typeof t\
his.bridge.log =\
== 'function' ||\
 typeof this.bri\
dge.log_batch ==\
= 'function' || \
typeof this.brid\
ge.logBatch === \
'function')) {\x0a \
       if (this.\
_passRateLimit()\
) {\x0a          co\
nst ok = this._e\
mit(msg);\x0a      \
    if (ok) retu\
rn true;\x0a       \
 }\x0a      }\x0a     \
 // Enqueue with\
 caps; avoid unb\
ounded growth\x0a  \
    this.queue.p\
ush(msg);\x0a      \
this.queueBytes \
+= msg.length * \
2;\x0a      this._m\
aybeDropForCaps(\
msg.length);\x0a   \
   this._arm();\x0a\
      return fal\
se;\x0a    }\x0a    //\
 Debug wrapper r\
especting per-na\
mespace switch.\x0a\
    debug(ns, li\
ne, ctx) {\x0a     \
 if (!this.isEna\
bled(ns)) return\
 false;\x0a      le\
t msg = `[${ns}]\
 ${line || ''}`;\
\x0a      if (typeo\
f ctx !== 'undef\
ined') msg += ' \
' + this.j(ctx);\
\x0a      return th\
is.log(msg);\x0a   \
 }\x0a    // Send a\
 batch of lines \
if bridge is rea\
dy.\x0a    flush(ma\
xPerTick = this.\
BATCH_MAX) {\x0a   \
   // Centralize\
d flush \xe2\x80\x93 no d\
irect cancelAnim\
ationFrame here;\
 RafManager gove\
rns frames.\x0a    \
  if (!this.brid\
ge && !(window.r\
untime && runtim\
e.bridge && type\
of runtime.bridg\
e.log === 'funct\
ion')) return 0;\
\x0a      const n =\
 Math.min(maxPer\
Tick, this.queue\
.length);\x0a      \
if (!n) return 0\
;\x0a      const ba\
tch = this.queue\
.splice(0, n);\x0a \
     // Update m\
emory accounting\
 before attempti\
ng emit to eager\
ly unlock memory\
\x0a      let bytes\
 = 0; for (let i\
 = 0; i < batch.\
length; i++) byt\
es += batch[i].l\
ength * 2;\x0a     \
 this.queueBytes\
 = Math.max(0, t\
his.queueBytes -\
 bytes);\x0a      c\
onst sent = this\
._emitBatch(batc\
h);\x0a\x0a      // fi\
x byte accountin\
g for partial se\
nds (re-queue re\
maining with acc\
urate bytes).\x0a  \
    if (sent < b\
atch.length) {\x0a \
       const rem\
ain = batch.slic\
e(sent);\x0a       \
 let remBytes = \
0; for (let i = \
0; i < remain.le\
ngth; i++) remBy\
tes += remain[i]\
.length * 2;\x0a   \
     // Prepend \
remaining back t\
o the queue pres\
erving order\x0a   \
     for (let i \
= remain.length \
- 1; i >= 0; i--\
) this.queue.uns\
hift(remain[i]);\
\x0a        this.qu\
eueBytes += remB\
ytes;\x0a      }\x0a  \
    return sent;\
\x0a    }\x0a    _sche\
duleTick(tick) {\
\x0a      // Prefer\
 idle scheduling\
 when bridge isn\
't available yet\
 \xe2\x80\x93 avoids 60fp\
s RAF churn.\x0a   \
   const preferI\
dle = !this.brid\
ge;\x0a\x0a      const\
 scheduleIdle = \
() => {\x0a        \
try { if (this._\
idleId) Utils.ca\
ncelIdle(this._i\
dleId); } catch \
(_) {}\x0a        /\
/ Use requestIdl\
eCallback when a\
vailable, fallba\
ck to small time\
out. Keeps UI co\
ld on idle.\x0a    \
    this._idleId\
 = Utils.idle(()\
 => { this._idle\
Id = 0; tick(); \
}, 800);\x0a      }\
;\x0a\x0a      if (pre\
ferIdle) { sched\
uleIdle(); retur\
n; }\x0a\x0a      if (\
this._rafSchedul\
ed) return;\x0a    \
  this._rafSched\
uled = true;\x0a   \
   const run = (\
) => { this._raf\
Scheduled = fals\
e; tick(); };\x0a  \
    try {\x0a      \
  if (this.raf &\
& typeof this.ra\
f.schedule === '\
function') {\x0a   \
       this.raf.\
schedule(this._r\
afKey, run, 'Log\
ger', 3);\x0a      \
  } else if (typ\
eof runtime !== \
'undefined' && r\
untime.raf && ty\
peof runtime.raf\
.schedule === 'f\
unction') {\x0a    \
      runtime.ra\
f.schedule(this.\
_rafKey, run, 'L\
ogger', 3);\x0a    \
    } else {\x0a   \
       Promise.r\
esolve().then(ru\
n);\x0a        }\x0a  \
    } catch (_) \
{\x0a        Promis\
e.resolve().then\
(run);\x0a      }\x0a \
   }\x0a    _arm() \
{\x0a      if (this\
.armed) return;\x0a\
      this.armed\
 = true; this.tr\
ies = 0;\x0a      c\
onst tick = () =\
> {\x0a        if (\
!this.armed) ret\
urn;\x0a        thi\
s.flush();\x0a     \
   this.tries++;\
\x0a        if (thi\
s.queue.length =\
== 0 || this.tri\
es > this.maxTri\
es) {\x0a          \
this.armed = fal\
se;\x0a          tr\
y { if (this._id\
leId) Utils.canc\
elIdle(this._idl\
eId); } catch (_\
) {}\x0a          t\
his._idleId = 0;\
\x0a          retur\
n;\x0a        }\x0a   \
     this._sched\
uleTick(tick);\x0a \
     };\x0a      th\
is._scheduleTick\
(tick);\x0a    }\x0a  \
}\x0a\x0a  // ========\
================\
================\
================\
================\
==\x0a  // RafManag\
er (rAF-only)\x0a  \
// =============\
================\
================\
================\
=============\x0a\x0a \
 class RafManage\
r {\x0a    // rAF-o\
nly task pump wi\
th soft budget p\
er flush to prev\
ent long frames.\
\x0a    constructor\
(cfg) {\x0a      th\
is.cfg = cfg || \
{ RAF: {}, ASYNC\
: {} };\x0a      th\
is.tasks = new M\
ap();\x0a      this\
.groups = new Ma\
p();\x0a      // rA\
F pump state\x0a   \
   this.tickId =\
 0;\x0a      this._\
mode = 'raf';\x0a  \
    this.schedul\
ed = false;\x0a    \
  this._flushInP\
rogress = false;\
\x0a\x0a      const R \
= (this.cfg && t\
his.cfg.RAF) || \
{};\x0a      this.F\
LUSH_BUDGET_MS =\
 Utils.g('RAF_FL\
USH_BUDGET_MS', \
R.FLUSH_BUDGET_M\
S ?? 7);\x0a      t\
his.MAX_TASKS_PE\
R_FLUSH = Utils.\
g('RAF_MAX_TASKS\
_PER_FLUSH', R.M\
AX_TASKS_PER_FLU\
SH ?? 120);\x0a    \
}\x0a    // Start t\
he pumping loop \
if needed.\x0a    _\
armPump() {\x0a    \
  if (this.sched\
uled) return;\x0a  \
    this.schedul\
ed = true;\x0a\x0a    \
  // Focus/visib\
ility-agnostic, \
rAF-only schedul\
ing (no timers, \
no watchdogs).\x0a \
     const canRA\
F = typeof reque\
stAnimationFrame\
 === 'function';\
\x0a      if (canRA\
F) {\x0a        thi\
s._mode = 'raf';\
\x0a        try {\x0a \
         this.ti\
ckId = requestAn\
imationFrame(() \
=> this.flush())\
;\x0a          retu\
rn;\x0a        } ca\
tch (_) {}\x0a     \
 }\x0a      // Fall\
back without tim\
ers: schedule a \
microtask flush.\
\x0a      this._mod\
e = 'raf';\x0a     \
 Promise.resolve\
().then(() => th\
is.flush());\x0a   \
 }\x0a    // Schedu\
le a function wi\
th an optional g\
roup and priorit\
y.\x0a    schedule(\
key, fn, group =\
 'default', prio\
rity = 0) {\x0a    \
  if (!key) key \
= { k: 'anon' };\
\x0a      this.task\
s.set(key, { fn,\
 group, priority\
 });\x0a      if (g\
roup) {\x0a        \
let set = this.g\
roups.get(group)\
;\x0a        if (!s\
et) { set = new \
Set(); this.grou\
ps.set(group, se\
t); }\x0a        se\
t.add(key);\x0a    \
  }\x0a      this._\
armPump();\x0a    }\
\x0a    // Run pend\
ing tasks within\
 a time and coun\
t budget.\x0a    fl\
ush() {\x0a      //\
 Cancel rAF hand\
le if any; nothi\
ng else (timers \
removed).\x0a      \
try { if (this.t\
ickId) cancelAni\
mationFrame(this\
.tickId); } catc\
h (_) {}\x0a      t\
his.tickId = 0;\x0a\
      this.sched\
uled = false;\x0a\x0a \
     // Snapshot\
 tasks and clear\
 the map; re-que\
ue leftovers if \
we exceed budget\
.\x0a      const li\
st = Array.from(\
this.tasks.entri\
es()).map(([key,\
 v]) => ({ key, \
...v }));\x0a      \
this.tasks.clear\
();\x0a      list.s\
ort((a, b) => a.\
priority - b.pri\
ority);\x0a\x0a      c\
onst start = Uti\
ls.now();\x0a      \
let processed = \
0;\x0a\x0a      for (l\
et idx = 0; idx \
< list.length; i\
dx++) {\x0a        \
const t = list[i\
dx];\x0a        try\
 { t.fn(); } cat\
ch (_) {}\x0a      \
  processed++;\x0a \
       if (t.gro\
up) {\x0a          \
const set = this\
.groups.get(t.gr\
oup);\x0a          \
if (set) { set.d\
elete(t.key); if\
 (set.size === 0\
) this.groups.de\
lete(t.group); }\
\x0a        }\x0a\x0a    \
    // Soft budg\
et: if long, re-\
queue the rest f\
or next rAF.\x0a   \
     const elaps\
ed = Utils.now()\
 - start;\x0a      \
  if (processed \
>= this.MAX_TASK\
S_PER_FLUSH || e\
lapsed >= this.F\
LUSH_BUDGET_MS) \
{\x0a          for \
(let j = idx + 1\
; j < list.lengt\
h; j++) {\x0a      \
      const r = \
list[j];\x0a       \
     this.tasks.\
set(r.key, { fn:\
 r.fn, group: r.\
group, priority:\
 r.priority });\x0a\
          }\x0a    \
      this._armP\
ump();\x0a         \
 return;\x0a       \
 }\x0a      }\x0a\x0a    \
  if (this.tasks\
.size) {\x0a       \
 this._armPump()\
;\x0a      }\x0a    }\x0a\
    // Force imm\
ediate flush or \
schedule next fr\
ame.\x0a    kick(fo\
rceImmediate = t\
rue) {\x0a      if \
(forceImmediate \
&& this.tasks.si\
ze) {\x0a        if\
 (this._flushInP\
rogress) return;\
\x0a        this._f\
lushInProgress =\
 true;\x0a        t\
ry { this.schedu\
led = true; this\
.flush(); } catc\
h (_) {} finally\
 { this._flushIn\
Progress = false\
; }\x0a        retu\
rn;\x0a      }\x0a    \
  this._armPump(\
);\x0a    }\x0a    // \
Cancel a specifi\
c scheduled task\
 by key.\x0a    can\
cel(key) {\x0a     \
 const t = this.\
tasks.get(key);\x0a\
      if (!t) re\
turn;\x0a      this\
.tasks.delete(ke\
y);\x0a      if (t.\
group) {\x0a       \
 const set = thi\
s.groups.get(t.g\
roup);\x0a        i\
f (set) { set.de\
lete(key); if (s\
et.size === 0) t\
his.groups.delet\
e(t.group); }\x0a  \
    }\x0a    }\x0a    \
// Cancel all ta\
sks in a group.\x0a\
    cancelGroup(\
group) {\x0a      c\
onst set = this.\
groups.get(group\
);\x0a      if (!se\
t) return;\x0a     \
 for (const key \
of set) this.tas\
ks.delete(key);\x0a\
      this.group\
s.delete(group);\
\x0a    }\x0a    // Ca\
ncel everything \
and reset the pu\
mp.\x0a    cancelAl\
l() {\x0a      this\
.tasks.clear();\x0a\
      this.group\
s.clear();\x0a     \
 try { if (this.\
tickId) cancelAn\
imationFrame(thi\
s.tickId); } cat\
ch (_) {}\x0a      \
this.tickId = 0;\
\x0a      this.sche\
duled = false;\x0a \
   }\x0a    isSched\
uled(key) { retu\
rn this.tasks.ha\
s(key); }\x0a\x0a    /\
/ Awaitable \x22nex\
t frame\x22 helper \
\xe2\x80\x93 resolves on \
next flush.\x0a    \
nextFrame() {\x0a  \
    return new P\
romise((resolve)\
 => {\x0a        co\
nst key = { t: '\
raf:nextFrame', \
i: Math.random()\
 };\x0a        this\
.schedule(key, (\
) => resolve(), \
'RafNext', 0);\x0a \
     });\x0a    }\x0a \
 }\x0a\x0a  // Return \
math rendering p\
olicy from windo\
w.MATH_STREAM_MO\
DE.\x0a  function g\
etMathMode() {\x0a \
   const v = Str\
ing(window.MATH_\
STREAM_MODE || '\
finalize-only').\
toLowerCase();\x0a \
   return (v ===\
 'idle' || v ===\
 'always' || v =\
== 'finalize-onl\
y') ? v : 'final\
ize-only';\x0a  }\x0a\x0a\
  // ===========\
================\
================\
================\
===============\x0a\
  // 0.2) Async \
runner (cooperat\
ive yielding for\
 heavy tasks)\x0a  \
// =============\
================\
================\
================\
=============\x0a\x0a \
 class AsyncRunn\
er {\x0a    // Coop\
erative schedule\
r (rAF-based) fo\
r CPU-heavy task\
s (keeps UI resp\
onsive).\x0a    con\
structor(cfg, ra\
f) {\x0a      this.\
cfg = cfg || {};\
\x0a      this.raf \
= raf || null;\x0a \
     const A = t\
his.cfg.ASYNC ||\
 {};\x0a      this.\
SLICE_MS = Utils\
.g('ASYNC_SLICE_\
MS', A.SLICE_MS \
?? 12);\x0a      th\
is.MIN_YIELD_MS \
= Utils.g('ASYNC\
_MIN_YIELD_MS', \
A.MIN_YIELD_MS ?\
? 0); // kept on\
ly for config co\
mpatibility\x0a    \
}\x0a    // Check w\
hether we should\
 yield control t\
o the browser.\x0a \
   _inputPending\
() {\x0a      try {\
\x0a        const s\
 = navigator && \
navigator.schedu\
ling;\x0a        re\
turn !!(s && s.i\
sInputPending &&\
 s.isInputPendin\
g({ includeConti\
nuous: true }));\
\x0a      } catch (\
_) { return fals\
e; }\x0a    }\x0a    s\
houldYield(start\
Ts) {\x0a      if (\
this._inputPendi\
ng()) return tru\
e;\x0a      return \
(Utils.now() - s\
tartTs) >= this.\
SLICE_MS;\x0a    }\x0a\
    // Yield coo\
peratively via n\
ext rAF frame (w\
ith a safe fallb\
ack).\x0a    async \
yield() {\x0a      \
// Centralized r\
AF yield; fallba\
ck to a tiny tim\
er if raf manage\
r is not ready y\
et.\x0a      if (th\
is.raf && typeof\
 this.raf.nextFr\
ame === 'functio\
n') {\x0a        aw\
ait this.raf.nex\
tFrame();\x0a      \
  return;\x0a      \
}\x0a      if (type\
of runtime !== '\
undefined' && ru\
ntime.raf && typ\
eof runtime.raf.\
nextFrame === 'f\
unction') {\x0a    \
    await runtim\
e.raf.nextFrame(\
);\x0a        retur\
n;\x0a      }\x0a     \
 await new Promi\
se(r => setTimeo\
ut(r, 16));\x0a    \
}\x0a    // Process\
 items in small \
batches with per\
iodic yields.\x0a  \
  async forEachC\
hunk(arr, fn, la\
bel) {\x0a      if \
(!arr || !arr.le\
ngth) return;\x0a  \
    let start = \
Utils.now();\x0a   \
   for (let i = \
0; i < arr.lengt\
h; i++) {\x0a      \
  await fn(arr[i\
], i);\x0a        i\
f (this.shouldYi\
eld(start)) { aw\
ait this.yield()\
; start = Utils.\
now(); }\x0a      }\
\x0a    }\x0a  }\x0a\x0a  //\
 ===============\
================\
================\
================\
===========\x0a  //\
 Config\x0a  // ===\
================\
================\
================\
================\
=======\x0a\x0a  class\
 Config {\x0a      \
constructor() {\x0a\
        // Proce\
ss identifier (p\
assed from host)\
.\x0a        this.P\
ID = Utils.g('PI\
D', 0);\x0a\x0a       \
 // UI: scroll b\
ehavior and busy\
/zoom signalling\
 thresholds (mil\
liseconds / pixe\
ls).\x0a        thi\
s.UI = {\x0a       \
   AUTO_FOLLOW_R\
EENABLE_PX: Util\
s.g('AUTO_FOLLOW\
_REENABLE_PX', 8\
),\x0a          SCR\
OLL_NEAR_MARGIN_\
PX: Utils.g('SCR\
OLL_NEAR_MARGIN_\
PX', 450),\x0a     \
     INTERACTION\
_BUSY_MS: Utils.\
g('UI_INTERACTIO\
N_BUSY_MS', 140)\
,\x0a          ZOOM\
_BUSY_MS: Utils.\
g('UI_ZOOM_BUSY_\
MS', 300)\x0a      \
  };\x0a\x0a        //\
 FAB (floating a\
ction button) vi\
sibility and deb\
ounce.\x0a        t\
his.FAB = {\x0a    \
      SHOW_DOWN_\
THRESHOLD_PX: Ut\
ils.g('SHOW_DOWN\
_THRESHOLD_PX', \
0),\x0a          TO\
GGLE_DEBOUNCE_MS\
: Utils.g('FAB_T\
OGGLE_DEBOUNCE_M\
S', 100)\x0a       \
 };\x0a\x0a        // \
Highlighting con\
trols and per-fr\
ame budget.\x0a    \
    this.HL = {\x0a\
          PER_FR\
AME: Utils.g('HL\
_PER_FRAME', 2),\
\x0a          DISAB\
LE_ALL: Utils.g(\
'DISABLE_SYNTAX_\
HIGHLIGHT', fals\
e)\x0a        };\x0a\x0a \
       // Inters\
ection-like marg\
ins (we do our o\
wn scan, but the\
se guide budgets\
).\x0a        this.\
OBSERVER = {\x0a   \
       CODE_ROOT\
_MARGIN: Utils.g\
('CODE_ROOT_MARG\
IN', '1000px 0px\
 1000px 0px'),\x0a \
         BOX_ROO\
T_MARGIN: Utils.\
g('BOX_ROOT_MARG\
IN', '1500px 0px\
 1500px 0px'),\x0a \
         CODE_TH\
RESHOLD: [0, 0.0\
01], BOX_THRESHO\
LD: 0\x0a        };\
\x0a\x0a        // Vie\
wport scan prelo\
ad distance (in \
pixels).\x0a       \
 this.SCAN = { P\
RELOAD_PX: Utils\
.g('SCAN_PRELOAD\
_PX', 1000) };\x0a\x0a\
        // Code \
scroll behavior \
(auto-follow re-\
enable margin an\
d near-bottom ma\
rgin).\x0a        t\
his.CODE_SCROLL \
= {\x0a          AU\
TO_FOLLOW_REENAB\
LE_PX: Utils.g('\
CODE_AUTO_FOLLOW\
_REENABLE_PX', 8\
),\x0a          NEA\
R_MARGIN_PX: Uti\
ls.g('CODE_SCROL\
L_NEAR_MARGIN_PX\
', 48)\x0a        }\
;\x0a\x0a        // St\
ream (snapshot) \
budgets and queu\
e limits.\x0a      \
  this.STREAM = \
{\x0a          MAX_\
PER_FRAME: Utils\
.g('STREAM_MAX_P\
ER_FRAME', 8),\x0a \
         EMERGEN\
CY_COALESCE_LEN:\
 Utils.g('STREAM\
_EMERGENCY_COALE\
SCE_LEN', 300),\x0a\
          COALES\
CE_MODE: Utils.g\
('STREAM_COALESC\
E_MODE', 'fixed'\
),\x0a          SNA\
PSHOT_MAX_STEP: \
Utils.g('STREAM_\
SNAPSHOT_MAX_STE\
P', 8000),\x0a     \
     QUEUE_MAX_I\
TEMS: Utils.g('S\
TREAM_QUEUE_MAX_\
ITEMS', 1200),\x0a \
         PRESERV\
E_CODES_MAX: Uti\
ls.g('STREAM_PRE\
SERVE_CODES_MAX'\
, 120)\x0a        }\
;\x0a\x0a        // Ma\
th (KaTeX) idle \
batching and per\
-batch hint.\x0a   \
     this.MATH =\
 {\x0a          IDL\
E_TIMEOUT_MS: Ut\
ils.g('MATH_IDLE\
_TIMEOUT_MS', 80\
0),\x0a          BA\
TCH_HINT: Utils.\
g('MATH_BATCH_HI\
NT', 24)\x0a       \
 };\x0a\x0a        // \
Icon URLs (provi\
ded by host app)\
.\x0a        this.I\
CONS = {\x0a       \
   EXPAND: Utils\
.g('ICON_EXPAND'\
, ''), COLLAPSE:\
 Utils.g('ICON_C\
OLLAPSE', ''),\x0a \
         CODE_ME\
NU: Utils.g('ICO\
N_CODE_MENU', ''\
), CODE_COPY: Ut\
ils.g('ICON_CODE\
_COPY', ''),\x0a   \
       CODE_RUN:\
 Utils.g('ICON_C\
ODE_RUN', ''), C\
ODE_PREVIEW: Uti\
ls.g('ICON_CODE_\
PREVIEW', '')\x0a  \
      };\x0a\x0a      \
  // Localized U\
I strings.\x0a     \
   this.LOCALE =\
 {\x0a          PRE\
VIEW: Utils.g('L\
OCALE_PREVIEW', \
'Preview'),\x0a    \
      RUN: Utils\
.g('LOCALE_RUN',\
 'Run'),\x0a       \
   COLLAPSE: Uti\
ls.g('LOCALE_COL\
LAPSE', 'Collaps\
e'),\x0a          E\
XPAND: Utils.g('\
LOCALE_EXPAND', \
'Expand'),\x0a     \
     COPY: Utils\
.g('LOCALE_COPY'\
, 'Copy'),\x0a     \
     COPIED: Uti\
ls.g('LOCALE_COP\
IED', 'Copied')\x0a\
        };\x0a\x0a    \
    // Code bloc\
k styling theme \
(hljs theme key \
or custom).\x0a    \
    this.CODE_ST\
YLE = Utils.g('C\
ODE_SYNTAX_STYLE\
', 'default');\x0a\x0a\
        // Adapt\
ive snapshot pro\
file for plain t\
ext.\x0a        thi\
s.PROFILE_TEXT =\
 {\x0a          bas\
e: Utils.g('PROF\
ILE_TEXT_BASE', \
4),\x0a          gr\
owth: Utils.g('P\
ROFILE_TEXT_GROW\
TH', 1.28),\x0a    \
      minInterva\
l: Utils.g('PROF\
ILE_TEXT_MIN_INT\
ERVAL', 4),\x0a    \
      softLatenc\
y: Utils.g('PROF\
ILE_TEXT_SOFT_LA\
TENCY', 60),\x0a   \
       adaptiveS\
tep: Utils.g('PR\
OFILE_TEXT_ADAPT\
IVE_STEP', false\
)\x0a        };\x0a\x0a  \
      // Adaptiv\
e snapshot profi\
le for code (lin\
e-aware).\x0a      \
  this.PROFILE_C\
ODE = {\x0a        \
  base: 2048,\x0a  \
        growth: \
2.6,\x0a          m\
inInterval: 500,\
\x0a          softL\
atency: 1200,\x0a  \
        minLines\
ForHL: Utils.g('\
PROFILE_CODE_HL_\
N_LINE', 25),\x0a  \
        minChars\
ForHL: Utils.g('\
PROFILE_CODE_HL_\
N_CHARS', 5000),\
\x0a          promo\
teMinInterval: 3\
00, promoteMaxLa\
tency: 800, prom\
oteMinLines: Uti\
ls.g('PROFILE_CO\
DE_HL_N_LINE', 2\
5),\x0a          ad\
aptiveStep: Util\
s.g('PROFILE_COD\
E_ADAPTIVE_STEP'\
, true),\x0a       \
   stopAfterLine\
s: Utils.g('PROF\
ILE_CODE_STOP_HL\
_AFTER_LINES', 3\
00),\x0a          s\
treamPlainAfterL\
ines: 0,\x0a       \
   streamPlainAf\
terChars: 0,\x0a   \
       maxFrozen\
Chars: 32000,\x0a  \
        finalHig\
hlightMaxLines: \
Utils.g('PROFILE\
_CODE_FINAL_HL_M\
AX_LINES', 1500)\
,\x0a          fina\
lHighlightMaxCha\
rs: Utils.g('PRO\
FILE_CODE_FINAL_\
HL_MAX_CHARS', 3\
50000)\x0a        }\
;\x0a\x0a        // De\
bounce for heavy\
 resets (ms).\x0a  \
      this.RESET\
 = {\x0a          H\
EAVY_DEBOUNCE_MS\
: Utils.g('RESET\
_HEAVY_DEBOUNCE_\
MS', 24)\x0a       \
 };\x0a\x0a        // \
Logging caps (us\
ed by Logger).\x0a \
       this.LOG \
= {\x0a          MA\
X_QUEUE: Utils.g\
('LOG_MAX_QUEUE'\
, 400),\x0a        \
  MAX_BYTES: Uti\
ls.g('LOG_MAX_BY\
TES', 256 * 1024\
),\x0a          BAT\
CH_MAX: Utils.g(\
'LOG_BATCH_MAX',\
 64),\x0a          \
RATE_LIMIT_PER_S\
EC: Utils.g('LOG\
_RATE_LIMIT_PER_\
SEC', 0)\x0a       \
 };\x0a\x0a        // \
Async tuning for\
 background work\
.\x0a        this.A\
SYNC = {\x0a       \
   SLICE_MS: Uti\
ls.g('ASYNC_SLIC\
E_MS', 12),\x0a    \
      MIN_YIELD_\
MS: Utils.g('ASY\
NC_MIN_YIELD_MS'\
, 0),\x0a          \
MD_NODES_PER_SLI\
CE: Utils.g('ASY\
NC_MD_NODES_PER_\
SLICE', 12)\x0a    \
    };\x0a\x0a        \
// RAF pump tuni\
ng (budget per f\
rame).\x0a        t\
his.RAF = {\x0a    \
      FLUSH_BUDG\
ET_MS: Utils.g('\
RAF_FLUSH_BUDGET\
_MS', 7),\x0a      \
    MAX_TASKS_PE\
R_FLUSH: Utils.g\
('RAF_MAX_TASKS_\
PER_FLUSH', 120)\
\x0a        };\x0a\x0a   \
     // Markdown\
 tuning \xe2\x80\x93 allo\
w/disallow inden\
ted code blocks.\
\x0a        this.MD\
 = {\x0a          A\
LLOW_INDENTED_CO\
DE: Utils.g('MD_\
ALLOW_INDENTED_C\
ODE', false)\x0a   \
     };\x0a\x0a       \
 // Custom marku\
p rules for simp\
le tags in text.\
\x0a        // NOTE\
: added nl2br + \
allowBr for thin\
k rules; rest un\
changed.\x0a       \
 this.CUSTOM_MAR\
KUP_RULES = Util\
s.g('CUSTOM_MARK\
UP_RULES', [\x0a   \
       { name: '\
cmd',        ope\
n: '[!cmd]',    \
 close: '[/!cmd]\
',     tag: 'div\
',   className: \
'cmd', innerMode\
: 'text' },\x0a\x0a   \
       // Think \
(Markdown-style)\
 \xe2\x80\x93 convert new\
lines to <br>, a\
llow real <br> t\
okens; safe-esca\
pe everything el\
se.\x0a          { \
name: 'think_md'\
,   open: '[!thi\
nk]',   close: '\
[/!think]',   ta\
g: 'think', clas\
sName: '',    in\
nerMode: 'text',\
 nl2br: true, al\
lowBr: true },\x0a\x0a\
          // Thi\
nk (HTML-style, \
streaming-friend\
ly)\x0a          { \
name: 'think_htm\
l', open: '<thin\
k>',    close: '\
</think>',    ta\
g: 'think', clas\
sName: '',    in\
nerMode: 'text',\
 stream: true, n\
l2br: true, allo\
wBr: true },\x0a\x0a  \
        { name: \
'tool', open: '<\
tool>',    close\
: '</tool>',    \
tag: 'div', clas\
sName: 'cmd',   \
 innerMode: 'tex\
t', stream: true\
 },\x0a\x0a          /\
/ Streams+final:\
 convert [!exec]\
... into fenced \
python code BEFO\
RE markdown-it\x0a \
         { name:\
 'exec_md',    o\
pen: '[!exec]', \
   close: '[/!ex\
ec]',    innerMo\
de: 'text', stre\
am: true,\x0a      \
      openReplac\
e: '```python\x5cn'\
, closeReplace: \
'\x5cn```', phase: \
'source' },\x0a\x0a   \
       // Stream\
s+final: convert\
 <execute>...</e\
xecute> into fen\
ced python code \
BEFORE markdown-\
it\x0a          { n\
ame: 'exec_html'\
,  open: '<execu\
te>',  close: '<\
/execute>',  inn\
erMode: 'text', \
stream: true,\x0a  \
          openRe\
place: '```pytho\
n\x5cn', closeRepla\
ce: '\x5cn```', pha\
se: 'source' }\x0a \
       ]);\x0a     \
 }\x0a    }\x0a\x0a  // =\
================\
================\
================\
================\
=========\x0a  // 1\
) DOM references\
\x0a  // ==========\
================\
================\
================\
================\
\x0a\x0a  class DOMRef\
s {\x0a    construc\
tor() { this.els\
 = {}; this.domO\
utputStream = nu\
ll; this.domStre\
amMsg = null; }\x0a\
    // Cache fre\
quently used ele\
ments by id.\x0a   \
 init() {\x0a      \
const ids = [\x0a  \
      'container\
','_nodes_','_ap\
pend_input_','_a\
ppend_output_bef\
ore_','_append_o\
utput_',\x0a       \
 '_append_live_'\
,'_footer_','_lo\
ader_','tips','s\
crollFab','scrol\
lFabIcon'\x0a      \
];\x0a      ids.for\
Each(id => { thi\
s.els[id] = docu\
ment.getElementB\
yId(id); });\x0a   \
 }\x0a    // Get el\
ement by id (rea\
ds cache first).\
\x0a    get(id) { r\
eturn this.els[i\
d] || document.g\
etElementById(id\
); }\x0a    // Rese\
t ephemeral poin\
ters (used durin\
g stream).\x0a    r\
esetEphemeral() \
{ this.domStream\
Msg = null; }\x0a  \
  // Release ref\
s and restore de\
fault scroll beh\
avior.\x0a    clean\
up() {\x0a      thi\
s.resetEphemeral\
(); this.domOutp\
utStream = null;\
 this.els = {};\x0a\
      try { hist\
ory.scrollRestor\
ation = \x22auto\x22; \
} catch (_) {}\x0a \
   }\x0a    // Fast\
er clear of a co\
ntainer by avoid\
ing innerHTML=''\
 (which is slow \
on large trees).\
\x0a    fastClear(i\
d) {\x0a      const\
 el = this.get(i\
d);\x0a      if (!e\
l) return null;\x0a\
      // Fast pa\
ths:\x0a      if (e\
l.firstChild) {\x0a\
        // Prefe\
r replaceChildre\
n, fallback to t\
extContent.\x0a    \
    if (el.repla\
ceChildren) el.r\
eplaceChildren()\
;\x0a        else e\
l.textContent = \
'';\x0a      }\x0a    \
  return el;\x0a   \
 }\x0a\x0a    // Clear\
 and ensure pain\
t on next frame \
(await before re\
ading layout).\x0a \
   async fastCle\
arAndPaint(id) {\
\x0a      const el \
= this.fastClear\
(id);\x0a      if (\
!el) return null\
;\x0a      // Yield\
 to the next fra\
me through the c\
entralized RafMa\
nager to ensure \
repaint.\x0a      t\
ry {\x0a        if \
(typeof runtime \
!== 'undefined' \
&& runtime.raf &\
& typeof runtime\
.raf.nextFrame =\
== 'function') {\
\x0a          await\
 runtime.raf.nex\
tFrame();\x0a      \
  } else {\x0a     \
     await new P\
romise(r => setT\
imeout(r, 16));\x0a\
        }\x0a      \
} catch (_) {}\x0a \
     return el;\x0a\
    }\x0a\x0a    // Ha\
rd clear by temp\
orarily hiding e\
lement to avoid \
intermediate pai\
nts.\x0a    fastCle\
arHidden(id) {\x0a \
     const el = \
this.get(id);\x0a  \
    if (!el) ret\
urn null;\x0a      \
const prevDispla\
y = el.style.dis\
play;\x0a      el.s\
tyle.display = '\
none';       // \
pause paints\x0a   \
   if (el.replac\
eChildren) el.re\
placeChildren();\
\x0a      else el.t\
extContent = '';\
\x0a      el.style.\
display = prevDi\
splay;  // resum\
e paints\x0a      r\
eturn el;\x0a    }\x0a\
    // Replace e\
lement node by a\
 shallow clone (\
drops children).\
\x0a    hardReplace\
ByClone(id) {\x0a  \
    const el = t\
his.get(id);\x0a   \
   if (!el || !e\
l.parentNode) re\
turn null;\x0a     \
 const clone = e\
l.cloneNode(fals\
e);\x0a      try { \
el.replaceWith(c\
lone); } catch (\
_) { el.innerHTM\
L = ''; }\x0a      \
this.els[id] = c\
lone;\x0a      if (\
id === '_append_\
output_') this.d\
omOutputStream =\
 clone;\x0a      re\
turn clone;\x0a    \
}\x0a    // Clear s\
treaming contain\
ers and transien\
t state.\x0a    har\
dResetStreamCont\
ainers() {\x0a     \
 this.resetEphem\
eral();\x0a      th\
is.domOutputStre\
am = null;\x0a     \
 this.fastClearH\
idden('_append_o\
utput_before_');\
\x0a      this.fast\
ClearHidden('_ap\
pend_output_');\x0a\
    }\x0a    // Ret\
urn output strea\
m container, cac\
hing reference.\x0a\
    getStreamCon\
tainer() {\x0a     \
 if (this.domOut\
putStream && thi\
s.domOutputStrea\
m.isConnected) r\
eturn this.domOu\
tputStream;\x0a    \
  const el = thi\
s.get('_append_o\
utput_'); if (el\
) this.domOutput\
Stream = el; ret\
urn el;\x0a    }\x0a  \
  // Get or crea\
te current strea\
ming message con\
tainer (.msg-box\
 > .msg > .md-sn\
apshot-root).\x0a  \
  getStreamMsg(c\
reate, name_head\
er) {\x0a      cons\
t container = th\
is.getStreamCont\
ainer(); if (!co\
ntainer) return \
null;\x0a      if (\
this.domStreamMs\
g && this.domStr\
eamMsg.isConnect\
ed) return this.\
domStreamMsg;\x0a\x0a \
     let box = c\
ontainer.querySe\
lector('.msg-box\
'); let msg = nu\
ll;\x0a      if (!b\
ox && create) {\x0a\
        box = do\
cument.createEle\
ment('div'); box\
.classList.add('\
msg-box', 'msg-b\
ot');\x0a        if\
 (name_header) {\
\x0a          const\
 name = document\
.createElement('\
div'); name.clas\
sList.add('name-\
header','name-bo\
t'); name.innerH\
TML = name_heade\
r; box.appendChi\
ld(name);\x0a      \
  }\x0a        msg \
= document.creat\
eElement('div');\
 msg.classList.a\
dd('msg');\x0a     \
   const snap = \
document.createE\
lement('div'); s\
nap.className = \
'md-snapshot-roo\
t';\x0a        msg.\
appendChild(snap\
); box.appendChi\
ld(msg); contain\
er.appendChild(b\
ox);\x0a      } els\
e if (box) {\x0a   \
     msg = box.q\
uerySelector('.m\
sg');\x0a        if\
 (msg && !msg.qu\
erySelector('.md\
-snapshot-root')\
) {\x0a          co\
nst snap = docum\
ent.createElemen\
t('div'); snap.c\
lassName = 'md-s\
napshot-root'; m\
sg.appendChild(s\
nap);\x0a        }\x0a\
      }\x0a      if\
 (msg) this.domS\
treamMsg = msg;\x0a\
      return msg\
;\x0a    }\x0a    // C\
lear the \x22before\
\x22 area (older me\
ssages area).\x0a  \
  clearStreamBef\
ore() {\x0a      if\
 (typeof window.\
hideTips === 'fu\
nction') { windo\
w.hideTips(); }\x0a\
      const el =\
 this.fastClearH\
idden('_append_o\
utput_before_');\
\x0a      if (el) {\
 /* no-op */ }\x0a \
   }\x0a    // Clea\
r output stream \
area.\x0a    clearO\
utput() { this.h\
ardResetStreamCo\
ntainers(); }\x0a  \
  // Clear messa\
ges list and res\
et state.\x0a    cl\
earNodes() {\x0a   \
   this.clearStr\
eamBefore();\x0a   \
   const el = th\
is.fastClearHidd\
en('_nodes_'); i\
f (el) { el.clas\
sList.add('empty\
_list'); }\x0a     \
 this.resetEphem\
eral();\x0a    }\x0a  \
  // Clear input\
 area.\x0a    clear\
Input() { this.f\
astClearHidden('\
_append_input_')\
; }\x0a    // Clear\
 live area and h\
ide it.\x0a    clea\
rLive() {\x0a      \
const el = this.\
fastClearHidden(\
'_append_live_')\
; if (!el) retur\
n;\x0a      el.clas\
sList.remove('vi\
sible');\x0a      e\
l.classList.add(\
'hidden');\x0a     \
 this.resetEphem\
eral();\x0a    }\x0a  \
}\x0a\x0a    // ======\
================\
================\
================\
================\
====\x0a    // 2) C\
ode scroll state\
 manager\x0a    // \
================\
================\
================\
================\
==========\x0a\x0a    \
class CodeScroll\
State {\x0a      co\
nstructor(cfg, r\
af) {\x0a        th\
is.cfg = cfg;\x0a  \
      this.raf =\
 raf;\x0a        th\
is.map = new Wea\
kMap();\x0a        \
this.rafMap = ne\
w WeakMap();\x0a   \
     this.rafIds\
 = new Set(); //\
 legacy\x0a        \
this.rafKeyMap =\
 new WeakMap();\x0a\
      }\x0a      //\
 Get or create p\
er-code element \
state.\x0a      sta\
te(el) {\x0a       \
 let s = this.ma\
p.get(el);\x0a     \
   if (!s) { s =\
 { autoFollow: f\
alse, lastScroll\
Top: 0, userInte\
racted: false, f\
reezeUntil: 0 };\
 this.map.set(el\
, s); }\x0a        \
return s;\x0a      \
}\x0a      // Check\
 if code block i\
s already finali\
zed (not streami\
ng).\x0a      isFin\
alizedCode(el) {\
\x0a        if (!el\
 || el.tagName !\
== 'CODE') retur\
n false;\x0a       \
 if (el.dataset \
&& el.dataset._a\
ctive_stream ===\
 '1') return fal\
se;\x0a        cons\
t highlighted = \
(el.getAttribute\
('data-highlight\
ed') === 'yes') \
|| el.classList.\
contains('hljs')\
;\x0a        return\
 highlighted;\x0a  \
    }\x0a      // I\
s element scroll\
ed close to the \
bottom by a marg\
in?\x0a      isNear\
BottomEl(el, mar\
gin = 100) {\x0a   \
     if (!el) re\
turn true;\x0a     \
   const distanc\
e = el.scrollHei\
ght - el.clientH\
eight - el.scrol\
lTop;\x0a        re\
turn distance <=\
 margin;\x0a      }\
\x0a      // Scroll\
 code element to\
 the bottom resp\
ecting interacti\
on state.\x0a      \
scrollToBottom(e\
l, live = false,\
 force = false) \
{\x0a        if (!e\
l || !el.isConne\
cted) return;\x0a  \
      if (!force\
 && this.isFinal\
izedCode(el)) re\
turn;\x0a\x0a        c\
onst st = this.s\
tate(el);\x0a      \
  const now = Ut\
ils.now();\x0a     \
   if (!force &&\
 st.freezeUntil \
&& now < st.free\
zeUntil) return;\
\x0a\x0a        const \
distNow = el.scr\
ollHeight - el.c\
lientHeight - el\
.scrollTop;\x0a    \
    if (!force &\
& distNow <= 1) \
{ st.lastScrollT\
op = el.scrollTo\
p; return; }\x0a\x0a  \
      const marg\
inPx = live ? 96\
 : this.cfg.CODE\
_SCROLL.NEAR_MAR\
GIN_PX;\x0a        \
const behavior =\
 'instant';\x0a\x0a   \
     if (!force)\
 {\x0a          if \
(live && st.auto\
Follow !== true)\
 return;\x0a       \
   if (!live && \
!(st.autoFollow \
=== true || this\
.isNearBottomEl(\
el, marginPx) ||\
 !st.userInterac\
ted)) return;\x0a  \
      }\x0a\x0a       \
 try { el.scroll\
To({ top: el.scr\
ollHeight, behav\
ior }); } catch \
(_) { el.scrollT\
op = el.scrollHe\
ight; }\x0a        \
st.lastScrollTop\
 = el.scrollTop;\
\x0a      }\x0a      /\
/ Schedule botto\
m scroll in rAF \
(coalesces multi\
ple calls).\x0a    \
  scheduleScroll\
(el, live = fals\
e, force = false\
) {\x0a        if (\
!el || !el.isCon\
nected) return;\x0a\
        if (!for\
ce && this.isFin\
alizedCode(el)) \
return;\x0a        \
if (this.rafMap.\
get(el)) return;\
\x0a        this.ra\
fMap.set(el, tru\
e);\x0a\x0a        let\
 key = this.rafK\
eyMap.get(el);\x0a \
       if (!key)\
 { key = { t: 'c\
odeScroll', el }\
; this.rafKeyMap\
.set(el, key); }\
\x0a\x0a        this.r\
af.schedule(key,\
 () => {\x0a       \
   this.rafMap.d\
elete(el);\x0a     \
     this.scroll\
ToBottom(el, liv\
e, force);\x0a     \
   }, 'CodeScrol\
l', 0);\x0a      }\x0a\
      // Attach \
scroll/wheel/tou\
ch handlers to m\
anage auto-follo\
w state.\x0a      a\
ttachHandlers(co\
deEl) {\x0a        \
if (!codeEl || c\
odeEl.dataset.cs\
Listeners === '1\
') return;\x0a     \
   codeEl.datase\
t.csListeners = \
'1';\x0a        con\
st st = this.sta\
te(codeEl);\x0a\x0a   \
     const onScr\
oll = (ev) => {\x0a\
          const \
top = codeEl.scr\
ollTop;\x0a        \
  const isUser =\
 !!(ev && ev.isT\
rusted === true)\
;\x0a          cons\
t now = Utils.no\
w();\x0a\x0a          \
if (this.isFinal\
izedCode(codeEl)\
) {\x0a            \
if (isUser) st.u\
serInteracted = \
true;\x0a          \
  st.autoFollow \
= false;\x0a       \
     st.lastScro\
llTop = top;\x0a   \
         return;\
\x0a          }\x0a\x0a  \
        if (isUs\
er) {\x0a          \
  if (top + 1 < \
st.lastScrollTop\
) {\x0a            \
  st.autoFollow \
= false; st.user\
Interacted = tru\
e; st.freezeUnti\
l = now + 1000;\x0a\
            } el\
se if (this.isNe\
arBottomEl(codeE\
l, this.cfg.CODE\
_SCROLL.AUTO_FOL\
LOW_REENABLE_PX)\
) {\x0a            \
  st.autoFollow \
= true;\x0a        \
    }\x0a          \
} else {\x0a       \
     if (this.is\
NearBottomEl(cod\
eEl, this.cfg.CO\
DE_SCROLL.AUTO_F\
OLLOW_REENABLE_P\
X)) st.autoFollo\
w = true;\x0a      \
    }\x0a          \
st.lastScrollTop\
 = top;\x0a        \
};\x0a\x0a        cons\
t onWheel = (ev)\
 => {\x0a          \
st.userInteracte\
d = true;\x0a      \
    const now = \
Utils.now();\x0a\x0a  \
        if (this\
.isFinalizedCode\
(codeEl)) { st.a\
utoFollow = fals\
e; return; }\x0a\x0a  \
        if (ev.d\
eltaY < 0) { st.\
autoFollow = fal\
se; st.freezeUnt\
il = now + 1000;\
 }\x0a          els\
e if (this.isNea\
rBottomEl(codeEl\
, this.cfg.CODE_\
SCROLL.AUTO_FOLL\
OW_REENABLE_PX))\
 { st.autoFollow\
 = true; }\x0a     \
   };\x0a\x0a        c\
odeEl.addEventLi\
stener('scroll',\
 onScroll, { pas\
sive: true });\x0a \
       codeEl.ad\
dEventListener('\
wheel', onWheel,\
 { passive: true\
 });\x0a        cod\
eEl.addEventList\
ener('touchstart\
', function () {\
 st.userInteract\
ed = true; }, { \
passive: true })\
;\x0a      }\x0a      \
// Attach handle\
rs to all bot co\
de blocks under \
root (or documen\
t).\x0a      // IMP\
ORTANT: We inten\
tionally do NOT \
auto-scroll fina\
lized/static cod\
e blocks to the \
bottom.\x0a      //\
 Only actively s\
treaming code bl\
ocks (data-_acti\
ve_stream=\x221\x22) a\
re auto-followed\
 live.\x0a      ini\
tScrollableBlock\
s(root) {\x0a      \
  const scope = \
root || document\
;\x0a        let no\
des = [];\x0a      \
  if (scope.node\
Type === 1 && sc\
ope.closest && s\
cope.closest('.m\
sg-box.msg-bot')\
) {\x0a          no\
des = scope.quer\
ySelectorAll('pr\
e code');\x0a      \
  } else {\x0a     \
     nodes = doc\
ument.querySelec\
torAll('.msg-box\
.msg-bot pre cod\
e');\x0a        }\x0a \
       if (!node\
s.length) return\
;\x0a\x0a        nodes\
.forEach((code) \
=> {\x0a          t\
his.attachHandle\
rs(code);\x0a      \
    // Live stre\
aming blocks: en\
able auto-follow\
 and keep them g\
lued to bottom.\x0a\
          if (co\
de.dataset._acti\
ve_stream === '1\
') {\x0a           \
 const st = this\
.state(code);\x0a  \
          st.aut\
oFollow = true;\x0a\
            this\
.scheduleScroll(\
code, true, fals\
e);\x0a          }\x0a\
          // Fin\
alized/static bl\
ocks: do nothing\
 (no initial scr\
oll-to-bottom).\x0a\
          // Thi\
s avoids surpris\
ing jumps when s\
tatic content is\
 rendered.\x0a     \
   });\x0a      }\x0a \
     // Transfer\
 stored scroll s\
tate between ele\
ments (after rep\
lace).\x0a      tra\
nsfer(oldEl, new\
El) {\x0a        if\
 (!oldEl || !new\
El || oldEl === \
newEl) return;\x0a \
       const old\
State = this.map\
.get(oldEl);\x0a   \
     if (oldStat\
e) this.map.set(\
newEl, { ...oldS\
tate });\x0a       \
 this.attachHand\
lers(newEl);\x0a   \
   }\x0a      // Ca\
ncel any schedul\
ed scroll tasks \
for code blocks.\
\x0a      cancelAll\
Scrolls() {\x0a    \
    try { this.r\
af.cancelGroup('\
CodeScroll'); } \
catch (_) {}\x0a   \
     this.rafMap\
 = new WeakMap()\
;\x0a        this.r\
afIds.clear();\x0a \
     }\x0a    }\x0a\x0a  \
// =============\
================\
================\
================\
=============\x0a  \
// 3) Highlighte\
r (hljs) + rAF v\
iewport scan\x0a  /\
/ ==============\
================\
================\
================\
============\x0a\x0a  \
class Highlighte\
r {\x0a    construc\
tor(cfg, codeScr\
oll, raf) {\x0a    \
  this.cfg = cfg\
;\x0a      this.cod\
eScroll = codeSc\
roll;\x0a      this\
.raf = raf;\x0a    \
  this.hlSchedul\
ed = false;\x0a    \
  this.hlQueue =\
 [];\x0a      this.\
hlQueueSet = new\
 Set();\x0a      th\
is.scanScheduled\
 = false;\x0a\x0a     \
 // Global scann\
ing state for bu\
dgeted viewport \
scans (prevents \
long frames).\x0a  \
    this._global\
ScanState = null\
;\x0a      // Budge\
t per scan step \
(ms) \xe2\x80\x93 based o\
n RAF budget hin\
t with a small c\
lamp.\x0a      cons\
t hint = (cfg &&\
 cfg.RAF && cfg.\
RAF.FLUSH_BUDGET\
_MS) ? cfg.RAF.F\
LUSH_BUDGET_MS :\
 7;\x0a      this.S\
CAN_STEP_BUDGET_\
MS = Math.max(3,\
 Math.min(12, hi\
nt));\x0a    }\x0a    \
_decodeEntitiesD\
eep(text, maxPas\
ses = 2) {\x0a     \
   if (!text || \
text.indexOf('&'\
) === -1) return\
 text || '';\x0a   \
     const ta = \
Highlighter._dec\
TA || (Highlight\
er._decTA = docu\
ment.createEleme\
nt('textarea'));\
\x0a        const d\
ecodeOnce = (s) \
=> { ta.innerHTM\
L = s; return ta\
.value; };\x0a     \
   let prev = St\
ring(text);\x0a    \
    let cur = de\
codeOnce(prev);\x0a\
        let pass\
es = 1;\x0a        \
while (passes < \
maxPasses && cur\
 !== prev) {\x0a   \
       prev = cu\
r;\x0a          cur\
 = decodeOnce(pr\
ev);\x0a          p\
asses++;\x0a       \
 }\x0a        retur\
n cur;\x0a      }\x0a \
   // Global swi\
tch to skip all \
highlighting.\x0a  \
  isDisabled() {\
 return !!this.c\
fg.HL.DISABLE_AL\
L; }\x0a    // Conf\
igure hljs once \
(safe if hljs no\
t present).\x0a    \
initHLJS() {\x0a   \
   if (this.isDi\
sabled()) return\
;\x0a      if (type\
of hljs !== 'und\
efined' && hljs)\
 { try { hljs.co\
nfigure({ ignore\
UnescapedHTML: t\
rue }); } catch \
(_) {} }\x0a    }\x0a \
   // Check if c\
ode is near view\
port (with prelo\
ad).\x0a    _nearVi\
ewport(el) {\x0a   \
   const preload\
 = this.cfg.SCAN\
.PRELOAD_PX;\x0a   \
   const vh = wi\
ndow.innerHeight\
 || Utils.SE.cli\
entHeight || 800\
;\x0a      const r \
= el.getBounding\
ClientRect();\x0a  \
    return r.bot\
tom >= -preload \
&& r.top <= (vh \
+ preload);\x0a    \
}\x0a    // Queue a\
 code element fo\
r highlight; ski\
p active streami\
ng code and heav\
y-known cases.\x0a \
   queue(codeEl,\
 activeCode) {\x0a \
     if (this.is\
Disabled()) retu\
rn;\x0a      if (!c\
odeEl || !codeEl\
.isConnected) re\
turn;\x0a      if (\
activeCode && co\
deEl === activeC\
ode.codeEl) retu\
rn;\x0a      if (co\
deEl.getAttribut\
e('data-highligh\
ted') === 'yes')\
 return;\x0a      i\
f (codeEl.datase\
t && (codeEl.dat\
aset.hlStreamSus\
pended === '1' |\
| codeEl.dataset\
.finalHlSkip ===\
 '1')) return; /\
/ skip heavy blo\
cks intentionall\
y left plain\x0a   \
   if (!codeEl.c\
losest('.msg-box\
.msg-bot')) retu\
rn;\x0a      if (!t\
his.hlQueueSet.h\
as(codeEl)) { th\
is.hlQueueSet.ad\
d(codeEl); this.\
hlQueue.push(cod\
eEl); }\x0a      if\
 (!this.hlSchedu\
led) {\x0a        t\
his.hlScheduled \
= true;\x0a        \
this.raf.schedul\
e('HL:flush', ()\
 => this.flush(a\
ctiveCode), 'Hig\
hlighter', 1);\x0a \
     }\x0a    }\x0a   \
 // Process a sm\
all batch of cod\
e elements per f\
rame.\x0a    flush(\
activeCode) {\x0a  \
    if (this.isD\
isabled()) { thi\
s.hlScheduled = \
false; this.hlQu\
eueSet.clear(); \
this.hlQueue.len\
gth = 0; return;\
 }\x0a      this.hl\
Scheduled = fals\
e;\x0a      let cou\
nt = 0;\x0a      wh\
ile (this.hlQueu\
e.length && coun\
t < this.cfg.HL.\
PER_FRAME) {\x0a   \
     const el = \
this.hlQueue.shi\
ft();\x0a        if\
 (el && el.isCon\
nected) this.saf\
eHighlight(el, a\
ctiveCode);\x0a    \
    if (el) this\
.hlQueueSet.dele\
te(el);\x0a        \
count++;\x0a       \
 try {\x0a         \
 const sched = (\
navigator && nav\
igator.schedulin\
g && navigator.s\
cheduling.isInpu\
tPending) ? navi\
gator.scheduling\
 : null;\x0a       \
   if (sched && \
sched.isInputPen\
ding({ includeCo\
ntinuous: true }\
)) {\x0a           \
 if (this.hlQueu\
e.length) {\x0a    \
          this.h\
lScheduled = tru\
e;\x0a             \
 this.raf.schedu\
le('HL:flush', (\
) => this.flush(\
activeCode),  'H\
ighlighter', 1);\
\x0a            }\x0a \
           retur\
n;\x0a          }\x0a \
       } catch (\
_) {}\x0a      }\x0a  \
    if (this.hlQ\
ueue.length) {\x0a \
       this.hlSc\
heduled = true;\x0a\
        this.raf\
.schedule('HL:fl\
ush', () => this\
.flush(activeCod\
e), 'Highlighter\
', 1);\x0a      }\x0a \
   }\x0a    // High\
light a single c\
ode block with s\
afety checks and\
 scroll preserva\
tion.\x0a    _needs\
DeepDecode(text)\
 {\x0a        if (!\
text) return fal\
se;\x0a        cons\
t s = String(tex\
t);\x0a        retu\
rn (s.indexOf('&\
amp;') !== -1) |\
| (s.indexOf('&#\
') !== -1);\x0a    \
  }\x0a\x0a      safeH\
ighlight(codeEl,\
 activeCode) {\x0a \
       if (this.\
isDisabled()) re\
turn;\x0a        if\
 (!window.hljs |\
| !codeEl || !co\
deEl.isConnected\
) return;\x0a      \
  if (!codeEl.cl\
osest('.msg-box.\
msg-bot')) retur\
n;\x0a        if (c\
odeEl.getAttribu\
te('data-highlig\
hted') === 'yes'\
) return;\x0a      \
  if (activeCode\
 && codeEl === a\
ctiveCode.codeEl\
) return;\x0a\x0a     \
   // fast-skip \
final highlight \
for gigantic blo\
cks using precom\
puted meta.\x0a    \
    try {\x0a      \
    const wrap =\
 codeEl.closest(\
'.code-wrapper')\
;\x0a          cons\
t maxLines = thi\
s.cfg.PROFILE_CO\
DE.finalHighligh\
tMaxLines | 0;\x0a \
         const m\
axChars = this.c\
fg.PROFILE_CODE.\
finalHighlightMa\
xChars | 0;\x0a\x0a   \
       let lines\
 = NaN, chars = \
NaN;\x0a          i\
f (wrap) {\x0a     \
       const nlA\
ttr = wrap.getAt\
tribute('data-co\
de-nl');\x0a       \
     const lenAt\
tr = wrap.getAtt\
ribute('data-cod\
e-len');\x0a       \
     if (nlAttr)\
 lines = parseIn\
t(nlAttr, 10);\x0a \
           if (l\
enAttr) chars = \
parseInt(lenAttr\
, 10);\x0a         \
 }\x0a\x0a          if\
 ((Number.isFini\
te(lines) && max\
Lines > 0 && lin\
es > maxLines) |\
|\x0a              \
(Number.isFinite\
(chars) && maxCh\
ars > 0 && chars\
 > maxChars)) {\x0a\
            // N\
EW: normalize en\
tities for reada\
bility even if w\
e skip final hig\
hlight\x0a         \
   try {\x0a       \
       const raw\
 = codeEl.textCo\
ntent || '';\x0a   \
           if (t\
his._needsDeepDe\
code(raw)) {\x0a   \
             con\
st dec = this._d\
ecodeEntitiesDee\
p(raw);\x0a        \
        if (dec \
!== raw) codeEl.\
textContent = de\
c;\x0a             \
 }\x0a            }\
 catch (_) {}\x0a\x0a \
           codeE\
l.classList.add(\
'hljs');\x0a       \
     codeEl.setA\
ttribute('data-h\
ighlighted', 'ye\
s');\x0a           \
 codeEl.dataset.\
finalHlSkip = '1\
';\x0a            t\
ry { this.codeSc\
roll.attachHandl\
ers(codeEl); } c\
atch (_) {}\x0a    \
        this.cod\
eScroll.schedule\
Scroll(codeEl, f\
alse, false);\x0a  \
          return\
;\x0a          }\x0a\x0a \
         // Fall\
back to reading \
actual text only\
 if wrapper meta\
 is missing.\x0a   \
       if (!Numb\
er.isFinite(line\
s) || !Number.is\
Finite(chars)) {\
\x0a            con\
st txt0 = codeEl\
.textContent || \
'';\x0a            \
const ln0 = Util\
s.countNewlines(\
txt0);\x0a         \
   if ((maxLines\
 > 0 && ln0 > ma\
xLines) || (maxC\
hars > 0 && txt0\
.length > maxCha\
rs)) {\x0a         \
     // NEW: nor\
malize entities \
here as well\x0a   \
           try {\
\x0a               \
 if (this._needs\
DeepDecode(txt0)\
) {\x0a            \
      const dec \
= this._decodeEn\
titiesDeep(txt0)\
;\x0a              \
    if (dec !== \
txt0) codeEl.tex\
tContent = dec;\x0a\
                \
}\x0a              \
} catch (_) {}\x0a\x0a\
              co\
deEl.classList.a\
dd('hljs');\x0a    \
          codeEl\
.setAttribute('d\
ata-highlighted'\
, 'yes');\x0a      \
        codeEl.d\
ataset.finalHlSk\
ip = '1';\x0a      \
        try { th\
is.codeScroll.at\
tachHandlers(cod\
eEl); } catch (_\
) {}\x0a           \
   this.codeScro\
ll.scheduleScrol\
l(codeEl, false,\
 false);\x0a       \
       return;\x0a \
           }\x0a   \
       }\x0a       \
 } catch (_) { /\
* safe fallback \
*/ }\x0a\x0a        co\
nst wasNearBotto\
m = this.codeScr\
oll.isNearBottom\
El(codeEl, 16);\x0a\
        const st\
 = this.codeScro\
ll.state(codeEl)\
;\x0a        const \
shouldAutoScroll\
After = (st.auto\
Follow === true)\
 || wasNearBotto\
m;\x0a\x0a        try \
{\x0a          try \
{ codeEl.classLi\
st.remove('hljs'\
); codeEl.remove\
Attribute('data-\
highlighted'); }\
 catch (_) {}\x0a\x0a \
         // NEW:\
 deep-decode tex\
t before highlig\
hting (fixes &am\
p;#x27; \xe2\x86\x92 ' et\
c.)\x0a          le\
t txt = codeEl.t\
extContent || ''\
;\x0a          if (\
this._needsDeepD\
ecode(txt)) {\x0a  \
          try { \
txt = this._deco\
deEntitiesDeep(t\
xt); } catch (_)\
 {}\x0a          }\x0a\
          codeEl\
.textContent = t\
xt; // ensure no\
 stale spans rem\
ain and normaliz\
ed text provided\
\x0a\x0a          hljs\
.highlightElemen\
t(codeEl);\x0a     \
     codeEl.setA\
ttribute('data-h\
ighlighted', 'ye\
s');\x0a        } c\
atch (_) {\x0a     \
     if (!codeEl\
.classList.conta\
ins('hljs')) cod\
eEl.classList.ad\
d('hljs');\x0a     \
   } finally {\x0a \
         try { t\
his.codeScroll.a\
ttachHandlers(co\
deEl); } catch (\
_) {}\x0a          \
const needInitFo\
rce = (codeEl.da\
taset && (codeEl\
.dataset.csInitB\
tm === '1' || co\
deEl.dataset.jus\
tFinalized === '\
1'));\x0a          \
const mustScroll\
 = shouldAutoScr\
ollAfter || need\
InitForce;\x0a     \
     if (mustScr\
oll) this.codeSc\
roll.scheduleScr\
oll(codeEl, fals\
e, !!needInitFor\
ce);\x0a          i\
f (codeEl.datase\
t) {\x0a           \
 if (codeEl.data\
set.csInitBtm ==\
= '1') codeEl.da\
taset.csInitBtm \
= '0';\x0a         \
   if (codeEl.da\
taset.justFinali\
zed === '1') cod\
eEl.dataset.just\
Finalized = '0';\
\x0a          }\x0a   \
     }\x0a      }\x0a\x0a\
    // Start a b\
udgeted global s\
can \xe2\x80\x93 split ac\
ross frames to a\
void long blocki\
ng.\x0a    _startGl\
obalScan(activeC\
ode) {\x0a      if \
(this.isDisabled\
()) return;\x0a    \
  const preload \
= this.cfg.SCAN_\
PRELOAD_PX || th\
is.cfg.SCAN.PREL\
OAD_PX;\x0a      co\
nst vh = window.\
innerHeight || U\
tils.SE.clientHe\
ight || 800;\x0a   \
   const rectTop\
 = 0 - preload, \
rectBottom = vh \
+ preload;\x0a     \
 const nodes = A\
rray.from(docume\
nt.querySelector\
All('.msg-box.ms\
g-bot pre code:n\
ot([data-highlig\
hted=\x22yes\x22])'));\
\x0a      this._glo\
balScanState = {\
 nodes, idx: 0, \
rectTop, rectBot\
tom, activeCode \
};\x0a      this._s\
canGlobalStep();\
\x0a    }\x0a    // Co\
ntinue global sc\
an for visible c\
ode elements und\
er a time budget\
.\x0a    _scanGloba\
lStep() {\x0a      \
const state = th\
is._globalScanSt\
ate;\x0a      if (!\
state || !state.\
nodes || state.i\
dx >= state.node\
s.length) { this\
._globalScanStat\
e = null; return\
; }\x0a      const \
start = Utils.no\
w();\x0a      while\
 (state.idx < st\
ate.nodes.length\
) {\x0a        cons\
t code = state.n\
odes[state.idx++\
];\x0a        if (!\
code || !code.is\
Connected) conti\
nue;\x0a        if \
(state.activeCod\
e && code === st\
ate.activeCode.c\
odeEl) continue;\
\x0a        try {\x0a \
         const r\
 = code.getBound\
ingClientRect();\
\x0a          if (r\
.bottom >= state\
.rectTop && r.to\
p <= state.rectB\
ottom) this.queu\
e(code, state.ac\
tiveCode);\x0a     \
   } catch (_) {\
}\x0a        if ((U\
tils.now() - sta\
rt) >= this.SCAN\
_STEP_BUDGET_MS)\
 {\x0a          // \
Schedule next sl\
ice to keep UI r\
esponsive.\x0a     \
     this.raf.sc\
hedule('HL:scanS\
tep', () => this\
._scanGlobalStep\
(), 'Highlighter\
', 2);\x0a         \
 return;\x0a       \
 }\x0a      }\x0a     \
 this._globalSca\
nState = null;\x0a \
   }\x0a\x0a    // Obs\
erve new code bl\
ocks and queue t\
hose near the vi\
ewport.\x0a    obse\
rveNewCode(root,\
 opts, activeCod\
e) {\x0a      const\
 scope = root ||\
 document;\x0a     \
 let nodes;\x0a    \
  if (scope.node\
Type === 1 && sc\
ope.closest && s\
cope.closest('.m\
sg-box.msg-bot')\
) nodes = scope.\
querySelectorAll\
('pre code');\x0a  \
    else nodes =\
 document.queryS\
electorAll('.msg\
-box.msg-bot pre\
 code');\x0a      i\
f (!nodes || !no\
des.length) retu\
rn;\x0a\x0a      const\
 options = Objec\
t.assign({ defer\
LastIfStreaming:\
 false, minLines\
ForLast: 2, minC\
harsForLast: 120\
 }, (opts || {})\
);\x0a      nodes.f\
orEach((code) =>\
 {\x0a        if (!\
code.closest('.m\
sg-box.msg-bot')\
) return;\x0a      \
  this.codeScrol\
l.attachHandlers\
(code);\x0a        \
if (this.isDisab\
led()) return;\x0a \
       if (activ\
eCode && code ==\
= activeCode.cod\
eEl) return;\x0a\x0a  \
      if (option\
s.deferLastIfStr\
eaming && active\
Code && code ===\
 activeCode.code\
El) {\x0a          \
const tailLen = \
(activeCode.tail\
El && activeCode\
.tailEl.textCont\
ent) ? activeCod\
e.tailEl.textCon\
tent.length : 0;\
\x0a          const\
 tailLines = (ty\
peof activeCode.\
tailLines === 'n\
umber') ? active\
Code.tailLines :\
 0;\x0a          if\
 (tailLines < op\
tions.minLinesFo\
rLast && tailLen\
 < options.minCh\
arsForLast) retu\
rn;\x0a        }\x0a  \
      if (this._\
nearViewport(cod\
e)) this.queue(c\
ode, activeCode)\
;\x0a      });\x0a    \
}\x0a    // Schedul\
e a viewport sca\
n in a budgeted \
way.\x0a    schedul\
eScanVisibleCode\
s(activeCode) {\x0a\
      if (this.i\
sDisabled()) ret\
urn;\x0a\x0a      // F\
ast bail-out: no\
thing to highlig\
ht and no active\
 streaming code.\
\x0a      try {\x0a   \
     const anyCa\
ndidate = docume\
nt.querySelector\
('.msg-box.msg-b\
ot pre code:not(\
[data-highlighte\
d=\x22yes\x22])');\x0a   \
     const hasAc\
tive = !!(active\
Code && activeCo\
de.codeEl && act\
iveCode.codeEl.i\
sConnected);\x0a   \
     if (!anyCan\
didate && !hasAc\
tive) return;\x0a  \
    } catch (_) \
{ /* safe fallba\
ck */ }\x0a\x0a      /\
/ If a scan is a\
lready in progre\
ss, just ensure \
next step is sch\
eduled; otherwis\
e schedule start\
.\x0a      if (this\
._globalScanStat\
e) {\x0a        thi\
s.raf.schedule('\
HL:scanStep', ()\
 => this._scanGl\
obalStep(), 'Hig\
hlighter', 2);\x0a \
       return;\x0a \
     }\x0a      if \
(this.scanSchedu\
led) return;\x0a   \
   this.scanSche\
duled = true;\x0a  \
    this.raf.sch\
edule('HL:scan',\
 () => {\x0a       \
 this.scanSchedu\
led = false;\x0a   \
     this._start\
GlobalScan(activ\
eCode || null);\x0a\
      }, 'Highli\
ghter', 2);\x0a    \
}\x0a\x0a    // Direct\
 scan (synchrono\
us) \xe2\x80\x93 used in \
places where roo\
t scope is small\
.\x0a    scanVisibl\
eCodes(activeCod\
e) {\x0a      this.\
_startGlobalScan\
(activeCode || n\
ull);\x0a    }\x0a    \
// Scan only ins\
ide a given root\
 (synchronous, s\
mall scopes).\x0a  \
  scanVisibleCod\
esInRoot(root, a\
ctiveCode) {\x0a   \
   if (this.isDi\
sabled()) return\
;\x0a      const pr\
eload = this.cfg\
.SCAN_PRELOAD_PX\
 || this.cfg.SCA\
N.PRELOAD_PX;\x0a  \
    const vh = w\
indow.innerHeigh\
t || Utils.SE.cl\
ientHeight || 80\
0;\x0a      const r\
ectTop = 0 - pre\
load, rectBottom\
 = vh + preload;\
\x0a      const sco\
pe = root || doc\
ument;\x0a      con\
st nodes = scope\
.querySelectorAl\
l('.msg-box.msg-\
bot pre code:not\
([data-highlight\
ed=\x22yes\x22])');\x0a  \
    // Local roo\
t scans tend to \
be small \xe2\x80\x93 kee\
p them synchrono\
us for simplicit\
y.\x0a      nodes.f\
orEach((code) =>\
 {\x0a        if (!\
code.isConnected\
) return;\x0a      \
  if (activeCode\
 && code === act\
iveCode.codeEl) \
return;\x0a        \
const r = code.g\
etBoundingClient\
Rect();\x0a        \
if (r.bottom >= \
rectTop && r.top\
 <= rectBottom) \
this.queue(code,\
 activeCode);\x0a  \
    });\x0a    }\x0a  \
  installBoxObse\
rver() { /* no-o\
p */ }\x0a    // Vi\
sit bot message \
boxes and call c\
allback (used fo\
r local scans).\x0a\
    observeMsgBo\
xes(root, onBoxI\
ntersect) {\x0a    \
  const scope = \
root || document\
;\x0a      let boxe\
s;\x0a      if (sco\
pe.nodeType === \
1) boxes = scope\
.querySelectorAl\
l('.msg-box.msg-\
bot');\x0a      els\
e boxes = docume\
nt.querySelector\
All('.msg-box.ms\
g-bot');\x0a      b\
oxes.forEach((bo\
x) => { onBoxInt\
ersect && onBoxI\
ntersect(box); }\
);\x0a    }\x0a    // \
Clear all intern\
al queues and sc\
heduled jobs.\x0a  \
  cleanup() {\x0a  \
    try { this.r\
af.cancelGroup('\
Highlighter'); }\
 catch (_) {}\x0a  \
    this.hlSched\
uled = false;\x0a  \
    this.scanSch\
eduled = false;\x0a\
      this._glob\
alScanState = nu\
ll;\x0a      this.h\
lQueueSet.clear(\
); this.hlQueue.\
length = 0;\x0a    \
}\x0a  }\x0a    // ===\
================\
================\
================\
================\
=======\x0a    // 4\
) Custom Markup \
Processor\x0a    //\
 ===============\
================\
================\
================\
===========\x0a\x0a   \
 class CustomMar\
kup {\x0a      cons\
tructor(cfg, log\
ger) {\x0a        t\
his.cfg = cfg ||\
 { CUSTOM_MARKUP\
_RULES: [] };\x0a  \
      this.logge\
r = logger || ne\
w Logger(cfg);\x0a \
       this.__co\
mpiled = null;\x0a \
       this.__ha\
sStreamRules = f\
alse; // Fast fl\
ag to skip strea\
m work if not ne\
eded\x0a      }\x0a   \
   _d(line, ctx)\
 { try { this.lo\
gger.debug('CM',\
 line, ctx); } c\
atch (_) {} }\x0a\x0a \
     // Decode H\
TML entities onc\
e (safe)\x0a      d\
ecodeEntitiesOnc\
e(s) {\x0a        i\
f (!s || s.index\
Of('&') === -1) \
return String(s \
|| '');\x0a        \
const ta = Custo\
mMarkup._decTA |\
| (CustomMarkup.\
_decTA = documen\
t.createElement(\
'textarea'));\x0a  \
      ta.innerHT\
ML = s;\x0a        \
return ta.value;\
\x0a      }\x0a\x0a      \
// Escape helper\
s\x0a      _escHtml\
(s) {\x0a        tr\
y { return Utils\
.escapeHtml(s); \
} catch (_) {\x0a  \
        return S\
tring(s || '').r\
eplace(/[&<>\x22']/\
g, m => ({'&':'&\
amp;','<':'&lt;'\
,'>':'&gt;','\x22':\
'&quot;',\x22'\x22:'&#\
039;'}[m]));\x0a   \
     }\x0a      }\x0a \
     _escapeHtml\
AllowBr(text, { \
convertNewlines \
= true } = {}) {\
\x0a        const P\
LACEHOLDER = '\x5cu\
0001__BR__\x5cu0001\
';\x0a        let s\
 = String(text |\
| '');\x0a        s\
 = s.replace(/<b\
r\x5cs*\x5c/?>/gi, PLA\
CEHOLDER);\x0a     \
   s = this._esc\
Html(s);\x0a       \
 if (convertNewl\
ines) s = s.repl\
ace(/\x5cr\x5cn|\x5cr|\x5cn/\
g, '<br>');\x0a    \
    s = s.replac\
e(new RegExp(PLA\
CEHOLDER, 'g'), \
'<br>');\x0a       \
 return s;\x0a     \
 }\x0a\x0a      hasAny\
OpenToken(text, \
rules) {\x0a       \
 if (!text || !r\
ules || !rules.l\
ength) return fa\
lse;\x0a        for\
 (let i = 0; i <\
 rules.length; i\
++) {\x0a          \
const r = rules[\
i];\x0a          if\
 (!r || !r.open)\
 continue;\x0a     \
     if (text.in\
dexOf(r.open) !=\
= -1) return tru\
e;\x0a        }\x0a   \
     return fals\
e;\x0a      }\x0a\x0a    \
  _materializeIn\
nerHTML(rule, te\
xt, MD) {\x0a      \
  let payload = \
String(text || '\
');\x0a        cons\
t wantsBr = !!(r\
ule && (rule.nl2\
br || rule.allow\
Br));\x0a\x0a        i\
f (rule && rule.\
decodeEntities &\
& payload && pay\
load.indexOf('&'\
) !== -1) {\x0a    \
      try { payl\
oad = this.decod\
eEntitiesOnce(pa\
yload); } catch \
(_) { /* keep or\
iginal */ }\x0a    \
    }\x0a\x0a        i\
f (wantsBr) {\x0a  \
        try { re\
turn this._escap\
eHtmlAllowBr(pay\
load, { convertN\
ewlines: !!rule.\
nl2br }); }\x0a    \
      catch (_) \
{ return this._e\
scHtml(payload);\
 }\x0a        }\x0a\x0a  \
      if (rule &\
& rule.innerMode\
 === 'markdown-i\
nline' && MD && \
typeof MD.render\
Inline === 'func\
tion') {\x0a       \
   try { return \
MD.renderInline(\
payload); } catc\
h (_) { return t\
his._escHtml(pay\
load); }\x0a       \
 }\x0a        retur\
n this._escHtml(\
payload);\x0a      \
}\x0a\x0a      _fragme\
ntFromHTML(html,\
 ctxNode) {\x0a    \
    let frag = n\
ull;\x0a        try\
 {\x0a          con\
st range = docum\
ent.createRange(\
);\x0a          con\
st ctx = (ctxNod\
e && ctxNode.par\
entNode) ? ctxNo\
de.parentNode : \
(document.body |\
| document.docum\
entElement);\x0a   \
       range.sel\
ectNode(ctx);\x0a  \
        frag = r\
ange.createConte\
xtualFragment(St\
ring(html || '')\
);\x0a          ret\
urn frag;\x0a      \
  } catch (_) {\x0a\
          const \
tmp = document.c\
reateElement('di\
v');\x0a          t\
mp.innerHTML = S\
tring(html || ''\
);\x0a          fra\
g = document.cre\
ateDocumentFragm\
ent();\x0a         \
 while (tmp.firs\
tChild) frag.app\
endChild(tmp.fir\
stChild);\x0a      \
    return frag;\
\x0a        }\x0a     \
 }\x0a      _replac\
eElementWithHTML\
(el, html) {\x0a   \
     if (!el || \
!el.parentNode) \
return;\x0a        \
const parent = e\
l.parentNode;\x0a  \
      const frag\
 = this._fragmen\
tFromHTML(html, \
el);\x0a        try\
 {\x0a          par\
ent.insertBefore\
(frag, el);\x0a    \
      parent.rem\
oveChild(el);\x0a  \
      } catch (_\
) {\x0a          co\
nst tmp = docume\
nt.createElement\
('span');\x0a      \
    tmp.innerHTM\
L = String(html \
|| '');\x0a        \
  while (tmp.fir\
stChild) parent.\
insertBefore(tmp\
.firstChild, el)\
;\x0a          pare\
nt.removeChild(e\
l);\x0a        }\x0a  \
    }\x0a\x0a      com\
pile(rules) {\x0a  \
      const src \
= Array.isArray(\
rules) ? rules :\
 (window.CUSTOM_\
MARKUP_RULES || \
this.cfg.CUSTOM_\
MARKUP_RULES || \
[]);\x0a        con\
st compiled = []\
;\x0a        let ha\
sStream = false;\
\x0a\x0a        for (c\
onst r of src) {\
\x0a          if (!\
r || typeof r.op\
en !== 'string' \
|| typeof r.clos\
e !== 'string') \
continue;\x0a\x0a     \
     const tag =\
 (r.tag || 'span\
').toLowerCase()\
;\x0a          cons\
t className = (r\
.className || r.\
class || '').tri\
m();\x0a          c\
onst innerMode =\
 (r.innerMode ==\
= 'markdown-inli\
ne' || r.innerMo\
de === 'text') ?\
 r.innerMode : '\
text';\x0a\x0a        \
  const stream =\
 !!(r.stream ===\
 true);\x0a        \
  const openRepl\
ace = String((r.\
openReplace != n\
ull ? r.openRepl\
ace : (r.openRep\
lace || '')) || \
'');\x0a          c\
onst closeReplac\
e = String((r.cl\
oseReplace != nu\
ll ? r.closeRepl\
ace : (r.closeRe\
place || '')) ||\
 '');\x0a\x0a         \
 const decodeEnt\
ities = (typeof \
r.decodeEntities\
 === 'boolean')\x0a\
            ? r.\
decodeEntities\x0a \
           : ((r\
.name || '').toL\
owerCase() === '\
cmd' || classNam\
e === 'cmd');\x0a\x0a \
         let pha\
seRaw = (typeof \
r.phase === 'str\
ing') ? r.phase.\
toLowerCase() : \
'';\x0a          if\
 (phaseRaw !== '\
source' && phase\
Raw !== 'html' &\
& phaseRaw !== '\
both') phaseRaw \
= '';\x0a          \
const looksLikeF\
ence = (openRepl\
ace.indexOf('```\
') !== -1) || (c\
loseReplace.inde\
xOf('```') !== -\
1);\x0a          co\
nst phase = phas\
eRaw || (looksLi\
keFence ? 'sourc\
e' : 'html');\x0a\x0a \
         const r\
e = new RegExp(U\
tils.reEscape(r.\
open) + '([\x5c\x5cs\x5c\x5c\
S]*?)' + Utils.r\
eEscape(r.close)\
, 'g');\x0a        \
  const reFull =\
 new RegExp('^' \
+ Utils.reEscape\
(r.open) + '([\x5c\x5c\
s\x5c\x5cS]*?)' + Util\
s.reEscape(r.clo\
se) + '$');\x0a    \
      const reFu\
llTrim = new Reg\
Exp('^\x5c\x5cs*' + Ut\
ils.reEscape(r.o\
pen) + '([\x5c\x5cs\x5c\x5cS\
]*?)' + Utils.re\
Escape(r.close) \
+ '\x5c\x5cs*$');\x0a\x0a   \
       const nl2\
br = !!r.nl2br;\x0a\
          const \
allowBr = !!r.al\
lowBr;\x0a\x0a        \
  const item = {\
\x0a            nam\
e: r.name || tag\
,\x0a            ta\
g, className, in\
nerMode,\x0a       \
     open: r.ope\
n, close: r.clos\
e,\x0a            d\
ecodeEntities,\x0a \
           re, r\
eFull, reFullTri\
m,\x0a            s\
tream,\x0a         \
   openReplace, \
closeReplace,\x0a  \
          phase,\
\x0a            isS\
ourceFence: look\
sLikeFence,\x0a    \
        nl2br, a\
llowBr\x0a         \
 };\x0a          co\
mpiled.push(item\
);\x0a          if \
(stream) hasStre\
am = true;\x0a     \
     this._d('CO\
MPILE_RULE', { n\
ame: item.name, \
phase: item.phas\
e, stream: item.\
stream, nl2br: i\
tem.nl2br, allow\
Br: item.allowBr\
 });\x0a        }\x0a\x0a\
        if (comp\
iled.length === \
0) {\x0a          c\
onst open = '[!c\
md]', close = '[\
/!cmd]';\x0a       \
   const item = \
{\x0a            na\
me: 'cmd', tag: \
'p', className: \
'cmd', innerMode\
: 'text', open, \
close,\x0a         \
   decodeEntitie\
s: true,\x0a       \
     re: new Reg\
Exp(Utils.reEsca\
pe(open) + '([\x5c\x5c\
s\x5c\x5cS]*?)' + Util\
s.reEscape(close\
), 'g'),\x0a       \
     reFull: new\
 RegExp('^' + Ut\
ils.reEscape(ope\
n) + '([\x5c\x5cs\x5c\x5cS]*\
?)' + Utils.reEs\
cape(close) + '$\
'),\x0a            \
reFullTrim: new \
RegExp('^\x5c\x5cs*' +\
 Utils.reEscape(\
open) + '([\x5c\x5cs\x5c\x5c\
S]*?)' + Utils.r\
eEscape(close) +\
 '\x5c\x5cs*$'),\x0a     \
       stream: f\
alse,\x0a          \
  openReplace: '\
', closeReplace:\
 '',\x0a           \
 phase: 'html', \
isSourceFence: f\
alse,\x0a          \
  nl2br: false, \
allowBr: false\x0a \
         };\x0a    \
      compiled.p\
ush(item);\x0a     \
     this._d('CO\
MPILE_RULE_FALLB\
ACK', { name: it\
em.name });\x0a    \
    }\x0a\x0a        t\
his.__hasStreamR\
ules = hasStream\
;\x0a        return\
 compiled;\x0a     \
 }\x0a\x0a      transf\
ormSource(src, o\
pts) {\x0a        l\
et s = String(sr\
c || '');\x0a      \
  this.ensureCom\
piled();\x0a       \
 const rules = t\
his.__compiled;\x0a\
        if (!rul\
es || !rules.len\
gth) return s;\x0a\x0a\
        // Candi\
dates: rules tha\
t want source-ph\
ase replacements\
 (``` fences)\x0a  \
      const cand\
idates = [];\x0a   \
     for (let i \
= 0; i < rules.l\
ength; i++) {\x0a  \
        const r \
= rules[i];\x0a    \
      if (!r) co\
ntinue;\x0a        \
  if ((r.phase =\
== 'source' || r\
.phase === 'both\
') && (r.openRep\
lace || r.closeR\
eplace)) candida\
tes.push(r);\x0a   \
     }\x0a        i\
f (!candidates.l\
ength) return s;\
\x0a\x0a        // Avo\
id touching cont\
ent already insi\
de Markdown code\
 fences\x0a        \
const fences = t\
his._findFenceRa\
nges(s);\x0a\x0a      \
  let result = '\
';\x0a        if (!\
fences.length) {\
\x0a          resul\
t = this._applyS\
ourceReplacement\
sInChunk(s, s, 0\
, candidates);\x0a \
       } else {\x0a\
          let ou\
t = '';\x0a        \
  let last = 0;\x0a\
          for (l\
et k = 0; k < fe\
nces.length; k++\
) {\x0a            \
const [a, b] = f\
ences[k];\x0a      \
      if (a > la\
st) {\x0a          \
    const chunk \
= s.slice(last, \
a);\x0a            \
  out += this._a\
pplySourceReplac\
ementsInChunk(s,\
 chunk, last, ca\
ndidates);\x0a     \
       }\x0a       \
     out += s.sl\
ice(a, b);\x0a     \
       last = b;\
\x0a          }\x0a   \
       if (last \
< s.length) {\x0a  \
          const \
tail = s.slice(l\
ast);\x0a          \
  out += this._a\
pplySourceReplac\
ementsInChunk(s,\
 tail, last, can\
didates);\x0a      \
    }\x0a          \
result = out;\x0a  \
      }\x0a\x0a       \
 // NEW: streami\
ng-aware partial\
 injection for u\
nmatched source-\
fence openers.\x0a \
       // If we \
are streaming an\
d see a last ope\
ner without a cl\
oser, convert it\
 to its openRepl\
ace\x0a        // (\
e.g., <execute> \
-> ```python\x5cn) \
so the snapshot \
immediately mate\
rializes a code \
block.\x0a        i\
f (opts && opts.\
streaming === tr\
ue) {\x0a          \
const fenceRules\
 = candidates.fi\
lter(r => !!r.is\
SourceFence);\x0a  \
        if (fenc\
eRules.length) {\
\x0a            res\
ult = this._inje\
ctUnmatchedSourc\
eOpeners(result,\
 fenceRules);\x0a  \
        }\x0a      \
  }\x0a\x0a        ret\
urn result;\x0a    \
  }\x0a\x0a      getSo\
urceFenceSpecs()\
 {\x0a        this.\
ensureCompiled()\
;\x0a        const \
rules = this.__c\
ompiled || [];\x0a \
       const out\
 = [];\x0a        f\
or (let i = 0; i\
 < rules.length;\
 i++) {\x0a        \
  const r = rule\
s[i];\x0a          \
if (!r || !r.isS\
ourceFence) cont\
inue;\x0a          \
if (r.phase !== \
'source' && r.ph\
ase !== 'both') \
continue;\x0a      \
    out.push({ o\
pen: r.open, clo\
se: r.close });\x0a\
        }\x0a      \
  return out;\x0a  \
    }\x0a\x0a      ens\
ureCompiled() {\x0a\
        if (!thi\
s.__compiled) {\x0a\
          this._\
_compiled = this\
.compile(window.\
CUSTOM_MARKUP_RU\
LES || this.cfg.\
CUSTOM_MARKUP_RU\
LES);\x0a          \
this._d('ENSURE_\
COMPILED', { cou\
nt: this.__compi\
led.length, hasS\
tream: this.__ha\
sStreamRules });\
\x0a        }\x0a     \
   return this._\
_compiled;\x0a     \
 }\x0a\x0a      setRul\
es(rules) {\x0a    \
    this.__compi\
led = this.compi\
le(rules);\x0a     \
   window.CUSTOM\
_MARKUP_RULES = \
Array.isArray(ru\
les) ? rules.sli\
ce() : (this.cfg\
.CUSTOM_MARKUP_R\
ULES || []).slic\
e();\x0a        thi\
s._d('SET_RULES'\
, { count: this.\
__compiled.lengt\
h, hasStream: th\
is.__hasStreamRu\
les });\x0a      }\x0a\
      getRules()\
 {\x0a        const\
 list = (window.\
CUSTOM_MARKUP_RU\
LES ? window.CUS\
TOM_MARKUP_RULES\
.slice()\x0a       \
                \
                \
          : (thi\
s.cfg.CUSTOM_MAR\
KUP_RULES || [])\
.slice());\x0a     \
   this._d('GET_\
RULES', { count:\
 list.length });\
\x0a        return \
list;\x0a      }\x0a\x0a \
     hasStreamRu\
les() {\x0a        \
this.ensureCompi\
led();\x0a        r\
eturn !!this.__h\
asStreamRules;\x0a \
     }\x0a\x0a      ha\
sStreamOpenerAtS\
tart(text) {\x0a   \
     if (!text) \
return false;\x0a  \
      this.ensur\
eCompiled();\x0a   \
     const rules\
 = (this.__compi\
led || []).filte\
r(r => !!r.strea\
m);\x0a        if (\
!rules.length) r\
eturn false;\x0a   \
     const t = S\
tring(text).trim\
Start();\x0a       \
 for (let i = 0;\
 i < rules.lengt\
h; i++) {\x0a      \
    const r = ru\
les[i];\x0a        \
  if (!r || !r.o\
pen) continue;\x0a \
         if (t.s\
tartsWith(r.open\
)) return true;\x0a\
        }\x0a      \
  return false;\x0a\
      }\x0a\x0a      i\
sInsideForbidden\
Context(node) {\x0a\
        const p \
= node.parentEle\
ment; if (!p) re\
turn true;\x0a     \
   return !!p.cl\
osest('pre, code\
, kbd, samp, var\
, script, style,\
 textarea, .math\
-pending, .hljs,\
 .code-wrapper, \
ul, ol, li, dl, \
dt, dd');\x0a      \
}\x0a      isInside\
ForbiddenElement\
(el) {\x0a        i\
f (!el) return t\
rue;\x0a        ret\
urn !!el.closest\
('pre, code, kbd\
, samp, var, scr\
ipt, style, text\
area, .math-pend\
ing, .hljs, .cod\
e-wrapper, ul, o\
l, li, dl, dt, d\
d');\x0a      }\x0a\x0a  \
    findNextMatc\
h(text, from, ru\
les) {\x0a        l\
et best = null;\x0a\
        for (con\
st rule of rules\
) {\x0a          ru\
le.re.lastIndex \
= from;\x0a        \
  const m = rule\
.re.exec(text);\x0a\
          if (m)\
 {\x0a            c\
onst start = m.i\
ndex, end = rule\
.re.lastIndex;\x0a \
           if (!\
best || start < \
best.start) best\
 = { rule, start\
, end, inner: m[\
1] || '' };\x0a    \
      }\x0a        \
}\x0a        return\
 best;\x0a      }\x0a \
     findFullMat\
ch(text, rules) \
{\x0a        for (c\
onst rule of rul\
es) {\x0a          \
if (rule.reFull)\
 {\x0a            c\
onst m = rule.re\
Full.exec(text);\
\x0a            if \
(m) return { rul\
e, inner: m[1] |\
| '' };\x0a        \
  } else {\x0a     \
       rule.re.l\
astIndex = 0;\x0a  \
          const \
m = rule.re.exec\
(text);\x0a        \
    if (m && m.i\
ndex === 0 && (r\
ule.re.lastIndex\
 === text.length\
)) {\x0a           \
   const m2 = ru\
le.re.exec(text)\
;\x0a              \
if (!m2) return \
{ rule: rule, in\
ner: m[1] || '' \
};\x0a            }\
\x0a          }\x0a   \
     }\x0a        r\
eturn null;\x0a    \
  }\x0a\x0a      setIn\
nerByMode(el, mo\
de, text, MD, de\
codeEntities = f\
alse, rule = nul\
l) {\x0a        let\
 payload = Strin\
g(text || '');\x0a \
       const wan\
tsBr = !!(rule &\
& (rule.nl2br ||\
 rule.allowBr));\
\x0a\x0a        if (de\
codeEntities && \
payload && paylo\
ad.indexOf('&') \
!== -1) {\x0a      \
    try { payloa\
d = this.decodeE\
ntitiesOnce(payl\
oad); } catch (_\
) {}\x0a        }\x0a\x0a\
        if (want\
sBr) {\x0a         \
 el.innerHTML = \
this._escapeHtml\
AllowBr(payload,\
 { convertNewlin\
es: !!(rule && r\
ule.nl2br) });\x0a \
         return;\
\x0a        }\x0a\x0a    \
    if (mode ===\
 'markdown-inlin\
e' && typeof win\
dow.markdownit !\
== 'undefined') \
{\x0a          try \
{\x0a            if\
 (MD && typeof M\
D.renderInline =\
== 'function') {\
 el.innerHTML = \
MD.renderInline(\
payload); return\
; }\x0a            \
const tempMD = w\
indow.markdownit\
({ html: false, \
linkify: true, b\
reaks: true, hig\
hlight: () => ''\
 });\x0a           \
 el.innerHTML = \
tempMD.renderInl\
ine(payload); re\
turn;\x0a          \
} catch (_) {}\x0a \
       }\x0a       \
 el.textContent \
= payload;\x0a     \
 }\x0a\x0a      _tryRe\
placeFullParagra\
ph(el, rules, MD\
) {\x0a        if (\
!el || el.tagNam\
e !== 'P') retur\
n false;\x0a       \
 if (this.isInsi\
deForbiddenEleme\
nt(el)) {\x0a      \
    this._d('P_S\
KIP_FORBIDDEN', \
{ tag: el.tagNam\
e });\x0a          \
return false;\x0a  \
      }\x0a        \
const t = el.tex\
tContent || '';\x0a\
        if (!thi\
s.hasAnyOpenToke\
n(t, rules)) ret\
urn false;\x0a\x0a    \
    for (const r\
ule of rules) {\x0a\
          if (!r\
ule) continue;\x0a \
         const m\
 = rule.reFullTr\
im ? rule.reFull\
Trim.exec(t) : n\
ull;\x0a          i\
f (!m) continue;\
\x0a\x0a          cons\
t innerText = m[\
1] || '';\x0a      \
    if (rule.pha\
se !== 'html' &&\
 rule.phase !== \
'both') continue\
;\x0a\x0a          if \
(rule.openReplac\
e || rule.closeR\
eplace) {\x0a      \
      const inne\
rHTML = this._ma\
terializeInnerHT\
ML(rule, innerTe\
xt, MD);\x0a       \
     const html \
= String(rule.op\
enReplace || '')\
 + innerHTML + S\
tring(rule.close\
Replace || '');\x0a\
            this\
._replaceElement\
WithHTML(el, htm\
l);\x0a            \
this._d('P_REPLA\
CED_AS_HTML', { \
rule: rule.name \
});\x0a            \
return true;\x0a   \
       }\x0a\x0a      \
    const outTag\
 = (rule.tag && \
typeof rule.tag \
=== 'string') ? \
rule.tag.toLower\
Case() : 'span';\
\x0a          const\
 out = document.\
createElement(ou\
tTag === 'p' ? '\
p' : outTag);\x0a  \
        if (rule\
.className) out.\
className = rule\
.className;\x0a    \
      out.setAtt\
ribute('data-cm'\
, rule.name);\x0a  \
        this.set\
InnerByMode(out,\
 rule.innerMode,\
 innerText, MD, \
!!rule.decodeEnt\
ities, rule);\x0a\x0a \
         try { e\
l.replaceWith(ou\
t); } catch (_) \
{\x0a            co\
nst par = el.par\
entNode; if (par\
) par.replaceChi\
ld(out, el);\x0a   \
       }\x0a       \
   this._d('P_RE\
PLACED', { rule:\
 rule.name, asTa\
g: outTag });\x0a  \
        return t\
rue;\x0a        }\x0a \
       this._d('\
P_NO_FULL_MATCH'\
, { preview: thi\
s.logger.pv(t, 1\
60) });\x0a        \
return false;\x0a  \
    }\x0a\x0a      app\
lyRules(root, MD\
, rules) {\x0a     \
   if (!root || \
!rules || !rules\
.length) return;\
\x0a\x0a        const \
scope = (root.no\
deType === 1 || \
root.nodeType ==\
= 11) ? root : d\
ocument;\x0a\x0a      \
  // Phase 1: to\
lerant <p> repla\
cements\x0a        \
try {\x0a          \
const paragraphs\
 = (typeof scope\
.querySelectorAl\
l === 'function'\
) ? scope.queryS\
electorAll('p') \
: [];\x0a          \
this._d('P_TOLER\
ANT_SCAN_START',\
 { count: paragr\
aphs.length });\x0a\
\x0a          if (p\
aragraphs && par\
agraphs.length) \
{\x0a            fo\
r (let i = 0; i \
< paragraphs.len\
gth; i++) {\x0a    \
          const \
p = paragraphs[i\
];\x0a             \
 if (p && p.getA\
ttribute && p.ge\
tAttribute('data\
-cm')) continue;\
\x0a              c\
onst tc = p && (\
p.textContent ||\
 '');\x0a          \
    if (!tc || !\
this.hasAnyOpenT\
oken(tc, rules))\
 continue;\x0a     \
         if (thi\
s.isInsideForbid\
denElement(p)) c\
ontinue;\x0a       \
       this._try\
ReplaceFullParag\
raph(p, rules, M\
D);\x0a            \
}\x0a          }\x0a  \
      } catch (e\
) {\x0a          th\
is._d('P_TOLERAN\
T_SCAN_ERR', Str\
ing(e));\x0a       \
 }\x0a\x0a        // P\
hase 2: per-text\
-node inline rep\
lacements\x0a      \
  const self = t\
his;\x0a        con\
st walker = docu\
ment.createTreeW\
alker(root, Node\
Filter.SHOW_TEXT\
, {\x0a          ac\
ceptNode: (node)\
 => {\x0a          \
  const val = no\
de && node.nodeV\
alue ? node.node\
Value : '';\x0a    \
        if (!val\
 || !self.hasAny\
OpenToken(val, r\
ules)) return No\
deFilter.FILTER_\
SKIP;\x0a          \
  if (self.isIns\
ideForbiddenCont\
ext(node)) retur\
n NodeFilter.FIL\
TER_REJECT;\x0a    \
        return N\
odeFilter.FILTER\
_ACCEPT;\x0a       \
   }\x0a        });\
\x0a\x0a        let no\
de;\x0a        whil\
e ((node = walke\
r.nextNode())) {\
\x0a          const\
 text = node.nod\
eValue;\x0a        \
  if (!text || !\
this.hasAnyOpenT\
oken(text, rules\
)) continue;\x0a   \
       const par\
ent = node.paren\
tElement;\x0a\x0a     \
     if (parent \
&& parent.tagNam\
e === 'P' && par\
ent.childNodes.l\
ength === 1) {\x0a \
           const\
 fm = this.findF\
ullMatch(text, r\
ules);\x0a         \
   if (fm) {\x0a   \
           if ((\
fm.rule.phase ==\
= 'html' || fm.r\
ule.phase === 'b\
oth') && (fm.rul\
e.openReplace ||\
 fm.rule.closeRe\
place)) {\x0a      \
          const \
innerHTML = this\
._materializeInn\
erHTML(fm.rule, \
fm.inner, MD);\x0a \
               c\
onst html = Stri\
ng(fm.rule.openR\
eplace || '') + \
innerHTML + Stri\
ng(fm.rule.close\
Replace || '');\x0a\
                \
this._replaceEle\
mentWithHTML(par\
ent, html);\x0a    \
            this\
._d('WALKER_FULL\
_REPLACE_HTML', \
{ rule: fm.rule.\
name, preview: t\
his.logger.pv(te\
xt, 160) });\x0a   \
             con\
tinue;\x0a         \
     }\x0a         \
     if (fm.rule\
.tag === 'p') {\x0a\
                \
const out = docu\
ment.createEleme\
nt('p');\x0a       \
         if (fm.\
rule.className) \
out.className = \
fm.rule.classNam\
e;\x0a             \
   out.setAttrib\
ute('data-cm', f\
m.rule.name);\x0a  \
              th\
is.setInnerByMod\
e(out, fm.rule.i\
nnerMode, fm.inn\
er, MD, !!fm.rul\
e.decodeEntities\
, fm.rule);\x0a    \
            try \
{ parent.replace\
With(out); } cat\
ch (_) {\x0a       \
           const\
 par = parent.pa\
rentNode; if (pa\
r) par.replaceCh\
ild(out, parent)\
;\x0a              \
  }\x0a            \
    this._d('WAL\
KER_FULL_REPLACE\
', { rule: fm.ru\
le.name, preview\
: this.logger.pv\
(text, 160) });\x0a\
                \
continue;\x0a      \
        }\x0a      \
      }\x0a        \
  }\x0a\x0a          l\
et i = 0;\x0a      \
    let didRepla\
ce = false;\x0a    \
      const frag\
 = document.crea\
teDocumentFragme\
nt();\x0a\x0a         \
 while (i < text\
.length) {\x0a     \
       const m =\
 this.findNextMa\
tch(text, i, rul\
es);\x0a           \
 if (!m) break;\x0a\
\x0a            if \
(m.start > i) {\x0a\
              fr\
ag.appendChild(d\
ocument.createTe\
xtNode(text.slic\
e(i, m.start)));\
\x0a            }\x0a\x0a\
            if (\
(m.rule.openRepl\
ace || m.rule.cl\
oseReplace) && (\
m.rule.phase ===\
 'html' || m.rul\
e.phase === 'bot\
h')) {\x0a         \
     const inner\
HTML = this._mat\
erializeInnerHTM\
L(m.rule, m.inne\
r, MD);\x0a        \
      const html\
 = String(m.rule\
.openReplace || \
'') + innerHTML \
+ String(m.rule.\
closeReplace || \
'');\x0a           \
   const part = \
this._fragmentFr\
omHTML(html, nod\
e);\x0a            \
  frag.appendChi\
ld(part);\x0a      \
        this._d(\
'WALKER_INLINE_M\
ATCH_HTML', { ru\
le: m.rule.name,\
 start: m.start,\
 end: m.end });\x0a\
              i \
= m.end; didRepl\
ace = true; cont\
inue;\x0a          \
  }\x0a\x0a           \
 if (m.rule.open\
Replace || m.rul\
e.closeReplace) \
{\x0a              \
frag.appendChild\
(document.create\
TextNode(text.sl\
ice(m.start, m.e\
nd)));\x0a         \
     this._d('WA\
LKER_INLINE_SKIP\
_SOURCE_PHASE_HT\
ML', { rule: m.r\
ule.name, start:\
 m.start, end: m\
.end });\x0a       \
       i = m.end\
; didReplace = t\
rue; continue;\x0a \
           }\x0a\x0a  \
          const \
tag = (m.rule.ta\
g === 'p') ? 'sp\
an' : m.rule.tag\
;\x0a            co\
nst el = documen\
t.createElement(\
tag);\x0a          \
  if (m.rule.cla\
ssName) el.class\
Name = m.rule.cl\
assName;\x0a       \
     el.setAttri\
bute('data-cm', \
m.rule.name);\x0a  \
          this.s\
etInnerByMode(el\
, m.rule.innerMo\
de, m.inner, MD,\
 !!m.rule.decode\
Entities, m.rule\
);\x0a            f\
rag.appendChild(\
el);\x0a           \
 this._d('WALKER\
_INLINE_MATCH', \
{ rule: m.rule.n\
ame, start: m.st\
art, end: m.end \
});\x0a\x0a           \
 i = m.end;\x0a    \
        didRepla\
ce = true;\x0a     \
     }\x0a\x0a        \
  if (!didReplac\
e) continue;\x0a   \
       if (i < t\
ext.length) frag\
.appendChild(doc\
ument.createText\
Node(text.slice(\
i)));\x0a\x0a         \
 const parentNod\
e = node.parentN\
ode;\x0a          i\
f (parentNode) {\
\x0a            par\
entNode.replaceC\
hild(frag, node)\
;\x0a            th\
is._d('WALKER_IN\
LINE_DONE', { pr\
eview: this.logg\
er.pv(text, 120)\
 });\x0a          }\
\x0a        }\x0a     \
 }\x0a\x0a      // Pub\
lic API: full pa\
ss (static)\x0a    \
  apply(root, MD\
) {\x0a        this\
.ensureCompiled(\
);\x0a        this.\
applyRules(root,\
 MD, this.__comp\
iled);\x0a      }\x0a\x0a\
      // Public \
API: stream pass\
 (stream-enabled\
 rules only)\x0a   \
   applyStream(r\
oot, MD) {\x0a     \
   this.ensureCo\
mpiled();\x0a      \
  if (!this.__ha\
sStreamRules) re\
turn;\x0a        co\
nst rules = this\
.__compiled.filt\
er(r => !!r.stre\
am);\x0a        if \
(!rules.length) \
return;\x0a\x0a       \
 // 1) Normal ht\
ml-phase replace\
ments where both\
 tokens are with\
in one node/para\
graph\x0a        th\
is.applyRules(ro\
ot, MD, rules);\x0a\
\x0a        // 2) I\
f only opener is\
 present, start \
pending block an\
d wrap the remai\
nder of the snap\
shot\x0a        try\
 { this.applyStr\
eamPartialOpener\
s(root, MD, rule\
s); } catch (_) \
{}\x0a\x0a        // 3\
) If a pending b\
lock exists and \
we can see a clo\
ser now \xe2\x80\x93 fina\
lize immediately\
\x0a        try { t\
his.applyStreamF\
inalizeClosers(r\
oot, rules); } c\
atch (_) {}\x0a    \
  }\x0a\x0a      // St\
reaming: begin p\
ending wrapper w\
hen opener is un\
matched in the s\
ame text node.\x0a \
     applyStream\
PartialOpeners(r\
oot, MD, rulesAl\
l) {\x0a        if \
(!root) return;\x0a\
\x0a        const r\
ules = (rulesAll\
 || []).filter(r\
 =>\x0a          (r\
 && (r.phase ===\
 'html' || r.pha\
se === 'both') &\
& !(r.openReplac\
e || r.closeRepl\
ace) && r.open &\
& r.close)\x0a     \
   );\x0a        if\
 (!rules.length)\
 return;\x0a\x0a      \
  const scope = \
(root.nodeType =\
== 1 || root.nod\
eType === 11) ? \
root : document;\
\x0a        const s\
elf = this;\x0a\x0a   \
     const walke\
r = document.cre\
ateTreeWalker(sc\
ope, NodeFilter.\
SHOW_TEXT, {\x0a   \
       acceptNod\
e(node) {\x0a      \
      const val \
= node && node.n\
odeValue ? node.\
nodeValue : '';\x0a\
            if (\
!val || !self.ha\
sAnyOpenToken(va\
l, rules)) retur\
n NodeFilter.FIL\
TER_SKIP;\x0a      \
      if (self.i\
sInsideForbidden\
Context(node)) r\
eturn NodeFilter\
.FILTER_REJECT;\x0a\
            retu\
rn NodeFilter.FI\
LTER_ACCEPT;\x0a   \
       }\x0a       \
 });\x0a\x0a        le\
t node;\x0a        \
while ((node = w\
alker.nextNode()\
)) {\x0a          c\
onst text = node\
.nodeValue || ''\
;\x0a          if (\
!text) continue;\
\x0a\x0a          let \
best = null; // \
{ rule, start }\x0a\
          for (l\
et i = 0; i < ru\
les.length; i++)\
 {\x0a            c\
onst r = rules[i\
];\x0a            i\
f (!r || !r.open\
 || !r.close) co\
ntinue;\x0a\x0a       \
     const idx =\
 text.lastIndexO\
f(r.open);\x0a     \
       if (idx =\
== -1) continue;\
\x0a\x0a            co\
nst after = text\
.indexOf(r.close\
, idx + r.open.l\
ength);\x0a        \
    if (after !=\
= -1) continue;\x0a\
\x0a            if \
(!best || idx > \
best.start) best\
 = { rule: r, st\
art: idx };\x0a    \
      }\x0a\x0a       \
   if (!best) co\
ntinue;\x0a\x0a       \
   const r = bes\
t.rule;\x0a        \
  const start = \
best.start;\x0a    \
      const open\
Len = r.open.len\
gth;\x0a          c\
onst prefixText \
= text.slice(0, \
start);\x0a        \
  const fromOffs\
et = start + ope\
nLen;\x0a\x0a         \
 try {\x0a         \
   const range =\
 document.create\
Range();\x0a       \
     range.setSt\
art(node, Math.m\
in(fromOffset, n\
ode.nodeValue.le\
ngth));\x0a\x0a       \
     // end to t\
he end of scope\x0a\
            let \
endNode = root;\x0a\
            try \
{\x0a              \
endNode = (root.\
nodeType === 11 \
|| root.nodeType\
 === 1) ? root :\
 node.parentNode\
;\x0a              \
while (endNode &\
& endNode.lastCh\
ild) endNode = e\
ndNode.lastChild\
;\x0a            } \
catch (_) {}\x0a   \
         if (end\
Node && endNode.\
nodeType === 3) \
range.setEnd(end\
Node, endNode.no\
deValue.length);\
\x0a            els\
e if (endNode) r\
ange.setEndAfter\
(endNode);\x0a     \
       else rang\
e.setEndAfter(no\
de);\x0a\x0a          \
  const remainde\
r = range.extrac\
tContents();\x0a\x0a  \
          const \
outTag = (r.tag \
&& typeof r.tag \
=== 'string') ? \
r.tag.toLowerCas\
e() : 'span';\x0a  \
          const \
hostTag = (outTa\
g === 'p') ? 'sp\
an' : outTag;\x0a  \
          const \
el = document.cr\
eateElement(host\
Tag);\x0a          \
  if (r.classNam\
e) el.className \
= r.className;\x0a \
           el.se\
tAttribute('data\
-cm', r.name);\x0a \
           el.se\
tAttribute('data\
-cm-pending', '1\
');\x0a\x0a           \
 el.appendChild(\
remainder);\x0a    \
        range.in\
sertNode(el);\x0a  \
          range.\
detach();\x0a\x0a     \
       try { nod\
e.nodeValue = pr\
efixText; } catc\
h (_) {}\x0a       \
     this._d('ST\
REAM_PENDING_OPE\
N_WRAP_WHOLE', {\
 rule: r.name, o\
pen: r.open, pre\
view: this.logge\
r.pv(text, 160) \
});\x0a\x0a           \
 return; // wrap\
-to-end done; ex\
it to avoid scan\
ning mutated sub\
tree\x0a          }\
 catch (err) {\x0a \
           // Fa\
llback: wrap onl\
y tail of curren\
t node\x0a         \
   try {\x0a       \
       const tai\
l = text.slice(s\
tart + r.open.le\
ngth);\x0a         \
     const frag \
= document.creat\
eDocumentFragmen\
t();\x0a\x0a          \
    if (prefixTe\
xt) frag.appendC\
hild(document.cr\
eateTextNode(pre\
fixText));\x0a\x0a    \
          const \
el = document.cr\
eateElement((r.t\
ag === 'p') ? 's\
pan' : r.tag);\x0a \
             if \
(r.className) el\
.className = r.c\
lassName;\x0a      \
        el.setAt\
tribute('data-cm\
', r.name);\x0a    \
          el.set\
Attribute('data-\
cm-pending', '1'\
);\x0a\x0a            \
  this.setInnerB\
yMode(el, r.inne\
rMode, tail, MD,\
 !!r.decodeEntit\
ies, r);\x0a       \
       frag.appe\
ndChild(el);\x0a\x0a  \
            node\
.parentNode.repl\
aceChild(frag, n\
ode);\x0a          \
    this._d('STR\
EAM_PENDING_OPEN\
_WRAP_FALLBACK',\
 { rule: r.name,\
 err: String(err\
) });\x0a          \
    return;\x0a    \
        } catch \
(_) { /* worst-c\
ase: keep node a\
s-is */ }\x0a      \
    }\x0a        }\x0a\
      }\x0a\x0a      /\
/ NEW: streaming\
 close finalizat\
ion \xe2\x80\x94 if a pen\
ding wrapper con\
tains a closing \
token,\x0a      // \
remove the token\
 and move the ta\
il outside the w\
rapper, then cle\
ar pending flag.\
\x0a      applyStre\
amFinalizeCloser\
s(root, rulesAll\
) {\x0a        if (\
!root) return;\x0a\x0a\
        const sc\
ope = (root.node\
Type === 1 || ro\
ot.nodeType === \
11) ? root : doc\
ument;\x0a        c\
onst pending = s\
cope.querySelect\
orAll('[data-cm]\
[data-cm-pending\
=\x221\x22]');\x0a       \
 if (!pending ||\
 !pending.length\
) return;\x0a\x0a     \
   const rulesBy\
Name = new Map()\
;\x0a        (rules\
All || []).forEa\
ch(r => { if (r \
&& r.name) rules\
ByName.set(r.nam\
e, r); });\x0a\x0a    \
    for (let i =\
 0; i < pending.\
length; i++) {\x0a \
         const e\
l = pending[i];\x0a\
          const \
name = el.getAtt\
ribute('data-cm'\
) || '';\x0a       \
   const rule = \
rulesByName.get(\
name);\x0a         \
 if (!rule || !r\
ule.close) conti\
nue;\x0a\x0a          \
// Look for the \
first text node \
inside 'el' that\
 contains the cl\
osing token\x0a    \
      const self\
 = this;\x0a       \
   const walker \
= document.creat\
eTreeWalker(el, \
NodeFilter.SHOW_\
TEXT, {\x0a        \
    acceptNode(n\
ode) {\x0a         \
     const val =\
 node && node.no\
deValue ? node.n\
odeValue : '';\x0a \
             if \
(!val || val.ind\
exOf(rule.close)\
 === -1) return \
NodeFilter.FILTE\
R_SKIP;\x0a        \
      if (self.i\
sInsideForbidden\
Context(node)) r\
eturn NodeFilter\
.FILTER_REJECT;\x0a\
              re\
turn NodeFilter.\
FILTER_ACCEPT;\x0a \
           }\x0a   \
       });\x0a\x0a    \
      let nodeWi\
thClose = null;\x0a\
          let id\
xInNode = -1;\x0a  \
        let tn;\x0a\
          while \
((tn = walker.ne\
xtNode())) {\x0a   \
         const i\
dx = tn.nodeValu\
e.indexOf(rule.c\
lose);\x0a         \
   if (idx !== -\
1) { nodeWithClo\
se = tn; idxInNo\
de = idx; break;\
 }\x0a          }\x0a \
         if (!no\
deWithClose) con\
tinue; // still \
pending; no clos\
er yet\x0a\x0a        \
  try {\x0a        \
    const tokenL\
en = rule.close.\
length;\x0a\x0a       \
     // Range fo\
r content AFTER \
the closing toke\
n -> will be mov\
ed outside\x0a     \
       const aft\
erRange = docume\
nt.createRange()\
;\x0a            af\
terRange.setStar\
t(nodeWithClose,\
 idxInNode + tok\
enLen);\x0a        \
    // End at en\
d of wrapper\x0a   \
         let end\
Node = el;\x0a     \
       while (en\
dNode && endNode\
.lastChild) endN\
ode = endNode.la\
stChild;\x0a       \
     if (endNode\
 && endNode.node\
Type === 3) afte\
rRange.setEnd(en\
dNode, endNode.n\
odeValue.length)\
;\x0a            el\
se afterRange.se\
tEndAfter(el.las\
tChild || el);\x0a\x0a\
            cons\
t tail = afterRa\
nge.extractConte\
nts();\x0a         \
   afterRange.de\
tach();\x0a\x0a       \
     // Delete t\
he closing token\
 itself\x0a        \
    const tok = \
document.createR\
ange();\x0a        \
    tok.setStart\
(nodeWithClose, \
idxInNode);\x0a    \
        tok.setE\
nd(nodeWithClose\
, idxInNode + to\
kenLen);\x0a       \
     tok.deleteC\
ontents();\x0a     \
       tok.detac\
h();\x0a\x0a          \
  // Move tail r\
ight after the w\
rapper (preserve\
 DOM order)\x0a    \
        if (el.p\
arentNode && tai\
l && tail.childN\
odes.length) {\x0a \
             el.\
parentNode.inser\
tBefore(tail, el\
.nextSibling);\x0a \
           }\x0a\x0a  \
          // Fin\
alize the wrappe\
r\x0a            el\
.removeAttribute\
('data-cm-pendin\
g');\x0a           \
 this._d('STREAM\
_PENDING_CLOSE_F\
INALIZED', { rul\
e: rule.name });\
\x0a\x0a          } ca\
tch (err) {\x0a    \
        this._d(\
'STREAM_PENDING_\
CLOSE_ERR', { ru\
le: rule.name, e\
rr: String(err) \
});\x0a            \
// If anything g\
oes wrong, keep \
the block pendin\
g (safer for nex\
t snapshot).\x0a   \
       }\x0a       \
 }\x0a      }\x0a\x0a    \
  // Source fenc\
e scan (unchange\
d)\x0a      _findFe\
nceRanges(s) {\x0a \
       const ran\
ges = [];\x0a      \
  const n = s.le\
ngth;\x0a        le\
t i = 0;\x0a       \
 let inFence = f\
alse;\x0a        le\
t fenceMark = ''\
;\x0a        let fe\
nceLen = 0;\x0a    \
    let startLin\
eStart = 0;\x0a\x0a   \
     while (i < \
n) {\x0a          c\
onst lineStart =\
 i;\x0a          le\
t j = lineStart;\
\x0a          while\
 (j < n && s.cha\
rCodeAt(j) !== 1\
0 && s.charCodeA\
t(j) !== 13) j++\
;\x0a          cons\
t lineEnd = j;\x0a \
         let nl \
= 0;\x0a          i\
f (j < n) {\x0a    \
        if (s.ch\
arCodeAt(j) === \
13 && j + 1 < n \
&& s.charCodeAt(\
j + 1) === 10) n\
l = 2;\x0a         \
   else nl = 1;\x0a\
          }\x0a\x0a   \
       let k = l\
ineStart;\x0a      \
    let indent =\
 0;\x0a          wh\
ile (k < lineEnd\
) {\x0a            \
const c = s.char\
CodeAt(k);\x0a     \
       if (c ===\
 32) { indent++;\
 if (indent > 3)\
 break; k++; }\x0a \
           else \
if (c === 9) { i\
ndent++; if (ind\
ent > 3) break; \
k++; }\x0a         \
   else break;\x0a \
         }\x0a\x0a    \
      if (!inFen\
ce) {\x0a          \
  if (indent <= \
3 && k < lineEnd\
) {\x0a            \
  const ch = s.c\
harCodeAt(k);\x0a  \
            if (\
ch === 0x60 || c\
h === 0x7E) {\x0a  \
              co\
nst mark = Strin\
g.fromCharCode(c\
h);\x0a            \
    let m = k;\x0a \
               w\
hile (m < lineEn\
d && s.charCodeA\
t(m) === ch) m++\
;\x0a              \
  const run = m \
- k;\x0a           \
     if (run >= \
3) { inFence = t\
rue; fenceMark =\
 mark; fenceLen \
= run; startLine\
Start = lineStar\
t; }\x0a           \
   }\x0a           \
 }\x0a          } e\
lse {\x0a          \
  if (indent <= \
3 && k < lineEnd\
 && s.charCodeAt\
(k) === fenceMar\
k.charCodeAt(0))\
 {\x0a             \
 let m = k;\x0a    \
          while \
(m < lineEnd && \
s.charCodeAt(m) \
=== fenceMark.ch\
arCodeAt(0)) m++\
;\x0a              \
const run = m - \
k;\x0a             \
 if (run >= fenc\
eLen) {\x0a        \
        let only\
WS = true;\x0a     \
           for (\
let t = m; t < l\
ineEnd; t++) {\x0a \
                \
 const cc = s.ch\
arCodeAt(t);\x0a   \
               i\
f (cc !== 32 && \
cc !== 9) { only\
WS = false; brea\
k; }\x0a           \
     }\x0a         \
       if (onlyW\
S) {\x0a           \
       const end\
Idx = lineEnd + \
nl;\x0a            \
      ranges.pus\
h([startLineStar\
t, endIdx]);\x0a   \
               i\
nFence = false; \
fenceMark = ''; \
fenceLen = 0; st\
artLineStart = 0\
;\x0a              \
  }\x0a            \
  }\x0a            \
}\x0a          }\x0a  \
        i = line\
End + nl;\x0a      \
  }\x0a        if (\
inFence) ranges.\
push([startLineS\
tart, n]);\x0a     \
   return ranges\
;\x0a      }\x0a\x0a     \
 _isTopLevelLine\
InSource(s, absI\
dx) {\x0a        le\
t ls = absIdx;\x0a \
       while (ls\
 > 0) {\x0a        \
  const ch = s.c\
harCodeAt(ls - 1\
);\x0a          if \
(ch === 10 || ch\
 === 13) break;\x0a\
          ls--;\x0a\
        }\x0a      \
  const prefix =\
 s.slice(ls, abs\
Idx);\x0a\x0a        l\
et i = 0, indent\
 = 0;\x0a        wh\
ile (i < prefix.\
length) {\x0a      \
    const c = pr\
efix.charCodeAt(\
i);\x0a          if\
 (c === 32) { in\
dent++; if (inde\
nt > 3) break; i\
++; }\x0a          \
else if (c === 9\
) { indent++; if\
 (indent > 3) br\
eak; i++; }\x0a    \
      else break\
;\x0a        }\x0a    \
    if (indent >\
 3) return false\
;\x0a        const \
rest = prefix.sl\
ice(i);\x0a\x0a       \
 if (/^>\x5cs?/.tes\
t(rest)) return \
false;\x0a        i\
f (/^[-+*]\x5cs/.te\
st(rest)) return\
 false;\x0a        \
if (/^\x5cd+[.)]\x5cs/\
.test(rest)) ret\
urn false;\x0a\x0a    \
    if (rest.tri\
m().length > 0) \
return false;\x0a  \
      return tru\
e;\x0a      }\x0a\x0a    \
  _applySourceRe\
placementsInChun\
k(full, chunk, b\
aseOffset, rules\
) {\x0a        let \
t = chunk;\x0a     \
   for (let i = \
0; i < rules.len\
gth; i++) {\x0a    \
      const r = \
rules[i];\x0a      \
    if (!r || !(\
r.openReplace ||\
 r.closeReplace)\
) continue;\x0a    \
      try {\x0a    \
        r.re.las\
tIndex = 0;\x0a    \
        t = t.re\
place(r.re, (mat\
ch, inner, offse\
t) => {\x0a        \
      const abs \
= baseOffset + (\
offset | 0);\x0a   \
           if (!\
this._isTopLevel\
LineInSource(ful\
l, abs)) return \
match;\x0a         \
     const open \
= r.openReplace \
|| '';\x0a         \
     const close\
 = r.closeReplac\
e || '';\x0a       \
       return op\
en + (inner || '\
') + close;\x0a    \
        });\x0a    \
      } catch (_\
) {}\x0a        }\x0a \
       return t;\
\x0a      }\x0a\x0a      \
// === NEW: stre\
aming helper for\
 unmatched sourc\
e-fence openers \
===\x0a      _injec\
tUnmatchedSource\
Openers(text, fe\
nceRules) {\x0a    \
    // Productio\
n-grade guardrai\
ls\x0a        let s\
 = String(text |\
| '');\x0a        i\
f (!s || !fenceR\
ules || !fenceRu\
les.length) retu\
rn s;\x0a\x0a        /\
/ Find the last \
opener (closest \
to the end) that\
 has no matching\
 closer after it\
\x0a        let bes\
t = null; // { r\
, idx }\x0a        \
for (let i = 0; \
i < fenceRules.l\
ength; i++) {\x0a  \
        const r \
= fenceRules[i];\
\x0a          if (!\
r || !r.open || \
!r.close || !r.o\
penReplace) cont\
inue;\x0a\x0a         \
 const idx = s.l\
astIndexOf(r.ope\
n);\x0a          if\
 (idx === -1) co\
ntinue;\x0a\x0a       \
   // Must be to\
p-level line (so\
 that ``` can st\
art a fenced blo\
ck)\x0a          if\
 (!this._isTopLe\
velLineInSource(\
s, idx)) continu\
e;\x0a\x0a          //\
 Ensure there is\
 no closer after\
 this opener in \
the current snap\
shot\x0a          c\
onst after = s.i\
ndexOf(r.close, \
idx + r.open.len\
gth);\x0a          \
if (after !== -1\
) continue;\x0a\x0a   \
       if (!best\
 || idx > best.i\
dx) best = { r, \
idx };\x0a        }\
\x0a\x0a        if (!b\
est) return s;\x0a\x0a\
        const r \
= best.r;\x0a      \
  const i = best\
.idx;\x0a\x0a        /\
/ Replace the ra\
w opener token w\
ith its source-f\
ence open replac\
ement (e.g. ```p\
ython\x5cn)\x0a       \
 // This is ephe\
meral per-snapsh\
ot; underlying b\
uffer remains un\
changed.\x0a       \
 const before = \
s.slice(0, i);\x0a \
       const aft\
er = s.slice(i +\
 r.open.length);\
\x0a        const i\
njected = String\
(r.openReplace |\
| '');\x0a        r\
eturn before + i\
njected + after;\
\x0a      }\x0a    }\x0a\x0a\
  // ===========\
================\
================\
================\
===============\x0a\
  // 5) Markdown\
 runtime (markdo\
wn-it + code wra\
pper + math plac\
eholders)\x0a  // =\
================\
================\
================\
================\
=========\x0a\x0a  cla\
ss MarkdownRende\
rer {\x0a      cons\
tructor(cfg, cus\
tomMarkup, logge\
r, asyncer, raf)\
 {\x0a        this.\
cfg = cfg; this.\
customMarkup = c\
ustomMarkup; thi\
s.MD = null;\x0a   \
     this.logger\
 = logger || new\
 Logger(cfg);\x0a  \
      // Coopera\
tive async utili\
ties available i\
n renderer for h\
eavy decode/rend\
er paths\x0a       \
 this.asyncer = \
asyncer || new A\
syncRunner(cfg, \
raf);\x0a        th\
is.raf = raf || \
null;\x0a\x0a        /\
/ Fast-path stre\
aming renderer w\
ithout linkify t\
o reduce regex w\
ork on hot path.\
\x0a        this.MD\
_STREAM = null;\x0a\
\x0a        this.ho\
oks = {\x0a        \
  observeNewCode\
: () => {},\x0a    \
      observeMsg\
Boxes: () => {},\
\x0a          sched\
uleMathRender: (\
) => {},\x0a       \
   codeScrollIni\
t: () => {}\x0a    \
    };\x0a      }\x0a \
     // Initiali\
ze markdown-it i\
nstances and plu\
gins.\x0a      init\
() {\x0a        if \
(!window.markdow\
nit) { this.logg\
er.log('[MD] mar\
kdown-it not fou\
nd \xe2\x80\x93 rendering\
 skipped.'); ret\
urn; }\x0a        /\
/ Full renderer \
(used for non-ho\
t paths, final r\
esults)\x0a        \
this.MD = window\
.markdownit({ ht\
ml: false, linki\
fy: true, breaks\
: true, highligh\
t: () => '' });\x0a\
        // Strea\
ming renderer (n\
o linkify) \xe2\x80\x93 h\
ot path\x0a        \
this.MD_STREAM =\
 window.markdown\
it({ html: false\
, linkify: false\
, breaks: true, \
highlight: () =>\
 '' });\x0a\x0a       \
 // SAFETY: disa\
ble CommonMark \x22\
indented code bl\
ocks\x22 unless exp\
licitly enabled.\
\x0a        if (!th\
is.cfg.MD || thi\
s.cfg.MD.ALLOW_I\
NDENTED_CODE !==\
 true) {\x0a       \
   try { this.MD\
.block.ruler.dis\
able('code'); } \
catch (_) {}\x0a   \
       try { thi\
s.MD_STREAM.bloc\
k.ruler.disable(\
'code'); } catch\
 (_) {}\x0a        \
}\x0a\x0a        const\
 escapeHtml = Ut\
ils.escapeHtml;\x0a\
\x0a        // Doll\
ar and bracket m\
ath placeholder \
plugins: generat\
e lightweight pl\
aceholders to be\
 picked up by Ka\
TeX later.\x0a     \
   const mathDol\
larPlaceholderPl\
ugin = (md) => {\
\x0a          funct\
ion notEscaped(s\
rc, pos) { let b\
ack = 0; while (\
pos - back - 1 >\
= 0 && src.charC\
odeAt(pos - back\
 - 1) === 0x5C) \
back++; return (\
back % 2) === 0;\
 }\x0a          fun\
ction math_block\
_dollar(state, s\
tartLine, endLin\
e, silent) {\x0a   \
         const p\
os = state.bMark\
s[startLine] + s\
tate.tShift[star\
tLine];\x0a        \
    const max = \
state.eMarks[sta\
rtLine];\x0a       \
     if (pos + 1\
 >= max) return \
false;\x0a         \
   if (state.src\
.charCodeAt(pos)\
 !== 0x24 || sta\
te.src.charCodeA\
t(pos + 1) !== 0\
x24) return fals\
e;\x0a            l\
et nextLine = st\
artLine + 1, fou\
nd = false;\x0a    \
        for (; n\
extLine < endLin\
e; nextLine++) {\
\x0a              l\
et p = state.bMa\
rks[nextLine] + \
state.tShift[nex\
tLine];\x0a        \
      const pe =\
 state.eMarks[ne\
xtLine];\x0a       \
       if (p + 1\
 < pe && state.s\
rc.charCodeAt(p)\
 === 0x24 && sta\
te.src.charCodeA\
t(p + 1) === 0x2\
4) { found = tru\
e; break; }\x0a    \
        }\x0a      \
      if (!found\
) return false;\x0a\
            if (\
silent) return t\
rue;\x0a\x0a          \
  const contentS\
tart = state.bMa\
rks[startLine] +\
 state.tShift[st\
artLine] + 2;\x0a  \
          const \
contentEndLine =\
 nextLine - 1;\x0a \
           let c\
ontent = '';\x0a   \
         if (con\
tentEndLine >= s\
tartLine + 1) {\x0a\
              co\
nst startIdx = s\
tate.bMarks[star\
tLine + 1];\x0a    \
          const \
endIdx = state.e\
Marks[contentEnd\
Line];\x0a         \
     content = s\
tate.src.slice(s\
tartIdx, endIdx)\
;\x0a            } \
else content = '\
';\x0a\x0a            \
const token = st\
ate.push('math_b\
lock_dollar', ''\
, 0);\x0a          \
  token.block = \
true; token.cont\
ent = content; s\
tate.line = next\
Line + 1; return\
 true;\x0a         \
 }\x0a          fun\
ction math_inlin\
e_dollar(state, \
silent) {\x0a      \
      const pos \
= state.pos, src\
 = state.src, ma\
x = state.posMax\
;\x0a            if\
 (pos >= max) re\
turn false;\x0a    \
        if (src.\
charCodeAt(pos) \
!== 0x24) return\
 false;\x0a        \
    if (pos + 1 \
< max && src.cha\
rCodeAt(pos + 1)\
 === 0x24) retur\
n false;\x0a       \
     const after\
 = pos + 1 < max\
 ? src.charCodeA\
t(pos + 1) : 0;\x0a\
            if (\
after === 0x20 |\
| after === 0x0A\
 || after === 0x\
0D) return false\
;\x0a            le\
t i = pos + 1;\x0a \
           while\
 (i < max) {\x0a   \
           const\
 ch = src.charCo\
deAt(i);\x0a       \
       if (ch ==\
= 0x24 && notEsc\
aped(src, i)) {\x0a\
                \
const before = i\
 - 1 >= 0 ? src.\
charCodeAt(i - 1\
) : 0;\x0a         \
       if (befor\
e === 0x20 || be\
fore === 0x0A ||\
 before === 0x0D\
) { i++; continu\
e; }\x0a           \
     break;\x0a    \
          }\x0a    \
          i++;\x0a \
           }\x0a   \
         if (i >\
= max || src.cha\
rCodeAt(i) !== 0\
x24) return fals\
e;\x0a\x0a            \
if (!silent) {\x0a \
             con\
st token = state\
.push('math_inli\
ne_dollar', '', \
0);\x0a            \
  token.block = \
false; token.con\
tent = src.slice\
(pos + 1, i);\x0a  \
          }\x0a    \
        state.po\
s = i + 1; retur\
n true;\x0a        \
  }\x0a\x0a          m\
d.block.ruler.be\
fore('fence', 'm\
ath_block_dollar\
', math_block_do\
llar, { alt: ['p\
aragraph', 'refe\
rence', 'blockqu\
ote', 'list'] })\
;\x0a          md.i\
nline.ruler.befo\
re('escape', 'ma\
th_inline_dollar\
', math_inline_d\
ollar);\x0a\x0a       \
   md.renderer.r\
ules.math_inline\
_dollar = (token\
s, idx) => {\x0a   \
         const t\
ex = tokens[idx]\
.content || '';\x0a\
            retu\
rn `<span class=\
\x22math-pending\x22 d\
ata-display=\x220\x22>\
<span class=\x22mat\
h-fallback\x22>$${e\
scapeHtml(tex)}$\
</span><script t\
ype=\x22math/tex\x22>$\
{escapeHtml(tex)\
}</script></span\
>`;\x0a          };\
\x0a          md.re\
nderer.rules.mat\
h_block_dollar =\
 (tokens, idx) =\
> {\x0a            \
const tex = toke\
ns[idx].content \
|| '';\x0a         \
   return `<div \
class=\x22math-pend\
ing\x22 data-displa\
y=\x221\x22><div class\
=\x22math-fallback\x22\
>$$${escapeHtml(\
tex)}$$</div><sc\
ript type=\x22math/\
tex; mode=displa\
y\x22>${escapeHtml(\
tex)}</script></\
div>`;\x0a         \
 };\x0a        };\x0a\x0a\
        const ma\
thBracketsPlaceh\
olderPlugin = (m\
d) => {\x0a        \
  function math_\
brackets(state, \
silent) {\x0a      \
      const src \
= state.src, pos\
 = state.pos, ma\
x = state.posMax\
;\x0a            if\
 (pos + 1 >= max\
 || src.charCode\
At(pos) !== 0x5C\
) return false;\x0a\
            cons\
t next = src.cha\
rCodeAt(pos + 1)\
;\x0a            if\
 (next !== 0x28 \
&& next !== 0x5B\
) return false;\x0a\
            cons\
t isInline = (ne\
xt === 0x28); co\
nst close = isIn\
line ? '\x5c\x5c)' : '\
\x5c\x5c]';\x0a          \
  const start = \
pos + 2; const e\
nd = src.indexOf\
(close, start);\x0a\
            if (\
end < 0) return \
false;\x0a         \
   const content\
 = src.slice(sta\
rt, end);\x0a      \
      if (!silen\
t) {\x0a           \
   const t = sta\
te.push(isInline\
 ? 'math_inline_\
bracket' : 'math\
_block_bracket',\
 '', 0);\x0a       \
       t.content\
 = content; t.bl\
ock = !isInline;\
\x0a            }\x0a \
           state\
.pos = end + 2; \
return true;\x0a   \
       }\x0a       \
   md.inline.rul\
er.before('escap\
e', 'math_bracke\
ts', math_bracke\
ts);\x0a          m\
d.renderer.rules\
.math_inline_bra\
cket = (tokens, \
idx) => {\x0a      \
      const tex \
= tokens[idx].co\
ntent || '';\x0a   \
         return \
`<span class=\x22ma\
th-pending\x22 data\
-display=\x220\x22><sp\
an class=\x22math-f\
allback\x22>\x5c\x5c(${es\
capeHtml(tex)}\x5c\x5c\
)</span><script \
type=\x22math/tex\x22>\
${escapeHtml(tex\
)}</script></spa\
n>`;\x0a          }\
;\x0a          md.r\
enderer.rules.ma\
th_block_bracket\
 = (tokens, idx)\
 => {\x0a          \
  const tex = to\
kens[idx].conten\
t || '';\x0a       \
     return `<di\
v class=\x22math-pe\
nding\x22 data-disp\
lay=\x221\x22><div cla\
ss=\x22math-fallbac\
k\x22>\x5c\x5c[${escapeHt\
ml(tex)}\x5c\x5c]</div\
><script type=\x22m\
ath/tex; mode=di\
splay\x22>${escapeH\
tml(tex)}</scrip\
t></div>`;\x0a     \
     };\x0a        \
};\x0a\x0a        this\
.MD.use(mathDoll\
arPlaceholderPlu\
gin);\x0a        th\
is.MD.use(mathBr\
acketsPlaceholde\
rPlugin);\x0a      \
  this.MD_STREAM\
.use(mathDollarP\
laceholderPlugin\
);\x0a        this.\
MD_STREAM.use(ma\
thBracketsPlaceh\
olderPlugin);\x0a\x0a \
       const cfg\
 = this.cfg; con\
st logger = this\
.logger;\x0a\x0a      \
  // STREAMING w\
rapper plugin (m\
odified header l\
abel guard)\x0a    \
    (function co\
deWrapperPlugin(\
md, logger) {\x0a  \
        let CODE\
_IDX = 1;\x0a      \
    const log = \
(line, ctx) => l\
ogger.debug('MD_\
LANG', line, ctx\
);\x0a\x0a          co\
nst DEDUP = (win\
dow.MD_LANG_LOG_\
DEDUP !== false)\
;\x0a          cons\
t seenFP = new S\
et();\x0a          \
const makeFP = (\
info, raw) => {\x0a\
            cons\
t head = (raw ||\
 '').slice(0, 96\
);\x0a            r\
eturn String(inf\
o || '') + '|' +\
 String((raw || \
'').length) + '|\
' + head;\x0a      \
    };\x0a\x0a        \
  const ALIAS = \
{\x0a            tx\
t: 'plaintext', \
text: 'plaintext\
', plaintext: 'p\
laintext',\x0a     \
       sh: 'bash\
', shell: 'bash'\
, zsh: 'bash', '\
shell-session': \
'bash',\x0a        \
    py: 'python'\
, python3: 'pyth\
on', py3: 'pytho\
n',\x0a            \
js: 'javascript'\
, node: 'javascr\
ipt', nodejs: 'j\
avascript',\x0a    \
        ts: 'typ\
escript', 'ts-no\
de': 'typescript\
',\x0a            y\
ml: 'yaml', kt: \
'kotlin', rs: 'r\
ust',\x0a          \
  csharp: 'cshar\
p', 'c#': 'cshar\
p', 'c++': 'cpp'\
,\x0a            ps\
: 'powershell', \
ps1: 'powershell\
', pwsh: 'powers\
hell', powershel\
l7: 'powershell'\
,\x0a            do\
cker: 'dockerfil\
e'\x0a          };\x0a\
          functi\
on normLang(s) {\
 if (!s) return \
''; const v = St\
ring(s).trim().t\
oLowerCase(); re\
turn ALIAS[v] ||\
 v; }\x0a          \
function isSuppo\
rtedByHLJS(lang)\
 { try { return \
!!(window.hljs &\
& hljs.getLangua\
ge && hljs.getLa\
nguage(lang)); }\
 catch (_) { ret\
urn false; } }\x0a \
         functio\
n classForHighli\
ght(lang) { if (\
!lang) return 'p\
laintext'; retur\
n isSupportedByH\
LJS(lang) ? lang\
 : 'plaintext'; \
}\x0a          func\
tion stripBOM(s)\
 { return (s && \
s.charCodeAt(0) \
=== 0xFEFF) ? s.\
slice(1) : s; }\x0a\
\x0a          funct\
ion detectFromFi\
rstLine(raw, rid\
) {\x0a            \
if (!raw) return\
 { lang: '', con\
tent: raw, isOut\
put: false };\x0a  \
          const \
lines = raw.spli\
t(/\x5cr?\x5cn/);\x0a    \
        if (!lin\
es.length) retur\
n { lang: '', co\
ntent: raw, isOu\
tput: false };\x0a \
           let i\
 = 0; while (i <\
 lines.length &&\
 !lines[i].trim(\
)) i++;\x0a        \
    if (i >= lin\
es.length) { log\
(`#${rid} first-\
line: only white\
space`); return \
{ lang: '', cont\
ent: raw, isOutp\
ut: false }; }\x0a \
           let f\
irst = stripBOM(\
lines[i]).trim()\
;\x0a            fi\
rst = first.repl\
ace(/^\x5cs*lang(?:\
uage)?\x5cs*[:=]\x5cs*\
/i, '').trim();\x0a\
            let \
token = first.sp\
lit(/\x5cs+/)[0].re\
place(/:$/, '');\
\x0a            if \
(!/^[A-Za-z][\x5cw#\
+\x5c-\x5c.]{0,30}$/.t\
est(token)) { lo\
g(`#${rid} first\
-line: no token \
match`, { first \
}); return { lan\
g: '', content: \
raw, isOutput: f\
alse }; }\x0a      \
      let cand =\
 normLang(token)\
;\x0a            if\
 (cand === 'outp\
ut') {\x0a         \
     const conte\
nt = lines.slice\
(i + 1).join('\x5cn\
');\x0a            \
  log(`#${rid} f\
irst-line: outpu\
t header`);\x0a    \
          return\
 { lang: 'python\
', headerLabel: \
'output', conten\
t, isOutput: tru\
e };\x0a           \
 }\x0a            c\
onst rest = line\
s.slice(i + 1).j\
oin('\x5cn');\x0a     \
       if (!rest\
.trim()) { log(`\
#${rid} first-li\
ne: directive bu\
t no content aft\
er, ignore`, { c\
and }); return {\
 lang: '', conte\
nt: raw, isOutpu\
t: false }; }\x0a  \
          log(`#\
${rid} first-lin\
e: directive acc\
epted`, { cand, \
restLen: rest.le\
ngth, hljs: isSu\
pportedByHLJS(ca\
nd) });\x0a        \
    return { lan\
g: cand, headerL\
abel: cand, cont\
ent: rest, isOut\
put: false };\x0a  \
        }\x0a\x0a     \
     md.renderer\
.rules.fence = (\
tokens, idx) => \
renderFence(toke\
ns[idx]);\x0a      \
    md.renderer.\
rules.code_block\
 = (tokens, idx)\
 => renderFence(\
{ info: '', cont\
ent: tokens[idx]\
.content || '' }\
);\x0a\x0a          fu\
nction resolveLa\
nguageAndContent\
(info, raw, rid)\
 {\x0a            c\
onst infoLangRaw\
 = (info || '').\
trim().split(/\x5cs\
+/)[0] || '';\x0a  \
          let ca\
nd = normLang(in\
foLangRaw);\x0a    \
        if (cand\
 === 'output') {\
\x0a              l\
og(`#${rid} info\
: output header`\
);\x0a             \
 return { lang: \
'python', header\
Label: 'output',\
 content: raw, i\
sOutput: true };\
\x0a            }\x0a \
           if (c\
and) {\x0a         \
     log(`#${rid\
} info: token`, \
{ infoLangRaw, c\
and, hljs: isSup\
portedByHLJS(can\
d) });\x0a         \
     return { la\
ng: cand, header\
Label: cand, con\
tent: raw, isOut\
put: false };\x0a  \
          }\x0a    \
        const de\
t = detectFromFi\
rstLine(raw, rid\
);\x0a            i\
f (det && (det.l\
ang || det.isOut\
put)) return det\
;\x0a            lo\
g(`#${rid} resol\
ve: fallback`);\x0a\
            retu\
rn { lang: '', h\
eaderLabel: 'cod\
e', content: raw\
, isOutput: fals\
e };\x0a          }\
\x0a\x0a          func\
tion renderFence\
(token) {\x0a      \
      const raw \
= token.content \
|| '';\x0a         \
   const rid = S\
tring(CODE_IDX +\
 '');\x0a          \
  const fp = mak\
eFP(token.info |\
| '', raw);\x0a    \
        const ca\
nLog = !DEDUP ||\
 !seenFP.has(fp)\
;\x0a            if\
 (canLog) log(`F\
ENCE_ENTER #${ri\
d}`, { info: (to\
ken.info || ''),\
 rawHead: logger\
.pv(raw) });\x0a\x0a  \
          const \
res = resolveLan\
guageAndContent(\
token.info || ''\
, raw, rid);\x0a   \
         const i\
sOutput = !!res.\
isOutput;\x0a\x0a     \
       // Choose\
 class and a saf\
e header label (\
avoid 'on', 'ml'\
, 's' etc.)\x0a    \
        const ra\
wToken = (res.la\
ng || '').trim()\
;\x0a            co\
nst langClass = \
isOutput ? 'pyth\
on' : classForHi\
ghlight(rawToken\
);\x0a\x0a            \
// Guard against\
 tiny unsupporte\
d tokens \xe2\x80\x93 sho\
w temporary 'cod\
e' instead of pa\
rtial suffix.\x0a  \
          let he\
aderLabel = isOu\
tput ? 'output' \
: (res.headerLab\
el || (rawToken \
|| 'code'));\x0a   \
         if (!is\
Output) {\x0a      \
        if (rawT\
oken && !isSuppo\
rtedByHLJS(rawTo\
ken) && rawToken\
.length < 3) {\x0a \
               h\
eaderLabel = 'co\
de';\x0a           \
   }\x0a           \
 }\x0a\x0a            \
if (canLog) {\x0a  \
            log(\
`FENCE_RESOLVE #\
${rid}`, { heade\
rLabel, langToke\
n: (res.lang || \
''), langClass, \
hljsSupported: i\
sSupportedByHLJS\
(res.lang || '')\
, contentLen: (r\
es.content || ''\
).length });\x0a   \
           if (D\
EDUP) seenFP.add\
(fp);\x0a          \
  }\x0a\x0a           \
 // precompute c\
ode meta to avoi\
d expensive .tex\
tContent on next\
 phases\x0a        \
    const conten\
t = res.content \
|| '';\x0a         \
   const len = c\
ontent.length;\x0a \
           const\
 head = content.\
slice(0, 64);\x0a  \
          const \
tail = content.s\
lice(-64);\x0a     \
       const hea\
dEsc = Utils.esc\
apeHtml(head);\x0a \
           const\
 tailEsc = Utils\
.escapeHtml(tail\
);\x0a            /\
/ Note: for full\
 renderer we wil\
l also persist d\
ata-code-nl (see\
 below).\x0a\x0a      \
      const inne\
r = Utils.escape\
Html(content);\x0a \
           const\
 idxLocal = CODE\
_IDX++;\x0a\x0a       \
     let actions\
 = '';\x0a         \
   if (langClass\
 === 'html') {\x0a \
             act\
ions += `<a href\
=\x22empty:${idxLoc\
al}\x22 class=\x22code\
-header-action c\
ode-header-previ\
ew\x22><img src=\x22${\
cfg.ICONS.CODE_P\
REVIEW}\x22 class=\x22\
action-img\x22 data\
-id=\x22${idxLocal}\
\x22><span>${Utils.\
escapeHtml(cfg.L\
OCALE.PREVIEW)}<\
/span></a>`;\x0a   \
         } else \
if (langClass ==\
= 'python' && he\
aderLabel !== 'o\
utput') {\x0a      \
        actions \
+= `<a href=\x22emp\
ty:${idxLocal}\x22 \
class=\x22code-head\
er-action code-h\
eader-run\x22><img \
src=\x22${cfg.ICONS\
.CODE_RUN}\x22 clas\
s=\x22action-img\x22 d\
ata-id=\x22${idxLoc\
al}\x22><span>${Uti\
ls.escapeHtml(cf\
g.LOCALE.RUN)}</\
span></a>`;\x0a    \
        }\x0a      \
      actions +=\
 `<a href=\x22empty\
:${idxLocal}\x22 cl\
ass=\x22code-header\
-action code-hea\
der-collapse\x22><i\
mg src=\x22${cfg.IC\
ONS.CODE_MENU}\x22 \
class=\x22action-im\
g\x22 data-id=\x22${id\
xLocal}\x22><span>$\
{Utils.escapeHtm\
l(cfg.LOCALE.COL\
LAPSE)}</span></\
a>`;\x0a           \
 actions += `<a \
href=\x22empty:${id\
xLocal}\x22 class=\x22\
code-header-acti\
on code-header-c\
opy\x22><img src=\x22$\
{cfg.ICONS.CODE_\
COPY}\x22 class=\x22ac\
tion-img\x22 data-i\
d=\x22${idxLocal}\x22>\
<span>${Utils.es\
capeHtml(cfg.LOC\
ALE.COPY)}</span\
></a>`;\x0a\x0a       \
     // attach p\
recomputed meta \
(len/head/tail) \
on wrapper for d\
ownstream optimi\
zations\x0a        \
    return (\x0a   \
           `<div\
 class=\x22code-wra\
pper highlight\x22 \
data-index=\x22${id\
xLocal}\x22` +\x0a    \
          ` data\
-code-lang=\x22${Ut\
ils.escapeHtml(r\
es.lang || '')}\x22\
` +\x0a            \
  ` data-code-le\
n=\x22${String(len)\
}\x22 data-code-hea\
d=\x22${headEsc}\x22 d\
ata-code-tail=\x22$\
{tailEsc}\x22` + //\
 meta (no nl her\
e \xe2\x80\x93 only in fu\
ll renderer)\x0a   \
           ` dat\
a-locale-collaps\
e=\x22${Utils.escap\
eHtml(cfg.LOCALE\
.COLLAPSE)}\x22 dat\
a-locale-expand=\
\x22${Utils.escapeH\
tml(cfg.LOCALE.E\
XPAND)}\x22` +\x0a    \
          ` data\
-locale-copy=\x22${\
Utils.escapeHtml\
(cfg.LOCALE.COPY\
)}\x22 data-locale-\
copied=\x22${Utils.\
escapeHtml(cfg.L\
OCALE.COPIED)}\x22 \
data-style=\x22${Ut\
ils.escapeHtml(c\
fg.CODE_STYLE)}\x22\
>` +\x0a           \
     `<p class=\x22\
code-header-wrap\
per\x22><span><span\
 class=\x22code-hea\
der-lang\x22>${Util\
s.escapeHtml(hea\
derLabel)}   </s\
pan>${actions}</\
span></p>` +\x0a   \
             `<p\
re><code class=\x22\
language-${Utils\
.escapeHtml(lang\
Class)} hljs\x22>${\
inner}</code></p\
re>` +\x0a         \
     `</div>`\x0a  \
          );\x0a   \
       }\x0a       \
 })(this.MD_STRE\
AM, this.logger)\
;\x0a\x0a        // FU\
LL renderer wrap\
per plugin (modi\
fied header labe\
l guard)\x0a       \
 (function codeW\
rapperPlugin(md,\
 logger) {\x0a     \
     // identica\
l core logic \xe2\x80\x93\
 augmented with \
data-code-nl for\
 full renderer\x0a \
         let COD\
E_IDX = 1;\x0a     \
     const log =\
 (line, ctx) => \
logger.debug('MD\
_LANG', line, ct\
x);\x0a\x0a          c\
onst DEDUP = (wi\
ndow.MD_LANG_LOG\
_DEDUP !== false\
);\x0a          con\
st seenFP = new \
Set();\x0a         \
 const makeFP = \
(info, raw) => {\
\x0a            con\
st head = (raw |\
| '').slice(0, 9\
6);\x0a            \
return String(in\
fo || '') + '|' \
+ String((raw ||\
 '').length) + '\
|' + head;\x0a     \
     };\x0a\x0a       \
   const ALIAS =\
 {\x0a            t\
xt: 'plaintext',\
 text: 'plaintex\
t', plaintext: '\
plaintext',\x0a    \
        sh: 'bas\
h', shell: 'bash\
', zsh: 'bash', \
'shell-session':\
 'bash',\x0a       \
     py: 'python\
', python3: 'pyt\
hon', py3: 'pyth\
on',\x0a           \
 js: 'javascript\
', node: 'javasc\
ript', nodejs: '\
javascript',\x0a   \
         ts: 'ty\
pescript', 'ts-n\
ode': 'typescrip\
t',\x0a            \
yml: 'yaml', kt:\
 'kotlin', rs: '\
rust',\x0a         \
   csharp: 'csha\
rp', 'c#': 'csha\
rp', 'c++': 'cpp\
',\x0a            p\
s: 'powershell',\
 ps1: 'powershel\
l', pwsh: 'power\
shell', powershe\
ll7: 'powershell\
',\x0a            d\
ocker: 'dockerfi\
le'\x0a          };\
\x0a          funct\
ion normLang(s) \
{ if (!s) return\
 ''; const v = S\
tring(s).trim().\
toLowerCase(); r\
eturn ALIAS[v] |\
| v; }\x0a         \
 function isSupp\
ortedByHLJS(lang\
) { try { return\
 !!(window.hljs \
&& hljs.getLangu\
age && hljs.getL\
anguage(lang)); \
} catch (_) { re\
turn false; } }\x0a\
          functi\
on classForHighl\
ight(lang) { if \
(!lang) return '\
plaintext'; retu\
rn isSupportedBy\
HLJS(lang) ? lan\
g : 'plaintext';\
 }\x0a          fun\
ction stripBOM(s\
) { return (s &&\
 s.charCodeAt(0)\
 === 0xFEFF) ? s\
.slice(1) : s; }\
\x0a\x0a          func\
tion detectFromF\
irstLine(raw, ri\
d) {\x0a           \
 if (!raw) retur\
n { lang: '', co\
ntent: raw, isOu\
tput: false };\x0a \
           const\
 lines = raw.spl\
it(/\x5cr?\x5cn/);\x0a   \
         if (!li\
nes.length) retu\
rn { lang: '', c\
ontent: raw, isO\
utput: false };\x0a\
            let \
i = 0; while (i \
< lines.length &\
& !lines[i].trim\
()) i++;\x0a       \
     if (i >= li\
nes.length) { lo\
g(`#${rid} first\
-line: only whit\
espace`); return\
 { lang: '', con\
tent: raw, isOut\
put: false }; }\x0a\
            let \
first = stripBOM\
(lines[i]).trim(\
);\x0a            f\
irst = first.rep\
lace(/^\x5cs*lang(?\
:uage)?\x5cs*[:=]\x5cs\
*/i, '').trim();\
\x0a            let\
 token = first.s\
plit(/\x5cs+/)[0].r\
eplace(/:$/, '')\
;\x0a            if\
 (!/^[A-Za-z][\x5cw\
#+\x5c-\x5c.]{0,30}$/.\
test(token)) { l\
og(`#${rid} firs\
t-line: no token\
 match`, { first\
 }); return { la\
ng: '', content:\
 raw, isOutput: \
false }; }\x0a     \
       let cand \
= normLang(token\
);\x0a            i\
f (cand === 'out\
put') {\x0a        \
      const cont\
ent = lines.slic\
e(i + 1).join('\x5c\
n');\x0a           \
   log(`#${rid} \
first-line: outp\
ut header`);\x0a   \
           retur\
n { lang: 'pytho\
n', headerLabel:\
 'output', conte\
nt, isOutput: tr\
ue };\x0a          \
  }\x0a            \
const rest = lin\
es.slice(i + 1).\
join('\x5cn');\x0a    \
        if (!res\
t.trim()) { log(\
`#${rid} first-l\
ine: directive b\
ut no content af\
ter, ignore`, { \
cand }); return \
{ lang: '', cont\
ent: raw, isOutp\
ut: false }; }\x0a \
           log(`\
#${rid} first-li\
ne: directive ac\
cepted`, { cand,\
 restLen: rest.l\
ength, hljs: isS\
upportedByHLJS(c\
and) });\x0a       \
     return { la\
ng: cand, header\
Label: cand, con\
tent: rest, isOu\
tput: false };\x0a \
         }\x0a\x0a    \
      md.rendere\
r.rules.fence = \
(tokens, idx) =>\
 renderFence(tok\
ens[idx]);\x0a     \
     md.renderer\
.rules.code_bloc\
k = (tokens, idx\
) => renderFence\
({ info: '', con\
tent: tokens[idx\
].content || '' \
});\x0a\x0a          f\
unction resolveL\
anguageAndConten\
t(info, raw, rid\
) {\x0a            \
const infoLangRa\
w = (info || '')\
.trim().split(/\x5c\
s+/)[0] || '';\x0a \
           let c\
and = normLang(i\
nfoLangRaw);\x0a   \
         if (can\
d === 'output') \
{\x0a              \
log(`#${rid} inf\
o: output header\
`);\x0a            \
  return { lang:\
 'python', heade\
rLabel: 'output'\
, content: raw, \
isOutput: true }\
;\x0a            }\x0a\
            if (\
cand) {\x0a        \
      log(`#${ri\
d} info: token`,\
 { infoLangRaw, \
cand, hljs: isSu\
pportedByHLJS(ca\
nd) });\x0a        \
      return { l\
ang: cand, heade\
rLabel: cand, co\
ntent: raw, isOu\
tput: false };\x0a \
           }\x0a   \
         const d\
et = detectFromF\
irstLine(raw, ri\
d);\x0a            \
if (det && (det.\
lang || det.isOu\
tput)) return de\
t;\x0a            l\
og(`#${rid} reso\
lve: fallback`);\
\x0a            ret\
urn { lang: '', \
headerLabel: 'co\
de', content: ra\
w, isOutput: fal\
se };\x0a          \
}\x0a\x0a          fun\
ction renderFenc\
e(token) {\x0a     \
       const raw\
 = token.content\
 || '';\x0a        \
    const rid = \
String(CODE_IDX \
+ '');\x0a         \
   const fp = ma\
keFP(token.info \
|| '', raw);\x0a   \
         const c\
anLog = !DEDUP |\
| !seenFP.has(fp\
);\x0a            i\
f (canLog) log(`\
FENCE_ENTER #${r\
id}`, { info: (t\
oken.info || '')\
, rawHead: logge\
r.pv(raw) });\x0a\x0a \
           const\
 res = resolveLa\
nguageAndContent\
(token.info || '\
', raw, rid);\x0a  \
          const \
isOutput = !!res\
.isOutput;\x0a\x0a    \
        // Choos\
e class and a sa\
fe header label \
(avoid 'on', 'ml\
', 's' etc.)\x0a   \
         const r\
awToken = (res.l\
ang || '').trim(\
);\x0a            c\
onst langClass =\
 isOutput ? 'pyt\
hon' : classForH\
ighlight(rawToke\
n);\x0a\x0a           \
 // Guard agains\
t tiny unsupport\
ed tokens \xe2\x80\x93 sh\
ow temporary 'co\
de' instead of p\
artial suffix.\x0a \
           let h\
eaderLabel = isO\
utput ? 'output'\
 : (res.headerLa\
bel || (rawToken\
 || 'code'));\x0a  \
          if (!i\
sOutput) {\x0a     \
         if (raw\
Token && !isSupp\
ortedByHLJS(rawT\
oken) && rawToke\
n.length < 3) {\x0a\
                \
headerLabel = 'c\
ode';\x0a          \
    }\x0a          \
  }\x0a\x0a           \
 if (canLog) {\x0a \
             log\
(`FENCE_RESOLVE \
#${rid}`, { head\
erLabel, langTok\
en: (res.lang ||\
 ''), langClass,\
 hljsSupported: \
isSupportedByHLJ\
S(res.lang || ''\
), contentLen: (\
res.content || '\
').length });\x0a  \
            if (\
DEDUP) seenFP.ad\
d(fp);\x0a         \
   }\x0a\x0a          \
  // precompute \
code meta\x0a      \
      const cont\
ent = res.conten\
t || '';\x0a       \
     const len =\
 content.length;\
\x0a            con\
st head = conten\
t.slice(0, 64);\x0a\
            cons\
t tail = content\
.slice(-64);\x0a   \
         const h\
eadEsc = Utils.e\
scapeHtml(head);\
\x0a            con\
st tailEsc = Uti\
ls.escapeHtml(ta\
il);\x0a           \
 const nl = Util\
s.countNewlines(\
content);\x0a\x0a     \
       const inn\
er = Utils.escap\
eHtml(content);\x0a\
            cons\
t idxLocal = COD\
E_IDX++;\x0a\x0a      \
      let action\
s = '';\x0a        \
    if (langClas\
s === 'html') {\x0a\
              ac\
tions += `<a hre\
f=\x22empty:${idxLo\
cal}\x22 class=\x22cod\
e-header-action \
code-header-prev\
iew\x22><img src=\x22$\
{cfg.ICONS.CODE_\
PREVIEW}\x22 class=\
\x22action-img\x22 dat\
a-id=\x22${idxLocal\
}\x22><span>${Utils\
.escapeHtml(cfg.\
LOCALE.PREVIEW)}\
</span></a>`;\x0a  \
          } else\
 if (langClass =\
== 'python' && h\
eaderLabel !== '\
output') {\x0a     \
         actions\
 += `<a href=\x22em\
pty:${idxLocal}\x22\
 class=\x22code-hea\
der-action code-\
header-run\x22><img\
 src=\x22${cfg.ICON\
S.CODE_RUN}\x22 cla\
ss=\x22action-img\x22 \
data-id=\x22${idxLo\
cal}\x22><span>${Ut\
ils.escapeHtml(c\
fg.LOCALE.RUN)}<\
/span></a>`;\x0a   \
         }\x0a     \
       actions +\
= `<a href=\x22empt\
y:${idxLocal}\x22 c\
lass=\x22code-heade\
r-action code-he\
ader-collapse\x22><\
img src=\x22${cfg.I\
CONS.CODE_MENU}\x22\
 class=\x22action-i\
mg\x22 data-id=\x22${i\
dxLocal}\x22><span>\
${Utils.escapeHt\
ml(cfg.LOCALE.CO\
LLAPSE)}</span><\
/a>`;\x0a          \
  actions += `<a\
 href=\x22empty:${i\
dxLocal}\x22 class=\
\x22code-header-act\
ion code-header-\
copy\x22><img src=\x22\
${cfg.ICONS.CODE\
_COPY}\x22 class=\x22a\
ction-img\x22 data-\
id=\x22${idxLocal}\x22\
><span>${Utils.e\
scapeHtml(cfg.LO\
CALE.COPY)}</spa\
n></a>`;\x0a\x0a      \
      return (\x0a \
             `<d\
iv class=\x22code-w\
rapper highlight\
\x22 data-index=\x22${\
idxLocal}\x22` +\x0a  \
            ` da\
ta-code-lang=\x22${\
Utils.escapeHtml\
(res.lang || '')\
}\x22` +\x0a          \
    ` data-code-\
len=\x22${String(le\
n)}\x22 data-code-h\
ead=\x22${headEsc}\x22\
 data-code-tail=\
\x22${tailEsc}\x22 dat\
a-code-nl=\x22${Str\
ing(nl)}\x22` +\x0a   \
           ` dat\
a-locale-collaps\
e=\x22${Utils.escap\
eHtml(cfg.LOCALE\
.COLLAPSE)}\x22 dat\
a-locale-expand=\
\x22${Utils.escapeH\
tml(cfg.LOCALE.E\
XPAND)}\x22` +\x0a    \
          ` data\
-locale-copy=\x22${\
Utils.escapeHtml\
(cfg.LOCALE.COPY\
)}\x22 data-locale-\
copied=\x22${Utils.\
escapeHtml(cfg.L\
OCALE.COPIED)}\x22 \
data-style=\x22${Ut\
ils.escapeHtml(c\
fg.CODE_STYLE)}\x22\
>` +\x0a           \
     `<p class=\x22\
code-header-wrap\
per\x22><span><span\
 class=\x22code-hea\
der-lang\x22>${Util\
s.escapeHtml(hea\
derLabel)}   </s\
pan>${actions}</\
span></p>` +\x0a   \
             `<p\
re><code class=\x22\
language-${Utils\
.escapeHtml(lang\
Class)} hljs\x22>${\
inner}</code></p\
re>` +\x0a         \
     `</div>`\x0a  \
          );\x0a   \
       }\x0a       \
 })(this.MD, thi\
s.logger);\x0a     \
 }\x0a      // Repl\
ace \x22sandbox:\x22 l\
inks with file:/\
/ in markdown so\
urce (host polic\
y).\x0a      prepro\
cessMD(s) { retu\
rn (s || '').rep\
lace(/\x5c]\x5c(sandbo\
x:/g, '](file://\
'); }\x0a      // D\
ecode base64 UTF\
-8 to string (sh\
ared TextDecoder\
).\x0a      b64ToUt\
f8(b64) {\x0a      \
  const bin = at\
ob(b64);\x0a       \
 const bytes = n\
ew Uint8Array(bi\
n.length);\x0a     \
   for (let i = \
0; i < bin.lengt\
h; i++) bytes[i]\
 = bin.charCodeA\
t(i);\x0a        re\
turn Utils.utf8D\
ecode(bytes);\x0a  \
    }\x0a\x0a      // \
Apply custom mar\
kup for bot mess\
ages only (metho\
d name kept for \
API).\x0a      appl\
yCustomMarkupFor\
Bots(root) {\x0a   \
     const MD = \
this.MD;\x0a       \
 try {\x0a         \
 const scope = r\
oot || document;\
\x0a          const\
 targets = [];\x0a\x0a\
          // If \
scope itself is \
a bot message bo\
x\x0a          if (\
scope && scope.n\
odeType === 1 &&\
 scope.classList\
 && scope.classL\
ist.contains('ms\
g-box') &&\x0a     \
         scope.c\
lassList.contain\
s('msg-bot')) {\x0a\
            targ\
ets.push(scope);\
\x0a          }\x0a\x0a  \
        // Colle\
ct bot message b\
oxes within the \
scope\x0a          \
if (scope && typ\
eof scope.queryS\
electorAll === '\
function') {\x0a   \
         const l\
ist = scope.quer\
ySelectorAll('.m\
sg-box.msg-bot')\
;\x0a            fo\
r (let i = 0; i \
< list.length; i\
++) targets.push\
(list[i]);\x0a     \
     }\x0a\x0a        \
  // If scope is\
 inside a bot me\
ssage, include t\
he closest ances\
tor as well\x0a    \
      if (scope \
&& scope.nodeTyp\
e === 1 && typeo\
f scope.closest \
=== 'function') \
{\x0a            co\
nst closestMsg =\
 scope.closest('\
.msg-box.msg-bot\
');\x0a            \
if (closestMsg) \
targets.push(clo\
sestMsg);\x0a      \
    }\x0a\x0a         \
 // Deduplicate \
and apply rules \
only to bot mess\
ages\x0a          c\
onst seen = new \
Set();\x0a         \
 for (const el o\
f targets) {\x0a   \
         if (!el\
 || !el.isConnec\
ted || seen.has(\
el)) continue;\x0a \
           seen.\
add(el);\x0a       \
     this.custom\
Markup.apply(el,\
 MD);\x0a          \
}\x0a        } catc\
h (_) {\x0a        \
  // Keep render\
 path resilient\x0a\
        }\x0a      \
}\x0a\x0a      // Help\
er: choose rende\
rer (hot vs full\
) for snapshot u\
se.\x0a      _md(st\
reamingHint) {\x0a \
       return st\
reamingHint ? (t\
his.MD_STREAM ||\
 this.MD) : (thi\
s.MD || this.MD_\
STREAM);\x0a      }\
\x0a\x0a      // Async\
, batched proces\
sing of [data-md\
64] / [md-block-\
markdown] to kee\
p UI responsive \
on heavy loads.\x0a\
      // Note: u\
ser messages are\
 rendered as pla\
in text (no mark\
down-it, no cust\
om markup, no Ka\
TeX).\x0a      asyn\
c renderPendingM\
arkdown(root) {\x0a\
        const MD\
 = this.MD; if (\
!MD) return;\x0a   \
     const scope\
 = root || docum\
ent;\x0a\x0a        //\
 Collect both le\
gacy base64 hold\
ers and new nati\
ve Markdown hold\
ers\x0a        cons\
t nodes = Array.\
from(scope.query\
SelectorAll('[da\
ta-md64], [md-bl\
ock-markdown]'))\
;\x0a        if (no\
des.length === 0\
) {\x0a          //\
 Nothing to mate\
rialize right no\
w. Avoid arming \
rAF work unless \
there is\x0a       \
   // actually s\
omething present\
 that needs high\
light/scroll/mat\
h.\x0a          try\
 {\x0a            c\
onst hasBots = !\
!(scope && scope\
.querySelector &\
& scope.querySel\
ector('.msg-box.\
msg-bot'));\x0a    \
        const ha\
sWrappers = !!(s\
cope && scope.qu\
erySelector && s\
cope.querySelect\
or('.code-wrappe\
r'));\x0a          \
  const hasCodes\
 = !!(scope && s\
cope.querySelect\
or && scope.quer\
ySelector('.msg-\
box.msg-bot pre \
code'));\x0a       \
     const hasUn\
highlighted = !!\
(scope && scope.\
querySelector &&\
 scope.querySele\
ctor('.msg-box.m\
sg-bot pre code:\
not([data-highli\
ghted=\x22yes\x22])'))\
;\x0a            co\
nst hasMath = !!\
(scope && scope.\
querySelector &&\
 scope.querySele\
ctor('script[typ\
e^=\x22math/tex\x22]')\
);\x0a\x0a            \
// Apply Custom \
Markup only if b\
ot messages are \
present.\x0a       \
     if (hasBots\
) { this.applyCu\
stomMarkupForBot\
s(scope); }\x0a\x0a   \
         // Rest\
ore collapsed st\
ate only if we c\
an actually find\
 wrappers.\x0a     \
       if (hasWr\
appers) { this.r\
estoreCollapsedC\
ode(scope); }\x0a\x0a \
           // In\
itialize code sc\
roll helpers for\
 current root.\x0a \
           this.\
hooks.codeScroll\
Init(scope);\x0a\x0a  \
          // Ini\
t code-scroll/hi\
ghlight observer\
s only when ther\
e are codes in D\
OM.\x0a            \
if (hasCodes) {\x0a\
              th\
is.hooks.observe\
MsgBoxes(scope);\
\x0a              t\
his.hooks.observ\
eNewCode(scope, \
{\x0a              \
  deferLastIfStr\
eaming: true,\x0a  \
              mi\
nLinesForLast: t\
his.cfg.PROFILE_\
CODE.minLinesFor\
HL,\x0a            \
    minCharsForL\
ast: this.cfg.PR\
OFILE_CODE.minCh\
arsForHL\x0a       \
       });\x0a     \
         if (has\
Unhighlighted &&\
 typeof runtime \
!== 'undefined' \
&& runtime.highl\
ighter) {\x0a      \
          runtim\
e.highlighter.sc\
anVisibleCodesIn\
Root(scope, runt\
ime.stream.activ\
eCode || null);\x0a\
              }\x0a\
            }\x0a\x0a \
           // Sc\
hedule KaTeX ren\
der only if ther\
e are math scrip\
ts present.\x0a    \
        if (hasM\
ath) { this.hook\
s.scheduleMathRe\
nder(scope); }\x0a \
           this.\
hooks.codeScroll\
Init(scope);\x0a\x0a  \
        } catch \
(_) { /* swallow\
: keep idle path\
 safe */ }\x0a\x0a    \
      return;\x0a  \
      }\x0a\x0a       \
 // Track which \
bot message boxe\
s actually chang\
ed to avoid a he\
avy global Custo\
m Markup pass.\x0a \
       const tou\
chedBoxes = new \
Set();\x0a\x0a        \
// Budgeted, coo\
perative loop: p\
rocess nodes one\
-by-one with per\
-frame yield whe\
n needed.\x0a      \
  const perSlice\
 = (this.cfg.ASY\
NC && this.cfg.A\
SYNC.MD_NODES_PE\
R_SLICE) || 12; \
// upper bound p\
er frame\x0a       \
 let sliceCount \
= 0;\x0a        let\
 startedAt = Uti\
ls.now();\x0a\x0a     \
   for (let j = \
0; j < nodes.len\
gth; j++) {\x0a    \
      const el =\
 nodes[j];\x0a     \
     if (!el || \
!el.isConnected)\
 continue;\x0a\x0a    \
      let md = '\
';\x0a          con\
st isNative = el\
.hasAttribute('m\
d-block-markdown\
');\x0a          co\
nst msgBox = (el\
.closest && el.c\
losest('.msg-box\
.msg-bot, .msg-b\
ox.msg-user')) |\
| null;\x0a        \
  const isUserMs\
g = !!(msgBox &&\
 msgBox.classLis\
t.contains('msg-\
user'));\x0a       \
   const isBotMs\
g = !!(msgBox &&\
 msgBox.classLis\
t.contains('msg-\
bot'));\x0a\x0a       \
   // Read sourc\
e text (do not p\
reprocess for us\
er messages to k\
eep it raw)\x0a    \
      if (isNati\
ve) {\x0a          \
  try { md = isU\
serMsg ? (el.tex\
tContent || '') \
: this.preproces\
sMD(el.textConte\
nt || ''); } cat\
ch (_) { md = ''\
; }\x0a            \
try { el.removeA\
ttribute('md-blo\
ck-markdown'); }\
 catch (_) {}\x0a  \
        } else {\
\x0a            con\
st b64 = el.getA\
ttribute('data-m\
d64'); if (!b64)\
 continue;\x0a     \
       try { md \
= this.b64ToUtf8\
(b64); } catch (\
_) { md = ''; }\x0a\
            el.r\
emoveAttribute('\
data-md64');\x0a   \
         if (!is\
UserMsg) { try {\
 md = this.prepr\
ocessMD(md); } c\
atch (_) {} }\x0a  \
        }\x0a\x0a     \
     if (isUserM\
sg) {\x0a          \
  // User messag\
e: replace place\
holder with raw \
plain text only.\
\x0a            con\
st span = docume\
nt.createElement\
('span');\x0a      \
      span.textC\
ontent = md;\x0a   \
         el.repl\
aceWith(span);\x0a \
           // In\
tentionally do N\
OT add to touche\
dBoxes; no Custo\
m Markup for use\
r.\x0a          } e\
lse if (isBotMsg\
) {\x0a            \
// Bot message: \
full markdown-it\
 render with Cus\
tom Markup.\x0a    \
        let html\
 = '';\x0a         \
     try {\x0a     \
           let s\
rc = md;\x0a       \
         // Pre-\
md transforms fo\
r source-phase r\
ules\x0a           \
     if (this.cu\
stomMarkup && ty\
peof this.custom\
Markup.transform\
Source === 'func\
tion') {\x0a       \
           src =\
 this.customMark\
up.transformSour\
ce(src, { stream\
ing: false });\x0a \
               }\
\x0a               \
 html = MD.rende\
r(src);\x0a        \
      } catch (_\
) { html = Utils\
.escapeHtml(md);\
 }\x0a\x0a            \
// build fragmen\
t directly (avoi\
d intermediate c\
ontainer allocat\
ions).\x0a         \
   let frag = nu\
ll;\x0a            \
try {\x0a          \
    const range \
= document.creat\
eRange();\x0a      \
        const ct\
x = el.parentNod\
e || document.bo\
dy || document.d\
ocumentElement;\x0a\
              ra\
nge.selectNode(c\
tx);\x0a           \
   frag = range.\
createContextual\
Fragment(html);\x0a\
            } ca\
tch (_) {\x0a      \
        const tm\
p = document.cre\
ateElement('div'\
);\x0a             \
 tmp.innerHTML =\
 html;\x0a         \
     frag = docu\
ment.createDocum\
entFragment();\x0a \
             whi\
le (tmp.firstChi\
ld) frag.appendC\
hild(tmp.firstCh\
ild);\x0a          \
  }\x0a\x0a           \
 // Apply Custom\
 Markup on a lig\
htweight Documen\
tFragment\x0a      \
      try { this\
.customMarkup.ap\
ply(frag, MD); }\
 catch (_) {}\x0a\x0a \
           el.re\
placeWith(frag);\
\x0a            tou\
chedBoxes.add(ms\
gBox);\x0a         \
 } else {\x0a      \
      // Outside\
 of any message \
box: materialize\
 as plain text.\x0a\
            cons\
t span = documen\
t.createElement(\
'span');\x0a       \
     span.textCo\
ntent = md;\x0a    \
        el.repla\
ceWith(span);\x0a  \
        }\x0a\x0a     \
     sliceCount+\
+;\x0a          // \
Yield by time bu\
dget or by count\
 to keep frame s\
hort and reactiv\
e.\x0a          if \
(sliceCount >= p\
erSlice || this.\
asyncer.shouldYi\
eld(startedAt)) \
{\x0a            aw\
ait this.asyncer\
.yield();\x0a      \
      startedAt \
= Utils.now();\x0a \
           slice\
Count = 0;\x0a     \
     }\x0a        }\
\x0a\x0a        // App\
ly Custom Markup\
 only to actuall\
y modified BOT m\
essages (keeps t\
his pass light).\
\x0a        try {\x0a \
         touched\
Boxes.forEach(bo\
x => { try { thi\
s.customMarkup.a\
pply(box, MD); }\
 catch (_) {} })\
;\x0a        } catc\
h (_) {}\x0a\x0a      \
  // Same post-p\
rocessing as bef\
ore (idempotent \
with external ca\
lls).\x0a        th\
is.restoreCollap\
sedCode(scope);\x0a\
        this.hoo\
ks.observeNewCod\
e(scope, {\x0a     \
     deferLastIf\
Streaming: true,\
\x0a          minLi\
nesForLast: this\
.cfg.PROFILE_COD\
E.minLinesForHL,\
\x0a          minCh\
arsForLast: this\
.cfg.PROFILE_COD\
E.minCharsForHL\x0a\
        });\x0a    \
    this.hooks.o\
bserveMsgBoxes(s\
cope);\x0a        t\
his.hooks.schedu\
leMathRender(sco\
pe);\x0a        thi\
s.hooks.codeScro\
llInit(scope);\x0a\x0a\
        if (type\
of runtime !== '\
undefined' && ru\
ntime.highlighte\
r) {\x0a         ru\
ntime.highlighte\
r.scanVisibleCod\
esInRoot(scope, \
runtime.stream.a\
ctiveCode || nul\
l);\x0a        }\x0a  \
    }\x0a\x0a      // \
Render streaming\
 snapshot.\x0a     \
 renderStreaming\
Snapshot(src) {\x0a\
        const md\
 = this._md(true\
);\x0a        if (!\
md) return '';\x0a \
       try {\x0a   \
       let s = S\
tring(src || '')\
;\x0a          // P\
re-markdown cust\
om transforms (e\
.g. [!exec]/<exe\
cute> => ```pyth\
on fences)\x0a     \
     if (this.cu\
stomMarkup && ty\
peof this.custom\
Markup.transform\
Source === 'func\
tion') {\x0a       \
     s = this.cu\
stomMarkup.trans\
formSource(s, { \
streaming: true \
});\x0a          }\x0a\
          return\
 md.render(s);\x0a \
       } catch (\
_) { return Util\
s.escapeHtml(src\
); }\x0a      }\x0a\x0a  \
    renderFinalS\
napshot(src) {\x0a \
       const md \
= this._md(false\
);\x0a        if (!\
md) return '';\x0a \
       try {\x0a   \
       let s = S\
tring(src || '')\
;\x0a          if (\
this.customMarku\
p && typeof this\
.customMarkup.tr\
ansformSource ==\
= 'function') {\x0a\
            s = \
this.customMarku\
p.transformSourc\
e(s, { streaming\
: false });\x0a    \
      }\x0a        \
  return md.rend\
er(s);\x0a        }\
 catch (_) { ret\
urn Utils.escape\
Html(src); }\x0a   \
   }\x0a\x0a      // R\
estore collapse/\
expand state of \
code blocks afte\
r DOM updates.\x0a \
     restoreColl\
apsedCode(root) \
{\x0a        const \
scope = root || \
document;\x0a      \
  const wrappers\
 = scope.querySe\
lectorAll('.code\
-wrapper');\x0a    \
    wrappers.for\
Each((wrapper) =\
> {\x0a          co\
nst index = wrap\
per.getAttribute\
('data-index');\x0a\
          const \
localeCollapse =\
 wrapper.getAttr\
ibute('data-loca\
le-collapse');\x0a \
         const l\
ocaleExpand = wr\
apper.getAttribu\
te('data-locale-\
expand');\x0a      \
    const source\
 = wrapper.query\
Selector('code')\
;\x0a          cons\
t isCollapsed = \
(window.__collap\
sed_idx || []).i\
ncludes(index);\x0a\
          if (!s\
ource) return;\x0a \
         const b\
tn = wrapper.que\
rySelector('.cod\
e-header-collaps\
e');\x0a          i\
f (isCollapsed) \
{\x0a            so\
urce.style.displ\
ay = 'none';\x0a   \
         if (btn\
) { const span =\
 btn.querySelect\
or('span'); if (\
span) span.textC\
ontent = localeE\
xpand; }\x0a       \
   } else {\x0a    \
        source.s\
tyle.display = '\
block';\x0a        \
    if (btn) { c\
onst span = btn.\
querySelector('s\
pan'); if (span)\
 span.textConten\
t = localeCollap\
se; }\x0a          \
}\x0a        });\x0a  \
    }\x0a    }\x0a  wi\
ndow.__collapsed\
_idx = window.__\
collapsed_idx ||\
 [];\x0a\x0a  // =====\
================\
================\
================\
================\
=====\x0a  // 6) Ma\
th renderer (asy\
nc, chunked)\x0a  /\
/ ==============\
================\
================\
================\
============\x0a  c\
lass MathRendere\
r {\x0a    construc\
tor(cfg, raf, as\
yncer) {\x0a      t\
his.cfg = cfg; t\
his.raf = raf; t\
his.asyncer = as\
yncer;\x0a      thi\
s.scheduled = fa\
lse;\x0a\x0a      // r\
AF key used by t\
he central pump \
(do not change \xe2\
\x80\x93 API compatibi\
lity).\x0a      thi\
s.rafKey = { t: \
'Math:render' };\
\x0a\x0a      // Pendi\
ng roots aggrega\
tion: if documen\
t-level render i\
s requested, it \
supersedes other\
s.\x0a      this._p\
endingRoots = ne\
w Set();\x0a      t\
his._pendingDoc \
= false;\x0a    }\x0a\x0a\
    // Async, co\
operative KaTeX \
rendering to avo\
id long blocking\
 on many formula\
s.\x0a    async ren\
derAsync(root) {\
\x0a      if (typeo\
f katex === 'und\
efined') return;\
\x0a      const sco\
pe = root || doc\
ument;\x0a      con\
st scripts = Arr\
ay.from(scope.qu\
erySelectorAll('\
script[type^=\x22ma\
th/tex\x22]'));\x0a   \
   const useToSt\
ring = (typeof k\
atex.renderToStr\
ing === 'functio\
n');\x0a\x0a      cons\
t batchFn = asyn\
c (script) => {\x0a\
        if (!scr\
ipt || !script.i\
sConnected) retu\
rn;\x0a        // O\
nly render math \
in bot messages\x0a\
        if (!scr\
ipt.closest('.ms\
g-box.msg-bot'))\
 return;\x0a       \
 const t = scrip\
t.getAttribute('\
type') || '';\x0a  \
      const disp\
layMode = t.inde\
xOf('mode=displa\
y') > -1;\x0a      \
  // avoid inner\
Text (it may tri\
gger layout). te\
xtContent is suf\
ficient here.\x0a  \
      const math\
Content = script\
.textContent || \
'';\x0a        cons\
t parent = scrip\
t.parentNode; if\
 (!parent) retur\
n;\x0a\x0a        try \
{\x0a          if (\
useToString) {\x0a \
           let h\
tml = '';\x0a      \
      try {\x0a    \
          html =\
 katex.renderToS\
tring(mathConten\
t, { displayMode\
, throwOnError: \
false });\x0a      \
      } catch (_\
) {\x0a            \
  const fb = dis\
playMode ? `\x5c\x5c[$\
{mathContent}\x5c\x5c]\
` : `\x5c\x5c(${mathCo\
ntent}\x5c\x5c)`;\x0a    \
          html =\
 (displayMode ? \
`<div>${Utils.es\
capeHtml(fb)}</d\
iv>` : `<span>${\
Utils.escapeHtml\
(fb)}</span>`);\x0a\
            }\x0a  \
          const \
host = document.\
createElement(di\
splayMode ? 'div\
' : 'span');\x0a   \
         host.in\
nerHTML = html;\x0a\
            cons\
t el = host.firs\
tElementChild ||\
 host;\x0a         \
   if (parent.cl\
assList && paren\
t.classList.cont\
ains('math-pendi\
ng')) parent.rep\
laceWith(el);\x0a  \
          else p\
arent.replaceChi\
ld(el, script);\x0a\
          } else\
 {\x0a            c\
onst el = docume\
nt.createElement\
(displayMode ? '\
div' : 'span');\x0a\
            try \
{ katex.render(m\
athContent, el, \
{ displayMode, t\
hrowOnError: fal\
se }); }\x0a       \
     catch (_) {\
 el.textContent \
= (displayMode ?\
 `\x5c\x5c[${mathConte\
nt}\x5c\x5c]` : `\x5c\x5c(${\
mathContent}\x5c\x5c)`\
); }\x0a           \
 if (parent.clas\
sList && parent.\
classList.contai\
ns('math-pending\
')) parent.repla\
ceWith(el);\x0a    \
        else par\
ent.replaceChild\
(el, script);\x0a  \
        }\x0a      \
  } catch (_) {\x0a\
          // Kee\
p fallback text \
intact on any er\
ror\x0a        }\x0a  \
    };\x0a\x0a      //\
 Process formula\
s cooperatively \
(rAF yields).\x0a  \
    await this.a\
syncer.forEachCh\
unk(scripts, bat\
chFn, 'MathRende\
rer');\x0a    }\x0a\x0a  \
  // Schedule ma\
th rendering for\
 a root. Coalesc\
es multiple call\
s.\x0a    schedule(\
root, _delayIgno\
red = 0, forceNo\
w = false) {\x0a   \
   // If KaTeX i\
s not available,\
 honor no-op. AP\
I stays intact.\x0a\
      if (typeof\
 katex === 'unde\
fined') return;\x0a\
\x0a      // Normal\
ize root (defaul\
t to whole docum\
ent).\x0a      cons\
t targetRoot = r\
oot || document;\
\x0a\x0a      // Fast \
existence check \
to avoid arming \
rAF when nothing\
 to do, but stil\
l\x0a      // keep \
aggregation sema\
ntics: if a job \
is already sched\
uled we can stil\
l\x0a      // merge\
 new roots into \
the pending set \
when they actual\
ly contain math.\
\x0a      let hasMa\
th = true;\x0a     \
 if (!forceNow) \
{\x0a        try {\x0a\
          hasMat\
h = !!(targetRoo\
t && targetRoot.\
querySelector &&\
 targetRoot.quer\
ySelector('scrip\
t[type^=\x22math/te\
x\x22]'));\x0a        \
} catch (_) { ha\
sMath = false; }\
\x0a        if (!ha\
sMath) return; /\
/ nothing to ren\
der for this roo\
t; safe early ex\
it\x0a      }\x0a\x0a    \
  // Aggregate r\
oots so nothing \
is lost while on\
e job is already\
 scheduled.\x0a    \
  if (targetRoot\
 === document ||\
 targetRoot === \
document.documen\
tElement || targ\
etRoot === docum\
ent.body) {\x0a    \
    this._pendin\
gDoc = true;    \
             // \
promote to a ful\
l-document sweep\
\x0a        this._p\
endingRoots.clea\
r();            \
  // small optim\
ization (documen\
t covers all)\x0a  \
    } else if (!\
this._pendingDoc\
) {\x0a        this\
._pendingRoots.a\
dd(targetRoot);\x0a\
      }\x0a\x0a      /\
/ If a task is a\
lready scheduled\
, do not arm ano\
ther \xe2\x80\x93 coalesc\
ing will take ca\
re of it.\x0a      \
if (this.schedul\
ed && this.raf &\
& typeof this.ra\
f.isScheduled ==\
= 'function' && \
this.raf.isSched\
uled(this.rafKey\
)) return;\x0a\x0a    \
  this.scheduled\
 = true;\x0a      c\
onst priority = \
forceNow ? 0 : 2\
;\x0a\x0a      // Sing\
le rAF job drain\
s all pending ro\
ots; renderAsync\
 remains public \
and unchanged.\x0a \
     this.raf.sc\
hedule(this.rafK\
ey, () => {\x0a    \
    this.schedul\
ed = false;\x0a\x0a   \
     const useDo\
c = this._pendin\
gDoc;\x0a        co\
nst roots = [];\x0a\
\x0a        if (use\
Doc) {\x0a         \
 roots.push(docu\
ment);\x0a        }\
 else {\x0a        \
  this._pendingR\
oots.forEach((r)\
 => {\x0a          \
  // Only keep c\
onnected element\
s to avoid usele\
ss work.\x0a       \
     try {\x0a     \
         if (r &\
& (r.isConnected\
 === undefined |\
| r.isConnected)\
) roots.push(r);\
\x0a            } c\
atch (_) {\x0a     \
         // Cons\
ervative: keep t\
he root; renderA\
sync guards inte\
rnally.\x0a        \
      roots.push\
(r);\x0a           \
 }\x0a          });\
\x0a        }\x0a\x0a    \
    // Reset agg\
regation state b\
efore running (n\
ew calls can agg\
regate afresh).\x0a\
        this._pe\
ndingDoc = false\
;\x0a        this._\
pendingRoots.cle\
ar();\x0a\x0a        /\
/ Fire-and-forge\
t async drain; k\
eep renderAsync \
API intact.\x0a    \
    (async () =>\
 {\x0a          for\
 (let i = 0; i <\
 roots.length; i\
++) {\x0a          \
  try { await th\
is.renderAsync(r\
oots[i]); } catc\
h (_) { /* swall\
ow \xe2\x80\x93 resilient\
 */ }\x0a          \
}\x0a        })();\x0a\
      }, 'Math',\
 priority);\x0a    \
}\x0a    // Cleanup\
 pending work an\
d state.\x0a    cle\
anup() {\x0a      t\
ry { this.raf.ca\
ncelGroup('Math'\
); } catch (_) {\
}\x0a      this.sch\
eduled = false;\x0a\
\x0a      // Ensure\
 pending state i\
s fully cleared \
on cleanup.\x0a    \
  try { this._pe\
ndingRoots.clear\
(); } catch (_) \
{}\x0a      this._p\
endingDoc = fals\
e;\x0a    }\x0a  }\x0a\x0a  \
// =============\
================\
================\
================\
=============\x0a  \
// 7) Scroll man\
ager + FAB\x0a  // \
================\
================\
================\
================\
==========\x0a\x0a  cl\
ass ScrollManage\
r {\x0a    construc\
tor(cfg, dom, ra\
f) {\x0a      this.\
cfg = cfg; this.\
dom = dom; this.\
raf = raf;\x0a     \
 this.autoFollow\
 = true; this.us\
erInteracted = f\
alse;\x0a      this\
.lastScrollTop =\
 0; this.prevScr\
oll = 0;\x0a      t\
his.currentFabAc\
tion = 'none'; t\
his.fabFreezeUnt\
il = 0;\x0a      th\
is.scrollSchedul\
ed = false; this\
.scrollFabUpdate\
Scheduled = fals\
e;\x0a      this.sc\
rollRAF = 0; thi\
s.scrollFabRAF =\
 0;\x0a    }\x0a    //\
 Is page near th\
e bottom by give\
n margin?\x0a    is\
NearBottom(margi\
nPx = 100) {\x0a   \
   const el = Ut\
ils.SE; const di\
stance = el.scro\
llHeight - el.cl\
ientHeight - el.\
scrollTop;\x0a     \
 return distance\
 <= marginPx;\x0a  \
  }\x0a    // Sched\
ule a page scrol\
l to bottom if a\
uto-follow allow\
s it.\x0a    schedu\
leScroll(live = \
false) {\x0a      i\
f (live === true\
 && this.autoFol\
low !== true) re\
turn;\x0a      if (\
this.scrollSched\
uled) return;\x0a  \
    this.scrollS\
cheduled = true;\
\x0a      this.raf.\
schedule('SM:scr\
oll', () => { th\
is.scrollSchedul\
ed = false; this\
.scrollToBottom(\
live); this.sche\
duleScrollFabUpd\
ate(); }, 'Scrol\
lManager', 1);\x0a \
   }\x0a    // Canc\
el any pending p\
age scroll.\x0a    \
cancelPendingScr\
oll() {\x0a      tr\
y { this.raf.can\
celGroup('Scroll\
Manager'); } cat\
ch (_) {}\x0a      \
this.scrollSched\
uled = false;\x0a  \
    this.scrollF\
abUpdateSchedule\
d = false;\x0a     \
 this.scrollRAF \
= 0; this.scroll\
FabRAF = 0;\x0a    \
}\x0a    // Jump to\
 bottom immediat\
ely (no smooth b\
ehavior).\x0a    fo\
rceScrollToBotto\
mImmediate() {\x0a \
     const el = \
Utils.SE; el.scr\
ollTop = el.scro\
llHeight; this.p\
revScroll = el.s\
crollHeight;\x0a   \
 }\x0a    // Scroll\
 window to botto\
m based on auto-\
follow and margi\
ns.\x0a    scrollTo\
Bottom(live = fa\
lse, force = fal\
se) {\x0a      cons\
t el = Utils.SE;\
 const marginPx \
= this.cfg.UI.SC\
ROLL_NEAR_MARGIN\
_PX; const behav\
ior = 'instant';\
\x0a      if (live \
=== true && this\
.autoFollow !== \
true) { this.pre\
vScroll = el.scr\
ollHeight; retur\
n; }\x0a      if ((\
live === true &&\
 this.userIntera\
cted === false) \
|| this.isNearBo\
ttom(marginPx) |\
| live === false\
 || force) {\x0a   \
     try { el.sc\
rollTo({ top: el\
.scrollHeight, b\
ehavior }); } ca\
tch (_) { el.scr\
ollTop = el.scro\
llHeight; }\x0a    \
  }\x0a      this.p\
revScroll = el.s\
crollHeight;\x0a   \
 }\x0a    // Check \
if window has ve\
rtical scroll ba\
r.\x0a    hasVertic\
alScroll() { con\
st el = Utils.SE\
; return (el.scr\
ollHeight - el.c\
lientHeight) > 1\
; }\x0a    // Compu\
te the current F\
AB action (none/\
up/down).\x0a    co\
mputeFabAction()\
 {\x0a      const e\
l = Utils.SE; co\
nst hasScroll = \
(el.scrollHeight\
 - el.clientHeig\
ht) > 1;\x0a      i\
f (!hasScroll) r\
eturn 'none';\x0a  \
    const dist =\
 el.scrollHeight\
 - el.clientHeig\
ht - el.scrollTo\
p;\x0a      if (dis\
t <= 2) return '\
up';\x0a      if (d\
ist >= this.cfg.\
FAB.SHOW_DOWN_TH\
RESHOLD_PX) retu\
rn 'down';\x0a     \
 return 'none';\x0a\
    }\x0a    // Upd\
ate FAB to show \
correct directio\
n and label.\x0a   \
 updateScrollFab\
(force = false, \
actionOverride =\
 null, bypassFre\
eze = false) {\x0a \
     const btn =\
 this.dom.get('s\
crollFab'); cons\
t icon = this.do\
m.get('scrollFab\
Icon');\x0a      if\
 (!btn || !icon)\
 return;\x0a      c\
onst now = Utils\
.now(); const ac\
tion = actionOve\
rride || this.co\
mputeFabAction()\
;\x0a      if (!for\
ce && !bypassFre\
eze && now < thi\
s.fabFreezeUntil\
 && action !== t\
his.currentFabAc\
tion) return;\x0a  \
    if (action =\
== 'none') {\x0a   \
     if (this.cu\
rrentFabAction !\
== 'none' || for\
ce) { btn.classL\
ist.remove('visi\
ble'); this.curr\
entFabAction = '\
none'; }\x0a       \
 return;\x0a      }\
\x0a      if (actio\
n !== this.curre\
ntFabAction || f\
orce) {\x0a        \
if (action === '\
up') {\x0a         \
 if (icon.datase\
t.dir !== 'up') \
{ icon.src = thi\
s.cfg.ICONS.COLL\
APSE; icon.datas\
et.dir = 'up'; }\
\x0a          btn.t\
itle = \x22Go to to\
p\x22;\x0a        } el\
se {\x0a          i\
f (icon.dataset.\
dir !== 'down') \
{ icon.src = thi\
s.cfg.ICONS.EXPA\
ND; icon.dataset\
.dir = 'down'; }\
\x0a          btn.t\
itle = \x22Go to bo\
ttom\x22;\x0a        }\
\x0a        btn.set\
Attribute('aria-\
label', btn.titl\
e);\x0a        this\
.currentFabActio\
n = action; btn.\
classList.add('v\
isible');\x0a      \
} else if (!btn.\
classList.contai\
ns('visible')) b\
tn.classList.add\
('visible');\x0a   \
 }\x0a    // Schedu\
le a FAB state r\
efresh.\x0a    sche\
duleScrollFabUpd\
ate() {\x0a      if\
 (this.scrollFab\
UpdateScheduled)\
 return;\x0a      t\
his.scrollFabUpd\
ateScheduled = t\
rue;\x0a      this.\
raf.schedule('SM\
:fab', () => {\x0a \
       this.scro\
llFabUpdateSched\
uled = false;\x0a  \
      const acti\
on = this.comput\
eFabAction(); if\
 (action !== thi\
s.currentFabActi\
on) this.updateS\
crollFab(false, \
action);\x0a      }\
, 'ScrollManager\
', 2);\x0a    }\x0a   \
 // If user is n\
ear bottom, enab\
le auto-follow a\
gain.\x0a    maybeE\
nableAutoFollowB\
yProximity() {\x0a \
     const el = \
Utils.SE;\x0a      \
if (!this.autoFo\
llow) {\x0a        \
const dist = el.\
scrollHeight - e\
l.clientHeight -\
 el.scrollTop;\x0a \
       if (dist \
<= this.cfg.UI.A\
UTO_FOLLOW_REENA\
BLE_PX) this.aut\
oFollow = true;\x0a\
      }\x0a    }\x0a  \
  // User-trigge\
red scroll to to\
p; disables auto\
-follow.\x0a    scr\
ollToTopUser() {\
\x0a      this.user\
Interacted = tru\
e; this.autoFoll\
ow = false;\x0a    \
  try { const el\
 = Utils.SE; el.\
scrollTo({ top: \
0, behavior: 'in\
stant' }); this.\
lastScrollTop = \
el.scrollTop; }\x0a\
      catch (_) \
{ const el = Uti\
ls.SE; el.scroll\
Top = 0; this.la\
stScrollTop = 0;\
 }\x0a    }\x0a    // \
User-triggered s\
croll to bottom;\
 may re-enable a\
uto-follow if ne\
ar bottom.\x0a    s\
crollToBottomUse\
r() {\x0a      this\
.userInteracted \
= true; this.aut\
oFollow = false;\
\x0a      try { con\
st el = Utils.SE\
; el.scrollTo({ \
top: el.scrollHe\
ight, behavior: \
'instant' }); th\
is.lastScrollTop\
 = el.scrollTop;\
 }\x0a      catch (\
_) { const el = \
Utils.SE; el.scr\
ollTop = el.scro\
llHeight; this.l\
astScrollTop = e\
l.scrollTop; }\x0a \
     this.maybeE\
nableAutoFollowB\
yProximity();\x0a  \
  }\x0a  }\x0a\x0a  // ==\
================\
================\
================\
================\
========\x0a  // 8)\
 Tips manager\x0a  \
// =============\
================\
================\
================\
=============\x0a\x0a \
 // Tips manager\
 (drop-in replac\
ement): rotates \
small hint messa\
ges in a top ove\
rlay.\x0a  class Ti\
psManager {\x0a    \
// Lightweight t\
ips rotator that\
 works with your\
 CSS (.tips/.vis\
ible)\x0a    // and\
 is backward-com\
patible with leg\
acy `let tips = \
[...]` injection\
.\x0a    constructo\
r(dom) {\x0a      t\
his.dom = dom;\x0a \
     this.hidden\
 = false;\x0a      \
this._timers = [\
];\x0a      this._r\
unning = false;\x0a\
      this._idx \
= 0;\x0a    }\x0a\x0a    \
// Resolve tips \
list from multip\
le legacy/new so\
urces.\x0a    _getL\
ist() {\x0a      //\
 New preferred: \
window.TIPS (arr\
ay)\x0a      const \
upper = (typeof \
window !== 'unde\
fined') ? window\
.TIPS : undefine\
d;\x0a      if (Arr\
ay.isArray(upper\
) && upper.lengt\
h) return upper;\
\x0a\x0a      // Legac\
y inline: window\
.tips (array or \
JSON string)\x0a   \
   const lower =\
 (typeof window \
!== 'undefined')\
 ? window.tips :\
 undefined;\x0a    \
  if (Array.isAr\
ray(lower) && lo\
wer.length) retu\
rn lower;\x0a      \
if (typeof lower\
 === 'string' &&\
 lower.trim().le\
ngth) {\x0a        \
try { const arr \
= JSON.parse(low\
er); if (Array.i\
sArray(arr)) ret\
urn arr; } catch\
 (_) {}\x0a      }\x0a\
\x0a      // Option\
al: data-tips='[\
\x22...\x22,\x22...\x22]' on\
 #tips\x0a      con\
st host = this._\
host();\x0a      if\
 (host && host.d\
ataset && typeof\
 host.dataset.ti\
ps === 'string')\
 {\x0a        try {\
 const arr = JSO\
N.parse(host.dat\
aset.tips); if (\
Array.isArray(ar\
r)) return arr; \
} catch (_) {}\x0a \
     }\x0a\x0a      re\
turn [];\x0a    }\x0a\x0a\
    _host() {\x0a  \
    return this.\
dom.get('tips') \
|| document.getE\
lementById('tips\
');\x0a    }\x0a\x0a    _\
clearTimers() {\x0a\
      for (const\
 t of this._time\
rs) { try { clea\
rTimeout(t); } c\
atch (_) {} }\x0a  \
    this._timers\
.length = 0;\x0a   \
 }\x0a\x0a    // Stop \
any running rota\
tion timers.\x0a   \
 stopTimers() {\x0a\
      this._clea\
rTimers();\x0a     \
 this._running =\
 false;\x0a    }\x0a\x0a \
   _applyBaseSty\
le(el) {\x0a      i\
f (!el) return;\x0a\
      // Keep yo\
ur flex layout a\
nd sizing; do no\
t overwrite widt\
h/height.\x0a      \
// Ensure it ren\
ders above other\
 layers.\x0a      c\
onst z = (typeof\
 window !== 'und\
efined' && typeo\
f window.TIPS_ZI\
NDEX !== 'undefi\
ned')\x0a        ? \
String(window.TI\
PS_ZINDEX) : '21\
47483000';\x0a     \
 el.style.zIndex\
 = z;\x0a    }\x0a\x0a   \
 // Hide tips la\
yer and stop rot\
ation.\x0a    hide(\
) {\x0a      if (th\
is.hidden) retur\
n;\x0a      this.st\
opTimers();\x0a    \
  const el = thi\
s._host();\x0a     \
 if (el) {\x0a     \
   // Remove vis\
ibility class an\
d hide hard (use\
d when stream st\
arts etc.)\x0a     \
   el.classList.\
remove('visible'\
);\x0a        el.cl\
assList.remove('\
hidden'); // in \
case it was set \
elsewhere\x0a      \
  el.style.displ\
ay = 'none';\x0a   \
   }\x0a      this.\
hidden = true;\x0a \
   }\x0a\x0a    // Sho\
w tips layer (do\
es not start rot\
ation).\x0a    show\
() {\x0a      const\
 list = this._ge\
tList(); if (!li\
st.length) retur\
n;\x0a      const e\
l = this._host()\
; if (!el) retur\
n;\x0a\x0a      this.h\
idden = false;\x0a \
     this._apply\
BaseStyle(el);\x0a \
     el.classLis\
t.remove('hidden\
');\x0a      el.sty\
le.display = 'bl\
ock'; // CSS han\
dles opacity via\
 .tips/.visible\x0a\
      // Do not \
add 'visible' ye\
t \xe2\x80\x93 cycle() ta\
kes care of fade\
-in steps.\x0a    }\
\x0a\x0a    // Show on\
e tip (by index)\
 and fade it in \
next frame.\x0a    \
_showOne(idx) {\x0a\
      const list\
 = this._getList\
(); if (!list.le\
ngth) return;\x0a  \
    const el = t\
his._host(); if \
(!el || this.hid\
den) return;\x0a\x0a  \
    this._applyB\
aseStyle(el);\x0a  \
    el.innerHTML\
 = list[idx % li\
st.length];\x0a\x0a   \
   // Centralize\
 \x22next-frame\x22 vi\
sibility toggle \
through RafManag\
er to guarantee \
CSS transition.\x0a\
      try {\x0a    \
    if (typeof r\
untime !== 'unde\
fined' && runtim\
e.raf && typeof \
runtime.raf.sche\
dule === 'functi\
on') {\x0a         \
 const key = { t\
: 'Tips:show', e\
l, i: Math.rando\
m() };\x0a         \
 runtime.raf.sch\
edule(key, () =>\
 {\x0a            i\
f (this.hidden |\
| !el.isConnecte\
d) return;\x0a     \
       el.classL\
ist.add('visible\
');\x0a          },\
 'Tips', 2);\x0a   \
     } else {\x0a  \
        // Fallb\
ack: no frame de\
lay \xe2\x80\x93 still fu\
nctional, transi\
tion may not pla\
y.\x0a          el.\
classList.add('v\
isible');\x0a      \
  }\x0a      } catc\
h (_) {\x0a        \
el.classList.add\
('visible');\x0a   \
   }\x0a    }\x0a\x0a    \
// Internal loop\
: show, wait, hi\
de, wait fade, n\
ext.\x0a    _cycleL\
oop() {\x0a      if\
 (this.hidden) r\
eturn;\x0a      con\
st el = this._ho\
st(); if (!el) r\
eturn;\x0a\x0a      co\
nst VISIBLE_MS =\
 (typeof window \
!== 'undefined' \
&& window.TIPS_V\
ISIBLE_MS) ? win\
dow.TIPS_VISIBLE\
_MS : 15000;\x0a   \
   const FADE_MS\
    = (typeof wi\
ndow !== 'undefi\
ned' && window.T\
IPS_FADE_MS) ? w\
indow.TIPS_FADE_\
MS : 1000;\x0a\x0a    \
  this._showOne(\
this._idx);\x0a\x0a   \
   // Sequence: \
visible -> wait \
-> remove 'visib\
le' -> wait fade\
 -> next\x0a      t\
his._timers.push\
(setTimeout(() =\
> {\x0a        if (\
this.hidden) ret\
urn;\x0a        el.\
classList.remove\
('visible');\x0a   \
     this._timer\
s.push(setTimeou\
t(() => {\x0a      \
    if (this.hid\
den) return;\x0a   \
       const lis\
t = this._getLis\
t(); if (!list.l\
ength) return;\x0a \
         this._i\
dx = (this._idx \
+ 1) % list.leng\
th;\x0a          th\
is._cycleLoop();\
\x0a        }, FADE\
_MS));\x0a      }, \
VISIBLE_MS));\x0a  \
  }\x0a\x0a    // Star\
t rotation with \
initial delay.\x0a \
   cycle() {\x0a   \
   const list = \
this._getList();\
 if (!list.lengt\
h || this._runni\
ng) return;\x0a    \
  this._running \
= true; this._id\
x = 0;\x0a      thi\
s.show(); // mak\
e sure the host \
is visible and c\
entered\x0a\x0a      c\
onst INIT_DELAY \
= (typeof window\
 !== 'undefined'\
 && window.TIPS_\
INIT_DELAY_MS) ?\
 window.TIPS_INI\
T_DELAY_MS : 100\
00;\x0a      this._\
timers.push(setT\
imeout(() => {\x0a \
       if (this.\
hidden) return;\x0a\
        this._cy\
cleLoop();\x0a     \
 }, Math.max(0, \
INIT_DELAY)));\x0a \
   }\x0a\x0a    // Sto\
p and reset.\x0a   \
 cleanup() {\x0a   \
   this.stopTime\
rs();\x0a      cons\
t el = this._hos\
t();\x0a      if (e\
l) el.classList.\
remove('visible'\
);\x0a    }\x0a  }\x0a\x0a  \
// =============\
================\
================\
================\
=============\x0a  \
// 9) Tool outpu\
t + Nodes manage\
r\x0a  // =========\
================\
================\
================\
================\
=\x0a\x0a  class ToolO\
utput {\x0a    // P\
laceholder for l\
oader show (can \
be extended by h\
ost).\x0a    showLo\
ader() { return;\
 }\x0a    // Hide s\
pinner elements \
in bot messages.\
\x0a    hideLoader(\
) {\x0a      const \
elements = docum\
ent.querySelecto\
rAll('.msg-bot')\
;\x0a      if (elem\
ents.length > 0)\
 elements.forEac\
h(el => { const \
s = el.querySele\
ctor('.spinner')\
; if (s) s.style\
.display = 'none\
'; });\x0a    }\x0a   \
 begin() { this.\
showLoader(); }\x0a\
    end() { this\
.hideLoader(); }\
\x0a    enable() { \
const els = docu\
ment.querySelect\
orAll('.tool-out\
put'); if (els.l\
ength) els[els.l\
ength - 1].style\
.display = 'bloc\
k'; }\x0a    disabl\
e() { const els \
= document.query\
SelectorAll('.to\
ol-output'); if \
(els.length) els\
[els.length - 1]\
.style.display =\
 'none'; }\x0a    /\
/ Append HTML in\
to the latest to\
ol-output conten\
t area.\x0a    appe\
nd(content) {\x0a  \
    this.hideLoa\
der(); this.enab\
le();\x0a      cons\
t els = document\
.querySelectorAl\
l('.tool-output'\
);\x0a      if (els\
.length) { const\
 contentEl = els\
[els.length - 1]\
.querySelector('\
.content'); if (\
contentEl) conte\
ntEl.insertAdjac\
entHTML('beforee\
nd', content); }\
\x0a    }\x0a    // Re\
place inner HTML\
 for the latest \
tool-output cont\
ent area.\x0a    up\
date(content) {\x0a\
      this.hideL\
oader(); this.en\
able();\x0a      co\
nst els = docume\
nt.querySelector\
All('.tool-outpu\
t');\x0a      if (e\
ls.length) { con\
st contentEl = e\
ls[els.length - \
1].querySelector\
('.content'); if\
 (contentEl) con\
tentEl.innerHTML\
 = content; }\x0a  \
  }\x0a    // Remov\
e children from \
the latest tool-\
output content a\
rea.\x0a    clear()\
 {\x0a      this.hi\
deLoader(); this\
.enable();\x0a     \
 const els = doc\
ument.querySelec\
torAll('.tool-ou\
tput');\x0a      if\
 (els.length) { \
const contentEl \
= els[els.length\
 - 1].querySelec\
tor('.content');\
 if (contentEl) \
contentEl.replac\
eChildren(); }\x0a \
   }\x0a    // Togg\
le visibility of\
 a specific tool\
 output block by\
 message id.\x0a   \
 toggle(id) {\x0a  \
    const el = d\
ocument.getEleme\
ntById('msg-bot-\
' + id); if (!el\
) return;\x0a      \
const outputEl =\
 el.querySelecto\
r('.tool-output'\
); if (!outputEl\
) return;\x0a      \
const contentEl \
= outputEl.query\
Selector('.conte\
nt');\x0a      if (\
contentEl) conte\
ntEl.style.displ\
ay = (contentEl.\
style.display ==\
= 'none') ? 'blo\
ck' : 'none';\x0a  \
    const toggle\
El = outputEl.qu\
erySelector('.to\
ggle-cmd-output \
img'); if (toggl\
eEl) toggleEl.cl\
assList.toggle('\
toggle-expanded'\
);\x0a    }\x0a  }\x0a\x0a  \
  // UserCollaps\
eManager \xe2\x80\x93 col\
lapsible user me\
ssages (msg-box.\
msg-user)\x0a    cl\
ass UserCollapse\
Manager {\x0a      \
constructor(cfg)\
 {\x0a        this.\
cfg = cfg || {};\
\x0a        // Coll\
apse threshold i\
n pixels (can be\
 overridden via \
window.USER_MSG_\
COLLAPSE_HEIGHT_\
PX).\x0a        thi\
s.threshold = Ut\
ils.g('USER_MSG_\
COLLAPSE_HEIGHT_\
PX', 1000);\x0a    \
    // Track pro\
cessed .msg elem\
ents to allow ch\
eap remeasure on\
 resize if neede\
d.\x0a        this.\
_processed = new\
 Set();\x0a\x0a       \
 // Visual indic\
ator attached wh\
ile collapsed (d\
oes not modify o\
riginal text).\x0a \
       this.elli\
psisText = ' [..\
.]';\x0a      }\x0a\x0a  \
    _icons() {\x0a \
       const I =\
 (this.cfg && th\
is.cfg.ICONS) ||\
 {};\x0a        ret\
urn { expand: I.\
EXPAND || '', co\
llapse: I.COLLAP\
SE || '' };\x0a    \
  }\x0a      _label\
s() {\x0a        co\
nst L = (this.cf\
g && this.cfg.LO\
CALE) || {};\x0a   \
     return { ex\
pand: L.EXPAND |\
| 'Expand', coll\
apse: L.COLLAPSE\
 || 'Collapse' }\
;\x0a      }\x0a\x0a     \
 // Schedule a f\
unction for next\
 frame (ensures \
layout is up-to-\
date before scro\
lling).\x0a      _a\
fterLayout(fn) {\
\x0a        try {\x0a \
         if (typ\
eof runtime !== \
'undefined' && r\
untime.raf && ty\
peof runtime.raf\
.schedule === 'f\
unction') {\x0a    \
        const ke\
y = { t: 'UC:aft\
erLayout', i: Ma\
th.random() };\x0a \
           runti\
me.raf.schedule(\
key, () => { try\
 { fn && fn(); }\
 catch (_) {} },\
 'UserCollapse',\
 0);\x0a           \
 return;\x0a       \
   }\x0a        } c\
atch (_) {}\x0a    \
    try { reques\
tAnimationFrame(\
() => { try { fn\
 && fn(); } catc\
h (_) {} }); }\x0a \
       catch (_)\
 { setTimeout(()\
 => { try { fn &\
& fn(); } catch \
(__){ } }, 0); }\
\x0a      }\x0a\x0a      \
// Bring toggle \
into view with m\
inimal scroll (u\
pwards if it mov\
ed above after c\
ollapse).\x0a      \
_scrollToggleInt\
oView(toggleEl) \
{\x0a        if (!t\
oggleEl || !togg\
leEl.isConnected\
) return;\x0a      \
  try { if (runt\
ime && runtime.s\
crollMgr) { runt\
ime.scrollMgr.us\
erInteracted = t\
rue; runtime.scr\
ollMgr.autoFollo\
w = false; } } c\
atch (_) {}\x0a    \
    this._afterL\
ayout(() => {\x0a  \
        try {\x0a  \
          if (to\
ggleEl.scrollInt\
oView) {\x0a       \
       // Prefer\
 minimal movemen\
t; keep behavior\
 non-animated an\
d predictable.\x0a \
             try\
 { toggleEl.scro\
llIntoView({ blo\
ck: 'nearest', i\
nline: 'nearest'\
, behavior: 'ins\
tant' }); }\x0a    \
          catch \
(_) { toggleEl.s\
crollIntoView(fa\
lse); }\x0a        \
    }\x0a          \
} catch (_) {}\x0a \
       });\x0a     \
 }\x0a\x0a      // Ens\
ure wrapper and \
toggle exist for\
 a given .msg el\
ement.\x0a      _en\
sureStructure(ms\
g) {\x0a        if \
(!msg || !msg.is\
Connected) retur\
n null;\x0a\x0a       \
 // Wrap all dir\
ect children int\
o a dedicated co\
ntent container \
to measure heigh\
t accurately.\x0a  \
      let conten\
t = msg.querySel\
ector('.uc-conte\
nt');\x0a        if\
 (!content) {\x0a  \
        content \
= document.creat\
eElement('div');\
\x0a          conte\
nt.className = '\
uc-content';\x0a   \
       const fra\
g = document.cre\
ateDocumentFragm\
ent();\x0a         \
 while (msg.firs\
tChild) frag.app\
endChild(msg.fir\
stChild);\x0a      \
    content.appe\
ndChild(frag);\x0a \
         msg.app\
endChild(content\
);\x0a        }\x0a\x0a  \
      // Ensure \
a single toggle \
exists (click an\
d keyboard acces\
sible).\x0a        \
let toggle = msg\
.querySelector('\
.uc-toggle');\x0a  \
      if (!toggl\
e) {\x0a          c\
onst icons = thi\
s._icons();\x0a    \
      const labe\
ls = this._label\
s();\x0a\x0a          \
toggle = documen\
t.createElement(\
'div');\x0a        \
  toggle.classNa\
me = 'uc-toggle'\
;\x0a          togg\
le.tabIndex = 0;\
\x0a          toggl\
e.setAttribute('\
role', 'button')\
;\x0a          togg\
le.setAttribute(\
'aria-expanded',\
 'false');\x0a     \
     toggle.titl\
e = labels.expan\
d;\x0a\x0a          co\
nst img = docume\
nt.createElement\
('img');\x0a       \
   img.className\
 = 'uc-toggle-ic\
on';\x0a          i\
mg.alt = labels.\
expand;\x0a        \
  img.src = icon\
s.expand;\x0a\x0a     \
     // Provide \
a sane default s\
ize even if CSS \
did not load yet\
 (CSS will overr\
ide when present\
).\x0a          img\
.width = 26;   /\
/ keep in sync w\
ith CSS fallback\
 var(--uc-toggle\
-icon-size, 26px\
)\x0a          img.\
height = 26;  //\
 ensures a consi\
stent, non-tiny \
control from the\
 first paint\x0a\x0a  \
        toggle.a\
ppendChild(img);\
\x0a\x0a          // A\
ttach local list\
eners (no global\
 handler change;\
 production-safe\
).\x0a          tog\
gle.addEventList\
ener('click', (e\
v) => {\x0a        \
    ev.preventDe\
fault();\x0a       \
     ev.stopProp\
agation();\x0a     \
       this.togg\
leFromToggle(tog\
gle);\x0a          \
});\x0a          to\
ggle.addEventLis\
tener('keydown',\
 (ev) => {\x0a     \
       if (ev.ke\
y === 'Enter' ||\
 ev.key === ' ')\
 {\x0a             \
 ev.preventDefau\
lt();\x0a          \
    ev.stopPropa\
gation();\x0a      \
        this.tog\
gleFromToggle(to\
ggle);\x0a         \
   }\x0a          }\
, { passive: fal\
se });\x0a\x0a        \
  msg.appendChil\
d(toggle);\x0a     \
   }\x0a\x0a        th\
is._processed.ad\
d(msg);\x0a        \
msg.dataset.ucIn\
it = '1';\x0a      \
  return { conte\
nt, toggle };\x0a  \
    }\x0a\x0a      // \
Create or update\
 the ellipsis in\
dicator inside c\
ontent (absolute\
 in the bottom-r\
ight corner).\x0a  \
    _ensureEllip\
sisEl(msg, conte\
ntEl) {\x0a        \
const content = \
contentEl || (ms\
g && msg.querySe\
lector('.uc-cont\
ent'));\x0a        \
if (!content) re\
turn null;\x0a\x0a    \
    // Ensure th\
e content become\
s a positioning \
context only whe\
n needed.\x0a      \
  if (getCompute\
dStyle(content).\
position === 'st\
atic') {\x0a       \
   content.style\
.position = 'rel\
ative';\x0a        \
}\x0a\x0a        let d\
ot = content.que\
rySelector('.uc-\
ellipsis');\x0a    \
    if (!dot) {\x0a\
          dot = \
document.createE\
lement('span');\x0a\
          dot.cl\
assName = 'uc-el\
lipsis';\x0a       \
   dot.textConte\
nt = this.ellips\
isText;\x0a        \
  // Inline, the\
me-agnostic styl\
es; kept minimal\
 and non-interac\
tive.\x0a          \
dot.style.positi\
on = 'absolute';\
\x0a          dot.s\
tyle.right = '0'\
;\x0a          dot.\
style.bottom = '\
0';\x0a          do\
t.style.paddingL\
eft = '6px';\x0a   \
       dot.style\
.pointerEvents =\
 'none';\x0a       \
   dot.style.zIn\
dex = '1';\x0a     \
     dot.style.f\
ontWeight = '500\
';\x0a          dot\
.style.opacity =\
 '0.75';\x0a\x0a      \
    content.appe\
ndChild(dot);\x0a  \
      }\x0a        \
return dot;\x0a    \
  }\x0a\x0a      // Sh\
ow ellipsis only\
 when there is h\
idden overflow (\
collapsed).\x0a    \
  _showEllipsis(\
msg, contentEl) \
{\x0a        const \
dot = this._ensu\
reEllipsisEl(msg\
, contentEl);\x0a  \
      if (dot) d\
ot.style.display\
 = 'inline';\x0a   \
   }\x0a      // Hi\
de and clean ell\
ipsis when not n\
eeded (expanded \
or short content\
).\x0a      _hideEl\
lipsis(msg) {\x0a  \
      const cont\
ent = msg && msg\
.querySelector('\
.uc-content');\x0a \
       if (!cont\
ent) return;\x0a   \
     const dot =\
 content.querySe\
lector('.uc-elli\
psis');\x0a        \
if (dot && dot.p\
arentNode) {\x0a   \
       // Remove\
 the indicator t\
o avoid accident\
al copy/select a\
nd keep DOM lean\
.\x0a          dot.\
parentNode.remov\
eChild(dot);\x0a   \
     }\x0a        /\
/ Drop positioni\
ng context when \
no indicator is \
present (keep st\
yles minimal).\x0a \
       try {\x0a   \
       if (conte\
nt && content.st\
yle && content.q\
uerySelector('.u\
c-ellipsis') == \
null) {\x0a        \
    content.styl\
e.position = '';\
\x0a          }\x0a   \
     } catch (_)\
 {}\x0a      }\x0a\x0a   \
   // Apply coll\
apse to all user\
 messages under \
root.\x0a      appl\
y(root) {\x0a      \
  const scope = \
root || document\
;\x0a        let li\
st;\x0a        if (\
scope.nodeType =\
== 1) list = sco\
pe.querySelector\
All('.msg-box.ms\
g-user .msg');\x0a \
       else list\
 = document.quer\
ySelectorAll('.m\
sg-box.msg-user \
.msg');\x0a        \
if (!list || !li\
st.length) retur\
n;\x0a\x0a        for \
(let i = 0; i < \
list.length; i++\
) {\x0a          co\
nst msg = list[i\
];\x0a          con\
st st = this._en\
sureStructure(ms\
g);\x0a          if\
 (!st) continue;\
\x0a          this.\
_update(msg, st.\
content, st.togg\
le);\x0a        }\x0a \
     }\x0a\x0a      //\
 Update collapse\
d/expanded state\
 depending on co\
ntent height.\x0a  \
    _update(msg,\
 contentEl, togg\
leEl) {\x0a        \
const c = conten\
tEl || (msg && m\
sg.querySelector\
('.uc-content'))\
;\x0a        if (!m\
sg || !c) return\
;\x0a\x0a        // Sp\
ecial-case: when\
 threshold = 0 (\
or '0'), auto-co\
llapse is global\
ly disabled.\x0a   \
     // We avoid\
 any measurement\
, force the cont\
ent to be fully \
expanded, and en\
sure the toggle \
is hidden.\x0a     \
   // This prese\
rves public API \
while providing \
an explicit opt-\
out, without imp\
acting existing \
behavior.\x0a      \
  if (this.thres\
hold === 0 || th\
is.threshold ===\
 '0') {\x0a        \
  const t = togg\
leEl || msg.quer\
ySelector('.uc-t\
oggle');\x0a       \
   const labels \
= this._labels()\
;\x0a\x0a          // \
Ensure expanded \
state and remove\
 any limiting cl\
asses.\x0a         \
 c.classList.rem\
ove('uc-collapse\
d');\x0a          c\
.classList.remov\
e('uc-expanded')\
; // No class =>\
 fully expanded \
by default CSS.\x0a\
          msg.da\
taset.ucState = \
'expanded';\x0a\x0a   \
       // Hide e\
llipsis in disab\
led mode.\x0a      \
    this._hideEl\
lipsis(msg);\x0a\x0a  \
        // Hide \
toggle in disabl\
ed mode to avoid\
 user interactio\
n.\x0a          if \
(t) {\x0a          \
  t.classList.re\
move('visible');\
\x0a            t.s\
etAttribute('ari\
a-expanded', 'fa\
lse');\x0a         \
   t.title = lab\
els.expand;\x0a    \
        const im\
g = t.querySelec\
tor('img');\x0a    \
        if (img)\
 { img.alt = lab\
els.expand; }\x0a  \
        }\x0a      \
    return; // D\
o not proceed wi\
th measuring or \
collapsing.\x0a    \
    }\x0a\x0a        /\
/ Temporarily re\
move limiting cl\
asses for precis\
e measurement.\x0a \
       c.classLi\
st.remove('uc-co\
llapsed');\x0a     \
   c.classList.r\
emove('uc-expand\
ed');\x0a\x0a        c\
onst fullHeight \
= Math.ceil(c.sc\
rollHeight);\x0a   \
     const label\
s = this._labels\
();\x0a        cons\
t icons = this._\
icons();\x0a       \
 const t = toggl\
eEl || msg.query\
Selector('.uc-to\
ggle');\x0a\x0a       \
 if (fullHeight \
> this.threshold\
) {\x0a          if\
 (t) t.classList\
.add('visible');\
\x0a          const\
 desired = msg.d\
ataset.ucState |\
| 'collapsed';\x0a \
         const e\
xpand = (desired\
 === 'expanded')\
;\x0a\x0a          if \
(expand) {\x0a     \
       c.classLi\
st.add('uc-expan\
ded');\x0a         \
   this._hideEll\
ipsis(msg); // E\
xpanded => no el\
lipsis\x0a         \
 } else {\x0a      \
      c.classLis\
t.add('uc-collap\
sed');\x0a         \
   this._showEll\
ipsis(msg, c); /\
/ Collapsed => s\
how ellipsis ove\
rlay\x0a          }\
\x0a\x0a          if (\
t) {\x0a           \
 const img = t.q\
uerySelector('im\
g');\x0a           \
 if (img) {\x0a    \
          if (ex\
pand) { img.src \
= icons.collapse\
; img.alt = labe\
ls.collapse; }\x0a \
             els\
e { img.src = ic\
ons.expand; img.\
alt = labels.exp\
and; }\x0a         \
   }\x0a           \
 t.setAttribute(\
'aria-expanded',\
 expand ? 'true'\
 : 'false');\x0a   \
         t.title\
 = expand ? labe\
ls.collapse : la\
bels.expand;\x0a   \
       }\x0a       \
 } else {\x0a      \
    // Short con\
tent \xe2\x80\x93 ensure \
fully expanded a\
nd hide toggle +\
 ellipsis.\x0a     \
     c.classList\
.remove('uc-coll\
apsed');\x0a       \
   c.classList.r\
emove('uc-expand\
ed');\x0a          \
msg.dataset.ucSt\
ate = 'expanded'\
;\x0a          this\
._hideEllipsis(m\
sg);\x0a          i\
f (t) {\x0a        \
    t.classList.\
remove('visible'\
);\x0a            t\
.setAttribute('a\
ria-expanded', '\
false');\x0a       \
     t.title = l\
abels.expand;\x0a  \
        }\x0a      \
  }\x0a      }\x0a\x0a   \
   // Toggle han\
dler via the tog\
gle element (div\
.uc-toggle).\x0a   \
   toggleFromTog\
gle(toggleEl) {\x0a\
        const ms\
g = toggleEl && \
toggleEl.closest\
 ? toggleEl.clos\
est('.msg-box.ms\
g-user .msg') : \
null;\x0a        if\
 (!msg) return;\x0a\
        this.tog\
gle(msg);\x0a      \
}\x0a\x0a      // Core\
 toggle logic.\x0a \
     toggle(msg)\
 {\x0a        if (!\
msg || !msg.isCo\
nnected) return;\
\x0a        const c\
 = msg.querySele\
ctor('.uc-conten\
t'); if (!c) ret\
urn;\x0a        con\
st t = msg.query\
Selector('.uc-to\
ggle');\x0a        \
const labels = t\
his._labels();\x0a \
       const ico\
ns = this._icons\
();\x0a\x0a        con\
st isCollapsed =\
 c.classList.con\
tains('uc-collap\
sed');\x0a        i\
f (isCollapsed) \
{\x0a          // E\
xpand \xe2\x80\x93 leave \
scroll as-is; re\
move ellipsis.\x0a \
         c.class\
List.remove('uc-\
collapsed');\x0a   \
       c.classLi\
st.add('uc-expan\
ded');\x0a         \
 msg.dataset.ucS\
tate = 'expanded\
';\x0a          thi\
s._hideEllipsis(\
msg);\x0a          \
if (t) {\x0a       \
     t.setAttrib\
ute('aria-expand\
ed', 'true');\x0a  \
          t.titl\
e = labels.colla\
pse;\x0a           \
 const img = t.q\
uerySelector('im\
g'); if (img) { \
img.src = icons.\
collapse; img.al\
t = labels.colla\
pse; }\x0a         \
 }\x0a        } els\
e {\x0a          //\
 Collapse \xe2\x80\x93 ap\
ply classes, sho\
w ellipsis, then\
 bring toggle in\
to view (scroll \
up if needed).\x0a \
         c.class\
List.remove('uc-\
expanded');\x0a    \
      c.classLis\
t.add('uc-collap\
sed');\x0a         \
 msg.dataset.ucS\
tate = 'collapse\
d';\x0a          th\
is._showEllipsis\
(msg, c);\x0a      \
    if (t) {\x0a   \
         t.setAt\
tribute('aria-ex\
panded', 'false'\
);\x0a            t\
.title = labels.\
expand;\x0a        \
    const img = \
t.querySelector(\
'img'); if (img)\
 { img.src = ico\
ns.expand; img.a\
lt = labels.expa\
nd; }\x0a          \
  // Follow the \
collapsing conte\
nt upward \xe2\x80\x93 ke\
ep the toggle vi\
sible.\x0a         \
   this._scrollT\
oggleIntoView(t)\
;\x0a          }\x0a  \
      }\x0a      }\x0a\
\x0a      // Option\
al public method\
 to re-evaluate \
height after lay\
out/resize.\x0a    \
  remeasureAll()\
 {\x0a        const\
 arr = Array.fro\
m(this._processe\
d || []);\x0a      \
  for (let i = 0\
; i < arr.length\
; i++) {\x0a       \
   const msg = a\
rr[i];\x0a         \
 if (!msg || !ms\
g.isConnected) {\
 this._processed\
.delete(msg); co\
ntinue; }\x0a      \
    this._update\
(msg);\x0a        }\
\x0a      }\x0a    }\x0a\x0a\
  class NodesMan\
ager {\x0a      con\
structor(dom, re\
nderer, highligh\
ter, math) {\x0a   \
     this.dom = \
dom;\x0a        thi\
s.renderer = ren\
derer;\x0a        t\
his.highlighter \
= highlighter;\x0a \
       this.math\
 = math;\x0a       \
 // User message\
 collapse manage\
r\x0a        this._\
userCollapse = n\
ew UserCollapseM\
anager(this.rend\
erer.cfg);\x0a     \
 }\x0a\x0a      // Che\
ck if HTML conta\
ins only user me\
ssages without a\
ny markdown or c\
ode features.\x0a  \
    _isUserOnlyC\
ontent(html) {\x0a \
       try {\x0a   \
       const tmp\
 = document.crea\
teElement('div')\
;\x0a          tmp.\
innerHTML = html\
;\x0a          cons\
t hasBot = !!tmp\
.querySelector('\
.msg-box.msg-bot\
');\x0a          co\
nst hasUser = !!\
tmp.querySelecto\
r('.msg-box.msg-\
user');\x0a        \
  const hasMD64 \
= !!tmp.querySel\
ector('[data-md6\
4]');\x0a          \
const hasMDNativ\
e = !!tmp.queryS\
elector('[md-blo\
ck-markdown]');\x0a\
          const \
hasCode = !!tmp.\
querySelector('p\
re code');\x0a     \
     const hasMa\
th = !!tmp.query\
Selector('script\
[type^=\x22math/tex\
\x22]');\x0a          \
return hasUser &\
& !hasBot && !ha\
sMD64 && !hasMDN\
ative && !hasCod\
e && !hasMath;\x0a \
       } catch (\
_) { return fals\
e; }\x0a      }\x0a\x0a  \
    // Convert u\
ser markdown pla\
ceholders into p\
lain text nodes.\
\x0a      _material\
izeUserMdAsPlain\
Text(scopeEl) {\x0a\
        try {\x0a  \
        const no\
des = scopeEl.qu\
erySelectorAll('\
.msg-box.msg-use\
r [data-md64], .\
msg-box.msg-user\
 [md-block-markd\
own]');\x0a        \
  nodes.forEach(\
el => {\x0a        \
    let txt = ''\
;\x0a            if\
 (el.hasAttribut\
e('data-md64')) \
{\x0a              \
const b64 = el.g\
etAttribute('dat\
a-md64') || '';\x0a\
              el\
.removeAttribute\
('data-md64');\x0a \
             try\
 { txt = this.re\
nderer.b64ToUtf8\
(b64); } catch (\
_) { txt = ''; }\
\x0a            } e\
lse {\x0a          \
    // Native Ma\
rkdown block in \
user message: ke\
ep as plain text\
 (no markdown-it\
)\x0a              \
try { txt = el.t\
extContent || ''\
; } catch (_) { \
txt = ''; }\x0a    \
          try { \
el.removeAttribu\
te('md-block-mar\
kdown'); } catch\
 (_) {}\x0a        \
    }\x0a          \
  const span = d\
ocument.createEl\
ement('span'); s\
pan.textContent \
= txt; el.replac\
eWith(span);\x0a   \
       });\x0a     \
   } catch (_) {\
}\x0a      }\x0a\x0a     \
 // Append HTML/\
text into the me\
ssage input cont\
ainer.\x0a        /\
/ If plain text \
is provided, wra\
p it into a mini\
mal msg-user box\
 to keep layout \
consistent.\x0a    \
    appendToInpu\
t(content) {\x0a   \
       const el \
= this.dom.get('\
_append_input_')\
; if (!el) retur\
n;\x0a\x0a          le\
t html = String(\
content || '');\x0a\
          const \
trimmed = html.t\
rim();\x0a\x0a        \
  // If already \
a full msg-user \
wrapper, append \
as-is; otherwise\
 wrap the plain \
text.\x0a          \
const isWrapped \
= (trimmed.start\
sWith('<div') &&\
 /class=[\x22']msg-\
box msg-user[\x22']\
/.test(trimmed))\
;\x0a          if (\
!isWrapped) {\x0a  \
          // Tre\
at incoming payl\
oad as plain tex\
t (escape + conv\
ert newlines to \
<br>).\x0a         \
   const safe = \
(typeof Utils !=\
= 'undefined' &&\
 Utils.escapeHtm\
l)\x0a             \
 ? Utils.escapeH\
tml(html)\x0a      \
        : String\
(html).replace(/\
[&<>\x22']/g, m => \
({'&':'&amp;','<\
':'&lt;','>':'&g\
t;','\x22':'&quot;'\
,\x22'\x22:'&#039;'}[m\
]));\x0a           \
 const body = sa\
fe.replace(/\x5cr?\x5c\
n/g, '<br>');\x0a  \
          // Min\
imal, margin-les\
s user message (\
no empty msg-ext\
ra to avoid extr\
a spacing).\x0a    \
        html = `\
<div class=\x22msg-\
box msg-user\x22><d\
iv class=\x22msg\x22><\
p style=\x22margin:\
0\x22>${body}</p></\
div></div>`;\x0a   \
       }\x0a\x0a      \
    // Synchrono\
us DOM update.\x0a \
         el.inse\
rtAdjacentHTML('\
beforeend', html\
);\x0a\x0a          //\
 Apply collapse \
to any user mess\
ages in input ar\
ea (now or later\
).\x0a          try\
 { this._userCol\
lapse.apply(el);\
 } catch (_) {}\x0a\
        }\x0a\x0a     \
 // Append nodes\
 into messages l\
ist and perform \
post-processing \
(markdown, code,\
 math).\x0a      ap\
pendNode(content\
, scrollMgr) {\x0a \
       // Keep s\
croll behavior c\
onsistent with e\
xisting logic\x0a  \
      scrollMgr.\
userInteracted =\
 false; scrollMg\
r.prevScroll = 0\
;\x0a        this.d\
om.clearStreamBe\
fore();\x0a\x0a       \
 const el = this\
.dom.get('_nodes\
_'); if (!el) re\
turn;\x0a        el\
.classList.remov\
e('empty_list');\
\x0a\x0a        const \
userOnly = this.\
_isUserOnlyConte\
nt(content);\x0a   \
     if (userOnl\
y) {\x0a          e\
l.insertAdjacent\
HTML('beforeend'\
, content);\x0a    \
      this._mate\
rializeUserMdAsP\
lainText(el);\x0a  \
        // Colla\
pse before scrol\
ling to ensure f\
inal height is u\
sed for scroll c\
omputations.\x0a   \
       try { thi\
s._userCollapse.\
apply(el); } cat\
ch (_) {}\x0a      \
    scrollMgr.sc\
rollToBottom(fal\
se);\x0a          s\
crollMgr.schedul\
eScrollFabUpdate\
();\x0a          re\
turn;\x0a        }\x0a\
\x0a        el.inse\
rtAdjacentHTML('\
beforeend', cont\
ent);\x0a\x0a        t\
ry {\x0a          /\
/ Defer post-pro\
cessing (highlig\
ht/math/collapse\
) and perform sc\
roll AFTER colla\
pse.\x0a          c\
onst maybePromis\
e = this.rendere\
r.renderPendingM\
arkdown(el);\x0a   \
       const pos\
t = () => {\x0a    \
        // Viewp\
ort highlight sc\
heduling\x0a       \
     try { this.\
highlighter.sche\
duleScanVisibleC\
odes(null); } ca\
tch (_) {}\x0a\x0a    \
        // In fi\
nalize-only mode\
 we must explici\
tly schedule KaT\
eX\x0a            t\
ry { if (getMath\
Mode() === 'fina\
lize-only') this\
.math.schedule(e\
l, 0, true); } c\
atch (_) {}\x0a\x0a   \
         // Coll\
apse user messag\
es now that DOM \
is materialized \
(ensures correct\
 height).\x0a      \
      try { this\
._userCollapse.a\
pply(el); } catc\
h (_) {}\x0a\x0a      \
      // Only no\
w scroll to bott\
om and update FA\
B \xe2\x80\x93 uses post-\
collapse heights\
.\x0a            sc\
rollMgr.scrollTo\
Bottom(false);\x0a \
           scrol\
lMgr.scheduleScr\
ollFabUpdate();\x0a\
          };\x0a\x0a  \
        if (mayb\
ePromise && type\
of maybePromise.\
then === 'functi\
on') {\x0a         \
   maybePromise.\
then(post);\x0a    \
      } else {\x0a \
           post(\
);\x0a          }\x0a \
       } catch (\
_) {\x0a          /\
/ In case of err\
or, do a conserv\
ative scroll to \
keep UX responsi\
ve.\x0a          sc\
rollMgr.scrollTo\
Bottom(false);\x0a \
         scrollM\
gr.scheduleScrol\
lFabUpdate();\x0a  \
      }\x0a      }\x0a\
\x0a      // Replac\
e messages list \
content entirely\
 and re-run post\
-processing.\x0a   \
   replaceNodes(\
content, scrollM\
gr) {\x0a        //\
 Same semantics \
as appendNode, b\
ut using a hard \
clone reset\x0a    \
    scrollMgr.us\
erInteracted = f\
alse; scrollMgr.\
prevScroll = 0;\x0a\
        this.dom\
.clearStreamBefo\
re();\x0a\x0a        c\
onst el = this.d\
om.hardReplaceBy\
Clone('_nodes_')\
; if (!el) retur\
n;\x0a        el.cl\
assList.remove('\
empty_list');\x0a\x0a \
       const use\
rOnly = this._is\
UserOnlyContent(\
content);\x0a      \
  if (userOnly) \
{\x0a          el.i\
nsertAdjacentHTM\
L('beforeend', c\
ontent);\x0a       \
   this._materia\
lizeUserMdAsPlai\
nText(el);\x0a     \
     // Collapse\
 before scrollin\
g to ensure fina\
l height is used\
 for scroll comp\
utations.\x0a      \
    try { this._\
userCollapse.app\
ly(el); } catch \
(_) {}\x0a         \
 scrollMgr.scrol\
lToBottom(false,\
 true);\x0a        \
  scrollMgr.sche\
duleScrollFabUpd\
ate();\x0a         \
 return;\x0a       \
 }\x0a\x0a        el.i\
nsertAdjacentHTM\
L('beforeend', c\
ontent);\x0a\x0a      \
  try {\x0a        \
  // Defer KaTeX\
 schedule to pos\
t-Markdown to av\
oid races and co\
llapse before sc\
roll.\x0a          \
const maybePromi\
se = this.render\
er.renderPending\
Markdown(el);\x0a  \
        const po\
st = () => {\x0a   \
         try { t\
his.highlighter.\
scheduleScanVisi\
bleCodes(null); \
} catch (_) {}\x0a \
           try {\
 if (getMathMode\
() === 'finalize\
-only') this.mat\
h.schedule(el, 0\
, true); } catch\
 (_) {}\x0a\x0a       \
     // Collapse\
 after materiali\
zation to comput\
e final heights \
correctly.\x0a     \
       try { thi\
s._userCollapse.\
apply(el); } cat\
ch (_) {}\x0a\x0a     \
       // Now sc\
roll and update \
FAB using the co\
llapsed layout.\x0a\
            scro\
llMgr.scrollToBo\
ttom(false, true\
);\x0a            s\
crollMgr.schedul\
eScrollFabUpdate\
();\x0a          };\
\x0a\x0a          if (\
maybePromise && \
typeof maybeProm\
ise.then === 'fu\
nction') {\x0a     \
       maybeProm\
ise.then(post);\x0a\
          } else\
 {\x0a            p\
ost();\x0a         \
 }\x0a        } cat\
ch (_) {\x0a       \
   scrollMgr.scr\
ollToBottom(fals\
e, true);\x0a      \
    scrollMgr.sc\
heduleScrollFabU\
pdate();\x0a       \
 }\x0a      }\x0a\x0a    \
  // Append \x22ext\
ra\x22 content into\
 a specific bot \
message and post\
-process locally\
.\x0a      appendEx\
tra(id, content,\
 scrollMgr) {\x0a  \
      const el =\
 document.getEle\
mentById('msg-bo\
t-' + id); if (!\
el) return;\x0a    \
    const extra \
= el.querySelect\
or('.msg-extra')\
; if (!extra) re\
turn;\x0a\x0a        e\
xtra.insertAdjac\
entHTML('beforee\
nd', content);\x0a\x0a\
        try {\x0a  \
        const ma\
ybePromise = thi\
s.renderer.rende\
rPendingMarkdown\
(extra);\x0a\x0a      \
    const post =\
 () => {\x0a       \
     const activ\
eCode = (typeof \
runtime !== 'und\
efined' && runti\
me.stream) ? run\
time.stream.acti\
veCode : null;\x0a\x0a\
            // A\
ttach observers \
after Markdown p\
roduced the node\
s\x0a            tr\
y {\x0a            \
  this.highlight\
er.observeNewCod\
e(extra, {\x0a     \
           defer\
LastIfStreaming:\
 true,\x0a         \
       minLinesF\
orLast: this.ren\
derer.cfg.PROFIL\
E_CODE.minLinesF\
orHL,\x0a          \
      minCharsFo\
rLast: this.rend\
erer.cfg.PROFILE\
_CODE.minCharsFo\
rHL\x0a            \
  }, activeCode)\
;\x0a              \
this.highlighter\
.observeMsgBoxes\
(extra, (box) =>\
 this._onBox(box\
));\x0a            \
} catch (_) {}\x0a\x0a\
            // K\
aTeX: honor stre\
am mode; in fina\
lize-only force \
immediate schedu\
le\x0a            t\
ry {\x0a           \
   const mm = ge\
tMathMode();\x0a   \
           if (m\
m === 'finalize-\
only') this.math\
.schedule(extra,\
 0, true);\x0a     \
         else th\
is.math.schedule\
(extra);\x0a       \
     } catch (_)\
 {}\x0a          };\
\x0a\x0a          if (\
maybePromise && \
typeof maybeProm\
ise.then === 'fu\
nction') {\x0a     \
       maybeProm\
ise.then(post);\x0a\
          } else\
 {\x0a            p\
ost();\x0a         \
 }\x0a        } cat\
ch (_) { /* swal\
low */ }\x0a\x0a      \
  scrollMgr.sche\
duleScroll(true)\
;\x0a      }\x0a\x0a     \
 // When a new m\
essage box appea\
rs, hook up code\
/highlight handl\
ers.\x0a      _onBo\
x(box) {\x0a       \
 const activeCod\
e = (typeof runt\
ime !== 'undefin\
ed' && runtime.s\
tream) ? runtime\
.stream.activeCo\
de : null;\x0a     \
   this.highligh\
ter.observeNewCo\
de(box, {\x0a      \
    deferLastIfS\
treaming: true,\x0a\
          minLin\
esForLast: this.\
renderer.cfg.PRO\
FILE_CODE.minLin\
esForHL,\x0a       \
   minCharsForLa\
st: this.rendere\
r.cfg.PROFILE_CO\
DE.minCharsForHL\
\x0a        }, acti\
veCode);\x0a       \
 this.renderer.h\
ooks.codeScrollI\
nit(box);\x0a      \
}\x0a\x0a      // Remo\
ve message by id\
 and keep scroll\
 consistent.\x0a   \
   removeNode(id\
, scrollMgr) {\x0a \
       scrollMgr\
.prevScroll = 0;\
\x0a        let el \
= document.getEl\
ementById('msg-u\
ser-' + id); if \
(el) el.remove()\
;\x0a        el = d\
ocument.getEleme\
ntById('msg-bot-\
' + id); if (el)\
 el.remove();\x0a  \
      this.dom.r\
esetEphemeral();\
\x0a        try { t\
his.renderer.ren\
derPendingMarkdo\
wn(); } catch (_\
) {}\x0a        scr\
ollMgr.scheduleS\
croll(true);\x0a   \
   }\x0a\x0a      // R\
emove all messag\
es from (and inc\
luding) a given \
message id.\x0a    \
  removeNodesFro\
mId(id, scrollMg\
r) {\x0a        scr\
ollMgr.prevScrol\
l = 0;\x0a        c\
onst container =\
 this.dom.get('_\
nodes_'); if (!c\
ontainer) return\
;\x0a        const \
elements = conta\
iner.querySelect\
orAll('.msg-box'\
);\x0a        let r\
emove = false;\x0a \
       elements.\
forEach((element\
) => {\x0a         \
 if (element.id \
&& element.id.en\
dsWith('-' + id)\
) remove = true;\
\x0a          if (r\
emove) element.r\
emove();\x0a       \
 });\x0a        thi\
s.dom.resetEphem\
eral();\x0a        \
try { this.rende\
rer.renderPendin\
gMarkdown(contai\
ner); } catch (_\
) {}\x0a        scr\
ollMgr.scheduleS\
croll(true);\x0a   \
   }\x0a    }\x0a\x0a   /\
/ ==============\
================\
================\
================\
============\x0a  /\
/ 9a) Template e\
ngine for JSON n\
odes\x0a  // ======\
================\
================\
================\
================\
====\x0a\x0a  class No\
deTemplateEngine\
 {\x0a    // JS-sid\
e templates for \
nodes rendered f\
rom JSON payload\
 (RenderBlock).\x0a\
    constructor(\
cfg, logger) {\x0a \
     this.cfg = \
cfg || {};\x0a     \
 this.logger = l\
ogger || { debug\
: () => {} };\x0a  \
  }\x0a\x0a    _esc(s)\
 { return (s == \
null) ? '' : Str\
ing(s); }\x0a    _e\
scapeHtml(s) { r\
eturn (typeof Ut\
ils !== 'undefin\
ed') ? Utils.esc\
apeHtml(s) : Str\
ing(s).replace(/\
[&<>\x22']/g, m => \
({'&':'&amp;','<\
':'&lt;','>':'&g\
t;','\x22':'&quot;'\
,\x22'\x22:'&#039;'}[m\
])); }\x0a\x0a    // R\
ender name heade\
r given role\x0a   \
 _nameHeader(rol\
e, name, avatarU\
rl) {\x0a      if (\
!name && !avatar\
Url) return '';\x0a\
      const cls \
= (role === 'use\
r') ? 'name-user\
' : 'name-bot';\x0a\
      const img \
= avatarUrl ? `<\
img src=\x22${this.\
_esc(avatarUrl)}\
\x22 class=\x22avatar\x22\
> ` : '';\x0a      \
return `<div cla\
ss=\x22name-header \
${cls}\x22>${img}${\
this._esc(name |\
| '')}</div>`;\x0a \
   }\x0a\x0a    // Ren\
der user message\
 block\x0a    _rend\
erUser(block) {\x0a\
      const id =\
 block.id;\x0a     \
 const inp = blo\
ck.input || {};\x0a\
      const msgI\
d = `msg-user-${\
id}`;\x0a\x0a      // \
NOTE: timestamps\
 intentionally d\
isabled on front\
end\x0a      // let\
 ts = '';\x0a      \
// if (inp.times\
tamp) { ... }\x0a\x0a \
     const perso\
nalize = !!(bloc\
k && block.extra\
 && block.extra.\
personalize === \
true);\x0a      con\
st nameHeader = \
personalize ? th\
is._nameHeader('\
user', inp.name \
|| '', inp.avata\
r_img || null) :\
 '';\x0a\x0a      cons\
t content = this\
._escapeHtml(inp\
.text || '').rep\
lace(/\x5cr?\x5cn/g, '\
<br>');\x0a      re\
turn `<div class\
=\x22msg-box msg-us\
er\x22 id=\x22${msgId}\
\x22>${nameHeader}<\
div class=\x22msg\x22>\
<p style=\x22margin\
:0\x22>${content}</\
p></div></div>`;\
\x0a    }\x0a\x0a    // R\
ender extra bloc\
ks (images/files\
/urls/docs/tool-\
extra)\x0a    _rend\
erExtras(block) \
{\x0a      const pa\
rts = [];\x0a\x0a     \
 // images\x0a     \
 const images = \
block.images || \
{};\x0a      const \
keysI = Object.k\
eys(images);\x0a   \
   if (keysI.len\
gth) {\x0a        k\
eysI.forEach((k)\
 => {\x0a          \
const it = image\
s[k];\x0a          \
if (!it) return;\
\x0a          const\
 url = this._esc\
(it.url); const \
path = this._esc\
(it.path); const\
 bn = this._esc(\
it.basename || '\
');\x0a          if\
 (it.is_video) {\
\x0a            con\
st src = (it.ext\
 === '.webm' || \
!it.webm_path) ?\
 path : this._es\
c(it.webm_path);\
\x0a            con\
st ext = (src.en\
dsWith('.webm') \
? 'webm' : (path\
.split('.').pop(\
) || 'mp4'));\x0a  \
          parts.\
push(\x0a          \
    `<div class=\
\x22extra-src-video\
-box\x22 title=\x22${u\
rl}\x22>` +\x0a       \
         `<video\
 class=\x22video-pl\
ayer\x22 controls>`\
 +\x0a             \
     `<source sr\
c=\x22${src}\x22 type=\
\x22video/${ext}\x22>`\
 +\x0a             \
   `</video>` +\x0a\
                \
`<p><a href=\x22bri\
dge://play_video\
/${url}\x22 class=\x22\
title\x22>${this._e\
scapeHtml(bn)}</\
a></p>` +\x0a      \
        `</div>`\
\x0a            );\x0a\
          } else\
 {\x0a            p\
arts.push(\x0a     \
         `<div c\
lass=\x22extra-src-\
img-box\x22 title=\x22\
${url}\x22>` +\x0a    \
            `<di\
v class=\x22img-out\
er\x22><div class=\x22\
img-wrapper\x22><a \
href=\x22${url}\x22><i\
mg src=\x22${path}\x22\
 class=\x22image\x22><\
/a></div>` +\x0a   \
             `<a\
 href=\x22${url}\x22 c\
lass=\x22title\x22>${t\
his._escapeHtml(\
bn)}</a></div>` \
+\x0a              \
`</div><br/>`\x0a  \
          );\x0a   \
       }\x0a       \
 });\x0a      }\x0a\x0a  \
    // files\x0a   \
   const files =\
 block.files || \
{};\x0a      const \
kF = Object.keys\
(files);\x0a      i\
f (kF.length) {\x0a\
        const ro\
ws = [];\x0a       \
 kF.forEach((k) \
=> {\x0a          c\
onst it = files[\
k]; if (!it) ret\
urn;\x0a          c\
onst url = this.\
_esc(it.url); co\
nst path = this.\
_esc(it.path);\x0a \
         const i\
con = (typeof wi\
ndow !== 'undefi\
ned' && window.I\
CON_ATTACHMENTS)\
 ? `<img src=\x22${\
window.ICON_ATTA\
CHMENTS}\x22 class=\
\x22extra-src-icon\x22\
>` : '';\x0a       \
   rows.push(`${\
icon} <b> [${k}]\
 </b> <a href=\x22$\
{url}\x22>${path}</\
a>`);\x0a        })\
;\x0a        if (ro\
ws.length) parts\
.push(`<div>${ro\
ws.join(\x22<br/><b\
r/>\x22)}</div>`);\x0a\
      }\x0a\x0a      /\
/ urls\x0a      con\
st urls = block.\
urls || {};\x0a    \
  const kU = Obj\
ect.keys(urls);\x0a\
      if (kU.len\
gth) {\x0a        c\
onst rows = [];\x0a\
        kU.forEa\
ch((k) => {\x0a    \
      const it =\
 urls[k]; if (!i\
t) return;\x0a     \
     const url =\
 this._esc(it.ur\
l);\x0a          co\
nst icon = (type\
of window !== 'u\
ndefined' && win\
dow.ICON_URL) ? \
`<img src=\x22${win\
dow.ICON_URL}\x22 c\
lass=\x22extra-src-\
icon\x22>` : '';\x0a  \
        rows.pus\
h(`${icon}<a hre\
f=\x22${url}\x22 title\
=\x22${url}\x22>${url}\
</a> <small> [${\
k}] </small>`);\x0a\
        });\x0a    \
    if (rows.len\
gth) parts.push(\
`<div>${rows.joi\
n(\x22<br/><br/>\x22)}\
</div>`);\x0a      \
}\x0a\x0a      // docs\
 (render on JS) \
or fallback to d\
ocs_html\x0a      c\
onst extra = blo\
ck.extra || {};\x0a\
      const docs\
Raw = Array.isAr\
ray(extra.docs) \
? extra.docs : n\
ull;\x0a\x0a      if (\
docsRaw && docsR\
aw.length) {\x0a   \
     const icon \
= (typeof window\
 !== 'undefined'\
 && window.ICON_\
DB) ? `<img src=\
\x22${window.ICON_D\
B}\x22 class=\x22extra\
-src-icon\x22>` : '\
';\x0a        const\
 prefix = (typeo\
f window !== 'un\
defined' && wind\
ow.LOCALE_DOC_PR\
EFIX) ? String(w\
indow.LOCALE_DOC\
_PREFIX) : 'Doc:\
';\x0a        const\
 limit = 3;\x0a\x0a   \
     // normaliz\
e: [{uuid, meta}\
] OR [{ uuid: {.\
..} }]\x0a        c\
onst normalized \
= [];\x0a        do\
csRaw.forEach((i\
t) => {\x0a        \
  if (!it || typ\
eof it !== 'obje\
ct') return;\x0a   \
       if ('uuid\
' in it && 'meta\
' in it && typeo\
f it.meta === 'o\
bject') {\x0a      \
      normalized\
.push({ uuid: St\
ring(it.uuid), m\
eta: it.meta || \
{} });\x0a         \
 } else {\x0a      \
      const keys\
 = Object.keys(i\
t);\x0a            \
if (keys.length \
=== 1) {\x0a       \
       const uui\
d = keys[0];\x0a   \
           const\
 meta = it[uuid]\
;\x0a              \
if (meta && type\
of meta === 'obj\
ect') {\x0a        \
        normaliz\
ed.push({ uuid: \
String(uuid), me\
ta });\x0a         \
     }\x0a         \
   }\x0a          }\
\x0a        });\x0a\x0a  \
      const rows\
 = [];\x0a        f\
or (let i = 0; i\
 < Math.min(limi\
t, normalized.le\
ngth); i++) {\x0a  \
        const d \
= normalized[i];\
\x0a          const\
 meta = d.meta |\
| {};\x0a          \
const entries = \
Object.keys(meta\
).map(k => `<b>$\
{this._escapeHtm\
l(k)}:</b> ${thi\
s._escapeHtml(St\
ring(meta[k]))}`\
).join(', ');\x0a  \
        rows.pus\
h(`<p><small>[${\
i + 1}] ${this._\
escapeHtml(d.uui\
d)}: ${entries}<\
/small></p>`);\x0a \
       }\x0a       \
 if (rows.length\
) {\x0a          pa\
rts.push(`<p>${i\
con}<small><b>${\
this._escapeHtml\
(prefix)}:</b></\
small></p>`);\x0a  \
        parts.pu\
sh(`<div class=\x22\
cmd\x22><p>${rows.j\
oin('')}</p></di\
v>`);\x0a        }\x0a\
      } else {\x0a \
       // backwa\
rd compat\x0a      \
  const docs_htm\
l = extra && ext\
ra.docs_html ? S\
tring(extra.docs\
_html) : '';\x0a   \
     if (docs_ht\
ml) parts.push(d\
ocs_html);\x0a     \
 }\x0a\x0a      // plu\
gin-driven tool \
extra HTML\x0a     \
 const tool_extr\
a_html = extra &\
& extra.tool_ext\
ra_html ? String\
(extra.tool_extr\
a_html) : '';\x0a  \
    if (tool_ext\
ra_html) parts.p\
ush(`<div class=\
\x22msg-extra\x22>${to\
ol_extra_html}</\
div>`);\x0a\x0a      r\
eturn parts.join\
('');\x0a    }\x0a\x0a   \
 // Render messa\
ge-level actions\
\x0a    _renderActi\
ons(block) {\x0a   \
   const extra =\
 block.extra || \
{};\x0a      const \
actions = extra.\
actions || [];\x0a \
     if (!action\
s || !actions.le\
ngth) return '';\
\x0a      const par\
ts = actions.map\
((a) => {\x0a      \
  const href = t\
his._esc(a.href \
|| '#');\x0a       \
 const title = t\
his._esc(a.title\
 || '');\x0a       \
 const icon = th\
is._esc(a.icon |\
| '');\x0a        c\
onst id = this._\
esc(a.id || bloc\
k.id);\x0a        r\
eturn `<a href=\x22\
${href}\x22 class=\x22\
action-icon\x22 dat\
a-id=\x22${id}\x22 rol\
e=\x22button\x22><span\
 class=\x22cmd\x22><im\
g src=\x22${icon}\x22 \
class=\x22action-im\
g\x22 title=\x22${titl\
e}\x22 alt=\x22${title\
}\x22 data-id=\x22${id\
}\x22></span></a>`;\
\x0a      });\x0a     \
 return `<div cl\
ass=\x22action-icon\
s\x22 data-id=\x22${th\
is._esc(block.id\
)}\x22>${parts.join\
('')}</div>`;\x0a  \
  }\x0a\x0a    // Rend\
er tool output w\
rapper (always c\
ollapsed by defa\
ult; wrapper vis\
ibility depends \
on flag)\x0a    // \
Inside class Nod\
eTemplateEngine\x0a\
    _renderToolO\
utputWrapper(blo\
ck) {\x0a      cons\
t extra = block.\
extra || {};\x0a\x0a  \
    // IMPORTANT\
: keep initial t\
ool output verba\
tim (HTML-ready)\
.\x0a      // Do NO\
T HTML-escape he\
re \xe2\x80\x93 the host \
already provides\
 a safe/HTML-rea\
dy string.\x0a     \
 // Escaping aga\
in would double-\
encode entities \
(e.g. \x22 -> \x22), w\
hich\x0a      // ca\
used visible \x22\x22\x22\
 in the UI inste\
ad of quotes.\x0a  \
    const tool_o\
utput_html = (ex\
tra.tool_output \
!= null) ? Strin\
g(extra.tool_out\
put) : '';\x0a\x0a    \
  // Wrapper vis\
ibility: show/hi\
de based on tool\
_output_visible.\
..\x0a      const w\
rapperDisplay = \
(extra.tool_outp\
ut_visible === t\
rue) ? '' : 'dis\
play:none';\x0a\x0a   \
   const toggleT\
itle = (typeof t\
rans !== 'undefi\
ned' && trans) ?\
 trans('action.c\
md.expand') : 'E\
xpand';\x0a      co\
nst expIcon = (t\
ypeof window !==\
 'undefined' && \
window.ICON_EXPA\
ND) ? window.ICO\
N_EXPAND : '';\x0a\x0a\
      return (\x0a \
       `<div cla\
ss='tool-output'\
 style='${wrappe\
rDisplay}'>` +\x0a \
         `<span \
class='toggle-cm\
d-output' onclic\
k='toggleToolOut\
put(${this._esc(\
block.id)});' ` \
+\x0a          `tit\
le='${this._esca\
peHtml(toggleTit\
le)}' role='butt\
on'>` +\x0a        \
    `<img src='$\
{this._esc(expIc\
on)}' width='25'\
 height='25' val\
ign='middle'>` +\
\x0a          `</sp\
an>` +\x0a         \
 // Content is i\
nitially collaps\
ed. We intention\
ally do NOT esca\
pe here,\x0a       \
   // to keep be\
havior consisten\
t with ToolOutpu\
t.append/update \
(HTML-in).\x0a     \
     `<div class\
='content' style\
='display:none' \
data-trusted='1'\
>${tool_output_h\
tml}</div>` +\x0a  \
      `</div>`\x0a \
     );\x0a    }\x0a\x0a \
   // Render bot\
 message block (\
md-block-markdow\
n)\x0a   _renderBot\
(block) {\x0a      \
const id = block\
.id;\x0a      const\
 out = block.out\
put || {};\x0a     \
 const msgId = `\
msg-bot-${id}`;\x0a\
\x0a      // NOTE: \
timestamps inten\
tionally disable\
d on frontend\x0a  \
    // let ts = \
'';\x0a      // if \
(out.timestamp) \
{ ... }\x0a\x0a      c\
onst personalize\
 = !!(block && b\
lock.extra && bl\
ock.extra.person\
alize === true);\
\x0a      const nam\
eHeader = person\
alize ? this._na\
meHeader('bot', \
out.name || '', \
out.avatar_img |\
| null) : '';\x0a\x0a \
     const mdTex\
t = this._escape\
Html(out.text ||\
 '');\x0a      cons\
t toolWrap = thi\
s._renderToolOut\
putWrapper(block\
);\x0a      const e\
xtras = this._re\
nderExtras(block\
);\x0a      const a\
ctions = (block.\
extra && block.e\
xtra.footer_icon\
s) ? this._rende\
rActions(block) \
: '';\x0a      cons\
t debug = (block\
.extra && block.\
extra.debug_html\
) ? String(block\
.extra.debug_htm\
l) : '';\x0a\x0a      \
return (\x0a       \
 `<div class='ms\
g-box msg-bot' i\
d='${msgId}'>` +\
\x0a          `${na\
meHeader}` +\x0a   \
       `<div cla\
ss='msg'>` +\x0a   \
         `<div c\
lass='md-block' \
md-block-markdow\
n='1'>${mdText}<\
/div>` +\x0a       \
     `<div class\
='msg-tool-extra\
'></div>` +\x0a    \
        `${toolW\
rap}` +\x0a        \
    `<div class=\
'msg-extra'>${ex\
tras}</div>` +\x0a \
           `${ac\
tions}${debug}` \
+\x0a          `</d\
iv>` +\x0a        `\
</div>`\x0a      );\
\x0a    }\x0a\x0a    // R\
ender one Render\
Block into HTML \
(may produce 1 o\
r 2 messages \xe2\x80\x93\
 input and/or ou\
tput)\x0a    render\
Node(block) {\x0a  \
    const parts \
= [];\x0a      if (\
block && block.i\
nput && block.in\
put.text) parts.\
push(this._rende\
rUser(block));\x0a \
     if (block &\
& block.output &\
& block.output.t\
ext) parts.push(\
this._renderBot(\
block));\x0a      r\
eturn parts.join\
('');\x0a    }\x0a\x0a   \
 // Render array\
 of blocks\x0a    r\
enderNodes(block\
s) {\x0a      if (!\
Array.isArray(bl\
ocks)) return ''\
;\x0a      const ou\
t = [];\x0a      fo\
r (let i = 0; i \
< blocks.length;\
 i++) {\x0a        \
const b = blocks\
[i] || null;\x0a   \
     if (!b) con\
tinue;\x0a        o\
ut.push(this.ren\
derNode(b));\x0a   \
   }\x0a      retur\
n out.join('');\x0a\
    }\x0a  }\x0a\x0a  // \
================\
================\
================\
================\
==========\x0a  // \
9b) Data receive\
r for append/rep\
lace nodes\x0a  // \
================\
================\
================\
================\
==========\x0a\x0a  cl\
ass DataReceiver\
 {\x0a    // Normal\
izes payload (HT\
ML string or JSO\
N) and delegates\
 to NodesManager\
.\x0a    constructo\
r(cfg, templates\
, nodes, scrollM\
gr) {\x0a      this\
.cfg = cfg || {}\
;\x0a      this.tem\
plates = templat\
es;\x0a      this.n\
odes = nodes;\x0a  \
    this.scrollM\
gr = scrollMgr;\x0a\
    }\x0a\x0a    _tryP\
arseJSON(s) {\x0a  \
    if (typeof s\
 !== 'string') r\
eturn s;\x0a      c\
onst t = s.trim(\
);\x0a      if (!t)\
 return null;\x0a  \
    // If it's l\
ike HTML, don't \
parse as JSON\x0a  \
    if (t[0] ===\
 '<') return nul\
l;\x0a      try { r\
eturn JSON.parse\
(t); } catch (_)\
 { return null; \
}\x0a    }\x0a\x0a    _no\
rmalizeToBlocks(\
obj) {\x0a      if \
(!obj) return []\
;\x0a      if (Arra\
y.isArray(obj)) \
return obj;\x0a    \
  if (obj.node) \
return [obj.node\
];\x0a      if (obj\
.nodes) return (\
Array.isArray(ob\
j.nodes) ? obj.n\
odes : []);\x0a    \
  // single node\
-like object\x0a   \
   if (typeof ob\
j === 'object' &\
& (obj.input || \
obj.output || ob\
j.id)) return [o\
bj];\x0a      retur\
n [];\x0a    }\x0a\x0a   \
 append(payload)\
 {\x0a      // Lega\
cy HTML string?\x0a\
      if (typeof\
 payload === 'st\
ring' && payload\
.trim().startsWi\
th('<')) {\x0a     \
   this.nodes.ap\
pendNode(payload\
, this.scrollMgr\
);\x0a        retur\
n;\x0a      }\x0a     \
 // Try JSON\x0a   \
   const obj = t\
his._tryParseJSO\
N(payload);\x0a    \
  if (!obj) {\x0a  \
      // Not JSO\
N \xe2\x80\x93 pass throu\
gh\x0a        this.\
nodes.appendNode\
(String(payload)\
, this.scrollMgr\
);\x0a        retur\
n;\x0a      }\x0a     \
 const blocks = \
this._normalizeT\
oBlocks(obj);\x0a  \
    if (!blocks.\
length) {\x0a      \
  this.nodes.app\
endNode('', this\
.scrollMgr);\x0a   \
     return;\x0a   \
   }\x0a      const\
 html = this.tem\
plates.renderNod\
es(blocks);\x0a    \
  this.nodes.app\
endNode(html, th\
is.scrollMgr);\x0a \
   }\x0a\x0a    replac\
e(payload) {\x0a   \
   // Legacy HTM\
L string?\x0a      \
if (typeof paylo\
ad === 'string' \
&& payload.trim(\
).startsWith('<'\
)) {\x0a        thi\
s.nodes.replaceN\
odes(payload, th\
is.scrollMgr);\x0a \
       return;\x0a \
     }\x0a      // \
Try JSON\x0a      c\
onst obj = this.\
_tryParseJSON(pa\
yload);\x0a      if\
 (!obj) {\x0a      \
  this.nodes.rep\
laceNodes(String\
(payload), this.\
scrollMgr);\x0a    \
    return;\x0a    \
  }\x0a      const \
blocks = this._n\
ormalizeToBlocks\
(obj);\x0a      if \
(!blocks.length)\
 {\x0a        this.\
nodes.replaceNod\
es('', this.scro\
llMgr);\x0a        \
return;\x0a      }\x0a\
      const html\
 = this.template\
s.renderNodes(bl\
ocks);\x0a      thi\
s.nodes.replaceN\
odes(html, this.\
scrollMgr);\x0a    \
}\x0a  }\x0a\x0a  // ====\
================\
================\
================\
================\
======\x0a  // 10) \
UI manager\x0a  // \
================\
================\
================\
================\
==========\x0a\x0a  cl\
ass UIManager {\x0a\
      // Replace\
 or insert app-l\
evel CSS in a <s\
tyle> tag.\x0a     \
 updateCSS(style\
s) {\x0a        let\
 style = documen\
t.getElementById\
('app-style');\x0a \
       if (!styl\
e) { style = doc\
ument.createElem\
ent('style'); st\
yle.id = 'app-st\
yle'; document.h\
ead.appendChild(\
style); }\x0a      \
  style.textCont\
ent = styles;\x0a  \
    }\x0a      // E\
nsure base style\
s for code heade\
r sticky behavio\
r exist.\x0a      e\
nsureStickyHeade\
rStyle() {\x0a     \
   let style = d\
ocument.getEleme\
ntById('code-sti\
cky-style');\x0a   \
     if (style) \
return;\x0a        \
style = document\
.createElement('\
style'); style.i\
d = 'code-sticky\
-style';\x0a       \
 style.textConte\
nt = [\x0a         \
 '.code-wrapper \
{ position: rela\
tive; }',\x0a      \
    '.code-wrapp\
er .code-header-\
wrapper { positi\
on: sticky; top:\
 var(--code-head\
er-sticky-top, 0\
px); z-index: 2;\
 box-shadow: 0 1\
px 0 rgba(0,0,0,\
.06); }',\x0a      \
    '.code-wrapp\
er pre { overflo\
w: visible; marg\
in-top: 0; }',\x0a \
         '.code-\
wrapper pre code\
 { display: bloc\
k; white-space: \
pre; max-height:\
 100dvh; overflo\
w: auto;',\x0a     \
     '  overscro\
ll-behavior: con\
tain; -webkit-ov\
erflow-scrolling\
: touch; overflo\
w-anchor: none; \
scrollbar-gutter\
: stable both-ed\
ges; scroll-beha\
vior: auto; }',\x0a\
          '#_loa\
der_.hidden { di\
splay: none !imp\
ortant; visibili\
ty: hidden !impo\
rtant; }',\x0a     \
     '#_loader_.\
visible { displa\
y: block; visibi\
lity: visible; }\
',\x0a\x0a          /*\
 User message co\
llapse (uc-*) */\
\x0a          '.msg\
-box.msg-user .m\
sg { position: r\
elative; }',\x0a   \
       '.msg-box\
.msg-user .msg >\
 .uc-content { d\
isplay: block; o\
verflow: visible\
; }',\x0a          \
'.msg-box.msg-us\
er .msg > .uc-co\
ntent.uc-collaps\
ed { max-height:\
 1000px; overflo\
w: hidden; }',\x0a \
         '.msg-b\
ox.msg-user .msg\
 > .uc-toggle { \
display: none; m\
argin-top: 8px; \
text-align: cent\
er; cursor: poin\
ter; user-select\
: none; }',\x0a    \
      '.msg-box.\
msg-user .msg > \
.uc-toggle.visib\
le { display: bl\
ock; }',\x0a\x0a      \
    /* Increased\
 toggle icon siz\
e to a comfortab\
le/default size.\
\x0a             Ov\
erridable via CS\
S var --uc-toggl\
e-icon-size to k\
eep host-level c\
ontrol. */\x0a     \
     '.msg-box.m\
sg-user .msg > .\
uc-toggle img { \
width: var(--uc-\
toggle-icon-size\
, 26px); height:\
 var(--uc-toggle\
-icon-size, 26px\
); opacity: .8; \
}',\x0a          '.\
msg-box.msg-user\
 .msg > .uc-togg\
le:hover img { o\
pacity: 1; }'\x0a  \
      ].join('\x5cn\
');\x0a        docu\
ment.head.append\
Child(style);\x0a  \
    }\x0a      // T\
oggle classes co\
ntrolling option\
al UI features.\x0a\
      enableEdit\
Icons() { docume\
nt.body && docum\
ent.body.classLi\
st.add('display-\
edit-icons'); }\x0a\
      disableEdi\
tIcons() { docum\
ent.body && docu\
ment.body.classL\
ist.remove('disp\
lay-edit-icons')\
; }\x0a      enable\
Timestamp() { do\
cument.body && d\
ocument.body.cla\
ssList.add('disp\
lay-timestamp');\
 }\x0a      disable\
Timestamp() { do\
cument.body && d\
ocument.body.cla\
ssList.remove('d\
isplay-timestamp\
'); }\x0a      enab\
leBlocks() { doc\
ument.body && do\
cument.body.clas\
sList.add('displ\
ay-blocks'); }\x0a \
     disableBloc\
ks() { document.\
body && document\
.body.classList.\
remove('display-\
blocks'); }\x0a    \
}\x0a\x0a  // ========\
================\
================\
================\
================\
==\x0a  // 11) Stre\
am snapshot engi\
ne + incremental\
 code streaming\x0a\
  // ===========\
================\
================\
================\
===============\x0a\
\x0a  class StreamE\
ngine {\x0a  constr\
uctor(cfg, dom, \
renderer, math, \
highlighter, cod\
eScroll, scrollM\
gr, raf, asyncer\
, logger) {\x0a    \
this.cfg = cfg; \
this.dom = dom; \
this.renderer = \
renderer; this.m\
ath = math;\x0a    \
this.highlighter\
 = highlighter; \
this.codeScroll \
= codeScroll; th\
is.scrollMgr = s\
crollMgr; this.r\
af = raf;\x0a    th\
is.asyncer = asy\
ncer;\x0a    this.l\
ogger = logger |\
| new Logger(cfg\
);\x0a\x0a    // Strea\
ming buffer (rop\
e-like) \xe2\x80\x93 avoi\
ds O(n^2) string\
 concatenation w\
hen many small c\
hunks arrive.\x0a  \
  // streamBuf h\
olds the already\
 materialized pr\
efix; _sbParts k\
eeps recent tail\
 parts; _sbLen t\
racks their leng\
th.\x0a    this.str\
eamBuf = '';    \
 // materialized\
 prefix (string \
used by render)\x0a\
    this._sbPart\
s = [];      // \
pending string c\
hunks (array) no\
t yet joined\x0a   \
 this._sbLen = 0\
;         // len\
gth of pending c\
hunks\x0a\x0a    this.\
fenceOpen = fals\
e; this.fenceMar\
k = '`'; this.fe\
nceLen = 3;\x0a    \
this.fenceTail =\
 ''; this.fenceB\
uf = '';\x0a    thi\
s.lastSnapshotTs\
 = 0; this.nextS\
napshotStep = cf\
g.PROFILE_TEXT.b\
ase;\x0a    this.sn\
apshotScheduled \
= false; this.sn\
apshotRAF = 0;\x0a\x0a\
    this.codeStr\
eam = { open: fa\
lse, lines: 0, c\
hars: 0 };\x0a    t\
his.activeCode =\
 null;\x0a\x0a    this\
.suppressPostFin\
alizePass = fals\
e;\x0a\x0a    this._pr\
omoteScheduled =\
 false;\x0a\x0a    // \
Guard to ensure \
first fence-open\
 is materialized\
 immediately whe\
n stream starts \
with code.\x0a    t\
his._firstCodeOp\
enSnapDone = fal\
se;\x0a\x0a    // Stre\
aming mode flag \
\xe2\x80\x93 controls red\
uced rendering (\
no linkify etc.)\
 on hot path.\x0a  \
  this.isStreami\
ng = false;\x0a\x0a   \
 // Tracks wheth\
er renderSnapsho\
t injected a one\
-off synthetic E\
OL for parsing a\
n open fence\x0a   \
 // (used to str\
ip it from the i\
nitial streaming\
 tail to avoid \x22\
#\x5cn foo\x22 on firs\
t line).\x0a    thi\
s._lastInjectedE\
OL = false;\x0a\x0a   \
 this._customFen\
ceSpecs = [];   \
// [{ open, clos\
e }, ...]\x0a    th\
is._fenceCustom \
= null;      // \
currently active\
 custom fence sp\
ec or null\x0a  }\x0a \
 _d(tag, data) {\
 this.logger.deb\
ug('STREAM', tag\
, data); }\x0a\x0a  se\
tCustomFenceSpec\
s(specs) {\x0a    t\
his._customFence\
Specs = Array.is\
Array(specs) ? s\
pecs.slice() : [\
];\x0a  }\x0a\x0a  // ---\
 Rope buffer hel\
pers (internal) \
---\x0a  _appendChu\
nk(s) {\x0a    if (\
!s) return;\x0a    \
this._sbParts.pu\
sh(s);\x0a    this.\
_sbLen += s.leng\
th;\x0a  }\x0a  getStr\
eamLength() {\x0a  \
  return (this.s\
treamBuf.length \
+ this._sbLen);\x0a\
  }\x0a  getStreamT\
ext() {\x0a    if (\
this._sbLen > 0)\
 {\x0a      // Join\
 pending parts i\
nto the material\
ized prefix and \
clear the tail.\x0a\
      // Single-\
part fast path a\
voids a temporar\
y array join.\x0a  \
    this.streamB\
uf += (this._sbP\
arts.length === \
1 ? this._sbPart\
s[0] : this._sbP\
arts.join(''));\x0a\
      this._sbPa\
rts.length = 0;\x0a\
      this._sbLe\
n = 0;\x0a    }\x0a   \
 return this.str\
eamBuf;\x0a  }\x0a  _c\
learStreamBuffer\
() {\x0a    this.st\
reamBuf = '';\x0a  \
  this._sbParts.\
length = 0;\x0a    \
this._sbLen = 0;\
\x0a  }\x0a\x0a  // Reset\
 all streaming s\
tate and counter\
s.\x0a  reset() {\x0a \
   this._clearSt\
reamBuffer();\x0a  \
  this.fenceOpen\
 = false; this.f\
enceMark = '`'; \
this.fenceLen = \
3;\x0a    this.fenc\
eTail = ''; this\
.fenceBuf = '';\x0a\
    this.lastSna\
pshotTs = 0; thi\
s.nextSnapshotSt\
ep = this.profil\
e().base;\x0a    th\
is.snapshotSched\
uled = false; th\
is.snapshotRAF =\
 0;\x0a    this.cod\
eStream = { open\
: false, lines: \
0, chars: 0 };\x0a \
   this.activeCo\
de = null; this.\
suppressPostFina\
lizePass = false\
;\x0a    this._prom\
oteScheduled = f\
alse;\x0a    this._\
firstCodeOpenSna\
pDone = false;\x0a\x0a\
    // Clear any\
 previous synthe\
tic EOL marker.\x0a\
    this._lastIn\
jectedEOL = fals\
e;\x0a    this._fen\
ceCustom = null;\
\x0a\x0a    this._d('R\
ESET', { });\x0a  }\
\x0a  defuseActiveT\
oPlain() {\x0a    i\
f (!this.activeC\
ode || !this.act\
iveCode.codeEl |\
| !this.activeCo\
de.codeEl.isConn\
ected) return;\x0a \
   const codeEl \
= this.activeCod\
e.codeEl;\x0a    co\
nst fullText = (\
this.activeCode.\
frozenEl?.textCo\
ntent || '') + (\
this.activeCode.\
tailEl?.textCont\
ent || '');\x0a    \
try {\x0a      code\
El.textContent =\
 fullText;\x0a     \
 codeEl.removeAt\
tribute('data-hi\
ghlighted');\x0a   \
   codeEl.classL\
ist.remove('hljs\
');\x0a      codeEl\
.dataset._active\
_stream = '0';\x0a \
     const st = \
this.codeScroll.\
state(codeEl); s\
t.autoFollow = f\
alse;\x0a    } catc\
h (_) {}\x0a    thi\
s._d('DEFUSE_ACT\
IVE_TO_PLAIN', {\
 len: fullText.l\
ength });\x0a    th\
is.activeCode = \
null;\x0a  }\x0a  defu\
seOrphanActiveBl\
ocks(root) {\x0a   \
 try {\x0a      con\
st scope = root \
|| document;\x0a   \
   const nodes =\
 scope.querySele\
ctorAll('pre cod\
e[data-_active_s\
tream=\x221\x22]');\x0a  \
    let n = 0;\x0a \
     nodes.forEa\
ch(codeEl => {\x0a \
       if (!code\
El.isConnected) \
return;\x0a        \
let text = '';\x0a \
       const fro\
zen = codeEl.que\
rySelector('.hl-\
frozen');\x0a      \
  const tail = c\
odeEl.querySelec\
tor('.hl-tail');\
\x0a        if (fro\
zen || tail) tex\
t = (frozen?.tex\
tContent || '') \
+ (tail?.textCon\
tent || '');\x0a   \
     else text =\
 codeEl.textCont\
ent || '';\x0a     \
   codeEl.textCo\
ntent = text;\x0a  \
      codeEl.rem\
oveAttribute('da\
ta-highlighted')\
;\x0a        codeEl\
.classList.remov\
e('hljs');\x0a     \
   codeEl.datase\
t._active_stream\
 = '0';\x0a        \
try { this.codeS\
croll.attachHand\
lers(codeEl); } \
catch (_) {}\x0a   \
     n++;\x0a      \
});\x0a      if (n)\
 this._d('DEFUSE\
_ORPHAN_ACTIVE_B\
LOCKS', { count:\
 n });\x0a    } cat\
ch (e) { this._d\
('DEFUSE_ORPHAN_\
ACTIVE_ERR', Str\
ing(e)); }\x0a  }\x0a \
 abortAndReset(o\
pts) {\x0a    const\
 o = Object.assi\
gn({\x0a      final\
izeActive: true,\
\x0a      clearBuff\
er: true,\x0a      \
clearMsg: false,\
\x0a      defuseOrp\
hans: true,\x0a    \
  reason: '',\x0a  \
    suppressLog:\
 false\x0a    }, (o\
pts || {}));\x0a\x0a  \
  try { this.raf\
.cancelGroup('St\
reamEngine'); } \
catch (_) {}\x0a   \
 try { this.raf.\
cancel('SE:snaps\
hot'); } catch (\
_) {}\x0a    this.s\
napshotScheduled\
 = false; this.s\
napshotRAF = 0;\x0a\
\x0a    const hadAc\
tive = !!this.ac\
tiveCode;\x0a    tr\
y {\x0a      if (th\
is.activeCode) {\
\x0a        if (o.f\
inalizeActive ==\
= true) this.fin\
alizeActiveCode(\
);\x0a        else \
this.defuseActiv\
eToPlain();\x0a    \
  }\x0a    } catch \
(e) {\x0a      this\
._d('ABORT_FINAL\
IZE_ERR', String\
(e));\x0a    }\x0a\x0a   \
 if (o.defuseOrp\
hans) {\x0a      tr\
y { this.defuseO\
rphanActiveBlock\
s(); }\x0a      cat\
ch (e) { this._d\
('ABORT_DEFUSE_O\
RPHANS_ERR', Str\
ing(e)); }\x0a    }\
\x0a\x0a    if (o.clea\
rBuffer) {\x0a     \
 this._clearStre\
amBuffer();\x0a    \
  this.fenceOpen\
 = false; this.f\
enceMark = '`'; \
this.fenceLen = \
3;\x0a      this.fe\
nceTail = ''; th\
is.fenceBuf = ''\
;\x0a      this.cod\
eStream.open = f\
alse; this.codeS\
tream.lines = 0;\
 this.codeStream\
.chars = 0;\x0a    \
  window.__lastS\
napshotLen = 0;\x0a\
    }\x0a    if (o.\
clearMsg === tru\
e) {\x0a      try {\
 this.dom.resetE\
phemeral(); } ca\
tch (_) {}\x0a    }\
\x0a    if (!o.supp\
ressLog) this._d\
('ABORT_AND_RESE\
T', { hadActive,\
 ...o });\x0a  }\x0a  \
profile() { retu\
rn this.fenceOpe\
n ? this.cfg.PRO\
FILE_CODE : this\
.cfg.PROFILE_TEX\
T; }\x0a  resetBudg\
et() { this.next\
SnapshotStep = t\
his.profile().ba\
se; }\x0a  onlyTrai\
lingWhitespace(s\
, from, end) {\x0a \
   for (let i = \
from; i < end; i\
++) { const c = \
s.charCodeAt(i);\
 if (c !== 0x20 \
&& c !== 0x09) r\
eturn false; }\x0a \
   return true;\x0a\
  }\x0a  updateFenc\
eHeuristic(chunk\
) {\x0a    const pr\
ev = (this.fence\
Buf || '');\x0a    \
const s = prev +\
 (chunk || '');\x0a\
    const preLen\
 = prev.length;\x0a\
    const n = s.\
length; let i = \
0;\x0a    let opene\
d = false; let c\
losed = false; l\
et splitAt = -1;\
\x0a    let atLineS\
tart = (preLen =\
== 0) ? true : /\
[\x5cn\x5cr]$/.test(pr\
ev);\x0a\x0a    const \
inNewOrCrosses =\
 (j, k) => (j >=\
 preLen) || (k >\
 preLen);\x0a\x0a    w\
hile (i < n) {\x0a \
     const ch = \
s[i];\x0a      if (\
ch === '\x5cr' || c\
h === '\x5cn') { at\
LineStart = true\
; i++; continue;\
 }\x0a      if (!at\
LineStart) { i++\
; continue; }\x0a  \
    atLineStart \
= false;\x0a\x0a      \
// Skip list/blo\
ckquote/indent n\
ormalization (ex\
isting logic)\x0a  \
    let j = i;\x0a \
     while (j < \
n) {\x0a        let\
 localSpaces = 0\
;\x0a        while \
(j < n && (s[j] \
=== ' ' || s[j] \
=== '\x5ct')) { loc\
alSpaces += (s[j\
] === '\x5ct') ? 4 \
: 1; j++; if (lo\
calSpaces > 3) b\
reak; }\x0a        \
if (j < n && s[j\
] === '>') { j++\
; if (j < n && s\
[j] === ' ') j++\
; continue; }\x0a\x0a \
       let saved\
 = j;\x0a        if\
 (j < n && (s[j]\
 === '-' || s[j]\
 === '*' || s[j]\
 === '+')) {\x0a   \
       let jj = \
j + 1; if (jj < \
n && s[jj] === '\
 ') { j = jj + 1\
; } else { j = s\
aved; }\x0a        \
} else {\x0a       \
   let k2 = j; l\
et hasDigit = fa\
lse;\x0a          w\
hile (k2 < n && \
s[k2] >= '0' && \
s[k2] <= '9') { \
hasDigit = true;\
 k2++; }\x0a       \
   if (hasDigit \
&& k2 < n && (s[\
k2] === '.' || s\
[k2] === ')')) {\
\x0a            k2+\
+; if (k2 < n &&\
 s[k2] === ' ') \
{ j = k2 + 1; } \
else { j = saved\
; }\x0a          } \
else { j = saved\
; }\x0a        }\x0a  \
      break;\x0a   \
   }\x0a\x0a      let \
indent = 0;\x0a    \
  while (j < n &\
& (s[j] === ' ' \
|| s[j] === '\x5ct'\
)) {\x0a        ind\
ent += (s[j] ===\
 '\x5ct') ? 4 : 1; \
j++; if (indent \
> 3) break;\x0a    \
  }\x0a      if (in\
dent > 3) { i = \
j; continue; }\x0a\x0a\
      // 1) Cust\
om fences first \
(e.g. [!exec] ..\
. [/!exec], <exe\
cute>...</execut\
e>)\x0a      if (!t\
his.fenceOpen &&\
 this._customFen\
ceSpecs && this.\
_customFenceSpec\
s.length) {\x0a    \
    for (let ci \
= 0; ci < this._\
customFenceSpecs\
.length; ci++) {\
\x0a          const\
 spec = this._cu\
stomFenceSpecs[c\
i];\x0a          co\
nst open = spec \
&& spec.open ? s\
pec.open : '';\x0a \
         if (!op\
en) continue;\x0a  \
        const k \
= j + open.lengt\
h;\x0a          if \
(k <= n && s.sli\
ce(j, k) === ope\
n) {\x0a           \
 if (!inNewOrCro\
sses(j, k)) { /*\
 seen fully in p\
revious prefix *\
/ }\x0a            \
else {\x0a         \
     this.fenceO\
pen = true; this\
._fenceCustom = \
spec; opened = t\
rue; i = k;\x0a    \
          this._\
d('FENCE_OPEN_DE\
TECTED_CUSTOM', \
{ open, idxStart\
: j, idxEnd: k, \
region: (j >= pr\
eLen) ? 'new' : \
'cross' });\x0a    \
          contin\
ue; // main whil\
e\x0a            }\x0a\
          }\x0a    \
    }\x0a      } el\
se if (this.fenc\
eOpen && this._f\
enceCustom && th\
is._fenceCustom.\
close) {\x0a       \
 const close = t\
his._fenceCustom\
.close;\x0a        \
const k = j + cl\
ose.length;\x0a    \
    if (k <= n &\
& s.slice(j, k) \
=== close) {\x0a   \
       // Requir\
e only trailing \
whitespace on th\
e line (consiste\
nt with ``` logi\
c)\x0a          let\
 eol = k; while \
(eol < n && s[eo\
l] !== '\x5cn' && s\
[eol] !== '\x5cr') \
eol++;\x0a         \
 const onlyWS = \
this.onlyTrailin\
gWhitespace(s, k\
, eol);\x0a        \
  if (onlyWS) {\x0a\
            if (\
!inNewOrCrosses(\
j, k)) { /* seen\
 in previous pre\
fix */ }\x0a       \
     else {\x0a    \
          this.f\
enceOpen = false\
; this._fenceCus\
tom = null; clos\
ed = true;\x0a     \
         const e\
ndInS = k;\x0a     \
         const r\
el = endInS - pr\
eLen;\x0a          \
    splitAt = Ma\
th.max(0, Math.m\
in((chunk ? chun\
k.length : 0), r\
el));\x0a          \
    i = k;\x0a     \
         this._d\
('FENCE_CLOSE_DE\
TECTED_CUSTOM', \
{ close, idxStar\
t: j, idxEnd: k,\
 splitAt, region\
: (j >= preLen) \
? 'new' : 'cross\
' });\x0a          \
    continue; //\
 main while\x0a    \
        }\x0a      \
    } else {\x0a   \
         this._d\
('FENCE_CLOSE_RE\
JECTED_CUSTOM_NO\
N_WS_AFTER', { c\
lose, idxStart: \
j, idxEnd: k });\
\x0a          }\x0a   \
     }\x0a      }\x0a\x0a\
      // 2) Stan\
dard markdown-it\
 fences (``` or \
~~~) \xe2\x80\x93 leave y\
our original log\
ic intact\x0a      \
if (j < n && (s[\
j] === '`' || s[\
j] === '~')) {\x0a \
       const mar\
k = s[j]; let k \
= j; while (k < \
n && s[k] === ma\
rk) k++; const r\
un = k - j;\x0a\x0a   \
     if (!this.f\
enceOpen) {\x0a    \
      if (run >=\
 3) {\x0a          \
  if (!inNewOrCr\
osses(j, k)) { i\
 = k; continue; \
}\x0a            th\
is.fenceOpen = t\
rue; this.fenceM\
ark = mark; this\
.fenceLen = run;\
 opened = true; \
i = k;\x0a         \
   this._d('FENC\
E_OPEN_DETECTED'\
, { mark, run, i\
dxStart: j, idxE\
nd: k, region: (\
j >= preLen) ? '\
new' : 'cross' }\
);\x0a            c\
ontinue;\x0a       \
   }\x0a        } e\
lse if (!this._f\
enceCustom) {\x0a  \
        if (mark\
 === this.fenceM\
ark && run >= th\
is.fenceLen) {\x0a \
           if (!\
inNewOrCrosses(j\
, k)) { i = k; c\
ontinue; }\x0a     \
       let eol =\
 k; while (eol <\
 n && s[eol] !==\
 '\x5cn' && s[eol] \
!== '\x5cr') eol++;\
\x0a            if \
(this.onlyTraili\
ngWhitespace(s, \
k, eol)) {\x0a     \
         this.fe\
nceOpen = false;\
 closed = true;\x0a\
              co\
nst endInS = k;\x0a\
              co\
nst rel = endInS\
 - preLen;\x0a     \
         splitAt\
 = Math.max(0, M\
ath.min((chunk ?\
 chunk.length : \
0), rel));\x0a     \
         i = k;\x0a\
              th\
is._d('FENCE_CLO\
SE_DETECTED', { \
mark, run, idxSt\
art: j, idxEnd: \
k, splitAt, regi\
on: (j >= preLen\
) ? 'new' : 'cro\
ss' });\x0a        \
      continue;\x0a\
            } el\
se {\x0a           \
   this._d('FENC\
E_CLOSE_REJECTED\
_NON_WS_AFTER', \
{ mark, run, idx\
Start: j, idxEnd\
: k });\x0a        \
    }\x0a          \
}\x0a        }\x0a    \
  }\x0a\x0a      i = j\
 + 1;\x0a    }\x0a\x0a   \
 const MAX_TAIL \
= 512;\x0a    this.\
fenceBuf = s.sli\
ce(-MAX_TAIL);\x0a \
   this.fenceTai\
l = s.slice(-3);\
\x0a    return { op\
ened, closed, sp\
litAt };\x0a  }\x0a\x0a  \
// Ensure messag\
e snapshot conta\
iner exists.\x0a  g\
etMsgSnapshotRoo\
t(msg) {\x0a    if \
(!msg) return nu\
ll;\x0a    let snap\
 = msg.querySele\
ctor('.md-snapsh\
ot-root');\x0a    i\
f (!snap) { snap\
 = document.crea\
teElement('div')\
; snap.className\
 = 'md-snapshot-\
root'; msg.appen\
dChild(snap); }\x0a\
    return snap;\
\x0a  }\x0a  hasStruct\
uralBoundary(chu\
nk) { if (!chunk\
) return false; \
return /\x5cn(\x5cn|[-\
*]\x5cs|\x5cd+\x5c.\x5cs|#{1\
,6}\x5cs|>\x5cs)/.test\
(chunk); }\x0a  sho\
uldSnapshotOnChu\
nk(chunk, chunkH\
asNL, hasBoundar\
y) {\x0a    const p\
rof = this.profi\
le(); const now \
= Utils.now();\x0a \
   if (this.acti\
veCode && this.f\
enceOpen) return\
 false;\x0a    if (\
(now - this.last\
SnapshotTs) < pr\
of.minInterval) \
return false;\x0a  \
  if (hasBoundar\
y) return true;\x0a\
\x0a    const delta\
 = Math.max(0, t\
his.getStreamLen\
gth() - (window.\
__lastSnapshotLe\
n || 0));\x0a    if\
 (this.fenceOpen\
) { if (chunkHas\
NL && delta >= t\
his.nextSnapshot\
Step) return tru\
e; return false;\
 }\x0a    if (delta\
 >= this.nextSna\
pshotStep) retur\
n true;\x0a    retu\
rn false;\x0a  }\x0a  \
maybeScheduleSof\
tSnapshot(msg, c\
hunkHasNL) {\x0a   \
 const prof = th\
is.profile(); co\
nst now = Utils.\
now();\x0a    if (t\
his.activeCode &\
& this.fenceOpen\
) return;\x0a    if\
 (this.fenceOpen\
 && this.codeStr\
eam.lines < 1 &&\
 !chunkHasNL) re\
turn;\x0a    if ((n\
ow - this.lastSn\
apshotTs) >= pro\
f.softLatency) t\
his.scheduleSnap\
shot(msg);\x0a  }\x0a \
 scheduleSnapsho\
t(msg, force = f\
alse) {\x0a    if (\
this.snapshotSch\
eduled && !this.\
raf.isScheduled(\
'SE:snapshot')) \
this.snapshotSch\
eduled = false;\x0a\
    if (!force) \
{\x0a      if (this\
.snapshotSchedul\
ed) return;\x0a    \
  if (this.activ\
eCode && this.fe\
nceOpen) return;\
\x0a    } else {\x0a  \
    if (this.sna\
pshotScheduled &\
& this.raf.isSch\
eduled('SE:snaps\
hot')) return;\x0a \
   }\x0a    this.sn\
apshotScheduled \
= true;\x0a    this\
.raf.schedule('S\
E:snapshot', () \
=> { this.snapsh\
otScheduled = fa\
lse; this.render\
Snapshot(msg); }\
, 'StreamEngine'\
, 0);\x0a  }\x0a\x0a  ens\
ureSplitCodeEl(c\
odeEl) {\x0a    if \
(!codeEl) return\
 null;\x0a    let f\
rozen = codeEl.q\
uerySelector('.h\
l-frozen'); let \
tail = codeEl.qu\
erySelector('.hl\
-tail');\x0a    if \
(frozen && tail)\
 return { codeEl\
, frozenEl: froz\
en, tailEl: tail\
 };\x0a    const te\
xt = codeEl.text\
Content || ''; c\
odeEl.innerHTML \
= '';\x0a    frozen\
 = document.crea\
teElement('span'\
); frozen.classN\
ame = 'hl-frozen\
';\x0a    tail = do\
cument.createEle\
ment('span'); ta\
il.className = '\
hl-tail';\x0a    co\
deEl.appendChild\
(frozen); codeEl\
.appendChild(tai\
l);\x0a    if (text\
) tail.textConte\
nt = text; retur\
n { codeEl, froz\
enEl: frozen, ta\
ilEl: tail };\x0a  \
}\x0a  setupActiveC\
odeFromSnapshot(\
snap) {\x0a    cons\
t codes = snap.q\
uerySelectorAll(\
'pre code'); if \
(!codes.length) \
return null;\x0a   \
 const last = co\
des[codes.length\
 - 1];\x0a    const\
 cls = Array.fro\
m(last.classList\
).find(c => c.st\
artsWith('langua\
ge-')) || 'langu\
age-plaintext';\x0a\
    const lang =\
 (cls.replace('l\
anguage-', '') |\
| 'plaintext');\x0a\
    const parts \
= this.ensureSpl\
itCodeEl(last); \
if (!parts) retu\
rn null;\x0a\x0a    //\
 If we injected \
a synthetic EOL \
for parsing an o\
pen fence, remov\
e it from the st\
reaming tail now\
.\x0a    if (this._\
lastInjectedEOL \
&& parts.tailEl \
&& parts.tailEl.\
textContent && p\
arts.tailEl.text\
Content.endsWith\
('\x5cn')) {\x0a      \
parts.tailEl.tex\
tContent = parts\
.tailEl.textCont\
ent.slice(0, -1)\
;\x0a      this._la\
stInjectedEOL = \
false;\x0a    }\x0a\x0a  \
  const st = thi\
s.codeScroll.sta\
te(parts.codeEl)\
; st.autoFollow \
= true; st.userI\
nteracted = fals\
e;\x0a    parts.cod\
eEl.dataset._act\
ive_stream = '1'\
;\x0a    const base\
FrozenNL = Utils\
.countNewlines(p\
arts.frozenEl.te\
xtContent || '')\
; const baseTail\
NL = Utils.count\
Newlines(parts.t\
ailEl.textConten\
t || '');\x0a    co\
nst ac = { codeE\
l: parts.codeEl,\
 frozenEl: parts\
.frozenEl, tailE\
l: parts.tailEl,\
 lang, frozenLen\
: parts.frozenEl\
.textContent.len\
gth, lastPromote\
Ts: 0,\x0a         \
        lines: 0\
, tailLines: bas\
eTailNL, linesSi\
ncePromote: 0, i\
nitialLines: bas\
eFrozenNL + base\
TailNL, haltHL: \
false, plainStre\
am: false };\x0a   \
 this._d('ACTIVE\
_CODE_SETUP', { \
lang, frozenLen:\
 ac.frozenLen, t\
ailLines: ac.tai\
lLines, initialL\
ines: ac.initial\
Lines });\x0a    re\
turn ac;\x0a  }\x0a  r\
ehydrateActiveCo\
de(oldAC, newAC)\
 {\x0a    if (!oldA\
C || !newAC) ret\
urn;\x0a    newAC.f\
rozenEl.innerHTM\
L = oldAC.frozen\
El ? oldAC.froze\
nEl.innerHTML : \
'';\x0a    const fu\
llText = newAC.c\
odeEl.textConten\
t || ''; const r\
emainder = fullT\
ext.slice(oldAC.\
frozenLen);\x0a    \
newAC.tailEl.tex\
tContent = remai\
nder;\x0a    newAC.\
frozenLen = oldA\
C.frozenLen; new\
AC.lang = oldAC.\
lang;\x0a    newAC.\
lines = oldAC.li\
nes; newAC.tailL\
ines = Utils.cou\
ntNewlines(remai\
nder);\x0a    newAC\
.lastPromoteTs =\
 oldAC.lastPromo\
teTs; newAC.line\
sSincePromote = \
oldAC.linesSince\
Promote || 0;\x0a  \
  newAC.initialL\
ines = oldAC.ini\
tialLines || 0; \
newAC.haltHL = !\
!oldAC.haltHL;\x0a \
   newAC.plainSt\
ream = !!oldAC.p\
lainStream;\x0a    \
this._d('ACTIVE_\
CODE_REHYDRATE',\
 { lang: newAC.l\
ang, frozenLen: \
newAC.frozenLen,\
 tailLines: newA\
C.tailLines, ini\
tialLines: newAC\
.initialLines, h\
alted: newAC.hal\
tHL, plainStream\
: newAC.plainStr\
eam });\x0a  }\x0a  ap\
pendToActiveTail\
(text) {\x0a    if \
(!this.activeCod\
e || !this.activ\
eCode.tailEl || \
!text) return;\x0a \
   this.activeCo\
de.tailEl.insert\
AdjacentText('be\
foreend', text);\
\x0a    const nl = \
Utils.countNewli\
nes(text);\x0a    t\
his.activeCode.t\
ailLines += nl; \
this.activeCode.\
linesSincePromot\
e += nl;\x0a    thi\
s.codeScroll.sch\
eduleScroll(this\
.activeCode.code\
El, true, false)\
;\x0a    if (this.l\
ogger.isEnabled(\
'STREAM') && (nl\
 > 0 || text.len\
gth >= 64)) {\x0a  \
    this._d('TAI\
L_APPEND', { add\
Len: text.length\
, addNL: nl, tot\
alTailNL: this.a\
ctiveCode.tailLi\
nes });\x0a    }\x0a  \
}\x0a  enforceHLSto\
pBudget() {\x0a    \
if (!this.active\
Code) return;\x0a  \
  if (this.cfg.H\
L.DISABLE_ALL) {\
 this.activeCode\
.haltHL = true; \
this.activeCode.\
plainStream = tr\
ue; return; }\x0a  \
  const stop = (\
this.cfg.PROFILE\
_CODE.stopAfterL\
ines | 0);\x0a    c\
onst streamPlain\
Lines = (this.cf\
g.PROFILE_CODE.s\
treamPlainAfterL\
ines | 0);\x0a    c\
onst streamPlain\
Chars = (this.cf\
g.PROFILE_CODE.s\
treamPlainAfterC\
hars | 0);\x0a    c\
onst maxFrozenCh\
ars = (this.cfg.\
PROFILE_CODE.max\
FrozenChars | 0)\
;\x0a\x0a    const tot\
alLines = (this.\
activeCode.initi\
alLines || 0) + \
(this.activeCode\
.lines || 0);\x0a  \
  const frozenCh\
ars = this.activ\
eCode.frozenLen \
| 0;\x0a    const t\
ailChars = (this\
.activeCode.tail\
El?.textContent \
|| '').length | \
0;\x0a    const tot\
alStreamedChars \
= frozenChars + \
tailChars;\x0a\x0a    \
if ((streamPlain\
Lines > 0 && tot\
alLines >= strea\
mPlainLines) ||\x0a\
        (streamP\
lainChars > 0 &&\
 totalStreamedCh\
ars >= streamPla\
inChars) ||\x0a    \
    (maxFrozenCh\
ars > 0 && froze\
nChars >= maxFro\
zenChars)) {\x0a   \
   this.activeCo\
de.haltHL = true\
;\x0a      this.act\
iveCode.plainStr\
eam = true;\x0a    \
  try { this.act\
iveCode.codeEl.d\
ataset.hlStreamS\
uspended = '1'; \
} catch (_) {}\x0a \
     this._d('ST\
REAM_HL_SUSPENDE\
D', { totalLines\
, totalStreamedC\
hars, frozenChar\
s, reason: 'budg\
et' });\x0a      re\
turn;\x0a    }\x0a\x0a   \
 if (stop > 0 &&\
 totalLines >= s\
top) {\x0a      thi\
s.activeCode.hal\
tHL = true;\x0a    \
  this.activeCod\
e.plainStream = \
true;\x0a      try \
{ this.activeCod\
e.codeEl.dataset\
.hlStreamSuspend\
ed = '1'; } catc\
h (_) {}\x0a      t\
his._d('STREAM_H\
L_SUSPENDED', { \
totalLines, stop\
After: stop, rea\
son: 'stopAfterL\
ines' });\x0a    }\x0a\
  }\x0a  _aliasLang\
(token) {\x0a    co\
nst ALIAS = {\x0a  \
    txt: 'plaint\
ext', text: 'pla\
intext', plainte\
xt: 'plaintext',\
\x0a      sh: 'bash\
', shell: 'bash'\
, zsh: 'bash', '\
shell-session': \
'bash',\x0a      py\
: 'python', pyth\
on3: 'python', p\
y3: 'python',\x0a  \
    js: 'javascr\
ipt', node: 'jav\
ascript', nodejs\
: 'javascript',\x0a\
      ts: 'types\
cript', 'ts-node\
': 'typescript',\
\x0a      yml: 'yam\
l', kt: 'kotlin'\
, rs: 'rust',\x0a  \
    csharp: 'csh\
arp', 'c#': 'csh\
arp', 'c++': 'cp\
p',\x0a      ps: 'p\
owershell', ps1:\
 'powershell', p\
wsh: 'powershell\
', powershell7: \
'powershell',\x0a  \
    docker: 'doc\
kerfile'\x0a    };\x0a\
    const v = St\
ring(token || ''\
).trim().toLower\
Case();\x0a    retu\
rn ALIAS[v] || v\
;\x0a  }\x0a  _isHLJSS\
upported(lang) {\
\x0a    try { retur\
n !!(window.hljs\
 && hljs.getLang\
uage && hljs.get\
Language(lang));\
 } catch (_) { r\
eturn false; }\x0a \
 }\x0a  _detectDire\
ctiveLangFromTex\
t(text) {\x0a    if\
 (!text) return \
null;\x0a    let s \
= String(text);\x0a\
    if (s.charCo\
deAt(0) === 0xFE\
FF) s = s.slice(\
1);\x0a    const li\
nes = s.split(/\x5c\
r?\x5cn/);\x0a    let \
i = 0; while (i \
< lines.length &\
& !lines[i].trim\
()) i++;\x0a    if \
(i >= lines.leng\
th) return null;\
\x0a    let first =\
 lines[i].trim()\
;\x0a    first = fi\
rst.replace(/^\x5cs\
*lang(?:uage)?\x5cs\
*[:=]\x5cs*/i, '').\
trim();\x0a    let \
token = first.sp\
lit(/\x5cs+/)[0].re\
place(/:$/, '');\
\x0a    if (!/^[A-Z\
a-z][\x5cw#+\x5c-\x5c.]{0\
,30}$/.test(toke\
n)) return null;\
\x0a\x0a    let cand =\
 this._aliasLang\
(token);\x0a    con\
st rest = lines.\
slice(i + 1).joi\
n('\x5cn');\x0a    if \
(!rest.trim()) r\
eturn null;\x0a\x0a   \
 let pos = 0, se\
en = 0;\x0a    whil\
e (seen < i && p\
os < s.length) {\
 const nl = s.in\
dexOf('\x5cn', pos)\
; if (nl === -1)\
 return null; po\
s = nl + 1; seen\
++; }\x0a    let en\
d = s.indexOf('\x5c\
n', pos);\x0a    if\
 (end === -1) en\
d = s.length; el\
se end = end + 1\
;\x0a    return { l\
ang: cand, delet\
eUpto: end };\x0a  \
}\x0a  _updateCodeL\
angClass(codeEl,\
 newLang) {\x0a    \
try {\x0a      Arra\
y.from(codeEl.cl\
assList).forEach\
(c => { if (c.st\
artsWith('langua\
ge-')) codeEl.cl\
assList.remove(c\
); });\x0a      cod\
eEl.classList.ad\
d('language-' + \
(newLang || 'pla\
intext'));\x0a    }\
 catch (_) {}\x0a  \
}\x0a  _updateCodeH\
eaderLabel(codeE\
l, newLabel, new\
LangToken) {\x0a   \
 try {\x0a      con\
st wrap = codeEl\
.closest('.code-\
wrapper');\x0a     \
 if (!wrap) retu\
rn;\x0a      const \
span = wrap.quer\
ySelector('.code\
-header-lang');\x0a\
      if (span) \
span.textContent\
 = newLabel || (\
newLangToken || \
'code');\x0a      w\
rap.setAttribute\
('data-code-lang\
', newLangToken \
|| '');\x0a    } ca\
tch (_) {}\x0a  }\x0a \
 maybePromoteLan\
guageFromDirecti\
ve() {\x0a    if (!\
this.activeCode \
|| !this.activeC\
ode.codeEl) retu\
rn;\x0a    if (this\
.activeCode.lang\
 && this.activeC\
ode.lang !== 'pl\
aintext') return\
;\x0a\x0a    const fro\
zenTxt = this.ac\
tiveCode.frozenE\
l ? this.activeC\
ode.frozenEl.tex\
tContent : '';\x0a \
   const tailTxt\
 = this.activeCo\
de.tailEl ? this\
.activeCode.tail\
El.textContent :\
 '';\x0a    const c\
ombined = frozen\
Txt + tailTxt;\x0a \
   if (!combined\
) return;\x0a\x0a    c\
onst det = this.\
_detectDirective\
LangFromText(com\
bined);\x0a    if (\
!det || !det.lan\
g) return;\x0a\x0a    \
const newLang = \
det.lang;\x0a    co\
nst newCombined \
= combined.slice\
(det.deleteUpto)\
;\x0a\x0a    try {\x0a   \
   const codeEl \
= this.activeCod\
e.codeEl;\x0a      \
codeEl.innerHTML\
 = '';\x0a      con\
st frozen = docu\
ment.createEleme\
nt('span'); froz\
en.className = '\
hl-frozen';\x0a    \
  const tail = d\
ocument.createEl\
ement('span'); t\
ail.className = \
'hl-tail';\x0a     \
 tail.textConten\
t = newCombined;\
\x0a      codeEl.ap\
pendChild(frozen\
); codeEl.append\
Child(tail);\x0a   \
   this.activeCo\
de.frozenEl = fr\
ozen; this.activ\
eCode.tailEl = t\
ail;\x0a      this.\
activeCode.froze\
nLen = 0;\x0a      \
this.activeCode.\
tailLines = Util\
s.countNewlines(\
newCombined);\x0a  \
    this.activeC\
ode.linesSincePr\
omote = 0;\x0a\x0a    \
  this.activeCod\
e.lang = newLang\
;\x0a      this._up\
dateCodeLangClas\
s(codeEl, newLan\
g);\x0a      this._\
updateCodeHeader\
Label(codeEl, ne\
wLang, newLang);\
\x0a\x0a      this._d(\
'LANG_PROMOTE', \
{ to: newLang, r\
emovedChars: det\
.deleteUpto, tai\
lLines: this.act\
iveCode.tailLine\
s });\x0a      this\
.schedulePromote\
Tail(true);\x0a    \
} catch (e) {\x0a  \
    this._d('LAN\
G_PROMOTE_ERR', \
String(e));\x0a    \
}\x0a  }\x0a  highligh\
tDeltaText(lang,\
 text) {\x0a    if \
(this.cfg.HL.DIS\
ABLE_ALL) return\
 Utils.escapeHtm\
l(text);\x0a    if \
(window.hljs && \
lang && hljs.get\
Language && hljs\
.getLanguage(lan\
g)) {\x0a      try \
{ return hljs.hi\
ghlight(text, { \
language: lang, \
ignoreIllegals: \
true }).value; }\
\x0a      catch (_)\
 { return Utils.\
escapeHtml(text)\
; }\x0a    }\x0a    re\
turn Utils.escap\
eHtml(text);\x0a  }\
\x0a  schedulePromo\
teTail(force = f\
alse) {\x0a    if (\
!this.activeCode\
 || !this.active\
Code.tailEl) ret\
urn;\x0a    if (thi\
s._promoteSchedu\
led) return;\x0a   \
 this._promoteSc\
heduled = true;\x0a\
    this.raf.sch\
edule('SE:promot\
eTail', () => {\x0a\
      this._prom\
oteScheduled = f\
alse;\x0a      this\
._promoteTailWor\
k(force);\x0a    },\
 'StreamEngine',\
 1);\x0a  }\x0a  async\
 _promoteTailWor\
k(force = false)\
 {\x0a    if (!this\
.activeCode || !\
this.activeCode.\
tailEl) return;\x0a\
\x0a    const now =\
 Utils.now(); co\
nst prof = this.\
cfg.PROFILE_CODE\
;\x0a    const tail\
Text0 = this.act\
iveCode.tailEl.t\
extContent || ''\
; if (!tailText0\
) return;\x0a\x0a    i\
f (!force) {\x0a   \
   if ((now - th\
is.activeCode.la\
stPromoteTs) < p\
rof.promoteMinIn\
terval) return;\x0a\
      const enou\
ghLines = (this.\
activeCode.lines\
SincePromote || \
0) >= (prof.prom\
oteMinLines || 1\
0);\x0a      const \
enoughChars = ta\
ilText0.length >\
= prof.minCharsF\
orHL;\x0a      if (\
!enoughLines && \
!enoughChars) re\
turn;\x0a    }\x0a\x0a   \
 const idx = tai\
lText0.lastIndex\
Of('\x5cn');\x0a    if\
 (idx <= -1 && !\
force) return;\x0a \
   const cut = (\
idx >= 0) ? (idx\
 + 1) : tailText\
0.length;\x0a    co\
nst delta = tail\
Text0.slice(0, c\
ut); if (!delta)\
 return;\x0a\x0a    th\
is.enforceHLStop\
Budget();\x0a    co\
nst usePlain = t\
his.activeCode.h\
altHL || this.ac\
tiveCode.plainSt\
ream || !this._i\
sHLJSSupported(t\
his.activeCode.l\
ang);\x0a\x0a    if (!\
usePlain) await \
this.asyncer.yie\
ld();\x0a\x0a    if (!\
this.activeCode \
|| !this.activeC\
ode.tailEl) retu\
rn;\x0a    const ta\
ilNow = this.act\
iveCode.tailEl.t\
extContent || ''\
;\x0a    if (!tailN\
ow.startsWith(de\
lta)) {\x0a      th\
is.schedulePromo\
teTail(false);\x0a \
     return;\x0a   \
 }\x0a\x0a    if (useP\
lain) {\x0a      th\
is.activeCode.fr\
ozenEl.insertAdj\
acentText('befor\
eend', delta);\x0a \
   } else {\x0a    \
  let html = Uti\
ls.escapeHtml(de\
lta);\x0a      try \
{ html = this.hi\
ghlightDeltaText\
(this.activeCode\
.lang, delta); }\
 catch (_) { htm\
l = Utils.escape\
Html(delta); }\x0a \
     this.active\
Code.frozenEl.in\
sertAdjacentHTML\
('beforeend', ht\
ml);\x0a    }\x0a\x0a    \
this.activeCode.\
tailEl.textConte\
nt = tailNow.sli\
ce(delta.length)\
;\x0a    this.activ\
eCode.frozenLen \
+= delta.length;\
\x0a    const promo\
tedLines = Utils\
.countNewlines(d\
elta);\x0a    this.\
activeCode.tailL\
ines = Math.max(\
0, (this.activeC\
ode.tailLines ||\
 0) - promotedLi\
nes);\x0a    this.a\
ctiveCode.linesS\
incePromote = Ma\
th.max(0, (this.\
activeCode.lines\
SincePromote || \
0) - promotedLin\
es);\x0a    this.ac\
tiveCode.lastPro\
moteTs = Utils.n\
ow();\x0a    this.c\
odeScroll.schedu\
leScroll(this.ac\
tiveCode.codeEl,\
 true, false);\x0a \
   this._d(usePl\
ain ? 'TAIL_PROM\
OTE_PLAIN' : 'TA\
IL_PROMOTE_ASYNC\
', { cut, promot\
edLines, lang: t\
his.activeCode.l\
ang, plain: useP\
lain });\x0a  }\x0a  f\
inalizeActiveCod\
e() {\x0a    if (!t\
his.activeCode) \
return;\x0a    cons\
t codeEl = this.\
activeCode.codeE\
l;\x0a    const fro\
mBottomBefore = \
Math.max(0, code\
El.scrollHeight \
- codeEl.clientH\
eight - codeEl.s\
crollTop);\x0a    c\
onst wasNearBott\
om = this.codeSc\
roll.isNearBotto\
mEl(codeEl, this\
.cfg.CODE_SCROLL\
.NEAR_MARGIN_PX)\
;\x0a    const full\
Text = (this.act\
iveCode.frozenEl\
.textContent || \
'') + (this.acti\
veCode.tailEl.te\
xtContent || '')\
;\x0a\x0a    try {\x0a   \
   codeEl.innerH\
TML = '';\x0a      \
codeEl.textConte\
nt = fullText;\x0a \
     codeEl.clas\
sList.add('hljs'\
);\x0a      codeEl.\
removeAttribute(\
'data-highlighte\
d');\x0a    } catch\
 (_) {}\x0a\x0a    con\
st st = this.cod\
eScroll.state(co\
deEl); st.autoFo\
llow = false;\x0a  \
  const maxScrol\
lTop = Math.max(\
0, codeEl.scroll\
Height - codeEl.\
clientHeight);\x0a \
   const target \
= wasNearBottom \
? maxScrollTop :\
 Math.max(0, max\
ScrollTop - from\
BottomBefore);\x0a \
   try { codeEl.\
scrollTop = targ\
et; } catch (_) \
{}\x0a    st.lastSc\
rollTop = codeEl\
.scrollTop;\x0a    \
codeEl.dataset._\
active_stream = \
'0';\x0a\x0a    try { \
codeEl.dataset.j\
ustFinalized = '\
1'; } catch (_) \
{}\x0a    this.code\
Scroll.scheduleS\
croll(codeEl, fa\
lse, true);\x0a\x0a   \
 try { if (!this\
.cfg.HL.DISABLE_\
ALL) this.highli\
ghter.queue(code\
El, null); } cat\
ch (_) {}\x0a\x0a    t\
his.suppressPost\
FinalizePass = t\
rue;\x0a\x0a    this._\
d('FINALIZE_CODE\
_NONBLOCK', { la\
ng: this.activeC\
ode.lang, len: f\
ullText.length, \
highlighted: fal\
se });\x0a    this.\
activeCode = nul\
l;\x0a  }\x0a  codeFin\
gerprint(codeEl)\
 {\x0a    const cls\
 = Array.from(co\
deEl.classList).\
find(c => c.star\
tsWith('language\
-')) || 'languag\
e-plaintext';\x0a  \
  const lang = c\
ls.replace('lang\
uage-', '') || '\
plaintext';\x0a    \
const t = codeEl\
.textContent || \
''; const len = \
t.length; const \
head = t.slice(0\
, 64); const tai\
l = t.slice(-64)\
;\x0a    return `${\
lang}|${len}|${h\
ead}|${tail}`;\x0a \
 }\x0a  codeFingerp\
rintFromWrapper(\
codeEl) {\x0a    tr\
y {\x0a      const \
wrap = codeEl.cl\
osest('.code-wra\
pper'); if (!wra\
p) return null;\x0a\
      const cls \
= Array.from(cod\
eEl.classList).f\
ind(c => c.start\
sWith('language-\
')) || 'language\
-plaintext';\x0a   \
   const lang = \
(cls.replace('la\
nguage-', '') ||\
 'plaintext');\x0a \
     const len =\
 wrap.getAttribu\
te('data-code-le\
n') || '';\x0a     \
 const head = wr\
ap.getAttribute(\
'data-code-head'\
) || '';\x0a      c\
onst tail = wrap\
.getAttribute('d\
ata-code-tail') \
|| '';\x0a      if \
(!len) return nu\
ll;\x0a      return\
 `${lang}|${len}\
|${head}|${tail}\
`;\x0a    } catch (\
_) {\x0a      retur\
n null;\x0a    }\x0a  \
}\x0a  preserveStab\
leClosedCodes(ol\
dSnap, newRoot, \
skipLastIfStream\
ing) {\x0a    try {\
\x0a      const old\
Codes = Array.fr\
om(oldSnap.query\
SelectorAll('pre\
 code')); if (!o\
ldCodes.length) \
return;\x0a      co\
nst newCodesPre \
= Array.from(new\
Root.querySelect\
orAll('pre code'\
));\x0a      if (ne\
wCodesPre.length\
 > this.cfg.STRE\
AM.PRESERVE_CODE\
S_MAX || oldCode\
s.length > this.\
cfg.STREAM.PRESE\
RVE_CODES_MAX) r\
eturn;\x0a\x0a      co\
nst map = new Ma\
p();\x0a      for (\
const el of oldC\
odes) {\x0a        \
if (el.querySele\
ctor('.hl-frozen\
')) continue;\x0a  \
      if (this.a\
ctiveCode && el \
=== this.activeC\
ode.codeEl) cont\
inue;\x0a        le\
t fp = this.code\
FingerprintFromW\
rapper(el);\x0a    \
    if (!fp) fp \
= el.dataset.fp \
|| (el.dataset.f\
p = this.codeFin\
gerprint(el));\x0a \
       const arr\
 = map.get(fp) |\
| []; arr.push(e\
l); map.set(fp, \
arr);\x0a      }\x0a  \
    const newCod\
es = newCodesPre\
;\x0a      const en\
d = (skipLastIfS\
treaming && newC\
odes.length > 0)\
 ? (newCodes.len\
gth - 1) : newCo\
des.length;\x0a    \
  let reuseCount\
 = 0;\x0a      for \
(let i = 0; i < \
end; i++) {\x0a    \
    const nc = n\
ewCodes[i];\x0a    \
    if (nc.getAt\
tribute('data-hi\
ghlighted') === \
'yes') continue;\
\x0a        let fp \
= this.codeFinge\
rprintFromWrappe\
r(nc);\x0a        i\
f (!fp) fp = thi\
s.codeFingerprin\
t(nc);\x0a        c\
onst arr = map.g\
et(fp);\x0a        \
if (arr && arr.l\
ength) {\x0a       \
   const oldEl =\
 arr.shift();\x0a  \
        if (oldE\
l && oldEl.isCon\
nected) {\x0a      \
      try {\x0a    \
          nc.rep\
laceWith(oldEl);\
\x0a              t\
his.codeScroll.a\
ttachHandlers(ol\
dEl);\x0a          \
    if (!oldEl.g\
etAttribute('dat\
a-highlighted'))\
 oldEl.setAttrib\
ute('data-highli\
ghted', 'yes');\x0a\
              co\
nst st = this.co\
deScroll.state(o\
ldEl); st.autoFo\
llow = false;\x0a  \
            reus\
eCount++;\x0a      \
      } catch (_\
) {}\x0a          }\
\x0a          if (!\
arr.length) map.\
delete(fp);\x0a    \
    }\x0a      }\x0a  \
    if (reuseCou\
nt) this._d('PRE\
SERVE_CODES_REUS\
ED', { reuseCoun\
t, skipLastIfStr\
eaming });\x0a    }\
 catch (e) {\x0a   \
   this._d('PRES\
ERVE_CODES_ERROR\
', String(e));\x0a \
   }\x0a  }\x0a  _ensu\
reBottomForJustF\
inalized(root) {\
\x0a    try {\x0a     \
 const scope = r\
oot || document;\
\x0a      const nod\
es = scope.query\
SelectorAll('pre\
 code[data-just-\
finalized=\x221\x22]')\
;\x0a      if (!nod\
es || !nodes.len\
gth) return;\x0a   \
   nodes.forEach\
((codeEl) => {\x0a \
       this.code\
Scroll.scheduleS\
croll(codeEl, fa\
lse, true);\x0a    \
    const key = \
{ t: 'JF:forceBo\
ttom', el: codeE\
l, n: Math.rando\
m() };\x0a        t\
his.raf.schedule\
(key, () => {\x0a  \
        this.cod\
eScroll.scrollTo\
Bottom(codeEl, f\
alse, true);\x0a   \
       try { cod\
eEl.dataset.just\
Finalized = '0';\
 } catch (_) {}\x0a\
        }, 'Code\
Scroll', 2);\x0a   \
   });\x0a    } cat\
ch (_) {}\x0a  }\x0a  \
kickVisibility()\
 {\x0a    const msg\
 = this.getMsg(f\
alse, '');\x0a    i\
f (!msg) return;\
\x0a    if (this.co\
deStream.open &&\
 !this.activeCod\
e) {\x0a      this.\
scheduleSnapshot\
(msg, true);\x0a   \
   return;\x0a    }\
\x0a    const needS\
nap = (this.getS\
treamLength() !=\
= (window.__last\
SnapshotLen || 0\
));\x0a    if (need\
Snap) this.sched\
uleSnapshot(msg,\
 true);\x0a    if (\
this.activeCode \
&& this.activeCo\
de.codeEl) {\x0a   \
   this.codeScro\
ll.scheduleScrol\
l(this.activeCod\
e.codeEl, true, \
false);\x0a      th\
is.schedulePromo\
teTail(true);\x0a  \
  }\x0a  }\x0a  stabil\
izeHeaderLabel(p\
revAC, newAC) {\x0a\
    try {\x0a      \
if (!newAC || !n\
ewAC.codeEl || !\
newAC.codeEl.isC\
onnected) return\
;\x0a\x0a      const w\
rap = newAC.code\
El.closest('.cod\
e-wrapper');\x0a   \
   if (!wrap) re\
turn;\x0a\x0a      con\
st span = wrap.q\
uerySelector('.c\
ode-header-lang'\
);\x0a      const c\
urLabel = (span \
&& span.textCont\
ent ? span.textC\
ontent.trim() : \
'').toLowerCase(\
);\x0a\x0a      if (cu\
rLabel === 'outp\
ut') return;\x0a\x0a  \
    const tokNow\
 = (wrap.getAttr\
ibute('data-code\
-lang') || '').t\
rim().toLowerCas\
e();\x0a      const\
 sticky = (wrap.\
getAttribute('da\
ta-lang-sticky')\
 || '').trim().t\
oLowerCase();\x0a  \
    const prev =\
 (prevAC && prev\
AC.lang && prevA\
C.lang !== 'plai\
ntext') ? prevAC\
.lang.toLowerCas\
e() : '';\x0a\x0a     \
 const valid = (\
t) => !!t && t !\
== 'plaintext' &\
& this._isHLJSSu\
pported(t);\x0a\x0a   \
   let finalTok \
= '';\x0a      if (\
valid(tokNow)) f\
inalTok = tokNow\
;\x0a      else if \
(valid(prev)) fi\
nalTok = prev;\x0a \
     else if (va\
lid(sticky)) fin\
alTok = sticky;\x0a\
\x0a      if (final\
Tok) {\x0a        t\
his._updateCodeL\
angClass(newAC.c\
odeEl, finalTok)\
;\x0a        this._\
updateCodeHeader\
Label(newAC.code\
El, finalTok, fi\
nalTok);\x0a       \
 try { wrap.setA\
ttribute('data-c\
ode-lang', final\
Tok); } catch (_\
) {}\x0a        try\
 { wrap.setAttri\
bute('data-lang-\
sticky', finalTo\
k); } catch (_) \
{}\x0a        newAC\
.lang = finalTok\
;\x0a      } else {\
\x0a        if (spa\
n && curLabel &&\
 curLabel.length\
 < 3) {\x0a        \
  span.textConte\
nt = 'code';\x0a   \
     }\x0a      }\x0a \
   } catch (_) {\
 /* defensive: n\
ever break strea\
ming path */ }\x0a \
 }\x0a\x0a  renderSnap\
shot(msg) {\x0a    \
const streaming \
= !!this.isStrea\
ming;\x0a    const \
snap = this.getM\
sgSnapshotRoot(m\
sg); if (!snap) \
return;\x0a\x0a    con\
st prevLen = (wi\
ndow.__lastSnaps\
hotLen || 0);\x0a  \
  const curLen =\
 this.getStreamL\
ength();\x0a    if \
(!this.fenceOpen\
 && !this.active\
Code && curLen =\
== prevLen) {\x0a  \
    this.lastSna\
pshotTs = Utils.\
now();\x0a      thi\
s._d('SNAPSHOT_S\
KIPPED_NO_DELTA'\
, { bufLen: curL\
en });\x0a      ret\
urn;\x0a    }\x0a\x0a    \
const t0 = Utils\
.now();\x0a\x0a    // \
When an open fen\
ce is present, a\
ppend a syntheti\
c EOL only if th\
e current buffer\
\x0a    // does not\
 already end wit\
h EOL. This stab\
ilizes markdown-\
it parsing witho\
ut polluting\x0a   \
 // the real cod\
e tail (we will \
strip this EOL f\
rom the active t\
ail right after \
snapshot).\x0a    c\
onst allText = t\
his.getStreamTex\
t();\x0a    const n\
eedSyntheticEOL \
= (this.fenceOpe\
n && !/[\x5cr\x5cn]$/.\
test(allText));\x0a\
    this._lastIn\
jectedEOL = !!ne\
edSyntheticEOL;\x0a\
    const src = \
needSyntheticEOL\
 ? (allText + '\x5c\
n') : allText;\x0a\x0a\
    const html =\
 streaming ? thi\
s.renderer.rende\
rStreamingSnapsh\
ot(src) : this.r\
enderer.renderFi\
nalSnapshot(src)\
;\x0a\x0a    let frag \
= null;\x0a    try \
{\x0a      const ra\
nge = document.c\
reateRange();\x0a  \
    range.select\
NodeContents(sna\
p);\x0a      frag =\
 range.createCon\
textualFragment(\
html);\x0a    } cat\
ch (_) {\x0a      c\
onst tmp = docum\
ent.createElemen\
t('div');\x0a      \
tmp.innerHTML = \
html;\x0a      frag\
 = document.crea\
teDocumentFragme\
nt();\x0a      whil\
e (tmp.firstChil\
d) frag.appendCh\
ild(tmp.firstChi\
ld);\x0a    }\x0a\x0a    \
// (stream-aware\
 custom markup):\
\x0a    try {\x0a     \
 if (this.render\
er && this.rende\
rer.customMarkup\
 && this.rendere\
r.customMarkup.h\
asStreamRules())\
 {\x0a        const\
 MDinline = this\
.renderer.MD_STR\
EAM || this.rend\
erer.MD || null;\
\x0a        this.re\
nderer.customMar\
kup.applyStream(\
frag, MDinline);\
\x0a      }\x0a    } c\
atch (_) { /* ke\
ep snapshot path\
 resilient */ }\x0a\
\x0a    this.preser\
veStableClosedCo\
des(snap, frag, \
this.fenceOpen =\
== true);\x0a\x0a    s\
nap.replaceChild\
ren(frag);\x0a\x0a    \
this.renderer.re\
storeCollapsedCo\
de(snap);\x0a    th\
is._ensureBottom\
ForJustFinalized\
(snap);\x0a\x0a    con\
st prevAC = this\
.activeCode;\x0a\x0a  \
  if (this.fence\
Open) {\x0a      co\
nst newAC = this\
.setupActiveCode\
FromSnapshot(sna\
p);\x0a\x0a      if (p\
revAC && newAC) \
{\x0a        this.r\
ehydrateActiveCo\
de(prevAC, newAC\
);\x0a        this.\
stabilizeHeaderL\
abel(prevAC, new\
AC);\x0a      }\x0a\x0a  \
    this.activeC\
ode = newAC || n\
ull;\x0a    } else \
{\x0a      this.act\
iveCode = null;\x0a\
    }\x0a\x0a    if (!\
this.fenceOpen) \
{\x0a      this.cod\
eScroll.initScro\
llableBlocks(sna\
p);\x0a    }\x0a    th\
is.highlighter.o\
bserveNewCode(sn\
ap, {\x0a      defe\
rLastIfStreaming\
: true,\x0a      mi\
nLinesForLast: t\
his.cfg.PROFILE_\
CODE.minLinesFor\
HL,\x0a      minCha\
rsForLast: this.\
cfg.PROFILE_CODE\
.minCharsForHL\x0a \
   }, this.activ\
eCode);\x0a    this\
.highlighter.obs\
erveMsgBoxes(sna\
p, (box) => {\x0a  \
    this.highlig\
hter.observeNewC\
ode(box, {\x0a     \
   deferLastIfSt\
reaming: true,\x0a \
       minLinesF\
orLast: this.cfg\
.PROFILE_CODE.mi\
nLinesForHL,\x0a   \
     minCharsFor\
Last: this.cfg.P\
ROFILE_CODE.minC\
harsForHL\x0a      \
}, this.activeCo\
de);\x0a      this.\
codeScroll.initS\
crollableBlocks(\
box);\x0a    });\x0a\x0a \
   const mm = ge\
tMathMode();\x0a   \
 if (!this.suppr\
essPostFinalizeP\
ass) {\x0a      if \
(mm === 'idle') \
this.math.schedu\
le(snap);\x0a      \
else if (mm === \
'always') this.m\
ath.schedule(sna\
p, 0, true);\x0a   \
 }\x0a\x0a    if (this\
.fenceOpen && th\
is.activeCode &&\
 this.activeCode\
.codeEl) {\x0a     \
 this.codeScroll\
.attachHandlers(\
this.activeCode.\
codeEl);\x0a      t\
his.codeScroll.s\
cheduleScroll(th\
is.activeCode.co\
deEl, true, fals\
e);\x0a    } else i\
f (!this.fenceOp\
en) {\x0a      this\
.codeScroll.init\
ScrollableBlocks\
(snap);\x0a    }\x0a\x0a \
   window.__last\
SnapshotLen = th\
is.getStreamLeng\
th();\x0a    this.l\
astSnapshotTs = \
Utils.now();\x0a\x0a  \
  const prof = t\
his.profile();\x0a \
   if (prof.adap\
tiveStep) {\x0a    \
  const maxStep \
= this.cfg.STREA\
M.SNAPSHOT_MAX_S\
TEP || 8000;\x0a   \
   this.nextSnap\
shotStep = Math.\
min(Math.ceil(th\
is.nextSnapshotS\
tep * prof.growt\
h), maxStep);\x0a  \
  } else {\x0a     \
 this.nextSnapsh\
otStep = prof.ba\
se;\x0a    }\x0a\x0a    t\
his.scrollMgr.sc\
heduleScroll(tru\
e);\x0a    this.scr\
ollMgr.fabFreeze\
Until = Utils.no\
w() + this.cfg.F\
AB.TOGGLE_DEBOUN\
CE_MS;\x0a    this.\
scrollMgr.schedu\
leScrollFabUpdat\
e();\x0a\x0a    if (th\
is.suppressPostF\
inalizePass) thi\
s.suppressPostFi\
nalizePass = fal\
se;\x0a\x0a    const d\
t = Utils.now() \
- t0;\x0a    this._\
d('SNAPSHOT', { \
fenceOpen: this.\
fenceOpen, activ\
eCode: !!this.ac\
tiveCode, bufLen\
: this.getStream\
Length(), timeMs\
: Math.round(dt)\
, streaming });\x0a\
  }\x0a\x0a  getMsg(cr\
eate, name_heade\
r) { return this\
.dom.getStreamMs\
g(create, name_h\
eader); }\x0a  begi\
nStream(chunk = \
false) {\x0a    thi\
s.isStreaming = \
true;\x0a    if (ch\
unk) runtime.loa\
ding.hide();\x0a   \
 this.scrollMgr.\
userInteracted =\
 false;\x0a    this\
.dom.clearOutput\
();\x0a    this.res\
et();\x0a    this.s\
crollMgr.forceSc\
rollToBottomImme\
diate();\x0a    thi\
s.scrollMgr.sche\
duleScroll();\x0a  \
  this._d('BEGIN\
_STREAM', { chun\
kFlag: !!chunk }\
);\x0a  }\x0a  endStre\
am() {\x0a    this.\
isStreaming = fa\
lse;\x0a\x0a    const \
msg = this.getMs\
g(false, '');\x0a  \
  if (msg) this.\
renderSnapshot(m\
sg);\x0a\x0a    this.s\
napshotScheduled\
 = false;\x0a    tr\
y { this.raf.can\
cel('SE:snapshot\
'); } catch (_) \
{}\x0a    this.snap\
shotRAF = 0;\x0a\x0a  \
  const hadActiv\
e = !!this.activ\
eCode;\x0a    if (t\
his.activeCode) \
this.finalizeAct\
iveCode();\x0a\x0a    \
if (!hadActive) \
{\x0a      if (this\
.highlighter.hlQ\
ueue && this.hig\
hlighter.hlQueue\
.length) {\x0a     \
   this.highligh\
ter.flush(this.a\
ctiveCode);\x0a    \
  }\x0a      const \
snap = msg ? thi\
s.getMsgSnapshot\
Root(msg) : null\
;\x0a      if (snap\
) this.math.rend\
erAsync(snap);\x0a \
   }\x0a\x0a    this.f\
enceOpen = false\
; this.codeStrea\
m.open = false; \
this.activeCode \
= null; this.las\
tSnapshotTs = Ut\
ils.now();\x0a    t\
his.suppressPost\
FinalizePass = f\
alse;\x0a    this._\
d('END_STREAM', \
{ hadActive });\x0a\
  }\x0a\x0a  // NEW: e\
ager snapshot de\
tection for cust\
om stream opener\
s (e.g., <think>\
, <tool>)\x0a  _may\
beEagerSnapshotF\
orCustomOpeners(\
msg, chunkStr) {\
\x0a    try {\x0a     \
 const CM = this\
.renderer && thi\
s.renderer.custo\
mMarkup;\x0a      i\
f (!CM || !CM.ha\
sStreamRules()) \
return;\x0a\x0a      /\
/ Do not interfe\
re with code fen\
ce streaming.\x0a  \
    if (this.fen\
ceOpen || this.c\
odeStream.open) \
return;\x0a\x0a      c\
onst isFirstSnap\
shot = ((window.\
__lastSnapshotLe\
n || 0) === 0);\x0a\
\x0a      if (isFir\
stSnapshot) {\x0a  \
      // Cheap f\
irst-chunk check\
: if stream star\
ts with a custom\
 opener, snapsho\
t immediately.\x0a \
       let head;\
\x0a        try { h\
ead = this.getSt\
reamText(); } ca\
tch (_) { head =\
 String(chunkStr\
 || ''); }\x0a     \
   if (CM.hasStr\
eamOpenerAtStart\
(head)) {\x0a      \
    this._d('CM_\
EAGER_SNAPSHOT_S\
TART', { openerA\
tStart: true });\
\x0a          this.\
scheduleSnapshot\
(msg, true);\x0a   \
       return;\x0a \
       }\x0a      }\
\x0a\x0a      // For l\
ater chunks: if \
this chunk conta\
ins any stream o\
pener token, sna\
pshot soon\x0a     \
 // to let Custo\
mMarkup.applyStr\
eamPartialOpener\
s extend the pen\
ding block.\x0a    \
  const rules = \
(CM.getRules() |\
| []).filter(r =\
> r && r.stream \
&& typeof r.open\
 === 'string');\x0a\
      if (rules.\
length && CM.has\
AnyOpenToken(Str\
ing(chunkStr || \
''), rules)) {\x0a \
       this._d('\
CM_EAGER_SNAPSHO\
T_CHUNK', { toke\
nFound: true, ch\
unkLen: (chunkSt\
r || '').length \
});\x0a        this\
.scheduleSnapsho\
t(msg); // norma\
l (coalesced) sc\
hedule is enough\
\x0a      }\x0a    } c\
atch (_) {\x0a     \
 // Keep streami\
ng resilient; ne\
ver throw here.\x0a\
    }\x0a  }\x0a\x0a  // \
Apply incoming c\
hunk to stream b\
uffer and update\
 DOM when needed\
.\x0a  applyStream(\
name_header, chu\
nk, alreadyBuffe\
red = false) {\x0a \
   if (!this.act\
iveCode && !this\
.fenceOpen) {\x0a  \
    try { if (do\
cument.querySele\
ctor('pre code[d\
ata-_active_stre\
am=\x221\x22]')) this.\
defuseOrphanActi\
veBlocks(); } ca\
tch (_) {}\x0a    }\
\x0a    if (this.sn\
apshotScheduled \
&& !this.raf.isS\
cheduled('SE:sna\
pshot')) this.sn\
apshotScheduled \
= false;\x0a\x0a    co\
nst msg = this.g\
etMsg(true, name\
_header); if (!m\
sg || !chunk) re\
turn;\x0a    const \
s = String(chunk\
);\x0a    if (!alre\
adyBuffered) thi\
s._appendChunk(s\
);\x0a\x0a    const ch\
ange = this.upda\
teFenceHeuristic\
(s);\x0a    const n\
lCount = Utils.c\
ountNewlines(s);\
 const chunkHasN\
L = nlCount > 0;\
\x0a\x0a    this._d('A\
PPLY_CHUNK', { l\
en: s.length, nl\
: nlCount, opene\
d: change.opened\
, closed: change\
.closed, splitAt\
: change.splitAt\
, fenceOpenBefor\
e: this.fenceOpe\
n || false, code\
OpenBefore: this\
.codeStream.open\
 || false, rebro\
adcast: !!alread\
yBuffered });\x0a\x0a \
   // Eager snap\
shot for custom \
stream openers (\
non-code context\
s).\x0a    // This \
ensures tags lik\
e <think> immedi\
ately turn on \x22p\
ending block\x22 be\
havior across th\
e rest of the sn\
apshot.\x0a    if (\
!change.opened &\
& !this.fenceOpe\
n) {\x0a      this.\
_maybeEagerSnaps\
hotForCustomOpen\
ers(msg, s);\x0a   \
 }\x0a\x0a    // Track\
 if we just mate\
rialized the fir\
st code-open sna\
pshot synchronou\
sly.\x0a    let did\
ImmediateOpenSna\
p = false;\x0a\x0a    \
if (change.opene\
d) {\x0a      this.\
codeStream.open \
= true; this.cod\
eStream.lines = \
0; this.codeStre\
am.chars = 0;\x0a  \
    this.resetBu\
dget();\x0a      th\
is.scheduleSnaps\
hot(msg);\x0a      \
this._d('CODE_ST\
REAM_OPEN', { })\
;\x0a\x0a      if (!th\
is._firstCodeOpe\
nSnapDone && !th\
is.activeCode &&\
 ((window.__last\
SnapshotLen || 0\
) === 0)) {\x0a    \
    try {\x0a      \
    this.renderS\
napshot(msg);\x0a  \
        try { th\
is.raf.cancel('S\
E:snapshot'); } \
catch (_) {}\x0a   \
       this.snap\
shotScheduled = \
false;\x0a         \
 this._firstCode\
OpenSnapDone = t\
rue;\x0a          d\
idImmediateOpenS\
nap = true;\x0a    \
      this._d('C\
ODE_OPEN_IMMEDIA\
TE_SNAPSHOT', { \
bufLen: this.get\
StreamLength() }\
);\x0a        } cat\
ch (_) {\x0a       \
   // Normal sch\
eduled snapshot \
will land soon.\x0a\
        }\x0a      \
}\x0a    }\x0a\x0a    if \
(this.codeStream\
.open) {\x0a      t\
his.codeStream.l\
ines += nlCount;\
 this.codeStream\
.chars += s.leng\
th;\x0a\x0a      if (t\
his.activeCode &\
& this.activeCod\
e.codeEl && this\
.activeCode.code\
El.isConnected) \
{\x0a        let pa\
rtForCode = s; l\
et remainder = '\
';\x0a\x0a        if (\
didImmediateOpen\
Snap) {\x0a        \
  partForCode = \
'';\x0a        } el\
se {\x0a          i\
f (change.closed\
 && change.split\
At >= 0 && chang\
e.splitAt <= s.l\
ength) {\x0a       \
     partForCode\
 = s.slice(0, ch\
ange.splitAt); r\
emainder = s.sli\
ce(change.splitA\
t);\x0a          }\x0a\
        }\x0a\x0a     \
   if (partForCo\
de) {\x0a          \
this.appendToAct\
iveTail(partForC\
ode);\x0a          \
this.activeCode.\
lines += Utils.c\
ountNewlines(par\
tForCode);\x0a\x0a    \
      this.maybe\
PromoteLanguageF\
romDirective();\x0a\
          this.e\
nforceHLStopBudg\
et();\x0a\x0a         \
 if (!this.activ\
eCode.haltHL) {\x0a\
            if (\
partForCode.inde\
xOf('\x5cn') >= 0 |\
| (this.activeCo\
de.tailEl.textCo\
ntent || '').len\
gth >= this.cfg.\
PROFILE_CODE.min\
CharsForHL) {\x0a  \
            this\
.schedulePromote\
Tail(false);\x0a   \
         }\x0a     \
     }\x0a        }\
\x0a        this.sc\
rollMgr.scrollFa\
bUpdateScheduled\
 = false;\x0a      \
  this.scrollMgr\
.scheduleScroll(\
true);\x0a        t\
his.scrollMgr.fa\
bFreezeUntil = U\
tils.now() + thi\
s.cfg.FAB.TOGGLE\
_DEBOUNCE_MS;\x0a  \
      this.scrol\
lMgr.scheduleScr\
ollFabUpdate();\x0a\
\x0a        if (cha\
nge.closed) {\x0a  \
        this.fin\
alizeActiveCode(\
);\x0a          thi\
s.codeStream.ope\
n = false; this.\
resetBudget(); t\
his.scheduleSnap\
shot(msg);\x0a     \
     this._d('CO\
DE_STREAM_CLOSE_\
FINALIZED', { re\
mainderLen: rema\
inder.length });\
\x0a          if (r\
emainder && rema\
inder.length) { \
this.applyStream\
(name_header, re\
mainder, true); \
}\x0a        }\x0a    \
    return;\x0a    \
  } else {\x0a     \
   if (!this.act\
iveCode && (this\
.codeStream.line\
s >= 2 || this.c\
odeStream.chars \
>= 80)) {\x0a      \
    this.schedul\
eSnapshot(msg, t\
rue);\x0a          \
return;\x0a        \
}\x0a        if (ch\
ange.closed) {\x0a \
         this.co\
deStream.open = \
false; this.rese\
tBudget(); this.\
scheduleSnapshot\
(msg);\x0a         \
 this._d('CODE_C\
LOSED_WITHOUT_AC\
TIVE', { sinceLa\
stSnapMs: Math.r\
ound(Utils.now()\
 - this.lastSnap\
shotTs), snapsho\
tScheduled: this\
.snapshotSchedul\
ed });\x0a        }\
 else {\x0a        \
  const boundary\
 = this.hasStruc\
turalBoundary(s)\
;\x0a          if (\
this.shouldSnaps\
hotOnChunk(s, ch\
unkHasNL, bounda\
ry)) this.schedu\
leSnapshot(msg);\
\x0a          else \
this.maybeSchedu\
leSoftSnapshot(m\
sg, chunkHasNL);\
\x0a        }\x0a     \
   return;\x0a     \
 }\x0a    }\x0a\x0a    if\
 (change.closed)\
 {\x0a      this.co\
deStream.open = \
false; this.rese\
tBudget(); this.\
scheduleSnapshot\
(msg);\x0a      thi\
s._d('CODE_STREA\
M_CLOSE', { });\x0a\
    } else {\x0a   \
   const boundar\
y = this.hasStru\
cturalBoundary(s\
);\x0a      if (thi\
s.shouldSnapshot\
OnChunk(s, chunk\
HasNL, boundary)\
) {\x0a        this\
.scheduleSnapsho\
t(msg);\x0a        \
this._d('SCHEDUL\
E_SNAPSHOT_BOUND\
ARY', { boundary\
 });\x0a      } els\
e {\x0a        this\
.maybeScheduleSo\
ftSnapshot(msg, \
chunkHasNL);\x0a   \
   }\x0a    }\x0a  }\x0a}\
\x0a\x0a  // =========\
================\
================\
================\
================\
=\x0a  // 12) Strea\
m queue\x0a  // ===\
================\
================\
================\
================\
=======\x0a\x0a  class\
 StreamQueue {\x0a \
   constructor(c\
fg, engine, scro\
llMgr, raf) {\x0a  \
    this.cfg = c\
fg; this.engine \
= engine; this.s\
crollMgr = scrol\
lMgr; this.raf =\
 raf;\x0a      this\
.q = []; this.dr\
ainScheduled = f\
alse;\x0a      this\
.batching = fals\
e; this.needScro\
ll = false;\x0a    \
}\x0a    // Coalesc\
e contiguous ent\
ries for the sam\
e header to redu\
ce overhead.\x0a   \
 _compactContigu\
ousSameName() {\x0a\
      // Coalesc\
e contiguous ent\
ries for the sam\
e message header\
 to reduce strin\
g objects count.\
\x0a      if (this.\
q.length < 2) re\
turn;\x0a      cons\
t out = [];\x0a    \
  let last = thi\
s.q[0];\x0a      fo\
r (let i = 1; i \
< this.q.length;\
 i++) {\x0a        \
const cur = this\
.q[i];\x0a        i\
f (cur.name_head\
er === last.name\
_header) {\x0a     \
     // Merge pa\
yloads; reduce o\
bject overhead w\
ithout changing \
semantics\x0a      \
    last.chunk =\
 (last.chunk || \
'') + (cur.chunk\
 || '');\x0a       \
 }\x0a        else \
{\x0a          out.\
push(last);\x0a    \
      last = cur\
;\x0a        }\x0a    \
  }\x0a      out.pu\
sh(last);\x0a      \
this.q = out;\x0a  \
  }\x0a    // Push \
new chunk into t\
he queue and sch\
edule drain.\x0a   \
 enqueue(name_he\
ader, chunk) {\x0a \
     this.q.push\
({ name_header, \
chunk });\x0a      \
// Guard against\
 unbounded growt\
h during bursts\x0a\
      if (this.q\
.length > this.c\
fg.STREAM.EMERGE\
NCY_COALESCE_LEN\
) this._compactC\
ontiguousSameNam\
e();\x0a      if (t\
his.q.length > t\
his.cfg.STREAM.Q\
UEUE_MAX_ITEMS) \
this._compactCon\
tiguousSameName(\
);\x0a      if (!th\
is.drainSchedule\
d) {\x0a        thi\
s.drainScheduled\
 = true;\x0a       \
 this.raf.schedu\
le('SQ:drain', (\
) => this.drain(\
), 'StreamQueue'\
, 0);\x0a      }\x0a  \
  }\x0a    // Drain\
 a limited numbe\
r of chunks per \
frame (adaptive \
if configured).\x0a\
    drain() {\x0a  \
    this.drainSc\
heduled = false;\
 let processed =\
 0;\x0a      const \
adaptive = (this\
.cfg.STREAM.COAL\
ESCE_MODE === 'a\
daptive');\x0a     \
 const coalesceA\
ggressive = adap\
tive && (this.q.\
length >= this.c\
fg.STREAM.EMERGE\
NCY_COALESCE_LEN\
);\x0a\x0a      // Ada\
ptive per-frame \
budget: increase\
 throughput if q\
ueue is long\x0a   \
   const basePer\
Frame = this.cfg\
.STREAM.MAX_PER_\
FRAME | 0;\x0a     \
 const perFrame \
= adaptive ? Mat\
h.min(basePerFra\
me + Math.floor(\
this.q.length / \
20), basePerFram\
e * 4) : basePer\
Frame;\x0a\x0a      th\
is.batching = tr\
ue;\x0a      while \
(this.q.length &\
& processed < pe\
rFrame) {\x0a      \
  let { name_hea\
der, chunk } = t\
his.q.shift();\x0a \
       if (chunk\
 && chunk.length\
 > 0) {\x0a        \
  const chunks =\
 [chunk];\x0a      \
    while (this.\
q.length) {\x0a    \
        const ne\
xt = this.q[0];\x0a\
            if (\
next.name_header\
 === name_header\
) {\x0a            \
  chunks.push(ne\
xt.chunk); this.\
q.shift();\x0a     \
         if (!co\
alesceAggressive\
) break;\x0a       \
     } else brea\
k;\x0a          }\x0a \
         chunk =\
 chunks.join('')\
;\x0a        }\x0a    \
    this.engine.\
applyStream(name\
_header, chunk);\
\x0a        process\
ed++;\x0a      }\x0a  \
    this.batchin\
g = false;\x0a     \
 if (this.needSc\
roll) { this.scr\
ollMgr.scheduleS\
croll(true); thi\
s.needScroll = f\
alse; }\x0a      if\
 (this.q.length)\
 {\x0a        this.\
drainScheduled =\
 true;\x0a        t\
his.raf.schedule\
('SQ:drain', () \
=> this.drain(),\
 'StreamQueue', \
0);\x0a      }\x0a    \
}\x0a    // Force a\
 drain soon.\x0a   \
 kick() {\x0a      \
if (this.q.lengt\
h || this.drainS\
cheduled) {\x0a    \
    this.drainSc\
heduled = true;\x0a\
        this.raf\
.schedule('SQ:dr\
ain', () => this\
.drain(), 'Strea\
mQueue', 0);\x0a   \
   }\x0a    }\x0a    /\
/ Clear queued w\
ork and cancel s\
cheduled drains.\
\x0a    clear() {\x0a \
     this.q.leng\
th = 0;\x0a      tr\
y { this.raf.can\
celGroup('Stream\
Queue'); } catch\
 (_) {}\x0a      th\
is.drainSchedule\
d = false;\x0a    }\
\x0a  }\x0a\x0a  // =====\
================\
================\
================\
================\
=====\x0a  // 13) B\
ridge manager (Q\
WebChannel)\x0a  //\
 ===============\
================\
================\
================\
===========\x0a\x0a  c\
lass BridgeManag\
er {\x0a    constru\
ctor(cfg, logger\
) {\x0a      this.c\
fg = cfg; this.l\
ogger = logger |\
| new Logger(cfg\
);\x0a      this.br\
idge = null; thi\
s.connected = fa\
lse;\x0a    }\x0a    l\
og(text) { try {\
 if (this.bridge\
 && this.bridge.\
log) this.bridge\
.log(text); } ca\
tch (_) {} }\x0a   \
 connect(onChunk\
, onNode, onNode\
Replace, onNodeI\
nput) {\x0a      if\
 (!this.bridge) \
return false; if\
 (this.connected\
) return true;\x0a \
     try {\x0a     \
   if (this.brid\
ge.chunk) this.b\
ridge.chunk.conn\
ect((name, chunk\
, type) => onChu\
nk(name, chunk, \
type));\x0a        \
if (this.bridge.\
node) this.bridg\
e.node.connect(o\
nNode);\x0a        \
if (this.bridge.\
nodeReplace) thi\
s.bridge.nodeRep\
lace.connect(onN\
odeReplace);\x0a   \
     if (this.br\
idge.nodeInput) \
this.bridge.node\
Input.connect(on\
NodeInput);\x0a    \
    this.connect\
ed = true; retur\
n true;\x0a      } \
catch (e) { this\
.log(e); return \
false; }\x0a    }\x0a \
   disconnect() \
{\x0a      if (!thi\
s.bridge) return\
 false; if (!thi\
s.connected) ret\
urn true;\x0a      \
try {\x0a        if\
 (this.bridge.ch\
unk) this.bridge\
.chunk.disconnec\
t();\x0a        if \
(this.bridge.nod\
e) this.bridge.n\
ode.disconnect()\
;\x0a        if (th\
is.bridge.nodeRe\
place) this.brid\
ge.nodeReplace.d\
isconnect();\x0a   \
     if (this.br\
idge.nodeInput) \
this.bridge.node\
Input.disconnect\
();\x0a      } catc\
h (_) {}\x0a      t\
his.connected = \
false; return tr\
ue;\x0a    }\x0a    in\
itQWebChannel(pi\
d, onReady) {\x0a  \
    try {\x0a      \
  new QWebChanne\
l(qt.webChannelT\
ransport, (chann\
el) => {\x0a       \
   this.bridge =\
 channel.objects\
.bridge;\x0a       \
   try { this.lo\
gger.bindBridge(\
this.bridge); } \
catch (_) {}\x0a   \
       onReady &\
& onReady(this.b\
ridge);\x0a        \
  if (this.bridg\
e && this.bridge\
.js_ready) this.\
bridge.js_ready(\
pid);\x0a        })\
;\x0a      } catch \
(e) { /* swallow\
 */ }\x0a    }\x0a    \
copyCode(text) {\
 if (this.bridge\
 && this.bridge.\
copy_text) this.\
bridge.copy_text\
(text); }\x0a    pr\
eviewCode(text) \
{ if (this.bridg\
e && this.bridge\
.preview_text) t\
his.bridge.previ\
ew_text(text); }\
\x0a    runCode(tex\
t) { if (this.br\
idge && this.bri\
dge.run_text) th\
is.bridge.run_te\
xt(text); }\x0a    \
updateScrollPosi\
tion(pos) { if (\
this.bridge && t\
his.bridge.updat\
e_scroll_positio\
n) this.bridge.u\
pdate_scroll_pos\
ition(pos); }\x0a  \
}\x0a\x0a  // ========\
================\
================\
================\
================\
==\x0a  // 14) Load\
ing indicator\x0a  \
// =============\
================\
================\
================\
=============\x0a\x0a \
 class Loading {\
\x0a    constructor\
(dom) { this.dom\
 = dom; }\x0a    //\
 Show loader ele\
ment (and hide t\
ips if visible).\
\x0a    show() { if\
 (typeof window.\
hideTips === 'fu\
nction') { windo\
w.hideTips(); } \
const el = this.\
dom.get('_loader\
_'); if (!el) re\
turn; if (el.cla\
ssList.contains(\
'hidden')) el.cl\
assList.remove('\
hidden'); el.cla\
ssList.add('visi\
ble'); }\x0a    // \
Hide loader elem\
ent.\x0a    hide() \
{ const el = thi\
s.dom.get('_load\
er_'); if (!el) \
return; if (el.c\
lassList.contain\
s('visible')) el\
.classList.remov\
e('visible'); el\
.classList.add('\
hidden'); }\x0a  }\x0a\
\x0a  // ==========\
================\
================\
================\
================\
\x0a  // 15) Event \
manager\x0a  // ===\
================\
================\
================\
================\
=======\x0a\x0a  class\
 EventManager {\x0a\
    constructor(\
cfg, dom, scroll\
Mgr, highlighter\
, codeScroll, to\
olOutput, bridge\
) {\x0a      this.c\
fg = cfg; this.d\
om = dom; this.s\
crollMgr = scrol\
lMgr; this.highl\
ighter = highlig\
hter;\x0a      this\
.codeScroll = co\
deScroll; this.t\
oolOutput = tool\
Output; this.bri\
dge = bridge;\x0a  \
    this.handler\
s = { wheel: nul\
l, scroll: null,\
 resize: null, f\
abClick: null, m\
ouseover: null, \
mouseout: null, \
click: null, key\
down: null, docC\
lickFocus: null,\
 visibility: nul\
l, focus: null, \
pageshow: null }\
;\x0a    }\x0a    _fin\
dWrapper(target)\
 { if (!target |\
| typeof target.\
closest !== 'fun\
ction') return n\
ull; return targ\
et.closest('.cod\
e-wrapper'); }\x0a \
   _getCodeEl(wr\
apper) { if (!wr\
apper) return nu\
ll; return wrapp\
er.querySelector\
('pre > code'); \
}\x0a    _collectCo\
deText(codeEl) {\
\x0a      if (!code\
El) return '';\x0a \
     const froze\
n = codeEl.query\
Selector('.hl-fr\
ozen'); const ta\
il = codeEl.quer\
ySelector('.hl-t\
ail');\x0a      if \
(frozen || tail)\
 return (frozen?\
.textContent || \
'') + (tail?.tex\
tContent || '');\
\x0a      return co\
deEl.textContent\
 || '';\x0a    }\x0a  \
  // Copy to cli\
pboard via bridg\
e if available, \
otherwise use br\
owser APIs.\x0a    \
async _copyTextR\
obust(text) {\x0a  \
    try { if (th\
is.bridge && typ\
eof this.bridge.\
copyCode === 'fu\
nction') { this.\
bridge.copyCode(\
text); return tr\
ue; } } catch (_\
) {}\x0a      try {\
 if (navigator &\
& navigator.clip\
board && navigat\
or.clipboard.wri\
teText) { await \
navigator.clipbo\
ard.writeText(te\
xt); return true\
; } } catch (_) \
{}\x0a      try {\x0a \
       const ta \
= document.creat\
eElement('textar\
ea');\x0a        ta\
.value = text; t\
a.setAttribute('\
readonly', ''); \
ta.style.positio\
n = 'fixed'; ta.\
style.top = '-99\
99px'; ta.style.\
opacity = '0';\x0a \
       document.\
body.appendChild\
(ta); ta.select(\
); const ok = do\
cument.execComma\
nd && document.e\
xecCommand('copy\
'); document.bod\
y.removeChild(ta\
); return !!ok;\x0a\
      } catch (_\
) { return false\
; }\x0a    }\x0a    //\
 Flash \x22Copied\x22 \
feedback on the \
copy button.\x0a   \
 _flashCopied(bt\
n, wrapper) {\x0a  \
    if (!btn || \
!wrapper) return\
;\x0a      const sp\
an = btn.querySe\
lector('span'); \
if (!span) retur\
n;\x0a      const L\
_COPY = wrapper.\
getAttribute('da\
ta-locale-copy')\
 || 'Copy';\x0a    \
  const L_COPIED\
 = wrapper.getAt\
tribute('data-lo\
cale-copied') ||\
 'Copied';\x0a     \
 try { if (btn._\
_copyTimer) { cl\
earTimeout(btn._\
_copyTimer); btn\
.__copyTimer = 0\
; } } catch (_) \
{}\x0a      span.te\
xtContent = L_CO\
PIED; btn.classL\
ist.add('copied'\
);\x0a      btn.__c\
opyTimer = setTi\
meout(() => { tr\
y { span.textCon\
tent = L_COPY; b\
tn.classList.rem\
ove('copied'); }\
 catch (_) {} bt\
n.__copyTimer = \
0; }, 1200);\x0a   \
 }\x0a    // Toggle\
 code collapse/e\
xpand and rememb\
er collapsed ind\
ices.\x0a    _toggl\
eCollapse(wrappe\
r) {\x0a      if (!\
wrapper) return;\
\x0a      const cod\
eEl = this._getC\
odeEl(wrapper); \
if (!codeEl) ret\
urn;\x0a      const\
 btn = wrapper.q\
uerySelector('.c\
ode-header-colla\
pse');\x0a      con\
st span = btn ? \
btn.querySelecto\
r('span') : null\
;\x0a      const L_\
COLLAPSE = wrapp\
er.getAttribute(\
'data-locale-col\
lapse') || 'Coll\
apse';\x0a      con\
st L_EXPAND = wr\
apper.getAttribu\
te('data-locale-\
expand') || 'Exp\
and';\x0a      cons\
t idx = String(w\
rapper.getAttrib\
ute('data-index'\
) || '');\x0a      \
const arr = wind\
ow.__collapsed_i\
dx || (window.__\
collapsed_idx = \
[]);\x0a      const\
 isHidden = (cod\
eEl.style.displa\
y === 'none');\x0a\x0a\
      if (isHidd\
en) {\x0a        co\
deEl.style.displ\
ay = 'block';\x0a  \
      if (span) \
span.textContent\
 = L_COLLAPSE;\x0a \
       const p =\
 arr.indexOf(idx\
); if (p !== -1)\
 arr.splice(p, 1\
);\x0a      } else \
{\x0a        codeEl\
.style.display =\
 'none';\x0a       \
 if (span) span.\
textContent = L_\
EXPAND;\x0a        \
if (!arr.include\
s(idx)) arr.push\
(idx);\x0a      }\x0a \
   }\x0a    // Atta\
ch global UI eve\
nt handlers and \
container-level \
interactions.\x0a  \
  install() {\x0a  \
    try { histor\
y.scrollRestorat\
ion = \x22manual\x22; \
} catch (_) {}\x0a\x0a\
      this.handl\
ers.keydown = (e\
vent) => {\x0a     \
   if (event.ctr\
lKey && event.ke\
y === 'f') { win\
dow.location.hre\
f = 'bridge://op\
en_find:' + runt\
ime.cfg.PID; eve\
nt.preventDefaul\
t(); }\x0a        i\
f (event.key ===\
 'Escape') { win\
dow.location.hre\
f = 'bridge://es\
cape'; event.pre\
ventDefault(); }\
\x0a      };\x0a      \
document.addEven\
tListener('keydo\
wn', this.handle\
rs.keydown, { pa\
ssive: false });\
\x0a\x0a      // Remov\
ed global click-\
to-focus navigat\
ion and visibili\
ty/focus wakeups\
 to keep the pum\
p rAF-only and c\
lick-agnostic.\x0a\x0a\
      const cont\
ainer = this.dom\
.get('container'\
);\x0a      const a\
ddClassToMsg = (\
id, className) =\
> { const el = d\
ocument.getEleme\
ntById('msg-bot-\
' + id); if (el)\
 el.classList.ad\
d(className); };\
\x0a      const rem\
oveClassFromMsg \
= (id, className\
) => { const el \
= document.getEl\
ementById('msg-b\
ot-' + id); if (\
el) el.classList\
.remove(classNam\
e); };\x0a\x0a      th\
is.handlers.mous\
eover = (event) \
=> { if (event.t\
arget.classList.\
contains('action\
-img')) { const \
id = event.targe\
t.getAttribute('\
data-id'); addCl\
assToMsg(id, 'ms\
g-highlight'); }\
 };\x0a      this.h\
andlers.mouseout\
 = (event) => { \
if (event.target\
.classList.conta\
ins('action-img'\
)) { const id = \
event.target.get\
Attribute('data-\
id'); const el =\
 document.getEle\
mentById('msg-bo\
t-' + id); if (e\
l) el.classList.\
remove('msg-high\
light'); } };\x0a  \
    if (containe\
r) {\x0a        con\
tainer.addEventL\
istener('mouseov\
er', this.handle\
rs.mouseover, { \
passive: true })\
;\x0a        contai\
ner.addEventList\
ener('mouseout',\
 this.handlers.m\
ouseout, { passi\
ve: true });\x0a   \
   }\x0a\x0a      this\
.handlers.click \
= async (ev) => \
{\x0a        const \
a = ev.target &&\
 (ev.target.clos\
est ? ev.target.\
closest('a.code-\
header-action') \
: null);\x0a       \
 if (!a) return;\
\x0a        const w\
rapper = this._f\
indWrapper(a);\x0a \
       if (!wrap\
per) return;\x0a\x0a  \
      ev.prevent\
Default();\x0a     \
   ev.stopPropag\
ation();\x0a\x0a      \
  const isCopy =\
 a.classList.con\
tains('code-head\
er-copy');\x0a     \
   const isColla\
pse = a.classLis\
t.contains('code\
-header-collapse\
');\x0a        cons\
t isRun = a.clas\
sList.contains('\
code-header-run'\
);\x0a        const\
 isPreview = a.c\
lassList.contain\
s('code-header-p\
review');\x0a\x0a     \
   let codeEl = \
null, text = '';\
\x0a        if (isC\
opy || isRun || \
isPreview) {\x0a   \
       codeEl = \
this._getCodeEl(\
wrapper);\x0a      \
    text = this.\
_collectCodeText\
(codeEl);\x0a      \
  }\x0a\x0a        try\
 {\x0a          if \
(isCopy) {\x0a     \
       const ok \
= await this._co\
pyTextRobust(tex\
t);\x0a            \
if (ok) this._fl\
ashCopied(a, wra\
pper);\x0a         \
 } else if (isCo\
llapse) {\x0a      \
      this._togg\
leCollapse(wrapp\
er);\x0a          }\
 else if (isRun)\
 {\x0a            i\
f (this.bridge &\
& typeof this.br\
idge.runCode ===\
 'function') thi\
s.bridge.runCode\
(text);\x0a        \
  } else if (isP\
review) {\x0a      \
      if (this.b\
ridge && typeof \
this.bridge.prev\
iewCode === 'fun\
ction') this.bri\
dge.previewCode(\
text);\x0a         \
 }\x0a        } cat\
ch (_) { /* swal\
low */ }\x0a      }\
;\x0a      if (cont\
ainer) container\
.addEventListene\
r('click', this.\
handlers.click, \
{ passive: false\
 });\x0a\x0a      this\
.handlers.wheel \
= (ev) => {\x0a    \
    runtime.scro\
llMgr.userIntera\
cted = true;\x0a   \
     if (ev.delt\
aY < 0) runtime.\
scrollMgr.autoFo\
llow = false;\x0a  \
      else runti\
me.scrollMgr.may\
beEnableAutoFoll\
owByProximity();\
\x0a        this.hi\
ghlighter.schedu\
leScanVisibleCod\
es(runtime.strea\
m.activeCode);\x0a \
     };\x0a      do\
cument.addEventL\
istener('wheel',\
 this.handlers.w\
heel, { passive:\
 true });\x0a\x0a     \
 this.handlers.s\
croll = () => {\x0a\
        const el\
 = Utils.SE; con\
st top = el.scro\
llTop;\x0a        i\
f (top + 1 < run\
time.scrollMgr.l\
astScrollTop) ru\
ntime.scrollMgr.\
autoFollow = fal\
se;\x0a        runt\
ime.scrollMgr.ma\
ybeEnableAutoFol\
lowByProximity()\
;\x0a        runtim\
e.scrollMgr.last\
ScrollTop = top;\
\x0a        const a\
ction = runtime.\
scrollMgr.comput\
eFabAction();\x0a  \
      if (action\
 !== runtime.scr\
ollMgr.currentFa\
bAction) runtime\
.scrollMgr.updat\
eScrollFab(false\
, action, true);\
\x0a        this.hi\
ghlighter.schedu\
leScanVisibleCod\
es(runtime.strea\
m.activeCode);\x0a \
     };\x0a      wi\
ndow.addEventLis\
tener('scroll', \
this.handlers.sc\
roll, { passive:\
 true });\x0a\x0a     \
 const fab = thi\
s.dom.get('scrol\
lFab');\x0a      if\
 (fab) {\x0a       \
 this.handlers.f\
abClick = (ev) =\
> {\x0a          ev\
.preventDefault(\
); ev.stopPropag\
ation();\x0a       \
   const action \
= runtime.scroll\
Mgr.computeFabAc\
tion();\x0a        \
  if (action ===\
 'up') runtime.s\
crollMgr.scrollT\
oTopUser();\x0a    \
      else if (a\
ction === 'down'\
) runtime.scroll\
Mgr.scrollToBott\
omUser();\x0a      \
    runtime.scro\
llMgr.fabFreezeU\
ntil = Utils.now\
() + this.cfg.FA\
B.TOGGLE_DEBOUNC\
E_MS;\x0a          \
runtime.scrollMg\
r.updateScrollFa\
b(true);\x0a       \
 };\x0a        fab.\
addEventListener\
('click', this.h\
andlers.fabClick\
, { passive: fal\
se });\x0a      }\x0a\x0a\
      this.handl\
ers.resize = () \
=> {\x0a        run\
time.scrollMgr.m\
aybeEnableAutoFo\
llowByProximity(\
);\x0a        runti\
me.scrollMgr.sch\
eduleScrollFabUp\
date();\x0a        \
this.highlighter\
.scheduleScanVis\
ibleCodes(runtim\
e.stream.activeC\
ode);\x0a      };\x0a \
     window.addE\
ventListener('re\
size', this.hand\
lers.resize, { p\
assive: true });\
\x0a\x0a      // Note:\
 visibility/focu\
s/pageshow kicke\
rs removed inten\
tionally.\x0a    }\x0a\
    // Detach al\
l installed hand\
lers and reset l\
ocal refs.\x0a    c\
leanup() {\x0a     \
 const container\
 = this.dom.get(\
'container');\x0a  \
    if (this.han\
dlers.wheel) doc\
ument.removeEven\
tListener('wheel\
', this.handlers\
.wheel);\x0a      i\
f (this.handlers\
.scroll) window.\
removeEventListe\
ner('scroll', th\
is.handlers.scro\
ll);\x0a      if (t\
his.handlers.res\
ize) window.remo\
veEventListener(\
'resize', this.h\
andlers.resize);\
\x0a      const fab\
 = this.dom.get(\
'scrollFab'); if\
 (fab && this.ha\
ndlers.fabClick)\
 fab.removeEvent\
Listener('click'\
, this.handlers.\
fabClick);\x0a     \
 if (container &\
& this.handlers.\
mouseover) conta\
iner.removeEvent\
Listener('mouseo\
ver', this.handl\
ers.mouseover);\x0a\
      if (contai\
ner && this.hand\
lers.mouseout) c\
ontainer.removeE\
ventListener('mo\
useout', this.ha\
ndlers.mouseout)\
;\x0a      if (cont\
ainer && this.ha\
ndlers.click) co\
ntainer.removeEv\
entListener('cli\
ck', this.handle\
rs.click);\x0a     \
 if (this.handle\
rs.keydown) docu\
ment.removeEvent\
Listener('keydow\
n', this.handler\
s.keydown);\x0a    \
  if (this.handl\
ers.docClickFocu\
s) document.remo\
veEventListener(\
'click', this.ha\
ndlers.docClickF\
ocus);\x0a      if \
(this.handlers.v\
isibility) docum\
ent.removeEventL\
istener('visibil\
itychange', this\
.handlers.visibi\
lity);\x0a      if \
(this.handlers.f\
ocus) window.rem\
oveEventListener\
('focus', this.h\
andlers.focus);\x0a\
      if (this.h\
andlers.pageshow\
) window.removeE\
ventListener('pa\
geshow', this.ha\
ndlers.pageshow)\
;\x0a      this.han\
dlers = {};\x0a    \
}\x0a  }\x0a\x0a  // ====\
================\
================\
================\
================\
======\x0a  // 16) \
Orchestrator run\
time\x0a  // ======\
================\
================\
================\
================\
====\x0a\x0a  class Ru\
ntime {\x0a    cons\
tructor() {\x0a    \
  this.cfg = new\
 Config();\x0a     \
 this.logger = n\
ew Logger(this.c\
fg);\x0a\x0a      this\
.dom = new DOMRe\
fs();\x0a      this\
.customMarkup = \
new CustomMarkup\
(this.cfg, this.\
logger);\x0a      t\
his.raf = new Ra\
fManager(this.cf\
g);\x0a\x0a      // En\
sure logger uses\
 central RafMana\
ger for its inte\
rnal tick pump.\x0a\
      try { this\
.logger.bindRaf(\
this.raf); } cat\
ch (_) {}\x0a\x0a     \
 this.async = ne\
w AsyncRunner(th\
is.cfg, this.raf\
);\x0a      this.re\
nderer = new Mar\
kdownRenderer(th\
is.cfg, this.cus\
tomMarkup, this.\
logger, this.asy\
nc, this.raf);\x0a\x0a\
      this.math \
= new MathRender\
er(this.cfg, thi\
s.raf, this.asyn\
c);\x0a      this.c\
odeScroll = new \
CodeScrollState(\
this.cfg, this.r\
af);\x0a      this.\
highlighter = ne\
w Highlighter(th\
is.cfg, this.cod\
eScroll, this.ra\
f);\x0a      this.s\
crollMgr = new S\
crollManager(thi\
s.cfg, this.dom,\
 this.raf);\x0a    \
  this.toolOutpu\
t = new ToolOutp\
ut();\x0a      this\
.loading = new L\
oading(this.dom)\
;\x0a      this.nod\
es = new NodesMa\
nager(this.dom, \
this.renderer, t\
his.highlighter,\
 this.math);\x0a   \
   this.bridge =\
 new BridgeManag\
er(this.cfg, thi\
s.logger);\x0a     \
 this.ui = new U\
IManager();\x0a    \
  this.stream = \
new StreamEngine\
(this.cfg, this.\
dom, this.render\
er, this.math, t\
his.highlighter,\
 this.codeScroll\
, this.scrollMgr\
, this.raf, this\
.async, this.log\
ger);\x0a      this\
.streamQ = new S\
treamQueue(this.\
cfg, this.stream\
, this.scrollMgr\
, this.raf);\x0a   \
   this.events =\
 new EventManage\
r(this.cfg, this\
.dom, this.scrol\
lMgr, this.highl\
ighter, this.cod\
eScroll, this.to\
olOutput, this.b\
ridge);\x0a\x0a      t\
ry {\x0a        thi\
s.stream.setCust\
omFenceSpecs(thi\
s.customMarkup.g\
etSourceFenceSpe\
cs());\x0a      } c\
atch (_) {}\x0a\x0a   \
   this.template\
s = new NodeTemp\
lateEngine(this.\
cfg, this.logger\
);\x0a      this.da\
ta = new DataRec\
eiver(this.cfg, \
this.templates, \
this.nodes, this\
.scrollMgr);\x0a\x0a  \
    this.tips = \
null;\x0a      this\
._lastHeavyReset\
Ms = 0;\x0a\x0a      t\
his.renderer.hoo\
ks.observeNewCod\
e = (root, opts)\
 => this.highlig\
hter.observeNewC\
ode(root, opts, \
this.stream.acti\
veCode);\x0a      t\
his.renderer.hoo\
ks.observeMsgBox\
es = (root) => t\
his.highlighter.\
observeMsgBoxes(\
root, (box) => {\
\x0a        this.hi\
ghlighter.observ\
eNewCode(box, {\x0a\
          deferL\
astIfStreaming: \
true,\x0a          \
minLinesForLast:\
 this.cfg.PROFIL\
E_CODE.minLinesF\
orHL,\x0a          \
minCharsForLast:\
 this.cfg.PROFIL\
E_CODE.minCharsF\
orHL\x0a        }, \
this.stream.acti\
veCode);\x0a       \
 this.codeScroll\
.initScrollableB\
locks(box);\x0a    \
  });\x0a      this\
.renderer.hooks.\
scheduleMathRend\
er = (root) => {\
\x0a        const m\
m = getMathMode(\
);\x0a        if (m\
m === 'idle') th\
is.math.schedule\
(root);\x0a        \
else if (mm === \
'always') this.m\
ath.schedule(roo\
t, 0, true);\x0a   \
   };\x0a      this\
.renderer.hooks.\
codeScrollInit =\
 (root) => this.\
codeScroll.initS\
crollableBlocks(\
root);\x0a    }\x0a\x0a  \
  // Reset strea\
m state and opti\
onally perform a\
 heavy reset of \
schedulers and o\
bservers.\x0a    re\
setStreamState(o\
rigin, opts) {\x0a \
     try { this.\
streamQ.clear();\
 } catch (_) {}\x0a\
\x0a      const def\
 = Object.assign\
({\x0a        final\
izeActive: true,\
 clearBuffer: tr\
ue, clearMsg: fa\
lse, defuseOrpha\
ns: true, forceH\
eavy: false, rea\
son: String(orig\
in || 'external-\
op')\x0a      }, (o\
pts || {}));\x0a\x0a  \
    const now = \
Utils.now();\x0a   \
   const withinD\
ebounce = (now -\
 (this._lastHeav\
yResetMs || 0)) \
<= (this.cfg.RES\
ET.HEAVY_DEBOUNC\
E_MS || 24);\x0a   \
   const mustHea\
vyByOrigin =\x0a   \
     def.forceHe\
avy === true || \
def.clearMsg ===\
 true ||\x0a       \
 origin === 'beg\
inStream' || ori\
gin === 'nextStr\
eam' ||\x0a        \
origin === 'clea\
rStream' || orig\
in === 'replaceN\
odes' ||\x0a       \
 origin === 'cle\
arNodes' || orig\
in === 'clearOut\
put' ||\x0a        \
origin === 'clea\
rLive' || origin\
 === 'clearInput\
';\x0a      const s\
houldHeavy = mus\
tHeavyByOrigin |\
| !withinDebounc\
e;\x0a      const s\
uppressLog = wit\
hinDebounce && o\
rigin !== 'begin\
Stream';\x0a\x0a      \
try { this.strea\
m.abortAndReset(\
{ ...def, suppre\
ssLog }); } catc\
h (_) {}\x0a\x0a      \
if (shouldHeavy)\
 {\x0a        try {\
 this.highlighte\
r.cleanup(); } c\
atch (_) {}\x0a    \
    try { this.m\
ath.cleanup(); }\
 catch (_) {}\x0a  \
      try { this\
.codeScroll.canc\
elAllScrolls(); \
} catch (_) {}\x0a \
       try { thi\
s.scrollMgr.canc\
elPendingScroll(\
); } catch (_) {\
}\x0a        try { \
this.raf.cancelA\
ll(); } catch (_\
) {}\x0a        thi\
s._lastHeavyRese\
tMs = now;\x0a     \
 } else {\x0a      \
  try { this.raf\
.cancelGroup('St\
reamQueue'); } c\
atch (_) {}\x0a    \
  }\x0a\x0a      try {\
 this.tips && th\
is.tips.hide(); \
} catch (_) {}\x0a \
   }\x0a    // API:\
 handle incoming\
 chunk (from bri\
dge).\x0a    api_on\
Chunk = (name, c\
hunk, type) => {\
\x0a      const t =\
 String(type || \
'text_delta');\x0a \
     if (t === '\
text_delta') {\x0a \
       this.api_\
appendStream(nam\
e, chunk);\x0a     \
   return;\x0a     \
 }\x0a      // Futu\
re-proof: add ot\
her chunk types \
here (attachment\
s, status, etc.)\
\x0a      // No-op \
for unknown type\
s to keep curren\
t behavior.\x0a    \
  this.logger.de\
bug('STREAM', 'I\
GNORED_NON_TEXT_\
CHUNK', { type: \
t, len: (chunk ?\
 String(chunk).l\
ength : 0) });\x0a \
   };\x0a    // API\
: begin stream.\x0a\
    api_beginStr\
eam = (chunk = f\
alse) => { this.\
tips && this.tip\
s.hide(); this.r\
esetStreamState(\
'beginStream', {\
 clearMsg: true,\
 finalizeActive:\
 false, forceHea\
vy: true }); thi\
s.stream.beginSt\
ream(chunk); };\x0a\
    // API: end \
stream.\x0a    api_\
endStream = () =\
> { this.stream.\
endStream(); };\x0a\
    // API: appl\
y chunk.\x0a    api\
_applyStream = (\
name, chunk) => \
{ this.stream.ap\
plyStream(name, \
chunk); };\x0a    /\
/ API: enqueue c\
hunk (drained on\
 rAF).\x0a    api_a\
ppendStream = (n\
ame, chunk) => {\
 this.streamQ.en\
queue(name, chun\
k); };\x0a    // AP\
I: move current \
output to \x22befor\
e\x22 area and prep\
are for next str\
eam.\x0a    api_nex\
tStream = () => \
{\x0a      this.tip\
s && this.tips.h\
ide();\x0a      con\
st element = thi\
s.dom.get('_appe\
nd_output_'); co\
nst before = thi\
s.dom.get('_appe\
nd_output_before\
_');\x0a      if (e\
lement && before\
) {\x0a        cons\
t frag = documen\
t.createDocument\
Fragment();\x0a    \
    while (eleme\
nt.firstChild) f\
rag.appendChild(\
element.firstChi\
ld);\x0a        bef\
ore.appendChild(\
frag);\x0a      }\x0a \
     this.resetS\
treamState('next\
Stream', { clear\
Msg: true, final\
izeActive: false\
, forceHeavy: tr\
ue });\x0a      thi\
s.scrollMgr.sche\
duleScroll();\x0a  \
  };\x0a    // API:\
 clear streaming\
 output area ent\
irely.\x0a    api_c\
learStream = () \
=> { this.tips &\
& this.tips.hide\
(); this.resetSt\
reamState('clear\
Stream', { clear\
Msg: true, force\
Heavy: true }); \
const el = this.\
dom.getStreamCon\
tainer(); if (!e\
l) return; el.re\
placeChildren();\
 };\x0a\x0a    // API:\
 append/replace \
messages (non-st\
reaming).\x0a    ap\
i_appendNode = (\
payload) => { th\
is.resetStreamSt\
ate('appendNode'\
); this.data.app\
end(payload); };\
\x0a    api_replace\
Nodes = (payload\
) => { this.rese\
tStreamState('re\
placeNodes', { c\
learMsg: true, f\
orceHeavy: true \
}); this.dom.cle\
arNodes(); this.\
data.replace(pay\
load); };\x0a\x0a    /\
/ API: append to\
 input area.\x0a   \
   api_appendToI\
nput = (payload)\
 => {\x0a        th\
is.nodes.appendT\
oInput(payload);\
\x0a\x0a        // Ens\
ure initial auto\
-follow is ON fo\
r the next strea\
m that will star\
t right after us\
er input.\x0a      \
  // Rationale: \
previously, if t\
he user had scro\
lled up, autoFol\
low could remain\
 false and the\x0a \
       // live s\
tream would not \
follow even thou\
gh we just sent \
a new input.\x0a   \
     this.scroll\
Mgr.autoFollow =\
 true;       // \
explicitly re-en\
able page auto-f\
ollow\x0a        th\
is.scrollMgr.use\
rInteracted = fa\
lse;  // Reset i\
nteraction so li\
ve scroll is all\
owed\x0a\x0a        //\
 Keep lastScroll\
Top in sync to a\
void misclassifi\
cation in the ne\
xt onscroll hand\
ler.\x0a        try\
 { this.scrollMg\
r.lastScrollTop \
= Utils.SE.scrol\
lTop | 0; } catc\
h (_) {}\x0a\x0a      \
  // Non-live sc\
roll to bottom r\
ight away, indep\
endent of autoFo\
llow state.\x0a    \
    this.scrollM\
gr.scheduleScrol\
l();\x0a        // \
NOTE: No resetSt\
reamState() here\
 to avoid flicke\
r/reflow issues \
while previewing\
 user input.\x0a   \
   };\x0a\x0a    // AP\
I: clear message\
s list.\x0a    api_\
clearNodes = () \
=> { this.dom.cl\
earNodes(); this\
.resetStreamStat\
e('clearNodes', \
{ clearMsg: true\
, forceHeavy: tr\
ue }); };\x0a    //\
 API: clear inpu\
t area.\x0a    api_\
clearInput = () \
=> { this.resetS\
treamState('clea\
rInput', { force\
Heavy: true }); \
this.dom.clearIn\
put(); };\x0a    //\
 API: clear outp\
ut area.\x0a    api\
_clearOutput = (\
) => { this.dom.\
clearOutput(); t\
his.resetStreamS\
tate('clearOutpu\
t', { clearMsg: \
true, forceHeavy\
: true }); };\x0a  \
  // API: clear \
live area.\x0a    a\
pi_clearLive = (\
) => { this.dom.\
clearLive(); thi\
s.resetStreamSta\
te('clearLive', \
{ forceHeavy: tr\
ue }); };\x0a\x0a    /\
/ API: tool outp\
ut helpers.\x0a    \
api_appendToolOu\
tput = (c) => th\
is.toolOutput.ap\
pend(c);\x0a    api\
_updateToolOutpu\
t = (c) => this.\
toolOutput.updat\
e(c);\x0a    api_cl\
earToolOutput = \
() => this.toolO\
utput.clear();\x0a \
   api_beginTool\
Output = () => t\
his.toolOutput.b\
egin();\x0a    api_\
endToolOutput = \
() => this.toolO\
utput.end();\x0a   \
 api_enableToolO\
utput = () => th\
is.toolOutput.en\
able();\x0a    api_\
disableToolOutpu\
t = () => this.t\
oolOutput.disabl\
e();\x0a    api_tog\
gleToolOutput = \
(id) => this.too\
lOutput.toggle(i\
d);\x0a\x0a    // API:\
 append extra co\
ntent to a bot m\
essage.\x0a    api_\
appendExtra = (i\
d, c) => this.no\
des.appendExtra(\
id, c, this.scro\
llMgr);\x0a    // A\
PI: remove one m\
essage by id.\x0a  \
  api_removeNode\
 = (id) => this.\
nodes.removeNode\
(id, this.scroll\
Mgr);\x0a    // API\
: remove all mes\
sages starting f\
rom id.\x0a    api_\
removeNodesFromI\
d = (id) => this\
.nodes.removeNod\
esFromId(id, thi\
s.scrollMgr);\x0a\x0a \
   // API: repla\
ce live area con\
tent (with local\
 post-processing\
).\x0a    api_repla\
ceLive = (conten\
t) => {\x0a      co\
nst el = this.do\
m.get('_append_l\
ive_'); if (!el)\
 return;\x0a      i\
f (el.classList.\
contains('hidden\
')) { el.classLi\
st.remove('hidde\
n'); el.classLis\
t.add('visible')\
; }\x0a      el.inn\
erHTML = content\
;\x0a\x0a      try {\x0a \
       const may\
bePromise = this\
.renderer.render\
PendingMarkdown(\
el);\x0a\x0a        co\
nst post = () =>\
 {\x0a          try\
 {\x0a            t\
his.highlighter.\
observeNewCode(e\
l, {\x0a           \
   deferLastIfSt\
reaming: true,\x0a \
             min\
LinesForLast: th\
is.cfg.PROFILE_C\
ODE.minLinesForH\
L,\x0a             \
 minCharsForLast\
: this.cfg.PROFI\
LE_CODE.minChars\
ForHL\x0a          \
  }, this.stream\
.activeCode);\x0a\x0a \
           this.\
highlighter.obse\
rveMsgBoxes(el, \
(box) => {\x0a     \
         this.hi\
ghlighter.observ\
eNewCode(box, {\x0a\
                \
deferLastIfStrea\
ming: true,\x0a    \
            minL\
inesForLast: thi\
s.cfg.PROFILE_CO\
DE.minLinesForHL\
,\x0a              \
  minCharsForLas\
t: this.cfg.PROF\
ILE_CODE.minChar\
sForHL\x0a         \
     }, this.str\
eam.activeCode);\
\x0a              t\
his.codeScroll.i\
nitScrollableBlo\
cks(box);\x0a      \
      });\x0a      \
    } catch (_) \
{}\x0a\x0a          tr\
y {\x0a            \
const mm = getMa\
thMode();\x0a      \
      // In fina\
lize-only we mus\
t force now; oth\
erwise normal sc\
hedule is fine.\x0a\
            if (\
mm === 'finalize\
-only') this.mat\
h.schedule(el, 0\
, true);\x0a       \
     else this.m\
ath.schedule(el)\
;\x0a          } ca\
tch (_) {}\x0a\x0a    \
      this.scrol\
lMgr.scheduleScr\
oll();\x0a        }\
;\x0a\x0a        if (m\
aybePromise && t\
ypeof maybePromi\
se.then === 'fun\
ction') {\x0a      \
    maybePromise\
.then(post);\x0a   \
     } else {\x0a  \
        post();\x0a\
        }\x0a      \
} catch (_) {\x0a  \
      // Worst-c\
ase: keep UX res\
ponsive even if \
something throws\
 before post-pro\
cessing\x0a        \
this.scrollMgr.s\
cheduleScroll();\
\x0a      }\x0a    };\x0a\
\x0a    // API: upd\
ate footer conte\
nt.\x0a    api_upda\
teFooter = (html\
) => { const el \
= this.dom.get('\
_footer_'); if (\
el) el.innerHTML\
 = html; };\x0a\x0a   \
 // API: toggle \
UI features.\x0a   \
 api_enableEditI\
cons = () => thi\
s.ui.enableEditI\
cons();\x0a    api_\
disableEditIcons\
 = () => this.ui\
.disableEditIcon\
s();\x0a    api_ena\
bleTimestamp = (\
) => this.ui.ena\
bleTimestamp();\x0a\
    api_disableT\
imestamp = () =>\
 this.ui.disable\
Timestamp();\x0a   \
 api_enableBlock\
s = () => this.u\
i.enableBlocks()\
;\x0a    api_disabl\
eBlocks = () => \
this.ui.disableB\
locks();\x0a    api\
_updateCSS = (st\
yles) => this.ui\
.updateCSS(style\
s);\x0a\x0a    // API:\
 sync scroll pos\
ition with host.\
\x0a    api_getScro\
llPosition = () \
=> { this.bridge\
.updateScrollPos\
ition(window.scr\
ollY); };\x0a    ap\
i_setScrollPosit\
ion = (pos) => {\
 try { window.sc\
rollTo(0, pos); \
this.scrollMgr.p\
revScroll = pars\
eInt(pos); } cat\
ch (_) {} };\x0a\x0a  \
  // API: show/h\
ide loading over\
lay.\x0a    api_sho\
wLoading = () =>\
 this.loading.sh\
ow();\x0a    api_hi\
deLoading = () =\
> this.loading.h\
ide();\x0a\x0a    // A\
PI: restore coll\
apsed state of c\
odes in a given \
root.\x0a    api_re\
storeCollapsedCo\
de = (root) => t\
his.renderer.res\
toreCollapsedCod\
e(root);\x0a\x0a    //\
 API: user-trigg\
ered page scroll\
.\x0a    api_scroll\
ToTopUser = () =\
> this.scrollMgr\
.scrollToTopUser\
();\x0a    api_scro\
llToBottomUser =\
 () => this.scro\
llMgr.scrollToBo\
ttomUser();\x0a\x0a   \
 // API: tips vi\
sibility control\
.\x0a    api_showTi\
ps = () => this.\
tips.show();\x0a   \
 api_hideTips = \
() => this.tips.\
hide();\x0a\x0a    // \
API: custom mark\
up rules control\
.\x0a    api_getCus\
tomMarkupRules =\
 () => this.cust\
omMarkup.getRule\
s();\x0a    api_set\
CustomMarkupRule\
s = (rules) => {\
\x0a      this.cust\
omMarkup.setRule\
s(rules);\x0a      \
// Keep StreamEn\
gine in sync wit\
h rules producin\
g fenced code\x0a  \
    try { this.s\
tream.setCustomF\
enceSpecs(this.c\
ustomMarkup.getS\
ourceFenceSpecs(\
)); } catch (_) \
{}\x0a    };\x0a\x0a    /\
/ Initialize run\
time (called on \
DOMContentLoaded\
).\x0a    init() {\x0a\
      this.highl\
ighter.initHLJS(\
);\x0a      this.do\
m.init();\x0a      \
this.ui.ensureSt\
ickyHeaderStyle(\
);\x0a\x0a      this.t\
ips = new TipsMa\
nager(this.dom);\
\x0a      this.even\
ts.install();\x0a\x0a \
     this.bridge\
.initQWebChannel\
(this.cfg.PID, (\
bridge) => {\x0a   \
     const onChu\
nk = (name, chun\
k, type) => this\
.api_onChunk(nam\
e, chunk, type);\
\x0a        const o\
nNode = (payload\
) => this.api_ap\
pendNode(payload\
);\x0a        const\
 onNodeReplace =\
 (payload) => th\
is.api_replaceNo\
des(payload);\x0a  \
      const onNo\
deInput = (html)\
 => this.api_app\
endToInput(html)\
;\x0a        this.b\
ridge.connect(on\
Chunk, onNode, o\
nNodeReplace, on\
NodeInput);\x0a    \
    try { this.l\
ogger.bindBridge\
(this.bridge.bri\
dge || this.brid\
ge); } catch (_)\
 {}\x0a      });\x0a\x0a \
     this.render\
er.init();\x0a     \
 try { this.rend\
erer.renderPendi\
ngMarkdown(docum\
ent); } catch (_\
) {}\x0a\x0a      this\
.highlighter.obs\
erveMsgBoxes(doc\
ument, (box) => \
{\x0a        this.h\
ighlighter.obser\
veNewCode(box, {\
\x0a          defer\
LastIfStreaming:\
 true,\x0a         \
 minLinesForLast\
: this.cfg.PROFI\
LE_CODE.minLines\
ForHL,\x0a         \
 minCharsForLast\
: this.cfg.PROFI\
LE_CODE.minChars\
ForHL\x0a        },\
 this.stream.act\
iveCode);\x0a      \
  this.codeScrol\
l.initScrollable\
Blocks(box);\x0a   \
   });\x0a      thi\
s.highlighter.ob\
serveNewCode(doc\
ument, {\x0a       \
 deferLastIfStre\
aming: true,\x0a   \
     minLinesFor\
Last: this.cfg.P\
ROFILE_CODE.minL\
inesForHL,\x0a     \
   minCharsForLa\
st: this.cfg.PRO\
FILE_CODE.minCha\
rsForHL\x0a      },\
 this.stream.act\
iveCode);\x0a      \
this.highlighter\
.scheduleScanVis\
ibleCodes(this.s\
tream.activeCode\
);\x0a\x0a      this.t\
ips.cycle();\x0a   \
   this.scrollMg\
r.updateScrollFa\
b(true);\x0a    }\x0a\x0a\
    // Cleanup r\
untime and detac\
h from DOM/bridg\
e.\x0a    cleanup()\
 {\x0a      this.ti\
ps.cleanup();\x0a  \
    try { this.b\
ridge.disconnect\
(); } catch (_) \
{}\x0a      this.ev\
ents.cleanup();\x0a\
      this.highl\
ighter.cleanup()\
;\x0a      this.mat\
h.cleanup();\x0a   \
   this.streamQ.\
clear();\x0a      t\
his.dom.cleanup(\
);\x0a    }\x0a  }\x0a\x0a  \
// Ensure RafMan\
ager.cancel uses\
 the correct gro\
up key cleanup.\x0a\
    if (typeof R\
afManager !== 'u\
ndefined' && Raf\
Manager.prototyp\
e && typeof RafM\
anager.prototype\
.cancel === 'fun\
ction') {\x0a      \
  RafManager.pro\
totype.cancel = \
function(key) {\x0a\
          const \
t = this.tasks.g\
et(key);\x0a       \
   if (!t) retur\
n;\x0a          thi\
s.tasks.delete(k\
ey);\x0a          i\
f (t.group) {\x0a  \
          const \
set = this.group\
s.get(t.group);\x0a\
            if (\
set) { set.delet\
e(key); if (set.\
size === 0) this\
.groups.delete(t\
.group); }\x0a     \
     }\x0a        }\
;\x0a      }\x0a\x0a  con\
st runtime = new\
 Runtime();\x0a\x0a  d\
ocument.addEvent\
Listener('DOMCon\
tentLoaded', () \
=> runtime.init(\
));\x0a\x0a  Object.de\
fineProperty(win\
dow, 'SE', { get\
() { return Util\
s.SE; } });\x0a\x0a  w\
indow.beginStrea\
m = (chunk) => r\
untime.api_begin\
Stream(chunk);\x0a \
 window.endStrea\
m = () => runtim\
e.api_endStream(\
);\x0a  window.appl\
yStream = (name,\
 chunk) => runti\
me.api_applyStre\
am(name, chunk);\
\x0a  window.append\
Stream = (name, \
chunk) => runtim\
e.api_appendStre\
am(name, chunk);\
\x0a  window.append\
StreamTyped = (t\
ype, name, chunk\
) => runtime.api\
_onChunk(name, c\
hunk, type);\x0a  w\
indow.nextStream\
 = () => runtime\
.api_nextStream(\
);\x0a  window.clea\
rStream = () => \
runtime.api_clea\
rStream();\x0a\x0a  wi\
ndow.appendNode \
= (payload) => r\
untime.api_appen\
dNode(payload);\x0a\
  window.replace\
Nodes = (payload\
) => runtime.api\
_replaceNodes(pa\
yload);\x0a  window\
.appendToInput =\
 (html) => runti\
me.api_appendToI\
nput(html);\x0a\x0a  w\
indow.clearNodes\
 = () => runtime\
.api_clearNodes(\
);\x0a  window.clea\
rInput = () => r\
untime.api_clear\
Input();\x0a  windo\
w.clearOutput = \
() => runtime.ap\
i_clearOutput();\
\x0a  window.clearL\
ive = () => runt\
ime.api_clearLiv\
e();\x0a\x0a  window.a\
ppendToolOutput \
= (c) => runtime\
.api_appendToolO\
utput(c);\x0a  wind\
ow.updateToolOut\
put = (c) => run\
time.api_updateT\
oolOutput(c);\x0a  \
window.clearTool\
Output = () => r\
untime.api_clear\
ToolOutput();\x0a  \
window.beginTool\
Output = () => r\
untime.api_begin\
ToolOutput();\x0a  \
window.endToolOu\
tput = () => run\
time.api_endTool\
Output();\x0a  wind\
ow.enableToolOut\
put = () => runt\
ime.api_enableTo\
olOutput();\x0a  wi\
ndow.disableTool\
Output = () => r\
untime.api_disab\
leToolOutput();\x0a\
  window.toggleT\
oolOutput = (id)\
 => runtime.api_\
toggleToolOutput\
(id);\x0a\x0a  window.\
appendExtra = (i\
d, c) => runtime\
.api_appendExtra\
(id, c);\x0a  windo\
w.removeNode = (\
id) => runtime.a\
pi_removeNode(id\
);\x0a  window.remo\
veNodesFromId = \
(id) => runtime.\
api_removeNodesF\
romId(id);\x0a\x0a  wi\
ndow.replaceLive\
 = (c) => runtim\
e.api_replaceLiv\
e(c);\x0a  window.u\
pdateFooter = (c\
) => runtime.api\
_updateFooter(c)\
;\x0a\x0a  window.enab\
leEditIcons = ()\
 => runtime.api_\
enableEditIcons(\
);\x0a  window.disa\
bleEditIcons = (\
) => runtime.api\
_disableEditIcon\
s();\x0a  window.en\
ableTimestamp = \
() => runtime.ap\
i_enableTimestam\
p();\x0a  window.di\
sableTimestamp =\
 () => runtime.a\
pi_disableTimest\
amp();\x0a  window.\
enableBlocks = (\
) => runtime.api\
_enableBlocks();\
\x0a  window.disabl\
eBlocks = () => \
runtime.api_disa\
bleBlocks();\x0a  w\
indow.updateCSS \
= (s) => runtime\
.api_updateCSS(s\
);\x0a\x0a  window.get\
ScrollPosition =\
 () => runtime.a\
pi_getScrollPosi\
tion();\x0a  window\
.setScrollPositi\
on = (pos) => ru\
ntime.api_setScr\
ollPosition(pos)\
;\x0a\x0a  window.show\
Loading = () => \
runtime.api_show\
Loading();\x0a  win\
dow.hideLoading \
= () => runtime.\
api_hideLoading(\
);\x0a\x0a  window.res\
toreCollapsedCod\
e = (root) => ru\
ntime.api_restor\
eCollapsedCode(r\
oot);\x0a  window.s\
crollToTopUser =\
 () => runtime.a\
pi_scrollToTopUs\
er();\x0a  window.s\
crollToBottomUse\
r = () => runtim\
e.api_scrollToBo\
ttomUser();\x0a\x0a  w\
indow.showTips =\
 () => runtime.a\
pi_showTips();\x0a \
 window.hideTips\
 = () => runtime\
.api_hideTips();\
\x0a\x0a  window.getCu\
stomMarkupRules \
= () => runtime.\
api_getCustomMar\
kupRules();\x0a  wi\
ndow.setCustomMa\
rkupRules = (rul\
es) => runtime.a\
pi_setCustomMark\
upRules(rules);\x0a\
\x0a  window.__pygp\
t_cleanup = () =\
> runtime.cleanu\
p();\x0a\x0a})();\
\x00\x01\xe2\xe2\
/\
*! markdown-it 1\
4.1.0 https://gi\
thub.com/markdow\
n-it/markdown-it\
 @license MIT */\
\x0a!function(t,e){\
\x22object\x22==typeof\
 exports&&\x22undef\
ined\x22!=typeof mo\
dule?module.expo\
rts=e():\x22functio\
n\x22==typeof defin\
e&&define.amd?de\
fine(e):(t=\x22unde\
fined\x22!=typeof g\
lobalThis?global\
This:t||self).ma\
rkdownit=e()}(th\
is,(function(){\x22\
use strict\x22;cons\
t t={};function \
e(r,n){\x22string\x22!\
=typeof n&&(n=e.\
defaultChars);co\
nst s=function(e\
){let r=t[e];if(\
r)return r;r=t[e\
]=[];for(let t=0\
;t<128;t++){cons\
t e=String.fromC\
harCode(t);r.pus\
h(e)}for(let t=0\
;t<e.length;t++)\
{const n=e.charC\
odeAt(t);r[n]=\x22%\
\x22+(\x220\x22+n.toStrin\
g(16).toUpperCas\
e()).slice(-2)}r\
eturn r}(n);retu\
rn r.replace(/(%\
[a-f0-9]{2})+/gi\
,(function(t){le\
t e=\x22\x22;for(let r\
=0,n=t.length;r<\
n;r+=3){const i=\
parseInt(t.slice\
(r+1,r+3),16);if\
(i<128)e+=s[i];e\
lse{if(192==(224\
&i)&&r+3<n){cons\
t n=parseInt(t.s\
lice(r+4,r+6),16\
);if(128==(192&n\
)){const t=i<<6&\
1984|63&n;e+=t<1\
28?\x22\x5cufffd\x5cufffd\
\x22:String.fromCha\
rCode(t),r+=3;co\
ntinue}}if(224==\
(240&i)&&r+6<n){\
const n=parseInt\
(t.slice(r+4,r+6\
),16),s=parseInt\
(t.slice(r+7,r+9\
),16);if(128==(1\
92&n)&&128==(192\
&s)){const t=i<<\
12&61440|n<<6&40\
32|63&s;e+=t<204\
8||t>=55296&&t<=\
57343?\x22\x5cufffd\x5cuf\
ffd\x5cufffd\x22:Strin\
g.fromCharCode(t\
),r+=6;continue}\
}if(240==(248&i)\
&&r+9<n){const n\
=parseInt(t.slic\
e(r+4,r+6),16),s\
=parseInt(t.slic\
e(r+7,r+9),16),o\
=parseInt(t.slic\
e(r+10,r+12),16)\
;if(128==(192&n)\
&&128==(192&s)&&\
128==(192&o)){le\
t t=i<<18&183500\
8|n<<12&258048|s\
<<6&4032|63&o;t<\
65536||t>1114111\
?e+=\x22\x5cufffd\x5cufff\
d\x5cufffd\x5cufffd\x22:(\
t-=65536,e+=Stri\
ng.fromCharCode(\
55296+(t>>10),56\
320+(1023&t))),r\
+=9;continue}}e+\
=\x22\x5cufffd\x22}}retur\
n e}))}e.default\
Chars=\x22;/?:@&=+$\
,#\x22,e.componentC\
hars=\x22\x22;const r=\
{};function n(t,\
e,s){\x22string\x22!=t\
ypeof e&&(s=e,e=\
n.defaultChars),\
void 0===s&&(s=!\
0);const i=funct\
ion(t){let e=r[t\
];if(e)return e;\
e=r[t]=[];for(le\
t t=0;t<128;t++)\
{const r=String.\
fromCharCode(t);\
/^[0-9a-z]$/i.te\
st(r)?e.push(r):\
e.push(\x22%\x22+(\x220\x22+\
t.toString(16).t\
oUpperCase()).sl\
ice(-2))}for(let\
 r=0;r<t.length;\
r++)e[t.charCode\
At(r)]=t[r];retu\
rn e}(e);let o=\x22\
\x22;for(let e=0,r=\
t.length;e<r;e++\
){const n=t.char\
CodeAt(e);if(s&&\
37===n&&e+2<r&&/\
^[0-9a-f]{2}$/i.\
test(t.slice(e+1\
,e+3)))o+=t.slic\
e(e,e+3),e+=2;el\
se if(n<128)o+=i\
[n];else if(n>=5\
5296&&n<=57343){\
if(n>=55296&&n<=\
56319&&e+1<r){co\
nst r=t.charCode\
At(e+1);if(r>=56\
320&&r<=57343){o\
+=encodeURICompo\
nent(t[e]+t[e+1]\
),e++;continue}}\
o+=\x22%EF%BF%BD\x22}e\
lse o+=encodeURI\
Component(t[e])}\
return o}functio\
n s(t){let e=\x22\x22;\
return e+=t.prot\
ocol||\x22\x22,e+=t.sl\
ashes?\x22//\x22:\x22\x22,e+\
=t.auth?t.auth+\x22\
@\x22:\x22\x22,t.hostname\
&&-1!==t.hostnam\
e.indexOf(\x22:\x22)?e\
+=\x22[\x22+t.hostname\
+\x22]\x22:e+=t.hostna\
me||\x22\x22,e+=t.port\
?\x22:\x22+t.port:\x22\x22,e\
+=t.pathname||\x22\x22\
,e+=t.search||\x22\x22\
,e+=t.hash||\x22\x22,e\
}function i(){th\
is.protocol=null\
,this.slashes=nu\
ll,this.auth=nul\
l,this.port=null\
,this.hostname=n\
ull,this.hash=nu\
ll,this.search=n\
ull,this.pathnam\
e=null}n.default\
Chars=\x22;/?:@&=+$\
,-_.!~*'()#\x22,n.c\
omponentChars=\x22-\
_.!~*'()\x22;const \
o=/^([a-z0-9.+-]\
+:)/i,u=/:[0-9]*\
$/,c=/^(\x5c/\x5c/?(?!\
\x5c/)[^\x5c?\x5cs]*)(\x5c?[\
^\x5cs]*)?$/,a=[\x22{\x22\
,\x22}\x22,\x22|\x22,\x22\x5c\x5c\x22,\x22^\
\x22,\x22`\x22].concat([\x22\
<\x22,\x22>\x22,'\x22',\x22`\x22,\x22\
 \x22,\x22\x5cr\x22,\x22\x5cn\x22,\x22\x5ct\
\x22]),l=[\x22'\x22].conc\
at(a),h=[\x22%\x22,\x22/\x22\
,\x22?\x22,\x22;\x22,\x22#\x22].co\
ncat(l),p=[\x22/\x22,\x22\
?\x22,\x22#\x22],f=/^[+a-\
z0-9A-Z_-]{0,63}\
$/,d=/^([+a-z0-9\
A-Z_-]{0,63})(.*\
)$/,_={javascrip\
t:!0,\x22javascript\
:\x22:!0},m={http:!\
0,https:!0,ftp:!\
0,gopher:!0,file\
:!0,\x22http:\x22:!0,\x22\
https:\x22:!0,\x22ftp:\
\x22:!0,\x22gopher:\x22:!\
0,\x22file:\x22:!0};fu\
nction g(t,e){if\
(t&&t instanceof\
 i)return t;cons\
t r=new i;return\
 r.parse(t,e),r}\
i.prototype.pars\
e=function(t,e){\
let r,n,s,i=t;if\
(i=i.trim(),!e&&\
1===t.split(\x22#\x22)\
.length){const t\
=c.exec(i);if(t)\
return this.path\
name=t[1],t[2]&&\
(this.search=t[2\
]),this}let u=o.\
exec(i);if(u&&(u\
=u[0],r=u.toLowe\
rCase(),this.pro\
tocol=u,i=i.subs\
tr(u.length)),(e\
||u||i.match(/^\x5c\
/\x5c/[^@\x5c/]+@[^@\x5c/\
]+/))&&(s=\x22//\x22==\
=i.substr(0,2),!\
s||u&&_[u]||(i=i\
.substr(2),this.\
slashes=!0)),!_[\
u]&&(s||u&&!m[u]\
)){let t,e,r=-1;\
for(let t=0;t<p.\
length;t++)n=i.i\
ndexOf(p[t]),-1!\
==n&&(-1===r||n<\
r)&&(r=n);e=-1==\
=r?i.lastIndexOf\
(\x22@\x22):i.lastInde\
xOf(\x22@\x22,r),-1!==\
e&&(t=i.slice(0,\
e),i=i.slice(e+1\
),this.auth=t),r\
=-1;for(let t=0;\
t<h.length;t++)n\
=i.indexOf(h[t])\
,-1!==n&&(-1===r\
||n<r)&&(r=n);-1\
===r&&(r=i.lengt\
h),\x22:\x22===i[r-1]&\
&r--;const s=i.s\
lice(0,r);i=i.sl\
ice(r),this.pars\
eHost(s),this.ho\
stname=this.host\
name||\x22\x22;const o\
=\x22[\x22===this.host\
name[0]&&\x22]\x22===t\
his.hostname[thi\
s.hostname.lengt\
h-1];if(!o){cons\
t t=this.hostnam\
e.split(/\x5c./);fo\
r(let e=0,r=t.le\
ngth;e<r;e++){co\
nst r=t[e];if(r&\
&!r.match(f)){le\
t n=\x22\x22;for(let t\
=0,e=r.length;t<\
e;t++)r.charCode\
At(t)>127?n+=\x22x\x22\
:n+=r[t];if(!n.m\
atch(f)){const n\
=t.slice(0,e),s=\
t.slice(e+1),o=r\
.match(d);o&&(n.\
push(o[1]),s.uns\
hift(o[2])),s.le\
ngth&&(i=s.join(\
\x22.\x22)+i),this.hos\
tname=n.join(\x22.\x22\
);break}}}}this.\
hostname.length>\
255&&(this.hostn\
ame=\x22\x22),o&&(this\
.hostname=this.h\
ostname.substr(1\
,this.hostname.l\
ength-2))}const \
a=i.indexOf(\x22#\x22)\
;-1!==a&&(this.h\
ash=i.substr(a),\
i=i.slice(0,a));\
const l=i.indexO\
f(\x22?\x22);return-1!\
==l&&(this.searc\
h=i.substr(l),i=\
i.slice(0,l)),i&\
&(this.pathname=\
i),m[r]&&this.ho\
stname&&!this.pa\
thname&&(this.pa\
thname=\x22\x22),this}\
,i.prototype.par\
seHost=function(\
t){let e=u.exec(\
t);e&&(e=e[0],\x22:\
\x22!==e&&(this.por\
t=e.substr(1)),t\
=t.substr(0,t.le\
ngth-e.length)),\
t&&(this.hostnam\
e=t)};var k,D=Ob\
ject.freeze({__p\
roto__:null,deco\
de:e,encode:n,fo\
rmat:s,parse:g})\
,C=/[\x5c0-\x5cuD7FF\x5cu\
E000-\x5cuFFFF]|[\x5cu\
D800-\x5cuDBFF][\x5cuD\
C00-\x5cuDFFF]|[\x5cuD\
800-\x5cuDBFF](?![\x5c\
uDC00-\x5cuDFFF])|(\
?:[^\x5cuD800-\x5cuDBF\
F]|^)[\x5cuDC00-\x5cuD\
FFF]/,y=/[\x5c0-\x5cx1\
F\x5cx7F-\x5cx9F]/,E=/\
[!-#%-\x5c*,-\x5c/:;\x5c?\
@\x5c[-\x5c]_\x5c{\x5c}\x5cxA1\x5c\
xA7\x5cxAB\x5cxB6\x5cxB7\x5c\
xBB\x5cxBF\x5cu037E\x5cu0\
387\x5cu055A-\x5cu055F\
\x5cu0589\x5cu058A\x5cu05\
BE\x5cu05C0\x5cu05C3\x5cu\
05C6\x5cu05F3\x5cu05F4\
\x5cu0609\x5cu060A\x5cu06\
0C\x5cu060D\x5cu061B\x5cu\
061D-\x5cu061F\x5cu066\
A-\x5cu066D\x5cu06D4\x5cu\
0700-\x5cu070D\x5cu07F\
7-\x5cu07F9\x5cu0830-\x5c\
u083E\x5cu085E\x5cu096\
4\x5cu0965\x5cu0970\x5cu0\
9FD\x5cu0A76\x5cu0AF0\x5c\
u0C77\x5cu0C84\x5cu0DF\
4\x5cu0E4F\x5cu0E5A\x5cu0\
E5B\x5cu0F04-\x5cu0F12\
\x5cu0F14\x5cu0F3A-\x5cu0\
F3D\x5cu0F85\x5cu0FD0-\
\x5cu0FD4\x5cu0FD9\x5cu0F\
DA\x5cu104A-\x5cu104F\x5c\
u10FB\x5cu1360-\x5cu13\
68\x5cu1400\x5cu166E\x5cu\
169B\x5cu169C\x5cu16EB\
-\x5cu16ED\x5cu1735\x5cu1\
736\x5cu17D4-\x5cu17D6\
\x5cu17D8-\x5cu17DA\x5cu1\
800-\x5cu180A\x5cu1944\
\x5cu1945\x5cu1A1E\x5cu1A\
1F\x5cu1AA0-\x5cu1AA6\x5c\
u1AA8-\x5cu1AAD\x5cu1B\
5A-\x5cu1B60\x5cu1B7D\x5c\
u1B7E\x5cu1BFC-\x5cu1B\
FF\x5cu1C3B-\x5cu1C3F\x5c\
u1C7E\x5cu1C7F\x5cu1CC\
0-\x5cu1CC7\x5cu1CD3\x5cu\
2010-\x5cu2027\x5cu203\
0-\x5cu2043\x5cu2045-\x5c\
u2051\x5cu2053-\x5cu20\
5E\x5cu207D\x5cu207E\x5cu\
208D\x5cu208E\x5cu2308\
-\x5cu230B\x5cu2329\x5cu2\
32A\x5cu2768-\x5cu2775\
\x5cu27C5\x5cu27C6\x5cu27\
E6-\x5cu27EF\x5cu2983-\
\x5cu2998\x5cu29D8-\x5cu2\
9DB\x5cu29FC\x5cu29FD\x5c\
u2CF9-\x5cu2CFC\x5cu2C\
FE\x5cu2CFF\x5cu2D70\x5cu\
2E00-\x5cu2E2E\x5cu2E3\
0-\x5cu2E4F\x5cu2E52-\x5c\
u2E5D\x5cu3001-\x5cu30\
03\x5cu3008-\x5cu3011\x5c\
u3014-\x5cu301F\x5cu30\
30\x5cu303D\x5cu30A0\x5cu\
30FB\x5cuA4FE\x5cuA4FF\
\x5cuA60D-\x5cuA60F\x5cuA\
673\x5cuA67E\x5cuA6F2-\
\x5cuA6F7\x5cuA874-\x5cuA\
877\x5cuA8CE\x5cuA8CF\x5c\
uA8F8-\x5cuA8FA\x5cuA8\
FC\x5cuA92E\x5cuA92F\x5cu\
A95F\x5cuA9C1-\x5cuA9C\
D\x5cuA9DE\x5cuA9DF\x5cuA\
A5C-\x5cuAA5F\x5cuAADE\
\x5cuAADF\x5cuAAF0\x5cuAA\
F1\x5cuABEB\x5cuFD3E\x5cu\
FD3F\x5cuFE10-\x5cuFE1\
9\x5cuFE30-\x5cuFE52\x5cu\
FE54-\x5cuFE61\x5cuFE6\
3\x5cuFE68\x5cuFE6A\x5cuF\
E6B\x5cuFF01-\x5cuFF03\
\x5cuFF05-\x5cuFF0A\x5cuF\
F0C-\x5cuFF0F\x5cuFF1A\
\x5cuFF1B\x5cuFF1F\x5cuFF\
20\x5cuFF3B-\x5cuFF3D\x5c\
uFF3F\x5cuFF5B\x5cuFF5\
D\x5cuFF5F-\x5cuFF65]|\
\x5cuD800[\x5cuDD00-\x5cu\
DD02\x5cuDF9F\x5cuDFD0\
]|\x5cuD801\x5cuDD6F|\x5c\
uD802[\x5cuDC57\x5cuDD\
1F\x5cuDD3F\x5cuDE50-\x5c\
uDE58\x5cuDE7F\x5cuDEF\
0-\x5cuDEF6\x5cuDF39-\x5c\
uDF3F\x5cuDF99-\x5cuDF\
9C]|\x5cuD803[\x5cuDEA\
D\x5cuDF55-\x5cuDF59\x5cu\
DF86-\x5cuDF89]|\x5cuD\
804[\x5cuDC47-\x5cuDC4\
D\x5cuDCBB\x5cuDCBC\x5cuD\
CBE-\x5cuDCC1\x5cuDD40\
-\x5cuDD43\x5cuDD74\x5cuD\
D75\x5cuDDC5-\x5cuDDC8\
\x5cuDDCD\x5cuDDDB\x5cuDD\
DD-\x5cuDDDF\x5cuDE38-\
\x5cuDE3D\x5cuDEA9]|\x5cu\
D805[\x5cuDC4B-\x5cuDC\
4F\x5cuDC5A\x5cuDC5B\x5cu\
DC5D\x5cuDCC6\x5cuDDC1\
-\x5cuDDD7\x5cuDE41-\x5cu\
DE43\x5cuDE60-\x5cuDE6\
C\x5cuDEB9\x5cuDF3C-\x5cu\
DF3E]|\x5cuD806[\x5cuD\
C3B\x5cuDD44-\x5cuDD46\
\x5cuDDE2\x5cuDE3F-\x5cuD\
E46\x5cuDE9A-\x5cuDE9C\
\x5cuDE9E-\x5cuDEA2\x5cuD\
F00-\x5cuDF09]|\x5cuD8\
07[\x5cuDC41-\x5cuDC45\
\x5cuDC70\x5cuDC71\x5cuDE\
F7\x5cuDEF8\x5cuDF43-\x5c\
uDF4F\x5cuDFFF]|\x5cuD\
809[\x5cuDC70-\x5cuDC7\
4]|\x5cuD80B[\x5cuDFF1\
\x5cuDFF2]|\x5cuD81A[\x5c\
uDE6E\x5cuDE6F\x5cuDEF\
5\x5cuDF37-\x5cuDF3B\x5cu\
DF44]|\x5cuD81B[\x5cuD\
E97-\x5cuDE9A\x5cuDFE2\
]|\x5cuD82F\x5cuDC9F|\x5c\
uD836[\x5cuDE87-\x5cuD\
E8B]|\x5cuD83A[\x5cuDD\
5E\x5cuDD5F]/,A=/[\x5c\
$\x5c+<->\x5c^`\x5c|~\x5cxA2\
-\x5cxA6\x5cxA8\x5cxA9\x5cxA\
C\x5cxAE-\x5cxB1\x5cxB4\x5cx\
B8\x5cxD7\x5cxF7\x5cu02C2\
-\x5cu02C5\x5cu02D2-\x5cu\
02DF\x5cu02E5-\x5cu02E\
B\x5cu02ED\x5cu02EF-\x5cu\
02FF\x5cu0375\x5cu0384\
\x5cu0385\x5cu03F6\x5cu04\
82\x5cu058D-\x5cu058F\x5c\
u0606-\x5cu0608\x5cu06\
0B\x5cu060E\x5cu060F\x5cu\
06DE\x5cu06E9\x5cu06FD\
\x5cu06FE\x5cu07F6\x5cu07\
FE\x5cu07FF\x5cu0888\x5cu\
09F2\x5cu09F3\x5cu09FA\
\x5cu09FB\x5cu0AF1\x5cu0B\
70\x5cu0BF3-\x5cu0BFA\x5c\
u0C7F\x5cu0D4F\x5cu0D7\
9\x5cu0E3F\x5cu0F01-\x5cu\
0F03\x5cu0F13\x5cu0F15\
-\x5cu0F17\x5cu0F1A-\x5cu\
0F1F\x5cu0F34\x5cu0F36\
\x5cu0F38\x5cu0FBE-\x5cu0\
FC5\x5cu0FC7-\x5cu0FCC\
\x5cu0FCE\x5cu0FCF\x5cu0F\
D5-\x5cu0FD8\x5cu109E\x5c\
u109F\x5cu1390-\x5cu13\
99\x5cu166D\x5cu17DB\x5cu\
1940\x5cu19DE-\x5cu19F\
F\x5cu1B61-\x5cu1B6A\x5cu\
1B74-\x5cu1B7C\x5cu1FB\
D\x5cu1FBF-\x5cu1FC1\x5cu\
1FCD-\x5cu1FCF\x5cu1FD\
D-\x5cu1FDF\x5cu1FED-\x5c\
u1FEF\x5cu1FFD\x5cu1FF\
E\x5cu2044\x5cu2052\x5cu2\
07A-\x5cu207C\x5cu208A\
-\x5cu208C\x5cu20A0-\x5cu\
20C0\x5cu2100\x5cu2101\
\x5cu2103-\x5cu2106\x5cu2\
108\x5cu2109\x5cu2114\x5c\
u2116-\x5cu2118\x5cu21\
1E-\x5cu2123\x5cu2125\x5c\
u2127\x5cu2129\x5cu212\
E\x5cu213A\x5cu213B\x5cu2\
140-\x5cu2144\x5cu214A\
-\x5cu214D\x5cu214F\x5cu2\
18A\x5cu218B\x5cu2190-\
\x5cu2307\x5cu230C-\x5cu2\
328\x5cu232B-\x5cu2426\
\x5cu2440-\x5cu244A\x5cu2\
49C-\x5cu24E9\x5cu2500\
-\x5cu2767\x5cu2794-\x5cu\
27C4\x5cu27C7-\x5cu27E\
5\x5cu27F0-\x5cu2982\x5cu\
2999-\x5cu29D7\x5cu29D\
C-\x5cu29FB\x5cu29FE-\x5c\
u2B73\x5cu2B76-\x5cu2B\
95\x5cu2B97-\x5cu2BFF\x5c\
u2CE5-\x5cu2CEA\x5cu2E\
50\x5cu2E51\x5cu2E80-\x5c\
u2E99\x5cu2E9B-\x5cu2E\
F3\x5cu2F00-\x5cu2FD5\x5c\
u2FF0-\x5cu2FFF\x5cu30\
04\x5cu3012\x5cu3013\x5cu\
3020\x5cu3036\x5cu3037\
\x5cu303E\x5cu303F\x5cu30\
9B\x5cu309C\x5cu3190\x5cu\
3191\x5cu3196-\x5cu319\
F\x5cu31C0-\x5cu31E3\x5cu\
31EF\x5cu3200-\x5cu321\
E\x5cu322A-\x5cu3247\x5cu\
3250\x5cu3260-\x5cu327\
F\x5cu328A-\x5cu32B0\x5cu\
32C0-\x5cu33FF\x5cu4DC\
0-\x5cu4DFF\x5cuA490-\x5c\
uA4C6\x5cuA700-\x5cuA7\
16\x5cuA720\x5cuA721\x5cu\
A789\x5cuA78A\x5cuA828\
-\x5cuA82B\x5cuA836-\x5cu\
A839\x5cuAA77-\x5cuAA7\
9\x5cuAB5B\x5cuAB6A\x5cuA\
B6B\x5cuFB29\x5cuFBB2-\
\x5cuFBC2\x5cuFD40-\x5cuF\
D4F\x5cuFDCF\x5cuFDFC-\
\x5cuFDFF\x5cuFE62\x5cuFE\
64-\x5cuFE66\x5cuFE69\x5c\
uFF04\x5cuFF0B\x5cuFF1\
C-\x5cuFF1E\x5cuFF3E\x5cu\
FF40\x5cuFF5C\x5cuFF5E\
\x5cuFFE0-\x5cuFFE6\x5cuF\
FE8-\x5cuFFEE\x5cuFFFC\
\x5cuFFFD]|\x5cuD800[\x5c\
uDD37-\x5cuDD3F\x5cuDD\
79-\x5cuDD89\x5cuDD8C-\
\x5cuDD8E\x5cuDD90-\x5cuD\
D9C\x5cuDDA0\x5cuDDD0-\
\x5cuDDFC]|\x5cuD802[\x5c\
uDC77\x5cuDC78\x5cuDEC\
8]|\x5cuD805\x5cuDF3F|\
\x5cuD807[\x5cuDFD5-\x5cu\
DFF1]|\x5cuD81A[\x5cuD\
F3C-\x5cuDF3F\x5cuDF45\
]|\x5cuD82F\x5cuDC9C|\x5c\
uD833[\x5cuDF50-\x5cuD\
FC3]|\x5cuD834[\x5cuDC\
00-\x5cuDCF5\x5cuDD00-\
\x5cuDD26\x5cuDD29-\x5cuD\
D64\x5cuDD6A-\x5cuDD6C\
\x5cuDD83\x5cuDD84\x5cuDD\
8C-\x5cuDDA9\x5cuDDAE-\
\x5cuDDEA\x5cuDE00-\x5cuD\
E41\x5cuDE45\x5cuDF00-\
\x5cuDF56]|\x5cuD835[\x5c\
uDEC1\x5cuDEDB\x5cuDEF\
B\x5cuDF15\x5cuDF35\x5cuD\
F4F\x5cuDF6F\x5cuDF89\x5c\
uDFA9\x5cuDFC3]|\x5cuD\
836[\x5cuDC00-\x5cuDDF\
F\x5cuDE37-\x5cuDE3A\x5cu\
DE6D-\x5cuDE74\x5cuDE7\
6-\x5cuDE83\x5cuDE85\x5cu\
DE86]|\x5cuD838[\x5cuD\
D4F\x5cuDEFF]|\x5cuD83\
B[\x5cuDCAC\x5cuDCB0\x5cu\
DD2E\x5cuDEF0\x5cuDEF1\
]|\x5cuD83C[\x5cuDC00-\
\x5cuDC2B\x5cuDC30-\x5cuD\
C93\x5cuDCA0-\x5cuDCAE\
\x5cuDCB1-\x5cuDCBF\x5cuD\
CC1-\x5cuDCCF\x5cuDCD1\
-\x5cuDCF5\x5cuDD0D-\x5cu\
DDAD\x5cuDDE6-\x5cuDE0\
2\x5cuDE10-\x5cuDE3B\x5cu\
DE40-\x5cuDE48\x5cuDE5\
0\x5cuDE51\x5cuDE60-\x5cu\
DE65\x5cuDF00-\x5cuDFF\
F]|\x5cuD83D[\x5cuDC00\
-\x5cuDED7\x5cuDEDC-\x5cu\
DEEC\x5cuDEF0-\x5cuDEF\
C\x5cuDF00-\x5cuDF76\x5cu\
DF7B-\x5cuDFD9\x5cuDFE\
0-\x5cuDFEB\x5cuDFF0]|\
\x5cuD83E[\x5cuDC00-\x5cu\
DC0B\x5cuDC10-\x5cuDC4\
7\x5cuDC50-\x5cuDC59\x5cu\
DC60-\x5cuDC87\x5cuDC9\
0-\x5cuDCAD\x5cuDCB0\x5cu\
DCB1\x5cuDD00-\x5cuDE5\
3\x5cuDE60-\x5cuDE6D\x5cu\
DE70-\x5cuDE7C\x5cuDE8\
0-\x5cuDE88\x5cuDE90-\x5c\
uDEBD\x5cuDEBF-\x5cuDE\
C5\x5cuDECE-\x5cuDEDB\x5c\
uDEE0-\x5cuDEE8\x5cuDE\
F0-\x5cuDEF8\x5cuDF00-\
\x5cuDF92\x5cuDF94-\x5cuD\
FCA]/,b=/[ \x5cxA0\x5c\
u1680\x5cu2000-\x5cu20\
0A\x5cu2028\x5cu2029\x5cu\
202F\x5cu205F\x5cu3000\
]/,F=Object.free\
ze({__proto__:nu\
ll,Any:C,Cc:y,Cf\
:/[\x5cxAD\x5cu0600-\x5cu\
0605\x5cu061C\x5cu06DD\
\x5cu070F\x5cu0890\x5cu08\
91\x5cu08E2\x5cu180E\x5cu\
200B-\x5cu200F\x5cu202\
A-\x5cu202E\x5cu2060-\x5c\
u2064\x5cu2066-\x5cu20\
6F\x5cuFEFF\x5cuFFF9-\x5c\
uFFFB]|\x5cuD804[\x5cu\
DCBD\x5cuDCCD]|\x5cuD8\
0D[\x5cuDC30-\x5cuDC3F\
]|\x5cuD82F[\x5cuDCA0-\
\x5cuDCA3]|\x5cuD834[\x5c\
uDD73-\x5cuDD7A]|\x5cu\
DB40[\x5cuDC01\x5cuDC2\
0-\x5cuDC7F]/,P:E,S\
:A,Z:b}),x=new U\
int16Array('\x5cu1d\
41<\x5cxd5\x5cu0131\x5cu0\
28a\x5cu049d\x5cu057b\x5c\
u05d0\x5cu0675\x5cu06d\
e\x5cu07a2\x5cu07d6\x5cu0\
80f\x5cu0a4a\x5cu0a91\x5c\
u0da1\x5cu0e6d\x5cu0f0\
9\x5cu0f26\x5cu10ca\x5cu1\
228\x5cu12e1\x5cu1415\x5c\
u149d\x5cu14c3\x5cu14d\
f\x5cu1525\x5c0\x5c0\x5c0\x5c0\x5c\
0\x5c0\x5cu156b\x5cu16cd\x5c\
u198d\x5cu1c12\x5cu1dd\
d\x5cu1f7e\x5cu2060\x5cu2\
1b0\x5cu228d\x5cu23c0\x5c\
u23fb\x5cu2442\x5cu282\
4\x5cu2912\x5cu2d08\x5cu2\
e48\x5cu2fce\x5cu3016\x5c\
u32ba\x5cu3639\x5cu37a\
c\x5cu38fe\x5cu3a28\x5cu3\
a71\x5cu3ae0\x5cu3b2e\x5c\
u0800EMabcfglmno\
prstu\x5c\x5cbfms\x5cx7f\x5c\
x84\x5cx8b\x5cx90\x5cx95\x5c\
x98\x5cxa6\x5cxb3\x5cxb9\x5c\
xc8\x5cxcflig\x5cu803b\
\x5cxc6\x5cu40c6P\x5cu803\
b&\x5cu4026cute\x5cu80\
3b\x5cxc1\x5cu40c1reve\
;\x5cu4102\x5cu0100iyx\
}rc\x5cu803b\x5cxc2\x5cu4\
0c2;\x5cu4410r;\x5cuc0\
00\x5cud835\x5cudd04ra\
ve\x5cu803b\x5cxc0\x5cu40\
c0pha;\x5cu4391acr;\
\x5cu4100d;\x5cu6a53\x5cu\
0100gp\x5cx9d\x5cxa1on\
;\x5cu4104f;\x5cuc000\x5c\
ud835\x5cudd38plyFu\
nction;\x5cu6061ing\
\x5cu803b\x5cxc5\x5cu40c5\
\x5cu0100cs\x5cxbe\x5cxc3\
r;\x5cuc000\x5cud835\x5cu\
dc9cign;\x5cu6254il\
de\x5cu803b\x5cxc3\x5cu40\
c3ml\x5cu803b\x5cxc4\x5cu\
40c4\x5cu0400acefor\
su\x5cxe5\x5cxfb\x5cxfe\x5cu\
0117\x5cu011c\x5cu0122\
\x5cu0127\x5cu012a\x5cu01\
00cr\x5cxea\x5cxf2ksla\
sh;\x5cu6216\x5cu0176\x5c\
xf6\x5cxf8;\x5cu6ae7ed\
;\x5cu6306y;\x5cu4411\x5c\
u0180crt\x5cu0105\x5cu\
010b\x5cu0114ause;\x5c\
u6235noullis;\x5cu6\
12ca;\x5cu4392r;\x5cuc\
000\x5cud835\x5cudd05p\
f;\x5cuc000\x5cud835\x5cu\
dd39eve;\x5cu42d8c\x5c\
xf2\x5cu0113mpeq;\x5cu\
624e\x5cu0700HOacde\
fhilorsu\x5cu014d\x5cu\
0151\x5cu0156\x5cu0180\
\x5cu019e\x5cu01a2\x5cu01\
b5\x5cu01b7\x5cu01ba\x5cu\
01dc\x5cu0215\x5cu0273\
\x5cu0278\x5cu027ecy;\x5c\
u4427PY\x5cu803b\x5cxa\
9\x5cu40a9\x5cu0180cpy\
\x5cu015d\x5cu0162\x5cu01\
7aute;\x5cu4106\x5cu01\
00;i\x5cu0167\x5cu0168\
\x5cu62d2talDiffere\
ntialD;\x5cu6145ley\
s;\x5cu612d\x5cu0200ae\
io\x5cu0189\x5cu018e\x5cu\
0194\x5cu0198ron;\x5cu\
410cdil\x5cu803b\x5cxc\
7\x5cu40c7rc;\x5cu4108\
nint;\x5cu6230ot;\x5cu\
410a\x5cu0100dn\x5cu01\
a7\x5cu01adilla;\x5cu4\
0b8terDot;\x5cu40b7\
\x5cxf2\x5cu017fi;\x5cu43\
a7rcle\x5cu0200DMPT\
\x5cu01c7\x5cu01cb\x5cu01\
d1\x5cu01d6ot;\x5cu629\
9inus;\x5cu6296lus;\
\x5cu6295imes;\x5cu629\
7o\x5cu0100cs\x5cu01e2\
\x5cu01f8kwiseConto\
urIntegral;\x5cu623\
2eCurly\x5cu0100DQ\x5c\
u0203\x5cu020fouble\
Quote;\x5cu601duote\
;\x5cu6019\x5cu0200lnp\
u\x5cu021e\x5cu0228\x5cu0\
247\x5cu0255on\x5cu010\
0;e\x5cu0225\x5cu0226\x5c\
u6237;\x5cu6a74\x5cu01\
80git\x5cu022f\x5cu023\
6\x5cu023aruent;\x5cu6\
261nt;\x5cu622fourI\
ntegral;\x5cu622e\x5cu\
0100fr\x5cu024c\x5cu02\
4e;\x5cu6102oduct;\x5c\
u6210nterClockwi\
seContourIntegra\
l;\x5cu6233oss;\x5cu6a\
2fcr;\x5cuc000\x5cud83\
5\x5cudc9ep\x5cu0100;C\
\x5cu0284\x5cu0285\x5cu62\
d3ap;\x5cu624d\x5cu058\
0DJSZacefios\x5cu02\
a0\x5cu02ac\x5cu02b0\x5cu\
02b4\x5cu02b8\x5cu02cb\
\x5cu02d7\x5cu02e1\x5cu02\
e6\x5cu0333\x5cu048d\x5cu\
0100;o\x5cu0179\x5cu02\
a5trahd;\x5cu6911cy\
;\x5cu4402cy;\x5cu4405\
cy;\x5cu440f\x5cu0180g\
rs\x5cu02bf\x5cu02c4\x5cu\
02c7ger;\x5cu6021r;\
\x5cu61a1hv;\x5cu6ae4\x5c\
u0100ay\x5cu02d0\x5cu0\
2d5ron;\x5cu410e;\x5cu\
4414l\x5cu0100;t\x5cu0\
2dd\x5cu02de\x5cu6207a\
;\x5cu4394r;\x5cuc000\x5c\
ud835\x5cudd07\x5cu010\
0af\x5cu02eb\x5cu0327\x5c\
u0100cm\x5cu02f0\x5cu0\
322ritical\x5cu0200\
ADGT\x5cu0300\x5cu0306\
\x5cu0316\x5cu031ccute\
;\x5cu40b4o\x5cu0174\x5cu\
030b\x5cu030d;\x5cu42d\
9bleAcute;\x5cu42dd\
rave;\x5cu4060ilde;\
\x5cu42dcond;\x5cu62c4\
ferentialD;\x5cu614\
6\x5cu0470\x5cu033d\x5c0\x5c\
0\x5c0\x5cu0342\x5cu0354\x5c\
0\x5cu0405f;\x5cuc000\x5c\
ud835\x5cudd3b\x5cu018\
0;DE\x5cu0348\x5cu0349\
\x5cu034d\x5cu40a8ot;\x5c\
u60dcqual;\x5cu6250\
ble\x5cu0300CDLRUV\x5c\
u0363\x5cu0372\x5cu038\
2\x5cu03cf\x5cu03e2\x5cu0\
3f8ontourIntegra\
\x5cxec\x5cu0239o\x5cu027\
4\x5cu0379\x5c0\x5c0\x5cu037\
b\x5cxbb\x5cu0349nArro\
w;\x5cu61d3\x5cu0100eo\
\x5cu0387\x5cu03a4ft\x5cu\
0180ART\x5cu0390\x5cu0\
396\x5cu03a1rrow;\x5cu\
61d0ightArrow;\x5cu\
61d4e\x5cxe5\x5cu02can\
g\x5cu0100LR\x5cu03ab\x5c\
u03c4eft\x5cu0100AR\
\x5cu03b3\x5cu03b9rrow\
;\x5cu67f8ightArrow\
;\x5cu67faightArrow\
;\x5cu67f9ight\x5cu010\
0AT\x5cu03d8\x5cu03der\
row;\x5cu61d2ee;\x5cu6\
2a8p\x5cu0241\x5cu03e9\
\x5c0\x5c0\x5cu03efrrow;\x5c\
u61d1ownArrow;\x5cu\
61d5erticalBar;\x5c\
u6225n\x5cu0300ABLR\
Ta\x5cu0412\x5cu042a\x5cu\
0430\x5cu045e\x5cu047f\
\x5cu037crrow\x5cu0180\
;BU\x5cu041d\x5cu041e\x5c\
u0422\x5cu6193ar;\x5cu\
6913pArrow;\x5cu61f\
5reve;\x5cu4311eft\x5c\
u02d2\x5cu043a\x5c0\x5cu0\
446\x5c0\x5cu0450ightV\
ector;\x5cu6950eeVe\
ctor;\x5cu695eector\
\x5cu0100;B\x5cu0459\x5cu\
045a\x5cu61bdar;\x5cu6\
956ight\x5cu01d4\x5cu0\
467\x5c0\x5cu0471eeVec\
tor;\x5cu695fector\x5c\
u0100;B\x5cu047a\x5cu0\
47b\x5cu61c1ar;\x5cu69\
57ee\x5cu0100;A\x5cu04\
86\x5cu0487\x5cu62a4rr\
ow;\x5cu61a7\x5cu0100c\
t\x5cu0492\x5cu0497r;\x5c\
uc000\x5cud835\x5cudc9\
frok;\x5cu4110\x5cu080\
0NTacdfglmopqstu\
x\x5cu04bd\x5cu04c0\x5cu0\
4c4\x5cu04cb\x5cu04de\x5c\
u04e2\x5cu04e7\x5cu04e\
e\x5cu04f5\x5cu0521\x5cu0\
52f\x5cu0536\x5cu0552\x5c\
u055d\x5cu0560\x5cu056\
5G;\x5cu414aH\x5cu803b\
\x5cxd0\x5cu40d0cute\x5cu\
803b\x5cxc9\x5cu40c9\x5cu\
0180aiy\x5cu04d2\x5cu0\
4d7\x5cu04dcron;\x5cu4\
11arc\x5cu803b\x5cxca\x5c\
u40ca;\x5cu442dot;\x5c\
u4116r;\x5cuc000\x5cud\
835\x5cudd08rave\x5cu8\
03b\x5cxc8\x5cu40c8eme\
nt;\x5cu6208\x5cu0100a\
p\x5cu04fa\x5cu04fecr;\
\x5cu4112ty\x5cu0253\x5cu\
0506\x5c0\x5c0\x5cu0512ma\
llSquare;\x5cu65fbe\
rySmallSquare;\x5cu\
65ab\x5cu0100gp\x5cu05\
26\x5cu052aon;\x5cu411\
8f;\x5cuc000\x5cud835\x5c\
udd3csilon;\x5cu439\
5u\x5cu0100ai\x5cu053c\
\x5cu0549l\x5cu0100;T\x5c\
u0542\x5cu0543\x5cu6a7\
5ilde;\x5cu6242libr\
ium;\x5cu61cc\x5cu0100\
ci\x5cu0557\x5cu055ar;\
\x5cu6130m;\x5cu6a73a;\
\x5cu4397ml\x5cu803b\x5cx\
cb\x5cu40cb\x5cu0100ip\
\x5cu056a\x5cu056fsts;\
\x5cu6203onentialE;\
\x5cu6147\x5cu0280cfio\
s\x5cu0585\x5cu0588\x5cu0\
58d\x5cu05b2\x5cu05ccy\
;\x5cu4424r;\x5cuc000\x5c\
ud835\x5cudd09lled\x5c\
u0253\x5cu0597\x5c0\x5c0\x5c\
u05a3mallSquare;\
\x5cu65fcerySmallSq\
uare;\x5cu65aa\x5cu037\
0\x5cu05ba\x5c0\x5cu05bf\x5c\
0\x5c0\x5cu05c4f;\x5cuc00\
0\x5cud835\x5cudd3dAll\
;\x5cu6200riertrf;\x5c\
u6131c\x5cxf2\x5cu05cb\
\x5cu0600JTabcdfgor\
st\x5cu05e8\x5cu05ec\x5cu\
05ef\x5cu05fa\x5cu0600\
\x5cu0612\x5cu0616\x5cu06\
1b\x5cu061d\x5cu0623\x5cu\
066c\x5cu0672cy;\x5cu4\
403\x5cu803b>\x5cu403e\
mma\x5cu0100;d\x5cu05f\
7\x5cu05f8\x5cu4393;\x5cu\
43dcreve;\x5cu411e\x5c\
u0180eiy\x5cu0607\x5cu\
060c\x5cu0610dil;\x5cu\
4122rc;\x5cu411c;\x5cu\
4413ot;\x5cu4120r;\x5c\
uc000\x5cud835\x5cudd0\
a;\x5cu62d9pf;\x5cuc00\
0\x5cud835\x5cudd3eeat\
er\x5cu0300EFGLST\x5cu\
0635\x5cu0644\x5cu064e\
\x5cu0656\x5cu065b\x5cu06\
66qual\x5cu0100;L\x5cu\
063e\x5cu063f\x5cu6265\
ess;\x5cu62dbullEqu\
al;\x5cu6267reater;\
\x5cu6aa2ess;\x5cu6277\
lantEqual;\x5cu6a7e\
ilde;\x5cu6273cr;\x5cu\
c000\x5cud835\x5cudca2\
;\x5cu626b\x5cu0400Aac\
fiosu\x5cu0685\x5cu068\
b\x5cu0696\x5cu069b\x5cu0\
69e\x5cu06aa\x5cu06be\x5c\
u06caRDcy;\x5cu442a\
\x5cu0100ct\x5cu0690\x5cu\
0694ek;\x5cu42c7;\x5cu\
405eirc;\x5cu4124r;\
\x5cu610clbertSpace\
;\x5cu610b\x5cu01f0\x5cu0\
6af\x5c0\x5cu06b2f;\x5cu6\
10dizontalLine;\x5c\
u6500\x5cu0100ct\x5cu0\
6c3\x5cu06c5\x5cxf2\x5cu0\
6a9rok;\x5cu4126mp\x5c\
u0144\x5cu06d0\x5cu06d\
8ownHum\x5cxf0\x5cu012\
fqual;\x5cu624f\x5cu07\
00EJOacdfgmnostu\
\x5cu06fa\x5cu06fe\x5cu07\
03\x5cu0707\x5cu070e\x5cu\
071a\x5cu071e\x5cu0721\
\x5cu0728\x5cu0744\x5cu07\
78\x5cu078b\x5cu078f\x5cu\
0795cy;\x5cu4415lig\
;\x5cu4132cy;\x5cu4401\
cute\x5cu803b\x5cxcd\x5cu\
40cd\x5cu0100iy\x5cu07\
13\x5cu0718rc\x5cu803b\
\x5cxce\x5cu40ce;\x5cu441\
8ot;\x5cu4130r;\x5cu61\
11rave\x5cu803b\x5cxcc\
\x5cu40cc\x5cu0180;ap\x5c\
u0720\x5cu072f\x5cu073\
f\x5cu0100cg\x5cu0734\x5c\
u0737r;\x5cu412aina\
ryI;\x5cu6148lie\x5cxf\
3\x5cu03dd\x5cu01f4\x5cu0\
749\x5c0\x5cu0762\x5cu010\
0;e\x5cu074d\x5cu074e\x5c\
u622c\x5cu0100gr\x5cu0\
753\x5cu0758ral;\x5cu6\
22bsection;\x5cu62c\
2isible\x5cu0100CT\x5c\
u076c\x5cu0772omma;\
\x5cu6063imes;\x5cu606\
2\x5cu0180gpt\x5cu077f\
\x5cu0783\x5cu0788on;\x5c\
u412ef;\x5cuc000\x5cud\
835\x5cudd40a;\x5cu439\
9cr;\x5cu6110ilde;\x5c\
u4128\x5cu01eb\x5cu079\
a\x5c0\x5cu079ecy;\x5cu44\
06l\x5cu803b\x5cxcf\x5cu4\
0cf\x5cu0280cfosu\x5cu\
07ac\x5cu07b7\x5cu07bc\
\x5cu07c2\x5cu07d0\x5cu01\
00iy\x5cu07b1\x5cu07b5\
rc;\x5cu4134;\x5cu4419\
r;\x5cuc000\x5cud835\x5cu\
dd0dpf;\x5cuc000\x5cud\
835\x5cudd41\x5cu01e3\x5c\
u07c7\x5c0\x5cu07ccr;\x5c\
uc000\x5cud835\x5cudca\
5rcy;\x5cu4408kcy;\x5c\
u4404\x5cu0380HJacf\
os\x5cu07e4\x5cu07e8\x5cu\
07ec\x5cu07f1\x5cu07fd\
\x5cu0802\x5cu0808cy;\x5c\
u4425cy;\x5cu440cpp\
a;\x5cu439a\x5cu0100ey\
\x5cu07f6\x5cu07fbdil;\
\x5cu4136;\x5cu441ar;\x5c\
uc000\x5cud835\x5cudd0\
epf;\x5cuc000\x5cud835\
\x5cudd42cr;\x5cuc000\x5c\
ud835\x5cudca6\x5cu058\
0JTaceflmost\x5cu08\
25\x5cu0829\x5cu082c\x5cu\
0850\x5cu0863\x5cu09b3\
\x5cu09b8\x5cu09c7\x5cu09\
cd\x5cu0a37\x5cu0a47cy\
;\x5cu4409\x5cu803b<\x5cu\
403c\x5cu0280cmnpr\x5c\
u0837\x5cu083c\x5cu084\
1\x5cu0844\x5cu084dute\
;\x5cu4139bda;\x5cu439\
bg;\x5cu67ealacetrf\
;\x5cu6112r;\x5cu619e\x5c\
u0180aey\x5cu0857\x5cu\
085c\x5cu0861ron;\x5cu\
413ddil;\x5cu413b;\x5c\
u441b\x5cu0100fs\x5cu0\
868\x5cu0970t\x5cu0500\
ACDFRTUVar\x5cu087e\
\x5cu08a9\x5cu08b1\x5cu08\
e0\x5cu08e6\x5cu08fc\x5cu\
092f\x5cu095b\x5cu0390\
\x5cu096a\x5cu0100nr\x5cu\
0883\x5cu088fgleBra\
cket;\x5cu67e8row\x5cu\
0180;BR\x5cu0899\x5cu0\
89a\x5cu089e\x5cu6190a\
r;\x5cu61e4ightArro\
w;\x5cu61c6eiling;\x5c\
u6308o\x5cu01f5\x5cu08\
b7\x5c0\x5cu08c3bleBra\
cket;\x5cu67e6n\x5cu01\
d4\x5cu08c8\x5c0\x5cu08d2\
eeVector;\x5cu6961e\
ctor\x5cu0100;B\x5cu08\
db\x5cu08dc\x5cu61c3ar\
;\x5cu6959loor;\x5cu63\
0aight\x5cu0100AV\x5cu\
08ef\x5cu08f5rrow;\x5c\
u6194ector;\x5cu694\
e\x5cu0100er\x5cu0901\x5c\
u0917e\x5cu0180;AV\x5c\
u0909\x5cu090a\x5cu091\
0\x5cu62a3rrow;\x5cu61\
a4ector;\x5cu695aia\
ngle\x5cu0180;BE\x5cu0\
924\x5cu0925\x5cu0929\x5c\
u62b2ar;\x5cu69cfqu\
al;\x5cu62b4p\x5cu0180\
DTV\x5cu0937\x5cu0942\x5c\
u094cownVector;\x5c\
u6951eeVector;\x5cu\
6960ector\x5cu0100;\
B\x5cu0956\x5cu0957\x5cu6\
1bfar;\x5cu6958ecto\
r\x5cu0100;B\x5cu0965\x5c\
u0966\x5cu61bcar;\x5cu\
6952ight\x5cxe1\x5cu03\
9cs\x5cu0300EFGLST\x5c\
u097e\x5cu098b\x5cu099\
5\x5cu099d\x5cu09a2\x5cu0\
9adqualGreater;\x5c\
u62daullEqual;\x5cu\
6266reater;\x5cu627\
6ess;\x5cu6aa1lantE\
qual;\x5cu6a7dilde;\
\x5cu6272r;\x5cuc000\x5cu\
d835\x5cudd0f\x5cu0100\
;e\x5cu09bd\x5cu09be\x5cu\
62d8ftarrow;\x5cu61\
daidot;\x5cu413f\x5cu0\
180npw\x5cu09d4\x5cu0a\
16\x5cu0a1bg\x5cu0200L\
Rlr\x5cu09de\x5cu09f7\x5c\
u0a02\x5cu0a10eft\x5cu\
0100AR\x5cu09e6\x5cu09\
ecrrow;\x5cu67f5igh\
tArrow;\x5cu67f7igh\
tArrow;\x5cu67f6eft\
\x5cu0100ar\x5cu03b3\x5cu\
0a0aight\x5cxe1\x5cu03\
bfight\x5cxe1\x5cu03ca\
f;\x5cuc000\x5cud835\x5cu\
dd43er\x5cu0100LR\x5cu\
0a22\x5cu0a2ceftArr\
ow;\x5cu6199ightArr\
ow;\x5cu6198\x5cu0180c\
ht\x5cu0a3e\x5cu0a40\x5cu\
0a42\x5cxf2\x5cu084c;\x5c\
u61b0rok;\x5cu4141;\
\x5cu626a\x5cu0400acef\
iosu\x5cu0a5a\x5cu0a5d\
\x5cu0a60\x5cu0a77\x5cu0a\
7c\x5cu0a85\x5cu0a8b\x5cu\
0a8ep;\x5cu6905y;\x5cu\
441c\x5cu0100dl\x5cu0a\
65\x5cu0a6fiumSpace\
;\x5cu605flintrf;\x5cu\
6133r;\x5cuc000\x5cud8\
35\x5cudd10nusPlus;\
\x5cu6213pf;\x5cuc000\x5c\
ud835\x5cudd44c\x5cxf2\
\x5cu0a76;\x5cu439c\x5cu0\
480Jacefostu\x5cu0a\
a3\x5cu0aa7\x5cu0aad\x5cu\
0ac0\x5cu0b14\x5cu0b19\
\x5cu0d91\x5cu0d97\x5cu0d\
9ecy;\x5cu440acute;\
\x5cu4143\x5cu0180aey\x5c\
u0ab4\x5cu0ab9\x5cu0ab\
eron;\x5cu4147dil;\x5c\
u4145;\x5cu441d\x5cu01\
80gsw\x5cu0ac7\x5cu0af\
0\x5cu0b0eative\x5cu01\
80MTV\x5cu0ad3\x5cu0ad\
f\x5cu0ae8ediumSpac\
e;\x5cu600bhi\x5cu0100\
cn\x5cu0ae6\x5cu0ad8\x5cx\
eb\x5cu0ad9eryThi\x5cx\
ee\x5cu0ad9ted\x5cu010\
0GL\x5cu0af8\x5cu0b06r\
eaterGreate\x5cxf2\x5c\
u0673essLes\x5cxf3\x5c\
u0a48Line;\x5cu400a\
r;\x5cuc000\x5cud835\x5cu\
dd11\x5cu0200Bnpt\x5cu\
0b22\x5cu0b28\x5cu0b37\
\x5cu0b3areak;\x5cu606\
0BreakingSpace;\x5c\
u40a0f;\x5cu6115\x5cu0\
680;CDEGHLNPRSTV\
\x5cu0b55\x5cu0b56\x5cu0b\
6a\x5cu0b7c\x5cu0ba1\x5cu\
0beb\x5cu0c04\x5cu0c5e\
\x5cu0c84\x5cu0ca6\x5cu0c\
d8\x5cu0d61\x5cu0d85\x5cu\
6aec\x5cu0100ou\x5cu0b\
5b\x5cu0b64ngruent;\
\x5cu6262pCap;\x5cu626\
doubleVerticalBa\
r;\x5cu6226\x5cu0180lq\
x\x5cu0b83\x5cu0b8a\x5cu0\
b9bement;\x5cu6209u\
al\x5cu0100;T\x5cu0b92\
\x5cu0b93\x5cu6260ilde\
;\x5cuc000\x5cu2242\x5cu0\
338ists;\x5cu6204re\
ater\x5cu0380;EFGLS\
T\x5cu0bb6\x5cu0bb7\x5cu0\
bbd\x5cu0bc9\x5cu0bd3\x5c\
u0bd8\x5cu0be5\x5cu626\
fqual;\x5cu6271ullE\
qual;\x5cuc000\x5cu226\
7\x5cu0338reater;\x5cu\
c000\x5cu226b\x5cu0338\
ess;\x5cu6279lantEq\
ual;\x5cuc000\x5cu2a7e\
\x5cu0338ilde;\x5cu627\
5ump\x5cu0144\x5cu0bf2\
\x5cu0bfdownHump;\x5cu\
c000\x5cu224e\x5cu0338\
qual;\x5cuc000\x5cu224\
f\x5cu0338e\x5cu0100fs\
\x5cu0c0a\x5cu0c27tTri\
angle\x5cu0180;BE\x5cu\
0c1a\x5cu0c1b\x5cu0c21\
\x5cu62eaar;\x5cuc000\x5c\
u29cf\x5cu0338qual;\
\x5cu62ecs\x5cu0300;EG\
LST\x5cu0c35\x5cu0c36\x5c\
u0c3c\x5cu0c44\x5cu0c4\
b\x5cu0c58\x5cu626equa\
l;\x5cu6270reater;\x5c\
u6278ess;\x5cuc000\x5c\
u226a\x5cu0338lantE\
qual;\x5cuc000\x5cu2a7\
d\x5cu0338ilde;\x5cu62\
74ested\x5cu0100GL\x5c\
u0c68\x5cu0c79reate\
rGreater;\x5cuc000\x5c\
u2aa2\x5cu0338essLe\
ss;\x5cuc000\x5cu2aa1\x5c\
u0338recedes\x5cu01\
80;ES\x5cu0c92\x5cu0c9\
3\x5cu0c9b\x5cu6280qua\
l;\x5cuc000\x5cu2aaf\x5cu\
0338lantEqual;\x5cu\
62e0\x5cu0100ei\x5cu0c\
ab\x5cu0cb9verseEle\
ment;\x5cu620cghtTr\
iangle\x5cu0180;BE\x5c\
u0ccb\x5cu0ccc\x5cu0cd\
2\x5cu62ebar;\x5cuc000\
\x5cu29d0\x5cu0338qual\
;\x5cu62ed\x5cu0100qu\x5c\
u0cdd\x5cu0d0cuareS\
u\x5cu0100bp\x5cu0ce8\x5c\
u0cf9set\x5cu0100;E\
\x5cu0cf0\x5cu0cf3\x5cuc0\
00\x5cu228f\x5cu0338qu\
al;\x5cu62e2erset\x5cu\
0100;E\x5cu0d03\x5cu0d\
06\x5cuc000\x5cu2290\x5cu\
0338qual;\x5cu62e3\x5c\
u0180bcp\x5cu0d13\x5cu\
0d24\x5cu0d4eset\x5cu0\
100;E\x5cu0d1b\x5cu0d1\
e\x5cuc000\x5cu2282\x5cu2\
0d2qual;\x5cu6288ce\
eds\x5cu0200;EST\x5cu0\
d32\x5cu0d33\x5cu0d3b\x5c\
u0d46\x5cu6281qual;\
\x5cuc000\x5cu2ab0\x5cu03\
38lantEqual;\x5cu62\
e1ilde;\x5cuc000\x5cu2\
27f\x5cu0338erset\x5cu\
0100;E\x5cu0d58\x5cu0d\
5b\x5cuc000\x5cu2283\x5cu\
20d2qual;\x5cu6289i\
lde\x5cu0200;EFT\x5cu0\
d6e\x5cu0d6f\x5cu0d75\x5c\
u0d7f\x5cu6241qual;\
\x5cu6244ullEqual;\x5c\
u6247ilde;\x5cu6249\
erticalBar;\x5cu622\
4cr;\x5cuc000\x5cud835\
\x5cudca9ilde\x5cu803b\
\x5cxd1\x5cu40d1;\x5cu439\
d\x5cu0700Eacdfgmop\
rstuv\x5cu0dbd\x5cu0dc\
2\x5cu0dc9\x5cu0dd5\x5cu0\
ddb\x5cu0de0\x5cu0de7\x5c\
u0dfc\x5cu0e02\x5cu0e2\
0\x5cu0e22\x5cu0e32\x5cu0\
e3f\x5cu0e44lig;\x5cu4\
152cute\x5cu803b\x5cxd\
3\x5cu40d3\x5cu0100iy\x5c\
u0dce\x5cu0dd3rc\x5cu8\
03b\x5cxd4\x5cu40d4;\x5cu\
441eblac;\x5cu4150r\
;\x5cuc000\x5cud835\x5cud\
d12rave\x5cu803b\x5cxd\
2\x5cu40d2\x5cu0180aei\
\x5cu0dee\x5cu0df2\x5cu0d\
f6cr;\x5cu414cga;\x5cu\
43a9cron;\x5cu439fp\
f;\x5cuc000\x5cud835\x5cu\
dd46enCurly\x5cu010\
0DQ\x5cu0e0e\x5cu0e1ao\
ubleQuote;\x5cu601c\
uote;\x5cu6018;\x5cu6a\
54\x5cu0100cl\x5cu0e27\
\x5cu0e2cr;\x5cuc000\x5cu\
d835\x5cudcaaash\x5cu8\
03b\x5cxd8\x5cu40d8i\x5cu\
016c\x5cu0e37\x5cu0e3c\
de\x5cu803b\x5cxd5\x5cu40\
d5es;\x5cu6a37ml\x5cu8\
03b\x5cxd6\x5cu40d6er\x5c\
u0100BP\x5cu0e4b\x5cu0\
e60\x5cu0100ar\x5cu0e5\
0\x5cu0e53r;\x5cu603ea\
c\x5cu0100ek\x5cu0e5a\x5c\
u0e5c;\x5cu63deet;\x5c\
u63b4arenthesis;\
\x5cu63dc\x5cu0480acfh\
ilors\x5cu0e7f\x5cu0e8\
7\x5cu0e8a\x5cu0e8f\x5cu0\
e92\x5cu0e94\x5cu0e9d\x5c\
u0eb0\x5cu0efcrtial\
D;\x5cu6202y;\x5cu441f\
r;\x5cuc000\x5cud835\x5cu\
dd13i;\x5cu43a6;\x5cu4\
3a0usMinus;\x5cu40b\
1\x5cu0100ip\x5cu0ea2\x5c\
u0eadncareplan\x5cx\
e5\x5cu069df;\x5cu6119\
\x5cu0200;eio\x5cu0eb9\
\x5cu0eba\x5cu0ee0\x5cu0e\
e4\x5cu6abbcedes\x5cu0\
200;EST\x5cu0ec8\x5cu0\
ec9\x5cu0ecf\x5cu0eda\x5c\
u627aqual;\x5cu6aaf\
lantEqual;\x5cu627c\
ilde;\x5cu627eme;\x5cu\
6033\x5cu0100dp\x5cu0e\
e9\x5cu0eeeuct;\x5cu62\
0fortion\x5cu0100;a\
\x5cu0225\x5cu0ef9l;\x5cu\
621d\x5cu0100ci\x5cu0f\
01\x5cu0f06r;\x5cuc000\
\x5cud835\x5cudcab;\x5cu4\
3a8\x5cu0200Ufos\x5cu0\
f11\x5cu0f16\x5cu0f1b\x5c\
u0f1fOT\x5cu803b\x22\x5cu\
4022r;\x5cuc000\x5cud8\
35\x5cudd14pf;\x5cu611\
acr;\x5cuc000\x5cud835\
\x5cudcac\x5cu0600BEac\
efhiorsu\x5cu0f3e\x5cu\
0f43\x5cu0f47\x5cu0f60\
\x5cu0f73\x5cu0fa7\x5cu0f\
aa\x5cu0fad\x5cu1096\x5cu\
10a9\x5cu10b4\x5cu10be\
arr;\x5cu6910G\x5cu803\
b\x5cxae\x5cu40ae\x5cu018\
0cnr\x5cu0f4e\x5cu0f53\
\x5cu0f56ute;\x5cu4154\
g;\x5cu67ebr\x5cu0100;\
t\x5cu0f5c\x5cu0f5d\x5cu6\
1a0l;\x5cu6916\x5cu018\
0aey\x5cu0f67\x5cu0f6c\
\x5cu0f71ron;\x5cu4158\
dil;\x5cu4156;\x5cu442\
0\x5cu0100;v\x5cu0f78\x5c\
u0f79\x5cu611cerse\x5c\
u0100EU\x5cu0f82\x5cu0\
f99\x5cu0100lq\x5cu0f8\
7\x5cu0f8eement;\x5cu6\
20builibrium;\x5cu6\
1cbpEquilibrium;\
\x5cu696fr\x5cxbb\x5cu0f7\
9o;\x5cu43a1ght\x5cu04\
00ACDFTUVa\x5cu0fc1\
\x5cu0feb\x5cu0ff3\x5cu10\
22\x5cu1028\x5cu105b\x5cu\
1087\x5cu03d8\x5cu0100\
nr\x5cu0fc6\x5cu0fd2gl\
eBracket;\x5cu67e9r\
ow\x5cu0180;BL\x5cu0fd\
c\x5cu0fdd\x5cu0fe1\x5cu6\
192ar;\x5cu61e5eftA\
rrow;\x5cu61c4eilin\
g;\x5cu6309o\x5cu01f5\x5c\
u0ff9\x5c0\x5cu1005ble\
Bracket;\x5cu67e7n\x5c\
u01d4\x5cu100a\x5c0\x5cu1\
014eeVector;\x5cu69\
5dector\x5cu0100;B\x5c\
u101d\x5cu101e\x5cu61c\
2ar;\x5cu6955loor;\x5c\
u630b\x5cu0100er\x5cu1\
02d\x5cu1043e\x5cu0180\
;AV\x5cu1035\x5cu1036\x5c\
u103c\x5cu62a2rrow;\
\x5cu61a6ector;\x5cu69\
5biangle\x5cu0180;B\
E\x5cu1050\x5cu1051\x5cu1\
055\x5cu62b3ar;\x5cu69\
d0qual;\x5cu62b5p\x5cu\
0180DTV\x5cu1063\x5cu1\
06e\x5cu1078ownVect\
or;\x5cu694feeVecto\
r;\x5cu695cector\x5cu0\
100;B\x5cu1082\x5cu108\
3\x5cu61bear;\x5cu6954\
ector\x5cu0100;B\x5cu1\
091\x5cu1092\x5cu61c0a\
r;\x5cu6953\x5cu0100pu\
\x5cu109b\x5cu109ef;\x5cu\
611dndImplies;\x5cu\
6970ightarrow;\x5cu\
61db\x5cu0100ch\x5cu10\
b9\x5cu10bcr;\x5cu611b\
;\x5cu61b1leDelayed\
;\x5cu69f4\x5cu0680HOa\
cfhimoqstu\x5cu10e4\
\x5cu10f1\x5cu10f7\x5cu10\
fd\x5cu1119\x5cu111e\x5cu\
1151\x5cu1156\x5cu1161\
\x5cu1167\x5cu11b5\x5cu11\
bb\x5cu11bf\x5cu0100Cc\
\x5cu10e9\x5cu10eeHcy;\
\x5cu4429y;\x5cu4428FT\
cy;\x5cu442ccute;\x5cu\
415a\x5cu0280;aeiy\x5c\
u1108\x5cu1109\x5cu110\
e\x5cu1113\x5cu1117\x5cu6\
abcron;\x5cu4160dil\
;\x5cu415erc;\x5cu415c\
;\x5cu4421r;\x5cuc000\x5c\
ud835\x5cudd16ort\x5cu\
0200DLRU\x5cu112a\x5cu\
1134\x5cu113e\x5cu1149\
ownArrow\x5cxbb\x5cu04\
1eeftArrow\x5cxbb\x5cu\
089aightArrow\x5cxb\
b\x5cu0fddpArrow;\x5cu\
6191gma;\x5cu43a3al\
lCircle;\x5cu6218pf\
;\x5cuc000\x5cud835\x5cud\
d4a\x5cu0272\x5cu116d\x5c\
0\x5c0\x5cu1170t;\x5cu621\
aare\x5cu0200;ISU\x5cu\
117b\x5cu117c\x5cu1189\
\x5cu11af\x5cu65a1nter\
section;\x5cu6293u\x5c\
u0100bp\x5cu118f\x5cu1\
19eset\x5cu0100;E\x5cu\
1197\x5cu1198\x5cu628f\
qual;\x5cu6291erset\
\x5cu0100;E\x5cu11a8\x5cu\
11a9\x5cu6290qual;\x5c\
u6292nion;\x5cu6294\
cr;\x5cuc000\x5cud835\x5c\
udcaear;\x5cu62c6\x5cu\
0200bcmp\x5cu11c8\x5cu\
11db\x5cu1209\x5cu120b\
\x5cu0100;s\x5cu11cd\x5cu\
11ce\x5cu62d0et\x5cu01\
00;E\x5cu11cd\x5cu11d5\
qual;\x5cu6286\x5cu010\
0ch\x5cu11e0\x5cu1205e\
eds\x5cu0200;EST\x5cu1\
1ed\x5cu11ee\x5cu11f4\x5c\
u11ff\x5cu627bqual;\
\x5cu6ab0lantEqual;\
\x5cu627dilde;\x5cu627\
fTh\x5cxe1\x5cu0f8c;\x5cu\
6211\x5cu0180;es\x5cu1\
212\x5cu1213\x5cu1223\x5c\
u62d1rset\x5cu0100;\
E\x5cu121c\x5cu121d\x5cu6\
283qual;\x5cu6287et\
\x5cxbb\x5cu1213\x5cu0580\
HRSacfhiors\x5cu123\
e\x5cu1244\x5cu1249\x5cu1\
255\x5cu125e\x5cu1271\x5c\
u1276\x5cu129f\x5cu12c\
2\x5cu12c8\x5cu12d1ORN\
\x5cu803b\x5cxde\x5cu40de\
ADE;\x5cu6122\x5cu0100\
Hc\x5cu124e\x5cu1252cy\
;\x5cu440by;\x5cu4426\x5c\
u0100bu\x5cu125a\x5cu1\
25c;\x5cu4009;\x5cu43a\
4\x5cu0180aey\x5cu1265\
\x5cu126a\x5cu126fron;\
\x5cu4164dil;\x5cu4162\
;\x5cu4422r;\x5cuc000\x5c\
ud835\x5cudd17\x5cu010\
0ei\x5cu127b\x5cu1289\x5c\
u01f2\x5cu1280\x5c0\x5cu1\
287efore;\x5cu6234a\
;\x5cu4398\x5cu0100cn\x5c\
u128e\x5cu1298kSpac\
e;\x5cuc000\x5cu205f\x5cu\
200aSpace;\x5cu6009\
lde\x5cu0200;EFT\x5cu1\
2ab\x5cu12ac\x5cu12b2\x5c\
u12bc\x5cu623cqual;\
\x5cu6243ullEqual;\x5c\
u6245ilde;\x5cu6248\
pf;\x5cuc000\x5cud835\x5c\
udd4bipleDot;\x5cu6\
0db\x5cu0100ct\x5cu12d\
6\x5cu12dbr;\x5cuc000\x5c\
ud835\x5cudcafrok;\x5c\
u4166\x5cu0ae1\x5cu12f\
7\x5cu130e\x5cu131a\x5cu1\
326\x5c0\x5cu132c\x5cu133\
1\x5c0\x5c0\x5c0\x5c0\x5c0\x5cu133\
8\x5cu133d\x5cu1377\x5cu1\
385\x5c0\x5cu13ff\x5cu140\
4\x5cu140a\x5cu1410\x5cu0\
100cr\x5cu12fb\x5cu130\
1ute\x5cu803b\x5cxda\x5cu\
40dar\x5cu0100;o\x5cu1\
307\x5cu1308\x5cu619fc\
ir;\x5cu6949r\x5cu01e3\
\x5cu1313\x5c0\x5cu1316y;\
\x5cu440eve;\x5cu416c\x5c\
u0100iy\x5cu131e\x5cu1\
323rc\x5cu803b\x5cxdb\x5c\
u40db;\x5cu4423blac\
;\x5cu4170r;\x5cuc000\x5c\
ud835\x5cudd18rave\x5c\
u803b\x5cxd9\x5cu40d9a\
cr;\x5cu416a\x5cu0100d\
i\x5cu1341\x5cu1369er\x5c\
u0100BP\x5cu1348\x5cu1\
35d\x5cu0100ar\x5cu134\
d\x5cu1350r;\x5cu405fa\
c\x5cu0100ek\x5cu1357\x5c\
u1359;\x5cu63dfet;\x5c\
u63b5arenthesis;\
\x5cu63ddon\x5cu0100;P\
\x5cu1370\x5cu1371\x5cu62\
c3lus;\x5cu628e\x5cu01\
00gp\x5cu137b\x5cu137f\
on;\x5cu4172f;\x5cuc00\
0\x5cud835\x5cudd4c\x5cu0\
400ADETadps\x5cu139\
5\x5cu13ae\x5cu13b8\x5cu1\
3c4\x5cu03e8\x5cu13d2\x5c\
u13d7\x5cu13f3rrow\x5c\
u0180;BD\x5cu1150\x5cu\
13a0\x5cu13a4ar;\x5cu6\
912ownArrow;\x5cu61\
c5ownArrow;\x5cu619\
5quilibrium;\x5cu69\
6eee\x5cu0100;A\x5cu13\
cb\x5cu13cc\x5cu62a5rr\
ow;\x5cu61a5own\x5cxe1\
\x5cu03f3er\x5cu0100LR\
\x5cu13de\x5cu13e8eftA\
rrow;\x5cu6196ightA\
rrow;\x5cu6197i\x5cu01\
00;l\x5cu13f9\x5cu13fa\
\x5cu43d2on;\x5cu43a5i\
ng;\x5cu416ecr;\x5cuc0\
00\x5cud835\x5cudcb0il\
de;\x5cu4168ml\x5cu803\
b\x5cxdc\x5cu40dc\x5cu048\
0Dbcdefosv\x5cu1427\
\x5cu142c\x5cu1430\x5cu14\
33\x5cu143e\x5cu1485\x5cu\
148a\x5cu1490\x5cu1496\
ash;\x5cu62abar;\x5cu6\
aeby;\x5cu4412ash\x5cu\
0100;l\x5cu143b\x5cu14\
3c\x5cu62a9;\x5cu6ae6\x5c\
u0100er\x5cu1443\x5cu1\
445;\x5cu62c1\x5cu0180\
bty\x5cu144c\x5cu1450\x5c\
u147aar;\x5cu6016\x5cu\
0100;i\x5cu144f\x5cu14\
55cal\x5cu0200BLST\x5c\
u1461\x5cu1465\x5cu146\
a\x5cu1474ar;\x5cu6223\
ine;\x5cu407ceparat\
or;\x5cu6758ilde;\x5cu\
6240ThinSpace;\x5cu\
600ar;\x5cuc000\x5cud8\
35\x5cudd19pf;\x5cuc00\
0\x5cud835\x5cudd4dcr;\
\x5cuc000\x5cud835\x5cudc\
b1dash;\x5cu62aa\x5cu0\
280cefos\x5cu14a7\x5cu\
14ac\x5cu14b1\x5cu14b6\
\x5cu14bcirc;\x5cu4174\
dge;\x5cu62c0r;\x5cuc0\
00\x5cud835\x5cudd1apf\
;\x5cuc000\x5cud835\x5cud\
d4ecr;\x5cuc000\x5cud8\
35\x5cudcb2\x5cu0200fi\
os\x5cu14cb\x5cu14d0\x5cu\
14d2\x5cu14d8r;\x5cuc0\
00\x5cud835\x5cudd1b;\x5c\
u439epf;\x5cuc000\x5cu\
d835\x5cudd4fcr;\x5cuc\
000\x5cud835\x5cudcb3\x5c\
u0480AIUacfosu\x5cu\
14f1\x5cu14f5\x5cu14f9\
\x5cu14fd\x5cu1504\x5cu15\
0f\x5cu1514\x5cu151a\x5cu\
1520cy;\x5cu442fcy;\
\x5cu4407cy;\x5cu442ec\
ute\x5cu803b\x5cxdd\x5cu4\
0dd\x5cu0100iy\x5cu150\
9\x5cu150drc;\x5cu4176\
;\x5cu442br;\x5cuc000\x5c\
ud835\x5cudd1cpf;\x5cu\
c000\x5cud835\x5cudd50\
cr;\x5cuc000\x5cud835\x5c\
udcb4ml;\x5cu4178\x5cu\
0400Hacdefos\x5cu15\
35\x5cu1539\x5cu153f\x5cu\
154b\x5cu154f\x5cu155d\
\x5cu1560\x5cu1564cy;\x5c\
u4416cute;\x5cu4179\
\x5cu0100ay\x5cu1544\x5cu\
1549ron;\x5cu417d;\x5c\
u4417ot;\x5cu417b\x5cu\
01f2\x5cu1554\x5c0\x5cu15\
5boWidt\x5cxe8\x5cu0ad\
9a;\x5cu4396r;\x5cu612\
8pf;\x5cu6124cr;\x5cuc\
000\x5cud835\x5cudcb5\x5c\
u0be1\x5cu1583\x5cu158\
a\x5cu1590\x5c0\x5cu15b0\x5c\
u15b6\x5cu15bf\x5c0\x5c0\x5c\
0\x5c0\x5cu15c6\x5cu15db\x5c\
u15eb\x5cu165f\x5cu166\
d\x5c0\x5cu1695\x5cu169b\x5c\
u16b2\x5cu16b9\x5c0\x5cu1\
6becute\x5cu803b\x5cxe\
1\x5cu40e1reve;\x5cu41\
03\x5cu0300;Ediuy\x5cu\
159c\x5cu159d\x5cu15a1\
\x5cu15a3\x5cu15a8\x5cu15\
ad\x5cu623e;\x5cuc000\x5c\
u223e\x5cu0333;\x5cu62\
3frc\x5cu803b\x5cxe2\x5cu\
40e2te\x5cu80bb\x5cxb4\
\x5cu0306;\x5cu4430lig\
\x5cu803b\x5cxe6\x5cu40e6\
\x5cu0100;r\x5cxb2\x5cu15\
ba;\x5cuc000\x5cud835\x5c\
udd1erave\x5cu803b\x5c\
xe0\x5cu40e0\x5cu0100e\
p\x5cu15ca\x5cu15d6\x5cu0\
100fp\x5cu15cf\x5cu15d\
4sym;\x5cu6135\x5cxe8\x5c\
u15d3ha;\x5cu43b1\x5cu\
0100ap\x5cu15dfc\x5cu0\
100cl\x5cu15e4\x5cu15e\
7r;\x5cu4101g;\x5cu6a3\
f\x5cu0264\x5cu15f0\x5c0\x5c\
0\x5cu160a\x5cu0280;ad\
sv\x5cu15fa\x5cu15fb\x5cu\
15ff\x5cu1601\x5cu1607\
\x5cu6227nd;\x5cu6a55;\
\x5cu6a5clope;\x5cu6a5\
8;\x5cu6a5a\x5cu0380;e\
lmrsz\x5cu1618\x5cu161\
9\x5cu161b\x5cu161e\x5cu1\
63f\x5cu164f\x5cu1659\x5c\
u6220;\x5cu69a4e\x5cxb\
b\x5cu1619sd\x5cu0100;\
a\x5cu1625\x5cu1626\x5cu6\
221\x5cu0461\x5cu1630\x5c\
u1632\x5cu1634\x5cu163\
6\x5cu1638\x5cu163a\x5cu1\
63c\x5cu163e;\x5cu69a8\
;\x5cu69a9;\x5cu69aa;\x5c\
u69ab;\x5cu69ac;\x5cu6\
9ad;\x5cu69ae;\x5cu69a\
ft\x5cu0100;v\x5cu1645\
\x5cu1646\x5cu621fb\x5cu0\
100;d\x5cu164c\x5cu164\
d\x5cu62be;\x5cu699d\x5cu\
0100pt\x5cu1654\x5cu16\
57h;\x5cu6222\x5cxbb\x5cx\
b9arr;\x5cu637c\x5cu01\
00gp\x5cu1663\x5cu1667\
on;\x5cu4105f;\x5cuc00\
0\x5cud835\x5cudd52\x5cu0\
380;Eaeiop\x5cu12c1\
\x5cu167b\x5cu167d\x5cu16\
82\x5cu1684\x5cu1687\x5cu\
168a;\x5cu6a70cir;\x5c\
u6a6f;\x5cu624ad;\x5cu\
624bs;\x5cu4027rox\x5c\
u0100;e\x5cu12c1\x5cu1\
692\x5cxf1\x5cu1683ing\
\x5cu803b\x5cxe5\x5cu40e5\
\x5cu0180cty\x5cu16a1\x5c\
u16a6\x5cu16a8r;\x5cuc\
000\x5cud835\x5cudcb6;\
\x5cu402amp\x5cu0100;e\
\x5cu12c1\x5cu16af\x5cxf1\
\x5cu0288ilde\x5cu803b\
\x5cxe3\x5cu40e3ml\x5cu80\
3b\x5cxe4\x5cu40e4\x5cu01\
00ci\x5cu16c2\x5cu16c8\
onin\x5cxf4\x5cu0272nt\
;\x5cu6a11\x5cu0800Nab\
cdefiklnoprsu\x5cu1\
6ed\x5cu16f1\x5cu1730\x5c\
u173c\x5cu1743\x5cu174\
8\x5cu1778\x5cu177d\x5cu1\
7e0\x5cu17e6\x5cu1839\x5c\
u1850\x5cu170d\x5cu193\
d\x5cu1948\x5cu1970ot;\
\x5cu6aed\x5cu0100cr\x5cu\
16f6\x5cu171ek\x5cu020\
0ceps\x5cu1700\x5cu170\
5\x5cu170d\x5cu1713ong\
;\x5cu624cpsilon;\x5cu\
43f6rime;\x5cu6035i\
m\x5cu0100;e\x5cu171a\x5c\
u171b\x5cu623dq;\x5cu6\
2cd\x5cu0176\x5cu1722\x5c\
u1726ee;\x5cu62bded\
\x5cu0100;g\x5cu172c\x5cu\
172d\x5cu6305e\x5cxbb\x5c\
u172drk\x5cu0100;t\x5c\
u135c\x5cu1737brk;\x5c\
u63b6\x5cu0100oy\x5cu1\
701\x5cu1741;\x5cu4431\
quo;\x5cu601e\x5cu0280\
cmprt\x5cu1753\x5cu175\
b\x5cu1761\x5cu1764\x5cu1\
768aus\x5cu0100;e\x5cu\
010a\x5cu0109ptyv;\x5c\
u69b0s\x5cxe9\x5cu170c\
no\x5cxf5\x5cu0113\x5cu01\
80ahw\x5cu176f\x5cu177\
1\x5cu1773;\x5cu43b2;\x5c\
u6136een;\x5cu626cr\
;\x5cuc000\x5cud835\x5cud\
d1fg\x5cu0380costuv\
w\x5cu178d\x5cu179d\x5cu1\
7b3\x5cu17c1\x5cu17d5\x5c\
u17db\x5cu17de\x5cu018\
0aiu\x5cu1794\x5cu1796\
\x5cu179a\x5cxf0\x5cu0760\
rc;\x5cu65efp\x5cxbb\x5cu\
1371\x5cu0180dpt\x5cu1\
7a4\x5cu17a8\x5cu17ado\
t;\x5cu6a00lus;\x5cu6a\
01imes;\x5cu6a02\x5cu0\
271\x5cu17b9\x5c0\x5c0\x5cu1\
7becup;\x5cu6a06ar;\
\x5cu6605riangle\x5cu0\
100du\x5cu17cd\x5cu17d\
2own;\x5cu65bdp;\x5cu6\
5b3plus;\x5cu6a04e\x5c\
xe5\x5cu1444\x5cxe5\x5cu1\
4adarow;\x5cu690d\x5cu\
0180ako\x5cu17ed\x5cu1\
826\x5cu1835\x5cu0100c\
n\x5cu17f2\x5cu1823k\x5cu\
0180lst\x5cu17fa\x5cu0\
5ab\x5cu1802ozenge;\
\x5cu69ebriangle\x5cu0\
200;dlr\x5cu1812\x5cu1\
813\x5cu1818\x5cu181d\x5c\
u65b4own;\x5cu65bee\
ft;\x5cu65c2ight;\x5cu\
65b8k;\x5cu6423\x5cu01\
b1\x5cu182b\x5c0\x5cu1833\
\x5cu01b2\x5cu182f\x5c0\x5cu\
1831;\x5cu6592;\x5cu65\
914;\x5cu6593ck;\x5cu6\
588\x5cu0100eo\x5cu183\
e\x5cu184d\x5cu0100;q\x5c\
u1843\x5cu1846\x5cuc00\
0=\x5cu20e5uiv;\x5cuc0\
00\x5cu2261\x5cu20e5t;\
\x5cu6310\x5cu0200ptwx\
\x5cu1859\x5cu185e\x5cu18\
67\x5cu186cf;\x5cuc000\
\x5cud835\x5cudd53\x5cu01\
00;t\x5cu13cb\x5cu1863\
om\x5cxbb\x5cu13cctie;\
\x5cu62c8\x5cu0600DHUV\
bdhmptuv\x5cu1885\x5cu\
1896\x5cu18aa\x5cu18bb\
\x5cu18d7\x5cu18db\x5cu18\
ec\x5cu18ff\x5cu1905\x5cu\
190a\x5cu1910\x5cu1921\
\x5cu0200LRlr\x5cu188e\
\x5cu1890\x5cu1892\x5cu18\
94;\x5cu6557;\x5cu6554\
;\x5cu6556;\x5cu6553\x5cu\
0280;DUdu\x5cu18a1\x5c\
u18a2\x5cu18a4\x5cu18a\
6\x5cu18a8\x5cu6550;\x5cu\
6566;\x5cu6569;\x5cu65\
64;\x5cu6567\x5cu0200L\
Rlr\x5cu18b3\x5cu18b5\x5c\
u18b7\x5cu18b9;\x5cu65\
5d;\x5cu655a;\x5cu655c\
;\x5cu6559\x5cu0380;HL\
Rhlr\x5cu18ca\x5cu18cb\
\x5cu18cd\x5cu18cf\x5cu18\
d1\x5cu18d3\x5cu18d5\x5cu\
6551;\x5cu656c;\x5cu65\
63;\x5cu6560;\x5cu656b\
;\x5cu6562;\x5cu655fox\
;\x5cu69c9\x5cu0200LRl\
r\x5cu18e4\x5cu18e6\x5cu1\
8e8\x5cu18ea;\x5cu6555\
;\x5cu6552;\x5cu6510;\x5c\
u650c\x5cu0280;DUdu\
\x5cu06bd\x5cu18f7\x5cu18\
f9\x5cu18fb\x5cu18fd;\x5c\
u6565;\x5cu6568;\x5cu6\
52c;\x5cu6534inus;\x5c\
u629flus;\x5cu629ei\
mes;\x5cu62a0\x5cu0200\
LRlr\x5cu1919\x5cu191b\
\x5cu191d\x5cu191f;\x5cu6\
55b;\x5cu6558;\x5cu651\
8;\x5cu6514\x5cu0380;H\
LRhlr\x5cu1930\x5cu193\
1\x5cu1933\x5cu1935\x5cu1\
937\x5cu1939\x5cu193b\x5c\
u6502;\x5cu656a;\x5cu6\
561;\x5cu655e;\x5cu653\
c;\x5cu6524;\x5cu651c\x5c\
u0100ev\x5cu0123\x5cu1\
942bar\x5cu803b\x5cxa6\
\x5cu40a6\x5cu0200ceio\
\x5cu1951\x5cu1956\x5cu19\
5a\x5cu1960r;\x5cuc000\
\x5cud835\x5cudcb7mi;\x5c\
u604fm\x5cu0100;e\x5cu\
171a\x5cu171cl\x5cu018\
0;bh\x5cu1968\x5cu1969\
\x5cu196b\x5cu405c;\x5cu6\
9c5sub;\x5cu67c8\x5cu0\
16c\x5cu1974\x5cu197el\
\x5cu0100;e\x5cu1979\x5cu\
197a\x5cu6022t\x5cxbb\x5c\
u197ap\x5cu0180;Ee\x5c\
u012f\x5cu1985\x5cu198\
7;\x5cu6aae\x5cu0100;q\
\x5cu06dc\x5cu06db\x5cu0c\
e1\x5cu19a7\x5c0\x5cu19e8\
\x5cu1a11\x5cu1a15\x5cu1a\
32\x5c0\x5cu1a37\x5cu1a50\
\x5c0\x5c0\x5cu1ab4\x5c0\x5c0\x5cu\
1ac1\x5c0\x5c0\x5cu1b21\x5cu\
1b2e\x5cu1b4d\x5cu1b52\
\x5c0\x5cu1bfd\x5c0\x5cu1c0c\
\x5cu0180cpr\x5cu19ad\x5c\
u19b2\x5cu19ddute;\x5c\
u4107\x5cu0300;abcd\
s\x5cu19bf\x5cu19c0\x5cu1\
9c4\x5cu19ca\x5cu19d5\x5c\
u19d9\x5cu6229nd;\x5cu\
6a44rcup;\x5cu6a49\x5c\
u0100au\x5cu19cf\x5cu1\
9d2p;\x5cu6a4bp;\x5cu6\
a47ot;\x5cu6a40;\x5cuc\
000\x5cu2229\x5cufe00\x5c\
u0100eo\x5cu19e2\x5cu1\
9e5t;\x5cu6041\x5cxee\x5c\
u0693\x5cu0200aeiu\x5c\
u19f0\x5cu19fb\x5cu1a0\
1\x5cu1a05\x5cu01f0\x5cu1\
9f5\x5c0\x5cu19f8s;\x5cu6\
a4don;\x5cu410ddil\x5c\
u803b\x5cxe7\x5cu40e7r\
c;\x5cu4109ps\x5cu0100\
;s\x5cu1a0c\x5cu1a0d\x5cu\
6a4cm;\x5cu6a50ot;\x5c\
u410b\x5cu0180dmn\x5cu\
1a1b\x5cu1a20\x5cu1a26\
il\x5cu80bb\x5cxb8\x5cu01\
adptyv;\x5cu69b2t\x5cu\
8100\x5cxa2;e\x5cu1a2d\
\x5cu1a2e\x5cu40a2r\x5cxe\
4\x5cu01b2r;\x5cuc000\x5c\
ud835\x5cudd20\x5cu018\
0cei\x5cu1a3d\x5cu1a40\
\x5cu1a4dy;\x5cu4447ck\
\x5cu0100;m\x5cu1a47\x5cu\
1a48\x5cu6713ark\x5cxb\
b\x5cu1a48;\x5cu43c7r\x5c\
u0380;Ecefms\x5cu1a\
5f\x5cu1a60\x5cu1a62\x5cu\
1a6b\x5cu1aa4\x5cu1aaa\
\x5cu1aae\x5cu65cb;\x5cu6\
9c3\x5cu0180;el\x5cu1a\
69\x5cu1a6a\x5cu1a6d\x5cu\
42c6q;\x5cu6257e\x5cu0\
261\x5cu1a74\x5c0\x5c0\x5cu1\
a88rrow\x5cu0100lr\x5c\
u1a7c\x5cu1a81eft;\x5c\
u61baight;\x5cu61bb\
\x5cu0280RSacd\x5cu1a9\
2\x5cu1a94\x5cu1a96\x5cu1\
a9a\x5cu1a9f\x5cxbb\x5cu0\
f47;\x5cu64c8st;\x5cu6\
29birc;\x5cu629aash\
;\x5cu629dnint;\x5cu6a\
10id;\x5cu6aefcir;\x5c\
u69c2ubs\x5cu0100;u\
\x5cu1abb\x5cu1abc\x5cu66\
63it\x5cxbb\x5cu1abc\x5cu\
02ec\x5cu1ac7\x5cu1ad4\
\x5cu1afa\x5c0\x5cu1b0aon\
\x5cu0100;e\x5cu1acd\x5cu\
1ace\x5cu403a\x5cu0100\
;q\x5cxc7\x5cxc6\x5cu026d\
\x5cu1ad9\x5c0\x5c0\x5cu1ae2\
a\x5cu0100;t\x5cu1ade\x5c\
u1adf\x5cu402c;\x5cu40\
40\x5cu0180;fl\x5cu1ae\
8\x5cu1ae9\x5cu1aeb\x5cu6\
201\x5cxee\x5cu1160e\x5cu\
0100mx\x5cu1af1\x5cu1a\
f6ent\x5cxbb\x5cu1ae9e\
\x5cxf3\x5cu024d\x5cu01e7\
\x5cu1afe\x5c0\x5cu1b07\x5cu\
0100;d\x5cu12bb\x5cu1b\
02ot;\x5cu6a6dn\x5cxf4\
\x5cu0246\x5cu0180fry\x5c\
u1b10\x5cu1b14\x5cu1b1\
7;\x5cuc000\x5cud835\x5cu\
dd54o\x5cxe4\x5cu0254\x5c\
u8100\x5cxa9;s\x5cu015\
5\x5cu1b1dr;\x5cu6117\x5c\
u0100ao\x5cu1b25\x5cu1\
b29rr;\x5cu61b5ss;\x5c\
u6717\x5cu0100cu\x5cu1\
b32\x5cu1b37r;\x5cuc00\
0\x5cud835\x5cudcb8\x5cu0\
100bp\x5cu1b3c\x5cu1b4\
4\x5cu0100;e\x5cu1b41\x5c\
u1b42\x5cu6acf;\x5cu6a\
d1\x5cu0100;e\x5cu1b49\
\x5cu1b4a\x5cu6ad0;\x5cu6\
ad2dot;\x5cu62ef\x5cu0\
380delprvw\x5cu1b60\
\x5cu1b6c\x5cu1b77\x5cu1b\
82\x5cu1bac\x5cu1bd4\x5cu\
1bf9arr\x5cu0100lr\x5c\
u1b68\x5cu1b6a;\x5cu69\
38;\x5cu6935\x5cu0270\x5c\
u1b72\x5c0\x5c0\x5cu1b75r\
;\x5cu62dec;\x5cu62dfa\
rr\x5cu0100;p\x5cu1b7f\
\x5cu1b80\x5cu61b6;\x5cu6\
93d\x5cu0300;bcdos\x5c\
u1b8f\x5cu1b90\x5cu1b9\
6\x5cu1ba1\x5cu1ba5\x5cu1\
ba8\x5cu622arcap;\x5cu\
6a48\x5cu0100au\x5cu1b\
9b\x5cu1b9ep;\x5cu6a46\
p;\x5cu6a4aot;\x5cu628\
dr;\x5cu6a45;\x5cuc000\
\x5cu222a\x5cufe00\x5cu02\
00alrv\x5cu1bb5\x5cu1b\
bf\x5cu1bde\x5cu1be3rr\
\x5cu0100;m\x5cu1bbc\x5cu\
1bbd\x5cu61b7;\x5cu693\
cy\x5cu0180evw\x5cu1bc\
7\x5cu1bd4\x5cu1bd8q\x5cu\
0270\x5cu1bce\x5c0\x5c0\x5cu\
1bd2re\x5cxe3\x5cu1b73\
u\x5cxe3\x5cu1b75ee;\x5cu\
62ceedge;\x5cu62cfe\
n\x5cu803b\x5cxa4\x5cu40a\
4earrow\x5cu0100lr\x5c\
u1bee\x5cu1bf3eft\x5cx\
bb\x5cu1b80ight\x5cxbb\
\x5cu1bbde\x5cxe4\x5cu1bd\
d\x5cu0100ci\x5cu1c01\x5c\
u1c07onin\x5cxf4\x5cu0\
1f7nt;\x5cu6231lcty\
;\x5cu632d\x5cu0980AHa\
bcdefhijlorstuwz\
\x5cu1c38\x5cu1c3b\x5cu1c\
3f\x5cu1c5d\x5cu1c69\x5cu\
1c75\x5cu1c8a\x5cu1c9e\
\x5cu1cac\x5cu1cb7\x5cu1c\
fb\x5cu1cff\x5cu1d0d\x5cu\
1d7b\x5cu1d91\x5cu1dab\
\x5cu1dbb\x5cu1dc6\x5cu1d\
cdr\x5cxf2\x5cu0381ar;\
\x5cu6965\x5cu0200glrs\
\x5cu1c48\x5cu1c4d\x5cu1c\
52\x5cu1c54ger;\x5cu60\
20eth;\x5cu6138\x5cxf2\
\x5cu1133h\x5cu0100;v\x5c\
u1c5a\x5cu1c5b\x5cu601\
0\x5cxbb\x5cu090a\x5cu016\
b\x5cu1c61\x5cu1c67aro\
w;\x5cu690fa\x5cxe3\x5cu0\
315\x5cu0100ay\x5cu1c6\
e\x5cu1c73ron;\x5cu410\
f;\x5cu4434\x5cu0180;a\
o\x5cu0332\x5cu1c7c\x5cu1\
c84\x5cu0100gr\x5cu02b\
f\x5cu1c81r;\x5cu61cat\
seq;\x5cu6a77\x5cu0180\
glm\x5cu1c91\x5cu1c94\x5c\
u1c98\x5cu803b\x5cxb0\x5c\
u40b0ta;\x5cu43b4pt\
yv;\x5cu69b1\x5cu0100i\
r\x5cu1ca3\x5cu1ca8sht\
;\x5cu697f;\x5cuc000\x5cu\
d835\x5cudd21ar\x5cu01\
00lr\x5cu1cb3\x5cu1cb5\
\x5cxbb\x5cu08dc\x5cxbb\x5cu\
101e\x5cu0280aegsv\x5c\
u1cc2\x5cu0378\x5cu1cd\
6\x5cu1cdc\x5cu1ce0m\x5cu\
0180;os\x5cu0326\x5cu1\
cca\x5cu1cd4nd\x5cu010\
0;s\x5cu0326\x5cu1cd1u\
it;\x5cu6666amma;\x5cu\
43ddin;\x5cu62f2\x5cu0\
180;io\x5cu1ce7\x5cu1c\
e8\x5cu1cf8\x5cu40f7de\
\x5cu8100\x5cxf7;o\x5cu1c\
e7\x5cu1cf0ntimes;\x5c\
u62c7n\x5cxf8\x5cu1cf7\
cy;\x5cu4452c\x5cu026f\
\x5cu1d06\x5c0\x5c0\x5cu1d0a\
rn;\x5cu631eop;\x5cu63\
0d\x5cu0280lptuw\x5cu1\
d18\x5cu1d1d\x5cu1d22\x5c\
u1d49\x5cu1d55lar;\x5c\
u4024f;\x5cuc000\x5cud\
835\x5cudd55\x5cu0280;\
emps\x5cu030b\x5cu1d2d\
\x5cu1d37\x5cu1d3d\x5cu1d\
42q\x5cu0100;d\x5cu035\
2\x5cu1d33ot;\x5cu6251\
inus;\x5cu6238lus;\x5c\
u6214quare;\x5cu62a\
1blebarwedg\x5cxe5\x5c\
xfan\x5cu0180adh\x5cu1\
12e\x5cu1d5d\x5cu1d67o\
wnarrow\x5cxf3\x5cu1c8\
3arpoon\x5cu0100lr\x5c\
u1d72\x5cu1d76ef\x5cxf\
4\x5cu1cb4igh\x5cxf4\x5cu\
1cb6\x5cu0162\x5cu1d7f\
\x5cu1d85karo\x5cxf7\x5cu\
0f42\x5cu026f\x5cu1d8a\
\x5c0\x5c0\x5cu1d8ern;\x5cu6\
31fop;\x5cu630c\x5cu01\
80cot\x5cu1d98\x5cu1da\
3\x5cu1da6\x5cu0100ry\x5c\
u1d9d\x5cu1da1;\x5cuc0\
00\x5cud835\x5cudcb9;\x5c\
u4455l;\x5cu69f6rok\
;\x5cu4111\x5cu0100dr\x5c\
u1db0\x5cu1db4ot;\x5cu\
62f1i\x5cu0100;f\x5cu1\
dba\x5cu1816\x5cu65bf\x5c\
u0100ah\x5cu1dc0\x5cu1\
dc3r\x5cxf2\x5cu0429a\x5c\
xf2\x5cu0fa6angle;\x5c\
u69a6\x5cu0100ci\x5cu1\
dd2\x5cu1dd5y;\x5cu445\
fgrarr;\x5cu67ff\x5cu0\
900Dacdefglmnopq\
rstux\x5cu1e01\x5cu1e0\
9\x5cu1e19\x5cu1e38\x5cu0\
578\x5cu1e3c\x5cu1e49\x5c\
u1e61\x5cu1e7e\x5cu1ea\
5\x5cu1eaf\x5cu1ebd\x5cu1\
ee1\x5cu1f2a\x5cu1f37\x5c\
u1f44\x5cu1f4e\x5cu1f5\
a\x5cu0100Do\x5cu1e06\x5c\
u1d34o\x5cxf4\x5cu1c89\
\x5cu0100cs\x5cu1e0e\x5cu\
1e14ute\x5cu803b\x5cxe\
9\x5cu40e9ter;\x5cu6a6\
e\x5cu0200aioy\x5cu1e2\
2\x5cu1e27\x5cu1e31\x5cu1\
e36ron;\x5cu411br\x5cu\
0100;c\x5cu1e2d\x5cu1e\
2e\x5cu6256\x5cu803b\x5cx\
ea\x5cu40ealon;\x5cu62\
55;\x5cu444dot;\x5cu41\
17\x5cu0100Dr\x5cu1e41\
\x5cu1e45ot;\x5cu6252;\
\x5cuc000\x5cud835\x5cudd\
22\x5cu0180;rs\x5cu1e5\
0\x5cu1e51\x5cu1e57\x5cu6\
a9aave\x5cu803b\x5cxe8\
\x5cu40e8\x5cu0100;d\x5cu\
1e5c\x5cu1e5d\x5cu6a96\
ot;\x5cu6a98\x5cu0200;\
ils\x5cu1e6a\x5cu1e6b\x5c\
u1e72\x5cu1e74\x5cu6a9\
9nters;\x5cu63e7;\x5cu\
6113\x5cu0100;d\x5cu1e\
79\x5cu1e7a\x5cu6a95ot\
;\x5cu6a97\x5cu0180aps\
\x5cu1e85\x5cu1e89\x5cu1e\
97cr;\x5cu4113ty\x5cu0\
180;sv\x5cu1e92\x5cu1e\
93\x5cu1e95\x5cu6205et\
\x5cxbb\x5cu1e93p\x5cu010\
01;\x5cu1e9d\x5cu1ea4\x5c\
u0133\x5cu1ea1\x5cu1ea\
3;\x5cu6004;\x5cu6005\x5c\
u6003\x5cu0100gs\x5cu1\
eaa\x5cu1eac;\x5cu414b\
p;\x5cu6002\x5cu0100gp\
\x5cu1eb4\x5cu1eb8on;\x5c\
u4119f;\x5cuc000\x5cud\
835\x5cudd56\x5cu0180a\
ls\x5cu1ec4\x5cu1ece\x5cu\
1ed2r\x5cu0100;s\x5cu1\
eca\x5cu1ecb\x5cu62d5l\
;\x5cu69e3us;\x5cu6a71\
i\x5cu0180;lv\x5cu1eda\
\x5cu1edb\x5cu1edf\x5cu43\
b5on\x5cxbb\x5cu1edb;\x5c\
u43f5\x5cu0200csuv\x5c\
u1eea\x5cu1ef3\x5cu1f0\
b\x5cu1f23\x5cu0100io\x5c\
u1eef\x5cu1e31rc\x5cxb\
b\x5cu1e2e\x5cu0269\x5cu1\
ef9\x5c0\x5c0\x5cu1efb\x5cxe\
d\x5cu0548ant\x5cu0100\
gl\x5cu1f02\x5cu1f06tr\
\x5cxbb\x5cu1e5dess\x5cxb\
b\x5cu1e7a\x5cu0180aei\
\x5cu1f12\x5cu1f16\x5cu1f\
1als;\x5cu403dst;\x5cu\
625fv\x5cu0100;D\x5cu0\
235\x5cu1f20D;\x5cu6a7\
8parsl;\x5cu69e5\x5cu0\
100Da\x5cu1f2f\x5cu1f3\
3ot;\x5cu6253rr;\x5cu6\
971\x5cu0180cdi\x5cu1f\
3e\x5cu1f41\x5cu1ef8r;\
\x5cu612fo\x5cxf4\x5cu035\
2\x5cu0100ah\x5cu1f49\x5c\
u1f4b;\x5cu43b7\x5cu80\
3b\x5cxf0\x5cu40f0\x5cu01\
00mr\x5cu1f53\x5cu1f57\
l\x5cu803b\x5cxeb\x5cu40e\
bo;\x5cu60ac\x5cu0180c\
ip\x5cu1f61\x5cu1f64\x5cu\
1f67l;\x5cu4021s\x5cxf\
4\x5cu056e\x5cu0100eo\x5c\
u1f6c\x5cu1f74ctati\
o\x5cxee\x5cu0559nenti\
al\x5cxe5\x5cu0579\x5cu09\
e1\x5cu1f92\x5c0\x5cu1f9e\
\x5c0\x5cu1fa1\x5cu1fa7\x5c0\
\x5c0\x5cu1fc6\x5cu1fcc\x5c0\
\x5cu1fd3\x5c0\x5cu1fe6\x5cu\
1fea\x5cu2000\x5c0\x5cu20\
08\x5cu205allingdot\
se\x5cxf1\x5cu1e44y;\x5cu\
4444male;\x5cu6640\x5c\
u0180ilr\x5cu1fad\x5cu\
1fb3\x5cu1fc1lig;\x5cu\
8000\x5cufb03\x5cu0269\
\x5cu1fb9\x5c0\x5c0\x5cu1fbd\
g;\x5cu8000\x5cufb00ig\
;\x5cu8000\x5cufb04;\x5cu\
c000\x5cud835\x5cudd23\
lig;\x5cu8000\x5cufb01\
lig;\x5cuc000fj\x5cu01\
80alt\x5cu1fd9\x5cu1fd\
c\x5cu1fe1t;\x5cu666di\
g;\x5cu8000\x5cufb02ns\
;\x5cu65b1of;\x5cu4192\
\x5cu01f0\x5cu1fee\x5c0\x5cu\
1ff3f;\x5cuc000\x5cud8\
35\x5cudd57\x5cu0100ak\
\x5cu05bf\x5cu1ff7\x5cu01\
00;v\x5cu1ffc\x5cu1ffd\
\x5cu62d4;\x5cu6ad9art\
int;\x5cu6a0d\x5cu0100\
ao\x5cu200c\x5cu2055\x5cu\
0100cs\x5cu2011\x5cu20\
52\x5cu03b1\x5cu201a\x5cu\
2030\x5cu2038\x5cu2045\
\x5cu2048\x5c0\x5cu2050\x5cu\
03b2\x5cu2022\x5cu2025\
\x5cu2027\x5cu202a\x5cu20\
2c\x5c0\x5cu202e\x5cu803b\
\x5cxbd\x5cu40bd;\x5cu615\
3\x5cu803b\x5cxbc\x5cu40b\
c;\x5cu6155;\x5cu6159;\
\x5cu615b\x5cu01b3\x5cu20\
34\x5c0\x5cu2036;\x5cu615\
4;\x5cu6156\x5cu02b4\x5cu\
203e\x5cu2041\x5c0\x5c0\x5cu\
2043\x5cu803b\x5cxbe\x5cu\
40be;\x5cu6157;\x5cu61\
5c5;\x5cu6158\x5cu01b6\
\x5cu204c\x5c0\x5cu204e;\x5c\
u615a;\x5cu615d8;\x5cu\
615el;\x5cu6044wn;\x5c\
u6322cr;\x5cuc000\x5cu\
d835\x5cudcbb\x5cu0880\
Eabcdefgijlnorst\
v\x5cu2082\x5cu2089\x5cu2\
09f\x5cu20a5\x5cu20b0\x5c\
u20b4\x5cu20f0\x5cu20f\
5\x5cu20fa\x5cu20ff\x5cu2\
103\x5cu2112\x5cu2138\x5c\
u0317\x5cu213e\x5cu215\
2\x5cu219e\x5cu0100;l\x5c\
u064d\x5cu2087;\x5cu6a\
8c\x5cu0180cmp\x5cu209\
0\x5cu2095\x5cu209dute\
;\x5cu41f5ma\x5cu0100;\
d\x5cu209c\x5cu1cda\x5cu4\
3b3;\x5cu6a86reve;\x5c\
u411f\x5cu0100iy\x5cu2\
0aa\x5cu20aerc;\x5cu41\
1d;\x5cu4433ot;\x5cu41\
21\x5cu0200;lqs\x5cu06\
3e\x5cu0642\x5cu20bd\x5cu\
20c9\x5cu0180;qs\x5cu0\
63e\x5cu064c\x5cu20c4l\
an\x5cxf4\x5cu0665\x5cu02\
00;cdl\x5cu0665\x5cu20\
d2\x5cu20d5\x5cu20e5c;\
\x5cu6aa9ot\x5cu0100;o\
\x5cu20dc\x5cu20dd\x5cu6a\
80\x5cu0100;l\x5cu20e2\
\x5cu20e3\x5cu6a82;\x5cu6\
a84\x5cu0100;e\x5cu20e\
a\x5cu20ed\x5cuc000\x5cu2\
2db\x5cufe00s;\x5cu6a9\
4r;\x5cuc000\x5cud835\x5c\
udd24\x5cu0100;g\x5cu0\
673\x5cu061bmel;\x5cu6\
137cy;\x5cu4453\x5cu02\
00;Eaj\x5cu065a\x5cu21\
0c\x5cu210e\x5cu2110;\x5c\
u6a92;\x5cu6aa5;\x5cu6\
aa4\x5cu0200Eaes\x5cu2\
11b\x5cu211d\x5cu2129\x5c\
u2134;\x5cu6269p\x5cu0\
100;p\x5cu2123\x5cu212\
4\x5cu6a8arox\x5cxbb\x5cu\
2124\x5cu0100;q\x5cu21\
2e\x5cu212f\x5cu6a88\x5cu\
0100;q\x5cu212e\x5cu21\
1bim;\x5cu62e7pf;\x5cu\
c000\x5cud835\x5cudd58\
\x5cu0100ci\x5cu2143\x5cu\
2146r;\x5cu610am\x5cu0\
180;el\x5cu066b\x5cu21\
4e\x5cu2150;\x5cu6a8e;\
\x5cu6a90\x5cu8300>;cd\
lqr\x5cu05ee\x5cu2160\x5c\
u216a\x5cu216e\x5cu217\
3\x5cu2179\x5cu0100ci\x5c\
u2165\x5cu2167;\x5cu6a\
a7r;\x5cu6a7aot;\x5cu6\
2d7Par;\x5cu6995ues\
t;\x5cu6a7c\x5cu0280ad\
els\x5cu2184\x5cu216a\x5c\
u2190\x5cu0656\x5cu219\
b\x5cu01f0\x5cu2189\x5c0\x5c\
u218epro\x5cxf8\x5cu20\
9er;\x5cu6978q\x5cu010\
0lq\x5cu063f\x5cu2196l\
es\x5cxf3\x5cu2088i\x5cxe\
d\x5cu066b\x5cu0100en\x5c\
u21a3\x5cu21adrtneq\
q;\x5cuc000\x5cu2269\x5cu\
fe00\x5cxc5\x5cu21aa\x5cu\
0500Aabcefkosy\x5cu\
21c4\x5cu21c7\x5cu21f1\
\x5cu21f5\x5cu21fa\x5cu22\
18\x5cu221d\x5cu222f\x5cu\
2268\x5cu227dr\x5cxf2\x5c\
u03a0\x5cu0200ilmr\x5c\
u21d0\x5cu21d4\x5cu21d\
7\x5cu21dbrs\x5cxf0\x5cu1\
484f\x5cxbb\x5cu2024il\
\x5cxf4\x5cu06a9\x5cu0100\
dr\x5cu21e0\x5cu21e4cy\
;\x5cu444a\x5cu0180;cw\
\x5cu08f4\x5cu21eb\x5cu21\
efir;\x5cu6948;\x5cu61\
adar;\x5cu610firc;\x5c\
u4125\x5cu0180alr\x5cu\
2201\x5cu220e\x5cu2213\
rts\x5cu0100;u\x5cu220\
9\x5cu220a\x5cu6665it\x5c\
xbb\x5cu220alip;\x5cu6\
026con;\x5cu62b9r;\x5c\
uc000\x5cud835\x5cudd2\
5s\x5cu0100ew\x5cu2223\
\x5cu2229arow;\x5cu692\
5arow;\x5cu6926\x5cu02\
80amopr\x5cu223a\x5cu2\
23e\x5cu2243\x5cu225e\x5c\
u2263rr;\x5cu61ffth\
t;\x5cu623bk\x5cu0100l\
r\x5cu2249\x5cu2253eft\
arrow;\x5cu61a9ight\
arrow;\x5cu61aaf;\x5cu\
c000\x5cud835\x5cudd59\
bar;\x5cu6015\x5cu0180\
clt\x5cu226f\x5cu2274\x5c\
u2278r;\x5cuc000\x5cud\
835\x5cudcbdas\x5cxe8\x5c\
u21f4rok;\x5cu4127\x5c\
u0100bp\x5cu2282\x5cu2\
287ull;\x5cu6043hen\
\x5cxbb\x5cu1c5b\x5cu0ae1\
\x5cu22a3\x5c0\x5cu22aa\x5c0\
\x5cu22b8\x5cu22c5\x5cu22\
ce\x5c0\x5cu22d5\x5cu22f3\
\x5c0\x5c0\x5cu22f8\x5cu2322\
\x5cu2367\x5cu2362\x5cu23\
7f\x5c0\x5cu2386\x5cu23aa\
\x5cu23b4cute\x5cu803b\
\x5cxed\x5cu40ed\x5cu0180\
;iy\x5cu0771\x5cu22b0\x5c\
u22b5rc\x5cu803b\x5cxe\
e\x5cu40ee;\x5cu4438\x5cu\
0100cx\x5cu22bc\x5cu22\
bfy;\x5cu4435cl\x5cu80\
3b\x5cxa1\x5cu40a1\x5cu01\
00fr\x5cu039f\x5cu22c9\
;\x5cuc000\x5cud835\x5cud\
d26rave\x5cu803b\x5cxe\
c\x5cu40ec\x5cu0200;in\
o\x5cu073e\x5cu22dd\x5cu2\
2e9\x5cu22ee\x5cu0100i\
n\x5cu22e2\x5cu22e6nt;\
\x5cu6a0ct;\x5cu622dfi\
n;\x5cu69dcta;\x5cu612\
9lig;\x5cu4133\x5cu018\
0aop\x5cu22fe\x5cu231a\
\x5cu231d\x5cu0180cgt\x5c\
u2305\x5cu2308\x5cu231\
7r;\x5cu412b\x5cu0180e\
lp\x5cu071f\x5cu230f\x5cu\
2313in\x5cxe5\x5cu078e\
ar\x5cxf4\x5cu0720h;\x5cu\
4131f;\x5cu62b7ed;\x5c\
u41b5\x5cu0280;cfot\
\x5cu04f4\x5cu232c\x5cu23\
31\x5cu233d\x5cu2341ar\
e;\x5cu6105in\x5cu0100\
;t\x5cu2338\x5cu2339\x5cu\
621eie;\x5cu69dddo\x5c\
xf4\x5cu2319\x5cu0280;\
celp\x5cu0757\x5cu234c\
\x5cu2350\x5cu235b\x5cu23\
61al;\x5cu62ba\x5cu010\
0gr\x5cu2355\x5cu2359e\
r\x5cxf3\x5cu1563\x5cxe3\x5c\
u234darhk;\x5cu6a17\
rod;\x5cu6a3c\x5cu0200\
cgpt\x5cu236f\x5cu2372\
\x5cu2376\x5cu237by;\x5cu\
4451on;\x5cu412ff;\x5c\
uc000\x5cud835\x5cudd5\
aa;\x5cu43b9uest\x5cu8\
03b\x5cxbf\x5cu40bf\x5cu0\
100ci\x5cu238a\x5cu238\
fr;\x5cuc000\x5cud835\x5c\
udcben\x5cu0280;Eds\
v\x5cu04f4\x5cu239b\x5cu2\
39d\x5cu23a1\x5cu04f3;\
\x5cu62f9ot;\x5cu62f5\x5c\
u0100;v\x5cu23a6\x5cu2\
3a7\x5cu62f4;\x5cu62f3\
\x5cu0100;i\x5cu0777\x5cu\
23aelde;\x5cu4129\x5cu\
01eb\x5cu23b8\x5c0\x5cu23\
bccy;\x5cu4456l\x5cu80\
3b\x5cxef\x5cu40ef\x5cu03\
00cfmosu\x5cu23cc\x5cu\
23d7\x5cu23dc\x5cu23e1\
\x5cu23e7\x5cu23f5\x5cu01\
00iy\x5cu23d1\x5cu23d5\
rc;\x5cu4135;\x5cu4439\
r;\x5cuc000\x5cud835\x5cu\
dd27ath;\x5cu4237pf\
;\x5cuc000\x5cud835\x5cud\
d5b\x5cu01e3\x5cu23ec\x5c\
0\x5cu23f1r;\x5cuc000\x5c\
ud835\x5cudcbfrcy;\x5c\
u4458kcy;\x5cu4454\x5c\
u0400acfghjos\x5cu2\
40b\x5cu2416\x5cu2422\x5c\
u2427\x5cu242d\x5cu243\
1\x5cu2435\x5cu243bppa\
\x5cu0100;v\x5cu2413\x5cu\
2414\x5cu43ba;\x5cu43f\
0\x5cu0100ey\x5cu241b\x5c\
u2420dil;\x5cu4137;\
\x5cu443ar;\x5cuc000\x5cu\
d835\x5cudd28reen;\x5c\
u4138cy;\x5cu4445cy\
;\x5cu445cpf;\x5cuc000\
\x5cud835\x5cudd5ccr;\x5c\
uc000\x5cud835\x5cudcc\
0\x5cu0b80ABEHabcde\
fghjlmnoprstuv\x5cu\
2470\x5cu2481\x5cu2486\
\x5cu248d\x5cu2491\x5cu25\
0e\x5cu253d\x5cu255a\x5cu\
2580\x5cu264e\x5cu265e\
\x5cu2665\x5cu2679\x5cu26\
7d\x5cu269a\x5cu26b2\x5cu\
26d8\x5cu275d\x5cu2768\
\x5cu278b\x5cu27c0\x5cu28\
01\x5cu2812\x5cu0180ar\
t\x5cu2477\x5cu247a\x5cu2\
47cr\x5cxf2\x5cu09c6\x5cx\
f2\x5cu0395ail;\x5cu69\
1barr;\x5cu690e\x5cu01\
00;g\x5cu0994\x5cu248b\
;\x5cu6a8bar;\x5cu6962\
\x5cu0963\x5cu24a5\x5c0\x5cu\
24aa\x5c0\x5cu24b1\x5c0\x5c0\
\x5c0\x5c0\x5c0\x5cu24b5\x5cu24\
ba\x5c0\x5cu24c6\x5cu24c8\
\x5cu24cd\x5c0\x5cu24f9ut\
e;\x5cu413amptyv;\x5cu\
69b4ra\x5cxee\x5cu084c\
bda;\x5cu43bbg\x5cu018\
0;dl\x5cu088e\x5cu24c1\
\x5cu24c3;\x5cu6991\x5cxe\
5\x5cu088e;\x5cu6a85uo\
\x5cu803b\x5cxab\x5cu40ab\
r\x5cu0400;bfhlpst\x5c\
u0899\x5cu24de\x5cu24e\
6\x5cu24e9\x5cu24eb\x5cu2\
4ee\x5cu24f1\x5cu24f5\x5c\
u0100;f\x5cu089d\x5cu2\
4e3s;\x5cu691fs;\x5cu6\
91d\x5cxeb\x5cu2252p;\x5c\
u61abl;\x5cu6939im;\
\x5cu6973l;\x5cu61a2\x5cu\
0180;ae\x5cu24ff\x5cu2\
500\x5cu2504\x5cu6aabi\
l;\x5cu6919\x5cu0100;s\
\x5cu2509\x5cu250a\x5cu6a\
ad;\x5cuc000\x5cu2aad\x5c\
ufe00\x5cu0180abr\x5cu\
2515\x5cu2519\x5cu251d\
rr;\x5cu690crk;\x5cu67\
72\x5cu0100ak\x5cu2522\
\x5cu252cc\x5cu0100ek\x5c\
u2528\x5cu252a;\x5cu40\
7b;\x5cu405b\x5cu0100e\
s\x5cu2531\x5cu2533;\x5cu\
698bl\x5cu0100du\x5cu2\
539\x5cu253b;\x5cu698f\
;\x5cu698d\x5cu0200aeu\
y\x5cu2546\x5cu254b\x5cu2\
556\x5cu2558ron;\x5cu4\
13e\x5cu0100di\x5cu255\
0\x5cu2554il;\x5cu413c\
\x5cxec\x5cu08b0\x5cxe2\x5cu\
2529;\x5cu443b\x5cu020\
0cqrs\x5cu2563\x5cu256\
6\x5cu256d\x5cu257da;\x5c\
u6936uo\x5cu0100;r\x5c\
u0e19\x5cu1746\x5cu010\
0du\x5cu2572\x5cu2577h\
ar;\x5cu6967shar;\x5cu\
694bh;\x5cu61b2\x5cu02\
80;fgqs\x5cu258b\x5cu2\
58c\x5cu0989\x5cu25f3\x5c\
u25ff\x5cu6264t\x5cu02\
80ahlrt\x5cu2598\x5cu2\
5a4\x5cu25b7\x5cu25c2\x5c\
u25e8rrow\x5cu0100;\
t\x5cu0899\x5cu25a1a\x5cx\
e9\x5cu24f6arpoon\x5cu\
0100du\x5cu25af\x5cu25\
b4own\x5cxbb\x5cu045ap\
\x5cxbb\x5cu0966eftarr\
ows;\x5cu61c7ight\x5cu\
0180ahs\x5cu25cd\x5cu2\
5d6\x5cu25derrow\x5cu0\
100;s\x5cu08f4\x5cu08a\
7arpoon\x5cxf3\x5cu0f9\
8quigarro\x5cxf7\x5cu2\
1f0hreetimes;\x5cu6\
2cb\x5cu0180;qs\x5cu25\
8b\x5cu0993\x5cu25fala\
n\x5cxf4\x5cu09ac\x5cu028\
0;cdgs\x5cu09ac\x5cu26\
0a\x5cu260d\x5cu261d\x5cu\
2628c;\x5cu6aa8ot\x5cu\
0100;o\x5cu2614\x5cu26\
15\x5cu6a7f\x5cu0100;r\
\x5cu261a\x5cu261b\x5cu6a\
81;\x5cu6a83\x5cu0100;\
e\x5cu2622\x5cu2625\x5cuc\
000\x5cu22da\x5cufe00s\
;\x5cu6a93\x5cu0280ade\
gs\x5cu2633\x5cu2639\x5cu\
263d\x5cu2649\x5cu264b\
ppro\x5cxf8\x5cu24c6ot\
;\x5cu62d6q\x5cu0100gq\
\x5cu2643\x5cu2645\x5cxf4\
\x5cu0989gt\x5cxf2\x5cu24\
8c\x5cxf4\x5cu099bi\x5cxe\
d\x5cu09b2\x5cu0180ilr\
\x5cu2655\x5cu08e1\x5cu26\
5asht;\x5cu697c;\x5cuc\
000\x5cud835\x5cudd29\x5c\
u0100;E\x5cu099c\x5cu2\
663;\x5cu6a91\x5cu0161\
\x5cu2669\x5cu2676r\x5cu0\
100du\x5cu25b2\x5cu266\
e\x5cu0100;l\x5cu0965\x5c\
u2673;\x5cu696alk;\x5c\
u6584cy;\x5cu4459\x5cu\
0280;acht\x5cu0a48\x5c\
u2688\x5cu268b\x5cu269\
1\x5cu2696r\x5cxf2\x5cu25\
c1orne\x5cxf2\x5cu1d08\
ard;\x5cu696bri;\x5cu6\
5fa\x5cu0100io\x5cu269\
f\x5cu26a4dot;\x5cu414\
0ust\x5cu0100;a\x5cu26\
ac\x5cu26ad\x5cu63b0ch\
e\x5cxbb\x5cu26ad\x5cu020\
0Eaes\x5cu26bb\x5cu26b\
d\x5cu26c9\x5cu26d4;\x5cu\
6268p\x5cu0100;p\x5cu2\
6c3\x5cu26c4\x5cu6a89r\
ox\x5cxbb\x5cu26c4\x5cu01\
00;q\x5cu26ce\x5cu26cf\
\x5cu6a87\x5cu0100;q\x5cu\
26ce\x5cu26bbim;\x5cu6\
2e6\x5cu0400abnoptw\
z\x5cu26e9\x5cu26f4\x5cu2\
6f7\x5cu271a\x5cu272f\x5c\
u2741\x5cu2747\x5cu275\
0\x5cu0100nr\x5cu26ee\x5c\
u26f1g;\x5cu67ecr;\x5c\
u61fdr\x5cxeb\x5cu08c1\
g\x5cu0180lmr\x5cu26ff\
\x5cu270d\x5cu2714eft\x5c\
u0100ar\x5cu09e6\x5cu2\
707ight\x5cxe1\x5cu09f\
2apsto;\x5cu67fcigh\
t\x5cxe1\x5cu09fdparro\
w\x5cu0100lr\x5cu2725\x5c\
u2729ef\x5cxf4\x5cu24e\
dight;\x5cu61ac\x5cu01\
80afl\x5cu2736\x5cu273\
9\x5cu273dr;\x5cu6985;\
\x5cuc000\x5cud835\x5cudd\
5dus;\x5cu6a2dimes;\
\x5cu6a34\x5cu0161\x5cu27\
4b\x5cu274fst;\x5cu621\
7\x5cxe1\x5cu134e\x5cu018\
0;ef\x5cu2757\x5cu2758\
\x5cu1800\x5cu65cange\x5c\
xbb\x5cu2758ar\x5cu010\
0;l\x5cu2764\x5cu2765\x5c\
u4028t;\x5cu6993\x5cu0\
280achmt\x5cu2773\x5cu\
2776\x5cu277c\x5cu2785\
\x5cu2787r\x5cxf2\x5cu08a\
8orne\x5cxf2\x5cu1d8ca\
r\x5cu0100;d\x5cu0f98\x5c\
u2783;\x5cu696d;\x5cu6\
00eri;\x5cu62bf\x5cu03\
00achiqt\x5cu2798\x5cu\
279d\x5cu0a40\x5cu27a2\
\x5cu27ae\x5cu27bbquo;\
\x5cu6039r;\x5cuc000\x5cu\
d835\x5cudcc1m\x5cu018\
0;eg\x5cu09b2\x5cu27aa\
\x5cu27ac;\x5cu6a8d;\x5cu\
6a8f\x5cu0100bu\x5cu25\
2a\x5cu27b3o\x5cu0100;\
r\x5cu0e1f\x5cu27b9;\x5cu\
601arok;\x5cu4142\x5cu\
8400<;cdhilqr\x5cu0\
82b\x5cu27d2\x5cu2639\x5c\
u27dc\x5cu27e0\x5cu27e\
5\x5cu27ea\x5cu27f0\x5cu0\
100ci\x5cu27d7\x5cu27d\
9;\x5cu6aa6r;\x5cu6a79\
re\x5cxe5\x5cu25f2mes;\
\x5cu62c9arr;\x5cu6976\
uest;\x5cu6a7b\x5cu010\
0Pi\x5cu27f5\x5cu27f9a\
r;\x5cu6996\x5cu0180;e\
f\x5cu2800\x5cu092d\x5cu1\
81b\x5cu65c3r\x5cu0100\
du\x5cu2807\x5cu280dsh\
ar;\x5cu694ahar;\x5cu6\
966\x5cu0100en\x5cu281\
7\x5cu2821rtneqq;\x5cu\
c000\x5cu2268\x5cufe00\
\x5cxc5\x5cu281e\x5cu0700\
Dacdefhilnopsu\x5cu\
2840\x5cu2845\x5cu2882\
\x5cu288e\x5cu2893\x5cu28\
a0\x5cu28a5\x5cu28a8\x5cu\
28da\x5cu28e2\x5cu28e4\
\x5cu0a83\x5cu28f3\x5cu29\
02Dot;\x5cu623a\x5cu02\
00clpr\x5cu284e\x5cu28\
52\x5cu2863\x5cu287dr\x5c\
u803b\x5cxaf\x5cu40af\x5c\
u0100et\x5cu2857\x5cu2\
859;\x5cu6642\x5cu0100\
;e\x5cu285e\x5cu285f\x5cu\
6720se\x5cxbb\x5cu285f\
\x5cu0100;s\x5cu103b\x5cu\
2868to\x5cu0200;dlu\
\x5cu103b\x5cu2873\x5cu28\
77\x5cu287bow\x5cxee\x5cu\
048cef\x5cxf4\x5cu090f\
\x5cxf0\x5cu13d1ker;\x5cu\
65ae\x5cu0100oy\x5cu28\
87\x5cu288cmma;\x5cu6a\
29;\x5cu443cash;\x5cu6\
014asuredangle\x5cx\
bb\x5cu1626r;\x5cuc000\
\x5cud835\x5cudd2ao;\x5cu\
6127\x5cu0180cdn\x5cu2\
8af\x5cu28b4\x5cu28c9r\
o\x5cu803b\x5cxb5\x5cu40b\
5\x5cu0200;acd\x5cu146\
4\x5cu28bd\x5cu28c0\x5cu2\
8c4s\x5cxf4\x5cu16a7ir\
;\x5cu6af0ot\x5cu80bb\x5c\
xb7\x5cu01b5us\x5cu018\
0;bd\x5cu28d2\x5cu1903\
\x5cu28d3\x5cu6212\x5cu01\
00;u\x5cu1d3c\x5cu28d8\
;\x5cu6a2a\x5cu0163\x5cu2\
8de\x5cu28e1p;\x5cu6ad\
b\x5cxf2\x5cu2212\x5cxf0\x5c\
u0a81\x5cu0100dp\x5cu2\
8e9\x5cu28eeels;\x5cu6\
2a7f;\x5cuc000\x5cud83\
5\x5cudd5e\x5cu0100ct\x5c\
u28f8\x5cu28fdr;\x5cuc\
000\x5cud835\x5cudcc2p\
os\x5cxbb\x5cu159d\x5cu01\
80;lm\x5cu2909\x5cu290\
a\x5cu290d\x5cu43bctim\
ap;\x5cu62b8\x5cu0c00G\
LRVabcdefghijlmo\
prstuvw\x5cu2942\x5cu2\
953\x5cu297e\x5cu2989\x5c\
u2998\x5cu29da\x5cu29e\
9\x5cu2a15\x5cu2a1a\x5cu2\
a58\x5cu2a5d\x5cu2a83\x5c\
u2a95\x5cu2aa4\x5cu2aa\
8\x5cu2b04\x5cu2b07\x5cu2\
b44\x5cu2b7f\x5cu2bae\x5c\
u2c34\x5cu2c67\x5cu2c7\
c\x5cu2ce9\x5cu0100gt\x5c\
u2947\x5cu294b;\x5cuc0\
00\x5cu22d9\x5cu0338\x5cu\
0100;v\x5cu2950\x5cu0b\
cf\x5cuc000\x5cu226b\x5cu\
20d2\x5cu0180elt\x5cu2\
95a\x5cu2972\x5cu2976f\
t\x5cu0100ar\x5cu2961\x5c\
u2967rrow;\x5cu61cd\
ightarrow;\x5cu61ce\
;\x5cuc000\x5cu22d8\x5cu0\
338\x5cu0100;v\x5cu297\
b\x5cu0c47\x5cuc000\x5cu2\
26a\x5cu20d2ightarr\
ow;\x5cu61cf\x5cu0100D\
d\x5cu298e\x5cu2993ash\
;\x5cu62afash;\x5cu62a\
e\x5cu0280bcnpt\x5cu29\
a3\x5cu29a7\x5cu29ac\x5cu\
29b1\x5cu29ccla\x5cxbb\
\x5cu02deute;\x5cu4144\
g;\x5cuc000\x5cu2220\x5cu\
20d2\x5cu0280;Eiop\x5c\
u0d84\x5cu29bc\x5cu29c\
0\x5cu29c5\x5cu29c8;\x5cu\
c000\x5cu2a70\x5cu0338\
d;\x5cuc000\x5cu224b\x5cu\
0338s;\x5cu4149ro\x5cx\
f8\x5cu0d84ur\x5cu0100\
;a\x5cu29d3\x5cu29d4\x5cu\
666el\x5cu0100;s\x5cu2\
9d3\x5cu0b38\x5cu01f3\x5c\
u29df\x5c0\x5cu29e3p\x5cu\
80bb\x5cxa0\x5cu0b37mp\
\x5cu0100;e\x5cu0bf9\x5cu\
0c00\x5cu0280aeouy\x5c\
u29f4\x5cu29fe\x5cu2a0\
3\x5cu2a10\x5cu2a13\x5cu0\
1f0\x5cu29f9\x5c0\x5cu29f\
b;\x5cu6a43on;\x5cu414\
8dil;\x5cu4146ng\x5cu0\
100;d\x5cu0d7e\x5cu2a0\
aot;\x5cuc000\x5cu2a6d\
\x5cu0338p;\x5cu6a42;\x5c\
u443dash;\x5cu6013\x5c\
u0380;Aadqsx\x5cu0b\
92\x5cu2a29\x5cu2a2d\x5cu\
2a3b\x5cu2a41\x5cu2a45\
\x5cu2a50rr;\x5cu61d7r\
\x5cu0100hr\x5cu2a33\x5cu\
2a36k;\x5cu6924\x5cu01\
00;o\x5cu13f2\x5cu13f0\
ot;\x5cuc000\x5cu2250\x5c\
u0338ui\x5cxf6\x5cu0b6\
3\x5cu0100ei\x5cu2a4a\x5c\
u2a4ear;\x5cu6928\x5cx\
ed\x5cu0b98ist\x5cu010\
0;s\x5cu0ba0\x5cu0b9fr\
;\x5cuc000\x5cud835\x5cud\
d2b\x5cu0200Eest\x5cu0\
bc5\x5cu2a66\x5cu2a79\x5c\
u2a7c\x5cu0180;qs\x5cu\
0bbc\x5cu2a6d\x5cu0be1\
\x5cu0180;qs\x5cu0bbc\x5c\
u0bc5\x5cu2a74lan\x5cx\
f4\x5cu0be2i\x5cxed\x5cu0\
bea\x5cu0100;r\x5cu0bb\
6\x5cu2a81\x5cxbb\x5cu0bb\
7\x5cu0180Aap\x5cu2a8a\
\x5cu2a8d\x5cu2a91r\x5cxf\
2\x5cu2971rr;\x5cu61ae\
ar;\x5cu6af2\x5cu0180;\
sv\x5cu0f8d\x5cu2a9c\x5cu\
0f8c\x5cu0100;d\x5cu2a\
a1\x5cu2aa2\x5cu62fc;\x5c\
u62facy;\x5cu445a\x5cu\
0380AEadest\x5cu2ab\
7\x5cu2aba\x5cu2abe\x5cu2\
ac2\x5cu2ac5\x5cu2af6\x5c\
u2af9r\x5cxf2\x5cu2966\
;\x5cuc000\x5cu2266\x5cu0\
338rr;\x5cu619ar;\x5cu\
6025\x5cu0200;fqs\x5cu\
0c3b\x5cu2ace\x5cu2ae3\
\x5cu2aeft\x5cu0100ar\x5c\
u2ad4\x5cu2ad9rro\x5cx\
f7\x5cu2ac1ightarro\
\x5cxf7\x5cu2a90\x5cu0180\
;qs\x5cu0c3b\x5cu2aba\x5c\
u2aealan\x5cxf4\x5cu0c\
55\x5cu0100;s\x5cu0c55\
\x5cu2af4\x5cxbb\x5cu0c36\
i\x5cxed\x5cu0c5d\x5cu010\
0;r\x5cu0c35\x5cu2afei\
\x5cu0100;e\x5cu0c1a\x5cu\
0c25i\x5cxe4\x5cu0d90\x5c\
u0100pt\x5cu2b0c\x5cu2\
b11f;\x5cuc000\x5cud83\
5\x5cudd5f\x5cu8180\x5cxa\
c;in\x5cu2b19\x5cu2b1a\
\x5cu2b36\x5cu40acn\x5cu0\
200;Edv\x5cu0b89\x5cu2\
b24\x5cu2b28\x5cu2b2e;\
\x5cuc000\x5cu22f9\x5cu03\
38ot;\x5cuc000\x5cu22f\
5\x5cu0338\x5cu01e1\x5cu0\
b89\x5cu2b33\x5cu2b35;\
\x5cu62f7;\x5cu62f6i\x5cu\
0100;v\x5cu0cb8\x5cu2b\
3c\x5cu01e1\x5cu0cb8\x5cu\
2b41\x5cu2b43;\x5cu62f\
e;\x5cu62fd\x5cu0180ao\
r\x5cu2b4b\x5cu2b63\x5cu2\
b69r\x5cu0200;ast\x5cu\
0b7b\x5cu2b55\x5cu2b5a\
\x5cu2b5flle\x5cxec\x5cu0\
b7bl;\x5cuc000\x5cu2af\
d\x5cu20e5;\x5cuc000\x5cu\
2202\x5cu0338lint;\x5c\
u6a14\x5cu0180;ce\x5cu\
0c92\x5cu2b70\x5cu2b73\
u\x5cxe5\x5cu0ca5\x5cu010\
0;c\x5cu0c98\x5cu2b78\x5c\
u0100;e\x5cu0c92\x5cu2\
b7d\x5cxf1\x5cu0c98\x5cu0\
200Aait\x5cu2b88\x5cu2\
b8b\x5cu2b9d\x5cu2ba7r\
\x5cxf2\x5cu2988rr\x5cu01\
80;cw\x5cu2b94\x5cu2b9\
5\x5cu2b99\x5cu619b;\x5cu\
c000\x5cu2933\x5cu0338\
;\x5cuc000\x5cu219d\x5cu0\
338ghtarrow\x5cxbb\x5c\
u2b95ri\x5cu0100;e\x5c\
u0ccb\x5cu0cd6\x5cu038\
0chimpqu\x5cu2bbd\x5cu\
2bcd\x5cu2bd9\x5cu2b04\
\x5cu0b78\x5cu2be4\x5cu2b\
ef\x5cu0200;cer\x5cu0d\
32\x5cu2bc6\x5cu0d37\x5cu\
2bc9u\x5cxe5\x5cu0d45;\
\x5cuc000\x5cud835\x5cudc\
c3ort\x5cu026d\x5cu2b0\
5\x5c0\x5c0\x5cu2bd6ar\x5cxe\
1\x5cu2b56m\x5cu0100;e\
\x5cu0d6e\x5cu2bdf\x5cu01\
00;q\x5cu0d74\x5cu0d73\
su\x5cu0100bp\x5cu2beb\
\x5cu2bed\x5cxe5\x5cu0cf8\
\x5cxe5\x5cu0d0b\x5cu0180\
bcp\x5cu2bf6\x5cu2c11\x5c\
u2c19\x5cu0200;Ees\x5c\
u2bff\x5cu2c00\x5cu0d2\
2\x5cu2c04\x5cu6284;\x5cu\
c000\x5cu2ac5\x5cu0338\
et\x5cu0100;e\x5cu0d1b\
\x5cu2c0bq\x5cu0100;q\x5c\
u0d23\x5cu2c00c\x5cu01\
00;e\x5cu0d32\x5cu2c17\
\x5cxf1\x5cu0d38\x5cu0200\
;Ees\x5cu2c22\x5cu2c23\
\x5cu0d5f\x5cu2c27\x5cu62\
85;\x5cuc000\x5cu2ac6\x5c\
u0338et\x5cu0100;e\x5c\
u0d58\x5cu2c2eq\x5cu01\
00;q\x5cu0d60\x5cu2c23\
\x5cu0200gilr\x5cu2c3d\
\x5cu2c3f\x5cu2c45\x5cu2c\
47\x5cxec\x5cu0bd7lde\x5c\
u803b\x5cxf1\x5cu40f1\x5c\
xe7\x5cu0c43iangle\x5c\
u0100lr\x5cu2c52\x5cu2\
c5ceft\x5cu0100;e\x5cu\
0c1a\x5cu2c5a\x5cxf1\x5cu\
0c26ight\x5cu0100;e\
\x5cu0ccb\x5cu2c65\x5cxf1\
\x5cu0cd7\x5cu0100;m\x5cu\
2c6c\x5cu2c6d\x5cu43bd\
\x5cu0180;es\x5cu2c74\x5c\
u2c75\x5cu2c79\x5cu402\
3ro;\x5cu6116p;\x5cu60\
07\x5cu0480DHadgilr\
s\x5cu2c8f\x5cu2c94\x5cu2\
c99\x5cu2c9e\x5cu2ca3\x5c\
u2cb0\x5cu2cb6\x5cu2cd\
3\x5cu2ce3ash;\x5cu62a\
darr;\x5cu6904p;\x5cuc\
000\x5cu224d\x5cu20d2a\
sh;\x5cu62ac\x5cu0100e\
t\x5cu2ca8\x5cu2cac;\x5cu\
c000\x5cu2265\x5cu20d2\
;\x5cuc000>\x5cu20d2nf\
in;\x5cu69de\x5cu0180A\
et\x5cu2cbd\x5cu2cc1\x5cu\
2cc5rr;\x5cu6902;\x5cu\
c000\x5cu2264\x5cu20d2\
\x5cu0100;r\x5cu2cca\x5cu\
2ccd\x5cuc000<\x5cu20d\
2ie;\x5cuc000\x5cu22b4\
\x5cu20d2\x5cu0100At\x5cu\
2cd8\x5cu2cdcrr;\x5cu6\
903rie;\x5cuc000\x5cu2\
2b5\x5cu20d2im;\x5cuc0\
00\x5cu223c\x5cu20d2\x5cu\
0180Aan\x5cu2cf0\x5cu2\
cf4\x5cu2d02rr;\x5cu61\
d6r\x5cu0100hr\x5cu2cf\
a\x5cu2cfdk;\x5cu6923\x5c\
u0100;o\x5cu13e7\x5cu1\
3e5ear;\x5cu6927\x5cu1\
253\x5cu1a95\x5c0\x5c0\x5c0\x5c\
0\x5c0\x5c0\x5c0\x5c0\x5c0\x5c0\x5c0\x5c\
0\x5c0\x5cu2d2d\x5c0\x5cu2d3\
8\x5cu2d48\x5cu2d60\x5cu2\
d65\x5cu2d72\x5cu2d84\x5c\
u1b07\x5c0\x5c0\x5cu2d8d\x5c\
u2dab\x5c0\x5cu2dc8\x5cu2\
dce\x5c0\x5cu2ddc\x5cu2e1\
9\x5cu2e2b\x5cu2e3e\x5cu2\
e43\x5cu0100cs\x5cu2d3\
1\x5cu1a97ute\x5cu803b\
\x5cxf3\x5cu40f3\x5cu0100\
iy\x5cu2d3c\x5cu2d45r\x5c\
u0100;c\x5cu1a9e\x5cu2\
d42\x5cu803b\x5cxf4\x5cu4\
0f4;\x5cu443e\x5cu0280\
abios\x5cu1aa0\x5cu2d5\
2\x5cu2d57\x5cu01c8\x5cu2\
d5alac;\x5cu4151v;\x5c\
u6a38old;\x5cu69bcl\
ig;\x5cu4153\x5cu0100c\
r\x5cu2d69\x5cu2d6dir;\
\x5cu69bf;\x5cuc000\x5cud\
835\x5cudd2c\x5cu036f\x5c\
u2d79\x5c0\x5c0\x5cu2d7c\x5c\
0\x5cu2d82n;\x5cu42dba\
ve\x5cu803b\x5cxf2\x5cu40\
f2;\x5cu69c1\x5cu0100b\
m\x5cu2d88\x5cu0df4ar;\
\x5cu69b5\x5cu0200acit\
\x5cu2d95\x5cu2d98\x5cu2d\
a5\x5cu2da8r\x5cxf2\x5cu1\
a80\x5cu0100ir\x5cu2d9\
d\x5cu2da0r;\x5cu69beo\
ss;\x5cu69bbn\x5cxe5\x5cu\
0e52;\x5cu69c0\x5cu018\
0aei\x5cu2db1\x5cu2db5\
\x5cu2db9cr;\x5cu414dg\
a;\x5cu43c9\x5cu0180cd\
n\x5cu2dc0\x5cu2dc5\x5cu0\
1cdron;\x5cu43bf;\x5cu\
69b6pf;\x5cuc000\x5cud\
835\x5cudd60\x5cu0180a\
el\x5cu2dd4\x5cu2dd7\x5cu\
01d2r;\x5cu69b7rp;\x5c\
u69b9\x5cu0380;adio\
sv\x5cu2dea\x5cu2deb\x5cu\
2dee\x5cu2e08\x5cu2e0d\
\x5cu2e10\x5cu2e16\x5cu62\
28r\x5cxf2\x5cu1a86\x5cu0\
200;efm\x5cu2df7\x5cu2\
df8\x5cu2e02\x5cu2e05\x5c\
u6a5dr\x5cu0100;o\x5cu\
2dfe\x5cu2dff\x5cu6134\
f\x5cxbb\x5cu2dff\x5cu803\
b\x5cxaa\x5cu40aa\x5cu803\
b\x5cxba\x5cu40bagof;\x5c\
u62b6r;\x5cu6a56lop\
e;\x5cu6a57;\x5cu6a5b\x5c\
u0180clo\x5cu2e1f\x5cu\
2e21\x5cu2e27\x5cxf2\x5cu\
2e01ash\x5cu803b\x5cxf\
8\x5cu40f8l;\x5cu6298i\
\x5cu016c\x5cu2e2f\x5cu2e\
34de\x5cu803b\x5cxf5\x5cu\
40f5es\x5cu0100;a\x5cu\
01db\x5cu2e3as;\x5cu6a\
36ml\x5cu803b\x5cxf6\x5cu\
40f6bar;\x5cu633d\x5cu\
0ae1\x5cu2e5e\x5c0\x5cu2e\
7d\x5c0\x5cu2e80\x5cu2e9d\
\x5c0\x5cu2ea2\x5cu2eb9\x5c0\
\x5c0\x5cu2ecb\x5cu0e9c\x5c0\
\x5cu2f13\x5c0\x5c0\x5cu2f2b\
\x5cu2fbc\x5c0\x5cu2fc8r\x5c\
u0200;ast\x5cu0403\x5c\
u2e67\x5cu2e72\x5cu0e8\
5\x5cu8100\x5cxb6;l\x5cu2\
e6d\x5cu2e6e\x5cu40b6l\
e\x5cxec\x5cu0403\x5cu026\
9\x5cu2e78\x5c0\x5c0\x5cu2e7\
bm;\x5cu6af3;\x5cu6afd\
y;\x5cu443fr\x5cu0280c\
impt\x5cu2e8b\x5cu2e8f\
\x5cu2e93\x5cu1865\x5cu2e\
97nt;\x5cu4025od;\x5cu\
402eil;\x5cu6030enk\
;\x5cu6031r;\x5cuc000\x5c\
ud835\x5cudd2d\x5cu018\
0imo\x5cu2ea8\x5cu2eb0\
\x5cu2eb4\x5cu0100;v\x5cu\
2ead\x5cu2eae\x5cu43c6\
;\x5cu43d5ma\x5cxf4\x5cu0\
a76ne;\x5cu660e\x5cu01\
80;tv\x5cu2ebf\x5cu2ec\
0\x5cu2ec8\x5cu43c0chf\
ork\x5cxbb\x5cu1ffd;\x5cu\
43d6\x5cu0100au\x5cu2e\
cf\x5cu2edfn\x5cu0100c\
k\x5cu2ed5\x5cu2eddk\x5cu\
0100;h\x5cu21f4\x5cu2e\
db;\x5cu610e\x5cxf6\x5cu2\
1f4s\x5cu0480;abcde\
mst\x5cu2ef3\x5cu2ef4\x5c\
u1908\x5cu2ef9\x5cu2ef\
d\x5cu2f04\x5cu2f06\x5cu2\
f0a\x5cu2f0e\x5cu402bc\
ir;\x5cu6a23ir;\x5cu6a\
22\x5cu0100ou\x5cu1d40\
\x5cu2f02;\x5cu6a25;\x5cu\
6a72n\x5cu80bb\x5cxb1\x5c\
u0e9dim;\x5cu6a26wo\
;\x5cu6a27\x5cu0180ipu\
\x5cu2f19\x5cu2f20\x5cu2f\
25ntint;\x5cu6a15f;\
\x5cuc000\x5cud835\x5cudd\
61nd\x5cu803b\x5cxa3\x5cu\
40a3\x5cu0500;Eacei\
nosu\x5cu0ec8\x5cu2f3f\
\x5cu2f41\x5cu2f44\x5cu2f\
47\x5cu2f81\x5cu2f89\x5cu\
2f92\x5cu2f7e\x5cu2fb6\
;\x5cu6ab3p;\x5cu6ab7u\
\x5cxe5\x5cu0ed9\x5cu0100\
;c\x5cu0ece\x5cu2f4c\x5cu\
0300;acens\x5cu0ec8\
\x5cu2f59\x5cu2f5f\x5cu2f\
66\x5cu2f68\x5cu2f7epp\
ro\x5cxf8\x5cu2f43urly\
e\x5cxf1\x5cu0ed9\x5cxf1\x5c\
u0ece\x5cu0180aes\x5cu\
2f6f\x5cu2f76\x5cu2f7a\
pprox;\x5cu6ab9qq;\x5c\
u6ab5im;\x5cu62e8i\x5c\
xed\x5cu0edfme\x5cu010\
0;s\x5cu2f88\x5cu0eae\x5c\
u6032\x5cu0180Eas\x5cu\
2f78\x5cu2f90\x5cu2f7a\
\x5cxf0\x5cu2f75\x5cu0180\
dfp\x5cu0eec\x5cu2f99\x5c\
u2faf\x5cu0180als\x5cu\
2fa0\x5cu2fa5\x5cu2faa\
lar;\x5cu632eine;\x5cu\
6312urf;\x5cu6313\x5cu\
0100;t\x5cu0efb\x5cu2f\
b4\x5cxef\x5cu0efbrel;\
\x5cu62b0\x5cu0100ci\x5cu\
2fc0\x5cu2fc5r;\x5cuc0\
00\x5cud835\x5cudcc5;\x5c\
u43c8ncsp;\x5cu6008\
\x5cu0300fiopsu\x5cu2f\
da\x5cu22e2\x5cu2fdf\x5cu\
2fe5\x5cu2feb\x5cu2ff1\
r;\x5cuc000\x5cud835\x5cu\
dd2epf;\x5cuc000\x5cud\
835\x5cudd62rime;\x5cu\
6057cr;\x5cuc000\x5cud\
835\x5cudcc6\x5cu0180a\
eo\x5cu2ff8\x5cu3009\x5cu\
3013t\x5cu0100ei\x5cu2\
ffe\x5cu3005rnion\x5cx\
f3\x5cu06b0nt;\x5cu6a1\
6st\x5cu0100;e\x5cu301\
0\x5cu3011\x5cu403f\x5cxf\
1\x5cu1f19\x5cxf4\x5cu0f1\
4\x5cu0a80ABHabcdef\
hilmnoprstux\x5cu30\
40\x5cu3051\x5cu3055\x5cu\
3059\x5cu30e0\x5cu310e\
\x5cu312b\x5cu3147\x5cu31\
62\x5cu3172\x5cu318e\x5cu\
3206\x5cu3215\x5cu3224\
\x5cu3229\x5cu3258\x5cu32\
6e\x5cu3272\x5cu3290\x5cu\
32b0\x5cu32b7\x5cu0180\
art\x5cu3047\x5cu304a\x5c\
u304cr\x5cxf2\x5cu10b3\
\x5cxf2\x5cu03ddail;\x5cu\
691car\x5cxf2\x5cu1c65\
ar;\x5cu6964\x5cu0380c\
denqrt\x5cu3068\x5cu30\
75\x5cu3078\x5cu307f\x5cu\
308f\x5cu3094\x5cu30cc\
\x5cu0100eu\x5cu306d\x5cu\
3071;\x5cuc000\x5cu223\
d\x5cu0331te;\x5cu4155\
i\x5cxe3\x5cu116emptyv\
;\x5cu69b3g\x5cu0200;d\
el\x5cu0fd1\x5cu3089\x5cu\
308b\x5cu308d;\x5cu699\
2;\x5cu69a5\x5cxe5\x5cu0f\
d1uo\x5cu803b\x5cxbb\x5cu\
40bbr\x5cu0580;abcf\
hlpstw\x5cu0fdc\x5cu30\
ac\x5cu30af\x5cu30b7\x5cu\
30b9\x5cu30bc\x5cu30be\
\x5cu30c0\x5cu30c3\x5cu30\
c7\x5cu30cap;\x5cu6975\
\x5cu0100;f\x5cu0fe0\x5cu\
30b4s;\x5cu6920;\x5cu6\
933s;\x5cu691e\x5cxeb\x5c\
u225d\x5cxf0\x5cu272el\
;\x5cu6945im;\x5cu6974\
l;\x5cu61a3;\x5cu619d\x5c\
u0100ai\x5cu30d1\x5cu3\
0d5il;\x5cu691ao\x5cu0\
100;n\x5cu30db\x5cu30d\
c\x5cu6236al\x5cxf3\x5cu0\
f1e\x5cu0180abr\x5cu30\
e7\x5cu30ea\x5cu30eer\x5c\
xf2\x5cu17e5rk;\x5cu67\
73\x5cu0100ak\x5cu30f3\
\x5cu30fdc\x5cu0100ek\x5c\
u30f9\x5cu30fb;\x5cu40\
7d;\x5cu405d\x5cu0100e\
s\x5cu3102\x5cu3104;\x5cu\
698cl\x5cu0100du\x5cu3\
10a\x5cu310c;\x5cu698e\
;\x5cu6990\x5cu0200aeu\
y\x5cu3117\x5cu311c\x5cu3\
127\x5cu3129ron;\x5cu4\
159\x5cu0100di\x5cu312\
1\x5cu3125il;\x5cu4157\
\x5cxec\x5cu0ff2\x5cxe2\x5cu\
30fa;\x5cu4440\x5cu020\
0clqs\x5cu3134\x5cu313\
7\x5cu313d\x5cu3144a;\x5c\
u6937dhar;\x5cu6969\
uo\x5cu0100;r\x5cu020e\
\x5cu020dh;\x5cu61b3\x5cu\
0180acg\x5cu314e\x5cu3\
15f\x5cu0f44l\x5cu0200\
;ips\x5cu0f78\x5cu3158\
\x5cu315b\x5cu109cn\x5cxe\
5\x5cu10bbar\x5cxf4\x5cu0\
fa9t;\x5cu65ad\x5cu018\
0ilr\x5cu3169\x5cu1023\
\x5cu316esht;\x5cu697d\
;\x5cuc000\x5cud835\x5cud\
d2f\x5cu0100ao\x5cu317\
7\x5cu3186r\x5cu0100du\
\x5cu317d\x5cu317f\x5cxbb\
\x5cu047b\x5cu0100;l\x5cu\
1091\x5cu3184;\x5cu696\
c\x5cu0100;v\x5cu318b\x5c\
u318c\x5cu43c1;\x5cu43\
f1\x5cu0180gns\x5cu319\
5\x5cu31f9\x5cu31fcht\x5c\
u0300ahlrst\x5cu31a\
4\x5cu31b0\x5cu31c2\x5cu3\
1d8\x5cu31e4\x5cu31eer\
row\x5cu0100;t\x5cu0fd\
c\x5cu31ada\x5cxe9\x5cu30\
c8arpoon\x5cu0100du\
\x5cu31bb\x5cu31bfow\x5cx\
ee\x5cu317ep\x5cxbb\x5cu1\
092eft\x5cu0100ah\x5cu\
31ca\x5cu31d0rrow\x5cx\
f3\x5cu0feaarpoon\x5cx\
f3\x5cu0551ightarro\
ws;\x5cu61c9quigarr\
o\x5cxf7\x5cu30cbhreet\
imes;\x5cu62ccg;\x5cu4\
2daingdotse\x5cxf1\x5c\
u1f32\x5cu0180ahm\x5cu\
320d\x5cu3210\x5cu3213\
r\x5cxf2\x5cu0feaa\x5cxf2\
\x5cu0551;\x5cu600fous\
t\x5cu0100;a\x5cu321e\x5c\
u321f\x5cu63b1che\x5cx\
bb\x5cu321fmid;\x5cu6a\
ee\x5cu0200abpt\x5cu32\
32\x5cu323d\x5cu3240\x5cu\
3252\x5cu0100nr\x5cu32\
37\x5cu323ag;\x5cu67ed\
r;\x5cu61fer\x5cxeb\x5cu1\
003\x5cu0180afl\x5cu32\
47\x5cu324a\x5cu324er;\
\x5cu6986;\x5cuc000\x5cud\
835\x5cudd63us;\x5cu6a\
2eimes;\x5cu6a35\x5cu0\
100ap\x5cu325d\x5cu326\
7r\x5cu0100;g\x5cu3263\
\x5cu3264\x5cu4029t;\x5cu\
6994olint;\x5cu6a12\
ar\x5cxf2\x5cu31e3\x5cu02\
00achq\x5cu327b\x5cu32\
80\x5cu10bc\x5cu3285qu\
o;\x5cu603ar;\x5cuc000\
\x5cud835\x5cudcc7\x5cu01\
00bu\x5cu30fb\x5cu328a\
o\x5cu0100;r\x5cu0214\x5c\
u0213\x5cu0180hir\x5cu\
3297\x5cu329b\x5cu32a0\
re\x5cxe5\x5cu31f8mes;\
\x5cu62cai\x5cu0200;ef\
l\x5cu32aa\x5cu1059\x5cu1\
821\x5cu32ab\x5cu65b9t\
ri;\x5cu69celuhar;\x5c\
u6968;\x5cu611e\x5cu0d\
61\x5cu32d5\x5cu32db\x5cu\
32df\x5cu332c\x5cu3338\
\x5cu3371\x5c0\x5cu337a\x5cu\
33a4\x5c0\x5c0\x5cu33ec\x5cu\
33f0\x5c0\x5cu3428\x5cu34\
48\x5cu345a\x5cu34ad\x5cu\
34b1\x5cu34ca\x5cu34f1\
\x5c0\x5cu3616\x5c0\x5c0\x5cu36\
33cute;\x5cu415bqu\x5c\
xef\x5cu27ba\x5cu0500;\
Eaceinpsy\x5cu11ed\x5c\
u32f3\x5cu32f5\x5cu32f\
f\x5cu3302\x5cu330b\x5cu3\
30f\x5cu331f\x5cu3326\x5c\
u3329;\x5cu6ab4\x5cu01\
f0\x5cu32fa\x5c0\x5cu32fc\
;\x5cu6ab8on;\x5cu4161\
u\x5cxe5\x5cu11fe\x5cu010\
0;d\x5cu11f3\x5cu3307i\
l;\x5cu415frc;\x5cu415\
d\x5cu0180Eas\x5cu3316\
\x5cu3318\x5cu331b;\x5cu6\
ab6p;\x5cu6abaim;\x5cu\
62e9olint;\x5cu6a13\
i\x5cxed\x5cu1204;\x5cu44\
41ot\x5cu0180;be\x5cu3\
334\x5cu1d47\x5cu3335\x5c\
u62c5;\x5cu6a66\x5cu03\
80Aacmstx\x5cu3346\x5c\
u334a\x5cu3357\x5cu335\
b\x5cu335e\x5cu3363\x5cu3\
36drr;\x5cu61d8r\x5cu0\
100hr\x5cu3350\x5cu335\
2\x5cxeb\x5cu2228\x5cu010\
0;o\x5cu0a36\x5cu0a34t\
\x5cu803b\x5cxa7\x5cu40a7\
i;\x5cu403bwar;\x5cu69\
29m\x5cu0100in\x5cu336\
9\x5cxf0nu\x5cxf3\x5cxf1t\
;\x5cu6736r\x5cu0100;o\
\x5cu3376\x5cu2055\x5cuc0\
00\x5cud835\x5cudd30\x5cu\
0200acoy\x5cu3382\x5cu\
3386\x5cu3391\x5cu33a0\
rp;\x5cu666f\x5cu0100h\
y\x5cu338b\x5cu338fcy;\
\x5cu4449;\x5cu4448rt\x5c\
u026d\x5cu3399\x5c0\x5c0\x5c\
u339ci\x5cxe4\x5cu1464\
ara\x5cxec\x5cu2e6f\x5cu8\
03b\x5cxad\x5cu40ad\x5cu0\
100gm\x5cu33a8\x5cu33b\
4ma\x5cu0180;fv\x5cu33\
b1\x5cu33b2\x5cu33b2\x5cu\
43c3;\x5cu43c2\x5cu040\
0;deglnpr\x5cu12ab\x5c\
u33c5\x5cu33c9\x5cu33c\
e\x5cu33d6\x5cu33de\x5cu3\
3e1\x5cu33e6ot;\x5cu6a\
6a\x5cu0100;q\x5cu12b1\
\x5cu12b0\x5cu0100;E\x5cu\
33d3\x5cu33d4\x5cu6a9e\
;\x5cu6aa0\x5cu0100;E\x5c\
u33db\x5cu33dc\x5cu6a9\
d;\x5cu6a9fe;\x5cu6246\
lus;\x5cu6a24arr;\x5cu\
6972ar\x5cxf2\x5cu113d\
\x5cu0200aeit\x5cu33f8\
\x5cu3408\x5cu340f\x5cu34\
17\x5cu0100ls\x5cu33fd\
\x5cu3404lsetm\x5cxe9\x5c\
u336ahp;\x5cu6a33pa\
rsl;\x5cu69e4\x5cu0100\
dl\x5cu1463\x5cu3414e;\
\x5cu6323\x5cu0100;e\x5cu\
341c\x5cu341d\x5cu6aaa\
\x5cu0100;s\x5cu3422\x5cu\
3423\x5cu6aac;\x5cuc00\
0\x5cu2aac\x5cufe00\x5cu0\
180flp\x5cu342e\x5cu34\
33\x5cu3442tcy;\x5cu44\
4c\x5cu0100;b\x5cu3438\
\x5cu3439\x5cu402f\x5cu01\
00;a\x5cu343e\x5cu343f\
\x5cu69c4r;\x5cu633ff;\
\x5cuc000\x5cud835\x5cudd\
64a\x5cu0100dr\x5cu344\
d\x5cu0402es\x5cu0100;\
u\x5cu3454\x5cu3455\x5cu6\
660it\x5cxbb\x5cu3455\x5c\
u0180csu\x5cu3460\x5cu\
3479\x5cu349f\x5cu0100\
au\x5cu3465\x5cu346fp\x5c\
u0100;s\x5cu1188\x5cu3\
46b;\x5cuc000\x5cu2293\
\x5cufe00p\x5cu0100;s\x5c\
u11b4\x5cu3475;\x5cuc0\
00\x5cu2294\x5cufe00u\x5c\
u0100bp\x5cu347f\x5cu3\
48f\x5cu0180;es\x5cu11\
97\x5cu119c\x5cu3486et\
\x5cu0100;e\x5cu1197\x5cu\
348d\x5cxf1\x5cu119d\x5cu\
0180;es\x5cu11a8\x5cu1\
1ad\x5cu3496et\x5cu010\
0;e\x5cu11a8\x5cu349d\x5c\
xf1\x5cu11ae\x5cu0180;\
af\x5cu117b\x5cu34a6\x5cu\
05b0r\x5cu0165\x5cu34a\
b\x5cu05b1\x5cxbb\x5cu117\
car\x5cxf2\x5cu1148\x5cu0\
200cemt\x5cu34b9\x5cu3\
4be\x5cu34c2\x5cu34c5r\
;\x5cuc000\x5cud835\x5cud\
cc8tm\x5cxee\x5cxf1i\x5cx\
ec\x5cu3415ar\x5cxe6\x5cu\
11be\x5cu0100ar\x5cu34\
ce\x5cu34d5r\x5cu0100;\
f\x5cu34d4\x5cu17bf\x5cu6\
606\x5cu0100an\x5cu34d\
a\x5cu34edight\x5cu010\
0ep\x5cu34e3\x5cu34eap\
silo\x5cxee\x5cu1ee0h\x5c\
xe9\x5cu2eafs\x5cxbb\x5cu\
2852\x5cu0280bcmnp\x5c\
u34fb\x5cu355e\x5cu120\
9\x5cu358b\x5cu358e\x5cu0\
480;Edemnprs\x5cu35\
0e\x5cu350f\x5cu3511\x5cu\
3515\x5cu351e\x5cu3523\
\x5cu352c\x5cu3531\x5cu35\
36\x5cu6282;\x5cu6ac5o\
t;\x5cu6abd\x5cu0100;d\
\x5cu11da\x5cu351aot;\x5c\
u6ac3ult;\x5cu6ac1\x5c\
u0100Ee\x5cu3528\x5cu3\
52a;\x5cu6acb;\x5cu628\
alus;\x5cu6abfarr;\x5c\
u6979\x5cu0180eiu\x5cu\
353d\x5cu3552\x5cu3555\
t\x5cu0180;en\x5cu350e\
\x5cu3545\x5cu354bq\x5cu0\
100;q\x5cu11da\x5cu350\
feq\x5cu0100;q\x5cu352\
b\x5cu3528m;\x5cu6ac7\x5c\
u0100bp\x5cu355a\x5cu3\
55c;\x5cu6ad5;\x5cu6ad\
3c\x5cu0300;acens\x5cu\
11ed\x5cu356c\x5cu3572\
\x5cu3579\x5cu357b\x5cu33\
26ppro\x5cxf8\x5cu32fa\
urlye\x5cxf1\x5cu11fe\x5c\
xf1\x5cu11f3\x5cu0180a\
es\x5cu3582\x5cu3588\x5cu\
331bppro\x5cxf8\x5cu33\
1aq\x5cxf1\x5cu3317g;\x5c\
u666a\x5cu0680123;E\
dehlmnps\x5cu35a9\x5cu\
35ac\x5cu35af\x5cu121c\
\x5cu35b2\x5cu35b4\x5cu35\
c0\x5cu35c9\x5cu35d5\x5cu\
35da\x5cu35df\x5cu35e8\
\x5cu35ed\x5cu803b\x5cxb9\
\x5cu40b9\x5cu803b\x5cxb2\
\x5cu40b2\x5cu803b\x5cxb3\
\x5cu40b3;\x5cu6ac6\x5cu0\
100os\x5cu35b9\x5cu35b\
ct;\x5cu6abeub;\x5cu6a\
d8\x5cu0100;d\x5cu1222\
\x5cu35c5ot;\x5cu6ac4s\
\x5cu0100ou\x5cu35cf\x5cu\
35d2l;\x5cu67c9b;\x5cu\
6ad7arr;\x5cu697bul\
t;\x5cu6ac2\x5cu0100Ee\
\x5cu35e4\x5cu35e6;\x5cu6\
acc;\x5cu628blus;\x5cu\
6ac0\x5cu0180eiu\x5cu3\
5f4\x5cu3609\x5cu360ct\
\x5cu0180;en\x5cu121c\x5c\
u35fc\x5cu3602q\x5cu01\
00;q\x5cu1222\x5cu35b2\
eq\x5cu0100;q\x5cu35e7\
\x5cu35e4m;\x5cu6ac8\x5cu\
0100bp\x5cu3611\x5cu36\
13;\x5cu6ad4;\x5cu6ad6\
\x5cu0180Aan\x5cu361c\x5c\
u3620\x5cu362drr;\x5cu\
61d9r\x5cu0100hr\x5cu3\
626\x5cu3628\x5cxeb\x5cu2\
22e\x5cu0100;o\x5cu0a2\
b\x5cu0a29war;\x5cu692\
alig\x5cu803b\x5cxdf\x5cu\
40df\x5cu0be1\x5cu3651\
\x5cu365d\x5cu3660\x5cu12\
ce\x5cu3673\x5cu3679\x5c0\
\x5cu367e\x5cu36c2\x5c0\x5c0\
\x5c0\x5c0\x5c0\x5cu36db\x5cu37\
03\x5c0\x5cu3709\x5cu376c\
\x5c0\x5c0\x5c0\x5cu3787\x5cu02\
72\x5cu3656\x5c0\x5c0\x5cu36\
5bget;\x5cu6316;\x5cu4\
3c4r\x5cxeb\x5cu0e5f\x5cu\
0180aey\x5cu3666\x5cu3\
66b\x5cu3670ron;\x5cu4\
165dil;\x5cu4163;\x5cu\
4442lrec;\x5cu6315r\
;\x5cuc000\x5cud835\x5cud\
d31\x5cu0200eiko\x5cu3\
686\x5cu369d\x5cu36b5\x5c\
u36bc\x5cu01f2\x5cu368\
b\x5c0\x5cu3691e\x5cu0100\
4f\x5cu1284\x5cu1281a\x5c\
u0180;sv\x5cu3698\x5cu\
3699\x5cu369b\x5cu43b8\
ym;\x5cu43d1\x5cu0100c\
n\x5cu36a2\x5cu36b2k\x5cu\
0100as\x5cu36a8\x5cu36\
aeppro\x5cxf8\x5cu12c1\
im\x5cxbb\x5cu12acs\x5cxf\
0\x5cu129e\x5cu0100as\x5c\
u36ba\x5cu36ae\x5cxf0\x5c\
u12c1rn\x5cu803b\x5cxf\
e\x5cu40fe\x5cu01ec\x5cu0\
31f\x5cu36c6\x5cu22e7e\
s\x5cu8180\x5cxd7;bd\x5cu\
36cf\x5cu36d0\x5cu36d8\
\x5cu40d7\x5cu0100;a\x5cu\
190f\x5cu36d5r;\x5cu6a\
31;\x5cu6a30\x5cu0180e\
ps\x5cu36e1\x5cu36e3\x5cu\
3700\x5cxe1\x5cu2a4d\x5cu\
0200;bcf\x5cu0486\x5cu\
36ec\x5cu36f0\x5cu36f4\
ot;\x5cu6336ir;\x5cu6a\
f1\x5cu0100;o\x5cu36f9\
\x5cu36fc\x5cuc000\x5cud8\
35\x5cudd65rk;\x5cu6ad\
a\x5cxe1\x5cu3362rime;\
\x5cu6034\x5cu0180aip\x5c\
u370f\x5cu3712\x5cu376\
4d\x5cxe5\x5cu1248\x5cu03\
80adempst\x5cu3721\x5c\
u374d\x5cu3740\x5cu375\
1\x5cu3757\x5cu375c\x5cu3\
75fngle\x5cu0280;dl\
qr\x5cu3730\x5cu3731\x5cu\
3736\x5cu3740\x5cu3742\
\x5cu65b5own\x5cxbb\x5cu1\
dbbeft\x5cu0100;e\x5cu\
2800\x5cu373e\x5cxf1\x5cu\
092e;\x5cu625cight\x5c\
u0100;e\x5cu32aa\x5cu3\
74b\x5cxf1\x5cu105aot;\
\x5cu65ecinus;\x5cu6a3\
alus;\x5cu6a39b;\x5cu6\
9cdime;\x5cu6a3bezi\
um;\x5cu63e2\x5cu0180c\
ht\x5cu3772\x5cu377d\x5cu\
3781\x5cu0100ry\x5cu37\
77\x5cu377b;\x5cuc000\x5c\
ud835\x5cudcc9;\x5cu44\
46cy;\x5cu445brok;\x5c\
u4167\x5cu0100io\x5cu3\
78b\x5cu378ex\x5cxf4\x5cu\
1777head\x5cu0100lr\
\x5cu3797\x5cu37a0efta\
rro\x5cxf7\x5cu084figh\
tarrow\x5cxbb\x5cu0f5d\
\x5cu0900AHabcdfghl\
moprstuw\x5cu37d0\x5cu\
37d3\x5cu37d7\x5cu37e4\
\x5cu37f0\x5cu37fc\x5cu38\
0e\x5cu381c\x5cu3823\x5cu\
3834\x5cu3851\x5cu385d\
\x5cu386b\x5cu38a9\x5cu38\
cc\x5cu38d2\x5cu38ea\x5cu\
38f6r\x5cxf2\x5cu03eda\
r;\x5cu6963\x5cu0100cr\
\x5cu37dc\x5cu37e2ute\x5c\
u803b\x5cxfa\x5cu40fa\x5c\
xf2\x5cu1150r\x5cu01e3\
\x5cu37ea\x5c0\x5cu37edy;\
\x5cu445eve;\x5cu416d\x5c\
u0100iy\x5cu37f5\x5cu3\
7farc\x5cu803b\x5cxfb\x5c\
u40fb;\x5cu4443\x5cu01\
80abh\x5cu3803\x5cu380\
6\x5cu380br\x5cxf2\x5cu13\
adlac;\x5cu4171a\x5cxf\
2\x5cu13c3\x5cu0100ir\x5c\
u3813\x5cu3818sht;\x5c\
u697e;\x5cuc000\x5cud8\
35\x5cudd32rave\x5cu80\
3b\x5cxf9\x5cu40f9\x5cu01\
61\x5cu3827\x5cu3831r\x5c\
u0100lr\x5cu382c\x5cu3\
82e\x5cxbb\x5cu0957\x5cxb\
b\x5cu1083lk;\x5cu6580\
\x5cu0100ct\x5cu3839\x5cu\
384d\x5cu026f\x5cu383f\
\x5c0\x5c0\x5cu384arn\x5cu01\
00;e\x5cu3845\x5cu3846\
\x5cu631cr\x5cxbb\x5cu384\
6op;\x5cu630fri;\x5cu6\
5f8\x5cu0100al\x5cu385\
6\x5cu385acr;\x5cu416b\
\x5cu80bb\x5cxa8\x5cu0349\
\x5cu0100gp\x5cu3862\x5cu\
3866on;\x5cu4173f;\x5c\
uc000\x5cud835\x5cudd6\
6\x5cu0300adhlsu\x5cu1\
14b\x5cu3878\x5cu387d\x5c\
u1372\x5cu3891\x5cu38a\
0own\x5cxe1\x5cu13b3ar\
poon\x5cu0100lr\x5cu38\
88\x5cu388cef\x5cxf4\x5cu\
382digh\x5cxf4\x5cu382\
fi\x5cu0180;hl\x5cu389\
9\x5cu389a\x5cu389c\x5cu4\
3c5\x5cxbb\x5cu13faon\x5c\
xbb\x5cu389aparrows\
;\x5cu61c8\x5cu0180cit\
\x5cu38b0\x5cu38c4\x5cu38\
c8\x5cu026f\x5cu38b6\x5c0\
\x5c0\x5cu38c1rn\x5cu0100\
;e\x5cu38bc\x5cu38bd\x5cu\
631dr\x5cxbb\x5cu38bdo\
p;\x5cu630eng;\x5cu416\
fri;\x5cu65f9cr;\x5cuc\
000\x5cud835\x5cudcca\x5c\
u0180dir\x5cu38d9\x5cu\
38dd\x5cu38e2ot;\x5cu6\
2f0lde;\x5cu4169i\x5cu\
0100;f\x5cu3730\x5cu38\
e8\x5cxbb\x5cu1813\x5cu01\
00am\x5cu38ef\x5cu38f2\
r\x5cxf2\x5cu38a8l\x5cu80\
3b\x5cxfc\x5cu40fcangl\
e;\x5cu69a7\x5cu0780AB\
Dacdeflnoprsz\x5cu3\
91c\x5cu391f\x5cu3929\x5c\
u392d\x5cu39b5\x5cu39b\
8\x5cu39bd\x5cu39df\x5cu3\
9e4\x5cu39e8\x5cu39f3\x5c\
u39f9\x5cu39fd\x5cu3a0\
1\x5cu3a20r\x5cxf2\x5cu03\
f7ar\x5cu0100;v\x5cu39\
26\x5cu3927\x5cu6ae8;\x5c\
u6ae9as\x5cxe8\x5cu03e\
1\x5cu0100nr\x5cu3932\x5c\
u3937grt;\x5cu699c\x5c\
u0380eknprst\x5cu34\
e3\x5cu3946\x5cu394b\x5cu\
3952\x5cu395d\x5cu3964\
\x5cu3996app\x5cxe1\x5cu2\
415othin\x5cxe7\x5cu1e\
96\x5cu0180hir\x5cu34e\
b\x5cu2ec8\x5cu3959op\x5c\
xf4\x5cu2fb5\x5cu0100;\
h\x5cu13b7\x5cu3962\x5cxe\
f\x5cu318d\x5cu0100iu\x5c\
u3969\x5cu396dgm\x5cxe\
1\x5cu33b3\x5cu0100bp\x5c\
u3972\x5cu3984setne\
q\x5cu0100;q\x5cu397d\x5c\
u3980\x5cuc000\x5cu228\
a\x5cufe00;\x5cuc000\x5cu\
2acb\x5cufe00setneq\
\x5cu0100;q\x5cu398f\x5cu\
3992\x5cuc000\x5cu228b\
\x5cufe00;\x5cuc000\x5cu2\
acc\x5cufe00\x5cu0100h\
r\x5cu399b\x5cu399fet\x5c\
xe1\x5cu369ciangle\x5c\
u0100lr\x5cu39aa\x5cu3\
9afeft\x5cxbb\x5cu0925\
ight\x5cxbb\x5cu1051y;\
\x5cu4432ash\x5cxbb\x5cu1\
036\x5cu0180elr\x5cu39\
c4\x5cu39d2\x5cu39d7\x5cu\
0180;be\x5cu2dea\x5cu3\
9cb\x5cu39cfar;\x5cu62\
bbq;\x5cu625alip;\x5cu\
62ee\x5cu0100bt\x5cu39\
dc\x5cu1468a\x5cxf2\x5cu1\
469r;\x5cuc000\x5cud83\
5\x5cudd33tr\x5cxe9\x5cu3\
9aesu\x5cu0100bp\x5cu3\
9ef\x5cu39f1\x5cxbb\x5cu0\
d1c\x5cxbb\x5cu0d59pf;\
\x5cuc000\x5cud835\x5cudd\
67ro\x5cxf0\x5cu0efbtr\
\x5cxe9\x5cu39b4\x5cu0100\
cu\x5cu3a06\x5cu3a0br;\
\x5cuc000\x5cud835\x5cudc\
cb\x5cu0100bp\x5cu3a10\
\x5cu3a18n\x5cu0100Ee\x5c\
u3980\x5cu3a16\x5cxbb\x5c\
u397en\x5cu0100Ee\x5cu\
3992\x5cu3a1e\x5cxbb\x5cu\
3990igzag;\x5cu699a\
\x5cu0380cefoprs\x5cu3\
a36\x5cu3a3b\x5cu3a56\x5c\
u3a5b\x5cu3a54\x5cu3a6\
1\x5cu3a6airc;\x5cu417\
5\x5cu0100di\x5cu3a40\x5c\
u3a51\x5cu0100bg\x5cu3\
a45\x5cu3a49ar;\x5cu6a\
5fe\x5cu0100;q\x5cu15f\
a\x5cu3a4f;\x5cu6259er\
p;\x5cu6118r;\x5cuc000\
\x5cud835\x5cudd34pf;\x5c\
uc000\x5cud835\x5cudd6\
8\x5cu0100;e\x5cu1479\x5c\
u3a66at\x5cxe8\x5cu147\
9cr;\x5cuc000\x5cud835\
\x5cudccc\x5cu0ae3\x5cu17\
8e\x5cu3a87\x5c0\x5cu3a8b\
\x5c0\x5cu3a90\x5cu3a9b\x5c0\
\x5c0\x5cu3a9d\x5cu3aa8\x5cu\
3aab\x5cu3aaf\x5c0\x5c0\x5cu\
3ac3\x5cu3ace\x5c0\x5cu3a\
d8\x5cu17dc\x5cu17dftr\
\x5cxe9\x5cu17d1r;\x5cuc0\
00\x5cud835\x5cudd35\x5cu\
0100Aa\x5cu3a94\x5cu3a\
97r\x5cxf2\x5cu03c3r\x5cx\
f2\x5cu09f6;\x5cu43be\x5c\
u0100Aa\x5cu3aa1\x5cu3\
aa4r\x5cxf2\x5cu03b8r\x5c\
xf2\x5cu09eba\x5cxf0\x5cu\
2713is;\x5cu62fb\x5cu0\
180dpt\x5cu17a4\x5cu3a\
b5\x5cu3abe\x5cu0100fl\
\x5cu3aba\x5cu17a9;\x5cuc\
000\x5cud835\x5cudd69i\
m\x5cxe5\x5cu17b2\x5cu010\
0Aa\x5cu3ac7\x5cu3acar\
\x5cxf2\x5cu03cer\x5cxf2\x5c\
u0a01\x5cu0100cq\x5cu3\
ad2\x5cu17b8r;\x5cuc00\
0\x5cud835\x5cudccd\x5cu0\
100pt\x5cu17d6\x5cu3ad\
cr\x5cxe9\x5cu17d4\x5cu04\
00acefiosu\x5cu3af0\
\x5cu3afd\x5cu3b08\x5cu3b\
0c\x5cu3b11\x5cu3b15\x5cu\
3b1b\x5cu3b21c\x5cu010\
0uy\x5cu3af6\x5cu3afbt\
e\x5cu803b\x5cxfd\x5cu40f\
d;\x5cu444f\x5cu0100iy\
\x5cu3b02\x5cu3b06rc;\x5c\
u4177;\x5cu444bn\x5cu8\
03b\x5cxa5\x5cu40a5r;\x5c\
uc000\x5cud835\x5cudd3\
6cy;\x5cu4457pf;\x5cuc\
000\x5cud835\x5cudd6ac\
r;\x5cuc000\x5cud835\x5cu\
dcce\x5cu0100cm\x5cu3b\
26\x5cu3b29y;\x5cu444e\
l\x5cu803b\x5cxff\x5cu40f\
f\x5cu0500acdefhios\
w\x5cu3b42\x5cu3b48\x5cu3\
b54\x5cu3b58\x5cu3b64\x5c\
u3b69\x5cu3b6d\x5cu3b7\
4\x5cu3b7a\x5cu3b80cut\
e;\x5cu417a\x5cu0100ay\
\x5cu3b4d\x5cu3b52ron;\
\x5cu417e;\x5cu4437ot;\
\x5cu417c\x5cu0100et\x5cu\
3b5d\x5cu3b61tr\x5cxe6\
\x5cu155fa;\x5cu43b6r;\
\x5cuc000\x5cud835\x5cudd\
37cy;\x5cu4436grarr\
;\x5cu61ddpf;\x5cuc000\
\x5cud835\x5cudd6bcr;\x5c\
uc000\x5cud835\x5cudcc\
f\x5cu0100jn\x5cu3b85\x5c\
u3b87;\x5cu600dj;\x5cu\
600c'.split(\x22\x22).\
map((t=>t.charCo\
deAt(0)))),w=new\
 Uint16Array(\x22\x5cu\
0200aglq\x5ct\x5cx15\x5cx\
18\x5cx1b\x5cu026d\x5cx0f\
\x5c0\x5c0\x5cx12p;\x5cu4026\
os;\x5cu4027t;\x5cu403\
et;\x5cu403cuot;\x5cu4\
022\x22.split(\x22\x22).m\
ap((t=>t.charCod\
eAt(0))));const \
v=new Map([[0,65\
533],[128,8364],\
[130,8218],[131,\
402],[132,8222],\
[133,8230],[134,\
8224],[135,8225]\
,[136,710],[137,\
8240],[138,352],\
[139,8249],[140,\
338],[142,381],[\
145,8216],[146,8\
217],[147,8220],\
[148,8221],[149,\
8226],[150,8211]\
,[151,8212],[152\
,732],[153,8482]\
,[154,353],[155,\
8250],[156,339],\
[158,382],[159,3\
76]]),z=null!==(\
k=String.fromCod\
ePoint)&&void 0!\
==k?k:function(t\
){let e=\x22\x22;retur\
n t>65535&&(t-=6\
5536,e+=String.f\
romCharCode(t>>>\
10&1023|55296),t\
=56320|1023&t),e\
+=String.fromCha\
rCode(t),e};var \
S;!function(t){t\
[t.NUM=35]=\x22NUM\x22\
,t[t.SEMI=59]=\x22S\
EMI\x22,t[t.EQUALS=\
61]=\x22EQUALS\x22,t[t\
.ZERO=48]=\x22ZERO\x22\
,t[t.NINE=57]=\x22N\
INE\x22,t[t.LOWER_A\
=97]=\x22LOWER_A\x22,t\
[t.LOWER_F=102]=\
\x22LOWER_F\x22,t[t.LO\
WER_X=120]=\x22LOWE\
R_X\x22,t[t.LOWER_Z\
=122]=\x22LOWER_Z\x22,\
t[t.UPPER_A=65]=\
\x22UPPER_A\x22,t[t.UP\
PER_F=70]=\x22UPPER\
_F\x22,t[t.UPPER_Z=\
90]=\x22UPPER_Z\x22}(S\
||(S={}));var q,\
B,L;function I(t\
){return t>=S.ZE\
RO&&t<=S.NINE}fu\
nction M(t){retu\
rn t>=S.UPPER_A&\
&t<=S.UPPER_F||t\
>=S.LOWER_A&&t<=\
S.LOWER_F}functi\
on T(t){return t\
===S.EQUALS||fun\
ction(t){return \
t>=S.UPPER_A&&t<\
=S.UPPER_Z||t>=S\
.LOWER_A&&t<=S.L\
OWER_Z||I(t)}(t)\
}!function(t){t[\
t.VALUE_LENGTH=4\
9152]=\x22VALUE_LEN\
GTH\x22,t[t.BRANCH_\
LENGTH=16256]=\x22B\
RANCH_LENGTH\x22,t[\
t.JUMP_TABLE=127\
]=\x22JUMP_TABLE\x22}(\
q||(q={})),funct\
ion(t){t[t.Entit\
yStart=0]=\x22Entit\
yStart\x22,t[t.Nume\
ricStart=1]=\x22Num\
ericStart\x22,t[t.N\
umericDecimal=2]\
=\x22NumericDecimal\
\x22,t[t.NumericHex\
=3]=\x22NumericHex\x22\
,t[t.NamedEntity\
=4]=\x22NamedEntity\
\x22}(B||(B={})),fu\
nction(t){t[t.Le\
gacy=0]=\x22Legacy\x22\
,t[t.Strict=1]=\x22\
Strict\x22,t[t.Attr\
ibute=2]=\x22Attrib\
ute\x22}(L||(L={}))\
;class R{constru\
ctor(t,e,r){this\
.decodeTree=t,th\
is.emitCodePoint\
=e,this.errors=r\
,this.state=B.En\
tityStart,this.c\
onsumed=1,this.r\
esult=0,this.tre\
eIndex=0,this.ex\
cess=1,this.deco\
deMode=L.Strict}\
startEntity(t){t\
his.decodeMode=t\
,this.state=B.En\
tityStart,this.r\
esult=0,this.tre\
eIndex=0,this.ex\
cess=1,this.cons\
umed=1}write(t,e\
){switch(this.st\
ate){case B.Enti\
tyStart:return t\
.charCodeAt(e)==\
=S.NUM?(this.sta\
te=B.NumericStar\
t,this.consumed+\
=1,this.stateNum\
ericStart(t,e+1)\
):(this.state=B.\
NamedEntity,this\
.stateNamedEntit\
y(t,e));case B.N\
umericStart:retu\
rn this.stateNum\
ericStart(t,e);c\
ase B.NumericDec\
imal:return this\
.stateNumericDec\
imal(t,e);case B\
.NumericHex:retu\
rn this.stateNum\
ericHex(t,e);cas\
e B.NamedEntity:\
return this.stat\
eNamedEntity(t,e\
)}}stateNumericS\
tart(t,e){return\
 e>=t.length?-1:\
(32|t.charCodeAt\
(e))===S.LOWER_X\
?(this.state=B.N\
umericHex,this.c\
onsumed+=1,this.\
stateNumericHex(\
t,e+1)):(this.st\
ate=B.NumericDec\
imal,this.stateN\
umericDecimal(t,\
e))}addToNumeric\
Result(t,e,r,n){\
if(e!==r){const \
s=r-e;this.resul\
t=this.result*Ma\
th.pow(n,s)+pars\
eInt(t.substr(e,\
s),n),this.consu\
med+=s}}stateNum\
ericHex(t,e){con\
st r=e;for(;e<t.\
length;){const n\
=t.charCodeAt(e)\
;if(!I(n)&&!M(n)\
)return this.add\
ToNumericResult(\
t,r,e,16),this.e\
mitNumericEntity\
(n,3);e+=1}retur\
n this.addToNume\
ricResult(t,r,e,\
16),-1}stateNume\
ricDecimal(t,e){\
const r=e;for(;e\
<t.length;){cons\
t n=t.charCodeAt\
(e);if(!I(n))ret\
urn this.addToNu\
mericResult(t,r,\
e,10),this.emitN\
umericEntity(n,2\
);e+=1}return th\
is.addToNumericR\
esult(t,r,e,10),\
-1}emitNumericEn\
tity(t,e){var r;\
if(this.consumed\
<=e)return null=\
==(r=this.errors\
)||void 0===r||r\
.absenceOfDigits\
InNumericCharact\
erReference(this\
.consumed),0;if(\
t===S.SEMI)this.\
consumed+=1;else\
 if(this.decodeM\
ode===L.Strict)r\
eturn 0;return t\
his.emitCodePoin\
t(function(t){va\
r e;return t>=55\
296&&t<=57343||t\
>1114111?65533:n\
ull!==(e=v.get(t\
))&&void 0!==e?e\
:t}(this.result)\
,this.consumed),\
this.errors&&(t!\
==S.SEMI&&this.e\
rrors.missingSem\
icolonAfterChara\
cterReference(),\
this.errors.vali\
dateNumericChara\
cterReference(th\
is.result)),this\
.consumed}stateN\
amedEntity(t,e){\
const{decodeTree\
:r}=this;let n=r\
[this.treeIndex]\
,s=(n&q.VALUE_LE\
NGTH)>>14;for(;e\
<t.length;e++,th\
is.excess++){con\
st i=t.charCodeA\
t(e);if(this.tre\
eIndex=P(r,n,thi\
s.treeIndex+Math\
.max(1,s),i),thi\
s.treeIndex<0)re\
turn 0===this.re\
sult||this.decod\
eMode===L.Attrib\
ute&&(0===s||T(i\
))?0:this.emitNo\
tTerminatedNamed\
Entity();if(n=r[\
this.treeIndex],\
s=(n&q.VALUE_LEN\
GTH)>>14,0!==s){\
if(i===S.SEMI)re\
turn this.emitNa\
medEntityData(th\
is.treeIndex,s,t\
his.consumed+thi\
s.excess);this.d\
ecodeMode!==L.St\
rict&&(this.resu\
lt=this.treeInde\
x,this.consumed+\
=this.excess,thi\
s.excess=0)}}ret\
urn-1}emitNotTer\
minatedNamedEnti\
ty(){var t;const\
{result:e,decode\
Tree:r}=this,n=(\
r[e]&q.VALUE_LEN\
GTH)>>14;return \
this.emitNamedEn\
tityData(e,n,thi\
s.consumed),null\
===(t=this.error\
s)||void 0===t||\
t.missingSemicol\
onAfterCharacter\
Reference(),this\
.consumed}emitNa\
medEntityData(t,\
e,r){const{decod\
eTree:n}=this;re\
turn this.emitCo\
dePoint(1===e?n[\
t]&~q.VALUE_LENG\
TH:n[t+1],r),3==\
=e&&this.emitCod\
ePoint(n[t+2],r)\
,r}end(){var t;s\
witch(this.state\
){case B.NamedEn\
tity:return 0===\
this.result||thi\
s.decodeMode===L\
.Attribute&&this\
.result!==this.t\
reeIndex?0:this.\
emitNotTerminate\
dNamedEntity();c\
ase B.NumericDec\
imal:return this\
.emitNumericEnti\
ty(0,2);case B.N\
umericHex:return\
 this.emitNumeri\
cEntity(0,3);cas\
e B.NumericStart\
:return null===(\
t=this.errors)||\
void 0===t||t.ab\
senceOfDigitsInN\
umericCharacterR\
eference(this.co\
nsumed),0;case B\
.EntityStart:ret\
urn 0}}}function\
 N(t){let e=\x22\x22;c\
onst r=new R(t,(\
t=>e+=z(t)));ret\
urn function(t,n\
){let s=0,i=0;fo\
r(;(i=t.indexOf(\
\x22&\x22,i))>=0;){e+=\
t.slice(s,i),r.s\
tartEntity(n);co\
nst o=r.write(t,\
i+1);if(o<0){s=i\
+r.end();break}s\
=i+o,i=0===o?s+1\
:s}const o=e+t.s\
lice(s);return e\
=\x22\x22,o}}function \
P(t,e,r,n){const\
 s=(e&q.BRANCH_L\
ENGTH)>>7,i=e&q.\
JUMP_TABLE;if(0=\
==s)return 0!==i\
&&n===i?r:-1;if(\
i){const e=n-i;r\
eturn e<0||e>=s?\
-1:t[r+e]-1}let \
o=r,u=o+s-1;for(\
;o<=u;){const e=\
o+u>>>1,r=t[e];i\
f(r<n)o=e+1;else\
{if(!(r>n))retur\
n t[e+s];u=e-1}}\
return-1}const O\
=N(x);function j\
(t,e=L.Legacy){r\
eturn O(t,e)}fun\
ction Z(t){retur\
n\x22[object String\
]\x22===function(t)\
{return Object.p\
rototype.toStrin\
g.call(t)}(t)}N(\
w);const $=Objec\
t.prototype.hasO\
wnProperty;funct\
ion U(t){return \
Array.prototype.\
slice.call(argum\
ents,1).forEach(\
(function(e){if(\
e){if(\x22object\x22!=\
typeof e)throw n\
ew TypeError(e+\x22\
must be object\x22)\
;Object.keys(e).\
forEach((functio\
n(r){t[r]=e[r]})\
)}})),t}function\
 H(t,e,r){return\
[].concat(t.slic\
e(0,e),r,t.slice\
(e+1))}function \
V(t){return!(t>=\
55296&&t<=57343)\
&&(!(t>=64976&&t\
<=65007)&&(!!(65\
535&~t&&65534!=(\
65535&t))&&(!(t>\
=0&&t<=8)&&(11!=\
=t&&(!(t>=14&&t<\
=31)&&(!(t>=127&\
&t<=159)&&!(t>11\
14111)))))))}fun\
ction G(t){if(t>\
65535){const e=5\
5296+((t-=65536)\
>>10),r=56320+(1\
023&t);return St\
ring.fromCharCod\
e(e,r)}return St\
ring.fromCharCod\
e(t)}const W=/\x5c\x5c\
([!\x22#$%&'()*+,\x5c-\
./:;<=>?@[\x5c\x5c\x5c]^_\
`{|}~])/g,J=new \
RegExp(W.source+\
\x22|\x22+/&([a-z#][a-\
z0-9]{1,31});/gi\
.source,\x22gi\x22),Q=\
/^#((?:x[a-f0-9]\
{1,8}|[0-9]{1,8}\
))$/i;function X\
(t){return t.ind\
exOf(\x22\x5c\x5c\x22)<0&&t.\
indexOf(\x22&\x22)<0?t\
:t.replace(J,(fu\
nction(t,e,r){re\
turn e||function\
(t,e){if(35===e.\
charCodeAt(0)&&Q\
.test(e)){const \
r=\x22x\x22===e[1].toL\
owerCase()?parse\
Int(e.slice(2),1\
6):parseInt(e.sl\
ice(1),10);retur\
n V(r)?G(r):t}co\
nst r=j(t);retur\
n r!==t?r:t}(t,r\
)}))}const Y=/[&\
<>\x22]/,K=/[&<>\x22]/\
g,tt={\x22&\x22:\x22&amp;\
\x22,\x22<\x22:\x22&lt;\x22,\x22>\x22\
:\x22&gt;\x22,'\x22':\x22&qu\
ot;\x22};function e\
t(t){return tt[t\
]}function rt(t)\
{return Y.test(t\
)?t.replace(K,et\
):t}const nt=/[.\
?*+^$[\x5c]\x5c\x5c(){}|-\
]/g;function st(\
t){switch(t){cas\
e 9:case 32:retu\
rn!0}return!1}fu\
nction it(t){if(\
t>=8192&&t<=8202\
)return!0;switch\
(t){case 9:case \
10:case 11:case \
12:case 13:case \
32:case 160:case\
 5760:case 8239:\
case 8287:case 1\
2288:return!0}re\
turn!1}function \
ot(t){return E.t\
est(t)||A.test(t\
)}function ut(t)\
{switch(t){case \
33:case 34:case \
35:case 36:case \
37:case 38:case \
39:case 40:case \
41:case 42:case \
43:case 44:case \
45:case 46:case \
47:case 58:case \
59:case 60:case \
61:case 62:case \
63:case 64:case \
91:case 92:case \
93:case 94:case \
95:case 96:case \
123:case 124:cas\
e 125:case 126:r\
eturn!0;default:\
return!1}}functi\
on ct(t){return \
t=t.trim().repla\
ce(/\x5cs+/g,\x22 \x22),\x22\
\x5cu1e7e\x22===\x22\x5cu1e9\
e\x22.toLowerCase()\
&&(t=t.replace(/\
\x5cu1e9e/g,\x22\x5cxdf\x22)\
),t.toLowerCase(\
).toUpperCase()}\
const at={mdurl:\
D,ucmicro:F};var\
 lt=Object.freez\
e({__proto__:nul\
l,arrayReplaceAt\
:H,assign:U,esca\
peHtml:rt,escape\
RE:function(t){r\
eturn t.replace(\
nt,\x22\x5c\x5c$&\x22)},from\
CodePoint:G,has:\
function(t,e){re\
turn $.call(t,e)\
},isMdAsciiPunct\
:ut,isPunctChar:\
ot,isSpace:st,is\
String:Z,isValid\
EntityCode:V,isW\
hiteSpace:it,lib\
:at,normalizeRef\
erence:ct,unesca\
peAll:X,unescape\
Md:function(t){r\
eturn t.indexOf(\
\x22\x5c\x5c\x22)<0?t:t.repl\
ace(W,\x22$1\x22)}});v\
ar ht=Object.fre\
eze({__proto__:n\
ull,parseLinkDes\
tination:functio\
n(t,e,r){let n,s\
=e;const i={ok:!\
1,pos:0,str:\x22\x22};\
if(60===t.charCo\
deAt(s)){for(s++\
;s<r;){if(n=t.ch\
arCodeAt(s),10==\
=n)return i;if(6\
0===n)return i;i\
f(62===n)return \
i.pos=s+1,i.str=\
X(t.slice(e+1,s)\
),i.ok=!0,i;92==\
=n&&s+1<r?s+=2:s\
++}return i}let \
o=0;for(;s<r&&(n\
=t.charCodeAt(s)\
,32!==n)&&!(n<32\
||127===n);)if(9\
2===n&&s+1<r){if\
(32===t.charCode\
At(s+1))break;s+\
=2}else{if(40===\
n&&(o++,o>32))re\
turn i;if(41===n\
){if(0===o)break\
;o--}s++}return \
e===s||0!==o||(i\
.str=X(t.slice(e\
,s)),i.pos=s,i.o\
k=!0),i},parseLi\
nkLabel:function\
(t,e,r){let n,s,\
i,o;const u=t.po\
sMax,c=t.pos;for\
(t.pos=e+1,n=1;t\
.pos<u;){if(i=t.\
src.charCodeAt(t\
.pos),93===i&&(n\
--,0===n)){s=!0;\
break}if(o=t.pos\
,t.md.inline.ski\
pToken(t),91===i\
)if(o===t.pos-1)\
n++;else if(r)re\
turn t.pos=c,-1}\
let a=-1;return \
s&&(a=t.pos),t.p\
os=c,a},parseLin\
kTitle:function(\
t,e,r,n){let s,i\
=e;const o={ok:!\
1,can_continue:!\
1,pos:0,str:\x22\x22,m\
arker:0};if(n)o.\
str=n.str,o.mark\
er=n.marker;else\
{if(i>=r)return \
o;let n=t.charCo\
deAt(i);if(34!==\
n&&39!==n&&40!==\
n)return o;e++,i\
++,40===n&&(n=41\
),o.marker=n}for\
(;i<r;){if(s=t.c\
harCodeAt(i),s==\
=o.marker)return\
 o.pos=i+1,o.str\
+=X(t.slice(e,i)\
),o.ok=!0,o;if(4\
0===s&&41===o.ma\
rker)return o;92\
===s&&i+1<r&&i++\
,i++}return o.ca\
n_continue=!0,o.\
str+=X(t.slice(e\
,i)),o}});const \
pt={};function f\
t(){this.rules=U\
({},pt)}function\
 dt(){this.__rul\
es__=[],this.__c\
ache__=null}func\
tion _t(t,e,r){t\
his.type=t,this.\
tag=e,this.attrs\
=null,this.map=n\
ull,this.nesting\
=r,this.level=0,\
this.children=nu\
ll,this.content=\
\x22\x22,this.markup=\x22\
\x22,this.info=\x22\x22,t\
his.meta=null,th\
is.block=!1,this\
.hidden=!1}funct\
ion mt(t,e,r){th\
is.src=t,this.en\
v=r,this.tokens=\
[],this.inlineMo\
de=!1,this.md=e}\
pt.code_inline=f\
unction(t,e,r,n,\
s){const i=t[e];\
return\x22<code\x22+s.\
renderAttrs(i)+\x22\
>\x22+rt(i.content)\
+\x22</code>\x22},pt.c\
ode_block=functi\
on(t,e,r,n,s){co\
nst i=t[e];retur\
n\x22<pre\x22+s.render\
Attrs(i)+\x22><code\
>\x22+rt(t[e].conte\
nt)+\x22</code></pr\
e>\x5cn\x22},pt.fence=\
function(t,e,r,n\
,s){const i=t[e]\
,o=i.info?X(i.in\
fo).trim():\x22\x22;le\
t u,c=\x22\x22,a=\x22\x22;if\
(o){const t=o.sp\
lit(/(\x5cs+)/g);c=\
t[0],a=t.slice(2\
).join(\x22\x22)}if(u=\
r.highlight&&r.h\
ighlight(i.conte\
nt,c,a)||rt(i.co\
ntent),0===u.ind\
exOf(\x22<pre\x22))ret\
urn u+\x22\x5cn\x22;if(o)\
{const t=i.attrI\
ndex(\x22class\x22),e=\
i.attrs?i.attrs.\
slice():[];t<0?e\
.push([\x22class\x22,r\
.langPrefix+c]):\
(e[t]=e[t].slice\
(),e[t][1]+=\x22 \x22+\
r.langPrefix+c);\
const n={attrs:e\
};return`<pre><c\
ode${s.renderAtt\
rs(n)}>${u}</cod\
e></pre>\x5cn`}retu\
rn`<pre><code${s\
.renderAttrs(i)}\
>${u}</code></pr\
e>\x5cn`},pt.image=\
function(t,e,r,n\
,s){const i=t[e]\
;return i.attrs[\
i.attrIndex(\x22alt\
\x22)][1]=s.renderI\
nlineAsText(i.ch\
ildren,r,n),s.re\
nderToken(t,e,r)\
},pt.hardbreak=f\
unction(t,e,r){r\
eturn r.xhtmlOut\
?\x22<br />\x5cn\x22:\x22<br\
>\x5cn\x22},pt.softbre\
ak=function(t,e,\
r){return r.brea\
ks?r.xhtmlOut?\x22<\
br />\x5cn\x22:\x22<br>\x5cn\
\x22:\x22\x5cn\x22},pt.text=\
function(t,e){re\
turn rt(t[e].con\
tent)},pt.html_b\
lock=function(t,\
e){return t[e].c\
ontent},pt.html_\
inline=function(\
t,e){return t[e]\
.content},ft.pro\
totype.renderAtt\
rs=function(t){l\
et e,r,n;if(!t.a\
ttrs)return\x22\x22;fo\
r(n=\x22\x22,e=0,r=t.a\
ttrs.length;e<r;\
e++)n+=\x22 \x22+rt(t.\
attrs[e][0])+'=\x22\
'+rt(t.attrs[e][\
1])+'\x22';return n\
},ft.prototype.r\
enderToken=funct\
ion(t,e,r){const\
 n=t[e];let s=\x22\x22\
;if(n.hidden)ret\
urn\x22\x22;n.block&&-\
1!==n.nesting&&e\
&&t[e-1].hidden&\
&(s+=\x22\x5cn\x22),s+=(-\
1===n.nesting?\x22<\
/\x22:\x22<\x22)+n.tag,s+\
=this.renderAttr\
s(n),0===n.nesti\
ng&&r.xhtmlOut&&\
(s+=\x22 /\x22);let i=\
!1;if(n.block&&(\
i=!0,1===n.nesti\
ng&&e+1<t.length\
)){const r=t[e+1\
];(\x22inline\x22===r.\
type||r.hidden||\
-1===r.nesting&&\
r.tag===n.tag)&&\
(i=!1)}return s+\
=i?\x22>\x5cn\x22:\x22>\x22,s},\
ft.prototype.ren\
derInline=functi\
on(t,e,r){let n=\
\x22\x22;const s=this.\
rules;for(let i=\
0,o=t.length;i<o\
;i++){const o=t[\
i].type;void 0!=\
=s[o]?n+=s[o](t,\
i,e,r,this):n+=t\
his.renderToken(\
t,i,e)}return n}\
,ft.prototype.re\
nderInlineAsText\
=function(t,e,r)\
{let n=\x22\x22;for(le\
t s=0,i=t.length\
;s<i;s++)switch(\
t[s].type){case\x22\
text\x22:case\x22html_\
inline\x22:case\x22htm\
l_block\x22:n+=t[s]\
.content;break;c\
ase\x22image\x22:n+=th\
is.renderInlineA\
sText(t[s].child\
ren,e,r);break;c\
ase\x22softbreak\x22:c\
ase\x22hardbreak\x22:n\
+=\x22\x5cn\x22}return n}\
,ft.prototype.re\
nder=function(t,\
e,r){let n=\x22\x22;co\
nst s=this.rules\
;for(let i=0,o=t\
.length;i<o;i++)\
{const o=t[i].ty\
pe;\x22inline\x22===o?\
n+=this.renderIn\
line(t[i].childr\
en,e,r):void 0!=\
=s[o]?n+=s[o](t,\
i,e,r,this):n+=t\
his.renderToken(\
t,i,e,r)}return \
n},dt.prototype.\
__find__=functio\
n(t){for(let e=0\
;e<this.__rules_\
_.length;e++)if(\
this.__rules__[e\
].name===t)retur\
n e;return-1},dt\
.prototype.__com\
pile__=function(\
){const t=this,e\
=[\x22\x22];t.__rules_\
_.forEach((funct\
ion(t){t.enabled\
&&t.alt.forEach(\
(function(t){e.i\
ndexOf(t)<0&&e.p\
ush(t)}))})),t._\
_cache__={},e.fo\
rEach((function(\
e){t.__cache__[e\
]=[],t.__rules__\
.forEach((functi\
on(r){r.enabled&\
&(e&&r.alt.index\
Of(e)<0||t.__cac\
he__[e].push(r.f\
n))}))}))},dt.pr\
ototype.at=funct\
ion(t,e,r){const\
 n=this.__find__\
(t),s=r||{};if(-\
1===n)throw new \
Error(\x22Parser ru\
le not found: \x22+\
t);this.__rules_\
_[n].fn=e,this._\
_rules__[n].alt=\
s.alt||[],this._\
_cache__=null},d\
t.prototype.befo\
re=function(t,e,\
r,n){const s=thi\
s.__find__(t),i=\
n||{};if(-1===s)\
throw new Error(\
\x22Parser rule not\
 found: \x22+t);thi\
s.__rules__.spli\
ce(s,0,{name:e,e\
nabled:!0,fn:r,a\
lt:i.alt||[]}),t\
his.__cache__=nu\
ll},dt.prototype\
.after=function(\
t,e,r,n){const s\
=this.__find__(t\
),i=n||{};if(-1=\
==s)throw new Er\
ror(\x22Parser rule\
 not found: \x22+t)\
;this.__rules__.\
splice(s+1,0,{na\
me:e,enabled:!0,\
fn:r,alt:i.alt||\
[]}),this.__cach\
e__=null},dt.pro\
totype.push=func\
tion(t,e,r){cons\
t n=r||{};this._\
_rules__.push({n\
ame:t,enabled:!0\
,fn:e,alt:n.alt|\
|[]}),this.__cac\
he__=null},dt.pr\
ototype.enable=f\
unction(t,e){Arr\
ay.isArray(t)||(\
t=[t]);const r=[\
];return t.forEa\
ch((function(t){\
const n=this.__f\
ind__(t);if(n<0)\
{if(e)return;thr\
ow new Error(\x22Ru\
les manager: inv\
alid rule name \x22\
+t)}this.__rules\
__[n].enabled=!0\
,r.push(t)}),thi\
s),this.__cache_\
_=null,r},dt.pro\
totype.enableOnl\
y=function(t,e){\
Array.isArray(t)\
||(t=[t]),this._\
_rules__.forEach\
((function(t){t.\
enabled=!1})),th\
is.enable(t,e)},\
dt.prototype.dis\
able=function(t,\
e){Array.isArray\
(t)||(t=[t]);con\
st r=[];return t\
.forEach((functi\
on(t){const n=th\
is.__find__(t);i\
f(n<0){if(e)retu\
rn;throw new Err\
or(\x22Rules manage\
r: invalid rule \
name \x22+t)}this._\
_rules__[n].enab\
led=!1,r.push(t)\
}),this),this.__\
cache__=null,r},\
dt.prototype.get\
Rules=function(t\
){return null===\
this.__cache__&&\
this.__compile__\
(),this.__cache_\
_[t]||[]},_t.pro\
totype.attrIndex\
=function(t){if(\
!this.attrs)retu\
rn-1;const e=thi\
s.attrs;for(let \
r=0,n=e.length;r\
<n;r++)if(e[r][0\
]===t)return r;r\
eturn-1},_t.prot\
otype.attrPush=f\
unction(t){this.\
attrs?this.attrs\
.push(t):this.at\
trs=[t]},_t.prot\
otype.attrSet=fu\
nction(t,e){cons\
t r=this.attrInd\
ex(t),n=[t,e];r<\
0?this.attrPush(\
n):this.attrs[r]\
=n},_t.prototype\
.attrGet=functio\
n(t){const e=thi\
s.attrIndex(t);l\
et r=null;return\
 e>=0&&(r=this.a\
ttrs[e][1]),r},_\
t.prototype.attr\
Join=function(t,\
e){const r=this.\
attrIndex(t);r<0\
?this.attrPush([\
t,e]):this.attrs\
[r][1]=this.attr\
s[r][1]+\x22 \x22+e},m\
t.prototype.Toke\
n=_t;const gt=/\x5c\
r\x5cn?|\x5cn/g,kt=/\x5c0\
/g;function Dt(t\
){return/^<\x5c/a\x5cs\
*>/i.test(t)}con\
st Ct=/\x5c+-|\x5c.\x5c.|\
\x5c?\x5c?\x5c?\x5c?|!!!!|,,\
|--/,yt=/\x5c((c|tm\
|r)\x5c)/i,Et=/\x5c((c\
|tm|r)\x5c)/gi,At={\
c:\x22\x5cxa9\x22,r:\x22\x5cxae\
\x22,tm:\x22\x5cu2122\x22};f\
unction bt(t,e){\
return At[e.toLo\
werCase()]}funct\
ion Ft(t){let e=\
0;for(let r=t.le\
ngth-1;r>=0;r--)\
{const n=t[r];\x22t\
ext\x22!==n.type||e\
||(n.content=n.c\
ontent.replace(E\
t,bt)),\x22link_ope\
n\x22===n.type&&\x22au\
to\x22===n.info&&e-\
-,\x22link_close\x22==\
=n.type&&\x22auto\x22=\
==n.info&&e++}}f\
unction xt(t){le\
t e=0;for(let r=\
t.length-1;r>=0;\
r--){const n=t[r\
];\x22text\x22!==n.typ\
e||e||Ct.test(n.\
content)&&(n.con\
tent=n.content.r\
eplace(/\x5c+-/g,\x22\x5c\
xb1\x22).replace(/\x5c\
.{2,}/g,\x22\x5cu2026\x22\
).replace(/([?!]\
)\x5cu2026/g,\x22$1..\x22\
).replace(/([?!]\
){4,}/g,\x22$1$1$1\x22\
).replace(/,{2,}\
/g,\x22,\x22).replace(\
/(^|[^-])---(?=[\
^-]|$)/gm,\x22$1\x5cu2\
014\x22).replace(/(\
^|\x5cs)--(?=\x5cs|$)/\
gm,\x22$1\x5cu2013\x22).r\
eplace(/(^|[^-\x5cs\
])--(?=[^-\x5cs]|$)\
/gm,\x22$1\x5cu2013\x22))\
,\x22link_open\x22===n\
.type&&\x22auto\x22===\
n.info&&e--,\x22lin\
k_close\x22===n.typ\
e&&\x22auto\x22===n.in\
fo&&e++}}const w\
t=/['\x22]/,vt=/['\x22\
]/g,zt=\x22\x5cu2019\x22;\
function St(t,e,\
r){return t.slic\
e(0,e)+r+t.slice\
(e+1)}function q\
t(t,e){let r;con\
st n=[];for(let \
s=0;s<t.length;s\
++){const i=t[s]\
,o=t[s].level;fo\
r(r=n.length-1;r\
>=0&&!(n[r].leve\
l<=o);r--);if(n.\
length=r+1,\x22text\
\x22!==i.type)conti\
nue;let u=i.cont\
ent,c=0,a=u.leng\
th;t:for(;c<a;){\
vt.lastIndex=c;c\
onst l=vt.exec(u\
);if(!l)break;le\
t h=!0,p=!0;c=l.\
index+1;const f=\
\x22'\x22===l[0];let d\
=32;if(l.index-1\
>=0)d=u.charCode\
At(l.index-1);el\
se for(r=s-1;r>=\
0&&(\x22softbreak\x22!\
==t[r].type&&\x22ha\
rdbreak\x22!==t[r].\
type);r--)if(t[r\
].content){d=t[r\
].content.charCo\
deAt(t[r].conten\
t.length-1);brea\
k}let _=32;if(c<\
a)_=u.charCodeAt\
(c);else for(r=s\
+1;r<t.length&&(\
\x22softbreak\x22!==t[\
r].type&&\x22hardbr\
eak\x22!==t[r].type\
);r++)if(t[r].co\
ntent){_=t[r].co\
ntent.charCodeAt\
(0);break}const \
m=ut(d)||ot(Stri\
ng.fromCharCode(\
d)),g=ut(_)||ot(\
String.fromCharC\
ode(_)),k=it(d),\
D=it(_);if(D?h=!\
1:g&&(k||m||(h=!\
1)),k?p=!1:m&&(D\
||g||(p=!1)),34=\
==_&&'\x22'===l[0]&\
&d>=48&&d<=57&&(\
p=h=!1),h&&p&&(h\
=m,p=g),h||p){if\
(p)for(r=n.lengt\
h-1;r>=0;r--){le\
t h=n[r];if(n[r]\
.level<o)break;i\
f(h.single===f&&\
n[r].level===o){\
let o,p;h=n[r],f\
?(o=e.md.options\
.quotes[2],p=e.m\
d.options.quotes\
[3]):(o=e.md.opt\
ions.quotes[0],p\
=e.md.options.qu\
otes[1]),i.conte\
nt=St(i.content,\
l.index,p),t[h.t\
oken].content=St\
(t[h.token].cont\
ent,h.pos,o),c+=\
p.length-1,h.tok\
en===s&&(c+=o.le\
ngth-1),u=i.cont\
ent,a=u.length,n\
.length=r;contin\
ue t}}h?n.push({\
token:s,pos:l.in\
dex,single:f,lev\
el:o}):p&&f&&(i.\
content=St(i.con\
tent,l.index,zt)\
)}else f&&(i.con\
tent=St(i.conten\
t,l.index,zt))}}\
}const Bt=[[\x22nor\
malize\x22,function\
(t){let e;e=t.sr\
c.replace(gt,\x22\x5cn\
\x22),e=e.replace(k\
t,\x22\x5cufffd\x22),t.sr\
c=e}],[\x22block\x22,f\
unction(t){let e\
;t.inlineMode?(e\
=new t.Token(\x22in\
line\x22,\x22\x22,0),e.co\
ntent=t.src,e.ma\
p=[0,1],e.childr\
en=[],t.tokens.p\
ush(e)):t.md.blo\
ck.parse(t.src,t\
.md,t.env,t.toke\
ns)}],[\x22inline\x22,\
function(t){cons\
t e=t.tokens;for\
(let r=0,n=e.len\
gth;r<n;r++){con\
st n=e[r];\x22inlin\
e\x22===n.type&&t.m\
d.inline.parse(n\
.content,t.md,t.\
env,n.children)}\
}],[\x22linkify\x22,fu\
nction(t){const \
e=t.tokens;var r\
;if(t.md.options\
.linkify)for(let\
 n=0,s=e.length;\
n<s;n++){if(\x22inl\
ine\x22!==e[n].type\
||!t.md.linkify.\
pretest(e[n].con\
tent))continue;l\
et s=e[n].childr\
en,i=0;for(let o\
=s.length-1;o>=0\
;o--){const u=s[\
o];if(\x22link_clos\
e\x22!==u.type){if(\
\x22html_inline\x22===\
u.type&&(r=u.con\
tent,/^<a[>\x5cs]/i\
.test(r)&&i>0&&i\
--,Dt(u.content)\
&&i++),!(i>0)&&\x22\
text\x22===u.type&&\
t.md.linkify.tes\
t(u.content)){co\
nst r=u.content;\
let i=t.md.linki\
fy.match(r);cons\
t c=[];let a=u.l\
evel,l=0;i.lengt\
h>0&&0===i[0].in\
dex&&o>0&&\x22text_\
special\x22===s[o-1\
].type&&(i=i.sli\
ce(1));for(let e\
=0;e<i.length;e+\
+){const n=i[e].\
url,s=t.md.norma\
lizeLink(n);if(!\
t.md.validateLin\
k(s))continue;le\
t o=i[e].text;o=\
i[e].schema?\x22mai\
lto:\x22!==i[e].sch\
ema||/^mailto:/i\
.test(o)?t.md.no\
rmalizeLinkText(\
o):t.md.normaliz\
eLinkText(\x22mailt\
o:\x22+o).replace(/\
^mailto:/,\x22\x22):t.\
md.normalizeLink\
Text(\x22http://\x22+o\
).replace(/^http\
:\x5c/\x5c//,\x22\x22);const\
 u=i[e].index;if\
(u>l){const e=ne\
w t.Token(\x22text\x22\
,\x22\x22,0);e.content\
=r.slice(l,u),e.\
level=a,c.push(e\
)}const h=new t.\
Token(\x22link_open\
\x22,\x22a\x22,1);h.attrs\
=[[\x22href\x22,s]],h.\
level=a++,h.mark\
up=\x22linkify\x22,h.i\
nfo=\x22auto\x22,c.pus\
h(h);const p=new\
 t.Token(\x22text\x22,\
\x22\x22,0);p.content=\
o,p.level=a,c.pu\
sh(p);const f=ne\
w t.Token(\x22link_\
close\x22,\x22a\x22,-1);f\
.level=--a,f.mar\
kup=\x22linkify\x22,f.\
info=\x22auto\x22,c.pu\
sh(f),l=i[e].las\
tIndex}if(l<r.le\
ngth){const e=ne\
w t.Token(\x22text\x22\
,\x22\x22,0);e.content\
=r.slice(l),e.le\
vel=a,c.push(e)}\
e[n].children=s=\
H(s,o,c)}}else f\
or(o--;s[o].leve\
l!==u.level&&\x22li\
nk_open\x22!==s[o].\
type;)o--}}}],[\x22\
replacements\x22,fu\
nction(t){let e;\
if(t.md.options.\
typographer)for(\
e=t.tokens.lengt\
h-1;e>=0;e--)\x22in\
line\x22===t.tokens\
[e].type&&(yt.te\
st(t.tokens[e].c\
ontent)&&Ft(t.to\
kens[e].children\
),Ct.test(t.toke\
ns[e].content)&&\
xt(t.tokens[e].c\
hildren))}],[\x22sm\
artquotes\x22,funct\
ion(t){if(t.md.o\
ptions.typograph\
er)for(let e=t.t\
okens.length-1;e\
>=0;e--)\x22inline\x22\
===t.tokens[e].t\
ype&&wt.test(t.t\
okens[e].content\
)&&qt(t.tokens[e\
].children,t)}],\
[\x22text_join\x22,fun\
ction(t){let e,r\
;const n=t.token\
s,s=n.length;for\
(let t=0;t<s;t++\
){if(\x22inline\x22!==\
n[t].type)contin\
ue;const s=n[t].\
children,i=s.len\
gth;for(e=0;e<i;\
e++)\x22text_specia\
l\x22===s[e].type&&\
(s[e].type=\x22text\
\x22);for(e=r=0;e<i\
;e++)\x22text\x22===s[\
e].type&&e+1<i&&\
\x22text\x22===s[e+1].\
type?s[e+1].cont\
ent=s[e].content\
+s[e+1].content:\
(e!==r&&(s[r]=s[\
e]),r++);e!==r&&\
(s.length=r)}}]]\
;function Lt(){t\
his.ruler=new dt\
;for(let t=0;t<B\
t.length;t++)thi\
s.ruler.push(Bt[\
t][0],Bt[t][1])}\
function It(t,e,\
r,n){this.src=t,\
this.md=e,this.e\
nv=r,this.tokens\
=n,this.bMarks=[\
],this.eMarks=[]\
,this.tShift=[],\
this.sCount=[],t\
his.bsCount=[],t\
his.blkIndent=0,\
this.line=0,this\
.lineMax=0,this.\
tight=!1,this.dd\
Indent=-1,this.l\
istIndent=-1,thi\
s.parentType=\x22ro\
ot\x22,this.level=0\
;const s=this.sr\
c;for(let t=0,e=\
0,r=0,n=0,i=s.le\
ngth,o=!1;e<i;e+\
+){const u=s.cha\
rCodeAt(e);if(!o\
){if(st(u)){r++,\
9===u?n+=4-n%4:n\
++;continue}o=!0\
}10!==u&&e!==i-1\
||(10!==u&&e++,t\
his.bMarks.push(\
t),this.eMarks.p\
ush(e),this.tShi\
ft.push(r),this.\
sCount.push(n),t\
his.bsCount.push\
(0),o=!1,r=0,n=0\
,t=e+1)}this.bMa\
rks.push(s.lengt\
h),this.eMarks.p\
ush(s.length),th\
is.tShift.push(0\
),this.sCount.pu\
sh(0),this.bsCou\
nt.push(0),this.\
lineMax=this.bMa\
rks.length-1}Lt.\
prototype.proces\
s=function(t){co\
nst e=this.ruler\
.getRules(\x22\x22);fo\
r(let r=0,n=e.le\
ngth;r<n;r++)e[r\
](t)},Lt.prototy\
pe.State=mt,It.p\
rototype.push=fu\
nction(t,e,r){co\
nst n=new _t(t,e\
,r);return n.blo\
ck=!0,r<0&&this.\
level--,n.level=\
this.level,r>0&&\
this.level++,thi\
s.tokens.push(n)\
,n},It.prototype\
.isEmpty=functio\
n(t){return this\
.bMarks[t]+this.\
tShift[t]>=this.\
eMarks[t]},It.pr\
ototype.skipEmpt\
yLines=function(\
t){for(let e=thi\
s.lineMax;t<e&&!\
(this.bMarks[t]+\
this.tShift[t]<t\
his.eMarks[t]);t\
++);return t},It\
.prototype.skipS\
paces=function(t\
){for(let e=this\
.src.length;t<e;\
t++){if(!st(this\
.src.charCodeAt(\
t)))break}return\
 t},It.prototype\
.skipSpacesBack=\
function(t,e){if\
(t<=e)return t;f\
or(;t>e;)if(!st(\
this.src.charCod\
eAt(--t)))return\
 t+1;return t},I\
t.prototype.skip\
Chars=function(t\
,e){for(let r=th\
is.src.length;t<\
r&&this.src.char\
CodeAt(t)===e;t+\
+);return t},It.\
prototype.skipCh\
arsBack=function\
(t,e,r){if(t<=r)\
return t;for(;t>\
r;)if(e!==this.s\
rc.charCodeAt(--\
t))return t+1;re\
turn t},It.proto\
type.getLines=fu\
nction(t,e,r,n){\
if(t>=e)return\x22\x22\
;const s=new Arr\
ay(e-t);for(let \
i=0,o=t;o<e;o++,\
i++){let t=0;con\
st u=this.bMarks\
[o];let c,a=u;fo\
r(c=o+1<e||n?thi\
s.eMarks[o]+1:th\
is.eMarks[o];a<c\
&&t<r;){const e=\
this.src.charCod\
eAt(a);if(st(e))\
9===e?t+=4-(t+th\
is.bsCount[o])%4\
:t++;else{if(!(a\
-u<this.tShift[o\
]))break;t++}a++\
}s[i]=t>r?new Ar\
ray(t-r+1).join(\
\x22 \x22)+this.src.sl\
ice(a,c):this.sr\
c.slice(a,c)}ret\
urn s.join(\x22\x22)},\
It.prototype.Tok\
en=_t;function M\
t(t,e){const r=t\
.bMarks[e]+t.tSh\
ift[e],n=t.eMark\
s[e];return t.sr\
c.slice(r,n)}fun\
ction Tt(t){cons\
t e=[],r=t.lengt\
h;let n=0,s=t.ch\
arCodeAt(n),i=!1\
,o=0,u=\x22\x22;for(;n\
<r;)124===s&&(i?\
(u+=t.substring(\
o,n-1),o=n):(e.p\
ush(u+t.substrin\
g(o,n)),u=\x22\x22,o=n\
+1)),i=92===s,n+\
+,s=t.charCodeAt\
(n);return e.pus\
h(u+t.substring(\
o)),e}function R\
t(t,e){const r=t\
.eMarks[e];let n\
=t.bMarks[e]+t.t\
Shift[e];const s\
=t.src.charCodeA\
t(n++);if(42!==s\
&&45!==s&&43!==s\
)return-1;if(n<r\
){if(!st(t.src.c\
harCodeAt(n)))re\
turn-1}return n}\
function Nt(t,e)\
{const r=t.bMark\
s[e]+t.tShift[e]\
,n=t.eMarks[e];l\
et s=r;if(s+1>=n\
)return-1;let i=\
t.src.charCodeAt\
(s++);if(i<48||i\
>57)return-1;for\
(;;){if(s>=n)ret\
urn-1;if(i=t.src\
.charCodeAt(s++)\
,!(i>=48&&i<=57)\
){if(41===i||46=\
==i)break;return\
-1}if(s-r>=10)re\
turn-1}return s<\
n&&(i=t.src.char\
CodeAt(s),!st(i)\
)?-1:s}const Pt=\
\x22<[A-Za-z][A-Za-\
z0-9\x5c\x5c-]*(?:\x5c\x5cs+\
[a-zA-Z_:][a-zA-\
Z0-9:._-]*(?:\x5c\x5cs\
*=\x5c\x5cs*(?:[^\x5c\x22'=<\
>`\x5c\x5cx00-\x5c\x5cx20]+|\
'[^']*'|\x5c\x22[^\x5c\x22]*\
\x5c\x22))?)*\x5c\x5cs*\x5c\x5c/?>\
\x22,Ot=\x22<\x5c\x5c/[A-Za-\
z][A-Za-z0-9\x5c\x5c-]\
*\x5c\x5cs*>\x22,jt=new R\
egExp(\x22^(?:\x22+Pt+\
\x22|\x22+Ot+\x22|\x5cx3c!--\
-?>|\x5cx3c!--(?:[^\
-]|-[^-]|--[^>])\
*--\x5cx3e|<[?][\x5c\x5cs\
\x5c\x5cS]*?[?]>|<![A-\
Za-z][^>]*>|<!\x5c\x5c\
[CDATA\x5c\x5c[[\x5c\x5cs\x5c\x5cS\
]*?\x5c\x5c]\x5c\x5c]>)\x22),Zt\
=new RegExp(\x22^(?\
:\x22+Pt+\x22|\x22+Ot+\x22)\x22\
),$t=[[/^<(scrip\
t|pre|style|text\
area)(?=(\x5cs|>|$)\
)/i,/<\x5c/(script|\
pre|style|textar\
ea)>/i,!0],[/^<!\
--/,/-->/,!0],[/\
^<\x5c?/,/\x5c?>/,!0],\
[/^<![A-Z]/,/>/,\
!0],[/^<!\x5c[CDATA\
\x5c[/,/\x5c]\x5c]>/,!0],\
[new RegExp(\x22^</\
?(\x22+[\x22address\x22,\x22\
article\x22,\x22aside\x22\
,\x22base\x22,\x22basefon\
t\x22,\x22blockquote\x22,\
\x22body\x22,\x22caption\x22\
,\x22center\x22,\x22col\x22,\
\x22colgroup\x22,\x22dd\x22,\
\x22details\x22,\x22dialo\
g\x22,\x22dir\x22,\x22div\x22,\x22\
dl\x22,\x22dt\x22,\x22fields\
et\x22,\x22figcaption\x22\
,\x22figure\x22,\x22foote\
r\x22,\x22form\x22,\x22frame\
\x22,\x22frameset\x22,\x22h1\
\x22,\x22h2\x22,\x22h3\x22,\x22h4\x22\
,\x22h5\x22,\x22h6\x22,\x22head\
\x22,\x22header\x22,\x22hr\x22,\
\x22html\x22,\x22iframe\x22,\
\x22legend\x22,\x22li\x22,\x22l\
ink\x22,\x22main\x22,\x22men\
u\x22,\x22menuitem\x22,\x22n\
av\x22,\x22noframes\x22,\x22\
ol\x22,\x22optgroup\x22,\x22\
option\x22,\x22p\x22,\x22par\
am\x22,\x22search\x22,\x22se\
ction\x22,\x22summary\x22\
,\x22table\x22,\x22tbody\x22\
,\x22td\x22,\x22tfoot\x22,\x22t\
h\x22,\x22thead\x22,\x22titl\
e\x22,\x22tr\x22,\x22track\x22,\
\x22ul\x22].join(\x22|\x22)+\
\x22)(?=(\x5c\x5cs|/?>|$)\
)\x22,\x22i\x22),/^$/,!0]\
,[new RegExp(Zt.\
source+\x22\x5c\x5cs*$\x22),\
/^$/,!1]];const \
Ut=[[\x22table\x22,fun\
ction(t,e,r,n){i\
f(e+2>r)return!1\
;let s=e+1;if(t.\
sCount[s]<t.blkI\
ndent)return!1;i\
f(t.sCount[s]-t.\
blkIndent>=4)ret\
urn!1;let i=t.bM\
arks[s]+t.tShift\
[s];if(i>=t.eMar\
ks[s])return!1;c\
onst o=t.src.cha\
rCodeAt(i++);if(\
124!==o&&45!==o&\
&58!==o)return!1\
;if(i>=t.eMarks[\
s])return!1;cons\
t u=t.src.charCo\
deAt(i++);if(124\
!==u&&45!==u&&58\
!==u&&!st(u))ret\
urn!1;if(45===o&\
&st(u))return!1;\
for(;i<t.eMarks[\
s];){const e=t.s\
rc.charCodeAt(i)\
;if(124!==e&&45!\
==e&&58!==e&&!st\
(e))return!1;i++\
}let c=Mt(t,e+1)\
,a=c.split(\x22|\x22);\
const l=[];for(l\
et t=0;t<a.lengt\
h;t++){const e=a\
[t].trim();if(!e\
){if(0===t||t===\
a.length-1)conti\
nue;return!1}if(\
!/^:?-+:?$/.test\
(e))return!1;58=\
==e.charCodeAt(e\
.length-1)?l.pus\
h(58===e.charCod\
eAt(0)?\x22center\x22:\
\x22right\x22):58===e.\
charCodeAt(0)?l.\
push(\x22left\x22):l.p\
ush(\x22\x22)}if(c=Mt(\
t,e).trim(),-1==\
=c.indexOf(\x22|\x22))\
return!1;if(t.sC\
ount[e]-t.blkInd\
ent>=4)return!1;\
a=Tt(c),a.length\
&&\x22\x22===a[0]&&a.s\
hift(),a.length&\
&\x22\x22===a[a.length\
-1]&&a.pop();con\
st h=a.length;if\
(0===h||h!==l.le\
ngth)return!1;if\
(n)return!0;cons\
t p=t.parentType\
;t.parentType=\x22t\
able\x22;const f=t.\
md.block.ruler.g\
etRules(\x22blockqu\
ote\x22),d=[e,0];t.\
push(\x22table_open\
\x22,\x22table\x22,1).map\
=d,t.push(\x22thead\
_open\x22,\x22thead\x22,1\
).map=[e,e+1],t.\
push(\x22tr_open\x22,\x22\
tr\x22,1).map=[e,e+\
1];for(let e=0;e\
<a.length;e++){c\
onst r=t.push(\x22t\
h_open\x22,\x22th\x22,1);\
l[e]&&(r.attrs=[\
[\x22style\x22,\x22text-a\
lign:\x22+l[e]]]);c\
onst n=t.push(\x22i\
nline\x22,\x22\x22,0);n.c\
ontent=a[e].trim\
(),n.children=[]\
,t.push(\x22th_clos\
e\x22,\x22th\x22,-1)}let \
_;t.push(\x22tr_clo\
se\x22,\x22tr\x22,-1),t.p\
ush(\x22thead_close\
\x22,\x22thead\x22,-1);le\
t m=0;for(s=e+2;\
s<r&&!(t.sCount[\
s]<t.blkIndent);\
s++){let n=!1;fo\
r(let e=0,i=f.le\
ngth;e<i;e++)if(\
f[e](t,s,r,!0)){\
n=!0;break}if(n)\
break;if(c=Mt(t,\
s).trim(),!c)bre\
ak;if(t.sCount[s\
]-t.blkIndent>=4\
)break;if(a=Tt(c\
),a.length&&\x22\x22==\
=a[0]&&a.shift()\
,a.length&&\x22\x22===\
a[a.length-1]&&a\
.pop(),m+=h-a.le\
ngth,m>65536)bre\
ak;if(s===e+2){t\
.push(\x22tbody_ope\
n\x22,\x22tbody\x22,1).ma\
p=_=[e+2,0]}t.pu\
sh(\x22tr_open\x22,\x22tr\
\x22,1).map=[s,s+1]\
;for(let e=0;e<h\
;e++){const r=t.\
push(\x22td_open\x22,\x22\
td\x22,1);l[e]&&(r.\
attrs=[[\x22style\x22,\
\x22text-align:\x22+l[\
e]]]);const n=t.\
push(\x22inline\x22,\x22\x22\
,0);n.content=a[\
e]?a[e].trim():\x22\
\x22,n.children=[],\
t.push(\x22td_close\
\x22,\x22td\x22,-1)}t.pus\
h(\x22tr_close\x22,\x22tr\
\x22,-1)}return _&&\
(t.push(\x22tbody_c\
lose\x22,\x22tbody\x22,-1\
),_[1]=s),t.push\
(\x22table_close\x22,\x22\
table\x22,-1),d[1]=\
s,t.parentType=p\
,t.line=s,!0},[\x22\
paragraph\x22,\x22refe\
rence\x22]],[\x22code\x22\
,function(t,e,r)\
{if(t.sCount[e]-\
t.blkIndent<4)re\
turn!1;let n=e+1\
,s=n;for(;n<r;)i\
f(t.isEmpty(n))n\
++;else{if(!(t.s\
Count[n]-t.blkIn\
dent>=4))break;n\
++,s=n}t.line=s;\
const i=t.push(\x22\
code_block\x22,\x22cod\
e\x22,0);return i.c\
ontent=t.getLine\
s(e,s,4+t.blkInd\
ent,!1)+\x22\x5cn\x22,i.m\
ap=[e,t.line],!0\
}],[\x22fence\x22,func\
tion(t,e,r,n){le\
t s=t.bMarks[e]+\
t.tShift[e],i=t.\
eMarks[e];if(t.s\
Count[e]-t.blkIn\
dent>=4)return!1\
;if(s+3>i)return\
!1;const o=t.src\
.charCodeAt(s);i\
f(126!==o&&96!==\
o)return!1;let u\
=s;s=t.skipChars\
(s,o);let c=s-u;\
if(c<3)return!1;\
const a=t.src.sl\
ice(u,s),l=t.src\
.slice(s,i);if(9\
6===o&&l.indexOf\
(String.fromChar\
Code(o))>=0)retu\
rn!1;if(n)return\
!0;let h=e,p=!1;\
for(;(h++,!(h>=r\
))&&(s=u=t.bMark\
s[h]+t.tShift[h]\
,i=t.eMarks[h],!\
(s<i&&t.sCount[h\
]<t.blkIndent));\
)if(t.src.charCo\
deAt(s)===o&&!(t\
.sCount[h]-t.blk\
Indent>=4||(s=t.\
skipChars(s,o),s\
-u<c||(s=t.skipS\
paces(s),s<i))))\
{p=!0;break}c=t.\
sCount[e],t.line\
=h+(p?1:0);const\
 f=t.push(\x22fence\
\x22,\x22code\x22,0);retu\
rn f.info=l,f.co\
ntent=t.getLines\
(e+1,h,c,!0),f.m\
arkup=a,f.map=[e\
,t.line],!0},[\x22p\
aragraph\x22,\x22refer\
ence\x22,\x22blockquot\
e\x22,\x22list\x22]],[\x22bl\
ockquote\x22,functi\
on(t,e,r,n){let \
s=t.bMarks[e]+t.\
tShift[e],i=t.eM\
arks[e];const o=\
t.lineMax;if(t.s\
Count[e]-t.blkIn\
dent>=4)return!1\
;if(62!==t.src.c\
harCodeAt(s))ret\
urn!1;if(n)retur\
n!0;const u=[],c\
=[],a=[],l=[],h=\
t.md.block.ruler\
.getRules(\x22block\
quote\x22),p=t.pare\
ntType;t.parentT\
ype=\x22blockquote\x22\
;let f,d=!1;for(\
f=e;f<r;f++){con\
st e=t.sCount[f]\
<t.blkIndent;if(\
s=t.bMarks[f]+t.\
tShift[f],i=t.eM\
arks[f],s>=i)bre\
ak;if(62===t.src\
.charCodeAt(s++)\
&&!e){let e,r,n=\
t.sCount[f]+1;32\
===t.src.charCod\
eAt(s)?(s++,n++,\
r=!1,e=!0):9===t\
.src.charCodeAt(\
s)?(e=!0,(t.bsCo\
unt[f]+n)%4==3?(\
s++,n++,r=!1):r=\
!0):e=!1;let o=n\
;for(u.push(t.bM\
arks[f]),t.bMark\
s[f]=s;s<i;){con\
st e=t.src.charC\
odeAt(s);if(!st(\
e))break;9===e?o\
+=4-(o+t.bsCount\
[f]+(r?1:0))%4:o\
++,s++}d=s>=i,c.\
push(t.bsCount[f\
]),t.bsCount[f]=\
t.sCount[f]+1+(e\
?1:0),a.push(t.s\
Count[f]),t.sCou\
nt[f]=o-n,l.push\
(t.tShift[f]),t.\
tShift[f]=s-t.bM\
arks[f];continue\
}if(d)break;let \
n=!1;for(let e=0\
,s=h.length;e<s;\
e++)if(h[e](t,f,\
r,!0)){n=!0;brea\
k}if(n){t.lineMa\
x=f,0!==t.blkInd\
ent&&(u.push(t.b\
Marks[f]),c.push\
(t.bsCount[f]),l\
.push(t.tShift[f\
]),a.push(t.sCou\
nt[f]),t.sCount[\
f]-=t.blkIndent)\
;break}u.push(t.\
bMarks[f]),c.pus\
h(t.bsCount[f]),\
l.push(t.tShift[\
f]),a.push(t.sCo\
unt[f]),t.sCount\
[f]=-1}const _=t\
.blkIndent;t.blk\
Indent=0;const m\
=t.push(\x22blockqu\
ote_open\x22,\x22block\
quote\x22,1);m.mark\
up=\x22>\x22;const g=[\
e,0];m.map=g,t.m\
d.block.tokenize\
(t,e,f),t.push(\x22\
blockquote_close\
\x22,\x22blockquote\x22,-\
1).markup=\x22>\x22,t.\
lineMax=o,t.pare\
ntType=p,g[1]=t.\
line;for(let r=0\
;r<l.length;r++)\
t.bMarks[r+e]=u[\
r],t.tShift[r+e]\
=l[r],t.sCount[r\
+e]=a[r],t.bsCou\
nt[r+e]=c[r];ret\
urn t.blkIndent=\
_,!0},[\x22paragrap\
h\x22,\x22reference\x22,\x22\
blockquote\x22,\x22lis\
t\x22]],[\x22hr\x22,funct\
ion(t,e,r,n){con\
st s=t.eMarks[e]\
;if(t.sCount[e]-\
t.blkIndent>=4)r\
eturn!1;let i=t.\
bMarks[e]+t.tShi\
ft[e];const o=t.\
src.charCodeAt(i\
++);if(42!==o&&4\
5!==o&&95!==o)re\
turn!1;let u=1;f\
or(;i<s;){const \
e=t.src.charCode\
At(i++);if(e!==o\
&&!st(e))return!\
1;e===o&&u++}if(\
u<3)return!1;if(\
n)return!0;t.lin\
e=e+1;const c=t.\
push(\x22hr\x22,\x22hr\x22,0\
);return c.map=[\
e,t.line],c.mark\
up=Array(u+1).jo\
in(String.fromCh\
arCode(o)),!0},[\
\x22paragraph\x22,\x22ref\
erence\x22,\x22blockqu\
ote\x22,\x22list\x22]],[\x22\
list\x22,function(t\
,e,r,n){let s,i,\
o,u,c=e,a=!0;if(\
t.sCount[c]-t.bl\
kIndent>=4)retur\
n!1;if(t.listInd\
ent>=0&&t.sCount\
[c]-t.listIndent\
>=4&&t.sCount[c]\
<t.blkIndent)ret\
urn!1;let l,h,p,\
f=!1;if(n&&\x22para\
graph\x22===t.paren\
tType&&t.sCount[\
c]>=t.blkIndent&\
&(f=!0),(p=Nt(t,\
c))>=0){if(l=!0,\
o=t.bMarks[c]+t.\
tShift[c],h=Numb\
er(t.src.slice(o\
,p-1)),f&&1!==h)\
return!1}else{if\
(!((p=Rt(t,c))>=\
0))return!1;l=!1\
}if(f&&t.skipSpa\
ces(p)>=t.eMarks\
[c])return!1;if(\
n)return!0;const\
 d=t.src.charCod\
eAt(p-1),_=t.tok\
ens.length;l?(u=\
t.push(\x22ordered_\
list_open\x22,\x22ol\x22,\
1),1!==h&&(u.att\
rs=[[\x22start\x22,h]]\
)):u=t.push(\x22bul\
let_list_open\x22,\x22\
ul\x22,1);const m=[\
c,0];u.map=m,u.m\
arkup=String.fro\
mCharCode(d);let\
 g=!1;const k=t.\
md.block.ruler.g\
etRules(\x22list\x22),\
D=t.parentType;f\
or(t.parentType=\
\x22list\x22;c<r;){i=p\
,s=t.eMarks[c];c\
onst e=t.sCount[\
c]+p-(t.bMarks[c\
]+t.tShift[c]);l\
et n=e;for(;i<s;\
){const e=t.src.\
charCodeAt(i);if\
(9===e)n+=4-(n+t\
.bsCount[c])%4;e\
lse{if(32!==e)br\
eak;n++}i++}cons\
t h=i;let f;f=h>\
=s?1:n-e,f>4&&(f\
=1);const _=e+f;\
u=t.push(\x22list_i\
tem_open\x22,\x22li\x22,1\
),u.markup=Strin\
g.fromCharCode(d\
);const m=[c,0];\
u.map=m,l&&(u.in\
fo=t.src.slice(o\
,p-1));const D=t\
.tight,C=t.tShif\
t[c],y=t.sCount[\
c],E=t.listInden\
t;if(t.listInden\
t=t.blkIndent,t.\
blkIndent=_,t.ti\
ght=!0,t.tShift[\
c]=h-t.bMarks[c]\
,t.sCount[c]=n,h\
>=s&&t.isEmpty(c\
+1)?t.line=Math.\
min(t.line+2,r):\
t.md.block.token\
ize(t,c,r,!0),t.\
tight&&!g||(a=!1\
),g=t.line-c>1&&\
t.isEmpty(t.line\
-1),t.blkIndent=\
t.listIndent,t.l\
istIndent=E,t.tS\
hift[c]=C,t.sCou\
nt[c]=y,t.tight=\
D,u=t.push(\x22list\
_item_close\x22,\x22li\
\x22,-1),u.markup=S\
tring.fromCharCo\
de(d),c=t.line,m\
[1]=c,c>=r)break\
;if(t.sCount[c]<\
t.blkIndent)brea\
k;if(t.sCount[c]\
-t.blkIndent>=4)\
break;let A=!1;f\
or(let e=0,n=k.l\
ength;e<n;e++)if\
(k[e](t,c,r,!0))\
{A=!0;break}if(A\
)break;if(l){if(\
p=Nt(t,c),p<0)br\
eak;o=t.bMarks[c\
]+t.tShift[c]}el\
se if(p=Rt(t,c),\
p<0)break;if(d!=\
=t.src.charCodeA\
t(p-1))break}ret\
urn u=l?t.push(\x22\
ordered_list_clo\
se\x22,\x22ol\x22,-1):t.p\
ush(\x22bullet_list\
_close\x22,\x22ul\x22,-1)\
,u.markup=String\
.fromCharCode(d)\
,m[1]=c,t.line=c\
,t.parentType=D,\
a&&function(t,e)\
{const r=t.level\
+2;for(let n=e+2\
,s=t.tokens.leng\
th-2;n<s;n++)t.t\
okens[n].level==\
=r&&\x22paragraph_o\
pen\x22===t.tokens[\
n].type&&(t.toke\
ns[n+2].hidden=!\
0,t.tokens[n].hi\
dden=!0,n+=2)}(t\
,_),!0},[\x22paragr\
aph\x22,\x22reference\x22\
,\x22blockquote\x22]],\
[\x22reference\x22,fun\
ction(t,e,r,n){l\
et s=t.bMarks[e]\
+t.tShift[e],i=t\
.eMarks[e],o=e+1\
;if(t.sCount[e]-\
t.blkIndent>=4)r\
eturn!1;if(91!==\
t.src.charCodeAt\
(s))return!1;fun\
ction u(e){const\
 r=t.lineMax;if(\
e>=r||t.isEmpty(\
e))return null;l\
et n=!1;if(t.sCo\
unt[e]-t.blkInde\
nt>3&&(n=!0),t.s\
Count[e]<0&&(n=!\
0),!n){const n=t\
.md.block.ruler.\
getRules(\x22refere\
nce\x22),s=t.parent\
Type;t.parentTyp\
e=\x22reference\x22;le\
t i=!1;for(let s\
=0,o=n.length;s<\
o;s++)if(n[s](t,\
e,r,!0)){i=!0;br\
eak}if(t.parentT\
ype=s,i)return n\
ull}const s=t.bM\
arks[e]+t.tShift\
[e],i=t.eMarks[e\
];return t.src.s\
lice(s,i+1)}let \
c=t.src.slice(s,\
i+1);i=c.length;\
let a=-1;for(s=1\
;s<i;s++){const \
t=c.charCodeAt(s\
);if(91===t)retu\
rn!1;if(93===t){\
a=s;break}if(10=\
==t){const t=u(o\
);null!==t&&(c+=\
t,i=c.length,o++\
)}else if(92===t\
&&(s++,s<i&&10==\
=c.charCodeAt(s)\
)){const t=u(o);\
null!==t&&(c+=t,\
i=c.length,o++)}\
}if(a<0||58!==c.\
charCodeAt(a+1))\
return!1;for(s=a\
+2;s<i;s++){cons\
t t=c.charCodeAt\
(s);if(10===t){c\
onst t=u(o);null\
!==t&&(c+=t,i=c.\
length,o++)}else\
 if(!st(t))break\
}const l=t.md.he\
lpers.parseLinkD\
estination(c,s,i\
);if(!l.ok)retur\
n!1;const h=t.md\
.normalizeLink(l\
.str);if(!t.md.v\
alidateLink(h))r\
eturn!1;s=l.pos;\
const p=s,f=o,d=\
s;for(;s<i;s++){\
const t=c.charCo\
deAt(s);if(10===\
t){const t=u(o);\
null!==t&&(c+=t,\
i=c.length,o++)}\
else if(!st(t))b\
reak}let _,m=t.m\
d.helpers.parseL\
inkTitle(c,s,i);\
for(;m.can_conti\
nue;){const e=u(\
o);if(null===e)b\
reak;c+=e,s=i,i=\
c.length,o++,m=t\
.md.helpers.pars\
eLinkTitle(c,s,i\
,m)}for(s<i&&d!=\
=s&&m.ok?(_=m.st\
r,s=m.pos):(_=\x22\x22\
,s=p,o=f);s<i;){\
if(!st(c.charCod\
eAt(s)))break;s+\
+}if(s<i&&10!==c\
.charCodeAt(s)&&\
_)for(_=\x22\x22,s=p,o\
=f;s<i;){if(!st(\
c.charCodeAt(s))\
)break;s++}if(s<\
i&&10!==c.charCo\
deAt(s))return!1\
;const g=ct(c.sl\
ice(1,a));return\
!!g&&(n||(void 0\
===t.env.referen\
ces&&(t.env.refe\
rences={}),void \
0===t.env.refere\
nces[g]&&(t.env.\
references[g]={t\
itle:_,href:h}),\
t.line=o),!0)}],\
[\x22html_block\x22,fu\
nction(t,e,r,n){\
let s=t.bMarks[e\
]+t.tShift[e],i=\
t.eMarks[e];if(t\
.sCount[e]-t.blk\
Indent>=4)return\
!1;if(!t.md.opti\
ons.html)return!\
1;if(60!==t.src.\
charCodeAt(s))re\
turn!1;let o=t.s\
rc.slice(s,i),u=\
0;for(;u<$t.leng\
th&&!$t[u][0].te\
st(o);u++);if(u=\
==$t.length)retu\
rn!1;if(n)return\
 $t[u][2];let c=\
e+1;if(!$t[u][1]\
.test(o))for(;c<\
r&&!(t.sCount[c]\
<t.blkIndent);c+\
+)if(s=t.bMarks[\
c]+t.tShift[c],i\
=t.eMarks[c],o=t\
.src.slice(s,i),\
$t[u][1].test(o)\
){0!==o.length&&\
c++;break}t.line\
=c;const a=t.pus\
h(\x22html_block\x22,\x22\
\x22,0);return a.ma\
p=[e,c],a.conten\
t=t.getLines(e,c\
,t.blkIndent,!0)\
,!0},[\x22paragraph\
\x22,\x22reference\x22,\x22b\
lockquote\x22]],[\x22h\
eading\x22,function\
(t,e,r,n){let s=\
t.bMarks[e]+t.tS\
hift[e],i=t.eMar\
ks[e];if(t.sCoun\
t[e]-t.blkIndent\
>=4)return!1;let\
 o=t.src.charCod\
eAt(s);if(35!==o\
||s>=i)return!1;\
let u=1;for(o=t.\
src.charCodeAt(+\
+s);35===o&&s<i&\
&u<=6;)u++,o=t.s\
rc.charCodeAt(++\
s);if(u>6||s<i&&\
!st(o))return!1;\
if(n)return!0;i=\
t.skipSpacesBack\
(i,s);const c=t.\
skipCharsBack(i,\
35,s);c>s&&st(t.\
src.charCodeAt(c\
-1))&&(i=c),t.li\
ne=e+1;const a=t\
.push(\x22heading_o\
pen\x22,\x22h\x22+String(\
u),1);a.markup=\x22\
########\x22.slice(\
0,u),a.map=[e,t.\
line];const l=t.\
push(\x22inline\x22,\x22\x22\
,0);return l.con\
tent=t.src.slice\
(s,i).trim(),l.m\
ap=[e,t.line],l.\
children=[],t.pu\
sh(\x22heading_clos\
e\x22,\x22h\x22+String(u)\
,-1).markup=\x22###\
#####\x22.slice(0,u\
),!0},[\x22paragrap\
h\x22,\x22reference\x22,\x22\
blockquote\x22]],[\x22\
lheading\x22,functi\
on(t,e,r){const \
n=t.md.block.rul\
er.getRules(\x22par\
agraph\x22);if(t.sC\
ount[e]-t.blkInd\
ent>=4)return!1;\
const s=t.parent\
Type;t.parentTyp\
e=\x22paragraph\x22;le\
t i,o=0,u=e+1;fo\
r(;u<r&&!t.isEmp\
ty(u);u++){if(t.\
sCount[u]-t.blkI\
ndent>3)continue\
;if(t.sCount[u]>\
=t.blkIndent){le\
t e=t.bMarks[u]+\
t.tShift[u];cons\
t r=t.eMarks[u];\
if(e<r&&(i=t.src\
.charCodeAt(e),(\
45===i||61===i)&\
&(e=t.skipChars(\
e,i),e=t.skipSpa\
ces(e),e>=r))){o\
=61===i?1:2;brea\
k}}if(t.sCount[u\
]<0)continue;let\
 e=!1;for(let s=\
0,i=n.length;s<i\
;s++)if(n[s](t,u\
,r,!0)){e=!0;bre\
ak}if(e)break}if\
(!o)return!1;con\
st c=t.getLines(\
e,u,t.blkIndent,\
!1).trim();t.lin\
e=u+1;const a=t.\
push(\x22heading_op\
en\x22,\x22h\x22+String(o\
),1);a.markup=St\
ring.fromCharCod\
e(i),a.map=[e,t.\
line];const l=t.\
push(\x22inline\x22,\x22\x22\
,0);return l.con\
tent=c,l.map=[e,\
t.line-1],l.chil\
dren=[],t.push(\x22\
heading_close\x22,\x22\
h\x22+String(o),-1)\
.markup=String.f\
romCharCode(i),t\
.parentType=s,!0\
}],[\x22paragraph\x22,\
function(t,e,r){\
const n=t.md.blo\
ck.ruler.getRule\
s(\x22paragraph\x22),s\
=t.parentType;le\
t i=e+1;for(t.pa\
rentType=\x22paragr\
aph\x22;i<r&&!t.isE\
mpty(i);i++){if(\
t.sCount[i]-t.bl\
kIndent>3)contin\
ue;if(t.sCount[i\
]<0)continue;let\
 e=!1;for(let s=\
0,o=n.length;s<o\
;s++)if(n[s](t,i\
,r,!0)){e=!0;bre\
ak}if(e)break}co\
nst o=t.getLines\
(e,i,t.blkIndent\
,!1).trim();t.li\
ne=i,t.push(\x22par\
agraph_open\x22,\x22p\x22\
,1).map=[e,t.lin\
e];const u=t.pus\
h(\x22inline\x22,\x22\x22,0)\
;return u.conten\
t=o,u.map=[e,t.l\
ine],u.children=\
[],t.push(\x22parag\
raph_close\x22,\x22p\x22,\
-1),t.parentType\
=s,!0}]];functio\
n Ht(){this.rule\
r=new dt;for(let\
 t=0;t<Ut.length\
;t++)this.ruler.\
push(Ut[t][0],Ut\
[t][1],{alt:(Ut[\
t][2]||[]).slice\
()})}function Vt\
(t,e,r,n){this.s\
rc=t,this.env=r,\
this.md=e,this.t\
okens=n,this.tok\
ens_meta=Array(n\
.length),this.po\
s=0,this.posMax=\
this.src.length,\
this.level=0,thi\
s.pending=\x22\x22,thi\
s.pendingLevel=0\
,this.cache={},t\
his.delimiters=[\
],this._prev_del\
imiters=[],this.\
backticks={},thi\
s.backticksScann\
ed=!1,this.linkL\
evel=0}function \
Gt(t){switch(t){\
case 10:case 33:\
case 35:case 36:\
case 37:case 38:\
case 42:case 43:\
case 45:case 58:\
case 60:case 61:\
case 62:case 64:\
case 91:case 92:\
case 93:case 94:\
case 95:case 96:\
case 123:case 12\
5:case 126:retur\
n!0;default:retu\
rn!1}}Ht.prototy\
pe.tokenize=func\
tion(t,e,r){cons\
t n=this.ruler.g\
etRules(\x22\x22),s=n.\
length,i=t.md.op\
tions.maxNesting\
;let o=e,u=!1;fo\
r(;o<r&&(t.line=\
o=t.skipEmptyLin\
es(o),!(o>=r))&&\
!(t.sCount[o]<t.\
blkIndent);){if(\
t.level>=i){t.li\
ne=r;break}const\
 e=t.line;let c=\
!1;for(let i=0;i\
<s;i++)if(c=n[i]\
(t,o,r,!1),c){if\
(e>=t.line)throw\
 new Error(\x22bloc\
k rule didn't in\
crement state.li\
ne\x22);break}if(!c\
)throw new Error\
(\x22none of the bl\
ock rules matche\
d\x22);t.tight=!u,t\
.isEmpty(t.line-\
1)&&(u=!0),o=t.l\
ine,o<r&&t.isEmp\
ty(o)&&(u=!0,o++\
,t.line=o)}},Ht.\
prototype.parse=\
function(t,e,r,n\
){if(!t)return;c\
onst s=new this.\
State(t,e,r,n);t\
his.tokenize(s,s\
.line,s.lineMax)\
},Ht.prototype.S\
tate=It,Vt.proto\
type.pushPending\
=function(){cons\
t t=new _t(\x22text\
\x22,\x22\x22,0);return t\
.content=this.pe\
nding,t.level=th\
is.pendingLevel,\
this.tokens.push\
(t),this.pending\
=\x22\x22,t},Vt.protot\
ype.push=functio\
n(t,e,r){this.pe\
nding&&this.push\
Pending();const \
n=new _t(t,e,r);\
let s=null;retur\
n r<0&&(this.lev\
el--,this.delimi\
ters=this._prev_\
delimiters.pop()\
),n.level=this.l\
evel,r>0&&(this.\
level++,this._pr\
ev_delimiters.pu\
sh(this.delimite\
rs),this.delimit\
ers=[],s={delimi\
ters:this.delimi\
ters}),this.pend\
ingLevel=this.le\
vel,this.tokens.\
push(n),this.tok\
ens_meta.push(s)\
,n},Vt.prototype\
.scanDelims=func\
tion(t,e){const \
r=this.posMax,n=\
this.src.charCod\
eAt(t),s=t>0?thi\
s.src.charCodeAt\
(t-1):32;let i=t\
;for(;i<r&&this.\
src.charCodeAt(i\
)===n;)i++;const\
 o=i-t,u=i<r?thi\
s.src.charCodeAt\
(i):32,c=ut(s)||\
ot(String.fromCh\
arCode(s)),a=ut(\
u)||ot(String.fr\
omCharCode(u)),l\
=it(s),h=it(u),p\
=!h&&(!a||l||c),\
f=!l&&(!c||h||a)\
;return{can_open\
:p&&(e||!f||c),c\
an_close:f&&(e||\
!p||a),length:o}\
},Vt.prototype.T\
oken=_t;const Wt\
=/(?:^|[^a-z0-9.\
+-])([a-z][a-z0-\
9.+-]*)$/i;const\
 Jt=[];for(let t\
=0;t<256;t++)Jt.\
push(0);function\
 Qt(t,e){let r;c\
onst n=[],s=e.le\
ngth;for(let i=0\
;i<s;i++){const \
s=e[i];if(126!==\
s.marker)continu\
e;if(-1===s.end)\
continue;const o\
=e[s.end];r=t.to\
kens[s.token],r.\
type=\x22s_open\x22,r.\
tag=\x22s\x22,r.nestin\
g=1,r.markup=\x22~~\
\x22,r.content=\x22\x22,r\
=t.tokens[o.toke\
n],r.type=\x22s_clo\
se\x22,r.tag=\x22s\x22,r.\
nesting=-1,r.mar\
kup=\x22~~\x22,r.conte\
nt=\x22\x22,\x22text\x22===t\
.tokens[o.token-\
1].type&&\x22~\x22===t\
.tokens[o.token-\
1].content&&n.pu\
sh(o.token-1)}fo\
r(;n.length;){co\
nst e=n.pop();le\
t s=e+1;for(;s<t\
.tokens.length&&\
\x22s_close\x22===t.to\
kens[s].type;)s+\
+;s--,e!==s&&(r=\
t.tokens[s],t.to\
kens[s]=t.tokens\
[e],t.tokens[e]=\
r)}}\x22\x5c\x5c!\x5c\x22#$%&'(\
)*+,./:;<=>?@[]^\
_`{|}~-\x22.split(\x22\
\x22).forEach((func\
tion(t){Jt[t.cha\
rCodeAt(0)]=1}))\
;var Xt={tokeniz\
e:function(t,e){\
const r=t.pos,n=\
t.src.charCodeAt\
(r);if(e)return!\
1;if(126!==n)ret\
urn!1;const s=t.\
scanDelims(t.pos\
,!0);let i=s.len\
gth;const o=Stri\
ng.fromCharCode(\
n);if(i<2)return\
!1;let u;i%2&&(u\
=t.push(\x22text\x22,\x22\
\x22,0),u.content=o\
,i--);for(let e=\
0;e<i;e+=2)u=t.p\
ush(\x22text\x22,\x22\x22,0)\
,u.content=o+o,t\
.delimiters.push\
({marker:n,lengt\
h:0,token:t.toke\
ns.length-1,end:\
-1,open:s.can_op\
en,close:s.can_c\
lose});return t.\
pos+=s.length,!0\
},postProcess:fu\
nction(t){const \
e=t.tokens_meta,\
r=t.tokens_meta.\
length;Qt(t,t.de\
limiters);for(le\
t n=0;n<r;n++)e[\
n]&&e[n].delimit\
ers&&Qt(t,e[n].d\
elimiters)}};fun\
ction Yt(t,e){fo\
r(let r=e.length\
-1;r>=0;r--){con\
st n=e[r];if(95!\
==n.marker&&42!=\
=n.marker)contin\
ue;if(-1===n.end\
)continue;const \
s=e[n.end],i=r>0\
&&e[r-1].end===n\
.end+1&&e[r-1].m\
arker===n.marker\
&&e[r-1].token==\
=n.token-1&&e[n.\
end+1].token===s\
.token+1,o=Strin\
g.fromCharCode(n\
.marker),u=t.tok\
ens[n.token];u.t\
ype=i?\x22strong_op\
en\x22:\x22em_open\x22,u.\
tag=i?\x22strong\x22:\x22\
em\x22,u.nesting=1,\
u.markup=i?o+o:o\
,u.content=\x22\x22;co\
nst c=t.tokens[s\
.token];c.type=i\
?\x22strong_close\x22:\
\x22em_close\x22,c.tag\
=i?\x22strong\x22:\x22em\x22\
,c.nesting=-1,c.\
markup=i?o+o:o,c\
.content=\x22\x22,i&&(\
t.tokens[e[r-1].\
token].content=\x22\
\x22,t.tokens[e[n.e\
nd+1].token].con\
tent=\x22\x22,r--)}}va\
r Kt={tokenize:f\
unction(t,e){con\
st r=t.pos,n=t.s\
rc.charCodeAt(r)\
;if(e)return!1;i\
f(95!==n&&42!==n\
)return!1;const \
s=t.scanDelims(t\
.pos,42===n);for\
(let e=0;e<s.len\
gth;e++){t.push(\
\x22text\x22,\x22\x22,0).con\
tent=String.from\
CharCode(n),t.de\
limiters.push({m\
arker:n,length:s\
.length,token:t.\
tokens.length-1,\
end:-1,open:s.ca\
n_open,close:s.c\
an_close})}retur\
n t.pos+=s.lengt\
h,!0},postProces\
s:function(t){co\
nst e=t.tokens_m\
eta,r=t.tokens_m\
eta.length;Yt(t,\
t.delimiters);fo\
r(let n=0;n<r;n+\
+)e[n]&&e[n].del\
imiters&&Yt(t,e[\
n].delimiters)}}\
;const te=/^([a-\
zA-Z0-9.!#$%&'*+\
/=?^_`{|}~-]+@[a\
-zA-Z0-9](?:[a-z\
A-Z0-9-]{0,61}[a\
-zA-Z0-9])?(?:\x5c.\
[a-zA-Z0-9](?:[a\
-zA-Z0-9-]{0,61}\
[a-zA-Z0-9])?)*)\
$/,ee=/^([a-zA-Z\
][a-zA-Z0-9+.-]{\
1,31}):([^<>\x5cx00\
-\x5cx20]*)$/;const\
 re=/^&#((?:x[a-\
f0-9]{1,6}|[0-9]\
{1,7}));/i,ne=/^\
&([a-z][a-z0-9]{\
1,31});/i;functi\
on se(t){const e\
={},r=t.length;i\
f(!r)return;let \
n=0,s=-2;const i\
=[];for(let o=0;\
o<r;o++){const r\
=t[o];if(i.push(\
0),t[n].marker==\
=r.marker&&s===r\
.token-1||(n=o),\
s=r.token,r.leng\
th=r.length||0,!\
r.close)continue\
;e.hasOwnPropert\
y(r.marker)||(e[\
r.marker]=[-1,-1\
,-1,-1,-1,-1]);c\
onst u=e[r.marke\
r][(r.open?3:0)+\
r.length%3];let \
c=n-i[n]-1,a=c;f\
or(;c>u;c-=i[c]+\
1){const e=t[c];\
if(e.marker===r.\
marker&&(e.open&\
&e.end<0)){let n\
=!1;if((e.close|\
|r.open)&&(e.len\
gth+r.length)%3=\
=0&&(e.length%3=\
=0&&r.length%3==\
0||(n=!0)),!n){c\
onst n=c>0&&!t[c\
-1].open?i[c-1]+\
1:0;i[o]=o-c+n,i\
[c]=n,r.open=!1,\
e.end=o,e.close=\
!1,a=-1,s=-2;bre\
ak}}}-1!==a&&(e[\
r.marker][(r.ope\
n?3:0)+(r.length\
||0)%3]=a)}}cons\
t ie=[[\x22text\x22,fu\
nction(t,e){let \
r=t.pos;for(;r<t\
.posMax&&!Gt(t.s\
rc.charCodeAt(r)\
);)r++;return r!\
==t.pos&&(e||(t.\
pending+=t.src.s\
lice(t.pos,r)),t\
.pos=r,!0)}],[\x22l\
inkify\x22,function\
(t,e){if(!t.md.o\
ptions.linkify)r\
eturn!1;if(t.lin\
kLevel>0)return!\
1;const r=t.pos;\
if(r+3>t.posMax)\
return!1;if(58!=\
=t.src.charCodeA\
t(r))return!1;if\
(47!==t.src.char\
CodeAt(r+1))retu\
rn!1;if(47!==t.s\
rc.charCodeAt(r+\
2))return!1;cons\
t n=t.pending.ma\
tch(Wt);if(!n)re\
turn!1;const s=n\
[1],i=t.md.linki\
fy.matchAtStart(\
t.src.slice(r-s.\
length));if(!i)r\
eturn!1;let o=i.\
url;if(o.length<\
=s.length)return\
!1;o=o.replace(/\
\x5c*+$/,\x22\x22);const \
u=t.md.normalize\
Link(o);if(!t.md\
.validateLink(u)\
)return!1;if(!e)\
{t.pending=t.pen\
ding.slice(0,-s.\
length);const e=\
t.push(\x22link_ope\
n\x22,\x22a\x22,1);e.attr\
s=[[\x22href\x22,u]],e\
.markup=\x22linkify\
\x22,e.info=\x22auto\x22;\
t.push(\x22text\x22,\x22\x22\
,0).content=t.md\
.normalizeLinkTe\
xt(o);const r=t.\
push(\x22link_close\
\x22,\x22a\x22,-1);r.mark\
up=\x22linkify\x22,r.i\
nfo=\x22auto\x22}retur\
n t.pos+=o.lengt\
h-s.length,!0}],\
[\x22newline\x22,funct\
ion(t,e){let r=t\
.pos;if(10!==t.s\
rc.charCodeAt(r)\
)return!1;const \
n=t.pending.leng\
th-1,s=t.posMax;\
if(!e)if(n>=0&&3\
2===t.pending.ch\
arCodeAt(n))if(n\
>=1&&32===t.pend\
ing.charCodeAt(n\
-1)){let e=n-1;f\
or(;e>=1&&32===t\
.pending.charCod\
eAt(e-1);)e--;t.\
pending=t.pendin\
g.slice(0,e),t.p\
ush(\x22hardbreak\x22,\
\x22br\x22,0)}else t.p\
ending=t.pending\
.slice(0,-1),t.p\
ush(\x22softbreak\x22,\
\x22br\x22,0);else t.p\
ush(\x22softbreak\x22,\
\x22br\x22,0);for(r++;\
r<s&&st(t.src.ch\
arCodeAt(r));)r+\
+;return t.pos=r\
,!0}],[\x22escape\x22,\
function(t,e){le\
t r=t.pos;const \
n=t.posMax;if(92\
!==t.src.charCod\
eAt(r))return!1;\
if(r++,r>=n)retu\
rn!1;let s=t.src\
.charCodeAt(r);i\
f(10===s){for(e|\
|t.push(\x22hardbre\
ak\x22,\x22br\x22,0),r++;\
r<n&&(s=t.src.ch\
arCodeAt(r),st(s\
));)r++;return t\
.pos=r,!0}let i=\
t.src[r];if(s>=5\
5296&&s<=56319&&\
r+1<n){const e=t\
.src.charCodeAt(\
r+1);e>=56320&&e\
<=57343&&(i+=t.s\
rc[r+1],r++)}con\
st o=\x22\x5c\x5c\x22+i;if(!\
e){const e=t.pus\
h(\x22text_special\x22\
,\x22\x22,0);s<256&&0!\
==Jt[s]?e.conten\
t=i:e.content=o,\
e.markup=o,e.inf\
o=\x22escape\x22}retur\
n t.pos=r+1,!0}]\
,[\x22backticks\x22,fu\
nction(t,e){let \
r=t.pos;if(96!==\
t.src.charCodeAt\
(r))return!1;con\
st n=r;r++;const\
 s=t.posMax;for(\
;r<s&&96===t.src\
.charCodeAt(r);)\
r++;const i=t.sr\
c.slice(n,r),o=i\
.length;if(t.bac\
kticksScanned&&(\
t.backticks[o]||\
0)<=n)return e||\
(t.pending+=i),t\
.pos+=o,!0;let u\
,c=r;for(;-1!==(\
u=t.src.indexOf(\
\x22`\x22,c));){for(c=\
u+1;c<s&&96===t.\
src.charCodeAt(c\
);)c++;const n=c\
-u;if(n===o){if(\
!e){const e=t.pu\
sh(\x22code_inline\x22\
,\x22code\x22,0);e.mar\
kup=i,e.content=\
t.src.slice(r,u)\
.replace(/\x5cn/g,\x22\
 \x22).replace(/^ (\
.+) $/,\x22$1\x22)}ret\
urn t.pos=c,!0}t\
.backticks[n]=u}\
return t.backtic\
ksScanned=!0,e||\
(t.pending+=i),t\
.pos+=o,!0}],[\x22s\
trikethrough\x22,Xt\
.tokenize],[\x22emp\
hasis\x22,Kt.tokeni\
ze],[\x22link\x22,func\
tion(t,e){let r,\
n,s,i,o=\x22\x22,u=\x22\x22,\
c=t.pos,a=!0;if(\
91!==t.src.charC\
odeAt(t.pos))ret\
urn!1;const l=t.\
pos,h=t.posMax,p\
=t.pos+1,f=t.md.\
helpers.parseLin\
kLabel(t,t.pos,!\
0);if(f<0)return\
!1;let d=f+1;if(\
d<h&&40===t.src.\
charCodeAt(d)){f\
or(a=!1,d++;d<h&\
&(r=t.src.charCo\
deAt(d),st(r)||1\
0===r);d++);if(d\
>=h)return!1;if(\
c=d,s=t.md.helpe\
rs.parseLinkDest\
ination(t.src,d,\
t.posMax),s.ok){\
for(o=t.md.norma\
lizeLink(s.str),\
t.md.validateLin\
k(o)?d=s.pos:o=\x22\
\x22,c=d;d<h&&(r=t.\
src.charCodeAt(d\
),st(r)||10===r)\
;d++);if(s=t.md.\
helpers.parseLin\
kTitle(t.src,d,t\
.posMax),d<h&&c!\
==d&&s.ok)for(u=\
s.str,d=s.pos;d<\
h&&(r=t.src.char\
CodeAt(d),st(r)|\
|10===r);d++);}(\
d>=h||41!==t.src\
.charCodeAt(d))&\
&(a=!0),d++}if(a\
){if(void 0===t.\
env.references)r\
eturn!1;if(d<h&&\
91===t.src.charC\
odeAt(d)?(c=d+1,\
d=t.md.helpers.p\
arseLinkLabel(t,\
d),d>=0?n=t.src.\
slice(c,d++):d=f\
+1):d=f+1,n||(n=\
t.src.slice(p,f)\
),i=t.env.refere\
nces[ct(n)],!i)r\
eturn t.pos=l,!1\
;o=i.href,u=i.ti\
tle}if(!e){t.pos\
=p,t.posMax=f;co\
nst e=[[\x22href\x22,o\
]];t.push(\x22link_\
open\x22,\x22a\x22,1).att\
rs=e,u&&e.push([\
\x22title\x22,u]),t.li\
nkLevel++,t.md.i\
nline.tokenize(t\
),t.linkLevel--,\
t.push(\x22link_clo\
se\x22,\x22a\x22,-1)}retu\
rn t.pos=d,t.pos\
Max=h,!0}],[\x22ima\
ge\x22,function(t,e\
){let r,n,s,i,o,\
u,c,a,l=\x22\x22;const\
 h=t.pos,p=t.pos\
Max;if(33!==t.sr\
c.charCodeAt(t.p\
os))return!1;if(\
91!==t.src.charC\
odeAt(t.pos+1))r\
eturn!1;const f=\
t.pos+2,d=t.md.h\
elpers.parseLink\
Label(t,t.pos+1,\
!1);if(d<0)retur\
n!1;if(i=d+1,i<p\
&&40===t.src.cha\
rCodeAt(i)){for(\
i++;i<p&&(r=t.sr\
c.charCodeAt(i),\
st(r)||10===r);i\
++);if(i>=p)retu\
rn!1;for(a=i,u=t\
.md.helpers.pars\
eLinkDestination\
(t.src,i,t.posMa\
x),u.ok&&(l=t.md\
.normalizeLink(u\
.str),t.md.valid\
ateLink(l)?i=u.p\
os:l=\x22\x22),a=i;i<p\
&&(r=t.src.charC\
odeAt(i),st(r)||\
10===r);i++);if(\
u=t.md.helpers.p\
arseLinkTitle(t.\
src,i,t.posMax),\
i<p&&a!==i&&u.ok\
)for(c=u.str,i=u\
.pos;i<p&&(r=t.s\
rc.charCodeAt(i)\
,st(r)||10===r);\
i++);else c=\x22\x22;i\
f(i>=p||41!==t.s\
rc.charCodeAt(i)\
)return t.pos=h,\
!1;i++}else{if(v\
oid 0===t.env.re\
ferences)return!\
1;if(i<p&&91===t\
.src.charCodeAt(\
i)?(a=i+1,i=t.md\
.helpers.parseLi\
nkLabel(t,i),i>=\
0?s=t.src.slice(\
a,i++):i=d+1):i=\
d+1,s||(s=t.src.\
slice(f,d)),o=t.\
env.references[c\
t(s)],!o)return \
t.pos=h,!1;l=o.h\
ref,c=o.title}if\
(!e){n=t.src.sli\
ce(f,d);const e=\
[];t.md.inline.p\
arse(n,t.md,t.en\
v,e);const r=t.p\
ush(\x22image\x22,\x22img\
\x22,0),s=[[\x22src\x22,l\
],[\x22alt\x22,\x22\x22]];r.\
attrs=s,r.childr\
en=e,r.content=n\
,c&&s.push([\x22tit\
le\x22,c])}return t\
.pos=i,t.posMax=\
p,!0}],[\x22autolin\
k\x22,function(t,e)\
{let r=t.pos;if(\
60!==t.src.charC\
odeAt(r))return!\
1;const n=t.pos,\
s=t.posMax;for(;\
;){if(++r>=s)ret\
urn!1;const e=t.\
src.charCodeAt(r\
);if(60===e)retu\
rn!1;if(62===e)b\
reak}const i=t.s\
rc.slice(n+1,r);\
if(ee.test(i)){c\
onst r=t.md.norm\
alizeLink(i);if(\
!t.md.validateLi\
nk(r))return!1;i\
f(!e){const e=t.\
push(\x22link_open\x22\
,\x22a\x22,1);e.attrs=\
[[\x22href\x22,r]],e.m\
arkup=\x22autolink\x22\
,e.info=\x22auto\x22;t\
.push(\x22text\x22,\x22\x22,\
0).content=t.md.\
normalizeLinkTex\
t(i);const n=t.p\
ush(\x22link_close\x22\
,\x22a\x22,-1);n.marku\
p=\x22autolink\x22,n.i\
nfo=\x22auto\x22}retur\
n t.pos+=i.lengt\
h+2,!0}if(te.tes\
t(i)){const r=t.\
md.normalizeLink\
(\x22mailto:\x22+i);if\
(!t.md.validateL\
ink(r))return!1;\
if(!e){const e=t\
.push(\x22link_open\
\x22,\x22a\x22,1);e.attrs\
=[[\x22href\x22,r]],e.\
markup=\x22autolink\
\x22,e.info=\x22auto\x22;\
t.push(\x22text\x22,\x22\x22\
,0).content=t.md\
.normalizeLinkTe\
xt(i);const n=t.\
push(\x22link_close\
\x22,\x22a\x22,-1);n.mark\
up=\x22autolink\x22,n.\
info=\x22auto\x22}retu\
rn t.pos+=i.leng\
th+2,!0}return!1\
}],[\x22html_inline\
\x22,function(t,e){\
if(!t.md.options\
.html)return!1;c\
onst r=t.posMax,\
n=t.pos;if(60!==\
t.src.charCodeAt\
(n)||n+2>=r)retu\
rn!1;const s=t.s\
rc.charCodeAt(n+\
1);if(33!==s&&63\
!==s&&47!==s&&!f\
unction(t){const\
 e=32|t;return e\
>=97&&e<=122}(s)\
)return!1;const \
i=t.src.slice(n)\
.match(jt);if(!i\
)return!1;if(!e)\
{const e=t.push(\
\x22html_inline\x22,\x22\x22\
,0);e.content=i[\
0],o=e.content,/\
^<a[>\x5cs]/i.test(\
o)&&t.linkLevel+\
+,function(t){re\
turn/^<\x5c/a\x5cs*>/i\
.test(t)}(e.cont\
ent)&&t.linkLeve\
l--}var o;return\
 t.pos+=i[0].len\
gth,!0}],[\x22entit\
y\x22,function(t,e)\
{const r=t.pos,n\
=t.posMax;if(38!\
==t.src.charCode\
At(r))return!1;i\
f(r+1>=n)return!\
1;if(35===t.src.\
charCodeAt(r+1))\
{const n=t.src.s\
lice(r).match(re\
);if(n){if(!e){c\
onst e=\x22x\x22===n[1\
][0].toLowerCase\
()?parseInt(n[1]\
.slice(1),16):pa\
rseInt(n[1],10),\
r=t.push(\x22text_s\
pecial\x22,\x22\x22,0);r.\
content=V(e)?G(e\
):G(65533),r.mar\
kup=n[0],r.info=\
\x22entity\x22}return \
t.pos+=n[0].leng\
th,!0}}else{cons\
t n=t.src.slice(\
r).match(ne);if(\
n){const r=j(n[0\
]);if(r!==n[0]){\
if(!e){const e=t\
.push(\x22text_spec\
ial\x22,\x22\x22,0);e.con\
tent=r,e.markup=\
n[0],e.info=\x22ent\
ity\x22}return t.po\
s+=n[0].length,!\
0}}}return!1}]],\
oe=[[\x22balance_pa\
irs\x22,function(t)\
{const e=t.token\
s_meta,r=t.token\
s_meta.length;se\
(t.delimiters);f\
or(let t=0;t<r;t\
++)e[t]&&e[t].de\
limiters&&se(e[t\
].delimiters)}],\
[\x22strikethrough\x22\
,Xt.postProcess]\
,[\x22emphasis\x22,Kt.\
postProcess],[\x22f\
ragments_join\x22,f\
unction(t){let e\
,r,n=0;const s=t\
.tokens,i=t.toke\
ns.length;for(e=\
r=0;e<i;e++)s[e]\
.nesting<0&&n--,\
s[e].level=n,s[e\
].nesting>0&&n++\
,\x22text\x22===s[e].t\
ype&&e+1<i&&\x22tex\
t\x22===s[e+1].type\
?s[e+1].content=\
s[e].content+s[e\
+1].content:(e!=\
=r&&(s[r]=s[e]),\
r++);e!==r&&(s.l\
ength=r)}]];func\
tion ue(){this.r\
uler=new dt;for(\
let t=0;t<ie.len\
gth;t++)this.rul\
er.push(ie[t][0]\
,ie[t][1]);this.\
ruler2=new dt;fo\
r(let t=0;t<oe.l\
ength;t++)this.r\
uler2.push(oe[t]\
[0],oe[t][1])}fu\
nction ce(t){ret\
urn Array.protot\
ype.slice.call(a\
rguments,1).forE\
ach((function(e)\
{e&&Object.keys(\
e).forEach((func\
tion(r){t[r]=e[r\
]}))})),t}functi\
on ae(t){return \
Object.prototype\
.toString.call(t\
)}function le(t)\
{return\x22[object \
Function]\x22===ae(\
t)}function he(t\
){return t.repla\
ce(/[.?*+^$[\x5c]\x5c\x5c\
(){}|-]/g,\x22\x5c\x5c$&\x22\
)}ue.prototype.s\
kipToken=functio\
n(t){const e=t.p\
os,r=this.ruler.\
getRules(\x22\x22),n=r\
.length,s=t.md.o\
ptions.maxNestin\
g,i=t.cache;if(v\
oid 0!==i[e])ret\
urn void(t.pos=i\
[e]);let o=!1;if\
(t.level<s){for(\
let s=0;s<n;s++)\
if(t.level++,o=r\
[s](t,!0),t.leve\
l--,o){if(e>=t.p\
os)throw new Err\
or(\x22inline rule \
didn't increment\
 state.pos\x22);bre\
ak}}else t.pos=t\
.posMax;o||t.pos\
++,i[e]=t.pos},u\
e.prototype.toke\
nize=function(t)\
{const e=this.ru\
ler.getRules(\x22\x22)\
,r=e.length,n=t.\
posMax,s=t.md.op\
tions.maxNesting\
;for(;t.pos<n;){\
const i=t.pos;le\
t o=!1;if(t.leve\
l<s)for(let n=0;\
n<r;n++)if(o=e[n\
](t,!1),o){if(i>\
=t.pos)throw new\
 Error(\x22inline r\
ule didn't incre\
ment state.pos\x22)\
;break}if(o){if(\
t.pos>=n)break}e\
lse t.pending+=t\
.src[t.pos++]}t.\
pending&&t.pushP\
ending()},ue.pro\
totype.parse=fun\
ction(t,e,r,n){c\
onst s=new this.\
State(t,e,r,n);t\
his.tokenize(s);\
const i=this.rul\
er2.getRules(\x22\x22)\
,o=i.length;for(\
let t=0;t<o;t++)\
i[t](s)},ue.prot\
otype.State=Vt;c\
onst pe={fuzzyLi\
nk:!0,fuzzyEmail\
:!0,fuzzyIP:!1};\
const fe={\x22http:\
\x22:{validate:func\
tion(t,e,r){cons\
t n=t.slice(e);r\
eturn r.re.http|\
|(r.re.http=new \
RegExp(\x22^\x5c\x5c/\x5c\x5c/\x22\
+r.re.src_auth+r\
.re.src_host_por\
t_strict+r.re.sr\
c_path,\x22i\x22)),r.r\
e.http.test(n)?n\
.match(r.re.http\
)[0].length:0}},\
\x22https:\x22:\x22http:\x22\
,\x22ftp:\x22:\x22http:\x22,\
\x22//\x22:{validate:f\
unction(t,e,r){c\
onst n=t.slice(e\
);return r.re.no\
_http||(r.re.no_\
http=new RegExp(\
\x22^\x22+r.re.src_aut\
h+\x22(?:localhost|\
(?:(?:\x22+r.re.src\
_domain+\x22)\x5c\x5c.)+\x22\
+r.re.src_domain\
_root+\x22)\x22+r.re.s\
rc_port+r.re.src\
_host_terminator\
+r.re.src_path,\x22\
i\x22)),r.re.no_htt\
p.test(n)?e>=3&&\
\x22:\x22===t[e-3]||e>\
=3&&\x22/\x22===t[e-3]\
?0:n.match(r.re.\
no_http)[0].leng\
th:0}},\x22mailto:\x22\
:{validate:funct\
ion(t,e,r){const\
 n=t.slice(e);re\
turn r.re.mailto\
||(r.re.mailto=n\
ew RegExp(\x22^\x22+r.\
re.src_email_nam\
e+\x22@\x22+r.re.src_h\
ost_strict,\x22i\x22))\
,r.re.mailto.tes\
t(n)?n.match(r.r\
e.mailto)[0].len\
gth:0}}},de=\x22a[c\
defgilmnoqrstuwx\
z]|b[abdefghijmn\
orstvwyz]|c[acdf\
ghiklmnoruvwxyz]\
|d[ejkmoz]|e[ceg\
rstu]|f[ijkmor]|\
g[abdefghilmnpqr\
stuwy]|h[kmnrtu]\
|i[delmnoqrst]|j\
[emop]|k[eghimnp\
rwyz]|l[abcikrst\
uvy]|m[acdeghklm\
nopqrstuvwxyz]|n\
[acefgilopruz]|o\
m|p[aefghklmnrst\
wy]|qa|r[eosuw]|\
s[abcdeghijklmno\
rtuvxyz]|t[cdfgh\
jklmnortvwz]|u[a\
gksyz]|v[aceginu\
]|w[fs]|y[et]|z[\
amw]\x22,_e=\x22biz|co\
m|edu|gov|net|or\
g|pro|web|xxx|ae\
ro|asia|coop|inf\
o|museum|name|sh\
op|\x5cu0440\x5cu0444\x22\
.split(\x22|\x22);func\
tion me(t){const\
 e=t.re=function\
(t){const e={};t\
=t||{},e.src_Any\
=C.source,e.src_\
Cc=y.source,e.sr\
c_Z=b.source,e.s\
rc_P=E.source,e.\
src_ZPCc=[e.src_\
Z,e.src_P,e.src_\
Cc].join(\x22|\x22),e.\
src_ZCc=[e.src_Z\
,e.src_Cc].join(\
\x22|\x22);const r=\x22[>\
<\x5cuff5c]\x22;return\
 e.src_pseudo_le\
tter=\x22(?:(?![><\x5c\
uff5c]|\x22+e.src_Z\
PCc+\x22)\x22+e.src_An\
y+\x22)\x22,e.src_ip4=\
\x22(?:(25[0-5]|2[0\
-4][0-9]|[01]?[0\
-9][0-9]?)\x5c\x5c.){3\
}(25[0-5]|2[0-4]\
[0-9]|[01]?[0-9]\
[0-9]?)\x22,e.src_a\
uth=\x22(?:(?:(?!\x22+\
e.src_ZCc+\x22|[@/\x5c\
\x5c[\x5c\x5c]()]).)+@)?\x22\
,e.src_port=\x22(?:\
:(?:6(?:[0-4]\x5c\x5cd\
{3}|5(?:[0-4]\x5c\x5cd\
{2}|5(?:[0-2]\x5c\x5cd\
|3[0-5])))|[1-5]\
?\x5c\x5cd{1,4}))?\x22,e.\
src_host_termina\
tor=\x22(?=$|[><\x5cuf\
f5c]|\x22+e.src_ZPC\
c+\x22)(?!\x22+(t[\x22---\
\x22]?\x22-(?!--)|\x22:\x22-\
|\x22)+\x22_|:\x5c\x5cd|\x5c\x5c.-\
|\x5c\x5c.(?!$|\x22+e.src\
_ZPCc+\x22))\x22,e.src\
_path=\x22(?:[/?#](\
?:(?!\x22+e.src_ZCc\
+\x22|\x22+r+\x22|[()[\x5c\x5c]\
{}.,\x5c\x22'?!\x5c\x5c-;]).\
|\x5c\x5c[(?:(?!\x22+e.sr\
c_ZCc+\x22|\x5c\x5c]).)*\x5c\
\x5c]|\x5c\x5c((?:(?!\x22+e.\
src_ZCc+\x22|[)]).)\
*\x5c\x5c)|\x5c\x5c{(?:(?!\x22+\
e.src_ZCc+'|[}])\
.)*\x5c\x5c}|\x5c\x5c\x22(?:(?!\
'+e.src_ZCc+'|[\x22\
]).)+\x5c\x5c\x22|\x5c\x5c\x5c'(?:\
(?!'+e.src_ZCc+\x22\
|[']).)+\x5c\x5c'|\x5c\x5c'(\
?=\x22+e.src_pseudo\
_letter+\x22|[-])|\x5c\
\x5c.{2,}[a-zA-Z0-9\
%/&]|\x5c\x5c.(?!\x22+e.s\
rc_ZCc+\x22|[.]|$)|\
\x22+(t[\x22---\x22]?\x22\x5c\x5c-\
(?!--(?:[^-]|$))\
(?:-*)|\x22:\x22\x5c\x5c-+|\x22\
)+\x22,(?!\x22+e.src_Z\
Cc+\x22|$)|;(?!\x22+e.\
src_ZCc+\x22|$)|\x5c\x5c!\
+(?!\x22+e.src_ZCc+\
\x22|[!]|$)|\x5c\x5c?(?!\x22\
+e.src_ZCc+\x22|[?]\
|$))+|\x5c\x5c/)?\x22,e.s\
rc_email_name='[\
\x5c\x5c-;:&=\x5c\x5c+\x5c\x5c$,\x5c\x5c\
.a-zA-Z0-9_][\x5c\x5c-\
;:&=\x5c\x5c+\x5c\x5c$,\x5c\x5c\x22\x5c\x5c\
.a-zA-Z0-9_]*',e\
.src_xn=\x22xn--[a-\
z0-9\x5c\x5c-]{1,59}\x22,\
e.src_domain_roo\
t=\x22(?:\x22+e.src_xn\
+\x22|\x22+e.src_pseud\
o_letter+\x22{1,63}\
)\x22,e.src_domain=\
\x22(?:\x22+e.src_xn+\x22\
|(?:\x22+e.src_pseu\
do_letter+\x22)|(?:\
\x22+e.src_pseudo_l\
etter+\x22(?:-|\x22+e.\
src_pseudo_lette\
r+\x22){0,61}\x22+e.sr\
c_pseudo_letter+\
\x22))\x22,e.src_host=\
\x22(?:(?:(?:(?:\x22+e\
.src_domain+\x22)\x5c\x5c\
.)*\x22+e.src_domai\
n+\x22))\x22,e.tpl_hos\
t_fuzzy=\x22(?:\x22+e.\
src_ip4+\x22|(?:(?:\
(?:\x22+e.src_domai\
n+\x22)\x5c\x5c.)+(?:%TLD\
S%)))\x22,e.tpl_hos\
t_no_ip_fuzzy=\x22(\
?:(?:(?:\x22+e.src_\
domain+\x22)\x5c\x5c.)+(?\
:%TLDS%))\x22,e.src\
_host_strict=e.s\
rc_host+e.src_ho\
st_terminator,e.\
tpl_host_fuzzy_s\
trict=e.tpl_host\
_fuzzy+e.src_hos\
t_terminator,e.s\
rc_host_port_str\
ict=e.src_host+e\
.src_port+e.src_\
host_terminator,\
e.tpl_host_port_\
fuzzy_strict=e.t\
pl_host_fuzzy+e.\
src_port+e.src_h\
ost_terminator,e\
.tpl_host_port_n\
o_ip_fuzzy_stric\
t=e.tpl_host_no_\
ip_fuzzy+e.src_p\
ort+e.src_host_t\
erminator,e.tpl_\
host_fuzzy_test=\
\x22localhost|www\x5c\x5c\
.|\x5c\x5c.\x5c\x5cd{1,3}\x5c\x5c.\
|(?:\x5c\x5c.(?:%TLDS%\
)(?:\x22+e.src_ZPCc\
+\x22|>|$))\x22,e.tpl_\
email_fuzzy='(^|\
[><\x5cuff5c]|\x22|\x5c\x5c(\
|'+e.src_ZCc+\x22)(\
\x22+e.src_email_na\
me+\x22@\x22+e.tpl_hos\
t_fuzzy_strict+\x22\
)\x22,e.tpl_link_fu\
zzy=\x22(^|(?![.:/\x5c\
\x5c-_@])(?:[$+<=>^\
`|\x5cuff5c]|\x22+e.sr\
c_ZPCc+\x22))((?![$\
+<=>^`|\x5cuff5c])\x22\
+e.tpl_host_port\
_fuzzy_strict+e.\
src_path+\x22)\x22,e.t\
pl_link_no_ip_fu\
zzy=\x22(^|(?![.:/\x5c\
\x5c-_@])(?:[$+<=>^\
`|\x5cuff5c]|\x22+e.sr\
c_ZPCc+\x22))((?![$\
+<=>^`|\x5cuff5c])\x22\
+e.tpl_host_port\
_no_ip_fuzzy_str\
ict+e.src_path+\x22\
)\x22,e}(t.__opts__\
),r=t.__tlds__.s\
lice();function \
n(t){return t.re\
place(\x22%TLDS%\x22,e\
.src_tlds)}t.onC\
ompile(),t.__tld\
s_replaced__||r.\
push(de),r.push(\
e.src_xn),e.src_\
tlds=r.join(\x22|\x22)\
,e.email_fuzzy=R\
egExp(n(e.tpl_em\
ail_fuzzy),\x22i\x22),\
e.link_fuzzy=Reg\
Exp(n(e.tpl_link\
_fuzzy),\x22i\x22),e.l\
ink_no_ip_fuzzy=\
RegExp(n(e.tpl_l\
ink_no_ip_fuzzy)\
,\x22i\x22),e.host_fuz\
zy_test=RegExp(n\
(e.tpl_host_fuzz\
y_test),\x22i\x22);con\
st s=[];function\
 i(t,e){throw ne\
w Error('(Linkif\
yIt) Invalid sch\
ema \x22'+t+'\x22: '+e\
)}t.__compiled__\
={},Object.keys(\
t.__schemas__).f\
orEach((function\
(e){const r=t.__\
schemas__[e];if(\
null===r)return;\
const n={validat\
e:null,link:null\
};if(t.__compile\
d__[e]=n,\x22[objec\
t Object]\x22===ae(\
r))return!functi\
on(t){return\x22[ob\
ject RegExp]\x22===\
ae(t)}(r.validat\
e)?le(r.validate\
)?n.validate=r.v\
alidate:i(e,r):n\
.validate=functi\
on(t){return fun\
ction(e,r){const\
 n=e.slice(r);re\
turn t.test(n)?n\
.match(t)[0].len\
gth:0}}(r.valida\
te),void(le(r.no\
rmalize)?n.norma\
lize=r.normalize\
:r.normalize?i(e\
,r):n.normalize=\
function(t,e){e.\
normalize(t)});!\
function(t){retu\
rn\x22[object Strin\
g]\x22===ae(t)}(r)?\
i(e,r):s.push(e)\
})),s.forEach((f\
unction(e){t.__c\
ompiled__[t.__sc\
hemas__[e]]&&(t.\
__compiled__[e].\
validate=t.__com\
piled__[t.__sche\
mas__[e]].valida\
te,t.__compiled_\
_[e].normalize=t\
.__compiled__[t.\
__schemas__[e]].\
normalize)})),t.\
__compiled__[\x22\x22]\
={validate:null,\
normalize:functi\
on(t,e){e.normal\
ize(t)}};const o\
=Object.keys(t._\
_compiled__).fil\
ter((function(e)\
{return e.length\
>0&&t.__compiled\
__[e]})).map(he)\
.join(\x22|\x22);t.re.\
schema_test=RegE\
xp(\x22(^|(?!_)(?:[\
><\x5cuff5c]|\x22+e.sr\
c_ZPCc+\x22))(\x22+o+\x22\
)\x22,\x22i\x22),t.re.sch\
ema_search=RegEx\
p(\x22(^|(?!_)(?:[>\
<\x5cuff5c]|\x22+e.src\
_ZPCc+\x22))(\x22+o+\x22)\
\x22,\x22ig\x22),t.re.sch\
ema_at_start=Reg\
Exp(\x22^\x22+t.re.sch\
ema_search.sourc\
e,\x22i\x22),t.re.pret\
est=RegExp(\x22(\x22+t\
.re.schema_test.\
source+\x22)|(\x22+t.r\
e.host_fuzzy_tes\
t.source+\x22)|@\x22,\x22\
i\x22),function(t){\
t.__index__=-1,t\
.__text_cache__=\
\x22\x22}(t)}function \
ge(t,e){const r=\
t.__index__,n=t.\
__last_index__,s\
=t.__text_cache_\
_.slice(r,n);thi\
s.schema=t.__sch\
ema__.toLowerCas\
e(),this.index=r\
+e,this.lastInde\
x=n+e,this.raw=s\
,this.text=s,thi\
s.url=s}function\
 ke(t,e){const r\
=new ge(t,e);ret\
urn t.__compiled\
__[r.schema].nor\
malize(r,t),r}fu\
nction De(t,e){i\
f(!(this instanc\
eof De))return n\
ew De(t,e);var r\
;e||(r=t,Object.\
keys(r||{}).redu\
ce((function(t,e\
){return t||pe.h\
asOwnProperty(e)\
}),!1)&&(e=t,t={\
})),this.__opts_\
_=ce({},pe,e),th\
is.__index__=-1,\
this.__last_inde\
x__=-1,this.__sc\
hema__=\x22\x22,this._\
_text_cache__=\x22\x22\
,this.__schemas_\
_=ce({},fe,t),th\
is.__compiled__=\
{},this.__tlds__\
=_e,this.__tlds_\
replaced__=!1,th\
is.re={},me(this\
)}De.prototype.a\
dd=function(t,e)\
{return this.__s\
chemas__[t]=e,me\
(this),this},De.\
prototype.set=fu\
nction(t){return\
 this.__opts__=c\
e(this.__opts__,\
t),this},De.prot\
otype.test=funct\
ion(t){if(this._\
_text_cache__=t,\
this.__index__=-\
1,!t.length)retu\
rn!1;let e,r,n,s\
,i,o,u,c,a;if(th\
is.re.schema_tes\
t.test(t))for(u=\
this.re.schema_s\
earch,u.lastInde\
x=0;null!==(e=u.\
exec(t));)if(s=t\
his.testSchemaAt\
(t,e[2],u.lastIn\
dex),s){this.__s\
chema__=e[2],thi\
s.__index__=e.in\
dex+e[1].length,\
this.__last_inde\
x__=e.index+e[0]\
.length+s;break}\
return this.__op\
ts__.fuzzyLink&&\
this.__compiled_\
_[\x22http:\x22]&&(c=t\
.search(this.re.\
host_fuzzy_test)\
,c>=0&&(this.__i\
ndex__<0||c<this\
.__index__)&&nul\
l!==(r=t.match(t\
his.__opts__.fuz\
zyIP?this.re.lin\
k_fuzzy:this.re.\
link_no_ip_fuzzy\
))&&(i=r.index+r\
[1].length,(this\
.__index__<0||i<\
this.__index__)&\
&(this.__schema_\
_=\x22\x22,this.__inde\
x__=i,this.__las\
t_index__=r.inde\
x+r[0].length)))\
,this.__opts__.f\
uzzyEmail&&this.\
__compiled__[\x22ma\
ilto:\x22]&&(a=t.in\
dexOf(\x22@\x22),a>=0&\
&null!==(n=t.mat\
ch(this.re.email\
_fuzzy))&&(i=n.i\
ndex+n[1].length\
,o=n.index+n[0].\
length,(this.__i\
ndex__<0||i<this\
.__index__||i===\
this.__index__&&\
o>this.__last_in\
dex__)&&(this.__\
schema__=\x22mailto\
:\x22,this.__index_\
_=i,this.__last_\
index__=o))),thi\
s.__index__>=0},\
De.prototype.pre\
test=function(t)\
{return this.re.\
pretest.test(t)}\
,De.prototype.te\
stSchemaAt=funct\
ion(t,e,r){retur\
n this.__compile\
d__[e.toLowerCas\
e()]?this.__comp\
iled__[e.toLower\
Case()].validate\
(t,r,this):0},De\
.prototype.match\
=function(t){con\
st e=[];let r=0;\
this.__index__>=\
0&&this.__text_c\
ache__===t&&(e.p\
ush(ke(this,r)),\
r=this.__last_in\
dex__);let n=r?t\
.slice(r):t;for(\
;this.test(n);)e\
.push(ke(this,r)\
),n=n.slice(this\
.__last_index__)\
,r+=this.__last_\
index__;return e\
.length?e:null},\
De.prototype.mat\
chAtStart=functi\
on(t){if(this.__\
text_cache__=t,t\
his.__index__=-1\
,!t.length)retur\
n null;const e=t\
his.re.schema_at\
_start.exec(t);i\
f(!e)return null\
;const r=this.te\
stSchemaAt(t,e[2\
],e[0].length);r\
eturn r?(this.__\
schema__=e[2],th\
is.__index__=e.i\
ndex+e[1].length\
,this.__last_ind\
ex__=e.index+e[0\
].length+r,ke(th\
is,0)):null},De.\
prototype.tlds=f\
unction(t,e){ret\
urn t=Array.isAr\
ray(t)?t:[t],e?(\
this.__tlds__=th\
is.__tlds__.conc\
at(t).sort().fil\
ter((function(t,\
e,r){return t!==\
r[e-1]})).revers\
e(),me(this),thi\
s):(this.__tlds_\
_=t.slice(),this\
.__tlds_replaced\
__=!0,me(this),t\
his)},De.prototy\
pe.normalize=fun\
ction(t){t.schem\
a||(t.url=\x22http:\
//\x22+t.url),\x22mail\
to:\x22!==t.schema|\
|/^mailto:/i.tes\
t(t.url)||(t.url\
=\x22mailto:\x22+t.url\
)},De.prototype.\
onCompile=functi\
on(){};const Ce=\
2147483647,ye=36\
,Ee=/^xn--/,Ae=/\
[^\x5c0-\x5cx7F]/,be=/\
[\x5cx2E\x5cu3002\x5cuFF0\
E\x5cuFF61]/g,Fe={o\
verflow:\x22Overflo\
w: input needs w\
ider integers to\
 process\x22,\x22not-b\
asic\x22:\x22Illegal i\
nput >= 0x80 (no\
t a basic code p\
oint)\x22,\x22invalid-\
input\x22:\x22Invalid \
input\x22},xe=Math.\
floor,we=String.\
fromCharCode;fun\
ction ve(t){thro\
w new RangeError\
(Fe[t])}function\
 ze(t,e){const r\
=t.split(\x22@\x22);le\
t n=\x22\x22;r.length>\
1&&(n=r[0]+\x22@\x22,t\
=r[1]);const s=f\
unction(t,e){con\
st r=[];let n=t.\
length;for(;n--;\
)r[n]=e(t[n]);re\
turn r}((t=t.rep\
lace(be,\x22.\x22)).sp\
lit(\x22.\x22),e).join\
(\x22.\x22);return n+s\
}function Se(t){\
const e=[];let r\
=0;const n=t.len\
gth;for(;r<n;){c\
onst s=t.charCod\
eAt(r++);if(s>=5\
5296&&s<=56319&&\
r<n){const n=t.c\
harCodeAt(r++);5\
6320==(64512&n)?\
e.push(((1023&s)\
<<10)+(1023&n)+6\
5536):(e.push(s)\
,r--)}else e.pus\
h(s)}return e}co\
nst qe=function(\
t,e){return t+22\
+75*(t<26)-((0!=\
e)<<5)},Be=funct\
ion(t,e,r){let n\
=0;for(t=r?xe(t/\
700):t>>1,t+=xe(\
t/e);t>455;n+=ye\
)t=xe(t/35);retu\
rn xe(n+36*t/(t+\
38))},Le=functio\
n(t){const e=[],\
r=t.length;let n\
=0,s=128,i=72,o=\
t.lastIndexOf(\x22-\
\x22);o<0&&(o=0);fo\
r(let r=0;r<o;++\
r)t.charCodeAt(r\
)>=128&&ve(\x22not-\
basic\x22),e.push(t\
.charCodeAt(r));\
for(let c=o>0?o+\
1:0;c<r;){const \
o=n;for(let e=1,\
s=ye;;s+=ye){c>=\
r&&ve(\x22invalid-i\
nput\x22);const o=(\
u=t.charCodeAt(c\
++))>=48&&u<58?u\
-48+26:u>=65&&u<\
91?u-65:u>=97&&u\
<123?u-97:ye;o>=\
ye&&ve(\x22invalid-\
input\x22),o>xe((Ce\
-n)/e)&&ve(\x22over\
flow\x22),n+=o*e;co\
nst a=s<=i?1:s>=\
i+26?26:s-i;if(o\
<a)break;const l\
=ye-a;e>xe(Ce/l)\
&&ve(\x22overflow\x22)\
,e*=l}const a=e.\
length+1;i=Be(n-\
o,a,0==o),xe(n/a\
)>Ce-s&&ve(\x22over\
flow\x22),s+=xe(n/a\
),n%=a,e.splice(\
n++,0,s)}var u;r\
eturn String.fro\
mCodePoint(...e)\
},Ie=function(t)\
{const e=[],r=(t\
=Se(t)).length;l\
et n=128,s=0,i=7\
2;for(const r of\
 t)r<128&&e.push\
(we(r));const o=\
e.length;let u=o\
;for(o&&e.push(\x22\
-\x22);u<r;){let r=\
Ce;for(const e o\
f t)e>=n&&e<r&&(\
r=e);const c=u+1\
;r-n>xe((Ce-s)/c\
)&&ve(\x22overflow\x22\
),s+=(r-n)*c,n=r\
;for(const r of \
t)if(r<n&&++s>Ce\
&&ve(\x22overflow\x22)\
,r===n){let t=s;\
for(let r=ye;;r+\
=ye){const n=r<=\
i?1:r>=i+26?26:r\
-i;if(t<n)break;\
const s=t-n,o=ye\
-n;e.push(we(qe(\
n+s%o,0))),t=xe(\
s/o)}e.push(we(q\
e(t,0))),i=Be(s,\
c,u===o),s=0,++u\
}++s,++n}return \
e.join(\x22\x22)},Me={\
version:\x222.3.1\x22,\
ucs2:{decode:Se,\
encode:t=>String\
.fromCodePoint(.\
..t)},decode:Le,\
encode:Ie,toASCI\
I:function(t){re\
turn ze(t,(funct\
ion(t){return Ae\
.test(t)?\x22xn--\x22+\
Ie(t):t}))},toUn\
icode:function(t\
){return ze(t,(f\
unction(t){retur\
n Ee.test(t)?Le(\
t.slice(4).toLow\
erCase()):t}))}}\
;const Te={defau\
lt:{options:{htm\
l:!1,xhtmlOut:!1\
,breaks:!1,langP\
refix:\x22language-\
\x22,linkify:!1,typ\
ographer:!1,quot\
es:\x22\x5cu201c\x5cu201d\
\x5cu2018\x5cu2019\x22,hi\
ghlight:null,max\
Nesting:100},com\
ponents:{core:{}\
,block:{},inline\
:{}}},zero:{opti\
ons:{html:!1,xht\
mlOut:!1,breaks:\
!1,langPrefix:\x22l\
anguage-\x22,linkif\
y:!1,typographer\
:!1,quotes:\x22\x5cu20\
1c\x5cu201d\x5cu2018\x5cu\
2019\x22,highlight:\
null,maxNesting:\
20},components:{\
core:{rules:[\x22no\
rmalize\x22,\x22block\x22\
,\x22inline\x22,\x22text_\
join\x22]},block:{r\
ules:[\x22paragraph\
\x22]},inline:{rule\
s:[\x22text\x22],rules\
2:[\x22balance_pair\
s\x22,\x22fragments_jo\
in\x22]}}},commonma\
rk:{options:{htm\
l:!0,xhtmlOut:!0\
,breaks:!1,langP\
refix:\x22language-\
\x22,linkify:!1,typ\
ographer:!1,quot\
es:\x22\x5cu201c\x5cu201d\
\x5cu2018\x5cu2019\x22,hi\
ghlight:null,max\
Nesting:20},comp\
onents:{core:{ru\
les:[\x22normalize\x22\
,\x22block\x22,\x22inline\
\x22,\x22text_join\x22]},\
block:{rules:[\x22b\
lockquote\x22,\x22code\
\x22,\x22fence\x22,\x22headi\
ng\x22,\x22hr\x22,\x22html_b\
lock\x22,\x22lheading\x22\
,\x22list\x22,\x22referen\
ce\x22,\x22paragraph\x22]\
},inline:{rules:\
[\x22autolink\x22,\x22bac\
kticks\x22,\x22emphasi\
s\x22,\x22entity\x22,\x22esc\
ape\x22,\x22html_inlin\
e\x22,\x22image\x22,\x22link\
\x22,\x22newline\x22,\x22tex\
t\x22],rules2:[\x22bal\
ance_pairs\x22,\x22emp\
hasis\x22,\x22fragment\
s_join\x22]}}}},Re=\
/^(vbscript|java\
script|file|data\
):/,Ne=/^data:im\
age\x5c/(gif|png|jp\
eg|webp);/;funct\
ion Pe(t){const \
e=t.trim().toLow\
erCase();return!\
Re.test(e)||Ne.t\
est(e)}const Oe=\
[\x22http:\x22,\x22https:\
\x22,\x22mailto:\x22];fun\
ction je(t){cons\
t e=g(t,!0);if(e\
.hostname&&(!e.p\
rotocol||Oe.inde\
xOf(e.protocol)>\
=0))try{e.hostna\
me=Me.toASCII(e.\
hostname)}catch(\
t){}return n(s(e\
))}function Ze(t\
){const r=g(t,!0\
);if(r.hostname&\
&(!r.protocol||O\
e.indexOf(r.prot\
ocol)>=0))try{r.\
hostname=Me.toUn\
icode(r.hostname\
)}catch(t){}retu\
rn e(s(r),e.defa\
ultChars+\x22%\x22)}fu\
nction $e(t,e){i\
f(!(this instanc\
eof $e))return n\
ew $e(t,e);e||Z(\
t)||(e=t||{},t=\x22\
default\x22),this.i\
nline=new ue,thi\
s.block=new Ht,t\
his.core=new Lt,\
this.renderer=ne\
w ft,this.linkif\
y=new De,this.va\
lidateLink=Pe,th\
is.normalizeLink\
=je,this.normali\
zeLinkText=Ze,th\
is.utils=lt,this\
.helpers=U({},ht\
),this.options={\
},this.configure\
(t),e&&this.set(\
e)}return $e.pro\
totype.set=funct\
ion(t){return U(\
this.options,t),\
this},$e.prototy\
pe.configure=fun\
ction(t){const e\
=this;if(Z(t)){c\
onst e=t;if(!(t=\
Te[e]))throw new\
 Error('Wrong `m\
arkdown-it` pres\
et \x22'+e+'\x22, chec\
k name')}if(!t)t\
hrow new Error(\x22\
Wrong `markdown-\
it` preset, can'\
t be empty\x22);ret\
urn t.options&&e\
.set(t.options),\
t.components&&Ob\
ject.keys(t.comp\
onents).forEach(\
(function(r){t.c\
omponents[r].rul\
es&&e[r].ruler.e\
nableOnly(t.comp\
onents[r].rules)\
,t.components[r]\
.rules2&&e[r].ru\
ler2.enableOnly(\
t.components[r].\
rules2)})),this}\
,$e.prototype.en\
able=function(t,\
e){let r=[];Arra\
y.isArray(t)||(t\
=[t]),[\x22core\x22,\x22b\
lock\x22,\x22inline\x22].\
forEach((functio\
n(e){r=r.concat(\
this[e].ruler.en\
able(t,!0))}),th\
is),r=r.concat(t\
his.inline.ruler\
2.enable(t,!0));\
const n=t.filter\
((function(t){re\
turn r.indexOf(t\
)<0}));if(n.leng\
th&&!e)throw new\
 Error(\x22Markdown\
It. Failed to en\
able unknown rul\
e(s): \x22+n);retur\
n this},$e.proto\
type.disable=fun\
ction(t,e){let r\
=[];Array.isArra\
y(t)||(t=[t]),[\x22\
core\x22,\x22block\x22,\x22i\
nline\x22].forEach(\
(function(e){r=r\
.concat(this[e].\
ruler.disable(t,\
!0))}),this),r=r\
.concat(this.inl\
ine.ruler2.disab\
le(t,!0));const \
n=t.filter((func\
tion(t){return r\
.indexOf(t)<0}))\
;if(n.length&&!e\
)throw new Error\
(\x22MarkdownIt. Fa\
iled to disable \
unknown rule(s):\
 \x22+n);return thi\
s},$e.prototype.\
use=function(t){\
const e=[this].c\
oncat(Array.prot\
otype.slice.call\
(arguments,1));r\
eturn t.apply(t,\
e),this},$e.prot\
otype.parse=func\
tion(t,e){if(\x22st\
ring\x22!=typeof t)\
throw new Error(\
\x22Input data shou\
ld be a String\x22)\
;const r=new thi\
s.core.State(t,t\
his,e);return th\
is.core.process(\
r),r.tokens},$e.\
prototype.render\
=function(t,e){r\
eturn e=e||{},th\
is.renderer.rend\
er(this.parse(t,\
e),this.options,\
e)},$e.prototype\
.parseInline=fun\
ction(t,e){const\
 r=new this.core\
.State(t,this,e)\
;return r.inline\
Mode=!0,this.cor\
e.process(r),r.t\
okens},$e.protot\
ype.renderInline\
=function(t,e){r\
eturn e=e||{},th\
is.renderer.rend\
er(this.parseInl\
ine(t,e),this.op\
tions,e)},$e}));\
\x0a\
\x00\x00\x0d\x96\
!\
function(e,t){\x22o\
bject\x22==typeof e\
xports&&\x22object\x22\
==typeof module?\
module.exports=t\
(require(\x22katex\x22\
)):\x22function\x22==t\
ypeof define&&de\
fine.amd?define(\
[\x22katex\x22],t):\x22ob\
ject\x22==typeof ex\
ports?exports.re\
nderMathInElemen\
t=t(require(\x22kat\
ex\x22)):e.renderMa\
thInElement=t(e.\
katex)}(\x22undefin\
ed\x22!=typeof self\
?self:this,(func\
tion(e){return f\
unction(){\x22use s\
trict\x22;var t={77\
1:function(t){t.\
exports=e}},r={}\
;function n(e){v\
ar i=r[e];if(voi\
d 0!==i)return i\
.exports;var a=r\
[e]={exports:{}}\
;return t[e](a,a\
.exports,n),a.ex\
ports}n.n=functi\
on(e){var t=e&&e\
.__esModule?func\
tion(){return e.\
default}:functio\
n(){return e};re\
turn n.d(t,{a:t}\
),t},n.d=functio\
n(e,t){for(var r\
 in t)n.o(t,r)&&\
!n.o(e,r)&&Objec\
t.defineProperty\
(e,r,{enumerable\
:!0,get:t[r]})},\
n.o=function(e,t\
){return Object.\
prototype.hasOwn\
Property.call(e,\
t)};var i={};ret\
urn function(){n\
.d(i,{default:fu\
nction(){return \
s}});var e=n(771\
),t=n.n(e),r=fun\
ction(e,t,r){for\
(var n=r,i=0,a=e\
.length;n<t.leng\
th;){var o=t[n];\
if(i<=0&&t.slice\
(n,n+a)===e)retu\
rn n;\x22\x5c\x5c\x22===o?n+\
+:\x22{\x22===o?i++:\x22}\
\x22===o&&i--,n++}r\
eturn-1},a=/^\x5c\x5cb\
egin{/,o=functio\
n(e,t){for(var n\
,i=[],o=new RegE\
xp(\x22(\x22+t.map((fu\
nction(e){return\
 e.left.replace(\
/[-/\x5c\x5c^$*+?.()|[\
\x5c]{}]/g,\x22\x5c\x5c$&\x22)}\
)).join(\x22|\x22)+\x22)\x22\
);-1!==(n=e.sear\
ch(o));){n>0&&(i\
.push({type:\x22tex\
t\x22,data:e.slice(\
0,n)}),e=e.slice\
(n));var l=t.fin\
dIndex((function\
(t){return e.sta\
rtsWith(t.left)}\
));if(-1===(n=r(\
t[l].right,e,t[l\
].left.length)))\
break;var d=e.sl\
ice(0,n+t[l].rig\
ht.length),s=a.t\
est(d)?d:e.slice\
(t[l].left.lengt\
h,n);i.push({typ\
e:\x22math\x22,data:s,\
rawData:d,displa\
y:t[l].display})\
,e=e.slice(n+t[l\
].right.length)}\
return\x22\x22!==e&&i.\
push({type:\x22text\
\x22,data:e}),i},l=\
function(e,r){va\
r n=o(e,r.delimi\
ters);if(1===n.l\
ength&&\x22text\x22===\
n[0].type)return\
 null;for(var i=\
document.createD\
ocumentFragment(\
),a=0;a<n.length\
;a++)if(\x22text\x22==\
=n[a].type)i.app\
endChild(documen\
t.createTextNode\
(n[a].data));els\
e{var l=document\
.createElement(\x22\
span\x22),d=n[a].da\
ta;r.displayMode\
=n[a].display;tr\
y{r.preProcess&&\
(d=r.preProcess(\
d)),t().render(d\
,l,r)}catch(e){i\
f(!(e instanceof\
 t().ParseError)\
)throw e;r.error\
Callback(\x22KaTeX \
auto-render: Fai\
led to parse `\x22+\
n[a].data+\x22` wit\
h \x22,e),i.appendC\
hild(document.cr\
eateTextNode(n[a\
].rawData));cont\
inue}i.appendChi\
ld(l)}return i},\
d=function e(t,r\
){for(var n=0;n<\
t.childNodes.len\
gth;n++){var i=t\
.childNodes[n];i\
f(3===i.nodeType\
){for(var a=i.te\
xtContent,o=i.ne\
xtSibling,d=0;o&\
&o.nodeType===No\
de.TEXT_NODE;)a+\
=o.textContent,o\
=o.nextSibling,d\
++;var s=l(a,r);\
if(s){for(var f=\
0;f<d;f++)i.next\
Sibling.remove()\
;n+=s.childNodes\
.length-1,t.repl\
aceChild(s,i)}el\
se n+=d}else 1==\
=i.nodeType&&fun\
ction(){var t=\x22 \
\x22+i.className+\x22 \
\x22;-1===r.ignored\
Tags.indexOf(i.n\
odeName.toLowerC\
ase())&&r.ignore\
dClasses.every((\
function(e){retu\
rn-1===t.indexOf\
(\x22 \x22+e+\x22 \x22)}))&&\
e(i,r)}()}},s=fu\
nction(e,t){if(!\
e)throw new Erro\
r(\x22No element pr\
ovided to render\
\x22);var r={};for(\
var n in t)t.has\
OwnProperty(n)&&\
(r[n]=t[n]);r.de\
limiters=r.delim\
iters||[{left:\x22$\
$\x22,right:\x22$$\x22,di\
splay:!0},{left:\
\x22\x5c\x5c(\x22,right:\x22\x5c\x5c)\
\x22,display:!1},{l\
eft:\x22\x5c\x5cbegin{equ\
ation}\x22,right:\x22\x5c\
\x5cend{equation}\x22,\
display:!0},{lef\
t:\x22\x5c\x5cbegin{align\
}\x22,right:\x22\x5c\x5cend{\
align}\x22,display:\
!0},{left:\x22\x5c\x5cbeg\
in{alignat}\x22,rig\
ht:\x22\x5c\x5cend{aligna\
t}\x22,display:!0},\
{left:\x22\x5c\x5cbegin{g\
ather}\x22,right:\x22\x5c\
\x5cend{gather}\x22,di\
splay:!0},{left:\
\x22\x5c\x5cbegin{CD}\x22,ri\
ght:\x22\x5c\x5cend{CD}\x22,\
display:!0},{lef\
t:\x22\x5c\x5c[\x22,right:\x22\x5c\
\x5c]\x22,display:!0}]\
,r.ignoredTags=r\
.ignoredTags||[\x22\
script\x22,\x22noscrip\
t\x22,\x22style\x22,\x22text\
area\x22,\x22pre\x22,\x22cod\
e\x22,\x22option\x22],r.i\
gnoredClasses=r.\
ignoredClasses||\
[],r.errorCallba\
ck=r.errorCallba\
ck||console.erro\
r,r.macros=r.mac\
ros||{},d(e,r)}}\
(),i=i.default}(\
)}));\
"

qt_resource_name = b"\
\x00\x02\
\x00\x00\x07\x13\
\x00j\
\x00s\
\x00\x10\
\x0b|\x14\x93\
\x00h\
\x00i\x00g\x00h\x00l\x00i\x00g\x00h\x00t\x00.\x00m\x00i\x00n\x00.\x00j\x00s\
\x00\x18\
\x0eQ\xc0s\
\x00m\
\x00a\x00r\x00k\x00d\x00o\x00w\x00n\x00-\x00i\x00t\x00-\x00k\x00a\x00t\x00e\x00x\
\x00.\x00m\x00i\x00n\x00.\x00j\x00s\
\x00\x0c\
\x0b\x11Ls\
\x00k\
\x00a\x00t\x00e\x00x\x00.\x00m\x00i\x00n\x00.\x00j\x00s\
\x00\x06\
\x06\x875\x13\
\x00a\
\x00p\x00p\x00.\x00j\x00s\
\x00\x12\
\x04\xa0S\x13\
\x00m\
\x00a\x00r\x00k\x00d\x00o\x00w\x00n\x00-\x00i\x00t\x00.\x00m\x00i\x00n\x00.\x00j\
\x00s\
\x00\x12\
\x0e\x87\x84\xb3\
\x00a\
\x00u\x00t\x00o\x00-\x00r\x00e\x00n\x00d\x00e\x00r\x00.\x00m\x00i\x00n\x00.\x00j\
\x00s\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x06\x00\x00\x00\x02\
\x00\x00\x00\x96\x00\x00\x00\x00\x00\x01\x00\x18\xb4 \
\x00\x00\x00\x84\x00\x00\x00\x00\x00\x01\x00\x14\xb5\xf0\
\x00\x00\x00f\x00\x00\x00\x00\x00\x01\x00\x10|\xd7\
\x00\x00\x00\x0a\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x000\x00\x00\x00\x00\x00\x01\x00\x10s\xea\
\x00\x00\x00\xc0\x00\x00\x00\x00\x00\x01\x00\x1a\x97\x06\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x01, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
